//
// Asura_Platform_Animation_RenderCache.h
//
// Ben Weston - 13/06/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_PLATFORM_ANIMATION_RENDERCACHE_H__
#define __ASURA_PLATFORM_ANIMATION_RENDERCACHE_H__

#if !defined(ASURA_USE_ANIMATION_PLATFORM_RENDER_CACHE)
#error PS3 must have ASURA_USE_ANIMATION_PLATFORM_RENDER_CACHE defined
#endif

#include "Core/Asura_Maths.h"
#include "Core/Asura_Timers.h"

#define NUM_ASURA_PLATFORM_ANIMATION_RENDERCACHE_VERTEXBUFFERS	2 //BW at present we have: shadows & opaque render, later we'll have z-prepass, transparent & (on AVP) the predator's cloak render, all of which use the same data as opaque render

class Asura_PS3_GCM_VertexBuffer;
class Asura_PS3_GCM_IndexBuffer;
class Asura_PS3_SPU_Skinning_Batch;

class Asura_Platform_Animation_RenderCache
{
friend class Asura_Animation;

public:
	Asura_AMatrix4x4* UpdateCachedBones( const Asura_AMatrix4x4* pxBones, u_int uNumBones );
	Asura_AMatrix4x4* GetCachedBones() { return m_pxBoneCache; }
	u_int GetNumCachedBones() { return m_uNumBonesAlloced; }
	bool IsBoneCacheCurrent() { return (Asura_Timers::GetFrameNumber() == m_uBoneFrameStamp); }

	Asura_PS3_SPU_Skinning_Batch* GetBatches( u_int uSkin, u_int uNumStrips );

	Asura_PS3_GCM_VertexBuffer* ObtainVertexBuffer( u_int uBuffer, u_int uSkin, u_int uNumVerts, size_t uVertexSize );
	Asura_PS3_GCM_VertexBuffer* ObtainVertexBuffer( u_int uBuffer, u_int uSkin ) { return m_apxVertexBuffer[uBuffer][uSkin]; }
	Asura_PS3_GCM_IndexBuffer* ObtainIndexBuffer( u_int uSkin, u_int uNumIndices );
	Asura_PS3_GCM_IndexBuffer* ObtainIndexBuffer( u_int uSkin ) { return m_apxIndexBuffer[uSkin]; }

	bool IsVertexCacheCurrent( u_int uBuffer, u_int uSkin ) { return (Asura_Timers::GetFrameNumber() == m_auVertexFrameStamp[uBuffer][uSkin]); }
	void SetVertexCacheCurrent( u_int uBuffer, u_int uSkin ) { m_auVertexFrameStamp[uBuffer][uSkin] = Asura_Timers::GetFrameNumber(); }

	void Destroy();

	// functions to globally manage all render caches
	static void Initialise(); //BW todo: this should really be passed the top LOD skin, then can remember buffer sizes for when used.
	static void Shutdown();
	static void Cleanup();
	static Asura_Platform_Animation_RenderCache* Create();

private:
	Asura_Platform_Animation_RenderCache();
	~Asura_Platform_Animation_RenderCache();
	Asura_Platform_Animation_RenderCache*				m_pxNext; // maintain linked lists for any globally applicable actions

	void SetBoneCacheCurrent() { m_uBoneFrameStamp = Asura_Timers::GetFrameNumber(); }
	

	// cached data
	// these buffers are allocated when needed (hence keep them private so we can create them when accessed)
	
	// bone cache
	Asura_AMatrix4x4*			m_pxBoneCache;
	u_int						m_uNumBonesAlloced;
	u_int						m_uBoneFrameStamp;

	// vertex buffers
	Asura_PS3_GCM_VertexBuffer*	m_apxVertexBuffer[NUM_ASURA_PLATFORM_ANIMATION_RENDERCACHE_VERTEXBUFFERS][ASURA_ANIMATION_SKIN_COUNT];

	// index buffers
	// only need 1 for each view where we cull geometry in the spus, which is only the main camera view (since we want multiple shadows without having to cull for each shadow view)
	// so no need to have multiple ones (apart from coping with multiple skins)
	Asura_PS3_GCM_IndexBuffer*	m_apxIndexBuffer[ASURA_ANIMATION_SKIN_COUNT];

	u_int						m_auVertexFrameStamp[NUM_ASURA_PLATFORM_ANIMATION_RENDERCACHE_VERTEXBUFFERS][ASURA_ANIMATION_SKIN_COUNT];

	// SPU vertex batch info
	Asura_PS3_SPU_Skinning_Batch*	m_apxBatches[ASURA_ANIMATION_SKIN_COUNT];	

	// static data
	static Asura_Platform_Animation_RenderCache*		s_apxCleanUp[2]; // double buffered list of render caches to delete later (after RSX has finished reading their data)
	static Asura_Platform_Animation_RenderCache*		s_pxActive; // should be unnecessary since clients should destroy thier caches
	static u_int										s_uCleanupBuffer;

};


#endif//__ASURA_PLATFORM_ANIMATION_RENDERCACHE_H__
