/**
*
* \file Asura_AnimTable.h
*
* Created by Richard May - 18/06/2002 17:19:17
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/


#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_ANIMTABLE_H__
#define __ASURA_ANIMTABLE_H__

#include "Core/Asura.h"

#ifdef ASURA_DISABLE_ANIM_TABLE_CODE
#else
/************/
/* Includes.*/
/************/

#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Handle.h"
#include "Core/Asura_Hash.h"
#include "Core/Memory/Asura_ResourceAllocator.h"

/***********/
/* Defines.*/
/***********/

#define  ASURA_ANIMTABLE_HEADER_SIZE     3
#define  ASURA_ANIMTABLE_SECONDARY_NONE  0
#ifndef ASURA_MAX_ANIMTABLE_PAGES
	#define ASURA_MAX_ANIMTABLE_PAGES 8
#endif //#ifndef ASURA_MAX_ANIMTABLE_PAGES

static const u_int uASURA_ANIMTABLE_HASHID_NOTAPPLICABLE = 0x00000DB3; // Asura_GetHashID("NA")
static const u_int uASURA_ANIMTABLE_HASHID_SLAVE         = 0x068721D7; // Asura_GetHashID("Slave")

/************/
/* Classes. */
/************/

union Asura_AnimTable_AnimFlags
{
    u_char m_ucAllFlags;

    struct 
    {
        u_char m_bMirrored     : 1,
               m_bReversed     : 1,
               m_ucUnusedFlags : 6;
    };
};

/*************************************************************
 *
 * Class Name    : Asura_AnimTable
 *
 * Author        : Richard May
 *
 * Description   : 
 *
 *************************************************************/
class Asura_AnimTable
{
	template<class Asura_AnimTable> friend class Asura_TemplateHashTree_Node;
	friend class Asura_AnimTable_Handler;

public:

	/* Constructors/destructor. */
	Asura_AnimTable( Asura_Hash_ID uNameHash, u_int uNoOfPrimaryStates, u_int uNoOfSecondaryStates ) 
	{
		m_uNameHash = uNameHash;
		
		m_uNoOfWeaponStates    = 0;
		m_uNoOfPrimaryStates   = uNoOfPrimaryStates;
		m_uNoOfSecondaryStates = uNoOfSecondaryStates;

		for (int i=0; i<ASURA_MAX_ANIMTABLE_PAGES; ++i)
		{
			m_appuAnimIDs[i] = NULL;
		}

		m_ppxAnimFlags = NULL;
#ifdef ASURA_TOOLS
		m_pszName = NULL;
#endif
	}

	~Asura_AnimTable()
	{
		// if m_uNoOfWeaponStates > 0 then we should be able to safely assume 
		// the rest of the table has been initialised also
		if (m_uNoOfWeaponStates > 0)
		{
			Asura_ResourcePushType(ASURA_RESOURCE_ANIMATION);
			{
				for( unsigned int uState=0; uState<m_uNoOfWeaponStates; ++uState )
				{
					if (m_appuAnimIDs[uState] != NULL)
					{
						for( unsigned int uPrimary=0; uPrimary<m_uNoOfPrimaryStates; ++uPrimary )
						{
							Asura_ResourceDeallocateArray(m_appuAnimIDs[uState][uPrimary]);
						} 
						Asura_ResourceDeallocateArray(m_appuAnimIDs[uState]);
					}

                    if( m_ppxAnimFlags )
                    {
						Asura_ResourceDeallocateArray( m_ppxAnimFlags[uState] );
                    }
				}

                Asura_ResourceDeallocateArray( m_ppxAnimFlags );
            }
			Asura_ResourcePopType();
		}
#ifdef ASURA_TOOLS
		SetName(NULL);
#endif
	}

    static Asura_AnimTable_AnimFlags s_xDefaultAnimFlags;

	Asura_Hash_ID m_uNameHash;

#ifdef ASURA_TOOLS
	void SetName(const char* pszName);
	const char* GetName() const { return m_pszName; }
#endif

private :

	Asura_Hash_ID**             m_appuAnimIDs[ASURA_MAX_ANIMTABLE_PAGES];
    Asura_AnimTable_AnimFlags** m_ppxAnimFlags;
	u_int                       m_uNoOfWeaponStates;
	u_int                       m_uNoOfPrimaryStates;
	u_int                       m_uNoOfSecondaryStates;
#ifdef ASURA_TOOLS
	Asura_Hash_ID               m_uParentTableHashID;
	char*					m_pszName;
#endif
	
	// Private constructor - used by "Asura_AnimTable_Handler"...
	Asura_AnimTable( void )
        : m_uNameHash( ASURA_HASH_ID_UNSET )
        , m_ppxAnimFlags( NULL )
        , m_uNoOfWeaponStates( 0 )
        , m_uNoOfPrimaryStates( 0 )
        , m_uNoOfSecondaryStates( 0 )
#if defined(ASURA_TOOLS)
		, m_uParentTableHashID( ASURA_HASH_ID_UNSET )
		, m_pszName(NULL)		
#endif	
		
	{
		for (int i=0; i<ASURA_MAX_ANIMTABLE_PAGES; ++i)
		{
			m_appuAnimIDs[i] = NULL;
		}
	}

	void SetInitialData(Asura_Hash_ID uNameHash, u_int uNoOfPrimaryStates, u_int uNoOfSecondaryStates)
	{
		m_uNameHash				= uNameHash;
		m_uNoOfPrimaryStates	= uNoOfPrimaryStates;
		m_uNoOfSecondaryStates	= uNoOfSecondaryStates;
	}

	
public:
	/* Public interface functions. */
	Asura_Hash_ID GetNameHashID(void) const
	{
		return m_uNameHash;
	}
	u_int GetNoOfWeaponStates(void) const
	{
		return m_uNoOfWeaponStates;
	}
	u_int GetNoOfPrimaryStates(void) const
	{
		return m_uNoOfPrimaryStates;
	}
	u_int GetNoOfSecondaryStates(void) const
	{
		return m_uNoOfSecondaryStates;
	}

	bool GetWeaponStateExists(u_int uWeaponState) const
	{
		return (uWeaponState < ASURA_MAX_ANIMTABLE_PAGES) && (m_appuAnimIDs[uWeaponState] != NULL);
	}

#ifdef ASURA_TOOLS
	Asura_Hash_ID GetParentTableHashID( void ) const
	{
		return m_uParentTableHashID;
	}

	void SetParentTableHashID( Asura_Hash_ID uParentTableHashID )
	{
		m_uParentTableHashID = uParentTableHashID;
	}
#endif

	/// Richard May - 20/09/2002 12:31:30 - table comparison function - are the two tables compatible?
	bool IsCompatible( const Asura_AnimTable* pxTable ) const
	{
		return ((m_uNoOfWeaponStates == pxTable->GetNoOfWeaponStates()) && 
			    (m_uNoOfPrimaryStates == pxTable->GetNoOfPrimaryStates()) && 
			    (m_uNoOfSecondaryStates == pxTable->GetNoOfSecondaryStates()));
	}

	bool IsAnimApplicable( unsigned int uState, unsigned int uPrimary, unsigned int uSecondary=ASURA_ANIMTABLE_SECONDARY_NONE ) const
	{
		return (GetAnimID( uState, uPrimary, uSecondary ) != uASURA_ANIMTABLE_HASHID_NOTAPPLICABLE);
	}

	Asura_Hash_ID GetAnimID( unsigned int uState, unsigned int uPrimary, unsigned int uSecondary=ASURA_ANIMTABLE_SECONDARY_NONE ) const
	{
		if( uState >= m_uNoOfWeaponStates || !m_appuAnimIDs[uState] )
		{
			Asura_Assert( false, "ERROR - Invalid WeaponState Requested, returning NA" );
			return 0x00000DB3; //Asura_GetHashID("NA");
		}

		if( uPrimary >= m_uNoOfPrimaryStates || !m_appuAnimIDs[uState][uPrimary] )
		{
			Asura_Assert( false, "ERROR - Invalid PrimaryState Requested, returning NA" );
			return 0x00000DB3; // Asura_GetHashID("NA");
		}

		if( uSecondary >= m_uNoOfSecondaryStates || !m_appuAnimIDs[uState][uPrimary][uSecondary] )
		{
			Asura_Assert( false, "ERROR - Invalid SecondaryState Requested, returning NA" );
			return 0x00000DB3; // Asura_GetHashID("NA");
		}

		return m_appuAnimIDs[uState][uPrimary][uSecondary];
	}

	void SetAnimID( const Asura_Hash_ID uAnimID, unsigned int uState, unsigned int uPrimary, unsigned int uSecondary )
	{
		Asura_Assert(uState < ASURA_MAX_ANIMTABLE_PAGES, "Too many pages in this anim table. Increase ASURA_MAX_ANIMTABLE_PAGES");
		if (uState >= ASURA_MAX_ANIMTABLE_PAGES)
		{
			// Give up
			return;
		}

		if( m_appuAnimIDs[uState] == NULL )
		{
			// need to allocate this block
			m_uNoOfWeaponStates = Asura_Maths::Max(uState+1, m_uNoOfWeaponStates);

			Asura_ResourcePushType(ASURA_RESOURCE_ANIMATION);
			{
				Asura_Hash_ID** ppxPrimarySubArray;

				Asura_ResourceAllocateArray(ppxPrimarySubArray, m_uNoOfPrimaryStates);
				m_appuAnimIDs[uState] = ppxPrimarySubArray;

				for( unsigned int uPrimary=0; uPrimary<m_uNoOfPrimaryStates; ++uPrimary )
				{
					Asura_Hash_ID* pxSecondarySubArray;

					Asura_ResourceAllocateArray(pxSecondarySubArray, m_uNoOfSecondaryStates);
					m_appuAnimIDs[uState][uPrimary] = pxSecondarySubArray;

					for( unsigned int uSecondary=0; uSecondary<m_uNoOfSecondaryStates; ++uSecondary )
					{
						m_appuAnimIDs[uState][uPrimary][uSecondary] = ASURA_HASH_ID_UNSET;
					}
				}
			}
			Asura_ResourcePopType();
		}

		if( uPrimary >= m_uNoOfPrimaryStates )
		{
			Asura_Assert( false, "ERROR - Invalid PrimaryState Requested" );
			return;
		}

		if( uSecondary >= m_uNoOfSecondaryStates )
		{
			Asura_Assert( false, "ERROR - Invalid SecondaryState Requested" );
			return;
		}

		m_appuAnimIDs[uState][uPrimary][uSecondary] = uAnimID;
	}

    void PreallocateAnimFlags( void )
    {
        Asura_Assert( !m_ppxAnimFlags, "Flags already allocated" );

		Asura_ResourcePushType( ASURA_RESOURCE_ANIMATION );

        Asura_ResourceAllocateArray( m_ppxAnimFlags, m_uNoOfWeaponStates );
        for( u_int u = 0; u < m_uNoOfWeaponStates; ++u )
        {
            Asura_ResourceAllocateArray( m_ppxAnimFlags[u], m_uNoOfPrimaryStates * m_uNoOfSecondaryStates );
        }

		Asura_ResourcePopType();
    }

    Asura_AnimTable_AnimFlags* AllocateAnimFlagsForState( u_int uState )
    {
		Asura_Assert( uState < ASURA_MAX_ANIMTABLE_PAGES, "Too many pages in this anim table. Increase ASURA_MAX_ANIMTABLE_PAGES" );

		Asura_ResourcePushType( ASURA_RESOURCE_ANIMATION );

        if( !m_ppxAnimFlags )
        {
            // Just allocate maximum if we haven't been told how many may
            // be required.
            Asura_ResourceAllocateArray( m_ppxAnimFlags, ASURA_MAX_ANIMTABLE_PAGES );
            for( u_int u = 0; u < ASURA_MAX_ANIMTABLE_PAGES; ++u )
            {
                m_ppxAnimFlags[u] = NULL;
            }
        }

        if( !m_ppxAnimFlags[uState] )
        {
            Asura_ResourceAllocateArray( m_ppxAnimFlags[uState], m_uNoOfPrimaryStates * m_uNoOfSecondaryStates );
        }

		Asura_ResourcePopType();

        return m_ppxAnimFlags[uState];
    }

    bool HasAnyAnimFlagData( void ) const
    {
        return( NULL != m_ppxAnimFlags );
    }

    const Asura_AnimTable_AnimFlags& GetAnimFlags( u_int uState, u_int uPrimary, u_int uSecondary ) const
    {
        if( m_ppxAnimFlags && uState < m_uNoOfWeaponStates && uPrimary < m_uNoOfPrimaryStates && uSecondary < m_uNoOfSecondaryStates )
        {
            return m_ppxAnimFlags[uState][uPrimary * m_uNoOfSecondaryStates + uSecondary];
        }
        else
        {
            Asura_Assert( false, "Invalid anim table flags requested, or else no flag data present" );
            return s_xDefaultAnimFlags;
        }
    }

    const Asura_AnimTable_AnimFlags* GetAllAnimFlagsForState( u_int uState ) const
    {
        return( m_ppxAnimFlags ? m_ppxAnimFlags[uState] : NULL );
    }

};
/* end of class Asura_AnimTable */


/*************************************************************
 *
 * Class Name    : Asura_SecondaryAnimHandle
 *
 * Author        : Richard May
 *
 * Description   : Expanded Handle for a secondary animation - 
 *				   contains the Asura_Handle and the animtable
 *				   state
 *
 *************************************************************/
class Asura_SecondaryAnimHandle
{

public:
	/* Constructors/destructor. */
	Asura_SecondaryAnimHandle	()
	{
		m_iHandle = ASURA_INVALID_HANDLE;
		m_uAnim   = static_cast<unsigned int> (ASURA_ANIMTABLE_SECONDARY_NONE);
	};

	~Asura_SecondaryAnimHandle () {}; 

public:
	/* Data. */
	Asura_Handle  m_iHandle;
	unsigned int  m_uAnim;

};
/* end of class Asura_SecondaryAnimHandle */


/****************/
/* End of file. */
/****************/
#endif// #else not ASURA_DISABLE_ANIM_TABLE_CODE
#endif//__ASURA_ANIMTABLE_H__
