/**
*
* \file Asura_AnimTable_System.h
*
* Created by Richard May - 18/06/2002 16:38:49
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/


#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_ANIMTABLE_SYSTEM_H__
#define __ASURA_ANIMTABLE_SYSTEM_H__

#ifdef ASURA_DISABLE_ANIM_TABLE_CODE
#else
#define  ASURA_MAX_NO_OF_ANIMTABLES  100  

/************/
/* Includes.*/
/************/

#include "Animation/Asura_AnimTable.h"
#include "Core/Asura_HashTree.h"

/***********/
/* Defines */
/***********/

/// Richard May - 20/06/2002 10:14:27 - 
/// set this to 1 to allow the animation table chunk loader to over-write 
/// any animation table - only relevant if you've got more than one file 
/// for the same weapon state)
#define  ASURA_ANIMTABLE_ALLOW_OVERWRITE  0

/************/
/* Classes. */
/************/

class Asura_Stream;

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_AnimTable_Handler
// Author       : Andy Tate
// Date         : 28-Mar-2006
//
// Description  : 
////////////////////////////////////////////////////////////////////////////////

class Asura_AnimTable_Handler
{
	friend class Asura_AnimTable_System;

public:
	// Public functions...
	Asura_AnimTable_Handler  (void)	{}
	~Asura_AnimTable_Handler ()		{}

	// Reset function...
	void						Reset			(void);

	// Creation functions...
	Asura_AnimTable*			AddNewTable		(Asura_Hash_ID uNameHash, u_int uNoOfPrimaryStates, u_int uNoOfSecondaryStates);
	Asura_AnimTable*			GetAnimTable	(Asura_Hash_ID uNameHash)		{ return(m_xLoadedTables.GetDataByHash(uNameHash)); }
	bool						IsAnimLoaded	(Asura_Hash_ID uNameHash, bool bAddToList);


protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...
	Asura_TemplateHashTree<Asura_AnimTable>	m_xLoadedTables;


	// Private functions...

	Asura_AnimTable_Handler  (const Asura_AnimTable_Handler& xOther);

	Asura_AnimTable_Handler&	operator=		(const Asura_AnimTable_Handler& xOther);

};   // End of class 'Asura_AnimTable_Handler'.



/*************************************************************
 *
 * Class Name    : Asura_AnimTable_System
 *
 * Author        : Richard May
 *
 * Description   : Animation Table manager (designed for Dredd)
 *				   Loads and provides access to animation tables
 *				   for animated entities.
 *
 *************************************************************/
class Asura_AnimTable_System
{
	friend class Asura_AnimTable_Handler;

public:
	// Public functions...
	static 	void 						Initialise						(void);
	static 	void 						Reset							(void);
	static 	void 						Shutdown						(void);

	static 	Asura_AnimTable* 			AddNewTable						(const char* szName, u_int uNoOfPrimaryStates, u_int uNoOfSecondaryStates)		{ return(GetCurrentHandler().AddNewTable(Asura_GetHashID(szName), uNoOfPrimaryStates, uNoOfSecondaryStates)); }
	static 	Asura_AnimTable* 			AddNewTable						(Asura_Hash_ID uNameHash, u_int uNoOfPrimaryStates, u_int uNoOfSecondaryStates)	{ return(GetCurrentHandler().AddNewTable(uNameHash, uNoOfPrimaryStates, uNoOfSecondaryStates)); }

	static 	Asura_AnimTable* 			GetAnimTable					(const char* szName)															{ return(GetCurrentHandler().GetAnimTable(Asura_GetHashID(szName))); }
	static 	Asura_AnimTable* 			GetAnimTable					(Asura_Hash_ID uNameHash)														{ return(GetCurrentHandler().GetAnimTable(uNameHash)); }

	static 	bool 						IsAnimLoaded					(const char* szName, bool bAddToList)											{ return(GetCurrentHandler().IsAnimLoaded(Asura_GetHashID(szName), bAddToList)); }
	static 	bool 						IsAnimLoaded					(Asura_Hash_ID uNameHash, bool bAddToList)										{ return(GetCurrentHandler().IsAnimLoaded(uNameHash, bAddToList)); }

	static 	int							ConvertHashToAnimState			(Asura_Hash_ID uID);

#ifdef ASURA_TOOLS
	static  int							GetNumTables();
    static  const char*					GetTableNameByIndex(u_int uIndex);
	static  Asura_Hash_ID				GetTableHashByIndex(u_int uIndex);
#endif

#if defined(ASURA_USE_MULTIPLE_HANDLERS)
	// Handler functions...
	static	void						ReparentHandler					(const Asura_Hash_ID uSourceHash, const Asura_Hash_ID uNewParentHash);
	static	void						ReparentHandler					(Asura_AnimTable_Handler& xSourceHandler, const Asura_Hash_ID uNewParentHash);

	static	Asura_AnimTable_Handler*	CreateHandler					(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash);
	static	bool						DestroyHandler					(const Asura_Hash_ID uHash);
	static	void						WriteHandlerReport				(u_int uHandler, Asura_Stream& xLog);

	static	void						CreateAndSetHandler				(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash)	{ s_pxCurrentHandler = CreateHandler(uHash, uParentHandlerHash); }
	static	void						SetCurrentHandler				(Asura_AnimTable_Handler* pxHandler)								{ s_pxCurrentHandler = pxHandler; }
	static	void						SetCurrentHandler				(const Asura_Hash_ID uHash)											{ s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHash); }
	static	Asura_AnimTable_Handler&	GetCurrentHandler				(void)																{ return(*s_pxCurrentHandler); }
	static	Asura_AnimTable_Handler*	GetHandler						(const Asura_Hash_ID uHash)											{ return(s_xHandlerTree.GetDataByHash(uHash)); }
#else
	// Handler functions...
	static	Asura_AnimTable_Handler&	GetCurrentHandler				(void)	{ return(s_xHandler); }
#endif	// defined(ASURA_USE_MULTIPLE_HANDLERS)


private:
#if defined(ASURA_USE_MULTIPLE_HANDLERS)
	// Private variables...
	static	Asura_AnimTable_Handler*						s_pxCurrentHandler;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_AnimTable_Handler>	s_xHandlerTree;			// Tree of all of the handlers available.
#else
	// Private variables...
	static	Asura_AnimTable_Handler							s_xHandler;				// Single static-instance of a zone.
#endif

	// Private functions...
	static	int							Project_ConvertHashToAnimState	(Asura_Hash_ID uID);
};
/* end of class Asura_AnimTable_System */

#endif	// #else	// #ifdef ASURA_DISABLE_ANIM_TABLE_CODE

#endif//__ASURA_ANIMTABLE_SYSTEM_H__


/****************/
/* End of file. */
/****************/
