/**
*
* \file Asura_Animation_Posable.h
*
* Created by Richard May - 30/08/2002 16:19:47
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/


#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_ANIMATION_POSABLE_H__
#define __ASURA_ANIMATION_POSABLE_H__

/************/
/* Includes.*/
/************/

#include "Animation/Asura_Animation.h"

/************/
/* Classes. */
/************/

enum ASURA_ANIMATION_POSABLE_BONE
{
	// (AW - 10.05.2004) - An enumeration of the bones that the posable animation system
	// is interested in.
	ASURA_ANIMATION_POSABLE_BONE_BIP_01 = 0,
	ASURA_ANIMATION_POSABLE_BONE_PELVIS,
	ASURA_ANIMATION_POSABLE_BONE_TURRET,
	ASURA_ANIMATION_POSABLE_BONE_SPINE_01,
	ASURA_ANIMATION_POSABLE_BONE_SPINE_02,   // Optional. Set bone index to -1 if doesn't exist.
	ASURA_ANIMATION_POSABLE_BONE_NECK,
	ASURA_ANIMATION_POSABLE_BONE_HEAD,
	NUM_ASURA_ANIMATION_POSABLE_BONES
};

/*************************************************************
 *
 * Class Name    : Asura_Animation_Posable
 *
 * Author        : Richard May
 *
 * Description   : An orientatable Animation is an asura animation
 *			       that can have certain bones reorientated to
 *				   allow for things like torso-twist and head tilt.
 *				   The current version supports Torso pitch & yaw,
 *				   Head pitch & yaw and Right Arm pitch & yaw.
 *
 *************************************************************/
class Asura_Animation_Posable : public Asura_Animation
{
	friend class Asura_Hierarchy_System;

public:
	/* Constructors/destructor. */
    Asura_Animation_Posable( Asura_Hash_ID uSkinNameHash );
    Asura_Animation_Posable( const char* szSkinName );
	~Asura_Animation_Posable( void );

	virtual void Copy( const Asura_Animation* pxSrcAnim, bool bCopySkin, bool bCopyOneShotAnims, bool bCopyAttachments );
    
    void ResetAllOrientations( void );

	void SetHipOrient( const Asura_Euler_3& xHipEuler )
	{
		m_xHipOrient = xHipEuler;
	}
	void SetHeadOrientation( const Asura_Euler_3& xHeadEuler )
	{
		m_xHeadOrient = xHeadEuler;		
	}

	const Asura_Euler_3& GetHipOrient(void) const
	{
		return m_xHipOrient;
	}
	const Asura_Euler_3& GetHeadOrient(void) const
	{
		return m_xHeadOrient;
	}
	// KOR - 23/01/2006 - if your UpdatePoseableBones does not use the posable bone index array,
	// return NULL from GetPosableBoneIndexArray() to avoid the assert in Asura_Hierarchy_System.cpp just before UpdatePosableBones()
	virtual const int *GetPosableBoneIndexArray() const = 0;
	virtual void UpdatePosableBones();

	// This is called before the tween store is updated in UpdateBoneData to allow anims to be set
	virtual void UpdatePoseAnims() {}
protected :
	Asura_Quat Asura_Quat_MultiplyByInverse(const Asura_Quat& q1, const Asura_Quat& q2);
	inline Asura_Quat Asura_Quat_Invert(const Asura_Quat& q)
	{
		return Asura_Quat(-q.x, -q.y, -q.z, q.w);
	}
 
	Asura_Euler_3 m_xHipOrient;
	Asura_Euler_3 m_xHeadOrient;
};
/* end of class Asura_Animation_Posable */


/****************/
/* End of file. */
/****************/
#endif//__ASURA_ANIMATION_POSABLE_H__
