
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Hierarchy_Tween.h
// Author : Andy Tate
// Date   : 15-Sep-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_HIERARCHY_TWEEN_H__
#define __ASURA_HIERARCHY_TWEEN_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_Maths.h"
#include "Core/Memory/Asura_Mem.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

#if (ASURA_MAX_NO_OF_BONES_PER_MODEL > 255)
#error "Too many bones defined per model - maximum is 255."
#endif	//(ASURA_MAX_NO_OF_BONES_PER_MODEL > 255)


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Hierarchy_TweenStore_BoneData
// Author       : Andy Tate
// Date         : 10-Jul-2007
//
// Description  : Class used to hold the bone-data used by the tween-store.
////////////////////////////////////////////////////////////////////////////////

class Asura_Hierarchy_TweenStore_BoneData
{
public:
	// Public functions...
	Asura_Hierarchy_TweenStore_BoneData  (void)	{}
//	~Asura_Hierarchy_TweenStore_BoneData ();

	// Assignment operators...
	Asura_Hierarchy_TweenStore_BoneData& operator=(const Asura_Hierarchy_TweenStore_BoneData& xOther);

	// Add functions (used for deltas)...
	void AddBoneData(const Asura_Quat& xOrientation);
	void AddBoneData(const Asura_Vector_3& xPosition);
	void AddBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther);
	void AddFromEmptyBoneData(const Asura_Quat& xOrientation);
	void AddFromEmptyBoneData(const Asura_Vector_3& xPosition);
	void AddFromEmptyBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther);

	void AddBoneData(const Asura_Quat& xOrientation, const float fWeight);
	void AddBoneData(const Asura_Vector_3& xPosition, const float fWeight);
	void AddBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther, const float fWeight);
	void AddFromEmptyBoneData(const Asura_Quat& xOrientation, const float fWeight);
	void AddFromEmptyBoneData(const Asura_Vector_3& xPosition, const float fWeight);
	void AddFromEmptyBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther, const float fWeight);

	// Blend functions...
	void BlendBoneData(const Asura_Quat& xOrientation);
	void BlendBoneData(const Asura_Vector_3& xPosition);
	void BlendBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther);

	void BlendBoneData(const Asura_Quat& xOrientation, const float fWeight);
	void BlendBoneData(const Asura_Vector_3& xPosition, const float fWeight);
	void BlendBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther, const float fWeight);

	// Accessor functions...
	void Reset(void);

	void SetPosition(const Asura_Vector_3& xPosition);
	void SetOrientation(const Asura_Quat& xOrientation);
	void SetOrientation(const Asura_Matrix_3x3& xOrientation);

	void GetPosition(Asura_Vector_3& xPosition) const;
	void GetOrientation(Asura_Quat& xOrientation) const;
	void GetOrientation(Asura_Matrix_3x3& xOrientation) const;
	void GetTransform(Asura_AMatrix4x4_OutParam xTransform) const;

	float GetPositionMagnitude(void) const	{ return(m_xPosition.Magnitude()); }


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...
	Asura_Vector_3	m_xPosition;
	Asura_Quat		m_xOrientation;

	// Private functions...
	Asura_Hierarchy_TweenStore_BoneData  (const Asura_Hierarchy_TweenStore_BoneData& xOther);

};   // End of class 'Asura_Hierarchy_TweenStore_BoneData'.


//////////////////////////////////////////////////////////////////////////
// Inlined functions...
//////////////////////////////////////////////////////////////////////////
inline Asura_Hierarchy_TweenStore_BoneData& Asura_Hierarchy_TweenStore_BoneData::operator=(const Asura_Hierarchy_TweenStore_BoneData& xOther)
{
	m_xPosition		= xOther.m_xPosition;
	m_xOrientation	= xOther.m_xOrientation;
	return(*this);
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddBoneData(const Asura_Quat& xOrientation)
{
	m_xOrientation	= m_xOrientation * xOrientation;
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddBoneData(const Asura_Vector_3& xPosition)
{
	Asura_Vector_3	xRotatedPosition = m_xOrientation.RotateVector(xPosition);

	m_xPosition	+= xRotatedPosition;
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther)
{
	// Position then orientation since we need the result of the orientation for the position calculation...
	AddBoneData(xOther.m_xPosition);
	AddBoneData(xOther.m_xOrientation);
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddFromEmptyBoneData(const Asura_Quat& xOrientation)
{
	m_xOrientation	= xOrientation;
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddFromEmptyBoneData(const Asura_Vector_3& xPosition)
{
	m_xPosition	= xPosition;
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddFromEmptyBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther)
{
	// Order doesn't matter when coming from an empty rotation...
	AddFromEmptyBoneData(xOther.m_xPosition);
	AddFromEmptyBoneData(xOther.m_xOrientation);
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddBoneData(const Asura_Quat& xOrientation, const float fWeight)
{
	m_xOrientation	= m_xOrientation * xOrientation.ScaleRotation(fWeight);
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddBoneData(const Asura_Vector_3& xPosition, const float fWeight)
{
	Asura_Vector_3	xRotatedPosition = m_xOrientation.RotateVector(xPosition);

	m_xPosition	+= (xRotatedPosition * fWeight);
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther, const float fWeight)
{
	// Position then orientation since we need the result of the orientation for the position calculation...
	AddBoneData(xOther.m_xPosition, fWeight);
	AddBoneData(xOther.m_xOrientation, fWeight);
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddFromEmptyBoneData(const Asura_Quat& xOrientation, const float fWeight)
{
	m_xOrientation	= xOrientation.ScaleRotation(fWeight);
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddFromEmptyBoneData(const Asura_Vector_3& xPosition, const float fWeight)
{
	m_xPosition	= (xPosition * fWeight);
}

inline void Asura_Hierarchy_TweenStore_BoneData::AddFromEmptyBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther, const float fWeight)
{
	// Order doesn't matter when coming from an empty rotation...
	AddFromEmptyBoneData(xOther.m_xPosition, fWeight);
	AddFromEmptyBoneData(xOther.m_xOrientation, fWeight);
}

inline void Asura_Hierarchy_TweenStore_BoneData::BlendBoneData(const Asura_Quat& xOrientation)
{
	m_xOrientation	= xOrientation;
}

inline void Asura_Hierarchy_TweenStore_BoneData::BlendBoneData(const Asura_Vector_3& xPosition)
{
	m_xPosition	= xPosition;
}

inline void Asura_Hierarchy_TweenStore_BoneData::BlendBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther)
{
	BlendBoneData(xOther.m_xPosition);
	BlendBoneData(xOther.m_xOrientation);
}

inline void Asura_Hierarchy_TweenStore_BoneData::BlendBoneData(const Asura_Quat& xOrientation, const float fWeight)
{
	m_xOrientation.Slerp(xOrientation, fWeight);
}

inline void Asura_Hierarchy_TweenStore_BoneData::BlendBoneData(const Asura_Vector_3& xPosition, const float fWeight)
{
	m_xPosition.Lerp(xPosition, fWeight);
}

inline void Asura_Hierarchy_TweenStore_BoneData::BlendBoneData(const Asura_Hierarchy_TweenStore_BoneData& xOther, const float fWeight)
{
	BlendBoneData(xOther.m_xPosition, fWeight);
	BlendBoneData(xOther.m_xOrientation, fWeight);
}

inline void Asura_Hierarchy_TweenStore_BoneData::Reset(void)
{
	m_xPosition.Zero();
	m_xOrientation.Identity();
}

inline void Asura_Hierarchy_TweenStore_BoneData::SetPosition(const Asura_Vector_3& xPosition)
{
	m_xPosition = xPosition;
}

inline void Asura_Hierarchy_TweenStore_BoneData::GetPosition(Asura_Vector_3& xPosition) const
{
	xPosition	= m_xPosition;
}

inline void Asura_Hierarchy_TweenStore_BoneData::SetOrientation(const Asura_Quat& xOrientation)
{
	m_xOrientation = xOrientation;
}

inline void Asura_Hierarchy_TweenStore_BoneData::GetOrientation(Asura_Quat& xOrientation) const
{
	xOrientation	= m_xOrientation;
}

inline void Asura_Hierarchy_TweenStore_BoneData::SetOrientation(const Asura_Matrix_3x3& xOrientation)
{
	m_xOrientation.FromMatrix(xOrientation);
}

inline void Asura_Hierarchy_TweenStore_BoneData::GetOrientation(Asura_Matrix_3x3& xOrientation) const
{
	m_xOrientation.ToMatrix(xOrientation);
}

inline void Asura_Hierarchy_TweenStore_BoneData::GetTransform(Asura_AMatrix4x4_OutParam xTransform) const
{
	const Asura_AMatrix4x4	xTemp(m_xOrientation, m_xPosition);

	xTransform	= xTemp;
}


////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Hierarchy_Tween
// Author       : Andy Tate
// Date         : 10-Jul-2007
//
// Description  : Class used to tween between multiple animations.
////////////////////////////////////////////////////////////////////////////////

class Asura_Hierarchy_Tween
{
public:
	// Constructor(s)/destructor...
	Asura_Hierarchy_Tween(void);
	~Asura_Hierarchy_Tween();

	void SetNumberOfBones(const u_int uNumberOfBones);
	void DeleteTweenStore(void);
	void Copy(const Asura_Hierarchy_Tween* pxTween);

	// Invalidate...
	void Invalidate(void);

	// Special pre-fetch function...
	void PreFetchBoneData(const u_int uBone);

	// Bone-data query functions...
	bool IsBoneDataValid(const u_int uBone) const;
	bool IsBoneDataDelta(const u_int uBone) const;

	// Bone functions...
	void SetBoneData(const u_int uBone, const Asura_Hierarchy_Bone_RestFrame& xRestFrameBone);

	void AddBoneData(const u_int uBone, const Asura_Quat& xOrientation, const float fWeight = 1.0f);
	void AddBoneData(const u_int uBone, const Asura_Vector_3& xPosition, const float fWeight = 1.0f);
	void AddBoneData(const u_int uBone, const Asura_Hierarchy_TweenStore_BoneData& xBoneData, const float fWeight = 1.0f);

	void BlendBoneData(const u_int uBone, const Asura_Vector_3& xPosition, const bool bDelta, const float fWeight = 1.0f);
	void BlendBoneData(const u_int uBone, const Asura_Quat& xOrientation, const bool bDelta, const float fWeight = 1.0f);
	void BlendBoneData(const u_int uBone, const Asura_Hierarchy_TweenStore_BoneData& xBoneData, const bool bDelta, const float fWeight = 1.0f);

	// Velocity tween functions...
	void AddVelocity(const Asura_Vector_3& xVelocity, const float fWeight);
	bool BlendEmptyVelocity(const float fWeight);
	void BlendVelocity(const Asura_Vector_3& xVelocity, const bool bDelta, const float fWeight, const bool bSlerp);

	// Angular-velocity tween functions...
	void AddAngularVelocity(const Asura_Vector_3& xAxis, const Asura_Angle fAngle, const float fWeight);
	bool BlendEmptyAngularVelocity(const float fWeight);
	void BlendAngularVelocity(const Asura_Vector_3& xAxis, const Asura_Angle fAngle, const bool bDelta, const float fWeight);

	// Movement-offset tween functions...
	void AddMovementOffset(const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const float fWeight);
	bool BlendEmptyMovementOffset(const float fWeight);
	void BlendMovementOffset(const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const bool bDelta, const float fWeight);

	// Tween-store tween functions...
	void AddBoneDataIntoTweenStore(const Asura_Hierarchy_Skin& xSkin, const u_int uBone, const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const;
	void BlendBoneDataIntoTweenStore(const Asura_Hierarchy_Skin& xSkin, const u_int uBone, const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const;

	void AddBoundingBoxIntoTweenStore(const Asura_Hierarchy_Skin& xSkin, const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const;
	void BlendBoundingBoxIntoTweenStore(const Asura_Hierarchy_Skin& xSkin, const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const;

	void AddVelocityIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const;
	void BlendVelocityIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore, const bool bSlerp) const;

	void AddAngularVelocityIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const;
	void BlendAngularVelocityIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const;

	void AddMovementOffsetIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const;
	void BlendMovementOffsetIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const;

	// Fill unused bone functions...
	void FillBoneDataWithRestPose(const Asura_Hierarchy_Skin& xSkin, const u_int uFirstBone, const u_int uLastBone);

	// Accessor functions...
	bool AreAllBonesUsingDeltas(void) const	{ return((m_uNumValidBones > 0) && (m_uNumDeltaBones == m_uNumValidBones)); }
	bool AreAnyBonesUsingDeltas(void) const	{ return(m_uNumDeltaBones > 0); }

	const Asura_Hierarchy_TweenStore_BoneData&	GetBoneData(const u_int uIndex) const;
	Asura_Hierarchy_TweenStore_BoneData&		GetBoneData(const u_int uIndex);

	bool GetVelocity(Asura_Vector_3& xVelocity) const;
	bool GetAngularVelocity(Asura_Vector_3& xAxis, Asura_Angle& fAngle) const;
	bool GetMovementOffset(Asura_Vector_3* pxPosition, Asura_Quat* pxOrientation) const;

	bool HasBoneData(void) const;

	u_int GetNumberOfBones(void) const;

	bool IsPositionDataValid(void) const;
	void SetPositionDataValid(const bool bSet);

	bool IsPositionDataInWorldspace(void) const;
	void SetPositionDataInWorldspace(const bool bSet);

	void SetBlendError( const bool bError = true ) { m_bBlendError = bError; }
	bool IsBlendError() const { return m_bBlendError; }

protected:
	// Protected functions...
	void SetBoneDataValid(const u_int uBone, const bool bDelta);


private:
	// Private enumerated types...
	enum
	{
		BONE_POSITION_DATA_VALID_BITS_PER_ENTRY	= (sizeof(u_int) * 8),
		NUM_BONE_POSITION_DATA_VALID_ENTRIES	= ((ASURA_MAX_NO_OF_BONES_PER_MODEL + (BONE_POSITION_DATA_VALID_BITS_PER_ENTRY - 1)) / BONE_POSITION_DATA_VALID_BITS_PER_ENTRY),
	};

	// Private variables...
	union
	{
		u_int		m_uFlags;
		struct 
		{
			u_int	m_uNumberOfBones			: 8,	// Total number of bones in this tween-store (maximum 255).
					m_uNumValidBones			: 8,	// The number of valid bones in this tween-store (maximum 255).
					m_uNumDeltaBones			: 8,	// The number of bones using delta-animations in this tween-store (maximum 255).

					m_bPositionDataValid		: 1,
					m_bPositionDataInWorldspace	: 1,
					m_bTweenStartDataValid		: 1,	// Used to check whether a frame has been captured in the store yet.

					m_bHasVelocity				: 1,
					m_bHasAngularVelocity		: 1,
					m_bHasMovementOffset		: 1,

					m_bBlendError				: 1,

												: 0;	// 1 spare-bits
		};
	};

	u_int									m_auBoneDataValid[NUM_BONE_POSITION_DATA_VALID_ENTRIES];	// Bits to say which of the bones has valid data.
	u_int									m_auBoneDataDelta[NUM_BONE_POSITION_DATA_VALID_ENTRIES];	// Bits to say which of the bones has delta animation data.

	Asura_Vector_3							m_xVelocity;				// The current velocity of the animation.
	Asura_Vector_3							m_xAngularVelocityAxis;		// The axis for the current angular-velocity of the animation.
	Asura_Angle								m_fAngularVelocityAngle;	// The angle for the current angular-velocity of the animation.
	Asura_Vector_3							m_xMovementOffsetPosition;	// The position of the current movement-offset of the animation.
	Asura_Quat								m_xMovementOffsetQuat;		// The orientation of the current movement-offset of the animation.

	Asura_Hierarchy_TweenStore_BoneData*	m_pxBoneData;				// The array of bone data for this tween-store.
};   // End of class 'Asura_Hierarchy_Tween'.


//////////////////////////////////////////////////////////////////////////
// Inlined functions...
//////////////////////////////////////////////////////////////////////////
inline Asura_Hierarchy_Tween::Asura_Hierarchy_Tween(void)
{
	m_uFlags = 0;

	m_xVelocity.Zero();
	m_xAngularVelocityAxis	= Asura_Vector_3(0.0f, 0.0f, 1.0f);
	m_fAngularVelocityAngle	= 0.0f;
	m_xMovementOffsetPosition.Zero();
	m_xMovementOffsetQuat.Identity();
	m_pxBoneData	= NULL;
}

inline Asura_Hierarchy_Tween::~Asura_Hierarchy_Tween()
{
	DeleteTweenStore();
}

inline void Asura_Hierarchy_Tween::Invalidate(void)
{
	// Ensure that we start with no valid bones...
	for(u_int u = 0; u < NUM_BONE_POSITION_DATA_VALID_ENTRIES; u++)
	{
		m_auBoneDataValid[u]	= 0;
		m_auBoneDataDelta[u]	= 0xFFFFFFFF;
	}

	// Reset the bone counts...
	m_uNumValidBones	= 0;
	m_uNumDeltaBones	= 0;

	// Reset the velocity flags...
	m_bHasVelocity		  	= false;
	m_bHasAngularVelocity 	= false;
	m_bHasMovementOffset  	= false;

	m_bBlendError			= false;
}

inline void Asura_Hierarchy_Tween::PreFetchBoneData(const u_int uBone)
{
	Asura_Mem::PreFetchData(&m_pxBoneData[uBone], sizeof(*m_pxBoneData));
}

inline void	Asura_Hierarchy_Tween::SetBoneDataValid(const u_int uBone, const bool bDelta)
{
	const u_int	uArrayIndex	= uBone / BONE_POSITION_DATA_VALID_BITS_PER_ENTRY;
	const u_int	uBitShift	= uBone & (BONE_POSITION_DATA_VALID_BITS_PER_ENTRY - 1);
	const u_int	uBitMask	= 1 << uBitShift;

	// Have we alread set this bone as valid?
	if((m_auBoneDataValid[uArrayIndex] & uBitMask) == 0)
	{
		// No - do so, and make sure we update whether its a delta or not...
		m_auBoneDataValid[uArrayIndex]	|= uBitMask;
		m_uNumValidBones++;

		if(bDelta)
		{
			m_uNumDeltaBones++;
		}
		else
		{
			m_auBoneDataDelta[uArrayIndex]	&= ~uBitMask;
		}
	}
	else
	{
		//////////////////////////////////////////////////////////////////////////
		// We don't need to do anything here since we cannot change the 
		// type of a bone once its been made valid...
		// - non-deltas can ONLY be blended onto other non-deltas
		// - deltas can be blended onto either non-deltas or deltas BUT the type doesn't change.
		//////////////////////////////////////////////////////////////////////////
	}
}

inline bool	Asura_Hierarchy_Tween::IsBoneDataValid(const u_int uBone) const
{
	const u_int	uArrayIndex	= uBone / BONE_POSITION_DATA_VALID_BITS_PER_ENTRY;
	const u_int	uBitShift	= uBone & (BONE_POSITION_DATA_VALID_BITS_PER_ENTRY - 1);
	const u_int	uBitMask	= 1 << uBitShift;

	return((m_auBoneDataValid[uArrayIndex] & uBitMask) != 0);
}

inline bool	Asura_Hierarchy_Tween::IsBoneDataDelta(const u_int uBone) const
{
	const u_int	uArrayIndex	= uBone / BONE_POSITION_DATA_VALID_BITS_PER_ENTRY;
	const u_int	uBitShift	= uBone & (BONE_POSITION_DATA_VALID_BITS_PER_ENTRY - 1);
	const u_int	uBitMask	= 1 << uBitShift;

	return((m_auBoneDataDelta[uArrayIndex] & uBitMask) != 0);
}

inline void Asura_Hierarchy_Tween::SetBoneData(const u_int uBone, const Asura_Hierarchy_Bone_RestFrame& xRestFrameBone)
{
	Asura_Hierarchy_TweenStore_BoneData&	xMyBoneData	= m_pxBoneData[uBone];

	xMyBoneData.SetPosition(xRestFrameBone.Position);
	xMyBoneData.SetOrientation(xRestFrameBone.Quat);
	SetBoneDataValid(uBone, false);
}

inline void Asura_Hierarchy_Tween::AddBoneData(const u_int uBone, const Asura_Quat& xOrientation, const float fWeight)
{
	Asura_Hierarchy_TweenStore_BoneData&	xMyBoneData	= m_pxBoneData[uBone];

	if(IsBoneDataValid(uBone))
	{
		if(fWeight >= 1.0f)
		{
			xMyBoneData.AddBoneData(xOrientation);
		}
		else
		{
			xMyBoneData.AddBoneData(xOrientation, fWeight);
		}
	}
	else
	{
		if(fWeight >= 1.0f)
		{
			xMyBoneData.AddFromEmptyBoneData(xOrientation);
		}
		else
		{
			xMyBoneData.AddFromEmptyBoneData(xOrientation, fWeight);
		}
	}

	SetBoneDataValid(uBone, true);
}

inline void Asura_Hierarchy_Tween::AddBoneData(const u_int uBone, const Asura_Vector_3& xPosition, const float fWeight)
{
	Asura_Hierarchy_TweenStore_BoneData&	xMyBoneData	= m_pxBoneData[uBone];

	if(IsBoneDataValid(uBone))
	{
		if(fWeight >= 1.0f)
		{
			xMyBoneData.AddBoneData(xPosition);
		}
		else
		{
			xMyBoneData.AddBoneData(xPosition, fWeight);
		}
	}
	else
	{
		if(fWeight >= 1.0f)
		{
			xMyBoneData.AddFromEmptyBoneData(xPosition);
		}
		else
		{
			xMyBoneData.AddFromEmptyBoneData(xPosition, fWeight);
		}
	}

	// Bone-data only needs to be set valid for orientations since they ALWAYS come after positions...
	//SetBoneDataValid(uBone, true);
}

inline void Asura_Hierarchy_Tween::AddBoneData(const u_int uBone, const Asura_Hierarchy_TweenStore_BoneData& xBoneData, const float fWeight)
{
	Asura_Hierarchy_TweenStore_BoneData&	xMyBoneData	= m_pxBoneData[uBone];

	if(IsBoneDataValid(uBone))
	{
		if(fWeight >= 1.0f)
		{
			xMyBoneData.AddBoneData(xBoneData);
		}
		else
		{
			xMyBoneData.AddBoneData(xBoneData, fWeight);
		}
	}
	else
	{
		if(fWeight >= 1.0f)
		{
			xMyBoneData.AddFromEmptyBoneData(xBoneData);
		}
		else
		{
			xMyBoneData.AddFromEmptyBoneData(xBoneData, fWeight);
		}
	}

	SetBoneDataValid(uBone, true);
}

inline void Asura_Hierarchy_Tween::BlendBoneData(const u_int uBone, const Asura_Vector_3& xPosition, const bool bDelta, const float fWeight)
{
	Asura_Hierarchy_TweenStore_BoneData&	xMyBoneData	= m_pxBoneData[uBone];

	if(fWeight >= 1.0f)
	{
		xMyBoneData.BlendBoneData(xPosition);

		// Bone-data only needs to be set valid for orientations since they ALWAYS come after positions...
		//SetBoneDataValid(uBone, bDelta);
	}
	else
	{
		if(IsBoneDataValid(uBone))
		{
			if(!bDelta && IsBoneDataDelta(uBone))
			{
				m_bBlendError = true;
				Asura_Assertf(false, ("Blending non-delta bone %d over a delta bone is not allowed.", uBone));
			}
			else
			{
				xMyBoneData.BlendBoneData(xPosition, fWeight);

				// Bone-data only needs to be set valid for orientations since they ALWAYS come after positions...
				//SetBoneDataValid(uBone, bDelta);
			}
		}
		else
		{
			if(bDelta)
			{
				xMyBoneData.AddFromEmptyBoneData(xPosition, fWeight);
			}
			else
			{
				m_bBlendError = true;
				Asura_Assertf(false, ("Blending bone %d from an invalid bone is not allowed.", uBone));
			}
		}
	}
}

inline void Asura_Hierarchy_Tween::BlendBoneData(const u_int uBone, const Asura_Quat& xOrientation, const bool bDelta, const float fWeight)
{
	Asura_Hierarchy_TweenStore_BoneData&	xMyBoneData	= m_pxBoneData[uBone];

	if(fWeight >= 1.0f)
	{
		xMyBoneData.BlendBoneData(xOrientation);
		SetBoneDataValid(uBone, bDelta);
	}
	else
	{
		if(IsBoneDataValid(uBone))
		{
			if(!bDelta && IsBoneDataDelta(uBone))
			{
				m_bBlendError = true;
				Asura_Assertf(false, ("Blending non-delta bone %d over a delta bone is not allowed.", uBone));
			}
			else
			{
				xMyBoneData.BlendBoneData(xOrientation, fWeight);
				SetBoneDataValid(uBone, bDelta);
			}
		}
		else
		{
			if(bDelta)
			{
				xMyBoneData.AddFromEmptyBoneData(xOrientation, fWeight);
			}
			else
			{
				m_bBlendError = true;
				Asura_Assertf(false, ("Blending bone %d from an invalid bone is not allowed.", uBone));
			}
		}
	}
}

inline void Asura_Hierarchy_Tween::BlendBoneData(const u_int uBone, const Asura_Hierarchy_TweenStore_BoneData& xBoneData, const bool bDelta, const float fWeight)
{
	Asura_Hierarchy_TweenStore_BoneData&	xMyBoneData	= m_pxBoneData[uBone];

	if(fWeight >= 1.0f)
	{
		xMyBoneData.BlendBoneData(xBoneData);
		SetBoneDataValid(uBone, bDelta);
	}
	else
	{
		if(IsBoneDataValid(uBone))
		{
			if(!bDelta && IsBoneDataDelta(uBone))
			{
				m_bBlendError = true;
				Asura_Assertf(false, ("Blending non-delta bone %d over a delta bone is not allowed.", uBone));
			}
			else
			{
				xMyBoneData.BlendBoneData(xBoneData, fWeight);
				SetBoneDataValid(uBone, bDelta);
			}
		}
		else
		{
			if(bDelta)
			{
				xMyBoneData.AddFromEmptyBoneData(xBoneData, fWeight);
			}
			else
			{
				m_bBlendError = true;
				Asura_Assertf(false, ("Blending bone %d from an invalid bone is not allowed.", uBone));
			}
		}
	}
}

inline void Asura_Hierarchy_Tween::AddBoneDataIntoTweenStore(const Asura_Hierarchy_Skin& xSkin, const u_int uBone, const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const
{
	if(IsBoneDataValid(uBone))
	{
		if(xIntoTweenStore.IsBoneDataValid(uBone))
		{
			xIntoTweenStore.AddBoneData(uBone, m_pxBoneData[uBone], fWeight);
		}
		else
		{
			xIntoTweenStore.SetBoneData(uBone, xSkin.BoneRestPose[uBone]);
			xIntoTweenStore.AddBoneData(uBone, m_pxBoneData[uBone], fWeight);
		}
	}
}

inline void Asura_Hierarchy_Tween::BlendBoneDataIntoTweenStore(const Asura_Hierarchy_Skin& xSkin, const u_int uBone, const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const
{
	if(IsBoneDataValid(uBone))
	{
		const bool	bDelta	= IsBoneDataDelta(uBone);

		xIntoTweenStore.BlendBoneData(uBone, m_pxBoneData[uBone], bDelta, fWeight);
	}
}

inline void Asura_Hierarchy_Tween::FillBoneDataWithRestPose(const Asura_Hierarchy_Skin& xSkin, const u_int uFirstBone, const u_int uLastBone)
{
	const Asura_Hierarchy_Bone_RestFrame*	pxFromRestFrame	= &xSkin.BoneRestPose[uFirstBone];
	Asura_Hierarchy_TweenStore_BoneData*	pxToBoneData	= &m_pxBoneData[uFirstBone];

	// Go through each bone in-turn...
	for(u_int uBone = uFirstBone; uBone <= uLastBone; uBone++, pxFromRestFrame++, pxToBoneData++)
	{
		if(!IsBoneDataValid(uBone))
		{
			SetBoneData(uBone, *pxFromRestFrame);
		}
	}
}

inline void Asura_Hierarchy_Tween::AddVelocity(const Asura_Vector_3& xVelocity, const float fWeight)
{
	if(m_bHasVelocity)
	{
		if(fWeight >= 1.0f)
		{
			m_xVelocity	+= xVelocity;
		}
		else
		{
			m_xVelocity	+= (xVelocity * fWeight);
		}
	}
	else
	{
		m_xVelocity		= xVelocity * fWeight;
		m_bHasVelocity	= true;
	}
}

inline void Asura_Hierarchy_Tween::AddVelocityIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const
{
	if(m_bHasVelocity)
	{
		xIntoTweenStore.AddVelocity(m_xVelocity, fWeight);
	}
}

inline bool Asura_Hierarchy_Tween::BlendEmptyVelocity(const float fWeight)
{
	if(m_bHasVelocity)
	{
		if(fWeight >= 1.0f)
		{
			m_xVelocity	= Asura_Maths::ZeroVector3;
			return(true);
		}
		else
		{
			m_xVelocity.Lerp(Asura_Maths::ZeroVector3, fWeight);
			return(true);
		}
	}
	else
	{
		m_xVelocity	= Asura_Maths::ZeroVector3;
		m_bHasVelocity = true;
		return(true);
	}
}

inline void Asura_Hierarchy_Tween::BlendVelocity(const Asura_Vector_3& xVelocity, const bool bDelta, const float fWeight, const bool bSlerp)
{
	if(fWeight >= 1.0f)
	{
		m_xVelocity		= xVelocity;
		m_bHasVelocity	= true;
	}
	else
	{
		if(m_bHasVelocity)
		{
			if(!bDelta && IsBoneDataDelta(0))
			{
				m_bBlendError = true;
				Asura_Assertf(false, ("Blending non-delta velocity over a delta velocity is not allowed."));
			}
			else
			{
				if ( bSlerp )
				{
					// This could be moved into Asura_AVector3::Slerp or maybe Asura_AVector3::SlerpWithMagnitude as it's becoming a common thing to do. (Also used in Direction_Slerpolator with no magnitude)
					Asura_AVector3 xOutVec;
					Asura_AVector3 xNormFrom( m_xVelocity );
					Asura_AVector3 xNormTo( xVelocity );
					const float fFromMag = xNormFrom.NormaliseReturningMagnitude();
					float fToMag = xNormTo.NormaliseReturningMagnitude();
					fToMag = Asura_Maths::Lerp( fFromMag, fToMag, fWeight );
					Asura_Quat xRot;
					Asura_AVector3 xAxis;
					xRot.RotationFromAToB( xNormFrom, xNormTo );
					float fAngle = xRot.ToRotation( xAxis );
					fAngle *= fWeight;
					xRot.FromRotation( fAngle, xAxis );
					xOutVec = xRot.RotateVector(xNormFrom);
					xOutVec *= fToMag;
					m_xVelocity = Asura_Vector_3( xOutVec );
				}
				else
				{
					m_xVelocity.Lerp(xVelocity, fWeight);
				}
			}
		}
		else
		{
			m_xVelocity		= xVelocity * fWeight;
			m_bHasVelocity	= true;
		}
	}
}

inline void Asura_Hierarchy_Tween::BlendVelocityIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore, const bool bSlerp) const
{
	if(m_bHasVelocity)
	{
		const bool	bDelta	= IsBoneDataDelta(0);

		xIntoTweenStore.BlendVelocity(m_xVelocity, bDelta, fWeight, bSlerp);
	}
}

inline void Asura_Hierarchy_Tween::AddAngularVelocity(const Asura_Vector_3& xAxis, const Asura_Angle fAngle, const float fWeight)
{
	if(m_bHasAngularVelocity)
	{
		if(fWeight >= 1.0f)
		{
			m_xAngularVelocityAxis *= m_fAngularVelocityAngle;
			m_xAngularVelocityAxis += xAxis * fAngle;
			m_fAngularVelocityAngle = m_xAngularVelocityAxis.Normalise();
		}
		else
		{
			m_xAngularVelocityAxis *= m_fAngularVelocityAngle;
			m_xAngularVelocityAxis += xAxis * fAngle * fWeight;
			m_fAngularVelocityAngle = m_xAngularVelocityAxis.Normalise();
		}
	}
	else
	{
		m_xAngularVelocityAxis	= xAxis;
		m_fAngularVelocityAngle = fAngle * fWeight;
		m_bHasAngularVelocity	= true;
	}
}

inline void Asura_Hierarchy_Tween::AddAngularVelocityIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const
{
	if(m_bHasAngularVelocity)
	{
		xIntoTweenStore.AddAngularVelocity(m_xAngularVelocityAxis, m_fAngularVelocityAngle, fWeight);
	}
}

inline bool Asura_Hierarchy_Tween::BlendEmptyAngularVelocity(const float fWeight)
{
	if(m_bHasAngularVelocity)
	{
		if(fWeight >= 1.0f)
		{
			m_fAngularVelocityAngle	= 0.0f;
			return(true);
		}
		else
		{
			m_fAngularVelocityAngle	*= fWeight;
			return(true);
		}
	}
	else
	{
		m_fAngularVelocityAngle	= 0.0f;
		m_bHasAngularVelocity = true;
		return(true);
	}
}

inline void Asura_Hierarchy_Tween::BlendAngularVelocity(const Asura_Vector_3& xAxis, const Asura_Angle fAngle, const bool bDelta, const float fWeight)
{
	if(fWeight >= 1.0f)
	{
		m_xAngularVelocityAxis	= xAxis;
		m_fAngularVelocityAngle	= fAngle;
		m_bHasAngularVelocity	= true;
	}
	else
	{
		if(m_bHasAngularVelocity)
		{
			if(!bDelta && IsBoneDataDelta(0))
			{
				m_bBlendError = true;
				Asura_Assertf(false, ("Blending non-delta velocity over a delta velocity is not allowed."));
			}
			else
			{
				m_xAngularVelocityAxis *= m_fAngularVelocityAngle;
				m_xAngularVelocityAxis.Lerp( xAxis * fAngle, fWeight );
				m_bHasAngularVelocity	= true;

				m_fAngularVelocityAngle = m_xAngularVelocityAxis.Normalise();
			}
		}
		else
		{
			m_xAngularVelocityAxis	= xAxis;
			m_fAngularVelocityAngle = fAngle * fWeight;
			m_bHasAngularVelocity	= true;
		}
	}
}

inline void Asura_Hierarchy_Tween::BlendAngularVelocityIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const
{
	if(m_bHasAngularVelocity)
	{
		const bool	bDelta	= IsBoneDataDelta(0);

		xIntoTweenStore.BlendAngularVelocity(m_xAngularVelocityAxis, m_fAngularVelocityAngle, bDelta, fWeight);
	}
}

inline void Asura_Hierarchy_Tween::AddMovementOffset(const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const float fWeight)
{
	if(m_bHasMovementOffset)
	{
		if(fWeight >= 1.0f)
		{
			m_xMovementOffsetPosition	+= xPosition;
			m_xMovementOffsetQuat		 = m_xMovementOffsetQuat * xOrientation;
		}
		else
		{
			m_xMovementOffsetPosition	+= (xPosition * fWeight);
			m_xMovementOffsetQuat		 = m_xMovementOffsetQuat * xOrientation.ScaleRotation(fWeight);
		}
	}
	else
	{
		m_xMovementOffsetPosition	= xPosition * fWeight;
		m_xMovementOffsetQuat		= xOrientation.ScaleRotation(fWeight);
		m_bHasMovementOffset		= true;
	}
}

inline void Asura_Hierarchy_Tween::AddMovementOffsetIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const
{
	if(m_bHasMovementOffset)
	{
		xIntoTweenStore.AddMovementOffset(m_xMovementOffsetPosition, m_xMovementOffsetQuat, fWeight);
	}
}

inline bool Asura_Hierarchy_Tween::BlendEmptyMovementOffset(const float fWeight)
{
	if(m_bHasMovementOffset)
	{
		if(fWeight >= 1.0f)
		{
			m_xMovementOffsetPosition	= Asura_Maths::ZeroVector3;
			m_xMovementOffsetQuat		= Asura_Maths::IdentityQuat;
			return(true);
		}
		else
		{
			m_xMovementOffsetPosition.Lerp(Asura_Maths::ZeroVector3, fWeight);
			m_xMovementOffsetQuat.Slerp(Asura_Maths::IdentityQuat, fWeight);
			return(true);
		}
	}
	else
	{
		m_xMovementOffsetPosition	= Asura_Maths::ZeroVector3;
		m_xMovementOffsetQuat		= Asura_Maths::IdentityQuat;
		m_bHasMovementOffset = true;
		return(true);
	}
}

inline void Asura_Hierarchy_Tween::BlendMovementOffset(const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const bool bDelta, const float fWeight)
{
	if(fWeight >= 1.0f)
	{
		m_xMovementOffsetPosition	= xPosition;
		m_xMovementOffsetQuat		= xOrientation;
		m_bHasMovementOffset		= true;
	}
	else
	{
		if(m_bHasMovementOffset)
		{
			if(!bDelta && IsBoneDataDelta(0))
			{
				m_bBlendError = true;
				Asura_Assertf(false, ("Blending non-delta velocity over a delta velocity is not allowed."));
			}
			else
			{
				m_xMovementOffsetPosition.Lerp(xPosition, fWeight);
				m_xMovementOffsetQuat.Slerp(xOrientation, fWeight);
				m_bHasMovementOffset	= true;
			}
		}
		else
		{
			m_xMovementOffsetPosition	= xPosition * fWeight;
			m_xMovementOffsetQuat		= xOrientation.ScaleRotation(fWeight);
			m_bHasMovementOffset		= true;
		}
	}
}

inline void Asura_Hierarchy_Tween::BlendMovementOffsetIntoTweenStore(const float fWeight, Asura_Hierarchy_Tween& xIntoTweenStore) const
{
	if(m_bHasMovementOffset)
	{
		const bool	bDelta	= IsBoneDataDelta(0);

		xIntoTweenStore.BlendMovementOffset(m_xMovementOffsetPosition, m_xMovementOffsetQuat, bDelta, fWeight);
	}
}

inline bool Asura_Hierarchy_Tween::HasBoneData(void) const
{
	return(m_pxBoneData != NULL);
}

inline const Asura_Hierarchy_TweenStore_BoneData& Asura_Hierarchy_Tween::GetBoneData(const u_int uIndex) const
{
	Asura_Assert(uIndex < m_uNumberOfBones, "Bone index out of range. Returning data for bone 0");
	if ( uIndex < m_uNumberOfBones )
	{
		return(m_pxBoneData[uIndex]);
	}
	else
	{
		return(m_pxBoneData[0]);
	}
}

inline Asura_Hierarchy_TweenStore_BoneData& Asura_Hierarchy_Tween::GetBoneData(const u_int uIndex)
{
	Asura_Assert(uIndex < m_uNumberOfBones, "Bone index out of range. Returning data for bone 0");
	if ( uIndex < m_uNumberOfBones )
	{
		return(m_pxBoneData[uIndex]);
	}
	else
	{
		return(m_pxBoneData[0]);
	}
}

inline bool Asura_Hierarchy_Tween::GetVelocity(Asura_Vector_3& xVelocity) const
{
	if(m_bHasVelocity)
	{
		xVelocity	= m_xVelocity;
		return(true);
	}
	else
	{
		return(false);
	}
}

inline bool Asura_Hierarchy_Tween::GetAngularVelocity(Asura_Vector_3& xAxis, Asura_Angle& fAngle) const
{
	if(m_bHasAngularVelocity)
	{
		xAxis	= m_xAngularVelocityAxis;
		fAngle	= m_fAngularVelocityAngle;
		return(true);
	}
	else
	{
		return(false);
	}
}

inline bool Asura_Hierarchy_Tween::GetMovementOffset(Asura_Vector_3* pxPosition, Asura_Quat* pxOrientation) const
{
	if(m_bHasMovementOffset)
	{
		if(pxPosition != NULL)
		{
			*pxPosition	= m_xMovementOffsetPosition;
		}
		if(pxOrientation != NULL)
		{
			*pxOrientation	= m_xMovementOffsetQuat;
		}
		return(true);
	}
	else
	{
		return(false);
	}
}

inline u_int Asura_Hierarchy_Tween::GetNumberOfBones(void) const
{
	return(m_uNumberOfBones);
}

inline bool Asura_Hierarchy_Tween::IsPositionDataValid(void) const
{
	return(m_bPositionDataValid);
}

inline void Asura_Hierarchy_Tween::SetPositionDataValid(const bool bSet)
{
	m_bPositionDataValid = bSet;
}

inline bool Asura_Hierarchy_Tween::IsPositionDataInWorldspace(void) const
{
	return(m_bPositionDataInWorldspace);
}

inline void Asura_Hierarchy_Tween::SetPositionDataInWorldspace(const bool bSet)
{
	m_bPositionDataInWorldspace = bSet;
}

#endif   // ifndef __ASURA_HIERARCHY_TWEEN_H__


/// End of file. ///
