#ifdef ASURA_USE_ANIMATION_IK_SOLVER

///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Animation_IK_Link.h
// Author : Will Plano
// Date   : 14/09/2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_ANIMATION_IK_LINK_H__
#define __ASURA_ANIMATION_IK_LINK_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class  : Asura_Animation_IK_Link
// Author : Will Plano
// Date   : 08/08/2007
//
// Description  : Class to store link data for inverse kinematics system.
////////////////////////////////////////////////////////////////////////////////
class Asura_Animation_IK_Link
{
public:
	Asura_Animation_IK_Link( int iBoneID = -1, int iTwistBoneID = -1 );

	// Just a bunch of getters
	const Asura_AVector3& GetPosition() const { return m_xPos; }
	const Asura_AVector3& GetAnimPosition() const { return m_xAnimPos; }
	const Asura_AVector3& GetLastPosition() const { return m_xLastPos; }
	const Asura_AVector3& GetLastAnimPosition() const { return m_xLastAnimPos; }
	const Asura_AVector3& GetVelocity() const { return m_xVel; }
	const Asura_AVector3& GetBendAxis() const { return m_xBendAxis; }
	const Asura_AVector3& GetRollAxis() const { return m_xRollAxis; }
	const Asura_Quat& GetQuat() const { return m_xQuat; }
	const Asura_Quat& GetAnimQuat() const { return m_xAnimQuat; }
	const Asura_Quat& GetLastQuat() const { return m_xLastQuat; }
	const Asura_Quat& GetLastAnimQuat() const { return m_xLastAnimQuat; }
	float GetLength() const { return m_fLength; }
	int GetBoneID() const { return m_iBoneID; }
	int GetTwistBoneID() const { return m_iTwistBoneID; }

	// These three setters will update the rest of the chain's position data (which is stored in worldspace) if the bool is set.
	// ie. setting a position will adjust the previous links orientation and setting an orientation will change the positions of all subsequent links in the chain.
	void SetPosition( const Asura_AVector3& xNewPos, bool bUpdateChain = true );
	void SetQuat( const Asura_Quat& xNewQuat, bool bUpdateChain = true );
	void IncQuat( const Asura_Quat& xIncQuat, bool bUpdateChain = true );

	// Just a bunch of setters
	void SetAnimPosition( const Asura_AVector3& xAnimPos ) { m_xAnimPos = xAnimPos; }
	void SetLastAnimPosition( const Asura_AVector3& xLastAnimPos ) { m_xLastAnimPos = xLastAnimPos; }
	void SetLastPosition( const Asura_AVector3& xLastPos ) { m_xLastPos = xLastPos; }
	void SetVelocity( const Asura_AVector3& xNewVel ) { m_xVel = xNewVel; }
	void SetAnimQuat( const Asura_Quat& xAnimQuat ) { m_xAnimQuat = xAnimQuat; }
	void SetLastQuat( const Asura_Quat& xLastQuat ) { m_xLastQuat = xLastQuat; }
	void SetLastAnimQuat( const Asura_Quat& xLastAnimQuat ) { m_xLastAnimQuat = xLastAnimQuat; }
	void SetLength( float fLength ) { m_fLength = fLength; }
	void SetBoneID( int iBoneID ) { m_iBoneID = iBoneID; }
	void SetTwistBoneID( int iTwistBoneID ) { m_iTwistBoneID = iTwistBoneID; }

	void IncPosition( const Asura_AVector3& xIncPos ) { SetPosition( m_xPos + xIncPos ); }
	void IncVelocity( const Asura_AVector3& xIncVel ) { m_xVel += xIncVel; }

	void SetAxes( const Asura_AVector3& xBendAxis, const Asura_AVector3& xRollAxis ) { m_xBendAxis = xBendAxis; m_xRollAxis = xRollAxis; }

	void SetAngularConstraintRange( const Asura_Euler_3& xMinAngles, const Asura_Euler_3& xMaxAngles ) { m_xMinAngles = xMinAngles; m_xMaxAngles = xMaxAngles; }
	// This will clamp the orientation data to the range set with the above function.
	bool SatisfyAngularConstraints();

	Asura_Animation_IK_Link* GetPrevLink() const { return m_pxPrevLink; }
	void SetPrevLink( Asura_Animation_IK_Link* xPrevLink ) { m_pxPrevLink = xPrevLink; }

	Asura_Animation_IK_Link* GetNextLink() const { return m_pxNextLink; }
	void SetNextLink( Asura_Animation_IK_Link* xNextLink ) { m_pxNextLink = xNextLink; }

	static const Asura_AVector3 s_xNullVector;

private:
	Asura_AVector3 m_xPos;
	Asura_AVector3 m_xAnimPos;
	Asura_AVector3 m_xLastPos;
	Asura_AVector3 m_xLastAnimPos;
	Asura_AVector3 m_xVel;
	Asura_AVector3 m_xBendAxis;
	Asura_AVector3 m_xRollAxis;
	Asura_Quat m_xQuat;
	Asura_Quat m_xAnimQuat;
	Asura_Quat m_xLastQuat;
	Asura_Quat m_xLastAnimQuat;
	int m_iBoneID;
	int m_iTwistBoneID;
	float m_fLength;

	Asura_Euler_3 m_xMinAngles;
	Asura_Euler_3 m_xMaxAngles;

	Asura_Animation_IK_Link* m_pxPrevLink; // Prev pointer to make a list of these guys.
	Asura_Animation_IK_Link* m_pxNextLink; // Next pointer to make a list of these guys.

}; // end class Asura_Animation_IK_Link

#endif // ifndef __ASURA_ANIMATION_IK_LINK_H__

#endif // ASURA_USE_ANIMATION_IK_SOLVER

/// End of file. ///
