//
// Asura_Animation_IK_Template.cpp
//
// Created by Richard Rice - 12/05/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Animation/IK/Asura_Animation_IK_Template.h"

#ifdef ASURA_USE_ANIMATION_IK_SOLVER

#include "Animation/IK/Asura_Animation_IK_Solver.h"
#include "Animation/IK/Asura_Animation_IK_Chain.h"
#include "Animation/IK/Asura_Animation_IK_Template_Chain.h"
#include "Animation/IK/Asura_Animation_IK_Template_Chain_LOS.h"
#include "Animation/IK/Asura_Animation_IK_Template_Chain_MarkerPoint.h"
#include "Animation/IK/Asura_Animation_IK_Template_System.h"
#include "Animation/Asura_Animation.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

#define ASURA_ANIMATION_IK_TEMPLATE_VERSION 0


Asura_Animation_IK_Template::Asura_Animation_IK_Template(const char* szName)
:m_xName(szName)
,m_uNumIKChains(0)
,m_ppxIKChains(0)
,m_uSkinID(0)
{
	m_uHashID = Asura_GetHashID(szName);
	Asura_Animation_IK_Template_System::AddTemplateToTree(*this);
}

Asura_Animation_IK_Template::~Asura_Animation_IK_Template()
{
	for(u_int i=0; i<m_uNumIKChains; i++)
	{
		delete m_ppxIKChains[i];
	}
	delete [] m_ppxIKChains;

	Asura_Animation_IK_Template_System::RemoveTemplateFromTree(*this);

}

void Asura_Animation_IK_Template::SetupIKSolver(Asura_Animation_IK_Solver* pxSolver, const Asura_Animation* pxAnim) const
{
	for(u_int i=0; i<m_uNumIKChains;i++)
	{
		Asura_Animation_IK_Chain* pxChain = m_ppxIKChains[i]->CreateChainFromTemplate(pxAnim);
		Asura_Assert(pxChain, "");
		if(pxChain)
		{
			pxChain->SetOwnedBySolver(true);
			pxChain->SetAxesFromRestPose(pxAnim->GetSkin()->BoneRestPose);
			pxSolver->AddChain(*pxChain);
		}
	}
}


bool Asura_Animation_IK_Template::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iVersion;
	xStream >> iVersion;

	if(iVersion > ASURA_ANIMATION_IK_TEMPLATE_VERSION)
	{
		Asura_Assertf(0, ("Unrecognised chunk version %d for IK template", iVersion));
		return false;
	}
	xStream >> m_uSkinID;
	xStream >> m_uNumIKChains;
	m_ppxIKChains = m_uNumIKChains ? new Asura_Animation_IK_Template_Chain*[m_uNumIKChains] : 0;

	//read in the IK chains.
	bool bOkay = true;
	for(u_int i=0; i<m_uNumIKChains; i++)
	{
		if(bOkay)
		{
			//read the chain type and allocate it
			int iType;
			xStream >> iType;
			
			switch(iType)
			{
				case ASURA_IKCHAIN_LOS:
					m_ppxIKChains[i] = new Asura_Animation_IK_Template_Chain_LOS();
					break;

				case ASURA_IKCHAIN_MARKERPOINT:
					m_ppxIKChains[i] = new Asura_Animation_IK_Template_Chain_MarkerPoint();
					break;
				default :
					Asura_Assertf(0, ("Unrecognised IK chain type %d", iType));
					m_ppxIKChains[i] = 0;
					bOkay = false;
					continue;
			}
			//read in the rest of the chain
			bOkay &= m_ppxIKChains[i]->ReadFromChunkStream(xStream);
		}
		else
		{
			//aborted reading the chunk , so just zero the remaining pointers to aid with clean up
			m_ppxIKChains[i] = 0;
		}
	}
	return bOkay;
}

#ifdef ASURA_TOOLS
void Asura_Animation_IK_Template::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << ASURA_ANIMATION_IK_TEMPLATE_VERSION;

	xStream << m_uSkinID;
	xStream << m_uNumIKChains;
	for(u_int i=0 ; i<m_uNumIKChains; i++)
	{
		xStream << (int) m_ppxIKChains[i]->GetType();
		m_ppxIKChains[i]->WriteToChunkStream(xStream);
	}
}
#endif


#endif //ASURA_USE_ANIMATION_IK_SOLVER