//
// Asura_Animation_IK_Template_Chain.cpp
//
// Created by Richard Rice - 12/05/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Animation/IK/Asura_Animation_IK_Template_Chain.h"

#ifdef ASURA_USE_ANIMATION_IK_SOLVER

#include "Animation/IK/Asura_Animation_IK_Chain.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

#define ASURA_ANIMATION_IK_TEMPLATE_CHAIN_VERSION 0

Asura_Animation_IK_Template_Chain::Asura_Animation_IK_Template_Chain()
:m_uBoneArraySize(0)
,m_piBoneArray(0)
,m_piTwistBoneArray(0)
,m_uFlags(0)
,m_fPosSmoothingSpeed(2.0f)
,m_fOriSmoothingSpeed(Asura_Maths::Deg180)
#ifdef ASURA_TOOLS
,m_pxBoneNames(0)
,m_pxTwistBoneNames(0)	   
#endif
{
	m_bUsePosSmoothing = true;
	m_bUseOriSmoothing = true;
}

Asura_Animation_IK_Template_Chain::~Asura_Animation_IK_Template_Chain()
{
	delete [] m_piBoneArray;
	delete [] m_piTwistBoneArray;
	#ifdef ASURA_TOOLS
	delete [] m_pxBoneNames;
	delete [] m_pxTwistBoneNames;
	#endif
}

void Asura_Animation_IK_Template_Chain::SetupChain(Asura_Animation_IK_Chain* pxChain) const
{
	pxChain->SetIsRootBoneMover(m_bRootBoneMover);

	pxChain->SetPosSmoothing(m_bUsePosSmoothing);
	pxChain->SetPosSmoothingSpeed(m_fPosSmoothingSpeed);
	
	pxChain->SetOriSmoothing(m_bUseOriSmoothing);
	pxChain->SetOriSmoothingSpeed(m_fOriSmoothingSpeed);
}

#ifdef ASURA_TOOLS
void Asura_Animation_IK_Template_Chain::SetTwistBone(u_int uIndex, int iBone, const char* szBoneName)
{
	Asura_Assert(uIndex < GetNumBones(), "");

	//allocate the twist bone arrays if needed
	if(iBone != -1 && !m_piTwistBoneArray)
	{
		m_piTwistBoneArray = new int[m_uBoneArraySize];
		m_pxTwistBoneNames = new Asura_String[m_uBoneArraySize-1];

		for(u_int i=0; i<m_uBoneArraySize; i++)
		{
			m_piTwistBoneArray[i] = -1;
		}
	}

	//update the entry for this bone
	if(m_piTwistBoneArray)
	{
		Asura_Assert(m_pxTwistBoneNames, "");

		m_piTwistBoneArray[uIndex] = iBone;
		if(iBone != -1)
		{
			m_pxTwistBoneNames[uIndex] = szBoneName;
			m_bHasTwistBones = true;
		}
		else
		{
			m_pxTwistBoneNames[uIndex].MakeNull();
		}
	}

}
#endif //ASURA_TOOLS



bool Asura_Animation_IK_Template_Chain::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iVersion;
	xStream >> iVersion;

	if(iVersion > ASURA_ANIMATION_IK_TEMPLATE_CHAIN_VERSION)
	{
		Asura_Assertf(0, ("Unrecognised chunk version %d for IK chain template", iVersion));
		return false;
	}

	xStream >> m_uBoneArraySize;
	xStream >> m_uFlags;

	//read bone array
	m_piBoneArray = new int[m_uBoneArraySize];
	for(u_int i=0; i<m_uBoneArraySize; i++)
	{
		xStream >> m_piBoneArray[i];
	}

	if(m_bHasTwistBones)
	{
		//read twist bone array
		m_piTwistBoneArray = new int[m_uBoneArraySize];

		for(u_int i=0; i<m_uBoneArraySize; i++)
		{
			xStream >> m_piTwistBoneArray[i];
		}
	}

	xStream >> m_fPosSmoothingSpeed;
	xStream >> m_fOriSmoothingSpeed;

	//remainder of chunk consists of names which are just needed by tools
	#ifdef ASURA_TOOLS
	xStream >> m_xName;

	m_pxBoneNames = new Asura_String[GetNumBones()];
	for(u_int i=0; i<GetNumBones(); i++)
	{
		xStream >> m_pxBoneNames[i];
	}

	if(m_bHasTwistBones)
	{
		m_pxTwistBoneNames = new Asura_String[GetNumBones()];
		for(u_int i=0; i<GetNumBones(); i++)
		{
			xStream >> m_pxTwistBoneNames[i];
		}
	}
	#else
	xStream.SkipString();

	for(u_int i=0; i<GetNumBones(); i++)
	{
		xStream.SkipString();
	}

	if(m_bHasTwistBones)
	{
		for(u_int i=0; i<GetNumBones(); i++)
		{
			xStream.SkipString();
		}
	}
	#endif
	return true;
}

#ifdef ASURA_TOOLS
void Asura_Animation_IK_Template_Chain::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << ASURA_ANIMATION_IK_TEMPLATE_CHAIN_VERSION;

	xStream << m_uBoneArraySize;

	Asura_WriteToStream_Bitfield_Wrapper<u_int>	xFlagsBitfield (m_uFlags);
	xFlagsBitfield.Add(m_bRootBoneMover,1);
	xFlagsBitfield.Add(m_bUsePosSmoothing,1);
	xFlagsBitfield.Add(m_bUseOriSmoothing,1);
	xFlagsBitfield.Add(m_bHasTwistBones,1);
	xStream << xFlagsBitfield;

	for(u_int i=0; i<m_uBoneArraySize; i++)
	{
		xStream << m_piBoneArray[i];
	}

	if(m_bHasTwistBones)
	{
		for(u_int i=0; i<m_uBoneArraySize; i++)
		{
			xStream << m_piTwistBoneArray[i];
		}
	}

	xStream << m_fPosSmoothingSpeed;
	xStream << m_fOriSmoothingSpeed;

	xStream << m_xName;

	for(u_int i=0; i<GetNumBones(); i++)
	{
		xStream << m_pxBoneNames[i];
	}

	if(m_bHasTwistBones)
	{
		for(u_int i=0; i<GetNumBones(); i++)
		{
			xStream << m_pxTwistBoneNames[i];
		}
	}

}
#endif //ASURA_TOOLS


#endif //ASURA_USE_ANIMATION_IK_SOLVER
