//
// Asura_Animation_IK_Template_Chain_MarkerPoint.cpp
//
// Created by Richard Rice - 12/05/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Animation/IK/Asura_Animation_IK_Template_Chain_MarkerPoint.h"

#ifdef ASURA_USE_ANIMATION_IK_SOLVER
#include "Animation/IK/Asura_Animation_IK_Chain_MarkerPoint.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Hash.h"

#define ASURA_ANIMATION_IK_TEMPLATE_CHAIN_MARKERPOINT_VERSION 0

Asura_Animation_IK_Template_Chain_MarkerPoint::Asura_Animation_IK_Template_Chain_MarkerPoint()
:m_uMarkerPointHash(0)
,m_uMarkerPointFlags(0)
{
}

Asura_Animation_IK_Chain* Asura_Animation_IK_Template_Chain_MarkerPoint::CreateChainFromTemplate(const Asura_Animation* pxAnim) const
{
	Asura_Animation_IK_Chain* pxChain = new Asura_Animation_IK_Chain_MarkerPoint(GetBoneArray(), GetTwistBoneArray(), m_uMarkerPointHash, pxAnim, m_bUseOri);
	SetupChain(pxChain);

	return pxChain;
}

void Asura_Animation_IK_Template_Chain_MarkerPoint::SetMarkerPoint(const char* szName)
{
	#ifdef ASURA_TOOLS
	m_xMarkerPointName = szName;
	#endif
	m_uMarkerPointHash = Asura_GetHashID(szName);
}

bool Asura_Animation_IK_Template_Chain_MarkerPoint::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//check this class's version
	int iVersion;
	xStream >> iVersion;

	if(iVersion > ASURA_ANIMATION_IK_TEMPLATE_CHAIN_MARKERPOINT_VERSION)
	{
		Asura_Assertf(0, ("Unrecognised chunk version %d for Marker point IK chain template", iVersion));
		return false;
	}

	//read the parent
	if(!PARENT::ReadFromChunkStream(xStream)) return false;

	//read the rest of this class
	xStream >> m_uMarkerPointHash;
	xStream >> m_uMarkerPointFlags;

	#ifdef ASURA_TOOLS
	xStream >> m_xMarkerPointName;
	#else
	xStream.SkipString();
	#endif

	return true;
}

#ifdef ASURA_TOOLS
void Asura_Animation_IK_Template_Chain_MarkerPoint::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	//write his class's version
	xStream << ASURA_ANIMATION_IK_TEMPLATE_CHAIN_MARKERPOINT_VERSION;

	//write the parent
	PARENT::WriteToChunkStream(xStream);
		
	//write the rest of this
	xStream << m_uMarkerPointHash;

	Asura_WriteToStream_Bitfield_Wrapper<u_int>	xFlagsBitfield (m_uMarkerPointFlags);
	xFlagsBitfield.Add(m_bUseOri,1);
	xStream << xFlagsBitfield;

	xStream << m_xMarkerPointName;
}
#endif //ASURA_TOOLS



#endif //ASURA_USE_ANIMATION_IK_SOLVER
