//
// Asura_Animation_IK_Template_System.cpp
//
// Created by Richard Rice - 12/05/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Animation/IK/Asura_Animation_IK_Template_System.h"

#ifdef ASURA_USE_ANIMATION_IK_SOLVER

#include "Animation/IK/Asura_Animation_IK_Template.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"

#ifdef ASURA_TOOLS
#include "Animation/IK/Tools_Animation_IK_Template_System.h"
#endif


static bool g_bShuttingDown = false;

Asura_TemplateHashTree<Asura_Animation_IK_Template*> Asura_Animation_IK_Template_System::s_xTemplateTree;

void Asura_Animation_IK_Template_System::Initialise()
{
	#ifdef ASURA_TOOLS
	LoadAsuraFile(Tools_Animation_IK_Template_System::GetIKTemplateFilename());
	#endif
}

void Asura_Animation_IK_Template_System::Shutdown()
{
	g_bShuttingDown = true;
	s_xTemplateTree.Traverse(DeleteTemplate);
	s_xTemplateTree.Reset();

	g_bShuttingDown = false;
}

void Asura_Animation_IK_Template_System::Reset()
{
	Shutdown();
	Initialise();
}

void Asura_Animation_IK_Template_System::DeleteTemplate(Asura_Hash_ID uHash,Asura_Animation_IK_Template** ppxTemplate)
{
	delete *ppxTemplate;
}


const Asura_Animation_IK_Template* Asura_Animation_IK_Template_System::FindTemplate(Asura_Hash_ID uHashID)
{
	Asura_Animation_IK_Template** pxResult = s_xTemplateTree.GetDataByHash(uHashID);
	if (pxResult)
	{
		return *pxResult;
	}
	return NULL;
}

void Asura_Animation_IK_Template_System::AddTemplateToTree( Asura_Animation_IK_Template& xTemplate )
{

	if (s_xTemplateTree.AddNode(xTemplate.GetHashID(), &xTemplate)) return;

	Asura_Assertf(false, ("IK Template 0x%08x already exists in tree",xTemplate.GetHashID()));
}

void Asura_Animation_IK_Template_System::RemoveTemplateFromTree( const Asura_Animation_IK_Template& xTemplate )
{
	//Don't bother removing entries from the tree while shutting down , since the tree will be fully reset afterwards.
	if(g_bShuttingDown) return;
#ifdef ASURA_ASSERT
	bool bOK = 
#endif
	s_xTemplateTree.DeleteNodeByHash( xTemplate.GetHashID() );
	Asura_Assert(bOK, "failed to delete node from hash tree");
}


#ifdef ASURA_TOOLS
u_int Asura_Animation_IK_Template_System::GetNumberOfTemplates()
{
	return s_xTemplateTree.GetNumberOfNodes();
}

Asura_Animation_IK_Template* Asura_Animation_IK_Template_System::GetTemplateByHash(u_int uHashID)
{
	Asura_Animation_IK_Template** pxResult = s_xTemplateTree.GetDataByHash(uHashID);
	if (pxResult)
	{
		return *pxResult;
	}
	return NULL;
}

Asura_Animation_IK_Template* Asura_Animation_IK_Template_System::GetTemplateByIndex(u_int uIndex)
{
	Asura_Animation_IK_Template** pxResult = s_xTemplateTree.GetDataByIndex(uIndex);
	if (pxResult)
	{
		return *pxResult;
	}
	return NULL;
}


bool Asura_Animation_IK_Template_System::RenameTemplate(const char* szOldName,const char* szNewName)
{
	Asura_Hash_ID uOldHash = Asura_GetHashID(szOldName);
	Asura_Hash_ID uNewHash = Asura_GetHashID(szNewName);

	//make sure the template exists
	Asura_Animation_IK_Template* pxTemplate = GetTemplateByHash(uOldHash);
	if(!pxTemplate) return false;

	//see if the hash replacement is okay
	if(uOldHash != uNewHash)
	{
		if(!s_xTemplateTree.ReplaceHash(uOldHash,uNewHash)) return false;
	}

	pxTemplate->m_xName = szNewName;
	pxTemplate->m_uHashID = uNewHash;

	return true;
}


#endif // AUSRA_TOOLS

#endif //ASURA_USE_ANIMATION_IK_SOLVER
