//////////////////////////////////////////////////////////////////////////////
//
// Asura_Blueprint.h
//
// Created by NJD - 06 September 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_BLUEPRINT_H__
#define __ASURA_BLUEPRINT_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_Hash.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Mempool.h"
#ifdef ASURA_TOOLS
#include "Core/Asura_HashTree.h"
#endif
#include "Debugging/Asura_Assert.h"

//////////////////////////////////////////////////////////////////////////////
//
// Asura_Blueprint_Override_Type
//
//////////////////////////////////////////////////////////////////////////////

enum Asura_Blueprint_Override_Type
{
    ABOT_NONE,
    ABOT_SCALE,
    ABOT_OFFSET,
    ABOT_FIXED,
    ABOT_END_OVERRIDE_TYPES
};

#ifdef ASURA_TOOLS
extern const char* s_apszOverrideTypeStrings[ ABOT_END_OVERRIDE_TYPES ];
extern const char* s_apszBooleanStrings[ 2 ];
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Asura_Value
//
//////////////////////////////////////////////////////////////////////////////

class Asura_Value
{
public:
    enum VALUE_TYPE
    {
        VT_INT,
        VT_FLOAT,
        VT_BOOL,
        VT_HASH,
        VT_END_VALUE_TYPES
    };

	Asura_Value()
    : m_eValueType( VT_INT )
    , m_xValue()
#ifdef ASURA_TOOLS
    , m_uHashDisplayTextHash( ASURA_HASH_ID_UNSET )
#endif
    {
        m_xValue.m_iValue = 0;
    }
    Asura_Value( const Asura_Value& xRhs )
    : m_eValueType( xRhs.m_eValueType )
    , m_xValue( xRhs.m_xValue )
#ifdef ASURA_TOOLS
    , m_uHashDisplayTextHash( xRhs.m_uHashDisplayTextHash )
#endif
    {
    }

    bool operator==( const Asura_Value& xRhs )
    {
        // Using the int value does a pure binary comparison of all types in the union. This may not be acceptable for floats.
        return ( ( m_eValueType == xRhs.m_eValueType ) && ( m_xValue.m_iValue == xRhs.m_xValue.m_iValue ) );
    }
    Asura_Value& operator=( const Asura_Value& xRhs )
    {
        if ( this != &xRhs )
        {
            m_eValueType           = xRhs.m_eValueType;
            m_xValue               = xRhs.m_xValue;
#ifdef ASURA_TOOLS
            m_uHashDisplayTextHash = xRhs.m_uHashDisplayTextHash;
#endif
        }

        return *this;
    }

    void SetValue( const int   iValue );
    void SetValue( const float fValue );
    void SetValue( const bool  bValue );
    void SetValue( const Asura_Hash_ID uHashID, const Asura_Char* pszString = NULL );
	void SetValue( const Asura_Value& xValue );

    VALUE_TYPE    GetValueType()   const { return m_eValueType; }
    int           GetIntValue()    const { return m_xValue.m_iValue; }
    float         GetFloatValue()  const { return m_xValue.m_fValue; }
    bool          GetBoolValue()   const { return m_xValue.m_bValue; }
    Asura_Hash_ID GetHashIDValue() const { return m_xValue.m_uValue; }
#ifdef ASURA_TOOLS
    const Asura_Char* GetHashStringValue() const;
    Asura_Hash_ID     GetHashDisplayTextHash() const { return m_uHashDisplayTextHash; }
#endif

    void ApplyOverride( const Asura_Blueprint_Override_Type eOverrideType, const Asura_Value& xValue );

#ifdef ASURA_TOOLS
    static bool IsValidOverride( const VALUE_TYPE eBaseValueType, const VALUE_TYPE eOverrideValueType, const Asura_Blueprint_Override_Type eOverrideType );
    static const char* s_apszValueTypeStrings[ VT_END_VALUE_TYPES ];
#endif

private:
    VALUE_TYPE m_eValueType;
    union Value
    {
        int           m_iValue;
        float         m_fValue;
        bool          m_bValue;
        Asura_Hash_ID m_uValue;
    } m_xValue;
#ifdef ASURA_TOOLS
    Asura_Hash_ID m_uHashDisplayTextHash;
    static Asura_TemplateHashTree< Asura_String > s_xHashIDStringTree;
#endif
};

//////////////////////////////////////////////////////////////////////////////
//
// Asura_Parameter
//
//////////////////////////////////////////////////////////////////////////////

class Asura_Parameter
{
	friend class Asura_Blueprint;

public:
    Asura_Parameter()
    : m_uHashID( ASURA_HASH_ID_UNSET )
    , m_eOverrideType( ABOT_NONE )
	, m_uTemplate( ASURA_HASH_ID_UNSET )
    , m_xValue()
#ifdef ASURA_TOOLS
    , m_xName()
    , m_xGroupName()
#endif
    {
    }

    Asura_Parameter( const Asura_Parameter& xRhs )
    : m_uHashID( xRhs.m_uHashID )
    , m_eOverrideType( xRhs.m_eOverrideType )
	, m_uTemplate( xRhs.m_uTemplate )
    , m_xValue( xRhs.m_xValue )
#ifdef ASURA_TOOLS
    , m_xName( xRhs.m_xName )
    , m_xGroupName( xRhs.m_xGroupName )
#endif
    {
    }

	void SetValue( const Asura_Value& xValue )								  { m_xValue = xValue; }
    void SetOverrideType( const Asura_Blueprint_Override_Type eOverrideType ) { m_eOverrideType = eOverrideType; }
	void SetTemplate( const Asura_Hash_ID uTemplate )						  { m_uTemplate = uTemplate; }

    Asura_Hash_ID                 GetHashID()                       const { return m_uHashID; }
    Asura_Value&                  GetValue()							  { return m_xValue; }
	const Asura_Value&            GetValue()						const { return m_xValue; }
    Asura_Blueprint_Override_Type GetOverrideType()                 const { return m_eOverrideType; }
    Asura_Hash_ID                 GetTemplate()                     const { return m_uTemplate; }

    Asura_Parameter& operator=( const Asura_Parameter& xRhs )
    {
        if ( this != &xRhs )
        {
            m_uHashID       = xRhs.m_uHashID;
            m_eOverrideType = xRhs.m_eOverrideType;
			m_uTemplate     = xRhs.m_uTemplate;
			m_xValue		= xRhs.m_xValue;
#ifdef ASURA_TOOLS
            m_xName      = xRhs.m_xName;
            m_xGroupName = xRhs.m_xGroupName;
#endif
        }

        return *this;
    }

#ifdef ASURA_TOOLS
    void SetGroupName( const Asura_String& xGroupName ) { m_xGroupName = xGroupName; }
    const Asura_String& GetName()      const { return m_xName; }
    const Asura_String& GetGroupName() const { return m_xGroupName; }

    static void         BuildGroupNameTreeAndList();
    static void         ReleaseGroupNameStorage();
    static int          GetNumParameterGroups() { return s_xGroupNameTree.GetNumberOfNodes(); }
    static int          GetGroupListIndex( const Asura_String& xGroupName );
    static void         GetGroupName( const int iGroupListIndex, Asura_String& xGroupName );
    // Only use GetGroupNameList for the drop-down list dialogues please, as it tweaks strings for the UI,
    // use GetGroupListIndex or GetGroupName instead.
    static const char** GetGroupNameList() { return s_ppszGroupNameList; }
    static void         AddGroupName( const Asura_String& xGroupName );
#endif

private:
	// SetHashID and SetName are private because we can't just change them whenever we want, we must use the RenameParameter
	// function in Asura_Blueprint.
	void SetHashID( const Asura_Hash_ID uHashID ) { m_uHashID = uHashID; }

#ifdef ASURA_TOOLS
	void SetName( const Asura_String& xName ) { m_xName = xName; }
    static void BuildGroupListFromTree();
#endif

private:
    // If the parameter is an override it will have the same hash ID as the parameter it is overriding.
    Asura_Hash_ID				  m_uHashID;
    Asura_Blueprint_Override_Type m_eOverrideType;
	Asura_Hash_ID				  m_uTemplate;
    Asura_Value					  m_xValue;
#ifdef ASURA_TOOLS
    Asura_String                  m_xName;
    Asura_String                  m_xGroupName;
#endif

#ifdef ASURA_TOOLS
    static Asura_TemplateHashTree< Asura_String > s_xGroupNameTree;
    static const char**                           s_ppszGroupNameList;
#endif
};

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Blueprint_ParameterEnumerator
// Author       : Andy Tate
// Date         : 22-Jan-2008
//
// Description  : Abstract class to handle enumeration of blueprint parameters.
////////////////////////////////////////////////////////////////////////////////

class Asura_Blueprint_ParameterEnumerator
{
public:
	virtual ~Asura_Blueprint_ParameterEnumerator()	{}

	virtual void BeginEnumeration()																	= 0;
	virtual bool Enumerate(const u_int uIndex, char const* const szName, const Asura_Value& xValue)	= 0;
	virtual void EndEnumeration(const bool bSuccess)												= 0;

	// If you override this format it must still support the same parameters in the same order i.e. string szBaseName, u_int uNumDigits, u_int uCount
	virtual const char* GetParameterFormat() const { return "%s%0*d"; }

protected:
	Asura_Blueprint_ParameterEnumerator() {}

private:
	Asura_Blueprint_ParameterEnumerator(const Asura_Blueprint_ParameterEnumerator& xOther);
	Asura_Blueprint_ParameterEnumerator&	operator=(const Asura_Blueprint_ParameterEnumerator& xOther);
};

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Blueprint_DummyParameterEnumerator
// Author       : Anthony Gregson
// Date         : 11/03/08
//
// Description  : Fills in the pure virtual functions of the above in the simplest possible way.
//				  This is used in Asura_Blueprint::CountEnumeratedParameters, as it does nothing with the values themselves.
////////////////////////////////////////////////////////////////////////////////

class Asura_Blueprint_DummyParameterEnumerator : public Asura_Blueprint_ParameterEnumerator
{
public:
	Asura_Blueprint_DummyParameterEnumerator()			{}
	virtual ~Asura_Blueprint_DummyParameterEnumerator()	{}

	virtual void BeginEnumeration()																	{}
	virtual bool Enumerate(const u_int uIndex, char const* const szName, const Asura_Value& xValue)	{ return true; }
	virtual void EndEnumeration(const bool bSuccess)												{}
};

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Blueprint_ParameterEnumerator_ByHash
// Author       : Anthony Gregson
// Date         : 21/04/09
//
// Description  : Variation on the parameter enumerator idea, taking a hash ID as the base instead.
//				  For when the string for the parameter name isn't needed, so we don't need to store it and continually calculate the full hash ID.
//				  Not supporting the format override that exists in the normal parameter enumerator in the first version.
////////////////////////////////////////////////////////////////////////////////

class Asura_Blueprint_ParameterEnumerator_ByHash
{
public:
	virtual ~Asura_Blueprint_ParameterEnumerator_ByHash()	{}

	virtual void BeginEnumeration()											= 0;
	virtual bool Enumerate(const u_int uIndex, const Asura_Value& xValue)	= 0;
	virtual void EndEnumeration(const bool bSuccess)						= 0;

protected:
	Asura_Blueprint_ParameterEnumerator_ByHash() {}

private:
	Asura_Blueprint_ParameterEnumerator_ByHash(const Asura_Blueprint_ParameterEnumerator_ByHash& xOther);
	Asura_Blueprint_ParameterEnumerator_ByHash& operator=(const Asura_Blueprint_ParameterEnumerator_ByHash& xOther);
};

//////////////////////////////////////////////////////////////////////////////
//
// Asura_Blueprint_SmallHashedList
// More compact implementation of a hashed list specifically for use with the blueprints.
// uHASH_SIZE must be a power of two. T is assumed to have a method GetHashID() and a default constructor.
//
//////////////////////////////////////////////////////////////////////////////

template< typename T, u_int uHASH_SIZE >
class Asura_Blueprint_SmallHashedList
{
public:
	Asura_Blueprint_SmallHashedList();
	~Asura_Blueprint_SmallHashedList();
	inline void Clear();

	inline u_int GetNumEntries() const { return m_usNumEntries; }
	inline T* GetByHash( Asura_Hash_ID uHashID ) const;
	inline T* GetByIndex( u_int uIdx ) const;
	inline const T* GetList() const { return m_pxList; }

	inline bool Preallocate( u_int uCount );
	// Caller is expected to ensure that number added exactly matches the number preallocated for.
	inline void AddEntryWithPreallocation( const T& xValue );

	inline bool AddEntry( const T& xValue );
	inline bool RemoveEntry( Asura_Hash_ID uHashID );

private:
	// End of bin dictated by next bin offset. If they're equal then bin is empty. Extra entry at the end to simplify code.
	u_short m_ausBinOffsets[ uHASH_SIZE + 1 ];
	u_short m_usNumEntries;
	T* m_pxList;
};

//////////////////////////////////////////////////////////////////////////////
//
// Asura_Blueprint
//
//////////////////////////////////////////////////////////////////////////////

class Asura_Blueprint
{
public:
    Asura_Blueprint();
    Asura_Blueprint( const Asura_Blueprint& xRhs );

    Asura_Hash_ID    GetHashID()        const { return m_uHashID; }
    Asura_Hash_ID    GetParentHashID()  const { return m_uParentHashID; }
    Asura_Blueprint* GetParent()        const { return m_pxParent; }
	bool             GetParameterValueByHashID( const Asura_Hash_ID uParameterHashID, Asura_Value& xValue ) const;
	bool             GetParameterValueByIndex( const int            iParameterIndex,  Asura_Value& xValue ) const;
	
	float			GetParameterAsFloat( Asura_Hash_ID uParameterID, float fDefaultValue ) const;
	int				GetParameterAsInt( Asura_Hash_ID uParameterID, int iDefaultValue ) const;
	u_int			GetParameterAsUInt( Asura_Hash_ID uParameterID, u_int uDefaultValue ) const;
	bool			GetParameterAsBool( Asura_Hash_ID uParameterID, bool bDefaultValue ) const;
	Asura_Hash_ID	GetParameterAsHash( Asura_Hash_ID uParameterID, Asura_Hash_ID uDefaultValue ) const;


    // NOTE: These GetParameter functions return the raw parameter, they do nothing to calculate the correct Asura_Value.
    // Use the GetParameterValue functions for accessing Asura_Values correctly.
    int              GetNumParameters() const;
    Asura_Parameter* GetParameterByIndex( const int iParameterIndex ) const;
    Asura_Parameter* GetParameterByHashID( const Asura_Hash_ID uParameterHashID, const bool bLookUpHierarchy = false ) const;

	// Sets up the link pointers.
    void EstablishLinkage( const Asura_Hash_ID uTypeHashID );
    // If bOnlyDirectChild is true then this will only return true if this parent is the direct parent.
    bool IsChildOf( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uParentBlueprintHashID, const bool bOnlyDirectChild ) const;

    void ReadFromChunkStream( class Asura_Chunk_Stream& xStream );

    Asura_Blueprint& operator=( const Asura_Blueprint& xRhs );

#ifdef ASURA_TOOLS
    void SetHashID( const Asura_Hash_ID uHashID )       { m_uHashID = uHashID; }
    void SetParentHashID( const Asura_Hash_ID uHashID ) { m_uParentHashID = uHashID; }
	void SetName( const Asura_String& xName )           { m_xName = xName; }
    void MarkForDeletion()                              { m_bToBeDeleted = true; }

	void SetExported(const bool bExportChildBlueprints)	{ m_bExported = true; m_bChildBlueprintsExported = bExportChildBlueprints; }
	void ResetExportFlags(void)							{ m_bExported = false; m_bChildBlueprintsExported = false; }

    const Asura_String& GetName() const                 { return m_xName; }
	Asura_String GetNameIncludingAllParentNames() const;
    bool IsToBeDeleted() const							{ return m_bToBeDeleted; }

	bool IsExported( const bool bExportChildBlueprints ) const { return ( bExportChildBlueprints ? m_bChildBlueprintsExported : m_bExported ); }

    void AddParameter( const Asura_String& xName, const Asura_String& xGroupName,
					   const Asura_Blueprint_Override_Type eOverrideType, const Asura_Value& xValue,
					   Asura_Hash_ID uTemplate = ASURA_HASH_ID_UNSET );
    void DeleteParameter( const Asura_String& xName );
	void RenameParameter( const Asura_String& xOldName, const Asura_String& xNewName );
    void WriteToChunkStream( class Asura_Chunk_Stream& xStream );
#endif

	// Enumeration function.
	u_int EnumerateParameters(const char* szBaseText, const u_int uNumDigits, Asura_Blueprint_ParameterEnumerator& xEnumerator) const;
	// Similar to the above, but just finds out how many they are rather than doing anything with them.
	u_int CountEnumeratedParameters(const char* szBaseText, const u_int uNumDigits) const;

	// Hash id variant on the above function, using the by-hash enumerator
	u_int EnumerateParameters(const Asura_Hash_ID uBaseHash, const u_int uNumDigits, Asura_Blueprint_ParameterEnumerator_ByHash& xEnumerator) const;

private:
    void ReadParameterFromStream( const int iChunkVersion, class Asura_Chunk_Stream& xStream, Asura_Parameter& xParameter );

    // Parameters and Values themselves don't have chunk versions stored, so if their format changes this chunk version needs to be bumped.
    enum { CURRENT_CHUNK_VERSION = 7 };

private:
    Asura_Hash_ID	 m_uHashID;
    Asura_Hash_ID	 m_uParentHashID;
    Asura_Blueprint* m_pxParent;
	enum { HASH_SIZE = 8 };
	Asura_Blueprint_SmallHashedList< Asura_Parameter, HASH_SIZE > m_xParameters;

#ifdef ASURA_TOOLS
	bool			m_bExported;					// Flag to say that we've handled this blueprint for exporting.
	bool			m_bChildBlueprintsExported;		// Flag to say that we've also handled the children of this blueprint for exporting.
	bool			m_bToBeDeleted;
	Asura_String	m_xName;
#endif
};

//////////////////////////////////////////////////////////////////////////////
//
// Asura_Blueprint_SmallHashedList implementation.
//
//////////////////////////////////////////////////////////////////////////////

template< typename T, u_int uHASH_SIZE >
void Asura_Blueprint_SmallHashedList< T, uHASH_SIZE >::Clear()
{
	delete[] m_pxList;
	m_pxList = NULL;
	m_usNumEntries = 0;

	for( u_int u = 0; u <= uHASH_SIZE; ++u )
	{
		m_ausBinOffsets[u] = 0;
	}
}

template< typename T, u_int uHASH_SIZE >
Asura_Blueprint_SmallHashedList< T, uHASH_SIZE >::Asura_Blueprint_SmallHashedList()
{
	m_pxList = NULL;
	Clear();
}

template< typename T, u_int uHASH_SIZE >
Asura_Blueprint_SmallHashedList< T, uHASH_SIZE >::~Asura_Blueprint_SmallHashedList()
{
	Clear();
}

template< typename T, u_int uHASH_SIZE >
inline T* Asura_Blueprint_SmallHashedList< T, uHASH_SIZE >::GetByHash( Asura_Hash_ID uHashID ) const
{
	u_int uBinIdx = uHashID & (uHASH_SIZE - 1);
	u_int uBinStart = m_ausBinOffsets[uBinIdx];
	u_int uBinEnd = m_ausBinOffsets[uBinIdx + 1];

	for( u_int u = uBinStart; u < uBinEnd; ++u )
	{
		if( m_pxList[u].GetHashID() == uHashID )
		{
			return &m_pxList[u];
		}
	}

	return NULL;
}

template< typename T, u_int uHASH_SIZE >
inline T* Asura_Blueprint_SmallHashedList< T, uHASH_SIZE >::GetByIndex( u_int uIdx ) const
{
	if( uIdx < m_usNumEntries )
	{
		return &m_pxList[uIdx];
	}
	else
	{
		Asura_Assert( false, "Asura_Blueprint_SmallHashedList::GetByIndex - bad index" );
		return NULL;
	}
}

template< typename T, u_int uHASH_SIZE >
inline bool Asura_Blueprint_SmallHashedList< T, uHASH_SIZE >::Preallocate( u_int uCount )
{
	if ( uCount > USHRT_MAX )
	{
		Asura_Assert( false, "Asura_Blueprint_SmallHashedList::Preallocate - count is too large" );
		return false;
	}

	Clear();
	m_pxList = new T[uCount];
	return( NULL != m_pxList );
}

template< typename T, u_int uHASH_SIZE >
inline void Asura_Blueprint_SmallHashedList< T, uHASH_SIZE >::AddEntryWithPreallocation( const T& xValue )
{
	u_int uBinIdx = xValue.GetHashID() & (uHASH_SIZE - 1);
	u_int uListIdx = m_ausBinOffsets[uBinIdx + 1];

	// Shuffle any later bins along.
	for( u_int u = uBinIdx + 1; u <= uHASH_SIZE; ++u )
	{
		++m_ausBinOffsets[u];
	}

	// Shuffle any later list entries along.
	for( u_int u = m_usNumEntries; u > uListIdx; --u )
	{
		m_pxList[u] = m_pxList[u - 1];
	}

	m_pxList[uListIdx] = xValue;
	++m_usNumEntries;
}

template< typename T, u_int uHASH_SIZE >
inline bool Asura_Blueprint_SmallHashedList< T, uHASH_SIZE >::AddEntry( const T& xValue )
{
	const u_int uNewCount = m_usNumEntries + 1;
	if ( uNewCount > USHRT_MAX )
	{
		Asura_Assert( false, "Asura_Blueprint_SmallHashedList::Preallocate - too many entries" );
		return false;
	}

	T* pxNewList = new T[uNewCount];
	if( !pxNewList ) { return false; }

	u_int uBinIdx = xValue.GetHashID() & (uHASH_SIZE - 1);
	u_int uListIdx = m_ausBinOffsets[uBinIdx + 1];

	// Shuffle any later bins along.
	for( u_int u = uBinIdx + 1; u <= uHASH_SIZE; ++u )
	{
		++m_ausBinOffsets[u];
	}

	// Populate new list.
	{
		const T* pxSrc = m_pxList;

		for( u_int u = 0; u < uListIdx; ++u )
		{
			pxNewList[u] = pxSrc[u];
		}

		pxNewList[uListIdx] = xValue;

		for( u_int u = uListIdx + 1; u < uNewCount; ++u )
		{
			pxNewList[u] = pxSrc[u - 1];
		}
	}

	// Swap lists over.
	delete[] m_pxList;
	m_pxList = pxNewList;
	m_usNumEntries = static_cast<u_short>( uNewCount );

	return true;
}

template< typename T, u_int uHASH_SIZE >
inline bool Asura_Blueprint_SmallHashedList< T, uHASH_SIZE >::RemoveEntry( Asura_Hash_ID uHashID )
{
	u_int uBinIdx = uHashID & (uHASH_SIZE - 1);
	u_int uBinStart = m_ausBinOffsets[uBinIdx];
	u_int uBinEnd = m_ausBinOffsets[uBinIdx + 1];

	u_int uListIdx;
	for( uListIdx = uBinStart; uListIdx < uBinEnd; ++uListIdx )
	{
		if( m_pxList[uListIdx].GetHashID() == uHashID )
		{
			break;
		}
	}

	if( uListIdx == uBinEnd ) { return false; }

	// Move later entries down.
	for( u_int u = uListIdx + 1; u < m_usNumEntries; ++u )
	{
		m_pxList[u - 1] = m_pxList[u];
	}

	// Update later bins' offsets.
	for( u_int u = uBinIdx + 1; u <= uHASH_SIZE; ++u )
	{
		--m_ausBinOffsets[u];
	}

	--m_usNumEntries;

	if( !m_usNumEntries )
	{
		delete[] m_pxList;
		m_pxList = NULL;
	}

	return true;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_BLUEPRINT_H__
