//*******************************************************************
//
// File: Asura_Botanical_Game_Render_Instance_Header.h
//
// Created: 30 Nov 2006
//
// Description: 
//
//*******************************************************************

#ifndef _ASURA_BOTANICAL_GAME_RENDER_INSTANCE_HEADER_H_
#define _ASURA_BOTANICAL_GAME_RENDER_INSTANCE_HEADER_H_

// other libraries
#include "Botanicals/Asura_Botanical_Game_Render_Constants.h"
#include "Botanicals/Asura_Botanical_Game_Render_Dynamic_Memory_Layout.h"
#include "Botanicals/Asura_Botanical_Game_Render_TypeID.h"
#include "Botanicals/Asura_Botanical_Game_Render_Types.h"
#include "Core/Asura_Maths.h"
#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
#include "Visibility/Asura_Visibility_System.h"
#endif	//defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)

// internal dependencies

// forward declarations

//class Asura_BGR_DynamicInstance;
//class Asura_BGR_DynamicUserData;
class Asura_BGR_Instance;
class Asura_BGR_InteractiveBasic;
class Asura_BGR_Type;

class Asura_Chunk_Stream;

//*******************************************************************
// Constants
//*******************************************************************

#if defined( _DEBUG)
	#define Asura_BGR_IH_Assert( A, B) Asura_Assert( A, B)
	#define Asura_BGR_IH_Assertf( A, B) Asura_Assertf( A, B) 
#else
	#define Asura_BGR_IH_Assert(A, B)
	#define Asura_BGR_IH_Assertf(A, B)
#endif



//*******************************************************************
// Class Definitions
//*******************************************************************
class Asura_BGR_Instances_Header
{
public:
	Asura_BGR_Instances_Header();
	virtual ~Asura_BGR_Instances_Header() = 0;
	
//	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
//	bool Export( Asura_Chunk_Stream& xStream);
#endif

	Asura_BGR_TypeID GetType() const;
	void SetType( const Asura_BGR_TypeID xType);
	
	const Asura_Bounding_Box& GetBoundingBox() const;
	void SetBoundingBox( Asura_Bounding_Box& xBoundingBox);
	
	virtual u_int GetInstanceCount() const = 0;
	virtual const Asura_BGR_Instance* GetInstance( const u_int uInstanceIndex) const = 0;

#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
	bool	IsVisibleFromCurrentCamera(void) const	{ return(m_xVisibilityQuery.IsAABBVisibleFromCurrentCamera(m_xBoundingBox)); }
#endif	//defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
	
	// DLH - 01 November 2006 14:58:03 - Dynamic Data Handling
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA

	bool SetupDynamicData( const Asura_BGR_Type& xType);
	void SetDynamicDataLayout( const Asura_BGR_DynamicMemoryLayout& xDynamicDataLayout);
	
	bool AllocateDynamicData( const u_int uDynamicCountInstances);
	void DeallocateDynamicData();
	void ClearDynamicData();
	
	void ResetDynamicData();
	
	bool HasDynamicInteractiveData() const;
	bool HasDynamicSectionVersionData() const;
	
	uint32 GetDynamicInstanceCount() const { return( this->m_uDynamicCountInstances);}
	
	const Asura_BGR_DynamicInstance* GetDynamicInstanceData( const u_int uInstanceIndex) const;
	Asura_BGR_DynamicInstance* GetDynamicInstanceData( const u_int uInstanceIndex);
	
	void SetDynamicSectionVersion( const Asura_BGR_DynamicInstance* const pxDynamicInstance, const Asura_BGR_Section_ID xSectionID, const Asura_BGR_Version_ID xVersion);
	Asura_BGR_Version_ID GetDynamicSectionVersion( const Asura_BGR_DynamicInstance* const pxDynamicInstance, const Asura_BGR_Section_ID xSectionID) const;
	const Asura_BGR_Version_ID* GetDynamicSectionVersions( const Asura_BGR_DynamicInstance* const pxDynamicInstance) const;
	Asura_BGR_Version_ID* GetDynamicSectionVersions( const Asura_BGR_DynamicInstance* const pxDynamicInstance);

	void SetState( const Asura_BGR_DynamicInstance* const pxDynamicInstance, const Asura_BGR_State_ID xStateID, const Asura_BGR_State_Value xValue);
	Asura_BGR_State_Value GetState( const Asura_BGR_DynamicInstance* const pxDynamicInstance, const Asura_BGR_State_ID xStateID) const;
	const Asura_BGR_State_Value* GetStates( const Asura_BGR_DynamicInstance* const pxDynamicInstance) const;
	Asura_BGR_State_Value* GetStates( const Asura_BGR_DynamicInstance* const pxDynamicInstance);

	const Asura_BGR_InteractiveBasic* GetDynamicInteractiveData( const Asura_BGR_DynamicInstance* const pxDynamicInstance) const;
	Asura_BGR_InteractiveBasic* GetDynamicInteractiveData( const Asura_BGR_DynamicInstance* const pxDynamicInstance);
	#if defined( ASURA_BOTANICAL_INSTANCE_USER_DATA_TYPE)
		const Asura_BGR_DynamicUserData* GetDynamicUserData( const Asura_BGR_DynamicInstance* const pxDynamicInstance) const;
	#endif
	
#endif

protected:
#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
	mutable Asura_Static_Visibility_Query	m_xVisibilityQuery;
#endif	//defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)

	Asura_BGR_TypeID m_xTypeID;
	Asura_Bounding_Box m_xBoundingBox;
	
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	Asura_BGR_DynamicMemoryLayout m_xDynamicDataLayout;
	
	u_int m_uDynamicCountInstances;
	uint8* m_pucDynamicInstanceData;
#endif
};

//*******************************************************************
// Inline Function Definitions
//*******************************************************************

inline Asura_BGR_Instances_Header::Asura_BGR_Instances_Header()
{
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	this->m_uDynamicCountInstances = 0;
	this->m_pucDynamicInstanceData = NULL;
#endif
}

inline Asura_BGR_Instances_Header::~Asura_BGR_Instances_Header()
{
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	delete[] this->m_pucDynamicInstanceData;
	this->m_pucDynamicInstanceData = NULL;
#endif
}

inline const Asura_Bounding_Box& Asura_BGR_Instances_Header::GetBoundingBox() const
{
	return( this->m_xBoundingBox);
}

inline void Asura_BGR_Instances_Header::SetBoundingBox( Asura_Bounding_Box& xBoundingBox)
{
	this->m_xBoundingBox = xBoundingBox;
}

inline Asura_BGR_TypeID Asura_BGR_Instances_Header::GetType() const
{
	return( this->m_xTypeID);
}

inline void Asura_BGR_Instances_Header::SetType( const Asura_BGR_TypeID xType)
{
	this->m_xTypeID = xType;
}

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline void Asura_BGR_Instances_Header::SetDynamicDataLayout( const Asura_BGR_DynamicMemoryLayout& xDynamicDataLayout)
{
	this->m_xDynamicDataLayout = xDynamicDataLayout;
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline const Asura_BGR_DynamicInstance* Asura_BGR_Instances_Header::GetDynamicInstanceData( const u_int uInstanceIndex) const
{
//	Asura_BGR_IH_Assert( NULL != this->m_pucDynamicInstanceData, "No dynamic Data Setup");
//	Asura_BGR_IH_Assertf( uInstanceIndex < this->m_uDynamicCountInstances, ("Index %d is out of range. Size %d.", uInstanceIndex, this->m_uDynamicCountInstances));
	
	void* pDynamicInstance = this->m_pucDynamicInstanceData + (this->m_xDynamicDataLayout.m_uDynamicSizeInstance * uInstanceIndex);
	const Asura_BGR_DynamicInstance* const pxDynamicInstance = static_cast< Asura_BGR_DynamicInstance*>( pDynamicInstance);
	
	return( pxDynamicInstance);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline Asura_BGR_DynamicInstance* Asura_BGR_Instances_Header::GetDynamicInstanceData( const u_int uInstanceIndex)
{
	const Asura_BGR_Instances_Header* pxConstInstanceHeader = this;
	Asura_BGR_DynamicInstance* pxDynamicInstance = const_cast< Asura_BGR_DynamicInstance*>( pxConstInstanceHeader->GetDynamicInstanceData( uInstanceIndex));
	
	return( pxDynamicInstance);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline const Asura_BGR_InteractiveBasic* Asura_BGR_Instances_Header::GetDynamicInteractiveData( const Asura_BGR_DynamicInstance* const pxDynamicInstance) const
{
	Asura_BGR_IH_Assert( NULL != pxDynamicInstance, "This method need a valid Dynamic Instance Pointer");
	Asura_BGR_IH_Assert( (this->m_xDynamicDataLayout.m_uDynamicOffsetInteractiveData + sizeof( Asura_BGR_InteractiveBasic) ) <= this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic Interactive data does not seem to be setup correctly");
	
	const uint8* pucDynamicInstance = static_cast< const uint8*>( static_cast< const void*>( pxDynamicInstance));
	const void* pInteractiveBasic = pucDynamicInstance + this->m_xDynamicDataLayout.m_uDynamicOffsetInteractiveData;
	const Asura_BGR_InteractiveBasic* const pxInteractiveBasic = static_cast< const Asura_BGR_InteractiveBasic*>( pInteractiveBasic);
	
	return( pxInteractiveBasic);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline Asura_BGR_InteractiveBasic* Asura_BGR_Instances_Header::GetDynamicInteractiveData( const Asura_BGR_DynamicInstance* const pxDynamicInstance)
{
	const Asura_BGR_Instances_Header* pxConstInstanceHeader = this;
	Asura_BGR_InteractiveBasic* pxInteractiveBasic = const_cast< Asura_BGR_InteractiveBasic*>( pxConstInstanceHeader->GetDynamicInteractiveData( pxDynamicInstance));
	
	return( pxInteractiveBasic);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
#if defined( ASURA_BOTANICAL_INSTANCE_USER_DATA_TYPE)
inline const Asura_BGR_DynamicUserData* Asura_BGR_Instances_Header::GetDynamicUserData( const Asura_BGR_DynamicInstance* const pxDynamicInstance) const
{
	Asura_BGR_IH_Assert( NULL != pxDynamicInstance, "This method need a valid Dynamic Instance Pointer");
	Asura_BGR_IH_Assert( (this->m_xDynamicDataLayout.m_uDynamicOffsetUserData + 0 ) <= this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic User data does not seem to be setup correctly");
	
	const uint8* pucDynamicInstance = static_cast< const uint8*>( static_cast< const void*>( pxDynamicInstance));
	const void* pUserData = pucDynamicInstance + this->m_xDynamicDataLayout.m_uDynamicOffsetUserData;
	const Asura_BGR_DynamicUserData* const pxUserData = static_cast< const Asura_BGR_DynamicUserData*>( pUserData);
	
	return( pxUserData);
}
#endif
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline void Asura_BGR_Instances_Header::SetDynamicSectionVersion( const Asura_BGR_DynamicInstance* const pxDynamicInstance, const Asura_BGR_Section_ID xSectionID, const Asura_BGR_Version_ID xVersion)
{
	Asura_BGR_IH_Assert( NULL != pxDynamicInstance, "This method need a valid Dynamic Instance Pointer");
	Asura_BGR_IH_Assert( (this->m_xDynamicDataLayout.m_uDynamicOffsetInteractiveData + sizeof( Asura_BGR_Version_ID) ) <= this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic Interactive data does not seem to be setup correctly");
	
	uint8* pucDynamicInstance = const_cast< uint8*>( static_cast< const uint8*>( static_cast< const void*>( pxDynamicInstance)));
	void* pVersionID0 = pucDynamicInstance + this->m_xDynamicDataLayout.m_uDynamicOffsetSectionData;
	Asura_BGR_Version_ID* pxVersionID0 = static_cast< Asura_BGR_Version_ID*>( pVersionID0);
	Asura_BGR_Version_ID* pxVersionID = pxVersionID0 + xSectionID;
	
	Asura_BGR_IH_Assert( static_cast< size_t>(static_cast< uint8*>( static_cast< void*>( pxVersionID)) - pucDynamicInstance) < this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic Interactive data does not seem to be setup correctly");
	
	*(pxVersionID) = xVersion;
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline Asura_BGR_Version_ID Asura_BGR_Instances_Header::GetDynamicSectionVersion( const Asura_BGR_DynamicInstance* const pxDynamicInstance, const Asura_BGR_Section_ID xSectionID) const
{
	Asura_BGR_IH_Assert( NULL != pxDynamicInstance, "This method need a valid Dynamic Instance Pointer");
	Asura_BGR_IH_Assert( (this->m_xDynamicDataLayout.m_uDynamicOffsetInteractiveData + sizeof( Asura_BGR_Version_ID) ) <= this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic Interactive data does not seem to be setup correctly");
	
	uint8* pucDynamicInstance = const_cast< uint8*>( static_cast< const uint8*>( static_cast< const void*>( pxDynamicInstance)));
	void* pVersionID0 = pucDynamicInstance + this->m_xDynamicDataLayout.m_uDynamicOffsetSectionData;
	Asura_BGR_Version_ID* pxVersionID0 = static_cast< Asura_BGR_Version_ID*>( pVersionID0);
	Asura_BGR_Version_ID* pxVersionID = pxVersionID0 + xSectionID;
	
	Asura_BGR_IH_Assert( static_cast< size_t>(static_cast< uint8*>( static_cast< void*>( pxVersionID)) - pucDynamicInstance) < this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic Interactive data does not seem to be setup correctly");
	
	Asura_BGR_Version_ID xVersion = *(pxVersionID);
	
	return( xVersion);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline const Asura_BGR_Version_ID* Asura_BGR_Instances_Header::GetDynamicSectionVersions( const Asura_BGR_DynamicInstance* const pxDynamicInstance) const
{
	Asura_BGR_IH_Assert( NULL != pxDynamicInstance, "This method need a valid Dynamic Instance Pointer");
	Asura_BGR_IH_Assert( (this->m_xDynamicDataLayout.m_uDynamicOffsetInteractiveData + sizeof( Asura_BGR_Version_ID) ) <= this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic Interactive data does not seem to be setup correctly");
	
	uint8* pucDynamicInstance = const_cast< uint8*>( static_cast< const uint8*>( static_cast< const void*>( pxDynamicInstance)));
	void* pVersionID0 = pucDynamicInstance + this->m_xDynamicDataLayout.m_uDynamicOffsetSectionData;
	Asura_BGR_Version_ID* pxVersionID0 = static_cast< Asura_BGR_Version_ID*>( pVersionID0);
	
	return( pxVersionID0);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline Asura_BGR_Version_ID* Asura_BGR_Instances_Header::GetDynamicSectionVersions( const Asura_BGR_DynamicInstance* const pxDynamicInstance)
{
	const Asura_BGR_Instances_Header* pxConstInstanceHeader = this;
	Asura_BGR_Version_ID* paxVersionID = const_cast< Asura_BGR_Version_ID*> (pxConstInstanceHeader->GetDynamicSectionVersions( pxDynamicInstance));
	
	return( paxVersionID);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline void Asura_BGR_Instances_Header::SetState( const Asura_BGR_DynamicInstance* const pxDynamicInstance, const Asura_BGR_State_ID xStateID, const Asura_BGR_State_Value xStateValue)
{
	Asura_BGR_IH_Assert( NULL != pxDynamicInstance, "This method need a valid Dynamic Instance Pointer");
	Asura_BGR_IH_Assert( (this->m_xDynamicDataLayout.m_uDynamicOffsetStateData + sizeof( Asura_BGR_State_Value) ) <= this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic State data does not seem to be setup correctly");
	
	uint8* pucDynamicInstance = const_cast< uint8*>( static_cast< const uint8*>( static_cast< const void*>( pxDynamicInstance)));
	void* pState0 = pucDynamicInstance + this->m_xDynamicDataLayout.m_uDynamicOffsetStateData;
	Asura_BGR_State_Value* pxState0 = static_cast< Asura_BGR_State_Value*>( pState0);
	Asura_BGR_State_Value* pxStateValue = pxState0 + xStateID;
	
	Asura_BGR_IH_Assert( static_cast< size_t>(static_cast< uint8*>( static_cast< void*>( pxStateValue)) - pucDynamicInstance) < this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic Interactive data does not seem to be setup correctly");
	
	*(pxStateValue) = xStateValue;
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline Asura_BGR_State_Value Asura_BGR_Instances_Header::GetState( const Asura_BGR_DynamicInstance* const pxDynamicInstance, const Asura_BGR_State_ID xStateID) const
{
	Asura_BGR_IH_Assert( NULL != pxDynamicInstance, "This method need a valid Dynamic Instance Pointer");
	Asura_BGR_IH_Assert( (this->m_xDynamicDataLayout.m_uDynamicOffsetStateData + sizeof( Asura_BGR_State_Value) ) <= this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic State data does not seem to be setup correctly");
	
	uint8* pucDynamicInstance = const_cast< uint8*>( static_cast< const uint8*>( static_cast< const void*>( pxDynamicInstance)));
	void* pState0 = pucDynamicInstance + this->m_xDynamicDataLayout.m_uDynamicOffsetStateData;
	Asura_BGR_State_Value* pxState0 = static_cast< Asura_BGR_State_Value*>( pState0);
	Asura_BGR_State_Value* pxStateValue = pxState0 + xStateID;
	
	Asura_BGR_IH_Assert( static_cast< size_t>(static_cast< uint8*>( static_cast< void*>( pxStateValue)) - pucDynamicInstance) < this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic Interactive data does not seem to be setup correctly");
	
	Asura_BGR_State_Value xStateValue = *(pxStateValue);
	
	return( xStateValue);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline const Asura_BGR_State_Value* Asura_BGR_Instances_Header::GetStates( const Asura_BGR_DynamicInstance* const pxDynamicInstance) const
{
	Asura_BGR_IH_Assert( NULL != pxDynamicInstance, "This method need a valid Dynamic Instance Pointer");
	Asura_BGR_IH_Assert( (this->m_xDynamicDataLayout.m_uDynamicOffsetInteractiveData + sizeof( Asura_BGR_Version_ID) ) <= this->m_xDynamicDataLayout.m_uDynamicSizeInstance, "Dynamic Interactive data does not seem to be setup correctly");
	
	uint8* pucDynamicInstance = const_cast< uint8*>( static_cast< const uint8*>( static_cast< const void*>( pxDynamicInstance)));
	void* pState0 = pucDynamicInstance + this->m_xDynamicDataLayout.m_uDynamicOffsetStateData;
	Asura_BGR_State_Value* pxState0 = static_cast< Asura_BGR_State_Value*>( pState0);
	
	return( pxState0);
}
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline Asura_BGR_State_Value* Asura_BGR_Instances_Header::GetStates( const Asura_BGR_DynamicInstance* const pxDynamicInstance)
{
	const Asura_BGR_Instances_Header* pxConstInstanceHeader = this;
	Asura_BGR_State_Value* paxStates = const_cast< Asura_BGR_State_Value*> (pxConstInstanceHeader->GetStates( pxDynamicInstance));
	
	return( paxStates);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline bool Asura_BGR_Instances_Header::HasDynamicInteractiveData() const 
{ 
	return( Asura_BGR_DynamicMemoryLayout::DYNAMIC_OFFSET_INVALID != this->m_xDynamicDataLayout.m_uDynamicOffsetInteractiveData);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
inline bool Asura_BGR_Instances_Header::HasDynamicSectionVersionData() const 
{ 
	return( this->m_xDynamicDataLayout.HasDynamicSectionVersionData());
}
#endif


//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif // _ASURA_BOTANICAL_GAME_RENDER_INSTANCE_HEADER_H_
