//*******************************************************************
//
// File: Asura_Botanicals_Game_Render_Interactive.cpp
//
// Created: 13 Nov 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_BOTANICALS)

// main header
//#include "Asura_Botanicals_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanical_Game_Render_Types.h"

#include "Botanicals/Asura_Botanical_Game_Render_Instance.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance_Header.h"
#include "Botanicals/Asura_Botanical_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanical_Game_Render_Type.h"
#include "Botanicals/Asura_Botanical_Game_Render_Type_BasicInteractive.h"
#include "Botanicals/Asura_Botanical_Game_Render_TypeID.h"

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_PLATFORM_DATA_UPDATE)
#include "Botanicals/Asura_Platform_Botanicals_Game_Render_Manager_Update_Dynamic_Data.h"
#endif

// other libraries
#include "Affectors/Asura_Affector.h"
#include "Affectors/Asura_Affector_System.h"
#include "Core/Asura_Mempool.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_CPUProfile.h"
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Interface/Asura_CommandConsole.h"
#endif
#include "Jobs/Asura_Job.h"

// internal dependencies

#if 0 && defined( WIN32)
	#define SUPPORT_VTUNE
#endif
#if defined( SUPPORT_VTUNE)
	#include <libittnotify.h>
	#pragma comment( lib, "libittnotify.lib")
#endif

#ifdef _XBOX
#include "Rendering/Asura_Xbox360_Graphics.h"
#endif

//*******************************************************************
// Constants
//*******************************************************************

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	#if !defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) || 0 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
		#error If ASURA_BOTANICAL_SUPPORT_INTERACTIVE is defined then you will also need ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	#endif
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE) && defined( DLH)
	#define ASURA_TEST_AFFECTOR_CODE
	
	#include "Core/Asura_LinkList.h"
	#include "Core/Asura_Timers.h"
#endif

// (MP - 23.01.2008) Currently the PS3 doesn't support job-based botanical
// updates but it does support ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE
// specifically for PFX. Turning on this option automatically is therefore
// no good at the moment and must be done in an engine configuration header
// instead.
//#if 1 && defined( ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE) && defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
//	#define ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED (1)
//#endif

#define ASURA_BGR_MANAGER_UPDATE_TIME_STEP_MINIMUM 0.001f

//*******************************************************************
// Local Classes
//*******************************************************************

float Asura_BGR_Manager::s_fDualSidedLightingLerp = 0.35f;

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	bool Asura_BGR_Manager::s_bDebugVarShow = true;
	bool Asura_BGR_Manager::s_bDebugVarShowPixels = true;
	#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
		bool g_bDebugVarUpdateMultiThreaded = true;
	#endif
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)

class Asura_BGR_Manager_Update_InstanceHeader;

class Asura_BGR_Manager_Update_InstanceHeader_AffectorData
{
public:
#if defined(ASURA_USE_AFFECTORS)
	static const u_int m_uAffectorArraySizeMax = 64;
	Asura_Affector* m_apxAffectors[ m_uAffectorArraySizeMax];
	
	u_int m_uAffectorArraySize;
#endif	// defined(ASURA_USE_AFFECTORS)
};

typedef bool (Asura_BGR_Manager_Update_InstanceHeader::*ptrProcessInstanceHeader)( const Asura_BGR_Manager_Update_InstanceHeader_AffectorData& xAffectorData);

class Asura_BGR_Manager_Update_InstanceHeader
{
public:
	static void Initialise();
	static void Reset();
	static void Shutdown();
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
	static void WaitForJobs();
	bool QueueJob();
#endif
	
	bool ProcessInstanceHeader();
	
private:
	template< bool bUseGlobalAffectors, bool bUseLocalAffectors, bool bTypeHasImposters> 
	bool ProcessInstanceHeaderInstances( const Asura_BGR_Manager_Update_InstanceHeader_AffectorData& xAffectorData);
	
	static bool ExecuteJob( void *pxJob);
	
public:
	float m_fTime;
	Asura_BGR_Instances_Header* m_pxInstanceHeader;
	const Asura_BGR_Type_BasicInteractive* m_pxBasicInteractive;
	Asura_BGR_Type* m_pxType;
	Asura_BGR_Manager* m_pxManager;
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_PLATFORM_DATA_UPDATE)
	Asura_Platform_BGR_Manager_Update_Dynamic_Data m_xPlatformUpdate;
#endif
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
	bool m_bDoTheUpdateLocalAffector;
	bool m_bDoTheUpdateGlobalAffector;
#endif

private:

	static const ptrProcessInstanceHeader s_pafnProcessPointers[ 8];
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
	static Asura_Job_Queue* s_pxJobQueue;
#endif
};

#endif

//*******************************************************************
// Local Data
//*******************************************************************

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)

#if defined( ASURA_TEST_AFFECTOR_CODE)
static Asura_LinkList< Asura_Affector*> g_xList_pxTestAffectors;
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
Asura_Job_Queue* Asura_BGR_Manager_Update_InstanceHeader::s_pxJobQueue = NULL;
#endif

const ptrProcessInstanceHeader Asura_BGR_Manager_Update_InstanceHeader::s_pafnProcessPointers[ 8] = 
{
	&Asura_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeaderInstances< false, false, false>,
	&Asura_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeaderInstances< false, false, true>,
	&Asura_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeaderInstances< false, true,  false>,
	&Asura_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeaderInstances< false, true,  true >,
	&Asura_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeaderInstances< true,  false, false>,
	&Asura_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeaderInstances< true,  false, true>,
	&Asura_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeaderInstances< true,  true,  false>,
	&Asura_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeaderInstances< true,  true,  true >
};
#endif

//*******************************************************************
// Function Definitions
//*******************************************************************

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
template< bool bUseGlobalAffectors, bool bUseLocalAffectors, bool bUsePlatformUpdate>
bool Asura_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeaderInstances( const Asura_BGR_Manager_Update_InstanceHeader_AffectorData& xAffectorData)
{
	const float fTime = this->m_fTime;
	const float fOneOverTime = 1.0f / fTime;	
	const Asura_AVector3 xOneOverTime( fOneOverTime, fOneOverTime, fOneOverTime);
	const Asura_AVector3 xTime( fTime, fTime, fTime);
	Asura_AVector3 xTimeTimeOver2 = xTime;
	xTimeTimeOver2 *= xTime;
	xTimeTimeOver2 *= 0.5f;

	const float fOneOverMass = 1.0f / this->m_pxBasicInteractive->m_fMass;
	const Asura_AVector3 xOneOverMass( fOneOverMass, fOneOverMass, fOneOverMass);
	
	// DLH - 14 November 2006 13:30:00 - The minus is to reverse the vector direction from the bone position offset
	const float fSpringStrength = -0.5f * this->m_pxBasicInteractive->m_fSpringStrength;
	const Asura_AVector3 xSpringStrength( fSpringStrength, fSpringStrength, fSpringStrength);
	
	const float fDrag = -this->m_pxBasicInteractive->m_fDrag;
	const Asura_AVector3 xDrag( fDrag, fDrag, fDrag);
	
	const float fBoneLength = this->m_pxBasicInteractive->m_fBoneLength;
	const Asura_AVector3 xBoneLength( fBoneLength, fBoneLength, fBoneLength);
	
	const float fSpringLengthMax = this->m_pxBasicInteractive->m_fSpringLength;
	
#if defined(ASURA_USE_AFFECTORS)
	const float fTypeBoundingRadius = this->m_pxType->GetBoundingRadius();
#endif	// defined(ASURA_USE_AFFECTORS)
	
	const Asura_AVector3 xBonePosition1 = this->m_pxBasicInteractive->m_xBonePosition1;

	const u_int uInstanceCount = this->m_pxInstanceHeader->GetInstanceCount();
	for( u_int uInstanceIndex = 0; uInstanceIndex < uInstanceCount; uInstanceIndex += 1)
	{
		const Asura_BGR_Instance* pxInstance = this->m_pxInstanceHeader->GetInstance( uInstanceIndex);
		
#if defined( ASURA_TOOLS)
		if( false == pxInstance->IsActive())
		{
			continue;
		}
#endif

		Asura_AVector3 xInstancePosition;
		
		pxInstance->GetInstancePosition( xInstancePosition);
		
		Asura_BGR_DynamicInstance* const pxDynamicInstance = this->m_pxInstanceHeader->GetDynamicInstanceData( uInstanceIndex);
		Asura_BGR_InteractiveBasic* const pxInteractiveBasic = this->m_pxInstanceHeader->GetDynamicInteractiveData( pxDynamicInstance);

		if ( pxInstance->IsVisible() ) //BW animate only if it was visible last frame, this technically creates a 1 frame glitch but makes the botanicals update many times faster.
		{
			Asura_AVector3 xWorldForce;
			
			xWorldForce.Zero();

#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
			if( bUseGlobalAffectors)
			{
				Asura_AVector3 xWorldForceGlobalAffector = Asura_Affector_System::CalculateGlobalAffectorForceAtPosition( xInstancePosition );
				
				xWorldForce += xWorldForceGlobalAffector;
			}
			if( bUseLocalAffectors)
			{
				Asura_AVector3 xWorldForceLocalAffector = Asura_Affector_System::CalculateLocalAffectorForceAtPosition( xInstancePosition, fTypeBoundingRadius, xAffectorData.m_apxAffectors, xAffectorData.m_uAffectorArraySize);

				xWorldForce += xWorldForceLocalAffector;
			}
#endif

			Asura_AMatrix3x3 xInstanceOrientation;
			
			pxInstance->GetOrientationTransposed( xInstanceOrientation);

			Asura_AVector3 xInstanceForces = xWorldForce * xInstanceOrientation;
			
			Asura_AVector3 xOldInstanceBoneOffset;
			xOldInstanceBoneOffset.LoadUnaligned( pxInteractiveBasic->m_afPosition);
			
			{
				// DLH - 14 November 2006 13:30:34 - Force = 0.5 * (k*x*x)
				Asura_AVector3 xSpringForce = xOldInstanceBoneOffset;
				float fOldInstanceBoneOffsetMagnitude = xOldInstanceBoneOffset.Magnitude();
				xSpringForce *= (xSpringStrength * fOldInstanceBoneOffsetMagnitude);
				
				xInstanceForces += xSpringForce;
			}
			
			Asura_AVector3 xOldVelocity;
			xOldVelocity.LoadUnaligned( pxInteractiveBasic->m_afVelocity);
			
			// DLH - 14 November 2006 14:16:18 - Drag Force
			Asura_AVector3 xInstanceDragForce = xDrag;
			xInstanceDragForce *= xOldVelocity;
			
			xInstanceForces += xInstanceDragForce;

			// DLH - 14 November 2006 12:56:35 - Force = mass * acceleration
			Asura_AVector3 xInstanceAccelaration = xInstanceForces;
			xInstanceAccelaration *= xOneOverMass;
			
			// DLH - 14 November 2006 12:59:05 - x = (u*t) + (0.5*a*t*t)
			Asura_AVector3 xInstancePositionVelocity = xOldVelocity;
			xInstancePositionVelocity *= xTime;
			
			Asura_AVector3 xInstancePositionAcceleration = xInstanceAccelaration;
			xInstancePositionAcceleration *= xTimeTimeOver2;
			
			Asura_AVector3 xNewInstanceBoneOffset = xInstancePositionVelocity;
			xNewInstanceBoneOffset += xInstancePositionAcceleration;
			xNewInstanceBoneOffset += xOldInstanceBoneOffset;
			
			{
				// DLH - 14 November 2006 16:50:02 - Move the new bone position to the end of the pole it is ment to be on
				Asura_AVector3 xLooseInstanceBonePosition = xNewInstanceBoneOffset + xBonePosition1;
				
				xLooseInstanceBonePosition.ApproximateNormaliseReturningMagnitude();
				xLooseInstanceBonePosition *= xBoneLength;
				
				xNewInstanceBoneOffset = xLooseInstanceBonePosition - xBonePosition1;
			}
			
			{
				// DLH - 14 November 2006 16:50:02 - Move the new bone position to the end of the pole it is ment to be on
				Asura_AVector3 xLooseInstanceBoneOffset = xNewInstanceBoneOffset;
				
				const float fOffsetLength = xLooseInstanceBoneOffset.ApproximateNormaliseReturningMagnitude();
				if( fSpringLengthMax < fOffsetLength)
				{
					xLooseInstanceBoneOffset *= fSpringLengthMax;
					xNewInstanceBoneOffset = xLooseInstanceBoneOffset;
				}
			}
			
	//			Asura_Assert( xNewInstanceBoneOffset.Magnitude() < 10000.0f, "The new bone offset position seems to be really large");
			xNewInstanceBoneOffset.StoreUnaligned( pxInteractiveBasic->m_afPosition);
			
			Asura_AVector3 xNewInstanceVelocity;
			{
				// DLH - 16 November 2006 16:24:08 - Calculate the Velocity
				const Asura_AVector3 xInstanceMoved = xNewInstanceBoneOffset - xOldInstanceBoneOffset;
				
				xNewInstanceVelocity = xInstanceMoved;
				xNewInstanceVelocity *= xOneOverTime;
				
				xNewInstanceVelocity.StoreUnaligned( pxInteractiveBasic->m_afVelocity);
			}
			
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_PLATFORM_DATA_UPDATE)
			if( true == bUsePlatformUpdate)
			{
				this->m_xPlatformUpdate.UpdateInstance( *(pxInstance), xNewInstanceBoneOffset, xNewInstanceVelocity);
			}
#endif

		}

		//reset visibility flag, so next frame's render will set it if drawn
		pxInstance->m_bVisible = false;
	}
	
	return( false);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
bool Asura_BGR_Manager_Update_InstanceHeader::ProcessInstanceHeader()
{
#ifdef _XBOX
	SCOPEDPIXEVENT((0xff00ff00,  "Botanicals Instance Header Update"));
#endif
#if defined( SUPPORT_VTUNE)
	__itt_event mark_event = __itt_event_create( "BotUpHead", 9 );
	__itt_event_start( mark_event);
#endif

	Asura_BGR_Manager_Update_InstanceHeader_AffectorData xAffectorData;
	
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
	xAffectorData.m_uAffectorArraySize = 0;
	if( true == this->m_bDoTheUpdateLocalAffector)
	{
		const Asura_Bounding_Box& xInstanceHeaderBoundingBox = this->m_pxInstanceHeader->GetBoundingBox();
		
		Asura_Affector_System::MakeAffectorListForBoundingBox( xInstanceHeaderBoundingBox, xAffectorData.m_uAffectorArraySizeMax, xAffectorData.m_apxAffectors, xAffectorData.m_uAffectorArraySize);
	}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_PLATFORM_DATA_UPDATE)
	this->m_xPlatformUpdate.StartInstanceHeader( *(this->m_pxManager), *(this->m_pxType), *(this->m_pxInstanceHeader));
	const bool bUsePlatformUpdate = this->m_xPlatformUpdate.NeedToUpdateInstances();
#endif

#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
	const u_int uCodePathMaskGlobalAffectors = 0x04;
	const u_int uCodePathMaskLocalAffectors = 0x02;
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_PLATFORM_DATA_UPDATE)
	const u_int uCodePathMaskPlatformUpdate = 0x01;
#endif
	u_int uCodePath = 0;
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
	uCodePath |= (m_bDoTheUpdateLocalAffector) ? uCodePathMaskLocalAffectors : 0;
	uCodePath |= (m_bDoTheUpdateGlobalAffector) ? uCodePathMaskGlobalAffectors : 0;
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_PLATFORM_DATA_UPDATE)
	uCodePath |= (true == bUsePlatformUpdate) ? uCodePathMaskPlatformUpdate : 0;
#endif

	{
		// DLH - 12 December 2006 08:22:57 - Get the required pfn for the optimial template version we want to run
		ptrProcessInstanceHeader pfnProcessPointers = s_pafnProcessPointers[ uCodePath];
		
		(this->*(pfnProcessPointers))( xAffectorData);
	}
		
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_PLATFORM_DATA_UPDATE)
	this->m_xPlatformUpdate.StopInstanceHeader( *(this->m_pxInstanceHeader));
#endif

#if defined( SUPPORT_VTUNE)
	__itt_event_end( mark_event);
#endif

	return( true);
}
#endif

// RWM - Static update method for Asura_FunctionSequencer to use.
void Asura_BGR_Manager::Update()
{
	Asura_BGR_Manager& xBGRManager = GetBGRManager();
	xBGRManager.Update( Asura_Timers::GetGameTimeSlice() );
}

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
bool Asura_BGR_Manager::Update( const float fTime)
{
	if( fTime < ASURA_BGR_MANAGER_UPDATE_TIME_STEP_MINIMUM)
	{
		return( true);
	}
	
#ifdef _XBOX
	SCOPEDPIXEVENT((0xff00ff00,  "Botanicals Update"));
#endif
	ASURA_CPU_PROFILE__START_LABEL( ASURA_CPU_PROFILE__BOTANICALS_UPDATE, "Botanicals Update" );
	
	// DLH - 14 November 2006 12:49:12 - Have to do a lot of initalise the variable and then modify it afterwards so the operator*= is used to multiply as the operator* is a dot product
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
	const bool	bDoTheUpdateLocalAffector	= Asura_Affector_System::HasActiveLocalAffectors();
	const bool	bDoTheUpdateGlobalAffector	= Asura_Affector_System::HasActiveGlobalAffectors();
#endif

#if defined( SUPPORT_VTUNE)
	__itt_event mark_event = __itt_event_create( "BotUpdate", 9 );
	__itt_event_start( mark_event);
#endif
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_PLATFORM_DATA_UPDATE)
	Asura_Platform_BGR_Manager_Update_Dynamic_Data::StartProcessing( *this);
#endif

	const u_int uInstanceHeaderCount = this->GetInstanceHeaderCount();
	for( u_int uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
	{
		Asura_BGR_Instances_Header* pxInstanceHeader = this->GetInstanceHeader( uInstanceHeaderIndex);
		if( NULL == pxInstanceHeader)
		{
			continue;
		}
		if(!pxInstanceHeader->GetDynamicInstanceCount())
		{
			continue;
		}
		const bool bHasInteractiveData = pxInstanceHeader->HasDynamicInteractiveData();
		if( false == bHasInteractiveData)
		{
			continue;
		}
		
		const Asura_BGR_TypeID xTypeID = pxInstanceHeader->GetType();
		Asura_BGR_Type* pxType = this->GetType( xTypeID);
		if( NULL == pxType)
		{
			continue;
		}
		
		const Asura_BGR_Type_BasicInteractive* pxBasicInteractive = pxType->GetBasicInteractive();
		if( NULL == pxBasicInteractive)
		{
			continue;
		}

		Asura_BGR_Manager_Update_InstanceHeader xInstanceHeaderUpdate;
		
		xInstanceHeaderUpdate.m_pxInstanceHeader = pxInstanceHeader;
		xInstanceHeaderUpdate.m_pxBasicInteractive = pxBasicInteractive;
		xInstanceHeaderUpdate.m_pxType = pxType;
		xInstanceHeaderUpdate.m_pxManager = this;
		xInstanceHeaderUpdate.m_fTime = fTime;
#if defined( ASURA_USE_AFFECTORS) && 1 == ASURA_USE_AFFECTORS
		xInstanceHeaderUpdate.m_bDoTheUpdateLocalAffector = bDoTheUpdateLocalAffector;
		xInstanceHeaderUpdate.m_bDoTheUpdateGlobalAffector = bDoTheUpdateGlobalAffector;
#endif
		
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
	#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
		if( false == g_bDebugVarUpdateMultiThreaded)
		{
			xInstanceHeaderUpdate.ProcessInstanceHeader( );
		}
		else
	#endif
		{
			xInstanceHeaderUpdate.QueueJob();
		}
#else
		xInstanceHeaderUpdate.ProcessInstanceHeader( );
#endif
		
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
//		Asura_BGR_Manager_Update_InstanceHeader::WaitForJobs();
#endif
	}
	
#if defined( SUPPORT_VTUNE)
	__itt_event mark_wait_event = __itt_event_create( "BotStop", 7 );
	__itt_event_start( mark_wait_event);
#endif

	// DLH - Must turn off the present profile section as we cannot assume only our own tasks will be performed in the wait for tasks
	ASURA_CPU_PROFILE__STOP( ASURA_CPU_PROFILE__BOTANICALS_UPDATE);
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
	Asura_BGR_Manager_Update_InstanceHeader::WaitForJobs();
#endif

	ASURA_CPU_PROFILE__START_LABEL( ASURA_CPU_PROFILE__BOTANICALS_UPDATE,"Botanicals Update");
	
#if defined( SUPPORT_VTUNE)
	__itt_event_end( mark_wait_event);
#endif
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_PLATFORM_DATA_UPDATE)
	Asura_Platform_BGR_Manager_Update_Dynamic_Data::StopProcessing( *this);
#endif

#if defined( SUPPORT_VTUNE)
	__itt_event_end( mark_event);
#endif
	
	ASURA_CPU_PROFILE__STOP( ASURA_CPU_PROFILE__BOTANICALS_UPDATE);
	
	return( true);
}
#else
bool Asura_BGR_Manager::Update( const float fTime)
{
	return( true);
}
#endif

#if !defined(ASURA_BGR_UPDATE_JOBS)
#define ASURA_BGR_UPDATE_JOBS 128
#endif	// !defined(ASURA_BGR_UPDATE_JOBS)

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
bool Asura_BGR_Manager_Update_InstanceHeader::QueueJob()
{
	s_pxJobQueue->Push( this);
	return( true);
}
#endif
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
bool Asura_BGR_Manager_Update_InstanceHeader::ExecuteJob( void *pJob)
{
	Asura_BGR_Manager_Update_InstanceHeader* pxJob = reinterpret_cast< Asura_BGR_Manager_Update_InstanceHeader*>( pJob);
	
	ASURA_CPU_PROFILE__START_MULTITHREADED_LABEL( ASURA_CPU_PROFILE__BOTANICALS_UPDATE, "Botanicals Update" );
	pxJob->ProcessInstanceHeader();
	ASURA_CPU_PROFILE__STOP_MULTITHREADED( ASURA_CPU_PROFILE__BOTANICALS_UPDATE);

	return( true);
}
#endif
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
void Asura_BGR_Manager_Update_InstanceHeader::WaitForJobs()
{
	s_pxJobQueue->WaitForAllJobsToFinish();
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
void Asura_BGR_Manager_Update_InstanceHeader::Initialise()
{
	s_pxJobQueue = new Asura_Job_Queue( ASURA_BGR_UPDATE_JOBS, sizeof( Asura_BGR_Manager_Update_InstanceHeader), ExecuteJob);
	s_pxJobQueue->SetThreadName( "Botanical Updates");
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
void Asura_BGR_Manager_Update_InstanceHeader::Reset()
{
	s_pxJobQueue->Clear();
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
void Asura_BGR_Manager_Update_InstanceHeader::Shutdown()
{
	delete s_pxJobQueue;
	s_pxJobQueue = NULL;
}
#endif

void Asura_BGR_Manager::Initialise()
{
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
	Asura_BGR_Manager_Update_InstanceHeader::Initialise();
#endif
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	Asura_CommandConsole::AddVar( L"Render.Botanical.Show", &(s_bDebugVarShow), Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.Botanical.ShowPixels", &(s_bDebugVarShowPixels), Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
		Asura_CommandConsole::AddVar( L"CPU.MultiThreading.Botanical.MultiThreadedUpdate", &(g_bDebugVarUpdateMultiThreaded), Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	#endif

	Asura_CommandConsole::AddVar( L"Render.Botanical.DualSidedLightingLerp",
		&s_fDualSidedLightingLerp, 0.0f, 1.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif
}

void Asura_BGR_Manager::Reset()
{
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
	Asura_BGR_Manager_Update_InstanceHeader::Reset();
#endif

	GetBGRManager().Platform_Reset();
}

void Asura_BGR_Manager::Shutdown()
{
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE_MULTITHREADED
	Asura_BGR_Manager_Update_InstanceHeader::Shutdown();
#endif
}




//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif	

