//*******************************************************************
//
// File: Asura_Botanical_Game_Render_StateTable.h
//
// Created: 05 December 2006 DLH
//
// Description: 
//
//*******************************************************************

#ifndef __ASURA_BOTANICAL_GAME_RENDER_STATE_TABLE_H__
#define __ASURA_BOTANICAL_GAME_RENDER_STATE_TABLE_H__

// other libraries
#include "Botanicals/Asura_Botanical_Game_Render_Constants.h"

#if (defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES)

#include "Botanicals/Asura_Botanical_Game_Render_Types.h"
#include "Botanicals/Asura_Botanicals_Array.h"

#include "Mesh/Asura_CollisionMesh.h"

// internal dependencies

// forward declarations

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Class Definitions
//*******************************************************************

class Asura_BGR_StateTable_Version
{
public:
	Asura_BGR_StateTable_Version();
	
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif


private:
public:
	u_int m_uProjectFlags;	// Used to store project-specific in the botanicals.

	Asura_BGR_Version_ID  m_xVersionID;

	Asura_BGR_State_Value m_xRangeStart;
	Asura_BGR_State_Value m_xRangeStop;
	
	bool				  m_bCollisionUseBoundingBox;		// Use the bounding box for foliage collisions
	Asura_String		  m_xCollisionMeshName;				// Collision mesh shape name (needed in tools to reload the shape)
	Asura_Hash_ID		  m_xCollisionMeshHash;				// Collision mesh shape hash
	float				  m_fCollisionMeshDensity;			// Collision mesh density value (0-1) - for AI LoS	

	mutable Asura_CollisionMesh*  m_pxCollisionMesh;		// The mesh itself - cached pointer for non-tools builds
	

};

class Asura_BGR_StateTable_Section
{
public:
	Asura_BGR_StateTable_Section();
	
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

private:
public:
	Asura_BGR_Section_ID m_xSectionID;
	
	Asura_Botanical_Array< Asura_BGR_StateTable_Version> m_xArray_xVersions;
};

class Asura_BGR_StateTable
{
public:
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

private:
public:
	Asura_Botanical_Array< Asura_BGR_StateTable_Section> m_xArray_xSections;
};

//*******************************************************************
// Inline Function Definitions
//*******************************************************************

inline Asura_BGR_StateTable_Version::Asura_BGR_StateTable_Version()
{
	this->m_xVersionID = ASURA_BGR_VERSION_ID_OFF;
	this->m_xRangeStart = 1;
	this->m_xRangeStop = 0;

	this->m_bCollisionUseBoundingBox = false;
	this->m_xCollisionMeshName = "";			
	this->m_xCollisionMeshHash = ASURA_HASH_ID_UNSET;
	this->m_fCollisionMeshDensity = 1;
	this->m_pxCollisionMesh = NULL;

}

inline Asura_BGR_StateTable_Section::Asura_BGR_StateTable_Section()
{
	this->m_xSectionID = ASURA_BGR_SECTION_ID_INVALID;
}

//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif // ASURA_BOTANICAL_SUPPORT_STATES

#endif // __ASURA_BOTANICAL_GAME_RENDER_STATE_TABLE_H__
