//*******************************************************************
//
// File: Asura_Botanical_Array.h                       
//
// Created: 15 Mar 2006
//
// Description: This is a simple array storage class.
//				It task to to look after you array memory. Error check it. Delete It.
//				It is not intended for resizing, sort or other vector type methods.
//
//*******************************************************************

#ifndef _ASURA_BOTANICAL_ARRAY_H_
#define _ASURA_BOTANICAL_ARRAY_H_

// other libraries
#include "Debugging/Asura_Assert.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

// internal dependencies

// forward declarations

class Asura_Chunk_SaveBlock_Stream;

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Class Definitions
//*******************************************************************

template< class T > class Asura_Botanical_Array
{
public:
	Asura_Botanical_Array();
	Asura_Botanical_Array( const Asura_Botanical_Array& xRHS);
	
	~Asura_Botanical_Array();
	
	void Clear( void);
	void SetSize( const u_int uSize);
	void Shrink( const u_int uSize);
	u_int Size() const;

#if defined( ASURA_TOOLS)
	void Exspand( const u_int uSize);
	void Reserve( const u_int uSize);
#endif
	
	Asura_Botanical_Array& operator= ( const Asura_Botanical_Array& xRHS);
	
	const T& operator[]( const u_int uIndex) const;
	T& operator[]( const u_int uIndex);
	
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

	bool ClearDirect3DData();
	bool CreateDirect3DData();

private:
	void Init();

	u_int m_uArraySize;
	T* m_ptArray;
#if defined( ASURA_TOOLS)
	u_int m_uArrayAllocatedSize;
#endif

#if defined( ASURA_TOOLS)
	void InitialiseRange( const u_int uFirstIndex, const u_int uLastIndex);
#endif
};

//*******************************************************************
// Inline Function Definitions
//*******************************************************************

template< class T > inline Asura_Botanical_Array< T>& Asura_Botanical_Array< T>::operator= ( const Asura_Botanical_Array< T> & xRHS)
{
	this->Clear();
	
	const u_int uSize = xRHS.m_uArraySize;
	this->m_uArraySize = uSize;
#if defined( ASURA_TOOLS)
	this->m_uArrayAllocatedSize = xRHS.m_uArrayAllocatedSize;
	this->m_ptArray = new T[ xRHS.m_uArrayAllocatedSize];
#else
	this->m_ptArray = new T[ uSize];
#endif

	for( u_int uIndex = 0; uIndex < uSize; uIndex += 1)
	{
		this->m_ptArray[ uIndex] = xRHS.m_ptArray[ uIndex];
	}
	
	return( *this);
}

template< class T > inline T& Asura_Botanical_Array< T>::operator[]( const u_int uIndex)
{
	Asura_Assert( NULL != this->m_ptArray, "Array has not be allocated.");
	Asura_Assertf( uIndex < this->m_uArraySize, ("Trying to access array item %d. The Array only has %d items.", uIndex, this->m_uArraySize));
	
	T& xData = this->m_ptArray[ uIndex];
	
	return( xData);
}

template< class T > inline const T& Asura_Botanical_Array< T>::operator[]( const u_int uIndex) const
{
	Asura_Assert( NULL != this->m_ptArray, "Array has not be allocated.");
	Asura_Assertf( uIndex < this->m_uArraySize, ("Trying to access array item %d. The Array only has %d items.", uIndex, this->m_uArraySize));
	
	const T& xData = this->m_ptArray[ uIndex];
	
	return( xData);
}

template< class T > inline u_int Asura_Botanical_Array< T>::Size( void) const
{
	// DLH - 15 March 2006 12:36:21 - Check that if the pointer is NOT setup then the size is 0
	Asura_Assert( NULL != this->m_ptArray || 0 == this->m_uArraySize, "Asura_Botanical_Array Mismatch in the data pointer and the size.");
	
	return( this->m_uArraySize);
}

template< class T > inline void Asura_Botanical_Array< T>::Init( void)
{	
	this->m_ptArray = NULL;
	this->m_uArraySize = 0;
#if defined( ASURA_TOOLS)
	this->m_uArrayAllocatedSize = 0;
#endif
}

template< class T > inline void Asura_Botanical_Array< T>::Clear( void)
{
	delete[] this->m_ptArray;
	
	this->Init();
}

template< class T > inline void Asura_Botanical_Array< T>::SetSize( const u_int uSize)
{
	Asura_Assertf( NULL == this->m_ptArray, ("Asura_Botanical_Array already has data in it. Please clear() it first."));
	Asura_Assertf( 0 == this->m_uArraySize, ("Asura_Botanical_Array already has data in it. Please clear() it first."));
	
	delete[] this->m_ptArray;
	this->m_ptArray = NULL;
	
	this->m_uArraySize = uSize;
#if defined( ASURA_TOOLS)
	this->m_uArrayAllocatedSize = uSize;
#endif

	if( 0 == uSize)
	{
		return;
	}
	
	this->m_ptArray = new T[ uSize];
	
	Asura_Assertf( NULL != this->m_ptArray, ("Asura_Botanical_Array failed to allocate array of size %d items. (%d bytes)", uSize, uSize * sizeof( T)));
}

template< class T > inline void Asura_Botanical_Array< T>::Shrink( const u_int uSize)
{
	Asura_Assertf( NULL != this->m_ptArray, ("Asura_Botanical_Array does not have any data in it!"));
	Asura_Assertf( 0 < this->m_uArraySize, ("Asura_Botanical_Array does not have any data in it!"));
	Asura_Assertf( uSize <= this->m_uArraySize, ("Asura_Botanical_Array does not have enought data in it to shink it to this size!"));
	
	if( this->m_uArraySize <= uSize)
	{
		return;
	}
	
#if defined( ASURA_TOOLS)
	this->InitialiseRange( uSize, this->m_uArraySize - 1);
#endif

	this->m_uArraySize = uSize;
}

#if defined( ASURA_TOOLS)
template< class T > inline void Asura_Botanical_Array< T>::Exspand( const u_int uSize)
{
	Asura_Assertf( this->m_uArraySize < uSize, ("Exspand Cannot be used to make the array smaller"));
	this->Reserve( uSize);
	
	this->m_uArraySize = uSize;
}
#endif

#if defined( ASURA_TOOLS)
template< class T > inline void Asura_Botanical_Array< T>::Reserve( const u_int uSize)
{
	if( uSize <= this->m_uArrayAllocatedSize)
	{
		return;
	}

	this->m_uArrayAllocatedSize = uSize;	
	T* ptNewData = new T[ uSize];
	T* ptOldData = this->m_ptArray;
	
	for( u_int uIndex = 0; uIndex < this->m_uArraySize; uIndex += 1)
	{
		ptNewData[ uIndex] = ptOldData[ uIndex];
	}
	
	delete[] ptOldData;
	
	this->m_ptArray = ptNewData;
}
#endif

#if defined( ASURA_TOOLS)
template< class T > inline void Asura_Botanical_Array< T>::InitialiseRange( const u_int uFirstIndex, const u_int uLastIndex)
{
	Asura_Assert( NULL != this->m_ptArray, "Initialise Range is being used when there is not memory allocated.");
	Asura_Assertf( uFirstIndex < this->m_uArraySize, ("Initialise Range Start Index %d is too high. Size %d", uFirstIndex, this->m_uArraySize));
	Asura_Assertf( uLastIndex < this->m_uArraySize, ("Initialise Range Start Index %d is too high. Size %d", uLastIndex, this->m_uArraySize));
	Asura_Assertf( uFirstIndex < uLastIndex, ("Initialise Range Start Index %d too high. Last Index %d", uFirstIndex, uLastIndex));
	for( u_int uIndex = uFirstIndex; uIndex <= uLastIndex; uIndex += 1)
	{
		this->m_ptArray[ uIndex] = T();
	}
}
#endif

template< class T > inline Asura_Botanical_Array< T>::Asura_Botanical_Array( void)
{	
	this->Init();
}

template< class T > inline Asura_Botanical_Array< T>::Asura_Botanical_Array( const Asura_Botanical_Array<T>& xRHS)
{
	this->Init();
	
	*this = xRHS;
}

template< class T > inline Asura_Botanical_Array< T>::~Asura_Botanical_Array( void)
{	
	this->Clear();
}

template< class T > inline bool Asura_Botanical_Array< T>::Process( Asura_Chunk_Stream& xStream)
{
	uint32 uArraySize;
	xStream >> ( uArraySize);
	
	this->SetSize( uArraySize);
	
	for( uint32 uArrayIndex = 0; uArrayIndex < uArraySize; uArrayIndex += 1)
	{
		T& tData = (*this)[ uArrayIndex];
		
		if( false == tData.Process( xStream))
		{
			return( false);
		}
	}
	
	return( true);
}

template< class T > 
inline Asura_Chunk_Stream& operator >> ( Asura_Chunk_Stream& xStream, Asura_Botanical_Array< T>& tArray)
{
	uint32 uArraySize;
	xStream >> ( uArraySize);
	
	tArray.SetSize( uArraySize);
	
	for( uint32 uArrayIndex = 0; uArrayIndex < uArraySize; uArrayIndex += 1)
	{
		T& tData = tArray[ uArrayIndex];
		
		xStream >> tData;
	}
	
	return( xStream);
}

#if defined( ASURA_TOOLS)
template< class T > inline bool Asura_Botanical_Array< T>::Export( Asura_Chunk_Stream& xStream) const
{
	const uint32 uArraySize = this->m_uArraySize;
	xStream << ( uArraySize);
	for( uint32 uArrayIndex = 0; uArrayIndex < uArraySize; uArrayIndex += 1)
	{
		const T& tData = (*this)[ uArrayIndex];
		
		if( false == tData.Export( xStream))
		{
			return( false);
		}
	}
	
	return( true);
}
#endif

#if defined( ASURA_TOOLS)
template< class T > 
inline Asura_Chunk_Stream& operator << ( Asura_Chunk_Stream& xStream, const Asura_Botanical_Array< T>& tArray)
{
	const uint32 uArraySize = tArray.Size();
	xStream << ( uArraySize);
	for( uint32 uArrayIndex = 0; uArrayIndex < uArraySize; uArrayIndex += 1)
	{
		const T& tData = tArray[ uArrayIndex];
		
		xStream << tData;
	}
	
	return( xStream);
}
#endif

template< class T > inline bool Asura_Botanical_Array< T>::ClearDirect3DData() 
{ 
	const uint32 uArraySize = this->m_uArraySize;
	for( uint32 uArrayIndex = 0; uArrayIndex < uArraySize; uArrayIndex += 1)
	{
		T& tData = (*this)[ uArrayIndex];
		
		if( false == tData.ClearDirect3DData())
		{
			return( false);
		}
	}
	
	return( true);
}

template< class T > inline bool Asura_Botanical_Array< T>::CreateDirect3DData()
{
	const uint32 uArraySize = this->m_uArraySize;
	for( uint32 uArrayIndex = 0; uArrayIndex < uArraySize; uArrayIndex += 1)
	{
		T& tData = (*this)[ uArrayIndex];
		
		if( false == tData.CreateDirect3DData())
		{
			return( false);
		}
	}
	
	return( true);
}



//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif // _ASURA_ARRAY_H_
