
/**
*
* \file Asura_Botanicals_InstanceManager.h
*
* Created by Dave Reed - 10/11/2005
*
* Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
*
**/


#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_BOTANICALS

#ifndef __ASURA_BOTANICALS_INSTANCEMANAGER_H__
#define __ASURA_BOTANICALS_INSTANCEMANAGER_H__

#ifdef ASURA_TOOLS

#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_LinkList.h"
#include "Shapes/Asura_Shapes.h"
#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
#include "Visibility/Asura_Visibility_System.h"
#endif	//defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)

template<class T> class List;
class Tools_Botanicals_Object;

#define ASURA_BOTANICAL_FLAG_RENDERMESHMERGED 0x00000001 //render mesh has been merged into environment
#define ASURA_BOTANICAL_FLAG_COLLISIONMESHMERGED 0x00000002 //collision mesh has been merged into environment

#define ASURA_BOTANICAL_FLAGS_ANYMERGE (ASURA_BOTANICAL_FLAG_RENDERMESHMERGED|ASURA_BOTANICAL_FLAG_COLLISIONMESHMERGED)

struct Asura_Botanicals_Instance
{
	// These all need saving
	Asura_Matrix_3x3		xOrientation;
	Asura_Vector_3			xPosition;
	float					fScale;
	int						iTemplateUID;						// Unique ID of the template type
	u_int					uFlags;
	float					fBrightness;
	//Unique ID of this instance. Used for tracking merge with environment
	u_int					uInstanceID;
	// Members that are only used at run-time
#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
	Asura_Visibility_Query	m_xVisibilityQuery;
#endif	//defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
	int						iModuleIdx;
	int						iTemplateIdx;					

	// For the tools render only
	Asura_Shape_Instance*	pxShapeInstance;
};

#endif // #ifdef ASURA_TOOLS

class Asura_Botanicals_InstanceManager
{	

public:
	static void Initialise();	
	static void Shutdown();
	static void Reset(); 		
	static void Render();	
	static void SetFade( const float fFade );
	static float GetFade( void );

	// In radians
	static float GetDefaultHorizFoV( void );
	static void SetDefaultHorizFoV( const float fHorizFoV );


#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	static void AddConsoleCommands( void );
	static bool ConsoleCmd_SetFade( float fFade, bool bInterrogate, bool bHelp );
#endif // #if defined(ASURA_SUPPORT_COMMAND_CONSOLE)

#ifdef ASURA_TOOLS

	static void RemapTemplates();				// Updates indices when the template list has
												// changed

	static void ConvertFromToolObjects(bool bLimitRange=false, Asura_Vector_3 *pxCamPos=0, float fRange=0);		// When leaving veg edit mode
	static void ConvertToToolObjects(bool bLimitRange=false, Asura_Vector_3 *pxCamPos=0, float fRange=0);		// When entering veg edit mode
	static void UpdateToolsObjectSet(Asura_Vector_3 *pxCamPos, float fRange, bool bForce);

	static void Save();
	static int  GetNumInstances();
	static int  GetNumInstancesOfTemplate(int iTemplate);
	static int  GetNumInstancesOfTemplateByUID(int iTemplateUID);

	static void AddFilesForExport( List<Asura_String>* pxList, const Asura_String& xEnvFilename);

	static Asura_LinkList<Asura_Botanicals_Instance*> &GetInstanceList()
	{
		return s_xInstanceList;
	};

	//generate ID for new instance
	static u_int GenerateInstanceID(){return s_uNextInstanceID++;}
	//makes sure next instance ID is greater than existing ones
	static void RegisterInstanceID(u_int uID){if(uID>=s_uNextInstanceID) s_uNextInstanceID = uID+1;}
	// Find out which module index the specific position is in
	static int GetModuleIndexFromPosition( const Asura_Vector_3& xPos , const Asura_Matrix_3x3 &xOrientation, bool bSmallObject=false);

	//Sets a callback for filtering the modules that a botanical can be attached to.
	//Should return true for any modules that can be used
	static void SetModuleFilterCallback(bool (*pfnModuleFilterCallback)(int iModule)) {s_pfnModuleFilterCallback = pfnModuleFilterCallback;}

	static void ResetModuleLinkingData(void);

#endif // #ifdef ASURA_TOOLS

#if defined( ASURA_BOTANICALS_BGR_IN_EDIT) && defined( ASURA_TOOLS)
	static void ConnectToolObjectsToBGR();
	static void ConnectToolObjectToBGR( Tools_Botanicals_Object *pxToolsObject);
	static void CreateToolObjectInBGR( Tools_Botanicals_Object *pxToolsObject);
#endif
private:

	static void Platform_Render( const float fFade );
	static void Platform_Initialise( void );
	static void Platform_Shutdown( void );
	static void Platform_Reset( void );
	
#ifdef ASURA_TOOLS

	static void TestModuleTreeRayCallback( const u_int uIndex );

	static Asura_Vector_3 s_xCamPos;
	static Asura_LinkList<Asura_Botanicals_Instance*> s_xInstanceList;

	static u_int s_uNextInstanceID;
	static int s_iModuleIndex;

	static bool (*s_pfnModuleFilterCallback)(int iModule);

#endif // #ifdef ASURA_TOOLS

	static float s_fFade;
	// In radians
	static float s_fDefaultHorizFoV;
};

inline void Asura_Botanicals_InstanceManager::SetFade( const float fFade )
{
	s_fFade = fFade;
}

inline float Asura_Botanicals_InstanceManager::GetFade( void )
{
	return s_fFade;
}

inline float Asura_Botanicals_InstanceManager::GetDefaultHorizFoV( void ) 
{ 
	return s_fDefaultHorizFoV; 
}

inline void Asura_Botanicals_InstanceManager::SetDefaultHorizFoV( const float fHorizFoV ) 
{ 
	s_fDefaultHorizFoV = fHorizFoV; 
}

#endif // __ASURA_BOTANICALS_INSTANCEMANAGER_H__

#endif //  ASURA_USE_BOTANICALS

