
/**
*
* \file Asura_Botanicals_ShapeManager.h
*
* Created by Dave Reed - 09/01/2005
*
* Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
*
**/

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_BOTANICALS

#include "Core/Asura_LinkList.h"
#include "Core/Asura_Hash.h"

#ifdef ASURA_TOOLS
#include "BotanicalCreation/Tools_BotanicalCreation_BotanicalShape.h"
#endif

#ifndef __Asura_Botanicals_ShapeManager_H__
#define __Asura_Botanicals_ShapeManager_H__

#ifdef ASURA_TOOLS

// The botanical templates reference either standard shapes (deprecated?) or
// BotanicalShapes. BotanicalShapes are the proper ones, with LOD

class Asura_BotanicalShape
{
public:
	Asura_BotanicalShape();
	~Asura_BotanicalShape();

	uint32				  GetHashID()		  {return m_uHashID;}
	void				  SetHashID(uint32 u) {m_uHashID=u;}

private:
	// Shapes are referenced by a hash of their name. 
	Asura_Hash_ID		  m_uHashID;

	// Platform-specific render data would probably go in here:
	// Asura_BotanicalShape_PlatformRenderData *m_pRenderData

#ifdef ASURA_TOOLS
	// This is the tools data that contains the LOD levels and links to meshes
	// This can be used to render the shape (slowly) in tools mode.
public:
	Tools_BotanicalShape*		GetToolsShape()								 {return m_pToolsShape;}
	const Tools_BotanicalShape* GetToolsShape() const						 {return m_pToolsShape;}
	void						SetToolsShape(Tools_BotanicalShape *pxShape) {m_pToolsShape=pxShape;}

private:
	Tools_BotanicalShape *m_pToolsShape;	
#endif
};




class Asura_Botanicals_ShapeManager
{
	friend class Asura_BotanicalShape;
public:

	static void						Initialise(void);
	static void						Shutdown(void);
	static void						Reset(void);

	static Asura_BotanicalShape*	GetShapeByHash(Asura_Hash_ID uHashID);
	static Asura_BotanicalShape*	AddShape(Asura_Hash_ID uNameHash);		
	static void						RemoveShape(Asura_BotanicalShape *pxShape);
private:

	static Asura_LinkList<Asura_BotanicalShape *> m_xShapeList;
};

#endif // #ifdef ASURA_TOOLS
#endif //__Asura_Botanicals_ShapeManager_H__
#endif //ASURA_USE_BOTANICALS
