//
// Asura_Chunk_Attractors.cpp
//
// Created by JGP 03/05/06
// 
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_USE_ATTRACTORS

// This file's header.
#include "Asura_Chunk_Attractors.h"

// Asura headers
#include "Asura_SaveBlock.h"
#include "Asura_Chunk_Stream.h"
#include "Attractors/Asura_Attractor_System.h"
#include "Attractors/Asura_Attractor_Compiled.h"
#include "Core/Asura_LinkList.h"
#include "Core/Asura_LBAABB_Tree.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#ifdef ASURA_TOOLS
#include "Entity/EntityTemplate/Asura_ServerEntity_Template.h"
#endif
#ifdef ASURA_USE_STREAMING
#include "Streaming/Asura_Streaming_System.h"
#endif

#ifdef ASURA_TOOLS

#include "Attractors/Tools_Attractor_System.h"
#include "Attractors/Tools_Attractor.h"
#include "Attractors/Tools_Attractor_Node.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Load&Save/Tools_SaveAsura.h"

#endif

///////////////////////////////////////////////////////////////////////////////////////
// Asura_Chunk_Attractors
///////////////////////////////////////////////////////////////////////////////////////


bool Asura_Chunk_Attractors::Process( Asura_Handle xFileHandle ) const
{
	if(Version < 3)
	{
		//Old versions made use of a remapping array which is no longer supported.
		//Insist on a re-export rather than trying to sort out the problems on load.
		//RAR 09/10/2006
		Asura_Assert(0,"Old version of attractor chunk not supported. Level will need to be re-exported");
		return false;
	}

	Asura_Assert( ( Version > 0 ) && ( Version <= CHUNK_VERSION ) , "Wrong version number" );
	Asura_Chunk_File_Stream xStream( xFileHandle );

	Asura_Attractor_Handler* const pxAttractorHandler = &Asura_Attractor_System::GetInstance()->GetCurrentHandler();

	Asura_ResourcePushType(ASURA_RESOURCE_ATTRACTORS);
	{
		//read links
		xStream >> pxAttractorHandler->m_uNumLinks;

		if(pxAttractorHandler->m_uNumLinks)
		{
			Asura_ResourceAllocateArray(pxAttractorHandler->m_puLinks,pxAttractorHandler->m_uNumLinks);
	
			Asura_ReadFromStream_Array_Wrapper<unsigned int> xArray( pxAttractorHandler->m_uNumLinks, pxAttractorHandler->m_puLinks );
			xStream >> xArray;
		}

	
		//read attractors
		xStream >> pxAttractorHandler->m_iNumCompiledAttractors;

		Asura_Assert( !pxAttractorHandler->m_pxCompiledAttractors, "Error, attractor data already loaded" );
		Asura_ResourceAllocateArray(pxAttractorHandler->m_pxCompiledAttractors,pxAttractorHandler->m_iNumCompiledAttractors);

		for ( int i = 0; i < pxAttractorHandler->m_iNumCompiledAttractors; i++ )
		{
			Asura_Hash_ID xType;
			xStream >> xType;

			float fRadius;
			xStream >> fRadius;

			Asura_Vector_3 xPos0, xPos1;
			xStream >> xPos0;
			xStream >> xPos1;

			Asura_Attractor_Compiled* const pxCompiled = pxAttractorHandler->m_pxCompiledAttractors + i;
			pxCompiled->SetType( xType );
			pxCompiled->SetRadius( fRadius );
			pxCompiled->GetNode( 0 )->SetPosition( xPos0 );
			pxCompiled->GetNode( 1 )->SetPosition( xPos1 );

			pxCompiled->m_uNumLinks = 0;
			pxCompiled->m_puLinks = 0;
			
			xStream >> pxCompiled->m_uID;
			pxAttractorHandler->m_xAttractorList.AddToList(pxCompiled);
			
			xStream >> pxCompiled->m_uNumLinks;
			if(pxCompiled->m_uNumLinks)
			{
				u_int uFirstLinkIndex;
				xStream >> uFirstLinkIndex;
				Asura_Assert(uFirstLinkIndex+pxCompiled->m_uNumLinks <= pxAttractorHandler->m_uNumLinks,"");

				pxCompiled->m_puLinks = &pxAttractorHandler->m_puLinks[uFirstLinkIndex];
			}

			// 5 has the normal
			if (Version >= 5)
			{
				xStream >> pxCompiled->m_xNormal;
			}

			if(Version >= 6)
			{
				xStream >> pxCompiled->m_uFlags;
			}
		}

		if(Version >= 4)
		{
			//read entity relative attractors
			xStream >> pxAttractorHandler->m_iNumRelativeAttractors;

			Asura_Assert( !pxAttractorHandler->m_pxRelativeAttractors, "Error, attractor data already loaded" );
			Asura_ResourceAllocateArray(pxAttractorHandler->m_pxRelativeAttractors,pxAttractorHandler->m_iNumRelativeAttractors);

			for ( int i = 0; i < pxAttractorHandler->m_iNumRelativeAttractors; i++ )
			{
				Asura_Hash_ID xType;
				xStream >> xType;

				float fRadius;
				xStream >> fRadius;

				Asura_Vector_3 xPos0, xPos1;
				xStream >> xPos0;
				xStream >> xPos1;

				Asura_Attractor_EntityRelative* const pxRelative = pxAttractorHandler->m_pxRelativeAttractors + i;
				pxRelative->SetType( xType );
				pxRelative->SetRadius( fRadius );
				pxRelative->GetNode( 0 )->SetPosition( xPos0 );
				pxRelative->GetNode( 1 )->SetPosition( xPos1 );

				pxRelative->m_uNumLinks = 0;
				pxRelative->m_puLinks = 0;
				
				xStream >> pxRelative->m_uID;
				pxAttractorHandler->m_xAttractorList.AddToList(pxRelative);
				
				xStream >> pxRelative->m_uNumLinks;
				if(pxRelative->m_uNumLinks)
				{
					u_int uFirstLinkIndex;
					xStream >> uFirstLinkIndex;
					Asura_Assert(uFirstLinkIndex+pxRelative->m_uNumLinks <= pxAttractorHandler->m_uNumLinks,"");

					pxRelative->m_puLinks = &pxAttractorHandler->m_puLinks[uFirstLinkIndex];
				}

				xStream >> pxRelative->m_uParentGuid;
				xStream >> pxRelative->m_axRelativePosition[0];
				xStream >> pxRelative->m_axRelativePosition[1];

				// 5 has the normal
				if (Version >= 5)
				{
					xStream >> pxRelative->m_xNormal;
				}

				if(Version >= 6)
				{
					xStream >> pxRelative->m_uFlags;
				}

				#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
				pxRelative->m_uParentGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid(pxRelative->m_uParentGuid);
				#endif
				

			}
		}

		// now read the tree

		Asura_LBAABB_Tree* const pxTree = &( pxAttractorHandler->m_xAttractorTree );
		pxTree->ReadTreeAndNodesFromChunkStream( xStream );

	}
	Asura_ResourcePopType();

	return true;
}


#ifdef ASURA_TOOLS


const bool Asura_Chunk_Attractors::Create( Asura_Chunk_Stream& xStream )
{
	Asura_Attractor_Handler* const pxAttractorHandler = &Asura_Attractor_System::GetInstance()->GetCurrentHandler();
	
	if(!pxAttractorHandler->m_iNumCompiledAttractors && !pxAttractorHandler->m_iNumRelativeAttractors)
	{
		//Don't bother with chunk if there aren't any attractors
		//Not really sure what the return value is supposed to indicate here ,so picking false at random.
		//RAR 12/10/2006
		return false;
	}

	//RAR 08/03/2006 For recentreing during level exports :
	const Asura_Vector_3 xPositionOffset = Tools_Level_Export::GetExportCentre();

	// save out data
	xStream.WriteChunk_Start( ASURA_CHUNK_ATTRACTORS, CHUNK_VERSION, 0 );

	
	//write links
	xStream << pxAttractorHandler->m_uNumLinks;
	for(u_int i=0; i< pxAttractorHandler->m_uNumLinks ; i++)
	{
		xStream << pxAttractorHandler->m_puLinks[i];
	}

	//write attractors

	xStream << pxAttractorHandler->m_iNumCompiledAttractors;
	const Asura_Attractor_Compiled* pxCompiled = pxAttractorHandler->m_pxCompiledAttractors;
	for ( int i = 0; i < pxAttractorHandler->m_iNumCompiledAttractors; i++, pxCompiled++ )
	{
		
		xStream << pxCompiled->GetType();
		xStream << pxCompiled->GetRadius();
		xStream << pxCompiled->GetNode( 0 )->GetPosition()-xPositionOffset;
		xStream << pxCompiled->GetNode( 1 )->GetPosition()-xPositionOffset;
		xStream << pxCompiled->GetID();
		xStream << pxCompiled->GetNumLinks();

		//find the offset into the link array		
		if(pxCompiled->GetNumLinks())
		{
			u_int uFirstLinkIndex = pxCompiled->m_puLinks - pxAttractorHandler->m_puLinks;
			Asura_Assert(uFirstLinkIndex+pxCompiled->m_uNumLinks <= pxAttractorHandler->m_uNumLinks,"Link pointer isn't part of the global link array");
			xStream << uFirstLinkIndex;
		}

		Asura_Vector_3 xNormal;
		pxCompiled->GetNormal(xNormal);
		xStream << xNormal;

		xStream << pxCompiled->GetFlags();
	}

	//write entity relative attractors

	xStream << pxAttractorHandler->m_iNumRelativeAttractors;
	const Asura_Attractor_EntityRelative* pxRelative = pxAttractorHandler->m_pxRelativeAttractors;
	for ( int i = 0; i < pxAttractorHandler->m_iNumRelativeAttractors; i++, pxRelative++ )
	{
		
		xStream << pxRelative->GetType();
		xStream << pxRelative->GetRadius();
		xStream << pxRelative->GetNode( 0 )->GetPosition()-xPositionOffset;
		xStream << pxRelative->GetNode( 1 )->GetPosition()-xPositionOffset;
		xStream << pxRelative->GetID();
		xStream << pxRelative->GetNumLinks();

		//find the offset into the link array		
		if(pxRelative->GetNumLinks())
		{
			u_int uFirstLinkIndex = pxRelative->m_puLinks - pxAttractorHandler->m_puLinks;
			Asura_Assert(uFirstLinkIndex+pxRelative->m_uNumLinks <= pxAttractorHandler->m_uNumLinks,"Link pointer isn't part of the global link array");
			xStream << uFirstLinkIndex;
		}
		
		xStream << pxRelative->m_uParentGuid;
		xStream << pxRelative->m_axRelativePosition[0];
		xStream << pxRelative->m_axRelativePosition[1];

		Asura_Vector_3 xNormal;
		pxRelative->GetRelativeNormal(xNormal);
		xStream << xNormal;
		
		xStream << pxRelative->GetFlags();

	}

	// write out tree

	pxAttractorHandler->m_xAttractorTree.WriteTreeAndNodesToChunkStream( xStream, -xPositionOffset );

	// finished writing

	xStream.WriteChunk_End();

	return true;

}

#endif


///////////////////////////////////////////////////////////////////////////////////////
// Asura_Chunk_Tools_Attractors
///////////////////////////////////////////////////////////////////////////////////////


#ifdef ASURA_TOOLS

bool Asura_Chunk_Tools_Attractors::Process( Asura_Handle xFileHandle ) const
{
	Asura_Chunk_File_Stream xStream( xFileHandle );

	return Process(xStream,0,0);
}

Tools_Attractor* Asura_Chunk_Tools_Attractors::ImportFromChunkBuffer(const char* pcChunkBuffer,u_int uIDToImport) const
{
	Asura_Chunk_Memory_Stream xStream(Size-sizeof(Asura_Chunk_Tools_Attractors),pcChunkBuffer);

	Tools_Attractor* pxAttractor;
	Process(xStream,uIDToImport,&pxAttractor);
	return pxAttractor;
}


bool Asura_Chunk_Tools_Attractors::Process( Asura_Chunk_Stream& xStream, u_int uIDToImport, Tools_Attractor** ppxOutputAttractor ) const
{
	Tools_Attractor_System* const pxAttractorSystem = Tools_Attractor_System::GetInstance();

	if(ppxOutputAttractor) *ppxOutputAttractor = 0;

	u_int uNextID;
	if(Version >= 2)
	{
		xStream >> uNextID;	
	}
	else
	{
		uNextID = 1;
	}
	if(!uIDToImport)//ignore next ID if importing a single attractor
	{
		pxAttractorSystem->SetNextID(uNextID);
	}

	int iNumAttractors;
	xStream >> iNumAttractors;

	for ( int i = 0; i < iNumAttractors; i++ )
	{
		u_int uAttractorID = 0;
		if(Version >= 2)
		{
			xStream >> uAttractorID;
		}

		//Is this an attractor we want?
		if(uIDToImport == 0 || uIDToImport == uAttractorID)
		{
			//When importing from a template file , we need to assign new IDs
			if(uIDToImport)	uAttractorID = pxAttractorSystem->GenerateID(Tools_SaveAsura::IsEditingMission());
			
			Tools_Attractor* const pxAttractor = new Tools_Attractor(uAttractorID);
			Asura_Assert(!pxAttractor->BelongsToMissionFile() || Tools_SaveAsura::IsEditingMission(),"Found an env hook flagged as belonging to a mission while not editing a mission file.\nThis will be lost when the file is next saved.");


			pxAttractorSystem->AddAttractor( pxAttractor );

			Asura_Hash_ID xType;
			xStream >> xType;
			pxAttractor->SetType( xType );


			float fRadius;
			xStream >> fRadius;
			pxAttractor->SetRadius( fRadius );

			bool bClosed;
			xStream >> bClosed;
			pxAttractor->SetClosed( bClosed );

			if(Version >= 3)
			{
				u_int uTemplateGuid;
				xStream >> uTemplateGuid;
				pxAttractor->SetTemplateGuid(uTemplateGuid);

				int iTemplateVersion;
				xStream >> iTemplateVersion;
				pxAttractor->SetTemplateVersion(iTemplateVersion);
			}

			if(Version >= 4)
			{
				Asura_Guid uParentGuid;
				xStream >> uParentGuid;
				pxAttractor->SetParentGuid(uParentGuid);
			}

			int iNumPoints;
			xStream >> iNumPoints;

			Tools_Attractor_Node* pxPrevNode = NULL;

			for ( int j = 0; j < iNumPoints; j++ )
			{
				Tools_Attractor_Node* const pxNode = new Tools_Attractor_Node( pxAttractor );
				pxAttractorSystem->AddAttractorNode( pxNode );

				if ( j == 0 )
				{
					pxAttractor->SetFirstNode( pxNode );
				}
				else
				{
					pxNode->SetPrevAndNext( pxPrevNode, NULL );
					pxPrevNode->SetPrevAndNext( pxPrevNode->GetPrev(), pxNode );
				}

				Asura_Vector_3 xPos;
				xStream >> xPos;
				pxNode->SetPosition( xPos );

				u_int uID;
				if(Version >= 2)
				{
					xStream >> uID;				
				}
				else
				{
					//generate an ID for old attractors
					uID = pxAttractorSystem->GenerateID(false);
				}
				//When importing from a template file , we need to assign new IDs
				if(uIDToImport)	uID = pxAttractorSystem->GenerateID(Tools_SaveAsura::IsEditingMission());

				pxNode->SetID(uID);

				if(Version >= 4)
				{
					xStream >> xPos;
					pxNode->SetRelativePosition(xPos);
				}


				pxPrevNode = pxNode;

			}

			//read links
			if(Version >= 2)
			{
				xStream >> pxAttractor->m_uNumLinks;
				if(pxAttractor->m_uNumLinks)
				{
					pxAttractor->m_puLinks = new u_int[pxAttractor->m_uNumLinks];

					for(u_int i=0;i<pxAttractor->m_uNumLinks;i++)
					{
						xStream >> pxAttractor->m_puLinks[i];
					}
				}
			}

			// 5 has the normal
			if (Version >= 5)
			{
				xStream >> pxAttractor->m_fAngleForNormal;
			}

			//version 6 adds flags and controller guid
			if(Version >= 6)
			{
				xStream >> pxAttractor->m_uFlags;
				
				//controller guid removed with version 8
				if(Version < 8)
				{
					xStream.Skip(sizeof(Asura_Guid));
				}
			}

			if(Version < 7)
			{
				//The way that the normal is calculated from the angle is changed with version 7.
				//So recalculate the angle to keep the same normal as before.
				//RAR 21/09/2007
				pxAttractor->HandleUpdateForChangeToNormalCalculationForNearVerticalEnvHooks();
			}

			if(ppxOutputAttractor)
			{
				*ppxOutputAttractor = pxAttractor;
				return false;//success , but returning false since we aren't reading the rest of the chunk
			}

		}
		else
		{
			//skip this attractor
			Asura_Assert(Version >= 3,"Shouldn't be getting earlier versions than this in template files");

			Asura_Hash_ID xType;
			xStream >> xType;

			float fRadius;
			xStream >> fRadius;

			bool bClosed;
			xStream >> bClosed;

			u_int uTemplateGuid;
			xStream >> uTemplateGuid;

			int iTemplateVersion;
			xStream >> iTemplateVersion;

			if(Version >= 4)
			{
				Asura_Guid uParentGuid;
				xStream >> uParentGuid;
			}

			int iNumPoints;
			xStream >> iNumPoints;

			//skip the nodes
			for ( int j = 0; j < iNumPoints; j++ )
			{

				Asura_Vector_3 xPos;
				xStream >> xPos;

				u_int uID;
				xStream >> uID;

				if(Version >= 4)
				{
					xStream >> xPos;
				}
			}

			//skip links
			u_int uNumLinks;
			xStream >> uNumLinks;
			xStream.Skip(uNumLinks * sizeof(u_int));

			// 5 has the normal
			if (Version >= 5)
			{
				float fAngle;
				xStream >> fAngle;
			}

			//version 6 adds flags and controller guid
			if(Version >= 6)
			{
				xStream.Skip(sizeof(u_int));
				//controller guid removed with version 8
				if(Version < 8)
				{
					xStream.Skip(sizeof(Asura_Guid));
				}

			}

		}
	}

	xStream.Skip4BytePadding();
	return true;
}



void Asura_Chunk_Tools_Attractors::Create( Asura_SaveBlock* pxSaveBlock, bool bMissionFile)
{

	Asura_Chunk_SaveBlock_Stream xStream( pxSaveBlock );

	xStream.WriteChunk_Start( ASURA_CHUNK_TOOLS_ATTRACTORS, CHUNK_VERSION, 0 );
	
	Tools_Attractor_System* const pxAttractorSystem = Tools_Attractor_System::GetInstance();

	xStream << pxAttractorSystem->GetNextID(bMissionFile);

	//exporting a template file?
	const Asura_ServerEntity_Template* pxTemplateEntity = Tools_SaveAsura::GetTemplateBeingExported();


	int iNumAttractors = 0;
	for ( Asura_Hashed_List_ConstIt<Tools_Attractor> xIt(pxAttractorSystem->GetAttractorList()); !xIt.Done() ; xIt.Next() )
	{
		const Tools_Attractor* const pxAttractor = xIt.GetCurrent();
		//skip attractors that aren't for this file
		if(bMissionFile != pxAttractor->BelongsToMissionFile()) continue;

		if(pxTemplateEntity && !pxTemplateEntity->AttractorBelongsToTemplate(pxAttractor->GetID())) continue;

		iNumAttractors++;
	}
	xStream << iNumAttractors;

	for ( Asura_Hashed_List_ConstIt<Tools_Attractor> xIt(pxAttractorSystem->GetAttractorList()); !xIt.Done() ; xIt.Next() )
	{
		const Tools_Attractor* const pxAttractor = xIt.GetCurrent();
		//skip attractors that aren't for this file
		if(bMissionFile != pxAttractor->BelongsToMissionFile()) continue;

		if(pxTemplateEntity && !pxTemplateEntity->AttractorBelongsToTemplate(pxAttractor->GetID())) continue;

		xStream << pxAttractor->GetID();

		const Asura_Hash_ID xType = pxAttractor->GetType();
		xStream << xType;

		const float fRadius = pxAttractor->GetRadius();
		xStream << fRadius;

		const bool bClosed = pxAttractor->IsClosed();
		xStream << bClosed;

		xStream << pxAttractor->GetTemplateGuid();
		xStream << pxAttractor->GetTemplateVersion();
		xStream << pxAttractor->GetParentGuid();

		// count points

		const Tools_Attractor_Node* pxNode = pxAttractor->GetFirstNode();
		int iNumNodes = 0;
		while ( pxNode )
		{
			iNumNodes++;
			pxNode = pxNode->GetNext();
		}

		xStream << iNumNodes;

		pxNode = pxAttractor->GetFirstNode();
		while ( pxNode )
		{
			const Asura_Vector_3 xPos = pxNode->GetPosition();
			if(pxTemplateEntity)
			{
				xStream << xPos - pxTemplateEntity->GetPosition();
			}
			else
			{
				xStream << xPos;
			}
			xStream << pxNode->GetID();
			xStream << pxNode->GetRelativePosition();

			pxNode = pxNode->GetNext();
		}

		// write links
		xStream << pxAttractor->m_uNumLinks;

		for(u_int i=0;i<pxAttractor->m_uNumLinks;i++)
		{
			xStream << pxAttractor->m_puLinks[i];
		}

		// Write the angle for the normal
		xStream << pxAttractor->m_fAngleForNormal;

		xStream << pxAttractor->GetFlags();
	}
	
	xStream.Write4BytePadding();
	xStream.WriteChunk_End();

}

#endif // ASURA_TOOLS

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Asura_Chunk_Attractors_InGame **
**------------------------------*/

bool Asura_Chunk_Attractors_InGame::Process(Asura_Chunk_Stream& xStream) const
{
	Asura_Attractor_Handler* pxHandler = &Asura_Attractor_System::GetInstance()->GetCurrentHandler();
	Asura_Assert(pxHandler,"");
	
	
	// Is this the correct version?

	if (Version > CHUNK_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for attractor (env hook) in-game state",Version));
		return false;
	}
	

	{
		int iNumSavedAttractors;
		xStream >> iNumSavedAttractors;

		//make sure the number of saved attractors matches the number that exist
		if(iNumSavedAttractors != pxHandler->m_iNumCompiledAttractors)
		{
			Asura_Assert(iNumSavedAttractors == pxHandler->m_iNumCompiledAttractors,"Incorrect number of attractors (env hooks) in game state chunk");
			return false;
		}

		Asura_Attractor_Compiled* pxAttractor = pxHandler->m_pxCompiledAttractors;
		for(int i=0; i < pxHandler->m_iNumCompiledAttractors; i++, pxAttractor++)
		{
			u_int uFlags;
			xStream >> uFlags;
			pxAttractor->SetFlags(uFlags);
		}

	}

	{
		int iNumSavedAttractors;
		xStream >> iNumSavedAttractors;

		//make sure the number of saved attractors matches the number that exist
		if(iNumSavedAttractors != pxHandler->m_iNumRelativeAttractors)
		{
			Asura_Assert(iNumSavedAttractors == pxHandler->m_iNumRelativeAttractors,"Incorrect number of attractors (env hooks) in game state chunk");
			return false;
		}

		Asura_Attractor_EntityRelative* pxAttractor = pxHandler->m_pxRelativeAttractors;
		for(int i=0; i < pxHandler->m_iNumRelativeAttractors; i++, pxAttractor++)
		{
			u_int uFlags;
			xStream >> uFlags;
			pxAttractor->SetFlags(uFlags);
		}

	}

	return true;
}

bool Asura_Chunk_Attractors_InGame::Process(const char* pcChunkBuffer) const
{
	Asura_Chunk_ChunkBuffer_Stream xStream(pcChunkBuffer);
	return Process(xStream);
}

void Asura_Chunk_Attractors_InGame::Create(Asura_Chunk_Stream& xStream)
{
	const Asura_Attractor_Handler* pxHandler = &Asura_Attractor_System::GetInstance()->GetCurrentHandler();
	Asura_Assert(pxHandler,"");
	
	u_int uChunkFlags = 0;
	xStream.WriteChunk_Start(ASURA_CHUNK_ATTRACTORS_INGAME, CHUNK_VERSION, uChunkFlags);

	//save the state of the fixed attractors
	{
		xStream << pxHandler->m_iNumCompiledAttractors;

		const Asura_Attractor_Compiled* pxAttractor = pxHandler->m_pxCompiledAttractors;
		for(int i=0; i < pxHandler->m_iNumCompiledAttractors; i++, pxAttractor++)
		{
			xStream << pxAttractor->GetFlags();
		}
	}


	//and the entity relative attractors
	{
		xStream << pxHandler->m_iNumRelativeAttractors;

		const Asura_Attractor_EntityRelative* pxAttractor = pxHandler->m_pxRelativeAttractors;
		for(int i=0; i < pxHandler->m_iNumRelativeAttractors; i++, pxAttractor++)
		{
			xStream << pxAttractor->GetFlags();
		}
	}
	


	xStream.WriteChunk_End();
}

void Asura_Chunk_Attractors_InGame::Create(Asura_SaveBlock_Base* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create(xStream);
}



#endif // ASURA_USE_ATTRACTORS
