/**
*
* \file Asura_Chunk_BillboardPlant_Batches.cpp
*
* Created by Andrew Weinkove - 06/12/2002
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
**/

#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_BillboardPlant_Batches.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_Handle.h"
#include "Core/Asura_String.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"
#include "SpecialFX/Asura_BillboardPlant_System.h"
#include "SpecialFX/Asura_BillboardPlant_Batch.h"

#ifdef ASURA_USE_BILLBOARD_PLANTS
#ifdef ASURA_TOOLS
bool Asura_Chunk_BillboardPlant_Batches::Create(const char* szFilename)
{
	unsigned uNumberOfBatches = Asura_BillboardPlant_System::GetNumberOfBatches();
	//Calculate size of chunk: Header + batches
	int iChunkSize = sizeof(Asura_Chunk_BillboardPlant_Batches);
	iChunkSize += sizeof(int);//uNumberOfBatches
	for (unsigned i=0;i<uNumberOfBatches;i++)
	{
		Asura_BillboardPlant_Batch* pxBatch = Asura_BillboardPlant_System::GetBillboardPlantBatch(i);
		iChunkSize += pxBatch->GetChunkBufferSize();
	}
	//Create saveblock and chunk
	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	Asura_Assert(pxSaveBlock, "Unable to create saveblock");
	if (!pxSaveBlock)
		return false;
	Asura_Chunk_BillboardPlant_Batches* pxChunk = (Asura_Chunk_BillboardPlant_Batches*)pxSaveBlock->GetAccessPointer(iChunkSize);

	//Fill chunk header
	pxChunk->ID = ASURA_CHUNK_BILLBOARDPLANTS_BATCHES;
	pxChunk->Size = iChunkSize;
	pxChunk->Version = 1;
	pxChunk->Flags = 0;

	char *pcChunkBuffer = (char*)(pxChunk+1);
	// Fill template data.
	ASURA_CHUNK_FILL(pcChunkBuffer, uNumberOfBatches, unsigned);
	for (unsigned i=0;i<uNumberOfBatches;i++)
	{
		Asura_BillboardPlant_Batch* pxBatch = Asura_BillboardPlant_System::GetBillboardPlantBatch(i);
		pxBatch->FillInChunkBuffer(pcChunkBuffer);
	}
	//Terminate and save chunk
	pxSaveBlock->AddAsuraTerminator();
	bool bRet = pxSaveBlock->SaveFile( szFilename, false );
	delete pxSaveBlock;
	return bRet;
}
#endif // ASURA_TOOLS

bool Asura_Chunk_BillboardPlant_Batches::Process(Asura_Handle fileHandle) const
{
	//Call shutdown first in case any batches are already loaded.
	Asura_BillboardPlant_System::Reset();

	//Create storeage space for batches
	unsigned uNumberOfBatches;
	ASURA_CHUNK_FILEEXTRACT(fileHandle, uNumberOfBatches, unsigned);
	if (!uNumberOfBatches)
		return true;
	Asura_BillboardPlant_System::AllocateBatchPointers(uNumberOfBatches);
	//Load data into batches store
	for (unsigned i=0;i<uNumberOfBatches;i++)
	{
		Asura_BillboardPlant_Batch *pxBatch = new Asura_BillboardPlant_Batch(fileHandle);
		Asura_BillboardPlant_System::SetBatchPointer(pxBatch, i);
	}
	return true;
}

#endif //ASURA_USE_BILLBOARD_PLANTS
