
/**
*
* \file Asura_Chunk_Botanicals_Templates.cpp
*
* Created by Dave Reed - 11/11/2005
*
* Copyright (C) 2002 Rebellion (Derby), All Rights Reserved.
*
**/

#include "Core/Asura.h"

#ifdef ASURA_USE_BOTANICALS

#ifdef ASURA_TOOLS

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_Handle.h"
#include "Core/Asura_String.h"
#include "Core/Asura_TextureFlags.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"

#include "ChunkSystem/Asura_Chunk_Botanicals_Templates.h"
#include "Botanicals/Asura_Botanicals_TemplateManager.h"
#include "Botanicals/Tools_Botanicals_Styles.h"


#define ASURA_CHUNK_BOTANICALS_TEMPLATES_CURRENT_VERSION 4

bool Asura_Chunk_Botanicals_Templates::Create(char* szFileName)
{
	int iNumberOfTemplates = Asura_Botanicals_TemplateManager::GetNumTemplates();
	
	//Create saveblock and chunk
	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	Asura_Assert(pxSaveBlock, "Unable to create saveblock");

	if(pxSaveBlock)
	{
		Asura_Chunk_SaveBlock_Stream	xStream (pxSaveBlock);

		xStream.WriteChunk_Start(ASURA_CHUNK_BOTANICALS_TEMPLATES,ASURA_CHUNK_BOTANICALS_TEMPLATES_CURRENT_VERSION,0);
		xStream << iNumberOfTemplates;
		xStream << Asura_Botanicals_TemplateManager::s_iNextUID;

		// Fill template data 
		for (int i=0;i<iNumberOfTemplates;i++)
		{
			Asura_Botanicals_Template *pxTemplate=Asura_Botanicals_TemplateManager::GetTemplate(i);		

			if (pxTemplate->xShapeName.IsNull())
			{
				pxTemplate->xShapeName="";
			}
			Asura_String xShapeNameStripped			= Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(pxTemplate->xShapeName.GetString());

			if (pxTemplate->xSelectionShapeName.IsNull())
			{
				pxTemplate->xSelectionShapeName="";
			}
			Asura_String xSelectionShapeNameStripped= Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(pxTemplate->xSelectionShapeName.GetString());

			xStream << pxTemplate->xTemplateName;
			xStream << xShapeNameStripped;
			xStream << xSelectionShapeNameStripped;
			xStream << ((int)(pxTemplate->eType));
			xStream << pxTemplate->iUID;
			xStream << ((int)(pxTemplate->bVisible));
			xStream << ((int)(pxTemplate->bLocked));
			xStream << ((int)(pxTemplate->bMergeRenderMeshWithEnv));
			xStream << ((int)(pxTemplate->bMergeCollisionMeshWithEnv));
			xStream << ((int)(pxTemplate->bPermanentlyMergeMesh));
			xStream << pxTemplate->fMinScale;
			xStream << pxTemplate->fMaxScale;
			xStream << pxTemplate->fMinYRotation;
			xStream << pxTemplate->fMaxYRotation;
			xStream << pxTemplate->fCollisionRadius;
			xStream << pxTemplate->fCollisionHeight;
			xStream << pxTemplate->uShapeID;

		}

		xStream.WriteChunk_End();
		xStream.WriteAsuraTerminator();

		bool bSuccess=pxSaveBlock->SaveFile( szFileName, false );
		
		delete pxSaveBlock;

		if (bSuccess)
		{
			return true;
		}
	}

	return false;
}

bool Asura_Chunk_Botanicals_Templates::Process(Asura_Handle iFileHandle) const
{
	Asura_Assertf(Version<=ASURA_CHUNK_BOTANICALS_TEMPLATES_CURRENT_VERSION,("Unexpected version of botanical templates (%d). These probably won't load correctly.",Version));

	int iNumberOfTemplates;
	int iTemp=0;

	// Get the number of templates
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	xStream >> iNumberOfTemplates;
		
	// Restore next free UID		
	xStream >> Asura_Botanicals_TemplateManager::s_iNextUID;

	// Allocate space for the templates
	Asura_Botanicals_TemplateManager::AllocateTemplates(iNumberOfTemplates);

	// Fill template data 
	for (int i=0;i<iNumberOfTemplates;i++)
	{
		Asura_Botanicals_Template* pxTemplate=Asura_Botanicals_TemplateManager::GetTemplate(i);		
		
		xStream >> pxTemplate->xTemplateName;

		Asura_String xTemp;
		xStream >> xTemp;
		if (xTemp.IsNull())
		{
			xTemp="";
		}
#ifdef ASURA_TOOLS
		pxTemplate->xShapeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(xTemp.GetString());
#else
		pxTemplate->xShapeName = xTemp;
#endif
		
		xStream >> xTemp;
		if (xTemp.IsNull())
		{
			xTemp="";
		}
#ifdef ASURA_TOOLS
		pxTemplate->xSelectionShapeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(xTemp.GetString());
#else
		pxTemplate->xSelectionShapeName = xTemp;
#endif

		xStream >> iTemp;
		pxTemplate->eType=(ASURA_BOTANICALS_TEMPLATE_TYPE)iTemp;
		xStream >> pxTemplate->iUID;

		if (Version>=2)
		{
			// size of a bool has changed.. This is for 4 byte bools			
			xStream >> iTemp;
			pxTemplate->bVisible= iTemp?true:false;
			xStream >> iTemp;
			pxTemplate->bLocked = iTemp?true:false;
		}
		else
		{
			// This is for 1-byte bools
			char u8Temp;
			xStream >> u8Temp;
			pxTemplate->bVisible= u8Temp?true:false;
			xStream >> u8Temp;
			pxTemplate->bLocked = u8Temp?true:false;			
		}

		if(Version>=3)
		{
			xStream >> iTemp;
			pxTemplate->bMergeRenderMeshWithEnv= iTemp?true:false;
			xStream >> iTemp;
			pxTemplate->bMergeCollisionMeshWithEnv = iTemp?true:false;
		}
		else
		{
			pxTemplate->bMergeRenderMeshWithEnv = false;
			pxTemplate->bMergeCollisionMeshWithEnv = false;
		}

		if(Version>=4)
		{
			xStream >> iTemp;
			pxTemplate->bPermanentlyMergeMesh = iTemp?true:false;
		}
		else
		{
			pxTemplate->bPermanentlyMergeMesh = false;
		}

		
		xStream >> pxTemplate->fMinScale;
		xStream >> pxTemplate->fMaxScale;
		xStream >> pxTemplate->fMinYRotation;
		xStream >> pxTemplate->fMaxYRotation;
		xStream >> pxTemplate->fCollisionRadius;
		xStream >> pxTemplate->fCollisionHeight;
		xStream >> pxTemplate->uShapeID;		

		// Init anything that's not loaded straight from the chunk
		pxTemplate->pxBotanicalShape=NULL;
		pxTemplate->uSelectionShapeID=ASURA_HASH_ID_UNSET;
	}

	return true;
}

bool Asura_Chunk_Botanicals_Styles::Create(char* szFileName)
{
	int iNumberOfStyles = Tools_Botanicals_StyleManager::GetNumStyles();

	//Create saveblock and chunk
	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	Asura_Assert(pxSaveBlock, "Unable to create saveblock");

	if(pxSaveBlock)
	{
		Asura_Chunk_SaveBlock_Stream	xStream (pxSaveBlock);

		xStream.WriteChunk_Start(ASURA_CHUNK_BOTANICALS_STYLES,1,0);
		
		xStream << iNumberOfStyles;

		for(int i=0;i<iNumberOfStyles;i++)
		{
			Tools_Botanicals_Style* pxStyle=Tools_Botanicals_StyleManager::GetStyle(i);			

			xStream << pxStyle->xStyleName;

			for(int j=0;j<TOOLS_BOTANICALS_ENTRIES_PER_STYLE;j++)
			{
				xStream << pxStyle->axEntries[j].iTemplateUID;
				xStream << pxStyle->axEntries[j].fWeight;
			}
		}

		xStream.WriteChunk_End();
		xStream.WriteAsuraTerminator();

		bool   bSuccess=pxSaveBlock->SaveFile( szFileName, false );
		delete pxSaveBlock;
		return bSuccess;		
	}
	
	return false;
}

bool Asura_Chunk_Botanicals_Styles::Process(Asura_Handle iFileHandle) const
{
	int iNumberOfStyles;		

	if (Version!=1)
	{
		return false;
	}

	Asura_Chunk_File_Stream	xStream(iFileHandle);
	xStream >> iNumberOfStyles;
	
	Tools_Botanicals_StyleManager::Reset();

	// Add the styles
	for (int i=0;i<iNumberOfStyles;i++)
	{
		Tools_Botanicals_Style* pxStyle=Tools_Botanicals_StyleManager::AddStyle();
	
		xStream >> pxStyle->xStyleName;

		for(int j=0;j<TOOLS_BOTANICALS_ENTRIES_PER_STYLE;j++)
		{
			xStream >> pxStyle->axEntries[j].iTemplateUID;
			xStream >> pxStyle->axEntries[j].fWeight;
		}
	}	

	// Get style indices from UIDs
	Tools_Botanicals_StyleManager::RemapStyles();

	return true;
}

#endif // #ifdef ASURA_TOOLS

#endif // ASURA_USE_BOTANICALS
