//
// Asura_Chunk_Dialogue.cpp
//
// Created by Richard Rice - 02/11/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_Dialogue.h"

#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_PrimeSearch.h"
#include "Core/Resources/Asura_ResourceSet.h"
#include "Sound/Asura_Dialogue.h"
#include "Sound/Asura_Dialogue_System.h"
#include "Sound/Asura_Sound_Mixer.h"
#include "Sound/Asura_Sound_Event.h"

#ifdef  ASURA_TOOLS
#include "Sound/Tools_Dialogue_System.h"
#endif

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Asura_Chunk_Dialogue_Event **
**----------------------------*/

#define ASURA_CHUNK_DIALOGUE_EVENT_VERSION 5
//Version 1 - Added static message stuff
//			RAR 15/11/2006
//Version 2 - Allowed messages both at start and end of line
//			RAR 22/11/2006
//Version 3 - Names of playables saved (instead of just the IDs)
//			RAR 27/11/2006
//Version 4 - Added m_uFinalMessage
//			RAR 01/06/2007
//Version 5 - Added "m_uFaceAnimationHash" (or blank)
//			ART 05/09/2008
//			AJG - 09/09/08 - This was the wrong version number to increase, as the new hash is in the line not the event. Leaving this here since the change has already appeared in exports.


bool Asura_Chunk_Dialogue_Event::Process(Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?

	if (Version > ASURA_CHUNK_DIALOGUE_EVENT_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for dialogue event",Version));
		return false;
	}


	char acName[128];
	xStream.ReadString(acName,sizeof(acName));

	//Allocate or find the dialogue event
	bool bAlreadyExists;
	Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::CreateEvent(acName, bAlreadyExists);
	Asura_Assert(pxEvent, "");

	//make sure the event doesn't already exist
	if(bAlreadyExists)
	{
		#ifdef ASURA_TOOLS
		Asura_Assertf(0,("Dialogue event %s already exists",acName));
		#endif
		//probably want to add some logging to file_contents.txt or similar to go along with this
		return false;
	}

	int iTemp;
	xStream >> iTemp;
	pxEvent->m_ePlaybackType = (ASURA_DIALOGUE_PLAYBACK_TYPE) iTemp;
	xStream >> pxEvent->m_fReplayDelayTime;
	xStream >> pxEvent->m_ucPlayProbability;
	xStream >> pxEvent->m_cPriority;
	xStream >> pxEvent->m_bGlobalEvent;

	//message store
	if(Version >= 1)
	{
		u_int uNumMessages;
		xStream >> uNumMessages;
		pxEvent->m_xMessageStore.SetSize(uNumMessages);

		for(u_int i=0;i<uNumMessages;i++)
		{
			Asura_Dialogue_Message* pxMessage = new Asura_Dialogue_Message;
			pxEvent->m_xMessageStore[i] = pxMessage;

			xStream >> pxMessage->m_uHashID;
			#ifdef ASURA_DIALOGUE_INCLUDE_TOOLS_DATA
			xStream >> pxMessage->m_xName;
			#else
			xStream.SkipString();
			#endif

			pxMessage->m_xMessageBlock.ReadFromChunkStream(xStream);
		}
	}
	
	if(Version >= 1)
	{
		xStream >> pxEvent->m_uInitialMessage;
	}
	
	if(Version >= 4)
	{
		xStream >> pxEvent->m_uFinalMessage;
	}
   	
	//playable list
	u_int uNumPlayable;
	xStream >> uNumPlayable;
	if(uNumPlayable)
	{
		#ifdef ASURA_TOOLS
		//In tools we just fill out the playable ID list for now. The pointers to the playables are found later , after
		//everything has been loaded.
		pxEvent->m_xPlayableIDList.SetSize(uNumPlayable);
		for(u_int i=0;i<uNumPlayable;i++)
		{
			Asura_Dialogue_PlayableIDListItem xItem;
			xStream >> xItem.m_xID.m_uHashID;
			xStream >> iTemp;
			xItem.m_xID.m_eType = (ASURA_DIALOGUE_PLAYABLE_TYPE) iTemp;
			xStream >> xItem.m_fDelay;
			xStream >> xItem.m_ucWeight;
			if(Version >= 1)
			{
				xStream >> xItem.m_uStartMessageID;
			}
			else
			{
				xItem.m_uStartMessageID = 0;
			}
			if(Version >= 2)
			{
				xStream >> xItem.m_uEndMessageID;
			}
			else
			{
				xItem.m_uEndMessageID = 0;
			}
			if(Version >= 3)
			{
				xStream >> xItem.m_xName;
			}

			pxEvent->m_xPlayableIDList[i] = xItem;
		}
		#else  //ASURA_TOOLS
		//In game any events/samples that this event depends on should already be loaded , so we can set up the playable
		//list directly.
		pxEvent->m_xPlayableList.SetSize(uNumPlayable);
		pxEvent->m_xDelayList.SetSize(uNumPlayable);
		pxEvent->m_xWeightsList.SetSize(uNumPlayable);
		pxEvent->m_xStartMessageList.SetSize(uNumPlayable);
		pxEvent->m_xEndMessageList.SetSize(uNumPlayable);

		u_int uIndex = 0;
		for(u_int i=0;i<uNumPlayable;i++)
		{
			Asura_Dialogue_PlayableID xID;
			xStream >> xID.m_uHashID;
			xStream >> iTemp;
			xID.m_eType = (ASURA_DIALOGUE_PLAYABLE_TYPE) iTemp;

			float fDelay;
			xStream >> fDelay;

			u_char ucWeight;
			xStream >> ucWeight;

			Asura_Hash_ID uStartMessageID = 0;
			Asura_Hash_ID uEndMessageID = 0;
			if(Version >= 1)
			{
				xStream >> uStartMessageID;
			}
			if(Version >= 2)
			{
				xStream >> uEndMessageID;
			}
			if(Version >= 3)
			{
				xStream.SkipString();//skips the name
			}

			const Asura_Dialogue_Playable* pxPlayable = 0;
			if(xID.m_eType == ASURA_DIALOGUE_PLAYABLE_LINE)
			{
				pxPlayable = Asura_Dialogue_Storage::FindLine(xID.m_uHashID);
			}
			else
			{
				Asura_Assert(xID.m_eType == ASURA_DIALOGUE_PLAYABLE_EVENT,"");
				pxPlayable = Asura_Dialogue_Storage::FindEvent(xID.m_uHashID);
			}
			Asura_Assertf(pxPlayable,("Failed to find %s 0x%x for dialogue event %s",xID.m_eType == ASURA_DIALOGUE_PLAYABLE_LINE ? "line" : "event",xID.m_uHashID,acName));
			if(pxPlayable)
			{
				pxEvent->m_xPlayableList[uIndex] = pxPlayable;
				pxEvent->m_xDelayList[uIndex] = fDelay;
				pxEvent->m_xWeightsList[uIndex] = ucWeight;
				pxEvent->m_xStartMessageList[uIndex] = uStartMessageID;
				pxEvent->m_xEndMessageList[uIndex] = uEndMessageID;
				uIndex++;
			}
		}
		if(uIndex != uNumPlayable)
		{
			//failed to find some of the events/samples
			pxEvent->m_xPlayableList.Shrink(uIndex);
			pxEvent->m_xDelayList.Shrink(uIndex);
			pxEvent->m_xWeightsList.Shrink(uIndex);
			pxEvent->m_xStartMessageList.Shrink(uIndex);
			pxEvent->m_xEndMessageList.Shrink(uIndex);
		}
		#endif //#else ASURA_TOOLS
	}


	//load/skip tools data
	#ifdef ASURA_DIALOGUE_INCLUDE_TOOLS_DATA
	xStream >> pxEvent->m_xFilename;
	#else
	xStream.SkipString();
	#endif

	// Ensure a prime search is allocated if this is a cd random
	if ((pxEvent->m_ePlaybackType == ASURA_DPT_CDPLAYERRANDOM) && (pxEvent->GetNumPlayables() > 0))
	{
		pxEvent->m_pxPrimeSearch = new Asura_PrimeSearch(pxEvent->GetNumPlayables());
	}

	return true;
}

bool Asura_Chunk_Dialogue_Event::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

#ifdef ASURA_TOOLS
void Asura_Chunk_Dialogue_Event::Create(const class Asura_Dialogue_Event* pxEvent,Asura_Chunk_Stream& xStream)
{
	u_int uChunkFlags = 0;

	xStream.WriteChunk_Start(ASURA_CHUNK_DIALOGUE_EVENT, ASURA_CHUNK_DIALOGUE_EVENT_VERSION, uChunkFlags);

	xStream << pxEvent->m_xName;

	xStream << (int)pxEvent->m_ePlaybackType;
	xStream << pxEvent->m_fReplayDelayTime;
	xStream << pxEvent->m_ucPlayProbability;
	xStream << pxEvent->m_cPriority;
	xStream << pxEvent->m_bGlobalEvent;
	
	//message store
	u_int uNumMessages = pxEvent->m_xMessageStore.Size();
	xStream << uNumMessages;
	for(u_int i=0;i<uNumMessages;i++)
	{
		const Asura_Dialogue_Message* pxMessage = pxEvent->m_xMessageStore[i];
		xStream << pxMessage->m_uHashID;
		xStream << pxMessage->m_xName;
		pxMessage->m_xMessageBlock.WriteToChunkStream(xStream);
	}

	xStream << pxEvent->m_uInitialMessage;
	xStream << pxEvent->m_uFinalMessage;

	//playable list
	xStream << pxEvent->GetNumPlayableID();
	for(u_int i=0;i<pxEvent->GetNumPlayableID();i++)
	{
		const Asura_Dialogue_PlayableIDListItem& xItem = pxEvent->GetPlayableID(i);
		xStream << xItem.m_xID.m_uHashID;
		xStream << (int) xItem.m_xID.m_eType;
		xStream << xItem.m_fDelay;
		xStream << xItem.m_ucWeight;
		xStream << xItem.m_uStartMessageID;
		xStream << xItem.m_uEndMessageID;
		xStream << xItem.m_xName;
	}


	xStream << pxEvent->m_xFilename;

	xStream.WriteChunk_End();
}

void Asura_Chunk_Dialogue_Event::Create(const class Asura_Dialogue_Event* pxEvent,Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create	(pxEvent,xStream);
}
#endif // ASURA_TOOLS


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------**
** Asura_Chunk_Dialogue_Line_Template **
**------------------------------------*/

#define ASURA_CHUNK_DIALOGUE_LINE_TEMPLATE_VERSION 5
// Version 1 - Added no subtitle flag (RAR 17/02/2007)
// Version 2 - Added streaming flag (MH - 28.03.2007) 
// Version 3 - Added mixer preset (PJC - 03.06.08)
// Version 4 - Added compression quality (TY - 02.02.09 )
// Version 5 - Added DSP Chain hash ( TY - 08.05.09 )

bool Asura_Chunk_Dialogue_Line_Template::Process(Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?

	if (Version > ASURA_CHUNK_DIALOGUE_LINE_TEMPLATE_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for dialogue line template",Version));
		return false;
	}


	char acName[128];
	xStream.ReadString(acName,sizeof(acName));

	//Allocate or find the dialogue event
	bool bAlreadyExists;
	Asura_Dialogue_Line_Template* pxTemplate = Asura_Dialogue_Storage::CreateLineTemplate(acName, bAlreadyExists);
	Asura_Assert(pxTemplate, "");


	//make sure the template doesn't already exist
	if(bAlreadyExists)
	{
		#ifdef ASURA_TOOLS
		Asura_Assertf(0,("Dialogue line template %s already exists",acName));
		#endif
		//probably want to add some logging to file_contents.txt or similar to go along with this
		return false;
	}

	//get the mixer
	char acMixerName[128];
	xStream.ReadString(acMixerName,sizeof(acMixerName));

	#ifdef ASURA_DIALOGUE_INCLUDE_TOOLS_DATA
	pxTemplate->m_xMixingChannelName = acMixerName;
	#endif

	//Just store the hash of the mixer for now , since the mixers may not have been loaded yet.
	pxTemplate->m_uMixerHash = Asura_GetHashID(acMixerName);

	xStream >> pxTemplate->m_fGain;
	xStream >> pxTemplate->m_fInnerRange;
	xStream >> pxTemplate->m_fOuterRange;
	xStream >> pxTemplate->m_fMinGain;
	xStream >> pxTemplate->m_bVoiceover;
	xStream >> pxTemplate->m_bNoWavForPSP;
	if(Version >= 1)
	{
		xStream >> pxTemplate->m_bNoSubtitle;
	}
	if(Version >= 3)
	{
		xStream >> pxTemplate->m_uMixerPreset;
	}

	//load/skip tools data
	#ifdef ASURA_DIALOGUE_INCLUDE_TOOLS_DATA
	xStream >> pxTemplate->m_usSampleRateForPS2;
	xStream >> pxTemplate->m_usSampleRateForPSP;
	#else
	u_short usJunk;
	xStream >> usJunk;
	xStream >> usJunk;
	#endif
	
	if (Version >= 2)
	{

		xStream >> pxTemplate->m_bStreamed;
	}

	if( Version >= 4 )
	{
		#ifdef ASURA_DIALOGUE_INCLUDE_TOOLS_DATA
		xStream >> pxTemplate->m_usCompressionQualityPS3;
		xStream >> pxTemplate->m_usCompressionQuality360;
		#else
		unsigned short usJunk;
		xStream >> usJunk;
		xStream >> usJunk;
		#endif
	}
	#ifdef ASURA_DIALOGUE_INCLUDE_TOOLS_DATA
	else
	{
		pxTemplate->m_usCompressionQualityPS3 = TSS_QUALITY_MEDIUM;
		pxTemplate->m_usCompressionQuality360 = TSS_QUALITY_MEDIUM;
	}
	#endif

	if( Version >= 5 )
	{
		xStream >> pxTemplate->m_uDSPChain;
	}
	else
	{
		pxTemplate->m_uDSPChain = ASURA_HASH_ID_UNSET;
	}

	return true;
}

bool Asura_Chunk_Dialogue_Line_Template::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

#ifdef ASURA_TOOLS
void Asura_Chunk_Dialogue_Line_Template::Create(const class Asura_Dialogue_Line_Template* pxTemplate,Asura_Chunk_Stream& xStream)
{
	u_int uChunkFlags = 0;

	xStream.WriteChunk_Start(ASURA_CHUNK_DIALOGUE_LINE_TEMPLATE, ASURA_CHUNK_DIALOGUE_LINE_TEMPLATE_VERSION, uChunkFlags);

	xStream << pxTemplate->m_xName;

	xStream << pxTemplate->m_xMixingChannelName;
	xStream << pxTemplate->m_fGain;
	xStream << pxTemplate->m_fInnerRange;
	xStream << pxTemplate->m_fOuterRange;
	xStream << pxTemplate->m_fMinGain;
	xStream << pxTemplate->m_bVoiceover;
	xStream << pxTemplate->m_bNoWavForPSP;
	xStream << pxTemplate->m_bNoSubtitle;
	xStream << pxTemplate->m_uMixerPreset;
	
	//tools data
	xStream << pxTemplate->m_usSampleRateForPS2;
	xStream << pxTemplate->m_usSampleRateForPSP;
	xStream << pxTemplate->m_bStreamed;
	xStream << pxTemplate->m_usCompressionQualityPS3;
	xStream << pxTemplate->m_usCompressionQuality360;

	xStream << pxTemplate->m_uDSPChain;

	xStream.WriteChunk_End();
}

void Asura_Chunk_Dialogue_Line_Template::Create(const class Asura_Dialogue_Line_Template* pxTemplate,Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create	(pxTemplate,xStream);
}
#endif // ASURA_TOOLS

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Asura_Chunk_Dialogue_Line **
**---------------------------*/

#define ASURA_CHUNK_DIALOGUE_LINE_VERSION 3
//Version 1 - Added animation
//			RAR 15/01/2007
//Version 2 - Added sound length
//			RAR 01/03/2007
//Version 3 - Added "m_uFaceAnimationHash" (or blank)
//			ART 05/09/2008 (Moved here where it was meant to be by AJG 09/09/08)


bool Asura_Chunk_Dialogue_Line::Process(Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?

	if (Version > ASURA_CHUNK_DIALOGUE_LINE_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for dialogue line",Version));
		return false;
	}


	char acName[128];
	xStream.ReadString(acName,sizeof(acName));

	//Allocate or find the dialogue line
	bool bAlreadyExists;
	Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::CreateLine(acName, bAlreadyExists);
	Asura_Assert(pxLine, "");

	//make sure the line doesn't already exist
	if(bAlreadyExists)
	{
		#ifdef ASURA_TOOLS
		Asura_Assertf(0,("Dialogue line %s already exists",acName));
		#endif
		//probably want to add some logging to file_contents.txt or similar to go along with this
		return false;
	}

	Asura_Hash_ID uTemplateHash;

	xStream >> pxLine->m_xCharacterID;
	xStream >> uTemplateHash;
	xStream >> pxLine->m_uSoundHash;
	if(Version >= 3)
	{
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		xStream >> pxLine->m_uFaceAnimationHash;
#else
		Asura_Hash_ID	uFaceAnimationHash;

		xStream >> uFaceAnimationHash;
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	}
	if(Version >= 1)
	{
		xStream >> pxLine->m_uAnimation;
	}
	if(Version >= 2)
	{
		xStream >> pxLine->m_fSoundLength;
	}

	pxLine->m_pxTemplate = Asura_Dialogue_Storage::FindLineTemplate(uTemplateHash);

	//read in the text
	int iLength;
	xStream >> iLength;
	Asura_UnicodeChar* wszText = iLength ? new Asura_UnicodeChar[iLength] : 0;
	for(int i=0;i<iLength;i++)
	{
		xStream >> wszText[i];
	}
	pxLine->m_wszText = wszText;



	return true;
}

bool Asura_Chunk_Dialogue_Line::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

#ifdef ASURA_TOOLS
void Asura_Chunk_Dialogue_Line::Create(const class Asura_Dialogue_Line* pxLine, int iLanguage,Asura_Chunk_Stream& xStream)
{
	u_int uChunkFlags = 0;

	xStream.WriteChunk_Start(ASURA_CHUNK_DIALOGUE_LINE, ASURA_CHUNK_DIALOGUE_LINE_VERSION, uChunkFlags);

	xStream << pxLine->m_xName;

	xStream << pxLine->m_xCharacterID;
	xStream << Asura_GetHashID(pxLine->m_xTemplateName.GetString());
	xStream << pxLine->m_uSoundHash;
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	xStream << pxLine->m_uFaceAnimationHash;
#else
	xStream << ASURA_HASH_ID_UNSET;
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	xStream << pxLine->m_uAnimation;
	xStream << pxLine->m_fSoundLength;

	//get the text to save
	const Asura_UnicodeChar* wszText = pxLine->GetTextForLanguage(iLanguage);
	if(!wszText) wszText = pxLine->m_wszText;//Use the current (english) text if the localised text isn't available
	
	int iLength = wszText ? wcslen(wszText)+1 : 0;
	xStream << iLength;

	for(int i=0;i<iLength;i++)
	{
		xStream << wszText[i];
	}



	xStream.WriteChunk_End();
}

void Asura_Chunk_Dialogue_Line::Create(const class Asura_Dialogue_Line* pxLine,int iLanguage, Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create	(pxLine, iLanguage,xStream);
}
#endif // ASURA_TOOLS



/////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------------**
** Asura_Chunk_Dialogue_ToolsGlobalSettings **
**------------------------------------------*/

#define ASURA_CHUNK_DIALOGUE_TOOLSGLOBALSETTINGS_VERSION 0

#ifdef ASURA_TOOLS //Whole chunk is tools only

bool Asura_Chunk_Dialogue_ToolsGlobalSettings::Process(Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?

	if (Version > ASURA_CHUNK_DIALOGUE_TOOLSGLOBALSETTINGS_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for tools global dialogue settings",Version));
		return false;
	}

	xStream >> Tools_Dialogue_System::s_uDefaultSampleRatePS2;
	xStream >> Tools_Dialogue_System::s_uDefaultSampleRatePSP;

	return true;
}

bool Asura_Chunk_Dialogue_ToolsGlobalSettings::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

void Asura_Chunk_Dialogue_ToolsGlobalSettings::Create(Asura_Chunk_Stream& xStream)
{
	u_int uChunkFlags = 0;
	xStream.WriteChunk_Start(ASURA_CHUNK_DIALOGUE_TOOLSGLOBALSETTINGS, ASURA_CHUNK_DIALOGUE_TOOLSGLOBALSETTINGS_VERSION, uChunkFlags);

	xStream << Tools_Dialogue_System::s_uDefaultSampleRatePS2;
	xStream << Tools_Dialogue_System::s_uDefaultSampleRatePSP;


	xStream.WriteChunk_End();
}

void Asura_Chunk_Dialogue_ToolsGlobalSettings::Create(Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create(xStream);
}


#endif // ASURA_TOOLS


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Asura_Chunk_Dialogue_ToolsHashToPath **
**--------------------------------------*/

#define ASURA_CHUNK_DIALOGUE_TOOLSHASHTOPATH_VERSION 0

#ifdef ASURA_TOOLS //Whole chunk is tools only

bool Asura_Chunk_Dialogue_ToolsHashToPath::Process(Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?

	if (Version > ASURA_CHUNK_DIALOGUE_TOOLSHASHTOPATH_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for dialogue hash to path chunk",Version));
		return false;
	}

	Asura_TemplateHashTree<Tools_Dialogue_HashToPath>& xHashToPathTree = Tools_Dialogue_System::s_xHashToPathTree;
	xHashToPathTree.Reset();

	int iNumNodes;
	xStream >> iNumNodes;

	for(int i=0;i<iNumNodes;i++)
	{
		Asura_Hash_ID uHash;
		xStream >> uHash;

		Tools_Dialogue_HashToPath* pxEntry = xHashToPathTree.AddNode(uHash);
		if(pxEntry)
		{
			pxEntry->m_uHash = uHash;
			xStream >> pxEntry->m_xFilename;
		}
		else
		{
			Asura_Assert(0,"Error reading Asura_Chunk_Dialogue_ToolsHashToPath : Duplicate hash value");
			xStream.SkipString();
		}
		
	}

	return true;
}

bool Asura_Chunk_Dialogue_ToolsHashToPath::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}

void Asura_Chunk_Dialogue_ToolsHashToPath::Create(Asura_Chunk_Stream& xStream)
{
	u_int uChunkFlags = 0;
	xStream.WriteChunk_Start(ASURA_CHUNK_DIALOGUE_TOOLSHASHTOPATH, ASURA_CHUNK_DIALOGUE_TOOLSHASHTOPATH_VERSION, uChunkFlags);

	Asura_TemplateHashTree<Tools_Dialogue_HashToPath>& xHashToPathTree = Tools_Dialogue_System::s_xHashToPathTree;

	int iNumNodes = xHashToPathTree.GetNumberOfNodes();

	xStream << iNumNodes;

	for(Asura_TemplateHashTree_SaveIterator xIt(iNumNodes);!xIt.Done();xIt.Next())
	{
		const Tools_Dialogue_HashToPath* pxEntry = xHashToPathTree.GetDataByIndex(xIt.GetCurrent());

		xStream << pxEntry->m_uHash;
		xStream << pxEntry->m_xFilename;
	}

	xStream.WriteChunk_End();
}

void Asura_Chunk_Dialogue_ToolsHashToPath::Create(Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create(xStream);
}


#endif // ASURA_TOOLS



/////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------**
** Asura_Chunk_Dialogue_InGame **
**-----------------------------*/

#define ASURA_CHUNK_DIALOGUE_INGAME_VERSION 0

bool Asura_Chunk_Dialogue_InGame::Process(Asura_Chunk_Stream& xStream) const
{
	
	
	// Is this the correct version?

	if (Version > ASURA_CHUNK_DIALOGUE_INGAME_VERSION)
	{
		Asura_Assertf(false, ("Unknown chunk version %d for dialogue in-game state",Version));
		return false;
	}

	u_int uEvents;
	xStream >> uEvents;

	for(u_int i=0;i<uEvents;i++)
	{
		Asura_Hash_ID uEventID;
		u_int uLastPlayedItem;

		//extract the data for this event
		xStream >> uEventID;
		xStream >> uLastPlayedItem;

		//and update the event (if we can find it)
		//can get away with a const pointer since the data that needs updating is mutable anyway
		const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::FindEvent(uEventID);
		Asura_Assert(pxEvent,"Failed to find dialogue event referred to by Asura_Chunk_Dialogue_InGame");
		if(pxEvent)
		{
			pxEvent->m_uLastPlayedItem = uLastPlayedItem;
		}

	}


	return true;
}

bool Asura_Chunk_Dialogue_InGame::Process(const char* pcChunkBuffer) const
{
	Asura_Chunk_ChunkBuffer_Stream xStream(pcChunkBuffer);
	return Process(xStream);
}

void Asura_Chunk_Dialogue_InGame::Create(Asura_Chunk_Stream& xStream)
{
	
	//determine the number of events that we need to save the state for.
	u_int uEventsToSave = 0;

	for(Asura_ResourceSet_Iterator<Asura_Dialogue_Event> xIt(Asura_Dialogue_Storage::s_xEventContainer); !xIt.Done(); xIt.Next())
	{
		Asura_Dialogue_Event* pxEvent = xIt.GetCurrent();
		if(pxEvent->GetPlaybackType() == ASURA_DPT_ONEBYONE)
		{
			uEventsToSave++;
		}
	}


	u_int uChunkFlags = 0;
	xStream.WriteChunk_Start(ASURA_CHUNK_DIALOGUE_INGAME, ASURA_CHUNK_DIALOGUE_INGAME_VERSION, uChunkFlags);

	xStream << uEventsToSave;
	
	//traverse the event tree again , saving details for those events that require it
	for(Asura_ResourceSet_Iterator<Asura_Dialogue_Event> xIt(Asura_Dialogue_Storage::s_xEventContainer); !xIt.Done(); xIt.Next())
	{
		Asura_Dialogue_Event* pxEvent = xIt.GetCurrent();
		if(pxEvent->GetPlaybackType() == ASURA_DPT_ONEBYONE)
		{

			xStream << pxEvent->GetHashID();
			xStream << pxEvent->m_uLastPlayedItem;
		}

	}


	xStream.WriteChunk_End();

}

void Asura_Chunk_Dialogue_InGame::Create(Asura_SaveBlock_Base* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	Create(xStream);
}


bool Asura_Chunk_Dialogue_InGame::CountEventsToSaveCallback(Asura_Hash_ID uHash,Asura_Dialogue_Event* pxEvent,void* pCount)
{
	//just interested in 'one by one' events
	if(pxEvent->GetPlaybackType() == ASURA_DPT_ONEBYONE)
	{
		(*(u_int*)pCount)++;
	}
	return true;
}

bool Asura_Chunk_Dialogue_InGame::SaveEventCallback(Asura_Hash_ID uHash,Asura_Dialogue_Event* pxEvent,void* pStream)
{
	//just interested in 'one by one' events
	if(pxEvent->GetPlaybackType() == ASURA_DPT_ONEBYONE)
	{
		Asura_Chunk_Stream* pxStream = (Asura_Chunk_Stream*) pStream;

		(*pxStream) << pxEvent->GetHashID();
		(*pxStream) << pxEvent->m_uLastPlayedItem;
	}

	return true;
}



#endif //ASURA_USE_DIALOGUE_SYSTEM
