
//
// Asura_Chunk_DynamicLightTemplate.cpp
//
// Created by Dave Reed - 25/04/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Asura_Chunk_Loaders.h"

#ifdef ASURA_TARGET_SM3
#ifdef ASURA_TOOLS

#include "ChunkSystem/Asura_Chunk_DynamicLightTemplate.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Misc/Tools_Timed_Messages.h"


// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_SaveBlock;


// C L A S S E S /////////////////////////////////////////////////////////////

Asura_DynamicLight_Parameters Asura_Chunk_DynamicLightTemplate::s_xParams;


bool Asura_Chunk_DynamicLightTemplate::Process(Asura_Handle iFileHandle)
{
	Asura_Chunk_File_Stream	xStream(iFileHandle);

	u_int uVersion;
	xStream >> uVersion;
	Asura_Assert( uVersion <= Asura_ServerEntity_DynamicLight::DYNLIGHT_CHUNK_VERSION, "Unhandled Asura_ServerEntity_DynamicLight chunk version" );

	Asura_ServerEntity_DynamicLight::ExtractParamsFromChunkStream(s_xParams, xStream, uVersion );

	return false;
}



bool Asura_Chunk_DynamicLightTemplate::Create( const Asura_DynamicLight_Parameters &xParams, const char *szFileName)
{
	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	Asura_Assert(pxSaveBlock, "Unable to create saveblock");

	if( NULL == pxSaveBlock)
	{
		return( false);
	}

	Asura_Chunk_SaveBlock_Stream	xStream (pxSaveBlock);
	xStream.WriteChunk_Start(ASURA_CHUNK_DYNAMICLIGHTTEMPLATE, 1, 0);

	// We just use the existing serialize/deserialize functions from the entity
	xStream << Asura_ServerEntity_DynamicLight::DYNLIGHT_CHUNK_VERSION;
	Asura_ServerEntity_DynamicLight::WriteParamsToChunkStream(xParams, xStream);

	xStream.WriteChunk_End();
	xStream.WriteAsuraTerminator();
	
	bool bSuccess=pxSaveBlock->SaveFile(szFileName, false );

	if (!bSuccess)
	{	
		Tools_Timed_Message_System::AddMessage(0xff0000, "Failed to save '%s' - File is probably read-only?",szFileName);
	}
	else
	{	
		Tools_Timed_Message_System::AddMessage(0x00ff00, "Saved light template '%s'",szFileName);
	}

	delete pxSaveBlock;
	return bSuccess;
}


#endif // ASURA_TOOLS
#endif // ASURA_TARGET_SM3