// Asura_Chunk_DynamicMusic.cpp
// Thomas Young - 21.08.08
// Copyright (C) 2008 Rebellion, All Rights Reserved.

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_DynamicMusic.h"
#include "Sound/Asura_Dynamic_Music_System.h"

// D E F I N E S /////////////////////////////////////////////////////////////

#define ASURA_CHUNK_DYNAMICMUSIC_VERSION 1

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

// ------------------------ Asura_Chunk_DynamicMusic ------------------------------------

bool Asura_Chunk_DynamicMusic::Process(Asura_Handle iFileHandle)
{

	Asura_Chunk_File_Stream	xStream(iFileHandle);
	return Process(xStream);
}


bool Asura_Chunk_DynamicMusic::Process (Asura_Chunk_Stream& xStream)
{
	Asura_Assert(Version <= ASURA_CHUNK_DYNAMICMUSIC_VERSION,"Unknown chunk version");

	Asura_Dynamic_Music_System::ClearTracks ();

	int iNumLayers = 0;

	xStream >> iNumLayers;

	for( int i = 0; i < iNumLayers; ++i )
	{
		xStream >> Asura_Dynamic_Music_System::s_axLayerNames[i];
	}
	
	int iNumberOfTracks = 0;

	xStream >> iNumberOfTracks;

	for( int i = 0; i < iNumberOfTracks; i++ )
	{
		Asura_Dynamic_Music_TrackData* pxTrack = Asura_Dynamic_Music_System::CreateNewTrack();

		if( pxTrack )
		{
			pxTrack->ReadFromChunkStream( xStream );
		}
	}

	int iNumTriggerData = 0;

	xStream >> iNumTriggerData;

	for( int i = 0; i < iNumTriggerData; i++ )
	{
		Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::CreateNewTriggerData();

		if( pxData )
		{
			pxData->ReadFromChunkStream( xStream );
		}
	}	

	return true;
}

#ifdef ASURA_TOOLS
void Asura_Chunk_DynamicMusic::Create( Asura_SaveBlock* pxSaveBlock )
{
	u_int uChunkFlags = 0;

	Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
	
	xStream.WriteChunk_Start(ASURA_CHUNK_DYNAMICMUSIC, ASURA_CHUNK_DYNAMICMUSIC_VERSION, uChunkFlags);

	int iNumLayers = ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS;
	
	xStream << iNumLayers;
	
	for( int i = 0; i < iNumLayers; ++i )
	{
		xStream << Asura_Dynamic_Music_System::s_axLayerNames[i];
	}

	int iNumTracks = Asura_Dynamic_Music_System::GetNumberOfTracks();

	xStream << iNumTracks;

	for( int i = 0; i < iNumTracks; ++i )
	{
		Asura_Dynamic_Music_TrackData* pxTrack = Asura_Dynamic_Music_System::GetTrackByIndex( i );
		if( pxTrack )
		{
			pxTrack->WriteToChunkStream( xStream );
		}
	}

	int iNumTriggerData = Asura_Dynamic_Music_System::GetNumberOfTriggerData();

	xStream << iNumTriggerData;

	for( int i = 0; i < iNumTriggerData; ++i )
	{
		Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::GetTriggerDatabyIndex( i );
		if( pxData )
		{
			pxData->WriteToChunkStream( xStream );
		}
	}

	xStream.WriteChunk_End();
}
#endif

// ------------------------ Asura_Chunk_DynamicMusic_Ingame_State ------------------------------------

bool Asura_Chunk_DynamicMusic_Ingame_State::Process(const char* pcChunkBuffer)
{
	Asura_Chunk_ChunkBuffer_Stream xStream(pcChunkBuffer);
	return Process(xStream);
}


bool Asura_Chunk_DynamicMusic_Ingame_State::Process(Asura_Chunk_Stream& xStream)
{
	Asura_Dynamic_Music_System::ReadStateFromChunkStream( xStream );
	return true;
}


void Asura_Chunk_DynamicMusic_Ingame_State::Create(Asura_SaveBlock_Base& xSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream(&xSaveBlock);

	xStream.WriteChunk_Start(ASURA_CHUNK_DYNAMICMUSIC_INGAME, 0, 0);
	
	Asura_Dynamic_Music_System::WriteStateToChunkStream ( xStream );
	
	xStream.WriteChunk_End	();
}

#endif //#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

//eof
