//
// Asura_Chunk_EnvironmentalLightTransport.cpp
//
// Created by KJFL - 2004-04-26 15:32:34
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_TOOLS // this chunk is only used in Tools builds

// This file's header.
#include "ChunkSystem/Asura_Chunk_EnvironmentalLightTransport.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_ModuleLinking.h"//for module index conversion
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Stream_WindowsFile.h"
#include "Core/Memory/Asura_Mem.h"
#include "Core/Asura_SortAndSearch.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Lighting/Tools_EnvironmentalLightTransport.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Module.h"

#include "Load&Save/Tools_Zlib.h"


struct Asura_Old_LightTransportModule
{
	u_int m_uNumberOfVertices;
	u_int m_uStartIndex;
	int m_iModuleIndex;
};

int SortLightTransportModule(const void* p1, const void* p2)
{
	const Asura_Old_LightTransportModule* pxModule1 = (const Asura_Old_LightTransportModule*)p1;
	const Asura_Old_LightTransportModule* pxModule2 = (const Asura_Old_LightTransportModule*)p2;
	if(pxModule1->m_uStartIndex < pxModule2->m_uStartIndex) return -1;
	if(pxModule1->m_uStartIndex > pxModule2->m_uStartIndex) return 1;
	return 0;
}


bool Asura_Chunk_EnvironmentalLightTransport::Process( Asura_Handle hFile ) const
{	
	Tools_EnvironmentalLightTransport_Handler& xHandler = Tools_EnvironmentalLightTransport::GetCurrentHandler();
	
	// first check the validity of the chunk 

	// Deja vu?
	if ( xHandler.IsEnvironmentProcessed() )
	{
		Asura_Assert( false, "ELT data loaded twice?");
		return false;
	}
	xHandler.m_bValidData = false;
	
	// Too new?
    if( Version > CHUNK_VERSION )
    {
        Asura_Assert( false, "Chunk version is too new. You need a new executable." );
        return false;
    }
	
	// Too old?
    if( Version < CHUNK_VERSION ) 
    {
        Asura_OutputToDebugger(( "INFO: Loading chunk v%i (current version is v%i.)\n", Version, CHUNK_VERSION ));
		// don't need to exit but we may need to touch up older data
    }

	Asura_Chunk_File_Stream xStream(hFile);


	// allocate space for per-module data
	if (m_uNumberOfModules ) // do we have any saved module data
	{
		//check number of modules with the module index conversion thing
		Asura_ModuleIndex_Conversion::CheckNumberOfModules(m_uNumberOfModules,Asura_Module_System::GetNumberOfModules());
		
		//allocate based on the current number of modules , rather than the saved data
		xHandler.m_uNumberOfModules = Asura_Module_System::GetNumberOfModules();
		xHandler.m_pxModules = new Tools_EnvironmentalLightTransport_Module[xHandler.m_uNumberOfModules];
	
		if(Version >= 2)
		{
			for(u_int i=0;i<m_uNumberOfModules;i++)
			{
				u_int uNumberOfVertices;
				xStream >> uNumberOfVertices;
				//determine the module index to load this one into
				int iModuleIndex = Asura_ModuleIndex_Conversion::ConvertIndex(i);
				if(iModuleIndex>=0)
				{
					Tools_EnvironmentalLightTransport_Module* pxModule = &xHandler.m_pxModules[iModuleIndex];
					pxModule->m_uNumberOfVertices = uNumberOfVertices;

					pxModule->m_pxLightTransportSHs = new Asura_SphericalHarmonics_Order3[uNumberOfVertices];
                    if( uNumberOfVertices )
                    {
					    xStream.Read(uNumberOfVertices*sizeof(Asura_SphericalHarmonics_Order3), pxModule->m_pxLightTransportSHs);
                    }
				}
				else
				{
                    if( uNumberOfVertices )
                    {
					    xStream.Skip(uNumberOfVertices*sizeof(Asura_SphericalHarmonics_Order3));
                    }
				}
			}
		}
		else
		{
			Asura_Old_LightTransportModule* pxOldModules = new Asura_Old_LightTransportModule[m_uNumberOfModules];
			
			for(u_int i=0;i<m_uNumberOfModules;i++)
			{
				//determine the module index to load this one into
				pxOldModules[i].m_iModuleIndex = Asura_ModuleIndex_Conversion::ConvertIndex(i);
				xStream >> pxOldModules[i].m_uNumberOfVertices;
				xStream >> pxOldModules[i].m_uStartIndex;
			}

			//sort by start index
			Asura_SortAndSearch::Quicksort(pxOldModules, m_uNumberOfModules, SortLightTransportModule);

			//read in the SHs for each module
			u_int uSHIndex = 0;
			
			for(u_int i=0;i<m_uNumberOfModules;i++)
			{
				Asura_Assert(uSHIndex == pxOldModules[i].m_uStartIndex, "Error converting old ELT data");
				
				//fill in this module if it is being used
				if(pxOldModules[i].m_iModuleIndex >= 0)
				{
					Tools_EnvironmentalLightTransport_Module* pxModule = &xHandler.m_pxModules[pxOldModules[i].m_iModuleIndex];

					pxModule->m_uNumberOfVertices = pxOldModules[i].m_uNumberOfVertices;
					pxModule->m_pxLightTransportSHs = new Asura_SphericalHarmonics_Order3[pxModule->m_uNumberOfVertices];

					xStream.Read(sizeof(Asura_SphericalHarmonics_Order3) * pxModule->m_uNumberOfVertices, pxModule->m_pxLightTransportSHs);
					uSHIndex += pxModule->m_uNumberOfVertices;
				}
				else
				{
					//skip past any unused SHs
					xStream.Skip(pxOldModules[i].m_uNumberOfVertices*sizeof(Asura_SphericalHarmonics_Order3));
					uSHIndex += pxOldModules[i].m_uNumberOfVertices;
				}
				
			}

			delete [] pxOldModules;

		}
	}
	else // no modules - set the data to zero values
	{
		xHandler.m_uNumberOfModules = 0;
		xHandler.m_pxModules = 0;
	}

	if(Version >= 1)
	{
		xStream >> Tools_EnvironmentalLightTransport_Handler::s_uDirectLightingExtraSharpenFactor;
		xStream >> Tools_EnvironmentalLightTransport_Handler::s_fDirectLightingExtraIntensityFactor;
		xStream >> Tools_EnvironmentalLightTransport_Handler::s_fInterreflectedScaleFactor;
	}

	// success

	xHandler.m_bValidData = !(Flags & ChunkFlag_Invalid);
	xHandler.m_bQuickData = (Flags & ChunkFlag_QuickData)!=0;
	return true;

}

bool Asura_Chunk_EnvironmentalLightTransport::Create( Asura_SaveBlock& xSaveBlock )
{
	Asura_Chunk_SaveBlock_Stream xStream(&xSaveBlock);
	return Create(xStream);
}

bool Asura_Chunk_EnvironmentalLightTransport::Create( Asura_Chunk_Stream& xStream )
{
	Tools_EnvironmentalLightTransport_Handler& xHandler = Tools_EnvironmentalLightTransport::GetCurrentHandler();

	u_int uChunkFlags = 0;

	if(!xHandler.m_bValidData)
	{
		uChunkFlags |= ChunkFlag_Invalid;
	}
	if(xHandler.m_bQuickData)
	{
		uChunkFlags |= ChunkFlag_QuickData;
	}

	xStream.WriteChunk_Start(ASURA_CHUNK_ENVIRONMENTALLIGHTTRANSPORT, CHUNK_VERSION, uChunkFlags);

  	xStream << xHandler.m_uNumberOfModules;
	xStream << (u_int)0;

	const Tools_EnvironmentalLightTransport_Module* pxModule = xHandler.m_pxModules;
	for(u_int i=0; i<xHandler.m_uNumberOfModules; i++, pxModule++)
	{
		xStream << pxModule->m_uNumberOfVertices;

		u_int uPerVertexDataSize = sizeof(Asura_SphericalHarmonics_Order3) * pxModule->m_uNumberOfVertices;
		xStream.Write(uPerVertexDataSize, pxModule->m_pxLightTransportSHs);
	}

	xStream << Tools_EnvironmentalLightTransport_Handler::s_uDirectLightingExtraSharpenFactor;
	xStream << Tools_EnvironmentalLightTransport_Handler::s_fDirectLightingExtraIntensityFactor;
	xStream << Tools_EnvironmentalLightTransport_Handler::s_fInterreflectedScaleFactor;

	xStream.WriteChunk_End();

    return true;
}

bool Asura_Chunk_EnvironmentalLightTransport::SaveFile( const char* szFilename )
{
	Asura_Chunk_Stream_WindowsFile xStream;
	if(!xStream.CreateFile(szFilename, true))
	{
		return false;
	}

	Asura_Chunk_ModuleNames::Create( xStream );

    if( !Create(xStream) )
	{
		xStream.DeleteFile();
		return false;
	}

	if(xStream.CloseFile())
	{
		#ifdef ASURA_WRITE_ZLIB
		Tools_Zlib::CompressFile(szFilename);
		#endif
		return true;
	}
	else
	{
		return false;
	}
}

#endif // #ifdef ASURA_TOOLS
