
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Chunk_FacialAnimation.cpp
// Author : Andy Tate
// Date   : 29-Aug-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "ChunkSystem/Asura_Chunk_FacialAnimation.h"

// Asura independent header-files...
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "FacialAnimation/Asura_FacialAnimation_System.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

const u_int	Asura_Chunk_FacialAnimation_Face::s_uChunkVersion			= 1;
const u_int	Asura_Chunk_FacialAnimation_FaceAnimation::s_uChunkVersion	= 1;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Chunk_FacialAnimation_Face::Process
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Asura_Chunk_FacialAnimation_Face::Process(const Asura_Handle iFileHandle)
{
	switch(Version)
	{
		case 1:
		{
			Asura_Chunk_File_Stream		xStream(iFileHandle);
			Asura_Hash_ID				uHash;

			xStream >> uHash;
			Asura_FacialAnimation_System::CreateFace(uHash, xStream);
			return(true);
		}

		default:
		{
			Asura_Assert(false, "Unexpected chunk-version found for this Tools facial-animation face.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Chunk_FacialAnimation_Face::Process'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Chunk_FacialAnimation_Face::Create
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Chunk_FacialAnimation_Face::Create(const Asura_Hash_ID uHash,
											  const Asura_FacialAnimation_Face& xFace,
											  Asura_Chunk_Stream& xStream)
{
	xStream.WriteChunk_Start(ASURA_CHUNK_FACIALANIMATION_FACE, s_uChunkVersion, 0);
	{
		xStream << uHash;
		xFace.WriteToChunkStream(xStream);
	}
	xStream.WriteChunk_End();
}

/// End of function 'Asura_Chunk_FacialAnimation_Face::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Chunk_FacialAnimation_FaceAnimation::Process
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Asura_Chunk_FacialAnimation_FaceAnimation::Process(const Asura_Handle iFileHandle)
{
	switch(Version)
	{
		case 1:
		{
			Asura_Chunk_File_Stream		xStream(iFileHandle);
			Asura_Hash_ID				uHash;

			xStream >> uHash;
			Asura_FacialAnimation_System::CreateFaceAnimation(uHash, xStream);
			return(true);
		}

		default:
		{
			Asura_Assert(false, "Unexpected chunk-version found for this Tools facial-animation face-animation.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Chunk_FacialAnimation_FaceAnimation::Process'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Chunk_FacialAnimation_FaceAnimation::Create
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Chunk_FacialAnimation_FaceAnimation::Create(const Asura_Hash_ID uHash,
													   const Asura_FacialAnimation_FaceAnimation& xFaceAnimation,
													   Asura_Chunk_Stream& xStream)
{
	xStream.WriteChunk_Start(ASURA_CHUNK_FACIALANIMATION_FACEANIMATION, s_uChunkVersion, 0);
	{
		xStream << uHash;
		xFaceAnimation.WriteToChunkStream(xStream);
	}
	xStream.WriteChunk_End();
}

/// End of function 'Asura_Chunk_FacialAnimation_FaceAnimation::Create'. ///

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
