//
// Asura_Chunk_FileInfo.cpp
//
// Created by Richard Rice - 07/12/05
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header
#include "ChunkSystem/Asura_Chunk_FileInfo.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"

#ifdef ASURA_MAIN
#include "Core/Asura_LocalisedText.h"

#include <string.h>
#endif

#if defined( ASURA_TOOLS)
AsuraFileType Asura_Chunk_FileInfo::s_eLastLoadedFileType = AsuraFile_Unknown;
#endif


static const int ASURA_CHUNK_FILEINFO_VERSION = 1;
bool Asura_Chunk_FileInfo::Process(Asura_Handle iFileHandle) const
{
	Asura_Assert(Version<=ASURA_CHUNK_FILEINFO_VERSION,"Unknown chunk version");

	#ifdef ASURA_MAIN
	if(Flags & ChunkFlag_HasLocalisedFile)
	{
		//There is a localised file to go along with this one. Load that first
		//RAR 14/03/2007
		const char* szFilename = Asura_FileAccess::GetFilename(iFileHandle);
		Asura_Assert(szFilename,"");

		char acLocalisedFilename[256];
		Asura_Assert(strlen(szFilename) < sizeof(acLocalisedFilename),"");
		strcpy(acLocalisedFilename,szFilename);

		//replace the extension by the two letter code for the language
		char* pcExtension = strrchr(acLocalisedFilename,'.');
		if(pcExtension)
		{
			pcExtension++;	
			strcpy(pcExtension, Asura_LocalisedText::GetTwoCharCodeForCurLang());

			#ifdef ASURA_PLATFORM_IS_BIG_ENDIAN
			pcExtension += 2;
			strcpy(pcExtension,ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX);
			#endif

			//Load this file , as long as its different from the one we are already loading
			if(strcmp(acLocalisedFilename,szFilename))
			{
				LoadAsuraFile(acLocalisedFilename);
			}
		}
	}
	#endif //ASURA_MAIN

	u_int uFileSize;
	ASURA_CHUNK_FILEEXTRACT(iFileHandle, uFileSize, u_int);
	//Only using the file size if its non-zero. This should prevent all the other files that now
	//have file info chunks from screwing up the loading bar. RAR 07/12/2005
	#ifdef ASURA_MAIN
	if(Asura_ChunkLoading_System::GetRecursionLevel() == 0 && uFileSize)
	{
    	Asura_ChunkLoading_System::SetFileSize(uFileSize);
	}
	#endif
	//Skip the file type if it exists ,since its not used by this loader
	if(Version==1)
	{
#if defined( ASURA_TOOLS)
		//and get the file type
		int iFileType = AsuraFile_Unknown;
		Asura_FileAccess::ReadData( iFileHandle, &iFileType, sizeof(int));
		
		s_eLastLoadedFileType = static_cast< AsuraFileType>( iFileType);
#else
		Asura_FileAccess::SkipData( iFileHandle,sizeof(int));
#endif
	}

    return true;
}

#ifdef ASURA_TOOLS
void Asura_Chunk_FileInfo::Create(class Asura_SaveBlock* pxSaveBlock, u_int uFileSize, AsuraFileType eFileType, bool bHasLocalisedFile)
{
	char* pcBuffer = reinterpret_cast<char*>(pxSaveBlock->GetAccessPointer(GetChunkSize()));
    Create(pcBuffer, uFileSize, eFileType, bHasLocalisedFile);
}

void Asura_Chunk_FileInfo::Create(char* pcBuffer, u_int uFileSize, AsuraFileType eFileType, bool bHasLocalisedFile)
{
	Asura_Chunk_Memory_Stream	xStream (GetChunkSize(), pcBuffer);

	Create(xStream, uFileSize, eFileType, bHasLocalisedFile);
}

void Asura_Chunk_FileInfo::Create(Asura_Chunk_Stream& xStream, u_int uFileSize, AsuraFileType eFileType, bool bHasLocalisedFile)
{

	// Write the data...
	u_int uFlags = 0;
	if(bHasLocalisedFile) uFlags |= ChunkFlag_HasLocalisedFile;

	xStream.WriteChunk_Start	(ASURA_CHUNK_FILEINFO, ASURA_CHUNK_FILEINFO_VERSION, uFlags);

	xStream << uFileSize;
	xStream << (int)eFileType;

	xStream.WriteChunk_End	();
}

#endif

int Asura_Chunk_FileInfo::GetChunkSize()
{
	int iChunkSize = sizeof(Asura_Chunk_FileInfo);
    return iChunkSize + sizeof(u_int) + sizeof(int);
}

