/**
*
* \file Asura_Chunk_Font.cpp
*
* Created by James Paulin - 22/07/2002 12:32:46
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

// This file's header.
#include "ChunkSystem/Asura_Chunk_Font.h"

// Engine independent.
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Core/Memory/Asura_Mem.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Rendering/Asura_Font_System.h"
#include "core/Asura_TextureManagement.h"

#ifdef _FONTMAKER

void Asura_Chunk_Font::Create (const Asura_Font* pxFont, Asura_Chunk_Stream& xStream)
{
	// Write the data...

	xStream.WriteChunk_Start	(ASURA_CHUNK_FONT, 3, 0);
	xStream.Write				(sizeof(char) * ASURA_FONT_MAX_FILENAME, pxFont->m_szFileName);

	xStream << pxFont->m_fFontHeight;
	xStream << pxFont->m_uNumGlyphPages;


	// Write the glyph pages in-turn...

	for (u_int uPage = 0; uPage < pxFont->m_uNumGlyphPages; uPage++)
	{
        const Asura_Font_GlyphPage&	xThisPage		= pxFont->m_pxGlyphPages[uPage];
		const int					iGlyphPointer	= 0;


		// Write the data for this page...

		xStream.Write	(sizeof(char) * ASURA_FONT_MAX_FILENAME, xThisPage.m_acTextureFileName);

		xStream << xThisPage.m_iFontTextureIndex;

		xStream << iGlyphPointer;	// this pointer will be filled out when size of array is known

        xStream << xThisPage.m_wcRangeMin;
        xStream << xThisPage.m_wcRangeMax;
        xStream << xThisPage.m_uTexWidth;
        xStream << xThisPage.m_uTexHeight;
		xStream << xThisPage.m_uNumChars;


		// Write each glyph within this page in-turn...

        for (u_int uGlyph = 0; uGlyph < xThisPage.m_uNumChars; uGlyph++)
        {
            const Asura_Font_Glyph&	xThisGlyph	= xThisPage.m_pxGlyphArray [uGlyph];


			// Write the data for this glyph...

			xStream << xThisGlyph.u;
			xStream << xThisGlyph.v;
            xStream << xThisGlyph.width;
			xStream << xThisGlyph.wcThisChar;
            xStream << xThisGlyph.usKerningIndex;
        }
	}

	xStream.WriteChunk_End	();
}

void Asura_Chunk_Font::Create( const Asura_Font* pxFont, Asura_SaveBlock* saveBlock )
{
#if 0
    // create an Asura_Chunk_Font from a Asura_Font  

    // calculate the true chunk size

	u_int uChunkSize = sizeof(Asura_Chunk_Font) + sizeof(char) * ASURA_FONT_MAX_FILENAME + sizeof(float) + sizeof(u_int);

    for (u_int uGlyphPage = 0; uGlyphPage < pxFont->m_uNumGlyphPages; uGlyphPage++)
    {
		uChunkSize += sizeof(Asura_Font_GlyphPage) +
			(sizeof(Asura_Font_Glyph_V2) + sizeof(u_int)) * 
			(pxFont->m_pxGlyphPages[uGlyphPage].m_uNumChars);
    }

	Asura_Chunk_Font* pxChunk = (Asura_Chunk_Font*)saveBlock->GetAccessPointer(uChunkSize);

	//Fill chunk header
	pxChunk->ID = ASURA_CHUNK_FONT;
    ASURA_CHUNK_VERIFY(pxChunk->ID, int);
	pxChunk->Size = uChunkSize;
	ASURA_CHUNK_VERIFY(pxChunk->Size, int);
	pxChunk->Version = 3;
    ASURA_CHUNK_VERIFY(pxChunk->Version, int);
	pxChunk->Flags = 0;
    ASURA_CHUNK_VERIFY(pxChunk->Flags, int);

    // write the other stuff...

    char* pcDest = reinterpret_cast<char*>(pxChunk + 1);
    Asura_Mem::Copy(pcDest, pxFont->m_szFileName, sizeof(char) * ASURA_FONT_MAX_FILENAME);
    pcDest += ASURA_FONT_MAX_FILENAME;

    ASURA_CHUNK_FILLANDVERIFY(pcDest, pxFont->m_fFontHeight, float);

    ASURA_CHUNK_FILLANDVERIFY(pcDest, pxFont->m_uNumGlyphPages, u_int);

	//Write glyph pages
	for (u_int uPage = 0; uPage < pxFont->m_uNumGlyphPages; uPage++)
	{
        Asura_Font_GlyphPage& xThisPage = pxFont->m_pxGlyphPages[uPage];

        Asura_Mem::Copy(pcDest, xThisPage.m_acTextureFileName, sizeof(char) * ASURA_FONT_MAX_FILENAME);
        pcDest += ASURA_FONT_MAX_FILENAME;

        ASURA_CHUNK_FILLANDVERIFY(pcDest, xThisPage.m_iFontTextureIndex, int);
        ASURA_CHUNK_FILLANDVERIFY(pcDest, 0, Asura_Font_Glyph*);    // this pointer will be filled out when size of array is known
        

        ASURA_CHUNK_FILLANDVERIFY(pcDest, xThisPage.m_wcRangeMin, Asura_UnicodeChar);
        ASURA_CHUNK_FILLANDVERIFY(pcDest, xThisPage.m_wcRangeMax, Asura_UnicodeChar);
        ASURA_CHUNK_FILLANDVERIFY(pcDest, xThisPage.m_uTexWidth, u_int);
        ASURA_CHUNK_FILLANDVERIFY(pcDest, xThisPage.m_uTexHeight, u_int);
		ASURA_CHUNK_FILLANDVERIFY(pcDest, xThisPage.m_uNumChars, u_int);

        for (u_int uGlyph = 0; uGlyph < xThisPage.m_uNumChars; uGlyph++)
        {
            const Asura_Font_Glyph& xThisGlyph = xThisPage.m_pxGlyphArray[uGlyph];
		    ASURA_CHUNK_FILLANDVERIFY( pcDest, xThisGlyph.u, float);
            ASURA_CHUNK_FILLANDVERIFY( pcDest, xThisGlyph.v, float);
            ASURA_CHUNK_FILLANDVERIFY( pcDest, xThisGlyph.width, float);
			ASURA_CHUNK_FILLANDVERIFY( pcDest, xThisGlyph.wcThisChar, u_int);
        }
	}
	u_int uRealChunkSize = static_cast<u_int>(pcDest - reinterpret_cast<char*>(pxChunk));
    Asura_Assert(static_cast<u_int>(pcDest - reinterpret_cast<char*>(pxChunk)) == uChunkSize, "Unexpected chunk size");
#else
	Asura_Chunk_SaveBlock_Stream	xStream (saveBlock);


	// Call the other version of this function...

	Create	(pxFont, xStream);
#endif
}

#else // not def _FONTMAKER

bool Asura_Chunk_Font::Process(Asura_Handle fileHandle) const
{
	//Create a temporary font structure to load data into
    // (MP - 15.10.2005) Actually, this is a permanent structure that gets
    // stored by the font system.
	Asura_Font* pxNewFont = new Asura_Font();

    // copy the data into the new font
    switch(Version)
    {

// Don't bother supporting ancient chunk versions outside of tools.
#ifdef ASURA_TOOLS

    case 2:
        {
            char acTextureFileName[ASURA_FONT_MAX_FILENAME];
            Asura_FileAccess::ReadData( fileHandle, acTextureFileName, sizeof(acTextureFileName) );

            // MC: I don't like this either, for what it's worth
            // (MP - 07.04.2006) I've fixed this so that it can't overrun
            // the buffer but obviously it could now get truncated instead
            // if the font name is very long.
            Asura_PrintfToBfr( pxNewFont->m_szFileName, "\\fonts\\%s", ASURA_FONT_MAX_FILENAME, true, acTextureFileName );

			// Change xThisPage.m_acTextureFileName to have no extension.

			int iStrLen;

			for(iStrLen = 0; ; iStrLen++)
			{
				if(pxNewFont->m_szFileName[iStrLen] == '.') break;
				if(pxNewFont->m_szFileName[iStrLen] == 0) break;	// Failsafe
			}

			// Drop the extension.
			pxNewFont->m_szFileName[iStrLen] = 0;
				

            Asura_FileAccess::ReadData(fileHandle, &(pxNewFont->m_fFontHeight), sizeof(float));
            Asura_FileAccess::ReadData(fileHandle, &(pxNewFont->m_uNumGlyphPages), sizeof(u_int));
            pxNewFont->m_pxGlyphPages = new Asura_Font_GlyphPage[pxNewFont->m_uNumGlyphPages];
            if (!pxNewFont->m_pxGlyphPages) 
            {
                Asura_Assert(false, "out of memory");
                delete pxNewFont;
                return false;
            }

            // read the glyph pages
            for (u_int uPage = 0; uPage < pxNewFont->m_uNumGlyphPages; uPage++)
	        {
				Asura_Font_GlyphPage& xThisPage = pxNewFont->m_pxGlyphPages[uPage];
				
				#if !defined( _WIN64)
					ASURA_STATIC_ASSERT( 0x78 == sizeof( Asura_Font_GlyphPage_V2));
				#else
					ASURA_STATIC_ASSERT( 0x80 == sizeof( Asura_Font_GlyphPage_V2));
				#endif
				
				//Asura_FileAccess::ReadData(fileHandle, &xThisPage, sizeof(Asura_Font_GlyphPage_V2));

				Asura_FileAccess::ReadData(fileHandle, &xThisPage.m_acTextureFileName, sizeof( xThisPage.m_acTextureFileName));
				Asura_FileAccess::ReadData(fileHandle, &xThisPage.m_iFontTextureIndex, sizeof( xThisPage.m_iFontTextureIndex));

				u_int upGlyphArray;
				Asura_FileAccess::ReadData(fileHandle, &upGlyphArray, sizeof( upGlyphArray));
				xThisPage.m_pxGlyphArray = NULL;

				Asura_FileAccess::ReadData(fileHandle, &xThisPage.m_wcRangeMin, sizeof( xThisPage.m_wcRangeMin));
				Asura_FileAccess::ReadData(fileHandle, &xThisPage.m_wcRangeMax, sizeof( xThisPage.m_wcRangeMax));

				Asura_FileAccess::ReadData(fileHandle, &xThisPage.m_uTexWidth, sizeof( xThisPage.m_uTexWidth));
				Asura_FileAccess::ReadData(fileHandle, &xThisPage.m_uTexHeight, sizeof( xThisPage.m_uTexHeight));

				Asura_Assert( -1 <= xThisPage.m_iFontTextureIndex, "Bad Font Index");
				Asura_Assert( xThisPage.m_wcRangeMin <= xThisPage.m_wcRangeMax, "Bad Range Values");
				Asura_Assert( 0 < xThisPage.m_uTexWidth, "Texture Too big");
				Asura_Assert( 0 < xThisPage.m_uTexHeight, "Texture Too big");
				Asura_Assert( xThisPage.m_uTexWidth <= 4096, "Texture Too small");
				Asura_Assert( xThisPage.m_uTexHeight <= 4096, "Texture Too small");

#ifdef ASURA_MAIN
                //Changed this so that it attempts to get index of existing texture first. This is to avoid the new assert
				//about textures being added multiple times with differing extensions. RAR 07/03/2006
				xThisPage.m_iFontTextureIndex = Asura_Texture_Management::GetTextureIndex(xThisPage.m_acTextureFileName);
				if(xThisPage.m_iFontTextureIndex == ASURA_TEXTURE_NONE)
#endif
				{
					xThisPage.m_iFontTextureIndex = Asura_Texture_Management::AddTexture(xThisPage.m_acTextureFileName);
				}

                xThisPage.m_uNumChars = xThisPage.m_wcRangeMax - xThisPage.m_wcRangeMin + 1;

                xThisPage.m_pxGlyphArray = new Asura_Font_Glyph[xThisPage.m_uNumChars];

                if (!xThisPage.m_pxGlyphArray)
                {
                    Asura_Assert(false, "out of memory");
                    delete pxNewFont;
                    return false;
                }
                
                // read the glyphs belonging to this page
                for (u_int uGlyph = 0; uGlyph < xThisPage.m_uNumChars; uGlyph++)
                {
		            Asura_FileAccess::ReadData( fileHandle, xThisPage.m_pxGlyphArray + uGlyph, sizeof(Asura_Font_Glyph_V2) );
					xThisPage.m_pxGlyphArray[uGlyph].wcThisChar =
                        static_cast< Asura_UnicodeChar >( pxNewFont->m_pxGlyphPages->m_wcRangeMin + uGlyph );
                }
	        }
        }
        break;

#endif // #ifdef ASURA_TOOLS

	case 3:
        {
			Asura_Chunk_File_Stream	xStream (fileHandle);

            char acTextureFileName[ASURA_FONT_MAX_FILENAME];
            xStream.Read( sizeof(acTextureFileName), acTextureFileName );

            // MC: I don't like this either, for what it's worth
            // (MP - 07.04.2006) I've fixed this so that it can't overrun
            // the buffer but obviously it could now get truncated instead
            // if the font name is very long.
            Asura_PrintfToBfr( pxNewFont->m_szFileName, "\\fonts\\%s", ASURA_FONT_MAX_FILENAME, true, acTextureFileName );

			// Change xThisPage.m_acTextureFileName to have no extension.

			int iStrLen;

			for(iStrLen = 0; ; iStrLen++)
			{
				if(pxNewFont->m_szFileName[iStrLen] == '.') break;
				if(pxNewFont->m_szFileName[iStrLen] == 0) break;	// Failsafe
			}

			// Drop the extension.
			pxNewFont->m_szFileName[iStrLen] = 0;
				

            xStream >> pxNewFont->m_fFontHeight;
            xStream >> pxNewFont->m_uNumGlyphPages;
            pxNewFont->m_pxGlyphPages = new Asura_Font_GlyphPage[pxNewFont->m_uNumGlyphPages];
            if (!pxNewFont->m_pxGlyphPages) 
            {
                Asura_Assert(false, "out of memory");
                delete pxNewFont;
                return false;
            }

            // read the glyph pages
            for (u_int uPage = 0; uPage < pxNewFont->m_uNumGlyphPages; uPage++)
	        {
                Asura_Font_GlyphPage&	xThisPage	= pxNewFont->m_pxGlyphPages[uPage];

				int						iNastyPointerPad;

				// Read this glyph-page...
				xStream.Read	(sizeof(xThisPage.m_acTextureFileName), xThisPage.m_acTextureFileName);

				xStream >> xThisPage.m_iFontTextureIndex;
				xStream >> iNastyPointerPad;
				xStream >> xThisPage.m_wcRangeMin;
				xStream >> xThisPage.m_wcRangeMax;
				xStream >> xThisPage.m_uTexWidth;
				xStream >> xThisPage.m_uTexHeight;
				xStream >> xThisPage.m_uNumChars;

#ifdef ASURA_MAIN
                //Changed this so that it attempts to get index of existing texture first. This is to avoid the new assert
				//about textures being added multiple times with differing extensions. RAR 07/03/2006
				xThisPage.m_iFontTextureIndex = Asura_Texture_Management::GetTextureIndex(xThisPage.m_acTextureFileName);
				if(xThisPage.m_iFontTextureIndex == ASURA_TEXTURE_NONE)
#endif
				{
					xThisPage.m_iFontTextureIndex = Asura_Texture_Management::AddTexture(xThisPage.m_acTextureFileName);
				}
				
                xThisPage.m_pxGlyphArray = new Asura_Font_Glyph[xThisPage.m_uNumChars];

                if (!xThisPage.m_pxGlyphArray)
                {
                    Asura_Assert(false, "out of memory");
                    delete pxNewFont;
                    return false;
                }
                
                // read the glyphs belonging to this page
                xStream.Read( xThisPage.m_uNumChars * sizeof(Asura_Font_Glyph), xThisPage.m_pxGlyphArray );
            }
        }
        break;
    default:
        {
            Asura_Assert(false, "Unsupported version number");
            return false;
        }
    }

	//Create the new font
	if( !Asura_Font_System::StoreFontDetails(pxNewFont) )
	{
		Asura_Assert(0,"ERROR - Could not process new font");
        delete pxNewFont;
	}

	return true;
}

#endif // else not def _FONTMAKER

//
// Asura_Chunk_Font_Kerning.
//

#ifdef _FONTMAKER

void Asura_Chunk_Font_Kerning::Create( const Asura_Font_KerningData* pxKerningData, u_int uNumEntries, Asura_Chunk_Stream& xStream )
{
    xStream.WriteChunk_Start( ASURA_CHUNK_FONT_KERNING, VERSION, 0 );

    // We require int alignment so round the count up if necessary, and just
    // write a garbage entry at the end that will be ignored.
    if( uNumEntries & 1 ) { ++uNumEntries; }

    xStream << uNumEntries;

    while( uNumEntries-- )
    {
        xStream << (pxKerningData++)->usCount; // Doesn't particularly matter which individual union member we write so use one the stream code already supports.
    }

    xStream.WriteChunk_End();
}

#else // not def _FONTMAKER

bool Asura_Chunk_Font_Kerning::Process( Asura_Handle iFileHandle ) const
{
    Asura_Assert( Version == VERSION, "Unsupported version number" );

    Asura_Chunk_File_Stream	xStream( iFileHandle );

    u_int uNumEntries;
    xStream >> uNumEntries;

    Asura_Font_KerningData* pxKerningData = new Asura_Font_KerningData[uNumEntries];
    xStream.Read( uNumEntries * sizeof(Asura_Font_KerningData), pxKerningData );

    if( !Asura_Font_System::StoreKerningData(pxKerningData, uNumEntries) )
    {
        delete[] pxKerningData;
    }

    return true;
}

#endif // else not def _FONTMAKER
