//
// Asura_Chunk_Loaders.cpp
//
// Created by Kevin Lea - 12/06/2001 11:18:56
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"

// KJFL - 2004-07-20 - NOTE:
//
// Please add new Chunk headers to Asura_Chunk_Definitions.h
//

#include "ChunkSystem/Asura_BackgroundChunkProcessing_System.h"
#include "ChunkSystem/Asura_Chunk_Definitions.h"
#include "ChunkSystem/Asura_Chunk_DynamicMusic.h"
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "ChunkSystem/Asura_Chunk_FacialAnimation.h"
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_RagdollDescription.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_Huffman.h"
#include "ChunkSystem/Asura_ProcessChunkTimingList.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#ifdef ASURA_TOOLS
#include "ChunkSystem/Tools_Chunk_Skybox.h"
#endif
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Menus/Asura_ProgressBar.h"
//#include "Physics/Asura_Physics_Ragdoll_Description.h"
#include "Rendering/Asura_Viewport.h"

#ifdef ASURA_TOOLS
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "ChunkSystem/Tools_Chunk_FacialAnimation.h"
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "Load&Save/Tools_LoadedFileList.h"
#include "ChunkSystem/Asura_Chunk_ModuleLinking.h"
#endif

// G L O B A L S /////////////////////////////////////////////////////////////

DEBUGVAR bool dbg_bLogSucessfulAsuraFileLoads = false;
DEBUGVAR bool dbg_bLogFailedAsuraFileLoads = true;
DEBUGVAR bool dbg_bPrintSuccessfulAsuraFileLoads = false;
DEBUGVAR bool dbg_bPrintFailedAsuraFileLoads = true;

#if defined(ASURA_MEMORYMANAGEMENT_FILL)
DEBUGVAR bool	dbg_bCheckGuardBytesForEveryChunk	= false;
#endif	//defined(ASURA_MEMORYMANAGEMENT_FILL)

#if defined(ASURA_LOGFILES_ENABLED)
static const char* FILELOADING_LOGFILE = "logfiles\\loads.txt";
#endif	// defined(ASURA_LOGFILES_ENABLED)


// C O N S T A N T S /////////////////////////////////////////////////////////

// S T A T I C   V A R I A B L E S ///////////////////////////////////////////
#if !defined(ASURA_NO_DEBUG_OUTPUT)
bool	Asura_ChunkLoading_System::s_bDisplayDebugInfo	= false;
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)


// D E F I N E S /////////////////////////////////////////////////////////////

// (MP - 13.09.2002) The slightly strange format of this case is to prevent
// Lint complaining about the empty statement between cases caused by the
// CHUNK_PROCESS() calls in the switch() statement below having semi-colons.
#define CHUNK_PROCESS(chunkID, chunkClass)	\
case (chunkID):	\
{	\
	Asura_Assert( sizeof(chunkClass) <= uASURA_MAX_CHUNKHEADER_SIZE, "chunk header too big" ); \
	xStream.ReadChunk_GetNextHeader(pcChunkBuffer, sizeof(chunkClass)); \
	class chunkClass *chunk2 = (chunkClass *)pxChunkHeader;	\
	bChunkReadCompletely = chunk2->Process(iHandle);	\
}	\
break

#define SKIP_KNOWN_CHUNK( chunk_id ) \
    case (chunk_id): bReadInChunkHeader = false; break

// G L O B A L   F U N C T I O N S ///////////////////////////////////////////
static ASURA_CHUNK_PARSER_FUNCTION ProjectSpecificChunkParser = 0;
void SetProjectSpecificChunkParser(ASURA_CHUNK_PARSER_FUNCTION function)
{
	ProjectSpecificChunkParser = function;
}

bool LoadAsuraFile(const char* szName)
{
    return Asura_ChunkLoading_System::LoadAsuraFile(szName);
}
    
    
// C L A S S E S /////////////////////////////////////////////////////////////
#ifdef ASURA_TOOLS
Asura_Chunk_Header::Asura_Chunk_Header(enum ASURA_CHUNKID id, int s, int v, int f)
{
	ID = id;
	ASURA_CHUNK_VERIFY(ID, int);
	Size = s;
	ASURA_CHUNK_VERIFY(Size, int);
	Version = v;
	ASURA_CHUNK_VERIFY(Version, int);
	Flags = f;
	ASURA_CHUNK_VERIFY(Flags, int);
}
#endif

bool Asura_Chunk_Padding::Process(Asura_Handle iFileHandle)
{
	Asura_FileAccess::SkipData(iFileHandle, Size - sizeof(Asura_Chunk_Padding));
	return true;
}

#ifdef ASURA_TOOLS
void Asura_Chunk_Padding::Create(Asura_SaveBlock* pxSaveBlock, int iNumBytes)
{
	Asura_Assert(iNumBytes > 0, "Zero sized padding?");

	int iChunkSize = sizeof(Asura_Chunk_Padding) + iNumBytes;
	Asura_Chunk_Padding* pxChunk = (Asura_Chunk_Padding*)pxSaveBlock->GetAccessPointer(iChunkSize);

    //char* const pcDataStart = reinterpret_cast<char*>( pxChunk );

	pxChunk->ID = ASURA_CHUNK_PADDING;
	ASURA_CHUNK_VERIFY(pxChunk->ID, int);
	pxChunk->Size = iChunkSize;
	ASURA_CHUNK_VERIFY(pxChunk->Size, int);
	pxChunk->Version = 0; 
	ASURA_CHUNK_VERIFY(pxChunk->Version, int);
	pxChunk->Flags = 0;
	ASURA_CHUNK_VERIFY(pxChunk->Flags, int);

	char* pcChunkBuffer = (char*)(pxChunk+1);
	Asura_Assert(iNumBytes < 16, "Do you really want to pad more than a QW");
	// Fill with junk.
	while (iNumBytes--)
	{
		ASURA_CHUNK_FILL(pcChunkBuffer, 0xCD, u_char);
	}

}
#endif


Asura_ProgressDisplay* Asura_ChunkLoading_System::s_pxProgressDisplay = NULL;
float Asura_ChunkLoading_System::s_fLoadingBarInitialValue;
float Asura_ChunkLoading_System::s_fLoadingBarEndValue;
void (*Asura_ChunkLoading_System::s_pfnRender)();  // function to call at each render step (if NULL, clears the screen)
float Asura_ChunkLoading_System::s_fUncompressedFileSize;
int Asura_ChunkLoading_System::s_iRecursionLevel = -1;
ASURA_CHUNKID Asura_ChunkLoading_System::s_eCurrentChunkID;
bool Asura_ChunkLoading_System::s_bShouldTerminate = false;
int Asura_ChunkLoading_System::s_iAllowTerminationCounter = 0;

#ifdef ASURA_TOOLS
bool (*Asura_ChunkLoading_System::s_pfnChunkLoadFilter)(enum ASURA_CHUNKID eChunkID) = 0;
#endif

void Asura_ChunkLoading_System::UseLoadingBar( Asura_ProgressDisplay* pxProgressDisplay, float fEndValue, void (*pfnRender)() )
{
    s_pxProgressDisplay = pxProgressDisplay;
    s_fLoadingBarInitialValue = pxProgressDisplay->GetProgress();
    Asura_Assert(s_fLoadingBarInitialValue >= 0.0f && s_fLoadingBarInitialValue <= 1.0f, "invalid initial value");
    Asura_Assert(fEndValue >= 0.0f && fEndValue <= 1.0f, "invalid end value");
    s_fLoadingBarEndValue = fEndValue;
    s_pfnRender = pfnRender;
}

void Asura_ChunkLoading_System::KillLoadingBar()
{
    s_pxProgressDisplay = NULL;
    s_pfnRender = NULL;
}


bool Asura_ChunkLoading_System::LoadAsuraFile(const char* szName, u_char* pucChecksumOutput /* = NULL */)
{
	if( !szName ) return false;

	#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	char szFilename[260];
	Asura_FileAccess::PlatformGetNetworkFilenameOrGetLocalCopy(szName,szFilename,260);
	szName = szFilename;
	#endif //ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	#if defined( ASURA_PC_ALLOW_LOAD_FROM_SEARCHPATHS)
		// DLH - 15 September 2006 15:44:29 - I would like this to be MAX_PATH but it is in an independent file
		const u_int uFilenameSize = 260;
		char szFilename[ uFilenameSize + 1] = { 0};
		Asura_FileAccess::PlatformSearchPathsGetExistingFilename( szName, szFilename, uFilenameSize);
		szName = szFilename;
	#endif

	s_iRecursionLevel++;
	if (s_pxProgressDisplay && s_iRecursionLevel==0) s_fLoadingBarInitialValue = s_pxProgressDisplay->GetProgress();

	bool bComputeChecksum = pucChecksumOutput != NULL;
	Asura_Handle iHandle = Asura_FileAccess::Open(szName, bComputeChecksum);

	if (iHandle==ASURA_INVALID_HANDLE) 
	{
		if (dbg_bLogFailedAsuraFileLoads)
		{
			Asura_Logfile_Retail( (FILELOADING_LOGFILE, "failed to load %s\n", szName) );
		}
        if (dbg_bPrintFailedAsuraFileLoads)
        {
            Asura_OutputToDebugger(("failed to load %s\n", szName));
        }
		s_iRecursionLevel--;
		return false;
	}
	else 
    {
        if (dbg_bLogSucessfulAsuraFileLoads)
	    {
		    Asura_Logfile( (FILELOADING_LOGFILE, "loaded %s\n", szName) );
	    }
        if (dbg_bPrintSuccessfulAsuraFileLoads)
        {
            Asura_OutputToDebugger(("loaded %s\n", szName));
        }
    }

	#ifdef ASURA_USE_TEXTURE_LOG
	if(s_iRecursionLevel==0)
	{
		//store filename to help with logging what loads what texture. RAR 07/03/2006
		Asura_Texture_Management::SetCurrentAsuraFile(szName);
	}
	#endif
	
#if !defined(ASURA_NO_DEBUG_OUTPUT)
	if(s_bDisplayDebugInfo)
	{
		Asura_OutputToDebugger(("\nLoading Asura file \"%s\"...\n", szName));
	}
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)

	bool bOkay = LoadAsuraFileMain(iHandle);

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	if(s_bDisplayDebugInfo)
	{
		if(bOkay)
		{
			Asura_OutputToDebugger(("...succeeded.\n\n"));
		}
		else
		{
			Asura_OutputToDebugger(("...failed.\n\n"));
		}
	}
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)


	#ifdef ASURA_USE_TEXTURE_LOG
	if(s_iRecursionLevel==0)
	{
		//force texture load now , so that errors are given at a more useful stage. RAR 07/03/2006
		Asura_Texture_Management::LoadTextures();
		Asura_Texture_Management::SetCurrentAsuraFile(0);
	}
	#endif


	Asura_FileAccess::Close( iHandle, pucChecksumOutput );
	s_iRecursionLevel--;
	return bOkay;
}

bool Asura_ChunkLoading_System::LoadAsuraFile(Asura_Handle iFileHandle)
{
	s_iRecursionLevel++;
	if (s_pxProgressDisplay && s_iRecursionLevel==0) s_fLoadingBarInitialValue = s_pxProgressDisplay->GetProgress();

	bool bOkay = LoadAsuraFileMain(iFileHandle);
	s_iRecursionLevel--;

	return bOkay;
}

bool Asura_ChunkLoading_System::ProcessAsuraFileHeader(Asura_Chunk_File_Stream& xStream)
{
	const Asura_Handle	iHandle	= xStream.GetFileHandle();


#if ASURA_LOG_FILE_CONTENTS
	static bool s_bCleared = false;
	if(!s_bCleared)
	{
		s_bCleared = true;
		Asura_SimpleLogfileClear(ASURA_FILE_CONTENTS_LOGFILE);
	}
	Asura_SimpleLogfileOutput((ASURA_FILE_CONTENTS_LOGFILE,"\n\n%s\n",Asura_FileAccess::GetFilename(iHandle)));
#endif


	if(Asura_FileAccess::ProcessAsuraFileHeader(iHandle))
	{
#if defined(ASURA_CHUNK_LOADERS_ALLOW_DEBUG_TIMING)
			// Reset the timing info...
			Asura_ProcessChunkTimingList::Reset();
#endif	// defined(ASURA_CHUNK_LOADERS_ALLOW_DEBUG_TIMING)
			return(true);
	}

	return false;
}

bool Asura_ChunkLoading_System::ProcessChunk(Asura_Chunk_File_Stream& xStream)
{
	// (MH - 12.04.2006) this is used to read in the chunk IDs and the headers
	// so it needs to be at least as big as the largest header, and suitably 
	// aligned. 512 bytes has done us Ok for many years but the alignment isn't
	// guaranteed to be more than 1-byte if the type is plain char.
	// This method of creating an instance of a large header and accessing it via
	// a char* breaks strict aliasing rules, but I can't use a union as Asura_ChunkHeader
	// has a constructor.
	Asura_ChunkBuffer xChunkBuffer;
	char* pcChunkBuffer = reinterpret_cast<char*>(&xChunkBuffer);
	int iHandle = xStream.GetFileHandle();

	// Read the next chunk...
	if(xStream.ReadChunk_GetNextID(pcChunkBuffer))
	{
		// KOR - note that only the chunk ID is valid at this point
		Asura_Chunk_Header*		pxChunkHeader	= reinterpret_cast<Asura_Chunk_Header*>( pcChunkBuffer );

		// Store the current chunk-ID...
		s_eCurrentChunkID		= pxChunkHeader->ID;

		//bChunkReadCompletely takes the return value from the Process function. Used to determine
		//whether to assert if it turns out that the file position hasn't been advanded to the end of the chunk
		//RAR 08/11/02
		bool bChunkReadCompletely = false;
		bool bReadInChunkHeader   = true;
#ifdef ASURA_TOOLS
		if(s_pfnChunkLoadFilter && !s_pfnChunkLoadFilter(s_eCurrentChunkID))
		{
			//skip this chunk
			bReadInChunkHeader = false;
		}
		else
#endif
		{
#if !defined(ASURA_NO_DEBUG_OUTPUT)
			if(s_bDisplayDebugInfo)
			{
				Asura_OutputToDebugger(("- Chunk \"%c%c%c%c\", size = %d bytes\n", pxChunkHeader->TextID[0], pxChunkHeader->TextID[1], pxChunkHeader->TextID[2], pxChunkHeader->TextID[3], pxChunkHeader->Size));
			}
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)

			switch(s_eCurrentChunkID)
			{ 
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_MESH_VERTICES,Asura_Chunk_Mesh_Vertices);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_POLYGONS,Asura_Chunk_Mesh_Polygons);
#ifdef ASURA_USE_SAVED_MESH_COLOURS
				CHUNK_PROCESS(ASURA_CHUNK_MESH_COLOURS,Asura_Chunk_Mesh_Colours);
#endif // ASURA_USE_SAVED_MESH_COLOURS
#ifdef ASURA_DISABLE_PAINTED_COLOURS
				SKIP_KNOWN_CHUNK( ASURA_CHUNK_MESH_PAINTEDCOLOURS );
#else // ASURA_DISABLE_PAINTED_COLOURS
				CHUNK_PROCESS(ASURA_CHUNK_MESH_PAINTEDCOLOURS,Asura_Chunk_Mesh_PaintedColours);
#endif // ASURA_DISABLE_PAINTED_COLOURS
				CHUNK_PROCESS(ASURA_CHUNK_MESH_NORMALS,Asura_Chunk_Mesh_Normals);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_TEXTUREINFO,Asura_Chunk_Mesh_TextureInfo);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_SMOOTHINGGROUP,Asura_Chunk_Mesh_SmoothingGroup);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_POLYGONFLAGS,Asura_Chunk_Mesh_PolygonFlags);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_LOD,Asura_Chunk_Mesh_LOD);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_MISC,Asura_Chunk_Mesh_Misc);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_COLLMESHGENDATA,Asura_Chunk_Mesh_CollMeshGenData);
#ifndef ASURAIMPORT_EXPORTS
				CHUNK_PROCESS(ASURA_CHUNK_ENTITY_NEXTGUID,Asura_Chunk_Entity_NextGuid);
				CHUNK_PROCESS( ASURA_CHUNK_ENVIRONMENTALLIGHTTRANSPORT, Asura_Chunk_EnvironmentalLightTransport );
				CHUNK_PROCESS( ASURA_CHUNK_GLOBALILLUMINATION, Asura_Chunk_GlobalIllumination );
				CHUNK_PROCESS( ASURA_CHUNK_POLYGONLIGHTS, Asura_Chunk_PolygonLights );
#endif // ASURAIMPORT_EXPORTS
#else // ASURA_TOOLS
				SKIP_KNOWN_CHUNK( ASURA_CHUNK_ENTITY_NEXTGUID );
#endif // ASURA_TOOLS

#ifndef ASURAIMPORT_EXPORTS
				CHUNK_PROCESS(ASURA_CHUNK_ENVIRONMENT_LBAABBTREES, Asura_Chunk_EnvironmentLBAABBTrees);
#endif // ASURAIMPORT_EXPORTS
				CHUNK_PROCESS(ASURA_CHUNK_ENVIRONMENT_MODULELIST,Asura_Chunk_Environment_ModuleList);
				CHUNK_PROCESS(ASURA_CHUNK_TEXTURENAMES,Asura_Chunk_TextureNames);
				CHUNK_PROCESS(ASURA_CHUNK_TEXTUREFLAGS,Asura_Chunk_TextureFlags);
				CHUNK_PROCESS(ASURA_CHUNK_MATERIAL,Asura_Chunk_Material);
				CHUNK_PROCESS(ASURA_CHUNK_MATERIALNAMES,Asura_Chunk_MaterialNames);

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(TOOLS_CHUNK_MATERIAL_RESPONSE, Tools_Chunk_MaterialResponse);
				CHUNK_PROCESS(TOOLS_CHUNK_MATERIAL_RESPONSE_NAMES, Tools_Chunk_MaterialResponseNames);
#ifndef ASURAIMPORT_EXPORTS
				CHUNK_PROCESS(TOOLS_CHUNK_MATERIAL_RESPONSE_OVERRIDES, Tools_Chunk_MaterialResponseOverrides);
				CHUNK_PROCESS(TOOLS_CHUNK_PFX_OVERRIDES, Tools_Chunk_PFXOverrides);
#endif
#endif // ASURA_TOOLS
#ifndef ASURAIMPORT_EXPORTS
				CHUNK_PROCESS(ASURA_CHUNK_MATERIAL_RESPONSE, Asura_Chunk_MaterialResponse );
				CHUNK_PROCESS(ASURA_CHUNK_MATERIAL_RESPONSE_OVERRIDES, Asura_Chunk_MaterialResponseOverrides);
#endif
#else
				SKIP_KNOWN_CHUNK(TOOLS_CHUNK_MATERIAL_RESPONSE_NAMES);
#endif // ASURA_USE_MATERIAL_RESPONSE_SYSTEM

				CHUNK_PROCESS(ASURA_CHUNK_TEXTUREANIMATION,Asura_Chunk_TextureAnimation);


				CHUNK_PROCESS(ASURA_CHUNK_STARTPOSITION,Asura_Chunk_StartPosition);

				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_SKIN,Asura_Chunk_Hierarchy_Skin);
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_SKINLOD,Asura_Chunk_Hierarchy_SkinLOD);
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_MARKERPOINTS,Asura_Chunk_Hierarchy_MarkerPoints);
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_ANIM,Asura_Chunk_Hierarchy_Anim);
				CHUNK_PROCESS(ASURA_CHUNK_SHAPEDATA, Asura_Chunk_ShapeData);

#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_MANUALSECTIONBBS, Asura_Chunk_Hierarchy_ManualSectionBBs );
				CHUNK_PROCESS(TOOLS_CHUNK_HELPEROBJECT, Tools_Chunk_HelperObject );
#endif //ASURA_TOOLS

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
				CHUNK_PROCESS(ASURA_CHUNK_FACIALANIMATION_FACE, Asura_Chunk_FacialAnimation_Face);
				CHUNK_PROCESS(ASURA_CHUNK_FACIALANIMATION_FACEANIMATION, Asura_Chunk_FacialAnimation_FaceAnimation);
#if defined(ASURA_TOOLS)
				CHUNK_PROCESS(TOOLS_CHUNK_FACIALANIMATION_FACE, Tools_Chunk_FacialAnimation_Face);
				CHUNK_PROCESS(TOOLS_CHUNK_FACIALANIMATION_FACEANIMATION, Tools_Chunk_FacialAnimation_FaceAnimation);
				CHUNK_PROCESS(TOOLS_CHUNK_FACIALANIMATION_FACEPOSE, Tools_Chunk_FacialAnimation_FacePose);
				CHUNK_PROCESS(TOOLS_CHUNK_FACIALANIMATION_FACETYPE, Tools_Chunk_FacialAnimation_FaceType);
#endif	//defined(ASURA_TOOLS)
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#ifndef ASURAIMPORT_EXPORTS

				CHUNK_PROCESS(ASURA_CHUNK_PHYSICSENVIRONMENT, Asura_Chunk_PhysicsEnvironment );

				CHUNK_PROCESS(ASURA_CHUNK_TILEDTEXTURE,Asura_Chunk_TiledTexture);
				CHUNK_PROCESS(ASURA_CHUNK_SHAPE,Asura_Chunk_Shape);
				CHUNK_PROCESS(ASURA_CHUNK_FRAGMENT,Asura_Chunk_Fragment);

				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_COMPRESSEDANIM,Asura_Chunk_Hierarchy_CompressedAnim);
				CHUNK_PROCESS(ASURA_CHUNK_GI_SOLUTION, Asura_Chunk_GI_Solution );
				CHUNK_PROCESS(ASURA_CHUNK_LIGHTS,Asura_Chunk_Lights);
				CHUNK_PROCESS(ASURA_CHUNK_CORONA,Asura_Chunk_Corona);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_LIGHTSCALE,Asura_Chunk_LightScale);
				CHUNK_PROCESS(TOOLS_CHUNK_TOOLSLIGHTS, Tools_Chunk_ToolsLights);
#endif // ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_SECTIONBBS, Asura_Chunk_Hierarchy_SectionBBs );
#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
				CHUNK_PROCESS(ASURA_CHUNK_REVERBVOLUMES,Asura_Chunk_ReverbVolumes);
#endif				

				CHUNK_PROCESS(ASURA_CHUNK_SOUNDEVENT_SAMPLE,Asura_Chunk_SoundEvent_Sample);
				CHUNK_PROCESS(ASURA_CHUNK_SOUNDEVENT_EVENT,Asura_Chunk_SoundEvent_Event);
				CHUNK_PROCESS(ASURA_CHUNK_SOUNDEVENT_MAPPING,Asura_Chunk_SoundEvent_Mapping);
				CHUNK_PROCESS(ASURA_CHUNK_SOUNDEVENT_PHONON,Asura_Chunk_SoundEvent_Phonon);
				CHUNK_PROCESS(ASURA_CHUNK_SOUNDEVENT_MIXER,Asura_Chunk_SoundEvent_Mixer);
				CHUNK_PROCESS(ASURA_CHUNK_SOUNDEVENT_GLOBALSETTINGS,Asura_Chunk_SoundEvent_GlobalSettings);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_SOUNDEVENT_TOOLSGLOBALSETTINGS,Asura_Chunk_SoundEvent_ToolsGlobalSettings);
#endif //ASURA_TOOLS

#ifdef ASURA_USE_DIALOGUE_SYSTEM
				CHUNK_PROCESS(ASURA_CHUNK_DIALOGUE_EVENT,Asura_Chunk_Dialogue_Event);
				CHUNK_PROCESS(ASURA_CHUNK_DIALOGUE_LINE,Asura_Chunk_Dialogue_Line);
				CHUNK_PROCESS(ASURA_CHUNK_DIALOGUE_LINE_TEMPLATE,Asura_Chunk_Dialogue_Line_Template);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_DIALOGUE_TOOLSGLOBALSETTINGS,Asura_Chunk_Dialogue_ToolsGlobalSettings);
				CHUNK_PROCESS(ASURA_CHUNK_DIALOGUE_TOOLSHASHTOPATH,Asura_Chunk_Dialogue_ToolsHashToPath);
#endif //ASURA_TOOLS
#endif //ASURA_USE_DIALOGUE_SYSTEM

#ifdef ASURA_USE_ARTICULATION_SYSTEM
				CHUNK_PROCESS(ASURA_CHUNK_LIPSYNC_ARTICULATION,				Asura_Chunk_LipSync);
#endif

#if ( defined ASURA_TOOLS ) && !( defined ASURA_USE_AUDIO_SYSTEM )
				CHUNK_PROCESS(ASURA_CHUNK_SOUNDNAMES,Asura_Chunk_SoundNames);
#endif
				CHUNK_PROCESS(ASURA_CHUNK_STREAMINGBACKGROUNDSOUND,Asura_Chunk_StreamingBackgroundSound);
#if !defined(ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM)
				CHUNK_PROCESS(ASURA_CHUNK_METAMUSIC,Asura_Chunk_MetaMusic);
#endif	//!defined(ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM)
				CHUNK_PROCESS(ASURA_CHUNK_MODULEREVERB,Asura_Chunk_ModuleReverb);
				CHUNK_PROCESS(ASURA_CHUNK_LIGHTVISIBILITIES,Asura_Chunk_LightVisibilities);

				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_SOUNDS,Asura_Chunk_Hierarchy_Sounds);
#if defined(ASURA_SUPPORT_BODYPART_SYSTEM)
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHYBODYPART,Asura_Chunk_HierarchyBodyPart);
				CHUNK_PROCESS(ASURA_CHUNK_DEATHANIMFILENAMES,Asura_Chunk_DeathAnimFilenames);
#endif	// defined(ASURA_SUPPORT_BODYPART_SYSTEM)

				//
				CHUNK_PROCESS(ASURA_CHUNK_SKYBOX,Asura_Chunk_SkyBox);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(TOOLS_CHUNK_SKYBOX,Tools_Chunk_SkyBox);
#endif
#if defined( ASURA_TARGET_SM3)
				CHUNK_PROCESS(ASURA_CHUNK_SKYBOX_SHADER,Asura_Chunk_SkyBox_Shader);
#endif

				CHUNK_PROCESS(ASURA_CHUNK_NAVIGATION,Asura_Chunk_Navigation);
#ifndef ASURAIMPORT_EXPORTS
				CHUNK_PROCESS(ASURA_CHUNK_WAYPOINTINGSETTINGS, Asura_Chunk_WaypointingSettings);
#endif

				CHUNK_PROCESS(ASURA_CHUNK_RESOURCEFILE,Asura_Chunk_ResourceFile);
				CHUNK_PROCESS(ASURA_CHUNK_RESOURCEFILELIST, Asura_Chunk_ResourceFileList);

				CHUNK_PROCESS(ASURA_CHUNK_CUTSCENE,Asura_Chunk_Cutscene);
				CHUNK_PROCESS(ASURA_CHUNK_CUTSCENEACTOR,Asura_Chunk_Actor);
				CHUNK_PROCESS(ASURA_CHUNK_CUTSCENETRACK,Asura_Chunk_CutsceneTrack);
				CHUNK_PROCESS(ASURA_CHUNK_CUTSCENEEVENT,Asura_Chunk_CutsceneEvent);
				CHUNK_PROCESS(ASURA_CHUNK_CUTSCENEATTACHMENT,Asura_Chunk_CutsceneAttachment);

				CHUNK_PROCESS(ASURA_CHUNK_MODULELINKING,Asura_Chunk_ModuleLinking);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_MODULENAMES,Asura_Chunk_ModuleNames);
#endif // ASURA_TOOLS
#if defined(ASURA_USE_STREAMING) && defined(ASURA_TOOLS)
				CHUNK_PROCESS(ASURA_CHUNK_STREAMINGMODULENAMES,Asura_Chunk_StreamingModuleNames);
#endif // #if defined(ASURA_USE_STREAMING) && defined(ASURA_TOOLS)

				CHUNK_PROCESS(ASURA_CHUNK_ENTITY,Asura_Chunk_Entity);
				CHUNK_PROCESS(ASURA_CHUNK_ENTITY_TOOLSDATA,Asura_Chunk_Entity_ToolsData);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_ENTITY_SECONDARYFILE,Asura_Chunk_Entity_SecondaryFile);
#endif // ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_STATICMESSAGES,Asura_Chunk_StaticMessages);
				CHUNK_PROCESS(ASURA_CHUNK_ANIMATIONTABLE,Asura_Chunk_AnimationTable);
#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
				CHUNK_PROCESS(ASURA_CHUNK_EXPORTEDANIMATIONTABLE,Asura_Chunk_ExportedAnimationTable);
#endif // !ASURA_DISABLE_ANIM_TABLE_CODE
				CHUNK_PROCESS(ASURA_CHUNK_FONT,Asura_Chunk_Font);
				CHUNK_PROCESS(ASURA_CHUNK_FONT_KERNING,Asura_Chunk_Font_Kerning);
				CHUNK_PROCESS(ASURA_CHUNK_LOCALTEXT,Asura_Chunk_LocalText);
				CHUNK_PROCESS(ASURA_CHUNK_HASHEDLOCALISEDTEXT,Asura_Chunk_HashedLocalisedText);
				CHUNK_PROCESS(ASURA_CHUNK_LOCALTEXT_PAGE,Asura_Chunk_LocalText_Page);
#ifdef ASURA_USE_BILLBOARD_PLANTS
				CHUNK_PROCESS(ASURA_CHUNK_BILLBOARDPLANTS_TEMPLATES, Asura_Chunk_BillboardPlant_Template);
				CHUNK_PROCESS(ASURA_CHUNK_BILLBOARDPLANTS_BATCHES, Asura_Chunk_BillboardPlant_Batches);
#endif // ASURA_USE_BILLBOARD_PLANTS
#ifdef ASURA_USE_BOTANICALS
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_BOTANICALS_TEMPLATES,	Asura_Chunk_Botanicals_Templates);
				CHUNK_PROCESS(ASURA_CHUNK_BOTANICALS_INSTANCES,	Asura_Chunk_Botanicals_Instances);
				CHUNK_PROCESS(ASURA_CHUNK_BOTANICALS_DEFINITION,Asura_Chunk_Botanicals_Definition);
				CHUNK_PROCESS(ASURA_CHUNK_BOTANICALS_STYLES,	Asura_Chunk_Botanicals_Styles);
#endif // ASURA_TOOLS
#endif // ASURA_USE_BOTANICALS
#ifdef ASURA_USE_CARPET
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_CARPET_STYLES,		Asura_Chunk_Carpet_Styles);
				CHUNK_PROCESS(ASURA_CHUNK_CARPET_MESH,			Asura_Chunk_Carpet_Mesh);
#endif // ifdef ASURA_TOOLS
#endif // ASURA_USE_CARPET
#ifdef ASURA_USE_FSFX
				CHUNK_PROCESS(ASURA_CHUNK_FSFX, Asura_Chunk_FSFX );
#ifdef ASURA_TOOLS
#ifndef ASURAIMPORT_EXPORTS
				CHUNK_PROCESS(TOOLS_CHUNK_FSFX_OVERRIDES, Tools_Chunk_FSFXOverrides);
#endif
#endif
				CHUNK_PROCESS(ASURA_CHUNK_FSFX_OVERRIDES, Asura_Chunk_FSFXOverrides);

#endif // ASURA_USE_FSFX
#ifdef ASURA_USE_ATTRACTORS
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_TOOLS_ATTRACTORS, Asura_Chunk_Tools_Attractors );
#endif // ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_ATTRACTORS, Asura_Chunk_Attractors );
#endif // ASURA_USE_ATTRACTORS
#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
				CHUNK_PROCESS(ASURA_CHUNK_WEATHERSYSTEM, Asura_Chunk_WeatherSystem);
				CHUNK_PROCESS(ASURA_CHUNK_FOG, Asura_Chunk_Fog );
#endif // ASURA_SUPPORT_WEATHER_SYSTEM
#ifdef ASURA_SUPPORT_MULTIPLAYER_COMMS
				CHUNK_PROCESS(ASURA_CHUNK_MPCOMMS_BANNEDLIST, Asura_Chunk_MPComms_BannedList );
#endif // ASURA_SUPPORT_MULTIPLAYER_COMMS
				CHUNK_PROCESS(ASURA_CHUNK_FILEINFO,Asura_Chunk_FileInfo);			
				CHUNK_PROCESS(ASURA_CHUNK_RENDERSETTINGS,Asura_Chunk_RenderSettings);			
				CHUNK_PROCESS(ASURA_CHUNK_PADDING, Asura_Chunk_Padding);

				CHUNK_PROCESS(ASURA_CHUNK_PFX_PCLE_TEMP, Asura_Chunk_PFX_ParticleTemp);
				CHUNK_PROCESS(ASURA_CHUNK_PFX_SOURCE_TEMP, Asura_Chunk_PFX_SourceTemp);
				CHUNK_PROCESS(ASURA_CHUNK_PFX_EFFECT_TEMP, Asura_Chunk_PFX_EffectTemp);
				CHUNK_PROCESS(ASURA_CHUNK_PFX_OVERRIDES, Asura_Chunk_PFXOverrides);
#ifdef ASURA_USE_WATER
				CHUNK_PROCESS(ASURA_CHUNK_WATERPACKET, Asura_Chunk_WaterPacket);
				CHUNK_PROCESS(ASURA_CHUNK_WATERCONTROL, Asura_Chunk_WaterControl);
#endif //ASURA_USE_WATER

				CHUNK_PROCESS(ASURA_CHUNK_FMVSUBTITLES, Asura_Chunk_FMVSubtitles);

#if defined( ASURA_USE_NEW_VISIBILITY_SYSTEM )
				CHUNK_PROCESS(ASURA_CHUNK_VISIBILITY, Asura_Chunk_Visibility );
#endif // ASURA_USE_NEW_VISIBILITY_SYSTEM

#if defined(ASURA_USE_TEXTURE_STREAMING)
				CHUNK_PROCESS(ASURA_CHUNK_TEXTURE_STREAMING, Asura_Chunk_Texture_Streaming );
#endif

#ifdef ASURA_USE_PSP_BLUR
				CHUNK_PROCESS(ASURA_CHUNK_PSP_BLUR, Asura_Chunk_PSP_Blur);
#endif //ASURA_USE_PSP_BLUR

#ifdef ASURA_USE_PSP_FILTER
				CHUNK_PROCESS(ASURA_CHUNK_PSP_FILTER, Asura_Chunk_PSP_Filter);
#endif //ASURA_USE_PSP_FILTER

#ifdef ASURA_USE_STREAMING
				CHUNK_PROCESS(ASURA_CHUNK_STREAMINGCONNECTIONS, Asura_Chunk_StreamingConnections);
#endif // ASURA_USE_STREAMING

                CHUNK_PROCESS(ASURA_CHUNK_BLUEPRINTS, Asura_Chunk_Blueprints);

#ifdef ASURA_USE_DECAL_MANAGER
				CHUNK_PROCESS(ASURA_CHUNK_DECALS, Asura_Chunk_Decals);
#endif // ASURA_USE_DECAL_MANAGER

#ifdef ASURA_USE_AXON
//                CHUNK_PROCESS(AXON_CHUNK_LOCATIONSYSTEM_NODES, Axon_Chunk_LocationSystem);
//#ifdef ASURA_TOOLS
//                CHUNK_PROCESS(AXON_CHUNK_LOCATIONSYSTEM_NODETOOLSDATA, Axon_Chunk_LocationSystem_ToolsData);
//#endif
                CHUNK_PROCESS(AXON_CHUNK_ANIMTEMPLATES, Axon_Chunk_AnimTemplates);
#endif // ASURA_USE_AXON

#ifdef ASURA_USE_GUI_MENU_SYSTEM
				CHUNK_PROCESS(ASURA_CHUNK_GUIMENU_ANIMSEQUENCE,Asura_Chunk_GUIMenu_AnimatorSequences);
				CHUNK_PROCESS(ASURA_CHUNK_GUIMENU_MENUPAGE,Asura_Chunk_GUIMenu);
				CHUNK_PROCESS(ASURA_CHUNK_GUIMENU_AUDIO,Asura_Chunk_GUIMenu_Audio);
				CHUNK_PROCESS(ASURA_CHUNK_GUIMENU_TEXTPAGE,Asura_Chunk_GUIMenu_TextPage);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_GUIMENU_FILE,Asura_Chunk_GUIMenu_File);
#endif
#endif // ASURA_USE_GUI_MENU_SYSTEM

#ifdef ASURA_TARGET_SM3
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_DYNAMICLIGHTTEMPLATE,Asura_Chunk_DynamicLightTemplate);
#endif
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(TOOLS_CHUNK_STATICDECALS,Tools_Chunk_StaticDecals);
#endif
#endif

#ifndef ASURA_PHYSICS_DISABLE_RAGDOLL
				CHUNK_PROCESS(ASURA_CHUNK_RAGDOLLDESCRIPTION,Asura_Chunk_RagdollDescription);
#endif

#ifdef ASURA_USE_CONTAINER_SYSTEM
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(TOOLS_CHUNK_CONTAINER_FILE,  Asura_Chunk_ToolsContainerFile);
				CHUNK_PROCESS(TOOLS_CHUNK_CONTAINER_GROUP, Asura_Chunk_ToolsContainerGroup);
				CHUNK_PROCESS(TOOLS_CHUNK_CONTAINER,       Asura_Chunk_ToolsContainer);
#endif //ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_CONTAINER,       Asura_Chunk_Container);
#endif //ASURA_USE_CONTAINER_SYSTEM

#ifdef ASURA_USE_ANIMATION_IK_SOLVER
				CHUNK_PROCESS(ASURA_CHUNK_IK_TEMPLATE,Asura_Chunk_IK_Template);
#endif

#ifdef ASURA_USE_AUDIO_SYSTEM
				CHUNK_PROCESS(ASURA_CHUNK_DSP,Asura_Chunk_DSP);
				CHUNK_PROCESS(ASURA_CHUNK_REVERB_PRESETS,Asura_Chunk_ReverbPresets);
				CHUNK_PROCESS(ASURA_CHUNK_STREAMING_SOUNDS,Asura_Chunk_StreamingSounds);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(TOOLS_CHUNK_STREAMING_SOUNDS,Tools_Chunk_StreamingSounds);
#endif
#endif

#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
				CHUNK_PROCESS( ASURA_CHUNK_DYNAMICMUSIC, Asura_Chunk_DynamicMusic );
#endif

#else // !ASURAIMPORT_EXPORTS
				SKIP_KNOWN_CHUNK( ASURA_CHUNK_FILEINFO );
#endif // !ASURAIMPORT_EXPORTS

				case ASURA_CHUNK_VOID:
				{
					// Can't happen; handled by IsValid() call above.
					Asura_Assert( false, "" );
					//lint -fallthrough
				}

				default:
				{
					if(ProjectSpecificChunkParser)
					{
						bChunkReadCompletely = ProjectSpecificChunkParser(pxChunkHeader,xStream);
						if(xStream.HasReadChunkHeader())
						{
							//If an attempt has been made to read the chunk header ,then assume the chunk was recognised by
							//the project code. RAR 30/06/2006
							break;
						}
					}

#if defined(ASURA_ASSERT) || !defined(ASURA_NO_DEBUG_OUTPUT)
					char szID[5];
#ifdef ASURA_PLATFORM_IS_BIG_ENDIAN
					Asura_PrintfToBfr( szID, "%c%c%c%c", 5, true, pxChunkHeader->TextID[3], pxChunkHeader->TextID[2], pxChunkHeader->TextID[1], pxChunkHeader->TextID[0] );
#else
					Asura_PrintfToBfr( szID, "%c%c%c%c", 5, true, pxChunkHeader->TextID[0], pxChunkHeader->TextID[1], pxChunkHeader->TextID[2], pxChunkHeader->TextID[3] );
#endif

#ifdef ASURAIMPORT_EXPORTS
					Asura_OutputToDebugger_Retail(( "ERROR: Unknown chunk type '%s'.\n", szID ));
#else
					Asura_Assertf( false, ("ERROR: Unknown chunk type '%s'.", szID) );
#endif
#endif

					bReadInChunkHeader = false;
					break;
				}
			}		// End of switch.

#if defined(ASURA_MEMORYMANAGEMENT_FILL)
			if(dbg_bCheckGuardBytesForEveryChunk)
			{
				Asura_MemoryManagement_System::CheckMallocGuardBytes();
			}
#endif	//defined(ASURA_MEMORYMANAGEMENT_FILL)

		}

		if( !bReadInChunkHeader )
		{
			// KOR - read the rest of the header to get the chunk size
			xStream.ReadChunk_GetNextHeader(pcChunkBuffer, sizeof(Asura_Chunk_Header));
		}
		// KOR - finish off the current chunk. The stream performs verification and skipping if necessary
		xStream.ReadChunk_End(bChunkReadCompletely);
		return(true);
	}
	else
	{
		return(false);
	}
}

void Asura_ChunkLoading_System::EndProcessAsuraFile(Asura_Chunk_File_Stream& xStream)
{
#if defined(ASURA_CHUNK_LOADERS_ALLOW_DEBUG_TIMING)
	const Asura_Handle	iFileHandle		= xStream.GetFileHandle();
	const char*			szFilename		= Asura_FileAccess::GetFilename(iFileHandle);

	// Output the timings...
	Asura_ProcessChunkTimingList::OutputToDebugger(szFilename);
#endif	// defined(ASURA_CHUNK_LOADERS_ALLOW_DEBUG_TIMING)
#ifdef ASURA_TOOLS
	//The module index conversion array is only valid for the current file , so reset it now
	//RAR 28/01/2008
	Asura_ModuleIndex_Conversion::ClearArray();
#endif
}


bool Asura_ChunkLoading_System::LoadAsuraFileMain(Asura_Handle iHandle)
{
	Asura_Assert(iHandle != ASURA_INVALID_HANDLE,"");

#ifdef ASURA_TOOLS
	Tools_LoadedFileList::AddFile(Asura_FileAccess::GetFilename(iHandle));
#endif

	// Create stream from this file
	Asura_Chunk_File_Stream xStream(iHandle);

	if(ProcessAsuraFileHeader(xStream))
	{
#if _DEBUG
		int iNumChunks = 0;
#endif
		bool	bMoreToProcess;

		if(s_iAllowTerminationCounter == 0)
		{
			s_bShouldTerminate = false;
		}
		++s_iAllowTerminationCounter;
		
		do 
		{
#if defined(ASURA_CHUNK_LOADERS_ALLOW_DEBUG_TIMING)
			// Start timing this chunk...
			Asura_ProcessChunkTimingList::BeginTiming();
#endif	// defined(ASURA_CHUNK_LOADERS_ALLOW_DEBUG_TIMING)
			{
				// Process the chunk itself...
				bMoreToProcess	= ProcessChunk(xStream);
			}
#if defined(ASURA_CHUNK_LOADERS_ALLOW_DEBUG_TIMING)
			// Stop timing this chunk...
			Asura_ProcessChunkTimingList::EndTiming(s_eCurrentChunkID);
#endif	// defined(ASURA_CHUNK_LOADERS_ALLOW_DEBUG_TIMING)

#if _DEBUG
			++iNumChunks;
#endif
			// Update the progress bar (if we have one)...
			if (s_pxProgressDisplay && s_iRecursionLevel==0) UpdateLoadingBar(iHandle);

		} while(bMoreToProcess && !s_bShouldTerminate);

		-- s_iAllowTerminationCounter;
		if(s_iAllowTerminationCounter == 0)
		{
			s_bShouldTerminate = false;
		}
		
		EndProcessAsuraFile(xStream);

		if (s_pxProgressDisplay && s_iRecursionLevel==0)
		{
			UpdateLoadingBar(iHandle);
		}

		return true;
	}
	else
	{
		return(false);
	}
}


#if defined(ASURA_USE_STREAMING)

bool Asura_ChunkLoading_System::StreamAsuraFile(const char* szName)
{
	Asura_Handle	iHandle	= Asura_FileAccess::OpenForStreaming(szName);

	s_iRecursionLevel++;

	if (iHandle==ASURA_INVALID_HANDLE) 
	{
		if (dbg_bLogFailedAsuraFileLoads)
		{
			Asura_Logfile_Retail( (FILELOADING_LOGFILE, "failed to stream %s\n", szName) );
		}
		if (dbg_bPrintFailedAsuraFileLoads)
		{
			Asura_OutputToDebugger(("failed to stream %s\n", szName));
		}
		s_iRecursionLevel--;
		return false;
	}
	else
	{
		if (dbg_bLogSucessfulAsuraFileLoads)
		{
			Asura_Logfile( (FILELOADING_LOGFILE, "streamed %s\n", szName) );
		}
		if (dbg_bPrintSuccessfulAsuraFileLoads)
		{
			Asura_OutputToDebugger(("streamed %s\n", szName));
		}
	}

#ifdef ASURA_USE_TEXTURE_LOG
	if(s_iRecursionLevel==0)
	{
		//store filename to help with logging what loads what texture. RAR 07/03/2006
		Asura_Texture_Management::SetCurrentAsuraFile(szName);
	}
#endif

	bool bOkay = LoadAsuraFileMain(iHandle);

#ifdef ASURA_USE_TEXTURE_LOG
	if(s_iRecursionLevel==0)
	{
		//force texture load now , so that errors are given at a more useful stage. RAR 07/03/2006
		Asura_Texture_Management::LoadTextures();
		Asura_Texture_Management::SetCurrentAsuraFile(0);
	}
#endif


	Asura_FileAccess::Close( iHandle );
	s_iRecursionLevel--;
	return bOkay;
}

#endif	// defined(ASURA_USE_STREAMING)


void Asura_ChunkLoading_System::UpdateLoadingBar( Asura_Handle iFileHandle )
{
    UpdateLoadingBarWithForcedFilePosition( Asura_FileAccess::GetFilePosition(iFileHandle) );
}

void Asura_ChunkLoading_System::UpdateLoadingBarWithForcedFilePosition( u_int uForcedFilePos )
{
	if (!s_pxProgressDisplay) return;
	if (s_fUncompressedFileSize <= 0.0f) { return; }

    float fProgress = uForcedFilePos / s_fUncompressedFileSize;
    fProgress *= (s_fLoadingBarEndValue - s_fLoadingBarInitialValue);
    fProgress += s_fLoadingBarInitialValue;
    Asura_Assert(fProgress >= 0.0f && fProgress <= 1.0f, "invalid progress");
    if (fProgress < 0.0f) fProgress = 0.0f;
    if (fProgress > 1.0f) fProgress = 1.0f;

    if (s_pxProgressDisplay->Update(fProgress))
    {
        Asura_Render::BeginFrame();
        if( s_pfnRender ) { s_pfnRender(); }
        s_pxProgressDisplay->Render();
        Asura_Render::EndFrame();
    }
}

void Asura_ChunkLoading_System::SetFileSize(u_int uFileSize)
{
    s_fUncompressedFileSize = static_cast<float>(uFileSize);
}

void Asura_ChunkLoading_System::YieldBackgroundProcessing(void)
{
#if defined(ASURA_USE_BACKGROUND_CHUNK_PROCESSING)
	// Time to check whether we need to yield...
	Asura_BackgroundChunkProcessing_System::YieldProcessing();
#endif	// defined(ASURA_USE_BACKGROUND_CHUNK_PROCESSING)
}

/// End of file. ///
