//
// Asura_Chunk_Loaders.h
//
// Created by Kevin Lea - 12/06/2001 11:19:11
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_CHUNK_LOADERS_H__
#define __ASURA_CHUNK_LOADERS_H__

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Core/Asura_Handle.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Chunk_File_Stream;
class Asura_Chunk_Stream;

// M A C R O S ///////////////////////////////////////////////////////////////

#define MAKEID(a,b,c,d) ((d<<24)+(c<<16)+(b<<8)+(a))

#if defined(ASURA_TOOLS) && defined(ASURA_MAIN)
#define ASURA_LOG_FILE_CONTENTS 1
#else
#define ASURA_LOG_FILE_CONTENTS 0
#endif

#if ASURA_LOG_FILE_CONTENTS
#define ASURA_FILE_CONTENTS_LOGFILE "logfiles\\file_contents.txt"
#endif

// T Y P E S /////////////////////////////////////////////////////////////////

// (MH - 30.08.2002) please stick to the same format for these to help when
// searching for duplicates. i.e. MAKEID('X','Y','Z','W') with no spaces.
enum ASURA_CHUNKID
{
	ASURA_CHUNK_VOID = 0, 
	ASURA_CHUNK_MESH_VERTICES							= MAKEID('M','S','H','V'),
	ASURA_CHUNK_MESH_POLYGONS							= MAKEID('M','S','H','P'),
	ASURA_CHUNK_MESH_PAINTEDCOLOURS						= MAKEID('M','H','P','C'),
	ASURA_CHUNK_MESH_COLOURS							= MAKEID('M','S','H','C'),
	ASURA_CHUNK_MESH_NORMALS							= MAKEID('N','O','R','M'),
//	ASURA_CHUNK_MESH_NORMALSINDICES						= MAKEID('N','R','M','I'),
	ASURA_CHUNK_MESH_TEXTUREINFO						= MAKEID('M','T','X','T'),
	ASURA_CHUNK_MESH_SMOOTHINGGROUP						= MAKEID('M','S','M','G'),
	ASURA_CHUNK_MESH_POLYGONFLAGS						= MAKEID('M','S','P','F'),
	ASURA_CHUNK_MESH_LOD								= MAKEID('M','S','L','D'),
	ASURA_CHUNK_MESH_MISC								= MAKEID('M','S','M','I'),
	ASURA_CHUNK_MESH_COLLMESHGENDATA					= MAKEID('C','M','G','D'),

	ASURA_CHUNK_ENVIRONMENT_AABBTREES					= MAKEID('A','A','B','B'),
	ASURA_CHUNK_ENVIRONMENT_LBAABBTREES					= MAKEID('L','B','A','B'),

	ASURA_CHUNK_ENVIRONMENT_MODULELIST					= MAKEID('E','M','O','D'),
	ASURA_CHUNK_MODULELINKING							= MAKEID('M','L','I','N'),
#ifdef ASURA_TOOLS
	ASURA_CHUNK_MODULENAMES								= MAKEID('M','L','N','M'),
#endif
	ASURA_CHUNK_STREAMINGMODULENAMES					= MAKEID('S','M','N','M'),

	ASURA_CHUNK_TEXTURENAMES							= MAKEID('T','E','X','T'),
	ASURA_CHUNK_TEXTUREFLAGS							= MAKEID('T','X','F','L'),
	ASURA_CHUNK_MATERIAL								= MAKEID('M','T','R','L'),
	ASURA_CHUNK_MATERIALNAMES							= MAKEID('M','T','N','M'),
	ASURA_CHUNK_TEXTUREANIMATION						= MAKEID('T','X','A','N'),

	ASURA_CHUNK_TILEDTEXTURE							= MAKEID('T','T','X','T'),

	ASURA_CHUNK_HIERARCHY_ANIM							= MAKEID('H','A','N','M'),
	ASURA_CHUNK_HIERARCHY_COMPRESSEDANIM				= MAKEID('H','C','A','N'),
//	ASURA_CHUNK_HIERARCHY_BONE							= MAKEID('H','B','O','N'),
	ASURA_CHUNK_HIERARCHY_SKIN							= MAKEID('H','S','K','N'),
	ASURA_CHUNK_HIERARCHY_SKINLOD						= MAKEID('H','S','K','L'),
	ASURA_CHUNK_HIERARCHY_MARKERPOINTS					= MAKEID('H','M','P','T'),
    ASURA_CHUNK_HIERARCHY_SECTIONBBS					= MAKEID('H','S','B','B'),
    ASURA_CHUNK_HIERARCHY_MANUALSECTIONBBS				= MAKEID('H','S','M','B'),
    ASURA_CHUNK_HIERARCHY_SOUNDS						= MAKEID('H','S','N','D'),
	ASURA_CHUNK_HIERARCHYBODYPART						= MAKEID('H','B','P','T'),
	ASURA_CHUNK_DEATHANIMFILENAMES						= MAKEID('D','A','F','N'),
	
	ASURA_CHUNK_GI_SOLUTION								= MAKEID('G','I','S','N'),

	ASURA_CHUNK_LIGHTS									= MAKEID('L','I','T','E'),
	ASURA_CHUNK_CORONA									= MAKEID('C','R','N','A'),
#ifdef ASURA_TOOLS
	ASURA_CHUNK_LIGHTSCALE								= MAKEID('L','S','C','L'),
#endif

//	ASURA_CHUNK_PHONONS									= MAKEID('P','H','O','N'),
//	ASURA_CHUNK_PHONON_TEMPLATEGUIDS					= MAKEID('P','H','T','G'),
	ASURA_CHUNK_MODULEREVERB							= MAKEID('M','R','V','B'),
	ASURA_CHUNK_SOUNDNAMES								= MAKEID('S','N','D','N'),
	ASURA_CHUNK_STREAMINGBACKGROUNDSOUND				= MAKEID('S','B','S','N'),
	ASURA_CHUNK_STREAMINGBACKGROUNDSOUND_INGAME			= MAKEID('S','B','I','G'),
	ASURA_CHUNK_METAMUSIC								= MAKEID('M','M','U','S'),
	ASURA_CHUNK_METAMUSIC_INGAME						= MAKEID('M','M','U','G'),
	ASURA_CHUNK_REVERBVOLUMES							= MAKEID('R','V','B','V'),

	ASURA_CHUNK_LIGHTVISIBILITIES						= MAKEID('L','V','I','S'),

	ASURA_CHUNK_SKYBOX									= MAKEID('S','K','Y','B'),	// Texture skybox
	ASURA_CHUNK_SKYBOX_SHADER							= MAKEID('S','K','Y','S'),	// Shader skybox
	TOOLS_CHUNK_SKYBOX									= MAKEID('S','K','Y','T'),	// Tools skybox

	ASURA_CHUNK_STARTPOSITION							= MAKEID('S','T','P','S'),
	ASURA_CHUNK_NAVIGATION								= MAKEID('N','A','V','1'),

    ASURA_CHUNK_RESOURCEFILELIST						= MAKEID('R','S','F','L'),
	ASURA_CHUNK_RESOURCEFILE							= MAKEID('R','S','C','F'),

	ASURA_CHUNK_CUTSCENE								= MAKEID('C','U','T','S'),
	ASURA_CHUNK_CUTSCENEACTOR							= MAKEID('C','T','A','C'),
	ASURA_CHUNK_CUTSCENETRACK							= MAKEID('C','T','T','R'),
	ASURA_CHUNK_CUTSCENEEVENT							= MAKEID('C','T','E','V'),
	ASURA_CHUNK_CUTSCENEATTACHMENT						= MAKEID('C','T','A','T'),

	ASURA_CHUNK_ENTITY									= MAKEID('E','N','T','I'),
	ASURA_CHUNK_ENTITY_TOOLSDATA						= MAKEID('E','N','T','T'),
	ASURA_CHUNK_ENTITY_NEXTGUID							= MAKEID('E','N','G','U'),
	ASURA_CHUNK_ENTITY_SECONDARYFILE					= MAKEID('E','N','S','F'),
	ASURA_CHUNK_STATICMESSAGES							= MAKEID('S','M','S','G'),
	ASURA_CHUNK_STATICMESSAGES_INGAME					= MAKEID('S','M','G','M'),
	ASURA_CHUNK_ANIMATIONTABLE							= MAKEID('A','T','B','L'),
	ASURA_CHUNK_EXPORTEDANIMATIONTABLE					= MAKEID('A','T','E','X'),
	ASURA_CHUNK_FONT									= MAKEID('F','O','N','T'),	
    ASURA_CHUNK_FONT_KERNING                            = MAKEID('F','N','T','K'),
	ASURA_CHUNK_LOCALTEXT								= MAKEID('L','T','X','T'),
	ASURA_CHUNK_HASHEDLOCALISEDTEXT						= MAKEID('H','T','X','T'),
	ASURA_CHUNK_LOCALTEXT_PAGE							= MAKEID('P','T','X','T'),
	
	ASURA_CHUNK_SHAPE									= MAKEID('S','H','A','P'),
    ASURA_CHUNK_COLLISIONMESH							= MAKEID('C','M','S','H'),

	// ART - 21/11/2008 - ShapeSound retired...
    //ASURA_CHUNK_SHAPESOUND								= MAKEID('S','H','S','N'),
	
	ASURA_CHUNK_PARTICLETEMPLATES						= MAKEID('P','C','L','T'),
	ASURA_CHUNK_BILLBOARDPLANTS_TEMPLATES				= MAKEID('B','P','L','T'),
	ASURA_CHUNK_BILLBOARDPLANTS_BATCHES					= MAKEID('B','P','L','B'),

	ASURA_CHUNK_FRAGMENT								= MAKEID('F','R','A','G'),
	ASURA_CHUNK_WEATHERSYSTEM							= MAKEID('W','T','H','R'),
    ASURA_CHUNK_FOG										= MAKEID('F','O','G',' '),
    ASURA_CHUNK_FILEINFO								= MAKEID('F','N','F','O'),

	ASURA_CHUNK_EXTENDED_PARTICLE_GENERATOR_TEMPLATE	= MAKEID('A','E','P','T'),
	ASURA_CHUNK_EXPLOSION_TEMPLATE						= MAKEID('E','X','P','T'),
	ASURA_CHUNK_SHAPEDATA								= MAKEID('S','H','P','D'),
	ASURA_CHUNK_EXTENDED_PARTICLE_RAIN_SYSTEM			= MAKEID('A','E','P','R'),
	ASURA_CHUNK_MPCOMMS_BANNEDLIST						= MAKEID('R','U','D','E'),
	
	ASURA_CHUNK_ENVIRONMENTALLIGHTTRANSPORT				= MAKEID('E','L','T','0'),
	ASURA_CHUNK_GLOBALILLUMINATION						= MAKEID('G','L','I','L'),
	ASURA_CHUNK_POLYGONLIGHTS							= MAKEID('P','L','I','T'),

	ASURA_CHUNK_PHYSICSENVIRONMENT						= MAKEID('P','H','E','N'),


	// PFX chunks
	ASURA_CHUNK_PFX_PCLE_TEMP							= MAKEID('F','X','P','T'),
	ASURA_CHUNK_PFX_SOURCE_TEMP							= MAKEID('F','X','S','T'),
	ASURA_CHUNK_PFX_EFFECT_TEMP							= MAKEID('F','X','E','T'),
	TOOLS_CHUNK_PFX_OVERRIDES							= MAKEID('T','P','F','O'),
	ASURA_CHUNK_PFX_OVERRIDES							= MAKEID('A','P','F','O'),

	ASURA_CHUNK_RENDERSETTINGS							= MAKEID('R','E','N','D'),

	ASURA_CHUNK_WATERPACKET								= MAKEID('W','T','P','K'),
	ASURA_CHUNK_WATERCONTROL							= MAKEID('W','T','C','N'),
	ASURA_CHUNK_PADDING									= MAKEID('P','A','D','D'),

	ASURA_CHUNK_FMVSUBTITLES							= MAKEID('S','U','B','T'),

	ASURA_CHUNK_BOTANICALS_TEMPLATES					= MAKEID('B','O','T','T'),
	ASURA_CHUNK_BOTANICALS_INSTANCES					= MAKEID('B','O','T','I'),
	ASURA_CHUNK_BOTANICALS_STYLES						= MAKEID('B','O','T','S'),
	ASURA_CHUNK_BOTANICALS_DEFINITION					= MAKEID('B','O','T','D'),
	ASURA_CHUNK_BOTANICALS_INGAME						= MAKEID('B','O','T','G'),

	ASURA_CHUNK_CARPET_STYLES						    = MAKEID('C','P','T','S'),
	ASURA_CHUNK_CARPET_MESH   						    = MAKEID('C','P','T','M'),

	ASURA_CHUNK_FSFX									= MAKEID('F','S','X','2'),
	TOOLS_CHUNK_FSFX_OVERRIDES							= MAKEID('T','F','S','O'),
	ASURA_CHUNK_FSFX_OVERRIDES							= MAKEID('A','F','S','O'),

	ASURA_CHUNK_TOOLS_ATTRACTORS						= MAKEID('A','T','R','C'),
	ASURA_CHUNK_ATTRACTORS								= MAKEID('A','T','R','T'),
	ASURA_CHUNK_ATTRACTORS_INGAME						= MAKEID('A','T','I','G'),

	ASURA_CHUNK_VISIBILITY								= MAKEID('V','I','S','I'),

	ASURA_CHUNK_PSP_BLUR								= MAKEID('P','B','L','R'),
	ASURA_CHUNK_PSP_FILTER								= MAKEID('P','F','L','T'),

	ASURA_CHUNK_AUDIO_LOCATORS							= MAKEID('A','U','L','O'),		
	ASURA_CHUNK_AUDIO_SAMPLES							= MAKEID('A','U','S','M'),	
	ASURA_CHUNK_AUDIO_MIXING_GROUPS						= MAKEID('A','U','M','G'),	
	ASURA_CHUNK_AUDIO_REVERB_TEMPLATES					= MAKEID('A','U','R','V'),	
	ASURA_CHUNK_AUDIO_EVENTS							= MAKEID('A','U','E','V'),	
	ASURA_CHUNK_AUDIO_PARAMETERS						= MAKEID('A','U','P','A'),	

	ASURA_CHUNK_UILAYOUT								= MAKEID('U','I','X','L'),

	ASURA_CHUNK_STREAMINGCONNECTIONS					= MAKEID('S','T','C','N'),
	ASURA_CHUNK_CURRENTSTREAMINGZONE					= MAKEID('C','S','Z','N'),

	//sound event system
	ASURA_CHUNK_SOUNDEVENT_EVENT						= MAKEID('S','D','E','V'),
	ASURA_CHUNK_SOUNDEVENT_MAPPING						= MAKEID('S','D','M','A'),
	ASURA_CHUNK_SOUNDEVENT_SAMPLE						= MAKEID('S','D','S','M'),
	ASURA_CHUNK_SOUNDEVENT_PHONON						= MAKEID('S','D','P','H'),
	ASURA_CHUNK_SOUNDEVENT_MIXER						= MAKEID('S','D','M','X'),
	ASURA_CHUNK_SOUNDEVENT_GLOBALSETTINGS				= MAKEID('S','D','G','S'),
	ASURA_CHUNK_SOUNDEVENT_TOOLSGLOBALSETTINGS			= MAKEID('S','D','T','G'),

	//dialogue event system
	ASURA_CHUNK_DIALOGUE_EVENT							= MAKEID('D','L','E','V'),
	ASURA_CHUNK_DIALOGUE_LINE							= MAKEID('D','L','L','N'),
	ASURA_CHUNK_DIALOGUE_LINE_TEMPLATE					= MAKEID('D','L','L','T'),
	ASURA_CHUNK_DIALOGUE_TOOLSGLOBALSETTINGS			= MAKEID('D','L','T','G'),
	ASURA_CHUNK_DIALOGUE_TOOLSHASHTOPATH				= MAKEID('D','L','H','P'),
	ASURA_CHUNK_DIALOGUE_INGAME							= MAKEID('D','L','I','G'),

	ASURA_CHUNK_DSP										= MAKEID('A','D','S','P'),
	ASURA_CHUNK_STREAMING_SOUNDS						= MAKEID('A','S','T','S'),
	TOOLS_CHUNK_STREAMING_SOUNDS						= MAKEID('T','S','T','S'),
	ASURA_CHUNK_REVERB_PRESETS							= MAKEID('R','V','B','P'),

	ASURA_CHUNK_LIPSYNC_ARTICULATION					= MAKEID('L','S','A','D'),

	TOOLS_CHUNK_OBJECTGROUPS							= MAKEID('O','B','G','R'),
	TOOLS_CHUNK_HELPEROBJECT							= MAKEID('H','L','O','B'),
	TOOLS_CHUNK_TOOLSLIGHTS								= MAKEID('T','L','T','S'),
	ASURA_CHUNK_WAYPOINTINGSETTINGS						= MAKEID('W','P','S','G'),

    ASURA_CHUNK_BLUEPRINTS                              = MAKEID('B','L','U','E'),

// (MH - 25.09.2008) animation sequence system retired - chunk IDs left here for reference
//	ASURA_CHUNK_ANIMATIONSEQUENCESET                    = MAKEID('A','N','S','S'),
//	ASURA_CHUNK_ANIMATIONSTATE                          = MAKEID('A','N','S','T'),
//	ASURA_CHUNK_ANIMATIONSEQUENCE                       = MAKEID('A','N','S','Q'),

    AXON_CHUNK_ANIMTEMPLATES                            = MAKEID('A','X','A','T'),

	// Material-response system...
	TOOLS_CHUNK_MATERIAL_RESPONSE_NAMES					= MAKEID('M','R','N','M'),
	TOOLS_CHUNK_MATERIAL_RESPONSE						= MAKEID('M','R','S','P'),
	ASURA_CHUNK_MATERIAL_RESPONSE						= MAKEID('M','A','R','E'),

	TOOLS_CHUNK_MATERIAL_RESPONSE_OVERRIDES				= MAKEID('T','M','R','O'),
	ASURA_CHUNK_MATERIAL_RESPONSE_OVERRIDES				= MAKEID('A','M','R','O'),

	// GUIMenu chunk system
	ASURA_CHUNK_GUIMENU_MENUPAGE						= MAKEID('U','I','M','P'),
	ASURA_CHUNK_GUIMENU_WIDGET							= MAKEID('U','I','W','G'),
	ASURA_CHUNK_GUIMENU_ELEMENT							= MAKEID('U','I','E','L'),
	ASURA_CHUNK_GUIMENU_FILE							= MAKEID('U','I','F','L'),
	ASURA_CHUNK_GUIMENU_ANIMSEQUENCE					= MAKEID('U','I','A','N'),
	ASURA_CHUNK_GUIMENU_AUDIO							= MAKEID('U','I','A','U'),
	ASURA_CHUNK_GUIMENU_TEXTPAGE						= MAKEID('U','I','T','P'),

	// Facial-animation system...
	ASURA_CHUNK_FACIALANIMATION_FACE					= MAKEID('F','A','C','E'),
	ASURA_CHUNK_FACIALANIMATION_FACEANIMATION			= MAKEID('F','A','A','N'),
	TOOLS_CHUNK_FACIALANIMATION_FACE					= MAKEID('T','F','A','C'),
	TOOLS_CHUNK_FACIALANIMATION_FACEANIMATION			= MAKEID('T','F','A','N'),
	TOOLS_CHUNK_FACIALANIMATION_FACEPOSE				= MAKEID('T','F','P','S'),
	TOOLS_CHUNK_FACIALANIMATION_FACETYPE				= MAKEID('T','F','T','Y'),

	// Miscellaneous...
	ASURA_CHUNK_DECALS									= MAKEID('D','C','A','L'),
	ASURA_CHUNK_DYNAMICLIGHTTEMPLATE					= MAKEID('D','Y','L','I'),
	ASURA_CHUNK_RAGDOLLDESCRIPTION						= MAKEID('R','A','G','D'),
	TOOLS_CHUNK_STATICDECALS							= MAKEID('S','D','E','C'),
	ASURA_CHUNK_TEXTURE_STREAMING						= MAKEID('T','X','S','T'),

	// Container chunk system
	TOOLS_CHUNK_CONTAINER_FILE							= MAKEID('C','O','N','F'),
	TOOLS_CHUNK_CONTAINER_GROUP							= MAKEID('C','O','N','G'),
	TOOLS_CHUNK_CONTAINER								= MAKEID('C','O','N','T'),
	ASURA_CHUNK_CONTAINER								= MAKEID('C','O','N','A'),

	ASURA_CHUNK_IK_TEMPLATE								= MAKEID('I','K','T','M'),

	ASURA_CHUNK_DYNAMICMUSIC							= MAKEID('D','Y','M','U'),
	ASURA_CHUNK_DYNAMICMUSIC_INGAME						= MAKEID('D','Y','M','G')


};

// (MH - 30.08.2002) please stick to the same format for these to help when
// searching for duplicates. i.e. MAKEID('X','Y','Z','W') with no spaces.

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_Chunk_Header
{
	public:
		Asura_Chunk_Header(){}
#ifdef ASURA_TOOLS
		Asura_Chunk_Header(enum ASURA_CHUNKID id, int s, int v, int f);
#endif

		bool Valid(void) const
		{
			if (!ID)
			{
				return false;
			}

			return true;
		}

		union
		{
			enum ASURA_CHUNKID ID;
			char TextID[4];
		};
		int Size; // in bytes
		int Version;
		int Flags;
};

class Asura_Chunk_Padding : public Asura_Chunk_Header
{
public:
	bool Process(Asura_Handle iFileHandle);   
#ifdef ASURA_TOOLS
	static void Create(class Asura_SaveBlock* pxSaveBlock, int iNumBytes);
#endif
};

static const u_int uASURA_MAX_CHUNKHEADER_SIZE = 512;
class Asura_ChunkBuffer : public Asura_Chunk_Header 
{
	char m_acBuffer[uASURA_MAX_CHUNKHEADER_SIZE - sizeof(Asura_Chunk_Header)];
};

class Asura_ChunkLoading_System
{
public:
    
	static bool LoadAsuraFile(const char* szName, u_char* pucChecksumOutput = NULL);	// checksum returned in 16-byte array if not NULL
	static bool LoadAsuraFile(Asura_Handle iFileHandle);//load using a file that has already been opened
    static void UseLoadingBar(class Asura_ProgressDisplay* pxProgressDisplay, float fEndValue, void (*pfnRender)() = NULL);
    static void KillLoadingBar();
    static void UpdateLoadingBarWithForcedFilePosition( u_int uForcedFilePos );

#if defined(ASURA_USE_STREAMING)
	static bool StreamAsuraFile(const char* szName);
#endif	// defined(ASURA_USE_STREAMING)

    static void SetFileSize(u_int uFileSize);
	static int GetRecursionLevel(){return s_iRecursionLevel;}

	static void TerminateLoading()		{ if(s_iAllowTerminationCounter > 0) { s_bShouldTerminate = true; } }
	static bool IsTerminatingLoading()	{ return s_bShouldTerminate; }

	#ifdef ASURA_TOOLS
	//Sets a callback function that will be used on each chunk read. Should return true if that chunk is wanted
	//and false if it is to be skipped
	static void SetChunkLoadFilter(bool (pfnChunkLoadFilter)(enum ASURA_CHUNKID)){s_pfnChunkLoadFilter = pfnChunkLoadFilter;}
	#endif
    
    static char* Platform_GetTempChunkMemory( u_int uBytesReqd );
    static void Platform_ReturnTempChunkMemory( char* pcTempMem );

	static bool ProcessAsuraFileHeader(Asura_Chunk_File_Stream& xStream);
	static bool ProcessChunk(Asura_Chunk_File_Stream& xStream);
	static void EndProcessAsuraFile(Asura_Chunk_File_Stream& xStream);

	static ASURA_CHUNKID GetCurrentChunkID(void) { return(s_eCurrentChunkID); }

	static void YieldBackgroundProcessing(void);

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	static void SetDisplayDebugInfo(const bool bSet)	{ s_bDisplayDebugInfo = bSet; }
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)

private:
	static bool LoadAsuraFileMain(Asura_Handle iFileHandle);
    static void UpdateLoadingBar(Asura_Handle iFileHandle);

    static class Asura_ProgressDisplay* s_pxProgressDisplay;
    static void (*s_pfnRender)();  // function to call at each render step (if NULL, clears the screen)
    static float s_fLoadingBarInitialValue;
    static float s_fLoadingBarEndValue;
    static float s_fUncompressedFileSize;
	static int s_iRecursionLevel;
	static ASURA_CHUNKID s_eCurrentChunkID;

	static bool s_bShouldTerminate;
	static int	s_iAllowTerminationCounter;

	#ifdef ASURA_TOOLS
	static bool (*s_pfnChunkLoadFilter)(enum ASURA_CHUNKID eChunkID);
	#endif

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	static bool s_bDisplayDebugInfo;
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)
};



// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int ASURA_FILE_ID_NUM_CHARS = 8;

// KOR - 13/06/2003 14:46:20 - Replaced consts with #defines to stop Gamecube compiler warnings
#define ASURA_UNCOMPRESSED_FILE_ID "Asura   "
//static const char* ASURA_UNCOMPRESSED_FILE_ID = "Asura   ";
static const u_int ASURA_ALL_FILE_ID_ACTUAL_CHARS = 5;
static const u_int ASURA_UNCOMPRESSED_FILE_ID_ACTUAL_CHARS = 8;

#define ASURA_COMPRESSED_FILE_ID "AsuraCmp"
//static const char* ASURA_COMPRESSED_FILE_ID = "AsuraCmp";
static const u_int ASURA_COMPRESSED_FILE_ID_ACTUAL_CHARS = 8;

#define ASURA_ZLIB_COMPRESSED_FILE_ID "AsuraZlb"
static const u_int ASURA_ZLIB_COMPRESSED_FILE_ID_ACTUAL_CHARS = 8;


#define ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX "BE"

#define ASURA_STANDARD_FILE_EXTENSION_LE "asr"
#define ASURA_STANDARD_FILE_EXTENSION_BE ASURA_STANDARD_FILE_EXTENSION_LE ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX

#ifdef ASURA_PLATFORM_IS_BIG_ENDIAN
#define ASURA_STANDARD_FILE_EXTENSION ASURA_STANDARD_FILE_EXTENSION_BE
#else
#define ASURA_STANDARD_FILE_EXTENSION ASURA_STANDARD_FILE_EXTENSION_LE
#endif

// P R O T O T Y P E S   A N D   I N L I N E   F U N C T I O N S /////////////

//For defining an extra function for handling any project specific chunks
//Function should return true if the chunk has been read in completely , and false if the chunk reading was aborted part way through.
//If the chunk is unrecognised the return value should also be false.
//Only the ID part of the chunkheader will have been read in at this point
//RAR 30/06/2006
typedef bool (*ASURA_CHUNK_PARSER_FUNCTION)(Asura_Chunk_Header* chunk, class Asura_Chunk_File_Stream& xStream);
void SetProjectSpecificChunkParser(ASURA_CHUNK_PARSER_FUNCTION function);

//extern bool LoadAsuraFile(LPTSTR name);
extern bool LoadAsuraFile(const char* name);

#endif//__ASURA_CHUNK_LOADERS_H__
