//
// Asura_Chunk_Mesh.h
//
// Created by Kevin Lea - 12/06/2001 11:19:47
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_CHUNK_MESH_H__
#define __ASURA_CHUNK_MESH_H__

#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_Handle.h"

#define MAX_NO_OF_LETTERS_IN_MESH_NAME 31

class Asura_Mesh;
class Asura_SaveBlock;
class Asura_Chunk_Stream;

////////////////////////////////////////////////////////////////////////////////////////////////
class Asura_Chunk_Mesh_Vertices : public Asura_Chunk_Header
{
public:
	bool Process(Asura_Handle fileHandle) const;
	//Reads chunk into specified mesh. If bAlreadyReadName then the chunk stream is assumed to be after the mesh name
	bool Process(Asura_Chunk_Stream& xStream, Asura_Mesh* pxMesh, bool bAlreadyReadName) const;
#ifdef ASURA_TOOLS
	static void Create(const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSsaveBlock);
	static void Create(const Asura_Mesh* pxMesh, Asura_Chunk_Stream& xStream);
#endif

	int NumberOfVertices;


};

////////////////////////////////////////////////////////////////////////////////////////////////
class Asura_Chunk_Mesh_Polygons : public Asura_Chunk_Header
{
public:
	bool Process(Asura_Handle fileHandle) const;
	//Reads chunk into specified mesh. If bAlreadyReadName then the chunk stream is assumed to be after the mesh name
	bool Process(Asura_Chunk_Stream& xStream, Asura_Mesh* pxMesh, bool bAlreadyReadName) const;
#ifdef ASURA_TOOLS
	static void Create( const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSaveBlock);
	static void Create( const Asura_Mesh* pxMesh, Asura_Chunk_Stream& xStream);
#endif

	int NumberOfPolygons;
	int NumberOfTriangles;
};

////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_Chunk_Mesh_PaintedColours : public Asura_Chunk_Header
{
public:
	bool Process(Asura_Handle fileHandle) const;
#ifdef ASURA_TOOLS
	static void Create( const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSaveBlock);
	static void Create( const Asura_Mesh* pxMesh, Asura_Chunk_Stream& xStream);
#endif
	int NumberOfPolygons;
};

////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_SAVED_MESH_COLOURS
class Asura_Chunk_Mesh_Colours : public Asura_Chunk_Header
{
public:
	bool Process(Asura_Handle fileHandle) const;
#ifdef ASURA_TOOLS
	static void Create( const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSaveBlock);
	static void Create( const Asura_Mesh* pxMesh, Asura_Chunk_Stream& xStream);
#endif
	int NumberOfPolygons;
};
#endif

////////////////////////////////////////////////////////////////////////////////////////////////
class Asura_Chunk_Mesh_Normals : public Asura_Chunk_Header
{
    
public:

	bool Process(Asura_Handle fileHandle) const;
#ifdef ASURA_TOOLS
	static void Create( const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSaveBlock);
	static void Create( const Asura_Mesh* pxMesh, Asura_Chunk_Stream& xStream);
#endif

	enum ChunkFlags
	{
		ChunkFlag_SmoothedAcrossModules = 0x00000001,
	};

	int NumberOfPolygons;

private:

    bool Platform_Process( Asura_Handle hFile ) const;

};


////////////////////////////////////////////////////////////////////////////////////////////////
#define ASURA_CHUNK_MESH_TEXTUREINFO_VERSION 5
//Version 3 - Added extra UV coordinates for second texture
//		RAR 13/05/04
//Version 4 - Added material response hash ID
//		AAM 11/12/06
//Version 5 - Added second material response ID
//		RAR 13/06/2007

class Asura_Chunk_Mesh_TextureInfo : public Asura_Chunk_Header
{

public:

	bool Process(Asura_Chunk_Stream& xStream) const;
	bool Process(Asura_Handle fileHandle) const;
#ifdef ASURA_TOOLS
	static void Create( const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSaveBlock,const int* piMaterialIDToIndexConversion);
	static void Create( const Asura_Mesh* pxMesh, Asura_Chunk_Stream& xStream,const int* piMaterialIDToIndexConversion);
#endif

	enum ChunkFlags
	{
		ChunkFlag_MaterialResponse_NotYetConvertedToDualMaterial = 0x00000001,
	};


	int NumberOfPolygons;

};

////////////////////////////////////////////////////////////////////////////////////////////////
class Asura_Chunk_Mesh_SmoothingGroup : public Asura_Chunk_Header
{
public:
	bool Process(Asura_Handle fileHandle) const;
#ifdef ASURA_TOOLS
	static void Create( const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSaveBlock);
	static void Create( const Asura_Mesh* pxMesh, Asura_Chunk_Stream& xStream);
#endif

	int NumberOfPolygons;
};

////////////////////////////////////////////////////////////////////////////////////////////////
//
// Version history:
//
//     (RAR - 16.06.2004) -  2 - All polygon flag values shuffled around.
//
//     (MP  - 19.07.2004) -  3 - Nasty fix for flag mix up in Rogue.
//
//     (RAR - 09.02.2005) -  4&5 - No LOS flag added to polygons with Dont Render.
//
//     (RAR - 11.05.2007) -  6 - walkable surface added to polygons flagged as floor or step.
//

//Note that changes to this will almost certaintly require changes to the collision mesh
//and to the max plugins at the same time.
#define ASURA_CHUNK_MESH_POLYGONFLAGS_VERSION (6)

class Asura_Chunk_Mesh_PolygonFlags : public Asura_Chunk_Header
{
public:
	bool Process(Asura_Handle fileHandle) const;
	//Reads chunk into specified mesh. If bAlreadyReadName then the chunk stream is assumed to be after the mesh name
	bool Process(Asura_Chunk_Stream& xStream, Asura_Mesh* pxMesh, bool bAlreadyReadName) const;
#ifdef ASURA_TOOLS
	static void Create( const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSaveBlock);
	static void Create( const Asura_Mesh* pxMesh, Asura_Chunk_Stream& xStream);
#endif

	int NumberOfPolygons;

};

////////////////////////////////////////////////////////////////////////////////////////////////
class Asura_Chunk_Mesh_LOD : public Asura_Chunk_Header
{
public :
	bool Process(Asura_Handle iFileHandle) const;
#ifdef ASURA_TOOLS
	static void Create( const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSaveBlock);
	static void Create( const Asura_Mesh* pxMesh, Asura_Chunk_Stream& xStream);
#endif

	float m_fMinDistance;
};

////////////////////////////////////////////////////////////////////////////////////////////////
//Hmm , thenumber of different chunks for meshes is getting silly.
//I'll start chucking extra stuff into this chunk. RAR 04/12/02
class Asura_Chunk_Mesh_Misc : public Asura_Chunk_Header
{
public :
	bool Process(Asura_Handle iFileHandle) const;
	//Reads chunk into specified mesh. If bAlreadyReadName then the chunk stream is assumed to be after the mesh name
	//bInMeshSystem should be true if the mesh is in Asura_Mesh_System
	bool Process(Asura_Chunk_Stream& xStream, Asura_Mesh* pxMesh, bool bAlreadyReadName, bool bInMeshSystem) const;
#ifdef ASURA_TOOLS
	static void Create( const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSaveBlock,const int* piMaterialIDToIndexConversion);
	static void Create( const Asura_Mesh* pxMesh, Asura_Chunk_Stream& xStream,const int* piMaterialIDToIndexConversion);
#endif

	enum ChunkFlags
	{
		ChunkFlag_AddToShapeTable = 	0x00000001,
		ChunkFlag_CollisionMesh = 		0x00000002,
		ChunkFlag_PolygonGuids = 		0x00000004,
		ChunkFlag_MergedEntities = 		0x00000008,
		ChunkFlag_MergedEntityCollisionMesh = 0x00000010,
		ChunkFlag_CollPolygonGuids = 	0x00000020,
		ChunkFlag_PolygonLightData =	0x00000040,
		ChunkFlag_MaxXRef =				0x00000080,//contains Max XRef information
		ChunkFlag_PolygonIndices =		0x00000100,
		ChunkFlag_ProjectPolygonData =	0x00000200,
		ChunkFlag_VertexFlags =			0x00000400,
		ChunkFlag_PlatformMesh =		0x00000800,//Contains details of platforms that this is for and of the main mesh
		ChunkFlag_SelectionSets =		0x00001000,//Names of selection sets from 3dsmax
	};

};

////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_Chunk_Mesh_CollMeshGenData : public Asura_Chunk_Header
{
public :
	bool Process(Asura_Handle iFileHandle) const;
	static void Create(const Asura_Mesh* pxMesh, Asura_SaveBlock* pxSaveBlock);
};
#endif//__ASURA_CHUNK_MESH_H__
