//
// Asura_Chunk_PFX.cpp
//
// Created by KOR - 21/06/2004 12:17:17
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"


// This file's header.
#include "ChunkSystem/Asura_Chunk_PFX.h"

// Asura headers
#include "Core/Asura_String.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "Core/Asura_TextureManagement.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "FileAccess/Asura_FileAccess.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "SpecialFX/PFX/Asura_PFX_Templates.h"

#ifdef ASURA_TOOLS
#include "Sound/Asura_Sound.h"
#include "Sound/Tools_PickSound.h"
#endif

#include "Debugging/Asura_Logfile.h"

/// PFX template chunk process methods ////////////////////////////////////////////////////////////

bool Asura_Chunk_PFX_ParticleTemp::Process(Asura_Handle iFileHandle) const
{
	Asura_Assert(0 <= Version && Version <= CHUNK_VERSION , "Wrong version number");

#if defined(ASURA_TOOLS) || defined(ASURA_ASSERT)
	Asura_String	xName(Asura_FileAccess::ReadString(iFileHandle));
#else
	Asura_FileAccess::ReadString(iFileHandle);
#endif	// defined(ASURA_TOOLS) || defined(ASURA_ASSERT)
		

	Asura_PFX_Particle_Template* pxNewTemplate = new Asura_PFX_Particle_Template(iFileHandle, Version);
#ifdef ASURA_TOOLS
	// Add by name
	if(!Asura_PFX_System::AddParticleTemplate(xName.GetString(), pxNewTemplate))
#else
	// Add by hash
	if(!Asura_PFX_System::AddParticleTemplate(m_uNameHash, pxNewTemplate))
#endif
	{
	// AJG - 14/11/07 - Outside of tools builds it's probably reasonable for a template to appear twice, as if you have separate level and character exports there's a chance that there'll be crossovers.
#ifdef ASURA_TOOLS
		Asura_Assertf(false, ("Failed to add PFX paricle-template \"%s\".", xName.GetString()));
#endif //#ifdef ASURA_TOOLS
		delete(pxNewTemplate);
	}
	return true;
}

bool Asura_Chunk_PFX_SourceTemp::Process(Asura_Handle iFileHandle) const
{
	Asura_Assert(0 <= Version && Version <= CHUNK_VERSION , "Wrong version number");

#ifndef ASURA_TOOLS
	if(Asura_PFX_System::AddExistingSourceTemplateToCurrentBank(m_uNameHash))
	{
		//Template was already loaded , so skip parsing this chunk.
		return false;
	}
#endif

#if defined(ASURA_TOOLS) || defined(_DEBUG) || defined(ASURA_ASSERT)
	Asura_String	xName(Asura_FileAccess::ReadString(iFileHandle));
#else
	Asura_FileAccess::ReadString(iFileHandle);
#endif	// defined(ASURA_TOOLS) || defined(_DEBUG) || defined(ASURA_ASSERT)
    
	Asura_PFX_Source_Template* pxNewTemplate = new Asura_PFX_Source_Template(iFileHandle, Version);
#ifdef ASURA_TOOLS
	// Add by name
	if(!Asura_PFX_System::AddSourceTemplate(xName.GetString(), pxNewTemplate))
#else
#if defined(_DEBUG)
	pxNewTemplate->m_xName = xName;
#endif	// defined(_DEBUG)
	// Add by hash
	if(!Asura_PFX_System::AddSourceTemplate(m_uNameHash, pxNewTemplate))
#endif
	{
		Asura_Assertf(false, ("Failed to add PFX source-template \"%s\".", xName.GetString()));
		delete(pxNewTemplate);
	}
	return true;
}

bool Asura_Chunk_PFX_EffectTemp::Process(Asura_Handle iFileHandle) const
{
	Asura_Assert(0 <= Version && Version <= CHUNK_VERSION , "Wrong version number");

#ifndef ASURA_TOOLS
	if(Asura_PFX_System::AddExistingEffectTemplateToCurrentBank(m_uNameHash))
	{
		//Template was already loaded , so skip parsing this chunk.
		return false;
	}
#endif


#if defined(ASURA_TOOLS) || defined(_DEBUG) || defined(ASURA_ASSERT)
	Asura_String	xName(Asura_FileAccess::ReadString(iFileHandle));
#else
	Asura_FileAccess::ReadString(iFileHandle);
#endif	// defined(ASURA_TOOLS) || defined(_DEBUG) || defined(ASURA_ASSERT)

	Asura_PFX_Effect_Template* pxNewTemplate = new Asura_PFX_Effect_Template(iFileHandle, Version);
#ifdef ASURA_TOOLS
	// Add by name
	if(!Asura_PFX_System::AddEffectTemplate(xName.GetString(), pxNewTemplate))
#else
#if defined(_DEBUG)
	pxNewTemplate->m_xName = xName;
#endif	// defined(_DEBUG)
	// Add by hash
	if(!Asura_PFX_System::AddEffectTemplate(m_uNameHash, pxNewTemplate))
#endif
	{
		Asura_Assertf(false, ("Failed to add PFX effect-template \"%s\".", xName.GetString()));
		delete(pxNewTemplate);
	}
	return true;
}

/// PFX template chunk create methods /////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS

Asura_Chunk_PFX_ParticleTemp::OutputOverride* Asura_Chunk_PFX_ParticleTemp::s_pfnOutputOverride = NULL;

void Asura_Chunk_PFX_ParticleTemp::Create(const Asura_PFX_Particle_Template* pxTemplate, const Asura_String* pxName, Asura_Chunk_Stream& xStream, bool bExport)
{
	// (SB - 08/05/07) Allow particle to be copied and overridden via a callback
	Asura_PFX_Particle_Template* pxOverriddenTemplate = NULL;
	if (NULL != s_pfnOutputOverride)
	{
		pxOverriddenTemplate = new Asura_PFX_Particle_Template(*pxTemplate);
		s_pfnOutputOverride(*pxOverriddenTemplate, pxName);
	}
	const Asura_PFX_Particle_Template* pxTemplateToUse = (NULL != s_pfnOutputOverride) ? pxOverriddenTemplate : pxTemplate;

	Asura_WriteToStream_Bitfield_Wrapper<u_int>	xTemplateFlagsBitfield (pxTemplateToUse->m_uFlags);

	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bUseLighting,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bUseMotionBlur,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bUseRandomRotation,				1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bLoopAnimation,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bUseFPS,						1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bSpriteRotation,				1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bAlwaysVertical,				1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bAlwaysHorizontal,				1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bRandomTextureOrient,			1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_b3DRotation,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bMirrorRotation,				1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bAlwaysExport,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bAlignEffectX,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bAlignEffectY,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bAlignEffectZ,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bFixedRotation,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bNearCameraFade,				1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bRandomLifetime,				1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bUseSizeScale,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bPreLighting,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bFarCameraFade,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bCrossfadeTextureFrames,		1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bCanUseExpensiveClipping,		1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bBillboardJustX,				1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bBillboardJustY,				1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bUseUVAsAspect,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bLooped,						1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bUseFogging,					1);
	xTemplateFlagsBitfield.Add	(pxTemplateToUse->m_bRenderEarly,					1);

	Asura_Assert(pxTemplateToUse && pxName, "Null pointer given");
	if(!pxTemplateToUse || !pxName) return;

	if(Tools_Level_Export::IsDoingFakeLevelExport()) bExport = false;

	xStream.WriteChunk_Start(ASURA_CHUNK_PFX_PCLE_TEMP, CHUNK_VERSION, 0);
	// Name
	xStream << Asura_GetHashID(pxName->GetString());
	xStream << *pxName;

	// Template attributes
	// Graphs
	pxTemplateToUse->m_xColour.WriteToChunkStream(xStream, bExport);
	pxTemplateToUse->m_xSize.WriteToChunkStream(xStream, bExport);
	pxTemplateToUse->m_xRotation.WriteToChunkStream(xStream, bExport, true);
	pxTemplateToUse->m_xSizeVariation.WriteToChunkStream(xStream, bExport);
	// Texture
	xStream << pxTemplateToUse->m_uNumTextureFrames;
	for(u_int u = 0; u < pxTemplateToUse->m_uNumTextureFrames; ++u)
	{
		xStream << Asura_WriteToStream_Array_Wrapper<float>(4, pxTemplateToUse->m_pxTextureFrames[u].m_afTextureUVs);
		xStream << pxTemplateToUse->m_pxTextureFrames[u].m_fTime;
	}

	xStream << pxTemplateToUse->m_eTranslucency;
	xStream << pxTemplateToUse->m_fMinLifetime;
	xStream << pxTemplateToUse->m_fMaxLifetimeDiff;
	xStream << pxTemplateToUse->m_fBlurLength;
	xStream << pxTemplateToUse->m_fFixedRotation;
	xStream << pxTemplateToUse->m_uNumFixedRotations;
	xStream << pxTemplateToUse->m_fLightingScale;
	xStream << pxTemplateToUse->m_fNearFadeDistance;
	xStream << pxTemplateToUse->m_fNearZeroDistance;
	xStream << xTemplateFlagsBitfield;
	xStream << pxTemplateToUse->m_fFarFadeDistance;
	xStream << pxTemplateToUse->m_fFarZeroDistance;
	xStream << pxTemplateToUse->m_fFPS;
	xStream << pxTemplateToUse->m_xTextureName;
	xStream << pxTemplateToUse->m_fVolumetricDepthScale;
	xStream << pxTemplateToUse->m_uSoundEvent;
	// Finish
	xStream.WriteChunk_End();

	// (SB - 08/05/07) Free copy of particle template
	if (NULL != s_pfnOutputOverride)
	{
		delete pxOverriddenTemplate;
	}
}
#endif // ASURA_TOOLS

Asura_PFX_Particle_Template::Asura_PFX_Particle_Template(Asura_Handle iFileHandle, int iVersion)
{
	Asura_ResourcePushType(ASURA_RESOURCE_PFX);
	{
#ifndef ASURA_TOOLS
		// Don't bother supporting ancient chunk versions outside tools.
		Asura_Assert( iVersion >= 13, "This chunk version is no longer supported outside tools." );
#endif

		m_xColour.ProcessFromFileHandle(iFileHandle);
		m_xSize.ProcessFromFileHandle(iFileHandle);
		m_xRotation.ProcessFromFileHandle(iFileHandle);
#ifdef ASURA_TOOLS
		if(iVersion < 2)
		{
			m_xSizeVariation.Initialise(16, 0.f, 0.f);
		}
		else
#endif
		{
			m_xSizeVariation.ProcessFromFileHandle(iFileHandle);
		}
#ifdef ASURA_TOOLS
		if(iVersion < 3)
		{
			m_uNumTextureFrames = 1;
			Asura_ResourceAllocateArray(m_pxTextureFrames, m_uNumTextureFrames);
			m_pxTextureFrames[0].m_fTime = 0.0f;
			if(iVersion >= 1)
			{
				Asura_FileAccess::ReadData(iFileHandle, m_pxTextureFrames[0].m_afTextureUVs, 4 * sizeof(float));
			}
			else
			{
				m_pxTextureFrames[0].m_afTextureUVs[0] = m_pxTextureFrames[0].m_afTextureUVs[1] = 0.f;
				m_pxTextureFrames[0].m_afTextureUVs[2] = m_pxTextureFrames[0].m_afTextureUVs[3] = 0.125f;
			}
		}
		else 
#endif
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle,m_uNumTextureFrames,u_int);
			Asura_ResourceAllocateArray(m_pxTextureFrames, m_uNumTextureFrames);

			for(u_int u=0 ;u<m_uNumTextureFrames;u++)
			{
				ASURA_CHUNK_FILEEXTRACT(iFileHandle,m_pxTextureFrames[u],Asura_PFX_TextureFrame);
			}
		}
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_eTranslucency, ASURA_TRANSLUCENCY_MODE);
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fMinLifetime, float);
#ifdef ASURA_TOOLS
		if(iVersion < 9)
		{
			m_fMaxLifetimeDiff = 0.f;
			m_bRandomLifetime = false;
		}
		else
#endif
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fMaxLifetimeDiff, float);
		}
#ifdef ASURA_TOOLS
		if(iVersion < 4)
		{
			m_fBlurLength = 0.1f;
		}
		else
#endif
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fBlurLength, float);
		}
#ifdef ASURA_TOOLS
		if(iVersion < 5)
		{
			m_fFixedRotation = 0.f;
			m_uNumFixedRotations = 1;
		}
		else
#endif
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fFixedRotation, float);
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uNumFixedRotations, u_int);
		}
#ifdef ASURA_TOOLS
		if(iVersion < 6)
		{
			m_fLightingScale = 1.f;
		}
		else
#endif
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fLightingScale, float);
		}

#ifdef ASURA_TOOLS
		if(iVersion < 7)
		{
			m_fNearFadeDistance = 1.f;
			m_fNearZeroDistance = .5f;
		}
		else
#endif
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fNearFadeDistance, float);
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fNearZeroDistance, float);
		}

		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uFlags, u_int);

#ifdef ASURA_TOOLS
		if(iVersion < 10)
		{
			m_fFarFadeDistance = 100.0f;
			m_fFarZeroDistance = 200.0f;
		}
		else
#endif
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fFarFadeDistance, float);
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fFarZeroDistance, float);
		}

#ifdef ASURA_TOOLS
		if(iVersion < 11)
		{
			m_fFPS = 1.0f;
			m_bLoopAnimation = false;
			m_bUseFPS = false;
		}
		else
#endif
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fFPS, float);
		}

		// KOR - 09/05/2006 - load texture name
		Asura_String xTextureName;
#ifdef ASURA_TOOLS
		if(iVersion < 12)
		{
#if !defined(ASURA_TARGET_SM3)
			xTextureName = "\\specialfx\\partclfx.tga";
#endif	//!defined(ASURA_TARGET_SM3)
		}
		else
#endif
		{
			ASURA_CHUNK_FILEEXTRACT_STRING(iFileHandle, xTextureName);
		}

#ifdef ASURA_TOOLS
		if (iVersion < 13)
		{
			m_fVolumetricDepthScale = 0.0f;
		}
		else
#endif
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fVolumetricDepthScale, float);
		}

		if(iVersion < 15)
		{
			m_uSoundEvent = 0;
		}
		else
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uSoundEvent, Asura_Hash_ID);
		}

		// KOR - bumping chunk and blanking flag for safety, in case it has ever been set.
		if(iVersion < 14)
		{
			m_bSpriteRotation = false;
		}

#if _DEBUG || defined(ASURA_TOOLS)
		// Only store it in tools
		m_xTextureName = xTextureName;
#endif

		// Assign the handle. Texture should have been loaded by now...
		m_iTextureHandle = Asura_Texture_Management::GetTextureIndex( xTextureName.GetString() );
		// ...unless we're in tools, in which case make sure it's loaded.
#ifdef ASURA_TOOLS
		if(m_iTextureHandle == ASURA_TEXTURE_NONE)
		{
			m_iTextureHandle = Asura_Texture_Management::AddTextureWithResetGroup(xTextureName, ASURA_TEXTURE_RESETTABLE_GROUP_COMMON);
			Asura_Texture_Management::LoadTextures();
		}
#endif
        Asura_Assertf( ASURA_TEXTURE_NONE != m_iTextureHandle, ("Required PFX texture (%s) not loaded", xTextureName.GetString()) );

#ifdef ASURA_TOOLS
		if(iVersion < 1)
		{
			// These will probably be uninitialised
			m_bSpriteRotation	= false;
			m_bAlwaysVertical	= false;
			m_bAlwaysHorizontal	= false;
		}
		if(iVersion < 5)
		{
			m_bAlignEffectX		= false;
			m_bAlignEffectY		= false;
			m_bAlignEffectZ		= false;
			if(m_bAlwaysHorizontal)
			{
				m_bFixedRotation = true;
				m_fFixedRotation = 90.f;
				m_uNumFixedRotations = 1;
			}
			else if(m_bAlwaysVertical)
			{
				m_bFixedRotation = true;
				m_fFixedRotation = 0.f;
				m_uNumFixedRotations = 1;
			}
			else
			{
				m_bFixedRotation = false;
			}
		}

		// (AW - 10.11.2004) - Setup the clamps for the graphs.
		SetupGraphClamps();
		m_bModified = false;
#endif

#ifdef ASURA_PFX_TEMPLATES_SUPPORT_DATABLOCK
		// (MH - 05.09.2005) create the data block for sending data by DMA.
		m_pDataBlock = NULL;
		CreateDataBlock();
#endif // #ifdef ASURA_PFX_TEMPLATES_SUPPORT_DATABLOCK
	}
	Asura_ResourcePopType();

}

#ifdef ASURA_TOOLS
void Asura_Chunk_PFX_SourceTemp::Create(const Asura_PFX_Source_Template* pxTemplate, const Asura_String* pxName, Asura_Chunk_Stream& xStream, bool bExport)
{
	Asura_Assert(pxTemplate && pxName, "Null pointer given");
	if(!pxTemplate || !pxName) return;

	Asura_WriteToStream_Bitfield_Wrapper<u_int>	xTemplateFlagsBitfield (pxTemplate->m_uFlags);

	if(Tools_Level_Export::IsDoingFakeLevelExport()) bExport = false;

	xTemplateFlagsBitfield.Add	(pxTemplate->m_bUseWind,				1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bUseParticleDynamics,	1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bUseSourceDynamics,		1); 
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bUseTimeScale,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bMovePclesWithSource,	1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bBurst,					1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bLooping,				1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bUseTurbulence,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bARPTS,					1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bFixedAirResistance,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bDirectionalGravity,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bPointGravity,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bInheritVelocity,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bUseRange,				1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bPregenerate,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bUseSpeedScale,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bStopInstantly,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bRotatePclesWithSource,  1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bScaleReleaseRateWithVolume,	1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bUseSourceBBForVisibility, 1);

	xStream.WriteChunk_Start(ASURA_CHUNK_PFX_SOURCE_TEMP, CHUNK_VERSION, 0);
	// Name
	xStream << Asura_GetHashID(pxName->GetString());
	xStream << *pxName;

	// Template attributes
	// Graphs
	pxTemplate->m_xSpeed.WriteToChunkStream(xStream, bExport);
	pxTemplate->m_xAngle.WriteToChunkStream(xStream, bExport);
	pxTemplate->m_xSpeedSpread.WriteToChunkStream(xStream, bExport);
	pxTemplate->m_xAngleSpread.WriteToChunkStream(xStream, bExport);
	pxTemplate->m_xReleaseRate.WriteToChunkStream(xStream, bExport);
	pxTemplate->m_xReleaseRand.WriteToChunkStream(xStream, bExport);
	pxTemplate->m_xGravityStrength.WriteToChunkStream(xStream, bExport);
	pxTemplate->m_xTurbulenceStrength.WriteToChunkStream(xStream, bExport);

	xStream << pxTemplate->m_xGravityVector;
	xStream << pxTemplate->m_uParticleTemplateHash;
	xStream << pxTemplate->m_fDuration;
	xStream << pxTemplate->m_fAirResistance;
	xStream << pxTemplate->m_fParticleBounce;
	xStream << pxTemplate->m_fSourceBounce;
	xStream << pxTemplate->m_fParticleFriction;
	xStream << pxTemplate->m_fTimeScale;
	xStream << pxTemplate->m_fVelocityInheritance;
	xStream << pxTemplate->m_fTurbulenceFrequency;
	xStream << pxTemplate->m_fRangeMin;
	xStream << pxTemplate->m_fRangeMax;
	xStream << pxTemplate->m_fRangeDelayTime;
	xStream << pxTemplate->m_uMaxParticles;
	xStream << pxTemplate->m_iSpawnSourceIndex;
	xStream << xTemplateFlagsBitfield;

	xStream.WriteChunk_End();
}
#endif // ASURA_TOOLS

Asura_PFX_Source_Template::Asura_PFX_Source_Template(Asura_Handle iFileHandle, int iVersion)
{
#ifndef ASURA_TOOLS
    // Don't bother supporting ancient chunk versions outside tools.
    Asura_Assert( iVersion >= 4, "This chunk version is no longer supported outside tools." );
#endif

#ifdef ASURA_TOOLS
	if(iVersion < 2)
	{
		//Volume removed , so skip the size of it. All these volumes should be 16 bytes , since
		//they were all the (old) default point structure.
		//RAR 13/09/04
		Asura_FileAccess::SkipData(iFileHandle,16);
	}
	if(iVersion < 1)
	{
		Asura_Graph<Asura_Vector_3> xVelocity(iFileHandle);
		Asura_Graph<Asura_Vector_3> xSpread(iFileHandle);
		const Asura_Graph_Point<Asura_Vector_3>* pxNode = xVelocity.GetControlPoints();
		while(pxNode)
		{
			m_xSpeed.AddPointInternal(pxNode->m_xValue.z, pxNode->m_fTime);
			m_xAngle.AddPointInternal(Asura_Vector_3(pxNode->m_xValue.x, pxNode->m_xValue.y, 0.f), pxNode->m_fTime);
			pxNode = pxNode->m_pxNext;
		}
		m_xSpeed.SetNumDiscreteSamples(xVelocity.GetNumDiscreteSamples());
		m_xAngle.SetNumDiscreteSamples(xVelocity.GetNumDiscreteSamples());
		pxNode = xSpread.GetControlPoints();
		while(pxNode)
		{
			m_xSpeedSpread.AddPointInternal(pxNode->m_xValue.z, pxNode->m_fTime);
			m_xAngleSpread.AddPointInternal(Asura_Vector_3(pxNode->m_xValue.x, pxNode->m_xValue.y, 0.f), pxNode->m_fTime);
			pxNode = pxNode->m_pxNext;
		}
		m_xSpeedSpread.SetNumDiscreteSamples(xSpread.GetNumDiscreteSamples());
		m_xAngleSpread.SetNumDiscreteSamples(xSpread.GetNumDiscreteSamples());
		// Set gravity and turbulence strength to default
		m_xGravityStrength = Asura_Graph<float>(16, 1.f, 1.f);
		m_xTurbulenceStrength = Asura_Graph<float>(16, 1.f, 1.f);
	}
	else if( iVersion < 3 )
	{
		Asura_Graph<Asura_Vector_2> xOldAngle;
		Asura_Graph<Asura_Vector_2> xOldAngleSpread;
		m_xSpeed.ProcessFromFileHandle(iFileHandle);
		xOldAngle.ProcessFromFileHandle(iFileHandle);
		m_xSpeedSpread.ProcessFromFileHandle(iFileHandle);
		xOldAngleSpread.ProcessFromFileHandle(iFileHandle);
		m_xReleaseRate.ProcessFromFileHandle(iFileHandle);
		m_xReleaseRand.ProcessFromFileHandle(iFileHandle);

		// Add roll to angle
		const Asura_Graph_Point<Asura_Vector_2>* pxNode = xOldAngle.GetControlPoints();
		while(pxNode)
		{
			m_xAngle.AddPointInternal(Asura_Vector_3(pxNode->m_xValue.x, pxNode->m_xValue.y, 0.f), pxNode->m_fTime);
			pxNode = pxNode->m_pxNext;
		}
		m_xAngle.SetNumDiscreteSamples(xOldAngle.GetNumDiscreteSamples());

		// Add roll to angle spread
		pxNode = xOldAngleSpread.GetControlPoints();
		while(pxNode)
		{
			m_xAngleSpread.AddPointInternal(Asura_Vector_3(pxNode->m_xValue.x, pxNode->m_xValue.y, 0.f), pxNode->m_fTime);
			pxNode = pxNode->m_pxNext;
		}
		m_xAngleSpread.SetNumDiscreteSamples(xOldAngleSpread.GetNumDiscreteSamples());
		// Set gravity and turbulence strength to default
		m_xGravityStrength = Asura_Graph<float>(16, 1.f, 1.f);
		m_xTurbulenceStrength = Asura_Graph<float>(16, 1.f, 1.f);
	}
	else
#endif
	{
		m_xSpeed.ProcessFromFileHandle(iFileHandle);
		m_xAngle.ProcessFromFileHandle(iFileHandle);
		m_xSpeedSpread.ProcessFromFileHandle(iFileHandle);
		m_xAngleSpread.ProcessFromFileHandle(iFileHandle);
		m_xReleaseRate.ProcessFromFileHandle(iFileHandle);
		m_xReleaseRand.ProcessFromFileHandle(iFileHandle);
		m_xGravityStrength.ProcessFromFileHandle(iFileHandle);
		m_xTurbulenceStrength.ProcessFromFileHandle(iFileHandle);
	}
    
	ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_xGravityVector, Asura_Vector_3);
	ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uParticleTemplateHash, Asura_Hash_ID);

#ifdef ASURA_TOOLS
	if(iVersion < 1)
	{
		float fReleaseRate;
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, fReleaseRate, float);
		m_xReleaseRate = Asura_Graph<float>(16, fReleaseRate, fReleaseRate);
		m_xReleaseRand = Asura_Graph<float>(16, 1.f, 1.f);
	}
#endif

	ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fDuration, float);
	ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fAirResistance, float);

#ifdef ASURA_TOOLS
	if(iVersion < 1)
	{
		m_fParticleBounce = 0.5f;
		m_fSourceBounce = 0.5f;
		m_fParticleFriction = 2.f;
	}
	else
#endif
	{
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fParticleBounce, float);
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fSourceBounce, float);
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fParticleFriction, float);
	}

#ifdef ASURA_TOOLS
	if(iVersion < 3)
	{
		m_fTimeScale = 1.f;
		m_fVelocityInheritance = 1.f;
		m_fTurbulenceFrequency = 1.f;
	}
	else
#endif
	{
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fTimeScale, float);
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fVelocityInheritance, float);
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fTurbulenceFrequency, float);
	}
#ifdef ASURA_TOOLS
	if(iVersion < 4)
	{
		m_fRangeMin = 0.f;
		m_fRangeMax = 100.f;
		m_fRangeDelayTime = 5.f;
		m_bUseRange = false;
	}
	else
#endif
	{
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fRangeMin, float);
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fRangeMax, float);
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fRangeDelayTime, float);
	}

	ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uMaxParticles, u_int);

	if(iVersion < 5)
	{
		m_iSpawnSourceIndex = -1;
	}
	else
	{
		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_iSpawnSourceIndex, int);
	}

	ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uFlags, u_int);
#ifdef ASURA_TOOLS
	if( iVersion < 1 )
	{
		m_bLooping = false;
		m_bUseTurbulence = false;
		m_bARPTS = false;
	}
	if(iVersion < 3)
	{
		// New flags
		m_bUseTimeScale = false;
		m_bMovePclesWithSource = false;
		m_bRotatePclesWithSource = false;
		m_bFixedAirResistance = (m_fAirResistance > 0.f);
		const float fGravityStrength = m_xGravityVector.Normalise();
		if(fGravityStrength > 0.f)
		{
			m_bDirectionalGravity = true;
			m_xGravityStrength *= fGravityStrength;
		}
		else
		{
			m_bDirectionalGravity = false;
		}
		m_bPointGravity = false;
		m_bInheritVelocity = false;
	}
#endif

    
#ifdef ASURA_TOOLS
	// (AW - 10.11.2004) - Setup the clamps for the graphs.
	SetupGraphClamps();
	m_bModified = false;
  	m_bMaxParticleWarning = false;

#endif //ASURA_TOOLS

#ifdef ASURA_PFX_TEMPLATES_SUPPORT_DATABLOCK
	// (MH - 05.09.2005) create the data block for sending data by DMA.
	m_pDataBlock = NULL;
	CreateDataBlock();
#endif // #ifdef ASURA_PFX_TEMPLATES_SUPPORT_DATABLOCK
}

#ifdef ASURA_TOOLS
//Asura_Chunk_PFX_EffectTemp::OutputOverride* Asura_Chunk_PFX_EffectTemp::s_pfnOutputOverride = NULL;

void Asura_Chunk_PFX_EffectTemp::Create(const Asura_PFX_Effect_Template* pxTemplate, const Asura_String* pxName, Asura_Chunk_Stream& xStream, bool bExport)
{
	Asura_Assert(pxTemplate && pxName, "Null pointer given");
	if(!pxTemplate || !pxName) return;

	Asura_WriteToStream_Bitfield_Wrapper<u_int>	xTemplateFlagsBitfield (pxTemplate->m_uFlags);

	if(Tools_Level_Export::IsDoingFakeLevelExport()) bExport = false;

	xTemplateFlagsBitfield.Add	(pxTemplate->m_bHasPhoton,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bLooping,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bOrderedRender,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bMoveWithEffect,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bAlwaysExport,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bHasSound,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bPregenerate,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bSoundLoops,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bParticlePhotons,	1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bAlwaysOnTop,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bUseSoundRadiusAndVolume,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bForceRenderOnMainThread,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bHudEffect,			1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_uUnused,				11);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bProjectFlag0,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bProjectFlag1,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bProjectFlag2,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bProjectFlag3,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bProjectFlag4,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bProjectFlag5,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bProjectFlag6,		1);
	xTemplateFlagsBitfield.Add	(pxTemplate->m_bProjectFlag7,		1);

	xStream.WriteChunk_Start(ASURA_CHUNK_PFX_EFFECT_TEMP, CHUNK_VERSION, 0);
	// Name
	xStream	<< Asura_GetHashID(pxName->GetString());
	xStream	<< *pxName;

	// Template attributes
	
	// Photon attributes
	pxTemplate->m_xPhotonColour.WriteToChunkStream(xStream, bExport);
	pxTemplate->m_xPhotonRadius.WriteToChunkStream(xStream, bExport);
	xStream << pxTemplate->m_iPhotonPerParticleSource;

	xStream << pxTemplate->m_uNumSources;
	for(u_int u = 0; u < pxTemplate->m_uNumSources; ++u)
	{
		const Asura_PFX_Effect_SourceProperties* pxSource = &pxTemplate->m_pxSources[u];

		xStream << pxSource->m_uTemplate;
		xStream << pxSource->m_xOffset;
		xStream << pxSource->m_xOrient;
		xStream << pxSource->m_xVelocity;
		xStream << pxSource->m_fTimeOffset;

		pxSource->m_xVolume.WriteToChunkStream (xStream);

		xStream << pxSource->m_iOrientationID;
	}

	xStream << pxTemplate->m_fLifeTime;


	xStream << xTemplateFlagsBitfield;

	//platform overrides
	if(Tools_Level_Export::IsExport())
	{
		xStream << 0;
	}
	else
	{
		xStream << pxTemplate->m_uNumPlatformOverrides;

		for(u_int u=0;u<pxTemplate->m_uNumPlatformOverrides;u++)
		{
			xStream << pxTemplate->m_pxPlatformOverrides[u].m_iPlatform;
			xStream << pxTemplate->m_pxPlatformOverrides[u].m_uEffect;
		}
	}

	xStream << pxTemplate->m_uSoundEvent;

	xStream.WriteChunk_End();
}
#endif // ASURA_TOOLS

Asura_PFX_Effect_Template::Asura_PFX_Effect_Template(Asura_Handle iFileHandle, int iVersion)
: m_uNumInstances(0)
{
	Asura_ResourcePushType(ASURA_RESOURCE_PFX);
	{
#ifndef ASURA_TOOLS
		// Don't bother supporting ancient chunk versions outside tools.
		Asura_Assert( iVersion >= 6, "This chunk version is no longer supported outside tools." );
#endif

		m_xPhotonColour.ProcessFromFileHandle(iFileHandle);
		m_xPhotonRadius.ProcessFromFileHandle(iFileHandle);
		if(iVersion >= 13)
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle,m_iPhotonPerParticleSource,int)
		}
		else
		{
			m_iPhotonPerParticleSource = -1;
		}

		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uNumSources, int);
		u_int uSource = 0;

		Asura_ResourceAllocateArray(m_pxSources, m_uNumSources);
		Asura_PFX_Effect_SourceProperties* pxSource = m_pxSources;

		for(uSource = 0; uSource < m_uNumSources; ++uSource , ++pxSource)
		{
#ifdef ASURA_TOOLS
			pxSource->m_bDisabled = false;
#endif
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, pxSource->m_uTemplate, Asura_Hash_ID);
			// Check this template exists
#ifdef ASURA_TOOLS
			// It's possible that this is an unfinished template with no source temp.
			// If hash is non-zero, check to see if that template has been retrieved
			if(pxSource->m_uTemplate)
#endif
				// If we're not in tools, we must get a source template, otherwise this effect is useless         
				if(!Asura_PFX_System::DoesSourceHashExist(pxSource->m_uTemplate))
				{
#ifdef ASURA_TOOLS
					Asura_String xFileName(Asura_FileAccess::GetFilename(iFileHandle));
					Asura_String xName;
					xFileName.SplitFilename(NULL, &xName, NULL);
					Asura_Assertf(0, ("Failed to find source template for effect template \"%s\"", xName.GetString()));
#else             
					Asura_Assertf(0, ("Failed to find source template with hash %x", pxSource->m_uTemplate));
#endif
				}
				ASURA_CHUNK_FILEEXTRACT(iFileHandle, pxSource->m_xOffset, Asura_Vector_3);
				ASURA_CHUNK_FILEEXTRACT(iFileHandle, pxSource->m_xOrient, Asura_Matrix_3x3);
				ASURA_CHUNK_FILEEXTRACT(iFileHandle, pxSource->m_xVelocity, Asura_Vector_3);
#ifdef ASURA_TOOLS
				if(iVersion < 1)
				{
					pxSource->m_fTimeOffset = 0.f;
				}
				else
#endif
				{
					ASURA_CHUNK_FILEEXTRACT(iFileHandle, pxSource->m_fTimeOffset, float);
				}
#ifdef ASURA_TOOLS
				if(iVersion >= 2)
#endif
				{
					pxSource->m_xVolume.ProcessFromFileHandle(iFileHandle);
				}
#ifdef ASURA_TOOLS
				if(iVersion < 5)
				{
					pxSource->m_iOrientationID = 0;
				}
				else
#endif
				{
					ASURA_CHUNK_FILEEXTRACT(iFileHandle, pxSource->m_iOrientationID, int);
				}
		}

#ifdef ASURA_TOOLS
		if(iVersion < 1)
		{
			float fRadius;
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, fRadius, float);
			m_fLifeTime = 10.f;
		}
		else
#endif
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_fLifeTime, float);
		}

		//Old style sounds need to be skipped from older chunks. RAR 09/07/2008
		if(iVersion >= 3 && iVersion < 15)
		{

			if(iVersion < 10)
			{

				Asura_FileAccess::SkipData(iFileHandle,16);
			}
			else
			{
				int iNumberOfSounds;
				ASURA_CHUNK_FILEEXTRACT(iFileHandle, iNumberOfSounds, int);

				//skip the rest of the sound data
				int iAmountToSkip = 8;//min and max pitch
				if(iVersion>=11) iAmountToSkip+=12; //inner and outer range , volume
				if(iVersion>=12) iAmountToSkip+=8; //min and max time delay
				Asura_FileAccess::SkipData(iFileHandle,iAmountToSkip);
				for(int i=0;i<iNumberOfSounds;i++)
				{
					Asura_FileAccess::ReadString(iFileHandle);
				}

			}
		}

		ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uFlags, u_int);
		//make sure the sound flag isn't set if no sound properties were loaded
		//(Obsolete - this is ignored when using sound events)
		m_bHasSound = false;

		if(iVersion < 9)
		{
			m_bAlwaysOnTop = false;
			m_bHudEffect = false;
#ifdef ASURA_TOOLS
			// Ensure any new flags that get added are zeroed.
			// Save off each of the flags
			bool bHasPhoton			= m_bHasPhoton;
			bool bLooping			= m_bLooping;		
			bool bOrderedRender		= m_bOrderedRender;
			bool bMoveWithEffect	= m_bMoveWithEffect;
			bool bAlwaysExport		= m_bAlwaysExport;
			bool bHasSound			= m_bHasSound;
			bool bPregenerate		= m_bPregenerate;
			bool bSoundLoops		= m_bSoundLoops;
			bool bParticlePhotons	= m_bParticlePhotons;
			bool bAlwaysOnTop		= m_bAlwaysOnTop;
			bool bHudEffect			= m_bHudEffect;
			// Set to zero
			m_uFlags = 0;
			// Restore flags
			m_bHasPhoton			= bHasPhoton;
			m_bLooping				= bLooping;		
			m_bOrderedRender		= bOrderedRender;
			m_bMoveWithEffect		= bMoveWithEffect;
			m_bAlwaysExport			= bAlwaysExport;
			m_bHasSound				= bHasSound;
			m_bPregenerate			= bPregenerate;
			m_bSoundLoops			= bSoundLoops;
			m_bParticlePhotons		= bParticlePhotons;
			m_bAlwaysOnTop			= bAlwaysOnTop;
			m_bHudEffect			= bHudEffect;
#endif
		}
		if(iVersion < 8)
		{
			m_bParticlePhotons = false;
		}
#ifdef ASURA_TOOLS
		if(iVersion < 6)
		{
			m_bSoundLoops = false;
		}
		if(iVersion < 4)
		{
			m_bPregenerate = false;
		}
		if(iVersion >= 7)
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uNumPlatformOverrides, u_int);

			Asura_ResourceAllocateArray(m_pxPlatformOverrides, m_uNumPlatformOverrides);
			for(u_int u=0;u<m_uNumPlatformOverrides; u++)
			{
				ASURA_CHUNK_FILEEXTRACT(iFileHandle,m_pxPlatformOverrides[u],Asura_PFX_Effect_PlatformOverride);

				//remove unused entries as we go
				if(m_pxPlatformOverrides[u].m_uEffect == ASURA_HASH_ID_UNSET)
				{
					m_uNumPlatformOverrides--;
					u--;
				}
			}
		}
		else
		{
			m_uNumPlatformOverrides = 0;
			m_pxPlatformOverrides = 0;
		}
#else
		if(iVersion >= 7)
		{
			//need to skip the platform override data
			//find out how many there are (in theory always 0 for exported levels)
			u_int uNumPlatformOverrides;
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, uNumPlatformOverrides, u_int);

			//and skip them
			if(uNumPlatformOverrides) Asura_FileAccess::SkipData(iFileHandle,uNumPlatformOverrides*sizeof(Asura_PFX_Effect_PlatformOverride));
		}
#endif

		if(iVersion >= 14)
		{
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uSoundEvent, Asura_Hash_ID);
		}
		else
		{
			m_uSoundEvent = 0;
		}


#ifdef ASURA_TOOLS
		if(iVersion < 1)
		{
			m_bLooping = false;
		}
		// (AW - 10.11.2004) - Setup the clamps for the graphs.
		SetupGraphClamps();
		m_iIsolatedSource = -1;
		m_bModified = false;
		m_bDoneAutosave = false;
		m_bDoneBackup = false;
		m_uLastBackupTime = 0;
#endif //ASURA_TOOLS
	}
	Asura_ResourcePopType();
}



/// End of file. ///
