//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_ResourceFileList.cpp
//
// Created by Morgan Parry.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_ResourceFileList.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#ifdef ASURA_TOOLS
#include "ChunkSystem/Asura_Chunk_Macros.h"
#endif
#include "FileAccess/Asura_FileAccess.h"

#if defined(ASURA_TOOLS) && defined(ASURA_MAIN)
#include "Load&Save/Tools_LoadedFileList.h"
#endif


// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool Asura_Chunk_ResourceFileList::Process( Asura_Handle hFile ) const
{
	#if defined(ASURA_TOOLS) && defined(ASURA_MAIN)
	//add all the filenames in thie file list to the list of loaded files , so the tools knows
	//that it doesn't have to load these files

	Asura_Assert(Version<=1,"Unknown chunk version");
	for(u_int i=0;i<uNumEntries;i++)
	{
		const Asura_Char* pszFilename = Asura_FileAccess::ReadString(hFile);
		Tools_LoadedFileList::AddFile(pszFilename,false);

		//skip to the next filename
		Asura_FileAccess::SkipData(hFile,12);
	}
	//return false since there may be some dead space at the end of the chunk
	return false;

	#else
    // Just skip this data in engine. This chunk is only required for the 
    // ResourcePacker program at the moment.
	Asura_FileAccess::SkipData( hFile, this->Size - sizeof(Asura_Chunk_ResourceFileList) );  

	return true;
	#endif
}

#ifdef ASURA_TOOLS
void Asura_Chunk_ResourceFileList::Create(int size, unsigned int entries)
{
	ID          = ASURA_CHUNK_RESOURCEFILELIST;
	ASURA_CHUNK_VERIFY(ID, int);
	Size        = size;
	ASURA_CHUNK_VERIFY(Size, int);
	Version     = 1;
	ASURA_CHUNK_VERIFY(Version, int);
	Flags       = 0;
	ASURA_CHUNK_VERIFY(Flags, int);
	uNumEntries = entries;
	ASURA_CHUNK_VERIFY(uNumEntries, unsigned int);

	return;
}
#endif
