/////////////////////////////////////////////////////////////////////////////////////////
//
//	FILE:			Asura_Chunk_Shape.cpp
//
//	DESCRIPTION:	
//
//	AUTHOR:			Richard M. Smith (8 August 2002)
//
/////////////////////////////////////////////////////////////////////////////////////////


//////////////
// INCLUDES //
//////////////

#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_Shape.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#ifdef ASURA_TOOLS
#include "ChunkSystem/Asura_SaveBlock.h"
#endif
#include "Core/Asura_PlatformBasics.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Shapes/Asura_Shape_System.h"
#include "Shapes/Asura_Shapes.h"


// RICHARDS	#ifdef ASURA_TOOLS
// RICHARDS	#include "Tools_Entity_Data.h"
// RICHARDS	#endif



/////////////
// DEFINES //
/////////////



///////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------**
** Asura_Chunk_Shape **
**--------------------*/


bool Asura_Chunk_Shape::Process( Asura_Handle iFileHandle ) const
{
	Asura_Shape*	pxShape;

	Asura_ResourcePushType(ASURA_RESOURCE_TEMPORARY);
	{
		int		iSize	= Size - sizeof( Asura_Chunk_Shape );
		char*	pcChunkBuffer;

		// Allocate the remaining memory...
		Asura_ResourceAllocateMemoryBlock(pcChunkBuffer, iSize, ASURA_ALIGN_4_BYTE);

		if(pcChunkBuffer == NULL)
		{
			pxShape	= NULL;
		}
		else
		{
			// ART - 14/03/06 - This is nasty, but I haven't the time to write chunk-stream functions
			// for all of the classes held here (particularly "Asura_Platform_Shape_RenderData").
			// This does need changing though. (ARTTODO)
			char*			pcBufferStart = pcChunkBuffer;

			Asura_Hash_ID	uHash;

			// Read in the rest of the chunk...
			Asura_FileAccess::ReadData( iFileHandle, pcChunkBuffer, iSize );

			// Read the hash-ID...
			ASURA_CHUNK_EXTRACT( pcChunkBuffer, uHash, Asura_Hash_ID );

			{
				Asura_Chunk_Memory_Stream	xStream(iSize - sizeof(Asura_Hash_ID), pcChunkBuffer);

				// Create the shape...
				pxShape	= Asura_Shape_System::NewShape(uHash, xStream);
			}

			Asura_ResourceDeallocateMemoryBlock(pcBufferStart);
		}
	}
	Asura_ResourcePopType();

	// Were we successful?
	return(pxShape != NULL);
} // End Function 'Asura_Chunk_Shape::Process'





#ifdef ASURA_TOOLS
/////////////////////////////////////////////////////////////////////////////////////////
// FUNCTION		Asura_Chunk_Shape::Create
// PARAMETERS	The shape
//				Save block
// RETURNS		None
// DESCRIPTION	
// AUTHOR		Richard M. Smith (8 August 2002)
/////////////////////////////////////////////////////////////////////////////////////////
void Asura_Chunk_Shape::Create( Asura_Shape* pxShape, Asura_SaveBlock& xSaveBlock, const List<Asura_Mesh*>& xMeshList )
{
// Calc the size required
	int iChunkSize = sizeof( Asura_Chunk_Shape ) + pxShape->GetChunkBufferSize();

	Asura_Chunk_Shape*	pxChunk = (Asura_Chunk_Shape*)xSaveBlock.GetAccessPointer( iChunkSize );

// Fill in the common chunk details
	pxChunk->ID			= ASURA_CHUNK_SHAPE;
	ASURA_CHUNK_VERIFY(pxChunk->ID, int);
	pxChunk->Size		= iChunkSize;
	ASURA_CHUNK_VERIFY(pxChunk->Size, int);
	pxChunk->Version	= 0;
	ASURA_CHUNK_VERIFY(pxChunk->Version, int);
	pxChunk->Flags		= 0;
	ASURA_CHUNK_VERIFY(pxChunk->Flags, int);

// Any common shape stuff would go here...

// The rest
	char* pcChunkBuffer = (char*)( pxChunk + 1 );
	/// Kim Randell - 14/02/2003 10:31:52 - TODO: endian swap FillInChunkBuffer
	pxShape->FillInChunkBuffer( pcChunkBuffer, xMeshList );

} // End Function 'Asura_Chunk_Shape::Create'
#endif//ASURA_TOOLS






// RICHARDS	///////////////////////////////////////////////////////////////////////////////////////////////
// RICHARDS	/*------------------------------**
// RICHARDS	** Asura_Chunk_Shape_ToolsData **
// RICHARDS	**------------------------------*/
// RICHARDS	
// RICHARDS	
// RICHARDS	void Asura_Chunk_Shape_ToolsData::Process(Asura_Handle fileHandle)
// RICHARDS	{
// RICHARDS		#ifdef ASURA_TOOLS
// RICHARDS		//find the entity
// RICHARDS		Asura_Entity* entity = Asura_Entity_System::Asura_EntityList_Server.FindEntity(Guid);
// RICHARDS		Asura_Assert(entity,"");
// RICHARDS		if(entity)
// RICHARDS		{
// RICHARDS			Asura_Assert(entity->ToolsData==0,"");
// RICHARDS			//allocate toolsdata
// RICHARDS			entity->ToolsData = new Tools_Entity_Data;
// RICHARDS			//and fill it in
// RICHARDS			entity->ToolsData->Position = Position;
// RICHARDS			entity->ToolsData->Name = Asura_FileAccess::ReadString(fileHandle);
// RICHARDS			entity->ToolsData->ToolsObject = 0;
// RICHARDS	
// RICHARDS			return;
// RICHARDS		}
// RICHARDS		#endif
// RICHARDS	
// RICHARDS		//skip the rest of the chunk
// RICHARDS		int size = Size - sizeof(Asura_Chunk_Shape_ToolsData);
// RICHARDS		Asura_FileAccess::SkipData(fileHandle,size);
// RICHARDS	}
// RICHARDS	
// RICHARDS	
// RICHARDS	#ifdef ASURA_TOOLS
// RICHARDS	void Asura_Chunk_Shape_ToolsData::Create(Asura_Entity_Server* entity,Asura_SaveBlock& saveBlock)
// RICHARDS	{
// RICHARDS		//don't bother if no tools data exists
// RICHARDS		if(!entity->ToolsData) return;
// RICHARDS		
// RICHARDS		//find the size required
// RICHARDS		int chunkSize = sizeof(Asura_Chunk_Shape_ToolsData);
// RICHARDS		chunkSize += entity->ToolsData->Name.GetSizeInInts()*4;
// RICHARDS		
// RICHARDS		Asura_Chunk_Shape_ToolsData *chunk = (Asura_Chunk_Shape_ToolsData*)saveBlock.GetAccessPointer(chunkSize);
// RICHARDS	
// RICHARDS		//fill in the common chunk details
// RICHARDS		chunk->ID = Asura_Chunk_Shape_TOOLSDATA;
// RICHARDS		chunk->Size = chunkSize;
// RICHARDS		chunk->Version = 0;
// RICHARDS		chunk->Flags = 0;
// RICHARDS	
// RICHARDS		//and the rest
// RICHARDS		chunk->Guid = entity->GetGuid();
// RICHARDS		chunk->Position = entity->ToolsData->Position;
// RICHARDS	
// RICHARDS		int* dest = (int*)(chunk+1);
// RICHARDS		dest = entity->ToolsData->Name.IntCopy(dest);
// RICHARDS	
// RICHARDS	}
// RICHARDS	#endif
// RICHARDS	
// RICHARDS	////////////////////////////////////////////////////////////////////////////////////////////
// RICHARDS	/*----------------------------**
// RICHARDS	** Asura_Chunk_StaticMessages **
// RICHARDS	**----------------------------*/
// RICHARDS	
// RICHARDS	
// RICHARDS	void Asura_Chunk_StaticMessages::Process(Asura_Handle fileHandle)
// RICHARDS	{
// RICHARDS		Asura_Assert(Version<=0,"Unknown chunk version");
// RICHARDS		//allocate message blocks
// RICHARDS		Asura_Assert(!Asura_StaticMessage_System::NumberOfMessageBlocks,"Static messages have already been allocated.");
// RICHARDS		Asura_StaticMessage_System::SetupStorageSpace((unsigned short)NumberOfMessageBlocks);
// RICHARDS	
// RICHARDS		//read in the message block sizes
// RICHARDS		if(NumberOfMessageBlocks)
// RICHARDS		{
// RICHARDS			Asura_FileAccess::ReadData(fileHandle,Asura_StaticMessage_System::NumberOfMessagesInBlock,NumberOfMessageBlocks);
// RICHARDS		}
// RICHARDS	
// RICHARDS		//read in the messages for each message block
// RICHARDS		for(int i=0;i<Asura_StaticMessage_System::NumberOfMessageBlocks;i++)
// RICHARDS		{
// RICHARDS			if(Asura_StaticMessage_System::NumberOfMessagesInBlock[i])
// RICHARDS			{
// RICHARDS				Asura_StaticMessage* message = Asura_StaticMessage_System::MessagesInBlock[i] = new Asura_StaticMessage[Asura_StaticMessage_System::NumberOfMessagesInBlock[i]];
// RICHARDS	
// RICHARDS				for(int j=0;j<Asura_StaticMessage_System::NumberOfMessagesInBlock[i];j++,message++)
// RICHARDS				{
// RICHARDS					int temp;
// RICHARDS					Asura_FileAccess::ReadData(fileHandle,&temp,sizeof(int));
// RICHARDS					message->MsgID = (enum ASURA_MSG_ID) temp;
// RICHARDS					Asura_FileAccess::ReadData(fileHandle,&message->SendTo,sizeof(Asura_Guid));
// RICHARDS				}
// RICHARDS			}
// RICHARDS			else
// RICHARDS			{
// RICHARDS				Asura_StaticMessage_System::MessagesInBlock[i] = 0;
// RICHARDS			}
// RICHARDS		}
// RICHARDS	}
// RICHARDS	
// RICHARDS	void Asura_Chunk_StaticMessages::Create(Asura_SaveBlock& saveBlock)
// RICHARDS	{
// RICHARDS		//first work out how large this chunk will be
// RICHARDS		int chunkSize= sizeof(Asura_Chunk_StaticMessages);
// RICHARDS		chunkSize += Asura_StaticMessage_System::NumberOfMessageBlocks;//for the NumberOfMessagesInBlock array
// RICHARDS	
// RICHARDS		for(int i=0;i<Asura_StaticMessage_System::NumberOfMessageBlocks;i++)
// RICHARDS		{
// RICHARDS			//space for message block i;
// RICHARDS			chunkSize += Asura_StaticMessage_System::NumberOfMessagesInBlock[i]*8;
// RICHARDS		}	
// RICHARDS	
// RICHARDS		//allocate the chunk
// RICHARDS		Asura_Chunk_StaticMessages *chunk = (Asura_Chunk_StaticMessages*)saveBlock.GetAccessPointer(chunkSize);
// RICHARDS	
// RICHARDS		//fill in the common chunk details
// RICHARDS		chunk->ID = ASURA_CHUNK_STATICMESSAGES;
// RICHARDS		chunk->Size = chunkSize;
// RICHARDS		chunk->Version = 0;
// RICHARDS		chunk->Flags = 0;
// RICHARDS	
// RICHARDS		chunk->NumberOfMessageBlocks = Asura_StaticMessage_System::NumberOfMessageBlocks;
// RICHARDS	
// RICHARDS		char* buffer = (char*)(chunk+1);
// RICHARDS	
// RICHARDS		//fill in all the message block sizes
// RICHARDS		for(int i=0;i<Asura_StaticMessage_System::NumberOfMessageBlocks;i++)
// RICHARDS		{
// RICHARDS			ASURA_CHUNK_FILL(buffer,Asura_StaticMessage_System::NumberOfMessagesInBlock[i],unsigned char);
// RICHARDS		}
// RICHARDS	
// RICHARDS		//and the messages for each block
// RICHARDS		for(int i=0;i<Asura_StaticMessage_System::NumberOfMessageBlocks;i++)
// RICHARDS		{
// RICHARDS			Asura_StaticMessage* message = Asura_StaticMessage_System::MessagesInBlock[i];
// RICHARDS			for(int j=0;j<Asura_StaticMessage_System::NumberOfMessagesInBlock[i];j++,message++)
// RICHARDS			{
// RICHARDS				ASURA_CHUNK_FILL(buffer,message->MsgID,int);
// RICHARDS				ASURA_CHUNK_FILL(buffer,message->SendTo,Asura_Guid);
// RICHARDS			}
// RICHARDS		}
// RICHARDS	
// RICHARDS	}






////////////////////////////////////// End of file //////////////////////////////////////
