//
// Asura_Chunk_StreamingConnections.cpp
//
// Created by Richard Rice - 06/03/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_StreamingConnections.h"

#ifdef ASURA_USE_STREAMING

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "FileAccess/Asura_FileAccess.h"

#include "Streaming/Asura_Streaming_System.h"
#include "Streaming/Asura_Streaming_Zone.h"
#include "Streaming/Asura_Streaming_Connection.h"


#define ASURA_CHUNK_STREAMINGCONNECTIONS_VERSION 3

bool Asura_Chunk_StreamingConnections::Process (Asura_Chunk_Stream& xStream) const
{
	// Is this the correct version?
	if (Version > ASURA_CHUNK_STREAMINGCONNECTIONS_VERSION)
	{
		Asura_Assertf	(false, ("Unknown chunk version %d for streaming connections",Version));
		return			(false);
	}

	if(Version <= 2)
	{
		Asura_String xFilename;
		xStream >> xFilename;	
	}

	Asura_Streaming_Zone* pxZone = Asura_Streaming_System::GetZoneToBeCreatedIn();
	Asura_Assert(pxZone,"Cannot find this streaming-zone.");

	if (pxZone == NULL)
	{
		return(false);
	}
	else
	{
		pxZone->ReadFromChunkStream(xStream,Version);
		return(true);
	}
}

bool Asura_Chunk_StreamingConnections::Process(Asura_Handle iFileHandle) const
{
	Asura_Chunk_File_Stream	xStream (iFileHandle);
	return	(Process(xStream));
}

#ifdef ASURA_TOOLS
void Asura_Chunk_StreamingConnections::Create (const Asura_Streaming_Zone* pxZone,Asura_Chunk_Stream& xStream)
{
	// Write the data...
	xStream.WriteChunk_Start	(ASURA_CHUNK_STREAMINGCONNECTIONS, ASURA_CHUNK_STREAMINGCONNECTIONS_VERSION, 0);

	pxZone->WriteToChunkStream(xStream);

	xStream.WriteChunk_End	();
}

void Asura_Chunk_StreamingConnections::Create(const Asura_Streaming_Zone* pxZone,Asura_SaveBlock* pxSaveBlock)
{
	Asura_Chunk_SaveBlock_Stream xStream (pxSaveBlock);
	Create(pxZone,xStream);
}
#endif // ASURA_TOOLS


#endif //ASURA_USE_STREAMING
