//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_StreamingSounds.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined ASURA_USE_AUDIO_SYSTEM

#include "ChunkSystem/Asura_Chunk_StreamingSounds.h"

// Asura Platform-independent
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_String.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Sound/Asura_Audio_System.h"

/******************************************************************************/
/* Asura_Chunk_StreamingSounds Static Class Data                              */
/******************************************************************************/

const int Asura_Chunk_StreamingSounds::iCURRENT_VERSION = 0;

/******************************************************************************/
/* Asura_Chunk_StreamingSounds Class Functions                                */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_StreamingSounds::Process()
//
// Processes a reatil streaming sounds chunk from a file handle
//
////////////////////////////////////////////////////////////////////////////////

bool Asura_Chunk_StreamingSounds::Process( const Asura_Handle iFileHandle )
{
	Asura_Chunk_File_Stream xStream( iFileHandle );
	return Process( xStream );
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_StreamingSounds::Process()
//
// Processes a reatil streaming sounds chunk from a stream
//
////////////////////////////////////////////////////////////////////////////////

bool Asura_Chunk_StreamingSounds::Process( Asura_Chunk_Stream& xStream )
{
	if( Version > iCURRENT_VERSION )
	{
		Asura_Assertf( false,
			("Streaming sounds chunk version (%d) is too new. Must be <= %d. You may need new code.",
			Version, iCURRENT_VERSION) );

		return false;
	}

	// Extract info about each streaming sound and add it to the tree in the audio system. Skip the
	// actual contents of the files though

	u_int uNumStreamingSounds;
	xStream >> uNumStreamingSounds;

	for( u_int u = 0; u < uNumStreamingSounds; ++u )
	{
		Asura_String xFilename;
		xStream >> xFilename;

		bool bLooping;
		xStream >> bLooping;

		u_int uFileSize;
		xStream >> uFileSize;

		// Now we're at the start of the wave file itself
		const u_int uLocation = xStream.GetPosition();

		// We'll need to stream from the package file containing this chunk
		const char* const szPackage = Asura_FileAccess::GetFilename( xStream.GetFileHandle() );

		Asura_Audio_System::AddStreamInfo( xFilename.GetString(), szPackage, uLocation, bLooping );

		// Skip the file contents, it'll be streamed later
		xStream.Skip( uFileSize );
	}

	return true;
}

#endif // defined ASURA_USE_AUDIO_SYSTEM
