//
// Asura_Zlib.h
//
// Created by Richard Rice - 21/08/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif


#ifndef _ASURA_ZLIB_H_
#define _ASURA_ZLIB_H_
#ifdef ASURA_USE_ZLIB

#include "Zlib/zlib.h"


//Determines the value of window bits used by compression and decompression.
//Setting to 13 should mean that Zlib uses a bit over 8K for decompression
#if !defined ASURA_ZLIB_DEFAULT_WINDOW_BITS
	#define ASURA_ZLIB_DEFAULT_WINDOW_BITS (13)
#endif

//Size of buffer used by decode for input. Determines how much it can read in at a time from the file
#if !defined ASURA_ZLIB_DECODER_INPUT_BUFFER_SIZE
	#define ASURA_ZLIB_DECODER_INPUT_BUFFER_SIZE (64)
#endif

//Size of buffer used by decoder for storing output from zlib. Using this seems to speed things
//up a bit by reducing the number of calls needed to inflate()
//This can be set to 0 , in which case any decompression will be done directly to the buffer
//passed to Asura_Zlib_Decoder::Decompress()
#if !defined ASURA_ZLIB_DECODER_OUTPUT_BUFFER_SIZE
	#define ASURA_ZLIB_DECODER_OUTPUT_BUFFER_SIZE (128)
#endif


struct Asura_Zlib_Header
{
	u_int m_uBigEndian; //non-zero if big endian
	u_int m_uCompressedSize;//size of the compressed data after the header
	u_int m_uUncompressedSize;//size of the original file
};

class Asura_Zlib_Decoder 
{

public:

    Asura_Zlib_Decoder();
    ~Asura_Zlib_Decoder();

	//Start decompression on a file or memory buffer
    void Begin( Asura_Handle iFileHandle );
	void Begin( const void* pSource );

	//Get the original uncompressed file size
    u_int GetDecompressedSize() const { return m_xHeader.m_uUncompressedSize; }

	//Decompress the new uNumBytes to the destination buffer.
	//returns the number actually decompressed
    u_int Decompress( void* pDest, u_int uNumBytes );

	//End decompression and cleanup
    void End();

private:
    void SetupCommonState();

    // These two are mutually exclusive.
    const unsigned char* m_pucSource;
    Asura_Handle m_iFileHandle;

	Asura_Zlib_Header m_xHeader;

	unsigned char* m_pucInputBuffer;
	unsigned char* m_pucOutputBuffer;
	u_int m_uRemainingSource;//how much of the source file has yet to be read
	
	unsigned char* m_pucOutputBufferReadPos;
	u_int m_uOutputBufferBytesLeft;

	bool m_bStreamOpen;
	z_stream m_xZlibStream;
        
};




#endif //ASURA_USE_ZLIB
#endif //_ASURA_ZLIB_H_
