//*******************************************************************
//
// File: Asura_Atomic.h
//
// Created: 29 June 2007 DLH
//
// Description: 
//
//*******************************************************************

#ifndef __ASURA_ATOMIC_H__
#define __ASURA_ATOMIC_H__

// other libraries

// internal dependencies

// forward declarations

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Class Definitions
//*******************************************************************

// A collection of atomic operations.
// Each of these methods are garenteed to happen with no interferace from other CPUs or threads

class Asura_Atomic
{
public:
	// Compares the present contense of the Variable or Pointer. 
	// If it is the same as the Compared value then swap it with the Set value.
	// Return the original value.
	// Mind set is normally, 'If the value is still what I think it should be then change it for what I want it to be'
	template< class T>
	static T* CompareAndSwapPointer( T* volatile &ptPointer, T* ptCompare, T* ptSet);
	
	static int CompareAndSwap( int volatile &iVariable, const int iCompare, const int iSet);
	static u_int CompareAndSwap( u_int volatile &uVariable, const u_int uCompare, const u_int uSet);
	
	// Exchanges the Variable with the new value
	// Returns the old value
	template< class T>
	static T* ExchangePointer( T* volatile &ptPointer, T* ptNew);
	
	// Increases the Variable by One
	// Returns the New Value
	static int Increment( u_int volatile &uVariable);
	static int Increment( int volatile &iVariable);
	
	// Decreases the Varable by One
	// Returns the New Value
	static int Decrement( u_int volatile &uVariable);
	static int Decrement( int volatile &iVariable);
};

//*******************************************************************
// Inline Function Definitions
//*******************************************************************

#include "Core/Asura_Platform_Atomic.h"


//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif // __ASURA_ATOMIC_H__
