//////////////////////////////////////////////////////////////////////////////
//
// Asura_BitPacker.h
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Description:
//
//     class to pack arbitrary-bit-length data into a buffer
//     based on Jon Blow's May02 Game Developer Mag code.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_BITPACKER_H___
#define ___ASURA_BITPACKER_H___ 1

class Asura_BitPacker 
{

public:

    Asura_BitPacker(u_int uBufferSizeInBytes);
	~Asura_BitPacker();

    void Pack(u_int value, u_int num_bits_to_pack);
	void PackBool(bool bValue) { Pack(bValue, 1); }
    void GetResult(const u_char **ppucDataReturn, u_int *puLenReturn) const;
	const void* GetResult(u_int& uLenReturn) const;

	// Reset just reinitialises the bitpacker
	void Reset(void);
	
	// SetByte sets specified byte to a specified value. This is to allow
	// say, a header to be written after packing all the data in. Obviously
	// any bytes you want to set must not be used for data, so you should 
	// call e.g. SkipBytes(1) before packing any data to skip a byte to write to.
	void SkipBytes(u_int uNumBytes);
	void SetByte(u_int uByte, u_char ucValue);

	// RoomForXXX return whether there is room to add the specified number
	// of bits or bytes
	bool RoomForBytes(u_int uNumBytes) const
	{
		return ( ( (m_uNextBitToWrite + 7) / 8 ) + uNumBytes ) <= m_uBufferSize;
	}

	bool RoomForBits(u_int uNumBits) const
	{
		return ( (m_uNextBitToWrite + uNumBits) <= (m_uBufferSize * 8));
	}

private:
	
    u_int m_uNextBitToWrite;     // Initialized to 0
	u_int m_uBytesSkipped;		 // just used to check integrity of byte skipping
	u_int m_uBufferSize;
    u_char* m_pucBuffer;
};

class Asura_BitUnpacker 
{

public:
	
    Asura_BitUnpacker(const void *pData, u_int uLen);

    u_int Unpack(u_int uNumBitsToUnpack);
	bool UnpackBool() { return (Unpack(1) != 0); }
	u_int GetNextBitToRead() const { return m_uNextBitToRead; }
	u_int GetNumBitsRemaining() const { return m_uNumBitsRemaining; }

	// peek allows a value to be examined without advancing the ptrs to the 
	// following data.
	u_int Peek(u_int uNumBits);

private:
	
    u_int m_uNumBitsRemaining;
    u_int m_uNextBitToRead;

    const u_char* const m_pucBuffer;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_BITPACKER_H___

