//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClassifiedList_System.cpp
//
// Created by Anthony Gregson - 28/04/04
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Description:
//
//     Classified list system functions.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Core/Asura_ClassifiedList_System.h"

// Asura headers.
#include "Core/Asura_Classified_List.h"

Asura_ResizingMempool* Asura_ClassifiedList_System::s_pxClassificationNodePool = NULL;
Asura_ResizingMempool* Asura_ClassifiedList_System::s_pxClassifiedNodePool = NULL;

#ifdef DEBUG_CLASSIFICATION_ALLOCATIONS
u_int Asura_ClassifiedList_System::s_uAllocatedClassifieds = 0;
u_int Asura_ClassifiedList_System::s_uAllocatedClassifications = 0;
#endif

void Asura_ClassifiedList_System::Initialise()
{
	// Be sure to update the sizes if the classified node  or classification node ever changes
	s_pxClassifiedNodePool = new Asura_ResizingMempool(256, sizeof(Asura_Classified_Node<int>), __FILE__, __LINE__);
	s_pxClassificationNodePool = new Asura_ResizingMempool(64, sizeof(Asura_Classification_Node<int>), __FILE__, __LINE__);
	Reset();
}
void Asura_ClassifiedList_System::Shutdown()
{
	if ( s_pxClassifiedNodePool )
	{
		delete s_pxClassifiedNodePool;
		s_pxClassifiedNodePool = NULL;
	}
	if (s_pxClassificationNodePool)
	{
		delete s_pxClassificationNodePool;
		s_pxClassificationNodePool = NULL;
	}
}
void Asura_ClassifiedList_System::Reset()
{
	// Place all classified nodes in the empty classified list
	s_pxClassifiedNodePool->Reset();

	// Place all the classification nodes in the classification list
	s_pxClassificationNodePool->Reset();
}

// These use void pointers because they will be cast to templated types for use, so we need to be ambiguous
// Note that the pointer size will be exactly that of the correct node, so please don't try to allocate something else from these pools.
void* Asura_ClassifiedList_System::AllocateClassificationNode()
{
#ifdef DEBUG_CLASSIFICATION_ALLOCATIONS
	s_uAllocatedClassifications++;
#endif
	return s_pxClassificationNodePool->Allocate();
}

void Asura_ClassifiedList_System::FreeClassificationNode(void* pxNode)
{
#ifdef DEBUG_CLASSIFICATION_ALLOCATIONS
	s_uAllocatedClassifications--;
#endif
	s_pxClassificationNodePool->Deallocate(pxNode);
}

void* Asura_ClassifiedList_System::AllocateClassifiedNode()
{
#ifdef DEBUG_CLASSIFICATION_ALLOCATIONS
	s_uAllocatedClassifieds++;
#endif
	return s_pxClassifiedNodePool->Allocate();
}

void Asura_ClassifiedList_System::FreeClassifiedNode(void* pxNode)
{
#ifdef DEBUG_CLASSIFICATION_ALLOCATIONS
	s_uAllocatedClassifieds--;
#endif
	s_pxClassifiedNodePool->Deallocate(pxNode);
}
