//
// Asura_FunctionSequencer.h
//
// Created by RWM - 31/03/2009
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
// 

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_FUNCTIONSEQUENCER_H__
#define __ASURA_FUNCTIONSEQUENCER_H__

#include "Core/Asura_LinkList.h"
#include "Core/Asura_Mempool.h"

#if defined (_DEBUG) || defined(ASURA_USE_PROFILING_SYSTEM)
#define AFS_ADD_AFTER(__add,__prev,__name,__section) AddAfter(__add,__prev,#__add,__name,__section)
#define AFS_ADD_BEFORE(__add,__next,__name,__section) AddBefore(__add,__next,#__add,__name,__section)
#define AFS_ADD_HEAD(__add,__name,__section) AddHead(__add,#__add,__name,__section)
#define AFS_ADD_TAIL(__add,__name,__section) AddTail(__add,#__add,__name,__section)
#else
#define AFS_ADD_AFTER(__add,__prev,__name,__section) AddAfter(__add,__prev)
#define AFS_ADD_BEFORE(__add,__next,__name,__section) AddBefore(__add,__next)
#define AFS_ADD_HEAD(__add,__name,__section) AddHead(__add)
#define AFS_ADD_TAIL(__add,__name,__section) AddTail(__add)
#endif


class Asura_FunctionSequencer_Node
{
public:
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
	Asura_FunctionSequencer_Node(void (*pfFunction)(void), const char* szFunctionString, const char* szReadableString, u_int uProfileID);
#else
	Asura_FunctionSequencer_Node(void (*pfFunction)(void));
#endif

	//Asura_FunctionSequencer_Node( const Asura_FunctionSequencer_Node& xOther );

	bool operator==(const Asura_FunctionSequencer_Node &other) const;

	void (*m_pfFunction)(void);
#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
	const char* m_szFunctionString;
	const char* m_szReadableString;
	u_int m_uProfileID;
#endif
	bool m_bEnabled;
};


class Asura_FunctionSequencer
{
public:
	Asura_FunctionSequencer();
	virtual ~Asura_FunctionSequencer();

#if defined(_DEBUG) || defined (ASURA_USE_PROFILING_SYSTEM)
	void AddBefore( void(*pfnFunctionToAdd)(void), void(*pfnNextFunction)(void), const char* szFunctionName, const char* szReadableName, u_int uProfileSection );
	void AddAfter( void(*pfnFunctionToAdd)(void), void(*pfnPrevFunction)(void), const char* szFunctionName, const char* szReadableName, u_int uProfileSection );
	void AddHead( void(*pfnFunctionToAdd)(void), const char* szFunctionName, const char* szReadableName, u_int uProfileSection );
	void AddTail( void(*pfnFunctionToAdd)(void), const char* szFunctionName, const char* szReadableName, u_int uProfileSection );
#else
	void AddBefore( void(*pfnFunctionToAdd)(void), void(*pfnNextFunction)(void) );
	void AddAfter( void(*pfnFunctionToAdd)(void), void(*pfnPrevFunction)(void) );
	void AddHead( void(*pfnFunctionToAdd)(void) );
	void AddTail( void(*pfnFunctionToAdd)(void) );
#endif

	void Remove( void(*pfnFunctionToRemove)(void) );
	void MoveBefore( void(*pfnFunctionToMove)(void), void(*pfnNextFunction)(void) );
	void MoveAfter( void(*pfnFunctionToMove)(void), void(*pfnPrevFunction)(void) );
	void Execute();
	void Reset();

	void SetFunctionEnabled( void(*pfnFunctionToModify)(void), bool bEnable );
	bool IsFunctionEnabled( void(*pfnFunctionToFind)(void) );

#ifndef ASURA_NO_DEBUG_OUTPUT
	void OutputToDebugger();
#endif

protected:
	Asura_ListElt<Asura_FunctionSequencer_Node>* FindListNodeByFunction( void(*pfnFunctionToFind)(void) );

	static void* AllocateFromListNodeMempool()                { return s_xFunctionNodePool.Allocate(); }
    static void  DeallocateToListNodeMempool( void* pMemory ) { s_xFunctionNodePool.Deallocate( pMemory ); }

	Asura_LinkList<Asura_FunctionSequencer_Node> m_xFunctionSequence;
	static Asura_Mempool s_xFunctionNodePool;
};

#endif//__ASURA_FUNCTIONSEQUENCER_H__
