//
// Asura_HashedLocalisedText.cpp
//
// Created by Richard Rice - 09/12/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Core/Asura_HashedLocalisedText.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_PlatformBasics.h"
#include "Debugging/Asura_Assert.h"


////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Asura_HashedLocalisedText **
**---------------------------*/


Asura_HashedLocalisedText::PageContainer	Asura_HashedLocalisedText::s_xPageContainer("Hashed-Localised Text");

void Asura_HashedLocalisedText::Initialise()
{
}

void Asura_HashedLocalisedText::Shutdown()
{
	s_xPageContainer.Reset();
}


Asura_HashedLocalisedText_Page* Asura_HashedLocalisedText::AllocateStorage(Asura_Hash_ID uPageID,int iNumEntries, u_int uNumBytes)
{
	if(!iNumEntries) return 0;
	
	//allocate entry in tree
	Asura_HashedLocalisedText_Page*	pxPage	= new Asura_HashedLocalisedText_Page(iNumEntries,uNumBytes);
	if(s_xPageContainer.AddResource(uPageID, *pxPage))
	{
		return pxPage;
	}
	else
	{
		Asura_Assert(0,"Error adding page to hashed localised text tree");
		return NULL;
	}
}

const Asura_UnicodeChar* Asura_HashedLocalisedText::GetString(Asura_Hash_ID uPageID,Asura_Hash_ID uStringID)
{
	if( uStringID != ASURA_HASH_ID_UNSET ) 
	{
		Asura_HashedLocalisedText_Page*	pxPage	= s_xPageContainer.GetResource(uPageID);
		if(pxPage != NULL)
		{
			return pxPage->GetString(uStringID);
		}

		Asura_Assertf(0,("No hashed localised text with page ID 0x%08x has been loaded", uPageID));
	}
	
	return 0;	
}

const Asura_UnicodeChar* Asura_HashedLocalisedText::GetStringByIndex(Asura_Hash_ID uPageID, u_int uIndex)
{
	Asura_HashedLocalisedText_Page*	pxPage	= s_xPageContainer.GetResource(uPageID);
	if(pxPage == NULL)
	{
		Asura_Assert(0,"No hashed localised text with that page ID has been loaded");
		return 0;	
	}
	else
	{
		return pxPage->GetStringByIndex(uIndex);
	}
}
#ifdef ASURA_TOOLS
const char* Asura_HashedLocalisedText::GetTextIDStringByIndex(Asura_Hash_ID uPageID, u_int uIndex)
{
	Asura_HashedLocalisedText_Page*	pxPage	= s_xPageContainer.GetResource(uPageID);
	if(pxPage == NULL)
	{
		Asura_Assertf(0,("No hashed localised text with page ID 0x%08x has been loaded", uPageID));
		return 0;	
	}
	else
	{
		return pxPage->GetTextIDStringByIndex(uIndex);
	}
}

Asura_Hash_ID Asura_HashedLocalisedText::GetTextIDByIndex(Asura_Hash_ID uPageID, u_int uIndex)
{
	Asura_HashedLocalisedText_Page*	pxPage	= s_xPageContainer.GetResource(uPageID);
	if(pxPage == NULL)
	{
		Asura_Assertf(0,("No hashed localised text with page ID 0x%08x has been loaded", uPageID));
		return 0;	
	}
	else
	{
		return pxPage->GetTextIDByIndex(uIndex);
	}
}



const Asura_UnicodeChar* Asura_HashedLocalisedText::GetStringFromAnyPage(Asura_Hash_ID uStringID)
{
	for(PageConstIterator xIterator(s_xPageContainer); !xIterator.Done(); xIterator.Next())
	{
		const Asura_HashedLocalisedText_Page*	pxPage	= xIterator.GetCurrent();

		if(pxPage != NULL)
		{
			if (pxPage->IsValidString(uStringID))
			{
				const Asura_UnicodeChar*	pxString	= pxPage->GetString(uStringID);

				if(pxString != NULL)
				{
					return pxString;
				}
			}
		}
	}
	Asura_Assert(0, "Failed to find string in any page");
	return NULL;
}

const char* Asura_HashedLocalisedText::GetTextIDStringFromAnyPage(Asura_Hash_ID uStringID) 
{
	for(PageConstIterator xIterator(s_xPageContainer); !xIterator.Done(); xIterator.Next())
	{
		const Asura_HashedLocalisedText_Page*	pxPage	= xIterator.GetCurrent();

		if(pxPage != NULL)
		{
			if (pxPage->IsValidString(uStringID))
			{
				
				const char*	pcString	= pxPage->GetTextIDString(uStringID);

				if(pcString != NULL)
				{
					return pcString;
				}
			}
		}
	}
	Asura_Assert(0, "Failed to find string in any page");
	return NULL;
}

#endif // ASURA_TOOLS

u_int Asura_HashedLocalisedText::GetNumberOfStrings(Asura_Hash_ID uPageID)
{
	Asura_HashedLocalisedText_Page*	pxPage	= s_xPageContainer.GetResource(uPageID);
	if(pxPage == NULL)
	{
		Asura_Assertf(0,("No hashed localised text with page ID 0x%08x has been loaded", uPageID));
		return 0;	
	}
	else
	{
		return pxPage->GetNumberOfStrings();
	}
}

bool Asura_HashedLocalisedText::IsValidString(Asura_Hash_ID uPageID,Asura_Hash_ID uStringID)
{
	Asura_HashedLocalisedText_Page*	pxPage	= s_xPageContainer.GetResource(uPageID);

	if(pxPage == NULL)
	{
		return false;
	}
	else
	{
		return pxPage->IsValidString(uStringID);
	}
}


const Asura_HashedLocalisedText_Page* Asura_HashedLocalisedText::GetPage(Asura_Hash_ID uPageID)
{
	Asura_HashedLocalisedText_Page*	pxPage	= s_xPageContainer.GetResource(uPageID);

	return(pxPage);
}

#if defined(ASURA_TOOLS)
void Asura_HashedLocalisedText::UnloadPage(Asura_Hash_ID uPageID)
{
	s_xPageContainer.DeleteResource(uPageID);
}
#endif	//defined(ASURA_TOOLS)

u_int Asura_HashedLocalisedText::GetNumberOfPages()
{
	return s_xPageContainer.GetNumResources();

}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------**
** Asura_HashedLocalisedText_Page **
**--------------------------------*/

Asura_HashedLocalisedText_Page::Asura_HashedLocalisedText_Page(int iNumEntries,u_int uNumBytes)
{
	m_iArraySize = iNumEntries;
	m_pwszStrings = new Asura_UnicodeChar*[m_iArraySize];

	m_pxHashTree = new Asura_HashTree;

	for(int i=0;i<m_iArraySize;i++)
	{
		m_pwszStrings[i] = 0;
	}

	m_pcStringBuffer = new char[uNumBytes];

#ifdef ASURA_TOOLS
	m_pszTextIDs = 0;
	m_pcTextIDBuffer = 0;
	m_xName = "UNKNOWN";
#endif
}

Asura_HashedLocalisedText_Page::~Asura_HashedLocalisedText_Page()
{
	delete [] m_pcStringBuffer;
	delete [] m_pwszStrings;
	delete m_pxHashTree;

	#ifdef ASURA_TOOLS
	delete [] m_pszTextIDs;
	delete [] m_pcTextIDBuffer;
	#endif
}

void Asura_HashedLocalisedText_Page::AddString(Asura_Hash_ID uHash,Asura_UnicodeChar* wszString)
{
	Asura_Assert(wszString,"");
	if(!m_pxHashTree || !m_pwszStrings)
	{
		Asura_Assert(0,"No storage space for strings");
		return;
	}
	if(m_pxHashTree->GetNumberOfNodes()>=m_iArraySize)
	{
		Asura_Assert(0,"Not enough space was allocated for strings");
		return;
	}
	
	//add the hash value
	int iIndex;
	if(!m_pxHashTree->FindNodeIndexByHash(uHash,true,iIndex))
	{
		m_pwszStrings[iIndex] = wszString;
	}
	else
	{
		Asura_Assert(0,"Entry for this hash value already exists");
	}
}



const Asura_UnicodeChar* Asura_HashedLocalisedText_Page::GetString(Asura_Hash_ID uStringID) const
{
	if( uStringID == ASURA_HASH_ID_UNSET ) 
	{
		return 0;
	}

	if(!m_pxHashTree || !m_pwszStrings)
	{
		Asura_Assert(0,"No strings loaded");
		return 0;
	}

	int iIndex = -1;
	if(m_pxHashTree->FindNodeIndexByHash(uStringID,false,iIndex))
	{
		Asura_Assert(iIndex>=0 && iIndex<m_iArraySize,"");
		return m_pwszStrings[iIndex];
	}
	else
	{
		Asura_Assert(0,"Failed to find string for this hash ID");
		return 0;
	}
}

const Asura_UnicodeChar* Asura_HashedLocalisedText_Page::GetStringByIndex(u_int uIndex) const
{
	if(!m_pxHashTree || !m_pwszStrings)
	{
		Asura_Assert(0,"No strings loaded");
		return 0;
	}

	Asura_Assert(uIndex < static_cast<u_int>(m_iArraySize),"GetStringByIndex: out of range index");
	if (uIndex < static_cast<u_int>(m_iArraySize)) return m_pwszStrings[uIndex];
	return 0;
}

bool Asura_HashedLocalisedText_Page::IsValidString(Asura_Hash_ID uStringID) const
{
	if(!m_pxHashTree || !m_pwszStrings)
	{
		return false;
	}
	int iIndex;
	return m_pxHashTree->FindNodeIndexByHash(uStringID,false,iIndex);
}

#ifdef ASURA_TOOLS
const char* Asura_HashedLocalisedText_Page::GetTextIDStringByIndex(int iIndex) const
{
	if(!m_pszTextIDs)
	{
		Asura_Assert(0,"Haven't got text ID strings for the localised text");
		return 0;
	}

	if(iIndex<0 || iIndex >= m_iArraySize)
	{
		Asura_Assert(0,"Index out of range");
		return 0;
	}

	return m_pszTextIDs[iIndex];
}

Asura_Hash_ID Asura_HashedLocalisedText_Page::GetTextIDByIndex(u_int uIndex) const
{
	if(!m_pszTextIDs)
	{
		Asura_Assert(0,"Haven't got text ID strings for the localised text");
		return 0;
	}

	if((int)uIndex >= m_iArraySize)
	{
		Asura_Assert(0,"Index out of range");
		return 0;
	}

	return m_pxHashTree->FindHashIDByIndex(uIndex);
}


const char* Asura_HashedLocalisedText_Page::GetTextIDString(Asura_Hash_ID uHashID) const
{
	if(!m_pszTextIDs) return 0;

	int iIndex = -1;
	if(m_pxHashTree->FindNodeIndexByHash(uHashID,false,iIndex))
	{
		Asura_Assert(iIndex>=0 && iIndex<m_iArraySize,"");
		return m_pszTextIDs[iIndex];
	}
	else
	{
		return 0;
	}
}

#endif
