/**
*
* \file Asura_PrimeSearch.h
*
* Created by Andrew Weinkove - 10/10/05
*
* Copyright (C) 2005 Rebellion, All Rights Reserved.
*
**/

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_PRIMESEARCH_H__
#define __ASURA_PRIMESEARCH_H__

//
// This code allows traversal of an array of data in a random order and will visit each
// and every member exactly once.
//
// The code is based on an example in Game Coding Complete (2nd edition) by Mike McShaffry (p87).
//
//	Example usage:
//
//	Object axArray[10];
//	Asura_PrimeSearch xPrimeSearch(10);
//	int iNextMember = xPrimeSearch.GetNext();
//	while (iNextMember != -1)
//	{
//		axArray[iNextMember].DoSomething();
//		iNextMember = xPrimeSearch.GetNext();
//	}


class Asura_PrimeSearch
{
public:
	Asura_PrimeSearch(int iNumElements);
	int GetNext(); // Will return -1 when finished.
	bool Done() const 
	{
		return m_iSearches == *m_piCurrentPrime;
	}
	// Regenerate() makes the list order again and picks a new starting point.
	void Regenerate();

private:
	// Restart() picks a new starting point for the current order and goes through it again.
	// this isn't generally what you want to do, so I've made it private. (MH - 04.07.2006)
	void Restart();

	int m_iSkip;
	int m_iCurrentPosition;
	int m_iMaxElements;
	const int* m_piCurrentPrime;
	int m_iSearches;

	static const int s_aiPrimeArray[];
};


#endif//__ASURA_PRIMESEARCH_H__
