//////////////////////////////////////////////////////////////////////////////
//
// Asura_SortAndSearch.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Core/Asura_SortAndSearch.h"

// Asura includes
#ifdef ASURA_SUPPORT_UNITTEST
#include "UnitTest/Asura_UnitTest.h"
#endif

void Asura_SortAndSearch::Qsort( void* pData, u_int uNumElements, u_int uElementSize, 
								int ( *CompareElements )( const void* pData1, const void* pData2 ) )
{
	qsort( pData, uNumElements, uElementSize, CompareElements );
}

// some specialisation for for signed types which have fast - operator (i.e. int, float)
template <>
int Asura_SortAndSearch::SortAscending<int>( const void* pData1, const void* pData2 )
{
	const int& iData1 = *reinterpret_cast<const int*>(pData1);
	const int& iData2 = *reinterpret_cast<const int*>(pData2);
	return iData1 - iData2;
}
template <>
int Asura_SortAndSearch::SortDescending<int>( const void* pData1, const void* pData2 )
{
	const int& iData1 = *reinterpret_cast<const int*>(pData1);
	const int& iData2 = *reinterpret_cast<const int*>(pData2);
	return iData2 - iData1;
}
template <>
int Asura_SortAndSearch::SortAscending<float>( const void* pData1, const void* pData2 )
{
	const float& fData1 = *reinterpret_cast<const float*>(pData1);
	const float& fData2 = *reinterpret_cast<const float*>(pData2);
	return static_cast<int>(fData1 - fData2);
}
template <>
int Asura_SortAndSearch::SortDescending<float>( const void* pData1, const void* pData2 )
{
	const float& fData1 = *reinterpret_cast<const float*>(pData1);
	const float& fData2 = *reinterpret_cast<const float*>(pData2);
	return static_cast<int>(fData2 - fData1);
}

// non-templatised versions of the template functions to help things compile on PS2.
// When we no longer need to support broken compilers like GCC 2.95.3 these can go.
int Asura_SortAndSearch::SortAscendingInt( const void* pData1, const void* pData2 )
{
	return SortAscending<int>(pData1, pData2);
}

int Asura_SortAndSearch::SortDescendingInt( const void* pData1, const void* pData2 )
{
	return SortDescending<int>(pData1, pData2);
}

int Asura_SortAndSearch::SortAscendingFloat( const void* pData1, const void* pData2 )
{
	return SortAscending<float>(pData1, pData2);
}

int Asura_SortAndSearch::SortDescendingFloat( const void* pData1, const void* pData2 )
{
	return SortDescending<float>(pData1, pData2);
}

int Asura_SortAndSearch::SortAscendingUint( const void* pData1, const void* pData2 )
{
	return SortAscending<u_int>(pData1, pData2);
}

int Asura_SortAndSearch::SortDescendingUint( const void* pData1, const void* pData2 )
{
	return SortDescending<u_int>(pData1, pData2);
}

// unit tests - TODO: add tests for Search functions.
#ifdef ASURA_SUPPORT_UNITTEST

class Asura_SortAndSearch_HashIDUnitTestClass
{
public:
	u_int GetHashID() const { return m_uHashID; }
	void SetHashID(u_int uHashID) { m_uHashID = uHashID; }

private:
	u_int m_uHashID;
};

int SortAscendingByHashID( const void* pData1, const void* pData2 )
{
	return Asura_SortAndSearch::SortAscendingByHashID<Asura_SortAndSearch_HashIDUnitTestClass*>(pData1, pData2);
}

ASURA_UNITTEST(SortAndSearch, Quicksort)
{
	float afNumbers1[8]			= { 4.f, 9.f, -5.f, -2.f, 4.f, 8.f, -3.f, 0.f };
	float afNumbers2[8]			= { 4.f, 9.f, -5.f, -2.f, 4.f, 8.f, -3.f, 0.f };
	int aiNumbers1[8]			= { 4, 9, -5, -2, 4, 8, -3, 0 };
	int aiNumbers2[8]			= { 4, 9, -5, -2, 4, 8, -3, 0 };
	u_int auNumbers3[8]			= { 0x40000000, 0x90000000, 0x50000000, 0x20000000, 0xc0000000, 0x80000000, 0xf0000000, 0x00000000 };
	u_int auNumbers4[8]			= { 0x40000000, 0x90000000, 0x50000000, 0x20000000, 0xc0000000, 0x80000000, 0xf0000000, 0x00000000 };
	const float afSortedAscending[8]	= { -5.f, -3.f, -2.f, 0.f, 4.f, 4.f, 8.f, 9.f };
	const float afSortedDescending[8]	= { 9.f, 8.f, 4.f, 4.f, 0.f, -2.f, -3.f, -5.f };
	const int aiSortedAscending[8]	= { -5, -3, -2, 0, 4, 4, 8, 9 };
	const int aiSortedDescending[8]	= { 9, 8, 4, 4, 0, -2, -3, -5 };
	const u_int auSortedAscending[8] = { 0x00000000, 0x20000000, 0x40000000, 0x50000000, 0x80000000, 0x90000000, 0xc0000000, 0xf0000000 };
	const u_int auSortedDescending[8] = { 0xf0000000, 0xc0000000, 0x90000000, 0x80000000, 0x50000000, 0x40000000, 0x20000000, 0x00000000,  };
	Asura_SortAndSearch_HashIDUnitTestClass axHashIDs[8];
	Asura_SortAndSearch_HashIDUnitTestClass* apxHashIDs[8];
	for (u_int u = 0; u < 8; ++u)
	{
		axHashIDs[u].SetHashID(auNumbers3[u]);
		apxHashIDs[u] = &axHashIDs[u];
	}

	// to make this work on PS2, had to swap the templatised functions for non-template versions.
	Asura_SortAndSearch::Quicksort(afNumbers1, 8, Asura_SortAndSearch::SortAscendingFloat);//Asura_SortAndSearch::SortAscending<float>);
	Asura_SortAndSearch::Qsort(afNumbers2, 8, sizeof(float), Asura_SortAndSearch::SortDescendingFloat);//Asura_SortAndSearch::SortDescending<float>);
	Asura_SortAndSearch::Quicksort(aiNumbers1, 8, Asura_SortAndSearch::SortAscendingInt);//Asura_SortAndSearch::SortAscending<int>);
	Asura_SortAndSearch::Qsort(aiNumbers2, 8, sizeof(int), Asura_SortAndSearch::SortDescendingInt);//Asura_SortAndSearch::SortDescending<int>);
	Asura_SortAndSearch::Qsort(auNumbers3, 8, sizeof(u_int), Asura_SortAndSearch::SortAscendingUint);//Asura_SortAndSearch::SortAscending<u_int>);
	Asura_SortAndSearch::Quicksort(auNumbers4, 8, Asura_SortAndSearch::SortDescendingUint);//Asura_SortAndSearch::SortDescending<u_int>);
	Asura_SortAndSearch::Quicksort(apxHashIDs, 8, SortAscendingByHashID);//Asura_SortAndSearch::SortAscendingByHashID<const Asura_SortAndSearch_HashIDUnitTestClass*>);

	for (u_int u = 0; u < 8; ++u)
	{
		ASURA_UNITTEST_CHECK_EQUAL( afSortedAscending[u], afNumbers1[u]	);
		ASURA_UNITTEST_CHECK_EQUAL( afSortedDescending[u], afNumbers2[u] );
		ASURA_UNITTEST_CHECK_EQUAL( aiSortedAscending[u], aiNumbers1[u]	);
		ASURA_UNITTEST_CHECK_EQUAL( aiSortedDescending[u], aiNumbers2[u] );
		ASURA_UNITTEST_CHECK_EQUAL( auSortedAscending[u], auNumbers3[u]	);
		ASURA_UNITTEST_CHECK_EQUAL( auSortedDescending[u], auNumbers4[u] );
		ASURA_UNITTEST_CHECK_EQUAL( auSortedAscending[u], apxHashIDs[u]->GetHashID() );
	}

}

ASURA_UNITTEST(SortAndSearch, QuicksortStrings)
{
	const char* aszStrings1[4] = { "banana", "Pear", "Banana", "apple" };
	const char* aszStrings2[4] = { "banana", "Pear", "Banana", "apple" };
	const char* aszSortedStrings1[4] = { "Banana", "Pear", "apple", "banana" };
	const char* aszSortedStrings2[4] = { "apple", "banana", "Banana", "Pear" };

	Asura_SortAndSearch::Quicksort( aszStrings1, 4, Asura_SortAndSearch::SortStringsCaseSensitive );
	Asura_SortAndSearch::Quicksort( aszStrings2, 4, Asura_SortAndSearch::SortStringsCaseInsensitive );

	for (u_int u = 0; u < 4; ++u)
	{
		ASURA_UNITTEST_CHECK( !stricmp(aszStrings1[u], aszSortedStrings1[u]) );
		ASURA_UNITTEST_CHECK( !stricmp(aszStrings2[u], aszSortedStrings2[u]) );
	}
}

#endif // #ifdef ASURA_SUPPORT_UNITTEST
