//
// Asura_UserInput.h
//
// Created by Kevin Lea - 12/06/2001 11:02:10
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_USERINPUT_H__
#define __ASURA_USERINPUT_H__

#include "core/Asura_PlatformBasics.h"

#define ASURA_NUMBER_OF_ASCII_KEYS 256

// (MH - 10.07.2002) this system is getting a bit creaky.
// These defines are reflected in the keyID below, but adding
// support for more keyboards, or more mice etc. will make 
// MAX_NUMBER_OF_ASURA_KEYIDS even larger and we're copying 
// that number of bytes every frame etc etc.
// This should change at some point.
#define ASURA_MAX_NUMBER_OF_CONTROLLER_PADS 4
#define ASURA_MAX_NUMBER_OF_KEYBOARDS 1
#define ASURA_MAX_NUMBER_OF_MICE 1
#define ASURA_MAX_NUMBER_OF_JOYSTICKS 1
#define ASURA_MAX_NUMBER_OF_LIGHTGUNS 2

enum ASURA_KEYID
{
	ASURA_KEYID_VOID=0, // used to indicate a blank spot in the key config

	// the first 256 keys correspond to ASCII codes
	ASURA_KEYID_BACKSPACE=8,
	ASURA_KEYID_TAB=9,
	ASURA_KEYID_ENTER=13,
	ASURA_KEYID_ESCAPE=27,
	ASURA_KEYID_SPACE=32,

	ASURA_KEYID_SHIFT=16+ASURA_NUMBER_OF_ASCII_KEYS,
	ASURA_KEYID_CTRL=17+ASURA_NUMBER_OF_ASCII_KEYS,
	ASURA_KEYID_ALT=18+ASURA_NUMBER_OF_ASCII_KEYS,
	ASURA_KEYID_PAUSE=19+ASURA_NUMBER_OF_ASCII_KEYS,
    ASURA_KEYID_CAPSLOCK = 20 + ASURA_NUMBER_OF_ASCII_KEYS,

	
	ASURA_KEYID_PAGEUP=33+ASURA_NUMBER_OF_ASCII_KEYS,
	ASURA_KEYID_PAGEDOWN,
	ASURA_KEYID_END,	
	ASURA_KEYID_HOME,	
	ASURA_KEYID_LEFT,
	ASURA_KEYID_UP,
	ASURA_KEYID_RIGHT,
	ASURA_KEYID_DOWN,
	
	ASURA_KEYID_INS=45+ASURA_NUMBER_OF_ASCII_KEYS,
	ASURA_KEYID_DEL,

	ASURA_KEYID_NUMPAD0=96+ASURA_NUMBER_OF_ASCII_KEYS,     
	ASURA_KEYID_NUMPAD1,     
	ASURA_KEYID_NUMPAD2,     
	ASURA_KEYID_NUMPAD3,     
	ASURA_KEYID_NUMPAD4,     
	ASURA_KEYID_NUMPAD5,     
	ASURA_KEYID_NUMPAD6,     
	ASURA_KEYID_NUMPAD7,     
	ASURA_KEYID_NUMPAD8,     
	ASURA_KEYID_NUMPAD9,     
	ASURA_KEYID_NUMPADMULTIPLY,
	ASURA_KEYID_NUMPADADD,   
	ASURA_KEYID_NUMPADENTER,//virtual key code/scan code for this is identical to main enter key. So this one doesn't get picked up at the moment. - Richard
	ASURA_KEYID_NUMPADSUB,   
	ASURA_KEYID_NUMPADDEL,
	ASURA_KEYID_NUMPADDIVIDE,

	ASURA_KEYID_F1=112+ASURA_NUMBER_OF_ASCII_KEYS,
	ASURA_KEYID_F2,
	ASURA_KEYID_F3,
	ASURA_KEYID_F4,
	ASURA_KEYID_F5,
	ASURA_KEYID_F6,
	ASURA_KEYID_F7,
	ASURA_KEYID_F8,								 
	ASURA_KEYID_F9,
	ASURA_KEYID_F10,
	ASURA_KEYID_F11,
	ASURA_KEYID_F12,
	ASURA_KEYID_F13,
	ASURA_KEYID_F14,
	ASURA_KEYID_F15,
	ASURA_KEYID_F16,

	ASURA_KEYID_NUMLOCK = 0x90 + ASURA_NUMBER_OF_ASCII_KEYS,		
	ASURA_KEYID_SCROLLOK,	

	ASURA_KEYID_LEFTSHIFT = 0xA0 + ASURA_NUMBER_OF_ASCII_KEYS,	
	ASURA_KEYID_RIGHTSHIFT,  
	ASURA_KEYID_LEFTALT,     
	ASURA_KEYID_RIGHTALT,    
	ASURA_KEYID_LEFTCTRL,	
	ASURA_KEYID_RIGHTCTRL,   
	

	ASURA_KEYID_LWIN,
	ASURA_KEYID_RWIN,
	ASURA_KEYID_APPS,

	ASURA_KEYID_MOUSEBUTTON1,
	ASURA_KEYID_MOUSEBUTTON2,
	ASURA_KEYID_MOUSEBUTTON3,
	ASURA_KEYID_MOUSEBUTTON4,
	ASURA_KEYID_MOUSEBUTTON5,

	ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON1,
	ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON2,


	ASURA_KEYID_MOUSEWHEELUP,
	ASURA_KEYID_MOUSEWHEELDOWN,

	ASURA_KEYID_END_OF_KEYBOARD,

	ASURA_KEYID_JOYSTICK_BUTTON_1,
	ASURA_KEYID_JOYSTICK_BUTTON_2,
	ASURA_KEYID_JOYSTICK_BUTTON_3,
	ASURA_KEYID_JOYSTICK_BUTTON_4,
	ASURA_KEYID_JOYSTICK_BUTTON_5,
	ASURA_KEYID_JOYSTICK_BUTTON_6,
	ASURA_KEYID_JOYSTICK_BUTTON_7,
	ASURA_KEYID_JOYSTICK_BUTTON_8,
	ASURA_KEYID_JOYSTICK_BUTTON_9,
	ASURA_KEYID_JOYSTICK_BUTTON_10,
	ASURA_KEYID_JOYSTICK_BUTTON_11,
	ASURA_KEYID_JOYSTICK_BUTTON_12,
	ASURA_KEYID_JOYSTICK_BUTTON_13,
	ASURA_KEYID_JOYSTICK_BUTTON_14,
	ASURA_KEYID_JOYSTICK_BUTTON_15,
	ASURA_KEYID_JOYSTICK_BUTTON_16,
	
	ASURA_KEYID_PAD1_BUTTON1, 
	ASURA_KEYID_PAD1_BUTTON2, 
	ASURA_KEYID_PAD1_BUTTON3, 
	ASURA_KEYID_PAD1_BUTTON4, 
	ASURA_KEYID_PAD1_BUTTON5, 
	ASURA_KEYID_PAD1_BUTTON6, 
	ASURA_KEYID_PAD1_BUTTON7, 
	ASURA_KEYID_PAD1_BUTTON8, 
	ASURA_KEYID_PAD1_BUTTON9, 
	ASURA_KEYID_PAD1_BUTTON10,
	ASURA_KEYID_PAD1_BUTTON11,
	ASURA_KEYID_PAD1_BUTTON12,
	ASURA_KEYID_PAD1_BUTTON13,
	ASURA_KEYID_PAD1_BUTTON14,
	ASURA_KEYID_PAD1_BUTTON15,
	ASURA_KEYID_PAD1_BUTTON16,

	ASURA_KEYID_PAD2_BUTTON1, 
	ASURA_KEYID_PAD2_BUTTON2, 
	ASURA_KEYID_PAD2_BUTTON3, 
	ASURA_KEYID_PAD2_BUTTON4, 
	ASURA_KEYID_PAD2_BUTTON5, 
	ASURA_KEYID_PAD2_BUTTON6, 
	ASURA_KEYID_PAD2_BUTTON7, 
	ASURA_KEYID_PAD2_BUTTON8, 
	ASURA_KEYID_PAD2_BUTTON9, 
	ASURA_KEYID_PAD2_BUTTON10,
	ASURA_KEYID_PAD2_BUTTON11,
	ASURA_KEYID_PAD2_BUTTON12,
	ASURA_KEYID_PAD2_BUTTON13,
	ASURA_KEYID_PAD2_BUTTON14,
	ASURA_KEYID_PAD2_BUTTON15,
	ASURA_KEYID_PAD2_BUTTON16,

	ASURA_KEYID_PAD3_BUTTON1, 
	ASURA_KEYID_PAD3_BUTTON2, 
	ASURA_KEYID_PAD3_BUTTON3, 
	ASURA_KEYID_PAD3_BUTTON4, 
	ASURA_KEYID_PAD3_BUTTON5, 
	ASURA_KEYID_PAD3_BUTTON6, 
	ASURA_KEYID_PAD3_BUTTON7, 
	ASURA_KEYID_PAD3_BUTTON8, 
	ASURA_KEYID_PAD3_BUTTON9, 
	ASURA_KEYID_PAD3_BUTTON10,
	ASURA_KEYID_PAD3_BUTTON11,
	ASURA_KEYID_PAD3_BUTTON12,
	ASURA_KEYID_PAD3_BUTTON13,
	ASURA_KEYID_PAD3_BUTTON14,
	ASURA_KEYID_PAD3_BUTTON15,
	ASURA_KEYID_PAD3_BUTTON16,
	
	ASURA_KEYID_PAD4_BUTTON1, 
	ASURA_KEYID_PAD4_BUTTON2, 
	ASURA_KEYID_PAD4_BUTTON3, 
	ASURA_KEYID_PAD4_BUTTON4, 
	ASURA_KEYID_PAD4_BUTTON5, 
	ASURA_KEYID_PAD4_BUTTON6, 
	ASURA_KEYID_PAD4_BUTTON7, 
	ASURA_KEYID_PAD4_BUTTON8, 
	ASURA_KEYID_PAD4_BUTTON9, 
	ASURA_KEYID_PAD4_BUTTON10,
	ASURA_KEYID_PAD4_BUTTON11,
	ASURA_KEYID_PAD4_BUTTON12,
	ASURA_KEYID_PAD4_BUTTON13,
	ASURA_KEYID_PAD4_BUTTON14,
	ASURA_KEYID_PAD4_BUTTON15,
	ASURA_KEYID_PAD4_BUTTON16,

    ASURA_KEYID_LIGHTGUN1_TRIGGER,
    ASURA_KEYID_LIGHTGUN1_BUTTON1,
    ASURA_KEYID_LIGHTGUN1_BUTTON2,
    ASURA_KEYID_LIGHTGUN1_BUTTON3,
    ASURA_KEYID_LIGHTGUN1_BUTTON4,
    ASURA_KEYID_LIGHTGUN1_BUTTON5,
    ASURA_KEYID_LIGHTGUN1_BUTTON6,
    ASURA_KEYID_LIGHTGUN1_BUTTON7,
    ASURA_KEYID_LIGHTGUN1_BUTTON8,
    ASURA_KEYID_LIGHTGUN1_BUTTON9,

    ASURA_KEYID_LIGHTGUN2_TRIGGER,
    ASURA_KEYID_LIGHTGUN2_BUTTON1,
    ASURA_KEYID_LIGHTGUN2_BUTTON2,
    ASURA_KEYID_LIGHTGUN2_BUTTON3,
    ASURA_KEYID_LIGHTGUN2_BUTTON4,
    ASURA_KEYID_LIGHTGUN2_BUTTON5,
    ASURA_KEYID_LIGHTGUN2_BUTTON6,
    ASURA_KEYID_LIGHTGUN2_BUTTON7,
    ASURA_KEYID_LIGHTGUN2_BUTTON8,
    ASURA_KEYID_LIGHTGUN2_BUTTON9,

	MAX_NUMBER_OF_ASURA_KEYIDS,

};

enum ASURA_AXIS_ID
{
	ASURA_AXIS_VOID, 

	ASURA_AXIS_MOUSE_H,
	ASURA_AXIS_MOUSE_V,

	ASURA_AXIS_PAD1_LSTICK_H,
	ASURA_AXIS_PAD1_LSTICK_V,
	
	ASURA_AXIS_PAD1_RSTICK_H,
	ASURA_AXIS_PAD1_RSTICK_V,
	
	ASURA_AXIS_PAD2_LSTICK_H,
	ASURA_AXIS_PAD2_LSTICK_V,
	
	ASURA_AXIS_PAD2_RSTICK_H,
	ASURA_AXIS_PAD2_RSTICK_V,

	ASURA_AXIS_PAD3_LSTICK_H,
	ASURA_AXIS_PAD3_LSTICK_V,
	
	ASURA_AXIS_PAD3_RSTICK_H,
	ASURA_AXIS_PAD3_RSTICK_V,
	
	ASURA_AXIS_PAD4_LSTICK_H,
	ASURA_AXIS_PAD4_LSTICK_V,
	
	ASURA_AXIS_PAD4_RSTICK_H,
	ASURA_AXIS_PAD4_RSTICK_V,

    ASURA_AXIS_LIGHTGUN1_H,
    ASURA_AXIS_LIGHTGUN1_V,

    ASURA_AXIS_LIGHTGUN2_H,
    ASURA_AXIS_LIGHTGUN2_V,

	ASURA_AXIS_PAD1_Z,

	ASURA_AXIS_PAD1_ROTATE_X,
	ASURA_AXIS_PAD1_ROTATE_Y,
	ASURA_AXIS_PAD1_ROTATE_Z,

	ASURA_AXIS_PAD2_Z,

	ASURA_AXIS_PAD2_ROTATE_X,
	ASURA_AXIS_PAD2_ROTATE_Y,
	ASURA_AXIS_PAD2_ROTATE_Z,

	ASURA_AXIS_PAD3_Z,

	ASURA_AXIS_PAD3_ROTATE_X,
	ASURA_AXIS_PAD3_ROTATE_Y,
	ASURA_AXIS_PAD3_ROTATE_Z,

	ASURA_AXIS_PAD4_Z,

	ASURA_AXIS_PAD4_ROTATE_X,
	ASURA_AXIS_PAD4_ROTATE_Y,
	ASURA_AXIS_PAD4_ROTATE_Z,

	ASURA_MAX_NUMBER_OF_AXES,
};


class Asura_UserInput
{
	public: // all functions are implemented on a platform-specific basis.
	
		static void Initialise(void);
		static void Shutdown(void);
		static void Reset(void);
        static bool IsInitialised() { return s_bInitialised; }
	
		static void Update(void); // Call once a frame
		
		static bool HasAnyKeyBeenPressed( void );
		static bool HasAnyAxisBeenMoved( void );
        static bool DebouncedHasAnyKeyBeenPressed( void );

        static inline bool KeyPressed( int i ) { return( i < MAX_NUMBER_OF_ASURA_KEYIDS ? (0 != CurrentInput[i]) : false ); }
        static inline bool DebouncedKeyPressed( int i ) { return( i < MAX_NUMBER_OF_ASURA_KEYIDS ? (CurrentInput[i] && !PreviousInput[i]) : false ); }
		static inline bool DebouncedKeyNotPressed( int i ) { return( i < MAX_NUMBER_OF_ASURA_KEYIDS ? (!CurrentInput[i] && PreviousInput[i]) : false ); }

        // Returns value in range [0, 255].
        static inline u_char GetKeyValue( int i ) { return CurrentInput[i]; }
        
        // Return value is generally in range [-1, +1].
        static inline float GetAxisValue( int iAxis ) { return AxisValue[iAxis]; }
        static inline int GetNumberOfActiveControllers(bool bConsecutive) 
        { 
            return Platform_GetNumberOfActiveControllers(bConsecutive); 
        }
		static inline bool IsControllerActive(int iPlayerNum)
		{
			return Platform_IsControllerActive(iPlayerNum);
		}
		static inline int GetPortForPlayer(int iPlayerNum)
		{
			return Platform_GetPortForPlayer(iPlayerNum);
		}

        static inline bool IsKeyToggled(int iKey) {return Platform_IsKeyToggled(iKey);}

        static void UpdateInputBuffer(Asura_UnicodeChar* wszBuffer, u_int uMaxChars, 
            u_int& uInputCursorPos, bool (*pfnHandleInputOverride)(ASURA_KEYID eKey), 
            const Asura_UnicodeChar* wszInputBuffer = s_wszInputBuffer,
            u_int uInputBufferSize = s_uInputBufferSize);

	private:
		static void Platform_Initialise(void);
		static void Platform_Shutdown(void);
		static void Platform_Reset(void);
		static void Platform_Update(void); // Call once a frame
        static int  Platform_GetNumberOfActiveControllers(bool bConsecutive);
		static bool Platform_IsControllerActive(int iPlayerNum);
		static int  Platform_GetPortForPlayer(int iPlayerNum);
		static void Platform_UpdateInputBuffer(Asura_UnicodeChar* wszBuffer, u_int uMaxChars, u_int& uInputCursorPos,
											   bool (*pfnHandleInputOverride)(ASURA_KEYID eKey), 
											   const Asura_UnicodeChar* wszInputBuffer = s_wszInputBuffer,
											   u_int uInputBufferSize = s_uInputBufferSize);

		static void PlatformProject_Initialise(void);
		static void PlatformProject_Shutdown(void);
		static void PlatformProject_Reset(void);
		static void PlatformProject_Update(void); // Call once a frame

        static bool Platform_IsKeyToggled(int iKey);

		static void HandleKeyInput( ASURA_KEYID eKey, Asura_UnicodeChar* wszBuffer, u_int uMaxChars, u_int& uInputCursorPos );
			
		static unsigned char	PreviousInput[MAX_NUMBER_OF_ASURA_KEYIDS];		
		static unsigned char	CurrentInput[MAX_NUMBER_OF_ASURA_KEYIDS];		
		static bool				AnyKeyPressedHasBeenEvaluated;
		static bool				AnyKeyHasBeenPressed;
        static bool             s_bDebouncedAnyKeyPressedHasBeenEvaluated;
        static bool             s_bDebouncedAnyKeyHasBeenPressed;
		static float			AxisValue[ASURA_MAX_NUMBER_OF_AXES];
        static bool             s_bInitialised;
		static bool				s_bAnyAxisMovedHasBeenEvaluated;
		static bool				s_bAnyAxisHasBeenMoved;

        // set in platform update code
        static u_int            s_uInputBufferSize;
        static const Asura_UnicodeChar* s_wszInputBuffer;
};

#endif//__ASURA_USERINPUT_H__
