
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Handler_System.cpp
// Author : Andy Tate
// Date   : 29-Jan-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_MULTIPLE_HANDLERS)

// This file's header...
#include "Core/Handlers/Asura_Handler_System.h"

// Asura independent header-files...
#include "FileAccess/Asura_DebugFileStream.h"
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
#include "Interface/Asura_CommandConsole.h"
#endif	//defined(ASURA_SUPPORT_COMMAND_CONSOLE)

// Tools header-files...
#if defined(ASURA_TOOLS)
#include "Streaming/Tools_Streaming_System.h"
#endif


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID	Asura_Handler_System::s_uLastHandlerSet	= ASURA_HASH_ID_UNSET;

#if defined(ASURA_USE_STREAMING)
Asura_Hash_ID	Asura_Handler_System::s_uLastSetAllZoneDataHash		= ASURA_HASH_ID_UNSET;
u_int			Asura_Handler_System::s_uLastSetAllZoneDataIndex	= 0;
#endif	//defined(ASURA_USE_STREAMING)

#if defined(ASURA_LOGFILES_ENABLED)
Asura_TemplateHashTree<Asura_String>	Asura_Handler_System::s_xHandlerNames;
#endif	// defined(ASURA_LOGFILES_ENABLED)

Asura_Handler_System_Callbacks_Independent	Asura_Handler_System::s_xIndependentCallbacks;

Asura_Handler_System_Callbacks*	Asura_Handler_System::s_pxPlatform_Callbacks		= NULL;
Asura_Handler_System_Callbacks*	Asura_Handler_System::s_pxProject_Callbacks			= NULL;
Asura_Handler_System_Callbacks*	Asura_Handler_System::s_pxPlatformProject_Callbacks	= NULL;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::CreateHandlerData
// Author       : Andy Tate
// Date         : 20-Mar-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System::CreateHandlerData(u_int& uHandlerIndex,
											 const Asura_Hash_ID uHandlerHash,
											 const u_int uDataIndex,
											 const char* szName,
											 const Asura_Hash_ID uParentHandlerHash)
{
#if defined(ASURA_LOGFILES_ENABLED)
	bool bDidExist = true;
	Asura_String* pxString = s_xHandlerNames.AddOrFindNode( uHandlerHash, &bDidExist );
	if (!bDidExist) *pxString = szName;
#endif

	const u_int	uHandlerGroup	= GetGroup(uHandlerIndex);

	switch(uHandlerGroup)
	{
		case INDEPENDENT_HANDLER_GROUP:
		{
			if(!s_xIndependentCallbacks.CreateHandlerData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash))
			{
				uHandlerIndex	= PLATFORM_HANDLER_GROUP;
			}
			return(true);
		}

		case PLATFORM_HANDLER_GROUP:
		{
			if( (s_pxPlatform_Callbacks == NULL) ||
				 !s_pxPlatform_Callbacks->CreateHandlerData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash) )
			{
				uHandlerIndex	= PROJECT_HANDLER_GROUP;
			}
			return(true);
		}

		case PROJECT_HANDLER_GROUP:
		{
			if( (s_pxProject_Callbacks == NULL) ||
				 !s_pxProject_Callbacks->CreateHandlerData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash) )
			{
				uHandlerIndex	= PLATFORMPROJECT_HANDLER_GROUP;
			}
			return(true);
		}

		case PLATFORMPROJECT_HANDLER_GROUP:
		{
			if( (s_pxPlatformProject_Callbacks == NULL) ||
				 !s_pxPlatformProject_Callbacks->CreateHandlerData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash) )
			{
				// That's - we've finished handler creation...
				return(false);
			}
			else
			{
				return(true);
			}
		}

		default:
		{
			Asura_Assert(false, "Unexpected group for sub-system handler to be created.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System::CreateHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::DestroyHandlerData
// Author       : Andy Tate
// Date         : 20-Mar-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System::DestroyHandlerData(u_int& uHandlerIndex,
											  const Asura_Hash_ID uHandlerHash,
											  const u_int uDataIndex)
{
#if defined(ASURA_LOGFILES_ENABLED)
	s_xHandlerNames.DeleteNodeByHash( uHandlerHash );
#endif

	const u_int	uHandlerGroup	= GetGroup(uHandlerIndex);

	// RDM - 03/02/08 - The order of this switch is now the opposite to the creation order
	switch(uHandlerGroup)
	{
		case PLATFORMPROJECT_HANDLER_GROUP:
		{
			if( (s_pxPlatformProject_Callbacks == NULL) ||
				!s_pxPlatformProject_Callbacks->DestroyHandlerData(uHandlerIndex, uHandlerHash, uDataIndex) )
			{
				uHandlerIndex	= PROJECT_HANDLER_GROUP;
			}
			return true;
		}

		case PROJECT_HANDLER_GROUP:
		{
			if( (s_pxProject_Callbacks == NULL) ||
				!s_pxProject_Callbacks->DestroyHandlerData(uHandlerIndex, uHandlerHash, uDataIndex) )
			{
				uHandlerIndex	= PLATFORM_HANDLER_GROUP;
			}
			return(true);
		}

		case PLATFORM_HANDLER_GROUP:
		{
			if( (s_pxPlatform_Callbacks == NULL) ||
				!s_pxPlatform_Callbacks->DestroyHandlerData(uHandlerIndex, uHandlerHash, uDataIndex) )
			{
				uHandlerIndex	= INDEPENDENT_HANDLER_GROUP;
			}
			return(true);
		}

		case INDEPENDENT_HANDLER_GROUP:
		{
			if(!s_xIndependentCallbacks.DestroyHandlerData(uHandlerIndex, uHandlerHash, uDataIndex))
			{
				// That's - we've finished handler destruction...
				return(false);
			}
			else
			{
				return(true);
			}
		}

		default:
		{
			Asura_Assert(false, "Unexpected group for sub-system handler to be destroyed.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System::DestroyHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::CreateZoneData
// Author       : Andy Tate
// Date         : 20-Mar-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System::CreateZoneData(u_int& uHandlerIndex,
										  const Asura_Hash_ID uHandlerHash,
										  const u_int uDataIndex,
										  const char* szName,
										  const Asura_Hash_ID uParentHandlerHash,
										  const Asura_Hash_ID uZoneHash,
										  const bool bAllowEnvironment)
{
	const u_int	uHandlerGroup	= GetGroup(uHandlerIndex);

	switch(uHandlerGroup)
	{
		case INDEPENDENT_ZONE_GROUP:
		{
			if(!s_xIndependentCallbacks.CreateZoneData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash, uZoneHash, bAllowEnvironment))
			{
				uHandlerIndex	= PLATFORM_ZONE_GROUP;
			}
			return(true);
		}

		case PLATFORM_ZONE_GROUP:
		{
			if( (s_pxPlatform_Callbacks == NULL) ||
				 !s_pxPlatform_Callbacks->CreateZoneData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash, uZoneHash, bAllowEnvironment) )
			{
				uHandlerIndex	= PROJECT_ZONE_GROUP;
			}
			return(true);
		}

		case PROJECT_ZONE_GROUP:
		{
			if( (s_pxProject_Callbacks == NULL) ||
				 !s_pxProject_Callbacks->CreateZoneData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash, uZoneHash, bAllowEnvironment) )
			{
				uHandlerIndex	= PLATFORMPROJECT_ZONE_GROUP;
			}
			return(true);
		}

		case PLATFORMPROJECT_ZONE_GROUP:
		{
			if( (s_pxPlatformProject_Callbacks == NULL) ||
				 !s_pxPlatformProject_Callbacks->CreateZoneData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash, uZoneHash, bAllowEnvironment) )
			{
				// That's - we've finished zone creation...
				return(false);
			}
			else
			{
				return(true);
			}
		}

		default:
		{
			if(CreateHandlerData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash))
			{
				return(true);
			}
			else
			{
				// We've created the handlers, now for the zones...
				uHandlerIndex	= INDEPENDENT_ZONE_GROUP;
				return(true);
			}
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System::CreateZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::DestroyZoneData
// Author       : Andy Tate
// Date         : 20-Mar-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System::DestroyZoneData(u_int& uHandlerIndex,
										   const Asura_Hash_ID uHandlerHash,
										   const u_int uDataIndex,
										   const bool bAllowEnvironment)
{
	const u_int	uHandlerGroup	= GetGroup(uHandlerIndex);

	// RDM - 29/02/08 - The order of this switch is now the opposite to the creation order
	switch(uHandlerGroup)
	{
		case PLATFORMPROJECT_ZONE_GROUP:
		{
			if( (s_pxPlatformProject_Callbacks == NULL) ||
				!s_pxPlatformProject_Callbacks->DestroyZoneData(uHandlerIndex, uHandlerHash, uDataIndex, bAllowEnvironment) )
			{
				uHandlerIndex	= PROJECT_ZONE_GROUP;
			}
			return(true);
		}		

		case PROJECT_ZONE_GROUP:
		{
			if( (s_pxProject_Callbacks == NULL) ||
				!s_pxProject_Callbacks->DestroyZoneData(uHandlerIndex, uHandlerHash, uDataIndex, bAllowEnvironment) )
			{
				uHandlerIndex	= PLATFORM_ZONE_GROUP;
			}
			return(true);
		}

		case PLATFORM_ZONE_GROUP:
		{
			if( (s_pxPlatform_Callbacks == NULL) ||
				!s_pxPlatform_Callbacks->DestroyZoneData(uHandlerIndex, uHandlerHash, uDataIndex, bAllowEnvironment) )
			{
				uHandlerIndex	= INDEPENDENT_ZONE_GROUP;
			}
			return(true);
		}	

		case INDEPENDENT_ZONE_GROUP:
		{
			if(!s_xIndependentCallbacks.DestroyZoneData(uHandlerIndex, uHandlerHash, uDataIndex, bAllowEnvironment))
			{
				// all zones done, now deal with the handlers
				uHandlerIndex	= PLATFORMPROJECT_HANDLER_GROUP;
			}
			return(true);
		}

		default:
		{
			if(DestroyHandlerData(uHandlerIndex, uHandlerHash, uDataIndex))
			{
				return(true);
			}
			else
			{
				// We've destroyed all the handlers, we're done...
				return(false);
			}
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System::DestroyZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::ReparentAllHandlerData
// Author       : Andy Tate
// Date         : 29-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System::ReparentAllHandlerData(const Asura_Hash_ID uHandlerHash,
												  const Asura_Hash_ID uParentHandlerHash)
{
	s_xIndependentCallbacks.ReparentAllHandlerData(uHandlerHash, uParentHandlerHash);

	if(s_pxPlatform_Callbacks != NULL)
	{
		s_pxPlatform_Callbacks->ReparentAllHandlerData(uHandlerHash, uParentHandlerHash);
	}

	if(s_pxProject_Callbacks != NULL)
	{
		s_pxProject_Callbacks->ReparentAllHandlerData(uHandlerHash, uParentHandlerHash);
	}

	if(s_pxPlatformProject_Callbacks != NULL)
	{
		s_pxPlatformProject_Callbacks->ReparentAllHandlerData(uHandlerHash, uParentHandlerHash);
	}
}

/// End of function 'Asura_Handler_System::ReparentAllHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::SetAllHandlerData
// Author       : Andy Tate
// Date         : 20-Mar-2006
//
// Description  : Sets the appropriate handler-data with the specified hash-ID.
//
// Parameters   : The streaming-system index and the corresponding hash-ID.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System::SetAllHandlerData(const Asura_Hash_ID uHandlerHash,
											 const u_int uDataIndex)
{
	// Store this handler...
	s_uLastHandlerSet	= uHandlerHash;

	// Do we have a valid hash?
	if(uHandlerHash != ASURA_HASH_ID_UNSET)
	{
		if(s_pxPlatform_Callbacks != NULL)
		{
			s_pxPlatform_Callbacks->SetAllHandlerData(uHandlerHash, uDataIndex);
		}

		s_xIndependentCallbacks.SetAllHandlerData(uHandlerHash, uDataIndex);

		if(s_pxProject_Callbacks != NULL)
		{
			s_pxProject_Callbacks->SetAllHandlerData(uHandlerHash, uDataIndex);
		}

		if(s_pxPlatformProject_Callbacks != NULL)
		{
			s_pxPlatformProject_Callbacks->SetAllHandlerData(uHandlerHash, uDataIndex);
		}
	}
}

/// End of function 'Asura_Handler_System::SetAllHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::ReparentAllZoneData
// Author       : Andy Tate
// Date         : 29-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System::ReparentAllZoneData(const Asura_Hash_ID uHandlerHash,
											   const Asura_Hash_ID uParentHandlerHash)
{
	s_xIndependentCallbacks.ReparentAllZoneData(uHandlerHash, uParentHandlerHash);

	if(s_pxPlatform_Callbacks != NULL)
	{
		s_pxPlatform_Callbacks->ReparentAllZoneData(uHandlerHash, uParentHandlerHash);
	}

	if(s_pxProject_Callbacks != NULL)
	{
		s_pxProject_Callbacks->ReparentAllZoneData(uHandlerHash, uParentHandlerHash);
	}

	if(s_pxPlatformProject_Callbacks != NULL)
	{
		s_pxPlatformProject_Callbacks->ReparentAllZoneData(uHandlerHash, uParentHandlerHash);
	}

	// Re-parent the remaining data...
	ReparentAllHandlerData(uHandlerHash, uParentHandlerHash);
}

/// End of function 'Asura_Handler_System::ReparentAllZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::SetAllZoneData
// Author       : Andy Tate
// Date         : 20-Mar-2006
//
// Description  : Sets the appropriate zone with the specified hash-ID.
//
// Parameters   : The streaming-system index and the corresponding hash-ID.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System::SetAllZoneData(const Asura_Hash_ID uHandlerHash,
										  const u_int uDataIndex,
										  const bool bAllowEnvironment)
{
#if defined(ASURA_USE_STREAMING)
	// Save the last hash/data-index used for SetAllZoneData.
	s_uLastSetAllZoneDataHash  = uHandlerHash;
	s_uLastSetAllZoneDataIndex = uDataIndex;
#endif	// defined(ASURA_USE_STREAMING)

	// Do we have a valid hash?
	if(uHandlerHash != ASURA_HASH_ID_UNSET)
	{
		if(s_pxPlatform_Callbacks != NULL)
		{
			s_pxPlatform_Callbacks->SetAllZoneData(uHandlerHash, uDataIndex, bAllowEnvironment);
		}

		s_xIndependentCallbacks.SetAllZoneData(uHandlerHash, uDataIndex, bAllowEnvironment);

		if(s_pxProject_Callbacks != NULL)
		{
			s_pxProject_Callbacks->SetAllZoneData(uHandlerHash, uDataIndex, bAllowEnvironment);
		}

		if(s_pxPlatformProject_Callbacks != NULL)
		{
			s_pxPlatformProject_Callbacks->SetAllZoneData(uHandlerHash, uDataIndex, bAllowEnvironment);
		}

#if defined(ASURA_TOOLS)
		// Tools data
		Tools_Streaming_System::SetZoneData(uHandlerHash);
#endif	//defined(ASURA_TOOLS)

		// Set the remaining data...
		SetAllHandlerData(uHandlerHash, uDataIndex);
	}
}

/// End of function 'Asura_Handler_System::SetAllZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::Initialise
// Author       : Andy Tate
// Date         : 29-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System::Initialise(void)
{
#if defined(ASURA_TOOLS)
	Asura_Assert(false,"Handler system should not be active in tools mode");
#endif	//defined(ASURA_TOOLS)

#if !defined(ASURA_USE_STREAMING)
	Project_Initialise();
#endif	//!defined(ASURA_USE_STREAMING)

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE) && defined(ASURA_LOGFILES_ENABLED)
	Asura_CommandConsole::AddCmd(L"Debug.Handlers.CreateReport", CreateReport, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
#endif	//defined(ASURA_SUPPORT_COMMAND_CONSOLE) && defined(ASURA_LOGFILES_ENABLED)
}

/// End of function 'Asura_Handler_System::Initialise'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::Shutdown
// Author       : Andy Tate
// Date         : 29-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System::Shutdown(void)
{
#if !defined(ASURA_USE_STREAMING)
	Project_Shutdown();
#endif	//!defined(ASURA_USE_STREAMING)
}

/// End of function 'Asura_Handler_System::Shutdown'. ///


#if defined(ASURA_USE_STREAMING)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::SetHandlersForSortedRenderer
// Author       : Martin G. Harris
// Date         : 2-Jun-2006
////////////////////////////////////////////////////////////////////////////////
void Asura_Handler_System::SetHandlersForSortedRenderer(const Asura_Hash_ID uHandlerHash,
														const u_int uDataIndex,
														Asura_Hash_ID uContainingZoneHash)
{
	if (uHandlerHash != ASURA_HASH_ID_UNSET)
	{
		if(s_pxPlatform_Callbacks != NULL)
		{
			s_pxPlatform_Callbacks->SetAllHandlerData(uHandlerHash, uDataIndex);
		}
		if(s_pxPlatformProject_Callbacks != NULL)
		{
			s_pxPlatformProject_Callbacks->SetAllHandlerData(uHandlerHash, uDataIndex);
		}
	}

#if !defined(ASURA_TOOLS)		
	// MGH 2/6/2006: Need to set the GI solution handler too in non-tools builds.
	if (uContainingZoneHash == ASURA_HASH_ID_UNSET)
	{
		// Actual containing zone isn't known, revert to mission/zone hash.
		uContainingZoneHash = uHandlerHash;
	}
	if (uContainingZoneHash != ASURA_HASH_ID_UNSET)
	{
		Asura_GI_Solution_Handler& xOldHandler = Asura_GI_Solution::GetCurrentHandler();
		Asura_GI_Solution::SetCurrentHandler(uContainingZoneHash);
		if (&Asura_GI_Solution::GetCurrentHandler() == 0)
		{
			// Probably a mission entity with unknown containing zone.
			Asura_Assert(0, "Couldn't set GI solution handler");
			Asura_GI_Solution::SetCurrentHandler(&xOldHandler);
		}
	}
#endif //!defined(ASURA_TOOLS)
}

#endif	//defined(ASURA_USE_STREAMING)


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::CreateAllHandlerData
// Author       : Andy Tate
// Date         : 8-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System::CreateAllHandlerData(const Asura_Hash_ID uHandlerHash,
												const u_int uDataIndex,
												const char* szName,
												const Asura_Hash_ID uParentHandlerHash)
{
	u_int	uHandlerIndex	= INDEPENDENT_HANDLER_GROUP;

	// Create ALL zone handlers...
	while(CreateHandlerData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash));
}

/// End of function 'Asura_Handler_System::CreateAllHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::DestroyAllHandlerData
// Author       : Andy Tate
// Date         : 8-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System::DestroyAllHandlerData(const Asura_Hash_ID uHandlerHash,
												 const u_int uDataIndex)
{
	u_int	uHandlerIndex	= PLATFORMPROJECT_HANDLER_GROUP;

	// Destroy ALL zone handlers...
	while(DestroyHandlerData(uHandlerIndex, uHandlerHash, uDataIndex));
}

/// End of function 'Asura_Handler_System::DestroyAllHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::CreateAllZoneData
// Author       : Andy Tate
// Date         : 8-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System::CreateAllZoneData(const Asura_Hash_ID uHandlerHash,
											 const u_int uDataIndex,
											 const char* szName,
											 const Asura_Hash_ID uParentHandlerHash,
											 const Asura_Hash_ID uZoneHash,
											 const bool bAllowEnvironment)
{
	u_int	uHandlerIndex	= INDEPENDENT_HANDLER_GROUP;

	// Create ALL zone handlers...
	while(CreateZoneData(uHandlerIndex, uHandlerHash, uDataIndex, szName, uParentHandlerHash, uZoneHash, bAllowEnvironment));

#if defined(ASURA_TOOLS) && defined(ASURA_USE_STREAMING)
	// Tools data...
	Tools_Streaming_System::CreateZoneData(uHandlerHash);
#endif	//defined(ASURA_TOOLS) && defined(ASURA_USE_STREAMING)
}

/// End of function 'Asura_Handler_System::CreateAllZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::DestroyAllZoneData
// Author       : Andy Tate
// Date         : 8-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System::DestroyAllZoneData(const Asura_Hash_ID uHandlerHash,
											  const u_int uDataIndex,
											  const bool bAllowEnvironment)
{
	// Destruction of zones/handlers is done in reverse order to creation, zones first, then handlers.
	u_int	uHandlerIndex	= PLATFORMPROJECT_ZONE_GROUP;

#if defined(ASURA_TOOLS) && defined(ASURA_USE_STREAMING)
	// Tools data...
	Tools_Streaming_System::DestroyZoneData(uHandlerHash);
#endif	//defined(ASURA_TOOLS) && defined(ASURA_USE_STREAMING)

	// Destroy ALL zone handlers...
	while(DestroyZoneData(uHandlerIndex, uHandlerHash, uDataIndex, bAllowEnvironment));
}

/// End of function 'Asura_Handler_System::DestroyAllZoneData'. ///


#if defined(ASURA_LOGFILES_ENABLED)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System::CreateReport
// Author       : Andy Tate
// Date         : 29-Jan-2008
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System::CreateReport(bool bHelp)
{
	if (bHelp)
	{
		Asura_CommandConsole::Printf( "Create a log of items in each handler" );
		return true;
	}

	Asura_DebugFileStream xLogStream;
	if (!xLogStream.OpenForWriting("LogFiles\\Handlers.txt", true))
	{
		Asura_Assert(false,"Couldn't open file for writing");
		return false;
	}

	const u_int uLOGBUFFERSIZE = 96;
	char szLogBuffer[uLOGBUFFERSIZE];

	xLogStream << "----------------------------------\n";
	xLogStream << "-  Asura_Handler_System Report   -\n";
	xLogStream << "----------------------------------\n\n";

	Asura_String* pxCurrentHandlerName = s_xHandlerNames.GetDataByHash(s_uLastHandlerSet);
	Asura_PrintfToBfr(szLogBuffer, "Current Handler: 0x%08x %s\n", uLOGBUFFERSIZE, true, s_uLastHandlerSet, (pxCurrentHandlerName) ? pxCurrentHandlerName->GetString() : "");
	xLogStream << szLogBuffer;

	xLogStream << '\n';

	Asura_TemplateHashTree_Iterator<Asura_String> xIt(s_xHandlerNames);
	for ( ; !xIt.Done(); xIt.Next())
	{
		Asura_Hash_ID uHash = xIt.GetCurrentHash();

		Asura_PrintfToBfr(szLogBuffer, "------------------\nHandler 0x%08x\n%s\n------------------\n", uLOGBUFFERSIZE, true, uHash, xIt.GetCurrent()->GetString());
		xLogStream << szLogBuffer;

		s_xIndependentCallbacks.CreateReport(uHash, xLogStream);

		if(s_pxPlatform_Callbacks != NULL)
		{
			s_pxPlatform_Callbacks->CreateReport(uHash, xLogStream);
		}

		if(s_pxProject_Callbacks != NULL)
		{
			s_pxPlatform_Callbacks->CreateReport(uHash, xLogStream);
		}

		if(s_pxPlatformProject_Callbacks != NULL)
		{
			s_pxPlatformProject_Callbacks->CreateReport(uHash, xLogStream);
		}

		xLogStream << '\n';
	}

	xLogStream.Close();
	return true;
}

/// End of function 'Asura_Handler_System::CreateReport'. ///

#endif //__#ifdef ASURA_LOGFILES_ENABLED__

#endif	//defined(ASURA_USE_MULTIPLE_HANDLERS)


/// End of file. ///
