
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Handler_System.h
// Author : Andy Tate
// Date   : 29-Jan-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_HANDLER_SYSTEM_H__
#define __ASURA_HANDLER_SYSTEM_H__

#if defined(ASURA_USE_MULTIPLE_HANDLERS)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Hash.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_String.h"
#include "Core/Handlers/Asura_Handler_System_Callbacks_Independent.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Stream;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Handler_System
// Author       : Andy Tate
// Date         : 31-Mar-2006
//
// Description  : Class to interface to the sub-system handlers for data.
////////////////////////////////////////////////////////////////////////////////

class Asura_Handler_System
{
public:
	// Public functions...
	static void	Initialise(void);
	static void	Shutdown(void);

	// Callback registration functions...
	static void	RegisterPlatformCallbacks(Asura_Handler_System_Callbacks* pxCallbacks)			{ s_pxPlatform_Callbacks = pxCallbacks; }
	static void	RegisterProjectCallbacks(Asura_Handler_System_Callbacks* pxCallbacks)			{ s_pxProject_Callbacks	= pxCallbacks; }
	static void	RegisterPlatformProjectCallbacks(Asura_Handler_System_Callbacks* pxCallbacks)	{ s_pxPlatformProject_Callbacks = pxCallbacks; }

	// Handler initialisation/destruction functions...
	static bool	CreateHandlerData(u_int& uHandlerIndex, const Asura_Hash_ID uHandlerHash, const u_int uDataIndex, const char* szName, const Asura_Hash_ID uParentHandlerHash);
	static bool	DestroyHandlerData(u_int& uHandlerIndex, const Asura_Hash_ID uHandlerHash, const u_int uDataIndex);

	static void	CreateAllHandlerData(const Asura_Hash_ID uHandlerHash, const u_int uDataIndex, const char* szName, const Asura_Hash_ID uParentHandlerHash);
	static void	DestroyAllHandlerData(const Asura_Hash_ID uHandlerHash, const u_int uDataIndex);

	static void CreateAllHandlerData(const Asura_Hash_ID uHandlerHash, const u_int uDataIndex, const Asura_String& xName, const Asura_Hash_ID uParentHandlerHash, const bool bAllowEnvironment)
	{
		CreateAllHandlerData(uHandlerHash, uDataIndex, xName.GetString(), uParentHandlerHash, bAllowEnvironment);
	}

	static void	ReparentAllHandlerData(const Asura_Hash_ID uHandlerHash, const Asura_Hash_ID uParentHandlerHash);
	static void	SetAllHandlerData(const Asura_Hash_ID uHandlerHash, const u_int uDataIndex);

	// Zone initialisation/destruction functions...
	static bool	CreateZoneData(u_int& uHandlerIndex, const Asura_Hash_ID uHandlerHash, const u_int uDataIndex, const char* szName, const Asura_Hash_ID uParentHandlerHash, const Asura_Hash_ID uZoneHash, const bool bAllowEnvironment);
	static bool	DestroyZoneData(u_int& uHandlerIndex, const Asura_Hash_ID uHandlerHash, const u_int uDataIndex, const bool bAllowEnvironment);

	static void	CreateAllZoneData(const Asura_Hash_ID uHandlerHash, const u_int uDataIndex, const char* szName, const Asura_Hash_ID uParentHandlerHash, const Asura_Hash_ID uZoneHash, const bool bAllowEnvironment);
	static void	DestroyAllZoneData(const Asura_Hash_ID uHandlerHash, const u_int uDataIndex, const bool bAllowEnvironment);

	static void CreateAllZoneData(const Asura_Hash_ID uHandlerHash, const u_int uDataIndex, const Asura_String& xName, const Asura_Hash_ID uParentHandlerHash, const Asura_Hash_ID uZoneHash, const bool bAllowEnvironment)
	{
		CreateAllZoneData(uHandlerHash, uDataIndex, xName.GetString(), uParentHandlerHash, uZoneHash, bAllowEnvironment);
	}

	static void	ReparentAllZoneData(const Asura_Hash_ID uHandlerHash, const Asura_Hash_ID uParentHandlerHash);
	static void	SetAllZoneData(const Asura_Hash_ID uHandlerHash, const u_int uDataIndex, const bool bAllowEnvironment);

#if defined(ASURA_LOGFILES_ENABLED)
	// Report creation...
	static bool CreateReport(bool bHelp = false);
#endif	//defined(ASURA_LOGFILES_ENABLED)

	// Accessor functions...
	static Asura_Hash_ID	GetLastHandlerSet(void)	{ return(s_uLastHandlerSet); }
	static u_int			GetIndexWithGroup(const u_int uHandlerIndex)	{ return(uHandlerIndex & 0x00000FFF); }

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	// Return value of hash passed in to last call to SetAllZoneData.
	static Asura_Hash_ID GetLastSetAllZoneDataHash()	{ return s_uLastSetAllZoneDataHash; }
	// Return value of DataIndex passed in to last call to SetAllZoneData.
	static u_int		 GetLastSetAllZoneDataIndex()	{ return s_uLastSetAllZoneDataIndex; }

	// Set the handlers required for the sorted render to work properly.
	static void SetHandlersForSortedRenderer(const Asura_Hash_ID uHandlerHash, const u_int uDataIndex, Asura_Hash_ID uContainingZoneHash);
#else
	static bool		GetResetGroup(const Asura_Hash_ID uHandlerHash, u_int& uResetGroup)	{ return(Project_GetResetGroup(uHandlerHash, uResetGroup)); }
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)


protected:
	// Protected enumerated types...
	enum Group
	{
		INDEPENDENT_HANDLER_GROUP		= 0x0000,
		PLATFORM_HANDLER_GROUP			= 0x1000,
		PROJECT_HANDLER_GROUP			= 0x2000,
		PLATFORMPROJECT_HANDLER_GROUP	= 0x3000,

		INDEPENDENT_ZONE_GROUP			= 0x4000,
		PLATFORM_ZONE_GROUP				= 0x5000,
		PROJECT_ZONE_GROUP				= 0x6000,
		PLATFORMPROJECT_ZONE_GROUP		= 0x7000,
	};

	// Protected variables...
	static Asura_Hash_ID								s_uLastHandlerSet;				// The hash-ID of the last handler set.

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	static Asura_Hash_ID								s_uLastSetAllZoneDataHash;
	static u_int										s_uLastSetAllZoneDataIndex;
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)

#if defined(ASURA_LOGFILES_ENABLED)
	static Asura_TemplateHashTree<Asura_String>			s_xHandlerNames;				// Tree of names of ALL of the handlers created.
#endif

	static Asura_Handler_System_Callbacks_Independent	s_xIndependentCallbacks;

	static Asura_Handler_System_Callbacks*				s_pxPlatform_Callbacks;			// A pointer to the platform-specific handler-system callbacks.
	static Asura_Handler_System_Callbacks*				s_pxProject_Callbacks;			// A pointer to the project-specific handler-system callbacks.
	static Asura_Handler_System_Callbacks*				s_pxPlatformProject_Callbacks;	// A pointer to the platform-project-specific handler-system callbacks.

	// Protected functions...
#if !defined(ASURA_USE_STREAMING)
	static void	Project_Initialise(void);
	static void	Project_Shutdown(void);
#endif	//!defined(ASURA_USE_STREAMING)

#if !defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	static bool Project_GetResetGroup(const Asura_Hash_ID uHandlerHash, u_int& uResetGroup);
#endif	//!defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)

	static Group	GetGroup(const u_int uHandlerIndex)	{ return(static_cast<Group>(uHandlerIndex & 0xFFFFF000)); }


private:
	// Private variables...

	// Private functions...
	Asura_Handler_System(void);
	Asura_Handler_System(const Asura_Handler_System& xOther);
	~Asura_Handler_System();

	Asura_Handler_System&	operator=(const Asura_Handler_System& xOther);

};   // End of class 'Asura_Handler_System'.

#endif	//defined(ASURA_USE_MULTIPLE_HANDLERS)

#endif	// ifndef __ASURA_HANDLER_SYSTEM_H__


/// End of file. ///
