//
// Asura_Task_Queue.cpp
//
// Created by KJFL - 5 March 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_USE_TASK_SYSTEM

// This file's header.
#include "Core/Threads/Asura_Task_Queue.h"

#include "Core/Threads/Asura_Task_System.h"

Asura_Task_Queue::Asura_Task_Queue( Asura_TaskFunction pfnTask, Asura_Task_Queue_Callback pfnQueueCallback )
: m_bTerminateTasks( false )
, m_pfnQueueCallback( pfnQueueCallback )
, Asura_Task( pfnTask )
{
	// Tell the system about this task
	Asura_Task_System::AddTask( this );

	// subtasks can start straight away
	SignalWorkIsAvailable();
}

Asura_Task_Queue::~Asura_Task_Queue()
{
	m_bTerminateTasks = true;

	// No locks required here since nothing internally signals work is available 
	// apart from the constructor.
	SignalWorkIsNotAvailable();

	// Ask to be removed from the system - this will not return until any in-progress
	// tasks have completed.
	Asura_Task_System::RemoveTask( this );

	// now it's safe to delete things

}

void* Asura_Task_Queue::GetTaskData( void )
{
	if ( m_bTerminateTasks ) return 0;

    // Try to get something from the Queue. Return 0 if nothing is available.
	void* pData = m_pfnQueueCallback();

	if ( !pData )
	{
		// We've ran out of work. Tell the worker threads to stop asking for new tasks.
		SignalWorkIsNotAvailable();
	}
	else
	{
		// Assume another task
		SignalWorkIsAvailable(); 
	}

	return pData;
}


#endif // #ifdef ASURA_USE_TASK_SYSTEM

