//
// Asura_Task_Stack.h
//
// Created by KJFL - 17 January 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_TASK_STACK_H__
#define __ASURA_TASK_STACK_H__

#ifdef ASURA_USE_TASK_SYSTEM

#include "Core/Threads/Asura_Task_System.h"

#include "Core/Asura_Multithread.h" // for mutex

// KJFL - 16 January 2008 - this Asura_Task_Stack operates in a First-in, last-out order (FILO).
// This is useful for situations like making a tree, where worker threads will be adding items to the 
// Stack as they go. If we were to operate in a FIFO order, we'd quickly fill the Stack if there are
// many tree nodes. Using FILO means that you will reach the leaves of the tree quicker, and then go 
// back to fill in the tree.
// 
class Asura_Task_Stack : public Asura_Task
{
	public:	
		Asura_Task_Stack( const char* szStackName, u_int uMaxStackSize, Asura_TaskFunction pfnTask );
		~Asura_Task_Stack();

		// Get something from the Stack - called by worker threads.
		void* GetTaskData( void );

		// Try to add something to the Stack. If the Stack is full, it will immediately return false.
		bool AddToStack_NonBlocking( void* pStackData );

		// Add something to the Stack, waiting if the Stack is full.
		void AddToStack( void* pStackData );

		// Returns the max size of the Stack, as passed into the constructor.
		u_int GetMaxStackSize( void ) const;
		
		// Returns the current size of the Stack - by design this does no locks etc,
		// so it's only useful for external routines trying to gauge fullness/progress.
		u_int GetCurrentStackSize( void ) const;

		// Called by worker threads if they are working on tasks they've generated 
		// themselves (rather than getting them from GetTaskData) and they need
		// to now whether time has been called.
		bool GetShouldTerminateTasks( void ) const { return m_bTerminateTasks; }

	private:

		// shared part of the AddToStack methods, once the Stack lock is active
		void AddToStack_PostLock( void* pData );

		const char* m_szStackName;

        u_int m_uMaxStackSize;
		u_int m_uCurrentStackSize;
	
		void** m_ppTaskStack;

		Asura_Semaphore*	m_pxStackSpace;

		bool				m_bTerminateTasks;

		// TODO: may be able to get rid of this mutex
		Asura_Mutex			m_xStackAccessMutex;

};

#endif //#ifdef ASURA_USE_TASK_SYSTEM

#endif//__ASURA_TASK_STACK_H__

