//
// Asura_Cutscene_Attachment.h
//
// Created by Richard Rice - 21/10/02
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_CUTSCENE_ATTACHMENT_H__
#define __ASURA_CUTSCENE_ATTACHMENT_H__

#include "Core/Asura_String.h"
#include "Cutscene/Asura_Cutscene_Actor.h"
#include "Shapes/Asura_Shapes.h"

#ifdef ASURA_TOOLS
#include "Misc/list_tem.hpp"
#endif

class Asura_Chunk_Stream;
class Asura_Cutscene_Actor;
class Asura_Cutscene_Actor_Hierarchy;


enum Asura_Cutscene_Attachment_Type
{
	AsuraCutsceneAttachment_Hierarchy,
	AsuraCutsceneAttachment_Shape,
	AsuraCutsceneAttachment_Actor,
	AsuraCutsceneAttachment_Entity,
	AsuraCutsceneAttachment_Corona,
	AsuraCutsceneAttachment_PFX,
	AsuraCutsceneAttachment_SoundEvent,
    
    AsuraCutsceneAttachment_FirstProjectSpecific = 1000,
};

#define AsuraCutsceneAttachmentFlag_InitiallyActive 0x00000001

///////////////////////////////////////////////////////////////////////////////////////////////

class Asura_Cutscene_Attachment
{
public :
	Asura_Cutscene_Attachment(Asura_Cutscene_Actor_Hierarchy* pxActor);
	virtual ~Asura_Cutscene_Attachment();

	virtual int GetType() const = 0;
	virtual void Initialise();
	virtual void Finish();
	virtual void Update(float fFrameTime){}
	virtual void Render(){}
	virtual void RenderShadow(){}
	virtual void PreDestructionCleanup(){};
	void Activate(bool bActive);
	bool IsActive() const {return m_bActive;}	

	//Invalidates frame stamps on any hierarchy attachments.
	virtual void InvalidateBonesUpdateFrameStamp(){};

	#ifdef ASURA_TOOLS
	virtual void Copy(Asura_Cutscene_Attachment* pxAttachment);
	virtual void ListShapesUsed(List<Asura_Hash_ID>& xShapeList){};
	virtual void BuildUsedTextureTree(Asura_NameTree* pxNameTree){}
	//called during level export to add extra asura files to export list
	virtual void AddExtraFilesToList( List<Asura_String>& xNameList ) const {}

    // (MH - 05.11.2002) code for prining unused skins and anims
    virtual bool UsesSkin(int iSkinID) const {return false;}
    virtual bool UsesAnim(int iAnimID) const {return false;}
	
	//Should return true if the passed actor is directly dependent on this one
	//(for sorting out ordering of actor array)
	virtual bool NeedsUpdateBefore(const Asura_Cutscene_Actor* pxActor) const {return false;}
	#endif

	static Asura_Cutscene_Attachment* AllocateAttachment(Asura_Cutscene_Actor_Hierarchy* pxActor,int iType);

	Asura_Cutscene* GetCutscene(){return m_pxActor->Cutscene;}
	
	bool GetPosition(Asura_Vector_3& xPos,Asura_Matrix_3x3& xMat);
	void SetMarkerName(Asura_String& xMarkerName);

	void SetName(const char* szName) {m_xName = szName;}

	u_int GetID() const {return m_uID;}

	bool IsPlayingOnServer();

	const Asura_Animation*	GetAnimation() const	{ return m_pxAnimation; }
	Asura_Animation*		GetAnimation()			{ return m_pxAnimation; }

	//Load/save functions , defined in Asura_Chunk_Cutscene.cpp
	virtual int GetChunkSize();
	virtual void SaveToChunk(char*& pcChunkBuffer);
	virtual void LoadFromChunk(const class Asura_Chunk_CutsceneAttachment* pxChunk,Asura_Handle iFileHandle);
//	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	#ifdef ASURA_TOOLS
	virtual void ResetGuidsAndMessages(){};
	#endif	

protected:

	virtual void HandleActiveStatusChange(){};

	Asura_Cutscene_Actor_Hierarchy* m_pxActor;
	Asura_String m_xMarkerName; //marker point on hierarchy to attach this to
	u_int m_uFlags;

	Asura_String m_xName; //optional , for helping identify this
	u_int m_uID;//id for identifcation by events

	//used during playback
	bool m_bActive;
	Asura_Animation* m_pxAnimation;

private:

	friend class Tools_Cutscene_Attachment;

    static Asura_Cutscene_Attachment* Project_AllocateAttachment( Asura_Cutscene_Actor_Hierarchy* pxParentActor, int iType );

};

///////////////////////////////////////////////////////////////////////////////////////////////

class Asura_Cutscene_Attachment_Hierarchy : public Asura_Cutscene_Attachment
{
public :
	Asura_Cutscene_Attachment_Hierarchy(Asura_Cutscene_Actor_Hierarchy* pxActor);
	~Asura_Cutscene_Attachment_Hierarchy();
	
	virtual int GetType() const {return AsuraCutsceneAttachment_Hierarchy;}
	virtual void Initialise();
	virtual void Finish();
	virtual void PreDestructionCleanup();
	virtual void InvalidateBonesUpdateFrameStamp();

	#ifdef ASURA_TOOLS
	void Copy(Asura_Cutscene_Attachment* pxAttachment);
    bool UsesSkin(int iSkinID) const;
    bool UsesAnim(int iAnimID) const {return (iAnimID == m_iAnimID);}
	//int  GetSkin(void) const { return m_iSkinID; }
	const char* GetSkinName(int iSkinIndex) const;
	#endif

	//Load/save functions , defined in Asura_Chunk_Cutscene.cpp
	virtual int GetChunkSize();
	virtual void SaveToChunk(char*& pcChunkBuffer);
	virtual void LoadFromChunk(const class Asura_Chunk_CutsceneAttachment* pxChunk,Asura_Handle iFileHandle);
//	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

protected :
	virtual void HandleActiveStatusChange();

	int m_aiSkinID[ASURA_ANIMATION_SKIN_COUNT];//indeces into the name trees in the cutscene
	int m_iAnimID;

	friend class Tools_Cutscene_Attachment_Hierarchy;

};

///////////////////////////////////////////////////////////////////////////////////////////////
class Asura_Cutscene_Attachment_Shape : public Asura_Cutscene_Attachment
{
public :
	Asura_Cutscene_Attachment_Shape(Asura_Cutscene_Actor_Hierarchy* pxActor);
	~Asura_Cutscene_Attachment_Shape();

	virtual void Initialise();
	virtual int GetType() const {return AsuraCutsceneAttachment_Shape;}
	virtual void Render();
	virtual void RenderShadow();

	#ifdef ASURA_TOOLS
	void Copy(Asura_Cutscene_Attachment* pxAttachment);
	virtual void ListShapesUsed(List<Asura_Hash_ID>& xShapeList);
	#endif

	//Load/save functions , defined in Asura_Chunk_Cutscene.cpp
	virtual int GetChunkSize();
	virtual void SaveToChunk(char*& pcChunkBuffer);
	virtual void LoadFromChunk(const class Asura_Chunk_CutsceneAttachment* pxChunk,Asura_Handle iFileHandle);
//	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	//NRM - 26/11/02 - Function to set shape so attachments can be created dynamically in games
	void SetShape(Asura_Hash_ID xShapeHashID);

protected :
	virtual void HandleActiveStatusChange();
	Asura_Shape_Instance m_xShapeInstance;

	friend class Tools_Cutscene_Attachment_Shape;

};

///////////////////////////////////////////////////////////////////////////////////////////////
class Asura_Cutscene_Attachment_Actor : public Asura_Cutscene_Attachment
{
public :
	Asura_Cutscene_Attachment_Actor(Asura_Cutscene_Actor_Hierarchy* pxActor);

	virtual int GetType() const {return AsuraCutsceneAttachment_Actor;}
	virtual void Initialise();
	virtual void Update(float fFrameTime);
	virtual void Render();
	virtual void RenderShadow();

	#ifdef ASURA_TOOLS
	void Copy(Asura_Cutscene_Attachment* pxAttachment);
	virtual bool NeedsUpdateBefore(const Asura_Cutscene_Actor* pxActor) const;

	const char* GetActorName() const {return m_xAttachedActorName.GetString();}
	void SetActorName(const char* szName) {m_xAttachedActorName = szName;}
	#endif

	Asura_Cutscene_Actor* GetActor() const{return m_pxAttachedActor;}

	//Load/save functions , defined in Asura_Chunk_Cutscene.cpp
	virtual int GetChunkSize();
	virtual void SaveToChunk(char*& pcChunkBuffer);
	virtual void LoadFromChunk(const class Asura_Chunk_CutsceneAttachment* pxChunk,Asura_Handle iFileHandle);
//	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

protected :
	virtual void HandleActiveStatusChange();

	Asura_String m_xAttachedActorName;
	Asura_Cutscene_Actor* m_pxAttachedActor;

	friend class Tools_Cutscene_Attachment_Actor;

};
///////////////////////////////////////////////////////////////////////////////////////////////

class Asura_Cutscene_Attachment_Entity : public Asura_Cutscene_Attachment
{
public :
	Asura_Cutscene_Attachment_Entity(Asura_Cutscene_Actor_Hierarchy* pxActor);

	virtual int GetType() const {return AsuraCutsceneAttachment_Entity;}
	virtual void Render();
	virtual void Initialise();
	virtual void Finish();

	#ifdef ASURA_TOOLS
	void Copy(Asura_Cutscene_Attachment* pxAttachment);
	#endif

	//Load/save functions , defined in Asura_Chunk_Cutscene.cpp
	virtual int GetChunkSize();
	virtual void SaveToChunk(char*& pcChunkBuffer);
	virtual void LoadFromChunk(const class Asura_Chunk_CutsceneAttachment* pxChunk,Asura_Handle iFileHandle);
//	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	#ifdef ASURA_TOOLS
	virtual void ResetGuidsAndMessages();
	#endif	

	static void ActivateEntity( Asura_Guid uGUID, u_short usClassification, bool bActivate, bool bIgnoreModuleLinking = false );
	static bool MoveEntity( Asura_Guid uGUID,
                            u_short usClassification,
                            const Asura_Vector_3& xPos,
                            const Asura_Matrix_3x3& xOri,
                            const Asura_Vector_3* pxVelocity,
							bool bFirstUpdate);

	Asura_Guid GetAttachedGuid() const {return m_uEntityGUID;}

protected:

	virtual void HandleActiveStatusChange();
	void UpdatePosition();


	Asura_Vector_3 m_xLastPosition;
	Asura_Guid     m_uEntityGUID;
    u_short        m_usEntityClassification;
	bool           m_bHasUpdated;
#if _DEBUG
	bool           m_bEntityTypeAsserted;
#endif

	friend class Tools_Cutscene_Attachment_Entity;

private:

    typedef Asura_Cutscene_Attachment PARENT;

};

#define AsuraCoronaAttachmentFlag_UseOrientation 0x00000001

class Asura_Cutscene_Attachment_Corona : public Asura_Cutscene_Attachment
{
public :
	Asura_Cutscene_Attachment_Corona(Asura_Cutscene_Actor_Hierarchy* pxActor);
	virtual int GetType() const {return AsuraCutsceneAttachment_Corona;}

	#ifdef ASURA_TOOLS
	void Copy(Asura_Cutscene_Attachment* pxAttachment);
	virtual void BuildUsedTextureTree(Asura_NameTree* pxNameTree);
	#endif

	//Load/save functions , defined in Asura_Chunk_Cutscene.cpp
	virtual int GetChunkSize();
	virtual void SaveToChunk(char*& pcChunkBuffer);
	virtual void LoadFromChunk(const class Asura_Chunk_CutsceneAttachment* pxChunk,Asura_Handle iFileHandle);
//	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

protected :
	virtual void HandleActiveStatusChange();

	float m_fRed;
	float m_fGreen;
	float m_fBlue;
	float m_fSize;
	unsigned m_uCoronaFlags;
	int m_iTextureID;
	int m_iTextureIDForSave;

	//playback data
	class Asura_AnimationAttachment_Corona* m_pxAnimationAttachment;

	friend class Tools_Cutscene_Attachment_Corona;

};

class Asura_Cutscene_Attachment_PFX : public Asura_Cutscene_Attachment
{
public :
	Asura_Cutscene_Attachment_PFX(Asura_Cutscene_Actor_Hierarchy* pxActor);
	virtual int GetType() const {return AsuraCutsceneAttachment_PFX;}

	#ifdef ASURA_TOOLS
	void Copy(Asura_Cutscene_Attachment* pxAttachment);
	virtual void AddExtraFilesToList( List<Asura_String>& xNameList ) const;
	#endif

	virtual void Finish();
	virtual void Update(float fTime);
	
	//Load/save functions , defined in Asura_Chunk_Cutscene.cpp
	virtual int GetChunkSize();
	virtual void SaveToChunk(char*& pcChunkBuffer);
	virtual void LoadFromChunk(const class Asura_Chunk_CutsceneAttachment* pxChunk,Asura_Handle iFileHandle);
//	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

protected :
	virtual void HandleActiveStatusChange();
	void UpdatePosition();

	Asura_Hash_ID m_uEffectTemplateHash;

	//playback data
	Asura_Handle m_iEffectID;
	bool m_bHasUpdatedPosition;

	friend class Tools_Cutscene_Attachment_PFX;
};

class Asura_Cutscene_Attachment_SoundEvent : public Asura_Cutscene_Attachment
{
public :
	Asura_Cutscene_Attachment_SoundEvent(Asura_Cutscene_Actor_Hierarchy* pxActor);
	virtual int GetType() const {return AsuraCutsceneAttachment_SoundEvent;}

	#ifdef ASURA_TOOLS
	void Copy(Asura_Cutscene_Attachment* pxAttachment);
	virtual void AddExtraFilesToList( List<Asura_String>& xNameList ) const;
	#endif

	Asura_Hash_ID GetSoundEventHash() const {return m_uSoundEvent;}

	virtual void Finish();
	virtual void Update(float fTime);
	
	//Load/save functions , defined in Asura_Chunk_Cutscene.cpp
	virtual int GetChunkSize();
	virtual void SaveToChunk(char*& pcChunkBuffer);
	virtual void LoadFromChunk(const class Asura_Chunk_CutsceneAttachment* pxChunk,Asura_Handle iFileHandle);
//	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

protected :
	virtual void HandleActiveStatusChange();
	void UpdatePosition();

	Asura_Hash_ID m_uSoundEvent;

	//playback data
	Asura_Handle m_iSoundHandle;
	bool m_bStartSoundWithNextPositionUpdate;

	friend class Tools_Cutscene_Attachment_SoundEvent;
};




#endif //__ASURA_CUTSCENE_ATTACHMENT_H__

