//
// Asura_FMVSubtitles.cpp
//
// Created by Matt Clemson - 14/11/2005
// Based on RT_Subtitles, created by Richard Rice
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Cutscene/Asura_FMVSubtitles.h"

// Engine independent.
#include "Core/Asura_HashedLocalisedText.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Render.h"

// Tools.
#ifdef ASURA_TOOLS
#include "Tools_System.h"
#endif

// G L O B A L S /////////////////////////////////////////////////////////////

DEBUGVAR bool dbg_bEnableSubtitles = true;

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

int Asura_FMVSubtitles::s_iNumSubtitles = 0;
Asura_FMVSubtitle* Asura_FMVSubtitles::s_pxSubtitles = 0;

void Asura_FMVSubtitles::Initialise()
{
}

void Asura_FMVSubtitles::Shutdown()
{
	delete [] s_pxSubtitles;
	s_pxSubtitles = 0;
	s_iNumSubtitles = 0;
}

const Asura_UnicodeChar* Asura_FMVSubtitles::GetSubtitleByTime( float fTime, float& fAlpha )
{
    const Asura_FMVSubtitle* pxSubtitle = NULL;

    // Find the subtitle we care about.
    for( int i = 0; i < s_iNumSubtitles; ++i )
    {
        const Asura_FMVSubtitle& xSubtitle = s_pxSubtitles[i];

        if( fTime >= xSubtitle.m_fStartTime && fTime < xSubtitle.m_fEndTime )
        {
            // Ok, this is the chap.
            pxSubtitle = &xSubtitle;

            // Fade based on proximity to either end of its duration.
            float fTimeFromEnd = Asura_Maths::Min( fTime - xSubtitle.m_fStartTime, xSubtitle.m_fEndTime - fTime );
            fAlpha = Asura_Maths::Min( 1.0f, fTimeFromEnd / fASURA_DEFAULT_SUBTITLE_FADE_TIME );

            break;
        }
    }

    if( !pxSubtitle || !pxSubtitle->m_uTextID ) { return NULL; }

    const Asura_UnicodeChar* wszText = Asura_HashedLocalisedText::GetString(uASURA_HASHEDTEXTPAGE_CUTSCENE, pxSubtitle->m_uTextID );
    
	return wszText;
}

void Asura_FMVSubtitles::RenderSubtitleByTime( float fTime )
{
	float fAlpha = 1.0f;

	const Asura_UnicodeChar* wszText = GetSubtitleByTime(fTime, fAlpha);

	if(wszText)
	{
		RenderSubtitle( wszText, fAlpha );
	}
}

void Asura_FMVSubtitles::RenderSubtitle( const Asura_UnicodeChar* wszText, float fAlpha )
{
    if( !dbg_bEnableSubtitles ) { return; }

	// For text that we want to be always displayed yet not having the special formatting:
	const bool bIsAlwaysDisplayedText = ( L'}' == *wszText ); 

    if( bIsAlwaysDisplayedText )
    {
        // Starting a subtitle string with this character denotes placename
        // text, or similar. These should be displayed regardless of whether
        // or not the player has enabled subtitles in their profile.
        ++wszText;
    }
    else
    {
		if(!Project_ShouldDisplaySubtitles(wszText)) {return;}
    }

	u_int uAlignFlags = ASURA_TEXTFORMAT_CENTREJUSTIFIED;
	Asura_Bounding_Rectangle xRect;
	PlatformProject_GetBoundingRect( wszText, xRect, uAlignFlags );

	Asura_Font_System::SetFont( PlatformProject_GetFontHandleForSubtitles() );

	u_int uFormatFlags = ASURA_TEXTFORMAT_RELATIVEPOSITION
					   | ASURA_TEXTFORMAT_WORDWRAP
					   | ASURA_TEXTFORMAT_DROPSHADOWS
					   | ASURA_TEXTFORMAT_RELATIVETORECT
					   | uAlignFlags;

	u_int uColour = Project_GetTextARGB();

	// Modulate alpha.
	{
		float fFullAlpha = static_cast<float>( Project_GetTextARGB() >> 24 );

		uColour &= 0x00FFFFFF;
		uColour |= static_cast<u_int>(Asura_Maths::FloatToInt(fFullAlpha * fAlpha)) << 24u;
	}   

	Asura_Font_System::PrintLongString( wszText, &xRect, 0.0f, 0.0f, 0.8f, PlatformProject_GetFontScale(), uColour, uFormatFlags );

	Asura_Render::FlushDisplay();
}
