 //
// Asura_Assert.h
//
// Created by Simon Watson - 2009/05/11 14:31
// 
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//

#ifdef _lint
// (MP - 16.09.2002) Shut Lint up about code like Asura_Assert( false, "" );
// and similar.
//lint -emacro(774,  Asura_Assert ) // Boolean always evaluates to true / false.
//lint -emacro(778,  Asura_Assert ) // Constant expression evaluates to 0.
//lint -emacro(1534, Asura_Assert ) // Static variable 'ignore' found within inline function in header.
//lint -emacro(774,  Asura_Assertf) // Boolean always evaluates to true / false.
//lint -emacro(778,  Asura_Assertf) // Constant expression evaluates to 0.
//lint -emacro(1534, Asura_Assertf) // Static variable 'ignore' found within inline function in header.
#endif

#include "Debugging/Asura_Platform_Assert.h"

#ifndef ASURA_PLATFORM_ASSERT_OVERRIDES_INDEPENDENT_CODE // Because PSP and PS2 are difficult

#ifndef __ASURA_ASSERT_H__
#define __ASURA_ASSERT_H__

#if defined(ASURA_ASSERT)

	extern u_char g_ucAssertIgnoreCounter;

	//// These three functions need to be implemented in the platform-specific assert file.
	extern bool Asura_AssertfSetup( const char* szFile, int iLine, const char* szCondition, u_char* pucIgnore);

	// Both of these functions return true iff we should break into the debugger.
	// They are assumed to dealing with exiting the application internally.
	extern bool Asura_AssertFired( const char* szFile, int iLine, const char* szCondition, const char* szDesc, u_char* pucIgnore );
	extern bool Asura_AssertfFired( const char* szDescFormat, ... );

	// Standard assert macro...
    #define Asura_Assert(cond, desc) \
    { \
		static u_char ucIgnore = 0; \
		if (ucIgnore != g_ucAssertIgnoreCounter) \
	    { \
		    if(!(cond)) \
		    { \
			    if (Asura_AssertFired(__FILE__, __LINE__, #cond, desc, &ucIgnore) ) \
			    { \
				    ASURA_PLATFORM_DEBUGBREAK(); \
			    } \
		    } \
	    } \
    } 

    // (MH - 01.11.2002) parameterised assert. Usage example (note the brackets around arg 2):
    // Asura_Assertf(false, ("%s is invalid", szString));
    #define Asura_Assertf( __cond, __params ) \
    { \
		static u_char ucIgnore = 0; \
		if (ucIgnore != g_ucAssertIgnoreCounter) \
	    { \
		    if( !(__cond) ) \
		    { \
                const bool bSetupOK = Asura_AssertfSetup(__FILE__, __LINE__, #__cond, &ucIgnore); \
			    if( bSetupOK && Asura_AssertfFired __params ) \
			    { \
				    ASURA_PLATFORM_DEBUGBREAK(); \
			    } \
		    } \
	    } \
    }
    
	//supresses all the assert dialogs , just leaving a single entry in the logfile per assert RAR 02/06/03
    void AsuraAssert_SetIgnoreAll(bool bIgnore);
	bool AsuraAssert_GetIgnoreAll(void);

	void AsuraAssert_ReEnableAll();	// Turn all asserts which have been skipped with "Ignore All" back on
	bool AsuraAssert_ConsoleCmd_EnableAsserts( bool bHelp );

	#if defined( ASURA_TOOLS)
		void AsuraAssert_LogXML( const bool bLogXML);
		void AsuraAssert_LogXMLFilename( const char* pszFilename);
		void AsuraAssert_LogClearLogOnNextAssert( const bool bClear);
	#endif

	// Internal functions
	void AsuraAssert_LogClearLog();
	void AsuraAssert_RecordAssertHitWhileIgnoringAll();

#if defined( ASURA_TOOLS)
	bool AsuraAssert_ShouldLogToXML();
	const char* AsuraAssert_GetXMLLogFilename();
#endif

#else
	#define Asura_Assert(ignore1,ignore2)
	#define Asura_Assertf(ignore1,ignore2)

	#define AsuraAssert_SetIgnoreAll(bIgnore)
	#define AsuraAssert_GetIgnoreAll(bIgnore) true
	#define AsuraAssert_ReEnableAll()
	#define AsuraAssert_ConsoleCmd_EnableAsserts(bHelp) true

	#if defined( ASURA_TOOLS)
		#define AsuraAssert_LogXML( bIgnore)
		#define AsuraAssert_LogXMLFilename( bIgnore)
		#define AsuraAssert_LogClearLogOnNextAssert( bIgnore)
	#endif

#endif	//defined(ASURA_ASSERT)

#endif //__ASURA_ASSERT_H__

#endif // ASURA_PLATFORM_ASSERT_OVERRIDES_INDEPENDENT_CODE
