
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_DebugLineRender.h
// Author : Andy Tate
// Date   : 22-Jan-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_DEBUGLINERENDER_H__
#define __ASURA_DEBUGLINERENDER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#if !defined(ASURA_NO_DEBUG_OUTPUT)

#include "Core/Asura_Maths.h"


////////////////////////////////////////////////////////////////////////////////
// Defines...
////////////////////////////////////////////////////////////////////////////////

#if !defined(ASURA_MAX_DEBUG_LINE_RENDER_SINGLE_LINES)
#define	ASURA_MAX_DEBUG_LINE_RENDER_SINGLE_LINES	1024
#endif	// !defined(ASURA_MAX_DEBUG_LINE_RENDER_SINGLE_LINES)


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Frustum;
class Asura_OBB;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_DebugLineRender_SingleLine
// Author       : Andy Tate
// Date         : 22-Jan-2007
//
// Description  : Class used by the debug-line renderer to store the data for
//				  a single line to be rendered.
////////////////////////////////////////////////////////////////////////////////

class Asura_DebugLineRender_SingleLine
{
public:
	// Public functions...
	Asura_DebugLineRender_SingleLine  (void)	{}
//	~Asura_DebugLineRender_SingleLine ();

	Asura_DebugLineRender_SingleLine(const Asura_Vector_3& xStart, const Asura_Vector_3& xEnd, const u_int uARGB)
	: m_xStart(xStart)
	, m_xEnd(xEnd)
	, m_uARGB(uARGB)
	{
	}

	Asura_DebugLineRender_SingleLine(const Asura_AVector3& xStart, const Asura_AVector3& xEnd, const u_int uARGB)
	: m_xStart(xStart)
	, m_xEnd(xEnd)
	, m_uARGB(uARGB)
	{
	}

	Asura_DebugLineRender_SingleLine(const Asura_DebugLineRender_SingleLine& xOther)
	: m_xStart(xOther.m_xStart)
	, m_xEnd(xOther.m_xEnd)
	, m_uARGB(xOther.m_uARGB)
	{
	}

	Asura_DebugLineRender_SingleLine& operator=(const Asura_DebugLineRender_SingleLine& xOther)
	{
		m_xStart	= xOther.m_xStart;
		m_xEnd		= xOther.m_xEnd;
		m_uARGB		= xOther.m_uARGB;
		return(*this);
	}

	// Render function...
	void	Render	(const float fAlpha) const;

	// Accessor functions...
	const Asura_Vector_3&	GetStart(void) const	{ return(m_xStart); }
	const Asura_Vector_3&	GetEnd(void) const		{ return(m_xEnd); }
	u_int					GetARGB(void) const		{ return(m_uARGB); }


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...
	Asura_Vector_3	m_xStart;	// The position of the start of the line.
	Asura_Vector_3	m_xEnd;		// The position of the end of the line.
	u_int			m_uARGB;	// The colour of the line to be rendered.

	// Private functions...

};   // End of class 'Asura_DebugLineRender_SingleLine'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_DebugLineRender_LineSet
// Author       : Andy Tate
// Date         : 23-Jan-2007
//
// Description  : Class which holds a maximum number of single lines.
////////////////////////////////////////////////////////////////////////////////

class Asura_DebugLineRender_LineSet
{
public:
	// Public functions...
	Asura_DebugLineRender_LineSet          (const u_int uMaxLines);
	virtual ~Asura_DebugLineRender_LineSet ();

	// Virtual manipulation functions...
	virtual	void	Reset(void)	{ m_uNumLines = 0; }

	virtual	void	Update(const float fTimeStep)	= 0;
	virtual	void	Render(void)					= 0;

	// Function to add a single-line to the renderer...
	u_int	AddLine(const Asura_Vector_3& xStart, const Asura_Vector_3& xEnd, const u_int uARGB = 0xFFFFFFFF);
	u_int	AddLine(const Asura_AVector3& xStart, const Asura_AVector3& xEnd, const u_int uARGB = 0xFFFFFFFF);

	// Functions to add complex shapes to the renderer...
	u_int	AddArrow(const Asura_Vector_3& xStart, const Asura_Vector_3& xEnd, const u_int uARGB = 0xFFFFFFFF, const float fHeadProportion = 0.125f, const float fCrossSize = 0.0f);
	u_int	AddArrow(const Asura_AVector3& xStart, const Asura_AVector3& xEnd, const u_int uARGB = 0xFFFFFFFF, const float fHeadProportion = 0.125f, const float fCrossSize = 0.0f);

	u_int	AddCross(const Asura_Vector_3& xPosition, const u_int uARGB = 0xFFFFFFFF, const float fSize = 1.0f);
	u_int	AddCylinder(const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const float fRadius, const float fHeight, const u_int uARGB = 0xFFFFFFFF, const u_int uNumSegments = 16);
	u_int	AddOBB(const Asura_OBB& xOBB, const u_int uARGB = 0xFFFFFFFF);
	u_int	AddSphere(const Asura_Vector_3& xPosition, const float fRadius, const u_int uARGB = 0xFFFFFFFF, const u_int uNumSegments = 16, const u_int uNumSlices = 8);

	// 3x3 matrix functions...
	u_int	AddAxis(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const float fSize = 1.0f, const float fAlpha = 1.0f);
	u_int	AddAxis (const Asura_AVector3& xPosition, const Asura_AMatrix3x3& xOrientation, const float fSize, const float fAlpha);
	
	u_int	AddBoundingBox(const Asura_Bounding_Box& xBB, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const u_int uARGB = 0xFFFFFFFF);
	u_int	AddFrustum(const Asura_Frustum& xFrustum, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const u_int uARGB = 0xFFFFFFFF);

	// Quaternion functions...
	u_int AddAxis(const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const float fSize = 1.0f, const float fAlpha = 1.0f)
	{
		Asura_Matrix_3x3	xOrientationMatrix;

		xOrientation.ToMatrix(xOrientationMatrix);
		return(AddAxis(xPosition, xOrientationMatrix, fSize, fAlpha));
	}

	u_int AddFrustum(const Asura_Frustum& xFrustum, const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const u_int uARGB = 0xFFFFFFFF)
	{
		Asura_Matrix_3x3	xOrientationMatrix;

		xOrientation.ToMatrix(xOrientationMatrix);
		return(AddFrustum(xFrustum, xPosition, xOrientationMatrix, uARGB));
	}

	u_int AddBoundingBox(const Asura_Bounding_Box& xBB, const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const u_int uARGB = 0xFFFFFFFF)
	{
		Asura_Matrix_3x3	xOrientationMatrix;

		xOrientation.ToMatrix(xOrientationMatrix);
		return(AddBoundingBox(xBB, xPosition, xOrientationMatrix, uARGB));
	}

	// Abstract query functions...
	virtual	bool	IsFinished(void) const	= 0;

	// Accessor functions...
	void									SetNextLineSet(Asura_DebugLineRender_LineSet* pxLineSet)	{ m_pxNextLineSet = pxLineSet; }
	Asura_DebugLineRender_LineSet*			GetNextLineSet(void)										{ return(m_pxNextLineSet); }
	const Asura_DebugLineRender_LineSet*	GetNextLineSet(void) const									{ return(m_pxNextLineSet); }


protected:
	// Protected variables...

	// Protected functions...
	void	Render	(const float fAlpha);


private:
	// Private variables...
	u_int								m_uNumLines;		// The current number of lines in this line-set.
	u_int								m_uMaxLines;		// The maximum number of lines in this line-set.
	Asura_DebugLineRender_SingleLine*	m_pxSingleLines;	// A pointer to the array of single-lines used by this line-set.

	Asura_DebugLineRender_LineSet*		m_pxNextLineSet;	// A pointer to the next line-set in the list.

	// Private functions...
	Asura_DebugLineRender_LineSet  (void);
	Asura_DebugLineRender_LineSet  (const Asura_DebugLineRender_LineSet& xOther);

	Asura_DebugLineRender_LineSet&	operator=	(const Asura_DebugLineRender_LineSet& xOther);

};   // End of class 'Asura_DebugLineRender_LineSet'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_DebugLineRender_OneShotLineSet
// Author       : Andy Tate
// Date         : 23-Jan-2007
//
// Description  : Class to handle one-shot line-set (i.e. set of lines which
//				  are rendered once ONLY.
////////////////////////////////////////////////////////////////////////////////

class Asura_DebugLineRender_OneShotLineSet : public Asura_DebugLineRender_LineSet
{
public:
	// Public functions...
	Asura_DebugLineRender_OneShotLineSet          (const u_int uMaxLines)	: PARENT(uMaxLines)	{}
//	virtual ~Asura_DebugLineRender_OneShotLineSet ();

	// Overridden manipulation functions...
//	virtual	void	Reset(void);

	virtual	void	Update(const float fTimeStep);
	virtual	void	Render(void);

	// Overridden query functions...
	virtual	bool	IsFinished(void) const	{ return(false); }


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private typedefs...
	typedef Asura_DebugLineRender_LineSet PARENT;

	// Private variables...

	// Private functions...
	Asura_DebugLineRender_OneShotLineSet  (void);
	Asura_DebugLineRender_OneShotLineSet  (const Asura_DebugLineRender_OneShotLineSet& xOther);

	Asura_DebugLineRender_OneShotLineSet&	operator=	(const Asura_DebugLineRender_OneShotLineSet& xOther);

};   // End of class 'Asura_DebugLineRender_OneShotLineSet'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_DebugLineRender_TimedLineSet
// Author       : Andy Tate
// Date         : 23-Jan-2007
//
// Description  : Class to handle line-sets which disappear after a period of
//				  time.
////////////////////////////////////////////////////////////////////////////////

class Asura_DebugLineRender_TimedLineSet : public Asura_DebugLineRender_LineSet
{
public:
	// Public functions...
	Asura_DebugLineRender_TimedLineSet          (const u_int uMaxLines, const float fDuration, const float fFadeDuration) : PARENT(uMaxLines), m_fTimeLeft(fDuration), m_fFadeTimeLeft(fFadeDuration), m_fFadeDuration(fFadeDuration)	{}
//	virtual ~Asura_DebugLineRender_TimedLineSet ();

	// Overridden manipulation functions...
//	virtual	void	Reset(void);

	virtual	void	Update(const float fTimeStep);
	virtual	void	Render(void);

	// Overridden query functions...
	virtual	bool	IsFinished	(void) const	{ return((m_fTimeLeft == 0.0f) && (m_fFadeTimeLeft == 0.0f)); }


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private typedefs...
	typedef Asura_DebugLineRender_LineSet PARENT;

	// Private variables...
	float	m_fTimeLeft;			// The amount of time this line-set has left before fading.
	float	m_fFadeTimeLeft;		// The amount of time this line-set has fading before it disappears.
	float	m_fFadeDuration;		// The duration this line-set fades for before it disappears.

	// Private functions...
	Asura_DebugLineRender_TimedLineSet  (void);
	Asura_DebugLineRender_TimedLineSet  (const Asura_DebugLineRender_TimedLineSet& xOther);

	Asura_DebugLineRender_TimedLineSet&	operator=	(const Asura_DebugLineRender_TimedLineSet& xOther);

};   // End of class 'Asura_DebugLineRender_TimedLineSet'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_DebugLineRender
// Author       : Andy Tate
// Date         : 22-Jan-2007
//
// Description  : Class to allow debugging lines to be rendered.
////////////////////////////////////////////////////////////////////////////////

class Asura_DebugLineRender
{
public:
	// Public functions...
	static void	Initialise(void);
	static void	Shutdown(void);
	static void	Reset(void);

	// Creation/destruction functions...
	static Asura_DebugLineRender_LineSet*	CreateOneShotLineSet(const u_int uMaxLines);
	static Asura_DebugLineRender_LineSet*	CreateTimedLineSet(const u_int uMaxLines, const float fDuration, const float fFadeDuration = 0.0f);

	static bool	DestroyLineSet(Asura_DebugLineRender_LineSet* pxLineSet);

	// Manipulation functions...
	static void	Update(void);
	static void	Render(void);

	// Function to add a single-line to the renderer...
	static u_int AddLine(const Asura_Vector_3& xStart, const Asura_Vector_3& xEnd, const u_int uARGB = 0xFFFFFFFF)
	{
		return(s_pxOneShotLineSet->AddLine(xStart, xEnd, uARGB));
	}

	static u_int AddLine(const Asura_AVector3& xStart, const Asura_AVector3& xEnd, const u_int uARGB = 0xFFFFFFFF)
	{
		return(s_pxOneShotLineSet->AddLine(xStart, xEnd, uARGB));
	}

	// Functions to add complex shapes to the renderer...
	static u_int AddArrow(const Asura_Vector_3& xStart, const Asura_Vector_3& xEnd, const u_int uARGB = 0xFFFFFFFF, const float fHeadProportion = 0.125f, const float fCrossSize = 0.0f)
	{
		return(s_pxOneShotLineSet->AddArrow(xStart, xEnd, uARGB, fHeadProportion, fCrossSize));
	}

	static u_int AddArrow(const Asura_AVector3& xStart, const Asura_AVector3& xEnd, const u_int uARGB = 0xFFFFFFFF, const float fHeadProportion = 0.125f, const float fCrossSize = 0.0f)
	{
		return(s_pxOneShotLineSet->AddArrow(xStart, xEnd, uARGB, fHeadProportion, fCrossSize));
	}

	static u_int AddCross(const Asura_Vector_3& xPosition, const u_int uARGB = 0xFFFFFFFF, const float fSize = 1.0f)
	{
		return(s_pxOneShotLineSet->AddCross(xPosition, uARGB, fSize));
	}

	static u_int AddCylinder(const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const float fRadius, const float fHeight, const u_int uARGB = 0xFFFFFFFF, const u_int uNumSegments = 16)
	{
		return(s_pxOneShotLineSet->AddCylinder(xPosition, xOrientation, fRadius, fHeight, uARGB, uNumSegments));
	}

	static u_int AddOBB(const Asura_OBB& xOBB, const u_int uARGB = 0xFFFFFFFF)
	{
		return(s_pxOneShotLineSet->AddOBB(xOBB, uARGB));
	}

	static u_int AddSphere(const Asura_Vector_3& xPosition, const float fRadius, const u_int uARGB = 0xFFFFFFFF, const u_int uNumSegments = 16, const u_int uNumSlices = 8)
	{
		return(s_pxOneShotLineSet->AddSphere(xPosition, fRadius, uARGB, uNumSegments, uNumSlices));
	}

	// 3x3 matrix functions...
	static u_int AddAxis(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const float fSize = 1.0f, const float fAlpha = 1.0f)
	{
		return(s_pxOneShotLineSet->AddAxis(xPosition, xOrientation, fSize, fAlpha));
	}

	static u_int AddAxis(const Asura_AVector3& xPosition, const Asura_AMatrix3x3& xOrientation, const float fSize = 1.0f, const float fAlpha = 1.0f)
	{
		return(s_pxOneShotLineSet->AddAxis(xPosition, xOrientation, fSize, fAlpha));
	}

	static u_int AddBoundingBox(const Asura_Bounding_Box& xBB, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const u_int uARGB = 0xFFFFFFFF)
	{
		return(s_pxOneShotLineSet->AddBoundingBox(xBB, xPosition, xOrientation, uARGB));
	}

	static u_int AddFrustum(const Asura_Frustum& xFrustum, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const u_int uARGB = 0xFFFFFFFF)
	{
		return(s_pxOneShotLineSet->AddFrustum(xFrustum, xPosition, xOrientation, uARGB));
	}

	// Quaternion functions...
	static u_int AddAxis(const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const float fSize = 1.0f, const float fAlpha = 1.0f)
	{
		return(s_pxOneShotLineSet->AddAxis(xPosition, xOrientation, fSize, fAlpha));
	}

	static u_int AddBoundingBox(const Asura_Bounding_Box& xBB, const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const u_int uARGB = 0xFFFFFFFF)
	{
		return(s_pxOneShotLineSet->AddBoundingBox(xBB, xPosition, xOrientation, uARGB));
	}

	static u_int AddFrustum(const Asura_Frustum& xFrustum, const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation, const u_int uARGB = 0xFFFFFFFF)
	{
		return(s_pxOneShotLineSet->AddFrustum(xFrustum, xPosition, xOrientation, uARGB));
	}


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...
	static Asura_DebugLineRender_LineSet*	s_pxFirstLineSet;		// The first line-set in the chain.
	static Asura_DebugLineRender_LineSet*	s_pxOneShotLineSet;	// The permanent line-set.

	// Private functions...
	Asura_DebugLineRender  (void);
	Asura_DebugLineRender  (const Asura_DebugLineRender& xOther);
	~Asura_DebugLineRender ();

	Asura_DebugLineRender&	operator=	(const Asura_DebugLineRender& xOther);

};   // End of class 'Asura_DebugLineRender'.

#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)

#endif   // ifndef __ASURA_DEBUGLINERENDER_H__


/// End of file. ///
