//////////////////////////////////////////////////////////////////////////////
//
// Asura_Profiling_BarRenderer.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_PROFILING_BARRENDERER_H___
#define ASURA_PROFILING_BARRENDERER_H___ 1

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_Profiling_BarRenderer_Data;


class Asura_Profiling_BarRenderer
{

public:

    // Caller should create and initialise a data instance and store it
    // externally. Every frame necessary new data should be set and then it
    // should be passed into this function.
    static void Render( Asura_Profiling_BarRenderer_Data& xData );

    // Some global settings.
    static void SetDivisionCountReductionDelay( float fDelay );
    static void SetDivisionCountIncreaseDelay( float fDelay );
    static void SetMaxDivisions( int iMax );
    static void SetTargetFrameRate( float fTarget );

};


class Asura_Profiling_BarRenderer_Data
{

public:

    Asura_Profiling_BarRenderer_Data( void );

    // Initial setup (typically set only once but there's no restriction on
    // this). Note that any pointers provided are assumed to reference
    // persistent data.
    void SetBackgroundColours( const u_int* puBackgroundARGBs, u_int uNumBackgroundARGBs );
    void SetBarColours( const u_int* puBarARGBs, u_int uNumBarARGBs, u_int uFrameRemainderARGB );
    void SetUseDefaultBackgroundColours( void );
    void SetYOffset( float fYOffset );

    // Per-frame setup.
    void SetNewFrameData( float fTotalFrameTime,
                          const float* pfTimers, u_int uNumTimers );

private:

    friend class Asura_Profiling_BarRenderer;

    // Fixed input data, set once.
    const u_int* m_puBackgroundARGBs;
    u_int        m_uNumBackgroundARGBs;
    const u_int* m_puBarARGBs;
    u_int        m_uNumBarARGBs;
    u_int        m_uFrameRemainderARGB;
    float        m_fYOffset;            // Proportion of the viewport.

    // Input data that should be set every frame by caller.
    float        m_fTotalFrameTime;
    const float* m_pfTimers;
    u_int        m_uNumTimers;

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_PROFILING_BARRENDERER_H___
