
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_ServerEntity_Affector.cpp
// Author : Andy Tate
// Date   : 6-Dec-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_AFFECTORS)

// This file's header...
#include "Entity/Affector/Asura_ServerEntity_Affector.h"

// Asura independent header-files...
#include "Affectors/Asura_Affector.h"
#include "Affectors/Asura_Affector_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Message.h"

#if defined(ASURA_TOOLS)
// Tools header-files...
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif	//defined(ASURA_TOOLS)


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Classified_List<Asura_ServerEntity_Affector>	Asura_ServerEntity_Affector::s_xList;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::Asura_ServerEntity_Affector
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

Asura_ServerEntity_Affector::Asura_ServerEntity_Affector (const Asura_Guid uGuid)
: Asura_Entity_Server(uGuid)
{
	Classification		= AsuraEntityClass_Affector;
	m_iAffectorHandle	= Asura_Affector_System::CreateAffector();

	s_xList.AddToList(this);
}

/// End of function 'Asura_ServerEntity_Affector::Asura_ServerEntity_Affector'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::~Asura_ServerEntity_Affector
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

Asura_ServerEntity_Affector::~Asura_ServerEntity_Affector ()
{
	Asura_Affector_System::DestroyAffector(m_iAffectorHandle);

	s_xList.RemoveFromList(this);
}

/// End of function 'Asura_ServerEntity_Affector::~Asura_ServerEntity_Affector'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::MakeClientUpdateMessage
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector::MakeClientUpdateMessage (void)
{
}

/// End of function 'Asura_ServerEntity_Affector::MakeClientUpdateMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::Create
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector::Create (const Asura_Guid uGuid)
{
	new Asura_ServerEntity_Affector (uGuid);
}

/// End of function 'Asura_ServerEntity_Affector::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::Create
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector::Create (const Asura_Guid uGuid,
										  Asura_Chunk_Stream& xStream)
{
	Asura_ServerEntity_Affector*	pxAffector	= new Asura_ServerEntity_Affector (uGuid);

	pxAffector->ReadFromChunkStream(xStream);
}

/// End of function 'Asura_ServerEntity_Affector::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector::ReadFromChunkStream (Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Read the version...
	xStream >> uVersion;
	if(uVersion > uAFFECTOR_CHUNK_VERSION)
	{
		Asura_Assert(false, "Unexpected chunk version for affector-entity.");
		return;
	}

	if (uVersion >= 2)
	{
		PARENT::ReadFromChunkStream(xStream);
	}

	Asura_Affector_System::DestroyAffector(m_iAffectorHandle);
	m_iAffectorHandle	= Asura_Affector_System::CreateAffector();

	if(m_iAffectorHandle == ASURA_INVALID_HANDLE)
	{
		Asura_Assert(false, "Failed to create affector for reading.");
	}
	else
	{
		Asura_Affector*	pxAffector	= Asura_Affector_System::GetAffectorPointer(m_iAffectorHandle);

		pxAffector->ReadFromChunkStream(xStream);
	}
}

/// End of function 'Asura_ServerEntity_Affector::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::WriteToChunkStream
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector::WriteToChunkStream (Asura_Chunk_Stream &xStream) const
{
	Asura_Affector*	pxAffector	= Asura_Affector_System::GetAffectorPointer(m_iAffectorHandle);

	xStream << uAFFECTOR_CHUNK_VERSION;

	PARENT::WriteToChunkStream(xStream);

	if(pxAffector == NULL)
	{
		Asura_Assert(false, "Failed to find affector to write.");
	}
	else
	{
		pxAffector->WriteToChunkStream(xStream);
	}
}

/// End of function 'Asura_ServerEntity_Affector::WriteToChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::OnEntityActivate
// Author       : Neil Young
// Date         : 21-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector::OnEntityActivate()
{
	PARENT::OnEntityActivate();

	Asura_Affector*	pxAffector	= Asura_Affector_System::GetAffectorPointer(m_iAffectorHandle);
	if(pxAffector == NULL)
	{
		Asura_Assert(false, "Failed to find affector to activate.");
		return;
	}
	pxAffector->SetAsActive(true);
}

/// End of function 'Asura_ServerEntity_Affector::OnEntityActivate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::OnEntityDeactivate
// Author       : Neil Young
// Date         : 21-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector::OnEntityDeactivate()
{
	PARENT::OnEntityDeactivate();

	Asura_Affector*	pxAffector	= Asura_Affector_System::GetAffectorPointer(m_iAffectorHandle);
	if(pxAffector == NULL)
	{
		Asura_Assert(false, "Failed to find affector to deactivate.");
		return;
	}
	pxAffector->SetAsActive(false);
}

/// End of function 'Asura_ServerEntity_Affector::OnEntityDeactivate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::GetAffector
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Affector* Asura_ServerEntity_Affector::GetAffector (void)
{
	return(Asura_Affector_System::GetAffectorPointer(m_iAffectorHandle));
}

/// End of function 'Asura_ServerEntity_Affector::GetPosition'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::GetAffector
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

const Asura_Affector* Asura_ServerEntity_Affector::GetAffector (void) const
{
	return(Asura_Affector_System::GetAffectorPointer(m_iAffectorHandle));
}

/// End of function 'Asura_ServerEntity_Affector::GetPosition'. ///


#if defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector::ApplyChangesForTemplateInstance
// Author       : Andy Tate
// Date         : 27-Mar-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector::ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance)
{
	Asura_Affector*			pxAffector		= GetAffector();
	const Asura_Vector_3&	xOldPosition	= pxAffector->GetPosition();

	Asura_Vector_3			xNewPosition;

	xNewPosition	= pxInstance->TransformPosition(xOldPosition);
	pxAffector->SetPosition(xNewPosition);
}

/// End of function 'Asura_ServerEntity_Affector::ApplyChangesForTemplateInstance'. ///

#endif	//defined(ASURA_TOOLS)

#endif	// defined(ASURA_USE_AFFECTORS)


/// End of file. ///
