
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_ServerEntity_Affector_Wind.cpp
// Author : Andy Tate
// Date   : 8-Dec-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/Affector/Asura_ServerEntity_Affector_Wind.h"

// Asura independent header-files...
#include "Affectors/Asura_Affector_Global.h"
#include "Affectors/Asura_Affector_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Message.h"


#if defined(ASURA_USE_AFFECTORS)

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const u_int	g_uChunkVersion	= 2;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Classified_List<Asura_ServerEntity_Affector_Wind>	Asura_ServerEntity_Affector_Wind::s_xList;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::Asura_ServerEntity_Affector_Wind
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

Asura_ServerEntity_Affector_Wind::Asura_ServerEntity_Affector_Wind (const Asura_Guid uGuid)
: Asura_Entity_Server(uGuid)
{
	Classification		= AsuraEntityClass_Affector_Wind;
	m_iAffectorHandle	= Asura_Affector_System::CreateGlobalAffector();

	s_xList.AddToList(this);
}

/// End of function 'Asura_ServerEntity_Affector_Wind::Asura_ServerEntity_Affector_Wind'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::~Asura_ServerEntity_Affector_Wind
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

Asura_ServerEntity_Affector_Wind::~Asura_ServerEntity_Affector_Wind ()
{
	Asura_Affector_System::DestroyGlobalAffector(m_iAffectorHandle);

	s_xList.RemoveFromList(this);
}

/// End of function 'Asura_ServerEntity_Affector_Wind::~Asura_ServerEntity_Affector_Wind'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::MakeClientUpdateMessage
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector_Wind::MakeClientUpdateMessage (void)
{
}

/// End of function 'Asura_ServerEntity_Affector_Wind::MakeClientUpdateMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::Create
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector_Wind::Create (const Asura_Guid uGuid)
{
	new Asura_ServerEntity_Affector_Wind (uGuid);
}

/// End of function 'Asura_ServerEntity_Affector_Wind::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::Create
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector_Wind::Create (const Asura_Guid uGuid,
												 Asura_Chunk_Stream& xStream)
{
	Asura_ServerEntity_Affector_Wind* pxAffector = new Asura_ServerEntity_Affector_Wind (uGuid);

	pxAffector->ReadFromChunkStream(xStream);
}

/// End of function 'Asura_ServerEntity_Affector_Wind::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector_Wind::ReadFromChunkStream (Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Read the version...
	xStream >> uVersion;

	if(uVersion>1) // CW - only read from parent in new chunk versions (Previous versions were missing PARENT call)
	{
		PARENT::ReadFromChunkStream(xStream);
	}
	
	if(uVersion > g_uChunkVersion)
	{
		Asura_Assert(false, "Unexpected chunk version for global-affector entity.");
	}
	else
	{
		Asura_Affector_System::DestroyGlobalAffector(m_iAffectorHandle);
		m_iAffectorHandle	= Asura_Affector_System::CreateGlobalAffector();

		if(m_iAffectorHandle == ASURA_INVALID_HANDLE)
		{
			Asura_Assert(false, "Failed to create global-affector for reading.");
		}
		else
		{
			Asura_Affector_Global*	pxAffector	= Asura_Affector_System::GetGlobalAffectorPointer(m_iAffectorHandle);

			pxAffector->ReadFromChunkStream(xStream);
		}
	}

	SetNeverUpdates(true);
}

/// End of function 'Asura_ServerEntity_Affector_Wind::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::WriteToChunkStream
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector_Wind::WriteToChunkStream (Asura_Chunk_Stream &xStream) const
{
	Asura_Affector_Global*	pxAffector	= Asura_Affector_System::GetGlobalAffectorPointer(m_iAffectorHandle);

	xStream << g_uChunkVersion;

	PARENT::WriteToChunkStream(xStream);

	if(pxAffector == NULL)
	{
		Asura_Assert(false, "Failed to find global-affector to write.");
	}
	else
	{
		pxAffector->WriteToChunkStream(xStream);
	}
}

/// End of function 'Asura_ServerEntity_Affector_Wind::WriteToChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::OnEntityActivate
// Author       : Neil Young
// Date         : 21-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector_Wind::OnEntityActivate()
{
	PARENT::OnEntityActivate();

	Asura_Affector_Global*	pxAffector	= Asura_Affector_System::GetGlobalAffectorPointer(m_iAffectorHandle);
	if(pxAffector == NULL)
	{
		Asura_Assert(false, "Failed to find global-affector to activate.");
		return;
	}
	pxAffector->SetAsActive(true);
}

/// End of function 'Asura_ServerEntity_Affector_Wind::OnEntityActivate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::OnEntityDeactivate
// Author       : Neil Young
// Date         : 21-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_Affector_Wind::OnEntityDeactivate()
{
	PARENT::OnEntityDeactivate();

	Asura_Affector_Global*	pxAffector	= Asura_Affector_System::GetGlobalAffectorPointer(m_iAffectorHandle);
	if(pxAffector == NULL)
	{
		Asura_Assert(false, "Failed to find global-affector to deactivate.");
		return;
	}
	pxAffector->SetAsActive(false);
}

/// End of function 'Asura_ServerEntity_Affector_Wind::OnEntityDeactivate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::GetAffector
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Affector_Global* Asura_ServerEntity_Affector_Wind::GetAffector (void)
{
	return(Asura_Affector_System::GetGlobalAffectorPointer(m_iAffectorHandle));
}

/// End of function 'Asura_ServerEntity_Affector_Wind::GetPosition'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_Affector_Wind::GetAffector
// Author       : Andy Tate
// Date         : 6-Dec-2006
////////////////////////////////////////////////////////////////////////////////

const Asura_Affector_Global* Asura_ServerEntity_Affector_Wind::GetAffector (void) const
{
	return(Asura_Affector_System::GetGlobalAffectorPointer(m_iAffectorHandle));
}

/// End of function 'Asura_ServerEntity_Affector_Wind::GetPosition'. ///

#endif	// defined(ASURA_USE_AFFECTORS)


/// End of file. ///
