//
// Asura_Entity_PeriodicClientUpdateList.h
//
// Created by KOR - 22/03/2006
// 
// Contains Asura_Entity_PeriodicClientUpdateList, which previously resided in Asura_Entity.cpp,
// but was split off for clarity.
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_ENTITY_PERIODICCLIENTUPDATELIST_H__
#define __ASURA_ENTITY_PERIODICCLIENTUPDATELIST_H__

#include "Network/Asura_Guid.h"

// Forward declares
class Asura_Entity_Server;

// Constants
const u_int uPERIODICCLIENTUPDATELIST_HASHSIZE = 32;	// must be power of 2

enum Asura_Entity_PeriodicClientUpdateList_UpdateMode
{
	ClientUpdateList_UpdateMode_Always,
	ClientUpdateList_UpdateMode_Burst,
	ClientUpdateList_UpdateMode_Heartbeat,
};

class Asura_Entity_PeriodicClientUpdateList_Node
{
	friend class Asura_Entity_PeriodicClientUpdateList;

private:

	Asura_Entity_PeriodicClientUpdateList_Node(Asura_Entity_Server* pxEntity)
	{
		m_pxEntity = pxEntity;
		m_uNextUpdateTime = 0;
		m_eMode = ClientUpdateList_UpdateMode_Always;
		m_pxNext = NULL;
	}

	Asura_Guid GetGuid() const;

	// would like to make this const, but MakeClientUpdateMessage() isn't a const member
	Asura_Entity_Server* m_pxEntity;
	u_int m_uNextUpdateTime;
	Asura_Entity_PeriodicClientUpdateList_UpdateMode m_eMode;
	Asura_Entity_PeriodicClientUpdateList_Node* m_pxNext;
};

class Asura_Entity_PeriodicClientUpdateList
{
public:

	Asura_Entity_PeriodicClientUpdateList();

	void Reset();

	Asura_Entity_Server* FindEntity( Asura_Guid uGuid, bool bIgnoreDestroyedEntities = true ) const;

	void SetAlwaysUpdate(Asura_Entity_Server* pxEntity, bool bEnable);
	void UpdateNeeded(Asura_Entity_Server* pxEntity);
	Asura_Entity_PeriodicClientUpdateList_Node* AddToList(Asura_Entity_Server* pxEntity);
	bool RemoveFromList(const Asura_Entity_Server* pxEntity);
	u_int GetHashIndex( Asura_Guid uGUID ) const { return uGUID & (uPERIODICCLIENTUPDATELIST_HASHSIZE - 1); }
    bool IsInList( Asura_Guid uGUID ) const { return( FindEntry(uGUID) != NULL ); }
	int GetUpdateMode( Asura_Guid uGUID ) const;

	void DoSomeClientUpdates();
	void FinishClientUpdates();

private:

	void CalculateNumUpdatesPerFrame();
	void DoClientUpdates();

	Asura_Entity_PeriodicClientUpdateList_Node* FindEntry( Asura_Guid uGUID ) const;

	Asura_Entity_PeriodicClientUpdateList_Node* m_apxHashTable[uPERIODICCLIENTUPDATELIST_HASHSIZE];
	u_int m_uLastHashIndex;
	Asura_Entity_PeriodicClientUpdateList_Node* m_pxNextEntry;
	u_int m_uNumUpdatesPerFrame;
	u_int m_uNumNodes;
};


#endif//__ASURA_ENTITY_PERIODICCLIENTUPDATELIST_H__
