//
// Asura_Entity_System.cpp
//
// Created by KOR - 22/03/2006
// 
// Contains Asura_Entity_System, which previously resided in Asura_Entity.cpp,
// but was split off for clarity.
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include <wchar.h>

// This file's header
#include "Entity/Asura_Entity_System.h"

// Asura headers
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Timers.h"
#include "Cutscene/Asura_ClientEntity_CutsceneController.h"
#ifdef ASURA_SUPPORT_SERVERCUTSCENES	// the Gunfighter-style cutscenes with entities
#include "Cutscene/Asura_ClientEntity_ServerCutsceneActor.h"
#endif //def ASURA_SUPPORT_SERVERCUTSCENES
#include "Debugging/Asura_CPUProfile.h"
#include "Debugging/Asura_Logfile.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_Entity_PeriodicClientUpdateList.h"
#ifdef ASURA_SUPPORT_ENTITYWATCH
#include "Entity/Asura_EntityWatch_System.h"
#endif//def ASURA_SUPPORT_ENTITYWATCH
#include "Entity/Asura_StaticMessage_System.h"

#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "Entity/Container/Asura_ClientEntity_Container.h"
#include "Entity/Container/Asura_Container_Messages.h"
#include "Entity/Container/Asura_ServerEntity_Container.h"
#endif //ASURA_USE_CONTAINER_SYSTEM

#include "Entity/DynamicLight/Asura_ServerEntity_DynamicLight.h"
#include "Entity/Glass/Asura_ClientEntity_Glass.h"
#include "Entity/Objective/Asura_ClientEntity_Objective.h"
#include "Entity/Lightshaft/Asura_ServerEntity_Lightshaft.h"
#include "Entity/ParticleGenerator/Asura_ServerEntity_PFX_Effect.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicsObject.h"
#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER
#include "Entity/Triggers/Asura_ClientEntity_DebugWaypointTrigger.h"
#endif
#ifndef ASURA_TARGET_SM3
#include "Entity/PhysicalObject/Asura_ClientEntity_AdvancedLight.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_AdvancedLight.h"
#endif // #ifndef ASURA_TARGET_SM3
#include "Entity/Triggers/EffectVolume/Asura_ClientEntity_EffectVolume.h"
#include "Entity/Triggers/Asura_ClientEntity_LookAtTrigger.h"
#include "Entity/Triggers/Asura_ServerEntity_VolumeTrigger.h"
#include "Interface/Asura_CommandConsole.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Player.h"
#include "Rendering/Asura_SortedRender.h"
#if defined(ASURA_USE_STREAMING)
#include "Streaming/Asura_Streaming_HandlerReplacement.h"
#include "Streaming/Asura_Streaming_System.h"
#endif	// defined(ASURA_USE_STREAMING)

// Tools headers
#ifdef ASURA_TOOLS
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Entity_Simulation.h"
#include "Load&Save/Tools_SaveAsura.h"
#endif

#if _DEBUG || defined(ASURA_TOOLS)
// These next 4 includes relate to rendering the dynamics bounding-boxes.
//#include "Dynamics/Asura_Collision_Object.h"
#include "Interface/Asura_CommandConsole.h"
//#include "Rendering/Asura_Render.h"
#endif

#ifdef ASURA_USE_ZPREPASS
#include "Rendering/Asura_Render.h"
#endif

// (MH - 06.11.2007) testing whether waiting for server entities to update the 
// client entities before calling the Update() function on client entities helps
// with off-by-one frame lag
//#define ASURA_ENTITY_DO_CLIENT_UPDATES_AT_END_OF_FRAME 1

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int DESTROY_LIST_MAX_LEN = 16;

static const Asura_UnicodeChar* const wszASURA_DEACTIVATE_DISTANT_ENTITIES = L"Debug.Entity.DeactivateDistantEntities";
static const Asura_UnicodeChar* const wszASURA_DEACTIVATE_ENTITY_DISTANCE = L"Debug.Entity.DeactivateDistance";

static const Asura_UnicodeChar* const wszASURA_NUM_RENDERABLE_ENTITY_DVS_ENTRY = L"Debug.Entity.RenderableEntities";

static const Asura_UnicodeChar* const wszASURA_NUM_CLIENTS_DVS_ENTRY = L"Debug.Entity.ClientEntities";
static const Asura_UnicodeChar* const wszASURA_NUM_UPDATING_CLIENT_ENTITY_DVS_ENTRY = L"Debug.Entity.ClientEntitiesUpdating";
static const Asura_UnicodeChar* const wszASURA_PROFILE_CLIENT_ENTITIES_DVS_ENTRY = L"Debug.Entity.ClientEntitiesUpdateTime (ms)";

static const Asura_UnicodeChar* const wszASURA_NUM_SERVERS_DVS_ENTRY = L"Debug.Entity.ServerEntities";
static const Asura_UnicodeChar* const wszASURA_NUM_UPDATING_SERVER_ENTITY_DVS_ENTRY = L"Debug.Entity.ServerEntitiesUpdating";
static const Asura_UnicodeChar* const wszASURA_PROFILE_SERVER_ENTITIES_DVS_ENTRY = L"Debug.Entity.ServerEntitiesUpdateTime (ms)";

static const Asura_UnicodeChar* const wszASURA_LOG_ENTITY_PROFILING = L"Debug.Entity.LogEntityProfiling";

static const u_int uASURA_MAX_ALLOWED_MESSAGE_RECURSION	= 20;

// G L O B A L S /////////////////////////////////////////////////////////////

static Asura_Entity*	g_apxDestroyList[DESTROY_LIST_MAX_LEN];
static u_int			g_uDestroyListLen			= 0;
static bool				g_bExceededMaxListLen		= false;
static bool				g_bFlushingDestroyList		= false;
static u_int			g_uMessageRecursionCount	= 0;

DVSVAR float			dvs_fDistToDeactivateEntities	= 50.0f;

bool Asura_Entity_System::s_bUpdateInactiveClientEntitiesAtEndOfFrame;
bool Asura_Entity_System::s_bCurrentlyDestroyingAllEntities	= false;

#if defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
// Externally accessible so that projects can choose to expose it as a console var, debugtext, etc
u_int	Asura_Entity_System::s_uNumRenderableEntities		= 0;
u_int	Asura_Entity_System::s_auNumRenderableEntitiesPerClassification[ASURA_MAX_ENTITY_PROFILE_CLASSIFICATIONS];

Asura_Entity_System_ProfileData	Asura_Entity_System::s_xClientEntityProfileData;
Asura_Entity_System_ProfileData	Asura_Entity_System::s_xServerEntityProfileData;
#endif

// S T A T I C S /////////////////////////////////////////////////////////////

ProjectDeliverMessageToServerEntityFunction* Asura_Entity_System::s_pfnProjectDeliverMessageToServerEntity = NULL;

// S T A T I C   F U N C T I O N S ///////////////////////////////////////////

#if defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)

template <class T> static u_int CreateTemporaryDVSEntityList(const Asura_UnicodeChar* wszEntryName,
															 u_int* puEntityCount,
															 Asura_Classified_List<T>& xList,
															 u_int* puEntitiesPerClassification)
{
	// if we're looking at the num rendering entities in the DVS, generate DVS entries for all the entity classifications.
	if(wcscmp(Asura_DebugVariableSystem::GetSelectedPath(), wszEntryName) == 0)
	{
		u_int								uNumEntityClassificationsUsed	= 0;

		Asura_Classified_List_Const_It<T>	xIterator(&xList);

		// Clean-out the old debug-variables...
		Asura_CommandConsole::RemoveVar(wszEntryName, true);
		Asura_CommandConsole::AddVar(wszEntryName, puEntityCount, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::MakeVarReadOnly(wszEntryName);

		while(!xIterator.Done())
		{
			const T*		pxEntity			= xIterator.GetCurrent();
			const u_short	usCurClassification = pxEntity->GetClassification();
			u_int			uCount				= 1;

			xIterator.Next();

			// continue iterating until we get to the next classification or the end of the list.
			while(!xIterator.Done())
			{
				pxEntity	= xIterator.GetCurrent();

				if(pxEntity->GetClassification() == usCurClassification)
				{
					++uCount;
					xIterator.Next();
				}
				else
				{
					break;
				}
			}

			// add var if it doesn't exist and set its value if it does
			if(uNumEntityClassificationsUsed < ASURA_MAX_ENTITY_PROFILE_CLASSIFICATIONS)
			{
				u_int&				uVar	= puEntitiesPerClassification[uNumEntityClassificationsUsed++];

				Asura_UnicodeChar	awcVarName[256];

				uVar = uCount;

				// remove at reset
				Asura_PrintfToBfr(awcVarName, L"%s.%S", 256, true, wszEntryName, Asura_Network::GetEntityClassificationDesc(usCurClassification));
				Asura_CommandConsole::AddVar( awcVarName, &uVar );
				Asura_CommandConsole::MakeVarReadOnly( awcVarName );
			}
			else
			{
				Asura_Assert(false, "too many entity classifications");
			}
		}

		return(uNumEntityClassificationsUsed);
	}
	else
	{
		return(0);
	}
}

#endif	//defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

Asura_Hashed_List<Asura_RecentlyDestroyedEntity> Asura_RecentlyDestroyedEntity_System::s_xList[2] = 
{
	Asura_Hashed_List<Asura_RecentlyDestroyedEntity>(5),
		Asura_Hashed_List<Asura_RecentlyDestroyedEntity>(5)		
};

float Asura_RecentlyDestroyedEntity_System::s_fTimer;
u_int Asura_RecentlyDestroyedEntity_System::s_uCurList;


void Asura_RecentlyDestroyedEntity_System::UpdateLists(void)
{
	s_fTimer += Asura_Timers::GetFrameTime();
	if (s_fTimer > 5.0f)
	{
		s_fTimer = 0.0f;
		s_uCurList = 1 - s_uCurList;
		EmptyList(s_uCurList);
	}
}

void Asura_RecentlyDestroyedEntity_System::Reset()
{
	s_fTimer = 0.0f;
	s_uCurList = 0;
	EmptyList(0);
	s_uCurList = 1;
	EmptyList(1);
	s_uCurList = 0;
}

bool Asura_RecentlyDestroyedEntity_System::HasRecentlyBeenDestroyed(Asura_Guid uGuid)
{
	if (s_xList[0].FindObject(uGuid)) return true;
	if (s_xList[1].FindObject(uGuid)) return true;
	return false;
}

void Asura_RecentlyDestroyedEntity_System::AddDestroyedEntity(Asura_Guid uGuid)
{
	// add it to the current list (even if it happens to be in the other list)
	if (!s_xList[s_uCurList].FindEntity(uGuid))
	{
		new Asura_RecentlyDestroyedEntity(uGuid);
	}
}

void Asura_RecentlyDestroyedEntity_System::EmptyList(u_int uListID)
{
	Asura_Hashed_List_It<Asura_RecentlyDestroyedEntity> xIt(&s_xList[uListID]);
	while (!xIt.Done())
	{
		Asura_RecentlyDestroyedEntity* pxEntry = xIt.GetCurrent();
		xIt.Next();
		delete pxEntry;
	}
}
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// Static Asura_Entity_System members
bool		Asura_Entity_System::s_bAssignGuidsFromCutsceneRange;

#if defined(ASURA_TOOLS)
Asura_Guid Asura_Entity_System::s_auNextToolsCreatedGuid[TOOLS_NUM_GUIDRANGE];

//Min and max guids for each range
static const Asura_Guid g_auMinToolsCreatedGuid[TOOLS_NUM_GUIDRANGE] =
{
	ASURA_GUID_TOOLS_ENTITYFILE_START,
	ASURA_GUID_TOOLS_CUTSCENEFILE_START,
	ASURA_GUID_TOOLS_SOUNDFILE_START,
	ASURA_GUID_TOOLS_MISSIONSOUNDFILE_START,
};

static const Asura_Guid g_auMaxToolsCreatedGuid[TOOLS_NUM_GUIDRANGE] =
{
	ASURA_GUID_TOOLS_ENTITYFILE_END,
	ASURA_GUID_TOOLS_CUTSCENEFILE_END,
	ASURA_GUID_TOOLS_SOUNDFILE_END,
	ASURA_GUID_TOOLS_MISSIONSOUNDFILE_END,
};
#endif	// defined(ASURA_TOOLS)

Asura_Guid	Asura_Entity_System::NextGameCreatedGuid;
Asura_Guid	Asura_Entity_System::NextClientGuid;

#if !defined(ASURA_USE_STREAMING)
Asura_Entity_PeriodicClientUpdateList  Asura_Entity_System::s_xPeriodicClientUpdateList;
Asura_Hashed_List<Asura_Entity_Server> Asura_Entity_System::s_xConstantClientUpdateList;
#endif //!defined(ASURA_USE_STREAMING)


Asura_DynamicBVH<Asura_Entity_Renderable> Asura_Entity_System::s_xLineOfSightBVH;

// KJFL - 6 November 2007 - TODO
// The heuristics calling Create & Refit could be moved to the BVH class itself?
// Haven't quite decided whether these are globals, members of the entity system (which
// is what they're currently named as i.e. s_) or should be in the DynamicBVH class.


static u_int s_uLineOfSightLastUpdatedStamp = 0;
static u_int s_uLineOfSightLastRefitStamp = 0;
static u_int s_uNumberOfRefits = 0;

void Asura_Entity_System::Initialise(void)
{
	NextGameCreatedGuid = ASURA_GUID_GAMECREATEDENTITY_START;
	NextClientGuid = ASURA_GUID_UNREGISTERED; 

#if defined(ASURA_TOOLS)
	for(int i=0; i<TOOLS_NUM_GUIDRANGE; i++)
	{
		s_auNextToolsCreatedGuid[i] = g_auMinToolsCreatedGuid[i];
	}
#endif	// defined(ASURA_TOOLS)
	s_bAssignGuidsFromCutsceneRange = false;

#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)

#if defined(ASURA_USE_PROFILING_SYSTEM)
	Asura_CommandConsole::AddVar( wszASURA_NUM_CLIENTS_DVS_ENTRY, &s_xClientEntityProfileData.m_uNumEntities, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszASURA_NUM_CLIENTS_DVS_ENTRY );
	Asura_CommandConsole::AddVar( wszASURA_NUM_SERVERS_DVS_ENTRY, &s_xServerEntityProfileData.m_uNumEntities, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszASURA_NUM_SERVERS_DVS_ENTRY );
	Asura_CommandConsole::AddVar( wszASURA_NUM_RENDERABLE_ENTITY_DVS_ENTRY, &s_uNumRenderableEntities, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszASURA_NUM_RENDERABLE_ENTITY_DVS_ENTRY );
	Asura_CommandConsole::AddVar( wszASURA_NUM_UPDATING_CLIENT_ENTITY_DVS_ENTRY, &s_xClientEntityProfileData.m_uNumEntitiesUpdating, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszASURA_NUM_UPDATING_CLIENT_ENTITY_DVS_ENTRY );
	Asura_CommandConsole::AddVar( wszASURA_NUM_UPDATING_SERVER_ENTITY_DVS_ENTRY, &s_xServerEntityProfileData.m_uNumEntitiesUpdating, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszASURA_NUM_UPDATING_SERVER_ENTITY_DVS_ENTRY );
	Asura_CommandConsole::AddVar( wszASURA_PROFILE_CLIENT_ENTITIES_DVS_ENTRY, &s_xClientEntityProfileData.m_fEntitiesUpdateTime, 0, FLT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszASURA_PROFILE_CLIENT_ENTITIES_DVS_ENTRY );
	Asura_CommandConsole::AddVar( wszASURA_PROFILE_SERVER_ENTITIES_DVS_ENTRY, &s_xServerEntityProfileData.m_fEntitiesUpdateTime, 0, FLT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszASURA_PROFILE_SERVER_ENTITIES_DVS_ENTRY );
	Asura_CommandConsole::SetVarHelpText( wszASURA_PROFILE_SERVER_ENTITIES_DVS_ENTRY, "microseconds updating all entities. Each entity in sublist indicates a percentage of this number spent updating all entities of that type." );

	Asura_CommandConsole::AddCmd( wszASURA_LOG_ENTITY_PROFILING, ConsoleCmd_LogEntityProfiling, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
#endif	//defined(ASURA_USE_PROFILING_SYSTEM)

	Asura_CommandConsole::AddCmd( wszASURA_DEACTIVATE_DISTANT_ENTITIES, ConsoleCmd_DeactivateDistantEntities, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( wszASURA_DEACTIVATE_ENTITY_DISTANCE, &dvs_fDistToDeactivateEntities, 5.0f, 999.9f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

#ifdef ASURA_TOOLS
	Asura_CommandConsole::AddCmd( L"Debug.Entity.PrintUpdatingReport", ConsoleCmd_PrintUpdatingEntityReport, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif

#endif	//defined(ASURA_DEBUG_VARIABLE_SYSTEM)

	s_uLineOfSightLastUpdatedStamp = 0;
	s_uLineOfSightLastRefitStamp = 0;
	s_uNumberOfRefits = 0;

}

void Asura_Entity_System::Reset(void)
{
    DestroyAllEntities();

    NextGameCreatedGuid = ASURA_GUID_GAMECREATEDENTITY_START;
	NextClientGuid = ASURA_GUID_UNREGISTERED; 
	
	//moved resetting of next tools guid to another function , since it shouldn't be
	//done when reloading entities from a save. RAR 28/03/03

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	// (MH - 02.09.2003) I'm assuming that this network flag will not be 
	// cleared until after this function has been called.
	if (Asura_Network::IsExternalNetworkActive())
	{
		Asura_RecentlyDestroyedEntity_System::Reset();
	}
#endif

#if defined(ASURA_USE_STREAMING)
	Asura_Streaming_System::ResetPeriodicClientUpdateList();
	Asura_Streaming_System::ResetConstantClientUpdateList();
#else
	s_xPeriodicClientUpdateList.Reset();

	for( Asura_Hashed_List_Deleter_It<Asura_Entity_Server> xIt(&s_xConstantClientUpdateList); !xIt.Done(); )
    {
        xIt.DeleteCurrent();
    }
#endif

#ifdef ASURA_TOOLS
	Tools_Entity_Simulation::ResetLogFile();
#endif

	s_uLineOfSightLastUpdatedStamp = 0;
	s_uLineOfSightLastRefitStamp = 0;
	s_uNumberOfRefits = 0;
	s_xLineOfSightBVH.Reset();

}

#if defined(ASURA_TOOLS)
void Asura_Entity_System::ResetNextToolsGuid(void)
{
	for(int i=0; i<TOOLS_NUM_GUIDRANGE; i++)
	{
		s_auNextToolsCreatedGuid[i] = g_auMinToolsCreatedGuid[i];
	}
	s_bAssignGuidsFromCutsceneRange = false;
}
#endif

void Asura_Entity_System::ResetRecentlyDestroyedSystem()
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	if (Asura_Network::IsExternalNetworkActive())
	{
		Asura_RecentlyDestroyedEntity_System::Reset();
	}
#endif
}

void Asura_Entity_System::Shutdown(void)
{
    DestroyAllEntities();
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_RecentlyDestroyedEntity_System::Shutdown();
#endif
	
#if defined(ASURA_USE_STREAMING)
	Asura_Streaming_System::ResetPeriodicClientUpdateList();
    Asura_ServerEntity_PhysicalObject::ShutdownLinkBlockPass();
#else
	s_xPeriodicClientUpdateList.Reset();
#endif
}

void Asura_Entity_System::InitialiseServerEntitiesFromBlueprints()
{
    for ( Asura_Classified_List_It< Asura_Entity_Server > xServerIter( Asura_Entity_Server::GetList() ); !xServerIter.Done(); xServerIter.Next() )
    {
        xServerIter.GetCurrent()->InitFromBlueprint();
    }
}

void Asura_Entity_System::PostLoadInitialise(void)
{
    // If this isn't a streaming build then post-load initialise all of the server entities.
    // If it is a streaming build then server entities will be post-load initialised on a per-zone basis as zones are loaded in.
#if defined(ASURA_TOOLS) || !defined(ASURA_USE_STREAMING)
	for ( Asura_Classified_List_It< Asura_Entity_Server > xServerIter( Asura_Entity_Server::GetList() ); !xServerIter.Done(); xServerIter.Next() )
	{
		xServerIter.GetCurrent()->PostLoadInitialise();
	}
#endif

}

void Asura_Entity_System::PostCreateInitialise(void)
{
	for (Asura_Classified_List_It<Asura_Entity_Client> xClientIter(Asura_Entity_Client::GetList()); !xClientIter.Done(); xClientIter.Next() )
	{
		xClientIter.GetCurrent()->PostCreateInitialise();
	}
}

// UpdateInactiveClientEntitiesAtEndOfFrame() - see declaration for description
void Asura_Entity_System::UpdateInactiveClientEntitiesAtEndOfFrame()
{ 
	Asura_Assert(!Asura_Network::IsExternalNetworkActive(), "shouldn't be called for multplayer games");
	s_bUpdateInactiveClientEntitiesAtEndOfFrame = true; 
}



void Asura_Entity_System::RegisterClientEntity(Asura_Entity* entity)
{
	Asura_Assert(entity,"Invalid pointer");
	// Asura_Assert(entity->Guid==ASURA_GUID_UNREGISTERED,"Already Registered?");

	/// Richard May - 19/04/2004 14:33:31 - in complex multiple inheritance structures (like Rogue's client 
	/// player) we will get here with a valid guid we wish to keep, so don't register if not necessary
	if( entity->Guid == ASURA_GUID_UNREGISTERED )
	{
		entity->Guid = NextClientGuid;
		NextClientGuid = ASURA_GUID_UNREGISTERED;
	}

	//Asura_Assert(entity->Guid!=ASURA_GUID_UNREGISTERED,"No guid provided");
}

Asura_Guid Asura_Entity_System::GetCreatedByGameGuid()
{
	return NextGameCreatedGuid++;
}

void Asura_Entity_System::RegisterServerEntity(Asura_Entity* entity, bool createdByTools)
{
	Asura_Assert(entity,"Invalid pointer");
	Asura_Assert(entity->Guid==ASURA_GUID_UNREGISTERED,"Already Registered?");

#if defined(ASURA_TOOLS)
	if (createdByTools)
	{
		entity->Guid = GetCreatedByToolsGuid();
	}
	else
#endif	// defined(ASURA_TOOLS)
	{
		entity->Guid = GetCreatedByGameGuid();
	}

#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
	entity->Guid	= Asura_Streaming_System::MakeCurrentMasterZoneGuid(entity->Guid);
#endif	// defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
}

void Asura_Entity_System::RegisterGameCreatedGuid(Asura_Guid guid)
{
	Asura_Assert(guid>=ASURA_GUID_GAMECREATEDENTITY_START,"");
	if(guid>=NextGameCreatedGuid)
	{
		NextGameCreatedGuid = guid+1;
	}
}

#if defined(ASURA_TOOLS)

Asura_Guid Asura_Entity_System::GetCreatedByToolsGuid(void)
{
	if(s_bAssignGuidsFromCutsceneRange)
		return GetCreatedByToolsGuid_CutsceneFile();
	else
		return GetCreatedByToolsGuid_EntityFile();
}

Asura_Guid Asura_Entity_System::GetCreatedByToolsGuid(Tools_GuidRange eRange)
{
	Asura_Assert(s_auNextToolsCreatedGuid[eRange] < g_auMaxToolsCreatedGuid[eRange]-10,"Running out of guids. Don't create any more entities");
	return s_auNextToolsCreatedGuid[eRange]++;
}


Asura_Guid Asura_Entity_System::GetNextToolsCreatedGuid()
{
	if(s_bAssignGuidsFromCutsceneRange)
		return GetNextToolsCreatedGuid(TOOLS_GUIDRANGE_CUTSCENE);
	else
		return GetNextToolsCreatedGuid(TOOLS_GUIDRANGE_ENTITY);
}

Asura_Guid Asura_Entity_System::GetNextToolsCreatedGuid(Tools_GuidRange eRange)
{
	Asura_Assert(s_auNextToolsCreatedGuid[eRange] < g_auMaxToolsCreatedGuid[eRange]-10,"Running out of guids. Don't create any more entities");
	return s_auNextToolsCreatedGuid[eRange];
}


void Asura_Entity_System::RegisterToolsGuid(Asura_Guid uGuid)
{
	if(uGuid>=ASURA_GUID_TOOLSCREATEDENTITY_START && uGuid<=ASURA_GUID_TOOLSCREATEDENTITY_END)
	{

		for(int i=0; i<TOOLS_NUM_GUIDRANGE; i++)
		{
			//Check to see if the guid is in this range.
			//Strictly speaking the test against the maximum should be <= , but it is probably safer to make sure
			//that RegisterToolsGuid can't push the next guid out of range.
			if(uGuid >= g_auMinToolsCreatedGuid[i] && uGuid < g_auMaxToolsCreatedGuid[i])
			{
				if(uGuid>=s_auNextToolsCreatedGuid[i])
				{
					s_auNextToolsCreatedGuid[i] = uGuid+1;
				}
				return;
			}
		}

		//Ignoring project guids (rightly or wrongly) since that is what was being done before.
		//Not sure they are being used at the moment anyway. RAR 30/01/2009
		if(uGuid>=ASURA_GUID_TOOLS_PROJECT_START && uGuid <=ASURA_GUID_TOOLS_PROJECT_END)
		{
			return;
		}
	}

	Asura_Assert(0,"Unexpected tools guid value being registered");
}

void Asura_Entity_System::SetNextToolsCreatedGuid(Tools_GuidRange eRange, Asura_Guid uGuid)
{
	if(uGuid >= g_auMinToolsCreatedGuid[eRange] && uGuid <= g_auMaxToolsCreatedGuid[eRange])
	{
		if(uGuid > s_auNextToolsCreatedGuid[eRange])
		{
			s_auNextToolsCreatedGuid[eRange] = uGuid;
		}
	}
	else
	{
		Asura_Assert(0, "Guid passed to SetNextToolsCreatedGuid() isn't within the acceptable range of guids");
	}
}

#endif	// defined(ASURA_TOOLS)

bool Asura_Entity_System::DeliverMessageToClientEntity(const Asura_Message* pxMessage)
{
    if( !Asura_Network::ShouldMessageBeHandledOnClient(pxMessage) ) { return true; }

	if (Asura_Entity_Client::HasRecentlyBeenDestroyed(pxMessage->From))
	{
		Asura_OutputToDebugger(("not delivering message ID %d for recently-destroyed %d.\n", pxMessage->GetMsgID(), pxMessage->From));
		return false;
	}
	if( g_uMessageRecursionCount > uASURA_MAX_ALLOWED_MESSAGE_RECURSION )
	{
		Asura_Assertf( false, ("Probable infinite loop in messages sent from GUID %u to GUID %u", pxMessage->From, pxMessage->To) );
		return false;
	}
	g_uMessageRecursionCount++;
	
    Asura_Entity_Client* pxEntity = Asura_Entity_Client::FindObject( pxMessage->From, false );

	bool bDealtWithMessage = false;
	if (pxEntity)
	{
        if( !pxEntity->IsDestroyed() )
        {
		    pxEntity->HandleMessage( pxMessage );
        }
		bDealtWithMessage = true;
	}
    else
	{
#if defined(ASURA_USE_STREAMING)
		const u_int	uNumMissions		= Asura_Streaming_System::GetNumZones(Asura_Streaming_System::MISSION_DATA);
		const u_int	uNumZones			= Asura_Streaming_System::GetNumZones(Asura_Streaming_System::ZONE_DATA);
		const bool	bCreationAllowed	= !g_bCurrentlyDestroyingAllEntities &&
										  ((uNumMissions > 0) || (uNumZones > 0));
#else
		const bool	bCreationAllowed	= !s_bCurrentlyDestroyingAllEntities;
#endif	// defined(ASURA_USE_STREAMING)

		if( bCreationAllowed )
		{
			NextClientGuid = pxMessage->From;

			if (Asura_Entity_System::ClientCreateEntity(pxMessage))
			{
				bDealtWithMessage = true;
#ifdef ASURA_ALLOW_INACTIVE_CLIENT_ENTITIES
				if ( pxMessage->GetMsgID() == ASURA_MSG_CLIENTCREATEINACTIVE )
				{
					Asura_Entity_Client* pxNewEntity = Asura_Entity_Client::FindEntity( pxMessage->From );
					if ( pxNewEntity )
					{
						pxNewEntity->SetEntityActive( false );
					}
				}
#endif //#ifdef ASURA_ALLOW_INACTIVE_CLIENT_ENTITIES
			}
			// (MH - 01.09.2003) must reset the NextClientGuid in case that didn't actually
			// create a client entity but returned true because it handled the message in 
			// some other way.
			NextClientGuid = ASURA_GUID_UNREGISTERED;
		}
	}

	--g_uMessageRecursionCount;

	return bDealtWithMessage;
}

bool Asura_Entity_System::DeliverMessageToServerEntity(const Asura_Message* pxMessage)
{
	if (!Asura_Network::IsServer())
	{
		Asura_Assert(0,"hey, we're not server");
		return false;
	}

	if(s_pfnProjectDeliverMessageToServerEntity)
	{
		if(s_pfnProjectDeliverMessageToServerEntity(pxMessage))
		{
			return true;
		}
	}

	Asura_Entity_Server* pxEntity = Asura_Entity_Server::FindEntity(pxMessage->To);

	if (pxEntity)
	{
		if( g_uMessageRecursionCount > uASURA_MAX_ALLOWED_MESSAGE_RECURSION )
		{
			Asura_Assertf( false, ("Probable infinite loop in messages sent from GUID %u to GUID %u", pxMessage->From, pxMessage->To) );
			return false;
		}
		g_uMessageRecursionCount++;

#ifdef ASURA_ASSERT
		bool bExpectingEventMessage = pxEntity->ExpectingBaseEventMessage(pxMessage->GetMsgID());
#endif

#if defined(ASURA_USE_STREAMING)
		// Save handlers, set new ones, restore old ones on leaving scope.
		Asura_Streaming_HandlerReplacement xSaveHandlers(pxEntity->GetZoneHashID(), pxEntity->GetContainingZoneHashID());
#endif	// defined(ASURA_USE_STREAMING)

		// get the entity to handle the message - this should also trigger any side-effects
		// such as sending event messages e.g. OnActivation.
		pxEntity->HandleMessage(pxMessage);

#ifdef ASURA_ASSERT
		if (bExpectingEventMessage)
		{
			Asura_Assertf(pxEntity->SentBaseEventMessage(), 
				("entity of type %s should have sent message on Activation/Deactivation but didn't", 
				Asura_Network::GetEntityClassificationDesc(pxEntity->GetClassification())));
		}
#endif // #ifdef ASURA_ASSERT
		--g_uMessageRecursionCount;

		return true;
	}	
#if 0
	else if (0)
	{
		Asura_Assert(0,"Server could not identify object");
		/// Kevin Lea - 04/12/2001 11:00:04 - probably not serious
		/// we've probably just destroyed this object recently

		return false;
	}
#endif	

	return false;
}

bool Asura_Entity_System::DeliverMessageToPlayerEntity(const Asura_Message* pxMessage)
{
	Asura_Entity_Player* pxEntity = Asura_Entity_Player::FindEntity(pxMessage->To);

	if (pxEntity)
	{
		if( g_uMessageRecursionCount > uASURA_MAX_ALLOWED_MESSAGE_RECURSION )
		{
			Asura_Assertf( false, ("Probable infinite loop in messages sent from GUID %u to GUID %u", pxMessage->From, pxMessage->To) );
			return false;
		}
		g_uMessageRecursionCount++;

		pxEntity->HandleMessage(pxMessage);

		g_uMessageRecursionCount--;
		return true;
	}	
	else
	{
		Asura_Assert(0,"No player with that ID");
		return false;
	}

	
}

bool Asura_Entity_System::DeliverMessageToOtherPlayers(const Asura_Message* pxMessage)
{
	/// Richard May - 02/08/2002 16:19:56 - broadcasts to all players (SNH: except the one in From field)
	Asura_Hashed_List_It<Asura_Entity_Player> xIterator(Asura_Entity_Player::GetList());
	while( !xIterator.Done() )
	{
		Asura_Entity_Player* pxPlayer = xIterator.GetCurrent();
		xIterator.Next();

		if( pxPlayer->GetGuid() != pxMessage->From )  
		{
			if( g_uMessageRecursionCount > uASURA_MAX_ALLOWED_MESSAGE_RECURSION )
			{
				Asura_Assertf( false, ("Probable infinite loop in messages sent from GUID %u to GUID %u", pxMessage->From, pxMessage->To) );
				return false;
			}
			g_uMessageRecursionCount++;

			pxPlayer->HandleMessage(pxMessage);
			--g_uMessageRecursionCount;
		}
	}

	return true;
}

bool Asura_Entity_System::DeliverMessageToAllPlayers(const Asura_Message* pxMessage)
{
	// SNH - 07/06/2003 11:51:44 - send to all players
	Asura_Hashed_List_It<Asura_Entity_Player> xIterator(Asura_Entity_Player::GetList());
	while( !xIterator.Done() )
	{
		Asura_Entity_Player* pxPlayer = xIterator.GetCurrent();
		xIterator.Next();

		if( pxPlayer )  
		{
			if( g_uMessageRecursionCount > uASURA_MAX_ALLOWED_MESSAGE_RECURSION )
			{
				Asura_Assertf( false, ("Probable infinite loop in messages sent from GUID %u to GUID %u", pxMessage->From, pxMessage->To) );
				return false;
			}
			g_uMessageRecursionCount++;

			pxPlayer->HandleMessage(pxMessage);
			--g_uMessageRecursionCount;
		}
	}

	return true;
}

template <class T> void Asura_Entity_System::UpdateEntities(Asura_Classified_List<T>& xEntityList)
{
	for(Asura_Classified_List_It<T> xIterator(&xEntityList); !xIterator.Done(); xIterator.Next())
	{
		T*	pxEntity	= xIterator.GetCurrent();

		Asura_Assert(pxEntity, "Iterator problem");
		if (pxEntity)
		{
			// If an entity sends a message as part of its update which causes another
			// entity to be flagged as destroyed or become inactive, the entity will
			// still remain in the updateable list until the list iteration is over.
			// If it happens to be later in the list it would still have it's update
			// called. To prevent this we check each entity for those conditions.
			if ( (!pxEntity->IsDestroyed()) && (pxEntity->IsEntityActive()) )
			{
				pxEntity->Update();
			}
		}
	}
}

#if defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)

template <class T> void Asura_Entity_System_ProfileData::CountEntities(Asura_Classified_List<T>& xEntityList)
{
	Asura_Classified_List_Const_It<T>	xIterator(&xEntityList);

	const T*							pxEntity			= (xIterator.Done()) ? NULL : xIterator.GetCurrent();
	u_short								usClassification	= (pxEntity == NULL) ? 0 : pxEntity->GetClassification();

	// Initialise the counts...
	m_uNumEntities				= 0;
	m_uNumEntitiesUpdating		= 0;
	m_uNumClassificationsFound	= 0;

	// Reset the data for the first classification...
	m_auNumEntitiesPerClassification[m_uNumClassificationsFound]			= 0;
	m_auNumEntitiesUpdatingPerClassification[m_uNumClassificationsFound]	= 0;
	m_afTimeUpdatingEntitiesPerClassification[m_uNumClassificationsFound]	= 0.0f;

	// Update each entity in-turn...
	while(pxEntity != NULL)
	{
		u_short	usNextClassification;

		// Do we need to update this entity?
		if(m_uNumClassificationsFound < ASURA_MAX_ENTITY_PROFILE_CLASSIFICATIONS)
		{
			m_auNumEntitiesPerClassification[m_uNumClassificationsFound]++;
		}

		m_uNumEntities++;

		// Step onto the next entity...
		xIterator.Next();

		pxEntity				= (xIterator.Done()) ? NULL : xIterator.GetCurrent();
		usNextClassification	= (pxEntity == NULL) ? 0 : pxEntity->GetClassification();

		// Is this entity of a new classification?
		if(usClassification != usNextClassification)
		{
			if(m_uNumClassificationsFound < ASURA_MAX_ENTITY_PROFILE_CLASSIFICATIONS)
			{
				m_ausEntityClassifications[m_uNumClassificationsFound]	= usClassification;
			}

			m_uNumClassificationsFound++;

			if(m_uNumClassificationsFound < ASURA_MAX_ENTITY_PROFILE_CLASSIFICATIONS)
			{
				m_auNumEntitiesPerClassification[m_uNumClassificationsFound]			= 0;
				m_auNumEntitiesUpdatingPerClassification[m_uNumClassificationsFound]	= 0;
				m_afTimeUpdatingEntitiesPerClassification[m_uNumClassificationsFound]	= 0.0f;
			}

			usClassification	= usNextClassification;
		}
	}
}

template <class T> void Asura_Entity_System_ProfileData::UpdateEntities(Asura_Classified_List<T>& xEntityList)
{
	Asura_Classified_List_It<T>	xIterator(&xEntityList);

	Asura_TickCount				xCurrentTicks			= Asura_Timers::GetCurTicks();
	const Asura_TickCount		xStartTicks				= xCurrentTicks;
	T*							pxEntity				= (xIterator.Done()) ? NULL : xIterator.GetCurrent();
	u_short						usClassification		= (pxEntity == NULL) ? 0 : pxEntity->GetClassification();
	u_int						uClassificationIndex	= 0;

	// Update each entity in-turn...
	while(pxEntity != NULL)
	{
		u_short	usNextClassification;

		// Do we need to update our classification index?
		while((usClassification != m_ausEntityClassifications[uClassificationIndex]) && (uClassificationIndex < m_uNumClassificationsFound))
		{
			uClassificationIndex++;
		}

		// Do we need to update this entity?
		if ( (!pxEntity->IsDestroyed()) && (pxEntity->IsEntityActive()) )
		{
			if(uClassificationIndex < ASURA_MAX_ENTITY_PROFILE_CLASSIFICATIONS)
			{
				m_auNumEntitiesUpdatingPerClassification[uClassificationIndex]++;
			}

			// Update the entity itself AND keep our count...
			pxEntity->Update();

			m_uNumEntitiesUpdating++;
		}

		// Step onto the next entity...
		xIterator.Next();

		pxEntity				= (xIterator.Done()) ? NULL : xIterator.GetCurrent();
		usNextClassification	= (pxEntity == NULL) ? 0 : pxEntity->GetClassification();

		// Is this entity of a new classification?
		if(usClassification != usNextClassification)
		{
			if(uClassificationIndex < ASURA_MAX_ENTITY_PROFILE_CLASSIFICATIONS)
			{
				const Asura_TickCount	xTicksTaken		= Asura_Timers::GetElapsedTicks(xCurrentTicks);
				const float				fSecondsTaken	= Asura_Timers::TicksToSeconds(xTicksTaken);

				m_ausEntityClassifications[uClassificationIndex]		= usClassification;
				m_afTimeUpdatingEntitiesPerClassification[uClassificationIndex]	= fSecondsTaken * 1000.f;

				xCurrentTicks	= Asura_Timers::GetCurTicks();
			}

			usClassification	= usNextClassification;
		}
	}

	// We've finished updating...
	{
		const Asura_TickCount	xTotalTicksTaken	= Asura_Timers::GetElapsedTicks(xStartTicks);
		const float				fTotalSecondsTaken	= Asura_Timers::TicksToSeconds(xTotalTicksTaken);

		m_fEntitiesUpdateTime = fTotalSecondsTaken * 1000.0f;
	}
}

template <class T> void Asura_Entity_System_ProfileData::UpdateDVS(const Asura_UnicodeChar* wszParentDVSEntry, T* pxValues)
{
	for (u_int u = 0; u < m_uNumClassificationsFound; ++u)
	{
		if(pxValues[u] != 0.0f)
		{
			Asura_UnicodeChar	wszVarName[256];

			const u_short		usClassification		= m_ausEntityClassifications[u];
			const Asura_Char*	szClassificationName	= Asura_Network::GetEntityClassificationDesc(usClassification);
			T*					pxValue					= &pxValues[u];

			// add var if it doesn't exist and set its value if it does
			Asura_PrintfToBfr(wszVarName, L"%s.%S", 256, true, wszParentDVSEntry, szClassificationName);
			Asura_CommandConsole_VarRepository::Var*	pxVar	= Asura_CommandConsole_VarRepository::Find(wszVarName);
			if(pxVar == NULL)
			{
				// remove at reset
				Asura_CommandConsole::AddVar(wszVarName, pxValue);
				Asura_CommandConsole::MakeVarReadOnly(wszVarName);
			}
			else
			{
				// update the data
				pxVar->SetData(pxValue);
			}
		}
	}
}

#endif	//defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)


void Asura_Entity_System::BeginUpdate (void)
{
	//Deal with any static messages that have been delayed from previous frames. RAR 01/02/2006
	Asura_StaticMessage_System::Update();

#if defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	if(Asura_DebugVariableSystem::IsOpen())
	{
		Asura_Classified_List<Asura_Entity_Client>*	pxClientList	= Asura_Entity_Client::GetList();
		Asura_Classified_List<Asura_Entity_Server>*	pxServerList	= Asura_Entity_Server::GetList();

		s_xClientEntityProfileData.CountEntities(*pxClientList);
		s_xServerEntityProfileData.CountEntities(*pxServerList);
	}
#endif

	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__ENTITY_UPDATE, "Entity Updates");
}

void Asura_Entity_System::Update(Asura_Classified_List<Asura_Entity_Client>& xClientList,
								 Asura_Classified_List<Asura_Entity_Server>& xServerList)
{
	// If we're the server, call Update() for all server entities
	if (Asura_Network::IsServer())
	{
#if defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
		if(Asura_DebugVariableSystem::IsOpen())
		{
			s_xServerEntityProfileData.UpdateEntities(xServerList);
		}
		else
#endif	//defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
		{
			UpdateEntities(xServerList);
		}
	}

#ifndef ASURA_ENTITY_DO_CLIENT_UPDATES_AT_END_OF_FRAME 	
	/// DWE - 07/11/2002 14:45:11 - Call the update function for clients that are on the update list.
#if defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	if(Asura_DebugVariableSystem::IsOpen())
	{
		s_xClientEntityProfileData.UpdateEntities(xClientList);
	}
	else
#endif	//defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	{
		UpdateEntities(xClientList);
	}
#endif

#if defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	// populate DVS with time spent updating each entity type.
	if(Asura_DebugVariableSystem::IsOpen())
	{
		const Asura_UnicodeChar*	wszDVSCommand	= Asura_DebugVariableSystem::GetSelectedPath();

		if (wcscmp(wszDVSCommand, wszASURA_PROFILE_CLIENT_ENTITIES_DVS_ENTRY) == 0)
		{
			s_xClientEntityProfileData.UpdateDVS(wszDVSCommand, s_xClientEntityProfileData.m_afTimeUpdatingEntitiesPerClassification);
		}
		else if (wcscmp(wszDVSCommand, wszASURA_NUM_CLIENTS_DVS_ENTRY) == 0)
		{
			s_xClientEntityProfileData.UpdateDVS(wszDVSCommand, s_xClientEntityProfileData.m_auNumEntitiesPerClassification);
		}
		else if (wcscmp(wszDVSCommand, wszASURA_NUM_UPDATING_CLIENT_ENTITY_DVS_ENTRY) == 0)
		{
			s_xClientEntityProfileData.UpdateDVS(wszDVSCommand, s_xClientEntityProfileData.m_auNumEntitiesUpdatingPerClassification);
		}
		else if (wcscmp(wszDVSCommand, wszASURA_PROFILE_SERVER_ENTITIES_DVS_ENTRY) == 0)
		{
			s_xServerEntityProfileData.UpdateDVS(wszDVSCommand, s_xServerEntityProfileData.m_afTimeUpdatingEntitiesPerClassification);
		}
		else if (wcscmp(wszDVSCommand, wszASURA_NUM_SERVERS_DVS_ENTRY) == 0)
		{
			s_xServerEntityProfileData.UpdateDVS(wszDVSCommand, s_xServerEntityProfileData.m_auNumEntitiesPerClassification);
		}
		else if (wcscmp(wszDVSCommand, wszASURA_NUM_UPDATING_SERVER_ENTITY_DVS_ENTRY) == 0)
		{
			s_xServerEntityProfileData.UpdateDVS(wszDVSCommand, s_xServerEntityProfileData.m_auNumEntitiesUpdatingPerClassification);
		}

		// if we're looking at the num entities in the DVS, generate DVS entries for all the entity classifications.
		if(Asura_Network::IsClient())
		{
			Asura_Classified_List<Asura_Entity_Renderable>*	pxRenderableList	= Asura_Entity_Renderable::GetRenderableList();

			CreateTemporaryDVSEntityList(wszASURA_NUM_RENDERABLE_ENTITY_DVS_ENTRY, &s_uNumRenderableEntities, *pxRenderableList, s_auNumRenderableEntitiesPerClassification);
		}
	}
#endif	//defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
}

void Asura_Entity_System::EndUpdate(void)
{
	/// Kevin Lea - 03/12/2001 15:03:18 - tidy away any dead entities
	PruneEntityLists();

	//Asura_EntityList_Triggers.Update();

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	if (Asura_Network::IsExternalNetworkActive())
	{
		Asura_RecentlyDestroyedEntity_System::UpdateLists();
	}
#endif

	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__ENTITY_UPDATE);

#ifdef ASURA_SUPPORT_ENTITYWATCH
	Asura_EntityWatch_System::Update();
#endif

#ifdef ASURA_ASSERT
    // Catch entities that haven't set their classification.
    {
        const Asura_Classified_Node<Asura_Entity_Client>* pxUnclassifiedClient = Asura_Entity_Client::GetList()->GetUnclassifiedList();
        for( ; pxUnclassifiedClient; pxUnclassifiedClient = pxUnclassifiedClient->m_pxNext )
        {
            Asura_Assert( pxUnclassifiedClient->m_pxObject->GetClassification() != AsuraEntityClass_NotSet, "Classification not set!" );
        }

        const Asura_Classified_Node<Asura_Entity_Server>* pxUnclassifiedServer = Asura_Entity_Server::GetList()->GetUnclassifiedList();
        for( ; pxUnclassifiedServer; pxUnclassifiedServer = pxUnclassifiedServer->m_pxNext )
        {
            Asura_Assert( pxUnclassifiedServer->m_pxObject->GetClassification() != AsuraEntityClass_NotSet, "Classification not set!" );
        }
    }
#endif
}

DEBUGVAR bool dbg_bUpdateAllServerEntities = false;
void Asura_Entity_System::Update(void)
{
	Asura_Classified_List<Asura_Entity_Client>&	xClientList		= Asura_Entity_Client::s_xUpdateableList;
	Asura_Classified_List<Asura_Entity_Server>*	pxServerList	= (dbg_bUpdateAllServerEntities)
																? Asura_Entity_Server::GetList()
																: &Asura_Entity_Server::s_xUpdateableList;

	// Handle the update in sections...
	BeginUpdate();
	Update(xClientList, *pxServerList);
	EndUpdate();
}

//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     PruneEntityLists().
// PARAMETERS   None.
// RETURNS      None.
// DESCRIPTION  deletes any entity which is ReadyToBeDestroyed
// AUTHOR       Mike Healey - 30.10.2002.
//
//////////////////////////////////////////////////////////////////////////////
void Asura_Entity_System::PruneEntityLists(void)
{
	FlushDestroyList();
}
// End function PruneEntityLists().


void Asura_Entity_System::GetClientUpdates( bool bSendingThisFrame )
{
	if( bSendingThisFrame )
	{
#if defined(ASURA_USE_STREAMING)
		Asura_Streaming_System::FinishPeriodicClientUpdates();
#else
		s_xPeriodicClientUpdateList.FinishClientUpdates();
#endif //defined(ASURA_USE_STREAMING)
	}
	else
	{
#if defined(ASURA_USE_STREAMING)
		Asura_Streaming_System::DoSomePeriodicClientUpdates();
#else
		s_xPeriodicClientUpdateList.DoSomeClientUpdates();
#endif //defined(ASURA_USE_STREAMING)
	}

#if defined(ASURA_USE_STREAMING)
	Asura_Streaming_System::DoConstantClientUpdates();
#else
    for( Asura_Hashed_List_It<Asura_Entity_Server> xIt(&s_xConstantClientUpdateList); !xIt.Done(); xIt.Next() )
    {
		if (!xIt()->IsDestroyed())
		{
			xIt()->MakeClientUpdateMessage();
		}
    }
#endif //defined(ASURA_USE_STREAMING)

	if (s_bUpdateInactiveClientEntitiesAtEndOfFrame)
	{
		s_bUpdateInactiveClientEntitiesAtEndOfFrame = false;
		for ( Asura_Classified_List_It< Asura_Entity_Server > xServerIter( Asura_Entity_Server::GetList() ); 
			  !xServerIter.Done(); 
			  xServerIter.Next() )
		{
			Asura_Entity_Server* pxEntity = xServerIter.GetCurrent();
			if (pxEntity->HasClientEntities()
				&& !pxEntity->IsEntityActive()
				&& pxEntity->m_bShouldAddToClientUpdateListWhenActivated
				&& ShouldCreateInactiveClientEntity(pxEntity))
			{
				// only do the creation if the client entity wasn't created already.
				Asura_Entity_Client* pxClientEntity = Asura_Entity_Client::FindEntity(pxEntity->GetGuid());
				if (!pxClientEntity)
				{
					pxEntity->MakeClientUpdateMessage();
					pxClientEntity = Asura_Entity_Client::FindEntity(pxEntity->GetGuid());
					Asura_Assert(pxClientEntity, "client update message didn't result in entity being created");
				}
				// deactivate the client entity whether or not we just created it.
				if (pxClientEntity)
				{
					pxClientEntity->SetEntityActive(false);
				}
			}
		}
	}


#ifdef ASURA_ENTITY_DO_CLIENT_UPDATES_AT_END_OF_FRAME
	UpdateEntities(Asura_Entity_Client::s_xUpdateableList);
#endif
	
}

// NY - 04 January 2008 - check if this entity should have a client created due to s_bUpdateInactiveClientEntitiesAtEndOfFrame
// this is to allow entities to opt out of the enforce client creation, and retain the usual behaviour of creation on activation
// currently no project version - if a project needs to opt entities out, that may need to change
bool Asura_Entity_System::ShouldCreateInactiveClientEntity(const Asura_Entity* pxEntity)
{
	if (pxEntity->GetClassification() == AsuraEntityClass_Objective)
	{
		// NY - 04 January 2008 - objectives don't want to be created until active, since client entities are always created active,
		// and the objective will fire off it's callback as if it's been activated.
		// s_bUpdateInactiveClientEntitiesAtEndOfFrame isn't needed for objectives anyway, as the server never accesses the clients directly,
		// and inactive clients are ommited from it's class-specific list
		return false;
	}
	return true;
}



void Asura_Entity_System::BeginRender(void)
{
#if defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	s_uNumRenderableEntities = 0;
#endif
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__ENTITY_RENDER, "Entity Render");

#ifdef ASURA_USE_ZPREPASS
	if( Asura_Render::IsZPrepass() )
	{
		Asura_SortedRender::Start();
	}
#else
	Asura_SortedRender::Start();
#endif
}

void Asura_Entity_System::Render(Asura_Classified_List<Asura_Entity_Renderable>& xList)
{
	// Traverse the different classes of entity
	Asura_Classified_List_It<Asura_Entity_Renderable> xIt(&xList);
	while (!xIt.Done())
	{
		Asura_Entity_Renderable* pxEntity = xIt.GetCurrent();
		xIt.Next();

		if( !pxEntity->IsDestroyed() )
		{
			pxEntity->Render();
#if defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
			s_uNumRenderableEntities++;
#endif
		}
	}
}

void Asura_Entity_System::EndRender(void)
{
#ifdef ASURA_USE_ZPREPASS
	if( Asura_Render::IsZPrepass() )
	{
		Asura_SortedRender::End();
	}
#else
	Asura_SortedRender::End();
#endif

	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__ENTITY_RENDER);
}

static bool dbg_bRenderAllEntities = false;
void Asura_Entity_System::Render(void)
{
	Asura_Classified_List<Asura_Entity_Renderable>*	pxList	= (dbg_bRenderAllEntities)
															? Asura_Entity_Renderable::GetList()
															: Asura_Entity_Renderable::GetRenderableList();

#ifdef ASURA_EDGE_ANTIALIASING
	Asura_Render::SetCollectAAEdgesFlag( true );
#endif

	// Handle the render in sections...
	BeginRender();
	Render(*pxList);
	EndRender();

#ifdef ASURA_EDGE_ANTIALIASING
	Asura_Render::SetCollectAAEdgesFlag( false );
#endif
}

Asura_DynamicBVH<Asura_Entity_Renderable>& Asura_Entity_System::GetLineOfSightBVH()
{
	RefreshLineOfSightBVH();
	return s_xLineOfSightBVH;
}


class RenderableEntityBVH_Iterator : public Asura_Classified_List_It<Asura_Entity_Renderable>
{
public:

	RenderableEntityBVH_Iterator( Asura_Classified_List<Asura_Entity_Renderable>* pxList )
        : Asura_Classified_List_It<Asura_Entity_Renderable>( pxList )
    {
    }

    inline void Next( void )
    {
        do
        {
            Asura_Classified_List_It<Asura_Entity_Renderable>::Next();
        }
        while( !Done() && GetCurrent()->IsDestroyed() );
    }
};


class RenderableEntityBVH_BBRetrievalWrapper
{
public:
    static inline void GetAABB( Asura_Entity_Renderable& xEntity, Asura_Bounding_Box& xAABB )
    {
        return xEntity.GetLineOfSightAABB( xAABB );
    }
};


// Create or refit the line of sight dynamic BVH if necessary.
void Asura_Entity_System::RefreshLineOfSightBVH()
{
	// KJFL - 6 November 2007 - TODO
	// The heuristics calling Create & Refit could be moved to the BVH class itself?
	const u_int uMAX_NUMBER_OF_REFITS = 100;

	// If the LOS list has been updated or if we've done more than 100
    // refits then we need to throw away the BVH and recreate it.
	if( (s_uLineOfSightLastUpdatedStamp != Asura_Entity_Renderable::GetLineOfSightListUpdateStamp())
        ||
        ((s_uLineOfSightLastRefitStamp != Asura_Timers::GetFrameNumber()) && (s_uNumberOfRefits >= uMAX_NUMBER_OF_REFITS)) )
    {
        RenderableEntityBVH_Iterator xIt( Asura_Entity_Renderable::GetLineOfSightList() );
		s_xLineOfSightBVH.Create<RenderableEntityBVH_BBRetrievalWrapper>( xIt );

		s_uLineOfSightLastUpdatedStamp = Asura_Entity_Renderable::GetLineOfSightListUpdateStamp();
		s_uLineOfSightLastRefitStamp = Asura_Timers::GetFrameNumber();
		s_uNumberOfRefits = 0;
	}
	else if( s_uLineOfSightLastRefitStamp != Asura_Timers::GetFrameNumber() )
	{
		s_xLineOfSightBVH.Refit<RenderableEntityBVH_BBRetrievalWrapper>();
		++s_uNumberOfRefits;
		s_uLineOfSightLastRefitStamp = Asura_Timers::GetFrameNumber();
	}
}


void Asura_Entity_System::FindEntityInLOS(const Asura_LOS_InputData* pxInputData, Asura_LOS_OutputData* pxOutputData )
{
	// MGH 5/2/03: NOTE: this function uses the max range defined in the Asura_LOS_OutputData (ie m_fDistanceToIntersection)
	// The m_fMaximumRange in Asura_LOS_InputData is ignored.
	if(!pxInputData || !pxOutputData)
	{
		Asura_Assert(0, "LOS, input or output data is null");
		return;
	}

    const Asura_Guid uIgnore = pxInputData->m_uEntityToIgnore;

    if( Asura_Network::IsClient() )
    {
		// KJFL - 14 November 2007 - TODO
		// This is a workaround for the nasty situation where when the bounding box of an anim that
		// is using IK is updated, an environment LOS call happens. So if the BVH queries such an anim's 
		// bounding box, this neatly trashes the static Asura_LOS_OutputData the Asura_LOS code keeps.
		// There are various ways round this (and making the Asura_LOS not have one static output data
		// area would be good) but fundamentally I don't really want the entity LOS calls to force updates
		// of all active IK solutions. The BVH only needs a rough estimate AABB to do it's job, so adding
		// a method to Asura_Animation to get a coarse, quick AABB would be preferable.
		Asura_LOS_OutputData xNewOutput = *pxOutputData;

		RefreshLineOfSightBVH();

		s_xLineOfSightBVH.TestRay( *pxInputData, xNewOutput ); 
		
		*pxOutputData = xNewOutput;
    }
    else
    {
        for( Asura_Hashed_List_It<Asura_ServerEntity_PhysicalObject> xIt(Asura_ServerEntity_PhysicalObject::GetList()); !xIt.Done(); xIt.Next() )
        {
            const Asura_ServerEntity_PhysicalObject* pxPO = xIt.GetCurrent();

			if( pxPO->IsDestroyed() ) { continue; }

			// KJFL - 12 November 2007 - now skip deactivated entities explicitly
			if ( !pxPO->IsEntityActive() ) continue;

            if( ASURA_GUID_UNREGISTERED != uIgnore && (pxPO->GetGuid() == uIgnore || pxPO->GetParentGuid() == uIgnore) ) { continue; }

			pxPO->LOSTest( pxInputData, pxOutputData );
        }
    }
}

bool Asura_Entity_System::ClientCreateEntity(const Asura_Message* pxMessage)
{
#ifdef ASURA_ASSERT
	if (Asura_Network::IsServer() && !Asura_Network::IsExternalNetworkActive())
	{
		// can only check for deactivate message - activate message sent even if client
		// entity has not been created yet.
        if (pxMessage->GetMsgID() == ASURA_MSG_DEACTIVATE)
		{
			// (MH - 30.10.2007) most likely this is result of server entity which has no client entities.
			// Such entities should have HasClientEntities() overridden.
			// AVT currently sends message on Deactivation which might happen in first frame.
			// same for lookat trigger. These should be refactored but ignore the error for now.
			// CutsceneController is created by activate message, so deactivate message should be ignored.
			if (AsuraEntityClass_AdvancedVolumeTrigger != pxMessage->GetEntityClassification() && 
				AsuraEntityClass_LookAtTrigger != pxMessage->GetEntityClassification() && 
				AsuraEntityClass_CutsceneController != pxMessage->GetEntityClassification())
			{
				Asura_Assertf(false, ("client entity of type %s received deactivate message when it doesn't exist", 
					Asura_Network::GetEntityClassificationDesc(pxMessage->GetEntityClassification()) ));
			}
		}
	}
#endif


	if (!Asura_Entity_System::Project_ClientCreateEntity(pxMessage))
    {
		// First, see if this is a message type that can trigger a creation request.
		// This is done first because the different messages may have completely different handling if not used here, but we don't want to have to add calls to request create to each of them.
		switch (pxMessage->GetMsgID())
		{
		case ASURA_MSG_CLIENTUPDATE:
		case ASURA_MSG_CONTAINER_CLIENTUPDATE_EXTENDED1:
		case ASURA_MSG_CONTAINER_CLIENTUPDATE_EXTENDED:
			{
				const u_short usClassification = pxMessage->GetEntityClassification();
				switch (usClassification)
				{
				default:
					break;

				// The following entities request creation.
				case AsuraEntityClass_LookAtTrigger: // I'm not sure this actually has an update message, but handling one for creation used to be done, so just in case it can happen it's included here.
#ifdef ASURA_USE_CONTAINER_SYSTEM
				case AsuraEntityClass_Container:
#endif //ASURA_USE_CONTAINER_SYSTEM
#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER
				case AsuraEntityClass_DebugWaypointTrigger:
#endif //ASURA_USE_DEBUG_MESSAGE_TRIGGER
					SendRequestCreateMessage(pxMessage->GetFromGuid(), usClassification);
					return true;
				}
			}

			break;

		default:
			break;
		}

		// Second, any other message handling (creation from update, asserts about unhandled, etc.)
        switch(pxMessage->GetMsgID())
	    {
			case ASURA_MSG_ACTIVATE:
			case ASURA_MSG_DEACTIVATE:
			{
				if (pxMessage->GetEntityClassification() == AsuraEntityClass_PFX_Effect ||
					pxMessage->GetEntityClassification() == AsuraEntityClass_ParticleGenerator)
				{
					// (AW - 25.10.2004) - Safe to ignore particle status messages. The entity 
					// will be loaded from chunk buffer on the client.
					break;
				}
				// Fallthrough if unhandled.
			}

			default:
		    {
#if ASURA_NETWORK_LOGGING
// SM - 22 September 2006 17:03:59 - Added extra debugging to show detailed, named information about message errors
				char acGuidFrom[32];
				char acGuidTo[32];
				const char* szGuidTo = NULL;
				const char* szGuidFrom = NULL;
				const char* szMsgID = NULL;
				Asura_Network::GetMsgIDDesc(pxMessage->GetMsgID(), &szMsgID);
				Asura_Network::GetGuidDesc(pxMessage->GetToGuid(), &szGuidTo);
				Asura_Network::GetGuidDesc(pxMessage->GetFromGuid(), &szGuidFrom);
				if (!szGuidFrom)
				{
					Asura_PrintfToBfr(acGuidFrom, "%u", 32, true, pxMessage->GetFromGuid());
					szGuidFrom = acGuidFrom;
				}
				if (!szGuidTo)
				{
					Asura_PrintfToBfr(acGuidTo, "%u", 32, true, pxMessage->GetToGuid());
					szGuidTo = acGuidTo;
				}
				Asura_Assertf(false,("Cannot create client-entity with this message (To: %s From: %s ID:%s)", szGuidTo, szGuidFrom, szMsgID));
#else
				Asura_Assertf(0,("Cannot create client-entity with this message (To: %d From: %d ID:%d)",pxMessage->To,pxMessage->From,pxMessage->GetMsgID()));
#endif
                return false;
		    }

			case ASURA_MSG_DESTRUCT:
			{
				UnhandledDestroyMessage(pxMessage->From);
			}
			break;

			case ASURA_MSG_HIDE:
			case ASURA_MSG_UNHIDE:
			{
				// Should be safe to ignore these, since the server will send this info in the
				// update message as well
			}
			break;

            case ASURA_MSG_CUTSCENECONTROLLER_PROFILE_CUTSCENE: // Intentional fallthrough.
		    case ASURA_MSG_CUTSCENECONTROLLER_ACTIVATE:
		    {
			    Asura_ClientEntity_CutsceneController* pxCutscene = new Asura_ClientEntity_CutsceneController();
                if( pxCutscene )
                {
    			    pxCutscene->HandleMessage(pxMessage);
                }
		    }
            break;

			case ASURA_MSG_CUTSCENECONTROLLER_UPDATEEVENTSTATE :
				//can ignore these messages safely if client controller doesn't exist yet. RAR 24/02/2005
				break;

#ifdef ASURA_SUPPORT_SERVERCUTSCENES	// the Gunfighter-style cutscenes with entities
            case ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_STANDARD:
            {
                const Asura_Message_ServerCutsceneActor_ClientUpdateStandard* pxData = 
                    static_cast<const Asura_Message_ServerCutsceneActor_ClientUpdateStandard*>( pxMessage->GetData() );
                Asura_ClientEntity_ServerCutsceneActorHierarchy* pxEntity = 
					new Asura_ClientEntity_ServerCutsceneActorHierarchy(pxData->uCutsceneHash, pxData->sSkinID, pxData->sAnimationID, pxData->iAnimationFlags, pxData->fAnimationDelay, pxData->uFlags, pxData->uTextureHash);
					
                if (pxEntity)
                {
                    pxEntity->ProcessNewPositionAndOrientationData(pxData->xPosition, pxData->xOrientation, pxMessage->GetSentTime());
                }
            }
            break;

            case ASURA_MSG_SERVERCUTSCENEACTORSHAPE_CLIENTUPDATE_STANDARD:
            {
                const Asura_Message_ServerCutsceneActorShape_ClientUpdateStandard* pxData = 
                    static_cast<const Asura_Message_ServerCutsceneActorShape_ClientUpdateStandard*>( pxMessage->GetData() );
                Asura_ClientEntity_ServerCutsceneActorShape* pxEntity = 
                    new Asura_ClientEntity_ServerCutsceneActorShape(pxData->m_uCutsceneHash, pxData->m_uShapeID);
                if (pxEntity)
                {
                    pxEntity->ProcessNewPositionAndOrientationData(pxData->m_xPosition, pxData->m_xOrientation, pxMessage->GetSentTime());
                }
            }
            break;
#endif // #ifdef ASURA_SUPPORT_SERVERCUTSCENES	// the Gunfighter-style cutscenes with entities

			case ASURA_MSG_GLASS_CLIENTUPDATE:
				{
						Asura_ClientEntity_Glass::Create(pxMessage);
						break;
				}
            
            case ASURA_MSG_CLIENTUPDATE:
            {
                switch (pxMessage->GetEntityClassification())
                {
		            default:
		            {
			            Asura_Assert(0, "Message not handled");
                        return false;
		            }

#ifdef ASURA_SUPPORT_SERVERCUTSCENES	// the Gunfighter-style cutscenes with entities
                    case AsuraEntityClass_ServerCutsceneActorCamera:
                    {
                        
                        Asura_ClientEntity_ServerCutsceneActorCamera* pxEntity = 
                            new Asura_ClientEntity_ServerCutsceneActorCamera;
                        if (pxEntity)
                        {
                            pxEntity->HandleMessage(pxMessage);
                        }
						break;
                    }
#endif // #ifdef ASURA_SUPPORT_SERVERCUTSCENES	// the Gunfighter-style cutscenes with entities
					case AsuraEntityClass_Objective:
					{
						if (!Asura_ClientEntity_Objective::CreateFromMessage(pxMessage))
						{
							Asura_Assert(0, "Message not handled");
							return false;
						}
						break;
					}
                }
                break;
            }

			case ASURA_MSG_CLIENTCREATE:
#ifdef ASURA_ALLOW_INACTIVE_CLIENT_ENTITIES
			case ASURA_MSG_CLIENTCREATEINACTIVE:
#endif //#ifdef ASURA_ALLOW_INACTIVE_CLIENT_ENTITIES
			{
				// Be sure to handle these classifications in Project_ClientCreateEntity if you want to create a project subclass.
				switch (pxMessage->GetEntityClassification())
				{
	            default:
					{
						Asura_Assertf(0, ("Create message not handled for entity classification %s", 
							Asura_Network::GetEntityClassificationDesc(pxMessage->GetEntityClassification())));
						return false;
					}

#ifndef ASURA_TARGET_SM3
				case AsuraEntityClass_AdvancedLight:
					{
						Asura_ClientEntity_AdvancedLight::Create(pxMessage);
						return true;
					}
#endif // #ifndef ASURA_TARGET_SM3

#ifdef ASURA_USE_CONTAINER_SYSTEM
				case AsuraEntityClass_Container:
					{
						Asura_ClientEntity_Container::Create(pxMessage);
						return true;
					}
#endif //ASURA_USE_CONTAINER_SYSTEM

#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER
				case AsuraEntityClass_DebugWaypointTrigger:
					{
						Asura_ClientEntity_DebugWaypointTrigger::Create(pxMessage);
						return true; 
					}
#endif // ASURA_USE_DEBUG_MESSAGE_TRIGGER

				case AsuraEntityClass_LookAtTrigger:
					{
						Asura_ClientEntity_LookAtTrigger::Create(pxMessage);
						return true;
					}

				case AsuraEntityClass_EffectVolume:
					{
						if (!Asura_ClientEntity_EffectVolume::Create(pxMessage))
						{
							Asura_Assert(0, "Message in Asura_ClientEntity_EffectVolume not handled");
							return false;
						}

						break;
					}

				}
			}
			break;
	    }
    }
	else
	{
		Asura_Assert(pxMessage->GetMsgID() != ASURA_MSG_DESTRUCT, "Asura destruct message shouldn't be handled by project code");
	}
	return true;
}


#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Asura_Entity_System::RegisterUpdateMessage().
// PARAMETERS   usMsgID                - 
//              usEntityClassification - 
// RETURNS      
// DESCRIPTION  For the system which reduces the amount of update data being 
//				sent over the network: this provides a means for entity types 
//				to  define the fields of their update message. This is analogous 
//				to ClientCreateEntity.
// AUTHOR       Mike Healey - 01.05.2003.
//
//////////////////////////////////////////////////////////////////////////////
bool Asura_Entity_System::RegisterUpdateMessage(u_short usMsgID, u_short usEntityClassification)
{
	if (!Project_RegisterUpdateMessage(usMsgID, usEntityClassification))
    {
#ifdef ASURA_USE_CONTAINER_SYSTEM
		if (usEntityClassification == AsuraEntityClass_Container)
		{
			switch (usMsgID)
			{
			case ASURA_MSG_CLIENTUPDATE:
				Asura_Message_Container_ClientUpdate::RegisterFields(usMsgID, usEntityClassification);
				return true;

			case ASURA_MSG_CONTAINER_CLIENTUPDATE_EXTENDED1:
				Asura_Message_Container_ClientUpdate_Extended1::RegisterFields(usMsgID, usEntityClassification);
				return true;

			case ASURA_MSG_CONTAINER_CLIENTUPDATE_EXTENDED:
				Asura_Message_Container_ClientUpdate_Extended::RegisterFields(usMsgID, usEntityClassification);
				return true;

			default:
				break;
			}
		}
#endif //ASURA_USE_CONTAINER_SYSTEM


        switch (usMsgID)
	    {
			case ASURA_MSG_CLIENTUPDATE:
			{
				// nested switch, required as this msg ID is used by lots of entity types
				switch (usEntityClassification)
				{
					case AsuraEntityClass_Lift:
					{
						Asura_Message_PhysicalObject_ClientUpdate::RegisterFields(usMsgID, usEntityClassification);
						break;
					}

					case AsuraEntityClass_PhysicsObject:
					{
						Asura_MsgData_PhysicsObject_ClientUpdate::RegisterFields(usMsgID, usEntityClassification);
						break;
					}

#ifndef ASURA_TARGET_SM3
					case AsuraEntityClass_AdvancedLight:
					{	
						Asura_Message_AdvancedLight_ClientUpdate::RegisterFields(usMsgID, usEntityClassification);
						break;
					}
#endif // #ifndef ASURA_TARGET_SM3

					default:
					{
						// not handled
						return false;
					}
				}

				break;
			}

		    default:
		    {
			    // not currently requiring every message to be handled by this 
				// system, the old system gets used if it not.
				//Asura_Assert(0, "Message not handled");
                return false;
		    }
		}
	}
	return true;
}
// End function Asura_Entity_System::RegisterUpdateMessage().


//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Asura_Entity_System::GetUpdateHeartbeat().
// PARAMETERS   usMsgID                - 
//              usEntityClassification - 
// RETURNS      
// DESCRIPTION  Allows for various types to use different rates for full updates. 
//				e.g. a player entity might send a full update every 2 seconds, 
//				a static object every 5 seconds.
// AUTHOR       Mike Healey - 01.05.2003.
//
//////////////////////////////////////////////////////////////////////////////
float Asura_Entity_System::GetUpdateHeartbeat(u_short usMsgID, u_short usEntityClassification)
{
	float fHeartbeat = Project_GetUpdateHeartbeat(usMsgID, usEntityClassification);
	if (fHeartbeat == 0.0f)
	{
		// default value of 5 seconds
		fHeartbeat = 5.0f;
	}
	return fHeartbeat;
}
// End function Asura_Entity_System::GetUpdateHeartbeat().

#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM

void Asura_Entity_System::AddToDestroyList( Asura_Entity* pxEntity )
{
	if( g_uDestroyListLen >= DESTROY_LIST_MAX_LEN )
    {
        g_bExceededMaxListLen = true;
    }
    else
    {
        g_apxDestroyList[g_uDestroyListLen++] = pxEntity;
    }
}

void Asura_Entity_System::RemoveFromDestroyList( Asura_Entity* pxEntity )
{
	if( !g_bFlushingDestroyList && !g_bExceededMaxListLen )
	{
		for( u_int u = 0; u < g_uDestroyListLen; ++u )
		{
			if( pxEntity == g_apxDestroyList[u] )
			{
				g_apxDestroyList[u] = g_apxDestroyList[--g_uDestroyListLen];
				break;
			}
		}
	}
}

void Asura_Entity_System::FlushDestroyList()
{
    g_bFlushingDestroyList = true;

    if ( g_bExceededMaxListLen )
    {
	    g_bExceededMaxListLen  = false;

     
        // Guess we'll have to do it the slow way.
		// Call PreDestructionShutdown on everything first, to allow entities to access other entities during this phase.
		for ( Asura_Classified_List_It< Asura_Entity_Server > xIt( Asura_Entity_Server::GetList() ); !xIt.Done(); xIt.Next() )
		{
			Asura_Entity_Server* pxCur = xIt.GetCurrent();
			if ( pxCur->IsDestroyed() )
			{
				pxCur->PreDestructionShutdown();
				pxCur->SetHasBeenShutdown();
			}
		}
		for ( Asura_Classified_List_It< Asura_Entity_Client > xIt( Asura_Entity_Client::GetList() ); !xIt.Done(); xIt.Next() )
		{
			Asura_Entity_Client* pxCur = xIt.GetCurrent();
			if ( pxCur->IsDestroyed() )
			{
				pxCur->PreDestructionShutdown();
				pxCur->SetHasBeenShutdown();
			}
		}

		// Now go through and delete the necessary entities.
		Asura_Classified_List_It<Asura_Entity_Server> xServerIt( Asura_Entity_Server::GetList() );
		while ( !xServerIt.Done() )
		{
			Asura_Entity_Server* pxCur = xServerIt.GetCurrent();
			xServerIt.Next();
			if ( pxCur->IsDestroyed() )
			{
				if ( !pxCur->HasBeenShutdown() )
				{
					pxCur->PreDestructionShutdown();
				}
				delete pxCur;
			}
		}
		Asura_Classified_List_It<Asura_Entity_Client> xClientIt( Asura_Entity_Client::GetList() );
		while ( !xClientIt.Done() )
		{
			Asura_Entity_Client* pxCur = xClientIt.GetCurrent();
			xClientIt.Next();
			if ( pxCur->IsDestroyed() )
			{
				if ( !pxCur->HasBeenShutdown() )
				{
					pxCur->PreDestructionShutdown();
				}
				delete pxCur;
			}
		}

    }
    else
    {
		// Call PreDestructionShutdown on everything first, to allow entities to access other entities during this phase.
		u_int uDestroyListLen = g_uDestroyListLen;
		while ( uDestroyListLen )
		{
			--uDestroyListLen;
			g_apxDestroyList[ uDestroyListLen ]->PreDestructionShutdown();
			g_apxDestroyList[ uDestroyListLen ]->SetHasBeenShutdown();
		}

		// Now go through and delete the necessary entities.
        // (MH - 16.01.2007) some entity destructors might cause SetAsDestroyed to be called
		// for other entities. This might increase g_uDestroyListLen but it is zero'd below. 
		// By deleting the last entry in the list until the list is empty we get around this.
		while ( g_uDestroyListLen )
		{
			--g_uDestroyListLen;
			if ( !g_apxDestroyList[ g_uDestroyListLen ]->HasBeenShutdown() )
			{
				g_apxDestroyList[ g_uDestroyListLen ]->PreDestructionShutdown();
			}
			delete g_apxDestroyList[ g_uDestroyListLen ];
		}
    }

    g_uDestroyListLen      = 0;
    g_bFlushingDestroyList = false;
}

void Asura_Entity_System::BeginEntityDestruction()
{
	s_bCurrentlyDestroyingAllEntities = true;
}

void Asura_Entity_System::EndEntityDestruction()
{
	s_bCurrentlyDestroyingAllEntities = false;
}

void Asura_Entity_System::DestroyAllEntities()
{
	BeginEntityDestruction();

    FlushDestroyList();

#ifndef ASURA_ALLOW_PLAYER_ENTITY_DELETE
	/// Richard May - 19/04/2004 15:31:44 - Due to legacy reasons, players are not counted 
	/// as real entities, and are not supposed to be deleted here. As in certain projects
	/// (currently Rogue), the client and player are the same class instance, the players
	/// need to be removed from the client list before the reset...
	Asura_Entity_Client* apxSavedClients[4];
	u_int uNumSavedClients = 0;

	Asura_Hashed_List_It<Asura_Entity_Player> xIterator(Asura_Entity_Player::GetList());
	while( !xIterator.Done() )
	{
		Asura_Entity_Player* pxPlayer = xIterator.GetCurrent();
		xIterator.Next();
		
		Asura_Entity_Client* pxClient = Asura_Entity_Client::FindEntity(pxPlayer->GetGuid());
		Asura_Entity* pxClientAsEntity = pxClient;
		if( pxPlayer && pxClientAsEntity 
		&& (pxPlayer == pxClientAsEntity) )
		{
			Asura_Assert(uNumSavedClients < 4, "More players here than expected, we're going to lose this one. Get a coder to change the number allowed.");
			if (uNumSavedClients < 4)
			{
				apxSavedClients[uNumSavedClients++] = pxClient;
				Asura_Entity_Client::GetList()->RemoveFromList(pxClient);
			}
		}
	}
#endif //#ifndef ASURA_ALLOW_PLAYER_ENTITY_DELETE

	Asura_Entity_Client::DestroyAllEntities();
	Asura_Entity_Server::DestroyAllEntities();

#ifdef ASURA_ASSERT
	// KOR - 23/03/2006 - Make sure lists are empty
	// No GetLength on classified lists, so test them by creating an iterator
	if (Asura_Network::IsServer())
	{
		Asura_Classified_List_It<Asura_Entity_Server> xIt(&Asura_Entity_Server::s_xUpdateableList);
		Asura_Assert(xIt.Done(), "Server entities remain in updateable list after DestroyAllEntities");
	}
	if (Asura_Network::IsClient())
	{
		Asura_Classified_List_It<Asura_Entity_Client> xIt(&Asura_Entity_Client::s_xUpdateableList);
		Asura_Assert(xIt.Done(), "Client entities remain in updateable list after DestroyAllEntities");
	}

	{
		Asura_Classified_List_It<Asura_Entity_Renderable> xRenderIt(Asura_Entity_Renderable::GetRenderableList());
#ifdef ASURA_ALLOW_PLAYER_ENTITY_DELETE
		Asura_Assert(xRenderIt.Done(), "Entities remain in renderable list after DestroyAllEntities");
#else //#ifdef ASURA_ALLOW_PLAYER_ENTITY_DELETE
		while(!xRenderIt.Done())
		{
			Asura_Entity_Renderable* pxEntity = xRenderIt.GetCurrent();
			xRenderIt.Next();

			bool bIsPlayer = false;
			for (u_int u=0; u<uNumSavedClients; u++)
			{
				if(pxEntity == apxSavedClients[u])
				{
					// It's one of our saved players
					bIsPlayer = true;
					break;
				}
			}
			Asura_Assert(bIsPlayer, "Non-player entity remains in renderable list after DestroyAllEntities");
		}
#endif //#else //#ifdef ASURA_ALLOW_PLAYER_ENTITY_DELETE
	}
#endif //#ifdef ASURA_ASSERT

#ifdef ASURA_ALLOW_PLAYER_ENTITY_DELETE
	Asura_Network::ResetPlayerStatus();
#else
	/// Richard May - 19/04/2004 15:33:03 - ...and added again after
	for (u_int u=0; u<uNumSavedClients; u++)
	{
		Asura_Entity_Client::GetList()->AddToList(apxSavedClients[u]);
	}
#endif //#ifdef ASURA_ALLOW_PLAYER_ENTITY_DELETE

    EndEntityDestruction();
}

void Asura_Entity_System::UnhandledDestroyMessage(Asura_Guid uGuid)
{
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	// (MH - 02.09.2003) I'm assuming that this network flag will not be 
	// cleared until after all entities have been destroyed.
	if (Asura_Network::IsExternalNetworkActive())
	{
		Asura_RecentlyDestroyedEntity_System::AddDestroyedEntity(uGuid);
	}
#endif
}

#ifdef ASURA_LOGFILES_ENABLED
static const Asura_Char* ENTITYLOGFILE = "Logfiles\\Entities.txt";
#endif

static void LogClassificationCount(int iCount, int iClassification)
{
	// Don't bother if there weren't any
	if(!iCount) { return; }

#if ASURA_NETWORK_LOGGING || defined ASURA_LOGFILES_ENABLED
	const char* szDesc = NULL;
#endif
#if ASURA_NETWORK_LOGGING 
	Asura_Network::GetEntityClassificationDesc(static_cast<u_short>(iClassification), &szDesc);
#endif
	Asura_SimpleLogfileOutput((ENTITYLOGFILE, "%3i instances of classification %i (%s)\n", iCount, iClassification, szDesc ? szDesc : "unknown"));
}

void Asura_Entity_System::LogEntities()
{
	int iCurrentClassification = -1;
	int iClassificationCount = 0;
	int iTotalCount = 0;
	Asura_SimpleLogfileClear( ENTITYLOGFILE );
	Asura_SimpleLogfileOutput((ENTITYLOGFILE, "Updateable server entities\n\n", iClassificationCount, iCurrentClassification));
	if (Asura_Network::IsServer())
	{
		Asura_Classified_List<Asura_Entity_Server>*		pxList	= (dbg_bUpdateAllServerEntities)
																? Asura_Entity_Server::GetList()
																: &Asura_Entity_Server::s_xUpdateableList;
		Asura_Classified_List_It<Asura_Entity_Server>	xIt(pxList);
		while (!xIt.Done())
		{
			Asura_Entity_Server* pxEntity = xIt.GetCurrent();
			xIt.Next();

			Asura_Assert(pxEntity, "Iterator problem");
			if (pxEntity && !pxEntity->IsDestroyed())
			{
				const int iClassification = pxEntity->GetClassification();
				if(iClassification != iCurrentClassification)
				{
					LogClassificationCount(iClassificationCount, iCurrentClassification);
					iCurrentClassification = iClassification;
					iClassificationCount = 0;
				}
				++iClassificationCount;
				++iTotalCount;
			}
		}
		// Make sure last classification is logged
		LogClassificationCount(iClassificationCount, iCurrentClassification);
	}
	Asura_SimpleLogfileOutput((ENTITYLOGFILE, "\nTotal: %i\n", iTotalCount));

	iCurrentClassification = -1;
	iClassificationCount = 0;
	iTotalCount = 0;
	Asura_SimpleLogfileOutput((ENTITYLOGFILE, "\n\nRenderable entities\n\n"));
	Asura_Classified_List_It<Asura_Entity_Renderable> xRenderIt(dbg_bRenderAllEntities ? Asura_Entity_Renderable::GetList() : Asura_Entity_Renderable::GetRenderableList());
	while (!xRenderIt.Done())
	{
		Asura_Entity_Renderable* pxEntity = xRenderIt.GetCurrent();
		xRenderIt.Next();

		if(pxEntity && !pxEntity->IsDestroyed() )
		{
			const int iClassification = pxEntity->GetClassification();
			if(iClassification != iCurrentClassification)
			{
				LogClassificationCount(iClassificationCount, iCurrentClassification);
				iCurrentClassification = iClassification;
				iClassificationCount = 0;
			}
			++iClassificationCount;
			++iTotalCount;
		}
	}
	// Make sure last classification is logged
	LogClassificationCount(iClassificationCount, iCurrentClassification);
	Asura_SimpleLogfileOutput((ENTITYLOGFILE, "\nTotal: %i\n", iTotalCount));
}


#ifdef ASURA_TOOLS
bool Asura_Entity_System::LogUpdatingEntityReport( void )
{	
	// Build the filename for the report by appending _UpdatingEntityReport.txt onto the
	// name of the asura file.
	
	Asura_String xFullEnvironmentFilename = Tools_SaveAsura::GetMainFilename();
	
	Asura_String xEnvFilename;
	xFullEnvironmentFilename.SplitFilename( NULL, &xEnvFilename, NULL );
	
	Asura_String xLogFilename = "logfiles\\";
	xLogFilename += xEnvFilename;
	xLogFilename += "_UpdatingEntityReport.txt";
	
	Asura_String xEntityName;

	Asura_SimpleLogfileClear(( xLogFilename.GetString() ));
    Asura_SimpleLogfileOutput(( xLogFilename.GetString(), "Updating entity report for '%s'\nEntities are reported as: GUID, classification, tools name (if available)\n\n", xFullEnvironmentFilename.GetString() ));

	Asura_SimpleLogfileOutput(( xLogFilename.GetString(), "Updating server entities:\n\n" ));	
	int iCount = 0;
	for ( Asura_Classified_List_Const_It<Asura_Entity_Server> xIt(&Asura_Entity_Server::s_xUpdateableList); !xIt.Done(); xIt.Next() )
	{
		const Asura_Entity_Server* pxEntity = xIt.GetCurrent();
		if( !pxEntity->IsDestroyed() )
		{
			++iCount;

            if( pxEntity->ToolsData )
            {
                xEntityName = "'";
                xEntityName += pxEntity->ToolsData->Name;
                xEntityName += "'";
            }
            else
            {
                xEntityName = "<no name>";
            }

            const char* szWarningString = ( pxEntity->IsEntityActive() ? "" : "   ** NB: entity is inactive but still in updateable list **" );

			Asura_SimpleLogfileOutput(( xLogFilename.GetString(), "%7u, %5u, %s%s\n",
                                        pxEntity->GetGuid(),
                                        pxEntity->GetClassification(),
                                        xEntityName.GetString(),
                                        szWarningString ));
		}
	}
	Asura_SimpleLogfileOutput(( xLogFilename.GetString(), "\nTotal number of updating server entities: %i\n\n\n", iCount ));	

	Asura_SimpleLogfileOutput(( xLogFilename.GetString(), "Renderable client entities:\n\n" ));	
	iCount = 0;
    for ( Asura_Classified_List_Const_It<Asura_Entity_Renderable> xIt(&Asura_Entity_Renderable::s_xRenderableList); !xIt.Done(); xIt.Next() )
    {
        const Asura_Entity_Renderable* pxRenderableEntity = xIt.GetCurrent();
        if( !pxRenderableEntity->IsDestroyed() )
        {
            ++iCount;

            // Client entities won't typically have any tools data of their
            // own but we'll check anyway.
            {
                const Tools_Entity_Data* pxToolsData = NULL;

                if( pxRenderableEntity->ToolsData )
                {
                    pxToolsData = pxRenderableEntity->ToolsData;
                }
                else
                {
                    // Ok, try and query the corresponding server entity.
                    const Asura_Entity_Server* pxServerEntity = Asura_Entity_Server::FindEntity( pxRenderableEntity->GetGuid() );
                    if( pxServerEntity )
                    {
                        pxToolsData = pxServerEntity->ToolsData;
                    }
                }
                
                if( pxToolsData )
                {
                    xEntityName = "'";
                    xEntityName += pxToolsData->Name;
                    xEntityName += "'";
                }
                else
                {
                    xEntityName = "<no name - runtime created?>";
                }
            }

            const char* szWarningString = ( pxRenderableEntity->IsInvisible() ? "   ** NB: entity is invisible but still in renderable list **" : "" );

			Asura_SimpleLogfileOutput(( xLogFilename.GetString(), "%7u, %5u, %s%s\n",
                                        pxRenderableEntity->GetGuid(),
                                        pxRenderableEntity->GetClassification(),
                                        xEntityName.GetString(),
                                        szWarningString ));
        }
    }
	Asura_SimpleLogfileOutput(( xLogFilename.GetString(), "\nTotal number of renderable client entities: %i\n\n\n", iCount ));	

	Asura_SimpleLogfileOutput(( xLogFilename.GetString(), "Updating client entities:\n\n" ));	
	iCount = 0;
    for ( Asura_Classified_List_Const_It<Asura_Entity_Client> xIt(&Asura_Entity_Client::s_xUpdateableList); !xIt.Done(); xIt.Next() )
    {
        const Asura_Entity_Client* pxClientEntity = xIt.GetCurrent();
        if( !pxClientEntity->IsDestroyed() )
        {
            ++iCount;

            // Same logic as for renderable entity output above.
            {
                const Tools_Entity_Data* pxToolsData = NULL;

                if( pxClientEntity->ToolsData )
                {
                    pxToolsData = pxClientEntity->ToolsData;
                }
                else
                {
                    // Ok, try and query the corresponding server entity.
                    const Asura_Entity_Server* pxServerEntity = Asura_Entity_Server::FindEntity( pxClientEntity->GetGuid() );
                    if( pxServerEntity )
                    {
                        pxToolsData = pxServerEntity->ToolsData;
                    }
                }
                
                if( pxToolsData )
                {
                    xEntityName = "'";
                    xEntityName += pxToolsData->Name;
                    xEntityName += "'";
                }
                else
                {
                    xEntityName = "<no name - runtime created?>";
                }
            }

			Asura_SimpleLogfileOutput(( xLogFilename.GetString(), "%7u, %5u, %s\n",
                                        pxClientEntity->GetGuid(),
                                        pxClientEntity->GetClassification(),
                                        xEntityName.GetString() ));
        }
    }
	Asura_SimpleLogfileOutput(( xLogFilename.GetString(), "\nTotal number of updating client entities: %i\n", iCount ));

    Asura_CommandConsole::Printf( "Report written to '%s'.", xLogFilename.GetString() );

	return true;
}
#endif

void Asura_Entity_System::SendRequestCreateMessage(const Asura_Guid uEntityGuid, const u_short usEntityClassification)
{
	if (uEntityGuid < ASURA_GUID_CLIENT_START)
	{
		Asura_Assertf(false, ("Invalid guid for create request (%u)", uEntityGuid));
		return;
	}

	Asura_Message* pxRequestMsg = Asura_Network::CreateMessage(ASURA_MSG_CLIENT_REQUESTCREATEMSG, 0, usEntityClassification);
	if ( pxRequestMsg )
	{
		pxRequestMsg->SetToGuid(uEntityGuid);
		pxRequestMsg->SetFromGuid(Asura_Network::GetClientID());
		Asura_Network::Send(pxRequestMsg);
	}
}

#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)

#if defined(ASURA_TOOLS)
bool Asura_Entity_System::ConsoleCmd_PrintUpdatingEntityReport( bool bHelp )
{
    if( bHelp )
    {
        Asura_CommandConsole::Printf( "Outputs a logfile, named after the current level, detailing updating server entities." );
        return true;
    }
    else
    {
        return Asura_Entity_System::LogUpdatingEntityReport();
    }
}
#endif	//defined(ASURA_TOOLS)

template <class T> u_int Asura_Entity_System::DeactivateDistantEntities(const Asura_Vector_3& xCameraPos, const float fDistSquared, const Asura_Hashed_List<T>* pxList)
{
	u_int	uNumDeactivated	= 0;

	for(Asura_Hashed_List_It<T>	xIterator(pxList, true); !xIterator.Done(); xIterator.Next())
	{
		T*	pxEntity	= xIterator.GetCurrent();

		if((pxEntity != NULL) && pxEntity->IsEntityActive())
		{
			const float	fEntityDistanceSquared	= pxEntity->CalculateSquaredDistanceTo(xCameraPos);

			if(fEntityDistanceSquared > fDistSquared)
			{
				pxEntity->SetEntityActive(false);
				uNumDeactivated++;
			}
		}
	}

	return(uNumDeactivated);
}

bool Asura_Entity_System::ConsoleCmd_DeactivateDistantEntities( bool bHelp )
{
	if (bHelp)
	{
		Asura_CommandConsole::Printf("deactivates entities which are further than a set dist from the camera (currently %.0fm)",
			Asura_Maths::FtoD(dvs_fDistToDeactivateEntities));
		return true;
	}
	else
	{
		const Asura_Vector_3*	pxCameraPos		= Asura_Camera::GetPosition();
		const float				fDistSquared	= dvs_fDistToDeactivateEntities * dvs_fDistToDeactivateEntities;

		Asura_CommandConsole::Printf("Deactivated...");
#if defined(ASURA_USE_CONTAINER_SYSTEM)
		Asura_CommandConsole::Printf("- %u containers", DeactivateDistantEntities(*pxCameraPos, fDistSquared, Asura_ServerEntity_Container::GetList()));
#endif	//defined(ASURA_USE_CONTAINER_SYSTEM)
#if defined(ASURA_TARGET_SM3)
		Asura_CommandConsole::Printf("- %u dynamic-lights", DeactivateDistantEntities(*pxCameraPos, fDistSquared, Asura_ServerEntity_DynamicLight::GetList()));
		Asura_CommandConsole::Printf("- %u light-shafts", DeactivateDistantEntities(*pxCameraPos, fDistSquared, Asura_ServerEntity_LightShaft::GetList()));
#endif	//defined(ASURA_TARGET_SM3)
		Asura_CommandConsole::Printf("- %u physical-objects", DeactivateDistantEntities(*pxCameraPos, fDistSquared, Asura_ServerEntity_PhysicalObject::GetList()));
		Asura_CommandConsole::Printf("- %u PFX-effects", DeactivateDistantEntities(*pxCameraPos, fDistSquared, Asura_ServerEntity_PFX_Effect::GetList()));
		Asura_CommandConsole::Printf("- %u volume-trigger", DeactivateDistantEntities(*pxCameraPos, fDistSquared, Asura_ServerEntity_VolumeTrigger::GetList()));

		Asura_CommandConsole::Printf("...%u entities now active.", Asura_Entity_System::GetNumServerEntitiesUpdating());
		return true;
	}
}

#if defined(ASURA_USE_PROFILING_SYSTEM)

void Asura_Entity_System::LogProfileData(const Asura_Char* szTitle, const Asura_Entity_System_ProfileData& xProfileData)
{
	Asura_OutputToDebugger_Retail(("\n********************************************************************************\n"));
	{
		// Log the title and basic stats...
		Asura_OutputToDebugger_Retail(("%s - %d classifications %d/%d updating...\n\n", szTitle, xProfileData.m_uNumClassificationsFound, xProfileData.m_uNumEntitiesUpdating, xProfileData.m_uNumEntities));

		// Log the data for each entity in-turn...
		for(u_int u = 0; u < xProfileData.m_uNumClassificationsFound; u++)
		{
			const u_short	usClassification		= xProfileData.m_ausEntityClassifications[u];
			const char*		szEntityName			= Asura_Network::GetEntityClassificationDesc(usClassification);
			const u_int		uNumEntities			= xProfileData.m_auNumEntitiesPerClassification[u];
			const u_int		uNumEntitiesUpdating	= xProfileData.m_auNumEntitiesUpdatingPerClassification[u];
			const float		fDuration				= xProfileData.m_afTimeUpdatingEntitiesPerClassification[u];

			Asura_OutputToDebugger_Retail(("- \"%s\" => %d/%d updating takes %.3fms\n", szEntityName, uNumEntitiesUpdating, uNumEntities, fDuration));
		}
	}
	Asura_OutputToDebugger_Retail(("********************************************************************************\n\n"));
}

bool Asura_Entity_System::ConsoleCmd_LogEntityProfiling(bool bHelp)
{
	if(bHelp)
	{
		Asura_CommandConsole::Printf("Logs the current state of entity profiling");
		return(true);
	}
	else
	{
		LogProfileData("Client-entities", s_xClientEntityProfileData);
		LogProfileData("Server-entities", s_xServerEntityProfileData);
		return(true);
	}
}

#endif	//defined(ASURA_USE_PROFILING_SYSTEM)

#endif	//defined(ASURA_DEBUG_VARIABLE_SYSTEM)


#if defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)
int Asura_Entity_System::GetNumClientEntitiesUpdating()
{
	return(s_xClientEntityProfileData.m_uNumEntitiesUpdating);
}

int Asura_Entity_System::GetNumServerEntitiesUpdating()
{
	return(s_xServerEntityProfileData.m_uNumEntitiesUpdating);
}

int Asura_Entity_System::GetNumRenderableEntities()
{
	return(s_uNumRenderableEntities);
}
#else
int Asura_Entity_System::GetNumClientEntitiesUpdating()
{
	return(Asura_Entity_Client::GetNumUpdating());
}

int Asura_Entity_System::GetNumServerEntitiesUpdating()
{
	return(Asura_Entity_Server::GetNumUpdating());
}

int Asura_Entity_System::GetNumRenderableEntities()
{
	return(Asura_Entity_Renderable::GetNumRenderable());
}
#endif	//defined(ASURA_USE_PROFILING_SYSTEM) && defined(ASURA_DEBUG_VARIABLE_SYSTEM)


/// End of file. ///
