//
// Asura_ServerEntity_AttractorController.h
//
// Created by Richard Rice - 09/05/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _ASURA_SERVERENTITY_ATTRACTORCONTROLLER_H_
#define _ASURA_SERVERENTITY_ATTRACTORCONTROLLER_H_

#ifdef ASURA_USE_ATTRACTORS


#include "Entity/Asura_Entity.h"


//Entity for forwarding activate/deactivate messages to the attractor system
//(aka Env Hooks)
class Asura_ServerEntity_AttractorController : public Asura_Entity_Server
{
	typedef Asura_Entity_Server PARENT;
	enum { CHUNK_VERSION = 0 };

	friend class Tools_Attractor_System;

public :
	static void Create(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);
	Asura_ServerEntity_AttractorController(Asura_Guid uGuid);

	~Asura_ServerEntity_AttractorController();

	// override of HasClientEntities() - we don't have any.
	virtual bool HasClientEntities() const { return false; }

	void SetToolsAttractorID(u_int uID){m_uToolsAttractorID = uID;}
	u_int GetToolsAttractorID() const {return m_uToolsAttractorID;}

	virtual bool HandleMessage(const Asura_Message* pxMessage);

	virtual bool	CanBeSavedToChunk() const {return true;}
	virtual	bool	IsFilledFromChunkBuffer		(void) const{ return (false); }
	virtual	void	ReadFromChunkStream			(Asura_Chunk_Stream& xStream);
	virtual	void	WriteToChunkStream			(Asura_Chunk_Stream& xStream) const;

protected :
	#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate() const;
	#endif


private :
   void SendAttractorFlagMessage(u_int uFlagsToSet,u_int uFlags);	

	#ifdef ASURA_USE_STREAMING
	Asura_Hash_ID m_uHandlerID;//hash ID of attractor handler
	#endif

	u_int m_uToolsAttractorID;

	//array of compiled attractor IDs that correspond with the single toold attractor
	u_int m_uNumCompiledAttractorIDs;
	u_int* m_puCompiledAttractorIDs;



#ifdef ASURA_TOOLS //entity list currently only needed by tools. RAR 09/05/2007
public :
	static Asura_ServerEntity_AttractorController* FindEntity( Asura_Guid uGuid )
	{
        return s_xList.FindEntity( uGuid );
    }
	static const Asura_Hashed_List<Asura_ServerEntity_AttractorController>* GetList( void )
    {
        return &s_xList;
    }

private:

	static Asura_Hashed_List<Asura_ServerEntity_AttractorController> s_xList;
#endif //ASURA_TOOLS

};

//Message used to forward activate/deactivate messages for attractors.
class Asura_Message_AttractorFlags
{
public :
	u_int m_uFlagsToSet;
	u_int m_uFlags;
	u_int m_uNumAttractorIDs;
	#ifdef ASURA_USE_STREAMING
	Asura_Hash_ID m_uHandlerID;//hash ID of attractor handler to use
	#endif

	//followed by array of attractor IDs
};

#endif //ASURA_USE_ATTRACTORS
#endif //_ASURA_SERVERENTITY_ATTRACTORCONTROLLER_H_
