/**
*
* \file Asura_ServerEntity_CutsceneController.h
*
* Created by Kevin Lea - 15/04/2002 12:12:20
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/
#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_SERVERENTITY_CUTSCENECONTROLLER_H__
#define __ASURA_SERVERENTITY_CUTSCENECONTROLLER_H__
 
#include "Core/Asura_String.h"
#include "Entity/Asura_Entity.h"

class Asura_Message;
class Asura_Chunk_Stream;
class Asura_Cutscene;

class Asura_ServerEntity_CutsceneController : public Asura_Entity_Server
{
public:

	Asura_ServerEntity_CutsceneController(const Asura_String& cutsceneName,Asura_Guid guid = ASURA_GUID_UNREGISTERED);
	Asura_ServerEntity_CutsceneController(Asura_Guid guid,const char*& chunkBuffer);
	Asura_ServerEntity_CutsceneController(Asura_Guid guid);
	virtual ~Asura_ServerEntity_CutsceneController();

	static void CreateFromChunkBuffer(Asura_Guid uGuid, const char*& pcChunkBuffer);

	virtual bool HandleMessage(const Asura_Message* pxMessage);

	const Asura_String* GetCutsceneName() const {return &CutsceneName;}

	//virtual function overides for saving
	virtual	bool IsFilledFromChunkBuffer(void) const { return (false); }

	virtual bool CanBeSavedToChunk() const {return true;}
	virtual int GetChunkBufferSize( void );
	virtual void FillInChunkBuffer( char*& pcChunkBuffer );
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;	
	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);

    void Update();

#ifdef ASURA_TOOLS
    virtual bool ShouldIncludeInLevelExport( void ) const;

    // static functions
    static void CutsceneRenamed(const Asura_String& xOldCutsceneName, const char* szNewCutsceneName);
#endif

	#ifdef ASURA_TOOLS
	virtual bool SupportsCustomEvents() const { return true; }
	virtual int GetNumCustomEvents() const;
	virtual unsigned short* GetCustomEvent(int iIndex);
	virtual void GetCustomEventName(int iEventIndex, Asura_String& xEventName);
	virtual int GetCustomEventMessageSet(int iEventIndex) const;
	#endif

	// Accessors
	float GetTimer() const { return m_fTimer; }
	u_int GetEnabledEvents() const { return m_uEnabledEvents; }
	bool GetDisabled() const { return m_uDisabled != 0; }
	bool GetPlaying() const  { return m_uPlaying != 0; }
	bool GetCheckStartActiveFlag() const { return m_uCheckStartActiveFlag != 0; }

	void SetDisabled( bool bDisable );

	// MGH 16/1/2007: Re-initialise the cutscene controller after a load, skipping forward to the specified time.
	// Don't send any static messages that would otherwise be triggered when skipping forward through time.
	void ReInitialise(float fTimer, u_int uEnabledEvents, bool bPlaying, bool bDisabled,
					  bool bCheckStartActiveFlag = false);

	// SK 24/04/2007 - I've moved this to from being a private function because i need to access it from outside.
	// I need to know if the controller thinks it's cutscene is still playing and the playing flag only seems 
	// to be updated when it receives an activate or deactivate message.
	// I'm also only calling it externally because i didn't want to mess with existing functionality
	// and this function might be processor intensive if it's constantly polled in this class update.
	// ORIGINAL NOTE: If flagged as playing , checks the timer to see if the cutscene should in fact still be playing
	void UpdatePlayingFlag();

	#ifdef ASURA_TOOLS
	static const Asura_Hashed_List<Asura_ServerEntity_CutsceneController>* GetList() { return &s_xList; }
	static Asura_ServerEntity_CutsceneController* FindEntity(Asura_Guid uGuid) { return s_xList.FindEntity(uGuid); }
	#endif

private:
    typedef Asura_Entity_Server PARENT;

	////If flagged as playing , checks the timer to see if the cutscene should in fact still be playing
	//void UpdatePlayingFlag();

	void Activate( bool bCutsceneStartsActive , float fTimer );

	Asura_String CutsceneName;

    union
    {
        u_int m_uStateFlags;

        struct
        {
			// NOTE: These bit's are written out, adding any new bits means
			// ~~~~~ that you will need to add those bits to the bitfield
			//       preparation code in "WriteToChunkStream".
            u_int m_uCheckStartActiveFlag : 1,
				  m_uDisabled : 1,
				  m_uManuallyDisabled : 1,
				  m_uPlaying : 1,
				  m_uCheckForCutsceneRestartAfterLoad :1,
				  //Change of plan... instead of completely disabling some cutscenes for PS2 export , we will
				  //just do enough of them to trigger any messages they may send.
				  //This does mean that having both diabled flgs above is a bit redundant , but i'm not sure
				  //how safe it would be to remove one of them. RAR 12/06/03
				  m_uTriggerMessagesOnly :1,
				  m_uSkipMessagesAfterLoad : 1,
                  m_uUnusedStateFlags      : 25;
        };
    };

	u_int m_uEnabledEvents;//bitfield of enabled events

	float m_fTimer;//Time since cutscene starter playing

	friend class Tools_Entity_CutsceneController_Object;

	#ifdef ASURA_TOOLS
	static Asura_Hashed_List<Asura_ServerEntity_CutsceneController> s_xList;
	#endif

};


//This message is saved out to static message system , so care needs to be taken about altering it
//Used by ASURA_MSG_CUTSCENE_ENABLEEVENT and ASURA_MSG_CUTSCENE_DISABLEEVENT
struct Asura_Message_CutsceneEventID
{
	int m_iEventID;
};

struct Asura_Message_CutsceneEventState
{
	u_int m_uEnabledEvents;
};

struct Asura_Message_Cutscene_SetCamera
{
	Asura_Hash_ID m_uCameraHash;//hash from camera actor's name
};

#ifdef ASURA_TOOLS
//For level export bit manipulation when modifying the chunks.
//Needs to correspond to the ordering in the flags union above
#define AsuraCutsceneControllerFlag_Disabled 0x00000002
#define AsuraCutsceneControllerFlag_TriggerMessagesOnly 0x00000020
#endif

#endif//__ASURA_SERVERENTITY_CUTSCENECONTROLLER_H__
