//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_ServerCutsceneActor.cpp
//
// Created by Mike Healey.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Entities to send updates from ServerCutscene to the client entities which 
// render the actors.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_SERVERCUTSCENES	// the Gunfighter-style cutscenes with entities

// This file's header.
#include "Entity/Cutscene/Asura_ServerEntity_ServerCutsceneActor.h"

// Engine independent.
#include "Animation/Asura_TextureAnimation.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_Actor.h"
#include "Cutscene/Asura_Cutscene_Event.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Cutscene/Asura_Cutscene_Track.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Cutscene/Asura_ClientEntity_ServerCutsceneActor.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"

Asura_ServerEntity_ServerCutsceneActor::Asura_ServerEntity_ServerCutsceneActor( Asura_Cutscene_Actor* pxActor,
                                                                                Asura_Guid uGUID )
: PARENT( uGUID )
{
    m_pxActor     = pxActor;
    m_uCutsceneHash = Asura_GetHashID( pxActor->Cutscene->CutsceneName.GetString() );
    m_uFlags      = 0;
    
    // See note at the top of Asura_ServerEntity_ServerCutsceneActor.h.
    SetUpdateClientEveryFrame( true );
}

Asura_ServerEntity_ServerCutsceneActor::~Asura_ServerEntity_ServerCutsceneActor()
{
    if (m_pxActor)
    {
        m_pxActor->NotifyDestruction(this);
    }
}

void Asura_ServerEntity_ServerCutsceneActor::NotifyActorDestroyed( Asura_Cutscene_Actor* pxActor )
{
	if(m_pxActor)
	{
		Asura_Assert(m_pxActor == pxActor,"");
		m_pxActor = 0;
	}
}

Asura_ServerEntity_ServerCutsceneActorHierarchy::Asura_ServerEntity_ServerCutsceneActorHierarchy( Asura_Cutscene_Actor* pxActor,
                                                                                                  int iSkinID,
                                                                                                  int iInitialAnimationID,
                                                                                                  int iAnimationFlags,
                                                                                                  float fAnimationDelay,
                                                                                                  Asura_Guid uGUID )
: Asura_Entity_Server( uGUID )
, PARENT( pxActor, uGUID )
{
    Asura_Cutscene_Actor_Hierarchy* pxHierarchyActor = (Asura_Cutscene_Actor_Hierarchy*) pxActor;
	// Check validity of cutscene by getting its pointer
	Asura_Assert(Asura_Cutscene_System::GetCutsceneByHash(m_uCutsceneHash), "failed to find cutscene");

	Classification = AsuraEntityClass_ServerCutsceneActorHierarchy;

    m_iSkinID         = iSkinID;
	m_iAnimationID    = iInitialAnimationID;
	m_iAnimationFlags = iAnimationFlags;
	m_fAnimationDelay = fAnimationDelay;
	m_uTextureHash = pxHierarchyActor->m_uTextureHash;

	if(pxHierarchyActor->m_uHierarchyFlags & ASURA_HIERARCHYACTORFLAG_NOTRENDERED)
	{
		m_uFlags |= Asura_SESCU_Flag_NotRendered;
	}

	
}


Asura_ServerEntity_ServerCutsceneActorHierarchy::~Asura_ServerEntity_ServerCutsceneActorHierarchy()
{}



void Asura_ServerEntity_ServerCutsceneActorHierarchy::MakeClientUpdateMessage(void)
{
    if (!m_pxActor)
    {
        Asura_Assert(m_pxActor, "");//try and find it here?
        return;
    }

	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_STANDARD,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateStandard));
	if (pxMessage)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdateStandard* pxData = 
            (Asura_Message_ServerCutsceneActor_ClientUpdateStandard*)pxMessage->GetData();
		
		//Determine if anim movement data should be used. RAR 06/01/2006
		float fSpeed = m_pxActor->Tracks[m_pxActor->CurrentTrack]->GetSpeedFromTime(m_pxActor->Cutscene->Timer);
		Asura_Cutscene_Actor_Hierarchy* pxHierActor = (Asura_Cutscene_Actor_Hierarchy*) m_pxActor;
		if(fSpeed!=0.0f || pxHierActor->GetIgnoreAnimMovement())
		{
			m_uFlags &=~ Asura_SESCU_Flag_UseAnimMovementData;
		}
		else
		{
			m_uFlags |= Asura_SESCU_Flag_UseAnimMovementData;
		}


	    pxData->uCutsceneHash = m_uCutsceneHash;
        pxData->sSkinID = static_cast<short>(m_iSkinID);
        pxData->sAnimationID = static_cast<short>(m_iAnimationID);
		pxData->iAnimationFlags = m_iAnimationFlags;
		pxData->fAnimationDelay = m_fAnimationDelay;
        pxData->uFlags = m_uFlags;
        pxData->xPosition = m_pxActor->Position;
	    pxData->xOrientation.FromMatrix(m_pxActor->Orientation);
        pxData->fSpeed = fSpeed;
        pxData->fCutsceneTime = m_pxActor->Cutscene->Timer;
		pxData->uTextureHash = m_uTextureHash;

	    Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}

void Asura_ServerEntity_ServerCutsceneActorHierarchy::ChangeAnimation(int animationID,
                                                                      float time, 
                                                                      int flags,
                                                                      float tweenDuration) const
{
    const Asura_String& xAnimName = Asura_Cutscene_System::GetCutsceneByHash(m_uCutsceneHash)->AnimationNames.GetName(animationID);
    Asura_Hash_ID uHashID = Asura_GetHashID(xAnimName.GetString());
    ChangeAnimation(uHashID, time, flags, tweenDuration);
}

void Asura_ServerEntity_ServerCutsceneActorHierarchy::ChangeAnimation(Asura_Hash_ID uAnimHashID,
                                                                      float fTime,
                                                                      int fFlags,
                                                                      float fTweenDuration) const
{
	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_PRIMARYANIMATION,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateChangePrimaryAnim));
	if (m)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdateChangePrimaryAnim* data = 
            (Asura_Message_ServerCutsceneActor_ClientUpdateChangePrimaryAnim*)m->GetData();

	    data->m_uAnimHashID = uAnimHashID;
        data->m_iAnimationFlags = fFlags;
        data->m_fTweenDuration = fTweenDuration;
        data->m_fCutsceneTime = fTime;

	    Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}

void Asura_ServerEntity_ServerCutsceneActorHierarchy::PlayAnimationToPos(int iAnimationID,
																		 float fTime,
																		 float fStartTimer,
																		 float fEndTimer,
																		 int iFlags,
																		 float fTweenDuration) const
{
	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_PRIMARYANIMATION_TOPOS,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdatePrimaryAnim_ToPos));
	if (m)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdatePrimaryAnim_ToPos* data = 
            (Asura_Message_ServerCutsceneActor_ClientUpdatePrimaryAnim_ToPos*)m->GetData();

	    data->AnimationID = static_cast<short>(iAnimationID);
        data->AnimationFlags = iFlags;
        data->TweenDuration = fTweenDuration;
		data->m_fStartTimer = fStartTimer;
		data->m_fEndTimer = fEndTimer;

	    Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}

void Asura_ServerEntity_ServerCutsceneActorHierarchy::PlaySecondaryAnimation(Asura_Hash_ID uAnimHashID,
																			 float fTime,
                                                                             int fFlags,
                                                                             float fWeight,
                                                                             float fTweenDuration) const
{
	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_SECONDARYANIMATION,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateSecondaryAnim));
	if (m)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdateSecondaryAnim* data = 
            (Asura_Message_ServerCutsceneActor_ClientUpdateSecondaryAnim*)m->GetData();

	    data->m_uAnimHashID = uAnimHashID;
        data->m_iAnimationFlags = fFlags;
        data->m_fWeight = fWeight;
        data->m_fTweenDuration = fTweenDuration;

	    Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
    }	
}


void Asura_ServerEntity_ServerCutsceneActorHierarchy::PlaySecondaryAnimation(int animationID,
                                                                             float time,
                                                                             int flags,
                                                                             float weight,
                                                                             float fTweenDuration) const
{
    const Asura_String& xAnimName = Asura_Cutscene_System::GetCutsceneByHash(m_uCutsceneHash)->AnimationNames.GetName(animationID);
    Asura_Hash_ID uHashID = Asura_GetHashID(xAnimName.GetString());
	PlaySecondaryAnimation(uHashID, time, flags, weight, fTweenDuration);

//	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_SECONDARYANIMATION,
//                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateSecondaryAnim));
//	if (m)
//    {
//        Asura_Message_ServerCutsceneActor_ClientUpdateSecondaryAnim* data = 
//            (Asura_Message_ServerCutsceneActor_ClientUpdateSecondaryAnim*)m->GetData();
//
//	    data->AnimationID = static_cast<short>(animationID);
//        data->AnimationFlags = flags;
//        data->Weight = weight;
//        data->Duration = duration;
//
//	    Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
//    }
}



void Asura_ServerEntity_ServerCutsceneActorHierarchy::PlaySecondaryAnimationToPos(int iAnimationID,
																				  float fTime,
																				  float fStartTimer,
																				  float fEndTimer,
																				  int iFlags,
																				  float fWeight,
																				  float fTweenDuration) const
{
	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_SECONDARYANIMATION_TOPOS,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateSecondaryAnim_ToPos));
	if (m)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdateSecondaryAnim_ToPos* data = 
            (Asura_Message_ServerCutsceneActor_ClientUpdateSecondaryAnim_ToPos*)m->GetData();

	    data->AnimationID = static_cast<short>(iAnimationID);
        data->AnimationFlags = iFlags;
        data->Weight = fWeight;
        data->TweenDuration = fTweenDuration;
		data->m_fStartTimer = fStartTimer;
		data->m_fEndTimer = fEndTimer;

	    Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}

void Asura_ServerEntity_ServerCutsceneActorHierarchy::ChangeTextureAnimation(int iSequence,float fSpeed,u_int uFlags) const
{
	//create a texture anim message
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_TEXTUREANIMATION,sizeof(Asura_Message_TextureAnimation));
	if(pxMessage)
	{
		Asura_Message_TextureAnimation* pxData = (Asura_Message_TextureAnimation*)pxMessage->GetData();

		pxData->m_uSequence = (u_int) iSequence;
		pxData->m_fSpeed = fSpeed;

		//can't just copy flags directly unfortunately since they are accessed in different ways
		pxData->m_uFlags = 0;
		pxData->m_bChangeSequence = (uFlags & ASURA_CUTSCENEEVENT_TEXTUREANIMATION_CHANGESEQUENCE) ? 1 : 0;
		pxData->m_bChangeSpeed = (uFlags & ASURA_CUTSCENEEVENT_TEXTUREANIMATION_CHANGESPEED) ? 1 : 0;

	    Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
	}

}

void Asura_ServerEntity_ServerCutsceneActorHierarchy::AddAttachment_Shape(u_int uID,Asura_Hash_ID uMarkerHash,Asura_Hash_ID uShapeHash)
{
	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_ADDATTACHMENTSHAPE,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentShape));
	if (m)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentShape* pxData = 
            (Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentShape*)m->GetData();

		pxData->m_uID = uID;
		pxData->m_uMarkerHash = uMarkerHash;
		pxData->m_uShapeHash = uShapeHash;

		Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}

void Asura_ServerEntity_ServerCutsceneActorHierarchy::AddAttachment_Hierarchy(u_int uID,Asura_Hash_ID uMarkerHash,int iSkinID,int iAnimID)
{
	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_ADDATTACHMENTHIERARCHY,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentHierarchy));
	if (m)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentHierarchy* pxData = 
            (Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentHierarchy*)m->GetData();

		pxData->m_uID = uID;
		pxData->m_uMarkerHash = uMarkerHash;
		pxData->m_iSkinID = iSkinID;
		pxData->m_iAnimID = iAnimID;

	    Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}

void Asura_ServerEntity_ServerCutsceneActorHierarchy::AddAttachment_Actor(u_int uID,Asura_Hash_ID uMarkerHash,Asura_Guid uActorID)
{
	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_ADDATTACHMENTACTOR,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentActor));
	if (m)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentActor* pxData = 
            (Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentActor*)m->GetData();

		pxData->m_uID = uID;
		pxData->m_uMarkerHash = uMarkerHash;
		pxData->m_uActorID = uActorID;

		Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}

void Asura_ServerEntity_ServerCutsceneActorHierarchy::AddAttachment_Entity(u_int uID,Asura_Hash_ID uMarkerHash,Asura_Guid uEntityGUID)
{
	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_ADDATTACHMENTENTITY,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentEntity));
	if (m)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentEntity* pxData = 
            (Asura_Message_ServerCutsceneActor_ClientUpdateAddAttachmentEntity*)m->GetData();

		pxData->m_uID = uID;
		pxData->m_uMarkerHash = uMarkerHash;
		pxData->m_uEntityGUID = uEntityGUID;

		Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}


void Asura_ServerEntity_ServerCutsceneActorHierarchy::RemoveAttachment(u_int uID)
{
	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTOR_CLIENTUPDATE_REMOVEATTACHMENT,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateRemoveAttachment));
	if (m)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdateRemoveAttachment* pxData = 
            (Asura_Message_ServerCutsceneActor_ClientUpdateRemoveAttachment*)m->GetData();

		pxData->m_uID = uID;

	    Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}




Asura_ServerEntity_ServerCutsceneActorCamera::Asura_ServerEntity_ServerCutsceneActorCamera( Asura_Cutscene_Actor* pxActor )
: Asura_Entity_Server( ASURA_GUID_UNREGISTERED )
, PARENT( pxActor, ASURA_GUID_UNREGISTERED )
{
}

void Asura_ServerEntity_ServerCutsceneActorCamera::MakeClientUpdateMessage()
{
    
	Asura_Message* m = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE,
                                                    sizeof(Asura_Message_ServerCutsceneActor_ClientUpdateCamera),
                                                    AsuraEntityClass_ServerCutsceneActorCamera);
	if (m)
    {
        Asura_Message_ServerCutsceneActor_ClientUpdateCamera* data = 
            (Asura_Message_ServerCutsceneActor_ClientUpdateCamera*)m->GetData();
		
		//Take the position from the cutscene's active camera , rather than the actor
		//that was used to create this entity. RAR 14/10/2005
		const Asura_Cutscene_Actor_Camera* pxActiveCamera = m_pxActor->Cutscene->GetActiveCamera();
		Asura_Assert(pxActiveCamera,"");
	    
		data->xPosition = pxActiveCamera->Position;
	    data->xOrientation.FromMatrix(pxActiveCamera->Orientation);
		data->m_fFOV = pxActiveCamera->GetFOV();
		data->m_ucWidescreen = (pxActiveCamera->Cutscene->m_iCutsceneFlags & AsuraCutsceneFlag_Widescreen) ? 1 : 0;

	    Send(m, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}


/////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------**
** Asura_ServerEntity_ServerCutsceneActorShape **
**---------------------------------------------*/

Asura_ServerEntity_ServerCutsceneActorShape::Asura_ServerEntity_ServerCutsceneActorShape( Asura_Cutscene_Actor* pxActor,
												                                          Asura_Hash_ID uShapeHashID,
                                                                                          Asura_Guid uGUID )
: Asura_Entity_Server( uGUID )
, PARENT( pxActor, uGUID )
{
	// Check validity of cutscene by getting its pointer
	Asura_Assert(Asura_Cutscene_System::GetCutsceneByHash(m_uCutsceneHash), "failed to find cutscene");

	Classification = AsuraEntityClass_ServerCutsceneActorShape;

	m_uShapeID = uShapeHashID;
}

void Asura_ServerEntity_ServerCutsceneActorShape::MakeClientUpdateMessage(void)
{
    if (!m_pxActor)
    {
        Asura_Assert(m_pxActor, "");//try and find it here?
        return;
    }

	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_SERVERCUTSCENEACTORSHAPE_CLIENTUPDATE_STANDARD,
                                                    sizeof(Asura_Message_ServerCutsceneActorShape_ClientUpdateStandard));
	if (pxMessage)
    {
        Asura_Message_ServerCutsceneActorShape_ClientUpdateStandard* pxData = 
            (Asura_Message_ServerCutsceneActorShape_ClientUpdateStandard*)pxMessage->GetData();

	    pxData->m_uCutsceneHash = m_uCutsceneHash;
        pxData->m_uShapeID = m_uShapeID;
        pxData->m_xPosition = m_pxActor->Position;
	    pxData->m_xOrientation.FromMatrix(m_pxActor->Orientation);
        pxData->m_fCutsceneTime = m_pxActor->Cutscene->Timer;

	    Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
    }
}

#endif // #ifdef ASURA_SUPPORT_SERVERCUTSCENES	// the Gunfighter-style cutscenes with entities
