//////////////////////////////////////////////////////////////////////////////
//
// Asura_Entity_Ephemeron.h
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Author: Mike Healey 28.11.02
//
// Description:
//
//     Base class for short-lived client-only objects with a shape instance.
//      
//     Evolution of Richard May's shell casing code.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_ENTITY_EPHEMERON_H___
#define ___ASURA_ENTITY_EPHEMERON_H___ 1

#include "Core/Asura_Maths.h"
#include "Entity/Asura_Entity.h"
#include "Shapes/Asura_Shapes.h"

class Asura_Entity_Ephemeron : public Asura_Entity_Renderable
{

public:
	/* Constructors/destructor. */
	Asura_Entity_Ephemeron(Asura_Hash_ID uShapeHashID,
                           const Asura_Vector_3& xPos, 
                           const Asura_Matrix_3x3& xOrient, 
                           const Asura_Vector_3& xInitialVelocity, 
                           float fMaximumSpinRate,
                           float fLifeTime);
	virtual ~Asura_Entity_Ephemeron	();

public:
	/* Public interface functions. */
    // virtual overrides of Asura_Entity_Renderable
	virtual void Update(void);
	void Render(void);

	const float& GetVelocityDecayRate() const { return m_fVelocityDecayRate; }
	void  SetVelocityDecayRate( float fVelocityDecayRate ) { m_fVelocityDecayRate = fVelocityDecayRate; }
	const Asura_Vector_3& GetAcceleration() const { return m_xAcceleration; } 
    void  SetAcceleration(const Asura_Vector_3& xAcceleration ) { m_xAcceleration = m_xAcceleration; }
	const Asura_Euler_3& GetSpinRate() const { return m_xSpinRate; }
	void  SetSpinRate( const Asura_Euler_3& xSpinRate) { m_xSpinRate = xSpinRate; }

	float GetLifeTimer() const { return m_fLifeTimer; }
	const Asura_Vector_3& GetPosition() { return m_xCurrentPosition; }
	const Asura_Matrix_3x3& GetOrientation() { return m_xCurrentOrientation; }

protected:
	/* Helper functions. */
    virtual void MakeImpactSound() {}
    virtual void DealWithImpact();

protected:
	/* Data. */
	Asura_Euler_3 m_xSpinRate;
	Asura_Euler_3 m_xSpinAngle;
	
	Asura_Shape_Instance m_xShapeInstance;
	Asura_Vector_3 m_xVelocity;
	Asura_Vector_3 m_xAcceleration;
	float   m_fLifeTimer;
	float   m_fVelocityDecayRate;
	bool    m_bActive;
	u_short m_usFlagsOfPolysToIgnore;
};


// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_ENTITY_EPHEMERON_H___
