
////////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_FMVController.h
// Author    : Jason Turner
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_SERVERENTITY_FMVCONTROLLER_H__
#define __ASURA_SERVERENTITY_FMVCONTROLLER_H__

#include "Core/Asura_String.h"
#include "Core/Asura_TemplateTree.h"
#include "Entity/Triggers/Asura_ServerEntity_Activatable.h"

#ifdef ASURA_TOOLS
#include "Entity/Asura_Entity_Classification.h"
#endif

class Asura_VideoToTexture;

class Asura_ServerEntity_FMVController : public Asura_ServerEntity_Activatable
{
	typedef Asura_ServerEntity_Activatable PARENT;

	public:
		Asura_ServerEntity_FMVController(Asura_Guid uGuid);
		virtual ~Asura_ServerEntity_FMVController();

		static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream &xStream );

		static Asura_ServerEntity_FMVController* FindEntity( const Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }
		static Asura_Hashed_List< Asura_ServerEntity_FMVController >* GetList() { return &s_xList; }

	protected:

		// Overrides
		virtual bool HandleMessage(const Asura_Message* pxMessage);

		// Read/write to chunk streams
		virtual	bool IsFilledFromChunkBuffer(void) const { return false; }
		virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
		virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );

		Asura_String	m_xFMVName;
		unsigned int	m_uFlags;

#ifdef ASURA_TOOLS
		virtual bool CanBeIncludedInTemplate() const {return Classification == AsuraEntityClass_FMVController;}
		#endif

		#ifdef ASURA_TOOLS
		friend class Tools_Entity_FMVController;
		#endif

		friend class Asura_Message_FMVController_ClientCreate;

    private:

		friend class Asura_ServerEntity_FMVSpeaker;

		#ifdef ASURA_TOOLS
		friend class Tools_ChooseObject_FMVSpeaker;
		#endif

		Asura_ServerEntity_FMVController();

		Asura_TemplateTree<Asura_Guid, Asura_Guid>& GetSpeakerList() {return m_xLinkedSpeakerGuids;};

		// if you change this, make sure ReadFromChunkStream/WriteToChunkStream still support old versions
		enum { CURRENT_CHUNK_VERSION = 1 };
		enum { MAX_LINKED_SPEAKERS = 8 };

		// list of FMV Speakers used by this Controller
		Asura_TemplateTree<Asura_Guid, Asura_Guid> m_xLinkedSpeakerGuids;

		static int GuidCompareFunction(const Asura_Guid& xKey1, const Asura_Guid& xKey2)
		{
			return static_cast<int>(xKey2 - xKey1);
		}
		void AddLinkedSpeaker(Asura_Guid uGuid);
		void RemoveLinkedSpeaker(Asura_Guid uGuid);

		static Asura_Hashed_List< Asura_ServerEntity_FMVController > s_xList;
};

#endif //__ASURA_SERVERENTITY_FMVCONTROLLER_H__
