//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_Glass.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_CLIENTENTITY_GLASS_H___
#define ASURA_CLIENTENTITY_GLASS_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Entity/Glass/Asura_WorldSpace_Rectangle.h"
#include "Entity/PhysicalObject/Asura_ClientEntity_PhysicsObject.h"

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_ClientEntity_Glass : public virtual Asura_ClientEntity_PhysicsObject
{

public:

	static void Create( const Asura_Message* pxMsg );
	static Asura_ClientEntity_Glass* Project_Create( const Asura_Message* pxMsg );
	
#ifdef ASURA_TOOLS

	static Asura_ClientEntity_Glass* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
	static const Asura_Hashed_List<Asura_ClientEntity_Glass>* GetList( void ) { return &s_xList; }

#endif

	virtual bool HandleMessage(const Asura_Message* pxMessage);
	virtual void Update() {} // don't pass to parent
	virtual void MakeDead() {} // do nothing

	// Copy position and orientation from dynamics block to pane.
	// Call this whenever you change position/orientation in tools.
	void CopyPositionToPane();

	int GetTextureIndex() const {return m_iTextureIndex;}

protected:

	Asura_ClientEntity_Glass();
	virtual ~Asura_ClientEntity_Glass() 
	{
#ifdef ASURA_TOOLS
		s_xList.RemoveFromList(this);
#endif
	}

	virtual void Render();
	virtual void Platform_Render(unsigned uCamVisFlags);
	virtual bool LOSTest(const Asura_LOS_InputData* pxInputData, Asura_LOS_OutputData* pxOutputData);
	virtual void InitialiseFromChunkBuffer(const char*& pcChunkBuffer);

	Asura_Vector_3 GetBlastDirection(bool bSmashInPositiveZDir) const; 
	void ApplyDecal(const Asura_Vector_2& xDecalCoordinates) const;
	void SmashGlass(const Asura_Vector_2& xHitCoordinates, float fPower, const Asura_Vector_3& xDirection);	// Power is in arbitrary units in range of 0-1.
	void SmashGlass(float fPower, bool bSmashInPositiveZDir);	// Un-specified hit position, for e.g. grenade hits, Power is in arbitrary units in range 0-1.
	void SetupCullingData();

#if defined(ASURA_USE_STREAMING)
	Asura_Hash_ID GetContainingZoneFromModuleTest() const { return m_uLastKnownZone; }
#endif //defined(ASURA_USE_STREAMING)

	Asura_WorldSpace_Rectangle m_xPane;
	float m_afU[4];
	float m_afV[4];
	unsigned int m_uColour;
	int m_iTextureIndex;
    union
    {
        struct
        {
            u_int m_bUseEnvironmentMap :  1,
                  SpareGlassBits       : 31;
        };

        u_int m_uGlassFlags;
    };
	// Culling data.
	Asura_Bounding_Box m_xBB;
	Asura_Vector_3 m_xCentre;
	float m_fRadius;
	int m_iLastKnownModule;

#if defined(ASURA_USE_STREAMING)
	Asura_Hash_ID m_uLastKnownZone;
#endif //defined(ASURA_USE_STREAMING)

	// Friends
	friend class Asura_SFX_GlassSmash;
	friend class Tools_Entity_Glass_Object;
	friend class Asura_SortedRender;
	friend struct Asura_SortedRenderItem_Glass;

private:

    typedef Asura_ClientEntity_PhysicsObject PARENT;

#ifdef ASURA_TOOLS
    static Asura_Hashed_List<Asura_ClientEntity_Glass> s_xList;
#endif

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_CLIENTENTITY_GLASS_H___
