
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_ServerEntity_Teleporter.h
// Author : Andy Tate
// Date   : 27-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_SERVERENTITY_TELEPORTER_H__
#define __ASURA_SERVERENTITY_TELEPORTER_H__

#if !defined(ASURA_DONT_USE_TELEPORTERS)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/Asura_Entity.h"
#include "Teleport/Asura_Teleport_System.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Tools_EntityTemplateInstance;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_ServerEntity_Teleporter
// Author       : Andy Tate
// Date         : 27-Feb-2007
//
// Description  : Class to handle the teleporting of entities to specified
//				  destination positions.
////////////////////////////////////////////////////////////////////////////////

class Asura_ServerEntity_Teleporter : public Asura_Entity_Server
{
	// Friend classes...
	friend class Tools_Entity_Teleporter;

public:
	// Public functions...
	static Asura_ServerEntity_Teleporter*	Create(const Asura_Guid uGuid);

	// Virtual functions...

	// override of HasClientEntities() - we don't have any.
	virtual bool HasClientEntities() const { return false; }

	virtual	bool	SetActive(const bool bActive);

	// Overridden functions...
	virtual	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

//	virtual	void	Update(void);
	virtual	bool	HandleMessage(const Asura_Message* pxMessage);

	// Legacy chunk handling, to be removed as soon as possible.
	virtual	bool	IsFilledFromChunkBuffer(void) const	{ return(false); }
	virtual	int		GetChunkBufferSize(void)			{ return(0); }

//	virtual	void	InitialiseFromChunkBuffer(const char*& pcChunkBfr);
//	virtual	void	FillInChunkBuffer(char*& pcChunkBfr);

	virtual	bool	CanBeSavedToChunk(void) const		{ return(true); }

#if defined(ASURA_TOOLS)
	// Tools creation function...
	static Asura_ServerEntity_Teleporter* CreateFromTools(const Asura_Guid uGuid, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation);

	// Template functions...
	virtual	void	ApplyChangesForTemplateInstance	(Tools_EntityTemplateInstance* pxInstance);
	virtual	bool	CanBeIncludedInTemplate			(void) const	{ return(true); }
#endif	// defined(ASURA_TOOLS)

	// Accessor functions...
	bool	IsActive	(void) const	{ return(m_bActive); }

	void	SetPerformFade	(const bool bSet)	{m_bPerformFade = bSet; }
	bool	CanPerformFade	(void) const		{ return(m_bPerformFade); }

	// List of teleporters accessor functions...
	static Asura_ServerEntity_Teleporter*							FindEntity(const Asura_Guid uGuid)	{ return(s_xList.FindEntity(uGuid)); }
	static const Asura_Hashed_List<Asura_ServerEntity_Teleporter>*	GetList(void)						{ return(&s_xList); }

	////////////////////////////////////////////////////////////////////////////////
	// Constant declarations...
	////////////////////////////////////////////////////////////////////////////////

	static const u_int	uTELEPORT_CHUNK_VERSION	 = 1;

protected:
	// Protected variables...
	union
	{
		u_int		m_uFlags;		// Single access-point for this entity's flags.

		struct
		{
			u_int	m_bActive		: 1,	// Flag to say whether this entity is currently active.
					m_bPerformFade	: 1,	// Flag to say that this teleporter fades.
									: 0;	// 31 spare-bits.
		};
	};

	float		m_fFadeOutDuration;													// The duration the fade-out before teleporting will take.
	float		m_fFadeInDuration;													// The duration the fade-in after teleporting will take.
	u_int		m_uFadeColour;														// The colour of the fade-out and fade-in used to hide the teleport.
	Asura_Guid	m_auDestinationGuids[ASURA_TELEPORTER_ENTITY_MAX_DESTINATIONS];		// The Guids of each of the start-points connected to this teleporter.


	// Protected functions...
	Asura_ServerEntity_Teleporter(const Asura_Guid uGuid);
	virtual ~Asura_ServerEntity_Teleporter();


private:
	// Private typedefs...
	typedef Asura_Entity_Server PARENT;

	// Private variables...
	// Static list of teleporters...
	static Asura_Hashed_List<Asura_ServerEntity_Teleporter>	s_xList;

	// Private functions...
	Asura_ServerEntity_Teleporter  (void);
	Asura_ServerEntity_Teleporter  (const Asura_ServerEntity_Teleporter& xOther);

	Asura_ServerEntity_Teleporter&	operator=	(const Asura_ServerEntity_Teleporter& xOther);

};   // End of class 'Asura_ServerEntity_Teleporter'.


#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

#endif   // ifndef __ASURA_SERVERENTITY_TELEPORTER_H__


/// End of file. ///
