//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_LightShaft.cpp
//
// Copyright (c) 2007 Rebellion (Derby). All rights reserved.
//
// Description:
//
// Volumetric-ish light shafts
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_TARGET_SM3

// This file's header.
#include "Entity/LightShaft/Asura_ClientEntity_LightShaft.h"

// Engine independent.
#include "Core/Asura_Timers.h"
#include "Core/Asura_TextureManagement.h"
#include "Camera/Asura_Camera.h"
#include "Network/Asura_Message.h"
#include "SpecialFX/Asura_LightShaft_System.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/PhysicalObject/Asura_ClientEntity_PhysicalObject.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

Asura_Hashed_List<Asura_ClientEntity_LightShaft> Asura_ClientEntity_LightShaft::s_xList;


Asura_ClientEntity_LightShaft::Asura_ClientEntity_LightShaft( Asura_Guid uGuid)
	: PARENT(uGuid)
	, m_xParams()
	, m_bAttachedToLight(false)
	, m_pxLightShaft(0)
{
	s_xList.AddToList(this);

	Classification = AsuraEntityClass_LightShaft;
}

Asura_ClientEntity_LightShaft* Asura_ClientEntity_LightShaft::CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream &xStream)
{
	Asura_ClientEntity_LightShaft* pxNew = new Asura_ClientEntity_LightShaft(uGuid);

	pxNew->ReadFromServerLightShaftChunkStream(xStream);

	pxNew->SetParams(pxNew->m_xParams);

	pxNew->Update();

	return pxNew;
}

#ifdef ASURA_TOOLS
Asura_ClientEntity_LightShaft* Asura_ClientEntity_LightShaft::CreateFromTools( Asura_Guid uGuid, const Asura_LightShaft_Parameters &xParams)
{
	Asura_ClientEntity_LightShaft* pxNew = new Asura_ClientEntity_LightShaft(uGuid);

	pxNew->SetEntityActiveFromLoad(pxNew->IsEntityActive());
	pxNew->SetEntityActive(pxNew->IsEntityActive());

	pxNew->SetParams(xParams);

	pxNew->Update();

	return pxNew;
}
#endif


Asura_ClientEntity_LightShaft::~Asura_ClientEntity_LightShaft( void )
{
	s_xList.RemoveFromList(this);

	if (m_pxLightShaft)
	{
		Asura_LightShaft_System::DestroyLightShaft(m_pxLightShaft);		
		m_pxLightShaft=NULL;
	}
}

void Asura_ClientEntity_LightShaft::ReadFromServerLightShaftChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int uVersion;
	
	xStream >> uVersion;
	Asura_Assert( uVersion <= Asura_ServerEntity_LightShaft::LIGHTSHAFT_CHUNK_VERSION, "" );

	bool bActive = true;

	if (uVersion >= 3)
	{
		Asura_ServerEntity_LightShaft::SkippedServerData xSkippedData;
		Asura_ServerEntity_LightShaft::SkipDataInChunkStream(xStream, &xSkippedData);
		bActive = xSkippedData.m_bActive && xSkippedData.m_bTemplateActive;
	}
	Asura_ServerEntity_LightShaft::ExtractParamsFromChunkStream(m_xParams, xStream, uVersion );
	SetEntityActiveFromLoad(bActive);
	SetEntityActive(bActive);
}

void Asura_ClientEntity_LightShaft::OnEntityActivate(void)
{
	AddToUpdateableClientList(this);
	PARENT::OnEntityActivate();

	SetParams(m_xParams);

	if (m_pxLightShaft && !m_bAttachedToLight)
	{
		m_pxLightShaft->SetEnable(true);
	}
}

void Asura_ClientEntity_LightShaft::OnEntityDeactivate(void)
{
	RemoveFromUpdateableClientList(this, false);
	PARENT::OnEntityDeactivate();

	SetParams(m_xParams);

	if (m_pxLightShaft && !m_bAttachedToLight)
	{
		m_pxLightShaft->SetEnable(false);
	}
}


void Asura_ClientEntity_LightShaft::Update( void )
{        
	// If texture isn't there, keep calling SetParams, as it may not have been loaded yet. Especially in tools
	if (m_pxLightShaft==NULL || m_pxLightShaft->m_aiTextureIndices[0]<0 || m_pxLightShaft->m_aiTextureIndices[1]<0)
	{
		SetParams(m_xParams);		
	}	
}



void Asura_ClientEntity_LightShaft::SetParams( const Asura_LightShaft_Parameters& xNewParams )
{
	m_xParams = xNewParams;

	if (m_pxLightShaft==NULL)
	{
		m_pxLightShaft = Asura_LightShaft_System::CreateLightShaft();		
		m_pxLightShaft->SetEnable(false);
	}

	if (m_pxLightShaft!=NULL)
	{
		// Update the pos/orient
		
		if (!m_bAttachedToLight)
		{		
			m_pxLightShaft->m_xPos	  = m_xParams.m_xPosition;
			m_pxLightShaft->m_xOrient = m_xParams.m_xOrientation;
		}

		// Update cone geometry params
		for(u_int u = 0; u < 2; ++u)
		{
			m_pxLightShaft->m_afDistances[u]  = m_xParams.m_afDistances[u];
			m_pxLightShaft->m_afRadii[u]      = m_xParams.m_afRadii[u];
			m_pxLightShaft->m_axColours[u]	  = m_xParams.m_axColours[u];						
		}

		// Update texturing params
		for(u_int u = 0; u < 2; ++u)
		{
			int iTex = Asura_Texture_Management::GetIndexFromHashID(m_xParams.m_axTextureIDs[u]);
			m_pxLightShaft->m_aiTextureIndices[u]	  = static_cast<ASURA_TEXTURE_ID>(iTex);
			m_pxLightShaft->m_afTextureScrollRates[u] = m_xParams.m_afTextureScrollRates[u];
			m_pxLightShaft->m_afTextureUScales[u]	  = m_xParams.m_afTextureUScales[u];
		}

		// Distance fade
		m_pxLightShaft->m_fDistFadeRange = m_xParams.m_fFadeoutDist;
		m_pxLightShaft->m_fDistFadeStart = m_xParams.m_fFadeoutStart;
		m_pxLightShaft->m_fCloseFadeRange = m_xParams.m_fFadeCloseDist;
		m_pxLightShaft->m_fCloseFadeStart = m_xParams.m_fFadeCloseStart;
		
		// Corona
		int iTex = Asura_Texture_Management::GetIndexFromHashID(m_xParams.m_xCoronaTextureID);
		m_pxLightShaft->m_iCoronaTextureIndex	= static_cast<ASURA_TEXTURE_ID>(iTex);
		m_pxLightShaft->m_fCoronaAngleRange		= m_xParams.m_fCoronaAngleRange;
		m_pxLightShaft->m_fCoronaSize			= m_xParams.m_fCoronaSize;
		m_pxLightShaft->m_fCoronaDepth			= m_xParams.m_fCoronaDepth;
		m_pxLightShaft->m_xCoronaColour			= m_xParams.m_xCoronaColour;

		m_pxLightShaft->UpdateBoundingBox();
	}	
}


void Asura_ClientEntity_LightShaft::AttachToLight(void)
{
	m_bAttachedToLight = true;
}

void Asura_ClientEntity_LightShaft::DetachFromLight(void)
{
	m_bAttachedToLight = false;
	SetParams(m_xParams);

	if (m_pxLightShaft)
	{
		m_pxLightShaft->m_bEnabled			= IsEntityActive();
		m_pxLightShaft->m_xInheritedColour  = Asura_RGBA(1.0f,1.0f,1.0f,1.0f);
	}
}


#ifdef ASURA_TOOLS
void Asura_ClientEntity_LightShaft::ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance)
{
	m_xParams.m_xPosition    = pxInstance->TransformPosition(m_xParams.m_xPosition);
	m_xParams.m_xOrientation = pxInstance->TransformMatrix(m_xParams.m_xOrientation);
}
#endif // ASURA_TOOLS




#endif //#ifdef ASURA_TARGET_SM3
