//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_LightShaft.cpp
//
// Copyright (c) 2007 Rebellion (Derby). All rights reserved.
//
// Description:
//
// Volumetric-ish light shafts
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_TARGET_SM3

// This file's header.
#include "Entity/LightShaft/Asura_ServerEntity_LightShaft.h"
#include "Entity/LightShaft/Asura_ClientEntity_LightShaft.h"

// Engine independent.
#include "Camera/Asura_Camera.h"
#include "Core/Asura_TextureManagement.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Message.h"

// Asura tools
#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

Asura_Hashed_List<Asura_ServerEntity_LightShaft> Asura_ServerEntity_LightShaft::s_xList;

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

#ifdef ASURA_TOOLS
Asura_ServerEntity_LightShaft* Asura_ServerEntity_LightShaft::CreateFromTools( Asura_Guid uGuid , const Asura_Vector_3& xCentre)
{
	Asura_ServerEntity_LightShaft* pxNew = new Asura_ServerEntity_LightShaft(uGuid);
	pxNew->m_xParams.m_xOrientation = *Asura_Camera::GetOrientation();
	pxNew->m_xParams.m_xPosition = xCentre;
	return pxNew;
}
#endif

Asura_ServerEntity_LightShaft::Asura_ServerEntity_LightShaft( Asura_Guid uGuid)
: PARENT( uGuid )
, m_xParams()
{
	Classification = AsuraEntityClass_LightShaft;
	SetNeverUpdateClient();

	s_xList.AddToList( this );
}

Asura_ServerEntity_LightShaft::~Asura_ServerEntity_LightShaft( void )
{	
	s_xList.RemoveFromList( this );
}

void Asura_ServerEntity_LightShaft::CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream &xStream )
{
	Asura_ServerEntity_LightShaft* pxNew = new Asura_ServerEntity_LightShaft( uGuid);
	pxNew->ReadFromChunkStream(xStream);
}

void Asura_ServerEntity_LightShaft::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int uVersion;
	xStream >> uVersion;
	Asura_Assert( uVersion <= LIGHTSHAFT_CHUNK_VERSION, "Unhandled Asura_ServerEntity_LightShaft chunk version" );

	if (uVersion >= 3)
	{
		PARENT::ReadFromChunkStream(xStream);
	}

	ExtractParamsFromChunkStream(m_xParams, xStream, uVersion );

	SetNeverUpdates(true);
}

void Asura_ServerEntity_LightShaft::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << LIGHTSHAFT_CHUNK_VERSION;

	PARENT::WriteToChunkStream(xStream);

	WriteParamsToChunkStream(m_xParams, xStream);
}



void Asura_ServerEntity_LightShaft::ExtractParamsFromChunkStream( Asura_LightShaft_Parameters &xParams , Asura_Chunk_Stream &xStream, u_int uVersion )
{
	Asura_LightShaft_Parameters *pxParams = &xParams;

	// Pos/Orient
	xStream >> pxParams->m_xPosition;
	xStream >> pxParams->m_xOrientation;
	xStream >> pxParams->m_uFlags;
	
	// Cone geometry
	for(u_int u = 0; u < 2; ++u)
	{	
		xStream >> pxParams->m_afRadii[u];
		xStream >> pxParams->m_afDistances[u];
		xStream >> pxParams->m_axColours[u];
	}

	// Texturing
	for(u_int u = 0; u < 2; ++u)
	{	
		xStream >> pxParams->m_axTextureIDs[u];
		xStream >> pxParams->m_axTextureNames[u];
		xStream >> pxParams->m_afTextureUScales[u];
		xStream >> pxParams->m_afTextureScrollRates[u];
	}

	if (uVersion >= 1)
	{
		xStream >> pxParams->m_fFadeoutStart;
		xStream >> pxParams->m_fFadeoutDist;
	}
	if (uVersion >= 4)
	{
		xStream >> pxParams->m_fFadeCloseStart;
		xStream >> pxParams->m_fFadeCloseDist;
	}

	if (uVersion >= 2)
	{
		xStream >> pxParams->m_xCoronaTextureID;
		xStream >> pxParams->m_xCoronaTextureName;
		xStream >> pxParams->m_fCoronaSize;
		xStream >> pxParams->m_fCoronaAngleRange;
		xStream >> pxParams->m_fCoronaDepth;
		xStream >> pxParams->m_xCoronaColour;
	}
}

void Asura_ServerEntity_LightShaft::WriteParamsToChunkStream( const Asura_LightShaft_Parameters &xParams , Asura_Chunk_Stream& xStream)
{
	// Pos/Orient
	xStream << xParams.m_xPosition;
	xStream << xParams.m_xOrientation;
	xStream << xParams.m_uFlags;

	// Cone geometry
	for(u_int u = 0; u < 2; ++u)
	{	
		xStream << xParams.m_afRadii[u];
		xStream << xParams.m_afDistances[u];
		xStream << xParams.m_axColours[u];
	}

	// Texturing
	for(u_int u = 0; u < 2; ++u)
	{	
		xStream << xParams.m_axTextureIDs[u];
		xStream << xParams.m_axTextureNames[u];
		xStream << xParams.m_afTextureUScales[u];
		xStream << xParams.m_afTextureScrollRates[u];
	}

	// Distance fade
	xStream << xParams.m_fFadeoutStart;
	xStream << xParams.m_fFadeoutDist;

	xStream << xParams.m_fFadeCloseStart;
	xStream << xParams.m_fFadeCloseDist;

	// Corona
	xStream << xParams.m_xCoronaTextureID;
	xStream << xParams.m_xCoronaTextureName;
	xStream << xParams.m_fCoronaSize;
	xStream << xParams.m_fCoronaAngleRange;
	xStream << xParams.m_fCoronaDepth;
	xStream << xParams.m_xCoronaColour;
}


#ifdef ASURA_TOOLS
void Asura_ServerEntity_LightShaft::SetParams( const Asura_LightShaft_Parameters& xNewParams )
{
    m_xParams = xNewParams;

	Asura_ClientEntity_LightShaft *pxClient = static_cast<Asura_ClientEntity_LightShaft*>(Asura_ClientEntity_LightShaft::FindEntity(GetGuid()));
	if (pxClient)
	{
		pxClient->SetParams( m_xParams );		
		//NY - inelegant but effective way of keeping the omnipresent client entity up to date...
		if (IsEntityActive())
		{
			Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_ACTIVATE );
			if( pxMessage )
			{
				pxMessage->UseGuaranteedDelivery();
				Send( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES );
			}
		}
		else
		{
			Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_DEACTIVATE );
			if( pxMessage )
			{
				pxMessage->UseGuaranteedDelivery();
				Send( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES );
			}
		}
	}
}
#endif

#ifdef ASURA_TOOLS
void Asura_ServerEntity_LightShaft::ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance)
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance);

	m_xParams.m_xPosition    = pxInstance->TransformPosition(m_xParams.m_xPosition);
	m_xParams.m_xOrientation = pxInstance->TransformMatrix(m_xParams.m_xOrientation);

}

void Asura_ServerEntity_LightShaft::TraverseUsedTextures( void (*pfnCallback)( int iTextureID,
	void* pUserData ), void* pUserData ) const
{
	PARENT::TraverseUsedTextures( pfnCallback, pUserData );

	u_int uNumTextures = sizeof( m_xParams.m_axTextureIDs ) / sizeof( m_xParams.m_axTextureIDs[0] );

	for( u_int u = 0; u < uNumTextures; ++u )
	{
		int iTexID = Asura_Texture_Management::GetIndexFromHashID( m_xParams.m_axTextureIDs[u] );
		pfnCallback( iTexID, pUserData );
	}

	int iTexID = Asura_Texture_Management::GetIndexFromHashID( m_xParams.m_xCoronaTextureID );
	pfnCallback( iTexID, pUserData );
}
#endif // ASURA_TOOLS


#endif //#ifdef ASURA_TARGET_SM3

