//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GuardZone.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// This file's header.
#include "Entity/Locations/GuardZone/Asura_ServerNode_GuardZone.h"

//Asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_KDTree.h"
#include "Core/Asura_OrientedVolume.h"
#include "Core/Memory/Asura_Mem.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Locations/OrientedNode/Asura_ServerNode_OrientedNode.h"
#include "Network/Asura_Message.h"

// Tools
#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GuardZone
//
//////////////////////////////////////////////////////////////////////////////
Asura_ServerNode_GuardZone::Asura_ServerNode_GuardZone( Asura_Guid uGuid )
: PARENT( uGuid )
, m_iNumGuardPoints( 0 )
, m_puGuardPointGuids( NULL )
, m_fRadius( 5.0f )
, m_bUseAdvancedVolumes( false )
, m_iNumVolumes( 0 )
, m_pxVolumes( NULL )
{
	Classification = AsuraEntityClass_GuardZone;
}

Asura_ServerNode_GuardZone::~Asura_ServerNode_GuardZone()
{
	delete [] m_puGuardPointGuids;
	delete [] m_pxVolumes;
}

void Asura_ServerNode_GuardZone::CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream )
{
	Asura_ServerNode_GuardZone* const pxGuardZone = new Asura_ServerNode_GuardZone( uGuid );
	if ( pxGuardZone )
	{
		pxGuardZone->ReadFromChunkStream( xStream );
	}
}

void Asura_ServerNode_GuardZone::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;

	// Parent.
	PARENT::ReadFromChunkStream( xStream );

	// Us.
	if ( iVersion == CURRENT_CHUNK_VERSION )
    {
        xStream >> m_iNumGuardPoints;

        if ( m_iNumGuardPoints > 0 )
        {
            m_puGuardPointGuids = new Asura_Guid[ m_iNumGuardPoints ];
            for ( int i = 0; i < m_iNumGuardPoints; ++i )
            {
                xStream >> m_puGuardPointGuids[ i ];
            }
        }

        xStream >> m_fRadius;

        {
        u_int uUseAdvancedVolumes;
        xStream >> uUseAdvancedVolumes;
        m_bUseAdvancedVolumes = ( uUseAdvancedVolumes != 0 ) ? true : false;
        }
        xStream >> m_iNumVolumes;

        if ( m_iNumVolumes > 0 )
        {
            //allocate volume array , and extract those
            m_pxVolumes = new Asura_OrientedVolume[ m_iNumVolumes ];
            for( int i = 0; i < m_iNumVolumes; ++i )
            {
                xStream >> m_pxVolumes[ i ].m_xBox;
                xStream >> m_pxVolumes[ i ].m_xOrient;
            }
        }
    }
    else if ( iVersion == 1 )
    {
        xStream >> m_iNumGuardPoints;

        if ( m_iNumGuardPoints > 0 )
        {
            m_puGuardPointGuids = new Asura_Guid[ m_iNumGuardPoints ];
            for ( int i = 0; i < m_iNumGuardPoints; ++i )
            {
                xStream >> m_puGuardPointGuids[ i ];
            }
        }

        xStream >> m_fRadius;

        xStream >> m_bUseAdvancedVolumes;
        xStream >> m_iNumVolumes;

        if ( m_iNumVolumes > 0 )
        {
            //allocate volume array , and extract those
            m_pxVolumes = new Asura_OrientedVolume[ m_iNumVolumes ];
            for( int i = 0; i < m_iNumVolumes; ++i )
            {
                xStream >> m_pxVolumes[ i ].m_xBox;
                xStream >> m_pxVolumes[ i ].m_xOrient;
            }
        }
    }
	else if ( iVersion == 0 )
	{
		xStream >> m_iNumGuardPoints;

		if ( m_iNumGuardPoints > 0 )
		{
			m_puGuardPointGuids = new Asura_Guid[ m_iNumGuardPoints ];
			for ( int i = 0; i < m_iNumGuardPoints; ++i )
			{
				xStream >> m_puGuardPointGuids[ i ];
			}
		}

		xStream >> m_fRadius;
	}
	else
	{
		Asura_Assertf( false, ( "Asura_ServerNode_GuardZone::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

void Asura_ServerNode_GuardZone::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	// (TI - 5.9.2004) - DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.

	// Version
	xStream << static_cast< int >( CURRENT_CHUNK_VERSION );

	// Parent
	PARENT::WriteToChunkStream(xStream);

	// Our data
	xStream << m_iNumGuardPoints;

	for ( int i = 0; i < m_iNumGuardPoints; ++i )
	{
		xStream << m_puGuardPointGuids[ i ];
	}

	xStream << m_fRadius;

    xStream << static_cast< u_int >( m_bUseAdvancedVolumes ? 1 : 0 );
	int iNumOfVolumesToWriteOut = m_bUseAdvancedVolumes ? m_iNumVolumes : 0;
	xStream << iNumOfVolumesToWriteOut;

	for ( int i = 0; i < iNumOfVolumesToWriteOut; ++i )
	{
		xStream << m_pxVolumes[ i ].m_xBox;
		xStream << m_pxVolumes[ i ].m_xOrient;
	}

	// (TI - 5.9.2004) - DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.
}

Asura_ServerNode_OrientedNode* Asura_ServerNode_GuardZone::GetRandomUnoccupiedGuardPoint() const
{
	Asura_ServerNode_OrientedNode** apxOrientedNodes = new Asura_ServerNode_OrientedNode*[ m_iNumGuardPoints ];
	if ( !apxOrientedNodes ) { return NULL; }
	Asura_Mem::Set( apxOrientedNodes, 0, m_iNumGuardPoints * sizeof( Asura_ServerNode_OrientedNode* ) );

	int iNumUnoccupiedGPs = 0;
	for ( int i = 0; i < m_iNumGuardPoints; ++i )
	{
		Asura_ServerNode_OrientedNode* pxGuardPoint = static_cast< Asura_ServerNode_OrientedNode* >( FindEntity( m_puGuardPointGuids[ i ] ) );
		if ( pxGuardPoint && pxGuardPoint->IsAvailable() )
		{
			apxOrientedNodes[ i ] = pxGuardPoint;
			++iNumUnoccupiedGPs;
		}
	}
	if ( iNumUnoccupiedGPs == 0 )
	{
		delete [] apxOrientedNodes;
		return NULL;
	}

	int iRandomUnoccupiedGP = Asura_Random::GetInt( 0, iNumUnoccupiedGPs - 1 );
	for ( int i = 0; i < m_iNumGuardPoints; ++i )
	{
		if ( apxOrientedNodes[ i ] && iRandomUnoccupiedGP-- == 0 )
		{
			Asura_ServerNode_OrientedNode* pxGuardPoint = apxOrientedNodes[ i ];
			delete [] apxOrientedNodes;
			return pxGuardPoint;
		}
	}

	Asura_Assert( false, "Asura_ServerNode_GuardZone::GetRandomUnoccupiedGuardPoint - algorithm error" );
	delete [] apxOrientedNodes;
	return NULL;
}

bool Asura_ServerNode_GuardZone::IsBaseOrDerivativeOf( u_short usClassification ) const
{
	if ( usClassification == AsuraEntityClass_GuardZone )
	{
		return true;
	}

	return PARENT::IsBaseOrDerivativeOf(usClassification);
}

#ifdef ASURA_TOOLS
void Asura_ServerNode_GuardZone::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance(pxInstance);

	if ( m_puGuardPointGuids )
	{
		for ( int i = 0; i < m_iNumGuardPoints; ++i )
		{
			m_puGuardPointGuids[ i ] = pxInstance->ConvertGuid( m_puGuardPointGuids[ i ] );
		}
	}

	for( int i = 0; i < m_iNumVolumes; ++i )
	{
		const Asura_Vector_3 xOldCentre = m_pxVolumes[ i ].m_xBox.CalculateCentre();
		const Asura_Vector_3 xNewCentre = pxInstance->TransformPosition( xOldCentre );
		m_pxVolumes[ i ].m_xBox.Displace( xNewCentre-xOldCentre );

		m_pxVolumes[ i ].m_xOrient = pxInstance->TransformQuat( m_pxVolumes[ i ].m_xOrient );
	}
}

bool Asura_ServerNode_GuardZone::CanBeIncludedInTemplate() const 
{
	return ( Classification == AsuraEntityClass_GuardZone );
}

void Asura_ServerNode_GuardZone::AddGuardPoint( Asura_Guid uGuid )
{
	Asura_Guid* const puOldGuidArray = new Asura_Guid[ m_iNumGuardPoints ];
	for ( int i = 0; i < m_iNumGuardPoints; ++i )
	{
		puOldGuidArray[ i ] = m_puGuardPointGuids[ i ];
	}
	++m_iNumGuardPoints;
	delete [] m_puGuardPointGuids;
	m_puGuardPointGuids = new Asura_Guid[ m_iNumGuardPoints ];
	for ( int i = 0; i < m_iNumGuardPoints - 1; ++i )
	{
		m_puGuardPointGuids[ i ] = puOldGuidArray[ i ];
	}
	m_puGuardPointGuids[ m_iNumGuardPoints - 1 ] = uGuid;
	delete [] puOldGuidArray;
}

bool Asura_ServerNode_GuardZone::RemoveGuardPoint( Asura_Guid uGuid )
{
	Asura_Guid* const puNewGuidArray = new Asura_Guid[ uGuid ];
	int iCurrentIndex = 0;
	bool bFoundGuid = false;
	for ( int i = 0; i < m_iNumGuardPoints; ++i )
	{
		if ( uGuid != m_puGuardPointGuids[ i ] )
		{
			puNewGuidArray[ iCurrentIndex ] = m_puGuardPointGuids[ i ];
			++iCurrentIndex;
		}
		else
		{
			bFoundGuid = true;
		}
	}

	if ( bFoundGuid )
	{
		delete [] m_puGuardPointGuids;
		--m_iNumGuardPoints;
		m_puGuardPointGuids = new Asura_Guid[ m_iNumGuardPoints ];
		for ( int i = 0; i < m_iNumGuardPoints; ++i )
		{
			m_puGuardPointGuids[ i ] = puNewGuidArray[ i ];
		}
	}

	delete [] puNewGuidArray;

	return bFoundGuid;
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
//
//////////////////////////////////////////////////////////////////////////////
#endif //ASURA_USE_AXON
