#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_Coverpoint.h
//
// Created by NJD - 12 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_SERVERNODE_COVERPOINT_H__
#define __ASURA_SERVERNODE_COVERPOINT_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Entity/Locations/OrientedNode/Asura_ServerNode_OrientedNode.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Bounding_Box;
class Asura_Collision_Box;
class Tools_EntityTemplateInstance;

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_Coverpoint
//
// Coverpoints are validated on a quit to game or during an export.
// Only valid coverpoints are added to the coverpoint list.
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerNode_Coverpoint : public Asura_ServerNode_OrientedNode
{
	friend class Tools_EntityNode_CoverpointNode;
    typedef Asura_ServerNode_OrientedNode PARENT;

public:
    Asura_ServerNode_Coverpoint(Asura_Guid uGuid);
    virtual ~Asura_ServerNode_Coverpoint();

	static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream );
    virtual	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

    u_int					GetEmergePointCount()										const { return m_uEmergepointCount;	}
	const Asura_Vector_3&	GetEmergePointPosition( unsigned int uEmergepointIndex )	const;
	const Asura_Euler_3&	GetEmergePointOrientation( unsigned int uEmergepointIndex )	const;

    bool         GetIsLowCover()       const { return m_bIsLowCover; }
    u_short 	 GetNearestWaypoint()  const { return m_usNearestWaypoint; }
	float		 GetCoverAngle()	   const { return m_fCoverAngle; }

    //const Axon_Node* GetEmergePoint( const int iPointIndex ) const;
    //const Axon_Node* GetNearestEmergePoint( const Asura_Vector_3& xPosition ) const;
	virtual bool ProvidesCoverFrom( const Asura_Vector_3& xPosition ) const;
	bool		 ProvidesCoverFrom( int iThreatCount, const Asura_Vector_3* axThreatPositions ) const;

	// ARC - 10 February 2009 12:34:51 - Removed SetEmergePointCount() as it wasn't implemented and is probably dangerous - should have AddEmergePoint() and RemoveEmergePoint() if needed in future
    //void         SetEmergePointCount( const int iNumPoints );
    void         SetEmergePointPosition( unsigned int uEmergepointIndex, const Asura_Vector_3& xPosition );
    void         SetIsLowCover( const bool bIsLowCover )   { m_bIsLowCover = bIsLowCover; }
    void		 SetNearestWaypoint( u_short usWP )        { m_usNearestWaypoint = usWP; }

	virtual bool IsBaseOrDerivativeOf(u_short usClassification) const;

#ifdef ASURA_TOOLS
    virtual bool CanBeIncludedInTemplate() const;
    virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );
#endif

    static Asura_Hashed_List< Asura_ServerNode_Coverpoint >* GetList() { return &s_xValidCoverpointList; }
	static Asura_ServerNode_Coverpoint* FindEntity( Asura_Guid uGuid ) { return s_xValidCoverpointList.FindEntity(uGuid); }

    // This must be added once projects start using coverpoints in earnest.
    //static void BuildSpaciallySortedArray();

protected:
#ifdef ASURA_TOOLS
	virtual void CheckForValidity();
#endif

protected:
    float m_fCosHalfFOV;
    float m_fCoverAngle;
    float m_fMinCoverRange;
	float m_fMaxCoverRange;

	union
	{
		u_int m_uCoverpointDataBitfield;
		struct
		{
			u_int m_bIsLowCover            : 1,
                  m_uEmergepointCount      : 2,
				  SpareBits_CoverpointNode : 13,
                  m_usNearestWaypoint      : 16;
		};
	};

	struct EmergePoint
	{
		Asura_Vector_3 m_xPosition;
		Asura_Euler_3  m_xOrientation;
	};

    // If we want more than two emerge points per coverpoint we'll have to consider a scheme like the one in Rogue,
    // with a single emerge point array and pointers into it in each coverpoint.
	enum { MAX_NUM_EMERGE_POINTS = 2 };
    EmergePoint m_axEmergePoint[ MAX_NUM_EMERGE_POINTS ];
    union
    {
        u_char m_ucEmergePointFlags;
        struct 
        {
            u_int m_bExists             : 1,
                  m_bIsValid            : 1,
                  SpareBits_EmergePoint : 6;
        };
    } m_aucEmergePointFlags[ MAX_NUM_EMERGE_POINTS ];

    static u_int                                            s_uNumValidCoverpoints;

private:
    static Asura_Hashed_List< Asura_ServerNode_Coverpoint > s_xValidCoverpointList;

private:
    enum { CURRENT_CHUNK_VERSION = 7 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_SERVERNODE_COVERPOINT_H__
#endif //ASURA_USE_AXON
