//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GS2Node.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GS2Node.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_Entity_Event.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Network/Asura_Message.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif
#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
#include "Streaming/Asura_Streaming_System.h"
#endif

// axon
#include "Behaviour/Axon_BehaviourParams.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"
#include "Core/Axon_Messages.h"

// tools
#ifdef ASURA_TOOLS
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Actor.h"
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GS2Node
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List< Asura_ServerNode_GS2Node > Asura_ServerNode_GS2Node::s_xList;

Asura_ServerNode_GS2Node::Asura_ServerNode_GS2Node( Asura_Guid uGuid )
: PARENT( uGuid )
, m_uFlags( 0 )
, m_pfLinkWeights( NULL )
, m_xOccupierGuids()
, m_iStructureType( 0 )
, m_iBehaviourSet( AXON_BEHAVIOUR_TYPE_UNSET )
, m_pxBehaviourParams( NULL )
{
	Classification = AsuraEntityClass_GS2Node;

	m_bTestForCollisions = true;

	s_xList.AddToList( this );
}

Asura_ServerNode_GS2Node::~Asura_ServerNode_GS2Node()
{
	if ( m_pfLinkWeights )
	{
		Asura_ResourcePushType( ASURA_RESOURCE_GAMESCENE );
		Asura_ResourceDeallocateArray( m_pfLinkWeights );
		Asura_ResourcePopType();
		m_pfLinkWeights = NULL;
	}

	delete m_pxBehaviourParams;
	m_pxBehaviourParams = NULL;

	m_xOccupierGuids.Clear();

	s_xList.RemoveFromList( this );
}

bool Asura_ServerNode_GS2Node::HandleMessage( const Asura_Message* pxMessage )
{
	bool bHandled = false;

	if ( m_pxBehaviourParams )
	{
		bHandled = m_pxBehaviourParams->DoHandleMessage( pxMessage );
	}

	switch ( pxMessage->GetMsgID() )
    {
		case ASURA_MSG_AXON_GS2_TAKE_LINK:
		case ASURA_MSG_AXON_GS2_ANIM_CHANGE:
		case ASURA_MSG_AXON_GS2_LEAVE_NODE:
		case ASURA_MSG_AXON_GS2_DONT_STOP_AT_NODE:
		case ASURA_MSG_AXON_GS2_TELEPORT_TO_NODE:
		case ASURA_MSG_AXON_GS2_PATHFIND_TO_NODE:
		//case ASURA_MSG_AXON_GS2_SET_NODE_PARAMS:
        {
			// Forward these messages on to our current users.
			for ( Asura_LinkList_ConstIt< Asura_Guid > xIt( m_xOccupierGuids ); !xIt.Done(); xIt.Next() )
			{
				Asura_Network::Forward( pxMessage, *xIt.GetCurrent(), GetGuid() );
			}

			bHandled = true;
			break;
        }

		case ASURA_MSG_AXON_GS2_EXIT_GAMESCENE:
		{
			// Special case. Take a copy of the list before forwarding the message on, because it may cause occupiers to be removed.
			Asura_LinkList< Asura_Guid > xOccupierCopy( m_xOccupierGuids );
			for ( Asura_LinkList_ConstIt< Asura_Guid > xIt( xOccupierCopy ); !xIt.Done(); xIt.Next() )
			{
				Asura_Network::Forward( pxMessage, *xIt.GetCurrent(), GetGuid() );
			}

			bHandled = true;
			break;
		}

		case ASURA_MSG_AXON_GS2_SET_LINK_WEIGHT:
		{
			const Axon_Msg_GS2_Set_Link_Weight* pxSLWMsg = static_cast< const Axon_Msg_GS2_Set_Link_Weight* >( pxMessage->GetData() );

			const Axon_Msg_GS2_Set_Link_Weight::BoolGuidWeight* const pxBGWArray = pxSLWMsg->GetBGWArray();
			for ( u_int u = 0; u < pxSLWMsg->m_uNumEntries; ++u )
			{
				if ( !pxBGWArray[ u ].m_bInUse ) { continue; }

				for ( u_int v = 0; v < GetNumberLinks(); ++v )
				{
					if ( pxBGWArray[ u ].m_uLinkGuid == GetLink( v ) )
					{
						m_pfLinkWeights[ v ] = pxBGWArray[ u ].m_fWeight;
						break;
					}
				}
			}

			m_bUseLinkWeights = true;

			bHandled = true;
			break;
		}

        default:
		{
			break;
		}
    }

	return bHandled ? true : PARENT::HandleMessage( pxMessage );
}

void Asura_ServerNode_GS2Node::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	Asura_ResourcePushType( ASURA_RESOURCE_GAMESCENE );

	u_int uVersion;
    xStream >> uVersion;

    // Parent.
    PARENT::ReadFromChunkStream( xStream );

	// Us.
    if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uFlags );
		xStream >> xFlagsBitfield;

		u_int uNumOccupiers;
		xStream >> uNumOccupiers;
		for ( u_int u = 0; u < uNumOccupiers; ++u )
		{
			Asura_Guid uOccupierGuid;
			xStream >> uOccupierGuid;
#if defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
			uOccupierGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( uOccupierGuid );
#endif
			m_xOccupierGuids.Push_Back( uOccupierGuid );
		}

		xStream >> m_iStructureType;

		xStream >> m_iBehaviourSet;
		Axon_BehaviourParams::CreateTreeFromChunkStream( xStream, m_pxBehaviourParams );

		if ( GetNumberLinks() )
		{
			Asura_ResourceAllocateArray( m_pfLinkWeights, GetNumberLinks() );
			Asura_ReadFromStream_Array_Wrapper< float > xWeightArray( GetNumberLinks(), m_pfLinkWeights );
			xStream >> xWeightArray;
		}
	}
	else if ( uVersion == 3 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uFlags );
		xStream >> xFlagsBitfield;
		m_bIgnoreCollisions = false;

		u_int uNumOccupiers;
		xStream >> uNumOccupiers;
		for ( u_int u = 0; u < uNumOccupiers; ++u )
		{
			Asura_Guid uOccupierGuid;
			xStream >> uOccupierGuid;
#if defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
			uOccupierGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( uOccupierGuid );
#endif
			m_xOccupierGuids.Push_Back( uOccupierGuid );
		}

		xStream >> m_iStructureType;

		xStream >> m_iBehaviourSet;
		Axon_BehaviourParams::CreateTreeFromChunkStream( xStream, m_pxBehaviourParams );

		if ( GetNumberLinks() )
		{
			Asura_ResourceAllocateArray( m_pfLinkWeights, GetNumberLinks() );
			Asura_ReadFromStream_Array_Wrapper< float > xWeightArray( GetNumberLinks(), m_pfLinkWeights );
			xStream >> xWeightArray;
		}
	}
	else if ( uVersion == 2 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uFlags );
		xStream >> xFlagsBitfield;
		m_bIgnoreCollisions = false;

		u_int uNumOccupiers;
		xStream >> uNumOccupiers;
		for ( u_int u = 0; u < uNumOccupiers; ++u )
		{
			Asura_Guid uOccupierGuid;
			xStream >> uOccupierGuid;
#if defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
			uOccupierGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( uOccupierGuid );
#endif
			m_xOccupierGuids.Push_Back( uOccupierGuid );
		}

		// Convert the blocks we used to store into events.
		u_short usMessageBlock;
		xStream >> usMessageBlock; // m_usNodeArrivalMessageBlock
		SetMessageBlockForEvent( ASURA_ENTITY_EVENT_ONARRIVEDAT, usMessageBlock );
		xStream >> usMessageBlock; // m_usNodeDepartureMessageBlock
		SetMessageBlockForEvent( ASURA_ENTITY_EVENT_ONDEPARTEDFROM, usMessageBlock );

		xStream >> m_iStructureType;

		xStream >> m_iBehaviourSet;
		Axon_BehaviourParams::CreateTreeFromChunkStream( xStream, m_pxBehaviourParams );

		if ( GetNumberLinks() )
		{
			Asura_ResourceAllocateArray( m_pfLinkWeights, GetNumberLinks() );
			Asura_ReadFromStream_Array_Wrapper< float > xWeightArray( GetNumberLinks(), m_pfLinkWeights );
			xStream >> xWeightArray;
		}
	}
#ifdef ASURA_TOOLS
	else if ( uVersion == 1 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uFlags );
		xStream >> xFlagsBitfield;
		m_uPendingCount = 0;
		m_bIgnoreCollisions = false;

		u_int uNumOccupiers;
		xStream >> uNumOccupiers;
		for ( u_int u = 0; u < uNumOccupiers; ++u )
		{
			Asura_Guid uOccupierGuid;
			xStream >> uOccupierGuid;
#if defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
			uOccupierGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( uOccupierGuid );
#endif
			m_xOccupierGuids.Push_Back( uOccupierGuid );
		}

		// Convert the blocks we used to store into events.
		u_short usMessageBlock;
		xStream >> usMessageBlock; // m_usNodeArrivalMessageBlock
		SetMessageBlockForEvent( ASURA_ENTITY_EVENT_ONARRIVEDAT, usMessageBlock );
		xStream >> usMessageBlock; // m_usNodeDepartureMessageBlock
		SetMessageBlockForEvent( ASURA_ENTITY_EVENT_ONDEPARTEDFROM, usMessageBlock );

		xStream >> m_iStructureType;

		xStream >> m_iBehaviourSet;
		Axon_BehaviourParams::CreateTreeFromChunkStream( xStream, m_pxBehaviourParams );

		if ( GetNumberLinks() )
		{
			Asura_ResourceAllocateArray( m_pfLinkWeights, GetNumberLinks() );
			Asura_ReadFromStream_Array_Wrapper< float > xWeightArray( GetNumberLinks(), m_pfLinkWeights );
			xStream >> xWeightArray;
		}
	}
	else if ( uVersion == 0 )
    {
        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uFlags );
        xStream >> xFlagsBitfield;
		m_uPendingCount = 0;
		m_bIgnoreCollisions = false;

		u_int uNumOccupiers;
		xStream >> uNumOccupiers;
		for ( u_int u = 0; u < uNumOccupiers; ++u )
		{
			Asura_Guid uOccupierGuid;
			xStream >> uOccupierGuid;
#if defined( ASURA_USE_STREAMING ) && !defined( ASURA_TOOLS )
			uOccupierGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( uOccupierGuid );
#endif
			m_xOccupierGuids.Push_Back( uOccupierGuid );
		}

		// Convert the blocks we used to store into events.
		u_short usMessageBlock;
		xStream >> usMessageBlock; // m_usNodeArrivalMessageBlock
		SetMessageBlockForEvent( ASURA_ENTITY_EVENT_ONARRIVEDAT, usMessageBlock );
		xStream >> usMessageBlock; // m_usNodeDepartureMessageBlock
		SetMessageBlockForEvent( ASURA_ENTITY_EVENT_ONDEPARTEDFROM, usMessageBlock );

		xStream >> m_iStructureType;

		xStream >> m_iBehaviourSet;
		Axon_BehaviourParams::CreateTreeFromChunkStream( xStream, m_pxBehaviourParams );

		if ( GetNumberLinks() )
		{
			// Create the link weights and set them to the default as we've no data to read.
			Asura_ResourceAllocateArray( m_pfLinkWeights, GetNumberLinks() );
			for ( u_int u = 0; u < GetNumberLinks(); ++u )
			{
				m_pfLinkWeights[ u ] = 1.0f;
			}
		}
		m_bUseLinkWeights = false;
	}
#endif // #ifdef ASURA_TOOLS
    else
    {
        Asura_Assertf( false, ( "Asura_ServerNode_GS2Node::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
    }

	Asura_ResourcePopType();
}

void Asura_ServerNode_GS2Node::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	// DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.

	// Version
	xStream << uCURRENT_CHUNK_VERSION;

	// Parent
	PARENT::WriteToChunkStream( xStream );

	// Our data
	Asura_WriteToStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uFlags );
    xFlagsBitfield.Add( m_bTestForCollisions,   1 );
    xFlagsBitfield.Add( m_bFollowSurface,	    1 );
    xFlagsBitfield.Add( m_bFollowSurfaceTools,  1 );
	xFlagsBitfield.Add( m_bUseLinkWeights,		1 );
	xFlagsBitfield.Add( m_uPendingCount,		8 );
	xFlagsBitfield.Add( m_bIgnoreCollisions,    1 );
	xStream << xFlagsBitfield;

	const u_int uNumOccupiers = m_xOccupierGuids.Size();
	xStream << uNumOccupiers;
	for ( Asura_LinkList_ConstIt< Asura_Guid > xIt( m_xOccupierGuids ); !xIt.Done(); xIt.Next() )
	{
		xStream << *xIt.GetCurrent();
	}

	xStream << m_iStructureType;

	xStream << m_iBehaviourSet;
	Axon_BehaviourParams::WriteTreeToChunkStream( m_pxBehaviourParams, xStream );

	Asura_WriteToStream_Array_Wrapper< float > xWeightArray( GetNumberLinks(), m_pfLinkWeights );
	xStream << xWeightArray;

	// DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.
}

bool Asura_ServerNode_GS2Node::IsBaseOrDerivativeOf( u_short usClassification ) const
{
	if ( usClassification == AsuraEntityClass_GS2Node )
	{
		return true;
	}
	return PARENT::IsBaseOrDerivativeOf( usClassification );
}

void Asura_ServerNode_GS2Node::EnterNode( const Asura_Guid uActorGuid )
{
	// Check we're not already here.
	Asura_Assert( !m_xOccupierGuids.Find( uActorGuid ), "Asura_ServerNode_GS2Spline::EnterSpline - guid already registered" );

	SetIsOccupied( true );
	m_xOccupierGuids.Push_Front( uActorGuid );

	#ifdef ASURA_TOOLS
	//## 2009/05/05 This can be removed once all levels have been re-saved.
	if ( !Tools_System::IsActive() )
	#endif
	{
		SendMessagesForEvent( ASURA_ENTITY_EVENT_ONARRIVEDAT );
	}
}

void Asura_ServerNode_GS2Node::ExitNode( const Asura_Guid uActorGuid )
{
	// Check we're here.
	Asura_Assert( m_xOccupierGuids.Find( uActorGuid ), "Asura_ServerNode_GS2Spline::ExitSpline - guid not registered" );

	#ifdef ASURA_TOOLS
	//## 2009/05/05 This can be removed once all levels have been re-saved.
	if ( !Tools_System::IsActive() )
	#endif
	{
		SendMessagesForEvent( ASURA_ENTITY_EVENT_ONDEPARTEDFROM );
	}

	m_xOccupierGuids.Remove( uActorGuid );
	SetIsOccupied( m_xOccupierGuids.Size() > 0 );
}

#ifndef ASURA_NO_DEBUG_OUTPUT
void Asura_ServerNode_GS2Node::DoRenderDebugInfo()
{
	Asura_Hashed_List<Asura_ServerNode_GS2Node>* pxGS2NodeList = Asura_ServerNode_GS2Node::GetList();
	Asura_Hashed_List_It<Asura_ServerNode_GS2Node> xIt(pxGS2NodeList);

	while ( !xIt.Done() )
	{
		Asura_ServerNode_GS2Node* pxGS2Node = xIt.GetCurrent();
		xIt.Next();

		if ( pxGS2Node )
		{
			pxGS2Node->RenderDebugInfo();
		}
	}
}

void Asura_ServerNode_GS2Node::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	if ( !DebugTextPosOnScreen() ) { return; }

	if ( IsBaseOrDerivativeOf( AsuraEntityClass_GS2Attractor ) )
	{
		Asura_Font_System::Print( GetDebugTextPos().x, GetDebugTextPos().y, 0.5f, 0xffffffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
			"GS2 Attractor: %u", GetGuid() );
	}
	else
	{
		Asura_Font_System::Print( GetDebugTextPos().x, GetDebugTextPos().y, 0.5f, 0xffffffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
			"GS2 Node: %u", GetGuid() );
	}
	IncrementDebugTextPos();
}
#endif

void Asura_ServerNode_GS2Node::Create( const Asura_Guid uGuid, Asura_Chunk_Stream& xStream )
{
	Asura_ServerNode_GS2Node* pxNode = new Asura_ServerNode_GS2Node( uGuid );
	pxNode->ReadFromChunkStream( xStream );
}

#ifdef ASURA_TOOLS
bool Asura_ServerNode_GS2Node::CanBeIncludedInTemplate() const
{
	return ( Classification == AsuraEntityClass_GS2Node );
}

void Asura_ServerNode_GS2Node::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance );

	// Current actor guids should all currently be ASURA_GUID_UNREGISTERED.

	if ( m_pxBehaviourParams )
	{
		m_pxBehaviourParams->DoApplyChangesForTemplateInstance( pxInstance );
	}
}

void Asura_ServerNode_GS2Node::UpdateFollowSurface()
{
	if ( !m_bFollowSurfaceTools )
	{
		m_bFollowSurface = false;
	}
	else
	{
		Asura_Matrix_3x3 xOrient;
		GetOrientation( xOrient );
		const Asura_Vector_3 xUp = Asura_Vector_3( 0.0f, -1.0f, 0.0f ) * xOrient;

		Asura_Quat xQOrient;
		GetOrientation( xQOrient );
		Asura_Vector_3 xTest( 0.0f, -1.0f, 0.0f );
		xTest = xQOrient.RotateVector( xTest);
		Asura_Assert( xUp.ApproximatelyEquals( xTest ), "" );

		float fDistanceToIntersection;
		if ( Axon_Behaviour_GS2Actor::FollowSurfaceTest( GetPosition(), -xUp, fDistanceToIntersection ) )
		{
			if ( Asura_Maths::Abs( fDistanceToIntersection ) > 0.02f )
			{
				// (AW - 12.04.2005) - Follow ground makes a difference, don't disable.
				m_bFollowSurface = true;
				return;
			}
		}
		// (AW - 12.04.2005) - Follow ground makes no difference.
		m_bFollowSurface = false;
	}
}

void Asura_ServerNode_GS2Node::SetBehaviourSet( int iBehaviourSet )
{
	m_iBehaviourSet = iBehaviourSet;

	if ( m_pxBehaviourParams )
	{
		delete m_pxBehaviourParams;
		m_pxBehaviourParams = NULL;
	}

	// Create the correct behaviour parameters for the behaviour type.
	Axon_BehaviourParams::CreateTree( m_iBehaviourSet, m_pxBehaviourParams );
}

bool Asura_ServerNode_GS2Node::UpdateBehaviourTree()
{
	Axon_BehaviourParams* pxBehaviourParams = NULL;
	if ( Axon_BehaviourParams::ConvertTree( m_iBehaviourSet, m_pxBehaviourParams, pxBehaviourParams ) )
	{
		delete m_pxBehaviourParams;
		m_pxBehaviourParams = pxBehaviourParams;
		return true;
	}

	return false;
}

int Asura_ServerNode_GS2Node::GetDefaultMessageEvent()
{
	return ASURA_ENTITY_EVENT_ONARRIVEDAT;
}

void Asura_ServerNode_GS2Node::AddNewSpline( const Asura_Guid uGuid )
{
	const u_int uNumLinksOld = GetNumberLinks();
	PARENT::AddNewSpline( uGuid );
	const u_int uNumLinksNew = GetNumberLinks();

	if ( uNumLinksNew > uNumLinksOld )
	{
		Asura_ResourcePushType(ASURA_RESOURCE_GAMESCENE);
		{
			float* pfNewWeights;
			Asura_ResourceAllocateArray( pfNewWeights, uNumLinksNew );
			Asura_Mem::Copy( pfNewWeights, m_pfLinkWeights, uNumLinksOld * sizeof( float ) );
			pfNewWeights[ uNumLinksNew - 1 ] = 1.0f;
			Asura_ResourceDeallocateArray( m_pfLinkWeights );
			m_pfLinkWeights = pfNewWeights;
		}
		Asura_ResourcePopType();
	}
}

void Asura_ServerNode_GS2Node::RemoveSpline( const Asura_Guid uGuid )
{
	u_int uToRemove = UINT_MAX;
	for ( u_int u = 0; u < GetNumberLinks(); ++u )
	{
		if ( GetLink( u ) == uGuid )
		{
			uToRemove = u;
		}
	}
	if ( uToRemove == UINT_MAX )
	{
		Asura_Assert( false, "Asura_ServerNode_GS2Node::RemoveSpline - could not find spline to remove" );
		return;
	}

	const u_int uNumLinksOld = GetNumberLinks();
	PARENT::RemoveSpline( uGuid );
	const u_int uNumLinksNew = GetNumberLinks();

	if ( uNumLinksNew < uNumLinksOld )
	{
		if ( uNumLinksNew )
		{
			float* pfNewWeights;
			Asura_ResourceAllocateArray( pfNewWeights, uNumLinksNew );
			for ( u_int uReadIndex = 0, uWriteIndex = 0; uReadIndex < uNumLinksOld; ++uReadIndex )
			{
				if ( uReadIndex != uToRemove )
				{
					pfNewWeights[ uWriteIndex++ ] = m_pfLinkWeights[ uReadIndex ];
				}
			}
			Asura_ResourceDeallocateArray( m_pfLinkWeights );
			m_pfLinkWeights = pfNewWeights;

			// If we've only one link left then turn off the weighting stuff.
			if ( uNumLinksNew == 1 )
			{
				m_bUseLinkWeights = false;
				m_pfLinkWeights[ 0 ] = 1.0f;
			}
		}
		else
		{
			Asura_ResourceDeallocateArray( m_pfLinkWeights );
			m_pfLinkWeights = NULL;
		}
	}
}
#endif // ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
