#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GS2Node.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_SERVERNODE_GS2NODE_H__
#define __ASURA_SERVERNODE_GS2NODE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_LinkList.h"
#include "Entity/Locations/OrientedNode/Asura_ServerNode_OrientedNode.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Chunk_Stream;
class Asura_Message;
class Axon_BehaviourParams;
class Tools_EntityTemplateInstance;

//////////////////////////////////////////////////////////////////////////////
//
// Axon_GS2_StructureType
//
//////////////////////////////////////////////////////////////////////////////

enum Axon_GS2_StructureType
{
	AXON_GS2_ST_NONE = 0,

	AXON_GS_ST_FIRST_PROJECT_SPECIFIC = 1000
};

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GS2Node
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerNode_GS2Node : public Asura_ServerNode_OrientedNode
{
	friend class Tools_EntityNode_GS2Node;
	typedef Asura_ServerNode_OrientedNode PARENT;

public:
	Asura_ServerNode_GS2Node( Asura_Guid uGuid );
	virtual ~Asura_ServerNode_GS2Node();

	virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual bool IsBaseOrDerivativeOf( u_short usClassification ) const;

	// Actors increment the pending count when they are on a spline or being attracted to it. They decrement it when they leave the spline/attraction.
	// This allows other actors to tell if the node is going to be occupied.
	void IncPendingCount() { ++m_uPendingCount; }
	void DecPendingCount() { Asura_Assert( m_uPendingCount > 0, "Asura_ServerNode_GS2Node::DecPendingCount - pending count going negative" ); --m_uPendingCount; }
	bool HasPendingActors() const { return ( m_uPendingCount > 0 ); }

	void EnterNode( const Asura_Guid uActorGuid );
	void ExitNode( const Asura_Guid uActorGuid );

	int					  GetStructureType()		const { return m_iStructureType; }
	int					  GetBehaviourSet()			const { return m_iBehaviourSet; }
	Axon_BehaviourParams* GetBehaviourParams()		const { return m_pxBehaviourParams; }
	bool				  ShouldTestForCollisions() const { return m_bTestForCollisions; }
	bool				  ShouldIgnoreCollisions()  const { return m_bIgnoreCollisions; }
	bool ShouldFollowSurface( const bool bIgnoreToolsOptimisation = false ) const
	{
		return ( ( bIgnoreToolsOptimisation && m_bFollowSurfaceTools ) || ( !bIgnoreToolsOptimisation && m_bFollowSurface ) );
	}
	bool ShouldUseLinkWeights() const { return m_bUseLinkWeights; }
	float GetLinkWeight( const u_int uLink ) const
	{
		Asura_Assert( uLink < GetNumberLinks(), "Asura_ServerNode_GS2Node::GetLinkWeight - bad link index" );
		return m_pfLinkWeights[ uLink ];
	}

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
	static void DoRenderDebugInfo();
#endif

	static void Create( const Asura_Guid uGuid, Asura_Chunk_Stream& xStream );
	static Asura_Hashed_List< Asura_ServerNode_GS2Node >* GetList() { return &s_xList; }
	static Asura_ServerNode_GS2Node* FindEntity( const Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }

#ifdef ASURA_TOOLS
	virtual bool CanBeIncludedInTemplate() const;
	virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );
	virtual void UpdateFollowSurface();

	void		 SetStructureType( int iStructureType ) { m_iStructureType = iStructureType; }
	void		 SetBehaviourSet( int iBehaviourSet );
	virtual bool UpdateBehaviourTree();

	virtual int GetDefaultMessageEvent();

	float* GetLinkWeightPointer( const u_int uLink ) const { return &m_pfLinkWeights[ uLink ]; }

	void SetShouldTestForCollisions( const bool bTest )   { m_bTestForCollisions = bTest; }
	void SetShouldIgnoreCollisions( const bool bIgnore )  { m_bIgnoreCollisions = bIgnore; }
    void SetShouldFollowSurface( const bool bFollow )     { m_bFollowSurfaceTools = bFollow; }
	void SetShouldUseLinkWeights( const bool bUse )       { m_bUseLinkWeights = bUse; }

	virtual void AddNewSpline( const Asura_Guid uGuid );
	virtual void RemoveSpline( const Asura_Guid uGuid );
#endif

protected:
	const Asura_LinkList< Asura_Guid >& GetOccupierList() const { return m_xOccupierGuids; }

private:
	union
	{
		u_int m_uFlags;
		struct
		{
			u_int m_bTestForCollisions	  : 1,
				  m_bFollowSurface		  : 1,
				  m_bFollowSurfaceTools	  : 1,
				  m_bUseLinkWeights		  : 1,
				  m_uPendingCount		  : 8,
				  m_bIgnoreCollisions	  : 1;
		};
	};

	enum
	{
		MESSAGE_BLOCK_NODE_ARRIVAL,
		MESSAGE_BLOCK_NODE_DEPARTURE,
		NUM_MESSAGE_BLOCKS,
	};

	float*				  m_pfLinkWeights;
	Asura_LinkList< Asura_Guid > m_xOccupierGuids;
	int					  m_iStructureType;
#if defined ASURA_TOOLS && defined ASURA_USE_GAMESCENE
public:
#endif
    int	                  m_iBehaviourSet;
	Axon_BehaviourParams* m_pxBehaviourParams;

private:
	static const u_int uCURRENT_CHUNK_VERSION = 4;
	static Asura_Hashed_List< Asura_ServerNode_GS2Node > s_xList;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_SERVERNODE_GS2NODE_H__
#endif // ASURA_USE_AXON
