#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GamesceneNode.h
//
// Created by NJD - 12 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_SERVERNODE_GAMESCENENODE_H__
#define __ASURA_SERVERNODE_GAMESCENENODE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_HashedList.h"
#include "Entity/Locations/Asura_Gamescene_Node_Base.h"
#include "Entity/Locations/OrientedNode/Asura_ServerNode_OrientedNode.h"

// E N U M S /////////////////////////////////////////////////////////////////

// These values are saved!
// Update s_szRagdollShoveStrings if you change these. // Tools-Side
enum ASURA_GAMESCENE_RAGDOLL_SHOVE_TYPE
{
    ASURA_GAMESCENE_RAGDOLL_SHOVE_NONE                    = 0,
    ASURA_GAMESCENE_RAGDOLL_SHOVE_COLLAPSE,
    ASURA_GAMESCENE_RAGDOLL_SHOVE_EXPLOSION_SMALL,
    ASURA_GAMESCENE_RAGDOLL_SHOVE_EXPLOSION_MEDIUM,
    ASURA_GAMESCENE_RAGDOLL_SHOVE_EXPLOSION_LARGE,
    ASURA_GAMESCENE_RAGDOLL_SHOVE_PROD,

    NUM_ASURA_GAMESCENE_RAGDOLL_SHOVE_TYPES,

    ASURA_GAMESCENE_RAGDOLL_SHOVE_TYPE_FIRST_PROJECT_SPECIFIC = 1000,
};

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GamesceneNode
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerNode_GamesceneNode : public Asura_ServerNode_OrientedNode, public virtual Asura_Gamescene_Node_Base
{
    typedef Asura_ServerNode_OrientedNode ENTITY_PARENT;
    typedef Asura_Gamescene_Node_Base     GAMESCENE_PARENT;

    friend class Tools_EntityNode_GamesceneNode;

public:
    Asura_ServerNode_GamesceneNode(Asura_Guid uGuid);
	virtual ~Asura_ServerNode_GamesceneNode();

    virtual bool HandleMessage( const class Asura_Message* pxMessage );

	float GetYOrient()                    const { Asura_Euler_3 xOrientE; GetOrientation( xOrientE ); return xOrientE.y; }
	bool  GetCanStandEmergeLeft()         const { return m_bCanStandEmergeLeft; }
	bool  GetCanStandEmergeRight()        const { return m_bCanStandEmergeRight; }
	bool  GetCanCrouchEmergeLeft()        const { return m_bCanCrouchEmergeLeft; }
	bool  GetCanCrouchEmergeRight()       const { return m_bCanCrouchEmergeRight; }
	bool  GetCanCrouchEmergeUp()          const { return m_bCanCrouchEmergeUp; }
	Asura_Guid GetCurrentActorGuid()      const { return m_uCurrentActorGuid; }
	float GetMaxCoverAngleClockwise()     const { return m_fMaxCoverAngleClockwise; }
	float GetMaxCoverAngleAnticlockwise() const { return m_fMaxCoverAngleAnticlockwise; }
	u_int GetShoveType()                  const { return m_uShoveType; }
	bool  GetStopHere()                   const { return m_bStopHere; }
	bool  IsLocked()                      const { return m_bLockedDueToShare; }
	bool  NeverStopHere()                 const { return m_bNeverStopHere; }

	void SetLock(bool bLocked) { m_bLockedDueToShare = bLocked; }

	void EnterNode(Asura_Guid uGuid);
	void ExitNode();
	void GetShoveDirection(Asura_Vector_3& xShoveDirection);

	virtual bool IsBaseOrDerivativeOf(u_short usClassification) const;
#ifdef ASURA_TOOLS
	virtual bool CanBeIncludedInTemplate() const;
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual void UpdateNeedFollowGround();

    void SetStopHere( const bool bBool ) { m_bStopHere = bBool; }
#endif //ASURA_TOOLS

	static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream );
    virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	u_short GetNodeArrivalMessageBlock() const { return m_usNodeArrivalMessageBlock; }
	u_short GetNodeDepartureMessageBlock() const { return m_usNodeDepartureMessageBlock; }


    static Asura_Hashed_List< Asura_ServerNode_GamesceneNode >* GetList() { return &s_xGamesceneNodeList; }
    static Asura_ServerNode_GamesceneNode* FindEntity( const Asura_Guid uGuid ) { return s_xGamesceneNodeList.FindEntity( uGuid ); }

protected:
	union
	{
		u_int m_uGamesceneNodeFlags;
		struct
		{
			u_int	m_bCanCrouchEmergeLeft  : 1,
					m_bCanCrouchEmergeRight : 1,
					m_bCanCrouchEmergeUp    : 1,
                    m_bCanStandEmergeLeft   : 1,
					m_bCanStandEmergeRight  : 1,
				    m_bLockedDueToShare     : 1,
				    m_bNeverStopHere        : 1,
                    m_bStopHere             : 1,
					SpareBits_GamesceneNode : 24;
		};
	};
	float          m_fMaxCoverAngleClockwise;
	float          m_fMaxCoverAngleAnticlockwise;
	float          m_fShoveOrientX;
	float          m_fShoveOrientY;
    u_int          m_uShoveType;
	Asura_Guid     m_uCurrentActorGuid;
	unsigned short m_usNodeArrivalMessageBlock;
	unsigned short m_usNodeDepartureMessageBlock;

private:
    enum { CURRENT_CHUNK_VERSION = 0 };
    static Asura_Hashed_List< Asura_ServerNode_GamesceneNode > s_xGamesceneNodeList;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_SERVERNODE_GAMESCENENODE_H__
#endif //ASURA_USE_AXON
