//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GS2Attractor.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Entity/Locations/OrientedNode/GamesceneNode/AttractorNode/Asura_ServerNode_GS2Attractor.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Timers.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Network/Asura_Message.h"

// axon
#include "Behaviour/Axon_BehaviourParams.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"
#include "Core/Axon_Messages.h"

// tools
#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GS2Attractor
//
//////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fTimeBetweenPulses = 0.25f;

Asura_Hashed_List< Asura_ServerNode_GS2Attractor > Asura_ServerNode_GS2Attractor::s_xList;

Asura_ServerNode_GS2Attractor::Asura_ServerNode_GS2Attractor( Asura_Guid uGuid )
: PARENT( uGuid )
, m_uGuidMask( 0 )
, m_fPulseTimer( 0.0f )
, m_fAttractRange( 0.0f )
, m_iAttractType( AXON_GS2_AAT_ANYTHING )
, m_uSpecificGuid( ASURA_GUID_UNREGISTERED )
, m_uAttractedGuid( ASURA_GUID_UNREGISTERED )
, m_iEnRouteBehaviourSet( AXON_BEHAVIOUR_TYPE_UNSET )
, m_pxEnRouteBehaviourParams( NULL )
{
	Classification = AsuraEntityClass_GS2Attractor;

	SetNeverUpdates( false );

	s_xList.AddToList( this );
}

Asura_ServerNode_GS2Attractor::~Asura_ServerNode_GS2Attractor()
{
	delete m_pxEnRouteBehaviourParams;
	m_pxEnRouteBehaviourParams = NULL;

	s_xList.RemoveFromList( this );
}

void Asura_ServerNode_GS2Attractor::OnEntityActivateMessage()
{
	PARENT::OnEntityActivateMessage();
	SendAttractionPulse();
}

void Asura_ServerNode_GS2Attractor::OnEntityDeactivateMessage()
{
	CancelAttraction();
	PARENT::OnEntityDeactivateMessage();
}

void Asura_ServerNode_GS2Attractor::Update()
{
	PARENT::Update();

	if ( IsEntityActive() && !IsAttractingSomething() )
	{
		m_fPulseTimer -= Asura_Timers::GetGameTimeSlice();
		if ( m_fPulseTimer < 0.0f )
		{
			SendAttractionPulse();
			m_fPulseTimer = Asura_Random::GetFloat( dbg_fTimeBetweenPulses * 0.9f, dbg_fTimeBetweenPulses * 1.1f );
		}
	}
}

bool Asura_ServerNode_GS2Attractor::HandleMessage( const Asura_Message* pxMessage )
{
	bool bHandled = false;

	if ( m_pxEnRouteBehaviourParams )
	{
		bHandled = m_pxEnRouteBehaviourParams->DoHandleMessage( pxMessage );
	}

	return bHandled ? true : PARENT::HandleMessage( pxMessage );
}

void Asura_ServerNode_GS2Attractor::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	// Parent.
	PARENT::ReadFromChunkStream( xStream );

	// Us.
	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_fAttractRange;
		xStream >> m_iAttractType;
		xStream >> m_uSpecificGuid;
		xStream >> m_uAttractedGuid;
		xStream >> m_iEnRouteBehaviourSet;
		Axon_BehaviourParams::CreateTreeFromChunkStream( xStream, m_pxEnRouteBehaviourParams );
	}
	else if ( uVersion == 0 )
	{
		xStream >> m_fAttractRange;
		xStream >> m_uSpecificGuid;
		xStream >> m_uAttractedGuid;
		xStream >> m_iEnRouteBehaviourSet;
		Axon_BehaviourParams::CreateTreeFromChunkStream( xStream, m_pxEnRouteBehaviourParams );

		m_iAttractType = ( m_uSpecificGuid == ASURA_GUID_UNREGISTERED ) ? AXON_GS2_AAT_ANYTHING : AXON_GS2_AAT_SPECIFIC_GUID;
	}
	else
	{
		Asura_Assertf( false, ( "Asura_ServerNode_GS2Attractor::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void Asura_ServerNode_GS2Attractor::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	// DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.

	// Version
	xStream << uCURRENT_CHUNK_VERSION;

	// Parent
	PARENT::WriteToChunkStream( xStream );

	// Our data
	xStream << m_fAttractRange;
	xStream << m_iAttractType;
	xStream << m_uSpecificGuid;
	xStream << m_uAttractedGuid;
	xStream << m_iEnRouteBehaviourSet;
	Axon_BehaviourParams::WriteTreeToChunkStream( m_pxEnRouteBehaviourParams, xStream );

	// DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.
}

bool Asura_ServerNode_GS2Attractor::IsBaseOrDerivativeOf( u_short usClassification ) const
{
	if ( usClassification == AsuraEntityClass_GS2Attractor )
	{
		return true;
	}
	return PARENT::IsBaseOrDerivativeOf( usClassification );
}

void Asura_ServerNode_GS2Attractor::StopAttractingMe( const Asura_Guid uAttractedGuid )
{
	Asura_Assert( uAttractedGuid == m_uAttractedGuid, "Asura_ServerNode_GS2Attractor::StopAttractingMe - wrong guid requesting" );

	// This is the entity currently being attracted telling us to stop.
	// Clear the attracted guid first otherwise we'll message them when we set ourselves inactive.
	SetAttractedGuid( ASURA_GUID_UNREGISTERED );
	SetIsActive( false );
}

void Asura_ServerNode_GS2Attractor::Create( const Asura_Guid uGuid, Asura_Chunk_Stream& xStream )
{
	Asura_ServerNode_GS2Attractor* pxAttractor = new Asura_ServerNode_GS2Attractor( uGuid );
	pxAttractor->ReadFromChunkStream( xStream );
}

#ifdef ASURA_TOOLS
void Asura_ServerNode_GS2Attractor::SetEnRouteBehaviourSet( int iBehaviourSet )
{
	m_iEnRouteBehaviourSet = iBehaviourSet;

	if ( m_pxEnRouteBehaviourParams )
	{
		delete m_pxEnRouteBehaviourParams;
		m_pxEnRouteBehaviourParams = NULL;
	}

	// Create the correct behaviour parameters for the behaviour type.
	Axon_BehaviourParams::CreateTree( m_iEnRouteBehaviourSet, m_pxEnRouteBehaviourParams );
}

bool Asura_ServerNode_GS2Attractor::UpdateBehaviourTree()
{
	const bool bParentUpdated = PARENT::UpdateBehaviourTree();

	Axon_BehaviourParams* pxBehaviourParams = NULL;
	if ( Axon_BehaviourParams::ConvertTree( m_iEnRouteBehaviourSet, m_pxEnRouteBehaviourParams, pxBehaviourParams ) )
	{
		delete m_pxEnRouteBehaviourParams;
		m_pxEnRouteBehaviourParams = pxBehaviourParams;
		return true;
	}

	return bParentUpdated;
}

void Asura_ServerNode_GS2Attractor::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance );

	m_uSpecificGuid  = pxInstance->ConvertGuid( m_uSpecificGuid );
	m_uAttractedGuid = pxInstance->ConvertGuid( m_uAttractedGuid );
	if ( m_pxEnRouteBehaviourParams )
	{
		m_pxEnRouteBehaviourParams->DoApplyChangesForTemplateInstance( pxInstance );
	}
}

bool Asura_ServerNode_GS2Attractor::CanBeIncludedInTemplate() const
{
	return ( Classification == AsuraEntityClass_GS2Attractor );
}

void Asura_ServerNode_GS2Attractor::SetAttractType( int iType )
{
	m_iAttractType = iType;
	if ( m_iAttractType == AXON_GS2_AAT_ANYTHING )
	{
		m_uSpecificGuid = ASURA_GUID_UNREGISTERED;
	}
}
#endif

void Asura_ServerNode_GS2Attractor::SendAttractionPulse()
{
	const float fAttractRangeSq = m_fAttractRange * m_fAttractRange;

	if ( m_iAttractType == AXON_GS2_AAT_ANYTHING )
	{
		// Try and attract anything.
		enum { GUID_MASK = 0x00000003 };
		if ( ++m_uGuidMask > GUID_MASK )
		{
			m_uGuidMask = 0;
		}

		for ( Asura_Hashed_List_ConstIt< Asura_ServerEntity_PhysicalObject > xPhyObjIter( Asura_ServerEntity_PhysicalObject::GetList() );
			!xPhyObjIter.Done();
			xPhyObjIter.Next() )
		{
			const Asura_ServerEntity_PhysicalObject* const pxPhyObj = xPhyObjIter.GetCurrent();

			const u_int uMaskedGuid = pxPhyObj->GetGuid() & GUID_MASK;
			if ( uMaskedGuid != m_uGuidMask )
			{
				continue;
			}

			// Is the entity not active or dead?
			if ( !pxPhyObj->IsEntityActive() || pxPhyObj->IsDead() )
			{
				continue;
			}

			// Is this entity within range?
			if ( m_fAttractRange > 0.0f && ( pxPhyObj->GetPosition() - GetPosition() ).MagnitudeSquared() > fAttractRangeSq )
			{
				continue;
			}

			SendAttractionPulseToEntity( pxPhyObj );
			// If we've got a bite we must stop iterating.
			if ( IsAttractingSomething() )
			{
				return;
			}
		}
	}
	else if ( m_uSpecificGuid != ASURA_GUID_UNREGISTERED )
	{
		// Try and attract our specific guid.
		const Asura_ServerEntity_PhysicalObject* const pxPhyObj = Asura_ServerEntity_PhysicalObject::FindEntity( m_uSpecificGuid );
		if ( pxPhyObj )
		{
			// Obviously, if the entity is dead, or not active, we don't want to attempt to attract.
			if ( pxPhyObj->IsEntityActive() && !pxPhyObj->IsDead() )
			{
				if ( m_fAttractRange == 0.0f || ( pxPhyObj->GetPosition() - GetPosition() ).MagnitudeSquared() < fAttractRangeSq )
				{
					SendAttractionPulseToEntity( pxPhyObj );
				}
			}
		}
	}
}

void Asura_ServerNode_GS2Attractor::CancelAttraction()
{
	if ( !IsAttractingSomething() ) { return; }

	// Message the entity being attracted to us (if they're not dead and active) to say that we're no longer worth going to.
	const Asura_ServerEntity_PhysicalObject* const pxPhysicalObject = Asura_ServerEntity_PhysicalObject::FindEntity( m_uAttractedGuid );
	if ( pxPhysicalObject && pxPhysicalObject->IsEntityActive() && !pxPhysicalObject->IsDead() )
	{
		Asura_Message* const pxMessage = Asura_Network::CreateMessage( ASURA_MSG_AXON_GS2_ATTRACTOR_CANCEL, sizeof( Axon_Msg_GS2_Attractor_Node_Cancel ) );
		if ( pxMessage )
		{
			Axon_Msg_GS2_Attractor_Node_Cancel* const pxData = static_cast< Axon_Msg_GS2_Attractor_Node_Cancel* >( pxMessage->GetData() );
			pxData->m_uAttractorGuid = GetGuid();
			Send( pxMessage, pxPhysicalObject->GetGuid() );
		}
	}

	SetAttractedGuid( ASURA_GUID_UNREGISTERED );
}

void Asura_ServerNode_GS2Attractor::SendAttractionPulseToEntity( const Asura_ServerEntity_PhysicalObject* const pxPhysicalObject ) const
{
	Asura_Message* const pxMessage = Asura_Network::CreateMessage( ASURA_MSG_AXON_GS2_ATTRACTOR_PULSE, sizeof( Axon_Msg_GS2_Attractor_Node_Pulse ) );
	if ( !pxMessage ) { return; }

	Axon_Msg_GS2_Attractor_Node_Pulse* const pxData = static_cast< Axon_Msg_GS2_Attractor_Node_Pulse* >( pxMessage->GetData() );
	pxData->m_uAttractorGuid     = GetGuid();
	pxData->m_xAttractorPosition = GetPosition();

	Send( pxMessage, pxPhysicalObject->GetGuid() );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
