//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GamesceneAttractor.cpp
//
// Created by NJD - 13 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Entity/Locations/OrientedNode/GamesceneNode/AttractorNode/Asura_ServerNode_GamesceneAttractor.h"

// asura
#include "Core/Asura_Timers.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#if defined(ASURA_USE_STREAMING)
#include "Streaming/Asura_Streaming_System.h"
#endif	// defined(ASURA_USE_STREAMING)

// axon
#include "Core/Axon_Messages.h"

// Tools
#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif


ProjectAttractServerEntityFunction*	Asura_ServerNode_GamesceneAttractor::s_pfnProjectAttractServerEntity = NULL;
Asura_Hashed_List< Asura_ServerNode_GamesceneAttractor > Asura_ServerNode_GamesceneAttractor::s_xAttractorList;

DEBUGVAR float dbg_fTimeBetweenChecks = 0.25f; // (TI - 08.06.2007) - Should keep this relatively small.

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_GamesceneAttractor
//
//////////////////////////////////////////////////////////////////////////////

Asura_ServerNode_GamesceneAttractor::Asura_ServerNode_GamesceneAttractor( Asura_Guid uGuid )
: PARENT( uGuid )
, m_uTargetType( ASURA_ATTRACTOR_TARGET_TYPE_GUID )
, m_uTargetGuid( ASURA_GUID_UNREGISTERED )
, m_uTargetCondition( ASURA_ATTRACTOR_CONDITION_RANGE )
, m_fTargetRange( 0.f )
, m_fAttractionSpeed( 1.f )
, m_uAttractorFlags( 0 )
, m_fCheckTimer ( Asura_Random::GetFloat( 0.0f, dbg_fTimeBetweenChecks ) )
, m_uEntityMaskingCount( 0 )
, m_uLockedByGuid( ASURA_GUID_UNREGISTERED )
{
    Classification = AsuraEntityClass_AttractorNode;

    SetNeverUpdates( false );

	s_xAttractorList.AddToList( this );
}

Asura_ServerNode_GamesceneAttractor::~Asura_ServerNode_GamesceneAttractor()
{
	s_xAttractorList.RemoveFromList( this );
}

bool Asura_ServerNode_GamesceneAttractor::HandleMessage( const Asura_Message* pxMessage )
{
	//switch ( pxMessage->GetMsgID() )
	//{
 //       case ASURA_MSG_AXON_GAMESCENE_ATTRACTOR_NODE_ARRIVED:
 //       {
	//		// If we've arrived we unregister the fact that someone was getting to us and then set ourselves inactive
	//		// (so we don't keep trying to attract people). If we don't unregister first we will cancel the guy that
	//		// has just arrived.
	//		SetAttractorLockedByGuid( ASURA_GUID_UNREGISTERED );
	//		SetIsActive( false );
	//		return true;
 //       }

	//	case ASURA_MSG_AXON_GAMESCENE_SET_ATTRACTOR_FLAGS:
	//	{
	//		const Asura_Msg_AIAttractor_Set_Flags* const pxData = static_cast< const Asura_Msg_AIAttractor_Set_Flags* >( pxMessage->GetData() );
	//		if ( pxData )
	//		{
	//			m_bOverrideGamesceneBehaviour = pxData->m_bOverrideGamesceneBehaviour;
	//		}
	//		return true;
	//	}

	//	default:
	//	{
			return PARENT::HandleMessage( pxMessage );
	//	}
	//}
}

void Asura_ServerNode_GamesceneAttractor::OnEntityActivateMessage()
{
    PARENT::OnEntityActivateMessage();
    SendAttractionPulse();
}

void Asura_ServerNode_GamesceneAttractor::OnEntityDeactivateMessage()
{
    CancelAttraction();
    PARENT::OnEntityDeactivateMessage();
}

void Asura_ServerNode_GamesceneAttractor::Update()
{
	PARENT::Update();

    if ( IsEntityActive() && !IsAttactorLocked() )
    {
		m_fCheckTimer -= Asura_Timers::GetGameTimeSlice();
		if ( m_fCheckTimer < 0.0f )
		{
			SendAttractionPulse();
			m_fCheckTimer = Asura_Random::GetFloat( dbg_fTimeBetweenChecks * 0.9f, dbg_fTimeBetweenChecks * 1.1f );
		}
    }
}

void Asura_ServerNode_GamesceneAttractor::CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream )
{
    Asura_ServerNode_GamesceneAttractor* const pxAttractorNode = new Asura_ServerNode_GamesceneAttractor( uGuid );
    if ( pxAttractorNode )
    {
        pxAttractorNode->ReadFromChunkStream( xStream );
    }
}

void Asura_ServerNode_GamesceneAttractor::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
    int iVersion;
    xStream >> iVersion;
	
	if ( iVersion == CURRENT_CHUNK_VERSION )
    {
        // Parent.
        PARENT::ReadFromChunkStream( xStream );

        // Us.
        xStream >> m_uTargetType;
        xStream >> m_uTargetGuid;
#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
        m_uTargetGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid(m_uTargetGuid);
#endif	// defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
        xStream >> m_uTargetCondition;
        xStream >> m_fTargetRange;
		xStream >> m_fAttractionSpeed;
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uAttractorFlags );
		xStream >> xFlagsBitfield;
    }
    else if ( iVersion == 1 )
    {
        // Parent.
        PARENT::ReadFromChunkStream( xStream );

        // Us.
        xStream >> m_uTargetType;
        xStream >> m_uTargetGuid;
#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
        m_uTargetGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid(m_uTargetGuid);
#endif	// defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
        xStream >> m_uTargetCondition;
        xStream >> m_fTargetRange;
		m_fAttractionSpeed = 1.f;
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uAttractorFlags );
		xStream >> xFlagsBitfield;
    }
	else if ( iVersion == 0 )
	{
		// Parent.
		PARENT::ReadFromChunkStream( xStream );

		// Us.
		xStream >> m_uTargetType;
		xStream >> m_uTargetGuid;
#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		m_uTargetGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid(m_uTargetGuid);
#endif	// defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		xStream >> m_uTargetCondition;
		xStream >> m_fTargetRange;
		m_fAttractionSpeed = 1.f;
	}
    else
    {
        Asura_Assertf( false, ( "Asura_ServerNode_GamesceneAttractor::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
    }
}

void Asura_ServerNode_GamesceneAttractor::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
    // (TI - 5.9.2004) - DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.

    // Version
    xStream << static_cast< int >( CURRENT_CHUNK_VERSION );

    // Parent
    PARENT::WriteToChunkStream(xStream);

    // Our data
    xStream << m_uTargetType;
    xStream << m_uTargetGuid;
    xStream << m_uTargetCondition;
    xStream << m_fTargetRange;
	xStream << m_fAttractionSpeed;
	Asura_WriteToStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uAttractorFlags );
	xFlagsBitfield.Add( m_bDEPRECATED, 1 );
	xFlagsBitfield.Add( m_bOverrideGamesceneBehaviour, 1 );
	xStream << xFlagsBitfield;

    // (TI - 5.9.2004) - DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.
}

void Asura_ServerNode_GamesceneAttractor::SendAttractionPulse()
{
    switch ( m_uTargetType )
    {
        case ASURA_ATTRACTOR_TARGET_TYPE_GUID:
        {
			if( s_pfnProjectAttractServerEntity && s_pfnProjectAttractServerEntity( m_uTargetGuid, this ) )
			{
				break;
			}

            const Asura_ServerEntity_PhysicalObject* const pxPhysicalObject = Asura_ServerEntity_PhysicalObject::FindEntity( m_uTargetGuid );
            if ( pxPhysicalObject )
            {
                // Obviously, if the entity is dead, or not active, we don't want to attempt to attract.
                if ( pxPhysicalObject->IsEntityActive() && !pxPhysicalObject->IsDead() )
                {
                    SendAttractionPulseToEntity( pxPhysicalObject );
                }
            }
            break;
        }

        case ASURA_ATTRACTOR_TARGET_TYPE_ANYONE:
        {
			//(TI - 08.06.2007) Go through and only do a 1/4th of the entities, this means that every PO will get checked every second with dbg_fTimeBetweenChecks = 0.25.
			enum
			{
				SELECTION_MASK = 0x00000003
			};

			if ( ++m_uEntityMaskingCount > 3 )
			{
				m_uEntityMaskingCount = 0;
			}

			Asura_Hashed_List_ConstIt< Asura_ServerEntity_PhysicalObject > xPhyObjIter( Asura_ServerEntity_PhysicalObject::GetList() );
			while ( !xPhyObjIter.Done() )
			{
				const Asura_ServerEntity_PhysicalObject* const pxPhyObj = xPhyObjIter.GetCurrent();
				xPhyObjIter.Next();

				const u_int uMaskedGuid = pxPhyObj->GetGuid() & SELECTION_MASK;
				if ( uMaskedGuid != m_uEntityMaskingCount )
				{
					continue;
				}

				SendAttractionPulseToEntity( pxPhyObj );
				if ( IsAttactorLocked() )
				{
					return;
				}
			}
            break;
        }

        default:
            Asura_Assert(false, "Unhandled gamescene attractor target type");
            break;
    }
}

void Asura_ServerNode_GamesceneAttractor::SendAttractionPulseToEntity( const Asura_ServerEntity_PhysicalObject* const pxPhysicalObject ) const
{
  //  Asura_Assert(pxPhysicalObject, "Asura_ServerNode_GamesceneAttractor::SendAttractionPulseToEntity sent a NULL pointer for a physical object");

  //  if ( pxPhysicalObject->IsEntityActive() && !pxPhysicalObject->IsDead() )
  //  {
  //      switch (m_uTargetCondition)
  //      {
  //          case ASURA_ATTRACTOR_CONDITION_RANGE:
  //          {
  //              if ( ( pxPhysicalObject->GetPosition() - GetPosition() ).MagnitudeSquared() > ( m_fTargetRange * m_fTargetRange ) )
  //              {
  //                  return;
  //              }
  //          }
  //          break;

  //      default:
  //          break;
  //      }

  //      Asura_Message* const pxMessage = Asura_Network::CreateMessage( ASURA_MSG_AXON_GAMESCENE_ATTRACTOR_NODE_PULSE, sizeof( Axon_Msg_Gamescene_Attractor_Node_Pulse ) );
  //      if ( !pxMessage ) { return; }
  //      Axon_Msg_Gamescene_Attractor_Node_Pulse* const pxData = static_cast< Axon_Msg_Gamescene_Attractor_Node_Pulse* >( pxMessage->GetData() );
  //      if ( !pxData ) { return; }

  //      pxData->m_uAttractorGuid      = GetGuid();
  //      pxData->m_xAttractorPosition  = GetPosition();
  //      pxData->m_uAttractorCondition = m_uTargetCondition;
		//pxData->m_fSpeed			  = m_fAttractionSpeed;
		//pxData->m_bOverrideGamesceneBehaviour = m_bOverrideGamesceneBehaviour;

  //      Send( pxMessage, pxPhysicalObject->GetGuid() );
  //  }
}

void Asura_ServerNode_GamesceneAttractor::CancelAttraction()
{
    //if (m_uLockedByGuid == ASURA_GUID_UNREGISTERED)
    //{
    //    return;
    //}

    //// (AW - 17.11.2004) - Message the entity that locked us to say (if they're not dead and active) that we're no longer worth going to.
    //const Asura_ServerEntity_PhysicalObject* const pxPhysicalObject = Asura_ServerEntity_PhysicalObject::FindEntity( m_uLockedByGuid );

    //if ( pxPhysicalObject && pxPhysicalObject->IsEntityActive() && !pxPhysicalObject->IsDead() )
    //{
    //    Asura_Message* const pxMessage = Asura_Network::CreateMessage( ASURA_MSG_AXON_GAMESCENE_ATTRACTOR_NODE_CANCEL, sizeof( Axon_Msg_Gamescene_Attractor_Node_Cancel ) );
    //    if ( !pxMessage ) { return; }
    //    Axon_Msg_Gamescene_Attractor_Node_Cancel* const pxData = static_cast< Axon_Msg_Gamescene_Attractor_Node_Cancel* >( pxMessage->GetData() );
    //    if ( !pxData ) { return; }

    //    pxData->m_uAttractorGuid = GetGuid();

    //    Send( pxMessage, pxPhysicalObject->GetGuid() );
    //}

    //SetAttractorLockedByGuid( ASURA_GUID_UNREGISTERED );
}

bool Asura_ServerNode_GamesceneAttractor::IsBaseOrDerivativeOf( u_short usClassification ) const
{
    if (usClassification == AsuraEntityClass_AttractorNode)
    {
        return true;
    }
    return PARENT::IsBaseOrDerivativeOf(usClassification);
}

#ifdef ASURA_TOOLS
void Asura_ServerNode_GamesceneAttractor::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
    PARENT::ApplyChangesForTemplateInstance(pxInstance);

    if (m_uTargetGuid == ASURA_ATTRACTOR_TARGET_TYPE_GUID)
    {
        m_uTargetGuid = pxInstance->ConvertGuid(m_uTargetGuid);
    }

    m_uLockedByGuid = pxInstance->ConvertGuid(m_uLockedByGuid);
}

bool Asura_ServerNode_GamesceneAttractor::CanBeIncludedInTemplate() const 
{
    return ( Classification == AsuraEntityClass_AttractorNode );
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
