//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_LiftNode.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Node class for lift splines
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/Locations/OrientedNode/LiftNode/Asura_ServerNode_LiftNode.h"

// asura
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_Entity_Event.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_LiftNode
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List< Asura_ServerNode_LiftNode > Asura_ServerNode_LiftNode::s_xNodeList;

Asura_ServerNode_LiftNode::Asura_ServerNode_LiftNode( Asura_Guid uGuid )
: PARENT( uGuid )
, m_uLiftNodeFlags( 0 )
{
    Classification = AsuraEntityClass_LiftNode;
	s_xNodeList.AddToList(this);
}

Asura_ServerNode_LiftNode::~Asura_ServerNode_LiftNode()
{
	s_xNodeList.RemoveFromList(this);
}

bool Asura_ServerNode_LiftNode::HandleMessage(const Asura_Message* pxMessage)
{
	if (!pxMessage) return false;

	switch(pxMessage->GetMsgID())
	{
		case ASURA_MSG_LIFT_NODE_SETFLAGS:
		{
			Asura_Assert(pxMessage->GetData(),"message has no data");
			if (pxMessage->GetData())
			{
				const Asura_Message_Lift_Node_SetFlags* pxLiftMessage = static_cast< const Asura_Message_Lift_Node_SetFlags* >( pxMessage->GetData() );
				if (pxLiftMessage->GetStopHereOff())
				{
					SetStopHere(false);
				}
				else if (pxLiftMessage->GetStopHereOn())
				{
					SetStopHere(true);
				}
			}
			return true;
		}
		default: break;
	}

	return PARENT::HandleMessage(pxMessage);
}

void Asura_ServerNode_LiftNode::CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream )
{
    Asura_ServerNode_LiftNode* pxLiftNode = new Asura_ServerNode_LiftNode( uGuid );
    
	if ( pxLiftNode )
    {
        pxLiftNode->ReadFromChunkStream( xStream );
    }
}

void Asura_ServerNode_LiftNode::SendOnArrivalMessages()
{
	SendMessagesForEvent(ASURA_ENTITY_EVENT_ONARRIVEDAT);
}

void Asura_ServerNode_LiftNode::SendOnDepartureMessages()
{
	SendMessagesForEvent(ASURA_ENTITY_EVENT_ONDEPARTEDFROM);
}

void Asura_ServerNode_LiftNode::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;

	if ( iVersion <= CURRENT_CHUNK_VERSION )
	{
		// Parent.
		PARENT::ReadFromChunkStream( xStream );

		// Us.
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uLiftNodeFlags );
		xStream >> xFlagsBitfield;
	}
	else
	{
		Asura_Assertf( false, ( "Asura_ServerNode_LiftNode::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

void Asura_ServerNode_LiftNode::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	// (TI - 5.9.2004) - DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.

	// Version
	xStream << static_cast< int >( CURRENT_CHUNK_VERSION );

	// Parent
	PARENT::WriteToChunkStream(xStream);

	// Our data
	Asura_WriteToStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uLiftNodeFlags );
	xFlagsBitfield.Add( m_bStopHere, 1 );
	xFlagsBitfield.Add( m_bBlockWhenOccupied, 1 );
	xFlagsBitfield.Add( m_bBlockWhenUnoccupied, 1 );
	xStream << xFlagsBitfield;

	// (TI - 5.9.2004) - DON'T FORGET TO UPDATE ApplyChangesForTemplateInstance IF YOU CHANGE STUFF.
}

bool Asura_ServerNode_LiftNode::IsBaseOrDerivativeOf(u_short usClassification) const
{
    if (usClassification == AsuraEntityClass_LiftNode)
    {
        return true;
    }

    return PARENT::IsBaseOrDerivativeOf(usClassification);
}

#ifdef ASURA_TOOLS
bool Asura_ServerNode_LiftNode::CanBeIncludedInTemplate() const 
{
    return ( Classification == AsuraEntityClass_LiftNode );
}
#endif

void Asura_Message_Lift_Node_SetFlags::FillInChunkBuffer(char*& pcChunkBuffer) const
{
	Asura_Chunk_ChunkBuffer_Stream xStream(pcChunkBuffer);
	WriteToChunkStream(xStream);
}

void Asura_Message_Lift_Node_SetFlags::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	Asura_WriteToStream_Bitfield_Wrapper<u_int> xFlagsBitfield( m_uFlags );
	xFlagsBitfield.Add(m_bStopHereOn, 1);
	xFlagsBitfield.Add(m_bStopHereOff, 1);
	xStream << xFlagsBitfield;
}

void Asura_Message_Lift_Node_SetFlags::InitialiseFromChunkBuffer(const char*& pcChunkBuffer)
{
	ASURA_CHUNK_EXTRACT( pcChunkBuffer, m_uFlags, u_int );
}
