//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_Spline.h
//
// Created by NJD - 13 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_SERVERNODE_SPLINE_H__
#define __ASURA_SERVERNODE_SPLINE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_Spline.h"
#include "Entity/Locations/Asura_ServerEntity_Node.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Message;

// M A C R O S ///////////////////////////////////////////////////////////////

// (TI - 13.08.2007) - There is a nasty bug somewhere in gamescene that is causing splne orientations to optain bad float values
// Sprinkle this macro around to try and catch where this is happening. Neat little trick here is that a bad float is not equal to itself.
#ifdef ASURA_ASSERT
#define ASURA_GAMESCENE_SPLINE_ORIENTATION_CHECK( xORIENTATION, uSPLINEGUID ) \
	Asura_Assertf( xORIENTATION == xORIENTATION, ("Spline orientation check failed, this will cause a crash at some point. Spline guid is %u", uSPLINEGUID) )
#else
#define ASURA_GAMESCENE_SPLINE_ORIENTATION_CHECK( xORIENTATION, uSPLINEGUID ) //nothing
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Asura_DistanceTime_GraphPoint
//
//////////////////////////////////////////////////////////////////////////////

struct Asura_DistanceTime_GraphPoint
{
    float InterpolateValue(const Asura_DistanceTime_GraphPoint& xNextValue, float fCumulativeTime) const
    {
        Asura_Assert( fCumulativeTime >= m_fCumulativeTime, "Out of range" );
        Asura_Assert( fCumulativeTime <= xNextValue.m_fCumulativeTime, "Out of range" );
        float fProportion = ( fCumulativeTime - m_fCumulativeTime ) / ( xNextValue.m_fCumulativeTime - m_fCumulativeTime );
        return ( xNextValue.m_fDistance * fProportion ) + ( ( 1.0f - fProportion ) * m_fDistance );
    }

    float m_fDistance;
    float m_fCumulativeTime;
};

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerNode_Spline
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerNode_Spline : public Asura_ServerEntity_Node, public Asura_Oriented_Spline
{
    typedef Asura_ServerEntity_Node PARENT_NODE;
	typedef Asura_Oriented_Spline   PARENT_SPLINE;

public:
	Asura_ServerNode_Spline(Asura_Guid uGuid);
	~Asura_ServerNode_Spline();

	Asura_Guid GetSourceGuid() const { return m_uSourceNode; }
	Asura_Guid GetTargetGuid() const { return m_uTargetNode; }

    // This returns the actual distance used in the calculation (clamped to the valid range along the spline).
	float GetPositionFromDistance(const float fDistanceRequest, Asura_Vector_3& xOutput, Asura_Quat* pxOutputOrientation = NULL) const;
	virtual void GetPositionFromProportion( float fProportionIntoTrack, Asura_Vector_3& xOutput, Asura_Quat* pxOutputOrientation = NULL ) const;
	void GetTangentFromProportion( float fProportionIntoTrack, Asura_Vector_3& xOutput ) const;

	virtual bool IsBaseOrDerivativeOf(u_short usClassification) const;

	u_int GetNumSplineEvents() const { return m_uNumSplineEvents; }
	float GetSplineEventDistance( const u_int uIndex ) const;
	bool GetSplineEventFlag_TriggerWhenReversed( const u_int uIndex ) const;
	void  SendMessagesForSplineEvent( const u_int uIndex ) const;

#ifdef ASURA_TOOLS
    void SetSourceGuid(Asura_Guid uNode) { m_uSourceNode = uNode; }
    void SetTargetGuid(Asura_Guid uNode) { m_uTargetNode = uNode; }

	virtual void CreateBasePoints();
	bool IsSmoothingPoint( const u_int uIndex ) const { return (uIndex == 2 || uIndex == (m_uNumberOfPoints - 3)); }

	// Spline event stuff.
	// Returns the index of the new event.
	u_int		  AddSplineEvent();
	void		  FlagSplineEventForDeletion( const u_int uIndex );
	void          RemoveDeletedSplineEvents();//Deletes any spline events that have been flagged for deletion
	float&		  GetSplineEventDistance( const u_int uIndex );
	u_short&	  GetSplineEventMessageBlock( const u_int uIndex );
	u_int&        GetSplineEventFlags( const u_int uIndex );
	Asura_String& GetSplineEventName( const u_int uIndex );

	void CalculateControlPointPositions();
	void CalculateStartControlPoint();
	void CalculateEndControlPoint();
	// Over-ride of Asura_Oriented_Spline version in order to smooth if necessary.
	virtual void CalculateEndControlPoints() { CalculateControlPointPositions(); }
	void SmoothingPointsChanged();
	const Asura_Vector_3& GetFirstPosition(bool bCountFromEnd) const;

	virtual bool CanBeIncludedInTemplate() const;
	virtual void ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance);
#endif //ASURA_TOOLS

	static void CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	#ifdef ASURA_TOOLS
	virtual bool SupportsCustomEvents() const { return true; }
	virtual int GetNumCustomEvents() const { return m_uNumSplineEvents; }
	virtual unsigned short* GetCustomEvent(int iIndex) { return &m_pxSplineEvents[iIndex].m_usMessageBlock; }
	virtual void GetCustomEventName(int iEventIndex, Asura_String& xEventName) { xEventName = m_pxSplineEvents[iEventIndex].m_xName; }
	#endif

private:
	// Both Source and Target nodes should be nodes, shouldn't have splines connected to splines.
	Asura_Guid m_uSourceNode;
	Asura_Guid m_uTargetNode;

	// Spline events. Messages should get sent when the distance along the spline is past. This is the responsibility of the traverser.
	struct DistanceAndMessageBlock
	{
		float		m_fDistance;
		u_short		m_usMessageBlock;
		union
		{
			u_int	m_uFlags;
			struct
			{
				u_int	m_bTriggerWhenReversed:1;//event should be used when the spline is being used in reverse	
			};
		};
#ifdef ASURA_TOOLS
		Asura_String m_xName;
		bool         m_bToBeDeleted;//Flags spline events that need to be removed
#endif

		DistanceAndMessageBlock();
	}*    m_pxSplineEvents;
	u_int m_uNumSplineEvents;

    enum { CURRENT_CHUNK_VERSION = 6 };
};

#ifdef ASURA_TOOLS
//Define for flags in DistanceAndMessageBlock for use by tools tickboxes
#define TOOLS_SPLINEEVENTFLAG_TRIGGERWHENREVERSED	0x00000001
#endif //ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__ASURA_SERVERNODE_SPLINE_H__
