//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_Objective.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//     Base Asura Objective Entity
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/Objective/Asura_ClientEntity_Objective.h"

// Asura includes
#include "Core/Asura_HashedLocalisedText.h"
#include "Interface/Asura_CommandConsole.h"
#include "Network/Asura_Message.h"

Asura_List<Asura_ClientEntity_Objective> Asura_ClientEntity_Objective::s_xSortedList;

Asura_ClientEntity_Objective::ObjectiveChangedCallback* Asura_ClientEntity_Objective::s_pfnChangeCallbackFunction = 0;

Asura_ClientEntity_Objective::Asura_ClientEntity_Objective()
: m_eObjectiveState(ASURA_OBJECTIVE_PENDING)
, m_uTextHash(ASURA_HASH_ID_UNSET)
, m_uObjectiveNumber(UINT_MAX)
, m_bInitialised(false)
{
	Classification = AsuraEntityClass_Objective;
}

Asura_ClientEntity_Objective::~Asura_ClientEntity_Objective()
{
#ifdef ASURA_ASSERT
	bool bInList = s_xSortedList.FindObject(this) != NULL;
	Asura_Assert(bInList == IsEntityActive(), "objective's active status is not in sync with presence on list!");
#endif // #ifdef ASURA_ASSERT
	if (IsEntityActive())
	{
		s_xSortedList.RemoveFromList(this);
	}
}

Asura_ClientEntity_Objective* Asura_ClientEntity_Objective::CreateFromMessage(const Asura_Message* pxMessage)
{
	Asura_Assert( pxMessage, "No message to create from!");
	if (!pxMessage)
	{
		return 0;
	}
	Asura_Assert( pxMessage->GetMsgID() == ASURA_MSG_CLIENTUPDATE, "can only create from update message");
	if (pxMessage->GetMsgID() != ASURA_MSG_CLIENTUPDATE)
	{
		return 0;
	}
	Asura_ClientEntity_Objective* pxObjective = new Asura_ClientEntity_Objective();

	if ( pxObjective )
	{
		//remove us from the render list. there's no good way to do this in the constructor,
		//and keep this in a create function also means we're not imposing on derived classes
		//derived classes not wanting to render should also call this in their create function.
		pxObjective->SetInvisibleFlag(true);
		//handle message as usual to initialise
		pxObjective->HandleMessage( pxMessage );
		//perform post-initialistion - derived classes will need to call this in their create function.
		pxObjective->OnCreate();
	}

	return pxObjective;
}

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE

bool Asura_ClientEntity_Objective::ConsoleCmd_Objective_GetNumber(u_int uIndex, bool bInterrogate, bool bHelp)
{
	if( bHelp || bInterrogate)
	{
		Asura_CommandConsole::Printf( L"Objective.GetNumber" );
		Asura_CommandConsole::Printf( L"    For GUIMenu use. Prints number of an objective from it's list index" );
		return true;
	}
	else
	{
		const Asura_ClientEntity_Objective* pxObjective = s_xSortedList.GetEntryByIndex(uIndex);
		if (pxObjective)
		{
			Asura_CommandConsole::Printf( L"%u", pxObjective->GetObjectiveNumber() );
			return true;
		}
	}
	return false;
}

bool Asura_ClientEntity_Objective::ConsoleCmd_Objective_GetText(u_int uIndex, bool bInterrogate, bool bHelp)
{
	if( bHelp || bInterrogate)
	{
		Asura_CommandConsole::Printf( L"Objective.GetText" );
		Asura_CommandConsole::Printf( L"    For GUIMenu use. Prints text of an objective from it's list index" );
		return true;
	}
	else
	{
		const Asura_ClientEntity_Objective* pxObjective = s_xSortedList.GetEntryByIndex(uIndex);
		if (pxObjective)
		{
			Asura_CommandConsole::Printf( L"%s", pxObjective->GetObjectiveText() );
			return true;
		}
	}
	return false;
}

bool Asura_ClientEntity_Objective::ConsoleCmd_Objective_IsPending(u_int uIndex, bool bInterrogate, bool bHelp)
{
	if( bHelp || bInterrogate)
	{
		Asura_CommandConsole::Printf( L"Objective.IsPending" );
		Asura_CommandConsole::Printf( L"    For GUIMenu use. Prints whether an objective is pending from it's list index" );
		return true;
	}
	else
	{
		const Asura_ClientEntity_Objective* pxObjective = s_xSortedList.GetEntryByIndex(uIndex);
		if (pxObjective)
		{
			Asura_CommandConsole::Printf( L"%u", pxObjective->IsPending()?1:0 );
			return true;
		}
	}
	return false;
}

bool Asura_ClientEntity_Objective::ConsoleCmd_Objective_IsReady(u_int uIndex, bool bInterrogate, bool bHelp)
{
	if( bHelp || bInterrogate)
	{
		Asura_CommandConsole::Printf( L"Objective.IsReady" );
		Asura_CommandConsole::Printf( L"    For GUIMenu use. Prints whether an objective is Ready from it's list index" );
		return true;
	}
	else
	{
		const Asura_ClientEntity_Objective* pxObjective = s_xSortedList.GetEntryByIndex(uIndex);
		if (pxObjective)
		{
			Asura_CommandConsole::Printf( L"%u", pxObjective->IsReady()?1:0 );
			return true;
		}
	}
	return false;
}

bool Asura_ClientEntity_Objective::ConsoleCmd_Objective_IsComplete(u_int uIndex, bool bInterrogate, bool bHelp)
{
	if( bHelp || bInterrogate)
	{
		Asura_CommandConsole::Printf( L"Objective.IsComplete" );
		Asura_CommandConsole::Printf( L"    For GUIMenu use. Prints whether an objective is Complete from it's list index" );
		return true;
	}
	else
	{
		const Asura_ClientEntity_Objective* pxObjective = s_xSortedList.GetEntryByIndex(uIndex);
		if (pxObjective)
		{
			Asura_CommandConsole::Printf( L"%u", pxObjective->IsComplete()?1:0 );
			return true;
		}
	}
	return false;
}

bool Asura_ClientEntity_Objective::ConsoleCmd_Objective_IsFailed(u_int uIndex, bool bInterrogate, bool bHelp)
{
	if( bHelp || bInterrogate)
	{
		Asura_CommandConsole::Printf( L"Objective.IsFailed" );
		Asura_CommandConsole::Printf( L"    For GUIMenu use. Prints whether an objective is Failed from it's list index" );
		return true;
	}
	else
	{
		const Asura_ClientEntity_Objective* pxObjective = s_xSortedList.GetEntryByIndex(uIndex);
		if (pxObjective)
		{
			Asura_CommandConsole::Printf( L"%u", pxObjective->IsFailed()?1:0 );
			return true;
		}
	}
	return false;
}

bool Asura_ClientEntity_Objective::ConsoleCmd_Objective_IsSkipped(u_int uIndex, bool bInterrogate, bool bHelp)
{
	if( bHelp || bInterrogate)
	{
		Asura_CommandConsole::Printf( L"Objective.IsSkipped" );
		Asura_CommandConsole::Printf( L"    For GUIMenu use. Prints whether an objective is Skipped from it's list index" );
		return true;
	}
	else
	{
		const Asura_ClientEntity_Objective* pxObjective = s_xSortedList.GetEntryByIndex(uIndex);
		if (pxObjective)
		{
			Asura_CommandConsole::Printf( L"%u", pxObjective->IsSkipped()?1:0 );
			return true;
		}
	}
	return false;
}

void Asura_ClientEntity_Objective::AddConsoleCommands()
{
	Asura_CommandConsole::AddCmd( L"Objective.GetNumber", ConsoleCmd_Objective_GetNumber, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Objective.GetText", ConsoleCmd_Objective_GetText, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Objective.IsPending", ConsoleCmd_Objective_IsPending, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Objective.IsReady", ConsoleCmd_Objective_IsReady, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Objective.IsComplete", ConsoleCmd_Objective_IsComplete, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Objective.IsFailed", ConsoleCmd_Objective_IsFailed, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddCmd( L"Objective.IsSkipped", ConsoleCmd_Objective_IsSkipped, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
}

#endif // #ifdef ASURA_SUPPORT_COMMAND_CONSOLE

bool Asura_ClientEntity_Objective::HandleMessage(const Asura_Message* pxMessage)
{
	if (pxMessage)
	{
		switch (pxMessage->GetMsgID())
		{
			case ASURA_MSG_CLIENTUPDATE:
			{
				//update message doubles as create, so make sure our list position doesn't change after creation
				const Asura_Message_Objective_ClientUpdate* pxUpdate = static_cast<const Asura_Message_Objective_ClientUpdate*>(pxMessage->GetData());
				if (pxUpdate)
				{
					pxUpdate->FillOutObjective(this);
				}
				return true;
			}
			default:
			{
				break;
			}
		}
	}

	return PARENT::HandleMessage(pxMessage);
}

const Asura_UnicodeChar* Asura_ClientEntity_Objective::GetObjectiveText() const
{
	return Asura_HashedLocalisedText::GetString(GetObjectiveLocalisedPageID(), m_uTextHash);
}

Asura_Hash_ID Asura_ClientEntity_Objective::GetObjectiveLocalisedPageID() const
{
	return uASURA_HASH_TEXTPAGE_OBJECTIVES;	
}

//add to list on activation
void Asura_ClientEntity_Objective::OnEntityActivate() 
{
	PARENT::OnEntityActivate();

	s_xSortedList.InsertInList(this, Compare);

	if (s_pfnChangeCallbackFunction)
	{
		//fire the callback on activation
		s_pfnChangeCallbackFunction(this, false);
	}
}

//remove from list on deactivation
void Asura_ClientEntity_Objective::OnEntityDeactivate() 
{
	PARENT::OnEntityDeactivate();

	s_xSortedList.RemoveFromList(this);
}

int Asura_ClientEntity_Objective::Compare( const Asura_ClientEntity_Objective& xOne, const Asura_ClientEntity_Objective& xTwo )
{
	return static_cast<int>(xOne.m_uObjectiveNumber) - static_cast<int>(xTwo.m_uObjectiveNumber);
}

void Asura_ClientEntity_Objective::SetStatus( ASURA_OBJECTIVE_STATE eNewStatus )
{
	if (m_eObjectiveState != eNewStatus)
	{
		m_eObjectiveState = eNewStatus;

		// pass state changes after initialisation to the callback
		if (m_bInitialised && s_pfnChangeCallbackFunction)
		{
			s_pfnChangeCallbackFunction(this, false);
		}
	}
}

void Asura_ClientEntity_Objective::SetObjectiveNumber( u_int uNewNumber )
{
	const u_int uOldNumber = m_uObjectiveNumber;
	m_uObjectiveNumber = uNewNumber;

	// if the index number has changed need to re-insert into the sorted list
	if( uOldNumber != m_uObjectiveNumber )
	{
		// remove if already present
		if( s_xSortedList.FindObject(this) )
		{							
			s_xSortedList.RemoveFromList( this );
		}

		s_xSortedList.InsertInList( this, Compare );
	}
}

void Asura_ClientEntity_Objective::OnCreate()
{
	//flag that the entity is ready to use
	m_bInitialised = true;

	//creation is implicitly activation, so fire the callback
	if (s_pfnChangeCallbackFunction)
	{
		s_pfnChangeCallbackFunction(this, true);
	}
}
