//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_Objective.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//     Base Asura Objective Entity
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_CLIENTENTITY_OBJECTIVE_H___
#define ASURA_CLIENTENTITY_OBJECTIVE_H___ 1

#include "Core/Asura_List.h"
#include "Core/Asura_PlatformBasics.h"
#include "Entity/Asura_Entity_Renderable.h"
#include "Entity/Objective/Asura_ServerEntity_Objective.h"

class Asura_ClientEntity_Objective : public Asura_Entity_Renderable
{
public :
	~Asura_ClientEntity_Objective();
	
	// can only be created from an incoming message
	static Asura_ClientEntity_Objective* CreateFromMessage(const Asura_Message* pxMessage);

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	static bool ConsoleCmd_Objective_GetNumber(u_int uIndex, bool bInterrogate, bool bHelp);
	static bool ConsoleCmd_Objective_GetText(u_int uIndex, bool bInterrogate, bool bHelp);
	static bool ConsoleCmd_Objective_IsPending(u_int uIndex, bool bInterrogate, bool bHelp);
	static bool ConsoleCmd_Objective_IsReady(u_int uIndex, bool bInterrogate, bool bHelp);
	static bool ConsoleCmd_Objective_IsComplete(u_int uIndex, bool bInterrogate, bool bHelp);
	static bool ConsoleCmd_Objective_IsFailed(u_int uIndex, bool bInterrogate, bool bHelp);
	static bool ConsoleCmd_Objective_IsSkipped(u_int uIndex, bool bInterrogate, bool bHelp);
	static void AddConsoleCommands();
#endif // #ifdef ASURA_SUPPORT_COMMAND_CONSOLE

	virtual bool HandleMessage(const Asura_Message* pxMessage);

	virtual void Render()
	{
		Asura_Assert(0, "Asura_ClientEntity_Objective::Render must be overriden if added to render list");
	}

	//state machine as boolean tests
	bool IsPending() const { return m_eObjectiveState == ASURA_OBJECTIVE_PENDING; }
	bool IsReady() const { return m_eObjectiveState == ASURA_OBJECTIVE_READY; }
	bool IsComplete() const { return m_eObjectiveState == ASURA_OBJECTIVE_COMPLETE; }
	bool IsFailed() const { return m_eObjectiveState == ASURA_OBJECTIVE_FAILED; }
	bool IsSkipped() const { return m_eObjectiveState == ASURA_OBJECTIVE_SKIPPED; }
	ASURA_OBJECTIVE_STATE GetState() const { return m_eObjectiveState; }

	u_int GetObjectiveNumber() const {return m_uObjectiveNumber;}
	const Asura_UnicodeChar* GetObjectiveText() const;
	Asura_Hash_ID GetObjectiveTextID() const	{ return m_uTextHash;}

	//get the hash id of the text page. virtual so projects can override, either to a different constant,
	//or to allow different pages per objective (page per level for eg)
	//make sure server function is overriden as well!
	virtual Asura_Hash_ID GetObjectiveLocalisedPageID() const;

	// Set accessors to force people to use the interface functions (e.g. in the FillOutObjective message function)
	// Most don't do anything at this level but may do more in derived classes. 
	virtual void SetStatus( ASURA_OBJECTIVE_STATE eNewStatus );
	virtual void SetObjectiveNumber( u_int uNewNumber ); // SetObjectiveNumber may cause the objective to be re-inserted into the sorted list
	virtual void SetTextHash( Asura_Hash_ID uTextHash ) { m_uTextHash = uTextHash; }
	virtual void OnCreate();		//called once entity is fully set up, to perform any messaging or similar

	//sorted list - use this for rendering
	static const Asura_List<Asura_ClientEntity_Objective>* GetSortedList() { return &s_xSortedList; }

	typedef void (ObjectiveChangedCallback)(const Asura_ClientEntity_Objective* pxObjective, bool IsBeingCreated);

	//stores a callback to fire when an objective activates or changes state
	//primarily for "new objective" type hud flashes
	static void SetObjectiveChangedCallback(ObjectiveChangedCallback* pfnChangeCallbackFunction) { s_pfnChangeCallbackFunction = pfnChangeCallbackFunction; }

protected :
	//protected constructor - can only be created from messages
	//just the one constructor to help maintain consistency
	Asura_ClientEntity_Objective();

	virtual	void OnEntityActivate();
	virtual	void OnEntityDeactivate();

	//comparison function for list sorting
	static int Compare( const Asura_ClientEntity_Objective& xOne, const Asura_ClientEntity_Objective& xTwo );

	ASURA_OBJECTIVE_STATE m_eObjectiveState;	//current state
	Asura_Hash_ID m_uTextHash;					//hash id for objective text
	u_int m_uObjectiveNumber;					//objective number for rendereding/sorting
	bool m_bInitialised;						//indicates entity is fully set up

private:
	static Asura_List<Asura_ClientEntity_Objective> s_xSortedList;

	static ObjectiveChangedCallback* s_pfnChangeCallbackFunction;

	typedef Asura_Entity_Renderable PARENT;
};
// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_CLIENTENTITY_OBJECTIVE_H___
