//
// Asura_ServerEntity_PFX_Effect.cpp
//
// Created by KOR - 15/03/2004 14:39:16
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"


// This file's header.
#include "Entity/ParticleGenerator/Asura_ServerEntity_PFX_Effect.h"

// Asura includes
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Logfile.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_Entity_Event.h"
#include "Entity/ParticleGenerator/Asura_ClientEntity_PFX_Effect.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "SpecialFX/PFX/Asura_PFX_Templates.h"

// Tools includes
#ifdef ASURA_TOOLS
#include "Tools_System.h"
#include "Entity/Tools_Entity_Object.h"
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "Misc/Tools_MessageLog.h"
#endif //#ifdef ASURA_TOOLS

Asura_Hashed_List<Asura_ServerEntity_PFX_Effect> Asura_ServerEntity_PFX_Effect::s_xList( 5 );

Asura_ServerEntity_PFX_Effect::Asura_ServerEntity_PFX_Effect(Asura_Guid uGuid)
: Asura_Entity_Server(uGuid)
, m_xPosition(Asura_Maths::ZeroVector3)
, m_xOrientation(Asura_Maths::IdentityMatrix3x3)
{
	//Set the chunk type for this entity
	Classification = AsuraEntityClass_PFX_Effect;

	SetNeverUpdates(true);
	SetNeverUpdateClient();

	s_xList.AddToList(this);
}

Asura_ServerEntity_PFX_Effect::Asura_ServerEntity_PFX_Effect(Asura_Guid uGuid, 
                                       const Asura_Vector_3& xPos, 
                                       const Asura_Matrix_3x3& xDir)
: Asura_Entity_Server(uGuid)
, m_xPosition(xPos)
, m_xOrientation(xDir)
{
	Classification = AsuraEntityClass_PFX_Effect;
	// Default to 1 metre below effect, pointing up
	m_xPFXEffectData.m_xCollisionPosition = xPos;
	m_xPFXEffectData.m_xCollisionPosition.y += 1.f;
	m_xPFXEffectData.m_xCollisionDirection.Zero();
	m_xPFXEffectData.m_xCollisionDirection.y = -1.f;

	SetNeverUpdates(true);
	SetNeverUpdateClient();

	s_xList.AddToList(this);
}

Asura_ServerEntity_PFX_Effect::~Asura_ServerEntity_PFX_Effect(void)
{
	s_xList.RemoveFromList(this);
}

bool Asura_ServerEntity_PFX_Effect::HandleMessage(const Asura_Message* pxMessage)
{
	if( pxMessage )	
	{
		switch( pxMessage->GetMsgID() )
		{
			case ASURA_MSG_ACTIVATE:
			case ASURA_MSG_DEACTIVATE:
			{
				// Simply pass onto client, as only they know if the effect is still running.
				Asura_Network::Forward(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, Guid, true);
				// Now let the parent know about the (de)activation, for chunking.
				break;
			}
			case ASURA_MSG_DESTRUCT :
			{
				SetThisAndClientsAsDestroyed();
				return true;
			}
		}
	}

	return Asura_Entity_Server::HandleMessage(pxMessage); 
}

void Asura_ServerEntity_PFX_Effect::OnEntityActivateMessage()
{
	// Don't call to parent here, because the activate/deactivate
	// messages are directly forwarded by HandleMessage
	// regardless of what we think our current active status is.

	// However, we still need to send the messages for the event.
	SendMessagesForEvent(ASURA_ENTITY_EVENT_ONACTIVATION);
}

void Asura_ServerEntity_PFX_Effect::OnEntityDeactivateMessage()
{
	// See OnEntityActivateMessage
	SendMessagesForEvent(ASURA_ENTITY_EVENT_ONDEACTIVATION);
}

// Chunk methods
int Asura_ServerEntity_PFX_Effect::GetChunkBufferSize()
{
	int iChunkSize = Asura_Entity_Server::GetChunkBufferSize();
	iChunkSize += sizeof(int);  // version number
    iChunkSize += sizeof(Asura_Entity_PFX_Effect_ChunkData);
	iChunkSize += m_xPFXEffectData.m_xSourceVolume.GetChunkBufferSize();
	return iChunkSize;
}

void Asura_ServerEntity_PFX_Effect::FillInChunkBuffer(char*& pcChunkBuffer)
{

	// Write the tools data version number
    int iVersionNumber = ASURA_SERVERENTITY_PFX_EFFECT_VERSION;
    ASURA_CHUNK_FILLANDVERIFY( pcChunkBuffer, iVersionNumber, int );

	// Write the parent data
	Asura_Entity_Server::FillInChunkBuffer(pcChunkBuffer);

	//Write this class's data
    Asura_Entity_PFX_Effect_ChunkData xToolsData;

	xToolsData.m_xPosition			  = m_xPosition;
	ASURA_CHUNK_ARRAY_VERIFY(xToolsData.m_xPosition, float, 3);
	Asura_Quat xTempQuat;
	xTempQuat.FromMatrix(m_xOrientation);
	xToolsData.m_xOrientation		  = Asura_Comp_Quat_Fast(xTempQuat);
	ASURA_CHUNK_ARRAY_VERIFY(xToolsData.m_xOrientation, u_int, 1);
	xToolsData.m_uEffectTemplateHash  = m_xPFXEffectData.m_uEffectTemplateHash;
	ASURA_CHUNK_VERIFY(xToolsData.m_uEffectTemplateHash, Asura_Hash_ID);
	xToolsData.m_xCollisionPosition	  = m_xPFXEffectData.m_xCollisionPosition;
	ASURA_CHUNK_ARRAY_VERIFY(xToolsData.m_xCollisionPosition, float, 3);
	xToolsData.m_xCollisionDirection  = m_xPFXEffectData.m_xCollisionDirection;
	ASURA_CHUNK_ARRAY_VERIFY(xToolsData.m_xCollisionDirection, float, 3);

#if defined ASURA_TOOLS
	if( Tools_System::IsActive() )
	{
		// (AAM - 11.10.2006) If saving the level in Tools, don't care about preserving the time since last activation
		xToolsData.m_fTimeSinceLastActivateMessage = 0.0f;
	}
	else
#endif
	{
		xToolsData.m_fTimeSinceLastActivateMessage = GetStartTime();
	}

	ASURA_CHUNK_VERIFY(xToolsData.m_fTimeSinceLastActivateMessage, float);

	xToolsData.m_fMinRetriggerTime = m_xPFXEffectData.m_fMinRetriggerTime;
	ASURA_CHUNK_VERIFY(xToolsData.m_fMinRetriggerTime, float);
	xToolsData.m_fMaxRetriggerTime = m_xPFXEffectData.m_fMaxRetriggerTime;
	ASURA_CHUNK_VERIFY(xToolsData.m_fMaxRetriggerTime, float);


	xToolsData.m_uFlags	= m_xPFXEffectData.m_uFlags;
	ASURA_CHUNK_BITFIELD_VERIFY(xToolsData.m_uFlags, u_int);
	

	ASURA_CHUNK_FILL( pcChunkBuffer, xToolsData, Asura_Entity_PFX_Effect_ChunkData);

	m_xPFXEffectData.m_xSourceVolume.FillInChunkBuffer(pcChunkBuffer);
}

void Asura_ServerEntity_PFX_Effect::CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream& xStream)
{
	Asura_ServerEntity_PFX_Effect* pxNewEffect = new Asura_ServerEntity_PFX_Effect(uGuid);
	pxNewEffect->ReadFromChunkStream(xStream);
}


void Asura_ServerEntity_PFX_Effect::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//Extract version number from chunk
	int iChunkVersion;
	xStream >> iChunkVersion;

	Asura_Assert(0 <= iChunkVersion && iChunkVersion <= ASURA_SERVERENTITY_PFX_EFFECT_VERSION, "Unrecognised chunk version");

	//read in parent class data if new enough
	if(iChunkVersion >= 9)
	{
		Asura_Entity_Server::ReadFromChunkStream(xStream);
	}

	if(iChunkVersion >= 10)
	{
		m_xPFXEffectData.ReadFromChunkStream(xStream, m_xPosition, m_xOrientation);
		float fStartTime;
		xStream >> fStartTime;
	}
	else if(iChunkVersion == 8 || iChunkVersion == 9)
	{
		Asura_Entity_PFX_Effect_ChunkData xChunkData;
		xStream.Read(sizeof(Asura_Entity_PFX_Effect_ChunkData), &xChunkData);
		// Extract attributes
		m_xPosition = xChunkData.m_xPosition;

		// extract orientation from compressed quat
		Asura_Quat xTempQuat;
		xChunkData.m_xOrientation.ToQuat(xTempQuat);
		xTempQuat.ToMatrix(m_xOrientation);

		m_xPFXEffectData.m_uEffectTemplateHash = xChunkData.m_uEffectTemplateHash;
#ifdef ASURA_TOOLS
		if(m_xPFXEffectData.m_uEffectTemplateHash && !Asura_PFX_System::DoesEffectHashExist(m_xPFXEffectData.m_uEffectTemplateHash))
		{
			Tools_MessageLog::AddMessage( "PFX", "Missing Template", new Tools_MessageLogAction_GotoEntity( Guid ), "Could not retrieve effect template for entity %u", Guid );
		}
#endif //ASURA_TOOLS

		// Collision plane data
		m_xPFXEffectData.m_xCollisionPosition = xChunkData.m_xCollisionPosition;
		m_xPFXEffectData.m_xCollisionDirection = xChunkData.m_xCollisionDirection;

		m_xPFXEffectData.m_fMinRetriggerTime = xChunkData.m_fMinRetriggerTime;
		m_xPFXEffectData.m_fMaxRetriggerTime = xChunkData.m_fMaxRetriggerTime;

		m_xPFXEffectData.m_uFlags = xChunkData.m_uFlags;


		m_xPFXEffectData.m_xSourceVolume.ReadFromChunkStream(xStream);
	}
#ifdef ASURA_TOOLS
	else if(iChunkVersion == 7)
	{
		Asura_Entity_PFX_Effect_ChunkData_V7 xChunkData;
		// Extract attributes
		xStream.Read(sizeof(Asura_Entity_PFX_Effect_ChunkData_V7), &xChunkData);
		m_xPosition = xChunkData.m_xPosition;

		// extract orientation from compressed quat
		Asura_Quat xTempQuat;
		xChunkData.m_xOrientation.ToQuat(xTempQuat);
		xTempQuat.ToMatrix(m_xOrientation);

		m_xPFXEffectData.m_uEffectTemplateHash = xChunkData.m_uEffectTemplateHash;
		if(m_xPFXEffectData.m_uEffectTemplateHash && !Asura_PFX_System::DoesEffectHashExist(m_xPFXEffectData.m_uEffectTemplateHash))
		{
			Tools_MessageLog::AddMessage( "PFX", "Missing Template", new Tools_MessageLogAction_GotoEntity( Guid ), "Could not retrieve effect template for entity %u", Guid );
		}

		// Collision plane data
		m_xPFXEffectData.m_xCollisionPosition = xChunkData.m_xCollisionPosition;
		m_xPFXEffectData.m_xCollisionDirection = xChunkData.m_xCollisionDirection;

		m_xPFXEffectData.m_uFlags = xChunkData.m_uFlags;

		m_xPFXEffectData.m_xSourceVolume.ReadFromChunkStream(xStream);
	}
	else if(iChunkVersion == 5 || iChunkVersion == 6)
	{
		Asura_Entity_PFX_Effect_ChunkData_V6 xChunkData;
		// Extract attributes
		xStream.Read(sizeof(Asura_Entity_PFX_Effect_ChunkData_V6), &xChunkData);
		m_xPosition = xChunkData.m_xPosition;

		// extract orientation from compressed quat
		Asura_Quat xTempQuat;
		if(iChunkVersion==5)
		{
			//version 5 used Asura_Comp_Quat instead of Asura_Comp_Quat_Fast
			((Asura_Comp_Quat*)&xChunkData.m_xOrientation)->ToQuat(xTempQuat);
		}
		else
		{
			xChunkData.m_xOrientation.ToQuat(xTempQuat);
		}
		xTempQuat.ToMatrix(m_xOrientation);

		m_xPFXEffectData.m_uEffectTemplateHash = xChunkData.m_uEffectTemplateHash;
		if(m_xPFXEffectData.m_uEffectTemplateHash && !Asura_PFX_System::DoesEffectHashExist(m_xPFXEffectData.m_uEffectTemplateHash))
		{
			Tools_MessageLog::AddMessage( "PFX", "Missing Template", new Tools_MessageLogAction_GotoEntity( Guid ), "Could not retrieve effect template for entity %u", Guid );
		}

		// Collision plane data
		m_xPFXEffectData.m_xCollisionPosition = xChunkData.m_xCollisionPosition;
		m_xPFXEffectData.m_xCollisionDirection = xChunkData.m_xCollisionDirection;

		m_xPFXEffectData.m_uFlags = xChunkData.m_uFlags;

		m_xPFXEffectData.m_xSourceVolume.ReadFromChunkStream(xStream);
	}
	else if(iChunkVersion >= 1)
	{
		Asura_Entity_PFX_Effect_ChunkData_V1 xChunkData;
		// Extract attributes
		xStream.Read(sizeof(Asura_Entity_PFX_Effect_ChunkData_V1), &xChunkData);
		m_xPosition = xChunkData.m_xPosition;
		m_xOrientation = xChunkData.m_xOrientation;
		m_xPFXEffectData.m_uEffectTemplateHash = xChunkData.m_uEffectTemplateHash;
		if(m_xPFXEffectData.m_uEffectTemplateHash && !Asura_PFX_System::DoesEffectHashExist(m_xPFXEffectData.m_uEffectTemplateHash))
		{
			Tools_MessageLog::AddMessage( "PFX", "Missing Template", new Tools_MessageLogAction_GotoEntity( Guid ), "Could not retrieve effect template for entity %u", Guid );
		}

		// Collision plane data
		m_xPFXEffectData.m_xCollisionPosition = xChunkData.m_xCollisionPosition;
		m_xPFXEffectData.m_xCollisionDirection = xChunkData.m_xCollisionDirection;

		if (iChunkVersion < 3)
		{
			// (AW - 08.11.2004) - Fix for bug where m_xPFXEffectData.m_bActive was an entire u_int instead of a single bit.
			m_xPFXEffectData.m_uFlags = 0;
			m_xPFXEffectData.m_bOldActiveFlag = (xChunkData.m_uFlags != 0);
		}
		else if (iChunkVersion == 3)
		{
			m_xPFXEffectData.m_uFlags = 0;
			m_xPFXEffectData.m_bOldActiveFlag = xChunkData.m_bOldActiveFlag;
		}
		else
		{
			m_xPFXEffectData.m_uFlags = xChunkData.m_uFlags;
		}

		if(iChunkVersion >= 2)
		{
			m_xPFXEffectData.m_xSourceVolume.ReadFromChunkStream(xStream);
		}
	}
	else if( iChunkVersion == 0 )
	{
		Asura_Entity_PFX_Effect_ChunkData_V0 xChunkData;
		// Extract attributes
		xStream.Read(sizeof(Asura_Entity_PFX_Effect_ChunkData_V0), &xChunkData);
		m_xPosition = xChunkData.m_xPosition;
		m_xOrientation = xChunkData.m_xOrientation;
		m_xPFXEffectData.m_uEffectTemplateHash = xChunkData.m_uEffectTemplateHash;
		if(m_xPFXEffectData.m_uEffectTemplateHash && !Asura_PFX_System::DoesEffectHashExist(m_xPFXEffectData.m_uEffectTemplateHash))
		{
			Tools_MessageLog::AddMessage( "PFX", "Missing Template", new Tools_MessageLogAction_GotoEntity( Guid ), "Could not retrieve effect template for entity %u", Guid );
		}

		// Collision plane data
		m_xPFXEffectData.m_xCollisionPosition = xChunkData.m_xCollisionPosition;
		m_xPFXEffectData.m_xCollisionDirection = xChunkData.m_xCollisionDirection;

		m_xPFXEffectData.m_uFlags = 0;
		m_xPFXEffectData.m_bOldActiveFlag = 0;
	}
#endif
	else
	{
		Asura_Assert( false, "Unsupported chunk version" );
	}

	if(iChunkVersion < 11)
	{
		//transfer the old PFX active flag to Asura_Entity
		SetEntityActiveFromLoad(m_xPFXEffectData.m_bOldActiveFlag);
	}
}

void Asura_ServerEntity_PFX_Effect::WriteToChunkStream (Asura_Chunk_Stream& xStream) const
{
	// Write the header...
	xStream << ASURA_SERVERENTITY_PFX_EFFECT_VERSION;

	// Write the parent data
	Asura_Entity_Server::WriteToChunkStream(xStream);

	// Write the data...
	m_xPFXEffectData.WriteToChunkStream(xStream, m_xPosition, m_xOrientation);

#if defined ASURA_TOOLS
	if( Tools_System::IsActive() || !IsActive() )
#else
	if(!IsActive())
#endif
	{
		// (AAM - 11.10.2006) If saving the level in Tools, don't care about preserving the time since last activation
		xStream << 0.0f;
	}
	else
	{
		xStream << GetStartTime();
	}
}

#ifdef ASURA_TOOLS
void Asura_ServerEntity_PFX_Effect::ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance)
{
	Asura_Entity_Server::ApplyChangesForTemplateInstance(pxInstance);

	m_xPosition = pxInstance->TransformPosition(m_xPosition);
	m_xOrientation = pxInstance->TransformMatrix(m_xOrientation);

	m_xPFXEffectData.ApplyChangesForTemplateInstance(pxInstance);
}

#endif // ASURA_TOOLS


float Asura_ServerEntity_PFX_Effect::GetStartTime(void) const
{
	const Asura_Guid				uGuid		= GetGuid();
	Asura_ClientEntity_PFX_Effect*	pxClient	= Asura_ClientEntity_PFX_Effect::FindEntity(uGuid);
	const float						fStartTime	= (pxClient == NULL) ? 0.0f : pxClient->GetStartTime();

	return(fStartTime);
}


/// End of file. ///
