//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_PhysicsObject.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_SERVERENTITY_PHYSICSOBJECT_H___
#define ASURA_SERVERENTITY_PHYSICSOBJECT_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_ServerEntity_PhysicsObject : public virtual Asura_ServerEntity_PhysicalObject
{

public:

    static Asura_ServerEntity_PhysicsObject* FindEntity( Asura_Guid uGUID ) { return s_xList.FindEntity( uGUID ); }
    static const Asura_Hashed_List<Asura_ServerEntity_PhysicsObject>* GetList( void ) { return &s_xList; }    

    bool GetClientUpdated( void ) const { return m_bClientUpdated; }
    virtual bool HandleMessage( const Asura_Message* pxMsg ) { return false; }    
    virtual void InitialiseFromChunkBuffer( const char*& pcChunkBfr );
    bool IsStatic( void ) const { return( 0 != (m_uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_STATIC) ); }
    void SetClientUpdated( bool bUpdated ) { m_bClientUpdated = bUpdated; }
    void SetupPhysicsObject( ASURA_PHYSICS_OBJECT_TYPE ePhysicsObjectType, bool bUpdateWaypointBlocking = true );
    virtual void Update( void ) { }

	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

    // Legacy chunk handling, to be removed as soon as possible.
    virtual void FillInChunkBuffer( char*& pcChunkBfr );
    virtual int GetChunkBufferSize( void );

#ifdef ASURA_TOOLS
    virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance ) { }
    virtual bool ShouldIncludeInLevelExport( void ) const;
#endif

protected:

    Asura_ServerEntity_PhysicsObject( bool bAllocateDynamicsBlock = true );
    virtual ~Asura_ServerEntity_PhysicsObject( void );

    virtual void MakeClientUpdateMessage( void );

    /// Richard May - 27/08/2003 17:26:59 - added this function to allow project code to force 
    /// static objects to update in certain game modes, situations, etc.
    void ForceUpdateMessage() { m_bClientUpdated = false; }
    
private:

    typedef Asura_ServerEntity_PhysicalObject PARENT;

    friend class Asura_ClientEntity_PhysicsObject;

    static Asura_Hashed_List<Asura_ServerEntity_PhysicsObject> s_xList;

    bool   m_bClientUpdated;        // Update once and once only (until death)

};


class Asura_MsgData_PhysicsObject_ClientUpdate : public Asura_Message_PhysicalObject_ClientUpdate
{

public:

    void FillPhysics( const Asura_ServerEntity_PhysicsObject* pxPhysicsObj ) { FillPhysical( pxPhysicsObj ); }

// Just use parent's versions for now since this subclass doesn't add of its
// own data anyway.
//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
//  static void RegisterFields( u_short usMsgID, u_short usEntityClassification );
//  static void RegisterFields( Asura_NetMsg* pxNetMsg );
//  static u_char GetNumFields( void );
//#endif

private:

    typedef Asura_Message_PhysicalObject_ClientUpdate PARENT;

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_SERVERENTITY_PHYSICSOBJECT_H___
