//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_Ragdoll.cpp
//
// Created by David Evans - 01/08/2002 10:29:51
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// includes
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifndef ASURA_PHYSICS_DISABLE_RAGDOLL

// header
#include "Entity/Ragdoll/Asura_ClientEntity_Ragdoll.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationAttachment.h"
#include "Animation/Asura_Hierarchy.h"

#include "Camera/Asura_Camera.h"

#include "ChunkSystem/Asura_Chunk_Stream.h"

#include "Core/Asura_LOS.h"
#include "Core/Asura_MaterialFlags.h"
#include "Core/Asura_Timers.h"

#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_PosAndOriInterpolator.h"
#include "Entity/Ragdoll/Asura_Ragdoll_Wrapper.h"

#include "Light/Asura_GI_Solution.h"

#include "Mesh/Asura_CollisionMesh.h"

#include "Physics/Asura_Physics_Ragdoll.h"

#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_SortedRender.h"

#ifdef ASURA_USE_INDEPENDENT_SHADOW_SYSTEM
#include "Shadows/Asura_Shadow_System.h"
#endif

static const u_int g_uChunkVersion = 0;
static const Asura_Bounding_Box g_xDefaultBB = { { { -1.0f, 1.0f, -1.0f, 1.0f, -1.0f, 1.0f } } };

Asura_Hashed_List< Asura_ClientEntity_Ragdoll > Asura_ClientEntity_Ragdoll::s_xClientEntityRagdollList;
int												Asura_ClientEntity_Ragdoll::s_iNextRagdollID = 0;
u_int											Asura_ClientEntity_Ragdoll::s_uTotalNumberOfRagdolls = 0;

Asura_ClientEntity_Ragdoll::Asura_ClientEntity_Ragdoll( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Animation* pxAnimation, Asura_Guid uGuid, const Asura_Physics_Ragdoll_Description& xRagdollDesc, bool bAnimateInitialPositions/* = false*/ )
{
	++s_uTotalNumberOfRagdolls;

	Classification = AsuraEntityClass_Ragdoll;

	m_pxAnimation = new Asura_Animation( pxAnimation );

	Asura_AnimationAttachment* pxAttach = m_pxAnimation->GetFirstAttachment();
	while(pxAttach)
	{
		pxAttach->RemoveRenderFlags(ASURA_HIERARCHY_RENDER_BONES_IN_WORLDSPACE);
		pxAttach = pxAttach->GetNextAttachment();
	}

	m_uRenderFlags = 0;

	m_iRagdollID = s_iNextRagdollID++;

	s_xClientEntityRagdollList.AddToList(this);

	m_iRagdollWrapperHandle = Asura_Ragdoll_Wrapper::Create( uGuid, uGuid, 5, xRagdollDesc, m_pxAnimation, xPosition, xOrientation );

	Asura_Ragdoll_Wrapper* pxWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );
	if ( pxWrapper )
	{
		pxWrapper->SetShouldFadeOut();
		pxWrapper->SetShouldDestroyClientEntity();
	}
	else
	{
		SetAsDestroyed();
	}
	
	m_uOriginalEntityGuid = uGuid;

#if defined(ASURA_USE_STREAMING)
	// MGH 5/6/2006: Set up the containing zone. Needed by Asura_SortedRender to set the GI lighting.
	UpdateContainingZone(xPosition);
#endif //defined(ASURA_USE_STREAMING)

#if !defined(ASURA_NO_NETWORK_LAG_COMPENSATION)
	// Remove the interpolator - its not appropriate for a ragdoll...
	delete(m_pxPosAndOriInterpolator);
	m_pxPosAndOriInterpolator	= NULL;
#endif	// !defined(ASURA_NO_NETWORK_LAG_COMPENSATION)
}

Asura_ClientEntity_Ragdoll::~Asura_ClientEntity_Ragdoll()
{
	Asura_Ragdoll_Wrapper::Destroy( m_iRagdollWrapperHandle, false );
	s_xClientEntityRagdollList.RemoveFromList(this);
	--s_uTotalNumberOfRagdolls;

	if(m_pxAnimation)
	{
		delete m_pxAnimation;
	}
}

void Asura_ClientEntity_Ragdoll::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	Asura_Ragdoll_Wrapper* pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );
	Asura_Physics_Ragdoll* pxRagdoll = NULL;
	if ( pxRagdollWrapper )
	{
		pxRagdoll = pxRagdollWrapper->GetPhysicsRagdollPointer();
	}
	if ( pxRagdoll )
	{
	u_int		uVersion;
	xStream >> uVersion;

		if( uVersion == 0 )
		{
			pxRagdoll->ReadFromChunkStream(xStream);
		}
		else
		{
			Asura_Assert(false, "Unexpected chunk-version for client-entity ragdoll.");
		}
	}
}

void Asura_ClientEntity_Ragdoll::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << static_cast<u_int>( g_uChunkVersion );

	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );
	Asura_Physics_Ragdoll* pxRagdoll = NULL;
	if ( pxRagdollWrapper )
	{
		pxRagdoll = pxRagdollWrapper->GetPhysicsRagdollPointer();
	}
	if ( pxRagdoll )
	{
		pxRagdoll->WriteToChunkStream( xStream );
	}
}

void Asura_ClientEntity_Ragdoll::Update()
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );
	if ( pxRagdollWrapper )
	{
		pxRagdollWrapper->Update( m_pxAnimation );
	}
}

void Asura_ClientEntity_Ragdoll::Render()
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );
	Asura_Physics_Ragdoll* pxRagdoll = NULL;
	if ( pxRagdollWrapper )
	{
		pxRagdoll = pxRagdollWrapper->GetPhysicsRagdollPointer();
	}
	if ( pxRagdoll )
	{
		Asura_Vector_3		xPosition;
		Asura_Matrix_3x3	xOrientation;
		
		pxRagdoll->UpdateAnimation();

		pxRagdoll->GetRenderingPosition(xPosition);
		pxRagdoll->GetRenderingOrientation(xOrientation);

		m_pxAnimation->Update(xPosition, xOrientation, ANIMTIME_USEGAMETIME, m_uRenderFlags | ASURA_HIERARCHY_RENDER_BONES_IN_WORLDSPACE);
	}
//	else
//	{
//		const Asura_Vector_3&	xPosition		= GetCurrentPosition();
//		const Asura_Matrix_3x3&	xOrientation	= GetCurrentOrientation();

//		m_pxAnimation->Update(xPosition, xOrientation, ANIMTIME_USEGAMETIME, m_uRenderFlags);
//	}
}

void Asura_ClientEntity_Ragdoll::ApplyImpulse( const int iHitBoneID, const Asura_Vector_3 &xImpulse )
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );
	if ( pxRagdollWrapper )
	{	
		const Asura_Hierarchy_Tween* pxTweenStorage = m_pxAnimation->GetTweenStore();

		Asura_Physics_Ragdoll* pxRagdoll = pxRagdollWrapper->GetPhysicsRagdollPointer();
		if ( pxRagdoll )
		{	
		
			// check to see if bone id is valid (it may be -1 for example)
			if ( (iHitBoneID >= 0) && (static_cast<u_int>(iHitBoneID) < pxTweenStorage->GetNumberOfBones()) )
			{
				const Asura_Hierarchy_TweenStore_BoneData&	xBoneData	= pxTweenStorage->GetBoneData(iHitBoneID);
				Asura_Vector_3								xPosition;

				xBoneData.GetPosition(xPosition);

				if( !pxTweenStorage->IsPositionDataInWorldspace() )
				{
					// Tween storage is in local-space, so we'll have to try using the animation's
					// cached position and orientation and hope they're still reasonably valid.
					// Note this should only really happen just after the ragdoll's been created,
					// before the tween storage has been filled with world-space data
					Asura_Animation* pxAnim = GetAnimation();
					if( pxAnim )
					{
						pxAnim->GetBoneLocation( iHitBoneID, &xPosition, NULL );
						xPosition = pxAnim->GetOrientQuat().RotateVector( xPosition);
						xPosition += pxAnim->GetPosition();
					}								
				}
				
				// give the main node most of the impulse
	 			//pxRagdoll->ApplyImpulseToNearestNode( xPosition, xImpulse );
				pxRagdoll->ApplyImpulseToNearbyNodes( xPosition, xImpulse );

				// give a base impulse to the whole body
	 			//pxRagdoll->ApplyImpulseToAll( xImpulse );
			}
			else // if we didn't have a valid bone id, apply the impulse to the whole body
			{
				pxRagdoll->ApplyImpulseToAll( xImpulse );
			}
		}
	}
}

void Asura_ClientEntity_Ragdoll::ApplyImpulse( const Asura_Vector_3 &xImpulse )
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );
	if (pxRagdollWrapper)
	{	
		Asura_Physics_Ragdoll* pxRagdoll = pxRagdollWrapper->GetPhysicsRagdollPointer();
		if ( pxRagdoll )
		{	
			pxRagdoll->ApplyImpulseToAll( xImpulse );
		}
	}
}

void Asura_ClientEntity_Ragdoll::SetAsFadingOut( void )
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );

	if (pxRagdollWrapper)
	{
		pxRagdollWrapper->SetAsFadingOut();
	}
}

bool Asura_ClientEntity_Ragdoll::IsFadingOut( void ) const
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );

	if (pxRagdollWrapper)
	{
		return pxRagdollWrapper->IsFadingOut();
	}
	else
	{
		return false;
	}
}

void Asura_ClientEntity_Ragdoll::Activate( void )
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );

	if (pxRagdollWrapper)
	{	
		pxRagdollWrapper->Activate();
	}
}

void Asura_ClientEntity_Ragdoll::Deactivate( void )
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );

	if (pxRagdollWrapper)
	{
		pxRagdollWrapper->Deactivate();
	}
}

bool Asura_ClientEntity_Ragdoll::IsActive( void ) const
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );

	if (pxRagdollWrapper)
	{
		return pxRagdollWrapper->IsActive();
	}
	else
	{
		return false;
	}
}

const Asura_Hierarchy_Skin* Asura_ClientEntity_Ragdoll::GetSkin( void ) const
{
	if (!m_pxAnimation)
	{
		return 0;
	}
	else
	{
        return m_pxAnimation->GetSkin();
	}
}

const Asura_Bounding_Box& Asura_ClientEntity_Ragdoll::GetBB( void ) const
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );

	if (pxRagdollWrapper)
	{	
		return pxRagdollWrapper->GetBB();
	}
	else
	{
		Asura_Assert( false, "Couldn't find Ragdoll Wrapper returning default BB, this is VERY bad." );
		return g_xDefaultBB;
	}
}

Asura_Physics_Ragdoll* Asura_ClientEntity_Ragdoll::GetPhysicsRagdollPointer() const
{
	Asura_Ragdoll_Wrapper *pxRagdollWrapper = Asura_Ragdoll_Wrapper::GetRagdollWrapper( m_iRagdollWrapperHandle );

	if (pxRagdollWrapper)
	{
		return pxRagdollWrapper->GetPhysicsRagdollPointer();
	}
	else
	{
		return NULL;
	}
}


Asura_ClientEntity_Ragdoll* Asura_ClientEntity_Ragdoll::FindRagdollEntity( int iRagdollID )
{
	Asura_Hashed_List_It<Asura_ClientEntity_Ragdoll> xIt(&s_xClientEntityRagdollList);

	Asura_ClientEntity_Ragdoll* pxRagdollEntity = 0;

	while(!xIt.Done())
	{
		pxRagdollEntity = xIt.GetCurrent();
		xIt.Next();

		if(pxRagdollEntity)
		{
			if(pxRagdollEntity->GetRagdollID() == iRagdollID)
			{
				return pxRagdollEntity;
			}
		}
	}

	return NULL;
}

Asura_ClientEntity_Ragdoll* Asura_ClientEntity_Ragdoll::Create( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xVelocity, const Asura_Vector_3& xImpactStrength, int iHitLocation, const Asura_Animation* pxAnimation, Asura_Guid uGuid, const Asura_Physics_Ragdoll_Description& xRagdollDesc )
{

	Asura_ClientEntity_Ragdoll* pxRagdollEntity = new Asura_ClientEntity_Ragdoll( xPosition, xOrientation, pxAnimation, uGuid, xRagdollDesc );

	// push the ragdoll around, based on the body's original velocity and the incoming impact
	if( pxRagdollEntity )
	{
		// apply velocity
		pxRagdollEntity->ApplyImpulse( xVelocity );

		// apply impact impulse
		pxRagdollEntity->ApplyImpulse( iHitLocation, xImpactStrength );
	}

	return pxRagdollEntity;
}
#endif // #ifndef ASURA_PHYSICS_DISABLE_RAGDOLL
