
//----------------------------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_Ragdoll_Handler.h
//
// Created by Will Plano - Tuesday, July 31, 2007
// Hacked out of KJFL's Asura_ClientEntity_Ragdoll minus the ClientEntity
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////
//----------------------------------------------------------------------------------------

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_RAGDOLL_HANDLER_SYSTEM_H__
#define __ASURA_RAGDOLL_HANDLER_SYSTEM_H__

#ifndef ASURA_PHYSICS_DISABLE_RAGDOLL

#include "Camera/Asura_Camera.h"
#include "Core/Asura_Handle.h"
#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )
#include "Visibility/Asura_Visibility_System.h"
#endif

class Asura_Animation;
class Asura_Chunk_Stream;
class Asura_Physics_Ragdoll;
class Asura_Physics_Ragdoll_Description;
class Asura_Entity_Renderable;

class Asura_Ragdoll_Wrapper
{
public:

	////////////////////////////////////////////////////////////
	//  Class methods
	////////////////////////////////////////////////////////////

	void SetShouldDestroyClientEntity( bool bDestroy = true ) { m_bDestroyClientEntity = bDestroy; }

	void SetShouldDestroyServerEntity( bool bDestroy = true ) { m_bDestroyServerEntity = bDestroy; }

	void SetShouldFadeOut( bool bFadeOut = true ) { m_bFadeOut = bFadeOut; }

	void SetFadeDelay( float fDelay ) { m_fFadeOutDelayTimer = m_fFadeOutDelay = fDelay; m_bFadeOut = true; }
	
	void SetFadeDuration( float fDuration ) { m_fFadeOutDuration = fDuration; m_bFadeOut = true; }

	void SetAsFadingOut( void );

	bool IsFadingOut( void ) const;
	
	// return a value between 0 and 1. 0 being fully faded out and about to be destroyed and 1 being fully in.
	float GetFadeParametric( void ) const { return IsFadingOut() ? m_fFadeOutTimer / m_fFadeOutDuration : 1.0f; }

	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;


	Asura_Physics_Ragdoll*	GetPhysicsRagdollPointer( void ) const;

	// Get the ID of the ragdoll. This is unique to each ragdoll, but it is *not* an entity guid
	// (since ragdolls are only client entities, and don't have a corresponding server entity).
	// This ID can be used in calls to FindRagdollEntity()
	//
	// KJFL - Friday, November 12, 2004 - I was going to make this a u_int, but existing Rogue code
	// uses an int to store ragdoll IDs.
	int GetWrapperID() const { return m_iHandlerID; }

	// Get the GUID of the original entity that spawned the ragdoll (e.g. an NPC).
	// Since ragdolls only exist on clients (i.e. their movement isn't duplicated 
	// exactly across networks) they don't have valid entity guids.
	u_int GetOwnerGuid() const { return m_uOwnerGuid; }

	// Query whether ragdoll is active (i.e. potentially moving around and taking up processor time) or just 'asleep'.
	bool IsActive( void ) const;

	// Make ragdoll inactive (i.e. it won't move or take processor time, unless something else makes it active such
	// as a player shooting at it). The physics system will generally make ragdolls inactive itself under the right
	// conditions, but this function is useful for situations where you know it should be inactive, e.g. you've just
	// made a ragdoll after completing a cutscene kill-move, and you just want a body in the right place.
	void Deactivate( void );

	void Activate( void );

	const Asura_Bounding_Box& GetBB( void ) const;

	void Update( Asura_Animation* pxAnimation );

#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )
	bool IsVisibleFromCurrentCamera( void ) { return m_xVisibilityQuery.IsAABBVisibleFromCurrentCamera( GetBB() ); }
#endif

	////////////////////////////////////////////////////////////
	//  Static member functions
	////////////////////////////////////////////////////////////

	// Factory functions
	static Asura_Handle Create( Asura_Guid uOwnerGuid, Asura_Guid uParentGuid, int iPriority, const Asura_Physics_Ragdoll_Description& xRagdollDesc, Asura_Animation* pxAnimation, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xVelocity = Asura_Maths::ZeroVector3, const Asura_Vector_3& xImpactStrength = Asura_Maths::ZeroVector3, const Asura_Vector_3& xImpactPosition = Asura_Maths::ZeroVector3 );
	static void Destroy( Asura_Handle pxHandler, bool bDestroyEntity = true );

	// Find a ragdoll handler from an ID. The ID of a ragdoll handler can be found by querying GetRagdollWrapperID().
	static Asura_Ragdoll_Wrapper* GetRagdollWrapper( int iRagdollWrapperID );

	static bool RemoveMostObsoleteRagdoll( const bool bRemoveImmediately, const bool bOnlyConsiderActiveRagdolls, Asura_Guid uParentGuid );

protected:

	//////////////////////////////////////////////////////////////////////////
	// Constructor 
	//////////////////////////////////////////////////////////////////////////

	Asura_Ragdoll_Wrapper( Asura_Guid uOwnerGuid, int iPriority, const Asura_Physics_Ragdoll_Description& xRagdollDesc, Asura_Animation* pxAnimation, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, bool bAnimateInitialPositions = false );
	
	//////////////////////////////////////////////////////////////////////////
	//  Destructor
	//////////////////////////////////////////////////////////////////////////		

	virtual ~Asura_Ragdoll_Wrapper();

	////////////////////////////////////////////////////////////
	//  Per instance data members
	////////////////////////////////////////////////////////////
	
	void SetServerEntityAsDestroyed( void ) const;

	void SetClientEntityAsDestroyed( void ) const;

	bool IsClientEntityDestroyed( void ) const;

	bool IsBetterToRemoveThan( const Asura_Ragdoll_Wrapper* pxOther, bool bRemoveImmediately );

	static u_int GetNumberOfFadingRagdolls();

	Asura_Entity_Renderable* GetClientEntity() const;

	// Internal handle to the Physics ragdoll system. This handle is never given out
	// to external classes - thats the job of the m_iRagdollID below.
	Asura_Handle			m_iPhysicsRagdollHandle;

	// Unique ID for the ragdolls so they can be searched for. Note this is NOT 
	// an entity guid. Since ragdolls only exist on clients (i.e. their movement
	// isn't duplicated exactly across networks) they don't have valid guids.
	// 
	// KJFL - Friday, November 12, 2004 - I was going to make this a u_int, but existing Rogue code
	// uses an int to store ragdoll IDs.
	int						m_iHandlerID;

	// internal handle for indexing arrays, shouldn't be given out.
	Asura_Handle			m_iHandlerHandle;

	// priority for removal system, 5 is default priority, lower priorities will be removed first.
	// Use SetPriority() to set it.
	int						m_iPriority;

	// The guid of the object that created the ragdoll. Since ragdolls only exist on 
	// clients (i.e. their movement isn't duplicated exactly across networks) they
	// don't have valid guids. Therefore they use the guid of the original entity
	// when answering LOS calls etc.
	Asura_Guid				m_uOwnerGuid;

	// Stores the frame number the ragdoll was created. Used to determine the relative
	// ages of ragdolls (exact age in seconds etc is overkill; a simple u_int to compare
	// against does the job nicely).
	u_int					m_uCreationFrameStamp;

	// Used when fading out a ragdoll - projects can set the fade out speed by
	// setting m_fFadeOutTime.
	float					m_fFadeOutDuration;
	float					m_fFadeOutTimer;
	float					m_fFadeOutDelay;
	float					m_fFadeOutDelayTimer;

#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )
	Asura_Visibility_Query	m_xVisibilityQuery;
#else // !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )

	// KJFL - 20 February 2008 - note on the code below: why would you need
	// m_iLastKnownModule *per camera*? Surely it's the same one!? Anyway,
	// replacing with the visibility stuff above.

	// Stores the last known module when doing visibility tests (the tests themselves
	// update the values).
	int						m_iLastKnownModule[ ASURA_MAX_NO_OF_CAMERAS ];
#if defined(ASURA_USE_STREAMING)
	Asura_Hash_ID			m_auLastKnownZone[ASURA_MAX_NO_OF_CAMERAS];
#endif //defined(ASURA_USE_STREAMING)

#endif // !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )

	// cache of the visibility state for when deciding which ragdolls to remove
	union
	{
		u_int		m_uFlags;

		struct
		{
			u_int	m_bIsLinkedInToVisibleModule	:1,
					m_bIsLinkedInAndInFrustum		:1,
					m_bWasRenderedThisFrame			:1,
					m_bDestroyServerEntity			:1,
					m_bDestroyClientEntity			:1,
					m_bFadeOut						:1,
													:0; // 26 spare-bits
		};
	};

	////////////////////////////////////////////////////////////
	//  Static data members
	////////////////////////////////////////////////////////////
	
	static Asura_Tracking_Handle_List					s_xHandlerHandleList;
	static Asura_Ragdoll_Wrapper*						s_apxHandlers[];
	static int											s_aiHandlerIDs[];
	static int											s_iNextHandlerID;

}; // end class Asura_Ragdoll_Wrapper

#endif // #ifndef ASURA_PHYSICS_DISABLE_RAGDOLL

#endif // end #ifndef __ASURA_RAGDOLL_HANDLER_SYSTEM_H__
