//
// Asura_ServerEntity_StreamingBackgroundSoundController.cpp
//
// Created by Richard Rice - 03/08/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Entity/Sound/Asura_ServerEntity_StreamingBackgroundSoundController.h"

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Message.h"
#include "Sound/Asura_StreamingBackgroundSound_System.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif



#ifdef ASURA_TOOLS
Asura_Hashed_List<Asura_ServerEntity_StreamingBackgroundSoundController> Asura_ServerEntity_StreamingBackgroundSoundController::s_xList( 5 );
#endif


Asura_ServerEntity_StreamingBackgroundSoundController::Asura_ServerEntity_StreamingBackgroundSoundController(Asura_Guid uGuid)
:Asura_Entity_Server(uGuid)
,m_uSoundID(0)
{
	Classification = AsuraEntityClass_StreamingBackgroundSoundController;

	
	SetNeverUpdateClient();
	SetNeverUpdates(true);

	#ifdef ASURA_TOOLS
	s_xList.AddToList(this);
	#endif
}

Asura_ServerEntity_StreamingBackgroundSoundController::~Asura_ServerEntity_StreamingBackgroundSoundController()
{
	#ifdef ASURA_TOOLS
	s_xList.RemoveFromList(this);
	#endif
}

void Asura_ServerEntity_StreamingBackgroundSoundController::Create(Asura_Guid uGuid, Asura_Chunk_Stream& xStream)
{
	Asura_ServerEntity_StreamingBackgroundSoundController* pxEntity = new Asura_ServerEntity_StreamingBackgroundSoundController(uGuid);
	pxEntity->ReadFromChunkStream(xStream);
}


void Asura_ServerEntity_StreamingBackgroundSoundController::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//check our version
	int iVersion;
	xStream >> iVersion;
	Asura_Assertf(iVersion <= CHUNK_VERSION,("Unknown chunk version %d for streaming background sound controller",iVersion));

	//read parent
	PARENT::ReadFromChunkStream(xStream);

	//read the rest of the data for this entity
	xStream >> m_uSoundID;
}

void Asura_ServerEntity_StreamingBackgroundSoundController::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	//save our version
	xStream << (int) CHUNK_VERSION;
	//save the parent
	PARENT::WriteToChunkStream(xStream);

	//save data for this entity
	xStream << m_uSoundID;
}

bool Asura_ServerEntity_StreamingBackgroundSoundController::HandleMessage(const Asura_Message* pxMessage)
{
	if(IsDestroyed()) return true;

	if (pxMessage)
	{
		switch (pxMessage->GetMsgID())
		{
			case ASURA_MSG_ACTIVATE:
			case ASURA_MSG_DEACTIVATE:
			{
				//forward activate/deactivate message to client
			    u_int uFlagToSet = ASURA_STREAMINGBACKGROUNDSOUNDFLAG_ACTIVE;
				u_int uFlags = (pxMessage->GetMsgID() == ASURA_MSG_ACTIVATE) ? uFlagToSet : 0;
				SendFlagMessage(uFlagToSet,uFlags);
				return true;
			}

			default :
				break;
		}
	}

	return false;
}


void Asura_ServerEntity_StreamingBackgroundSoundController::SendFlagMessage(u_int uFlagsToSet,u_int uFlags)
{

	u_int uMsgSize = sizeof(Asura_Message_StreamingBackgroundSoundFlags);

	Asura_Message* pxMsg = Asura_Network::CreateMessage( ASURA_MSG_STREAMINGBACKGROUNDSOUND_FLAGS,  uMsgSize);
	Asura_Message_StreamingBackgroundSoundFlags* pxMessageData = reinterpret_cast<Asura_Message_StreamingBackgroundSoundFlags*>( pxMsg->GetData() );

	pxMessageData->m_uFlagsToSet = uFlagsToSet;
	pxMessageData->m_uFlags = uFlags;
	pxMessageData->m_uSoundID = m_uSoundID;

	pxMsg->To = ASURA_GUID_ALL_CLIENTMACHINES;
	pxMsg->From = GetGuid();

	Asura_Network::Send(pxMsg);

}

#ifdef ASURA_TOOLS
void Asura_ServerEntity_StreamingBackgroundSoundController::ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance)
{
	Asura_Entity_Server::ApplyChangesForTemplateInstance(pxInstance);

	//update the sound id
	m_uSoundID = pxInstance->ConvertStreamingSoundID(m_uSoundID);

}
#endif // ASURA_TOOLS






