//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_StartPoint.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     Player start point entity
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/Spawners/Asura_ServerEntity_StartPoint.h"

#include "Core/Memory/Asura_Mem.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Message.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

// static data
Asura_Hashed_List<Asura_ServerEntity_StartPoint> Asura_ServerEntity_StartPoint::s_xList;

//////////////////////////////////////////////////////////////////////////
// Ctor / ~tor
//////////////////////////////////////////////////////////////////////////

Asura_ServerEntity_StartPoint::Asura_ServerEntity_StartPoint(Asura_Guid uGuid)
: Asura_Entity_Server(uGuid)
, m_xPosition( Asura_Maths::ZeroVector3 )
, m_xOrientation( Asura_Maths::IdentityMatrix3x3 )
{
	Classification	= AsuraEntityClass_StartPoint;

	// we have no client entity to update.
	SetNeverUpdateClient();
	SetNeverUpdates(true);

	s_xList.AddToList(this);
}

Asura_ServerEntity_StartPoint::~Asura_ServerEntity_StartPoint()
{
	s_xList.RemoveFromList(this);
}

//////////////////////////////////////////////////////////////////////////
//	Static create functions
//////////////////////////////////////////////////////////////////////////
Asura_ServerEntity_StartPoint* Asura_ServerEntity_StartPoint::Create(Asura_Guid uGuid)
{
	return new Asura_ServerEntity_StartPoint(uGuid);
}

void Asura_ServerEntity_StartPoint::Create(Asura_Guid uGuid, Asura_Chunk_Stream& xStream)
{
	Asura_ServerEntity_StartPoint* pxStartPoint = new Asura_ServerEntity_StartPoint(uGuid);
	if(pxStartPoint)
	{
		pxStartPoint->ReadFromChunkStream( xStream );
	}
}

//////////////////////////////////////////////////////////////////////////
//	Chunk handling
//////////////////////////////////////////////////////////////////////////

void Asura_ServerEntity_StartPoint::WriteToChunkStream (Asura_Chunk_Stream& xStream) const
{
	const int iVersion = CURRENT_CHUNK_VERSION;

	// Write the version.
	xStream << iVersion;
	// Write parent class
	Asura_Entity_Server::WriteToChunkStream(xStream);

	//Write this class
	xStream << m_xPosition;
	xStream << m_xOrientation;
}

void Asura_ServerEntity_StartPoint::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	// get version number
	int iChunkVersion = -1;
	xStream >> iChunkVersion;
	
	//read in parent class data if new enough
	if(iChunkVersion >= 1)
	{
		Asura_Entity_Server::ReadFromChunkStream(xStream);
	}

	if (iChunkVersion > CURRENT_CHUNK_VERSION || iChunkVersion < 0)
	{
		Asura_Assert( 0, "Unknown chunk buffer version for start point.\n");
		return;
	}

	// Currently in all versions
	xStream >> m_xPosition;
	xStream >> m_xOrientation;

	if (iChunkVersion <= 1)
	{
		u_int uDummyFlags = 0;
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( uDummyFlags );
		xStream >> xFlagsBitfield;
		if (uDummyFlags) { SetEntityActiveFromLoad( true ); }
	}
}

#ifdef ASURA_TOOLS
void Asura_ServerEntity_StartPoint::ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance)
{
	Asura_Entity_Server::ApplyChangesForTemplateInstance(pxInstance);

	//update position and orientation
	m_xPosition = pxInstance->TransformPosition(m_xPosition);
	m_xOrientation = pxInstance->TransformMatrix(m_xOrientation);
}
#endif // ASURA_TOOLS

