//////////////////////////////////////////////////////////////////////////////
//
// Asura_ClientEntity_LookAtTrigger.h
//
// Created by Anthony Gregson - 26/04/04
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Description:
//
//     Client-side version of the look-at triggers. Needed as long as camera
//     data is not available on the server, and the functionality should be
//     merged into the server entity as soon as it is.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_CLIENTENTITY_LOOKATTRIGGER_H___
#define ___ASURA_CLIENTENTITY_LOOKATTRIGGER_H___ 1

#include "Entity/Asura_Entity.h"

class Asura_Message;

class Asura_ClientEntity_LookAtTrigger : public Asura_Entity_Client
{
public:
	~Asura_ClientEntity_LookAtTrigger();

	void Update();
	bool HandleMessage(const class Asura_Message* pxMessage);

	static Asura_ClientEntity_LookAtTrigger* Create(const Asura_Message* pxMessage);

	#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	#endif

private:
	Asura_ClientEntity_LookAtTrigger(Asura_Guid uGuid);

	// Control parameters
	float m_fMaxDistance;
	float m_fMinDistance;
	float m_fMaxAngleOrEdgeDeadzone;	// Which of the two options this is is dictated by the m_bUseScreenArea bool
	float m_fMinAngleOrCentreDeadzone;	// Ditto
	float m_fInwardConeCosMaxAngle;
	u_int m_uTriggeringTypes;
	int m_iSpecifiedPlayer;
	Asura_Vector_3 m_xPosition;
	Asura_Vector_3 m_xDirection;

	union
	{
		u_int m_uLookAtFlags;
		struct
		{
			u_int m_bUseScreenArea			: 1;
			u_int m_bUseLOS					: 1;
			u_int m_bTriggerOnNotLooking	: 1;
			u_int m_bAdjustForFOV			: 1;
			u_int m_bReceivedCreateMessage	: 1;
		};
	};

	// Current data
	int m_iNumberCurrentlyTriggering;
	float m_fUpdateTimer;

	typedef Asura_Entity_Client PARENT;
	friend class Tools_Entity_LookAtTrigger_Object;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_CLIENTENTITY_LOOKATTRIGGER_H___
