//
// Asura_ClientEntity_Trigger.cpp
//
// Created by Richard Rice - 22/11/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Entity/Triggers/Asura_ClientEntity_Trigger.h"

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Asura_StaticMessage_System.h"

#define ASURA_CLIENTENTITY_TRIGGER_VERSION_NUMBER 4

Asura_ClientEntity_Trigger::Asura_ClientEntity_Trigger(Asura_Guid uGuid)
:Asura_ClientEntity_Activatable(uGuid)
{
	Asura_Entity_Client::AddToUpdateableClientList( this );
	for (int i=0; i<ASURA_TRIGGER_ENTITY_NOOFSTATES; i++)
	{
		MessageBlock[i] = ASURA_STATICMESSAGE_EMPTYBLOCK;
	}
}

Asura_ClientEntity_Trigger::Asura_ClientEntity_Trigger(Asura_Guid uGuid,const char*& pcChunkBuffer)
:Asura_ClientEntity_Activatable(uGuid,pcChunkBuffer)
{
	Asura_Entity_Client::AddToUpdateableClientList( this );
	int iVersion;
	ASURA_CHUNK_EXTRACT( pcChunkBuffer, iVersion,int);
	Asura_Assert(iVersion >= 4 && iVersion <= ASURA_CLIENTENTITY_TRIGGER_VERSION_NUMBER, "FATAL ERROR: unknown version number for Asura_ClientEntity_Trigger");

	for(int i=0; i < ASURA_TRIGGER_ENTITY_NOOFSTATES; i++)
	{
		ASURA_CHUNK_EXTRACT( pcChunkBuffer, MessageBlock[i],unsigned short);
	}

	// (MH - 08.01.2003) fix alignment
	pcChunkBuffer += sizeof(u_short);
}

void Asura_ClientEntity_Trigger::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	Asura_ClientEntity_Activatable::ReadFromChunkStream(xStream);
	int iVersion;
	xStream >> iVersion;
	Asura_Assert(iVersion >= 4 && iVersion <= ASURA_CLIENTENTITY_TRIGGER_VERSION_NUMBER, "FATAL ERROR: unknown version number for Asura_ClientEntity_Trigger");

	for(int i=0; i < ASURA_TRIGGER_ENTITY_NOOFSTATES; i++)
	{
		xStream >> MessageBlock[i];
	}

	xStream.Skip4BytePadding(); // replaces pcChunkBuffer += sizeof(u_short)
}

Asura_ClientEntity_Trigger::~Asura_ClientEntity_Trigger()
{
	Asura_Entity_Client::RemoveFromUpdateableClientList(this, false);
}

void Asura_ClientEntity_Trigger::Update()
{
    if( IsDestroyed() ) { return; }

	if (IsActive())
	{
		if(IsTriggered())
		{
			OnTrigger();
		}
		else if (IsUnTriggered())
		{
			OnUnTrigger();
		}
	}
}

void Asura_ClientEntity_Trigger::OnActivation()
{
	Asura_ClientEntity_Activatable::OnActivation();
	SendMessages(ASURA_TRIGGER_ENTITY_ONACTIVATION);
	Reset();
}

void Asura_ClientEntity_Trigger::OnDeactivation()
{
	Asura_ClientEntity_Activatable::OnDeactivation();
	SendMessages(ASURA_TRIGGER_ENTITY_ONDEACTIVATION);
}

void Asura_ClientEntity_Trigger::OnTrigger()
{
	Asura_ClientEntity_Activatable::OnTrigger();
	SendMessages(ASURA_TRIGGER_ENTITY_ONBEINGTRIGGERED);
}

void Asura_ClientEntity_Trigger::OnDestruction()
{
	SendMessages(ASURA_TRIGGER_ENTITY_ONDESTRUCTION);
	Asura_ClientEntity_Activatable::OnDestruction();
}

void Asura_ClientEntity_Trigger::OnUnTrigger()
{
	Asura_ClientEntity_Activatable::OnUnTrigger();
	SendMessages(ASURA_TRIGGER_ENTITY_ONUNTRIGGER);
}

void Asura_ClientEntity_Trigger::SendMessages(enum ASURA_TRIGGER_ENTITY_STATE state)
{
	if(SendMessagesOnClient())
	{
		if (MessageBlock[state]!=ASURA_STATICMESSAGE_EMPTYBLOCK)
		{
			Asura_StaticMessage_System::SendMessages(MessageBlock[state],Guid);
		}
	}
}
