//
// Asura_ServerEntity_ClientVolumeTrigger.cpp
//
// Created by Richard Rice - 22/11/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Entity/Triggers/Asura_ServerEntity_ClientVolumeTrigger.h"

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Network.h"
#include "Network/Asura_Message.h"

Asura_ServerEntity_ClientVolumeTrigger::Asura_ServerEntity_ClientVolumeTrigger(const Asura_Bounding_Box& xBB)
:Asura_ServerEntity_AdvancedVolumeTrigger(xBB,true)
{
	Classification = AsuraEntityClass_ClientVolumeTrigger;
}

Asura_ServerEntity_ClientVolumeTrigger::Asura_ServerEntity_ClientVolumeTrigger(Asura_Guid uGuid)
:Asura_ServerEntity_AdvancedVolumeTrigger(uGuid)
{
	Classification = AsuraEntityClass_ClientVolumeTrigger;
}

Asura_ServerEntity_ClientVolumeTrigger::Asura_ServerEntity_ClientVolumeTrigger(Asura_Guid uGuid,const char*& pcChunkBuffer)
:Asura_ServerEntity_AdvancedVolumeTrigger(uGuid,pcChunkBuffer)
{
	Classification = AsuraEntityClass_ClientVolumeTrigger;

	//check the version
	int iVersion;
	ASURA_CHUNK_EXTRACT( pcChunkBuffer, iVersion, int );
    Asura_Assert( iVersion <= ChunkData::CURRENT_VERSION,"Asura_ServerEntity_ClientVolumeTrigger - incompatible chunk version number");

}

void Asura_ServerEntity_ClientVolumeTrigger::CreateFromChunkBuffer(Asura_Guid uGuid, const char*& pcChunkBuffer)
{
#ifdef TEST_ENTITY_CHUNK_STREAM
	TestStream<Asura_ServerEntity_ClientVolumeTrigger>(uGuid, pcChunkBuffer);
#endif
	new Asura_ServerEntity_ClientVolumeTrigger(uGuid, pcChunkBuffer);
}

void Asura_ServerEntity_ClientVolumeTrigger::OnActivation()
{
	//tell the client to activate
    Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_ACTIVATE,0);
    pxMessage->To = ASURA_GUID_MY_CLIENT_ENTITIES;
    pxMessage->From = Guid;
    Asura_Network::Send(pxMessage);

}

void Asura_ServerEntity_ClientVolumeTrigger::OnDeactivation()
{
	//tell the client to deactivate
    Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_DEACTIVATE,0);
    pxMessage->To = ASURA_GUID_MY_CLIENT_ENTITIES;
    pxMessage->From = Guid;
    Asura_Network::Send(pxMessage);
}

int Asura_ServerEntity_ClientVolumeTrigger::GetChunkBufferSize()
{
	return Asura_ServerEntity_AdvancedVolumeTrigger::GetChunkBufferSize() + sizeof(int);
}

void Asura_ServerEntity_ClientVolumeTrigger::FillInChunkBuffer(char*& pcChunkBuffer)
{
	//fill in base class
	Asura_ServerEntity_AdvancedVolumeTrigger::FillInChunkBuffer( pcChunkBuffer );

	int iVersion = ChunkData::CURRENT_VERSION;
	ASURA_CHUNK_FILLANDVERIFY( pcChunkBuffer, iVersion, int );
}

void Asura_ServerEntity_ClientVolumeTrigger::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// Ordering needs correcting
	Asura_ServerEntity_AdvancedVolumeTrigger::WriteToChunkStream( xStream );

	int iVersion = ChunkData::CURRENT_VERSION;
	xStream << iVersion;
}

void Asura_ServerEntity_ClientVolumeTrigger::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	// Ordering needs correcting
	Asura_ServerEntity_AdvancedVolumeTrigger::ReadFromChunkStream( xStream );

	int iVersion = 0;
	xStream >> iVersion;
	Asura_Assert( iVersion <= ChunkData::CURRENT_VERSION,"Asura_ServerEntity_ClientVolumeTrigger - incompatible chunk version number");
}
