//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_CountedVolumeTrigger.h
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_COUNTEDVOLUMETRIGGER_H___
#define ___ASURA_SERVERENTITY_COUNTEDVOLUMETRIGGER_H___ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// includes
//
//////////////////////////////////////////////////////////////////////////////

// asura
#include "Core/Asura_Maths.h"
#include "Core/Asura_OrientedVolume.h"
#include "Entity/Triggers/Asura_ServerEntity_Trigger.h"

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
// flag values for use with tickboxes... must correspond to the bitfield positions below
#define ASURACOUNTEDVOLUMETRIGGERFLAG_TRIGGERONCE 0x00000002
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_CountedVolumeTrigger
//
// Keeps a count of things entering/leaving the volume.
// Can trigger on every thing or multiples of things entering or leaving.
// Can also trigger at specific counts.
//
// Note that much functionality is expected to be provide by the project
// specific overriding class.
// Storage for the relevant hashed lists, or whatever, should be declared.
// The chunking for these triggers should also be done at project level,
// as these cannot exist at Asura level.
// Also, the count of whatever it is we're counting should be returned by the
// project specific version of GetTriggerObjectCount.
//
//////////////////////////////////////////////////////////////////////////////

class Asura_ServerEntity_CountedVolumeTrigger : public Asura_ServerEntity_Trigger
{
    typedef Asura_ServerEntity_Trigger PARENT;

    friend class Tools_Entity_CountedVolumeTrigger;
    friend class Tools_CountedVolumeTrigger_VolumeEdit_Controller;

public:
    // constructor from tools
    Asura_ServerEntity_CountedVolumeTrigger( const Asura_Bounding_Box& xBB, bool bInitiallyActive = false );
    // constructor from load
    Asura_ServerEntity_CountedVolumeTrigger( Asura_Guid uGuid, const char*& pcChunkBuffer );
    virtual ~Asura_ServerEntity_CountedVolumeTrigger();

    int                         GetNumberOfVolumes() const     { return m_iNumVolumes; }
    const Asura_OrientedVolume& GetVolume( const int i ) const { return m_pxVolumes[ i ]; }
    // checks to see if a point lies in any of the volumes associated with this trigger
    bool                        Contains( const Asura_Vector_3& xPosition ) const;

#ifdef ASURA_TOOLS
    virtual void ApplyChangesForTemplateInstance( class Tools_EntityTemplateInstance* pxInstance );
#endif

protected:
    virtual int  GetChunkBufferSize();
    virtual void FillInChunkBuffer( char*& pcChunkBuffer );

    // to be overridden by derived project specific classes
    virtual unsigned int GetTriggerObjectCount() { return 0; }

    virtual void Update();

    virtual bool IsTriggered();
    virtual bool IsUnTriggered();
    virtual void OnTrigger();
    virtual void OnUnTrigger();

protected:
    enum
    {
        INCREMENT_TRIGGER_MULTIPLE_INDEX,
        DECREMENT_TRIGGER_MULTIPLE_INDEX,
        LIMIT_TRIGGER_INDEX,
        NUM_TRIGGER_TYPES
    };

    // flags to show which project specific lists of things we should check
    int                   m_iListFlags;
    int                   m_iNumVolumes;
    Asura_OrientedVolume* m_pxVolumes;
    // we use the parent's message block storage for activation, deactivation and destruction messages
    // we allocate our own storage for the various triggers
    unsigned short        m_ausMultipleTriggerMessageBlocks[ NUM_TRIGGER_TYPES ];

    // increment/decrement count multiple at which we should trigger (0 = off)
    unsigned int          m_uIncrementTriggerMultiple;
    unsigned int          m_uDecrementTriggerMultiple;
    // limit at which we should trigger
    unsigned int          m_uTriggerCount;
    union
    {
        struct
        {
            unsigned int m_bHasTriggered : 1,
                         m_bTriggerOnce  : 1, // trigger is deactivated when un-triggered (so it only gets triggered once per activation)
                         A_SE_CVT_Unused : 30;
        };
        unsigned int m_uFlags;
    };

private:
    // count of whatever it is we're counting at the project level
    unsigned int m_uPrevCount;
    unsigned int m_uCurrentCount;
    enum
    {
        INCREMENT_TRIGGER = 0x00000001,
        DECREMENT_TRIGGER = 0x00000002,
        LIMIT_TRIGGER     = 0x00000004
    };
    unsigned int m_uTriggerType;

private:
    struct ChunkData
    {
        enum { CURRENT_VERSION = 0 };

        int          m_iListFlags;
        int          m_iNumVolumes;
        int          m_iNumMessageBlocks;
        unsigned int m_uIncrementTriggerMultiple;
        unsigned int m_uDecrementTriggerMultiple;
        unsigned int m_uTriggerCount;
        unsigned int m_uFlags;
        unsigned int m_uPrevCount;
        unsigned int m_uCurrentCount;

        // this is followed by m_iNumVolumes 'Asura_Bounding_Box's
        // then followed by m_iNumMessageBlocks unsigned shorts
        // then followed by padding so that the size is a multiple of 4 bytes
    };
};

//////////////////////////////////////////////////////////////////////////////
//
//
//
//////////////////////////////////////////////////////////////////////////////

#endif
