//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_DebugMessageTrigger.h
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Description:
//
//     trigger that displays a debug message on screen when it it triggered
//
// History:
//
//     (JCS  - 08.12.2005) - Created as a copy of Asura_ServerEntity_CountedTrigger.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////



#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_DEBUGMESSAGETRIGGER_H___
#define ___ASURA_SERVERENTITY_DEBUGMESSAGETRIGGER_H___ 1

#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER


#include "Asura_ServerEntity_Trigger.h"
#ifdef ASURA_TOOLS
#include "Entity/Asura_Entity_Classification.h"
#endif
#include "Core/Asura_String.h"


struct Asura_ServerEntity_DebugMessageTrigger_Unit
{
	float	fPreDisplayTime;
	float	fTimeToDisplayString;
	bool	bTextActive;
};

class Asura_ServerEntity_DebugMessageTrigger : public Asura_ServerEntity_Trigger
{
	friend class Tools_Entity_DebugMessageTrigger_Object;

public:

	Asura_ServerEntity_DebugMessageTrigger( void );
	// constructor from load
	Asura_ServerEntity_DebugMessageTrigger( Asura_Guid uGuid );

	// override of HasClientEntities() - most triggers don't, but we do.
	virtual bool HasClientEntities() const { return true; }

	static void Create( Asura_Guid uGuid, Asura_Chunk_Stream& xStream );

#ifdef ASURA_TOOLS
	virtual bool CanBeIncludedInTemplate() const {return Classification == AsuraEntityClass_DebugMessageTrigger;}
#endif


	static void SetEnableMessages(bool bEnable) { s_bEnableMessages = bEnable; }
	static bool GetEnableMessages(void) { return s_bEnableMessages; }
	static void AddConsoleCommands( void );

protected:

	virtual bool HandleMessage(const Asura_Message* pxMessage);
	virtual void Update();

	bool IsTriggered(void);
	void OnTrigger();
	void OnUnTrigger();
	void InitData();

	//virtual function overides for saving
	virtual	bool IsFilledFromChunkBuffer( void ) const { return false; }
	virtual	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	static u_int m_uStringDisplayCount;

	Asura_String m_xStringToDisplay;
	float	m_fPreDisplayTime;
	float	m_fTimeToDisplayString;
	u_int	m_uDisplayOffset;

	float	m_fTimer;
	float	m_fPreDisplayTimer;
	bool	m_bDisplayingString;

	float	m_fR;
	float	m_fG;
	float	m_fB;

	int		m_iDrawInWorld;

	static bool s_bEnableMessages;

	// Console cmds
	static bool ConsoleCmd_DebugMessageTriggerEnable( bool bEnable, bool bInterrogate, bool bHelp );

private:
	typedef Asura_ServerEntity_Trigger PARENT;
};



//  These classes are used to buffer the messages we want to render this frame:

#define ASURA_DEBUGMESSAGETRIGGER_MAXMESSAGES 32

struct Asura_DebugMessageTrigger_Message
{
	float	 fX;
	float	 fY;
	uint32	 uColor;
	char	*szString;
};


class Asura_DebugMessageTrigger_Messages
{

public:
	static void Reset();
	static void Render(int iFontHandle);
	static void Render();
	static void BufferDebugMessage(float fX, float fY, uint32 uColor, char *szString);
	static void SetFont( int iFontHandle ) { s_iFontHandle = iFontHandle; }

private:	
	static int								 s_iFontHandle;
	static int								 s_iNumDebugMessages;
	static Asura_DebugMessageTrigger_Message s_axDebugMessages[ASURA_DEBUGMESSAGETRIGGER_MAXMESSAGES];
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif //ASURA_USE_DEBUG_MESSAGE_TRIGGER

#endif // #ifndef ___ASURA_SERVERENTITY_DEBUGMESSAGETRIGGER_H___

