//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_DebugWaypointTrigger.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// Description:
//
//		Copied from Asura_ServerEntity_DebugMessageTrigger and extended
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_DEBUGWAYPOINTTRIGGER_H___
#define ___ASURA_SERVERENTITY_DEBUGWAYPOINTTRIGGER_H___ 1

#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER

#include "Entity/Triggers/Asura_ServerEntity_Trigger.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Core/Asura_String.h"

enum {eASURA_DEBUGWAYPOINTTRIGGER_TEXTLEN = 41};

struct Asura_ServerEntity_DebugWaypointTrigger_Unit
{
	float	fTimeToDisplayString;
	bool	bTextActive;
};

class Asura_Message_DebugWaypoint_Create
{
	public:
	
	//Shape and animation
	Asura_Hash_ID m_uShapeID;
	Asura_Hash_ID m_uAnimID;

	//Position & Orientation
	Asura_Vector_3 m_xPosition;
	Asura_Matrix_3x3 m_xOrientation;

	//Triggering
	float m_fTriggerRadius;
	float m_fTimeToDisplayString;

	float m_fR;
	float m_fG;
	float m_fB;

	union
	{
		u_int m_uFlags;
		struct
		{
			u_int	m_bUseRadiusTriggering	: 1,
					m_bHidden				: 1,
					m_bAlwaysShowTitle		: 1,
					m_bUseLookAtTest		: 1,
					m_bUseLOSOcclusion		: 1,
					m_bActive				: 1,
					m_bSkin					: 1,
											: 0;
		};
	};

	char m_szTitle[eASURA_DEBUGWAYPOINTTRIGGER_TEXTLEN];
	char m_szText[8][eASURA_DEBUGWAYPOINTTRIGGER_TEXTLEN];
};

class Asura_Message_DebugWaypoint_ClientUpdate
{
public:
	u_int	GetFlags() const				{ return m_uFlags; }
	void	SetFlags(u_int uFlags)			{ m_uFlags = uFlags; }
protected:
	u_int m_uFlags; 
};

class Asura_ServerEntity_DebugWaypointTrigger : public Asura_ServerEntity_Trigger
{
	friend class Tools_Entity_DebugWaypointTrigger_Object;

public:

	//__________ Functions _________
	Asura_ServerEntity_DebugWaypointTrigger( void );
	Asura_ServerEntity_DebugWaypointTrigger( Asura_Guid uGuid );
	~Asura_ServerEntity_DebugWaypointTrigger();

	static void Create( Asura_Guid uGuid, Asura_Chunk_Stream& xStream );

	#ifdef ASURA_TOOLS
	virtual bool CanBeIncludedInTemplate() const { return Classification == AsuraEntityClass_DebugWaypointTrigger; }
	virtual void ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance);
	virtual void TraverseUsedShapes(void (*pfnCallback)(Asura_Hash_ID uShapeHash,void* pUserData), void* pUserData) const; // Callback to see what shapes are used by this entity

	#endif //#ifdef ASURA_TOOLS
	virtual void OnDestruction();

	static void SetEnableMessages(bool bEnable) { s_bEnableMessages = bEnable; }
	static bool GetEnableMessages(void) { return s_bEnableMessages; }
	static void AddConsoleCommands( void );
	static bool GetRenderTriggerShapes( void );

	float GetTriggerRadius() const { return m_xClientData.m_fTriggerRadius; }
	void SetTriggerRadius( const float fNewRadius ) { m_xClientData.m_fTriggerRadius = fNewRadius; }

	Asura_Hash_ID GetShapeID() const { return m_xClientData.m_uShapeID; }	

	void SetShape( Asura_Hash_ID uShape );
	void SetSkin( Asura_Hash_ID uSkin );
	void SetAnim( Asura_Hash_ID uAnim );
	
	static void SetDefaultShapeName( const char* const pszDefaultShapeName );
	static void SetDefaultShapeID( const Asura_Hash_ID uDefaultShapeID );
	static Asura_Hash_ID GetDefaultShapeID();

	//_______ Getters/Setters _______

	void SetHidden( const bool bHidden ) { m_bHidden = bHidden; }

	static void SetTriggeringGuid(Asura_Guid uGuid) { s_uTriggeringGuid = uGuid; }

protected:

	//________ Enumerations _________
	
	enum { CURRENT_VERSION = 15 };


	//_________ Functions ____________
	
	virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual void Update();
	virtual bool GetHasCreateMessage() const { return true; }
	virtual void MakeCreateMessage(Asura_Guid uRequestingClient, bool bSendGuaranteed = false);
	virtual void MakeClientUpdateMessage(); 

	bool IsTriggered( void );
	void OnTrigger( void );
	void OnUnTrigger( void );
	virtual void OnActivation();
	virtual void OnDeactivation();

	void InitData( void );

	// virtual function overides for saving - using streams now
	virtual	bool IsFilledFromChunkBuffer( void ) const { return false; }
	virtual	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	// Console cmds
	static bool ConsoleCmd_DebugWaypointTriggerEnable( bool bEnable, bool bInterrogate, bool bHelp );

	//_________ Variables __________

	static u_int s_uStringDisplayCount;

	Asura_String m_xTitleString;
	Asura_String m_xStringToDisplay[8];

	// Flags we save to the chunk stream. If you add new flags here remember to add them to the bitfield wrapper in WriteToChunkStream
	union
	{
		u_int m_uChunkedFlags;
		struct
		{
			u_int	m_bUseRadiusTriggering	: 1,
					m_bAlwaysShowTitle		: 1,
					m_bUseLookAtTest		: 1,
					m_bUseLOSOcclusion		: 1,
					m_bSkin					: 1,
											: 0;
		};
	};

	// Unsaved state flags. New flags can be added here without needing further handling
	union
	{
		u_int m_uUnchunkedFlags;
		struct
		{
			u_int	m_bPlayerInVolume	: 1,
					m_bTriggered		: 1,
					m_bHidden			: 1,
										: 0;
		};
	};

	// This stores most of the data for the server too (although some things are copied, notably the strings and some of the flags).
	Asura_Message_DebugWaypoint_Create m_xClientData;

	static Asura_Guid s_uTriggeringGuid;
	static bool s_bEnableMessages;

private:
	
	void UpdateClientFlags();
	void UpdateClientStrings();
	
	typedef Asura_ServerEntity_Trigger PARENT;

};


//  These classes are used to buffer the messages we want to render this frame:

#define ASURA_DEBUGWAYPOINTTRIGGER_MAXMESSAGES 128

struct Asura_DebugWaypointTrigger_Message
{
	Asura_Vector_2	xScreenPos;
	float			fXOffset;
	float			fYOffset;
	uint32			uColor;
	char*			szString;
};


class Asura_DebugWaypointTrigger_Messages
{

public:
	static void Reset();
	static void Render(int iFontHandle);
	static void Render();
	static void BufferDebugMessage( const Asura_Vector_3& xScreenPos, const float fYOffset, uint32 uColor, char *szString );	
	static void SetFont(int iFontHandle) { s_iFontHandle = iFontHandle; }

private:
	static int								  s_iFontHandle;
	static int								  s_iNumDebugMessages;
	static Asura_DebugWaypointTrigger_Message s_axDebugMessages[ASURA_DEBUGWAYPOINTTRIGGER_MAXMESSAGES];
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif //ASURA_USE_DEBUG_MESSAGE_TRIGGER

#endif // #ifndef ___ASURA_SERVERENTITY_DEBUGWAYPOINTTRIGGER_H___

