//
// Asura_ServerEntity_LogicTrigger.cpp
//
// Created by Richard Rice - 07/10/2004
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"

// This file's header.
#include "Entity/Triggers/Asura_ServerEntity_LogicTrigger.h"

// Engine independent.
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_StaticMessage_System.h"
#ifdef ASURA_SUPPORT_ENTITYWATCH
#include "Entity/Asura_EntityWatch_System.h"
#endif

#ifdef ASURA_TOOLS
#include "Entity/Tools_Message_Editing.h"
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif



#define ASURA_SERVERENTITY_LOGICTRIGGER_VERSION_NUMBER 2

//////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------**
** Asura_LogicTriggerInput **
**-------------------------*/

void Asura_LogicTriggerInput::SetDefaultValues()
{
	m_uFlags = 0;
	m_bActive = 0;

	#ifdef ASURA_TOOLS
	m_xName = "";
	#endif
}

//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Asura_ServerEntity_LogicTrigger **
**---------------------------------*/

Asura_Hashed_List<Asura_ServerEntity_LogicTrigger> Asura_ServerEntity_LogicTrigger::s_xList(5);


Asura_ServerEntity_LogicTrigger::Asura_ServerEntity_LogicTrigger()
:Asura_ServerEntity_Activatable()
{
	s_xList.AddToList(this);

	Classification = AsuraEntityClass_LogicTrigger;

	m_usMessageBlockOn = ASURA_STATICMESSAGE_EMPTYBLOCK;
	m_usMessageBlockOff = ASURA_STATICMESSAGE_EMPTYBLOCK;
	m_uLogicFlags = ASURA_LOGICTRIGGERFLAG_TRIGGER_ON_STATE_CHANGE;
	
	m_uNumberOfInputs = 0;
	m_pxInputs = 0;
	m_uType = ASURA_LOGICTRIGGER_AND;
}

Asura_ServerEntity_LogicTrigger::Asura_ServerEntity_LogicTrigger(Asura_Guid uGuid)
:Asura_ServerEntity_Activatable(uGuid)
{
	s_xList.AddToList(this);
	Classification = AsuraEntityClass_LogicTrigger;

	m_usMessageBlockOn = ASURA_STATICMESSAGE_EMPTYBLOCK;
	m_usMessageBlockOff = ASURA_STATICMESSAGE_EMPTYBLOCK;
	m_uLogicFlags = ASURA_LOGICTRIGGERFLAG_TRIGGER_ON_STATE_CHANGE;
	
	m_uNumberOfInputs = 0;
	m_pxInputs = 0;
	m_uType = ASURA_LOGICTRIGGER_AND;
}

Asura_ServerEntity_LogicTrigger::~Asura_ServerEntity_LogicTrigger()
{
	s_xList.RemoveFromList(this);

	delete [] m_pxInputs;
}


void Asura_ServerEntity_LogicTrigger::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	PARENT::ReadFromChunkStream(xStream);
	
	int iVersion;
	xStream >> iVersion;
	Asura_Assert(iVersion>=1 && iVersion <= ASURA_SERVERENTITY_LOGICTRIGGER_VERSION_NUMBER,"Unknown chunk version");

	xStream >> m_uType;
	xStream >> m_uNumberOfInputs;


	xStream >> m_usMessageBlockOn;
	xStream >> m_usMessageBlockOff;

	if(iVersion >= 2)
	{
		xStream >> m_uLogicFlags;
	}


	m_pxInputs = NULL;
	if (m_uNumberOfInputs)
	{
		m_pxInputs = new Asura_LogicTriggerInput[m_uNumberOfInputs];
		for(unsigned i=0;i<m_uNumberOfInputs;i++)
		{
			xStream >> m_pxInputs[i].m_uHashID;
			xStream >> m_pxInputs[i].m_uFlags;
		}
	}

	int iToolsDataSize;
	xStream >> iToolsDataSize;

	#ifdef ASURA_TOOLS
	if(iToolsDataSize)
	{
		for(u_int i=0;i<m_uNumberOfInputs;i++)
		{
			xStream >> m_pxInputs[i].m_xName;
		}
	}
	else
	{
		for(u_int i=0;i<m_uNumberOfInputs;i++)
		{
			m_pxInputs[i].m_xName = "";
		}

	}
	#else
	//not in tools , so just skip this
	if(iToolsDataSize)
	{
		xStream.Skip(iToolsDataSize);
	}
	#endif // ASURA_TOOLS	

	// Prevent us from ever updating...
	SetNeverUpdates(true);
}

void Asura_ServerEntity_LogicTrigger::Create( Asura_Guid uGuid, Asura_Chunk_Stream& xStream )
{
	Asura_ServerEntity_LogicTrigger* pxTrigger = new Asura_ServerEntity_LogicTrigger(uGuid);

	if( pxTrigger )
	{
		pxTrigger->ReadFromChunkStream(xStream);
	}
}

#ifdef ASURA_TOOLS
int Asura_ServerEntity_LogicTrigger::GetToolsDataSize() const
{
	int iChunkSize = 0;
	for(u_int i=0;i<m_uNumberOfInputs;i++)
	{
		iChunkSize += m_pxInputs[i].m_xName.GetSizeInInts()*4;
	}
	return iChunkSize;
}
#endif //ASURA_TOOLS



void Asura_ServerEntity_LogicTrigger::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	/*
	This entity type can be used in template files , so any changes to FillInChunkBuffer may
	require ApplyChangesForTemplateInstance() to be updated as well.
	Please see the comment at the top of Tools/EntityTemplate/Tools_EntityTemplate_System.h before 
	making any changes.
	RAR 29/04/2004
	*/
	PARENT::WriteToChunkStream(xStream);

	int iVersion =   ASURA_SERVERENTITY_LOGICTRIGGER_VERSION_NUMBER;
	xStream << iVersion;
	xStream << m_uType;
	xStream << m_uNumberOfInputs;

	xStream << m_usMessageBlockOn;
	xStream << m_usMessageBlockOff;

	xStream << m_uLogicFlags;
	
	for(unsigned i=0;i<m_uNumberOfInputs;i++)
	{
		xStream << m_pxInputs[i].m_uHashID;

		Asura_WriteToStream_Bitfield_Wrapper<u_int> xBitfieldWrapper( m_pxInputs[i].m_uFlags );
		xBitfieldWrapper.Add( m_pxInputs[i].m_bActive, 1 );

		xStream << xBitfieldWrapper;

	}

	#ifdef ASURA_TOOLS
	if(!Tools_Level_Export::IsExport())//exported levels won't want the tools data
	{
		//save out data only needed by tools
		int iToolsDataSize = GetToolsDataSize();

		xStream << iToolsDataSize;

		for(u_int i=0;i<m_uNumberOfInputs;i++)
		{
			xStream << m_pxInputs[i].m_xName;
		}
	}
	else
	#endif
	{
		//Either not in tools , or doing a level export.
		//In either case , we don't want the tools data.
		int iToolsDataSize = 0;
		xStream << iToolsDataSize;
	}

}


bool Asura_ServerEntity_LogicTrigger::HandleMessage(const Asura_Message* pxMessage)
{
	if (!pxMessage) return false;

	switch(pxMessage->GetMsgID())
	{
		//these use the same message structure  so deal with them at the same time
		case ASURA_MSG_LOGICTRIGGER_INPUTON :
		case ASURA_MSG_LOGICTRIGGER_INPUTOFF :
		{
			const Asura_Message_LogicTrigger_Input* pxMessageData = reinterpret_cast
				<const Asura_Message_LogicTrigger_Input*>(pxMessage->GetData());
			
			//search for a matching input
			for(u_int i=0;i<m_uNumberOfInputs;i++)
			{
				if(m_pxInputs[i].m_uHashID == pxMessageData->m_uInputHash)
				{
					bool bOldState = EvaluateState();
					m_pxInputs[i].m_bActive = (pxMessage->GetMsgID()==ASURA_MSG_LOGICTRIGGER_INPUTON);
					bool bNewState = EvaluateState();

					//if the state changed , send the appropriate messages
					if(m_uLogicFlags & ASURA_LOGICTRIGGERFLAG_TRIGGER_ON_STATE_CHANGE)
					{
						if(bNewState != bOldState && IsActive())
						{
							if(bNewState)
							{
								if(ASURA_STATICMESSAGE_EMPTYBLOCK != m_usMessageBlockOn)
								{
									Asura_StaticMessage_System::SendMessages(m_usMessageBlockOn,Guid);
								}
							}
							else
							{
								if(ASURA_STATICMESSAGE_EMPTYBLOCK != m_usMessageBlockOff)
								{
									Asura_StaticMessage_System::SendMessages(m_usMessageBlockOff,Guid);
								}
							}
						}
					}
					break;
				}
			}

			return true;
		}

		case ASURA_MSG_TRIGGER :
		{
			//'Trigger' causes the messages for the current state to be sent (if active)
			if(IsActive())
			{
				bool bState = EvaluateState();

				if(bState)
				{
					if(ASURA_STATICMESSAGE_EMPTYBLOCK != m_usMessageBlockOn)
					{
						Asura_StaticMessage_System::SendMessages(m_usMessageBlockOn,Guid);
					}
				}
				else
				{
					if(ASURA_STATICMESSAGE_EMPTYBLOCK != m_usMessageBlockOff)
					{
						Asura_StaticMessage_System::SendMessages(m_usMessageBlockOff,Guid);
					}
				}
			}
			return true;
		}
		
		default :
			break;
	}

	// Pass to parent.
	return Asura_ServerEntity_Activatable::HandleMessage(pxMessage);
}



#ifdef ASURA_TOOLS
void Asura_ServerEntity_LogicTrigger::ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance)
{
	Asura_ServerEntity_Activatable::ApplyChangesForTemplateInstance(pxInstance);

	//update the message blocks
	m_usMessageBlockOn = pxInstance->ConvertMessageBlock(m_usMessageBlockOn);
	m_usMessageBlockOff = pxInstance->ConvertMessageBlock(m_usMessageBlockOff);

}
#endif // ASURA_TOOLS


bool Asura_ServerEntity_LogicTrigger::EvaluateState()
{
	if(!m_uNumberOfInputs) return false;

	bool m_bResult = false;

	switch(m_uType)
	{
		case ASURA_LOGICTRIGGER_AND :
			m_bResult = m_pxInputs[0].m_bActive;
			for(u_int i=1;i<m_uNumberOfInputs;i++)
			{
				m_bResult = m_bResult && m_pxInputs[i].m_bActive;
			}
			break;

		case ASURA_LOGICTRIGGER_OR :
			m_bResult = m_pxInputs[0].m_bActive;
			for(u_int i=1;i<m_uNumberOfInputs;i++)
			{
				m_bResult = m_bResult || m_pxInputs[i].m_bActive;
			}
			break;

		case ASURA_LOGICTRIGGER_XOR :
		{
			m_bResult = m_pxInputs[0].m_bActive;
			for(u_int i=1;i<m_uNumberOfInputs;i++)
			{
				if(m_pxInputs[i].m_bActive) m_bResult = !m_bResult;
			}
			break;
		}

		default :
			Asura_Assert(0,"Unexpected logic trigger type");
			break;
	}
	return m_bResult;
}

#ifdef ASURA_TOOLS
void Asura_ServerEntity_LogicTrigger::GetCustomEventName(int iEventIndex, Asura_String& xEventName)
{
	switch(iEventIndex)
	{
		case 0:	xEventName = "On Messages";		return;
		case 1:	xEventName = "Off Messages";	return;
	}

	Asura_Assert(FALSE, "Invalid logictrigger event index");

	xEventName = "";
}
#endif // ASURA_TOOLS

#ifdef ASURA_TOOLS
unsigned short* Asura_ServerEntity_LogicTrigger::GetCustomEvent(int iIndex)
{
	switch(iIndex)
	{
		case 0:	return &m_usMessageBlockOn;
		case 1:	return &m_usMessageBlockOff;
	}

	Asura_Assert(FALSE, "Invalid logictrigger event index");
	return NULL;
}
#endif // ASURA_TOOLS

#ifdef ASURA_TOOLS
const char* Asura_ServerEntity_LogicTrigger::GetMessageDescription(const Asura_StaticMessage* pxMessage)
{
	if(pxMessage->m_usMsgID != ASURA_MSG_LOGICTRIGGER_INPUTON &&
		pxMessage->m_usMsgID != ASURA_MSG_LOGICTRIGGER_INPUTOFF) return 0;

	const Asura_Message_LogicTrigger_Input* pxData = (const Asura_Message_LogicTrigger_Input*) pxMessage->m_pData;


	//Find the entity being triggered
	Asura_ServerEntity_LogicTrigger* pxEntity = FindEntity(Tools_Message_Editing::GetFinalDestinationGuid(pxMessage));
	if(!pxEntity) return 0;

	static char szText[100];
	for(u_int i=0;i<pxEntity->m_uNumberOfInputs;i++)
	{
		if(pxData->m_uInputHash == pxEntity->m_pxInputs[i].m_uHashID)
		{
			if(pxMessage->m_usMsgID == ASURA_MSG_LOGICTRIGGER_INPUTON)
				Asura_PrintfToBfr(szText,"Input On(%s)",100,true,pxEntity->m_pxInputs[i].m_xName.GetString());
			else
				Asura_PrintfToBfr(szText,"Input Off(%s)",100,true,pxEntity->m_pxInputs[i].m_xName.GetString());
			return szText;
		}
	}
	return 0;
}
#endif //ASURA_TOOLS

#ifdef ASURA_SUPPORT_ENTITYWATCH
void Asura_ServerEntity_LogicTrigger::DisplayEntityWatchText()
{
	Asura_ServerEntity_Activatable::DisplayEntityWatchText();
	Asura_EntityWatch_System::PrintText(16,"State : %s",EvaluateState() ? "On" : "Off");

	//get state of all the inputs
	char szInputTextBuffer[100] = "Inputs : ";
	char* pcPos = szInputTextBuffer+strlen(szInputTextBuffer);

	for(u_int i=0;i<m_uNumberOfInputs;i++)
	{
		strcpy(pcPos,m_pxInputs[i].m_bActive ? "On  " : "Off ");
		pcPos += 4;
	}

	Asura_EntityWatch_System::PrintText(strlen(szInputTextBuffer)+2,szInputTextBuffer);
}
#endif
