//
// Asura_ServerEntity_ProxyTrigger.h
//
// Created by Richard Rice - 21/09/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_SERVERENTITY_PROXYTRIGGER_H__
#define __ASURA_SERVERENTITY_PROXYTRIGGER_H__

#include "Core/Asura_LinkList.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_Entity.h"

class Asura_ServerEntity_ProxyTrigger : public Asura_Entity_Server
{
	public :
		#ifdef ASURA_TOOLS
		//for newly created proxy trigger in tools
		Asura_ServerEntity_ProxyTrigger();
		#endif
		~Asura_ServerEntity_ProxyTrigger();

		static void Create(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);

		// override of HasClientEntities() - we don't have any.
		virtual bool HasClientEntities() const { return false; }

		virtual bool HandleMessage(const Asura_Message* pxMessage);


		#ifdef ASURA_SUPPORT_ENTITYWATCH
		//virtual void DisplayEntityWatchText();
		#endif
		
		#ifdef ASURA_TOOLS
		//Returns the guid of one of the targets if there is one.
		//Otherwise attempts to find the guid of any entity of the correct class
		Asura_Guid GetGuidOfTargetClass() const;

		//IsEntityTargeted() returns true if the specified guid is one of the current targets.
		// This doesn't account for changing targets on the fly.
		bool IsEntityTargeted( Asura_Guid uGuid ) const;
		#endif

	protected :

		virtual bool CanBeSavedToChunk() const {return true;}
		virtual	bool IsFilledFromChunkBuffer(void) const{ return (false); }
		virtual	void ReadFromChunkStream (Asura_Chunk_Stream& xStream);
		virtual	void WriteToChunkStream (Asura_Chunk_Stream& xStream) const;

		#ifdef ASURA_TOOLS
		virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
		virtual bool CanBeIncludedInTemplate() const {return Classification == AsuraEntityClass_ProxyTrigger;}
		#endif


	protected:
		//constructor from load
		Asura_ServerEntity_ProxyTrigger(Asura_Guid uGuid);

		void Initialise();

		void ForwardMessage(const class Asura_Message* pxMessage, Asura_Guid uToGuid);

		int m_iTargetClass;//entity class that this is a proxy for
		Asura_LinkList<Asura_Guid> m_xTargetList;
		u_int m_uProxyFlags;//no flags at the moment


	#ifdef ASURA_TOOLS
	public:
		static Asura_ServerEntity_ProxyTrigger* FindEntity(Asura_Guid uGuid)
		{
			return s_xList.FindEntity(uGuid);
		}

		static const Asura_Hashed_List<Asura_ServerEntity_ProxyTrigger>* GetList()
		{
			return &s_xList;
		}


	private:
		static Asura_Hashed_List<Asura_ServerEntity_ProxyTrigger> s_xList;
	#endif // ASURA_TOOLS


		friend class Tools_Entity_ProxyTrigger_Object;
	
};

class Asura_Message_ProxyTrigger_Target
{
public :
	Asura_Guid m_uTargetGuid;
};



#endif //__ASURA_SERVERENTITY_PROXYTRIGGER_H__
