//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_SoundController.h
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Description:
//
//     entities which control phonons
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SERVERENTITY_SOUNDCONTROLLER_H___
#define ___ASURA_SERVERENTITY_SOUNDCONTROLLER_H___ 1

#include "Asura_ServerEntity_Activatable.h"
#ifdef ASURA_TOOLS
#include "Entity/Asura_Entity_Classification.h"
#endif


static const int ASURA_SERVERENTITY_SOUNDCONTROLLER_VERSION_NUMBER = 1;

class Asura_ServerEntity_SoundController : public Asura_ServerEntity_Activatable
{

public:
	
	static void CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);

    
    Asura_ServerEntity_SoundController(Asura_Guid uGuid, Asura_Guid uPhononGuid, bool bInitiallyActive);


    ~Asura_ServerEntity_SoundController();

    virtual bool HandleMessage(const Asura_Message* pxMessage);

    // static function to retrieve entity from phonon guid
    static Asura_ServerEntity_SoundController* GetControllerFromPhonon(Asura_Guid uPhononGuid);

    // access to starts active which is set by the phonon
    void SetStartsActive(bool bActive)  
    {
    	SetInternalActiveFlag(bActive);
		SetNeverUpdates(true);
    }

	Asura_Guid GetPhononGuid() const {return m_uPhononGuid;}

	#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate() const {return Classification == AsuraEntityClass_SoundController;}
	#endif

	#ifdef ASURA_TOOLS
	#endif

	static const Asura_Hashed_List<Asura_ServerEntity_SoundController>* GetList()
	{
		return &s_xList;
	}


protected:
	virtual	bool IsFilledFromChunkBuffer(void) const { return false; }


	//virtual function overides for saving
	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

    // Asura_ServerEntity_Activatableoverrides, these just send the message to 
    // the clients. Don't call up to the base though. There should be nothing in 
    // there we need to do.
    virtual void OnActivation();
	virtual void OnDeactivation();
	virtual void OnTrigger();

    // data
    Asura_Guid m_uPhononGuid;
	
private:
	//constructor for initialising from load.
	//Need to distinguish from the version that gets a phonon guid ,so a dummy float param has been added
	Asura_ServerEntity_SoundController(Asura_Guid uGuid, float fDummy);


    // static data
	// The only thing put in will be sound controllers, so there's no advantage in using a classified list
    static Asura_Hashed_List<Asura_ServerEntity_SoundController> s_xList;
	friend class Tools_Phonon_Object;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_SERVERENTITY_SOUNDCONTROLLER_H___
