/**
*
* \file Asura_ServerEntity_TimeTrigger.h
*
* Created by Kevin Lea - 22/04/2002 10:38:17
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/
#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_SERVERENTITY_TIMETRIGGER_H__
#define __ASURA_SERVERENTITY_TIMETRIGGER_H__

#include "Entity/Triggers/Asura_ServerEntity_Trigger.h"
#include "Network/Asura_Guid.h"
#ifdef ASURA_TOOLS
#include "Entity/Asura_Entity_Classification.h"
#endif

#define ASURA_TIMETRIGGERFLAG_LOOP						0x00000001
#define ASURA_TIMETRIGGERFLAG_PAUSE_DURING_CUTSCENES	0x00000002

class Asura_ServerEntity_TimeTrigger : public Asura_ServerEntity_Trigger
{
	public:
		Asura_ServerEntity_TimeTrigger(float time, bool bActive = false);		
		Asura_ServerEntity_TimeTrigger(Asura_Guid uGuid);
		Asura_ServerEntity_TimeTrigger(Asura_Guid guid,const char*& chunkBuffer); //constructor from load
		~Asura_ServerEntity_TimeTrigger();

		static void CreateFromChunkBuffer( Asura_Guid uGuid, const char*& pcChunkBuffer );

		virtual void Update();

		#ifdef ASURA_TOOLS
		virtual bool CanBeIncludedInTemplate() const {return Classification == AsuraEntityClass_TimeTrigger;}
		#endif


		#ifdef ASURA_SUPPORT_ENTITYWATCH
		virtual void DisplayEntityWatchText();
		#endif

		bool IsLooped() const {return m_uTimeTriggerFlags & ASURA_TIMETRIGGERFLAG_LOOP;}

		float GetRemainingTime() const { return m_fTimeRemaining; }
		
		float GetElapsedTime() const { return (Time - m_fTimeRemaining); }

	protected:

		bool IsTriggered(void);
		void Reset(void);

		//virtual function overides for saving
		virtual	bool	IsFilledFromChunkBuffer		(void) const	{ return (false); }

		virtual	int		GetChunkBufferSize			(void);
		virtual	void	FillInChunkBuffer			(char*& pcChunkBuffer);
//		virtual	void	InitialiseFromChunkBuffer	(const char*& pcChunkBuffer);

		virtual	void	ReadFromChunkStream			(Asura_Chunk_Stream& xStream);
		virtual	void	WriteToChunkStream			(Asura_Chunk_Stream& xStream) const;

		virtual bool HandleMessage(const Asura_Message* pxMessage);

		float Time; // amount of time delay before triggering
		float m_fTimeRemaining; // Time left to go before being triggered
		u_int m_uTimeTriggerFlags;


		friend class Tools_Entity_TimeTrigger_Object;
			
};

//These messages are saved out to static message system , so care needs to be taken about altering them
//(also used by timeline trigger)
struct Asura_Message_SetTimer
{
	float m_fTimer;
};

struct Asura_Message_TimeTrigger_SetTriggerTime
{
	float m_fTriggerTime;
};

struct Asura_Message_TimeTrigger_AddTime
{
	float m_fTimeDelta;
};

struct Asura_Message_TimeTrigger_AddToTriggerTime
{
	float m_fTriggerTimeDelta;
};


#endif//__ASURA_SERVERENTITY_TIMETRIGGER_H__
