//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_WaypointBlockerVolume.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/Triggers/Asura_ServerEntity_WaypointBlockerVolume.h"

//Asura
#include "AI/Asura_AI_Navigation.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Memory/Asura_Mem.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Message.h"
//Tools
#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_WaypointBlockerVolume
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
u_short*                      Asura_ServerEntity_WaypointBlockerVolume::s_pusWaypoints           = NULL;
u_int                         Asura_ServerEntity_WaypointBlockerVolume::s_uWaypointArrayCapacity = 0;
u_int                         Asura_ServerEntity_WaypointBlockerVolume::s_uNumWaypoints          = 0;
Asura_AI_NavConnection_Index* Asura_ServerEntity_WaypointBlockerVolume::s_pxLinksToBlock         = NULL;
u_int                         Asura_ServerEntity_WaypointBlockerVolume::s_uLinkArrayCapacity     = 0;
u_int                         Asura_ServerEntity_WaypointBlockerVolume::s_uNumLinksToBlock       = 0;
#endif

Asura_Hashed_List<Asura_ServerEntity_WaypointBlockerVolume> Asura_ServerEntity_WaypointBlockerVolume::s_xList(2);

Asura_ServerEntity_WaypointBlockerVolume::Asura_ServerEntity_WaypointBlockerVolume( bool bActive /*= true*/ )
: PARENT( bActive )
, m_fSphericalVolumeRadius( 5.0f )
, m_iNumVolumes( 0 )
, m_pxVolumes( NULL )
, m_uNumLinksToBlock( 0 )
, m_pxLinksToBlock( NULL )
, m_uWBVFlags( 0 )
{
	Classification = AsuraEntityClass_WaypointBlockerVolume;

	s_xList.AddToList(this);
}

Asura_ServerEntity_WaypointBlockerVolume::Asura_ServerEntity_WaypointBlockerVolume( Asura_Guid uGuid )
: PARENT( uGuid )
, m_fSphericalVolumeRadius( 5.0f )
, m_iNumVolumes( 0 )
, m_pxVolumes( NULL )
, m_uNumLinksToBlock( 0 )
, m_pxLinksToBlock( NULL )
, m_uWBVFlags( 0 )
{
	Classification = AsuraEntityClass_WaypointBlockerVolume;

	s_xList.AddToList(this);
}

Asura_ServerEntity_WaypointBlockerVolume::~Asura_ServerEntity_WaypointBlockerVolume()
{
	if ( m_iNumVolumes > 0 )
	{
		delete [] m_pxVolumes;
	}

	if ( m_uNumLinksToBlock )
	{
		delete [] m_pxLinksToBlock;
	}

	s_xList.RemoveFromList(this);
}

void Asura_ServerEntity_WaypointBlockerVolume::PostLoadInitialise()
{
	PARENT::PostLoadInitialise();

	// (TI - 17.07.2008) - Because WBVs are created with the chunk in the constructor
	// this was never getting called so WBVs that started active were not working.
	UpdateWaypointLinkBlocks();

	// This doesn't need to update
	SetNeverUpdates(true);
}

void Asura_ServerEntity_WaypointBlockerVolume::PreDestructionShutdown()
{
	if ( m_bBlockedLinks )
	{
		BlockLinks( false );
	}

	PARENT::PreDestructionShutdown();
}

void Asura_ServerEntity_WaypointBlockerVolume::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	// (TI - 11.10.2007) - Even though the correct way to do chunks is Our Version, Parent, Then our data.  Due to the structure of 
	// the constructor i don't really have a choice but to do Parent, Our Version, Our Data.
	PARENT::ReadFromChunkStream(xStream);

	u_int uVersion;
	xStream >> uVersion;

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_fSphericalVolumeRadius;

		xStream >> m_iNumVolumes;
		if ( m_iNumVolumes > 0 )
		{
			m_pxVolumes = new Asura_OrientedVolume[ m_iNumVolumes ];
			for ( int i = 0; i < m_iNumVolumes; ++i )
			{
				xStream >> m_pxVolumes[ i ].m_xBox;
				xStream >> m_pxVolumes[ i ].m_xOrient;
			}
		}

		xStream >> m_uNumLinksToBlock;
		if ( m_uNumLinksToBlock > 0 )
		{
			m_pxLinksToBlock = new Asura_AI_NavConnection_Index[ m_uNumLinksToBlock ];
			for ( u_int u = 0; u < m_uNumLinksToBlock; ++u )
			{
				xStream >> m_pxLinksToBlock[ u ].m_usWaypointIndex;
				xStream >> m_pxLinksToBlock[ u ].m_usConnectionIndex;
			}
		}

		// (TI - 22.10.2007) - I added 4 byte padding because higher up in the entity hierarchy it makes use of the old 
		// chunkbuffer which do not automatically align themselves.
		xStream.Skip4BytePadding();
	}
	else if ( uVersion == 1 )
	{
		xStream >> m_fSphericalVolumeRadius;

		xStream >> m_iNumVolumes;
		if ( m_iNumVolumes > 0 )
		{
			m_pxVolumes = new Asura_OrientedVolume[ m_iNumVolumes ];
			for ( int i = 0; i < m_iNumVolumes; ++i )
			{
				xStream >> m_pxVolumes[ i ].m_xBox;
				xStream >> m_pxVolumes[ i ].m_xOrient;
			}
		}

		u_char ucNumLinksToBlock;
		xStream >> ucNumLinksToBlock;
		m_uNumLinksToBlock = ucNumLinksToBlock;
		if ( m_uNumLinksToBlock > 0 )
		{
			m_pxLinksToBlock = new Asura_AI_NavConnection_Index[ m_uNumLinksToBlock ];
			for ( u_int u = 0; u < m_uNumLinksToBlock; ++u )
			{
				xStream >> m_pxLinksToBlock[ u ].m_usWaypointIndex;
				xStream >> m_pxLinksToBlock[ u ].m_usConnectionIndex;
			}
		}

		u_int uFlags;
		xStream >> uFlags;

		// (TI - 22.10.2007) - I added 4 byte padding because higher up in the entity hierarchy it makes use of the old 
		// chunkbuffer which do not automatically align themselves.
		xStream.Skip4BytePadding();
	}
	else if ( uVersion == 0 )
	{
		xStream >> m_fSphericalVolumeRadius;

		xStream >> m_iNumVolumes;
		if ( m_iNumVolumes > 0 )
		{
			m_pxVolumes = new Asura_OrientedVolume[ m_iNumVolumes ];
			for ( int i = 0; i < m_iNumVolumes; ++i )
			{
				xStream >> m_pxVolumes[ i ].m_xBox;
				xStream >> m_pxVolumes[ i ].m_xOrient;
			}
		}

		u_char ucNumLinksToBlock;
		xStream >> ucNumLinksToBlock;
		m_uNumLinksToBlock = ucNumLinksToBlock;
		if ( m_uNumLinksToBlock > 0 )
		{
			m_pxLinksToBlock = new Asura_AI_NavConnection_Index[ m_uNumLinksToBlock ];
			for ( u_int u = 0; u < m_uNumLinksToBlock; ++u )
			{
				xStream >> m_pxLinksToBlock[ u ].m_usWaypointIndex;
				xStream >> m_pxLinksToBlock[ u ].m_usConnectionIndex;
			}
		}

		u_int uFlags;
		xStream >> uFlags;
	}
	else
	{
		Asura_Assertf( false, ( "Asura_ServerEntity_WaypointBlockerVolume::ReadFromChunkStream - unhandled chunk version number (%u)", uVersion ) );
	}
}

void Asura_ServerEntity_WaypointBlockerVolume::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	// (TI - 11.10.2007) - Even though the correct way to do chunks is Our Version, Parent, Then our data.  Due to the structure of 
	// the constructor i don't really have a choice but to do Parent, Our Version, Our Data.
	PARENT::WriteToChunkStream( xStream );

	xStream << uCURRENT_CHUNK_VERSION;

	xStream << m_fSphericalVolumeRadius;

	xStream << m_iNumVolumes;
	for ( int i = 0; i < m_iNumVolumes; ++i )
	{
		xStream << m_pxVolumes[ i ].m_xBox;
		xStream << m_pxVolumes[ i ].m_xOrient;
	}

	u_int uNumLinksToBlock = m_uNumLinksToBlock;
	#ifdef ASURA_TOOLS
	//If not all waypoints are being exported, then scan through the list of blocked connections and see how
	//many are still valid. RAR 09/04/2008
	if( Tools_Level_Export::IsExport() && Asura_AI_Navigation_System::HasSubsetForExport() )
	{
		for ( u_int u = 0; u < m_uNumLinksToBlock; ++u )
		{
			Asura_AI_NavConnection_Index xConnection;
			if ( !Asura_AI_Navigation_System::ConvertNavConnectionIndexForExport(m_pxLinksToBlock[u], xConnection) )
			{
				--uNumLinksToBlock;
			}
		}
	}
	#endif
	xStream << uNumLinksToBlock;

	#ifdef ASURA_TOOLS
	if( Tools_Level_Export::IsExport() && Asura_AI_Navigation_System::HasSubsetForExport() )
	{
		// Write out converted waypoint connections that are valid for the subset of waypoints being exported.
		for ( u_int u = 0; u < m_uNumLinksToBlock; ++u )
		{
			Asura_AI_NavConnection_Index xConnection;
			if ( Asura_AI_Navigation_System::ConvertNavConnectionIndexForExport(m_pxLinksToBlock[u], xConnection) )
			{
				xStream << xConnection.m_usWaypointIndex;
				xStream << xConnection.m_usConnectionIndex;
			}
		}
	}
	else
	#endif
	{
		for ( u_int u = 0; u < m_uNumLinksToBlock; ++u )
		{
			xStream << m_pxLinksToBlock[ u ].m_usWaypointIndex;
			xStream << m_pxLinksToBlock[ u ].m_usConnectionIndex;
		}
	}

	xStream.Write4BytePadding();
}

void Asura_ServerEntity_WaypointBlockerVolume::Create(Asura_Guid uGuid, Asura_Chunk_Stream& xStream)
{
	Asura_ServerEntity_WaypointBlockerVolume* pxWaypointBlockerVolume = new Asura_ServerEntity_WaypointBlockerVolume(uGuid);
	if(pxWaypointBlockerVolume)
	{
		pxWaypointBlockerVolume->ReadFromChunkStream( xStream );
	}
}

bool Asura_ServerEntity_WaypointBlockerVolume::HandleMessage(const Asura_Message* pxMessage)
{
	bool bDone = PARENT::HandleMessage(pxMessage);

	if (pxMessage)
	{
		switch (pxMessage->GetMsgID())
		{
			case ASURA_MSG_ACTIVATE:
			{
				SetNeverUpdates(true); // The evils of server entity activatable will turn this on in response
									   // to an activate message which isnt desired.
				bDone = true;
				break;
			}
			default:
			{
				break;
			}
		}
	}
	return bDone;
}


void Asura_ServerEntity_WaypointBlockerVolume::OnActivation()
{
	PARENT::OnActivation();
	UpdateWaypointLinkBlocks();
}

void Asura_ServerEntity_WaypointBlockerVolume::OnDeactivation()
{
	UpdateWaypointLinkBlocks();
	PARENT::OnDeactivation();
}

void Asura_ServerEntity_WaypointBlockerVolume::OnDestruction()
{
	UpdateWaypointLinkBlocks();
	PARENT::OnDestruction();
}

void Asura_ServerEntity_WaypointBlockerVolume::UpdateWaypointLinkBlocks()
{
	BlockLinks( IsActive() );
}

void Asura_ServerEntity_WaypointBlockerVolume::BlockLinks( bool bBlockLinks )
{
	if ( ( m_bBlockedLinks != 0 ) == bBlockLinks )
	{
		return;
	}

	for ( u_int u = 0; u < m_uNumLinksToBlock; ++u )
	{
		Asura_AI_Navigation_System::MarkConnectionAsBlocked( m_pxLinksToBlock[ u ], bBlockLinks );
	}

	m_bBlockedLinks = bBlockLinks;
}

#ifdef ASURA_TOOLS
void Asura_ServerEntity_WaypointBlockerVolume::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance );

	for( int i = 0; i < m_iNumVolumes; i++ )
	{
		Asura_Vector_3 xOldCentre = m_pxVolumes[i].m_xBox.CalculateCentre();
		Asura_Vector_3 xNewCentre = pxInstance->TransformPosition( xOldCentre );
		m_pxVolumes[i].m_xBox.Displace( xNewCentre-xOldCentre );

		m_pxVolumes[i].m_xOrient = pxInstance->TransformQuat( m_pxVolumes[i].m_xOrient );
	}

	// Any blocked waypoints read from template file will be inappropriate, so delete them.
	if ( m_uNumLinksToBlock )
	{
		delete [] m_pxLinksToBlock;
		m_pxLinksToBlock   = NULL;
		m_uNumLinksToBlock = 0;
	}
}

bool Asura_ServerEntity_WaypointBlockerVolume::DiscardWaypointLinkBlockInformation()
{
	const bool bHadLinks = ( m_uNumLinksToBlock != 0 );

	delete [] m_pxLinksToBlock;
	m_pxLinksToBlock   = NULL;
	m_uNumLinksToBlock = 0;
	m_bBlockedLinks = false;

	return bHadLinks;
}

bool Asura_ServerEntity_WaypointBlockerVolume::CalculateWaypointLinkBlocks()
{
	if ( !WaypointLinkBlockCalculationsInitialised() )
	{
		Asura_Assert( false, "Asura_ServerEntity_WaypointBlockerVolume::CalculateWaypointLinkBlocks - calculations not initialised yet" );
		return false;
	}

	// Clear the blocked waypoint list, keeping the old list so we can compare later on.
	const u_int                               uOldCount = m_uNumLinksToBlock;
	const Asura_AI_NavConnection_Index* const pxOldList = m_pxLinksToBlock;
    // We're calculating new links to block, make sure any old links we may have marked as blocked are unmarked.
	BlockLinks( false );
	m_uNumLinksToBlock = 0;
	m_pxLinksToBlock = NULL;

	// Calculate the links that we block.
	s_uNumWaypoints    = 0;
	s_uNumLinksToBlock = 0;

	// Different methods if we have advanced volumes or a spherical volume.
	if ( m_iNumVolumes > 0 )
	{
		for ( int i = 0; i < m_iNumVolumes; ++i )
		{
			// See if the orientation is being used.
			if ( m_pxVolumes[ i ].m_xOrient.w == 1.0f )
			{
				// Nope.
				Asura_AI_Navigation_System::SearchWaypoints( m_pxVolumes[ i ].m_xBox, AddWaypointToTest );
				TestForBlockedLinks( true, i, false );
			}
			else
			{
				// Yup. Expand a new AABB to fit the OBB and then test the new links against the OB.
				Asura_Bounding_Box xAABB;
				Asura_Matrix_3x3 xOri;
				m_pxVolumes[ i ].m_xOrient.ToMatrix( xOri );
				xAABB.FromOBB( m_pxVolumes[ i ].m_xBox, xOri, true );

				Asura_AI_Navigation_System::SearchWaypoints( xAABB, AddWaypointToTest );
				TestForBlockedLinks( true, i, true );
			}
		}
	}
	else
	{
		Asura_Bounding_Box xBox;
		xBox.SetToPoint( GetPosition() );
		xBox.Grow( GetSphericalVolumeRadius() );

		Asura_AI_Navigation_System::SearchWaypoints( xBox, AddWaypointToTest );
		TestForBlockedLinks( false );
	}

	// Fill in the details, if any.
	if ( s_uNumLinksToBlock )
	{
		m_uNumLinksToBlock = s_uNumLinksToBlock;
		m_pxLinksToBlock   = new Asura_AI_NavConnection_Index[ m_uNumLinksToBlock ];
		Asura_Mem::Copy( m_pxLinksToBlock, s_pxLinksToBlock, m_uNumLinksToBlock * sizeof( Asura_AI_NavConnection_Index ) );
	}

	if ( uOldCount != m_uNumLinksToBlock )
	{
		delete [] pxOldList;
		return true;
	}
	else
	{
		bool bDifferent = false;
		for ( u_int u = 0; u < m_uNumLinksToBlock; ++u )
		{
			if ( pxOldList[ u ] != m_pxLinksToBlock[ u ] )
			{
				bDifferent = true;
				break;
			}
		}
		delete [] pxOldList;
		return bDifferent;
	}
}

void Asura_ServerEntity_WaypointBlockerVolume::TestForBlockedLinks( const bool bUsingAdvancedVolumes,
																    const int iVolumeIndex /*= 0*/,
																	const bool bUseOrient /*= false*/ ) const
{
	for ( u_int uWP = 0; uWP < s_uNumWaypoints; ++uWP )
	{
		const u_short             usWPIndex = s_pusWaypoints[ uWP ];
		const Asura_AI_NavVertex* pxWP      = Asura_AI_Navigation_System::GetWaypoint( usWPIndex );
		if ( !pxWP ) continue;

		for ( u_int uLink = 0; uLink < pxWP->GetNumLinks(); ++uLink )
		{
			const u_int               uTargetWPIndex = pxWP->GetLink( uLink ).GetIndex();
			const Asura_AI_NavVertex* pxTargetWP     = Asura_AI_Navigation_System::GetWaypoint( uTargetWPIndex );

			for ( float fHeightOffset = 0.0f; fHeightOffset < 1.51f; fHeightOffset += 0.5f )
			{
				Asura_Vector_3 xAlpha = pxWP->m_xPosition;
				Asura_Vector_3 xBeta = pxTargetWP->m_xPosition - pxWP->m_xPosition;
				// Are we're testing against an OBB?
				if ( bUsingAdvancedVolumes && bUseOrient )
				{
					//yeah, rotate points into OBB local space
					//get the offset from box
					const Asura_Vector_3 xBoxCentre = m_pxVolumes[ iVolumeIndex ].m_xBox.CalculateCentre();
					Asura_Vector_3 xOffset = xAlpha - xBoxCentre;
					//rotate offset to local space
					Asura_Quat xOrientInverse = m_pxVolumes[ iVolumeIndex ].m_xOrient;
					xOrientInverse.Invert();
					xOffset = xOrientInverse.RotateVector( xOffset);
					//the new position
					xAlpha = xBoxCentre + xOffset;

					//do the same for target position
					Asura_Vector_3 xTargetPosition = pxTargetWP->m_xPosition;
					xOffset = xTargetPosition - xBoxCentre;
					xOffset = xOrientInverse.RotateVector( xOffset);
					//The new position
					xTargetPosition = xBoxCentre + xOffset;
					//new beta
					xBeta = xTargetPosition - xAlpha;
				}
				xAlpha.y -= fHeightOffset;
				const float fMaxDist = xBeta.Magnitude();
				if ( fMaxDist == 0.0f ) continue;
				xBeta *= 1.0f / fMaxDist;

				bool bIntersects = false;
				float fCollisionDist;

				if ( bUsingAdvancedVolumes )
				{
					bIntersects	= Asura_Maths::RayIntersectsBoundingBox( xAlpha, xBeta, m_pxVolumes[ iVolumeIndex ].m_xBox, &fCollisionDist );
				}
				else
				{
					bIntersects = Asura_Maths::RayIntersectsSphere( xAlpha, xBeta, GetPosition(), GetSphericalVolumeRadius(), &fCollisionDist );
				}

				if ( bIntersects )
				{
					if ( fCollisionDist <= fMaxDist )
					{
						if ( !AddLinkToBlock( usWPIndex, uLink ) ) { return; }

						// If the other waypoint is also in our array of waypoints to test then we don't need to block the link
						// going the other way as the link would only get tested again (and end up being blocked 4 times).
						bool bBlockLinkComingOtherWay = true;
						for ( u_int uWP2 = 0; uWP2 < s_uNumWaypoints && bBlockLinkComingOtherWay; ++uWP2 )
						{
							if ( uTargetWPIndex == s_pusWaypoints[ uWP2 ] )
							{
								bBlockLinkComingOtherWay = false;
							}
						}

						if ( bBlockLinkComingOtherWay )
						{
							// Block the link coming the other way (if there is one) also.
							for ( u_int uTargetLinkIndex = 0; uTargetLinkIndex < pxTargetWP->GetNumLinks(); ++uTargetLinkIndex )
							{
								if ( pxTargetWP->GetLink( uTargetLinkIndex ).GetIndex() == usWPIndex )
								{
									if ( !AddLinkToBlock( uTargetWPIndex, uTargetLinkIndex ) ) { return; }
									break;
								}
							}
						}

						// Break out of the height increment loop.
						break;
					}
				}
			}
		}
	}
}

bool Asura_ServerEntity_WaypointBlockerVolume::AddWaypointToTest( u_short usWaypointIndex )
{
	if ( s_uNumWaypoints == s_uWaypointArrayCapacity )
	{
		static const u_int uARRAY_SIZE_INCREMENT = 256;

		// We must enlarge the array.
		u_short* pusNewArray = new u_short[ s_uWaypointArrayCapacity + uARRAY_SIZE_INCREMENT ];
		if ( !pusNewArray )
		{
			Asura_Assert( false, "Asura_ServerEntity_WaypointBlockerVolume::AddWaypointToTest - cannot allocate any more memory" );
			return false;
		}
		// Copy over the old entries.
		Asura_Mem::Copy( pusNewArray, s_pusWaypoints, s_uWaypointArrayCapacity * sizeof( u_short ) );
		s_uWaypointArrayCapacity += uARRAY_SIZE_INCREMENT;
		// Swap the arrays over.
		delete [] s_pusWaypoints;
		s_pusWaypoints = pusNewArray;
	}

	s_pusWaypoints[ s_uNumWaypoints++ ] = usWaypointIndex;
	return true;
}

bool Asura_ServerEntity_WaypointBlockerVolume::AddLinkToBlock( const u_int uWPID, const u_int uLinkIndex )
{
	if ( s_uNumLinksToBlock == s_uLinkArrayCapacity )
	{
		static const u_int uARRAY_SIZE_INCREMENT = 256;

		// We must enlarge the array.
		Asura_AI_NavConnection_Index* pusNewArray = new Asura_AI_NavConnection_Index[ s_uLinkArrayCapacity + uARRAY_SIZE_INCREMENT ];
		if ( !pusNewArray )
		{
			Asura_Assert( false, "Asura_ServerEntity_WaypointBlockerVolume::AddLinkToBlock - cannot allocate any more memory" );
			return false;
		}
		// Copy over the old entries.
		Asura_Mem::Copy( pusNewArray, s_pxLinksToBlock, s_uLinkArrayCapacity * sizeof( Asura_AI_NavConnection_Index ) );
		s_uLinkArrayCapacity += uARRAY_SIZE_INCREMENT;
		// Swap the arrays over.
		delete [] s_pxLinksToBlock;
		s_pxLinksToBlock = pusNewArray;
	}

	s_pxLinksToBlock[ s_uNumLinksToBlock ].m_usWaypointIndex   = static_cast< u_short >( uWPID );
	s_pxLinksToBlock[ s_uNumLinksToBlock ].m_usConnectionIndex = static_cast< u_short >( uLinkIndex );
	++s_uNumLinksToBlock;
	return true;
}

void Asura_ServerEntity_WaypointBlockerVolume::InitialiseWaypointLinkBlockCalculations()
{
	if ( !WaypointLinkBlockCalculationsInitialised() )
	{
		static const u_int uINITIAL_ARRAY_SIZE = 512;

		s_uWaypointArrayCapacity = uINITIAL_ARRAY_SIZE;
		s_pusWaypoints           = new u_short[ s_uWaypointArrayCapacity ];
		s_uNumWaypoints          = 0;

		s_uLinkArrayCapacity = uINITIAL_ARRAY_SIZE;
		s_pxLinksToBlock     = new Asura_AI_NavConnection_Index[ s_uLinkArrayCapacity ];
		s_uNumLinksToBlock   = 0;
	}
}

void Asura_ServerEntity_WaypointBlockerVolume::ShutdownWaypointLinkBlockCalculations()
{
	if ( WaypointLinkBlockCalculationsInitialised() )
	{
		delete [] s_pusWaypoints;
		s_pusWaypoints           = NULL;
		s_uWaypointArrayCapacity = 0;
		s_uNumWaypoints          = 0;

		delete [] s_pxLinksToBlock;
		s_pxLinksToBlock     = NULL;
		s_uLinkArrayCapacity = 0;
		s_uNumLinksToBlock   = 0;
	}
}
#endif //ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
//
//////////////////////////////////////////////////////////////////////////////
