//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_WaypointBlockerVolume.h
//
// (TI - 11.10.2007) - Although this has similarities to 
// AdvancedVolumeTriggers it's not inheriting from that class because 
// they have some unneeded variables which would just be a bit of a waste of space.
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_SERVERENTITY_WAYPOINTBLOCKERVOLUME_H___
#define ASURA_SERVERENTITY_WAYPOINTBLOCKERVOLUME_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////
#include "Entity/Triggers/Asura_ServerEntity_Activatable.h"

#include "Core/Asura_OrientedVolume.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////
class Asura_AI_NavConnection_Index;
class Tools_EntityTemplateInstance;

//////////////////////////////////////////////////////////////////////////////
//
// Asura_ServerEntity_WaypointBlockerVolume
//
//////////////////////////////////////////////////////////////////////////////
class Asura_ServerEntity_WaypointBlockerVolume : public Asura_ServerEntity_ActivatableWithPosition
{
	typedef Asura_ServerEntity_ActivatableWithPosition PARENT;
	friend class Tools_Entity_WaypointBlockerVolume;
	friend class Tools_WaypointBlockerVolume_VolumeEdit_Controller;

public:
	
	Asura_ServerEntity_WaypointBlockerVolume( bool bActive = true );
	//constructor for subsequent initialisation using ReadFromChunkStream
	Asura_ServerEntity_WaypointBlockerVolume( Asura_Guid uGuid );

	virtual ~Asura_ServerEntity_WaypointBlockerVolume();

	virtual void PostLoadInitialise();
	virtual void PreDestructionShutdown();
	virtual	bool IsFilledFromChunkBuffer() const { return false; }
	virtual bool HandleMessage(const Asura_Message* pxMessage);

	void UpdateWaypointLinkBlocks();

	static void Create(Asura_Guid uGuid, Asura_Chunk_Stream& xStream);

	// static members for list management
	static Asura_ServerEntity_WaypointBlockerVolume* FindEntity( Asura_Guid uGuid )
	{
		return s_xList.FindEntity(uGuid);
	}
	static Asura_Hashed_List<Asura_ServerEntity_WaypointBlockerVolume>* GetList()
	{
		return &s_xList;
	}

	#ifdef ASURA_TOOLS
	virtual void  ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );
	virtual bool  CanBeIncludedInTemplate()  const { return true; }
	float GetSphericalVolumeRadius() const { return m_fSphericalVolumeRadius; }
	bool  IsUsingAdvancedVolumes()   const { return (m_iNumVolumes > 0); }
	int   GetNumAdvancedVolumes()    const { return m_iNumVolumes; }
	const Asura_OrientedVolume* GetVolume( const int iIndex )
	{
		if ( iIndex >= 0 && iIndex < m_iNumVolumes )
		{
			return &m_pxVolumes[iIndex];
		}
		else
		{
			return NULL;
		}
	}
	u_int GetNumLinksToBlock() const { return m_uNumLinksToBlock; }

	// Before calculating the waypoint link blocks the memory for the calculations must be initialised.
	// After all necessary objects have calculated their blocked links the memory should be let go.
	// These functions accomplish that.
	static bool WaypointLinkBlockCalculationsInitialised() { return ( s_pxLinksToBlock != NULL ); }
	static void InitialiseWaypointLinkBlockCalculations();
	static void ShutdownWaypointLinkBlockCalculations();

	bool DiscardWaypointLinkBlockInformation();
	bool CalculateWaypointLinkBlocks();

	void TestForBlockedLinks( const bool bUsingAdvancedVolumes, const int iVolumeIndex = 0, const bool bUseOrient = false ) const;
	static bool AddWaypointToTest( u_short usWaypointIndex );
	static bool AddLinkToBlock( const u_int uWPID, const u_int uLinkIndex );
	#endif // ASURA_TOOLS

protected:
	virtual void OnActivation();
	virtual void OnDeactivation();
	virtual void OnDestruction();

	virtual	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	virtual void BlockLinks( bool bBlockLinks );

    float m_fSphericalVolumeRadius;
	int m_iNumVolumes;
	Asura_OrientedVolume* m_pxVolumes;

	// These will contain the links that we're meant to block.
	u_int                         m_uNumLinksToBlock;
	Asura_AI_NavConnection_Index* m_pxLinksToBlock;

	// Note that these flags are not chunked currently, as there is no need.
	union
	{
		unsigned int m_uWBVFlags;
		struct 
		{
			// Has already marked links as blocked.
			unsigned int m_bBlockedLinks : 1;
		};
	};

private:
	#ifdef ASURA_TOOLS
	// The static storage used when calculating each object's links to block.
	static u_short*                      s_pusWaypoints;
	static u_int                         s_uWaypointArrayCapacity;
	static u_int                         s_uNumWaypoints;
	static Asura_AI_NavConnection_Index* s_pxLinksToBlock;
	static u_int                         s_uLinkArrayCapacity;
	static u_int                         s_uNumLinksToBlock;
	#endif

private:
	static const u_int uCURRENT_CHUNK_VERSION = 2;

	static Asura_Hashed_List<Asura_ServerEntity_WaypointBlockerVolume> s_xList;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_SERVERENTITY_WAYPOINTBLOCKERVOLUME_H___
