//
// Asura_ClientEntity_EffectVolume.h
//
// Created by Emil Popovic - 15/12/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_CLIENTENTITY_EFFECTVOLUME_H__
#define __ASURA_CLIENTENTITY_EFFECTVOLUME_H__

#include "Entity/Asura_Entity.h"
#include "Entity/Triggers/EffectVolume/Asura_EffectVolume_EntityParameters.h"


// Entity that contains all data for EffectVolume. In fact EffectVolume is only ClientSide effect
// and it has no UpdateMessage from server side. ServerObject exists mainly for Creating, Activating,
// Deactivating and Destroying this object.

// Both Server and Client Entity contains only one value: Asura_EffectVolume_EntityParameters. So that means
// all relevant values are stored in the m_xData object. This object is quite complex, so when ServerSide is
// creating Client side object, it simply Call WriteToClientCreateStream. It will serialize it to "memfile"
// and this memfile is then send over network so ClientSide can ReadFromClientCreateStream. 

class Asura_ClientEntity_EffectVolume : public Asura_Entity_Client
{
	typedef Asura_Entity_Client PARENT;

public :
	// Can only be created from an incoming message
	static Asura_ClientEntity_EffectVolume* Create(const Asura_Message* pxMessage);


	// Accessor function to Data object.
	Asura_EffectVolume_EntityParameters& GetData()	{return m_xData;}


	bool HandleMessage(const Asura_Message* pxMessage);

	// Constructor and Destructor
	virtual ~Asura_ClientEntity_EffectVolume();
	Asura_ClientEntity_EffectVolume();

	// Accessors to s_xList
	static Asura_ClientEntity_EffectVolume* FindEntity( Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }
	static const Asura_Hashed_List<Asura_ClientEntity_EffectVolume>* GetList() { return &s_xList; }    

private:
	void ReadFromClientCreateStream(Asura_Chunk_Memory_Stream& xStream);

	static Asura_Hashed_List<Asura_ClientEntity_EffectVolume> s_xList;

	// Important value! Actually all values that are relevant to EffectVolume either Client or Server Side should be put here in this class! NOT OUTSIDE!
	// It is because that the same data are used in server and in client side object. And they should be synchronized all the time!
	Asura_EffectVolume_EntityParameters	m_xData;
};


#endif //__ASURA_CLIENTENTITY_EFFECTVOLUME_H__
