//
// Asura_EffectVolume_Effect.cpp
//
// Created by Emil Popovic - 15/12/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

// core
#include "Core/Asura.h"

// header
#include "Entity/Triggers/EffectVolume/Asura_EffectVolume_Effect.h"

// asura
#include "SpecialFX/EffectVolume/Asura_EffectVolume_System.h"

// tools




//-------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		EP		Created.
////////////////////////////////////////////////////////////////////////////////
const u_int	Asura_EffectVolume_Effect::s_uChunkVersion	= 1;


bool Asura_EffectVolume_Effect::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	// check our version
	u_int uVersion;
	xStream >> uVersion;
	if( uVersion>s_uChunkVersion )
	{
		Asura_Assertf( false,("Unknown chunk version %d for streaming Asura_EffectVolume_Effect",uVersion));
		return false;
	}

	//save data for this entity
	xStream >> m_uEffectTypeHash;
	xStream >> m_uBlueprintHash;

	return true;
}

void Asura_EffectVolume_Effect::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//save our version
	xStream << s_uChunkVersion;

	//save data for this entity
	xStream << m_uEffectTypeHash;
	xStream << m_uBlueprintHash;
}


//-------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		EP		Created.
////////////////////////////////////////////////////////////////////////////////
const u_int	Asura_EffectVolume_Effect_Fog::s_uChunkVersion	= 1;


Asura_EffectVolume_EffectState* Asura_EffectVolume_Effect_Fog::CreateEffectState()
{
	Asura_EffectVolume_EffectState_Fog* pxState = new Asura_EffectVolume_EffectState_Fog;
	Asura_Assert( pxState, "Failed to allocate memory for EffectState_Fog" );
	if( pxState )
	{
		// Copy parameters.
		pxState->m_xFogParameters = m_xFogParamteres;
	}
	return static_cast<Asura_EffectVolume_EffectState*> (pxState);
}

bool Asura_EffectVolume_Effect_Fog::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	// check our version
	u_int uVersion;
	xStream >> uVersion;
	if( uVersion>s_uChunkVersion )
	{
		Asura_Assertf( false,("Unknown chunk version %d for streaming Asura_EffectVolume_Effect_Fog",uVersion));
		return false;
	}

	if( !PARENT::ReadFromChunkStream( xStream ) )
	{
		return false;
	}

	m_xFogParamteres.ReadFromChunkStream( xStream );

	return true;
}

void Asura_EffectVolume_Effect_Fog::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//save our version
	xStream << s_uChunkVersion;

	PARENT::WriteToChunkStream( xStream );

	m_xFogParamteres.WriteToChunkStream( xStream );
}

#ifdef ASURA_TOOLS
void Asura_EffectVolume_Effect_Fog::ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance)
{
	PARENT::ApplyChangesForTemplateInstance(pxInstance);
}
#endif


//-------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------

#if defined(ASURA_USE_FSFX)

////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		EP		Created.
////////////////////////////////////////////////////////////////////////////////
const u_int	Asura_FSFXTemplate_Parameters::s_uChunkVersion	= 1;


void Asura_FSFXTemplate_Parameters::Clear()
{
	m_uTemplateHash = ASURA_HASH_ID_UNSET;
	m_fStrength = 0.0f;
}

bool Asura_FSFXTemplate_Parameters::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	//check our version
	u_int uVersion;
	xStream >> uVersion;
	if( uVersion>s_uChunkVersion )
	{
		Asura_Assertf( false,("Unknown chunk version %d for Asura_FSFXTemplate_Parameters",uVersion));
		return false;
	}

	xStream >> m_uTemplateHash;
	xStream >> m_fStrength;

	return true;
}

void Asura_FSFXTemplate_Parameters::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//save our version
	xStream << s_uChunkVersion;

	//save data for this entity
	xStream << m_uTemplateHash;
	xStream << m_fStrength;
}


//-------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		EP		Created.
////////////////////////////////////////////////////////////////////////////////
const u_int	Asura_EffectVolume_Effect_FSFX::s_uChunkVersion	= 1;


Asura_EffectVolume_EffectState* Asura_EffectVolume_Effect_FSFX::CreateEffectState()
{
	Asura_EffectVolume_EffectState_FSFX* pxState = new Asura_EffectVolume_EffectState_FSFX;
	Asura_Assert( pxState, "Failed to allocate memory for EffectState_FSFX" );
	if( pxState )
	{
		// Copy parameters.
		pxState->SetTemplateParameter( m_xFSFXTemplateParameters );
	}


	return static_cast<Asura_EffectVolume_EffectState*> (pxState);
}

bool Asura_EffectVolume_Effect_FSFX::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	// check our version
	u_int uVersion;
	xStream >> uVersion;
	if( uVersion>s_uChunkVersion )
	{
		Asura_Assertf( false,("Unknown chunk version %d for streaming Asura_EffectVolume_Effect_FSFX",uVersion));
		return false;
	}

	if( !PARENT::ReadFromChunkStream( xStream ) )
	{
		return false;
	}

	if( !m_xFSFXTemplateParameters.ReadFromChunkStream( xStream ) )
	{
		return false;
	}

	return true;
}

void Asura_EffectVolume_Effect_FSFX::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//save our version
	xStream << s_uChunkVersion;

	PARENT::WriteToChunkStream( xStream );

	m_xFSFXTemplateParameters.WriteToChunkStream( xStream );
}


#ifdef ASURA_TOOLS
void Asura_EffectVolume_Effect_FSFX::ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance)
{
	PARENT::ApplyChangesForTemplateInstance(pxInstance);
}
#endif

#endif	//defined(ASURA_USE_FSFX)

