
//////////////////////////////////////////////////////////////////////////////
//
// Asura_VideoToTexture_System.cpp
// Author    : Jason Turner
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE) 
#include "Interface/Asura_CommandConsole.h"
#endif

#ifdef ASURA_USE_PRIMITIVE_RENDER
#include "Rendering/Asura_Primitive_Render.h"
#endif

#include "FMV/Asura_VideoToTexture_System.h"

Asura_LinkList<Asura_VideoToTexture*> Asura_VideoToTexture_System::m_xVideoStorage;
bool Asura_VideoToTexture_System::m_bInitFirstTime = false;
bool Asura_VideoToTexture_System::m_bInitOnDemand = false;

////////////////////////////////////////////////////////////////////////////////
//
// Asura_VideoToTexture_System::Initialise
//
////////////////////////////////////////////////////////////////////////////////

static bool g_bDisplayDebugInfo = false;

void Asura_VideoToTexture_System::Initialise(void)
{
	// if we're already initialised then bail out
	if (m_bInitOnDemand)
	{
		return;
	}

	Platform_Initialise();

	if (!m_bInitFirstTime)
	{
		#if defined(ASURA_SUPPORT_COMMAND_CONSOLE) 
		Asura_CommandConsole::AddVar( L"FMV.DisplayDebugInfo", &g_bDisplayDebugInfo, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		#endif

		#ifdef ASURA_USE_PRIMITIVE_RENDER
		/*const Asura_Hash_ID uCatHash =*/ Asura_Primitive_Render::RegisterCategory("FMV");
	//	Asura_Primitive_Render::EnableCategory(uCatHash, false);
		#endif //ASURA_USE_PRIMITIVE_RENDER
	}

	m_bInitFirstTime = true;
	m_bInitOnDemand = true;
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_VideoToTexture_System::Reset
//
////////////////////////////////////////////////////////////////////////////////

void Asura_VideoToTexture_System::Reset(void)
{
	Shutdown();
	m_bInitFirstTime = false;
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_VideoToTexture_System::Render
//
// Called between BeginFrame/EndFrame
////////////////////////////////////////////////////////////////////////////////

void Asura_VideoToTexture_System::Render(void)
{
	Asura_ListElt<Asura_VideoToTexture*>* pxElement;
	for (pxElement = m_xVideoStorage.Front(); pxElement; pxElement = pxElement->Next())
	{
		Asura_VideoToTexture* pxData = pxElement->Data();
		pxData->Render();
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_VideoToTexture_System::Shutdown
//
////////////////////////////////////////////////////////////////////////////////

void Asura_VideoToTexture_System::Shutdown(void)
{
	if (!m_bInitOnDemand)
	{
		return;
	}

	// Clear this list manually, since we need to delete the data.
	Asura_ListElt<Asura_VideoToTexture*>* pxElement = m_xVideoStorage.Front();
	while (pxElement)
	{
		Asura_ListElt<Asura_VideoToTexture*>* pxCurrentElement = pxElement;
		pxElement = pxElement->Next();
		Asura_VideoToTexture* pxData = pxCurrentElement->Data();
		m_xVideoStorage.Remove(pxCurrentElement);
		delete pxData;
	}

	Platform_Shutdown();
	m_bInitOnDemand = false;
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_VideoToTexture_System::Update
//
////////////////////////////////////////////////////////////////////////////////

void Asura_VideoToTexture_System::Update(void)
{
	// Initialise on demand
	if (!m_bInitOnDemand)
	{
		Initialise();
	}

	#ifdef ASURA_USE_PRIMITIVE_RENDER
	if (Asura_Primitive_Render::IsActive())
	{
		if (g_bDisplayDebugInfo)
		{
			const Asura_Hash_ID uCatHash = Asura_GetHashID("FMV");
			Asura_Primitive_Render::SetRenderParams(0.0f, uCatHash);
			Asura_Primitive_Render::DrawConsoleText("Currently playing FMV's:");
			Asura_Primitive_Render::DrawConsoleText(" ");
		}
	}
	#endif

	Asura_ListElt<Asura_VideoToTexture*>* pxElement;
	for (pxElement = m_xVideoStorage.Front(); pxElement; pxElement = pxElement->Next())
	{
		Asura_VideoToTexture* pxData = pxElement->Data();
		pxData->Update();

		#ifdef ASURA_USE_PRIMITIVE_RENDER
		if (Asura_Primitive_Render::IsActive())
		{
			if (g_bDisplayDebugInfo)
			{
				pxData->RenderDebugInfo();
			}
		}
		#endif //ASURA_USE_PRIMITIVE_RENDER
	}

	#ifdef ASURA_USE_PRIMITIVE_RENDER
	if (Asura_Primitive_Render::IsActive())
	{
		if (g_bDisplayDebugInfo)
		{
			Asura_Primitive_Render::DrawConsoleText("-----------------------------------------------");
		}
	}
	#endif

	Platform_Update();
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_VideoToTexture_System::RemoveVideo
//
////////////////////////////////////////////////////////////////////////////////

bool Asura_VideoToTexture_System::RemoveVideo(Asura_VideoToTexture* pxVideo)
{
	return (m_xVideoStorage.Remove(pxVideo));
}

#endif // ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
