//
// Asura_FacialAnimation_AnimationHandler_Cutscene.cpp
//
// Created by Richard Rice 18/11/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "FacialAnimation/Asura_FacialAnimation_AnimationHandler_Cutscene.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#include "Cutscene/Asura_Cutscene_Actor.h"
#include "Cutscene/Asura_Cutscene.h"

Asura_FacialAnimation_AnimationHandler_Cutscene::Asura_FacialAnimation_AnimationHandler_Cutscene(Asura_Cutscene_Actor_Hierarchy* pxActor,
																											 const Asura_Hash_ID uFaceHash)
: PARENT(uFaceHash, "FaceAnim", 0, 0)
, m_pxActor(pxActor)
, m_fLastCutsceneTime(0.0f)
, m_fAnimStartTime(0.0f)
, m_uAnimHash(0)
{
	SetExternalEyeData(0.0f,0.0f,0.0f,0.0f,1.0f,0.0f);
}


void Asura_FacialAnimation_AnimationHandler_Cutscene::Update(const float fTimeStep)
{
	Asura_Assert(m_pxActor, "");

	//Ignore the timestep passed in , and check the cutscene timer instead
	float fCutsceneTimer = m_pxActor->GetCutscene()->GetTimer();
	float fCutsceneTimeStep = 0.0f;

	if(fCutsceneTimer > m_fAnimStartTime)
	{
		if(m_fLastCutsceneTime > m_fAnimStartTime)
		{
			fCutsceneTimeStep = fCutsceneTimer - m_fLastCutsceneTime;
		}
		else
		{
			//Animation triggered part of the way through the frame , so just take the time since the trigger time
			fCutsceneTimeStep = fCutsceneTimer - m_fAnimStartTime;
		}
	}

	Asura_Animation* pxAnimation = m_pxActor->GetAnimation();
	if(pxAnimation)
	{
		UpdateAnimation(fCutsceneTimeStep, *pxAnimation);
	}

	m_fLastCutsceneTime = fCutsceneTimer;

}

void Asura_FacialAnimation_AnimationHandler_Cutscene::Cutscene_PlayFacialAnimation( Asura_Hash_ID uAnimHash, float fStartTime, bool bLooped)
{
	//Just keep one animation going , to avoid problems with needing a different time step for each
	//animation.
	if(m_uAnimHash)
	{
		StopFaceAnimation(m_uAnimHash);
		m_uAnimHash = 0;
	}

	if(uAnimHash)
	{
		if(PlayFaceAnimation(uAnimHash, bLooped))
		{
			m_uAnimHash = uAnimHash;
			m_fAnimStartTime = fStartTime;

		}
	}
}


#endif //defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)