
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_FacialAnimation_Face.cpp
// Author : Andy Tate
// Date   : 9-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Asura_FacialAnimation_Face.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationBlend.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "FacialAnimation/Asura_FacialAnimation_MuscleGroup.h"

//Tools files
#ifdef ASURA_TOOLS
#include "FacialAnimation/Tools_FacialAnimation_Face.h"
#endif


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		ART		Created.
// 2		WGP		Added eye bones and limits
// 3		RAR		Changed format of eye bones
////////////////////////////////////////////////////////////////////////////////
const u_int	Asura_FacialAnimation_Face::s_uChunkVersion	= 3;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_Face::Asura_FacialAnimation_Face
// Author       : Andy Tate
// Date         : 22-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_FacialAnimation_Face::Asura_FacialAnimation_Face(void)
: m_uNumMuscleGroups(0)
, m_pxMuscleGroups(NULL)
, m_uNumEyeBones(0)
, m_pxEyeBones(NULL)
{
	m_xEyeValues.SetDefaults();
}

/// End of function 'Asura_FacialAnimation_Face::Asura_FacialAnimation_Face'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_Face::Asura_FacialAnimation_Face
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_FacialAnimation_Face::Asura_FacialAnimation_Face(const Asura_Hash_ID uHash,
													   Asura_Chunk_Stream& xStream)
: m_pxMuscleGroups(NULL)
, m_uNumEyeBones(0)
, m_pxEyeBones(NULL)
{
	ReadFromChunkStream(xStream);
}

/// End of function 'Asura_FacialAnimation_Face::Asura_FacialAnimation_Face'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_Face::~Asura_FacialAnimation_Face
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_FacialAnimation_Face::~Asura_FacialAnimation_Face()
{
	delete[](m_pxMuscleGroups);
	delete[](m_pxEyeBones);
}

/// End of function 'Asura_FacialAnimation_Face::~Asura_FacialAnimation_Face'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_Face::CreateAnimationGroup
// Author       : Andy Tate
// Date         : 22-May-2008
////////////////////////////////////////////////////////////////////////////////
#ifdef ASURA_MAIN
Asura_AnimationController_Group* Asura_FacialAnimation_Face::CreateAnimationGroup(const char* szGroupName,
																				  const u_int uGroupPriority,
																				  const Asura_Hash_ID uParentGroupHash,
																				  Asura_Animation& xAnimation) const
{
	const u_int							uGroupFlags		= AAC_FLAG_ACTIVE;
	Asura_AnimationBlend*				pxNewBlend		= new Asura_AnimationBlend (1.0f);
	Asura_AnimationController_Group*	pxParentGroup	= (uParentGroupHash == ASURA_HASH_ID_UNSET)
														? NULL
														: xAnimation.GetControllerGroupFromHash(uParentGroupHash);
	Asura_AnimationController_Group*	pxNewGroup		= xAnimation.CreateControllerGroup(szGroupName, uGroupFlags, uGroupPriority, pxNewBlend, pxParentGroup);

	if(pxNewGroup == NULL)
	{
		Asura_Assert(false, "Failed to create animation-group for facial-animation.");
		return(NULL);
	}
	else
	{
		const Asura_FacialAnimation_MuscleGroup*	pxMuscleGroup			= m_pxMuscleGroups;
		const char*									szEmptyAnimation		= "EmptyFacialAnimation";
		const u_int									uAnimationFlags			= AAC_FLAG_ACTIVE | AACA_FLAG_USING_SUBSET_OF_BONES;
		u_int										uAnimationPriority		= 0x00008000;
		Asura_AnimationController_Animation*		pxAnimationController	= xAnimation.CreateControllerAnimation(szEmptyAnimation, uAnimationFlags, uAnimationPriority, ASURA_HASH_ID_UNSET, NULL, pxNewGroup);

		if(pxAnimationController == NULL)
		{
			Asura_Assert(false, "Failed to create animation-controller for empty-animation.");
		}
		else
		{
			// Ensure that the animation-controller knows that we're using deltas...
			pxAnimationController->SetEmptyAnimationAsDelta();

			// Create the appropriate animations for each of the muscle-groups...
			for(u_int u = 0; u < m_uNumMuscleGroups; u++, pxMuscleGroup++)
			{
				const u_int	uNumAnimations	= pxMuscleGroup->GetNumAnimations();

				for(u_int v = 0; v < uNumAnimations; v++)
				{
					const Asura_Hash_ID						uAnimationHash	= pxMuscleGroup->GetAnimationHash(v);
					Asura_AnimationBlend*					pxNewBlend		= new Asura_AnimationBlend (0.0f);
					Asura_AnimationController_Animation*	pxNewAnimation	= xAnimation.CreateControllerAnimation(uAnimationFlags, uAnimationPriority, uAnimationHash, pxNewBlend, pxNewGroup);

					if(pxNewAnimation != NULL)
					{
						pxNewAnimation->Play();
						pxNewAnimation->Pause();
					}
				}
			}
		}

		return(pxNewGroup);
	}
}
#endif //ASURA_MAIN
/// End of function 'Asura_FacialAnimation_Face::CreateAnimationGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_Face::CreateMuscleGroups
// Author       : Andy Tate
// Date         : 22-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_FacialAnimation_MuscleGroup* Asura_FacialAnimation_Face::CreateMuscleGroups(const u_int uNumMuscleGroups)
{
	delete[](m_pxMuscleGroups);

	m_pxMuscleGroups	= new Asura_FacialAnimation_MuscleGroup [uNumMuscleGroups];
	m_uNumMuscleGroups	= uNumMuscleGroups;

	return(m_pxMuscleGroups);
}

/// End of function 'Asura_FacialAnimation_Face::CreateMuscleGroups'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_Face::UpdateAnimation
// Author       : Andy Tate
// Date         : 22-May-2008
////////////////////////////////////////////////////////////////////////////////

u_int Asura_FacialAnimation_Face::UpdateAnimation(const float* pfMuscleGroupWeights,
												  const float* pfMuscleGroupBalances,
												  Asura_Animation& xAnimation,
												  Asura_AnimationController_Group& xGroup) const
{
	const Asura_FacialAnimation_MuscleGroup*	pxMuscleGroup	= m_pxMuscleGroups;
	u_int										uNumAnimations	= 0;

	for(u_int u = 0; u < m_uNumMuscleGroups; u++, pxMuscleGroup++)
	{
		// Update the current muscle-group...
		uNumAnimations	+= pxMuscleGroup->UpdateAnimation(*pfMuscleGroupWeights, *pfMuscleGroupBalances, xAnimation, xGroup);

		// Step onto the next muscle-group...
		pfMuscleGroupWeights++;
		pfMuscleGroupBalances++;
	}

	return(uNumAnimations);
}

/// End of function 'Asura_FacialAnimation_Face::UpdateAnimation'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_Face::CalculateTotalNumMuscleGroupAnimations
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////

u_int Asura_FacialAnimation_Face::CalculateTotalNumMuscleGroupAnimations(void) const
{
	const Asura_FacialAnimation_MuscleGroup*	pxMuscleGroup		= m_pxMuscleGroups;
	u_int										uTotalNumAnimations	= 0;

	// Count up the number of animations in each muscle-group...
	for(u_int u = 0; u < m_uNumMuscleGroups; u++, pxMuscleGroup++)
	{
		const u_int	uNumAnimations	= pxMuscleGroup->GetNumAnimations();

		uTotalNumAnimations	+= uNumAnimations;
	}

	return(uTotalNumAnimations);
}

/// End of function 'Asura_FacialAnimation_Face::CalculateTotalNumMuscleGroupAnimations'. ///





////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_Face::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_FacialAnimation_Face::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// First, read (and verify) the chunk-version.
	xStream >> uVersion;

	switch(uVersion)
	{
		case 1:
		{
			xStream >> m_uNumMuscleGroups;

			CreateMuscleGroups(m_uNumMuscleGroups);

			for(u_int u = 0; u < m_uNumMuscleGroups; u++)
			{
				m_pxMuscleGroups[u].ReadFromChunkStream(xStream);
			}

			m_xEyeValues.SetDefaults();

			break;
		}

		case 2:
		{
			Asura_Assert(0,"Version not supported");
			
			xStream >> m_uNumMuscleGroups;

			CreateMuscleGroups(m_uNumMuscleGroups);

			for(u_int u = 0; u < m_uNumMuscleGroups; u++)
			{
				m_pxMuscleGroups[u].ReadFromChunkStream(xStream);
			}

			xStream.Skip(22*4);//skip 6 ints and 16 floats

			m_xEyeValues.SetDefaults();
			
			break;
		}
		case 3:
		{
			xStream >> m_uNumMuscleGroups;

			CreateMuscleGroups(m_uNumMuscleGroups);

			for(u_int u = 0; u < m_uNumMuscleGroups; u++)
			{
				m_pxMuscleGroups[u].ReadFromChunkStream(xStream);
			}

			xStream >> m_xEyeValues.m_fInLimit;
			xStream >> m_xEyeValues.m_fOutLimit;
			xStream >> m_xEyeValues.m_fUpLimit;
			xStream >> m_xEyeValues.m_fDownLimit;
			xStream >> m_xEyeValues.m_fUpSideLimit;
			xStream >> m_xEyeValues.m_fDownSideLimit;
			xStream >> m_xEyeValues.m_fLeftRightScale;
			xStream >> m_xEyeValues.m_fUpDownScale;
		
			xStream >> m_uNumEyeBones;
			m_pxEyeBones = m_uNumEyeBones ? new Asura_FacialAnimation_Face_EyeBone[m_uNumEyeBones] : 0;
			for(u_int i=0; i<m_uNumEyeBones; i++)
			{
				xStream >> m_pxEyeBones[i].m_iBoneIndex;
				xStream >> m_pxEyeBones[i].m_fRotSquint;
				xStream >> m_pxEyeBones[i].m_fRotClose;
				xStream >> m_pxEyeBones[i].m_fAngMin;
				xStream >> m_pxEyeBones[i].m_fAngMax;
				xStream >> m_pxEyeBones[i].m_uFlags;
			}

			break;
		}

		default:
		{
			Asura_Assertf(false, ("Unexpected chunk-version %d", uVersion));
			break;
		}
	}		// End of switch.
}

/// End of function 'Asura_FacialAnimation_Face::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_FacialAnimation_Face::WriteToChunkStream
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_FacialAnimation_Face::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// First write the chunk-version...
	xStream << s_uChunkVersion;

	xStream << m_uNumMuscleGroups;

	for(u_int u = 0; u < m_uNumMuscleGroups; u++)
	{
		m_pxMuscleGroups[u].WriteToChunkStream(xStream);
	}

	xStream << m_xEyeValues.m_fInLimit;
	xStream << m_xEyeValues.m_fOutLimit;
	xStream << m_xEyeValues.m_fUpLimit;
	xStream << m_xEyeValues.m_fDownLimit;
	xStream << m_xEyeValues.m_fUpSideLimit;
	xStream << m_xEyeValues.m_fDownSideLimit;
	xStream << m_xEyeValues.m_fLeftRightScale;
	xStream << m_xEyeValues.m_fUpDownScale;

	xStream << m_uNumEyeBones;
	for(u_int i=0; i<m_uNumEyeBones; i++)
	{
		xStream << m_pxEyeBones[i].m_iBoneIndex;
		xStream << m_pxEyeBones[i].m_fRotSquint;
		xStream << m_pxEyeBones[i].m_fRotClose;
		xStream << m_pxEyeBones[i].m_fAngMin;
		xStream << m_pxEyeBones[i].m_fAngMax;

		Asura_WriteToStream_Bitfield_Wrapper<u_int>	xFlagsBitfield (m_pxEyeBones[i].m_uFlags);
		xFlagsBitfield.Add( m_pxEyeBones[i].m_bEye, 1);
		xFlagsBitfield.Add( m_pxEyeBones[i].m_bRight, 1);
		xStream << xFlagsBitfield;
	}
}

/// End of function 'Asura_FacialAnimation_Face::WriteToChunkStream'. ///

const Asura_FacialAnimation_Face_EyeBone* Asura_FacialAnimation_Face::GetEyeBone(u_int uIndex) const 
{
	Asura_Assert(uIndex<m_uNumEyeBones,"");
	return &m_pxEyeBones[uIndex];
}


#ifdef ASURA_TOOLS
void Asura_FacialAnimation_Face::Tools_CopyEyeData(u_int uNumEyeBones, const struct Tools_FacialAnimation_Face_EyeBone* pxEyeBones, const Asura_FacialAnimation_Face_EyeValues& xEyeValues)
{
	m_xEyeValues = xEyeValues;

	delete [] m_pxEyeBones;
	m_uNumEyeBones = uNumEyeBones;
	m_pxEyeBones = uNumEyeBones ? new Asura_FacialAnimation_Face_EyeBone[uNumEyeBones] : 0;
	for(u_int i=0; i<uNumEyeBones ; i++)
	{
		m_pxEyeBones[i].m_iBoneIndex = pxEyeBones[i].m_iBoneIndex; 
		m_pxEyeBones[i].m_fRotSquint = pxEyeBones[i].m_fRotSquint; 
		m_pxEyeBones[i].m_fRotClose = pxEyeBones[i].m_fRotClose; 
		m_pxEyeBones[i].m_fAngMin = pxEyeBones[i].m_fAngMin; 
		m_pxEyeBones[i].m_fAngMax = pxEyeBones[i].m_fAngMax; 
		m_pxEyeBones[i].m_uFlags = pxEyeBones[i].m_uFlags; 
	}

	//Ignore the Scale values from Max at the moment. RAR 17/10/2008
	m_xEyeValues.m_fLeftRightScale = Asura_Maths::Max( m_xEyeValues.m_fInLimit, m_xEyeValues.m_fOutLimit );
	m_xEyeValues.m_fUpDownScale = Asura_Maths::Max( m_xEyeValues.m_fUpLimit, m_xEyeValues.m_fDownLimit );
	
}
#endif//ASURA_TOOLS


void Asura_FacialAnimation_Face_EyeValues::SetDefaults()
{
	m_fInLimit = Asura_Maths::DegToRad( 31.0f );
	m_fOutLimit = Asura_Maths::DegToRad( 40.0f );
	m_fUpLimit = Asura_Maths::DegToRad( 24.0f );
	m_fDownLimit = Asura_Maths::DegToRad( 28.0f );
	m_fUpSideLimit = Asura_Maths::DegToRad( 12.0f );
	m_fLeftRightScale = Asura_Maths::Max( m_fInLimit, m_fOutLimit );
	m_fUpDownScale = Asura_Maths::Max( m_fUpLimit, m_fDownLimit );
}

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
