
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_FacialAnimation_Face.h
// Author : Andy Tate
// Date   : 8-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_FACIALANIMATION_FACE_H__
#define __ASURA_FACIALANIMATION_FACE_H__

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_List.h"
#include "FacialAnimation/Asura_FacialAnimation_MuscleGroup.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Animation;
class Asura_AnimationController_Group;
class Asura_Chunk_Stream;

//Contains the movement limits for the eyes for a given face
struct Asura_FacialAnimation_Face_EyeValues
{
	void SetDefaults();

	float	m_fInLimit;
	float	m_fOutLimit;
	float	m_fUpLimit;
	float	m_fDownLimit;
	float	m_fUpSideLimit;
	float	m_fDownSideLimit;
	float	m_fLeftRightScale;
	float	m_fUpDownScale;
};

//Details for each eye bone controlled by the face
struct Asura_FacialAnimation_Face_EyeBone
{
	int m_iBoneIndex;
	float m_fRotSquint;
	float m_fRotClose;
	float m_fAngMin;
	float m_fAngMax;
	union
	{
		u_int m_uFlags;
		struct
		{
			u_int m_bEye:1;//Otherwise an eyelid
			u_int m_bRight:1;		
		};
	};
};



////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_FacialAnimation_Face
// Author       : Andy Tate
// Date         : 8-May-2008
//
// Description  : Class used to store the set of muscle-groups used when making
//				  up a face in the facial-animation system.
////////////////////////////////////////////////////////////////////////////////

class Asura_FacialAnimation_Face
{
public:
	// Public functions...
	Asura_FacialAnimation_Face(void);
	Asura_FacialAnimation_Face(const Asura_Hash_ID uHash, Asura_Chunk_Stream& xStream);
	~Asura_FacialAnimation_Face();

	// Animation functions...
	Asura_AnimationController_Group*	CreateAnimationGroup(const char* szGroupName, const u_int uGroupPriority, const Asura_Hash_ID uParentGroupHash, Asura_Animation& xAnimation) const;
	u_int								UpdateAnimation(const float* pfMuscleGroupWeights, const float* pfMuscleGroupBalances, Asura_Animation& xAnimation, Asura_AnimationController_Group& xGroup) const;

	u_int								CalculateTotalNumMuscleGroupAnimations(void) const;

	// Muscle-group functions...
	Asura_FacialAnimation_MuscleGroup*	CreateMuscleGroups(const u_int uNumMuscleGroups);

	// Chunk-stream functions...
	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	// Accessor functions...
	u_int	GetNumMuscleGroups(void) const		{ return(m_uNumMuscleGroups); }
	const Asura_FacialAnimation_MuscleGroup* GetMuscleGroup(u_int uIndex) const {return &m_pxMuscleGroups[uIndex];}

												 
	float 	GetEyeInLimit()				{ return m_xEyeValues.m_fInLimit; }
	float 	GetEyeOutLimit()			{ return m_xEyeValues.m_fOutLimit; }
	float 	GetEyeUpLimit()				{ return m_xEyeValues.m_fUpLimit; }
	float 	GetEyeDownLimit()			{ return m_xEyeValues.m_fDownLimit; }
	float 	GetEyeUpSideLimit()			{ return m_xEyeValues.m_fUpSideLimit; }
	float 	GetEyeDownSideLimit()		{ return m_xEyeValues.m_fDownSideLimit; }
	float 	GetEyeLeftRightScale()		{ return m_xEyeValues.m_fLeftRightScale; }
	float 	GetEyeUpDownScale()			{ return m_xEyeValues.m_fUpDownScale; }

	u_int	GetNumEyeBones() const		{ return m_uNumEyeBones;}
	const Asura_FacialAnimation_Face_EyeBone* GetEyeBone(u_int uIndex) const;

	//Tools functions
	#ifdef ASURA_TOOLS
	void	Tools_CopyEyeData(u_int uNumEyeBones, const struct Tools_FacialAnimation_Face_EyeBone* pxEyeBones, const Asura_FacialAnimation_Face_EyeValues& xEyeValues);
	#endif

protected:
	// Protected variables...

	// Protected functions...


private:

	// Private variables...
	u_int								m_uNumMuscleGroups;	// The number of muscle-groups for this face.
	Asura_FacialAnimation_MuscleGroup*	m_pxMuscleGroups;	// A pointer to the array of muscle-groups for this face.

	u_int								m_uNumEyeBones;
	Asura_FacialAnimation_Face_EyeBone*	m_pxEyeBones; 

	Asura_FacialAnimation_Face_EyeValues m_xEyeValues;


	static const u_int					s_uChunkVersion;	// The version used when reading/writing.

	// Private functions...
	Asura_FacialAnimation_Face(const Asura_FacialAnimation_Face& xOther);

	Asura_FacialAnimation_Face&	operator=(const Asura_FacialAnimation_Face& xOther);

};   // End of class 'Asura_FacialAnimation_Face'.



#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#endif   // ifndef __ASURA_FACIALANIMATION_FACE_H__


/// End of file. ///
