// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Audio.cpp														//
//																						//
//	Description: Audio Implementation for GUIMenu										//
//																						//
//	Created: Steve Keen - 12/04/07														//
//																						//
//	Copyright (C) 2007 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //


#include "Core/Asura.h"
#include "GUIMenu/Asura_GUIMenu_Audio.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_System.h"
#include "Sound/Asura_Sound_Event_Storage.h"

#ifdef ASURA_TOOLS
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#endif

// ---------------------------------------------------------------------------------------

Asura_Hash_ID	Asura_GUIMenu_Audio::s_uSoundEventHash[ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST];
bool			Asura_GUIMenu_Audio::s_bDataChanged;

// ---------------------------------------------------------------------------------------

void Asura_GUIMenu_Audio::Initialise( void)
{
	ClearLocalVariables();
}

// ---------------------------------------------------------------------------------------

void Asura_GUIMenu_Audio::Reset( void)
{
}

// ---------------------------------------------------------------------------------------

void Asura_GUIMenu_Audio::Shutdown( void)
{
}

// ---------------------------------------------------------------------------------------

void Asura_GUIMenu_Audio::Play( Asura_GUIMenu_Audio_Type eType)
{
	Asura_Assert( eType < ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST, "Asura_GUIMenu_Audio::SetAudioType - Invalid GuiMenu Audio Type");

	if( eType < ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST)
	{
		const Asura_Hash_ID	uEventHash	= s_uSoundEventHash[eType];

		if( uEventHash != ASURA_HASH_ID_UNSET)
		{
			const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent( uEventHash );
			if(pxEvent == NULL)
			{
				Asura_Assert(false, "Failed to find sound-event for GUIMenu audio.");
			}
			else
			{
				Asura_Sound_Event_System::PlayEvent( pxEvent, NULL );
			}
		}
	}
}

// ---------------------------------------------------------------------------------------

void Asura_GUIMenu_Audio::SetAudioType( Asura_GUIMenu_Audio_Type eType, Asura_Hash_ID uSoundEventHash)
{
	Asura_Assert( eType < ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST, "Asura_GUIMenu_Audio::SetAudioType - Invalid GuiMenu Audio Type");

	if( eType < ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST)
	{
		s_uSoundEventHash[eType] = uSoundEventHash;
	
		s_bDataChanged = true;
	}
}

// ---------------------------------------------------------------------------------------

Asura_Hash_ID Asura_GUIMenu_Audio::GetAudioHash( Asura_GUIMenu_Audio_Type eType)
{
	Asura_Hash_ID uRetHash = ASURA_HASH_ID_UNSET;

	Asura_Assert( eType < ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST, "Asura_GUIMenu_Audio::SetAudioType - Invalid GuiMenu Audio Type");

	if( eType < ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST)
	{
		uRetHash = s_uSoundEventHash[eType];
	}

	return uRetHash;
}

// ---------------------------------------------------------------------------------------

void Asura_GUIMenu_Audio::ReadFromChunkStream( Asura_Chunk_Stream& xStream)
{
	u_int u;
	u_int uItemCount = 0;
	u_int uActualSpace, uTemp;

	xStream >> uItemCount;

	Asura_Assert( uItemCount <= ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST, "Asura_GUIMenu_Audio::ReadFromChunkStream - Too many sound types in GUIMenu audio file.");

	uActualSpace = ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST;

	for( u=0; u<uItemCount; u++)
	{
		xStream >> uTemp;

		if( u<uActualSpace)
		{
			s_uSoundEventHash[u] = uTemp;
		}
	}
}

// ---------------------------------------------------------------------------------------

#ifdef ASURA_TOOLS
void Asura_GUIMenu_Audio::WriteToChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int u;

	// -- write this out as a safety measure.
	xStream << (u_int)ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST;

	for( u=0; u<ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST; u++)
	{
		xStream << s_uSoundEventHash[u];
	}
}

void Asura_GUIMenu_Audio::AddFilesToExportList(void)
{
	for(u_int u = 0; u < ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST; u++)
	{
		const Asura_Hash_ID			uEventHash	= s_uSoundEventHash[u];
		const Asura_Sound_Event*	pxEvent		= Asura_Sound_Event_Storage::GetEventByHash(uEventHash);

		if((pxEvent != NULL) && !pxEvent->IsGlobalSound())
		{
			Tools_Level_Export::AddSoundEventToExportList(pxEvent);
		}
	}
}
#endif

// ---------------------------------------------------------------------------------------

void Asura_GUIMenu_Audio::ClearLocalVariables( void)
{
	u_int u;
	
	for( u=0; u<ASURA_GUIMENU_AUDIO_TYPE_END_OF_LIST; u++)
	{
		s_uSoundEventHash[u] = ASURA_HASH_ID_UNSET;
	}

	s_bDataChanged = false;
}

// ---------------------------------------------------------------------------------------

#endif // ASURA_USE_GUI_MENU_SYSTEM
