// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Message.h														//
//																						//
//	Description: Structure definition for the GUIMenu message							//
//																						//
//	Created: Steve Keen - 17/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_GUIMENU_MESSAGE_H__
#define __ASURA_GUIMENU_MESSAGE_H__

#include "Core/Asura_Hash.h"

struct Asura_Message_GUIMenu_Notify
{
	u_int uActionType;
	u_int WidgetType;
	Asura_Hash_ID uToID;
	Asura_Hash_ID uFromID;
	u_int uData;
};

#define ASURA_GUIMENU_GET_MAIN_MESSAGE( x)	((x)&0xffff)
#define ASURA_GUIMENU_GET_SUB_MESSAGE( x)	(((x)>>16)&0xffff)

#define ASURA_GUIMENU_MAKE_ACTION_MESSAGE( x, y) ((((y)&0xffff)<<16) | ((x)&0xffff))
#define ASURA_GUIMENU_HAS_SUB_MESSAGE( x) ((((x)>>16)&0xffff) != 0)

#define ASURA_GUIMENU_MAKE_ANIM_DATA( uSeq, uAnimType, uRepeat)	((((uRepeat)&0xffff)<<16) | (((uAnimType)&0xff)<<8) | ((uSeq)&0xff))
#define ASURA_GUIMENU_GET_ANIM_SEQUENCE( x)	((x)&0x000000ff)
#define ASURA_GUIMENU_GET_ANIM_TYPE( x)		(((x)>>8)&0x000000ff)
#define ASURA_GUIMENU_GET_ANIM_REPEAT( x)		(((x)>>16)&0x0000ffff)

// -- Message cracker macros --------------------------------------------------------------

#define ASURA_GUIMENU_CATCH_MSG_BY_SENDER_ID( xMsgNotifyPtr)		\
	switch( (xMsgNotifyPtr)->uFromID)						\
	{

#define ASURA_GUIMENU_CATCH_MSG_END								\
	default:											\
		break;											\
	}

#define ASURA_GUIMENU_CATCH_ID_FOUND( xIdToMatch)		case (xIdToMatch):

// ----------------------------------------------------------------------------------------
// -- Widget helper Macros ----------------------------------------------------------------

#define ASURA_GUIMENU_ENABLE_WIDGET( idTo)	\
	Asura_GUIMenu_MessagePump::PostMessage( (idTo),						\
	ASURA_GUIMENU_EXTERNAL_NOTIFY,	\
	ASURA_GUIMENU_UI_MSG_ENABLE_DISABLE, \
	0,	\
	1);

#define ASURA_GUIMENU_DISABLE_WIDGET( idTo)	\
	Asura_GUIMenu_MessagePump::PostMessage( (idTo),						\
	ASURA_GUIMENU_EXTERNAL_NOTIFY,	\
	ASURA_GUIMENU_UI_MSG_ENABLE_DISABLE, \
	0,	\
	0);

// ----------------------------------------------------------------------------------------

#define ASURA_GUIMENU_MSG_SEND_REQUESTED_DATA( MsgNotifyPtr, data)					\
	Asura_GUIMenu_MessagePump::PostMessage( (MsgNotifyPtr)->uFromID,			\
											(MsgNotifyPtr)->uToID,			\
											ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE,			\
											(MsgNotifyPtr)->WidgetType,		\
											data);

#define ASURA_GUIMENU_MSG_SEND_DATA( uHashToID, uWidgetType, data)					\
	Asura_GUIMenu_MessagePump::PostMessage( uHashToID,			\
											ASURA_GUIMENU_EXTERNAL_NOTIFY,			\
											ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE,			\
											uWidgetType,		\
											data);

// ----------------------------------------------------------------------------------------
// -- ListBox Macros ----------------------------------------------------------------------

#define ASURA_GUIMENU_TEXTLISTBOX_SEND_TEXT( idTo, pszText)						\
	Asura_GUIMenu_MessagePump::PostMessage( (idTo),						\
											ASURA_GUIMENU_EXTERNAL_NOTIFY,	\
											ASURA_GUIMENU_MAKE_ACTION_MESSAGE( ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TEXTLISTBOX_ADD_TEXT), \
											Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX,	\
											(u_int)(pszText))


#define ASURA_GUIMENU_TEXTLISTBOX_SET_SELECTED( idTo, iData)						\
	Asura_GUIMenu_MessagePump::PostMessage( (idTo),					\
											ASURA_GUIMENU_EXTERNAL_NOTIFY,		\
											ASURA_GUIMENU_MAKE_ACTION_MESSAGE( ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TEXTLISTBOX_SET_SELECTION_INDEX),	\
											Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX,	\
											(iData))

#define ASURA_GUIMENU_TEXTLISTBOX_CLEAR_ALL( idTo)										\
	Asura_GUIMenu_MessagePump::PostMessage( (idTo),							\
											ASURA_GUIMENU_EXTERNAL_NOTIFY,		\
											ASURA_GUIMENU_MAKE_ACTION_MESSAGE( ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TEXTLISTBOX_CLEAR), \
											Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX)

// ----------------------------------------------------------------------------------------
// -- AnimImage Macros --------------------------------------------------------------------

#define ASURA_GUIMENU_ANIMIMAGE_SEND_COMMAND( idTo, command, data)		\
	Asura_GUIMenu_MessagePump::PostMessage( (idTo),				\
											ASURA_GUIMENU_EXTERNAL_NOTIFY,									\
											ASURA_GUIMENU_MAKE_ACTION_MESSAGE( ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, (command)),		\
											Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_ANIMATINGIMAGE,	\
											(u_int)(data))

// iCount should be -1 for loop forever, otherwise the count is how many times it loops, or pingpongs
// 
#define ASURA_GUIMENU_ANIMIMAGE_PLAY_SEQUENCE( idTo, iSequence, iMode, iCount)	\
	ASURA_GUIMENU_ANIMIMAGE_SEND_COMMAND( (idTo), ASURA_GUIMENU_ANIMIMAGE_PLAY, ASURA_GUIMENU_MAKE_ANIM_DATA( (iSequence), iMode, (iCount)))

#define ASURA_GUIMENU_ANIMIMAGE_PLAY_SEQUENCE_BACKWARDS( idTo, iSequence, iMode, iCount)	\
	ASURA_GUIMENU_ANIMIMAGE_SEND_COMMAND( (idTo), ASURA_GUIMENU_ANIMIMAGE_PLAY_BACKWARDS, ASURA_GUIMENU_MAKE_ANIM_DATA( (iSequence), iMode, (iCount)))

#define ASURA_GUIMENU_ANIMIMAGE_STOP_SEQUENCE( idTo)	\
	ASURA_GUIMENU_ANIMIMAGE_SEND_COMMAND( (idTo), ASURA_GUIMENU_ANIMIMAGE_STOP, 0)

// ----------------------------------------------------------------------------------------
// -- ProgressBar Macros ------------------------------------------------------------------

#define ASURA_GUIMENU_PROGRESSBAR_SEND_COMMAND( idTo, command, data)		\
	Asura_GUIMenu_MessagePump::PostMessage( (idTo),				\
	ASURA_GUIMENU_EXTERNAL_NOTIFY,									\
	ASURA_GUIMENU_MAKE_ACTION_MESSAGE( ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, (command)),		\
	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_PROGRESSBAR,	\
	(u_int)(data))

#define ASURA_GUIMENU_ENCODE_PROGRESSBAR_DATA( data)	(Asura_Maths::FloatToInt( data * 100.0f))
#define ASURA_GUIMENU_DECODE_PROGRESSBAR_DATA( data)	(Asura_Maths::FloatToInt( data * 0.01f))

#define ASURA_GUIMENU_PROGRESSBAR_SETPOS( idTo, dataValue)	\
	ASURA_GUIMENU_PROGRESSBAR_SEND_COMMAND( (idTo), ASURA_GUIMENU_PROGRESSBAR_SETPOSITION, ASURA_GUIMENU_ENCODE_PROGRESSBAR_DATA( dataValue))

#define ASURA_GUIMENU_PROGRESSBAR_GETPOS( idTo)	\
	ASURA_GUIMENU_PROGRESSBAR_SEND_COMMAND( (idTo), ASURA_GUIMENU_PROGRESSBAR_GETPOSITION, 0)

#define ASURA_GUIMENU_PROGRESSBAR_STEP_FORWARD( idTo)	\
	ASURA_GUIMENU_PROGRESSBAR_SEND_COMMAND( (idTo), ASURA_GUIMENU_PROGRESSBAR_ADD_STEP, 0)

#define ASURA_GUIMENU_PROGRESSBAR_STEP_BACKWARD( idTo)	\
	ASURA_GUIMENU_PROGRESSBAR_SEND_COMMAND( (idTo), ASURA_GUIMENU_PROGRESSBAR_SUBTRACT_STEP, 0)

// ----------------------------------------------------------------------------------------
// -- Numeric Macros ------------------------------------------------------------------

#define ASURA_GUIMENU_NUMERIC_SEND_COMMAND( idTo, command, data)		\
											Asura_GUIMenu_MessagePump::PostMessage( (idTo),				\
											ASURA_GUIMENU_EXTERNAL_NOTIFY,									\
											ASURA_GUIMENU_MAKE_ACTION_MESSAGE( ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, (command)),		\
											Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_NUMERIC,	\
											(u_int)(data))

#define ASURA_GUIMENU_NUMERIC_SET_INTVALUE( idTo, dataValue)	\
	ASURA_GUIMENU_NUMERIC_SEND_COMMAND( (idTo), ASURA_GUIMENU_NUMERIC_SET_INT, dataValue)

#endif // __ASURA_GUIMENU_MESSAGE_H__
