// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_System.h																//
//																						//
//	Description: Header file for the UI Factory											//
//																						//
//	Created: Steve Keen - 9/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_GUIMENU_SYSTEM_H__
#define __ASURA_GUIMENU_SYSTEM_H__

#ifdef ASURA_USE_GUI_MENU_SYSTEM

// -- Includes ------------------------------------------------------------------------
#include "Core/Asura_PlatformBasics.h"
#ifdef ASURA_TOOLS
#include "Core/Asura_NameList.h"
#else
#include "Core/Asura_HashTree.h"
#endif
#include "Core/Asura_String.h"
#include "Core/Asura_StringBuffer.h"
#include "Core/Asura_Timers.h"
#include "Input/Asura_ControlMap.h"

#include "GUIMenu/Asura_GUIMenu_Root.h"

// -- Forward Declarations ------------------------------------------------------------

// -- Enums ---------------------------------------------------------------------------

// -- Defines -------------------------------------------------------------------------
#ifndef ASURA_GUIMENU_MAX_MENU_PAGES
#ifdef ASURA_TOOLS
#define ASURA_GUIMENU_MAX_MENU_PAGES	(512)
#else
#define ASURA_GUIMENU_MAX_MENU_PAGES	(160)
#endif
#endif

#ifndef ASURA_GUIMENU_MAX_MENU_PAGES_FADING_OUT
#define ASURA_GUIMENU_MAX_MENU_PAGES_FADING_OUT	(4)
#endif

#define ASURA_GUIMENU_STANDARD_FILE_EXTENSION_BE_SUFFIX "BE"

#define ASURA_GUIMENU_STANDARD_FILE_EXTENSION_LE "gui"
#define ASURA_GUIMENU_STANDARD_FILE_EXTENSION_BE ASURA_GUIMENU_STANDARD_FILE_EXTENSION_LE ASURA_GUIMENU_STANDARD_FILE_EXTENSION_BE_SUFFIX

#ifdef ASURA_PLATFORM_IS_BIG_ENDIAN
#define ASURA_GUIMENU_STANDARD_FILE_EXTENSION ASURA_GUIMENU_STANDARD_FILE_EXTENSION_BE
#else
#define ASURA_GUIMENU_STANDARD_FILE_EXTENSION ASURA_GUIMENU_STANDARD_FILE_EXTENSION_LE
#endif

#ifndef ASURA_GUIMENU_TEXT_DIRECTORY
#define ASURA_GUIMENU_TEXT_DIRECTORY "Misc"
#endif


// these flags will be used with s_aucMenuPageUsageFlags
#define ASURA_GUIMENU_MENU_USAGE_FLAG_ACTIVE				(1<<0)
#define ASURA_GUIMENU_MENU_USAGE_FLAG_CODE_UPATE_AND_RENDER	(1<<1)

// -- Forward Declarations -----------------------------------------------------------

class Asura_Bounding_Rectangle;
class Asura_GUIMenu_Root;
class Asura_GUIMenu_Widget_Base;
class Asura_GUIMenu_Element_Base;
class Asura_GUIMenu_Element_Text;
class Asura_GUIMenu_Menu_Page;
class Asura_GUIMenu_OnScreenKeyboard;
struct Asura_Message_GUIMenu_Notify;
class Asura_RGBA;
class Asura_Vector_3;

// -- Structures ----------------------------------------------------------------------

struct Asura_Message_GUIMenu_Activate
{
	Asura_Hash_ID uHashMenuPageID;
};

#ifdef ASURA_GUIMENU_ENABLE_METRICS
struct Asura_GUIMenu_Metrics
{
	u_int uGetConsoleVarCalls_Bool;
	u_int uGetConsoleVarCalls_Int;
	u_int uGetConsoleVarCalls_Float;
	u_int uGetConsoleVarCalls_String;

	u_int uUpdateCalls_MenuPages;

	u_int uQueuePeak_MessagePump;
	u_int uQueuePeakPerSecond_MessagePump;
};
#endif

struct Asura_GUIMenu_Menu_Widget_Match
{
	const Asura_GUIMenu_Menu_Page* pxMenuPage;
	Asura_GUIMenu_Widget_Base* pxWidgetBase;
};

// -- Class ---------------------------------------------------------------------------

class Asura_GUIMenu_System
{
public:
	typedef void ProjectCallbackFunction();


	friend class Asura_GUIMenu_OnScreenKeyboard;

	// -- public typedef --
	typedef void ExternMessageOverrideCallbackFunction( const Asura_Message_GUIMenu_Notify* pxMessage);

	enum MenuControls
	{
		ASURA_GUIMENU_CONTROL_UP = 0,
		ASURA_GUIMENU_CONTROL_DOWN,
		ASURA_GUIMENU_CONTROL_LEFT,
		ASURA_GUIMENU_CONTROL_RIGHT,
		ASURA_GUIMENU_CONTROL_ACCEPT,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT1,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT2,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT3,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT4,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT5,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT6,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT7,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT8,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT9,
		ASURA_GUIMENU_CONTROL_CANCEL,
		ASURA_GUIMENU_CONTROL_UP_HELD,
		ASURA_GUIMENU_CONTROL_DOWN_HELD,
		ASURA_GUIMENU_CONTROL_LEFT_HELD,
		ASURA_GUIMENU_CONTROL_RIGHT_HELD,
		ASURA_GUIMENU_CONTROL_ACCEPT_HELD,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD,
		ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD,
		ASURA_GUIMENU_CONTROL_CANCEL_HELD,

#ifdef ASURA_USE_CONTROLMAP_MANAGER
		ASURA_GUIMENU_CONTROL_DVS_ACTIVATE,
#endif
		ASURA_GUIMENU_CONTROL_CLICK,
		ASURA_GUIMENU_CONTROL_CLICK_HELD,

		ASURA_GUIMENU_CONTROL_AXIS_H,
		ASURA_GUIMENU_CONTROL_AXIS_V,

		ASURA_GUIMENU_CONTROL_MAX
	};
	
	static void						Initialise(void);
	static void						Reset( void);
	static void						Shutdown(void);	

	static void						Update(void);
	static void						Render(void);

	static void						RenderWidescreenBars(void);

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	static void						Platform_RenderSafetyBorder( void);
#endif // ASURA_DEBUG_VARIABLE_SYSTEM
	static Asura_GUIMenu_Root*			Create( u_int uType, const Asura_Bounding_Rectangle& xRect );
	static Asura_GUIMenu_Element_Text*	CreateTextElementArray( u_int uCount );	// assumes xRect is 0s.

	static void						IgnoreControlsThisUpdate() { s_bShouldUpdateControls = false; }
	static void						RegisterControlEntry( u_int uControllerIndex, MenuControls eControl, int iKeyID, const Asura_ControlMapping::KeyHoldType eHoldType, char* pcName = NULL);

	// -- Helper functions --
	static void						RegisterExternalMessageCallback( Asura_GUIMenu_System::ExternMessageOverrideCallbackFunction* pfCallback);

	static Asura_GUIMenu_Root*		CreateTextElement( const Asura_UnicodeChar* pwszText, const Asura_String& xFontName, const Asura_Bounding_Rectangle& xRect, const Asura_RGBA* pxColour = NULL);

	// Normally called (indirectly) by Asura_GUIMenu_System::Update, but made public here so code driven pages can use the message pump without needing to do a full GUI update - PJC
	static void						FlushMessagePump();

	// SK 28.02.07 - This is a compatibility function to allow old type calls.  This is bad because
	// Asura_String is a char* internally...hence no foreign text.
	static Asura_GUIMenu_Root*		CreateTextElement( const Asura_String& xText, const Asura_String& xFontName, const Asura_Bounding_Rectangle& xRect, const Asura_RGBA* pxColour);
	static Asura_GUIMenu_Root*		CreateStatusTextElement( const Asura_String& xFontName, const Asura_Bounding_Rectangle& xRect, const Asura_RGBA* pxColour);
	static Asura_GUIMenu_Root*		CreateImageElement( const Asura_String& xText,  const Asura_Bounding_Rectangle& xRect, const Asura_RGBA* pxColour = NULL);
	static Asura_GUIMenu_Root*		CreatePathElement( const Asura_Vector_2* pxPointArray, u_int uPointCount, const Asura_RGBA* pxColour = NULL);

	static Asura_GUIMenu_Root*		CreateMenuPage( const Asura_String& xMenuName, float fPosX, float fPosY, float fWidth, float fHeight);
	static Asura_GUIMenu_Root*		CreateMenuPage( const Asura_String& xMenuName, const Asura_Bounding_Rectangle& xBoundingRect);
	static bool						DestroyMenuPage( Asura_Hash_ID uMenuPageID, bool bForceDelete = false);
	static bool						DestroyMenuPage( Asura_GUIMenu_Menu_Page* pxMenuPage, bool bForceDelete = false);
	static void						DestroyAllMenuPages( void);

	static bool						AddToMenuList( Asura_GUIMenu_Menu_Page* pxMenu);
	static Asura_GUIMenu_Menu_Page*	GetMenuPageByIndex( u_int uIndex);
	static Asura_GUIMenu_Menu_Page*	GetMenuPageByID( Asura_Hash_ID uHashID);
	static bool						RemoveFromMenuList( Asura_GUIMenu_Menu_Page* pxMenuPage);
	static u_int					GetCurrentMenuListCount( void) { return s_uMenuPageCount;	}

	static Asura_GUIMenu_Root*			FindRootObjectByID( Asura_Hash_ID uHashID);
	static Asura_GUIMenu_Widget_Base*	FindWidgetObjectByID( Asura_Hash_ID uHashID);
	static Asura_GUIMenu_Root*			FindParentMenuOfObjectID( Asura_Hash_ID uHashID);

	
	static void						SetMenuPageActiveByID( Asura_Hash_ID uHashID, bool bTF);
	static void						SetMenuPageActive( int iMenuPageIndex, bool bTF);
	static bool						IsMenuPageActive( int iMenuPageIndex);
	static bool						IsMenuPageHashActive( Asura_Hash_ID uHashMenuPageID);
    static bool						DoesMenuPageExist( Asura_Hash_ID uHashID);
	static bool						DoesWidgetExistOnMenuPage( Asura_Hash_ID uHashMenuPageID, Asura_Hash_ID uHashWidgetID);

	static bool						AnyMenuPageHasFocus( );

	static int						FindExistingMenuPage( const Asura_Hash_ID uNameHash);

	static int						SetFocusedMenuByIndex( int iFocusedMenu );
	static int						GetFocusedMenuIndex( void );
	static int						SetFocusedMenuByID( Asura_Hash_ID uHashID );
	static Asura_Hash_ID			GetFocusedMenuHashID( void );
	static Asura_GUIMenu_Menu_Page* GetFocusedMenu( void );

	static void						SetPageHandledInputHash( Asura_Hash_ID uPageHash ) { s_uPageHandlingInput = uPageHash; }
	static Asura_Hash_ID			GetPageHandledInputHash( void ) { return s_uPageHandlingInput; }

	static bool						IsMenuFadingOut( void ) { return s_uNumMenusFadingOut != 0; }	// returns true iff a menu page is in the fade out state
	static bool						IsSpecificMenuFadingOut( const Asura_GUIMenu_Menu_Page* pxMenu );
	static void						StartMenuFadingOut( Asura_GUIMenu_Menu_Page* pxMenu );
	static void						EndMenuFadingOut( const Asura_GUIMenu_Menu_Page* pxMenu );


	static bool						OpenMenuPage( Asura_Hash_ID uHashMenuPage, float fWidth, float fHeight, bool bGiveControlFocus = true, bool bSetBackLink = true);
	static bool						OpenMenuPage( Asura_Hash_ID uHashMenuPage, bool bGiveControlFocus = true, bool bSetBackLink = true);
	static bool						OpenMenuPageOnWidget( Asura_Hash_ID uHashMenuPage, Asura_Hash_ID uHashWidget, bool bGiveControlFocus = true, bool bSetBackLink = true);
	static bool						CloseMenuPage( Asura_Hash_ID uHashMenuPage);

	static bool						InitMenuPageData( Asura_Hash_ID uMenuPageHash, bool bGiveControlFocus = false);
	static bool						DeInitMenuPageData( Asura_Hash_ID uMenuPageHash);

	static void						FlagMenuPageAsCodeDrivenByID( Asura_Hash_ID uHashID, bool bTF);
	static void						FlagMenuPageAsCodeDriven( int iMenuPageIndex, bool bTF);

#ifdef ASURA_TOOLS
	static void						AddAllTexturesToExportList( void);
	//adds textures used by a single menu to the export list
	static void						AddTexturesToExportList(Asura_GUIMenu_Menu_Page* pxMenuItem);

	//Calls the callback function on each texture index used by the menu widgets/elements
	static void						TraverseUsedTextures(void (*pfnCallback)(int iTextureID,void* pUserData),void* pUserData);

	static void						AddFilesToExportList(Asura_GUIMenu_Menu_Page* pxMenuItem);

	static void						RestoreResizedSettings( void);
#endif

	static void						LoadGenericAudioData( void);

	//loads the asr file containing the localised text (if no language is specified , the current language is used)
	// Provide filenames in the format "trc", in order to load "misc\menu\trc_en.asr", for example.
	static void						LoadMenuText(const char* szFilename, Asura_Hash_ID uTextPageHash, int iLang = -1, bool bReload = false);
#ifdef ASURA_TOOLS
	static void						ReloadAllMenuText(int iLang = -1);
#endif

#ifdef ASURA_TOOLS
	static char**					GetMenuControlTextStrings( void);
#endif

	static int						GetCurrentLanguage() {return s_iLanguage;} //gets the current language being previewed

	// widget or element doing text entry passes its ID along with a buffer to modify and an optional message prompt.
	// The object will receive a call to TextInputEnded() when the buffer has finished being changed.
	static void						StartTextInput(Asura_Hash_ID uID, Asura_UnicodeChar* wszBuffer, u_int uBufferMaxChars, const Asura_UnicodeChar* wszMessage = NULL);
	static bool						IsDoingTextInput() { return s_uTextInputID != ASURA_HASH_ID_UNSET; }
	static Asura_Hash_ID			GetIDOfTextInputWidget() { return s_uTextInputID; }

	//Sets most recent active widget , for reading via the console variable system.
	static void 					SetActiveWidgetConsoleVar(Asura_Hash_ID uHash);
	static void 					SetFocusedPageConsoleVar(Asura_Hash_ID uHash) {s_iConsoleVar_FocusedPage = uHash;}

	static void						InitTypeInstance( u_int uType, Asura_GUIMenu_Root* pxItem, const Asura_Bounding_Rectangle& xRect );
	
	// creates a new instance of a instance of source within parent and returns a pointer to this copy. If no name is supplied the source's name is used
	static Asura_GUIMenu_Root*		CreateCopy( Asura_GUIMenu_Root* pxDestParent,const Asura_GUIMenu_Root* pxSource,const char* const szDestName = NULL );

	static bool						SetMenuPageDefaultPos( Asura_Hash_ID uMenuPageID, Asura_Vector_2& xPos);

	static u_int					GetCurrentButtonPresses( void)	{	return s_uButtonInput;	}

	static void						SetControlMapIndexToUse( u_int uControllerIndex);
	static u_int					GetCurrentControlMapIndex( void)	{	return s_uControlMapIndexInUse;	}
	static Asura_ControlMap*		GetCurrentControlMap() { return (s_uControlMapIndexInUse < Platform_GetNumControlMaps()) ? &(s_pxMenuControlMaps[s_uControlMapIndexInUse]) : NULL; }

	// SK - 31.5.07 i've made these public because i need to access them externally of Asura_GUIMenu_System.
	// platform text input functions (e.g. for PSP OSK, PC input buffer updating)
	static void						Platform_BeginTextInput(const Asura_UnicodeChar* wszMessage, Asura_UnicodeChar* wszBuffer, u_int uBufferMaxChars);
	static void						Platform_UpdateTextInput();
	static void						Platform_BeginRender();
	static void						Platform_RenderTextInput();
	static void						Platform_EndRender();
	static bool						Platform_IsTextInputFinished();
	static bool						Platform_IsTextInputCancelled();

	static void						Platform_EndTextInput();

	static bool						Platform_ProcessMenuBackControls( u_int uButtonsPressed, u_int uButtonsTrapped);

	static Asura_GUIMenu_Menu_Widget_Match*	FastSearchForWidget( Asura_Hash_ID uWidgetHash);
	static void						AddWidgetToSearchCache( Asura_GUIMenu_Widget_Base* pxWidgetBase, const Asura_GUIMenu_Menu_Page* pxMenuOwner);
	static void						AddLoadingWidgetToSearchCache( Asura_GUIMenu_Widget_Base* pxWidgetBase);
	static void 					ResetFastWidgetLookUpTree();
	static void						SetLoadingMenu(const Asura_GUIMenu_Menu_Page* pxLoadingMenu) { s_pxLoadingMenu = pxLoadingMenu;}
	static void						RemoveWidgetFromSearchCache( Asura_Hash_ID uWidgetHash);

#ifdef ASURA_TOOLS
	static void RegisterMenuTextPage( Asura_Hash_ID uTextPageHash, const char* szFilename );
	static const Asura_TemplateHashTree<Asura_String>& GetMenuTextFilenameTree() { return s_xMenuTextFilenames; }
	static const Asura_String* GetMenuTextFilenameFromHash(Asura_Hash_ID uHash);
#endif

#ifdef ASURA_GUIMENU_ENABLE_METRICS
	static Asura_GUIMenu_Metrics*	Metrics_GetDataPtr( void)	{	return &s_xMetrics;	}
	static void						Metrics_Begin( void);
	static void						Metrics_End( void);
	static void						Metrics_Render( void);
#endif
	// Returns true if any active menu has the ShouldPause flag set.
	static bool						ShouldPause();

	//Indicates whether the position of the cursor defined by Asura_Cursor should be used 
	static void						SetUseCursor(bool bUse) {s_bUseCursor = bUse;}
	static bool						GetUseCursor() {return s_bUseCursor;}
	// NY - 09 December 2008 - control whether moving the cursor off an item will deselect it
	static void						SetAllowCursorDeselect(bool bAllow) {s_bAllowCursorDeselect = bAllow;}
	static bool						GetAllowCursorDeselect() {return s_bAllowCursorDeselect;}
	

	static void						NotifyVideoModeChange( void);

	static bool						ResizeMenuPage( Asura_Hash_ID uMenuPageID, float fNewWidth, float fNewHeight);

	static void						SetAutoMenuSizes( float fWidth, float fHeight);
	static const Asura_Vector_2&	GetAutoMenuSizes() {return s_xAutoMenuSizeDims;}
	static void						SetAutoMenuOffsets( float fXOffset, float fYOffset );
	static const Asura_Vector_2&	GetAutoMenuOffsets() { return s_xAutoMenuOffsetDims; }
	static bool						GetAutoMenuSizesSet() {return s_bAutoMenuSizesSet;}
	static void						SetVirtualResolution( float fWidth, float fHeight);
	static const Asura_Vector_2&	GetVirtualResolution() {return s_xVirtualResolution;}
	static bool						HasVirtualResolution();
	static const Asura_Vector_2&	GetVirtualScreenScale();

	static void SetProjectPostInitialiseCallback( ProjectCallbackFunction* const pfnProjectPostInitialiseCallback ) { s_pfnProjectPostInitialiseCallback = pfnProjectPostInitialiseCallback; }
	static void SetProjectPreResetCallback(	ProjectCallbackFunction* const pfnProjectPreResetCallback )	          { s_pfnProjectPreResetCallback	  = pfnProjectPreResetCallback; }
	static void SetProjectPreShutdownCallback( ProjectCallbackFunction* const pfnProjectPreShutdownCallback )	  { s_pfnProjectPreShutdownCallback   = pfnProjectPreShutdownCallback; }

	static void	DisableLoadGenericAudioDataDuringInitialisation(void)	{ s_bLoadGenericAudioDataDuringInitialisation = false; }
	static void	EnableLoadGenericAudioDataDuringInitialisation(void)	{ s_bLoadGenericAudioDataDuringInitialisation = true; }

	// Get the Asura frame time modified by the current GUI Menu system speed.
	static float GetFrameTime() { return Asura_Timers::GetFrameTime() * s_fMenuSystemSpeed; }

	// Set the speed for the GUIMenu system. 
	// A value of 1.0f is normal speed. 
	// A value of 0.5f is half speed. 
	static void SetSpeed(float fSpeed) { s_fMenuSystemSpeed = fSpeed; }
	static float GetSpeed() { return s_fMenuSystemSpeed; }

	static void EnableWidescreenBarsThisFrame() { s_bRenderWidescreenBarsThisFrame = true; }

protected:
	static void						ClearLocalVariables( void);

	static void						SendExternalMessage( const Asura_Message_GUIMenu_Notify* pxMessage);


	static void						Handle_MenuPageUpdate( u_int uButtons);
	static u_int					Handle_ControllerInput( void);

	static bool						IsActionRequested(const MenuControls eMapping);

	static void						GenerateUniqueMenuName( void);
	static void						GenerateUniqueWidgetName( u_int uType);
	static void						GenerateUniqueElementName( u_int uType);

	// -- Platform specific calls - Asura_Platform_GUIMenuInput.cpp needs to be available for each platform
	static void						Platform_InitInputMappings( void);
	static void						Platform_ShutdownInputMappings( void);

	// platform functions to control whether to bother with input, and with menu page rendering. Used on PSP
	// to prevent input and rendering when interactive System Utility screens are being displayed.
	// Platform_ShouldRenderMenuPages() can also be used to clear the screen to black etc.
	static bool						Platform_ShouldUpdateControls();
	static bool						Platform_ShouldRenderMenuPages();

	static u_int					Platform_GetNumControlMaps( void);

	// SK 31.5.07 - I'm making these functions public.  I need to access them externally of the GUIMenu system.
	//// platform text input functions (e.g. for PSP OSK, PC input buffer updating)
	//static void						Platform_BeginTextInput(const Asura_UnicodeChar* wszMessage, Asura_UnicodeChar* wszBuffer, u_int uBufferMaxChars);
	//static void						Platform_UpdateTextInput();
	//static void						Platform_RenderTextInput();
	//static bool						Platform_IsTextInputFinished();
	//static void						Platform_EndTextInput();

	static void						ReleaseAllMenuPages( void);

	#ifdef ASURA_TOOLS
	//texture traversal callback for export
	static void						AddTextureToExportList(int iTextureIndex,void* pJunk);
	#endif

	// Play the named music track.
	static bool ConsoleCmd_PlayMusicTrack(const char* szMusicName, bool bInterrogate, bool bHelp);
	// Stop all music from playing
	static bool ConsoleCmd_StopMusic(bool bHelp);

	static bool ConsoleCmd_OpenMenuPage(const char* szMenuName, bool bInterrogate, bool bHelp);
	static bool ConsoleCmd_ChangeMenuPage(const char* szMenuName, bool bInterrogate, bool bHelp);

public:
#ifdef ASURA_GUIMENU_ENABLE_METRICS
		static Asura_GUIMenu_Metrics	s_xMetrics;
#endif

private:
	static bool						s_bInitialised;
	static bool						s_bLoadGenericAudioDataDuringInitialisation;

	static Asura_ControlMap*		s_pxMenuControlMaps;
	static u_int					s_uControlMapIndexInUse;
//	static Asura_ControlMap			s_xMenuControlMap;
	static bool						s_bControlMapOnStack;
	static u_int					s_uTextInputID;
	static float					s_fLastHAxisValue;
	static float					s_fLastVAxisValue;

	static bool						s_bShouldUpdateControls;
	static u_int					s_uButtonInput;

	static Asura_GUIMenu_Menu_Page*	s_apxMenuPages[ASURA_GUIMENU_MAX_MENU_PAGES];
	static u_char					s_aucMenuPageUsageFlags[ASURA_GUIMENU_MAX_MENU_PAGES];
	static u_int					s_uMenuPageCount;

	static int						s_iSelectedMenuIndex;
	static u_int					s_uNumMenusFadingOut;
	static Asura_GUIMenu_Menu_Page*	s_apxMenuPagesFadingOut[ASURA_GUIMENU_MAX_MENU_PAGES_FADING_OUT];

	static Asura_String64			s_xUniqueName;
	static u_int					s_uUniqueMenuNumber;
	static u_int					s_uUniqueWidgetNumber;
	static u_int					s_uUniqueElementNumber;

//	#ifdef ASURA_TOOLS
	static int						s_iLanguage;
	static bool						s_bAllowBack;
//	#endif

	//store of the most recent active widget for access via the console var system.
	static int						s_iConsoleVar_ActiveWidget;
	static int						s_iConsoleVar_LastActiveWidget;
	static int						s_iConsoleVar_FocusedPage;
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	static bool						s_bRenderDebugInfo;
	static bool						s_bRenderDebugBounds;
	static bool						s_bRenderSafetyBorder;
#endif // ASURA_DEBUG_VARIABLE_SYSTEM
	static ExternMessageOverrideCallbackFunction* s_pfMessageCallback;

	//for cursor control via mouse, or wii thingymajig etc
	static bool						s_bUseCursor;
	static bool						s_bAllowCursorDeselect;

	static bool						s_bAutoMenuSizesSet;
	static Asura_Vector_2			s_xAutoMenuSizeDims;
	static Asura_Vector_2			s_xAutoMenuOffsetDims;
	static Asura_Vector_2			s_xVirtualResolution;
	static Asura_Vector_2			s_xVirtualScreenScale;

#ifdef ASURA_TOOLS
	static char*					s_pcToolControlNames[ASURA_GUIMENU_CONTROL_MAX];
	static Asura_TemplateHashTree<Asura_String>	s_xMenuTextFilenames;
#endif

#ifdef ASURA_GUIMENU_ENABLE_METRICS
	static bool						s_bUpdateMetrics;	
#endif

	static Asura_Hash_ID 		s_uPageHandlingInput;

	static Asura_TemplateHashTree<Asura_GUIMenu_Menu_Widget_Match> s_xFastWidgetLookUpTree;
	static const Asura_GUIMenu_Menu_Page* s_pxLoadingMenu;

	// project callbacks function pointers - will be call at the begining(end) of the 
	// (Initialise(),) Reset() and Shutdown() functions
	static ProjectCallbackFunction* s_pfnProjectPostInitialiseCallback;
	static ProjectCallbackFunction* s_pfnProjectPreResetCallback;
	static ProjectCallbackFunction* s_pfnProjectPreShutdownCallback;
	
	static float s_fMenuSystemSpeed;

	static bool						s_bRenderWidescreenBarsThisFrame;
};
#endif // ASURA_USE_GUI_MENU_SYSTEM
#endif // __ASURA_GUIMENU_SYSTEM_H__
