// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Widget_Button.h													//
//																						//
//	Description: button widget class													//
//																						//
//	Created: Steve Keen - 9/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#ifndef __ASURA_GUIMENU_WIDGET_BUTTON_H__
#define __ASURA_GUIMENU_WIDGET_BUTTON_H__

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"
#include "GUIMenu/Asura_GUIMenu_CommandManager.h"
//#include "GUIMenu/Asura_GUIMenu_CommandConsole.h"

// -- Typedef -------------------------------------------------------------------------

// -- Enums ---------------------------------------------------------------------------

// -- Defines -------------------------------------------------------------------------
#define ASURA_GUIMENU_BUTTON_FLAG_CLOSE_MENU_ONCLICKED	(1<<0)
#define ASURA_GUIMENU_BUTTON_FLAG_AUTOCLICK				(1<<1)	// todo: should this only be allowed if close menu on clicked is set?
#define ASURA_GUIMENU_BUTTON_FLAG_DONT_SET_BACKLINK		(1<<2)	// used for when a button navigates back in the menu tree
#define ASURA_GUIMENU_BUTTON_FLAG_TRAP_BACK_KEYPRESS	(1<<3)	// should the menu trap the "cancel" keypress and click this button?
#define ASURA_GUIMENU_BUTTON_FLAG_POLL_BUTTON_PRESSES	(1<<4)	// when set, will always check for button presses whether selected or not.
#define ASURA_GUIMENU_BUTTON_FLAG_GO_BACK				(1<<5)	// button acts as back button.
#define ASURA_GUIMENU_BUTTON_FLAG_DUPLICATE_BACKLINK	(1<<6)	// traverse menu tree sideways - replace this page with another

#define ASURA_GUIMENU_WIDGET_BUTTON_LOCAL_FLAG_START	16

// versions: ASURA_CHUNK_WIDGET_BUTTON_VERSION
// 0: initial data
// 1: added console command - MH
// 2: changed console command to use console manager - SK
// 3: added m_fMinTimeBeforeActive and m_fTimeBeforeAutoclick;
#define ASURA_CHUNK_WIDGET_BUTTON_VERSION 3

// -- Structures ----------------------------------------------------------------------

// -- Class ---------------------------------------------------------------------------

class Asura_GUIMenu_Widget_Button : public Asura_GUIMenu_Widget_Base
{
//	friend class Asura_Chunk_GUIMenu;			// for access to m_xCommand; until we have ReadFromChunkStream() member.
	friend class Tools_GUIMenu_Widget_Button;	// for access to m_xCommand when creating tools dialog.
	friend class Asura_GUIMenu_CommandManager;
	// -- Functions -----------------------------------------------------------------------
public:
	enum Asura_GUIMenu_Inherit_Flag_Widget_Button_Type
	{
		ASURA_GUIMENU_WIDGET_BUTTON_LOCAL_FLAG_LINKTO	= (1<<(ASURA_GUIMENU_WIDGET_BUTTON_LOCAL_FLAG_START+0)),
		ASURA_GUIMENU_WIDGET_BUTTON_LOCAL_FLAG_FLAGS	= (1<<(ASURA_GUIMENU_WIDGET_BUTTON_LOCAL_FLAG_START+1)),
		//ASURA_GUIMENU_WIDGET_BUTTON_OVERIDE_MAX is here to stop another programmer adding past bit flag 31
		ASURA_GUIMENU_WIDGET_BUTTON_LOCAL_FLAG_MAX		= (1<<(ASURA_GUIMENU_WIDGET_BUTTON_LOCAL_FLAG_START+15))
	};

	Asura_GUIMenu_Widget_Button();
	virtual ~Asura_GUIMenu_Widget_Button();

	virtual void 				Update( float fElapsedTime);
	virtual void				Copy( const Asura_GUIMenu_Root* pxSource );

	// -- UI Message handler functions --
	virtual bool				OnInit( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnDeInit( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnDataChange( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnDataRequest( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnClicked( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnRollIn( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnRollOut( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnEnable( const Asura_Message_GUIMenu_Notify* pxMessage);

	Asura_Hash_ID				GetLinkToHash( void) const;//	{ return m_uHashLinkToObject; }
	void						SetLinkToHash( Asura_Hash_ID uHashLinkTo)	{ m_uHashLinkToObject = uHashLinkTo; }

	bool						IsButtonFlagSet( u_int uButtonFlag) const;
	bool						DontSetBackLink(void) const;
	bool						DuplicateBackLink(void) const;
	bool						TrapBackKeyPress(void) const;
	void						SetButtonFlag( u_int uButtonFlag, bool bTrueOrFalse);
	u_int						GetButtonFlags( void) const;
	void						SetButtonFlags( u_int uButtonFlags);
	u_int*						GetButtonFlagsAsPtr( void);

	bool						DoesButtonHaveActionCommands() { return (m_xCommandManager.GetActionButtonCount() > 0); }

#ifdef ASURA_TOOLS
	virtual void				UpdateLocalDataFromStyle( void);

	virtual void				WriteToChunkStream( Asura_Chunk_Stream& xStream) const;
	virtual void				ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const;
#endif // ASURA_TOOLS

	virtual void				ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion);

protected:
	void						SetupDefaultElements( void);
	bool						UseHotLink( bool& bIsSoundEffectPlayed );
	void						DoClickActions( u_int uButton, bool bProcessAcceptOnly = false);

	void						PlayHotLinkSoundEffect( void );

private:

	// -- Variables -----------------------------------------------------------------------
public:

protected:
	u_int 							m_uButtonFlags;
	Asura_Hash_ID					m_uHashLinkToObject;
	Asura_GUIMenu_CommandManager	m_xCommandManager;
	float							m_fMinTimeBeforeActive;		// not rendered until this many seconds after opening menu 
	float							m_fTimeBeforeAutoclick;		// if autoclick is set - autoclick happens after this time.

private:
	typedef Asura_GUIMenu_Widget_Base PARENT;

};
#endif // ASURA_USE_GUI_MENU_SYSTEM
#endif // __ASURA_GUIMenu_WIDGET_BUTTON_H__
