//
// Asura_GUIMenu_Widget_ListBox.cpp
//
// Created by RDM - 23 April 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

/************/
/* Includes */
/************/

// main asura header
#include "Core/Asura.h"

// This file's header.
#include "GUIMenu/Asura_GUIMenu_Widget_ListBox.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include <wchar.h>

// asura
#include "Core/Asura_Hash.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Timers.h"
#include "Core/Memory/Asura_Mem.h"
#include "Chunksystem/Asura_Chunk_Stream.h"

#include "GUIMenu/Asura_GUIMenu_Audio.h"
#include "GUIMenu/Asura_GUIMenu_Element_Animation.h"
#include "GUIMenu/Asura_GUIMenu_Element_Base.h"
#include "GUIMenu/Asura_GUIMenu_Element_Image.h"
#include "GUIMenu/Asura_GUIMenu_Element_Path.h"
#include "GUIMenu/Asura_GUIMenu_Element_Text.h"
#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_MessagePump.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_Profiling_GUIMenu.h"

#ifdef ASURA_TOOLS
#include "GUIMenu/Tools_GUIMenu_PreviewOverlay.h"
#include "GUIMenu/Tools_GUIMenu_XMLExporter.h"
#include "GUIMenu/Tools_Mode_GUIMenu.h"
#endif //ASURA_TOOLS

#include "Rendering/Asura_Render.h"

/********/
/* Data */
/********/

// constants

// static data
Asura_GUIMenu_Widget_ListBoxEntry*		 Asura_GUIMenu_Widget_ListBoxEntry::s_pxPopulationEntry    = NULL;
const Asura_GUIMenu_Widget_ListBoxEntry* Asura_GUIMenu_Widget_ListBoxEntry::s_pxPopulationTemplate = NULL;
u_int Asura_GUIMenu_Widget_ListBoxEntry::s_uPopulationItem = 0;
bool Asura_GUIMenu_Widget_ListBoxEntry::s_bVisibileFromCallback = false;

/*************/
/* Functions */
/*************/

// quick and dirty parsing class
class Asura_GUIMenu_ListBox_PopulationFuncParser
{
public :
	Asura_GUIMenu_ListBox_PopulationFuncParser( const Asura_UnicodeChar* pwszString ) 
	: m_pwszParseString(pwszString) 
	{
	};

	~Asura_GUIMenu_ListBox_PopulationFuncParser()
	{
		Asura_Assert( !m_pwszParseString[0], "Not all data has been parsed!" );
	}

	bool GetValue( u_int* puOutput )
	{		
		if( !m_pwszParseString || !m_pwszParseString[0] ) return false; // no data to parse

		if( Asura_StringUtil::GetIntValue( m_pwszParseString, reinterpret_cast<int*>(puOutput) ) )
		{
            SkipToken();
			return true;
		}
		else
		{
			Asura_Assert( false, "Parse error!" );
			return false;
		}
	};

	bool GetValue( int* piOutput )
	{
		if( !m_pwszParseString || !m_pwszParseString[0] ) return false; // no data to parse

		if( Asura_StringUtil::GetIntValue( m_pwszParseString, piOutput ) )
		{
			SkipToken();
			return true;
		}
		else
		{
			Asura_Assert( false, "Parse error!" );
			return false;
		}
	};

	bool GetValue( float* pfOutput )
	{
		if( !m_pwszParseString || !m_pwszParseString[0] ) return false; // no data to parse
			
		if( Asura_StringUtil::GetFloatValue( m_pwszParseString, pfOutput ) )
		{
			SkipToken();
			return true;
		}
		else
		{
			Asura_Assert( false, "Parse error!" );
			return false;
		}
	};

	void SkipToken( void )
	{
		// move to first piece of whitespace
		while( m_pwszParseString[0] && (m_pwszParseString[0] != L' ') )
		{
			m_pwszParseString++;
		}		

		// skip whitespace
		while( m_pwszParseString[0] && (m_pwszParseString[0] == L' ') )
		{
			m_pwszParseString++;
		}
	}

protected:
	const Asura_UnicodeChar* m_pwszParseString;
};

// Asura_GUIMenu_Widget_ListBoxEntry //////////////////////////////////////////////////////////////////////////////////

Asura_GUIMenu_Widget_ListBoxEntry::Asura_GUIMenu_Widget_ListBoxEntry(void)
: m_pxParentListBox( NULL )
{
	m_uType = ASURA_GUIMENU_TYPEID_WIDGET_LISTBOXENTRY;
}


bool Asura_GUIMenu_Widget_ListBoxEntry::IsEntryTemplate( void ) const
{ 
	return (m_pxParentListBox && (&m_pxParentListBox->GetEntryTemplate() == this)); 
}

Asura_GUIMenu_Element_Base* Asura_GUIMenu_Widget_ListBoxEntry::GetElementToPopulate( Asura_GUIMenu_Element_Base* pxTemplateElement )
{
	// Get the current element in this container, if the types match, we can re-use it
	Asura_GUIMenu_Element_Base* pxCurrentElement = static_cast<Asura_GUIMenu_Element_Base*> (GetContainerItem( s_uPopulationItem ));
	if( !pxCurrentElement || (pxCurrentElement->GetType() != pxTemplateElement->GetType()) )
	{
		// otherwise we need to delete and renew it
		if( pxCurrentElement )
		{
			RemoveFromThisContainer( pxCurrentElement );
			delete pxCurrentElement;
		}

		// create new element
		Asura_Bounding_Rectangle xDummy;
		xDummy.MinX = 0.0f;
		xDummy.MaxX = 0.0f;
		xDummy.MinY = 0.0f;
		xDummy.MaxY = 0.0f;

		pxCurrentElement = static_cast<Asura_GUIMenu_Element_Base*> (Asura_GUIMenu_System::Create( pxTemplateElement->GetType(), xDummy ));

		// copy its properties
		pxCurrentElement->Copy( pxTemplateElement );

		// add to the entry
		if( !AddToThisContainer( pxCurrentElement ) )
		{
			Asura_Assert( false, "Failed to add new element to entry!" );
		}
	}
	return pxCurrentElement;
}

// returns true if the values in this are different from pxEntry
bool Asura_GUIMenu_Widget_ListBoxEntry::IsDifferent( const Asura_UnicodeChar* pwszTextString ) const
{
	bool bIsDifferent(false);

	Asura_GUIMenu_Root* pxCurrentElement = static_cast<Asura_GUIMenu_Root*> (GetContainerItem( s_uPopulationItem ));

	if( pxCurrentElement && (pxCurrentElement->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT) )
	{
		// this element claims to be a text element - static cast and hope for the best - NDB
		Asura_GUIMenu_Element_Text* pxTextElement = static_cast< Asura_GUIMenu_Element_Text* >( pxCurrentElement );
	
		// don't bother with expensive string cmp if we already think the data has changed - NDB
		if( pwszTextString && pxTextElement->GetText() )
		{
			bIsDifferent |= wcscmp( pxTextElement->GetText(),pwszTextString ) != 0;
		}
		else 
		{
			// account for case where one string is NULL
			bIsDifferent |= pwszTextString != pxTextElement->GetText();
		}
	}
	else
	{
		bIsDifferent = true; // GUI menu element types are not the same
	}

	return bIsDifferent;
}

// returns true if the values in this are different from pxEntry
bool Asura_GUIMenu_Widget_ListBoxEntry::IsDifferent( const u_int uSkinID, const u_int uAnimID, const u_int uAnimFlags ) const
{
	bool bIsDifferent(false);

	Asura_GUIMenu_Root* pxCurrentElement = static_cast<Asura_GUIMenu_Root*> (GetContainerItem( s_uPopulationItem ));

	if( pxCurrentElement && (pxCurrentElement->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_ANIMATION) )
	{
		// this element claims to be an anim - static cast and hope for the best - NDB
		Asura_GUIMenu_Element_Animation* pxAnimElement = static_cast< Asura_GUIMenu_Element_Animation* >( pxCurrentElement );
		
		bIsDifferent |= uSkinID		!= pxAnimElement->GetSkinID();
		bIsDifferent |= uAnimID		!= pxAnimElement->GetAnimID();
		bIsDifferent |= uAnimFlags	!= pxAnimElement->GetAnimFlags();
	}
	else
	{
		bIsDifferent = true; // GUI menu element types are not the same
	}

	return bIsDifferent;
}

// returns true if the values in this are different from pxEntry
bool Asura_GUIMenu_Widget_ListBoxEntry::IsDifferent( const Asura_GUIMenu_ListBoxEntryDesc_Image& xImageDesc ) const
{
	bool bIsDifferent(false);

	Asura_GUIMenu_Root* pxCurrentElement = static_cast<Asura_GUIMenu_Root*> (GetContainerItem( s_uPopulationItem ));

	if( pxCurrentElement && (pxCurrentElement->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE) )
	{
		// this element claims to be an image - static cast and hope for the best - NDB
		Asura_GUIMenu_Element_Image* pxImageElement = static_cast< Asura_GUIMenu_Element_Image* >( pxCurrentElement );
		
		bIsDifferent |= static_cast<int>(pxImageElement->GetTexture())	!= xImageDesc.GetTextureID();
		bIsDifferent |= pxImageElement->GetUV(0)		!= xImageDesc.GetTopLeftUV();
		bIsDifferent |= pxImageElement->GetUV(1)		!= xImageDesc.GetBottomRightUV();

		if( !bIsDifferent && xImageDesc.HasRenderCoords() )
		{
			Asura_Vector_2 xRenderPos;
			xRenderPos.x = xImageDesc.GetPosX();
			xRenderPos.y = xImageDesc.GetPosY();

			Asura_Bounding_Rectangle xBounds;
			xBounds.SetTopLeft(xRenderPos);
			xBounds.SetBottomRight(xRenderPos + xImageDesc.GetDims());

			bIsDifferent |= pxImageElement->GetUnscaledLocalBounds() != xBounds;
		}			
	}
	else
	{
		bIsDifferent = true; // GUI menu element types are not the same
	}

	return bIsDifferent;
}

void Asura_GUIMenu_Widget_ListBoxEntry::AddTextElement( const Asura_UnicodeChar* pwszTextString )
{	
	m_bDataChanged = IsDifferent( pwszTextString );

	if( HasDataChanged() )
	{
		Asura_GUIMenu_Element_Text* pxTemplateElement = static_cast<Asura_GUIMenu_Element_Text*> (s_pxPopulationTemplate->GetContainerItem(s_uPopulationItem));
		if( pxTemplateElement )
		{
			Asura_GUIMenu_Element_Text* pxTextElement = static_cast<Asura_GUIMenu_Element_Text*> (GetElementToPopulate(pxTemplateElement));
			if( pxTextElement )
			{
				pxTextElement->SetText( pwszTextString );			
			}
		}
	}
}

void Asura_GUIMenu_Widget_ListBoxEntry::AddAnimElement( const Asura_UnicodeChar* pwszAnimString )
{	
	Asura_Hash_ID uSkinID = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uAnimID = ASURA_HASH_ID_UNSET;
	u_int uAnimFlags = 0;

	Asura_GUIMenu_ListBox_PopulationFuncParser xParser( pwszAnimString );

	xParser.GetValue( &uSkinID );
	xParser.GetValue( &uAnimID );
	xParser.GetValue( &uAnimFlags );

	m_bDataChanged = IsDifferent( uSkinID,uAnimID,uAnimFlags );

	if( HasDataChanged() )
	{
		Asura_GUIMenu_Element_Animation* pxTemplateElement = static_cast<Asura_GUIMenu_Element_Animation*> (s_pxPopulationTemplate->GetContainerItem(s_uPopulationItem));
		if( pxTemplateElement )
		{
			Asura_GUIMenu_Element_Animation* pxAnimElement = static_cast<Asura_GUIMenu_Element_Animation*> (GetElementToPopulate(pxTemplateElement));
			if( pxAnimElement )
			{
				

				pxAnimElement->SetSkin( uSkinID );
				pxAnimElement->SetAnim( uAnimID, uAnimFlags );
			}
		}
	}
}

void Asura_GUIMenu_Widget_ListBoxEntry::AddImageElement( const Asura_GUIMenu_ListBoxEntryDesc_Image& xImageDesc )
{	
	m_bDataChanged = IsDifferent( xImageDesc );

	if( HasDataChanged() )
	{
		Asura_GUIMenu_Element_Image* pxTemplateElement = static_cast<Asura_GUIMenu_Element_Image*> (s_pxPopulationTemplate->GetContainerItem(s_uPopulationItem));
		if( pxTemplateElement )
		{
			Asura_GUIMenu_Element_Image* pxImageElement = static_cast<Asura_GUIMenu_Element_Image*> (GetElementToPopulate(pxTemplateElement));
			if( pxImageElement )
			{
                int iTextureID = xImageDesc.GetTextureID();
                if( iTextureID != ASURA_TEXTURE_INVALID )
                {
				    pxImageElement->SetTexture( iTextureID );
                }

				pxImageElement->SetUV( 0, xImageDesc.GetTopLeftUV() );
				pxImageElement->SetUV( 1, xImageDesc.GetBottomRightUV() );

				if( xImageDesc.HasRenderCoords() )
				{
					Asura_Vector_2 xRenderPos;
					xRenderPos.x = xImageDesc.GetPosX();
					xRenderPos.y = xImageDesc.GetPosY();

					Asura_Bounding_Rectangle xBounds;
					xBounds.SetTopLeft(xRenderPos);
					xBounds.SetBottomRight(xRenderPos + xImageDesc.GetDims());
					pxImageElement->SetUnscaledLocalBounds( xBounds );
				}			
			}
		}
	}
}


void Asura_GUIMenu_Widget_ListBoxEntry::AddImageElementOldCallback( const Asura_UnicodeChar* wszPopulationData)
{
	Asura_Assert(false, "image element population function must be updated");
}

bool Asura_GUIMenu_Widget_ListBoxEntry::ShouldUsePopulationFunction( const Asura_GUIMenu_ConsoleCmd* pxPopFunction ) const
{
	if( !pxPopFunction ) return false;
	if( !pxPopFunction->m_xConsoleCmdName.GetLength() ) return false;
#ifdef ASURA_TOOLS
	if( !pxPopFunction->m_xConsoleCmdName.Compare(ASURA_GUIMENU_CONSOLEVAR_NONE) ) return false;
#endif					
	
	return true;
}

void Asura_GUIMenu_Widget_ListBoxEntry::VisibilityCallback( const Asura_UnicodeChar* wszOutput )
{
	if (wszOutput && wszOutput[0] == '1')
	{
		s_bVisibileFromCallback = true;
	}
	else if (wszOutput && wszOutput[0] == '0')
	{
		s_bVisibileFromCallback = false;
	}
	else
	{
		Asura_Assert(0, "unhandled data from visibility command");
	}
}


bool Asura_GUIMenu_Widget_ListBoxEntry::PopulateElements( u_int uEntry, const Asura_GUIMenu_Widget_ListBoxEntry& xTemplate )
{
	char acBuffer[12];
	Asura_PrintfToBfr( acBuffer, "%u", 12, true, uEntry );

	s_pxPopulationEntry    = this;
	s_pxPopulationTemplate = &xTemplate;

	bool bValidEntry = false;

	const u_int uNumItems = xTemplate.GetContainerCount();
	for( s_uPopulationItem=0; s_uPopulationItem<uNumItems; ++s_uPopulationItem )
	{	
		Asura_GUIMenu_Root* pxTemplateItem = xTemplate.GetContainerItem(s_uPopulationItem);
		Asura_GUIMenu_ConsoleCmd* pxPopFunction = xTemplate.GetPopulationFunctionForItem( pxTemplateItem->GetID() );
		Asura_GUIMenu_ConsoleCmd* pxVisFunction = xTemplate.GetVisibilityFunctionForItem( pxTemplateItem->GetID() );

		bool bVisible = true;
		if (ShouldUsePopulationFunction(pxVisFunction))
		{
			s_bVisibileFromCallback = false;
			bValidEntry |= pxVisFunction->Execute( acBuffer, VisibilityCallback );
			if (!s_bVisibileFromCallback)
			{
				bVisible = false;
			}
#ifdef ASURA_TOOLS
			// in tools preview, visibility functions are likely not to exist, so alternate visibility
			if (Tools_Mode_GUIMenu::IsPreview() && !bValidEntry)
			{
				const bool abVisibility[8] = { true, false, false, true, false, true, true, false };
				static u_int uArrayEntry = 0;
				static u_int uLastFrameNumber = 0;
				// reset array every frame
				if (uLastFrameNumber != Asura_Timers::GetFrameNumber())
				{
					uLastFrameNumber = Asura_Timers::GetFrameNumber();
					uArrayEntry = 0;
				}
				bVisible = abVisibility[uArrayEntry];
				uArrayEntry = (uArrayEntry + 1) & 7;
			}
#endif
		}
		if( pxTemplateItem )
		{
			Asura_GUIMenu_Element_Base* pxElement = GetElementToPopulate(static_cast<Asura_GUIMenu_Element_Base*>(pxTemplateItem));
			if (pxElement)
			{
				pxElement->SetRenderingDisabledFlag(!bVisible);
			}
		}
		switch( pxTemplateItem->GetType() )
		{
			case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT :
			{								
				if( ShouldUsePopulationFunction(pxPopFunction) )
				{
#ifdef ASURA_TOOLS
					// in tools preview, populate using the specified text as population functions are unlikely to work.
					if (Tools_Mode_GUIMenu::IsPreview())
					{
						Asura_UnicodeChar awcText[64];
						if (Tools_GUIMenu_PreviewOverlay::GetPopulationText()[0] == '1')
						{
							Asura_PrintfToBfr( awcText, L"%u%S", 64, true, uEntry + 1, Tools_GUIMenu_PreviewOverlay::GetPopulationText() + 1 );
						}
						else
						{
							Asura_PrintfToBfr( awcText, L"%S", 64, true, Tools_GUIMenu_PreviewOverlay::GetPopulationText() );
						}
						AddTextElementCallback(awcText);
					}
					else
#endif // ASURA_TOOLS
					{
						// get text from population function
						bValidEntry |= pxPopFunction->Execute( acBuffer, AddTextElementCallback );
					}
				}
				else
				{
					// just get data from template
					Asura_GUIMenu_Element_Text* pxTextTemplate = static_cast<Asura_GUIMenu_Element_Text*> (pxTemplateItem);
					if( pxTextTemplate->GetText() )
					{
						AddTextElement( pxTextTemplate->GetText() );
					}
					else
					{
						AddTextElement( pxTextTemplate->GetTextUsingTextID() );
					}					
				}

				break;
			}

			case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_ANIMATION :
			{								
				if( ShouldUsePopulationFunction(pxPopFunction) )
				{
					// get text from population function
					bValidEntry |= pxPopFunction->Execute( acBuffer, AddAnimElementCallback );
				}
				else
				{
					// just get data from template
					Asura_GUIMenu_Element_Animation* pxAnimTemplate = static_cast<Asura_GUIMenu_Element_Animation*> (pxTemplateItem);
					
					Asura_UnicodeChar wszTemp[64];
					Asura_PrintfToBfr( wszTemp, L"%d %d %d", 64, true, pxAnimTemplate->GetSkinID(), pxAnimTemplate->GetAnimID(), pxAnimTemplate->GetAnimFlags() );

					AddAnimElement( wszTemp );
				}

				break;
			}

			case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE :
			{								
				if( ShouldUsePopulationFunction(pxPopFunction) )
				{
					// get text from population function
					bValidEntry |= pxPopFunction->Execute( acBuffer, AddImageElementOldCallback );
				}
				else
				{
					// just get data from template
					Asura_GUIMenu_Element_Image* pxImageTemplate = static_cast<Asura_GUIMenu_Element_Image*> (pxTemplateItem);
					Asura_GUIMenu_ListBoxEntryDesc_Image xImageDesc(pxImageTemplate->GetTexture(), 
						pxImageTemplate->GetUV(0).u, 
						pxImageTemplate->GetUV(0).v,
						pxImageTemplate->GetUV(1).u,
						pxImageTemplate->GetUV(1).v );

					AddImageElement( xImageDesc );
				}

				break;
			}

			// add new element types here

			default : 
			{
				Asura_Assertf( false, ("Unhandled element type in listbox '%s' entry template. Type '%d' is not yet handled.",m_pxParentListBox->GetNameAsChar(),pxTemplateItem->GetType()) ) ;
				break;
			}
		}

#ifdef ASURA_TOOLS
		// in tools preview, population functions are likely to return no valid entries, so pretend they're all valid.
		if (Tools_Mode_GUIMenu::IsPreview())
		{
			bValidEntry = true;
		}
#endif
	}

	s_pxPopulationEntry    = NULL;
	s_pxPopulationTemplate = NULL;
	s_uPopulationItem	   = 0;

	return bValidEntry;
}




bool Asura_GUIMenu_Widget_ListBoxEntry::AddToThisContainer( Asura_GUIMenu_Root* pxObject )
{
	if( !PARENT::AddToThisContainer( pxObject ) ) 
	{
		return false;
	}

	// if this is the entry template (m_pxParentListBox is non-null), add a 
	// population function to the list in the list box
	if( IsEntryTemplate() )
	{
		pxObject->GenerateHashID(); // need to make sure the hash ID has been generated
		m_pxParentListBox->AddPopulationFunction( pxObject->GetID() );
		m_pxParentListBox->AddVisibilityFunction( pxObject->GetID() );
	}

	return true;
}



bool Asura_GUIMenu_Widget_ListBoxEntry::RemoveFromThisContainer( Asura_GUIMenu_Root* pxObject )
{
	if( !PARENT::RemoveFromThisContainer( pxObject ) ) 
	{
		return false;
	}

	if( IsEntryTemplate() )
	{
		// remove the population function that corresponds to this item
		m_pxParentListBox->RemovePopulationFunction( pxObject->GetID() );
		m_pxParentListBox->RemoveVisibilityFunction( pxObject->GetID() );
	}

	return true;
}

void Asura_GUIMenu_Widget_ListBoxEntry::ChildIDChanged( Asura_Hash_ID uOldID, Asura_Hash_ID uNewID )
{
	PARENT::ChildIDChanged( uOldID, uNewID );

	if( IsEntryTemplate() )
	{
		// need to re-map existing population function to new hash ID
		m_pxParentListBox->RemapPopulationFunction( uOldID, uNewID );
		m_pxParentListBox->RemapVisibilityFunction( uOldID, uNewID );
	}
}



Asura_GUIMenu_ConsoleCmd* Asura_GUIMenu_Widget_ListBoxEntry::GetPopulationFunctionForItem( Asura_Hash_ID uItem ) const
{
	Asura_Assert( IsEntryTemplate(), "Not an entry template: Shouldn't be calling this function!" );
	if( !m_pxParentListBox )
	{		
		return NULL;
	}

	return m_pxParentListBox->GetPopulationFunctionForItem(uItem);
}

Asura_GUIMenu_ConsoleCmd* Asura_GUIMenu_Widget_ListBoxEntry::GetVisibilityFunctionForItem( Asura_Hash_ID uItem ) const
{
	Asura_Assert( IsEntryTemplate(), "Not an entry template: Shouldn't be calling this function!" );
	if( !m_pxParentListBox )
	{		
		return NULL;
	}

	return m_pxParentListBox->GetVisibilityFunctionForItem(uItem);
}

void Asura_GUIMenu_Widget_ListBoxEntry::ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion )
{
	PARENT::ReadFromChunkStream( xStream, pxParentObject, 0xffffffff );
	// NY - 11 September 2008 - offset used to be ignored - reset it when loading old data, so that behaviour is preserved
	if (uChunkVersion < 1)
	{
		SetUnscaledLocalPos(Asura_Maths::ZeroVector2);
	}

	ReadAllChildObjects( xStream, this );
	
	// NY - 26 August 2008 - make sure this is set up correctly
	if (!GetParentObject())
	{
		SetParentObject(m_pxParentListBox);
	}
}


#ifdef ASURA_TOOLS

void Asura_GUIMenu_Widget_ListBoxEntry::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	WriteLocalChunkHeader( xStream, ASURA_CHUNK_WIDGET_LISTBOXENTRY_VERSION, GetType() );

	PARENT::WriteToChunkStream( xStream );

	// -- write out any children --
	WriteChildObjectData( xStream );
}

void Asura_GUIMenu_Widget_ListBoxEntry::ExportToXML( Tools_GUIMenu_XMLExporter* pxExporter ) const
{
	PARENT::ExportToXML( pxExporter );

	//no data to export, don't bother
	//pxExporter->AddToBuffer( pxExporter->GetTabs() );
	//pxExporter->AddToBuffer( "<ListBoxEntry_data>\n" );
	//pxExporter->IncTabs();
	//data goes here?
	//pxExporter->DecTabs();
	//pxExporter->AddToBuffer( pxExporter->GetTabs() );
	//pxExporter->AddToBuffer( "</ListBoxEntry_data>\n" );
	
}

#endif // ASURA_TOOLS

Asura_GUIMenu_Root*	Asura_GUIMenu_Widget_ListBoxEntry::GetParentObjectForIDGeneration() const
{	
	return m_pxParentListBox;	
}
void Asura_GUIMenu_Widget_ListBoxEntry::SetParentListBox( Asura_GUIMenu_Widget_ListBox* pxParent )
{
	m_pxParentListBox = pxParent;
	SetParentObject(pxParent);
}


// Asura_GUIMenu_Widget_ListBox ///////////////////////////////////////////////////////////////////////////

// Constructor
Asura_GUIMenu_Widget_ListBox::Asura_GUIMenu_Widget_ListBox(void)
: m_fElapsedTime( 0.0f )
, m_uButtonRepeatPhase( 0 )
, m_fCurrentRepeatDelay( 0 )
, m_fInitialRepeatDelay( fASURA_GUIMENU_WIDGET_LISTBOX_CONTROL_INITIAL_REPEAT_DELAY )
, m_fSecondaryRepeatDelay( fASURA_GUIMENU_WIDGET_LISTBOX_CONTROL_SECONDARY_REPEAT_DELAY )
, m_pxEntries( NULL )
, m_uEntryCount( 0 )
, m_pxDecImage( NULL)
, m_pxIncImage( NULL)
, m_pxScrollBar( NULL)
, m_bDragging(false)
, m_xFlashColour(1.0f, 1.0f, 1.0f, 1.0f)
, m_fDecFlashTimerCountdown( 0.0f )
, m_fIncFlashTimerCountdown( 0.0f )
, m_fFlashTimerStartVal( 0.1f )
, m_uSelectedIndex( 0 )
, m_fDisplayListOffset( 0.0f )
, m_fDisplayTargetOffset( 0.0f )
, m_fMoveDir( 0.0f )
, m_uMaxAllowedEntries( 1 )
, m_uMaxDisplayedEntries( 1 )
, m_fEntryOffset( 0.0f )
, m_fBoxSizeAgainstOffset( 0.0f )
, m_bDirtyRect( false )
, m_uListBoxFlags( 0 ) 
, m_xOffsetVector(Asura_Vector_2(0.0f, 1.0f))
{
	m_uType = ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX;
	m_xEntryTemplate.SetParentListBox( this );

	ASURA_GUIMENU_PROFILE_COUNTINSTANCE(ASURA_GUIMENU_COUNTER__WIDGET_LISTBOX);
}

// Destructor
Asura_GUIMenu_Widget_ListBox::~Asura_GUIMenu_Widget_ListBox(void)
{
	DeallocateEntryList();
	
	ASURA_GUIMENU_PROFILE_COUNTDESTROYINSTANCE(ASURA_GUIMENU_COUNTER__WIDGET_LISTBOX);
}



void Asura_GUIMenu_Widget_ListBox::Render( u_int uState)
{
	if( IsRenderingDisabled() ) return;

	ASURA_GUIMENU_PROFILE_START_TASK(ASURA_GUIMENU_PROFILE__WIDGET_LISTBOX);

	u_int uDecColBackup = 0xffffffff;
	u_int uIncColBackup = 0xffffffff;

	if( m_pxDecImage)
	{
		uDecColBackup = m_pxDecImage->GetColour().ToUIntRGBA();

		if (m_pxDecImage->GetModifierCount())
		{
			if (m_fDecFlashTimerCountdown > 0.0f)
			{
				m_pxDecImage->SetStateOverride(m_pxDecImage->GetStateOverride() | ASURA_GUIMENU_WIDGET_STATE_CLICKED);
			}
			else
			{
				m_pxDecImage->SetStateOverride(m_pxDecImage->GetStateOverride() & ~ASURA_GUIMENU_WIDGET_STATE_CLICKED);
			}
		}
		else
		{
			if( m_fDecFlashTimerCountdown > 0.0f)
			{
				Asura_RGBA xColour = GetFlashColour();
				m_pxDecImage->SetColour( &xColour);
			}
		}
	}

	if( m_pxIncImage)
	{
		uIncColBackup = m_pxIncImage->GetColour().ToUIntRGBA();

		if (m_pxIncImage->GetModifierCount())
		{
			if (m_fIncFlashTimerCountdown > 0.0f)
			{
				m_pxIncImage->SetStateOverride(m_pxIncImage->GetStateOverride() | ASURA_GUIMENU_WIDGET_STATE_CLICKED);
			}
			else
			{
				m_pxIncImage->SetStateOverride(m_pxIncImage->GetStateOverride() & ~ASURA_GUIMENU_WIDGET_STATE_CLICKED);
			}
		}
		else
		{
			if( m_fIncFlashTimerCountdown > 0.0f)
			{
				Asura_RGBA xColour = GetFlashColour();
				m_pxIncImage->SetColour( &xColour);
			}
		}
	}

	if (m_pxScrollBar)
	{
		if (m_uEntryCount <= m_uMaxDisplayedEntries)
		{
			m_pxScrollBar->SetStateOverride(m_pxScrollBar->GetStateOverride() | ASURA_GUIMENU_WIDGET_STATE_DISABLED);
		}
		else
		{
			m_pxScrollBar->SetStateOverride(m_pxScrollBar->GetStateOverride() & ~ASURA_GUIMENU_WIDGET_STATE_DISABLED);
		}
		if (m_bDragging)
		{
			m_pxScrollBar->SetStateOverride(m_pxScrollBar->GetStateOverride() | ASURA_GUIMENU_WIDGET_STATE_CLICKED);
		}
		else
		{
			m_pxScrollBar->SetStateOverride(m_pxScrollBar->GetStateOverride() & ~ASURA_GUIMENU_WIDGET_STATE_CLICKED);
		}
	}

	PARENT::Render( uState);
	Asura_Render::FlushDisplay();

	if( m_pxDecImage)
	{
		m_pxDecImage->SetColour( uDecColBackup);
	}

	if( m_pxIncImage)
	{
		m_pxIncImage->SetColour( uIncColBackup);
	}


	for( u_int u = 0; u < m_uEntryCount; ++u )
	{
		float fScroll = (u - m_fDisplayTargetOffset) * GetEntryOffset();
		if (m_bCarousel
			&& m_bLoopSelection)
		{
			if (fScroll <= -0.1f * GetEntryOffset())
			{
				fScroll += m_uEntryCount * GetEntryOffset();
			}
			if (fScroll >= ((m_uMaxDisplayedEntries - 0.9f) * GetEntryOffset()))
			{
				fScroll -= m_uEntryCount * GetEntryOffset();
			}
		}
		if ((fScroll >= -0.1f * GetEntryOffset()
			&& fScroll <= ((m_uMaxDisplayedEntries - 0.9f) * GetEntryOffset()))
			|| (m_bAllowSubScroll
			&& fScroll >= -1.0f * GetEntryOffset()
			&& fScroll <= (m_uMaxDisplayedEntries * GetEntryOffset())))
		{
			// ensure only the selected entry renders as highlighted
			Asura_GUIMenu_Widget_State eWidgetState =
				(u == m_uSelectedIndex && GetCurrentState() == ASURA_GUIMENU_WIDGET_STATE_HILIGHTED) ? ASURA_GUIMENU_WIDGET_STATE_HILIGHTED : ASURA_GUIMENU_WIDGET_STATE_DEFAULT;

			m_pxEntries[u].SetCurrentState( eWidgetState );			
			m_pxEntries[u].SetRenderingDisabledFlag(m_pxEntries[u].IsHiddenByConditionVar());
		}
		else
		{
			m_pxEntries[u].SetRenderingDisabledFlag(true);
		}
	}

	for( u_int u = 0; u < m_uEntryCount; ++u )
	{
		m_pxEntries[u].Render( uState );
	}

	ASURA_GUIMENU_PROFILE_END_TASK();
}

void Asura_GUIMenu_Widget_ListBox::Update( float fElapsedTime)
{
	ASURA_GUIMENU_PROFILE_START_TASK(ASURA_GUIMENU_PROFILE__WIDGET_LISTBOX);

	m_fElapsedTime = fElapsedTime;
	// SK 14.03.07 - the state of this widget is managed in the WidgetBase::Update() function.
	// This should now mean that this widget will handle states correctly, in case of modifiers.
	PARENT::Update( fElapsedTime);

	// (MH - 19.09.2008) don't want to update if hidden
	if (IsHiddenByConditionVar())
	{
		ASURA_GUIMENU_PROFILE_END_TASK();
		return;
	}

	// create entries if none exist
	if( !m_pxEntries )
	{
		AllocateEntryList( GetMaxAllowedEntries() );
	}	
	
	u_int uOldCount = m_uEntryCount;
	ClearAllEntries();

	m_bEntryDataChanged = false;

	for( u_int u=0; u<m_uMaxAllowedEntries; ++u )
	{
		if( !m_pxEntries[u].PopulateElements( u, m_xEntryTemplate ) )
		{
			// populate returned false, no point adding this element
			break;
		}

		// record any change in entry caused by pop function that would require a resize
		m_bEntryDataChanged |= m_pxEntries[u].HasDataChanged();

		// valid entry, update it
		m_pxEntries[u].Update( fElapsedTime );
		m_uEntryCount++;
	}

	if( m_uSelectedIndex >= m_uEntryCount )
	{
		m_uSelectedIndex = m_uEntryCount - 1;
		FocusOnSelected();
	}
	else if (uOldCount != m_uEntryCount)
	{
		FocusOnSelected();
	}

	if( GetCurrentState() != ASURA_GUIMENU_WIDGET_STATE_DISABLED)
	{
		if (m_bCarousel)
		{
			// JL - 03 February 2009 - Clamp the selection so that non-looping carousels dont show any empty entries at the top and bottom.
			if ( !m_bLoopSelection )
			{
				u_int uMinSelectableIndex = m_uMaxDisplayedEntries / 2u;
				u_int uMaxSelectableIndex = m_uEntryCount - ( m_uMaxDisplayedEntries / 2u );
				if ( m_uMaxDisplayedEntries % 2 != 0 ) uMaxSelectableIndex--;

				if ( uMaxSelectableIndex < uMinSelectableIndex )
				{
					m_uSelectedIndex = uMinSelectableIndex;
				}
				else
				{
					m_uSelectedIndex = Asura_Maths::ClampToRange( m_uSelectedIndex, uMinSelectableIndex, uMaxSelectableIndex );
				}
			}

			m_fDisplayListOffset = static_cast<float>(static_cast<int>(m_uSelectedIndex) - static_cast<int>(m_uMaxDisplayedEntries / 2));
			m_fMoveDir = m_fDisplayListOffset - m_fDisplayTargetOffset;
			if (Asura_Maths::Abs(m_fMoveDir) < 1.0f)
			{
				m_fMoveDir = Asura_Maths::Sign(m_fMoveDir);
			}
		}
		if( m_fMoveDir != 0.0f)
		{
			m_bDirtyRect = true;
			m_fDisplayTargetOffset += 7.0f * fElapsedTime * m_fMoveDir;

			if( m_fMoveDir > 0.0f && m_fDisplayTargetOffset > m_fDisplayListOffset)
			{
				m_fDisplayTargetOffset = m_fDisplayListOffset;
				m_fMoveDir = 0.0f;
			}

			if( m_fMoveDir < 0.0f && m_fDisplayTargetOffset < m_fDisplayListOffset)
			{
				m_fDisplayTargetOffset = m_fDisplayListOffset;
				m_fMoveDir = 0.0f;
			}
		}
	}

	// NY - 17 October 2008 - if only one item being display always select it
	if (m_uMaxDisplayedEntries == 1)
	{
		m_uSelectedIndex = Asura_Maths::ClampToRange(static_cast<u_int>(Asura_Maths::FloatToInt_Near(m_fDisplayListOffset)), 0u, m_uMaxAllowedEntries - 1u);
	}

	if(!m_xCurrentValueVar.IsBlank())
	{
		m_xCurrentValueVar.SetIntValue(static_cast<int>(m_uSelectedIndex));
	}

	UpdateIcons();
	UpdateFlashingElements();

	// update position of elements and resize iff data has changed
	if( HasDataChanged() )
	{
		Asura_Vector_2 xRenderPos( Asura_Maths::ZeroVector2 );
		float fScroll( 0.0f );

		for( u_int u = 0; u < m_uEntryCount; ++u )
		{
			fScroll = (u - m_fDisplayTargetOffset) * GetEntryOffset();

			m_pxEntries[u].SetUnscaledLocalPos( m_xOffsetVector * fScroll + m_xEntryTemplate.GetUnscaledLocalPos() );		 // fix up local position
		}

		SetScaledWorldBounds(GetScaledWorldBounds());
		
		m_bDirtyRect = false;
	}

	ASURA_GUIMENU_PROFILE_END_TASK();
}

void Asura_GUIMenu_Widget_ListBox::Copy( const Asura_GUIMenu_Root* pxSource )
{
	PARENT::Copy( pxSource );

	const Asura_GUIMenu_Widget_ListBox* pxSourceListBox = static_cast<const Asura_GUIMenu_Widget_ListBox*> (pxSource);

	SetMaxDisplayedEntries( pxSourceListBox->GetMaxDisplayedEntries() );
	SetMaxAllowedEntries( pxSourceListBox->GetMaxAllowedEntries() );

	m_xCommandManager.Copy( &pxSourceListBox->m_xCommandManager );

	//copy template entry
	m_xEntryTemplate.Copy( &pxSourceListBox->m_xEntryTemplate );
	m_xOffsetVector = pxSourceListBox->m_xOffsetVector;

	SetFlashColour( pxSourceListBox->m_xFlashColour);
	SetFlashTime( pxSourceListBox->m_fFlashTimerStartVal);

	m_xInitValueVar.Copy(&pxSourceListBox->m_xInitValueVar);
	m_xCurrentValueVar.Copy(&pxSourceListBox->m_xCurrentValueVar);
	m_uListBoxFlags = pxSourceListBox->m_uListBoxFlags;

	//copying of population functions has to be done tools-side, unfortunately
	//pxSourceListBox->m_xPopulationFunctionTree.Traverse( Traverse_Copy, &m_xPopulationFunctionTree );
}

void Asura_GUIMenu_Widget_ListBox::UpdateIcons()
{
	bool bDisableDisplayDec = false;
	bool bDisableDisplayInc = false;

	if (m_pxScrollBar)
	{
		float fMax = GetMaxListOffset();
		if (!m_bCarousel
			|| !m_bLoopSelection)
		{
			if (m_fDisplayTargetOffset <= 0.0f)
			{
				bDisableDisplayDec = true;
			}
			if( m_fDisplayTargetOffset >= fMax)
			{
				bDisableDisplayInc = true;
			}
		}
		float fVal = 0.0f;
		if (m_bCarousel)
		{
			if (m_uEntryCount)
			{
				fVal = static_cast<float>(m_uSelectedIndex) / static_cast<float>(m_uEntryCount - 1);
			}
		}
		else
		{
			if (fMax > 0.0f)
			{
				fVal = m_fDisplayTargetOffset / fMax;
			}
		}
		m_pxScrollBar->SetCurrentPathValue(fVal);
	}
	else if (!m_bCarousel
		|| !m_bLoopSelection)
	{
		if( m_uSelectedIndex == 0)
		{
			bDisableDisplayDec = true;
		}
		if( m_uSelectedIndex >= m_uMaxAllowedEntries - 1)
		{
			bDisableDisplayInc = true;
		}
	}
	if (m_fDecFlashTimerCountdown > 0.0f)
	{
		bDisableDisplayDec = false;
	}
	if (m_fIncFlashTimerCountdown > 0.0f)
	{
		bDisableDisplayInc = false;
	}

	if( m_pxDecImage)
	{
		if (m_pxDecImage->GetModifierCount())
		{
			if (bDisableDisplayDec)
			{
				m_pxDecImage->SetStateOverride(m_pxDecImage->GetStateOverride() | ASURA_GUIMENU_WIDGET_STATE_DISABLED);
			}
			else
			{
				m_pxDecImage->SetStateOverride(m_pxDecImage->GetStateOverride() & ~ASURA_GUIMENU_WIDGET_STATE_DISABLED);
			}
		}
		else
		{
			m_pxDecImage->SetRenderingDisabledFlag( bDisableDisplayDec | m_pxDecImage->IsHiddenByConditionVar());
		}
	}

	if( m_pxIncImage)
	{
		if (m_pxIncImage->GetModifierCount())
		{
			if (bDisableDisplayInc)
			{
				m_pxIncImage->SetStateOverride(m_pxIncImage->GetStateOverride() | ASURA_GUIMENU_WIDGET_STATE_DISABLED);
			}
			else
			{
				m_pxIncImage->SetStateOverride(m_pxIncImage->GetStateOverride() & ~ASURA_GUIMENU_WIDGET_STATE_DISABLED);
			}
		}
		else
		{
			m_pxIncImage->SetRenderingDisabledFlag( bDisableDisplayInc | m_pxIncImage->IsHiddenByConditionVar());
		}
	}
}

void Asura_GUIMenu_Widget_ListBox::UpdateFlashingElements()
{
	if( m_fDecFlashTimerCountdown > 0.0f)
	{
		m_fDecFlashTimerCountdown -= m_fElapsedTime;
	}

	if( m_fIncFlashTimerCountdown > 0.0f)
	{
		m_fIncFlashTimerCountdown -= m_fElapsedTime;
	}

}

void Asura_GUIMenu_Widget_ListBox::StartDecrementFlash()
{
	m_fDecFlashTimerCountdown = GetFlashTime();
}

void Asura_GUIMenu_Widget_ListBox::StartIncrementFlash()
{
	m_fIncFlashTimerCountdown = GetFlashTime();
}

void Asura_GUIMenu_Widget_ListBox::SetName( const Asura_String& xName )
{
	PARENT::SetName( xName );

	Asura_String xTemplateName = xName;
	xTemplateName += "_Entry";
		
	m_xEntryTemplate.SetName( xTemplateName );
}



void Asura_GUIMenu_Widget_ListBox::SetName( const char* szName )
{
	PARENT::SetName( szName );

	char szTemp[128];
	Asura_PrintfToBfr( szTemp, "%s_Entry", 128, true, szName );
	
	m_xEntryTemplate.SetName( szTemp );
}



bool Asura_GUIMenu_Widget_ListBox::HandleInputAutoRepeat( bool bHeldValue )
{
	bool bRetVal = bHeldValue;

	if( bHeldValue == true )
	{
		switch( m_uButtonRepeatPhase )
		{
		case 1:
			m_fCurrentRepeatDelay -= m_fElapsedTime;
			if( m_fCurrentRepeatDelay < 0.0f )
			{
				m_fCurrentRepeatDelay += m_fSecondaryRepeatDelay;
				m_uButtonRepeatPhase++;
			}
			else
			{
				bRetVal = false;
			}
			break;

		case 2:
			m_fCurrentRepeatDelay -= m_fElapsedTime;
			if( m_fCurrentRepeatDelay < 0.0f )
			{
				m_fCurrentRepeatDelay += m_fSecondaryRepeatDelay;
			}
			else
			{
				bRetVal = false;		
			}
			break;

		default:
			// -- setup case 1: details
			m_uButtonRepeatPhase++;
			m_fCurrentRepeatDelay = m_fInitialRepeatDelay;	
			break;
		}
	}
	else
	{
		m_uButtonRepeatPhase = 0;
		m_fCurrentRepeatDelay = 0.0f;
	}

	return bRetVal;
}

float Asura_GUIMenu_Widget_ListBox::GetMaxListOffset() const
{
	float fMax = Asura_Maths::Max(0.0f, static_cast<float>(m_uEntryCount) - static_cast<float>(m_uMaxDisplayedEntries));
	if (m_bAllowSubScroll && m_fBoxSizeAgainstOffset < m_fEntryOffset)
	{
		fMax += 1.0f - (m_fBoxSizeAgainstOffset / m_fEntryOffset);
	}
	return fMax;
}

bool Asura_GUIMenu_Widget_ListBox::GetCropRectForChildren(Asura_Bounding_Rectangle& xBounds) const
{
	// NY - 03 April 2009 - crop child objects to the edges of the listbox, so they don't overflow when scrolling
	xBounds = m_xScaledWorldBounds;
	return true;
}

u_int Asura_GUIMenu_Widget_ListBox::ProcessInputActivity( const Asura_GUIMenu_Input& xInput )
{
	u_int uButtonsTrapped = 0;
	const u_int uButtonPresses = xInput.m_uButtonPresses;

	u_int uMaxAllowed = GetMaxAllowedEntries();
	u_int uMaxDisplayed = GetMaxDisplayedEntries();

	bool bDecPressed = false;
	bool bIncPressed = false;
	u_int uButtonsToTrap = 0;
	if (m_xOffsetVector.y > 0.7f)
	{
		bDecPressed = ((uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_UP_HELD) != 0);
		bIncPressed = ((uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_DOWN_HELD) != 0);
		uButtonsToTrap = uButtonPresses & (ASURA_GUIMENU_ACTION_BUTTON_UP_HELD | ASURA_GUIMENU_ACTION_BUTTON_DOWN_HELD);
	}
	else if (m_xOffsetVector.y < -0.7f)
	{
		bDecPressed = ((uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_DOWN_HELD) != 0);
		bIncPressed = ((uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_UP_HELD) != 0);
		uButtonsToTrap = uButtonPresses & (ASURA_GUIMENU_ACTION_BUTTON_DOWN_HELD | ASURA_GUIMENU_ACTION_BUTTON_UP_HELD);
	}
	else if (m_xOffsetVector.x > 0.7f)
	{
		bDecPressed = ((uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_LEFT_HELD) != 0);
		bIncPressed = ((uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_RIGHT_HELD) != 0);
		uButtonsToTrap = uButtonPresses & (ASURA_GUIMENU_ACTION_BUTTON_LEFT_HELD | ASURA_GUIMENU_ACTION_BUTTON_RIGHT_HELD);
	}
	else if (m_xOffsetVector.x < -0.7f)
	{
		bDecPressed = ((uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_RIGHT_HELD) != 0);
		bIncPressed = ((uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_LEFT_HELD) != 0);
		uButtonsToTrap = uButtonPresses & (ASURA_GUIMENU_ACTION_BUTTON_RIGHT_HELD | ASURA_GUIMENU_ACTION_BUTTON_LEFT_HELD);
	}
	else
	{
		Asura_Assert(0, "listbox stuck - no valid direction");
	}

	bool bPressed = bDecPressed | bIncPressed;

	if( HandleInputAutoRepeat( bPressed) == false )
	{
		bDecPressed = false;
		bIncPressed = false;
		uButtonsTrapped |= uButtonsToTrap;
	}

	if(xInput.m_bUseCursor && ContainsPos(xInput.m_xCursorPos))
	{
		for(u_int u = 0; u < m_uEntryCount; ++u)
		{
			if ((m_fDisplayTargetOffset - 0.5f) < static_cast<float>(u)
				&& (m_fDisplayTargetOffset - 0.5f + static_cast<float>(m_uMaxDisplayedEntries)) > static_cast<float>(u))
			{
				if (m_pxEntries[u].ContainsPos(xInput.m_xCursorPos))
				{
					if (!m_bCarousel)
					{
						m_uSelectedIndex = u;
					}
					if (uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_CLICK)
					{
						if (m_uSelectedIndex == u)
						{
							uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_CLICK;
							m_xCommandManager.Execute( ASURA_GUIMENU_ACTION_BUTTON_ACCEPT, m_uSelectedIndex, true);
						}
					}
					break;
				}
			}
		}
	}

	if(xInput.m_bUseCursor
		&& (uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_CLICK)
		&& ((uButtonsTrapped & ASURA_GUIMENU_ACTION_BUTTON_CLICK) == 0))
	{
		//start editing if the cursor was clicked within the path element
		if(m_pxScrollBar && m_pxScrollBar->ContainsPos(xInput.m_xCursorPos))
		{
			uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_CLICK;
			m_bDragging = true;
		}

		if(m_pxDecImage && m_pxDecImage->ContainsPos(xInput.m_xCursorPos))
		{
			uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_CLICK;
			if (m_bCarousel)
			{
				bDecPressed = true;
			}
			else
			{
				if (m_fDisplayListOffset > 0.0f)
				{
					float fDelta = 1.0f;
					if (m_bAllowSubScroll && m_fBoxSizeAgainstOffset < m_fEntryOffset)
					{
						fDelta = m_fBoxSizeAgainstOffset / m_fEntryOffset;
					}
					m_fDisplayListOffset = Asura_Maths::Max(m_fDisplayListOffset - fDelta, 0.0f);
					m_fMoveDir = -1.0f;
					StartDecrementFlash();
				}
			}
		}

		if(m_pxIncImage && m_pxIncImage->ContainsPos(xInput.m_xCursorPos))
		{
			uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_CLICK;
			if (m_bCarousel)
			{
				bIncPressed = true;
			}
			else
			{
				float fMax = GetMaxListOffset();
				if (m_fDisplayListOffset < fMax)
				{
					float fDelta = 1.0f;
					if (m_bAllowSubScroll && m_fBoxSizeAgainstOffset < m_fEntryOffset)
					{
						fDelta = m_fBoxSizeAgainstOffset / m_fEntryOffset;
					}
					m_fDisplayListOffset = Asura_Maths::Min(m_fDisplayListOffset + fDelta, fMax);
					m_fMoveDir = 1.0f;
					StartIncrementFlash();
				}
			}
		}
	}

	if(m_bDragging)
	{
		if(xInput.m_bUseCursor && (uButtonPresses & ASURA_GUIMENU_ACTION_BUTTON_CLICK_HELD) && m_pxScrollBar)
		{
			float fPosition = m_pxScrollBar->GetPathValueNearestPosition(xInput.m_xCursorPos);
			if (m_bCarousel)
			{
				if (m_uEntryCount)
				{
					m_uSelectedIndex = static_cast<u_int>(Asura_Maths::ClampToRange(Asura_Maths::FloatToInt_Near(fPosition * (m_uEntryCount - 1)), 0, static_cast<int>(m_uEntryCount) - 1));
				}
			}
			else
			{
				float fMax = GetMaxListOffset();
				m_fDisplayListOffset = fPosition * fMax;
				if (!m_bAllowSubScroll)
				{
					m_fDisplayListOffset = static_cast<float>(Asura_Maths::FloatToInt_Near(m_fDisplayListOffset));
				}
				m_fMoveDir = m_fDisplayListOffset - m_fDisplayTargetOffset;
			}
		}
		else
		{
			m_bDragging = false;
		}
	}

	if( bDecPressed )
	{
		if( (m_uSelectedIndex > 0) || m_bLoopSelection )
		{
			StartDecrementFlash();
			if( m_uSelectedIndex == 0 )
			{
				m_uSelectedIndex = (m_uEntryCount - 1);
				if (m_bCarousel)
				{
					m_fDisplayTargetOffset += static_cast<float>(m_uEntryCount);
					m_fDisplayListOffset += static_cast<float>(m_uEntryCount);
				}
				else
				{
					FocusOnSelected();
				}
			}
			else
			{
				m_uSelectedIndex--;

				if(!m_bCarousel
					&& m_uSelectedIndex < static_cast<u_int>(Asura_Maths::FloatToInt_Near( m_fDisplayListOffset)))
				{
					m_bDirtyRect = true;

					m_fDisplayListOffset -= 1.0f;
					m_fMoveDir = -1.0f;
					if( m_fDisplayListOffset < 0)
					{
						m_fDisplayListOffset = 0;
						m_fMoveDir = 0.0f;
					}
				}
			}
			
			// -- play the navigate sound, we're moving up the list --
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_NAVIGATE);

			if( GetFlag( ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_IMMEDIATE) == true)
			{
				Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_REQUEST, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX, m_uSelectedIndex);
			}
			uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_UP_HELD;
		}
		else if (GetNavExitID(ASURA_GUIMENU_UI_NAV_EXIT_UP) == ASURA_HASH_ID_UNSET)
		{	
			// -- play the failed sound, we're at the top of the list --
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_FAILURE);
			uButtonsTrapped |= uButtonsToTrap;
		}
	}
	else if( bIncPressed )
	{
		if( (m_uSelectedIndex < (m_uEntryCount-1)) || m_bLoopSelection || m_bAllowSubScroll)
		{
			StartIncrementFlash();
			if( m_uSelectedIndex == (m_uEntryCount-1))
			{
				if (m_bLoopSelection)
				{
					m_uSelectedIndex = 0;
					if (m_bCarousel)
					{
						m_fDisplayTargetOffset -= static_cast<float>(m_uEntryCount);
						m_fDisplayListOffset -= static_cast<float>(m_uEntryCount);
					}
					else
					{
						FocusOnSelected();
					}
				}
				else if (m_bAllowSubScroll)
				{
					float fMax = GetMaxListOffset();
					if (m_fDisplayListOffset < fMax)
					{
						m_fDisplayListOffset = Asura_Maths::Min(m_fDisplayListOffset + 1.0f, fMax);
						m_fMoveDir = 1.0f;
						StartIncrementFlash();
					}
				}
			}
			else
			{
				m_uSelectedIndex++;

				if(!m_bCarousel
					&& m_uSelectedIndex >= (static_cast<u_int>(Asura_Maths::FloatToInt_Near(m_fDisplayListOffset)) + GetMaxDisplayedEntries()))
				{
					m_bDirtyRect = true;
					m_fDisplayListOffset += 1.0f;
					m_fMoveDir = 1.0f;
					if( m_fDisplayListOffset > (uMaxAllowed - uMaxDisplayed))
					{
						m_fDisplayTargetOffset = static_cast<float>(uMaxAllowed - uMaxDisplayed);
						m_fMoveDir = 0.0f;
					}
				}
			}
			
			// -- play the navigate sound, we're moving up the list --
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_NAVIGATE);

			if( GetFlag( ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_IMMEDIATE) == true)
			{
				Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_REQUEST, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX, m_uSelectedIndex);
			}
			uButtonsTrapped |= ASURA_GUIMENU_ACTION_BUTTON_DOWN_HELD;
		}
		else if (GetNavExitID(ASURA_GUIMENU_UI_NAV_EXIT_DOWN) == ASURA_HASH_ID_UNSET)
		{
			// -- play the failed sound, we're at the top of the list --
			Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_FAILURE);
			uButtonsTrapped |= uButtonsToTrap;
		}

	}
	// NY - 09 September 2008 - bit of a bodge - need to rewrite each classes definition of this function to use inheritance properly, so they don't need to re-implement parent functionality
	if (!uButtonsTrapped)
	{
		u_int uParentButtonsTrapped = PARENT::ProcessInputActivity( xInput);

		uButtonsTrapped |= uParentButtonsTrapped;
	}

	return uButtonsTrapped;
}


// --------------------------------------------------------------------------------------- 

Asura_GUIMenu_Widget_Base* Asura_GUIMenu_Widget_ListBox::FindSelectableWidgetAtPos(const Asura_Vector_2& xPos)
{
	if(IsSelectable())
	{
		for(u_int u = 0; u < m_uEntryCount; ++u)
		{
			if ((m_fDisplayTargetOffset - 0.5f) < static_cast<float>(u)
				&& (m_fDisplayTargetOffset - 0.5f + static_cast<float>(m_uMaxDisplayedEntries)) > static_cast<float>(u))
			{
				if (m_pxEntries[u].ContainsPos(xPos))
				{
					return this;
				}
			}
		}
	}
	return PARENT::FindSelectableWidgetAtPos(xPos);
}

// --------------------------------------------------------------------------------------- 


bool Asura_GUIMenu_Widget_ListBox::OnInit( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	PARENT::OnInit( pxMessage);

	if(!m_xInitValueVar.IsBlank())
	{
		int iInitVal;
		m_xInitValueVar.GetIntValue(iInitVal);
		m_uSelectedIndex = Asura_Maths::Min(static_cast<u_int>(iInitVal), m_uEntryCount - 1);
		FocusOnSelected();
	}
	else if (m_bAllowSubScroll)
	{
		m_uSelectedIndex = 0;
		FocusOnSelected();
	}

	Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_REQUEST, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_REQUEST, ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX);
	return true;
}



bool Asura_GUIMenu_Widget_ListBox::OnDeInit( const Asura_Message_GUIMenu_Notify* pxMessage)
{
	if( GetFlag( ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_ON_EXIT) == true)
	{
		Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_REQUEST, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX, m_uSelectedIndex);
	}
	return true;
}



bool Asura_GUIMenu_Widget_ListBox::OnDataChange( const Asura_Message_GUIMenu_Notify* pxMessage )
{
	if( pxMessage->uFromID != GetID())
	{
		// -- something external has changed the data --
		if( pxMessage->WidgetType == ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX)
		{
			switch( ASURA_GUIMENU_GET_SUB_MESSAGE( pxMessage->uActionType))
			{
			case ASURA_GUIMENU_LISTBOX_CLEAR:
				ClearAllEntries();
				break;

			case ASURA_GUIMENU_LISTBOX_GET_SELECTION_INDEX:
				Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_REQUEST, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX, m_uSelectedIndex);
				break;

			case ASURA_GUIMENU_LISTBOX_SET_SELECTION_INDEX:
				if( pxMessage->uData < m_uEntryCount)
				{
					m_uSelectedIndex = pxMessage->uData;	
					FocusOnSelected();
				}
				break;

			case ASURA_GUIMENU_LISTBOX_NONE:
			default:
				break;
			}
		}
	}
	return true;
}



void Asura_GUIMenu_Widget_ListBox::FocusOnSelected( void )
{
	u_int uMaxDispEntries = GetMaxDisplayedEntries();

	if (m_uEntryCount)
	{
		m_bDirtyRect = true;
		m_fDisplayListOffset = (float)m_uSelectedIndex;

		if( m_uSelectedIndex > m_uEntryCount - uMaxDispEntries)
		{
			m_fDisplayListOffset = (float)(m_uEntryCount - uMaxDispEntries);
		}

		if( m_uSelectedIndex < uMaxDispEntries)
		{
			m_fDisplayListOffset = 0;
		}


		m_fDisplayTargetOffset = m_fDisplayListOffset;
	}
}



bool Asura_GUIMenu_Widget_ListBox::OnDataRequest( const Asura_Message_GUIMenu_Notify* pxMessage )
{
	if( pxMessage->uFromID != GetID())
	{
		Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_REQUEST, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX, m_uSelectedIndex);
	}
	return true;
}



bool Asura_GUIMenu_Widget_ListBox::OnClicked( const Asura_Message_GUIMenu_Notify* pxMessage )
{
	if( (GetFlag( ASURA_GUIMENU_ACTION_FLAG_NOTIFY_DATA_CHANGE_IMMEDIATE) == true) &&
		(pxMessage->uData == ASURA_GUIMENU_ACTION_BUTTON_ACCEPT))
	{
		Asura_GUIMenu_MessagePump::PostMessage( ASURA_GUIMENU_EXTERNAL_REQUEST, GetID(), ASURA_GUIMENU_UI_MSG_ON_DATA_CHANGE, ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX, m_uSelectedIndex);

		// -- play the navigate sound, we're moving up the list --
		Asura_GUIMenu_Audio::Play( ASURA_GUIMENU_AUDIO_TYPE_ON_NAVIGATE);
	}

	// -- uData is the accept button type
	m_xCommandManager.Execute( pxMessage->uData, m_uSelectedIndex, true );

	return true;
}



bool Asura_GUIMenu_Widget_ListBox::OnRollIn( const Asura_Message_GUIMenu_Notify* pxMessage )
{
	SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_ROLL_ON );
	return true;
}



bool Asura_GUIMenu_Widget_ListBox::OnRollOut( const Asura_Message_GUIMenu_Notify* pxMessage )
{
	SetCurrentState( ASURA_GUIMENU_WIDGET_STATE_ROLL_OFF );
	return true;
}



bool Asura_GUIMenu_Widget_ListBox::OnEnable( const Asura_Message_GUIMenu_Notify* pxMessage )
{
	Asura_GUIMenu_Widget_State eNewState;

	if( pxMessage->uData != 0)
	{	
		if( IsFocusedObject())
		{	
			eNewState = ASURA_GUIMENU_WIDGET_STATE_HILIGHTED;	
		}
		else
		{	
			eNewState = ASURA_GUIMENU_WIDGET_STATE_DEFAULT;	
		}
	}
	else
	{
		eNewState = ASURA_GUIMENU_WIDGET_STATE_DISABLED;	
	}

	SetCurrentState( eNewState );

	return true;

}



void Asura_GUIMenu_Widget_ListBox::SetMaxAllowedEntries( u_int uMaxEntries)
{
	if( m_pxEntries && (uMaxEntries != GetMaxAllowedEntries()) )
	{
		AllocateEntryList( uMaxEntries );
	}	

	m_uMaxAllowedEntries = uMaxEntries;	

	m_bDirtyRect = true;
}



void Asura_GUIMenu_Widget_ListBox::SetSelectedIndex( u_int uSelected)
{
	m_uSelectedIndex = uSelected;

	if (!m_bCarousel)
	{
		FocusOnSelected();
	}
}



void Asura_GUIMenu_Widget_ListBox::SetMaxDisplayedEntries( u_int uMaxDisplayedEntries)
{
	m_uMaxDisplayedEntries = uMaxDisplayedEntries;

	UpdateEntryOffset();
}

void Asura_GUIMenu_Widget_ListBox::SetOffsetVector( Asura_Vector_2 xOffsetVector )
{
	m_xOffsetVector = xOffsetVector;

	UpdateEntryOffset();
}

void Asura_GUIMenu_Widget_ListBox::UpdateEntryOffset()
{
	if (m_uMaxDisplayedEntries > 1)
	{
		const Asura_Bounding_Rectangle& xBounds = GetUnscaledLocalBounds();
		const Asura_Bounding_Rectangle& xTemplateBounds = m_xEntryTemplate.GetUnscaledLocalBounds();
		float fXOffset;
		if (m_xOffsetVector.x > 0.0f)
		{
			fXOffset = (xBounds.GetWidth() - xTemplateBounds.GetWidth() - 2.0f * xTemplateBounds.MinX) / (m_uMaxDisplayedEntries - 1); // update spacing between entries
		}
		else
		{
			fXOffset = (xBounds.GetWidth() - xTemplateBounds.GetWidth() - 2.0f * (xBounds.GetWidth() - xTemplateBounds.MaxX)) / (m_uMaxDisplayedEntries - 1); // update spacing between entries
		}
		if (Asura_Maths::Abs(m_xOffsetVector.x) > 0.0001f)
		{
			fXOffset /= Asura_Maths::Abs(m_xOffsetVector.x);
		}
		else
		{
			fXOffset = 999.0f;
		}

		float fYOffset;
		if (m_xOffsetVector.y > 0.0f)
		{
			fYOffset = (xBounds.GetHeight() - xTemplateBounds.GetHeight() - 2.0f * xTemplateBounds.MinY) / (m_uMaxDisplayedEntries - 1); // update spacing between entries
		}
		else
		{
			fYOffset = (xBounds.GetHeight() - xTemplateBounds.GetHeight() - 2.0f * (xBounds.GetHeight() - xTemplateBounds.MaxY)) / (m_uMaxDisplayedEntries - 1); // update spacing between entries
		}
		if (Asura_Maths::Abs(m_xOffsetVector.y) > 0.0001f)
		{
			fYOffset /= Asura_Maths::Abs(m_xOffsetVector.y);
		}
		else
		{
			fYOffset = 999.0f;
		}

		m_fEntryOffset = Asura_Maths::Max(0.0f, Asura_Maths::Min(fXOffset, fYOffset));
	}
	else if (m_uMaxDisplayedEntries == 1)
	{
		const Asura_Bounding_Rectangle& xTemplateBounds = m_xEntryTemplate.GetUnscaledLocalBounds();
		if (Asura_Maths::Abs(m_xOffsetVector.x) > Asura_Maths::Abs(m_xOffsetVector.y))
		{
			m_fEntryOffset = xTemplateBounds.GetWidth() / Asura_Maths::Abs(m_xOffsetVector.x);
		}
		else
		{
			m_fEntryOffset = xTemplateBounds.GetHeight() / Asura_Maths::Abs(m_xOffsetVector.y);
		}
	}
	else
	{
		m_fEntryOffset = 0.0f;
	}
	if (Asura_Maths::Abs(m_xOffsetVector.x) > Asura_Maths::Abs(m_xOffsetVector.y))
	{
		m_fBoxSizeAgainstOffset = m_xUnscaledLocalBounds.GetWidth() / Asura_Maths::Abs(m_xOffsetVector.x);
	}
	else
	{
		m_fBoxSizeAgainstOffset = m_xUnscaledLocalBounds.GetHeight() / Asura_Maths::Abs(m_xOffsetVector.y);
	}
}

u_int Asura_GUIMenu_Widget_ListBox::GetMaxAllowedEntries( void ) const
{
	return m_uMaxAllowedEntries;	
}



u_int Asura_GUIMenu_Widget_ListBox::GetMaxDisplayedEntries( void ) const
{
	return m_uMaxDisplayedEntries;	
}



void Asura_GUIMenu_Widget_ListBox::SetUnscaledLocalBounds( const Asura_Bounding_Rectangle& xBounds )
{
	PARENT::SetUnscaledLocalBounds(xBounds);

	UpdateEntryOffset();
}


float Asura_GUIMenu_Widget_ListBox::GetInitialControlRepeatDelay( void )
{
	return m_fInitialRepeatDelay;	
}

float Asura_GUIMenu_Widget_ListBox::GetSecondaryControlRepeatDelay()
{
	return m_fSecondaryRepeatDelay;	
}

const Asura_GUIMenu_Element_Image* Asura_GUIMenu_Widget_ListBox::GetDecImage() const
{
	return m_pxDecImage;
}

void Asura_GUIMenu_Widget_ListBox::SetDecImage( Asura_GUIMenu_Element_Image* pxDecImage)
{
	m_pxDecImage = pxDecImage;
}

const Asura_GUIMenu_Element_Image* Asura_GUIMenu_Widget_ListBox::GetIncImage() const
{
	return m_pxIncImage;
}

void Asura_GUIMenu_Widget_ListBox::SetIncImage( Asura_GUIMenu_Element_Image* pxIncImage)
{
	m_pxIncImage = pxIncImage;
}

const Asura_GUIMenu_Element_Path* Asura_GUIMenu_Widget_ListBox::GetScrollBar() const
{
	return m_pxScrollBar;
}

void Asura_GUIMenu_Widget_ListBox::SetScrollBar( Asura_GUIMenu_Element_Path* pxScrollBar)
{
	m_pxScrollBar = pxScrollBar;
}

bool Asura_GUIMenu_Widget_ListBox::RemoveFromThisContainer( Asura_GUIMenu_Root* pxObject)
{
	//Check for one of the children that we have a pointer to being removed.
	if(m_pxDecImage == pxObject) m_pxDecImage = 0;
	if(m_pxIncImage == pxObject) m_pxIncImage = 0;
	if(m_pxScrollBar == pxObject) m_pxScrollBar = 0;

	return PARENT::RemoveFromThisContainer(pxObject);
}

#ifdef ASURA_TOOLS
void Asura_GUIMenu_Widget_ListBox::UpdateLocalDataFromStyle()
{
	PARENT::UpdateLocalDataFromStyle();

	if( m_pxStyleObject != NULL)
	{
		Asura_GUIMenu_Widget_ListBox* pxStyleListBox = static_cast<Asura_GUIMenu_Widget_ListBox*>(m_pxStyleObject);
		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_WIDTH) == 0)
		{
			m_xUnscaledLocalBounds.MaxX = pxStyleListBox->GetUnscaledLocalBounds().GetWidth() + m_xUnscaledLocalBounds.MinX;
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_HEIGHT) == 0)
		{
			m_xUnscaledLocalBounds.MaxY = pxStyleListBox->GetUnscaledLocalBounds().GetHeight() + m_xUnscaledLocalBounds.MinY;
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_MAX_DISPLAY_ENTRIES) == 0)
		{
			SetMaxDisplayedEntries( pxStyleListBox->GetMaxDisplayedEntries());
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_MAX_ALLOWED_ENTRIES) == 0)
		{
			SetMaxAllowedEntries( pxStyleListBox->GetMaxAllowedEntries());
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_CONTROL_REPEAT) == 0)
		{
			m_fInitialRepeatDelay = pxStyleListBox->m_fInitialRepeatDelay;
			m_fSecondaryRepeatDelay = pxStyleListBox->m_fSecondaryRepeatDelay;
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_FLASH_DETAILS) == 0)
		{
			m_fFlashTimerStartVal = pxStyleListBox->m_fFlashTimerStartVal;
			m_xFlashColour = pxStyleListBox->m_xFlashColour;
		}

		if( (m_uStyleUseLocalDataFlags&ASURA_GUIMENU_WIDGET_LISTBOX_LOCAL_FLAG_OFFSET_VECTOR) == 0)
		{
			m_xOffsetVector = pxStyleListBox->m_xOffsetVector;
		}
	}
}
#endif

Asura_GUIMenu_Root* Asura_GUIMenu_Widget_ListBox::GetResizeChild( const u_int u ) 
{
	Asura_GUIMenu_Root* pxRtn(NULL);

	if( u < PARENT::GetResizeChildCount() )
	{
		pxRtn = PARENT::GetResizeChild(u);
	}
	else if( u == PARENT::GetResizeChildCount() )
	{
		pxRtn = &m_xEntryTemplate;
	}
	else if( u < 1 + m_uEntryCount + PARENT::GetResizeChildCount() )
	{
		pxRtn = &m_pxEntries[u - PARENT::GetResizeChildCount() - 1];
	}

	return pxRtn;
}

u_int Asura_GUIMenu_Widget_ListBox::GetResizeChildCount() const 
{ 
	return PARENT::GetResizeChildCount() + 1 + m_uEntryCount;
}

void Asura_GUIMenu_Widget_ListBox::ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion)
{
	float fWidth = 0.0f;
	float fHeight = 0.0f;
	u_int uMaxAllowed;
	u_int uMaxDisplayed;

	PARENT::ReadFromChunkStream( xStream, pxParentObject, 0xffffffff);

	ReadAllChildObjects( xStream, this);

	// read in ListBox specific data --
	if (uChunkVersion < 6)
	{
		xStream >> fWidth;
		xStream >> fHeight;
	}
	xStream >> uMaxAllowed;
	xStream >> uMaxDisplayed;

	m_xCommandManager.ReadFromChunkStream( xStream );

	u_int uJunkType = 0;
	u_int uTemplateVersion = m_xEntryTemplate.ReadLocalChunkHeader( xStream, uJunkType );
	Asura_Assert( (uJunkType == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOXENTRY), "Entry template is incorrect type!" );
	m_xEntryTemplate.ReadFromChunkStream( xStream, NULL, uTemplateVersion );

	if ( uChunkVersion >= 11 )
	{
		u_int uFunctionCountToRead;

		xStream >> uFunctionCountToRead;

		Asura_Assert(uFunctionCountToRead == m_xEntryTemplate.GetContainerCount(),"Number of population functions doesn't match number of container items");

		for( u_int u = 0; u < uFunctionCountToRead; ++u )
		{
			Asura_Hash_ID uID;

			// Read the ID of the population function.
			xStream >> uID;

			// Get the population function object
			Asura_GUIMenu_ConsoleCmd* pxPopFunc = m_xPopulationFunctionTree.GetDataByHash(uID);
			if( pxPopFunc )
			{
				pxPopFunc->ReadFromChunkStream( xStream );
			}
			else
			{
				Asura_Assertf( pxPopFunc, ("Failed to find population function for entry template item '%s' (ID: 0x%x)\nPopulation function data for this item will be lost!",m_xEntryTemplate.GetContainerItem(u)->GetNameAsChar(), uID) );
				
				// if the ID is set but we can't find the function then something's probably gone awry. 
				// not reading anything from the stream will almost certainly trash the stream position
				// and lead to reading in junk. To fix this, create a dummy object and read into that.
				Asura_GUIMenu_ConsoleCmd xDummy;
				xDummy.ReadFromChunkStream( xStream );
			}
		}

		for( u_int u = 0; u < uFunctionCountToRead; ++u )
		{
			Asura_Hash_ID uID;

			// Read the ID of the population function.
			xStream >> uID;

			// Get the population function object
			Asura_GUIMenu_ConsoleCmd* pxVisFunc = m_xVisibilityFunctionTree.GetDataByHash(uID);

			if( pxVisFunc )
			{
				pxVisFunc->ReadFromChunkStream( xStream );
			}
			else
			{
				Asura_Assertf( 0, ("Failed to find visibility function for entry template item '%s' (ID: 0x%x)\nVisibility function data for this item will be lost!",m_xEntryTemplate.GetContainerItem(u)->GetNameAsChar(), uID) );
				
				// if the ID is set but we can't find the function then something's probably gone awry. 
				// not reading anything from the stream will almost certainly trash the stream position
				// and lead to reading in junk. To fix this, create a dummy object and read into that.
				Asura_GUIMenu_ConsoleCmd xDummy;
				xDummy.ReadFromChunkStream( xStream );
			}
		}
	}
	else
	{
		// read in population functions
		if( uChunkVersion >= 1 )
		{	
			for( u_int u=0; u<m_xEntryTemplate.GetContainerCount(); ++u )
			{
				Asura_Hash_ID uID = m_xEntryTemplate.GetContainerItem(u)->GetID();
				
				// RDM - 16/11/07 - At some point I decided it was a good idea to save out the IDs of the
				// population functions - probably to maintain the match-up between element and population function
				// - but since they're added to the list using the a freshly generated hashID when they're added 
				// to the container this hash ID can get horribly out-of-date, leading to the failure case below 
				// where we have to read in the saved data into a dummy console command.
				if( uChunkVersion >= 2 )
				{
					xStream >> uID;
				}

				Asura_GUIMenu_ConsoleCmd* pxPopFunc = m_xPopulationFunctionTree.GetDataByHash(uID);
				if( pxPopFunc )
				{
					pxPopFunc->ReadFromChunkStream( xStream );
				}
				else
				{
					Asura_Assertf( pxPopFunc, ("Failed to find population function for entry template item '%s' (ID: 0x%x)\nPopulation function data for this item will be lost!",m_xEntryTemplate.GetContainerItem(u)->GetNameAsChar(), uID) );
					
					// if the ID is set but we can't find the function then something's probably gone awry. 
					// not reading anything from the stream will almost certainly trash the stream position
					// and lead to reading in junk. To fix this, create a dummy object and read into that.
					Asura_GUIMenu_ConsoleCmd xDummy;
					xDummy.ReadFromChunkStream( xStream );
				}
			}
		}

		// read in visibility functions
		if( uChunkVersion >= 4 )
		{
			for( u_int u=0; u<m_xEntryTemplate.GetContainerCount(); ++u )
			{
				Asura_Hash_ID uID;
				xStream >> uID;
				Asura_GUIMenu_ConsoleCmd* pxVisFunc = m_xVisibilityFunctionTree.GetDataByHash(uID);
				if( pxVisFunc )
				{
					pxVisFunc->ReadFromChunkStream( xStream );
				}
				else
				{
					Asura_Assertf( 0, ("Failed to find visibility function for entry template item '%s' (ID: 0x%x)\nVisibility function data for this item will be lost!",m_xEntryTemplate.GetContainerItem(u)->GetNameAsChar(), uID) );
					
					// if the ID is set but we can't find the function then something's probably gone awry. 
					// not reading anything from the stream will almost certainly trash the stream position
					// and lead to reading in junk. To fix this, create a dummy object and read into that.
					Asura_GUIMenu_ConsoleCmd xDummy;
					xDummy.ReadFromChunkStream( xStream );
				}
			}
		}
	}

	xStream >> m_fInitialRepeatDelay;
	xStream >> m_fSecondaryRepeatDelay;

	if( uChunkVersion >= 3 && uChunkVersion <= 11)
	{
		bool bLoop;
		xStream >> bLoop;
		m_bLoopSelection = bLoop;
	}
	else if (uChunkVersion >= 12)
	{
		Asura_ReadFromStream_Bitfield_Wrapper<u_int> xFlags(m_uListBoxFlags);
		xStream >> xFlags;
	}

	if (uChunkVersion >= 5)
	{
		u_int uHashDecID;
		u_int uHashIncID;
		u_int uScrHash;

		xStream >> uHashDecID;
		xStream >> uHashIncID;
		xStream >> uScrHash;

		Asura_GUIMenu_Root* pxTempRoot = FindContainerItemByID( uHashDecID);
		SetDecImage( static_cast<Asura_GUIMenu_Element_Image*>(pxTempRoot));

		pxTempRoot = FindContainerItemByID( uHashIncID);
		SetIncImage( static_cast<Asura_GUIMenu_Element_Image*>(pxTempRoot));

		pxTempRoot = FindContainerItemByID( uScrHash);
		SetScrollBar( static_cast<Asura_GUIMenu_Element_Path*>(pxTempRoot));
	}

	// init based on settings
	if (uChunkVersion < 6)
	{
		m_xUnscaledLocalBounds.MaxX = fWidth + m_xUnscaledLocalBounds.MinX;
		m_xUnscaledLocalBounds.MaxY = fHeight + m_xUnscaledLocalBounds.MinY;
	}

	if (uChunkVersion >= 7)
	{
		xStream >> m_xOffsetVector;
	}

	if (uChunkVersion >= 8)
	{
		xStream >> m_xFlashColour;

		xStream >> m_fFlashTimerStartVal;
	}

	if (uChunkVersion >= 9)
	{
		// console var
		m_xInitValueVar.ReadFromChunkStream(xStream);
	}

	if (uChunkVersion >= 10)
	{
		// console var
		m_xCurrentValueVar.ReadFromChunkStream(xStream);
	}

	SetMaxDisplayedEntries( uMaxDisplayed);
	SetMaxAllowedEntries( uMaxAllowed);
}



#ifdef ASURA_TOOLS

struct ListBoxFunctionTraverseData
{
	Asura_Chunk_Stream* m_pxStream;
	const Asura_GUIMenu_Widget_ListBox* m_pxListBox;
};

static bool Traverse_WriteToChunkStream( Asura_Hash_ID uHash, Asura_GUIMenu_ConsoleCmd const* pxData, void* pvUserData )
{
	ListBoxFunctionTraverseData* pxTraverseData = (static_cast<ListBoxFunctionTraverseData*>(pvUserData));

	Asura_Assert(pxTraverseData != NULL,"Traverse_WriteToChunkStream doesn't have any user data");

	Asura_Chunk_Stream& xStream = *pxTraverseData->m_pxStream;

	// Test if the hashID should be exported for the required platform
	if(Tools_Level_Export::IsExport())
	{
		Asura_Assert(pxTraverseData->m_pxListBox != NULL,"Traverse_WriteToChunkStream doesn't have a listbox");
		// Find the container object with the matching hashID and
		// test if it was exported for this platform.
		const Asura_GUIMenu_Widget_ListBox* pxListBox = pxTraverseData->m_pxListBox;

		const Asura_GUIMenu_Widget_ListBoxEntry& entryTemplate = pxListBox->GetEntryTemplate();

		Asura_GUIMenu_Root* pxTemp = entryTemplate.FindContainerItemByID(uHash);

		Asura_Assert(pxTemp != NULL,"Traverse_WriteToChunkStream: Can't find container object");
		
		if((pxTemp == NULL) || 
		   (pxTemp->ShouldExportToPlatform(Tools_Level_Export::s_eExportType) == false))
		{
			// Don't export this data as its not need for the platform.
			// Return true as we haven't actually caused an error.
			return true;
		}
	}

	xStream << uHash;
	pxData->WriteToChunkStream( xStream );
	return true;
}

static bool Traverse_ExportToXML( Asura_Hash_ID uHash, Asura_GUIMenu_ConsoleCmd const* pxData, void* pUserData )
{
	Tools_GUIMenu_XMLExporter* pxExporter = static_cast<Tools_GUIMenu_XMLExporter*> (pUserData);
	pxData->ExportToXML( pxExporter );
	return true;
}

void Asura_GUIMenu_Widget_ListBox::WriteToChunkStream( Asura_Chunk_Stream& xStream) const
{
	WriteLocalChunkHeader( xStream, ASURA_CHUNK_WIDGET_LISTBOX_VERSION, GetType());

	PARENT::WriteToChunkStream( xStream);

	// -- write out any children --
	WriteChildObjectData( xStream);

	// -- write out the listbox specific data --
	xStream << m_uMaxAllowedEntries;
	xStream << m_uMaxDisplayedEntries;
	
	m_xCommandManager.WriteToChunkStream( xStream );
	m_xEntryTemplate.WriteToChunkStream( xStream );

	u_int uContainerCount = static_cast<u_int> (m_xEntryTemplate.GetContainerCount());
	u_int uFunctionCountToWrite = uContainerCount;
	if(Tools_Level_Export::IsExport())
	{
		//determine how many Population and Visibility function
		//are going to be exported for this platform
		//This depends on the platform flags set for each child object
		for( u_int u=0; u<uContainerCount; u++)
		{
			if(m_xEntryTemplate.GetContainerItem(u)->ShouldExportToPlatform(Tools_Level_Export::s_eExportType) == false)
			{
				uFunctionCountToWrite--;
			}
		}
	}
	xStream << uFunctionCountToWrite;

	Asura_Assert(m_xEntryTemplate.GetContainerCount() == m_xPopulationFunctionTree.GetNumberOfNodes(),"Number of population functions doesn't match number of container items");
	Asura_Assert(m_xEntryTemplate.GetContainerCount() == m_xVisibilityFunctionTree.GetNumberOfNodes(),"Number of visibility functions doesn't match number of container items");

	ListBoxFunctionTraverseData xTraverseData;
	xTraverseData.m_pxStream = &xStream;
	xTraverseData.m_pxListBox = this;

	// write out population functions
	m_xPopulationFunctionTree.Traverse( Traverse_WriteToChunkStream, &xTraverseData );
	m_xVisibilityFunctionTree.Traverse( Traverse_WriteToChunkStream, &xTraverseData );

	xStream << m_fInitialRepeatDelay;
	xStream << m_fSecondaryRepeatDelay;
	Asura_WriteToStream_Bitfield_Wrapper<u_int> xFlags(m_uListBoxFlags);
	xFlags.Add(m_bLoopSelection, 1);
	xFlags.Add(m_bCarousel, 1);
	xFlags.Add(m_bAllowSubScroll, 1);
	xStream << xFlags;

	u_int uHashDecID = ASURA_HASH_ID_UNSET;
	u_int uHashIncID = ASURA_HASH_ID_UNSET;
	u_int uHashScrollBar = ASURA_HASH_ID_UNSET;

	if( m_pxDecImage)
	{
		uHashDecID = m_pxDecImage->GetID();
	}
	if( m_pxIncImage)
	{
		uHashIncID = m_pxIncImage->GetID();
	}
	if( m_pxScrollBar)
	{
		uHashScrollBar = m_pxScrollBar->GetID();
	}

	xStream << uHashDecID;
	xStream << uHashIncID;
	xStream << uHashScrollBar;
	xStream << m_xOffsetVector;

	xStream << m_xFlashColour;
	xStream << m_fFlashTimerStartVal;

	m_xInitValueVar.WriteToChunkStream(xStream);
	m_xCurrentValueVar.WriteToChunkStream(xStream);
}

void Asura_GUIMenu_Widget_ListBox::ExportToXML( Tools_GUIMenu_XMLExporter* pxExporter ) const
{
	PARENT::ExportToXML( pxExporter );
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<ListBox_data>\n" );
	pxExporter->IncTabs();

	// -- write out the listbox specific data --
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<max_allowed_entries>" );
	pxExporter->AddToBuffer( m_uMaxAllowedEntries );
	pxExporter->AddToBuffer( "</max_allowed_entries>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<max_displayed_entries>" );
	pxExporter->AddToBuffer( m_uMaxDisplayedEntries );
	pxExporter->AddToBuffer( "</max_displayed_entries>\n" );

	m_xCommandManager.ExportToXML( pxExporter );
	m_xEntryTemplate.ExportToXML( pxExporter );

	// write out population functions
	m_xPopulationFunctionTree.Traverse( Traverse_ExportToXML, pxExporter );
	m_xVisibilityFunctionTree.Traverse( Traverse_ExportToXML, pxExporter );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<initial_repeat_delay>" );
	pxExporter->AddToBuffer( m_fInitialRepeatDelay );
	pxExporter->AddToBuffer( "</initial_repeat_delay>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<secondary_repeat_delay>" );
	pxExporter->AddToBuffer( m_fSecondaryRepeatDelay );
	pxExporter->AddToBuffer( "</secondary_repeat_delay>\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<loop_selection>" );
	pxExporter->AddToBuffer( m_bLoopSelection );
	pxExporter->AddToBuffer( "</loop_selection>\n" );
	
	if( m_pxDecImage)
	{
		pxExporter->AddToBuffer( pxExporter->GetTabs() );
		pxExporter->AddToBuffer( "<dec_image>" );
		pxExporter->AddToBufferHex( m_pxDecImage->GetID() );
		pxExporter->AddToBuffer( "</dec_image>\n" );
	}
	if( m_pxIncImage)
	{
		pxExporter->AddToBuffer( pxExporter->GetTabs() );
		pxExporter->AddToBuffer( "<inc_image>" );
		pxExporter->AddToBufferHex( m_pxIncImage->GetID() );
		pxExporter->AddToBuffer( "</inc_image>\n" );
	}
	if( m_pxScrollBar)
	{
		pxExporter->AddToBuffer( pxExporter->GetTabs() );
		pxExporter->AddToBuffer( "<scroll_bar>" );
		pxExporter->AddToBufferHex( m_pxScrollBar->GetID() );
		pxExporter->AddToBuffer( "</scroll_bar>\n" );
	}

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<flash_colour r=\"" );
	pxExporter->AddToBuffer( m_xFlashColour.m_fR );
	pxExporter->AddToBuffer( "\" g=\"" );
	pxExporter->AddToBuffer( m_xFlashColour.m_fG );
	pxExporter->AddToBuffer( "\" b=\"" );
	pxExporter->AddToBuffer( m_xFlashColour.m_fB );
	pxExporter->AddToBuffer( "\" a=\"" );
	pxExporter->AddToBuffer( m_xFlashColour.m_fA );
	pxExporter->AddToBuffer("\" />\n" );

	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "<flash_timer_start_value>" );
	pxExporter->AddToBuffer( m_fFlashTimerStartVal );
	pxExporter->AddToBuffer( "</flash_timer_start_value>\n" );
	pxExporter->AddToBuffer( pxExporter->GetTabs() );

	pxExporter->AddToBuffer( "<offset vector X=\"" );
	pxExporter->AddToBuffer( m_xOffsetVector.x );
	pxExporter->AddToBuffer( "\" Y=\"" );
	pxExporter->AddToBuffer( m_xOffsetVector.y );
	pxExporter->AddToBuffer("\" />\n" );

	// console var
	pxExporter->AddToBuffer( "<init_value_var>" );
	m_xInitValueVar.ExportToXML(pxExporter);
	pxExporter->AddToBuffer( "</init_value_var>" );

	pxExporter->AddToBuffer( "<current_value_var>" );
	m_xCurrentValueVar.ExportToXML(pxExporter);
	pxExporter->AddToBuffer( "</current_value_var>" );

	pxExporter->DecTabs();
	pxExporter->AddToBuffer( pxExporter->GetTabs() );
	pxExporter->AddToBuffer( "</ListBox_data>\n" );

}

void Asura_GUIMenu_Widget_ListBox::TraverseUsedTextures(void (*pfnCallback)(int iTextureID,void* pUserData),void* pUserData)
{
	m_xEntryTemplate.TraverseUsedTextures(pfnCallback,pUserData);

	PARENT::TraverseUsedTextures(pfnCallback, pUserData);
}

#endif // ASURA_TOOLS




void Asura_GUIMenu_Widget_ListBox::ClearAllEntries( void )
{
	m_uEntryCount = 0;
}



void Asura_GUIMenu_Widget_ListBox::AllocateEntryList( u_int uNewListSize )
{
	Asura_GUIMenu_Widget_ListBoxEntry* pxList = new Asura_GUIMenu_Widget_ListBoxEntry[uNewListSize];
	Asura_Assert( pxList != NULL, "Asura_GUIMenu_Widget_ListBox::AllocateEntryList - Failed to allocate entry widgets");

	for( u_int u = 0; u < uNewListSize; ++u )
	{
		Asura_GUIMenu_System::InitTypeInstance( Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOXENTRY, &pxList[u], m_xEntryTemplate.GetUnscaledLocalBounds() );
		pxList[u].SetParentListBox(this); 

        // (MH - 06.06.2007) copy the modifiers and animators
		pxList[u].CopyModifiers( m_xEntryTemplate );
		pxList[u].CopyAnimators( m_xEntryTemplate );
		pxList[u].SetScalingFlags(m_xEntryTemplate.GetScalingFlags());
		pxList[u].SetPosMode(0, m_xEntryTemplate.GetPosMode(0));
		pxList[u].SetPosMode(1, m_xEntryTemplate.GetPosMode(1));
	}

	// free the old data
	DeallocateEntryList();

	// set our pointers to the new data
	m_pxEntries = pxList;
}



void Asura_GUIMenu_Widget_ListBox::DeallocateEntryList( void )
{
	if( m_pxEntries != NULL )
	{
		ClearAllEntries();

		delete []m_pxEntries;
		m_pxEntries = NULL;
	}
}



Asura_GUIMenu_ConsoleCmd* Asura_GUIMenu_Widget_ListBox::GetPopulationFunctionForItem( Asura_Hash_ID uItem )
{
	return m_xPopulationFunctionTree.GetDataByHash( uItem );
}



Asura_GUIMenu_ConsoleCmd* Asura_GUIMenu_Widget_ListBox::AddPopulationFunction( Asura_Hash_ID uItem )
{
	Asura_Assertf( !m_xPopulationFunctionTree.GetDataByHash(uItem), ("Population for item '%x' already exists.", uItem ) );
	return m_xPopulationFunctionTree.AddNode( uItem );
}



void Asura_GUIMenu_Widget_ListBox::RemovePopulationFunction( Asura_Hash_ID uItem )
{
	m_xPopulationFunctionTree.DeleteNodeByHash( uItem );
}



void Asura_GUIMenu_Widget_ListBox::RemapPopulationFunction( Asura_Hash_ID uOldID, Asura_Hash_ID uNewID )
{
	m_xPopulationFunctionTree.ReplaceHash( uOldID, uNewID );
}


Asura_GUIMenu_ConsoleCmd* Asura_GUIMenu_Widget_ListBox::GetVisibilityFunctionForItem( Asura_Hash_ID uItem )
{
	return m_xVisibilityFunctionTree.GetDataByHash( uItem );
}



Asura_GUIMenu_ConsoleCmd* Asura_GUIMenu_Widget_ListBox::AddVisibilityFunction( Asura_Hash_ID uItem )
{
	Asura_Assertf( !m_xVisibilityFunctionTree.GetDataByHash(uItem), ("Visibility for item '%x' already exists.", uItem ) );
	return m_xVisibilityFunctionTree.AddNode( uItem );
}



void Asura_GUIMenu_Widget_ListBox::RemoveVisibilityFunction( Asura_Hash_ID uItem )
{
	m_xVisibilityFunctionTree.DeleteNodeByHash( uItem );
}



void Asura_GUIMenu_Widget_ListBox::RemapVisibilityFunction( Asura_Hash_ID uOldID, Asura_Hash_ID uNewID )
{
	m_xVisibilityFunctionTree.ReplaceHash( uOldID, uNewID );
}
#endif // ASURA_USE_GUI_MENU_SYSTEM

/***************/
/* End of File */
/***************/
