// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIMenu_Widget_TextListBox.h												//
//																						//
//	Description: ListBox widget class													//
//																						//
//	Created: Steve Keen - 9/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#ifndef __ASURA_GUIMENU_WIDGET_SLIDER_H__
#define __ASURA_GUIMENU_WIDGET_SLIDER_H__

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Asura_GUIMenu_Widget_Base.h"

// -- Typedef -------------------------------------------------------------------------

// -- Enums ---------------------------------------------------------------------------

// -- Defines -------------------------------------------------------------------------

#define fASURA_GUIMENU_WIDGET_SLIDER_CONTROL_INITIAL_REPEAT_DELAY			( 0.3f)
#define fASURA_GUIMENU_WIDGET_SLIDER_CONTROL_SECONDARY_REPEAT_DELAY			( 0.1f)

#define ASURA_GUIMENU_WIDGET_SLIDER_LOCAL_FLAG_START	16

#define ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_CHANGE_VOLUME_ON_LOCAL_AUDIO_EVENT	(1<<0)
#define ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_LOOP_LOCAL_AUDIO_EVENT				(1<<1)
#define ASURA_GUIMENU_WIDGET_SLIDER_FLAGS_ALWAYS_IN_EDIT_MODE_WHEN_HIGHLIGHTED	(1<<2)

// versions: ASURA_CHUNK_WIDGET_SLIDER_VERSION
// 0: initial data
// 1: added slider flags, data change 'audio' hash & step value.
// 2: added console var
// 3: changed to store u_int num steps instead of float step size
#define ASURA_CHUNK_WIDGET_SLIDER_VERSION 3

// -- Structures ----------------------------------------------------------------------

// -- Class ---------------------------------------------------------------------------
class Asura_GUIMenu_Element_Path;
class Asura_GUIMenu_Widget_Base;

class Asura_GUIMenu_Widget_Slider : public Asura_GUIMenu_Widget_Base
{
	friend class Asura_Chunk_GUIMenu;
	friend class Tools_GUIMenu_Widget_Slider;

	// -- Functions -----------------------------------------------------------------------
public:
	enum Asura_GUIMenu_Inherit_Flag_Widget_Slider_Type
	{
// SK 16.02.07 - Temporarily disabled, i'm not sure this is necessary.
//		ASURA_GUIMENU_WIDGET_SLIDER_OVERIDE_PATH	  = (1<<(ASURA_GUIMENU_WIDGET_SLIDER_OVERRIDE_START+0)),
		ASURA_GUIMENU_WIDGET_SLIDER_NUM_STEPS  			= (1<<(ASURA_GUIMENU_WIDGET_SLIDER_LOCAL_FLAG_START+0)),
		ASURA_GUIMENU_WIDGET_SLIDER_FLAGS				= (1<<(ASURA_GUIMENU_WIDGET_SLIDER_LOCAL_FLAG_START+1)),
		ASURA_GUIMENU_WIDGET_SLIDER_LOCAL_FLAG_MAX  	= (1<<(ASURA_GUIMENU_WIDGET_SLIDER_LOCAL_FLAG_START+15))
	};

	Asura_GUIMenu_Widget_Slider();
	virtual ~Asura_GUIMenu_Widget_Slider();

	virtual void 				Update( float fElapsedTime);	//, u_int uButtons, POINT ptMousePos);
//	virtual void 				Render( u_int uState);
	virtual void				Copy( const Asura_GUIMenu_Root* pxSource);

	virtual u_int				ProcessInputActivity( const Asura_GUIMenu_Input& xInput);

	// -- UI Message handler functions --
	virtual bool				OnInit( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnDeInit( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnDataChange( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnDataRequest( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnClicked( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnRollIn( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnRollOut( const Asura_Message_GUIMenu_Notify* pxMessage);
	virtual bool				OnEnable( const Asura_Message_GUIMenu_Notify* pxMessage);

	void						SetActivePathFromHash( Asura_Hash_ID uHashID);
	Asura_Hash_ID				GetActivePathHash( void) const;

	#ifdef ASURA_TOOLS
	virtual void				UpdateLocalDataFromStyle( void);
	virtual void				WriteToChunkStream( Asura_Chunk_Stream& xStream) const;
	virtual void				ExportToXML(Tools_GUIMenu_XMLExporter* pxExporter) const;

	virtual void				PostMenuPreviewCleanup();
	#endif // ASURA_TOOLS

	virtual void				ReadFromChunkStream( Asura_Chunk_Stream& xStream, Asura_GUIMenu_Root* pxParentObject, u_int uChunkVersion);

//	virtual void				UpdateInheritedValues( bool bOnInit = false);

	void						SetEditingAudioHash( Asura_Hash_ID uEditingAudioHash)	{	m_uEditingAudioHash = uEditingAudioHash;	}
	Asura_Hash_ID				GetEditingAudioHash( void) const	{	return m_uEditingAudioHash;	}

	// retained for compatibility
	float						GetStepValue( void ) const	{	return 1.0f / m_uNumSteps; }
	void						SetStepValue( float fStepValue )	{ m_uNumSteps = Asura_Maths::FloatToInt_Near( 1.0f / fStepValue ); }

	u_int						GetNumSteps( void ) const	{	return m_uNumSteps;	}
	void						SetNumSteps( u_int uNumSteps )	{	m_uNumSteps = uNumSteps;	}

	u_int						GetSliderFlags( void) const	{	return m_uSliderFlags;	}
	u_int*						GetSliderFlagsPtr( void)	{	return &m_uSliderFlags;	}
	void						SetSliderFlags( u_int uSliderFlags)	{	m_uSliderFlags = uSliderFlags;	}
	bool						IsSliderFlagSet( u_int uSliderFlag) const;
	void						SetSliderFlag( u_int uSliderFlag, bool bTrueOrFalse);

	virtual bool				RemoveFromThisContainer( Asura_GUIMenu_Root* pxObject);

	void						InitFromConsoleVar( void);

protected:
	void						SetupDefaultElements( void);

private:
	void						PlayEditingAudio( void);
	void						StopEditingAudio( void);

	void						SetConsoleVar( void) const;

	void						SetSliderFromCursorPos(const Asura_Vector_2& xCursorPos);

	bool						HandleInputAutoRepeat( bool bHeldValue);

//	void						ClearLocalVariables();

	// -- Variables -----------------------------------------------------------------------
public:

protected:

private:
	Asura_GUIMenu_Element_Path*		m_pxPath;
	float							m_fPosition;
	float							m_fOldPosition;
	float							m_fTimeElapsed;

	u_int							m_uNumSteps;
	
	Asura_Hash_ID					m_uEditingAudioHash;
	u_int							m_uSliderFlags;
	Asura_Handle					m_hAudio;

	float							m_fConsoleVarLowerLimit;
	float							m_fConsoleVarUpperLimit;
	Asura_GUIMenu_ConsoleVar		m_xConsoleVar;

	bool							m_bCursorDrag;//set to true if the edit state was entered as a result of a cursor click

	u_int 							m_uButtonRepeatPhase;
	float 							m_fCurrentRepeatDelay;
	float 							m_fInitialRepeatDelay;
	float 							m_fSecondaryRepeatDelay;

	typedef Asura_GUIMenu_Widget_Base PARENT;
};
#endif // ASURA_USE_GUI_MENU_SYSTEM
#endif // __ASURA_GUIMENU_WIDGET_SLIDER_H__
