// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_GUIModal_System.h														//
//																						//
//	Description: Header file for handling a modal style of menu page					//
//																						//
//	Created: Steve Keen - 9/5/07														//
//																						//
//	Copyright (C) 2007 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_GUIMODAL_SYSTEM_H__
#define __ASURA_GUIMODAL_SYSTEM_H__

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#ifdef ASURA_USE_GUI_MODAL_SYSTEM

// -- Includes ------------------------------------------------------------------------
#include "Core/Asura_PlatformBasics.h"
// -- Typedef -------------------------------------------------------------------------

// -- Enums ---------------------------------------------------------------------------

enum Asura_GUIModal_ReturnValues
{
	ASURA_GUIMODAL_RESULT_DIALOG_FAILURE = -1,
	ASURA_GUIMODAL_RESULT_NOT_SET = 0,
	ASURA_GUIMODAL_RESULT_NONE,
	ASURA_GUIMODAL_RESULT_YES,
	ASURA_GUIMODAL_RESULT_NO,
	ASURA_GUIMODAL_RESULT_CONTINUE,
	ASURA_GUIMODAL_RESULT_CANCEL,
	ASURA_GUIMODAL_RESULT_RETRY,
	ASURA_GUIMODAL_RESULT_OK
};

enum Asura_GUIModal_QueryType
{
	ASURA_GUIMODAL_QUERYTYPE_NONE = -1,
	ASURA_GUIMODAL_QUERYTYPE_ARE_YOU_SURE,
	ASURA_GUIMODAL_QUERYTYPE_RETRY_OR_CANCEL,
	ASURA_GUIMODAL_QUERYTYPE_QUERY3,
	ASURA_GUIMODAL_QUERYTYPE_QUERY4,
	ASURA_GUIMODAL_QUERYTYPE_QUERY5,
	ASURA_GUIMODAL_QUERYTYPE_QUERY6,
	ASURA_GUIMODAL_QUERYTYPE_QUERY7,
	ASURA_GUIMODAL_QUERYTYPE_QUERY8,
	ASURA_GUIMODAL_QUERYTYPE_QUERY9,
	ASURA_GUIMODAL_QUERYTYPE_QUERY10,
	ASURA_GUIMODAL_QUERYTYPE_QUERY11,
	ASURA_GUIMODAL_QUERYTYPE_QUERY12,
	ASURA_GUIMODAL_QUERYTYPE_QUERY13,
	ASURA_GUIMODAL_QUERYTYPE_QUERY14,
	ASURA_GUIMODAL_QUERYTYPE_QUERY15,
	ASURA_GUIMODAL_QUERYTYPE_QUERY16,
	ASURA_GUIMODAL_QUERYTYPE_QUERY17,
	ASURA_GUIMODAL_QUERYTYPE_QUERY18,
	ASURA_GUIMODAL_QUERYTYPE_QUERY19,
	ASURA_GUIMODAL_QUERYTYPE_QUERY20,
	ASURA_GUIMODAL_QUERYTYPE_ENDOFLIST
};

// -- Enums ---------------------------------------------------------------------------

enum Asura_GUIModal_Options
{
	ASURA_GUIMODAL_OPTIONS_NONE = 0,
	ASURA_GUIMODAL_OPTIONS_YESNO,
	ASURA_GUIMODAL_OPTIONS_RETRYCANCEL,
	ASURA_GUIMODAL_OPTIONS_CONTINUECANCEL,
	ASURA_GUIMODAL_OPTIONS_CONTINUE,
	ASURA_GUIMODAL_OPTIONS_CANCEL,
	ASURA_GUIMODAL_OPTIONS_OK,
	ASURA_GUIMODAL_OPTIONS_ENDOFLIST
};

enum Asura_GUIModal_DefaultButton
{
	ASURA_GUIMODAL_BUTTON_NONE = ASURA_GUIMODAL_RESULT_NONE,
	ASURA_GUIMODAL_BUTTON_YES = ASURA_GUIMODAL_RESULT_YES,
	ASURA_GUIMODAL_BUTTON_NO = ASURA_GUIMODAL_RESULT_NO,
	ASURA_GUIMODAL_BUTTON_CONTINUE = ASURA_GUIMODAL_RESULT_CONTINUE,
	ASURA_GUIMODAL_BUTTON_CANCEL = ASURA_GUIMODAL_RESULT_CANCEL,
	ASURA_GUIMODAL_BUTTON_RETRY = ASURA_GUIMODAL_RESULT_RETRY,
	ASURA_GUIMODAL_BUTTON_OK = ASURA_GUIMODAL_RESULT_OK
};

// -- Defines -------------------------------------------------------------------------
const u_int uASURA_MODAL_DIALOG_SMALL = 0x920a7cf5;
const u_int uASURA_MODAL_DIALOG_SMALL_YESNO_YES_BUTTON = 0xc2146476;
const u_int uASURA_MODAL_DIALOG_SMALL_YESNO_NO_BUTTON = 0x719d8e52;
const u_int uASURA_MODAL_DIALOG_SMALL_RETRYCANCEL_RETRY_BUTTON = 0xeff564b1;
const u_int uASURA_MODAL_DIALOG_SMALL_RETRYCANCEL_CANCEL_BUTTON = 0xf4e32d91;
const u_int uASURA_MODAL_DIALOG_SMALL_CONTINUECANCEL_CONTINUE_BUTTON = 0x4d82522b;
const u_int uASURA_MODAL_DIALOG_SMALL_CONTINUECANCEL_CANCEL_BUTTON = 0xf41ee4fe;
const u_int uASURA_MODAL_DIALOG_SMALL_CONTINUE_BUTTON = 0x8d0bd481;
const u_int uASURA_MODAL_DIALOG_SMALL_CANCEL_BUTTON = 0xcfb104d4;
const u_int uASURA_MODAL_DIALOG_SMALL_OK_BUTTON = 0xf2842136;

// -- Structures ----------------------------------------------------------------------


// -- Class ---------------------------------------------------------------------------
class Asura_GUIMenu_Menu_Page;

class Asura_GUIModal_System
{
public:

	static void						Initialise( void);
	static void						Reset( void);
	static void						Shutdown( void);

	static void						LoadModalMenuPages( void);

	static Asura_GUIModal_ReturnValues	DoModalSmall( Asura_GUIModal_QueryType eQueryType, Asura_GUIModal_Options eType, Asura_GUIModal_DefaultButton eDefaultButton = ASURA_GUIMODAL_BUTTON_NONE);

	// -- Helper functions --
	static bool						IsDialogActive( void)	{	return s_bDialogActive;	}

	static void						SetResult( Asura_GUIModal_ReturnValues eReturnValue);
	static void						CloseModalSmall( void);
protected:
	static void						ClearLocalVariables( void);


	static bool						ConsoleCmd_SetResult( const Asura_Char* pcCmd, bool bInterrogate, bool bHelp);

	static void						Platform_HandleUpdate( void);

private:
	static	bool						s_bInitialised;
	static	bool						s_bDialogActive;
	static	Asura_GUIModal_Options		s_eModalOptionType;
	static  Asura_GUIModal_ReturnValues	s_eModalReturnValue;
	static	Asura_GUIModal_QueryType	s_eQueryType;

#ifdef ASURA_TOOLS
	static bool							s_bTestMode;
#endif // ASURA_TOOLS
};
#endif // ASURA_USE_GUI_MODAL_SYSTEM
#endif // ASURA_USE_GUI_MENU_SYSTEM
#endif // __ASURA_GUIMODAL_SYSTEM_H__
