//////////////////////////////////////////////////////////////////////////////
//
// Asura_Profiling_GUIMenu.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// ASURA_USE_GUIMENU_PROFILING == 1 Enables profiling GUIMenu tasks 
// ASURA_USE_GUIMENU_PROFILING == 0 All macro profiling commands still exist for compiling but are empty.

//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "GUIMenu/Asura_Profiling_GUIMenu.h"

// Engine independent.
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_DebugText.h"
#include "Interface/Asura_CommandConsole.h"
#include "Rendering/Asura_Font_System.h"


// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

#ifdef ASURA_USE_GUIMENU_PROFILING

////////////////////////////////////////////////////////////////////////////////////////////////
//
// Internal data & functions for Asura_Profiling_GUIMenu
//
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

// Must be in same order as enum Asura_GUIMenu_InstanceCounter
const char* Asura_Profiling_GUIMenu::s_aszCounterNames[] =
{
	"GUIMenu System",			//ASURA_GUIMENU_COUNTER__GUIMENU_SYSTEM
	"GUIMenu Root",				//ASURA_GUIMENU_COUNTER__GUIMENU_ROOT
	"GUIMenu Page",				//ASURA_GUIMENU_COUNTER__GUIMENU_PAGE
	"Element Animation",		//ASURA_GUIMENU_COUNTER__ELEMENT_ANIMATION
	"Element Base",				//ASURA_GUIMENU_COUNTER__ELEMENT_BASE
	"Element Image",			//ASURA_GUIMENU_COUNTER__ELEMENT_IMAGE
	"Element Path",				//ASURA_GUIMENU_COUNTER__ELEMENT_PATH
	"Element Status Text",		//ASURA_GUIMENU_COUNTER__ELEMENT_STATUSTEXT
	"Element Text",				//ASURA_GUIMENU_COUNTER__ELEMENT_TEXT
	"Element FMV",				//ASURA_GUIMENU_COUNTER__ELEMENT_FMV
	"Widget Anim Image",		//ASURA_GUIMENU_COUNTER__WIDGET_ANIMATINGIMAGE
	"Widget Base",				//ASURA_GUIMENU_COUNTER__WIDGET_BASE
	"Widget Button",			//ASURA_GUIMENU_COUNTER__WIDGET_BUTTON
	"Widget Check Box",			//ASURA_GUIMENU_COUNTER__WIDGET_CHECKBOX
	"Widget Group",				//ASURA_GUIMENU_COUNTER__WIDGET_GROUP
	"Widget List Box",			//ASURA_GUIMENU_COUNTER__WIDGET_LISTBOX
	"Widget Numeric",			//ASURA_GUIMENU_COUNTER__WIDGET_NUMERIC
	"Widget Progress Bar",		//ASURA_GUIMENU_COUNTER__WIDGET_PROGRESSBAR
	"Widget Slider",			//ASURA_GUIMENU_COUNTER__WIDGET_SLIDER
	"Widget Text Box",			//ASURA_GUIMENU_COUNTER__WIDGET_TEXTBOX
	"Widget Text List Box",		//ASURA_GUIMENU_COUNTER__WIDGET_TEXTLISTBOX
};

// Must be in same order as enum Asura_GUIMenu_ProfileSection
const char* Asura_Profiling_GUIMenu::s_aszSectionNames[] =
{
	"GUIMenu Update",			//ASURA_GUIMENU_PROFILE__UPDATE
	"GUIMenu Render",			//ASURA_GUIMENU_PROFILE__RENDER
};

// Must be in same order as enum Asura_GUIMenu_ProfileTask
const char* Asura_Profiling_GUIMenu::s_aszTaskNames[] =
{
	"GUIMenu System",			//ASURA_GUIMENU_PROFILE__GUIMENU_SYSTEM
	"GUIMenu Root",				//ASURA_GUIMENU_PROFILE__GUIMENU_ROOT
	"GUIMenu Page",				//ASURA_GUIMENU_PROFILE__GUIMENU_PAGE
	"Elem Animation",			//ASURA_GUIMENU_PROFILE__ELEMENT_ANIMATION
	"Elem Base",				//ASURA_GUIMENU_PROFILE__ELEMENT_BASE
	"Elem Image",				//ASURA_GUIMENU_PROFILE__ELEMENT_IMAGE
	"Elem Path",				//ASURA_GUIMENU_PROFILE__ELEMENT_PATH
	"Elem Status Text",			//ASURA_GUIMENU_PROFILE__ELEMENT_STATUSTEXT
	"Elem Text",				//ASURA_GUIMENU_PROFILE__ELEMENT_TEXT
	"Elem FMV",					//ASURA_GUIMENU_PROFILE__ELEMENT_FMV
	"Widg Anim Image",			//ASURA_GUIMENU_PROFILE__WIDGET_ANIMATINGIMAGE
	"Widg Base",				//ASURA_GUIMENU_PROFILE__WIDGET_BASE
	"Widg Button",				//ASURA_GUIMENU_PROFILE__WIDGET_BUTTON
	"Widg Check Box",			//ASURA_GUIMENU_PROFILE__WIDGET_CHECKBOX
	"Widg Group",				//ASURA_GUIMENU_PROFILE__WIDGET_GROUP
	"Widg List Box",			//ASURA_GUIMENU_PROFILE__WIDGET_LISTBOX
	"Widg Numeric",				//ASURA_GUIMENU_PROFILE__WIDGET_NUMERIC
	"Widg Progress Bar",		//ASURA_GUIMENU_PROFILE__WIDGET_PROGRESSBAR
	"Widg Slider",				//ASURA_GUIMENU_PROFILE__WIDGET_SLIDER
	"Widg Text Box",			//ASURA_GUIMENU_PROFILE__WIDGET_TEXTBOX
	"Widg Text List Box",		//ASURA_GUIMENU_PROFILE__WIDGET_TEXTLISTBOX
	"Handle Page Update",		//ASURA_GUIMENU_PROFILE__HANDLE_MENUPAGE_UPDATE
	"Input Activity",			//ASURA_GUIMENU_PROFILE__INPUT_ACTIVITY
};


u_int Asura_Profiling_GUIMenu::s_auInstanceCount[ASURA_GUIMENU_COUNTER__MAXCOUNTERS];
u_int Asura_Profiling_GUIMenu::s_auClassSize[ASURA_GUIMENU_COUNTER__MAXCOUNTERS];

u_int Asura_Profiling_GUIMenu::s_uProfileDepth = 0;
bool Asura_Profiling_GUIMenu::s_bDisplayMemory;
bool Asura_Profiling_GUIMenu::s_bDisplayTimings;
bool Asura_Profiling_GUIMenu::s_bWasDisplayMemoryLastFrame;
bool Asura_Profiling_GUIMenu::s_bWasDisplayTimingsLastFrame;
u_int Asura_Profiling_GUIMenu::s_bMaxHistoricFrames;
float Asura_Profiling_GUIMenu::s_fPeakDelaySecs;
float Asura_Profiling_GUIMenu::s_fHighlightMilliSecs;
u_int Asura_Profiling_GUIMenu::s_uTimingsX;
u_int Asura_Profiling_GUIMenu::s_uTimingsY;
u_int Asura_Profiling_GUIMenu::s_uMemoryX;
u_int Asura_Profiling_GUIMenu::s_uMemoryY;
bool Asura_Profiling_GUIMenu::s_bEnableRecording = false;

Asura_GUIMenu_ProfileSection Asura_Profiling_GUIMenu::m_eCurrentSection = ASURA_GUIMENU_PROFILE__MAXSECTIONS;
Asura_Profiling_GUIMenu::Asura_Profile_Entry Asura_Profiling_GUIMenu::s_axProfileStack[uASURA_GUIMENU_PROFILE_MAX_STACK_DEPTH];
Asura_Profiling_GUIMenu::Asura_Profile_Results_Entry Asura_Profiling_GUIMenu::s_axProfileTime[ASURA_GUIMENU_PROFILE__MAXTASKS][ASURA_GUIMENU_PROFILE__MAXSECTIONS];
Asura_Profiling_GUIMenu::Asura_Profile_Section Asura_Profiling_GUIMenu::s_axSectionData[ASURA_GUIMENU_PROFILE__MAXSECTIONS];

////////////////////////////////////////////////////////////////////////////////
// Public Functions...
////////////////////////////////////////////////////////////////////////////////

// Initialise the GUIMenu profiling system
void Asura_Profiling_GUIMenu::Initialise()
{
	u_int i;
	u_int j;

	for ( i = 0; i < ASURA_GUIMENU_COUNTER__MAXCOUNTERS; i++)
	{
		s_auInstanceCount[i] = 0;
		s_auClassSize[i] = 0;
	}

	for ( i = 0; i < ASURA_GUIMENU_PROFILE__MAXTASKS; i++)
	{
		for ( j = 0; j < ASURA_GUIMENU_PROFILE__MAXSECTIONS; j++)
		{
			s_axProfileTime[i][j].Init();
		}
	}

	for ( i = 0; i < ASURA_GUIMENU_PROFILE__MAXSECTIONS; i++)
	{
		s_axSectionData[i].m_iTotalDuration = 0;
		s_axSectionData[i].m_iFrameDuration[0] = 0;
		s_axSectionData[i].m_iFrameStartTime = 0;
		s_axSectionData[i].m_uFrameCount = 0;
	}

	s_bEnableRecording = false;

	m_eCurrentSection = ASURA_GUIMENU_PROFILE__MAXSECTIONS;

	s_bDisplayMemory = false;
	s_bDisplayTimings = false;
	s_bWasDisplayMemoryLastFrame = false;
	s_bWasDisplayTimingsLastFrame = false;

	s_bMaxHistoricFrames = uMAX_NUMBER_RESULTS_FRAMES;
	s_fPeakDelaySecs = 4.0f;
	s_fHighlightMilliSecs = 0.3f;

	s_uTimingsX = 40;
	s_uTimingsY = 8;

	s_uMemoryX = 72;
	s_uMemoryY = 8;

	// Add menu profile commands
	Asura_CommandConsole::AddVar( L"GUIMenu.Profile.DisplayMemory",			&s_bDisplayMemory,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( L"GUIMenu.Profile.DisplayTimings",		&s_bDisplayTimings,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( L"GUIMenu.Profile.HistoricResults",		&s_bMaxHistoricFrames,	1,		uMAX_NUMBER_RESULTS_FRAMES, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"GUIMenu.Profile.PeakDelaySeconds",		&s_fPeakDelaySecs,		1.0f,	10.0f,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( L"GUIMenu.Profile.HighlightMilliSecs",	&s_fHighlightMilliSecs, 0.0f,	20.0f,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN);

	Asura_CommandConsole::AddVar (L"GUIMenu.Profile.PosTiming.X",	&s_uTimingsX,	0,	80,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar (L"GUIMenu.Profile.PosTiming.Y",	&s_uTimingsY,	0,	50,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar (L"GUIMenu.Profile.PosMemory.X",	&s_uMemoryX,	0,	80,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar (L"GUIMenu.Profile.PosMemory.Y",	&s_uMemoryY,	0,	50,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
}

// Reset the system when Core::Reset is called
void Asura_Profiling_GUIMenu::Reset()
{
}

// Record the instantiation of a GUIMenu type
void Asura_Profiling_GUIMenu::DebugCounterInstance( Asura_GUIMenu_InstanceCounter eType )
{
	Asura_Assert(eType < ASURA_GUIMENU_COUNTER__MAXCOUNTERS, "Unknown Asura_GUIMenu_InstanceCounter enum type");
	s_auInstanceCount[eType]++;
}

// Record the destruction of a GUIMenu type
void Asura_Profiling_GUIMenu::DebugCounterDestroyInstance( Asura_GUIMenu_InstanceCounter eType )
{
	Asura_Assert(s_auInstanceCount[eType] > 0, "Mismatched Instance counter. More GUIMenu objects destroyed than created.");
	s_auInstanceCount[eType]--;
}

// Record the size of a GUIMenu type
void Asura_Profiling_GUIMenu::DebugCounterClassSize( Asura_GUIMenu_InstanceCounter eType, u_int uSize )
{
	Asura_Assert(eType < ASURA_GUIMENU_COUNTER__MAXCOUNTERS, "Unknown Asura_GUIMenu_InstanceCounter enum type");
	s_auClassSize[eType] = uSize;
}

// Record the instantiation of a GUIMenu type
void Asura_Profiling_GUIMenu::DisplayDebugCounters()
{
	UpdateDisplayStates();

	if ( s_bDisplayMemory == false )
	{
		return;
	}

	u_int uTextColour = 0xFFFFFFFF;

	u_int	uX	= s_uMemoryX;
	u_int	uY	= s_uMemoryY;

	u_int i;

	Asura_DebugText::AddTextBlock(uX, uY++, uTextColour, 0, "%-20s %5s %5s %3s", 
		"Name", 
		"Bytes", 
		"Count", 
		"(k)" );

	uY++;

	u_int uTotalInstanceCount = 0;
	float uTotalSizeK = 0;

	for ( i = 0; i < ASURA_GUIMENU_COUNTER__MAXCOUNTERS; i++)
	{
		if ( s_auInstanceCount[i] > 0 )
		{
			float fSizeK = (s_auClassSize[i]*s_auInstanceCount[i]) / 1024.0f;
			Asura_DebugText::AddTextBlock(uX, uY++, uTextColour, 0, "%-20s %4d %4d %7.2fk", 
												s_aszCounterNames[i], 
												s_auClassSize[i], 
												s_auInstanceCount[i], 
												fSizeK );

			uTotalInstanceCount += s_auInstanceCount[i];
			uTotalSizeK += fSizeK;
		}
	}

	uY++;

	Asura_DebugText::AddTextBlock(uX, uY++, uTextColour, 0, "%-20s      %4d %7.2fk", 
																"Total", 
																uTotalInstanceCount, 
																uTotalSizeK );
}

void Asura_Profiling_GUIMenu::StartSection(Asura_GUIMenu_ProfileSection eSection)
{
	u_int i;

	for ( i = 0; i < ASURA_GUIMENU_PROFILE__MAXTASKS; i++)
	{
		s_axProfileTime[i][eSection].FrameInit();
	}

	Asura_Assert(m_eCurrentSection == ASURA_GUIMENU_PROFILE__MAXSECTIONS,"Already inside another profile section. EndSection must be called first");
	m_eCurrentSection = eSection;

	s_axSectionData[m_eCurrentSection].BeginSection(Asura_Timers::GetCurTicks());

	EnableRecording(true);
}

void Asura_Profiling_GUIMenu::EndSection()
{
	s_axSectionData[m_eCurrentSection].EndSection(Asura_Timers::GetCurTicks());

	m_eCurrentSection = ASURA_GUIMENU_PROFILE__MAXSECTIONS;

	EnableRecording(false);
}

void Asura_Profiling_GUIMenu::StartProfileTask( Asura_GUIMenu_ProfileTask eTask)
{
	if ( s_bEnableRecording == false )
	{
		return;
	}

	Asura_Assert(s_uProfileDepth < uASURA_GUIMENU_PROFILE_MAX_STACK_DEPTH,"Call Stack depth exceeded for GUIMenu Profiler");

	if ( s_uProfileDepth >= uASURA_GUIMENU_PROFILE_MAX_STACK_DEPTH)
	{
		return;
	}

	s_axProfileStack[s_uProfileDepth].Start(Asura_Timers::GetCurTicks(), eTask, m_eCurrentSection);
	s_uProfileDepth++;
}

void Asura_Profiling_GUIMenu::EndProfileTask()
{
	if ( s_bEnableRecording == false )
	{
		return;
	}

	u_int i;

	Asura_Assert(s_uProfileDepth > 0,"Mismatched call to Asura_Profiling_GUIMenu::EndProfileTask");

	if ( s_uProfileDepth == 0 )
	{
		return;
	}

	s_uProfileDepth--;
	Asura_TickCount iNowTicks = Asura_Timers::GetCurTicks();

	Asura_TickCount iInclusiveTicks = s_axProfileStack[s_uProfileDepth].InclusiveTicks(iNowTicks);
	Asura_TickCount iExclusiveTicks = s_axProfileStack[s_uProfileDepth].ExclusiveTicks(iNowTicks);
	Asura_GUIMenu_ProfileSection eSection = s_axProfileStack[s_uProfileDepth].m_eSection;
	Asura_GUIMenu_ProfileTask eTask = s_axProfileStack[s_uProfileDepth].m_eTask;

	s_axProfileTime[eTask][eSection].RecordResults(iInclusiveTicks, iExclusiveTicks);

	// Now walk back up the profile stack and record this as child time
	// Only record the exclusive time for this profile task. Don't include its child time as
	// this would have been included by the children themselves.
	for ( i = 0; i < s_uProfileDepth; i++)
	{
		s_axProfileStack[i].m_uChildTime += iExclusiveTicks;
	}
}

void Asura_Profiling_GUIMenu::DisplayProfileData()
{	
	UpdateDisplayStates();

	if ( s_bDisplayTimings == false )
	{
		return;
	}

	u_int uTextColour = 0xFFFFFFFF;

	u_int	uX	= s_uTimingsX;
	u_int	uY	= s_uTimingsY;

	u_int i;
	u_int j;

	Asura_TickCount iCurTicks = Asura_Timers::GetCurTicks();

	u_int uSize = 0;

	uSize += sizeof(s_auInstanceCount);
	uSize += sizeof(s_auClassSize);
	uSize += sizeof(s_axProfileStack);
	uSize += sizeof(s_axProfileTime);
	uSize += sizeof(s_axSectionData);

	Asura_DebugText::AddTextBlock(uX, uY++, uTextColour, 0, "Profile Memory = %0.2fk", 
		uSize/1024.0f);

	uY++;

	Asura_DebugText::AddTextBlock(uX, uY++, uTextColour, 0, "%-18s    %4s %5s %5s  %4s %4s  | %9s %9s ",
		"    Task",
		"#",
		"In Ms",
		"Ex Ms",
		"In %",
		"Ex %",
		"Peak In",
		"Peak Ex");

	uY++;

	for ( j = 0; j < ASURA_GUIMENU_PROFILE__MAXSECTIONS; j++)
	{
		Asura_TickCount iSectionTicks = s_axSectionData[j].GetAverageFrameDuration();
		float fSectionFrameMs = 1000.0f * Asura_Timers::TicksToSeconds(iSectionTicks);
	
		Asura_DebugText::AddTextBlock(uX, uY++, uTextColour, 0, "%-18s         %5.2f", 
			s_aszSectionNames[j],
			fSectionFrameMs);

		uX += 3;

		for ( i = 0; i < ASURA_GUIMENU_PROFILE__MAXTASKS; i++)
		{
			Asura_Profile_Results frameRes = s_axProfileTime[i][j].GetAverageFrameResults();

			if ( frameRes.m_uCallCount > 0 )
			{
				s_axProfileTime[i][j].UpdatePeakTimes(iCurTicks);

				float fFrameInclusiveMS = 1000.0f * Asura_Timers::TicksToSeconds(frameRes.m_iInclusiveTime);
				float fFrameExclusiveMS = 1000.0f * Asura_Timers::TicksToSeconds(frameRes.m_iExclusiveTime);

				// Take the total frame section time and calculate a percentage for the profile type
				float fFrameAvgIncPer = (fFrameInclusiveMS / fSectionFrameMs) * 100.0f;
				float fFrameAvgExcPer = (fFrameExclusiveMS / fSectionFrameMs) * 100.0f;

				// Cap the percentage as there is only room to draw 99.9%.
				if ( fFrameAvgIncPer > 99.9f ) fFrameAvgIncPer = 99.9f;
				if ( fFrameAvgExcPer > 99.9f ) fFrameAvgExcPer = 99.9f;

				float fIncPeakTimeMs = 1000.0f * Asura_Timers::TicksToSeconds(s_axProfileTime[i][j].m_iIncPeakTime);
				float fExcPeakTimeMs = 1000.0f * Asura_Timers::TicksToSeconds(s_axProfileTime[i][j].m_iExcPeakTime);
				u_int uIncPeakFrame = s_axProfileTime[i][j].m_uIncPeakFrame % 100; //( 0-99 )
				u_int uExcPeakFrame = s_axProfileTime[i][j].m_uExcPeakFrame % 100; //( 0-99 )

				if ( fExcPeakTimeMs >= s_fHighlightMilliSecs )
				{
					uTextColour = 0xFFFF1111;
				}

				Asura_DebugText::AddTextBlock(uX, uY++, uTextColour, 0, "%-18s %4d %5.2f %5.2f [%4.1f %4.1f] | %5.2f(%02d) %5.2f(%02d)", 
					s_aszTaskNames[i],
					frameRes.m_uCallCount,
					fFrameInclusiveMS,
					fFrameExclusiveMS,
					fFrameAvgIncPer, 
					fFrameAvgExcPer,
					fIncPeakTimeMs,
					uIncPeakFrame,
					fExcPeakTimeMs,
					uExcPeakFrame
					);

				uTextColour = 0xFFFFFFFF; 
			}
		}

		uY++;
		uX -= 3;
	}
}

////////////////////////////////////////////////////////////////////////////////
// Private Functions...
////////////////////////////////////////////////////////////////////////////////

void Asura_Profiling_GUIMenu::UpdateDisplayStates()
{
	if ( s_bDisplayMemory != s_bWasDisplayMemoryLastFrame )
	{
		// Display memory setting has changed since last frame
		s_bWasDisplayMemoryLastFrame = s_bDisplayMemory;
		s_bDisplayTimings = false;
		s_bWasDisplayTimingsLastFrame = false;
	}

	if ( s_bDisplayTimings != s_bWasDisplayTimingsLastFrame )
	{
		// Display timings settings has changed since last frame
		s_bWasDisplayTimingsLastFrame = s_bDisplayTimings;
		s_bDisplayMemory = false;
		s_bWasDisplayMemoryLastFrame = false;
	}
}

void Asura_Profiling_GUIMenu::EnableRecording(bool bEnable)
{
	s_bEnableRecording = bEnable;
}

////////////////////////////////////////////////////////////////////////////////////////////////
//
// Internal data & functions for Asura_Profile_Results_Entry
//
// Record the results for a task within a section.
// Holds the total profile results for a task, the last uMAX_NUMBER_RESULTS_FRAMES frames 
// of data and the peak timings
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Public Functions...
////////////////////////////////////////////////////////////////////////////////

void Asura_Profiling_GUIMenu::Asura_Profile_Results_Entry::UpdatePeakTimes(Asura_TickCount iNowTicks)
{
	// Calculate the peak time;
	Asura_TickCount lastTimePeak = iNowTicks - m_iLastTimePeakChanged;

	if ( lastTimePeak > Asura_Timers::SecondsToTicks(Asura_Profiling_GUIMenu::s_fPeakDelaySecs) )
	{
		m_iIncPeakTime = 0;
		m_iExcPeakTime = 0;

		m_uIncPeakFrame = 0;
		m_uIncPeakFrame = 0;
	}

	if ( m_xFrame[m_uCurrentFrameIndex].m_iInclusiveTime > m_iIncPeakTime )
	{
		m_iIncPeakTime = m_xFrame[m_uCurrentFrameIndex].m_iInclusiveTime;
		m_iLastTimePeakChanged = iNowTicks;
		m_uIncPeakFrame = m_xTotal.m_uCallCount;
	}
	if ( m_xFrame[m_uCurrentFrameIndex].m_iExclusiveTime > m_iExcPeakTime )
	{
		m_iExcPeakTime = m_xFrame[m_uCurrentFrameIndex].m_iExclusiveTime;
		m_iLastTimePeakChanged = iNowTicks;
		m_uExcPeakFrame = m_xTotal.m_uCallCount;
	}
}

Asura_Profiling_GUIMenu::Asura_Profile_Results Asura_Profiling_GUIMenu::Asura_Profile_Results_Entry::GetAverageFrameResults()
{
	Asura_Profile_Results results;

	results.Init();

	// Go through the list of results for each frame a calculate a frame average
	u_int uFrameCount = 0;
	u_int uCurrentFrame = m_uCurrentFrameIndex;
	u_int uNumFramesAvailable = m_uNumberFrames;

	if ( uNumFramesAvailable >  Asura_Profiling_GUIMenu::s_bMaxHistoricFrames)
	{
		uNumFramesAvailable =  Asura_Profiling_GUIMenu::s_bMaxHistoricFrames;
	}

	Asura_TickCount iIncTime = 0;
	Asura_TickCount iExcTime = 0;
	u_int uCallCount = 0;

	while ( uNumFramesAvailable > 0 )
	{
		iIncTime += m_xFrame[uCurrentFrame].m_iInclusiveTime;
		iExcTime += m_xFrame[uCurrentFrame].m_iExclusiveTime;
		uCallCount += m_xFrame[uCurrentFrame].m_uCallCount;

		if ( uCurrentFrame == 0 )
		{
			uCurrentFrame = uMAX_NUMBER_RESULTS_FRAMES;
		}
		uCurrentFrame--;
		uNumFramesAvailable--;
		uFrameCount++;
	}

	results.m_iInclusiveTime = iIncTime / uFrameCount;
	results.m_iExclusiveTime = iExcTime / uFrameCount;
	results.m_uCallCount = uCallCount / uFrameCount;

	return results;
}

////////////////////////////////////////////////////////////////////////////////////////////////
//
// Internal data & functions for Asura_Profile_Section
//
// Holds the timing data for a section
// Holds the total time for a section and the last 
// uMAX_NUMBER_RESULTS_FRAMES number of frames of timing data
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Public Functions...
////////////////////////////////////////////////////////////////////////////////

void Asura_Profiling_GUIMenu::Asura_Profile_Section::BeginSection(Asura_TickCount iNowTicks)
{
	m_iFrameStartTime = iNowTicks;

	m_uFrameCount++;

	u_int index = (m_uFrameCount-1) % uMAX_NUMBER_RESULTS_FRAMES;
	m_iFrameDuration[index] = 0;
}

void Asura_Profiling_GUIMenu::Asura_Profile_Section::EndSection(Asura_TickCount iNowTicks)
{
	Asura_TickCount iElapsedTime = iNowTicks - m_iFrameStartTime; 

	// Get the index in which the frame duration needs to be written
	u_int index = (m_uFrameCount-1) % uMAX_NUMBER_RESULTS_FRAMES;

	m_iFrameDuration[index] = iElapsedTime;
	m_iTotalDuration += iElapsedTime;
}

Asura_TickCount Asura_Profiling_GUIMenu::Asura_Profile_Section::GetCurrentFrameDuration()
{
	u_int uIndex = (m_uFrameCount-1) % uMAX_NUMBER_RESULTS_FRAMES;
	return m_iFrameDuration[uIndex];
}

Asura_TickCount Asura_Profiling_GUIMenu::Asura_Profile_Section::GetAverageFrameDuration()
{
	u_int i;

	// How many results are available
	u_int uMaxResults = Asura_Maths::Min(m_uFrameCount,Asura_Profiling_GUIMenu::s_bMaxHistoricFrames);

	// Which index contains the latest result
	u_int uLastIndex = (m_uFrameCount-1) % uMAX_NUMBER_RESULTS_FRAMES;

	Asura_TickCount uTotal = 0;

	for ( i = 0; i < uMaxResults; i++)
	{
		uTotal += m_iFrameDuration[uLastIndex];

		if ( uLastIndex == 0 )
		{
			uLastIndex = uMAX_NUMBER_RESULTS_FRAMES;
		}
		uLastIndex--;
	}

	return (uTotal / uMaxResults);
}

////////////////////////////////////////////////////////////////////////////////////////////////
//
// Internal data & functions for Asura_Profile_Entry
//
// Entry objects are held in the s_axProfileStack call stack.
// They contain the start time, task id, and section id. Also contain
// how long any child tasks have taken to run.
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Public Functions...
////////////////////////////////////////////////////////////////////////////////

void Asura_Profiling_GUIMenu::Asura_Profile_Entry::Start(Asura_TickCount iNowTicks, Asura_GUIMenu_ProfileTask eTask, Asura_GUIMenu_ProfileSection eSection )
{
	m_uStartTime = iNowTicks;
	m_uChildTime = 0;
	m_eSection = eSection;
	m_eTask = eTask;
}

Asura_TickCount Asura_Profiling_GUIMenu::Asura_Profile_Entry::InclusiveTicks(Asura_TickCount iNowTicks)
{
	Asura_Assert(iNowTicks >= m_uStartTime,"Profile task finished before it started. Is time running backwards?");
	return iNowTicks - m_uStartTime;
}

Asura_TickCount Asura_Profiling_GUIMenu::Asura_Profile_Entry::ExclusiveTicks(Asura_TickCount iNowTicks)
{
	Asura_Assert((iNowTicks - m_uStartTime) >= s_axProfileStack[s_uProfileDepth].m_uChildTime,
		"Children took longer to run than parent did. How is that possible?");

	return (iNowTicks - m_uStartTime) - s_axProfileStack[s_uProfileDepth].m_uChildTime;
}


void Asura_Profiling_GUIMenu::Asura_Profile_Results_Entry::Init()
{
	u_int i;

	m_xTotal.Init();

	for ( i = 0; i < uMAX_NUMBER_RESULTS_FRAMES; i++)
	{
		m_xFrame[i].Init();
	}

	m_iIncPeakTime = 0;
	m_iExcPeakTime = 0;
	m_iLastTimePeakChanged = 0;

	m_uCurrentFrameIndex = uMAX_NUMBER_RESULTS_FRAMES;
	m_uNumberFrames = 0;
}

void Asura_Profiling_GUIMenu::Asura_Profile_Results_Entry::FrameInit()
{
	m_uCurrentFrameIndex++;
	m_uNumberFrames++;

	if ( m_uCurrentFrameIndex >= uMAX_NUMBER_RESULTS_FRAMES)
	{
		m_uCurrentFrameIndex = 0;
	}

	m_xFrame[m_uCurrentFrameIndex].Init();
}

void Asura_Profiling_GUIMenu::Asura_Profile_Results_Entry::RecordResults(Asura_TickCount iInclusiveTicks, Asura_TickCount iExclusiveTicks)
{
	m_xFrame[m_uCurrentFrameIndex].m_iInclusiveTime += iInclusiveTicks; 
	m_xFrame[m_uCurrentFrameIndex].m_iExclusiveTime += iExclusiveTicks; 
	m_xFrame[m_uCurrentFrameIndex].m_uCallCount++;

	Asura_Assert(m_xFrame[m_uCurrentFrameIndex].m_iInclusiveTime >= m_xFrame[m_uCurrentFrameIndex].m_iExclusiveTime,"Inclusive time less than exclusive time");

	m_xTotal.m_iInclusiveTime += iInclusiveTicks; 
	m_xTotal.m_iExclusiveTime += iExclusiveTicks; 
	m_xTotal.m_uCallCount++;

	Asura_Assert(m_xTotal.m_iInclusiveTime >= m_xTotal.m_iExclusiveTime,"Inclusive time less than exclusive time");
}

#endif //#ifdef ASURA_USE_GUIMENU_PROFILING