///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_ControlManager.h
// Author : Steven Keen (Derby)
// Date   : 12-Jan-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_CONTROLMANAGER_H___
#define ASURA_CONTROLMANAGER_H___ 1

#ifdef ASURA_USE_CONTROLMAP_MANAGER

// I N C L U D E   F I L E S /////////////////////////////////////////////////

#include "Core/Asura_List.h"
#include "Core/Asura_Hash.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_ControlMap;
template<typename T> class Asura_TemplateHashTree;

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_ControlManager
{
public:
	// Public functions...
	static void Initialise( void);
	static void Shutdown( void);
	static void Reset( void);

	static void Update( void);
    static void ResetAllControlMapValues( void );

	// Functions to handle default control-maps...
	static bool AddDefaultControls( const Asura_Hash_ID uDefaultControlMapHash, Asura_ControlMap* pxGameControlMap );
	static bool RemoveDefaultControls( const Asura_Hash_ID uDefaultControlMapHash );

	static Asura_ControlMap* PushDefaultControls( const Asura_Hash_ID uDefaultControlMapHash, Asura_ControlMap* pxControlMap );
	static Asura_ControlMap* PopDefaultControls( const Asura_Hash_ID uDefaultControlMapHash );

	// Old functions - should be deprecated ASAP as they now call the new functions...
	static void AddDefaultControls( Asura_ControlMap* pxGameControlMap );
	static void RemoveDefaultControls( Asura_ControlMap* pxGameControlMap );

	// Functions to handle override control-maps...
	static Asura_ControlMap* PushOverrideControls( Asura_ControlMap* pxControlMap );
	static Asura_ControlMap* PopOverrideControls( Asura_ControlMap* pxControlMap );
	static bool RemoveOverrideControls( Asura_ControlMap* pxControlMap );

	// Accessor functions...
	static const Asura_ControlMap* GetCurrentControlMap(const Asura_Hash_ID uDefaultControlMapHash);
	static const Asura_ControlMap* GetCurrentOverrideControlMap(void);

	static bool	HasOverrideControls(void)	{ return(!s_xOverrideControlMapStack.IsEmpty()); }


private:
	// Private typedefs...
	typedef Asura_List<Asura_ControlMap>			ControlMapStack;
	typedef Asura_TemplateHashTree<ControlMapStack> StackTree;

	// Private variables...
	static Asura_Hash_ID		s_uDefaultControlsUniqueHash;	// Needed to support old code where no hash-ID is specified when adding default controls - should be depracated ASAP.

	static StackTree			s_xDefaultStackTree;			// Tree of stacks used for default control-maps.
	static ControlMapStack		s_xOverrideControlMapStack;		// Stack of control-maps used to override the default control-maps.

	static Asura_ControlMap*	s_pxControlMapToFind;			// A pointer to the control-map we're to search for.
	static Asura_Hash_ID		s_uFoundStackHash;				// The hash-ID of the stack we've found following a search.

	// Private functions...
	static void Traverse_FindControlMap(Asura_Hash_ID uDefaultControlMapHash, ControlMapStack* pxStack);
	static void Traverse_Reset(Asura_Hash_ID uDefaultControlMapHash, ControlMapStack* pxStack);
	static void Traverse_Activate(Asura_Hash_ID uDefaultControlMapHash, ControlMapStack* pxStack);
	static void Traverse_Update(Asura_Hash_ID uDefaultControlMapHash, ControlMapStack* pxStack);
	static void Traverse_Deactivate(Asura_Hash_ID uDefaultControlMapHash, ControlMapStack* pxStack);

	static void Reset(ControlMapStack& xStack);
	static void Activate(ControlMapStack& xStack);
	static void Update(ControlMapStack& xStack);
	static void Deactivate(ControlMapStack& xStack);
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifdef ASURA_USE_CONTROLMAP_MANAGER

#endif // #ifndef ASURA_CONTROLMANAGER_H___
