//
// Asura_Input.h
//
// Created by KOR - 05 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_INPUT_H__
#define __ASURA_INPUT_H__

// This needs to go in a platform specific header

// On the PC, is there a theoretical limit to the number of controllers?
#define ASURA_INPUT_PLATFORM_MAX_CONTROLLERS (4)

// Forward declares
class Asura_Controller;

// Class definition
class Asura_Input
{
public:
	static void Initialise();
	static void Shutdown();
	static void Reset();
	static void Update();

	// Controller registration
	static void RegisterController(u_int uControllerIndex, Asura_Controller* pxController);
	static void UnregisterController(u_int uControllerIndex);
	static Asura_Controller* GetController(u_int uControllerIndex);

	static bool HasAnyKeyBeenPressed();
	static bool DebouncedHasAnyKeyBeenPressed();

	static bool IsKeyPressed(u_int uControllerIndex, u_int uKeyID );
	static bool IsDebouncedKeyPressed(u_int uControllerIndex, u_int uKeyID );
	static bool IsDebouncedKeyNotPressed(u_int uControllerIndex,  u_int uKeyID );

	static float GetKeyValue(u_int uControllerIndex, u_int uKeyID );
	static float GetAxisValue(u_int uControllerIndex, u_int uPositiveKeyID, u_int uNegativeKeyID );

	static u_int GetNumConnectedControllers() { return Platform_GetNumConnectedControllers(); }
private:
	static void Platform_Initialise(void);
	static void Platform_Shutdown(void);
	static void Platform_Reset(void);
	static void Platform_Update(void);
	static u_int Platform_GetNumConnectedControllers();

	// Controller storage
	static Asura_Controller* m_apxControllers[ASURA_INPUT_PLATFORM_MAX_CONTROLLERS];

	static bool s_bAnyKeyPressedEvaluated;
	static bool s_bAnyKeyPressed;
	static bool s_bDebouncedAnyKeyPressedHasBeenEvaluated;
	static bool s_bDebouncedAnyKeyHasBeenPressed;
};

#endif//__ASURA_INPUT_H__
