/**
*
* \file Asura_Rumble_System.cpp
*
* Created by James Paulin - 14/08/2002 16:45:52
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
* System redesign - AG - 20/01/2003
*
**/

#include "Core/Asura.h"

#include "Input/Asura_Rumble_System.h"

#include "Core/Asura_Maths.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_Entity.h"

#include <stdio.h>

//*MEMBER FUNCTIONS************************************************

float Asura_Rumble_System::s_fCurrentMotorMagnitudes[PLATFORM_NUM_CONTROLLERS][PLATFORM_NUM_RUMBLE_MOTORS];
Asura_TrackingHandlePool<Asura_HUDRumble>* Asura_Rumble_System::s_pxHUDRumbleList;
Asura_TrackingHandlePool<Asura_WorldRumble>* Asura_Rumble_System::s_pxWorldRumbleList;
Asura_TrackingHandlePool<Asura_Platform_Rumble>* Asura_Rumble_System::s_pxDynamicPlatformRumbleList;
Asura_TrackingHandlePool<Asura_Rumble>* Asura_Rumble::s_pxDynamicRumbleList;
Asura_TrackingHandlePool<Asura_Rumble_Step>* Asura_Rumble::s_pxDynamicRumbleStepList;
float Asura_Rumble_System::s_fCurrentTimeSlice = 0.0f;
bool Asura_Rumble_System::s_bPaused = false;
bool Asura_Rumble_System::s_bIsInitialised = false;

CurrentRumblers Asura_Rumble_System::s_axCurrentUsers[PLATFORM_NUM_CONTROLLERS];

ProjectIsRumbleActiveFunction* Asura_Rumble_System::s_pfnProjectIsRumbleActive = NULL;
ProjectGetRumblerPositionFunction* Asura_Rumble_System::s_pfnProjectGetRumblerPosition = NULL;

void Asura_Rumble_System::Initialise()
{
	s_pxHUDRumbleList = new Asura_TrackingHandlePool<Asura_HUDRumble>(16);
	s_pxWorldRumbleList = new Asura_TrackingHandlePool<Asura_WorldRumble>(16);
	s_pxDynamicPlatformRumbleList = new Asura_TrackingHandlePool<Asura_Platform_Rumble>(16);
	Asura_Rumble::s_pxDynamicRumbleList = new Asura_TrackingHandlePool<Asura_Rumble>(16);
	Asura_Rumble::s_pxDynamicRumbleStepList = new Asura_TrackingHandlePool<Asura_Rumble_Step>(16);

	s_bPaused = false;

	Platform_Initialise();
	Project_Initialise();

	ResetPlayers();

	s_bIsInitialised = true;
}

void Asura_Rumble_System::Shutdown()
{
	StopAllRumbling();
	
	delete s_pxHUDRumbleList;
	s_pxHUDRumbleList = NULL;
	delete s_pxWorldRumbleList;
	s_pxWorldRumbleList = NULL;
	delete s_pxDynamicPlatformRumbleList;
	s_pxDynamicPlatformRumbleList = NULL;
	delete Asura_Rumble::s_pxDynamicRumbleList;
	Asura_Rumble::s_pxDynamicRumbleList = NULL;
	delete Asura_Rumble::s_pxDynamicRumbleStepList;
	Asura_Rumble::s_pxDynamicRumbleStepList = NULL;

	ResetPlayers();
	Platform_Shutdown();
	Project_Shutdown();

	s_bIsInitialised = false;
}

void Asura_Rumble_System::Reset()
{
	StopAllRumbling();

	// (MH and AG - 23.04.2004) these weren't being reset here.
	// we found that in Sniper this function wasn't even called at all.
	s_pxHUDRumbleList->Reset();
	s_pxWorldRumbleList->Reset();
	s_pxDynamicPlatformRumbleList->Reset();
	Asura_Rumble::s_pxDynamicRumbleList->Reset();
	Asura_Rumble::s_pxDynamicRumbleStepList->Reset();
	
	s_bPaused = false;
	Platform_Reset();
	Project_Reset();

	ResetPlayers();
}

void Asura_Rumble_System::AddPlayer(int iPlayerID, Asura_Guid uPlayer)
{
    Asura_Assert(iPlayerID >= 0 && iPlayerID < PLATFORM_NUM_CONTROLLERS, "Invalid player id");
	s_axCurrentUsers[iPlayerID].uGuid = uPlayer;
    s_axCurrentUsers[iPlayerID].iController = iPlayerID;
    s_axCurrentUsers[iPlayerID].fRumbleMultiplier = 1.f;
}

void Asura_Rumble_System::SetPlayerRumbleMultiplier(u_int uPlayerID, float fMultiplier)
{
    if (uPlayerID < static_cast<u_int>(PLATFORM_NUM_CONTROLLERS))
    {
        Asura_Assert(fMultiplier <= 1.f, "Unexpected rumble value");
        s_axCurrentUsers[uPlayerID].fRumbleMultiplier = fMultiplier;
    }
    else
    {
        Asura_Assert(false, "Invalid player id");
    }
}

float Asura_Rumble_System::GetPlayerRumbleMultiplier(u_int uPlayerID)
{
    if (uPlayerID < static_cast<u_int>(PLATFORM_NUM_CONTROLLERS))
    {
        return s_axCurrentUsers[uPlayerID].fRumbleMultiplier;
    }
    else
    {
        Asura_Assert(false, "Invalid player id");
        return 0.f;
    }
}


void Asura_Rumble_System::RemovePlayer(int iPlayerID)
{
    Asura_Assert(iPlayerID >= 0 && iPlayerID < PLATFORM_NUM_CONTROLLERS, "Invalid player id");
	s_axCurrentUsers[iPlayerID].uGuid = ASURA_GUID_UNREGISTERED;
	s_axCurrentUsers[iPlayerID].iController = 0;
}

void Asura_Rumble_System::ResetPlayers()
{
	for (int i=0; i<PLATFORM_NUM_CONTROLLERS; i++)
	{
    	s_axCurrentUsers[i].uGuid = ASURA_GUID_UNREGISTERED;
	    s_axCurrentUsers[i].iController = 0;
	}
}

void Asura_Rumble_System::Update()
{
    // (NRM - 01.04.2003) Changed this from GetGameTimeSlice to GetFrameTime
	s_fCurrentTimeSlice = Asura_Timers::GetFrameTime();
    // (NRM - 23.07.2003) removed this code as when you die (in Dredd at least) the time scale is not set to zero
    // (NRM - 23.07.2003) so you now have to call PauseAllRumbling and UnpauseAllRumbling explicitly
//    if(Asura_Timers::GetGameTimeScale() == 0.0f)
//    {
//        PauseAllRumbling();
//    }
//    else
//    {
//        UnpauseAllRumbling();
//    }
	Update2();
}

void Asura_Rumble_System::Update2()
{
	for (int i=0; i<PLATFORM_NUM_CONTROLLERS; i++)
	{
		for (int j=0; j<PLATFORM_NUM_RUMBLE_MOTORS; j++)
		{
			s_fCurrentMotorMagnitudes[i][j] = 0.0f;
		}
	}

	if (!s_bPaused)
	{
        s_pxHUDRumbleList->Traverse(CheckHUDRumbleCallback);
        s_pxWorldRumbleList->Traverse(CheckWorldRumbleCallback);

        for (int i=0; i<PLATFORM_NUM_CONTROLLERS; i++)
	    {
			bool bShouldPlay = true;
			if(s_pfnProjectIsRumbleActive)
			{
				if(!s_pfnProjectIsRumbleActive(i))
				{
					bShouldPlay = false;
				}
			}

			if(bShouldPlay)
			{
				for (int j=0; j<PLATFORM_NUM_RUMBLE_MOTORS; j++)
				{
				    s_fCurrentMotorMagnitudes[i][j] *= s_axCurrentUsers[i].fRumbleMultiplier;
			    }
			}
			else
			{
				for (int j=0; j<PLATFORM_NUM_RUMBLE_MOTORS; j++)
				{
				    s_fCurrentMotorMagnitudes[i][j] = 0.0f;
			    }
			}
	    }

		SanityCheck();
	}
	
	Platform_PlayCurrentRumbles();
}

void Asura_Rumble_System::CheckHUDRumbleCallback(Asura_HUDRumble* pxThisRumble)
{
	if (pxThisRumble != NULL)
	{
		if (!pxThisRumble->m_bPaused)
		{
			Asura_Assert(pxThisRumble->m_iEffectPlaysRemaining, "No plays remaining?");
			if(!pxThisRumble->m_iEffectPlaysRemaining)
			{
				return;
			}

			bool bContinueLooping = true;

			float fMinOverlap;
			if (pxThisRumble->m_bStarted)
			{
				fMinOverlap = s_fCurrentTimeSlice;
			}
			else
			{
				fMinOverlap = 0.0f;
				pxThisRumble->m_bStarted = true;
			}
			while (bContinueLooping)
			{
				float fCurrentTimeSlice = fMinOverlap;
				for (int i=0; i<PLATFORM_NUM_RUMBLE_MOTORS; i++)
				{
					if (pxThisRumble->m_pxActiveRumble &&
                        pxThisRumble->m_pxActiveRumble->m_apxMotorRumbles[i])
					{
						float fUpdateValue = pxThisRumble->Update(fCurrentTimeSlice, i);
						if (fUpdateValue < 0.0f)
						{
							if (fMinOverlap > -fUpdateValue)
							{
								fMinOverlap = -fUpdateValue;
							}
						}
						else
						{
							s_fCurrentMotorMagnitudes[pxThisRumble->m_iController][i] += fUpdateValue;
							bContinueLooping = false;
						}
					}
					else
					{
                    // (NRM - 28.03.2003) - no longer needed as NULL data is now valid.
					//	Asura_Assert(0, "Rumble has no motor data");
					}
				}
	            if (bContinueLooping)
	            {
		            if (pxThisRumble->m_iEffectPlaysRemaining == 1)
		            {
            	        s_pxHUDRumbleList->Deallocate(pxThisRumble->m_iMyHandle);
			            bContinueLooping = false;
		            }
		            else 
		            {
			            if (pxThisRumble->m_iEffectPlaysRemaining > 1)
			            {
				            pxThisRumble->m_iEffectPlaysRemaining--;
			            }
			            pxThisRumble->ResetPositions();
		            }
	            }
			}
		}
	}
	else
	{
		Asura_Assert(0, "Rumble handle has null pointer");
	}
}

void Asura_Rumble_System::CheckWorldRumbleCallback(Asura_WorldRumble* pxThisRumble)
{
	if (pxThisRumble == NULL)
	{
		Asura_Assert(0, "Rumble has null pointer");
        return;
    }

	if (pxThisRumble->m_bPaused)
	{
        return;
    }	

    // Calculate distances from users to explosion
	float axUserDistances[PLATFORM_NUM_CONTROLLERS];
	for (int iController=0; iController<PLATFORM_NUM_CONTROLLERS; iController++)
	{
		axUserDistances[iController] = -1.0f;
		
		bool bPositionValid = false;
		Asura_Vector_3 xPosition;
		
		if(s_pfnProjectGetRumblerPosition)
		{
			bPositionValid = s_pfnProjectGetRumblerPosition(s_axCurrentUsers[iController].uGuid, xPosition);
		}
		else
		{
			if (s_axCurrentUsers[iController].uGuid != ASURA_GUID_UNREGISTERED)
			{
				Asura_Entity_Renderable* pxEntity = Asura_Entity_Renderable::FindEntity(s_axCurrentUsers[iController].uGuid);
				Asura_Assert(pxEntity != NULL, "No entity with given guid");
				if (pxEntity != NULL)
				{
					xPosition = pxEntity->GetCurrentPosition();
					// MRD 07/11/08: Integrated changelist 151370  from CoD5, this allows world rumbles to work
					bPositionValid = true;
				}
			}
		}
		if(bPositionValid)
		{			
			xPosition -= pxThisRumble->m_xPosition;
			axUserDistances[iController] = xPosition.Magnitude();			
		}
	}

	float fMinOverlap;
	if (pxThisRumble->m_bStarted)
	{
		fMinOverlap = s_fCurrentTimeSlice;
	}
	else
	{
		fMinOverlap = 0.0f;
		pxThisRumble->m_bStarted = true;
	}

	Asura_Assert(pxThisRumble->m_iEffectPlaysRemaining, "No plays remaining?");
	if(!pxThisRumble->m_iEffectPlaysRemaining)
	{
		return;
	}

	bool bContinueLooping = true;
	while (bContinueLooping)
	{
		float fCurrentTimeSlice = fMinOverlap;
		for (int iMotor=0; iMotor<PLATFORM_NUM_RUMBLE_MOTORS; iMotor++)
		{
			if (pxThisRumble->m_pxActiveRumble &&
                pxThisRumble->m_pxActiveRumble->m_apxMotorRumbles[iMotor])
			{
				float fUpdateValue = pxThisRumble->Update(fCurrentTimeSlice, iMotor);
				if (fUpdateValue < 0.0f)
				{
					if (fMinOverlap > -fUpdateValue)
					{
						fMinOverlap = -fUpdateValue;
					}
				}
				else
				{
					for (int iPlayer=0; iPlayer<PLATFORM_NUM_CONTROLLERS; iPlayer++)
					{
						if (axUserDistances[iPlayer] < 0.0f) { continue; }
						if (axUserDistances[iPlayer] < pxThisRumble->m_fOuterRadius)
						{
							if (axUserDistances[iPlayer] <= pxThisRumble->m_fInnerRadius)
							{
								s_fCurrentMotorMagnitudes[s_axCurrentUsers[iPlayer].iController][iMotor] += fUpdateValue;
							}
							else
							{
								//AG - 03/02/03 - Subtracting 1-(distance adjustment) now instead of multiplying by distance adjustment.
								// Advantage: Low magnitude rumbles aren't felt for the full radius (which seems reasonable)
								// Disadvantage: Rumbles whose greatest magnitude is less than 1 will never be felt at the full radius, which would therefore have to be taken into account when deciding on an appropriate radius.
								const float fMod =  fUpdateValue - 1.0f + ((pxThisRumble->m_fOuterRadius - axUserDistances[iPlayer]) / (pxThisRumble->m_fOuterRadius - pxThisRumble->m_fInnerRadius));
								if (fMod>0.0f)
								{
									s_fCurrentMotorMagnitudes[s_axCurrentUsers[iPlayer].iController][iMotor] += fMod;
								}
							}
						}
					}
					bContinueLooping = false;
				}
			}
		}
	    if (bContinueLooping)
	    {
		    if (pxThisRumble->m_iEffectPlaysRemaining == 1)
		    {
            	s_pxWorldRumbleList->Deallocate(pxThisRumble->m_iMyHandle);
			    bContinueLooping = false;
		    }
		    else 
		    {
			    if (pxThisRumble->m_iEffectPlaysRemaining > 1)
			    {
				    pxThisRumble->m_iEffectPlaysRemaining--;
			    }
			    pxThisRumble->ResetPositions();
		    }
	    }
	}
}

void Asura_Rumble_System::PauseRumble(Asura_Handle iHandle)
{
	PauseRumbleCallback(s_pxWorldRumbleList->GetPointer(iHandle));
}	

void Asura_Rumble_System::UnpauseRumble(Asura_Handle iHandle)
{
	UnpauseRumbleCallback(s_pxWorldRumbleList->GetPointer(iHandle));
}	

bool Asura_Rumble_System::IsHUDRumbleActive(Asura_Handle iHandle)
{
	return s_pxHUDRumbleList->GetPointer(iHandle) != NULL;
}


void Asura_Rumble_System::PauseRumbleCallback(Asura_WorldRumble *pxThisRumble)
{
	if (pxThisRumble != NULL)
	{
		pxThisRumble->m_bPaused = true;
	}
	else
	{
		Asura_Assert(0, "Rumble handle has null pointer");
	}
}	

void Asura_Rumble_System::UnpauseRumbleCallback(Asura_WorldRumble *pxThisRumble)
{
	if (pxThisRumble != NULL)
	{
		pxThisRumble->m_bPaused = false;
	}
	else
	{
		Asura_Assert(0, "Rumble handle has null pointer");
	}
}

//Note - controllers will be numbered 1-(max controllers) for this function, converting to the range 0-(max controllers - 1) here
Asura_Handle Asura_Rumble_System::AddHUDRumble(Asura_Platform_Rumble* pxRumble, int iControllerNumber, 
                                               int iEffectPlays, float fMaxMagnitude, 
                                               float fMinMagnitude, float fDecay, bool bPause)
{
	if(s_bPaused)
	{
		//Dont add rumbles if we are paused, there is no point letting them batch up
		return ASURA_INVALID_HANDLE;
	}

	if (iControllerNumber < 1 || iControllerNumber > PLATFORM_NUM_CONTROLLERS)
	{
		Asura_Assert(0, "Invalid controller given for rumble");
		return ASURA_INVALID_HANDLE;
	}
	else
	{
		Asura_Handle iNewHandle = s_pxHUDRumbleList->Allocate();
		if (iNewHandle == ASURA_INVALID_HANDLE)
		{
			Asura_Assert(0, "Problem occurred allocating handle for new active rumble");
			return ASURA_INVALID_HANDLE;
		}
		else
		{
			Asura_HUDRumble* pxNewRumble = s_pxHUDRumbleList->GetPointer(iNewHandle);
			pxNewRumble->m_iController = iControllerNumber-1;
			AddRumbleCommon(pxNewRumble, iNewHandle, pxRumble, iEffectPlays, fMaxMagnitude, fMinMagnitude, fDecay, bPause, ASURA_HUD_RUMBLE);
			return iNewHandle;
		}
	}
}
Asura_Handle Asura_Rumble_System::AddHUDRumble(int iControllerNumber, const Asura_HUDRumble_Template& xRumbleTemplate, 
                                                bool bPause)
{
    return AddHUDRumble(xRumbleTemplate.m_pxRumble, iControllerNumber, xRumbleTemplate.m_iEffectPlays,
                        xRumbleTemplate.m_fMaxMagnitude, xRumbleTemplate.m_fMinMagnitude, 
                        xRumbleTemplate.m_fDecay, bPause);
}

Asura_Handle Asura_Rumble_System::AddWorldRumble(Asura_Platform_Rumble* pxRumble, const Asura_Vector_3& xPosition, 
                                                 float fOuterRadius, float fInnerRadius, int iEffectPlays, 
											     float fMaxMagnitude, float fMinMagnitude, float fDecay, bool bPause)
{
	if(s_bPaused)
	{
		//Dont add rumbles if we are paused, there is no point letting them batch up
		return ASURA_INVALID_HANDLE;
	}

	Asura_Handle iNewHandle = s_pxWorldRumbleList->Allocate();
	if (iNewHandle == ASURA_INVALID_HANDLE)
	{
		Asura_Assert(0, "Problem occurred allocating handle for new active rumble");
		return ASURA_INVALID_HANDLE;
	}
	else
	{
		Asura_WorldRumble* pxNewRumble = s_pxWorldRumbleList->GetPointer(iNewHandle);
		pxNewRumble->m_xPosition = xPosition;
		pxNewRumble->m_fOuterRadius = fOuterRadius;
		pxNewRumble->m_fInnerRadius = fInnerRadius;
		AddRumbleCommon(pxNewRumble, iNewHandle, pxRumble, iEffectPlays, 
                        fMaxMagnitude, fMinMagnitude, fDecay, bPause, ASURA_WORLD_RUMBLE);
		return iNewHandle;
	}
}

Asura_Handle Asura_Rumble_System::AddWorldRumble(const Asura_Vector_3 &xPosition, 
                                                 Asura_WorldRumble_Template &xRumbleTemplate,
                                                 bool bPause)
{
    return AddWorldRumble(xRumbleTemplate.m_pxRumble, xPosition, xRumbleTemplate.m_fOuterRadius,
                          xRumbleTemplate.m_fInnerRadius, xRumbleTemplate.m_iEffectPlays,
                          xRumbleTemplate.m_fMaxMagnitude, xRumbleTemplate.m_fMinMagnitude,
                          xRumbleTemplate.m_fDecay, bPause);
}

void Asura_Rumble_System::AddRumbleCommon(Asura_ActiveRumble* pxNewRumble, Asura_Handle iNewHandle, 
                                          Asura_Platform_Rumble* pxActiveRumble, int iEffectPlays, 
                                          float fMaxMagnitude, float fMinMagnitude, float fDecay, 
                                          bool bPause, ASURA_ACTIVE_RUMBLE_TYPE eType)
{
	pxNewRumble->m_pxActiveRumble = pxActiveRumble;
	pxNewRumble->m_iEffectPlaysRemaining = iEffectPlays;
	pxNewRumble->m_fMaxMagnitude = fMaxMagnitude;
	pxNewRumble->m_fMinMagnitude = fMinMagnitude;
	pxNewRumble->m_fDecay = fDecay;
	pxNewRumble->m_bPaused = bPause;
	pxNewRumble->m_iMyHandle = iNewHandle;
	pxNewRumble->m_bStarted = false;
	pxNewRumble->m_eMyType = eType;
	pxNewRumble->ResetPositions();
}

void Asura_Rumble_System::ChangeRumbleMaxMinRelativeMagnitude(Asura_Handle iHandle, float fNewMaxMagnitude)
{
	Asura_WorldRumble* pxRumble = s_pxWorldRumbleList->GetPointer(iHandle);
	if (pxRumble == NULL)
	{
		Asura_Assert(0, "Invalid handle for magnitude change");
	}
	else
	{
		pxRumble->ChangeMaxMinRelativeMagnitude(fNewMaxMagnitude);
	}
}

void Asura_Rumble_System::ChangeRumbleMaxMagnitude(Asura_Handle iHandle, float fNewMaxMagnitude)
{
	Asura_WorldRumble* pxRumble = s_pxWorldRumbleList->GetPointer(iHandle);
	if (pxRumble == NULL)
	{
		Asura_Assert(0, "Invalid handle for magnitude change");
	}
	else
	{
		pxRumble->ChangeMaxMagnitude(fNewMaxMagnitude);
	}
}

void Asura_Rumble_System::ChangeRumbleMinMagnitude(Asura_Handle iHandle, float fNewMinMagnitude)
{
	Asura_WorldRumble* pxRumble = s_pxWorldRumbleList->GetPointer(iHandle);
	if (pxRumble == NULL)
	{
		Asura_Assert(0, "Invalid handle for magnitude change");
	}
	else
	{
		pxRumble->ChangeMinMagnitude(fNewMinMagnitude);
	}
}

void Asura_Rumble_System::ChangeRumbleDecay(Asura_Handle iHandle, float fNewDecay)
{
	Asura_WorldRumble* pxRumble = s_pxWorldRumbleList->GetPointer(iHandle);
	if (pxRumble == NULL)
	{
		Asura_Assert(0, "Invalid handle for decay change");
	}
	else
	{
		pxRumble->ChangeDecay(fNewDecay);
	}
}

void Asura_Rumble_System::StopRumble(Asura_Handle iHandle)
{
	s_pxWorldRumbleList->Deallocate(iHandle);
}

void Asura_Rumble_System::StopHUDRumble(Asura_Handle iHandle)
{
	s_pxHUDRumbleList->Deallocate(iHandle);
}

void Asura_Rumble_System::StopAllRumbling()
{
	// JAT 1/11/07 - only do the HuD rumble if the list exists!
    // HUD
	if ( s_pxHUDRumbleList != NULL )
	{
		Asura_TrackingHandlePool<Asura_HUDRumble>::Iterator itHUDRumble = s_pxHUDRumbleList->GetIterator();
		for(itHUDRumble.Begin(); !itHUDRumble.End(); itHUDRumble.Next())
		{
		    //Asura_HUDRumble* pxHUDRumble = itHUDRumble.GetCurrentPointer();
		    s_pxHUDRumbleList->Deallocate(itHUDRumble.GetCurrentHandle());
		}
	}
    
	// JAT 1/11/07 - only do the HuD rumble if the list exists!
	if ( s_pxWorldRumbleList != NULL )
	{
		// WORLD
		Asura_TrackingHandlePool<Asura_WorldRumble>::Iterator itWorldRumble = s_pxWorldRumbleList->GetIterator();
		for(itWorldRumble.Begin(); !itWorldRumble.End(); itWorldRumble.Next())
		{
		    //Asura_WorldRumble* pxWorldRumble = itWorldRumble.GetCurrentPointer();
		    s_pxWorldRumbleList->Deallocate(itWorldRumble.GetCurrentHandle());
		}
	}
	s_bPaused = false;

	// (MH and AG - 23.04.2004) - NRM had added calls to PlatformReset and ProjectReset
	// which looked like they were there to fix a bug where the rumbling didn't stop
	// immediately. We think this new function is a better solution.
	Platform_StopAllRumbles();
}


void Asura_Rumble_System::StopPlayerRumbles(int iControllerNumber)
{
    Asura_TrackingHandlePool<Asura_HUDRumble>::Iterator itHUDRumble = s_pxHUDRumbleList->GetIterator();
    for(itHUDRumble.Begin(); !itHUDRumble.End(); itHUDRumble.Next())
    {
        Asura_HUDRumble* pxHUDRumble = itHUDRumble.GetCurrentPointer();
        if(pxHUDRumble->m_iController == iControllerNumber)
        {
            s_pxHUDRumbleList->Deallocate(itHUDRumble.GetCurrentHandle());
        }
    }    
}

void Asura_Rumble_System::StopHUDRumbles()
{
	s_pxHUDRumbleList->Reset();
}

Asura_Handle Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_EFFECT eBase, RUMBLE_ADJUST eAdjust, float fMaxMagnitude,
					float fMinMagnitude, float fDecay, int iPlays)
{
	Asura_Handle iNewHandle = s_pxDynamicRumbleList->Allocate();
	Asura_Handle iNewStepHandle = s_pxDynamicRumbleStepList->Allocate();
	if (iNewHandle == ASURA_INVALID_HANDLE || iNewStepHandle == ASURA_INVALID_HANDLE)
	{
		Asura_Assert(0, "Problem occurred allocating handle for new rumble or rumble step");
		return ASURA_INVALID_HANDLE;
	}
	else
	{
		Asura_Rumble* pxNewRumble = s_pxDynamicRumbleList->GetPointer(iNewHandle);
		Asura_Rumble_Step* pxStep = s_pxDynamicRumbleStepList->GetPointer(iNewStepHandle);
		pxStep->eBaseEffect = eBase;
		pxStep->eAdjustment = eAdjust;
		pxStep->fStepMaxMagnitude = fMaxMagnitude;
		pxStep->fStepMinMagnitude = fMinMagnitude;
		pxStep->fStepDecay = fDecay;
		pxStep->iStepEffectPlays = iPlays;
		pxStep->pxNextStep = NULL;
		pxNewRumble->m_pxFirstStep = pxStep;
		pxNewRumble->m_fMotorDecay = 1.0;
		pxNewRumble->m_fMotorMaxMagnitude = 1.0f;
		pxNewRumble->m_fMotorMinMagnitude = 0.0f;
		return iNewHandle;
	}
}

void Asura_Rumble::AddRumbleStep(Asura_Handle iRumbleHandle, RUMBLE_BASE_EFFECT eBase, RUMBLE_ADJUST eAdjust, float fMaxMagnitude,
					float fMinMagnitude, float fDecay, int iPlays)
{
	Asura_Handle iNewStepHandle = s_pxDynamicRumbleStepList->Allocate();
	if (iNewStepHandle == ASURA_INVALID_HANDLE)
	{
		Asura_Assert(0, "Problem occurred allocating handle for new rumble step");
	}
	else
	{
		Asura_Rumble* pxRumble = s_pxDynamicRumbleList->GetPointer(iRumbleHandle);
		Asura_Rumble_Step* pxStep = s_pxDynamicRumbleStepList->GetPointer(iNewStepHandle);
		pxStep->eBaseEffect = eBase;
		pxStep->eAdjustment = eAdjust;
		pxStep->fStepMaxMagnitude = fMaxMagnitude;
		pxStep->fStepMinMagnitude = fMinMagnitude;
		pxStep->fStepDecay = fDecay;
		pxStep->iStepEffectPlays = iPlays;
		pxStep->pxNextStep = NULL;
		Asura_Rumble_Step* pxSearchStep = pxRumble->m_pxFirstStep;
		while (pxSearchStep->pxNextStep != NULL)
		{
			pxSearchStep = pxSearchStep->pxNextStep;
		}
		pxSearchStep->pxNextStep = pxStep;
	}
}

float Asura_ActiveRumble::Update(float fTimeSlice, int iMotor)
{
	if (m_pxActiveRumble->m_apxMotorRumbles[iMotor] == NULL)
	{
		Asura_Assert(0, "Rumble is missing motor data");
		return 0.0f;
	}
	else
	{
		Asura_Rumble* pxMotorRumble = m_pxActiveRumble->m_apxMotorRumbles[iMotor];
		RumblePosition* pxPosition = &m_axMotorEffectPositions[iMotor];
		float fTimePassed = fTimeSlice * m_fDecay * pxMotorRumble->m_fMotorDecay 
			* pxPosition->pxCurrentStep->fStepDecay;
		fTimePassed += pxPosition->fStepPosition;
		for (int iStepsPassed = Asura_Maths::FloatToInt_Chop(fTimePassed); iStepsPassed>0;)
		{
			if (!pxPosition->pxCurrentStep->iStepEffectPlays)
			{
				fTimePassed -= (float)iStepsPassed;
				iStepsPassed = 0;
			}
			else if (pxPosition->iStepEffectPlaysRemaining > iStepsPassed)
			{
				fTimePassed -= (float)iStepsPassed;
				pxPosition->iStepEffectPlaysRemaining -= iStepsPassed;
				iStepsPassed = 0;
			}
			else
			{
				fTimePassed -= (float)(pxPosition->iStepEffectPlaysRemaining);
				fTimePassed /= pxPosition->pxCurrentStep->fStepDecay;
				if (pxPosition->pxCurrentStep->pxNextStep == NULL)
				{
					pxPosition->fStepPosition = 1.0f;
					return -fTimePassed;
				}
				else
				{
					pxPosition->pxCurrentStep = pxPosition->pxCurrentStep->pxNextStep;
					fTimePassed *= pxPosition->pxCurrentStep->fStepDecay;
					pxPosition->iStepEffectPlaysRemaining = pxPosition->pxCurrentStep->iStepEffectPlays;
					iStepsPassed = Asura_Maths::FloatToInt_Chop(fTimePassed);
				}
			}
		}
		pxPosition->fStepPosition = fTimePassed;

		float fFunctionValue = Asura_Rumble_System::GetBaseEffect(pxPosition->fStepPosition, 
			pxPosition->pxCurrentStep->eBaseEffect, pxPosition->pxCurrentStep->eAdjustment);

		fFunctionValue *= pxPosition->pxCurrentStep->fStepMaxMagnitude - pxPosition->pxCurrentStep->fStepMinMagnitude;
		fFunctionValue += pxPosition->pxCurrentStep->fStepMinMagnitude;
		
		fFunctionValue *= m_fMaxMagnitude - m_fMinMagnitude;
		fFunctionValue += m_fMinMagnitude;

		return fFunctionValue;
	}
}

void Asura_HUDRumble_Template::Create(Asura_Platform_Rumble* pxRumble, float fMaxMagnitude,
                                      float fMinMagnitude, float fDecay,
                                      int   iEffectPlays)
{
    m_pxRumble = pxRumble;
    m_fMaxMagnitude = fMaxMagnitude;
    m_fMinMagnitude = fMinMagnitude;
    m_fDecay = fDecay;
    m_iEffectPlays = iEffectPlays;
}

void Asura_WorldRumble_Template::Create(Asura_Platform_Rumble* pxRumble, float fMaxMagnitude,
                                        float fMinMagnitude, float fDecay,
                                        int   iEffectPlays,  float fOuterRadius, float fInnerRadius)
{
    m_pxRumble = pxRumble;
    m_fMaxMagnitude = fMaxMagnitude;
    m_fMinMagnitude = fMinMagnitude;
    m_fDecay = fDecay;
    m_iEffectPlays = iEffectPlays;
    m_fOuterRadius = fOuterRadius;
    m_fInnerRadius = fInnerRadius;
}

// JCT - Stubbed out platform functions if rumble isn't supported.
// This is so projects can still call independent rumble code, even when not supported.
#if !(defined ASURA_SUPPORT_CONTROLLER_RUMBLE) || (ASURA_SUPPORT_CONTROLLER_RUMBLE == 0)

void Asura_Rumble_System::Platform_Initialise() {};
void Asura_Rumble_System::Platform_Shutdown() {};
void Asura_Rumble_System::Platform_Reset() {};
void Asura_Rumble_System::Platform_StopAllRumbles() {};
void Asura_Rumble_System::Platform_PlayCurrentRumbles() {};

#endif