//////////////////////////////////////////////////////////////////////////////
//
// Asura_CommandConsole_IO.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_COMMANDCONSOLE_IO_H___
#define ___ASURA_COMMANDCONSOLE_IO_H___ 1

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Core/Asura_Handle.h"
#ifndef ASURA_COMMAND_CONSOLE_READONLY
#include "Core/Asura_UserInput.h"
#endif

// Engine platform-specific.
#include "Core/Asura_PlatformBasics.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_CommandConsole;

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int CONSOLE_NUM_LINES_TO_DISPLAY       = 14;


// C L A S S E S /////////////////////////////////////////////////////////////

// Interface.
class Asura_CommandConsole_IO
{

    public:
    
        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        virtual ~Asura_CommandConsole_IO( void ) { }

        // M E T H O D S /////////////////////////////////////////////////////

        virtual void Reset( void ) = 0;

        virtual void Update( void ) = 0;
        virtual void Render( void ) = 0;

        virtual bool CanBeHidden( void ) const = 0;                             // Obviously must always return the same value.
        virtual void SetDown( bool bDown, bool bInstantTransition ) = 0;        // Doesn't need to do anything if CanBeHidden() returns false.
		virtual void SetPartiallyDown(u_int uLines, bool bInstantTransition){};	// Displays just some lines of the console

#ifndef ASURA_COMMAND_CONSOLE_READONLY
        virtual bool CanSupplyInput( void ) const = 0;                          // Obviously must always return the same value.
        virtual void GetInput( void(*pfnInputCallback)(ASURA_KEYID) ) = 0;      // Doesn't need to do anything if CanSupplyInput() returns false.
#endif
        
        virtual bool CanArchiveOutput( void ) const = 0;                        // Obviously must always return the same value.
        virtual void AddOutput( const Asura_UnicodeChar* wszLatestOutput ) = 0; // Doesn't need to do anything if CanArchiveOutput() returns false.
        virtual void ClearOutput( void ) = 0;                                   //                                "

        virtual void Scroll( bool bDown, bool bToEnd ) = 0;

        virtual void SetTextScale( float fScale ) = 0;
		virtual float GetTextScale( void ) const { return 0.0f; }
        virtual void SetBorderX( float fBorderX ) = 0;
		virtual float GetBorderX( void ) const { return 0.0f; }

		virtual void SetTexture( int iTextureID ) {}
		virtual void SetBackgroundColour( u_int uColour ) {}
		virtual void SetFont( const char* szFontName ) {}
		virtual bool SetFont( Asura_Handle iHandle ) { return false; }

        virtual const Asura_UnicodeChar* GetLineSplitPoint(const Asura_UnicodeChar* wszLine)=0;

		virtual float GetBottomY() const {return 0.0f;} //get the current bottom of the console
		virtual u_int GetFirstLineToDisplay( void ) { return 0; }

		virtual void SetFirstLineToDisplay(u_int uLine) {}
};

// Default implementation for standard in-game console.
class Asura_CommandConsole_IO_Default : public Asura_CommandConsole_IO
{

    public:

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        Asura_CommandConsole_IO_Default( void );
        virtual ~Asura_CommandConsole_IO_Default( void ) { }

        // M E T H O D S /////////////////////////////////////////////////////

        virtual void Reset( void );

        virtual void Update( void );
        virtual void Render( void );
#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)
		void RenderDebugVariableSystem();
		static void SetDVSBorder( float fBorder ) { s_fDVSBorder = fBorder; } // relative to screen width as 0-1
protected:
		void RenderDVSMenu(float& fXPos, float& fYPos, u_int uLevel);
public:
#endif // ASURA_DEBUG_VARIABLE_SYSTEM

        virtual bool CanBeHidden( void ) const { return true; }
        virtual void SetDown( bool bDown, bool bInstantTransition );
		virtual void SetPartiallyDown(u_int uLines, bool bInstantTransition);

#ifndef ASURA_COMMAND_CONSOLE_READONLY
        virtual bool CanSupplyInput( void ) const { return false; }
        virtual void GetInput( void(*pfnInputCallback)(ASURA_KEYID) ) { }
#endif

        virtual bool CanArchiveOutput( void ) const { return false; }
        virtual void AddOutput( const Asura_UnicodeChar* wszLatestOutput ) { }
        virtual void ClearOutput( void ) { }

        virtual void Scroll( bool bDown, bool bToEnd );

        virtual void SetTextScale( float fScale ) { m_fFontScale = fScale; }
        virtual void SetBorderX( float fBorderX ) { m_fBorderX = fBorderX; }
		virtual float GetTextScale( void ) const { return m_fFontScale; }
		virtual float GetBorderX( void ) const { return m_fBorderX; }

		virtual void SetTexture( int iTextureID ) { m_iTextureIdx = iTextureID; }
		virtual void SetBackgroundColour( u_int uColour ) { m_uBackgroundColour = uColour; }
		virtual void SetFont( const char* szFontName );
		virtual bool SetFont( Asura_Handle iHandle );

        virtual const Asura_UnicodeChar* GetLineSplitPoint(const Asura_UnicodeChar* wszLine);

		virtual float GetBottomY() const {return m_fPosY;} 
		virtual u_int GetFirstLineToDisplay( void ) { return m_uFirstLineToDisplay; }

		virtual void SetFirstLineToDisplay(u_int uLine) { m_uFirstLineToDisplay = uLine; }

		static void CatchCommandOutputCallback( const Asura_UnicodeChar* wszOutput );
    private:

		void CheckFontHandle( void );
		
        // S T A T E /////////////////////////////////////////////////////////

        bool         m_bDown;
		bool         m_bDisplayInputBuffer;
        float        m_fPosY;
		float        m_fDestY;
        u_int        m_uFirstLineToDisplay;

        float        m_fBorderX;
        int          m_iTextureIdx;
		u_int		 m_uBackgroundColour;
        Asura_Handle m_iFontHandle;
		const char*  m_szFontName;
        float        m_fFontScale;

        float        m_fTimeUntilNextBlink;
        float        m_fDurationOfCurBlink;

		static Asura_UnicodeChar* s_wszCommandOutputBuffer;
		static u_int s_uCommandOutputBufferSize;

#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)
		static float s_fDVSBorder;
#endif

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifdef ASURA_SUPPORT_COMMAND_CONSOLE

#endif // #ifndef ___ASURA_COMMANDCONSOLE_IO_H___
