//
// Asura_Graph_Dialog.h
//
// Created by KOR - 17/02/2004 11:14:08
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_GRAPH_DIALOG_H__
#define __ASURA_GRAPH_DIALOG_H__

#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Graph_Gadget.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Text_Output.h"

#include "Interface/Tools_ColourPicker.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Interface/Tools_Overlay_EnlargedGraph.h"
#include "Misc/Tools_Dialog.h"

template <class T> class Asura_Graph_Dialog : public Asura_Dialog
{
	public:
		Asura_Graph_Dialog( const char* szName, 
							Asura_Vector_2 xOffset,
							Asura_Graph<T>* pxGraph,
							float fRangeMin,
							float fRangeMax,
							bool* pbFlag = 0)
		: Asura_Dialog(szName,xOffset)
		, m_pxGraph(pxGraph)
		, m_fRangeMin(fRangeMin)
		, m_fRangeMax(fRangeMax)
		, m_fCurrentValue(fRangeMin)
		, m_fCurrentTime(0.f)
		, m_bCurrentValueAltered(false)
		, m_bCurrentTimeAltered(false)
		, m_fR(0.f)
		, m_fG(0.f)
		, m_fB(0.f)
		, m_bColourChanged(false)
		, m_bApplyColourChangeToAllNodes(false)
		, m_pxColourPicker(NULL)
		, m_pxColourPickerButton(NULL)
		, m_bShowPointControls(true)
		, m_bForceNormalisedAlpha(false)
		, m_pbChangedFlag(pbFlag)
		, m_fGraphPixelWidth(ASURA_GRAPH_WIDTH)
		, m_fGraphPixelHeight(ASURA_GRAPH_HEIGHT)
		, m_fPreResizeWidth(ASURA_GRAPH_WIDTH)
		, m_fPreResizeHeight(ASURA_GRAPH_HEIGHT)
		, m_bOpenedGraphOverlay(false)
		, m_bMemberOfGraphOverlay(false)
		{
			float fMinimum;
			float fMaximum;
			if (!pxGraph->GetClampMin(fMinimum))
			{
				fMinimum = -Asura_Maths::LargeFloat;
			}
			if (!pxGraph->GetClampMax(fMaximum))
			{
				fMaximum = Asura_Maths::LargeFloat;
			}
			Asura_Assert(fMinimum < fMaximum, "Min and max wrong way around?");
			Asura_Assert(fRangeMin < fRangeMax, "Min and max wrong way around?");
			Asura_Assert(fMinimum <= fRangeMin, "Visible range minimum less than graph minimum");
			Asura_Assert(fMaximum >= fRangeMax, "Visible range maximum greater than graph maximum");
			// Create associated graph gadget
			Asura_Gadget* pxGadget;
			Asura_Vector_2 xNewOffset = xOffset;
			pxGadget = m_pxGraphGadget = new Asura_Graph_Gadget<T>(xNewOffset, m_pxGraph, this, fRangeMin, fRangeMax, pbFlag);
			AddGadget(pxGadget);

			
			// (Note that positions of gadgets are now not set until RepositionGadgets() is called at the end of this function)
			// Graph dimension filter buttons (top right)
			u_int uNumDimensions = m_pxGraphGadget->GetNumberOfDimensions();
			m_ppxFilterIcons = uNumDimensions ? new Asura_Icon_Gadget*[uNumDimensions] : 0;
			if(uNumDimensions > 1)
			{
				for(u_int i=0;i<uNumDimensions;i++)
				{
					//determine icon to use from first letter of label
					const char* szLabel = m_pxGraphGadget->GetLabelForDimension((u_char)i);
					int iLetter = 0;
					if(szLabel && *szLabel)
					{
						iLetter = Asura_Maths::ClampToRange((int)(szLabel[0]-'A'),0,25);
					}
					
					Asura_Icon_Gadget* pxGadget = m_ppxFilterIcons[i]= new Asura_Icon_Gadget(xNewOffset,Tools_Dialog::GetIconTexture(),TOOLS_ICON_A+iLetter);
					pxGadget->SetCheckboxFlag(&m_pxGraphGadget->m_uVisibleDimensionFlags,1<<i);
					AddGadget(pxGadget);
				}
			}

			// Range maximum (offset to top left of graph)
			m_pxRangeMaximum = new Asura_Float_Gadget(xNewOffset, &(m_pxGraphGadget->m_fRangeMax), fMinimum, fMaximum, "%4.3g", NULL, false);
			AddGadget(m_pxRangeMaximum);
			
			// Range minimum (offset to bottom left of graph)
			m_pxRangeMinimum = new Asura_Float_Gadget(xNewOffset, &(m_pxGraphGadget->m_fRangeMin), fMinimum, fMaximum, "%4.3g", NULL, false);
			AddGadget(m_pxRangeMinimum);

			// Current value (offset to bottom right of graph)
			// Gadget
			m_pxGadgetCurrentValue = new Asura_Float_Gadget(xNewOffset, &m_fCurrentValue, fMinimum, fMaximum, "%6.4g", &m_bCurrentValueAltered, true);
			AddGadget(m_pxGadgetCurrentValue);
			m_pxGadgetCurrentValue->SetHidden(true);
			// Label
			m_pxCurrentValueLabel = new Asura_StringDisplay_Gadget(xNewOffset, 2);
			AddGadget(m_pxCurrentValueLabel);
			m_pxCurrentValueLabel->SetLabel(" :");
			m_pxCurrentValueLabel->SetHidden(true);

			// Current time (offset to left of current value)
			// Gadget
			m_pxGadgetCurrentTime = new Asura_Float_Gadget(xNewOffset, &m_fCurrentTime, m_pxGraph->GetDomainMin(), m_pxGraph->GetDomainMax(), "%3.2f", &m_bCurrentTimeAltered, true);
			AddGadget(m_pxGadgetCurrentTime);
			m_pxGadgetCurrentTime->SetHidden(true);
			// Label
			m_pxCurrentTimeLabel = new Asura_StringDisplay_Gadget(xNewOffset, 5);
			AddGadget(m_pxCurrentTimeLabel);
			m_pxCurrentTimeLabel->SetLabel("Time:");
			m_pxCurrentTimeLabel->SetHidden(true);

			// Limit scale/shift to selected point button (top right)
			m_pxLimitShiftAndScaleButton = new Asura_Icon_Gadget( xNewOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_A+'L'-'A' );
			extern bool evilg_bAsura_Graph_Gadget_LimitShiftOrScaleToCurrentPoint;
			m_pxLimitShiftAndScaleButton->SetCheckboxBool(&evilg_bAsura_Graph_Gadget_LimitShiftOrScaleToCurrentPoint);
			AddGadget( m_pxLimitShiftAndScaleButton );

			// Shift button (in right border)
			m_pxShiftButton = new Asura_Icon_Gadget( xNewOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_DOUBLEARROW );
			AddGadget( m_pxShiftButton );

			// Scale button (below shift)
			m_pxScaleButton = new Asura_Icon_Gadget( xNewOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_SCALES );
			AddGadget( m_pxScaleButton );

			// Snap left endpoint button (below scale)
			m_pxSnapLeftEndButton = new Asura_Icon_Gadget( xNewOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_GRAPH_SNAPLEFTEND );
			m_pxSnapLeftEndButton->SetClickedFunction( Asura_Graph_Gadget<T>::SnapLeftEnd, reinterpret_cast< int >( m_pxGraphGadget ) );
			AddGadget( m_pxSnapLeftEndButton );

			// Snap right endpoint button (below snap left)
			m_pxSnapRightEndButton = new Asura_Icon_Gadget( xNewOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_GRAPH_SNAPRIGHTEND );
			m_pxSnapRightEndButton->SetClickedFunction( Asura_Graph_Gadget<T>::SnapRightEnd, reinterpret_cast< int >( m_pxGraphGadget ) );
			AddGadget( m_pxSnapRightEndButton );

			// Copy
			m_pxCopyButton = new Asura_Icon_Gadget( xNewOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_COPY );
			m_pxCopyButton->SetClickedFunction( Asura_Graph_Gadget<T>::Copy, reinterpret_cast< int >( m_pxGraphGadget ) );
			AddGadget( m_pxCopyButton );

			// Paste
			m_pxPasteButton = new Asura_Icon_Gadget( xNewOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_PASTE );
			m_pxPasteButton->SetClickedFunction( Asura_Graph_Gadget<T>::Paste, reinterpret_cast< int >( m_pxGraphGadget ) );
			AddGadget( m_pxPasteButton );



			// Undo button (bottom right, in border)
			m_pxUndoButton = new Asura_Icon_Gadget( xNewOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UNDO );
			m_pxUndoButton->SetClickedFunction( Asura_Graph_Gadget<T>::UndoPoint_Static, reinterpret_cast< int >( m_pxGraphGadget ) );
			AddGadget( m_pxUndoButton );

			// Zoom (top left)
			m_pxZoomButton = new Asura_Icon_Gadget( xNewOffset, Tools_Dialog::GetIconTexture(), TOOLS_ICON_UVEDIT_ZOOMEXTENT );
			Asura_Callback_MemberFunction<void, Asura_Graph_Dialog<T>> xZoomCallback(this, &Asura_Graph_Dialog<T>::CreateGraphOverlay);
			m_pxZoomButton->SetClickedFunction( xZoomCallback);
			AddGadget(m_pxZoomButton);


			m_xMaxCoord.x = m_xOffset.x + (m_fGraphPixelWidth + 2 * ASURA_GRAPH_BORDER_SIZE)/ASURA_TEXT_OUTPUT_FONT_WIDTH;
			m_xMaxCoord.y = m_xOffset.y + (m_fGraphPixelHeight + 2 * ASURA_GRAPH_BORDER_SIZE)/ASURA_TEXT_OUTPUT_FONT_HEIGHT;


			RepositionGadgets();
			// This got really annoying
			//SetHelpText("Asura Graph Help\nCtrl + LMB: Create node\nShift + LMB: delete node\nLMB: Select node\nN: select nearest node\n\
			//	While Selected\nLMB drag: move node\nShift + arrows: move node\nCtrl + arrows: move node fast\nAlt + arrows: change node\nRMB: cancel move");

			Template_Constructor();

		}

		~Asura_Graph_Dialog()
		{
			if(m_bOpenedGraphOverlay)
			{
				//Make sure there isn't an overlay with an enlarged version of this graph still out there
				Tools_Overlay_EnlargedGraph::CloseOverlay(this);
			}

			delete [] m_ppxFilterIcons;
			Template_Destructor();
		}

		void Update(const Asura_Vector_2& xOrigin)
		{
			// Perform parent update
			Asura_Dialog::Update(xOrigin);

			if(!m_bIsEnabled) return;

			m_pxCurrentValueLabel->SetLabel(m_pxGraphGadget->GetCurrentValueLabel());

			if(m_bCurrentValueAltered || m_bCurrentTimeAltered)
			{
				m_pxGraphGadget->SetCurrentValues(m_fCurrentValue, m_fCurrentTime);
				m_bCurrentValueAltered = m_bCurrentTimeAltered = false;
			}

			if( m_bShowPointControls )
			{
				bool bNoSelection = !m_pxGraphGadget->GetCurrentValues(&m_fCurrentValue, &m_fCurrentTime);
				m_pxCurrentValueLabel->SetHidden(bNoSelection);
				m_pxGadgetCurrentValue->SetHidden(bNoSelection);
				m_pxGadgetCurrentTime->SetHidden(bNoSelection);
				m_pxCurrentTimeLabel->SetHidden(bNoSelection);
				if(m_pxColourPickerButton)
				{
					m_pxColourPickerButton->SetHidden(bNoSelection);
					if(bNoSelection && m_pxColourPicker && m_pxColourPicker->IsOverlayOpen())
					{
						m_pxColourPicker->CloseOverlay();
					}
					if(m_pxColourPickerButton->IsClickedThisFrame())
					{
						Template_OnValueClick();
					}
				}
				if(m_pxCurrentValueLabel->IsClickedThisFrame())
				{
					Template_OnValueClick();
				}
			}

			if( Asura_UserInput::KeyPressed( ASURA_KEYID_MOUSEBUTTON1 ) )
			{
				if( !m_pxGraphGadget->m_bScaling && !m_pxGraphGadget->m_bShifting )
				{
					Asura_Cursor::GetScreenSpaceCoords( m_pxGraphGadget->m_iFreezeMouseX, m_pxGraphGadget->m_iFreezeMouseY );

					if( m_pxScaleButton->IsMouseOver() )
					{
						m_pxGraphGadget->m_bScaling = true;
					}

					if( m_pxShiftButton->IsMouseOver() )
					{
						m_pxGraphGadget->m_bShifting = true;
					}
				}
			}
			else if( Asura_UserInput::KeyPressed( ASURA_KEYID_MOUSEBUTTON2 ) && m_pxGraphGadget->IsMouseOver() )
			{
				Tools_ContextMenu_System::SetRightClickUsed();
				if( !m_pxGraphGadget->m_bScaling && !m_pxGraphGadget->m_bShifting )
				{
					Asura_Cursor::GetScreenSpaceCoords( m_pxGraphGadget->m_iFreezeMouseX, m_pxGraphGadget->m_iFreezeMouseY );

					if( Asura_UserInput::KeyPressed( ASURA_KEYID_ALT ) )
					{
						m_pxGraphGadget->m_bScaling = true;
					}

					if( Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT ) )
					{
						m_pxGraphGadget->m_bShifting = true;
					}
				}
			}
			else
			{
				m_pxGraphGadget->m_bScaling = false;
				m_pxGraphGadget->m_bShifting = false;
			}

			m_pxUndoButton->SetEnabledFlag( m_bIsEnabled && m_pxGraphGadget->CanUndo(), false );
			m_pxPasteButton->SetEnabledFlag( m_bIsEnabled && m_pxGraphGadget->CanPaste(), false);
		}

		void PerFrameUpdate()
		{
			Asura_Dialog::PerFrameUpdate();
			Template_PerFrameUpdate();

			if(m_bMemberOfGraphOverlay) m_pxZoomButton->SetHidden(true);
		}

		// Per template methods. Just used for RGBA colour picker at the moment
		void Template_Constructor();
		void Template_Destructor();
		void Template_OnValueClick();
		void Template_PerFrameUpdate();

		virtual Asura_Vector_2 GetTextOffset() const
		{
			float fX = ASURA_GRAPH_BORDER_SIZE + m_fGraphPixelWidth * 0.5f;
			fX /= ASURA_TEXT_OUTPUT_FONT_WIDTH;
			fX -= Asura_Text_Output::WidthOfString( m_xName.GetString(), ASURA_TEXT_FORMAT_SMALLFONT ) * 0.5f;

			return Asura_Vector_2( fX, 0.0f );
		}

		bool GetShowPointControls() const { return m_bShowPointControls; }
		void SetShowPointControls( const bool bShow ) { m_bShowPointControls = bShow; }

		//bool GetForceNormalisedAlpha () const { return m_bForceNormalisedAlpha; }
		//void SetForceNormalisedAlpha (const bool bFlag) { m_bForceNormalisedAlpha = bFlag; }

		bool GetForceNormalisedAlpha () const { return m_bForceNormalisedAlpha; }
		void SetForceNormalisedAlpha (const bool bFlag) { m_pxGraphGadget->SetForceNormalisedAlpha (bFlag); m_bForceNormalisedAlpha = bFlag; }

		bool GetShowDiscreteSamples() const { return m_pxGraphGadget->m_bShowDiscreteSamples; }
		void SetShowDiscreteSamples( const bool bShow ) { m_pxGraphGadget->m_bShowDiscreteSamples = bShow; }

		// Set spacing values to <=0 to disable rendering

		float GetGridlineSpacingX() const { return m_pxGraphGadget->m_fGridlineSpacingX; }
		void SetGridlineSpacingX( const float fSpacing ) { m_pxGraphGadget->m_fGridlineSpacingX = fSpacing; }

		float GetGridlineSpacingY() const { return m_pxGraphGadget->m_fGridlineSpacingY; }
		void SetGridlineSpacingY( const float fSpacing ) { m_pxGraphGadget->m_fGridlineSpacingY = fSpacing; }

		//Size of the main graph gadget in pixels
		float GetGraphPixelWidth() const {return m_fGraphPixelWidth;}
		float GetGraphPixelHeight() const {return m_fGraphPixelHeight;}


		virtual void StartResize(u_int uResizeFlags)
		{
			m_fPreResizeWidth = m_fGraphPixelWidth;
			m_fPreResizeHeight = m_fGraphPixelHeight;
		}
		
		virtual void UpdateResize(u_int uResizeFlags, const Asura_Vector_2& xDelta)
		{
			if(!m_bAllowResize) return;

			float fWidth,fHeight;

			if(uResizeFlags & ASURA_OVERLAY_RESIZE_INITIAL)
			{
				//initialise the size based on the last size chosen by the user
				if(!m_pxUserSetSize) return;
				if(*m_pxUserSetSize == Asura_Maths::ZeroVector2) return;

				fWidth = Asura_Maths::Max(ASURA_GRAPH_WIDTH, m_pxUserSetSize->x);
				fHeight = Asura_Maths::Max(ASURA_GRAPH_HEIGHT, m_pxUserSetSize->y);
			}
			else
			{
				
				fWidth = Asura_Maths::Max(ASURA_GRAPH_WIDTH, m_fPreResizeWidth + xDelta.x*ASURA_TEXT_OUTPUT_FONT_WIDTH);
				fHeight = Asura_Maths::Max(ASURA_GRAPH_HEIGHT, m_fPreResizeHeight + xDelta.y*ASURA_TEXT_OUTPUT_FONT_HEIGHT);

				fWidth = (float)Asura_Maths::FloatToInt_Near(fWidth);
				fHeight = (float)Asura_Maths::FloatToInt_Near(fHeight);

				if(m_pxUserSetSize)
				{
					m_pxUserSetSize->x = fWidth;
					m_pxUserSetSize->y = fHeight;
				}
			}

			if(fWidth != m_fGraphPixelWidth || fHeight != m_fGraphPixelHeight)
			{
				m_fGraphPixelWidth = fWidth;
				m_fGraphPixelHeight = fHeight;
				RepositionGadgets();
			}
		}

		virtual u_int GetAllowedResizeFlags() const
		{
			return m_bAllowResize ? ASURA_OVERLAY_RESIZE_HORIZONTAL | ASURA_OVERLAY_RESIZE_VERTICAL : 0;
		}

		

	protected:
		virtual void ShowHelpText()
		{
			if( m_pxShiftButton && m_pxShiftButton->IsMouseOver() )
			{
				Asura_Overlay_System::SetHelpText( "Click and hold here, then drag up and down, to shift the graph values." );
			}
			else if( m_pxScaleButton && m_pxScaleButton->IsMouseOver() )
			{
				Asura_Overlay_System::SetHelpText( "Click and hold here, then drag up and down, to scale the graph values." );
			}
			else if( m_pxLimitShiftAndScaleButton && m_pxLimitShiftAndScaleButton->IsMouseOver() )
			{
				Asura_Overlay_System::SetHelpText( "Limits shifting and scaling to affecting just the components of the currently selected graph point." );
			}
			else if( m_pxUndoButton && m_pxUndoButton->IsMouseOver() )
			{
				Asura_Overlay_System::SetHelpText( "Undoes the last point drag." );
			}
			else if( m_pxSnapLeftEndButton && m_pxSnapLeftEndButton->IsMouseOver() )
			{
				Asura_Overlay_System::SetHelpText( "Snaps the left endpoint level with the second point." );
			}
			else if( m_pxSnapRightEndButton && m_pxSnapRightEndButton->IsMouseOver() )
			{
				Asura_Overlay_System::SetHelpText( "Snaps the right endpoint level with the second to last point." );
			}
			else if( m_pxCopyButton && m_pxCopyButton->IsMouseOver() )
			{
				Asura_Overlay_System::SetHelpText( "Copies the graph points to the paste buffer." );
			}
			else if( m_pxPasteButton && m_pxPasteButton->IsMouseOver() )
			{
				Asura_Overlay_System::SetHelpText( "Pastes the most recently copied graph points.\nThis must be from a graph of the same type with the same clamping values." );
			}
			else if( m_pxZoomButton && m_pxZoomButton->IsMouseOver() )
			{
				Asura_Overlay_System::SetHelpText( "Creates an enlarged version of this graph in a separate overlay." );
			}
			else
			{
				Asura_Overlay_System::SetHelpText( GetHelpText() );
			}
		}

		void RepositionGadgets()
		{
			Asura_Vector_2 xNewOffset = m_xOffset;

			// graph gadget
			m_pxGraphGadget->m_xOffset = m_xOffset;
			m_pxGraphGadget->m_fWidth = (m_fGraphPixelWidth + ASURA_GRAPH_BORDER_SIZE * 2) / 10.0f;
			m_pxGraphGadget->m_fHeight = (m_fGraphPixelHeight + ASURA_GRAPH_BORDER_SIZE * 2) / 14.0f;

			// Graph dimension filter buttons (top right)
			u_int uNumDimensions = m_pxGraphGadget->GetNumberOfDimensions();
			if(uNumDimensions > 1)
			{
				xNewOffset = m_xOffset;
				xNewOffset.x += (ASURA_GRAPH_BORDER_SIZE + m_fGraphPixelWidth)/ASURA_TEXT_OUTPUT_FONT_WIDTH
							  - Asura_Icon_Gadget::s_fWidth * uNumDimensions - 0.5f;

				for(u_int i=0;i<uNumDimensions;i++)
				{
					m_ppxFilterIcons[i]->m_xOffset = xNewOffset;					
					xNewOffset.x += Asura_Icon_Gadget::s_fWidth;
				}
			}

			// Range maximum (offset to top left of graph)
			xNewOffset = m_xOffset;
			xNewOffset.x += ASURA_GRAPH_BORDER_SIZE/ASURA_TEXT_OUTPUT_FONT_WIDTH + 0.2f;
			xNewOffset.y += ASURA_GRAPH_BORDER_SIZE/ASURA_TEXT_OUTPUT_FONT_HEIGHT - 1.0f;
			m_pxRangeMaximum->m_xOffset = xNewOffset;
			
			// Range minimum (offset to bottom left of graph)
			xNewOffset = m_xOffset;
			xNewOffset.x += ASURA_GRAPH_BORDER_SIZE/ASURA_TEXT_OUTPUT_FONT_WIDTH + 0.2f;
			xNewOffset.y += (m_fGraphPixelHeight + ASURA_GRAPH_BORDER_SIZE)/ASURA_TEXT_OUTPUT_FONT_HEIGHT;
			m_pxRangeMinimum->m_xOffset = xNewOffset;

			// Current value (offset to bottom right of graph)
			xNewOffset = m_xOffset;
			xNewOffset.x += (m_fGraphPixelWidth + ASURA_GRAPH_BORDER_SIZE)/ASURA_TEXT_OUTPUT_FONT_WIDTH;
			xNewOffset.y += (m_fGraphPixelHeight + ASURA_GRAPH_BORDER_SIZE)/ASURA_TEXT_OUTPUT_FONT_HEIGHT;
			// Gadget
			m_pxGadgetCurrentValue->m_xOffset = xNewOffset;
			m_pxGadgetCurrentValue->m_xOffset.x -= m_pxGadgetCurrentValue->m_fWidth;
			// Label
			xNewOffset.x -= m_pxGadgetCurrentValue->m_fWidth + 2.0f;
			m_pxCurrentValueLabel->m_xOffset = xNewOffset;

			// Current time (offset to left of current value)
			xNewOffset.x -= 1.0f;
			// Gadget
			m_pxGadgetCurrentTime->m_xOffset = xNewOffset;
			m_pxGadgetCurrentTime->m_xOffset.x -= m_pxGadgetCurrentTime->m_fWidth;
			// Label
			xNewOffset.x -= m_pxGadgetCurrentTime->m_fWidth + 5.0f;
			m_pxCurrentTimeLabel->m_xOffset = xNewOffset;

			// Limit scale/shift to selected point button (top right)
			xNewOffset = m_xOffset;
			xNewOffset.x += (ASURA_GRAPH_BORDER_SIZE * 1.5f + m_fGraphPixelWidth) / ASURA_TEXT_OUTPUT_FONT_WIDTH - Asura_Icon_Gadget::s_fWidth * 0.5f;
			m_pxLimitShiftAndScaleButton->m_xOffset = xNewOffset;

			// Shift button (in right border)
			xNewOffset = m_xOffset;
			xNewOffset.x += (ASURA_GRAPH_BORDER_SIZE * 1.5f + m_fGraphPixelWidth) / ASURA_TEXT_OUTPUT_FONT_WIDTH - Asura_Icon_Gadget::s_fWidth * 0.5f;
			xNewOffset.y += ASURA_GRAPH_BORDER_SIZE / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
			m_pxShiftButton->m_xOffset = xNewOffset;

			// Scale button (below shift)
			xNewOffset.y += Asura_Icon_Gadget::s_fHeight + 0.2f;
			m_pxScaleButton->m_xOffset = xNewOffset;

			// Snap left endpoint button (below scale)
			xNewOffset.y += Asura_Icon_Gadget::s_fHeight + 0.2f;
			m_pxSnapLeftEndButton->m_xOffset = xNewOffset;

			// Snap right endpoint button (below snap left)
			xNewOffset.y += Asura_Icon_Gadget::s_fHeight + 0.2f;
			m_pxSnapRightEndButton->m_xOffset = xNewOffset;

			// Copy
			xNewOffset.y += Asura_Icon_Gadget::s_fHeight + 0.4f;
			m_pxCopyButton->m_xOffset = xNewOffset;

			// Paste
			xNewOffset.y += Asura_Icon_Gadget::s_fHeight + 0.2f;
			m_pxPasteButton->m_xOffset = xNewOffset;

			// Undo button (bottom right, in border)
			xNewOffset.y = m_xOffset.y + (ASURA_GRAPH_BORDER_SIZE + m_fGraphPixelHeight) / ASURA_TEXT_OUTPUT_FONT_HEIGHT - Asura_Icon_Gadget::s_fHeight;
			m_pxUndoButton->m_xOffset = xNewOffset;

			// Zoom (top left)
			m_pxZoomButton->m_xOffset = m_xOffset;

			// Colour picker button (left of the time label)
			if(m_pxColourPickerButton)
			{
				m_pxColourPickerButton->m_xOffset = m_pxCurrentTimeLabel->m_xOffset;
				m_pxColourPickerButton->m_xOffset.x -= m_pxColourPickerButton->m_fWidth + 0.5f;
			
			}


			m_xMaxCoord.x = m_xOffset.x + (m_fGraphPixelWidth + 2 * ASURA_GRAPH_BORDER_SIZE)/ASURA_TEXT_OUTPUT_FONT_WIDTH;
			m_xMaxCoord.y = m_xOffset.y + (m_fGraphPixelHeight + 2 * ASURA_GRAPH_BORDER_SIZE)/ASURA_TEXT_OUTPUT_FONT_HEIGHT;
		}

		void CreateGraphOverlay()
		{
			//Don't allow a graph that has been made for a graph overlay to produce another one
			if(m_bMemberOfGraphOverlay) return;

			//Make a copy of this graph dialog for use in the overlay
			Asura_Graph_Dialog<T>* pxDialogCopy = new Asura_Graph_Dialog<T>(GetName().GetString(), Asura_Vector_2(1.0f,3.0f), m_pxGraph, m_fRangeMin, m_fRangeMax, m_pbChangedFlag);
			pxDialogCopy->m_bMemberOfGraphOverlay = true;
			pxDialogCopy->SetShowPointControls( GetShowPointControls());
			pxDialogCopy->SetForceNormalisedAlpha( GetForceNormalisedAlpha());
			pxDialogCopy->SetShowDiscreteSamples( GetShowDiscreteSamples());
			pxDialogCopy->SetGridlineSpacingX( GetGridlineSpacingX());
			pxDialogCopy->SetGridlineSpacingY( GetGridlineSpacingY());
 
			//and create the overlay
			Tools_Overlay_EnlargedGraph::Create(this, pxDialogCopy);
			m_bOpenedGraphOverlay = true;
		}

//	private:
	public:
		Asura_Graph_Gadget<T>*		m_pxGraphGadget;
		Asura_Graph<T>*				m_pxGraph;
		float						m_fRangeMin;
		float						m_fRangeMax;
		float						m_fCurrentValue;
		float						m_fCurrentTime;
		bool						m_bCurrentValueAltered;
		bool						m_bCurrentTimeAltered;
		Asura_Icon_Gadget**			m_ppxFilterIcons;
		Asura_Float_Gadget*			m_pxRangeMaximum;
		Asura_Float_Gadget*			m_pxRangeMinimum;
		Asura_StringDisplay_Gadget*	m_pxCurrentValueLabel;
		Asura_StringDisplay_Gadget*	m_pxCurrentTimeLabel;
		Asura_Gadget*				m_pxGadgetCurrentValue;
		Asura_Gadget*				m_pxGadgetCurrentTime;
		Asura_Gadget*				m_pxColourPickerButton;
		Asura_Gadget*				m_pxShiftButton;
		Asura_Gadget*				m_pxScaleButton;
		Asura_Icon_Gadget*			m_pxLimitShiftAndScaleButton;
		Asura_Icon_Gadget*			m_pxUndoButton;
		Asura_Icon_Gadget*			m_pxSnapLeftEndButton;
		Asura_Icon_Gadget*			m_pxSnapRightEndButton;
		Asura_Icon_Gadget*			m_pxCopyButton;
		Asura_Icon_Gadget*			m_pxPasteButton;
		Asura_Icon_Gadget*			m_pxZoomButton;
		// Only used for Asura_RGBA
		Tools_ColourPicker*			m_pxColourPicker;
		float						m_fR;
		float						m_fG;
		float						m_fB;
		bool						m_bColourChanged;
		bool						m_bApplyColourChangeToAllNodes;
		bool						m_bShowPointControls;
		bool						m_bForceNormalisedAlpha;
		bool*						m_pbChangedFlag;

		//width/height of the main graph gadget in pixels
		float						m_fGraphPixelWidth;
		float						m_fGraphPixelHeight;
		float						m_fPreResizeWidth;
		float						m_fPreResizeHeight;

		bool						m_bOpenedGraphOverlay;//true if this dialog has opened an enlarged graph overlay(which may or may not still be open)
		bool						m_bMemberOfGraphOverlay;//true if this has been created to be part of an enlarged graph overlay

};


#endif//__ASURA_GRAPH_DIALOG_H__
