//
// Asura_Menubar.cpp
//
// Created by Kevin Lea - 19/06/2001 12:38:38
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_TOOLS

#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Overlay_System.h"
#include "Core/Asura_UserInput.h"
#include "Rendering/Asura_Text_Output.h"

void Asura_Menubar_Item::Update(Asura_Vector_2 offset, float width)
{
	if( m_pfnIsEnabled )
	{
		m_bIsEnabled = m_pfnIsEnabled();
	}
	else if(m_pfnIsEnabledWithUserData)
	{
		m_bIsEnabled	= m_pfnIsEnabledWithUserData(m_pUserData);
	}

	if( m_bIsEnabled )
	{
		if ((Asura_Menubar_System::CursorX>=offset.x)
			&&(Asura_Menubar_System::CursorX<offset.x+width)
			&&(Asura_Menubar_System::CursorY>=offset.y)
			&&(Asura_Menubar_System::CursorY<offset.y+1.5f))
		{
			Selected=true;
		}
		else 
		{
			Selected=false;
		}

		if( Selected && 
			(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) || Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON2)) )
		{
			if(SelectFunction)
			{
				SelectFunction();
			}
			else if(SelectFunctionWithUserData)
			{
				SelectFunctionWithUserData(m_pUserData);
			}
		}
	}

	offset.y+=1.5f;
	if (NextItem) NextItem->Update(offset,width);
}
void Asura_Menubar_Item::Render(Asura_Vector_2 offset, float width)
{
	unsigned int colour = ( m_bIsEnabled ? 0xffffffff : 0xFF555555u );

	int fontflags = 0;
	if ( Asura_Overlay_System::UseSkinnedUI() )
	{
		fontflags = ASURA_TEXT_FORMAT_SMALLFONT;
	}	

	if (!Selected)
	{	
		colour >>= 1;
	}

	// Kevin Lea - 20/06/2001 11:49:09 - check to see if a function
	// has been provided to tell if a tick should be drawn
	if( (HasATick && HasATick()) ||
		(HasATickWithUserData && HasATickWithUserData(m_pUserData)) )
	{
		Asura_Text_Output::Print(offset.x+0.5f,offset.y,colour,ASURA_TEXT_FORMAT_LEFTJUSTIFIED,"%c",ASURA_CHAR_TICK);
	}
	Asura_Text_Output::Print(offset.x+2.0f,offset.y,colour,fontflags,Name.GetString());

	offset.y+=1.5f;
	if (NextItem)
	{	NextItem->Render(offset,width);}
}

void Asura_Menubar_Title::Update(Asura_Vector_2 offset)
{
	if ((Asura_Menubar_System::CursorX>=offset.x)
		&&(Asura_Menubar_System::CursorX<offset.x+WidthOfTitle)
		&&(Asura_Menubar_System::CursorY>=offset.y)
		&&(Asura_Menubar_System::CursorY<offset.y+1.5f))
	{
		Selected=true;
		Asura_Menubar_System::TitleIsActive(this);
	}
	else if(Asura_Menubar_System::AreAnyOtherTitlesActive(this))
	{
		Selected=false;
	}

	if(!AreWidthsCalculated) CalculateWidths();

	offset.y+=1.5f;
	if (Selected && FirstItem) FirstItem->Update(offset,WidthOfItems);
	offset.y-=1.5f;

	offset.x += WidthOfTitle;
	if (NextTitle) NextTitle->Update(offset);
}

void Asura_Menubar_Title::Render(Asura_Vector_2 offset)
{
	unsigned int colour;

	if( Asura_Overlay_System::UseSkinnedUI() )
	{	
		if (Selected)
		{
			colour= 0xffffffff;
		}
		else
		{
			colour= 0xffbfbfbf;
		}

		Asura_Text_Output::Print(offset.x+1.1f,offset.y+0.1f,0xff000000,ASURA_TEXT_FORMAT_SMALLFONT,Name.GetString());
		Asura_Text_Output::Print(offset.x+1.0f,offset.y,colour,ASURA_TEXT_FORMAT_SMALLFONT,Name.GetString());
	}
	else
	{
		if (Selected)
		{
			colour= 0xffffffff;
		}
		else
		{
			colour= 0x7fffff00;
		}

		Asura_Text_Output::Print(offset.x+1.0f,offset.y,colour,0,Name.GetString());
	}


	if (Selected && FirstItem)
	{
		offset.y+=1.5f;

		Asura_Menubar_System::RenderBox(offset,WidthOfItems,DepthOfItems);


		FirstItem->Render(offset,WidthOfItems);
		offset.y-=1.5f;
	}

	offset.x += WidthOfTitle;
	if (NextTitle) NextTitle->Render(offset);
}

void Asura_Menubar_Title::CalculateWidths(void)
{
	Asura_Menubar_Item* item = FirstItem;

	WidthOfItems = 0;
	DepthOfItems = 0.5f;

	while(item)
	{
		float length = 0.0f;

		if( Asura_Overlay_System::UseSkinnedUI() )
		{
			length = (float)Asura_Text_Output::WidthOfString(item->Name.GetString(), ASURA_TEXT_FORMAT_SMALLFONT);
		}
		else
		{
			length = (float)Asura_Text_Output::LengthOfString(item->Name.GetString());
		}
		

		if (length>WidthOfItems)
		{
			WidthOfItems = length;
		}

		item = item->NextItem;
		DepthOfItems+=1.5f;
	}

	WidthOfItems+=3.0f;

	if( Asura_Overlay_System::UseSkinnedUI() )
	{
		WidthOfTitle = (float)(Asura_Text_Output::WidthOfString(Name.GetString(), ASURA_TEXT_FORMAT_SMALLFONT))+2.0f;
	}
	else
	{
		WidthOfTitle = (float)(Asura_Text_Output::LengthOfString(Name.GetString())+2.0f);
	}

	AreWidthsCalculated = true;
}

void Asura_Menubar_Title::AddItem( const Asura_Char* szName, void(*pfnAction)(void), bool(*pfnHasTick)(void), bool(*pfnIsEnabled)(void) )
{
	if(m_bNeedSeparator)
	{
		m_bNeedSeparator = false;
		//Insert a blank item first
		//(Need to disable sorting while doing this)
		//RAR 26/01/2006
		bool bSortItems = m_bSortItems;
		StopSort();
		AddItem("");
		if(bSortItems) StartSort();
	}

	//allocate the new menu item
	Asura_Menubar_Item* pxItem = new Asura_Menubar_Item;
	pxItem->Name = szName;
	pxItem->SelectFunction = pfnAction;
	pxItem->HasATick = pfnHasTick;
	pxItem->m_pfnIsEnabled = pfnIsEnabled;


	Asura_Menubar_Item** ppxInsertionPoint = &FirstItem;

	if(m_bSortItems && m_ppxSortStart)
	{
		//if sorting part of list , start part of the way down
		ppxInsertionPoint = m_ppxSortStart;
	}

	//find the end of the item list , or appropriate insertion point for sorting
	while(*ppxInsertionPoint)
	{
		if(m_bSortItems && ((*ppxInsertionPoint)->Name.Compare(pxItem->Name)>0))
		{
			break;
		}
		ppxInsertionPoint = &((*ppxInsertionPoint)->NextItem);
	}

	pxItem->NextItem = *ppxInsertionPoint;
	*ppxInsertionPoint = pxItem;


	//will need to recalculate the size
	AreWidthsCalculated = false;
}

void Asura_Menubar_Title::AddItemWithUserData(const Asura_Char* szName,
											  const void* pUserData,
											  void(*pfnAction)(const void* pUserData),
											  bool(*pfnHasTick)(const void* pUserData),
											  bool(*pfnIsEnabled)(const void* pUserData))
{
	if(m_bNeedSeparator)
	{
		m_bNeedSeparator = false;
		//Insert a blank item first
		//(Need to disable sorting while doing this)
		//RAR 26/01/2006
		bool bSortItems = m_bSortItems;
		StopSort();
		AddItem("");
		if(bSortItems) StartSort();
	}

	//allocate the new menu item
	Asura_Menubar_Item* pxItem = new Asura_Menubar_Item;
	pxItem->Name = szName;
	pxItem->m_pUserData = pUserData;
	pxItem->SelectFunctionWithUserData = pfnAction;
	pxItem->HasATickWithUserData = pfnHasTick;
	pxItem->m_pfnIsEnabledWithUserData = pfnIsEnabled;


	Asura_Menubar_Item** ppxInsertionPoint = &FirstItem;

	if(m_bSortItems && m_ppxSortStart)
	{
		//if sorting part of list , start part of the way down
		ppxInsertionPoint = m_ppxSortStart;
	}

	//find the end of the item list , or appropriate insertion point for sorting
	while(*ppxInsertionPoint)
	{
		if(m_bSortItems && ((*ppxInsertionPoint)->Name.Compare(pxItem->Name)>0))
		{
			break;
		}
		ppxInsertionPoint = &((*ppxInsertionPoint)->NextItem);
	}

	pxItem->NextItem = *ppxInsertionPoint;
	*ppxInsertionPoint = pxItem;


	//will need to recalculate the size
	AreWidthsCalculated = false;
}

void Asura_Menubar_Title::StartSort()
{
	m_bSortItems = true;

	//store current insertion point ,since we only want to sort new entries
	m_ppxSortStart = &FirstItem;
	while(*m_ppxSortStart)
	{
		m_ppxSortStart = &((*m_ppxSortStart)->NextItem);
	}
}

void Asura_Menubar_Title::StopSort()
{
	m_bSortItems = false;
	m_ppxSortStart = 0;
}


void Asura_Menubar::Update(Asura_Vector_2 offset)
{

	if (FirstTitle) FirstTitle->Update(offset);
}
void Asura_Menubar::Render(Asura_Vector_2 offset)
{

	if (FirstTitle) FirstTitle->Render(offset);
}

#endif // #ifdef ASURA_TOOLS
