//
// Asura_Menubar.h
//
// Created by Kevin Lea - 19/06/2001 12:37:55
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_MENUBAR_H__
#define __ASURA_MENUBAR_H__

#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"


class Asura_Menubar_Item
{
	public:

		Asura_Menubar_Item()
		{
			NextItem = NULL;
			HasATick = NULL;
			SelectFunction = NULL;
            m_pfnIsEnabled = NULL;
			HasATickWithUserData = NULL;
			SelectFunctionWithUserData = NULL;
			m_pfnIsEnabledWithUserData = NULL;
            m_bIsEnabled = true;
			m_pUserData = NULL;
		}
		~Asura_Menubar_Item()
		{
			if (NextItem) delete NextItem;
		}

		Asura_String Name;
		bool Selected;
        bool m_bIsEnabled;
		const void*	m_pUserData;
	
		void Update(Asura_Vector_2 offset, float width);
		void Render(Asura_Vector_2 offset, float width);

		void (*SelectFunction)(void);
		bool (*HasATick)(void);
        bool (*m_pfnIsEnabled)(void);

		void (*SelectFunctionWithUserData)(const void* pUserData);
		bool (*HasATickWithUserData)(const void* pUserData);
		bool (*m_pfnIsEnabledWithUserData)(const void* pUserData);

		Asura_Menubar_Item* NextItem;
};

class Asura_Menubar_Title
{
	public:

		Asura_Menubar_Title()
		{
			FirstItem = 0;
			NextTitle = 0;
			AreWidthsCalculated = false;

			m_bSortItems = false;
			m_ppxSortStart = 0;
			m_bNeedSeparator = false;
		}

		~Asura_Menubar_Title()
		{
			if (FirstItem) delete FirstItem;
			if (NextTitle) delete NextTitle;
		}

		void AddItem( const Asura_Char* name, void(*action)(void) = 0, bool(*hasTick)(void) = 0, bool(*IsEnabled)(void) = NULL );
		void AddItemWithUserData( const Asura_Char* szName, const void* pUserData, void(*pfnAction)(const void* pUserData) = NULL, bool(*pfnHasTick)(const void* pUserData) = NULL, bool(*pfnIsEnabled)(const void* pUserData) = NULL );

		//causes subsequent items to be sorted alphabetically
		void StartSort();
		void StopSort();

		//Causes a separator to be added before the next item to be added(if any)
		void AddSeparator(){m_bNeedSeparator = true;}

		
		Asura_String Name;
		bool Selected;
		float WidthOfTitle;
		float WidthOfItems;
		float DepthOfItems;


		void Update(Asura_Vector_2 offset);
		void Render(Asura_Vector_2 offset);

		bool AreWidthsCalculated;
		void CalculateWidths(void);

		Asura_Menubar_Item* FirstItem;
		
		Asura_Menubar_Title* NextTitle;

	protected :
		bool m_bNeedSeparator;
		bool m_bSortItems;
		Asura_Menubar_Item** m_ppxSortStart;

};


class Asura_Menubar
{
	public:

		Asura_Menubar()
		{
			FirstTitle = 0;
		}
		~Asura_Menubar()
		{
			if (FirstTitle) delete FirstTitle;
		}

		void Update(Asura_Vector_2 offset);
		void Render(Asura_Vector_2 offset);

		Asura_Menubar_Title* FirstTitle;
};

#endif//__ASURA_MENUBAR_H__
