//
// Asura_MultiLineString_Dialog.h
//
// Created by Richard Rice - 04/01/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_MULTILINESTRING_DIALOG_H__
#define __ASURA_MULTILINESTRING_DIALOG_H__

#include "Core/Asura_String.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"


class Asura_MultiLineString_Gadget;

//////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Asura_MultiLineString_Dialog **
**------------------------------*/

class Asura_MultiLineString_Dialog : public Asura_Dialog
{
public :
	Asura_MultiLineString_Dialog(const Asura_Vector_2& xOffset, class Asura_String* pxString, float fWidth, float fHeight, bool* pbChangedFlag);

	float GetVerticalScrollPos() const {return m_fVScrollPos;}

protected :
	virtual void Render( const Asura_Vector_2& xOrigin );
	virtual void PerFrameUpdate();

private :
	void UpdateScrollbar();

	Asura_MultiLineString_Gadget* m_pxStringGadget;	
	Asura_Scrollbar_Gadget* m_pxVerticalBar;

	float m_fVScrollPos;

	//width/height of main string gadget
	float m_fViewWidth;
	float m_fViewHeight;
};

//////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Asura_MultiLineString_Gadget **
**------------------------------*/

class Asura_MultiLineString_Gadget : public Asura_Gadget
{
public  :
	Asura_MultiLineString_Gadget(const Asura_Vector_2& xOffset, const Asura_MultiLineString_Dialog* pxStringDialog,Asura_String* pxString, float fWidth, float fHeight, bool* pbChangedFlag);
	~Asura_MultiLineString_Gadget();

	void SetReadOnly(bool bReadOnly){m_bReadOnly = bReadOnly;}
	void SetDefaultFocus(bool bSet) {m_bDefaultFocus = true;}


	float GetTextPixelHeight() const {return m_fTextHeight;}

	virtual void FocusGained(int iFocusType);
	virtual void FocusLost(int iFocusType);

	virtual void TakeTabFocus();
	virtual bool CanTakeTabFocus() const {return !m_bReadOnly;}

protected :
	virtual void Update( const Asura_Vector_2& xOrigin );
	virtual void Render( const Asura_Vector_2& xOrigin );

private :
	void HandleKeyboardInput();
	void FillTextBuffer();
	bool ReadFromTextBuffer();

	//helper functions while editing text
	void DeleteSelection();
	bool IsTextSelected() const;
	void CopyText() const;
	void PasteText();

	//Does a check for a debounced key , but allows for it to be repeated if held long enough
	bool IsRepeatableKeyPressed(int iKey);

	void InsertCharacter(Asura_UnicodeChar wKeyPressed);

	void FindSplitPoints();
	void GetCharacterPosition(int iIndex, bool bRight, float& fX, int& iLine);
	int GetIndexFromPosition(int iLine, float fX);

		
	Asura_UnicodeChar* m_wszTextBuffer;
	int m_iTextBufferSize;
	int m_iCursorPos;
	int m_iSelectionStart;//marks start of selected text , which goes up to the cursor position
	bool m_bDefaultFocus;//determines whether this control should autoamtically get keyboard focus
	bool m_bDiscardChanges;
	bool m_bReadOnly;

	//For simulating a key repeat rate while a key is pressed down
	float m_fRepeatTimer;
	int m_iRepeatKey;


	const Asura_MultiLineString_Dialog* m_pxStringDialog;

	Asura_String* m_pxString;
	bool* m_pbChangedFlag;

	struct LineData
	{
		int m_iStart;
		int m_iEnd;
		float m_fWidth;
	};

	int m_iNumLines;
	int m_iLineArraySize;
	LineData* m_pxLines;
	float m_fTextHeight;
	

};

#endif //__ASURA_MULTILINESTRING_DIALOG_H__
