//
// Asura_Overlay_System.cpp
//
// Created by Kevin Lea - 12/06/2001 11:11:21
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_TOOLS

#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Font_System.h"
#include "Core/Asura_SortAndSearch.h"
#include "Core/Asura_UserInput.h"
#include "Rendering/Asura_Render.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Gadget.h"
#include "Debugging/Asura_Assert.h"
#include "Rendering/Asura_DisplayLimits.h"

#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"

unsigned int Asura_OverlayBrightness;

Asura_Overlay* Asura_Overlay_System::OverlayList[];
int Asura_Overlay_System::NumberOfActiveOverlays;
float Asura_Overlay_System::MaxOverlayY;
float Asura_Overlay_System::PanelMinY;
bool Asura_Overlay_System::CursorIsOnAnOverlay;
Asura_Gadget* Asura_Overlay_System::FocusGadget = 0;
Asura_Overlay* Asura_Overlay_System::CurrentOverlay = 0;
bool Asura_Overlay_System::HasKeyboardFocus;
int Asura_Overlay_System::FocusType;
bool Asura_Overlay_System::s_bFocusSetThisFrame;
bool Asura_Overlay_System::s_bUsedKeypressThisFrame = false;
u_int Asura_Overlay_System::s_uOverlayColour = 0x007f7f;
u_int Asura_Overlay_System::s_uFadedOverlayAlpha = 127;

Asura_String Asura_Overlay_System::s_xHelpText;

bool Asura_Overlay_System::s_bCodeHideOverlays = false;
bool Asura_Overlay_System::s_bUserHideOverlays = false;
int Asura_Overlay_System::s_iHideOverlaysNestLevel = 0;
bool Asura_Overlay_System::s_bUseSkinnedUI = true;
bool Asura_Overlay_System::s_bLastMouseClickWasOnOverlay = false;

u_int Asura_Overlay_System::s_uNextSubmissionIndex = 1;

Asura_Callback_Wrapper<void> Asura_Overlay_System::s_xCallbackFunction;

#define OVERLAY_HOMING_SPEED 300.0f

#include "Core/Asura_TextureManagement.h"

int MenuBackdropTextureIndex = ASURA_TEXTURE_NONE;
int MenuSkinTextureIndex = ASURA_TEXTURE_NONE;

#define MENUBACKDROPTEXTURENAME "\\tools\\fog.bmp"
#define MENUSKINTEXTURENAME		"\\tools\\toolskin.tga" 


#include "Rendering/Asura_Viewport.h"

static Asura_Overlay*            g_pxCtrlTabOverlay         = NULL;
static Asura_CtrlTabList_Dialog* g_pxCtrlTabDialog          = NULL;
static bool                      g_bDisplayCtrlTabOverlay   = false;
static u_int                     g_uCtrlTabPosInOverlayList = 0;

static bool                      g_bDoingMainUpdate			= false;


void Asura_Overlay_System::CreateCtrlTabOverlay( void )
{
	Asura_Assert( !g_pxCtrlTabOverlay, "" );
	Asura_Assert( !g_pxCtrlTabDialog,  "" );

	g_pxCtrlTabOverlay = new Asura_Overlay( "Ctrl-tab overlay", true );
	g_pxCtrlTabOverlay->m_bHasWindowsDialog = false;

	g_pxCtrlTabDialog = new Asura_CtrlTabList_Dialog( 3 );
	g_pxCtrlTabOverlay->AddDialog( g_pxCtrlTabDialog );

	g_bDisplayCtrlTabOverlay   = false;
	g_uCtrlTabPosInOverlayList = 0;
}

void Asura_Overlay_System::Initialise(void)
{
	NumberOfActiveOverlays=0;
	for(int i=0; i<MAX_NUMBER_OF_OVERLAYS; i++)
	{
		OverlayList[i] = 0;
	}

	CreateCtrlTabOverlay();

	MenuBackdropTextureIndex = Asura_Texture_Management::AddTextureWithResetGroup(MENUBACKDROPTEXTURENAME, ASURA_TEXTURE_LOADED_PERMANENT);

	if( Asura_Overlay_System::UseSkinnedUI() )
	{
		LoadSkinnedUITexture();
	}
}

void Asura_Overlay_System::Shutdown(void)
{
	ReleaseFocus();
	CurrentOverlay = NULL;
	NumberOfActiveOverlays=0;
	for(int i=0; i<MAX_NUMBER_OF_OVERLAYS; i++)
	{
		if (OverlayList[i]) delete OverlayList[i];
		OverlayList[i] = 0;
	}

	if( g_pxCtrlTabOverlay )
	{
		delete g_pxCtrlTabOverlay;
		g_pxCtrlTabOverlay = NULL;
	}
	g_pxCtrlTabDialog = NULL;
}

void Asura_Overlay_System::SubmitNewOverlay( Asura_Overlay* overlay, bool hasCloseGadget, bool hasMinimiseGadget, bool bDisplayName )
{
	if(NumberOfActiveOverlays>=MAX_NUMBER_OF_OVERLAYS)
	{
		CheckForClosingOverlays();
		if(NumberOfActiveOverlays>=MAX_NUMBER_OF_OVERLAYS)
		{
			//no room for anymore overlays
			Asura_Assert(0,"Too many overlays open");
			return;	
		}

	}

	overlay->CalculateOverlaySize();

	if(!overlay->m_pxPositionBackup)
	{
		// Open all new overlays, by default, centered on the primary monitor
		Asura_Bounding_Rectangle xBounds = Asura_DisplayLimits::Platform_GetPrimaryDisplayBounds();
		xBounds.MinX /= ASURA_TEXT_OUTPUT_FONT_WIDTH; xBounds.MaxX /= ASURA_TEXT_OUTPUT_FONT_WIDTH;
		xBounds.MinY /= ASURA_TEXT_OUTPUT_FONT_HEIGHT; xBounds.MaxY /= ASURA_TEXT_OUTPUT_FONT_HEIGHT;
		xBounds.MinY += 2.0f;
		overlay->m_xPosition.x = (xBounds.MaxX+xBounds.MinX - overlay->m_fWidth)/2.0f;
		overlay->m_xPosition.y = (xBounds.MaxY+xBounds.MinY - overlay->m_fFullHeight)/2.0f;
	}

#if 0
	if(overlay->OnPanel)
	{
		overlay->Origin = overlay->Position;
	}
#else
	//Kill of the swooshing. RAR 05/03/03
	overlay->m_xOrigin = overlay->m_xPosition;
#endif

	if(overlay->IsMinimisedAtBottom())
	{
		//if the overlay is starting minimised , set its origin to the corner , so it
		//doesn't 'swoop' down the screen immediately
		overlay->m_xOrigin = Asura_Vector_2(0,MaxOverlayY);
	}

	{
		bool* pbShouldClose = NULL;
		u_int* puMinimisedState = NULL;

		if (!overlay->m_bAlwaysOnTop && !overlay->m_bOnPanel)
		{
			if (hasCloseGadget)	pbShouldClose = &overlay->m_bClosedByCloseGadget;
			if (hasMinimiseGadget) puMinimisedState = &overlay->m_uMinimisedState;
		}

		if( bDisplayName || pbShouldClose || puMinimisedState )
		{
			const Asura_Vector_2 xOffset( 0.5f, 0.5f );
			overlay->AddDialog( new Asura_Windows_Dialog(overlay->m_xName.GetString(), xOffset, overlay->m_fWidth, pbShouldClose, puMinimisedState) );
		}
		else
		{
			overlay->m_bHasWindowsDialog = false;
		}
	}

	
	//Check for any resizable controls that need to update their size before starting
	for(Asura_DialogIterator xIt(overlay->m_pxDialogList); !xIt.Done(); xIt.Next())
	{
		Asura_Dialog* pxDialog = xIt.GetCurrent();
		if(pxDialog->GetAllowedResizeFlags())
		{
			pxDialog->StartResize(ASURA_OVERLAY_RESIZE_INITIAL);
			pxDialog->UpdateResize(ASURA_OVERLAY_RESIZE_INITIAL, Asura_Maths::ZeroVector2);
			overlay->FlagForResize();
		}
	}
	overlay->SetSubmissionIndex(s_uNextSubmissionIndex++);

	OverlayList[NumberOfActiveOverlays++] = overlay;
}

static void VectorHome(Asura_Vector_2& v1, Asura_Vector_2& v2)
{	
	Asura_Vector_2 delta = v2-v1;
	delta.Normalise();

	delta*=OVERLAY_HOMING_SPEED*Asura_Timers::GetFrameTime();

	v1+=delta;

	if ((v2.x-v1.x)*delta.x<0.0f) v1.x = v2.x;
	if ((v2.y-v1.y)*delta.y<0.0f) v1.y = v2.y;


}

float Asura_Overlay_System::GetMaxScreenX()
{
	return Asura_DisplayLimits::Platform_GetBoundingRectangle().MaxX;
}

float Asura_Overlay_System::GetMaxScreenY()
{
	return Asura_DisplayLimits::Platform_GetBoundingRectangle().MaxY;
}

void Asura_Overlay_System::Update(bool lockedOut)
{
	s_bUsedKeypressThisFrame = false;
	CursorIsOnAnOverlay = false;
	ClearCallbackFunctionToCall();

	//Only allow HaasKeyboardFocus to go false at the beginning of the frame , so that any keypresses
	//used thathappen when focus is lost , aren't picked up again later on in the same frame.
	HasKeyboardFocus = (FocusType & OverlayFocusType_Keyboard) ? true : false;

	Asura_Bounding_Rectangle xBounds = Asura_DisplayLimits::Platform_GetBoundingRectangle();
	xBounds.MinX /= ASURA_TEXT_OUTPUT_FONT_WIDTH;
	xBounds.MaxX /= ASURA_TEXT_OUTPUT_FONT_WIDTH;
	xBounds.MinY /= ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	xBounds.MaxY /= ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	int i;

	//kill off any overlays that have been flagged for closing
	CheckForClosingOverlays();
	UpdateOverlayPanelSize();

	if (lockedOut)
	{
		Asura_Cursor::SetType(ASURA_CURSOR_ARROW);

		for (i=0; i<NumberOfActiveOverlays; i++)
		{	
			OverlayList[i]->m_uBrightness = s_uFadedOverlayAlpha;
		}
		return;
	}

	Asura_Cursor::SetType(Tools_Object_System::AreControlsEnabled() ? ASURA_CURSOR_OBJECT : ASURA_CURSOR_CROSSHAIR);

	Asura_Vector_2 minimise(0,MaxOverlayY);

	if(!Asura_UserInput::KeyPressed(ASURA_KEYID_ALT))
	{
		//check for minimise all shortcut (Ctrl+M)
		if (Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && Asura_UserInput::DebouncedKeyPressed('m'))
		{
			//Minimise anything that isn't AlwaysOnTop
			//Note , I'm not checking to see whether the overlay has a minimise gadget , which
			//may be an issue if there is ever reason to create one without
			for (i=0; i<NumberOfActiveOverlays; i++)
			{
				if(!OverlayList[i]->m_bAlwaysOnTop && !OverlayList[i]->m_bOnPanel)
				{
					OverlayList[i]->SetMinimised(true, false);	
				}
			}

		}

		//and hide overlays (Ctrl+O)
		bool bToggleHideOverlays = false;
		if (Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && Asura_UserInput::DebouncedKeyPressed('o'))
		{
			bToggleHideOverlays = true;
		}
		
		if(!GetFocusGadget())
		{
			// Alternative shortcut for hide overlays
			if (Asura_UserInput::DebouncedKeyPressed('`') && !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && !Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
			{
				bToggleHideOverlays = true;
			}
		}

		if (bToggleHideOverlays)
		{
			bool bHide = !GetUserHideOverlays();
			SetUserHideOverlays(bHide);
			Tools_Timed_Message_System::AddMessage(0xffffff,"Overlays %s",bHide ? "Hidden" : "Shown");
		}


	}

	bool bAnyOverlaysAlwaysOnTop = AreAnyOverlaysAlwaysOnTop();


	// Check for switch overlay shortcut.
	if( Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) )
	{
		if( NumberOfActiveOverlays > 1 )
		{
			// Check no overlays have "always on top" on.


			if( bAnyOverlaysAlwaysOnTop )
			{
				if( g_bDisplayCtrlTabOverlay )
				{
					g_bDisplayCtrlTabOverlay = false;
					g_uCtrlTabPosInOverlayList = 0;
					g_pxCtrlTabDialog->ForgetOverlayList();
				}
			}
			else
			{
				if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_TAB) )
				{
					++g_uCtrlTabPosInOverlayList;
					if( g_uCtrlTabPosInOverlayList >= static_cast<u_int>(NumberOfActiveOverlays) )
					{
						g_uCtrlTabPosInOverlayList = 0;
					}
				}

				Asura_Assert( g_pxCtrlTabOverlay && g_pxCtrlTabDialog, "" );

				if( !g_bDisplayCtrlTabOverlay )
				{
					// Only display once they've pressed Tab for the first
					// time.
					if( g_uCtrlTabPosInOverlayList )
					{
						g_bDisplayCtrlTabOverlay = true;

						ReleaseFocus();

						g_pxCtrlTabDialog->SetOverlayList( OverlayList, NumberOfActiveOverlays );

						// Centre overlay on screen.
						g_pxCtrlTabOverlay->CalculateOverlaySize();
						g_pxCtrlTabOverlay->SetPosition( 0.5f * (xBounds.MaxX - xBounds.MinX - g_pxCtrlTabOverlay->m_fWidth), 0.5f * (xBounds.MaxY - xBounds.MinY - g_pxCtrlTabOverlay->m_fHeight), true );
					}
				}

				if( g_bDisplayCtrlTabOverlay )
				{
					g_pxCtrlTabDialog->SetSelectedEntry( g_uCtrlTabPosInOverlayList );
				}
			}
		}
	}
	else
	{
		if( g_bDisplayCtrlTabOverlay )
		{
			// Now that CTRL has been released, check if we need to switch.
			if( g_uCtrlTabPosInOverlayList )
			{
				Asura_Assert( g_uCtrlTabPosInOverlayList < static_cast<u_int>(NumberOfActiveOverlays), "" );
				OverlayList[g_uCtrlTabPosInOverlayList]->SetMinimised(false, false);
				OverlayList[g_uCtrlTabPosInOverlayList]->m_bSortToFront = true;
				g_uCtrlTabPosInOverlayList = 0;
			}

			g_bDisplayCtrlTabOverlay = false;
			g_pxCtrlTabDialog->ForgetOverlayList();
		}
	}
	// End ctrl-tab processing.

	s_bFocusSetThisFrame = false;

	Asura_Assert(!g_bDoingMainUpdate, "Already in another call to Asura_Overlay_System::Update()");
	g_bDoingMainUpdate = true;//Prevent the overlay array from being fiddled with while doing this part of the update
	for (i=0; i<NumberOfActiveOverlays; i++)
	{	

		if( g_bDisplayCtrlTabOverlay )
		{
			// NB: Even if the overlay is flagged as "always fullbright", I
			// think we want to make it duller when the ctrl-tab overlay is
			// on.
			OverlayList[i]->m_uBrightness     = 127;
			OverlayList[i]->m_bIsBeingDragged = false;
			OverlayList[i]->m_uResizeFlags = 0;
			OverlayList[i]->PerFrameUpdate();
			continue;
		}

		// update overlay's position
		if (!OverlayList[i]->IsMinimisedAtBottom() && !OverlayList[i]->IsHidden())
		{
			if (OverlayList[i]->m_bIsBeingDragged && !OverlayList[i]->m_bOnPanel)
			{
				float cx,cy;
				Asura_Cursor::GetScreenDeltas(cx,cy);

				OverlayList[i]->m_xPosition.x += cx;
				OverlayList[i]->m_xPosition.y += cy;
				OverlayList[i]->m_xOrigin.x += cx;
				OverlayList[i]->m_xOrigin.y += cy;

			}
			if(OverlayList[i]->m_uResizeFlags)
			{
				OverlayList[i]->UpdateResize();
			}
			if(!OverlayList[i]->m_bOnPanel)
			{
				//If overlay is too wide to fit on screen  , or goes off the left of the screen  , align to the left side
				if (OverlayList[i]->m_fWidth>=xBounds.MaxX || OverlayList[i]->m_xPosition.x<xBounds.MinX) OverlayList[i]->m_xPosition.x=xBounds.MinX;
				//If overlay can fit on screen , but goes off the right , align it to the right
				else if (OverlayList[i]->m_xPosition.x>xBounds.MaxX-OverlayList[i]->m_fWidth) OverlayList[i]->m_xPosition.x=xBounds.MaxX-OverlayList[i]->m_fWidth;

				//similarly for top/bottom
				if (OverlayList[i]->m_xPosition.y<xBounds.MinY + 2.0f) OverlayList[i]->m_xPosition.y=xBounds.MinY + 2.0f;
				else if (OverlayList[i]->m_xPosition.y>xBounds.MaxY-2.0f) OverlayList[i]->m_xPosition.y=xBounds.MaxY-2.0f;
			}

			VectorHome(OverlayList[i]->m_xOrigin,OverlayList[i]->m_xPosition);
			if(!OverlayList[i]->IsMinimised())
			{
				OverlayList[i]->m_fHeight = OverlayList[i]->m_fFullHeight;
			}
			if(OverlayList[i]->m_pxPositionBackup)
			{
				*OverlayList[i]->m_pxPositionBackup = OverlayList[i]->m_xPosition;
			}
		}

		bool CursorIsOnThisOverlay = OverlayList[i]->HasCursorOnTop();
		if(OverlayList[i]->m_bAlwaysOnTop)
		{
			//if overlay is on top , cursor always counts as being on it
			CursorIsOnThisOverlay = true;

		}
		else if((AreOverlaysHidden() && !OverlayList[i]->GetOverrideOverlaysHiddenFlag()) || OverlayList[i]->IsHidden())
		{
			//if overlays are hidden , cursor can't be over it.
			//also make sure no overlay hidden in this way has focus
			if(FocusGadget && OverlayList[i] == CurrentOverlay)
			{
				ReleaseFocus();
			}
			CursorIsOnThisOverlay = false;
		}
		else if(FocusGadget)
		{
			//while a gadget has focus , its overlay remains the current one
			CursorIsOnThisOverlay = OverlayList[i] == CurrentOverlay;

		}
		else if(OverlayList[i]->m_bIsBeingDragged || OverlayList[i]->m_uResizeFlags)
		{
			CursorIsOnThisOverlay = true;
		}
		else if(Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEBUTTON1) && !Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) &&!OverlayList[i]->m_bClickedOnOverlay)
		{
			//The mouse button is held down but wasn't pressed while over the overlay
			//Let the mouse go 'behind' the overlay , so that drag operations aren't
			//interfered with
			CursorIsOnThisOverlay = false;

		}

		if(OverlayList[i]->m_bJustCreated)
		{
			//Prevent the overlay from getting focus on the first frame. This is so that if it
			//came in existence as a result of a mouse click , then we won't pick it up
			//as a click on a control as well.
			//RAR 05/03/03
			CursorIsOnThisOverlay = false;
		}

		OverlayList[i]->m_bJustCreated = false;



		bool bWasClickedOnOverlay = OverlayList[i]->m_bClickedOnOverlay;
		OverlayList[i]->m_bClickedOnOverlay = false;
		if(OverlayList[i]->m_bAlwaysFullbright)
		{
			OverlayList[i]->m_uBrightness = 255;
		}
		else
		{
			OverlayList[i]->m_uBrightness = s_uFadedOverlayAlpha;
		}

		// update an overlay if the cursor is over it
		if (!CursorIsOnAnOverlay && CursorIsOnThisOverlay)
		{
			//take note of the current overlay
			if(CurrentOverlay != OverlayList[i])
			{
				ReleaseFocus();
				CurrentOverlay = OverlayList[i];
			}

			if((Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))||
				(bWasClickedOnOverlay && Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEBUTTON1)))
			{
				OverlayList[i]->m_bClickedOnOverlay = true;
			}

			#if 1
			OverlayList[i]->UpdateDragAndResizeState();
			#else
			if( OverlayList[i]->m_bCanBeMovedByUser && OverlayList[i]->HasCursorOnTop())
			{
				//Only allow dragging to start if the cursor isn't currently over another control on the overlay.
				//RAR 12/12/2003
				if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) && !OverlayList[i]->IsCursorOverADialog())
				{
					OverlayList[i]->IsBeingDragged = true;
				}
				else if(bWasBeingDragged && Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEBUTTON1))
				{
					OverlayList[i]->IsBeingDragged = true;
				}
			}
			#endif

			Asura_Cursor::SetType(ASURA_CURSOR_ARROW);

			//(get the minimised status before calling update.. prevents me from immediately maximising again)
			u_int uOldMinimisedState = OverlayList[i]->m_uMinimisedState;
			OverlayList[i]->Update();

			CursorIsOnAnOverlay=true;

			if(!OverlayList[i]->IsMinimisedAtBottom()) 
			{
				if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1)
					||Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON2))
				{
					OverlayList[i]->m_bSortToFront = true;
				}
			}
			else
			{
				OverlayList[i]->m_uBrightness = 255;
			}

			if(uOldMinimisedState == OverlayList[i]->m_uMinimisedState)
			{
				
				//Clicking on the bar of a minimised overlay restores it
				//(if minimised at the bottom)
				if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
				{
					if(OverlayList[i]->m_uMinimisedState == ASURA_OVERLAY_MINIMISED_ATBOTTOM)
					{
						OverlayList[i]->SetMinimised(false, false);
					}
				}
				if (Asura_UserInput::KeyPressed(ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON1) && !OverlayList[i]->m_bOnPanel)
				{
					//double clicking will also restore an overlay that has been minimised in either way
					if(OverlayList[i]->m_uMinimisedState != ASURA_OVERLAY_MINIMISED_NOTMINIMISED)
					{
						OverlayList[i]->SetMinimised(false, false);
					}
					//Or minimise an overlay if done on the title bar
					else if(OverlayList[i]->HasCursorOverTitleBar() && !OverlayList[i]->m_bAlwaysOnTop)
					{
						OverlayList[i]->SetMinimised(true, false);
					}
				}
			}
		}
		else
		{
			OverlayList[i]->m_bIsBeingDragged = false;
			OverlayList[i]->m_uResizeFlags = 0;
		}

		OverlayList[i]->PerFrameUpdate();

		// if overlay has just been minimised, set it's size and push it to the back of the list
		if (OverlayList[i]->IsMinimised() && OverlayList[i]->m_fHeight == OverlayList[i]->m_fFullHeight)
		{
			OverlayList[i]->m_bSortToBack = true;
			OverlayList[i]->m_fHeight = 2.0f;

			// NB: I think minimised overlays should be duller even if they're
			// flagged as "always fullbright"; that should only apply when
			// they're visible.
			OverlayList[i]->m_uBrightness = 127;
		}

	}

	if(FocusGadget)
	{
		Asura_Assert(CurrentOverlay,"If a gadget has focus then there should be a current overlay");
		//check for some things that result in the loss of focus
		//being minimised loses it
		//as does clicking on something
		if(CurrentOverlay->IsMinimised())
		{
			ReleaseFocus();
		}
		else if(!s_bFocusSetThisFrame)//only clear focus from clicking ,if focus hasn't changed this frame
		{
			if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) ||
				Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON2))
			{
				ReleaseFocus();
			}
		}
	}
	g_bDoingMainUpdate = false;//Should be safe for the overlay array to be altered again.


	for (i=0; i<NumberOfActiveOverlays; )
	{	
		// position minimised overlays
		if (OverlayList[i]->IsMinimisedAtBottom() && !OverlayList[i]->IsHidden())
		{
			if (xBounds.MaxX - minimise.x < OverlayList[i]->m_fWidth)
			{
				minimise.x = 0;
				minimise.y -= 2.0f;
			}
			VectorHome(OverlayList[i]->m_xOrigin,minimise);

			minimise.x += OverlayList[i]->m_fWidth;
		}
		// update IsMinimisedBackup
		OverlayList[i]->UpdateExternalMinimisedState();


		i++;
	}
	//re-order overlays if needed
	SortOverlays();

	// topmost overlay should be lit up (unless it's minimised, of course)
	if (NumberOfActiveOverlays)
	{
		if (!OverlayList[0]->IsMinimised())
		{
			OverlayList[0]->m_uBrightness = 255;
		}
	}
	//kill off any overlays that have been flagged for closing
	CheckForClosingOverlays();

	if(s_xCallbackFunction.IsValid())
	{
		s_xCallbackFunction.Execute();
		s_xCallbackFunction.MakeNull();
	}


	//if the mouse is over the panel area at the bottom ,then consider it to be over an overlay , regardless
	//of whether it is in fact over one of the individual overlays
	float cx,cy;
	Asura_Cursor::GetScreenCoords(cx,cy);
	if(cy>PanelMinY)
	{
		CursorIsOnAnOverlay = true;
	}

	//also if a gadget has mouse focus
	if(FocusType & OverlayFocusType_Mouse)
	{
		CursorIsOnAnOverlay = true;
	}

	// Also if the ctrl-tab overlay is on.
	if( g_bDisplayCtrlTabOverlay )
	{
		CursorIsOnAnOverlay = true;
	}

	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
	{
		s_bLastMouseClickWasOnOverlay = false;

		for(int n = 0; n < NumberOfActiveOverlays; n++)
		{
			if(OverlayList[n]->HasCursorOnTop())
			{
				s_bLastMouseClickWasOnOverlay = true;
			}
		}
	}
}

void Asura_Overlay_System::Render(void)
{
	CheckForClosingOverlays();

	//Make sure overlays doesn't get done in wireframe. RAR 14/05/03
	Asura_Render::CurrentStates.RequestWireFrameMode( false );

	bool bRenderedPanelBackdrop = false;

	// render overlays
	for (int i=NumberOfActiveOverlays-1; i>=0; i--)
	{	
		if(((AreOverlaysHidden() && !OverlayList[i]->GetOverrideOverlaysHiddenFlag()) || OverlayList[i]->IsHidden()) && !OverlayList[i]->m_bAlwaysOnTop) continue;

		Asura_OverlayBrightness = OverlayList[i]->m_uBrightness<<24;
		if(OverlayList[i]->m_bOnPanel)
		{
			Asura_OverlayBrightness = 0xff000000;
			if(!bRenderedPanelBackdrop)
			{
				bRenderedPanelBackdrop = true;
				RenderPanelBackdrop();
			}
		}
		OverlayList[i]->RenderBackdrop(s_uOverlayColour);

		Asura_Render::FlushDisplay();

		OverlayList[i]->Render();

		// JAT 27/11/06 - render any batched lines
		Asura_Render::FlushDisplay();
	}

	if( g_bDisplayCtrlTabOverlay )
	{
		Asura_Assert( g_pxCtrlTabOverlay, "" );

		Asura_OverlayBrightness = 0xff000000;
		g_pxCtrlTabOverlay->RenderBackdrop( 0x7f1111 );

		Asura_Render::FlushDisplay();

		g_pxCtrlTabOverlay->Render();

		// JAT 27/11/06 - render any batched lines
		Asura_Render::FlushDisplay();
	}

	if(!s_xHelpText.IsBlank())
	{
		RenderHelpText(s_xHelpText.GetString());
		s_xHelpText.MakeNull();
	}
}

void Asura_Overlay_System::RenderHelpText(const char* pszText)
{
	if(!pszText) return;
	Asura_Handle iFont = Asura_Text_Output::GetSmallFontHandle();
	if(iFont == ASURA_INVALID_HANDLE) return;


	int iLength = strlen(pszText);
	Asura_UnicodeChar* pszUnicodeText = new Asura_UnicodeChar[iLength+1];
	Asura_PrintfToBfr(pszUnicodeText,L"%S",iLength+1,true,pszText);


	u_int uTextColour = 0xff4fff4f;
	u_int uFormat = ASURA_TEXTFORMAT_WORDWRAP|ASURA_TEXTFORMAT_RELATIVETORECT;

	float fScreenWidth = (float)Asura_Viewport::GetWidth();
	float fScreenHeight = (float)Asura_Viewport::GetHeight();



	//first determine the size of box needed for the text
	Asura_Bounding_Rectangle xRect;
	xRect.MinX = 0;
	xRect.MaxX = 400.0f;
	xRect.MinY = 0;
	xRect.MaxY = fScreenHeight;

	Asura_Font_System::SetFont(iFont);
	//Asura_Font_System::Print(xRect,0,0,1.0f,1.0f,uTextColour,uFormat | ASURA_TEXTFORMAT_DONTRENDER,"%s",pszText);
	Asura_Font_System::PrintLongString(pszUnicodeText,&xRect,0,0,1.0f,1.0f,uTextColour,uFormat | ASURA_TEXTFORMAT_DONTRENDER);

	//shrink the box so just large enough for the text
	xRect.MaxY = Asura_Font_System::GetNextLinesYCoordinate()+2.0f;
	xRect.MaxX = Asura_Font_System::GetMaxPrintedXCoordinate();


	//reposition box near cursor
	int iCursorX,iCursorY;
	Asura_Cursor::GetScreenSpaceCoords(iCursorX,iCursorY);

	xRect.MinX = iCursorX+16.0f;
	if(xRect.MinX + xRect.MaxX > fScreenWidth)
	{
		xRect.MinX = fScreenWidth - xRect.MaxX;
	}
	xRect.MaxX += xRect.MinX;

	if(xRect.MaxY<iCursorY)
	{
		//position above the cursor , if there is space
		xRect.MinY = iCursorY-xRect.MaxY;
	}
	else
	{
		//otherwise try putting below
		xRect.MinY = iCursorY+16.0f;
		if(xRect.MinY + xRect.MaxY > fScreenHeight)
		{
			xRect.MinY = fScreenHeight - xRect.MaxY;
		}
	}
	xRect.MaxY += xRect.MinY;


	//draw a background rectangle for the text
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);

	Asura_Render::Overlay.Colour = 0xbf000000;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;

	Asura_Render::Overlay.SetRectXY(xRect.MinX-2.0f,xRect.MinY,(xRect.MaxX-xRect.MinX)+2.0f,(xRect.MaxY-xRect.MinY));
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();


	//and draw the text
	//	Asura_Font_System::Print(xRect,0,0,1.0f,1.0f,uTextColour,uFormat,"%s",pszText);
	Asura_Font_System::PrintLongString(pszUnicodeText,&xRect,0,0,1.0f,1.0f,uTextColour,uFormat);

	delete [] pszUnicodeText;

}

void Asura_Overlay_System::RenderPanelBackdrop()
{
	bool anyOverlaysOnPanel = false;
	for (int i=NumberOfActiveOverlays-1; i>=0; i--)
	{
		if(!OverlayList[i]->m_bOnPanel) continue;
		anyOverlaysOnPanel = true;
		break;
	}
	if(!anyOverlaysOnPanel) return;//if no overlays , then don't draw this



	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);

	unsigned int colour = (0xff << 24) | (s_uOverlayColour);

	Asura_Render::Overlay.Colour = colour;
	Asura_Render::Overlay.TextureIndex = MenuBackdropTextureIndex;
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;

	float y=Asura_Maths::FloatToInt(PanelMinY*ASURA_TEXT_OUTPUT_FONT_HEIGHT)-0.5f;

	Asura_Render::Overlay.SetRectUV(0.0f,0.0f,1.0f,1.0f);
	Asura_Render::Overlay.SetRectXY(0,y,(float)Asura_Viewport::GetWidth(),Asura_Viewport::GetHeight()-y);

	Asura_Render::Overlay.Render();

	Asura_Render::FlushDisplay();
}

bool Asura_Overlay_System::DoesOverlayExist(unsigned int id)
{
	for (int i=0; i<NumberOfActiveOverlays; i++)
	{
		if (OverlayList[i]->IsClosing()) continue;//ignore overlays scheduled for destruction
		if (OverlayList[i]->m_uID == id) return true;
	}

	return false;
}

void Asura_Overlay_System::CloseOverlays(unsigned int id)
{
	bool bAnyClosed = false;
	for (int i=0; i<NumberOfActiveOverlays; i++)
	{
		if (OverlayList[i]->m_uID == id)
		{
			OverlayList[i]->m_bShouldClose = true;
			bAnyClosed = true;
		}
	}
	if(bAnyClosed) CheckForClosingOverlays();
}

Asura_Overlay* Asura_Overlay_System::GetOverlay(unsigned int id)
{
	for (int i=0; i<NumberOfActiveOverlays; i++)
	{
		if (OverlayList[i]->IsClosing()) continue;//ignore overlays scheduled for destruction
		if (OverlayList[i]->m_uID == id) return OverlayList[i];
	}
	return 0;
}

int Asura_Overlay_System::GetOverlayIndex( u_int uOverlayID )
{
	for (int i=0; i<NumberOfActiveOverlays; i++)
	{
		if (OverlayList[i]->m_uID == uOverlayID) return i;
	}
	return -1;
}

void Asura_Overlay_System::ReleaseFocus(int focusType)
{
	if(FocusGadget)
	{
		//tell the gadget that it no longer has focus
		FocusGadget->FocusLost(focusType & FocusType);
	}
	FocusType &=~ focusType;
	if(!FocusType)
	{
		FocusGadget = NULL;
	}
}

void Asura_Overlay_System::TakeFocus(Asura_Gadget* gadget,int focusType)
{
	//make sure nothing else has focus
	if(gadget!=FocusGadget)
	{
		ReleaseFocus();
	}
	FocusGadget = gadget;
	gadget->FocusGained(focusType);
	FocusType |= focusType;
	if(focusType & OverlayFocusType_Keyboard)
	{
		HasKeyboardFocus = true;
	}
	s_bFocusSetThisFrame = true;
}


void Asura_Overlay_System::CheckForClosingOverlays()
{
	if(g_bDoingMainUpdate) return;

	const int iPrevNumberOfActiveOverlays = NumberOfActiveOverlays;

	//kill off any overlays that have been flagged for closing
	for(int i=0;i<NumberOfActiveOverlays;)
	{
		if (OverlayList[i]->IsClosing())
		{
			//if closing the current overlay , then keyboard focus should be lost (if it had it)
			if(OverlayList[i] == CurrentOverlay)
			{
				ReleaseFocus();
				CurrentOverlay = NULL;
			}

			Asura_Overlay* pxOverlayToDelete = OverlayList[i];
			NumberOfActiveOverlays--;

			for(int j=i; j<NumberOfActiveOverlays; j++)
			{
				OverlayList[j] = OverlayList[j+1];
			}
			OverlayList[NumberOfActiveOverlays] = 0;

			delete pxOverlayToDelete;
			//Reset loop over overlays , since another may have been marked for deletion as a side effect.
			//(Currently can happen when deleting an overlay that contains a colour picker control).
			//RAR 29/06/2006
			//May be redudant now that CheckForClosingOverlays() is called by CloseOverlays() , but
			//probably no harm in doing it.
			//RAR 19/09/2007
			i = 0;
			continue;

		}
		i++;
	}

	if( iPrevNumberOfActiveOverlays != NumberOfActiveOverlays )
	{
		g_uCtrlTabPosInOverlayList = 0;
		g_bDisplayCtrlTabOverlay   = false;

		Asura_Assert(g_pxCtrlTabDialog, "CTRL-TAB dialog doesn't exist");
		if ( NULL != g_pxCtrlTabDialog )
		{
			g_pxCtrlTabDialog->ForgetOverlayList();
		}
	}
}

void Asura_Overlay_System::UpdateOverlayPanelSize()
{
	PanelMinY = Asura_Viewport::GetHeight()/ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	for(int i=0;i<NumberOfActiveOverlays;i++)
	{
		if((AreOverlaysHidden() && !OverlayList[i]->GetOverrideOverlaysHiddenFlag()) && !OverlayList[i]->m_bAlwaysOnTop) continue;
		if(OverlayList[i]->m_bOnPanel)
		{
			if(PanelMinY > OverlayList[i]->m_xPosition.y)
			{
				PanelMinY = OverlayList[i]->m_xPosition.y;
			}
		}
	}
	MaxOverlayY = PanelMinY-2.0f;
}

void Asura_Overlay_System::SetCallbackFunctionToCall(void(*function)(void))
{
	SetCallbackFunctionToCall(Asura_Callback_Void<void>(function));
}

void Asura_Overlay_System::SetCallbackFunctionToCall(void(*function)(int),int iUserData)
{
	SetCallbackFunctionToCall(Asura_Callback_UserData<void,int>(function, iUserData));
}

void Asura_Overlay_System::SetCallbackFunctionToCall(const Asura_Callback_Wrapper<void>& xCallback)
{
	s_xCallbackFunction = xCallback;
}

void Asura_Overlay_System::SetCallbackFunctionToCall(const Asura_Callback_Base<void>& xCallback)
{
	s_xCallbackFunction.SetCallback(&xCallback);
}

void Asura_Overlay_System::ClearCallbackFunctionToCall()
{
	s_xCallbackFunction.MakeNull();
}

void Asura_Overlay_System::BringOverlayToTop(u_int uOverlayID)
{
	//deal with any existing overlays marked as sort to front
	SortOverlays();

	//search for the specified overlay
	Asura_Overlay* pxOverlay = 0;
	int i=0;
	for(;i<NumberOfActiveOverlays;i++)
	{
		if(OverlayList[i]->m_uID == uOverlayID)
		{
			pxOverlay = OverlayList[i];
			break;
		}
	}
	if(!pxOverlay) return;

	for(;i;i--)
	{
		OverlayList[i] = OverlayList[i-1];
	}

	OverlayList[0] = pxOverlay;
	pxOverlay->SetMinimised(false, false);

}

u_int Asura_Overlay_System::GetTopOverlayID()
{
	CheckForClosingOverlays();

	//deal with any existing overlays marked as sort to front
	SortOverlays();
	if(NumberOfActiveOverlays)
	{
		return OverlayList[0]->m_uID;
	}
	else
	{
		return 0xffffffff;
	}

}

Asura_Overlay* Asura_Overlay_System::GetTopOverlay()
{
	CheckForClosingOverlays();

	//deal with any existing overlays marked as sort to front
	SortOverlays();
	if(NumberOfActiveOverlays)
	{
		return OverlayList[0];
	}

	return NULL;
}

void Asura_Overlay_System::SetAlwaysOnTop(Asura_Overlay* pxOverlay, bool bSet)
{
	Asura_Assert(pxOverlay, "");
	if(pxOverlay->m_bAlwaysOnTop != bSet)
	{
		pxOverlay->m_bAlwaysOnTop = bSet;

		if(bSet)
		{
			//If being brought to the top , update the submission index to give it higher priority
			pxOverlay->SetSubmissionIndex(s_uNextSubmissionIndex++);
			pxOverlay->SetMinimised(false, false);
		}

	}
}

bool Asura_Overlay_System::AreAnyOverlaysAlwaysOnTop()
{
	for( int i = NumberOfActiveOverlays; i--; )
	{
		if( OverlayList[i]->m_bAlwaysOnTop )
		{
			return true;
			break;
		}
	}
	return false;
}

bool Asura_Overlay_System::AreAnyOverlaysVisible()
{
	//if(AreOverlaysHidden()) return false;

	//see if there are any non-hidden overlays
	for( int i = NumberOfActiveOverlays; i--; )
	{
		if(!AreOverlaysHidden() || OverlayList[i]->GetOverrideOverlaysHiddenFlag())
		{
			if(!OverlayList[i]->IsHidden()) return true;
		}
	}

	return false;
}

void Asura_Overlay_System::SetHideFlagOnCurrentOverlays(u_int uFlags,bool bSet)
{
	for( int i = NumberOfActiveOverlays; i--; )
	{
		OverlayList[i]->SetHideFlag(uFlags,bSet);
	}
}

void Asura_Overlay_System::LoadSkinnedUITexture()
{
	MenuSkinTextureIndex = Asura_Texture_Management::AddTextureWithResetGroup(MENUSKINTEXTURENAME, ASURA_TEXTURE_LOADED_PERMANENT);

	// if the texture didn't load properly, disable skinning
	if( Asura_Texture_Management::GetTextureLoadState( MenuSkinTextureIndex ) == ASURA_TEXTURE_ERROR_LOADING )
	{
		Asura_Overlay_System::SetUseSkinnedUI( false );
	}
}


int Asura_Overlay_System::SortOverlaysCallback(const void* ppOverlay1,const void* ppOverlay2)
{
	const Asura_Overlay* pxOverlay1 = *((const Asura_Overlay**) ppOverlay1);
	const Asura_Overlay* pxOverlay2 = *((const Asura_Overlay**) ppOverlay2);

	if(pxOverlay1->m_bAlwaysOnTop && pxOverlay2->m_bAlwaysOnTop)
	{
		//put the most recently added overlay first
		if(pxOverlay1->GetSubmissionIndex() > pxOverlay2->GetSubmissionIndex())
		{
			return -1;		
		}
		else
		{
			return 1;
		}
	}
	if(pxOverlay1->m_bAlwaysOnTop)
	{
		return -1;
	}
	if(pxOverlay2->m_bAlwaysOnTop)
	{
		return 1;
	}

	//check the sort to front / sort to back flags	
	int iSortDir1 = 0;
	if(pxOverlay1->m_bSortToFront) iSortDir1 = -1;
	else if(pxOverlay1->m_bSortToBack) iSortDir1 = 1;
	int iSortDir2 = 0;
	if(pxOverlay2->m_bSortToFront) iSortDir2 = -1;
	else if(pxOverlay2->m_bSortToBack) iSortDir2 = 1;

	if(iSortDir1 < iSortDir2) return -1;
	if(iSortDir1 > iSortDir2) return 1;

	//resolve tie breaks using the previous ordering
	return (pxOverlay1->GetTieBreakIndex() < pxOverlay2->GetTieBreakIndex()) ? -1 : 1;

}

void Asura_Overlay_System::SortOverlays()
{
	if(g_bDoingMainUpdate) return;
	
	//store the current ordering for resolving tie breaks
	for(int i=0; i<NumberOfActiveOverlays; i++)
	{
		OverlayList[i]->SetTieBreakIndex(i);
	}

	//and sort the overlays
	Asura_SortAndSearch::Quicksort(OverlayList, NumberOfActiveOverlays, SortOverlaysCallback);

	//Do a second sort to put th panel overlays in a clump together
	int iFirstPanelOverlay = -1;
	for(int i=0; i<NumberOfActiveOverlays; i++)
	{
		if(OverlayList[i]->m_bOnPanel)
		{
			if(iFirstPanelOverlay == -1)
			{
				iFirstPanelOverlay = i;
			}
			OverlayList[i]->m_bSortToFront = true;
		}
		else
		{
			OverlayList[i]->m_bSortToFront = false;
		}
	}
	if(iFirstPanelOverlay>=0 && iFirstPanelOverlay+1 < NumberOfActiveOverlays)
	{
		Asura_SortAndSearch::Quicksort(&OverlayList[iFirstPanelOverlay+1], NumberOfActiveOverlays-(iFirstPanelOverlay+1), SortOverlaysCallback);
	}
	

	//clear the sort flags
	for(int i=0; i<NumberOfActiveOverlays; i++)
	{
		OverlayList[i]->m_bSortToFront = false;
		OverlayList[i]->m_bSortToBack = false;
	}
}

#endif // #ifdef ASURA_TOOLS
