// Asura_WAVAlign_Dialog.cpp
// Thomas Young - 18.08.08
// Copyright (C) 2008 Rebellion, All Rights Reserved.

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined ASURA_TOOLS

#include "Core/Asura_List.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Interface/Asura_WAVAlign_Dialog.h"
#include "Interface/Asura_Cursor.h"
#include "Rendering/Asura_Render.h"
#include "Misc/Tools_Modal_Overlay.h"

// G L O B A L   V A R I A B L E S ///////////////////////////////////////////

static const float fASURA_WAVALIGN_DIALOG_TICKHEIGHT		= 6.0f;
static const float fASURA_WAVALIGN_DIALOG_DRAGMARKERHEIGHT	= 16.0f;

static const u_int uASURA_WAVALIGN_MARKER_COLOUR		= 0x0083D69D;
static const u_int uASURA_WAVALIGN_LOOPAREA_COLOUR		= 0x21F5D20A;
static const u_int uASURA_WAVALIGN_PLAYMARKER_COLOUR	= 0x00629FFB;
static const u_int uASURA_WAVALIGN_BACKGROUND_COLOUR	= 0x00555555;
static const u_int uASURA_WAVALIGN_TICKTEXT_COLOUR		= 0x00888888;
static const u_int uASURA_WAVALIGN_BAROVERLAY1_COLOUR	= 0x15000000;
static const u_int uASURA_WAVALIGN_BAROVERLAY2_COLOUR	= 0x40000000;

static const float fASURA_WAVALIGN_MARKER_SELECT_THRESH = 1.0f;

static const float fASURA_WAVALIGN_ZOOM_SCALE = 0.33f;
static const float fASURA_WAVALIGN_MIN_ZOOM = 1.0f;		//Smallest time to show in seconds

static const float fASURA_WAVALIGN_LOAD_DATA_SWAP_INTERVAL = 0.05f;

// S T A T I C   M E M B E R   V A R I A B L E S /////////////////////////////

volatile bool	Asura_WAVRender_Gadget::m_bAnyGadgetIsLoading = false;
Asura_Mutex		Asura_WAVRender_Gadget::m_xGadgetLoadFlagMutex;

// D E F I N E S /////////////////////////////////////////////////////////////

#define ORIGIN_TO_SCREEN_COORDS(origin)		(origin).x *= ASURA_TEXT_OUTPUT_FONT_WIDTH; (origin).y *= ASURA_TEXT_OUTPUT_FONT_HEIGHT;

// WAVE format chunk IDs
static const uint32 CHUNK_RIFF = 'FFIR';
static const uint32 CHUNK_WAVE = 'EVAW';
static const uint32 CHUNK_PAD  = ' DAP';
static const uint32 CHUNK_fmt  = ' tmf';
static const uint32 CHUNK_data = 'atad';

// E N U M E R A T I O N S ///////////////////////////////////////////////////

enum ASURA_WAVDIALOG_DRAG_STATE
{
	ASURA_WAVDIALOG_DRAG_STATE_NONE = 0,
	ASURA_WAVDIALOG_DRAG_STATE_INTRO,
	ASURA_WAVDIALOG_DRAG_STATE_OUTRO
};

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

static int FindChunkInWAVFileData(const char* pcData, int iDataSize, uint32 uChunkName)
{
	const char* pcRead = pcData;

	if ( reinterpret_cast<const uint32*>(pcRead)[0] == CHUNK_RIFF )
	{
		pcRead += sizeof(uint32) * 2;  // Skip RIFF & Size field
	}
	else
	{
		return -1;
	}

	// Search all data until chunk name located
	while ( iDataSize > 0 )
	{
		if ( reinterpret_cast<const uint32*>(pcRead)[0] == uChunkName )
		{
			return sizeof(uint32) + ( int(pcRead) - int(pcData) );
		}

		pcRead++;
		iDataSize--;
	}

	return -1;
}


Asura_WAVAlign_Dialog::Asura_WAVAlign_Dialog (const Asura_Vector_2& xOffset, const float fWidth , const float fWAVHeight, bool* pbModified) : Asura_Dialog ( NULL, xOffset, 10 )
{
	m_xOffset		= xOffset;
	m_fWidth		= fWidth;
	m_fWAVHeight	= fWAVHeight;

	m_fIntroMarkerTime = 0.0f;
	m_fOutroMarkerTime = 0.0f;

	m_fVisibleTimeStart = 0.0f;
	m_fVisibleTimeEnd = 0.0f;

	m_xMaxCoord.x = xOffset.x + m_fWidth / ASURA_TEXT_OUTPUT_FONT_WIDTH;
	m_xMaxCoord.y = xOffset.y + fASURA_WAVALIGN_DIALOG_TICKHEIGHT / ASURA_TEXT_OUTPUT_FONT_HEIGHT + 1.0f;

	Asura_Vector_2 xNewOffset = xOffset;
	xNewOffset.y += fASURA_WAVALIGN_DIALOG_TICKHEIGHT / ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	m_uDragState = ASURA_WAVDIALOG_DRAG_STATE_NONE;

	m_bIntroMarkerHighlighted = false;
	m_bOutroMarkerHighlighted = false;

	m_fBarDuration = 0.0f;

	m_fPlaybackMarkerTime = -1.0f;

	m_pbModified = pbModified;

	// Render everything by default
	m_uRenderFlags =	ASURA_WAVALIGN_RENDER_PLAYBACK_MARKER
					|	ASURA_WAVALIGN_RENDER_LOOP_MARKERS
					|	ASURA_WAVALIGN_RENDER_XAXIS;

	m_fTicksHighlight = 0.0f;

	m_bShowBarsOnXAxis		= false;
	m_bCanShowBarsOnXAxis	= false;
}

Asura_WAVAlign_Dialog::~Asura_WAVAlign_Dialog	()
{
	
}

void Asura_WAVAlign_Dialog::PerFrameUpdate()
{
	Asura_Dialog::PerFrameUpdate();
}

void Asura_WAVAlign_Dialog::ShowHelpText()
{
	// TODO: Maybe do something cool here
}

int Asura_WAVAlign_Dialog::GetNumWAVEntries ()
{
	return m_xWAVRenderList.GetNumEntries();
}

void Asura_WAVAlign_Dialog::AddWAVEntry ( int iHandle )
{
	Asura_Vector_2 xWAVOffset = m_xOffset;
	int iWavEntries = GetNumWAVEntries();
	xWAVOffset.y += (static_cast<float>(iWavEntries) * m_fWAVHeight) / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	Asura_WAVRender_Gadget* pxGadget = new Asura_WAVRender_Gadget (iHandle, xWAVOffset, m_fWidth, m_fWAVHeight );
	if( pxGadget )
	{
		AddGadget( pxGadget );
		m_xWAVRenderList.AddToList( pxGadget );
	}

	++iWavEntries;

	m_xMaxCoord.y += pxGadget->m_fHeight;

	UpdateVisibleTimeText ();
}

void Asura_WAVAlign_Dialog::RemoveWAVEntry ( int iHandle )
{
	Asura_WAVRender_Gadget* pxGadget = FindWAVRenderGadget(iHandle);
	
	if(pxGadget)
	{
		DeleteSingleGadget				( pxGadget );
		m_xWAVRenderList.RemoveFromList	( pxGadget );

		m_xMaxCoord.y -= pxGadget->m_fHeight;
	}
}

bool Asura_WAVAlign_Dialog::LoadWAV ( const char* const szFilename, int iHandle )
{
	ClearWAV( iHandle );

	Asura_WAVRender_Gadget* pxRenderGadget = FindWAVRenderGadget(iHandle);
	if(pxRenderGadget)
	{
		if( m_fVisibleTimeEnd != 0.0f )
		{
			pxRenderGadget->SetVisibleTime( m_fVisibleTimeStart, m_fVisibleTimeEnd );
			const bool bLoaded = pxRenderGadget->LoadFromWAV( szFilename );
			if( bLoaded )
			{
				//Check that the duration of this file matches the others we have loaded
				bool bDurationMatches = true;

				Asura_List_It<Asura_WAVRender_Gadget> xIter(m_xWAVRenderList);
				
				if( xIter.Done() )
				{
					return true;
				}
				
				Asura_WAVRender_Gadget* pxOtherGadget = 0;

				do 
				{
					pxOtherGadget = xIter.GetCurrent();
					
					if( ( pxOtherGadget != pxRenderGadget ) && pxOtherGadget->HasWAV() )
					{
						bDurationMatches = ( pxRenderGadget->GetDuration() == pxOtherGadget->GetDuration() );
						if(!bDurationMatches)
						{
							break;
						}
					}

					xIter.Next ();
				} 
				while( !xIter.Done() );

				
				if( !bDurationMatches )
				{
					ShowMessageOverlay("Size Difference\n\nYou have selected a sample with a different length from the existing sample(s)\n\nExisting file(s):\n %.2f seconds long (%u samples)\n\nSelected file:\n %.2f seconds long (%u samples)\n", pxOtherGadget->GetDuration(), pxOtherGadget->GetSamples(), pxRenderGadget->GetDuration(), pxRenderGadget->GetSamples() );
					pxRenderGadget->ClearWAV ();
					return false;
				}
				return true;
			}
		}
		else
		{
			//if the visible end time is zero it should be safe to assume no other wavs are being displayed
			pxRenderGadget->SetVisibleTime( 0.0f, 0.0f );
			const bool bLoaded = pxRenderGadget->LoadFromWAV( szFilename );
			if( bLoaded )
			{
				m_fVisibleTimeEnd = pxRenderGadget->GetDuration();
				UpdateVisibleTimeText ();
				m_fOutroMarkerTime = m_fVisibleTimeEnd;
				return true;
			}
		}
	}
	return false;
}

void Asura_WAVAlign_Dialog::ClearWAV ( int iHandle )
{
	Asura_WAVRender_Gadget* pxGadget = FindWAVRenderGadget(iHandle);
	if(pxGadget)
	{
		static_cast<Asura_WAVRender_Gadget*>(pxGadget)->ClearWAV();
	}

	if( NoWAVsLoaded() )
	{
		m_fVisibleTimeStart = 0.0f;
		m_fVisibleTimeEnd = 0.0f;

		m_fIntroMarkerTime = 0.0f;
		m_fOutroMarkerTime = 0.0f;
	}

	UpdateVisibleTimeText ();
}


void Asura_WAVAlign_Dialog::SetBarDuration ( float fDuration )
{ 
	m_fBarDuration = fDuration;
	if( m_fBarDuration > 0.0f )
	{
		m_bCanShowBarsOnXAxis	= true;
	}
	else
	{
		m_bCanShowBarsOnXAxis	= false;
	}
}


void Asura_WAVAlign_Dialog::Render( const Asura_Vector_2& xOrigin )
{
	float fWavGadgetsHeight = static_cast<float>( GetNumWAVEntries() ) * m_fWAVHeight;
	float fTotalHeight = fWavGadgetsHeight;

	if( m_uRenderFlags & ASURA_WAVALIGN_RENDER_XAXIS )
	{
		fTotalHeight += fASURA_WAVALIGN_DIALOG_TICKHEIGHT;
	}

	Asura_Vector_2 xActualOrigin = xOrigin + m_xOffset;
	ORIGIN_TO_SCREEN_COORDS(xActualOrigin);

	// Background
	Asura_Render::Overlay.Colour			= ( uASURA_WAVALIGN_BACKGROUND_COLOUR | Asura_OverlayBrightness );
	Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.SetRectXY( xActualOrigin.x, xActualOrigin.y, m_fWidth, fTotalHeight );
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	//Render the gadgets
	Asura_Dialog::Render( xOrigin );

	if( !NoWAVsLoaded() )
	{
		if( m_uRenderFlags & ASURA_WAVALIGN_RENDER_LOOP_MARKERS )
		{
			//Intro/Outro markers
			float fIntroDisplayPos = -1.0f;
			float fOutroDisplayPos = -1.0f;

			if( m_fOutroMarkerTime != 0.0f )
			{
				if( ( m_fIntroMarkerTime >= m_fVisibleTimeStart ) && ( m_fIntroMarkerTime <= m_fVisibleTimeEnd ) )
				{
					fIntroDisplayPos = ( m_fIntroMarkerTime - m_fVisibleTimeStart ) / ( m_fVisibleTimeEnd - m_fVisibleTimeStart ) * m_fWidth;
				}
				if( ( m_fOutroMarkerTime >= m_fVisibleTimeStart ) && ( m_fOutroMarkerTime <= m_fVisibleTimeEnd ) )
				{
					fOutroDisplayPos = ( m_fOutroMarkerTime - m_fVisibleTimeStart ) / ( m_fVisibleTimeEnd - m_fVisibleTimeStart ) * m_fWidth;
				}
			}

			if( ( m_fOutroMarkerTime >= m_fVisibleTimeStart ) && ( m_fIntroMarkerTime <= m_fVisibleTimeEnd ) )
			{
				Asura_Render::Overlay.Colour			= uASURA_WAVALIGN_LOOPAREA_COLOUR;
				Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
				Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;

				float fMinX = -1.0f;
				float fMaxX = -1.0f;

				if(fIntroDisplayPos == -1.0f)
				{
					fMinX = xActualOrigin.x;
					if(fOutroDisplayPos == -1.0f)
					{
						fMaxX = m_fWidth;
					}
					else
					{
						fMaxX = fOutroDisplayPos;
					}
				}
				else
				{
					fMinX = xActualOrigin.x + fIntroDisplayPos;
					if(fOutroDisplayPos == -1.0f)
					{
						fMaxX = m_fWidth - fIntroDisplayPos;
					}
					else
					{
						fMaxX = fOutroDisplayPos - fIntroDisplayPos;
					}
				}

				Asura_Render::Overlay.SetRectXY( fMinX, xActualOrigin.y, fMaxX, fWavGadgetsHeight );
				Asura_Render::Overlay.Render();
				Asura_Render::FlushDisplay();
			}

			//Render our lines for the intro/outro over the top
			if(fIntroDisplayPos != -1.0f )
			{
				Asura_Render::Render2DLine(xActualOrigin.x + fIntroDisplayPos, xActualOrigin.y + 1.0f, xActualOrigin.x + fIntroDisplayPos, xActualOrigin.y + fWavGadgetsHeight - 1.0f, Asura_OverlayBrightness | uASURA_WAVALIGN_MARKER_COLOUR );
				if(m_bIntroMarkerHighlighted)
				{
					if( m_uDragState == ASURA_WAVDIALOG_DRAG_STATE_INTRO )
					{
						Asura_Render::Render2DLine(xActualOrigin.x + fIntroDisplayPos + 1.0f, xActualOrigin.y + 1.0f, xActualOrigin.x + fIntroDisplayPos + 1.0f, xActualOrigin.y + fWavGadgetsHeight - 1.0f, 0x88FF0000 );
						Asura_Render::Render2DLine(xActualOrigin.x + fIntroDisplayPos - 1.0f, xActualOrigin.y + 1.0f, xActualOrigin.x + fIntroDisplayPos - 1.0f, xActualOrigin.y + fWavGadgetsHeight - 1.0f, 0x88FF0000 );
					}

					Asura_Render::FlushDisplay();

					const float fIconRow	= 3.0f;
					const float fIconColumn	= 0.0f;

					Asura_Render::Overlay.TextureIndex = Tools_Dialog::GetIconTexture();
					Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
					Asura_Render::Overlay.SetRectUV (0.0625f * fIconColumn, 0.0625f * fIconRow, 0.0625f, 0.0625f);
					Asura_Render::Overlay.SetRectXY (xActualOrigin.x + fIntroDisplayPos - 8.0f,  xActualOrigin.y + 1.0f, 16.0f, 16.0f);
					Asura_Render::Overlay.Colour = 0xFFFFFFFF;
					Asura_Render::Overlay.Render ();

					Asura_Render::FlushDisplay();
				}
			}
			if(fOutroDisplayPos != -1.0f )
			{
				Asura_Render::Render2DLine(xActualOrigin.x + fOutroDisplayPos, xActualOrigin.y + 1.0f, xActualOrigin.x + fOutroDisplayPos, xActualOrigin.y + fWavGadgetsHeight - 1.0f, Asura_OverlayBrightness | uASURA_WAVALIGN_MARKER_COLOUR );
				if(m_bOutroMarkerHighlighted)
				{
					if( m_uDragState == ASURA_WAVDIALOG_DRAG_STATE_OUTRO )
					{
						Asura_Render::Render2DLine(xActualOrigin.x + fOutroDisplayPos + 1.0f, xActualOrigin.y + 1.0f, xActualOrigin.x + fOutroDisplayPos + 1.0f, xActualOrigin.y + fWavGadgetsHeight - 1.0f, 0x88FF0000 );
						Asura_Render::Render2DLine(xActualOrigin.x + fOutroDisplayPos - 1.0f, xActualOrigin.y + 1.0f, xActualOrigin.x + fOutroDisplayPos - 1.0f, xActualOrigin.y + fWavGadgetsHeight - 1.0f, 0x88FF0000 );
					}

					Asura_Render::FlushDisplay();

					const float fIconRow	= 3.0f;
					const float fIconColumn	= 0.0f;

					Asura_Render::Overlay.TextureIndex = Tools_Dialog::GetIconTexture();
					Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
					Asura_Render::Overlay.SetRectUV (0.0625f * fIconColumn, 0.0625f * fIconRow, 0.0625f, 0.0625f);
					Asura_Render::Overlay.SetRectXY (xActualOrigin.x + fOutroDisplayPos - 8.0f,  xActualOrigin.y + 1.0f, 16.0f, 16.0f);
					Asura_Render::Overlay.Colour = 0xFFFFFFFF;
					Asura_Render::Overlay.Render ();

					Asura_Render::FlushDisplay();
				}
			}
		}

		// Playback marker

		if( m_uRenderFlags & ASURA_WAVALIGN_RENDER_PLAYBACK_MARKER )
		{
			float fPlaybackMarkerPos = -1.0f;
			if( ( m_fPlaybackMarkerTime > m_fVisibleTimeStart ) && ( m_fPlaybackMarkerTime < m_fVisibleTimeEnd ) )
			{
				fPlaybackMarkerPos = ( m_fPlaybackMarkerTime - m_fVisibleTimeStart ) / ( m_fVisibleTimeEnd - m_fVisibleTimeStart ) * m_fWidth;
			}
			if(fPlaybackMarkerPos != -1.0f )
			{
				Asura_Render::Render2DLine(xActualOrigin.x + fPlaybackMarkerPos, xActualOrigin.y + 1.0f, xActualOrigin.x + fPlaybackMarkerPos, xActualOrigin.y + fWavGadgetsHeight - 1.0f, Asura_OverlayBrightness | uASURA_WAVALIGN_PLAYMARKER_COLOUR );
			}
		}

		//Render our x-axis ( ticks and labels )
		if( m_uRenderFlags & ASURA_WAVALIGN_RENDER_XAXIS )
		{
			if( m_bShowBarsOnXAxis )
			{
				Asura_Assert( m_fBarDuration != 0.0f, "Expected a valid bar duration" );

				// Work out a nice scale
				int iTotalBars = static_cast<int>( (m_fVisibleTimeEnd - m_fVisibleTimeStart) / m_fBarDuration );
				
				int iInterval = 1;
				while ( static_cast<float>( iTotalBars / iInterval ) > ( m_fWidth * 0.45f ) )
				{
					iInterval *= 2;
				}
				
				float fFirstTickTime = RoundTimeToBars( m_fVisibleTimeStart );
				if( fFirstTickTime < m_fVisibleTimeStart )
				{
					fFirstTickTime += m_fBarDuration;
				}
				int iTickCount = 0;
				char szText[16];
				int iBarNum = static_cast<int>( fFirstTickTime / m_fBarDuration );
				float fLastBarTextPos = -100.0f;
				bool bTickHasLabel = false;
				while(true)
				{
					const float fTickTime = fFirstTickTime + m_fBarDuration * static_cast<float>(iTickCount);
					if( fTickTime > m_fVisibleTimeEnd )
					{
						//done
						break;
					}
					const float fRenderPos = ( fTickTime - m_fVisibleTimeStart ) / ( m_fVisibleTimeEnd - m_fVisibleTimeStart ) * m_fWidth;

					bTickHasLabel = false;
					if( fRenderPos - fLastBarTextPos > ( ASURA_TEXT_OUTPUT_FONT_WIDTH * 3.0f ) )
					{
						Asura_PrintfToBfr( szText, "%d", 16, true, iBarNum );
						Asura_Text_Output::Print ( ( xActualOrigin.x + fRenderPos ) / ASURA_TEXT_OUTPUT_FONT_WIDTH - Asura_Text_Output::WidthOfString( szText, ASURA_TEXT_FORMAT_SMALLFONT ) * 0.5f, ( xActualOrigin.y + fWavGadgetsHeight + fASURA_WAVALIGN_DIALOG_TICKHEIGHT ) / ASURA_TEXT_OUTPUT_FONT_HEIGHT, ( uASURA_WAVALIGN_TICKTEXT_COLOUR | Asura_OverlayBrightness ), ASURA_TEXT_FORMAT_SMALLFONT, szText );
						fLastBarTextPos = fRenderPos;
						bTickHasLabel = true;
					}

					Asura_Render::Render2DLine(xActualOrigin.x + fRenderPos, xActualOrigin.y + fWavGadgetsHeight, xActualOrigin.x + fRenderPos, xActualOrigin.y + fWavGadgetsHeight + fASURA_WAVALIGN_DIALOG_TICKHEIGHT - ( bTickHasLabel ? 2.0f : 0.0f ) , Asura_OverlayBrightness );

					if( m_fTicksHighlight > 0.0f )
					{
						const u_int uCol = ScaleOpacity( ( bTickHasLabel ? uASURA_WAVALIGN_BAROVERLAY2_COLOUR : uASURA_WAVALIGN_BAROVERLAY1_COLOUR ), m_fTicksHighlight );
						Asura_Render::Render2DLine( xActualOrigin.x + fRenderPos, xActualOrigin.y, xActualOrigin.x + fRenderPos, xActualOrigin.y + fWavGadgetsHeight, uCol );
					}

					iBarNum		+= iInterval;
					iTickCount	+= iInterval;
				}
			}
			else
			{
				// Render the time on the xaxis rather than bars

				// First pick a sensible tick unit, one which results in decent pixel spacing between ticks
				int iInterval = 1;	// in seconds
				int iMajorTick = 5;
				while ( ( m_fVisibleTimeEnd - m_fVisibleTimeStart ) / static_cast<float>( iInterval ) > ( m_fWidth * 0.33f ) )
				{
					switch( iInterval )
					{
						case 1:		iInterval = 5;		iMajorTick = 10;	break;
						case 5:		iInterval = 10;		iMajorTick = 30;	break;
						case 15:	iInterval = 20;		iMajorTick = 60;	break;
						case 20:	iInterval = 30;		iMajorTick = 120;	break;
						case 30:	iInterval = 60;		iMajorTick = 300;	break;
						case 60:	iInterval = 300;	iMajorTick = 600;	break;

						default:	Asura_Assert( false, "Failed to find appropriate tick scale" );
					}
				}

				const float fInterval = static_cast<float>( iInterval );
				float fRemainder = m_fVisibleTimeStart;
				while ( fRemainder > 0.0f )
				{
					fRemainder -= fInterval;
				}
				float fFirstTickTime = m_fVisibleTimeStart - fRemainder;

				char szText[16];
				int iTickCount = 0;
				float fLastTimeTextPos = -100.0f;
				while(true)
				{
					const float fTickTime = fFirstTickTime + fInterval * static_cast<float>( iTickCount );
					if( fTickTime > m_fVisibleTimeEnd )
					{
						//done
						break;
					}
					const float fRenderPos = ( fTickTime - m_fVisibleTimeStart ) / ( m_fVisibleTimeEnd - m_fVisibleTimeStart ) * m_fWidth;
					
					bool bTickHasLabel = false;
					if(		( fRenderPos - fLastTimeTextPos > ( ASURA_TEXT_OUTPUT_FONT_WIDTH * 5.0f ) )
						&&	( static_cast<int>(fTickTime) % iMajorTick == 0 ) 
						&&	( fRenderPos > ASURA_TEXT_OUTPUT_FONT_WIDTH )
						&&	( fRenderPos < m_fWidth - ASURA_TEXT_OUTPUT_FONT_WIDTH ) )
					{
						int iMinutes = static_cast<int>(fTickTime / 60.0f);
						int iSeconds = static_cast<int>(fTickTime) % 60;
						Asura_PrintfToBfr(szText,"%02d:%02d", 16, true, iMinutes, iSeconds);
						
						Asura_Text_Output::Print ( ( xActualOrigin.x + fRenderPos ) / ASURA_TEXT_OUTPUT_FONT_WIDTH - Asura_Text_Output::WidthOfString( szText, ASURA_TEXT_FORMAT_SMALLFONT ) * 0.5f, ( xActualOrigin.y + fWavGadgetsHeight + fASURA_WAVALIGN_DIALOG_TICKHEIGHT ) / ASURA_TEXT_OUTPUT_FONT_HEIGHT, ( uASURA_WAVALIGN_TICKTEXT_COLOUR | Asura_OverlayBrightness ), ASURA_TEXT_FORMAT_SMALLFONT, szText );
						fLastTimeTextPos = fRenderPos;
						bTickHasLabel = true;
					}

					Asura_Render::Render2DLine( xActualOrigin.x + fRenderPos, xActualOrigin.y + fWavGadgetsHeight, xActualOrigin.x + fRenderPos, xActualOrigin.y + fWavGadgetsHeight + fASURA_WAVALIGN_DIALOG_TICKHEIGHT - ( bTickHasLabel ? 2.0f : 0.0f ) , Asura_OverlayBrightness );

					if( m_fTicksHighlight > 0.0f )
					{
						const u_int uCol = ScaleOpacity( ( bTickHasLabel ? uASURA_WAVALIGN_BAROVERLAY2_COLOUR : uASURA_WAVALIGN_BAROVERLAY1_COLOUR ), m_fTicksHighlight );
						Asura_Render::Render2DLine( xActualOrigin.x + fRenderPos, xActualOrigin.y, xActualOrigin.x + fRenderPos, xActualOrigin.y + fWavGadgetsHeight, uCol );
					}
					++ iTickCount;
				}
			}
		}
	}
}

void Asura_WAVAlign_Dialog::Update( const Asura_Vector_2& xOrigin )
{
	Asura_Dialog::Update( xOrigin );

	float fCursorX, fCursorY;
	Asura_Cursor::GetScreenCoords(fCursorX, fCursorY);

	Asura_Vector_2 xActualOrigin = xOrigin + m_xOffset; 

	const float fWavGadgetsHeight = static_cast<float>( GetNumWAVEntries() ) * m_fWAVHeight;

	bool bZoomedThisUpdate = false;

	if(		( m_uDragState == ASURA_WAVDIALOG_DRAG_STATE_NONE )
		&&	( fCursorX > xActualOrigin.x )
		&&	( fCursorX < xActualOrigin.x + m_fWidth / ASURA_TEXT_OUTPUT_FONT_WIDTH )
		&&	( fCursorY > xActualOrigin.y )
		&&	( fCursorY < xActualOrigin.y + fWavGadgetsHeight / ASURA_TEXT_OUTPUT_FONT_HEIGHT ) 
		&&	( !NoWAVsLoaded() ) )
	{
		if( Asura_UserInput::KeyPressed( ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON1 ) )
		{
			//The user double clicked in on the waveforms, so zoom in
			const float fZoomToDuration = (m_fVisibleTimeEnd - m_fVisibleTimeStart) * fASURA_WAVALIGN_ZOOM_SCALE;
			if( fZoomToDuration > fASURA_WAVALIGN_MIN_ZOOM )
			{
				const float fRelativePos	= ( fCursorX - xActualOrigin.x ) / ( m_fWidth / ASURA_TEXT_OUTPUT_FONT_WIDTH );
				const float fZoomFocalTime	= m_fVisibleTimeStart + (m_fVisibleTimeEnd - m_fVisibleTimeStart) * fRelativePos;
				if( ( fZoomFocalTime - fZoomToDuration * 0.5f ) < m_fVisibleTimeStart )
				{
					m_fVisibleTimeEnd = m_fVisibleTimeStart + fZoomToDuration;
				}
				else if( ( fZoomFocalTime + fZoomToDuration * 0.5f ) > m_fVisibleTimeEnd )
				{
					m_fVisibleTimeStart = m_fVisibleTimeEnd - fZoomToDuration;
				}
				else
				{
					m_fVisibleTimeStart = fZoomFocalTime - fZoomToDuration * 0.5f;
					m_fVisibleTimeEnd	= fZoomFocalTime + fZoomToDuration * 0.5f;
				}
				bZoomedThisUpdate = true;
			}
		}
		else if ( Asura_UserInput::KeyPressed( ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON2 ) )
		{
			//The user double right clicked on the waveforms, so zoom out
			const float fTrackDuration = GetTrackDuration();

			//Assuming we are not already zoomed out fully...
			if( !( ( m_fVisibleTimeStart == 0.0f ) && ( m_fVisibleTimeEnd == fTrackDuration ) ) )
			{
				const float fZoomToDuration = (m_fVisibleTimeEnd - m_fVisibleTimeStart) * 1.02f / fASURA_WAVALIGN_ZOOM_SCALE;

				if( fZoomToDuration >= fTrackDuration )
				{
					m_fVisibleTimeStart = 0.0f;
					m_fVisibleTimeEnd	= fTrackDuration;
				}
				else
				{
					const float fCurrentCentreTime = m_fVisibleTimeStart + ( m_fVisibleTimeEnd - m_fVisibleTimeStart ) * 0.5f;
					if( fCurrentCentreTime + fZoomToDuration * 0.5f > fTrackDuration )
					{
						m_fVisibleTimeStart = fTrackDuration - fZoomToDuration;
						m_fVisibleTimeEnd	= fTrackDuration;
					}
					else if ( fCurrentCentreTime - fZoomToDuration * 0.5f < 0.0f )
					{
						m_fVisibleTimeStart = 0.0f;
						m_fVisibleTimeEnd = fZoomToDuration;
					}
					else
					{
						m_fVisibleTimeStart = fCurrentCentreTime - fZoomToDuration * 0.5f;
						m_fVisibleTimeEnd	= fCurrentCentreTime + fZoomToDuration * 0.5f;
					}
				}
				bZoomedThisUpdate = true;
			}
		}
	}

	if( bZoomedThisUpdate )
	{
		//Set all of the gadgets to this time scale
		Asura_List_It<Asura_WAVRender_Gadget> xIter(m_xWAVRenderList);
		if( !xIter.Done() )
		{
			do 
			{
				Asura_WAVRender_Gadget* pxGadget = xIter.GetCurrent();
				if( pxGadget )
				{
					pxGadget->SetVisibleTime( m_fVisibleTimeStart, m_fVisibleTimeEnd );
				}
				xIter.Next();
			}
			while( !xIter.Done() );
		}
		UpdateVisibleTimeText ();
	}
	else
	{
		//If we did not zoom, check the drag state

		switch(m_uDragState)
		{
			case ASURA_WAVDIALOG_DRAG_STATE_NONE:
			{
				if( NoWAVsLoaded() )
				{
					break;
				}

				float fIntroDisplayPos = -1.0f;
				float fOutroDisplayPos = -1.0f;

				if( m_fOutroMarkerTime != 0.0f )
				{
					if( ( m_fIntroMarkerTime >= m_fVisibleTimeStart ) && ( m_fIntroMarkerTime <= m_fVisibleTimeEnd ) )
					{
						fIntroDisplayPos = ( ( m_fIntroMarkerTime - m_fVisibleTimeStart ) / ( m_fVisibleTimeEnd - m_fVisibleTimeStart ) * m_fWidth ) / ASURA_TEXT_OUTPUT_FONT_WIDTH;
					}
					if( ( m_fOutroMarkerTime >= m_fVisibleTimeStart ) && ( m_fOutroMarkerTime <= m_fVisibleTimeEnd ) )
					{
						fOutroDisplayPos = ( ( m_fOutroMarkerTime - m_fVisibleTimeStart ) / ( m_fVisibleTimeEnd - m_fVisibleTimeStart ) * m_fWidth ) / ASURA_TEXT_OUTPUT_FONT_WIDTH;
					}
				}

				if(fIntroDisplayPos != -1.0f )
				{
					if(		( fCursorX > ( xActualOrigin.x + fIntroDisplayPos ) - fASURA_WAVALIGN_MARKER_SELECT_THRESH ) 
						&&	( fCursorX < ( xActualOrigin.x + fIntroDisplayPos ) + fASURA_WAVALIGN_MARKER_SELECT_THRESH ) 
						&&	( fCursorY > xActualOrigin.y )
						&&	( fCursorY < xActualOrigin.y + fWavGadgetsHeight / ASURA_TEXT_OUTPUT_FONT_HEIGHT ) )
					{
						m_bIntroMarkerHighlighted = true;

						// If we are over the button and clicking, start to drag
						if(		( fCursorY < xActualOrigin.y + fASURA_WAVALIGN_DIALOG_DRAGMARKERHEIGHT / ASURA_TEXT_OUTPUT_FONT_HEIGHT )
							&&	Asura_UserInput::KeyPressed( ASURA_KEYID_MOUSEBUTTON1 ) )
						{
							m_uDragState = ASURA_WAVDIALOG_DRAG_STATE_INTRO;
						}
					}
					else
					{
						m_bIntroMarkerHighlighted = false;
					}
				}

				if(fOutroDisplayPos != -1.0f )
				{
					if(		( fCursorX > ( xActualOrigin.x + fOutroDisplayPos ) - fASURA_WAVALIGN_MARKER_SELECT_THRESH ) 
						&&	( fCursorX < ( xActualOrigin.x + fOutroDisplayPos ) + fASURA_WAVALIGN_MARKER_SELECT_THRESH ) 
						&&	( fCursorY > xActualOrigin.y )
						&&	( fCursorY < xActualOrigin.y + fWavGadgetsHeight / ASURA_TEXT_OUTPUT_FONT_HEIGHT ) )
					{
						m_bOutroMarkerHighlighted = true;

						// If we are over the button and clicking, start to drag
						if(		( fCursorY < xActualOrigin.y + fASURA_WAVALIGN_DIALOG_DRAGMARKERHEIGHT / ASURA_TEXT_OUTPUT_FONT_HEIGHT )
							&&	Asura_UserInput::KeyPressed( ASURA_KEYID_MOUSEBUTTON1 ) )
						{
							m_uDragState = ASURA_WAVDIALOG_DRAG_STATE_OUTRO;
						}
					}
					else
					{
						m_bOutroMarkerHighlighted = false;
					}
				}
			}
			break;

			case ASURA_WAVDIALOG_DRAG_STATE_INTRO:
			case ASURA_WAVDIALOG_DRAG_STATE_OUTRO:
			{
				if( !Asura_UserInput::KeyPressed( ASURA_KEYID_MOUSEBUTTON1 ) )
				{
					m_uDragState = ASURA_WAVDIALOG_DRAG_STATE_NONE;
					if(m_fOutroMarkerTime < m_fIntroMarkerTime)
					{
						const float fTemp = m_fOutroMarkerTime;
						m_fOutroMarkerTime = m_fIntroMarkerTime;
						m_fIntroMarkerTime = fTemp;
					}
					if(m_pbModified)
					{
						*m_pbModified = true;
					}
				}

				float fDesiredTime = -1.0f;

				if( fCursorX <= xActualOrigin.x )
				{
					fDesiredTime = m_fVisibleTimeStart;
				}
				else if ( fCursorX >= xActualOrigin.x + m_fWidth )
				{
					fDesiredTime = m_fVisibleTimeEnd;
				}
				else
				{
					const float fRelativePos = ( fCursorX - xActualOrigin.x ) / ( m_fWidth / ASURA_TEXT_OUTPUT_FONT_WIDTH );
					fDesiredTime = m_fVisibleTimeStart + (m_fVisibleTimeEnd - m_fVisibleTimeStart) * fRelativePos;
				}
				fDesiredTime = RoundTimeToBars( fDesiredTime );
				if( fDesiredTime < m_fVisibleTimeStart )
				{
					fDesiredTime += m_fBarDuration;
					if( fDesiredTime < m_fVisibleTimeStart ) //Bar duration is zero
					{
						fDesiredTime = m_fVisibleTimeStart;
					}
				}
				else if ( fDesiredTime > m_fVisibleTimeEnd )
				{
					fDesiredTime -= m_fBarDuration;
					if( fDesiredTime > m_fVisibleTimeEnd ) //Bar duration is zero
					{
						fDesiredTime = m_fVisibleTimeEnd;
					}
				}
				if(m_uDragState == ASURA_WAVDIALOG_DRAG_STATE_INTRO)
				{
					m_fIntroMarkerTime = fDesiredTime;
				}
				else if (m_uDragState == ASURA_WAVDIALOG_DRAG_STATE_OUTRO)
				{
					m_fOutroMarkerTime = fDesiredTime;
				}
			}
			break;
		}
	}

	// Check if we should show bar hints
	if(		( m_uDragState == ASURA_WAVDIALOG_DRAG_STATE_NONE )
		&&	( fCursorX > xActualOrigin.x )
		&&	( fCursorX < xActualOrigin.x + m_fWidth / ASURA_TEXT_OUTPUT_FONT_WIDTH )
		&&	( fCursorY > xActualOrigin.y + fWavGadgetsHeight / ASURA_TEXT_OUTPUT_FONT_HEIGHT )
		&&	( fCursorY < xActualOrigin.y + fWavGadgetsHeight / ASURA_TEXT_OUTPUT_FONT_HEIGHT + fASURA_WAVALIGN_DIALOG_TICKHEIGHT / ASURA_TEXT_OUTPUT_FONT_HEIGHT ) 
		&&	( !NoWAVsLoaded() ) )
	{
		m_fTicksHighlight = 1.0f;

		if( Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_MOUSEBUTTON1 ) )
		{
			// Toggle the display of bars/time
			m_bShowBarsOnXAxis = !m_bShowBarsOnXAxis && m_bCanShowBarsOnXAxis;
		}
	}
	else if( m_fTicksHighlight > 0.0f )
	{
		m_fTicksHighlight -= Asura_Timers::GetActualFrameTime() * 0.5f;
	}
}

float Asura_WAVAlign_Dialog::RoundTimeToBars ( float fTime )
{
	if(m_fBarDuration == 0.0f)
	{
		return fTime;
	}

	fTime += m_fBarDuration * 0.5f;

	const int iNumBars = static_cast<int>(fTime / m_fBarDuration);
	const float fRemainder = fTime - ( m_fBarDuration * static_cast<float>( iNumBars ) );
	fTime -= fRemainder;

	return fTime;
}

void Asura_WAVAlign_Dialog::UpdateVisibleTimeText ()
{

}

Asura_WAVRender_Gadget* Asura_WAVAlign_Dialog::FindWAVRenderGadget ( int iHandle )
{
	Asura_List_It<Asura_WAVRender_Gadget> xIter(m_xWAVRenderList);
	if( xIter.Done() )
	{
		return NULL;
	}

	do 
	{
		Asura_WAVRender_Gadget* pxGadget = xIter.GetCurrent();
		if( pxGadget && ( pxGadget->GetHandle() == iHandle ) )
		{
			return pxGadget;
		}
		xIter.Next();
	} 
	while( !xIter.Done() );

	return NULL;
}


bool Asura_WAVAlign_Dialog::NoWAVsLoaded ()
{
	Asura_List_It<Asura_WAVRender_Gadget> xIter(m_xWAVRenderList);
	if( xIter.Done() )
	{
		return NULL;
	}

	do
	{
		Asura_WAVRender_Gadget* pxGadget = xIter.GetCurrent();

		if( pxGadget && pxGadget->HasWAV() )
		{
			return false;
		}
		xIter.Next();
	}
	while ( !xIter.Done() );
	
	return true;
}

float Asura_WAVAlign_Dialog::GetTrackDuration ()
{
	Asura_List_It<Asura_WAVRender_Gadget> xIter(m_xWAVRenderList);
	if( xIter.Done() )
	{
		return NULL;
	}

	do
	{
		Asura_WAVRender_Gadget* pxGadget = xIter.GetCurrent();

		if( pxGadget && pxGadget->HasWAV() )
		{
			return pxGadget->GetDuration();
		}
		xIter.Next();
	}
	while ( !xIter.Done() );

	return 0.0f;
}


void Asura_WAVAlign_Dialog::SetRenderFlag ( const u_int uFlag, bool bSet )
{
	if( bSet )
	{
		m_uRenderFlags |= uFlag;
	}
	else
	{
		m_uRenderFlags &= ~uFlag;
	}
}

u_int Asura_WAVAlign_Dialog::ScaleOpacity( const u_int uBaseColour, const float fRelativeOpacity )
{
	// Get the base opacity
	float fBaseOpacity = static_cast<float>( uBaseColour >> 24 );

	// Make a 0-255 value for the new opacity
	fBaseOpacity = ( fBaseOpacity / 255.0f ) * fRelativeOpacity * 255.0f;

	const u_int uNewOpacity = static_cast<u_int>( fBaseOpacity ) << 24;

	return ( ( uBaseColour & 0x00FFFFFF ) | uNewOpacity );
}


//-----------------------------------------------------------------------------

Asura_WAVRender_Gadget::Asura_WAVRender_Gadget	( const int iHandle, const Asura_Vector_2 xOffset, float fWidth, float fHeight ) : Asura_Gadget ()
{
	m_fDuration				= 0.0f;
	m_uSamples				= 0;
	m_fVisibleStart			= 0.0f;
	m_fVisibleEnd			= 0.0f;
	m_afHighRenderSamples	= 0;
	m_afLowRenderSamples	= 0;
	m_afRenderSamplesAlpha	= 0;
	m_iHandle				= iHandle;
	m_xOffset				= xOffset;

	m_fWidth				= fWidth	/ ASURA_TEXT_OUTPUT_FONT_WIDTH;
	m_fHeight				= fHeight	/ ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	m_bHasLoadedWAV			= false;

	m_fWidthUnscaled		= fWidth;
	m_fHeightUnscaled		= fHeight;

	m_bFinishedLoadingSamples = true;
	m_bAbortLoading = false;
}

Asura_WAVRender_Gadget::~Asura_WAVRender_Gadget ()
{
	if( IsStillLoading() )
	{
		AbortLoading();
	}
	WaitUntilSafe ();

	delete []m_afHighRenderSamples;
	delete []m_afLowRenderSamples;
	delete []m_afRenderSamplesAlpha;
}

void Asura_WAVRender_Gadget::WaitUntilSafe ()
{
	while(!m_bFinishedLoadingSamples)
	{
		Asura_Thread_System::DelayCurrentThread( 0.05f );
	}
}

bool Asura_WAVRender_Gadget::LoadFromWAV ( const char* const pxWavFilename )
{
	if( IsStillLoading() )
	{
		AbortLoading();
	}
	WaitUntilSafe ();

	m_xWavFilename = pxWavFilename;

	return RecalculateSamples ();
}

void Asura_WAVRender_Gadget::ClearWAV ()
{
	if( IsStillLoading() )
	{
		AbortLoading();
	}
	WaitUntilSafe ();

	m_xWavFilename	= "";
	m_fDuration		= 0.0f;
	m_uSamples		= 0;
	m_fVisibleStart	= 0.0f;
	m_fVisibleEnd	= 0.0f;

	delete []m_afHighRenderSamples;
	delete []m_afLowRenderSamples;
	delete []m_afRenderSamplesAlpha;

	m_afHighRenderSamples = 0;
	m_afLowRenderSamples = 0;
	m_afRenderSamplesAlpha = 0;

	m_bHasLoadedWAV = false;
}

void Asura_WAVRender_Gadget::Update	( const Asura_Vector_2& xOrigin )
{

}

void Asura_WAVRender_Gadget::Render( const Asura_Vector_2& xOrigin )
{
	Asura_Gadget::Render( xOrigin );

	Asura_Vector_2 xActualOrigin = xOrigin + m_xOffset;
	ORIGIN_TO_SCREEN_COORDS(xActualOrigin);

	// Background
	Asura_Render::Overlay.Colour = Asura_OverlayBrightness;
	Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.SetRectXY ( xActualOrigin.x, xActualOrigin.y, m_fWidthUnscaled, m_fHeightUnscaled );
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	// Foreground
	Asura_Render::Overlay.Colour			= ( IsEnabled() && HasWAV() ) ? ( 0x00ffffff | Asura_OverlayBrightness ) : ( 0x00555555 | Asura_OverlayBrightness );
	Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.SetRectXY ( xActualOrigin.x + 1.0f, xActualOrigin.y + 1.0f, m_fWidthUnscaled - 2.0f, m_fHeightUnscaled - 2.0f);
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	const float fHalfHeight = m_fHeightUnscaled * 0.5f;

	m_xRenderSamplesMutex.Lock();

	if(m_afHighRenderSamples && m_afLowRenderSamples)
	{
		Asura_Render::Render2DLine(xActualOrigin.x, xActualOrigin.y + m_fHeightUnscaled * 0.5f, xActualOrigin.x + m_fWidthUnscaled, xActualOrigin.y + m_fHeightUnscaled * 0.5f, 0x22222222 );

		for( int i = 0; i < m_iThreadedReadProgress; ++i )
		{
			const float fScreenX = static_cast<float>( xActualOrigin.x + i );
			const float fScreenY = static_cast<float>( xActualOrigin.y + m_fHeightUnscaled * 0.5f );

			m_afRenderSamplesAlpha[i] = Asura_Maths::Min( m_afRenderSamplesAlpha[i] + Asura_Timers::GetActualFrameTime() * 4.0f, 1.0f );
			const float fDesiredAlpha = static_cast<float>( Asura_OverlayBrightness >> 24 ) / 255.0f * m_afRenderSamplesAlpha[i];

			u_int uRenderCol = static_cast<u_int>( fDesiredAlpha * 255.0f ) << 24;

			Asura_Render::Render2DLine(fScreenX, fScreenY + m_afHighRenderSamples[ i ] * fHalfHeight, fScreenX, fScreenY + m_afLowRenderSamples[ i ] * fHalfHeight, uRenderCol );
		}
	}

	m_xRenderSamplesMutex.Unlock();
}

void Asura_WAVRender_Gadget::SetVisibleTime (float fStart, float fEnd)
{
	if( IsStillLoading() )
	{
		AbortLoading();
	}
	WaitUntilSafe ();

	m_fVisibleStart	= fStart;
	m_fVisibleEnd	= fEnd;

	RecalculateSamples ();
}

bool Asura_WAVRender_Gadget::RecalculateSamples ()
{
	if( m_xWavFilename.GetLength() == 0 )
	{
		return false;
	}
	PrepareToReadSampleData();
	
	return Asura_Thread_System::CreateThread( "Wav loader", ThreadFuncLoadSamples, this );
}

bool Asura_WAVRender_Gadget::PrepareToReadSampleData ()
{
	const int iMaxToBuffer = 1024;

	m_xRenderSamplesMutex.Lock();

	delete []m_afHighRenderSamples;
	delete []m_afLowRenderSamples;
	delete []m_afRenderSamplesAlpha;

	m_iNumRenderSamples = static_cast<u_int>(m_fWidthUnscaled);

	m_afHighRenderSamples	= new float [m_iNumRenderSamples];
	m_afLowRenderSamples	= new float [m_iNumRenderSamples];
	m_afRenderSamplesAlpha	= new float [m_iNumRenderSamples];

	m_iThreadedReadProgress = 0;

	m_xRenderSamplesMutex.Unlock();

	m_bAbortLoading = false;

	Asura_Mem::Set( m_afHighRenderSamples	, 0, sizeof(float) * m_iNumRenderSamples );
	Asura_Mem::Set( m_afLowRenderSamples	, 0, sizeof(float) * m_iNumRenderSamples );
	Asura_Mem::Set( m_afRenderSamplesAlpha	, 0, sizeof(float) * m_iNumRenderSamples );
	
	// Open WAV file for reading
	Asura_Handle xWAVFile = Asura_FileAccess::OpenRaw( m_xWavFilename.GetString() );
	if ( xWAVFile == ASURA_INVALID_HANDLE )
	{
		Asura_Assertf( false, ( "Failed to open wav file %s", m_xWavFilename.GetString() ) );
		return false;
	}

	int iFileSize = Asura_FileAccess::GetFileSize(xWAVFile);

	if(iFileSize < 1024)
	{
		//This is a fairly ungraceful way of dealing with the fact that small files might break this algorithm
		Asura_Assertf( false, ("WAV file is too small %s", m_xWavFilename.GetString() ) );
		Asura_FileAccess::Close( xWAVFile );
		return false;
	}

	char* pcBuffer = new char[iMaxToBuffer];

	if( !Asura_FileAccess::ReadData( xWAVFile, pcBuffer, iMaxToBuffer ) )
	{
		Asura_Assertf( false, ( "Failed to read wav file %s", m_xWavFilename.GetString() ) );
		delete []pcBuffer;
		Asura_FileAccess::Close( xWAVFile );
		return false;
	}

	// Check for wave header
	int iChunkOffset = FindChunkInWAVFileData(pcBuffer, iMaxToBuffer, CHUNK_WAVE);
	if ( iChunkOffset < 0 )
	{
		Asura_Assertf(false, ("Missing 'WAVE' chunk from file %s", m_xWavFilename.GetString() ) );
		delete []pcBuffer;
		Asura_FileAccess::Close( xWAVFile );
		return false;
	}

	WAVEFORMATEX xWaveFormat;

	Asura_Mem::Copy(&xWaveFormat, &pcBuffer[iChunkOffset + 8], sizeof(WAVEFORMATEX) );	// +8 skips "fmt " and size

	if( xWaveFormat.wFormatTag != WAVE_FORMAT_PCM )
	{
		Asura_Assertf( false, ( "Expected PCM format data in WAV file %s", m_xWavFilename.GetString() ) );
		delete []pcBuffer;
		Asura_FileAccess::Close( xWAVFile );
		return false;
	}
	if( xWaveFormat.wBitsPerSample != 16 )
	{
		Asura_Assertf( false, ( "Expected 16 bit samples in WAV file %s", m_xWavFilename.GetString() ) );
		delete []pcBuffer;
		Asura_FileAccess::Close( xWAVFile );
		return false;
	}

	//Now that we have the header and have validated the format, find the data chunk
	iChunkOffset = FindChunkInWAVFileData(pcBuffer, iMaxToBuffer, CHUNK_data);
	if( iChunkOffset < 0 )
	{
		//Its possible (but unlikely) that the buffer is too small
		Asura_Assertf( false, ("Failed to find data chunk in wav file %s", m_xWavFilename.GetString() ) );
		delete []pcBuffer;
		Asura_FileAccess::Close( xWAVFile );
		return false;
	}

	int* piDataSize = reinterpret_cast<int*>( &pcBuffer[iChunkOffset] );

	//We can calculate the duration of the music from what we have now...
	m_fDuration = static_cast<float>( *piDataSize / ( xWaveFormat.nBlockAlign ) ) / static_cast<float>( xWaveFormat.nSamplesPerSec );
	m_uSamples = static_cast<u_int>( *piDataSize );

	if( ( ( m_fVisibleStart == 0.0f ) && ( m_fVisibleEnd == 0.0f ) ) || m_fVisibleEnd > m_fDuration )
	{
		m_fVisibleStart = 0.0f;
		m_fVisibleEnd	= m_fDuration;
	}

	delete []pcBuffer;
	Asura_FileAccess::Close( xWAVFile );

	m_bHasLoadedWAV = true;

	return true;
}

void Asura_WAVRender_Gadget::ThreadFuncLoadSamples ( const void* pData )
{
	Asura_WAVRender_Gadget* pxThis = reinterpret_cast<Asura_WAVRender_Gadget*>( const_cast<void*>( pData ) );
	
	Asura_String	xFilename			= pxThis->m_xWavFilename;
	float			fVisibleStart		= pxThis->m_fVisibleStart;
	float			fVisibleEnd			= pxThis->m_fVisibleEnd;
	int				iNumRenderSamples	= pxThis->m_iNumRenderSamples;

	pxThis->m_bFinishedLoadingSamples = false;

	const int iMaxToBuffer = 1024;

	//We cant open lots of files at once so wait for other gadgets to be finished
	bool bAllowedToOpenFile = false;
	while( !bAllowedToOpenFile )
	{
		m_xGadgetLoadFlagMutex.Lock();
		if( !m_bAnyGadgetIsLoading )
		{
			//I can load now
			m_bAnyGadgetIsLoading = true;
			bAllowedToOpenFile = true;
		}
		m_xGadgetLoadFlagMutex.Unlock();
		if( !bAllowedToOpenFile )
		{
			Asura_Thread_System::DelayCurrentThread( 0.1f );
		}
		if( pxThis->m_bAbortLoading )
		{
			pxThis->m_bFinishedLoadingSamples = true;
			return;
		}
	}

	Asura_Handle xWAVFile = Asura_FileAccess::OpenRaw( xFilename.GetString() );

	// Open WAV file for reading
	if ( xWAVFile == ASURA_INVALID_HANDLE )
	{
		Asura_Assertf( false, ( "Failed to open wav file %s", xFilename.GetString() ) );
		pxThis->m_bFinishedLoadingSamples = true;
		//Let any other gadgets know they may continue
		m_xGadgetLoadFlagMutex.Lock();
		m_bAnyGadgetIsLoading = false;
		m_xGadgetLoadFlagMutex.Unlock();
		return;
	}

	char* pcBuffer = new char[iMaxToBuffer];

	if( !Asura_FileAccess::ReadData( xWAVFile, pcBuffer, iMaxToBuffer ) )
	{
		Asura_Assertf( false, ( "Failed to read wav file %s", xFilename.GetString() ) );
		delete []pcBuffer;
		Asura_FileAccess::Close( xWAVFile );
		pxThis->m_bFinishedLoadingSamples = true;
		//Let any other gadgets know they may continue
		m_xGadgetLoadFlagMutex.Lock();
		m_bAnyGadgetIsLoading = false;
		m_xGadgetLoadFlagMutex.Unlock();
		return;
	}

	// Check for wave header
	int iChunkOffset = FindChunkInWAVFileData(pcBuffer, iMaxToBuffer, CHUNK_WAVE);
	if ( iChunkOffset < 0 )
	{
		Asura_Assertf(false, ("Missing 'WAVE' chunk from file %s", xFilename.GetString() ) );
		delete []pcBuffer;
		Asura_FileAccess::Close( xWAVFile );
		pxThis->m_bFinishedLoadingSamples = true;
		//Let any other gadgets know they may continue
		m_xGadgetLoadFlagMutex.Lock();
		m_bAnyGadgetIsLoading = false;
		m_xGadgetLoadFlagMutex.Unlock();
		return;
	}

	WAVEFORMATEX xWaveFormat;

	Asura_Mem::Copy(&xWaveFormat, &pcBuffer[iChunkOffset + 8], sizeof(WAVEFORMATEX) );	// +8 skips "fmt " and size

	//find the data chunk
	iChunkOffset = FindChunkInWAVFileData(pcBuffer, iMaxToBuffer, CHUNK_data);
	if( iChunkOffset < 0 )
	{
		//Its possible (but unlikely) that the buffer is too small
		Asura_Assertf( false, ("Failed to find data chunk in wav file %s", xFilename.GetString() ) );
		delete []pcBuffer;
		Asura_FileAccess::Close( xWAVFile );
		pxThis->m_bFinishedLoadingSamples = true;
		//Let any other gadgets know they may continue
		m_xGadgetLoadFlagMutex.Lock();
		m_bAnyGadgetIsLoading = false;
		m_xGadgetLoadFlagMutex.Unlock();
		return;
	}

	float* afHighRenderSamples	= new float [iNumRenderSamples];
	float* afLowRenderSamples	= new float [iNumRenderSamples];

	Asura_Mem::Set( afHighRenderSamples, 0, sizeof(float) * iNumRenderSamples );
	Asura_Mem::Set( afLowRenderSamples , 0, sizeof(float) * iNumRenderSamples );

	const float fVisible				= fVisibleEnd - fVisibleStart;
	const float fSamplingTimeStep		= fVisible / static_cast<float>(iNumRenderSamples);
	const int	iVisibleStartSamples	= static_cast<int>(fVisibleStart * static_cast<float>( xWaveFormat.nSamplesPerSec ) * xWaveFormat.nBlockAlign);
	int			iSamplesToTake			= iMaxToBuffer /  xWaveFormat.nBlockAlign;

	// If the total number of samples is less than the number of samples cumlatively read for our algorithm pick a smaller number to read
	if( static_cast<float>( xWaveFormat.nSamplesPerSec ) * fVisible < ( iSamplesToTake * iNumRenderSamples ) )
	{
		iSamplesToTake = static_cast<int>( fSamplingTimeStep * static_cast<float>( xWaveFormat.nSamplesPerSec ) * 1.05f );
	}

	struct Sample16Bit_Stereo
	{
		short sLeftChannel;
		short sRightChannel;
	};

	struct Sample16Bit_Mono
	{
		short sMonoChannel;
	};

	float fLastSwapTime = Asura_Timers::GetLocalUnscaledTime();

	for( int i = 0; i < iNumRenderSamples; ++i )
	{
		//Offset to the start of the data chunk, past the header info, up to the visible start time
		int iSeekPos = iChunkOffset + 4 + iVisibleStartSamples;
		//seek up to the desired offset
		iSeekPos += static_cast<int>( fSamplingTimeStep * static_cast<float>( i ) * static_cast<float>( xWaveFormat.nSamplesPerSec ) * static_cast<float>( xWaveFormat.nBlockAlign ) );
		//Check that our seek position lies on a sample boundry, if not then shift it
		const int iShift = ( iSeekPos - (iChunkOffset + 4) ) % xWaveFormat.nBlockAlign;
		iSeekPos -= iShift;

		//now we can read some samples
		if(	! Asura_FileAccess::SeekFile( xWAVFile, static_cast<u_int>(iSeekPos) ) ||
			! Asura_FileAccess::ReadData( xWAVFile, pcBuffer, iSamplesToTake * xWaveFormat.nBlockAlign ) )
		{
			// If we fail to read or seek its probably just the slight error from our algorithm, ignore it
			// okay this isnt perfect but in light of a more sophisticated waveform renderer it will do
			break;
		}

		//use the highest amplitude of all the samples we have read
		float	fHighSample = -32767.0f;
		float	fLowSample = 32768.0f;

		if( xWaveFormat.nChannels == 2 )
		{
			Sample16Bit_Stereo* pxSample = reinterpret_cast<Sample16Bit_Stereo*>( pcBuffer );
			for( int iSample = 0; iSample < iSamplesToTake; ++iSample )
			{
				fHighSample = Asura_Maths::Max( static_cast<float>(pxSample->sLeftChannel), fHighSample );
				fLowSample = Asura_Maths::Min( static_cast<float>(pxSample->sLeftChannel), fLowSample );
				++pxSample;
			}
		}
		else if ( xWaveFormat.nChannels == 1 )
		{
			Sample16Bit_Mono* pxSample = reinterpret_cast<Sample16Bit_Mono*>( pcBuffer );
			for( int iSample = 0; iSample < iSamplesToTake; ++iSample )
			{
				fHighSample = Asura_Maths::Max( static_cast<float>(pxSample->sMonoChannel), fHighSample );
				fLowSample = Asura_Maths::Min( static_cast<float>(pxSample->sMonoChannel), fLowSample );
				++pxSample;
			}
		}

		fHighSample /= 32767.0f;
		fLowSample /= 32768.0f;

		afHighRenderSamples	[i] = fHighSample;
		afLowRenderSamples	[i] = fLowSample;

		//See if we should swap
		float fNewTime = Asura_Timers::GetLocalUnscaledTime();
		if( ( fNewTime - fLastSwapTime ) > fASURA_WAVALIGN_LOAD_DATA_SWAP_INTERVAL )
		{
			fLastSwapTime = fNewTime;

			pxThis->m_xRenderSamplesMutex.Lock();

			Asura_Mem::Copy( pxThis->m_afHighRenderSamples, afHighRenderSamples, iNumRenderSamples * sizeof( float ) );
			Asura_Mem::Copy( pxThis->m_afLowRenderSamples , afLowRenderSamples , iNumRenderSamples * sizeof( float ) );

			pxThis->m_iThreadedReadProgress = i;

			pxThis->m_xRenderSamplesMutex.Unlock();
		}

		if( pxThis->m_bAbortLoading )
		{
			pxThis->m_bAbortLoading = false;
			break;
		}
	}
	Asura_FileAccess::Close( xWAVFile );

	//Let any other gadgets know they may continue
	m_xGadgetLoadFlagMutex.Lock();
	m_bAnyGadgetIsLoading = false;
	m_xGadgetLoadFlagMutex.Unlock();

	//Write out the final samples
	pxThis->m_xRenderSamplesMutex.Lock();

	Asura_Mem::Copy( pxThis->m_afHighRenderSamples, afHighRenderSamples, iNumRenderSamples * sizeof( float ) );
	Asura_Mem::Copy( pxThis->m_afLowRenderSamples , afLowRenderSamples , iNumRenderSamples * sizeof( float ) );

	pxThis->m_iThreadedReadProgress = iNumRenderSamples;

	pxThis->m_xRenderSamplesMutex.Unlock();

	delete []pcBuffer;
	delete []afHighRenderSamples;
	delete []afLowRenderSamples;

	pxThis->m_bFinishedLoadingSamples = true;
}

#endif

//eof
