//////////////////////////////////////////////////////////////////////////////
//
// Asura_Job_PFX.h
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_JOB_PFX_H___
#define ___ASURA_JOB_PFX_H___ 1

#ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE

// I N C L U D E S ///////////////////////////////////////////////////////////
#include "Core/Asura_Maths.h"
#include "Rendering/Asura_Colour.h"

// C O N S T A N T S /////////////////////////////////////////////////////////
#define MAX_CAMERAS_FOR_PFX_JOB (4)

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////
class Asura_Job_Queue;
class Asura_PFX_Particle;
class Asura_PFX_Source_Template;
template <typename T> class Asura_Spline;

// C L A S S E S /////////////////////////////////////////////////////////////
class Asura_Job_PFX_Update
{
public:

	Asura_Job_PFX_Update()
		: m_pxParticles(NULL)
		, m_uNumParticles(0)
		, m_pxTemplate(NULL)
		, m_pSourceTemplateDataBlock(NULL)
		, m_pParticleTemplateDataBlock(NULL)
		, m_pxSpline(NULL)
		, m_pPlatformRenderData(NULL)
	{}

	void InitialiseData(Asura_PFX_Particle* pxParticles, u_int uNumParticles, u_int uNumParticlesToUpdate, 
					const Asura_PFX_Source_Template* pxSourceTemplate, 
					bool bUseCollisionPlane, const Asura_Vector_3& xCollisionPosition, const Asura_Vector_3& xCollisionDirection, 
					bool bUsePointGravity, const Asura_Vector_3& xGravityPosition, float fSizeScale, void* pRenderData, 
					const Asura_RGBA& xLightColour, const Asura_Matrix_3x3& xEffectOrientation, u_int uCameraFlags, const Asura_Spline<Asura_Vector_3>* pxSpline );
	void Execute();
	void AddToQueue();
	bool IsValid() const
	{
		return m_pxParticles != NULL;
	}

	static void FinishAllJobs();
	static void InitialiseAllocations();
	static void ResetAllocations();
	static void FreeAllocations();


	Asura_PFX_Particle*							GetParticles() const { return m_pxParticles; }
	unsigned int								GetNumParticles() const { return m_uNumParticles; }
	unsigned int								GetNumParticlesToUpdate() const { return m_uNumParticlesToUpdate; }
	const Asura_PFX_Source_Template*			GetTemplate() const { return m_pxTemplate; }
	const void*									GetSourceTemplateDataBlock() const { return m_pSourceTemplateDataBlock; }
	u_int										GetSourceTemplateDataBlockSize() const { return m_uSourceTemplateDataBlockSize; }
	const void*									GetParticleTemplateDataBlock() const { return m_pParticleTemplateDataBlock; }
	u_int										GetParticleTemplateDataBlockSize() const { return m_uParticleTemplateDataBlockSize; }
	bool 										GetUseCollisionPlane() const { return m_bUseCollisionPlane; }
	bool 										GetUsePointGravity() const { return m_bUsePointGravity; }
	const Asura_Vector_3&						GetCollisionPosition() const { return m_xCollisionPosition; }
	const Asura_Vector_3&						GetCollisionDirection() const { return m_xCollisionDirection; }
	const Asura_Vector_3&						GetGravityPosition() const { return m_xGravityPosition; }
	float										GetSizeScale() const { return m_fSizeScale; }
	float										GetTimeSlice() const { return m_fTimeSlice; }
	float										GetTurbulencePhase() const { return m_fTurbulencePhase; }
	void*										GetPlatformRenderData() const { return m_pPlatformRenderData; }
	const Asura_RGBA&							GetLightColour() const { return m_xLightColour; }
	const Asura_Matrix_3x3&						GetEffectOrientation() const { return m_xEffectOrientation; }
	const Asura_Vector_3&						GetCameraPosition(int iCamera) const { return m_axCameraPosition[iCamera]; }
	const Asura_Vector_3&						GetCameraDeltaX(int iCamera) const { return m_axCameraDeltaX[iCamera]; }
	const Asura_Vector_3&						GetCameraDeltaY(int iCamera) const { return m_axCameraDeltaY[iCamera]; }
	int											GetNumberOfCameras() const { return m_iNumberOfCameras; }
	u_int										GetCameraFlags() const { return m_uCameraFlags; }

private:

	// member data
	Asura_PFX_Particle*							m_pxParticles;
	unsigned int								m_uNumParticles;
	unsigned int								m_uNumParticlesToUpdate;
	const Asura_PFX_Source_Template*			m_pxTemplate;
	const void*									m_pSourceTemplateDataBlock;
	u_int										m_uSourceTemplateDataBlockSize;
	const void*									m_pParticleTemplateDataBlock;
	u_int										m_uParticleTemplateDataBlockSize;
	bool 										m_bUseCollisionPlane;
	bool 										m_bUsePointGravity;
	Asura_Vector_3 								m_xCollisionPosition;
	Asura_Vector_3 								m_xCollisionDirection;
	Asura_Vector_3 								m_xGravityPosition;
	float										m_fSizeScale;
	float										m_fTimeSlice;
	float										m_fTurbulencePhase;
	const Asura_Spline<Asura_Vector_3>*			m_pxSpline;
	// extra data for rendering
	void*										m_pPlatformRenderData;
	Asura_RGBA									m_xLightColour;
	Asura_Matrix_3x3							m_xEffectOrientation;
	Asura_Vector_3								m_axCameraPosition[MAX_CAMERAS_FOR_PFX_JOB];
	Asura_Vector_3								m_axCameraDeltaX[MAX_CAMERAS_FOR_PFX_JOB];
	Asura_Vector_3								m_axCameraDeltaY[MAX_CAMERAS_FOR_PFX_JOB];
	int											m_iNumberOfCameras;
	u_int										m_uCameraFlags;


	static bool ExecuteJob(void *pxJob);
	static Asura_Job_Queue* s_pxQueue;
#ifdef	SN_TARGET_PS3
} __attribute__((aligned(16)));
#else
};
#endif

#endif // #ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_JOB_PFX_H___
