//*******************************************************************
//
// File: Asura_Job_TaskQueue.h
//
// Created: 27 June 2007 DLH
//
// Description: 
//
//*******************************************************************

#ifndef __ASURA_JOB_TASKQUEUE_H__
#define __ASURA_JOB_TASKQUEUE_H__

#if defined( ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE)
// other libraries

// internal dependencies

// forward declarations
class Asura_Job_TaskQueue;

//*******************************************************************
// Constants
//*******************************************************************

// DLH - This is used to add locks to the Queue access. It is enabled until I can find and fix the annoying bugs in the lock-free algorithum
//#define ASURA_JOB_TASK_QUEUE_USE_LOCK

// DLH - This is used to store a log of all the ins and outs of the Pop() and Push() of each list. If something goes wrong this is invaluble information. Just ashame I have not worked out any nice way of formatiing the data for human read ablility! :(
//#define ASURA_JOB_TASK_QUEUE_DEBUG

// DLH - Which Lock-Free algorithum to use.
//#define ASURA_JOB_TASK_QUEUE_JDV
#define ASURA_JOB_TASK_QUEUE_TAG

//*******************************************************************
// Class Definitions
//*******************************************************************

// ---------------------------------------------------
// Asura_Job_TaskQueue
// 
// This is a thread safe queue of tasks
// Tasks can be push and poped.
// Or they can be removed, have their data copied or used and then gived back to the list.
// The idea being that no un-nessary data coping is going on
//
// The Job system when doing a job does the following:
// 1) Pop the Task data out of the Queue. This Task entry is not free to be re-used. But is not on the Queue anymore.
// 2) Process the Task data
// 3) Return the Task data to the Queue as a free item for reuse.
// ---------------------------------------------------
class Asura_Job_TaskQueue
{
public:
	class Task;
	
private:
	class TaskQueuePointer
	{
	public:
		TaskQueuePointer();
		TaskQueuePointer( const TaskQueuePointer& xPointer);
		TaskQueuePointer( Task* pxTask, Task* pxTaskBase);
		TaskQueuePointer( Task* pxTask, Task* pxTaskBase, u_int uTag);

		TaskQueuePointer& operator =( TaskQueuePointer& xPointer);
		//Task* operator =( Task* pxTask);
		
		bool operator ==( TaskQueuePointer& xPointer);
		
		Task* GetPointer( Task* pxTaskBase);
		Task* SetPointer( Task* pxTask, Task* pxTaskBase);

#if defined( ASURA_JOB_TASK_QUEUE_JDV)
		Task* volatile &GetAtomic();
#endif
#if defined( ASURA_JOB_TASK_QUEUE_TAG)
		uint32 volatile &GetAtomic();
		u_int GetTag();
		void SetTag( u_int uTag);
#endif		

private:
#if defined( ASURA_JOB_TASK_QUEUE_TAG)
		uint16 PointerToMem( Task* pxTask, Task* pxTaskBase);
		Task* MemToPointer( uint16 uTag, Task* pxTaskBase);
#endif		

		
	private:
#if defined( ASURA_JOB_TASK_QUEUE_JDV)
		Task* volatile m_pxTask;
#endif
#if defined( ASURA_JOB_TASK_QUEUE_TAG)
		union Uint32Data
		{
			struct
			{
				uint16 m_uTag;
				uint16 m_uMem;
			}m_xData32;
			uint32 m_uData32;
		};
		Uint32Data volatile m_xData;
#endif
	};


public:
	class Task
	{
	public:
		void* GetData(){ return( this->m_pData);}
		
	private:
		void CS_AssertTaskValid();
		
	private:
		TaskQueuePointer m_xNext;
		void* m_pData;
		
		friend class Asura_Job_TaskQueue;
	};
	
private:
	class TaskQueue
	{
	public:
		TaskQueuePointer m_xTasksQueueStart;
		TaskQueuePointer m_xTasksQueueLast;
		
		volatile u_int m_uTasksQueueSize;
#if defined( ASURA_JOB_TASK_QUEUE_USE_LOCK)	
		volatile u_int m_uLocked;
#endif

#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
		struct Debug
		{
			Debug();
			
			enum Type
			{
				UNSET,
				POP_START,
				POP_END,
				PUSH_START,
				PUSH_END,
			};
			u_int m_uTransactionID;
			Task* m_pxTask;
			TaskQueuePointer m_xTaskStart;
			TaskQueuePointer m_xTaskEnd;
			u_int m_uSize;
			u_int m_uActive;
			Type m_eTransactionType;
		};
		
		u_int m_uDebugIndex;
		u_int m_uDebugActive;
		static const u_int s_uDebugSize = 2048;
		Debug m_axDebug[ s_uDebugSize];
		
		u_int StartPop();
		void EndPop( u_int uTransactionID, Task* pxTask);
		
		u_int StartPush( Task* pxTask);
		void EndPush( u_int uTransactionID);
#endif
	};
	
public:
	Asura_Job_TaskQueue( u_int uTaskNumber, u_int uDataSize);
	~Asura_Job_TaskQueue();
	
	void Clear();
	
	bool PopTask( Task* &pxTask);
	bool ReturnTask( Task* &pxTask);
	
	bool Pop( void* pData);
	bool Push( const void* pData);
	
	u_int GetNumberOfQueuedTasks() const;
	u_int GetNumberOfFreeTasks() const;
	
	bool IsIdle() const;
	
	void SetName( const char* pszName);
	char* GetName() { return( this->m_szName);}
	
	bool IsValid();
	
private:
	Task* CS_QueuePop();
	void CS_QueuePush( Task* pxTask);
	
	Task* CS_FreePop();
	void CS_FreePush( Task* pxTask);

	Task* CS_Pop( TaskQueue &xQueue);
	void CS_Push( TaskQueue &xQueue, Task* pxTask);
	
#if defined( ASURA_JOB_TASK_QUEUE_DEBUG)
	bool IsValid( TaskQueue &xQueue);
#endif
	
private:

	const u_int m_uTaskNumber;
	const u_int m_uDataSize;
	
	Task* m_paxTasks;
	uint8* m_pucData;

	TaskQueue m_xTaskQueue;
	TaskQueue m_xFreeQueue;
	
	u_int m_uFreeTaskSize;
	
	char m_szName[ 32];
};


//*******************************************************************
// Inline Function Definitions
//*******************************************************************


//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE)

#endif // __ASURA_JOB_TASKQUEUE_H__
