//////////////////////////////////////////////////////////////////////////////
//
// Asura_LipSync_Manager.cpp
//
// Created by Stuart Middleton - 20 June 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////


#include "Core/Asura.h"

#ifdef ASURA_USE_ARTICULATION_SYSTEM

#include "LipSync/Asura_Lipsync_Manager.h"

#include "ChunkSystem/Asura_chunk_lipsync.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Debugging/Asura_Logfile.h"
#include "Sound/Asura_Dialogue.h"

#define ASURA_LIPSYNC_ARTICULATION_VARNAME(x, y) (u_int)((x << 16) + y )

#ifdef ASURA_TOOLS
#include "PC/FileSystem/Tools_FileSystem.h"
#endif // ASURA_TOOLS

Asura_LipSync_Articulation* Asura_LipSync_Manager::s_pxBase = NULL;
Asura_LipSync_Articulation_Instance* Asura_LipSync_Manager::s_apxPlayingArticulations[ASURA_LIPSYNC_MANAGER_MAX_PLAYING_ARTICULATIONS];
u_int Asura_LipSync_Manager::s_auPhonemeList[] = {	ASURA_LIPSYNC_PHONEME_X, ASURA_LIPSYNC_PHONEME_M, ASURA_LIPSYNC_PHONEME_P,
ASURA_LIPSYNC_PHONEME_B, ASURA_LIPSYNC_PHONEME_V, ASURA_LIPSYNC_PHONEME_F, ASURA_LIPSYNC_PHONEME_ZH, ASURA_LIPSYNC_PHONEME_CH, 
ASURA_LIPSYNC_PHONEME_J, ASURA_LIPSYNC_PHONEME_SH, ASURA_LIPSYNC_PHONEME_S, ASURA_LIPSYNC_PHONEME_Z, ASURA_LIPSYNC_PHONEME_DH, 
ASURA_LIPSYNC_PHONEME_TH, ASURA_LIPSYNC_PHONEME_T, ASURA_LIPSYNC_PHONEME_NG, ASURA_LIPSYNC_PHONEME_G, ASURA_LIPSYNC_PHONEME_K, 
ASURA_LIPSYNC_PHONEME_H, ASURA_LIPSYNC_PHONEME_R, ASURA_LIPSYNC_PHONEME_ER, ASURA_LIPSYNC_PHONEME_W, ASURA_LIPSYNC_PHONEME_UH, 
ASURA_LIPSYNC_PHONEME_UW, ASURA_LIPSYNC_PHONEME_AW, ASURA_LIPSYNC_PHONEME_IY, ASURA_LIPSYNC_PHONEME_Y, ASURA_LIPSYNC_PHONEME_IH, 
ASURA_LIPSYNC_PHONEME_AE, ASURA_LIPSYNC_PHONEME_AY, ASURA_LIPSYNC_PHONEME_EY, ASURA_LIPSYNC_PHONEME_EH, ASURA_LIPSYNC_PHONEME_N, 
ASURA_LIPSYNC_PHONEME_D, ASURA_LIPSYNC_PHONEME_L, ASURA_LIPSYNC_PHONEME_AA, ASURA_LIPSYNC_PHONEME_AH, ASURA_LIPSYNC_PHONEME_AO, 
ASURA_LIPSYNC_PHONEME_OW, ASURA_LIPSYNC_PHONEME_OY};

int Asura_LipSync_Manager::s_iNumPlayingArticulations = 0;

Asura_Lipsync_PhonemeViseme_Map_Container* Asura_Lipsync_PhonemeViseme_Map_Manager::s_pxMapContainer = NULL;

void Asura_LipSync_Manager::Initialise(void)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER("Asura_LipSync_Manager::Initialise");

	s_pxBase = NULL;
	s_iNumPlayingArticulations = 0;

	Asura_Lipsync_PhonemeViseme_Map_Manager::Initialise();

	Asura_Lipsync_PhonemeViseme_Map_Manager::CreatePhonemeVisemeMap();

	for(int i =0; i < ASURA_LIPSYNC_MANAGER_MAX_PLAYING_ARTICULATIONS; i++)
	{	
		s_apxPlayingArticulations[i] = NULL;
	}

#ifdef ASURA_TOOLS
	List<Asura_String> xOutputList;
	// TEMP needs to fetch current language directory
	Tools_FileSystem::FindFiles( "Sounds\\Speech\\en\\*.asr", xOutputList, (TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR) );
	for(int i = 0; i < xOutputList.size(); i++)
	{
		Asura_String xFilename;
		xOutputList.get_entry(i).SplitFilename(NULL, &xFilename, NULL);
		Asura_LipSync_Manager::CreateNewArticulation(xFilename.GetHash(), xOutputList.get_entry(i).GetString());
//		LoadAsuraFile(xOutputList.get_entry(i).GetString());
	}
#endif
}

void Asura_LipSync_Manager::Reset(void)
{
	Asura_LipSync_Articulation* pxDelete = s_pxBase;
	Asura_LipSync_Articulation* pxNext;
	Asura_LipSync_Articulation* pxPrev = NULL;

	while (pxDelete)
	{
		pxNext = pxDelete->m_pxNext;

		if (!pxDelete->IsPermanent())
		{
			delete pxDelete;
			if (pxPrev)
			{
				pxPrev->m_pxNext = pxNext;
			}
			if (s_pxBase == pxDelete)
			{
				s_pxBase = pxNext;
			}
		}
		else
		{
			pxPrev = pxDelete;
		}

		pxDelete = pxNext;
	}


	ClearPlayingList();

	Asura_Lipsync_PhonemeViseme_Map_Manager::Reset();
}

void Asura_LipSync_Manager::Shutdown()
{
	Asura_LipSync_Articulation* pxDelete = s_pxBase;
	Asura_LipSync_Articulation* pxNext;

	while (pxDelete)
	{
		pxNext = pxDelete->m_pxNext;
		delete pxDelete;
		pxDelete = pxNext;
	}

	s_pxBase = NULL;

	ClearPlayingList();

	Asura_Lipsync_PhonemeViseme_Map_Manager::Shutdown();
}

void Asura_Lipsync_PhonemeViseme_Map_Manager::Initialise()
{
	s_pxMapContainer = NULL;
}

void Asura_Lipsync_PhonemeViseme_Map_Manager::Reset()
{
	delete s_pxMapContainer;
	s_pxMapContainer = NULL;
}

void Asura_Lipsync_PhonemeViseme_Map_Manager::Shutdown()
{
	delete s_pxMapContainer;
	s_pxMapContainer = NULL;
}

Asura_LipSync_Articulation* Asura_LipSync_Manager::CreateNewArticulation( const Asura_Hash_ID uHash, const Asura_Char* pszFilename /*=NULL*/ )
{
	if(GetArticulationByHash(uHash) != NULL)
	{
		DeleteArticulationByHash(uHash);
	}

	Asura_LipSync_Articulation* pxNew = new Asura_LipSync_Articulation();
	pxNew->m_uHash = uHash;
	pxNew->m_pxNext = NULL;

	if(!s_pxBase)
	{	s_pxBase = pxNew;}
	else
	{
		pxNew->m_pxNext = s_pxBase;
		s_pxBase = pxNew;
	}

#ifdef ASURA_TOOLS
	pxNew->m_pszFilename = pszFilename;
#endif

	return pxNew;
}

Asura_LipSync_Articulation* Asura_LipSync_Manager::GetArticulationByHash( const Asura_Hash_ID uHash )
{
	Asura_LipSync_Articulation* pxCurr = s_pxBase;
	while(pxCurr != NULL)
	{
		if(pxCurr->m_uHash == uHash)
		{	return pxCurr;}
		pxCurr = pxCurr->m_pxNext;
	}
	return NULL;
}

bool Asura_LipSync_Manager::DeleteArticulationByHash( Asura_Hash_ID uHash )
{
	Asura_LipSync_Articulation* pxCurr = s_pxBase;
	Asura_LipSync_Articulation* pxPrev = NULL;
	while(pxCurr != NULL)
	{
		if(pxCurr->m_uHash == uHash)
		{
			if(pxPrev == NULL)
			{
				s_pxBase = pxCurr->m_pxNext;
			}
			else
			{
				pxPrev->m_pxNext = pxCurr->m_pxNext;
			}
			Asura_Assert(!pxCurr->IsPermanent(), "deleting permanent lip sync articulation");
			delete pxCurr;
			return true;
		}
		pxPrev = pxCurr;
		pxCurr = pxCurr->m_pxNext;
	}

	return false;
}

int Asura_LipSync_Manager::GetNumPlayingArticulations()
{
	int iCount = 0;
	for(int i=0; i < ASURA_LIPSYNC_MANAGER_MAX_PLAYING_ARTICULATIONS; i++)
	{
		if(s_apxPlayingArticulations[i] != NULL)
		{
			iCount++;
		}
	}

	return iCount;
}

Asura_LipSync_Articulation_Instance* Asura_LipSync_Manager::GetPlayingArticulationByIndex( const int iIndex ) 
{
	return s_apxPlayingArticulations[iIndex];
}

Asura_LipSync_Articulation_Instance* Asura_LipSync_Manager::GetPlayingArticulationByOwner( const Asura_Hash_ID uGuid )
{
#ifdef _DEBUG
	Asura_Assert(s_iNumPlayingArticulations == GetNumPlayingArticulations(), "Error s_iNumPlayingArticulations is incorrect");
#endif //_DEBUG

	if (s_iNumPlayingArticulations == 0)
	{
		return NULL;
	}

	// Count playing articulations.
	int iCount = 0;

	for(int i=0; i<ASURA_LIPSYNC_MANAGER_MAX_PLAYING_ARTICULATIONS; i++)
	{
		if (s_apxPlayingArticulations[i])
		{
			if (s_apxPlayingArticulations[i]->GetSpeakerGuid() == uGuid)
			{
				return s_apxPlayingArticulations[i];
			}

			++iCount;
			if (iCount == s_iNumPlayingArticulations)
			{
				// Early exit. We have checked all the playing articulations.
				return NULL;
			}
		}
	}
	return NULL;
}

void Asura_LipSync_Manager::ClearPlayingList()
{
	for(int i =0; i < ASURA_LIPSYNC_MANAGER_MAX_PLAYING_ARTICULATIONS; i++ )
	{
		if(s_apxPlayingArticulations[i] != NULL)
		{
			delete s_apxPlayingArticulations[i];
			s_apxPlayingArticulations[i] = NULL;
		}
	}

	s_iNumPlayingArticulations = 0;
}

int Asura_LipSync_Manager::AddToPlayingList( Asura_LipSync_Articulation_Instance* const pxInst )
{
	for(int i=0; i < ASURA_LIPSYNC_MANAGER_MAX_PLAYING_ARTICULATIONS; i++)
	{
		if(s_apxPlayingArticulations[i] == NULL)
		{
			s_apxPlayingArticulations[i] = pxInst;
			++s_iNumPlayingArticulations;

//			Asura_OutputToDebugger_Retail(("%u: add: index=%d guid=%u  (num=%d)\n", Asura_Timers::GetFrameNumber(), i, (pxInst ? pxInst->GetSpeakerGuid() : 0), s_iNumPlayingArticulations));

			return i;
		}
	}
	return -1;
}

void Asura_LipSync_Manager::RemoveFromPlayingList(int iIndex)
{
	if ( iIndex < 0 )
	{	
		return;
	}
	if ( iIndex >= ASURA_LIPSYNC_MANAGER_MAX_PLAYING_ARTICULATIONS )
	{
		return;
	}

	Asura_Assert(s_apxPlayingArticulations[iIndex] != NULL, "Tried to remove an articulation which isn't playing.");
	if(s_apxPlayingArticulations[iIndex] != NULL)
	{
//		Asura_OutputToDebugger_Retail(("%u: remove: index=%d guid=%u  (num=%d)\n", Asura_Timers::GetFrameNumber(), iIndex, s_apxPlayingArticulations[iIndex]->GetSpeakerGuid(), s_iNumPlayingArticulations-1));

		delete s_apxPlayingArticulations[iIndex];
		s_apxPlayingArticulations[iIndex] = NULL;
		--s_iNumPlayingArticulations;
		
		Asura_Assert(s_iNumPlayingArticulations>=0, "Error s_iNumPlayingArticulations is negative");
	}
}

void Asura_Lipsync_PhonemeViseme_Map_Manager::DeletePhonemeVisemeMap()
{
	if(s_pxMapContainer)
	{
		delete s_pxMapContainer;
		s_pxMapContainer = NULL;
	}
}

Asura_Lipsync_PhonemeViseme_Map_Container* Asura_Lipsync_PhonemeViseme_Map_Manager::CreatePhonemeVisemeMap()
{
	// Create a blank mapping table which can be filled in by the game with viseme mappping for each phoneme
	Asura_Assert( s_pxMapContainer == NULL, "" );

	s_pxMapContainer = new Asura_Lipsync_PhonemeViseme_Map_Container();

	for(int i=0; i<ASURA_LIPSYNC_PHONEME_COUNT; i++)
	{
		s_pxMapContainer->m_axMap[i].m_uPhoneme = Asura_LipSync_Manager::s_auPhonemeList[i];
		s_pxMapContainer->m_axMap[i].m_uVisemeHash = 0;
		s_pxMapContainer->m_axMap[i].m_uVisemeIndex = 0;
	}

	s_pxMapContainer->m_uNumPhonemes = ASURA_LIPSYNC_PHONEME_COUNT;
	s_pxMapContainer->m_uNumVisemes = 0;

	return s_pxMapContainer;
}

bool Asura_Lipsync_PhonemeViseme_Map_Manager::AddPhonemeToVisemeMapping(u_int uPhoneme, Asura_Hash_ID uViseme, void* pxData, char* szName)
{
	Asura_Assert(Asura_LipSync_Manager::s_auPhonemeList != NULL, "No Phoneme map defined");
	if(!Asura_LipSync_Manager::s_auPhonemeList)
	{	return false;}

	// Figure out if this is a new viseme to the system
	bool bFound = false;
	for(int i=0; i<ASURA_LIPSYNC_PHONEME_COUNT; i++)
	{
		if(s_pxMapContainer->m_axMap[i].m_uVisemeHash == uViseme)
		{
			bFound = true;
		}
	}
	if(!bFound)
	{
		// Increment the number of visemes and add the new viseme to the viseme array
		s_pxMapContainer->m_axVisemeList[s_pxMapContainer->m_uNumVisemes].m_uVisemeHash = uViseme;
		s_pxMapContainer->m_axVisemeList[s_pxMapContainer->m_uNumVisemes].m_xVisemeName = szName;
		s_pxMapContainer->m_uNumVisemes++;
	}

	// Map the viseme to the phoneme
	for(int i=0; i<ASURA_LIPSYNC_PHONEME_COUNT; i++)
	{
		if(s_pxMapContainer->m_axMap[i].m_uPhoneme == uPhoneme)
		{
			s_pxMapContainer->m_axMap[i].m_uVisemeHash = uViseme;

			// fill in the index for quicker reference
			for(u_int k=0; k<s_pxMapContainer->m_uNumVisemes; k++)
			{
				if(s_pxMapContainer->m_axVisemeList[k].m_uVisemeHash == uViseme)
				{
					s_pxMapContainer->m_axMap[i].m_uVisemeIndex = k;
					break;
				}
			}

			return true;
		}
	}

	Asura_Assert(false,"Phoneme passed to mapping does not match any phoneme in system");
	return false;
}

const Asura_Lipsync_PhonemeViseme_Map* Asura_Lipsync_PhonemeViseme_Map_Manager::GetPhonemeMapByIndex( const u_int uPhonemeIndex )
{
	if(uPhonemeIndex < s_pxMapContainer->m_uNumPhonemes)
	{	return &s_pxMapContainer->m_axMap[uPhonemeIndex];}

	return NULL;
}

Asura_LipSync_Viseme_FrameData* Asura_Lipsync_PhonemeViseme_Map_Manager::MapPhonemeFrameToVisemeFrame( const Asura_LipSync_Articulation_FrameData* const pxFrame )
{
	Asura_LipSync_Viseme_FrameData* pxVisFrame = new Asura_LipSync_Viseme_FrameData();

	pxVisFrame->m_pxVisemeData = new Asura_LipSync_Articulation_VisemeData[s_pxMapContainer->m_uNumVisemes];

	// Clear out the weighting array
	for(u_int i = 0; i < s_pxMapContainer->m_uNumVisemes; i++)
	{
		pxVisFrame->m_pxVisemeData[i].m_fWeight = 0.0f;
		pxVisFrame->m_pxVisemeData[i].m_uViseme = s_pxMapContainer->m_axVisemeList[i].m_uVisemeHash;
	}

	pxVisFrame->m_uNumConsts = s_pxMapContainer->m_uNumVisemes;
	pxVisFrame->m_uMSStart = pxFrame->m_uMSStart;
	pxVisFrame->m_uMSEnd = pxFrame->m_uMSEnd;

	for(u_int j = 0; j < pxFrame->m_uNumConsts; j++)
	{
		const Asura_LipSync_Articulation_PhononData* pxData = &pxFrame->m_pxPhononData[j];
		pxVisFrame->m_pxVisemeData[s_pxMapContainer->m_axMap[pxData->m_uPhoneme].m_uVisemeIndex].m_fWeight += pxData->m_fWeight;
	}

	return pxVisFrame;
}

const char* Asura_Lipsync_PhonemeViseme_Map_Manager::GetVisemeName( Asura_Hash_ID uViseme )
{
	for (u_int i=0; i<s_pxMapContainer->m_uNumVisemes; i++)
	{
		if(s_pxMapContainer->m_axVisemeList[i].m_uVisemeHash == uViseme)
		{
			return s_pxMapContainer->m_axVisemeList[i].m_xVisemeName.GetString();
		}
	}

	return NULL;
}

u_int Asura_LipSync_Manager::GetPhonemeIndex( u_int uPhoneme )
{
	for(u_int i=0; i < ASURA_LIPSYNC_PHONEME_COUNT; i++)
	{
		if(uPhoneme == s_auPhonemeList[i])
		{	
			return i;
		}
	}

	Asura_Assert(false, "Cannot find phoneme in list");
	return 0;
}

Asura_LipSync_Articulation_FrameData::Asura_LipSync_Articulation_FrameData()
: m_uMSStart(0)
, m_uMSEnd(0)
, m_uNumConsts(0)
, m_pxPhononData(NULL)
{
}

Asura_LipSync_Articulation_FrameData::~Asura_LipSync_Articulation_FrameData()
{
	if(m_pxPhononData)
	{	
		delete[] m_pxPhononData;
	}
}


Asura_LipSync_Articulation::Asura_LipSync_Articulation()
{
	m_pxNext = NULL;
	m_uHash = ASURA_HASH_ID_UNSET;
	m_uFrames = 0;
	m_bPermanent = false;
	m_pxFrameData = NULL;
}

Asura_LipSync_Articulation::~Asura_LipSync_Articulation()
{
	if(m_pxFrameData)
	{	
		delete[] m_pxFrameData;
	}
}

int Asura_LipSync_Articulation::Start(const Asura_Handle iSoundHandle, const Asura_Guid uSpeakerGuid)
{
	Asura_LipSync_Articulation_Instance* pxInstance = new Asura_LipSync_Articulation_Instance();
	pxInstance->m_pxArticulation = this;
	pxInstance->m_uSpeakerGuid = uSpeakerGuid;
	pxInstance->m_iSoundHandle = iSoundHandle;
	pxInstance->m_bStarted = false;
	pxInstance->m_uMSStart = static_cast<u_int>((Asura_Timers::GetLocalScaledTime() * 1000.0f));

	return Asura_LipSync_Manager::AddToPlayingList(pxInstance);
}

bool Asura_LipSync_Articulation::Stop(int iIndex) const
{
	Asura_LipSync_Manager::RemoveFromPlayingList( iIndex );
	return false;
}

bool Asura_LipSync_Articulation::ReadArtData(Asura_Chunk_Stream& xStream)
{
	u_int uHead;
	u_int uHash;
	int iVar;
	int iSize;
	Asura_LipSync_Articulation* pxArt = NULL;

	u_int uCount = 0;

	bool bLoop = true;

	while(bLoop)
	{
		xStream >> uHead;
		iVar = ((uHead &0xFFFF0000) >> 16);
		iSize = uHead & 0xFFFF;

		switch(iVar)
		{
		case SS2_LA_ART_HASH:
			{
				xStream >> uHash;
				pxArt = Asura_LipSync_Manager::GetArticulationByHash(uHash);
				if(pxArt == NULL)
				{	pxArt = Asura_LipSync_Manager::CreateNewArticulation(uHash);}

				pxArt->m_uHash = uHash;

				break;
			}

		case SS2_LA_ART_FRAME_COUNT:
			{
				xStream >> pxArt->m_uFrames;
				if (pxArt->m_uFrames)
				{
					if(pxArt->m_pxFrameData != NULL)
					{	delete[] pxArt->m_pxFrameData;}
					pxArt->m_pxFrameData = new Asura_LipSync_Articulation_FrameData[pxArt->m_uFrames];
				}
				uCount = 0;
				break;
			}

		case SS2_LA_ART_FRAME:
			{
				xStream >> pxArt->m_pxFrameData[uCount].m_uMSStart;
				xStream >> pxArt->m_pxFrameData[uCount].m_uMSEnd;
				xStream >> pxArt->m_pxFrameData[uCount].m_uNumConsts;
				pxArt->m_pxFrameData[uCount].m_pxPhononData = new Asura_LipSync_Articulation_PhononData[pxArt->m_pxFrameData[uCount].m_uNumConsts];
				for(u_int i=0; i<pxArt->m_pxFrameData[uCount].m_uNumConsts; i++)
				{
					xStream >> pxArt->m_pxFrameData[uCount].m_pxPhononData[i].m_uPhoneme;
					xStream >> pxArt->m_pxFrameData[uCount].m_pxPhononData[i].m_fWeight;
				}
				uCount++;
				break;
			}

		case SS2_LA_END_CHUNK:
			{
				bLoop = false;
				break;
			}

		default:
			xStream.Skip(iSize);
			break;
		}
	}

	Asura_Assert( uCount == pxArt->m_uFrames, "didn't read in data for all lipsync frames");
	return true;
}

#ifdef ASURA_TOOLS

void Asura_LipSync_Articulation::WriteArtData(Asura_LipSync_Articulation* pxArt, Asura_Chunk_SaveBlock_Stream& xStream)
{
	xStream << ASURA_LIPSYNC_ARTICULATION_VARNAME(SS2_LA_ART_HASH, 4);
	xStream << pxArt->m_uHash;

	xStream << ASURA_LIPSYNC_ARTICULATION_VARNAME(SS2_LA_ART_FRAME_COUNT, 4);
	xStream << pxArt->m_uFrames;

	for(u_int a=0; a<pxArt->m_uFrames; a++)
	{
		Asura_LipSync_Articulation_FrameData* pxFrame = &pxArt->m_pxFrameData[a];
		xStream << ASURA_LIPSYNC_ARTICULATION_VARNAME(SS2_LA_ART_FRAME, 4 + 4 + 4 + (pxFrame->m_uNumConsts * (4 + 4) ) );
		xStream << pxFrame->m_uMSStart;	//4
		xStream << pxFrame->m_uMSEnd;	//4
		xStream << pxFrame->m_uNumConsts;	//4
		for(u_int i=0; i<pxFrame->m_uNumConsts; i++)
		{
			xStream << pxFrame->m_pxPhononData[i].m_uPhoneme;	//4
			xStream << pxFrame->m_pxPhononData[i].m_fWeight;	//4
		}
	}

	xStream << ASURA_LIPSYNC_ARTICULATION_VARNAME(SS2_LA_END_CHUNK, 0);
}

#endif	// ASURA_TOOLS

Asura_LipSync_Articulation_FrameData* Asura_LipSync_Articulation::GetFrame( const u_int uMs, u_int& uLastFrame ) const
{
#ifdef ASURA_TOOLS
	if(m_pxFrameData == NULL)	// Load on demand
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_LipSync_Articulation::GetFrame - Load on demand" );

		LoadAsuraFile(m_pszFilename.GetString());
	}
#endif // ASURA_TOOLS

	if(uLastFrame >= m_uFrames)
	{	return NULL;}

	if(m_pxFrameData[uLastFrame].m_uMSStart > uMs)
	{	uLastFrame = 0;}					// Just in case last frame is invalid we do a full search

	for(u_int i=uLastFrame; i< m_uFrames; i++)
	{
		if(m_pxFrameData[i].m_uMSStart <= uMs && m_pxFrameData[i].m_uMSEnd >= uMs)
		{
			uLastFrame = i;
			return &m_pxFrameData[i];
		}
	}

	uLastFrame = 0;
	return NULL;
}

Asura_LipSync_Articulation_Instance::Asura_LipSync_Articulation_Instance()
: m_pxArticulation(NULL)
, m_uMSStart(0)
, m_uLastFrame(0)
, m_iSoundHandle(ASURA_INVALID_HANDLE)
, m_bStarted(false)
, m_uSpeakerGuid(ASURA_GUID_UNREGISTERED)
{
}

Asura_LipSync_Articulation_FrameData* Asura_LipSync_Articulation_Instance::GetFrame( const u_int iMs ) 
{
	Asura_LipSync_Articulation_FrameData* pxFrame = m_pxArticulation->GetFrame( iMs - m_uMSStart, m_uLastFrame );
	return pxFrame;
}

#endif // ASURA_USE_ARTICULATION_SYSTEM


#if defined( ASURA_USE_ARTICULATION_SYSTEM)
u_int Asura_Lipsync_PhonemeViseme_Map_Container::MapPhonemeIndexToVisemeIndex( const u_int uPhoneme ) const
{
	return m_axMap[uPhoneme].m_uVisemeHash;
}
#endif

#if defined( ASURA_USE_ARTICULATION_SYSTEM)
u_int Asura_Lipsync_PhonemeViseme_Map_Container::GetVisemeIndexFromHash( const Asura_Hash_ID uHash )
{
	for(u_int i=0; i<m_uNumVisemes; i++)
	{
		if(m_axVisemeList[i].m_uVisemeHash == uHash)
		{	return i;}
	}

	Asura_Assertf(false, ("Hash %x not found in viseme map", uHash));
	return 0;
}
#endif
