//////////////////////////////////////////////////////////////////////////////
//
// Asura_LipSync_Manager.h
//
// Created by Stuart Middleton - 20 June 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef ___ASURA_LIPSYNC_MANAGER_H___
#define ___ASURA_LIPSYNC_MANAGER_H___

#ifdef ASURA_USE_ARTICULATION_SYSTEM

#ifdef ASURA_TOOLS
#include "Annosoft/lib/liblipsync.h"
#include "Annosoft/lib/libarticulator.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "LipSync/Tools_Lipsync.h"
#endif
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_chunk_lipsync.h"
#include "Core/Asura_List.h"
#include "Core/Asura_LinkList.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_StringBuffer.h"
#include "Core/Asura_FixedArray.h"
#include "Core/Asura_StringBuffer.h"

class Asura_LipSync_Articulation;
class Asura_LipSync_Articulation_Instance;
class Asura_Dialogue_Line;
struct Asura_Lipsync_PhonemeViseme_Map;
class Asura_LipSync_Articulation_FrameData;
struct Asura_LipSync_Articulation_VisemeData;
class Asura_LipSync_Viseme_FrameData;
class Asura_Lipsync_PhonemeViseme_Map_Manager;
class Asura_Lipsync_PhonemeViseme_Map_Container;

#define ASURA_LIPSYNC_MANAGER_MAX_PLAYING_ARTICULATIONS 32

enum
{
	ASURA_LIPSYNC_PHONEME_X = 0x78,	//'x'
	ASURA_LIPSYNC_PHONEME_M = 0x6D,	//'m',
	ASURA_LIPSYNC_PHONEME_P = 0x70,	//'p',
	ASURA_LIPSYNC_PHONEME_B = 0x62, //'b',
	ASURA_LIPSYNC_PHONEME_V = 0x76, //'v',
	ASURA_LIPSYNC_PHONEME_F = 0x66, //'f',
	ASURA_LIPSYNC_PHONEME_ZH = 0x485A, //'ZH',
	ASURA_LIPSYNC_PHONEME_CH = 0x4843, //'CH',
	ASURA_LIPSYNC_PHONEME_J = 0x6A, //'j',
	ASURA_LIPSYNC_PHONEME_SH = 0x4853, //'SH',
	ASURA_LIPSYNC_PHONEME_S = 0x73, //'s',
	ASURA_LIPSYNC_PHONEME_Z = 0x7A, //'z',
	ASURA_LIPSYNC_PHONEME_DH = 0x4844, //'DH',
	ASURA_LIPSYNC_PHONEME_TH = 0x4854, //'TH',
	ASURA_LIPSYNC_PHONEME_T = 0x74, //'t',
	ASURA_LIPSYNC_PHONEME_NG = 0x474E, //'NG',
	ASURA_LIPSYNC_PHONEME_G = 0x67, //'g',
	ASURA_LIPSYNC_PHONEME_K = 0x6B, //'k',
	ASURA_LIPSYNC_PHONEME_H = 0x68, //'h',
	ASURA_LIPSYNC_PHONEME_R = 0x72, //'r',
	ASURA_LIPSYNC_PHONEME_ER = 0x5245, //'ER',
	ASURA_LIPSYNC_PHONEME_W = 0x77, //'w',
	ASURA_LIPSYNC_PHONEME_UH = 0x4855, //'UH',
	ASURA_LIPSYNC_PHONEME_UW = 0x5755, //'UW',
	ASURA_LIPSYNC_PHONEME_AW = 0x5741, //'AW',
	ASURA_LIPSYNC_PHONEME_IY = 0x5949, //'IY',
	ASURA_LIPSYNC_PHONEME_Y = 0x79, //'y',
	ASURA_LIPSYNC_PHONEME_IH = 0x4849, //'IH',
	ASURA_LIPSYNC_PHONEME_AE = 0x4541, //'AE',
	ASURA_LIPSYNC_PHONEME_AY = 0x5941, //'AY',
	ASURA_LIPSYNC_PHONEME_EY = 0x5945, //'EY',
	ASURA_LIPSYNC_PHONEME_EH = 0x4845, //'EH',
	ASURA_LIPSYNC_PHONEME_N = 0x6E, //'n',
	ASURA_LIPSYNC_PHONEME_D = 0x64, //'d',
	ASURA_LIPSYNC_PHONEME_L = 0x6C, //'l',
	ASURA_LIPSYNC_PHONEME_AA = 0x4141, //'AA',
	ASURA_LIPSYNC_PHONEME_AH = 0x4841, //'AH',
	ASURA_LIPSYNC_PHONEME_AO = 0x4F41, //'AO',
	ASURA_LIPSYNC_PHONEME_OW = 0x574F, //'OW',
	ASURA_LIPSYNC_PHONEME_OY = 0x594F, //'OY',

	ASURA_LIPSYNC_PHONEME_COUNT = 40
};


// The manager handles the pool of articulations and the list of playing articulations
// It is initialized before the audio system because the data needs to be loaded first so the dialogue
// system can attach articulations to lines of dialogue.
class Asura_LipSync_Manager
{
	friend class Asura_Lipsync_PhonemeViseme_Map_Manager;
	friend class Asura_LipSync_Articulation;

public:
	static void Initialise(void);
	static void Reset(void);
	static void Shutdown();
	static Asura_LipSync_Articulation* CreateNewArticulation( const Asura_Hash_ID uHash, const Asura_Char* pszFilename = NULL );
	static Asura_LipSync_Articulation* GetArticulationByHash( const Asura_Hash_ID uHash );
	static Asura_LipSync_Articulation_Instance* GetPlayingArticulationByOwner( const Asura_Hash_ID uHash );
	static bool DeleteArticulationByHash( const Asura_Hash_ID uHash );

	static int GetNumPlayingArticulations();
	static Asura_LipSync_Articulation_Instance* GetPlayingArticulationByIndex( const int iIndex );
	static u_int GetPhonemeIndex( const u_int uPhoneme );

protected:
	static void ClearPlayingList();
	static int AddToPlayingList( Asura_LipSync_Articulation_Instance* const pxArt );
	static void RemoveFromPlayingList( const int iIndex );

	static u_int s_auPhonemeList[];
private:

	static Asura_LipSync_Articulation* s_pxBase;

	static Asura_LipSync_Articulation_Instance* s_apxPlayingArticulations[ASURA_LIPSYNC_MANAGER_MAX_PLAYING_ARTICULATIONS];
	static int s_iNumPlayingArticulations;

};

// The Phonon data holds a dword of data that represents the phoneme to use and a normalized weight at which to apply it
// Phonon data is part of an array of data to be applied at any point in time.
// The phonon data is a dword with the bottom 2 bytes being a char representation of the phoneme
struct Asura_LipSync_Articulation_PhononData
{
	u_int m_uPhoneme;
	float m_fWeight;
};

// The Viseme data holds a hash that represents the viseme to use and a normalized weight at which to apply it
// Viseme data has been remapped by the viseme to phoneme map in the manager
struct Asura_LipSync_Articulation_VisemeData
{
	Asura_Hash_ID m_uViseme;
	float m_fWeight;
};

// Frame data holds the start and end time in miliseconds for a frame of phonon data
// The phonon data is an array with m_uNumConsts entries in it
class Asura_LipSync_Articulation_FrameData
{
	friend class Asura_LipSync_Articulation;
	friend class Asura_Lipsync_PhonemeViseme_Map_Manager;
#ifdef ASURA_TOOLS
	friend Tools_LipSync;
#endif // ASURA_TOOLS
public:
	Asura_LipSync_Articulation_FrameData();
	~Asura_LipSync_Articulation_FrameData();

	u_int GetNumConsts( void) const { return m_uNumConsts; }
	const Asura_LipSync_Articulation_PhononData* GetPhononData( void ) const { return m_pxPhononData; }
private:
	u_int m_uMSStart;
	u_int m_uMSEnd;
	u_int m_uNumConsts;
	Asura_LipSync_Articulation_PhononData* m_pxPhononData;
};

// A viseme frame is the same as an articulation frame except that instead of phoneme data it contains
// remapped viseme data converted using the phoneme to viseme map contained in the manager
class Asura_LipSync_Viseme_FrameData
{
	friend class Asura_Lipsync_PhonemeViseme_Map_Manager;
public:

	Asura_LipSync_Viseme_FrameData(){ m_pxVisemeData = NULL; }
	~Asura_LipSync_Viseme_FrameData(){ delete[] m_pxVisemeData; }

	u_int GetNumConts( void ) const { return m_uNumConsts; }
	const Asura_LipSync_Articulation_VisemeData* GetVisemeData( void ) const { return m_pxVisemeData; }

protected:
private:
	u_int m_uMSStart;
	u_int m_uMSEnd;
	u_int m_uNumConsts;
	Asura_LipSync_Articulation_VisemeData* m_pxVisemeData;
};

// The articulation is a container for a complete articulation. It contains the frame data.
// In tools the articulation can write itself to a stream
// It also has a back reference to the line to which it is attached

class Asura_Chunk_LipSync;
class Tools_LipSync;

class Asura_LipSync_Articulation
{
	friend class Asura_LipSync_Manager;
	friend class Asura_Chunk_LipSync;
#ifdef ASURA_TOOLS
	friend Tools_LipSync;
#endif // ASURA_TOOLS
public:

	enum
	{	SS2_LIPSYNC_ARTICULATION_CHUNK_VERSION = 0};

	Asura_LipSync_Articulation();
	~Asura_LipSync_Articulation();
	
	static bool ReadArtData(Asura_Chunk_Stream& xStream);
	Asura_LipSync_Articulation_FrameData* GetFrame( const u_int uMs, u_int& uLastFrame ) const;

	int Start( const Asura_Handle iHandle, const Asura_Guid uSpeakerGuid );
	bool Stop( const int iIndex) const;

#ifdef ASURA_TOOLS
	void WriteArtData( Asura_LipSync_Articulation* pxArt, Asura_Chunk_SaveBlock_Stream& xStream );
#endif

	void FlagAsPermanent() { m_bPermanent = true; }
	bool IsPermanent() const { return m_bPermanent; }

protected:

	Asura_LipSync_Articulation* m_pxNext;
	Asura_Hash_ID m_uHash;

	enum StreamVariableTypes
	{	// NEVER reorder or delete entries: saved to file
		SS2_LA_END_CHUNK		= 0,
		SS2_LA_ART_FRAME_COUNT	= 1,
		SS2_LA_ART_FRAME		= 2,
		SS2_LA_EXP_FRAME		= 3,
		SS2_LA_ART_HASH			= 4,
	};

	u_int m_uFrames;
	bool m_bPermanent;
	Asura_LipSync_Articulation_FrameData* m_pxFrameData;
#ifdef ASURA_TOOLS
	Asura_StringBuffer<256> m_pszFilename;
#endif
};

// An articulation instance is the wrapper for a currently playing articulation. It holds a pointer to the articulation to play
// it also has a start time in milliseconds when the sample started playing.
class Asura_LipSync_Articulation_Instance
{
	friend class Asura_LipSync_Articulation;
public:
	Asura_LipSync_Articulation_Instance();
	~Asura_LipSync_Articulation_Instance(){};

	Asura_LipSync_Articulation_FrameData* GetFrame( const u_int iMs );

	Asura_Guid GetSpeakerGuid( void ) const { return m_uSpeakerGuid; }
	Asura_Handle GetSoundHandle( void ) const { return m_iSoundHandle; }
	bool HasStarted( void ) const { return m_bStarted; }
	void SetStart(int iMSStart) { m_uMSStart = iMSStart; m_bStarted = true;}

private:
	Asura_LipSync_Articulation* m_pxArticulation;
	u_int m_uMSStart;
	u_int m_uLastFrame;
	int m_iSoundHandle;
	bool m_bStarted;
	Asura_Guid m_uSpeakerGuid;
};


// Manages the mapping between the visemes and phonemes
class Asura_Lipsync_PhonemeViseme_Map_Manager
{

public:
	static void Initialise();
	static void Reset(void);
	static void Shutdown();

	static Asura_Lipsync_PhonemeViseme_Map_Container* CreatePhonemeVisemeMap();
	static void DeletePhonemeVisemeMap();
	static bool AddPhonemeToVisemeMapping( const u_int uPhoneme,  const Asura_Hash_ID uViseme, void* pxData, char* szName = NULL);
	static void SetPhonemeVisemeMap( Asura_Lipsync_PhonemeViseme_Map_Container*  const pxMap){ s_pxMapContainer = pxMap;}
	static Asura_Lipsync_PhonemeViseme_Map_Container* GetPhonemeVisemeMapContainer(){ return s_pxMapContainer;}
	static const Asura_Lipsync_PhonemeViseme_Map* GetPhonemeMapByIndex( const u_int uPhoneme );

	static Asura_LipSync_Viseme_FrameData* MapPhonemeFrameToVisemeFrame( const Asura_LipSync_Articulation_FrameData* const pxFrame );

	static const char* GetVisemeName( const Asura_Hash_ID uViseme );

protected:
private:
	static Asura_Lipsync_PhonemeViseme_Map_Container* s_pxMapContainer;
};


struct Asura_Lipsync_VisemeData
{
	Asura_Hash_ID m_uVisemeHash;
	Asura_StringBuffer<32> m_xVisemeName;
};

// A single mapping between a phoneme and a viseme
struct Asura_Lipsync_PhonemeViseme_Map
{
	u_int m_uPhoneme;
	Asura_Hash_ID m_uVisemeHash;
	Asura_Hash_ID m_uVisemeIndex;
};

// Contains the phoneme to viseme map and counters
class Asura_Lipsync_PhonemeViseme_Map_Container
{
	friend class Asura_Lipsync_PhonemeViseme_Map_Manager;
	friend struct Asura_Lipsync_PhonemeViseme_Map;

public:
	Asura_Lipsync_PhonemeViseme_Map_Container(){};
	~Asura_Lipsync_PhonemeViseme_Map_Container(){};
	u_int GetPhonemeCount(){	return m_uNumPhonemes;}
	u_int GetVisemeCount(){	return m_uNumVisemes;}
	const Asura_Lipsync_PhonemeViseme_Map* GetPhonemeMapArray( void ) const { return &m_axMap[0]; }
	const Asura_Lipsync_VisemeData* GetVisemeMapArray( void ) const { return &m_axVisemeList[0]; }
	u_int MapPhonemeIndexToVisemeIndex( const u_int uPhoneme ) const;
	u_int GetVisemeIndexFromHash(const Asura_Hash_ID uViseme);
protected:
	u_int m_uNumVisemes;
	u_int m_uNumPhonemes;
	Asura_FixedArray<Asura_Lipsync_PhonemeViseme_Map, ASURA_LIPSYNC_PHONEME_COUNT> m_axMap;
	Asura_FixedArray<Asura_Lipsync_VisemeData, ASURA_LIPSYNC_PHONEME_COUNT> m_axVisemeList;
private:
};




#endif // ASURA_USE_ARTICULATION_SYSTEM
#endif // ___ASURA_LIPSYNC_MANAGER_H___
