//////////////////////////////////////////////////////////////////////////////
//
// Asura_ProgressBar.cpp
//
// Copyright (c) 2002-2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Menus/Asura_ProgressBar.h"

// Engine independent.
#include "Debugging/Asura_Assert.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

//
// Asura_ProgressDisplay.
//

Asura_ProgressDisplay::Asura_ProgressDisplay( void )
{
     m_fProgress = 0.0f;
}

Asura_ProgressDisplay::~Asura_ProgressDisplay( void )
{
}

bool Asura_ProgressDisplay::Update( float fProgress )
{
    Asura_Assert( 0.0f <= fProgress && fProgress <= 1.0f, "" );
    m_fProgress = fProgress;
    return false;
}

void Asura_ProgressDisplay::Reset( void )
{
    m_fProgress = 0.0f;
}

//
// Asura_ProgressDisplay_Bar.
//

DEBUGVAR float dbg_fTextScreenRatio = .0015f;

Asura_ProgressDisplay_Bar::Asura_ProgressDisplay_Bar( void ) : PARENT()
{
	m_xPosition.x = 0.0f;
	m_xPosition.y = 0.0f;
    m_fMinUpdate = 1.0f;
	m_wszText = NULL;
	m_hFontHandle = ASURA_INVALID_HANDLE;
	m_fFontScale = 1.0f;
}

//Asura_ProgressDisplay_Bar::Asura_ProgressDisplay_Bar(float fX, float fY, float fLength, float fHeight)
//{
//    Asura_Assert(fX >= 0.0f && fX <= 1.0f, "progress bar needs normalised coordinates");
//    Asura_Assert(fY >= 0.0f && fY <= 1.0f, "progress bar needs normalised coordinates");
//    Asura_Assert(fLength >= 0.0f && fLength <= 1.0f, "progress bar needs normalised coordinates");
//    Asura_Assert(fHeight >= 0.0f && fHeight <= 1.0f, "progress bar needs normalised coordinates");
//
//	m_xPosition.x = fX;
//	m_xPosition.y = fY;
//	m_fLength = fLength;
//	m_fHeight = fHeight;
//	m_axSpriteData = new Asura_Overlay_Quad[2];		// 0 is surround, 1 is actual bar
//	CreateSpriteData();
//}

Asura_ProgressDisplay_Bar::~Asura_ProgressDisplay_Bar(void)
{
}

void Asura_ProgressDisplay_Bar::SetPosition(float fX, float fY)
{
    Asura_Assert(fX >= 0.0f && fX <= 1.0f, "progress bar needs normalised coordinates");
    Asura_Assert(fY >= 0.0f && fY <= 1.0f, "progress bar needs normalised coordinates");
	m_xPosition.x = fX;
	m_xPosition.y = fY;
	UpdateSpriteData();
}

void Asura_ProgressDisplay_Bar::SetDimensions(float fLength, float fHeight)
{
    Asura_Assert(fLength >= 0.0f && fLength <= 1.0f, "progress bar needs normalised coordinates");
    Asura_Assert(fHeight >= 0.0f && fHeight <= 1.0f, "progress bar needs normalised coordinates");
	m_fLength = fLength;
	m_fHeight = fHeight;
	UpdateSpriteData();
}

void Asura_ProgressDisplay_Bar::SetBorderColourFlat(Asura_Colour_Union xColour)
{
    m_axSpriteData[PB_BORDER].Colour = xColour.Colour;
}

void Asura_ProgressDisplay_Bar::SetBorderColourGouraud(Asura_Colour_Union xColourTL, 
                            Asura_Colour_Union xColourTR, 
                            Asura_Colour_Union xColourBL, 
                            Asura_Colour_Union xColourBR)
{
    m_axSpriteData[PB_BORDER].SetGouraudColour(xColourTL.Colour, xColourTR.Colour, xColourBL.Colour, xColourBR.Colour);
}

void Asura_ProgressDisplay_Bar::SetInnerColourFlat(Asura_Colour_Union xColour)
{
    m_axSpriteData[PB_INNER_BAR].Colour = xColour.Colour;
}

void Asura_ProgressDisplay_Bar::SetInnerColourGouraud(Asura_Colour_Union xColourTL, 
                            Asura_Colour_Union xColourTR, 
                            Asura_Colour_Union xColourBL, 
                            Asura_Colour_Union xColourBR)
{
    m_axSpriteData[PB_INNER_BAR].SetGouraudColour(xColourTL.Colour, xColourTR.Colour, xColourBL.Colour, xColourBR.Colour);
}

void Asura_ProgressDisplay_Bar::CommitSettings(void)
{
    // call this after setting up the colour, dimensions etc.
    // also if the screen resolution changes
    UpdateSpriteData();
}

bool Asura_ProgressDisplay_Bar::Update( float fProgress )
{
    PARENT::Update( fProgress );

	// Calculate the length and adjust the bar (only the mask part which overs up the bar)
	float fScreenWidth  = static_cast<float>(Asura_Viewport::GetWidth());
    float fCurWidth = m_fLength * fProgress * fScreenWidth;
    float fX = m_xPosition.x * fScreenWidth + fCurWidth;
	
    if (fabsf(m_axSpriteData[PB_INNER_MASK].X[0] - fX) >= m_fMinUpdate)
    {
        m_axSpriteData[PB_INNER_MASK].X[0] = fX;
        m_axSpriteData[PB_INNER_MASK].X[3] = fX;
        return true;
    }
    return false;
}

void Asura_ProgressDisplay_Bar::Render(void)
{
    Asura_Render::FlushDisplay();
    Asura_Render::AllowFullScreenRendering();

	// Draw the bar as untextured for now.
	// Surround
	m_axSpriteData[PB_BORDER].Render(false);
    Asura_Render::FlushDisplay();
	
    m_axSpriteData[PB_MIDDLE].Render(false);
    Asura_Render::FlushDisplay();

    m_axSpriteData[PB_INNER_BAR].Render(false);
    Asura_Render::FlushDisplay();

    m_axSpriteData[PB_INNER_MASK].Render(false);
    Asura_Render::FlushDisplay();

	if(m_hFontHandle != ASURA_INVALID_HANDLE)
	{
	    Asura_Font_System::SetFont(m_hFontHandle);
	}

	if(m_wszText)
	{
		Asura_Vector_2 xCentrePos;
		xCentrePos.x = m_xPosition.x + (m_fLength/2);
		xCentrePos.y = m_xPosition.y + (m_fHeight/2);

		Asura_Font_System::Print
		(
			xCentrePos.x,
			xCentrePos.y, 
			m_fFontScale * dbg_fTextScreenRatio * Asura_Viewport::GetWidth() / 1.5f,
			0xffffffff,
			ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_VERTICALCENTREJUSTIFIED | ASURA_TEXTFORMAT_WORDWRAP,
			m_wszText
		);
	}
}

void Asura_ProgressDisplay_Bar::SetBorder(float fXBorder, float fYBorder)
{
	m_fXBorder = fXBorder;
	m_fYBorder = fYBorder;
	UpdateSpriteData();
}

void Asura_ProgressDisplay_Bar::UpdateSpriteData(void)
{
	// 0 is surround, 1 is actual bar
	float fScreenWidth  = static_cast<float>(Asura_Viewport::GetWidth());
	float fScreenHeight = static_cast<float>(Asura_Viewport::GetHeight());

	// Do surround - one pixel at the moment
	m_axSpriteData[PB_BORDER].TextureIndex = ASURA_TEXTURE_NONE;
	m_axSpriteData[PB_BORDER].TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
	m_axSpriteData[PB_BORDER].SetRectXY((m_xPosition.x  * fScreenWidth - 2.0f * m_fXBorder), 
							          (m_xPosition.y  * fScreenHeight - 2.0f * m_fYBorder), 
							          (m_fLength * fScreenWidth + m_fXBorder * 4.0f), 
							          (m_fHeight * fScreenHeight + m_fYBorder * 4.0f));
//	m_axSpriteData[PB_BORDER].Colour = 0xFF333333;

    // middle bit - just surround with black
	m_axSpriteData[PB_MIDDLE].TextureIndex = ASURA_TEXTURE_NONE;
	m_axSpriteData[PB_MIDDLE].TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
	m_axSpriteData[PB_MIDDLE].SetRectXY((m_xPosition.x * fScreenWidth - m_fXBorder), 
							          (m_xPosition.y  * fScreenHeight - m_fYBorder), 
							          (m_fLength * fScreenWidth + m_fXBorder * 2.0f), 
							          (m_fHeight * fScreenHeight + m_fYBorder * 2.0f));
    m_axSpriteData[PB_MIDDLE].Colour = 0xff000000;
    
	// Do inner part -  full width
	m_axSpriteData[PB_INNER_BAR].TextureIndex = ASURA_TEXTURE_NONE;
	m_axSpriteData[PB_INNER_BAR].TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
	m_axSpriteData[PB_INNER_BAR].SetRectXY(m_xPosition.x * fScreenWidth,
							          m_xPosition.y * fScreenHeight,
							          m_fLength * fScreenWidth,
							          m_fHeight * fScreenHeight);
//	m_axSpriteData[PB_INNER_BAR].Colour = 0xFFFFFFFF;

	// Do inner part - cover up what we haven't reached. This is to allow Gouraud shading.
	m_axSpriteData[PB_INNER_MASK].TextureIndex = ASURA_TEXTURE_NONE;
	m_axSpriteData[PB_INNER_MASK].TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
	m_axSpriteData[PB_INNER_MASK].SetRectXY(m_xPosition.x * fScreenWidth,// will be modified
							          m_xPosition.y * fScreenHeight,
							          m_fLength * fScreenWidth, // will be modified
							          m_fHeight * fScreenHeight);
    m_axSpriteData[PB_INNER_MASK].Colour = 0xff000000;
}

//
// Asura_ProgressDisplay_BackgroundFiller.
//

Asura_ProgressDisplay_BackgroundFiller::Asura_ProgressDisplay_BackgroundFiller( void )
{
    Reset();
}

Asura_ProgressDisplay_BackgroundFiller::~Asura_ProgressDisplay_BackgroundFiller( void )
{
}

void Asura_ProgressDisplay_BackgroundFiller::Reset( void )
{
    PARENT::Reset();
    m_xCurDimensions.Zero();

	Platform_Reset();
}

bool Asura_ProgressDisplay_BackgroundFiller::Update( float fProgress )
{
    PARENT::Update( fProgress );

    float fScrW = static_cast<float>(Asura_Viewport::GetWidth());
    float fScrH = static_cast<float>(Asura_Viewport::GetHeight());

    float fScrOriginX = fScrW * m_xOrigin.x;
    float fScrOriginY = fScrH * m_xOrigin.y;

    float fNewExtentsX = fScrW * (m_bFillHorizontally ? fProgress : 1.0f) * m_xExtents.x;
    float fNewExtentsY = fScrH * (m_bFillHorizontally ? 1.0f : fProgress) * m_xExtents.y;

    float fNewWidth  = Asura_Maths::Abs( fNewExtentsX );
    float fNewHeight = Asura_Maths::Abs( fNewExtentsY );

    if( (fNewWidth - m_xCurDimensions.x) + (fNewHeight - m_xCurDimensions.y) < 1.0f ) { return false; }

    m_xCurTL.x = Asura_Maths::Min( fScrOriginX, fScrOriginX + fNewExtentsX );
    m_xCurTL.y = Asura_Maths::Min( fScrOriginY, fScrOriginY + fNewExtentsY );

    m_xCurDimensions.x = fNewWidth;
    m_xCurDimensions.y = fNewHeight;

    return true;
}

void Asura_ProgressDisplay_BackgroundFiller::Render( void )
{
    // The blend we're trying to do, ie. (src_col * dst_alpha + dst_col *
    // (1 - dst_alpha)), isn't supported by the independent render state
    // interface so it seems easiest to just deal with the problem in
    // platform-specific code.
    Platform_Render();
}

void Asura_ProgressDisplay_BackgroundFiller::SetExtents( const Asura_Vector_2& xExtents )
{
    Asura_Assert( -1.0f <= xExtents.x && xExtents.x <= 1.0f, "" );
    Asura_Assert( -1.0f <= xExtents.y && xExtents.y <= 1.0f, "" );
    m_xExtents = xExtents;
}

void Asura_ProgressDisplay_BackgroundFiller::SetFillColour( u_int uABGR )
{
    m_xFillColour.Colour = uABGR;
}

void Asura_ProgressDisplay_BackgroundFiller::SetFillDir( bool bHorizontal )
{
    m_bFillHorizontally = bHorizontal;
}

void Asura_ProgressDisplay_BackgroundFiller::SetOrigin( const Asura_Vector_2& xOrigin )
{
    Asura_Assert( 0.0f <= xOrigin.x && xOrigin.x <= 1.0f, "" );
    Asura_Assert( 0.0f <= xOrigin.y && xOrigin.y <= 1.0f, "" );
    m_xOrigin = xOrigin;
}
