//////////////////////////////////////////////////////////////////////////////
//
// Asura_ProgressBar.h
//
// Copyright (c) 2003-2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PROGRESSBAR_H___
#define ___ASURA_PROGRESSBAR_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"
#include "Rendering/Asura_Render.h"

// Engine platform-specific.
#include "Core/Asura_PlatformBasics.h"

// C L A S S E S /////////////////////////////////////////////////////////////

// Abstract base class for some kind of display of progress as a value in the
// range [0, 1].
class Asura_ProgressDisplay
{

    public:

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        Asura_ProgressDisplay( void );
        virtual ~Asura_ProgressDisplay( void );

        // M E T H O D S /////////////////////////////////////////////////////

        virtual void Render( void ) = 0;
        virtual bool Update( float fProgress ); // Should return true if the progress was different enough to require another render.
        virtual void Reset( void );

        float GetProgress( void ) const { return m_fProgress; }
		const float* GetProgressPtr( void ) const { return &m_fProgress; }

    private:
        
        // S T A T E /////////////////////////////////////////////////////////

        float m_fProgress; // [0, 1]

};
// End class Asura_ProgressDisplay.

class Asura_ProgressDisplay_Bar : public Asura_ProgressDisplay
{

	public:

		Asura_ProgressDisplay_Bar(void);
		virtual ~Asura_ProgressDisplay_Bar(void);

		void SetPosition(float fX, float fY);				            // Top-left coordinate, in normalised screen units, ie. [0, 1].
		void SetDimensions(float fLength, float fHeight);	            // again, in normalised pixels
		void SetBorder(float fXBorderPixels, float fYBorderPixels);		// Pixels, for a change
        void SetBorderColourFlat(Asura_Colour_Union xColour);
        void SetBorderColourGouraud(Asura_Colour_Union xColourTL, 
                                    Asura_Colour_Union xColourTR, 
                                    Asura_Colour_Union xColourBL, 
                                    Asura_Colour_Union xColourBR);
        void SetInnerColourFlat(Asura_Colour_Union xColour);
        void SetInnerColourGouraud(Asura_Colour_Union xColourTL, 
                                    Asura_Colour_Union xColourTR, 
                                    Asura_Colour_Union xColourBL, 
                                    Asura_Colour_Union xColourBR);

        void SetOverlayText(const Asura_UnicodeChar* wszText)
		{
			m_wszText = wszText;
		}

		void SetFontHandle(const Asura_Handle hFontHandle)
		{
			m_hFontHandle = hFontHandle;
		}

		void SetFontScale(const float fFontScale)
		{
			m_fFontScale = fFontScale;
		}

        void SetMinUpdate(float fMinUpdate) { m_fMinUpdate = fMinUpdate; }
        void CommitSettings(void);  // call this after setting up the colour, dimensions etc.
        // also if the screen resolution changes

		virtual bool Update( float fProgress );             // update returns true if the value changed.
		virtual void Render(void);
		
    private:

        typedef Asura_ProgressDisplay PARENT;

        enum SpriteID
        {
            PB_BORDER,
            PB_MIDDLE,
            PB_INNER_BAR,   // bar is always full width to allow gouranud shading
            PB_INNER_MASK,  // unused bit is covered up with middle colour.
            PB_NUM_SPRITES
        };

		void UpdateSpriteData(void);

		Asura_Vector_2 m_xPosition;
		Asura_Overlay_Quad m_axSpriteData[PB_NUM_SPRITES];
		float m_fLength;
		float m_fHeight;
		float m_fXBorder;
		float m_fYBorder;
        float m_fMinUpdate;

		const Asura_UnicodeChar*  m_wszText;
		Asura_Handle m_hFontHandle;
		float m_fFontScale;
};
// End class Asura_ProgressDisplay_Bar.

// Fills a specified area, assuming that the background image provides a
// mask in its alpha channel, ready for use in a destination alpha test.
class Asura_ProgressDisplay_BackgroundFiller : public Asura_ProgressDisplay
{

    public:

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        Asura_ProgressDisplay_BackgroundFiller( void );
        virtual ~Asura_ProgressDisplay_BackgroundFiller( void );

        // M E T H O D S /////////////////////////////////////////////////////

        virtual void Render( void );
        virtual bool Update( float fProgress );
        virtual void Reset( void );

        // Appearance.
        void SetExtents( const Asura_Vector_2& xExtents );    // In normalised screen units, ie. [-1, 1].
        void SetFillColour( u_int uABGR );
        void SetFillDir( bool bHorizontal );
        void SetOrigin( const Asura_Vector_2& xOrigin );      // In normalised screen units, ie. [0, 1]. Expansion is relative to this.

    private:
        
        // T Y P E S /////////////////////////////////////////////////////////

        typedef Asura_ProgressDisplay PARENT;

        // M E T H O D S /////////////////////////////////////////////////////

        void Platform_Render( void );
		void Platform_Reset( void );

        // S T A T E /////////////////////////////////////////////////////////

        // Parameters.
        Asura_Vector_2     m_xOrigin;
        Asura_Vector_2     m_xExtents;
        Asura_Colour_Union m_xFillColour;
        bool               m_bFillHorizontally;

        // Current state.
        Asura_Vector_2     m_xCurTL;
        Asura_Vector_2     m_xCurDimensions;

};
// End class Asura_ProgressDisplay_BackgroundFiller.

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PROGRESSBAR_H___
