//
// Asura_Env_System.h
//
// Created by Kevin Lea - 12/06/2001 11:15:39
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_ENV_SYSTEM_H__
#define __ASURA_ENV_SYSTEM_H__

#include "Core/Asura_HashTree.h"
#include "Core/Asura_String.h"
#include "Mesh/Asura_Env.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////


// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_Environment_System
{
public:
	// C L A S S   M E T H O D S /////////////////////////////////////////

	static 	void 				Initialise				(void);
	static 	void 				Shutdown				(void);
	static 	void 				Reset					(void);

	static 	void 				Update					(void);

	// Environment functions...
	static	bool				EnvironmentExists		(void)								{ return s_pxCurrentEnvironment != NULL; }
	static	Asura_Environment*	CreateEnvironment		(const char* szName);
	static	bool				DestroyEnvironment		(const Asura_Hash_ID uHash);

	static	void				CreateAndSetEnvironment	(const char* szName)				{ s_pxCurrentEnvironment = CreateEnvironment(szName); }
	static	void				SetCurrentEnvironment	(Asura_Environment* pxEnvironment)	{ s_pxCurrentEnvironment = pxEnvironment; }
	static	void				SetCurrentEnvironment	(const Asura_Hash_ID uHash)			{ s_pxCurrentEnvironment = s_xEnvironmentTree.GetDataByHash(uHash); }
	static	Asura_Environment&	GetCurrentEnvironment	(void)								{ return(*s_pxCurrentEnvironment); }
	static	Asura_Environment*	GetEnvironment			(const Asura_Hash_ID uHash)			{ return(s_xEnvironmentTree.GetDataByHash(uHash)); }

	// Accessor functions...
	static	Asura_Environment*	GetEnvironment			(const char* szName );
	static	Asura_Environment*	GetEnvironment			(const Asura_String& xName )		{ return(GetEnvironment(xName.GetString())); }
	static	Asura_Environment*	GetEnvironment			(void)								{ return(&GetCurrentEnvironment()); }

	static u_int GetNumEnvironments() {return s_xEnvironmentTree.GetNumberOfNodes();}

#if defined(ASURA_TOOLS)
	// Functions used to start entities at the camera position from Tools...
	static void SetUseCameraAsStartPoint(void);

	static void ResetUseCameraAsStartPoint(void)
	{
		s_bUseCameraPositionAsStartPoint	= false;
	}

	static bool GetUseCameraAsStartPoint(Asura_Vector_3& xPosition, Asura_Matrix_3x3& xOrientation)
	{
		if(s_bUseCameraPositionAsStartPoint)
		{
			xPosition		= s_xCameraPositionForStartPoint;
			xOrientation	= s_xCameraOrientationForStartPoint;
			return(true);
		}
		else
		{
			return(false);
		}
	}
#endif	// defined(ASURA_TOOLS)


private:
	// Private variables...
	static	Asura_Environment*							s_pxCurrentEnvironment;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_Environment>	s_xEnvironmentTree;			// Tree of all of the handlers available.

#if defined(ASURA_TOOLS)
	// Variables used to start entities at the camera position from Tools...
	static	bool				s_bUseCameraPositionAsStartPoint;
	static	Asura_Vector_3		s_xCameraPositionForStartPoint;
	static	Asura_Matrix_3x3	s_xCameraOrientationForStartPoint;
#endif	// defined(ASURA_TOOLS)

	// C L A S S   M E T H O D S /////////////////////////////////////////

	static 	void 				Platform_Initialise		(void);
	static 	void 				Platform_Update			(void);
	static 	void 				Platform_Reset			(void);
	static 	void 				Platform_Shutdown		(void);
};


#endif//__ASURA_ENV_SYSTEM_H__
