//
// Asura_Mesh_System.cpp
//
// Created by Kevin Lea - 12/06/2001 11:15:59
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_TOOLS

#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"

#include "Debugging/Asura_Assert.h"


void Asura_Mesh_System::Initialise(void)
{
	NumberOfMeshesStored = 0;

	NameTree = new Asura_NameTree;

}
void Asura_Mesh_System::Shutdown(void)
{
	for (int i=0; i<NumberOfMeshesStored; i++)
	{
		delete (MeshStorage[i]);
	}
	NumberOfMeshesStored = 0;
	
	if (NameTree)
	{
		delete NameTree;
		NameTree = 0;
	}
}

void Asura_Mesh_System::Reset(void)
{
//	Shutdown();
//	Initialise();
	for (int i=0; i<NumberOfMeshesStored; i++)
	{
		delete MeshStorage[i];
        MeshStorage[i] = NULL;
	}
	NumberOfMeshesStored = 0;
	
	if (NameTree)
	{
        NameTree->Reset();
    }
}

Asura_Mesh* Asura_Mesh_System::GetMesh( const Asura_Char* szName, bool bCreate)
{
	int index;

	if (bCreate)
	{
        // (MH - 30.08.2002) added fail safe here
        Asura_Assert(NumberOfMeshesStored<ASURA_MAX_NO_OF_MESHES,"Maximum number of meshes exceeded!");
        if (NumberOfMeshesStored >= ASURA_MAX_NO_OF_MESHES)
        {
            return 0;
        }

		if (!NameTree->FindNodeIndexByName(szName,true,index))
		{
			//Asura_Assert(NumberOfMeshesStored<ASURA_MAX_NO_OF_MESHES,"Maximum number of meshes exceeded!");

			MeshStorage[index] = new Asura_Mesh( szName );
			MeshStorage[index]->Index = index;
			NumberOfMeshesStored++;
		}

		return MeshStorage[index];
	}
	else
	{
		if (NameTree->FindNodeIndexByName(szName,false,index))
		{
			return MeshStorage[index];
		}
	}

	return 0;
}



Asura_Mesh* Asura_Mesh_System::MeshExistsWithThisName( const Asura_String& name)
{
	return 0;
}

void Asura_Mesh_System::DeleteMesh(Asura_Mesh* pxMesh)
{
	if(!pxMesh) return;
	int iIndex;

	if(NameTree->DeleteNodeByName(pxMesh->Name,iIndex))
	{
#if defined(ASURA_TOOLS) && defined(ASURA_MAIN)
		PlatformDeleteMesh( iIndex );
#endif // defined(ASURA_TOOLS) && defined(ASURA_MAIN)

		MeshStorage[iIndex] = MeshStorage[--NumberOfMeshesStored];
		MeshStorage[iIndex]->Index = iIndex;
	}
	else
	{
		Asura_Assert(0,"Failed to find mesh");
	}
	delete pxMesh;

}

#ifdef ASURA_TOOLS
bool Asura_Mesh_System::RenameMesh(Asura_Mesh* pxMesh,const char* pszName)
{
	Asura_Assert(pxMesh,"");
	Asura_Assert(pszName,"");

	if(GetMesh(pszName))
	{
		Asura_Assert(0,"Can't rename mesh since name is already used");
		return false;
	}


	Asura_String xName = pszName;
	//update the name in the name tree
	if(NameTree->RenameNode(pxMesh->Name,xName))
	{
		//and in the mesh
		pxMesh->Name = pszName;
	}
	else
	{
		Asura_Assert(0,"Unexpected error renaming node in mesh name tree");
		return false;
	}

	return true;
}
#endif // ASURA_TOOLS



Asura_Mesh* Asura_Mesh_System::MeshStorage[ASURA_MAX_NO_OF_MESHES];
int Asura_Mesh_System::NumberOfMeshesStored;
Asura_NameTree* Asura_Mesh_System::NameTree;

#if 0

#ifndef ASURAIMPORT_EXPORTS
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
/*************************************************************
 *
 * Name          : Asura_Mesh_System ::RenderAsuraMesh
 *
 * Author        : Richard May
 *
 * Description   : Renders an Asura_Mesh using the correct camera 
 *				   parameters, etc. A little more high-level than
 *				   the straight 'Asura_PlatformRenderMesh' stuff
 *				** NOTE: This still ultimately uses PlatformRenderMesh **
 *
 * Parameters    : mesh - pointer to the Asura_Mesh to render
 *				   guid - guid of the camera (and therefore player?)
 *						  this mesh is being rendered for, allowing us
 *						  to ignore the mesh for certain cameras
 *				   position - pointer to the position vector of the mesh
 *				   orientation - pointer to the orientation matrix of the mesh
 *
 * Return value  : void
 *
 *************************************************************/
void Asura_Mesh_System ::RenderAsuraMesh ( Asura_Mesh* mesh, Asura_Guid guid, const Asura_Vector_3* position, const Asura_Matrix_3x3* orientation, const unsigned uCamVisFlags )
{
	/// Kevin Lea - 14/03/2002 10:30:50 - check to see if anim is visible in ANY camera view
	// All clipping should now be done BEFORE this point, and passed in uCamVisFlags.aw.
	Asura_Assert(uCamVisFlags, "Shouldn't get here unless there is some camera visibility.");
	for( int iCam = Asura_Camera::GetNumberOfCameras(); iCam--; )
    {
		Asura_Camera::SetCameraIndex( iCam );
		if (Asura_Camera::IsCameraActive() && 
			((1 << iCam) & uCamVisFlags) &&
			(Asura_Camera::GetEntityToIgnore() != guid) )
        {
			Asura_Render::ApplyCameraSettings();
			
			Asura_Mesh* pxMeshToRender = mesh;
			if(mesh->m_pxNextLowerMesh)
			{
				//mesh has LOD models , work out which one to use
				float fDistance = (*Asura_Camera::GetPosition() - *position).Magnitude();
				pxMeshToRender = mesh->GetDetailMeshFromDistance(fDistance);
			}
			PlatformRenderMesh( pxMeshToRender, position, orientation );
		} // end if IsCameraActive
    } // end for iCam
		

}
/* end of function RenderAsuraMesh */

#endif //ifndef ASURAIMPORT_EXPORTS

#endif

#endif // #ifdef ASURA_TOOLS
