/**
*
* \file Asura_Multiplayer_Comms.cpp
*
* Created by Richard May - 06/08/2003 16:39:14
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/




/************/
/* Includes.*/
/************/

// main asura header
#include "Core/Asura.h"

// local header
#include "Multiplayer/Asura_Multiplayer_Comms_System.h"

// asura
#include "Core/Asura_String.h"
#include "Core/Asura_Timers.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Interface/Asura_CommandConsole.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"

// independent stdlib
#include <wchar.h>

#ifdef ASURA_SUPPORT_MULTIPLAYER_COMMS

/************/
/* Defines. */
/************/

#define  ASURA_MPCOMMS_CURSOR_DISPLAY_TIME  0.5f
#define  ASURA_MPCOMMS_CURSOR_OFF_TIME      0.5f


/*********/
/* Data. */
/*********/

// variables
bool  Asura_Multiplayer_Comms_System::s_bSessionOpen = false;
u_int Asura_Multiplayer_Comms_System::s_uSenderGuid  = ASURA_GUID_UNREGISTERED;
u_int Asura_Multiplayer_Comms_System::s_uCursorPosition = 0;
float Asura_Multiplayer_Comms_System::s_fCursorFlashTimer = 0.0f;
float Asura_Multiplayer_Comms_System::s_fMessageDelayTimer = 0.0f;
Asura_UnicodeChar Asura_Multiplayer_Comms_System::s_wszInputBuffer[];
u_int Asura_Multiplayer_Comms_System::s_uSessionMode = Asura_Multiplayer_Comms_System::MPCOMMS_SESSIONMODE_TALK;

Asura_HashTree Asura_Multiplayer_Comms_System::s_xProfanityList;

/**************/
/* Functions. */
/**************/

/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::Initialise
 *
 * Author        : Richard May
 *
 * Description   : Initialisation and setup function
 *
 *************************************************************/
void Asura_Multiplayer_Comms_System::Initialise ( void )
{
	s_uSenderGuid = ASURA_GUID_UNREGISTERED;
	s_uCursorPosition = 0;
	s_wszInputBuffer[0] = 0;
	s_bSessionOpen = false;
	s_uSessionMode = MPCOMMS_SESSIONMODE_TALK;
	s_fCursorFlashTimer = 0.0f;
	s_fMessageDelayTimer = s_fMessageDelayPeriod;

}
/* end of function Initialise */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::Shutdown
 *
 * Author        : Richard May
 *
 * Description   : Shutdown and clearup function
 *
 *************************************************************/
void Asura_Multiplayer_Comms_System::Shutdown ( void )
{
}
/* end of function Shutdown */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::Reset
 *
 * Author        : Richard May
 *
 * Description   : Reset function (shuts down and then re-intialises)
 *
 *************************************************************/
void Asura_Multiplayer_Comms_System::Reset ( void )
{
	Shutdown();
	Initialise();
}
/* end of function Reset */

void Asura_Multiplayer_Comms_System::LoadBannedWordList( void )
{
	InitialiseProfanityList();
}

void Asura_Multiplayer_Comms_System::UnloadBannedWordList( void )
{
	s_xProfanityList.Reset();
}

/* Session management functions */

/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::OpenCommsSession
 *
 * Author        : Richard May
 *
 * Description   : Opens a communications session
 *
 * Parameters    : uSenderGuid - the guid of the entity that opened 
 *								 the session
 *
 *************************************************************/
void Asura_Multiplayer_Comms_System::OpenCommsSession ( Asura_Guid uSenderGuid, u_int uMode )
{
	//if( !Asura_Network::IsExternalNetworkActive() ) return;
	if( IsSessionOpen() ) return;
	
	// clear previous key buffers
	Asura_UserInput::Update();

	// clear previous input buffer if we're opening a new kind of session
	if( s_uSessionMode != uMode )
	{
		ClearInputBuffer();
	}

	s_uSenderGuid  = uSenderGuid;
	s_bSessionOpen = true;
	s_uSessionMode = uMode;
}
/* end of function OpenCommsSession */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::CloseCommsSession
 *
 * Author        : Richard May
 *
 * Description   : Closes the comms session - public so can be used
 *				   to force closure by main game code
 *
 *************************************************************/
void Asura_Multiplayer_Comms_System::CloseCommsSession ( void )
{
	s_bSessionOpen = false;
	ClearInputBuffer();
}
/* end of function CloseCommsSession */




/* Main Functions */

//extern bool Asura_KeyboardTextInput_Update(Asura_UnicodeChar* wszBuffer, u_int uMaxChars, u_int& uConsoleInputCursorPosition, u_int uFlags);

/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::Update
 *
 * Author        : Richard May
 *
 * Description   : Main update function - a lot of this has been
 *				   borrowed from the command console stuff
 *				   written by James Paulin and Morgan Parry
 *
 *************************************************************/
void Asura_Multiplayer_Comms_System::Update ( void )
{
	s_fMessageDelayTimer -= Asura_Timers::GetGameTimeSlice();
	
	Project_Update();

	// don't update if session isn't open
	if( !s_bSessionOpen ) return;

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	// don't process any input whilst the console is down
	if( Asura_CommandConsole::IsConsoleDown() )	return;	
#endif

	// input handling has been replaced with this function	
	Asura_UserInput::UpdateInputBuffer( s_wszInputBuffer, ASURA_MPCOMMS_MAX_MSG_SIZE, s_uCursorPosition, HandleInput );

	s_fCursorFlashTimer -= Asura_Timers::GetGameTimeSlice();
	if( s_fCursorFlashTimer < -ASURA_MPCOMMS_CURSOR_DISPLAY_TIME )
	{
		s_fCursorFlashTimer = ASURA_MPCOMMS_CURSOR_OFF_TIME;
	}
}
/* end of function Update */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::Render
 *
 * Author        : Richard May
 *
 * Description   : Display function for comms systems
 *
 *************************************************************/
void Asura_Multiplayer_Comms_System::Render ( void )
{
	Project_Render();
}
/* end of function Render */


/* Messaging Functions */

/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::Talk
 *
 * Author        : Richard May
 *
 * Description   : Standard broad cast across the network
 *
 * Parameters    : wszMsg - message text
 *
 * Return value  : bool - returns true if successful
 *
 *************************************************************/
bool Asura_Multiplayer_Comms_System::Talk( const Asura_UnicodeChar* wszMsg )
{	
	// calculate size of message text		
	u_int uLength = wcslen(wszMsg)+1; // include null char

    Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MULTIPLAYERCOMMS_TALK, sizeof(Asura_Msg_MultiPlayerComms_Talk) );
	if(pxMessage)
	{
		pxMessage->To   = ASURA_GUID_SERVER;
		pxMessage->From = s_uSenderGuid;

		Asura_Msg_MultiPlayerComms_Talk* pxData = static_cast<Asura_Msg_MultiPlayerComms_Talk*> (pxMessage->GetData());
		if(pxData)
		{
			pxData->m_uFrom = pxMessage->From;	
			
			for( u_int u=0; u<Asura_Maths::Min(uLength,(u_int)ASURA_MPCOMMS_MAX_MSG_SIZE); u++ )
			{
				pxData->m_wszMsg[u] = wszMsg[u];
			}
		}

		Asura_Network::Send(pxMessage);

		// prevent them from sending another message for 5 seconds
		s_fMessageDelayTimer = s_fMessageDelayPeriod; 

		return true;
	}

	return false;
}
/* end of function Talk */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::Team
 *
 * Author        : Richard May
 *
 * Description   : Team-only broadcast messaging
 *
 * Parameters    : wszMsg - message text
 *
 * Return value  : bool - returns true if successful
 *
 *************************************************************/
bool Asura_Multiplayer_Comms_System::Team( const Asura_UnicodeChar* wszMsg )
{	
	// calculate size of message text		
	u_int uLength = wcslen(wszMsg)+1; // include null char
	
    Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MULTIPLAYERCOMMS_TEAMTALK, sizeof(Asura_Msg_MultiPlayerComms_Talk) );
	if(pxMessage)
	{
		pxMessage->To   = ASURA_GUID_SERVER;
		pxMessage->From = s_uSenderGuid;
		
		Asura_Msg_MultiPlayerComms_Talk* pxData = static_cast<Asura_Msg_MultiPlayerComms_Talk*> (pxMessage->GetData());
		if(pxData)
		{
			pxData->m_uFrom = pxMessage->From;				
			for( u_int u=0; u<Asura_Maths::Min(uLength,(u_int)ASURA_MPCOMMS_MAX_MSG_SIZE); u++ )
			{
				pxData->m_wszMsg[u] = wszMsg[u];
			}
		}
		
		Asura_Network::Send(pxMessage);

		// prevent them from sending another message for 5 seconds
		s_fMessageDelayTimer = s_fMessageDelayPeriod; 

		return true;
	}

	return false;
}
/* end of function Team */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::Tell
 *
 * Author        : Richard May
 *
 * Description   : Private messaging to a single player
 *
 * Parameters    : wszMsg - message text
 *
 * Return value  : bool - returns true if successful
 *
 *************************************************************/
bool Asura_Multiplayer_Comms_System::Tell( const Asura_UnicodeChar* wszMsg )
{
	Asura_Guid uTo = ASURA_GUID_UNREGISTERED;
	u_int u;

	// extract player name from string
	Asura_UnicodeChar wszPlayerName[ASURA_MPCOMMS_MAX_MSG_SIZE];

	// if the player name has been surrounded by speech marks (it has to be for multi-word names)
	Asura_UnicodeChar wcEndDelimiter = 0;
	if( (wszMsg[0] == '"') || (wszMsg[0] == '\'') )
	{	
		wcEndDelimiter = (wszMsg[0] == '"') ? '"' : '\''; 

		for( u=0; u<ASURA_MPCOMMS_MAX_MSG_SIZE-1; u++ )
		{			
			wszPlayerName[u] = wszMsg[u+1];
			if( (wszMsg[u+1] == wcEndDelimiter) || (wszMsg[u+1] == '\0') ) break;
		}
		wszPlayerName[u] = 0; // null terminate
	}
	else
	{
		for( u=0; u<ASURA_MPCOMMS_MAX_MSG_SIZE; u++ )
		{			
			wszPlayerName[u] = wszMsg[u];
			if( (wszMsg[u] == ' ') || (wszMsg[u] == '\0') ) break;
		}
		wszPlayerName[u] = 0; // null terminate
	}

	// strip whitespace from the end
	for( u=wcslen(wszPlayerName); wszPlayerName[u] == ' '; --u )
	{  
		wszPlayerName[u] = 0;
	}

	ConvertUnderscores(wszPlayerName);

	// make sure player exists
	uTo = Asura_Network::GetPlayerGUID( wszPlayerName, false );
	if( uTo == ASURA_GUID_UNREGISTERED ) return false; 

	// calculate size of message text		
	u_int uLength = wcslen(wszMsg) + 1; // include null char
	
    Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MULTIPLAYERCOMMS_TELL, sizeof(Asura_Msg_MultiPlayerComms_Tell) );
	if(pxMessage)
	{
		pxMessage->To   = ASURA_GUID_SERVER;
		pxMessage->From = s_uSenderGuid;
		
		Asura_Msg_MultiPlayerComms_Tell* pxData = static_cast<Asura_Msg_MultiPlayerComms_Tell*> (pxMessage->GetData());
		if(pxData)
		{
			pxData->m_uTo = uTo;
			pxData->m_uFrom = pxMessage->From;			
			
			// move u on to the next letter before beginning to copy				
			u_int v=0;
			
			u_int uStartPos = wcEndDelimiter ? wcslen(wszPlayerName)+3 : wcslen(wszPlayerName)+1;
			if( uStartPos >= ASURA_MPCOMMS_MAX_MSG_SIZE ) return false; // name too long!

			for( u=uStartPos; u<Asura_Maths::Min(uLength,(u_int)ASURA_MPCOMMS_MAX_MSG_SIZE); u++ )
			{
				pxData->m_wszMsg[v] = wszMsg[u];
				v++;
			}
		}
		
		Asura_Network::Send(pxMessage);

		// prevent them from sending another message for 5 seconds
		s_fMessageDelayTimer = s_fMessageDelayPeriod; 

		return true;
	}

	return false;
}
/* end of function Tell */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::SendMessage
 *
 * Author        : Richard May
 *
 * Description   : Sends the message using the correct message mode
 *
 * Parameters    : void
 *
 * Return value  : bool - whether the message was sent successfully
 *
 *************************************************************/
bool Asura_Multiplayer_Comms_System::SendMessage ( void )
{
	bool bSuccess = false;

	// let the project-specific code handle the mode first. If that fails, revert to default handling
	if( !Project_SendMessage() )
	{
		switch(s_uSessionMode)
		{
		case MPCOMMS_SESSIONMODE_TALK : bSuccess = Talk( s_wszInputBuffer ); break;
		case MPCOMMS_SESSIONMODE_TEAM : bSuccess = Team( s_wszInputBuffer ); break;
		case MPCOMMS_SESSIONMODE_TELL : bSuccess = Tell( s_wszInputBuffer ); break;

		default : Asura_Assertf( false, ("Unrecognised comms mode (%d)",s_uSessionMode) ); break;
		}
	}	

	// clear buffer
	ClearInputBuffer();

	return bSuccess;
}
/* end of function SendMessage */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::ClearInputBuffer
 *
 * Author        : Richard May
 *
 * Description   : Clears the input buffer and resets cursor position
 *
 *************************************************************/
void Asura_Multiplayer_Comms_System::ClearInputBuffer ( void )
{
	s_wszInputBuffer[0] = 0;
	s_uCursorPosition = 0;
}
/* end of function ClearInputBuffer */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::MakeLowerCase
 *
 * Author        : Richard May
 *
 * Description   : Converts a unicode string to lower case
 *
 * Parameters    : wszString - unicode string to covert
 *
 * Return value  : void
 *
 *************************************************************/
void Asura_Multiplayer_Comms_System::MakeLowerCase ( Asura_UnicodeChar* wszString )
{	
	u_int u=0;
	while( wszString[u] )
	{			
		if( (wszString[u] >= 'A') && (wszString[u] <= 'Z') )
		{
			wszString[u] += 'a' - 'A';
		}
		u++;
	}			
}
/* end of function MakeLowerCase */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::ConvertUnderscores
 *
 * Author        : Richard May
 *
 * Description   : Converts all underscores in the string into spaces
 *
 * Parameters    : wszString - unicode string to covert
 *
 * Return value  : void
 *
 *************************************************************/
void Asura_Multiplayer_Comms_System::ConvertUnderscores ( Asura_UnicodeChar* wszString )
{	
	u_int u=0;
	while( wszString[u] )
	{			
		if( (wszString[u] == '_') )
		{
			wszString[u] = ' ';
		}
		u++;
	}			
}
/* end of function MakeLowerCase */

#include "Debugging/Asura_DebugText.h"

/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::FilterProfanities
 *
 * Author        : Richard May
 *
 * Description   : Replaces any recognised profanities with %$*! 
 *				   style punctuation
 *
 * Parameters    : wszMsg - string to filter
 *
 * Return value  : bool - whether the string contained any 
 *						  'bad words' or not
 *
 *************************************************************/
bool Asura_Multiplayer_Comms_System::FilterProfanities ( Asura_UnicodeChar* wszMsg )
{
	Asura_Assert(wcslen(wszMsg) < ASURA_MPCOMMS_MAX_MSG_SIZE,"string may be too long for profanity filter");
	
	// account for null string
	if(!wszMsg) return false;
	
	// if filtering is disabled, we don't find any bad words
	if(!Project_GetProfanityFilterEnabled())
	{
		return false;
	}

	bool  bFound = false;
	u_int uWordLength = 0;
	Asura_UnicodeChar* pwcStringPtr = wszMsg;
	Asura_UnicodeChar wszWord[ASURA_MPCOMMS_MAX_MSG_SIZE];
	
	// while we've got any of the string to still consider...
	do
	{
		bFound = false;

		// strip white space and punctuation		
		while( (*pwcStringPtr == ' ') || (*pwcStringPtr == '!') || (*pwcStringPtr == '?') ||
			   (*pwcStringPtr == '-') || (*pwcStringPtr == ':') || (*pwcStringPtr == ';') ||
			   (*pwcStringPtr == '.') || (*pwcStringPtr == '"') || (*pwcStringPtr == '\'') )
		{
			pwcStringPtr++;
		}	

		// get next word (run to whitespace, hyphen or null)
		uWordLength = 0;
		Asura_UnicodeChar* pwcWordPtr = pwcStringPtr;		
		while( (*pwcStringPtr != ' ') && (*pwcStringPtr != '-') && (*pwcStringPtr != 0) )
		{
			Asura_Assert(uWordLength < ASURA_MPCOMMS_MAX_MSG_SIZE,"word too long for profanity filter");
			wszWord[uWordLength] = *pwcStringPtr++;
			uWordLength++;
		}
		wszWord[uWordLength] = 0;			
		
		// if the profanity is found, replace _entire_ word with random punctuation
		if( MatchProfanity(wszWord) != MATCH_NONE )
		{
			bFound = true;
		}
		else
		{
			// try removing any punctuation from the word
			u_int u=0;
			u_int v=0;
			Asura_UnicodeChar wszStrippedWord[ASURA_MPCOMMS_MAX_MSG_SIZE];

			while( wszWord[u] )
			{
				if( iswalpha(wszWord[u]) )
				{
					wszStrippedWord[v] = wszWord[u];
					v++;
				}	
				u++;
			}
			wszStrippedWord[v] = 0;

			if( MatchProfanity(wszStrippedWord) )
			{
				bFound = true;
			}
			else
			{
				// try 'de-l33ting' the word
				u=0;
				Asura_UnicodeChar wszStrippedWord[ASURA_MPCOMMS_MAX_MSG_SIZE];
				
				while( wszWord[u] )
				{
					switch( wszWord[u] )
					{
					case '!' : wszStrippedWord[u] = 'i'; break;
					case '$' : wszStrippedWord[u] = 's'; break;
					case '4' : wszStrippedWord[u] = 'a'; break;
					case '3' : wszStrippedWord[u] = 'i'; break;
					case '0' : wszStrippedWord[u] = 'o'; break;
					case '@' : wszStrippedWord[u] = 'a'; break;
					case '1' : wszStrippedWord[u] = 'i'; break;
					default : wszStrippedWord[u] = wszWord[u]; break;
					}
					u++;
				}
				wszStrippedWord[u] = 0;

				if( MatchProfanity(wszStrippedWord) )
				{
					bFound = true;
				}
			}
		}		
		
		// write random punctuation from the beginning of the word
		if(bFound)
		{
			for( u_int u=0; u<uWordLength; u++ )
			{
				*pwcWordPtr = GetRandomPunctuation();
				pwcWordPtr++;
			}
		}
	}
	while( *pwcStringPtr != 0 );
	
	// return whether or not any naughty words were removed
	return bFound;
}
/* end of function FilterProfanities */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::MatchProfanity
 *
 * Author        : Richard May
 *
 * Description   : Compares a given string to the profanity list
 *				   and returns true if it matches a known profanity
 *
 * Parameters    : wszWord - word to check
 *
 * Return value  : MATCH_RESULT - 'yes', 'no' or 'maybe'
 *
 *************************************************************/
Asura_Multiplayer_Comms_System::MATCH_RESULT Asura_Multiplayer_Comms_System::MatchProfanity ( const Asura_UnicodeChar* wszMsg )
{
	/// Richard May - 28/10/2003 11:54:22 - Needs to test for 'multi-word' profanities,
	/// like 'son of a bitch', 'cock sucker' and so forth

	// need to convert to ASCII currently (hashing doesn't support unicode as it stands)
	char szTemp[ASURA_MPCOMMS_MAX_MSG_SIZE];
	Asura_PrintfToBfr( szTemp, "%S", UINT_MAX, true, wszMsg );

	// for now, just check the profanity tree
	int iIndex; // required by find node
	
	if( s_xProfanityList.FindNodeIndexByName( szTemp, false, iIndex ) )
	{
		return MATCH_POSITIVE;
	}

	return MATCH_NONE;
}
/* end of function MatchProfanity */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::GetRandomPunctuation
 *
 * Author        : Richard May
 *
 * Description   : Returns a random punctuation character
 *
 *************************************************************/
Asura_UnicodeChar Asura_Multiplayer_Comms_System::GetRandomPunctuation ( void )
{
	const u_int uNumChars = 7;
	static const Asura_UnicodeChar s_wcPuncatuation[uNumChars] = { '!','$','&','*','@','#','?' };

	return s_wcPuncatuation[Asura_Random::GetIntU() % uNumChars];
}
/* end of function GetRandomPunctuation */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::InitialiseProfanityList
 *
 * Author        : Richard May
 *
 * Description   : Sets up the hash tree used for profanity
 *				   filtering
 *
 *************************************************************/
bool Asura_Multiplayer_Comms_System::InitialiseProfanityList ( void )
{
	// load profanity list text file
	return Asura_ChunkLoading_System::LoadAsuraFile(s_szProfanityFile);
}
/* end of function InitialiseProfanityList */


/*************************************************************
 *
 * Name          : Asura_Multiplayer_Comms_System::HandleInput
 *
 * Author        : Richard May
 *
 * Description   : Special input handler for user input
 *
 * Parameters    : uKey - key to handle
 *
 * Return value  : bool - true if key handled successfully
 *
 *************************************************************/
bool Asura_Multiplayer_Comms_System::HandleInput ( ASURA_KEYID uKey )
{
	s_fCursorFlashTimer = 0.0f; // stop the cursor from flashing if a key is pressed

	switch(uKey)
	{
		case ASURA_KEYID_ESCAPE : 
		{	
			CloseCommsSession(); return true;
		}

		case ASURA_KEYID_ENTER :
		{
			if( s_fMessageDelayTimer <= 0.0f ) // can't send more a message more than once every five seconds
			{
				// the user pressed enter...
				if( wcslen(s_wszInputBuffer) && !SendMessage() ) // if there's nothing in the message buffer, don't send a message		
				{
					Asura_Assert( false, "Send message failed." );			
				}
			}	
			CloseCommsSession();
			return true;
		}

		default : return false;	// not handled here, let input buffer handle it	
	}
}
/* end of function HandleInput */


#endif

/****************/
/* End of file. */
/****************/
