//
// Asura_NetConnection.h
//
// Created by KOR - 03/11/2003 11:10:25
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_NETCONNECTION_H__
#define __ASURA_NETCONNECTION_H__

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#define ASURA_CHALLENGE_LENGTH           32
#define ASURA_RESPONSE_LENGTH            32

// Asura headers
#include "Network/Asura_NetTransport_System.h"
#include "Network/Asura_Platform_NetSocket.h"

// Maximum number of expected sequence numbers to store
const u_int ASURA_NETCONNECTION_MAX_EXPECTED_SNS = 64;
// Maximum number of sequence numbers contained in an ACK
const u_int ASURA_NETCONNECTION_ACK_SIZE = 32;
// Buffer for helper simplemessage method
const u_int ASURA_NETCONNECTION_MAX_SIMPLEMESSAGE_LENGTH = 256;
// Milliseconds before connection timeout
const u_int ASURA_NETCONNECTION_TIMEOUT = 30000;
// Milliseconds between keep alive
const u_int ASURA_KEEPALIVE_PERIOD = 1000;

class Asura_NetSocket;

class Asura_NetConnection
{
public:
	Asura_NetConnection();
	~Asura_NetConnection();

	// (AW - 20.05.2005) - Activate/Deactivate functions used instead of new and delete
	// to avoid fatal memory fragmentation.
	void	Activate(Asura_NetSocket* pxSocket,
					 Asura_NetAddress xAddress,
					 unsigned short uRemotePort,
					 int iTimeOut,
					 bool bBlocking,
					 bool bInitiate);
	void	Deactivate();
	bool	IsActive() const {return m_bActive;}

	void	Reject(const char* pcMessage, int iLength);
	bool	Accept();
	bool	Ping();
	bool	Reset();
	bool	Update();
	bool	HandleAck(const char* pcMessage, int iLength);
	bool	HandleAccept(const char* pcMessage, int iLength);
	bool	HandleReject(const char* pcMessage, int iLength);
	bool	HandlePing(const char* pcMessage, int iLength);
	bool	HandlePong(const char* pcMessage, int iLength);
	bool	HandleClose(const char* pcMessage, int iLength);
	bool	HandleMessage(const char* pcMessage, int iLength);
	bool	HandleClientRequest(const char* pcMessage, int iLength);
	//bool	HandleChallengeRequest(const char* pcMessage, int iLength);
	bool	HandleChallengeResponse(const char* pcMessage, int iLength);
	bool	HandleNetSimSettings(const char* pcMessage, int iLength);
	// Returns true if message is a duplicate, false otherwise
	bool	RegisterSequenceNumber(unsigned short usSequenceNumber);
	bool	Send(const void* pvMessage,	int iLength, bool bReliable = false);
	bool	SendSimple(unsigned short usID, bool bReliable, const char* pcData = NULL, int iLength = 0);
	bool	SendAccept();
	bool	SendAck();
	bool	SendClosed(Asura_Close_Reason eCloseReason);
	bool	SendKeepAlive();
	bool	SendRetries();
	bool	SetClosed(float fTime);
	bool	Close(bool bForce, Asura_Close_Reason eReason);
	void	CheckBufferIntegrity();
	// Access functions
	Asura_Network_MachineInfo*	GetMachineInfo() const								{ return m_pxMachineInfo; }
	void						SetMachineInfo(Asura_Network_MachineInfo* pxData)	{ m_pxMachineInfo = pxData; }
	Asura_NetAddress			GetAddress() const									{ return m_xAddress; }
	unsigned short				GetPort() const										{ return m_usPort; }
	Asura_Connection_State		GetState() const									{ return m_eState; }
	bool						GetInitiated() const								{ return m_bInitiated; }
	Asura_Network_Result		GetResult() const									{ return m_eConnectionResult; }
	Asura_Close_Reason			GetCloseReason() const								{ return m_eCloseReason; }
	u_int						GetIdleTime() const;
	bool						HasTimedOut() const;

	char*						GetClientRequestData() const						{ return m_pcClientRequestData; }
	void						ClearClientRequestData();

	// These need to be public due to the NetSocket code needing to access them
	// NOTE:	the names of the functions may be slightly misleading, they only apply to challenge/connection messages.
	//			Also, remember that platform data may be seen by other platforms in x-platform networking situations
	static u_int Platform_GetMessageDataSize( const u_int uMessageID );
	static void	Platform_FillMessageData( const u_int uMessageID, void* pData );
	static void	Platform_ProcessMessageData( const u_int uMessageID, const void* pData );
	
	
private:
	enum MsgLogType
	{
		ANTSCON_SENT,
		ANTSCON_RECEIVED,
		ANTSCON_RETRY,
	};

	void						LogMessage(u_short usMessageID, u_short usSN, MsgLogType eType); 
	void						LogToFile(const char* szMessage, ...);
	unsigned short				GetNextValidSequenceNumber	();

	void						Platform_CloseConnection		();
	bool						Platform_HandlesClientRequests	();

	Asura_NetAddress			m_xAddress;
	unsigned short				m_usPort;
	Asura_NetSocket*			m_pxSocket;
	Asura_Connection_State		m_eState;
	Asura_Network_Result		m_eConnectionResult;
	Asura_Close_Reason			m_eCloseReason;
	Asura_Network_MachineInfo*	m_pxMachineInfo;
	Asura_Network_Buffer		m_xOutgoingBuffer;
	bool						m_bInitiated;
	bool						m_bActive;
	unsigned short				m_usSequenceNumber;
	unsigned int				m_uLastReceiveTime;
	unsigned int				m_uLastKeepAliveSendTime;
	unsigned int				m_uLastPongSendTime;
	// This array starts with the ASURA_NETTRANSPORT_MSG_ACK identifier and the gamever
	// so that it can be used directly as the ACK message.
	unsigned short				m_ausNBOReceivedSNs[ASURA_NETCONNECTION_ACK_SIZE + 2];
	unsigned int				m_uReceivedSNCount;
	unsigned int				m_uLastAckSendTime;
	unsigned int				m_uLastResendTime;
	// Missing message buffer
	unsigned int				m_uNumberOfExpectedSNs;
	unsigned short				m_ausExpectedSNs[ASURA_NETCONNECTION_MAX_EXPECTED_SNS];
	unsigned short				m_usHighestSNReceived;
	// timer for connection closing
	float						m_fClosingTimer;

	// Saved message data if we need to delay processing of it for whatever reason
	char*						m_pcClientRequestData;
	static u_int				s_uNumSavedClientRequests;
};

#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#endif//__ASURA_NETCONNECTION_H__
