//
// Asura_NetSocket.cpp
//
// Created by KOR - 17/02/2005 11:10:20
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// This file's header.
#include "Network/Asura_NetSocket_BSD.h"

// Stdlib independent.
#include <stdlib.h>

// Asura headers
#ifdef ASURA_ENABLE_INTERNET_SIMULATION
#include "Network/Asura_InternetSimulation.h"
#endif
// ARC - 27 April 2006 11:48:35 - DemonWare integration
#if defined(ASURA_SUPPORT_DEMONWARE)
#include "Network/Asura_DemonWare.h"
#endif // ASURA_SUPPORT_DEMONWARE

// RJT NOTE: Required for GCC4 (well, psp-gcc 2.5.2 anyhow)
#ifndef	FD_SETSIZE
	#define	FD_SETSIZE	64
#endif

// on PS3 these are defined to check for negative numbers, not just 
// SOCKET_ERROR/INVALID_SOCKET. This is because PS3 "helpfully" returns 
// the actual error code for many functions although this seems to be 
// undocumented, the docs just say "negative number: error" and to check
// errno in the usual way. On Win32 not all functions return signed types
// (e.g. socket()) so we must check for equality with the error value.
#ifndef ASURA_NETSOCKET_IS_ERROR_CODE
#define ASURA_NETSOCKET_IS_ERROR_CODE( iReturnValue ) ((iReturnValue) == SOCKET_ERROR)
#endif
#ifndef ASURA_NETSOCKET_IS_INVALID_SOCKET
#define ASURA_NETSOCKET_IS_INVALID_SOCKET( iReturnValue ) ((iReturnValue) == INVALID_SOCKET)
#endif

int Asura_NetSocket_BSD::s_iProtocolToUse = ASURA_NETSOCKET_PROTOCOL;
// allow platform to optionally override default socket type (currently done on PS3)
#ifdef ASURA_NETSOCKET_SOCKETTYPE
int Asura_NetSocket_BSD::s_iSocketTypeToUse = ASURA_NETSOCKET_SOCKETTYPE;
#else
int Asura_NetSocket_BSD::s_iSocketTypeToUse = SOCK_DGRAM;
#endif


bool Asura_NetSocket_BSD::API_Bind(Asura_NetAddress xInAddress, u_short &usPort)
{
	Asura_NetSocket_BSD_AddressUnion xAddress;
	int iLength;

	bool bAllowPortScan = false;
	if (usPort == 0)
	{
		usPort = 
#ifdef ASURA_NETSOCK_ALTERNATIVE_PROTOCOL
			(m_bUseAlternativeProtocol) ? ASURA_NETSOCKET_ALTERNATIVE_PORT : 
#endif
			ASURA_NETSOCKET_DEFAULTPORT;
		Asura_Assert(usPort < ASURA_NETSOCKET_FIRST_DYNAMIC_PORT, "first dynamic port must be higher than default port");
		bAllowPortScan = true;
	}
	
	int iErrorCode;
	do
	{
		iErrorCode = 0;
		// Set up default options
		m_xAddress = xInAddress;
		m_usPort = usPort;
		m_xSocket = INVALID_SOCKET;
		m_bCloseRequested = false;
		m_bError = false;
		m_bConnected = false;

		// RJF - 28 January 2005 - Select protocol to use. If a broadcast protocol is not defined this will always evaluate to ASURA_NETSOCKET_PROTOCOL
		// otherwise it chooses between PROTOCOL and PROTOCOL_BROADCAST based on m_bBroadcastFlag
		int iProtocol =
#ifdef ASURA_NETSOCKET_ALTERNATIVE_PROTOCOL
			(m_bUseAlternativeProtocol) ? ASURA_NETSOCKET_ALTERNATIVE_PROTOCOL :
#endif
			s_iProtocolToUse;

		// Attempt to create socket
		m_xSocket = socket(AF_INET, s_iSocketTypeToUse, iProtocol);
		if(ASURA_NETSOCKET_IS_INVALID_SOCKET(m_xSocket))
		{
			iErrorCode = WSAGetLastError();
			switch(iErrorCode)
			{
				// (MH - 20.07.2004) sadly there is no equivalent to WSANOTINITIALISED on PS2
				// case WSANOTINITIALISED:		Asura_Assert(0, "Failed to create socket: WSANOTINITIALISED");		break;
				case WSAENETDOWN:			Asura_Assert(0, "Failed to create socket: WSAENETDOWN");			break;
				case WSAEAFNOSUPPORT:		Asura_Assert(0, "Failed to create socket: WSAEAFNOSUPPORT");		break;
				case WSAENOBUFS:			Asura_Assert(0, "Failed to create socket: WSAENOBUFS");				break;
				#ifdef WSAEPROTONOSUPPORT	// not defined on PSP
				case WSAEPROTONOSUPPORT:	Asura_Assert(0, "Failed to create socket: WSAEPROTONOSUPPORT");		break;
				#endif // #ifdef WSAEPROTONOSUPPORT
				default:					Asura_Assert(0, "Failed to create socket: Unknown error");			break;
			}		
			return false;
		}


		// Bind socket
		Asura_Mem::Set(&xAddress, 0, sizeof(SOCKADDR_IN));
		xAddress.xSockAddrIn.sin_family = AF_INET;
		xAddress.xSockAddrIn.sin_addr.s_addr = INADDR_ANY;
		xAddress.xSockAddrIn.sin_port = htons(usPort);
#ifdef ASURA_NETSOCKET_FIXUPSOCKADDRIN
		ASURA_NETSOCKET_FIXUPSOCKADDRIN(xAddress.xSockAddrIn);
#endif
		int iRet = bind(m_xSocket, &xAddress.xSockAddr, sizeof(SOCKADDR_IN));
		if (ASURA_NETSOCKET_IS_ERROR_CODE(iRet))
		{
			iErrorCode = WSAGetLastError();
			switch(iErrorCode)
			{
				// (MH - 20.07.2004) sadly there is no equivalent to WSANOTINITIALISED on PS2
				// case WSANOTINITIALISED:		Asura_Assert(0, "Failed to create socket: WSANOTINITIALISED");		break;
				case WSAEADDRINUSE:
					{
						//Asura_CommandConsole::Printf(L"Port %d already in use.", usPort);
						break;
					}
				case WSAENETDOWN:			Asura_Assert(0, "Failed to create socket: WSAENETDOWN");			break;
				case WSAEAFNOSUPPORT:		Asura_Assert(0, "Failed to create socket: WSAEAFNOSUPPORT");		break;
				case WSAENOBUFS:			Asura_Assert(0, "Failed to create socket: WSAENOBUFS");				break;
				#ifdef WSAEPROTONOSUPPORT // not available on PSP
				case WSAEPROTONOSUPPORT:	Asura_Assert(0, "Failed to create socket: WSAEPROTONOSUPPORT");		break;
				#endif // #ifdef WSAEPROTONOSUPPORT
				default:					Asura_Assert(0, "Failed to create socket: Unknown error");			break;
			}		
			Asura_Assert(iErrorCode == WSAEADDRINUSE, "Failed to bind socket.");
			closesocket(m_xSocket);
			m_xSocket = INVALID_SOCKET;

			if ( (!bAllowPortScan) || (usPort >= (ASURA_NETSOCKET_FIRST_DYNAMIC_PORT + 64)) || (iErrorCode != WSAEADDRINUSE) )
			{
				// only allow scan of up to 64 ports to find a free one
				return false;
			}

			// if default port failed, use dynamic port range.
			usPort = (usPort == ASURA_NETSOCKET_DEFAULTPORT) ? ASURA_NETSOCKET_FIRST_DYNAMIC_PORT : (++usPort);
		}
	}
	while (iErrorCode != 0);

	//Asura_CommandConsole::Printf(L"Network socket binded. Using Port %d.", usPort);

	// Set up members
	iLength = sizeof(SOCKADDR_IN);
	getsockname(m_xSocket, &xAddress.xSockAddr, &iLength);
	m_xAddress = xAddress.xSockAddrIn.sin_addr.s_addr;
	m_usPort = ntohs(xAddress.xSockAddrIn.sin_port);
	m_bConnected = true;
	m_bListening = false;
	m_uNumConnections = 0;

	// Set as non blocking
	Platform_SetToNonBlocking();
	
	return true;
}

void Asura_NetSocket_BSD::API_Close()
{
	if(m_bConnected)
	{
// ARC - 26 April 2006 10:50:09 - Don't close the socket - handled by DemonWare
#if !defined(ASURA_SUPPORT_DEMONWARE)
		closesocket(m_xSocket);
#endif // !ASURA_SUPPORT_DEMONWARE
		m_bConnected = false;
	}
}

bool Asura_NetSocket_BSD::API_ReceiveMessages()
{
// ARC - 26 April 2006 10:50:48 - Messages are received in Asura_DemonWare_PacketInterceptor
#if defined(ASURA_SUPPORT_DEMONWARE)
	Asura_Assert(Asura_Demonware::IsRunning(), "Asura_Demonware not running in API_ReceiveMessages()");
	if (Asura_Demonware::IsRunning())
	{
		Asura_Demonware* pxDemonWare = Asura_Demonware::GetInstance();
		while (pxDemonWare->ArePacketsPending())
		{
			if (!pxDemonWare->HandleNextPacket(this))
			{
				return false;
			}
		}

		return true;
	}

	// DemonWare not running so no messages received
	return false;
#else // !ASURA_SUPPORT_DEMONWARE
#ifdef ASURA_ENABLE_INTERNET_SIMULATION
	// get any buffered messages from Asura_InternetSimulation
	Asura_InternetSimulation::Update(this, HandleMessage_Callback);
#endif // #ifdef ASURA_ENABLE_INTERNET_SIMULATION

	fd_set xFDSet;
	struct timeval timeout = {0, 0};
	int iNumSockets;

	FD_ZERO(&xFDSet);
	FD_SET(m_xSocket, &xFDSet);

	while((iNumSockets = select(FD_SETSIZE, &xFDSet, NULL, NULL, &timeout)) > 0)
	{
		Asura_NetSocket_BSD_AddressUnion xAddress;
		int iAddressLength;
		int iBytesReceived;
		char pcBuffer[ASURA_RECEIVE_BUFFER_SIZE];

		iAddressLength = sizeof(xAddress);
		iBytesReceived = recvfrom(m_xSocket, pcBuffer, sizeof(pcBuffer), 0, &xAddress.xSockAddr, &iAddressLength);
		if(ASURA_NETSOCKET_IS_ERROR_CODE(iBytesReceived))
		{
			int iLastError = WSAGetLastError();
			if(iLastError == WSAECONNRESET)
			{
				if(!ResetConnection(Asura_NetAddress(xAddress.xSockAddrIn.sin_addr.s_addr), ntohs(xAddress.xSockAddrIn.sin_port)))
				{
					return false;
				}
			}
			else if(iLastError != WSAEMSGSIZE)
			{
				FatalError();
				return false;
			}
		}
		else
		{
			Asura_Assert(iBytesReceived < ASURA_RECEIVE_BUFFER_SIZE, "Message too large - buffer over-run. Will probably crash soon");

#ifdef ASURA_ENABLE_INTERNET_SIMULATION
			if (!Asura_InternetSimulation::AddToBuffer( pcBuffer, iBytesReceived, Asura_NetAddress(xAddress.xSockAddrIn.sin_addr.s_addr), ntohs(xAddress.xSockAddrIn.sin_port )))
#endif
			{
				if(!HandleMessage(pcBuffer, iBytesReceived, Asura_NetAddress(xAddress.xSockAddrIn.sin_addr.s_addr), ntohs(xAddress.xSockAddrIn.sin_port)))
				{
					return false;
				}
			}
		}
	}
	Asura_Assert(!ASURA_NETSOCKET_IS_ERROR_CODE(iNumSockets), "Error selecting socket.");
	return true;
#endif // !ASURA_SUPPORT_DEMONWARE
}

bool Asura_NetSocket_BSD::API_SendTo(const Asura_NetSocket_MessageInfo* pxMessageInfo)
{
#if defined(ASURA_SUPPORT_DEMONWARE)
// ARC - 27 April 2006 11:48:53 - Use DemonWare to send the packet
	Asura_Assert(Asura_Demonware::IsRunning(), "Asura_Demonware not running in API_SendTo()");
	return Asura_Demonware::GetInstance()->SendPacket(pxMessageInfo);
#else
	// Do a select to find out if we can send
	fd_set xFDSet;
	struct timeval xTimeOut = {0, 0};
	int iNumSockets;

	FD_ZERO(&xFDSet);
	FD_SET(m_xSocket, &xFDSet);

	iNumSockets = select(FD_SETSIZE, NULL, &xFDSet, NULL, &xTimeOut);
	Asura_Assert(!ASURA_NETSOCKET_IS_ERROR_CODE(iNumSockets), "Error selecting socket");
	if(ASURA_NETSOCKET_IS_ERROR_CODE(iNumSockets) || !iNumSockets)
	{
		return false;
	}

	return Platform_SendTo(pxMessageInfo);
#endif // ASURA_SUPPORT_DEMONWARE
}

void Asura_NetSocket_BSD::API_SetBroadcastEnabled(bool bEnabled)
{
	int iBroadcast = bEnabled ? 1 : 0;
#ifdef ASURA_ASSERT
	int iRcode =
#endif
	setsockopt(m_xSocket, SOL_SOCKET, SO_BROADCAST, (char *)&iBroadcast, sizeof(iBroadcast));
	Asura_Assert(!ASURA_NETSOCKET_IS_ERROR_CODE(iRcode), "setsockopt error");
	return;
}

int Asura_NetSocket_BSD::API_GetLastError()
{
	return WSAGetLastError();
}

// static function
void Asura_NetSocket_BSD::GetAddressFromString(const char* szAddress, u_int* puAddress, unsigned short* pusPort)
{
	// Passing in a null address is valid - simply return 0
	//Asura_Assert(szAddress, "No address given");
	unsigned int uIP;
	unsigned short usPort;

	// Check whether the string is null or empty
	if(!szAddress || !szAddress[0])
	{
		uIP = 0;
		usPort = 0;
	}
	else
	{
		char szAddressCopy[32];
		strcpy(szAddressCopy, szAddress);
		// Find the port
		char* szPort = strchr(szAddressCopy, ':');
		if(!szPort)
		{
			usPort = 0;
		}
		else
		{
			int iPort = atoi(szPort + 1);
			if(iPort < 0 || iPort > 0xffff)
			{
				usPort = 0;
			}
			else
			{
				usPort = (unsigned short)iPort;
			}
			szPort[0] = 0;
		}
		// Use inet_addr to get the IP
		uIP = inet_addr(szAddressCopy);
		if(uIP == INADDR_NONE)
		{
			uIP = 0;
		}
	}
	// Copy values into pointers
	if(puAddress) *puAddress = uIP;
	if(pusPort) * pusPort = usPort;
}

// static function
const char* Asura_NetSocket_BSD::GetStringFromAddress(u_int uAddress, unsigned short usPort)
{
	in_addr xAddr;
	xAddr.s_addr = uAddress;
	static char szAddress[sizeof("XXX.XXX.XXX.XXX:XXXXX")];
	Asura_PrintfToBfr(szAddress, "%s:%u", sizeof("XXX.XXX.XXX.XXX:XXXXX"), true,
					 inet_ntoa(xAddr), usPort);
	return szAddress;
}

#endif //#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
