//
// Asura_NetTransport_System.cpp
//
// Created by KOR - 29/10/2003 17:35:46
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// This file's header.
#include "Network/Asura_NetTransport_System.h"

// Asura includes
#include "Core/Asura_Hash.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_Timers.h"
#include "Core/Memory/Asura_Mem.h"
#include "Debugging/Asura_DebugText.h"
#include "Debugging/Asura_Logfile.h"
#include "Network/Asura_InternetSimulation.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_MessageHeader.h"
#include "Network/Asura_NetMsg.h"
#include "Network/Asura_Network.h"
#include "Network/Asura_NetConnection.h"
#include "Network/Asura_NetSocket.h"
#ifdef ASURA_SUPPORT_LOBBY
#include "Network/Online/Asura_Lobby.h"
#endif // #ifdef ASURA_SUPPORT_LOBBY

// middleware includes
#ifdef ASURA_SUPPORT_GAMESPY
#include "Network/GameSpy/Asura_GameSpy_Peer.h"
#include "Network/GameSpy/Asura_GameSpy_QR2.h"
#endif//#ifdef ASURA_SUPPORT_GAMESPY
#ifdef ASURA_SUPPORT_NOVAWORLD
#include "NovaWorld/NovaworldAPI.h"
#endif//#ifdef ASURA_SUPPORT_NOVAWORLD
// ARC - 10 April 2006 17:26:44 - Add DemonWare support
#if defined(ASURA_SUPPORT_DEMONWARE)
#include "Network/Demonware/Asura_DemonWare.h"
#endif // ASURA_SUPPORT_DEMONWARE

#include "Interface/Asura_CommandConsole.h"

// D E F I N E S /////////////////////////////////////////////////////////////

#define ASURA_NETTRANSPORT_SYSTEM_BUFFERSIZE (2048)	// max amount of data (in bytes) that can be sent in one go.
//#define ASURA_NETTRANSPORT_SYSTEM_SOCKET_INCOMING_BUFFERSIZE (4096)	
//#define ASURA_NETTRANSPORT_SYSTEM_SOCKET_OUTGOING_BUFFERSIZE (4096)	

#if defined(ASURA_SUPPORT_GAMESPY)
const u_int ASURA_GAMESPY_ANTS_NATNEG_CLIENT = 0;
const u_int ASURA_GAMESPY_ANTS_NATNEG_SERVER = 1;
#endif

// D E B U G V A R S //////////////////////////////////////////////////////////////////////////////

DEBUGVAR u_int dbg_uPingInterval = 2000;
DEBUGVAR float dbg_fEnumHostsRefreshTime = 1.5f;
DEBUGVAR float dbg_fEnumHostsExpiryTime = 2.0f;
DEBUGVAR float dbg_fTimeBetweenBandwidthCalculations = 5.0f;
DVSVAR bool	g_bDisplayStats	= false;

#ifdef ASURA_NETTRANSPORT_BANDWIDTH_LOG
#define ASURA_NETTRANSPORT_BANDWIDTH_LOG_FILENAME "logfiles\\bandwidth.csv"
#ifndef ASURA_NETTRANSPORT_BANDWIDTH_LOG_CALCULATIONS_PER_ENTRY
#define ASURA_NETTRANSPORT_BANDWIDTH_LOG_CALCULATIONS_PER_ENTRY (3)
#endif
DEBUGVAR float dbg_fTimeBetweenBandwidthLogs = dbg_fTimeBetweenBandwidthCalculations * ASURA_NETTRANSPORT_BANDWIDTH_LOG_CALCULATIONS_PER_ENTRY;
#endif

// S T A T I C   D A T A //////////////////////////////////////////////////////////////////////////

// general
bool						Asura_NetTransport_System::s_bSocketCreatedByANTS = true;
Asura_NetSocket*			Asura_NetTransport_System::s_pxSocket = 0;
unsigned short				Asura_NetTransport_System::s_usPort = 0;		// client port. if not explicitly set, set to platform default
unsigned short				Asura_NetTransport_System::s_usServerPort = 0;	// port to connect to.  if not explicitly set, set to platform default
int							Asura_NetTransport_System::s_iNetAdapter = 0;
u_int						Asura_NetTransport_System::s_uPasswordHash = 0;
u_int						Asura_NetTransport_System::s_uServerSessionID;	// randomly-generated ID for this hosting session
u_int						Asura_NetTransport_System::s_auServerChallenge[2];	// current and previous server challenge value
u_int						Asura_NetTransport_System::s_uServerChallengeTime;
u_char*						Asura_NetTransport_System::s_pucSessionData = NULL;
u_int						Asura_NetTransport_System::s_uSessionDataSize = 0;
float						Asura_NetTransport_System::s_fTimeSinceLastBandwidthCalculation;
u_int						Asura_NetTransport_System::s_uSendBPS;
u_int						Asura_NetTransport_System::s_uReceiveBPS;
u_int						Asura_NetTransport_System::s_uPeakSendBPS;
u_int						Asura_NetTransport_System::s_uPeakReceiveBPS;
u_int						Asura_NetTransport_System::s_uDirectSendBytes;
u_int						Asura_NetTransport_System::s_uDirectReceiveBytes;
#ifdef ASURA_NETTRANSPORT_BANDWIDTH_LOG
u_int*						Asura_NetTransport_System::s_puAdverageBPSSent = NULL;
u_int*						Asura_NetTransport_System::s_puAdverageBPSReceived = NULL;
float						Asura_NetTransport_System::s_fTimeSinceLastBandwidthLog = 0.0f;
#endif
Asura_Network_MachineInfo*	Asura_NetTransport_System::s_pxMachineInfo = 0;
u_int						Asura_NetTransport_System::s_uMaxNumberOfMachines = 0;
// static data - timeout
u_int						Asura_NetTransport_System::s_uNetConnectionTimeOut = ASURA_NETCONNECTION_TIMEOUT;
#if _DEBUG
bool						Asura_NetTransport_System::s_bDoConnectionTimeouts = false;
#else
bool						Asura_NetTransport_System::s_bDoConnectionTimeouts = true;
#endif// static data - server
bool						Asura_NetTransport_System::s_bServerReportingOn = false;
u_int						Asura_NetTransport_System::s_uMaxPlayersPerMachine = 1;
// static data - client
Asura_NetTransport_System::ClientConnectionState Asura_NetTransport_System::s_eClientConnectionState = CCS_NotConnected;
u_int						Asura_NetTransport_System::s_uNumberOfActiveSessions;
Asura_Network_SessionInfo** Asura_NetTransport_System::s_ppxSessionsStorage;
float						Asura_NetTransport_System::s_fLastSessionRequestBroadcastTime;
Asura_NetAddress			Asura_NetTransport_System::s_xAddressToConnectTo;
#if defined(ASURA_SUPPORT_GAMESPY)
int							Asura_NetTransport_System::s_iNatNegCookie = 0;
bool						Asura_NetTransport_System::s_bUseNatNeg = false;
#endif
bool						Asura_NetTransport_System::s_bPortSetExternally = false;

// G L O B A L S //////////////////////////////////////////////////////////////////////////////////

// RDM - 06/11/06 - Changed how this is allocated to ensure 4-byte alignment. This was previously breaking the PSP.
static int   g_aiBuffer[ASURA_NETTRANSPORT_SYSTEM_BUFFERSIZE/sizeof(int)];
static char* g_pcBufferPtr = reinterpret_cast<char*> (g_aiBuffer);

// C L A S S   I M P L E M E N T A T I O N ////////////////////////////////////////////////////////

// Asura_Network_Buffer ///////////////////////////////////////////////////////////////////////////

Asura_Network_Buffer::Asura_Network_Buffer()
: m_pxIndexTree(new Asura_HashTree())
, m_uNumberOfMessages(0)
, m_uTotalMessageSize(0)
{
}

Asura_Network_Buffer::~Asura_Network_Buffer()
{
	Reset();
	delete m_pxIndexTree;
}

void Asura_Network_Buffer::Reset()
{
	m_pxIndexTree->Reset();
	for(u_int u = 0; u < m_uNumberOfMessages; ++u)
	{
		Asura_Assert(m_axMessageBuffer[u].IsValid(), "Missing message data");
		m_axMessageBuffer[u].DeleteBuffer();
		m_axMessageBuffer[u].RemoveEntry();
	}
	m_uNumberOfMessages = 0;
	m_uTotalMessageSize = 0;
}

bool Asura_Network_Buffer::AddMessage(unsigned int uSendTime, char *pcMessage, unsigned int uMessageLength)
{
	Asura_Assert(pcMessage && uMessageLength, "No message.");
	if(!(pcMessage && uMessageLength))
	{
		return false;
	}

	Asura_Assert(m_uTotalMessageSize + uMessageLength <= ASURA_NETTRANSPORT_BUFFER_SIZE_LIMIT, "Buffer too large.");
	if(m_uTotalMessageSize + uMessageLength > ASURA_NETTRANSPORT_BUFFER_SIZE_LIMIT)
	{
		Asura_OutputToDebugger(("outgoing buffer: out of memory ( %u + %u > %u )\n", m_uTotalMessageSize, uMessageLength, ASURA_NETTRANSPORT_BUFFER_SIZE_LIMIT ));
		return false;
	}

	Asura_Assert(m_uNumberOfMessages < ASURA_NETWORK_BUFFER_MAX_MESSAGES, "Too many messages");
	if(m_uNumberOfMessages >= ASURA_NETWORK_BUFFER_MAX_MESSAGES)
	{
// (MH - 20.07.2004) if we have to remove a message from the buffer that means it probably
// won't get delivered. Better to disconnect that risk a cucial message going astray.
//		// Remove the oldest message
//		unsigned int uEarliestTime = m_axMessageBuffer[0].m_uSendTime;
//		unsigned int uEarliestIndex = 0;
//		for(u_int u = 1; u < ASURA_NETWORK_BUFFER_MAX_MESSAGES; ++u)
//		{
//			if(uEarliestTime > m_axMessageBuffer[u].m_uSendTime)
//			{
//				uEarliestTime = m_axMessageBuffer[u].m_uSendTime;
//				uEarliestIndex = u;
//			}
//		}
//		RemoveMessage(*(unsigned short*)m_axMessageBuffer[uEarliestIndex].m_pcBuffer);
		Asura_OutputToDebugger(("outgoing buffer: too many messages\n"));
		return false;
	}

	unsigned short usSN = *(unsigned short*)pcMessage;
	int iIndex;
	if(m_pxIndexTree->FindNodeIndexByHash((unsigned int)usSN, true, iIndex))
	{
		Asura_Assert(0, "Message already exists in buffer");
		return false;
	}
	Asura_Assert(iIndex >= 0 && iIndex < int(ASURA_NETWORK_BUFFER_MAX_MESSAGES), "Index out of range");
	m_axMessageBuffer[iIndex].m_uSendTime = uSendTime;
	m_axMessageBuffer[iIndex].m_pcBuffer = pcMessage;
	m_axMessageBuffer[iIndex].m_uBufferSize = uMessageLength;
#ifdef _DEBUG
	m_axMessageBuffer[iIndex].m_usSN = *(unsigned short*)pcMessage;
#endif
	m_uNumberOfMessages++;
	m_uTotalMessageSize+=uMessageLength;
	return true;
}

void Asura_Network_Buffer::RemoveMessage(unsigned short usSN)
{
	int iIndex = -1;
	if(!m_pxIndexTree->DeleteNodeByHash((unsigned int)usSN, iIndex))
	{
		// It's possible for two consecutive acks for the same message to arrive, so just ignore
		//Asura_Assert(0, "Message not found");
		return;
	}
	--m_uNumberOfMessages;
	m_uTotalMessageSize -= m_axMessageBuffer[iIndex].m_uBufferSize;
	m_axMessageBuffer[iIndex].DeleteBuffer();
	m_axMessageBuffer[iIndex] = m_axMessageBuffer[m_uNumberOfMessages];
	m_axMessageBuffer[m_uNumberOfMessages].RemoveEntry();
}
/*
const Asura_Network_Buffer_Entry *Asura_Network_Buffer::GetMessage(unsigned short usSN) const
{
	int iIndex;
	if(!m_pxIndexTree->FindNodeIndexByHash((unsigned int)usSN, true, iIndex))
	{
		return 0;
	}
	return &m_axMessageBuffer[iIndex];
}
*/
const Asura_Network_Buffer_Entry *Asura_Network_Buffer::GetMessage(unsigned int uIndex) const
{
	Asura_Assert(uIndex < m_uNumberOfMessages, "Index out of range.");
	if(uIndex >= m_uNumberOfMessages)
	{
		return NULL;
	}
	Asura_Assert(m_axMessageBuffer[uIndex].IsValid(), "Message is invalid.");
	return &m_axMessageBuffer[uIndex];
}

void Asura_Network_Buffer::CheckIntegrity() const
{
#ifdef _DEBUG
	for(u_int u = 0; u < m_uNumberOfMessages; ++u)
	{
		Asura_Assert(m_axMessageBuffer[u].IsValid(), "Message is invalid.");
	}
#endif
}

// Asura_NetTransport_System //////////////////////////////////////////////////////////////////////

void Asura_NetTransport_System::Initialise( void )
{
	Asura_Assert(!s_pxMachineInfo, "Asura Network Transport System already initialised");
	if(s_pxMachineInfo) return;

	// (MH - 20.07.2004) if the server port hasn't been explicitly set (e.g on command line) 
	// set it to the platform's default.
	if (s_usServerPort == 0)
	{
		s_usServerPort = ASURA_NETSOCKET_DEFAULTPORT;
	}

	Platform_Initialise();

	// RJF - 31 January 2005 - BEWARE
	// This relies on data pulled from Asura_Network - which is not guaranteed to be running
	// If you start ANTS without Asura_Network - e.g. for the NetSession system - the data WILL 
	// be stale in such a situation. This really needs fixing at some point...

	// If the external network is not active, then this function was not called by Asura network, and we should
	// assume that its being used in a serverless manner (i.e., game finding, voice transmission)
	// NOTE: Don't just check for Asura_Network::IsActive, as the network could be active locally to provide
	// a menu background, a la BF3 - PJC 09/04/09

	if (Asura_Network::IsServer() && Asura_Network::IsExternalNetworkActive())
	{
		s_uMaxNumberOfMachines = Asura_Network::GetMaxNumberOfMachines();
	}
	else
	{
		s_uMaxNumberOfMachines = 1;
	}
	s_pxMachineInfo = new Asura_Network_MachineInfo[s_uMaxNumberOfMachines];

// SNH - 13/10/2004 11:05:40 - changed to be consistant with GT2
//	s_pxSocket = new Asura_NetSocket();
//	// if s_usPort is non-zero that port will be used or the Bind will fail.
//	// if it is zero, the used value is written back.
//	s_pxSocket->Bind(0, s_usPort);
//	Asura_Assert(s_pxSocket->IsConnected(), "failed to create local socket");
	if(!s_pxSocket)
	{
		GetSocket();
		s_bSocketCreatedByANTS = true;
	}
	else
	{
		// socket already present, so leave it when we shutdown
		s_bSocketCreatedByANTS = false;
	}
	
	s_pucSessionData = NULL;
	s_uSessionDataSize = 0;
	s_fLastSessionRequestBroadcastTime = 0.0f;

	s_fTimeSinceLastBandwidthCalculation = 0.0f;
	s_uPeakSendBPS = 0;
	s_uPeakReceiveBPS = 0;
	s_uDirectSendBytes = 0;
	s_uDirectReceiveBytes = 0;

#ifdef ASURA_NETTRANSPORT_BANDWIDTH_LOG
	u_int uNumSlots = s_uMaxNumberOfMachines + 1;
	s_puAdverageBPSSent = new u_int[uNumSlots];
	s_puAdverageBPSReceived = new u_int[uNumSlots];
	Asura_Mem::Set( s_puAdverageBPSSent, 0, sizeof(u_int) * uNumSlots );
	Asura_Mem::Set( s_puAdverageBPSReceived, 0, sizeof(u_int) * uNumSlots );

	Asura_SimpleLogfileClear( ASURA_NETTRANSPORT_BANDWIDTH_LOG_FILENAME );
	Asura_SimpleLogfileOutput((ASURA_NETTRANSPORT_BANDWIDTH_LOG_FILENAME, "UP: Upstream average bandwidth from (M) specific machine or (SVR) all data incoming to server. Unit - BPS.\n" ));
	Asura_SimpleLogfileOutput((ASURA_NETTRANSPORT_BANDWIDTH_LOG_FILENAME, "DWN: Downstream average bandwidth to (M) specific machine or (SVR) all data outgoing from server. Unit - BPS.\n" ));

	const u_int uLogTextSize = 1024;
	char acLogText[uLogTextSize];
    u_int uCharsOutput = 0;

	uCharsOutput += Asura_PrintfToBfr( &acLogText[uCharsOutput], "FRAME,", uLogTextSize - uCharsOutput, true );
	for (u_int uSent = 0; uSent < 2; uSent++)
	{
        for (u_int uMachine = 0; uMachine < s_uMaxNumberOfMachines; uMachine++)
        {
            uCharsOutput += Asura_PrintfToBfr( &acLogText[uCharsOutput], " M%02d %s,", uLogTextSize - uCharsOutput, true, uMachine, (uSent ? "DWN" : "UP") );
        }

        uCharsOutput += Asura_PrintfToBfr( &acLogText[uCharsOutput], " SVR %s", uLogTextSize - uCharsOutput, true, (uSent ? "DWN" : "UP") );
	}

	Asura_SimpleLogfileOutput(( ASURA_NETTRANSPORT_BANDWIDTH_LOG_FILENAME, "%s\n", acLogText ));
#endif

	//-------------------------------------------------------------------
	// OR - 30/11/2004 10:15:32 - Force logs
	//-------------------------------------------------------------------
	Asura_OutputToDebugger(("Asura_NetTransport_System :: Initialised\n"));
	SetConnectionsTimeOutEnabled(s_bDoConnectionTimeouts);
	SetConnectionsTimeOutLimit(s_uNetConnectionTimeOut);
	
#ifdef ASURA_ENABLE_INTERNET_SIMULATION
	Asura_InternetSimulation::Initialise();
#endif

	// (MH - 14.04.2005) if hosting a game, call the same code that gets called when clients connect.

	// If the external network is not active, then this function was not called by Asura network, and we should
	// assume that its being used in a serverless manner (i.e., game finding, voice transmission)
	// NOTE: Don't just check for Asura_Network::IsActive, as the network could be active locally to provide
	// a menu background, a la BF3 - PJC 09/04/09
	if (Asura_Network::IsServer() && Asura_Network::GetNumberOfPlayers() && Asura_Network::IsExternalNetworkActive())
	{
		u_int uMessageSize = Asura_NetTransport_System::GetSizeOfConnectData();
		char* pcUserConnectData = new char[uMessageSize];
		if (pcUserConnectData)
		{
			Asura_NetTransport_System::FillOutConnectData(pcUserConnectData);
			const void* pExtraConnectData = pcUserConnectData + sizeof(Asura_Network_ConnectData);
			Project_ProcessConnection( Asura_NetAddress(0u), 0, Asura_Network::GetClientID(), pExtraConnectData );

#ifdef ASURA_SUPPORT_LOBBY
			if (Asura_Lobby_System::IsInitialised())
			{
				Asura_Assert(Asura_Network::GetNumberOfPlayers() == 1,"Asura_Lobby does not currently support mutiple local players");
				Asura_Lobby_System::SetLocalPlayerGuid( ASURA_GUID_CLIENT_START );
			}
#endif
		}
		delete[] pcUserConnectData;
	}
}

Asura_NetSocket* Asura_NetTransport_System::GetSocket()
{
	if(!s_pxSocket)
	{
		s_pxSocket = Platform_AllocateNetSocket( s_uMaxNumberOfMachines );
#if defined(ASURA_SUPPORT_DEMONWARE)
// ARC - 26 April 2006 10:52:22 - Get the DemonWare port
		Asura_Assert(Asura_Demonware::IsRunning(), "Asura_Demonware not running in GetSocket()");
		s_usPort = Asura_Demonware::GetInstance()->GetPort();

		// Set as non blocking
		//s_pxSocket->Platform_SetToNonBlocking();
#else
		Asura_NetAddress xAddress = Asura_NetAddress(0u);
		s_pxSocket->Bind(xAddress, s_usPort);
#endif // ASURA_SUPPORT_DEMONWARE
		// (MH - 10.10.2005) in case locally-bound port is to be determined 
		// by socket code, set it now, otherwise other calls may fail.
		if (!s_usPort)
		{
			s_usPort = s_pxSocket->GetPort();
		}
		Asura_Assert( GetPort() == s_pxSocket->GetPort(), "local port mismatch");

		// mark the socket for manual deletion. If ANTS is creating this
		//	socket itself (in Initialise() above) this is set to true there...
		s_bSocketCreatedByANTS = false;

		Asura_Assert(s_pxSocket->IsConnected(), "failed to create local socket");
	}
	
	return s_pxSocket; 
}

void Asura_NetTransport_System::CloseSocket()
{
	Asura_Assert(!s_pxMachineInfo, "Can't remove socket when ANTS initialised");

	if(s_pxSocket && !s_pxMachineInfo && !s_bSocketCreatedByANTS)
	{
		Asura_NetSocket::Destroy( s_pxSocket );
		s_pxSocket = NULL;
		if (!s_bPortSetExternally)
		{
			s_usPort = 0;
		}
	}
}

const char* Asura_NetTransport_System::GetNetAdapterIPStr( int iAdapter )
{
	static char s_szAddress[32];

	u_int uIP = GetNetAdapterIP(iAdapter);
	u_short usPort = GetPort();

	Asura_PrintfToBfr(s_szAddress, "%u.%u.%u.%u:%d", 32, true, ((uIP >> 0) & 0xff), ((uIP >> 8) & 0xff), ((uIP >> 16) & 0xff), ((uIP >> 24) & 0xff), usPort);

	return s_szAddress;
}

const Asura_UnicodeChar* Asura_NetTransport_System::GetNetAdapterIPWStr( int iAdapter )
{
	static Asura_UnicodeChar s_wszAddress[32];
	
	const char* szIP = GetNetAdapterIPStr(iAdapter);

	Asura_PrintfToBfr(s_wszAddress, L"%S", 32, true, szIP);

	return s_wszAddress;
}

void Asura_NetTransport_System::Update( void )
{
#ifdef ASURA_SUPPORT_NOVAWORLD
	NovaworldAPI::Update();
#endif
		
#ifdef ASURA_SUPPORT_NOVAWORLD
	if (Asura_Network::IsExternalNetworkActive() && Asura_Network::IsServer())
	{
		NovaworldAPI_Server::Update();
	}
	NovaworldAPI_BaffleProtocol::Update();
#endif

	if (Asura_Network::IsExternalNetworkActive() && Asura_Network::IsServer())
	{
		// update the challenge value
		u_int uCurrentTime = Asura_Timers::GetMSecsSinceStart();
		if (s_uServerChallengeTime < uCurrentTime)
		{
			// remeber the current challenge and create a new one
			s_auServerChallenge[1] = s_auServerChallenge[0];
			s_auServerChallenge[0] = Asura_Random::GetIntU();
			s_uServerChallengeTime = uCurrentTime + 5000;
			
#ifdef ASURA_ENABLE_INTERNET_SIMULATION
			// send the netsim settings to all clients
			for (u_int u = 0; u < s_uMaxNumberOfMachines; ++u)
			{
				Asura_Network_MachineInfo& xMachineInfo = s_pxMachineInfo[u];
				if (xMachineInfo.IsValid())
				{
					Asura_NetSim_Settings xSettings;
					if (Asura_InternetSimulation::IsEnabled())
					{
						xSettings = Asura_InternetSimulation::GetSettings();
					}
					else
					{
						xSettings.m_uLatencyStddev = 0;
						xSettings.m_uLatencyMean = 0;
						xSettings.m_fDropPercent = 0.0f;
					}
					ASURA_NTS_CHANGE_BYTE_ORDER(xSettings.m_uLatencyStddev);
					ASURA_NTS_CHANGE_BYTE_ORDER(xSettings.m_uLatencyMean);
					ASURA_NTS_CHANGE_BYTE_ORDER(xSettings.m_fDropPercent);
					xMachineInfo.GetConnectionHandle()->SendSimple(ASURA_NETTRANSPORT_MSG_NETSIM_SETTINGS, 
						false, reinterpret_cast<const char*>(&xSettings), sizeof(Asura_NetSim_Settings));
				}
			}
#endif // #ifdef ASURA_ENABLE_INTERNET_SIMULATION
		}
	}

	if(s_pxMachineInfo)
	{
		u_int uCurrentTime = Asura_Timers::GetMSecsSinceStart();
		for (u_int u = 0; u < s_uMaxNumberOfMachines; ++u)
		{
			Asura_Network_MachineInfo& xMachineInfo = s_pxMachineInfo[u];
			if (xMachineInfo.IsValid())
			{
				u_int uTimeSinceLastPing = uCurrentTime - xMachineInfo.GetTimeOfLastPing();
				if (uTimeSinceLastPing > dbg_uPingInterval)
				{
					xMachineInfo.GetConnectionHandle()->Ping();
					xMachineInfo.SetTimeOfLastPing(uCurrentTime);
				}
			}
		}
	}

#if defined(ASURA_SUPPORT_GAMESPY)
	// Update gamespy SDKs - ok to do it every frame
	Asura_GameSpy_Peer::Update();
	Asura_GameSpy_QR2::Update();
	NNThink();
#endif

// ARC - 10 April 2006 17:26:58 - Pump the DemonWare core
#if defined(ASURA_SUPPORT_DEMONWARE)
	if (Asura_Demonware::IsRunning())
	{
		Asura_Demonware::GetInstance()->Update(Asura_Timers::GetGameTimeSlice());
	}
#endif // ASURA_SUPPORT_DEMONWARE

	// SNH - 14/10/2004 10:38:00 - in a change from how this previously worked, StartClient no longer does the connecting.
	//	This allows us to wait for natneg to complete (if necessary) before actually connecting here
	switch(s_eClientConnectionState)
	{
		case CCS_WaitingForNatNeg:
		{
			break;
		}
		case CCS_ReadyToConnect:
		{
			ConnectToHost(s_xAddressToConnectTo, s_usServerPort);
			s_eClientConnectionState = CCS_WaitingForConnection;
			break;
		}
		default:	
		{
			// not a client, or nothing to do
			break;
		}
	}

	UpdateSocket();
	Platform_Update();

	PlatformProject_UpdateInternetSession();
	UpdateBandwidthStats();
}

void Asura_NetTransport_System::UpdateSocket( void )
{
	// SNH - 05/10/2004 11:01:16 - Asura_GameSpy_Peer may call this function before a socket has been created.
//	Asura_Assert(s_pxSocket, "no socket");
	if (s_pxSocket)
	{
		s_pxSocket->Update();
	}
}

void Asura_NetTransport_System::EndOfFrame( void )
{
	if (s_pxSocket)
	{
		s_pxSocket->Update();
	}

#if defined(ASURA_USE_DEBUG_TEXT)
	if(g_bDisplayStats)
	{
		Asura_DebugText::AddDefaultTextBlock(0xFFFFFFFF, 0, "BPS In:  %d (Max: %d)", s_uReceiveBPS, s_uPeakReceiveBPS);
		Asura_DebugText::AddDefaultTextBlock(0xFFFFFFFF, 0, "BPS Out: %d (Max: %d)", s_uSendBPS, s_uPeakSendBPS);
		Asura_DebugText::IncrementDefaultTextY();
	}
#endif	//defined(ASURA_USE_DEBUG_TEXT)
}


void Asura_NetTransport_System::Shutdown( void )
{
	s_uServerSessionID = 0;

#ifdef ASURA_ENABLE_INTERNET_SIMULATION
	Asura_InternetSimulation::Shutdown();
#endif

#ifdef ASURA_SUPPORT_NOVAWORLD
	NovaworldAPI_BaffleProtocol::Shutdown();
#endif

#if defined(ASURA_SUPPORT_GAMESPY)
	s_bUseNatNeg = false;
	s_iNatNegCookie = 0;
#endif

	s_eClientConnectionState = CCS_NotConnected;
	memset(&s_xAddressToConnectTo, 0, sizeof(Asura_NetAddress));

	//Asura_Assert(s_pxSocket, "no socket");
	if (s_pxSocket)
	{
		// SNH - 18/10/2004 11:12:52 - if socket was already created before ANTS was initialised, it's being used by 
		//	something else (eg Asura_GameSpy_Peer). Therefore don't delete it.
		if (s_bSocketCreatedByANTS)
		{
			Asura_NetSocket::Destroy( s_pxSocket );
			s_pxSocket = NULL;
			if (!s_bPortSetExternally)
			{
				s_usPort = 0;
			}
		}
		else
		{
			s_pxSocket->CloseAllConnections(true, ASURA_CLOSE_LOCAL);
		}
	}

	// (MH - 16.04.2005) process disconnection of host player
	if (Asura_Network::IsServer() && Asura_Network::GetNumberOfPlayers())
	{
		Project_ProcessDisconnection( Asura_Network::GetClientID() );
	}

#ifdef ASURA_NETTRANSPORT_BANDWIDTH_LOG
	Asura_SimpleLogfileOutput(( ASURA_NETTRANSPORT_BANDWIDTH_LOG_FILENAME, "\nSESSION PEAK SVR UP: %d", s_uPeakReceiveBPS ));
	Asura_SimpleLogfileOutput(( ASURA_NETTRANSPORT_BANDWIDTH_LOG_FILENAME, "\nSESSION PEAK SVR DWN: %d", s_uPeakSendBPS ));

	if (s_puAdverageBPSSent) delete[] s_puAdverageBPSSent;
	if (s_puAdverageBPSReceived) delete[] s_puAdverageBPSReceived;
#endif

	delete[] s_pxMachineInfo;
	s_pxMachineInfo = NULL;
	s_uMaxNumberOfMachines = 0;

	s_uSessionDataSize = 0;
	delete[] s_pucSessionData;
	s_pucSessionData = NULL;

	Platform_Shutdown();
}


#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
void Asura_NetTransport_System::AddConsoleCmdsAndVars( void )
{
	Asura_CommandConsole::AddVar( L"Network.DoTimeouts", &s_bDoConnectionTimeouts, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Network.TimeoutMilliseconds", &s_uNetConnectionTimeOut, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	// add some read-only vars for DVS display
	const Asura_UnicodeChar* wszBandwidthIn = L"Network.BpsIn";
	const Asura_UnicodeChar* wszBandwidthOut = L"Network.BpsOut";
	const Asura_UnicodeChar* wszBandwidthInMax = L"Network.BpsInMax";
	const Asura_UnicodeChar* wszBandwidthOutMax = L"Network.BpsOutMax";
	const Asura_UnicodeChar* wszDisplayStats = L"Network.DisplayStats";

	Asura_CommandConsole::AddVar( wszBandwidthIn, &s_uReceiveBPS, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN  );
	Asura_CommandConsole::MakeVarReadOnly( wszBandwidthIn );
	Asura_CommandConsole::AddVar( wszBandwidthOut, &s_uSendBPS, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszBandwidthOut );
	Asura_CommandConsole::AddVar( wszBandwidthInMax, &s_uPeakReceiveBPS, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN  );
	Asura_CommandConsole::MakeVarReadOnly( wszBandwidthInMax );
	Asura_CommandConsole::AddVar( wszBandwidthOutMax, &s_uPeakSendBPS, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( wszBandwidthOutMax );

	Asura_CommandConsole::AddVar(wszDisplayStats, &g_bDisplayStats, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
#endif
}
#endif


// Server code ////////////////////////////////////////////////////////////////////////////////////

bool Asura_NetTransport_System::StartServerAndHostSession(bool bInternetGame)
{
	// random number for session ID
	if (!s_uServerSessionID)
	{
		s_uServerSessionID = Asura_Random::GetIntU();
	}
	s_auServerChallenge[0] = Asura_Random::GetIntU();
	s_auServerChallenge[1] = Asura_Random::GetIntU();
	s_uServerChallengeTime = Asura_Timers::GetMSecsSinceStart() + 5000;

	Platform_StartServer(bInternetGame);
	Asura_Assert(s_pxSocket, "no socket");
	if (s_pxSocket)
	{
		bool bOK = true;

#ifdef ASURA_SUPPORT_GAMESPY
		// (MH - 13.12.2004) if peer is initialised then assume that will deal with reporting.
		// otherwise assume we need to setup QR2.
#ifndef ASURA_GAMESPY_PEER_CHAT_ONLY
		if (!Asura_GameSpy_Peer::IsInitialised())
#endif
		{
			bool bLanOnly = !bInternetGame;
			bOK = Asura_GameSpy_QR2::Initialise(bLanOnly, s_pxSocket->GetSOCKET(), s_usPort);
		}
#endif // #ifdef ASURA_SUPPORT_GAMESPY

		s_pxSocket->SetListening(true);
		if(bInternetGame)
		{
#ifdef ASURA_SUPPORT_NOVAWORLD
			NovaworldAPI_Server_Callbacks xCallbacks( Project_ServerDataCallback, Project_PlayerDataCallback );
			return NovaworldAPI_Server::Initialise(s_pxSocket->GetSOCKET(), xCallbacks);
#else
			return bOK;
#endif
		}
		else
		{
			// Enable broadcasts on the socket, for LAN server finding
			s_pxSocket->SetBroadcastEnabled(true);

			return bOK;
		}
	}
	return false;
}


void Asura_NetTransport_System::StopServer( void )
{
	Platform_StopServer();
	Asura_Assert(s_pxSocket, "no socket");
	if (s_pxSocket)
	{
		s_pxSocket->SetListening(false);

#ifdef ASURA_SUPPORT_NOVAWORLD
		NovaworldAPI_Server::Shutdown();
#endif

#ifdef ASURA_SUPPORT_GAMESPY
		// (MH - 13.12.2004) if QR2 was initialised then assume that we can shut it down now.
		Asura_GameSpy_QR2::Shutdown();
#endif // #ifdef ASURA_SUPPORT_GAMESPY
	}

	s_bServerReportingOn = false;
}


void Asura_NetTransport_System::SetServerLANReporting( bool bEnable )
{
	s_bServerReportingOn = bEnable;
}


void Asura_NetTransport_System::SendMessageToClient( u_int uMachine, const Asura_Message* pxMessage )
{
	Asura_Network_MachineInfo* pxMachineInfo = GetMachineInfoFromIndex(uMachine);
	if (pxMachineInfo && pxMachineInfo->IsValid())
	{
		SendAsuraMessage(pxMachineInfo->GetConnectionHandle(), pxMessage);
	}
}


void Asura_NetTransport_System::SendMessageToAllClients( const Asura_Message* pxMessage )
{
	bool bGuaranteed = pxMessage->IsUsingGuaranteedDelivery();
	// copy the message data to the temp global buffer.
	u_int uTotalSize = SetTempGlobalBufferFromAsuraMessage(pxMessage);

#ifdef ASURA_NETWORK_USE_BROADCAST_FOR_GAME_DATA
	// this should only be defined for platforms where it makes sense to spam
	// the entire network with game data, e.g. PSP in AdHoc mode where every 
	// machine on the local network is actaully in the game.
	if (!bGuaranteed)
	{
		// ASURA_NETADDRESS_BROADCAST should be defined in Asura_Platform_NetSocket.h 
		// iff ASURA_NETWORK_USE_BROADCAST_FOR_GAME_DATA is defined.
		Asura_NetAddress xAddress = ASURA_NETADDRESS_BROADCAST;
		SendMessageToAddress( g_pcBufferPtr, uTotalSize, xAddress, true );
		return;
	}
#endif

	for (u_int u = 0; u < s_uMaxNumberOfMachines; ++u)
	{
		if (s_pxMachineInfo[u].IsValid())
		{
			Asura_NetConnection* pxConnection = s_pxMachineInfo[u].GetConnectionHandle();
			pxConnection->Send(g_pcBufferPtr, uTotalSize, bGuaranteed);
			pxConnection->CheckBufferIntegrity();
		}
	}
}

void Asura_NetTransport_System::SetServerName( const Asura_UnicodeChar* wszServerName )
{
	// TODO - LAN broadcast stuff
	// (MH - 13.12.2004) this is done automatically in Platform_SendEnumHostsResponse() now.
}

//void Asura_NetTransport_System::SetSessionData( const Asura_Network_SessionData* pxData, u_int uNumBytes )
//{
//	if (s_pucSessionData)
//	{
//		Asura_Assert(uNumBytes == s_uSessionDataSize, "session data size changed");
//		// could delete and new here...
//	}
//	else
//	{
//		s_pucSessionData = new u_char[uNumBytes];
//		s_uSessionDataSize = uNumBytes;
//	}
//	Asura_Mem::Copy(s_pucSessionData, pxData, uNumBytes);
//
//	if (Asura_Network::IsServer())
//	{
//		Asura_Message* pxMsg = Asura_Network::CreateMessage(ASURA_MSG_NETWORKSESSIONDATA, uNumBytes, 0, pxData);
//		pxMsg->To = ASURA_GUID_ALL_OTHER_CLIENTMACHINES;
//		pxMsg->UseGuaranteedDelivery();
//		Asura_Network::Send(pxMsg);
//	}
//}


u_int Asura_NetTransport_System::GetLatencyForMachine( u_int uMachine )
{
	Asura_Assert(uMachine < s_uMaxNumberOfMachines, "invalid machine index");
	const Asura_Network_MachineInfo* pxInfo = &s_pxMachineInfo[uMachine];
	return pxInfo->GetPing();	
}

float Asura_NetTransport_System::GetPacketLossPercentForMachine( u_int uMachine )
{
	Asura_Assert(uMachine < s_uMaxNumberOfMachines, "invalid machine index");
	const Asura_Network_MachineInfo* pxInfo = &s_pxMachineInfo[uMachine];
	return pxInfo->GetPacketLossPercent();	
}


#if defined(ASURA_SUPPORT_GAMESPY)
void Asura_NetTransport_System::ServerBeginNatNeg(int iCookie)
{
	Asura_NetSocket* pxSocket = GetSocket();
	if(pxSocket)
	{
		#if defined ASURA_ASSERT
			const NegotiateError eNatNegError =
		#endif
			NNBeginNegotiationWithSocket(pxSocket->GetSOCKET(), iCookie, ASURA_GAMESPY_ANTS_NATNEG_SERVER, NatNegProgress_ServerCallback, NatNegComplete_ServerCallback, NULL);

		Asura_Assert(ne_noerror == eNatNegError, "natneg error");
	}
	else
	{
		Asura_Assert(false, "Can't start natneg without a socket");
	}
}
#endif

void Asura_NetTransport_System::CloseConnection( u_int uMachine, float fTime )
{
	Asura_Assert(uMachine < s_uMaxNumberOfMachines, "invalid machine index");
	if (uMachine >= s_uMaxNumberOfMachines)
	{
		return;
	}

	Asura_Network_MachineInfo* pxInfo = &s_pxMachineInfo[uMachine];
	Asura_Assert(pxInfo->GetConnectionHandle(), "no connection");

	if (!pxInfo->GetConnectionHandle())
	{
		return;
	}

	pxInfo->GetConnectionHandle()->SetClosed( fTime );
}

// Client code ////////////////////////////////////////////////////////////////////////////////////

void Asura_NetTransport_System::StartClient(Asura_NetAddress xAddress)
{
	Asura_Assert(s_pxSocket, "no socket");
	if (!s_pxSocket) return;

	//Asura_Assert(xAddress, "no server to connect to!");
	//if (uIP)
	{
#ifdef ASURA_SUPPORT_NOVAWORLD
		// init the baffle protocol
		NovaworldAPI_BaffleProtocol::InitialiseClient( s_pxSocket->GetSOCKET() );
#endif	

#if defined(ASURA_SUPPORT_GAMESPY)
		if(s_bUseNatNeg)
		{
			// SNH - 14/10/2004 10:42:17 - connection is now made in Update, to allow NatNeg to finish first.
			s_eClientConnectionState = CCS_WaitingForNatNeg;
			s_xAddressToConnectTo = xAddress;

			#if defined ASURA_ASSERT
				const NegotiateError eNatNegError =
			#endif
				NNBeginNegotiationWithSocket(s_pxSocket->GetSOCKET(), s_iNatNegCookie, ASURA_GAMESPY_ANTS_NATNEG_CLIENT, NatNegProgress_ClientCallback, NatNegComplete_ClientCallback, NULL);

			Asura_Assert(ne_noerror == eNatNegError, "natneg error");
		}
		else
#endif
		{
			//ConnectToHost(uIP, s_usServerPort);
			s_xAddressToConnectTo = xAddress;
			s_eClientConnectionState = CCS_ReadyToConnect;// CCS_WaitingForConnection;
		}
	}
}


void Asura_NetTransport_System::StopClient( void )
{
//	if (s_pxMachineInfo->IsValid())
//	{
//		delete s_pxMachineInfo->GetConnectionHandle();
//	}
}


//void Asura_NetTransport_System::ConnectToSession( u_int uSessionIndex )
//{
//	Asura_Assert(uSessionIndex < s_uNumberOfActiveSessions, "invalid session");
//	Asura_Network_SessionInfo* pxSessionInfo = s_ppxSessionsStorage[uSessionIndex];
//	u_int uIP;
//	u_short usPort;
//	pxSessionInfo->GetAddress(&uIP, &usPort);
//	ConnectToHost(uIP, usPort);
//
//	// set our copy of the session data to be this session's
//	SetSessionData(reinterpret_cast<const Asura_Network_SessionData*>(pxSessionInfo->GetSessionData()), pxSessionInfo->GetSessionDataSize());
//
//	// can get rid of the session data now.
//	for (u_int u = 0; u < s_uNumberOfActiveSessions; ++u)
//	{
//		delete s_ppxSessionsStorage[u];
//	}
//	s_uNumberOfActiveSessions = 0;
//	delete[] s_ppxSessionsStorage;
//	s_ppxSessionsStorage = NULL;
//}

void Asura_NetTransport_System::StartSessionManager( bool bUseCustomSocket /* = false */ )
{
	if(!bUseCustomSocket) // custom sockets must sort out broadcast modes themselves
	{
		Asura_Assert(s_pxSocket, "no socket");
		if (!s_pxSocket) return;

		// Enable broadcasts on the socket, for LAN server finding
		s_pxSocket->SetBroadcastEnabled(true);
	}

	// init client stuff
	s_uNumberOfActiveSessions = 0;
	s_ppxSessionsStorage = new Asura_Network_SessionInfo*[uASURA_NETWORK_MAX_NUMBER_OF_SESSIONS];
	
	// RJF - 31 January 2005 - TODO - why does this call the platform function directly? It skips my lovely log message... :(
	//								  check to see if a platform requires it, but if not, recommend changing to a regular ANTS::SendEnumHostsBroadcast() call
	Platform_SendEnumHostsBroadcast();
}


void Asura_NetTransport_System::StopSessionManager( void )
{
	for (u_int u = 0; u < s_uNumberOfActiveSessions; ++u)
	{
		delete s_ppxSessionsStorage[u];
	}
	s_uNumberOfActiveSessions = 0;
	delete[] s_ppxSessionsStorage;
	s_ppxSessionsStorage = NULL;
}

void Asura_NetTransport_System::SendMessageToServer( const Asura_Message* pxMessage )
{
	Asura_Assert( s_pxMachineInfo, "Asura_NetTransport_System::SendMessageToServer, Asura Network Transport System not initialised" );
	if( ( s_pxMachineInfo ) && (s_pxMachineInfo->IsValid() ) )
	{
		SendAsuraMessage(s_pxMachineInfo->GetConnectionHandle(), pxMessage);
	}
}


u_int Asura_NetTransport_System::GetLatency( void )
{
	if (s_pxMachineInfo &&
		s_pxMachineInfo->IsValid())
	{
		return s_pxMachineInfo->GetPing();
	}
	return 999;
}

float Asura_NetTransport_System::GetPacketLossPercent( void )
{
	if (s_pxMachineInfo &&
		s_pxMachineInfo->IsValid())
	{
		return s_pxMachineInfo->GetPacketLossPercent();
	}
	return 0.0f;
}

void Asura_NetTransport_System::UpdateLANSessions( void )
{
	Asura_Assert(s_pxMachineInfo, "Asura Network Transport System not initialised");
	if(!s_pxMachineInfo) return;
	
	float fCurrentTime = Asura_Timers::GetLocalUnscaledTime();

	if ( (fCurrentTime - s_fLastSessionRequestBroadcastTime) > dbg_fEnumHostsRefreshTime)
	{
		Platform_SendEnumHostsBroadcast();
	}

	for (u_int u = 0; u < s_uNumberOfActiveSessions; u++)
	{
		Asura_Network_SessionInfo* pxSession = s_ppxSessionsStorage[u];
		Asura_Assert(pxSession, "no session data");

		float fTimeSinceLastUpdate = fCurrentTime - pxSession->GetLastUpdateTime();

		if (fTimeSinceLastUpdate > dbg_fEnumHostsExpiryTime) 
		{
			delete pxSession;
			// swap with last, and set the loop to process this session
			s_ppxSessionsStorage[u] = s_ppxSessionsStorage[--s_uNumberOfActiveSessions];
		}
		else
		{
			Asura_Network_Session_System::SetSession(	u, 
														pxSession->GetHostAddr(),
														pxSession->GetHostPort(),
														pxSession->GetNumPlayers(),
														pxSession->GetMaxPlayers(),
														pxSession->GetSessionName(),
														pxSession->GetPing(),
														pxSession->GetServerSessionID(),
														pxSession->GetPlatformData(),
														pxSession->GetPlatformDataSize(),
														pxSession->GetSessionData(),
														pxSession->GetSessionDataSize() );
		}
	}

	// reset the remaining sessions in asura session system
	for (u_int u = s_uNumberOfActiveSessions; u < uASURA_NETWORK_MAX_NUMBER_OF_SESSIONS; ++u)
	{
		Asura_Network_Session_System::ClearSession(u);
	}

	// confirm the number of active sessions
	Asura_Network_Session_System::SetNumberOfSessions(s_uNumberOfActiveSessions);

}

// Client and server code /////////////////////////////////////////////////////////////////////////

void Asura_NetTransport_System::SetPassword( const char* szPassword )
{
	Asura_Assert(szPassword, "no password");
	if (szPassword)
	{
		s_uPasswordHash = Asura_GetHashID(szPassword);
	}
}


void Asura_NetTransport_System::GetNetworkBandwidthUsage(u_int& uSend, u_int& uReceive)
{
	uSend = s_uSendBPS;
	uReceive = s_uReceiveBPS;
}


void Asura_NetTransport_System::GetPeakNetworkBandwidthUsage(u_int& uSend, u_int& uReceive)
{
	uSend = s_uPeakSendBPS;
	uReceive = s_uPeakReceiveBPS;
}

void Asura_NetTransport_System::ResetNetworkBandwidthStats()
{
	s_uPeakSendBPS = 0;
	s_uPeakReceiveBPS = 0;
	if (s_pxMachineInfo && s_pxMachineInfo->IsValid())
	{
		s_pxMachineInfo->ResetPacketLossStats();
	}
}

void Asura_NetTransport_System::StartGame()
{
	Platform_StartGame();
}

// Server utilities ///////////////////////////////////////////////////////////////////////////////

void Asura_NetTransport_System::ProcessMachineConnection( Asura_NetConnection* pxConnection, 
														 const Asura_Network_ConnectData* pxConnectData, 
														 u_int uPing, 
														 const void* pExtraConnectData )
{
#ifdef ASURA_SUPPORT_GAMESPY
	u_int uPrevNumPlayers = Asura_Network::GetNumberOfPlayers();
#endif

	// store machine info
	const u_short usNumPlayers = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(pxConnectData->m_usNBONumPlayers);
	Asura_Handle iMachineHandle = Asura_Network::AddMachineToGame(usNumPlayers);

	Asura_Assert(iMachineHandle != ASURA_INVALID_HANDLE, "Too many machines! Should have been stopped before now!");
	Asura_Assert(static_cast<u_int>(iMachineHandle) < s_uMaxNumberOfMachines, "too many machines");
			
	s_pxMachineInfo[iMachineHandle].SetConnectionHandle(pxConnection);
	s_pxMachineInfo[iMachineHandle].InitPing(uPing);
    
	// need to send client guids to these players.
	for (u_int u = 0; u < usNumPlayers; ++u)
	{
		// get client guid, this function will add the guid to the Asura_Network machine info
		Asura_Guid uClientGuid = Asura_Network::GetNewClientGuid(iMachineHandle);
		Asura_Network::AddPlayerToMachine(uClientGuid, static_cast<u_int>(iMachineHandle));

		Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_VALIDCLIENTGUID,sizeof(Asura_MsgData_ValidClientGuid));

		Asura_MsgData_ValidClientGuid* pxData = reinterpret_cast
			<Asura_MsgData_ValidClientGuid*>(pxMessage->GetData());

		pxData->Guid = uClientGuid;

		pxMessage->To = ASURA_GUID_PLAYERS_CLIENTMACHINE;
		pxMessage->From = uClientGuid;
		pxMessage->UseGuaranteedDelivery();

		Asura_Network::Send(pxMessage);

		Project_ProcessConnection( pxConnection->GetAddress(), pxConnection->GetPort(), uClientGuid, pExtraConnectData );

#ifdef ASURA_SUPPORT_LOBBY
		if (Asura_Lobby_System::IsInitialised())
		{
 			Asura_Lobby_GameConnectData xLobbyData = pxConnectData->m_xNBOLobbyData;
 			xLobbyData.ChangeByteOrder();

			Asura_Lobby_System::SetPlayerGuid( xLobbyData, uClientGuid );
		}
#endif
	}

#ifdef ASURA_SUPPORT_GAMESPY
	// if going from empty to non-empty, or we have just filled up the server, tell GameSpy immediately.
	// (Empty servers are not listed in Quick Game, so it's an important transition.)
	if ( (uPrevNumPlayers == 0) || (usNumPlayers + uPrevNumPlayers == Asura_Network::GetMaxNumberOfPlayers()) )
	{
		Asura_GameSpy_QR2::GameModeChanged();
	}
#endif // #ifdef ASURA_SUPPORT_GAMESPY

//	// send the session data
//	Asura_Message* pxMsg = Asura_Network::CreateMessage(ASURA_MSG_NETWORKSESSIONDATA, s_uSessionDataSize, 0, s_pucSessionData);
//	pxMsg->To = ASURA_GUID_CLIENT;
//	pxMsg->UseGuaranteedDelivery();
//	SendAsuraMessage(pxConnection, pxMsg);
//	Asura_Network::DestroyMessage(pxMsg);
}

// Server callbacks ///////////////////////////////////////////////////////////////////////////////

void Asura_NetTransport_System::Server_ClientDisconnected(Asura_NetConnection* pxConnection)
{
	Asura_Network_MachineInfo* pxMachineInfo = GetMachineInfo(pxConnection);

	if(!pxMachineInfo) return;

	// use pointer arithmetic to find the machine index.
	int iMachineIndex = (pxMachineInfo - s_pxMachineInfo);

	// do project disconnection handling. This is used in NovaWorld for player metric information.
	for (u_int uPlayer = 0; uPlayer < Asura_Network::GetNumberOfPlayers(); ++uPlayer)
	{
		Asura_Guid uGuid = Asura_Network::GetPlayerGuid(uPlayer);
		if (Asura_Network::GetMachineForPlayer(uGuid, false) == iMachineIndex)
		{
			Project_ProcessDisconnection(uGuid);
		}
	}

	// calling this function to remove the machine will cause the players also
	// to be removed from the system, sending out disconnected messages etc.
	Asura_Network::RemoveMachineFromGame(iMachineIndex);
		
	pxMachineInfo->ClearConnection();
}

// NATNEG callbacks
#if defined(ASURA_SUPPORT_GAMESPY)
void Asura_NetTransport_System::NatNegProgress_ServerCallback(NegotiateState eState, void *pUserdata)
{
#if _DEBUG
	const char* szState = "unknown";
	switch(eState)
	{
	case ns_initsent:
		szState = "ns_initsent";
		break;
	case ns_initack:
		szState = "ns_initack";
		break;
	case ns_connectping:
		szState = "ns_connectping";
		break;
	case ns_finished:
		szState = "ns_finished";
		break;
	case ns_canceled:
		break;
	}
	Asura_OutputToDebugger(("NatNegProgress_ServerCallback: state = %s.\n", szState));
#endif
}

void Asura_NetTransport_System::NatNegComplete_ServerCallback(NegotiateResult eResult, SOCKET xGameSocket, struct sockaddr_in* pxRemoteAddr, void *pUserdata)
{
#if _DEBUG
	const char* szResult = "unknown";
	switch(eResult)
	{
	case nr_success:
		szResult = "nr_success";
		break;
	case nr_deadbeatpartner:
		szResult = "nr_deadbeatpartner";
		break;
	case nr_inittimeout:
		szResult = "nr_inittimeout";
		break;
	case nr_unknownerror:
		szResult = "nr_unknownerror";
		break;
	}
	Asura_OutputToDebugger(("NatNegComplete_ServerCallback for %d:%d: result = %s.\n", 
		pxRemoteAddr ? pxRemoteAddr->sin_addr.s_addr : 0, 
		pxRemoteAddr ? ntohs(pxRemoteAddr->sin_port) : 0, 
		szResult));
	//Asura_Logfile(("sblog.txt", "NatNegComplete_ServerCallback: result = %s.\n", szResult));
#endif
}

#endif

void Asura_NetTransport_System::ProcessEnumHostsRequest( Asura_NetAddress xAddress, u_short usPort, const char* pcData, u_int uDataLength )
{
	Platform_ProcessEnumHostsRequest(xAddress, usPort, pcData, uDataLength);
}

// Client functions ///////////////////////////////////////////////////////////////////////////////

void Asura_NetTransport_System::ProcessEnumHostsResponse( Asura_NetAddress xAddress, u_short usPort, const char* pcData, int iDataSize )
{
	Asura_OutputToDebugger(("Received an enum hosts response from %s with %i bytes of data.\n", GetStringFromAddress(xAddress, usPort), iDataSize));
	Platform_ProcessEnumHostsResponse(xAddress, usPort, pcData, iDataSize);
}

void Asura_NetTransport_System::SendEnumHostsBroadcast( void )
{
	Asura_OutputToDebugger(("Sending an enum hosts broadcast.\n"));
	Platform_SendEnumHostsBroadcast();
}

#if _DEBUG
void Asura_NetTransport_System::BroadcastTestNetMsg(const void* pData, u_int uDataSize)
{
	if (!s_pxSocket)
	{
		Asura_OutputToDebugger(("can't send a test net msg broadcast; no socket.\n"));
		return;
	}
	Asura_OutputToDebugger(("Sending a test net msg broadcast.\n"));

	const u_int uSizeToSend = uDataSize + sizeof(Asura_NTS_PacketHeader);
	char* pcDataToSend = new char[uSizeToSend];
	Asura_NTS_PacketHeader* pxHeader = reinterpret_cast<Asura_NTS_PacketHeader*>(pcDataToSend);
	pxHeader->m_usNBOSequenceNumber = 0; // no need to endian swap 0
	// abuse checksum for application guid test.
	pxHeader->m_usNBOCheckSum = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(static_cast<u_short>((Project_GetApplicationGuid()) & 0xffff));
	pxHeader->m_usNBOMsgID = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(static_cast<u_short>(ASURA_NETTRANSPORT_MSG_TESTNETMSG));
	pxHeader->m_usNBOGameVer= ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(Project_GetVersionID());

	// copy the data over.
	Asura_Mem::Copy(pxHeader + 1, pData, uDataSize);

#ifdef ASURA_NETADDRESS_BROADCAST
	Asura_NetAddress xBroadcast(ASURA_NETADDRESS_BROADCAST);
#else
	Asura_NetAddress xBroadcast(0xffffffff);
#endif

	// send to 2 most common ports
	s_pxSocket->Send( xBroadcast, ASURA_NETSOCKET_DEFAULTPORT, pcDataToSend, uSizeToSend );
	s_pxSocket->Send( xBroadcast, ASURA_NETSOCKET_FIRST_DYNAMIC_PORT, pcDataToSend, uSizeToSend );

	delete[] pcDataToSend;
}

void Asura_NetTransport_System::ProcessTestNetMsg(Asura_NetAddress xAddress, u_short usPort, const char* pcData, u_int uDataLength)
{
	Asura_OutputToDebugger(("Received a test net msg broadcast from %s.\n", GetStringFromAddress(xAddress, usPort)));
	const Asura_NTS_PacketHeader* pxHeader = reinterpret_cast
		<const Asura_NTS_PacketHeader*>(pcData);

	if (pxHeader->m_usNBOSequenceNumber != 0)
	{
		Asura_OutputToDebugger(("ignoring test net msg broadcast - bad sequence number.\n"));
		return;
	}

	// abuse checksum for application guid test.
	const u_short usChecksum = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(pxHeader->m_usNBOCheckSum);
	if (usChecksum != (static_cast<u_short>(Project_GetApplicationGuid()) & 0xffff))
	{
		Asura_OutputToDebugger(("ignoring test net msg broadcast - wrong game.\n"));
		return;
	}

	const u_short usMessageID = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(pxHeader->m_usNBOMsgID);
	if (usMessageID != ASURA_NETTRANSPORT_MSG_TESTNETMSG)
	{
		Asura_OutputToDebugger(("ignoring test net msg broadcast - wrong msg ID.\n"));
		return;
	}

	const u_short usGamever = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(pxHeader->m_usNBOGameVer);
	if (usGamever != Project_GetVersionID())
	{
		Asura_OutputToDebugger(("ignoring test net msg broadcast - wrong game version.\n"));
		return;
	}

	const void* pTestData = pxHeader + 1;

	Asura_NetMsg_System::ProcessTestData(pTestData, uDataLength - sizeof(Asura_NTS_PacketHeader));

}
#endif// _DEBUG

void Asura_NetTransport_System::ConnectToHost(Asura_NetAddress xAddress, unsigned short uPort)
{
	Asura_NetConnection *pxConnection = s_pxSocket->CreateConnection(xAddress, uPort, 0, false, true);
	Asura_Network_Result eResult = pxConnection ? pxConnection->GetResult() : ASURA_NETWORK_OUTOFMEMORY;
	Asura_Assertf(eResult == ASURA_NETWORK_SUCCESS, ("connection failed, reason = %d.\n", eResult));
	if (eResult != ASURA_NETWORK_SUCCESS)
	{
		Asura_Network::SetWaitingForConnection(false);
		Asura_Network::SetConnectionOkay(false);
	}
}


// functions to create the data required to initiate a connection. 
// called by Asura_NetConnection when sending the request message.
u_int Asura_NetTransport_System::GetSizeOfConnectData()
{
	return sizeof(Asura_Network_ConnectData) + Project_GetExtraConnectDataSize();
}

void Asura_NetTransport_System::FillOutConnectData( char* pcUserConnectData )
{
	Asura_Network_ConnectData* pxUserConnectData = reinterpret_cast<Asura_Network_ConnectData*>(pcUserConnectData);
	pxUserConnectData->m_usNBOGameVersion = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(Project_GetVersionID());
	pxUserConnectData->m_usNBONumPlayers = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(static_cast<u_short>(1));	// TODO - get number of local players from Asura_Network
	pxUserConnectData->m_uNBOPasswordHash = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(s_uPasswordHash);
	pxUserConnectData->m_uNBOServerSessionID = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(s_uServerSessionID);
	pxUserConnectData->m_uNBOChallengeResponse = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(Asura_GetHashID_Binary( &s_auServerChallenge[0], sizeof(u_int) ));
#ifdef ASURA_SUPPORT_LOBBY
 	pxUserConnectData->m_xNBOLobbyData.Fill();
 	pxUserConnectData->m_xNBOLobbyData.ChangeByteOrder();
#endif
	pxUserConnectData->m_uNBOExtraDataSize = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(Project_GetExtraConnectDataSize());

	// tack the extra data onto the end.
	Project_FillOutExtraConnectData( pxUserConnectData + 1 );
}

// Client callbacks ///////////////////////////////////////////////////////////////////////////////

void Asura_NetTransport_System::Client_ClientConnected( Asura_NetConnection* pxConnection, Asura_Network_Result eResult, const char* pucMessage, int iMessageLen )
{
	Asura_Network::SetWaitingForConnection(false);
	
	if (eResult == ASURA_NETWORK_SUCCESS)
	{
		Asura_Network::SetConnectionOkay(true);
		s_pxMachineInfo->SetConnectionHandle(pxConnection);
		s_pxMachineInfo->InitPing(0);//TODO

		s_eClientConnectionState = CCS_Connected;
	}
	else
	{
		if (iMessageLen == sizeof(Asura_Network_ConnectionRejectedData))
		{
			const Asura_Network_ConnectionRejectedData* pxResultCode = 
				reinterpret_cast<const Asura_Network_ConnectionRejectedData*>(pucMessage);
			const u_int uReason = ASURA_NTS_CHANGE_BYTE_ORDER_RETVAL(pxResultCode->m_uNBOReason);
			Asura_Network::SetReasonForDisconnection(static_cast<enum ASURA_NETWORK_DISCONNECT_REASON>(uReason));
		}
		else
		{
			Asura_Network::SetReasonForDisconnection(ASURA_NDR_VERSION);
		}

		s_eClientConnectionState = CCS_NotConnected;
	}
	Platform_Client_ClientConnected(pxConnection);
}


void Asura_NetTransport_System::Client_ClientDisconnected(Asura_NetConnection* pxConnection)
{
	Asura_Assert(!s_pxMachineInfo->IsValid() || 
		s_pxMachineInfo->GetConnectionHandle() == pxConnection, "unexpected connection ptr");

	s_pxMachineInfo->ClearConnection();

	// if we thought we were connected, set the disconnected reason
	if (Asura_Network::GetReasonForDisconnection() == ASURA_NDR_NOTDISCONNECTED)
	{
		switch (pxConnection->GetCloseReason())
		{
			case ASURA_CLOSE_ERROR_BUFFERFULL:
			case ASURA_CLOSE_ERROR_MESSAGETOOLARGE:
			case ASURA_CLOSE_ERROR_OUTOFMEMORY:
			case ASURA_CLOSE_ERROR_SOCKETERROR:
			{
				Asura_Network::SetReasonForDisconnection(ASURA_NDR_NETWORKING_ERROR);
				break;
			}
			case ASURA_CLOSE_ERROR_COMMUNICATION:
			{
				Asura_Network::SetReasonForDisconnection(ASURA_NDR_TIMEOUT);
				break;
			}
			case ASURA_CLOSE_LOCAL:
			{
				// The connection was closed with Asura_Connection::Close
				break;
			}
			case ASURA_CLOSE_ERROR_HOSTUNREACHABLE:
			case ASURA_CLOSE_REMOTE:
			{
				// The connection was closed remotely.
				Asura_Network::SetReasonForDisconnection(ASURA_NDR_SERVERQUIT);
				break;
			}
			case ASURA_CLOSE_ERROR_WRONGVERSION:
			{
				Asura_Network::SetReasonForDisconnection(ASURA_NDR_VERSION);
				break;
			}
			case ASURA_CLOSE_ERROR_CHECKSUM:
			{
				Asura_Network::SetReasonForDisconnection(ASURA_NDR_CHECKSUM);
				break;
			}
			default:
			{
				Asura_Network::SetReasonForDisconnection(ASURA_NDR_UNKNOWN);
				break;
			}
		}
	}

	s_eClientConnectionState = CCS_NotConnected;

	Asura_Network::SetConnectionOkay(false);
	Asura_Network::SetWaitingForConnection(false);

	Platform_Client_ClientDisconnected(pxConnection);
}

void Asura_NetTransport_System::SocketError( Asura_NetSocket* pxSocket )
{
	Asura_Assert(false, "fatal error on network socket");
	Asura_Assert(pxSocket == s_pxSocket, "unexpected socket ptr");
	s_pxSocket = NULL;
	Asura_Network::SetConnectionOkay(false);
	Asura_Network::SetWaitingForConnection(false);
}


// general callbacks
void Asura_NetTransport_System::PingCallback( Asura_NetConnection* pxConnection, int iLatency )
{
	Asura_Network_MachineInfo* pxMachineInfo = GetMachineInfo(pxConnection);
	pxMachineInfo->SetPing(static_cast<u_int>(iLatency));
}

bool Asura_NetTransport_System::HandleMessage(Asura_NetAddress xAddress, u_short usPort, const char* pcMessage, int iLength)
{
	// Pass it straight on to the platform handler
	// At the moment this is just used for Xbox voice, but could be expanded to any project or platform specific messages
	return Platform_HandleMessage(xAddress, usPort, pcMessage, iLength);
}


// NATNEG callbacks
#if defined(ASURA_SUPPORT_GAMESPY)
void Asura_NetTransport_System::NatNegProgress_ClientCallback(NegotiateState eState, void *pUserdata)
{
	Asura_Assert(s_eClientConnectionState == CCS_WaitingForNatNeg, "invalid state in NatNegCompleteCallback");

#if _DEBUG
	const char* szState = "unknown";
	switch (eState)
	{
	case ns_initsent:
		szState = "ns_initsent";
		break;
	case ns_initack:
		szState = "ns_initack";
		break;
	case ns_connectping:
		szState = "ns_connectping";
		break;
	case ns_finished:
		szState = "ns_finished";
		break;
	case ns_canceled:
		szState = "ns_canceled";
		break;
	}
	Asura_OutputToDebugger(("NatNegProgress_ClientCallback: state = %s.\n", szState));
#endif
}

void Asura_NetTransport_System::NatNegComplete_ClientCallback(NegotiateResult eResult, SOCKET xGameSocket, struct sockaddr_in* pxRemoteAddr, void *pUserdata)
{
	Asura_Assert(s_eClientConnectionState == CCS_WaitingForNatNeg, "invalid state in NatNegCompleteCallback");

#if _DEBUG
	const char* szResult = "unknown";
	switch(eResult) 
	{
	case nr_success:
		szResult = "nr_success";
		break;
	case nr_deadbeatpartner:
		szResult = "nr_deadbeatpartner";
		break;
	case nr_inittimeout:
		szResult = "nr_inittimeout";
		break;
	case nr_unknownerror:
		szResult = "nr_unknownerror";
		break;
	}
	Asura_OutputToDebugger(("NatNegComplete_ClientCallback: result = %s.\n", szResult));
	//Asura_Logfile(("sblog.txt", "NatNegComplete_ClientCallback: result = %s.\n", szResult));
#endif

	if (eResult == nr_success)
	{
		s_eClientConnectionState = CCS_ReadyToConnect;

		// set up the specified addresses for the connection
		s_xAddressToConnectTo = pxRemoteAddr->sin_addr.s_addr;
		SetServerPort(htons(pxRemoteAddr->sin_port));
	}
	else
	{
		s_eClientConnectionState = CCS_NotConnected;
		Asura_Network::SetReasonForDisconnection(ASURA_NDR_TIMEOUT);
		Asura_Network::SetConnectionOkay(false);
		Asura_Network::SetWaitingForConnection(false);
	}
}

#endif // ASURA_SUPPORT_GAMESPY

// General utility functions //////////////////////////////////////////////////////////////////////

u_short Asura_NetTransport_System::ComputeChecksum( const void* pData, int iLength )
{
	// paranoia - if we got hit with a large packet it could be expensive to compute the checksum.
	// Asura packets should be less than MTU (about 1300 bytes) anyway.
	if (iLength > 2048)
	{
		Asura_Assert(false, "packet too long");
		return 0;
	}

	// (MH - 18.03.2005) TODO - use a more robust checksum algorithm?
	u_int uHash = Asura_GetHashID_Binary( pData, iLength );
	u_int uChecksum = ((uHash >> 16) ^ (uHash & 0xffff)) & 0xffff;
	return static_cast<u_short>(uChecksum);
}

u_int Asura_NetTransport_System::SetTempGlobalBufferFromAsuraMessage( const Asura_Message* pxMessage )
{
	Asura_MessageHeader* pxHeader = (Asura_MessageHeader*)g_pcBufferPtr;
	
	*pxHeader = *pxMessage;

	unsigned int uHeaderSize = sizeof(Asura_MessageHeader);
	unsigned int uTotalSize = uHeaderSize + pxHeader->GetDataSize();

	Asura_Assertf(uTotalSize < ASURA_NETTRANSPORT_SYSTEM_BUFFERSIZE, 
		("Packet buffer size too small for message %s - message will not be sent",
		Asura_Network::GetMsgIDDesc(pxMessage->GetMsgID())));

	if (uTotalSize >= ASURA_NETTRANSPORT_SYSTEM_BUFFERSIZE)
	{
		return 0;
	}

	Asura_Mem::Copy(&g_pcBufferPtr[uHeaderSize], pxMessage->GetData(), pxHeader->GetDataSize());
	pxHeader->ChangeByteOrder();

	return uTotalSize;
}

void Asura_NetTransport_System::SendAsuraMessage(Asura_NetConnection* pxConnection, const Asura_Message* pxMessage )
{
	unsigned int uTotalSize = SetTempGlobalBufferFromAsuraMessage( pxMessage );
	pxConnection->Send(g_pcBufferPtr, uTotalSize, pxMessage->IsUsingGuaranteedDelivery());
	pxConnection->CheckBufferIntegrity();
}


bool Asura_NetTransport_System::ShouldAddMessageToQueue(u_int uMsgID)
{
	//if ready for messages, check with the project
	if(Asura_Network::IsReadyForMessages())
	{
		return Project_ShouldAddMessageToQueue(uMsgID);
	}

	//else, just handle a few specific messages
	switch(uMsgID)
	{
		case ASURA_MSG_VALIDCLIENTGUID :
			return true;

		default :
			break;
	}
	return false;
}

void Asura_NetTransport_System::UpdateBandwidthStats( void )
{
#ifdef ASURA_NETTRANSPORT_BANDWIDTH_LOG
	s_fTimeSinceLastBandwidthLog += Asura_Timers::GetFrameTime();
#endif
	s_fTimeSinceLastBandwidthCalculation += Asura_Timers::GetFrameTime();
	if (s_fTimeSinceLastBandwidthCalculation > dbg_fTimeBetweenBandwidthCalculations)
	{
		u_int uSentCount = 0;
		u_int uReceivedCount = 0;

		for (u_int u = 0; u < s_uMaxNumberOfMachines; ++u)
		{
			Asura_Network_MachineInfo& xMachineInfo = s_pxMachineInfo[u];
			if (xMachineInfo.IsValid())
			{
				u_int uMachineSent = xMachineInfo.GetAndClearBytesSent();
				u_int uMachineReceived = xMachineInfo.GetAndClearBytesReceived();

				uSentCount += uMachineSent;
				uReceivedCount += uMachineReceived;

#ifdef ASURA_NETTRANSPORT_BANDWIDTH_LOG
				s_puAdverageBPSSent[u] += static_cast<u_int>(Asura_Maths::FloatToInt(uMachineSent / dbg_fTimeBetweenBandwidthLogs));
				s_puAdverageBPSReceived[u] += static_cast<u_int>(Asura_Maths::FloatToInt(uMachineReceived / dbg_fTimeBetweenBandwidthLogs));
#endif
			}
		}

		float fSentThisPeriod = static_cast<float>(uSentCount);
		float fReceivedThisPeriod = static_cast<float>(uReceivedCount);
		
		fSentThisPeriod += s_uDirectSendBytes;
		s_uDirectSendBytes = 0;
		fReceivedThisPeriod += s_uDirectReceiveBytes;
		s_uDirectReceiveBytes = 0;

#ifdef ASURA_NETTRANSPORT_BANDWIDTH_LOG
		//Last entry is the server
		s_puAdverageBPSSent[s_uMaxNumberOfMachines] += static_cast<u_int>(Asura_Maths::FloatToInt(fSentThisPeriod / dbg_fTimeBetweenBandwidthLogs));
		s_puAdverageBPSReceived[s_uMaxNumberOfMachines] += static_cast<u_int>(Asura_Maths::FloatToInt(fReceivedThisPeriod / dbg_fTimeBetweenBandwidthLogs));
#endif

		s_uSendBPS = Asura_Maths::FloatToInt(fSentThisPeriod / s_fTimeSinceLastBandwidthCalculation);
		s_uReceiveBPS = Asura_Maths::FloatToInt(fReceivedThisPeriod / s_fTimeSinceLastBandwidthCalculation);

		if (s_uSendBPS > s_uPeakSendBPS)
		{
			s_uPeakSendBPS = s_uSendBPS;
		}
		if (s_uReceiveBPS > s_uPeakReceiveBPS)
		{
			s_uPeakReceiveBPS = s_uReceiveBPS;
		}
		
#ifdef ASURA_NETTRANSPORT_BANDWIDTH_LOG
		if (s_fTimeSinceLastBandwidthLog > dbg_fTimeBetweenBandwidthLogs)
		{
			const int uLogTextSize = 1024;
			char acLogText[uLogTextSize];
            u_int uCharsOutput = 0;

			uCharsOutput += Asura_PrintfToBfr( &acLogText[uCharsOutput], "%.0f,", uLogTextSize - uCharsOutput, true, Asura_Timers::GetServerAbsoluteTime() );

			for (u_int uSent = 0; uSent < 2; uSent++)
			{
				for (u_int uMachine = 0; uMachine <= s_uMaxNumberOfMachines; uMachine++)
				{
					u_int uBPS = (uSent) ? s_puAdverageBPSSent[uMachine] : s_puAdverageBPSReceived[uMachine];
					uCharsOutput += Asura_PrintfToBfr( &acLogText[uCharsOutput], " %d,", uLogTextSize - uCharsOutput, true, uBPS );
				}
			}

			Asura_SimpleLogfileOutput((ASURA_NETTRANSPORT_BANDWIDTH_LOG_FILENAME, "%s\n", acLogText))

			Asura_Mem::Set( s_puAdverageBPSSent, 0, sizeof(u_int) * (s_uMaxNumberOfMachines + 1) );
			Asura_Mem::Set( s_puAdverageBPSReceived, 0, sizeof(u_int) * (s_uMaxNumberOfMachines + 1) );
			s_fTimeSinceLastBandwidthLog = 0.0f;
		}
#endif

		s_fTimeSinceLastBandwidthCalculation = 0.0f;
	}
}

bool Asura_NetTransport_System::SendMessageToAddress(const void* pMessage, int iLength, Asura_NetAddress xAddress, bool bEncrypted)
{
	Asura_Assert(s_pxSocket && s_pxMachineInfo, "No socket or not initialised yet?");
	Asura_Assert(pMessage && iLength, "No message");
	if(!s_pxSocket)
	{
		return false;		
	}
	// Add the unreliable message u_short identifier to the message
	// RJF - 11 April 2005 - and the checksum too
	char* pcFullMessage = new char[iLength + 4];
	((u_short*)pcFullMessage)[0] = 0;
	((u_short*)pcFullMessage)[1] = ComputeChecksum(pMessage, iLength);
	Asura_Mem::Copy(pcFullMessage + 4, pMessage, iLength);
	bool bSuccess = s_pxSocket->Send(xAddress, s_usPort, pcFullMessage, iLength + 4, bEncrypted);
	// add to bytes sent (function adds extra overhead for packet header itself)
	AddDirectSendBytes( iLength + 4 );
	// Clean up
	delete [] pcFullMessage;
	return bSuccess;
}


// OR - 26/11/2004 08:55:20 - Net connection time outs
/////////////////////////////////////////////////////////////////
// Timeout code
/////////////////////////////////////////////////////////////////
u_int Asura_NetTransport_System::GetIdleTimeForMachine( u_int uMachine )
{
	Asura_Assert(uMachine < s_uMaxNumberOfMachines, "invalid machine index");
	if (s_pxMachineInfo)
	{
		const Asura_Network_MachineInfo* pxInfo = &s_pxMachineInfo[uMachine];
		return pxInfo->GetIdleTime();
	}
	return 0;
}

void  Asura_NetTransport_System::SetConnectionsTimeOutLimit(u_int uTimeInMS) 
{ 
	Asura_OutputToDebugger(("Asura_NetTransport_System : Connection Timeout limit set to %d secs.\n", uTimeInMS / 1000));
	
	s_uNetConnectionTimeOut = uTimeInMS; 
}
u_int Asura_NetTransport_System::GetConnectionsTimeOutLimit() 
{ 
	return s_uNetConnectionTimeOut; 
}
bool Asura_NetTransport_System::CanConnectionsTimeOut() 
{ 
	return (IsConnectionsTimeOutEnabled()) && (s_uNetConnectionTimeOut > 0); 
}

bool Asura_NetTransport_System::IsConnectionsTimeOutEnabled()
{
	return s_bDoConnectionTimeouts;
}
void Asura_NetTransport_System::SetConnectionsTimeOutEnabled(bool bOnOff)
{
	Asura_OutputToDebugger(("Asura_NetTransport_System : Connection Timeout enabled : (%s).\n", bOnOff? "true" : "false"));
	
	s_bDoConnectionTimeouts = bOnOff;
}
u_int Asura_NetTransport_System::GetIdleTime( void )
{
	if (s_pxMachineInfo &&
		s_pxMachineInfo->IsValid())
	{
		return s_pxMachineInfo->GetIdleTime();
	}
	return 0;
}

#if ASURA_NETWORK_LOGGING

bool Asura_NetTransport_System::GetMsgIDDesc(u_int uMsgID, const char** pszDesc)
{
    #define CASE_TO_STR( _str, _case ) \
        case (_case): (_str) = #_case; return true

    static char acBuffer[32];
    Asura_PrintfToBfr( acBuffer, "[misc (%d)]", sizeof(acBuffer), true, uMsgID );
    *pszDesc = acBuffer;

    switch( uMsgID )
    {
	    CASE_TO_STR( *pszDesc, ASURA_MSG_INITIALISE );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_ENUMHOSTS );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_HOSTREPLY );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_CONNECTION_REQUEST );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_CONNECTION_ACCEPT );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_CONNECTION_REJECT );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_CONNECTION_CLOSE );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_KEEPALIVE );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_PING );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_PONG );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_ACK );
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_CHALLENGE_REQUEST);
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_CHALLENGE_RESPONSE);
	    CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_NETSIM_SETTINGS);
		CASE_TO_STR( *pszDesc, ASURA_NETTRANSPORT_MSG_CLIENTLIST);

		default:
			return false;
    }

    #undef CASE_TO_STR
}

#endif

// Asura_Network_SessionInfo //////////////////////////////////////////////////////////////////////

Asura_Network_SessionInfo::Asura_Network_SessionInfo(Asura_NetAddress xAddress, 
													 u_short usHostPort, 
													 u_int uPing, 
													 u_int uServerSessionID, 
													 const void* pPlatformData,
													 u_int uPlatformDataSize,
													 const void* pSessionData, 
													 u_int uSessionDataSize, 
													 const Asura_UnicodeChar* pszSessionName,
													 u_char ucNumPlayers, 
													 u_char ucMaxPlayers)
:m_xHostAddr(xAddress)
,m_usHostPort(usHostPort)
,m_fLastUpdateTime( Asura_Timers::GetLocalUnscaledTime() )
,m_uPing(uPing)
,m_uServerSessionID(uServerSessionID)
,m_uPlatformDataSize(uPlatformDataSize)
,m_uSessionDataSize(uSessionDataSize)
,m_ucNumPlayers(ucNumPlayers)
,m_ucMaxPlayers(ucMaxPlayers)
{
	m_pPlatformData = Asura_Allocate(uPlatformDataSize);
	Asura_Mem::Copy(m_pPlatformData, pPlatformData, uPlatformDataSize);

	m_pSessionData = Asura_Allocate(uSessionDataSize);
	Asura_Mem::Copy(m_pSessionData, pSessionData, uSessionDataSize);

	Asura_PrintfToBfr(m_wszSessionName, L"%s", uASURA_NETWORK_MAX_SESSION_NAME + 1, true, pszSessionName);
}

Asura_Network_SessionInfo::~Asura_Network_SessionInfo()
{
	Asura_Deallocate(m_pPlatformData);
	Asura_Deallocate(m_pSessionData);
}

void Asura_Network_SessionInfo::Refresh(u_int uPing, 
										const void* pPlatformData,
										u_int uPlatformDataSize,
										const void* pSessionData, 
										u_int uSessionDataSize, 
										const Asura_UnicodeChar* pszSessionName,
										u_char ucNumPlayers,
										u_char ucMaxPlayers)
{
	Asura_Assert(uPlatformDataSize == m_uPlatformDataSize, "Platform data size changed!");
	if (uPlatformDataSize != m_uPlatformDataSize)
	{
		return;
	}

	Asura_Assert(uSessionDataSize == m_uSessionDataSize, "Session data size changed!");
	if (uSessionDataSize != m_uSessionDataSize)
	{
		return;
	}

	m_fLastUpdateTime = Asura_Timers::GetLocalUnscaledTime();
	m_uPing = uPing;

	Asura_Mem::Copy(m_pPlatformData, pPlatformData, uPlatformDataSize);
	Asura_Mem::Copy(m_pSessionData, pSessionData, uSessionDataSize);

	Asura_PrintfToBfr(m_wszSessionName, L"%s", uASURA_NETWORK_MAX_SESSION_NAME + 1, true, pszSessionName);

	m_ucNumPlayers = ucNumPlayers;
	m_ucMaxPlayers = ucMaxPlayers;
}

// Asura_Network_MachineInfo //////////////////////////////////////////////////////////////////////

Asura_Network_MachineInfo* Asura_NetTransport_System::GetMachineInfo( Asura_NetConnection* pxConnection )
{
	Asura_Network_MachineInfo* pxData = pxConnection->GetMachineInfo();
	// May not have data yet
	//Asura_Assert(pxData, "no machine data for connection");
	if(pxData)
	{
		Asura_Assert(pxData->GetConnectionHandle() == pxConnection, "machine/connection mismatch");
	}
	return pxData;
}


Asura_Network_MachineInfo* Asura_NetTransport_System::GetMachineInfoFromIndex( u_int uMachine )
{
	Asura_Assert(uMachine < s_uMaxNumberOfMachines, "invalid machine handle");
	return &s_pxMachineInfo[uMachine];
}

u_int Asura_NetTransport_System::GetMachineIndexFromInfo( const Asura_Network_MachineInfo* pxMachine )
{
	u_int uIndex = pxMachine - s_pxMachineInfo;

	if (uIndex > s_uMaxNumberOfMachines) 
	{
		Asura_Assert(false, "Invalid machine.");
		return 0xFFFFFFFF;
	}

	return uIndex;
}

void Asura_Network_MachineInfo::SetConnectionHandle( Asura_NetConnection* pxConnection )
{
	Asura_Assert(!IsValid(), "overwriting valid connection");
	m_pxConnectionHandle = pxConnection;
	m_pxConnectionHandle->SetMachineInfo(this);
}

void Asura_Network_MachineInfo::ClearConnection( void )
{
	if(m_pxConnectionHandle)
	{
		m_pxConnectionHandle->SetMachineInfo(0);
		m_pxConnectionHandle = 0;
	}
	m_uPing = 999;
	m_uSentBytes = 0;
	m_uRecvBytes = 0;
	m_uNumRetries = 0;
	m_uNumAcks = 0;
}

u_int Asura_Network_MachineInfo::GetIdleTime( void ) const 
{ 
	if ( !IsValid() )
	{
		return 0;
	}

	return m_pxConnectionHandle->GetIdleTime();
}

void Asura_Network_MachineInfo::ResetPacketLossStats()
{
	m_uNumRetries = 0;
	m_uNumAcks = 0;
}

#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
