//
// Asura_DemonWare.cpp
//
// Created by Mark Dieterman - 19/10/07
//
// Copyright (C) 2007 Rebellion (Liverpool), All Rights Reserved.
//
// Wrapper class to allow integration of DemonWare's Matchmaking+ SDK
//
// Current Demonware version 1.80
//

// Asura include
#include "Core/Asura.h"

#if defined(ASURA_SUPPORT_EXTERNAL_NETWORK) && defined (ASURA_SUPPORT_DEMONWARE)

#if defined(SN_TARGET_PS2)
// ARC - 23 March 2006 16:13:24 - I know this is supposed to be platform independent, but it's necessary for the DemonWare stuff
#pragma mathwarn off
#endif // SN_TARGET_PS2

// Turn off Asura's memory management
#include "Core/Asura_MemoryManagement_DebugOff.h"

#include "bdNet/bdNet.h"
#include "bdConnection/bdConnection.h"
#include "bdConnection/bdConnectionStore.h"
#include "bdSocket/bdSocketRouter.h"
#include "bdNet/bdDispatch\bdDispatcher.h"
#include "bdLobby/bdLobbyErrorCode.h"
#include "bdLobby/bdAuthService.h"
#include "bdLobby/bdAuthTicket.h"
#include "bdLobby/bdLobbyService.h"
#include "bdLobby/bdMatchMaking/bdMatchMaking.h"
#include "bdLobby/bdMatchMaking/bdMatchMakingInfo.h"
#include "bdLobby/bdRemoteTaskManager/bdRemoteTask.h"
#include "bdLobby/bdMatchMaking/bdSessionCreateResult.h"
#include "bdLobby/bdLobbyCommon/bdCustomQueryParams.h"
#include "bdLobby/bdLobbyCommon/bdQueryResult.h"
#include "Network/Asura_Demonware_CustomMatchMakingInfo.h"
#include "Network/Asura_NetSocket.h"
#include "Network/Asura_DemonWare_PacketInterceptor.h"
#include "Network/Asura_Demonware_ConnectionListener.h"
#include "Network/Asura_DemonWare_MatchMakingManager.h"

#if defined(SN_TARGET_PS2)
#pragma mathwarn on
#endif // SN_TARGET_PS2

// Engine includes
#include "Debugging/Asura_Assert.h"

// Turn off Asura's memory management
#include "Core/Asura_MemoryManagement_DebugOff.h"

#include "Core/Asura_Timers.h"

#include "Network/Asura_DemonWare_LogSubscriber.h"
#include "Network/Asura_DemonWare_MemoryManager.h"

// This file's header
#include "Network/Asura_DemonWare.h"
#include <bdNet/bdGetHostByName.h>

#if defined(SN_TARGET_PS2)
	// This is needed because in eenet_name.h (included by libeenet.h) we have the lovely #define sleep sceEENetSleep
	#undef sleep
	// ... and #define connect sceEENetConnect
	#undef connect
#endif // SN_TARGET_PS2

#define TITLE_ID (0x1383)
#define USERNAME "MMPExamples"
#define PASSWORD "MyPassword"
#define INITIAL_GAME_TYPE (1)
#define CHANGED_GAME_TYPE (2)
#define DNS_LOOKUP_TIMEOUT (5.0f)

Asura_Demonware* Asura_Demonware::s_pxInstance = NULL;

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::Asura_Demonware
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Constructor for Asura_Demonware, inits members and
//				  sets the memory allocation functions for the demonware
//                memory system to use Asura's, It then initializes the core demonware
//				  system telling it not to use the DW memory functions
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
Asura_Demonware::Asura_Demonware():
m_pxLogSubscriber(NULL),
m_eTaskStatus(ASURA_DW_READY),
m_pxAuthAddr(NULL),
m_pxAuthGetHostByName(NULL),
m_pxAuthService(NULL),
m_pxlsgAddr(NULL),
m_pxLsgGetHostByName(NULL),
m_pxSession(NULL),
m_pxRemoteTask(NULL),
m_bPumpLobbyConnection(false),
m_foundSessions(NULL),
m_uNumSessions(0),
m_pxPacketInterceptor(NULL),
m_pxConnectionListener(NULL)
{
	// Register our memory manager functions with DemonWare
	bdMemory::setAllocateFunc(Asura_DemonWare_Allocate);
	bdMemory::setAlignedAllocateFunc(Asura_DemonWare_AllocateAlign);
	bdMemory::setDeallocateFunc(Asura_DemonWare_Deallocate);
	bdMemory::setAlignedDeallocateFunc(Asura_DemonWare_DeallocateAlign);
	bdMemory::setReallocateFunc(Asura_DemonWare_Reallocate);
	bdMemory::setAlignedReallocateFunc(Asura_DemonWare_ReallocateAlign);

	bdCore::init(false);

}

Asura_Demonware::~Asura_Demonware()
{
	ShutDown();
}

void Asura_Demonware::Init()
{
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::ShutDown
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : deletes the instance and quits the DW core system
//                as well as our memory manager.
//                This is a final shutdown and no DW features will be available
//                beyond this call
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::ShutDown()
{
	delete s_pxInstance;
	bdCore::quit();
	Asura_DemonWare_MemoryManager::ShutDown();
	s_pxInstance = NULL;

}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::GetInstance
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : returns the singleton instance for this class or
//                creates one if it does not exist
//
// Parameters   : 
//
// Return value : a pointer to the instance
////////////////////////////////////////////////////////////////////////////////
Asura_Demonware* Asura_Demonware::GetInstance()
{
	if (s_pxInstance == NULL)
	{
		Asura_DemonWare_MemoryManager::Initialise();
		s_pxInstance = new Asura_Demonware();

		// This should always work if the memory manager initialises correctly (which it will)
		Asura_Assert(s_pxInstance != NULL, "Fatal error: unable to instantiate Asura_DemonWare class");
	}

	return s_pxInstance;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::CloseRemoteTask
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : remove the remote task from the system
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void CloseRemoteTask(bdRemoteTask **remoteTask)
{
	if(remoteTask)
	{
		bdRemoteTask *const remoteTaskPtr = *remoteTask;
		if(remoteTaskPtr)
		{
			remoteTaskPtr->close();
			*remoteTask = NULL;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::Update
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Deals with handling of events from start through to
//                completion
//                
//
// Parameters   : fTimeSlice : the time step for this frame
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::Update( float fTimeSlice )
{
	Asura_Demonware_TaskStatus eOldTaskStatus = m_eTaskStatus;
	if( m_eCurrentTask == ASURA_DW_TASK_STOPPED ) return;
	if( ASURA_DW_READY == m_eTaskStatus )
	{
		// MRD 12/10/07 - We are ready to start the current task, so set it off
		StartTask();
	}

	if( (ASURA_DW_DONE == m_eTaskStatus) || (ASURA_DW_PENDING == m_eTaskStatus) || (ASURA_DW_FAILED == m_eTaskStatus) )
	{

		// pump even if the status is DONE or FAILED to read out any results
		// and close any remote task handles
		PumpTask();
	}

	if( (ASURA_DW_DONE == m_eTaskStatus) || (ASURA_DW_SKIPPED == m_eTaskStatus) || (ASURA_DW_FAILED == m_eTaskStatus) )
	{
		if( ASURA_DW_READY == eOldTaskStatus )
		{
			eOldTaskStatus = m_eTaskStatus;
		}
		HandleTaskEnd();
	}

	if( m_bPumpLobbyConnection )
	{
		bdBool bLostConnection = true;
		bdLobbyServiceImpl* const pxLobbyService = bdLobbyService::getInstance();

		if( pxLobbyService )
		{
			const bdLobbyConnection::Status eStatus = pxLobbyService->getStatus();

			switch( eStatus )
			{
			case bdConnection::BD_NOT_CONNECTED:
			case bdConnection::BD_CONNECTING:
			case bdConnection::BD_DISCONNECTING:
			case bdConnection::BD_DISCONNECTED:
				{
					bLostConnection = true;
				}
				break;
			case bdConnection::BD_CONNECTED:
				{
					bLostConnection = false;
				}
				break;
			}
		}

		if(	bLostConnection)
		{
			// set up for a reconnect
			m_bPumpLobbyConnection = false;
			m_eCurrentTask = ASURA_DW_TASK_STOP_NETWORK;
			m_eTaskStatus = ASURA_DW_READY;
			CloseRemoteTask(&m_pxRemoteTask);
			m_lastTaskDescription = "Lost connection to lobby.";
		}
	}

	if( m_eTaskStatus == ASURA_DW_DONE )
	{
		m_eTaskStatus = ASURA_DW_READY;

	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::HandleTaskEnd
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : When a task has completed this deals with the states and 
//                progression (if needed)of events.  After running this function
//                the system will be ready to accept a new task, ready to start
//                the next task automatically, or have any errors detected 
//                ***NOTE***: This is setup to simply cycle through the sequence of
//                connection, ie init, start the network, connect to lobby,
//                login, advertise a session, find a session, log off and so on
//                for any real games it will feed back to Asura_Demonware and
//                the game code would set off each step.
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::HandleTaskEnd()
{
		Asura_Demonware_Task nextTaskIfDone = m_eCurrentTask;
		Asura_Demonware_Task nextTaskIfFailed = m_eCurrentTask;
		Asura_Demonware_Task nextTaskIfSkipped = m_eCurrentTask;

		bdBool displayLobbyError = true;

		switch(m_eCurrentTask)
		{
		case ASURA_DW_TASK_START_NETWORK:
			{
				nextTaskIfDone = ASURA_DW_TASK_CHECK_DNS;
				nextTaskIfFailed = ASURA_DW_TASK_START_NETWORK;
				nextTaskIfSkipped = ASURA_DW_TASK_START_NETWORK;
				displayLobbyError = false;
			}
			break;
		case ASURA_DW_TASK_CHECK_DNS:
			{
				nextTaskIfDone = ASURA_DW_TASK_CREATE_ACCOUNT;
				nextTaskIfFailed = ASURA_DW_TASK_CHECK_DNS;
				nextTaskIfSkipped = ASURA_DW_TASK_CREATE_ACCOUNT;
				displayLobbyError = false;
			}
			break;
		case ASURA_DW_TASK_CREATE_ACCOUNT:
			{
				CheckAuthServiceError();
				nextTaskIfDone = ASURA_DW_TASK_AUTHENTICATE_ACCOUNT;
				nextTaskIfFailed = ASURA_DW_TASK_CREATE_ACCOUNT;
				nextTaskIfSkipped = ASURA_DW_TASK_AUTHENTICATE_ACCOUNT;			
				// MRD 22/10/07: If account exists, move on to authenticate it
				if( m_eLobbyErrorCode == BD_AUTH_CREATE_USERNAME_EXISTS )
				{
					nextTaskIfFailed = ASURA_DW_TASK_AUTHENTICATE_ACCOUNT;			
				}

			}
			break;
		case ASURA_DW_TASK_AUTHENTICATE_ACCOUNT:
			{
				CheckAuthServiceError();
				nextTaskIfDone = ASURA_DW_TASK_LOBBY_CONNECT;
				nextTaskIfFailed = ASURA_DW_TASK_AUTHENTICATE_ACCOUNT;
				nextTaskIfSkipped = ASURA_DW_TASK_LOBBY_CONNECT;			
			}
			break;		
		case ASURA_DW_TASK_LOBBY_CONNECT:
			{
				nextTaskIfDone = ASURA_DW_TASK_ADVERTIZE_SESSION;
				nextTaskIfFailed = ASURA_DW_TASK_LOBBY_CONNECT;
				nextTaskIfSkipped = ASURA_DW_TASK_ADVERTIZE_SESSION;
			}
			break;
		case ASURA_DW_TASK_ADVERTIZE_SESSION:
			{
				CheckMatchMakingError();
				nextTaskIfDone = ASURA_DW_TASK_FIND_SESSION_BY_ID;
				nextTaskIfFailed = ASURA_DW_TASK_ADVERTIZE_SESSION;
				nextTaskIfSkipped = ASURA_DW_TASK_FIND_SESSION_BY_ID;
			}
			break;
		case ASURA_DW_TASK_FIND_SESSION_BY_ID:
			{
				CheckMatchMakingError();
				nextTaskIfDone = ASURA_DW_TASK_UPDATE_SESSION;
				nextTaskIfFailed = ASURA_DW_TASK_FIND_SESSION_BY_ID;
				nextTaskIfSkipped = ASURA_DW_TASK_UPDATE_SESSION;
			}
			break;
		case ASURA_DW_TASK_UPDATE_SESSION:
			{
				CheckMatchMakingError();
				nextTaskIfDone = ASURA_DW_TASK_FIND_SESSIONS_BY_GAME_TYPE;
				nextTaskIfFailed = ASURA_DW_TASK_UPDATE_SESSION;
				nextTaskIfSkipped = ASURA_DW_TASK_FIND_SESSIONS_BY_GAME_TYPE;
			}
			break;
		case ASURA_DW_TASK_FIND_SESSIONS_BY_GAME_TYPE:
			{
				CheckMatchMakingError();
				nextTaskIfDone = ASURA_DW_TASK_FIND_SESSIONS_BY_GAME_TYPE;//ASURA_DW_TASK_LOBBY_DISCONNECT;
				nextTaskIfFailed = ASURA_DW_TASK_FIND_SESSIONS_BY_GAME_TYPE;
				nextTaskIfSkipped = ASURA_DW_TASK_LOBBY_DISCONNECT;
			}
			break;
		case ASURA_DW_TASK_LOBBY_DISCONNECT:
			{
				nextTaskIfDone = ASURA_DW_TASK_STOP_NETWORK;
				nextTaskIfFailed = ASURA_DW_TASK_LOBBY_DISCONNECT;
				nextTaskIfSkipped = ASURA_DW_TASK_STOP_NETWORK;
			}
			break;
		case ASURA_DW_TASK_STOP_NETWORK:
			{
				nextTaskIfDone = ASURA_DW_TASK_START_NETWORK;
				nextTaskIfFailed = ASURA_DW_TASK_STOP_NETWORK;
				nextTaskIfSkipped = ASURA_DW_TASK_CREATE_ACCOUNT;
			}
			break;
		}

		switch(m_eTaskStatus)
		{
		case ASURA_DW_READY:
		case ASURA_DW_PENDING:
			{
				Asura_Assert(false, "Unexpected task status.");
			}
			break;
		case ASURA_DW_DONE:
			{
				m_eCurrentTask = nextTaskIfDone;
			}
			break;
		case ASURA_DW_FAILED:
			{
				m_eCurrentTask = nextTaskIfFailed;
			}
			break;
		case ASURA_DW_SKIPPED:
			{
				m_eCurrentTask = nextTaskIfSkipped;
			}
			break;
		}


		m_eTaskStatus = ASURA_DW_READY;

}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::CheckMatchMakingError
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Checks if the matchmaking system has an error, sets 
//                a member var so we can use it in any reporting
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::CheckMatchMakingError()
{
	const bdMatchMaking *const matchmaking = GetMatchMaking();	
	if(matchmaking)
	{
		m_eLobbyErrorCode = matchmaking->getErrorCode();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartTask
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Calls the relevant function to start a requested task
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StartTask()
{
	bdString szTaskDescription("Unknown Task");

	switch( m_eCurrentTask )
	{
		case ASURA_DW_TASK_START_NETWORK:
			{
				StartNetwork();
			}break;
		case ASURA_DW_TASK_CHECK_DNS:
			{
				StartCheckDNS();
			}break;
		case ASURA_DW_TASK_CREATE_ACCOUNT:
			{
				StartCreateAccount();
			}break;
		case ASURA_DW_TASK_AUTHENTICATE_ACCOUNT:
			{
				StartAuthenticateAccount();
			}break;
		case ASURA_DW_TASK_LOBBY_CONNECT:
			{
				StartLobbyServiceConnect();
			}break;
		case ASURA_DW_TASK_ADVERTIZE_SESSION:
			{
				StartAdvertizeSession();
			}break;
		case ASURA_DW_TASK_FIND_SESSION_BY_ID:
			{
				StartFindSessionByID();
			}break;
		case ASURA_DW_TASK_UPDATE_SESSION:
			{
				StartUpdateSession();
			}break;
		case ASURA_DW_TASK_FIND_SESSIONS_BY_GAME_TYPE:
			{
				StartFindSessionsByGameType();
			}break;
		case ASURA_DW_TASK_LOBBY_DISCONNECT:
			{
				StartLobbyServiceDisconnect();
			}break;
		case ASURA_DW_TASK_STOP_NETWORK:
			{
				StopNetwork();
			}break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpTask
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Calls the relevant function to update a requested task
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpTask()
{
	switch( m_eCurrentTask )
	{
	case ASURA_DW_TASK_START_NETWORK:
		{
			PumpStartNetwork();
		}break;
	case ASURA_DW_TASK_CHECK_DNS:
		{
			PumpCheckDNS();
		}break;
	case ASURA_DW_TASK_CREATE_ACCOUNT:
		{
			PumpCreateAccount();
		}break;
	case ASURA_DW_TASK_AUTHENTICATE_ACCOUNT:
		{
			PumpAuthenticateAccount();
		}break;
	case ASURA_DW_TASK_LOBBY_CONNECT:
		{
			PumpLobbyServiceConnect();
		}break;
	case ASURA_DW_TASK_ADVERTIZE_SESSION:
		{
			PumpAdvertizeSession();
		}break;
	case ASURA_DW_TASK_FIND_SESSION_BY_ID:
		{
			PumpFindSessionByID();
		}break;
	case ASURA_DW_TASK_UPDATE_SESSION:
		{
			PumpUpdateSession();
		}break;
	case ASURA_DW_TASK_FIND_SESSIONS_BY_GAME_TYPE:
		{
			PumpFindSessionsByGameType();
		}break;
	case ASURA_DW_TASK_LOBBY_DISCONNECT:
		{
			PumpLobbyServiceDisconnect();
		}break;
	case ASURA_DW_TASK_STOP_NETWORK:
		{
		}break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartNetwork
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Initialises the platforms sockets, sets the system
//                to start the Demonware network
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StartNetwork()
{
	m_eTaskStatus = ASURA_DW_FAILED;

	bdNetImpl* const pxNet = bdNet::getInstance();
	
	if( pxNet )
	{
		bdNetStartParams xParams;

		xParams.m_onlineGame = true;

		Platform_Initialise(xParams);

		if( pxNet->start( xParams ) )
		{
			m_eTaskStatus = ASURA_DW_PENDING;
			m_eCurrentTask = ASURA_DW_TASK_START_NETWORK;
			m_foundSessions = new bdMatchMakingInfo[25];
			m_uNumSessions = 0;
			m_pxlsgAddr = new bdInetAddr;
			m_pxSession = new Asura_Demonware_CustomMatchMakingInfo;
			m_pxAuthAddr = new bdInetAddr;
		}
		else
		{
			m_eTaskStatus = ASURA_DW_FAILED;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StopNetwork
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : 
//
// Parameters   : Stops the DW network system and releases any allocated 
//                member vars.  DW will still be available after this point
//                if you call bdNet::start() first
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StopNetwork()
{
	m_eTaskStatus = ASURA_DW_FAILED;

	bdNetImpl *const pxNet = bdNet::getInstance();

	if( pxNet )
	{
		pxNet->stop();

		if( pxNet->getStatus() == bdNetImpl::BD_NET_STOPPED)
		{
			Platform_ShutDown();

			delete m_pxSession;
			delete [] m_foundSessions;
			delete m_pxlsgAddr;
			delete m_pxAuthAddr;
			if( m_pxPacketInterceptor )
			{
				delete m_pxPacketInterceptor;
			}
			if( m_pxConnectionListener )
			{
				delete m_pxConnectionListener;
			}

			m_eTaskStatus = ASURA_DW_DONE;

		}
	}


}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpStartNetwork
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : checks the status of Demonware's network start
//				  sets up any vars to the relevant values to indicate
//				  if it has completed, failed or pending etc.
//                Upon success the task status will be ASURA_DW_DONE, and
//                the DW network will be up and running.
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpStartNetwork()
{
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	bdNetImpl *const pxNet = bdNet::getInstance();

	if( pxNet )
	{

		pxNet->pump();

		const bdNetImpl::bdNetStatus eNetworkStatus = pxNet->getStatus();

		switch(eNetworkStatus)
		{
		case bdNetImpl::BD_NET_BIND_FAILED:
		case bdNetImpl::BD_NET_ONLINE_FAILED:
		case bdNetImpl::BD_NET_INIT_FAILED:
		case bdNetImpl::BD_NET_STOPPED:
			{
				// error conditions
				m_eTaskStatus = ASURA_DW_FAILED;
			}
			break;
		case bdNetImpl::BD_NET_PENDING:
			{
				// wait
				m_eTaskStatus = ASURA_DW_PENDING;
			}
			break;
		case bdNetImpl::BD_NET_DONE:
			{
				// complete
				m_eTaskStatus = ASURA_DW_DONE;
				// Register the packet interceptor
				m_pxPacketInterceptor = new Asura_DemonWare_PacketInterceptor();
				Asura_Assert(m_pxPacketInterceptor != NULL, "Asura_DemonWare::State_StartingUp() : failed to create packet interceptor");
				pxNet->getSocketRouter()->registerInterceptor(m_pxPacketInterceptor);

				// Register the connection listener
				m_pxConnectionListener = new Asura_DemonWare_ConnectionListener();
				Asura_Assert(m_pxConnectionListener != NULL, "Asura_DemonWare::State_StartingUp() : failed to create connection listener");
				pxNet->getConnectionStore()->registerListener(m_pxConnectionListener);

			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartCheckDNS
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Sets the system to start a DNS check
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StartCheckDNS()
{
	
	const Asura_Demonware_TaskStatus eAuthDNSStatus = StartGetHostByName(ASURA_DEMONWARE_AUTHORISATION_SERVER, &m_pxAuthGetHostByName, *m_pxAuthAddr);
	const Asura_Demonware_TaskStatus eLsgDNSStatus = StartGetHostByName(ASURA_DEMONWARE_LOBBY_SERVER, &m_pxLsgGetHostByName, *m_pxlsgAddr);
	if((eAuthDNSStatus == ASURA_DW_FAILED) || (eLsgDNSStatus == ASURA_DW_FAILED))
	{
		m_eTaskStatus = ASURA_DW_FAILED;
	}

	if((eAuthDNSStatus == ASURA_DW_PENDING) || (eLsgDNSStatus == ASURA_DW_PENDING))
	{
		m_eTaskStatus = ASURA_DW_PENDING;
	}

	if((eAuthDNSStatus == ASURA_DW_DONE) && (eLsgDNSStatus == ASURA_DW_DONE))
	{
		m_eTaskStatus = ASURA_DW_DONE;
	}

}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpCheckDNS
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Updates the DNS check and watches for errors popping up
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpCheckDNS()
{
	
	Asura_Demonware_TaskStatus eAuthDNSStatus = ASURA_DW_DONE;
	Asura_Demonware_TaskStatus eLsgDNSStatus = ASURA_DW_DONE;

	if(m_pxAuthGetHostByName)
	{
		eAuthDNSStatus = PumpGetHostByName(&m_pxAuthGetHostByName, *m_pxAuthAddr);
	}

	if(m_pxLsgGetHostByName)
	{
		eLsgDNSStatus = PumpGetHostByName(&m_pxLsgGetHostByName, *m_pxlsgAddr);
	}

	if((eAuthDNSStatus == ASURA_DW_FAILED) || (eLsgDNSStatus == ASURA_DW_FAILED))
	{
		m_eTaskStatus = ASURA_DW_FAILED;
	}

	if((eAuthDNSStatus == ASURA_DW_PENDING) || (eLsgDNSStatus == ASURA_DW_PENDING))
	{
		m_eTaskStatus = ASURA_DW_PENDING;
	}

	if((eAuthDNSStatus == ASURA_DW_DONE) && (eLsgDNSStatus == ASURA_DW_DONE))
	{
		m_eTaskStatus = ASURA_DW_DONE;
	}
	
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::GetAuthService
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : returns a pointer to the demonware auth service,
//                or creates one if it does not exist
//
// Parameters   : 
//
// Return value : pointer to a valid bdAuthService
////////////////////////////////////////////////////////////////////////////////
bdAuthService* Asura_Demonware::GetAuthService()
{
	
	if(m_pxAuthService == BD_NULL)
	{
		const bdAddr authAddr(*m_pxAuthAddr, AUTH_PORT);
		m_pxAuthService = new bdAuthService(TITLE_ID, authAddr);
	}

	return m_pxAuthService;
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::GetAuthInfo
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : returns a pointer to the demonware auth information,
//                or returns NULL if the auth service does not exist
//
// Parameters   : 
//
// Return value : pointer to a valid bdAuthInfo
////////////////////////////////////////////////////////////////////////////////
const bdAuthInfo *Asura_Demonware::GetAuthInfo()
{
	
	if(m_pxAuthService != BD_NULL)
	{
		return m_pxAuthService->getAuthInfo();
	}

	return BD_NULL;
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::CheckAuthServiceError
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : checks the DW auth service system for an error code,
//                it then sets the lobby error code member var to the returned value
//                so we can use it later for error reporting or the like
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::CheckAuthServiceError()
{
	
	bdAuthService *const authService = GetAuthService();
	if(authService)
	{
		m_eLobbyErrorCode = authService->getErrorCode();
	}
	
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpAuthServiceTask
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Updates the auth service task at hand, checking for completion/errors
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpAuthServiceTask()
{
	
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	bdAuthService *const authService = GetAuthService();
	if(authService)
	{
		m_eTaskStatus = ASURA_DW_PENDING;
		const bdAuthService::bdStatus status = authService->getStatus();
		if(status == bdAuthService::BD_READY)
		{			
			const bdLobbyErrorCode lobbyErrorCode = authService->getErrorCode();
			if(lobbyErrorCode == BD_AUTH_NO_ERROR)
			{
				m_eTaskStatus = ASURA_DW_DONE;
			}
			else
			{
				m_eTaskStatus = ASURA_DW_FAILED;
			}
		}
	}
	
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartLobbyServiceDisconnect
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Begins the disconnection from the lobby
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StartLobbyServiceDisconnect()
{

	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	bdLobbyServiceImpl *const pxLobbyService = bdLobbyService::getInstance();

	if(pxLobbyService)
	{
		pxLobbyService->disconnect();
		m_eTaskStatus = ASURA_DW_PENDING;
	}

}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpLobbyServiceDisconnect
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Updates the disconnection from the lobby,
//                checking progress etc
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpLobbyServiceDisconnect()
{
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	bdLobbyServiceImpl *const pxLobbyService = bdLobbyService::getInstance();
	if(pxLobbyService)
	{
		const bdLobbyConnection::Status eStatus = pxLobbyService->getStatus();
		switch(eStatus)
		{
		case bdConnection::BD_DISCONNECTED:
		case bdConnection::BD_NOT_CONNECTED:
			{
				m_eTaskStatus = ASURA_DW_DONE;
			}
			break;
		case bdConnection::BD_CONNECTING:
		case bdConnection::BD_CONNECTED:
			{
				m_eTaskStatus = ASURA_DW_FAILED;
			}
			break;
		case bdConnection::BD_DISCONNECTING:
			{
				m_eTaskStatus = ASURA_DW_PENDING;
			}
			break;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::GetMatchMaking
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : retrieves the matchmaking service from 
//                the demonware instance
//
// Parameters   :
// Return value : pointer to a valid bdMatchMaking class
//                NULL if none exists
////////////////////////////////////////////////////////////////////////////////
bdMatchMaking* Asura_Demonware::GetMatchMaking()
{
	bdMatchMaking *matchmaking = BD_NULL;
	bdLobbyServiceImpl *const lobbyService = bdLobbyService::getInstance();
	if(lobbyService)
	{
		matchmaking = lobbyService->getMatchMaking();
	}
	return matchmaking;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartFindSessionsByGameType
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : sets the system to find a game advertised on the
//                games demonware server.  Uses GameType as a condition
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StartFindSessionsByGameType()
{
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	bdMatchMaking *const matchMaking = GetMatchMaking();
	if(matchMaking)
	{
		FindSessionsByGameType query(MAX_NUM_SESSIONS, CHANGED_GAME_TYPE);
		m_pxRemoteTask = matchMaking->findSessions(&query);
		m_eTaskStatus = CheckRemoteTaskStatus(m_pxRemoteTask);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::CheckRemoteTaskStatus
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Used to check how the remote task previously initiated 
//                is doing.  
//
// Parameters   :
// Return value : The current status of the remote task
////////////////////////////////////////////////////////////////////////////////
Asura_Demonware_TaskStatus Asura_Demonware::CheckRemoteTaskStatus(bdRemoteTask *const remoteTask)
{
	// fail safe
	Asura_Demonware_TaskStatus status = ASURA_DW_FAILED;

	if(remoteTask)
	{
		const bdRemoteTask::bdStatus remoteTaskStatus = remoteTask->getStatus();
		switch(remoteTaskStatus)
		{
		case bdRemoteTask::BD_EMPTY:
		case bdRemoteTask::BD_TIMED_OUT:
			{
				status = ASURA_DW_FAILED;
			}
			break;
		case bdRemoteTask::BD_PENDING:
			{
				status = ASURA_DW_PENDING;
			}
			break;
		case bdRemoteTask::BD_DONE:
			{
				status = ASURA_DW_DONE;
			}
			break;
		default:
			break;
		}
	}
	return status;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpFindSessionsByGameType
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : updates the find sessions remote task, setting
//                any vars it needs to so we can pick up when it
//                completes, fails etc
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpFindSessionsByGameType()
{	
	m_eTaskStatus = CheckRemoteTaskStatus(m_pxRemoteTask);

	if(m_eTaskStatus == ASURA_DW_DONE)
	{
		// fail safe
		m_eTaskStatus = ASURA_DW_FAILED;

		bdMatchMaking *const matchMaking = GetMatchMaking();
		if(matchMaking)
		{
			FindSessionsByGameTypeResult results;
			const bdBool taskHandled = matchMaking->handleTask(m_pxRemoteTask, &results);
			if(taskHandled)
			{
				m_eTaskStatus = ASURA_DW_DONE;
				m_uNumSessions = 0;
				bdBool ok = true;
				for(bdUInt i = 0; ok && (i < results.getNumResults()); i++)
				{
					m_foundSessions[m_uNumSessions++] = results.m_customMMInfo[i];
					ok = (MAX_FIND_SESSIONS_BY_GAME_TYPE_RESULTS < m_uNumSessions);
				}
			}
		}
	}

	if((m_eTaskStatus == ASURA_DW_DONE) || (m_eTaskStatus == ASURA_DW_FAILED))
	{
		CloseRemoteTask(&m_pxRemoteTask);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartUpdateSession
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : regularly called to update a session on the
//                servers
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StartUpdateSession()
{
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	bdMatchMaking *const matchMaking = GetMatchMaking();
	if(matchMaking)
	{
		// change the game type
		m_pxSession->SetGameType(CHANGED_GAME_TYPE);

		m_pxRemoteTask = matchMaking->updateSession(0, m_pxSession);
		m_eTaskStatus = CheckRemoteTaskStatus(m_pxRemoteTask);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpUpdateSession
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : called to update a session on the
//                servers, checks progress and feeds
//                it back into our Asura_Demonware system
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpUpdateSession()
{
	m_eTaskStatus = CheckRemoteTaskStatus(m_pxRemoteTask);

	if(m_eTaskStatus == ASURA_DW_DONE)
	{
		// fail safe
		m_eTaskStatus = ASURA_DW_FAILED;

		bdMatchMaking *const matchMaking = GetMatchMaking();
		if(matchMaking)
		{
			const bdBool taskHandled = matchMaking->handleTask(m_pxRemoteTask);
			if(taskHandled)
			{
				m_eTaskStatus = ASURA_DW_DONE;
			}
		}
	}

	if((m_eTaskStatus == ASURA_DW_DONE) || (m_eTaskStatus == ASURA_DW_FAILED))
	{
		CloseRemoteTask(&m_pxRemoteTask);
	}
}

void Asura_Demonware::StartFindSessionByID()
{
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	bdMatchMaking *const matchMaking = GetMatchMaking();
	if(matchMaking)
	{
		m_pxRemoteTask = matchMaking->findSessionFromID(m_pxSession->getSecurityID());
		m_eTaskStatus = CheckRemoteTaskStatus(m_pxRemoteTask);
	}
}

void Asura_Demonware::PumpFindSessionByID()
{
	m_eTaskStatus = CheckRemoteTaskStatus(m_pxRemoteTask);

	if(m_eTaskStatus == ASURA_DW_DONE)
	{
		// fail safe
		m_eTaskStatus = ASURA_DW_FAILED;

		bdMatchMaking *const matchMaking = GetMatchMaking();
		if(matchMaking)
		{
			FindSessionByIDResult sessionInfo;	
			const bdBool taskHandled = matchMaking->handleTask(m_pxRemoteTask, &sessionInfo);
			if(taskHandled)
			{
				m_eTaskStatus = ASURA_DW_DONE;
				//m_foundSessions.clear();
				m_uNumSessions = 0;

				if(sessionInfo.getNumResults() > 0)
				{						
//					m_foundSessions.add(sessionInfo.m_customMMInfo);
					m_foundSessions[m_uNumSessions++] = sessionInfo.m_customMMInfo;
				}
			}
		}
	}

	if((m_eTaskStatus == ASURA_DW_DONE) || (m_eTaskStatus == ASURA_DW_FAILED))
	{
		CloseRemoteTask(&m_pxRemoteTask);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartAdvertizeSession
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Sends information about our current game (if we are a host)
//                and posts it on the games Demonware server so other people can
//                see it to join
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StartAdvertizeSession()
{
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	bdNetImpl *const net = bdNet::getInstance();
	bdMatchMaking *const matchMaking = GetMatchMaking();

	if(matchMaking && net)
	{
		// the session details
		const bdNChar8 *const sessionName = "CoD5 T1";
		const bdInt levelNum = 1;
		const bdInt gameType = INITIAL_GAME_TYPE;
		const bdCommonAddrRef hostAddress = net->getLocalCommonAddr();
		const bdByte8 numPublicOpen    = 16;
		const bdByte8 numPrivateOpen   = 0;
		const bdByte8 numPublicFilled  = 0;
		const bdByte8 numPrivateFilled = 1;
		Asura_Demonware_CustomMatchMakingInfo session;
		m_pxSession->setHostAddr(hostAddress);
		m_pxSession->setPublicOpen(numPublicOpen);
		m_pxSession->setPublicFilled(numPublicFilled);
		m_pxSession->setPrivateOpen(numPrivateOpen);
		m_pxSession->setPrivateFilled(numPrivateFilled);
		m_pxSession->SetSessionName(sessionName);
		m_pxSession->SetLevelNum(levelNum);
		m_pxSession->SetGameType(gameType);

		// send to the lobby service
		m_pxRemoteTask = matchMaking->createSession(0, m_pxSession);
		m_eTaskStatus = CheckRemoteTaskStatus(m_pxRemoteTask);		
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpAdvertizeSession
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Updates the advertising, checking on progress
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpAdvertizeSession()
{
	m_eTaskStatus = CheckRemoteTaskStatus(m_pxRemoteTask);

	if(m_eTaskStatus == ASURA_DW_DONE)
	{
		// fail safe
		m_eTaskStatus = ASURA_DW_FAILED;

		bdMatchMaking *const matchMaking = GetMatchMaking();
		if(matchMaking)
		{
			bdSessionCreateResult advertizeResult;		
			const bdBool taskHandled = matchMaking->handleTask(m_pxRemoteTask, &advertizeResult);
			if(taskHandled)
			{
				m_eTaskStatus = ASURA_DW_DONE;
				// store the security  id and key returned by the server
				m_pxSession->setSecurityID(advertizeResult.m_secID);
				m_pxSession->setSecurityKey(advertizeResult.m_secKey);
			}
		}
	}

	if((m_eTaskStatus == ASURA_DW_DONE) || (m_eTaskStatus == ASURA_DW_FAILED))
	{
		CloseRemoteTask(&m_pxRemoteTask);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartLobbyServiceConnect
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Initiates connection to the games online lobby
//                so the user can find games, advertise etc
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StartLobbyServiceConnect()
{
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	// Connect to the Lobby Server
	bdLobbyServiceImpl *const lobbyService = bdLobbyService::getInstance();
	if(lobbyService)
	{
		const bdAddr lsgAddr(*m_pxlsgAddr, LSG_PORT);
		const bdBool connecting = lobbyService->connect(lsgAddr, GetAuthInfo());
		if(connecting)
		{
			m_eTaskStatus = ASURA_DW_PENDING;
		}
		// no need to keep the bdAuthService instance after we are connected to the lobby
		delete m_pxAuthService;
		m_pxAuthService = BD_NULL;
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpLobbyServiceConnect
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Updates the connection status and feeds back to Asura_Demonware
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpLobbyServiceConnect()
{
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	m_bPumpLobbyConnection = false;

	bdLobbyServiceImpl *const lobbyService = bdLobbyService::getInstance();
	if(lobbyService)
	{
		const bdLobbyConnection::Status connectionStatus = lobbyService->getStatus();
		switch(connectionStatus)
		{			
		case bdConnection::BD_NOT_CONNECTED:
		case bdConnection::BD_DISCONNECTING:
		case bdConnection::BD_DISCONNECTED:
			{
				m_eTaskStatus = ASURA_DW_FAILED;
			}
			break;
		case bdConnection::BD_CONNECTING:
			{
				m_eTaskStatus = ASURA_DW_PENDING;
			}
			break;
		case bdConnection::BD_CONNECTED:
			{
				m_eTaskStatus = ASURA_DW_DONE;
				m_bPumpLobbyConnection = true;
			}
			break;			
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartAuthenticateAccount
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Used for a login, begins authentication of a users given login
//                details on the games online server
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StartAuthenticateAccount()
{
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	bdAuthService *const authService = GetAuthService();
	if(authService)
	{
		if(authService->authorizeAccount(USERNAME, PASSWORD))
		{
			m_eTaskStatus = ASURA_DW_PENDING;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpAuthenticateAccount
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : updates the login checking for progress and errors, feeding that
//                back to Asura_Demonware to handle problems or progress
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpAuthenticateAccount()
{
	PumpAuthServiceTask();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartCreateAccount
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Asks the games online service to create a user account with the
//                given details (name/password).
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::StartCreateAccount()
{
	// fail safe
	m_eTaskStatus = ASURA_DW_FAILED;

	bdAuthService *const authService = GetAuthService();
	if(authService)
	{
		if(authService->createAccount(USERNAME, PASSWORD))
		{
			m_eTaskStatus = ASURA_DW_PENDING;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpCreateAccount
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Updates the create account task looking for a return from the server
//                which will typically be a failure due to existing account, success
//                or some connection error.  Feeds this back into Asura_Demonware
//                so we can handle any problems/successes
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
void Asura_Demonware::PumpCreateAccount()
{
	PumpAuthServiceTask();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::StartGetHostByName
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : initiates a host lookup from a given name
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
Asura_Demonware_TaskStatus Asura_Demonware::StartGetHostByName(const bdNChar8 *const domainNameStr,
									  bdGetHostByName **getHostByName,
									  const bdInetAddr resolvedAddr)
{
	// fail safe
	Asura_Demonware_TaskStatus eTaskStatus = ASURA_DW_FAILED;

	// is the address already resolved
	if(!resolvedAddr.isValid())
	{
		// DNS lookup objects are transitory so we create them on the heap
		bdGetHostByName *getHostByNamePtr = BD_NULL;

		if(*getHostByName == BD_NULL)
		{
			*getHostByName = new bdGetHostByName();
			getHostByNamePtr = *getHostByName;
		}

		if(getHostByNamePtr)
		{
			if(getHostByNamePtr->start(domainNameStr, DNS_LOOKUP_TIMEOUT))
			{
				eTaskStatus = ASURA_DW_PENDING;
			}
		}
	}
	else
	{
		eTaskStatus = ASURA_DW_DONE;
	}

	return eTaskStatus;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::PumpGetHostByName
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : Updates the gethostbyname, checking the status of it.  If sucessful it
//                stores the IP and feeds it into Asura_Demonware so we can act on it
//
// Parameters   :
// Return value : 
////////////////////////////////////////////////////////////////////////////////
Asura_Demonware_TaskStatus Asura_Demonware::PumpGetHostByName(bdGetHostByName **getHostByName,
									 bdInetAddr &resolvedAddr)
{
	// fail safe
	Asura_Demonware_TaskStatus eTaskStatus = ASURA_DW_FAILED;

	if(getHostByName)
	{
		bdGetHostByName *const getHostByNamePtr = *getHostByName;
		if(getHostByNamePtr)
		{
			const bdGetHostByName::bdStatus getHostByNameStatus = getHostByNamePtr->getStatus();

			// there is no need for a switch here, but with it in the compiler will
			// warn us if any new states are added to bdGetHostByName::Status
			switch(getHostByNameStatus)
			{
			case bdGetHostByName::BD_FAILED:
				{
					// error conditions
					eTaskStatus = ASURA_DW_FAILED;
				}
				break;
			case bdGetHostByName::BD_PENDING:
				{
					// wait
					eTaskStatus = ASURA_DW_PENDING;
				}
				break;
			case bdGetHostByName::BD_READY:
				{
					// finished
					const bdBool addrResolved = getHostByNamePtr->getNumAddresses() > 0;
					if(addrResolved)
					{
						const bdInAddr inAddr = getHostByNamePtr->getAddressAt(0);
						resolvedAddr.set(inAddr);
					}

					if(addrResolved && resolvedAddr.isValid())
					{
						eTaskStatus = ASURA_DW_DONE;
					}
				}
			}

			if((eTaskStatus == ASURA_DW_FAILED) || (eTaskStatus == ASURA_DW_DONE))
			{
				delete getHostByNamePtr;
				*getHostByName = BD_NULL;
			}
		}
	}

	return eTaskStatus;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Demonware::GetSocket
// Author       : Mark Dieterman
// Date         : 20/10/2007
//
// Description  : retrieves the socket handle we are using for comms
//
// Parameters   :
// Return value : handle of the socket currently setup for use
////////////////////////////////////////////////////////////////////////////////
SOCKET Asura_Demonware::GetSocket(void)
{
	Asura_Assert(m_eCurrentTask > ASURA_DW_TASK_START_NETWORK, "Asura_DemonWare::GetSocket() called when m_eState <= ASURA_DEMONWARE_STATE_STARTINGUP");
	// Return the DemonWare created socket handle
	return bdNet::getInstance()->getParams().m_socket->getHandle();
}

//
//	SendPacket()
//
bool Asura_Demonware::SendPacket(const Asura_NetSocket_MessageInfo* const pxMessageInfo)
{
	Asura_Assert(m_pxPacketInterceptor != NULL, "Asura_DemonWare::SendPacket() called when m_pxPacketInterceptor is NULL");
	return m_pxPacketInterceptor->sendPacket(pxMessageInfo);
}


//
//	ArePacketsPending
//
bool Asura_Demonware::ArePacketsPending(void)
{
	Asura_Assert(m_pxPacketInterceptor != NULL, "Asura_DemonWare::ArePacketsPending() called when m_pxPacketInterceptor is NULL");
	return m_pxPacketInterceptor->ArePacketsPending();
}

//
//	HandleNextPacket
//
bool Asura_Demonware::HandleNextPacket(Asura_NetSocket* pxAsuraSocket)
{
	Asura_Assert(m_pxPacketInterceptor != NULL, "Asura_DemonWare::HandleNextPacket() called when m_pxPacketInterceptor is NULL");
	return m_pxPacketInterceptor->HandleNextPacket(pxAsuraSocket);
}


#endif // ASURA_SUPPORT_EXTERNAL_NETWORK && ASURA_SUPPORT_DEMONWARE
