//
// Asura_DemonWare_LogSubscriber.cpp
//
// Created by Andrew Catlender - 15 March 2006 14:50:47
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//
//
// Wrapper class to allow integration of DemonWare's Matchmaking+ SDK
//

// Asura include
#include "Core/Asura.h"

#if defined(ASURA_SUPPORT_EXTERNAL_NETWORK) && defined (ASURA_SUPPORT_DEMONWARE) && defined(ASURA_LOGFILES_ENABLED)

// Engine includes
#include "Core/Asura_String.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_Maths.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"

// This file's header
#include "Network/Asura_DemonWare_LogSubscriber.h"

// Static initialisation
Asura_DemonWare_LogSubscriber_Singleton* Asura_DemonWare_LogSubscriber_Singleton::s_pxInstance = NULL;

// Asura log file name
#define ASURA_DEMONWARE_LOGFILE "logfiles\\demonware.txt"
#define ASURA_DEMONWARE_LOGFILE_NEWLINE "\n"

//
//	Constructor
//
Asura_DemonWare_LogSubscriber::Asura_DemonWare_LogSubscriber(void)
{
	Asura_SimpleLogfileClear(ASURA_DEMONWARE_LOGFILE);
}


//
//	Destructor
//
Asura_DemonWare_LogSubscriber::~Asura_DemonWare_LogSubscriber(void)
{
}


//
//	publish()
//
void Asura_DemonWare_LogSubscriber::publish(const char* const szFullChannelName, const char* const szFile, const char* const szFunction, const unsigned int uiLine, const char* const szMessage)
{
#if defined(ASURA_DEMONWARE_LOG_CALLER)
	Asura_SimpleLogfileOutput((ASURA_DEMONWARE_LOGFILE, "[%s] %s %s(%i): ", szFullChannelName, GetTimeString(), szFile, uiLine));

	if (*szFunction != 0)
	{
		Asura_SimpleLogfileOutput((ASURA_DEMONWARE_LOGFILE, "in function [%s]:", szFunction));
	}

	Asura_SimpleLogfileOutput((ASURA_DEMONWARE_LOGFILE, ASURA_DEMONWARE_LOGFILE_NEWLINE "%s" ASURA_DEMONWARE_LOGFILE_NEWLINE, szMessage));
#else
	Asura_SimpleLogfileOutput((ASURA_DEMONWARE_LOGFILE, "[%s] %s %s" ASURA_DEMONWARE_LOGFILE_NEWLINE, szFullChannelName, GetTimeString(), szMessage));
#endif // ASURA_DEMONWARE_LOG_VERBOSE
}


//
//	location()
//
void Asura_DemonWare_LogSubscriber::location(const char* const szFullChannelName, const char* const szFile, const char* const szFunction, const unsigned int uiLine)
{
#if defined(ASURA_DEMONWARE_LOG_CALLER)
	Asura_SimpleLogfileOutput((ASURA_DEMONWARE_LOGFILE, "[%s] %s %s(%i): ", szFullChannelName, GetTimeString(), szFile, uiLine));

	if (*szFunction != 0)
	{
		Asura_SimpleLogfileOutput((ASURA_DEMONWARE_LOGFILE, "in function [%s]:", szFunction));
	}

	Asura_SimpleLogfileOutput((ASURA_DEMONWARE_LOGFILE, ASURA_DEMONWARE_LOGFILE_NEWLINE));
#else
	Asura_SimpleLogfileOutput((ASURA_DEMONWARE_LOGFILE, "[%s] %s ", szFullChannelName, GetTimeString()));
#endif // ASURA_DEMONWARE_LOG_VERBOSE
}

//
//	message()
//
void Asura_DemonWare_LogSubscriber::message(const char* const szFormat, ...)
{
	va_list xArguments;
	va_start(xArguments, szFormat);
	Asura_PrintfToBfr(m_szBuffer, szFormat, xArguments, sizeof(m_szBuffer), true);
	va_end(xArguments);

	Asura_SimpleLogfileOutput((ASURA_DEMONWARE_LOGFILE, "%s" ASURA_DEMONWARE_LOGFILE_NEWLINE, m_szBuffer));
}


//
//	GetTimeString()
//
char* Asura_DemonWare_LogSubscriber::GetTimeString(void)
{
	float fTime = Asura_Timers::GetLocalUnscaledTime();
	u_int uiTime = Asura_Maths::FloatToInt_Chop(fTime);

	u_int uiSeconds = Asura_Maths::FloatToInt_Chop(fTime) % 60;
	u_int uiMinutes = uiTime / 60;
	u_int uiHours = uiTime / 3600;
	u_int uiThousandthsOfASecond = Asura_Maths::FloatToInt_Chop((fTime - static_cast<float>(Asura_Maths::FloatToInt_Chop(fTime))) * 1000.0f);

	static char szBuffer[16];
	Asura_PrintfToBfr(szBuffer, "[%02i:%02i:%02i:%03i]", sizeof(szBuffer), true, uiHours, uiMinutes, uiSeconds, uiThousandthsOfASecond);
	return szBuffer;
}

//
//	Constructor
//
Asura_DemonWare_LogSubscriber_Singleton::Asura_DemonWare_LogSubscriber_Singleton(void)
{
}


//
//	Destructor
//
Asura_DemonWare_LogSubscriber_Singleton::~Asura_DemonWare_LogSubscriber_Singleton(void)
{
}


//
//	GetInstance()
//
Asura_DemonWare_LogSubscriber_Singleton* const Asura_DemonWare_LogSubscriber_Singleton::GetInstance(void)
{
	if (s_pxInstance == NULL)
	{
		s_pxInstance = new Asura_DemonWare_LogSubscriber_Singleton();

		// This will always work if the memory manager initialises correctly (which it will)
		Asura_Assert(s_pxInstance != NULL, "Fatal error: unable to instantiate Asura_DemonWare_LogSubscriber_Singleton class");
	}

	return s_pxInstance;
}


//
//	DeleteInstance()
//
void Asura_DemonWare_LogSubscriber_Singleton::DeleteInstance(void)
{
	if (s_pxInstance != NULL)
	{
		Asura_DemonWare_LogSubscriber_Singleton* pxTemporary = s_pxInstance;
		s_pxInstance = NULL;
		delete pxTemporary;
	}
}


//
//	GetLog()
//
Asura_DemonWare_LogSubscriber* Asura_DemonWare_LogSubscriber_Singleton::GetLog(void)
{
	return &(s_pxInstance->m_xLogSubscriber);
}

#endif // ASURA_SUPPORT_EXTERNAL_NETWORK && ASURA_SUPPORT_DEMONWARE && ASURA_LOGFILES_ENABLED
