//
// Asura_DemonWare_MatchMakingManager.cpp
//
// Created by Andrew Catlender - 30 May 2006 07:29:20
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//
//
// MatchMaking information manager class
//

#include "Core/Asura.h"

#if defined(ASURA_SUPPORT_EXTERNAL_NETWORK) && defined (ASURA_SUPPORT_DEMONWARE)

// Middleware includes (must be guarded by Asura_MemoryManagement_DebugOff/On)
#if defined(SN_TARGET_PS2)
// ARC - 23 March 2006 16:13:24 - I know this is supposed to be platform independent, but it's necessary for the DemonWare stuff
#pragma mathwarn off
#endif

#include "Core/Asura_MemoryManagement_DebugOff.h"
#include "DemonWare/bdLobby/bdMatchMaking/bdMatchMakingInfo.h"
#include "DemonWare/bdLobby/bdLobbyCommon/bdCustomQueryParams.h"
#include "DemonWare/bdLobby/bdLobbyCommon/bdQueryResult.h"
#include <bdLobby/bdMatchMaking/bdMatchMakingInfo.h>

#if defined(SN_TARGET_PS2)
#pragma mathwarn on
#endif // SN_TARGET_PS2

// Engine includes
#include "Debugging/Asura_Assert.h"


// This file's header
#include "Network/Asura_DemonWare_MatchMakingManager.h"

//
//	Constructor
//
Asura_DemonWare_MatchMakingManager::Asura_DemonWare_MatchMakingManager(u_int uiMaxResults)
: m_pxCurrentSession(NULL),
  m_pxQuery(NULL),
  m_pxResultHandler(NULL),
  m_uiMaxResults(uiMaxResults)
{
	Project_Initialise(uiMaxResults);
}

//
//	Destructor
//
Asura_DemonWare_MatchMakingManager::~Asura_DemonWare_MatchMakingManager()
{
	if (m_pxCurrentSession != NULL)
	{
		bdMatchMakingInfo* pxTemporary = m_pxCurrentSession;
		m_pxCurrentSession = NULL;
		delete pxTemporary;
	}

	if (m_pxQuery != NULL)
	{
		bdCustomQueryParams* pxTemporary = m_pxQuery;
		m_pxQuery = NULL;
		delete pxTemporary;
	}

	if (m_pxResultHandler != NULL)
	{
		bdQueryResult* pxTemporary = m_pxResultHandler;
		m_pxResultHandler = NULL;
		delete pxTemporary;
	}
}


//
//	CreateQuery()
//
bdCustomQueryParams* Asura_DemonWare_MatchMakingManager::CreateQuery(u_int uiQueryID)
{
	m_pxQuery->m_queryID = uiQueryID;
	m_pxQuery->m_maxNumResults = m_uiMaxResults;

	return m_pxQuery;
}
//#include "MatchMakingQueries.h"

#define BD_LOG_LEVEL "MM Queries"

FindSessionByIDResult::FindSessionByIDResult()
{
}

FindSessionByIDResult::~FindSessionByIDResult()
{
}

bdBool FindSessionByIDResult::handleResult(const bdUInt numResults, 
										   bdBitBufferRef buffer)
{
	reset();

	bdBool ok = true;	
	BD_ASSERT(numResults <= 1, "Only expecting 1 or 0 results.");

	if(numResults > 0)
	{
		ok = m_customMMInfo.deserialize(buffer);
		if(ok)
		{		
			m_numResults = 1;
		}
	}

	return ok;
}

FindSessionsByGameType::FindSessionsByGameType(const bdUInt maxNumResults,
											   const bdInt gameType)
											   : bdCustomQueryParams(FIND_SESSIONS_BY_GAME_TYPE, maxNumResults),
											   m_gameType(gameType)
{
	if(m_maxNumResults > MAX_FIND_SESSIONS_BY_GAME_TYPE_RESULTS)
	{
		BD_WARN(BD_LOG_LEVEL, "Results class can only store %u results, %u requested.", 
			MAX_FIND_SESSIONS_BY_GAME_TYPE_RESULTS, m_maxNumResults);
	}
}

FindSessionsByGameType::~FindSessionsByGameType()
{
}

void FindSessionsByGameType::serialize(bdBitBuffer& buffer)
{
	bdCustomQueryParams::serialize(buffer);

	buffer.writeInt32(m_gameType);
}

FindSessionsByGameTypeResult::FindSessionsByGameTypeResult()
{
}

FindSessionsByGameTypeResult::~FindSessionsByGameTypeResult()
{
}

bdBool FindSessionsByGameTypeResult::handleResult(const bdUInt numResults, 
												  bdBitBufferRef buffer)
{
	reset();

	bdBool ok = true;

	const bdUInt resultsToRead = BD_MIN(numResults, MAX_FIND_SESSIONS_BY_GAME_TYPE_RESULTS);
	if(resultsToRead != numResults)
	{
		BD_WARN(BD_LOG_LEVEL, "Received %u results but can only store %u.", 
			numResults, MAX_FIND_SESSIONS_BY_GAME_TYPE_RESULTS);
	}

	for(bdUInt i=0; ok && (i < resultsToRead); i++)
	{
		ok = m_customMMInfo[i].deserialize(buffer);
		if(ok)
		{
			m_numResults++;
		}
	}

	return ok;
}

#endif // ASURA_SUPPORT_EXTERNAL_NETWORK && ASURA_SUPPORT_DEMONWARE
