//////////////////////////////////////////////////////////////////////////////
//
// Asura_GameSpy.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     common gamespy functionality e.g. GameName, VersionID etc.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_GAMESPY_H___
#define ___ASURA_GAMESPY_H___ 1

#ifdef ASURA_SUPPORT_GAMESPY

enum ASURA_GAMESPY_STATE
{
	ASURA_GAMESPY_STATE_INVALID,
	ASURA_GAMESPY_STATE_AVAILABLE_CHECK,
	ASURA_GAMESPY_STATE_INITIALISED,
	ASURA_GAMESPY_STATE_UNAVAILABLE,
	ASURA_GAMESPY_STATE_TEMPORARILY_UNAVAILABLE,
	ASURA_GAMESPY_STATE_DNS_FAILED,
};

class Asura_GameSpy
{
	
	public:

		static void Initialise( void );
		static void Shutdown( void );
		static void Update( void );
		static void InitThreadFunc( const void* );
		static bool IsInitialised( void ) { return s_eState != ASURA_GAMESPY_STATE_INVALID; }
		static ASURA_GAMESPY_STATE GetState( void ) { return s_eState; }
		
		static const char* GetGameName( void )
		{
			return Project_GetGameName();
		}

		static const char* GetGameKey( void )
		{
			return Project_GetGameKey();
		}

		static int GetGameVersion( void );

		static int GetGameID( void )
		{
			return Project_GetGameID();
		}

		static int GetProductID( void )
		{
			return Project_GetProductID();
		}

		// unlocalised game title e.g. "Rogue Trooper (PS2)"
		static const char* GetGameTitle( void )
		{
			return Project_GetGameTitle();
		}

		static bool ShouldUseDevelopmentBackend( void )
		{
			return Project_ShouldUseDevelopmentBackend();
		}

		static bool GetGameSpySocketErrorFlag();
		static void ResetGameSpySocketErrorFlag();

		static char GetConnectionProgress()
		{
			return s_uConnectionProgress;
		}

	private:

		// This should be set to the result of calling 
		// StringHash( Project_GetGameName(), NUM_MASTER_SERVERS )
		// StringHash is defined in sb_serverlist.c and sb_server.c
		static u_int Project_GetMasterServerNumber( void );

		static const char* Project_GetGameName( void );
		static const char* Project_GetGameKey( void );
		static int Project_GetGameVersion( void );
		static int Project_GetGameID( void );
		static int Project_GetProductID( void );
		static const char* Project_GetGameTitle( void );	// e.g. "Rogue Trooper (PS2)"
		static bool Project_ShouldUseDevelopmentBackend( void );

		static ASURA_GAMESPY_STATE s_eState;
		static char s_uConnectionProgress;
};

#endif //#ifdef ASURA_SUPPORT_GAMESPY
// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_GAMESPY_H___
