//////////////////////////////////////////////////////////////////////////////
//
// Asura_Leaderboards.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     game- and platform- independent interface for leaderboards
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_LEADERBOARDS

// This file's header.
#include "Network/Online/Asura_Leaderboards.h"

// Asura independent
#include "Core/Asura_SortAndSearch.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Core/Memory/Asura_Mem.h"
#include "Interface/Asura_CommandConsole.h"
#include "Network/Online/Asura_Lobby_Types.h"	// for player name buffer size

//for wcslen
#include <wchar.h>

// I N T E R N A L   C L A S S E S ///////////////////////////////////////////
class Asura_Leaderboard_Row
{
public:
	u_int m_uRank;
	Asura_UnicodeChar m_awcName[ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE];
	float m_afData[uASURA_LEADERBOARD_MAX_COLUMNS];
};

// helper classes to generate template code for each columns access function
class Asura_Leaderboard_Col0 { public: static const u_int uCOL_IDX = 0; };
class Asura_Leaderboard_Col1 { public: static const u_int uCOL_IDX = 1; };
class Asura_Leaderboard_Col2 { public: static const u_int uCOL_IDX = 2; };
class Asura_Leaderboard_Col3 { public: static const u_int uCOL_IDX = 3; };
class Asura_Leaderboard_Col4 { public: static const u_int uCOL_IDX = 4; };
class Asura_Leaderboard_Col5 { public: static const u_int uCOL_IDX = 5; };
class Asura_Leaderboard_Col6 { public: static const u_int uCOL_IDX = 6; };

// S T A T I C   C L A S S   D A T A /////////////////////////////////////////
bool Asura_Leaderboard_System::s_bError;
bool Asura_Leaderboard_System::s_bFriendsOnly;
u_int Asura_Leaderboard_System::s_uCurrentLeaderboard;
Asura_Leaderboard** Asura_Leaderboard_System::s_ppxLeaderboards;
Asura_Leaderboard** Asura_Leaderboard_System::s_ppxFriendsLeaderboards;
Asura_Leaderboard* Asura_Leaderboard_System::s_pxLeaderboard;

// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////

//////////////////////////////////////////////////////////////////////////
// Asura_Leaderboard_System
//////////////////////////////////////////////////////////////////////////
void Asura_Leaderboard_System::AddConsoleCmdsAndVars()
{
	if(!Asura_CommandConsole::CmdExists(L"Asura.LB.Init"))
	{
		// init/update/shutdown commands can be called on entering/updating/exiting 
		// the leaderboards area of the menus.
		Asura_CommandConsole::AddCmd(L"Asura.LB.Init", Initialise, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Update", Update, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Shutdown", Shutdown, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);

		Asura_CommandConsole::AddVar(L"Asura.LB.Current",		&s_uCurrentLeaderboard, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddVar(L"Asura.LB.Error",			&s_bError, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddVar(L"Asura.LB.FriendsOnly",	&s_bFriendsOnly, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::MakeVarReadOnly( L"Asura.LB.Current" );
		Asura_CommandConsole::MakeVarReadOnly( L"Asura.LB.Error" );
		Asura_CommandConsole::MakeVarReadOnly( L"Asura.LB.FriendsOnly" );
		Asura_CommandConsole::AddCmd(L"Asura.LB.ResetErrorFlag",	ConsoleCmd_ResetErrorFlag, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.NumEntries",		ConsoleCmd_GetNumEntries, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.IsDataPending",		ConsoleCmd_IsDataPending, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.SetCurrent",		SetCurrentLeaderboard, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.ToggleFriendsOnly",	ConsoleCmd_ToggleFriendsOnly, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.PageUp",			ConsoleCmd_PageUp, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.PageDown",			ConsoleCmd_PageDown, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Rank", 				ConsoleCmd_GetRank, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Name", 				ConsoleCmd_GetName, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Col0", 				ConsoleCmd_GetCol<Asura_Leaderboard_Col0>, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Col1", 				ConsoleCmd_GetCol<Asura_Leaderboard_Col1>, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Col2", 				ConsoleCmd_GetCol<Asura_Leaderboard_Col2>, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Col3", 				ConsoleCmd_GetCol<Asura_Leaderboard_Col3>, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Col4", 				ConsoleCmd_GetCol<Asura_Leaderboard_Col4>, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Col5", 				ConsoleCmd_GetCol<Asura_Leaderboard_Col5>, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddCmd(L"Asura.LB.Col6", 				ConsoleCmd_GetCol<Asura_Leaderboard_Col6>, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	}

#ifdef ASURA_ASSERT
	// check the project doesn't have more than 7 columns in any leaderboard
	ASURA_STATIC_ASSERT(uASURA_LEADERBOARD_MAX_COLUMNS == 7);
	for (u_int u = 0; u < Project_GetNumLeaderboards(); ++u)
	{
		Asura_Assert(Project_GetNumLeaderboardColumns(u) <= uASURA_LEADERBOARD_MAX_COLUMNS, "need to add column accessor");
	}
#endif
}

// init/update/shutdown functions that double up as console commands
bool Asura_Leaderboard_System::Initialise( bool bHelp )
{
	if (!bHelp)
	{
		// prepare platform code
		Platform_Initialise();

		// create all the project leaderboards
		Asura_Assert(!s_ppxLeaderboards, "memory leak");
		const u_int uNumLeaderboards = Project_GetNumLeaderboards();
		s_ppxLeaderboards = new Asura_Leaderboard*[uNumLeaderboards];
		s_ppxFriendsLeaderboards = new Asura_Leaderboard*[uNumLeaderboards];
		for (u_int u = 0; u < uNumLeaderboards; ++u)
		{
			s_ppxLeaderboards[u] = new Asura_Leaderboard(u, false);
			s_ppxFriendsLeaderboards[u] = new Asura_Leaderboard(u, true);
		}
		Asura_Assert(s_uCurrentLeaderboard < Project_GetNumLeaderboards(), "invalid leaderboard");
		s_pxLeaderboard = s_ppxLeaderboards[s_uCurrentLeaderboard];
	}
	return true;
}

bool Asura_Leaderboard_System::Update( bool bHelp )
{
	if (!bHelp)
	{
		// update the current leaderboard. if it's up-to-date, we could update the others.
		if (s_pxLeaderboard)
		{
			s_pxLeaderboard->Update();
			if (s_pxLeaderboard->HasErrorOccurred())
			{
				s_bError = true;
			}
		}
	}
	return true;
}

bool Asura_Leaderboard_System::Shutdown( bool bHelp )
{
	if (!bHelp)
	{
		// destroy all the leaderboards
		const u_int uNumLeaderboards = Project_GetNumLeaderboards();
		for (u_int u = 0; u < uNumLeaderboards; ++u)
		{
			delete s_ppxFriendsLeaderboards[u];
			delete s_ppxLeaderboards[u];
		}
		delete[] s_ppxFriendsLeaderboards;
		delete[] s_ppxLeaderboards;
		s_ppxFriendsLeaderboards = NULL;
		s_ppxLeaderboards = NULL;
		s_pxLeaderboard = NULL;

		// shutdown platform code
		Platform_Shutdown();
	}
	return true;
}

bool Asura_Leaderboard_System::ConsoleCmd_GetNumEntries( bool bHelp )
{
	if (!bHelp)
	{
		if (s_pxLeaderboard)
		{
			Asura_CommandConsole::Printf("%d", s_pxLeaderboard->GetTotalEntries());
			return true;
		}
	}
	return false;
}

bool Asura_Leaderboard_System::ConsoleCmd_IsDataPending( bool bHelp )
{
	if (!bHelp)
	{
		if (s_pxLeaderboard)
		{
			Asura_CommandConsole::Printf("%d", s_pxLeaderboard->IsCurrentPagePending());
			return true;
		}
	}
	return false;
}

bool Asura_Leaderboard_System::ConsoleCmd_ResetErrorFlag( bool bHelp )
{
	if (!bHelp)
	{
		s_bError = false;
		if (s_ppxLeaderboards)
		{
			const u_int uNumLeaderboards = Project_GetNumLeaderboards();
			for (u_int u = 0; u < uNumLeaderboards; ++u)
			{
				s_ppxLeaderboards[u]->ResetErrorFlag();
			}
		}
	}
	return true;
}

bool Asura_Leaderboard_System::SetCurrentLeaderboard( u_int uIndex, bool bInterrogate, bool bHelp )
{
	if (bHelp || bInterrogate)
	{
		return false;
	}
	if (uIndex == s_uCurrentLeaderboard)
	{
		// nothing to do.
		return true;
	}
	if (uIndex < Project_GetNumLeaderboards())
	{
		s_uCurrentLeaderboard = uIndex;
		s_pxLeaderboard = s_bFriendsOnly ? s_ppxFriendsLeaderboards[uIndex] : s_ppxLeaderboards[uIndex];
		return true;
	}
	return false;
}

bool Asura_Leaderboard_System::ConsoleCmd_ToggleFriendsOnly( bool bHelp )
{
	if (bHelp)
	{
		return false;
	}
	else
	{
		if (s_uCurrentLeaderboard < Project_GetNumLeaderboards())
		{
			s_bFriendsOnly = !s_bFriendsOnly;
			s_pxLeaderboard = s_bFriendsOnly ? 
				s_ppxFriendsLeaderboards[s_uCurrentLeaderboard] : 
				s_ppxLeaderboards[s_uCurrentLeaderboard];
			return true;
		}
	}
	return false;
}

bool Asura_Leaderboard_System::ConsoleCmd_PageUp( bool bHelp )
{
	if (!bHelp)
	{
		if (s_pxLeaderboard)
		{
			s_pxLeaderboard->PageUp();
			return true;
		}
	}
	return false;
}

bool Asura_Leaderboard_System::ConsoleCmd_PageDown( bool bHelp )
{
	if (!bHelp)
	{
		if (s_pxLeaderboard)
		{
			s_pxLeaderboard->PageDown();
			return true;
		}
	}
	return false;
}


// console cmds take input from 0 to (numrows-1)
bool Asura_Leaderboard_System::ConsoleCmd_GetRank( u_int uIndex, bool bInterrogate, bool bHelp )
{
	if (bHelp || bInterrogate) { return false; }
	if (s_pxLeaderboard && uIndex < s_pxLeaderboard->GetNumAvailableRowsInPage())
	{
		const u_int uRank = s_pxLeaderboard->GetRank(uIndex);
		Asura_CommandConsole::Printf( "%u", uRank );
		return true;
	}
	return false;
}

bool Asura_Leaderboard_System::ConsoleCmd_GetName( u_int uIndex, bool bInterrogate, bool bHelp )
{
	if (bHelp || bInterrogate) { return false; }
	if (s_pxLeaderboard && uIndex < s_pxLeaderboard->GetNumAvailableRowsInPage())
	{
		const Asura_UnicodeChar* wszName = s_pxLeaderboard->GetName(uIndex);
		Asura_CommandConsole::Printf( L"%s", wszName );
		return true;
	}
	return false;
}

template <typename T> 
bool Asura_Leaderboard_System::ConsoleCmd_GetCol( u_int uIndex, bool bInterrogate, bool bHelp )
{
	if (bHelp || bInterrogate) { return false; }
	if (s_pxLeaderboard && uIndex < s_pxLeaderboard->GetNumAvailableRowsInPage())
	{
		PrintColumnToConsole(uIndex, T::uCOL_IDX);
		return true;
	}
	return false;
}


void Asura_Leaderboard_System::PrintColumnToConsole( u_int uRow, u_int uColumn )
{
	if (uColumn >= Project_GetNumLeaderboardColumns(s_uCurrentLeaderboard))
	{
		// print a blank column, but it's not necessarily an error, and we want to continue 
		// with subsequent rows. 
		Asura_CommandConsole::Printf("");
		return;
	}
	const char* szFormat = Project_GetDataFormatString(s_uCurrentLeaderboard, uColumn);
	Asura_Assert(strstr(szFormat, "%") && strstr(szFormat, "f"), "format string must be float specifier");
	const float fData = s_pxLeaderboard->GetData(uRow, uColumn);
	Asura_CommandConsole::Printf( szFormat, fData );
}

//////////////////////////////////////////////////////////////////////////
// Asura_Leaderboard
//////////////////////////////////////////////////////////////////////////
Asura_Leaderboard::Asura_Leaderboard( u_int uLeaderboardIndex, bool bFriendsOnly )
: m_bFriendsOnly( bFriendsOnly )
, m_uLeaderboardIndex(uLeaderboardIndex)
, m_uNumRows(0)
, m_uFirstRow(0)
, m_uViewOffset(0)
, m_uTotalEntries(uASURA_LEADERBOARD_INVALID_NUM_ENTRIES)
, m_bDataValid(false)
, m_bError(false)
, m_bFetchInProgress(false)
, m_bFetchQueued(false)
, m_pPlatformData(NULL)
{
	m_uNumColumns = Asura_Leaderboard_System::Project_GetNumLeaderboardColumns(uLeaderboardIndex);

	m_pxData = new Asura_Leaderboard_Row[uASURA_LEADERBOARD_MAX_ROWS_IN_MEMORY];
	Asura_Mem::Set(m_pxData, 0, sizeof(Asura_Leaderboard_Row) * uASURA_LEADERBOARD_MAX_ROWS_IN_MEMORY);

	Platform_Initialise();
}

Asura_Leaderboard::~Asura_Leaderboard()
{
	Platform_Shutdown();

	delete[] m_pxData;
}

void Asura_Leaderboard::Update()
{
	if(m_bError)
	{
		return;
	}

	if (m_bFetchInProgress)
	{
		if (Platform_Update())
		{
			m_bFetchInProgress = false;
			m_bDataValid = !m_bError;
		}
	}
	else if (m_bFetchQueued || !m_bDataValid)
	{
		m_bFetchInProgress = Platform_FetchData();
		m_bFetchQueued = false;
		m_bDataValid = false;
		m_bError = false;
	}
}

u_int Asura_Leaderboard::GetNumAvailableRowsInPage() const
{
	if (m_bDataValid && m_uTotalEntries)
	{
		Asura_Assert(m_uNumRows > m_uViewOffset, "view offset past end of data");
		return Asura_Maths::Min(Asura_Leaderboard_System::Project_GetRowsPerPage(), m_uNumRows - m_uViewOffset);
	}
	return 0;
}

void Asura_Leaderboard::SetViewPosition( u_int uPos )
{
	if (m_bDataValid)
	{
		Asura_Assert(m_uTotalEntries != uASURA_LEADERBOARD_INVALID_NUM_ENTRIES, "valid data should set total entries");
		// prevent position being set past end of server data.
		uPos = Asura_Maths::Min(uPos, m_uTotalEntries - 1);

		// does this position keep the current page within the data we have?
		const u_int uEndOfRange = Asura_Maths::Min(uPos + Asura_Leaderboard_System::Project_GetRowsPerPage(), m_uTotalEntries);
		if (uPos >= m_uFirstRow && uEndOfRange <= m_uFirstRow + m_uNumRows)
		{
			// yes, just change the view offset within the current data
			m_uViewOffset = uPos - m_uFirstRow;
			return;
		}
	}
	// we need to get the requested data. Set the first row to the requested row and view offset to 0.
	// TODO: this is optimised for scrolling down. scrolling up will fetch the data every time.
	if (m_uFirstRow != uPos)
	{
		m_uFirstRow = uPos;
		m_bFetchQueued = true;
	}
	m_uViewOffset = 0;
}

void Asura_Leaderboard::PageUp()
{
	const u_int uRowsPerPage = Asura_Leaderboard_System::Project_GetRowsPerPage();
	const u_int uCurViewPosition = GetViewPosition();
	if (uCurViewPosition > uRowsPerPage)
	{
		SetViewPosition(uCurViewPosition - uRowsPerPage);
	}
	else
	{
		SetViewPosition(0);
	}
}

void Asura_Leaderboard::PageDown()
{
	if (m_uTotalEntries == uASURA_LEADERBOARD_INVALID_NUM_ENTRIES)
	{
		return;
	}

	const u_int uRowsPerPage = Asura_Leaderboard_System::Project_GetRowsPerPage();
	// if we're already on the last page, do nothing.
	if (GetViewPosition() + GetNumAvailableRowsInPage() >= m_uTotalEntries)
	{
		return;
	}
	const u_int uLastValidPosition = m_uTotalEntries - 1;
	const u_int uNewViewPosition = Asura_Maths::Min(uLastValidPosition, GetViewPosition() + uRowsPerPage);
	SetViewPosition(uNewViewPosition);
}

const Asura_UnicodeChar* Asura_Leaderboard::GetName(u_int uRowOffset) const
{
	Asura_Assert(m_bDataValid, "no valid data, returning 0");
	if (!m_bDataValid)
	{
		return L"";
	}
	uRowOffset += m_uViewOffset;
	Asura_Assert(uRowOffset < m_uNumRows, "invalid row offset");
	return m_pxData[uRowOffset].m_awcName;
}

u_int Asura_Leaderboard::GetRank(u_int uRowOffset) const
{
	Asura_Assert(m_bDataValid, "no valid data, returning 0");
	if (!m_bDataValid)
	{
		return 0;
	}
	uRowOffset += m_uViewOffset;
	Asura_Assert(uRowOffset < m_uNumRows, "invalid row offset");
	return m_pxData[uRowOffset].m_uRank;
}

float Asura_Leaderboard::GetData(u_int uRowOffset, u_int uColumn) const
{
	Asura_Assert(m_bDataValid, "no valid data, returning 0");
	if (!m_bDataValid)
	{
		return 0.0f;
	}
	Asura_Assert(uColumn < m_uNumColumns, "invalid leaderboard column");
	uRowOffset += m_uViewOffset;
	Asura_Assert(uRowOffset < m_uNumRows, "invalid row offset");
	return m_pxData[uRowOffset].m_afData[uColumn];
}

void Asura_Leaderboard::SetData( u_int uRow, u_int uColumn, float fData )
{
	Asura_Assert(uColumn < m_uNumColumns, "invalid leaderboard column");
	Asura_Assert(uRow < m_uNumRows, "invalid row offset");
	m_pxData[uRow].m_afData[uColumn] = fData;
}

void Asura_Leaderboard::SetASCIINameAndRank(u_int uRow, const char* szName, u_int uRank)
{
	Asura_Assert(uRow < m_uNumRows, "invalid row offset");
	Asura_Assert(strlen(szName) < ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE, "player name too long");
	Asura_PrintfToBfr(m_pxData[uRow].m_awcName, L"%S", ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE, true, szName);
	m_pxData[uRow].m_uRank = uRank;
}

void Asura_Leaderboard::SetUCS2NameAndRank(u_int uRow, const Asura_UnicodeChar* wszName, u_int uRank)
{
	Asura_Assert(uRow < m_uNumRows, "invalid row offset");
	Asura_Assert(wcslen(wszName) < ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE, "player name too long");
	Asura_PrintfToBfr(m_pxData[uRow].m_awcName, L"%s", ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE, true, wszName);
	m_pxData[uRow].m_uRank = uRank;
}

void Asura_Leaderboard::SetUTF8NameAndRank(u_int uRow, const char* szName, u_int uRank)
{
	Asura_Assert(uRow < m_uNumRows, "invalid row offset");
	// NY - 24 April 2009 - name length check non-trivial with utf8
	Asura_UTF8ToUnicode(szName, m_pxData[uRow].m_awcName, ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE);
	m_pxData[uRow].m_uRank = uRank;
}

void Asura_Leaderboard::SortData()
{
	Asura_SortAndSearch::Qsort(m_pxData, m_uNumRows, sizeof(Asura_Leaderboard_Row), Asura_SortAndSearch::SortAscendingUint);
}

#endif // #ifdef ASURA_SUPPORT_LEADERBOARDS
