//
// Asura_Physics_CollisionReport.h
//
// Created by KJFL - 09 September 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PHYSICS_COLLISIONREPORT_H__
#define __ASURA_PHYSICS_COLLISIONREPORT_H__

#include "Core/Asura_Handle.h"
#include "Network/Asura_Guid.h"

// An object-object collision report - these are accessed by querying Asura_Physics_Object.
// Creation/deletion is handled internally by the Physics system, so users should only
// see them as a read-only construct.
class Asura_Physics_ObjectCollisionReport
{
	public:

		inline Asura_Guid GetEntityGuid( void ) const
		{
			return m_uEntityGuid;
		}

		inline Asura_Handle GetPhysicsHandle( void ) const
		{
			return m_iPhysicsHandle;
		}

		inline const Asura_Physics_ObjectCollisionReport* GetNextReport( void ) const
		{
			return m_pxNextReport;
		}

	private:

		friend class Asura_Physics_RigidBodyInstance;

		// private constructor/destructor to stop outside systems abusing class
		Asura_Physics_ObjectCollisionReport();
		~Asura_Physics_ObjectCollisionReport();

		Asura_Handle							m_iPhysicsHandle;
		Asura_Guid								m_uEntityGuid;

		Asura_Physics_ObjectCollisionReport*	m_pxNextReport;
};

class Asura_Physics_EnvironmentCollisionReport
{
	public:

		// Get the unique environment Face ID. You can use this with Asura_Physics_Environment
		// to access all the data available for the face.
		inline u_int GetFaceID( void ) const { return m_uFaceID; }

		// Get the face's normal. Normals are lazily evaluated (e.g. the Asura_Physics_Environment
		// system doesn't actually store them) so we'll cache this here since the Physics will
		// have already worked it out.
		inline const Asura_AVector3& GetFaceNormal( void ) const { return m_xNormal; }

		// Get the the poly flags for the face. Note, these aren't stored in the report since the
		// Physics doesn't always have to query them. Effectively this code will just make a call
		// to Asura_Physics_Environment the same as the caller could - the method is provided here
		// to make things easier.
		u_short GetPolyFlags( void ) const;

#ifdef ASURA_TOOLS
        // Similar to the above function, except it returns the full set of flags including tools
        // only flags.
        u_int Tools_GetPolyFlags( void ) const;
#ifdef TOOLS_INCLUDE_PHYSICS_MODULE_AND_POLY_LOOKUP_DATA
		void Tools_CalcModuleAndPolyIndex( u_int& uModuleIndex, u_int& uPolyIndex ) const;
#endif
#endif // ASURA_TOOLS

		inline const Asura_Physics_EnvironmentCollisionReport* GetNextReport( void ) const
		{
			return m_pxNextReport;
		}

	private:

		friend class Asura_Physics_RigidBodyInstance;

		// private constructor/destructor to stop outside systems abusing class
		Asura_Physics_EnvironmentCollisionReport();
		~Asura_Physics_EnvironmentCollisionReport();

		Asura_AVector3								m_xNormal;
		u_int										m_uFaceID;

		Asura_Physics_EnvironmentCollisionReport*	m_pxNextReport;
};

#endif//__ASURA_PHYSICS_COLLISIONREPORT_H__
