//
// Asura_Physics_RestFrame.h
//
// Created by KJFL - 13 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PHYSICS_RESTFRAME_H__
#define __ASURA_PHYSICS_RESTFRAME_H__


#include "Core/Asura_Maths.h"

class Asura_Physics_RigidBodyInstance;

class Asura_Physics_RestFrame
{
	
	public:	

		// "System" level methods to Initialise, Reset and Shutdown the list of RestFrames.
		static void Initialise( void );
		static void Reset( void );
		static void Shutdown( void );

		// "System" level method to Update all of the RestFrames
		static void UpdateAll( void );

		// Create / Destroy a single RestFrame instance
		static Asura_Physics_RestFrame* Create( const Asura_Physics_RigidBodyInstance* pxParent );
		static void Destroy( Asura_Physics_RestFrame* pxRestFrame );

		void Link( void );
		void Unlink( void );

		// Set the RestFrame's parent, from which it will take its movement
		void SetParent( const Asura_Physics_RigidBodyInstance* pxParent );

		// Check the RestFrame is valid (e.g. it hasn't been destroyed)
		bool IsValid( void ) const { return m_pxParentObject != 0; } 

		bool IsUsed( void ) const { return m_uReferenceCount != 0; }  

		void TransformPosition( Asura_AVector3& xPosition ) const;

		void TransformOrientation( Asura_AMatrix3x3& xOrient ) const;

	private:

		static Asura_Physics_RestFrame*			s_pxHeadOfList;

		Asura_Physics_RestFrame(){};
		~Asura_Physics_RestFrame(){};

		void Update( void );

		Asura_AMatrix3x3						m_xLastOrientationTransposed;
		Asura_AMatrix3x3						m_xOrientation;

		Asura_AVector3							m_xPosition;
		Asura_AVector3							m_xLastPosition;

		const Asura_Physics_RigidBodyInstance*	m_pxParentObject;

		u_int									m_uReferenceCount;

		Asura_Physics_RestFrame*				m_pxNext;
};


#endif//__ASURA_PHYSICS_RESTFRAME_H__
