//
// Asura_Physics_RigidBodyDefinition.cpp
//
// Created by KJFL - 2004-03-22 10:50:59
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Physics/Asura_Physics_RigidBodyDefinition.h"

#include "Core/Memory/Asura_Mem.h"
#include "Dynamics/Asura_Collision_Object.h"

Asura_Physics_RigidBody_ObjectMesh* Asura_Physics_RigidBody_ObjectMesh::Create( void )
{
	// will replace with mempool etc 
	return new Asura_Physics_RigidBody_ObjectMesh;
}

// cube face definitions, listing vertices counter-clockwise
static const u_short g_ausCubeFaceVertices[] = 
{
	0,1,2,
	1,3,2,
	
	1,5,3,
	5,7,3,
	
	4,6,7,
	7,5,4,
	
	0,2,6,
	6,4,0,
	
	2,3,7,
	7,6,2,

	0,5,1,
	0,4,5
};


static const Asura_AVector3 g_axCubeFaceNormals[] = 
{
	Asura_AVector3(-1.0f,  0.0f,  0.0f), 
	Asura_AVector3(-1.0f,  0.0f,  0.0f), 

	Asura_AVector3( 0.0f,  0.0f, +1.0f), 
	Asura_AVector3( 0.0f,  0.0f, +1.0f), 

	Asura_AVector3(+1.0f,  0.0f,  0.0f), 
	Asura_AVector3(+1.0f,  0.0f,  0.0f), 

	Asura_AVector3( 0.0f,  0.0f, -1.0f), 
	Asura_AVector3( 0.0f,  0.0f, -1.0f), 
									  
	Asura_AVector3( 0.0f, +1.0f,  0.0f), 
	Asura_AVector3( 0.0f, +1.0f,  0.0f), 
									  
	Asura_AVector3( 0.0f, -1.0f,  0.0f), 
	Asura_AVector3( 0.0f, -1.0f,  0.0f), 
};

static u_char g_aucEdgeVertices[] = 
{
	0,1,
	1,3,
	2,3,
	0,2,
	0,4,
	4,6,
	2,6,
	6,7,
	4,5,
	5,7,
	1,5,
	3,7
};

static u_char g_aucEdgeFaces[] = 
{
	0,10,
	1,2,
	1,8,
	0,6,
	7,11,
	4,7,
	6,9,
	4,9,
	5,11,
	3,5,
	2,10,
	3,8
};


// hexahedron face definitions, listing vertices counter-clockwise
static const u_short g_ausHexahedronFaceVertices[] = 
{
	0,1,3,
	0,3,2,
	0,2,1,

	1,4,3,
	3,4,2,
	2,4,1
};

static u_char g_aucHexahedronEdgeVertices[] = 
{
	0,1,
	0,2,
	0,3,
	1,3,
	3,2,
	2,1,
	4,1,
	4,2,
	4,3
};

static u_char g_aucHexahedronEdgeFaces[] = 
{
	0,2,
	1,2,
	0,1,
	0,3,
	1,4,
	2,5,
	3,5,
	4,5,
	3,4

};

#define SQRT_3 1.73205f
#define SQRT_2 1.41421f
#define SQRT_HALF 0.70711f

static const float g_afHexahedronVertices[] = 
{
	0.0f, -1.0f, 0.0f,
	-SQRT_3/2.0f, 0.0f, -0.5f,
	0.0f, 0.0f, 1.0f,
	+SQRT_3/2.0f, 0.0f, -0.5f,
	0.0f, +1.0f, 0.0f,
};


Asura_Physics_RigidBody_ObjectMesh* Asura_Physics_RigidBody_ObjectMesh::CreateCuboid( float fWidth, float fHeight, float fDepth )
{
	Asura_Physics_RigidBody_ObjectMesh* pxMesh = new Asura_Physics_RigidBody_ObjectMesh;

	// create the vertices
	pxMesh->m_uNumberOfVertices = 8;
	pxMesh->m_pxVertexPositions = new Asura_AVector3[pxMesh->m_uNumberOfVertices];
	pxMesh->m_pxVertexNormals = new Asura_AVector3[pxMesh->m_uNumberOfVertices];

	// The loops below create the cuboid based on a cube with a vertex order like this:
	//
	// { -1.0f, -1.0f, -1.0f, 1.0f }
	// { -1.0f, -1.0f, +1.0f, 1.0f }
	// { -1.0f, +1.0f, -1.0f, 1.0f }
	// { -1.0f, +1.0f, +1.0f, 1.0f }
	// { +1.0f, -1.0f, -1.0f, 1.0f }
	// { +1.0f, -1.0f, +1.0f, 1.0f }
	// { +1.0f, +1.0f, -1.0f, 1.0f }
	// { +1.0f, +1.0f, +1.0f, 1.0f }
	//
	// The vertex normal is based on averaging the normals of the vertex's adjacent faces.
	// So, for a cube this means the normals all point away from the cube's centre.

	{
		u_int u = 0;

		const float fNormalise = 1.0f / SQRT_3;

		for (float fX = -1.0f; fX <= 1.0f; fX += 2.0f)
		{
			for (float fY = -1.0f; fY <= 1.0f; fY += 2.0f)
			{
				for (float fZ = -1.0f; fZ <= 1.0f; fZ += 2.0f)
				{
					pxMesh->m_pxVertexPositions[u].SetX(fWidth * fX);
					pxMesh->m_pxVertexPositions[u].SetY(fHeight * (fY - 1.0f));
					pxMesh->m_pxVertexPositions[u].SetZ(fDepth * fZ);
					//pxMesh->m_pxVertexPositions[u].SetW(1.0f);

					pxMesh->m_pxVertexNormals[u].SetX(fX * fNormalise);
					pxMesh->m_pxVertexNormals[u].SetY(fY * fNormalise);
					pxMesh->m_pxVertexNormals[u].SetZ(fZ * fNormalise);
					//pxMesh->m_pxVertexNormals[u].SetW(0.0f);
			
					u++;
				}
			}
		}
	}

	// Faces - fairly straight-forward, but the faces are actually triangles so there's 12 of them
	pxMesh->m_uNumberOfFaces = 12;
	pxMesh->m_pxFaceNormals = new Asura_AVector3[pxMesh->m_uNumberOfFaces];
	pxMesh->m_pusFaceVertexIndices = new u_short[3*pxMesh->m_uNumberOfFaces];

	Asura_Mem::Copy( pxMesh->m_pxFaceNormals, g_axCubeFaceNormals, pxMesh->m_uNumberOfFaces * sizeof(Asura_AVector3) );
	Asura_Mem::Copy( pxMesh->m_pusFaceVertexIndices, g_ausCubeFaceVertices, pxMesh->m_uNumberOfFaces * 3 * sizeof(u_short));



	// Next, let's do edges. We only need to consider 'sticky-out' edges that might
	// be involved in collisions, so not edges between coplanar faces.
	pxMesh->m_uNumberOfEdges = 12;
	pxMesh->m_pxEdges = new Asura_Physics_RigidBody_ObjectEdge[pxMesh->m_uNumberOfEdges];

	for (u_int u=0; u<pxMesh->m_uNumberOfEdges; u++)
	{
		pxMesh->m_pxEdges[u].m_FirstVertexIndex = g_aucEdgeVertices[u*2];
		pxMesh->m_pxEdges[u].m_SecondVertexIndex = g_aucEdgeVertices[u*2+1];

		u_int uFace1 = g_aucEdgeFaces[u*2];
		u_int uFace2 = g_aucEdgeFaces[u*2+1];
		
		pxMesh->m_pxEdges[u].m_xNormal = pxMesh->m_pxFaceNormals[uFace1] + pxMesh->m_pxFaceNormals[uFace2];
		pxMesh->m_pxEdges[u].m_xNormal.Normalise();
	}


	

	return pxMesh; 
}



Asura_Physics_RigidBody_ObjectMesh* Asura_Physics_RigidBody_ObjectMesh::CreateCuboid( const Asura_Bounding_Box& xBB )
{
	Asura_Physics_RigidBody_ObjectMesh* pxMesh = new Asura_Physics_RigidBody_ObjectMesh;

	// create the vertices
	pxMesh->m_uNumberOfVertices = 8;
	pxMesh->m_pxVertexPositions = new Asura_AVector3[pxMesh->m_uNumberOfVertices];
	pxMesh->m_pxVertexNormals = new Asura_AVector3[pxMesh->m_uNumberOfVertices];

	// The loops below create the cuboid based on a cube with a vertex order like this:
	//
	// { -1.0f, -1.0f, -1.0f, 1.0f }
	// { -1.0f, -1.0f, +1.0f, 1.0f }
	// { -1.0f, +1.0f, -1.0f, 1.0f }
	// { -1.0f, +1.0f, +1.0f, 1.0f }
	// { +1.0f, -1.0f, -1.0f, 1.0f }
	// { +1.0f, -1.0f, +1.0f, 1.0f }
	// { +1.0f, +1.0f, -1.0f, 1.0f }
	// { +1.0f, +1.0f, +1.0f, 1.0f }
	//
	// The vertex normal is based on averaging the normals of the vertex's adjacent faces.
	// So, for a cube this means the normals all point away from the cube's centre.

	{
		u_int u = 0;

		const float fNormalise = 1.0f / SQRT_3;

		for (float fX = -1.0f; fX <= 1.0f; fX += 2.0f)
		{
			for (float fY = -1.0f; fY <= 1.0f; fY += 2.0f)
			{
				for (float fZ = -1.0f; fZ <= 1.0f; fZ += 2.0f)
				{
					pxMesh->m_pxVertexNormals[u].SetX(fX * fNormalise);
					pxMesh->m_pxVertexNormals[u].SetY(fY * fNormalise);
					pxMesh->m_pxVertexNormals[u].SetZ(fZ * fNormalise);
					//pxMesh->m_pxVertexNormals[u].SetW(0.0f);
			
					u++;
				}
			}
		}
	}

	pxMesh->m_pxVertexPositions[0].SetX(xBB.MinX);
	pxMesh->m_pxVertexPositions[0].SetY(xBB.MinY);
	pxMesh->m_pxVertexPositions[0].SetZ(xBB.MinZ);
	
	pxMesh->m_pxVertexPositions[1].SetX(xBB.MinX);
	pxMesh->m_pxVertexPositions[1].SetY(xBB.MinY);
	pxMesh->m_pxVertexPositions[1].SetZ(xBB.MaxZ);
	
	pxMesh->m_pxVertexPositions[2].SetX(xBB.MinX);
	pxMesh->m_pxVertexPositions[2].SetY(xBB.MaxY);
	pxMesh->m_pxVertexPositions[2].SetZ(xBB.MinZ);

	pxMesh->m_pxVertexPositions[3].SetX(xBB.MinX);
	pxMesh->m_pxVertexPositions[3].SetY(xBB.MaxY);
	pxMesh->m_pxVertexPositions[3].SetZ(xBB.MaxZ);

	pxMesh->m_pxVertexPositions[4].SetX(xBB.MaxX);
	pxMesh->m_pxVertexPositions[4].SetY(xBB.MinY);
	pxMesh->m_pxVertexPositions[4].SetZ(xBB.MinZ);

	pxMesh->m_pxVertexPositions[5].SetX(xBB.MaxX);
	pxMesh->m_pxVertexPositions[5].SetY(xBB.MinY);
	pxMesh->m_pxVertexPositions[5].SetZ(xBB.MaxZ);

	pxMesh->m_pxVertexPositions[6].SetX(xBB.MaxX);
	pxMesh->m_pxVertexPositions[6].SetY(xBB.MaxY);
	pxMesh->m_pxVertexPositions[6].SetZ(xBB.MinZ);

	pxMesh->m_pxVertexPositions[7].SetX(xBB.MaxX);
	pxMesh->m_pxVertexPositions[7].SetY(xBB.MaxY);
	pxMesh->m_pxVertexPositions[7].SetZ(xBB.MaxZ);




	// Faces - fairly straight-forward, but the faces are actually triangles so there's 12 of them
	pxMesh->m_uNumberOfFaces = 12;
	pxMesh->m_pxFaceNormals = new Asura_AVector3[pxMesh->m_uNumberOfFaces];
	pxMesh->m_pusFaceVertexIndices = new u_short[3*pxMesh->m_uNumberOfFaces];

	Asura_Mem::Copy( pxMesh->m_pxFaceNormals, g_axCubeFaceNormals, pxMesh->m_uNumberOfFaces * sizeof(Asura_AVector3) );
	Asura_Mem::Copy( pxMesh->m_pusFaceVertexIndices, g_ausCubeFaceVertices, pxMesh->m_uNumberOfFaces * 3 * sizeof(u_short));



	// Next, let's do edges. We only need to consider 'sticky-out' edges that might
	// be involved in collisions, so not edges between coplanar faces.
	pxMesh->m_uNumberOfEdges = 12;
	pxMesh->m_pxEdges = new Asura_Physics_RigidBody_ObjectEdge[pxMesh->m_uNumberOfEdges];

	for (u_int u=0; u<pxMesh->m_uNumberOfEdges; u++)
	{
		pxMesh->m_pxEdges[u].m_FirstVertexIndex = g_aucEdgeVertices[u*2];
		pxMesh->m_pxEdges[u].m_SecondVertexIndex = g_aucEdgeVertices[u*2+1];

		u_int uFace1 = g_aucEdgeFaces[u*2];
		u_int uFace2 = g_aucEdgeFaces[u*2+1];
		
		pxMesh->m_pxEdges[u].m_xNormal = pxMesh->m_pxFaceNormals[uFace1] + pxMesh->m_pxFaceNormals[uFace2];
		pxMesh->m_pxEdges[u].m_xNormal.Normalise();
	}


	

	return pxMesh; 
}

Asura_Physics_RigidBody_ObjectMesh* Asura_Physics_RigidBody_ObjectMesh::CreateHexahedron( float fWidth, float fHeight )
{
	Asura_Physics_RigidBody_ObjectMesh* pxMesh = new Asura_Physics_RigidBody_ObjectMesh;

	// create the vertices
	pxMesh->m_uNumberOfVertices = 5;
	pxMesh->m_pxVertexPositions = new Asura_AVector3[pxMesh->m_uNumberOfVertices];
	pxMesh->m_pxVertexNormals = new Asura_AVector3[pxMesh->m_uNumberOfVertices];

	for (u_int u=0; u<pxMesh->m_uNumberOfVertices; u++)
	{
		pxMesh->m_pxVertexPositions[u].SetX(g_afHexahedronVertices[u*3+0] * fWidth);
		pxMesh->m_pxVertexPositions[u].SetY(g_afHexahedronVertices[u*3+1] * fHeight);
		pxMesh->m_pxVertexPositions[u].SetZ(g_afHexahedronVertices[u*3+2] * fWidth);
//		pxMesh->m_pxVertexPositions[u].SetW(1.0f);

		pxMesh->m_pxVertexNormals[u] = pxMesh->m_pxVertexPositions[u];
		pxMesh->m_pxVertexNormals[u].Normalise();
	}

	pxMesh->m_uNumberOfFaces = 6;
	pxMesh->m_pxFaceNormals = new Asura_AVector3[pxMesh->m_uNumberOfFaces];
	pxMesh->m_pusFaceVertexIndices = new u_short[3*pxMesh->m_uNumberOfFaces];

	Asura_Mem::Copy( pxMesh->m_pusFaceVertexIndices, g_ausHexahedronFaceVertices, pxMesh->m_uNumberOfFaces * 3 * sizeof(u_short));
	
	for (u_int u=0; u<pxMesh->m_uNumberOfFaces; u++)
	{
		const Asura_AVector3& xV0 = pxMesh->m_pxVertexNormals[pxMesh->m_pusFaceVertexIndices[u*3+0]];
		const Asura_AVector3& xV1 = pxMesh->m_pxVertexNormals[pxMesh->m_pusFaceVertexIndices[u*3+1]];
		const Asura_AVector3& xV2 = pxMesh->m_pxVertexNormals[pxMesh->m_pusFaceVertexIndices[u*3+2]];

		// get the normal of the face
		Asura_AVector3 xNormal = ((xV1 - xV0)^(xV2 - xV0));
		xNormal.Normalise();

		pxMesh->m_pxFaceNormals[u] = xNormal;
	}

	// Next, let's do edges. We only need to consider 'sticky-out' edges that might
	// be involved in collisions, so not edges between coplanar faces.
	pxMesh->m_uNumberOfEdges = 9;
	pxMesh->m_pxEdges = new Asura_Physics_RigidBody_ObjectEdge[pxMesh->m_uNumberOfEdges];

	for (u_int u=0; u<pxMesh->m_uNumberOfEdges; u++)
	{
		pxMesh->m_pxEdges[u].m_FirstVertexIndex = g_aucHexahedronEdgeVertices[u*2];
		pxMesh->m_pxEdges[u].m_SecondVertexIndex = g_aucHexahedronEdgeVertices[u*2+1];

		u_int uFace1 = g_aucHexahedronEdgeFaces[u*2];
		u_int uFace2 = g_aucHexahedronEdgeFaces[u*2+1];
		
		pxMesh->m_pxEdges[u].m_xNormal = pxMesh->m_pxFaceNormals[uFace1] + pxMesh->m_pxFaceNormals[uFace2];
		pxMesh->m_pxEdges[u].m_xNormal.Normalise();
	}


	

	return pxMesh; 
}


Asura_Physics_RigidBodyDefinition::Asura_Physics_RigidBodyDefinition( const Asura_Collision_Object* pxCollisionObject )
{
	const Asura_Bounding_Box& xBB = pxCollisionObject->GetBoundingBox();

	m_pxObjectMesh = Asura_Physics_RigidBody_ObjectMesh::CreateCuboid( xBB );

	CalculateOBBHalfExtents( xBB );

	m_pxCollisionMesh = pxCollisionObject->GetMesh();
}


void Asura_Physics_RigidBodyDefinition::CalculateOBBHalfExtents( const Asura_Bounding_Box& xBB )
{
    m_fOBBHalfExtentsX = (xBB.MaxX - xBB.MinX) * 0.5f;
	m_fOBBHalfExtentsY = (xBB.MaxY - xBB.MinY) * 0.5f;
	m_fOBBHalfExtentsZ = (xBB.MaxZ - xBB.MinZ) * 0.5f;

	m_xOBBCentreOffset = xBB.CalculateCentre();
}
