//
// Asura_Physics_Vehicle_Speedboat_Description.h
//
// Created by Andrew Weinkove - 30/08/05
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PHYSICS_VEHICLE_SPEEDBOAT_DESCRIPTION_H__
#define __ASURA_PHYSICS_VEHICLE_SPEEDBOAT_DESCRIPTION_H__

//
// (AW - 31/08/05) - Structure to describe speedboat settings. 
//
// While I have attempted to use clear names and have commented each parameter,
// I fear that few of these numbers make much sense.
//

class Asura_Physics_Vehicle_Speedboat_Description
{
public:
	float m_fDepthOffset;					// Added on to all (GetPosition().y - fWaterLevel) calculations.
	float m_fTiltDepthOffset;				// How much tilting back changes depth calculation.
	float m_fMaxBouyancyDepth;				// Depth beyond which bouyancy is constant. 
	float m_fMaxBouyancyForce;				// Bouyancy when at max bouyancy depth.
	float m_fSpeedboatGravity;				// Wouldn't it be nice if 10m/s/s looked right?
	float m_fEnginePower;					// Engine power
	float m_fReversePowerProportion;		// Proportion of power when in reverse.
	float m_fDepthZeroEnginePower;			// Depth at which engine power is zero.
	float m_fDepthFullEnginePower;			// Depth at which engine power is 100%
	float m_fMinFrictionDepth;				// Depth at which friction (acting on velocity) is at its minimum.
	float m_fMaxFrictionDepth;				// Depth at which friction (acting on velocity) is at full strength.
	float m_fMinFrictionProportion;			// Proportion of friction (acting on velocity) used when at minimum friction depth.
	float m_fFrictionGoingForwards;			// Friction when going forwards.
	float m_fFrictionGoingBackwards;		// Friction when going backwards.
	float m_fFrictionForVerticalMovement;	// Friction for vertical movement.
	float m_fFrictionAngularXZ;				// Friction applied to Euler.x and Euler.z components of angular velocity.
	float m_fFrictionAngularY;				// Friction applied to Euler.y component of angular velocity.
	float m_fTiltCorrectionMax;				// Strength of force that corrects tilt (while fully in water)
	float m_fTiltCorrectionMin;				// Strength of force that corrects tilt (while fully out of water)
	float m_fMaxTiltCorrectionDepth;		// Depth at which to use m_fTiltCorrectionMax.
	float m_fMinTiltCorrectionDepth;		// Depth at which to use m_fTiltCorrectionMin.
	float m_fTiltAcceleratingForward;		// Tilt caused by accelerating forwards.
	float m_fTiltAcceleratingSideways;		// Tilt caused by accelerating sideways.
	float m_fTiltTravellingForward;			// Tilt caused by travelling forwards.
	float m_fTiltTravellingSideways;		// Tilt caused by travelling sideways.
	float m_fMinRudderDepth;				// Depth at which rudder effectiveness is zero.
	float m_fMaxRudderDepth;				// Depth at which rudder effectiveness is at its maximum.
	float m_fRudderVelocityForceMax;		// Max force resulting from rudder (doesn't include rotational forces from engine orientation)
	float m_fVelocityRudderEffectiveness;	// How quickly the rudder affects the boats orientation as a result of rudder orientation and velocity.
	float m_fThrustRudderEffectiveness;		// How quickly the engine orientation affects the boats orientation
	float m_fChangeDirectionAbility;		// Ability of the boat to transfer its momentum to its new direction.
	float m_fYRotationCentreAdjustment;		// How much the centre of rotation is adjusted as a result of tilting.
	float m_fMaxRotationCentreOffset;		// Maximum distance the centre of rotation can move back.
	float m_fNoiseProportionAngularX;		// Proportion of force that we add/substract to angular X tilt
	float m_fMinTurningSpeed;				// Min speed applied to turn the boat on the spot
};

#endif//__ASURA_PHYSICS_VEHICLE_SPEEDBOAT_DESCRIPTION_H__
