//
// Asura_Environment_Renderer.cpp
//
// Created by KJFL - 11 March 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

//////////////////////////////////////////////////////////////////////////
// ART - 19/11/08 - Temporarily changed #ifdef to SM3 platforms only.
// Technically there's no reason this class shouldn't exist for all 
// platforms but it does require the new visibility system (which is 
// currently only feasible for SM3 platforms).
//////////////////////////////////////////////////////////////////////////
#if defined(ASURA_TARGET_SM3)

// This file's header.
#include "Rendering/Asura_Environment_Renderer.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Asura_Texture_Streaming.h"
#include "Core/Asura_Timers.h"
#include "Rendering/Asura_Module.h"
#include "Visibility/Asura_Visibility_System.h"

u_int					Asura_Environment_Renderer::s_uStripVisibilityQueryFrameCount	= 0;
u_int					Asura_Environment_Renderer::s_uNumStripsForVisibility			= 0;

bool*					Asura_Environment_Renderer::s_pbStripVisibleFromCurrentCameraThisFrame			= NULL;
bool*					Asura_Environment_Renderer::s_pbStripVisibleFromCurrentCameraIgnoringDirectionThisFrame			= NULL;
Asura_Static_Visibility_Query*	Asura_Environment_Renderer::s_pxStripVisbilityQueries	= NULL;

void Asura_Environment_Renderer::Initialise(void)
{
}

void Asura_Environment_Renderer::Reset(void)
{
	delete[](s_pbStripVisibleFromCurrentCameraThisFrame);
	delete[](s_pbStripVisibleFromCurrentCameraIgnoringDirectionThisFrame);
	delete[](s_pxStripVisbilityQueries);

	s_uStripVisibilityQueryFrameCount	= 0;
	s_uNumStripsForVisibility			= 0;

	s_pbStripVisibleFromCurrentCameraThisFrame			= NULL;
	s_pbStripVisibleFromCurrentCameraIgnoringDirectionThisFrame			= NULL;
	s_pxStripVisbilityQueries	= NULL;
}

void Asura_Environment_Renderer::Shutdown(void)
{
	Reset();
}

#if defined(ASURA_USE_TEXTURE_STREAMING)

void Asura_Environment_Renderer::SetTexturePriorities( void )
{
	const Asura_Vector_3& xCameraPosition = *(Asura_Camera::GetPosition());
	Asura_Vector_3 xCameraDirection;

	Asura_Camera::GetForwardVector( xCameraDirection );

	const float fLoadingDistance = 30.0f;
	const float fLoadingDistanceSqdRcp = 1.0f / ( fLoadingDistance*fLoadingDistance );

	// Ensure that our strip visibility is up-to-date this frame...
	UpdateVisibilityOfStrips();

	PrefetchStrip(0);

	for( u_int u = 0; u < GetNumberOfStrips(); ++u )
	{
		PrefetchStrip(u + 1);

		// TODO - don't query module system, but go via Visibility system direct?
		// With the visibility system on this effectively does "linked in" modules, 
		// which is what we want.
		if( !Asura_Module_System::IsModuleVisible( GetModuleIndex(u) ) ) continue;

		u_int uBasePriority = 10000;
		// TODO - each strip should store it's own visibility query to cache last results

		// TODO - is there a way of combining the work of these tests? (We know the BB hasn't changed.)
		if(!IsStripVisibleFromCurrentCameraIgnoringDirection(u)) continue;
	
		// Priority drops to 0 at 100m distance
//		uBasePriority = Asura_Maths::Max( 0, 10000 - Asura_Maths::FloatToInt( GetBoundingBox(u).GetSqdMinDistFromPoint( xCameraPosition ) ) );

		// find the point on the bounding box closest to the camera position

		const Asura_Bounding_Box&	xAABB	= GetBoundingBox(u);
		Asura_Vector_3 xClosestPoint
		(
			Asura_Maths::ClampToRange( xCameraPosition.GetX(), xAABB.MinX, xAABB.MaxX ),
			Asura_Maths::ClampToRange( xCameraPosition.GetY(), xAABB.MinY, xAABB.MaxY ),
			Asura_Maths::ClampToRange( xCameraPosition.GetZ(), xAABB.MinZ, xAABB.MaxZ )
		);

		Asura_Vector_3 xToStrip = xClosestPoint - xCameraPosition;

		float fDistanceSqd = xToStrip.MagnitudeSquared();

		if ( fDistanceSqd > 0.0001f )
		{
			float fRecip = Asura_Maths::InvSqrt( fDistanceSqd );

			float fDot = Asura_Maths::Max( 0.0f, 1.0f + (xToStrip*xCameraDirection)*fRecip );

			float fPriority = Asura_Maths::Max( 0.0f, (1.0f - fDistanceSqd * fLoadingDistanceSqdRcp)*fDot ) * 10000.f;

			uBasePriority = Asura_Maths::FloatToInt(fPriority);
		}
		else // the strip bounding box overlaps the camera or is very close
		{
			uBasePriority *= 2;
		}



		
		for ( u_int uTexturePass=0; uTexturePass<2; ++uTexturePass )
		{
			const Asura_MaterialResponse* pxMaterial = GetMaterial( u, uTexturePass );
		
			// TODO - project set priority scales for texture types?
			// TODO - screen area heuristics for texture priorities? Or at least distance to strip BB.
			// TODO - is number of polys per strip... or total surface area of strip good for prioritising? (e.g. surface area / distance)
			// TODO - reduce priorities on secondary texturing?
			// TODO - make priorities accumulate so a texture used by many strips counts more?

			if (pxMaterial)
			{
				Asura_Texture_Streaming_System::SetTexturePriority( pxMaterial->GetTexture(Asura_MaterialResponse::TRANSPARENCY_MAP),		uBasePriority*4 );	
				Asura_Texture_Streaming_System::SetTexturePriority( pxMaterial->GetTexture(Asura_MaterialResponse::ALBEDO_TEXTURE),			uBasePriority*3 );
				Asura_Texture_Streaming_System::SetTexturePriority( pxMaterial->GetTexture(Asura_MaterialResponse::DETAIL_MAP),				uBasePriority*2 );
				Asura_Texture_Streaming_System::SetTexturePriority( pxMaterial->GetTexture(Asura_MaterialResponse::NORMAL_MAP),				uBasePriority*2 );
				Asura_Texture_Streaming_System::SetTexturePriority( pxMaterial->GetTexture(Asura_MaterialResponse::DETAIL_NORMAL_MAP),		uBasePriority*2 );
				Asura_Texture_Streaming_System::SetTexturePriority( pxMaterial->GetTexture(Asura_MaterialResponse::PARALLAX_MAP),			uBasePriority );
				Asura_Texture_Streaming_System::SetTexturePriority( pxMaterial->GetTexture(Asura_MaterialResponse::ENVIRONMENT_MAP),		uBasePriority );
				Asura_Texture_Streaming_System::SetTexturePriority( pxMaterial->GetTexture(Asura_MaterialResponse::SELFILLUMINATION_MAP),	uBasePriority );
				Asura_Texture_Streaming_System::SetTexturePriority( pxMaterial->GetTexture(Asura_MaterialResponse::SPECULAR_MAP),			uBasePriority );
			}
		}
	
	}
}

void Asura_Environment_Renderer::GetTexturesUsed( void ( *pfnCallback )( int iTextureID, int  iTextureUsage) )
{
	for( u_int u = 0; u < GetNumberOfStrips(); ++u )
	{
		for ( u_int uTexturePass=0; uTexturePass<2; ++uTexturePass )
		{
			const Asura_MaterialResponse* pxMaterial = GetMaterial( u, uTexturePass );
		
			if (pxMaterial)
			{
				pfnCallback( pxMaterial->GetTexture(Asura_MaterialResponse::TRANSPARENCY_MAP), Asura_MaterialResponse::TRANSPARENCY_MAP );		
				pfnCallback( pxMaterial->GetTexture(Asura_MaterialResponse::ALBEDO_TEXTURE), Asura_MaterialResponse::ALBEDO_TEXTURE );			
				pfnCallback( pxMaterial->GetTexture(Asura_MaterialResponse::NORMAL_MAP), Asura_MaterialResponse::NORMAL_MAP );				
				pfnCallback( pxMaterial->GetTexture(Asura_MaterialResponse::DETAIL_NORMAL_MAP), Asura_MaterialResponse::DETAIL_NORMAL_MAP );				
				pfnCallback( pxMaterial->GetTexture(Asura_MaterialResponse::PARALLAX_MAP), Asura_MaterialResponse::PARALLAX_MAP );			
				pfnCallback( pxMaterial->GetTexture(Asura_MaterialResponse::ENVIRONMENT_MAP), Asura_MaterialResponse::ENVIRONMENT_MAP );		
				pfnCallback( pxMaterial->GetTexture(Asura_MaterialResponse::SELFILLUMINATION_MAP), Asura_MaterialResponse::SELFILLUMINATION_MAP );	
				pfnCallback( pxMaterial->GetTexture(Asura_MaterialResponse::SPECULAR_MAP), Asura_MaterialResponse::SPECULAR_MAP );			
				pfnCallback( pxMaterial->GetTexture(Asura_MaterialResponse::DETAIL_MAP), Asura_MaterialResponse::DETAIL_MAP );					}
		}
	
	}
}

#endif	//defined(ASURA_USE_TEXTURE_STREAMING)

u_int Asura_Environment_Renderer::GetNumberOfStrips( void )
{
	return Platform_GetNumberOfStrips();
}

void Asura_Environment_Renderer::PrefetchStrip(u_int uStrip)
{
	Platform_PrefetchStrip(uStrip);
}

u_int Asura_Environment_Renderer::GetModuleIndex( u_int uStrip )
{
	return Platform_GetModuleIndex( uStrip );
}

const Asura_Bounding_Box& Asura_Environment_Renderer::GetBoundingBox( u_int uStrip )
{
	return Platform_GetBoundingBox( uStrip );
}

const Asura_MaterialResponse* Asura_Environment_Renderer::GetMaterial( u_int uStrip, u_int uPass )
{
	return Platform_GetMaterial( uStrip, uPass );
}

void Asura_Environment_Renderer::UpdateVisibilityOfStrips(void)
{
	const u_int	uNumStrips		= GetNumberOfStrips();
	const u_int	uFrameNumber	= Asura_Timers::GetFrameNumber();

	// Do we need to allocate our visibility?
	if(s_uNumStripsForVisibility != uNumStrips)
	{
		delete[](s_pbStripVisibleFromCurrentCameraThisFrame);
		delete[](s_pbStripVisibleFromCurrentCameraIgnoringDirectionThisFrame);
		delete[](s_pxStripVisbilityQueries);

		s_uNumStripsForVisibility			= uNumStrips;
		s_uStripVisibilityQueryFrameCount	= 0;

		s_pbStripVisibleFromCurrentCameraThisFrame			= new bool [uNumStrips];
		s_pbStripVisibleFromCurrentCameraIgnoringDirectionThisFrame			= new bool [uNumStrips];
		s_pxStripVisbilityQueries	= new Asura_Static_Visibility_Query [uNumStrips];
	}

	// Do we need to check this frame?
	if(s_uStripVisibilityQueryFrameCount != uFrameNumber)
	{
		// Yes - update each strip in-turn...
		PrefetchStrip(0);
		for(u_int u = 0; u < s_uNumStripsForVisibility; u++)
		{
			PrefetchStrip(u + 1);

			const Asura_Bounding_Box&	xStripBB		= GetBoundingBox(u);
			const u_int					uModuleIndex	= GetModuleIndex(u);

			if(Asura_Visibility_System::IsModuleLinkedInFromCurrentCamera(uModuleIndex))
			{
				s_pbStripVisibleFromCurrentCameraIgnoringDirectionThisFrame[u]	= s_pxStripVisbilityQueries[u].IsAABBVisibleFromCurrentCameraIgnoringDirection(xStripBB);

				if(s_pbStripVisibleFromCurrentCameraIgnoringDirectionThisFrame[u])
				{
					s_pbStripVisibleFromCurrentCameraThisFrame[u]	=  s_pxStripVisbilityQueries[u].IsAABBVisibleFromCurrentCamera(xStripBB);
				}
				else
				{
					s_pbStripVisibleFromCurrentCameraThisFrame[u]	=  false;
				}
			}
			else
			{
				s_pbStripVisibleFromCurrentCameraIgnoringDirectionThisFrame[u]	= false;
				s_pbStripVisibleFromCurrentCameraThisFrame[u]					= false;
			}
		}

		s_uStripVisibilityQueryFrameCount	= uFrameNumber;
	}
}

#endif	//defined(ASURA_TARGET_SM3)


/// End of file. ///
