//
// Asura_Environment_Renderer.h
//
// Created by KJFL - 11 March 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_ENVIRONMENT_RENDERER_H__
#define __ASURA_ENVIRONMENT_RENDERER_H__

//////////////////////////////////////////////////////////////////////////
// ART - 19/11/08 - Temporarily changed #ifdef to SM3 platforms only.
// Technically there's no reason this class shouldn't exist for all 
// platforms but it does require the new visibility system (which is 
// currently only feasible for SM3 platforms).
//////////////////////////////////////////////////////////////////////////
#if defined(ASURA_TARGET_SM3)

#if !defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
#error "Asura_Environment_Renderer" needs "ASURA_USE_NEW_VISIBILITY_SYSTEM" defined.
#endif	//!defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)

class Asura_Bounding_Box;
class Asura_MaterialResponse;
class Asura_Static_Visibility_Query;

class Asura_Environment_Renderer
{
	public:
		static void Initialise(void);
		static void Reset(void);
		static void Shutdown(void);
		
		static u_int GetNumberOfStrips( void );

		static void	PrefetchStrip(u_int uStrip);

		static const Asura_Bounding_Box& GetBoundingBox( u_int uStrip );

		static const Asura_MaterialResponse* GetMaterial( u_int uStrip, u_int uPass );

		static u_int GetModuleIndex( u_int uStrip );

#if defined(ASURA_USE_TEXTURE_STREAMING)
		static void SetTexturePriorities( void );
		static void GetTexturesUsed( void ( *pfnCallback )( int iTextureID, int iTextureUsage  ) );
#endif	//defined(ASURA_USE_TEXTURE_STREAMING)

		static void UpdateVisibilityOfStrips(void);
		static bool IsStripVisibleFromCurrentCamera(const u_int uStripIndex)					{ return(s_pbStripVisibleFromCurrentCameraThisFrame[uStripIndex]); }
		static bool IsStripVisibleFromCurrentCameraIgnoringDirection(const u_int uStripIndex)	{ return(s_pbStripVisibleFromCurrentCameraIgnoringDirectionThisFrame[uStripIndex]); }

	private:
		// Private variables...
		static u_int							s_uStripVisibilityQueryFrameCount;
		static u_int							s_uNumStripsForVisibility;
		static bool*							s_pbStripVisibleFromCurrentCameraThisFrame;
		static bool*							s_pbStripVisibleFromCurrentCameraIgnoringDirectionThisFrame;
		static Asura_Static_Visibility_Query*	s_pxStripVisbilityQueries;
	
		static u_int Platform_GetNumberOfStrips( void );

		static void	Platform_PrefetchStrip(u_int uStrip);

		static const Asura_Bounding_Box& Platform_GetBoundingBox( u_int uStrip );

		static const Asura_MaterialResponse* Platform_GetMaterial( u_int uStrip, u_int uPass );

		static u_int Platform_GetModuleIndex( u_int uStrip );


};

#endif	//defined(ASURA_TARGET_SM3)

#endif//__ASURA_ENVIRONMENT_RENDERER_H__


/// End of file. ///
