
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Frustum.h
// Author : Andy Tate
// Date   : 12-Sep-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_FRUSTUM_H__
#define __ASURA_FRUSTUM_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Maths.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class Asura_Frustum
{
public:
	// Public enumerated types...

	enum Vertex
	{
		TOP_LEFT_NEAR,
		TOP_RIGHT_NEAR,
		BOTTOM_LEFT_NEAR,
		BOTTOM_RIGHT_NEAR,

		TOP_LEFT_FAR,
		TOP_RIGHT_FAR,
		BOTTOM_LEFT_FAR,
		BOTTOM_RIGHT_FAR,

		MAX_VERTICES
	};

	enum Edge
	{
		TOP_NEAR,
		BOTTOM_NEAR,
		LEFT_NEAR,
		RIGHT_NEAR,

		TOP_FAR,
		BOTTOM_FAR,
		LEFT_FAR,
		RIGHT_FAR,

		TOP_LEFT,
		BOTTOM_LEFT,
		TOP_RIGHT,
		BOTTOM_RIGHT,

		MAX_EDGES
	};

	enum Plane
	{
		NEAR_PLANE,
		FAR_PLANE,
		TOP_PLANE,
		BOTTOM_PLANE,
		LEFT_PLANE,
		RIGHT_PLANE,

		MAX_PLANES
	};


	// Public functions...

	Asura_Frustum  (void)	{}
	~Asura_Frustum ()		{}

	// Initialisation functions...
	void	SetForDistance	(const float fNearPlane, const float fFarPlane, const float fDistance, const float fWidth, const float fHeight);
	void	SetForFOV		(const float fNearPlane, const float fFarPlane, const float fTanHalfHorFOV, const float fTanHalfVerFOV);
	void	SetForSphere	(const float fNearPlane, const float fFarPlane, const float fDistance, const float fRadius);

	//void SetForNearPlane (const float fNearPlane, const float fFarPlane, const float fWidth, const float fHeight)
	//{
	//	SetForDistance	(fNearPlane, fFarPlane, fNearPlane, fWidth, fHeight);
	//}

	//void SetForFarPlane (const float fNearPlane, const float fFarPlane, const float fWidth, const float fHeight)
	//{
	//	SetForDistance	(fNearPlane, fFarPlane, fFarPlane, fWidth, fHeight);
	//}

	// Query functions...
	bool	Contains	(const Asura_Vector_3& xPosition) const;

	bool	Intersects	(const Asura_Vector_3& xPosition, const float fRadius) const;
	bool	Intersects	(const Asura_Frustum& xFrustum, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation) const;
	bool	Intersects	(const Asura_Frustum& xFrustum, const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation) const;

	// Vertex generation functions...
	void	GenerateVertices	(Asura_Vector_3* pxVertices) const;
	void	GenerateVertices	(Asura_Vector_3* pxVertices, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation) const;
	void	GenerateVertices	(Asura_Vector_3* pxVertices, const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation) const;

	// Accessor functions...
	float		GetNearPlane				(void) const	{ return (m_fNearPlane); }
	float		GetFarPlane					(void) const	{ return (m_fFarPlane); }

	float		GetTanHorizontalFOV			(void) const	{ return (m_fTanHalfHorFOV); }
	float		GetTanVerticalFOV			(void) const	{ return (m_fTanHalfVerFOV); }

	Asura_Angle	GetHorizontalFOVInDegrees	(void) const;
	Asura_Angle	GetVerticalFOVInDegrees		(void) const;

	// GetBoundingBox returns a conservative AABB around the frustum, given the orientation and position of the frustum "viewpoint".
	// Intended for visibility/intersection queries; might be worth storing inside the class itself, but currently class doesn't know
	// about it's position and orientation relative to the world etc.
	void	GetBoundingBox		(Asura_Bounding_Box& xBoundingBox, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xPosition) const;
	void	GetBoundingBox		(Asura_Bounding_Box& xBoundingBox, const Asura_Quat& xOrientation, const Asura_Vector_3& xPosition) const;

	float	GetBoundingSphere	(Asura_Vector_3& xCentre) const;
	float	GetBoundingSphere	(Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xPosition) const;
	float	GetBoundingSphere	(Asura_Vector_3& xCentre, const Asura_Quat& xOrientation, const Asura_Vector_3& xPosition) const;

	void	GetClosestVertex	(const Asura_Vector_3& xTo, Asura_Vector_3& xClosestVertex) const;


protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...

	// Distances to the near/far planes...
	float	m_fNearPlane;
	float	m_fFarPlane;

	// Frustum dimensions...
	float	m_fTanHalfHorFOV;
	float	m_fTanHalfVerFOV;

	// Private functions...
	Asura_Frustum  (const Asura_Frustum& xOther);

	Asura_Frustum&	operator=					(const Asura_Frustum& xOther);

};   // End of class 'Asura_Frustum'.


#endif   // ifndef __ASURA_FRUSTUM_H__


/// End of file. ///
