//
// Asura_Module.h
//
// Created by Kevin Lea - 12/06/2001 11:05:08
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_MODULE_H__
#define __ASURA_MODULE_H__

#include "Camera/Asura_Camera.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_Maths.h"
#include "Rendering/Asura_Module_Defines.h"


//////////////////////////
// Forward declarations //
//////////////////////////

#ifdef ASURA_TOOLS
class Asura_Mesh;
#endif

class Asura_CollisionMesh;
class Asura_Environment;
class Asura_LBAABB_Tree;
class Asura_LOS_InputData;
class Asura_LOS_OutputData;
class Asura_SaveBlock;

//////////////////
// Asura_Module //
//////////////////
class Asura_Module
{
public:

	Asura_Module();
	~Asura_Module();

#ifdef ASURA_TOOLS
	void SetModuleLink(int linkIndex,bool link);
	void ClearModuleLinks();
	void DeleteModule(int iModuleIndex);

	//updates m_pxCollisionMesh from Mesh
	void UpdateCollisionMesh();
#endif

#if defined(ASURA_TOOLS) && defined(ASURA_USE_STREAMING)
	//Fixes indices to deal with module and/or connections being added/deleted. RAR 11/04/2006
	void RemapModuleLinkConnectionIndices(u_int uArraySize,const int* piRemap);
	void RemapModuleLinkModuleIndices(u_int uConnectionIndex,u_int uArraySize,const int* piRemap);
#endif

	// Data
#ifdef ASURA_TOOLS
	Asura_Mesh* m_pxDefaultMesh;//The first mesh in chain of platform specific meshes , which gets used as the default.
	Asura_Mesh* Mesh;
	bool m_bCollisionMeshInvalid;
#endif
	Asura_CollisionMesh* m_pxCollisionMesh;

	int* VisibleModules;//for streaming this will contain a connection index (offset by 1) in the top 16 bits
	int NumberOfVisibleModules;
	int VisibleModulesArraySize;

	Asura_Vector_3 Centre;
	Asura_Vector_3 Position;   

};

//////////////////////////
// Asura_Module_Handler //
//////////////////////////
class Asura_Module_Handler
{
	// Friend classes...
	friend class Asura_Module_System;
	friend class Asura_Chunk_ModuleLinking;
	friend class Asura_Chunk_ModuleNames;
	friend class Asura_Chunk_Environment_ModuleList;
	friend class Asura_Chunk_EnvironmentLBAABBTrees;
	friend class Asura_Streaming_ModuleList_Base;
	friend class Tools_LOS;

public:
	// Public typedefs...
	typedef void ModuleTestCallbackFunction (const u_int uIndex);
    typedef void RenderFunction(const Asura_Module_Handler* pxHandler);

	// Public functions...
	Asura_Module_Handler  (void);
	~Asura_Module_Handler ();

	// Load/save functions...
			void AllocateModuleStorage( Asura_Environment& xEnvironment );
			void PreLoad(void);
			void Load(void);
			void Save( Asura_SaveBlock* pxSaveBlock );//(in Asura_Chunk_ModuleLinking)

			void Reset(void);

			void Update(const Asura_Vector_3* pxViewPosition,const int* piModuleIndex);
			void Render(void);
			void RenderLate(void);

			void Project_UpdateLinkedInListForPlayers (void);

#ifdef ASURA_TOOLS
	//add/remove a module link to a module
			void SetModuleLink(int moduleIndex,int linkIndex,Asura_Hash_ID uLinkZoneHash,bool link);
			void ClearModuleLinks(int iModuleIndex);
#endif
#if defined(ASURA_TOOLS) && defined(ASURA_USE_STREAMING)
			//Fixes indices to deal with module and/or connections being added/deleted. RAR 11/04/2006
			void RemapModuleLinkConnectionIndices(u_int uArraySize,const int* piRemap);
			void RemapModuleLinkModuleIndices(u_int uConnectionIndex,u_int uArraySize,const int* piRemap);
#endif
#if defined(ASURA_USE_STREAMING) || defined(ASURA_TOOLS)
			bool IsModuleLinked(int moduleIndex,int linkIndex,Asura_Hash_ID uLinkZoneHash);
#endif
			bool IsModuleLinked(int moduleIndex,int linkIndex);

#ifdef ASURA_TOOLS
			bool IsPointVisible( const Asura_Vector_3& alpha, const Asura_Vector_3& beta, bool (pfnPolygonFilter)(const Asura_Mesh* pxMesh,int iPolyIndex) = 0,bool b2SidedPolygons = false);
#endif

	/// Kevin Lea - 13/03/2002 15:59:23 - returns true if 'position' is inside a visible module
	/// (i.e. a module which is both linked in and in the view frustum)
	/// N.B. If lastKnownModule is non-zero, then that module will be checked first, and if the 
	/// point is found to be in another module, that index will be written back to *lastKnownModule
			bool IsPointInVisibleModule(const Asura_Vector_3& position, int* lastKnownModule=0);

	// (MH - 08.05.2002) addition for vis module culling of characters
			bool DoesBBIntersectVisibleModule( const Asura_Bounding_Box& xBB, int* piLastKnownModule = NULL );

	/// DWE - 23/01/2003 11:46:07 - Checks if a BB overlaps ANY vis module (i.e. All active cameras)
			bool DoesBBIntersectLinkedInModule( const Asura_Bounding_Box& xBB);
			bool DoesBBIntersectLinkedInModule( const Asura_Bounding_Box& xBB, int* piLastKnownModule);


	/// Kevin Lea - 02/10/2002 14:59:56 - warning - the distance parameter also sets the maximum distance to check, as well as providing
	/// a return mechanism for the distance to the poly (if the fn returns true).
			void FillModuleList( u_short* pusModuleList );

			void SetLengthOfVariableLengthCullingList( const u_short usListLength );
			bool FillVariableLengthModuleList( u_short* pusModuleList );

			// MGH 25/5/2006: Fill in the array starting at the specified position.
			// Return the number of modules added, or -1 if the list is invalid.
			int FillVariableLengthModuleListNoTerminator( u_short* pusModuleList);

#ifdef ASURA_TOOLS
			Asura_Mesh* GetMeshForModule(int moduleIndex) const;
#endif
			const Asura_CollisionMesh* GetCollisionMeshForModule(int iModuleIndex);

			int GetNumberOfModules(void) const					{ return m_iNumberOfModules; }

#ifdef _DEBUG
			bool HasNonSceneryModules() const;
#endif

			//Returns true if the box intersects any non-exterior (and non-scenery) module
			bool DoesBBIntersectNonExteriorModule(const Asura_Bounding_Box& xBB);

	// MH 5.4.02 -- returns index of first module found whose bounding box contains the point.
	// Returns -1 if point not found in any module.
	// (MH - 10.06.2002) added last_known_module_id, which is checked first, if not -1
			int GetModuleIndexFromPosition(const Asura_Vector_3& position, int last_known_module_id = -1, const bool bCheckScenery = false);

			int GetCameraModuleIndexFromPosition(const Asura_Vector_3& xPos, int iLastKnownModule);

			void GetPositionOfModule(int moduleIndex, Asura_Vector_3& position) const;
			const Asura_Vector_3& GetPositionOfModule( int iModuleIdx ) const	{ return m_pxModuleStorage[iModuleIdx].Position; }
			const Asura_Vector_3& GetCentreOfModule( int iModuleIndex ) const;
			float GetRadiusOfModule(int moduleIndex) const;
			const Asura_Bounding_Box& GetBoundingBoxOfModule( int iModuleIndex) const;
#ifdef ASURA_TOOLS
			u_int GetNumberOfLinkedInPolys( void ) const		{ return s_uNumberOfLinkedInPolys;     }
			u_int GetNumberOfLinkedInTriangles( void ) const	{ return s_uNumberOfLinkedInTriangles; }
			u_int GetNumLinkedInRenderMeshTris( void ) const	{ return s_uNumLinkedInRenderMeshTris; }
#endif

	inline bool IsModuleVisible( int moduleIndex ) const
	{
		return IsModuleVisible( moduleIndex, Asura_Camera::GetCameraIndexBitFlag() );
	}
	inline bool IsModuleVisible( int moduleIndex, u_int uCamMask ) const
	{
#ifdef ASURA_TOOLS
		// (MH - 15.02.2006) crash fix for Botanical mode.
		Asura_Assert(m_pucRenderList, "IsModuleVisible called but m_pucRenderList not allocated");
#endif //ASURA_TOOLS
		if (NULL == m_pucRenderList)
			return false;

		Asura_Assertf(moduleIndex>=0 && moduleIndex<m_iNumberOfModules,("Module number %d is out of bounds. %d Modules.", moduleIndex, m_iNumberOfModules));
		return( (m_pucRenderList[moduleIndex] & uCamMask) != 0 );
	}
	inline u_int GetModuleVisFlags( int iModule ) const
	{
		return m_pucRenderList[iModule];
	}
	inline u_int GetModuleLinkedFlags( int iModule ) const
	{
		return m_pucLinkedInList[iModule];
	}

			bool IsModuleInFrustum(int moduleIndex) const;
			bool IsModuleLinkedInForCurrentCamera(int moduleIndex) const;// (MH - 11.06.2002) uses new linked-in list
			bool IsModuleLinkedInForAnyActiveCamera(int moduleIndex) const;// DWE - similar to above but for all active cameras

	/// Kevin Lea - 24/06/2002 12:47:24 - access module data (not that you should normally need to do this)
			const Asura_Module* GetModuleData(int moduleIndex) const;
			Asura_Module* GetModuleData(int moduleIndex);

			void SetLOSProjectCallBack(ModuleTestCallbackFunction* pfnLOSProjectCallback);

#ifdef ASURA_TOOLS
			void SetupModuleMeshForFakeEnv( int iModuleIdx, Asura_Mesh* pxMesh );
			void SetupModulePosForFakeEnv( int iModuleIdx, const Asura_Vector_3& xPos );

	//Used to mark collision meshes that will need to be redone , because of changes
	//to the Asura_Mesh. RAR 19/03/03
			void InvalidateCollisionMesh(int iModuleIndex);
	//Recalculate all the collision meshes that have been flagged as invalid , and redo
	//the module tree
			void UpdateInvalidCollisionMeshes();

	//Deletes the module , and moves the last module into its place.
			void DeleteModule(int iModuleIndex);

			void InvalidateAllCollisionData(void);

			//Switches the specified module to use the mesh for a different platform. Returns true if this resulted in a different mesh being used.
			bool SwitchPlatformMesh(int iModuleIndex, u_int uPlatformBit);

#endif

	// LBAABB-Tree creation functions...
			bool				HasLBAABBModuleTree() const {return m_pxLBAABBModuleTree !=0;}
			Asura_LBAABB_Tree*	CreateLBAABBModuleTree					(void);
			u_int*				CreateModuleTreeLBAABBConversionArray	(const u_int uNumNodes);

	// this is a better solution in my book:-
			void TestModuleTree( const Asura_Bounding_Box& xBB, ModuleTestCallbackFunction* pfnOutputCallback );

#if defined(ASURA_USE_STREAMING)
			void FindConnectedHandlers();
			Asura_Hash_ID GetHandlerHashID() const		   { return m_uHandlerHash; }
#endif

	// Module culling functions...
			void SetUseModLinks(const bool bSet)		   { m_bUseModLinks = bSet; }
			bool IsUseModLinks(void) const				   { return(m_bUseModLinks); }

			void SetUseFrustumCulling(const bool bSet)	   { m_bUseFrustumCulling = bSet; }
			bool IsUseFrustumCulling(void) const		   { return(m_bUseFrustumCulling); }

			void SetUseCameraPositionOverride( bool bUseOverride, const Asura_Vector_3* pxOverride=NULL );
			bool IsUsingCameraPositionOverride(void) const { return m_bUseCameraPositionOverride; }

			// KOR - use this to specify a project render function. See Asura_Module_Handler::Render definition.
			static  void SetRenderFunction(RenderFunction* pfnFunction) { s_pfnRenderFunction = pfnFunction; }

			// Kevin Lea - 08/10/2001 16:50:44 - platform specific wrappers for Render call
			void Platform_BeginRender(void);
			void Platform_Render();
			void Platform_EndRender(void);

		#ifdef ASURA_TARGET_SM3
			// DMR 17-05-07 Added this one for rendering water/translucency after the deferred lights
			void Platform_RenderLate();					
		#endif

			bool IsExteriorModule(u_int uModuleIndex) const;

		#ifdef ASURA_TOOLS
			// return the frame stamp of when the RenderList last changed.
			u_int GetRenderListChangedFrameStamp() const { return m_uRenderListChangedFrameStamp; }
		#endif

		#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM
			u_char* GetRenderList( void )
			{
				return m_pucRenderList;
			}
		#endif

private:
	//Initialises m_pxLBAABBModuleTree - RAR 20/03/03
			void InitMainModuleTree();
	// (MH - 19.06.2002) moved this from Asura_Dynamics_System
			void InitAllModuleTrees(void);
	// RJF - 04 October 2004 - Refactored InitAllModuleTrees out so that individual modules can be re-treed if necessary
			void InitTreeForSingleModule(int moduleIndex);



			void ClearRenderAndLinkedInListsForCurCam( void );
			void AddAllModulesWithoutModLinkingToRenderListForCurCam();
			void AddAllModulesToRenderListForCurCam();
			void AddAllModulesToLinkedInListForCurCam();
			void CullRenderListForCurCam();
#ifdef ASURA_TOOLS
			void CheckIfRenderListChanged();
#endif

	// Private variables...
			Asura_Module*				m_pxModuleStorage; 

			Asura_LBAABB_Tree*			m_pxLBAABBModuleTree;
			u_int*						m_puModuleTreeLBAABBConversionArray;

			int							m_iNumberOfModules;
			unsigned char*				m_pucRenderList;
	// (MH - 11.06.2002) added linked-in list this so we have the results 
	// without frustum culling so sounds can be occluded based on visible module.
			unsigned char*				m_pucLinkedInList; 

#ifdef ASURA_TOOLS
			unsigned char*				m_pucPrevRenderList; 
			u_int						m_uRenderListChangedFrameStamp;
#endif

			bool						m_bModuleStorageAllocated;
			bool						m_bModuleLinkingLoaded;
			bool						m_bInitialised;
			bool						m_bUseModLinks;			// Local version of this flag - used by streaming.
			bool						m_bUseFrustumCulling;
			bool						m_bUseCameraPositionOverride;

			Asura_Vector_3				m_xCameraPositionOverride;

#ifdef ASURA_USE_STREAMING
			//Array of handlers of connected zones , that is a parallel of
			//the connection list in Asura_Streaming_Zone.
			//Note that some of the handlers may be zero , if that zone isn't loaded
			//RAR 06/04/2006
			u_int						m_uNumConnectedHandlers;
			Asura_Module_Handler**		m_ppxConnectedHandlers;

			Asura_Hash_ID				m_uZoneHash;
			Asura_Hash_ID				m_uHandlerHash;
#endif //ASURA_USE_STREAMING

			//Pointer to env stored , since it is needed a fair bit
			Asura_Environment*			m_pxEnv;

	// Handler used for callback functions...
	static	Asura_Module_Handler*		s_pxCallbackHandler;

#if defined(ASURA_TOOLS)
	static	u_int						s_uNumberOfLinkedInPolys;
	static	u_int						s_uNumberOfLinkedInTriangles;
	static	u_int						s_uNumLinkedInRenderMeshTris;
	static	bool						s_bCollisionMeshesInvalid;	//flag to indicate that one or more collision mesh needs recalculation.
#endif	// defined(ASURA_TOOLS)

	// (MH - 19.06.2002) stuff to remember LOS state between callbacks from AABBTree system        
	// cur module data
	static	int							s_iLastModuleIndex;
	static	ModuleTestCallbackFunction*	s_pModuleTestCallbackFunction;
	static	Asura_Bounding_Box			s_xModuleTestBB;


	// Module culling test data.
	static	unsigned short				s_ausModuleList[NUM_MODULES_TO_STORE_FOR_CLIPPING];
	static	unsigned short				s_usNumModulesInList;

	static	unsigned short				s_ausVariableLengthModuleList[ MAX_VARIABLE_LENGTH_CLIPPING_COUNT ];
	static	unsigned short				s_usNumModulesInVariableList;
	static	unsigned short				s_usMaxNumModulesInVariableList;

	static	int							s_iVisibleModule;

	static	int							s_iLinkedInModule;

	static	bool						s_bInNonExteriorModule; //For result from texting whether a point/bb is in a non-exterior module

    static  RenderFunction*     s_pfnRenderFunction;

	// MH 5.4.02 -- functions called by Initialise / Shutdown to setup platform specific stuff.
			void Platform_Initialise(void);
			void Platform_Shutdown(void);

	// Callback functions...
			void BuildLinkedInListCallback(const u_int uIndex);
			void AddLinksFromModule(const u_int uIndex);
			void CullingListModuleCallback( const u_int uModuleIdx );
			void CullingTestModuleCallback( const u_int uModuleIdx );
			void LinkedInForActiveCamerasCallback(const u_int uModuleIndex);	/// DWE - 23/01/2003 11:47:14 - Callback for DoesBBIntersectLinkedInModule
			void ModuleIndexFromPositionCallback( const u_int uIndex );
			void ModuleIndexIncludingSceneryFromPositionCallback( const u_int uIndex );
			void CameraModuleIndexFromPositionCallback( const u_int uIndex );
			void ModuleTestCallback(const u_int uIndex);	// (AW - 18.08.2005) - Stuff to convert module index during BB tests.
			void VariableLengthCullingListCallback( const u_int uModuleIdx );
			void NonExteriorModuleCallback( const u_int uModuleIdx );

	// Callback functions...
	static	void BuildLinkedInListInternalCallback(const u_int uIndex) { s_pxCallbackHandler->BuildLinkedInListCallback(uIndex); }
	static	void BuildRenderListInternalCallback(const u_int uIndex) { s_pxCallbackHandler->AddLinksFromModule(uIndex); }
	static	void CullingListModuleInternalCallback( const u_int uModuleIdx ) { s_pxCallbackHandler->CullingListModuleCallback(uModuleIdx); }
	static	void CullingTestModuleInternalCallback( const u_int uModuleIdx ) { s_pxCallbackHandler->CullingTestModuleCallback(uModuleIdx); }
	static	void LinkedInForActiveCamerasInternalCallback(const u_int uModuleIndex) { s_pxCallbackHandler->LinkedInForActiveCamerasCallback(uModuleIndex); }
	static	void ModuleIndexFromPositionInternalCallback( const u_int uIndex ) { s_pxCallbackHandler->ModuleIndexFromPositionCallback(uIndex); }
	static	void ModuleIndexIncludingSceneryFromPositionInternalCallback( const u_int uIndex ) { s_pxCallbackHandler->ModuleIndexIncludingSceneryFromPositionCallback(uIndex); }
	static	void CameraModuleIndexFromPositionInternalCallback( const u_int uIndex ) { s_pxCallbackHandler->CameraModuleIndexFromPositionCallback(uIndex); }
	static	void ModuleTestInternalCallback(const u_int uIndex) { s_pxCallbackHandler->ModuleTestCallback(uIndex); }
	static	void VariableLengthCullingListInternalCallback( const u_int uModuleIdx ) { s_pxCallbackHandler->VariableLengthCullingListCallback(uModuleIdx); }
	static	void NonExteriorModuleInternalCallback( const u_int uModuleIdx ) { s_pxCallbackHandler->NonExteriorModuleCallback(uModuleIdx);}
};



/////////////////////////
// Asura_Module_System //
/////////////////////////
class Asura_Module_System
{
	// Friend classes...
	friend class Asura_Streaming_ModuleList_Base;

public:
	// Public variables...
	static bool UseModLinks;

	// Public functions...
	static	void	Initialise		(void);
	static	void	Shutdown		(void);
	static	void	Reset			(void);

	static	bool	IsInitialised	(void)									{ return(GetCurrentHandler().GetNumberOfModules() > 0); }

	static	void	Load			(void)									{ GetCurrentHandler().Load(); }
	static	void	Save			( Asura_SaveBlock* pxSaveBlock )		{ GetCurrentHandler().Save( pxSaveBlock ); }

	//Standard game module linking update... updates visibilities based on the position of the camera
	static	void	Update			(void)									{ Update(0,0); }
	//Updates visibilities based on the view position passed
	static	void	Update			(const Asura_Vector_3* pxViewPosition)	{ Update(pxViewPosition,0); }
	//Takes visibilities from a single passed module
	static	void	Update			(const int* piModuleIndex)				{ Update(0,piModuleIndex); }
	//Main update function , called by the previous 3 update functions
	static	void	Update			(const Asura_Vector_3* pxViewPosition,const int* piModuleIndex);
	static	void	Render			(void);
	static	void	RenderLate		(void);

#ifdef ASURA_TOOLS
	//add/remove a module link to a module
	static	void	SetModuleLink	(int moduleIndex,int linkIndex,Asura_Hash_ID uLinkZoneHash,bool link)	{ GetCurrentHandler().SetModuleLink(moduleIndex, linkIndex, uLinkZoneHash, link); }
	static	void	ClearModuleLinks(int iModuleIndex) { GetCurrentHandler().ClearModuleLinks(iModuleIndex); }
#endif
#if defined(ASURA_USE_STREAMING) || defined(ASURA_TOOLS)
	static	bool	IsModuleLinked	(int moduleIndex,int linkIndex, Asura_Hash_ID uLinkZoneHash)				{ return(GetCurrentHandler().IsModuleLinked(moduleIndex, linkIndex, uLinkZoneHash)); }
#endif
	static	bool	IsModuleLinked	(int moduleIndex,int linkIndex)				{ return(GetCurrentHandler().IsModuleLinked(moduleIndex, linkIndex)); }

#ifdef ASURA_TOOLS
	static	bool	IsPointVisible	(const Asura_Vector_3& alpha, const Asura_Vector_3& beta, bool (pfnPolygonFilter)(const Asura_Mesh* pxMesh,int iPolyIndex) = 0,bool b2SidedPolygons = false)	{ return(GetCurrentHandler().IsPointVisible(alpha, beta, pfnPolygonFilter, b2SidedPolygons)); }
#endif

	/// Kevin Lea - 13/03/2002 15:59:23 - returns true if 'position' is inside a visible module
	/// (i.e. a module which is both linked in and in the view frustum)
	/// N.B. If lastKnownModule is non-zero, then that module will be checked first, and if the 
	/// point is found to be in another module, that index will be written back to *lastKnownModule
	static bool IsPointInVisibleModule(const Asura_Vector_3& position, int* lastKnownModule=0)				{ return(GetCurrentHandler().IsPointInVisibleModule(position, lastKnownModule)); }

	// (MH - 08.05.2002) addition for vis module culling of characters
	static bool DoesBBIntersectVisibleModule(const Asura_Bounding_Box& xBB, int* piLastKnownModule = NULL)	{ return(GetCurrentHandler().DoesBBIntersectVisibleModule(xBB, piLastKnownModule)); }

#if defined(ASURA_USE_STREAMING)
	// MGH 26/5/2006: When the zone is unknown or may change then this is a better bet than other versions of this function.
	static bool DoesBBIntersectVisibleModule(const Asura_Bounding_Box& xBB, int* piLastKnownModule, Asura_Hash_ID* puLastKnownZone);
#endif //defined(ASURA_USE_STREAMING)

	/// DWE - 23/01/2003 11:46:07 - Checks if a BB overlaps ANY vis module (i.e. All active cameras)
	static bool DoesBBIntersectLinkedInModule(const Asura_Bounding_Box& xBB)								{ return(GetCurrentHandler().DoesBBIntersectLinkedInModule(xBB)); }
	static bool DoesBBIntersectLinkedInModule(const Asura_Bounding_Box& xBB, int* piLastKnownModule)		{ return(GetCurrentHandler().DoesBBIntersectLinkedInModule(xBB, piLastKnownModule)); }

	/// Kevin Lea - 02/10/2002 14:59:56 - warning - the distance parameter also sets the maximum distance to check, as well as providing
	/// a return mechanism for the distance to the poly (if the fn returns true).
	static void FillModuleList( u_short* pusModuleList )													{ GetCurrentHandler().FillModuleList(pusModuleList); }

	static void SetLengthOfVariableLengthCullingList( const u_short usListLength )								{ GetCurrentHandler().SetLengthOfVariableLengthCullingList(usListLength); }
	static bool FillVariableLengthModuleList( u_short* pusModuleList )											{ return(GetCurrentHandler().FillVariableLengthModuleList(pusModuleList)); }

#ifdef ASURA_TOOLS
	static	Asura_Mesh*					GetMeshForModule			(int moduleIndex)	{ return(GetCurrentHandler().GetMeshForModule(moduleIndex)); }
#endif
	static	const Asura_CollisionMesh*	GetCollisionMeshForModule	(int iModuleIndex)	{ return(GetCurrentHandler().GetCollisionMeshForModule(iModuleIndex)); }
	static	int							GetNumberOfModules			(void)				{ return(GetCurrentHandler().GetNumberOfModules()); }

	// MH 5.4.02 -- returns index of first module found whose bounding box contains the point.
	// Returns -1 if point not found in any module.
	// (MH - 10.06.2002) added last_known_module_id, which is checked first, if not -1
	static int GetModuleIndexFromPosition(const Asura_Vector_3& position, int last_known_module_id = -1, const bool bCheckScenery = false)	{ return(GetCurrentHandler().GetModuleIndexFromPosition(position, last_known_module_id, bCheckScenery)); }
	static int GetCameraModuleIndexFromPosition(const Asura_Vector_3& xPos, int iLastKnownModule = -1)	{ return(GetCurrentHandler().GetCameraModuleIndexFromPosition(xPos, iLastKnownModule)); }

	static void GetPositionOfModule(int moduleIndex, Asura_Vector_3& position)								{ return(GetCurrentHandler().GetPositionOfModule(moduleIndex, position)); }
	static const Asura_Vector_3& GetPositionOfModule( int iModuleIdx )										{ return(GetCurrentHandler().GetPositionOfModule(iModuleIdx)); }
	static const Asura_Vector_3& GetCentreOfModule( int iModuleIndex )										{ return(GetCurrentHandler().GetCentreOfModule(iModuleIndex)); }
	static float GetRadiusOfModule(int moduleIndex)															{ return(GetCurrentHandler().GetRadiusOfModule(moduleIndex)); }
	static const Asura_Bounding_Box& GetBoundingBoxOfModule( int iModuleIndex)								{ return(GetCurrentHandler().GetBoundingBoxOfModule(iModuleIndex)); }
#ifdef ASURA_TOOLS
	static u_int GetNumberOfLinkedInPolys( void )		{ return(GetCurrentHandler().GetNumberOfLinkedInPolys()); }
	static u_int GetNumberOfLinkedInTriangles( void )	{ return(GetCurrentHandler().GetNumberOfLinkedInTriangles()); }
	static u_int GetNumLinkedInRenderMeshTris( void )	{ return(GetCurrentHandler().GetNumLinkedInRenderMeshTris()); }

	static void SetRenderAllWhenInNoModule(bool bFlag)	{s_bRenderAllWhenInNoModule = bFlag;}
	static bool GetRenderAllWhenInNoModule()			{return s_bRenderAllWhenInNoModule;}
#endif

	static inline bool IsModuleVisible( int moduleIndex )					{ return(GetCurrentHandler().IsModuleVisible(moduleIndex)); }
	static inline bool IsModuleVisible( int moduleIndex, u_int uCamMask )	{ return(GetCurrentHandler().IsModuleVisible(moduleIndex, uCamMask)); }
	static inline u_int GetModuleVisFlags( int iModule )					{ return(GetCurrentHandler().GetModuleVisFlags(iModule)); }
	static inline u_int GetModuleLinkedFlags( int iModule )					{ return(GetCurrentHandler().GetModuleLinkedFlags(iModule)); }

	static bool IsModuleInFrustum(int moduleIndex)							{ return(GetCurrentHandler().IsModuleInFrustum(moduleIndex)); }
	static bool IsModuleLinkedInForCurrentCamera(int moduleIndex)			{ return(GetCurrentHandler().IsModuleLinkedInForCurrentCamera(moduleIndex)); }
	static bool IsModuleLinkedInForAnyActiveCamera(int moduleIndex)			{ return(GetCurrentHandler().IsModuleLinkedInForAnyActiveCamera(moduleIndex)); }		

	// this is a better solution in my book:-
	static void TestModuleTree( const Asura_Bounding_Box& xBB, Asura_Module_Handler::ModuleTestCallbackFunction* pfnOutputCallback ) { GetCurrentHandler().TestModuleTree(xBB, pfnOutputCallback); }


	/// Kevin Lea - 24/06/2002 12:47:24 - access module data (not that you should normally need to do this)
	static const Asura_Module* GetModuleData(int moduleIndex)							{ return(GetCurrentHandler().GetModuleData(moduleIndex)); }

#ifdef ASURA_TOOLS
	//Used to mark collision meshes that will need to be redone , because of changes
	//to the Asura_Mesh. RAR 19/03/03
	static void InvalidateCollisionMesh(int iModuleIndex)								{ GetCurrentHandler().InvalidateCollisionMesh(iModuleIndex); }
	//Recalculate all the collision meshes that have been flagged as invalid , and redo
	//the module tree
	static void UpdateInvalidCollisionMeshes()											{ GetCurrentHandler().UpdateInvalidCollisionMeshes(); }

	//Deletes the module , and moves the last module into its place.
	static void DeleteModule(int iModuleIndex)											{ GetCurrentHandler().DeleteModule(iModuleIndex); }

	static void InvalidateAllCollisionData(void)										{ GetCurrentHandler().InvalidateAllCollisionData(); }

	//Switches the specified module to use the mesh for a different platform. Returns true if this resulted in a different mesh being used.
	static bool SwitchPlatformMesh(int iModuleIndex, u_int uPlatformBit)					{ return GetCurrentHandler().SwitchPlatformMesh(iModuleIndex, uPlatformBit);}

#endif

	// Callback functions...
	static 	void 					BuildLinkedInListCallback			(const u_int uIndex)	{ GetCurrentHandler().BuildLinkedInListCallback(uIndex); }
	static 	void 					BuildRenderListCallback				(const u_int uIndex)	{ GetCurrentHandler().AddLinksFromModule(uIndex); }
	static 	void 					CullingListModuleCallback			(const u_int uIndex)	{ GetCurrentHandler().CullingListModuleCallback(uIndex); }
	static 	void 					CullingTestModuleCallback			(const u_int uIndex)	{ GetCurrentHandler().CullingTestModuleCallback(uIndex); }
	static 	void 					LinkedInForActiveCamerasCallback	(const u_int uIndex)	{ GetCurrentHandler().LinkedInForActiveCamerasCallback(uIndex); }
	static 	void 					ModuleIndexFromPositionCallback		(const u_int uIndex)	{ GetCurrentHandler().ModuleIndexFromPositionCallback(uIndex); }
	static 	void 					ModuleTestCallback					(const u_int uIndex)	{ GetCurrentHandler().ModuleTestCallback(uIndex); }
	static	void					VariableLengthCullingListCallback	(const u_int uIndex)	{ GetCurrentHandler().VariableLengthCullingListCallback(uIndex); }

#if defined(ASURA_USE_STREAMING)
	// Handler functions...
	static	Asura_Module_Handler*	CreateHandler						(const Asura_Hash_ID uHandlerHash, const Asura_Hash_ID uZoneHash);
	static	bool					DestroyHandler						(const Asura_Hash_ID uHandlerHash);

	static	void					CreateAndSetHandler					(const Asura_Hash_ID uHandlerHash, const Asura_Hash_ID uZoneHash)	{ s_pxCurrentHandler = CreateHandler(uHandlerHash, uZoneHash); }
	static	void					SetCurrentHandler					(Asura_Module_Handler* pxHandler)									{ s_pxCurrentHandler = pxHandler; }
	static	void					SetCurrentHandler					(const Asura_Hash_ID uHandlerHash)									{ s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHandlerHash); }
	static	Asura_Module_Handler&	GetCurrentHandler					(void)																{ return(*s_pxCurrentHandler); }
	static	Asura_Module_Handler*	GetHandler							(const Asura_Hash_ID uHandlerHash)									{ return(s_xHandlerTree.GetDataByHash(uHandlerHash)); }
#else
	// Private functions...
	static	Asura_Module_Handler&	GetCurrentHandler					(void)	{ return(s_xHandler); }
#endif	// defined(ASURA_USE_STREAMING)

#if defined(ASURA_USE_STREAMING)
	//Used internally to update each handlers list of connected handlers
	static void FindConnectedHandlers();
#endif

	static void ClearRenderAndLinkedInListsForCurCam();

private:
	static void AddAllModulesWithoutModLinkingToRenderListForCurCam();
	static void AddAllModulesToRenderListForCurCam();
	static void AddAllModulesToLinkedInListForCurCam();
	static void CullRenderListForCurCam();

	#ifdef ASURA_USE_STREAMING
	static bool UpdateCallback(Asura_Hash_ID uHandlerHash,Asura_Module_Handler* pxHandler,void* pPosition);
	static void FindConnectedHandlersCallback(Asura_Hash_ID uHandlerHash,Asura_Module_Handler* pxHandler);
	static void ClearRenderAndLinkedInListsForCurCamCallback(Asura_Hash_ID uHandlerHash,Asura_Module_Handler* pxHandler);
	static void AddAllModulesWithoutModLinkingToRenderListForCurCamCallback(Asura_Hash_ID uHandlerHash,Asura_Module_Handler* pxHandler);
	static void AddAllModulesToRenderListForCurCamCallback(Asura_Hash_ID uHandlerHash,Asura_Module_Handler* pxHandler);
	static void AddAllModulesToLinkedInListForCurCamCallback(Asura_Hash_ID uHandlerHash,Asura_Module_Handler* pxHandler);
	static void CullRenderListForCurCamCallback(Asura_Hash_ID uHandlerHash,Asura_Module_Handler* pxHandler);
	static bool DoesBBIntersectVisibleModule_Callback(Asura_Hash_ID uHandlerHash, Asura_Module_Handler* pxHandler, void* pUserData);
	static bool DoesBBIntersectNonExteriorModule_Callback(Asura_Hash_ID uHandlerHash, Asura_Module_Handler* pxHandler, void* pUserData);
	#endif

#if defined(ASURA_USE_STREAMING)
	// Private variables...
	static	Asura_Module_Handler*							s_pxCurrentHandler;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_Module_Handler>	s_xHandlerTree;			// Tree of all of the handlers available.
#else
	// Private variables...
	static	Asura_Module_Handler							s_xHandler;				// Single static-instance of a zone.
#endif

#if defined(ASURA_TOOLS)
	static	bool											s_bRenderAllWhenInNoModule;
#endif	// defined(ASURA_TOOLS)
};

#endif//__ASURA_MODULE_H__
