//
// Asura_Overlay_Quad.cpp
//
// Created by Andrew Weinkove - 03/06/04
// 
// Asura_Overlay_Quad class moved out of Asura_Render into its own file.
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Rendering/Asura_Overlay_Quad.h"

#include "Core/Asura_Maths.h"


#if defined(ASURA_TOOLS)
const Asura_Bounding_Rectangle*	Asura_Overlay_Quad::s_pxToolsDrawArea	= NULL;
#endif	// defined(ASURA_TOOLS)


////////////////////////
// Asura_Overlay_Quad //
////////////////////////
void Asura_Overlay_Quad::SetRectUV(float tlu, float tlv, float du, float dv)
{
	U[0] = tlu;
	V[0] = tlv;

	U[1] = tlu+du;
	V[1] = tlv;

	U[2] = tlu+du;
	V[2] = tlv+dv;

	U[3] = tlu;
	V[3] = tlv+dv;

}


void Asura_Overlay_Quad::SetRectXY(float tlx, float tly, float dx, float dy)
{
	X[0] = tlx;
	Y[0] = tly;

	X[1] = tlx+dx;
	Y[1] = tly;

	X[2] = tlx+dx;
	Y[2] = tly+dy;

	X[3] = tlx;
	Y[3] = tly+dy;
    
    m_bNonRotated = true;
}


// Makes axis-aligned overlay clip to axis-aligned box.
// Assumes uv coordinates have already been set.
// Valid area is a closed interval (inclusive)
void Asura_Overlay_Quad::SetClippedRectXY(float fX, float fY, float fWidth, float fHeight, float fMinX, float fMinY, float fMaxX, float fMaxY)
{
	//Trivial Accept
	if(fX >= fMinX && fY >= fMinY && fX+fWidth <= fMaxX && fY+fHeight <= fMaxY)
	{
		SetRectXY(fX, fY, fWidth, fHeight);
	}

	//Variables to store clipped dimensions
	float fXNew, fYNew, fWidthNew, fHeightNew;

	//Initialise to unclipped dimensions
	fXNew = fX;
	fYNew = fY;
	fWidthNew = fWidth;
	fHeightNew = fHeight;

	//Clip against each bound.
	//Lower x bound
	if(fXNew < fMinX)
	{
		fXNew = fMinX;
		fWidthNew = fWidth + (fX - fXNew);
	}

	//Lower y bound
	if(fYNew < fMinY)
	{
		fYNew = fMinY;
		fHeightNew = fHeight + (fY - fYNew);
	}

	//Upper x bound
	if(fXNew + fWidthNew > fMaxX)
	{
		fWidthNew = fMaxX - fXNew;
	}

	//Upper y bound
	if(fYNew + fHeightNew > fMaxY)
	{
		fHeightNew = fMaxY - fYNew;
	}

	//Early reject
	if(fWidthNew <= 0.f || fHeightNew <= 0.f)
	{
		for(int ctr = 0; ctr < 4; ctr++)
		{
			U[ctr] = V[ctr] = X[ctr] = Y[ctr] = 0.f;
		}
		
		return;
	}

	//Retrieve full uv information from internal storage.
	float fU, fV, fUWidth, fVWidth;
		
	fU = U[0];
	fUWidth = U[1] - U[0];

	fV = V[0];
	fVWidth = V[3] - V[0];

	//Work out mapping between xy coordinates and uv coordinates
	float fUXGradient, fVYGradient, fUXOrigin, fVYOrigin;

	//Mapping partial derivatives.
	fUXGradient = fUWidth / fWidth;
	fVYGradient = fVWidth / fHeight;
	
	//Avoid zero gradients with a delta.
	if(fUXGradient == 0.f) fUXGradient = 0.000000001f;
	if(fVYGradient == 0.f) fVYGradient = 0.000000001f;

	//Mapping origin.
	fUXOrigin = fX - fU/fUXGradient;
	fVYOrigin = fY - fV/fVYGradient;

	//Clip UVs according to xy -> uv mapping
	fUWidth = fWidthNew * fUXGradient;
	fVWidth = fHeightNew * fVYGradient;
	fU = (fXNew - fUXOrigin) * fUXGradient;
	fV = (fYNew - fVYOrigin) * fVYGradient;

	//Reset with new UVs.
	SetRectUV(fU, fV, fUWidth, fVWidth);

	//Set new XYs
	SetRectXY(fXNew, fYNew, fWidthNew, fHeightNew);
}


/// Richard May - 19/11/2002 17:14:16 - helper function to allow you to create a rotated overlay
/// fX and fY are the centre coordinates (rotation is around centre of overlay)
void Asura_Overlay_Quad::SetRotatedRectXY( float fX, float fY, float fWidth, float fHeight, float fTheta )
{	
	float fHalfWidth  = fWidth * 0.5f;
	float fHalfHeight = fHeight * 0.5f;
	float fSineTheta = Asura_Maths::Sine(fTheta);
	float fCosTheta  = Asura_Maths::Cosine(fTheta);

	SetRectXY( -fHalfWidth, -fHalfHeight, fWidth, fHeight );

	for( u_int u=0; u<4; ++u )
	{
   		float fNewX = fX + (fCosTheta * X[u]) - (fSineTheta * Y[u]);
		float fNewY = fY + (fCosTheta * Y[u]) + (fSineTheta * X[u]);

		X[u] = fNewX;
		Y[u] = fNewY;		
 	}	

    m_bNonRotated = false;
}

void Asura_Overlay_Quad::SetGouraudColour(u_int uGouraudColourTL, 
                                          u_int uGouraudColourTR, 
                                          u_int uGouraudColourBL, 
                                          u_int uGouraudColourBR)
{
    m_bGouraudShaded = true;
    GouraudColour[0] = uGouraudColourTL;
    GouraudColour[1] = uGouraudColourTR;
    GouraudColour[2] = uGouraudColourBR;
    GouraudColour[3] = uGouraudColourBL;
}

void Asura_Overlay_Quad::SetGouraudColour(Colour_Components xGouraudColourTL, 
                                          Colour_Components xGouraudColourTR, 
                                          Colour_Components xGouraudColourBL, 
                                          Colour_Components xGouraudColourBR)
{
    m_bGouraudShaded = true;
    GouraudColourComponents[0] = xGouraudColourTL;
    GouraudColourComponents[1] = xGouraudColourTR;
    GouraudColourComponents[2] = xGouraudColourBR;
    GouraudColourComponents[3] = xGouraudColourBL;
}

void Asura_Overlay_Quad::Render(bool bResetGouraudAndRotated/* = true*/)
{
#if defined(ASURA_TOOLS)
	if((s_pxToolsDrawArea != NULL) && m_bNonRotated)
	{
		const float	fLeft	= X[0];
		const float	fTop	= Y[0];
		const float	fWidth	= X[1] - X[0];
		const float	fHeight	= Y[2] - Y[0];

		SetClippedRectXY(fLeft, fTop, fWidth, fHeight, s_pxToolsDrawArea->MinX, s_pxToolsDrawArea->MinY, s_pxToolsDrawArea->MaxX, s_pxToolsDrawArea->MaxY);
	}
#endif	// defined(ASURA_TOOLS)

	Platform_Render();
    if (bResetGouraudAndRotated)
    {
        m_bGouraudShaded = false;
        m_bNonRotated = false;

		// (AAM - 26.03.2007) Similarly, reset the texture op, since old code won't be setting this
		m_eTextureOp = ASURA_TEXTUREOP_SINGLE_PASS;
    }
}

