//
// Asura_PatchSubdivision.cpp
//
// Created by Richard Rice - 09/09/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_TOOLS

#include "Rendering/Asura_PatchSubdivision.h"

#include "AI/Asura_AI_Navigation.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_Maths.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_Actor.h"
#include "Cutscene/Asura_Cutscene_Track.h"
#include "Debugging/Asura_Assert.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Render.h"


#define PATCHSUBDIVISION_MAX_POINTS 10000


Asura_PatchSubdivision::SubdivideMode Asura_PatchSubdivision::s_eSubdivideMode = Asura_PatchSubdivision::SUBDIVIDE_CAMERA; 
bool Asura_PatchSubdivision::s_bSubdivideFlat = false;
bool Asura_PatchSubdivision::s_bAvoidTriangleSlivers = true;

int Asura_PatchSubdivision::s_iNumberOfPoints = 0;
Asura_Vector_3* Asura_PatchSubdivision::s_pxPoints = 0;

int Asura_PatchSubdivision::s_iNumberOfPointsForModule = 0;
void** Asura_PatchSubdivision::s_ppPointsForModule = 0;

float Asura_PatchSubdivision::s_fFlatSubdivideSize = ASURA_PATCH_DEFAULT_SUBDIVIDE_SIZE*ASURA_PATCH_DEFAULT_SUBDIVIDE_SIZE;
float Asura_PatchSubdivision::s_fCurveSubdivideSize = ASURA_PATCH_DEFAULT_SUBDIVIDE_SIZE*ASURA_PATCH_DEFAULT_SUBDIVIDE_SIZE;

float Asura_PatchSubdivision::s_fPointSearchInitialDistance = 100.0f;
float Asura_PatchSubdivision::s_fPointSearchMinimumDistance = 5.0f;
float Asura_PatchSubdivision::s_fDistancePower = 1.0f;

void Asura_PatchSubdivision::SubdivideEvenly()
{
	s_eSubdivideMode = SUBDIVIDE_EVENLY;
	Asura_Render::SetStaticSubdivision(true);
	
	DeallocateArrays();
}


void Asura_PatchSubdivision::SubdivideNearCamera()
{
	s_eSubdivideMode = SUBDIVIDE_CAMERA;
	Asura_Render::SetStaticSubdivision(false);
	
	DeallocateArrays();
}


void Asura_PatchSubdivision::SubdivideNearWaypoints()
{
	s_eSubdivideMode = SUBDIVIDE_WAYPOINT;
	Asura_Render::SetStaticSubdivision(true);
	
	// no longer caching points for this subdivison mode
	//InitialiseArrays();
	DeallocateArrays();
}

void Asura_PatchSubdivision::SubdivideNearTracks()
{
	s_eSubdivideMode = SUBDIVIDE_TRACK;
	Asura_Render::SetStaticSubdivision(true);
	
	InitialiseArrays();

	for(int i=0;i<Asura_Cutscene_System::GetNumberOfCutscenes();i++)
	{
		Asura_Cutscene* pxCutscene = Asura_Cutscene_System::GetCutsceneByIndex(i);

		//search for the camera track in this cutscene
		Asura_Cutscene_Actor* pxCameraActor = 0;
		for(int j=0;j<pxCutscene->NumberOfActors;j++)
		{
			//if(pxCutscene->Actors[j]->Type == ACAT_Camera)
            if(pxCutscene->Actors[j]->IsCamera())
			{
				pxCameraActor = pxCutscene->Actors[j];
				break;
			}
		}
		if(!pxCameraActor) continue;

		for(int j=0;j<pxCameraActor->NumberOfTracks;j++)
		{
			Asura_Cutscene_Track* pxTrack = pxCameraActor->Tracks[j];

			//get one point every metre or so along the track
			float fLength = pxTrack->GetLength();
			int iNumPoints = Asura_Maths::FloatToInt(fLength)+1;
			
			if(iNumPoints+s_iNumberOfPoints>=PATCHSUBDIVISION_MAX_POINTS)
			{
				Asura_Assert(iNumPoints+s_iNumberOfPoints<PATCHSUBDIVISION_MAX_POINTS,"");
				return;
			}

			for(int k=0;k<iNumPoints;k++)
			{
				float fDist = (fLength*k)/iNumPoints;
				pxTrack->GetPositionFromDistance(fDist,&s_pxPoints[s_iNumberOfPoints++]);
			}
		}
	}	

}

void Asura_PatchSubdivision::InitialiseArrays()
{
	s_iNumberOfPoints = 0;
	s_iNumberOfPointsForModule = 0;

	if(!s_pxPoints) s_pxPoints = new Asura_Vector_3[PATCHSUBDIVISION_MAX_POINTS];
	if(!s_ppPointsForModule) s_ppPointsForModule = new void*[PATCHSUBDIVISION_MAX_POINTS];
}

void Asura_PatchSubdivision::DeallocateArrays()
{
	if(s_pxPoints) delete [] s_pxPoints;
	if(s_ppPointsForModule) delete [] s_ppPointsForModule; 

	s_iNumberOfPoints = 0;
	s_iNumberOfPointsForModule = 0;
	s_pxPoints = 0;
	s_ppPointsForModule = 0;
}

float Asura_PatchSubdivision::GetDistanceSquared(const Asura_Vector_3& xPos,int iMaterialID)
{
	const float fMinDistance = s_fPointSearchMinimumDistance*s_fPointSearchMinimumDistance;//minimum distance allowed to reduce the amount of subdivision very near the waypoints
	float fLowDistance = s_fPointSearchInitialDistance*s_fPointSearchInitialDistance;
	
	if(s_eSubdivideMode == SUBDIVIDE_CAMERA)
	{
		Asura_Vector_3 xOffset = xPos - *Asura_Camera::GetPosition();
		return xOffset*xOffset;	
	}
	else if(s_eSubdivideMode == SUBDIVIDE_TRACK)
	{
		
		Asura_Vector_3** ppxPointsForModule = (Asura_Vector_3**)s_ppPointsForModule;
	
		for(int i=0;i<s_iNumberOfPointsForModule;i++)
		{
			Asura_Vector_3 xOffset = xPos - *ppxPointsForModule[i];
			float fDistance = xOffset*xOffset;
			if(fDistance<fLowDistance)
			{
				fLowDistance = fDistance;
				if(fLowDistance<fMinDistance)
				{
					fLowDistance = fMinDistance;
					break;
				}
			}
		}
	}
	else
	{
		Asura_Assert(s_eSubdivideMode == SUBDIVIDE_WAYPOINT,"");

		/*Asura_AI_NavVertex** ppxWaypointsForModule = (Asura_AI_NavVertex**)s_ppPointsForModule;

		for(int i=0;i<s_iNumberOfPointsForModule;i++)
		{
			Asura_Vector_3 xOffset = xPos - ppxWaypointsForModule[i]->m_xPosition;

			//work out distance from the waypoint , considering it to be a horizontal disc
			float fVDistSquared = xOffset.y*xOffset.y;

			float fHDistSquared = xOffset.x*xOffset.x + xOffset.z*xOffset.z;
			fHDistSquared -= (ppxWaypointsForModule[i]->m_fRadius*ppxWaypointsForModule[i]->m_fRadius);
			if(fHDistSquared<0) fHDistSquared = 0;

			float fDistance = fHDistSquared+fVDistSquared;

			if(fDistance<fLowDistance)
			{
				fLowDistance = fDistance;
				if(fLowDistance<fMinDistance)
				{
					fLowDistance = fMinDistance;
					break;
				}
			}
		}*/
		if (Asura_AI_Navigation_System::GetNumberOfWaypoints())
		{
			unsigned short usWaypointID = static_cast<u_short>(Asura_AI_Navigation_System::FindWaypointNearestTo(xPos));
			Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint(usWaypointID);

			if(pxWP)
			{
				Asura_Vector_3 xOffset = xPos - pxWP->m_xPosition;

				//work out distance from the waypoint , considering it to be a horizontal disc
				float fVDistSquared = xOffset.y*xOffset.y;
				float fHDistSquared = xOffset.x*xOffset.x + xOffset.z*xOffset.z;
				fHDistSquared -= (pxWP->GetRadius()*pxWP->GetRadius());
				if(fHDistSquared<0) fHDistSquared = 0;
				fLowDistance = fHDistSquared+fVDistSquared;

				if(fLowDistance<fMinDistance) fLowDistance = fMinDistance;
			}
		}	
	}
	if(Asura_Material_System::IsMaterialSubdividePolygon(iMaterialID))
	{
		fLowDistance = fMinDistance;
	}

	return fLowDistance;
}


void Asura_PatchSubdivision::FindPointsNearModule(Asura_Mesh* pxMesh,const Asura_Vector_3& xPos)
{
	s_iNumberOfPointsForModule = 0;
	if(s_eSubdivideMode == SUBDIVIDE_WAYPOINT)
	{
		// we're no longer caching waypoints - we're using the AABB nav tree per-vertex
		//FindWaypointsNearModule(pxMesh,xPos);
	}
	else if(s_eSubdivideMode == SUBDIVIDE_TRACK)
	{
		FindTrackPointsNearModule(pxMesh,xPos);
	}
}

void Asura_PatchSubdivision::FindTrackPointsNearModule(Asura_Mesh* pxMesh,const Asura_Vector_3& xPos)
{
	Asura_Assert(pxMesh,"");
	s_iNumberOfPointsForModule = 0;

	if(!s_iNumberOfPoints) return;

	if(!pxMesh->BoundingBoxDataValid) pxMesh->CalculateBoundingBox();
	Asura_Bounding_Box xBox = pxMesh->BoundingBox;
	xBox.Displace(xPos);

	//expand the bounding box a bit , so as to get points that are near the module
	xBox.MinX -= 10.0f;
	xBox.MinY -= 10.0f;
	xBox.MinZ -= 10.0f;
	xBox.MaxX += 10.0f;
	xBox.MaxY += 10.0f;
	xBox.MaxZ += 10.0f;

	for(int i=0;i<s_iNumberOfPoints;i++)
	{
		if(xBox.Contains(s_pxPoints[i]))
		{
			s_ppPointsForModule[s_iNumberOfPointsForModule++] = &s_pxPoints[i];
		}
	}

	if(s_iNumberOfPointsForModule) return;
	#if 1
	//no points , look a bit further afield
	xBox.MinX -= 40.0f;
	xBox.MinY -= 40.0f;
	xBox.MinZ -= 40.0f;
	xBox.MaxX += 40.0f;
	xBox.MaxY += 40.0f;
	xBox.MaxZ += 40.0f;

	for(int i=0;i<s_iNumberOfPoints;i++)
	{
		if(xBox.Contains(s_pxPoints[i]))
		{
			s_ppPointsForModule[s_iNumberOfPointsForModule++] = &s_pxPoints[i];
		}
	}
	#endif
}

void Asura_PatchSubdivision::FindWaypointsNearModule(Asura_Mesh* pxMesh,const Asura_Vector_3& xPos)
{
	/*Asura_Assert(pxMesh,"");
	s_iNumberOfPointsForModule = 0;

	if(!pxMesh->BoundingBoxDataValid) pxMesh->CalculateBoundingBox();
	Asura_Bounding_Box xBox = pxMesh->BoundingBox;
	xBox.Displace(xPos);

	// RJF 12-04-05: We were seeing problems with subdivision at module boundaries because adjacent
	// modules were getting different waypoint lists; this is to be expected, but is a problem when boundary
	// edges are being subdivided according to a waypoint that is not in both modules
	// The new approach searches for waypoints both inside the module and outside, and collects
	// results from a box expanded by [nearest waypoint outside module distance] on each side
	// this guarantees at least one result (assuming the level has at least one waypoint).

	float fNearestOutsideDistanceSq = -1;

	// run through the waypoint list, collecting points inside the module as well as the nearest point outside the box
	for(int i=0; i < Asura_AI_Navigation_System::GetNumberOfWaypoints(); i++)
	{
		if(s_iNumberOfPointsForModule == PATCHSUBDIVISION_MAX_POINTS) return;

		// is the point inside the module?
		Asura_AI_NavVertex* pxVertex = Asura_AI_Navigation_System::GetWaypoint(i);
		if(xBox.IntersectsSphere(pxVertex->m_xPosition,pxVertex->m_fRadius))
		{
			s_ppPointsForModule[s_iNumberOfPointsForModule++] = pxVertex;
		}
		else
		{
			float fDistanceSq = (pxVertex->m_xPosition - xPos).MagnitudeSquared();
			if((fDistanceSq < fNearestOutsideDistanceSq) || (fNearestOutsideDistanceSq < 0))
			{
				fNearestOutsideDistanceSq = fDistanceSq;
			}
		}
	}

	// nearest outside distance will be -1 still if (a) all waypoints were inside module or (b) no waypoints
	if(fNearestOutsideDistanceSq > 0)
	{

		// Expand the box on each side
		float fNearestDistance = Asura_Maths::Sqrt(fNearestOutsideDistanceSq) + 10.f;
		xBox.MinX -= fNearestDistance;
		xBox.MinY -= fNearestDistance;
		xBox.MinZ -= fNearestDistance;
		xBox.MaxX += fNearestDistance;
		xBox.MaxY += fNearestDistance;
		xBox.MaxZ += fNearestDistance;

		// search again

		// this is actually going to cause most waypoints inside the module to be collected twice,
		// but that's what the original code did.

		for(int i=0;i<Asura_AI_Navigation_System::GetNumberOfWaypoints();i++)
		{
			if(s_iNumberOfPointsForModule==PATCHSUBDIVISION_MAX_POINTS) return;

			Asura_AI_NavVertex* pxVertex = Asura_AI_Navigation_System::GetWaypoint(i);
			if(xBox.IntersectsSphere(pxVertex->m_xPosition,pxVertex->m_fRadius))
			{
				s_ppPointsForModule[s_iNumberOfPointsForModule++] = pxVertex;
			}
		}
	}*/

	Asura_Assert(false, "No longer implemented");
}

#endif // #ifdef ASURA_TOOLS
