//
// Asura_Render.cpp
//
// Created by Kevin Lea - 12/06/2001 11:05:14
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Rendering/Asura_Render.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Core/Asura_TextureManagement.h"
#ifdef ASURA_USE_DEBUG_TEXTURE_VIEWER
#include "Debugging/Asura_Debug_Texture_Viewer.h"
#endif
#include "Debugging/Asura_DebugText.h"
#include "Debugging/Asura_Debug_Timers.h"
#if !defined(ASURA_NO_DEBUG_OUTPUT)
#include "Debugging/Asura_DebugLineRender.h"
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)
#if defined( ASURA_USE_DEBUGOVERLAY )
#include "Debugging/Asura_DebugOverlay.h"
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )
#if defined( ASURA_USE_PRIMITIVE_RENDER )
#include "Rendering/Asura_Primitive_Render.h"
#endif // #if defined( ASURA_USE_PRIMITIVE_RENDER )
#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "GUIMenu/Asura_GUIMenu_System.h"
#endif // ASURA_USE_GUI_MENU_SYSTEM
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Interface/Asura_CommandConsole.h"
#endif
#if defined(ASURA_TARGET_SM3)
#include "Rendering/Asura_Environment_Renderer.h"
#endif	//defined(ASURA_TARGET_SM3)
#include "Rendering/Asura_Viewport.h"
#include "Sound/Asura_Sound_Mixer.h"
#include "Core/Asura_MemTracker.h"

Asura_Overlay_Quad Asura_Render::Overlay;
Asura_Render_States Asura_Render::CurrentStates;

bool  Asura_Render::s_bCutsceneHasCameraControl = false;
bool  Asura_Render::s_bPatchesOn = false;
int   Asura_Render::s_iPatchesSubdivLevel = 3;
bool  Asura_Render::s_bStaticSubdivision = true;
enum  ASURA_RENDERSTYLE_ID Asura_Render::s_eRenderStyle = ASURA_RENDERSTYLE_NORMAL;
bool  Asura_Render::s_bRenderMirrored = false;
bool  Asura_Render::s_bRenderForWidescreen = false;
#if defined ASURA_RENDER_TTYPES
int   Asura_Render::s_aiNumberOfTrianglesRendered[ASURA_RENDER_TTYPE_COUNT];
#else
int   Asura_Render::s_iNumberOfTrianglesRendered = 0;
#endif
int   Asura_Render::s_iNumTriPassesRendered = 0;
int   Asura_Render::s_iNumberOfPatchesRendered = 0;
bool  Asura_Render::s_bCurrentlyRendering = false;
bool  Asura_Render::s_bWireframe = false;
u_int Asura_Render::s_uNumParticlesRendered = 0;
u_int Asura_Render::s_uNumSkinHierarchiesOutOfView = 0;
u_int Asura_Render::s_uNumSmoothSkinsRendered = 0;
u_int Asura_Render::s_uNumHierarchiesRendered = 0;
u_int Asura_Render::s_uNumShapesRendered = 0;
bool  Asura_Render::s_bEnableBatchedLineRendering = true;
Asura_Bounding_Rectangle Asura_Render::s_xSafeArea(0.f, 0.f, 1.f, 1.f);

#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
bool  Asura_Render::s_bEnableHiResScreenshotMode = false;
bool  Asura_Render::s_bDisableMipMaps = false;
#endif	//defined(ASURA_USE_SCREENSHOT_SYSTEM)

#ifndef ASURA_TARGET_SM3
int Asura_Render::s_iReflectanceMapTextureIndex = ASURA_TEXTURE_NONE;
Asura_String Asura_Render::s_xReflectanceMapTextureName;
#endif //#ifndef ASURA_TARGET_SM3

Asura_Vector_2	Asura_Render::s_xSubPixelOffset (0.0f, 0.0f);


bool					 Asura_Render::s_abSystemEnabled[ASURA_RENDER_NUM_SYSTEMS];

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
const Asura_UnicodeChar* Asura_Render::s_aszSystemEnableDVSName[ASURA_RENDER_NUM_SYSTEMS]=
{
	L"Render.Enable.Skybox",				// ASURA_RENDER_SYSTEM_SKYBOX,
	L"Render.Enable.Env",					// ASURA_RENDER_SYSTEM_ENV,
	L"Render.Enable.Objects",				// ASURA_RENDER_SYSTEM_SHAPES,
	L"Render.Enable.Botanicals",			// ASURA_RENDER_SYSTEM_BOTANICALS,
	L"Render.Enable.Carpet",				// ASURA_RENDER_SYSTEM_CARPET,
	L"Render.Enable.Water",					// ASURA_RENDER_SYSTEM_WATER,
	L"Render.Enable.Shadows",				// ASURA_RENDER_SYSTEM_SHADOWS,
	L"Render.Enable.Beams",					// ASURA_RENDER_SYSTEM_BEAMS,
	L"Render.Enable.Decals",				// ASURA_RENDER_SYSTEM_DECALS,
	L"Render.Enable.Photons",				// ASURA_RENDER_SYSTEM_PHOTONS,
	L"Render.Enable.Projectors",			// ASURA_RENDER_SYSTEM_PROJECTORS,
	L"Render.Enable.PFX",					// ASURA_RENDER_SYSTEM_PFX,	
	L"Render.Enable.Coronas",				// ASURA_RENDER_SYSTEM_CORONAS,
	L"Render.Enable.LightShafts",			// ASURA_RENDER_SYSTEM_LIGHTSHAFTS,
	L"Render.Enable.Fog",					// ASURA_RENDER_SYSTEM_FOG,
	L"Render.Enable.FSFX",					// ASURA_RENDER_SYSTEM_FSFX,	
	L"Render.Enable.HUD",					// ASURA_RENDER_SYSTEM_HUD,
	L"Render.Enable.ClearScreen",			// ASURA_RENDER_SYSTEM_CLEARSCREEN,
	L"Render.Enable.CSM",					// ASURA_RENDER_SYSTEM_CSM,
};

static bool g_bTestScreenBorder = false;
#endif

#ifdef ASURA_USE_ZPREPASS
bool Asura_Render::s_bIsZprepass = false;
#endif

#ifdef ASURA_ASSERT
bool Asura_Render::s_bInProtectedRenderCode = false;
#endif

#ifdef ASURA_USE_HDR
bool Asura_Render::m_bIsCurrentlyHDR = false;
#endif

//////////////////
// Asura_Render //
//////////////////
void Asura_Render::Initialise(void)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Render::Initialise");
	
	for(int i=0;i<ASURA_RENDER_NUM_SYSTEMS;i++)
	{	
		s_abSystemEnabled[i] = true;
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
		Asura_CommandConsole::AddVar(s_aszSystemEnableDVSName[i], &s_abSystemEnabled[i], Asura_CommandConsole::REMOVE_AT_SHUTDOWN );	
#endif
	}

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	#if defined(ASURA_USE_DEBUG_TEXTURE_VIEWER)
		Asura_Debug_Texture_Viewer::AddConsoleCommands();
	#endif

	Asura_CommandConsole::AddVar(L"Render.Wireframe", &s_bWireframe, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar(L"Render.Test.Borders", &g_bTestScreenBorder, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
#endif
	
	s_bCutsceneHasCameraControl = false;
	Platform_Initialise();

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	Asura_DebugLineRender::Initialise();
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)
#if defined(ASURA_TARGET_SM3)
	Asura_Environment_Renderer::Initialise();
#endif	//defined(ASURA_TARGET_SM3)
}

void Asura_Render::EarlyShutdown (void)
{
#if defined(ASURA_TARGET_SM3)
	Asura_Environment_Renderer::Shutdown();
#endif	//defined(ASURA_TARGET_SM3)
#if !defined(ASURA_NO_DEBUG_OUTPUT)
	Asura_DebugLineRender::Shutdown();
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)

	Platform_EarlyShutdown();
}

void Asura_Render::LateShutdown (void)
{
	Platform_LateShutdown();
}

void Asura_Render::Reset(void)
{
	Platform_Reset();

#ifndef ASURA_TARGET_SM3
	s_iReflectanceMapTextureIndex = ASURA_TEXTURE_NONE;
	s_xReflectanceMapTextureName.MakeNull();
#endif //#ifndef ASURA_TARGET_SM3

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	Asura_DebugLineRender::Reset();
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)
#if defined(ASURA_TARGET_SM3)
	Asura_Environment_Renderer::Reset();
#endif	//defined(ASURA_TARGET_SM3)
}

void Asura_Render::BeginFrame(void)
{
	Asura_Assert( !s_bCurrentlyRendering, "Mis-matched BeginFrame() / EndFrame() calls?" );

#ifdef ASURA_ASSERT
	SetInProtectedRenderCode(true);
#endif

#if defined ASURA_RENDER_TTYPES
	for (int i = 0; i < ASURA_RENDER_TTYPE_COUNT; i++)
		s_aiNumberOfTrianglesRendered[i] = 0;
#else
	s_iNumberOfTrianglesRendered = 0;
#endif
	s_iNumTriPassesRendered = 0;
	s_iNumberOfPatchesRendered = 0;
	s_uNumParticlesRendered = 0;
	s_uNumSkinHierarchiesOutOfView = 0;
	s_uNumSmoothSkinsRendered = 0;
	s_uNumHierarchiesRendered = 0;
	s_uNumShapesRendered = 0;
	s_bCurrentlyRendering = true;

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	Asura_DebugLineRender::Update();
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)
#if defined( ASURA_USE_DEBUGOVERLAY )
	Asura_DebugOverlay::Update();
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )
#if defined( ASURA_USE_PRIMITIVE_RENDER )
	Asura_Primitive_Render::Update();
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )

#ifdef ASURA_USE_HDR
	m_bIsCurrentlyHDR = true;
#endif

	Platform_BeginFrame();

#ifdef ASURA_ASSERT
	SetInProtectedRenderCode(false);
#endif

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	if (g_bTestScreenBorder) Asura_Render::ActivateScreenBorder(Asura_Render::SCREEN_BORDER_STRETCH_TO_FIT, Asura_Viewport::GetWidth()/2, Asura_Viewport::GetHeight(), 0xff8000);
#endif
}

void Asura_Render::EndFrame(void)
{
	Asura_Assert( s_bCurrentlyRendering, "Mis-matched BeginFrame() / EndFrame() calls?" );

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	if (g_bTestScreenBorder) Asura_Render::DeactivateScreenBorder();
#endif

#ifdef ASURA_ASSERT
	SetInProtectedRenderCode(true);
#endif

#ifdef ASURA_USE_DEBUG_TEXT
	Asura_DebugText::RenderAll();
#endif
	// JAT 21/10/07 - do this before the debug line render because DebugOverlay system uses DebugLineRender system to draw some of its primitives
#if defined( ASURA_USE_DEBUGOVERLAY )
	Asura_DebugOverlay::Render();
#endif // #if defined( ASURA_USE_DEBUGOVERLAY )
#if defined( ASURA_USE_PRIMITIVE_RENDER )
	Asura_Primitive_Render::Render();
#endif // #if defined( ASURA_USE_PRIMITIVE_RENDER )
#if !defined(ASURA_NO_DEBUG_OUTPUT)
	Asura_DebugLineRender::Render();
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)

#ifdef ASURA_USE_DEBUG_TEXTURE_VIEWER
	Asura_Debug_Texture_Viewer::Render();
#endif

#ifdef ASURA_SOUND_MIXER_RENDER_DEBUG
	Asura_Sound_Mixer_System::DebuggingRender();
#endif

#ifdef ASURA_ENABLE_MEMTRACKER
	Asura_MemTracker::Render ();
#endif

	Platform_EndFrame();
	
#ifdef ASURA_ASSERT
	SetInProtectedRenderCode(false);
#endif

	s_bCurrentlyRendering = false;
}

void Asura_Render::RenderLine(const Asura_Vector_3& xPoint1, const Asura_Vector_3& xPoint2, unsigned int uColour)
{
	CheckInFrame();
	Platform_RenderLine(xPoint1, xPoint2, uColour);
}

void Asura_Render::RenderLine(const Asura_AVector3& xPoint1, const Asura_AVector3& xPoint2, unsigned int uColour)
{
	CheckInFrame();
	Platform_RenderLine(Asura_Vector_3(xPoint1), Asura_Vector_3(xPoint2), uColour);
}

void Asura_Render::Render2DLine( const Asura_Vector_2& xP0, const Asura_Vector_2& xP1, u_int uColour)
{
	CheckInFrame();
    Platform_Render2DLine( xP0, xP1, uColour );
}

void Asura_Render::Render2DLine( float fX0, float fY0, float fX1, float fY1, u_int uColour )
{
	CheckInFrame();
	Platform_Render2DLine( Asura_Vector_2(fX0, fY0), Asura_Vector_2(fX1, fY1), uColour);
}

void Asura_Render::RenderBB( const Asura_Bounding_Box& xBB, const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xOri, const u_int uARGB )
{
	CheckInFrame();
    // Disregard degenerate BBs.
    if( xBB.MaxX <= xBB.MinX || xBB.MaxY <= xBB.MinY || xBB.MaxZ <= xBB.MinZ ) { return; }

    // Cull against frustum(s).
    {
        bool bVis = false;

        Asura_Vector_3 xCentre = xBB.CalculateCentre();
        xCentre += xPos;

        float fRadius = (Asura_Vector_3(xBB.MaxX, xBB.MaxY, xBB.MaxZ)).Magnitude();

        for( int i = 0; i < Asura_Camera::GetNumberOfCameras(); ++i )
        {
            Asura_Camera::SetCameraIndex( i );
            if( Asura_Camera::IsCameraActive() && Asura_Camera::IsSphereInFrustum(xCentre, fRadius) )
            {
                bVis = true;
                break;
            }
        }

        if( !bVis ) { return; }
    }

    Asura_Vector_3 axVertices[8];
    for( u_int uVert = 0; uVert < 8; ++uVert )
    {
        axVertices[uVert].x = ( uVert & 1 ? xBB.MaxX : xBB.MinX );
        axVertices[uVert].y = ( uVert & 2 ? xBB.MaxY : xBB.MinY );
        axVertices[uVert].z = ( uVert & 4 ? xBB.MaxZ : xBB.MinZ );

        axVertices[uVert] = axVertices[uVert] * xOri + xPos;
    }

    const u_short ausIndices[24] =
    {
        0, 2, 2, 3, 3, 1, 1, 0,
        0, 4, 2, 6, 3, 7, 1, 5,
        4, 6, 6, 7, 7, 5, 5, 4,
    };
    
    for( u_int uEdge = 0; uEdge < 12; ++uEdge )
    {
        u_int uIdx = uEdge << 1;
        Asura_Render::RenderLine( axVertices[ausIndices[uIdx]], axVertices[ausIndices[uIdx + 1]], uARGB );
    }
}


// JAT 13/02/07 - helper function for drawing a axis set (with standard colours)
void Asura_Render::RenderAxes( const Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOrient, const float fSize, const float fAlpha )
{
	CheckInFrame();
	// show some coloured lines to represent the axes
	const u_int	uAlpha = static_cast<u_int>( Asura_Maths::FloatToInt( 255.0f * fAlpha ) ) << 24;
	Asura_Vector_3 xEnd;

	xEnd = xCentre;
	xEnd.x += xOrient.e11 * fSize;
	xEnd.y += xOrient.e21 * fSize;
	xEnd.z += xOrient.e31 * fSize;
	Platform_RenderLine( xCentre, xEnd, uAlpha | 0xFF0000 ); // X:red

	xEnd = xCentre;
	xEnd.x += xOrient.e12 * fSize;
	xEnd.y += xOrient.e22 * fSize;
	xEnd.z += xOrient.e32 * fSize;
	Platform_RenderLine( xCentre, xEnd, uAlpha | 0x00FF00 ); //Y:green

	xEnd = xCentre;
	xEnd.x += xOrient.e13 * fSize;
	xEnd.y += xOrient.e23 * fSize;
	xEnd.z += xOrient.e33 * fSize;
	Platform_RenderLine( xCentre, xEnd, uAlpha | 0x0000FF ); //Z:blue
}

// Aligned vector & matrix version 
void Asura_Render::RenderAxes( const Asura_AVector3& xCentre, const Asura_AMatrix3x3& xOrient, const float fSize, const float fAlpha )
{
	CheckInFrame();
	// show some coloured lines to represent the axes
	const u_int	uAlpha = static_cast<u_int>( Asura_Maths::FloatToInt( 255.0f * fAlpha ) ) << 24;
	Asura_AVector3 xEnd;

	xEnd = xCentre;
	xEnd += xOrient.GetColumn0() * fSize;
	Platform_RenderLine( Asura_Vector_3( xCentre ), Asura_Vector_3( xEnd ), uAlpha | 0xFF0000 ); // X:red

	xEnd = xCentre;
	xEnd += xOrient.GetColumn1() * fSize;
	Platform_RenderLine( Asura_Vector_3( xCentre ), Asura_Vector_3( xEnd ), uAlpha | 0x00FF00 ); // Y:green

	xEnd = xCentre;
	xEnd += xOrient.GetColumn2() * fSize;
	Platform_RenderLine( Asura_Vector_3( xCentre ), Asura_Vector_3( xEnd ), uAlpha | 0x0000FF ); // Z:blue
}

#if _DEBUG
void Asura_Render::CheckInFrame( void )
{
	Asura_Assert( s_bCurrentlyRendering, "Something rendered outside of BeginFrame() -> EndFrame()" );
}
#endif

#ifndef ASURA_TARGET_SM3

int Asura_Render::GetReflectanceMap()
{
	if(s_iReflectanceMapTextureIndex == ASURA_TEXTURE_NONE)
	{
		s_iReflectanceMapTextureIndex = Asura_Texture_Management::AddTexture(GetReflectanceMapName());
	}
	return s_iReflectanceMapTextureIndex;
}

const char* Asura_Render::GetReflectanceMapName()
{
	if(s_xReflectanceMapTextureName.IsBlank())
	{
		//return the default texture then
		return "\\specialfx\\spheremap1.bmp";
	}
	return s_xReflectanceMapTextureName.GetString();
}

void Asura_Render::SetReflectanceMapName(const char* pszName)
{
	s_xReflectanceMapTextureName	= pszName;
#if defined(ASURA_IGNORE_REFLECTION_MAP_TEXTURES)
	s_iReflectanceMapTextureIndex	= ASURA_TEXTURE_NONE;
#else
	s_iReflectanceMapTextureIndex	= Asura_Texture_Management::AddTexture(GetReflectanceMapName());
#endif	// defined(ASURA_IGNORE_REFLECTION_MAP_TEXTURES)
}

#elif defined(ASURA_TOOLS) //#ifndef ASURA_TARGET_SM3

// Tools needs to have this function in order to create the render settings chunk.
const char* Asura_Render::GetReflectanceMapName()
{
	return "";
}

#endif //#elif defined (ASURA_TOOLS) //#ifndef ASURA_TARGET_SM3

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render::ActivateScreenBorder
// Author       : Andy Tate
// Date         : 25-Jul-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Render::ActivateScreenBorder( const ScreenBorderType eType,
										 const u_int uWidth,
										 const u_int uHeight,
										 const u_int uColour)
{
	u_int uActualWidth  = uWidth;
	u_int uActualHeight = uHeight;
	u_int uBorderColour = uColour;

	if (g_bTestScreenBorder && eType != SCREEN_BORDER_DISABLED)
	{
		// overide it with test settings
		uActualWidth = Asura_Viewport::GetWidth()/2;
		uActualHeight = Asura_Viewport::GetHeight();
		uBorderColour = 0xff8000;
	}

	const u_int uScreenWidth	= Asura_Viewport::GetWidth();
	const u_int uScreenHeight	= Asura_Viewport::GetHeight();
	const float	fScreenWidth	= static_cast<float>(uScreenWidth);
	const float	fScreenHeight	= static_cast<float>(uScreenHeight);

	float		fAspectRatio;

	// Is the border enabled?
	if(eType == SCREEN_BORDER_DISABLED)
	{
		const Asura_ViewingArea	xViewArea(0.0f, 0.0f, 1.0f, 1.0f);

		// Update the viewing area...
		Asura_Camera::SetViewingArea(xViewArea);
		fAspectRatio	= fScreenWidth / fScreenHeight;

		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		// and update the desired menu sizes for resized menus. 
		Asura_GUIMenu_System::SetAutoMenuSizes( fScreenWidth, fScreenHeight );
		Asura_GUIMenu_System::SetAutoMenuOffsets( 0.0f, 0.0f );
		#endif //ASURA_USE_GUI_MENU_SYSTEM
	}
	else
	{
		const float	fActualWidth	= static_cast<float>(uActualWidth);
		const float	fActualHeight	= static_cast<float>(uActualHeight);

		float 		fWidthOffset;
		float 		fHeightOffset;
		float 		fWidth;
		float 		fHeight;

		// Render the border quad first...
		Asura_Render::Overlay.Colour		= uBorderColour;
		Asura_Render::Overlay.TextureIndex	= ASURA_TEXTURE_NONE;
		Asura_Render::Overlay.SetRectUV(0.0f, 0.0f, 1.0f, 1.0f);
		Asura_Render::Overlay.SetRectXY(0.0f, 0.0f, static_cast<float>(Asura_Viewport::GetWidth()), static_cast<float>(Asura_Viewport::GetHeight()));
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
		Asura_Render::Overlay.Render();
		Asura_Render::FlushDisplay();

		// What sort of screen do we need?
		switch(eType)
		{
			case SCREEN_BORDER_ACTUAL_SIZE:
			{
				fWidthOffset	= static_cast<float>((uScreenWidth - uActualWidth) >> 1) / fScreenWidth;
				fHeightOffset	= static_cast<float>((uScreenHeight - uActualHeight) >> 1) / fScreenHeight;
				fWidth			= 1.0f - (fWidthOffset * 2.0f);
				fHeight			= 1.0f - (fHeightOffset * 2.0f);
				break;
			}

			case SCREEN_BORDER_STRETCH_TO_FIT:
			{
				fHeight	= (fActualHeight / fScreenHeight) * (fScreenWidth / fActualWidth);

				if(fHeight > 1.0f)
				{
					fWidth	= 1.0f / fHeight;
					fHeight	= 1.0f;
				}
				else
				{
					fWidth = 1.0f;
				}

				fWidthOffset	= (1.0f - fWidth) * 0.5f;
				fHeightOffset	= (1.0f - fHeight) * 0.5f;
				break;
			}

			default:
			{
				Asura_Assert(false, "Unexpected screen-border type.");
				return;
			}
		}		// End of switch.

		// Update the viewing-area...
		Asura_ViewingArea xViewArea(fWidthOffset, fHeightOffset, fWidth, fHeight);
		Asura_Camera::SetViewingArea(xViewArea);
		fAspectRatio = fActualWidth / fActualHeight;

		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		// and update the desired menu sizes for resized menus. 
		fWidth *= Asura_Viewport::GetWidth();
		fHeight *= Asura_Viewport::GetHeight();
		Asura_GUIMenu_System::SetAutoMenuSizes( fWidth, fHeight );
		Asura_GUIMenu_System::SetAutoMenuOffsets( static_cast<float>( Asura_Viewport::GetWidth() ) * fWidthOffset, static_cast<float>( Asura_Viewport::GetHeight() ) * fHeightOffset );
		#endif // ASURA_USE_GUI_MENU_SYSTEM
	}	

	// Finally, update the camera settings...
	Asura_Camera::SetPhysicalScreenAspectRatio(fAspectRatio);
	Asura_Camera::SetHorizontalFieldOfView(Asura_Camera::GetHorizontalFieldOfView());
	Asura_Render::ApplyCameraSettings();
}

/// End of function 'Asura_Render::ActivateScreenBorder'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render::DeactivateScreenBorder
// Author       : Andy Tate
// Date         : 25-Jul-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Render::DeactivateScreenBorder(void)
{
	ActivateScreenBorder(SCREEN_BORDER_DISABLED,0,0,0);
}

/// End of function 'Asura_Render::DeactivateScreenBorder'. ///


/// End of switch. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Render::SetVideoMode
// Author       : Steve Keen 
// Date         : 12-Oct-2007
//
// Note			: This has been changed to notify the GUIMenu system if the resolutions change
//
////////////////////////////////////////////////////////////////////////////////

bool Asura_Render::SetVideoMode( int iWidth, int iHeight, bool bFSAA)
{
	bool bRet = Platform_SetVideoMode( iWidth, iHeight, bFSAA ); 

#ifdef ASURA_USE_GUI_MENU_SYSTEM
	Asura_GUIMenu_System::NotifyVideoModeChange();
#endif // ASURA_USE_GUI_MENU_SYSTEM

	return bRet;
}

#ifdef ASURA_EDGE_ANTIALIASING
void Asura_Render::SetCollectAAEdgesFlag( bool bCollectEdges )
{
	Platform_SetCollectAAEdges( bCollectEdges );
}
//
void Asura_Render::RenderEdgeAntiAliasing( void )
{
	Platform_RenderEdgeAntiAliasing();
}
#endif
/// End of function 'Asura_Render::SetVideoMode'. ///
