//
// Asura_RenderStates.h
//
// Created by Kevin Lea - 12/06/2001 11:05:33
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_RENDERSTATES_H__
#define __ASURA_RENDERSTATES_H__

#include "Core/Asura_TextureFlags.h" //for enum ASURA_TEXTURE_ID. Moved to get rid of some unwanted dependencies in the plugins. Richard 25/04/02.
#include "Debugging/Asura_Assert.h"

#if defined( ASURA_TARGET_SM3)
	// Any cards that support the new renderer should be able to cope with this
	// Any cards that SM30 must support 16 Samplers
	#define MAX_TEXTURES_PER_POLYGON 16 
#else
	#define MAX_TEXTURES_PER_POLYGON 4
#endif

// DLH - The hard coded values are to lock down the enum as some systems save them to files 
enum ASURA_TRANSLUCENCY_MODE
{
	// DLH - ADD NEW VALUES AT THE END OF THE LIST
	
	ASURA_TRANSLUCENCY_OFF = 0,
	ASURA_TRANSLUCENCY_NORMAL = 1,
	ASURA_TRANSLUCENCY_INVCOLOUR = 2,
	ASURA_TRANSLUCENCY_COLOUR = 3,
	ASURA_TRANSLUCENCY_DOUBLECOLOUR = 4,
	ASURA_TRANSLUCENCY_GLOWING = 5,
	ASURA_TRANSLUCENCY_DARKENINGCOLOUR = 6,
	ASURA_TRANSLUCENCY_JUSTSETZ = 7,
	ASURA_TRANSLUCENCY_ALPHACOMPARE = 8,
	ASURA_TRANSLUCENCY_UGH = 9,
	ASURA_TRANSLUCENCY_NOT_SET = 10,

	// (MH - 29.07.2003) probably a good idea not to change any of these values 
	// as they will be saved in particle templates. I am adding a special one 
	// to use with heathaze particles. The idea of modifying the particle template
	// code fills me with dread and I'm certain I won't be able to do it without
	// breaking something...
	ASURA_TRANSLUCENCY_SPECIAL_HEATHAZE = 11,
	ASURA_TRANSLUCENCY_MINIMUM = 12, // KOR - 10/01/2006 - yet to be implemented on platforms other than PSP
	ASURA_TRANSLUCENCY_ADD = 13,
	
	// DLH - ADD NEW VALUES HERE
};


enum ASURA_ZBUFFERCMP_ID
{
	ASURA_ZBUFFERCMP_LESSEQUAL,
	ASURA_ZBUFFERCMP_ALWAYS,
	ASURA_ZBUFFERCMP_EQUALS,
	ASURA_ZBUFFERCMP_GREATER,
};

enum ASURA_TEXTUREOP_ID
{
	ASURA_TEXTUREOP_SINGLE_PASS,
	ASURA_TEXTUREOP_SINGLE_PASS_2X,
	ASURA_TEXTUREOP_WOLFVISION,
	ASURA_TEXTUREOP_MODULATE,
	ASURA_TEXTUREOP_TOON,
	ASURA_TEXTUREOP_SELFILLUMINATE,
	ASURA_TEXTUREOP_PAINTERLY,
	ASURA_TEXTUREOP_SPECULAR,
	ASURA_TEXTUREOP_OPAQUE,
	ASURA_TEXTUREOP_LIGHTPASS,
	ASURA_TEXTUREOP_PLAINTEXTURE,
	ASURA_TEXTUREOP_BRIGHTGREEN,
	ASURA_TEXTUREOP_REFLECTIVE,
	ASURA_TEXTUREOP_JUSTALPHA,
    ASURA_TEXTUREOP_WHITE,          // Untextured and fullbright white.
	ASURA_TEXTUREOP_DUALTEXTURE,
	ASURA_TEXTUREOP_LERP,
	ASURA_TEXTUREOP_COLOUR,
	ASURA_TEXTUREOP_COPYALPHA,
	ASURA_TEXTUREOP_VIEWTEXTURECOLOUR,
	ASURA_TEXTUREOP_VIEWTEXTUREALPHA,

	// DLH - 28 June 2006 15:03:18 - Special Texture Op mode that does not do anything! 
	// DLH - 28 June 2006 15:03:18 - If you are going to modify the Texture Stages call this first. Else the Asura_Shader::RequestTextureOpMode() cache will not update update the texture stages when an actual mode is required.
	ASURA_TEXTUREOP_CUSTOM, 
	ASURA_TEXTUREOP_INVALID
};

enum ASURA_CULLMODE_ID
{
	ASURA_CULLMODE_NONE,
	ASURA_CULLMODE_CW,
	ASURA_CULLMODE_CCW
};

static const int iASURA_ALPHATEST_REF_DEFAULT = 10;

class Asura_Render_States
{
	public:
		void ResetAll(void)
		{
			for(int i=0; i<MAX_TEXTURES_PER_POLYGON; i++)
			{
				SetTextureIndex(i,ASURA_TEXTURE_NONE);
			}
			SetWireFrameMode(false);
			SetDitheringMode(true);
			SetZBufferEnabled(true);
			SetZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
			SetZBufferWriteEnabled(true);
			SetAlphaBlending(false);
			SetTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
			SetTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
			SetAlphaTest(true);
			SetDynamicLights(false);
			SetCullMode(ASURA_CULLMODE_NONE);
			m_iAlphaTestRef = iASURA_ALPHATEST_REF_DEFAULT;
			SetStencilEnabled(false);
		}
		// DLH - 15 June 2006 10:56:05 - Clears all the bound textures from the start vcalue up to the last.
		inline void ResetTextureIndexToLast( const u_int uTextureStartIndex)
		{
			for( u_int uTextureIndex = uTextureStartIndex; uTextureIndex < MAX_TEXTURES_PER_POLYGON; uTextureIndex++)
			{
				SetTextureIndex( uTextureIndex,ASURA_TEXTURE_NONE);
			}
		}
		inline void RequestTextureIndex(const int t, const int i)
		{
			Asura_Assert( (0 <= t) && (t < MAX_TEXTURES_PER_POLYGON), "Invalid texture number passed to Asura_Render_States::RequestTextureIndex()." );

			if (ASURA_TEXTURE_INVALID == i)
			{
				TextureIndex [t]	= ASURA_TEXTURE_INVALID;
			}

			else if ( (i != TextureIndex [t]) || (ASURA_TEXTURE_INVALID == TextureIndex [t]) )
			{
				SetTextureIndex	(t, i);
			}
		}

		inline void RequestWireFrameMode(bool b)
		{
			if (WireFrameMode!=b) SetWireFrameMode(b);
		}
		inline void RequestDitheringMode(bool b)
		{
			if (DitheringMode!=b) SetDitheringMode(b);
		}
		inline void RequestZBufferEnabled(bool b)
		{
			if (ZBufferEnabled!=b) SetZBufferEnabled(b);
		}
		inline void RequestZBufferCompareMode(enum ASURA_ZBUFFERCMP_ID mode)
		{
			if (ZBufferCompareMode!=mode) SetZBufferCompareMode(mode);
		}
		inline void RequestZBufferWriteEnabled(bool b)
		{
			if (ZBufferWriteEnabled!=b) SetZBufferWriteEnabled(b);
		}
		inline bool GetZBufferWriteEnabled()
		{
			return (ZBufferWriteEnabled);
		}
		inline void RequestTranslucencyMode(enum ASURA_TRANSLUCENCY_MODE mode)
		{
			if (TranslucencyMode != mode) SetTranslucencyMode(mode);
		}
		inline void InvalidateTextureOpMode()
		{
			this->TextureOpMode = ASURA_TEXTUREOP_INVALID;
		}
		inline void RequestTextureOpMode(enum ASURA_TEXTUREOP_ID mode)
		{
			if (TextureOpMode != mode) SetTextureOpMode(mode);
		}
		inline void RequestAlphaTest(bool b, int iRefValue = iASURA_ALPHATEST_REF_DEFAULT)
		{			
			// DMR (21/11/05) - When alpha-testing without blending, we need to be able to
			//                  specify the threshold value
			if ( (AlphaTest != b) || (m_iAlphaTestRef != iRefValue) )
			{			
				m_iAlphaTestRef = iRefValue;			
				SetAlphaTest(b);
			}
		}
		inline void RequestDynamicLights(bool b)
		{
			if (DynamicLights!=b) SetDynamicLights(b);
		}
		inline void RequestCullMode(enum ASURA_CULLMODE_ID eMode)
		{
			if (m_eCullMode != eMode) SetCullMode(eMode);
		
		}
		inline void RequestStencilEnabled(bool b)
		{
			if (StencilEnabled!=b) SetStencilEnabled(b);
		}


	private:
		// KJL 15:08:17 11/08/99 - the implementation of the following
		// methods is platform specific, and is in PS_RenderStates.cpp
		void SetTextureIndex(int t, int i);
		void SetWireFrameMode(bool b);
		void SetDitheringMode(bool b);
		void SetZBufferEnabled(bool b);
		void SetZBufferCompareMode(enum ASURA_ZBUFFERCMP_ID mode);
		void SetZBufferWriteEnabled(bool b);
		void SetAlphaBlending(bool b);
		void SetTranslucencyMode(enum ASURA_TRANSLUCENCY_MODE mode);
		void SetTextureOpMode(enum ASURA_TEXTUREOP_ID mode);
		void SetAlphaTest(bool b);
		void SetDynamicLights(bool b);
		void SetCullMode(enum ASURA_CULLMODE_ID eMode);
		void SetStencilEnabled(bool b);

		// Asura's internal memory of the render states
		int TextureIndex[MAX_TEXTURES_PER_POLYGON];
		bool WireFrameMode;
		bool DitheringMode;
		bool ZBufferEnabled;
		enum ASURA_ZBUFFERCMP_ID ZBufferCompareMode;
		bool ZBufferWriteEnabled;
		bool AlphaBlendingEnabled;
		enum ASURA_TRANSLUCENCY_MODE TranslucencyMode;
		enum ASURA_TEXTUREOP_ID TextureOpMode;
		bool AlphaTest;
		bool DynamicLights;
		enum ASURA_CULLMODE_ID m_eCullMode;
		int  m_iAlphaTestRef;
		bool StencilEnabled;
};


#endif//__ASURA_RENDERSTATES_H__
