//
// Asura_SortedRender.h
//
// Created by Richard Rice - 16/12/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_SORTEDRENDER_H__
#define __ASURA_SORTEDRENDER_H__

#include "Core/Asura_Maths.h"

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
#include "Core/Asura_Hash.h"
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)

/*
This is for queueing up a series of shapes,skins and hierarchies so that they can be sorted
by texture before rendering. RAR
*/

#ifndef ASURA_SORTEDRENDER_QUEUESIZE 
	#define ASURA_SORTEDRENDER_QUEUESIZE (200) //maximum number of items that can be queued up
#endif

#ifndef ASURA_SORTEDRENDER_STORAGESIZE 
	#define ASURA_SORTEDRENDER_STORAGESIZE (8192) //storage space in bytes for the items to be rendered
#endif

class  Asura_Animation;
class  Asura_ClientEntity_Glass;
class  Asura_Shape_Instance;
struct Asura_SortedRenderItem;

class Asura_SortedRender
{
public :

	//start storing items to be sorted and rendered later
	static void Start();
	//render all the opaque stored items , and deactivate
	static void End();

#if defined ASURA_TARGET_SM3
	//render all the transparent stored items
	static void RenderTransparentItems();

	static bool IsTransparentPass() { return s_bTransparentPass; }
	static void SetTransparentPass(bool bIsTansparentPASS) { s_bTransparentPass = bIsTansparentPASS; }
#endif

	//Is this currently accepting items for later rendering
	static bool IsActive(){return s_bActive;}

	static void AddShape( Asura_Shape_Instance* pxShapeInstance, u_int uCamVisFlags );
	static void AddAnimation( Asura_Animation* pxAnim,
                              const Asura_Vector_3& xPos,
                              const Asura_Matrix_3x3& xOri,
                              u_int uCamVisFlags,
                              u_int uRenderFlags,
                              const Asura_Vector_3& xBSCentre,
                              float fBSRadius );
	static void AddGlass( Asura_ClientEntity_Glass* pxGlass, u_int uCamVisFlags );

	#ifdef ASURA_TOOLS
	static void AddToolsMesh(class Asura_Mesh* pxMesh,
								const Asura_Vector_3& xPosition,
								const Asura_Matrix_3x3& xOrientation,
								bool bWithLighting,
								float fAlpha,
								const Asura_Vector_3& xScale,
								int iTextureID);

	static void AddToolsCuboid(const Asura_Bounding_Box& xBox,
								const Asura_Vector_3& xPosition,
								const Asura_Matrix_3x3& xOrient,
								unsigned int iColour, 
								bool bWireframe,
								int iHighlightSide,
								int iHighlightColour);
	#endif

protected :

	static void* AllocateSpace(unsigned uSize);

	static void Platform_BeginRender(void);
	static void Platform_EndRender(void);

	static bool s_bActive;

	static int s_iQueueSize;
	static struct Asura_SortedRenderItem* s_apxRenderQueue[ASURA_SORTEDRENDER_QUEUESIZE];

	static unsigned s_uStorageUsed;
	static char s_acRenderItemStorage[ASURA_SORTEDRENDER_STORAGESIZE];

	static int CompareItems( const void* pItem1, const void* pItem2 );

#if defined ASURA_TARGET_SM3
	static int s_iFirstTransparentItem;
	static bool s_bTransparentPass;
#endif
};

enum Asura_SortedRenderItem_Type
{
    ASURA_SORTEDRENDERITEM_TYPE_SHAPE,
    ASURA_SORTEDRENDERITEM_TYPE_ANIMATION_SMOOTH_SKIN,
    ASURA_SORTEDRENDERITEM_TYPE_ANIMATION_HIERARCHICAL,
    ASURA_SORTEDRENDERITEM_TYPE_GLASS,
	ASURA_SORTEDRENDERITEM_TYPE_TOOLSMESH, 
	ASURA_SORTEDRENDERITEM_TYPE_TOOLSCUBOID, 
    ASURA_SORTEDRENDERITEM_NUM_TYPES,
};

struct Asura_SortedRenderItem
{
	void RenderItem( void ) const;

    void SetupSortKey( int iTexID, bool bAlpha );
#ifdef ASURA_SORTEDRENDER_SORTALPHAONDISTANCE
    void SetupAlphaSortKey( int iCam, const Asura_Vector_3& xPos );
#endif

    // Each bias should be a integer value in range [0, 0x7FFF]. Lower biases
    // will get the item rendered earlier.
    u_int Platform_GetSortBias( void ) const;
    
    Asura_SortedRenderItem_Type m_eType;
	u_int m_uSortKey;

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	Asura_Hash_ID m_uZoneHashID; // Zone to use for relevant handlers.
	Asura_Hash_ID m_uContainingZoneHashID; // Zone that contains the shape/animation etc.
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
};

struct Asura_SortedRenderItem_Shape : Asura_SortedRenderItem
{
	void Render( void ) const;

	Asura_Shape_Instance* m_pxSI;
	u_int m_uCamVisFlags;
};

struct Asura_SortedRenderItem_Animation : Asura_SortedRenderItem
{
	void Render( void ) const;

	Asura_Animation* m_pxAnimation;
	Asura_Vector_3 m_xPosition;
	Asura_Matrix_3x3 m_xOrient;
	unsigned m_uCamVisFlags;
	unsigned m_uRenderFlags;
    Asura_Vector_3 m_xBSCentre;
    float m_fBSRadius;
};

struct Asura_SortedRenderItem_Glass : Asura_SortedRenderItem
{
	void Render( void ) const;

	Asura_ClientEntity_Glass* m_pxGlass;
	u_int m_uCamVisFlags;
};

#ifdef ASURA_TOOLS
struct Asura_SortedRenderItem_ToolsMesh : Asura_SortedRenderItem
{
	void Render( void ) const;

	class Asura_Mesh* m_pxMesh;
	Asura_Vector_3 m_xPosition;
	Asura_Matrix_3x3 m_xOrientation;
	bool m_bWithLighting;
	float m_fAlpha;
	Asura_Vector_3 m_xScale;
	int m_iTextureID;

};

struct Asura_SortedRenderItem_ToolsCuboid : Asura_SortedRenderItem
{
	void Render( void ) const;

	Asura_Bounding_Box m_xBox;
	Asura_Vector_3 m_xPosition;
	Asura_Matrix_3x3 m_xOrient;
	unsigned int m_iColour;
	bool m_bWireframe;
	int m_iHighlightSide;
	int m_iHighlightColour;

};

#endif //ASURA_TOOLS



#endif //__ASURA_SORTEDRENDER_H__
