//
// Asura_Surface.cpp
//
// Created by Kevin Lea - 12/06/2001 11:05:41
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_TOOLS

#include "Rendering/Asura_Surface.h"

#include "Core/Asura_Material_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Bicubic_Patch.h"
#include "Rendering/Asura_PatchSubdivision.h"
#include "Rendering/Asura_Render.h" // for defn of ASURA_TEXTURE_NONE

#include "Rendering/Asura_Direct3D.h"

#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Misc/Tools_Render.h"

Asura_Surface_Mesh Asura_Surface_Mesh_System::s_axSurfaceMeshStorage[ASURA_MAX_NO_OF_MESHES];
VertexFormat* VertexArray;
unsigned short* IndexArray;

void Asura_Surface_Mesh_System::Initialise(void)
{
	VertexArray = 0;
	IndexArray = 0;
}
void Asura_Surface_Mesh_System::Shutdown(void)
{
	if (VertexArray)
	{
		delete [] VertexArray;
		VertexArray = 0;
	}
	if (IndexArray)
	{
		delete [] IndexArray;
		IndexArray = 0;
	}
}

Asura_Bicubic_Patch patch;


static int SurfaceSortSub(const void *cmp1, const void *cmp2)
{
    if (((const Asura_Surface*)cmp1)->m_iMaterialID < ((const Asura_Surface*)cmp2)->m_iMaterialID)
        return  1;
    if (((const Asura_Surface*)cmp1)->m_iMaterialID > ((const Asura_Surface*)cmp2)->m_iMaterialID)
        return -1;

    return 0;
}

void Asura_Surface_Mesh_System::SortOnTexture(Asura_Surface_Mesh* surfaceMesh)
{
	qsort(surfaceMesh->Surfaces, surfaceMesh->NumberOfSurfaces, sizeof(Asura_Surface), SurfaceSortSub);

}

void Asura_Surface_Mesh_System::InitMeshForSurfaceRender(Asura_Mesh& xMesh, const Asura_Vector_3& xPosition)
{
	Asura_Surface_Mesh* pxSurfaceMesh = &s_axSurfaceMeshStorage[xMesh.Index];

	patch.SetMaxDetailLevel(Asura_Render::GetPatchesSubdivLevel());

	if (!xMesh.AreNormalsValid())
	{
		xMesh.CreateNormals();
	}

	if (!pxSurfaceMesh->SurfaceDataValid)
	{
		ProcessMeshIntoSurfaces(xMesh, xPosition);
	}	
}

void Asura_Surface_Mesh_System::RenderAsSurfaces(Asura_Mesh& xMesh, const Asura_Vector_3& xPosition)
{
	InitMeshForSurfaceRender(xMesh, xPosition);
	if (Asura_Render::GetStaticSubdivision())
	{
		MakeVertexBufferAndIndexList(xMesh, xPosition);
		RenderFromVertexBuffer(xMesh);
	//	delete surfaceMesh;
	}
	else
	{
		Asura_PatchSubdivision::FindPointsNearModule(&xMesh, xPosition);

		Asura_Surface_Mesh* pxSurfaceMesh = &s_axSurfaceMeshStorage[xMesh.Index];
		for (int i=0; i<pxSurfaceMesh->NumberOfSurfaces; i++)
		{
			Asura_Surface* pxSurfaceDataPtr = &pxSurfaceMesh->Surfaces[i];
			//void DrawPatch(Asura_Surface* surface, int meshID, int patchNumber);
			//DrawPatch(surfaceDataPtr,mesh.Index,i);
			patch.Render(pxSurfaceDataPtr);
		}
	}
}

void Asura_Surface_Mesh_System::ProcessMeshIntoSurfaces(Asura_Mesh& mesh, const Asura_Vector_3& position)
{
	Asura_Surface_Mesh* surfaceMesh = &s_axSurfaceMeshStorage[mesh.Index];
	
	surfaceMesh->NumberOfSurfaces = mesh.NumberOfPolygons;
	surfaceMesh->Surfaces = new Asura_Surface[mesh.NumberOfPolygons];
	surfaceMesh->SurfaceDataValid = true;
	

	int i;

	#if defined(ASURA_TARGET_SM3)
	//This version of ProcessMeshIntoSurfaces() seems to be lacking a whole load of initialisation
	//that gets done in the module version , which would result in a fair amount of calculation using junk values.
	//If this is actually getting used , then it probably needs to be updated.
	//RAR 09/07/2007
	Asura_Assert(0,"Asura_Surface_Mesh_System::ProcessMeshIntoSurfaces() being used.\nThis version is possibly needs updating if it is being used.");
	#endif


	// KJL - 21-May-01 15:24:08 - make surfaces
	for (i=0; i<mesh.NumberOfPolygons; i++)
	{
		//Initialise each surface from its polygon data - RAR
		surfaceMesh->Surfaces[i].Initialise(&mesh,i);
	}



	#if 01
	for (i=0; i<mesh.NumberOfPolygons; i++)
	{
		if (surfaceMesh->Surfaces[i].NumberOfVertices==0) continue;

		// Kevin Lea - 22/08/2001 11:08:51 - if flagged as 'not curved' skip surface
		if ((mesh.SmoothingGroupDataPtr[i]&(1<<31))) continue;

		// KJL 21/05/00 14:48:04 - are you a flat surface?
		if (surfaceMesh->Surfaces[i].IsCurved) continue;

		// KJL 21/05/00 14:49:43 - okay, we have a flat surface. Now look for surfaces
		// which share an edge, and are curved.
		for (int j=0; j<mesh.NumberOfPolygons; j++)
		{
			Asura_Surface_Mesh* surfaceMesh2 = &s_axSurfaceMeshStorage[mesh.Index];
			
			if (i==j && surfaceMesh==surfaceMesh2) continue;
			//if (surfaceMesh->Surfaces[j].NumberOfVertices==0) continue;
			//if (!surfaceMesh2->Surfaces[j].IsCurved) continue;

			{ 
				int iv1=-1;
				int iv2=-1;
				int jv1=-1;
				int jv2=-1;
				
				{
					for (int a=0; a<surfaceMesh->Surfaces[i].NumberOfVertices; a++)
					{
						for (int b=0; b<surfaceMesh2->Surfaces[j].NumberOfVertices; b++)
						{
							if (  (surfaceMesh->Surfaces[i].VertexID[a]==surfaceMesh2->Surfaces[j].VertexID[b]) )
							{
								jv1 = b;
								iv1 = a;
								break;
							}
						}
						if (iv1!=-1) break;
					}
				}

				if (iv1==-1) continue;

				{
					for (int a=0; a<surfaceMesh->Surfaces[i].NumberOfVertices; a++)
					{
						if (a==iv1) continue;
						for (int b=0; b<surfaceMesh2->Surfaces[j].NumberOfVertices; b++)
						{
							if (  (surfaceMesh->Surfaces[i].VertexID[a]==surfaceMesh2->Surfaces[j].VertexID[b]) )
							{
								jv2 = b;
								iv2 = a;
								break;
							}
						}
						if (iv2!=-1) break;
					}
				}
				
				if (iv2==-1) continue;
			
				// KJL 21/05/00 15:05:02 - found shared edge.
				
				int is,io=0;
				switch (iv1+iv2+iv1*iv2)
				{
					default:
					case 1: // edge 0 - of a quad or triangle
					{
						is=0;
						break;
					}
					case 5: // edge 1 - of a quad or triangle
					{
						is=1;
						break;
					}
					case 11: // edge 2 - of a quad or triangle
					{
						is=2;
						break;
					}
					case 3: // edge 3 - of a quad
					{	
						is=3;
						break;
					}
					case 2: // edge 2 - of a triangle
					{
						is=2;
						break;						
					}
				}
				if (iv1!=is) io++;


				int js,jo=0;
				switch (jv1+jv2+jv1*jv2)
				{
					default:
					case 1: // edge 0 - of a quad or triangle
					{
						js=0;
						break;
					}
					case 5: // edge 1 - of a quad or triangle
					{
						js=1;
						break;
					}
					case 11: // edge 2 - of a quad or triangle
					{
						js=2;
						break;
					}
					case 3: // edge 3 - of a quad
					{	
						js=3;
						break;
					}
					case 2: // edge 2 - of a triangle
					{
						js=2;
						break;						
					}
				}
				if (jv1!=js) jo++;
				int io2 = (io+1)&1;
				int jo2 = (jo+1)&1;
				
	
				if(surfaceMesh2->Surfaces[j].IsCurved)
				{
					surfaceMesh->Surfaces[i].CNormalID[is][io]  = surfaceMesh2->Surfaces[j].CNormalID[js][jo];
					surfaceMesh->Surfaces[i].CNormalID[is][io2] = surfaceMesh2->Surfaces[j].CNormalID[js][jo2];

					surfaceMesh->Surfaces[i].IsEndCap = true;
					
				}

				if(surfaceMesh->Surfaces[i].m_bHasNoCurveFlag || surfaceMesh2->Surfaces[j].m_bHasNoCurveFlag)
				{
					surfaceMesh->Surfaces[i].m_abStraightEdge[is] = true;
					surfaceMesh2->Surfaces[j].m_abStraightEdge[js] = true;
				}
					
			}
		}
	}
	#endif



	// KJL 02/06/00 15:34:43 - blend curves
	#if 01
	for (i=0; i<mesh.NumberOfPolygons; i++)
	{
		if (surfaceMesh->Surfaces[i].NumberOfVertices==0) continue;

		// KJL 21/05/00 14:48:04 - are you a curve surface?
		if (!surfaceMesh->Surfaces[i].IsCurved) continue;

		// KJL 21/05/00 14:49:43 - okay, we have a curve surface. Now look for surfaces
		// which share an edge, and are curved.
		for (int j=0; j<mesh.NumberOfPolygons; j++)
		{
			if (i==j) continue;		
			if (surfaceMesh->Surfaces[j].NumberOfVertices==0) continue;
			if (!surfaceMesh->Surfaces[j].IsCurved) continue;

			{ 
				int iv1=-1;
				int iv2=-1;
				int jv1=-1;
				int jv2=-1;
				
				{
					for (int a=0; a<surfaceMesh->Surfaces[i].NumberOfVertices; a++)
					{
						for (int b=0; b<surfaceMesh->Surfaces[j].NumberOfVertices; b++)
						{
							if (  (surfaceMesh->Surfaces[i].VertexID[a]==surfaceMesh->Surfaces[j].VertexID[b]) )
							{
								jv1 = b;
								iv1 = a;
								break;
							}
						}
						if (iv1!=-1) break;
					}
				}

				if (iv1==-1) continue;

				{
					for (int a=0; a<surfaceMesh->Surfaces[i].NumberOfVertices; a++)
					{
						if (a==iv1) continue;
						for (int b=0; b<surfaceMesh->Surfaces[j].NumberOfVertices; b++)
						{
							if (  (surfaceMesh->Surfaces[i].VertexID[a]==surfaceMesh->Surfaces[j].VertexID[b]) )
							{
								jv2 = b;
								iv2 = a;
								break;
							}
						}
						if (iv2!=-1) break;
					}
				}
				
				if (iv2==-1) continue;
			
				// KJL 21/05/00 15:05:02 - found shared edge.
				
				int is=-1,io=0;
				switch (iv1+iv2+iv1*iv2)
				{
					case 1: // edge 0 - of a quad or triangle
					{
						is=0;
						break;
					}
					case 5: // edge 1 - of a quad or triangle
					{
						is=1;
						break;
					}
					case 11: // edge 2 - of a quad or triangle
					{
						is=2;
						break;
					}
					case 3: // edge 3 - of a quad
					{	
						is=3;
						break;
					}
					case 2: // edge 2 - of a triangle
					{
						is=2;
						break;						
					}
				}
				if (is==-1) continue;
				if (iv1!=is) io++;


				int js=-1,jo=0;
				switch (jv1+jv2+jv1*jv2)
				{
					case 1: // edge 0 - of a quad or triangle
					{
						js=0;
						break;
					}
					case 5: // edge 1 - of a quad or triangle
					{
						js=1;
						break;
					}
					case 11: // edge 2 - of a quad or triangle
					{
						js=2;
						break;
					}
					case 3: // edge 3 - of a quad
					{	
						js=3;
						break;
					}
					case 2: // edge 2 - of a triangle
					{
						js=2;
						break;						
					}
				}
				if (js==-1) continue;
				if (jv1!=js) jo++;
				int io2 = (io+1)&1;
				int jo2 = (jo+1)&1;

				surfaceMesh->Surfaces[i].CNormalID[is][io] += surfaceMesh->Surfaces[j].CNormalID[js][jo];
				surfaceMesh->Surfaces[i].CNormalID[is][io2] += surfaceMesh->Surfaces[j].CNormalID[js][jo2];
				surfaceMesh->Surfaces[i].CNormalID[is][io].Normalise();
				surfaceMesh->Surfaces[i].CNormalID[is][io2].Normalise();

				surfaceMesh->Surfaces[j].CNormalID[js][jo] = surfaceMesh->Surfaces[i].CNormalID[is][io];
				surfaceMesh->Surfaces[j].CNormalID[js][jo2] = surfaceMesh->Surfaces[i].CNormalID[is][io2];

				//surfaceMesh->Surfaces[i].LNormalID[is] = surfaceMesh->Surfaces[j].LNormalID[js];
		


			}
		}
	}
	#endif
	{
		for (int pID=0; pID<surfaceMesh->NumberOfSurfaces; pID++)
		{
			Asura_Surface* surfaceDataPtr = &surfaceMesh->Surfaces[pID];
			if (surfaceDataPtr->NumberOfVertices==3)
			{
				Asura_Vector_3& v0 = mesh.VertexDataPtr[surfaceDataPtr->VertexID[0]].Position;
				Asura_Vector_3& v1 = mesh.VertexDataPtr[surfaceDataPtr->VertexID[1]].Position;
				Asura_Vector_3& v2 = mesh.VertexDataPtr[surfaceDataPtr->VertexID[2]].Position;
 
				surfaceDataPtr->ControlPoint[0][0] = v0;
				surfaceDataPtr->ControlPoint[3][0] = v1;
				surfaceDataPtr->ControlPoint[3][3] = v1;
				surfaceDataPtr->ControlPoint[0][3] = v2;
		
				// KJL 17/05/00 15:10:29 - tri edge 1
				{

					if(surfaceDataPtr->m_abStraightEdge[0])
					{
						surfaceDataPtr->ControlPoint[1][0] = (v0*2.0f+v1)/3.0f;
						surfaceDataPtr->ControlPoint[2][0] = (v0+v1*2.0f)/3.0f;
					}
					else
					{
						Asura_Vector_3 edgedir = v1-v0;
						float edgelength = edgedir.Magnitude()/3.0f;
						
						Asura_Vector_3& n0 = surfaceDataPtr->CNormalID[0][0];
						Asura_Vector_3& n1 = surfaceDataPtr->CNormalID[0][1];
						Asura_Vector_3 normal0 = (edgedir)^(n0);
						Asura_Vector_3 normal1 = (edgedir)^(n1);
						
						normal0.Normalise();
						normal1.Normalise();

						surfaceDataPtr->ControlPoint[1][0] = surfaceDataPtr->ControlPoint[0][0]+(n0^normal0)*edgelength;
						surfaceDataPtr->ControlPoint[2][0] = surfaceDataPtr->ControlPoint[3][0]-(n1^normal1)*edgelength;
					}
				}

				// KJL 17/05/00 15:10:29 - tri edge 2
				{

					if(surfaceDataPtr->m_abStraightEdge[1])
					{
						surfaceDataPtr->ControlPoint[1][3] = (v1*2.0f+v2)/3.0f;
						surfaceDataPtr->ControlPoint[2][3] = (v1+v2*2.0f)/3.0f;
					}
					else
					{
						Asura_Vector_3 edgedir = v2-v1;
						float edgelength = edgedir.Magnitude()/3.0f;

						Asura_Vector_3& n1 = surfaceDataPtr->CNormalID[1][0];
						Asura_Vector_3& n2 = surfaceDataPtr->CNormalID[1][1];
						Asura_Vector_3 normal2 = (edgedir)^(n2);
						Asura_Vector_3 normal1 = (edgedir)^(n1);
						
						normal2.Normalise();
						normal1.Normalise();

						surfaceDataPtr->ControlPoint[1][3] = surfaceDataPtr->ControlPoint[0][3]-(n2^normal2)*edgelength;
						surfaceDataPtr->ControlPoint[2][3] = surfaceDataPtr->ControlPoint[3][0]+(n1^normal1)*edgelength;
					}
				}

				// KJL 17/05/00 15:10:29 - tri edge 3
				{

					if(surfaceDataPtr->m_abStraightEdge[2])
					{
						surfaceDataPtr->ControlPoint[0][2] = (v2*2.0f+v0)/3.0f;
						surfaceDataPtr->ControlPoint[0][1] = (v2+v0*2.0f)/3.0f;
					}
					else
					{
						Asura_Vector_3 edgedir = v0-v2;
						float edgelength = edgedir.Magnitude()/3.0f;

						Asura_Vector_3& n2 = surfaceDataPtr->CNormalID[2][0];
						Asura_Vector_3& n0 = surfaceDataPtr->CNormalID[2][1];
						Asura_Vector_3 normal2 = (edgedir)^(n2);
						Asura_Vector_3 normal0 = (edgedir)^(n0);
					
						normal2.Normalise();
						normal0.Normalise();
					
						surfaceDataPtr->ControlPoint[0][2] = surfaceDataPtr->ControlPoint[0][3]+(n2^normal2)*edgelength;
						surfaceDataPtr->ControlPoint[0][1] = surfaceDataPtr->ControlPoint[0][0]-(n0^normal0)*edgelength;
					}
				}

				// KJL 17/05/00 15:20:31 - duplicate points
				surfaceDataPtr->ControlPoint[2][1] = surfaceDataPtr->ControlPoint[2][0];
				surfaceDataPtr->ControlPoint[2][2] = surfaceDataPtr->ControlPoint[2][3];
				surfaceDataPtr->ControlPoint[3][1] = surfaceDataPtr->ControlPoint[3][0];
				surfaceDataPtr->ControlPoint[3][2] = surfaceDataPtr->ControlPoint[3][0];	

				// KJL 17/05/00 15:22:28 - interpolated
				surfaceDataPtr->ControlPoint[1][1] = surfaceDataPtr->ControlPoint[0][1]+surfaceDataPtr->ControlPoint[1][0]-surfaceDataPtr->ControlPoint[0][0];
				surfaceDataPtr->ControlPoint[1][2] = surfaceDataPtr->ControlPoint[0][2]+surfaceDataPtr->ControlPoint[1][3]-surfaceDataPtr->ControlPoint[0][3];
			}
			else
			{
				Asura_Vector_3& v0 = mesh.VertexDataPtr[surfaceDataPtr->VertexID[0]].Position;
				Asura_Vector_3& v1 = mesh.VertexDataPtr[surfaceDataPtr->VertexID[1]].Position;
				Asura_Vector_3& v2 = mesh.VertexDataPtr[surfaceDataPtr->VertexID[2]].Position;
				Asura_Vector_3& v3 = mesh.VertexDataPtr[surfaceDataPtr->VertexID[3]].Position;
 
				surfaceDataPtr->ControlPoint[0][0] = v0;
				surfaceDataPtr->ControlPoint[3][0] = v1;
				surfaceDataPtr->ControlPoint[3][3] = v2;
				surfaceDataPtr->ControlPoint[0][3] = v3;
				
				// KJL 17/05/00 15:10:29 - quad edge 1
				{

					if(surfaceDataPtr->m_abStraightEdge[0])
					{
						surfaceDataPtr->ControlPoint[1][0] = (v0*2.0f+v1)/3.0f;
						surfaceDataPtr->ControlPoint[2][0] = (v0+v1*2.0f)/3.0f;
					}
					else
					{
						Asura_Vector_3 edgedir = v1-v0;
						float edgelength = edgedir.Magnitude()/3.0f;

						Asura_Vector_3& n0 = surfaceDataPtr->CNormalID[0][0];
						Asura_Vector_3& n1 = surfaceDataPtr->CNormalID[0][1];
						Asura_Vector_3 normal0 = (edgedir)^(n0);
						Asura_Vector_3 normal1 = (edgedir)^(n1);
						
						normal0.Normalise();
						normal1.Normalise();

						surfaceDataPtr->ControlPoint[1][0] = surfaceDataPtr->ControlPoint[0][0]+(n0^normal0)*edgelength;
						surfaceDataPtr->ControlPoint[2][0] = surfaceDataPtr->ControlPoint[3][0]-(n1^normal1)*edgelength;
					}
				}

				// KJL 17/05/00 15:10:29 - quad edge 2
				{

					if(surfaceDataPtr->m_abStraightEdge[1])
					{
						surfaceDataPtr->ControlPoint[3][2] = (v2*2.0f+v1)/3.0f;
						surfaceDataPtr->ControlPoint[3][1] = (v2+v1*2.0f)/3.0f;
					}
					else
					{
						Asura_Vector_3 edgedir = v2-v1;
						float edgelength = edgedir.Magnitude()/3.0f;

						Asura_Vector_3& n1 = surfaceDataPtr->CNormalID[1][0];
						Asura_Vector_3& n2 = surfaceDataPtr->CNormalID[1][1];
						Asura_Vector_3 normal2 = (edgedir)^(n2);
						Asura_Vector_3 normal1 = (edgedir)^(n1);
						
						normal2.Normalise();
						normal1.Normalise();

						surfaceDataPtr->ControlPoint[3][2] = surfaceDataPtr->ControlPoint[3][3]-(n2^normal2)*edgelength;
						surfaceDataPtr->ControlPoint[3][1] = surfaceDataPtr->ControlPoint[3][0]+(n1^normal1)*edgelength;
					}
				}

				// KJL 17/05/00 15:10:29 - quad edge 3
				{

					if(surfaceDataPtr->m_abStraightEdge[2])
					{
						surfaceDataPtr->ControlPoint[2][3] = (v2*2.0f+v3)/3.0f;
						surfaceDataPtr->ControlPoint[1][3] = (v2+v3*2.0f)/3.0f;
					}
					else
					{
						Asura_Vector_3 edgedir = v3-v2;
						float edgelength = edgedir.Magnitude()/3.0f;

						Asura_Vector_3& n2 = surfaceDataPtr->CNormalID[2][0];
						Asura_Vector_3& n3 = surfaceDataPtr->CNormalID[2][1];
						Asura_Vector_3 normal2 = (edgedir)^(n2);
						Asura_Vector_3 normal3 = (edgedir)^(n3);
						
						normal2.Normalise();
						normal3.Normalise();

						surfaceDataPtr->ControlPoint[2][3] = surfaceDataPtr->ControlPoint[3][3]+(n2^normal2)*edgelength;
						surfaceDataPtr->ControlPoint[1][3] = surfaceDataPtr->ControlPoint[0][3]-(n3^normal3)*edgelength;
					}
				}


				// KJL 17/05/00 15:10:29 - quad edge 4
				{

					if(surfaceDataPtr->m_abStraightEdge[3])
					{
						surfaceDataPtr->ControlPoint[0][2] = (v3*2.0f+v0)/3.0f;
						surfaceDataPtr->ControlPoint[0][1] = (v3+v0*2.0f)/3.0f;
					}
					else
					{
						Asura_Vector_3 edgedir = v0-v3;
						float edgelength = edgedir.Magnitude()/3.0f;

						Asura_Vector_3& n3 = surfaceDataPtr->CNormalID[3][0];
						Asura_Vector_3& n0 = surfaceDataPtr->CNormalID[3][1];
						Asura_Vector_3 normal3 = (edgedir)^(n3);
						Asura_Vector_3 normal0 = (edgedir)^(n0);
						
						normal3.Normalise();
						normal0.Normalise();

						surfaceDataPtr->ControlPoint[0][2] = surfaceDataPtr->ControlPoint[0][3]+(n3^normal3)*edgelength;
						surfaceDataPtr->ControlPoint[0][1] = surfaceDataPtr->ControlPoint[0][0]-(n0^normal0)*edgelength;
					}
				}

			
				// KJL 17/05/00 15:22:28 - interpolated
				surfaceDataPtr->ControlPoint[1][1] = surfaceDataPtr->ControlPoint[0][1]+surfaceDataPtr->ControlPoint[1][0]-surfaceDataPtr->ControlPoint[0][0];
				surfaceDataPtr->ControlPoint[1][2] = surfaceDataPtr->ControlPoint[0][2]+surfaceDataPtr->ControlPoint[1][3]-surfaceDataPtr->ControlPoint[0][3];
				surfaceDataPtr->ControlPoint[2][1] = surfaceDataPtr->ControlPoint[2][0]+surfaceDataPtr->ControlPoint[3][1]-surfaceDataPtr->ControlPoint[3][0];
				surfaceDataPtr->ControlPoint[2][2] = surfaceDataPtr->ControlPoint[3][2]+surfaceDataPtr->ControlPoint[2][3]-surfaceDataPtr->ControlPoint[3][3];
			}

			{
				for(int x=0; x<4; x++)
				for(int y=0; y<4; y++)
				{
					patch.ControlPoint[x][y]=position+surfaceDataPtr->ControlPoint[x][y];
					surfaceDataPtr->ControlPoint[x][y] = patch.ControlPoint[x][y];
					//TranslatePointIntoViewspace(&patch.ControlPoint[x][y]);
				}
			}

			surfaceDataPtr->ControlPointsValid=true;
				
			for(int n=0; n<4; n++)
			{
				surfaceDataPtr->Normal[n] = surfaceDataPtr->LNormalID[n];
			}
			if (surfaceDataPtr->NumberOfVertices==3)
			{
				surfaceDataPtr->Normal[2] = surfaceDataPtr->LNormalID[1];
				surfaceDataPtr->Normal[3] = surfaceDataPtr->LNormalID[2];
				surfaceDataPtr->U[3] = surfaceDataPtr->U[2];
				surfaceDataPtr->U[2] = surfaceDataPtr->U[1];
				surfaceDataPtr->V[3] = surfaceDataPtr->V[2];
				surfaceDataPtr->V[2] = surfaceDataPtr->V[1];
				surfaceDataPtr->U2[3] = surfaceDataPtr->U2[2];
				surfaceDataPtr->U2[2] = surfaceDataPtr->U2[1];
				surfaceDataPtr->V2[3] = surfaceDataPtr->V2[2];
				surfaceDataPtr->V2[2] = surfaceDataPtr->V2[1];
				surfaceDataPtr->Colour[3] = surfaceDataPtr->Colour[2];
				surfaceDataPtr->Colour[2] = surfaceDataPtr->Colour[1];
			}

		

			for (int i=0; i<4; i++)
			{
				float u,v;
				switch(i)
				{
					default:
					case 0:
						u=v=0.0f;
						break;
					case 1:
						u=1.0f;
						v=0.0f;
						break;
					case 2:
						u=v=1.0f;
						break;
					case 3:
						u=0.0f;
						v=1.0f;
						break;
				}
				surfaceDataPtr->du0dv0[i] = patch.EvaluateCurve(u,v);
				surfaceDataPtr->du2dv0[i] = patch.EvaluateCurve_d2u(u,v);
				surfaceDataPtr->du0dv2[i] = patch.EvaluateCurve_d2v(u,v);
				//surfaceDataPtr->Normal[i] = patch.EvaluateCurve_Normal(u,v);
			
			}

		
			surfaceDataPtr->SubdivideDataValid=true;

		}

//		SortOnTexture(surfaceMesh);
	}
}



void Asura_Surface::Initialise(const class Asura_Mesh* pxMesh,int iPolygonIndex)
{
	int n = pxMesh->PolygonDataPtr[iPolygonIndex].NumberOfVertices();

	NumberOfVertices = n;
	FaceNormal = pxMesh->NormalsDataPtr[iPolygonIndex].FaceNormal;//NormalsDataPtr[NormalsIndicesDataPtr[i*4]];
	ControlPointsValid = false;
	SubdivideDataValid = false;

	const Asura_Mesh_TextureInfo* pxTexInfo = 0;
	if(pxMesh->TextureDataValid)
	{
		pxTexInfo = pxMesh->m_pxProcessedTextureInfo ? &pxMesh->m_pxProcessedTextureInfo[iPolygonIndex] : &pxMesh->TextureDataPtr[iPolygonIndex];
	}


	if (pxMesh->TextureDataValid)
	{
		m_iMaterialID = pxTexInfo->m_iMaterialID;

		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		for(int i=0;i<TOOLS_NUM_MATERIALRESPONSE_PER_POLY;i++)
		{
			m_auMaterialResponseHashID[i] = pxTexInfo->m_auMaterialResponseHashID[i];
		}
		#endif
	}
	else
	{
		m_iMaterialID = ASURA_MATERIAL_NONE;

		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		for(int i=0;i<TOOLS_NUM_MATERIALRESPONSE_PER_POLY;i++)
		{
			m_auMaterialResponseHashID[i] = ASURA_HASH_ID_UNSET;
		}
		#endif
	}
	
	int offset = 0;
	#if 01
	{
		int index0 = pxMesh->PolygonDataPtr[iPolygonIndex].VertexID[0];
		int index1 = pxMesh->PolygonDataPtr[iPolygonIndex].VertexID[1];
		int index2 = pxMesh->PolygonDataPtr[iPolygonIndex].VertexID[2];
		int index3 = pxMesh->PolygonDataPtr[iPolygonIndex].VertexID[3];

		float e1 = (pxMesh->VertexDataPtr[index1].Position-pxMesh->VertexDataPtr[index0].Position).Magnitude();
		float e2 = (pxMesh->VertexDataPtr[index2].Position-pxMesh->VertexDataPtr[index1].Position).Magnitude();
		float e3,e4;
		
		if (index3!=0xffff)
		{
			e3 = (pxMesh->VertexDataPtr[index3].Position-pxMesh->VertexDataPtr[index2].Position).Magnitude();
			e4 = (pxMesh->VertexDataPtr[index0].Position-pxMesh->VertexDataPtr[index3].Position).Magnitude();
		}
		else
		{
			e3 = (pxMesh->VertexDataPtr[index0].Position-pxMesh->VertexDataPtr[index2].Position).Magnitude();
			e4 = 1e6;
		}
		
		if (e1<e4||e2<e4||e3<e4)
		{
			if (e1<e3 || e2<e3)
			{
				if (e1<e2)
				{
					if (e1<e4)
					{
						// e1 is least
						offset = 1;
					}
				}
				else
				{
					if (e2<e4)
					{
						// e2 is least
						offset = 2;
					}
				}
			}
			else
			{
				// e3 is least
				offset = 3;
			}
		}

		
	}
	#endif
	
	bool bUseLighting = pxMesh->PrelightingDataValid && (Tools_Level_Export::IsExport() || Tools_Render::LightingIsOn());

	for (int a=0; a<n; a++)
	{
		int b = (a+offset)%n;
	   VertexID[a] = pxMesh->PolygonDataPtr[iPolygonIndex].VertexID[b];
	   CNormalID[a][0] = pxMesh->NormalsDataPtr[iPolygonIndex].VertexNormal[b]; 
	   CNormalID[a][1] = pxMesh->NormalsDataPtr[iPolygonIndex].VertexNormal[(b+1)%n];//NormalsDataPtr[NormalsIndicesDataPtr[i*4+((b+1)%3)+1]];
	   LNormalID[a] =pxMesh->NormalsDataPtr[iPolygonIndex].VertexNormal[b];
	   m_abStraightEdge[a] = false;

		if (bUseLighting)
		{
			Colour[a] = pxMesh->PrelightingDataPtr[iPolygonIndex].Colour[b] | 0xff000000;
		}
		else
		{
			Colour[a] = 0xff808080;
		}
 
		
		if (pxTexInfo)
		{
			U[a] = pxTexInfo->U[0][b];//PolygonSurfacesDataPtr[iPolygonIndex].u[b];
			V[a] = pxTexInfo->V[0][b];//PolygonSurfacesDataPtr[iPolygonIndex].v[b];

			U2[a] = pxTexInfo->U[1][b];
			V2[a] = pxTexInfo->V[1][b];

			// Handle dual-textured polygons

			#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

				// Copy alpha value from the texture data block
				Colour[a] = (Asura_Maths::FloatToInt( pxTexInfo->m_afAlpha[b] * 255.0f) << 24) | (Colour[a] & 0xFFFFFF);

			#else // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

				if (Asura_Material_System::IsMaterialDualTexture(m_iMaterialID))
				{
					// Copy alpha value from the texture data block
					Colour[a] = (Asura_Maths::FloatToInt(pxTexInfo->m_afAlpha[b] * 255.0f) << 24) | (Colour[a] & 0xFFFFFF);
				}

			#endif // !defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		}
		else
		{
			U[a] = 0;
			V[a] = 0;

			U2[a] = 0.0f;
			V2[a] = 0.0f;
		}
		
	}

	m_bHasNoCurveFlag = (pxMesh->SmoothingGroupDataPtr[iPolygonIndex]&(1<<31)) != 0;

	// Kevin Lea - 22/08/2001 11:09:43 - check the 'not curved' flag
	if (!m_bHasNoCurveFlag)
	{
		CalculateIfCurved();
	}
	else
	{
		for(int i=0;i<4;i++)
		{
			m_abStraightEdge[i] = true;
		}
		IsCurved = false;
	}
	IsEndCap = false;
	
}

void Asura_Surface_Mesh_System::InvalidateAll()
{
	for (int i=0; i<ASURA_MAX_NO_OF_MESHES; i++)
	{
		s_axSurfaceMeshStorage[i].Invalidate();
	}
}

#endif // #ifdef ASURA_TOOLS
