//////////////////////////////////////////////////////////////////////////////
//
// Asura_CSM.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_CSM_H__
#define __ASURA_CSM_H__

#if defined ASURA_USE_CSM

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura_FixedArray.h"

/******************************************************************************/
/* Macros                                                                     */
/******************************************************************************/

#if !defined ASURA_CSM_MAX_MAPS
	#define ASURA_CSM_MAX_MAPS 4
#endif

#if !defined ASURA_CSM_SIZE
	#define ASURA_CSM_SIZE 1024
#endif

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_Matrix_3x3;
class Asura_Overlay;
class Asura_Vector_2;
class Asura_Vector_3;

////////////////////////////////////////////////////////////////////////////////
//
// Asura_CSM
//
// System to manage cascaded shadow mapping
//
////////////////////////////////////////////////////////////////////////////////

class Asura_CSM
{
public:

	static void Initialise();
	static void Reset();
	static void Shutdown();

	static void SetEnabled( const bool bEnabled ) { s_bEnabled = bEnabled; }
	static bool IsEnabled() { return s_bEnabled; }

	static void SetStrength( const u_int u, const float fStrength ) { s_afStrengths[u]=fStrength; }
	static float GetStrength( const u_int u ) { return s_afStrengths[u]; }

	static void SetShadowDir( const Asura_Vector_3& xDir );
	static const Asura_Vector_3& GetShadowDir();

	static void SetSplit( const u_int u, const float fVal ) { s_afSplits[u] = fVal; }
	static float GetSplit( const u_int u ) { return s_afSplits[u]; }

	static void Update();

	static void CreateShadowMaps();
	static void ApplyToScene();

#if defined ASURA_TOOLS
	static void AddOverlayControls( Asura_Overlay* const pxOverlay, Asura_Vector_2& xOffset,
		bool& bModified );
#endif

protected:

	static void Platform_Initialise();
	static void Platform_Shutdown();
	static void Platform_Reset();

	static void Platform_CreateShadowMaps();
	static void Platform_ApplyToScene();

	static void ResetSplits();

	// Data

	static bool s_bEnabled;
	static float s_fExtraFrustumDepth;

	static Asura_Matrix_3x3 s_axShadowViewMatrixOrients[ ASURA_CSM_MAX_MAPS ];
	static Asura_Vector_3 s_axShadowViewMatrixTrans[ ASURA_CSM_MAX_MAPS ];

	static u_int s_uNumMaps;

	static Asura_FixedArray< float, ASURA_CSM_MAX_MAPS + 1 > s_afSplits;
	static Asura_FixedArray< float, ASURA_CSM_MAX_MAPS > s_afStrengths;
	static Asura_FixedArray< float, ASURA_CSM_MAX_MAPS > s_afBSRadii;
	static Asura_FixedArray< Asura_Vector_3, ASURA_CSM_MAX_MAPS > s_axBSCentres;
};

#endif // defined ASURA_USE_CSM

#endif // !defined __ASURA_CSM_H__
