///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Shadow_Instance.cpp
// Author : Andy Tate
// Date   : 28-Nov-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Shadows/Asura_Shadow_Instance.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationAttachment.h"
#include "Camera/Asura_Camera.h"
#include "Debugging/Asura_DebugLineRender.h"
#include "Shadows/Asura_Shadow.h"
#include "Shadows/Asura_Shadow_System.h"
#include "Shapes/Asura_Shapes.h"
#include "Entity/DynamicLight/Asura_ClientEntity_DynamicLight.h"

// Asura platform header-files...

// Game independent header-files...

// Game platform header-files...

// Tools header-files...


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


// Camera variables...
Asura_Vector_3	Asura_Shadow_Instance::s_xCameraPosition;
Asura_Quat		Asura_Shadow_Instance::s_xCameraOrientation;
Asura_Quat		Asura_Shadow_Instance::s_xCameraInverseOrientation;
Asura_Frustum	Asura_Shadow_Instance::s_xCameraFrustum;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

#if !defined(ASURA_NO_DEBUG_OUTPUT)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance::RenderDebugInfo
// Author       : Andy Tate
// Date         : 1-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance::RenderDebugInfo (const Asura_Vector_3& xCentre,
											 const float fRadius,
											 const u_int uLevel) const
{
	if((Asura_Shadow_System::GetConfig().m_iDebugLevel > 0) && (uLevel > 0))
	{
		Asura_Matrix_3x3	xOrientation;
		Asura_Vector_3		xBSCentre;
		float				fBSRadius;

		Asura_DebugLineRender::AddFrustum(s_xCameraFrustum, s_xCameraPosition, s_xCameraOrientation, 0xFFFFFFFF);

		GetLightOrientation(xOrientation);
		fBSRadius	= m_xFrustum.GetBoundingSphere(xBSCentre, xOrientation, m_xLightPosition);

		if(uLevel > 1)
		{
			if(Asura_Shadow_System::GetConfig().m_iDebugLevel > 1)
			{
				Asura_DebugLineRender::AddFrustum(m_xFrustum, m_xLightPosition, xOrientation, 0x80FFFFFF);				
				Asura_DebugLineRender::AddSphere(xCentre, fRadius, 0x8000FF00);
			}
		}
		else
		{
			Asura_DebugLineRender::AddFrustum(m_xFrustum, m_xLightPosition, xOrientation, 0x80808080);
			Asura_DebugLineRender::AddSphere(xCentre, fRadius, 0x80808080);
		}

		Asura_DebugLineRender::AddAxis(m_xLightPosition, xOrientation, 0.500f);
//		Asura_DebugLineRender::AddAxis(xBSCentre, Asura_Maths::IdentityMatrix3x3, fBSRadius * 0.500f);

		if((Asura_Shadow_System::GetConfig().m_iDebugLevel > 2) && (uLevel > 2))
		{
			Asura_DebugLineRender::AddSphere(xBSCentre, fBSRadius, 0x80C0C0C0);			
//			Asura_DebugLineRender::AddAxis(xCentre, Asura_Maths::IdentityMatrix3x3, fRadius * 0.500f);
		}
	}
}

/// End of function 'Asura_Shadow_Instance::RenderDebugInfo'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance_Animation::RenderDebugInfo
// Author       : Andy Tate
// Date         : 1-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance_Animation::RenderDebugInfo (const Asura_Vector_3& xCentre,
													   const float fRadius,
													   const u_int uLevel) const
{
	PARENT::RenderDebugInfo(xCentre, fRadius, uLevel);

	if((Asura_Shadow_System::GetConfig().m_iDebugLevel > 2) && (uLevel > 2))
	{
		const Asura_Hierarchy_Skin*		pxSkin			= m_xAnimation.GetSkin();

		Asura_Vector_3			xPosition;
		Asura_Matrix_3x3		xOrientation;

		for (int i = 0; i < pxSkin->NumberOfBones;++i )
		{
			const Asura_Bounding_Box *pxSkinBB = m_xAnimation.GetBoneBB(i);
			
			if (pxSkinBB && m_xAnimation.GetBoneLocation(i, &xPosition, &xOrientation, true))
			{
				Asura_DebugLineRender::AddBoundingBox(*pxSkinBB, xPosition, xOrientation, 0x80FF00FF);
			}
		}
	}
}

/// End of function 'Asura_Shadow_Instance_Animation::RenderDebugInfo'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance_Shape::RenderDebugInfo
// Author       : Andy Tate
// Date         : 1-Dec-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance_Shape::RenderDebugInfo (const Asura_Vector_3& xCentre,
												   const float fRadius,
												   const u_int uLevel) const
{
	PARENT::RenderDebugInfo(xCentre, fRadius, uLevel);

	if((Asura_Shadow_System::GetConfig().m_iDebugLevel > 2) && (uLevel > 2))
	{
		const Asura_Bounding_Box&	xShapeBB		= m_xShapeInstance.GetBB();
		const Asura_Vector_3&		xPosition		= Asura_Maths::ZeroVector3;			//m_xShapeInstance.GetPosition();
		const Asura_Matrix_3x3&		xOrientation	= Asura_Maths::IdentityMatrix3x3;	//m_xShapeInstance.GetOrientation();

		Asura_DebugLineRender::AddBoundingBox(xShapeBB, xPosition, xOrientation, 0x40FFFF00);
	}
}

/// End of function 'Asura_Shadow_Instance_Shape::RenderDebugInfo'. ///

#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance::CalculateFrustum
// Author       : Andy Tate
// Date         : 28-Nov-2006
////////////////////////////////////////////////////////////////////////////////
DEBUGVAR float dbg_fShadowStrengthMultiplier = 1.5f;
bool Asura_Shadow_Instance::CalculateFrustum (const Asura_Vector_3& xCentre,
											  const float fRadius,
											  const Asura_Vector_3& xLightPosition,
											  const float fLightRange)
{

	// DMR - 27/03/07
	// We now pass a light position and range into this function, instead of trying to make one up one here
	// We use the light range to set the far plane - as this is the distance at which we want the shadow to
	// be completely faded out. This still needs some improvement, as we want the shadow strength to be
	// at it's maximum at the shadow camera pos, not at it's near plane, as it is now.

	if(fRadius > 0.0f)
	{				
		Asura_Matrix_3x3	xOrientation;		

		// Calculate the shadow direction and camera orientation/position
		m_xLightPosition	 = xLightPosition;
		Asura_Vector_3 xOfs = xCentre-m_xLightPosition;
		Asura_Maths::OrientationFromDirection(xOrientation, xOfs);
		SetLightOrientation(xOrientation);
		float fDistance = xOfs.Normalise();
		
		float fNear = fDistance - fRadius;

		// If the object is too close to the light (<0.5m for now), move the light away,
		// to avoid creating a frustum with an excessively wide FoV
		if (fNear<0.5f)
		{
			float fShift = 0.5f-fNear;
			m_xLightPosition += (xOfs * -fShift);
			fNear = 0.5f;			
			fDistance = fNear+fRadius;
		}


		if (fLightRange<fNear)
		{				
			return false;
		}
		else
		{
			// Calculate the shadow strength (for the near plane)
			if (m_eLightSourceType == ASURA_SHADOW_LIGHTSOURCE_AVERAGED)
			{
				m_fStrength = m_fLightSourceShadowStrength * dbg_fShadowStrengthMultiplier;
			}
			else
			{			
				m_fStrength = m_fLightSourceShadowStrength * (1.0f - (fNear/fLightRange));
			}

			// Calculate the frustum data
			m_xFrustum.SetForSphere(fNear, fLightRange, fDistance, fRadius);
			
			return true;
		}
	}
	else
	{
		return(false);
	}
}

/// End of function 'Asura_Shadow_Instance::CalculateFrustum'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance::CalculateDistance
// Author       : Andy Tate
// Date         : 29-Nov-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance::CalculateDistance (const Asura_Vector_3 &xCentre,
											   const float fRadius,
											   const Asura_Vector_3& xLightPosition, 
										   	   const float fLightRange)
{
	// Calculate the frustum...
	if(CalculateFrustum(xCentre, fRadius, xLightPosition, fLightRange))
	{
		Asura_Quat		xInverseLightOrientation(m_xLightOrientation);
		Asura_Vector_3	xCameraPositionInLightSpace;

		// Transform the camera into frustum-space...
		xInverseLightOrientation.Invert();
		xCameraPositionInLightSpace = xInverseLightOrientation.RotateVector((s_xCameraPosition - m_xLightPosition));

		// Does this intersect with the camera?
		if(m_xFrustum.Contains(xCameraPositionInLightSpace))
		{
			m_fDistance	= 0.0f;
#if !defined(ASURA_NO_DEBUG_OUTPUT)
			RenderDebugInfo(xCentre, fRadius, 4);
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)
		}
		else
		{
			Asura_Quat		xLightOrientationInCameraSpace;
			Asura_Vector_3	xLightPositionInCameraSpace;

			xLightOrientationInCameraSpace	= s_xCameraInverseOrientation * m_xLightOrientation;
			xLightPositionInCameraSpace = s_xCameraInverseOrientation.RotateVector((m_xLightPosition - s_xCameraPosition));

			if(s_xCameraFrustum.Intersects(m_xFrustum, xLightPositionInCameraSpace, xLightOrientationInCameraSpace))
			{
				Asura_Vector_3	xDifference(xCentre - s_xCameraPosition);

				m_fDistance	= xDifference.Magnitude() - fRadius;
				m_fDistance	= Asura_Maths::Max(0.0f, m_fDistance);

#if !defined(ASURA_NO_DEBUG_OUTPUT)
				// Debug render...
				RenderDebugInfo(xCentre, fRadius, 3);
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)
			}
			else
			{
#if !defined(ASURA_NO_DEBUG_OUTPUT)
				RenderDebugInfo(xCentre, fRadius, 1);
#endif	// !defined(ASURA_NO_DEBUG_OUTPUT)
				InvalidateDistance();
			}
		}
	}
	else
	{
		InvalidateDistance();
	}
}

/// End of function 'Asura_Shadow_Instance::CalculateDistance'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance::Asura_Shadow_Instance
// Author       : Andy Tate
// Date         : 29-Nov-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Shadow_Instance::Asura_Shadow_Instance (const Asura_Vector_3& xLightDirection, bool bCubeMap /*= false*/, bool bEnvironment /* = false*/ )
: m_xLightDirection(xLightDirection), m_uFlags((bCubeMap?IS_CUBE_MAP:0)|(bEnvironment?IS_ENVIRONMENT:0))
{
	// Nothing more to initialise...
	InvalidateDistance();


}

/// End of function 'Asura_Shadow_Instance::Asura_Shadow_Instance'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance::Asura_Shadow_Instance
// Author       : Andy Tate
// Date         : 31-Jan-2007
////////////////////////////////////////////////////////////////////////////////

Asura_Shadow_Instance::Asura_Shadow_Instance (const Asura_Shadow& xShadow)
: m_xLightDirection(xShadow.GetLightDirection()), m_uFlags(0)
{
	// Nothing more to initialise...
	InvalidateDistance();
}

/// End of function 'Asura_Shadow_Instance::Asura_Shadow_Instance'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance::SetupCamera
// Author       : Andy Tate
// Date         : 29-Nov-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance::SetupCamera (const u_int uCameraIndex,
										 const float fCutoffDistance)
{
	const Asura_CameraData&	xCameraData	= Asura_Camera::GetCameraData(uCameraIndex);
	const float				fNearClip	= 0.100f;

	// Set-up the camera position/orientation...
	s_xCameraPosition	= xCameraData.m_xPosition;
	s_xCameraOrientation.FromMatrix(xCameraData.m_xOrientation);
	s_xCameraInverseOrientation	= s_xCameraOrientation;
	s_xCameraInverseOrientation.Invert();

#if 0
	{
		const float	fTanFovX	= Asura_Maths::Tangent(xCameraData.m_xHalfFieldOfViewX * 0.500f);
		const float	fTanFovY	= Asura_Maths::Tangent(xCameraData.m_xHalfFieldOfViewY * 0.500f);

		// Use half of the camera-frustum - for debug purposes...
		s_xCameraFrustum.SetForFOV(fNearClip, fCutoffDistance, fTanFovX, fTanFovY);
	}
#else
	// Set-up the camera-frustum...
	s_xCameraFrustum.SetForFOV(fNearClip, fCutoffDistance, xCameraData.m_fTanHalfFovX, xCameraData.m_fTanHalfFovY);
#endif
}

/// End of function 'Asura_Shadow_Instance::SetupCamera'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance_Animation::CalculateDistance
// Author       : Andy Tate
// Date         : 28-Nov-2006
////////////////////////////////////////////////////////////////////////////////
DEBUGVAR float dbg_fAveragedPosDistanceMultiplier = 4.f;
DEBUGVAR float dbg_fAveragedPosRangeMultiplier = 2.f;
void Asura_Shadow_Instance_Animation::CalculateDistance (void)
{
	const Asura_Animation*	pxParentAnimation	= m_xAnimation.GetParent();

	if(pxParentAnimation == NULL)
	{
		// DMR - 27/03/07
		//
		// Make up a fake light position if we don't have a real one (if the shadow isn't coming from a single light)
		// We can't do this using the centre of the bounding sphere, or it causes nasty shadow popping if an anim makes
		// the bounding sphere change!
		Asura_Vector_3 xLightPos;		
		float          fLightRange;

		const Asura_Vector_3& xBSCentre = m_xAnimation.GetBoundingSphereCentre();
		const float fBSRadius = m_xAnimation.GetTightBoundingSphereRadius();

		if (m_eLightSourceType == ASURA_SHADOW_LIGHTSOURCE_AVERAGED)
		{
			// KOR - 12/2/2009 - ensure that the light is a reasonable distance from the bounding sphere
			// Otherwise we end up with a ridiculously wide frustum.
			const float fLightDistance = dbg_fAveragedPosDistanceMultiplier * fBSRadius;
			xLightPos   = m_xAnimation.GetPosition() + (m_xLightDirection * fLightDistance);
			// Also ensure range is long enough.
			fLightRange = fLightDistance + fBSRadius * dbg_fAveragedPosRangeMultiplier;
		}
		else
		{
			m_xLightDirection = m_xLightSourcePosition - m_xAnimation.GetPosition();
			m_xLightDirection.Normalise();
			xLightPos   = m_xLightSourcePosition;			
			fLightRange = m_fLightSourceRange;
		}
		Asura_Shadow_Instance::CalculateDistance(xBSCentre, fBSRadius, xLightPos, fLightRange);
	}
	else
	{
		InvalidateDistance();
	}
}

/// End of function 'Asura_Shadow_Instance_Animation::CalculateDistance'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance_Animation::Render
// Author       : Andy Tate
// Date         : 28-Nov-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance_Animation::Render (Asura_Shadow& xShadow) const
{
	const bool			bFirstUse	= true;
	
	Asura_Vector_3		xPosition;
	Asura_Matrix_3x3	xOrientation;

	// Apply the movement offset...
	if(m_bUseMovementOffset)
	{
		Asura_Quat		xQuat(m_xAnimation.GetOrientQuat());
		Asura_Vector_3	xPositionOffset;
		Asura_Quat		xOrientationOffset;

		if(m_xAnimation.GetMovementOffset(&xPositionOffset, &xOrientationOffset))
		{
			xPosition = xQuat.RotateVector(xPositionOffset);
			xPosition	+= m_xAnimation.GetPosition();
			xQuat		 = xQuat * xOrientationOffset;
		}
		else
		{
			xQuat.ToMatrix(xOrientation);
			xPosition	= m_xAnimation.GetPosition();
		}
	}
	else
	{
		m_xAnimation.GetOrientQuat().ToMatrix(xOrientation);
		xPosition	= m_xAnimation.GetPosition();
	}

	// Initialisation...
	if(bFirstUse)
	{
		xShadow.PrepareForRendering();
	}

	// Render the shadows (including attachments)...
	Render(xShadow, xPosition, xOrientation);

	for(const Asura_AnimationAttachment* pxAttachment = m_xAnimation.GetFirstAttachment(); pxAttachment; pxAttachment = pxAttachment->GetNextAttachment())
	{
		pxAttachment->RenderShadow(xShadow, m_uRenderFlags);
	}

	// Shutdown...
	if(bFirstUse)
	{
		xShadow.FinishedRendering();
	}
}

/// End of function 'Asura_Shadow_Instance_Animation::Render'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance_Animation::Render
// Author       : Andy Tate
// Date         : 31-Jan-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance_Animation::Render (Asura_Shadow& xShadow,
											  const Asura_Vector_3& xPosition,
											  const Asura_Matrix_3x3& xOrientation) const
{
	Asura_Animation*	pxAnimation	= const_cast<Asura_Animation*>(&m_xAnimation);

	Asura_Hierarchy_System::RenderShadow(*pxAnimation, xShadow, xPosition, xOrientation, m_uRenderFlags);
}

/// End of function 'Asura_Shadow_Instance_Animation::Render'. ///


#if defined(ASURA_TARGET_SM3)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance_Environment::Render
// Author       : Andy Southgate
// Date         : 30-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance_Environment::Render (Asura_Shadow& xShadow) const
{
	const bool			bFirstUse	= true;

	// Initialisation...
	if(bFirstUse)
	{
		xShadow.PrepareForRendering();
	}

 	Asura_Shadow_System::RenderEnvironmentShadowMap(xShadow);

	// Shutdown...
	if(bFirstUse)
	{
		xShadow.FinishedRendering();
	}
}

/// End of function 'Asura_Shadow_Instance_Environment::Render'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance_Environment::CalculateDistance
// Author       : Andy Southgate
// Date         : 30-Jan-2008

////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance_Environment::CalculateDistance (void)
{
	if (IsCubeMapped())
	{
		Asura_Vector_3	xCameraPositionInLightSpace(s_xCameraPosition - m_xLightPosition);
		float fFarClip = m_xFrustum.GetFarPlane();

		// is the camera inside the shadow volume
		if (xCameraPositionInLightSpace.MagnitudeSquared() < fFarClip)
		{
			m_fDistance	= -1.0f;
		} 
		else 
		{
			Asura_Vector_3	xLightPositionInCameraSpace;

			xLightPositionInCameraSpace = s_xCameraInverseOrientation.RotateVector(m_xLightPosition - s_xCameraPosition);

			if(s_xCameraFrustum.Intersects(xLightPositionInCameraSpace, fFarClip))
			{
				m_fDistance	= xCameraPositionInLightSpace.Magnitude() - fFarClip;
				m_fDistance	= Asura_Maths::Max(0.0f, m_fDistance);
			} 
			else 
			{
				InvalidateDistance();
			}

		}
	} else {
		Asura_Quat		xInverseLightOrientation(m_xLightOrientation);
		Asura_Vector_3	xCameraPositionInLightSpace;

		// Transform the camera into frustum-space...
		xInverseLightOrientation.Invert();
		xCameraPositionInLightSpace = xInverseLightOrientation.RotateVector((s_xCameraPosition - m_xLightPosition));

		// Does this intersect with the camera?
		if(m_xFrustum.Contains(xCameraPositionInLightSpace))
		{
			m_fDistance	= -1.0f;
		}
		else
		{
			Asura_Quat		xLightOrientationInCameraSpace;
			Asura_Vector_3	xLightPositionInCameraSpace;

			xLightOrientationInCameraSpace	= s_xCameraInverseOrientation * m_xLightOrientation;
			xLightPositionInCameraSpace = s_xCameraInverseOrientation.RotateVector((m_xLightPosition - s_xCameraPosition));

			if(s_xCameraFrustum.Intersects(m_xFrustum, xLightPositionInCameraSpace, xLightOrientationInCameraSpace))
			{
				Asura_Vector_3	xDifference(m_xLightPosition - s_xCameraPosition);

				m_fDistance	= xDifference.Magnitude() - m_xFrustum.GetFarPlane();
				m_fDistance	= Asura_Maths::Max(0.0f, m_fDistance);
			}
			else
			{
				InvalidateDistance();
			}
		}
	}

}

/// End of function 'Asura_Shadow_Instance_Shape::CalculateDistance'. ///

bool Asura_Shadow_Instance_Environment::CalculateFrustum(float fTanHalfHFov,float fTanHalfVFov, float fNearDist, float fFarDist)
{
	return true;
}

#endif	//defined(ASURA_TARGET_SM3)


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance_Shape::CalculateDistance
// Author       : Andy Tate
// Date         : 28-Nov-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance_Shape::CalculateDistance (void)
{
	const Asura_Shape*	pxShape	= m_xShapeInstance.GetShape();

	if(pxShape == NULL)
	{
		InvalidateDistance();
	}
	else
	{
		const Asura_Bounding_Box&	xBB	= m_xShapeInstance.GetBB();

		// DMR - 27/03/07
		//
		// Make up a fake light position if we don't have a real one (if the shadow isn't coming from a single light)
		// We can't do this using the centre of the bounding sphere, or it causes nasty shadow popping if an anim makes
		// the bounding sphere change!

		Asura_Vector_3 xLightPos;		
		float          fLightRange;
		Asura_Vector_3 xPos = xBB.CalculateCentre();

		if (m_eLightSourceType == ASURA_SHADOW_LIGHTSOURCE_AVERAGED)
		{								
			xLightPos   = m_xShapeInstance.GetPosition() + (m_xLightDirection * 5.0f);
			fLightRange = 10.0f;
		}
		else
		{			
			m_xLightDirection = m_xLightSourcePosition - m_xShapeInstance.GetPosition();
			m_xLightDirection.Normalise();
			xLightPos   = m_xLightSourcePosition;			
			fLightRange = m_fLightSourceRange;
		}

		Asura_Shadow_Instance::CalculateDistance(xPos, xBB.CalculateRadius(), xLightPos, fLightRange);
	}
}

/// End of function 'Asura_Shadow_Instance_Shape::CalculateDistance'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance_Shape::Render
// Author       : Andy Tate
// Date         : 29-Nov-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance_Shape::Render (Asura_Shadow &xShadow) const
{
	const bool	bFirstUse	= true;

	// Initialisation...
	if(bFirstUse)
	{
		xShadow.PrepareForRendering();
	}

	// Render the shadow...
	Render(xShadow, Asura_Maths::ZeroVector3, Asura_Maths::IdentityMatrix3x3);

	if(bFirstUse)
	{
		xShadow.FinishedRendering();
	}
}

/// End of function 'Asura_Shadow_Instance_Shape::Render'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Shadow_Instance_Shape::Render
// Author       : Andy Tate
// Date         : 31-Jan-2007
////////////////////////////////////////////////////////////////////////////////

void Asura_Shadow_Instance_Shape::Render (Asura_Shadow& xShadow,
										  const Asura_Vector_3& xPosition,
										  const Asura_Matrix_3x3& xOrientation) const
{
	const Asura_Shape*	pxShape	= m_xShapeInstance.GetShape();

	// Render the shadow...
	pxShape->Platform_RenderShadow(&m_xShapeInstance, xShadow, m_uCamVisFlags);
}

/// End of function 'Asura_Shadow_Instance_Shape::Render'. ///


/// End of file. ///
