// Asura_Audio_DSP.h
// Thomas Young / Andrew McDonald - 17.03.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

// F I L E   W R A P P E R ///////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_AUDIO_DSP_H__
#define __ASURA_AUDIO_DSP_H__

#ifdef ASURA_USE_AUDIO_SYSTEM

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura_String.h"

// D E F I N E S /////////////////////////////////////////////////////////////

#define ASURA_DSP_MAX_EFFECTS_PER_CHAIN		5

// This macro assumes you have a 'Params' structure
#define ASURA_DSP_USE_DEFAULT_PARAMFUNCS( classname )	public:		const void* GetParams	( ) const { return &m_xParams;			}											\
																	u_int GetParamSize		( ) const { return sizeof( Params );	}											\
																																												\
																	virtual Asura_DSP_Params& operator=	( const Asura_DSP_Params& xRHS )										\
																	{																											\
																		Asura_Mem::Copy( &m_xParams, &reinterpret_cast<const classname*>(&xRHS)->m_xParams, sizeof(Params) );	\
																		return *this;																							\
																	}																											\
																																												\
																	Params m_xParams;																							\
																
// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Chunk_Stream;
class Asura_DSP_Params;

// E N U M E R A T I O N S ///////////////////////////////////////////////////

enum Asura_Audio_DSPType
{
	ASURA_DSP_SIMPLE_FILTER		= 0,
	ASURA_DSP_ADD_NOISE			= 1,
	ASURA_DSP_FLANGER			= 2,
	ASURA_DSP_CHORUS			= 3,

	ASURA_DSP_NUM_TYPES
};

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

// ___________________________ Asura_Audio_DSPChain ________________________________

class Asura_Audio_DSPChain
{
	// This class holds a list of Asura_DSP_Params

	public:

		// _______________ Functions ___________________

		Asura_Audio_DSPChain	( );
		~Asura_Audio_DSPChain	( );

		void AddEffect			( const Asura_Audio_DSPType eType );
		void MoveEffectUp		( const u_int uIndex );
		void DeleteEffect		( const u_int uIndex );
		void Clear				( );

		Asura_Audio_DSPChain& operator=	( const Asura_Audio_DSPChain& xRhs );

		// _______________ Variables ___________________

		u_int					m_uNumEffects;
		Asura_DSP_Params**		m_ppxEffects;
		Asura_String			m_xName;

};

// ___________________________ Asura_DSP_Params ________________________________

class Asura_DSP_Params
{
	// This is the base class for DSP params

	public:

		Asura_DSP_Params			( )	{};
		virtual ~Asura_DSP_Params	( )	{};

		static Asura_DSP_Params*	CreateEffect			( const Asura_Audio_DSPType eType );

		virtual Asura_Audio_DSPType	GetType					( ) const = 0;
		
		virtual const void*			GetParams				( ) const = 0;
		virtual u_int				GetParamSize			( ) const = 0;

		virtual void				WriteToChunkStream		( Asura_Chunk_Stream &xStream ) const = 0;
		virtual void				ReadFromChunkStream		( Asura_Chunk_Stream &xStream ) = 0;

		virtual Asura_DSP_Params& operator=	( const Asura_DSP_Params& xRHS ) = 0;

};

// ___________________________ Asura_DSP_Params_SimpleFilter ________________________________

class Asura_DSP_Params_SimpleFilter : public Asura_DSP_Params
{
	public:

		static const u_int uFILTER_NUM_COEFFICIENTS		= 25;
		static const u_int uREMEZ_GRID_DENSITY			= 16;
		static const u_int uREMEZ_MAX_ITERATIONS		= 40;
		static const u_int uCHUNK_VERSION				= 0;

		enum FILTER_TYPE
		{
			FILTER_TYPE_LOWPASS,
			FILTER_TYPE_HIGHPASS,
			FILTER_TYPE_BANDPASS,
			FILTER_TYPE_BANDSTOP
		};

		Asura_DSP_Params_SimpleFilter();

		virtual void WriteToChunkStream		( Asura_Chunk_Stream &xStream ) const;
		virtual void ReadFromChunkStream	( Asura_Chunk_Stream &xStream );

		virtual Asura_Audio_DSPType	GetType () const { return ASURA_DSP_SIMPLE_FILTER; }

		struct Params
		{
			u_int m_uType;
			float m_fFrequency;		// Hz
			float m_fFrequency2;	// only used for bandpass/stop
			float m_fAttenuation;
			float m_fGain;
			bool m_bValid;

			float m_afCoefficients[ uFILTER_NUM_COEFFICIENTS ];	// Precalced filter coefficients
		};

		#ifdef ASURA_TOOLS
		void ReCalcCoefficients ();
		#endif

	private:

		#ifdef ASURA_TOOLS
		// Filter coefficient calculation functions...
		bool	RemezExchange			( double dCoeffs[], int iNumTaps, int iNumBands, double adFreqs[], double adGains[], double adWeights[] );
		bool	Remez_IsDone			( int iNumExtrema, int piExtrema[], double pdExtrema[] );
		void	Remez_FreqSample		( int iNumTaps, double pdTaps[], double pdCoeffs[] );
		void	Remez_Search			( int iNumExtrema, int piExtrema[], int iGridSize, double pdExtrema[] );
		void	Remez_CalcError			( int iNumExtrema, double pdParamD[], double pdParamX[], double pdParamY[], int iGridSize, double pdGrid[], double pdScratchD[], double pdScratchW[], double pdScratchE[] );
		double	Remez_ComputeA			( double dFreq, int iNumExtrema, double pdParamD[], double pdParamX[], double pdParamY[] );
		void	Remez_CalcParams		( int iNumExtrema, int piExtrema[], double pdGrid[], double pdParamD[], double pdParamW[], double pdParamAD[], double pdParamX[], double pdParamY[] );
		void	Remez_InitialGuess		( int iNumExtrema, int piExtrema[], int iGridsize );
		void	Remez_CreateDenseGrid	( int iNumExtrema, int iNumTaps, int iNumBands, double dBands[], double dDesired[], double dWeights[], int *piGridSize, double dGrid[], double pScratchD[], double pScratchW[] );
		#endif
		
	ASURA_DSP_USE_DEFAULT_PARAMFUNCS( Asura_DSP_Params_SimpleFilter )
};

// ___________________________ Asura_DSP_Params_AddNoise ________________________________

class Asura_DSP_Params_AddNoise : public Asura_DSP_Params
{
	public:

		static const u_int uCHUNK_VERSION = 0;

		Asura_DSP_Params_AddNoise();

		virtual void WriteToChunkStream		( Asura_Chunk_Stream &xStream ) const;
		virtual void ReadFromChunkStream	( Asura_Chunk_Stream &xStream );

		virtual Asura_Audio_DSPType GetType () const { return ASURA_DSP_ADD_NOISE; }

		struct Params
		{
			float m_fNoiseAmount;
		};

	ASURA_DSP_USE_DEFAULT_PARAMFUNCS( Asura_DSP_Params_AddNoise )

};

// ___________________________ Asura_DSP_Params_Flanger ________________________________

class Asura_DSP_Params_Flanger : public Asura_DSP_Params
{
public:

	static const u_int uCHUNK_VERSION = 0;

	Asura_DSP_Params_Flanger();

	virtual void WriteToChunkStream		( Asura_Chunk_Stream &xStream ) const;
	virtual void ReadFromChunkStream	( Asura_Chunk_Stream &xStream );

	virtual Asura_Audio_DSPType GetType () const { return ASURA_DSP_FLANGER; }

	struct Params
	{
		float m_fRange;		// maximum distance between samples
		float m_fWetDry;	// Wet/Dry mix from zero (original signal) to one (fully DSP'd)
		float m_fGain;		// Gain adjustment
	};

	ASURA_DSP_USE_DEFAULT_PARAMFUNCS( Asura_DSP_Params_Flanger )

};

// ___________________________ Asura_DSP_Params_Flanger ________________________________

class Asura_DSP_Params_Chorus : public Asura_DSP_Params
{
public:

	static const u_int uCHUNK_VERSION = 0;

	Asura_DSP_Params_Chorus();

	virtual void WriteToChunkStream		( Asura_Chunk_Stream &xStream ) const;
	virtual void ReadFromChunkStream	( Asura_Chunk_Stream &xStream );

	virtual Asura_Audio_DSPType GetType () const { return ASURA_DSP_CHORUS; }

	struct Params
	{
		float m_fDelay;			// delay time
		float m_fSweepDepth;	// LFO modulation amplitude
		float m_fWetDry;		// Wet/Dry mix from zero (original signal) to one (fully DSP'd)
		float m_fGain;			// Gain adjustment
	};

	ASURA_DSP_USE_DEFAULT_PARAMFUNCS( Asura_DSP_Params_Chorus )

};

#endif // defined ASURA_USE_AUDIO_SYSTEM

#endif // !defined __ASURA_AUDIO_DSP_H__

//eof
