//
// Asura_MetaMusic_Zone.cpp
//
// Created by Richard Rice - 11/01/05
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

#include "Sound/Asura_MetaMusic_Zone.h"

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Triggers/Asura_ServerEntity_MetaMusicTrigger.h"
#include "Sound/Asura_MetaMusic_System.h"


#define ASURA_METAMUSIC_ZONE_VERSION 0 
#define ASURA_METAMUSIC_ZONE_INGAME_VERSION 0

void Asura_MetaMusic_Zone_Trigger::Initialise()
{
	m_iNumNPCsToTrigger = 0;
	m_fMinimumTime= 0;
	m_uTriggerFlags = 0;
	m_uTrackID = 0;
}

bool Asura_MetaMusic_Zone_Trigger::IsUsed() const
{
	const u_int uFlagsToCheck =	ASURA_METAMUSICTRIGGER_FLAG_AMBIENT 
							  | ASURA_METAMUSICTRIGGER_FLAG_TENSION
							  | ASURA_METAMUSICTRIGGER_FLAG_ACTION
							  | ASURA_METAMUSICTRIGGER_FLAG_CHANGETRACK
							  | ASURA_METAMUSICTRIGGER_FLAG_RESTARTLASTTRACK;
								
	return (m_uTriggerFlags & uFlagsToCheck) ? true : false;
}

// if we're creating from a chunk buffer we do not need to supply a new zone id
// as it will have been set already
Asura_MetaMusic_Zone::Asura_MetaMusic_Zone( const bool bCreatingFromChunkBuffer )
: Asura_Zone( bCreatingFromChunkBuffer )
{
	if(!bCreatingFromChunkBuffer)
	{
		m_axTrigger[0].Initialise();
		m_axTrigger[1].Initialise();
	}
	m_uMetaMusicFlags = ASURA_METAMUSICZONE_FLAG_ACTIVE;
}


void Asura_MetaMusic_Zone::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	Asura_Zone::ReadFromChunkStream( xStream );

	int iVersion;
	xStream >> iVersion;
	Asura_Assert(iVersion<=ASURA_METAMUSIC_ZONE_VERSION,"Unknown chunk version in Asura_MetaMusic_Zone::ReadFromChunkStream");
	
	xStream >> m_uMetaMusicFlags;

	for(int i=0;i<2;i++)
	{
		xStream >> m_axTrigger[i].m_iNumNPCsToTrigger;
		xStream >> m_axTrigger[i].m_fMinimumTime;
		xStream >> m_axTrigger[i].m_uTriggerFlags;
		xStream >> m_axTrigger[i].m_uTrackID;
	}

}


#ifdef ASURA_TOOLS
void Asura_MetaMusic_Zone::WriteToChunkStream( Asura_Chunk_Stream& xStream )
{
	Asura_Zone::WriteToChunkStream( xStream );

	xStream << (int)ASURA_METAMUSIC_ZONE_VERSION;

	xStream << m_uMetaMusicFlags;

	for(int i=0;i<2;i++)
	{
		xStream << m_axTrigger[i].m_iNumNPCsToTrigger;
		xStream << m_axTrigger[i].m_fMinimumTime;
		xStream << m_axTrigger[i].m_uTriggerFlags;
		xStream << m_axTrigger[i].m_uTrackID;
	}
}

#endif //ASURA_TOOLS

void Asura_MetaMusic_Zone::ReadInGameDataFromChunkStream( Asura_Chunk_Stream& xStream )
{

	int iVersion;
	xStream >> iVersion;
	Asura_Assert(iVersion<=ASURA_METAMUSIC_ZONE_INGAME_VERSION,"Unknown chunk version in Asura_MetaMusic_Zone::ReadInGameDataFromChunkStream");
	
	xStream >> m_uMetaMusicFlags;
}


void Asura_MetaMusic_Zone::WriteInGameDataToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << (int)ASURA_METAMUSIC_ZONE_INGAME_VERSION;

	//Only the flags are in danger of being changed at the moment
	xStream << m_uMetaMusicFlags;
}



int Asura_MetaMusic_Zone::GetTriggerLevel(int iNumNPCs,int iPrevLevel,float fTimer) const
{
	for(int iLevel=2;iLevel>0;iLevel--)
	{
		const Asura_MetaMusic_Zone_Trigger* pxTrigger = &m_axTrigger[iLevel-1];
		if(!pxTrigger->IsUsed()) continue;//ignore triggers that don't do anything

		if(iNumNPCs >= pxTrigger->m_iNumNPCsToTrigger)
		{
			//got enough NPCs , so go with this one
			return iLevel;
		}
		else if(iPrevLevel == iLevel)
		{
			//not enough NPCs , but was previously at this level.
			//Has the minimum time been reached yet?
			if(fTimer<pxTrigger->m_fMinimumTime) return iLevel;
		}
		else if(iPrevLevel > iLevel)
		{
			//Was previously at a higher level ,and are now dropping down.
			//Go for this one if it has a minimum time
			if(pxTrigger->m_fMinimumTime) return iLevel;
		}
	}
	//Not triggered any more.
	return 0;
}

void Asura_MetaMusic_Zone::Trigger(int iTriggerLevel) const
{
	Asura_Assert(iTriggerLevel>=1 && iTriggerLevel<=2,"");

	const Asura_MetaMusic_Zone_Trigger* pxTrigger = &m_axTrigger[iTriggerLevel-1];


    if ( pxTrigger->m_uTriggerFlags & ASURA_METAMUSICTRIGGER_FLAG_CHANGETRACK)
    {
        if ( pxTrigger->m_uTrackID == ASURA_HASH_ID_UNSET )
        {
            Asura_MetaMusic_System::StopTrack( true );
        }
        else
        {
            Asura_MetaMusic_System::PlayTrack( pxTrigger->m_uTrackID, true );
        }
    }
    else if ( pxTrigger->m_uTriggerFlags & ASURA_METAMUSICTRIGGER_FLAG_RESTARTLASTTRACK )
    {
        Asura_MetaMusic_System::RestartLastTrack();
    }

    if ( pxTrigger->m_uTriggerFlags & ASURA_METAMUSICTRIGGER_FLAG_AMBIENT)
    {
        Asura_MetaMusic_System::ChangeState( ASURA_METAMUSIC_AMBIENT );
    }
    else if ( pxTrigger->m_uTriggerFlags & ASURA_METAMUSICTRIGGER_FLAG_TENSION )
    {
        Asura_MetaMusic_System::ChangeState( ASURA_METAMUSIC_TENSION );
    }
    else if ( pxTrigger->m_uTriggerFlags & ASURA_METAMUSICTRIGGER_FLAG_ACTION )
    {
        Asura_MetaMusic_System::ChangeState( ASURA_METAMUSIC_ACTION );
    }
}

void Asura_MetaMusic_Zone::SetActive(bool bSet)
{
	if(bSet)
		m_uMetaMusicFlags |= ASURA_METAMUSICZONE_FLAG_ACTIVE;
	else
		m_uMetaMusicFlags &=~ ASURA_METAMUSICZONE_FLAG_ACTIVE;

}

#endif

//eof
