//////////////////////////////////////////////////////////////////////////////
//
// Asura_Music.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_MUSIC_H___
#define ___ASURA_MUSIC_H___ 1

#if !defined(ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM)

#if defined(ASURA_USE_AUDIO_SYSTEM)
#error "Asura_Music_System" only works with "Asura_Sound_System" - "ASURA_USE_AUDIO_SYSTEM" should not be defined
#endif	//defined(ASURA_USE_AUDIO_SYSTEM)

#include "Core/Asura_Hash.h"
#include "Sound/Asura_Project_Music.h"

class Asura_Music_System
{
public:

    static void Initialise();
    static void Shutdown();
    static void Update();

    //Pause and Unpause from Asura_MetaMusic_System should be used instead of there two , 
	//since they will update the music system correctly even if meta music isn't being used.
	//RAR 19/08/2005
    static void Pause(bool bInstant);
    static void Unpause(bool bInstant);

    static void Play(Asura_Music_TrackID eTrack,bool bLoop = true, float fCrossfadeTime = 0.5f);
	static void Play(Asura_Music_TrackID eTrack,float fVolume,bool bLoop, float fCrossfadeTime);
	static void Play(const char* pszFilename,float fVolume, bool bLoop, float fCrossfadeTime);
	
	static bool Prepare (const char* pszFilename);
	static bool Prepare (Asura_Music_TrackID eTrack);

	static void ClearPreparedTrack ();

    static void Stop(float fFadeoutTime = 0.5f);

	//Stops the secondary track immediately , releasing its sound handle
	//(No fade option given since the secondary track would already be fading out anyway , so
	//the only point is for an immedite stop).
	//Returns true if a sound handle was released in the process.
	static bool StopSecondary();

    static Asura_Music_TrackID GetCurrentTrack();
    static void GetCurrentTrackData( Asura_Music_TrackID& eTrackId, bool& bLoop );
	static Asura_Hash_ID GetCurrentTrackHash();//get the hashed filename of the current track
	static float GetCurrentTrackPosition();//get the position (in seconds) of the current track
	static float GetCurrentTrackLength();//get the length (in seconds) of the current track

	static bool IsCurrentTrackFading();//returns true if a track is playing , and it is fading in/out or crossfading
	static bool IsCurrentTrackLooping();//returns true if there is a primary track , and it is being played as looping

	static bool IsSecondaryTrackFading(); //returns true if a secondary track is playing , and it is fading in/out or crossfading

	static int GetNumberOfSoundHandlesUsed();//returns number of streaming sound handles that music system currently has

	static float GetCurrentTrackVolumeBias()
	{
		return s_fCurrentTrackVolumeBias;
	};
	static void SetCurrentTrackVolumeBias(const float fVolBias);

	#ifdef ASURA_TOOLS
	static const char* GetTrackName(int iIndex);
	#endif

	#if defined( ASURA_USE_DEBUGOVERLAY )
	static void DoMusicDebugOverlay ( void* const pxIgnore );
	#endif

private:

    class ActiveTrack;

    friend class ActiveTrack;
    
    // defined in Asura_Project_Music.cpp
    static const char* const s_szPath;
    static const char* const s_aszFilenames[ASURA_NUM_MUSIC_TRACKS];

	#ifdef ASURA_TOOLS
    static const char* const s_aszNames[ASURA_NUM_MUSIC_TRACKS];
	#endif

    // internal state
    static u_int s_uPrimaryTrack;
    static ActiveTrack* s_axActiveTracks;
	static ActiveTrack* s_pxPreparedTrack;
	static float s_fCurrentTrackVolumeBias;	// Volume multiplier applied to PrimaryTrack. Used to fade it up/down using SetCurrentTrackVolumeBias() in your project.
	static float s_fMixerVolume; //Volume multiplier from the music mixer , if it exists.
};

//Message sent by the music trigger entity to start playing a track. RAR 18/03/03
class Asura_MsgData_MusicSystem_Play
{
public :
	float m_fFadeTime;
	unsigned char m_ucMusicTrack;
	unsigned char m_ucLooping;
	unsigned char m_ucVolume;//volume in range 0 to 255
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif	//!defined(ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM)

#endif // #ifndef ___ASURA_MUSIC_H___
