//
// Asura_ReverbVolume_System.cpp
//
// Created by Richard Rice - 10/10/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Sound/Asura_ReverbVolume_System.h"

#ifdef ASURA_USE_REVERBVOLUME_SYSTEM

#include "Camera/Asura_Camera.h"
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_ReverbVolume.h"
#include "Sound/Asura_Sound.h"

/////////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Asura_ReverbVolume_Handler **
**----------------------------*/

Asura_ReverbVolume_Handler::Asura_ReverbVolume_Handler() :
	m_uZoneHashID(ASURA_HASH_ID_UNSET)
{
#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	Initialise();
#endif
}

Asura_ReverbVolume_Handler::~Asura_ReverbVolume_Handler()
{
#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	Shutdown();
#endif
}

void Asura_ReverbVolume_Handler::Initialise()
{
	//Asura_ResourcePushType(ASURA_RESOURCE_REVERBVOLUME);
	{
		m_pxReverbVolumeController = new Asura_ReverbVolume_Controller;
	}
	//Asura_ResourcePopType();
}


void Asura_ReverbVolume_Handler::Shutdown()
{
	//Asura_ResourcePushType(ASURA_RESOURCE_REVERBVOLUME);
	{
		delete m_pxReverbVolumeController;
		m_pxReverbVolumeController = 0;
	}
	//Asura_ResourcePopType();
}

void Asura_ReverbVolume_Handler::Reset()
{
	if (m_pxReverbVolumeController)
	{
		m_pxReverbVolumeController->Reset();
	}
}

const Asura_ReverbVolume* Asura_ReverbVolume_Handler::GetReverbVolumeFromPosition(const Asura_Vector_3& xPosition)
{
	Asura_Assert(m_pxReverbVolumeController,"");

	return (const Asura_ReverbVolume*) m_pxReverbVolumeController->GetZoneFromPosition(xPosition);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Asura_ReverbVolume_System  **
**----------------------------*/

#if !defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
Asura_ReverbVolume_Handler Asura_ReverbVolume_System::s_xHandler;
#else
Asura_ReverbVolume_Handler* Asura_ReverbVolume_System::s_pxCurrentHandler = NULL;
Asura_TemplateHashTree<Asura_ReverbVolume_Handler>	Asura_ReverbVolume_System::s_xHandlerTree;	// Tree of all of the handlers available.
#endif	

void Asura_ReverbVolume_System::Initialise()
{
	#if !defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	s_xHandler.Initialise();
	#endif
}

void Asura_ReverbVolume_System::Shutdown()
{
#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	// Reset the entire handler tree...
	s_xHandlerTree.Reset();

	// Remove the current handler...
	s_pxCurrentHandler	= NULL;
#else
	Asura_ReverbVolume_Handler& xHandler = GetCurrentHandler();

	// Reset the current instance...
	xHandler.Shutdown();
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)

}

void Asura_ReverbVolume_System::Reset()
{
#if !defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	s_xHandler.Reset();
#elif !defined(ASURA_USE_STREAMING)
	GetCurrentHandler().Reset();
#else
	Shutdown();
	Initialise();
#endif
}

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)

Asura_ReverbVolume_Handler* Asura_ReverbVolume_System::CreateHandler (const Asura_Hash_ID uHash)
{
	Asura_ReverbVolume_Handler* pxHandler = s_xHandlerTree.GetDataByHash(uHash);

	// Do we already have this handler?
	if(pxHandler == NULL)
	{
		// No - create one...
		pxHandler = s_xHandlerTree.AddNode(uHash);
		if (pxHandler)
		{
			pxHandler->SetZoneHashID(uHash);
		}
	}
	else
	{
		// Yes - this shouldn't happen...
		Asura_Assert(false, "Already have a reverb volume handler with this hash-ID.");
	}

	// Return the handler...
	return(pxHandler);
}

bool Asura_ReverbVolume_System::DestroyHandler (const Asura_Hash_ID uHash)
{
	// Is this the current handler?
	if(s_pxCurrentHandler == s_xHandlerTree.GetDataByHash(uHash))
	{
		s_pxCurrentHandler	= NULL;
	}

	// Call the delete function...
	return s_xHandlerTree.DeleteNodeByHash(uHash);
}
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)


void Asura_ReverbVolume_System::Update()
{
	//update the current reverb settings from the reverb volume
	//TBD - Streaming builds probably want to check all the handlers
	const Asura_ReverbVolume* pxReverbVolume = GetCurrentHandler().GetReverbVolumeFromPosition(*Asura_Camera::GetPosition());
	if(pxReverbVolume)
	{
		u_int uReverbType = pxReverbVolume->GetReverbType();
		float fReverbAmount = pxReverbVolume->GetReverbAmount();

		#if defined(ASURA_USE_AUDIO_SYSTEM)
			Asura_Audio_System::SetCurrentReverb(uReverbType, fReverbAmount);
		#else
			Asura_Sound_System::SetReverb((Asura_Sound_System::ReverbType) uReverbType, fReverbAmount);
		#endif
	}
}

#endif //ASURA_USE_REVERBVOLUME_SYSTEM
