//
// Asura_ReverbVolume_System.h
//
// Created by Richard Rice - 10/10/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _ASURA_REVERBVOLUME_SYSTEM_H_
#define _ASURA_REVERBVOLUME_SYSTEM_H_
#ifdef ASURA_USE_REVERBVOLUME_SYSTEM

#include "Core/Asura_HashTree.h"
#include "Core/Asura_Maths.h"

/////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_ReverbVolume_Controller;
class Asura_ReverbVolume;

class Asura_ReverbVolume_Handler
{
public:
	// Public functions...
	Asura_ReverbVolume_Handler();
	~Asura_ReverbVolume_Handler();

	void Initialise();
	void Reset();
	void Shutdown();

	const Asura_ReverbVolume* GetReverbVolumeFromPosition(const Asura_Vector_3& xPos);


	// Get zone hash of the handler, for streaming builds.
	Asura_Hash_ID GetZoneHashID() const { return m_uZoneHashID; }
	void SetZoneHashID(Asura_Hash_ID uZoneHashID) { m_uZoneHashID = uZoneHashID; }

	Asura_ReverbVolume_Controller* GetReverbVolumeController() {return m_pxReverbVolumeController;}

protected:

	Asura_Hash_ID m_uZoneHashID;

	//class that actually looks after all the volumes
	Asura_ReverbVolume_Controller* m_pxReverbVolumeController;	
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_ReverbVolume_System
{
public :
	static void Initialise();
	static void Shutdown();
	static void Reset();

	static void Update();

	static Asura_ReverbVolume_Controller* GetReverbVolumeController() {return GetCurrentHandler().GetReverbVolumeController();}

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	// Handler functions...
	static	Asura_ReverbVolume_Handler*	CreateHandler		(const Asura_Hash_ID uHash);
	static	bool						DestroyHandler		(const Asura_Hash_ID uHash);

	static	void						CreateAndSetHandler	(const Asura_Hash_ID uHash)				{ s_pxCurrentHandler = CreateHandler(uHash); }
	static	void						SetCurrentHandler	(Asura_ReverbVolume_Handler* pxHandler)	{ s_pxCurrentHandler = pxHandler; }
	static	void						SetCurrentHandler	(const Asura_Hash_ID uHash)				{ s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHash); }

	static	Asura_ReverbVolume_Handler&	GetCurrentHandler()	{ return *s_pxCurrentHandler; }
    
    static  Asura_ReverbVolume_Handler*	GetHandler (const Asura_Hash_ID uHandlerHash) { return(s_xHandlerTree.GetDataByHash(uHandlerHash)); }
#else
	// Handler functions...
	static	Asura_ReverbVolume_Handler&	GetCurrentHandler()	{ return s_xHandler; }
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)


protected :

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	// Protected variables...
	static	Asura_ReverbVolume_Handler* s_pxCurrentHandler;	// Currently active handler.
	static	Asura_TemplateHashTree<Asura_ReverbVolume_Handler>	s_xHandlerTree;	// Tree of all of the handlers available.
#else
	// Protected variables...
	static	Asura_ReverbVolume_Handler	s_xHandler;	// Single static-instance of a zone.
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)

};


#endif //ASURA_USE_REVERBVOLUME_SYSTEM
#endif //_ASURA_REVERBVOLUME_SYSTEM_H_

