//////////////////////////////////////////////////////////////////////////////
//
// Asura_Sound_Event_Storage.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     storage and lookups of sound events and related objects
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_SOUND_EVENT_STORAGE_H___
#define ___ASURA_SOUND_EVENT_STORAGE_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////
#include "Core/Asura_Hash.h"
#include "Core/Asura_HashTree.h"

//#include "Core/Resources/Asura_ResourceSet.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////
class Asura_Sound_Event;
class Asura_Sound_Sample;
class Asura_Stream;

template <class T> class Asura_ResourceSet;
template <class T> class Asura_ResourceSet_Iterator;
template <class T> class Asura_ResourceSet_ConstIterator;


// C L A S S E S /////////////////////////////////////////////////////////////

//This is used by the storage system to map multiple hash values to the same sound event.
//(There isn't any hash value in the mapping itself , since the tree takes care of that).
//Note that in tools it is assumed that there is only ever one mapping per event.
//RAR 30/08/2006
class Asura_Sound_Event_Mapping
{
public :
	~Asura_Sound_Event_Mapping();

	Asura_Sound_Event* m_pxEvent;
};



class Asura_Sound_Event_Storage
{
	friend class Asura_Sound_Event;
	friend class Asura_Sound_Sample;
	friend class Asura_Sound_Event_Handler;

public:

	static void Reset();
	static void Shutdown();

	static const Asura_Sound_Event* FindEvent(Asura_Hash_ID uHashID);
	static const Asura_Sound_Sample* FindSample(Asura_Hash_ID uHashID);


	//Allocates a new sample and adds it to the current resource bank.
	//If the sample already existed , then a reference will be added to the current resource bank
	//and the existing sample will be returned	
	static Asura_Sound_Sample* CreateSample(const char* szName, bool& bAlreadyExists);
	static Asura_Sound_Event* CreateEvent(const char* szName, bool& bAlreadyExists);

#ifndef ASURA_TOOLS 
	//Adds a mapping of a hash ID to an existing event.
	//This is not supported in tools builds , to prevent problems when editing the events.
	//RAR 30/08/2006
	static void AddEventMapping(Asura_Hash_ID uNewHashID, Asura_Hash_ID uExistingHashID);
#endif

#ifdef ASURA_TOOLS
	//Tools accessors for the samples and events.
	//Note that new samples/events aren't added to the end of the 'indexed' list , so
	//it isn't safe to store an index value for any length of time.
	//RAR 04/07/2006
	static u_int GetNumberOfEvents();
	static Asura_Sound_Event* GetEventByHash(Asura_Hash_ID uHashID);
	static Asura_Sound_Event* GetEventByIndex(u_int uIndex);

	static u_int GetNumberOfSamples();
	static Asura_Sound_Sample* GetSampleByHash(Asura_Hash_ID uHashID);
	static Asura_Sound_Sample* GetSampleByIndex(u_int uIndex);

	static bool RenameSample(const char* szOldName,const char* szNewName);
	static bool RenameEvent(const char* szOldName,const char* szNewName);

	static void DeleteEvent(Asura_Sound_Event* pxEvent);
	static void DeleteSample(Asura_Sound_Sample* pxSample);
#endif





private:

	static Asura_ResourceSet<Asura_Sound_Event_Mapping> s_xEventMappingContainer;
	static Asura_ResourceSet<Asura_Sound_Sample> s_xSampleContainer;
	
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_SOUND_EVENT_STORAGE_H___
