//////////////////////////////////////////////////////////////////////////////
//
// Asura_XAudio2_Voice.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_XAUDIO2_VOICE_H__
#define __ASURA_XAUDIO2_VOICE_H__

#if defined ASURA_USE_AUDIO_SYSTEM

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura Platform-independent
#include "Core/Asura_Maths.h"
#include "Sound/Asura_Audio_Voice.h"

#include <xaudio2.h>

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_XAudio2_Stream;

////////////////////////////////////////////////////////////////////////////////
//
// Asura_XAudio2_Voice
//
// Class wrapping an XAudio2 playback voice
//
////////////////////////////////////////////////////////////////////////////////

class Asura_XAudio2_Voice : public Asura_Audio_Voice, public IXAudio2VoiceCallback
{
	friend class Asura_XAudio2;

public:

	// Types

	typedef void BufferCallback( void* );

	// Methods

	Asura_XAudio2_Voice();
	virtual ~Asura_XAudio2_Voice();

	virtual void EnqueueWave( const Asura_Hash_ID uWave );

	virtual void EnqueueData( void* const pData, const u_int uDataSize, const bool bFinalPacket,
		void* const pContext = NULL );

	virtual void SetBufferEndCallback( BufferCallback* const pfnCallback );

	virtual void Start();
	virtual void Stop();
	virtual bool IsPlaying() const;

	virtual void Pause()	{ SetPausedExternally(true); }
	virtual void Unpause()	{ SetPausedExternally(false); }

	virtual void SetMixerChannel( const Asura_Hash_ID uChannel );

	virtual void SeekAtPlaybackTime( const float fSeekPosition, const float fPlaybackTime );
	virtual bool SeekIsPending();

	virtual void Update( const bool bFirstUpdate = false );

	virtual float GetPlayPosition() const;

	virtual void UpdateFilters();
	
	virtual void SetStartOffset ( const float fOffsetTime);

	virtual void SetupSubbusRouting ();
	virtual bool SetSubmixOutputs( const Asura_Handle* piSubmixVoices, const u_int uNumVoices );
	virtual void ClearSubmixOutputs ();

	virtual bool IsPreparing() const;

	virtual bool IsPendingDestruction() const { return m_iPendingDestructionHandle != ASURA_INVALID_HANDLE; }
	virtual void SetPendingDestruction(Asura_Handle iHandle) { m_iPendingDestructionHandle = iHandle; }
	
protected:

	virtual void SetPreparing( const bool bPrep )	{ m_bPreparing = bPrep; }

	virtual void SetPausedByPitch( bool bPaused = true );
	virtual bool IsPausedByPitch()							{ return m_bPausedByPitch; }

	virtual void SetPausedExternally( bool bPaused = true );
	virtual bool IsPausedExternally()						{ return m_bPausedExternally; }

	// Called when the voice finishes processing a buffer
	STDMETHOD_( void, OnBufferEnd )( void* pBufferContext );
	STDMETHOD_( void, OnBufferStart )( void* pBufferContext );

	// Bits we don't care about, but have to stub out
	STDMETHOD_( void, OnVoiceProcessingPassStart )( UINT32 uBytesRequired ) {}
	STDMETHOD_( void, OnVoiceProcessingPassEnd )() {}
	STDMETHOD_( void, OnStreamEnd )() {}
	STDMETHOD_( void, OnLoopEnd )( void* pBufferContext ) {}
	STDMETHOD_( void, OnVoiceError )( void* pBufferContext, HRESULT xError ) {}

	// Data

	IXAudio2SourceVoice* m_pxSourceVoice;
	IXAudio2SubmixVoice* m_pxSubmixVoice;

#ifdef ASURA_USE_XAUDIO_OCCLUSION
	IXAudio2SubmixVoice* m_pxDirectPathSubmixVoice;	// this is used for obstructing the direct voice path
#endif

	BufferCallback* m_pfnBufferEndCallback;

	// ASURA_INVALID_HANDLE if not streaming
	Asura_Handle m_iStreamHandle;
	Asura_Handle m_iPendingDestructionHandle;

	INT64 m_iSamplesPlayedOffset;

	// Properties of this voice
	u_int m_bPlaying			: 1;
	u_int m_bSeekPending		: 1;
	u_int m_bPausedByPitch		: 1;
	u_int m_bPausedExternally	: 1;
	u_int m_bSubmixOutputs		: 1;
	u_int m_bExternalVoice		: 1;
	u_int m_bPlayAfterPrepared	: 1;
	volatile u_int m_bPreparing	: 1;
};

#endif // defined ASURA_USE_AUDIO_SYSTEM

#endif // !defined __ASURA_XAUDIO2_VOICE_H__
