/**
*
* \file Asura_BillboardPlant_System.cpp
*
* Created by Andrew Weinkove - 11/12/2002
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
**/

#include "Core/Asura.h"
#include "SpecialFX/Asura_BillboardPlant_System.h"
#include "SpecialFX/Asura_BillboardPlant_Batch.h"
#ifdef ASURA_USE_BILLBOARD_PLANTS

unsigned Asura_BillboardPlant_System::m_uNumberOfBillboardPlantBatches;
Asura_BillboardPlant_Batch **Asura_BillboardPlant_System::m_ppxBillboardPlantBatches;

void Asura_BillboardPlant_System::Initialise()
{
	m_uNumberOfBillboardPlantBatches = 0;
	m_ppxBillboardPlantBatches = NULL;
}

void Asura_BillboardPlant_System::Shutdown()
{
	for (unsigned i=0;i<m_uNumberOfBillboardPlantBatches;i++)
	{
		delete m_ppxBillboardPlantBatches[i];
	}
	if (m_ppxBillboardPlantBatches)
		delete[] m_ppxBillboardPlantBatches;
}

void Asura_BillboardPlant_System::Reset()
{
	Shutdown();
	Initialise();
}

void Asura_BillboardPlant_System::Update()
{
	for (unsigned i=0;i<m_uNumberOfBillboardPlantBatches;i++)
	{
		m_ppxBillboardPlantBatches[i]->Update();
	}
}

void Asura_BillboardPlant_System::Render()
{
	for (unsigned i=0;i<m_uNumberOfBillboardPlantBatches;i++)
	{
		m_ppxBillboardPlantBatches[i]->Render();
	}
}

void Asura_BillboardPlant_System::AllocateBatchPointers(unsigned uNumberOfBatches)
{
	Asura_Assert(uNumberOfBatches, "Trying to allocate array of zero size for plant batches");
	m_uNumberOfBillboardPlantBatches = uNumberOfBatches;
	m_ppxBillboardPlantBatches = new Asura_BillboardPlant_Batch*[uNumberOfBatches];
}

void Asura_BillboardPlant_System::SetBatchPointer(Asura_BillboardPlant_Batch *pxBatch, int iIndex)
{
	Asura_Assert(m_ppxBillboardPlantBatches, "");
	m_ppxBillboardPlantBatches[iIndex] = pxBatch;
}

#ifdef ASURA_TOOLS
unsigned Asura_BillboardPlant_System::GetNumberOfBatches()
{
	return m_uNumberOfBillboardPlantBatches;
}
Asura_BillboardPlant_Batch *Asura_BillboardPlant_System::GetBillboardPlantBatch(int iIndex)
{
	Asura_Assert(iIndex < (int)m_uNumberOfBillboardPlantBatches && iIndex >= 0, "Attempting to access out-of-range plant batch");
	return m_ppxBillboardPlantBatches[iIndex];
}
void Asura_BillboardPlant_System::RemoveAllBatches()
{
	Reset();
}
void Asura_BillboardPlant_System::AddNewBatch(Asura_BillboardPlant_Batch *pxBatch)
{
	Asura_BillboardPlant_Batch **ppxBuffer = 0;
	if (m_uNumberOfBillboardPlantBatches)
	{
		ppxBuffer = new Asura_BillboardPlant_Batch*[m_uNumberOfBillboardPlantBatches];
		for (unsigned i=0;i<m_uNumberOfBillboardPlantBatches;i++)
		{
			ppxBuffer[i] = m_ppxBillboardPlantBatches[i];
		}
		delete[] m_ppxBillboardPlantBatches;
	}
	m_uNumberOfBillboardPlantBatches++;
	m_ppxBillboardPlantBatches = new Asura_BillboardPlant_Batch*[m_uNumberOfBillboardPlantBatches];
	if (ppxBuffer)
	{
		for (unsigned i=0;i<m_uNumberOfBillboardPlantBatches-1;i++)
		{
			m_ppxBillboardPlantBatches[i] = ppxBuffer[i];
		}
		delete[] ppxBuffer;
	}
	m_ppxBillboardPlantBatches[m_uNumberOfBillboardPlantBatches-1] = pxBatch;
}


#endif //ASURA_TOOLS

#endif //ASURA_USE_BILLBOARD_PLANTS
