
/**
*
* \file Asura_Decal_Manager.cpp
*
* Created by Dave Reed (08/12/06)
*
* Manages decal template definitions when the decal editor is being used
*
* Copyright (C) 2006 Rebellion, All Rights Reserved.
*
**/

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_DECAL_MANAGER_H__
#define __ASURA_DECAL_MANAGER_H__

#ifdef ASURA_USE_DECAL_MANAGER

#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "SpecialFX/Asura_Decal_System.h"

class Asura_DecalManager_Decal
{
public:

	Asura_DecalManager_Decal()
	{
		m_iDecalTemplate = 0;
		m_fU			 = 0.0f;
		m_fV			 = 0.0f;
		m_fDeltaU		 = 1.0f;
		m_fDeltaV		 = 1.0f;
		m_fWidth		 = 0.25f;
		m_fHeight		 = 0.25f;
		m_iFlags		 = 0;
	}

	int   m_iDecalTemplate;					
	float m_fU;
	float m_fV;
	float m_fDeltaU;
	float m_fDeltaV;
	float m_fWidth;	
	float m_fHeight;	
	int   m_iFlags;
};


enum ASURA_DECALMANAGER_DECAL_FLAGS
{
	ASURA_DECALMANAGER_FLAG_UFLIP		= (1<<0),
	ASURA_DECALMANAGER_FLAG_VFLIP		= (1<<1)
};

struct Asura_DecalManager_DecalSet
{		
public:

	Asura_DecalManager_DecalSet()
	{
		m_uMtlNameHash = 0;	
		m_uSetNameHash = 0;
		m_iNumDecals   = 0;
		m_pxDecals     = 0;
		m_bRandomOrientation = true;
	}

	~Asura_DecalManager_DecalSet()
	{
		if (m_pxDecals)
		{
			delete [] m_pxDecals;
		}
	}

	void operator = ( Asura_DecalManager_DecalSet& xSrc)
	{
		m_xSetName			= xSrc.m_xSetName;
		m_uSetNameHash      = xSrc.m_uSetNameHash;
		m_uMtlNameHash      = xSrc.m_uMtlNameHash;
		m_iNumDecals		= xSrc.m_iNumDecals;
		m_bRandomOrientation= xSrc.m_bRandomOrientation;

		if (xSrc.m_pxDecals)
		{
			m_pxDecals = new Asura_DecalManager_Decal[m_iNumDecals];
			memcpy(m_pxDecals, xSrc.m_pxDecals, m_iNumDecals*sizeof(Asura_DecalManager_Decal));
		}		
	}

	void ChangeNumDecalTypes(int iCount);

	Asura_String			  m_xSetName;
	uint32					  m_uSetNameHash;
	uint32					  m_uMtlNameHash;	

	int					 	  m_iNumDecals;
	bool					  m_bRandomOrientation;
	Asura_DecalManager_Decal *m_pxDecals;

	// These are set up by RebuildTemplates
	int						  m_iFirstAsuraDecalTemplate;
#if defined(ASURA_TARGET_SM3)
	int						  m_iAsuraDecalMaterial;
#else
	int						  m_iAsuraDecalTexture;
#endif

};

struct Asura_DecalManager_Material
{
	Asura_DecalManager_Material()
	{
		m_uMtlNameHash		= 0;		
#if !defined(ASURA_TARGET_SM3)
		m_iBlendMode		= 0;
		m_xTextureName		= "";
		m_xTextureHash		= 0;
		m_iTextureIndex		= 0;
#else
		m_uMaterialResponseHash = ASURA_HASH_ID_UNSET;
#endif
		m_iMaxDecals		= 64;
		m_fFadeOutLength	= 5;
		m_fFadeInLength		= 0.2f;
		m_fLifeTime			= 30.0f;


		m_bEnableFades		= true;
		
		m_bStaticDecalBackFaceCull		= true;
		m_bStaticDecalBackFaceProject	= true;
		m_bExportAsStaticDecal			= false;

		m_fUScrollSpeed					= 0.0f;
		m_fVScrollSpeed					= 0.0f;
		m_fUScrollRange					= 0.0f;
		m_fVScrollRange					= 0.0f;
	}

	void operator = ( Asura_DecalManager_Material& xSrc)
	{
		m_xMtlName			= xSrc.m_xMtlName;

		m_uMtlNameHash		= xSrc.m_uMtlNameHash;
#if !defined(ASURA_TARGET_SM3)
		m_iBlendMode		= xSrc.m_iBlendMode;
		m_iTextureIndex		= xSrc.m_iTextureIndex;
		m_xTextureName		= xSrc.m_xTextureName;		
		m_xTextureHash		= xSrc.m_xTextureHash;
#else
		m_uMaterialResponseHash = xSrc.m_uMaterialResponseHash;
#endif
		m_iMaxDecals		= xSrc.m_iMaxDecals;
		m_fFadeInLength		= xSrc.m_fFadeInLength;
		m_fFadeOutLength	= xSrc.m_fFadeOutLength;
		m_fLifeTime			= xSrc.m_fLifeTime;
		m_bEnableFades		= xSrc.m_bEnableFades;

		m_fUScrollSpeed		= xSrc.m_fUScrollSpeed;
		m_fVScrollSpeed		= xSrc.m_fVScrollSpeed;
		m_fUScrollRange		= xSrc.m_fUScrollRange;
		m_fVScrollRange		= xSrc.m_fVScrollRange;

		m_bExportAsStaticDecal		  = xSrc.m_bExportAsStaticDecal;

		m_bStaticDecalBackFaceCull	  = xSrc.m_bStaticDecalBackFaceCull;
		m_bStaticDecalBackFaceProject = xSrc.m_bStaticDecalBackFaceProject;		
	}


	Asura_String	m_xMtlName;
	uint32			m_uMtlNameHash;

#if !defined(ASURA_TARGET_SM3)
	Asura_String	m_xTextureName;
	Asura_Hash_ID	m_xTextureHash;

	int				m_iTextureIndex;
#endif

#if defined(ASURA_TARGET_SM3)
	Asura_Hash_ID   m_uMaterialResponseHash;
#endif
		
	int				m_iMaxDecals;
#if !defined(ASURA_TARGET_SM3)
	int				m_iBlendMode;
#endif
	bool			m_bEnableFades;
	float			m_fFadeInLength;	
	float			m_fFadeOutLength;	
	float			m_fLifeTime;

	bool			m_bStaticDecalBackFaceCull;
	bool			m_bStaticDecalBackFaceProject;
	bool			m_bExportAsStaticDecal;

	// UV Animation
	float			m_fUScrollSpeed;
	float			m_fVScrollSpeed;
	float			m_fUScrollRange;
	float			m_fVScrollRange;
};

class Asura_Decal_Manager
{
	friend class Asura_Chunk_Decals;

public:
	Asura_Decal_Manager();
	~Asura_Decal_Manager();

	static void							Initialise();
	static void							Shutdown();
	static void							Reset();			

	static const Asura_Decal_Template  *GetDecalTemplate(u_int uDecalTemplateID);
#if defined(ASURA_TARGET_SM3)
	static const Asura_Decal_Material  *GetDecalMaterial(u_int uDecalMaterialID);	
#endif

	static Asura_DecalManager_DecalSet *GetDecalSet(uint32 uNameHash);
	static Asura_DecalManager_Material *GetMaterial(uint32 uNameHash);

#if defined (ASURA_TARGET_SM3)
	static int							GetMaterialIndex(uint32 uNameHash);
#else
	static int							GetTextureIndex(uint32 uNameHash);
#endif
	static int							GetDecalSetIndex(uint32 uNameHash);

	static int							GetRandomDecalFromSet(uint32 uNameHash);

	static int GetNumDecalMaterials() {return s_iNumMaterials;}
	static int GetNumDecalSets()	  {return s_iNumSets;}

	static Asura_DecalManager_DecalSet *GetDecalSetByIndex(int iIndex) {return &s_pxSets[iIndex];}
	static Asura_DecalManager_Material *GetDecalMaterialByIndex(int iIndex) {return &s_pxMaterials[iIndex];}	

	static void AddDecal(const Asura_Vector_3 &xPos, const Asura_Vector_3 &xNormal, uint32 uDecalSetHash, int iDecalIndex=-1, float fScale=1.0f, float fOrientation=0.0f);

	// Add decal, but also specify the decal top direction.
	static void AddDecal(const Asura_Vector_3& xPos, const Asura_Vector_3& xNormal, const Asura_Vector_3& xTopDir,
						 uint32 uDecalSetHash, int iDecalIndex = -1,
						 float fScale = 1.0f, float fOrientation = 0.0f);

	static int							GetTextureUsage( uint32 uTextureNameHash );

#ifdef ASURA_TOOLS
	static Asura_DecalManager_DecalSet *AddDecalSet(Asura_String xName);
	static Asura_DecalManager_Material *AddDecalMaterial(Asura_String xName);	

	static void							RemoveDecalSet(int iIndex);
	static void							RemoveDecalMaterial(int iIndex);	
#endif	

	// Converts Decal Manager data to Asura_Decal_System data
	static void							RebuildTemplates(bool bResetDecalSystem = true);
	static void							PostLoadInitialise(void);
private:	

	// Asura_Decal_System data
	static Asura_Decal_Template			*s_pxDecalTemplates;
#if defined(ASURA_TARGET_SM3)
	static Asura_Decal_Material			*s_pxDecalMaterials;
#else
	static int							*s_iDecalTextures;
#endif
	static int							s_iNumDecalTemplates;
	static int							s_iNumDecalMaterials;	

	// Decal Manager decal store
	static Asura_DecalManager_Material *s_pxMaterials;	
	static Asura_DecalManager_DecalSet *s_pxSets;	
	static int							s_iNumMaterials;
	static int						    s_iNumSets;	
};

#endif // ASURA_USE_DECAL_MANAGER

#endif	// __ASURA_DECAL_MANAGER_H__
