//
// Asura_Deformatron_System.cpp
//
// Created by KOR - 24/11/2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#if defined(ASURA_USE_DEFORMATRON_SYSTEM)

// This file's header.
#include "SpecialFX/Asura_Deformatron_System.h"

// Asura headers


// Static members
Asura_MempoolFactory_Template<Asura_Deformatron>*	Asura_Deformatron_System::s_pxFactory;
Asura_Deformatron*									Asura_Deformatron_System::s_apxDeformatronList[ASURA_MAX_DEFORMATRONS_IN_LIST];
u_int												Asura_Deformatron_System::s_uNumDeformatronsInList;
Asura_Bounding_Box*									Asura_Deformatron_System::s_pxProximityTestBoundingBox;


void Asura_Deformatron_System::Initialise()
{
	s_pxFactory = new Asura_MempoolFactory_Template<Asura_Deformatron>();
	s_pxFactory->Reset();
}

void Asura_Deformatron_System::Reset()
{
	s_pxFactory->Reset();
}

void Asura_Deformatron_System::Shutdown()
{
	if(s_pxFactory)
	{
		delete s_pxFactory;
		s_pxFactory = 0;
	}
}

void Asura_Deformatron_System::Update()
{
}

Asura_Handle Asura_Deformatron_System::CreateDeformatron()
{
	return s_pxFactory->CreateInstance();
}

void Asura_Deformatron_System::DestroyDeformatron(Asura_Handle iHandle)
{
	s_pxFactory->DestroyInstance(iHandle);
}

Asura_Deformatron* Asura_Deformatron_System::GetDeformatron(Asura_Handle iHandle)
{
	return s_pxFactory->GetInstancePointer(iHandle);
}

void Asura_Deformatron_System::MakeDeformatronListForBB(Asura_Bounding_Box& xBB, u_int uMaxDeformatrons /* = ASURA_MAX_DEFORMATRONS_IN_LIST */)
{
	Asura_Assert(uMaxDeformatrons <= ASURA_MAX_DEFORMATRONS_IN_LIST, "Cannot request more than ASURA_MAX_DEFORMATRONS_IN_LIST");

	// Reset list
	s_uNumDeformatronsInList = 0;

	// Set up search
	s_pxProximityTestBoundingBox = &xBB;

	// Do search
	Asura_TrackingHandlePool<Asura_Deformatron>* pxHandlePool = s_pxFactory->GetPool();
	pxHandlePool->Traverse(DeformatronCallback);

	// Clamp number of deformatrons
	s_uNumDeformatronsInList = Asura_Maths::Min(s_uNumDeformatronsInList, uMaxDeformatrons);
}

void Asura_Deformatron_System::DeformatronCallback(Asura_Deformatron* pxDeformatron)
{
	const float fRange = pxDeformatron->m_fRange;
	if (! (fRange > 0.0f) ) return;

	// Check for intersection
	if(s_pxProximityTestBoundingBox->IntersectsSphere(pxDeformatron->m_xPosition,fRange))
	{
		// Check whether we've reached the limit
		if(s_uNumDeformatronsInList == ASURA_MAX_DEFORMATRONS_IN_LIST)
		{
			Asura_Assert(0, "Too many deformatrons in this bounding box");
			return;
		}
		// Add it to the list
		s_apxDeformatronList[s_uNumDeformatronsInList] = pxDeformatron;
		s_uNumDeformatronsInList++;
	}
}

u_int Asura_Deformatron_System::GetNumDeformatronsInList()
{
	return s_uNumDeformatronsInList;
}

const Asura_Deformatron* Asura_Deformatron_System::GetDeformatronInList(u_int uNum)
{
	Asura_Assert(uNum < s_uNumDeformatronsInList, "Deformatron index out of range");
	return s_apxDeformatronList[uNum];
}
#endif
