
//
// Asura_LightShaft.h
//
// Created by Dave Reed - 13/07/2007 
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_LIGHTSHAFT_H__
#define __ASURA_LIGHTSHAFT_H__

#ifdef ASURA_TARGET_SM3

#include "Core/Asura_Maths.h"
#include "Core/Asura_TextureManagement.h"
#include "Rendering/Asura_Colour.h"

#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )
#include "Visibility/Asura_Visibility_System.h"
#endif

class Asura_LightShaft
{
	friend class Asura_LightShaft_System;
	friend class Asura_ClientEntity_LightShaft;	
	friend class Asura_ClientContainerInstance_LightShaft;
	
public:
	Asura_LightShaft()
	{
		ResetToDefaults();
	}

	void ResetToDefaults()
	{
		m_xPos						= Asura_Vector_3(0,0,0);
		m_xOrient.Identity();		

		m_afDistances[0]			= 0.0f;
		m_afDistances[1]			= 10.0f;

		m_afRadii[0]				= 0.0f;
		m_afRadii[1]				= 2.5f;
		
		m_axColours[0]				= Asura_RGBA(1,1,1,1);
		m_axColours[1]				= Asura_RGBA(0,0,0,1);

		m_aiTextureIndices[0]		= ASURA_TEXTURE_NONE;
		m_aiTextureIndices[1]		= ASURA_TEXTURE_NONE;

		m_afTextureUScales[0]		= 1.0f;
		m_afTextureUScales[1]		= 1.0f;

		m_afTextureScrollRates[0]	= 0.0f;
		m_afTextureScrollRates[1]	= 0.0f;

		m_fDistFadeStart			= 150.0f;
		m_fDistFadeRange			= 10.0f;		
		m_fCloseFadeStart			= 0.0f;
		m_fCloseFadeRange			= 10.0f;		

		m_iCoronaTextureIndex		= ASURA_TEXTURE_NONE;		
		m_fCoronaSize				= 3.0f;
		m_fCoronaAngleRange			= 0.6f;	
		m_fCoronaDepth				= 2.0f;				
		m_xCoronaColour				= Asura_RGBA(1,1,1,1);
		m_xInheritedColour			= Asura_RGBA(1,1,1,1);

		m_bEnabled					= true;		
#if !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )
		m_iLastKnownModule			= -1;
#endif
	}


	void SetPosition				( const Asura_Vector_3& xInput)	  { m_xPos = xInput;	}
	void SetOrientation				( const Asura_Matrix_3x3& xInput) { m_xOrient = xInput; }	
	void SetFade					( float fStart, float fRange)	  { m_fDistFadeStart = fStart; m_fDistFadeRange = fRange;}
	
	int								GetHandle(void)					  { return m_iHandle;						}
	const Asura_Vector_3&			GetPosition(void)			const { return m_xPos;							}
	const Asura_Matrix_3x3&			GetOrient(void)				const { return m_xOrient;						}
	const Asura_Bounding_Box&		GetBoundingBox(void)		const { return m_xBB;							}
	float							GetNearRadius(void)			const { return m_afRadii[0];					}
	float							GetNearDistance(void)		const { return m_afDistances[0];				}
	float							GetFarRadius(void)			const { return m_afRadii[1];					}
	float							GetFarDistance(void)		const { return m_afDistances[1];				}
	const Asura_RGBA&				GetNearColour(void)			const { return m_axColours[0];					}
	const Asura_RGBA&				GetFarColour(void)			const { return m_axColours[1];					}
	int								GetTextureIndex(int iLayer) const { return m_aiTextureIndices[iLayer];		}
	float							GetUScale(int iLayer)		const { return m_afTextureUScales[iLayer];		}
	float							GetUScrollRate(int iLayer)	const { return m_afTextureScrollRates[iLayer];	}
	float							GetDistanceFadeStart(void)  const { return m_fDistFadeStart;				}
	float							GetDistanceFadeDist(void)   const { return m_fDistFadeRange;				}	
	float							GetCloseFadeStart(void)		const { return m_fCloseFadeStart;				}
	float							GetCloseFadeDist(void)		const { return m_fCloseFadeRange;				}	

	int								GetCoronaTextureIndex(void) const { return m_iCoronaTextureIndex;			}
	float							GetCoronaSize(void)			const { return m_fCoronaSize;					}
	float							GetCoronaDepth(void)		const { return m_fCoronaDepth;					}
	float							GetCoronaAngleRange(void)	const { return m_fCoronaAngleRange;				}
	const Asura_RGBA&				GetCoronaColour(void)		const { return m_xCoronaColour;					}

	bool							GetEnable(void)				const { return m_bEnabled;						}
	const Asura_RGBA&				GetInheritedColour(void)	const { return m_xInheritedColour;				}

	void							SetEnable(bool bEnable)				{ m_bEnabled = bEnable; }
	void							SetInheritedColour(Asura_RGBA xCol) { m_xInheritedColour = xCol; }


	void							UpdateBoundingBox(void);
#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
	bool							IsVisible( void ) { return m_xVisibilityQuery.IsAABBVisibleFromCurrentCamera( m_xBB ); }
#endif

private:	

	int					 m_iHandle;

	Asura_Vector_3		 m_xPos;
	Asura_Matrix_3x3	 m_xOrient;
	
	float				 m_afRadii[2];						// top, bottom
	float				 m_afDistances[2];					// top, bottom
	Asura_RGBA			 m_axColours[2];							
	ASURA_TEXTURE_ID	 m_aiTextureIndices[2];				// 2 layers of texturing	
	float				 m_afTextureUScales[2];
	float				 m_afTextureScrollRates[2];	

	float				 m_fDistFadeStart;
	float				 m_fDistFadeRange;
	float				 m_fCloseFadeStart;
	float				 m_fCloseFadeRange;
	Asura_Bounding_Box   m_xBB;

	ASURA_TEXTURE_ID	 m_iCoronaTextureIndex;	
	float				 m_fCoronaSize;
	float				 m_fCoronaAngleRange;
	float				 m_fCoronaDepth;					// Amount to move it forwards
	Asura_RGBA			 m_xCoronaColour;

	// Used when attached to dynamic lights
	bool				 m_bEnabled;
	Asura_RGBA			 m_xInheritedColour;
#if defined (ASURA_USE_NEW_VISIBILITY_SYSTEM )
	Asura_Visibility_Query m_xVisibilityQuery;
#else
	int					 m_iLastKnownModule;
#endif
	
};

#endif //#ifdef ASURA_TARGET_SM3
#endif //__ASURA_LIGHTSHAFT_H__
