
//
// Asura_LightShaft_System.h
//
// Created by Dave Reed - 13/07/2007 
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_LIGHTSHAFT_SYSTEM_H__
#define __ASURA_LIGHTSHAFT_SYSTEM_H__

#ifdef ASURA_TARGET_SM3

#include "Core/Asura_Handle.h"
#include "SpecialFX/Asura_LightShaft.h"

template<typename T> class Asura_TrackingHandlePool;


class Asura_LightShaftVertex
{
public:
	Asura_Vector_3 xPos;	
	float		   fU;
	float		   fV;
	float		   fAlpha;
	float		   fBottom;
};

class Asura_LightShaftCoronaVertex
{
public:
	Asura_Vector_3 xPos;	
	float		   fR;
	float		   fG;
	float		   fB;
	float		   fA;
	float		   fU;
	float          fV;
	float		   fDepth;	
	float		   fUnused;
};

class Asura_LightShaft_System
{
public:
	static void Initialise(void);	
	static void Shutdown(void);
	static void Reset(void);

	static void Platform_Initialise(void);	
	static void Platform_Shutdown(void);
	static void Platform_Reset(void);
	static void Platform_RenderStart(void);
	static void Platform_RenderEnd(void);
	static void Platform_RenderLightShaft(Asura_LightShaft *pxLightShaft);

	static void Update(void);
	static void Render(void);

	static Asura_LightShaft* CreateLightShaft(void);
	static void				 DestroyLightShaft(const Asura_LightShaft* pxLightShaft);
	static Asura_LightShaft* GetLightShaftPointer(Asura_Handle iHandle);
	
	static void				 CreateConeVertices(int iRadialSegments, int iLateralSegments, Asura_LightShaftVertex *pxVerts, int iNumVerts);

private:	
	static Asura_TrackingHandlePool<Asura_LightShaft>*	s_pxLightShaftList;

#if !defined (ASURA_USE_NEW_VISIBILITY_SYSTEM)
	static Asura_Bounding_Box							s_xBBOfAllVisibleModules;
#endif

	static void RenderCallback(Asura_LightShaft* pxLightShaft);	

#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
	static bool ReloadShaders ( bool bHelp );
#endif
};


#endif //#ifdef ASURA_TARGET_SM3
#endif //__ASURA_LIGHTSHAFT_SYSTEM_H__
