//
// Asura_Skybox.cpp
//
// Created by Kevin Lea - 02/10/2001 10:42:22
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "SpecialFX/Asura_Skybox.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Camera/Asura_Camera.h"
#include "Debugging/Asura_CPUProfile.h"
#ifdef ASURA_TOOLS
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#endif
#include "Entity/PhysicalObject/Asura_ClientEntity_PhysicalObject.h"
#include "Interface/Asura_CommandConsole.h"
#ifdef ASURA_TOOLS
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#endif
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_SortedRender.h"

#ifdef ASURA_TOOLS
#include "Tools_System.h"
#include "Entity/Tools_Entity_Data.h"
#include "Lighting/Tools_LightSets.h"
#endif

#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM
#include "Visibility/Asura_Visibility_System.h"
#endif

Asura_String	Asura_Skybox::s_axTextureName[ASURA_SKYBOX_NUM_TEXTURES];
Asura_Angle		Asura_Skybox::s_fOrientationAroundYAxis;
float			Asura_Skybox::s_fRed;
float			Asura_Skybox::s_fGreen;
float			Asura_Skybox::s_fBlue;
float			Asura_Skybox::s_fObjectRed;
float			Asura_Skybox::s_fObjectGreen;
float			Asura_Skybox::s_fObjectBlue;
bool			Asura_Skybox::s_bDrawClouds;
bool			Asura_Skybox::s_bUseSecondClouds;
bool			Asura_Skybox::s_bBackTextureIsFrontUpsideDown;
bool			Asura_Skybox::s_bRightTextureIsLeftUpsideDown;
bool			Asura_Skybox::s_bMirrorBelowHorizon		= false;
bool			Asura_Skybox::s_bSkyboxVisible			= true;
#if defined( ASURA_TARGET_SM3)

bool			Asura_Skybox::s_bDisableHaze			= false;
bool			Asura_Skybox::s_bDisable2Haze			= true;
bool			Asura_Skybox::s_bDisableSun				= true;
bool			Asura_Skybox::s_bDisable2Sun			= true;
bool 			Asura_Skybox::s_b1MirrorBelowHorizon	= false;
bool 			Asura_Skybox::s_b2MirrorBelowHorizon	= false;

float Asura_Skybox::s_afSkyboxColour[ 3 ]	= { 0.0f * 255.0f, 0.0f * 255.0f, 0.3f * 255.0f };
float Asura_Skybox::s_afHazeColour1[ 3 ]	= { 0.7f * 255.0f, 0.8f * 255.0f, 0.9f * 255.0f };
float Asura_Skybox::s_fHazeStrength1		= 1.0f;
float Asura_Skybox::s_afHazeColour2[ 3 ]	= { 1.0f * 255.0f, 1.0f * 255.0f, 1.0f * 255.0f };
float Asura_Skybox::s_fHazeStrength2		= 1.0f;
float Asura_Skybox::s_afSunColour1[ 3 ]		= { 2.0f * 255.0f, 1.8f * 255.0f, 1.5f * 255.0f };
float Asura_Skybox::s_fSunStrength1			= 98.0f;
float Asura_Skybox::s_afSunColour2[ 3 ]		= { 1.0f * 255.0f, 0.8f * 255.0f, 0.5f * 255.0f };
float Asura_Skybox::s_fSunStrength2			= 91.0f;
#ifdef ASURA_TOOLS
float Asura_Skybox::s_afSunDir1[ 2 ]		= { 0.0f, 0.0f };
float Asura_Skybox::s_afSunDir2[ 2 ]		= { 0.0f, 0.0f };
#else
float Asura_Skybox::s_afSunDir1[ 3 ]		= { 0.0f, 0.0f, 0.0f };
float Asura_Skybox::s_afSunDir2[ 3 ]		= { 0.0f, 0.0f, 0.0f };
#endif
bool  Asura_Skybox::s_bUseNewSkyboxShader	= false;

#endif

#ifdef ASURA_TOOLS
Asura_String 	Asura_Skybox::s_xSkyboxName;
#endif

Asura_Hashed_List<Asura_ClientEntity_PhysicalObject> Asura_Skybox::s_xSkyboxObjects;

#ifdef ASURA_TOOLS
bool			Asura_Skybox::s_bNeedToRegenerateSkyboxObjects = false;
#endif

static bool g_bNeedToDoRenderSetup;

DVSVAR bool g_bShouldRenderSkyboxObjects = true;

void Asura_Skybox::Initialise( void )
{
	// Kevin Lea - 03/10/2001 10:53:53 - I'm going to let Asura_Skybox::Initialise
	// set 'safe' defaults, so that if there is no skybox chunk (i.e. if a level designer
	// has not yet chosen a sky) a blank, coloured sky will be drawn.
	// Obviously this isn't really necessary in a 'final' product.
	SetColours(40.0f,40.0f,40.0f);
	SetObjectColours(255.0f,255.0f,255.0f);
	SetOrientationAroundYAxis(0.0f);
    s_bDrawClouds = true;
	s_bUseSecondClouds = false;
	s_bBackTextureIsFrontUpsideDown = false;
	s_bRightTextureIsLeftUpsideDown = false;
	s_bSkyboxVisible = true;

    ResetInternal( false );

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	Asura_CommandConsole::AddVar( L"Render.Skybox.RenderObjects", &g_bShouldRenderSkyboxObjects,
		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif

    Platform_Initialise();
}

void Asura_Skybox::Reset( void )
{
    ResetInternal( true );
}

void Asura_Skybox::ResetInternal( bool bCallPlatformReset )
{
	// (MH - 17.12.2002) trying to fix fragmentation
	for (int i = 0; i < ASURA_SKYBOX_NUM_TEXTURES; i++)
	{
		s_axTextureName[i].MakeNull();// was set to ""
	} 
	#ifdef ASURA_TOOLS
	s_xSkyboxName.MakeNull();
	#endif

    if( bCallPlatformReset ) { Platform_Reset(); }

	s_xSkyboxObjects.Clear();

    g_bNeedToDoRenderSetup = true;
	s_bSkyboxVisible = true;
}

void Asura_Skybox::Shutdown( void )
{
    ResetInternal( false );
    Platform_Shutdown();
}


void Asura_Skybox::Render(void)
{
	if (!Asura_Render::IsSystemEnabled(ASURA_RENDER_SYSTEM_SKYBOX) || Asura_Render::GetWireframe())
	{
		return;
	}

#ifdef ASURA_USE_NEW_VISIBILITY_SYSTEM
	if ( Asura_Visibility_System::ShouldRenderInXRayMode() ) return;
#endif

	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__WORLD_RENDER, "World Render");


	if( g_bNeedToDoRenderSetup )
    {
        Platform_DoRenderSetup();
        g_bNeedToDoRenderSetup = false;
    }
	
	Platform_Render();

	if( g_bShouldRenderSkyboxObjects )
	{
        const bool bModifyFog = Platform_ShouldModifyFogForSkyboxObjects();

		// temporarily fix up fog so that our objects will all get the skybox fog value
		const bool bOldFogEnabled = Asura_Fog::IsFogEnabled(), bOldGraphFog = Asura_Fog::IsGraphFogEnabled();
		const float fOldFarPlane = Asura_Fog::GetFogFarPlane();
		const float fOldFarPlaneValue = Asura_Fog::GetFogValueAtFarPlane();

		if( bModifyFog )
        {
            if( bOldFogEnabled )
		    {
			    Asura_Fog::EnableFog(true);
			    Asura_Fog::EnableGraphFog(false);
			    Asura_Fog::SetFogFarPlane(0.0f);
			    Asura_Fog::SetFogValueAtFarPlane(Asura_Fog::GetSkyboxFogBlendFactor());
		    }
        }

		#ifdef ASURA_TOOLS
		if(Tools_System::IsActive())
		{
			if(s_bNeedToRegenerateSkyboxObjects)
			{
				//Search for any physical objects that are marked as merge into skybox and ensure
				//that their client entities exist and are up to date
				for(Asura_Hashed_List_It<Asura_ServerEntity_PhysicalObject> xIt(Asura_ServerEntity_PhysicalObject::GetList()); !xIt.Done(); xIt.Next())
				{
					Asura_ServerEntity_PhysicalObject* pxEntity = xIt.GetCurrent();
					if(pxEntity->IsMergedWithSkybox())
					{
						if(!Asura_ClientEntity_PhysicalObject::FindEntity(pxEntity->GetGuid()))
						{
							//client entity doesn't exist , so send a request to make it.
						    Asura_Message* pxRequestMsg = Asura_Network::CreateMessage( ASURA_MSG_CLIENT_REQUESTCREATEMSG, 0, pxEntity->GetClassification() );
						    if( !pxRequestMsg ) { return; }

						    pxRequestMsg->SetToGuid( pxEntity->GetGuid() );
						    pxRequestMsg->SetFromGuid( Asura_Network::GetClientID() );

						    Asura_Network::Send( pxRequestMsg );

						}
						
						pxEntity->ForceClientUpdate();
					}
				}
				s_bNeedToRegenerateSkyboxObjects = false;
			}
		}
		#endif //ASURA_TOOLS
	
#ifndef ASURA_TARGET_SM3
		Asura_SortedRender::Start();
#endif

#ifdef ASURA_TARGET_SM3
		Asura_Render::BeginSkyboxRendering();
		for (int iPass = 0; iPass < 2; iPass++)
		{
			Asura_SortedRender::SetTransparentPass(iPass == 1);
#endif

			for(Asura_Hashed_List_It<Asura_ClientEntity_PhysicalObject> xIt(&s_xSkyboxObjects); !xIt.Done(); xIt.Next())
			{
				Asura_ClientEntity_PhysicalObject* pxClientEntity = xIt.GetCurrent();

#ifdef ASURA_TOOLS
				//make sure the corresponding server entity is flagged for the skybox and should be rendered
				//for this light set
				if(Tools_System::IsActive())
				{
					Asura_ServerEntity_PhysicalObject* pxServerEntity = Asura_ServerEntity_PhysicalObject::FindEntity(pxClientEntity->GetGuid());
					if(!pxServerEntity) continue;
					if(!pxServerEntity->IsMergedWithSkybox()) continue;
					if(!(Tools_LightSets::IsMemberOfCurrentLightSets(pxServerEntity->ToolsData->m_uLightSets))) continue;

				}
#endif
				pxClientEntity->RenderInSkybox(*Asura_Camera::GetPosition(),s_fObjectRed,s_fObjectGreen,s_fObjectBlue);
			}
#ifdef ASURA_TARGET_SM3
		}
		Asura_SortedRender::SetTransparentPass(false);
		Asura_Render::EndSkyboxRendering();
#endif	

#ifndef ASURA_TARGET_SM3
		Asura_SortedRender::End();
#endif

        if( bModifyFog )
        {
		    // restore the fog setup
		    Asura_Fog::SetFogFarPlane(fOldFarPlane);
		    Asura_Fog::SetFogValueAtFarPlane(fOldFarPlaneValue);
		    Asura_Fog::EnableFog(bOldFogEnabled);
		    Asura_Fog::EnableGraphFog(bOldGraphFog);
        }
	}

	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__WORLD_RENDER);
}

void Asura_Skybox::SetTexture( const Asura_Char* szName, int iFace)
{
	s_axTextureName[iFace] = szName;
    g_bNeedToDoRenderSetup = true;
}

void Asura_Skybox::SetOrientationAroundYAxis(Asura_Angle fAngle)
{
	s_fOrientationAroundYAxis = fAngle;
    g_bNeedToDoRenderSetup = true;
}

// Kevin Lea - 02/10/2001 11:09:01 - colour values in range 0.0 to 255.0 inclusive
void Asura_Skybox::SetColours(float fRed, float fGreen, float fBlue)
{
	s_fRed = fRed;
	s_fGreen = fGreen;
	s_fBlue = fBlue;
    g_bNeedToDoRenderSetup = true;
}

void Asura_Skybox::SetObjectColours(float fRed, float fGreen, float fBlue)
{
	s_fObjectRed = fRed;
	s_fObjectGreen = fGreen;
	s_fObjectBlue = fBlue;

	//update any existing objects
	/*
	for(Asura_List_It<SkyboxObject> xIt(&s_xSkyboxObjects);!xIt.Done();xIt.Next())
	{
		SkyboxObject* pxObj = xIt.GetCurrent();
		if(pxObj->m_pxShapeInst)
		{
			pxObj->m_pxShapeInst->Platform_SetAmbientRGB((u_char)Asura_Maths::FloatToInt(s_fObjectRed/2.0f),(u_char)Asura_Maths::FloatToInt(s_fObjectGreen/2.0f),(u_char)Asura_Maths::FloatToInt(s_fObjectBlue/2.0f));
		}
	}
	*/

}


void Asura_Skybox::SetBackTextureIsFrontUpsideDown(bool bBackIsFront) 
{
	s_bBackTextureIsFrontUpsideDown = bBackIsFront;
    g_bNeedToDoRenderSetup = true;
}

void Asura_Skybox::SetRightTextureIsLeftUpsideDown(bool bRightIsLeft) 
{
	s_bRightTextureIsLeftUpsideDown = bRightIsLeft;
    g_bNeedToDoRenderSetup = true;
}

void Asura_Skybox::SetMirrorBelowHorizon(bool bMirror)
{
	s_bMirrorBelowHorizon = bMirror;
	g_bNeedToDoRenderSetup = true;
}

void Asura_Skybox::GetColours(float& fRed, float& fGreen, float& fBlue)
{ 
	fRed = s_fRed;
	fGreen = s_fGreen;
	fBlue = s_fBlue;
}

void Asura_Skybox::GetObjectColours(float& fRed, float& fGreen, float& fBlue)
{ 
	fRed = s_fObjectRed;
	fGreen = s_fObjectGreen;
	fBlue = s_fObjectBlue;
}

u_int Asura_Skybox::GetNumSkyboxTextures( void )
{
    return ASURA_SKYBOX_NUM_TEXTURES;
}

void Asura_Skybox::AddSkyboxObject(Asura_ClientEntity_PhysicalObject* pxEntity)
{
	s_xSkyboxObjects.AddToList(pxEntity);	
}

void Asura_Skybox::RemoveSkyboxObject(Asura_ClientEntity_PhysicalObject* pxEntity)
{
	s_xSkyboxObjects.RemoveFromList(pxEntity);	
}

void Asura_Skybox::RequestRenderSetup()
{
	g_bNeedToDoRenderSetup = true;
}
