//
// Asura_Sphere.cpp
//
// Created by Kevin Lea - 12/06/2001 11:10:28
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "SpecialFX/Asura_Sphere.h"

#define MakeVertex(o,fx,fy,fz)	\
{ \
	o->x=(fx); \
	o->y=(fy); \
	o->z=(fz); \
	o++; \
}

#define MakeFace(f,a,b,c) \
{ \
	f->v[0] = (a); \
	f->v[1] = (b); \
	f->v[2] = (c); \
	f++; \
}

Asura_Vector_3 OctantVertex[(SPHERE_ORDER+1)*(SPHERE_ORDER+2)/2];
TRI_FACE SphereFace[SPHERE_FACES];

Asura_Sphere_Vertex SphereVertex[SPHERE_VERTICES];
unsigned short SphereIndices[SPHERE_INDICES];





static void Generate_SphereOctant(void)
{
	float i,j;

	Asura_Vector_3 *o = OctantVertex;

	/* i=0, j=0 */
	MakeVertex(o,0.0f,0.0f,SPHERE_RADIUS);

	
	for (i=1.0f; i<SPHERE_ORDER_AS_FLOAT; i++)
	{
		float cosPhi, sinPhi;
		{
			Asura_Angle phi = Asura_Maths::Pi/2.0f/SPHERE_ORDER_AS_FLOAT*i;	 
			cosPhi = Asura_Maths::Cosine(phi);
			sinPhi = Asura_Maths::Sine(phi);
		}

		/* 0<i<n, j=0 */
		/* => cosTheta = 1, sinTheta = 0 */
		MakeVertex(o,sinPhi,0.0f,cosPhi);

		for (j=1.0f; j<i; j++)
		{
			float cosTheta, sinTheta;
			{
				Asura_Angle theta =  Asura_Maths::Pi/2.0f*j/i;
				cosTheta = Asura_Maths::Cosine(theta);
				sinTheta = Asura_Maths::Sine(theta);
			}

			/* 0<i<n, 0<j<i */
			MakeVertex(o,cosTheta*sinPhi,sinTheta*sinPhi,cosPhi);
		}

		/* 0<i<n, j=i */
		MakeVertex(o,0.0f,sinPhi,cosPhi);
	}

	/* i=n, j=0 */
	MakeVertex(o,SPHERE_RADIUS,0.0f,0.0f);
			
	for (j=1.0f; j<SPHERE_ORDER_AS_FLOAT; j++)
	{
		float cosTheta, sinTheta;
		{
			Asura_Angle theta = Asura_Maths::Pi/2.0f/SPHERE_ORDER_AS_FLOAT*j;
			cosTheta = Asura_Maths::Cosine(theta);
			sinTheta = Asura_Maths::Sine(theta);
		}
		/* i=n, 0<j<i */
		MakeVertex(o,cosTheta,sinTheta,0.0f);
	}

	/* i=n, j=i */
	MakeVertex(o,0.0f,SPHERE_RADIUS,0.0f);
}

void Generate_Sphere(void)
{
	/* first generate vertices */
	{
		int i,j;
		Asura_Sphere_Vertex *v = SphereVertex;
		Asura_Vector_3 *o = OctantVertex;
				
		Generate_SphereOctant();

		/* north pole */
		*v++ = *o;
		for (i=0; ++i<=SPHERE_ORDER;)
		{
			o += i;
			/* 1st Quadrant */
			for (j=i; --j>=0; o++, v++)
			{
				*v = *o;
			}
			/* 2nd Quadrant */
			for (j=i; --j>=0; o--, v++)
			{
				v->x = -o->x;
				v->y = o->y;
				v->z = o->z;
			}
			/* 3rd Quadrant */											 
			for (j=i; --j>=0; o++, v++)
			{
				v->x = -o->x;
				v->y = -o->y;
				v->z = o->z;
			}
			/* 4th Quadrant */
			for (j=i; --j>=0; o--, v++)
			{
				v->x = o->x;
				v->y = -o->y;
				v->z = o->z;
			}
		}
		for (; --i>1;)
		{
			o -= i;
			/* 5th Quadrant */
			for (j=i; --j>0; o++, v++)
			{
				v->x = o->x;
				v->y = o->y;
				v->z = -o->z;
			}
			/* 6th Quadrant */
			for (j=i; --j>0; o--, v++)
			{
				v->x = -o->x;
				v->y = o->y;
				v->z = -o->z;
			}
			/* 7th Quadrant */
			for (j=i; --j>0; o++, v++)
			{
				v->x = -o->x;
				v->y = -o->y;
				v->z = -o->z;
			}
			/* 8th Quadrant */
			for (j=i; --j>0; o--, v++)
			{
				v->x = o->x;
				v->y = -o->y;
				v->z = -o->z;
			}
		}
		o--;
		/* south pole */
		v->x = -o->x;
		v->y = -o->y;
		v->z = -o->z;
	}
	
	/* now generate face data */
	{
		TRI_FACE *f = SphereFace;
		int kv,kw,ko,kv0,kw0,i,j;

		kv = 0, kw = 1;

		for(i=0; i<SPHERE_ORDER; i++)
		{
			kv0 = kv, kw0 = kw;
			for (ko=1; ko<=3; ko++)
			{
				for (j=i;; j--)
				{
					MakeFace(f,kv,kw,++kw);
					if (j==0) break;
					MakeFace(f,kv,kw,++kv);
				}
			}
			for (j=i;;j--)
			{
				if (j==0)
				{
					MakeFace(f,kv0,kw,kw0);
					kv++;
					kw++;
					break;
				}
				MakeFace(f,kv,kw,++kw);
				if (j==1)
				{
					MakeFace(f,kv,kw,kv0);
				}
				else MakeFace(f,kv,kw,++kv);
			}
		}
		for(; --i>=0;)
		{
			kv0=kv,kw0=kw;
			for(ko=5;ko<=7;ko++)
			{
				for (j=i;; j--)
				{
					MakeFace(f,kv,kw,++kv);
					if (j==0) break;	 
					MakeFace(f,kv,kw,++kw);
				}
			}
			for (j=i;;j--)
			{
				if (j==0)
				{
					MakeFace(f,kv,kw0,kv0);
					kv++;
					kw++;
					break;
				}
				MakeFace(f,kv,kw,++kv);
				if (j==1)
				{
					MakeFace(f,kv,kw,kw0);
				}
				else MakeFace(f,kv,kw,++kw);
			}
		}
	}
	{
		int i;
		Asura_Sphere_Vertex *vSphere = SphereVertex;
		for(i=0;i<SPHERE_VERTICES;i++,vSphere++)
		{

			SphereVertex[i].U = (Asura_Maths::ArcCosine(vSphere->y)/(2.0f*Asura_Maths::Pi)+0.5f)*(float)SPHERE_TEXTURE_WRAP;//*8;
			SphereVertex[i].V = (Asura_Maths::ArcTangent(vSphere->z,vSphere->x)/(2.0f*Asura_Maths::Pi)+0.5f)*(float)SPHERE_TEXTURE_WRAP;//*8;
			SphereVertex[i].Colour = 0x3fffffff;
		
		}
		for (i=0; i<SPHERE_FACES; i++)
		{
			for (int n=0; n<3; n++)
				SphereIndices[i*3+n] = (unsigned short)SphereFace[i].v[n];
		}
	}
}
