/**
*
* \file Asura_Weather_System.h
*
* Created by Kevin Lea - 21/11/2002 16:48:10
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/
#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_WEATHER_SYSTEM_H__
#define __ASURA_WEATHER_SYSTEM_H__

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Vector_3;

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_Weather_System
{
#ifdef ASURA_TOOLS
	friend class Tools_Mode_Weather;
#endif
	friend class Asura_Chunk_WeatherSystem;
	
	public:

		static void Initialise(void);
		static void Reset(void);
		static void Shutdown(void);

		static void Update(void);
		static void Render(void);

        //
        // Sun effects.
        //

        static bool IsSunGlareEnabled( void ) {return( 0 != (s_uSunFlags & SUNFLAG_GLARE) );}
        static void SetSunGlareEnabled( bool b ) { s_uSunFlags &= ~SUNFLAG_GLARE; s_uSunFlags |= (b ? SUNFLAG_GLARE : 0); }

        static bool IsLensFlareEnabled( void ) { return( 0 != (s_uSunFlags & SUNFLAG_LENSFLARE) ); }
        static void SetLensFlareEnabled( bool b ) { s_uSunFlags &= ~SUNFLAG_LENSFLARE; s_uSunFlags |= (b ? SUNFLAG_LENSFLARE : 0); }

		static bool IsHorizonGlareEnabled( void ) { return( 0 != (s_uSunFlags & SUNFLAG_HORIZONGLARE) ); }
		static void SetHorizonGlareEnabled( bool b ) { s_uSunFlags &= ~SUNFLAG_HORIZONGLARE; s_uSunFlags |= (b ? SUNFLAG_HORIZONGLARE : 0); }

        static const Asura_Vector_3& GetSunPos( void ) { return s_xSunPos; }
        
        static float GetSunColourR( void ) { return s_fSunColourR; }
        static float GetSunColourG( void ) { return s_fSunColourG; }
        static float GetSunColourB( void ) { return s_fSunColourB; }
		
		// size of horizon glare sprite above and below the horizon line from 0.0f-1.0f (max is top/bottom of screen)
		static float GetHorizonGlareSizeDown() { return s_fHorizonGlareSizeDown; }
		static float GetHorizonGlareSizeUp() { return s_fHorizonGlareSizeUp; }

		#ifdef ASURA_TOOLS
		//Used by tools for disabling rendering of sunglare. IsSunGlareEnabled() will still return true (assuming it would otherwise be on). RAR 29/06/2006
		static void SetToolsDisableSunGlare(bool bDisable){s_bToolsDisableSunGlare = bDisable;}
		#endif

	private:

        static void Platform_Initialise(void);
		static void Platform_Reset(void);
		static void Platform_Shutdown(void);

        static void Platform_RenderSunGlare( void );
		static void Platform_RenderHorizonGlare( void );
		static void Platform_RenderLensFlare( void );

        //
        // Sun effects.
        //

        enum
        {
            SUNFLAG_GLARE = 1,
			SUNFLAG_LENSFLARE = 2,
			SUNFLAG_HORIZONGLARE = 4,
        };

        static u_int          s_uSunFlags; // See SUNFLAG_... constants above.
        static Asura_Vector_3 s_xSunPos;
        static float          s_fSunColourR, s_fSunColourG, s_fSunColourB;
		static float		  s_fHorizonGlareSizeDown, s_fHorizonGlareSizeUp;

		#ifdef ASURA_TOOLS
		static bool s_bToolsDisableSunGlare;
		#endif

};

#endif//__ASURA_WEATHER_SYSTEM_H__
