//
// Asura_PFX_Entity_Effect.cpp
//
// Created by KOR - 15/07/2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"


// This file's header.
#include "SpecialFX/PFX/Asura_PFX_Entity_Effect.h"

// Asura headers
#include "Core/Asura_Spline.h"
#include "Core/Asura_Timers.h"
#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
#include "Core/Handlers/Asura_Handler_System.h"
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
#if _DEBUG
#include "Debugging/Asura_Logfile.h"
#endif
#include "Light/Asura_Photon.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Sound_Event_System.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "SpecialFX/PFX/Asura_PFX_Classes.h"
#include "SpecialFX/PFX/Asura_PFX_Source.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "SpecialFX/PFX/Asura_PFX_Templates.h"

// Tools headers
#ifdef ASURA_TOOLS
#include "Particles/Tools_Mode_PFX.h"
#endif

#ifdef ASURA_TOOLS
#define ASURA_PFX_ENTITY_EFFECT_MAX_PHOTONS (256)
#endif

/// Asura_PFX_Entity_Effect ///////////////////////////////////////////////////////////////////////

// Static create method
unsigned int Asura_PFX_Entity_Effect::Create(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xDirection, Asura_Hash_ID uTemplate, const Asura_PFX_Volume* pxVolumeOverride, u_short usPFXFlags, Asura_Guid uParentGuid)
{
	Asura_Assert(uTemplate!=ASURA_HASH_ID_UNSET , "Asura_PFX_Entity_Effect::Create called with unset template hash.");

	if(uTemplate!=ASURA_HASH_ID_UNSET && Asura_PFX_System::CanCreateEffect())
	{
		Asura_PFX_Entity_Effect* pxEffect = new Asura_PFX_Entity_Effect(xPosition, xDirection, uTemplate, pxVolumeOverride, usPFXFlags, uParentGuid);
		if(pxEffect && pxEffect->GetState() != ASURA_PFX_EFFECT_DESTROYED)
		{
			return pxEffect->GetID();
		}
	}
	return ASURA_PFX_INVALID_EFFECT_ID;
}

unsigned int Asura_PFX_Entity_Effect::Create(const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, Asura_Hash_ID uTemplate, const Asura_PFX_Volume* pxVolumeOverride, u_short usPFXFlags, Asura_Guid uParentGuid)
{
	Asura_Matrix_3x3 xOri;
	Asura_Maths::OrientationFromDirection(xOri, xDirection);
	return Create(xPosition, xOri, uTemplate, pxVolumeOverride, usPFXFlags, uParentGuid);
}

// Constructor
Asura_PFX_Entity_Effect::Asura_PFX_Entity_Effect(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xDirection, Asura_Hash_ID uTemplate, const Asura_PFX_Volume* pxVolumeOverride, u_short usPFXFlags, Asura_Guid uParentGuid )
: Asura_PFX_Effect(usPFXFlags)
, m_xPosition(xPosition)
, m_xDirection(xDirection)
, m_ppxPhotons(NULL)
, m_uCurrentNumberOfPhotons(0)
, m_iSoundEventHandle(ASURA_INVALID_HANDLE)
, m_pxSpline(NULL)
, m_fAge(0.f)
, m_uParentGuid( uParentGuid )
, m_uFlags(0) // initialise individual flags in body below
, m_ppxSources(NULL)
{	
	m_bSourceVolumeOverridden = (pxVolumeOverride!=0);
	m_bHasCollisionPlane      = false;
	m_bPreserveWhileInactive  = (usPFXFlags & ASURA_PFX_EFFECT_PRESERVE_WHILE_INACTIVE) > 0;
	m_bNeedToPregenerate	  = true;
	m_bInvisibleByAttachment  = false;
	m_fMaxParticleLifespan = 0.0f;

	m_xMovementBB.SetToPoint(m_xPosition);
	m_xPrevMovementBB.SetToPoint(m_xPosition);
	m_fMovementBBTimer = 0.f;
	m_bUsingMovementBBs = false;

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	//Find out which zone this belongs to. RAR 20/10/2006
	m_uZoneHashID = Asura_Handler_System::GetLastSetAllZoneDataHash();
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)

	m_pxTemplate = Asura_PFX_System::GetEffectTempByHash(uTemplate);
	Asura_Assertf(m_pxTemplate, ("No template found for effect (0x%08x) (%s) (%f, %f, %f)",
		uTemplate,
		Asura_HashedFilename_System::GetFilename(uTemplate, false),
		Asura_Maths::FtoD(m_xPosition.GetX()), Asura_Maths::FtoD(m_xPosition.GetY()), Asura_Maths::FtoD(m_xPosition.GetZ()) ) );
	if(m_pxTemplate)
	{
		#ifdef ASURA_TOOLS
		//In tools we may want to view the effects for another platform. RAR 01/04/2005
		m_pxTemplate = m_pxTemplate->GetTemplateForPlatform(Tools_Mode_PFX::GetPreviewPlatform());
		#endif
		
		m_pxTemplate->IncrementNumInstances();

		m_ppxSources = new Asura_PFX_Source*[m_pxTemplate->m_uNumSources];
		bool bValidSource = false;
		const Asura_PFX_Effect_SourceProperties* pxSourceProp = m_pxTemplate->m_pxSources;
		for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource, ++pxSourceProp)
		{
			if(pxVolumeOverride)
			{
				//use the passed volume/position/orientation instead of the values in the template
				m_ppxSources[uSource] = new Asura_PFX_Source(this, 
															 xPosition,
															 xDirection,
															 pxSourceProp->m_uTemplate,
															 *pxVolumeOverride);

			}
			else
			{
				m_ppxSources[uSource] = new Asura_PFX_Source(this, 
															 pxSourceProp->m_xOffset * xDirection + xPosition,
															 xDirection * pxSourceProp->m_xOrient,
															 pxSourceProp->m_uTemplate,
															 pxSourceProp->m_xVolume);
			}
			Asura_PFX_Source_Template *pxTemplate = Asura_PFX_System::GetSourceTempByHash(m_ppxSources[uSource]->m_uTemplateNameHash);
			Asura_PFX_Particle_Template *pxParticleTemplate = pxTemplate ? Asura_PFX_System::GetParticleTempByHash(pxTemplate->m_uParticleTemplateHash) : NULL;

			if(pxTemplate && pxParticleTemplate)
			{
				//Set the velocity regardless of dynamics settings while in tools , since
				//it will mean that the source will work correctly if m_bUseSourceDynamics
				//is toggled on while the particle system is being previewed.
				//RAR 15/09/04
				#ifndef ASURA_TOOLS
				if(pxTemplate->m_bUseSourceDynamics)
				#endif
				{
					m_ppxSources[uSource]->SetVelocity(pxSourceProp->m_xVelocity * xDirection);
				}
				bValidSource = true;
			}

			if(pxParticleTemplate)
			{
				//get the longest living particle type
				m_fMaxParticleLifespan = Asura_Maths::Max(m_fMaxParticleLifespan,pxParticleTemplate->GetMaxLifetime());
			}
		}
		if(bValidSource)
		{
			if(usPFXFlags & ASURA_PFX_EFFECT_START_INACTIVE)
			{
				m_eState = ASURA_PFX_EFFECT_STANDBY;
			}
			else
			{
				OnStart();
			}
			// Can only calculate BB if we have a valid source.
			CalculateBoundingBox();
		}
		else
		{
			m_eState = ASURA_PFX_EFFECT_DESTROYED;
		}
		if(m_pxTemplate->m_bAlwaysOnTop)
		{
			m_usPFXFlags |= (ASURA_PFX_EFFECT_HUD_EFFECT | ASURA_PFX_EFFECT_ALWAYS_ON_TOP);
		}
		if(m_pxTemplate->m_bHudEffect)
		{
			m_usPFXFlags |= ASURA_PFX_EFFECT_HUD_EFFECT;
		}

		#ifdef ASURA_TOOLS
		m_bTemplateWasAlwaysOnTop = m_pxTemplate->m_bAlwaysOnTop;
		#endif

		if ( m_pxTemplate->m_bForceRenderOnMainThread )
		{
			m_usPFXFlags |= ASURA_PFX_EFFECT_FORCE_RENDER_ON_MAIN_THREAD;
		}
	}
	else
	{
		m_eState = ASURA_PFX_EFFECT_DESTROYED;
	}

	// Signal that this is indeed an entity effect so Asura_PFX_System::GetEntityEffect can confirm.
	// Could have been done with a virtual function but I'd rather avoid bloating the vtable.
	m_usPFXFlags |= ASURA_PFX_EFFECT_ENTITY_EFFECT;
}

// Destructor
Asura_PFX_Entity_Effect::~Asura_PFX_Entity_Effect()
{

	if (m_pxTemplate)
	{
		Asura_Assert(m_pxTemplate->GetNumInstances(), "this PFX template says it has no instances but we're destroying one");
		m_pxTemplate->DecrementNumInstances();

		if(m_ppxSources)
		{
			for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource)
			{
				delete m_ppxSources[uSource];
			}
			delete [] m_ppxSources;
		}
	}
	ShutdownPhotons();
	if( m_iSoundEventHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Sound_Event_System::Stop(m_iSoundEventHandle);
	}
}

void Asura_PFX_Entity_Effect::Update()
{
	if(!m_ppxSources) { return; }

	const float fTimeSlice = Asura_Timers::GetGameTimeSlice();

	if(m_bUsingMovementBBs)
	{
		//have been moving recently , see if we can contract the bounding box because of expired
		//particles
		m_fMovementBBTimer += fTimeSlice;
		if(m_fMovementBBTimer >= m_fMaxParticleLifespan)
		{
			//All particles generated in the time tracked by m_xPrevMovementBB will now
			//have expired (since they will have been generated at least m_fMovementBBTimer seconds ago).
			//So , we can discount that section of movement
			m_fMovementBBTimer = 0.0f;
			m_xPrevMovementBB = m_xMovementBB;
			m_xMovementBB.SetToPoint(m_xPosition);
			//if the 2 bounding are both just equal to a point , then the effect
			//has been stationary for long enough that we can ignore any movement
			m_bUsingMovementBBs = (m_xMovementBB != m_xPrevMovementBB);

			//update the bounding box in the effect
			Asura_Bounding_Box xBB = m_xPrevMovementBB;

			xBB.MinX += m_xRelativeStaticBB.MinX;
			xBB.MinY += m_xRelativeStaticBB.MinY;
			xBB.MinZ += m_xRelativeStaticBB.MinZ;

			xBB.MaxX += m_xRelativeStaticBB.MaxX;
			xBB.MaxY += m_xRelativeStaticBB.MaxY;
			xBB.MaxZ += m_xRelativeStaticBB.MaxZ;

			SetBoundingBox(xBB);
		}
	}
	const bool bLinkedIn = GetCameraFlags(false) != 0;

	if(bLinkedIn && !m_bWasLinkedIn)
	{
		if(m_pxTemplate->m_bPregenerate && m_pxTemplate->m_bLooping)
		{
			Pregenerate();
		}
	}
	m_bWasLinkedIn = bLinkedIn;

	//Determine visibility of this system. Ignore the frustum test so the particles will
	//still get moved if nearby but behind the player.
	// KOR - 25/09/2008 - if the effect is not moving and is looping,
	// it doesn't really matter if the effect doesn't update outside the frustum.
	bool bInFrustum = false;
	if(bLinkedIn)
	{
		const bool bMoving = (m_xDelta.MagnitudeSquared() > 0.f);
		const bool bUpdateOutsideFrustum = (bMoving || !m_pxTemplate->m_bLooping);
		bInFrustum = bUpdateOutsideFrustum || (GetCameraFlags(true) != 0);
	}

	const float fAgeIncrement = m_pxTemplate->m_fLifeTime > 0.0f ? fTimeSlice / m_pxTemplate->m_fLifeTime : 0.0f;

	m_fAge += fAgeIncrement;
	if(m_fAge > 1.f)
	{
		if(m_pxTemplate->m_bLooping && m_eState == ASURA_PFX_EFFECT_RUNNING)
		{
			Start();
			//Initialising the entity age to take the full frame time after a loop , so
			//that it matches the behaviour of the sources (which will also add the full
			//frame time in the upcoming update). RAR 13/12/2004
			m_fAge = fAgeIncrement;
		}
		else
		{
			// (AW - 20.10.2004) - Set as inactive so this will disappear when particles have
			// finished.
			m_eState = ASURA_PFX_EFFECT_STOPPING;
		}
	}

#ifdef ASURA_TOOLS
	// Always on top could be toggled in tools
	if(m_pxTemplate->m_bAlwaysOnTop != m_bTemplateWasAlwaysOnTop)
	{
		if(m_pxTemplate->m_bAlwaysOnTop)
		{
			m_usPFXFlags |= (ASURA_PFX_EFFECT_HUD_EFFECT | ASURA_PFX_EFFECT_ALWAYS_ON_TOP);
		}
		else
		{
			m_usPFXFlags &= ~(ASURA_PFX_EFFECT_HUD_EFFECT | ASURA_PFX_EFFECT_ALWAYS_ON_TOP);
		}
		if(m_pxTemplate->m_bHudEffect)
		{
			m_usPFXFlags |= ASURA_PFX_EFFECT_HUD_EFFECT;
		}
		m_bTemplateWasAlwaysOnTop = m_pxTemplate->m_bAlwaysOnTop;
	}


	// Photons could be toggled on or off in tools
	if (m_pxTemplate->m_bHasPhoton && ((m_usPFXFlags & ASURA_PFX_EFFECT_DONT_CREATE_LIGHT) == 0))
	{
		InitialisePhotons();
	}
	else
	{
		ShutdownPhotons();
	}
#endif
	// Update photon properties
	UpdatePhotons();

	// Update sources
	const float fSeconds = m_fAge * m_pxTemplate->m_fLifeTime;
	const Asura_PFX_Effect_SourceProperties* pxSourceProp = m_pxTemplate->m_pxSources;
	for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource , ++pxSourceProp)
	{
		if(fSeconds > pxSourceProp->m_fTimeOffset)
		{
			m_ppxSources[uSource]->Update(bLinkedIn, bInFrustum, m_pxSpline);
		}
	}

	// Update sound
	UpdateSound(fTimeSlice);

	// check if we need to stop and if we should
	if(m_eState == ASURA_PFX_EFFECT_STOPPING)
	{
		bool bFinish = true;
		// Check sound if there is one
		if(m_iSoundEventHandle != ASURA_INVALID_HANDLE)
		{
			if(Asura_Sound_Event_System::IsLooping(m_iSoundEventHandle))
			{
				Asura_Sound_Event_System::Stop(m_iSoundEventHandle);
			}
			else
			{
				bFinish = !Asura_Sound_Event_System::IsPlaying(m_iSoundEventHandle);
			}
		}
		// If there's no playing sound, check through sources for remaining particles
		if(bFinish)
		{
			for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource)
			{
				if(m_ppxSources[uSource]->m_uNumParticles)
				{
					// Keep alive while there are any particles
					bFinish = false;
				}
			}
		}
		if(bFinish)
		{
			OnFinish();
		}
	}
}

void Asura_PFX_Entity_Effect::Render()
{
	if(m_ppxSources && !m_bInvisibleByAttachment)
	{
#ifdef ASURA_TOOLS
		if(m_pxTemplate->m_iIsolatedSource != -1)
		{
			m_ppxSources[m_pxTemplate->m_iIsolatedSource]->Render();
		}
		else
#endif
		for(int iSource = m_pxTemplate->m_uNumSources - 1; iSource >= 0 ; --iSource)
		{
			#ifdef ASURA_TOOLS
			if(m_pxTemplate->m_pxSources[iSource].m_bDisabled) continue;
			#endif
			
			m_ppxSources[iSource]->Render(m_pxTemplate->m_bOrderedRender);
		}
	}
}

void Asura_PFX_Entity_Effect::Stop(bool bImmediate)
{
	if(bImmediate)
	{
		OnFinish();
	}
	else
	{
		m_eState = ASURA_PFX_EFFECT_STOPPING;
	}
	if(m_ppxSources)
	{
		for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource)
		{
			m_ppxSources[uSource]->Stop(bImmediate);
		}
	}
}

void Asura_PFX_Entity_Effect::Start()
{
	m_fAge = 0.f;
	// Reset source ages as well
	//And positions and orientations
	const Asura_PFX_Effect_SourceProperties* pxSourceProp = m_pxTemplate->m_pxSources;
	for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource, ++pxSourceProp)
	{
		m_ppxSources[uSource]->m_fAge = 0.f;
		m_ppxSources[uSource]->m_bHasDoneBurst = false;
		//Not resetting m_fParticleReleaseCounter so as to prevent discontinuities in looping
		//effects that are supposed to be generating a constant stream of particles. 
		//RAR 30/08/2005
		if(m_bSourceVolumeOverridden)
		{
			m_ppxSources[uSource]->m_xPosition  = m_xPosition;
			m_ppxSources[uSource]->m_xDirection = m_xDirection;
		}
		else
		{
			m_ppxSources[uSource]->m_xPosition  = pxSourceProp->m_xOffset * m_xDirection + m_xPosition;
			m_ppxSources[uSource]->m_xDirection = m_xDirection * pxSourceProp->m_xOrient ;
		}

		Asura_PFX_Source_Template *pxTemplate = Asura_PFX_System::GetSourceTempByHash(m_ppxSources[uSource]->m_uTemplateNameHash);
		if(pxTemplate)
		{
			#ifndef ASURA_TOOLS
			if(pxTemplate->m_bUseSourceDynamics)
			#endif
			{
				m_ppxSources[uSource]->SetVelocity(pxSourceProp->m_xVelocity * m_xDirection);
			}
		}
	}
	OnStart();
}

void Asura_PFX_Entity_Effect::SetPosition(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xDirection,bool bTeleport)
{
	SetPosAndOri(xPosition, &xDirection, bTeleport);
}

void Asura_PFX_Entity_Effect::SetPosition(const Asura_Vector_3& xPosition,bool bTeleport)
{
	SetPosAndOri(xPosition, NULL, bTeleport);
}

void Asura_PFX_Entity_Effect::SetPosAndOri(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3* pxOri, bool bTeleport)
{
	if(!m_pxTemplate) return;

	if(m_xPosition != xPosition)
	{
		//Update the culling bounding box to take into account the move.
		if(m_usPFXFlags & ASURA_PFX_EFFECT_MOVE_PARTICLES_WITH_EFFECT)
		{
			//The particles are being moved with the entity , so we just need
			//to move the bounding box
			Asura_Bounding_Box xBB = m_xRelativeStaticBB;
			xBB.Displace(xPosition);
			SetBoundingBox(xBB);
		}
		else
		{
			
			if(!m_bUsingMovementBBs)
			{
				m_bUsingMovementBBs = true;
				m_xMovementBB.SetToPoint(m_xPosition);
				m_xPrevMovementBB.SetToPoint(m_xPosition);
				m_fMovementBBTimer = 0.0f;
			}
			
			//expand the bounding box for keeping track of most recent movement
			m_xMovementBB.Set(xPosition);

			//combine the bounding boxes of all recent movement
			Asura_Bounding_Box xBB = m_xMovementBB;
			xBB.Set(m_xPrevMovementBB);

			//and add on the static BB to determine the maximum area that could be covered
			xBB.MinX += m_xRelativeStaticBB.MinX;
			xBB.MinY += m_xRelativeStaticBB.MinY;
			xBB.MinZ += m_xRelativeStaticBB.MinZ;

			xBB.MaxX += m_xRelativeStaticBB.MaxX;
			xBB.MaxY += m_xRelativeStaticBB.MaxY;
			xBB.MaxZ += m_xRelativeStaticBB.MaxZ;

			SetBoundingBox(xBB);

		}
	}

	if(m_ppxSources)
	{
		const Asura_PFX_Effect_SourceProperties* pxSourceProp = m_pxTemplate->m_pxSources;
		for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource, ++pxSourceProp)
		{
			Asura_Matrix_3x3 xOldDirection = m_ppxSources[uSource]->m_xDirection;
			Asura_Vector_3   xOldPosition  = m_ppxSources[uSource]->m_xPosition;

			if(m_bSourceVolumeOverridden)
			{
				m_ppxSources[uSource]->m_xPosition  = xPosition;
				if(pxOri)
				{
					m_ppxSources[uSource]->m_xDirection = *pxOri;
				}
			}
			else
			{
				if(pxOri)
				{
					m_ppxSources[uSource]->m_xPosition  = pxSourceProp->m_xOffset * (*pxOri) + xPosition;
					m_ppxSources[uSource]->m_xDirection = (*pxOri) * pxSourceProp->m_xOrient;
				}
				else
				{
					m_ppxSources[uSource]->m_xPosition  = pxSourceProp->m_xOffset * m_xDirection + xPosition;
				}
			}

			bool bMoveParticles = false;
			bool bRotateParticles = false;

			Asura_PFX_Source_Template *pxTemplate = Asura_PFX_System::GetSourceTempByHash(m_ppxSources[uSource]->m_uTemplateNameHash);
			if(pxTemplate)
			{
				if(pxTemplate->m_bMovePclesWithSource)
				{
					bMoveParticles = true;
				}
				else
				#ifndef ASURA_TOOLS
				if(pxTemplate->m_bUseSourceDynamics)
				#endif
				{
					if(pxOri)
					{
						m_ppxSources[uSource]->SetVelocity(pxSourceProp->m_xVelocity * (*pxOri));
					}
				}

				if(pxTemplate->m_bRotatePclesWithSource)
				{
					bRotateParticles = true;
				}
			}
			if(m_usPFXFlags & ASURA_PFX_EFFECT_MOVE_PARTICLES_WITH_EFFECT || m_pxTemplate->m_bMoveWithEffect)
			{
				bMoveParticles = true;
			}

			if (bMoveParticles)
			{
				Asura_Vector_3 xDisplacement = xPosition - m_xPosition;

				
				if(pxOri && bRotateParticles)
				{
					for (u_int i=0;i<m_ppxSources[uSource]->m_uNumParticles;i++)
					{
						m_ppxSources[uSource]->m_pxParticles[i].m_xPosition -= m_xPosition;
						m_ppxSources[uSource]->m_pxParticles[i].m_xPosition  = m_xDirection.ApplyTranspose( m_ppxSources[uSource]->m_pxParticles[i].m_xPosition );
						m_ppxSources[uSource]->m_pxParticles[i].m_xPosition  = m_ppxSources[uSource]->m_pxParticles[i].m_xPosition * *pxOri;												
						m_ppxSources[uSource]->m_pxParticles[i].m_xPosition += xPosition;

						m_ppxSources[uSource]->m_pxParticles[i].m_xVelocity  = xOldDirection.ApplyTranspose( m_ppxSources[uSource]->m_pxParticles[i].m_xVelocity );
						m_ppxSources[uSource]->m_pxParticles[i].m_xVelocity  = m_ppxSources[uSource]->m_pxParticles[i].m_xVelocity * m_ppxSources[uSource]->m_xDirection;
					}
				}
				else
				{				
					// (AW - 21.10.2004) - Update particle positions. This is required in certain 
					// cases to keep particles in the appropriate positions for e.g. Muzzle flashes.
					// This won't work if used in conjunction with certain other options like dynamics.				
					for (u_int i=0;i<m_ppxSources[uSource]->m_uNumParticles;i++)
					{
						m_ppxSources[uSource]->m_pxParticles[i].m_xPosition += xDisplacement;
					}
				}
			}
		}
	}

	if(bTeleport)
	{
		m_xDelta.Zero();
	}
	else if( !((m_usPFXFlags & ASURA_PFX_EFFECT_MOVE_PARTICLES_WITH_EFFECT) || m_pxTemplate->m_bMoveWithEffect) )
	{
		m_xDelta = xPosition - m_xPosition;
	}

	m_xPosition = xPosition;
	if(pxOri)
	{
		m_xDirection = (*pxOri);
	}
}

void Asura_PFX_Entity_Effect::SetOrientation(const Asura_Vector_3& xDirection, int iOrientationID /* = 0 */)
{
	Asura_Matrix_3x3 xOri;
	Asura_Maths::OrientationFromDirection(xOri, xDirection);
	SetOrientation(xOri, iOrientationID);
}

void Asura_PFX_Entity_Effect::SetOrientation(const Asura_Matrix_3x3& xOrientation, int iOrientationID /* = 0 */)
{
	if(!m_pxTemplate) return;

	if(m_ppxSources)
	{
		const Asura_PFX_Effect_SourceProperties* pxSourceProp = m_pxTemplate->m_pxSources;
		for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource, ++pxSourceProp)
		{
			// Only update the requested orientations
			if(pxSourceProp->m_iOrientationID == iOrientationID)
			{
				if(m_bSourceVolumeOverridden)
				{
					m_ppxSources[uSource]->m_xDirection = xOrientation;
				}
				else
				{
					m_ppxSources[uSource]->m_xPosition  = pxSourceProp->m_xOffset * xOrientation + m_xPosition;
					m_ppxSources[uSource]->m_xDirection = xOrientation * pxSourceProp->m_xOrient;
				}
			}
		}
	}
	// ID zero is the default one.
	if(!iOrientationID)
	{
		m_xDirection = xOrientation;
	}
}

void Asura_PFX_Entity_Effect::SetAge(float fAge)
{
	m_fAge = fAge;
	float fAgeInSeconds = m_fAge * m_pxTemplate->m_fLifeTime;
	for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource)
	{
		Asura_PFX_Source_Template *pxTemplate = Asura_PFX_System::GetSourceTempByHash(m_ppxSources[uSource]->m_uTemplateNameHash);

		if(!pxTemplate) continue;
		Asura_Assert(pxTemplate->m_fDuration > 0.0f, "We need a duration in order to set the age of a source.");
		if (pxTemplate->m_fDuration > 0.0f)
		{
			m_ppxSources[uSource]->m_fAge = fAgeInSeconds / pxTemplate->m_fDuration;
		}
		else
		{
			m_ppxSources[uSource]->m_fAge = 1.0f;
		}
	}
}

void Asura_PFX_Entity_Effect::SetInvisibleByAttachment( bool bInvisible )
{
	Asura_Assert((m_bInvisibleByAttachment ? true : false) != bInvisible, "Visibility already set");
	m_bInvisibleByAttachment = bInvisible;
}

void Asura_PFX_Entity_Effect::CalculateBoundingBox()
{
	if(!m_ppxSources) return;

	Asura_Bounding_Box xBB;
	xBB.Reset();

	if(m_pxSpline)
	{
		// Find the maximum particle size across all sources.
		float fMaxSize = 0.f;
		for(u_int uSource=0;uSource<m_pxTemplate->m_uNumSources;uSource++)
		{
			Asura_PFX_Source_Template *pxTemplate = Asura_PFX_System::GetSourceTempByHash(m_ppxSources[uSource]->m_uTemplateNameHash);
			Asura_PFX_Particle_Template *pxParticleTemplate = pxTemplate ? Asura_PFX_System::GetParticleTempByHash(pxTemplate->m_uParticleTemplateHash) : NULL;

			const float fSourceMaxSize = pxParticleTemplate->m_xSize.GetMaxDiscreteValue();
			fMaxSize = Asura_Maths::Max(fMaxSize, fSourceMaxSize);
		}
		// Use the spline points to generate the BB.
		const u_int uNumSplinePoints = m_pxSpline->GetNumberOfPoints();
		for(u_int uPoint = 1; uPoint < uNumSplinePoints - 1; ++uPoint)
		{
			xBB.Set(m_pxSpline->GetPointPosition(uPoint));
		}
		// Expand by max particle size.
		xBB.Grow(fMaxSize);
	}
	else
	{
		for(u_int uSource=0;uSource<m_pxTemplate->m_uNumSources;uSource++)
		{
			m_ppxSources[uSource]->ExpandBoundingBox(xBB);
		}
	}

	m_xRelativeStaticBB = xBB;
	m_xRelativeStaticBB.Displace(-m_xPosition);

	SetBoundingBox(xBB);
}

void Asura_PFX_Entity_Effect::OnStart()
{
	m_eState = ASURA_PFX_EFFECT_RUNNING;
	// Activate photon if template says so
	if (m_pxTemplate->m_bHasPhoton && ((m_usPFXFlags & ASURA_PFX_EFFECT_DONT_CREATE_LIGHT) == 0))
	{
        // count the max number of particles required
        u_int uMaxTotalParticles = 0;
		for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource)
		{
			if(m_pxTemplate->m_iPhotonPerParticleSource==-1 || m_pxTemplate->m_iPhotonPerParticleSource == (int)uSource)
			{
				Asura_PFX_Source_Template *pxTemplate = Asura_PFX_System::GetSourceTempByHash(m_ppxSources[uSource]->m_uTemplateNameHash);

				if(pxTemplate)
				{
					uMaxTotalParticles += pxTemplate->m_uMaxParticles;
				}
			}
        }

        InitialisePhotons( uMaxTotalParticles );
		// (AW - 23.03.2005) - Setup initial lighting values for this effect. Fixes occasional
		// lighting corruption bug.
		UpdatePhotons();
	}
	
	if(m_pxTemplate->m_uSoundEvent)
	{
		if(m_iSoundEventHandle != ASURA_INVALID_HANDLE)
		{
			//If already playing a looping sound event , then we keep it
			if(!Asura_Sound_Event_System::IsLooping(m_iSoundEventHandle))
			{
				Asura_Sound_Event_System::Stop(m_iSoundEventHandle);
			}
		}
		
		if(m_iSoundEventHandle == ASURA_INVALID_HANDLE)		
		{
#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
			//For streaming builds , we need to make sure that the sound event is found from the right handler
			//RAR 20/10/2006
			Asura_Sound_Event_Handler* pxHandler = Asura_Sound_Event_Storage::GetHandler(m_uZoneHashID);
			Asura_Assert(pxHandler,"Failed to get sound event handler for PFX's zone");
			const Asura_Sound_Event* pxEvent = pxHandler ? pxHandler->FindEvent(m_pxTemplate->m_uSoundEvent) : 0;
#else
			const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent(m_pxTemplate->m_uSoundEvent);
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
			if(pxEvent)
			{
				Asura_Sound_Event_System::PlayEvent(pxEvent,&m_iSoundEventHandle,&m_xPosition,1.0f,m_uParentGuid);
			}
		}
	}

	// Only pregenerate if looping - doesn't make sense otherwise
	if(m_pxTemplate->m_bPregenerate && m_bNeedToPregenerate && m_pxTemplate->m_bLooping)
	{
		Pregenerate();
		m_bNeedToPregenerate = false;
	}
}

void Asura_PFX_Entity_Effect::Pregenerate()
{
	Asura_Assert(m_ppxSources, "No sources");
	if(!m_ppxSources) return;
	// Loop through sources
	Asura_PFX_Source *pThisSource;
	for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource)
	{
		pThisSource = m_ppxSources[uSource];
		Asura_PFX_Source_Template *pxTemplate = Asura_PFX_System::GetSourceTempByHash(pThisSource->m_uTemplateNameHash);
		Asura_PFX_Particle_Template *pxParticleTemplate = pxTemplate ? Asura_PFX_System::GetParticleTempByHash(pxTemplate->m_uParticleTemplateHash) : NULL;

		Asura_Assert(pxTemplate, "No source template");
		Asura_Assert(pxParticleTemplate, "No particle template");
		if(!pxTemplate || !pxParticleTemplate) return;
		if(!pThisSource->m_bInRange) return;		// don't allow particles to restart if the source isn't in range !!

		const float fSourceLifeTime = pxTemplate->m_fDuration;
		const float fParticleLifeTime = pxParticleTemplate->GetMaxLifetime();
		// Check whether source lifetime is shorter than effect lifetime
		if(fSourceLifeTime < m_pxTemplate->m_fLifeTime)
		{
			pThisSource = m_ppxSources[uSource];
			 
			if(!pThisSource->m_bInRange) return;		// don't allow particles to restart if the source isn't in range !!

			// Generate in waves
			float fTime = 0.f;
			while(fTime < fParticleLifeTime)
			{
				pThisSource->Generate(fTime, fSourceLifeTime);
				fTime += m_pxTemplate->m_fLifeTime;
			}
		}
		else
		{
			// Generate in one block
			pThisSource->Generate(0.f, fParticleLifeTime);
		}
	}
}

void Asura_PFX_Entity_Effect::OnFinish()
{
	if(m_bPreserveWhileInactive)
	{
		m_eState = ASURA_PFX_EFFECT_STANDBY;
	}
	else
	{
		m_eState = ASURA_PFX_EFFECT_DESTROYED;
	}
	ShutdownPhotons();
	// Kill off the sound
	if(m_iSoundEventHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Sound_Event_System::Stop(m_iSoundEventHandle);
	}
	m_bNeedToPregenerate = true;
}


#ifdef ASURA_TOOLS
void Asura_PFX_Entity_Effect::UpdateNumberOfSources(u_int iNumberOfSources)
{
	// Check for any change
	if(m_pxTemplate->m_uNumSources == iNumberOfSources) return;

	// Allocate new array
	Asura_PFX_Source** ppxNewSources = new Asura_PFX_Source*[iNumberOfSources];

	u_int uSource;
	if(iNumberOfSources < m_pxTemplate->m_uNumSources)
	{
		for(uSource = 0; uSource < iNumberOfSources; ++uSource)
		{
			ppxNewSources[uSource] = m_ppxSources[uSource];
		}
		for(; uSource < m_pxTemplate->m_uNumSources; ++uSource)
		{
			delete m_ppxSources[uSource];
		}
	}
	else
	{
		for(uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource)
		{
			ppxNewSources[uSource] = m_ppxSources[uSource];
		}
		for(; uSource < iNumberOfSources; ++uSource)
		{
			ppxNewSources[uSource] = new Asura_PFX_Source(this, 
															m_xPosition, 
															m_xDirection, 
															m_pxTemplate->m_pxSources[uSource].m_uTemplate, 
															m_pxTemplate->m_pxSources[uSource].m_xVolume);
		}
	}
	delete [] m_ppxSources;
	m_ppxSources = ppxNewSources;
}

void Asura_PFX_Entity_Effect::UpdateMaximumParticles(u_int uSourceNumber, u_int uMaximumParticles)
{
	Asura_Assert(uSourceNumber < m_pxTemplate->m_uNumSources, "Source out of range");
	if(uSourceNumber >= m_pxTemplate->m_uNumSources) return;

	// Allocate new array
	Asura_PFX_Particle* pxNewPcles = new Asura_PFX_Particle[uMaximumParticles];
	Asura_PFX_Particle* pxOldPcles = m_ppxSources[uSourceNumber]->m_pxParticles;

	u_int uParticle;
	if(uMaximumParticles < m_ppxSources[uSourceNumber]->m_uNumParticles)
	{
		for(uParticle = 0; uParticle < uMaximumParticles; ++uParticle)
		{
			pxNewPcles[uParticle] = pxOldPcles[uParticle];
		}
		 m_ppxSources[uSourceNumber]->m_uNumParticles = uMaximumParticles;
	}
	else
	{
		for(uParticle = 0; uParticle < m_ppxSources[uSourceNumber]->m_uNumParticles; ++uParticle)
		{
			pxNewPcles[uParticle] = pxOldPcles[uParticle];
		}
	}
	delete [] m_ppxSources[uSourceNumber]->m_pxParticles;
	m_ppxSources[uSourceNumber]->m_pxParticles = pxNewPcles;
}

Asura_Hash_ID Asura_PFX_Entity_Effect::GetCurrentTemplateHash() const
{
	return m_pxTemplate ? m_pxTemplate->GetTemplateHash() : ASURA_HASH_ID_UNSET;
}

#endif

u_int Asura_PFX_Entity_Effect::GetCurNumParticles( void ) const
{
    u_int uCount = 0;

    u_int uNumSources = m_pxTemplate->m_uNumSources;
    for( u_int u = 0; u < uNumSources; ++u )
    {
        uCount += m_ppxSources[u]->m_uNumParticles;
    }

    return uCount;
}

void Asura_PFX_Entity_Effect::GenerateParticles(u_int uSourceNum, u_int uNumParticles)
{
    Asura_Assert(uSourceNum < m_pxTemplate->m_uNumSources, "Source number out of range");
	Asura_Assert(m_ppxSources[uSourceNum], "Missing source");
	if(m_ppxSources[uSourceNum])
	{
		while(uNumParticles--)
		{
			m_ppxSources[uSourceNum]->CreateParticle(0.f);
		}
	}
}

void Asura_PFX_Entity_Effect::SetVelocity(const Asura_Vector_3& xVelocity)
{
	m_xDelta = xVelocity * Asura_Timers::GetGameTimeSlice();
}

const Asura_PFX_Source* Asura_PFX_Entity_Effect::GetSource(u_int uSourceNum) const
{
	Asura_Assert(uSourceNum < m_pxTemplate->m_uNumSources, "Source out of range");
	if(uSourceNum < m_pxTemplate->m_uNumSources)
	{
		return m_ppxSources[uSourceNum];
	}
	return NULL;
}

void Asura_PFX_Entity_Effect::InitialisePhotons(u_int uMaxTotalParticles /* = 0 */)
{
	// Make sure we have the array
	if(!m_ppxPhotons)
	{
		u_int uPhotonArraySize;
#ifdef ASURA_TOOLS
		uPhotonArraySize = ASURA_PFX_ENTITY_EFFECT_MAX_PHOTONS;
#else//ASURA_TOOLS
		if(m_pxTemplate->m_bParticlePhotons)
		{
			uPhotonArraySize = uMaxTotalParticles;
		}
		else
		{
			uPhotonArraySize = 1;
		}
#endif//!ASURA_TOOLS
		m_ppxPhotons = new Asura_Photon*[uPhotonArraySize];
		for(u_int uPhoton = 0; uPhoton < uPhotonArraySize; ++uPhoton)
		{
			m_ppxPhotons[uPhoton] = 0;
		}
		m_uCurrentNumberOfPhotons = 0;
	}
	// Make sure we have the correct number of photons
	if(!m_pxTemplate->m_bParticlePhotons)
	{
		if(!m_uCurrentNumberOfPhotons)
		{
			m_ppxPhotons[0] = Asura_Photon_System::CreatePhoton();
			Asura_Assert(m_ppxPhotons[0], "Failed to create photon");
			if(m_ppxPhotons[0]) m_uCurrentNumberOfPhotons = 1;
		}
		else
		{
			if(m_uCurrentNumberOfPhotons > 1)
			{
				for(u_int uPhoton = 1; uPhoton < m_uCurrentNumberOfPhotons; ++uPhoton)
				{
					Asura_Photon_System::DestroyPhoton(m_ppxPhotons[uPhoton]);
					m_ppxPhotons[uPhoton] = 0;
				}
			}
		}
	}
	m_bDonePhotonKillCheck = false;
}

void Asura_PFX_Entity_Effect::ShutdownPhotons()
{
	if(m_ppxPhotons)
	{
		while(m_uCurrentNumberOfPhotons)
		{
			Asura_Photon_System::DestroyPhoton(m_ppxPhotons[--m_uCurrentNumberOfPhotons]);
		}
		delete [] m_ppxPhotons;
		m_ppxPhotons = 0;
	}
}

void Asura_PFX_Entity_Effect::UpdatePhotons()
{
	if (m_ppxPhotons)
	{
		if(m_pxTemplate->m_bParticlePhotons)
		{
			u_int uPhoton = 0;
			for(u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource)
			{
				if(m_pxTemplate->m_iPhotonPerParticleSource==-1 || m_pxTemplate->m_iPhotonPerParticleSource == (int)uSource)
				{
					for(u_int uParticle = 0; uParticle < m_ppxSources[uSource]->m_uNumParticles; ++uParticle)
					{
#ifdef ASURA_TOOLS
						if(uPhoton >= ASURA_PFX_ENTITY_EFFECT_MAX_PHOTONS) break;
#endif
						if(!m_ppxPhotons[uPhoton])
						{
							m_ppxPhotons[uPhoton] = Asura_Photon_System::CreatePhoton();
							Asura_Assert(m_ppxPhotons[uPhoton], "Failed to create photon");
							if(!m_ppxPhotons[uPhoton]) continue;
						}
						const float fAge = m_ppxSources[uSource]->m_pxParticles[uParticle].m_fAge;
						const Asura_RGBA xPhotonColour = m_pxTemplate->m_xPhotonColour.GetValueAtTime_Discrete(fAge);
						m_ppxPhotons[uPhoton]->SetColours(xPhotonColour.m_fR, xPhotonColour.m_fG, xPhotonColour.m_fB);
						m_ppxPhotons[uPhoton]->SetRange(m_pxTemplate->m_xPhotonRadius.GetValueAtTime_Discrete(fAge));
						m_ppxPhotons[uPhoton]->SetPosition(m_ppxSources[uSource]->m_pxParticles[uParticle].m_xPosition);
						++uPhoton;
					}
				}
			}
			u_int uNewNumPhotons = uPhoton;
			for(; uPhoton < m_uCurrentNumberOfPhotons; ++uPhoton)
			{
				Asura_Photon_System::DestroyPhoton(m_ppxPhotons[uPhoton]);
				m_ppxPhotons[uPhoton] = 0;
			}
			m_uCurrentNumberOfPhotons = uNewNumPhotons;
		}
		else
		{
			Asura_Assert(m_ppxPhotons[0], "No photon");
			if(!m_ppxPhotons[0]) return;

			const float fRange = m_pxTemplate->m_xPhotonRadius.GetValueAtTime_Discrete(m_fAge);
			bool bKillPhoton = false;
			if(!m_pxTemplate->m_bLooping)
			{
				if(fRange <= 0.f && !m_bDonePhotonKillCheck)
				{
					bKillPhoton = true;
					// Check whether we ever turn back on again.
					Asura_Graph<float>& xGraph = m_pxTemplate->m_xPhotonRadius;
					const u_int uNumSamples = xGraph.GetNumDiscreteSamples();
					const u_int uNextSample = Asura_Maths::FloatToInt_Chop(m_fAge * (uNumSamples - 1)) + 1;
					for(u_int u = uNextSample; u < uNumSamples; ++u)
					{
						if(xGraph.GetDiscreteSample(u) > 0.f)
						{
							bKillPhoton = false;
						}
					}
					m_bDonePhotonKillCheck = true;
				}
				else if(fRange > 0.f && m_bDonePhotonKillCheck)
				{
					// Reset the flag so that we check again next time.
					m_bDonePhotonKillCheck = false;
				}
			}
			if(bKillPhoton)
			{
				ShutdownPhotons();
			}
			else
			{
				const Asura_RGBA xPhotonColour = m_pxTemplate->m_xPhotonColour.GetValueAtTime_Discrete(m_fAge);
				m_ppxPhotons[0]->SetColours(xPhotonColour.m_fR, xPhotonColour.m_fG, xPhotonColour.m_fB);
				m_ppxPhotons[0]->SetRange(fRange);
				m_ppxPhotons[0]->SetPosition(m_xPosition);
			}
		}
	}
}


void Asura_PFX_Entity_Effect::UpdateSound(float fTimeSlice)
{
	if(m_iSoundEventHandle != ASURA_INVALID_HANDLE)
	{
		if(Asura_Sound_Event_System::IsPlaying(m_iSoundEventHandle))
		{
			Asura_Sound_Event_System::UpdatePosition(m_iSoundEventHandle,m_xPosition);		
		}
		else
		{
			Asura_Sound_Event_System::Stop(m_iSoundEventHandle);
		}
	}
}

#if _DEBUG

DEBUGVAR bool dbg_bDumpSources = false;

const char* szPFX_STATS_LOGFILE = "logfiles\\pfx_stats.txt";
void Asura_PFX_Entity_Effect::DumpStats( void )
{
#if defined(ASURA_LOGFILES_ENABLED)
	const char* szState;
	switch( GetState() )
	{
	case ASURA_PFX_EFFECT_RUNNING:   szState = "running";   break;
	case ASURA_PFX_EFFECT_STOPPING:  szState = "stopping";  break;
	case ASURA_PFX_EFFECT_STANDBY:   szState = "standby";   break;
	case ASURA_PFX_EFFECT_DESTROYED: szState = "destroyed"; break;
	default:                         szState = "INVALID";   break;
	}
	const char* szVisible;
	bool bUpdating = true;
	if(GetCameraFlags(true))
	{
		szVisible = "In frustum";
	}
	else if(GetCameraFlags(false))
	{
		szVisible = "Linked in";
		const bool bMoving = (m_xDelta.MagnitudeSquared() > 0.f);
		bUpdating = (bMoving || !m_pxTemplate->m_bLooping);
	}
	else
	{
		szVisible = "Not visible";
		bUpdating = false;
	}
	const char* szUpdating = bUpdating ? "Updating" : "Not updating";
	u_int uTotal = 0;
	u_int uTotalMax = 0;
	for( u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource )
	{
		const Asura_PFX_Source* pxSource = m_ppxSources[uSource];
		Asura_PFX_Source_Template *pxTemplate = Asura_PFX_System::GetSourceTempByHash(pxSource->m_uTemplateNameHash);
		uTotal += pxSource->m_uNumParticles;
		uTotalMax += pxTemplate->m_uMaxParticles;
	}
	Asura_SimpleLogfileOutput(( szPFX_STATS_LOGFILE, "%u\t%s\t%u\t%u\t%s\t%s\t%s\n", GetID(), m_pxTemplate->m_xName.GetString(), uTotal, uTotalMax, szState, szVisible, szUpdating));
	if(dbg_bDumpSources)
	{
		for( u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource )
		{
			const Asura_PFX_Source* pxSource = m_ppxSources[uSource];
			Asura_PFX_Source_Template *pxTemplate = Asura_PFX_System::GetSourceTempByHash(pxSource->m_uTemplateNameHash);

			Asura_SimpleLogfileOutput(( "logfiles\\pfx_stats.txt",
				"%08u\t%u\t%u\t%s\n",
				GetID(), pxSource->m_uNumParticles, pxTemplate->m_uMaxParticles, szState ));
			const char* szTemplateName = pxTemplate ? pxTemplate->m_xName.GetString() : "NO TEMPLATE";
			Asura_SimpleLogfileOutput(( szPFX_STATS_LOGFILE, "\t%s\t%u\t%u\n", szTemplateName, pxSource->m_uNumParticles, pxTemplate->m_uMaxParticles));
		}
	}
#endif	// defined(ASURA_LOGFILES_ENABLED)
}

#endif

#ifdef ASURA_TOOLS
void Asura_PFX_Entity_Effect::SetRNG(Asura_RandomGenerator* pxRNG)
{
	Asura_PFX_Effect::SetRNG(pxRNG);

	//set the RNG in each source
    for( u_int uSource = 0; uSource < m_pxTemplate->m_uNumSources; ++uSource )
    {
		m_ppxSources[uSource]->SetRNG(pxRNG);
	}

}
#endif //ASURA_TOOLS



/// End of file. ///
