//
// Asura_PFX_System.cpp
//
// Created by KOR - 22/03/2004 14:31:19
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"


// This file's header.
#include "SpecialFX/PFX/Asura_PFX_System.h"

// Asura headers
#include "Core/Asura_HashTree.h"
#include "Core/Asura_RandTree.h"
#include "Core/Asura_TextureManagement.h"
#include "Debugging/Asura_Debug_Timers.h"
#if _DEBUG
#include "Debugging/Asura_Logfile.h"
#endif
#ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE
#include "Jobs/Asura_Job_PFX.h"
#endif
#include "SpecialFX/PFX/Asura_PFX_Classes.h"
#include "SpecialFX/PFX/Asura_PFX_Effect.h"
#include "SpecialFX/PFX/Asura_PFX_Entity_Effect.h"
#include "SpecialFX/PFX/Asura_PFX_Source.h"
#include "SpecialFX/PFX/Asura_PFX_Templates.h"
#include "Rendering/Asura_Render.h"

#ifdef ASURA_TOOLS
#include "Core/Asura_NameTree.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Cutscene/Tools_CutsceneModeAVI.h"
#include "Particles/Tools_Mode_PFX.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#endif


DEBUGVAR bool dbg_bEnablePFXHeatHaze = true;
DEBUGVAR bool dbg_bEnablePFXUpdate = true;
DEBUGVAR bool dbg_bEnablePFXRender = true;

#if _DEBUG
static bool g_bDumpPFXStats = false;
#endif

// Static members
#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
Asura_PFX_Handler*				Asura_PFX_System::s_pxCurrentHandler;
Asura_TemplateHashTree<Asura_PFX_Handler> Asura_PFX_System::s_xHandlerTree;
ASURA_PFX_RENDER_TYPE			Asura_PFX_System::s_eCallbackRenderType = ASURA_PFX_RENDER_TYPE_INVALID;
#else
Asura_PFX_Handler				Asura_PFX_System::s_xHandler;
#endif


u_int							Asura_PFX_System::s_uNextEffectID;

// Template store members
Asura_PFX_ParticleTemplateResourceSet	Asura_PFX_System::s_xParticleTemplateResourceSet("PFXParticleTemplate");
Asura_PFX_SourceTemplateResourceSet	Asura_PFX_System::s_xSourceTemplateResourceSet("PFXSoundTemplate");
Asura_PFX_EffectTemplateResourceSet	Asura_PFX_System::s_xEffectTemplateResourceSet("PFXEffectTemplate");

// Heat haze.
bool							Asura_PFX_System::s_bIsRenderingHeatHazeSources = false;

// Render queue...
Asura_PFX_RenderQueue	Asura_PFX_System::s_xRenderQueue;

#ifdef ASURA_TOOLS
Asura_TemplateHashTree<Asura_String> Asura_PFX_System::s_xParticleTempNames;
Asura_TemplateHashTree<Asura_String> Asura_PFX_System::s_xSourceTempNames;
Asura_TemplateHashTree<Asura_String> Asura_PFX_System::s_xEffectTempNames;
#endif

#ifdef ASURA_TOOLS
bool							Asura_PFX_System::s_bPFXSystemDisabled = false;
bool							Asura_PFX_System::s_bAllowTemplateReload = false;
#endif

bool							Asura_PFX_System::s_bEnableMultithreading = true;

ASURA_PFX_RENDER_TYPE			Asura_PFX_System::s_eCurrentRenderType = ASURA_PFX_RENDER_TYPE_INVALID;

u_int							Asura_PFX_System::s_uNumPFXSounds;

bool							Asura_PFX_System::s_bClearTemplatesOnReset = true;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------**
** Asura_PFX_Handler **
**-------------------*/

Asura_PFX_Handler::Asura_PFX_Handler()
{
	// Initialise effect array
	m_uNumEffects = 0;
    m_ppxEffects = new Asura_PFX_Effect*[ASURA_PFX_MAX_ACTIVE_EFFECTS];
	m_xEffectTree.Reset();

	m_uNumHeatHazeSources = 0;
}

Asura_PFX_Handler::~Asura_PFX_Handler()
{
	// Delete effect storage
	ClearAllEffects();
    delete [] m_ppxEffects;
}

void Asura_PFX_Handler::Reset()
{
	// This will probably be called between levels. Shutdown all effects, and clear out the templates
	ClearAllEffects();
}

void Asura_PFX_Handler::ClearAllEffects()
{
	#ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE
	//Need to make sure no PFX updates are running on another thread before the effects get deleted.
	//RAR 19/01/2007
	Asura_Job_PFX_Update::FinishAllJobs();
	#endif
	
	// Clear effect array
    while(m_uNumEffects)
	{
		--m_uNumEffects;
		delete m_ppxEffects[m_uNumEffects];
		m_ppxEffects[m_uNumEffects] = 0;
	}
	m_xEffectTree.Reset();

	m_uNumHeatHazeSources = 0;
}

#ifdef ASURA_TOOLS
void Asura_PFX_Handler::ClearAllEffectsUsingEffectTemplate(Asura_Hash_ID uEffectHash)
{
	#ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE
	//Need to make sure no PFX updates are running on another thread before the effects get deleted.
	//RAR 19/01/2007
	Asura_Job_PFX_Update::FinishAllJobs();
	#endif

	for(u_int uEffect = 0; uEffect < m_uNumEffects; )
	{
		bool bDelete = false;
		if(m_ppxEffects[uEffect]->IsEntityEffect())
		{
			Asura_PFX_Entity_Effect* pxEffect = (Asura_PFX_Entity_Effect*) m_ppxEffects[uEffect];
			if(pxEffect->GetCurrentTemplateHash() == uEffectHash)
			{
				bDelete = true;
			}
		}
		else
		{
			//Clearing any non-entity effects to be safe , to avoid having to worry about which ones might
			//be using the templates
			bDelete = true;
		}

		
		if(bDelete)
		{
			int iIndex;
			if(!m_xEffectTree.DeleteNodeByKey(m_ppxEffects[uEffect]->GetID(), iIndex))
			{
				Asura_Assert(0, "Effect not found in tree");
			}
			Asura_Assert(static_cast<u_int>(iIndex) == uEffect, "Index from tree does not match our own");
			delete m_ppxEffects[uEffect];
			--m_uNumEffects;
			m_ppxEffects[uEffect] = m_ppxEffects[m_uNumEffects];
			m_ppxEffects[m_uNumEffects] = 0;
			continue;
		}
		uEffect++;
	}
}
#endif //ASURA_TOOLS

void Asura_PFX_Handler::Update()
{
	if(!dbg_bEnablePFXUpdate) return;

	//RAR 15/03/2006 Check to make sure that heat haze sources are being cleared out , since it now relies on a separate
	//render call being called from outside the system.
	Asura_Assert(m_uNumHeatHazeSources == 0,"Expecting buffered heat haze sources to be cleared by now");

	u_int uEffect;
	for(uEffect = 0; uEffect < m_uNumEffects; ++uEffect)
	{
		// Only update if running or stopping - don't bother for standby or destroyed
		if(m_ppxEffects[uEffect]->GetState() <= ASURA_PFX_EFFECT_STOPPING)
		{
			m_ppxEffects[uEffect]->Update();
		}
	}
    
#if _DEBUG
    if( g_bDumpPFXStats )
    {
        Asura_SimpleLogfileClear(( "logfiles\\pfx_stats.txt" ));

        for( uEffect = 0; uEffect < m_uNumEffects; ++uEffect )
        {
            m_ppxEffects[uEffect]->DumpStats();
        }

        extern u_int g_uParticlesAllocatedInSources;
        extern u_int g_uParticlesUsedInSources;
        Asura_SimpleLogfileOutput(( "logfiles\\pfx_stats.txt",
                                    "\ntotals: %u\t%u\t%.1f\n",
                                    g_uParticlesUsedInSources,
                                    g_uParticlesAllocatedInSources,
                                    Asura_Maths::FtoD(static_cast<float>(g_uParticlesUsedInSources) / g_uParticlesAllocatedInSources) ));

        g_bDumpPFXStats = false;
    }
#endif
}

void Asura_PFX_Handler::Render(ASURA_PFX_RENDER_TYPE eRenderType)
{
	if(!dbg_bEnablePFXRender) return;

    // (MH - 09.09.2005) record current render type so platform render can use it.
	Asura_PFX_System::s_eCurrentRenderType = eRenderType;

	Asura_PFX_System::Platform_BeginRender();
	for(u_int uEffect = 0; uEffect < m_uNumEffects; )
	{
		// KOR - if we get many more render types, may consider keeping separate lists of them to save looping through too many times.
		if (m_ppxEffects[uEffect]->GetRenderType() == eRenderType)
		{
			// (AW - 22.10.2004) - Because effects may be destroyed at any time, I am now
			// pruning the dead effects during the render loop.
			if(m_ppxEffects[uEffect]->IsDestroyed() )	// (Check for ASURA_PFX_EFFECT_DESTROYED)
			{
				int iIndex;
				if(!m_xEffectTree.DeleteNodeByKey(m_ppxEffects[uEffect]->GetID(), iIndex))
				{
					Asura_Assert(0, "Effect not found in tree");
				}
				Asura_Assert(static_cast<u_int>(iIndex) == uEffect, "Index from tree does not match our own");
				delete m_ppxEffects[uEffect];
				--m_uNumEffects;
				m_ppxEffects[uEffect] = m_ppxEffects[m_uNumEffects];
				m_ppxEffects[m_uNumEffects] = 0;
				continue;
			}
			else
			{
				m_ppxEffects[uEffect]->Render();
			}			
		}
		uEffect++;
	}
	// RJF - 19 October 2004 - Queued render commands
	Asura_PFX_System::GetRenderQueue().Flush();

	Asura_PFX_System::Platform_EndRender();

	
	Asura_PFX_System::RenderHeatHaze();

	Asura_PFX_System::s_eCurrentRenderType = ASURA_PFX_RENDER_TYPE_INVALID;
}

bool Asura_PFX_Handler::CanCreateEffect()
{
	return m_uNumEffects < ASURA_PFX_MAX_ACTIVE_EFFECTS;
}

Asura_PFX_Effect* Asura_PFX_Handler::GetEffect( u_int uEffectID )
{
    if( ASURA_PFX_INVALID_EFFECT_ID == uEffectID ) { return NULL; }

    int iIndex = m_xEffectTree.FindNodeIndexByKey( uEffectID );
    return( iIndex >= 0 ? m_ppxEffects[iIndex] : NULL );
}

// KOR - 03/04/2006 - Most PFX effects are actually entity effects -
// there's been much less code customisation than I predicted.
// To avoid having to cast everywhere we want to get an entity effect, I've
// added this function. It also does some extra safety checks to avoid casting
// another type of effect to an entity effect.
Asura_PFX_Entity_Effect* Asura_PFX_Handler::GetEntityEffect(u_int uEffectID)
{
	if( ASURA_PFX_INVALID_EFFECT_ID == uEffectID ) { return NULL; }

	int iIndex = m_xEffectTree.FindNodeIndexByKey( uEffectID );
	if(iIndex >= 0)
	{
		Asura_Assert(m_ppxEffects[iIndex]->IsEntityEffect(), "Handle passed into GetEntityEffect is for a non entity effect");
		if(m_ppxEffects[iIndex]->IsEntityEffect())
		{
			return static_cast<Asura_PFX_Entity_Effect*>(m_ppxEffects[iIndex]);
		}
	}
	return NULL;
}

bool Asura_PFX_Handler::SetEntityEffectPosition(u_int& uEffectID, const Asura_Vector_3& xPosition)
{
	if (ASURA_PFX_INVALID_EFFECT_ID != uEffectID)
	{
		Asura_PFX_Entity_Effect* pxEffect = GetEntityEffect(uEffectID);
		if (pxEffect)
		{
			pxEffect->SetPosition(xPosition);
			return true;
		}
		else
		{
			uEffectID = ASURA_PFX_INVALID_EFFECT_ID;
		}
	}
	return false;
}


bool Asura_PFX_Handler::SetEntityEffectPosition(u_int& uEffectID, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection)
{
	if (ASURA_PFX_INVALID_EFFECT_ID != uEffectID)
	{
		Asura_PFX_Entity_Effect* pxEffect = GetEntityEffect(uEffectID);
		if (pxEffect)
		{
			Asura_Matrix_3x3 xOri;
			Asura_Maths::OrientationFromDirection(xOri, xDirection);
			pxEffect->SetPosition(xPosition, xOri);
			return true;
		}
		else
		{
			uEffectID = ASURA_PFX_INVALID_EFFECT_ID;
		}
	}
	return false;
}

// Only difference between this and the above version is that direction is a matrix instead of a vector
bool Asura_PFX_Handler::SetEntityEffectPosition(u_int& uEffectID, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xDirection)
{
	if (ASURA_PFX_INVALID_EFFECT_ID != uEffectID)
	{
		Asura_PFX_Entity_Effect* pxEffect = GetEntityEffect(uEffectID);
		if (pxEffect)
		{
			pxEffect->SetPosition(xPosition, xDirection);
			return true;
		}
		else
		{
			uEffectID = ASURA_PFX_INVALID_EFFECT_ID;
		}
	}
	return false;
}

bool Asura_PFX_Handler::StopEntityEffect(u_int uEffectID, bool bImmediate)
{
	Asura_PFX_Entity_Effect* pxEffect = GetEntityEffect(uEffectID);
	if (pxEffect)
	{
		pxEffect->Stop(bImmediate);
		return true;
	}
	return false;
}

void Asura_PFX_Handler::DestroyEffect(u_int& uEffectID)
{
	Asura_PFX_Effect *pxEffect = GetEffect(uEffectID);
	if (pxEffect)
	{
		pxEffect->Destroy();
	}
	uEffectID = ASURA_PFX_INVALID_EFFECT_ID;
}

void Asura_PFX_Handler::RegisterEffect(Asura_PFX_Effect* pxEffect,u_int uEffectID)
{
	Asura_Assert(m_uNumEffects < ASURA_PFX_MAX_ACTIVE_EFFECTS, "Should have checked CanCreateEffect() before creating effect");
	pxEffect->m_uEffectID = uEffectID;
	int iIndex;
#ifdef ASURA_ASSERT
	bool bAlreadyExisted = 
#endif
		m_xEffectTree.InsertKey( uEffectID, iIndex );
	Asura_Assert( !bAlreadyExisted, "" );
	Asura_Assert(static_cast<u_int>(iIndex) == m_uNumEffects, "Unexpected index for new effect given by hash tree");
	m_ppxEffects[iIndex] = pxEffect;
	++m_uNumEffects;
}

bool Asura_PFX_Handler::AddHeatHazeSource(Asura_PFX_Source *pxHeatHazeSource)
{
	// (AW - 26.10.2004) - Heat haze gets buffered and rendered after all the other
	// particle systems.
	Asura_Assert(m_uNumHeatHazeSources < ASURA_MAX_NO_OF_PFX_HEATHAZE_SOURCES, "Too many heat haze sources, consider increasing ASURA_MAX_NO_OF_PFX_HEATHAZE_SOURCES");
	if (m_uNumHeatHazeSources < ASURA_MAX_NO_OF_PFX_HEATHAZE_SOURCES)
	{
		m_apxHeatHazeSources[m_uNumHeatHazeSources] = pxHeatHazeSource;
		m_uNumHeatHazeSources++;
	}
	return true;
}

void Asura_PFX_Handler::RenderHeatHaze()
{

	// render any buffered heathaze effects
	if (dbg_bEnablePFXHeatHaze && 
		m_uNumHeatHazeSources)
	{
		Asura_PFX_System::Platform_BeginRender();
		for (u_int u = 0; u < m_uNumHeatHazeSources; ++u)
		{
			m_apxHeatHazeSources[u]->Render();
		}
		Asura_PFX_System::GetRenderQueue().Flush();
		Asura_PFX_System::Platform_EndRender();
	}
	#ifdef ASURA_TOOLS
	else if(Tools_CutsceneModeAVI::IsRecording())
	{
		//Force begin/end render so that the number of random numbers used isn't affected by which
		//PFX effects are enabled. RAR 23/05/2006
		Asura_PFX_System::Platform_BeginRender();
		Asura_PFX_System::Platform_EndRender();
	}
	#endif
	m_uNumHeatHazeSources = 0;
}


#if defined(ASURA_USE_STREAMING)
// Called after a zone has loaded. Gives handler a chance to update module lists.
void Asura_PFX_Handler::PostLoadInitialise()
{
	// MGH 13/7/2006: Look for any PFX effects that aren't in any zone/module and re-test them
	// to see whether they are in fact in the zone that has just been loaded.
	if (!m_ppxEffects) return;
	
	for(u_int u = 0; u < m_uNumEffects; ++u)
	{
		if (m_ppxEffects[u] && m_ppxEffects[u]->m_xModuleList.IsEmpty())
		{
			// Found an effect with a valid and empty module list.
			// Invalidate the list to make sure that it is recalculated.
			m_ppxEffects[u]->InvalidateModuleList();
		}
	}
}
#endif //defined(ASURA_USE_STREAMING)



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------**
** Asura_PFX_System **
**------------------*/

Asura_PFX_System::ParticleColourOverrideHook* Asura_PFX_System::s_pfnParticleColourOverride = NULL;
Asura_PFX_System::PFXRenderStateOverrideHook* Asura_PFX_System::s_pfnRenderStateOverride = NULL;

void Asura_PFX_System::Initialise()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_PFX_System::Initialise");

	s_uNextEffectID = 1;
	s_bIsRenderingHeatHazeSources = false;

	// Only load particle templates in tools.
	// For the game, the templates will be kept in the level exports
#ifdef ASURA_TOOLS
	LoadAllTemplateFiles();
#endif

	Platform_Initialise();
}

#ifdef ASURA_TOOLS
void Asura_PFX_System::LoadAllTemplateFiles()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_PFX_System::LoadAllTemplateFiles");
	
	LoadTemplate(3);
}

void Asura_PFX_System::ReloadAllTemplates()
{
	//delete all the current templates.
	ClearAllEffects();
	ClearAllTemplates();

	//And attempt to load them again
	//(or at any rate , allocate an empty slot for them)
	LoadAllTemplateFiles();
}

void Asura_PFX_System::LoadTemplate(int iType, Asura_Hash_ID uHash /* = 0 */)
{
	// Load templates from project particles directory
	char szNetSearch[MAX_PATH+10];
	const char* pszLocalSearch;

	//get the local and network search paths
	switch(iType)
	{
		// KOR - deprecated, should be removed
		case 0 :
			pszLocalSearch = "PFXTemps\\Particle\\*.pcl";
			Asura_FileAccess::PlatformGetNetworkDir("PFXTemps\\Particle",szNetSearch,MAX_PATH);
			strcat(szNetSearch,"\\*.pcl");
			break;
		case 1 :
			pszLocalSearch = "PFXTemps\\Source\\*.src";
			Asura_FileAccess::PlatformGetNetworkDir("PFXTemps\\Source",szNetSearch,MAX_PATH);
			strcat(szNetSearch,"\\*.src");
			break;
		case 2 :
			pszLocalSearch = "PFXTemps\\Effect\\*.fct";
			Asura_FileAccess::PlatformGetNetworkDir("PFXTemps\\Effect",szNetSearch,MAX_PATH);
			strcat(szNetSearch,"\\*.fct");
			break;
		case 3 :
			pszLocalSearch = "PFXTemps\\*.pfx";
			Asura_FileAccess::PlatformGetNetworkDir("PFXTemps",szNetSearch,MAX_PATH);
			strcat(szNetSearch,"\\*.pfx");
			break;
		default:
			{
				Asura_Assert(0, "Unknown type");
				break;
			}
	}

	List<Asura_String> xTemplateList;
	Tools_FileSystem::FindFiles(pszLocalSearch, xTemplateList, TOOLS_FILEFIND_RECURSIVE|TOOLS_FILEFIND_IGNOREBACKUPDIR);
	Tools_FileSystem::FindFiles(szNetSearch, xTemplateList, TOOLS_FILEFIND_RECURSIVE|TOOLS_FILEFIND_IGNOREBACKUPDIR);

	// If a hash is specified, try to load that file
	if(uHash)
	{
		for(LIF<Asura_String> xTemplateLIF(&xTemplateList);!xTemplateLIF.done();xTemplateLIF.next())
		{
			Asura_String xName;
			xTemplateLIF().SplitFilename(NULL, &xName, NULL);
			if(Asura_GetHashID(xName.GetString()) == uHash)
			{
				LoadAsuraFile(xTemplateLIF().GetString());
				break;
			}
		}
		return;
	}

	// Otherwise add all the template names to the system

	//Get a sorted list of names
	Asura_NameTree xNameTree;
	char szBuffer[MAX_PATH];
	for(LIF<Asura_String> xTemplateLIF(&xTemplateList);!xTemplateLIF.done();xTemplateLIF.next())
	{
		char *pLocalName = strstr(xTemplateLIF().GetString(), "PFXTemps\\");

		strcpy(szBuffer,pLocalName ? pLocalName + sizeof("PFXTemps\\") - 1 : xTemplateLIF().GetString());
		char* pcDot = strrchr(szBuffer,'.');
		if(pcDot) * pcDot = 0;
		

		int iJunkIndex;
		xNameTree.FindNodeIndexByName(szBuffer,true,iJunkIndex);
	}

	int iNumNames = xNameTree.GetNumberOfNodes();
	const char** ppszNameList = new const char*[iNumNames];
	xNameTree.MakeSortedNameList(ppszNameList);

	//add all the names to the template system
	for(int i=0;i<iNumNames;i++)
	{
		switch(iType)
		{
			case 0:
			{
				AddParticleTemplate(ppszNameList[i],0);
				break;
			}
			case 1:
			{
				AddSourceTemplate(ppszNameList[i],0);
				break;
			}
			case 2:
			// Fall-through
			case 3 :
			{
				AddEffectTemplate(ppszNameList[i],0);
				break;
			}
			default:
			{
				Asura_Assert(0, "Unknown type");
				break;
			}
		}
	}
	delete [] ppszNameList;

}

#endif // ASURA_TOOLS

void Asura_PFX_System::Shutdown()
{
	Platform_Shutdown();

	ClearAllEffects();

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	// Reset the entire handler tree...
	s_xHandlerTree.Reset();

	// Remove the current handler...
	s_pxCurrentHandler	= NULL;
#else
	Asura_PFX_Handler& xHandler	= GetCurrentHandler();

	// Reset the current instance...
	xHandler.Reset();
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)


	// Template store shutdown
	ClearAllTemplates();
}

void Asura_PFX_System::Reset()
{
	Platform_Reset();

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	// Reset the entire handler tree...
	s_xHandlerTree.Reset();
#else
	// Reset the current instance...
	GetCurrentHandler().Reset();
#endif	//defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)

	//Don't bother resetting the particle templates in tools builds , to avoid having to
	//reload them all again , which can be fairly slow.
	//RAR 26/07/2005
#ifndef ASURA_TOOLS
	if (s_bClearTemplatesOnReset)
	{
		ClearAllTemplates();
	}
#endif

}

void Asura_PFX_System::Update()
{
	s_uNumPFXSounds = 0;

#ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE
	Platform_BeginUpdate();
#endif 

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	s_xHandlerTree.Traverse(Update_Callback);
#else
	GetCurrentHandler().Update();
#endif
}

void Asura_PFX_System::Render(ASURA_PFX_RENDER_TYPE eRenderType)
{
	if (!Asura_Render::IsSystemEnabled(ASURA_RENDER_SYSTEM_PFX))
	{
		return;
	}

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	// Save the parameter for use by the callback.
	s_eCallbackRenderType = eRenderType;
	s_xHandlerTree.Traverse(Render_Callback);
#else
	GetCurrentHandler().Render(eRenderType);
#endif
}


#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
Asura_PFX_Handler* Asura_PFX_System::CreateHandler (const Asura_Hash_ID uHash)
{
	Asura_PFX_Handler*	pxHandler	= s_xHandlerTree.GetDataByHash(uHash);

	// Do we already have this handler?
	if(pxHandler == NULL)
	{
		// No - create one...
		pxHandler	= s_xHandlerTree.AddNode(uHash);
	}
	else
	{
		// Yes - this shouldn't happen...
		Asura_Assert(false, "Already have a PFX handler with this hash-ID.");
	}

	// Return the handler...
	return(pxHandler);
}

bool Asura_PFX_System::DestroyHandler (const Asura_Hash_ID uHash)
{
	// Is this the current handler?
	if(s_pxCurrentHandler == s_xHandlerTree.GetDataByHash(uHash))
	{
		s_pxCurrentHandler	= NULL;
	}

	// Call the delete function...
	return(s_xHandlerTree.DeleteNodeByHash(uHash));
}
#endif	// defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)

void Asura_PFX_System::ClearAllEffectsCallback(Asura_Hash_ID uHash, Asura_PFX_Handler* pxHandler)
{
	pxHandler->ClearAllEffects();
}

void Asura_PFX_System::ClearAllEffects()
{
	s_bIsRenderingHeatHazeSources = false;

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	s_xHandlerTree.Traverse(ClearAllEffectsCallback);
#else
	//RAR 14/03/2006 May be desirable to clear all effects for all handlers here
	s_xHandler.ClearAllEffects();
#endif
}

#ifdef ASURA_TOOLS
bool Asura_PFX_System::ClearAllEffectsUsingEffectTemplateCallback(Asura_Hash_ID uHash, Asura_PFX_Handler* pxHandler, void* pEffectHash)
{
	pxHandler->ClearAllEffectsUsingEffectTemplate(*(Asura_Hash_ID*)pEffectHash);
	return true;
}

void Asura_PFX_System::ClearAllEffectsUsingEffectTemplate(Asura_Hash_ID uEffectHash)
{
	s_bIsRenderingHeatHazeSources = false;

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	s_xHandlerTree.Traverse(ClearAllEffectsUsingEffectTemplateCallback, &uEffectHash);
#else
	s_xHandler.ClearAllEffectsUsingEffectTemplate(uEffectHash);
#endif
}
#endif //ASURA_TOOLS



// Effect registration interface

bool Asura_PFX_System::CanCreateEffect()
{
	#ifdef ASURA_TOOLS
	if(s_bPFXSystemDisabled) return false;
	#endif

	return GetCurrentHandler().CanCreateEffect();
}

#ifdef ASURA_TOOLS
void Asura_PFX_System::DisablePFXSystem(bool bDisable)
{
	if(bDisable && !s_bPFXSystemDisabled) ClearAllEffects();
	s_bPFXSystemDisabled = bDisable;
}
#endif

bool Asura_PFX_System::AddHeatHazeSource(Asura_PFX_Source *pxHeatHazeSource)
{
	// (AW - 26.10.2004) - Heat haze gets buffered and rendered after all the other
	// particle systems.
	if (!Platform_SupportsHeatHaze())
	{
		// (AW - 26.10.2004) - Return true in this case so that heat haze particles aren't 
		// drawn at all.
		return true;
	}
	Asura_Assert(pxHeatHazeSource->IsHeatHazeSource(), "Sanity check");
	if (s_bIsRenderingHeatHazeSources)
	{
		return false;
	}

	return GetCurrentHandler().AddHeatHazeSource(pxHeatHazeSource);

}

void Asura_PFX_System::RenderHeatHaze()
{
	s_bIsRenderingHeatHazeSources = true;
	GetCurrentHandler().RenderHeatHaze();
	s_bIsRenderingHeatHazeSources = false;
}

/// End of file. ///
