//
// Asura_PFX_System.h
//
// Created by KOR - 22/03/2004 14:31:31
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PFX_SYSTEM_H__
#define __ASURA_PFX_SYSTEM_H__

#include "Core/Asura_RandTree.h"
#include "Core/Resources/Asura_ResourceSet.h"
#include "Core/Resources/Asura_ResourceSetWithOverride.h"
#include "SpecialFX/PFX/Asura_PFX_Enums.h"
#include "SpecialFX/PFX/Asura_PFX_Templates.h"
#include "SpecialFX/PFX/Asura_PFX_RenderQueue.h"

// Forward declares
class Asura_PFX_Effect;
class Asura_PFX_Entity_Effect;
class Asura_PFX_Source;
class Asura_PFX_Particle;
class Asura_RGBA;
class Asura_String;
class Asura_Vector_3;
class Asura_Matrix_3x3;

typedef Asura_ResourceSetWithOverride<Asura_PFX_Particle_Template>								Asura_PFX_ParticleTemplateResourceSet;
typedef Asura_ResourceSetWithOverride<Asura_PFX_Source_Template>								Asura_PFX_SourceTemplateResourceSet;
typedef Asura_ResourceSetWithOverride<Asura_PFX_Effect_Template>								Asura_PFX_EffectTemplateResourceSet;

#ifdef ASURA_TOOLS
typedef Asura_ResourceSetOverrideMap<Asura_PFX_Particle_Template>								Asura_PFX_ParticleTemplateOverrideMap;
typedef Asura_ResourceSetOverrideMap<Asura_PFX_Source_Template>									Asura_PFX_SourceTemplateOverrideMap;
typedef Asura_ResourceSetOverrideMap<Asura_PFX_Effect_Template>									Asura_PFX_EffectTemplateOverrideMap;

typedef Asura_ResourceSetOverride<Asura_PFX_Particle_Template>									Asura_PFX_ParticleTemplateOverrideEntry;
typedef Asura_ResourceSetOverride<Asura_PFX_Source_Template>									Asura_PFX_SourceTemplateOverrideEntry;
typedef Asura_ResourceSetOverride<Asura_PFX_Effect_Template>									Asura_PFX_EffectTemplateOverrideEntry;

// Iterate over a Resource set to enumerate the resources
typedef Asura_ResourceSetWithOverride_ResourceIterator<Asura_PFX_Particle_Template>				Asura_PFX_ParticleTemplateResourceSet_ResourceIterator;
typedef Asura_ResourceSetWithOverride_ResourceIterator<Asura_PFX_Source_Template>				Asura_PFX_SourceTemplateResourceSet_ResourceIterator;
typedef Asura_ResourceSetWithOverride_ResourceIterator<Asura_PFX_Effect_Template>				Asura_PFX_EffectTemplateResourceSet_ResourceIterator;

typedef Asura_ResourceSetWithOverride_ConstResourceIterator<Asura_PFX_Particle_Template>		Asura_PFX_ParticleTemplateResourceSet_ConstResourceIterator;
typedef Asura_ResourceSetWithOverride_ConstResourceIterator<Asura_PFX_Source_Template>			Asura_PFX_SourceTemplateResourceSet_ConstResourceIterator;
typedef Asura_ResourceSetWithOverride_ConstResourceIterator<Asura_PFX_Effect_Template>			Asura_PFX_EffectTemplateResourceSet_ConstResourceIterator;

// Iterate over a Resource set to enumerate the override maps
typedef Asura_ResourceSetWithOverride_ConstOverrideMapIterator<Asura_PFX_Particle_Template>		Asura_PFX_ParticleTemplateResourceSet_ConstMapIterator;
typedef Asura_ResourceSetWithOverride_ConstOverrideMapIterator<Asura_PFX_Source_Template>		Asura_PFX_SourceTemplateResourceSet_ConstMapIterator;
typedef Asura_ResourceSetWithOverride_ConstOverrideMapIterator<Asura_PFX_Effect_Template>		Asura_PFX_EffectTemplateResourceSet_ConstMapIterator;

typedef Asura_ResourceSetWithOverride_OverrideMapIterator<Asura_PFX_Particle_Template>			Asura_PFX_ParticleTemplateResourceSet_MapIterator;
typedef Asura_ResourceSetWithOverride_OverrideMapIterator<Asura_PFX_Source_Template>			Asura_PFX_SourceTemplateResourceSet_MapIterator;
typedef Asura_ResourceSetWithOverride_OverrideMapIterator<Asura_PFX_Effect_Template>			Asura_PFX_EffectTemplateResourceSet_MapIterator;

// Iterate over an override map to enumerate the overrides
typedef Asura_ResourceSetOverrideMap_ConstIterator<Asura_PFX_Particle_Template>					Asura_PFX_ParticleTemplateMap_ConstIterator;
typedef Asura_ResourceSetOverrideMap_ConstIterator<Asura_PFX_Source_Template>					Asura_PFX_SourceTemplateMap_ConstIterator;
typedef Asura_ResourceSetOverrideMap_ConstIterator<Asura_PFX_Effect_Template>					Asura_PFX_EffectTemplateMap_ConstIterator;

typedef Asura_ResourceSetOverrideMap_Iterator<Asura_PFX_Particle_Template>						Asura_PFX_ParticleTemplateMap_Iterator;
typedef Asura_ResourceSetOverrideMap_Iterator<Asura_PFX_Source_Template>						Asura_PFX_SourceTemplateMap_Iterator;
typedef Asura_ResourceSetOverrideMap_Iterator<Asura_PFX_Effect_Template>						Asura_PFX_EffectTemplateMap_Iterator;

#endif

/// CLASSES ///////////////////////////////////////////////////////////////////////////////////////

class Asura_PFX_Handler
{
public :
	Asura_PFX_Handler();
	~Asura_PFX_Handler();

	void Reset();
	void ClearAllEffects(); // Used for quick loads and the like, where particles need reseting but templates do not.
	#ifdef ASURA_TOOLS
	//For PFX Mode - stops all entity effects using a given template.
	//Stops all non-entity effects just to be on the safe side.
	void ClearAllEffectsUsingEffectTemplate(Asura_Hash_ID uEffectHash);
	#endif
	void Update();
	void Render(ASURA_PFX_RENDER_TYPE eRenderType);


	//Get number of active PFX effects.
	u_int GetNumEffects(){return m_uNumEffects;}

	// Effect registration interface
	bool CanCreateEffect();
	Asura_PFX_Effect* GetEffect(u_int uEffectID);
	Asura_PFX_Entity_Effect* GetEntityEffect(u_int uEffectID);
	// Sets the position and optionally direction of the Asura_PFX_Entity_Effect with given ID
	// Returns true if the effect exists, false otherwise.
	// If the effect does not exist, uEffectID will be set to ASURA_PFX_INVALID_EFFECT_ID.
	bool SetEntityEffectPosition(u_int& uEffectID, const Asura_Vector_3& xPosition);
	bool SetEntityEffectPosition(u_int& uEffectID, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection);
	bool SetEntityEffectPosition(u_int& uEffectID, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xDirection);
	// Stops the Asura_PFX_Entity_Effect with given ID, using the immediate flag given.
	// Returns true if the effect exists, false otherwise.
	bool StopEntityEffect(u_int uEffectID, bool bImmediate);
	// Destroys the effect if it exists and sets the ID to ASURA_PFX_INVALID_EFFECT_ID
	void DestroyEffect(u_int& uEffectID);
	// No need for the user to call this - it is called by the effect constructor
	void RegisterEffect(Asura_PFX_Effect* pxEffect,u_int uEffectID);

	bool AddHeatHazeSource(Asura_PFX_Source *pxHeatHazeSource);
	void RenderHeatHaze();

#if defined(ASURA_USE_STREAMING)
	// Called after a zone has loaded. Gives handler a chance to update module lists.
	void PostLoadInitialise();
#endif //defined(ASURA_USE_STREAMING)

private :
	// Effect management
	Asura_RandTree<u_int>			m_xEffectTree;
	u_int							m_uNumEffects;
	Asura_PFX_Effect**				m_ppxEffects;

	// Heat haze.
	Asura_PFX_Source*		 		m_apxHeatHazeSources[ASURA_MAX_NO_OF_PFX_HEATHAZE_SOURCES];
	u_int							m_uNumHeatHazeSources;


};

/// Asura_PFX_System //////////////////////////////////////////////////////////////////////////////
// The controlling system. Handles effect creation/registration, updating, rendering, deleting,
// finding, and template store management
///////////////////////////////////////////////////////////////////////////////////////////////////

class Asura_PFX_System
{
#ifdef ASURA_TOOLS
	friend class Tools_Mode_PFX;
#endif
	friend class Asura_PFX_Handler;
public:
	typedef void (ParticleColourOverrideHook)(const Asura_PFX_Effect* pxEffect, const Asura_PFX_Particle_Template* pxTemplate, u_int uParticle, Asura_RGBA& xColour);
	typedef void (PFXRenderStateOverrideHook)();

	// Standard Asura interfaces
	static void Initialise();
	static void Shutdown();
	static void Reset();

	static void ClearAllEffects();// Used for quick loads and the like, where particles need reseting but templates do not.
	#ifdef ASURA_TOOLS
	//For PFX Mode - stops all entity effects using a given template.
	//Stops all non-entity effects just to be on the safe side.
	static void ClearAllEffectsUsingEffectTemplate(Asura_Hash_ID uEffectHash);
	#endif
	static void Update();
	static void Render(ASURA_PFX_RENDER_TYPE eRenderType);



	//Renders buffered heat haze effects , should be called after the main PFX system Render.
	//It has been split into a separate function to allow streaming system to do the normal render for multiple zones , and
	//then come back and deal with the heat haze for each zone in turn. RAR 15/03/2006
	static void RenderHeatHaze();

	static void SetEnableMultithreading(bool bEnable)	{ s_bEnableMultithreading = bEnable; }
	static bool GetEnableMultithreading(void)			{ return s_bEnableMultithreading; }

	//Get number of active PFX effects.
	static u_int GetNumEffects()	{return GetCurrentHandler().GetNumEffects();}

	// Effect registration interface
	static bool CanCreateEffect();
	static inline Asura_PFX_Effect* GetEffect(u_int uEffectID)		{return GetCurrentHandler().GetEffect(uEffectID);}
	// Returns the effect if the effect ID given is for a valid Asura_PFX_Entity_Effect, null otherwise.
	static inline Asura_PFX_Entity_Effect* GetEntityEffect(u_int uEffectID)
	{
		return GetCurrentHandler().GetEntityEffect(uEffectID);
	}
	// Sets the position and optionally direction of the Asura_PFX_Entity_Effect with given ID
	// Returns true if the effect exists, false otherwise.
	// If the effect does not exist, uEffectID will be set to ASURA_PFX_INVALID_EFFECT_ID.
	static inline bool SetEntityEffectPosition(u_int& uEffectID, const Asura_Vector_3& xPosition)
	{
		return GetCurrentHandler().SetEntityEffectPosition(uEffectID, xPosition);
	}
	static inline bool SetEntityEffectPosition(u_int& uEffectID, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection)
	{
		return GetCurrentHandler().SetEntityEffectPosition(uEffectID, xPosition, xDirection);
	}
	static inline bool SetEntityEffectPosition(u_int& uEffectID, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xDirection)
	{
		return GetCurrentHandler().SetEntityEffectPosition(uEffectID, xPosition, xDirection);
	}
	// Stops the Asura_PFX_Entity_Effect with given ID, using the immediate flag given.
	// Returns true if the effect exists, false otherwise.
	static inline bool StopEntityEffect(u_int uEffectID, bool bImmediate)
	{
		return GetCurrentHandler().StopEntityEffect(uEffectID, bImmediate);
	}
	// Destroys the effect if it exists and sets the ID to ASURA_PFX_INVALID_EFFECT_ID
	static inline void DestroyEffect(u_int& uEffectID)
	{
		GetCurrentHandler().DestroyEffect(uEffectID);
	}
	// No need for the user to call this - it is called by the effect constructor
	static inline void RegisterEffect(Asura_PFX_Effect* pxEffect)	{GetCurrentHandler().RegisterEffect(pxEffect,s_uNextEffectID++);}

#ifdef ASURA_SUPPORT_MULTITHREADED_JOB_QUEUE
	static void Platform_BeginUpdate();
	static void* Platform_GetRenderData( u_int uNumParticles );	// area to be filled out with vertex data, e.g. vertex buffer.
#endif
	static void Platform_BeginRender();
	static void Platform_EndRender();
	static bool Platform_SupportsHeatHaze();
	static void Platform_Initialise();
	static void Platform_Reset();
	static void Platform_Shutdown();

	static bool AddHeatHazeSource(Asura_PFX_Source *pxHeatHazeSource);
	static inline bool IsRenderingHeatHazeSources() { return s_bIsRenderingHeatHazeSources; }

	// Template store interface

	// Retrieve template by hash
	static Asura_PFX_Particle_Template* GetParticleTempByHash(Asura_Hash_ID uHashID, bool bIgnoreOverrides = false);
	static Asura_PFX_Source_Template*	GetSourceTempByHash(Asura_Hash_ID uHashID, bool bIgnoreOverrides = false);
	static Asura_PFX_Effect_Template*	GetEffectTempByHash(Asura_Hash_ID uHashID, bool bIgnoreOverrides = false);

	//Returns true if the Hash ID corresponds to a template.
	//Primarily for tools , since unlike GetParticleTemp etc it won't
	//cause the template to be loaded.
	static bool DoesParticleHashExist(Asura_Hash_ID uHashID);
	static bool DoesSourceHashExist(Asura_Hash_ID uHashID);
	static bool DoesEffectHashExist(Asura_Hash_ID uHashID);

	static Asura_PFX_ParticleTemplateResourceSet&	GetParticleTempResourceSet(void)	{ return(s_xParticleTemplateResourceSet); }
	static Asura_PFX_SourceTemplateResourceSet&		GetSourceTempResourceSet(void)		{ return(s_xSourceTemplateResourceSet); }
	static Asura_PFX_EffectTemplateResourceSet&		GetEffectTempResourceSet(void)		{ return(s_xEffectTemplateResourceSet); }

#ifdef ASURA_TOOLS
	

	// Tools only interface functions

	// Retrieve name by hash
	static const Asura_TemplateHashTree<Asura_String>&	GetParticleTempNames(void)	{ return(s_xParticleTempNames); }
	static const Asura_TemplateHashTree<Asura_String>&	GetSourceTempNames(void)	{ return(s_xSourceTempNames); }
	static const Asura_TemplateHashTree<Asura_String>&	GetEffectTempNames(void)	{ return(s_xEffectTempNames); }

	static const Asura_String* GetParticleTempNameByHash(Asura_Hash_ID uHashID);
	static const Asura_String* GetSourceTempNameByHash(Asura_Hash_ID uHashID);
	static const Asura_String* GetEffectTempNameByHash(Asura_Hash_ID uHashID);

	// Delete template methods
	static void DeleteParticleTemplate(Asura_Hash_ID uHashID);
	static void DeleteSourceTemplate(Asura_Hash_ID uHashID);
	static void DeleteEffectTemplate(Asura_Hash_ID uHashID);

	//Returns true if the hash value corresponds to a valid template , and
	//that template has been loaded.
	static bool IsParticleTempLoadedByHash(Asura_Hash_ID uHashID);
	static bool IsSourceTempLoadedByHash(Asura_Hash_ID uHashID);
	static bool IsEffectTempLoadedByHash(Asura_Hash_ID uHashID);

	static bool RenameParticleTemplate(const char* szOldName,const char* szNewName);
	static bool RenameSourceTemplate(const char* szOldName,const char* szNewName);
	static bool RenameEffectTemplate(const char* szOldName,const char* szNewName);

#endif

	// Add template methods
	// By hash, from pointer
	static bool AddParticleTemplate(Asura_Hash_ID uNameHash, Asura_PFX_Particle_Template* pxTemplate);
	static bool AddSourceTemplate(Asura_Hash_ID uNameHash, Asura_PFX_Source_Template* pxTemplate);
	static bool AddEffectTemplate(Asura_Hash_ID uNameHash, Asura_PFX_Effect_Template* pxTemplate);

	//These check for the existence of a template with the given ID , and if the template exists , then
	//a reference to that template is added to the current resource bank
	//Returns true if the template existed and false if not
	//The particle templates aren't done in quite the same way because of a nasty need to
	//update the texture handle on the PS3
	static bool AddExistingSourceTemplateToCurrentBank(Asura_Hash_ID uNameHash);
	static bool AddExistingEffectTemplateToCurrentBank(Asura_Hash_ID uNameHash);

#ifdef ASURA_TOOLS
	// By name, from pointer
	static bool AddParticleTemplate(const char* szName, Asura_PFX_Particle_Template* pxTemplate);
	static bool AddSourceTemplate(const char* szName, Asura_PFX_Source_Template* pxTemplate);
	static bool AddEffectTemplate(const char* szName, Asura_PFX_Effect_Template* pxTemplate);
#endif

#ifdef ASURA_TOOLS
	//clears out existing effects , and prevents any new ones being created
	static void DisablePFXSystem(bool bDisable);

	//Sets an RNG to be used by the heat haze system to allow it to be consistent when recording AVIs
	static void SetHeatHazeRNG(Asura_RandomGenerator* pxRNG);
#endif

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	// Handler functions...
	static	Asura_PFX_Handler*			CreateHandler			(const Asura_Hash_ID uHash);
	static	bool						DestroyHandler			(const Asura_Hash_ID uHash);

	static	void						SetCurrentHandler		(Asura_PFX_Handler* pxHandler)	{ s_pxCurrentHandler = pxHandler; }
	static	void						SetCurrentHandler		(const Asura_Hash_ID uHash)		{ s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHash); }
	static	Asura_PFX_Handler*			GetCurrentHandler		(const Asura_Hash_ID uHash)		{ return(s_xHandlerTree.GetDataByHash(uHash)); }
	static	Asura_PFX_Handler&			GetCurrentHandler		(void)							{ return(*s_pxCurrentHandler); }

	static void							CreateAndSetHandler		(const Asura_Hash_ID uHash)		{ s_pxCurrentHandler = CreateHandler(uHash); }

	// Called after a zone has loaded. Gives handlers a chance to update module lists.
	static void PostLoadInitialise() { s_xHandlerTree.Traverse(PostLoadInitialise_Callback); }
	
	static void PostLoadInitialise_Callback(Asura_Hash_ID uHash, Asura_PFX_Handler* pxHandler)	{ pxHandler->PostLoadInitialise(); }
#endif

	static void IncrementPlayingSounds() { ++s_uNumPFXSounds; }
	static u_int GetNumPlayingSounds() { return s_uNumPFXSounds; }
#ifdef ASURA_TOOLS
	//Reloading of templates should only be allowed when we are explicitly attempting to reload them.
	//This is to fix issues from particle templates being reloaded with the new system because they exist
	//in multiple effect files , and causing all the effect instances to be deleted at awkward moments.
	//RAR 19/01/2007
	static void SetAllowTemplateReload(bool bAllow) {s_bAllowTemplateReload = bAllow;}
#endif

#ifdef ASURA_TOOLS
	//Deletes all templates and 'reloads' them
	//(Actually just sets up slots for them , so that they will be loaded when next needed).
	static void ReloadAllTemplates();
#endif

	static ASURA_PFX_RENDER_TYPE GetCurrentRenderType() { return s_eCurrentRenderType; }

	static void SetRenderstateOverrideHook(PFXRenderStateOverrideHook* const pfnRenderStateHook = NULL) { s_pfnRenderStateOverride = pfnRenderStateHook; }
	static void SetParticleColourOverrideHook(ParticleColourOverrideHook* const pfnParticleColourHook = NULL) { s_pfnParticleColourOverride = pfnParticleColourHook; }
	static PFXRenderStateOverrideHook* GetRenderStateOverride() { return s_pfnRenderStateOverride; }
	static ParticleColourOverrideHook* GetParticleColourOverride() { return s_pfnParticleColourOverride; }

	static void SetClearTemplatesOnReset( bool bClear ) { s_bClearTemplatesOnReset = bClear; }

	static Asura_PFX_RenderQueue&	GetRenderQueue(void)	{ return(s_xRenderQueue); }


private:
	// Template store helper functions
	static void ClearAllTemplates();
	static bool CanParticleTemplateBeAdded(Asura_Hash_ID uNameHash);
	static bool CanSourceTemplateBeAdded(Asura_Hash_ID uNameHash);
	static bool CanEffectTemplateBeAdded(Asura_Hash_ID uNameHash);

#ifdef ASURA_TOOLS
	static void LoadAllTemplateFiles();
	static void LoadTemplate(int iType, Asura_Hash_ID uHash = 0);

	//For loading templates on demand (TM)
	static Asura_PFX_Particle_Template* LoadParticleTemplateByHash(Asura_Hash_ID uHash);
	static Asura_PFX_Source_Template* LoadSourceTemplateByHash(Asura_Hash_ID uHash);
	static Asura_PFX_Effect_Template* LoadEffectTemplateByHash(Asura_Hash_ID uHash);
	static Asura_PFX_Effect_Template* LoadTemplateByHash(Asura_Hash_ID uEffectHash);
	static Asura_PFX_Effect_Template* LoadTemplate( const Asura_String& xTemplateName );
	static Asura_PFX_Effect_Template* LoadEffectTemplateByOverrideName( const Asura_String& xOverrideName );
#endif


	static void ClearAllEffectsCallback(Asura_Hash_ID uHash, Asura_PFX_Handler* pxHandler);
	#ifdef ASURA_TOOLS
	static bool ClearAllEffectsUsingEffectTemplateCallback(Asura_Hash_ID uHash, Asura_PFX_Handler* pxHandler, void* pEffectHash);
	#endif

	// Effect management
	static u_int							s_uNextEffectID;

	// Template store members

	static Asura_PFX_ParticleTemplateResourceSet	s_xParticleTemplateResourceSet;
	static Asura_PFX_SourceTemplateResourceSet		s_xSourceTemplateResourceSet;
	static Asura_PFX_EffectTemplateResourceSet		s_xEffectTemplateResourceSet;

	static bool								s_bIsRenderingHeatHazeSources;

	//Render queue
	static Asura_PFX_RenderQueue	s_xRenderQueue;

#ifdef ASURA_TOOLS
	static Asura_TemplateHashTree<Asura_String>    s_xParticleTempNames;
	static Asura_TemplateHashTree<Asura_String>    s_xSourceTempNames;
	static Asura_TemplateHashTree<Asura_String>    s_xEffectTempNames;
#endif

#ifdef ASURA_TOOLS
	static bool								s_bPFXSystemDisabled;
	static bool								s_bAllowTemplateReload;
#endif
	static bool								s_bEnableMultithreading;

	static ASURA_PFX_RENDER_TYPE			s_eCurrentRenderType;

	static u_int							s_uNumPFXSounds;

	static bool								s_bClearTemplatesOnReset;

#if defined(ASURA_USE_MULTIPLE_ZONE_HANDLERS)
	// Private variables...
	static	Asura_PFX_Handler*								s_pxCurrentHandler;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_PFX_Handler>		s_xHandlerTree;			// Tree of all of the handlers available.
	static  ASURA_PFX_RENDER_TYPE s_eCallbackRenderType;

	// Private functions...
	static void Render_Callback(Asura_Hash_ID uHash, Asura_PFX_Handler* pxData)
	{
		if (pxData) pxData->Render(s_eCallbackRenderType);
	}
	static void Update_Callback(Asura_Hash_ID uHash, Asura_PFX_Handler* pxData)
	{
		if (pxData) pxData->Update();
	}
#else
	// Private variables...
	static	Asura_PFX_Handler 								s_xHandler;// Single static-instance of a zone.

	// Private functions...
	static	Asura_PFX_Handler& GetCurrentHandler(void) { return(s_xHandler); }
#endif

	static ParticleColourOverrideHook* s_pfnParticleColourOverride;
	static PFXRenderStateOverrideHook* s_pfnRenderStateOverride;
};



#endif//__ASURA_PFX_SYSTEM_H__
