//
// Asura_PFX_Volume.cpp
//
// Created by KOR - 15/07/2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"


// This file's header.
#include "SpecialFX/PFX/Asura_PFX_Volume.h"

// Asura headers
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

/// Asura_PFX_Volume chunk methods ////////////////////////////////////////////////////////////////
Asura_PFX_Volume::Asura_PFX_Volume(Asura_Handle iFileHandle)
{
	ProcessFromFileHandle(iFileHandle);
}

void Asura_PFX_Volume::ProcessFromFileHandle(Asura_Handle iFileHandle)
{
	ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_eType, ASURA_PFX_VOLUME_TYPE);
	ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_uFlags, u_int);
	switch(m_eType)
	{
		case ASURA_PFX_VOLUME_POINT:
			break;
		case ASURA_PFX_VOLUME_CUBOID:
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_xCuboidDesc, Asura_PFX_Volume_Cuboid_Desc);
			break;
		case ASURA_PFX_VOLUME_SPHERE:
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_xSphereDesc, Asura_PFX_Volume_Sphere_Desc);
			break;
		case ASURA_PFX_VOLUME_SPHEROID:
			ASURA_CHUNK_FILEEXTRACT(iFileHandle, m_xSpheroidDesc, Asura_PFX_Volume_Spheroid_Desc);
			break;
		default: Asura_Assert(0, "Unknown volume type");
	}
}

void Asura_PFX_Volume::ProcessFromChunkBuffer(const char *&pcChunkBuffer)
{
	ASURA_CHUNK_EXTRACT(pcChunkBuffer, m_eType, ASURA_PFX_VOLUME_TYPE);
	ASURA_CHUNK_EXTRACT(pcChunkBuffer, m_uFlags, u_int);
	switch(m_eType)
	{
		case ASURA_PFX_VOLUME_POINT:
			break;
		case ASURA_PFX_VOLUME_CUBOID:
			ASURA_CHUNK_EXTRACT(pcChunkBuffer, m_xCuboidDesc, Asura_PFX_Volume_Cuboid_Desc);
			break;
		case ASURA_PFX_VOLUME_SPHERE:
			ASURA_CHUNK_EXTRACT(pcChunkBuffer, m_xSphereDesc, Asura_PFX_Volume_Sphere_Desc);
			break;
		case ASURA_PFX_VOLUME_SPHEROID:
			ASURA_CHUNK_EXTRACT(pcChunkBuffer, m_xSpheroidDesc, Asura_PFX_Volume_Spheroid_Desc);
			break;
		default: Asura_Assert(0, "Unknown volume type");
	}
}


void Asura_PFX_Volume::FillInChunkBuffer(char *&pcChunkBuffer) const
{
	ASURA_CHUNK_FILLANDVERIFY(pcChunkBuffer, m_eType, ASURA_PFX_VOLUME_TYPE);
	ASURA_CHUNK_BITFIELD_FILLANDVERIFY(pcChunkBuffer, m_uFlags, u_int);
	switch(m_eType)
	{
		case ASURA_PFX_VOLUME_POINT:
			break;
		case ASURA_PFX_VOLUME_CUBOID:
			ASURA_CHUNK_ARRAY_FILLANDVERIFY(pcChunkBuffer, m_xCuboidDesc, Asura_PFX_Volume_Cuboid_Desc, float, 6);
			break;
		case ASURA_PFX_VOLUME_SPHERE:
			ASURA_CHUNK_ARRAY_FILLANDVERIFY(pcChunkBuffer, m_xSphereDesc, Asura_PFX_Volume_Sphere_Desc, float, 4);
			break;
		case ASURA_PFX_VOLUME_SPHEROID:
			ASURA_CHUNK_ARRAY_FILLANDVERIFY(pcChunkBuffer, m_xSpheroidDesc, Asura_PFX_Volume_Spheroid_Desc, float, 6);
			break;
		default: Asura_Assert(0, "Unknown volume type");
	}
}

void Asura_PFX_Volume::ReadFromChunkStream (Asura_Chunk_Stream& xStream)
{
	Asura_ReadFromStream_Bitfield_Wrapper<u_int>			xFlagsBitfield (m_uFlags);
	Asura_ReadFromStream_Enum_Wrapper<ASURA_PFX_VOLUME_TYPE>	xVolumeTypeEnum (m_eType);

	xStream >> xVolumeTypeEnum;
	xStream >> xFlagsBitfield;

	switch(m_eType)
	{
		case ASURA_PFX_VOLUME_POINT:
			break;
		case ASURA_PFX_VOLUME_CUBOID:
			xStream >> m_xCuboidDesc.m_xBB;
			break;
		case ASURA_PFX_VOLUME_SPHERE:
			xStream >> m_xSphereDesc.m_fRadius;
			break;
		case ASURA_PFX_VOLUME_SPHEROID:
			xStream >> m_xSpheroidDesc.m_fExtentX >> m_xSpheroidDesc.m_fExtentY >> m_xSpheroidDesc.m_fExtentZ;
			break;
		default: Asura_Assert(0, "Unknown volume type");
	}	
}

void Asura_PFX_Volume::WriteToChunkStream (Asura_Chunk_Stream& xStream) const
{
	Asura_WriteToStream_Bitfield_Wrapper<u_int>			xFlagsBitfield (m_uFlags);
	Asura_WriteToStream_Enum_Wrapper<ASURA_PFX_VOLUME_TYPE>	xVolumeTypeEnum (m_eType);


	// Create the bitfield...

	xFlagsBitfield.Add	(m_bSpawnOnSurface,	1);
	xFlagsBitfield.Add	(m_bUseOrientation,	1);


	// Write the data...

	xStream << xVolumeTypeEnum;
	xStream << xFlagsBitfield;

	switch(m_eType)
	{
		case ASURA_PFX_VOLUME_POINT:
			break;
		case ASURA_PFX_VOLUME_CUBOID:
			xStream << m_xCuboidDesc.m_xBB;
			break;
		case ASURA_PFX_VOLUME_SPHERE:
			xStream << m_xSphereDesc.m_fRadius;
			break;
		case ASURA_PFX_VOLUME_SPHEROID:
			xStream << m_xSpheroidDesc.m_fExtentX << m_xSpheroidDesc.m_fExtentY << m_xSpheroidDesc.m_fExtentZ;
			break;
		default: Asura_Assert(0, "Unknown volume type");
	}	
}


u_int Asura_PFX_Volume::GetChunkBufferSize() const
{
	switch(m_eType)
	{
		case ASURA_PFX_VOLUME_POINT:
			return sizeof(ASURA_PFX_VOLUME_TYPE) + sizeof(u_int);
		case ASURA_PFX_VOLUME_CUBOID:
			return sizeof(ASURA_PFX_VOLUME_TYPE) + sizeof(Asura_PFX_Volume_Cuboid_Desc) + sizeof(u_int);
		case ASURA_PFX_VOLUME_SPHERE:
			return sizeof(ASURA_PFX_VOLUME_TYPE) + sizeof(Asura_PFX_Volume_Sphere_Desc) + sizeof(u_int);
		case ASURA_PFX_VOLUME_SPHEROID:
			return sizeof(ASURA_PFX_VOLUME_TYPE) + sizeof(Asura_PFX_Volume_Spheroid_Desc) + sizeof(u_int);
		default: Asura_Assert(0, "Unknown volume type"); return 0;
	}
}


Asura_Vector_3 Asura_PFX_Volume::GeneratePointInVolume(Asura_RandomGenerator* pxRNG) const
{
	switch(m_eType)
	{
		case ASURA_PFX_VOLUME_POINT :
			return Asura_Maths::ZeroVector3;

		case ASURA_PFX_VOLUME_CUBOID :
		{
			if(m_bSpawnOnSurface)
			{
				//pick a random side to spawn on (should this be weighted by area?)
				int iSide = Asura_Random::GetInt(0,5);
				
				Asura_Vector_3 xPos;
				if( iSide == 0)
				{
					xPos.x = m_xCuboidDesc.m_xBB.MinX;
				}
				else if(iSide == 1)
				{
					xPos.x = m_xCuboidDesc.m_xBB.MaxX;
				}
				else
				{
					xPos.x = pxRNG->GetFloat(m_xCuboidDesc.m_xBB.MinX,m_xCuboidDesc.m_xBB.MaxX);
				}

				if( iSide == 2)
				{
					xPos.y = m_xCuboidDesc.m_xBB.MinY;
				}
				else if(iSide == 3)
				{
					xPos.y = m_xCuboidDesc.m_xBB.MaxY;
				}
				else
				{
					xPos.y = pxRNG->GetFloat(m_xCuboidDesc.m_xBB.MinY,m_xCuboidDesc.m_xBB.MaxY);
				}

				if( iSide == 4)
				{
					xPos.z = m_xCuboidDesc.m_xBB.MinZ;
				}
				else if(iSide == 5)
				{
					xPos.z = m_xCuboidDesc.m_xBB.MaxZ;
				}
				else
				{
					xPos.z = pxRNG->GetFloat(m_xCuboidDesc.m_xBB.MinZ,m_xCuboidDesc.m_xBB.MaxZ);
				}

				return xPos;
	
			}
			else
			{
				Asura_Vector_3 xPos;
				xPos.x = pxRNG->GetFloat(m_xCuboidDesc.m_xBB.MinX,m_xCuboidDesc.m_xBB.MaxX);
				xPos.y = pxRNG->GetFloat(m_xCuboidDesc.m_xBB.MinY,m_xCuboidDesc.m_xBB.MaxY);
				xPos.z = pxRNG->GetFloat(m_xCuboidDesc.m_xBB.MinZ,m_xCuboidDesc.m_xBB.MaxZ);

				return xPos;
			}
		}

		case ASURA_PFX_VOLUME_SPHERE :
		{
			Asura_Vector_3 xPos;
			if(m_bSpawnOnSurface)
			{
				//get point on unit sphere	
				xPos.RandomDirection(pxRNG);
			}
			else
			{
				//get point in unit sphere	
				xPos.RandomVector(pxRNG);
			}

			//and scale by the radius
			xPos *= m_xSphereDesc.m_fRadius;

			return xPos;
		}

		case ASURA_PFX_VOLUME_SPHEROID :
		{
			Asura_Vector_3 xPos;
			if(m_bSpawnOnSurface)
			{
				//get point on unit sphere	
				xPos.RandomDirection(pxRNG);
			}
			else
			{
				//get point in unit sphere	
				xPos.RandomVector(pxRNG);
			}

			//and scale by the extents
			xPos.x *= m_xSpheroidDesc.m_fExtentX;
			xPos.y *= m_xSpheroidDesc.m_fExtentY;
			xPos.z *= m_xSpheroidDesc.m_fExtentZ;

			return xPos;
		}


		default: Asura_Assert(0, "Unknown volume type"); return Asura_Maths::ZeroVector3;
	}
}

float Asura_PFX_Volume::GetVolumeSize() const
{
	switch(m_eType)
	{
		case ASURA_PFX_VOLUME_POINT: return 0.f;
		case ASURA_PFX_VOLUME_CUBOID:
		{
			const Asura_Bounding_Box& xBB = m_xCuboidDesc.m_xBB;
			const float fX = xBB.MaxX - xBB.MinX;
			const float fY = xBB.MaxY - xBB.MinY;
			const float fZ = xBB.MaxZ - xBB.MinZ;
			return fX * fY * fZ;
		}
		case ASURA_PFX_VOLUME_SPHERE:
		{
			const float fRadius = m_xSphereDesc.m_fRadius;
			return (4.f / 3.f) * Asura_Maths::Pi * fRadius * fRadius * fRadius;
		}
		case ASURA_PFX_VOLUME_SPHEROID:
		{
			const float fX = m_xSpheroidDesc.m_fExtentX;
			const float fY = m_xSpheroidDesc.m_fExtentY;
			const float fZ = m_xSpheroidDesc.m_fExtentZ;
			return (4.f / 3.f) * Asura_Maths::Pi * fX * fY * fZ;
		}
		default: Asura_Assert(0, "Unknown volume type"); return 0.f;
	}
}



/// End of file. ///
