//
// Asura_PFX_Volume.h
//
// Created by KOR - 15/07/2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PFX_VOLUME_H__
#define __ASURA_PFX_VOLUME_H__

#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"

class Asura_Chunk_Stream;

/// Asura_PFX_Volume //////////////////////////////////////////////////////////////////////////////
// Used for source generation volume definition
///////////////////////////////////////////////////////////////////////////////////////////////////

enum ASURA_PFX_VOLUME_TYPE
{
	ASURA_PFX_VOLUME_POINT, 
	ASURA_PFX_VOLUME_CUBOID, 
	ASURA_PFX_VOLUME_SPHERE, 
	ASURA_PFX_VOLUME_SPHEROID,
};


//All the volume information is taken relative to a centre position that
//is stored in the source template itself. Hence the lack of any structure for the
//point description , since no extra information is needed.
//RAR  13/09/04

struct Asura_PFX_Volume_Cuboid_Desc
{
	Asura_Bounding_Box m_xBB;
};

struct Asura_PFX_Volume_Sphere_Desc
{
	float			m_fRadius;
};

struct Asura_PFX_Volume_Spheroid_Desc
{
	float			m_fExtentX;
	float			m_fExtentY;
	float			m_fExtentZ;
};

class Asura_PFX_Volume
{
public:
	// Point constructor
	Asura_PFX_Volume()
	: m_eType(ASURA_PFX_VOLUME_POINT)
	,m_uFlags(0)
	{
	}
	// Cuboid constructor
	Asura_PFX_Volume(const Asura_Bounding_Box& xBB)
	: m_eType(ASURA_PFX_VOLUME_CUBOID)
	,m_uFlags(0)
	{
		m_xCuboidDesc.m_xBB = xBB;
	}
	// Sphere constructor
	Asura_PFX_Volume(float fRadius)
	: m_eType(ASURA_PFX_VOLUME_SPHERE)
	,m_uFlags(0)
	{
		m_xSphereDesc.m_fRadius = fRadius;
	}
	// Spheroid constructor
	Asura_PFX_Volume(const Asura_Vector_3& xExtent)
	: m_eType(ASURA_PFX_VOLUME_SPHEROID)
	,m_uFlags(0)
	{
		m_xSpheroidDesc.m_fExtentX = xExtent.x;
		m_xSpheroidDesc.m_fExtentY = xExtent.y;
		m_xSpheroidDesc.m_fExtentZ = xExtent.z;
	}
	// File constructor
	Asura_PFX_Volume(Asura_Handle iFileHandle);
	void ProcessFromFileHandle(Asura_Handle iFileHandle);
	void ProcessFromChunkBuffer(const char *&pcChunkBuffer);

	// Chunk methods
	void FillInChunkBuffer(char *&pcChunkBuffer) const;
	u_int GetChunkBufferSize() const;
	void ReadFromChunkStream (Asura_Chunk_Stream& xStream);
	void WriteToChunkStream (Asura_Chunk_Stream& xStream) const;

	//Picks a random point in the volume
	Asura_Vector_3 GeneratePointInVolume(Asura_RandomGenerator* pxRNG) const;

	// Returns volume of volume in m^3
	float GetVolumeSize() const;

	Asura_PFX_Volume& operator *= ( float fScale )
	{
		// Bounding box has the most members, so let's just use that
		m_xCuboidDesc.m_xBB.MinX *= fScale;
		m_xCuboidDesc.m_xBB.MaxX *= fScale;
		m_xCuboidDesc.m_xBB.MinY *= fScale;
		m_xCuboidDesc.m_xBB.MaxY *= fScale;
		m_xCuboidDesc.m_xBB.MinZ *= fScale;
		m_xCuboidDesc.m_xBB.MaxZ *= fScale;
		return *this;
	}

	// Members
	ASURA_PFX_VOLUME_TYPE m_eType;
	union
	{
		Asura_PFX_Volume_Cuboid_Desc	m_xCuboidDesc;	// 24
		Asura_PFX_Volume_Sphere_Desc	m_xSphereDesc;	// 4
		Asura_PFX_Volume_Spheroid_Desc	m_xSpheroidDesc;// 12
	};
													// Max: 24 bytes
	union
	{
		struct
		{
			// NOTE: These bit's are written out, adding any new bits means
			// ~~~~~ that you will need to add those bits to the bitfield
			//       preparation code in "WriteToChunkStream".
			u_int	m_bSpawnOnSurface : 1;//Particles use surface of volume , instead  of being generated at random points within.
			u_int	m_bUseOrientation : 1;//The volume should be orientated by the direction of the source
		};

		u_int m_uFlags;
	};
};

#ifdef ASURA_TOOLS
//defines for flags for use with flag tickboxes
#define ASURA_PFXVOLUME_SPAWNONSURFACE 0x00000001
#define ASURA_PFXVOLUME_USEORIENTATION 0x00000002
#endif

#endif//__ASURA_PFX_VOLUME_H__
