///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Streaming_HandlerReplacement.h
// Author : Martin G. Harris
// Date   : 23-May-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
// Helper classes to temporarily replace handlers and then restore them later.
//
///////////////////////////////////////////////////////////////////////////////

// RJT NOTE: Required for GCC builds as the file list generation doesn't account for VC project exclusion from build
#ifdef ASURA_USE_STREAMING

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_STREAMING_HANDLER_REPLACEMENT_H__
#define __ASURA_STREAMING_HANDLER_REPLACEMENT_H__

// Includes
#include "Core/Asura_Hash.h"

////////////////////////////////////////
// Asura_Streaming_HandlerReplacement //
////////////////////////////////////////
// Replace all relevant handlers.
class Asura_Streaming_HandlerReplacement
{
public:
	// Pass in the hashes of the master zone for the entity and the containing zone.
	Asura_Streaming_HandlerReplacement(const Asura_Hash_ID uZoneHash, const Asura_Hash_ID uContainingZoneHash);
	~Asura_Streaming_HandlerReplacement();

private:
	class Asura_AI_Navigation_Handler*	m_pxAINavHandler;
};

///////////////////////////////////////////////
// Asura_Streaming_TexAnimHandlerReplacement //
///////////////////////////////////////////////
// Replace texture animation handler.
class Asura_Streaming_TexAnimHandlerReplacement
{
public:
	Asura_Streaming_TexAnimHandlerReplacement(const Asura_Hash_ID uZoneHash);
	~Asura_Streaming_TexAnimHandlerReplacement();

private:
	class Asura_TextureAnimation_Handler* m_pxTexAnimHandler;
};

//////////////////////////////////////////////
// Asura_Streaming_ModuleHandlerReplacement //
//////////////////////////////////////////////
// Replace module and environment handlers.
class Asura_Streaming_ModuleHandlerReplacement
{
public:
	Asura_Streaming_ModuleHandlerReplacement();		// Save old handlers.
	~Asura_Streaming_ModuleHandlerReplacement();	// Restore old handlers.

	// Are current handlers loaded?
	static bool IsLoaded();

	// Set new handlers.
	static void SetCurrent(const Asura_Hash_ID uContainingZoneHash);

private:
	class Asura_Module_Handler*	m_pxOldModuleHandler;
	class Asura_Environment*	m_pxOldEnvironmentHandler;
};


#endif //__ASURA_STREAMING_HANDLER_REPLACEMENT_H__

#endif	// ASURA_USE_STREAMING
