
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Streaming_ZoneEntities.cpp
// Author : Andy Tate
// Date   : 18-Apr-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// RJT NOTE: Required for GCC builds as the file list generation doesn't account for VC project exclusion from build
#ifdef ASURA_USE_STREAMING

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Streaming/Asura_Streaming_ZoneEntities.h"

// Asura independent header-files...
#include "Entity/Asura_Entity_System.h"
#include "Network/Asura_Network.h"

// Asura platform header-files...

// Game independent header-files...

// Game platform header-files...

// Tools header-files...


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_ZoneEntities::Asura_Streaming_ZoneEntities
// Author       : Andy Tate
// Date         : 18-Apr-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Streaming_ZoneEntities::Asura_Streaming_ZoneEntities (void)
: m_xClientEntityList(5)
, m_xServerEntityList(5)
{
	// Nothing more to initialise...
}

/// End of function 'Asura_Streaming_ZoneEntities::Asura_Streaming_ZoneEntities'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_ZoneEntities::~Asura_Streaming_ZoneEntities
// Author       : Andy Tate
// Date         : 18-Apr-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Streaming_ZoneEntities::~Asura_Streaming_ZoneEntities ()
{
	// Destroy all of the lists...
	Asura_Entity_System::BeginEntityDestruction();

	m_xClientEntityList.DestroyAllObjects();
	m_xServerEntityList.DestroyAllObjects();

	Asura_Entity_System::EndEntityDestruction();
}

/// End of function 'Asura_Streaming_ZoneEntities::~Asura_Streaming_ZoneEntities'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_ZoneEntities::Update
// Author       : Andy Tate
// Date         : 18-Apr-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_ZoneEntities::Update (void)
{
	// Ensure we've sent any outstanding messages...
	FlushMessageQueue();

	// Update the appropriate list(s)...
	Asura_Entity_System::Update(m_xClientEntityUpdateableList, m_xServerEntityUpdateableList);
//	m_xServerEntityUpdateableList.OutputToDebugger("Updating Server entities");
//	m_xClientEntityUpdateableList.OutputToDebugger("Updating Client entities");
}

/// End of function 'Asura_Streaming_ZoneEntities::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_ZoneEntities::Render
// Author       : Andy Tate
// Date         : 18-Apr-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_ZoneEntities::Render (void)
{
	// Render the appropriate list(s)...
	Asura_Entity_System::Render(m_xRenderableEntityList);
//	m_xRenderableEntityList.OutputToDebugger("Renderable entities");
}

/// End of function 'Asura_Streaming_ZoneEntities::Render'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_ZoneEntities::ResetConstantClientUpdateList
// Author       : M.G.Harris
// Date         : 27-Apr-2006
////////////////////////////////////////////////////////////////////////////////
void Asura_Streaming_ZoneEntities::ResetConstantClientUpdateList()
{
	for( Asura_Hashed_List_Deleter_It<Asura_Entity_Server> xIt(&m_xConstantClientUpdateList); !xIt.Done(); )
    {
        xIt.DeleteCurrent();
    }
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_ZoneEntities::DoConstantClientUpdates
// Author       : M.G.Harris
// Date         : 27-Apr-2006
////////////////////////////////////////////////////////////////////////////////
void Asura_Streaming_ZoneEntities::DoConstantClientUpdates()
{
	for( Asura_Hashed_List_It<Asura_Entity_Server> xIt(&m_xConstantClientUpdateList); !xIt.Done(); xIt.Next() )
    {
		Asura_Entity_Server* pxEntity = xIt.GetCurrent();
		if( pxEntity && !pxEntity->IsDestroyed())
		{
			pxEntity->MakeClientUpdateMessage();
		}		
    }
}

void Asura_Streaming_ZoneEntities::DestroySavedEntities()
{
	// Client lists can go completely
	m_xClientEntityList.DestroyAllObjects();

	Asura_Classified_List_Deleter_It<Asura_Entity_Server> xDeleterIt(m_xServerEntityList.GetList());

	while (!xDeleterIt.Done())
	{
		if (xDeleterIt.GetCurrent() && xDeleterIt.GetCurrent()->IsDestroyedOnQuickload())
		{
			xDeleterIt.DeleteCurrent(); // This goes to the next anyway
		}
		else
		{
			xDeleterIt.Next();
		}
	}

	// Also clear out the message queue, as those messages are associated with things we're about to delete/reload
	ClearMessageQueue();
}



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_ZoneEntities::FlushMessageQueue
// Author       : Andy Tate
// Date         : 1-Jun-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_ZoneEntities::FlushMessageQueue (void)
{
	// Loop through the messages on the queue...
	while(m_xMessageQueue.Size() > 0)
	{
		Asura_Message*	pxMessage	= m_xMessageQueue.Front()->Data();

		// Send the message...
		Asura_Entity_System::DeliverMessageToServerEntity(pxMessage);

		// Tidy-up...
		Asura_Network::DestroyMessage(pxMessage);
		m_xMessageQueue.Remove(m_xMessageQueue.Front());
	}
}

/// End of function 'Asura_Streaming_ZoneEntities::FlushMessageQueue'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_ZoneEntities::ClearMessageQueue
// Author       : Anthony Gregson
// Date         : 21/06/06
////////////////////////////////////////////////////////////////////////////////

// As above, but don't send the messages
void Asura_Streaming_ZoneEntities::ClearMessageQueue(void)
{
	// Loop through the messages on the queue...
	while(m_xMessageQueue.Size() > 0)
	{
		Asura_Message*	pxMessage	= m_xMessageQueue.Front()->Data();

		Asura_Network::DestroyMessage(pxMessage);

		m_xMessageQueue.Remove(m_xMessageQueue.Front());
	}
}

/// End of function 'Asura_Streaming_ZoneEntities::FlushMessageQueue'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_ZoneEntities::MigrateEntity
// Author       : Anthony Gregson
// Date         : 14/09/06
////////////////////////////////////////////////////////////////////////////////

// Migrate an entity (that is allowed to do so) from one instance to another
void Asura_Streaming_ZoneEntities::MigrateEntity(Asura_Entity_Client* pxClientEntity, Asura_Streaming_ZoneEntities* pxOldEntities, Asura_Streaming_ZoneEntities* pxNewEntities)
{
	Asura_Assert(pxClientEntity && pxClientEntity->GetCanChangeZone(), "Cannot change the zone of this entity");
	Asura_Assert(pxOldEntities && pxNewEntities, "Missing zone entities instance");

	if (pxClientEntity && pxClientEntity->GetCanChangeZone() && pxOldEntities && pxNewEntities)
	{
		Asura_Guid uEntityGuid = pxClientEntity->GetGuid();

		if (pxOldEntities->m_xClientEntityList.FindEntity(uEntityGuid))
		{
			pxOldEntities->RemoveEntityFromList(*pxClientEntity);
			pxNewEntities->AddEntityToList(*pxClientEntity);
		}

		if (pxOldEntities->m_xClientEntityUpdateableList.FindEntity(uEntityGuid))
		{
			pxOldEntities->RemoveEntityFromUpdateableList(*pxClientEntity, true);
			pxNewEntities->AddEntityToUpdateableList(*pxClientEntity);
		}
	}
}

void Asura_Streaming_ZoneEntities::MigrateEntity(Asura_Entity_Renderable* pxRenderableEntity, Asura_Streaming_ZoneEntities* pxOldEntities, Asura_Streaming_ZoneEntities* pxNewEntities)
{
	Asura_Assert(pxRenderableEntity && pxRenderableEntity->GetCanChangeZone(), "Cannot change the zone of this entity");
	Asura_Assert(pxOldEntities && pxNewEntities, "Missing zone entities instance");

	if (pxRenderableEntity && pxRenderableEntity->GetCanChangeZone() && pxOldEntities && pxNewEntities)
	{
		Asura_Guid uEntityGuid = pxRenderableEntity->GetGuid();

		if (pxOldEntities->m_xClientEntityList.FindEntity(uEntityGuid))
		{
			pxOldEntities->RemoveEntityFromList(*pxRenderableEntity);
			pxNewEntities->AddEntityToList(*pxRenderableEntity);
		}

		if (pxOldEntities->m_xClientEntityUpdateableList.FindEntity(uEntityGuid))
		{
			pxOldEntities->RemoveEntityFromUpdateableList(*pxRenderableEntity, true);
			pxNewEntities->AddEntityToUpdateableList(*pxRenderableEntity);
		}

		if (pxOldEntities->m_xRenderableEntityList.FindEntity(uEntityGuid))
		{
			pxOldEntities->RemoveRenderableEntityFromList(*pxRenderableEntity, true);
			pxNewEntities->AddRenderableEntityToList(*pxRenderableEntity);
		}
	}
}

void Asura_Streaming_ZoneEntities::MigrateEntity(Asura_Entity_Server* pxServerEntity, Asura_Streaming_ZoneEntities* pxOldEntities, Asura_Streaming_ZoneEntities* pxNewEntities)
{
	Asura_Assert(pxServerEntity && pxServerEntity->GetCanChangeZone(), "Cannot change the zone of this entity");
	Asura_Assert(pxOldEntities && pxNewEntities, "Missing zone entities instance");

	if (pxServerEntity && pxServerEntity->GetCanChangeZone() && pxOldEntities && pxNewEntities)
	{
		Asura_Guid uEntityGuid = pxServerEntity->GetGuid();

		if (pxOldEntities->m_xServerEntityList.FindEntity(uEntityGuid))
		{
			pxOldEntities->RemoveEntityFromList(*pxServerEntity);
			pxNewEntities->AddEntityToList(*pxServerEntity);
		}

		if (pxOldEntities->m_xServerEntityUpdateableList.FindEntity(uEntityGuid))
		{
			pxOldEntities->RemoveEntityFromUpdateableList(*pxServerEntity, true);
			pxNewEntities->AddEntityToUpdateableList(*pxServerEntity);
		}

		if (pxOldEntities->m_xPeriodicClientUpdateList.IsInList(uEntityGuid))
		{
			pxOldEntities->m_xPeriodicClientUpdateList.RemoveFromList(pxServerEntity);
			pxNewEntities->m_xPeriodicClientUpdateList.AddToList(pxServerEntity);
		}

		if (pxOldEntities->m_xConstantClientUpdateList.FindEntity(uEntityGuid))
		{
			pxOldEntities->m_xConstantClientUpdateList.RemoveFromList(pxServerEntity);
			pxNewEntities->m_xConstantClientUpdateList.AddToList(pxServerEntity);
		}
	}
}

/// End of function 'Asura_Streaming_ZoneEntities::MigrateEntity'. ///

#endif	// ASURA_USE_STREAMING

/// End of file. ///
