
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Streaming_Job.cpp
// Author : Andy Tate
// Date   : 4-May-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// RJT NOTE: Required for GCC builds as the file list generation doesn't account for VC project exclusion from build
#ifdef ASURA_USE_STREAMING

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Streaming/Jobs/Asura_Streaming_Job.h"

// Asura independent header-files...
#include "Core/Asura_LinkList.h"
#include "Streaming/Jobs/Asura_Streaming_JobQueue.h"

// Asura platform header-files...

// Game independent header-files...

// Game platform header-files...

// Tools header-files...


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job::Asura_Streaming_Job
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Streaming_Job::Asura_Streaming_Job (const char* szName) :
	m_pxChildQueue(NULL),
	m_szName(szName),
	m_uNumUpdates(0)
{
	// Nothing more to initialise...
}

/// End of function 'Asura_Streaming_Job::Asura_Streaming_Job'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job::~Asura_Streaming_Job
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Streaming_Job::~Asura_Streaming_Job ()
{
	// Ensure we've removed the child job-queue...
	delete(m_pxChildQueue);
}

/// End of function 'Asura_Streaming_Job::~Asura_Streaming_Job'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job::BeginUpdate
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_Job::BeginUpdate (void)
{
}

/// End of function 'Asura_Streaming_Job::BeginUpdate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job::Update
// Author       : Andy Tate
// Date         : 4-May-2006
//
// Description  : Handles the update of the streaming job.
//
// Parameters	: The time allowed for this update.
//
// Return value : "true" if we've finished updating, "false" if this job should
//				  still be handled next time.
////////////////////////////////////////////////////////////////////////////////

bool Asura_Streaming_Job::Update (float& fTimeAllowed)
{
	// Increase the number of updates...
	m_uNumUpdates++;

	// Do we have a child job-queue?
	if(m_pxChildQueue == NULL)
	{
		return(true);
	}
	else
	{
		return(m_pxChildQueue->Update(fTimeAllowed));
	}
}

/// End of function 'Asura_Streaming_Job::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job::EndUpdate
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_Job::EndUpdate (void)
{
}

/// End of function 'Asura_Streaming_Job::EndUpdate'. ///

#endif	// ASURA_USE_STREAMING

/// End of file. ///
