
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Streaming_Job_LoadZoneDataState.cpp
// Author : Andy Tate
// Date   : 4-May-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// RJT NOTE: Required for GCC builds as the file list generation doesn't account for VC project exclusion from build
#ifdef ASURA_USE_STREAMING

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Streaming/Jobs/Asura_Streaming_Job_StreamZoneDataState.h"

// Asura independent header-files...
#include "Core/Handlers/Asura_Handler_System.h"
#include "Streaming/Asura_Streaming_Zone.h"
#include "Streaming/Asura_Streaming_ZoneDataState.h"
#include "Streaming/Asura_Streaming_ZoneHandler.h"
#include "Streaming/Jobs/Asura_Streaming_JobQueue.h"
#include "Streaming/Jobs/Asura_Streaming_Jobs_FileAccess.h"

// Asura platform header-files...

// Game independent header-files...

// Game platform header-files...

// Tools header-files...


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_CreateZoneSubsystemHandlers::Asura_Streaming_Job_CreateZoneSubsystemHandlers
// Author       : Andy Tate
// Date         : 8-May-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Streaming_Job_CreateZoneSubsystemHandlers::Asura_Streaming_Job_CreateZoneSubsystemHandlers (Asura_Streaming_ZoneDataState& xZoneDataState,
																								  const bool bAllowEnvironment)
: Asura_Streaming_Job("Asura_Streaming_Job_CreateZoneSubsystemHandlers")
, m_bAllowEnvironment(bAllowEnvironment)
, m_xZoneDataState(xZoneDataState)
, m_uOldZoneHash(ASURA_HASH_ID_UNSET)
, m_uHandlerIndex(0)
{
	// Nothing more to initialise...
}

/// End of function 'Asura_Streaming_Job_CreateZoneSubsystemHandlers::Asura_Streaming_Job_CreateZoneSubsystemHandlers'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_CreateZoneSubsystemHandlers::BeginUpdate
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_Job_CreateZoneSubsystemHandlers::BeginUpdate (void)
{
	Asura_Streaming_Zone&					xZone			= m_xZoneDataState.GetZone();
	const Asura_Streaming_Zone::DataState	eDataState		= xZone.GetDataState(m_xZoneDataState);
	Asura_Streaming_ZoneHandler&			xZoneHandler	= xZone.GetHandler();
	const Asura_Streaming_Zone*				pxCurrentZone	= xZoneHandler.GetCurrentZone();

	// Store the previous current zone...
	m_uOldZoneHash	= (pxCurrentZone == NULL)
					? ASURA_HASH_ID_UNSET
					: pxCurrentZone->GetHashID();

	// Set the appropriate zone...
	xZoneHandler.SetCurrentZone(xZone.GetHashID(), eDataState, false);
}

/// End of function 'Asura_Streaming_Job_CreateZoneSubsystemHandlers::BeginUpdate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_CreateZoneSubsystemHandlers::Update
// Author       : Andy Tate
// Date         : 8-May-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Streaming_Job_CreateZoneSubsystemHandlers::Update (float& fTimeAllowed)
{
	bool	bCreating	= Asura_Handler_System::CreateZoneData(m_uHandlerIndex,
															   m_xZoneDataState.GetHandlerHash(),
															   m_xZoneDataState.GetDataIndex(),
															   m_xZoneDataState.GetName().GetString(),
															   m_xZoneDataState.GetParentHandlerHash(),
															   m_xZoneDataState.GetZone().GetHashID(),
															   m_bAllowEnvironment);

	// Call the parent update function...
	PARENT::Update(fTimeAllowed);

	// Have we finished updating?
	return(!bCreating);
}

/// End of function 'Asura_Streaming_Job_CreateZoneSubsystemHandlers::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_CreateZoneSubsystemHandlers::EndUpdate
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_Job_CreateZoneSubsystemHandlers::EndUpdate (void)
{
	Asura_Streaming_Zone&			xZone			= m_xZoneDataState.GetZone();
	Asura_Streaming_ZoneHandler&	xZoneHandler	= xZone.GetHandler();

	// Restore the previous zone...
	xZoneHandler.SetCurrentZone(m_uOldZoneHash);
}

/// End of function 'Asura_Streaming_Job_CreateZoneSubsystemHandlers::EndUpdate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_LoadZoneDataState::Asura_Streaming_Job_LoadZoneDataState
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Streaming_Job_LoadZoneDataState::Asura_Streaming_Job_LoadZoneDataState (Asura_Streaming_ZoneDataState& xZoneDataState)
: Asura_Streaming_Job("Asura_Streaming_Job_LoadZoneDataState")
, m_xZoneDataState(xZoneDataState)
, m_uOldZoneHash(ASURA_HASH_ID_UNSET)
{
	const Asura_String&	xFilename	= m_xZoneDataState.GetFilename();
	const char*			szFilename	= xFilename.GetString();

	// Create a child job-queue and fill it with appropriate jobs...
	m_pxChildQueue	= new Asura_Streaming_JobQueue;

	m_pxChildQueue->AddJobToEnd(new Asura_Streaming_Job_OpenStreamingCachedAsuraFile (szFilename, m_pxFileStream));

	m_pxChildQueue->AddJobToEnd(new Asura_Streaming_Job_ReadAsuraFileHeader (m_pxFileStream));
	m_pxChildQueue->AddJobToEnd(new Asura_Streaming_Job_ReadAsuraFileChunkData (m_pxFileStream));

	m_pxChildQueue->AddJobToEnd(new Asura_Streaming_Job_CloseStreamingAsuraFile (m_pxFileStream));
}

/// End of function 'Asura_Streaming_Job_LoadZoneDataState::Asura_Streaming_Job_LoadZoneDataState'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_LoadZoneDataState::BeginUpdate
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_Job_LoadZoneDataState::BeginUpdate (void)
{
	Asura_Streaming_Zone&					xZone			= m_xZoneDataState.GetZone();
	const Asura_Streaming_Zone::DataState	eDataState		= xZone.GetDataState(m_xZoneDataState);
	Asura_Streaming_ZoneHandler&			xZoneHandler	= xZone.GetHandler();
	const Asura_Streaming_Zone*				pxCurrentZone	= xZoneHandler.GetCurrentZone();

	// Store the previous current zone...
	m_uOldZoneHash	= (pxCurrentZone == NULL)
					? ASURA_HASH_ID_UNSET
					: pxCurrentZone->GetHashID();

	// Set the appropriate zone...
	xZoneHandler.SetCurrentZone(xZone.GetHashID(), eDataState, false);
}

/// End of function 'Asura_Streaming_Job_LoadZoneDataState::BeginUpdate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_LoadZoneDataState::EndUpdate
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_Job_LoadZoneDataState::EndUpdate (void)
{
	Asura_Streaming_Zone&			xZone			= m_xZoneDataState.GetZone();
	Asura_Streaming_ZoneHandler&	xZoneHandler	= xZone.GetHandler();

	// Restore the previous zone...
	xZoneHandler.SetCurrentZone(m_uOldZoneHash);
}

/// End of function 'Asura_Streaming_Job_LoadZoneDataState::EndUpdate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_LoadZoneDataStateComplete::Asura_Streaming_Job_LoadZoneDataStateComplete
// Author       : Andy Tate
// Date         : 5-May-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Streaming_Job_LoadZoneDataStateComplete::Asura_Streaming_Job_LoadZoneDataStateComplete (Asura_Streaming_ZoneDataState& xZoneDataState)
: Asura_Streaming_Job("Asura_Streaming_Job_LoadZoneDataStateComplete")
, m_xZoneDataState(xZoneDataState)
{
	// Nothing more to initialise...
}

/// End of function 'Asura_Streaming_Job_LoadZoneDataStateComplete::Asura_Streaming_Job_LoadZoneDataStateComplete'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_LoadZoneDataStateComplete::Update
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Streaming_Job_LoadZoneDataStateComplete::Update (float& fTimeAllowed)
{
	// Call the parent update function...
	PARENT::Update(fTimeAllowed);

	// Inform the system that we've finished loading...
	m_xZoneDataState.MarkLoadCompletion();
	return(true);
}

/// End of function 'Asura_Streaming_Job_LoadZoneDataStateComplete::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_DestroyZoneSubsystemHandlers::Asura_Streaming_Job_DestroyZoneSubsystemHandlers
// Author       : Andy Tate
// Date         : 8-May-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Streaming_Job_DestroyZoneSubsystemHandlers::Asura_Streaming_Job_DestroyZoneSubsystemHandlers (Asura_Streaming_ZoneDataState& xZoneDataState,
																									const bool bAllowEnvironment) 
: Asura_Streaming_Job("Asura_Streaming_Job_DestroyZoneSubsystemHandlers")
, m_xZoneDataState(xZoneDataState)
, m_bAllowEnvironment(bAllowEnvironment)
, m_uOldZoneHash(ASURA_HASH_ID_UNSET)
, m_uHandlerIndex(0)
{
	// Nothing more to initialise...
}

/// End of function 'Asura_Streaming_Job_DestroyZoneSubsystemHandlers::Asura_Streaming_Job_DestroyZoneSubsystemHandlers'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_DestroyZoneSubsystemHandlers::BeginUpdate
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_Job_DestroyZoneSubsystemHandlers::BeginUpdate (void)
{
	Asura_Streaming_Zone&					xZone			= m_xZoneDataState.GetZone();
	const Asura_Streaming_Zone::DataState	eDataState		= xZone.GetDataState(m_xZoneDataState);
	Asura_Streaming_ZoneHandler&			xZoneHandler	= xZone.GetHandler();
	const Asura_Streaming_Zone*				pxCurrentZone	= xZoneHandler.GetCurrentZone();

	// Store the previous current zone...
	m_uOldZoneHash	= (pxCurrentZone == NULL)
					? ASURA_HASH_ID_UNSET
					: pxCurrentZone->GetHashID();

	// Set the appropriate zone...
	xZoneHandler.SetCurrentZone(xZone.GetHashID(), eDataState, true);
}

/// End of function 'Asura_Streaming_Job_DestroyZoneSubsystemHandlers::BeginUpdate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_DestroyZoneSubsystemHandlers::Update
// Author       : Andy Tate
// Date         : 8-May-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Streaming_Job_DestroyZoneSubsystemHandlers::Update (float& fTimeAllowed)
{
	bool	bDestroying	= Asura_Handler_System::DestroyZoneData(m_uHandlerIndex,
																m_xZoneDataState.GetHandlerHash(),
																m_xZoneDataState.GetDataIndex(),
																m_bAllowEnvironment);

	// Call the parent update function...
	PARENT::Update(fTimeAllowed);

	// Have we finished destruction?
	return(!bDestroying);
}

/// End of function 'Asura_Streaming_Job_DestroyZoneSubsystemHandlers::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_DestroyZoneSubsystemHandlers::EndUpdate
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_Job_DestroyZoneSubsystemHandlers::EndUpdate (void)
{
	Asura_Streaming_Zone&			xZone			= m_xZoneDataState.GetZone();
	Asura_Streaming_ZoneHandler&	xZoneHandler	= xZone.GetHandler();

	// Restore the previous zone...
	xZoneHandler.SetCurrentZone(m_uOldZoneHash);
}

/// End of function 'Asura_Streaming_Job_DestroyZoneSubsystemHandlers::EndUpdate'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_DestroyZoneDataStateComplete::Asura_Streaming_Job_DestroyZoneDataStateComplete
// Author       : Andy Tate
// Date         : 5-May-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Streaming_Job_DestroyZoneDataStateComplete::Asura_Streaming_Job_DestroyZoneDataStateComplete (Asura_Streaming_ZoneDataState& xZoneDataState)
: Asura_Streaming_Job("Asura_Streaming_Job_DestroyZoneDataStateComplete")
, m_xZoneDataState(xZoneDataState)
{
	// Nothing more to initialise...
}

/// End of function 'Asura_Streaming_Job_DestroyZoneDataStateComplete::Asura_Streaming_Job_DestroyZoneDataStateComplete'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_Job_DestroyZoneDataStateComplete::Update
// Author       : Andy Tate
// Date         : 4-May-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Streaming_Job_DestroyZoneDataStateComplete::Update (float& fTimeAllowed)
{
	// Call the parent update function...
	PARENT::Update(fTimeAllowed);

	// Inform the system that we've finished destruction...
	m_xZoneDataState.MarkDestructionCompletion();
	return(true);
}

/// End of function 'Asura_Streaming_Job_DestroyZoneDataStateComplete::Update'. ///

#endif	// ASURA_USE_STREAMING

/// End of file. ///
