
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Teleport_Messages.h
// Author : Andy Tate
// Date   : 28-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_TELEPORT_MESSAGES_H__
#define __ASURA_TELEPORT_MESSAGES_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#if !defined(ASURA_DONT_USE_TELEPORTERS)
#include "Teleport/Asura_Teleport_System.h"
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////
class Asura_NetMsg;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Message_Teleport
// Author       : Andy Tate
// Date         : 28-Feb-2007
//
// Description  : Class to handle the message telling a client where its to be
//				  teleported to.
////////////////////////////////////////////////////////////////////////////////

class Asura_Message_Teleport
{
public:
	// Public functions...
	void	SetPosition	(const Asura_Vector_3& xPosition)	{ m_xPosition = xPosition; }
	void	GetPosition	(Asura_Vector_3& xPosition) const	{ xPosition = m_xPosition; }

	void	SetOrientation	(const Asura_Euler_3& xEuler)			{ m_xOrientation.FromEuler(xEuler); }
	void	SetOrientation	(const Asura_Matrix_3x3& xOrientation)	{ m_xOrientation.FromMatrix(xOrientation); }
	void	SetOrientation	(const Asura_Quat& xOrientation)		{ m_xOrientation = xOrientation; }
	void	GetOrientation	(Asura_Quat& xOrientation) const		{ xOrientation = m_xOrientation; }
	void	GetOrientation	(Asura_Matrix_3x3& xOrientation) const	{ m_xOrientation.ToMatrix(xOrientation); }

#ifdef ASURA_NETMSG_PACK_ALL_DATA
	void RegisterFields(Asura_NetMsg* pxNetMsg) const;
	static u_char GetNumFields() { return 2; }
#endif

private:
	// Private variables...
	Asura_Vector_3	m_xPosition;		// The position to teleport to.
	Asura_Quat		m_xOrientation;		// The orientation to be facing following teleport.

};   // End of class 'Asura_Message_Teleport'.



#if !defined(ASURA_DONT_USE_TELEPORTERS)

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Message_TeleportEntity
// Author       : Andy Tate
// Date         : 27-Feb-2007
//
// Description  : Class to handle the message telling a teleporter to handle
//				  the teleporting of a specified entity.
////////////////////////////////////////////////////////////////////////////////

class Asura_Message_TeleportEntity
{
public:
	// Public functions...
	void		SetGuidToTeleport	(const Asura_Guid uGuid)	{ m_uGuidToTeleport = uGuid; }
	Asura_Guid	GetGuidToTeleport	(void) const				{ return(m_uGuidToTeleport); }

private:
	// Private variables...
	Asura_Guid	m_uGuidToTeleport;		// The Guid of the entity to teleport.

};   // End of class 'Asura_Message_TeleportEntity'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Message_BeginPlayerTeleport
// Author       : Andy Tate
// Date         : 27-Feb-2007
//
// Description  : Class to handle the message requesting the player to begin
//				  being teleported.
////////////////////////////////////////////////////////////////////////////////

class Asura_Message_BeginPlayerTeleport
{
public:
	// Public functions...
	void Reset(void)
	{
		m_uDestinationDataPresentBits = 0;
	}

	void	SetFadeInDuration	(const float fDuration)	{ m_fFadeInDuration = fDuration; }
	float	GetFadeInDuration	(void) const			{ return(m_fFadeInDuration); }

	void	SetFadeOutDuration	(const float fDuration)	{ m_fFadeOutDuration = fDuration; }
	float	GetFadeOutDuration	(void) const			{ return(m_fFadeOutDuration); }

	void	SetFadeColour		(const u_int uColour)	{ m_uFadeColour = uColour; }
	u_int	GetFadeColour		(void) const			{ return(m_uFadeColour); }

	void SetDestinationData(const u_int uIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation)
	{
		m_uDestinationDataPresentBits	|= (1 << uIndex);
		m_axPositions[uIndex]			 = xPosition;
		m_axOrientations[uIndex].FromMatrix(xOrientation);
	}

	void SetDestinationData(const u_int uIndex, const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation)
	{
		m_uDestinationDataPresentBits	|= (1 << uIndex);
		m_axPositions[uIndex]			 = xPosition;
		m_axOrientations[uIndex]		 = xOrientation;
	}

	bool GetDestinationData(const u_int uIndex, Asura_Vector_3& xPosition, Asura_Quat& xOrientation) const
	{
		if((m_uDestinationDataPresentBits & (1 << uIndex)) == 0)
		{
			return(false);
		}
		else
		{
			xPosition		= m_axPositions[uIndex];
			xOrientation	= m_axOrientations[uIndex];
			return(true);
		}
	}

private:
	// Private variables...
	u_int			m_uDestinationDataPresentBits;
	Asura_Vector_3	m_axPositions[ASURA_TELEPORTER_ENTITY_MAX_DESTINATIONS];
	Asura_Quat		m_axOrientations[ASURA_TELEPORTER_ENTITY_MAX_DESTINATIONS];

	float			m_fFadeInDuration;		// The duration over which we fade-in after teleporting.
	float			m_fFadeOutDuration;		// The duration over which we fade-out before teleporting.
	u_int			m_uFadeColour;			// The colour to fade to.

};   // End of class 'Asura_Message_BeginPlayerTeleport'.

#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

#endif   // ifndef __ASURA_TELEPORT_MESSAGES_H__


/// End of file. ///
