
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Teleport_System.h
// Author : Andy Tate
// Date   : 27-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_TELEPORT_SYSTEM_H__
#define __ASURA_TELEPORT_SYSTEM_H__

#if !defined(ASURA_DONT_USE_TELEPORTERS)

// Ensure that we've got some destinations...
#if !defined(ASURA_TELEPORTER_ENTITY_MAX_DESTINATIONS)
#define ASURA_TELEPORTER_ENTITY_MAX_DESTINATIONS	1
#endif	// !defined(ASURA_TELEPORTER_ENTITY_MAX_DESTINATIONS)


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Maths.h"
#include "Network/Asura_Guid.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Message;
class Asura_Message_BeginPlayerTeleport;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

struct Asura_TeleportedPlayerData
{
	Asura_Guid		m_uPlayerGuid;		// The Guid of the player to be teleported.
	Asura_Vector_3	m_xPosition;		// The position to teleport to.
	Asura_Quat		m_xOrientation;		// The orientation to face once teleported.
};

////////////////////////////////////////////////////////////////////////////////
// Typedefs...
////////////////////////////////////////////////////////////////////////////////
typedef bool ProjectTeleportServerEntityFunction(const Asura_Guid uGuidToTeleport,
												 const Asura_Vector_3& xPosition,
												 const Asura_Quat& xOrientation);


////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Teleport_System
// Author       : Andy Tate
// Date         : 27-Feb-2007
//
// Description  : Static class used to handle control of entity teleporters.
////////////////////////////////////////////////////////////////////////////////

class Asura_Teleport_System
{
public:
	// Public functions...

	// Initialisation/shutdown functions...
	static void	Initialise	(void);
	static void	Reset		(void);
	static void	Shutdown	(void);

	// Player registration functions...
	static void	RegisterPlayerGuid	(const u_int uIndex, const Asura_Guid uGuid);

	// Teleporting functions...
	static void TeleportEntity	(const Asura_Guid uGuidToTeleport, const Asura_Guid uDestinationGuid);

	// Functions to be called once-per-frame...
	static void	Update	(void);
	static void	Render	(void);

	// Message handling functions...
	static bool HandleClientMessages	(const Asura_Message* pxMessage);

	static void SetProjectTeleportServerEntityFunction(ProjectTeleportServerEntityFunction* const pfnProjectTeleportServerEntity)
										{ s_pfnProjectTeleportServerEntity = pfnProjectTeleportServerEntity; }


	static const Asura_TeleportedPlayerData* GetPlayerTeleportData(const Asura_Guid uGuidTeleported);

protected:
	// Protected variables...

	// Protected functions...


private:
	// Private enumerated types...
	enum State
	{
		UNUSED,
		FADE_OUT,
#if defined(ASURA_USE_STREAMING)
		WAITING_FOR_STREAMING,
#endif	// defined(ASURA_USE_STREAMING)
		TELEPORTING,
		FADE_IN,

		MAX_STATES
	};

	// Private variables...
	static State		s_eCurrentState;		// The current state we're in.
	static State		s_eNewState;			// The state we're to change to.
	static u_int		s_uCurrentFadeColour;	// The current colour of the fade used to hide teleporting.
	static float		s_fTimeInState;			// The number of seconds we've been in this state.

	static float		s_fFadeInDuration;		// The duration of fade-in to use.
	static float		s_fFadeOutDuration;		// The duration of fade-out to use.
	static u_int		s_uTargetFadeColour;	// The target colour of the fade used to hide teleporting.

	static Asura_TeleportedPlayerData	s_axTeleportedPlayerData[ASURA_TELEPORTER_ENTITY_MAX_DESTINATIONS];	// Array of Guids for the player entities.

	// Private functions...
	Asura_Teleport_System  (void);
	Asura_Teleport_System  (const Asura_Teleport_System& xOther);
	~Asura_Teleport_System ();

	Asura_Teleport_System&	operator=	(const Asura_Teleport_System& xOther);

	static bool ChangeState				(const State eNewState);

	static void	BeginPlayerTeleport		(const Asura_Message_BeginPlayerTeleport* pxData);

	static void TeleportServerEntity	(const Asura_Guid uGuidToTeleport, const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation);
	static void TeleportClientEntity	(const Asura_Guid uGuidToTeleport, const Asura_Vector_3& xPosition, const Asura_Quat& xOrientation);

	static void	TeleportPlayers			(void);

	static ProjectTeleportServerEntityFunction* s_pfnProjectTeleportServerEntity;

};   // End of class 'Asura_Teleport_System'.


#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

#endif   // ifndef __ASURA_TELEPORT_SYSTEM_H__


/// End of file. ///
