//////////////////////////////////////////////////////////////////////////////
//
// Asura_SpeedTest_MatrixVector3.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     speed test instances for Matrix3x3 and Vector3
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_UNITTEST

#include "UnitTest/Asura_SpeedTest.h"

#include "Core/Asura_Maths.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int uNUM_TESTS = 100000;
static const u_int uNUM_DATA  = 4;
static const u_int uDATA_MASK = uNUM_DATA - 1;

// G L O B A L S /////////////////////////////////////////////////////////////

static Asura_Vector_3 g_ax3VectorInput[uNUM_DATA];
static Asura_Matrix_3x3 g_ax3MatrixInput[uNUM_DATA];
static Asura_Quat g_axQuatInput[uNUM_DATA];

static Asura_Vector_3 g_ax3VectorOutput[uNUM_DATA];
static Asura_Matrix_3x3 g_ax3MatrixOutput[uNUM_DATA];


ASURA_SPEEDTEST_INITIALISER(MatrixVector3)
{
	Asura_Vector_3 xTemp;

  	for( u_int u = 0; u < uNUM_DATA; ++u )
  	{
		g_ax3VectorInput[u].RandomDirection();

        xTemp.RandomDirection();
        Asura_Maths::OrientationFromDirection( g_ax3MatrixInput[u], xTemp );	    

        g_axQuatInput[u].FromMatrix( g_ax3MatrixInput[u] );
  	}
}


ASURA_SPEEDTEST(MatrixVector3, M3xM3)
{
	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
  	{
  		g_ax3MatrixOutput[u & uDATA_MASK] = g_ax3MatrixInput[u & uDATA_MASK] * g_ax3MatrixInput[(u + 1) & uDATA_MASK];
  	}
}

ASURA_SPEEDTEST(MatrixVector3, M3xV3)
{
	for( u_int u = 0; u < uNUM_TESTS; ++u )
  	{
  		g_ax3VectorOutput[u & uDATA_MASK] = g_ax3VectorInput[u & uDATA_MASK] * g_ax3MatrixInput[u & uDATA_MASK];
  	}
}

ASURA_SPEEDTEST(MatrixVector3, M3xV3x4)
{
	for( u_int u = 0; u < uNUM_TESTS; u += 4)
  	{
  		g_ax3VectorOutput[(u + 0) & uDATA_MASK] = g_ax3VectorInput[(u + 0) & uDATA_MASK] * g_ax3MatrixInput[(u + 0) & uDATA_MASK];
  		g_ax3VectorOutput[(u + 1) & uDATA_MASK] = g_ax3VectorInput[(u + 1) & uDATA_MASK] * g_ax3MatrixInput[(u + 1) & uDATA_MASK];
  		g_ax3VectorOutput[(u + 2) & uDATA_MASK] = g_ax3VectorInput[(u + 2) & uDATA_MASK] * g_ax3MatrixInput[(u + 2) & uDATA_MASK];
  		g_ax3VectorOutput[(u + 3) & uDATA_MASK] = g_ax3VectorInput[(u + 3) & uDATA_MASK] * g_ax3MatrixInput[(u + 3) & uDATA_MASK];
  	}
}

ASURA_SPEEDTEST(MatrixVector3, M3xV3_V3AddSub_Random)
{
    Asura_Vector_3 xTmpVec;
	for( u_int u = 0; u < uNUM_TESTS; ++u )
  	{
        u_int uIdx0 = u & uDATA_MASK;
        u_int uIdx1 = Asura_Random::GetIntU() & uDATA_MASK;
        u_int uIdx2 = Asura_Random::GetIntU() & uDATA_MASK;
        
        xTmpVec = g_ax3VectorInput[uIdx0] + g_ax3VectorInput[uIdx1] * Asura_Random::GetFloat();
  		g_ax3VectorOutput[uIdx0] = g_ax3VectorInput[uIdx0] * g_ax3MatrixInput[uIdx0] + g_ax3VectorInput[uIdx2];

        xTmpVec -= g_ax3VectorInput[uIdx2];
        g_ax3VectorOutput[uIdx1] = xTmpVec * g_ax3MatrixInput[uIdx1] + g_ax3VectorInput[uIdx0] * 4.0f;
    }
}

ASURA_SPEEDTEST(MatrixVector3, DynamicsStyleCode)
{
    Asura_Vector_3 xResult;

    for( u_int u = 0; u < uNUM_TESTS; ++u )
    {
        xResult.RandomDirection();
        xResult *= Asura_Random::GetFloat( 0.0f, 100.0f );

        u_int uIdx0 = u & uDATA_MASK;
        u_int uIdx1 = Asura_Random::GetIntU() & uDATA_MASK;
        u_int uIdx2 = Asura_Random::GetIntU() & uDATA_MASK;

        // collision - elasticity 
        {
            float magOfPerpImp = (g_ax3VectorInput[uIdx0] * g_ax3VectorInput[uIdx1]) * (1.0f + g_ax3VectorInput[uIdx0].y);
            xResult -= g_ax3VectorInput[uIdx2] * magOfPerpImp;
        }

        // resolve player's movement vector against the collision plane
        {
            Asura_Vector_3 v = g_ax3VectorOutput[uIdx0];

            // KJL 17:37:52 28/02/00 - Exciting new system to make things more slidey
            if (g_ax3VectorInput[uIdx1].x!=0 || g_ax3VectorInput[uIdx1].y!=0 || g_ax3VectorInput[uIdx1].z!=0)
            {
                float perp = g_ax3VectorInput[uIdx2] * g_ax3VectorInput[uIdx1];
        
                if( v != g_ax3VectorInput[uIdx1] * perp )
                {
                    v -= g_ax3VectorInput[uIdx1] * perp;
                    v.Normalise();
                }
            }
    
            // awkward problem here to do with non-exact normals
            float magOfPerpVel = 1.01f*(v * xResult);
            xResult -= v * magOfPerpVel;
        }

        xResult = xResult * g_ax3MatrixInput[uIdx1];

        g_ax3VectorOutput[uIdx0] = xResult;
    }
}

ASURA_SPEEDTEST(MatrixVector3, RecurseBones)
{
    Asura_Matrix_3x3 xOriMat;

    for( u_int u = 0; u < uNUM_TESTS; ++u )
    {
        u_int uIdx0 = u & uDATA_MASK;

		const Asura_Quat&     xOriQuat = g_axQuatInput[uIdx0];
		const Asura_Vector_3& xPos     = g_ax3VectorInput[uIdx0];	

		xOriQuat.ToMatrix( xOriMat );

		if( uIdx0 )
		{
			const int iParentIndex = Asura_Random::GetIntU() & uDATA_MASK;
		
			g_ax3VectorOutput[uIdx0] = xPos * g_ax3MatrixInput[iParentIndex] + g_ax3VectorInput[iParentIndex];
			g_ax3MatrixOutput[uIdx0] = g_ax3MatrixInput[iParentIndex] * xOriMat;
		}
		else
		{
			g_ax3VectorOutput[uIdx0] = xPos;
			g_ax3MatrixOutput[uIdx0] = xOriMat;
		}
	}
}


ASURA_SPEEDTEST_ENDOFFILE(MatrixVector3)

#endif // #ifdef ASURA_SUPPORT_UNITTEST
