//////////////////////////////////////////////////////////////////////////////
//
// Asura_SpeedTest_MatrixVector4.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     speed test instances for Aligned Matrix3x3 and Aligned Vector3
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_UNITTEST

#include "UnitTest/Asura_SpeedTest.h"

#include "Core/Asura_Maths.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int uNUM_TESTS = 100000;
static const u_int uNUM_DATA  = 4;
static const u_int uDATA_MASK = uNUM_DATA - 1;

// G L O B A L S /////////////////////////////////////////////////////////////

static Asura_AVector3 g_ax4VectorInput[uNUM_DATA];
static Asura_AMatrix3x3 g_ax4MatrixInput[uNUM_DATA];
static Asura_Quat g_axQuatInput[uNUM_DATA];

static Asura_AVector3 g_ax4VectorOutput[uNUM_DATA];
static Asura_AMatrix3x3 g_ax4MatrixOutput[uNUM_DATA];


ASURA_SPEEDTEST_INITIALISER(MatrixVector4)
{
	Asura_Vector_3 xTempVec3;
	Asura_Matrix_3x3 xTempMat3;

	// Need to initialise the random number generator before we can do these tests!
	Asura_Random::Initialise();

  	for( u_int u = 0; u < uNUM_DATA; ++u )
  	{
        xTempVec3.RandomDirection();
		g_ax4VectorInput[u] = xTempVec3;

		xTempVec3.RandomDirection();
        Asura_Maths::OrientationFromDirection( xTempMat3, xTempVec3 );
		g_ax4MatrixInput[u] = Asura_AMatrix3x3( Asura_AVector3(xTempMat3[0]), 
			Asura_AVector3(xTempMat3[1]), Asura_AVector3(xTempMat3[2]) );

        g_axQuatInput[u].FromMatrix( xTempMat3 );
  	}

	// fix warning
	g_ax4MatrixOutput[0] = g_ax4MatrixOutput[1];
}


ASURA_SPEEDTEST(MatrixVector4, M4xM4)
{
	for( u_int u = 0; u < uNUM_TESTS - 1; ++u )
  	{
  		g_ax4MatrixOutput[u & uDATA_MASK] = g_ax4MatrixInput[u & uDATA_MASK] * g_ax4MatrixInput[(u + 1) & uDATA_MASK];
  	}
}

ASURA_SPEEDTEST(MatrixVector4, M4xV4)
{
	for( u_int u = 0; u < uNUM_TESTS; ++u )
  	{
  		g_ax4VectorOutput[u & uDATA_MASK] = g_ax4VectorInput[u & uDATA_MASK] * g_ax4MatrixInput[u & uDATA_MASK];
  	}
}

ASURA_SPEEDTEST(MatrixVector4, M4xV4x4)
{
	for( u_int u = 0; u < uNUM_TESTS; u += 4)
  	{
  		g_ax4VectorOutput[(u + 0) & uDATA_MASK] = g_ax4VectorInput[(u + 0) & uDATA_MASK] * g_ax4MatrixInput[(u + 0) & uDATA_MASK];
  		g_ax4VectorOutput[(u + 1) & uDATA_MASK] = g_ax4VectorInput[(u + 1) & uDATA_MASK] * g_ax4MatrixInput[(u + 1) & uDATA_MASK];
  		g_ax4VectorOutput[(u + 2) & uDATA_MASK] = g_ax4VectorInput[(u + 2) & uDATA_MASK] * g_ax4MatrixInput[(u + 2) & uDATA_MASK];
  		g_ax4VectorOutput[(u + 3) & uDATA_MASK] = g_ax4VectorInput[(u + 3) & uDATA_MASK] * g_ax4MatrixInput[(u + 3) & uDATA_MASK];
  	}
}

ASURA_SPEEDTEST(MatrixVector4, M4xV4_V4AddSub_Random)
{
    Asura_AVector3 xTmpVec;
	for( u_int u = 0; u < uNUM_TESTS; ++u )
  	{
        u_int uIdx0 = u & uDATA_MASK;
        u_int uIdx1 = Asura_Random::GetIntU() & uDATA_MASK;
        u_int uIdx2 = Asura_Random::GetIntU() & uDATA_MASK;
        
        xTmpVec = g_ax4VectorInput[uIdx0] + g_ax4VectorInput[uIdx1] * Asura_Random::GetFloat();
  		g_ax4VectorOutput[uIdx0] = g_ax4VectorInput[uIdx0] * g_ax4MatrixInput[uIdx0] + g_ax4VectorInput[uIdx2];

        xTmpVec -= g_ax4VectorInput[uIdx2];
        g_ax4VectorOutput[uIdx1] = xTmpVec * g_ax4MatrixInput[uIdx1] + g_ax4VectorInput[uIdx0] * 4.0f;
    }
}

ASURA_SPEEDTEST(MatrixVector4, DynamicsStyleCode)
{
    Asura_AVector3 xResult;

    for( u_int u = 0; u < uNUM_TESTS; ++u )
    {
		Asura_Vector_3 xTemp;
        xTemp.RandomDirection();
        xTemp *= Asura_Random::GetFloat( 0.0f, 100.0f );
		xResult = xTemp;

        u_int uIdx0 = u & uDATA_MASK;
        u_int uIdx1 = Asura_Random::GetIntU() & uDATA_MASK;
        u_int uIdx2 = Asura_Random::GetIntU() & uDATA_MASK;

        // collision - elasticity 
        {
            float magOfPerpImp = (g_ax4VectorInput[uIdx0] * g_ax4VectorInput[uIdx1]) * (1.0f + g_ax4VectorInput[uIdx0].GetY());
            xResult -= g_ax4VectorInput[uIdx2] * magOfPerpImp;
        }

        // resolve player's movement vector against the collision plane
        {
            Asura_AVector3 v = g_ax4VectorOutput[uIdx0];

            // KJL 17:37:52 28/02/00 - Exciting new system to make things more slidey
            //if (g_ax4VectorInput[uIdx1].x!=0 || g_ax4VectorInput[uIdx1].y!=0 || g_ax4VectorInput[uIdx1].z!=0)
			if (g_ax4VectorInput[uIdx1].GetX()!=0 || g_ax4VectorInput[uIdx1].GetY()!=0 || g_ax4VectorInput[uIdx1].GetZ()!=0)
            {
                float perp = g_ax4VectorInput[uIdx2] * g_ax4VectorInput[uIdx1];
        
                if( v != g_ax4VectorInput[uIdx1] * perp )
                {
                    v -= g_ax4VectorInput[uIdx1] * perp;
                    v.Normalise();
                }
            }
    
            // awkward problem here to do with non-exact normals
            float magOfPerpVel = 1.01f*(v * xResult);
            xResult -= v * magOfPerpVel;
        }

        xResult = xResult * g_ax4MatrixInput[uIdx1];

        g_ax4VectorOutput[uIdx0] = xResult;
    }
}

ASURA_SPEEDTEST(MatrixVector4, RecurseBones)
{
    Asura_AMatrix3x3 xOriMat;

    for( u_int u = 0; u < uNUM_TESTS; ++u )
    {
        u_int uIdx0 = u & uDATA_MASK;

		const Asura_Quat&     xOriQuat = g_axQuatInput[uIdx0];
		const Asura_AVector3& xPos     = g_ax4VectorInput[uIdx0];	

		xOriQuat.ToMatrix( xOriMat );

		if( uIdx0 )
		{
			const int iParentIndex = Asura_Random::GetIntU() & uDATA_MASK;
		
			g_ax4VectorOutput[uIdx0] = xPos * g_ax4MatrixInput[iParentIndex] + g_ax4VectorInput[iParentIndex];
			g_ax4MatrixOutput[uIdx0] = g_ax4MatrixInput[iParentIndex] * xOriMat;
		}
		else
		{
			g_ax4VectorOutput[uIdx0] = xPos;
			g_ax4MatrixOutput[uIdx0] = xOriMat;
		}
	}
}


ASURA_SPEEDTEST_ENDOFFILE(MatrixVector4)

#endif // #ifdef ASURA_SUPPORT_UNITTEST
