//////////////////////////////////////////////////////////////////////////////
//
// Asura_UnitTest_64Bit.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     64Bit testing code; Asura_UnitTest.cpp
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_UNITTEST

#include "Core/Asura_Core.h"
#include "Core/Asura_Maths.h"
#include "Debugging/Asura_Assert.h"

#include "UnitTest/Asura_UnitTest.h"

//SizeOf /////////////////////////////////////////////////////////////////////

ASURA_UNITTEST(SizeTests, SixtyFourBit)
{
	int64 lTestInt;
	uint64 ulTestUInt;
	ASURA_UNITTEST_CHECK_EQUAL(8,sizeof(lTestInt));
	ASURA_UNITTEST_CHECK_EQUAL(8,sizeof(ulTestUInt));

	// Try an empty and max int64/uint64

	ASURA_UNITTEST_CHECK_EQUAL(8,sizeof(0LL));
	ASURA_UNITTEST_CHECK_EQUAL(8,sizeof(0ULL));

	ASURA_UNITTEST_CHECK_EQUAL(8,sizeof(0LL + (0ULL - 1)/2));
	ASURA_UNITTEST_CHECK_EQUAL(8,sizeof(0ULL - 1));
}

// Overflow and Wrap Around ////////////////////////////////////////////////////////////////////

ASURA_UNITTEST(OverflowTests, SixtyFourBit)
{
	int64 lZero = 0LL;
	int64 lOne = 1LL;
	uint64 ulZero = 0ULL;
	uint64 ulOne = 1ULL;
	uint64 ulMax = 18446744073709551615ULL;

	ASURA_UNITTEST_CHECK((ulZero - ulOne) == ulMax);
	ASURA_UNITTEST_CHECK(static_cast<int64>(lZero - ulMax) == lOne);
}

// The following tests make sure that the Asura_Maths template functions work
// as expected with int64 and uint64

// Min/Max ///////////////////////////////////////////////////////////////////

ASURA_UNITTEST(Min_Max_Tests, SixtyFourBit)
{
	ASURA_UNITTEST_CHECK( 0LL == Asura_Maths::Min( 0LL, 1LL ) );
	ASURA_UNITTEST_CHECK( 0LL == Asura_Maths::Min( 1LL, 0LL ) );
	ASURA_UNITTEST_CHECK( 0LL == Asura_Maths::Min( 0LL, 0LL ) );
	ASURA_UNITTEST_CHECK( 1LL == Asura_Maths::Min( 1LL, 1LL ) );
	ASURA_UNITTEST_CHECK( 1LL == Asura_Maths::Min( 1LL, 2LL ) );
	ASURA_UNITTEST_CHECK( 1LL == Asura_Maths::Min( 2LL, 1LL ) );
	ASURA_UNITTEST_CHECK( 998LL == Asura_Maths::Min( 998LL, 999LL ) );
	ASURA_UNITTEST_CHECK( -1LL == Asura_Maths::Min( 0LL, -1LL ) );
	ASURA_UNITTEST_CHECK( -1LL == Asura_Maths::Min( -1LL, 0LL ) );
	ASURA_UNITTEST_CHECK( -2LL == Asura_Maths::Min( -2LL, -1LL ) );
	ASURA_UNITTEST_CHECK( -2LL == Asura_Maths::Min( -1LL, -2LL ) );
	ASURA_UNITTEST_CHECK( -999LL == Asura_Maths::Min( -998LL, -999LL ) );

	ASURA_UNITTEST_CHECK( 1LL == Asura_Maths::Max( 0LL, 1LL ) );
	ASURA_UNITTEST_CHECK( 1LL == Asura_Maths::Max( 1LL, 0LL ) );
	ASURA_UNITTEST_CHECK( 0LL == Asura_Maths::Max( 0LL, 0LL ) );
	ASURA_UNITTEST_CHECK( 1LL == Asura_Maths::Max( 1LL, 1LL ) );
	ASURA_UNITTEST_CHECK( 2LL == Asura_Maths::Max( 1LL, 2LL ) );
	ASURA_UNITTEST_CHECK( 2LL == Asura_Maths::Max( 2LL, 1LL ) );
	ASURA_UNITTEST_CHECK( 999LL == Asura_Maths::Max( 998LL, 999LL ) );
	ASURA_UNITTEST_CHECK( 0LL == Asura_Maths::Max( 0LL, -1LL ) );
	ASURA_UNITTEST_CHECK( 0LL == Asura_Maths::Max( -1LL, 0LL ) );
	ASURA_UNITTEST_CHECK( -1LL == Asura_Maths::Max( -2LL, -1LL ) );
	ASURA_UNITTEST_CHECK( -1LL == Asura_Maths::Max( -1LL, -2LL ) );
	ASURA_UNITTEST_CHECK( -998LL == Asura_Maths::Max( -998LL, -999LL ) );

	ASURA_UNITTEST_CHECK( 0ULL == Asura_Maths::Min( 0ULL, 1ULL ) );
	ASURA_UNITTEST_CHECK( 0ULL == Asura_Maths::Min( 1ULL, 0ULL ) );
	ASURA_UNITTEST_CHECK( 0ULL == Asura_Maths::Min( 0ULL, 0ULL ) );
	ASURA_UNITTEST_CHECK( 1ULL == Asura_Maths::Min( 1ULL, 1ULL ) );
	ASURA_UNITTEST_CHECK( 1ULL == Asura_Maths::Min( 1ULL, 2ULL ) );
	ASURA_UNITTEST_CHECK( 1ULL == Asura_Maths::Min( 2ULL, 1ULL ) );
	ASURA_UNITTEST_CHECK( 998ULL == Asura_Maths::Min( 998ULL, 999ULL ) );
	ASURA_UNITTEST_CHECK( 0ULL == Asura_Maths::Min( 0ULL, (0ULL - 1) ));

	ASURA_UNITTEST_CHECK( 1ULL == Asura_Maths::Max( 0ULL, 1ULL ) );
	ASURA_UNITTEST_CHECK( 1ULL == Asura_Maths::Max( 1ULL, 0ULL ) );
	ASURA_UNITTEST_CHECK( 0ULL == Asura_Maths::Max( 0ULL, 0ULL ) );
	ASURA_UNITTEST_CHECK( 1ULL == Asura_Maths::Max( 1ULL, 1ULL ) );
	ASURA_UNITTEST_CHECK( 2ULL == Asura_Maths::Max( 1ULL, 2ULL ) );
	ASURA_UNITTEST_CHECK( 2ULL == Asura_Maths::Max( 2ULL, 1ULL ) );
	ASURA_UNITTEST_CHECK( 999ULL == Asura_Maths::Max( 998ULL, 999ULL ) );
	ASURA_UNITTEST_CHECK( (0ULL - 1) == Asura_Maths::Max( 0ULL, (0ULL - 1) ));
}

// Clamp to Range ////////////////////////////////////////////////////////////

ASURA_UNITTEST( ClampToRange, SixtyFourBit )
{
	ASURA_UNITTEST_CHECK( 3 == Asura_Maths::ClampToRange<int64>( 1LL ,3LL ,5LL) );
	ASURA_UNITTEST_CHECK( 20 == Asura_Maths::ClampToRange<int64>(192LL, 12LL, 20LL) );
	ASURA_UNITTEST_CHECK( 7 == Asura_Maths::ClampToRange<int64>(7LL, 1LL, 10LL) );
	ASURA_UNITTEST_CHECK( 3 == Asura_Maths::ClampToRange<uint64>( 1ULL ,3ULL ,5ULL) );
	ASURA_UNITTEST_CHECK( 20 == Asura_Maths::ClampToRange<uint64>(192ULL, 12ULL, 20ULL) );
	ASURA_UNITTEST_CHECK( 7 == Asura_Maths::ClampToRange<uint64>(7ULL, 1ULL, 10ULL) );
}

// Sort ///////////////////////////////////////////////////////////////////////

ASURA_UNITTEST( Sort, SixtyFourBit )
{
	int64 lA, lB;

	lA = 10;
	lB = -3;
	Asura_Maths::Sort( lA, lB );
	ASURA_UNITTEST_CHECK( lA == -3 );
	ASURA_UNITTEST_CHECK( lB == 10 );

	lA = 5;
	lB = 5;
	Asura_Maths::Sort( lA, lB );
	ASURA_UNITTEST_CHECK( lA == 5 );
	ASURA_UNITTEST_CHECK( lB == 5 );

	uint64 ulA, ulB;

	ulA = 10;
	ulB = 3;
	Asura_Maths::Sort( ulA, ulB );
	ASURA_UNITTEST_CHECK( ulA == 3 );
	ASURA_UNITTEST_CHECK( ulB == 10 );

	ulA = 5;
	ulB = 5;
	Asura_Maths::Sort( ulA, ulB );
	ASURA_UNITTEST_CHECK( ulA == 5 );
	ASURA_UNITTEST_CHECK( ulB == 5 );
}

#endif
