
//////////////////////////////////////////////////////////////////////////////
//
// Asura_UnitTest_FixedArray.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Unit tests for Asura_FixedArray class
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "Core/Asura_Core.h"
#include "Core/Memory/Asura_Mem.h"
#include "Debugging/Asura_Assert.h"

#include "Core/Asura_FixedArray.h"

#include "UnitTest/Asura_UnitTest.h"

#ifdef ASURA_SUPPORT_UNITTEST

ASURA_UNITTEST(GeneralTests, FixedArray)
{
	Asura_FixedArray<int,4> xArray;
	xArray[0] = 10;
	xArray[1] = 16;
	xArray[2] = 32;
	xArray[3] = 64;
    ASURA_UNITTEST_CHECK_EQUAL( 4, xArray.GetSize() );
    ASURA_UNITTEST_CHECK_EQUAL( 64, xArray[3] );
    ASURA_UNITTEST_CHECK_EQUAL( 32, xArray[2] );
    ASURA_UNITTEST_CHECK_EQUAL( 16, xArray[1] );
    ASURA_UNITTEST_CHECK_EQUAL( 10, xArray[0] );
    ASURA_UNITTEST_CHECK_EQUAL( 64, xArray[3u] );
    ASURA_UNITTEST_CHECK_EQUAL( 32, xArray[2u] );
    ASURA_UNITTEST_CHECK_EQUAL( 16, xArray[1u] );
    ASURA_UNITTEST_CHECK_EQUAL( 10, xArray[0u] );

	const int iTest = xArray[3];
	ASURA_UNITTEST_CHECK_EQUAL( 64, iTest );

	Asura_FixedArrayOfBools<4> xBoolArray1;
	ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1.GetSize(), 1 );
	ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1.GetNumberOfBools(), 4 );
	ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1.GetSize(), static_cast<u_int>(sizeof(xBoolArray1)) );

	Asura_FixedArrayOfBools<8> xBoolArray2;
	ASURA_UNITTEST_CHECK_EQUAL( xBoolArray2.GetSize(), 1 );
	ASURA_UNITTEST_CHECK_EQUAL( xBoolArray2.GetNumberOfBools(), 8 );
	ASURA_UNITTEST_CHECK_EQUAL( xBoolArray2.GetSize(), static_cast<u_int>(sizeof(xBoolArray2)) );

	Asura_FixedArrayOfBools<100> xBoolArray3;
	ASURA_UNITTEST_CHECK_EQUAL( xBoolArray3.GetSize(), 13 );
	ASURA_UNITTEST_CHECK_EQUAL( xBoolArray3.GetNumberOfBools(), 100 );
	ASURA_UNITTEST_CHECK_EQUAL( xBoolArray3.GetSize(), static_cast<u_int>(sizeof(xBoolArray3)) );

	Asura_Mem::Set(xBoolArray1, 0, xBoolArray1.GetSize());

	for (u_int i = 0; i < xBoolArray1.GetNumberOfBools(); ++i)
	{
		ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1[i] = (i & 1) == 1, (i & 1) == 1);
	}
	for (u_int i = 0; i < xBoolArray1.GetNumberOfBools(); ++i)
	{
		ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1[i], (i & 1) == 1);
	}
	for (u_int i = 0; i < xBoolArray1.GetNumberOfBools(); ++i)
	{
		ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1[i] ^= (i & 2) == 2, ((i & 2) == 2) ^ ((i & 1) == 1) );
	}
	for (u_int i = 0; i < xBoolArray1.GetNumberOfBools(); ++i)
	{
		ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1[i], ((i & 2) == 2) ^ ((i & 1) == 1) );
	}

	for (u_int i = 0; i < xBoolArray1.GetNumberOfBools(); ++i)
	{
		xBoolArray2[i] = xBoolArray1[i] = (i & 1) == 1;
		ASURA_UNITTEST_CHECK_EQUAL( xBoolArray2[i], xBoolArray1[i]);
		ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1[i] &= ((i & 2) == 2), ((i & 1) == 1) & ((i & 2) == 2));
	}

	for (u_int i = 0; i < xBoolArray1.GetNumberOfBools(); ++i)
	{
		ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1[i], ((i & 1) == 1) & ((i & 2) == 2));
	}

	for (u_int i = 0; i < xBoolArray1.GetNumberOfBools(); ++i)
	{
		xBoolArray1[i] = xBoolArray2[i];
		ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1[i] |= ((i & 2) == 2), ((i & 1) == 1) | ((i & 2) == 2));
	}
	for (u_int i = 0; i < xBoolArray1.GetNumberOfBools(); ++i)
	{
		ASURA_UNITTEST_CHECK_EQUAL( xBoolArray1[i], ((i & 1) == 1) | ((i & 2) == 2));
	}
	for (u_int i = 0; i < xBoolArray3.GetNumberOfBools(); ++i)
	{
		xBoolArray3.Set(i, ((i&1)!=0) ^ ((i&2)!=0) ^ ((i&4)!=0) ^ ((i&8)!=0) ^ ((i&16)!=0) ^ ((i&32)!=0) ^ ((i&64)!=0));
	}
	for (u_int i = 0; i < xBoolArray3.GetNumberOfBools(); ++i)
	{
		ASURA_UNITTEST_CHECK_EQUAL( xBoolArray3.Get(i), ((i&1)!=0) ^ ((i&2)!=0) ^ ((i&4)!=0) ^ ((i&8)!=0) ^ ((i&16)!=0) ^ ((i&32)!=0) ^ ((i&64)!=0));
	}


//	To test assert's and array overruns
//	xArray[10] = 10;
//	xBoolArray1[4] = true;
}

#endif // #ifdef ASURA_SUPPORT_UNITTEST
