//////////////////////////////////////////////////////////////////////////////
//
// Asura_UnitTest_Maths.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     Unit tests for classes such as Asura_LinkList, AsuraMempool etc.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "UnitTest/Asura_UnitTest.h"

#include "Core/Asura_LinkList.h"
#include "Core/Asura_Maths.h"

#ifdef ASURA_SUPPORT_UNITTEST

static int IncreasingSort( int const & iOne, int const & iTwo )
{
	if ( iOne < iTwo )
	{
		return -1;
	}
	else if ( iTwo < iOne )
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

ASURA_UNITTEST(GeneralTests, LinkList)
{
    // create 1,2,3,4 using both functions that can insert at the back
	Asura_LinkList<int> xList;
	ASURA_UNITTEST_CHECK_EQUAL(0, xList.Size());

	xList.Push_Back(1); 

	ASURA_UNITTEST_CHECK_EQUAL(1, xList.Size());
	ASURA_UNITTEST_CHECK(xList.Front() == xList.Back());

    xList.Push_Back(2); 

	ASURA_UNITTEST_CHECK_EQUAL(2, xList.Size());
	ASURA_UNITTEST_CHECK(2 == xList.Back()->Data());
	ASURA_UNITTEST_CHECK(1 == xList.Front()->Data());

    Asura_ListElt<int>* pxElement = xList.Find(2);
	ASURA_UNITTEST_CHECK(pxElement);

	xList.Insert_After(pxElement, 3);
	ASURA_UNITTEST_CHECK_EQUAL(3, xList.Size());
	ASURA_UNITTEST_CHECK(3 == xList.Back()->Data());
	ASURA_UNITTEST_CHECK(1 == xList.Front()->Data());
	
    xList.Push_Back(4); 
	ASURA_UNITTEST_CHECK_EQUAL(4, xList.Size());
	ASURA_UNITTEST_CHECK(4 == xList.Back()->Data());
	ASURA_UNITTEST_CHECK(1 == xList.Front()->Data());

	// clear list and check it's empty
	xList.Clear();

	ASURA_UNITTEST_CHECK_EQUAL(0, xList.Size());

    pxElement = xList.Find(2);
	ASURA_UNITTEST_CHECK(!pxElement);

    // create 1,2,3,4 using both functions that can insert at the front
	xList.Push_Front(4);

	ASURA_UNITTEST_CHECK_EQUAL(1, xList.Size());
	ASURA_UNITTEST_CHECK(xList.Front() == xList.Back());

	xList.Push_Front(3);
	ASURA_UNITTEST_CHECK_EQUAL(2, xList.Size());
	ASURA_UNITTEST_CHECK(4 == xList.Back()->Data());
	ASURA_UNITTEST_CHECK(3 == xList.Front()->Data());

    pxElement = xList.Find(3);
	ASURA_UNITTEST_CHECK(pxElement);

	xList.Insert_Before(pxElement, 2);
	ASURA_UNITTEST_CHECK_EQUAL(3, xList.Size());
	ASURA_UNITTEST_CHECK(4 == xList.Back()->Data());
	ASURA_UNITTEST_CHECK(2 == xList.Front()->Data());
	
    xList.Push_Front(1); 
	ASURA_UNITTEST_CHECK_EQUAL(4, xList.Size());
	ASURA_UNITTEST_CHECK(4 == xList.Back()->Data());
	ASURA_UNITTEST_CHECK(1 == xList.Front()->Data());

    // create 1,2,3,4 using mixture of functions, including insertion to middle of list
	xList.Clear();
	ASURA_UNITTEST_CHECK_EQUAL(0, xList.Size());

	xList.Push_Front(1); 
	xList.Push_Back(4); 

    pxElement = xList.Find(1);
	ASURA_UNITTEST_CHECK(pxElement);

	xList.Insert_After(pxElement, 3);

    pxElement = xList.Find(3);
	ASURA_UNITTEST_CHECK(pxElement);

	xList.Insert_Before(pxElement, 2);

	ASURA_UNITTEST_CHECK_EQUAL(1, xList.Front()->Data());
	ASURA_UNITTEST_CHECK_EQUAL(2, xList.Front()->Next()->Data());
	ASURA_UNITTEST_CHECK_EQUAL(3, xList.Front()->Next()->Next()->Data());
	ASURA_UNITTEST_CHECK_EQUAL(4, xList.Front()->Next()->Next()->Next()->Data());
	ASURA_UNITTEST_CHECK_EQUAL(4, xList.Back()->Data());
	ASURA_UNITTEST_CHECK_EQUAL(3, xList.Back()->Prev()->Data());
	ASURA_UNITTEST_CHECK_EQUAL(2, xList.Back()->Prev()->Prev()->Data());
	ASURA_UNITTEST_CHECK_EQUAL(1, xList.Back()->Prev()->Prev()->Prev()->Data());

	// check popping back and front works
	xList.PopFront();
	ASURA_UNITTEST_CHECK_EQUAL(2, xList.Front()->Data());
	xList.PopBack();
	ASURA_UNITTEST_CHECK_EQUAL(3, xList.Back()->Data());

	xList.PopBack();
	ASURA_UNITTEST_CHECK_EQUAL(2, xList.Front()->Data());
	ASURA_UNITTEST_CHECK_EQUAL(2, xList.Back()->Data());
	xList.PopBack();
	ASURA_UNITTEST_CHECK_EQUAL(0, xList.Size());

	xList.Push_Front(2);
	xList.Push_Front(1);
	xList.PopFront();
	ASURA_UNITTEST_CHECK_EQUAL(2, xList.Front()->Data());
	ASURA_UNITTEST_CHECK_EQUAL(2, xList.Back()->Data());
	xList.PopFront();
	ASURA_UNITTEST_CHECK_EQUAL(0, xList.Size());

	// checking sorted insertion
	xList.Insert( 0, IncreasingSort );
	xList.Insert( 0, IncreasingSort );
	xList.Insert( 0, IncreasingSort );
	for ( int i = 0; i < 10; ++i )
	{
		xList.Insert( Asura_Random::GetInt(), IncreasingSort );
	}
	pxElement = xList.Front();
	int iOne = pxElement->Data();
	pxElement = pxElement->Next();
	while ( pxElement )
	{
		int iTwo = pxElement->Data();
		ASURA_UNITTEST_CHECK( iOne <= iTwo );
		iOne = iTwo;
		pxElement = pxElement->Next();
	}
	ASURA_UNITTEST_CHECK_EQUAL(13, xList.Size());
	xList.Clear();
}

#endif

