
//////////////////////////////////////////////////////////////////////////////
//
// Asura_UnitTest_String.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Unit tests for Asura_String class & Asura_StringUtil global functions
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_UNITTEST

#include "Core/Asura_Core.h"
#include "Debugging/Asura_Assert.h"

#include "Core/Asura_String.h"
#include "Core/Asura_StringBuffer.h"

#include "UnitTest/Asura_UnitTest.h"

ASURA_UNITTEST(GeneralTests, String)
{
	const char* const szTestString = "Australia";
	Asura_String xString( szTestString );
	ASURA_UNITTEST_CHECK( ( Asura_StringUtil::Contains( szTestString, "tra" ) == true ) );
	ASURA_UNITTEST_CHECK( ( Asura_StringUtil::Contains( szTestString, "jake" ) == false ) );
	ASURA_UNITTEST_CHECK( ( xString.Contains( "tra" ) == true ) );
	ASURA_UNITTEST_CHECK( ( xString.Contains( "jake" ) == false ) );
	float fValue;
	int iValue;
	Asura_StringUtil::GetFloatValue( "-4.0", &fValue );
	ASURA_UNITTEST_CHECK_EQUAL( -4.0f, fValue );
	Asura_StringUtil::GetFloatValue( "4.5", &fValue );
	ASURA_UNITTEST_CHECK_EQUAL( 4.5f, fValue );
	Asura_StringUtil::GetIntValue( "4", &iValue );
	ASURA_UNITTEST_CHECK_EQUAL( 4, iValue );
	Asura_StringUtil::GetIntValue( "-4", &iValue );
	ASURA_UNITTEST_CHECK_EQUAL( -4, iValue );
	
	{
		Asura_String xString;
		
		ASURA_UNITTEST_CHECK( (0 == xString.Compare( NULL)) );
		ASURA_UNITTEST_CHECK( (0 == xString.Compare( "")) );
		ASURA_UNITTEST_CHECK( (0 != xString.Compare( "Fred")) );
	}
	{
		Asura_String xString("");
		
		ASURA_UNITTEST_CHECK( (0 == xString.Compare( NULL)) );
		ASURA_UNITTEST_CHECK( (0 == xString.Compare( "")) );
		ASURA_UNITTEST_CHECK( (0 != xString.Compare( "Fred")) );
	}
	{
		Asura_String xString("George");
		
		ASURA_UNITTEST_CHECK( (0 != xString.Compare( NULL)) );
		ASURA_UNITTEST_CHECK( (0 != xString.Compare( "")) );
		ASURA_UNITTEST_CHECK( (0 == xString.Compare( "George")) );
	}

	{
		Asura_String xTest2( "AustraliaShort.guff" );
		ASURA_UNITTEST_CHECK( ( xTest2.EqualsIgnoringExtension( "AustraliaShort.notguff" ) == true ) );
	}
}

ASURA_UNITTEST(GeneralTests, StringBuffer)
{
	Asura_String xTestString( "Australia.guff" );
	Asura_StringBuffer<64> xTest1( xTestString.GetString() );

	ASURA_UNITTEST_CHECK( ( Asura_StringUtil::Contains( xTest1.GetString(), xTestString.GetString() ) == true ) );

	ASURA_UNITTEST_CHECK( ( xTest1.GetLength() == xTestString.GetLength() ) );
	ASURA_UNITTEST_CHECK( ( xTest1.GetMaxLength() == 64 ) );

	ASURA_UNITTEST_CHECK( ( xTest1.GetHash() == xTestString.GetHash() ) );
	ASURA_UNITTEST_CHECK( ( xTest1.GetHash_IgnoreExtension() == xTestString.GetHash_IgnoreExtension() ) );

	ASURA_UNITTEST_CHECK( ( xTest1.IsBlank() == false ) );
	xTest1.MakeBlank();
	ASURA_UNITTEST_CHECK( ( xTest1.IsBlank() == true ) );

	xTest1 = "Australia64";
	ASURA_UNITTEST_CHECK( ( xTest1[0] == 'A' ) );
	const Asura_String256 xTestC( "Australia64" );
	const Asura_Char& cA = xTestC[2];
	ASURA_UNITTEST_CHECK( ( cA == 's' ) );

	Asura_StringBuffer<8> xTest8;
	xTest8 = "Australia64";
	ASURA_UNITTEST_CHECK( ( xTest8[6] == 'l' ) );

	xTest8 += "help";

	xTest8 = "Fake";
	xTest8 += "Overrun";

	Asura_String256 xTest2( "AustraliaShort.guff" );
	ASURA_UNITTEST_CHECK( ( xTest2.CompareIgnoringExtension( "australiaShort.notguff" ) == 0 ) );
	ASURA_UNITTEST_CHECK( ( xTest2.EqualsIgnoringExtension( "australiaShort.notguff" ) == true ) );
	ASURA_UNITTEST_CHECK( ( xTest2.LessThanIgnoringExtension( "australiaShorz.notguff" ) == true ) );
	ASURA_UNITTEST_CHECK( ( xTest2.GreaterThanIgnoringExtension( "AustraliaShora.notguff" ) == true ) );
	// Test for the second string must have an extension to be matched as true
	ASURA_UNITTEST_CHECK( ( xTest2.EqualsIgnoringExtension( "AustraliaShort" ) == false ) );

	// Test for non case insensivity
	ASURA_UNITTEST_CHECK( ( xTest2.EqualsIgnoringExtension( "australiaShort", false ) == false ) );

	ASURA_UNITTEST_CHECK( ( xTest2.Compare( "australiashort" ) != 0 ) );
	ASURA_UNITTEST_CHECK( ( xTest2.Compare( "australiashort.guff" ) == 0 ) );

	ASURA_UNITTEST_CHECK( ( xTest2.Compare( "AustraliaShort", false ) != 0 ) );
	ASURA_UNITTEST_CHECK( ( xTest2.Compare( "AustraliaShort.guff", false ) == 0 ) );

	ASURA_UNITTEST_CHECK( ( xTest2 < "AustraliaShort.gufz" ) );
	ASURA_UNITTEST_CHECK( ( xTest2 > "AustraliaShort.gufa" ) );

	ASURA_UNITTEST_CHECK( ( xTest2 == "AustraliaShort.guff" ) );
	ASURA_UNITTEST_CHECK( ( xTest2 != "Jake.guff" ) );

	Asura_String2048 xTest3( "AustraliaLong" );
	ASURA_UNITTEST_CHECK( ( xTest3.StartsWith( "aus" ) == true ) );
	ASURA_UNITTEST_CHECK( ( xTest3.StartsWith( "aus", false ) == false ) );
	ASURA_UNITTEST_CHECK( ( xTest3.StartsWith( "Aus", false ) == true ) );

	ASURA_UNITTEST_CHECK( ( xTest3.EndsWith( "long" ) == true ) );
	ASURA_UNITTEST_CHECK( ( xTest3.EndsWith( "long", false ) == false ) );
	ASURA_UNITTEST_CHECK( ( xTest3.EndsWith( "Long", false ) == true ) );

	ASURA_UNITTEST_CHECK( ( xTest3.Contains( "liaL" ) == true ) );

	ASURA_UNITTEST_CHECK( ( xTest3.Contains( "alon" ) == false ) );

	int iPlace;
	ASURA_UNITTEST_CHECK( ( xTest3.Contains( "tral", &iPlace ) == true ) );
	ASURA_UNITTEST_CHECK( ( iPlace == 3 ) );

	xTest3.MakeLowercase();
	ASURA_UNITTEST_CHECK( ( xTest3.Compare( "australialong", false ) == 0 ) );
	xTest3.MakeUppercase();
	ASURA_UNITTEST_CHECK( ( xTest3.Compare( "AUSTRALIALONG", false ) == 0 ) );

	Asura_String256 xFullname( "c:/test/him/her/jake.asr" );
	Asura_String256 xPath;
	Asura_String256 xFilename;
	Asura_String256 xExtension;

	xPath.MakeBlank();
	xFilename.MakeBlank();
	xExtension.MakeBlank();
	xFullname.SplitFilename( &xPath, &xFilename, &xExtension );
	ASURA_UNITTEST_CHECK( ( xPath == "c:/test/him/her" ) );
	ASURA_UNITTEST_CHECK( ( xFilename == "jake" ) );
	ASURA_UNITTEST_CHECK( ( xExtension == "asr" ) );

	xPath.MakeBlank();
	xFilename.MakeBlank();
	xExtension.MakeBlank();
	xFullname.SplitFilename( NULL, &xFilename, &xExtension );
	ASURA_UNITTEST_CHECK( ( xFilename == "jake" ) );
	ASURA_UNITTEST_CHECK( ( xExtension == "asr" ) );

	xPath.MakeBlank();
	xFilename.MakeBlank();
	xExtension.MakeBlank();
	xFullname.SplitFilename( NULL, NULL, &xExtension );
	ASURA_UNITTEST_CHECK( ( xExtension == "asr" ) );

	xPath.MakeBlank();
	xFilename.MakeBlank();
	xExtension.MakeBlank();
	xFullname.SplitFilename( &xPath, NULL, NULL );
	ASURA_UNITTEST_CHECK( ( xPath == "c:/test/him/her" ) );

	xFullname.Truncate( 4 );
	ASURA_UNITTEST_CHECK( ( xFullname.GetLength() == 4 ) );
	// These should truncate to the max length of the string buffer
	xFullname.Truncate( xFullname.GetMaxLength() + 10 );
	xFullname.Truncate( (u_int)-10 );

	xFullname = "c:/test/him/her/jake.asr";
	xFullname.ReplaceAllChars( '/', '\\' );
	ASURA_UNITTEST_CHECK( ( xFullname == "c:\\test\\him\\her\\jake.asr" ) );

	Asura_StringBuffer<16> xString16;
	xString16 = "Start End Here Goes							";
	xString16.StripWhitespace( Asura_StringUtil::STRIP_BOTH_ENDS );
	ASURA_UNITTEST_CHECK( ( xString16 == "Start End Here" ) );

	xString16 = "						 Start End Here Goes							";
	xString16.StripWhitespace( Asura_StringUtil::STRIP_BOTH_ENDS );
	ASURA_UNITTEST_CHECK( ( xString16 == "Start En" ) );

	xString16.Copy( "Help me" );
	ASURA_UNITTEST_CHECK( ( xString16 == "Help me" ) );

	xString16.NCopy( "Help me", 4 );
	ASURA_UNITTEST_CHECK( ( xString16 == "Help" ) );

	xString16.Cat( " me" );
	ASURA_UNITTEST_CHECK( ( xString16 == "Help me" ) );

	xString16.Cat( 'J' );
	ASURA_UNITTEST_CHECK( ( xString16 == "Help meJ" ) );

	xString16 = "true";
	bool bValue;
	ASURA_UNITTEST_CHECK( ( xString16.GetBoolValue( &bValue ) == true ) );
	ASURA_UNITTEST_CHECK( ( bValue == true ) );

	xString16 = "0";
	ASURA_UNITTEST_CHECK( ( xString16.GetBoolValue( &bValue ) == true ) );
	ASURA_UNITTEST_CHECK( ( bValue == false ) );

	xString16 = "jake";
	ASURA_UNITTEST_CHECK( ( xString16.GetBoolValue( &bValue ) == false ) );

	xString16 = "666.0f";
	float fValue;
	ASURA_UNITTEST_CHECK( ( xString16.GetFloatValue( &fValue ) == true ) );
	ASURA_UNITTEST_CHECK( ( fValue == 666.0f ) );

	xString16 = "12345";
	int iValue;
	ASURA_UNITTEST_CHECK( ( xString16.GetIntValue( &iValue ) == true ) );
	ASURA_UNITTEST_CHECK( ( iValue == 12345 ) );

	xString16 = "0xDEADBEAF";
	ASURA_UNITTEST_CHECK( ( xString16.GetIntValue( &iValue ) == true ) );
	ASURA_UNITTEST_CHECK( ( iValue == (int)0xDEADBEAF ) );

	xString16 = "DEADBEAF";
	ASURA_UNITTEST_CHECK( ( xString16.GetIntValue( &iValue, true ) == true ) );
	ASURA_UNITTEST_CHECK( ( iValue == (int)0xDEADBEAF ) );

	ASURA_UNITTEST_CHECK( ( xString16.FindChar( 'E' ) == 1 ) );
	ASURA_UNITTEST_CHECK( ( xString16.ReverseFindChar( 'E' ) == 5 ) );

	ASURA_UNITTEST_CHECK( ( xString16.FindChar( 'D' ) == 0 ) );
	ASURA_UNITTEST_CHECK( ( xString16.ReverseFindChar( 'F' ) == 7 ) );

	ASURA_UNITTEST_CHECK( ( xString16.FindChar( 'Z' ) == -1 ) );
	ASURA_UNITTEST_CHECK( ( xString16.ReverseFindChar( 'Z' ) == -1 ) );

	xString16.MakeBlank();
	ASURA_UNITTEST_CHECK( ( xString16.FindChar( 'D' ) == -1 ) );
	ASURA_UNITTEST_CHECK( ( xString16.ReverseFindChar( 'F' ) == -1 ) );

	xString16.Print( "%s=%d", "j", 10 );
	ASURA_UNITTEST_CHECK( ( xString16 == "j=10" ) );

	xString16.CatPrint( " Thea=15" );
	ASURA_UNITTEST_CHECK( ( xString16 == "j=10 Thea=15" ) );

	xString16.CatPrint( " Rowan=6" );
	ASURA_UNITTEST_CHECK( ( xString16 == "j=10 Thea=15 Ro" ) );

	Asura_String256 xStringBuffer("Jim's Streaming Masterclass");
	Asura_Chunk_Memory_Stream xStream; xStream << xStringBuffer;

	xStream.SetPosition(0);
	Asura_String xString; xStream >> xString;
	ASURA_UNITTEST_CHECK( ( xString == "Jim's Streaming Masterclass") );

	xStream.SetPosition(0);
	Asura_String256 xStringBuffer2; xStream >> xStringBuffer2;
	ASURA_UNITTEST_CHECK( ( xStringBuffer2 == "Jim's Streaming Masterclass") );
}

#endif // #ifdef ASURA_SUPPORT_UNITTEST
