//////////////////////////////////////////////////////////////////////
// Asura_VolumeGroup												//
// SJL 6/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//																	//
// A container class for a collection of Asura_Volume objects    	//
//////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef ASURA_VOLUMEGROUP_H
#define ASURA_VOLUMEGROUP_H

#include "Core/Asura_Maths.h"

// Forward class declarations
class Asura_Chunk_Stream;
class Asura_Volume;
class Asura_LBAABB_Tree;
class Asura_VolumeGroup;
class Asura_DebugLineRender_LineSet;

typedef void(*Asura_VolumeGroup_Contains_Callback)(u_int uIndex);

struct Asura_VolumeGroup_Test_Parameters
{
	const Asura_VolumeGroup* m_pxGroup;
	const Asura_Vector_3* m_pxPosition;
	const Asura_Bounding_Box* m_pxBB;
	bool m_bResult;
	Asura_VolumeGroup_Contains_Callback m_pfnCallback;
};

typedef void(*Asura_VolumeGroup_Reorder_Callback)(const u_int* puTranslationArray);

class Asura_VolumeGroup
{
	public:

		// constructor / destructor
		Asura_VolumeGroup();
		virtual ~Asura_VolumeGroup();

		// Deletes all volumes.
		void Clear();

		// Simple accessors
		u_int GetNumberOfVolumes() const { return m_uNumberOfVolumes; }
		Asura_Volume* GetVolume(u_int uIndex) { return m_ppxVolumes[uIndex]; }
		const Asura_Volume* GetVolume(u_int uIndex) const { return m_ppxVolumes[uIndex]; }

		// Other public fns.
		void AddVolume(Asura_Volume* pxVolume);
		void RemoveAndDestroyVolume(Asura_Volume* pxVolume);
		void RemoveAndDestroyVolume(u_int uIndex);
		bool Contains(const Asura_Vector_3& xPosition) const;
		bool Contains(const Asura_Vector_3& xPosition, Asura_VolumeGroup_Contains_Callback pfnCallback) const;
		Asura_Quat* GetVolumeOrient(u_int uIndex);
		void DisplaceVolumes(const Asura_Vector_3& xOffset);
		void RotateVolumesAbout(const Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOri);
		void Copy(const Asura_VolumeGroup* pxFrom);

		//Tests for overlap with an AABB. Added since some versions of multi damage trigger need it.
		//This isn't fully implemented for all volume types , and is probably not wonderfully fast either.
		bool Overlaps(const Asura_Bounding_Box& xBB) const;

        // Provides an AABB that encompasses all volumes.
        void GetEncompassingAABB( Asura_Bounding_Box& xBB ) const;

#if !defined(ASURA_NO_DEBUG_OUTPUT)
		virtual void AddToLineSet(const u_int uARGB, Asura_DebugLineRender_LineSet& xLineSet) const;
#endif	//!defined(ASURA_NO_DEBUG_OUTPUT)

		#ifdef ASURA_TOOLS
		//Moves/rotates volumes that are part of an entity template
		void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
		#endif


		// load/save stream functions.
		void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
		// When bAddLBAABBTree is true, you may use the callback in order to obtain
		// the translation array for the version that was saved. Note that this is
		// not the same as the version that remains in memory (this function is const).
		void WriteToChunkStream(Asura_Chunk_Stream& xStream, bool bAddLBAABBTree = false, Asura_VolumeGroup_Reorder_Callback pfnReorderCallback = NULL) const;

		// for converting old entities - reads in an array of Asura_OrientedVolume
		// and converts them to Asura_Volume_Cuboid.
		void ReadOrientedVolumeArrayFromChunkStream(Asura_Chunk_Stream& xStream, u_int uNumberOfVolumes);

		static void LBAABBContainsCallback(const u_int uIndex);
		static void LBAABBOverlapsCallback(const u_int uIndex);
#ifdef ASURA_TOOLS
		// Optional callback takes the re-ordering translation array.
		// This array is deleted at the end of the function, so do not take a shallow copy of it.
		void GenerateLBAABBTree(Asura_VolumeGroup_Reorder_Callback pfnReorderCallback = NULL);
#endif
		void RemoveLBAABBTree();
	protected:

		// data
		u_int m_uNumberOfVolumes;
		Asura_Volume** m_ppxVolumes;
		Asura_LBAABB_Tree* m_pxLBAABBTree;

	private:

		// Readies space for n volumes to avoid unnecessary fragmentation when creating or streaming in volumes.
		void SetCapacity(u_int uNumberOfVolumes);

		// static data
		static const u_int s_uCHUNKVERSION;
		static const u_int s_uCHUNK_FLAG_HAS_LBAABB = 0x00000001;
		static Asura_VolumeGroup_Test_Parameters s_xTestParams;
};

#endif // ASURA_VOLUMEGROUP_H

