// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_PC_SUPPORT_STEAMWORKS

#include "Core/Asura_List.h"								// Asura
#include "Core/Asura_Timers.h"
#include "Core/Asura_Memheap.h"
#include "Network/Asura_NetSocket.h"
#include "Network/Asura_Platform_NetSocket.h"
#include "Network/Steamworks/Asura_PC_Steamworks.h"
#include "Network/Steamworks/Asura_PC_SteamworksClient.h"
#include "Network/Steamworks/Asura_PC_SteamworksMessages.h"
#include "Network/Steamworks/Asura_PC_NetSocketSteamworks.h"
#include "Network/Steamworks/Asura_PC_SteamworksServer.h"
//#include "Sound/Asura_XAudio2.h"
//#include "Sound/Asura_XAudio2_Voice.h"

#include "steam/steam_api.h"								// Steam

// D E F I N E S /////////////////////////////////////////////////////////////

#define ASURA_PC_STEAMWORKS_CLIENT_VOICE_REC_UPDATE_INTERVAL	0.25f				// 1/4 Second
#define ASURA_PC_STEAMWORKS_CLIENT_MAX_VOICE_REC_DATA			1024
#define ASURA_PC_STEAMWORKS_CLIENT_MAX_VOICE_PLAY_DATA			( 1024 * 64 )
#define ASURA_PC_STEAMWORKS_CLIENT_DECOMPRESS_BUFFER_SIZE		( 1024 * 32 )

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

class Asura_PC_Steamworks_ClientPeer
{
	public:

		Asura_NetAddress				m_xAddress;
		unsigned short					m_usPort;
		bool							m_bIsServer;

		Asura_PC_NetSocketSteamworks*	m_pxSocket;

		bool							m_bShouldSendVoice;

		Asura_PC_Steamworks_ClientPeer ()
		{
			m_usPort = 0;
			m_pxSocket = 0;
			m_bShouldSendVoice = true;
		}

		~Asura_PC_Steamworks_ClientPeer ()
		{
			if( m_pxSocket )
			{
				delete m_pxSocket;
			}
		}
};

class Asura_PC_Steamworks_PeerVoice
{
	public:
	
		Asura_Handle	m_xVoiceHandle;
		uint64			m_uSteamID;

		Asura_PC_Steamworks_PeerVoice ()
		{
			m_xVoiceHandle	= ASURA_INVALID_HANDLE;
			m_uSteamID		= CSteamID().ConvertToUint64();
		}

		~Asura_PC_Steamworks_PeerVoice ()
		{
			if( m_xVoiceHandle != ASURA_INVALID_HANDLE )
			{
				//Asura_XAudio2::DestroyVoice( m_xVoiceHandle );
			}
		}
};

// S T A T I C   M E M B E R   V A R I A B L E S /////////////////////////////

bool											Asura_PC_SteamworksClient::s_bConnectedToServer		= false;
Asura_UnicodeChar								Asura_PC_SteamworksClient::s_awcSteamIDText			[ASURA_PC_STEAM_MAX_STEAMID_LENGTH];
char*											Asura_PC_SteamworksClient::s_pcAuthBlob				= 0;
u_int											Asura_PC_SteamworksClient::s_uAuthBlobSize			= 0;

bool											Asura_PC_SteamworksClient::s_bVoiceRecActive		= false;
char*											Asura_PC_SteamworksClient::s_pcVoiceRecData			= 0;
u_int											Asura_PC_SteamworksClient::s_uVoiceRecDataSize		= 0;
float											Asura_PC_SteamworksClient::s_fLastVoiceRecUpdate		= 0.0f;

Asura_List<Asura_PC_Steamworks_ClientPeer>		Asura_PC_SteamworksClient::s_xClientList;
Asura_PC_NetSocketSteamworks*					Asura_PC_SteamworksClient::s_pxListenSocket			= 0;
bool											Asura_PC_SteamworksClient::s_bAllowVoicePlayback	= true;
Asura_List<Asura_PC_Steamworks_PeerVoice>		Asura_PC_SteamworksClient::s_xPeerVoices;
Asura_MemHeap*									Asura_PC_SteamworksClient::s_pxVoicePlaybackMemHeap;
char*											Asura_PC_SteamworksClient::s_pcVoicePlaybackMemory;

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

// _____________________ Asura_PC_SteamworksClient ___________________________

Asura_PC_SteamworksClient::Asura_PC_SteamworksClient ()
{
}

Asura_PC_SteamworksClient::~Asura_PC_SteamworksClient ()
{
}


void Asura_PC_SteamworksClient::Initialise ()
{
	if( !Asura_PC_Steamworks::IsInitialised() )
	{
		Asura_Assert( false, "Steamworks not initialised" );
		return;
	}

	if( SteamFriends() )
	{
		const char* szPersona = SteamFriends()->GetPersonaName();

		// Save the steam ID of the user
		MultiByteToWideChar( CP_UTF8, 0, szPersona, -1, s_awcSteamIDText, ASURA_PC_STEAM_MAX_STEAMID_LENGTH );
	}

	s_pcVoiceRecData			= static_cast<char*>( Asura_Allocate( ASURA_PC_STEAMWORKS_CLIENT_MAX_VOICE_REC_DATA ) );
	s_pcVoicePlaybackMemory		= static_cast<char*>( Asura_Allocate( ASURA_PC_STEAMWORKS_CLIENT_MAX_VOICE_PLAY_DATA ) );
	s_pxVoicePlaybackMemHeap	= new Asura_MemHeap( s_pcVoicePlaybackMemory, ASURA_PC_STEAMWORKS_CLIENT_MAX_VOICE_PLAY_DATA );

	s_pxVoicePlaybackMemHeap->EnableThreadSafety();
}


void Asura_PC_SteamworksClient::Update ()
{
	// Check for new messages
	if( s_pxListenSocket )
	{
		s_pxListenSocket->Update();
		s_pxListenSocket->API_ReceiveMessages();
	}
	
	// Do we have any voice chat to send from the client?
	if( SteamUser() )
	{
		if( ( Asura_Timers::GetLocalUnscaledTime() - s_fLastVoiceRecUpdate ) > ASURA_PC_STEAMWORKS_CLIENT_VOICE_REC_UPDATE_INTERVAL )
		{
			s_fLastVoiceRecUpdate = Asura_Timers::GetLocalUnscaledTime();
			
			#ifdef ASURA_ASSERT
			EVoiceResult xResult = SteamUser()->GetCompressedVoice( s_pcVoiceRecData, ASURA_PC_STEAMWORKS_CLIENT_MAX_VOICE_REC_DATA, &s_uVoiceRecDataSize );
			switch( xResult )
			{
				case k_EVoiceResultNotInitialized:	Asura_Assert( false, "Steam: Voice not initialised" );			break;
				case k_EVoiceResultBufferTooSmall:	Asura_Assert( false, "Steam: voice data buffer too small" );	break;
				case k_EVoiceResultDataCorrupted:	Asura_Assert( false, "Steam: Voice data corrupt" );				break;
			}
			#else
			SteamUser()->GetCompressedVoice( s_pcVoiceRecData, ASURA_PC_STEAMWORKS_CLIENT_MAX_VOICE_REC_DATA, &s_uVoiceRecDataSize );
			#endif

			// Make a message from this data
			if( s_uVoiceRecDataSize > 0 )
			{
				unsigned short uMsgSize = static_cast< unsigned short >( Asura_PC_SteamMsg_SendHdr::Size() + Asura_PC_SteamMsg_VoiceHdr::Size() + s_uVoiceRecDataSize );
				char* pcMsgData = static_cast<char*>( Asura_Allocate( uMsgSize ) );

				Asura_PC_SteamMsg_SendHdr* pxMsgHdr = reinterpret_cast< Asura_PC_SteamMsg_SendHdr* >( pcMsgData );

				pxMsgHdr->m_usSequence	= 0;
				pxMsgHdr->m_usChecksum	= 0;
				pxMsgHdr->m_usMsgID		= ASURA_NETTRANSPORT_MSG_VOICE_DATA;
				pxMsgHdr->m_usMsgSize	= static_cast<unsigned short>( uMsgSize );

				Asura_PC_SteamMsg_VoiceHdr* pxVoiceHdr = reinterpret_cast< Asura_PC_SteamMsg_VoiceHdr* >( pcMsgData + Asura_PC_SteamMsg_SendHdr::Size() );
				pxVoiceHdr->m_uSteamID = Asura_PC_SteamworksClient::GetSteamID_Int64();
				Asura_Mem::Copy( &pxVoiceHdr->m_cVoiceData, s_pcVoiceRecData, s_uVoiceRecDataSize );

				pxMsgHdr->m_usChecksum	= Asura_NetTransport_System::ComputeChecksum( pcMsgData + 4, uMsgSize - 4 );

				// loop over every peer and send the data to them
				for( u_int uPeer = 0; uPeer < static_cast< u_int >( s_xClientList.GetNumEntries() ); ++uPeer )
				{
					Asura_PC_Steamworks_ClientPeer* pxPeer = s_xClientList.GetEntryByIndex( uPeer );
					
					if( pxPeer->m_bShouldSendVoice )
					{
						// Send the voice to the client... if the client is also the server use our primary socket
						unsigned short  usPort = ASURA_PC_STEAMWORKS_VOICE_PORT;
						Asura_PC_NetSocketSteamworks* pxSocket = pxPeer->m_pxSocket;
						
						if( pxPeer->m_bIsServer )
						{
							pxSocket = reinterpret_cast< Asura_PC_NetSocketSteamworks* >( Asura_NetTransport_System::GetSocket() );
							usPort = ASURA_PC_STEAMWORKS_GAME_PORT;
						}

						if( pxSocket )
						{		
							pxSocket->UseReliableSend( false );

							#ifdef ASURA_ASSERT
							const bool bSentVoiceOK =
							#endif
							pxSocket->Send( pxPeer->m_xAddress, usPort, pcMsgData, uMsgSize, false );
							Asura_Assert( bSentVoiceOK, "Failed to send voice data" );
						}
					}
				}
				Asura_Deallocate( pcMsgData );
			}
		}
	}

	// Check if we need to remove any peers playing voice data
	for( int iPeer = ( s_xPeerVoices.GetNumEntries() - 1 ); iPeer >= 0; --iPeer )
	{
		//Asura_PC_Steamworks_PeerVoice* pxPeerVoice = s_xPeerVoices.GetEntryByIndex( iPeer );
		
		// If our voice playback is finished, destroy the peer entry in our list
		//Asura_XAudio2_Voice* pxVoice = Asura_XAudio2::GetVoice( pxPeerVoice->m_xVoiceHandle );
		//if( pxVoice && !pxVoice->IsPlaying() )
		//{
		//	Asura_XAudio2::DestroyVoice( pxPeerVoice->m_xVoiceHandle );

		//	s_xPeerVoices.RemoveFromList( pxPeerVoice );
		//	s_pxVoicePlaybackMemHeap->Free( pxPeerVoice );
		//}
	}
}


void Asura_PC_SteamworksClient::Shutdown ()
{
	Asura_PC_SteamworksClient::ReleaseAuthBlob();
	
	while( s_xPeerVoices.GetNumEntries() > 0 )
	{
		delete s_xPeerVoices.PopHeadObject();
	}

	while( s_xClientList.GetNumEntries() > 0 )
	{
		delete s_xClientList.PopHeadObject();
	}

	Asura_Deallocate( s_pcVoiceRecData );
	s_pcVoiceRecData = 0;
	s_uVoiceRecDataSize = 0;

	delete s_pxVoicePlaybackMemHeap;
	s_pxVoicePlaybackMemHeap = 0;

	Asura_Deallocate( s_pcVoicePlaybackMemory );
	s_pcVoicePlaybackMemory = 0;
	
	delete s_pxListenSocket;
	s_pxListenSocket = 0;
}


CSteamID Asura_PC_SteamworksClient::GetSteamID ()
{
	if( SteamUser() )
	{
		return SteamUser()->GetSteamID();
	}
	return CSteamID();
}


uint64 Asura_PC_SteamworksClient::GetSteamID_Int64 ()
{
	if( SteamUser() )
	{
		return SteamUser()->GetSteamID().ConvertToUint64();
	}
	return 0;
}


const Asura_UnicodeChar* Asura_PC_SteamworksClient::GetSteamIDText	()
{
	return s_awcSteamIDText;
}


bool Asura_PC_SteamworksClient::InitiateServerConnection ( const CSteamID xServerSteamID, const ASURA_PC_STEAMWORKS_AUTH_LEVEL eAuthLevel, const Asura_NetAddress xAddress, const unsigned short usPort )
{
	if( !SteamUser() )
	{
		Asura_Assert( false, "No steam user in steamworks" );
		return false;
	}

	ReleaseAuthBlob();

	// TODO: Sort out the voice comms
	/*if( !Asura_PC_SteamworksServer::IsServerRunning() )
	{
		// This is as good a time as any to create our listen socket for voice
		Asura_Assert( !s_pxListenSocket, "Listen socket should not be created here" );
		s_pxListenSocket = new Asura_PC_NetSocketSteamworks( ASURA_PC_STEAMWORKS_CLIENT_MAX_PEERS, ASURA_PC_STEAMSOCKET_TYPE_SERVER );
		
		unsigned short usVoicePort = ASURA_PC_STEAMWORKS_VOICE_PORT;
		s_pxListenSocket->Bind( Asura_NetAddress(0), usVoicePort );
	}*/

	s_pcAuthBlob = new char[ ASURA_PC_STEAM_MAX_AUTH_BLOB_SIZE ];

	// Now call InitiateGameConnection, this will give us our auth blob
	if( eAuthLevel > ASURA_PC_STEAMWORKS_NO_AUTHENTICATE )
	{
		const bool bVACSecure = ( eAuthLevel == ASURA_PC_STEAMWORKS_AUTHENTICATE_AND_VAC );

		s_uAuthBlobSize = SteamUser()->InitiateGameConnection( s_pcAuthBlob, ASURA_PC_STEAM_MAX_AUTH_BLOB_SIZE, xServerSteamID, xAddress.m_uAddress, usPort, bVACSecure );
	}
	else
	{
		// Dont authenticate, but still call InitiateGameConnection to communicate presence data
		s_uAuthBlobSize = SteamUser()->InitiateGameConnection( 0, 0, k_steamIDNonSteamGS, xAddress.m_uAddress, usPort, false );
	}

	s_bConnectedToServer = true;

	return true;
}


bool Asura_PC_SteamworksClient::AcquireAuthBlob ( void* &pBlob, u_int &uBlobSize )
{
	if( !s_pcAuthBlob )
	{
		return false;
	}

	pBlob = s_pcAuthBlob;
	uBlobSize = s_uAuthBlobSize;

	return true;
}


void Asura_PC_SteamworksClient::ReleaseAuthBlob ( )
{
	if( s_pcAuthBlob )
	{
		delete []s_pcAuthBlob;
		s_pcAuthBlob = 0;
		s_uAuthBlobSize = 0;
	}
}


void Asura_PC_SteamworksClient::TerminateServerConnection ( )
{
	if( !IsConnectedToServer() )
	{
		return;
	}

	s_bConnectedToServer = false;

	Asura_NetAddress xAddress;
	unsigned short usPort;

	if( Asura_PC_SteamworksServer::IsServerRunning() )
	{
		// If a server is running we are just disconnecting from ourselves
		xAddress.m_uAddress = ( 1 ) + ( 0 << 8 ) + ( 0 << 16 ) + ( 127 << 24 );
		usPort = ASURA_PC_STEAMWORKS_GAME_PORT;
	}
	else
	{
		// Grab the server IP and Port from the steam socket
		const Asura_NetSocket* pxSocket = Asura_NetTransport_System::GetSocket();
		if( !pxSocket )
		{
			Asura_Assert( false, "Failed to get netsocked" );
			return;
		}

		xAddress = pxSocket->GetAddress();
		usPort = pxSocket->GetPort();

		// Scrap our listen socket
		delete s_pxListenSocket;
		s_pxListenSocket = 0;
	}

	// Clear our client list
	while( s_xClientList.GetNumEntries() )
	{
		Asura_PC_Steamworks_ClientPeer* pxClient = s_xClientList.PopHeadObject();
		delete pxClient;
	}

	// Clear our peer voices
	while( s_xPeerVoices.GetNumEntries() )
	{
		Asura_PC_Steamworks_PeerVoice* pxPeerVoice = s_xPeerVoices.PopHeadObject();
		delete pxPeerVoice;
	}

	SteamUser()->TerminateGameConnection( xAddress.m_uAddress, usPort );
}


void Asura_PC_SteamworksClient::StartVoiceRecording ( )
{
	// Prevent a conflict with steam voice chat
	if( SteamFriends() )
	{
		SteamFriends()->SetInGameVoiceSpeaking( GetSteamID(), true );
	}
	if( SteamUser() )
	{
		SteamUser()->StartVoiceRecording();
	}

	s_bVoiceRecActive = true;
}


void Asura_PC_SteamworksClient::StopVoiceRecording ( )
{
	if( SteamFriends() )
	{
		SteamFriends()->SetInGameVoiceSpeaking( GetSteamID(), false );
	}
	if( SteamUser() )
	{
		SteamUser()->StopVoiceRecording();
	}
	
	s_bVoiceRecActive = false;
}


bool Asura_PC_SteamworksClient::HandleMessage ( const void* pData, const u_int uDataSize )
{
	const char* pcData = static_cast<const char*>( pData );
	
	const Asura_PC_SteamMsg_RecvHdr* pxMsgHdr = reinterpret_cast< const Asura_PC_SteamMsg_RecvHdr* >( pcData );

	switch( pxMsgHdr->m_usMsgID )
	{
		case ASURA_NETTRANSPORT_MSG_CLIENTLIST:		HandleMessage_ClientList( pData, uDataSize );	return true;
		case ASURA_NETTRANSPORT_MSG_VOICE_DATA:		HandleMessage_VoiceData	( pData, uDataSize );	return true;
	}

	return false;
}


void Asura_PC_SteamworksClient::HandleMessage_ClientList ( const void* pData, const u_int uDataSize )
{
	// Update our client list

	const char* pcData = static_cast< const char* >( pData );
		
	const Asura_PC_SteamMsg_ClientListHdr* pxClientMsgHdr = reinterpret_cast< const Asura_PC_SteamMsg_ClientListHdr* >( pcData + Asura_PC_SteamMsg_RecvHdr::Size() );
	const Asura_PC_SteamMsg_ClientEntry* pxClientMsgEntry = &pxClientMsgHdr->m_xClients;

	for( u_int uMsgClient = 0; uMsgClient < pxClientMsgHdr->m_uNumClients; ++uMsgClient )
	{
		if( !pxClientMsgEntry[uMsgClient].m_bIsClientSentTo )		// we dont care about adding ourselves to our client list
		{
			bool bNeedToAdd = true;

			// Check if this client is already present in our list
			for( u_int uExisting = 0; uExisting < static_cast< u_int >( s_xClientList.GetNumEntries() ); ++uExisting )
			{
				Asura_PC_Steamworks_ClientPeer* pxExistingClient = s_xClientList.GetEntryByIndex( uExisting );

				if( pxExistingClient->m_xAddress == pxClientMsgEntry[ uMsgClient ].m_xAddress )
				{
					bNeedToAdd = false;
					break;
				}
			}

			if( bNeedToAdd )
			{
				Asura_PC_Steamworks_ClientPeer* pxNewClient = new Asura_PC_Steamworks_ClientPeer;

				pxNewClient->m_xAddress = pxClientMsgEntry[uMsgClient].m_xAddress;
				pxNewClient->m_usPort = pxClientMsgEntry[uMsgClient].m_usPort;
				pxNewClient->m_bIsServer = pxClientMsgEntry[uMsgClient].m_bIsServer;
				pxNewClient->m_pxSocket = 0;
				
				if( pxNewClient->m_bIsServer )
				{
					// Update the address of the server (it will be localhost) from our socket
					Asura_NetSocket* pxSocket = Asura_NetTransport_System::GetSocket();
					if( pxSocket )
					{
						pxNewClient->m_xAddress = pxSocket->GetAddress();
					}
				}
				else
				{
					// Create a socket to this client
					pxNewClient->m_pxSocket = new Asura_PC_NetSocketSteamworks( 1, ASURA_PC_STEAMSOCKET_TYPE_CLIENT );
				}
				
				s_xClientList.AddToList( pxNewClient );
			}
		}
	}

	// Check if we need to remove any clients
	for( int iExisting = static_cast<int>( s_xClientList.GetNumEntries() ) - 1; iExisting >= 0 ; --iExisting )
	{
		Asura_PC_Steamworks_ClientPeer* pxExistingClient = s_xClientList.GetEntryByIndex( iExisting );

		if( pxExistingClient->m_bIsServer )
		{
			// we assume the server is never lost, we cannot compare its address anyway since it will have sent localhost
			// as its address, and we have stored the actual address from our primary socket
			continue;
		}

		bool bNeedToRemove = true;
		for( u_int uMsgClient = 0; uMsgClient < pxClientMsgHdr->m_uNumClients; ++uMsgClient )
		{
			if( pxClientMsgEntry[uMsgClient].m_xAddress == pxExistingClient->m_xAddress )
			{
				bNeedToRemove = false;
				break;
			}
		}

		if( bNeedToRemove )
		{
			s_xClientList.RemoveFromList( pxExistingClient );
			delete pxExistingClient;
		}
	}
}


// need placement new
#include "Core/Asura_MemoryManagement_DebugOff.h"

void Asura_PC_SteamworksClient::HandleMessage_VoiceData ( const void* pData, const u_int uDataSize )
{
	const char* pcData = static_cast< const char* >( pData );

	u_int uVoiceSize = uDataSize - ( Asura_PC_SteamMsg_RecvHdr::Size() + Asura_PC_SteamMsg_VoiceHdr::Size() );

	const Asura_PC_SteamMsg_VoiceHdr* pxVoiceHdr = reinterpret_cast< const Asura_PC_SteamMsg_VoiceHdr* >( pcData + Asura_PC_SteamMsg_RecvHdr::Size() );

	// See if we need to create a new voice for this peer
	Asura_PC_Steamworks_PeerVoice* pxPeerVoice = 0;

	for( u_int uPeerVoice = 0; uPeerVoice < static_cast< u_int >( s_xPeerVoices.GetNumEntries() ); ++uPeerVoice )
	{
		pxPeerVoice = s_xPeerVoices.GetEntryByIndex( uPeerVoice );
		if( pxPeerVoice->m_uSteamID == pxVoiceHdr->m_uSteamID )
		{
			break;
		}
		else
		{
			pxPeerVoice = 0;
		}
	}

	if( !pxPeerVoice )
	{
		// ...we do (allocate it on a memheap since we might be destroying and creating a lot of them)
		void* pxMem = s_pxVoicePlaybackMemHeap->Alloc( sizeof( Asura_PC_Steamworks_PeerVoice ) );

		if( !pxMem )
		{
			Asura_Assert( false, "Ran out of memory in steam voice playback heap" );
			return;
		}

		pxPeerVoice = new ( pxMem ) Asura_PC_Steamworks_PeerVoice;

		if( pxPeerVoice )
		{
			s_xPeerVoices.AddToList( pxPeerVoice );

			pxPeerVoice->m_uSteamID = pxVoiceHdr->m_uSteamID;

			WAVEFORMATEX xFormat;
			xFormat.wFormatTag			= WAVE_FORMAT_PCM;
			xFormat.nChannels			= 1;
			xFormat.nSamplesPerSec		= 11025;
			xFormat.nAvgBytesPerSec		= 22050;	// 2 * 11025
			xFormat.nBlockAlign			= 2;
			xFormat.wBitsPerSample		= 16;
			xFormat.cbSize				= 0;

			// create the audio voice
			//pxPeerVoice->m_xVoiceHandle = Asura_XAudio2::CreateVoice( xFormat, false, false );

			//Asura_XAudio2_Voice* pxVoice = Asura_XAudio2::GetVoice( pxPeerVoice->m_xVoiceHandle );

			//if( pxVoice )
			//{
			//	pxVoice->SetBufferEndCallback( AudioVoiceBufferCallback );
			//}
		}
	}
	
	if( !pxPeerVoice )
	{
		Asura_Assert( false, "Could not create peer voice structure" );
		return;
	}

	// Decompress the received data
	Asura_Assert( s_pxVoicePlaybackMemHeap, "No voice heap" );

	// This is a bit big to go on the stack really
	char* pcDecompressBuffer[ ASURA_PC_STEAMWORKS_CLIENT_DECOMPRESS_BUFFER_SIZE ];
	uint32 iBytesWritten = 0;

	EVoiceResult xResult = SteamUser()->DecompressVoice( const_cast<char*>( &pxVoiceHdr->m_cVoiceData ), uVoiceSize, pcDecompressBuffer, ASURA_PC_STEAMWORKS_CLIENT_DECOMPRESS_BUFFER_SIZE, &iBytesWritten );

	Asura_Assertf( xResult == k_EVoiceResultOK, ( "Voice decompress failed (%d)", xResult ) );

	if( xResult == k_EVoiceResultOK )
	{
		// Put this data on our memheap and pass it as a new buffer to the voice
		char* pcNewVoiceData = static_cast<char*>( s_pxVoicePlaybackMemHeap->Alloc( iBytesWritten ) );
		if( pcNewVoiceData )
		{
			Asura_Mem::Copy( pcNewVoiceData, pcDecompressBuffer, iBytesWritten );

			//Asura_XAudio2_Voice* pxVoice = Asura_XAudio2::GetVoice( pxPeerVoice->m_xVoiceHandle );

			//if( pxVoice )
			//{
			//	pxVoice->EnqueueData( pcNewVoiceData, static_cast<u_int>( iBytesWritten ), false, pcNewVoiceData );
			//	if( !pxVoice->IsPlaying() )
			//	{
			//		pxVoice->Start();
			//	}
			//}
		}
	}
}

#include "Core/Asura_MemoryManagement_DebugOn.h"

void Asura_PC_SteamworksClient::SetShouldSendVoiceToPeer ( const CSteamID xPeer, const bool bShouldSend )
{
	for( u_int uClient = 0; uClient < static_cast< u_int >( s_xClientList.GetNumEntries() ); ++uClient )
	{
		Asura_PC_Steamworks_ClientPeer* pxExistingClient = s_xClientList.GetEntryByIndex( uClient );

		if( pxExistingClient->m_xAddress.m_xSteamID == xPeer )
		{
			pxExistingClient->m_bShouldSendVoice = bShouldSend;
		}
	}
}


void Asura_PC_SteamworksClient::AudioVoiceBufferCallback ( void* pContext )
{
	// The context will contain the address of the buffer so we can free it
	Asura_Assert( s_pxVoicePlaybackMemHeap, "No voice heap" );

	s_pxVoicePlaybackMemHeap->Free( pContext );
}


bool Asura_PC_SteamworksClient::IsPeerVoicePlaying ( const CSteamID xSteamID )
{
	for( u_int uPeerVoice = 0; uPeerVoice < static_cast< u_int >( s_xPeerVoices.GetNumEntries() ); ++uPeerVoice )
	{
		Asura_PC_Steamworks_PeerVoice* pxPeerVoice = s_xPeerVoices.GetEntryByIndex( uPeerVoice );
		if( CSteamID( pxPeerVoice->m_uSteamID ) == xSteamID )
		{
			return true;
		}
	}
	
	return false;
}

bool Asura_PC_SteamworksClient::WriteToRemoteStorage ( const void* pData, const u_int uDataSize, const char* szFilename )
{
	if( !SteamRemoteStorage() )
	{
		Asura_Assert( false, "No remote storage instance in steamworks");
		return false;
	}

	return SteamRemoteStorage()->FileWrite( szFilename, pData, static_cast<int32>( uDataSize ) );
}


bool Asura_PC_SteamworksClient::ReadFromRemoteStorage ( void* pData, const u_int uDataSize, const char* szFilename )
{
	if( !SteamRemoteStorage() )
	{
		Asura_Assert( false, "No remote storage instance in steamworks");
		return false;
	}

	int32 iToRead = static_cast<int32>( uDataSize );

	return ( SteamRemoteStorage()->FileRead( szFilename, pData, iToRead ) == iToRead );
}


bool Asura_PC_SteamworksClient::RemoteStorageFileExists ( const char* szFilename )
{
	if( !SteamRemoteStorage() )
	{
		Asura_Assert( false, "No remote storage instance in steamworks");
		return false;
	}

	return SteamRemoteStorage()->FileExists( szFilename );
}


bool Asura_PC_SteamworksClient::IsFriend ( const CSteamID xSteamID )
{
	if( SteamFriends() )
	{
		return ( SteamFriends()->GetFriendRelationship( xSteamID ) == k_EFriendRelationshipFriend );
	}
	return false;
}

#endif	// ASURA_PC_SUPPORT_STEAMWORKS

//eof
