//*******************************************************************
//
// File: Asura_Direct3D_Botanicals_Game_Render_Manager.h
//
// Created: 15 Mar 2006
//
// Description: 
//
//*******************************************************************

#ifndef _ASURA_DIRECT3D_BOTANICALS_GAME_RENDER_MANAGER_H_
#define _ASURA_DIRECT3D_BOTANICALS_GAME_RENDER_MANAGER_H_

#if defined( ASURA_USE_BOTANICALS)

#if !defined( WIN32)
	#error This header should not be included into anything other than a Windows build
#endif

// other libraries
#include "Core/Asura_LinkList.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_Multithread.h"
#include "Core/Asura_String.h"

#include "Light/Asura_Photon.h" // to get dynamic lights
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_DIPCache.h"
#include "Rendering/Asura_Direct3D_VertexBuffer_Dynamic.h"
#include "Rendering/Asura_Direct3D_IndexBuffer_Dynamic.h"
#include "Rendering/Asura_Direct3D_MaterialResponse.h"

// internal dependencies
#include "Botanicals/Asura_Botanicals_Array.h"

#include "Botanicals/Asura_Botanical_Game_Render_Constants.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance_Header.h"
#include "Botanicals/Asura_Botanical_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanical_Game_Render_Types.h"

#include "Botanicals/Asura_Botanical_Game_Render_Type.h"
#include "Botanicals/Asura_Botanical_Game_Render_TypeID.h"

// forward declarations

class Asura_Direct3D_BGR_Manager;
class Asura_Chunk_Stream;
class Asura_DIPCache_DIPData;
class Asura_Direct3D_Effect;
class Asura_Direct3D_BGR_ProcessInstanceHeader;

#if defined( ASURA_TOOLS)
	class Tools_BotanicalShape;
	class Tools_Botanicals_Object;
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	class Asura_BGR_Type_BasicInteractive;
#endif

//*******************************************************************
// Constants
//*******************************************************************

// DLH - 23 October 2006 10:30:55 - Setup the Correct Render Code
#if !defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30)
	#if (defined(ASURA_TARGET_SM3)
		#define ASURA_DIRECT3D_BGR_SUPPORT_SM30 (1)
	#endif
#endif

#if !defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) || 0 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	#if !defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) || 0 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
		#define ASURA_DIRECT3D_BGR_SUPPORT_SM11 (1)
	#endif
#endif

// DLH - 03 April 2006 14:44:10 - If none of these are defined in the EngineConfigure File then turn the following on
#if !defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D) && !defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES) && !defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	#define ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D
	#define ASURA_DIRECT3D_BGR_SUPPORT_LEAVES
	#define ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS
	#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
		#define ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE
	#endif
	#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS)
		#define ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_SECTION_VERSIONS
	#endif
#endif

#define ASURA_DIRECT3D_BGR_SUPPORT_NORMALS

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA)
	//#define ASURA_BOTANICAL_INSTANCE_USER_DATA_TYPE __m128
	//#define ASURA_BOTANICAL_INSTANCE_USER_DATA_TYPE uint8[3]
	#if defined( ASURA_BOTANICAL_INSTANCE_USER_DATA_TYPE)
		typedef ASURA_BOTANICAL_INSTANCE_USER_DATA_TYPE Asura_Direct3D_BGR_DynamicUserData;
	#endif
#endif

#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Cache.h"

//*******************************************************************
// Class Definitions
//*******************************************************************

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
struct Asura_Direct3D_BGR_Type_VertexGeometry3DStorage
{
public:
	Asura_AVector3 m_xPosition;
	Asura_AVector3 m_xNormal;
	uint32 m_uColour;
	float m_afTextcoord0[ 2];

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	float m_afTextcoord1[ 2];
	Asura_AVector3 m_xBinormal;
	Asura_AVector3 m_xTangent;
	float m_fBoneWeight0;
	float m_fBoneWeight1;
#endif

public:
	bool operator == ( const Asura_Direct3D_BGR_Type_VertexGeometry3DStorage& xRHS) const;
	inline bool operator != ( const Asura_Direct3D_BGR_Type_VertexGeometry3DStorage& xRHS) const { return( !( *this == xRHS));}

	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif
};

struct Asura_Direct3D_BGR_Type_VertexGeometry3DBasic
{
public:
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
	float m_afPosition[ 3];
	uint32 m_uColour;
	float m_afTextcoord0[ 2];
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_NORMALS)
		float m_afNormal[ 3];
	#endif
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	float m_afPosition[ 3];		// DLH - 3D Center of the Leaf
	float m_afNormal[3];		// DLH - Billboards corner Normal
	float m_afTangent[3];		// DLH - TextureUV Tangent
	float m_fBoneWeight1;		// DLH - How strong is the bone1 influance
	float m_afBinormal[3];		// DLH - TextureUV Binormal
	float m_fBoneWeight0;		// DLH - How strong is the bone0 influance
	float m_afTextcoord0[ 2];	// DLH - Texture coord0 of the corner
	float m_afTextcoord1[ 2];	// DLH - Texture coord1 of the corner
	uint32 m_uColour;			// DLH - 
#endif
};
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
struct Asura_Direct3D_BGR_Type_VertexGeometryLeafStorage
{
public:
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

public:
	Asura_AVector3 m_xPosition;		// DLH - 3D Center of the Leaf
	float m_afCorner[ 2];		// DLH - 2D Width and Height offset of the leaf corner. This is in tree space scale.
	uint32 m_uColour;			// DLH - 
	float m_afTextcoord0[ 2];	// DLH - Texture coord of the corner
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	float m_afTextcoord1[ 2];	// DLH - Texture coord of the corner
	Asura_AVector3 m_xNormal;
	Asura_AVector3 m_xBinormal;
	Asura_AVector3 m_xTangent;
	float m_fBoneWeight0;		// DLH - How strong the is the bone0 influance
	float m_fBoneWeight1;		// DLH - How strong the is the bone0 influance
#endif

};

struct Asura_Direct3D_BGR_Type_VertexGeometryLeaf
{
public:
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
	float m_afPosition[ 3];		// DLH - 3D Center of the Leaf
	uint32 m_uColour;			// DLH - 
	float m_afTextcoord0[ 2];	// DLH - Texture coord of the corner
	float m_afCorner[ 2];		// DLH - 2D Width and Height offset of the leaf corner. This is in tree space scale.
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	float m_afPosition[ 3];		// DLH - 3D Center of the Leaf
	float m_afNormal[3];		// DLH - Billboards corner Normal
	float m_afTangent[3];		// DLH - TextureUV Tangent
	float m_fBoneWeight1;		// DLH - How strong the is the bone1 influance
	float m_afBinormal[3];		// DLH - TextureUV Binormal
	float m_fBoneWeight0;		// DLH - How strong the is the bone0 influance
	float m_afTextcoord0[ 2];	// DLH - Texture coord0 of the corner
	float m_afTextcoord1[ 2];	// DLH - Texture coord1 of the corner
	uint32 m_uColour;			// DLH - 
	float m_afCorner[ 2];		// DLH - 2D Width and Height offset of the leaf corner. This is in tree space scale.
#endif
};
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
struct Asura_Direct3D_BGR_Type_VertexGeometryImposterStorage
{
public:
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

public:
	Asura_AVector3 m_xDirection;
	float m_fFieldOfViewDotOutter;
	float m_fFieldOfViewAngleOutter;
	float m_fFieldOfViewAngleInner;
	
	float m_fWidth;
	float m_fHeight;
	float m_afTextcoord0[2];
	float m_afTextcoord1[2];
	
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
	Asura_Hash_ID m_uMaterialResponseHashID;
#else
	int32 m_iTextureDiffuse;
	int32 m_iTextureBump;
#endif
};
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
struct Asura_Direct3D_BGR_Type_VertexGeometryImposter
{
public:
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
	float m_afPosition[ 3];		// DLH - 3D Center of the Leaf
	uint32 m_uColour;			// DLH - 
	float m_afUp[3];
	uint32 m_uSphericalR;			// DLH - 
	uint32 m_uSphericalG;			// DLH - 
	uint32 m_uSphericalB;			// DLH - 
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	float m_afPosition[3];		// DLH - 3D Center of the Leaf
	float m_afUp[3];

	float m_afRedSH[4];
	float m_afGreenSH[4];
	float m_afBlueSH[4];

	uint8 m_ucCorner;			// DLH - Corner Index w
	uint8 m_uDummy[3];
#endif
};
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
struct Asura_Direct3D_BGR_Type_VertexGeometryImposterInteractive
{
public:
	float m_afBonePosition[4];
};
#endif


class Asura_Direct3D_BGR_Type_GeometryLODRange
{
public:
	Asura_Direct3D_BGR_Type_GeometryLODRange();
	
	bool IsValid() const;
	bool IsInRange( const float fDelta) const;
	
	// DLH - Returns a 0-1 value 
	float CalcStrength(const float fDelta) const;
	
	void CalcScales();
	
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

public:
	enum
	{
		LOW_0 = 0,
		LOW_1 = 1,
		LOW_SCALE = 2,
		HIGH_1 = 3,
		HIGH_0 = 4, 
		HIGH_SCALE = 5,
		DELTA_SIZE
	};
	// DLH - Four values that are a ramp up, flat are, ramp down
	// 1      2    3
	//        /----\
	//       /      \
	//      /        \
	//    1/          \4
	// 0--/            \--
	float m_afDelta[ DELTA_SIZE];
};

class Asura_Direct3D_BGR_Material
{
public:
	Asura_Direct3D_BGR_Material();
	
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
	bool SetMaterialResponseHashID( const Asura_Hash_ID xMaterialResponseHashID);
#else
	bool SetMaterial( const int iMaterialID);
#endif
	
public:
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
	Asura_Hash_ID m_xMaterialResponseHashID;
#else
	int m_iDiffuseTexture0;
	int m_iDiffuseTexture1;
	bool m_bDiffuseDualPassTexture;
	
	int m_iSpecularTexture0;
	int m_iSpecularTexture1;
	bool m_bSpecularDualPassTexture;
	
	bool m_bHasNormalMaps;
	int m_iNormalTexture0;
	int m_iNormalTexture1;
	bool m_bNormalDualPassTexture;
	
	bool m_bParallaxMap;
	bool m_bSelfIlluminating;
	bool m_bSpecular;
	bool m_bOutputAlphaOfTexture;
#endif
};

class Asura_Direct3D_BGR_Type_Geometry
{
public:
	Asura_Direct3D_BGR_Type_Geometry();
	virtual ~Asura_Direct3D_BGR_Type_Geometry();

	Asura_Direct3D_BGR_Type_Geometry& operator =( const Asura_Direct3D_BGR_Type_Geometry& xRHS);

protected:
	bool Process( Asura_Chunk_Stream& xStream);
	bool ProcessIndices( Asura_Chunk_Stream& xStream, Asura_Botanical_Array< uint32>& xArrayIndices);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
	bool ExportIndices( Asura_Chunk_Stream& xStream, const Asura_Botanical_Array< uint32>& xArrayIndices) const;
#endif

private:
	Asura_Direct3D_BGR_Type_Geometry( const Asura_Direct3D_BGR_Type_Geometry& xRHS);
	
	void Init();
	void Clear();
	
	bool CalcStripIndexCounts( uint32 uSizeStrip, const uint16* pusStrip, uint32 &uSizeStripLeading, uint32 &uSizeStripTrailing);

public:
	Direct3D_VertexBuffer* m_pxVertexBuffer;
	Direct3D_IndexBuffer* m_pxIndexBuffer;
	
	Asura_Direct3D_BGR_Type_GeometryLODRange m_xLodRange;
	// DLH - This is a 0 to 1 range alpha cut value for the fully visiable geom
	float m_fAlphaCutValue;
	
//	int32 m_iTextureDiffuse;
//	int32 m_iTextureBump;
	Asura_Direct3D_BGR_Material xMaterial;
	
	uint32 m_uBufferInstances;
	
	uint32 m_uVBStart;
	uint32 m_uVBSize;
	
	uint32 m_uIBStart;
	uint32 m_uIBSize;
	
	uint32 m_uPrimitiveCount;
};

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
class Asura_Direct3D_BGR_Type_Geometry3D : public Asura_Direct3D_BGR_Type_Geometry
{
public:
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	Asura_Bounding_Box GetBoundingBox() const;
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif
	
public:
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type_VertexGeometry3DStorage> m_xArrayVertices;
	Asura_Botanical_Array< uint32> m_xArrayIndices;
};
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
class Asura_Direct3D_BGR_Type_GeometryLeaves : public Asura_Direct3D_BGR_Type_Geometry
{
public:
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	Asura_Bounding_Box GetBoundingBox() const;
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif
	
public:
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type_VertexGeometryLeafStorage> m_xArrayVertices;
	Asura_Botanical_Array< uint32> m_xArrayIndices;
};
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
class Asura_Direct3D_BGR_Type_GeometryBillboard : public Asura_Direct3D_BGR_Type_Geometry
{
public:
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	Asura_Bounding_Box GetBoundingBox() const;
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif
	
public:
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type_VertexGeometryImposterStorage> m_xArrayBillboards;
};
#endif

class Asura_Direct3D_BGR_Geometry
{
public:
	Asura_Direct3D_BGR_Geometry();
	~Asura_Direct3D_BGR_Geometry();
	
	void Clear();

	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

public:
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type_Geometry3D> m_xArrayBranchs;
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type_Geometry3D> m_xArrayFronds;
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type_GeometryLeaves> m_xArrayLeaves;
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type_GeometryBillboard> m_xArrayBillboards;
#endif
};

class Asura_Direct3D_BGR_Section_Version
{
public:

	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

public:
#if defined( ASURA_TOOLS)
	Asura_String m_xName;
#endif

	Asura_Direct3D_BGR_Geometry m_xGeometry;
};

class Asura_Direct3D_BGR_Section
{
public:

	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

public:
#if defined( ASURA_TOOLS)
	Asura_String m_xName;
#endif
	Asura_Botanical_Array< u_int> m_xArray_uVersionIDToVersionIndex;
	Asura_Botanical_Array< Asura_BGR_Version_ID> m_xArray_uVersionIndexToVersionID;
	
	Asura_Botanical_Array< Asura_Direct3D_BGR_Section_Version> m_xArray_xVersions;
};

class Asura_Direct3D_BGR_Type : public Asura_BGR_Type
{
public:
	Asura_Direct3D_BGR_Type();
	~Asura_Direct3D_BGR_Type();

	// DLH - Parent Methods	
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	virtual u_int GetSectionCount() const;
	virtual Asura_BGR_Section_ID GetSectionID( const Asura_Hash_ID& xHashID) const;
	virtual Asura_BGR_Version_ID GetVersionID( const Asura_Hash_ID& xHashID) const;
	
	#if defined( ASURA_TOOLS)
		virtual Asura_BGR_Section_ID GetSectionID( const char* pszSectionName) const;
		virtual Asura_BGR_Version_ID GetVersionID( const char* pszVersionName) const;
	#endif
#endif

	// DLH - Own Methods
	void Clear();
	void ClearGeometry();
	
	const uint32 GetTreeId() const { return( this->m_xTypeID.GetID());}
	
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

	void CalcBoundingZeroRadius();

public:
#if defined( ASURA_TOOLS)
	bool m_bHidden;
#endif	

	Asura_Botanical_Array< Asura_Direct3D_BGR_Section> m_xArray_xSections;
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	Asura_TemplateHashTree< Asura_BGR_Section_ID> m_xHashTree_xSectionID;
	Asura_TemplateHashTree< Asura_BGR_Version_ID> m_xHashTree_xVersionID;
#endif
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	bool m_bHasImposters;
#endif
	
//	Asura_Bounding_Box m_xBoundingBox;
//	float m_fBoundingRadius;
	float m_fLodLimitNear;
	float m_fLodLimitFar;
	float m_fLodLimitDelta;
	float m_fLodLimitDeltaImposterNear;
	float m_fLodAuthoredFOVWidth;
	float m_fLodAuthoredRes;
	uint32 m_uInstanceMaxCount;
	
};

class Asura_Direct3D_BGR_Instance : public Asura_BGR_Instance
{
public:
};

class Asura_Direct3D_BGR_Instances_Header : public Asura_BGR_Instances_Header
{
public:
	// DLH - 01 November 2006 14:57:40 - General Instance Procssing
	virtual u_int GetInstanceCount() const { return( this->m_xArrayInstances.Size());}
	virtual const Asura_BGR_Instance* GetInstance( const uint32 uInstanceIndex) const;
	
	Asura_Direct3D_BGR_Instance* GetDirect3DInstance( const uint32 uInstanceIndex);
	const Asura_Direct3D_BGR_Instance* GetDirect3DInstance( const uint32 uInstanceIndex) const;
	
	// DLH - 01 November 2006 14:57:25 - Export/Import
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

#if defined( ASURA_TOOLS)
	u_int FindInActiveInstance();
	void ExtendBoundingBox( const Asura_Direct3D_BGR_Type* pxBGRType, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const float fScale);
	bool RecalculateBoundingBox( const Asura_Direct3D_BGR_Type& xType);
#endif

public:
//	Asura_BGR_TypeID m_xTypeID;
	Asura_Botanical_Array< Asura_Direct3D_BGR_Instance> m_xArrayInstances;
    Asura_Botanical_Array<Asura_Static_Visibility_Query> m_xInstanceVisQueries;
	
//	Asura_Bounding_Box m_xBoundingBox;
	
	uint32 m_uImposterVBOffset;
	uint32 m_uImposterVBStart;
	uint32 m_uImposterVBSize;
	uint32 m_uImposterIBStart;
	uint32 m_uImposterIBSize;
};

class Asura_Direct3D_BGR_Module
{
public:
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
#endif

public:
	uint32 m_uInstanceHeaderStartIndex;
	uint32 m_uInstanceHeadeCount;
};

template< class Tvertex, class Tindex>
class Asura_Direct3D_BGR_D3DBuffer
{
public:
	Asura_Direct3D_BGR_D3DBuffer();
	
	bool Clear();
	bool ClearDirect3DData();
	
	bool CreateDirect3DData( const bool bDynamic = false);
	
	bool Lock( const bool bDynamic = false);
	bool Unlock();	
	
	void GetBufferBytes( uint32& uVertexBytes, uint32& uIndexBytes) const;
	
public:
	uint32 m_uVBOffset;
	uint32 m_uVBSize;
	uint32 m_uIBSize;
	
	Direct3D_VertexBuffer* m_pxVertexBuffer;
	Direct3D_IndexBuffer* m_pxIndexBuffer;
	
	Tvertex* m_pxArrayVertex;
	Tindex* m_pxArrayIndex;
};

class Asura_Direct3D_BGR_DynamicLightCache
{
public:
	Asura_Direct3D_BGR_DynamicLightCache();
	
	bool ClearDirect3DData();
	bool CreateDirect3DData();

	void GetCacheGeometrys( Asura_Direct3D_BGR_GeometryVertexCache* &pxCacheVertexGeometrys) { pxCacheVertexGeometrys = &(this->m_xCacheVertexGeometrys); }
	void GetCacheGeometrys( Asura_Direct3D_BGR_GeometryIndexCache* &pxCacheIndexGeometrys) { pxCacheIndexGeometrys = &(this->m_xCacheIndexGeometrys); }
	
public:
	Asura_Direct3D_BGR_Cache m_xCacheInstances;
	Asura_Direct3D_BGR_GeometryVertexCache m_xCacheVertexGeometrys;
	Asura_Direct3D_BGR_GeometryIndexCache m_xCacheIndexGeometrys;
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	Asura_DIPCache_Static< 50> m_xDIPCache;
#endif
	const Asura_Photon* m_pxPhoton;
	
};

class Asura_Direct3D_BGR_RenderCache
{
public:
	Asura_Direct3D_BGR_RenderCache();
	
	void Reset();
	void Clear();
	void ClearInstances( const Asura_Direct3D_BGR_Type* pxTreeDescription);
	bool AddPhotonToDynamicLightCache( const Asura_Photon* pxPhoton);
	
	bool ClearDirect3DData();
	bool CreateDirect3DData();
	
	void GetCacheGeometrys( Asura_Direct3D_BGR_GeometryVertexCache* &pxCacheVertexGeometrys) { pxCacheVertexGeometrys = &(this->m_xCacheVertexGeometrys); }
	void GetCacheGeometrys( Asura_Direct3D_BGR_GeometryIndexCache* &pxCacheIndexGeometrys) { pxCacheIndexGeometrys = &(this->m_xCacheIndexGeometrys); }
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	void CreateSectionCache( const Asura_Direct3D_BGR_Manager& xBGRManager);
#endif

public:
	bool m_bDynamicLightInstancesExist;
	
	Asura_Botanical_Array< Asura_Direct3D_BGR_DynamicLightCache> m_xArray_xDynamicLightCache;
	Asura_Direct3D_BGR_Cache m_xCacheInstances;
	Asura_Direct3D_BGR_GeometryVertexCache m_xCacheVertexGeometrys;
	Asura_Direct3D_BGR_GeometryIndexCache m_xCacheIndexGeometrys;
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	Asura_DIPCache_Static< 50> m_xDIPCache;
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	Asura_Botanical_Array< Asura_Botanical_Array< bool> > m_xArraySection_xArrayVersion_bUsed;
#endif
};

class Asura_Direct3D_BGR_Manager : public Asura_BGR_Manager
{
public:
	Asura_Direct3D_BGR_Manager();
	~Asura_Direct3D_BGR_Manager();
	
// DLH - 30 November 2006 11:08:33 - Parent Mehods
	virtual u_int GetInstanceHeaderCount( void) const;
	virtual const Asura_BGR_Instances_Header* GetInstanceHeader( const u_int uInstanceHeaderIndex) const;
	virtual Asura_BGR_Instances_Header* GetInstanceHeader( const u_int uInstanceHeaderIndex);
	virtual Asura_BGR_Type* GetType( const Asura_BGR_TypeID xTypeID);

	virtual void Platform_Reset();

// DLH - 30 November 2006 11:08:35 - Own Methods 
	void Clear();
	bool ClearDirect3DData();
	
	bool CreateDirect3DData();
	
	Asura_Direct3D_BGR_Instances_Header* CreateInstanceHeader( const uint32 uInstanceCount);
	void DeleteInstanceHeader( Asura_Direct3D_BGR_Instances_Header*& pxInstanceHeader);
	
	void ClearInstanceFromTypeCache( void);
	void SetupTypeCache();
	void AddToCache( const Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader);
	void AddToCache( const Asura_Botanical_Array< Asura_Direct3D_BGR_Instances_Header*>& xArray_pxInstanceHeader);
	void DrawCache();
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	void SetupSectionCache( void);
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
//	bool Update( const float fTime);
#endif	
	
	Asura_Direct3D_BGR_Type* GetDirect3DType( const Asura_BGR_TypeID& xTypeID);
	uint32 GetTypeIDCeiling(void) const;
	
	static uint32 GetTypeFlags();
	
	bool Process( Asura_Chunk_Stream& xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_Stream& xStream) const;
	u_int GetModuleIndexFromPosition( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation);
#endif
	
#if defined( ASURA_TOOLS) && defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	void CreateToolObjectInBGR( Tools_Botanicals_Object* pxToolsObject);
    bool FindExistingInstance( Tools_Botanicals_Object* pxToolsObject);
	void MoveInstanceTo( Tools_Botanicals_Object* pxToolsObject, Asura_Direct3D_BGR_Instances_Header*& pxInstanceHeaderFound, u_int& uBGRInstanceHeaderInstanceIndexFound);
	Asura_Direct3D_BGR_Instances_Header* GetModuleTypeInstanceHeader( const u_int uModuleIndex, const Asura_BGR_TypeID& xTypeID);
	bool AddMissingTypeToModuleInstanceHeaderArrays();
	bool DeleteTypeID( const Asura_BGR_TypeID& xTypeID);
	bool InstanceHeaderTypeChanged( const Asura_Direct3D_BGR_Type& xType);
#endif
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	static bool DrawTreeBillboardInstancesDIPCache( const Asura_DIPCache_DIPData& xDIPData, const uint32 uCallbackData);
#endif
	
	void InvalidateDirect3DData() { this->m_bDirect3DImpostersOutofDate = true; }

	Asura_Mutex& GetLoadingMutex() { return m_xLoadingMutex; }

private:
	void Init();
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	typedef uint32 (Asura_Direct3D_BGR_Manager::*ptrDrawTreeVertexInstances)( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, Asura_Direct3D_BGR_GeometryVertexInstanceCache& xCacheGeometryInstances, const void* pxDataPointer, Asura_Direct3D_Effect* pxD3DEffect);
	typedef uint32 (Asura_Direct3D_BGR_Manager::*ptrDrawTreeIndexInstances)( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, Asura_Direct3D_BGR_GeometryIndexInstanceCache& xCacheGeometryInstances, const void* pxDataPointer, Asura_Direct3D_Effect* pxD3DEffect);
	typedef bool (Asura_Direct3D_BGR_Manager::*ptrDrawTreeRenderStates)( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, const void* pxDataPointer, Asura_Direct3D_Effect*& pxD3DEffect);
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
	typedef uint32 (Asura_Direct3D_BGR_Manager::*ptrDrawTreeInstances)( const Asura_Direct3D_BGR_Type_Geometry &geometry, const struct Asura_Direct3D_BGR_GeometryInstanceConstants *cacheConstants, const uint32 countCacheInstances, const void* pxDataPointer);
	typedef bool (Asura_Direct3D_BGR_Manager::*ptrDrawTreeRenderStates)( const Asura_Direct3D_BGR_Type_Geometry &geometry, const void* pxDataPointer);
#endif

	template< bool bUsePhotonSources> void DrawCache( Asura_TrackingHandlePool<Asura_Photon>* pxPhotonList);
	template< bool bUsePhotonSources, bool bInteractive> void DrawCache_ProcessInstanceHeader( const Asura_Direct3D_BGR_ProcessInstanceHeader& xProcessInstanceHeader, const Asura_Direct3D_BGR_Type* pxTreeDescription, Asura_Direct3D_BGR_RenderCache& xRenderCache, const Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader, const u_int uIndexTreeBatchStart, const u_int uIndexTreeBatchStop);

	bool ResetRenderStates( void);

	bool DrawTreeBillboardInstancesDIPCacheRenderStates_Start( const Asura_Direct3D_BGR_Type& xTreeDescription, const float fScreenDistanceScale, const Asura_Direct3D_BGR_Type_Geometry &geometry, Asura_Direct3D_Effect*& pxD3DEffect);
	bool DrawTreeBillboardInstancesDIPCacheRenderStates_End( Asura_Direct3D_Effect* pxD3DEffect);
	
	bool DrawTreeBillboardDynamicRenderStates( const Asura_Photon& xPhoton, Asura_Direct3D_Effect*& pxD3DEffect);
	
	uint32 DrawTreeBillboardInstances( const Asura_Direct3D_BGR_Type_Geometry &geometry, const struct Asura_Direct3D_BGR_GeometryInstanceConstants *cacheConstants, const uint32 countCacheInstances, const void* pxDataPointer);

//	template< class Tgeometry> void DrawBillboards( const Asura_Botanical_Array< Tgeometry> &arrayGeometry, const Asura_Direct3D_BGR_Cache &cacheInstances, ptrDrawTreeRenderStates funcRenderStates, ptrDrawTreeVertexInstances funcDraw, const void* pxDataPointer);
	
	template< const bool bUsePhotonSources> void DrawCache( const Asura_Direct3D_BGR_Type* treeDescription, Asura_Direct3D_BGR_RenderCache &xRenderCache);
	
	// DLH - 23 October 2006 10:45:54 - SM30 Headers
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	bool DrawTreeBillboardInstancesDIPCacheRenderStates_Start_IB( const Asura_Direct3D_BGR_Type& xTreeDescription, const float fScreenDistanceScale, const Asura_Direct3D_BGR_Type_Geometry &geometry, Asura_Direct3D_Effect*& pxD3DEffect);
	bool DrawTreeBillboardInstancesDIPCacheRenderStates_Start_All( const Asura_Direct3D_BGR_Type& xTreeDescription, const float fScreenDistanceScale, const Asura_Direct3D_BGR_Type_Geometry &geometry, Asura_Direct3D_Effect*& pxD3DEffect);

	bool ClearDirect3DData_SM30();
	bool CreateDirect3DData_SM30();
	
	bool Draw_SM30_GlobalRenderStates( void);
	bool Draw_SM30_StaticLightRenderStates();
	bool Draw_SM30_DynamicLightRenderStates( const Asura_Photon& xPhoton);

	template< class Tgeometry > void DrawTree_SM30_MaterialRenderStates( const Tgeometry &tGeometry, Asura_Direct3D_Effect* pxEffect);
	
	bool DrawTree_SM30_ImposterRenderStates( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, const void* pxDataPointer, Asura_Direct3D_Effect*& pxD3DEffect);
	bool DrawTree_SM30_LeafRenderStates( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, const void* pxDataPointer, Asura_Direct3D_Effect*& pxD3DEffect);
	bool DrawTree_SM30_3D1SidedRenderStates( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, const void* pxDataPointer, Asura_Direct3D_Effect*& pxD3DEffect);
	bool DrawTree_SM30_3D2SidedRenderStates( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, const void* pxDataPointer, Asura_Direct3D_Effect*& pxD3DEffect);

	uint32 DrawTree_SM30_ImposterInstances( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, Asura_Direct3D_BGR_GeometryIndexInstanceCache& xCacheGeometryInstances, const void* pxDataPointer, Asura_Direct3D_Effect* pxD3DEffect);
	uint32 DrawTree_SM30_LeafInstances( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, Asura_Direct3D_BGR_GeometryVertexInstanceCache& xCacheGeometryInstances, const void* pxDataPointer, Asura_Direct3D_Effect* pxD3DEffect);
	uint32 DrawTree_SM30_3D1SidedInstances( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, Asura_Direct3D_BGR_GeometryVertexInstanceCache& xCacheGeometryInstances, const void* pxDataPointer, Asura_Direct3D_Effect* pxD3DEffect);
	uint32 DrawTree_SM30_3D2SidedInstances( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Direct3D_BGR_Type_Geometry &geometry, Asura_Direct3D_BGR_GeometryVertexInstanceCache& xCacheGeometryInstances, const void* pxDataPointer, Asura_Direct3D_Effect* pxD3DEffect);

	template< class Tgeometry, class TgeometryCache, class TgeometryInstanceCache> bool DrawCacheSM30_LockGeometryCache(  const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, TgeometryCache& xCacheGeometrys);
	template< class Tgeometry> bool DrawCacheSM30_LockGeometryCache(  const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, Asura_Direct3D_BGR_GeometryVertexCache& xCacheGeometrys);
	template< class Tgeometry> bool DrawCacheSM30_LockGeometryCache(  const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, Asura_Direct3D_BGR_GeometryIndexCache& xCacheGeometrys);

	template< class Tgeometry, class TgeometryCache, class TgeometryInstanceCache> void DrawCacheSM30_UnlockGeometryCache(  const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, TgeometryCache& xCacheGeometrys);
	template< class Tgeometry> void DrawCacheSM30_UnlockGeometryCache(  const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, Asura_Direct3D_BGR_GeometryVertexCache& xCacheGeometrys);
	template< class Tgeometry> void DrawCacheSM30_UnlockGeometryCache(  const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, Asura_Direct3D_BGR_GeometryIndexCache& xCacheGeometrys);

	template< class Tgeometry, class TgeometryCache, class TgeometryInstanceCache> void DrawCacheSM30_ClearGeometryCache(  const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, TgeometryCache& xCacheGeometrys);
	template< class Tgeometry> void DrawCacheSM30_ClearGeometryCache(  const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, Asura_Direct3D_BGR_GeometryIndexCache& xCacheGeometrys);
	template< class Tgeometry> void DrawCacheSM30_ClearGeometryCache(  const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, Asura_Direct3D_BGR_GeometryVertexCache& xCacheGeometrys);
	
	template< const bool bUseSections> void DrawCacheSM30_FillGeometryCache( const Asura_Botanical_Array< Asura_Direct3D_BGR_Type_GeometryBillboard>& xArrayGeometry, const Asura_BGR_Section_ID xGeometrySection, const Asura_BGR_Version_ID xGeometrySectionVersion, Asura_Direct3D_BGR_Cache& xCacheInstances, Asura_Direct3D_BGR_GeometryIndexCache& xCacheGeometrys);
	template< class Tgeometry, class TgeometryCache, class TgeometryInstanceCache, const bool bUseSections> void DrawCacheSM30_FillGeometryCache( const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, const Asura_BGR_Section_ID xGeometrySection, const Asura_BGR_Version_ID xGeometrySectionVersion, Asura_Direct3D_BGR_Cache& xCacheInstances, TgeometryCache& xCacheGeometrys);
	template< class Tgeometry, const bool bUseSections> void DrawCacheSM30_FillGeometryCache( const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, const Asura_BGR_Section_ID xGeometrySection, const Asura_BGR_Version_ID xGeometrySectionVersion, Asura_Direct3D_BGR_Cache& xCacheInstances, Asura_Direct3D_BGR_GeometryVertexCache& xCacheGeometrys);
	template< class Tgeometry, const bool bUseSections> void DrawCacheSM30_FillGeometryCache( const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, const Asura_BGR_Section_ID xGeometrySection, const Asura_BGR_Version_ID xGeometrySectionVersion, Asura_Direct3D_BGR_Cache& xCacheInstances, Asura_Direct3D_BGR_GeometryIndexCache& xCacheGeometrys);

	template< class Tgeometry, class TgeometryCache, class TgeometryInstanceCache, class TptrDrawTreeInstances> void DrawCacheSM30_DrawGeometryCache( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, const u_int uIndexGeometry, TgeometryCache& xCacheGeometrys, ptrDrawTreeRenderStates funcRenderStates, TptrDrawTreeInstances funcDraw, const void* pxDataPointer);
	template< class Tgeometry, class TptrDrawTreeInstances> void DrawCacheSM30_DrawGeometryCache( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, const u_int uIndexGeometry, Asura_Direct3D_BGR_GeometryVertexCache& xCacheGeometrys, ptrDrawTreeRenderStates pfnRenderStates, TptrDrawTreeInstances pfnDraw, const void* pxDataPointer);
	template< class Tgeometry, class TptrDrawTreeInstances> void DrawCacheSM30_DrawGeometryCache( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, const u_int uIndexGeometry, Asura_Direct3D_BGR_GeometryIndexCache& xCacheGeometrys, ptrDrawTreeRenderStates pfnRenderStates, TptrDrawTreeInstances pfnDraw, const void* pxDataPointer);

	template< class Tgeometry, class TgeometryCache, class TptrDrawTreeInstances, const bool bUsePhotonSources, const bool bUseSections> void DrawCacheSM30( const Asura_Direct3D_BGR_Type& xTreeDescription, const Asura_Botanical_Array< Tgeometry> &arrayGeometry, const Asura_BGR_Section_ID xGeometrySection, const Asura_BGR_Version_ID xGeometrySectionVersion, Asura_Direct3D_BGR_RenderCache &xRenderCache, ptrDrawTreeRenderStates funcRenderStates, TptrDrawTreeInstances funcDraw);
#endif

	// DLH - 23 October 2006 10:45:54 - SM11 Headers
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
	bool Draw_SM11_GlobalRenderStates( void);
	bool Draw_SM11_StaticLightRenderStates();
	bool Draw_SM11_DynamicLightRenderStates( const Asura_Photon& xPhoton);
	
	bool DrawTree_SM11_LeafRenderStates( const Asura_Direct3D_BGR_Type_Geometry &geometry, const void* pxDataPointer);
	bool DrawTree_SM11_3D1SidedRenderStates( const Asura_Direct3D_BGR_Type_Geometry &geometry, const void* pxDataPointer);
	bool DrawTree_SM11_3D2SidedRenderStates( const Asura_Direct3D_BGR_Type_Geometry &geometry, const void* pxDataPointer);

	uint32 DrawTree_SM11_LeafInstances( const Asura_Direct3D_BGR_Type_Geometry &geometry, const struct Asura_Direct3D_BGR_GeometryInstanceConstants *cacheConstants, const uint32 countCacheInstances, const void* pxDataPointer);
	uint32 DrawTree_SM11_3D1SidedInstances( const Asura_Direct3D_BGR_Type_Geometry &geometry, const struct Asura_Direct3D_BGR_GeometryInstanceConstants *cacheConstants, const uint32 countCacheInstances, const void* pxDataPointer);
	uint32 DrawTree_SM11_3D2SidedInstances( const Asura_Direct3D_BGR_Type_Geometry &geometry, const struct Asura_Direct3D_BGR_GeometryInstanceConstants *cacheConstants, const uint32 countCacheInstances, const void* pxDataPointer);

	template< class Tgeometry, class TgeometryCache> void DrawCacheSM11_ClearGeometryCache(  const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, TgeometryCache& xCacheGeometrys);
	template< class Tgeometry, class TgeometryCache> void DrawCacheSM11_FillGeometryCache( const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, Asura_Direct3D_BGR_Cache& xCacheInstances, TgeometryCache& xCacheGeometrys);
	template< class Tgeometry, class TgeometryCache> void DrawCacheSM11_DrawGeometryCache( const Asura_Botanical_Array< Tgeometry>& xArrayGeometry, const u_int uIndexGeometry, const TgeometryCache& xCacheGeometrys, ptrDrawTreeRenderStates funcRenderStates, ptrDrawTreeInstances funcDraw, const void* pxDataPointer);

	template< class Tgeometry, const bool bUsePhotonSources> void DrawCacheSM11( const Asura_Botanical_Array< Tgeometry> &arrayGeometry, Asura_Direct3D_BGR_RenderCache &xRenderCache, ptrDrawTreeRenderStates funcRenderStates, ptrDrawTreeInstances funcDraw);
#endif
	
	// DLH - 23 October 2006 10:57:46 - Data Creation Methods
	bool CreateDirect3DData_Imposters();

	bool CreateDirect3DData_Type( Asura_Direct3D_BGR_Type& xType);
	template< class Tgeometry> bool CreateDirect3DData_Type_ArrayGeometry( const Asura_Direct3D_BGR_Type& xType, Asura_Botanical_Array< Tgeometry>& tArrayGeometry);
	template< class Tgeometry> bool CreateDirect3DData_Type_Geometry( const Asura_Direct3D_BGR_Type& xType, Tgeometry& tGeometry);
	
	bool CreateDirect3DData_Type_CalcBuffers( Asura_Direct3D_BGR_Type& xType);
	template< class Tgeometry> bool CreateDirect3DData_Type_ArrayGeometry_CalcBuffers( const Asura_Direct3D_BGR_Type& xType, Asura_Botanical_Array< Tgeometry>& tArrayGeometry);
	template< class Tgeometry> bool CreateDirect3DData_Type_Geometry_CalcBuffers( const Asura_Direct3D_BGR_Type& xType, Tgeometry& tGeometry);
	
	bool ClearDirect3DData_Type( Asura_Direct3D_BGR_Type& xType);
	template< class Tgeometry> bool ClearDirect3DData_Type_ArrayGeometry( Asura_Botanical_Array< Tgeometry>& tArrayGeometry);
	template< class Tgeometry> bool ClearDirect3DData_Type_Geometry( Tgeometry& tGeometry);
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
	bool CreateDirect3DData_Type_Geometry( const Asura_Direct3D_BGR_Type& xType, Asura_Direct3D_BGR_Type_Geometry3D& xGeometry);
	bool CreateDirect3DData_Type_Geometry_CalcBuffers( const Asura_Direct3D_BGR_Type& xType, Asura_Direct3D_BGR_Type_Geometry3D& xGeometry);
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
	bool CreateDirect3DData_Type_Geometry( const Asura_Direct3D_BGR_Type& xType, Asura_Direct3D_BGR_Type_GeometryLeaves& xGeometry);
	bool CreateDirect3DData_Type_Geometry_CalcBuffers( const Asura_Direct3D_BGR_Type& xType, Asura_Direct3D_BGR_Type_GeometryLeaves& xGeometry);
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	bool ClearDirect3DData_Type_Imposters( Asura_Direct3D_BGR_Type& xType);
	bool ClearDirect3DData_Imposters();
	
	bool CreateDirect3DData_Type_Imposters( Asura_Direct3D_BGR_Type& xType);
	bool CreateDirect3DData_Type_CalcBuffers_Imposters( Asura_Direct3D_BGR_Type& xType);
	bool CreateDirect3DData_Type_Geometry_Imposters( const Asura_Direct3D_BGR_Type& xType);
	bool CreateDirect3DData_Type_Geometry_CalcBuffers_Imposters( const Asura_Direct3D_BGR_Type& xType);
#endif

	void AdjustInstanceDataWithDynamicPhotoEmitter( const Asura_Vector_3& xInstancePosition, const Asura_Direct3D_BGR_Type& xTreeDescription, Asura_Direct3D_BGR_RenderCache& xRenderCache, Asura_Direct3D_BGR_InstanceCache& xInstanceCache);
	
#ifdef ASURA_TOOLS
    struct InstanceBoundingSphere
    {
        Asura_Vector_3 xCentre;
        float fRadius;
    };
    Asura_LinkList<InstanceBoundingSphere> m_xRenderedInstanceBoundingSpheres;

    void RenderDebugInfo( void );
#endif

public:
	
	Asura_Botanical_Array< Asura_Direct3D_BGR_Type> m_xArray_xTypes;
	
	Asura_Botanical_Array< Asura_Direct3D_BGR_Instances_Header*> m_xArray_pxInstanceHeaders;
	Asura_Botanical_Array< Asura_Direct3D_BGR_Module> m_xArray_xModule;
	
	Asura_Direct3D_BGR_RenderCache m_xRenderCache;
	
	Asura_Botanical_Array< Asura_Direct3D_BGR_CacheType > m_xArrayTypeCache;
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
	Asura_Direct3D_BGR_D3DBuffer< Asura_Direct3D_BGR_Type_VertexGeometry3DBasic, uint16> m_xD3DBuffer3DGeomtery;
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
	Asura_Direct3D_BGR_D3DBuffer< Asura_Direct3D_BGR_Type_VertexGeometryLeaf, uint16> m_xD3DBuffer3DLeaves;
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	Asura_Direct3D_BGR_D3DBuffer< Asura_Direct3D_BGR_Type_VertexGeometryImposter, uint16> m_xD3DBufferImposter;
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	Asura_Direct3D_BGR_D3DBuffer< Asura_Direct3D_BGR_Type_VertexGeometryImposterInteractive, uint16> m_xD3DBufferImposterInteractive;
#endif

	int m_iAttenuationMap;

	bool m_bDrawGlobalRenderStatesSet;
	
	bool m_bDirect3DImpostersOutofDate;
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	Direct3D_VertexDeclaration* m_pxVertexDeclaration_3DGeom;
	Direct3D_VertexDeclaration* m_pxVertexDeclaration_Leaf;
	Direct3D_VertexDeclaration* m_pxVertexDeclaration_Imposter;
	Direct3D_VertexDeclaration* m_pxVertexDeclaration_Interactive_Imposter;
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	Asura_Botanical_Array< u_int > m_xArraySection_xVersionCountMax;
#endif

	private:
		Asura_Mutex m_xLoadingMutex; // Mutex to fix issues with Alt-Tabbing whilst loading data.
};

//*******************************************************************
// Inline Function Definitions
//*******************************************************************

#define _ASURA_DIRECT3D_BOTANICALS_GAME_RENDER_MANAGER_H_INLINE_CODE_GUARD_
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Manager_Inlines.h"
#undef _ASURA_DIRECT3D_BOTANICALS_GAME_RENDER_MANAGER_H_INLINE_CODE_GUARD_

//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_BOTANICALS)

#endif // _ASURA_DIRECT3D_BOTANICALS_GAME_RENDER_MANAGER_H_
