//*******************************************************************
//
// File: Asura_Direct3D_Botanicals_Game_Render_Resource.cpp
//
// Created: 13 Mar 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_BOTANICALS)

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && ASURA_DIRECT3D_BGR_SUPPORT_SM30
	#if !defined( ASURA_TARGET_SM3)
		#error I want ASURA_TARGET_SM3 turned on as is ASURA_DIRECT3D_BGR_SUPPORT_SM30 is turned on
	#endif
	#if !defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM ) || 1 != ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		#error I want ASURA_USE_MATERIAL_RESPONSE_SYSTEM turned on as is ASURA_DIRECT3D_BGR_SUPPORT_SM30 is turned on
	#endif
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && ASURA_DIRECT3D_BGR_SUPPORT_SM11
	#if defined( ASURA_TARGET_SM3)
		#error ASURA_TARGET_SM3 is turned on but ASURA_DIRECT3D_BGR_SUPPORT_SM11 is defined this should not happen
	#endif
#endif

// main header
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"

// other libraries
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Manager.h"

// internal dependencies

#include "Core/Asura_LinkList.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Core/Asura_TextureManagement.h"

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************


//*******************************************************************
// Function Definitions
//*******************************************************************

template< class Tgeometry> 
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_Geometry_CalcBuffers( const Asura_Direct3D_BGR_Type& xType, Tgeometry& tGeometry)
{
//	SAFE_RELEASE( tGeometry.m_pxVertexBuffer);
//	SAFE_RELEASE( tGeometry.m_pxIndexBuffer);
	
	tGeometry.m_uBufferInstances = 0;
	
	tGeometry.m_uVBStart = 0;
	tGeometry.m_uVBSize = 0;
	
	tGeometry.m_uIBStart = 0;
	tGeometry.m_uIBSize = 0;
	
	tGeometry.m_uPrimitiveCount = 0;
	
	
	return( true);
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_Geometry_CalcBuffers_Imposters( const Asura_Direct3D_BGR_Type& xType)
{
	const Asura_Botanical_Array< Asura_Direct3D_BGR_Instances_Header*>& xArray_pxInstanceHeaders = this->m_xArray_pxInstanceHeaders;
	const uint32 uInstanceHeaderCount = xArray_pxInstanceHeaders.Size();
	
	for( uint32 uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
	{
		Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
		if( NULL == pxInstanceHeader)
		{
			continue;
		}
		
		if( pxInstanceHeader->GetType() != xType.GetType())
		{
			continue;
		}
		
		Asura_Assert( 0 == pxInstanceHeader->m_uImposterIBStart, "IB Start already setup!");
		Asura_Assert( 0 == pxInstanceHeader->m_uImposterIBSize, "IB Size already setup!");
		Asura_Assert( 0 == pxInstanceHeader->m_uImposterVBOffset, "VB Offset already setup!");
		Asura_Assert( 0 == pxInstanceHeader->m_uImposterVBStart, "VB Start already setup!");
		Asura_Assert( 0 == pxInstanceHeader->m_uImposterVBSize, "VB Size already setup!");
		
#if defined( ASURA_TOOLS)
		const uint32 uInstanceHeaderInstanceCount = pxInstanceHeader->GetInstanceCount();
		uint32 uInstanceCount = 0;
		
		for( uint32 uInstanceHeaderInstanceIndex = 0; uInstanceHeaderInstanceIndex < uInstanceHeaderInstanceCount; uInstanceHeaderInstanceIndex += 1)
		{
			Asura_Direct3D_BGR_Instance* pxInstance = pxInstanceHeader->GetDirect3DInstance( uInstanceHeaderInstanceIndex);
			if( NULL == pxInstance)
			{
				continue;
			}
			if( false == pxInstance->IsActive())
			{
				continue;
			}
			
			uInstanceCount += 1;
		}
#else
		const uint32 uInstanceCount = pxInstanceHeader->GetInstanceCount();
#endif

	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
		{		
			const uint32 uIBSize = 6 * uInstanceCount;
			const uint32 uVBSize = 4 * uInstanceCount;
			
			Asura_Assert( uVBSize <= 0x10000, "Too many imposter vertices");
			
			const uint32 uVBWindowTop = this->m_xD3DBufferImposter.m_uVBSize + uVBSize - this->m_xD3DBufferImposter.m_uVBOffset;
			if( 0x10000 < uVBWindowTop)
			{
				this->m_xD3DBufferImposter.m_uVBOffset = this->m_xD3DBufferImposter.m_uVBSize;
			}
			
			pxInstanceHeader->m_uImposterVBOffset = this->m_xD3DBufferImposter.m_uVBOffset;
			pxInstanceHeader->m_uImposterVBStart = this->m_xD3DBufferImposter.m_uVBSize - pxInstanceHeader->m_uImposterVBOffset;
			pxInstanceHeader->m_uImposterIBStart = this->m_xD3DBufferImposter.m_uIBSize;
			pxInstanceHeader->m_uImposterVBSize = uVBSize;
			pxInstanceHeader->m_uImposterIBSize = uIBSize;
			
			this->m_xD3DBufferImposter.m_uVBSize += uVBSize;
			this->m_xD3DBufferImposter.m_uIBSize += uIBSize;
			
			Asura_Assert( (pxInstanceHeader->m_uImposterIBStart + pxInstanceHeader->m_uImposterIBSize) == this->m_xD3DBufferImposter.m_uIBSize, "Mis calc in the required size of the IB");
			Asura_Assert( (pxInstanceHeader->m_uImposterVBStart + pxInstanceHeader->m_uImposterVBSize + pxInstanceHeader->m_uImposterVBOffset) == this->m_xD3DBufferImposter.m_uVBSize, "Mis calc in the required size of the VB");
		}
	#endif		
		
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
		{
			const uint32 uVBSize = 4 * uInstanceCount;
			
			this->m_xD3DBufferImposterInteractive.m_uVBSize += uVBSize;
			
			Asura_Assert( ((pxInstanceHeader->m_uImposterVBStart + pxInstanceHeader->m_uImposterVBSize + this->m_xD3DBufferImposter.m_uVBOffset)) == this->m_xD3DBufferImposterInteractive.m_uVBSize, "Mis calc in the required size of the VB");
		}
	#endif
		
	}
	
//	xGeometry.m_uBufferInstances = 1;
	
	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_Geometry_CalcBuffers( const Asura_Direct3D_BGR_Type& xType, Asura_Direct3D_BGR_Type_GeometryLeaves& xGeometry)
{
	// DLH - Vertex Information
	xGeometry.m_uVBSize = xGeometry.m_xArrayVertices.Size();
	xGeometry.m_uIBSize = xGeometry.m_xArrayIndices.Size();
	xGeometry.m_uPrimitiveCount = xGeometry.m_uIBSize / 3;
	
	Asura_Assert( (xGeometry.m_uPrimitiveCount * 3) == xGeometry.m_uIBSize, "Index Buffer Size is not multiple of 3. This might not be in Triangle List Format!");

#if defined(ASURA_TARGET_SM3)
	xGeometry.m_uBufferInstances = 1;
#else
	// DLH - 23 March 2006 12:08:46 - 16 bit primitive count
	const uint32 uCountMaxIBSize = 3 * 0x10000;
	// DLH - 23 March 2006 12:08:46 - 16 bit Index Reference
	const uint32 uCountMaxVBSize = 0x10000;
	uint32 uCountMaxVBInstances = uCountMaxVBSize / xGeometry.m_uVBSize;
	uint32 uCountMaxIBInstances = uCountMaxIBSize / xGeometry.m_uIBSize;
	uint32 uCountMaxBInstances = Asura_Maths::Min( uCountMaxVBInstances, uCountMaxIBInstances);

	Asura_Assert( ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE0 < Asura_Direct3D_Caps::GetNumberOfVertexShaderConstants(), "There does notseem to be enought VertexShader Constants!");
	// DLH - 24 March 2006 11:54:44 - Shader constant limits
	const uint32 uCountConsts = Asura_Direct3D_Caps::GetNumberOfVertexShaderConstants() - ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE0;
	const uint32 uCountMaxConstInstances = uCountConsts / ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE_SIZE;
	// DLH - 24 March 2006 11:54:59 - Alpha Value limits
	const uint32 uCountMaxAlphaInstances = 256;
	// DLH - 24 March 2006 11:55:14 - The actual number of instances for the type limits
	const uint32 uCountMaxInstances = xType.m_uInstanceMaxCount;

	xGeometry.m_uBufferInstances = Asura_Maths::Min( uCountMaxBInstances, uCountMaxConstInstances);
	xGeometry.m_uBufferInstances = Asura_Maths::Min( uCountMaxInstances, xGeometry.m_uBufferInstances);
	xGeometry.m_uBufferInstances = Asura_Maths::Min( uCountMaxAlphaInstances, xGeometry.m_uBufferInstances);
	xGeometry.m_uBufferInstances = Asura_Maths::Max( uint32( 1), xGeometry.m_uBufferInstances);
#endif

	xGeometry.m_uVBStart = this->m_xD3DBuffer3DLeaves.m_uVBSize;
	xGeometry.m_uIBStart = this->m_xD3DBuffer3DLeaves.m_uIBSize;
	
	this->m_xD3DBuffer3DLeaves.m_uVBSize += (xGeometry.m_uBufferInstances * xGeometry.m_uVBSize);
	this->m_xD3DBuffer3DLeaves.m_uIBSize += (xGeometry.m_uBufferInstances * xGeometry.m_uIBSize);
	
	Asura_Assert( (xGeometry.m_uIBStart + (xGeometry.m_uBufferInstances * xGeometry.m_uIBSize)) == this->m_xD3DBuffer3DLeaves.m_uIBSize, "Mis calc in the required size of the IB");
	Asura_Assert( (xGeometry.m_uVBStart + (xGeometry.m_uBufferInstances * xGeometry.m_uVBSize)) == this->m_xD3DBuffer3DLeaves.m_uVBSize, "Mis calc in the required size of the VB");
	
	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_Geometry_CalcBuffers( const Asura_Direct3D_BGR_Type& xType, Asura_Direct3D_BGR_Type_Geometry3D& xGeometry)
{
	// DLH - Vertex Information
	xGeometry.m_uVBSize = xGeometry.m_xArrayVertices.Size();
	xGeometry.m_uIBSize = xGeometry.m_xArrayIndices.Size();
	xGeometry.m_uPrimitiveCount = xGeometry.m_uIBSize / 3;
	
	Asura_Assert( (xGeometry.m_uPrimitiveCount * 3) == xGeometry.m_uIBSize, "Index Buffer Size is not multiple of 3. This might not be in Triangle List Format!");

#if defined(ASURA_TARGET_SM3)
	xGeometry.m_uBufferInstances = 1;
#else
	// DLH - 23 March 2006 12:08:46 - 16 bit primitive count
	const uint32 uCountMaxIBSize = 3 * 0x10000;
	// DLH - 23 March 2006 12:08:46 - 16 bit Index Reference
	const uint32 uCountMaxVBSize = 0x10000;
	uint32 uCountMaxVBInstances = uCountMaxVBSize / xGeometry.m_uVBSize;
	uint32 uCountMaxIBInstances = uCountMaxIBSize / xGeometry.m_uIBSize;
	uint32 uCountMaxBInstances = Asura_Maths::Min( uCountMaxVBInstances, uCountMaxIBInstances);

	Asura_Assert( ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE0 < Asura_Direct3D_Caps::GetNumberOfVertexShaderConstants(), "There does notseem to be enought VertexShader Constants!");
	// DLH - 24 March 2006 11:54:44 - Shader constant limits
	const uint32 uCountConsts = Asura_Direct3D_Caps::GetNumberOfVertexShaderConstants() - ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE0;
	const uint32 uCountMaxConstInstances = uCountConsts / ASURA_DIRECT3D_BGR_VS_CONSTANT_INSTANCE_SIZE;
	// DLH - 24 March 2006 11:54:59 - Alpha Value limits
	const uint32 uCountMaxAlphaInstances = 256;
	// DLH - 24 March 2006 11:55:14 - The actual number of instances for the type limits
	const uint32 uCountMaxInstances = xType.m_uInstanceMaxCount;

	xGeometry.m_uBufferInstances = Asura_Maths::Min( uCountMaxBInstances, uCountMaxConstInstances);
	xGeometry.m_uBufferInstances = Asura_Maths::Min( uCountMaxInstances, xGeometry.m_uBufferInstances);
	xGeometry.m_uBufferInstances = Asura_Maths::Min( uCountMaxAlphaInstances, xGeometry.m_uBufferInstances);
	xGeometry.m_uBufferInstances = Asura_Maths::Max( uint32( 1), xGeometry.m_uBufferInstances);
#endif

	xGeometry.m_uVBStart = this->m_xD3DBuffer3DGeomtery.m_uVBSize;
	xGeometry.m_uIBStart = this->m_xD3DBuffer3DGeomtery.m_uIBSize;
	
	this->m_xD3DBuffer3DGeomtery.m_uVBSize += (xGeometry.m_uBufferInstances * xGeometry.m_uVBSize);
	this->m_xD3DBuffer3DGeomtery.m_uIBSize += (xGeometry.m_uBufferInstances * xGeometry.m_uIBSize);
	
	Asura_Assert( (xGeometry.m_uIBStart + (xGeometry.m_uBufferInstances * xGeometry.m_uIBSize)) == this->m_xD3DBuffer3DGeomtery.m_uIBSize, "Mis calc in the required size of the IB");
	Asura_Assert( (xGeometry.m_uVBStart + (xGeometry.m_uBufferInstances * xGeometry.m_uVBSize)) == this->m_xD3DBuffer3DGeomtery.m_uVBSize, "Mis calc in the required size of the VB");
	
	return( true);
}
#endif

template< class Tgeometry> 
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_ArrayGeometry_CalcBuffers( const Asura_Direct3D_BGR_Type& xType, Asura_Botanical_Array< Tgeometry>& tArrayGeometry)
{
	const uint32 uGeometryCount = tArrayGeometry.Size();
	for( uint32 uGeometryIndex = 0; uGeometryIndex < uGeometryCount; uGeometryIndex += 1)
	{
		Tgeometry& tGeometry = tArrayGeometry[ uGeometryIndex];
		
		this->CreateDirect3DData_Type_Geometry_CalcBuffers( xType, tGeometry);
	}
	return( true);
}

bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_CalcBuffers( Asura_Direct3D_BGR_Type& xType)
{
	u_int uSectionCount = xType.m_xArray_xSections.Size();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Asura_Direct3D_BGR_Section& xSection = xType.m_xArray_xSections[ uSectionIndex];
		
		u_int uSectionVersionCount = xSection.m_xArray_xVersions.Size();
		for( u_int uSectionVersionIndex = 0; uSectionVersionIndex < uSectionVersionCount; uSectionVersionIndex += 1)
		{
			Asura_Direct3D_BGR_Section_Version& xSectionVersion = xSection.m_xArray_xVersions[ uSectionVersionIndex];
			Asura_Direct3D_BGR_Geometry& xGeometry = xSectionVersion.m_xGeometry;
	
			#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
				this->CreateDirect3DData_Type_ArrayGeometry_CalcBuffers( xType, xGeometry.m_xArrayBranchs);
				this->CreateDirect3DData_Type_ArrayGeometry_CalcBuffers( xType, xGeometry.m_xArrayFronds);
			#endif
			#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
				this->CreateDirect3DData_Type_ArrayGeometry_CalcBuffers( xType, xGeometry.m_xArrayLeaves);
			#endif
		}
	}
	
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
		this->CreateDirect3DData_Type_Geometry_CalcBuffers_Imposters( xType);
	#endif
	
	return( true);
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_CalcBuffers_Imposters( Asura_Direct3D_BGR_Type& xType)
{
	u_int uSectionCount = xType.m_xArray_xSections.Size();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Asura_Direct3D_BGR_Section& xSection = xType.m_xArray_xSections[ uSectionIndex];
		
		u_int uSectionVersionCount = xSection.m_xArray_xVersions.Size();
		for( u_int uSectionVersionIndex = 0; uSectionVersionIndex < uSectionVersionCount; uSectionVersionIndex += 1)
		{
			Asura_Direct3D_BGR_Section_Version& xSectionVersion = xSection.m_xArray_xVersions[ uSectionVersionIndex];
			Asura_Direct3D_BGR_Geometry& xGeometry = xSectionVersion.m_xGeometry;
	
			this->CreateDirect3DData_Type_ArrayGeometry_CalcBuffers( xType, xGeometry.m_xArrayBillboards);
	
		}
	}
	
	return( true);
}
#endif

template< class Tgeometry> 
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_Geometry( const Asura_Direct3D_BGR_Type& xType, Tgeometry& tGeometry)
{
	
	return( true);
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_Geometry_Imposters( const Asura_Direct3D_BGR_Type& xType)
{
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	if( NULL == this->m_xD3DBufferImposter.m_pxVertexBuffer || NULL == this->m_xD3DBufferImposter.m_pxIndexBuffer)
	{
		return( false);
	}
	
	Asura_Assert( NULL != this->m_xD3DBufferImposter.m_pxVertexBuffer, "VB Not Setup!");
	Asura_Assert( NULL != this->m_xD3DBufferImposter.m_pxIndexBuffer, "IB Not Setup!");
	
	Asura_Assert( NULL != this->m_xD3DBufferImposter.m_pxArrayVertex, "VB Not Locked!");
	Asura_Assert( NULL != this->m_xD3DBufferImposter.m_pxArrayIndex, "IB Not Locked!");
	
//	Asura_Assert( NULL == xGeometry.m_pxVertexBuffer, "VB Already Setup!");
//	Asura_Assert( NULL == xGeometry.m_pxIndexBuffer, "IB Already Setup!");
	
//	xGeometry.m_pxVertexBuffer = this->m_xD3DBufferImposter.m_pxVertexBuffer;
//	this->m_xD3DBufferImposter.m_pxVertexBuffer->AddRef();
	
//	xGeometry.m_pxIndexBuffer = this->m_xD3DBufferImposter.m_pxIndexBuffer;
//	this->m_xD3DBufferImposter.m_pxIndexBuffer->AddRef();
#endif
	
	const Asura_Botanical_Array< Asura_Direct3D_BGR_Instances_Header*>& xArray_pxInstanceHeaders = this->m_xArray_pxInstanceHeaders;
	const uint32 uInstanceHeaderCount = xArray_pxInstanceHeaders.Size();
	
	for( uint32 uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
	{
		Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
		if( NULL == pxInstanceHeader)
		{
			continue;
		}
		
		if( pxInstanceHeader->GetType() != xType.GetType())
		{
			continue;
		}
		
		const uint32 uInstanceHeaderInstanceCount = pxInstanceHeader->GetInstanceCount();
		
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
		uint16* const pxIBIndex0 = this->m_xD3DBufferImposter.m_pxArrayIndex;
		if( NULL != pxIBIndex0)
		{
			uint16* pxIBIndex = pxIBIndex0 + pxInstanceHeader->m_uImposterIBStart;
			
#if !defined( ASURA_TOOLS)
			Asura_Assert( (pxInstanceHeader->m_uImposterIBStart + (uInstanceHeaderInstanceCount * 6)) <= this->m_xD3DBufferImposter.m_uIBSize, "Mis calc in the required size of the IB");
#endif
			
			uint32 uInstanceIndex = 0;
			for( uint32 uInstanceHeaderInstanceIndex = 0; uInstanceHeaderInstanceIndex < uInstanceHeaderInstanceCount; uInstanceHeaderInstanceIndex += 1)
			{
#if defined( ASURA_TOOLS)
				const Asura_Direct3D_BGR_Instance* pxInstance = pxInstanceHeader->GetDirect3DInstance( uInstanceHeaderInstanceIndex);
				if( false == pxInstance->IsActive())
				{
					continue;
				}
#endif
				uint32 uIndexInstance0 = 4 * uInstanceIndex;
				
				const uint32 uIndex = pxInstanceHeader->m_uImposterVBStart + uIndexInstance0;
				
				Asura_Assert( (this->m_xD3DBufferImposter.m_pxArrayIndex + pxInstanceHeader->m_uImposterIBStart) <= pxIBIndex, "Pointer is not in valid IB Range");
				Asura_Assert( (pxIBIndex+ 6) <= (this->m_xD3DBufferImposter.m_pxArrayIndex + pxInstanceHeader->m_uImposterIBStart + pxInstanceHeader->m_uImposterIBSize), "Pointer is not in valid IB Range");
				Asura_Assert( (uIndex + 4) <= (0x10000), "Vertex Index is larger then 0x10000");
				
				pxIBIndex[0] = static_cast< uint16 >( uIndex + 2);
				pxIBIndex[1] = static_cast< uint16 >( uIndex + 1);
				pxIBIndex[2] = static_cast< uint16 >( uIndex + 0);
				pxIBIndex[3] = static_cast< uint16 >( uIndex + 1);
				pxIBIndex[4] = static_cast< uint16 >( uIndex + 2);
				pxIBIndex[5] = static_cast< uint16 >( uIndex + 3);
				
				pxIBIndex += 6;
				uInstanceIndex += 1;
			}
		}
#endif
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)	
		{
			const bool bHasDynamicInstanceData = pxInstanceHeader->HasDynamicInteractiveData();
			if( true == bHasDynamicInstanceData)
			{
				Asura_Direct3D_BGR_Type_VertexGeometryImposterInteractive* const pxVBVertex0 = this->m_xD3DBufferImposterInteractive.m_pxArrayVertex;
				if( NULL != pxVBVertex0)
				{
					Asura_Direct3D_BGR_Type_VertexGeometryImposterInteractive* pxVBVertex = pxVBVertex0 + (pxInstanceHeader->m_uImposterVBStart + pxInstanceHeader->m_uImposterVBOffset);
					
					for( uint32 uInstanceHeaderInstanceIndex = 0; uInstanceHeaderInstanceIndex < uInstanceHeaderInstanceCount; uInstanceHeaderInstanceIndex += 1)
					{
	#if defined( ASURA_TOOLS)
						const Asura_Direct3D_BGR_Instance* pxInstance = pxInstanceHeader->GetDirect3DInstance( uInstanceHeaderInstanceIndex);
						if( false == pxInstance->IsActive())
						{
							continue;
						}
	#endif
						Asura_BGR_DynamicInstance* const pxDynamicInstance = pxInstanceHeader->GetDynamicInstanceData( uInstanceHeaderInstanceIndex);
						Asura_BGR_InteractiveBasic* const pxInteractiveBasic = pxInstanceHeader->GetDynamicInteractiveData( pxDynamicInstance);
						
						pxVBVertex->m_afBonePosition[ 0] = pxInteractiveBasic->m_afPosition[ 0];
						pxVBVertex->m_afBonePosition[ 1] = pxInteractiveBasic->m_afPosition[ 1];
						pxVBVertex->m_afBonePosition[ 2] = pxInteractiveBasic->m_afPosition[ 2];
						pxVBVertex->m_afBonePosition[ 3] = 1.0f;
						
						pxVBVertex += 1;
						
						pxVBVertex->m_afBonePosition[ 0] = pxInteractiveBasic->m_afPosition[ 0];
						pxVBVertex->m_afBonePosition[ 1] = pxInteractiveBasic->m_afPosition[ 1];
						pxVBVertex->m_afBonePosition[ 2] = pxInteractiveBasic->m_afPosition[ 2];
						pxVBVertex->m_afBonePosition[ 3] = 1.0f;
						
						pxVBVertex += 1;
						
						pxVBVertex->m_afBonePosition[ 0] = pxInteractiveBasic->m_afPosition[ 0];
						pxVBVertex->m_afBonePosition[ 1] = pxInteractiveBasic->m_afPosition[ 1];
						pxVBVertex->m_afBonePosition[ 2] = pxInteractiveBasic->m_afPosition[ 2];
						pxVBVertex->m_afBonePosition[ 3] = 1.0f;
						
						pxVBVertex += 1;
						
						pxVBVertex->m_afBonePosition[ 0] = pxInteractiveBasic->m_afPosition[ 0];
						pxVBVertex->m_afBonePosition[ 1] = pxInteractiveBasic->m_afPosition[ 1];
						pxVBVertex->m_afBonePosition[ 2] = pxInteractiveBasic->m_afPosition[ 2];
						pxVBVertex->m_afBonePosition[ 3] = 1.0f;
						
						pxVBVertex += 1;
					}
				}
			}
		}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)	
		Asura_Direct3D_BGR_Type_VertexGeometryImposter* const pxVBVertex0 = this->m_xD3DBufferImposter.m_pxArrayVertex;
		if( NULL != pxVBVertex0)
		{
			Asura_Direct3D_BGR_Type_VertexGeometryImposter* pxVBVertex = pxVBVertex0 + pxInstanceHeader->m_uImposterVBStart + pxInstanceHeader->m_uImposterVBOffset;

#if defined( ASURA_ASSERT)
			const uint32 uVertexCount = 4;
#endif
#if !defined( ASURA_TOOLS) && defined( ASURA_ASSERT)
			Asura_Assert( ( pxInstanceHeader->m_uImposterVBOffset + pxInstanceHeader->m_uImposterVBStart + (uInstanceHeaderInstanceCount * uVertexCount)) <= this->m_xD3DBufferImposter.m_uVBSize, "Mis calc in the required size of the VB");
#endif
			
			uint32 uInstanceIndex = 0;
			for( uint32 uInstanceHeaderInstanceIndex = 0; uInstanceHeaderInstanceIndex < uInstanceHeaderInstanceCount; uInstanceHeaderInstanceIndex += 1)
			{
				const Asura_Direct3D_BGR_Instance* pxInstance = pxInstanceHeader->GetDirect3DInstance( uInstanceHeaderInstanceIndex);
#if defined( ASURA_TOOLS)
				if( false == pxInstance->IsActive())
				{
					continue;
				}
#endif

				Asura_AVector3 xPosition;
				pxInstance->GetInstancePosition( xPosition);
				Asura_AVector3 xUp;
				pxInstance->GetInstanceUp( xUp);
				
				// DLH - 26 October 2006 12:10:38 - Leave the Scale in the Vector for the Vertex Shader to calculate and normalise
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
				const float fScale = xUp.Magnitude();
				xUp.Normalise();
#endif
				
				Asura_Assert( pxVBVertex0 <= pxVBVertex && pxVBVertex < (pxVBVertex0 + this->m_xD3DBufferImposter.m_uVBSize), "VB Vertex Pointer out of range");
				
				Asura_Direct3D_BGR_Type_VertexGeometryImposter* const pxVertex = pxVBVertex;
				
#if defined( ASURA_ASSERT)
				Asura_Assert( (pxVertex - pxVBVertex0 + uVertexCount) <= this->m_xD3DBufferImposter.m_uVBSize, "Mis calc in the required size of the VB");
				Asura_Assert( (pxVertex + uVertexCount) <= (this->m_xD3DBufferImposter.m_pxArrayVertex + pxInstanceHeader->m_uImposterVBStart + pxInstanceHeader->m_uImposterVBOffset + pxInstanceHeader->m_uImposterVBSize), "");
#endif

				pxVertex[0].m_afPosition[0] = xPosition.GetX();
				pxVertex[0].m_afPosition[1] = xPosition.GetY();
				pxVertex[0].m_afPosition[2] = xPosition.GetZ();
				pxVertex[0].m_afUp[0] = xUp.GetX();
				pxVertex[0].m_afUp[1] = xUp.GetY();
				pxVertex[0].m_afUp[2] = xUp.GetZ();
				
				pxVertex[1].m_afPosition[0] = xPosition.GetX();
				pxVertex[1].m_afPosition[1] = xPosition.GetY();
				pxVertex[1].m_afPosition[2] = xPosition.GetZ();
				pxVertex[1].m_afUp[0] = xUp.GetX();
				pxVertex[1].m_afUp[1] = xUp.GetY();
				pxVertex[1].m_afUp[2] = xUp.GetZ();
				
				pxVertex[2].m_afPosition[0] = xPosition.GetX();
				pxVertex[2].m_afPosition[1] = xPosition.GetY();
				pxVertex[2].m_afPosition[2] = xPosition.GetZ();
				pxVertex[2].m_afUp[0] = xUp.GetX();
				pxVertex[2].m_afUp[1] = xUp.GetY();
				pxVertex[2].m_afUp[2] = xUp.GetZ();
				
				pxVertex[3].m_afPosition[0] = xPosition.GetX();
				pxVertex[3].m_afPosition[1] = xPosition.GetY();
				pxVertex[3].m_afPosition[2] = xPosition.GetZ();
				pxVertex[3].m_afUp[0] = xUp.GetX();
				pxVertex[3].m_afUp[1] = xUp.GetY();
				pxVertex[3].m_afUp[2] = xUp.GetZ();

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM11) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM11
				{
					const uint32 uScale = static_cast< uint32> (255.0f * fScale / ASURA_DIRECT3D_BGR_INSTANCE_SCALE_MAX);
					uint32 uColorBase =  (uScale << 16);
						
					pxVertex[0].m_uColour = uColorBase | (0 << 24);
					pxVertex[1].m_uColour = uColorBase | (1 << 24);
					pxVertex[2].m_uColour = uColorBase | (2 << 24);
					pxVertex[3].m_uColour = uColorBase | (3 << 24);
				}
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
				{
					pxVertex[0].m_ucCorner = 0;
					pxVertex[1].m_ucCorner = 1;
					pxVertex[2].m_ucCorner = 2;
					pxVertex[3].m_ucCorner = 3;
				}

#endif
				
				{	
					float afSphereicalR[4];
					float afSphereicalG[4];
					float afSphereicalB[4];
					
					pxInstance->GetInstanceSpherical( afSphereicalR, afSphereicalG, afSphereicalB);
					
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30

					for( u_int u = 0; u < 4; ++u )
					{
						// No need to swizzle here, it's already done in Asura_BGR_Instance::Relight
						// and Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_InstanceHeader
						pxVertex[u].m_afRedSH[0] = afSphereicalR[0];
						pxVertex[u].m_afRedSH[1] = afSphereicalR[1];
						pxVertex[u].m_afRedSH[2] = afSphereicalR[2];
						pxVertex[u].m_afRedSH[3] = afSphereicalR[3];

						pxVertex[u].m_afGreenSH[0] = afSphereicalG[0];
						pxVertex[u].m_afGreenSH[1] = afSphereicalG[1];
						pxVertex[u].m_afGreenSH[2] = afSphereicalG[2];
						pxVertex[u].m_afGreenSH[3] = afSphereicalG[3];

						pxVertex[u].m_afBlueSH[0] = afSphereicalB[0];
						pxVertex[u].m_afBlueSH[1] = afSphereicalB[1];
						pxVertex[u].m_afBlueSH[2] = afSphereicalB[2];
						pxVertex[u].m_afBlueSH[3] = afSphereicalB[3];
					}
#else
					const float fScale = 0.25f * 0x80;
					const float fOffset = 0x80;
					
					afSphereicalR[0] = (fScale * afSphereicalR[0]) + fOffset;
					afSphereicalR[1] = (fScale * afSphereicalR[1]) + fOffset;
					afSphereicalR[2] = (fScale * afSphereicalR[2]) + fOffset;
					afSphereicalR[3] = (fScale * afSphereicalR[3]) + fOffset;
					
					afSphereicalG[0] = (fScale * afSphereicalG[0]) + fOffset;
					afSphereicalG[1] = (fScale * afSphereicalG[1]) + fOffset;
					afSphereicalG[2] = (fScale * afSphereicalG[2]) + fOffset;
					afSphereicalG[3] = (fScale * afSphereicalG[3]) + fOffset;
					
					afSphereicalB[0] = (fScale * afSphereicalB[0]) + fOffset;
					afSphereicalB[1] = (fScale * afSphereicalB[1]) + fOffset;
					afSphereicalB[2] = (fScale * afSphereicalB[2]) + fOffset;
					afSphereicalB[3] = (fScale * afSphereicalB[3]) + fOffset;
					
					Asura_Assertf( 0.0f < afSphereicalR[0] && afSphereicalR[0] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalR[0]));
					Asura_Assertf( 0.0f < afSphereicalR[1] && afSphereicalR[1] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalR[1]));
					Asura_Assertf( 0.0f < afSphereicalR[2] && afSphereicalR[2] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalR[2]));
					Asura_Assertf( 0.0f < afSphereicalR[3] && afSphereicalR[3] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalR[3]));
																																			 
					Asura_Assertf( 0.0f < afSphereicalG[0] && afSphereicalG[0] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalG[0]));
					Asura_Assertf( 0.0f < afSphereicalG[1] && afSphereicalG[1] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalG[1]));
					Asura_Assertf( 0.0f < afSphereicalG[2] && afSphereicalG[2] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalG[2]));
					Asura_Assertf( 0.0f < afSphereicalG[3] && afSphereicalG[3] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalG[3]));
																																			 
					Asura_Assertf( 0.0f < afSphereicalB[0] && afSphereicalB[0] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalB[0]));
					Asura_Assertf( 0.0f < afSphereicalB[1] && afSphereicalB[1] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalB[1]));
					Asura_Assertf( 0.0f < afSphereicalB[2] && afSphereicalB[2] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalB[2]));
					Asura_Assertf( 0.0f < afSphereicalB[3] && afSphereicalB[3] < 256.0f, ("Spherical Harmonic Value %f is out of range.", afSphereicalB[3]));
					
					const uint32 uSphericalR = 
						+ (static_cast< uint32>( afSphereicalR[2]) << 0)
						+ (static_cast< uint32>( afSphereicalR[1]) << 8)
						+ (static_cast< uint32>( afSphereicalR[0]) << 16)
						+ (static_cast< uint32>( afSphereicalR[3]) << 24)
						;
					const uint32 uSphericalG = 
						+ (static_cast< uint32>( afSphereicalG[2]) << 0)
						+ (static_cast< uint32>( afSphereicalG[1]) << 8)
						+ (static_cast< uint32>( afSphereicalG[0]) << 16)
						+ (static_cast< uint32>( afSphereicalG[3]) << 24)
						;
					const uint32 uSphericalB = 
						+ (static_cast< uint32>( afSphereicalB[2]) << 0)
						+ (static_cast< uint32>( afSphereicalB[1]) << 8)
						+ (static_cast< uint32>( afSphereicalB[0]) << 16)
						+ (static_cast< uint32>( afSphereicalB[3]) << 24)
						;
						
					pxVertex[0].m_uSphericalR = uSphericalR;
					pxVertex[0].m_uSphericalG = uSphericalG;
					pxVertex[0].m_uSphericalB = uSphericalB;
						
					pxVertex[1].m_uSphericalR = uSphericalR;
					pxVertex[1].m_uSphericalG = uSphericalG;
					pxVertex[1].m_uSphericalB = uSphericalB;
						
					pxVertex[2].m_uSphericalR = uSphericalR;
					pxVertex[2].m_uSphericalG = uSphericalG;
					pxVertex[2].m_uSphericalB = uSphericalB;
						
					pxVertex[3].m_uSphericalR = uSphericalR;
					pxVertex[3].m_uSphericalG = uSphericalG;
					pxVertex[3].m_uSphericalB = uSphericalB;
#endif
				}
				
				pxVBVertex += 4;
				uInstanceIndex += 1;
			}
		}
#endif
	}
	
	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_Geometry( const Asura_Direct3D_BGR_Type& xType, Asura_Direct3D_BGR_Type_GeometryLeaves& xGeometry)
{
	if( 0 == xGeometry.m_uIBSize || 0 == xGeometry.m_uVBSize)
	{
		return( true);
	}
	
	Asura_Assert( NULL != this->m_xD3DBuffer3DLeaves.m_pxVertexBuffer, "VB Not Setup!");
	Asura_Assert( NULL != this->m_xD3DBuffer3DLeaves.m_pxIndexBuffer, "IB Not Setup!");
	
	Asura_Assert( NULL != this->m_xD3DBuffer3DLeaves.m_pxArrayVertex, "VB Not Locked!");
	Asura_Assert( NULL != this->m_xD3DBuffer3DLeaves.m_pxArrayIndex, "IB Not Locked!");
	
	Asura_Assert( NULL == xGeometry.m_pxVertexBuffer, "VB Already Setup!");
	Asura_Assert( NULL == xGeometry.m_pxIndexBuffer, "IB Already Setup!");
	
	if( NULL == this->m_xD3DBuffer3DLeaves.m_pxVertexBuffer || NULL == this->m_xD3DBuffer3DLeaves.m_pxIndexBuffer)
	{
		return( false);
	}
	
	xGeometry.m_pxVertexBuffer = this->m_xD3DBuffer3DLeaves.m_pxVertexBuffer;
	this->m_xD3DBuffer3DLeaves.m_pxVertexBuffer->AddRef();
	
	xGeometry.m_pxIndexBuffer = this->m_xD3DBuffer3DLeaves.m_pxIndexBuffer;
	this->m_xD3DBuffer3DLeaves.m_pxIndexBuffer->AddRef();
	
	uint16* const pxIBIndex0 = this->m_xD3DBuffer3DLeaves.m_pxArrayIndex;
	if( NULL != pxIBIndex0)
	{
		uint16* pxIBIndex = pxIBIndex0 + xGeometry.m_uIBStart;
		
		const uint32 uInstanceCount = xGeometry.m_uBufferInstances;
		const uint32 uIndexCount = xGeometry.m_uIBSize;
	
		Asura_Assert( (xGeometry.m_uIBStart + (uInstanceCount * uIndexCount)) <= this->m_xD3DBuffer3DLeaves.m_uIBSize, "Mis calc in the required size of the IB");
		
		for( uint32 uInstanceIndex = 0; uInstanceIndex < uInstanceCount; uInstanceIndex += 1)
		{
			uint32 uIndexInstance0 = uInstanceIndex * xGeometry.m_uVBSize;
			for( uint32 uIndexIndex = 0; uIndexIndex < uIndexCount; uIndexIndex += 1, pxIBIndex += 1)
			{
				Asura_Assert( pxIBIndex0 <= pxIBIndex && pxIBIndex < (pxIBIndex0 + this->m_xD3DBuffer3DLeaves.m_uIBSize), "IB Index Pointer out of range");
				
				uint32 uIndex = xGeometry.m_xArrayIndices[ uIndexIndex] + uIndexInstance0;
				*pxIBIndex = static_cast< uint16 >( uIndex);
			}
		}
	}
	
	Asura_Direct3D_BGR_Type_VertexGeometryLeaf* const pxVBVertex0 = this->m_xD3DBuffer3DLeaves.m_pxArrayVertex;
	if(NULL != pxVBVertex0)
	{
		Asura_Direct3D_BGR_Type_VertexGeometryLeaf* pxVBVertex = pxVBVertex0 + xGeometry.m_uVBStart;
		
		const uint32 uInstanceCount = xGeometry.m_uBufferInstances;
		const uint32 uVertexCount = xGeometry.m_uVBSize;
		
		Asura_Assert( (xGeometry.m_uVBStart + (uInstanceCount * uVertexCount)) <= this->m_xD3DBuffer3DLeaves.m_uVBSize, "Mis calc in the required size of the VB");
		
		for( uint32 uInstanceIndex = 0; uInstanceIndex < uInstanceCount; uInstanceIndex += 1)
		{
			uint32 uColorBase = ((uInstanceIndex) << 24);
			
			for( uint32 uVertexIndex = 0; uVertexIndex < uVertexCount; uVertexIndex += 1, pxVBVertex += 1)
			{
				Asura_Assert( pxVBVertex0 <= pxVBVertex && pxVBVertex < (pxVBVertex0 + this->m_xD3DBuffer3DLeaves.m_uVBSize), "VB Vertex Pointer out of range");
				
				const Asura_Direct3D_BGR_Type_VertexGeometryLeafStorage& xVertex = xGeometry.m_xArrayVertices[ uVertexIndex];
				
				pxVBVertex->m_afPosition[ 0] = xVertex.m_xPosition.GetX();
				pxVBVertex->m_afPosition[ 1] = xVertex.m_xPosition.GetY();
				pxVBVertex->m_afPosition[ 2] = xVertex.m_xPosition.GetZ();
				
				pxVBVertex->m_afCorner[ 0] = xVertex.m_afCorner[ 0];
				pxVBVertex->m_afCorner[ 1] = xVertex.m_afCorner[ 1];
				
				pxVBVertex->m_afTextcoord0[ 0] = xVertex.m_afTextcoord0[ 0];
				pxVBVertex->m_afTextcoord0[ 1] = xVertex.m_afTextcoord0[ 1];
				
				pxVBVertex->m_uColour = ( 0x00FFFFFF & xVertex.m_uColour) | uColorBase;
				
				#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
					pxVBVertex->m_afTextcoord1[ 0] = xVertex.m_afTextcoord1[ 0];
					pxVBVertex->m_afTextcoord1[ 1] = xVertex.m_afTextcoord1[ 1];
					
					pxVBVertex->m_afNormal[0] = xVertex.m_xNormal.GetX();
					pxVBVertex->m_afNormal[1] = xVertex.m_xNormal.GetY();
					pxVBVertex->m_afNormal[2] = xVertex.m_xNormal.GetZ();

					pxVBVertex->m_afBinormal[0] = xVertex.m_xBinormal.GetX();
					pxVBVertex->m_afBinormal[1] = xVertex.m_xBinormal.GetY();
					pxVBVertex->m_afBinormal[2] = xVertex.m_xBinormal.GetZ();
					pxVBVertex->m_fBoneWeight0 = xVertex.m_fBoneWeight0;

					pxVBVertex->m_afTangent[0] = xVertex.m_xTangent.GetX();
					pxVBVertex->m_afTangent[1] = xVertex.m_xTangent.GetY();
					pxVBVertex->m_afTangent[2] = xVertex.m_xTangent.GetZ();
					pxVBVertex->m_fBoneWeight1 = xVertex.m_fBoneWeight1;
				#endif
			}
		}
	}
	
	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_Geometry( const Asura_Direct3D_BGR_Type& xType, Asura_Direct3D_BGR_Type_Geometry3D& xGeometry)
{
	if( 0 == xGeometry.m_uIBSize || 0 == xGeometry.m_uVBSize)
	{
		return( true);
	}
	
	Asura_Assert( NULL != this->m_xD3DBuffer3DGeomtery.m_pxVertexBuffer, "VB Not Setup!");
	Asura_Assert( NULL != this->m_xD3DBuffer3DGeomtery.m_pxIndexBuffer, "IB Not Setup!");
	
	Asura_Assert( NULL != this->m_xD3DBuffer3DGeomtery.m_pxArrayVertex, "VB Not Locked!");
	Asura_Assert( NULL != this->m_xD3DBuffer3DGeomtery.m_pxArrayIndex, "IB Not Locked!");
	
	Asura_Assert( NULL == xGeometry.m_pxVertexBuffer, "VB Already Setup!");
	Asura_Assert( NULL == xGeometry.m_pxIndexBuffer, "IB Already Setup!");
	
	if( NULL == this->m_xD3DBuffer3DGeomtery.m_pxVertexBuffer || NULL == this->m_xD3DBuffer3DGeomtery.m_pxIndexBuffer)
	{
		return( false);
	}
	
	xGeometry.m_pxVertexBuffer = this->m_xD3DBuffer3DGeomtery.m_pxVertexBuffer;
	this->m_xD3DBuffer3DGeomtery.m_pxVertexBuffer->AddRef();
	
	xGeometry.m_pxIndexBuffer = this->m_xD3DBuffer3DGeomtery.m_pxIndexBuffer;
	this->m_xD3DBuffer3DGeomtery.m_pxIndexBuffer->AddRef();
	
	uint16* const pxIBIndex0 = this->m_xD3DBuffer3DGeomtery.m_pxArrayIndex;
	if( NULL != pxIBIndex0)
	{
		uint16* pxIBIndex = pxIBIndex0 + xGeometry.m_uIBStart;
		
		const uint32 uInstanceCount = xGeometry.m_uBufferInstances;
		const uint32 uIndexCount = xGeometry.m_uIBSize;
	
		Asura_Assert( (xGeometry.m_uIBStart + (uInstanceCount * uIndexCount)) <= this->m_xD3DBuffer3DGeomtery.m_uIBSize, "Mis calc in the required size of the IB");
		
		for( uint32 uInstanceIndex = 0; uInstanceIndex < uInstanceCount; uInstanceIndex += 1)
		{
			uint32 uIndexInstance0 = uInstanceIndex * xGeometry.m_uVBSize;
			for( uint32 uIndexIndex = 0; uIndexIndex < uIndexCount; uIndexIndex += 1, pxIBIndex += 1)
			{
				Asura_Assert( pxIBIndex0 <= pxIBIndex && pxIBIndex < (pxIBIndex0 + this->m_xD3DBuffer3DGeomtery.m_uIBSize), "IB Index Pointer out of range");
				
				uint32 uIndex = xGeometry.m_xArrayIndices[ uIndexIndex] + uIndexInstance0;
				*pxIBIndex = static_cast< uint16 >( uIndex);
			}
		}
	}
	
	Asura_Direct3D_BGR_Type_VertexGeometry3DBasic* const pxVBVertex0 = this->m_xD3DBuffer3DGeomtery.m_pxArrayVertex;
	if( NULL != pxVBVertex0)
	{
		Asura_Direct3D_BGR_Type_VertexGeometry3DBasic* pxVBVertex = pxVBVertex0 + xGeometry.m_uVBStart;
		
		const uint32 uInstanceCount = xGeometry.m_uBufferInstances;
		const uint32 uVertexCount = xGeometry.m_uVBSize;
		
		Asura_Assert( (xGeometry.m_uVBStart + (uInstanceCount * uVertexCount)) <= this->m_xD3DBuffer3DGeomtery.m_uVBSize, "Mis calc in the required size of the VB");
		
		for( uint32 uInstanceIndex = 0; uInstanceIndex < uInstanceCount; uInstanceIndex += 1)
		{
			uint32 uColorBase = ((uInstanceIndex) << 24);
			
			for( uint32 uVertexIndex = 0; uVertexIndex < uVertexCount; uVertexIndex += 1, pxVBVertex += 1)
			{
				Asura_Assert( pxVBVertex0 <= pxVBVertex && pxVBVertex < (pxVBVertex0 + this->m_xD3DBuffer3DGeomtery.m_uVBSize), "VB Vertex Pointer out of range");
				
				const Asura_Direct3D_BGR_Type_VertexGeometry3DStorage& xVertex = xGeometry.m_xArrayVertices[ uVertexIndex];
				
				pxVBVertex->m_afPosition[ 0] = xVertex.m_xPosition.GetX();
				pxVBVertex->m_afPosition[ 1] = xVertex.m_xPosition.GetY();
				pxVBVertex->m_afPosition[ 2] = xVertex.m_xPosition.GetZ();
				
				pxVBVertex->m_afTextcoord0[ 0] = xVertex.m_afTextcoord0[ 0];
				pxVBVertex->m_afTextcoord0[ 1] = xVertex.m_afTextcoord0[ 1];
				
				pxVBVertex->m_uColour = ( 0x00FFFFFF & xVertex.m_uColour) | uColorBase;
				
				#if defined( ASURA_DIRECT3D_BGR_SUPPORT_NORMALS)
					pxVBVertex->m_afNormal[ 0] = xVertex.m_xNormal.GetX();
					pxVBVertex->m_afNormal[ 1] = xVertex.m_xNormal.GetY();
					pxVBVertex->m_afNormal[ 2] = xVertex.m_xNormal.GetZ();
				#endif
				
				#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
					pxVBVertex->m_afTextcoord1[ 0] = xVertex.m_afTextcoord1[ 0];
					pxVBVertex->m_afTextcoord1[ 1] = xVertex.m_afTextcoord1[ 1];

					pxVBVertex->m_afBinormal[0] = xVertex.m_xBinormal.GetX();
					pxVBVertex->m_afBinormal[1] = xVertex.m_xBinormal.GetY();
					pxVBVertex->m_afBinormal[2] = xVertex.m_xBinormal.GetZ();
					pxVBVertex->m_fBoneWeight0 = xVertex.m_fBoneWeight0;

					pxVBVertex->m_afTangent[0] = xVertex.m_xTangent.GetX();
					pxVBVertex->m_afTangent[1] = xVertex.m_xTangent.GetY();
					pxVBVertex->m_afTangent[2] = xVertex.m_xTangent.GetZ();
					pxVBVertex->m_fBoneWeight1 = xVertex.m_fBoneWeight1;
				#endif
			}
		}
	}
	
	return( true);
}
#endif

template< class Tgeometry> 
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_ArrayGeometry( const Asura_Direct3D_BGR_Type& xType, Asura_Botanical_Array< Tgeometry>& tArrayGeometry)
{
	const uint32 uGeometryCount = tArrayGeometry.Size();
	for( uint32 uGeometryIndex = 0; uGeometryIndex < uGeometryCount; uGeometryIndex += 1)
	{
		Tgeometry& tGeometry = tArrayGeometry[ uGeometryIndex];
		
		this->CreateDirect3DData_Type_Geometry( xType, tGeometry);
	}
	return( true);
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type_Imposters( Asura_Direct3D_BGR_Type& xType)
{
	u_int uSectionCount = xType.m_xArray_xSections.Size();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Asura_Direct3D_BGR_Section& xSection = xType.m_xArray_xSections[ uSectionIndex];
		
		u_int uSectionVersionCount = xSection.m_xArray_xVersions.Size();
		for( u_int uSectionVersionIndex = 0; uSectionVersionIndex < uSectionVersionCount; uSectionVersionIndex += 1)
		{
			Asura_Direct3D_BGR_Section_Version& xSectionVersion = xSection.m_xArray_xVersions[ uSectionVersionIndex];
			Asura_Direct3D_BGR_Geometry& xGeometry = xSectionVersion.m_xGeometry;
	
			this->CreateDirect3DData_Type_ArrayGeometry( xType, xGeometry.m_xArrayBillboards);
		}
	}
	
	return( true);
}
#endif

bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Type( Asura_Direct3D_BGR_Type& xType)
{
	u_int uSectionCount = xType.m_xArray_xSections.Size();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Asura_Direct3D_BGR_Section& xSection = xType.m_xArray_xSections[ uSectionIndex];
		
		u_int uSectionVersionCount = xSection.m_xArray_xVersions.Size();
		for( u_int uSectionVersionIndex = 0; uSectionVersionIndex < uSectionVersionCount; uSectionVersionIndex += 1)
		{
			Asura_Direct3D_BGR_Section_Version& xSectionVersion = xSection.m_xArray_xVersions[ uSectionVersionIndex];
			Asura_Direct3D_BGR_Geometry& xGeometry = xSectionVersion.m_xGeometry;
	
		#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
			this->CreateDirect3DData_Type_ArrayGeometry( xType, xGeometry.m_xArrayBranchs);
			this->CreateDirect3DData_Type_ArrayGeometry( xType, xGeometry.m_xArrayFronds);
		#endif
		#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
			this->CreateDirect3DData_Type_ArrayGeometry( xType, xGeometry.m_xArrayLeaves);
		#endif
	
		}
	}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	this->CreateDirect3DData_Type_Geometry_Imposters( xType);
#endif

	return( true);
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
bool Asura_Direct3D_BGR_Manager::CreateDirect3DData_Imposters()
{
	this->ClearDirect3DData_Imposters();
	
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
		this->m_xD3DBufferImposter.Clear();
	#endif
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
		this->m_xD3DBufferImposterInteractive.Clear();
	#endif
	
	uint32 uTypeCount = this->m_xArray_xTypes.Size();
	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_Type& xType = this->m_xArray_xTypes[ uTypeIndex];
		
		this->CreateDirect3DData_Type_CalcBuffers_Imposters( xType);
	}

	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
		this->m_xD3DBufferImposter.CreateDirect3DData();
	#endif
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
		this->m_xD3DBufferImposterInteractive.CreateDirect3DData();
	#endif
	
	
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
		this->m_xD3DBufferImposter.Lock();
	#endif
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
		this->m_xD3DBufferImposterInteractive.Lock();
	#endif
	
	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_Type& xType = this->m_xArray_xTypes[ uTypeIndex];
		
		this->CreateDirect3DData_Type_Imposters( xType);
	}

	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
		this->m_xD3DBufferImposterInteractive.Unlock();
	#endif
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
		this->m_xD3DBufferImposter.Unlock();
	#endif
	
	{	
		Asura_Direct3D_BGR_RenderCache& xRenderCache = this->m_xRenderCache;
		xRenderCache.CreateDirect3DData();
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
		xRenderCache.CreateSectionCache( *this);
#endif
	}

	this->m_bDirect3DImpostersOutofDate = false;
	
	return( true);
}
#endif

bool Asura_Direct3D_BGR_RenderCache::CreateDirect3DData()
{
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	this->m_xCacheVertexGeometrys.CreateDirect3DData();
	this->m_xCacheIndexGeometrys.CreateDirect3DData();
#endif
	
	const u_int uDynamicLightCacheCount = this->m_xArray_xDynamicLightCache.Size();
	for( u_int uDynamicLightCacheIndex = 0; uDynamicLightCacheIndex < uDynamicLightCacheCount; uDynamicLightCacheIndex += 1)
	{
		Asura_Direct3D_BGR_DynamicLightCache& xDynamicLightCache = this->m_xArray_xDynamicLightCache[ uDynamicLightCacheIndex];

		xDynamicLightCache.CreateDirect3DData();
	}
	
	return( true);
}

bool Asura_Direct3D_BGR_Manager::CreateDirect3DData()
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return( false);
	}
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
 	this->ClearDirect3DData_SM30();
 	this->CreateDirect3DData_SM30();
#endif
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
	this->m_xD3DBuffer3DGeomtery.Clear();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
	this->m_xD3DBuffer3DLeaves.Clear();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	this->m_xD3DBufferImposter.Clear();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	this->m_xD3DBufferImposterInteractive.Clear();
#endif
	
	uint32 uTypeCount = this->m_xArray_xTypes.Size();
	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_Type& xType = this->m_xArray_xTypes[ uTypeIndex];
		
		this->CreateDirect3DData_Type_CalcBuffers( xType);
	}


#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
	this->m_xD3DBuffer3DGeomtery.CreateDirect3DData();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
	this->m_xD3DBuffer3DLeaves.CreateDirect3DData();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	this->m_xD3DBufferImposter.CreateDirect3DData();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	this->m_xD3DBufferImposterInteractive.CreateDirect3DData( true);
#endif
	

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	this->m_xD3DBufferImposterInteractive.Lock();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) 
	this->m_xD3DBufferImposter.Lock();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
	this->m_xD3DBuffer3DGeomtery.Lock();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
	this->m_xD3DBuffer3DLeaves.Lock();
#endif
	
	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_Type& xType = this->m_xArray_xTypes[ uTypeIndex];
		
		this->CreateDirect3DData_Type( xType);
	}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
	this->m_xD3DBuffer3DGeomtery.Unlock();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
	this->m_xD3DBuffer3DLeaves.Unlock();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	this->m_xD3DBufferImposter.Unlock();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	this->m_xD3DBufferImposterInteractive.Unlock();
#endif
	
	{	
		Asura_Direct3D_BGR_RenderCache& xRenderCache = this->m_xRenderCache;
		xRenderCache.CreateDirect3DData();
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
		xRenderCache.CreateSectionCache( *this);
#endif
	}
	
	Asura_Assert( ASURA_TEXTURE_INVALID == this->m_iAttenuationMap, "Attenuation Texture Already Setup");
//	this->m_iAttenuationMap = Asura_Texture_Management::AddTextureWithResetGroup("\\graphicnovel\\expattenuation.dds", ASURA_TEXTURE_LOADED_PERMANENT );

	return( true);
}

template< class Tgeometry> 
bool Asura_Direct3D_BGR_Manager::ClearDirect3DData_Type_Geometry( Tgeometry& tGeometry)
{
	SAFE_RELEASE( tGeometry.m_pxVertexBuffer);
	SAFE_RELEASE( tGeometry.m_pxIndexBuffer);
	
	tGeometry.m_uBufferInstances = 0;
	
	tGeometry.m_uVBStart = 0;
	tGeometry.m_uVBSize = 0;
	
	tGeometry.m_uIBStart = 0;
	tGeometry.m_uIBSize = 0;
	
	tGeometry.m_uPrimitiveCount = 0;
	
	return( true);
}

template< class Tgeometry> 
bool Asura_Direct3D_BGR_Manager::ClearDirect3DData_Type_ArrayGeometry( Asura_Botanical_Array< Tgeometry>& tArrayGeometry)
{
	const uint32 uGeometryCount = tArrayGeometry.Size();
	for( uint32 uGeometryIndex = 0; uGeometryIndex < uGeometryCount; uGeometryIndex += 1)
	{
		Tgeometry& tGeometry = tArrayGeometry[ uGeometryIndex];
		
		this->ClearDirect3DData_Type_Geometry( tGeometry);
	}
	return( true);
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
bool Asura_Direct3D_BGR_Manager::ClearDirect3DData_Type_Imposters( Asura_Direct3D_BGR_Type& xType)
{
	u_int uSectionCount = xType.m_xArray_xSections.Size();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Asura_Direct3D_BGR_Section& xSection = xType.m_xArray_xSections[ uSectionIndex];
		
		u_int uSectionVersionCount = xSection.m_xArray_xVersions.Size();
		for( u_int uSectionVersionIndex = 0; uSectionVersionIndex < uSectionVersionCount; uSectionVersionIndex += 1)
		{
			Asura_Direct3D_BGR_Section_Version& xSectionVersion = xSection.m_xArray_xVersions[ uSectionVersionIndex];
			Asura_Direct3D_BGR_Geometry& xGeometry = xSectionVersion.m_xGeometry;
	
			this->ClearDirect3DData_Type_ArrayGeometry( xGeometry.m_xArrayBillboards);
	
		}
	}

	return( true);
}
#endif

bool Asura_Direct3D_BGR_Manager::ClearDirect3DData_Type( Asura_Direct3D_BGR_Type& xType)
{
	u_int uSectionCount = xType.m_xArray_xSections.Size();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Asura_Direct3D_BGR_Section& xSection = xType.m_xArray_xSections[ uSectionIndex];
		
		u_int uSectionVersionCount = xSection.m_xArray_xVersions.Size();
		for( u_int uSectionVersionIndex = 0; uSectionVersionIndex < uSectionVersionCount; uSectionVersionIndex += 1)
		{
			Asura_Direct3D_BGR_Section_Version& xSectionVersion = xSection.m_xArray_xVersions[ uSectionVersionIndex];
			Asura_Direct3D_BGR_Geometry& xGeometry = xSectionVersion.m_xGeometry;
	
		#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
			this->ClearDirect3DData_Type_ArrayGeometry( xGeometry.m_xArrayBranchs);
			this->ClearDirect3DData_Type_ArrayGeometry( xGeometry.m_xArrayFronds);
		#endif
		#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
			this->ClearDirect3DData_Type_ArrayGeometry( xGeometry.m_xArrayLeaves);
		#endif
		#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
			this->ClearDirect3DData_Type_ArrayGeometry( xGeometry.m_xArrayBillboards);
		#endif
	
		}
	}

	return( true);
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
bool Asura_Direct3D_BGR_Manager::ClearDirect3DData_Imposters()
{
	// DLH - 31 July 2006 16:53:51 - Has some reports of the BGR crashing intermintly, but not enough information to track it down. This is to test a possiblilty.
	Asura_Assert( NULL != this, "BGR Manager is not setup correctly");
	
	uint32 uTypeCount = this->m_xArray_xTypes.Size();
	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_Type& xType = this->m_xArray_xTypes[ uTypeIndex];
		
		this->ClearDirect3DData_Type_Imposters( xType);
	}
	
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
		this->m_xD3DBufferImposter.Clear();
	#endif
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
		this->m_xD3DBufferImposterInteractive.Clear();
	#endif
	
	{
		// DLH - 07 April 2006 10:39:18 - Removed the Buffer range references
		const Asura_Botanical_Array< Asura_Direct3D_BGR_Instances_Header*>& xArray_pxInstanceHeaders = this->m_xArray_pxInstanceHeaders;
		const uint32 uInstanceHeaderCount = xArray_pxInstanceHeaders.Size();
		
		for( uint32 uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
		{
			Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
			if( NULL == pxInstanceHeader)
			{
				continue;
			}

			pxInstanceHeader->m_uImposterVBOffset = 0;			
			pxInstanceHeader->m_uImposterIBStart = 0;
			pxInstanceHeader->m_uImposterIBSize = 0;
			pxInstanceHeader->m_uImposterVBStart = 0;
			pxInstanceHeader->m_uImposterVBSize = 0;
		}
	}
		

	return( true);
}
#endif

bool Asura_Direct3D_BGR_RenderCache::ClearDirect3DData()
{
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	this->m_xCacheVertexGeometrys.ClearDirect3DData();
	this->m_xCacheIndexGeometrys.ClearDirect3DData();
#endif
	
	const u_int uDynamicLightCacheCount = this->m_xArray_xDynamicLightCache.Size();
	for( u_int uDynamicLightCacheIndex = 0; uDynamicLightCacheIndex < uDynamicLightCacheCount; uDynamicLightCacheIndex += 1)
	{
		Asura_Direct3D_BGR_DynamicLightCache& xDynamicLightCache = this->m_xArray_xDynamicLightCache[ uDynamicLightCacheIndex];

		xDynamicLightCache.ClearDirect3DData();
	}
	
	return( true);
}

bool Asura_Direct3D_BGR_Manager::ClearDirect3DData()
{
	// DLH - 31 July 2006 16:53:51 - Has some reports of the BGR crashing intermintly, but not enough information to track it down. This is to test a possiblilty.
	Asura_Assert( NULL != this, "BGR Manager is not setup correctly");
	
	uint32 uTypeCount = this->m_xArray_xTypes.Size();
	for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
	{
		Asura_Direct3D_BGR_Type& xType = this->m_xArray_xTypes[ uTypeIndex];
		
		this->ClearDirect3DData_Type( xType);
	}

	{	
		Asura_Direct3D_BGR_RenderCache& xRenderCache = this->m_xRenderCache;
		xRenderCache.ClearDirect3DData();
	}
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
	this->m_xD3DBuffer3DGeomtery.Clear();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
	this->m_xD3DBuffer3DLeaves.Clear();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS_INTERACTIVE)
	this->m_xD3DBufferImposterInteractive.Clear();
#endif
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
	this->m_xD3DBufferImposter.Clear();
	
	{
		// DLH - 07 April 2006 10:39:18 - Removed the Buffer range references
		const Asura_Botanical_Array< Asura_Direct3D_BGR_Instances_Header*>& xArray_pxInstanceHeaders = this->m_xArray_pxInstanceHeaders;
		const uint32 uInstanceHeaderCount = xArray_pxInstanceHeaders.Size();
		
		for( uint32 uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
		{
			Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
			if( NULL == pxInstanceHeader)
			{
				continue;
			}
			
			pxInstanceHeader->m_uImposterIBStart = 0;
			pxInstanceHeader->m_uImposterIBSize = 0;
			pxInstanceHeader->m_uImposterVBOffset = 0;
			pxInstanceHeader->m_uImposterVBStart = 0;
			pxInstanceHeader->m_uImposterVBSize = 0;
		}
	}
		
#endif

	this->m_iAttenuationMap = ASURA_TEXTURE_INVALID;
	
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
	this->ClearDirect3DData_SM30();
#endif
		
	return( true);
}

bool Asura_Direct3D_Botanicals_Game_Render::Initialise()
{
	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();
	
	xBGRManager.CreateDirect3DData();
	
	return( true);
}

void Asura_Direct3D_Botanicals_Game_Render::ShutDown()
{
	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();
	
	xBGRManager.ClearDirect3DData();
	xBGRManager.Clear();
}

bool Asura_Direct3D_Botanicals_Game_Render::PreD3DReset()
{
	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();
	
	xBGRManager.ClearDirect3DData();
	
	return( true);
}

bool Asura_Direct3D_Botanicals_Game_Render::PostD3DReset()
{
	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();
	
	xBGRManager.CreateDirect3DData();
	
	return( true);
}


//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_BOTANICALS)
