//*******************************************************************
//
// File: Asura_Direct3D_Carpet_Game_Render.h 
//
// Created: 19 Jan 2006
//
// Description: 
// This is the optimised PC render for the carpet system.
// Dispite its name it will be used in teh tools system to draw carpet stuff when not in carpet edit mode
//
//*******************************************************************

#ifndef _ASURA_DIRECT3D_CARPET_GAME_RENDER_H_
#define _ASURA_DIRECT3D_CARPET_GAME_RENDER_H_

// other libraries
#include "Carpet/Asura_Carpet_MeshManager.h"
#include "Core/Asura_LinkList.h"
#include "Rendering/Asura_Direct3D.h"

// internal dependencies

// forward declarations

class Asura_Carpet_Mesh_Data_Frame;
struct Asura_Carpet_Triangle;

class Asura_Chunk_File_Stream;
class Asura_Chunk_ResourceFile_PC_Carpet;
class Asura_Chunk_SaveBlock_Stream;

class Asura_Direct3D_CGR_Mesh;

class Asura_Photon;

class Asura_RTree_Builder;

class Tools_Carpet_Statistics;



//*******************************************************************
// Constants
//*******************************************************************

#define ASURA_DIRECT3D_CGR_USE_D3D_BUFFERS 1
#define ASURA_DIRECT3D_CGR_POLYGONSTYLE_LOOKUP_LENGTH 16
#define ASURA_DIRECT3D_CGR_D3DFVF_STREAMVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX1)

//*******************************************************************
// Class Definitions
//*******************************************************************

class Asura_Carpet_Mesh_Data_Frame
{
public:
	Asura_Carpet_Mesh_Data_Frame( const float fTime);

public:

	const float m_fTime;

	// DLH - 17 January 2006 11:26:57 - Camera Related Data
	Asura_Vector_3 m_xCameraForwards;
	Asura_Vector_3 m_xCameraPosition;
	float m_fCameraPlaneDistance;

	// DLH - 17 January 2006 11:48:48 - Billboards
	Asura_Vector_3 m_xBillboardUp;
	Asura_Vector_3 m_xBillboardRight;

	// DLH - 17 January 2006 11:26:41 - Wind wave animation data
	float m_afWaveX[ ASURA_CARPET_MESH_DATA_FRAME_SIZE_WAVEX];
	float m_afWaveY[ ASURA_CARPET_MESH_DATA_FRAME_SIZE_WAVEX];
	float m_afWaveZ[ ASURA_CARPET_MESH_DATA_FRAME_SIZE_WAVEX];
};

class Asura_Direct3D_CGR_Camera
{
public:
	bool GetCurrentCamera( const float fFarPlane);
	
	Asura_Matrix_3x3 m_xOrientation;

	Asura_Vector_3 m_xPosition;

	float m_fOneOverCosHalfFovX;
	float m_fTanHalfFovX;
	float m_fYOverXAspectRatio;
	float m_fOneOverCosHalfFovY;
	float m_fFarPlane;
};

class Asura_Direct3D_CGR_Camera_And_Box : public Asura_Direct3D_CGR_Camera
{
public:
	Asura_Bounding_Box m_xBoundingBox;
};

struct Asura_Direct3D_CGR_RTreeNode
{
public:
	Asura_Direct3D_CGR_RTreeNode();
	
	bool Process( Asura_Chunk_File_Stream &xStream, Asura_Direct3D_CGR_RTreeNode* pxRTreeNodeBase);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_SaveBlock_Stream& xStream, const Asura_Direct3D_CGR_RTreeNode* pxRTreeNodeBase) const;
#endif

public:
	Asura_Bounding_Box m_xBoundingBox;
	Asura_Direct3D_CGR_RTreeNode* m_pxSibling;
	Asura_Direct3D_CGR_RTreeNode* m_pxChild;
	uint32 m_uData;
};

struct Asura_Direct3D_CGR_PolygonBatch
{
public:
	Asura_Direct3D_CGR_PolygonBatch();
	
	bool Process( Asura_Chunk_File_Stream &xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_SaveBlock_Stream& xStream) const;
#endif

public:
	// DLH - 31 January 2006 14:19:32 - Offset and run length of polygons in the Mesh structure
	uint32 m_uDataOffset;
	uint32 m_uDataSize;
	
	// DLH - 31 January 2006 14:20:06 - Information about the location of the data run on directX buffers
	uint32 m_uTimeStampLastUsed;
	uint32 m_uVertexBufferOffset;
	uint32 m_uVertexBufferSize;
};

class Asura_Direct3D_CGR_RangeManager
{
public:
	Asura_Direct3D_CGR_RangeManager();
	void Clear();
	void SetMaxRange( const uint32 uMaxRange);
	
	bool FindFreeRange( const Asura_Direct3D_CGR_Mesh& xCGRMesh, Asura_Direct3D_CGR_PolygonBatch* pxCGRBatch, const uint32 uRequiredSize, uint32& uFounfOffset);
	bool AddBatch( const uint32 uTimeStampLast, Asura_Direct3D_CGR_Mesh& xCGRMesh, Asura_Direct3D_CGR_PolygonBatch& xCGRBatch);
	bool RemoveBatch( Asura_Direct3D_CGR_PolygonBatch& xCGRBatch);
	bool DeleteOldBatches( const uint32 uTimeStampLast);
	bool DeleteAllBatches( void);
	
	uint32 m_uRangeMaxCount;
	uint32 m_uRangeUsedCount;
	
//	uint32 m_uBlockMaxCount;
	uint32 m_uBlockUsedCount;
	
	Asura_LinkList< Asura_Direct3D_CGR_PolygonBatch* > m_xListBatches;
};

struct Asura_Direct3D_CGR_StreamVertexSimple
{
	float m_fX, m_fY, m_fZ;													// Position 0

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30)
	float m_afDiffuse[4];
#else
	uint32 m_uDiffuse;														// Color 0 ZYXW
#endif

	uint8 m_ucCornerIndex, m_ucWaveTableIndex, m_ucStyleIndex, m_ucScale;	// Color 1 ZYXW
};

class Asura_Direct3D_CGR_Vertex
{
public:
	Asura_Direct3D_CGR_Vertex();
	
	bool Process( Asura_Chunk_File_Stream &xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_SaveBlock_Stream& xStream) const;
#endif

public:
	Asura_Vector_3 m_xPosition;
	
	Asura_Vector_3 m_xColourRGB;
	float m_fDensity;
#if defined( ASURA_TOOLS)
	Asura_Vector_3 m_xNormal;
	u_int m_uPoly;
	u_int m_uVtx;
	u_int m_uModule;
#endif
};

class Asura_Direct3D_CGR_Polygon
{
public:
	Asura_Direct3D_CGR_Polygon();
	
	bool Process( Asura_Chunk_File_Stream &xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_SaveBlock_Stream& xStream) const;
#endif

public:
	float m_fLodFadeFar;
	float m_fDenisty;
	uint32 m_uInstanceCount;
	uint32 m_uRandomSeed;
	uint32 m_uPolygonStyle;
	
	uint32 m_uVertexIndex[3];
	
#if defined( ASURA_TOOLS)
	Asura_Vector_3 m_xCentre;
	float m_fRadius;
#endif
};

class Asura_Direct3D_CGR_PolygonStyle
{
public:
	Asura_Direct3D_CGR_PolygonStyle();
	
	bool Process( Asura_Chunk_File_Stream &xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_SaveBlock_Stream& xStream) const;
#endif

public:
	static const uint32 uiInvalidStyleIndex;

	uint32 m_aucStyleLookup[ ASURA_DIRECT3D_CGR_POLYGONSTYLE_LOOKUP_LENGTH];
	
#if defined( ASURA_TOOLS)
	uint32 m_uStyle;
	float m_fLodFadeFar;
#endif
};

class Asura_Direct3D_CGR_Style
{
public:
	Asura_Direct3D_CGR_Style();
	
	bool Process( Asura_Chunk_File_Stream &xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_SaveBlock_Stream& xStream) const;
#endif

public:
	float m_fWidth;
	float m_fHeight;
	float m_afU[2];
	float m_afV[2];
	float m_fScaleMin; // DLH - 0-255 encoded 0.0f-1.0f
	float m_fLodFadeFar;
	float m_fLodFadeNear;
	float m_fLodFadeDelta;
	float m_fMovementScale;

#if defined( ASURA_TOOLS)
	uint32 m_uTriangleCount;
#endif
};

class Asura_Direct3D_CGR_Mesh
{
public:
	Asura_Direct3D_CGR_Mesh();
	~Asura_Direct3D_CGR_Mesh();
	
	bool Process( Asura_Chunk_File_Stream &xStream);
#if defined( ASURA_TOOLS)
	bool Export( Asura_Chunk_SaveBlock_Stream& xStream) const;
#endif

public:
	int32 m_iTexture;
	
	uint32 m_uVertexCount;
	uint32 m_uPolygonCount;
	uint32 m_uPolygonStyleCount;
	uint32 m_uStyleCount;
	uint32 m_uRTreeCount;
	uint32 m_uPolygonBatchCount;
	
//	Asura_Bounding_Box m_xBoundingBox;
	
	Asura_Direct3D_CGR_Vertex* m_pxVertices;
	Asura_Direct3D_CGR_Polygon* m_pxPolygons;
	Asura_Direct3D_CGR_PolygonStyle* m_pxPolygonStyles;
	Asura_Direct3D_CGR_Style* m_pxStyles;
	Asura_Direct3D_CGR_RTreeNode* m_pxRTree;
	Asura_Direct3D_CGR_PolygonBatch* m_pxPolygonBatches;
	
//	uint32 m_uBatchVertexBufferPresentSize;
	uint32 m_uBatchVertexBufferSize;
	Direct3D_VertexBuffer* m_pxBatchVertexBuffer;
	
	float m_fLodFadeFar;
	
	Asura_Bounding_Box m_xBoundingBox;
	Asura_Direct3D_CGR_RangeManager m_xVBRangeManager;
};

class Asura_Direct3D_Carpet_Game_Render
{
public:
	enum VertexMode 
	{
		SHADER_SIMPLE,
		SHADER_MULTIPLE,
		SHADER_MODULUS
	};
	
	enum CullState
	{
		CULL_INSIDE,
		CULL_OUTSIDE,
		CULL_TOUCHING
	};
	

public:
	static bool Initialise();
	static void ShutDown();
	
	static bool PreD3DReset( void*);
	static bool PostD3DReset( void*);
	
	static bool ClearDataSet();
	static bool HasDataSet() { return( 0 != s_uMeshesCount);}
	
	static uint32 GetMeshCount() { return( s_uMeshesCount);}
	static const Asura_Direct3D_CGR_Mesh& GetMesh( const uint32 uiMeshIndex) { Asura_Assert( NULL != s_pxMeshes, "Mesh Array has not been setup"); Asura_Assert( uiMeshIndex < s_uMeshesCount, "Mesh Index is out of range"); return( s_pxMeshes[ uiMeshIndex]);}
	
	static uint32 GetBatchIndexBufferSize( void) { return( s_uBatchIndexBufferSize);}

	static bool Render();
	
#if defined( ASURA_TOOLS)
	static bool ExportPC( HANDLE xFileHandle);
	static bool ExportXBox360( HANDLE xFileHandle);
	static bool ExportPS3( HANDLE xFileHandle);
	static bool MakeDataSet( const Asura_LinkList< Asura_Carpet_Triangle>& pxListCarpetTriangles);
	
	static bool RelightEverything( void);
	static bool DeleteModule( const u_int uModuleIndex);
#endif

private:
	static bool BeginRenderStatic( const Asura_Carpet_Mesh_Data_Frame& xDataFrame);
	static bool BeginRenderDynamic( const Asura_Carpet_Mesh_Data_Frame& xDataFrame, const Asura_Photon& xPhoton);
	static void EndRender( void);
	
	static bool RenderSetShaderConstantsStatic( const Asura_Carpet_Mesh_Data_Frame& xDataFrame, const Asura_Photon* pxPhoton);
	static bool RenderSetShaderConstantsStyles( const uint32 uiStyleIndex, const Asura_Direct3D_CGR_Style& xStyle);
	template< class Tcamera> static bool RenderMesh( const Asura_Carpet_Mesh_Data_Frame& xDataFrame, const Tcamera& tCamera, Asura_Direct3D_CGR_Mesh& xCGRMesh);
	static bool RenderMeshBatch( const Asura_Direct3D_CGR_Mesh& xCGRMesh, const Asura_Direct3D_CGR_PolygonBatch& xCGRBatch);
	
	static bool CopyMeshPolygonToVertex( const Asura_Direct3D_CGR_Mesh& xCGRMesh, const Asura_Direct3D_CGR_Polygon& xCGRPolygon, Asura_Direct3D_CGR_StreamVertexSimple* pxVertex0, uint32& uVertexMaxCount);
	
	static CullState GetCullStateOfSphereInFrustum( const Asura_Direct3D_CGR_Camera& xCamera, const Asura_Vector_3& xCentre, const float fRadius);
	static CullState GetCullStateOfAABBInFrustum( const Asura_Direct3D_CGR_Camera& xCamera, const Asura_Bounding_Box& xBB );
	static CullState GetCullStateOfAABBInFrustum( const Asura_Direct3D_CGR_Camera_And_Box& xCamera, const Asura_Bounding_Box& xBB );
	template< class TvisibleBounds, bool bCheckVisability> static bool CalcVisibleBatches( const TvisibleBounds& xCamera, const Asura_Direct3D_CGR_RTreeNode* pxRTree, uint32* puiBatchIndices, uint32& uiBatchCount);
	
	static bool CreateVertexFormat();
	static bool CreateIndexBuffer();
	static bool CreateVertexBufferMesh( Asura_Direct3D_CGR_Mesh& xCGRMesh);
	static bool CreateVertexBuffer();
	static bool CreateDirect3DData();
	static bool ClearDirect3DData();
	static bool ClearDirect3DData_AllBuffers();
	
	static bool BatchRemoveFromBuffers( Asura_Direct3D_CGR_Mesh& xCGRMesh, Asura_Direct3D_CGR_PolygonBatch& xCGRBatch);
	static bool BatchAddToBuffers( Asura_Direct3D_CGR_Mesh& xCGRMesh, Asura_Direct3D_CGR_PolygonBatch& xCGRBatch);
	static bool BatchGetRequiredBufferSize( const Asura_Direct3D_CGR_Mesh& xCGRMesh, const Asura_Direct3D_CGR_PolygonBatch& xCGRBatch, uint32& uIndexCount, uint32& uVertexCount);
	static bool BatchCopyToBuffers( Asura_Direct3D_CGR_Mesh& xCGRMesh, const Asura_Direct3D_CGR_PolygonBatch& xCGRBatch);
	static bool BatchCopyAllToBuffers();
	static bool BatchFindOldBatches( Asura_Direct3D_CGR_Mesh& xCGRMesh, Asura_Direct3D_CGR_PolygonBatch** ppxCGRBatchs, uint32& uBatchCount);

	static void SetTextureID( const int iTextureID);
	static void RenderCache();
	
#if defined( ASURA_TOOLS)
	static bool MakeDataSet_RTreeBuilderToCGRRTree( Asura_RTree_Builder &xRTreeBuilder, Asura_Direct3D_CGR_Mesh& xMesh);
	static bool MakeDataSet_CalcMinimumVertexBufferSize( Asura_Direct3D_CGR_Mesh* pxMesh);
	static bool MakeDataSet_MeshesBatches( Asura_Direct3D_CGR_Mesh& xMesh, Asura_RTree_Builder &xRTreeBuilder);
	static bool MakeDataSet_MeshesRTree( Asura_Direct3D_CGR_Mesh* pxMesh);
	static bool MakeDataSet_Meshes( const Asura_LinkList< Asura_Carpet_Triangle>& pxListCarpetTriangles);
#endif

private:	
	static uint32 s_uTimeStampLast;
	static uint32 s_uMeshesCount;
	static Asura_Direct3D_CGR_Mesh* s_pxMeshes;
	
	// DLH - 23 January 2006 15:39:08 - direct3d vertex data
	static VertexMode s_eVertexMode;
	static Direct3D_VertexDeclaration* s_pxVertexDeclarationSimple;
	static Direct3D_VertexElement* s_paxVSD_Carpet_Simple;
	
	static int s_iDrawCacheTextureID;
	static int s_iAttenuationMap;


	static uint32 s_uBatchIndexBufferSize;
	static Direct3D_IndexBuffer* s_pxBatchIndexBuffer;
	
	friend class Tools_Carpet_Statistics;
	friend class Asura_Direct3D_CGR_RangeManager;
	friend class Asura_Chunk_ResourceFile_PC_Carpet;
};

//*******************************************************************
// Inline Function Definitions
//*******************************************************************


//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif // _ASURA_DIRECT3D_CARPET_GAME_RENDER_H_
