//*******************************************************************
//
// File: Asura_Direct3D_Carpet_Render.cpp    
//
// Created: 13 Jan 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_CARPET)

// other libraries
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Direct3d.h"
#include "Rendering/Asura_Direct3D_Shaders.h"

// main header
#include "Carpet/Asura_Direct3D_Carpet_Render.h"

#if defined( ASURA_TOOLS)
#include "Carpet/Tools_Carpet_Statistics.h"
#endif
// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************

#define TOOLS_CARPET_MESH_DRAW_CACHE_QUADS (uint32( 0x10000 / 4))
//#define TOOLS_CARPET_MESH_DRAW_CACHE_QUADS 1000
//#define TOOLS_CARPET_MESH_DRAW_CACHE_INDICES (6 * TOOLS_CARPET_MESH_DRAW_CACHE_QUADS)

//*******************************************************************
// Local Classes
//*******************************************************************


//*******************************************************************
// Local Data
//*******************************************************************

uint32 Asura_Direct3D_Carpet_Render::s_uDrawCacheMaxQuads = TOOLS_CARPET_MESH_DRAW_CACHE_QUADS;

int Asura_Direct3D_Carpet_Render::s_iDrawCacheTextureID = -1;
uint32 Asura_Direct3D_Carpet_Render::s_uDrawCacheUsed;
#if defined( ASURA_DIRECT3D_CARPET_RENDER_USE_D3D_BUFFERS)
	Asura_Direct3D_Carpet_Render_Vertex* Asura_Direct3D_Carpet_Render::s_paxDrawCacheVertexArray = NULL;
	Direct3D_IndexBuffer* Asura_Direct3D_Carpet_Render::s_pxIndexBuffer = NULL;
	Direct3D_VertexBuffer* Asura_Direct3D_Carpet_Render::s_pxVertexBuffer = NULL;
#else
	uint16 Asura_Direct3D_Carpet_Render::s_axDrawCacheIndexArray[ 6 * TOOLS_CARPET_MESH_DRAW_CACHE_QUADS] = {0};
	Asura_Direct3D_Carpet_Render_Vertex Asura_Direct3D_Carpet_Render::s_axDrawCacheVertexArray[4 * TOOLS_CARPET_MESH_DRAW_CACHE_QUADS] = {0};
#endif

#undef TOOLS_CARPET_MESH_DRAW_CACHE_QUADS

//*******************************************************************
// Function Definitions
//*******************************************************************

bool Asura_Direct3D_Carpet_Render::CreateVertexBuffer()
{
#if defined( ASURA_DIRECT3D_CARPET_RENDER_USE_D3D_BUFFERS)
	HRESULT xResult;
	const uint32 uiVertexDataSize = 4 * s_uDrawCacheMaxQuads * sizeof( Asura_Direct3D_Carpet_Render_Vertex);
	
	if( NULL == s_pxVertexBuffer)
	{
		xResult = Asura_Direct3D::D3DDevice->CreateVertexBuffer
		(
			uiVertexDataSize,
			D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC,
			ASURA_DIRECT3D_CARPET_RENDER_D3DFVF_VERTEX,
			D3DPOOL_DEFAULT,
			&s_pxVertexBuffer,
			0
		);
		if( ASURA_D3D_FAILED( xResult))
		{
			return( false);
		}
	}
#endif
		
	return( true);
}

bool Asura_Direct3D_Carpet_Render::CreateIndexBuffer()
{
#if defined( ASURA_DIRECT3D_CARPET_RENDER_USE_D3D_BUFFERS)
	HRESULT xResult;
	const uint32 uiIndexDataSize = 6 * s_uDrawCacheMaxQuads * sizeof( uint16);
	
	if( NULL == s_pxIndexBuffer)
	{
		xResult = Asura_Direct3D::D3DDevice->CreateIndexBuffer
		(
			uiIndexDataSize,
			D3DUSAGE_WRITEONLY,
			D3DFMT_INDEX16,
			D3DPOOL_MANAGED,
			&s_pxIndexBuffer,
			0
		);
		if( ASURA_D3D_FAILED( xResult))
		{
			return( false);
		}
	}
		
	uint16* pausVertexIndexData = NULL;
	xResult = s_pxIndexBuffer->Lock( 0, uiIndexDataSize, (void**)&pausVertexIndexData, 0);
	if( ASURA_D3D_FAILED( xResult))
	{
		return( false);
	}
#else
	uint16* pausVertexIndexData = s_axDrawCacheIndexArray;
#endif
	
	// DLH - 12 January 2006 16:07:27 - This should really be in an index buffer
	// DLH - 12 January 2006 16:07:13 - If this array has not been initialise then fill it with indices
	for( uint32 uIndexQuad = 0; uIndexQuad < s_uDrawCacheMaxQuads; uIndexQuad += 1)
	{
		const uint32 uIndexVertex = 4 * uIndexQuad;
		const uint16 usIndexVertex = static_cast<uint16>( uIndexVertex );
		const uint32 uIndexIndex = 6 * uIndexQuad;
		
		Asura_Assert( usIndexVertex == uIndexVertex, "Index Too big for uint16!");
		
		//uint16* paxIndex = &s_axDrawCacheIndexArray[ uiIndexIndex];
		uint16* paxIndex = &pausVertexIndexData[ uIndexIndex];
	
#if 0	
		paxIndex[ 0] = usIndexVertex + 0;
		paxIndex[ 1] = usIndexVertex + 0;
		paxIndex[ 2] = usIndexVertex + 1;
		paxIndex[ 3] = usIndexVertex + 3;
		paxIndex[ 4] = usIndexVertex + 2;
		paxIndex[ 5] = usIndexVertex + 2;
#endif
		paxIndex[ 0] = usIndexVertex + 0;
		paxIndex[ 1] = usIndexVertex + 1;
		paxIndex[ 2] = usIndexVertex + 3;
		paxIndex[ 3] = usIndexVertex + 1;
		paxIndex[ 4] = usIndexVertex + 3;
		paxIndex[ 5] = usIndexVertex + 2;
	}
	
#if defined( ASURA_DIRECT3D_CARPET_RENDER_USE_D3D_BUFFERS)
	xResult = s_pxIndexBuffer->Unlock();
	ASURA_D3D_OK( xResult);
#endif
	
	return( true);
}

void Asura_Direct3D_Carpet_Render::Initialise()
{
	CreateIndexBuffer();
	CreateVertexBuffer();
}

void Asura_Direct3D_Carpet_Render::ShutDown()
{
#if defined( ASURA_DIRECT3D_CARPET_RENDER_USE_D3D_BUFFERS)
	SAFE_RELEASE( s_pxVertexBuffer);
	SAFE_RELEASE( s_pxIndexBuffer);
#endif
}

void Asura_Direct3D_Carpet_Render::PreD3DReset()
{
	ShutDown();
}

void Asura_Direct3D_Carpet_Render::PostD3DReset()
{
	Initialise();
}

void Asura_Direct3D_Carpet_Render::BeginRender()
{
	SCOPEDPIXEVENT(( 0, L"Begin render" ));

	HRESULT xResult;
	// Temporary render function, using the rendering code from the tools, until we get the 
	// proper optimized renderer (with scene management, batching, and using VU/shaders)

	// Set some renderstates
	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(ASURA_DIRECT3D_CARPET_RENDER_D3DFVF_VERTEX);
	xResult = Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	ASURA_D3D_OK( xResult);	
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);
	Asura_Render::CurrentStates.RequestAlphaTest( true , 0x00);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	
	xResult = Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP );
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP );
	ASURA_D3D_OK( xResult);	

#if defined( ASURA_TARGET_SM3)
	xResult = Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_SRGBTEXTURE, TRUE );
	ASURA_D3D_OK( xResult);
#endif
	
	Asura_Direct3D_Shaders::RequestPixelShader( ASURA_D3D_PS_CARPET_LIGHTS);
	
#if defined( ASURA_DIRECT3D_CARPET_RENDER_USE_D3D_BUFFERS)
	xResult = Asura_Direct3D::D3DDevice->SetStreamSource( 0, s_pxVertexBuffer, 0, sizeof(Asura_Direct3D_Carpet_Render_Vertex));
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Direct3D::D3DDevice->SetIndices( s_pxIndexBuffer);
	ASURA_D3D_OK( xResult);	
#endif
	
	s_uDrawCacheUsed = 0;
	s_iDrawCacheTextureID = -1;
	
}

void Asura_Direct3D_Carpet_Render::EndRender()
{
	SCOPEDPIXEVENT(( 0, L"End render" ));

	if( 0 != s_uDrawCacheUsed)
	{
		RenderCache();
		s_uDrawCacheUsed = 0;
	}

	// Must restore default states after - DMR
	HRESULT xResult;
	xResult = Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
	ASURA_D3D_OK( xResult);	
	xResult = Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );
	ASURA_D3D_OK( xResult);

#if defined( ASURA_TARGET_SM3)
	xResult = Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_SRGBTEXTURE, FALSE );
	ASURA_D3D_OK( xResult);
#endif
}

Asura_Direct3D_Carpet_Render_Vertex* Asura_Direct3D_Carpet_Render::GetDrawQuad()
{
	// DLH - 12 January 2006 15:49:53 - How many elements in the array?
	const uint32 uiDrawCacheVertexArraySize = 4 * s_uDrawCacheMaxQuads;
	if( uiDrawCacheVertexArraySize <= s_uDrawCacheUsed)
	{
		RenderCache();
		s_uDrawCacheUsed = 0;
	}
	
#if defined( ASURA_DIRECT3D_CARPET_RENDER_USE_D3D_BUFFERS)
	HRESULT xResult;
	
	if( NULL == s_paxDrawCacheVertexArray)
	{
		xResult = s_pxVertexBuffer->Lock( 0, 0, (void**)&s_paxDrawCacheVertexArray, D3DLOCK_DISCARD);
		ASURA_D3D_OK( xResult);
	}
	
	Asura_Direct3D_Carpet_Render_Vertex* paxVertex0 = &s_paxDrawCacheVertexArray[ s_uDrawCacheUsed];
#else
	
	Asura_Direct3D_Carpet_Render_Vertex* paxVertex0 = &s_axDrawCacheVertexArray[ s_uDrawCacheUsed];
#endif
	s_uDrawCacheUsed += 4;
	
	Asura_Assert( s_uDrawCacheUsed <= uiDrawCacheVertexArraySize, "To many vertices in the Carpet Cache");
	
	return( paxVertex0);
}

void Asura_Direct3D_Carpet_Render::SetTextureID( const int iTextureID)
{
	if( iTextureID != s_iDrawCacheTextureID)
	{
		RenderCache();
		s_uDrawCacheUsed = 0;
		
		Asura_Render::CurrentStates.RequestTextureIndex( 0, iTextureID);					
		s_iDrawCacheTextureID = iTextureID;
	}
}

void Asura_Direct3D_Carpet_Render::RenderCache()
{
	SCOPEDPIXEVENT((0xff00ff00, L"Carpet: RenderCache"));

	HRESULT xResult = D3D_OK;
	
	if( 0 == s_uDrawCacheUsed)
	{
		return;
	}
	
#if defined( ASURA_DIRECT3D_CARPET_RENDER_USE_D3D_BUFFERS)
	if( NULL != s_paxDrawCacheVertexArray)
	{
		xResult = s_pxVertexBuffer->Unlock();
		ASURA_D3D_OK( xResult);	
		s_paxDrawCacheVertexArray = NULL;
	}
	
	//xResult = Asura_Direct3D::D3DDevice->SetStreamSource( 0, s_pxVertexBuffer, 0, sizeof(Asura_Direct3D_Carpet_Render_Vertex));
	//ASURA_D3D_OK( xResult);	
	
	// DLH - 12 January 2006 15:58:23 - Convert to # quads then to # triangles
	const uint32 uCountPrimitives = (2 * (s_uDrawCacheUsed / 4));
	xResult = Asura_Direct3D::D3DDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0, 0, s_uDrawCacheUsed, 0, uCountPrimitives);
	ASURA_D3D_OK( xResult);	
	
#else
	const uint32 uCountPrimitives = (2 * (s_uDrawCacheUsed / 4));
	xResult = Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP( D3DPT_TRIANGLELIST, 0, s_uDrawCacheUsed, uCountPrimitives, s_axDrawCacheIndexArray, D3DFMT_INDEX16, (void*)s_axDrawCacheVertexArray, sizeof( s_axDrawCacheVertexArray[0]));
	ASURA_D3D_OK( xResult);	
#endif

#if defined( ASURA_TOOLS)
	TOOLS_CARPET_STATISTICS_ADD( s_uDIPCount, 1);
	TOOLS_CARPET_STATISTICS_ADD( s_uDIPIndexCount, 0);
	TOOLS_CARPET_STATISTICS_ADD( s_uDIPVertexCount, s_uDrawCacheUsed);
	TOOLS_CARPET_STATISTICS_ADD( s_uDIPPrimitiveCount, uCountPrimitives);
#endif
}

#endif //define( ASURA_USE_CARPET)


//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

