//*******************************************************************
//
// File: Asura_Chunk_ResourceFile_PC_Carpet.cpp
//
// Created: 17 Feb 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_CARPET)

// main header
#include "Carpet/Asura_Direct3D_Carpet_Game_Render.h"

// other libraries
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"

#include "ChunkSystem/Asura_Chunk_ResourceFile_PC.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PC_Carpet.h"

#if defined ASURA_TOOLS
	#include "XBox360/ChunkSystem/Asura_Chunk_ResourceFile_Xbox360.h"
	#include "XBox360/ChunkSystem/Asura_Chunk_ResourceFile_Xbox360_Carpet.h"
	#include "PS3/ChunkSystem/Asura_Chunk_ResourceFile_PS3.h"
	#include "PC/ChunkSystem/Asura_Chunk_ResourceFile_PC_Carpet.h"
#endif

#include "Core/Asura_Hash.h"
#include "Core/Asura_Texturemanagement.h"

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************


//*******************************************************************
// Function Definitions
//*******************************************************************


#if defined( ASURA_TOOLS)

bool Asura_Direct3D_CGR_RTreeNode::Export( Asura_Chunk_SaveBlock_Stream& xStream, const Asura_Direct3D_CGR_RTreeNode* pxRTreeNodeBase) const
{
	xStream << this->m_xBoundingBox;
	xStream << this->m_uData;
	
	const uint32 uSiblingIndex = (NULL != this->m_pxSibling) ? (this->m_pxSibling - pxRTreeNodeBase) : uint32( -1);
	const uint32 uChildIndex = (NULL != this->m_pxChild) ? (this->m_pxChild - pxRTreeNodeBase) : uint32( -1);
	
	xStream << uSiblingIndex;
	xStream << uChildIndex;
	
	return( true);
}

bool Asura_Direct3D_CGR_PolygonStyle::Export( Asura_Chunk_SaveBlock_Stream& xStream) const
{
	for( uint32 uTableIndex = 0; uTableIndex < ASURA_DIRECT3D_CGR_POLYGONSTYLE_LOOKUP_LENGTH; uTableIndex += 1)
	{
		xStream << this->m_aucStyleLookup[ uTableIndex];
	}
	
	return( true);
}

bool Asura_Direct3D_CGR_Style::Export( Asura_Chunk_SaveBlock_Stream& xStream) const
{
	xStream << this->m_fWidth;
	xStream << this->m_fHeight;
	xStream << this->m_afU[0];
	xStream << this->m_afU[1];
	xStream << this->m_afV[0];
	xStream << this->m_afV[1];
	xStream << this->m_fScaleMin;
	xStream << this->m_fLodFadeFar;
	xStream << this->m_fLodFadeNear;
	xStream << this->m_fLodFadeDelta;
	xStream << this->m_fMovementScale;

	return( true);
}

bool Asura_Direct3D_CGR_PolygonBatch::Export( Asura_Chunk_SaveBlock_Stream& xStream) const
{
	xStream << this->m_uDataOffset;
	xStream << this->m_uDataSize;
	
	return( true);
}

bool Asura_Direct3D_CGR_Vertex::Export( Asura_Chunk_SaveBlock_Stream& xStream) const
{
	xStream << this->m_fDensity;
	xStream << this->m_xColourRGB;
	xStream << this->m_xPosition;
	
	return( true);
}

bool Asura_Direct3D_CGR_Polygon::Export( Asura_Chunk_SaveBlock_Stream& xStream) const
{
	xStream << this->m_fLodFadeFar;
	xStream << this->m_fDenisty;
	xStream << this->m_uInstanceCount;
	xStream << this->m_uRandomSeed;
	xStream << this->m_uPolygonStyle;
	
	xStream << this->m_uVertexIndex[0];
	xStream << this->m_uVertexIndex[1];
	xStream << this->m_uVertexIndex[2];
	
	return( true);
}

bool Asura_Direct3D_CGR_Mesh::Export( Asura_Chunk_SaveBlock_Stream& xStream) const
{
	Asura_Hash_ID xTextureHash = Asura_Texture_Management::GetHashIDFromIndex( this->m_iTexture);

	xStream << xTextureHash;
	xStream << this->m_xBoundingBox;
	xStream << float( this->m_fLodFadeFar);
	
	xStream << uint32( this->m_uPolygonCount);
	xStream << uint32( this->m_uPolygonBatchCount);
	xStream << uint32( this->m_uPolygonStyleCount);
	xStream << uint32( this->m_uRTreeCount);
	xStream << uint32( this->m_uStyleCount);
	xStream << uint32( this->m_uVertexCount);
	
	for( uint32 uVertexIndex = 0; uVertexIndex < this->m_uVertexCount; uVertexIndex += 1)
	{
		if( false == this->m_pxVertices[ uVertexIndex].Export( xStream))
		{
			return( false);
		}
	}
	
	for( uint32 uPolygonIndex = 0; uPolygonIndex < this->m_uPolygonCount; uPolygonIndex += 1)
	{
		if( false == this->m_pxPolygons[ uPolygonIndex].Export( xStream))
		{
			return( false);
		}
	}
	
	for( uint32 uPolygonStyleIndex = 0; uPolygonStyleIndex < this->m_uPolygonStyleCount; uPolygonStyleIndex += 1)
	{
		if( false == this->m_pxPolygonStyles[ uPolygonStyleIndex].Export( xStream))
		{
			return( false);
		}
	}
	
	for( uint32 uPolygonBatchIndex = 0; uPolygonBatchIndex < this->m_uPolygonBatchCount; uPolygonBatchIndex += 1)
	{
		if( false == this->m_pxPolygonBatches[ uPolygonBatchIndex].Export( xStream))
		{
			return( false);
		}
	}
	
	for( uint32 uRTreeNodeIndex = 0; uRTreeNodeIndex < this->m_uRTreeCount; uRTreeNodeIndex += 1)
	{
		if( false == this->m_pxRTree[ uRTreeNodeIndex].Export( xStream, this->m_pxRTree))
		{
			return( false);
		}
	}
	
	for( uint32 uStyleIndex = 0; uStyleIndex < this->m_uStyleCount; uStyleIndex += 1)
	{
		if( false == this->m_pxStyles[ uStyleIndex].Export( xStream))
		{
			return( false);
		}
	}
	
	return( true);
}

bool Asura_Direct3D_Carpet_Game_Render::ExportPC( HANDLE xFileHandle)
{
	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	Asura_Assert( pxSaveBlock, "Unable to create saveblock");

	if( NULL == pxSaveBlock)
	{
		return( false);
	}
	
	Asura_Chunk_SaveBlock_Stream xStream( pxSaveBlock);

	xStream.WriteChunk_Start( ASURA_RESOURCEFILE_TYPE_PC_CARPET, Asura_Chunk_ResourceFile_PC_Carpet::CURRENT_VERSION, 0);
	
	uint32 uCGRMeshCount = s_uMeshesCount;
	
	xStream << uCGRMeshCount;
	
	for( uint32 uCGRMeshIndex = 0; uCGRMeshIndex < uCGRMeshCount; uCGRMeshIndex += 1)
	{
		const Asura_Direct3D_CGR_Mesh& xCGRMesh = s_pxMeshes[ uCGRMeshIndex];
		
		if( false == xCGRMesh.Export( xStream))
		{
			return( false);
		}
	}

	xStream << Asura_Carpet_MeshManager::GetMovementSpeed();

	xStream.WriteChunk_End();
	
#if 0
	//add the contents to the file
	// DLH - 16 February 2006 16:46:59 - Code Copied from Tools_Level_Export::AddAnimationTables()
	{
		DWORD uBytesWritten;
		uint32 uBytesToWrite = pxSaveBlock->GetSize() - ASURA_FILE_ID_NUM_CHARS;
		const void* pDataStart = pxSaveBlock->GetStartPtr() + ASURA_FILE_ID_NUM_CHARS;
		BOOL bSuccess = WriteFile( xFileHandle, pDataStart, uBytesToWrite, &uBytesWritten, 0);

		if( 0 == bSuccess || uBytesToWrite != uBytesWritten)
		{
			return( false);
			//AddMessage(MT_ERROR,"Error writing to file.");
			//SetAbortFlag();
		}
	}
#endif

	#define WRITEOUTDATA(a,b) WriteFile( xFileHandle, a, b, &bytesWritten, 0 );
	
	{
		const uint32 uBytesHeaderSize = sizeof( Asura_Chunk_Header) + ASURA_FILE_ID_NUM_CHARS;
		//const uint32 uBytesHeaderSize = ASURA_FILE_ID_NUM_CHARS;
		uint32 uBytesToWrite = pxSaveBlock->GetSize() - uBytesHeaderSize;
		const void* pDataStart = pxSaveBlock->GetStartPtr() + uBytesHeaderSize;
		uint32 uChunkSize = uBytesToWrite + 4 + sizeof( Asura_Chunk_ResourceFile);
		
		Asura_Chunk_ResourceFile* pxResourceFile = new Asura_Chunk_ResourceFile;

		pxResourceFile->Create
		(
			uChunkSize,
			0,
			ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
			ASURA_RESOURCEFILE_TYPE_PC_CARPET,
			uBytesToWrite,
			Asura_Chunk_ResourceFile_PC_Carpet::CURRENT_VERSION
		);

		unsigned long bytesWritten;

		WRITEOUTDATA( pxResourceFile, sizeof(Asura_Chunk_ResourceFile) );
		const char* szStringResourceFilename = "Cpt";
		WRITEOUTDATA( szStringResourceFilename, 4 );
		
		WRITEOUTDATA( pDataStart, uBytesToWrite);
		
		delete pxResourceFile;
	}
	
	#undef WRITEOUTDATA
	
	delete pxSaveBlock;

	return( true);
}

bool Asura_Direct3D_Carpet_Game_Render::ExportXBox360( HANDLE xFileHandle)
{
	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	Asura_Assert( pxSaveBlock, "Unable to create saveblock");

	if( NULL == pxSaveBlock)
	{
		return( false);
	}
	
	Asura_Chunk_SaveBlock_Stream xStream( pxSaveBlock);

	xStream.WriteChunk_Start( ASURA_RESOURCEFILE_TYPE_PC_CARPET, Asura_Chunk_ResourceFile_Xbox360_Carpet::CURRENT_VERSION, 0);
	
	uint32 uCGRMeshCount = s_uMeshesCount;
	
	xStream << uCGRMeshCount;
	
	for( uint32 uCGRMeshIndex = 0; uCGRMeshIndex < uCGRMeshCount; uCGRMeshIndex += 1)
	{
		const Asura_Direct3D_CGR_Mesh& xCGRMesh = s_pxMeshes[ uCGRMeshIndex];
		
		if( false == xCGRMesh.Export( xStream))
		{
			return( false);
		}
	}

	xStream << Asura_Carpet_MeshManager::GetMovementSpeed();

	xStream.WriteChunk_End();
	
#if 0
	//add the contents to the file
	// DLH - 16 February 2006 16:46:59 - Code Copied from Tools_Level_Export::AddAnimationTables()
	{
		DWORD uBytesWritten;
		uint32 uBytesToWrite = pxSaveBlock->GetSize() - ASURA_FILE_ID_NUM_CHARS;
		const void* pDataStart = pxSaveBlock->GetStartPtr() + ASURA_FILE_ID_NUM_CHARS;
		BOOL bSuccess = WriteFile( xFileHandle, pDataStart, uBytesToWrite, &uBytesWritten, 0);

		if( 0 == bSuccess || uBytesToWrite != uBytesWritten)
		{
			return( false);
			//AddMessage(MT_ERROR,"Error writing to file.");
			//SetAbortFlag();
		}
	}
#endif

	#define WRITEOUTDATA(a,b) WriteFile( xFileHandle, a, b, &bytesWritten, 0 );
	
	{
		const uint32 uBytesHeaderSize = sizeof( Asura_Chunk_Header) + ASURA_FILE_ID_NUM_CHARS;
		//const uint32 uBytesHeaderSize = ASURA_FILE_ID_NUM_CHARS;
		uint32 uBytesToWrite = pxSaveBlock->GetSize() - uBytesHeaderSize;
		const void* pDataStart = pxSaveBlock->GetStartPtr() + uBytesHeaderSize;
		uint32 uChunkSize = uBytesToWrite + 4 + sizeof( Asura_Chunk_ResourceFile);
		
		Asura_Chunk_ResourceFile* pxResourceFile = new Asura_Chunk_ResourceFile;

		pxResourceFile->Create
		(
			uChunkSize,
			0,
			ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
			ASURA_RESOURCEFILE_TYPE_XBOX360_CARPET,
			uBytesToWrite,
			Asura_Chunk_ResourceFile_Xbox360_Carpet::CURRENT_VERSION
		);

		unsigned long bytesWritten;

		WRITEOUTDATA( pxResourceFile, sizeof(Asura_Chunk_ResourceFile) );
		const char* szStringResourceFilename = "Cpt";
		WRITEOUTDATA( szStringResourceFilename, 4 );
		
		WRITEOUTDATA( pDataStart, uBytesToWrite);
		
		delete pxResourceFile;
	}
	
	#undef WRITEOUTDATA
	
	delete pxSaveBlock;

	return( true);
}

bool Asura_Direct3D_Carpet_Game_Render::ExportPS3( HANDLE xFileHandle)
{
	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	Asura_Assert( pxSaveBlock, "Unable to create saveblock");

	if( NULL == pxSaveBlock)
	{
		return( false);
	}

	Asura_Chunk_SaveBlock_Stream xStream( pxSaveBlock);

	xStream.WriteChunk_Start( ASURA_RESOURCEFILE_TYPE_PS3_CARPET, ASURA_PS3_CARPET_CHUNK_VERSION, 0);

	uint32 uCGRMeshCount = s_uMeshesCount;

	xStream << uCGRMeshCount;

	for( uint32 uCGRMeshIndex = 0; uCGRMeshIndex < uCGRMeshCount; uCGRMeshIndex += 1)
	{
		const Asura_Direct3D_CGR_Mesh& xCGRMesh = s_pxMeshes[ uCGRMeshIndex];

		if( false == xCGRMesh.Export( xStream))
		{
			return( false);
		}
	}

	xStream << Asura_Carpet_MeshManager::GetMovementSpeed();

	xStream.WriteChunk_End();

#if 0
	//add the contents to the file
	// DLH - 16 February 2006 16:46:59 - Code Copied from Tools_Level_Export::AddAnimationTables()
	{
		DWORD uBytesWritten;
		uint32 uBytesToWrite = pxSaveBlock->GetSize() - ASURA_FILE_ID_NUM_CHARS;
		const void* pDataStart = pxSaveBlock->GetStartPtr() + ASURA_FILE_ID_NUM_CHARS;
		BOOL bSuccess = WriteFile( xFileHandle, pDataStart, uBytesToWrite, &uBytesWritten, 0);

		if( 0 == bSuccess || uBytesToWrite != uBytesWritten)
		{
			return( false);
			//AddMessage(MT_ERROR,"Error writing to file.");
			//SetAbortFlag();
		}
	}
#endif

#define WRITEOUTDATA(a,b) WriteFile( xFileHandle, a, b, &bytesWritten, 0 );

	{
		const uint32 uBytesHeaderSize = sizeof( Asura_Chunk_Header) + ASURA_FILE_ID_NUM_CHARS;
		//const uint32 uBytesHeaderSize = ASURA_FILE_ID_NUM_CHARS;
		uint32 uBytesToWrite = pxSaveBlock->GetSize() - uBytesHeaderSize;
		const void* pDataStart = pxSaveBlock->GetStartPtr() + uBytesHeaderSize;
		uint32 uChunkSize = uBytesToWrite + 4 + sizeof( Asura_Chunk_ResourceFile);

		Asura_Chunk_ResourceFile* pxResourceFile = new Asura_Chunk_ResourceFile;

		pxResourceFile->Create
			(
			uChunkSize,
			0,
			ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
			ASURA_RESOURCEFILE_TYPE_PS3_CARPET,
			uBytesToWrite,
			ASURA_PS3_CARPET_CHUNK_VERSION
			);

		unsigned long bytesWritten;

		WRITEOUTDATA( pxResourceFile, sizeof(Asura_Chunk_ResourceFile) );
		const char* szStringResourceFilename = "Cpt";
		WRITEOUTDATA( szStringResourceFilename, 4 );

		WRITEOUTDATA( pDataStart, uBytesToWrite);

		delete pxResourceFile;
	}

#undef WRITEOUTDATA

	delete pxSaveBlock;

	return( true);
}
#endif

bool Asura_Direct3D_CGR_PolygonStyle::Process( Asura_Chunk_File_Stream &xStream)
{
	for( uint32 uTableIndex = 0; uTableIndex < ASURA_DIRECT3D_CGR_POLYGONSTYLE_LOOKUP_LENGTH; uTableIndex += 1)
	{
		xStream >> this->m_aucStyleLookup[ uTableIndex];
	}

	return( true);
}

bool Asura_Direct3D_CGR_Style::Process( Asura_Chunk_File_Stream &xStream)
{
	xStream >> this->m_fWidth;
	xStream >> this->m_fHeight;
	xStream >> this->m_afU[0];
	xStream >> this->m_afU[1];
	xStream >> this->m_afV[0];
	xStream >> this->m_afV[1];
	xStream >> this->m_fScaleMin;
	xStream >> this->m_fLodFadeFar;
	xStream >> this->m_fLodFadeNear;
	xStream >> this->m_fLodFadeDelta;
	xStream >> this->m_fMovementScale;
	return( true);
}

bool Asura_Direct3D_CGR_PolygonBatch::Process( Asura_Chunk_File_Stream &xStream)
{
	xStream >> this->m_uDataOffset;
	xStream >> this->m_uDataSize;
	
	return( true);
}

bool Asura_Direct3D_CGR_Vertex::Process( Asura_Chunk_File_Stream &xStream)
{
	xStream >> this->m_fDensity;
	xStream >> this->m_xColourRGB;
	xStream >> this->m_xPosition;
	
	return( true);
}

bool Asura_Direct3D_CGR_Polygon::Process( Asura_Chunk_File_Stream &xStream)
{
	xStream >> this->m_fLodFadeFar;
	xStream >> this->m_fDenisty;
	xStream >> this->m_uInstanceCount;
	xStream >> this->m_uRandomSeed;
	xStream >> this->m_uPolygonStyle;
	
	xStream >> this->m_uVertexIndex[0];
	xStream >> this->m_uVertexIndex[1];
	xStream >> this->m_uVertexIndex[2];
	
	return( true);
}

bool Asura_Direct3D_CGR_RTreeNode::Process( Asura_Chunk_File_Stream &xStream, Asura_Direct3D_CGR_RTreeNode* pxRTreeNodeBase)
{
	xStream >> this->m_xBoundingBox;
	xStream >> this->m_uData;
	
	uint32 uSiblingIndex;
	uint32 uChildIndex;
	xStream >> uSiblingIndex;
	xStream >> uChildIndex;
	
	this->m_pxSibling = (uint32( -1) != uSiblingIndex) ? (pxRTreeNodeBase + uSiblingIndex) : NULL;
	this->m_pxChild = (uint32( -1) != uChildIndex) ? (pxRTreeNodeBase + uChildIndex) : NULL;
	
	return( true);
}

bool Asura_Direct3D_CGR_Mesh::Process( Asura_Chunk_File_Stream &xStream)
{
	Asura_Hash_ID xTextureHash;
	xStream >> xTextureHash;
	this->m_iTexture = Asura_Texture_Management::GetIndexFromHashID( xTextureHash);
	
	xStream >> this->m_xBoundingBox;
	xStream >> this->m_fLodFadeFar;
	
	{
		uint32 uPolygonCount;
		uint32 uPolygonBatchCount;
		uint32 uPolygonStyleCount;
		uint32 uRTreeCount;
		uint32 uStyleCount;
		uint32 uVertexCount;
		
		xStream >> uPolygonCount;
		xStream >> uPolygonBatchCount;
		xStream >> uPolygonStyleCount;
		xStream >> uRTreeCount;
		xStream >> uStyleCount;
		xStream >> uVertexCount;
		
		this->m_uPolygonCount = uPolygonCount;
		this->m_uPolygonBatchCount = uPolygonBatchCount;
		this->m_uPolygonStyleCount = uPolygonStyleCount;
		this->m_uRTreeCount = uRTreeCount;
		this->m_uStyleCount = uStyleCount;
		this->m_uVertexCount = uVertexCount;
	}
	
	this->m_pxPolygons = new Asura_Direct3D_CGR_Polygon[ this->m_uPolygonCount];
	this->m_pxPolygonBatches = new Asura_Direct3D_CGR_PolygonBatch[ this->m_uPolygonBatchCount];
	this->m_pxPolygonStyles = new Asura_Direct3D_CGR_PolygonStyle[ this->m_uPolygonStyleCount];
	this->m_pxRTree = new Asura_Direct3D_CGR_RTreeNode[ this->m_uRTreeCount];
	this->m_pxStyles = new Asura_Direct3D_CGR_Style[ this->m_uStyleCount];
	this->m_pxVertices = new Asura_Direct3D_CGR_Vertex[ this->m_uVertexCount];
	
	for( uint32 uVertexIndex = 0; uVertexIndex < this->m_uVertexCount; uVertexIndex += 1)
	{
		if( false == this->m_pxVertices[ uVertexIndex].Process( xStream))
		{
			return( false);
		}
	}
	
	for( uint32 uPolygonIndex = 0; uPolygonIndex < this->m_uPolygonCount; uPolygonIndex += 1)
	{
		if( false == this->m_pxPolygons[ uPolygonIndex].Process( xStream))
		{
			return( false);
		}
	}
	
	for( uint32 uPolygonStyleIndex = 0; uPolygonStyleIndex < this->m_uPolygonStyleCount; uPolygonStyleIndex += 1)
	{
		if( false == this->m_pxPolygonStyles[ uPolygonStyleIndex].Process( xStream))
		{
			return( false);
		}
	}
	
	for( uint32 uPolygonBatchIndex = 0; uPolygonBatchIndex < this->m_uPolygonBatchCount; uPolygonBatchIndex += 1)
	{
		if( false == this->m_pxPolygonBatches[ uPolygonBatchIndex].Process( xStream))
		{
			return( false);
		}
	}
	
	for( uint32 uRTreeNodeIndex = 0; uRTreeNodeIndex < this->m_uRTreeCount; uRTreeNodeIndex += 1)
	{
		if( false == this->m_pxRTree[ uRTreeNodeIndex].Process( xStream, this->m_pxRTree))
		{
			return( false);
		}
	}
	
	for( uint32 uStyleIndex = 0; uStyleIndex < this->m_uStyleCount; uStyleIndex += 1)
	{
		if( false == this->m_pxStyles[ uStyleIndex].Process( xStream))
		{
			return( false);
		}
	}
	
	return( true);
}

bool Asura_Chunk_ResourceFile_PC_Carpet::Process( Asura_Handle hFile )
{
	Asura_Chunk_File_Stream	xStream( hFile);
	
	if (Version!= Asura_Chunk_ResourceFile_PC_Carpet::CURRENT_VERSION )
	{
		Asura_Assert(false, "Carpet chunk version has changed - Please re-export level");
		return false;
	}

	uint32 uCGRMeshCount;
	xStream >> uCGRMeshCount;
	
	Asura_Assert( NULL == Asura_Direct3D_Carpet_Game_Render::s_pxMeshes, "Need to clear the CGR Meshes");
	Asura_Assert( 0 == Asura_Direct3D_Carpet_Game_Render::s_uMeshesCount, "Need to clear the CGR Meshes");
	Asura_Direct3D_Carpet_Game_Render::s_uMeshesCount = uCGRMeshCount;
	Asura_Direct3D_Carpet_Game_Render::s_pxMeshes = new Asura_Direct3D_CGR_Mesh[ Asura_Direct3D_Carpet_Game_Render::s_uMeshesCount];
	
	for( uint32 uCGRMeshIndex = 0; uCGRMeshIndex < uCGRMeshCount; uCGRMeshIndex += 1)
	{
		Asura_Direct3D_CGR_Mesh& xCGRMesh = Asura_Direct3D_Carpet_Game_Render::s_pxMeshes[ uCGRMeshIndex];
		
		if( false == xCGRMesh.Process( xStream))
		{
			return false;
		}
	}

	float fMovementSpeed;
	xStream >> fMovementSpeed;
	Asura_Carpet_MeshManager::SetMovementSpeed(fMovementSpeed);

	Asura_Direct3D_Carpet_Game_Render::CreateDirect3DData();
	Asura_Direct3D_Carpet_Game_Render::BatchCopyAllToBuffers();

	return true;
}


//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_CARPET)
