//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_Stream_WindowsFile.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Stream for reading / writing using windows file by HANDLE
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_CHUNK_STREAM_WINDOWSFILE_H___
#define ___ASURA_CHUNK_STREAM_WINDOWSFILE_H___ 1

#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_PlatformBasics.h"
#include "Debugging/Asura_Assert.h"



//This now has a small buffer to store the output before writing to the file.
//If the file is being closed or otherwise manipulated by other means before the
//stream goes out of scope (or is deleted) then FlushOutput() will need to be called
//to make sure any remaining data gets written first.
//RAR 08/08/2008

class Asura_Chunk_Stream_WindowsFile : public Asura_Chunk_Stream
{
	static const u_int uBUFFER_SIZE = 1024;
public:

	//Constructor for a stream using a previously opened file
	Asura_Chunk_Stream_WindowsFile(HANDLE xFile);

	//Constructor for a stream that will handle opening/closing the file itself
	//If using this then CreateFile() should be used to open the file
	//Once finished CloseFile() should be used to finish writing to the file.
	Asura_Chunk_Stream_WindowsFile();

	~Asura_Chunk_Stream_WindowsFile();
	
	//Starts writing to a new file
	//The file will be opened using a temporary filename , and then renamed when it is closed
	bool CreateFile(const char* szFilename,bool bAddAsuraHeaderAndTerminator);

	//Finishes writing to the file , and renames it
	bool CloseFile();

	//Aborts writing to the file and deletes it
	void DeleteFile();

	// Implementation of pure virtual Asura_Chunk_Stream functions
	virtual u_int Read(u_int uNumberOfBytes, void* pData);
	virtual u_int Skip(u_int uNumberOfBytes);
	virtual u_int Write(u_int uNumberOfBytes, const void* pData);
	virtual u_int Write(u_int uNumberOfBytes, const void* pData, u_int uPosition);
	virtual u_int GetPosition(void) const;
	virtual int PeekInt();

	u_int GetFilePosition(void) const;
	void SetFilePosition( const u_int uPosition) const;

	//Forces any buffered output to be written to the file now.
	void FlushOutput();

private:

	// Private variables...
	HANDLE m_xFile;

	u_int m_uBufferPosition;
	char m_acBuffer[uBUFFER_SIZE];

	u_int m_uBufferStartFilePosition;

	//For streams that are looking after their own file handle :
	Asura_String m_xFilename;//final filename to use
	Asura_String m_xTempFilename;//temporary filename to save under
	bool m_bOwnsFileHandle;
	bool m_bAddAsuraHeaderAndTerminator;

	// Private functions... (unimplemented copy ctor and operator=)
	Asura_Chunk_Stream_WindowsFile(const Asura_Chunk_Stream_WindowsFile& xOther);
	Asura_Chunk_Stream_WindowsFile& operator=(const Asura_Chunk_Stream_WindowsFile& xOther);

};   // End of class 'Asura_Chunk_Stream_WindowsFile'.


// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_CHUNK_STREAM_WINDOWSFILE_H___
