//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_Chunk_Mesh.cpp
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Description:
//
//     Platform implementations of mesh chunk processing functionality.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_TOOLS

#include "ChunkSystem/Asura_Chunk_Mesh.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Core/Asura_Handle.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"
//#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Material_System.h"

#include <float.h>

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool Asura_Chunk_Mesh_Normals::Platform_Process( Asura_Handle hFile ) const
{
	const Asura_Char* szMeshname = Asura_FileAccess::ReadString( hFile );
	
	Asura_Mesh* pxMeshPtr = Asura_Mesh_System::GetMesh( szMeshname, true );

	if( pxMeshPtr->AreNormalsValid() )
	{
		// Kevin Lea - 13/09/2001 15:52:55 - mesh already exists
		// this could be a naming problem (duplicates)
		// otherwise if this happens regularly we'll have to read off
		// the remainder of the chunk to keep things in sync
		Asura_Assertf( false, ("Mesh: '%s'  - Already exists", szMeshname) );
        Asura_FileAccess::SkipData( hFile, (5 * sizeof(Asura_Vector_3)) * NumberOfPolygons );
		return true;
	}
	if(pxMeshPtr->NumberOfPolygons != NumberOfPolygons)
	{
		//Return here if there are the wrong number of polygons , since there can be problems when duplicate
		//meshes are loaded if the first version didn't have any normals loaded (and so won't get
		//caught by the earlier assertion). RAR 29/05/03
		Asura_Assert( pxMeshPtr->NumberOfPolygons == NumberOfPolygons, "Wrong number of polygons in normal data" );
		return false;
	}

	pxMeshPtr->NormalsDataPtr = new Asura_Mesh_Normals[NumberOfPolygons];
    for( int i = 0; i < NumberOfPolygons; ++i )
    {
	    Asura_FileAccess::ReadData( hFile, &pxMeshPtr->NormalsDataPtr[i].FaceNormal, sizeof(Asura_Vector_3) );

        //pxMeshPtr->NormalsDataPtr[i].VertexNormal = new Asura_Vector_3[4];
        Asura_FileAccess::ReadData( hFile, pxMeshPtr->NormalsDataPtr[i].VertexNormal, 4 * sizeof(Asura_Vector_3) );
    }

	pxMeshPtr->m_bFaceNormalsValid = true;
	pxMeshPtr->m_bVertexNormalsValid = true;
	#ifdef ASURA_USE_STREAMING
	//For streaming always assume that the vertex normals will need to be recalculated , to avoid problems with
	//tracking changes to meshes in adjacent zones that may cause the normals to be invalidated.
	//RAR 13/07/2006
	pxMeshPtr->m_bSmoothedAcrossModules = false;
	#else
	pxMeshPtr->m_bSmoothedAcrossModules = (Flags & ChunkFlag_SmoothedAcrossModules) ? true : false;
	#endif

	return true;
}


#endif // #ifdef ASURA_TOOLS
