
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Handler_System_Callbacks_PC.cpp
// Author : Andy Tate
// Date   : 30-Jan-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_MULTIPLE_HANDLERS)

// This file's header...
#include "Core/Handlers/Asura_Handler_System_Callbacks_PC.h"

// Asura independent header-files...
#include "Core/Handlers/Asura_Handler_System.h"

// Asura platform header-files...
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)
#include "Rendering/Asura_PlatformRenderCharacter.h"
#endif	//!defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)
#include "Rendering/Asura_PlatformRenderObject.h"
#include "Rendering/Asura_Platform_RenderObjectHierarchy.h"
#if defined(ASURA_TARGET_SM3)
#include "Rendering/Asura_Direct3D_EnvironmentRenderer.h"
#else
#include "Rendering/Asura_PlatformRenderEnvironment.h"
#endif	// defined(ASURA_TARGET_SM3)


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

// Creating a single instance forces registration with the "Asura_Handler_System"...
static Asura_Handler_System_Callbacks_PC	s_xPlatformHandlerSystemCallbacks;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PC::Asura_Handler_System_Callbacks_PC
// Author       : Andy Tate
// Date         : 30-Jan-2008
////////////////////////////////////////////////////////////////////////////////

Asura_Handler_System_Callbacks_PC::Asura_Handler_System_Callbacks_PC(void)
{
	Asura_Handler_System::RegisterPlatformCallbacks(this);
}

/// End of function 'Asura_Handler_System_Callbacks_PC::Asura_Handler_System_Callbacks_PC'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PC::CreateHandlerData
// Author       : Andy Tate
// Date         : 24-Mar-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System_Callbacks_PC::CreateHandlerData(u_int& uHandlerIndex, 
														  const Asura_Hash_ID uHandlerHash,
														  const u_int uDataIndex,
														  const char* szName,
														  const Asura_Hash_ID uParentHandlerHash)
{
	const u_int	uIndex	= Asura_Handler_System::GetIndexWithGroup(uHandlerIndex);

	switch(uIndex)
	{
		case 0:
		{
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)
			Asura_Direct3D_CharacterRenderer::CreateAndSetHandler(uHandlerHash, uParentHandlerHash);
#endif	//!defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)
			uHandlerIndex++;
			return(true);
		}

		case 1:
		{
			Asura_Direct3D_ObjectRenderer::CreateAndSetHandler(uHandlerHash, uParentHandlerHash);
			uHandlerIndex++;
			return(true);
		}

		case 2:
		{
			Asura_Direct3D_ObjectHierarchyRenderer::CreateAndSetHandler(uHandlerHash, uParentHandlerHash);
			return(false);
		}

		default:
		{
			Asura_Assert(false, "Unexpected index for sub-system handler to be created.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System_Callbacks_PC::CreateHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PC::DestroyHandlerData
// Author       : Andy Tate
// Date         : 24-Mar-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System_Callbacks_PC::DestroyHandlerData(u_int& uHandlerIndex,
														   const Asura_Hash_ID uHandlerHash,
														   const u_int uDataIndex)
{
	const u_int	uIndex	= Asura_Handler_System::GetIndexWithGroup(uHandlerIndex);

	switch(uIndex)
	{
		case 0:
		{
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)
			Asura_Direct3D_CharacterRenderer::DestroyHandler(uHandlerHash);
#endif	//!defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)
			uHandlerIndex++;
			return(true);
		}

		case 1:
		{
			Asura_Direct3D_ObjectRenderer::DestroyHandler(uHandlerHash);
			uHandlerIndex++;
			return(true);
		}

		case 2:
		{
			Asura_Direct3D_ObjectHierarchyRenderer::DestroyHandler(uHandlerHash);
			return(false);
		}

		default:
		{
			Asura_Assert(false, "Unexpected index for sub-system handler to be destroyed.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System_Callbacks_PC::DestroyHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PC::ReparentAllHandlerData
// Author       : Andy Tate
// Date         : 29-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System_Callbacks_PC::ReparentAllHandlerData(const Asura_Hash_ID uHandlerHash,
															   const Asura_Hash_ID uParentHandlerHash)
{
	// SetAll the render data...
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)
	Asura_Direct3D_CharacterRenderer::ReparentHandler(uHandlerHash, uParentHandlerHash);
#endif	//!defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)
	Asura_Direct3D_ObjectRenderer::ReparentHandler(uHandlerHash, uParentHandlerHash);
	Asura_Direct3D_ObjectHierarchyRenderer::ReparentHandler(uHandlerHash, uParentHandlerHash);
}

/// End of function 'Asura_Handler_System_Callbacks_PC::ReparentAllHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PC::SetAllHandlerData
// Author       : Andy Tate
// Date         : 24-Mar-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System_Callbacks_PC::SetAllHandlerData(const Asura_Hash_ID uHandlerHash,
														  const u_int uDataIndex)
{
	// SetAll the render data...
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)
	Asura_Direct3D_CharacterRenderer::SetCurrentHandler(uHandlerHash);
#endif	//!defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)
	Asura_Direct3D_ObjectRenderer::SetCurrentHandler(uHandlerHash);
	Asura_Direct3D_ObjectHierarchyRenderer::SetCurrentHandler(uHandlerHash);
}

/// End of function 'Asura_Handler_System_Callbacks_PC::SetAllHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PC::CreateZoneData
// Author       : Andy Tate
// Date         : 24-Mar-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System_Callbacks_PC::CreateZoneData(u_int& uHandlerIndex, 
													   const Asura_Hash_ID uHandlerHash,
													   const u_int uDataIndex,
													   const char* szName,
													   const Asura_Hash_ID uParentHandlerHash,
													   const Asura_Hash_ID uZoneHash,
													   const bool bAllowEnvironment)
{
	const u_int	uIndex	= Asura_Handler_System::GetIndexWithGroup(uHandlerIndex);

	switch(uIndex)
	{
		case 0:
		{
			if(bAllowEnvironment)
			{
#if defined(ASURA_USE_STREAMING)
#if defined(ASURA_TARGET_SM3)
				Asura_Direct3D_EnvRenderer::CreateAndSetZone(uHandlerHash);
#endif	//defined(ASURA_TARGET_SM3)
#endif	//defined(ASURA_USE_STREAMING)
			}
			return(false);
		}

		default:
		{
			Asura_Assert(false, "Unexpected index for sub-system handler to be created.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System_Callbacks_PC::CreateZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PC::DestroyZoneData
// Author       : Andy Tate
// Date         : 24-Mar-2006
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System_Callbacks_PC::DestroyZoneData(u_int& uHandlerIndex,
														const Asura_Hash_ID uHandlerHash,
														const u_int uDataIndex,
														const bool bAllowEnvironment)
{
	const u_int	uIndex	= Asura_Handler_System::GetIndexWithGroup(uHandlerIndex);

	switch(uIndex)
	{
		case 0:
		{
			if(bAllowEnvironment)
			{
#if defined(ASURA_USE_STREAMING)
				Asura_Direct3D_EnvRenderer::DestroyZone(uHandlerHash);
#else
#if defined(ASURA_TARGET_SM3)
				Asura_Direct3D_EnvironmentRenderer::Reset();
#else
				Asura_Direct3D_EnvRenderer::Reset();
#endif	//defined(ASURA_TARGET_SM3)
#endif	//defined(ASURA_USE_STREAMING)
			}
			return(false);
		}

		default:
		{
			Asura_Assert(false, "Unexpected index for sub-system handler to be destroyed.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System_Callbacks_PC::DestroyZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PC::ReparentAllZoneData
// Author       : Andy Tate
// Date         : 29-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System_Callbacks_PC::ReparentAllZoneData(const Asura_Hash_ID uHandlerHash,
															const Asura_Hash_ID uParentHandlerHash)
{
}

/// End of function 'Asura_Handler_System_Callbacks_PC::ReparentAllZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PC::SetAllZoneData
// Author       : Andy Tate
// Date         : 24-Mar-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System_Callbacks_PC::SetAllZoneData(const Asura_Hash_ID uHandlerHash,
													   const u_int uDataIndex,
													   const bool bAllowEnvironment)
{
	// SetAll the render data...
	if(bAllowEnvironment)
	{
#if defined(ASURA_USE_STREAMING)
#if defined(ASURA_TARGET_SM3)
		Asura_Direct3D_EnvRenderer::SetCurrentZone(uHandlerHash);
#endif	//defined(ASURA_TARGET_SM3)
#endif	//defined(ASURA_USE_STREAMING)
	}
}

/// End of function 'Asura_Handler_System_Callbacks_PC::SetAllZoneData'. ///


#if defined(ASURA_LOGFILES_ENABLED)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PC::CreateReport
// Author       : Andy Tate
// Date         : 29-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System_Callbacks_PC::CreateReport(const Asura_Hash_ID uHandlerHash,
													 Asura_Stream& xOutputStream)
{
}

/// End of function 'Asura_Handler_System_Callbacks_PC::CreateReport'. ///

#endif	//defined(ASURA_LOGFILES_ENABLED)

#endif	//defined(ASURA_USE_MULTIPLE_HANDLERS)


/// End of file. ///
