//*******************************************************************
//
// File: Asura_PlatformFSFX_GammaRGB.cpp     
//
// Created: 22 Feb 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_FSFX)

// main header
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"
#include "FSFX/Asura_FSFX_Component.h"

#include "Rendering/Asura_ViewPort.h"
#include "Rendering/Asura_Render.h"

#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_Shaders.h"


// other libraries


// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************

struct SimpleVertex
{
	float m_fX, m_fY, m_fZ;
	float m_fU, m_fV;
};

static SimpleVertex axQUAD_VERTEX_DATA[4] =
{
	{ -1.0f,  1.0f,  0.0f,  0.0f,  0.0f },
	{  1.0f,  1.0f,  0.0f,  1.0f,  0.0f },
	{  1.0f, -1.0f,  0.0f,  1.0f,  1.0f },
	{ -1.0f, -1.0f,  0.0f,  0.0f,  1.0f },
};



//*******************************************************************
// Function Definitions
//*******************************************************************

static void Asura_FSFX_Component_Cubic_Platform_Render( const ASURA_D3D_VERTEXSHADER_ID eVertexShaderID, const ASURA_D3D_PIXELSHADER_ID ePixelShaderID, const uint32 uCoeffCount, const float* pxCoeffs)
{
	// set up some states that persist across the effect

	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_OPAQUE);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestAlphaTest(false);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	// Use clamping of tex co-ords

	DWORD uOldAddressU[4], uOldAddressV[4];
	Direct3D_BaseTexture* pxOldTexture[4];
	for (u_int u = 0; u < 1; ++u)
	{
		Asura_Direct3D::D3DDevice->GetSamplerState(u, D3DSAMP_ADDRESSU, &uOldAddressU[u]);
		Asura_Direct3D::D3DDevice->GetSamplerState(u, D3DSAMP_ADDRESSV, &uOldAddressV[u]);

		Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
		Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);

		Asura_Direct3D::D3DDevice->GetTexture(u, &pxOldTexture[u]);
	}

	// ensure that we have a backbuffer capture knocking about
	if(!Asura_Direct3D_BackBufferCapture::HasCapturedThisFrame())
	{
		Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();
	}
	Asura_Render::AllowFullScreenRendering();

	// DLH - 22 February 2006 12:52:16 - I have 2* the blur buffer to get the size back to what the original buffer is. Stops the vertical bluring twice as fast as the horizontal.
	const float fInvBufferWidth = 0.5f / static_cast<float>( Asura_Direct3D::GetPresentParameters().BackBufferWidth);
	const float fInvBufferHeight = 0.5f / static_cast<float>( Asura_Direct3D::GetPresentParameters().BackBufferHeight);
	
	axQUAD_VERTEX_DATA[0].m_fU = 0.0f + fInvBufferWidth;
	axQUAD_VERTEX_DATA[0].m_fV = 0.0f + fInvBufferHeight;
	axQUAD_VERTEX_DATA[1].m_fU = 1.0f + fInvBufferWidth;
	axQUAD_VERTEX_DATA[1].m_fV = 0.0f + fInvBufferHeight;
	axQUAD_VERTEX_DATA[2].m_fU = 1.0f + fInvBufferWidth;
	axQUAD_VERTEX_DATA[2].m_fV = 1.0f + fInvBufferHeight;
	axQUAD_VERTEX_DATA[3].m_fU = 0.0f + fInvBufferWidth;
	axQUAD_VERTEX_DATA[3].m_fV = 1.0f + fInvBufferHeight;
	
	// set up the blur shaders
	Asura_Direct3D_Shaders::RequestVertexShader( eVertexShaderID);
	Asura_Direct3D_Shaders::RequestPixelShader( ePixelShaderID);
	Asura_Direct3D::D3DDevice->SetPixelShaderConstantF(0, pxCoeffs, uCoeffCount);

	// set the backbuffer capture as the textures
	Asura_Direct3D::D3DDevice->SetTexture( 0, Asura_Direct3D_BackBufferCapture::GetCaptureTexture());

	#if defined ASURA_TARGET_SM3
		// At this point, the frame buffer should be in sRGB space
		const bool bSRGB = Asura_Direct3D::UseGammaCorrection();
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_SRGBTEXTURE, bSRGB );
	#endif

	axQUAD_VERTEX_DATA[0].m_fZ = axQUAD_VERTEX_DATA[1].m_fZ = axQUAD_VERTEX_DATA[2].m_fZ = axQUAD_VERTEX_DATA[3].m_fZ = 0.0f;

	Asura_Direct3D::D3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, axQUAD_VERTEX_DATA, sizeof(SimpleVertex));

////// Finished

	// clean up
	for(u_int u = 0; u < 1; ++u)
	{
		Asura_Direct3D::D3DDevice->SetTexture(u, pxOldTexture[u]);
		if(pxOldTexture[u]) pxOldTexture[u]->Release();

		Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_ADDRESSU, uOldAddressU[u]);
		Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_ADDRESSV, uOldAddressV[u]);
	}
	
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);

	Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_NONE);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	
	Asura_Direct3D_BackBufferCapture::AdviseNextFrame();
	Asura_Render::ApplyCameraSettings();

	#if defined ASURA_TARGET_SM3
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_SRGBTEXTURE, FALSE );
	#endif
}

static void Asura_FSFX_Component_RGBGamma_Platform_Render( const Asura_Vector_3& xGamma)
{
	const float fTangentDelta = 0.1f;
	
	const Asura_Vector_3 xy0( 0.0f, 0.0f, 0.0f);
	const Asura_Vector_3 xy1( 0.5f * pow( fTangentDelta, 1.0f / xGamma.x),		0.5f * pow( fTangentDelta, 1.0f / xGamma.y),		0.5f * pow( fTangentDelta, 1.0f / xGamma.z));
	const Asura_Vector_3 xy2( 0.5f * pow( 1.0f-fTangentDelta, 1.0f / xGamma.x),	0.5f * pow( 1.0f-fTangentDelta, 1.0f / xGamma.y),	0.5f * pow( 1.0f-fTangentDelta, 1.0f / xGamma.z));
	const Asura_Vector_3 xy3( 0.5f, 0.5f, 0.5f);
	
	// DLH - 23 February 2006 12:38:16 - Cubic Hermit Spline
	const Asura_Vector_3 xp0 = xy0;
	const Asura_Vector_3 xp1 = xy3;
	const Asura_Vector_3 xm0 = (xy1 - xy0) / fTangentDelta;
	const Asura_Vector_3 xm1 = (xy3 - xy2) / fTangentDelta;
	const Asura_Vector_3 xt3 = (xp0 * 2.0f) + (xm0 * 1.0f) + (xp1 * -2.0f) + (xm1 * 1.0f);
	const Asura_Vector_3 xt2 = (xp0 * -3.0f) + (xm0 * -2.0f) + (xp1 * +3.0f) + (xm1 * -1.0f);
	const Asura_Vector_3 xt1 = (xm0);
	const Asura_Vector_3 xt0 = (xp0);
	

	float afPSConsts[4][4] =
	{
		{ xt0.x, xt0.y, xt0.z, 1.0f},
		{ xt1.x, xt1.y, xt1.z, 0.0f},
		{ xt2.x, xt2.y, xt2.z, 0.0f},
		{ xt3.x, xt3.y, xt3.z, 0.0f}
	};
	
#if defined( _DEBUG)
	float fValue = 0.0f;
	const Asura_Vector_3 xTest0 = (xt0) + (xt1 * fValue) + (xt2 * fValue * fValue) + (xt3 * fValue * fValue * fValue);
	fValue = 0.333f;
	const Asura_Vector_3 xTest13 = (xt0) + (xt1 * fValue) + (xt2 * fValue * fValue) + (xt3 * fValue * fValue * fValue);
	fValue = 0.5f;
	const Asura_Vector_3 xTest12 = (xt0) + (xt1 * fValue) + (xt2 * fValue * fValue) + (xt3 * fValue * fValue * fValue);
	fValue = 0.666f;
	const Asura_Vector_3 xTest23 = (xt0) + (xt1 * fValue) + (xt2 * fValue * fValue) + (xt3 * fValue * fValue * fValue);
	fValue = 1.0f;
	const Asura_Vector_3 xTest33 = (xt0) + (xt1 * fValue) + (xt2 * fValue * fValue) + (xt3 * fValue * fValue * fValue);
#endif
	
	Asura_FSFX_Component_Cubic_Platform_Render( ASURA_D3D_VS_CUBIC, ASURA_D3D_PS_CUBIC, 4, &afPSConsts[0][0]);
}

void Asura_FSFX_Component_BrightnessAndContrast::Platform_Render( const float fIntensity )
{
	SCOPEDPIXEVENT(( 0xFFE0D0C0, L"FSFX: Brightness and Contrast" ));

	const float fBrightness = fIntensity * this->m_fBrightness / 100.0f;
	const float fScale = (fIntensity * this->m_fContrast / 100.0f) + 1.0f;
	
	const float fTerm0 = 0.5f * ((-0.5f * fScale) + 0.5f + fBrightness);
	const float fTerm1 = 0.5f * (fScale);
	
	float afPSConsts[][4] =
	{
		{ fTerm0, fTerm0, fTerm0, 0.0f},
		{ fTerm1, fTerm1, fTerm1, 1.0f},
	};
	
	Asura_FSFX_Component_Cubic_Platform_Render( ASURA_D3D_VS_LINEAR, ASURA_D3D_PS_LINEAR, 2, &afPSConsts[0][0]);
}

void Asura_FSFX_Component_RGBGamma::Platform_Render( const float fIntensity )
{
	SCOPEDPIXEVENT(( 0xFFE0D0C0, L"FSFX: Gamma RGB" ));

	const float fInvIntensity = 1.0f - fIntensity;
	const Asura_Vector_3 xGamma( fInvIntensity + (fIntensity * this->m_fAmountR), fInvIntensity + (fIntensity * this->m_fAmountG), fInvIntensity + (fIntensity * this->m_fAmountB));
	Asura_FSFX_Component_RGBGamma_Platform_Render( xGamma);
}

void Asura_FSFX_Component_Gamma::Platform_Render( const float fIntensity )
{
	SCOPEDPIXEVENT(( 0xFFE0D0C0, L"FSFX: Gamma" ));

	const float fInvIntensity = 1.0f - fIntensity;
	const float fGamma = fInvIntensity + (fIntensity * this->m_fAmount);
	const Asura_Vector_3 xGamma( fGamma, fGamma, fGamma);
	
	Asura_FSFX_Component_RGBGamma_Platform_Render( xGamma);
}

//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_FSFX)
