//*******************************************************************
//
// File: Asura_PlatformFSFX_Noise.cpp        
//
// Created: 27 Feb 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_FSFX)

#include "Camera/Asura_Camera.h"
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"
#include "FSFX/Asura_FSFX_Component.h"

#include "Rendering/Asura_ViewPort.h"
#include "Rendering/Asura_Render.h"

#include "Rendering/Asura_Direct3D.h"

// main header

// other libraries

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************


//*******************************************************************
// Function Definitions
//*******************************************************************

void Asura_FSFX_Component_Noise::Platform_Render( const float fIntensity )
{
	if( ASURA_TEXTURE_INVALID == this->m_iTextureID)
	{
		return;
	}

	SCOPEDPIXEVENT(( 0xFFE0D0C0, L"FSFX: Noise" ));

	const Asura_ViewingArea*	pxViewingArea	= Asura_Camera::GetViewingArea();
	const float 				fScreenWidth	= static_cast<float>(Asura_Viewport::GetWidth());
	const float 				fScreenHeight	= static_cast<float>(Asura_Viewport::GetHeight());
	const float 				fXOffset		= static_cast<float>(pxViewingArea->m_fXOffset * fScreenWidth);
	const float 				fYOffset		= static_cast<float>(pxViewingArea->m_fYOffset * fScreenHeight);
	const float 				fWidth			= static_cast<float>(pxViewingArea->m_fWidth * fScreenWidth);
	const float 				fHeight			= static_cast<float>(pxViewingArea->m_fHeight * fScreenHeight);

	const float fTextureRandOffsetU = Asura_Random::GetFloat( 0.0f, 1.0f);
	const float fTextureRandOffsetV = Asura_Random::GetFloat( 0.0f, 1.0f);
	
	const uint32 uTextureTileU = this->m_uTextureTileX;
	const uint32 uTextureTileV = this->m_uTextureTileY;
	const float fURatio = 1.0f / static_cast< float>( uTextureTileU);
	const float fVRatio = 1.0f / static_cast< float>( uTextureTileV);
	const float fUScale = (Asura_Random::GetFloat( 0.0f, 1.0f) <= 0.5f) ? 1.0f : -1.0f;
	const float fVScale = (Asura_Random::GetFloat( 0.0f, 1.0f) <= 0.5f) ? 1.0f : -1.0f;
	
	float fUStart = fTextureRandOffsetU;
	float fVStart = fTextureRandOffsetV;
	float fUSize = fUScale / fURatio;
	float fVSize = fVScale / fVRatio;
	
	const float fEffectStrength = fIntensity * this->m_fAmount / 100.0f;
	const uint32 uEffectStrength = static_cast< uint32>( 0x80 * fEffectStrength);
	const uint32 uEffectColour = 0xFF000000 | (uEffectStrength << 16) | (uEffectStrength << 8) | (uEffectStrength << 0);
	
	Asura_Render::Overlay.Colour           = uEffectColour;
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_UGH;
	Asura_Render::Overlay.TextureIndex     = this->m_iTextureID;

	Asura_Render::Overlay.SetRectXY( fXOffset, fYOffset, fWidth, fHeight );
	Asura_Render::Overlay.SetRectUV( fUStart, fVStart, fUSize, fVSize);
	Asura_Render::Overlay.Render();   
	// TODO stop this using batched quads!
	Asura_Direct3D::FlushQuadBatches();
}


//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_FSFX)
