//*******************************************************************
//
// File: Asura_PlatformFSFX_Bloom.cpp        
//
// Created: 2 Mar 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_FSFX)

#ifdef ASURA_TARGET_SM3

// main header
#include "FSFX/Asura_FSFX_Component.h"
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"

#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_UserOptions.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_ViewPort.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Overlay_Quad.h"

#include "FSFX/Asura_Platform_FSFX_SM30.h"

// other libraries
#include "SpecialFX/Asura_Glow_System.h"

//*******************************************************************
// Globals
//*******************************************************************

Asura_Direct3D_RenderTarget* g_apxBloomBuffers[2]			= {NULL, NULL};
static int						    g_iAccumBuffer				= 0;

void Asura_FSFX_Component_SM30_Bloom::Platform_Initialise()
{
	for(int i=0;i<2;i++)
	{	
		g_apxBloomBuffers[i]= NULL;
	}	
}



void Asura_FSFX_Component_SM30_Bloom::Platform_Reset()
{
	Platform_Shutdown();
	Platform_Initialise();
}


void Asura_FSFX_Component_SM30_Bloom::Platform_Shutdown()
{
	for(int i=0;i<2;i++)
	{	
		if (g_apxBloomBuffers[i])
		{		
			g_apxBloomBuffers[i]->MarkAsUnused();
			g_apxBloomBuffers[i]= NULL;
		}
	}	
}


void Asura_FSFX_Component_SM30_Bloom::Platform_Render( const float fIntensity )
{

	HRESULT xResult = 0;
	(void) xResult;		// frig to get rid of warnings when asserts are disabled...

	if ( fIntensity == 0.0f || Asura_Platform_FSFX_SM30::g_pxBloomEffect == NULL )
	{
		return;
	}

	SCOPEDPIXEVENT(( 0xFFA89C90, L"FSFX: Advanced Bloom" ));
	
	bool bAccumBufferJustAllocated=false;

	for (int i=0;i<2;i++)
	{
		if (g_apxBloomBuffers[i]==NULL)
		{	
			g_apxBloomBuffers[i] = Asura_Direct3D_RenderTargetManager::FindUnusedRenderTarget( Asura_Platform_FSFX_SM30::g_fBloomEffectBlurBufferScale, Asura_Platform_FSFX_SM30::g_fBloomEffectBlurBufferScale, D3DFMT_A8R8G8B8, D3DFMT_UNKNOWN, true, 1, false);
			bAccumBufferJustAllocated = true;
		}
	}


	if( NULL == g_apxBloomBuffers[0] || NULL == g_apxBloomBuffers[1])
	{
		return;
	}

	g_iAccumBuffer = (g_iAccumBuffer+1)&1;

	// Ensure that we have a back buffer capture
	if(!Asura_Direct3D_BackBufferCapture::HasCapturedThisFrame())
	{
		Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();
	}

	Asura_Render::AllowFullScreenRendering();

	Asura_Render::CurrentStates.RequestAlphaTest(false);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestZBufferEnabled(false);

	// Get the current render target
	Direct3D_Surface* pxOldRenderTargetBuffer = NULL;
	Direct3D_Surface* pxOldRenderTargetDepth = NULL;
	xResult = Asura_Direct3D::D3DDevice->GetRenderTarget(0, &pxOldRenderTargetBuffer);
	ASURA_D3D_OK( xResult);
	xResult = Asura_Direct3D::D3DDevice->GetDepthStencilSurface(&pxOldRenderTargetDepth);
	ASURA_D3D_OK( xResult);

	const bool bSRGB = Asura_Direct3D::UseGammaCorrection();
	Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_SRGBTEXTURE, bSRGB );
	Asura_Direct3D::D3DDevice->SetSamplerState( 1, D3DSAMP_SRGBTEXTURE, bSRGB );

	// First pass - from the back buffer to the blur accumulation buffer, extracting the parts we want to bloom
	// If we're accumulating it, it's an additive pass, else alpha blending is disabled
	// ---------------------------------------------------------------------------------------------------------------
	const float fViewWidth		= static_cast< float> ( Asura_Direct3D::GetPresentParameters().BackBufferWidth);
	const float fViewHeight		= static_cast< float> ( Asura_Direct3D::GetPresentParameters().BackBufferHeight);
	const float fBufferWidth	= static_cast< float> ( g_apxBloomBuffers[g_iAccumBuffer]->GetWidth());
	const float fBufferHeight	= static_cast< float> ( g_apxBloomBuffers[g_iAccumBuffer]->GetHeight());
	
	float fHalfTexelU = 0.5f/fViewWidth;
	float fHalfTexelV = 0.5f/fViewHeight;

	BloomVertex axVerts[4];

	axVerts[0].fX = -1.0f;     axVerts[0].fY =  1.0f;
	axVerts[1].fX = -1.0f;     axVerts[1].fY = -1.0f;
	axVerts[2].fX =  1.0f;     axVerts[2].fY =  1.0f; 
	axVerts[3].fX =  1.0f;     axVerts[3].fY = -1.0f; 
	
	axVerts[0].fU = 0.0f+fHalfTexelU;	   axVerts[0].fV = 0.0f+fHalfTexelV;
	axVerts[1].fU = 0.0f+fHalfTexelU;	   axVerts[1].fV = 1.0f+fHalfTexelV;
	axVerts[2].fU = 1.0f+fHalfTexelU;	   axVerts[2].fV = 0.0f+fHalfTexelV;
	axVerts[3].fU = 1.0f+fHalfTexelU;	   axVerts[3].fV = 1.0f+fHalfTexelV;

	float afSecondaryUVOffset[2] = {-fHalfTexelU + (0.5f/fBufferWidth), -fHalfTexelV + (0.5f/fBufferWidth)};

	// Clear any unused render targets (in case deferred shading targets are still active)
	unsigned int u;
	for (u = 1; u < Asura_Direct3D_Caps::GetMaxNumberOfRenderTargets(); ++u)
	{
		Asura_Direct3D::D3DDevice->SetRenderTarget(u, NULL);
	}
	
	// Set the render target
	Asura_Direct3D::D3DDevice->SetRenderTarget(0, g_apxBloomBuffers[g_iAccumBuffer]->GetSurface());
	Asura_Direct3D::D3DDevice->SetDepthStencilSurface(NULL);
	
	// If the accumulation buffer has just been allocated, it's a non-additive first pass
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	
	Asura_Direct3D::D3DDevice->SetVertexDeclaration(Asura_Platform_FSFX_SM30::g_pxBloomVertexDecl);
	Asura_Direct3D_Effects_System::BeginEffect(*Asura_Platform_FSFX_SM30::g_pxBloomEffect);
	Asura_Assert(Asura_Platform_FSFX_SM30::g_pxBloomEffect->IsD3DEffectValid(), "bloom effect invalid");
	
	if (Asura_Platform_FSFX_SM30::g_pxBloomEffect->IsD3DEffectValid())
	{
		float afUVOffsets[2] = {0.0f,0.0f};
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloatArray(Asura_Platform_FSFX_SM30::g_xBloomEffectUVOffsetHandle, afUVOffsets, 2);

		float afInputThreshold[4] = {m_fRBloomThreshold, m_fGBloomThreshold, m_fBBloomThreshold,0};
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloatArray(Asura_Platform_FSFX_SM30::g_xBloomEffectInputThresholdHandle,afInputThreshold, 4);		
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloat(Asura_Platform_FSFX_SM30::g_xBloomEffectInputScaleHandle, m_fInputScale);

		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetBool(Asura_Platform_FSFX_SM30::g_xBloomEffectAccumulateHandle, (m_bAccumulate && !bAccumBufferJustAllocated));
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloat(Asura_Platform_FSFX_SM30::g_xBloomEffectAccumFadeHandle, m_fAccumFade);

		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetTexture(Asura_Platform_FSFX_SM30::g_xBloomEffectAccumTextureHandle, g_apxBloomBuffers[1-g_iAccumBuffer]->GetTexture());
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloatArray(Asura_Platform_FSFX_SM30::g_xBloomEffectSecondaryUVOfsHandle, afSecondaryUVOffset, 2);
		
		// No blurring on this pass, but we want do subtract the input threshold and desaturate it
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetBool(Asura_Platform_FSFX_SM30::g_xBloomEffectBlurHandle, false);
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetBool(Asura_Platform_FSFX_SM30::g_xBloomEffectExtractBrightBitsHandle, true);
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloat(Asura_Platform_FSFX_SM30::g_xBloomEffectSaturationHandle, m_fSaturation);

		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetTexture(Asura_Platform_FSFX_SM30::g_xBloomEffectSourceTextureHandle, Asura_Direct3D_BackBufferCapture::GetCaptureTexture());

		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->BeginPass(0);
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->CommitChanges();
		Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, axVerts, sizeof(BloomVertex) );
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->EndPass();

		// Flush any effects we may still be using
		Asura_Direct3D_Effects_System::FlushEffects();
	}
	


	// Blur passes - From the accum buffer to the temp buffer, and back
	// ---------------------------------------------------------------------------------------------------------------
	
	// Update the UVs
	fHalfTexelU = 0.5f/fBufferWidth;
	fHalfTexelV = 0.5f/fBufferHeight;

	float fUMax =  (fViewWidth  * Asura_Platform_FSFX_SM30::g_fBloomEffectBlurBufferScale)  / fBufferWidth;
	float fVMax = (fViewHeight * Asura_Platform_FSFX_SM30::g_fBloomEffectBlurBufferScale)  / fBufferHeight;

	axVerts[0].fU = 0.0f+fHalfTexelU;	   axVerts[0].fV = 0.0f+fHalfTexelV;
	axVerts[1].fU = 0.0f+fHalfTexelU;	   axVerts[1].fV = fVMax+fHalfTexelV;
	axVerts[2].fU = fUMax+fHalfTexelU;	   axVerts[2].fV = 0.0f+fHalfTexelV;
	axVerts[3].fU = fUMax+fHalfTexelU;	   axVerts[3].fV = fVMax+fHalfTexelV;


	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);

	for (int iPass = 0; iPass<m_iBlurPasses; iPass++)
	{
		for (int iDir = 0; iDir<2; iDir++)
		{					

			// Set up the UVs and viewport
			if (iDir==0)
			{				
				// Accum buffer -> Temp buffer, Horiz blur
				float afUVOffsets[2] = {0.0025f*m_fBlurRadius,0.0f};
				Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloatArray(Asura_Platform_FSFX_SM30::g_xBloomEffectUVOffsetHandle, afUVOffsets, 2);				
				Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetTexture(Asura_Platform_FSFX_SM30::g_xBloomEffectSourceTextureHandle, g_apxBloomBuffers[g_iAccumBuffer]->GetTexture());										
				Asura_Direct3D::D3DDevice->SetRenderTarget(0, g_apxBloomBuffers[1-g_iAccumBuffer]->GetSurface());
			}
			else
			{
				// Temp buffer -> Accum buffer, Vert blur
				float afUVOffsets[2] = {0.0f,0.0025f*m_fBlurRadius};					
				Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloatArray(Asura_Platform_FSFX_SM30::g_xBloomEffectUVOffsetHandle, afUVOffsets, 2);
				Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetTexture(Asura_Platform_FSFX_SM30::g_xBloomEffectSourceTextureHandle, g_apxBloomBuffers[1-g_iAccumBuffer]->GetTexture());					
				Asura_Direct3D::D3DDevice->SetRenderTarget(0, g_apxBloomBuffers[g_iAccumBuffer]->GetSurface());
			}

			
			Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetTexture(Asura_Platform_FSFX_SM30::g_xBloomEffectAccumTextureHandle, NULL);


			Asura_Direct3D_Effects_System::BeginEffect(*Asura_Platform_FSFX_SM30::g_pxBloomEffect);
			Asura_Assert(Asura_Platform_FSFX_SM30::g_pxBloomEffect->IsD3DEffectValid(), "bloom effect invalid");

			if (Asura_Platform_FSFX_SM30::g_pxBloomEffect->IsD3DEffectValid())
			{									
				// Blurring enabled
				float afOutputColour[4] = {1.0f,1.0f,1.0f,1.0f};
				Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloatArray(Asura_Platform_FSFX_SM30::g_xBloomEffectOutputColourHandle, afOutputColour, 4);				
				Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetBool(Asura_Platform_FSFX_SM30::g_xBloomEffectBlurHandle, true);
				Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetBool(Asura_Platform_FSFX_SM30::g_xBloomEffectExtractBrightBitsHandle, false);				

				Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->BeginPass(0);
				Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->CommitChanges();
				Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, axVerts, sizeof(BloomVertex) );
				Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->EndPass();

				// Flush any effects we may still be using
				Asura_Direct3D_Effects_System::FlushEffects();
			}
		}
	}

	// Final pass - From the accum buffer to the back buffer
	// ---------------------------------------------------------------------------------------------------------------

	// Restore the render target
	xResult = Asura_Direct3D::D3DDevice->SetRenderTarget(0, pxOldRenderTargetBuffer);
	pxOldRenderTargetBuffer->Release();
	ASURA_D3D_OK( xResult);
	if( NULL != pxOldRenderTargetDepth)
	{
		xResult = Asura_Direct3D::D3DDevice->SetDepthStencilSurface( pxOldRenderTargetDepth);
		pxOldRenderTargetDepth->Release();
		ASURA_D3D_OK( xResult);
	}

	// This is always an additive pass, unless we're debugging

	if (m_bShowBlurBuffer)
	{
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	}
	else
	{	
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
	}

	Asura_Direct3D_Effects_System::BeginEffect(*Asura_Platform_FSFX_SM30::g_pxBloomEffect);
	Asura_Assert(Asura_Platform_FSFX_SM30::g_pxBloomEffect->IsD3DEffectValid(), "bloom effect invalid");

	if (Asura_Platform_FSFX_SM30::g_pxBloomEffect->IsD3DEffectValid())
	{
		float afUVOffsets[2] = {0.0f, 0.0f};
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloatArray(Asura_Platform_FSFX_SM30::g_xBloomEffectUVOffsetHandle, afUVOffsets, 2);

		float afOutputColour[4] = {m_fROutputCol * fIntensity, m_fGOutputCol * fIntensity, m_fBOutputCol * fIntensity,1};
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetFloatArray(Asura_Platform_FSFX_SM30::g_xBloomEffectOutputColourHandle, afOutputColour, 4);

		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetBool(Asura_Platform_FSFX_SM30::g_xBloomEffectBlurHandle, false);
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetBool(Asura_Platform_FSFX_SM30::g_xBloomEffectExtractBrightBitsHandle, false);		
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetTexture(Asura_Platform_FSFX_SM30::g_xBloomEffectSourceTextureHandle, g_apxBloomBuffers[g_iAccumBuffer]->GetTexture());

		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->BeginPass(0);
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->CommitChanges();
		Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, axVerts, sizeof(BloomVertex) );
		Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->EndPass();

		// Flush any effects we may still be using
		Asura_Direct3D_Effects_System::FlushEffects();
	}


	// This is very important. If we leave any render target bound in any shader, it'll cause
	// device resets to fail - shader has a reference to the RT, so it won't be released
	Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetTexture(Asura_Platform_FSFX_SM30::g_xBloomEffectAccumTextureHandle, NULL);
	Asura_Platform_FSFX_SM30::g_pxBloomEffect->GetD3DEffect()->SetTexture(Asura_Platform_FSFX_SM30::g_xBloomEffectSourceTextureHandle, NULL);

	Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_SRGBTEXTURE, FALSE );
	Asura_Direct3D::D3DDevice->SetSamplerState( 1, D3DSAMP_SRGBTEXTURE, FALSE );

	Asura_Direct3D_BackBufferCapture::AdviseNextFrame();
	Asura_Render::ApplyCameraSettings();
}

#endif //ASURA_TARGET_SM3
#endif //defined( ASURA_USE_FSFX)
