// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_Platform_GUIMenuInput.cpp												//
//																						//
//	Description: this handles the platform specific menu control system					//
//																						//
//	Created: Steve Keen - 16/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "Core/Asura_Timers.h"
#include "Input/Asura_PlatformInput.h"
#include "Input/Asura_ControlMap.h"
#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"
#ifdef ASURA_USE_GUI_OSK
#include "GUIMenu/Asura_GUIMenu_OnScreenKeyboard.h"
#endif
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

#include "Rendering/Asura_Direct3D.h"

#define ASURA_GUIMENU_MAX_CONTROLLERS	1
// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_InitInputMappings( void)
{
#ifdef ASURA_TOOLS

#define REGISTER_CONTROL_ENTRY( _mapTo, _buttonID, _pressType, _desc ) \
	RegisterControlEntry( 0, _mapTo, _buttonID, _pressType, _desc)

#else

#define REGISTER_CONTROL_ENTRY( _mapTo, _buttonID, _pressType, _UNUSED ) \
	RegisterControlEntry( 0, _mapTo, _buttonID, _pressType)

#endif // ASURA_TOOLS

	if(s_pxMenuControlMaps == NULL)
	{
		s_pxMenuControlMaps = new Asura_ControlMap[ASURA_GUIMENU_MAX_CONTROLLERS];

		s_pxMenuControlMaps[0].CreateMappings( ASURA_GUIMENU_CONTROL_MAX);

#if 0
		// -- This is a fix for the dodgy mapping on my pad --
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP, ASURA_PC_KEYID_PAD1_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PC_KEYID_PAD1_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PC_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PC_KEYID_PAD1_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);

		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PC_KEYID_PAD1_UP, Asura_ControlMapping::ON_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PC_KEYID_PAD1_LEFT, Asura_ControlMapping::ON_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PC_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PC_KEYID_PAD1_DOWN, Asura_ControlMapping::ON_PRESS, NULL);
#endif

		// -- Keyboard controls --
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP, ASURA_KEYID_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN, ASURA_KEYID_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT, ASURA_KEYID_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_KEYID_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);

		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT,		ASURA_KEYID_ENTER,			Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Accept - X");
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, 	ASURA_KEYID_DEL,			Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - Sq");				// SQUARE
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, 	ASURA_KEYID_HOME,			Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - O / Tri(psp)");		// TRIANGLE
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, 	ASURA_KEYID_NUMPADDEL,		Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - R3 / NA(psp)");		// R3
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4, 	ASURA_KEYID_TAB,			Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - Select");			// SELECT
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, 	ASURA_KEYID_NUMPADENTER,	Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - Start");			// START
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6, 	ASURA_KEYID_NUMPAD4, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - L1");				// L1
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7, 	ASURA_KEYID_NUMPAD6, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - R1");				// R1
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8, 	ASURA_KEYID_NUMPAD7, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - L2 / NA(psp)");		// L2
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, 	ASURA_KEYID_NUMPAD9, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - R2 / NA(psp)");		// R2

		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_KEYID_UP, Asura_ControlMapping::ON_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_KEYID_DOWN, Asura_ControlMapping::ON_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_KEYID_LEFT, Asura_ControlMapping::ON_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_KEYID_RIGHT, Asura_ControlMapping::ON_PRESS, NULL);

		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT_HELD,		ASURA_KEYID_ENTER,			Asura_ControlMapping::ON_PRESS, "Held Accept - X");
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_KEYID_DEL,			Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - Sq");				// SQUARE
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_KEYID_HOME,			Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - O / Tri(psp)");	// TRIANGLE
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD, ASURA_KEYID_NUMPADDEL,		Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - R3 / NA(psp)");	// R3
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD, ASURA_KEYID_TAB,			Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - Select");			// SELECT
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, ASURA_KEYID_NUMPADENTER,	Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - Start");			// START
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD, ASURA_KEYID_NUMPAD4, 		Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - L1");				// L1
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD, ASURA_KEYID_NUMPAD6, 		Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - R1");				// R1
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD, ASURA_KEYID_NUMPAD7, 		Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - L2 / NA(psp)");	// L2
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD, ASURA_KEYID_NUMPAD9, 		Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - R2 / NA(psp)");	// R2

		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_KEYID_BACKSPACE, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_KEYID_BACKSPACE, Asura_ControlMapping::ON_PRESS, NULL);

		// Use the Escape key for canceling menus. Falls in line with how other games work
		// and Eidos PC Game Standards.
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_KEYID_ESCAPE, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_KEYID_ESCAPE, Asura_ControlMapping::ON_PRESS, NULL);

		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CLICK, ASURA_KEYID_MOUSEBUTTON1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CLICK_HELD, ASURA_KEYID_MOUSEBUTTON1, Asura_ControlMapping::ON_PRESS, NULL);

		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP, ASURA_KEYID_MOUSEWHEELUP, Asura_ControlMapping::ON_PRESS, NULL);
		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN, ASURA_KEYID_MOUSEWHEELDOWN, Asura_ControlMapping::ON_PRESS, NULL);

		if(Asura_PC_Input_Joystick::GetNumJoysticks() > 0)
		{
			const Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE	eJoystickType	= Asura_PC_Input_Joystick::GetJoystickType(0);

			switch(eJoystickType)
			{
				case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_XBOX360_GAMEPAD:
				{
					// -- XBox 360 Pad controls --
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP, ASURA_PC_KEYID_XBOX360PAD1_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PC_KEYID_XBOX360PAD1_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PC_KEYID_XBOX360PAD1_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PC_KEYID_XBOX360PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_PC_KEYID_XBOX360PAD1_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, ASURA_PC_KEYID_XBOX360PAD1_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_PC_KEYID_XBOX360PAD1_B, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PC_KEYID_XBOX360PAD1_UP, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PC_KEYID_XBOX360PAD1_DOWN, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PC_KEYID_XBOX360PAD1_LEFT, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PC_KEYID_XBOX360PAD1_RIGHT, Asura_ControlMapping::ON_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_PC_KEYID_XBOX360PAD1_A, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, ASURA_PC_KEYID_XBOX360PAD1_START, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_PC_KEYID_XBOX360PAD1_B, Asura_ControlMapping::ON_PRESS, NULL);

#ifdef ASURA_USE_CONTROLMAP_MANAGER
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PC_KEYID_XBOX360PAD1_BACK, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL) ;
#endif 

					// Added Axis movement to allow navigation and value changes using the analogue sticks.
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_UP, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f, -1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, 0.5f, -1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f, 1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, -0.5f, 1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f, 1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, -0.5f, 1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f, -1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, 0.5f, -1);
					break;
				}

				#ifdef ASURA_PC_SUPPORT_XINPUT
				case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_XBOX360_XINPUT_GAMEPAD:
				{
					// -- XBox 360 Pad controls --
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP, ASURA_PC_KEYID_XINPUT360PAD1_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PC_KEYID_XINPUT360PAD1_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PC_KEYID_XINPUT360PAD1_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PC_KEYID_XINPUT360PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_PC_KEYID_XINPUT360PAD1_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, ASURA_PC_KEYID_XINPUT360PAD1_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_PC_KEYID_XINPUT360PAD1_B, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PC_KEYID_XINPUT360PAD1_UP, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PC_KEYID_XINPUT360PAD1_DOWN, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PC_KEYID_XINPUT360PAD1_LEFT, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PC_KEYID_XINPUT360PAD1_RIGHT, Asura_ControlMapping::ON_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_PC_KEYID_XINPUT360PAD1_A, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, ASURA_PC_KEYID_XINPUT360PAD1_START, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_PC_KEYID_XINPUT360PAD1_B, Asura_ControlMapping::ON_PRESS, NULL);

#ifdef ASURA_USE_CONTROLMAP_MANAGER
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PC_KEYID_XINPUT360PAD1_BACK, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL) ;
#endif 
					// Added Axis movement to allow navigation and value changes using the analogue sticks.
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_UP, ASURA_AXIS_PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f, -1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_AXIS_PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, 0.5f, -1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_DOWN, ASURA_AXIS_PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f, 1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_AXIS_PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, -0.5f, 1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_LEFT, ASURA_AXIS_PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f, 1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_AXIS_PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, -0.5f, 1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_AXIS_PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f, -1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_AXIS_PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, 0.5f, -1);
				}
				#endif	// ASURA_PC_SUPPORT_XINPUT

				case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_PS3_GAMEPAD:
				{
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP, ASURA_PC_KEYID_PS3PAD1_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PC_KEYID_PS3PAD1_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PC_KEYID_PS3PAD1_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PC_KEYID_PS3PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT,		ASURA_PC_KEYID_PS3PAD1_CROSS,	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Accept - X");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, 	ASURA_PC_KEYID_PS3PAD1_SQUARE, 	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - Sq");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, 	ASURA_PC_KEYID_PS3PAD1_CIRCLE, 	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - O / Tri(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, 	ASURA_PC_KEYID_PS3PAD1_R3,		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - R3 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4, 	ASURA_PC_KEYID_PS3PAD1_SELECT,	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - Select");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, 	ASURA_PC_KEYID_PS3PAD1_START,	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - Start");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6, 	ASURA_PC_KEYID_PS3PAD1_L1, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - L1");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7, 	ASURA_PC_KEYID_PS3PAD1_R1, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - R1");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8, 	ASURA_PC_KEYID_PS3PAD1_L2, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - L2 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, 	ASURA_PC_KEYID_PS3PAD1_R2, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - R2 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL,		ASURA_PC_KEYID_PS3PAD1_TRIANGLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PC_KEYID_PS3PAD1_UP, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PC_KEYID_PS3PAD1_DOWN, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PC_KEYID_PS3PAD1_LEFT, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PC_KEYID_PS3PAD1_RIGHT, Asura_ControlMapping::ON_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT_HELD,		ASURA_PC_KEYID_PS3PAD1_CROSS,	Asura_ControlMapping::ON_PRESS, "Held Accept - X");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_PC_KEYID_PS3PAD1_SQUARE, 	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - Sq");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_PC_KEYID_PS3PAD1_CIRCLE, 	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - O / Tri(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD, ASURA_PC_KEYID_PS3PAD1_R3,		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - R3 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD, ASURA_PC_KEYID_PS3PAD1_SELECT,	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - Select");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, ASURA_PC_KEYID_PS3PAD1_START,	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - Start");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD, ASURA_PC_KEYID_PS3PAD1_L1, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - L1");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD, ASURA_PC_KEYID_PS3PAD1_R1, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - R1");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD, ASURA_PC_KEYID_PS3PAD1_L2, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - L2 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD, ASURA_PC_KEYID_PS3PAD1_R2, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - R2 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_PC_KEYID_PS3PAD1_TRIANGLE, Asura_ControlMapping::ON_PRESS, NULL);

#ifdef ASURA_USE_CONTROLMAP_MANAGER
					//REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PC_KEYID_PS3PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL) ;
					//pxControlMap->AddHoldKeyMapping(Simp_Player::MAPPING_START_DVS, uPadOffset + ASURA_PS3_KEYID_PAD1_SELECT, 1.0f );
					s_pxMenuControlMaps[0].AddHoldKeyMapping(ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PC_KEYID_PS3PAD1_SELECT, 1.0f );
#endif 
					// Added Axis movement to allow navigation and value changes using the analogue sticks.
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_UP, ASURA_PC_AXISID_PS3PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f, -1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PC_AXISID_PS3PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, 0.5f, -1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PC_AXISID_PS3PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f, 1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PC_AXISID_PS3PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, -0.5f, 1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PC_AXISID_PS3PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f, 1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PC_AXISID_PS3PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, -0.5f, 1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PC_AXISID_PS3PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f, -1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PC_AXISID_PS3PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, 0.5f, -1);
					break;
				}

				case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_PS2_GAMEPAD:
				default:
				{
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP, ASURA_PC_KEYID_PS2PAD1_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PC_KEYID_PS2PAD1_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PC_KEYID_PS2PAD1_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PC_KEYID_PS2PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT,		ASURA_PC_KEYID_PS2PAD1_CROSS,	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Accept - X");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, 	ASURA_PC_KEYID_PS2PAD1_SQUARE, 	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - Sq");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, 	ASURA_PC_KEYID_PS2PAD1_CIRCLE, 	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - O / Tri(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, 	ASURA_PC_KEYID_PS2PAD1_R3,		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - R3 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4, 	ASURA_PC_KEYID_PS2PAD1_SELECT,	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - Select");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, 	ASURA_PC_KEYID_PS2PAD1_START,	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - Start");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6, 	ASURA_PC_KEYID_PS2PAD1_L1, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - L1");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7, 	ASURA_PC_KEYID_PS2PAD1_R1, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - R1");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8, 	ASURA_PC_KEYID_PS2PAD1_L2, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - L2 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, 	ASURA_PC_KEYID_PS2PAD1_R2, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - R2 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL,		ASURA_PC_KEYID_PS2PAD1_TRIANGLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PC_KEYID_PS2PAD1_UP, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PC_KEYID_PS2PAD1_DOWN, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PC_KEYID_PS2PAD1_LEFT, Asura_ControlMapping::ON_PRESS, NULL);
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PC_KEYID_PS2PAD1_RIGHT, Asura_ControlMapping::ON_PRESS, NULL);

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT_HELD,		ASURA_PC_KEYID_PS2PAD1_CROSS,	Asura_ControlMapping::ON_PRESS, "Held Accept - X");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_PC_KEYID_PS2PAD1_SQUARE, 	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - Sq");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_PC_KEYID_PS2PAD1_CIRCLE, 	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - O / Tri(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD, ASURA_PC_KEYID_PS2PAD1_R3,		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - R3 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD, ASURA_PC_KEYID_PS2PAD1_SELECT,	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - Select");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, ASURA_PC_KEYID_PS2PAD1_START,	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - Start");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD, ASURA_PC_KEYID_PS2PAD1_L1, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - L1");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD, ASURA_PC_KEYID_PS2PAD1_R1, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - R1");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD, ASURA_PC_KEYID_PS2PAD1_L2, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - L2 / NA(psp)");
					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD, ASURA_PC_KEYID_PS2PAD1_R2, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - R2 / NA(psp)");

					REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_PC_KEYID_PS2PAD1_TRIANGLE, Asura_ControlMapping::ON_PRESS, NULL);

#ifdef ASURA_USE_CONTROLMAP_MANAGER
					//REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PC_KEYID_PS2PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL) ;
					//pxControlMap->AddHoldKeyMapping(Simp_Player::MAPPING_START_DVS, uPadOffset + ASURA_PS2_KEYID_PAD1_SELECT, 1.0f );
					s_pxMenuControlMaps[0].AddHoldKeyMapping(ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PC_KEYID_PS2PAD1_SELECT, 1.0f );
#endif 
					// Added Axis movement to allow navigation and value changes using the analogue sticks.
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_UP, ASURA_PC_AXISID_PS2PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f, -1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PC_AXISID_PS2PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, 0.5f, -1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PC_AXISID_PS2PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f, 1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PC_AXISID_PS2PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, -0.5f, 1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PC_AXISID_PS2PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f, 1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PC_AXISID_PS2PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, -0.5f, 1);

					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PC_AXISID_PS2PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f, -1);
					s_pxMenuControlMaps[0].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PC_AXISID_PS2PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, 0.5f, -1);
					break;
				}
			}		// End of switch.
		}
	}
}

//void Asura_GUIMenu_System::Platform_InitInputMappings( void)
//{
//	s_xMenuControlMap.CreateMappings( ASURA_GUIMENU_CONTROL_MAX);
//
//#ifdef ASURA_TOOLS
//
//	#define REGISTER_CONTROL_ENTRY( _mapTo, _buttonID, _pressType, _desc ) \
//				RegisterControlEntry( _mapTo, _buttonID, _pressType, _desc)
//
//#else
//
//	#define REGISTER_CONTROL_ENTRY( _mapTo, _buttonID, _pressType, _UNUSED ) \
//		RegisterControlEntry( _mapTo, _buttonID, _pressType)
//
//#endif // ASURA_TOOLS
//
//	// detect PS2JOY pad being used
//	// avoid button clash with XBOX 360 pad by not initialising control map if pad not connected PDC 17/5/2007
//	// also add check for 360 pads
//	bool bPS2Joy = false;
//	bool bXBOX360Joy = false;
//	if (Asura_PC_Input_Joystick::GetNumJoysticks())
//	{
//		GUID xGUID;
//		if( Asura_PC_Input_Joystick::GetJoystickGUIDFromName("GIC USB Joystick", xGUID) ||
//			Asura_PC_Input_Joystick::GetJoystickGUIDFromName("GIC Psx Vibration feedback (USB) Joystick", xGUID) ||
//			Asura_PC_Input_Joystick::GetJoystickGUIDFromName("Dual PSX-USB Adaptor ", xGUID) ||
//			Asura_PC_Input_Joystick::GetJoystickGUIDFromName("PS Vibration Feedback Converter ", xGUID) ||
//			Asura_PC_Input_Joystick::GetJoystickGUIDFromName("USB Vibration Gamepad", xGUID) ||
//			Asura_PC_Input_Joystick::GetJoystickGUIDFromName("GIC USB Joystick", xGUID) ||
//			Asura_PC_Input_Joystick::GetJoystickGUIDFromName("GIC Psx Vibration feedback (USB) Joystick", xGUID) ||
//			Asura_PC_Input_Joystick::GetJoystickGUIDFromName("USB GAME PAD", xGUID) ||
//			Asura_PC_Input_Joystick::GetJoystickGUIDFromName("USB Joystick     ", xGUID) ||
//			Asura_PC_Input_Joystick::GetJoystickGUIDFromName("PSJOY (USB) Joystick", xGUID) ||
//			Asura_PC_Input_Joystick::GetJoystickGUIDFromName("SmartJoy PLUS Adapter", xGUID))
//		{
//			bPS2Joy = true;
//		}
//
//		if (Asura_PC_Input_Joystick::GetJoystickGUIDFromName("XBOX 360 For Windows (Controller)", xGUID))
//		{
//			bXBOX360Joy = true;
//		}
//		if (Asura_PC_Input_Joystick::GetJoystickGUIDFromName("Controller (XBOX 360 For Windows)", xGUID))
//		{
//			bXBOX360Joy = true;
//		}
//		if (Asura_PC_Input_Joystick::GetJoystickGUIDFromName("Controller (Xbox 360 Wireless Receiver for Windows)", xGUID))
//		{
//			bXBOX360Joy = true;
//		}
//
//	}
//
//	// -- This is a fix for the dodgy mapping on my pad --
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP, ASURA_PC_KEYID_PAD1_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PC_KEYID_PAD1_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PC_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PC_KEYID_PAD1_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PC_KEYID_PAD1_UP, Asura_ControlMapping::ON_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PC_KEYID_PAD1_LEFT, Asura_ControlMapping::ON_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PC_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PC_KEYID_PAD1_DOWN, Asura_ControlMapping::ON_PRESS, NULL);
//
//	// -- Keyboard controls --
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP, ASURA_KEYID_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN, ASURA_KEYID_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT, ASURA_KEYID_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_KEYID_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT,		ASURA_KEYID_ENTER,			Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Accept - X");
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, 	ASURA_KEYID_DEL,			Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - Sq");				// SQUARE
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, 	ASURA_KEYID_HOME,			Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - O / Tri(psp)");		// TRIANGLE
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, 	ASURA_KEYID_NUMPADDEL,		Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - R3 / NA(psp)");		// R3
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4, 	ASURA_KEYID_TAB,			Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - Select");			// SELECT
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, 	ASURA_KEYID_NUMPADENTER,	Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - Start");			// START
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6, 	ASURA_KEYID_NUMPAD4, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - L1");				// L1
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7, 	ASURA_KEYID_NUMPAD6, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - R1");				// R1
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8, 	ASURA_KEYID_NUMPAD7, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - L2 / NA(psp)");		// L2
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, 	ASURA_KEYID_NUMPAD9, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS,	"Alt Accept - R2 / NA(psp)");		// R2
//
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_KEYID_BACKSPACE, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_KEYID_UP, Asura_ControlMapping::ON_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_KEYID_DOWN, Asura_ControlMapping::ON_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_KEYID_LEFT, Asura_ControlMapping::ON_PRESS, NULL);
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_KEYID_RIGHT, Asura_ControlMapping::ON_PRESS, NULL);
//
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT_HELD,		ASURA_KEYID_ENTER,			Asura_ControlMapping::ON_PRESS, "Held Accept - X");
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_KEYID_DEL,			Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - Sq");				// SQUARE
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_KEYID_HOME,			Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - O / Tri(psp)");	// TRIANGLE
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD, ASURA_KEYID_NUMPADDEL,		Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - R3 / NA(psp)");	// R3
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD, ASURA_KEYID_TAB,			Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - Select");			// SELECT
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, ASURA_KEYID_NUMPADENTER,	Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - Start");			// START
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD, ASURA_KEYID_NUMPAD4, 		Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - L1");				// L1
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD, ASURA_KEYID_NUMPAD6, 		Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - R1");				// R1
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD, ASURA_KEYID_NUMPAD7, 		Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - L2 / NA(psp)");	// L2
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD, ASURA_KEYID_NUMPAD9, 		Asura_ControlMapping::ON_PRESS,	"Held Alt Accept - R2 / NA(psp)");	// R2
//
//	REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_KEYID_BACKSPACE, Asura_ControlMapping::ON_PRESS, NULL);
//
//
//	if ( bPS2Joy )
//	{
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT,		ASURA_PC_KEYID_PS2PAD1_CROSS,	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Accept - X");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, 	ASURA_PC_KEYID_PS2PAD1_SQUARE, 	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - Sq");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, 	ASURA_PC_KEYID_PS2PAD1_CIRCLE, 	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - O / Tri(psp)");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, 	ASURA_PC_KEYID_PS2PAD1_R3,		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - R3 / NA(psp)");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4, 	ASURA_PC_KEYID_PS2PAD1_SELECT,	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - Select");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, 	ASURA_PC_KEYID_PS2PAD1_START,	Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - Start");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6, 	ASURA_PC_KEYID_PS2PAD1_L1, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - L1");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7, 	ASURA_PC_KEYID_PS2PAD1_R1, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - R1");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8, 	ASURA_PC_KEYID_PS2PAD1_L2, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - L2 / NA(psp)");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, 	ASURA_PC_KEYID_PS2PAD1_R2, 		Asura_ControlMapping::ON_DEBOUNCED_PRESS, "Alt Accept - R2 / NA(psp)");
//
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL,		ASURA_PC_KEYID_PS2PAD1_TRIANGLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT_HELD,		ASURA_PC_KEYID_PS2PAD1_CROSS,	Asura_ControlMapping::ON_PRESS, "Held Accept - X");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_PC_KEYID_PS2PAD1_SQUARE, 	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - Sq");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_PC_KEYID_PS2PAD1_CIRCLE, 	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - O / Tri(psp)");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD, ASURA_PC_KEYID_PS2PAD1_R3,		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - R3 / NA(psp)");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD, ASURA_PC_KEYID_PS2PAD1_SELECT,	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - Select");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, ASURA_PC_KEYID_PS2PAD1_START,	Asura_ControlMapping::ON_PRESS, "Held Alt Accept - Start");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD, ASURA_PC_KEYID_PS2PAD1_L1, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - L1");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD, ASURA_PC_KEYID_PS2PAD1_R1, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - R1");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD, ASURA_PC_KEYID_PS2PAD1_L2, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - L2 / NA(psp)");
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD, ASURA_PC_KEYID_PS2PAD1_R2, 		Asura_ControlMapping::ON_PRESS, "Held Alt Accept - R2 / NA(psp)");
//	
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_PC_KEYID_PS2PAD1_TRIANGLE, Asura_ControlMapping::ON_PRESS, NULL);
//
//#ifdef ASURA_USE_CONTROLMAP_MANAGER
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PC_KEYID_PS2PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL) ;
//#endif 
//	}
//
//	if ( bXBOX360Joy )
//	{
//		// -- XBox 360 Pad controls --
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP, ASURA_PC_KEYID_XBOX360PAD1_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PC_KEYID_XBOX360PAD1_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PC_KEYID_XBOX360PAD1_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PC_KEYID_XBOX360PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_PC_KEYID_XBOX360PAD1_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_PC_KEYID_XBOX360PAD1_B, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL);
//
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PC_KEYID_XBOX360PAD1_UP, Asura_ControlMapping::ON_PRESS, NULL);
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PC_KEYID_XBOX360PAD1_DOWN, Asura_ControlMapping::ON_PRESS, NULL);
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PC_KEYID_XBOX360PAD1_LEFT, Asura_ControlMapping::ON_PRESS, NULL);
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PC_KEYID_XBOX360PAD1_RIGHT, Asura_ControlMapping::ON_PRESS, NULL);
//
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_PC_KEYID_XBOX360PAD1_A, Asura_ControlMapping::ON_PRESS, NULL);
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_PC_KEYID_XBOX360PAD1_B, Asura_ControlMapping::ON_PRESS, NULL);
//
//#ifdef ASURA_USE_CONTROLMAP_MANAGER
//		REGISTER_CONTROL_ENTRY( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PC_KEYID_XBOX360PAD1_BACK, Asura_ControlMapping::ON_DEBOUNCED_PRESS, NULL) ;
//#endif 
//	}
//}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_ShutdownInputMappings( void)
{
	u_int u;

	if( s_pxMenuControlMaps)
	{
		for( u=0; u<ASURA_GUIMENU_MAX_CONTROLLERS; u++)
		{
			s_pxMenuControlMaps[u].DeleteMappings();
		}

		delete []s_pxMenuControlMaps;
		s_pxMenuControlMaps = NULL;
	}
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_System::Platform_ShouldUpdateControls()
{
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_System::Platform_ShouldRenderMenuPages()
{
	return true;
}

// --------------------------------------------------------------------------------------- 
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
void Asura_GUIMenu_System::Platform_RenderSafetyBorder( void)
{
	if( s_bRenderSafetyBorder)
	{
		float fScreenWidth, fScreenHeight;
		float fSafetyWidth, fSafetyHeight;
		Asura_Vector_2 xTopLeft;
		Asura_Vector_2 xBottomRight;

		fScreenWidth = static_cast<float>(Asura_Viewport::GetWidth());
		fScreenHeight = static_cast<float>(Asura_Viewport::GetHeight());
		fSafetyWidth = fScreenWidth * 0.85f;
		fSafetyHeight = fScreenHeight * 0.85f;

		xTopLeft.x = (fScreenWidth-fSafetyWidth) * 0.5f;
		xTopLeft.y = (fScreenHeight-fSafetyHeight) * 0.5f;
		xBottomRight.x = xTopLeft.x + fSafetyWidth;
		xBottomRight.y = xTopLeft.y + fSafetyHeight;

		// left edge
		Asura_Render::Render2DLine( xTopLeft.x, xTopLeft.y, xTopLeft.x, xBottomRight.y, 0xFFFF0000);
		// right edge
		Asura_Render::Render2DLine( xBottomRight.x, xTopLeft.y, xBottomRight.x, xBottomRight.y, 0xFFFF0000);
		// top edge
		Asura_Render::Render2DLine( xTopLeft.x, xTopLeft.y, xBottomRight.x, xTopLeft.y, 0xFFFF0000);
		// bottom edge
		Asura_Render::Render2DLine( xTopLeft.x, xBottomRight.y, xBottomRight.x, xBottomRight.y, 0xFFFF0000);
	}
}
#endif // ASURA_DEBUG_VARIABLE_SYSTEM

// --------------------------------------------------------------------------------------- 

u_int Asura_GUIMenu_System::Platform_GetNumControlMaps( void)
{
	return ASURA_GUIMENU_MAX_CONTROLLERS;
}

// --------------------------------------------------------------------------------------- 

// (MH - 27.02.2007) text input functions
static bool g_bDoingTextInput = false;
static Asura_UnicodeChar* g_wszTextInputBuffer;
static u_int g_uTextInputBufferSize;
static u_int g_uTextInputCursorPos;
static u_int g_uTextInputFrameNumber;

void Asura_GUIMenu_System::Platform_BeginTextInput( const Asura_UnicodeChar* wszMessage, 
												    Asura_UnicodeChar* wszBuffer, 
													u_int uBufferMaxChars )
{
#ifdef ASURA_USE_GUI_OSK
	Asura_GUIMenu_OnScreenKeyboard::Initialise( wszBuffer, uBufferMaxChars);
#else
	// uses Mike Healeys original pc implementation
	Asura_Assert(!g_bDoingTextInput, "already doing text input");
	g_bDoingTextInput = true;
	g_wszTextInputBuffer = wszBuffer;
	g_uTextInputBufferSize = uBufferMaxChars + 1;
	g_uTextInputCursorPos = uBufferMaxChars + 1;// this indicates "clear on first input"
	g_uTextInputFrameNumber = Asura_Timers::GetFrameNumber();
#endif
}

void Asura_GUIMenu_System::Platform_UpdateTextInput()
{
#ifdef ASURA_USE_GUI_OSK
	Asura_GUIMenu_OnScreenKeyboard::Update();
#else	
	Asura_UserInput::UpdateInputBuffer( g_wszTextInputBuffer, g_uTextInputBufferSize, g_uTextInputCursorPos, NULL );
#endif
}

void Asura_GUIMenu_System::Platform_BeginRender()
{
	D3DPERFCALL( BeginEvent( D3DCOLOR_XRGB( 0xFF, 0xFF, 0xFF ), L"GUIMenu"  ) );

	Asura_Render::CurrentStates.RequestAlphaTest(true, 1);
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, true);
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_ALPHAREF, 1);
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);

}

void Asura_GUIMenu_System::Platform_RenderTextInput()
{
	// just leave the menu to render it? what about cursors?
}

void Asura_GUIMenu_System::Platform_EndRender()
{
	Asura_Render::CurrentStates.RequestAlphaTest(false);
	D3DPERFCALL( EndEvent() ); // GUIMenu
}

bool Asura_GUIMenu_System::Platform_IsTextInputFinished()
{
#ifdef ASURA_USE_GUI_OSK
	bool bRet = false;

	switch( Asura_GUIMenu_OnScreenKeyboard::GetExitType())
	{
		case ASURA_GUIMENU_OSK_DONE:
		case ASURA_GUIMENU_OSK_CANCEL:
			bRet = true;
			break;

		case ASURA_GUIMENU_OSK_NOEXIT:
		default:
			break;
	}

	return bRet;
#else
	// currently not needed for OSK
	if (g_uTextInputFrameNumber == Asura_Timers::GetFrameNumber())
	{
		return false;
	}
	return Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_ENTER );
#endif 
}

bool Asura_GUIMenu_System::Platform_IsTextInputCancelled()
{
#ifdef ASURA_USE_GUI_OSK
	return (Asura_GUIMenu_OnScreenKeyboard::GetExitType() == ASURA_GUIMENU_OSK_CANCEL);
#else
	return false;
#endif 
}

void Asura_GUIMenu_System::Platform_EndTextInput()
{
#ifdef ASURA_USE_GUI_OSK
	Asura_GUIMenu_OnScreenKeyboard::Shutdown();
#else
	Asura_Assert(g_bDoingTextInput, "not doing text input");
	g_bDoingTextInput = false;
	g_wszTextInputBuffer = NULL;
	g_uTextInputBufferSize = 0;
#endif
}

// -------------------------------------------------------------------------------------------------

bool Asura_GUIMenu_System::Platform_ProcessMenuBackControls( u_int uButtonsPressed, u_int uButtonsTrapped)
{
	bool bRet = false;

	// filters out any buttons presses not required, then we do an equality check
	uButtonsPressed &= (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD);

	if( uButtonsPressed == (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD) )
	{
		if( (uButtonsTrapped&ASURA_GUIMENU_ACTION_BUTTON_CANCEL) == 0)
		{
			bRet = true;
		}
	}

	return bRet;
}
// --------------------------------------------------------------------------------------- 

#endif // ASURA_USE_GUI_MENU_SYSTEM
