#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _included_AsuraMouse_h_
#define _included_AsuraMouse_h_

class Asura_Mouse
{	   
public:
	float VerticalVelocity;
	float HorizontalVelocity;
	float WheelStatus;
	float DeltaX;
	float DeltaY;

	bool Initialise(void);
	void Shutdown(void);
	void Update(float& deltaX, float& deltaY);
	void OncePerFrameUpdate(void);

	//Swaps left and right mouse buttons. 
	//Probably for tools use only. RAR 06/01/2005
	static void SetSwapMouseButtons(bool bSwap);
	static bool GetSwapMouseButtons();

	//Sets exlusive mode for mouse. If set to non-exclusive then the mouse cursor
	//can leave the window and be used with other windows programs. RAR 23/08/2006
	static void SetExclusive(bool bExclusive);
	static bool IsExclusive() {return s_bExclusive;}

	//Alternate to using SetExclusive to prevent the windows cursor from leaving the window
	//RAR 11/11/2008
	static void SetRestrictWindowsCursorToWindow(bool bSet);
	static bool GetRestrictWindowsCursorToWindow() {return s_bRestrictWindowsCursorToWindow;}

	//Causes the windows mouse cursor to be used , and positions to be taken
	//from that instead of the values coming in from direct input
	static void SetUseWindowsCursor(bool bUse);
	static bool GetUseWindowsCursor(){return s_bUseWindowsCursor;}

	//Causes the input from the mouse to be ignored during the next update. Added to leftover mouse clicks
	//when alt_tabbing back. RAR 20/04/2007
	static void IgnoreNextFramesInput() {s_bIgnoreNextFramesInput = true;}

	//For discarding button presses outside of the main window
	static bool ShouldAcceptButtonPresses();

private :
	static void SetExclusiveInternal();


	static bool s_bExclusive;
	static bool s_bUseWindowsCursor;
	static bool s_bExclusiveInternal;
	static bool s_bIgnoreNextFramesInput;
	static bool s_bRestrictWindowsCursorToWindow;


};


#endif//_included_AsuraMouse_h_
