//
// Asura_MouseThread.cpp
//
// Created by Kevin Lea - 05/10/2001 16:40:35
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Core/Asura_PlatformBasics.h"
#include <process.h>

#include "Input/Asura_MouseThread.h"

bool UseHardwareMouse=false;


CRITICAL_SECTION MouseUpdate;

bool Asura_MouseThread::ThreadRunning = false;
static bool TerminateMouseThread = false;

void Asura_MouseThread::Initialise(void)
{
	#ifndef NDEBUG
	UseHardwareMouse = false;
	#endif
	
	if (UseHardwareMouse)
	{
		InitializeCriticalSection(&MouseUpdate);
		_beginthread(Process,0,0);
	}

}
void Asura_MouseThread::Shutdown(void)
{
	//flag the mouse thread to stop
	TerminateMouseThread = true;
	//and wait for it to do so
	while(ThreadRunning) Sleep(0); 
	TerminateMouseThread = false;

}
#include "Input/Asura_PlatformInput.h"
#include "Interface/Asura_Cursor.h"
#include "Rendering/Asura_Direct3D.h"

void Asura_MouseThread::Process(void* dummy)
{
	ThreadRunning = true;

	while(!TerminateMouseThread)
	{
		if (UseHardwareMouse)
		{
			// read mouse
			EnterCriticalSection(&MouseUpdate);
			float deltaX,deltaY;
			Asura_PlatformInput::Mouse.Update(deltaX,deltaY);
			#ifdef ASURA_USE_CURSOR
			Asura_Cursor::Update(deltaX,deltaY);
			#endif
			Asura_Direct3D::UpdateCursor();
			LeaveCriticalSection(&MouseUpdate);
		}
		Sleep(20);
	};

	DeleteCriticalSection(&MouseUpdate);
	ThreadRunning = false;
	_endthread();
}

void Asura_MouseThread::Update(void)
{
	if (!UseHardwareMouse)
	{
		float deltaX,deltaY;
		Asura_PlatformInput::Mouse.Update(deltaX,deltaY);
		#ifdef ASURA_USE_CURSOR
		Asura_Cursor::Update(deltaX,deltaY);
		#endif
	}
}
