//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_Rumble_DirectInput.cpp
// JCT - 01.04.09
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Input/Asura_Platform_Rumble_DirectInput.h"

#if (defined ASURA_SUPPORT_CONTROLLER_RUMBLE) && (ASURA_SUPPORT_CONTROLLER_RUMBLE != 0)

#include "Input/Asura_PlatformInput.h"
#include "Input/Asura_Rumble_System.h"

/******************************************************************************/
/* Static Class Data                                                          */
/******************************************************************************/

/******************************************************************************/
/* Global Variables                                                           */
/******************************************************************************/

static IDirectInputEffect* g_pxDIEffect = NULL;

/******************************************************************************/
/* Class Functions                                                            */
/******************************************************************************/

void Asura_Rumble_DirectInput::Platform_Initialise()
{
	// initialise DirectInput force feedback

	if (!Asura_PlatformInput::s_xJoystick.m_pxDevice)
	{
		return;
	}

	DICONSTANTFORCE xConstantForce;
	xConstantForce.lMagnitude = DI_FFNOMINALMAX;

	DWORD auiAxis[1] = { DIJOFS_X };
	LONG aiDirection[1] = { 0 };

	DIEFFECT xEffectParams;

	xEffectParams.dwSize = sizeof(DIEFFECT);
	xEffectParams.dwFlags = DIEFF_CARTESIAN | DIEFF_OBJECTOFFSETS;
	xEffectParams.dwDuration = INFINITE;
	xEffectParams.dwSamplePeriod = 0;
	xEffectParams.dwGain = DI_FFNOMINALMAX;
	xEffectParams.dwTriggerButton = DIEB_NOTRIGGER;
	xEffectParams.dwTriggerRepeatInterval = INFINITE;
	xEffectParams.cAxes = 1;
	xEffectParams.rgdwAxes = auiAxis;
	xEffectParams.rglDirection = aiDirection;
	xEffectParams.lpEnvelope = NULL;
	xEffectParams.cbTypeSpecificParams = sizeof(DICONSTANTFORCE);
	xEffectParams.lpvTypeSpecificParams = &xConstantForce;
	xEffectParams.dwStartDelay = 0;

	Asura_PlatformInput::s_xJoystick.m_pxDevice->CreateEffect(GUID_ConstantForce, &xEffectParams, &g_pxDIEffect, NULL);
}

void Asura_Rumble_DirectInput::Platform_Shutdown()
{
	if (g_pxDIEffect)
	{
		g_pxDIEffect->Stop();
		g_pxDIEffect->Release();
		g_pxDIEffect = NULL;
	}
}

void Asura_Rumble_DirectInput::Platform_PlayCurrentRumbles()
{
	if (!g_pxDIEffect)
	{
		return;
	}

	if (Asura_Rumble_System::GetCurrentMotorMagnitude(0, 0) > 0.f)
	{
		DIEFFECT xEffectParams;

		xEffectParams.dwSize = sizeof(DIEFFECT);
		xEffectParams.dwGain = static_cast <DWORD> (Asura_Rumble_System::GetCurrentMotorMagnitude(0, 0) * DI_FFNOMINALMAX);

		g_pxDIEffect->SetParameters(&xEffectParams, DIEP_GAIN);

		DWORD uiStatus;

		g_pxDIEffect->GetEffectStatus(&uiStatus);

		if (!(uiStatus & DIEGES_PLAYING))
		{
			g_pxDIEffect->Start(1, 0);
		}
	}

	else
	{
		g_pxDIEffect->Stop();
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Rumble_System::Platform_StopAllRumbles()
//
// PC implementation. Stops all rumbling
//
// Parameters:		None
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Rumble_DirectInput::Platform_StopAllRumbles()
{
	if (g_pxDIEffect)
	{
		g_pxDIEffect->Stop();
	}
}

#endif // (defined ASURA_SUPPORT_CONTROLLER_RUMBLE) && (ASURA_SUPPORT_CONTROLLER_RUMBLE != 0)
