//////////////////////////////////////////////////////////////////////////////
//
// Asura_PC_ConsoleWindowIO.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     Deals with input and output from and to the DOS console window.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___DREDDDS_CONSOLEWINDOWIO_H___
#define ___DREDDDS_CONSOLEWINDOWIO_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Stdlib platform-specific.
#include <windows.h>

// Engine independent.
#include "Core/Asura_PlatformBasics.h"
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_CommandConsole_IO.h"

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PC_ConsoleWindowIO : public Asura_CommandConsole_IO
{

    public:

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////

        Asura_PC_ConsoleWindowIO( const char* szGameName );
        virtual ~Asura_PC_ConsoleWindowIO( void );
        
        // M E T H O D S /////////////////////////////////////////////////////

        virtual void Reset( void );

        virtual void Update( void );
        virtual void Render( void );

        virtual bool CanBeHidden( void ) const { return false; }
        virtual void SetDown( bool bDown, bool bInstantTransition ) { }

        virtual bool CanSupplyInput( void ) const { return true; }
        virtual void GetInput( void(*pfnInputCallback)(ASURA_KEYID) );

        virtual bool CanArchiveOutput( void ) const { return true; }
        virtual void AddOutput( const Asura_UnicodeChar* wszOutput );
        virtual void ClearOutput( void );

        virtual void Scroll( bool bDown, bool bToEnd ) { }

        virtual void SetTextScale( float fScale ) { }
        virtual void SetBorderX( float fBorderX ) { }

        virtual const Asura_UnicodeChar* GetLineSplitPoint(const Asura_UnicodeChar* wszLine);
        virtual int Printf(int x, int y, const char* szformat, ...);
        virtual int PrintArgs(int x, int y, const char* szformat, va_list& args);
        virtual int PrintStr(int x, int y, const char* szstring);

		// static functions to pass on windows QUIT messages
		static bool IsQuitRequested() { return s_bQuitRequested; }
		static void RequestQuit() { s_bQuitRequested = true; }

    private:
        
        // M E T H O D S /////////////////////////////////////////////////////

        void ClearInputBfrDisplay( void );
        void RefreshInputBfrDisplay( bool bCheckIfReqd, bool bNeedToClearFirst );

        void AddOutputInternal( const Asura_UnicodeChar* wszOutput, bool bAppendNewline, bool bUpdateCursorPos );

        // S T A T E /////////////////////////////////////////////////////////

        HANDLE            m_hStdIn;
        HANDLE            m_hStdOut;


        Asura_UnicodeChar m_wszCachedInputBfr[ASURA_COMMAND_CONSOLE_INPUT_BUFFER_SIZE];

        u_int             m_uCachedCursorPos;
        COORD             m_xOutputCursorPos;

        bool              m_bCtrlC;

		static bool		  s_bQuitRequested;

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___DREDDDS_CONSOLEWINDOWIO_H___
