//
// Asura_PC_DirectPlay_Client.cpp
//
// Created by Kevin Lea - 14/06/2001 09:53:44
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Network/Asura_PC_DirectPlay_Client.h"

// Engine independent.
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"

// Engine platform-specific.
#include "Core/Asura_dxutil.h"
#include "Network/Asura_PC_DirectPlay.h"
#ifdef ASURA_PC_SUPPORT_GCDKEY
#include "Network/Asura_PC_GCDKey.h"
#endif


#define ASURA_ENUMSESSION_TIMEOUT_IN_MS 2000

extern GUID Project_GUID;





// Kevin Lea - 14/06/2001 10:17:47 - Asura_PC_DirectPlay client stuff
Asura_PC_DirectPlay_Client*       Asura_PC_DirectPlay::s_pxClient;
DPNHANDLE                         Asura_PC_DirectPlay::AsyncOp_EnumSessions;
DPNHANDLE                         Asura_PC_DirectPlay::AsyncOp_Connecting;
CRITICAL_SECTION                  Asura_PC_DirectPlay::CriticalSection_EnumSessions;
Asura_PC_DirectPlay_SessionInfo** Asura_PC_DirectPlay::SessionsStorage;
int                               Asura_PC_DirectPlay::NumberOfActiveSessions;
bool                              Asura_PC_DirectPlay::EnumSessionsActive;

DEBUGVAR bool dbg_bPrintSendToServerTimeouts = false;

//#include "Rendering/Asura_Text_Output.h"

#include "Network/Asura_Network_Session.h"


void Asura_PC_DirectPlay::ShowSessions(void)
{
	// Kevin Lea - 14/06/2001 12:05:00 - since multiple threads may be accessing the session
	// list, we need to lock out other threads for the duration of this function
	EnterCriticalSection(&CriticalSection_EnumSessions);

	for (u_int n=0; n<Asura_Network::GetMaxNumberOfSessions(); n++)
	{
		Asura_Network_Session* pxSession = Asura_Network::GetSession(n);

		if (SessionsStorage[n])
		{
			if (timeGetTime() - SessionsStorage[n]->LastPollTime>ASURA_ENUMSESSION_TIMEOUT_IN_MS) 
			{
				SessionsStorage[n]->FreeMemory();
				SAFE_DELETE(SessionsStorage[n]);
				NumberOfActiveSessions--;
			}
			else
			{
                const DPN_APPLICATION_DESC* pxAppDesc = SessionsStorage[n]->AppDesc;

				pxSession->Set( Project_ModifyCurNumPlayers(pxAppDesc->dwCurrentPlayers, pxAppDesc->pvApplicationReservedData),
                                Project_ModifyMaxNumPlayers(pxAppDesc->dwMaxPlayers, pxAppDesc->pvApplicationReservedData),
                                pxAppDesc->pwszSessionName,
                                SessionsStorage[n]->RoundTripLatency,
                                pxAppDesc->pvApplicationReservedData,
                                pxAppDesc->dwApplicationReservedDataSize );
			}
		}
		else
		{
			pxSession->Reset();
		}
	}
	LeaveCriticalSection(&CriticalSection_EnumSessions);
}

bool Asura_PC_DirectPlay::StartClientAndFindSessions(const char* szIPAddress /* = 0 */)
{
	GUID xSPGuid = CLSID_DP8SP_TCPIP;

    if (!s_pxClient)
    {
        Asura_PC_DirectPlay_Client::CreateInstance( &s_pxClient );
    }
    if( !s_pxClient ) { return false; }

	HRESULT hr;

    hr = s_pxClient->Initialize( NULL, ClientMessageHandler, 0 );

	IDirectPlay8Address*   pDP8AddressHost  = NULL;
    IDirectPlay8Address*   pDP8AddressLocal = NULL;
   
    // Create the local device address object
    if( FAILED( hr = CoCreateInstance( CLSID_DirectPlay8Address, NULL, 
                                       CLSCTX_ALL, IID_IDirectPlay8Address,
                                       (LPVOID*) &pDP8AddressLocal ) ) )
    {
		Asura_Assert(false, "failed to create local address");
        goto LCleanup;
    }

    // (MH - 14.08.2003) hack to get network simulator working
	// same as code in StartServer, expect called on client objects.
	// Note that the client and server SPs must match.
	if (s_bUseNetworkSimulator && szIPAddress)
	{
		HRESULT                     hr              = S_OK;
		DPN_SERVICE_PROVIDER_INFO*  pdnSPInfo       = NULL;
		DPN_SERVICE_PROVIDER_INFO*  pdnSPInfoEnum   = NULL;
		DWORD                       dwItems         = 0;
		DWORD                       dwSize          = 0;   

		// Enumerate all the Service Providers available
		hr = s_pxClient->EnumServiceProviders(NULL, NULL, NULL, &dwSize, &dwItems, 0);

		if( FAILED(hr) && hr != DPNERR_BUFFERTOOSMALL)
		{
			Asura_Assert(false, "failed to count SPs");
		}
		else
		{
			pdnSPInfo = (DPN_SERVICE_PROVIDER_INFO*) new BYTE[dwSize];
    
			if( FAILED( hr = s_pxClient->EnumServiceProviders(NULL, NULL, pdnSPInfo, &dwSize, &dwItems, 0 ) ) )
			{
				Asura_Assert(false, "failed to list SPs");
			}
			else
			{
				pdnSPInfoEnum = pdnSPInfo;
				for ( u_int i = 0; i < dwItems; i++ )
				{
					//if (pdnSPInfoEnum->dwFlags & DPNSPINFO_NETWORKSIMULATORDEVICE)
					// (MH - 14.08.2003) hmm, that flag doesn't seem to be getting set...
					if (wcsstr(pdnSPInfoEnum->pwszName, L"Network Simulator") != 0)
					{
						xSPGuid = pdnSPInfoEnum->guid;
						break;
					}
					pdnSPInfoEnum++;
				}
			}

			SAFE_DELETE_ARRAY(pdnSPInfo);

			Asura_Assert(xSPGuid != CLSID_DP8SP_TCPIP, "network simulator not available");
		}
	}
    
	// Set IP service provider
    //if( FAILED( hr = pDP8AddressLocal->SetSP( &CLSID_DP8SP_TCPIP ) ) )
	if( FAILED( hr = pDP8AddressLocal->SetSP( &xSPGuid ) ) )
    {
		 Asura_Assert(false, "failed to set local address SP");
         goto LCleanup;
    }


    // Create the remote host address object
    if( FAILED( hr = CoCreateInstance( CLSID_DirectPlay8Address, NULL, 
                                       CLSCTX_ALL, IID_IDirectPlay8Address,
                                       (LPVOID*) &pDP8AddressHost ) ) )
    {
		Asura_Assert(false, "failed to create host address");
        goto LCleanup;
    }

    // Set IP service provider
    //if( FAILED( hr = pDP8AddressHost->SetSP( &CLSID_DP8SP_TCPIP ) ) )
	if( FAILED( hr = pDP8AddressHost->SetSP( &xSPGuid ) ) )
    {
		Asura_Assert(false, "failed to set host address SP");
        goto LCleanup;
    }

    DPN_APPLICATION_DESC   dpnAppDesc;
	ZeroMemory( &dpnAppDesc, sizeof( DPN_APPLICATION_DESC ) );
    dpnAppDesc.dwSize = sizeof( DPN_APPLICATION_DESC );
    dpnAppDesc.guidApplication = Project_GUID;

	if (szIPAddress)
	{
		hr = pDP8AddressHost->AddComponent(DPNA_KEY_HOSTNAME,			//pwszName
										  szIPAddress,                  //lpvData
										  strlen(szIPAddress) + 1,		//dwDataSize in bytes
										  DPNA_DATATYPE_STRING_ANSI );  //dwDataType

		if (FAILED(hr))
		{
			Asura_Assert(false, "failed to set host address IP");
			goto LCleanup;
		}

		DWORD dwPort = s_uPort;

		hr = pDP8AddressHost->AddComponent(DPNA_KEY_PORT,			//pwszName
										  &dwPort,					//lpvData
										  sizeof(dwPort),			//dwDataSize in bytes
										  DPNA_DATATYPE_DWORD);		//dwDataType

		if (FAILED(hr))
		{
			Asura_Assert(false, "failed to set host address port");
			goto LCleanup;
		}

		DPN_APPLICATION_DESC dpnAppDesc;
		ZeroMemory( &dpnAppDesc, sizeof(DPN_APPLICATION_DESC) );
		dpnAppDesc.dwSize           = sizeof( DPN_APPLICATION_DESC );
		dpnAppDesc.guidApplication  = Project_GUID;

//		// do we need to send a password?
//		if (s_uPasswordHash)
//		{
//			// password is sent as wide string, so encode a sting with the hashed value.
//			// this needs testing to ensure the DirectPlay password comparison doesn't
//			// mess about with the string.
//			u_short* pusPassword = reinterpret_cast<u_short*>(&s_uPasswordHash);
//			u_short ausPassword[3];
//			ausPassword[0] = pusPassword[0];
//			ausPassword[1] = pusPassword[1];
//			ausPassword[2] = 0;
//			dpnAppDesc.pwszPassword = ausPassword;
//			dpnAppDesc.dwFlags = DPNSESSION_REQUIREPASSWORD;
//		}
	
		Asura_PC_DirectPlayer_UserConnectData xUserConnectData;
		xUserConnectData.m_uVersionID = Project_GetVersionID();
		xUserConnectData.m_uPasswordHash = s_uPasswordHash;

		// connect asynchronously - callback should be called to set ConnectionOK and WaitingForConnection flags.
		hr = s_pxClient->Connect( &dpnAppDesc,       // the application desc
                                  pDP8AddressHost,      // address of the host of the session
                                  pDP8AddressLocal,    // address of the local device
                                  NULL, NULL,                          // DPN_SECURITY_DESC, DPN_SECURITY_CREDENTIALS
                                  &xUserConnectData, sizeof(xUserConnectData), // user data, user data size
                                  NULL, &AsyncOp_Connecting,            // async context, async handle,
                                  0 ); // flags

		if (FAILED(hr))
		{
			Asura_Assert(false, "client failed to connect");
			goto LCleanup;
		}
	}
	else
	{
		InitializeCriticalSection( &CriticalSection_EnumSessions );
		EnumSessionsActive = true;
		
		hr = s_pxClient->EnumHosts
			 (
				&dpnAppDesc,
				pDP8AddressHost, 
				pDP8AddressLocal, 
				NULL, 
				0,
				INFINITE,
				0,
				INFINITE,
				NULL,
				&AsyncOp_EnumSessions,
				0
			 );

		if( FAILED(hr) )
		{
			Asura_Assert(false, "failed to start hosts enum");
			if( hr != DPNERR_INVALIDDEVICEADDRESS && 
				hr != DPNERR_ADDRESSING ) // This will be returned if the ip address is is invalid. 
			{

			}
			goto LCleanup;
		}
		else
		{
			Asura_Network::SetChoosingSession(true);
		}
	}

LCleanup:
    SAFE_RELEASE( pDP8AddressHost);
    SAFE_RELEASE( pDP8AddressLocal );
  
    if( hr == DPNERR_PENDING )  hr = DPN_OK;

	return (hr == DPN_OK);
}


void Asura_PC_DirectPlay::ConnectToSession(u_int uSessionIndex)
{
	StopSessionsEnum();

	if (!SessionsStorage[uSessionIndex])
	{
		Asura_Network::SetWaitingForConnection(false);
		Asura_Network::SetConnectionOkay(false);
		#ifdef ASURA_PC_SUPPORT_GCDKEY
		Asura_PC_GCDKey::SetWaitingForAuthorisation(false);
		#endif

		return;
	}
	//Asura_Assert(false,"ahah");

	DPN_PLAYER_INFO dpPlayerInfo;
    ZeroMemory( &dpPlayerInfo, sizeof(DPN_PLAYER_INFO) );
    dpPlayerInfo.dwSize = sizeof(DPN_PLAYER_INFO);
    dpPlayerInfo.dwInfoFlags = DPNINFO_NAME;
//    dpPlayerInfo.pwszName = name.GetString();//L"Son of Deadmeat";
	dpPlayerInfo.pwszName = L"Son of Deadmeat";
	//swprintf(dpPlayerInfo.pwszName,L"%s",name.GetString());

	HRESULT hr;

    // Set the peer info, and use the DPNOP_SYNC since by default this
    // is an async call.  If it is not DPNOP_SYNC, then the peer info may not
    // be set by the time we call Connect() below.
    hr = s_pxClient->SetClientInfo( &dpPlayerInfo, NULL, NULL, DPNOP_SYNC );

	Asura_PC_DirectPlayer_UserConnectData xUserConnectData;
	xUserConnectData.m_uVersionID = Project_GetVersionID();
	xUserConnectData.m_uPasswordHash = s_uPasswordHash;

	// Connect to an existing session. DPNCONNECT_OKTOQUERYFORADDRESSING allows
    // DirectPlay to prompt the user using a dialog box for any device address
    // or host address information that is missing
    // We also pass in copies of the app desc and host addr, since pDPHostEnumSelected
    // might be deleted from another thread that calls SessionsDlgExpireOldHostEnums().
    // This process could also be done using reference counting instead.
    // (MP - 03.09.2003) I don't think we want to pass the
    // DPNCONNECT_OKTOQUERYFORADDRESSING flag somehow...

//	if (SessionsStorage[uSessionIndex]->AppDesc->dwFlags & DPNSESSION_REQUIREPASSWORD)
//	{
//		// password is sent as wide string, so encode a sting with the hashed value.
//		// this needs testing to ensure the DirectPlay password comparison doesn't
//		// mess about with the string.
//		u_short* pusPassword = reinterpret_cast<u_short*>(&s_uPasswordHash);
//		u_short ausPassword[3];
//		ausPassword[0] = pusPassword[0];
//		ausPassword[1] = pusPassword[1];
//		ausPassword[2] = 0;
//		SessionsStorage[uSessionIndex]->AppDesc->pwszPassword = ausPassword;
//	}

 	hr = s_pxClient->Connect( SessionsStorage[uSessionIndex]->AppDesc,       // the application desc
                              SessionsStorage[uSessionIndex]->HostAddr,      // address of the host of the session
                              SessionsStorage[uSessionIndex]->DeviceAddr,    // address of the local device the enum responses were received on
                              NULL, NULL,                          // DPN_SECURITY_DESC, DPN_SECURITY_CREDENTIALS
                              &xUserConnectData, sizeof(xUserConnectData), // user data, user data size
                              NULL, &AsyncOp_Connecting,            // async context, async handle,
                              0 ); // flags

	

//	Asura_Assert(false,"wait for it!");

	u_int uTime = timeGetTime();

	while (uTime+500 > timeGetTime())
	{

	}
}

const Asura_PC_DirectPlay_SessionInfo* Asura_PC_DirectPlay::GetSessionInfo(u_int uSessionIndex)
{
    return uSessionIndex < Asura_Network::GetMaxNumberOfSessions() ? SessionsStorage[uSessionIndex] : NULL;
}



void Asura_PC_DirectPlay::StopSessionsEnum(void)
{
	EnterCriticalSection(&CriticalSection_EnumSessions);	EnumSessionsActive = false;
	
	LeaveCriticalSection(&CriticalSection_EnumSessions);

	s_pxClient->CancelAsyncOperation(AsyncOp_EnumSessions,0);

}

void Asura_PC_DirectPlay::DeleteSessionsInfo(void)
{
	for (u_int n=0; n<Asura_Network::GetMaxNumberOfSessions(); n++)
	{
		if (SessionsStorage[n])
		{
			SessionsStorage[n]->FreeMemory();
			SAFE_DELETE(SessionsStorage[n]);
			NumberOfActiveSessions--;
		}
	}
}


void Asura_PC_DirectPlay_SessionInfo::CopyInfo(PDPNMSG_ENUM_HOSTS_RESPONSE msg)
{
	const DPN_APPLICATION_DESC* appDesc = msg->pApplicationDescription;
	
	HRESULT hr;
	
	hr = msg->pAddressSender->Duplicate(&this->HostAddr);

	hr = msg->pAddressDevice->Duplicate(&this->DeviceAddr);

	// Deep copy the DPN_APPLICATION_DESC from
	this->AppDesc = new DPN_APPLICATION_DESC;
	ZeroMemory( this->AppDesc, sizeof(DPN_APPLICATION_DESC) );

	memcpy( this->AppDesc, appDesc, sizeof(DPN_APPLICATION_DESC) );

	if( appDesc->pwszSessionName )
	{
		this->AppDesc->pwszSessionName = new WCHAR[ wcslen(appDesc->pwszSessionName)+1 ];
		wcscpy( this->AppDesc->pwszSessionName,	appDesc->pwszSessionName );
	}

	if( appDesc->dwApplicationReservedDataSize)
	{
		this->AppDesc->pvApplicationReservedData = new char[appDesc->dwApplicationReservedDataSize];
		memcpy (this->AppDesc->pvApplicationReservedData, appDesc->pvApplicationReservedData, appDesc->dwApplicationReservedDataSize);
	}
	else
	{
		this->AppDesc->pvApplicationReservedData = 0;
	}

	// Update the time this was done, so that we can expire this host
	// if it doesn't refresh w/in a certain amount of time
	this->LastPollTime = timeGetTime();

	this->RoundTripLatency = msg->dwRoundTripLatencyMS;
}

void Asura_PC_DirectPlay_SessionInfo::UpdateInfo(PDPNMSG_ENUM_HOSTS_RESPONSE msg)
{
	FreeMemory();

	CopyInfo(msg);
}

void Asura_PC_DirectPlay_SessionInfo::FreeMemory(void)
{
	if(this->AppDesc)
    {
        SAFE_DELETE_ARRAY( this->AppDesc->pwszSessionName );
		SAFE_DELETE_ARRAY(this->AppDesc->pvApplicationReservedData);
		SAFE_DELETE( this->AppDesc );
    }
    SAFE_RELEASE( this->HostAddr );
    SAFE_RELEASE( this->DeviceAddr );

}

void Asura_PC_DirectPlay::AddSessionInfo(PDPNMSG_ENUM_HOSTS_RESPONSE msg)
{
	// Kevin Lea - 14/06/2001 12:05:00 - since multiple threads may be accessing the session
	// list, we need to lock out other threads for the duration of this function
	EnterCriticalSection(&CriticalSection_EnumSessions);

	const DPN_APPLICATION_DESC* appDesc = msg->pApplicationDescription;

	// Kevin Lea - 14/06/2001 14:57:13 - if this message refers to a session we already know
	// about, just update the info.
	for (u_int n=0; n<Asura_Network::GetMaxNumberOfSessions(); n++)
	{
		if (SessionsStorage[n])
		{
			if (SessionsStorage[n]->AppDesc->guidInstance == appDesc->guidInstance)
			{
				SessionsStorage[n]->UpdateInfo(msg);
				goto AllDone;
			}
		}
	}

	// Kevin Lea - 14/06/2001 14:58:20 - new session; add it to the list if there's room
	for (u_int n=0; n<Asura_Network::GetMaxNumberOfSessions(); n++)
	{
		if (!SessionsStorage[n])
		{
			SessionsStorage[n] = new Asura_PC_DirectPlay_SessionInfo;
			SessionsStorage[n]->CopyInfo(msg);
			NumberOfActiveSessions++;
			goto AllDone;
		}
	}

AllDone:
	
	LeaveCriticalSection(&CriticalSection_EnumSessions);
}

HRESULT WINAPI Asura_PC_DirectPlay::ClientMessageHandler( PVOID pvUserContext, DWORD dwMessageId, PVOID pMsgBuffer )
{
    // Try not to stay in this message handler for too long, otherwise
    // there will be a backlog of data.  The best solution is to 
    // queue data as it comes in, and then handle it on other threads.
    
    // This function is called by the DirectPlay message handler pool of 
    // threads, so be careful of thread synchronization problems with shared memory

    switch( dwMessageId )
    {
	    case DPN_MSGID_ENUM_HOSTS_RESPONSE: 
	    {
			// Kevin Lea - 13/06/2001 16:10:36 - We've found a session
			if (EnumSessionsActive)
			{
				PDPNMSG_ENUM_HOSTS_RESPONSE msg = (PDPNMSG_ENUM_HOSTS_RESPONSE)pMsgBuffer;
			
				AddSessionInfo(msg);
			}
			break;
		}
	
		case DPN_MSGID_CONNECT_COMPLETE:
		{
			// Kevin Lea - 15/06/2001 12:20:52 - the server's reply to our
			// connection request

			//Asura_Assert(false,"Got connection message");
			PDPNMSG_CONNECT_COMPLETE pxMsg = (PDPNMSG_CONNECT_COMPLETE)pMsgBuffer;

			Asura_Network::SetWaitingForConnection(false);

			switch (pxMsg->hResultCode)
			{
				case S_OK:
				{
					Asura_Network::SetConnectionOkay(true);
					break;
				}
				case DPNERR_HOSTREJECTEDCONNECTION: 
				{
					if (pxMsg->dwApplicationReplyDataSize == sizeof(Asura_PC_DirectPlay_UserConnectReplyData))
					{
						const Asura_PC_DirectPlay_UserConnectReplyData* pxResultCode = 
							reinterpret_cast<const Asura_PC_DirectPlay_UserConnectReplyData*>(pxMsg->pvApplicationReplyData);
						Asura_Network::SetReasonForDisconnection(static_cast<enum ASURA_NETWORK_DISCONNECT_REASON>(pxResultCode->m_uResultCode));
					}
					else
					{
						Asura_Network::SetReasonForDisconnection(ASURA_NDR_VERSION);
					}
					break;
				}
// (MH - 09.10.2003) password is done via user data now
//				case DPNERR_INVALIDPASSWORD:
//				{
//					Asura_Network::SetReasonForDisconnection(ASURA_NDR_PASSWORD);
//					break;
//				}
				case DPNERR_SESSIONFULL:
				{
					Asura_Network::SetReasonForDisconnection(ASURA_NDR_SERVERFULL);
					break;
				}

				default:
				{
					// unknown error
					Asura_Network::SetReasonForDisconnection(ASURA_NDR_UNKNOWN);
					break;
				}
			}
			
			break;
		}
		case DPN_MSGID_TERMINATE_SESSION:
        {
            PDPNMSG_TERMINATE_SESSION pTerminateSessionMsg;
            pTerminateSessionMsg = (PDPNMSG_TERMINATE_SESSION)pMsgBuffer;

			// (MH - 14.08.2003) maybe this is better than below
			Asura_OutputToDebugger(("terminate session message received, code 0x%x (%s).\n", 
				pTerminateSessionMsg->hResultCode, pTerminateSessionMsg->hResultCode == DPNERR_CONNECTIONLOST ? "connection lost" : "see DX Error Lookup"));
			Asura_Network::SetConnectionOkay(false);
            break;
        }

		case DPN_MSGID_SEND_COMPLETE:
		{
#if defined(_DEBUG) || defined(ASURA_ASSERT)
			// SNH - 30/07/2003 11:06:18 - DirectPlay sends one of these for every time Send is called.
			//		The timed-out hResultCode seems to be a good indication of a dropped connection.
			PDPNMSG_SEND_COMPLETE msg = (PDPNMSG_SEND_COMPLETE)pMsgBuffer;

			if (dbg_bPrintSendToServerTimeouts)
			{
				if(FAILED(msg->hResultCode))
				{
					const char* szError = "unknown";
					switch (msg->hResultCode)
					{
					case DPNERR_CONNECTIONLOST: szError = "connection lost"; break;
					case DPNERR_INVALIDFLAGS: szError = "invalid flags"; break;
					case DPNERR_INVALIDPARAM: szError = "invalid param"; break;
					case DPNERR_INVALIDPLAYER: szError = "invalid player"; break;
					case DPNERR_TIMEDOUT: szError = "timed out"; break;
					}

					Asura_OutputToDebugger(("send message to server failed. reason: %s.\n", szError));
				}
			}

			// (MH - 01.09.2003) we set user context to be non-zero iff msg was guaranteed
			if (msg->pvUserContext)
			{
				Asura_Assert(!(DPNERR_TIMEDOUT == msg->hResultCode), "guaranteed message timed out, connection lost?");
			}
#endif
			break;
		}

        case DPN_MSGID_RECEIVE:
        {
		//	if (Asura_Network::IsReadyForMessages())
			{
				Asura_Assert(s_pxClient,"Client not active");

				PDPNMSG_RECEIVE pReceiveMsg;
				pReceiveMsg = (PDPNMSG_RECEIVE)pMsgBuffer;

				BYTE* data = pReceiveMsg->pReceiveData;
				u_int uNumBytes = pReceiveMsg->dwReceiveDataSize;
				bool bWasGuaranteed = (pReceiveMsg->dwReceiveFlags & DPNRECEIVE_GUARANTEED) != 0;
				
				UnpackMessageFromBuffer(data, uNumBytes, bWasGuaranteed);
			}
			    
			return DPN_OK;
        }
	}
	return S_OK;
}

//
// Asura_PC_DirectPlay_Client.
//

void Asura_PC_DirectPlay_Client::CreateInstance( Asura_PC_DirectPlay_Client** ppxInstance )
{
    Asura_Assert( !*ppxInstance, "" );

    HRESULT iRes;
    Asura_PC_DirectPlay_Client* pxNew;

    iRes = CoInitialize( NULL );
    if( SUCCEEDED(iRes) )
    {
        pxNew = new Asura_PC_DirectPlay_Client();
        iRes = ( pxNew ? S_OK : -1 );

        if( SUCCEEDED(iRes) )
        {
            iRes = CoCreateInstance( CLSID_DirectPlay8Client, NULL, CLSCTX_INPROC_SERVER, IID_IDirectPlay8Client, reinterpret_cast<void**>(&pxNew->m_pxDPClient) );
            if( FAILED(iRes) ) { delete pxNew; }
        }

        if( FAILED(iRes) ) { CoUninitialize(); }
    }

    if( SUCCEEDED(iRes) ) { *ppxInstance = pxNew; }
}

void Asura_PC_DirectPlay_Client::DestroyInstance( Asura_PC_DirectPlay_Client** ppxInstance )
{
    if( !*ppxInstance ) { return; }

    IDirectPlay8Client* const pxDPClient = (*ppxInstance)->m_pxDPClient;
    pxDPClient->CancelAsyncOperation( 0, DPNCANCEL_ALL_OPERATIONS );
    pxDPClient->Close( 0 );
    pxDPClient->Release();

    delete *ppxInstance;
    *ppxInstance = NULL;

    CoUninitialize();
}

HRESULT Asura_PC_DirectPlay_Client::Send( const DPN_BUFFER_DESC *const prgBufferDesc,
                                          const DWORD cBufferDesc,
                                          const DWORD dwTimeOut,
                                          void *const pvAsyncContext,
                                          DPNHANDLE *const phAsyncHandle,
                                          const DWORD dwFlags )
{
#ifdef ASURA_DIRECTPLAY_MESSAGELOGGING
    if( Asura_PC_DirectPlay::MSG_LOG_LEVEL_NONE != Asura_PC_DirectPlay::GetMsgLogLevel() )
    {
        Asura_PC_DirectPlay::LogMsgInfo( prgBufferDesc );
    }
#endif

    return m_pxDPClient->Send( prgBufferDesc, cBufferDesc, dwTimeOut, pvAsyncContext, phAsyncHandle, dwFlags );
}
