// Asura_Platform_NetConnection.cpp
// Thomas Young - 21.01.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#include "Network/Asura_NetConnection.h"
#include "Network/Asura_NetSocket.h"
#ifdef ASURA_PC_SUPPORT_STEAMWORKS
#include "Network/Steamworks/Asura_PC_SteamworksServer.h"
#include "Network/Steamworks/Asura_PC_SteamworksClient.h"
#endif

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

// TY - All of this code is no longer required unless we want to authorise clients over
// Asura_NetSocket connections. 

#if 0
#ifdef ASURA_PC_SUPPORT_STEAMWORKS

class Asura_NetConnection_SteamServerInfo
{
	public:
		
		uint64							m_uSteamID;
		ASURA_PC_STEAMWORKS_AUTH_LEVEL	m_eAuthLevel;
};

class Asura_NetConnection_SteamClientInfo
{
	public:

		u_int	m_uAuthBlobSize;
		void*	m_pAuthBlob;
};

#endif	// ASURA_PC_SUPPORT_STEAMWORKS
#endif	// 0

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

u_int Asura_NetConnection::Platform_GetMessageDataSize( const u_int uMessageID )
{
	#if 0
	#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	if( Asura_PC_Steamworks::IsSteamworksEnabled() )
	{
		switch( uMessageID )
		{
			case ASURA_NETTRANSPORT_MSG_CHALLENGE_RESPONSE:
			{
				if( Asura_PC_SteamworksServer::IsServerRunning() )
				{
					return sizeof( Asura_NetConnection_SteamServerInfo );
				}
			}
			break;

			case ASURA_NETTRANSPORT_MSG_CONNECTION_REQUEST:
			{
				if( !Asura_Network::IsServer() )
				{
					void* pAuthBlob;
					u_int uBlobSize;
					if( Asura_PC_SteamworksClient::AcquireAuthBlob( pAuthBlob, uBlobSize ) )
					{
						return ( sizeof( Asura_NetConnection_SteamClientInfo ) - sizeof( void* ) + uBlobSize );
					}
					#ifdef ASURA_ASSERT
					else
					{
						Asura_Assert( false, "Failed to get auth blob?" );
					}
					#endif
				}
			}
			break;

			default:
			{
				return 0;
			}
		}
		return 0;
	}
	else
	#endif	// ASURA_PC_SUPPORT_STEAMWORKS
	#endif	// 0
	{
		return 0;
	}
}

void Asura_NetConnection::Platform_FillMessageData( const u_int uMessageID, void* pData )
{
	#if 0
	#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	if( Asura_PC_Steamworks::IsSteamworksEnabled() )
	{
		switch( uMessageID )
		{
			case ASURA_NETTRANSPORT_MSG_CHALLENGE_RESPONSE:
			{
				if( Asura_PC_SteamworksServer::IsServerRunning() )
				{
					// This is a new client connecting to us, we should send it enough information for it
					// to get its auth blob
					Asura_NetConnection_SteamServerInfo* pxMsgData = reinterpret_cast< Asura_NetConnection_SteamServerInfo* >( pData );

					pxMsgData->m_eAuthLevel = Asura_PC_SteamworksServer::GetAuthLevel();
					pxMsgData->m_uSteamID = Asura_PC_SteamworksServer::GetSteamID_uint64();
				}
			}
			break;

			case ASURA_NETTRANSPORT_MSG_CONNECTION_REQUEST:
			{
				if( !Asura_Network::IsServer() && !Asura_PC_SteamworksServer::IsServerRunning() )
				{
					// We are going to connect to the server, send our auth blob
					void* pAuthBlob;
					u_int uBlobSize;
					if( Asura_PC_SteamworksClient::AcquireAuthBlob( pAuthBlob, uBlobSize ) )
					{
						Asura_NetConnection_SteamClientInfo* pxClientInfo = reinterpret_cast< Asura_NetConnection_SteamClientInfo* >( pData );
						
						pxClientInfo->m_uAuthBlobSize = uBlobSize;
						Asura_Mem::Copy( &pxClientInfo->m_pAuthBlob, pAuthBlob, uBlobSize );
					}
					#ifdef ASURA_ASSERT
					else
					{
						Asura_Assert( false, "Failed to get auth blob?" );
					}
					#endif
				}
			}
			break;
		}
	}
	#endif	// ASURA_PC_SUPPORT_STEAMWORKS
	#endif	// 0
}


void Asura_NetConnection::Platform_ProcessMessageData( const u_int uMessageID, const void* pData )
{
	#if 0
	#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	if( Asura_PC_Steamworks::IsSteamworksEnabled() )
	{
		switch( uMessageID )
		{
			case ASURA_NETTRANSPORT_MSG_CHALLENGE_RESPONSE:
			{
				if( !Asura_Network::IsServer() )
				{
					const Asura_NetConnection_SteamServerInfo* pxServerInfo = reinterpret_cast< const Asura_NetConnection_SteamServerInfo* >( pData );
					if( pxServerInfo )
					{
						// Grab the netsocket to find the address of the server
						const Asura_NetSocket* pxSocket = Asura_NetTransport_System::GetSocket();
						if( !pxSocket )
						{
							return;
						}

						Asura_NetAddress xAddress = pxSocket->GetAddress();

						#ifdef ASURA_ASSERT
						const bool bServerConOk =
						#endif
						Asura_PC_SteamworksClient::InitiateServerConnection( CSteamID( pxServerInfo->m_uSteamID ), pxServerInfo->m_eAuthLevel, xAddress, pxSocket->GetPort() );
						Asura_Assert( bServerConOk, "Failed to initiate server connection" );
					}
				}
			}
			break;

			case ASURA_NETTRANSPORT_MSG_CONNECTION_REQUEST:
			{
				if( Asura_PC_SteamworksServer::IsServerRunning() )
				{
					const Asura_NetConnection_SteamClientInfo* pxClientInfo = reinterpret_cast< const Asura_NetConnection_SteamClientInfo* >( pData );

					// The client should have sent us their auth blob
					Asura_PC_SteamworksServer::HandleUserConnection( m_xAddress.m_uAddress, m_usPort, &pxClientInfo->m_pAuthBlob, pxClientInfo->m_uAuthBlobSize );
				}
			}
			break;
		}
	}
	#endif	// ASURA_PC_SUPPORT_STEAMWORKS
	#endif	// 0
}


bool Asura_NetConnection::Platform_HandlesClientRequests ( )
{
	#if 0
	#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	if( Asura_PC_Steamworks::IsSteamworksEnabled() )
	{
		// If we are running a steam server and it is authenticating users, we will need to do some extra stuff...
		if( Asura_PC_SteamworksServer::IsServerRunning() && ( Asura_PC_SteamworksServer::GetAuthLevel() > ASURA_PC_STEAMWORKS_NO_AUTHENTICATE ) )
		{
			return true;
		}
		return false;
	}
	else
	#endif
	#endif
	{
		return false;
	}
}


void Asura_NetConnection::Platform_CloseConnection ( )
{
	#if 0
	#ifdef ASURA_PC_SUPPORT_STEAMWORKS
	if( Asura_PC_Steamworks::IsSteamworksEnabled() )
	{
		if( Asura_PC_SteamworksClient::IsConnectedToServer() )
		{
			Asura_PC_SteamworksClient::TerminateServerConnection();
		}
		if( Asura_PC_SteamworksServer::IsServerRunning() )
		{
			Asura_PC_SteamworksServer::HandleUserDisconnection( m_xAddress, m_usPort );
		}
	}
	#endif
	#endif
}

#endif

//eof
