//////////////////////////////////////////////////////////////////////////////
//
// Asura_PC_SteamworksAchievements.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
// Description:
//
//     Asura wrapper class for steam achievements
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_REWARDS
#ifdef ASURA_PC_SUPPORT_STEAMWORKS

// This file's header.
#include "Network/Steamworks/Asura_PC_SteamworksAchievements.h"

#include "Debugging/Asura_Logfile.h"
#include "Network/Steamworks/Asura_PC_SteamworksStats.h"

Asura_PC_Steam_Callback< UserAchievementStored_t > Asura_PC_SteamworksAchievements::s_xCallback_AchievementStored(Asura_PC_SteamworksAchievements::Callback_AchievementStored);

void Asura_PC_SteamworksAchievements::Initialise()
{
	s_xCallback_AchievementStored.Register();
}

void Asura_PC_SteamworksAchievements::Shutdown()
{
	s_xCallback_AchievementStored.Unregister();
}

bool Asura_PC_SteamworksAchievements::IsInitComplete()
{
	return Asura_PC_SteamworksStats::IsInitComplete();
}

void Asura_PC_SteamworksAchievements::AchievementProgress(const char* szName, u_int uProgress, u_int uTarget)
{
	if( Asura_PC_Steamworks::IsInitialised() )
	{
		SteamUserStats()->IndicateAchievementProgress(szName, uProgress, uTarget);
	}
}

void Asura_PC_SteamworksAchievements::UnlockAchievement(const char* szName)
{
	if( Asura_PC_Steamworks::IsInitialised() )
	{
		Asura_Assert(Asura_PC_SteamworksStats::IsInitComplete(), "trying to unlock achievement before steam has checked in - may not work");
		SteamUserStats()->SetAchievement(szName);
		Asura_PC_SteamworksStats::StoreStats();
	}
}

void Asura_PC_SteamworksAchievements::LockAchievement(const char* szName)
{
	if( Asura_PC_Steamworks::IsInitialised() )
	{
		Asura_Assert(Asura_PC_SteamworksStats::IsInitComplete(), "trying to lock achievement before steam has checked in - may not work");
		SteamUserStats()->ClearAchievement(szName);
		Asura_PC_SteamworksStats::StoreStats();
	}
}

bool Asura_PC_SteamworksAchievements::IsAchievementUnlocked(const char* szName)
{
	bool bUnlocked(false);
	if (Asura_PC_SteamworksStats::IsInitComplete())
	{
		SteamUserStats()->GetAchievement(szName, &bUnlocked);
	}
	return bUnlocked;
}

void Asura_PC_SteamworksAchievements::Callback_AchievementStored ( const UserAchievementStored_t* const pxParam )
{
	// we may get callbacks for other games' stats arriving, ignore them
	CGameID xGameID( SteamUtils()->GetAppID() );
	if ( xGameID.ToUint64() == pxParam->m_nGameID )
	{
		if (pxParam->m_nMaxProgress == 0 )
		{
			Asura_OutputToDebugger(( "Steamworks callback : Achievement '%s' unlocked!", pxParam->m_rgchAchievementName ));
		}
		else
		{
			Asura_OutputToDebugger(( "Steamworks callback : Achievement '%s' progress callback, (%d,%d)\n", 
				pxParam->m_rgchAchievementName, pxParam->m_nCurProgress, pxParam->m_nMaxProgress ));
		}
	}
}

#endif // #ifdef ASURA_PC_SUPPORT_STEAMWORKS
#endif // #ifdef ASURA_USE_REWARDS
