// Asura_PC_SteamworksLobby.cpp
// Thomas Young - 23.04.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_PC_SUPPORT_STEAMWORKS

#include "Core/Asura_List.h"

#include "Network/Steamworks/Asura_PC_SteamworksClient.h"
#include "Network/Steamworks/Asura_PC_SteamworksLobby.h"
#include "Network/Steamworks/Asura_PC_SteamworksServer.h"

// S T A T I C   M E M B E R   V A R I A B L E S /////////////////////////////

Asura_List< Asura_PC_SteamworksLobby >	Asura_PC_SteamworksLobby_System::s_xLobbyList;
Asura_PC_SteamLobbyHandle				Asura_PC_SteamworksLobby_System::s_xNextLobbyHandle	= 1;

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

// ______________________________ Asura_PC_SteamworksLobby_System _________________________________

void Asura_PC_SteamworksLobby_System::Initialise ()
{

}


void Asura_PC_SteamworksLobby_System::Update ()
{

}


void Asura_PC_SteamworksLobby_System::Shutdown ()
{
	
}


Asura_PC_SteamLobbyHandle Asura_PC_SteamworksLobby_System::CreateLobby ( const Asura_List<Asura_PC_Steamworks_KeyValuePair>* pxMetaData, const bool bPrivate )
{
	if( !Asura_PC_Steamworks::IsInitialised() )
	{
		Asura_Assert( false, "Steamworks not initialised" );
		return Asura_PC_SteamInvalidLobbyHandle;
	}

	// Add a lobby to our list

	Asura_PC_SteamworksLobby* pxLobby = new Asura_PC_SteamworksLobby( );

	if( pxLobby )
	{
		s_xLobbyList.AddToList( pxLobby );

		// Set this lobby to start creating a steam lobby

		pxLobby->SetHandle	( s_xNextLobbyHandle );
		pxLobby->Create		( pxMetaData, bPrivate );

		++s_xNextLobbyHandle;

		return ( pxLobby->GetHandle() );
	}
	
	return Asura_PC_SteamInvalidLobbyHandle;
}


Asura_PC_SteamLobbyHandle Asura_PC_SteamworksLobby_System::JoinLobby ( const CSteamID xLobbySteamID )
{
	if( !Asura_PC_Steamworks::IsInitialised() )
	{
		Asura_Assert( false, "Steamworks not initialised" );
		return Asura_PC_SteamInvalidLobbyHandle;
	}

	// Add a lobby to our list

	Asura_PC_SteamworksLobby* pxLobby = new Asura_PC_SteamworksLobby( );

	if( pxLobby )
	{
		s_xLobbyList.AddToList( pxLobby );

		// Set this lobby to start joining a steam lobby

		pxLobby->SetHandle	( s_xNextLobbyHandle );
		pxLobby->Join		( xLobbySteamID );

		++s_xNextLobbyHandle;

		return ( pxLobby->GetHandle() );
	}

	return Asura_PC_SteamInvalidLobbyHandle;
}


Asura_PC_SteamworksLobby* Asura_PC_SteamworksLobby_System::GetLobby ( const CSteamID xSteamID )
{
	Asura_List_It< Asura_PC_SteamworksLobby > xIter( s_xLobbyList );

	for( ; !xIter.Done(); xIter.Next() )
	{
		Asura_PC_SteamworksLobby* pxLobby = xIter.GetCurrent();
		
		if( pxLobby && ( pxLobby->GetSteamID() == xSteamID ) )
		{
			return pxLobby;
		}
	}

	Asura_Assert( false, "Failed to find lobby" );
	return NULL;
}


Asura_PC_SteamworksLobby* Asura_PC_SteamworksLobby_System::GetLobby ( const Asura_PC_SteamLobbyHandle xHandle )
{
	Asura_List_It< Asura_PC_SteamworksLobby > xIter( s_xLobbyList );

	for( ; !xIter.Done(); xIter.Next() )
	{
		Asura_PC_SteamworksLobby* pxLobby = xIter.GetCurrent();

		if( pxLobby && ( pxLobby->GetHandle() == xHandle ) )
		{
			return pxLobby;
		}
	}

	Asura_Assertf( false, ( "Failed to find lobby with handle %u ", xHandle ) );
	return NULL;
}


void Asura_PC_SteamworksLobby_System::LeaveLobby ( const Asura_PC_SteamLobbyHandle xHandle )
{
	Asura_PC_SteamworksLobby* pxLobby = GetLobby( xHandle );

	if( !pxLobby )
	{
		// Dont assert, getlobby wil do it for us
		return;
	}

	s_xLobbyList.RemoveFromList( pxLobby );
	delete pxLobby;
}

// ______________________________ Asura_PC_SteamworksLobby _________________________________

Asura_PC_SteamworksLobby::Asura_PC_SteamworksLobby ( )
{
	m_eState = ASURA_STEAM_LOBBY_STATE_IDLE;
}


Asura_PC_SteamworksLobby::~Asura_PC_SteamworksLobby ( )
{
	while( !m_xMetaData.IsEmpty() )
	{
		delete m_xMetaData.PopHeadObject();
	}

	if( SteamMatchmaking() && IsActive() )
	{
		SteamMatchmaking()->LeaveLobby( m_xSteamID );
	}
}


void Asura_PC_SteamworksLobby::Join ( CSteamID xLobbySteamID )
{
	ASURA_PC_STEAM_RETURN_VOID_IF_UNINITIALISED

	if( !SteamMatchmaking() )
	{
		Asura_Assert( false, "No steam matchmaking API object" );
		return;
	}

	m_xSteamID = xLobbySteamID;

	SteamAPICall_t xResult = SteamMatchmaking()->JoinLobby( xLobbySteamID );

	m_xCallResult_EnterLobby.Set( xResult, this,  &Asura_PC_SteamworksLobby::Callback_EnterLobby );		// Callback_EnterLobby will be called

	m_eState = ASURA_STEAM_LOBBY_STATE_JOINING;
}


void Asura_PC_SteamworksLobby::Create ( const Asura_List<Asura_PC_Steamworks_KeyValuePair>* pxMetaData, const bool bPrivate )
{
	ASURA_PC_STEAM_RETURN_VOID_IF_UNINITIALISED

	if( m_eState != ASURA_STEAM_LOBBY_STATE_IDLE )
	{
		Asura_Assert( false, "Lobby should be idle when calling create" );
		return;
	}

	if( !SteamMatchmaking() )
	{
		Asura_Assert( false, "No steam matchmaking API object" );
		return;
	}

	if( pxMetaData )
	{
		Asura_List_ConstIt<Asura_PC_Steamworks_KeyValuePair> xIter( *pxMetaData );

		for( ; !xIter.Done(); xIter.Next() )
		{
			Asura_PC_Steamworks_KeyValuePair* pxNewData = new Asura_PC_Steamworks_KeyValuePair( *xIter.GetCurrent() );

			m_xMetaData.AddToList( pxNewData );
		}
	}

	SteamAPICall_t xResult = SteamMatchmaking()->CreateLobby( bPrivate ? k_ELobbyTypeFriendsOnly : k_ELobbyTypePublic );

	// 'set' our callback object so that this result will trigger it
	m_xCallResult_LobbyCreated.Set( xResult, this, &Asura_PC_SteamworksLobby::Callback_LobbyCreated );

	m_eState = ASURA_STEAM_LOBBY_STATE_CREATING;
}


CSteamID Asura_PC_SteamworksLobby::GetSteamID ( ) const
{
	if( !IsActive() )
	{
		Asura_Assert( false, "Not connected" );
		return CSteamID();
	}
	return m_xSteamID;
}


uint64 Asura_PC_SteamworksLobby::GetSteamID_uint64 ( ) const
{
	if( !IsActive() )
	{
		Asura_Assert( false, "Not connected" );
		return CSteamID().ConvertToUint64();
	}

	return m_xSteamID.ConvertToUint64();
}


CSteamID Asura_PC_SteamworksLobby::GetOwner ( ) const
{
	if( !SteamMatchmaking() )
	{
		Asura_Assert( false, "No steam matchmaking API object" );
		return CSteamID();
	}

	if( !IsActive() )
	{
		Asura_Assert( false, "Not connected" );
		return CSteamID();
	}

	return SteamMatchmaking()->GetLobbyOwner( m_xSteamID );
}


bool Asura_PC_SteamworksLobby::OwnerIsMe ( ) const
{
	if( !SteamMatchmaking() )
	{
		Asura_Assert( false, "No steam matchmaking API object" );
		return false;
	}

	if( !IsActive() )
	{
		Asura_Assert( false, "Not connected" );
		return false;
	}

	return ( GetOwner() == Asura_PC_SteamworksClient::GetSteamID() );
}


void Asura_PC_SteamworksLobby::SetServer ( const CSteamID xServerSteamID, const u_int uServerIP, const u_short usServerPort )
{
	if( !SteamMatchmaking() )
	{
		Asura_Assert( false, "No steam matchmaking API object" );
		return;
	}

	if( !IsActive() )
	{
		return;
	}

	SteamMatchmaking()->SetLobbyGameServer( m_xSteamID, uServerIP, usServerPort, xServerSteamID );
}


void Asura_PC_SteamworksLobby::UpdateMetaData ( Asura_List<Asura_PC_Steamworks_KeyValuePair>* pxMetaData )
{
	if( !SteamMatchmaking() )
	{
		Asura_Assert( false, "No steam matchmaking API object" );
		return;
	}

	if( !IsActive() )
	{
		Asura_Assert( false, "Lobby is not running yet, cannot update meta data" );
		return;
	}

	if( !pxMetaData )
	{
		return;
	}

	// Set our meta data
	Asura_List_It<Asura_PC_Steamworks_KeyValuePair> xIter( *pxMetaData );

	for( ; !xIter.Done(); xIter.Next() )
	{
		Asura_PC_Steamworks_KeyValuePair* pxMetaDataItem = xIter.GetCurrent();
		SteamMatchmaking()->SetLobbyData( m_xSteamID, pxMetaDataItem->m_szKey, pxMetaDataItem->m_szValue ) ;
	}
}


void Asura_PC_SteamworksLobby::Callback_LobbyCreated( LobbyCreated_t *pxParam, bool bIOFailure )
{
	if( m_eState != ASURA_STEAM_LOBBY_STATE_CREATING )
	{
		return;
	}

	if( pxParam->m_eResult == k_EResultOK )
	{
		// Set the variables that say we are alive

		m_xSteamID = pxParam->m_ulSteamIDLobby;

		m_eState = ASURA_STEAM_LOBBY_STATE_ACTIVE;

		// Set the meta data based on what we saved at creation time

		UpdateMetaData( &m_xMetaData );

		// and delete it since we dont need to store it for any reason

		while( !m_xMetaData.IsEmpty() )
		{
			delete m_xMetaData.PopHeadObject();
		}
	}
	else
	{
		Asura_Assertf( false, ( "Failed to create lobby, reason: %d", static_cast<int>( pxParam->m_eResult ) ) );
		m_eState = ASURA_STEAM_LOBBY_STATE_ERROR;
	}
}


void Asura_PC_SteamworksLobby::Callback_EnterLobby ( LobbyEnter_t* pxParam, bool bIOFailure )
{
	if( m_eState != ASURA_STEAM_LOBBY_STATE_JOINING )
	{
		return;
	}
	
	if( pxParam->m_ulSteamIDLobby == m_xSteamID.ConvertToUint64() )
	{
		if( pxParam->m_EChatRoomEnterResponse == k_EChatRoomEnterResponseSuccess )
		{
			m_eState = ASURA_STEAM_LOBBY_STATE_ACTIVE;
		}
		else
		{
			m_eState = ASURA_STEAM_LOBBY_STATE_ERROR;
		}
	}
}


Asura_PC_SteamLobbyHandle Asura_PC_SteamworksLobby::GetHandle ( ) const
{
	return m_xHandle;
}


void Asura_PC_SteamworksLobby::SetHandle ( const Asura_PC_SteamLobbyHandle xHandle )
{
	m_xHandle = xHandle;
}


bool Asura_PC_SteamworksLobby::IsActive ( ) const
{ 
	return ( m_eState == ASURA_STEAM_LOBBY_STATE_ACTIVE );
}


ASURA_STEAM_LOBBY_STATE Asura_PC_SteamworksLobby::GetState ( ) const
{
	return m_eState;
}

#endif ASURA_PC_SUPPORT_STEAMWORKS

//eof
