// Asura_PC_SteamworksServer.h
// Thomas Young - 13.01.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

// F I L E   W R A P P E R ///////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef ___ASURA_STEAMWORKSSERVER_H___
#define ___ASURA_STEAMWORKSSERVER_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_PC_SUPPORT_STEAMWORKS

#include "Network/Steamworks/Asura_PC_Steamworks.h"

#include "Steam/Steam_api.h"
#include "Steam/Steam_gameserver.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

struct Asura_NetAddress;

class Asura_PC_Steamworks_ClientInfo;

template< class Asura_PC_Steamworks_ClientInfo > class Asura_List;

// T Y P E D E F S /////////////////////////////////////////////////////////

typedef void( *Asura_PC_SteamServer_Callback_ClientAccept ) ( const u_int uIP, const unsigned short usPort );
typedef void( *Asura_PC_SteamServer_Callback_ClientReject ) ( const u_int uIP, const unsigned short usPort, const u_int uReason );

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////

// _______________________ Asura_PC_Steamworks_ServerConfig ___________________________

class Asura_PC_Steamworks_ServerConfig
{
	// This class is passed to the creation function to configure the server
	public:

		char m_szServerName	 [64];
		char m_szDescription [64];
		char m_szMapName	 [64];
		unsigned short m_usMaxPlayers;
		bool m_bRequireAuthentication;
		ASURA_PC_STEAMWORKS_AUTH_LEVEL m_eAuthLevel;
};

// _______________________ Asura_PC_SteamworksServer ___________________________

class Asura_PC_SteamworksServer
{
	friend class Asura_PC_Steamworks;

	public:

		// ______________________________ Functions _________________________________

		static void									Create							( const Asura_PC_Steamworks_ServerConfig* pxConfig );
		static void									Terminate						( );

		static void									AddServerProperty				( const char* szKey, const char* szValue );
		static ASURA_PC_STEAMWORKS_AUTH_LEVEL		GetAuthLevel					( )		{ return s_eAuthLevel; }
		static bool									IsServerRunning					( ) 	{ return s_bServerRunning; }
		
		static CSteamID 							GetSteamID						( );
		static uint64								GetSteamID_uint64				( );
		static u_int								GetPublicIP						( );

		static void									HandleUserConnection			( const u_int uIP, const unsigned short usPort, const void* pAuthBlob, const u_int uAuthBlobSize );
		static void									HandleUserDisconnection			( const Asura_NetAddress xAddress, const unsigned short usPort );

		static void									RegisterCallback_ClientAccept	( Asura_PC_SteamServer_Callback_ClientAccept pfnCallback );
		static void									RegisterCallback_ClientReject	( Asura_PC_SteamServer_Callback_ClientReject pfnCallback );

		static void									SetGamePort						( const unsigned short usPort );
		static unsigned short						GetGamePort						( );

	protected:

		// ______________________________ Functions _________________________________

		static void	Initialise	( );		// Called by Asura_PC_Steamworks
		static void	Update		( );
		static void	Shutdown	( );

	private:
	
		// ______________________________ Functions _________________________________

		Asura_PC_SteamworksServer				( )	{}
		~Asura_PC_SteamworksServer				( )	{}

		static void Callback_SteamConnected		( const SteamServersConnected_t*	const pxParam );
		static void Callback_SteamDisconnected	( const SteamServersDisconnected_t*	const pxParam );
		static void Callback_ClientApprove		( const GSClientApprove_t*			const pxParam );
		static void Callback_ClientDeny			( const GSClientDeny_t*				const pxParam );
		static void Callback_PolicyResponse		( const GSPolicyResponse_t*			const pxParam );

		static void ResendClientList			( );

		static void RunCallback_AcceptClient	( const u_int uIP, const unsigned short usPort );
		static void RunCallback_RejectClient	( const u_int uIP, const unsigned short usPort, const u_int uReason );

		// ______________________________ Variables _________________________________

		static Asura_PC_Steam_Callback<	SteamServersConnected_t >		s_xCallback_SteamConnected;
		static Asura_PC_Steam_Callback< SteamServersDisconnected_t >	s_xCallback_SteamDisconnected;
		static Asura_PC_Steam_Callback< GSClientApprove_t >				s_xCallback_ClientApprove;
		static Asura_PC_Steam_Callback< GSClientDeny_t >				s_xCallback_ClientDeny;
		static Asura_PC_Steam_Callback< GSPolicyResponse_t >			s_xCallback_PolicyResponse;

		static const char*												s_szGameName;
		static bool														s_bServerRunning;

		static bool														s_bSteamConnected;
		static ASURA_PC_STEAMWORKS_AUTH_LEVEL							s_eAuthLevel;
		
		static Asura_PC_SteamServer_Callback_ClientAccept				s_pfnAsuraCallback_ClientAccept;
		static Asura_PC_SteamServer_Callback_ClientReject				s_pfnAsuraCallback_ClientReject;

		static Asura_List<Asura_PC_Steamworks_ClientInfo>				s_xClientList;

		static unsigned short											s_usGamePort;
};

#endif // ASURA_PC_SUPPORT_STEAMWORKS

#endif // ___ASURA_STEAMWORKSSERVER_H___

//eof
