
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PSP_SCEUtilityWrapper.h
// Author : Andy Tate
// Date   : 4-Oct-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_PSP_SCEUTILITYWRAPPER_H__
#define __ASURA_PSP_SCEUTILITYWRAPPER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_PlatformBasics.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_PSP_SCEUtility_Handler;


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const u_int	uASURA_PSP_NICKNAME_MAX_CHARS	= 128;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class Asura_PSP_SCEUtilityWrapper
{
public:
	// Public typedefs...

	typedef bool LoadDataValidationFunction (const void* pxData, const u_int uDataSize, const u_int uBufferSize);


	// Public enumerated types...

	enum CommandState
	{
		COMMAND_INACTIVE,
		COMMAND_IN_PROGRESS,
		COMMAND_COMPLETED,
		COMMAND_CANCELLED,
		COMMAND_ERROR,
		COMMAND_RETRY,

		MAX_COMMAND_STATES
	};

	enum OSKCharacterSet
	{
		LATIN_ALPHABET,
		LATIN_ALPHABET_UPPERCASE,
		LATIN_ALPHABET_LOWERCASE,
		LATIN_NUMERALS,
		LATIN_ALPHANUMERIC,
		LATIN_ALPHANUMERIC_UPPERCASE,
		LATIN_ALPHANUMERIC_LOWERCASE,

		MAX_OSK_CHARACTER_SET
	};


	// Public functions...

	// Static initialisation/shutdown functions...
	static	void						Initialise						(const Asura_UnicodeChar* wszTitleName);
	static	void						Shutdown						(void);

	// Initialisation functions...
	static	CommandState				InitialiseOnScreenKeyboard		(Asura_UnicodeChar const* wszMessage, Asura_UnicodeChar const* wszTextBeforeEdit, Asura_UnicodeChar* wszTextAfterEdit, const int iMaxChars, const OSKCharacterSet eCharacterSet);

	static	CommandState				InitialiseSaveDataAutoload		(void* pvData, const u_int uDataSize, const u_int uBufferSize, LoadDataValidationFunction* pfnValidation = NULL);
	static	CommandState				InitialiseSaveDataAutosave		(Asura_UnicodeChar const* wszSaveTitle, Asura_UnicodeChar const* wszSaveDescription, Asura_UnicodeChar const* wszNewData, void const* pvData, const u_int uDataSize, const u_int uBufferSize);
	static	CommandState				InitialiseSaveDataDelete		(void);
	static	CommandState				InitialiseSaveDataLoadList		(const int iNumSaveSlots, void* pvData, const u_int uDataSize, const u_int uBufferSize, LoadDataValidationFunction* pfnValidation = NULL);
	static	CommandState				InitialiseSaveDataSaveList		(const int iNumSaveSlots, Asura_UnicodeChar const* wszSaveTitle, Asura_UnicodeChar const* wszSaveDescription, Asura_UnicodeChar const* wszNewData, void const* pvData, const u_int uDataSize, const u_int BufferSize);

	static	CommandState				InitialiseSaveDataLoadSingle	(const char* szUserID, void* pvData, const u_int uDataSize, const u_int uBufferSize, LoadDataValidationFunction* pfnValidation = NULL);
	static	CommandState				InitialiseSaveDataSaveSingle	(const char* szUserID, Asura_UnicodeChar const* wszSaveTitle, Asura_UnicodeChar const* wszSaveDescription, Asura_UnicodeChar const* wszNewData, void const* pvData, const u_int uDataSize, const u_int BufferSize);
	static	CommandState				InitialiseSaveDataAutoloadSingle	(const char* szUserID, void* pvData, const u_int uDataSize, const u_int uBufferSize, LoadDataValidationFunction* pfnValidation = NULL);
	static	CommandState				InitialiseSaveDataAutosaveSingle	(const char* szUserID, Asura_UnicodeChar const* wszSaveTitle, Asura_UnicodeChar const* wszSaveDescription, Asura_UnicodeChar const* wszNewData, void const* pvData, const u_int uDataSize, const u_int BufferSize);

	static	CommandState				InitialiseMessageDialog			(const u_int uErrorNumber);

	// Update function...
	static	CommandState				BeginFrame						(void);
	static	CommandState				Update							(void);
	static	CommandState				Render							(void);
	static	CommandState				EndFrame						(void);

	// Memory-stick functions...
	static	bool						IsMemoryStickPresent			(void)	{ return (s_bMemoryStickPreset); }

	// Accessor functions...
	static	bool						IsAbleToPerformSavedataAutosave	(void)	{ return (s_bAbleToPerformAutosave); }
	static	bool						IsCircleButtonEnter				(void)	{ return (s_bCircleButtonIsEnter); }
	static	Asura_UnicodeChar const*	pGetNickname					(void)	{ return (s_wszNickname); }
	static bool GetNeedDelete () { return false; }
	static void ResetNeedDelete () {}

	static void SetGoToDeleteIfNoSpace (const bool bGoToDeleteIfNoSpace) {}

protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...

	// The current handler being used...
	static	Asura_PSP_SCEUtility_Handler*	s_pxCurrentHandler;

	// The current state of the Utility class...
	static	CommandState					s_eCommandState;

	static	bool							s_bAbleToPerformAutosave;
	static	bool							s_bCircleButtonIsEnter;
	static	bool							s_bMemoryStickPreset;

	static	Asura_UnicodeChar				s_wszNickname [uASURA_PSP_NICKNAME_MAX_CHARS];


	// Private functions...

	Asura_PSP_SCEUtilityWrapper  (void);
	Asura_PSP_SCEUtilityWrapper  (const Asura_PSP_SCEUtilityWrapper& xOther);
	~Asura_PSP_SCEUtilityWrapper ();

	Asura_PSP_SCEUtilityWrapper&	operator=	(const Asura_PSP_SCEUtilityWrapper& xOther);

};   // End of class 'Asura_PSP_SCEUtilityWrapper'.


#endif   // ifndef __ASURA_PSP_SCEUTILITYWRAPPER_H__


/// End of file. ///
