
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PSP_SCEUtility_FileData.h
// Author : Andy Tate
// Date   : 6-Oct-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_PSP_SCEUTILITY_FILEDATA_H__
#define __ASURA_PSP_SCEUTILITY_FILEDATA_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_String.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Stream;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class Asura_PSP_SCEUtility_FileData
{
public:
	// Public functions...

	Asura_PSP_SCEUtility_FileData  (const char* szFilename);
	~Asura_PSP_SCEUtility_FileData ();

	// Data updating functions...
			void							UpdateHeaderData			(const Asura_UnicodeChar* wszTitle, const Asura_UnicodeChar* wszDescription, const u_int uDataSize);

	// File-access functions...
			bool							ReadHeaderData				(void);

			bool							ReadData					(u_int& uDataSize, void* pvData);
			bool							WriteData					(const u_int uDataSize, const void* pvData) const;

	// Accessor functions...
			const Asura_Char*				GetFilename					(void) const	{ return (m_xFilename.GetString()); }
			const Asura_Char*				GetDataAndTimeString		(void) const	{ return (m_szDateAndTime); }
			const Asura_UnicodeChar*		GetTitle					(void) const	{ return (m_wszTitle); }
			const Asura_UnicodeChar*		GetDescription				(void) const	{ return (m_wszDescription); }
			u_int							GetDataSize					(void) const	{ return (m_uDataSize); }


protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...

	// Filename (including path) to be used...
	Asura_String		m_xFilename;

	// The text used to store the date and time...
	Asura_Char*			m_szDateAndTime;

	// The title and description of the save...
	Asura_UnicodeChar*	m_wszTitle;
	Asura_UnicodeChar*	m_wszDescription;

	// The size of the data itself...
	u_int				m_uDataSize;


	// Private functions...

	Asura_PSP_SCEUtility_FileData  (void);
	Asura_PSP_SCEUtility_FileData  (const Asura_PSP_SCEUtility_FileData& xOther);

			Asura_PSP_SCEUtility_FileData&	operator=					(const Asura_PSP_SCEUtility_FileData& xOther);

	// File-access functions...
	static	bool							CreateDirectoryRecursive	(const Asura_String& xFullPath);
	static	bool							CreateDirectoryFromFilename	(const Asura_String& xFilename);

			void							ReadHeaderData				(Asura_Stream& xStream);
			void							WriteHeaderData				(Asura_Stream& xStream, const u_int uDataSize) const;

};   // End of class 'Asura_PSP_SCEUtility_FileData'.


#endif   // ifndef __ASURA_PSP_SCEUTILITY_FILEDATA_H__


/// End of file. ///
