
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PSP_SCEUtility_Handler.cpp
// Author : Andy Tate
// Date   : 5-Oct-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "PSPEmul/SCEUtility/Asura_PSP_SCEUtility_Handler.h"

// Asura independent header-files...
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Viewport.h"

// Asura platform header-files...

// Game independent header-files...

// Game platform header-files...

// Tools header-files...
#if defined(ASURA_TOOLS)
#include "Misc/Tools_Render_Options.h"
#endif	// defined(ASURA_TOOLS)


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PSP_SCEUtility_Handler::CalculateScreenScale
// Author       : Andy Tate
// Date         : 5-Oct-2005
//
// Description  : Calculates the screen scale needed to map the PSP screen onto
//				  the current PC window.
//
// Return value : The scale needed.
////////////////////////////////////////////////////////////////////////////////

float Asura_PSP_SCEUtility_Handler::CalculateScreenScale (void) const
{
#if defined(ASURA_TOOLS)
	// Are the tools in PSP mode?
	if (Tools_Render_Options::IsInPSPMode() && !Tools_Render_Options::IsPSPModeActualSize())
	{
		const float	fScale	= Asura_Viewport::GetWidth() / 480.0f;

		// Yes - return the scale...
		return	 (fScale);
	}

	else
	{
		// No - no scale needed...
		return	(1.0f);
	}
#else
	// No scale needed...
	return	(1.0f);
#endif	// defined(ASURA_TOOLS)
}

/// End of function 'Asura_PSP_SCEUtility_Handler::CalculateScreenScale'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PSP_SCEUtility_Handler::CalculateScreenPosition
// Author       : Andy Tate
// Date         : 5-Oct-2005
//
// Description  : Maps the specified PSP screen position onto the current PC
//				  window (in pixels).
//
// Parameters   : The current screen-scale to use and references to the input
//				  PSP screen-position and the output PC screen-position.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_PSP_SCEUtility_Handler::CalculateScreenPosition (const float fScale,
															const Asura_IntVector_2& xPSPPosition,
															Asura_Vector_2& xPCPosition) const
{
	Asura_ViewingArea const*	pxViewingArea	= Asura_Camera::GetViewingArea();

	const float					fMinX			= (pxViewingArea->XOffset * Asura_Viewport::GetWidth());
	const float					fMinY			= (pxViewingArea->YOffset * Asura_Viewport::GetHeight());

	// Calculate the PC equivalents..,
	xPCPosition [0]	= (static_cast<float>(xPSPPosition [0]) * fScale) + fMinX;
	xPCPosition [1]	= (static_cast<float>(xPSPPosition [1]) * fScale) + fMinY;
}

/// End of function 'Asura_PSP_SCEUtility_Handler::CalculateScreenPosition'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PSP_SCEUtility_Handler::CalculateScreenPosition
// Author       : Andy Tate
// Date         : 5-Oct-2005
//
// Description  : Maps the specified PSP screen position onto the current PC
//				  window (in pixels).
//
// Parameters   : References to the input PSP screen-position and the output 
//				  PC screen-position.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_PSP_SCEUtility_Handler::CalculateScreenPosition (const Asura_IntVector_2& xPSPPosition,
															Asura_Vector_2& xPCPosition) const
{
	const float	fScale	= CalculateScreenScale ();

	// Calculate the screen-position...
	CalculateScreenPosition	(fScale, xPSPPosition, xPCPosition);
}

/// End of function 'Asura_PSP_SCEUtility_Handler::CalculateScreenPosition'. ///


/// End of file. ///
