//////////////////////////////////////////////////////////////////////////////
//
// Asura_Direct3D_AmbientOcclusion.cpp
//
// Created by Filip Strugar 10/03/2009
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////


// INCLUDES

// to enable edit and continue...
// #include "AVP_PC_PCH.h"

#include "Core/Asura.h"
 
#ifdef ASURA_USE_AMBIENT_OCCLUSION

// this file's header
#include "Rendering/Asura_Direct3D_AmbientOcclusion.h"

// independent
#include "Camera/Asura_Camera.h"
#include "Interface/Asura_CommandConsole.h"
#include "Rendering/Asura_AmbientOcclusion.h"
#include "Rendering/Asura_Viewport.h"

// main header
#include "FSFX/Asura_FSFX_Component.h"
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"

#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_UserOptions.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_Direct3D_StoreRenderTargets.h"
#include "Rendering/Asura_Overlay_Quad.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_ViewPort.h"

// The number of ambient occlusion blurred levels, not counting sampling direct from the depth buffer for the first (sharpest) level (if we're doing that)
// i.e. this is the number of textures that need to be generated by pre-processing before the actual SSAO pass.
//#define NUM_AMBIENT_OCCLUSION_LEVELS	(3)	// the shader has a corresponding value, hard-coded so it can unroll loops
#define NUM_AMBIENT_OCCLUSION_LEVELS			2
#define NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS	2 //corresponding to declaration in AmbientOcclusion_FP.fpx


#if !defined(ASURA_TARGET_SM3)
#error( ASURA_TARGET_SM3 must be defined when ASURA_USE_AMBIENT_OCCLUSION is defined )
#endif


//// Macro to auto-assert on dx error, used for debugging during coding only
//#if ( defined( ASURA_ASSERT ) && ( ASURA_ASSERT == 1 ) )
//	#define DX_VERIFY(x)           { HRESULT hr = (x); if( FAILED(hr) ) { Asura_Assertf( false, ("DirectX returns FAILED(hr), hr == %x", hr) ); } }
//#else
	#define DX_VERIFY(x)           { (x); }
//#endif


// Globals

static Asura_Direct3D_Effect*			g_pxSSAOEffect			=	NULL;

// DEBUG

enum SSAO_DEV_MODE {
	SSAO_OFF,
	SSAO_ON,
	SSAO_OPAQUE,	// replace screen colour with ambient occlusion result
	//SSAO_TEST_OFF,	// disable the second step of SSAO, but still do the first (for profiling)
	NUM_SSAO_MODES
};
//
char* g_aszSSAOModeName[] = {
	"OFF",
	"ON",
	"OPAQUE",
//	"TEST-OFF",
};

static int		g_iSSAOMode = SSAO_ON;

static int		g_iSampleDistance		= 1;		// 0 = sample adjacent pixels, bigger values artificially expand the sample range
static int		g_iReadBackScale		= 2;		// factor of g_iSampleDistance to use when reading the blurred buffers
static int		g_iSampleTightness		= 0;
static float	g_fDepthThreshold		= 0.3f;		// depth-difference between pixels at which they are treated as seperate objects
static float	g_fDistanceThreshold	= 0.8f;		// a similar value to the depth difference, but applied in the final stage to fade the edge. The interaction between these two vals is unfortunate but with tweaking can yield good results
static float	g_fBrightness			= -1.2f;
static float	g_fContrast				= 2.5f;
#if ( NUM_AMBIENT_OCCLUSION_LEVELS > NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS )
	static bool g_bShowBuffers = false;
#else
	static const bool g_bShowBuffers = false;
#endif
//static bool		g_b16TapFilter			= true;		// more taps, fewer passes. This means we can eliminate intermediate buffers at the cost of some extra processing.


// DirectX9 Variable Handles

static bool		g_bEffectVariableHandlesInitialised	= false;

static D3DXHANDLE		g_xRTTextureResolution				= NULL;
static D3DXHANDLE		g_xRTPixelSize						= NULL;
static D3DXHANDLE		g_xDepthBufferTexture				= NULL;
static D3DXHANDLE		g_xParams							= NULL;
static D3DXHANDLE		g_xDepthProjection					= NULL;
static D3DXHANDLE		g_xViewMatrix						= NULL;
static D3DXHANDLE		g_xCameraTanHalfFOV					= NULL;
static D3DXHANDLE		g_xTwoOverScreen					= NULL;
static D3DXHANDLE		g_xBrightnessContrast				= NULL;
static D3DXHANDLE		g_xDistanceFade						= NULL;
static D3DXHANDLE		g_xSamplePositions					= NULL;
static D3DXHANDLE		g_xDepthAdjustment					= NULL;
static D3DXHANDLE		g_xDeferredRT_NI_Texture			= NULL;
//static D3DXHANDLE		g_xInvViewMatrix					= NULL;
//static D3DXHANDLE		g_xCameraTanHalfVertFOV			= NULL;
//static D3DXHANDLE		g_xCameraTanHalfHorizFOV			= NULL;
//static D3DXHANDLE		g_xTwoOverScreenWidth				= NULL;
//static D3DXHANDLE		g_xTwoOverScreenHeight				= NULL;

static D3DXHANDLE		g_axDepthDownsampleTexture[NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS]	= { NULL };


// FUNCTIONS

void Asura_AmbientOcclusion::Platform_Initialise()			{ Asura_Direct3D_AmbientOcclusion::Initialise(); }
void Asura_AmbientOcclusion::Platform_Shutdown()			{ Asura_Direct3D_AmbientOcclusion::Shutdown(); }
void Asura_AmbientOcclusion::Platform_Reset()				{ }
void Asura_AmbientOcclusion::Platform_PrepareDepthData()	{ }
void Asura_AmbientOcclusion::Platform_Render()				{ Asura_Direct3D_AmbientOcclusion::Render(); }

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
static bool ChangeMode ( bool bInc, bool bInterrogate, bool bHelp )
{
	if ( bHelp || bInterrogate )
	{
		Asura_CommandConsole::Printf( "%s", g_aszSSAOModeName[g_iSSAOMode] );
		return true;
	}

	g_iSSAOMode += bInc?1:-1;
	if ( g_iSSAOMode < 0 ) g_iSSAOMode = NUM_SSAO_MODES-1;
	if ( g_iSSAOMode >= NUM_SSAO_MODES ) g_iSSAOMode = 0;

	return true;
}
#endif

#if ASURA_LOAD_EFFECT_FILES == 1
static void OnEffectLoaded(Asura_Direct3D_Effect* pxEffect)
{
	//	CaptureEffect();
}

static void OnEffectUnloaded(Asura_Direct3D_Effect* pxEffect)
{
	//	s_pxEffect = NULL;
	g_bEffectVariableHandlesInitialised = false;
}
#endif

void Asura_Direct3D_AmbientOcclusion::Initialise()
{
	// Debugging stuff

	#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
		Asura_CommandConsole::AddCmd( L"Render.SSAO.Mode", ChangeMode, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	//#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
	//	Asura_CommandConsole::AddCmd( L"Render.SSAO.ReloadAmbOccShader", ReloadAmbientOcclusionShader, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	//	Asura_CommandConsole::AddCmd( L"Render.SSAO.ReloadDepthShader", ReloadDepthDownsampleShader, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	//#endif
		Asura_CommandConsole::AddVar( L"Render.SSAO.SampleDistance", &g_iSampleDistance, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.SampleReadScale", &g_iReadBackScale, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.TightnessTweak", &g_iSampleTightness, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.DepthThreshold", &g_fDepthThreshold, 0, FLT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.DistanceThreshold", &g_fDistanceThreshold, 0, FLT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.Contrast", &g_fContrast, -FLT_MAX, FLT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		Asura_CommandConsole::AddVar( L"Render.SSAO.Brightness", &g_fBrightness, -FLT_MAX, FLT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	#if ( NUM_AMBIENT_OCCLUSION_LEVELS > NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS )
		Asura_CommandConsole::AddVar( L"Render.SSAO.ShowBuffers",  &g_bShowBuffers,  Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	#endif
		//Asura_CommandConsole::AddVar( L"Render.SSAO.16TapFilter",  &g_b16TapFilter,  Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
		
	#endif

	g_pxSSAOEffect = Asura_Direct3D_Effects_System::GetEffect("ScreenSpaceAmbientOcclusion");
	g_pxSSAOEffect->SetOnLoadCallback( OnEffectLoaded );
	g_pxSSAOEffect->SetOnUnloadCallback( OnEffectUnloaded );
}

static void InitialiseEffectVariableHandles()
{
	Asura_Assert( g_pxSSAOEffect != NULL, "The SSAO effect is not initialised!" );
	if( g_pxSSAOEffect == NULL )
		return;


 	g_xRTTextureResolution				= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xRTTextureResolution" );
	g_xRTPixelSize						= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xRTPixelSize" );
	g_xDepthBufferTexture				= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xDepthBufferTexture" );
	g_xParams							= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xParams" );
	g_xDepthProjection					= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xDepthProjection" );
	g_xViewMatrix						= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xViewMatrix" );
	g_xCameraTanHalfFOV					= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xCameraTanHalfFOV" );
	g_xTwoOverScreen					= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xTwoOverScreen" );
	g_xBrightnessContrast				= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xBrightnessContrast" );
	g_xDistanceFade						= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xDistanceFade" );
	g_xSamplePositions					= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_axSamplePositions" );
	g_xDepthAdjustment					= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xDepthAdjustment" );
	g_xDeferredRT_NI_Texture			= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xDeferredRT_NI_Texture" );
	// g_xInvViewMatrix					= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_xInvViewMatrix" );
	// g_xCameraTanHalfVertFOV			= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fCameraTanHalfVertFOV" );
	// g_xCameraTanHalfHorizFOV			= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fCameraTanHalfHorizFOV" );
	// g_xTwoOverScreenWidth			= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fTwoOverScreenWidth" );
	// g_xTwoOverScreenHeight			= g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, "g_fTwoOverScreenHeight" );

	for( int i = 0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
	{
		Asura_Char szBuff[1024]; Asura_PrintfToBfr( szBuff, "g_axDepthDownsampleTexture%d", sizeof(szBuff), true, i );
		g_axDepthDownsampleTexture[i] = g_pxSSAOEffect->GetD3DEffect()->GetParameterByName( NULL, szBuff );
	}
}

void Asura_Direct3D_AmbientOcclusion::Shutdown()
{
	g_bEffectVariableHandlesInitialised = false;
}

struct FSQ_VERTEX
{
	D3DXVECTOR4 p;

	FSQ_VERTEX() {}
	FSQ_VERTEX( float x, float y, float u, float v ) : p(x, y, 0.0f, 1.0f) { u; v; }
};

static void DrawFullscreenQuad( )
{
	// Set up the verts for a full-screen pass
	// ----------------------------------------------------------------------------------------------------
	float fViewWidth		= 0;
	float fViewHeight		= 0;
	{
		IDirect3DSurface9 * pxRenderTarget = NULL;
		DX_VERIFY( Asura_Direct3D::D3DDevice->GetRenderTarget( 0, &pxRenderTarget ) );
		D3DSURFACE_DESC xSurfDesc;
		DX_VERIFY( pxRenderTarget->GetDesc(&xSurfDesc) );
		SAFE_RELEASE( pxRenderTarget );
		fViewWidth	= (float)xSurfDesc.Width;
		fViewHeight = (float)xSurfDesc.Height;

		float xFltArr[4];
		xFltArr[0]	= fViewWidth;
		xFltArr[1]	= fViewHeight;
		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloatArray( g_xRTTextureResolution, xFltArr, 2 ) );
		xFltArr[0]	= 1.0f / fViewWidth;
		xFltArr[1]	= 1.0f / fViewHeight;
		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloatArray( g_xRTPixelSize, xFltArr, 2 ) );
	}

	FSQ_VERTEX axVerts[4];

	axVerts[0] = FSQ_VERTEX( 0 - 0.5f, 0 - 0.5f,						0.0f, 0.0f );
	axVerts[1] = FSQ_VERTEX( fViewWidth - 0.5f, 0 - 0.5f,				1.0f, 0.0f );
	axVerts[2] = FSQ_VERTEX( 0 - 0.5f, fViewHeight - 0.5f,				0.0f, 1.0f );
	axVerts[3] = FSQ_VERTEX( fViewWidth - 0.5f, fViewHeight - 0.5f,		1.0f, 1.0f );

	DX_VERIFY( Asura_Direct3D::D3DDevice->SetFVF( D3DFVF_XYZRHW ) );
	DX_VERIFY( Asura_Direct3D::D3DDevice->SetVertexShader( NULL ) );
	g_pxSSAOEffect->GetD3DEffect()->CommitChanges();
	DX_VERIFY( Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLESTRIP, 2, axVerts, sizeof(FSQ_VERTEX) ) );
}


static void ProcessDepthBuffers( Asura_Direct3D_RenderTarget* xBlurredDepthBuffers[NUM_AMBIENT_OCCLUSION_LEVELS] )
{
	SCOPEDPIXEVENT((0xff0080ff, L"SSAO Depth Buffers Processing"));

	Asura_Direct3D_RenderTarget* pxRenderTarget = Asura_Direct3D::GetDeferredRenderingTarget();

	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_Render::CurrentStates.RequestDynamicLights( false );
	Asura_Render::CurrentStates.RequestZBufferEnabled( false );
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );

	// Smart-blur the depth buffer as a Z24S8 texture
	// The "smart" algorithm tries to represent the pixels that will cause occlusion, ignoring ones too far back to be part of the same object
	// the stencil value stores a confidence value, represent what percentage of the pixels that were sampled by the blur were actually used in creating this pixel's averaged (blurred) value.
	// when we read this back we use confidence to reduce occlusion, simulating occlusion from a smaller object (e.g. you get less occlusion from 1 far away pixel than a 32x32 box)

	g_pxSSAOEffect->Begin( false, 0 );

	//int uNumPasses = g_b16TapFilter?NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS:NUM_AMBIENT_OCCLUSION_LEVELS;
	int uNumPasses = NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS;
	for ( int i=0; i < uNumPasses; i++ )
	{
		//-> g_apxBlurredDepthBuffers[i]->Bind();
		Asura_Direct3D::ClearRenderTargets( 1 );
		Asura_Direct3D::D3DDevice->SetRenderTarget( 0, xBlurredDepthBuffers[i]->GetSurface() );

		//Asura_PS3_FragmentShaderCopy* pxDepthSmartBlurFP;

		if ( i == 0 )
		{
			// DepthDownsample
			g_pxSSAOEffect->BeginPass( 0 );

			//special case, read the actual depth buffer
			g_pxSSAOEffect->SetParameterTexture( g_xDepthBufferTexture, pxRenderTarget->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX ) );
		}
		else
		{
			// DepthBlur
			g_pxSSAOEffect->BeginPass( 1 );

			//read the texture from the previous step
			g_pxSSAOEffect->SetParameterTexture( g_xDepthBufferTexture, xBlurredDepthBuffers[i-1]->GetTexture() );
		}

		// set the threshold - perhaps this should change with each level? Not sure if it should get smaller (because 2D range is bigger), or bigger (because legitimate depth-difference caused by gentle slope is bigger)
		float fNear = Asura_Camera::GetNearClip();
		float fFar = Asura_Camera::GetFarClip();
		float fDepthProject1 = (fNear*fFar) / (fFar-fNear);	// imitate the calculation done when creating the projection matrix
		float fDepthProject2 = (fFar) / (fFar-fNear);

		float fFactor = 1;//i+1;//1<<i; // the acceptible error must increase with each level or the blurriest level will fail on steep planar surfaces, however this causes echoes
		float fThresholdFactor = fFactor * g_fDepthThreshold / fDepthProject1;

		int iSampleExp = i;
		if ( /*g_b16TapFilter && */ i > 0 ) iSampleExp = i*2-1; //0,1,3,5,... because 16-tap counts for 2 passes (first pass is only 4-tap)
		if ( i > 0 ) iSampleExp -= 1; //the first pass downsamples so halve all later passes
		float fSampleDistance = 0.5f * (float)(1<<iSampleExp);	// 0.5, 1/2, 4/2, 16/2,...
		fSampleDistance += (float)(g_iSampleDistance<<iSampleExp);

		float afParams[4] = { fSampleDistance, fThresholdFactor, (float)i, 0 };
		g_pxSSAOEffect->GetD3DEffect()->SetFloatArray( g_xParams, afParams, 4 );

		float afDepthProjection[4] = { fDepthProject1, fDepthProject2, fNear, fFar };
		// -> pxDepthSmartBlurFP->SetFloats( Asura_PS3_DepthDownsample_FP::FP_DepthProjection, 4, afDepthProjection );
		g_pxSSAOEffect->GetD3DEffect()->SetFloatArray( g_xDepthProjection, afDepthProjection, 4 );

		DrawFullscreenQuad();

		//Asura_Direct3D::D3DDevice->Clear( 0, NULL, D3DCLEAR_TARGET, 0xFF0000FF, 0, 0 );

		// Remove textures from samplers for textures that are later used as render targets
		g_pxSSAOEffect->GetD3DEffect()->SetTexture(g_xDepthBufferTexture, NULL );
		g_pxSSAOEffect->GetD3DEffect()->CommitChanges();

		g_pxSSAOEffect->EndPass();
	}

	g_pxSSAOEffect->End();
}

static void ComputeSamplePositions( float* pfSamplePositions )
{
	// compute the relative placement of the samples from the blurred buffers.
	// output an array of x,y,x,y,x,y with 1 sample position for each buffer.


	// the shader samples each buffer 4 times, rotating this sample point by 90 degrees for each sample
	// we pick coordinates to give this kind of arrangement:
	//
	//   +---+
	//   |	  +---+
	// +-+-+-+	  |
	// |   +-+-+-+
	// +---+   |
	//     +---+
	//
	// with the offsets each successively blurred buffer arranging it's boxes to cover the area beyond the previous one.
	// the buffers are blurred to to represent averages of 2x2, 8x8, 32x32, etc. pixels
	
	// A couple of modifications to that:
	//	- we downsample the first buffer by 1/2, so it's boxes can't be neatly arranged around our pixel. Instead the first level just uses 4 blocks arranged in a square,
	//	  which represents the smart-blurred average of 4x4 pixels, including the pixel we're shading! Fortunately there aren't any artefacts.
	//	- g_iSampleTightness slides those boxes against each other to tighten the iris in the middle, partially/totally overlapping the boxes of later levels with the earlier
	//	  levels. This is a touchy-feely tweak to help improve the visual quality (neither average perfectly represents all of it's constituent pixels, so overlaps might help)
	//	- g_iSampleDistance gives a result sparsely sampled, so rather than the first level representing 2x2 adjacent pixels it represents 4 pixels that were further apart
	//	  obviously this can cause artefacts, but within this function what we need to do is treat it as if it's been sampled from a larger area. Again this is a tweak to
	//	  trade the visual quality of larger blurs vs the artefacts, without incurring any performance hit (from doing more blur stages).

//notes on coordinates:
//
//  full-size (not downsampled) buffers:
// 	.5, 1.5, 5.5
// 	1.5, 6.5, 26.5
// 
// 	i = 0,1,2
// 	.5+[1<<(i-1)*2+[1<<(i-2)*2]]		 //[] - if it shifts right too we're good. (i.e. 1<<-1 = 0, 2<<-1*2 = 0)
// 	.5+1<<i*2+[2<<(i-1)*2+[2<<(i-2)*2]]
// 
// 	downsampled 1/2 (positions in full-screen pixels, for comparison):
// 	basepos rounded to nearest 2x2 corner, (-1,1)
// 	basepos rounded to nearest 2x2 corner, (-2,6)
// 	basepos rounded to nearest 2x2 corner, (-6,26)
// 
// 	1+[1<<(i-1)*2+[1<<(i-2)*2]]
// 	1<<i*2+[2<<(i-1)*2+[2<<(i-2)*2]]
// 
// 	iris:
// 	offset (-1,-1)*i*g_fIris. this screws it in, overlapping the inner layer
// 
// 	sample spacings:
// 	pretty sure this is just a scale applied to the offsets
// 	in the case of non-aligned ones we would add .5 afterwards

	// sample positions are in screen pixels, the shader re-scales to the pixels of the buffer.
	float fX = -1;
	float fY = 1;

	for ( int i=0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
	{
		pfSamplePositions[2*i+0] = fX*(1+g_iReadBackScale*g_iSampleDistance) - g_iSampleTightness*2*i; // *2 to force symmetry in 1/2 downsampled buffers (otherwise amb-occ gets some directionality)
		pfSamplePositions[2*i+1] = fY*(1+g_iReadBackScale*g_iSampleDistance) - g_iSampleTightness*2*i;
		
		fX -= 1<<(i*2);
		fY += (1<<((i+1)*2)) + (1<<(i*2));
	}
}

static void ApplyShadows( Asura_Direct3D_RenderTarget* xBlurredDepthBuffers[NUM_AMBIENT_OCCLUSION_LEVELS] )
{
	SCOPEDPIXEVENT((0xff0080ff, L"SSAO Shadow Calculation"));

	Asura_Direct3D_RenderTarget* pxRenderTarget = Asura_Direct3D::GetDeferredRenderingTarget();


	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestTranslucencyMode( (g_iSSAOMode == SSAO_OPAQUE)?(ASURA_TRANSLUCENCY_OFF):(ASURA_TRANSLUCENCY_COLOUR) );
	Asura_Render::CurrentStates.RequestDynamicLights( false );
	Asura_Render::CurrentStates.RequestZBufferEnabled( false );
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );

	g_pxSSAOEffect->Begin( true, 0 );

	D3DXMATRIX xViewMatrix;
	xViewMatrix = Asura_Direct3D::GetViewMatrix();	
	DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetMatrix( g_xViewMatrix, &xViewMatrix ) );

	/*
	this is required to perform final occlusion computation in world space
	{
		D3DXMATRIX xViewMatrix;
		D3DXMATRIX xViewMatrixInv;
		xViewMatrix = Asura_Direct3D::GetViewMatrix( );
		D3DXMatrixInverse( &xViewMatrixInv, NULL, &xViewMatrix );

		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetMatrix( g_xInvViewMatrix, &xViewMatrixInv ) );

		float z_fCameraTanHalfVertFOV	= Asura_Maths::Tangent( Asura_Camera::GetVerticalFieldOfView() * 0.5f );
		float z_fCameraTanHalfHorizFOV  = z_fCameraTanHalfVertFOV * Asura_Camera::GetXOverYAspectRatio();
		float z_fTwoOverScreenWidth		= 2.0f / Asura_Viewport::GetWidth();
		float z_fTwoOverScreenHeight	= 2.0f / Asura_Viewport::GetHeight();	

		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloat( g_xCameraTanHalfVertFOV, z_fCameraTanHalfVertFOV ) );
		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloat( g_xCameraTanHalfHorizFOV, z_fCameraTanHalfHorizFOV) );
		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloat( g_xTwoOverScreenWidth, z_fTwoOverScreenWidth ) );
		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloat( g_xTwoOverScreenHeight, z_fTwoOverScreenHeight ) );
	}
	*/

	float afParams[4];
	float fCameraTanHalfVertFOV	= Asura_Maths::Tangent( Asura_Camera::GetVerticalFieldOfView() * 0.5f );
	// <-> float fCameraTanHalfHorizFOV= fCameraTanHalfVertFOV * Asura_PS3_Graphics::GetScreenAspectRatio();// Asura_Camera::GetXOverYAspectRatio();
	// TODO: ask why not GetXOverYAspectRatio? or Asura_Camera::GetPhysicalScreenAspectRatio()?
	float fCameraTanHalfHorizFOV= fCameraTanHalfVertFOV * Asura_Camera::GetXOverYAspectRatio();
	afParams[0] = fCameraTanHalfHorizFOV;
	afParams[1] = fCameraTanHalfVertFOV;
	// <-> pxAmbientOcclusionFP->SetFloats( Asura_PS3_AmbientOcclusion_FP::FP_CameraTanHalfFOV, 2, &afParams[0] );
	DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloatArray( g_xCameraTanHalfFOV, afParams, 2 ) );
	
	// !!!!!! <-> !!!!!! Asura_PS3_MaterialResponse_System::SetDepthParameters( pxAmbientOcclusionFP, Asura_PS3_AmbientOcclusion_FP::FP_DepthAdjustment ); !!

	afParams[0] = 2.0f / Asura_Viewport::GetWidth();
	afParams[1] = 2.0f / Asura_Viewport::GetHeight();	
	// <-> pxAmbientOcclusionFP->SetFloats( Asura_PS3_AmbientOcclusion_FP::FP_TwoOverScreen, 2, &afParams[0] );
	DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloatArray( g_xTwoOverScreen, afParams, 2 ) );
	
	afParams[0] = g_fContrast;
	afParams[1] = g_fBrightness;
	// <-> pxAmbientOcclusionFP->SetFloats( Asura_PS3_AmbientOcclusion_FP::FP_BrightnessContrast, 2, &afParams[0] );
	DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloatArray( g_xBrightnessContrast, afParams, 2 ) );
	
	afParams[0] = g_fDistanceThreshold;
	// <-> pxAmbientOcclusionFP->SetFloats( Asura_PS3_AmbientOcclusion_FP::FP_DistanceFade, 1, &afParams[0] );
	DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloatArray( g_xDistanceFade, afParams, 1 ) );

	float afSamplePositions[8];
	ComputeSamplePositions(afSamplePositions);
	// <-> pxAmbientOcclusionFP->SetArray( Asura_PS3_AmbientOcclusion_FP::FP_SamplePositions, 2, 4, &afSamplePositions[0] );
	DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloatArray( g_xSamplePositions, afSamplePositions, 8 ) );

	//////////////////////////////////////////////////////////////////////////
	// use this in the ProcessDepthBuffers too, instead of the existing stuff, to 
	// make it more readable
	float f = Asura_Camera::GetFarClip(), n = Asura_Camera::GetNearClip();
	const float afDepthAdjustment[2] = { (f*n)/(f-n), f/(f-n) };
	DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetFloatArray( g_xDepthAdjustment, afDepthAdjustment, 2 ) );
	//////////////////////////////////////////////////////////////////////////

	// <-> pxAmbientOcclusionFP->SetTexture( Asura_PS3_AmbientOcclusion_FP::FP_NormalMaterialBufferSampler, Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->GetRTAsTexture( ASURA_DEFERRED_RENDER_TARGET_NORMAL_MATERIAL ), false );
 	// <-> pxAmbientOcclusionFP->SetTexture( Asura_PS3_AmbientOcclusion_FP::FP_DepthBufferSampler, Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->GetZBufferAsTexture(), false );
	DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetTexture(g_xDeferredRT_NI_Texture, pxRenderTarget->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX ) ) );

	for ( int i=0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
	{
		int iTex = i;
		//if ( !g_b16TapFilter ) iTex *= 2;
	
		// <-> pxAmbientOcclusionFP->SetTexture( Asura_PS3_AmbientOcclusion_FP::FP_DepthBlurSamplers+i, g_apxBlurredDepthBuffers[iTex]->GetRTAsTexture(0), false );
		// Asura_Char szBuff[1024]; Asura_PrintfToBfr( szBuff, "g_axDepthDownsampleTexture%d", sizeof(szBuff), true, i );
		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetTexture( g_axDepthDownsampleTexture[i], xBlurredDepthBuffers[iTex]->GetTexture() ) );
	}

	Asura_Direct3D::ClearRenderTargets( 1 );

	Asura_Direct3D::D3DDevice->SetRenderTarget( 0, pxRenderTarget->GetSurface( ASURA_DIRECT3D_DEFFEDRENDERTARGET_FRONT_INDEX ) );

	g_pxSSAOEffect->BeginPass(2);

	DrawFullscreenQuad();

	// Remove textures from samplers for textures that are later used as render targets
	for ( int i=0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
	{
		//Asura_Char szBuff[1024]; Asura_PrintfToBfr( szBuff, "g_axDepthDownsampleTexture%d", sizeof(szBuff), true, i );
		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetTexture( g_axDepthDownsampleTexture[i], NULL ) );
	}
	DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetTexture(g_xDeferredRT_NI_Texture, NULL ) );

	g_pxSSAOEffect->GetD3DEffect()->CommitChanges();
	g_pxSSAOEffect->EndPass();
	g_pxSSAOEffect->End();
}


void Asura_Direct3D_AmbientOcclusion::Render()
{
	if ( g_iSSAOMode == SSAO_OFF )
	{
		SCOPEDPIXEVENT((0xff0080ff, L"SSAO (switched off)"));
		return;
	}
	SCOPEDPIXEVENT((0xff0080ff, L"SSAO"));

	if( !g_bEffectVariableHandlesInitialised )
	{
		InitialiseEffectVariableHandles();
		g_bEffectVariableHandlesInitialised = true;
	}

	u_int uScreenWidth	= Asura_Direct3D::GetPresentParameters().BackBufferWidth;
	u_int uScreenHeight	= Asura_Direct3D::GetPresentParameters().BackBufferHeight;

	Asura_Direct3D_StoreRenderTargets xStoreRenderTargets;
	xStoreRenderTargets.StoreRenderTargets();

	Asura_Direct3D_RenderTarget* xBlurredDepthBuffers[NUM_AMBIENT_OCCLUSION_LEVELS];

	// Create/reuse render target
	{
		int w = uScreenWidth >> 1;
		int h = uScreenHeight >> 1;

		for ( int i=0; i < NUM_AMBIENT_OCCLUSION_LEVELS; i++ )
		{
			xBlurredDepthBuffers[i] = Asura_Direct3D_RenderTargetManager::FindUnusedRenderTarget( (float)w, (float)h, D3DFMT_A8R8G8B8, D3DFMT_UNKNOWN, false );
		}
	}

	DWORD auMinFilter[3], auMagFilter[3], auMipFilter[3];
	// Preserve initial filtering state
	for( u_int u = 0; u < 3; ++u )
	{
		Asura_Direct3D::D3DDevice->GetSamplerState( u, D3DSAMP_MINFILTER, &auMinFilter[u] );
		Asura_Direct3D::D3DDevice->GetSamplerState( u, D3DSAMP_MAGFILTER, &auMagFilter[u] );
		Asura_Direct3D::D3DDevice->GetSamplerState( u, D3DSAMP_MIPFILTER, &auMipFilter[u] );
	}

	ProcessDepthBuffers( xBlurredDepthBuffers );

	ApplyShadows( xBlurredDepthBuffers );

	// used for debugging (to enable, enable it in shaders too)
	/*
	if( 0 )
	{
		SCOPEDPIXEVENT((0xff0080ff, L"SSAO debug rendering"));

		float xFltArr[4];

		Asura_Direct3D_RenderTarget* pxRenderTarget = Asura_Direct3D::GetDeferredRenderingTarget();
		Direct3D_Surface* pxRTFrontSurface = pxRenderTarget->GetSurface( ASURA_DIRECT3D_DEFFEDRENDERTARGET_FRONT_INDEX );

		Asura_Direct3D::ClearRenderTargets( 1 );
		Asura_Direct3D::D3DDevice->SetRenderTarget( 0, pxRTFrontSurface );

		for ( int i=0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
		{
			int iTex = i;
			//if ( !g_b16TapFilter ) iTex *= 2;

			// <-> pxAmbientOcclusionFP->SetTexture( Asura_PS3_AmbientOcclusion_FP::FP_DepthBlurSamplers+i, g_apxBlurredDepthBuffers[iTex]->GetRTAsTexture(0), false );
			//Asura_Char szBuff[1024]; Asura_PrintfToBfr( szBuff, "g_axDepthDownsampleTexture%d", sizeof(szBuff), true, i );
			DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetTexture( g_axDepthDownsampleTexture[i], xBlurredDepthBuffers[iTex]->GetTexture() ) );
		}

		//g_pxSSAOEffect->Begin(true, SHADER_JUSTCOPY) ;
		//if( !g_pxSSAOEffect->IsD3DEffectValid() ) return;
		g_pxSSAOEffect->Begin( false, 0 );

		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetTexture( "g_xSourceTexture", xBlurredDepthBuffers[1]->GetTexture() ) );
		//DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetTexture("g_xSourceTexture", pxRenderTarget->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX ) ) );

		DX_VERIFY( g_pxSSAOEffect->BeginPass(3) );

		DrawFullscreenQuad();

		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetTexture( "g_xSourceTexture", NULL ) );
		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->CommitChanges() );

		DX_VERIFY( g_pxSSAOEffect->EndPass() );

		for ( int i=0; i < NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS; i++ )
		{
			//Asura_Char szBuff[1024]; Asura_PrintfToBfr( szBuff, "g_axDepthDownsampleTexture%d", sizeof(szBuff), true, i );
			DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetTexture( g_axDepthDownsampleTexture[i], NULL ) );
		}
		DX_VERIFY( g_pxSSAOEffect->GetD3DEffect()->SetTexture(g_xDeferredRT_NI_Texture, NULL ) );
		g_pxSSAOEffect->GetD3DEffect()->CommitChanges();

		g_pxSSAOEffect->End();
	}
	*/

	for ( int i=0; i < NUM_AMBIENT_OCCLUSION_LEVELS; i++ )
	{
		xBlurredDepthBuffers[i]->MarkAsUnused();
	}

	xStoreRenderTargets.RestoreRenderTargets();

	Asura_Direct3D_Effects_System::FlushEffects();

	// Restore initial filtering state
	for( u_int u = 0; u < 3; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_MINFILTER, auMinFilter[u] );
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_MAGFILTER, auMagFilter[u] );
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_MIPFILTER, auMipFilter[u] );
	}
}

#endif // #if defined(ASURA_USE_AMBIENT_OCCLUSION)
