/**
*
* \file Asura_Direct3D_Caps.cpp
*
* Created by Kevin Lea - 16/07/2002 14:13:31
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_Direct3D.h"
#include "Debugging/Asura_Assert.h"


bool	Asura_Direct3D_Caps::HardwarePixelShaderSupported;
bool	Asura_Direct3D_Caps::HardwareVertexShaderSupported;
bool	Asura_Direct3D_Caps::HardwareTnLSupported;
int		Asura_Direct3D_Caps::MaxNumberOfTexturesPerPass;
u_int	Asura_Direct3D_Caps::s_uMaxNumberOfTextureStages = 0;
u_int	Asura_Direct3D_Caps::s_uMaxAnisotropy;
u_int	Asura_Direct3D_Caps::s_uNumberOfVertexShaderConstants;
float	Asura_Direct3D_Caps::s_fVertexShaderVersion;
float	Asura_Direct3D_Caps::s_fPixelShaderVersion;
int		Asura_Direct3D_Caps::DepthTextureBitDepth = 0;
bool	Asura_Direct3D_Caps::s_bIsNonPow2Conditional = false;
bool	Asura_Direct3D_Caps::s_bTextureNonPOW2 = false;
u_int	Asura_Direct3D_Caps::s_uNumberOfRenderTargets = 0;
bool	Asura_Direct3D_Caps::s_bMRTSupportsPostPSBlending = false;
bool	Asura_Direct3D_Caps::s_bMRTSupportsIndependentBitDepths = false;



void Asura_Direct3D_Caps::QueryCaps(void)
{
	Asura_Assert(Asura_Direct3D::D3DDevice,"Direct3D not initialised?");


	Direct3D_Caps caps;
	Asura_Direct3D::D3DDevice->GetDeviceCaps(&caps);  


	s_fVertexShaderVersion = static_cast<float>(D3DSHADER_VERSION_MINOR( caps.VertexShaderVersion ));
	s_fVertexShaderVersion *= .1f;
	s_fVertexShaderVersion += D3DSHADER_VERSION_MAJOR( caps.VertexShaderVersion );

	s_fPixelShaderVersion = static_cast<float>(D3DSHADER_VERSION_MINOR( caps.PixelShaderVersion ));
	s_fPixelShaderVersion *= .1f;
	s_fPixelShaderVersion += D3DSHADER_VERSION_MAJOR( caps.PixelShaderVersion );
	

	/// Kevin Lea - 16/07/2002 14:29:42 - Check for Vertex Shader support
	if( D3DSHADER_VERSION_MAJOR( caps.VertexShaderVersion ) < 1 )
	{
		HardwareVertexShaderSupported = false;
	}
	else
	{
		HardwareVertexShaderSupported = true;
	}

	/// Kevin Lea - 16/07/2002 12:30:09 - Check for Pixel Shader support
	/// Anything above ps.1.1 is fine - basically the level of the drivers
	/// for the first GeForce3s. Extra info (probably just keeping major and 
	/// minor around) will be required if we want different effects dependent
	/// on availability of higher versions (e.g. ps.1.4).
	{
			
		// skip version 1.0 (does it even exist?)
		HardwarePixelShaderSupported = (s_fPixelShaderVersion > 1.0f);
	}

	/// Kevin Lea - 16/07/2002 14:37:04 - texture passes stuff
	MaxNumberOfTexturesPerPass = caps.MaxSimultaneousTextures;
	s_uMaxNumberOfTextureStages = caps.MaxTextureBlendStages;
	s_uNumberOfRenderTargets = caps.NumSimultaneousRTs;

	HardwareTnLSupported = (caps.DevCaps&D3DDEVCAPS_HWTRANSFORMANDLIGHT)!=0;
	

	// KJL - 10/09/2003 10:50:32 - anisotropic texture filtering?
	
	// KJL - 16/09/2003 12:26:02 - discovered that some nVidia cards don't report the filtering caps, so I'll just use MaxAnisotropy
	//if(	(caps.TextureFilterCaps & D3DPTFILTERCAPS_MINFANISOTROPIC) 
	//	  &&(caps.TextureFilterCaps & D3DPTFILTERCAPS_MAGFANISOTROPIC)
	//	  &&(caps.MaxAnisotropy > 1) )

	if ( caps.MaxAnisotropy > 1 )
	{
		s_uMaxAnisotropy = caps.MaxAnisotropy; // max number of samples
	}
	else
	{
		s_uMaxAnisotropy = 0; // no anisotropic filtering
	}
	
	s_uNumberOfVertexShaderConstants = caps.MaxVertexShaderConst;	

	D3DDISPLAYMODE xDisplayMode;
	Asura_Direct3D::D3DDevice->GetDisplayMode( 0, &xDisplayMode );

	//  RJF:  (15/11/2004 16:45:00):  Test for depth texture support
	HRESULT hr;
	D3DDEVICE_CREATION_PARAMETERS d3dcp;
	Asura_Direct3D::D3DDevice->GetCreationParameters(&d3dcp);
	hr = Asura_Direct3D::D3DObject->CheckDeviceFormat(	d3dcp.AdapterOrdinal,
														d3dcp.DeviceType,
														xDisplayMode.Format,
														D3DUSAGE_DEPTHSTENCIL,
														D3DRTYPE_TEXTURE,
														D3DFMT_D24S8);
	if(SUCCEEDED(hr))
	{
		DepthTextureBitDepth = 24;
	}
	else
	{
		// Try 16-bit
		hr = Asura_Direct3D::D3DObject->CheckDeviceFormat(	d3dcp.AdapterOrdinal,
															d3dcp.DeviceType,
															xDisplayMode.Format,
															D3DUSAGE_DEPTHSTENCIL,
															D3DRTYPE_TEXTURE,
															D3DFMT_D24S8);
		if(SUCCEEDED(hr))
		{
			DepthTextureBitDepth = 16;
		}
		else
		{
			DepthTextureBitDepth = 0; // no support
		}
	}

	// RJF - 05 September 2005 - nonpow2conditional
	s_bIsNonPow2Conditional = (caps.TextureCaps & D3DPTEXTURECAPS_NONPOW2CONDITIONAL) != 0;
	s_bTextureNonPOW2 = (caps.TextureCaps & D3DPTEXTURECAPS_POW2) == 0;

	// RJF - 15 August 2006 - Multiple render target stuff
	s_bMRTSupportsPostPSBlending = (caps.PrimitiveMiscCaps & D3DPMISCCAPS_MRTPOSTPIXELSHADERBLENDING) != 0;
	s_bMRTSupportsIndependentBitDepths = (caps.PrimitiveMiscCaps & D3DPMISCCAPS_MRTINDEPENDENTBITDEPTHS) != 0;
}
