
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Direct3D_Effect.h
// Author : Andy Tate
// Date   : 11-May-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __ASURA_DIRECT3D_EFFECT_H__
#define __ASURA_DIRECT3D_EFFECT_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_HashTree.h"
#include "Core/Asura_String.h"
#include "Rendering/Asura_Direct3D.h"

// JAT 22/02/07 on next-gen want to support loading effects in all builds, current-gen only want this in tools builds
#if defined( ASURA_LOAD_EFFECT_FILES )
#undef ASURA_LOAD_EFFECT_FILES 
#endif // #if defined( ASURA_LOAD_EFFECT_FILES )

#if defined( ASURA_TARGET_SM3)
#define ASURA_LOAD_EFFECT_FILES 1
#else // #if defined( ASURA_TARGET_SM3 ) || defined( ASURA_TOOLS )
#define ASURA_LOAD_EFFECT_FILES 0
#endif // #if defined( ASURA_TARGET_SM3 ) || defined( ASURA_TOOLS )

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////
class Asura_Direct3D_Effect
{
public:
	// Public enumerated types...


	// Public typedefs...

	typedef bool CommitChangesPerPassFn (const u_int uPass);


	// Public functions...

	Asura_Direct3D_Effect  (const void* pvData, const u_int uDataSize );
	// JAT 22/02/07 only want this function in tools builds on current-gen (but on all builds for next-gen)
#if ASURA_LOAD_EFFECT_FILES == 1
	Asura_Direct3D_Effect  (const char* szEffectName, const char* szFilename );
#endif	// #if ASURA_LOAD_EFFECT_FILES == 1
	~Asura_Direct3D_Effect ();

#if ASURA_LOAD_EFFECT_FILES == 1 && !defined( ASURA_NO_LOAD_EFFECTS_FILES_FROM_FILES)
	// Tools functions...
	void					Tools_Reload(void);

	const Asura_String&		GetFilename() { return m_xFilename; }

	// JAT 17/02/07 set a path for loading shaders from (in tools)
	static void SetShadersPath( const char* const szShadersPath );
	static const char* const GetShadersPath( void );

#if defined(ASURA_PROJECT_SHADER_PATH)
	// DJG 20/03/2007 : Allows the setting of a project specific shader path to be used
	// when tools builds load shaders from files rather than resources.
	static void SetProjectShadersPath( const char* const szProjectShadersPath );
	static const char* const GetProjectShadersPath( void );
#endif // defined(ASURA_PROJECT_SHADER_PATH)

#endif // ASURA_LOAD_EFFECT_FILES == 1 && !defined( ASURA_NO_LOAD_EFFECTS_FILES_FROM_FILES)

	void ResetParameterTree();
	bool PreReset();

	// Parameter set functions...
			void					SetParameter					(const D3DXHANDLE xHandle, const bool bValue);
			void					SetParameter					(const D3DXHANDLE xHandle, const int iValue);
			void					SetParameter					(const D3DXHANDLE xHandle, const float fValue);
			void					SetParameter					(const D3DXHANDLE xHandle, const D3DXMATRIX& xValue);
			void					SetParameter					(const D3DXHANDLE xHandle, const D3DXVECTOR4& xValue);
			void					SetParameter					(const D3DXHANDLE xHandle, const void* pData, const u_int uSize);
			void					SetParameter					(const D3DXHANDLE xHandle, Direct3D_BaseTexture* const pxTexture);
			void					SetParameterTexture				(const D3DXHANDLE xHandle, Direct3D_BaseTexture* const pxTexture);

			void					SetParameterByName				(const char* szName, const bool bValue);
			void					SetParameterByName				(const char* szName, const int iValue);
			void					SetParameterByName				(const char* szName, const float fValue);
			void					SetParameterByName				(const char* szName, const D3DXMATRIX& xValue);
			void					SetParameterByName				(const char* szName, const D3DXVECTOR4& xValue);
			void					SetParameterByName				(const char* szName, const void* pData, const u_int uSize);
			void					SetParameterByName				(const char* szName, Direct3D_BaseTexture* const pxTexture);
			void					SetParameterTextureByName		(const char* szName, Direct3D_BaseTexture* const pxTexture);

			void					SetParameterByHash				(const char* szName, Asura_Hash_ID uHashID, const bool bValue);
			void					SetParameterByHash				(const char* szName, Asura_Hash_ID uHashID, const int iValue);
			void					SetParameterByHash				(const char* szName, Asura_Hash_ID uHashID, const float fValue);
			void					SetParameterByHash				(const char* szName, Asura_Hash_ID uHashID, const D3DXMATRIX& xValue);
			void					SetParameterByHash				(const char* szName, Asura_Hash_ID uHashID, const D3DXVECTOR4& xValue);
			void					SetParameterByHash				(const char* szName, Asura_Hash_ID uHashID, const void* pData, const u_int uSize);
			void					SetParameterByHash				(const char* szName, Asura_Hash_ID uHashID, Direct3D_BaseTexture* const pxTexture);
			void					SetParameterTextureByHash		(const char* szName, Asura_Hash_ID uHashID, Direct3D_BaseTexture* const pxTexture);

	// Render functions...
			void					Begin							(const bool bSaveState = false, const u_int uTechnique = 0);
			void					BeginPass						(const u_int uPass);

			void					SetTextureRenderStates			(const int iMaterial);
			void					RenderAllIndexedPrimitivePasses	(const D3DPRIMITIVETYPE ePrimitiveType, const u_int uStartVertex, const u_int uNumVertices, const u_int uStartIndex, const u_int uNumPrimitives, CommitChangesPerPassFn* pxCommitChangesPerPassFn, const u_int uBaseVertexIndex = 0);
			void					RenderAllPrimitivePasses	(const D3DPRIMITIVETYPE ePrimitiveType, const u_int uStartVertex, const u_int uNumPrimitives, CommitChangesPerPassFn* pxCommitChangesPerPassFn = NULL);

			void					EndPass							(void);
			void					End								(void);

	// Accessor functions...
		 	Direct3D_Effect*		GetD3DEffect					(void)													{ return (m_pxD3DEffect); }
		 	const Direct3D_Effect*	GetD3DEffect					(void) const											{ return (m_pxD3DEffect); }

			bool					IsD3DEffectValid				(void) const											{ return (NULL != m_pxD3DEffect); }

// JAT 22/02/07 on next-gen want this in all builds, current-gen only want this in tools builds
#if ASURA_LOAD_EFFECT_FILES == 1
	// Callback setting functions
	// Some "system" shaders (e.g. smooth skin renderer) need to be informed when their D3DShader(s) are loaded/unloaded so that they can
	// capture and release parameter handles
			typedef void (*EffectEventCallbackType)(Asura_Direct3D_Effect*);
			void					SetOnLoadCallback(EffectEventCallbackType pfnCallback) { m_pfnOnLoadCallback = pfnCallback; }
			void					SetOnUnloadCallback(EffectEventCallbackType pfnCallback) { m_pfnOnUnloadCallback = pfnCallback; }
#endif // #if ASURA_LOAD_EFFECT_FILES == 1

		
	// Static effect-pool functions...
	static	void					CreateEffectPool				(void);
	static	void					DestroyEffectPool				(void);


protected:
	// Protected variables...


	// Protected functions...


private:
	// Private variables...

	// The Direct3D effect we're to create...
			Direct3D_Effect*		m_pxD3DEffect;

	// Array of technique handles...
			D3DXHANDLE*				m_pxTechniqueHandles;

	// The number of render passes...
			u_int					m_uNumPasses;
			u_int					m_uNumTechniques;

	// The flags describing the resources used by this effect...
			u_int					m_uTextureTypeFlags;
			u_int					m_uVertexShaderFlags;

	// For next-gen need this data in all builds to load the effect as a resource
#if ASURA_LOAD_EFFECT_FILES == 1
			const char*				m_szName;
			Asura_String			m_xFilename;
#endif	// #if ASURA_LOAD_EFFECT_FILES == 1

	// MH: tree of handles to parameters organised by hash of name.
	Asura_TemplateHashTree<D3DXHANDLE> m_xParameterHandleTree;

	// AJG - 07/06/05 - I really don't like this as a way of getting the detail map, we already store this in two other platform files.
	//					However, storing it again means I'm not going to impact wat anyone else is doing.
	//					At some point this should have a nice little one-for-all location to live in.
	static int s_iDetailMapTextureIndex;

	// Static occurrance of the Direct3D effect-pool...
	static	Direct3D_EffectPool*	s_pxD3D_EffectPool;

// JAT 22/02/07 on next-gen want this in all builds, current-gen only want this in tools builds
#if ASURA_LOAD_EFFECT_FILES == 1
	// Callbacks
	EffectEventCallbackType m_pfnOnLoadCallback;
	EffectEventCallbackType m_pfnOnUnloadCallback;
#endif // #if ASURA_LOAD_EFFECT_FILES == 1

#if !defined( ASURA_NO_LOAD_EFFECTS_FILES_FROM_FILES)
	static Asura_String s_xShadersPath;

#if defined(ASURA_PROJECT_SHADER_PATH)
	static Asura_String s_xProjectShadersPath;
#endif // defined(ASURA_PROJECT_SHADER_PATH)

#endif // !defined( ASURA_NO_LOAD_EFFECTS_FILES_FROM_FILES)
	static bool s_bLoadShadersFromDisc;

	// Private functions...

	Asura_Direct3D_Effect  (void);
	Asura_Direct3D_Effect  (const Asura_Direct3D_Effect& xOther);

			Asura_Direct3D_Effect&	operator=						(const Asura_Direct3D_Effect& xOther);

	// Effect creations/deletion functions...
	
	// JAT 22/02/07 only want this function in tools builds on current-gen (but on all builds for next-gen)
#if ASURA_LOAD_EFFECT_FILES == 1
	void LoadEffectFromFile( void );
#endif // #if ASURA_LOAD_EFFECT_FILES == 1

	void ReleaseEffect( void );

	void QueryTechniqueHandles( void );

};   // End of class 'Asura_Direct3D_Effect'.


#endif   // ifndef __ASURA_DIRECT3D_EFFECT_H__


/// End of file. ///
