//
// Asura_Direct3D_EnvironmentRenderer.cpp
//
// Created by KJFL - 04 May 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#if defined(ASURA_TARGET_SM3)

// This file's header.
#include "Rendering/Asura_Direct3D_EnvironmentRenderer.h"

#include "Animation/Asura_TextureAnimation.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_HeapArray.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"
#include "Debugging/Asura_DebugText.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Debugging/Asura_CPUProfile.h"
#include "Debugging/Asura_LogFile.h"
#include "Interface/Asura_CommandConsole.h"
#include "Light/Asura_GI_Solution.h"
#include "Light/Asura_Photon.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_DeferredShading.h"
#include "Rendering/Asura_Environment_Renderer.h"
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_DIPCache.h"
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Direct3D_EffectParameterBlock.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_IndexBuffer_Dynamic.h"
#include "Rendering/Asura_Direct3D_MaterialResponse.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_Fog.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Platform_RenderObjectHierarchy.h"
#include "Rendering/Asura_PlatformRenderObject.h"
#include "SpecialFX/Asura_SpecialFX.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "Visibility/Asura_Visibility_System.h"

#if !defined( ASURA_D3D_ENABLE_D3D_OK) || ASURA_D3D_ENABLE_D3D_OK < 1
#pragma message( __FILE__": Please define ASURA_D3D_ENABLE_D3D_OK to be 1 or 2")
#endif

// these headers are required on Tools builds, where data is constructed on the fly
#if defined ASURA_TOOLS
	#include "Tools_System.h"
	#include "Materials/Tools_MaterialResponse_System.h"
	#include "MeshProcessing/Tools_EnvironmentProcessor.h"
	#include "Misc/Tools_Render.h"
	#include "Misc/Tools_Render_Options.h"
	#include "Misc/Tools_Timed_Messages.h"
	#include "PC/Tools_PC_Export.h"
	#include "Rendering/Tools_RenderModules.h"
	#include "Visibility/Tools_Visibility_System.h"
#endif

// cube face definitions, listing vertices counter-clockwise
static const u_short g_ausCubeFaceVertices[] = 
{
	0,1,2,
	1,3,2,
	
	1,5,3,
	5,7,3,
	
	4,6,7,
	7,5,4,
	
	0,2,6,
	6,4,0,
	
	2,3,7,
	7,6,2,

	0,5,1,
	0,4,5
};

class Asura_Direct3D_AABB_Vertex
{
	public:
		float afPosition[3];
};

static const Asura_Direct3D_AABB_Vertex g_axCubeVertices[] = 
{
	{ 0.0f, 0.0f, 0.0f },
	{ 0.0f, 0.0f, 1.0f },
	{ 0.0f, 1.0f, 0.0f },
	{ 0.0f, 1.0f, 1.0f },
	{ 1.0f, 0.0f, 0.0f },
	{ 1.0f, 0.0f, 1.0f },
	{ 1.0f, 1.0f, 0.0f },
	{ 1.0f, 1.0f, 1.0f }
};

// This is externed through the header for use in Tools mesh processing
extern const Direct3D_VertexElement axASURA_D3D_ENV_VERTEX_DECLARATION[] =
{
	// Vertex position and colour...
	{0,	0,		D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0},
	{0,	12,		D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_COLOR,		0},

	// Vertex normal, binormal and tangent...
	{0,	16,		D3DDECLTYPE_SHORT4N,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_NORMAL,	0},
	{0,	24,		D3DDECLTYPE_SHORT4N,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TANGENT,	0},
	{0,	32,		D3DDECLTYPE_SHORT4N,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TANGENT,	1},

 	// Texture co-ordinates - only dual-pass allowed..
 	{0,	40,		D3DDECLTYPE_FLOAT2,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	0},
 	{0,	48,		D3DDECLTYPE_FLOAT2,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	1},
 
 	// Diffuse spherical harmonics, R,G & B...
 	{0,	56,		D3DDECLTYPE_FLOAT4,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	2},
 	{0,	72,		D3DDECLTYPE_FLOAT4,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	3},
 	{0,	88,		D3DDECLTYPE_FLOAT4,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	4},

	// That's all folks...
	D3DDECL_END()
};

Asura_Direct3D_Effect*		 					Asura_Direct3D_EnvironmentRenderer::s_pxEnvironmentEffect	= NULL;
Asura_Direct3D_Effect*		 					Asura_Direct3D_EnvironmentRenderer::s_pxEnvShadowEffect	= NULL;

Direct3D_VertexBuffer*							Asura_Direct3D_EnvironmentRenderer::s_pxAABBVertexBuffer;
Direct3D_IndexBuffer*							Asura_Direct3D_EnvironmentRenderer::s_pxAABBIndexBuffer;


u_int											Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfStrips;
Asura_Direct3D_EnvironmentRenderer_Strip*		Asura_Direct3D_EnvironmentRenderer::s_pxStrips;

u_int											Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfVertexBuffers;
Direct3D_VertexBuffer**							Asura_Direct3D_EnvironmentRenderer::s_ppxVertexBuffer = NULL;

u_int											Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfIndexBuffers;
Direct3D_IndexBuffer**							Asura_Direct3D_EnvironmentRenderer::s_ppxIndexBuffer = NULL;

// DLH - Data for Dynmic Index Buffer Rendering
u_int											Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfIndices = 0;
uint16*											Asura_Direct3D_EnvironmentRenderer::s_pusIndexData = NULL;
uint16**										Asura_Direct3D_EnvironmentRenderer::s_ppusIndexData = NULL;


#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
Asura_Direct3D_MaterialResponse_ShaderData	Asura_Direct3D_EnvironmentRenderer::s_xCacheMaterialShaderData_Base;
Asura_Direct3D_MaterialResponse_ShaderData_Env	Asura_Direct3D_EnvironmentRenderer::s_xCacheMaterialShaderData_Env;
Asura_Direct3D_MaterialResponse_ShaderData_Parallax	Asura_Direct3D_EnvironmentRenderer::s_xCacheMaterialShaderData_Parallax;
Asura_Direct3D_EffectParameterBlock*			Asura_Direct3D_EnvironmentRenderer::s_pxParamBlock = NULL;
#endif //#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM

int												Asura_Direct3D_EnvironmentRenderer::s_iCurrentlyUsedMaterial;

bool											Asura_Direct3D_EnvironmentRenderer::s_bUsingPreloadedData;
bool											Asura_Direct3D_EnvironmentRenderer::s_bVertexAndIndexBuffersCreatedInternally;
bool											Asura_Direct3D_EnvironmentRenderer::s_bUseTriLists;

bool											Asura_Direct3D_EnvironmentRenderer::s_bDisplayOcclusionText	= false;
bool											Asura_Direct3D_EnvironmentRenderer::s_bXRayMode			= false;

#ifdef ASURA_TOOLS
bool											Asura_Direct3D_EnvironmentRenderer::s_bStripsValid		= false;
bool											Asura_Direct3D_EnvironmentRenderer::s_bDisableDualPass	= false;
#endif

Asura_Direct3D_Effect*		 					Asura_Direct3D_EnvironmentRenderer::s_pxWaterEffect	= NULL;
Asura_Direct3D_Water_ParamBlockData				Asura_Direct3D_EnvironmentRenderer::s_xWaterParamData;
Asura_Direct3D_EffectParameterBlock*			Asura_Direct3D_EnvironmentRenderer::s_pxWaterParamBlock = NULL;
const Asura_Direct3D_EnvironmentRenderer_Strip*	Asura_Direct3D_EnvironmentRenderer::s_pxQueuedWaterStrips[ASURA_MAX_VISIBLE_WATER_STRIPS];
int												Asura_Direct3D_EnvironmentRenderer::s_iNumQueuedWaterStrips = 0;


static u_int									g_uStripsRendered;
static u_int									g_uOcclusionQueries;

static Direct3D_VertexDeclaration*				g_pxStandardVertexDecl = 0;
static Direct3D_VertexDeclaration*				g_pxAABBVertexDecl = 0;

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
static bool s_bAsura_Direct3D_EnvironmentRenderer_EnablePixels = true;
static bool s_bAsura_Direct3D_EnvironmentRenderer_EnableDrawCall = true;
static bool s_bAsura_Direct3D_EnvironmentRenderer_DrawOnlyOnePrimitive = false;
static bool s_bAsura_Direct3D_EnvironmentRenderer_OutputLog = false;

#define ASURA_DIRECT3D_ENVIRONMENTRENDERER_OUTPUTLOG_FILENAME "Environment.txt"
#endif


void Asura_Direct3D_EnvironmentRenderer::Initialise( void )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	Asura_CommandConsole::AddVar( L"Render.Environment.DrawCalls", &s_bAsura_Direct3D_EnvironmentRenderer_EnableDrawCall, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);	
	Asura_CommandConsole::AddVar( L"Render.Environment.DrawPixels", &s_bAsura_Direct3D_EnvironmentRenderer_EnablePixels, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);	
	Asura_CommandConsole::AddVar( L"Render.Environment.DrawOnlyOnePrimitive", &s_bAsura_Direct3D_EnvironmentRenderer_DrawOnlyOnePrimitive, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);	
	Asura_CommandConsole::AddVar( L"Render.Environment.OutputLog", &s_bAsura_Direct3D_EnvironmentRenderer_OutputLog, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);	
#endif

	s_uTotalNumberOfStrips = 0;
	s_pxStrips = 0;

	s_uTotalNumberOfVertexBuffers	= 0;
	s_ppxVertexBuffer				= NULL;

	s_uTotalNumberOfIndexBuffers	= 0;
	s_ppxIndexBuffer				= NULL;
	
	s_pusIndexData					= NULL;
	s_ppusIndexData					= NULL;
	
	// by default we always use TriLists
	s_bUseTriLists = true;

	s_bUsingPreloadedData = false;
	s_bVertexAndIndexBuffersCreatedInternally = false;

#ifdef ASURA_TOOLS
	s_bStripsValid = false;
#endif

	Direct3D_Device* pxD3DDevice = Asura_Direct3D::D3DDevice;
	if( NULL != pxD3DDevice)
	{
		s_pxEnvironmentEffect = Asura_Direct3D_Effects_System::GetEffect( "Env_MaterialResponse" );
		s_pxEnvShadowEffect= Asura_Direct3D_Effects_System::GetEffect( "Env_Shadow" );

		Asura_Direct3D::D3DDevice->CreateVertexDeclaration( axASURA_D3D_ENV_VERTEX_DECLARATION, &(g_pxStandardVertexDecl) );

		{
			Direct3D_VertexElement auVSD[] =
			{
				{	0,	0,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },
				D3DDECL_END()
			};
			Asura_Direct3D::D3DDevice->CreateVertexDeclaration( auVSD, &(g_pxAABBVertexDecl) );
		}

		CreateAABBBuffers();	




		// Init the shader param block for water materials	
		s_pxWaterEffect     = Asura_Direct3D_Effects_System::GetEffect( "Env_Water" );
		s_pxWaterParamBlock = new Asura_Direct3D_EffectParameterBlock();

		s_pxWaterParamBlock->AddDynamicParameter<bool>("g_bUseTransparencyMap",					&s_xWaterParamData.m_bUseTransparencyMap);
		s_pxWaterParamBlock->AddDynamicParameter<bool>("g_bDualNormalMap",						&s_xWaterParamData.m_bDualNormalMap);
		s_pxWaterParamBlock->AddDynamicParameter<float>("g_fWaterFogScale",						&s_xWaterParamData.m_fWaterFogScale);
		s_pxWaterParamBlock->AddDynamicParameter<float>("g_fVolumetricDepthScale",				&s_xWaterParamData.m_fVolumetricDepthScale);
		s_pxWaterParamBlock->AddDynamicParameter<float>("g_fRippleScale",						&s_xWaterParamData.m_fRippleScale);
		s_pxWaterParamBlock->AddDynamicParameter<float>("g_fDistortionScale",					&s_xWaterParamData.m_fDistortionScale);
		s_pxWaterParamBlock->AddDynamicParameter<float>("g_fTwoOverScreenWidth",				&s_xWaterParamData.m_fTwoOverScreenWidth);
		s_pxWaterParamBlock->AddDynamicParameter<float>("g_fTwoOverScreenHeight",				&s_xWaterParamData.m_fTwoOverScreenHeight);
		s_pxWaterParamBlock->AddDynamicParameter<D3DXVECTOR4>("g_xWaterReflectionColour",		&s_xWaterParamData.m_xWaterReflectionColour);
		s_pxWaterParamBlock->AddDynamicParameter<D3DXVECTOR4>("g_xWaterRefractionColour",		&s_xWaterParamData.m_xWaterRefractionColour);
		s_pxWaterParamBlock->AddDynamicParameter<D3DXVECTOR4>("g_xWaterFogColour",				&s_xWaterParamData.m_xWaterFogColour);
		s_pxWaterParamBlock->AddDynamicParameter<D3DXVECTOR4>("g_xUVScroll",					&s_xWaterParamData.m_xUVScroll);
		s_pxWaterParamBlock->AddDynamicParameter<D3DXVECTOR4>("g_xUVScroll2",					&s_xWaterParamData.m_xUVScroll2);
		s_pxWaterParamBlock->AddDynamicParameter<D3DXVECTOR4>("g_xWavesX",						&s_xWaterParamData.m_xWavesX);
		s_pxWaterParamBlock->AddDynamicParameter<D3DXVECTOR4>("g_xWavesZ",						&s_xWaterParamData.m_xWavesZ);	
		s_pxWaterParamBlock->AddDynamicParameter<Direct3D_BaseTexture*>("g_xDepthTexture",		&s_xWaterParamData.m_pxDepthTexture);
		s_pxWaterParamBlock->AddDynamicParameter<Direct3D_BaseTexture*>("g_xBackBufferTexture",	&s_xWaterParamData.m_pxBackBufferTexture);
		s_pxWaterParamBlock->AddDynamicParameter<Direct3D_BaseTexture*>("g_xTransparencyMap",	&s_xWaterParamData.m_pxTransparencyMap);
		s_pxWaterParamBlock->AddDynamicParameter<Direct3D_BaseTexture*>("g_xNormalMap",			&s_xWaterParamData.m_pxNormalMap);
		s_pxWaterParamBlock->AddDynamicParameter<Direct3D_BaseTexture*>("g_xNormalMap2",		&s_xWaterParamData.m_pxNormalMap2);
		s_pxWaterParamBlock->AddDynamicParameter<Direct3D_BaseTexture*>("g_xReflectionTexture",	&s_xWaterParamData.m_pxReflectionMap);

		s_pxWaterParamBlock->Bind( s_pxWaterEffect );

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		s_pxParamBlock = new Asura_Direct3D_EffectParameterBlock();
		
		s_xCacheMaterialShaderData_Base.AddDataToParameterBlock( s_pxParamBlock);
		s_xCacheMaterialShaderData_Env.AddDataToParameterBlock( s_pxParamBlock);
		s_xCacheMaterialShaderData_Parallax.AddDataToParameterBlock( s_pxParamBlock);

		s_pxParamBlock->Bind( s_pxEnvironmentEffect);
#endif //#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	}
}

void Asura_Direct3D_EnvironmentRenderer::Reset( void )
{

	Asura_ResourceDeallocateArray(s_pxStrips);

	#if defined ASURA_TOOLS
		s_bStripsValid = false;
	#endif

	if( s_bVertexAndIndexBuffersCreatedInternally )
	{
		for( u_int u = 0; u < s_uTotalNumberOfVertexBuffers; ++u )
		{
			SAFE_RELEASE( s_ppxVertexBuffer[u] );
		}

		for( u_int u = 0; u < s_uTotalNumberOfIndexBuffers; ++u )
		{
			SAFE_RELEASE( s_ppxIndexBuffer[u] );
		}
	}

	Asura_ResourceDeallocateArray(s_pusIndexData);
	Asura_ResourceDeallocateArray(s_ppusIndexData);

	Asura_ResourceDeallocateArray(s_ppxVertexBuffer);
	Asura_ResourceDeallocateArray(s_ppxIndexBuffer);

	// DMR 05/06/07 Fixes crash on quitting tools... 
	s_bVertexAndIndexBuffersCreatedInternally = false;

	// JAT 20/06/07 reset the number of queued water strips and zero out the memory
	s_iNumQueuedWaterStrips = 0;
	Asura_Mem::Set( s_pxQueuedWaterStrips, NULL, ASURA_MAX_VISIBLE_WATER_STRIPS * sizeof( Asura_Direct3D_EnvironmentRenderer_Strip*	) );
}

void Asura_Direct3D_EnvironmentRenderer::Shutdown( void )
{
	Reset();

	delete s_pxWaterParamBlock;
	s_pxWaterParamBlock = NULL;

	SAFE_RELEASE( g_pxStandardVertexDecl );
	SAFE_RELEASE( g_pxAABBVertexDecl );
	SAFE_RELEASE( s_pxAABBIndexBuffer );
	SAFE_RELEASE( s_pxAABBVertexBuffer );
	
	delete s_pxParamBlock;
	s_pxParamBlock = NULL;

	g_pxStandardVertexDecl = NULL;
	g_pxAABBVertexDecl = NULL;
	s_pxAABBVertexBuffer = NULL;
	s_pxAABBIndexBuffer = NULL;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_EnvironmentRenderer::HandleKeyPresses
// Author       : Andy Tate
// Date         : 2-Jun-2005
//
// Description  : Checks whether the appropriate effect keys are being pressed.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_EnvironmentRenderer::HandleKeyPresses (void)
{
	//These keys are now shown in the tools help system.
	//Please update GetRendererDebugHelpStrings() below , if any changes are made
	//RAR 25/08/2006


	//RAR 25/08/2006 Ignore key presses if Shift etc are pressed , to avoid some key clashes
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) return;
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL)) return;
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return;

	// Do we wish to toggle the occlusion text?

	if (Asura_UserInput::DebouncedKeyPressed('o'))
	{
		s_bDisplayOcclusionText	= !s_bDisplayOcclusionText;
	}


	// Do we wish to toggle x-ray mode?

	if (Asura_UserInput::DebouncedKeyPressed('x'))
	{
		s_bXRayMode	= !s_bXRayMode;
	}


//	// Do we wish to cycle the object renderer?
//
//	if (Asura_UserInput::DebouncedKeyPressed('b'))
//	{
//		// Yes - change the type of shader...
//
//#if defined(ASURA_TOOLS)
//		const bool bNewObjects =
//#endif
//		Asura_Direct3D_ObjectRenderer::ToggleNewRenderers();
//
//#if defined(ASURA_TOOLS)
//		// Display the appropriate message...
//
//		Tools_Timed_Message_System::AddMessage	(0xFFFFFFFF, bNewObjects ? "Objects effect renderer enabled" : "Objects original renderer enabled");
//#endif	// defined(ASURA_TOOLS)
//	}
//
//	if (Asura_UserInput::DebouncedKeyPressed('h'))
//	{
//		// Yes - change the type of shader...
//
//#if defined(ASURA_TOOLS)
//		const bool bNewObjects =
//#endif
//		Asura_Direct3D_ObjectHierarchyRenderer::ToggleNewRenderers();
//
//#if defined(ASURA_TOOLS)
//		// Display the appropriate message...
//
//		Tools_Timed_Message_System::AddMessage	(0xFFFFFFFF, bNewObjects ? "Hierarchy effect renderer enabled" : "Hierarchy original renderer enabled");
//#endif	// defined(ASURA_TOOLS)
//	}
}

/// End of function 'Asura_Direct3D_EnvironmentRenderer::HandleKeyPresses'. ///


#ifdef ASURA_TOOLS
//Gets the strings to be shown in the tools help system.
//RAR 25/08/2006
const char** Asura_Direct3D_EnvironmentRenderer::GetRendererDebugHelpStrings()
{
	static const char* s_apszRendererDebugHelpStrings[] =
	{
		"These keys only work in standby mode :",
		"",
		"O :           Toggle occulusion debug text",
		"X :           Toggle X-Ray mode",
		NULL //terminator
	};

	return s_apszRendererDebugHelpStrings; 
}
#endif //ASURA_TOOLS

//-----------------------------------------------------
// DLH - Render Material
//-----------------------------------------------------

void Asura_Direct3D_EnvironmentRenderer::Render_SetMaterials( const Asura_MaterialResponse* pxMaterial1, const Asura_MaterialResponse* pxMaterial2)
{
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	s_xCacheMaterialShaderData_Base.Setup( pxMaterial1, pxMaterial2, NULL);
	s_xCacheMaterialShaderData_Env.Setup( pxMaterial1, pxMaterial2);
	s_xCacheMaterialShaderData_Parallax.Setup( pxMaterial1, pxMaterial2);
	
	s_pxParamBlock->Apply();
#endif //#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	
	Asura_Direct3D_MaterialResponse::SetBlendStates( pxMaterial1 );
}

//-----------------------------------------------------
// DLH - Render Normal
//-----------------------------------------------------

void Asura_Direct3D_EnvironmentRenderer::Render_Normal( Asura_Direct3D_Effect* const pxEffect, const bool bTransparentStrips)
{
#if 1
	HRESULT xResult;
	
	Direct3D_VertexBuffer* pxVertexBufferLast = NULL;
	Direct3D_IndexBuffer* pxIndexBufferLast = NULL;
	Asura_Hash_ID uMaterialHash1Last = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uMaterialHash2Last = ASURA_HASH_ID_UNSET;
	bool bSetupMaterials = true;

	// Ensure that our visibility is up-to-date...
	Asura_Environment_Renderer::UpdateVisibilityOfStrips();

#ifdef ASURA_TOOLS

	// Need to render transparent decal-type strips after opaque ones, but still before the deferred
	// lights. In retail builds, strips will be output with transparent ones last anyway, so we
	// don't need to do this outside of Tools. For the non-decal transparency pass, just do it once

	const u_int uNumPasses = bTransparentStrips ? 1 : 2;

	if (!bTransparentStrips)
	{	
		Asura_Direct3D_MaterialResponse::SetFogParameters( false, NULL, *pxEffect );
	}

	for( u_int uPass = 0; uPass < uNumPasses; ++uPass )
#endif
	{
		for( u_int u = 0; u < s_uTotalNumberOfStrips; ++u )
		{
			const Asura_Direct3D_EnvironmentRenderer_Strip& xStrip = s_pxStrips[u];

			if(!Asura_Environment_Renderer::IsStripVisibleFromCurrentCamera(u)) continue;

#ifdef ASURA_TOOLS
			if ( Tools_System::IsActive() )
			{
				if( !Tools_Render_Options::IsModuleWithinViewDistance( xStrip.m_uModule ) ) continue;
			}
#endif

			SCOPEDPIXEVENT(( 0xFF800000, L"Strip %u", u ));

			const Asura_Hash_ID uMaterialHash1 = xStrip.m_auMaterialResponseHashID[0];
			const Asura_MaterialResponse* pxMaterial1 =
				Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

#ifdef ASURA_TOOLS
			const Asura_Hash_ID uMaterialHash2 =
				(s_bDisableDualPass ? ASURA_HASH_ID_UNSET : xStrip.m_auMaterialResponseHashID[1]);
#else
			const Asura_Hash_ID uMaterialHash2 = xStrip.m_auMaterialResponseHashID[1];
#endif
			const Asura_MaterialResponse* pxMaterial2 =
				uMaterialHash2 ? Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash2 ) : 0;

			{
				bool bUpdateMaterials = ((true == bSetupMaterials) || (uMaterialHash1Last != uMaterialHash1) || (uMaterialHash2Last != uMaterialHash2));
			
				uMaterialHash1Last = uMaterialHash1;
				uMaterialHash2Last = uMaterialHash2;
				
				if( pxMaterial1 )
				{
					if( pxMaterial1->GetDontRender() ) continue;

					if( bTransparentStrips )
					{
						if( !pxMaterial1->IsTransparent() ) continue;
					}
					else
					{
						if( pxMaterial1->IsTransparent() ) continue;

						#ifdef ASURA_TOOLS

							// Render opaque materials in the first pass, transparent decal ones after

							const bool bOpaqueMaterial =
								(pxMaterial1->GetBlendMode() == Asura_MaterialResponse::BLEND_MODE_OPAQUE);

							const bool bOpaquePass = (uPass == 0);
							if( bOpaqueMaterial != bOpaquePass ) continue;

						#endif // defined ASURA_TOOLS
					}

					if (pxMaterial1->GetSpecialFXMode()==Asura_MaterialResponse::SPECIALFX_MODE_WATER)
					{
						QueueWaterStrip(&xStrip);
						continue;
					}

					if( true == bUpdateMaterials)
					{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
						if( true == s_bAsura_Direct3D_EnvironmentRenderer_OutputLog)
						{
							Asura_SimpleLogfileOutput(( ASURA_DIRECT3D_ENVIRONMENTRENDERER_OUTPUTLOG_FILENAME, "Materials 0x%08X 0x%08X ", uMaterialHash1, uMaterialHash2));
						}
#endif
						
						SCOPEDPIXEVENT(( 0xFF800000, L"Materials 0x%08X 0x%08X", uMaterialHash1, uMaterialHash2));

						if( !bTransparentStrips)
						{
							Asura_RGBA xPrelightingColour = pxMaterial1->GetAmbientColour();
							xPrelightingColour *= pxMaterial1->GetAmbientContribution();

							Asura_Direct3D::D3DDevice->SetVertexShaderConstantF( 0, &xPrelightingColour.m_fR, 1 );

							Asura_Direct3D_MaterialResponse::SetBlendStates( pxMaterial1 );
						}
						else
						{
							Render_SetMaterials( pxMaterial1, pxMaterial2 );
						}

						bSetupMaterials = false;

						if (Asura_MaterialResponse_System::GetMaterialProjectContextCallback())
						{
							Asura_MaterialResponse_System::GetMaterialProjectContextCallback()(pxMaterial1->GetProjectContext(), 
								pxMaterial2 ? pxMaterial2->GetProjectContext() : ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID);
						}

					}

					// For transparent env strips, grab an SH at the centre of the bounding box and
					// use that for lighting

					if( bTransparentStrips )
					{
						Asura_SphericalHarmonics_Order2_RGB& xSH =
							Asura_GI_Solution::s_xDynamicSHForCurrentRender;

						const Asura_Vector_3 xStripCentre = xStrip.m_xBB.CalculateCentre();

						Asura_GI_Solution::GetDynamicIrradiance( xSH, xStripCentre );

						// Static SH will come from the per-vertex values
						Asura_GI_Solution::s_xStaticSHForCurrentRender.Zero();

						Asura_Direct3D_MaterialResponse::SetSHParameters( pxMaterial1, *pxEffect );

						// Also need to set up fogging for transparent strips
						Asura_Direct3D_MaterialResponse::SetFogParameters( true, pxMaterial1, *pxEffect );

						pxEffect->GetD3DEffect()->CommitChanges();
					}
				}
			}

			Direct3D_VertexBuffer* pxVertexBufferPresent = s_ppxVertexBuffer[ xStrip.m_uVertexBuffer ];
			if( pxVertexBufferLast != pxVertexBufferPresent)
			{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
				if( true == s_bAsura_Direct3D_EnvironmentRenderer_OutputLog)
				{
					Asura_SimpleLogfileOutput(( ASURA_DIRECT3D_ENVIRONMENTRENDERER_OUTPUTLOG_FILENAME, "Change VB "));
				}
#endif
				Asura_Direct3D::D3DDevice->SetStreamSource( 0, pxVertexBufferPresent, 0, sizeof( Asura_Direct3D_EnvironmentRenderer_Vertex ) );
				pxVertexBufferLast = pxVertexBufferPresent;
			}

			Direct3D_IndexBuffer* pxIndexBufferPresent = s_ppxIndexBuffer[ xStrip.m_uIndexBuffer];
			if( pxIndexBufferLast != pxIndexBufferPresent)
			{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
				if( true == s_bAsura_Direct3D_EnvironmentRenderer_OutputLog)
				{
					Asura_SimpleLogfileOutput(( ASURA_DIRECT3D_ENVIRONMENTRENDERER_OUTPUTLOG_FILENAME, "Change VB "));
				}
#endif
				Asura_Direct3D::D3DDevice->SetIndices( pxIndexBufferPresent);
				pxIndexBufferLast = pxIndexBufferPresent;
			}

			if( !bTransparentStrips )
			{
				// Replace the pixel shader from the effect with the faster version. TODO: remove
				// the effect entirely

				Asura_Hash_ID uCombinedHash = Asura_MaterialResponse_System::CombineShaderHashes(
					pxMaterial1, pxMaterial2, false );

				#ifdef ASURA_TOOLS
				if( s_bDisableDualPass )
				{
					uCombinedHash = Tools_PC_Export::GetMRPShaderHashWithoutDualPass( uCombinedHash );
				}
				#endif

				Asura_Direct3D_MaterialResponse::SetMaterialConstants( uCombinedHash, pxMaterial1,
					pxMaterial2, NULL );
			}

			const D3DPRIMITIVETYPE ePrimitiveType = GetPrimType();
												  
			u_int uPrimitiveCount = xStrip.m_uNumberOfPrimitives;
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
			if( true == s_bAsura_Direct3D_EnvironmentRenderer_DrawOnlyOnePrimitive && 0 < uPrimitiveCount)
			{
				uPrimitiveCount = 1;
			}
#endif
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
			if( false == s_bAsura_Direct3D_EnvironmentRenderer_EnableDrawCall)
			{
				continue;
			}
#endif

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE) && defined(ASURA_LOGFILES_ENABLED)
			if( true == s_bAsura_Direct3D_EnvironmentRenderer_OutputLog)
			{
				u_int uIndexCount = (!s_bUseTriLists) ? (uPrimitiveCount + 2) : (3 * uPrimitiveCount);
				Asura_SimpleLogfileOutput(( ASURA_DIRECT3D_ENVIRONMENTRENDERER_OUTPUTLOG_FILENAME, "Strip %4d: 0x%04X-0x%04X 0x%04X-0x%04X\n", u, xStrip.m_uStartVertex, xStrip.m_uStartVertex + xStrip.m_uNumberOfVertices, xStrip.m_uStartIndex, xStrip.m_uStartIndex + uIndexCount));
			}
#endif

			const u_int uBaseVertexIndex = (s_bUseTriLists&&s_bUsingPreloadedData) ? xStrip.m_uStartVertex : 0;

			xResult = Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
				ePrimitiveType,
				uBaseVertexIndex,	 
				xStrip.m_uStartVertex,
				xStrip.m_uNumberOfVertices,
				xStrip.m_uStartIndex,
				uPrimitiveCount
			);
			ASURA_D3D_OK( xResult);

			Asura_Render::AddToTrianglesRendered( uPrimitiveCount);
		}
	}
#endif
}

//-----------------------------------------------------
// DLH - Render Entry Point
//-----------------------------------------------------
void Asura_Direct3D_EnvironmentRenderer::Render( const bool bTransparentStrips )
{
	if( s_uTotalNumberOfStrips == 0 ) return;

	if ( bTransparentStrips == false )
	{
		// JAT 20/06/07 reset the number of queued water strips - in case Render gets called twice without a call to RenderWater inbetween
		s_iNumQueuedWaterStrips = 0;
	}

	#if defined ASURA_TOOLS

		if( !s_bUsingPreloadedData && !s_bStripsValid )
		{
			Tools_CreateData( Asura_Module_System::GetNumberOfModules() );
		}

	#endif // defined ASURA_TOOLS

	SCOPEDPIXEVENT(( 0xFFFF0000, L"Environment (%s)",
		bTransparentStrips ? L"Transparent Strips" : L"Opaque Strips" ));

	// Ensure we've got the correct camera settings...
	Asura_Render::ApplyCameraSettings();

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	RECT xOriginalScissorRect = { 0};
	if( false == s_bAsura_Direct3D_EnvironmentRenderer_EnablePixels)
	{
		Asura_Direct3D::D3DDevice->GetScissorRect( &xOriginalScissorRect);
		RECT xNewScissorRect = { 0, 0, 1, 1};
		Asura_Direct3D::D3DDevice->SetScissorRect( &xNewScissorRect);
		Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_SCISSORTESTENABLE, TRUE);
	}
	if( true == s_bAsura_Direct3D_EnvironmentRenderer_OutputLog)
	{
		Asura_SimpleLogfileClear( ASURA_DIRECT3D_ENVIRONMENTRENDERER_OUTPUTLOG_FILENAME);
	}
#endif

	Asura_Direct3D_Effect* const pxEffect = s_pxEnvironmentEffect;
	if( !pxEffect || !pxEffect->IsD3DEffectValid() ) 
	{
		return;
	}

	const u_int uTechnique = bTransparentStrips ? 1 : 0;
	Asura_Direct3D_Effects_System::BeginEffect( *pxEffect, uTechnique );

	pxEffect->BeginPass( 0);
		
	// Set main materials data texture
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	s_xCacheMaterialShaderData_Base.SetupCommon();
#endif //#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	Asura_Direct3D::D3DDevice->SetVertexDeclaration( g_pxStandardVertexDecl );

	// Set render states
	{
		// (SB - 23/05/08) The clip shader intrinsic is broken on ATI X series cards so had to be replaced with an alpha test
		Asura_Render::CurrentStates.RequestAlphaTest( true );
		Asura_Render::CurrentStates.RequestDitheringMode( true );
		Asura_Render::CurrentStates.RequestDynamicLights( false );
		Asura_Render::CurrentStates.RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
		Asura_Render::CurrentStates.RequestZBufferEnabled( true );

#ifdef ASURA_TOOLS
        if( Tools_Render_Options::NoBackFaceCull )
        {
		    Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
        }
        else
#endif
        {
		    Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CW );
        }

		// Leave wireframe as it was set elsewhere
	}

	for( u_int u = 0; u < 16; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR );
	}

	Render_Normal( pxEffect, bTransparentStrips );

	pxEffect->EndPass();

	Asura_Direct3D_Effects_System::EndEffect( *pxEffect);

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	s_bAsura_Direct3D_EnvironmentRenderer_OutputLog = false;
#endif

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	if( false == s_bAsura_Direct3D_EnvironmentRenderer_EnablePixels)
	{
		Asura_Direct3D::D3DDevice->SetScissorRect( &xOriginalScissorRect);
		Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_SCISSORTESTENABLE, FALSE);
	}
#endif

	const float fDepthBias = 0.0f;
	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_DEPTHBIAS, *(DWORD*)&fDepthBias );

	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );
	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE2, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );

	for( u_int u = 0; u < 15; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_SRGBTEXTURE, FALSE );
	}

	Asura_Direct3D_Effects_System::FlushEffects();
}

void Asura_Direct3D_EnvironmentRenderer::RenderToShadow(float fNear, float fFar, bool bCubeMap)
{
#if 1
	#if defined ASURA_TOOLS
		if( !s_bUsingPreloadedData && !s_bStripsValid )
		{
			Tools_CreateData( Asura_Module_System::GetNumberOfModules() );
		}
	#endif

	SCOPEDPIXEVENT(( 0, L"Render Environment to Shadow" ));

	if( s_uTotalNumberOfStrips == 0
	 || !s_pxEnvShadowEffect
	 || !s_pxEnvShadowEffect->IsD3DEffectValid() )
	{
		return;
	}

	// Set up
	{
		SCOPEDPIXEVENT(( 0, L"Set Up" ));

		Asura_Render::ApplyCameraSettings();
		Asura_Direct3D_Effects_System::SetSharedParameters();

		Asura_Direct3D_Effects_System::BeginEffect( *s_pxEnvShadowEffect, (u_int)bCubeMap );
		s_pxEnvShadowEffect->BeginPass(0);

		Asura_Direct3D::D3DDevice->SetVertexDeclaration( g_pxStandardVertexDecl );

		Asura_Render::CurrentStates.RequestAlphaTest( true );
		Asura_Render::CurrentStates.RequestAlphaTest( false );
		Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CW );
		Asura_Render::CurrentStates.RequestDitheringMode( true );
		Asura_Render::CurrentStates.RequestDynamicLights( false );
		Asura_Render::CurrentStates.RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
		Asura_Render::CurrentStates.RequestZBufferEnabled( true );

		// Leave wireframe as it was set elsewhere
	}

	Direct3D_VertexBuffer* pxLastVB = NULL;
	Direct3D_IndexBuffer* pxLastIB = NULL;

	// make a rough bounding box around the light, for trivial rejection of env strips
	Asura_Bounding_Box xLightAABB;
	xLightAABB.SetToPoint(*Asura_Camera::GetPosition());
	xLightAABB.Grow( fFar );

	for( u_int u = 0; u < s_uTotalNumberOfStrips; ++u )
	{
		const Asura_Direct3D_EnvironmentRenderer_Strip& xStrip = s_pxStrips[u];
		
		if (!xLightAABB.Intersects(xStrip.m_xBB)) continue;

		if( !Asura_Camera::IsAABBInFrustum( xStrip.m_xBB ) ) continue;

		#if defined ASURA_TOOLS
			if( Tools_System::IsActive() && !Tools_Render_Options::IsModuleWithinViewDistance( xStrip.m_uModule ) )
			{
				continue;
			}
		#endif

		// Strip out unwanted materials
		{
			const Asura_Hash_ID uMaterial1Hash = xStrip.m_auMaterialResponseHashID[0];
			const Asura_MaterialResponse* const pxMaterial1 =
				Asura_MaterialResponse_System::GetMaterialByHashID( uMaterial1Hash );

			if( pxMaterial1 )
			{
				if( pxMaterial1->GetDontCastShadow() || pxMaterial1->IsTransparent() ) continue;

				const bool bSpecialRender =
					pxMaterial1->GetSpecialFXMode() != Asura_MaterialResponse::SPECIALFX_MODE_NONE;

				if( bSpecialRender ) continue;
			}
		}

		SCOPEDPIXEVENT(( 0, L"Strip %u", u ));

		// Switch VB or IB if necessary
		{
			Direct3D_VertexBuffer* const pxCurrVB = s_ppxVertexBuffer[ xStrip.m_uVertexBuffer ];
			if( pxCurrVB != pxLastVB )
			{
				const u_int uStride = sizeof( Asura_Direct3D_EnvironmentRenderer_Vertex );
				Asura_Direct3D::D3DDevice->SetStreamSource( 0, pxCurrVB, 0, uStride );
				pxLastVB = pxCurrVB;
			}

			Direct3D_IndexBuffer* const pxCurrIB = s_ppxIndexBuffer[ xStrip.m_uIndexBuffer ];
			if( pxCurrIB != pxLastIB )
			{
				Asura_Direct3D::D3DDevice->SetIndices( pxCurrIB );
				pxLastIB = pxCurrIB;
			}
		}

		// Render
		{
			const D3DPRIMITIVETYPE ePrimType = GetPrimType();
			const u_int uBaseVertexIndex = (s_bUseTriLists&&s_bUsingPreloadedData) ? xStrip.m_uStartVertex : 0;

			Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
			(
				ePrimType, 
				uBaseVertexIndex,
				xStrip.m_uStartVertex,
				xStrip.m_uNumberOfVertices,
				xStrip.m_uStartIndex,
				xStrip.m_uNumberOfPrimitives
			);

			Asura_Render::AddToTriPassesRendered( xStrip.m_uNumberOfPrimitives );
		}
	}

	// Clean up
	{
		SCOPEDPIXEVENT(( 0, L"Clean Up" ));

		s_pxEnvShadowEffect->EndPass();
		Asura_Direct3D_Effects_System::EndEffect( *s_pxEnvShadowEffect );

		Asura_Direct3D_Effects_System::FlushEffects();
	}
#endif
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_EnvironmentRenderer::QueueWaterStrip
// Author       : Dave Reed
// Date         : 17-May-2007
//
// Description  : Queues a water strip for rendering after the deferred lighting passes
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_EnvironmentRenderer::QueueWaterStrip(const Asura_Direct3D_EnvironmentRenderer_Strip *pxStrip)
{
	if (s_iNumQueuedWaterStrips<ASURA_MAX_VISIBLE_WATER_STRIPS)
	{
		s_pxQueuedWaterStrips[s_iNumQueuedWaterStrips] = pxStrip;
		s_iNumQueuedWaterStrips++;
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_EnvironmentRenderer::RenderWater
// Author       : Dave Reed
// Date         : 17-May-2007
//
// Description  : Called after the deferred lighting passes to render any visible water
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_EnvironmentRenderer::RenderWater(void)
{
	if (s_iNumQueuedWaterStrips==0)
	{
		return;
	}

	if (!Asura_Render::IsSystemEnabled(ASURA_RENDER_SYSTEM_WATER))
	{
		s_iNumQueuedWaterStrips = 0;
		return;
	}

	SCOPEDPIXEVENT(( 0xFF0000FF, L"Environment (Water Strips)" ));

	// Clear the dest alpha to 0 first
	float fWidth =  (float)Asura_Viewport::GetWidth();		
	float fHeight = (float)Asura_Viewport::GetHeight();			
	Asura_Overlay_Quad xQuad;
	xQuad.SetRectXY(0,0, fWidth, fHeight);
	xQuad.Colour = 0x00ffffff;
	xQuad.TextureIndex = ASURA_TEXTURE_NONE;
	xQuad.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::CurrentStates.RequestAlphaTest( false , 0x00);
	Asura_Render::CurrentStates.RequestStencilEnabled( false);
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_ALPHA);
	xQuad.Render(false);
	Asura_Direct3D::FlushQuadBatches();
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_RED|D3DCOLORWRITEENABLE_GREEN|D3DCOLORWRITEENABLE_BLUE|D3DCOLORWRITEENABLE_ALPHA);

	// Water ripples - esentially a heightmap - gets rendered into the the dest. alpha channel	
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__WORLD_RENDER);

	// If we're using multithreaded PFX, this will wait for all job to finish, if neccessary
	Asura_SpecialFX_System::FinishAllJobs();
	// Render any ripples
	Asura_PFX_System::Render(ASURA_PFX_RENDER_WATER_DISTORTER);

	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__WORLD_RENDER, "World Render");	

	Asura_Direct3D::ClearRenderTargets(1);

	if(!Asura_Direct3D_BackBufferCapture::HasCapturedThisFrame())
	{
		Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();
	}

	
	Asura_Direct3D_Effect* const pxEffect = s_pxWaterEffect;
	if( !pxEffect || !pxEffect->IsD3DEffectValid() ) return;
	Asura_Direct3D_Effects_System::BeginEffect(*pxEffect);

	// Set render states
	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );		
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );
	Asura_Render::CurrentStates.RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
	Asura_Render::CurrentStates.RequestZBufferEnabled( true );

	Asura_Direct3D::D3DDevice->SetVertexDeclaration( g_pxStandardVertexDecl ); 

	s_xWaterParamData.m_fTwoOverScreenWidth  = (2.0f / Asura_Viewport::GetWidth());
	s_xWaterParamData.m_fTwoOverScreenHeight = (2.0f / Asura_Viewport::GetHeight());
	s_xWaterParamData.m_pxDepthTexture		 = Asura_Direct3D::GetDeferredRenderingTarget()->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX);
	s_xWaterParamData.m_pxBackBufferTexture  = Asura_Direct3D_BackBufferCapture::GetCaptureTexture();

	for(int i=0;i<s_iNumQueuedWaterStrips;i++)
	{
		const Asura_Direct3D_EnvironmentRenderer_Strip  *pxStrip = s_pxQueuedWaterStrips[i];
		// JAT 20/06/07 bullet proof catching of the list going wrong
		Asura_Assert( pxStrip, "" );
#ifdef ASURA_TOOLS
		if ( pxStrip == NULL )
		{
			continue;
		}
#endif // #ifdef ASURA_TOOLS
		
		const Asura_Hash_ID uMaterialHash1		 = pxStrip->m_auMaterialResponseHashID[0];
		const Asura_MaterialResponse* pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

		const Asura_Hash_ID uMaterialHash2		 = pxStrip->m_auMaterialResponseHashID[1];
		const Asura_MaterialResponse* pxMaterial2 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash2 );

		if( pxMaterial1 )
		{
			s_xWaterParamData.m_pxTransparencyMap			= Asura_Direct3D::GetDirect3DTextureByIndex(pxMaterial1->GetTexture( Asura_MaterialResponse::TRANSPARENCY_MAP ));
			s_xWaterParamData.m_pxNormalMap					= Asura_Direct3D::GetDirect3DTextureByIndex(pxMaterial1->GetTexture( Asura_MaterialResponse::NORMAL_MAP ));
			s_xWaterParamData.m_pxNormalMap2				= Asura_Direct3D::GetDirect3DTextureByIndex(pxMaterial2 ? pxMaterial2->GetTexture( Asura_MaterialResponse::NORMAL_MAP ) : ASURA_TEXTURE_NONE);
			s_xWaterParamData.m_pxReflectionMap				= Asura_Direct3D::GetDirect3DTextureByIndex(pxMaterial1->GetTexture( Asura_MaterialResponse::ENVIRONMENT_MAP));

			s_xWaterParamData.m_bDualNormalMap				= (pxMaterial2 && pxMaterial2->GetTexture( Asura_MaterialResponse::NORMAL_MAP ) != ASURA_TEXTURE_NONE);
			s_xWaterParamData.m_bUseTransparencyMap			= (pxMaterial1->GetTexture( Asura_MaterialResponse::TRANSPARENCY_MAP ) != ASURA_TEXTURE_NONE);

			s_xWaterParamData.m_xWaterReflectionColour.x	= pxMaterial1->GetWaterReflectionColour().m_fR;
			s_xWaterParamData.m_xWaterReflectionColour.y	= pxMaterial1->GetWaterReflectionColour().m_fG;
			s_xWaterParamData.m_xWaterReflectionColour.z	= pxMaterial1->GetWaterReflectionColour().m_fB;
			s_xWaterParamData.m_xWaterReflectionColour.w	= pxMaterial1->GetWaterReflectionColour().m_fA;			

			s_xWaterParamData.m_xWaterRefractionColour.x	= pxMaterial1->GetWaterRefractionColour().m_fR;
			s_xWaterParamData.m_xWaterRefractionColour.y	= pxMaterial1->GetWaterRefractionColour().m_fG;
			s_xWaterParamData.m_xWaterRefractionColour.z	= pxMaterial1->GetWaterRefractionColour().m_fB;
			s_xWaterParamData.m_xWaterRefractionColour.w	= pxMaterial1->GetWaterRefractionColour().m_fA;

		 	s_xWaterParamData.m_xWaterFogColour.x			= pxMaterial1->GetWaterFogColour().m_fR;
			s_xWaterParamData.m_xWaterFogColour.y			= pxMaterial1->GetWaterFogColour().m_fG;
			s_xWaterParamData.m_xWaterFogColour.z			= pxMaterial1->GetWaterFogColour().m_fB;
			s_xWaterParamData.m_xWaterFogColour.w			= pxMaterial1->GetWaterFogColour().m_fA;			

			s_xWaterParamData.m_fWaterFogScale				= pxMaterial1->GetWaterFogStrength();
			s_xWaterParamData.m_fVolumetricDepthScale		= 5.0f/pxMaterial1->GetWaterEdgeSoftness();
			s_xWaterParamData.m_fRippleScale				= pxMaterial1->GetWaterRippleScale();
			s_xWaterParamData.m_fDistortionScale			= pxMaterial1->GetWaterDistortionScale();

			s_xWaterParamData.m_xUVScroll.x					= pxMaterial1->GetWaterUVControl(0) * Asura_Timers::GetLocalScaledTime();
			s_xWaterParamData.m_xUVScroll.y					= pxMaterial1->GetWaterUVControl(1) * Asura_Timers::GetLocalScaledTime();
			s_xWaterParamData.m_xUVScroll.z					= pxMaterial1->GetWaterUVControl(2);			
			s_xWaterParamData.m_xUVScroll2.x				= pxMaterial1->GetWaterUVControl(3) * Asura_Timers::GetLocalScaledTime();
			s_xWaterParamData.m_xUVScroll2.y				= pxMaterial1->GetWaterUVControl(4) * Asura_Timers::GetLocalScaledTime();
			s_xWaterParamData.m_xUVScroll2.z				= pxMaterial1->GetWaterUVControl(5);

			s_xWaterParamData.m_xWavesX.x					= pxMaterial1->GetWaterWaveControl(0);
			s_xWaterParamData.m_xWavesX.y					= pxMaterial1->GetWaterWaveControl(1);
			s_xWaterParamData.m_xWavesX.z					= pxMaterial1->GetWaterWaveControl(2) * Asura_Timers::GetLocalScaledTime();
			s_xWaterParamData.m_xWavesZ.x					= pxMaterial1->GetWaterWaveControl(3);
			s_xWaterParamData.m_xWavesZ.y					= pxMaterial1->GetWaterWaveControl(4);
			s_xWaterParamData.m_xWavesZ.z					= pxMaterial1->GetWaterWaveControl(5) * Asura_Timers::GetLocalScaledTime();

			pxEffect->SetParameterByHash( "g_bFogEnabled", 0xDBC836CD, Asura_Fog::IsFogEnabled() );

			const D3DXVECTOR4 xFogData( Asura_Fog::GetFogNearPlane(),
				Asura_Fog::GetOneOverFogPlaneSeparation(), 0.0f, 0.0f );

			pxEffect->SetParameterByHash( "g_xFogParams", 0x06F86524, xFogData );

			const Asura_RGBA& xFogColour = Asura_Fog::GetFogColour();
			const D3DXVECTOR4 xFogColourVec( xFogColour.m_fR, xFogColour.m_fG, xFogColour.m_fB,
				xFogColour.m_fA );

			pxEffect->SetParameterByHash( "g_xFogColour", 0xF18C296A, xFogColourVec );

			pxEffect->SetParameterTextureByHash( "g_xFogTexture",
				0xB2DDDD3D, Asura_Direct3D_Fog::GetTexture() );
			
			s_pxWaterParamBlock->Apply();

			Asura_Direct3D::D3DDevice->SetStreamSource( 0, s_ppxVertexBuffer[ pxStrip->m_uVertexBuffer ], 0, sizeof( Asura_Direct3D_EnvironmentRenderer_Vertex ) );
			Asura_Direct3D::D3DDevice->SetIndices( s_ppxIndexBuffer[ pxStrip->m_uVertexBuffer ] );

			const D3DPRIMITIVETYPE ePrimitiveType = GetPrimType();


			pxEffect->RenderAllIndexedPrimitivePasses(  ePrimitiveType,
														pxStrip->m_uStartVertex,
														pxStrip->m_uNumberOfVertices,
														pxStrip->m_uStartIndex,
														pxStrip->m_uNumberOfPrimitives,
														NULL );
		}
		// JAT 20/06/07 clear out the water strip ptr after it has been used
		s_pxQueuedWaterStrips[i] = NULL;
	}

	Asura_Direct3D_Effects_System::EndEffect(*pxEffect);
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );

	// Got to clear these, or it will crash on device reset... 
	s_xWaterParamData.m_pxDepthTexture		 = NULL;
	s_xWaterParamData.m_pxBackBufferTexture  = NULL;
	s_pxWaterParamBlock->Apply();

	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CW );	
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( true );

	// This ensures that we get a fresh capture before doing any FSFX
	Asura_Direct3D_BackBufferCapture::AdviseNextFrame();

	s_iNumQueuedWaterStrips=0;
}

Direct3D_VertexDeclaration* Asura_Direct3D_EnvironmentRenderer::GetVD()
{
	return g_pxStandardVertexDecl;
}

D3DPRIMITIVETYPE Asura_Direct3D_EnvironmentRenderer::GetPrimType()
{
	return s_bUseTriLists ? D3DPT_TRIANGLELIST : D3DPT_TRIANGLESTRIP;
}

const Asura_Direct3D_EnvironmentRenderer_Strip& Asura_Direct3D_EnvironmentRenderer::GetStrip(
	const u_int u )
{
	return s_pxStrips[u];
}

Direct3D_VertexBuffer* Asura_Direct3D_EnvironmentRenderer::GetVB( const u_int u )
{
	return s_ppxVertexBuffer[u];
}

Direct3D_IndexBuffer* Asura_Direct3D_EnvironmentRenderer::GetIB( const u_int u )
{
	return s_ppxIndexBuffer[u];
}

void Asura_Direct3D_EnvironmentRenderer::RenderGeometry()
{
	// Renders all strip geometry, assuming render state has been set up by the caller. No
	// visibility culling at the moment, not sure how best to implement that yet

	SCOPEDPIXEVENT(( 0, L"Environment geometry" ));

	Direct3D_Device* const pxDevice = Asura_Direct3D::D3DDevice;
	if( !pxDevice ) return;

	pxDevice->SetVertexDeclaration( g_pxStandardVertexDecl );

	const D3DPRIMITIVETYPE ePrimType = GetPrimType();

	for( u_int u = 0; u < s_uTotalNumberOfStrips; ++u )
	{
		SCOPEDPIXEVENT(( 0, L"Strip %u", u ));

		const Asura_Direct3D_EnvironmentRenderer_Strip& xStrip = s_pxStrips[u];

		pxDevice->SetStreamSource( 0, s_ppxVertexBuffer[ xStrip.m_uVertexBuffer ], 0,
			sizeof( Asura_Direct3D_EnvironmentRenderer_Vertex ) );

		pxDevice->SetIndices( s_ppxIndexBuffer[ xStrip.m_uIndexBuffer ] );

		const u_int uBaseVertexIndex = (s_bUseTriLists&&s_bUsingPreloadedData) ? xStrip.m_uStartVertex : 0;

		pxDevice->DrawIndexedPrimitive
		(
			ePrimType,
			uBaseVertexIndex,
			xStrip.m_uStartVertex,
			xStrip.m_uNumberOfVertices,
			xStrip.m_uStartIndex,
			xStrip.m_uNumberOfPrimitives
		);
	}
}

bool Asura_Direct3D_EnvironmentRenderer::RenderAABB( const Asura_Bounding_Box& xBB )
{	
	Asura_Direct3D_Effect*	pxEffect	= Asura_Direct3D_Effects_System::BeginEffect ("Env_ZPassAABB");


	// Do we have an effect?

	if ((NULL != pxEffect) && pxEffect->IsD3DEffectValid())
	{
		const D3DPRIMITIVETYPE	ePrimitiveType	= D3DPT_TRIANGLELIST;

		D3DXVECTOR4				xScale( xBB.MaxX-xBB.MinX, xBB.MaxY-xBB.MinY, xBB.MaxZ-xBB.MinZ, 1.0f );
		D3DXVECTOR4				xOffset( xBB.MinX, xBB.MinY, xBB.MinZ, 0.0f );


		// Disable Z-buffer writes...

		Asura_Render::CurrentStates.RequestZBufferWriteEnabled	(false);


		// Set the appropriate streams...

		SetStreams	(*pxEffect, s_pxAABBVertexBuffer, sizeof(Asura_Direct3D_AABB_Vertex), s_pxAABBIndexBuffer);


		// Force the vertex declaration...

		Asura_Direct3D::D3DDevice->SetVertexDeclaration	(g_pxAABBVertexDecl);


		// Set any render states...

		Asura_Render::CurrentStates.RequestTranslucencyMode	(ASURA_TRANSLUCENCY_OFF);
		Asura_Render::CurrentStates.RequestTextureIndex		(0, ASURA_TEXTURE_NONE);
		Asura_Render::CurrentStates.RequestCullMode			(ASURA_CULLMODE_NONE);


		// Set the effect parameters...

		pxEffect->SetParameterByName	("g_xScale",	xScale);
		pxEffect->SetParameterByName	("g_xOffset",	xOffset);


		// Render this AABB...

		pxEffect->RenderAllIndexedPrimitivePasses	(ePrimitiveType, 0, 8, 0, 12, NULL);


		// Enable Z-Buffer writes...

		Asura_Render::CurrentStates.RequestZBufferWriteEnabled	(true);
	}

	return	(0);
}


void Asura_Direct3D_EnvironmentRenderer::CreateAABBBuffers( void )
{
	{
		u_int uVertexDataSize = sizeof(Asura_Direct3D_AABB_Vertex)*8;

		// create vertex buffer
		Asura_Direct3D::D3DDevice->CreateVertexBuffer
		(
			uVertexDataSize,
			D3DUSAGE_WRITEONLY,
			0,
			D3DPOOL_MANAGED,
			&s_pxAABBVertexBuffer,
			0
		);

		// lock the buffer so we can write to it
		void*	pxVertexData;
		if (SUCCEEDED(s_pxAABBVertexBuffer->Lock(0,uVertexDataSize,&pxVertexData,0)))
		{
			// copy across the vertex data
			memcpy( pxVertexData, g_axCubeVertices, uVertexDataSize );

			// writing finished, unlock buffer
			s_pxAABBVertexBuffer->Unlock();
		} else {
			Asura_Assert(0, "Failed to lock vertex buffer");
		}

	}

	{
		u_int uIndexDataSize = 36*sizeof(unsigned short);
		// create the index buffer
		Asura_Direct3D::D3DDevice->CreateIndexBuffer
		(
			uIndexDataSize,
			D3DUSAGE_WRITEONLY,
			D3DFMT_INDEX16,
			D3DPOOL_MANAGED,
			&s_pxAABBIndexBuffer,
			0
		);

		// lock the buffer so we can write to it
		unsigned short* pusIndexData;
		if (SUCCEEDED(s_pxAABBIndexBuffer->Lock(0,uIndexDataSize, (void**)&pusIndexData,0)))
		{
			// copy across the indices
			memcpy( pusIndexData, g_ausCubeFaceVertices, uIndexDataSize );

			// writing finished, unlock buffer
			s_pxAABBIndexBuffer->Unlock();
		} else {
			Asura_Assert(0, "Failed to lock index buffer");
		}
		
	}
}

#ifdef ASURA_TOOLS
class Tools_Export_ExportEnvironment_SM3_BufferDesc
{
public:
	Tools_Export_ExportEnvironment_SM3_BufferDesc()
	{
		this->m_uIBSize = 0;
		this->m_uVBSize = 0;
		this->m_uVBSizePre = 0;
		//this->m_uIBSizePre = 0;
	}

public:
	u_int m_uVBSize;
	u_int m_uIBSize;
	
	u_int m_uVBSizePre;
	//u_int m_uIBSizePre;
};

class Tools_Export_ExportEnvironment_SM3_Strip
{
public:
	Tools_Export_ExportEnvironment_SM3_Strip()
	{
		this->m_uBufferIndex = 0;
		this->m_uIBStart = 0;
		this->m_uVBStart = 0;
	}

public:
	u_int m_uBufferIndex;
	u_int m_uVBStart;
	u_int m_uIBStart;
};

void Asura_Direct3D_EnvironmentRenderer::Tools_CreateData( void )
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_EnvironmentRenderer::Tools_CreateData");

	static const u_int uBufferVertexCountMax = 0x10000;
	static const u_int uBufferIndexCountMax = 0x30000;
	
	HRESULT xResult;
	
	Reset();

	// Start the Tools environment processor, and get the meshes as strips sorted by material
	Tools_EnvironmentProcessor::Initialise( false, false );

	const Tools_EnvironmentProcessor_Strip* pxToolsStrips = Tools_EnvironmentProcessor::GetStripsSortedByMaterial();
	const u_int uNumberOfStrips = Tools_EnvironmentProcessor::GetTotalNumberOfStrips();

	Asura_HeapArray< Tools_Export_ExportEnvironment_SM3_Strip> xArray_xStrips;
	Asura_HeapArray< Tools_Export_ExportEnvironment_SM3_BufferDesc> xArray_xBufferDescs;
	
	xArray_xStrips.SetSize( uNumberOfStrips);
	xArray_xBufferDescs.SetSize( uNumberOfStrips);

	// DLH - Find out how many verts are needed per material
	Asura_TemplateHashTree< u_int> xHashTree_uMaterialVertexCount;

	for ( u_int u=0; u<uNumberOfStrips; ++u )
	{
		const Tools_EnvironmentProcessor_Strip& xToolsStrip = pxToolsStrips[u];
		if( 0 == xToolsStrip.m_uNumberOfVertices || 0 == xToolsStrip.m_uNumberOfIndices)
		{
			continue;
		}
		

		u_int uStripVertexCount = xToolsStrip.m_uNumberOfVertices;
		
		Asura_Hash_ID xMaterialHash0 = xToolsStrip.m_auMaterialResponseHashID[0];
		u_int* puMaterialVertexCount = xHashTree_uMaterialVertexCount.GetDataByHash( xMaterialHash0);
		if( NULL != puMaterialVertexCount)
		{
			*puMaterialVertexCount += uStripVertexCount;
		}
		else
		{
			xHashTree_uMaterialVertexCount.AddNode( xMaterialHash0, uStripVertexCount);
		}
	}
	
	// DLH - Place all a Material's verts in one Buffer.
	Asura_TemplateHashTree< u_int> xHashTree_uMaterialBufferStart;
	
	{
		u_int uNodeCount = xHashTree_uMaterialVertexCount.GetNumberOfNodes();
		u_int uBufferStart = 0;
		
		for( u_int uNodeIndex = 0; uNodeIndex < uNodeCount; uNodeIndex += 1)
		{
			const Asura_Hash_ID xMaterialHash0 = xHashTree_uMaterialVertexCount.GetHashByIndex( uNodeIndex);
			u_int* puMateriualVertexCount = xHashTree_uMaterialVertexCount.GetDataByIndex( uNodeIndex);
			Asura_Assert( NULL != puMateriualVertexCount, "This should not be NULL");
			
			if( NULL == puMateriualVertexCount)
			{
				continue;
			}
			
			u_int uMateriualVertexCount = *puMateriualVertexCount;
			u_int uBufferSize = xArray_xBufferDescs.Size();
			for( u_int uBufferIndex = uBufferStart; uBufferIndex < uBufferSize; uBufferIndex += 1)
			{
				Tools_Export_ExportEnvironment_SM3_BufferDesc& xBuffer = xArray_xBufferDescs[ uBufferIndex];
				
				// DLH - The Material wants more than can fit in one buffer
				// DLH - Find first empty buffer and fill from there. Assumed first empty buffer has no filled buffers after it.
				if( uBufferVertexCountMax < uMateriualVertexCount)
				{
					if( 0 != xBuffer.m_uVBSizePre)
					{
						continue;
					}

					xHashTree_uMaterialBufferStart.AddNode( xMaterialHash0, uBufferIndex);
					
					for( u_int uBufferIndex2 = uBufferIndex; 0 < uMateriualVertexCount && uBufferIndex2 < uBufferSize; uBufferIndex2 += 1)
					{
						Tools_Export_ExportEnvironment_SM3_BufferDesc& xBuffer2 = xArray_xBufferDescs[ uBufferIndex2];
						
						Asura_Assertf( 0 == xBuffer2.m_uVBSizePre, ("This buffer already has %d vertices in it.", xBuffer2.m_uVBSizePre));
						
						const u_int uMateriualVertexCountLimited = Asura_Maths::Min( uBufferVertexCountMax, uMateriualVertexCount);
						xBuffer2.m_uVBSizePre += uMateriualVertexCountLimited;
						uMateriualVertexCount -= uMateriualVertexCountLimited;
						
						uBufferStart = uBufferIndex2 + 1;
					}
					break;
				}
				
				const u_int uBufferVBSpace = uBufferVertexCountMax - xBuffer.m_uVBSizePre;
				
				if( uMateriualVertexCount <= uBufferVBSpace)
				{
					xHashTree_uMaterialBufferStart.AddNode( xMaterialHash0, uBufferIndex);
					xBuffer.m_uVBSizePre += uMateriualVertexCount;
					break;
				}
				
				uBufferStart = uBufferIndex + 1;
			}
			
		}
	}
	
	// Count how many invalid strips there are so we can omit them
	u_int uBufferCount = 0;
	u_int uTotalIndices = 0;
		
	for ( u_int u=0; u<uNumberOfStrips; ++u )
	{
		const Tools_EnvironmentProcessor_Strip& xToolsStrip = pxToolsStrips[u];
		if( 0 == xToolsStrip.m_uNumberOfVertices || 0 == xToolsStrip.m_uNumberOfIndices)
		{
			continue;
		}
		
		Tools_Export_ExportEnvironment_SM3_Strip& xBufferStrip = xArray_xStrips[ u];
		
		Asura_Assertf( xToolsStrip.m_uNumberOfIndices == (3 * xToolsStrip.m_uNumberOfTriangles), ("The Number of Indices %d and Triangles %d do not match", xToolsStrip.m_uNumberOfIndices, 3 * xToolsStrip.m_uNumberOfTriangles));
		
		const u_int uIBSize = xToolsStrip.m_uNumberOfIndices;
		
		uTotalIndices += uIBSize;
		
		// DLH - Find Buffer with enough space
		u_int uBufferSize = xArray_xBufferDescs.Size();
		u_int uBufferStart = 0;
		{
			Asura_Hash_ID xMaterialHash0 = xToolsStrip.m_auMaterialResponseHashID[0];
			u_int* puMaterialBufferStart = xHashTree_uMaterialBufferStart.GetDataByHash( xMaterialHash0);
			if( NULL != puMaterialBufferStart)
			{
				uBufferStart = *puMaterialBufferStart;
			}
		}
		for( u_int uBufferIndex = uBufferStart; uBufferIndex < uBufferSize; uBufferIndex += 1)
		{
			Tools_Export_ExportEnvironment_SM3_BufferDesc& xBuffer = xArray_xBufferDescs[ uBufferIndex];
			
			u_int uBufferVBSizeNew = xBuffer.m_uVBSize + xToolsStrip.m_uNumberOfVertices;
			u_int uBufferIBSizeNew = xBuffer.m_uIBSize + uIBSize;
			
			{
				if( uBufferVertexCountMax < uBufferVBSizeNew && 0 != xBuffer.m_uVBSize)
				{
					continue;
				}
				Asura_Assertf( uBufferVBSizeNew <= uBufferVertexCountMax, ("Vertex Count %d too big for Vertex Buffer", uBufferVBSizeNew, uBufferVertexCountMax));
			}
#if 0
			{
				if( uBufferIndexCountMax < uBufferIBSizeNew && 0 != xBuffer.m_uIBSize)
				{
					continue;
				}
				Asura_Assertf( uBufferIBSizeNew <= uBufferIndexCountMax, ("Index Count %d too big for Index Buffer", uBufferIBSizeNew, uBufferIndexCountMax));
			}
#endif
			
			// DLH - Update Buffer Strip
			xBufferStrip.m_uBufferIndex = uBufferIndex;
			xBufferStrip.m_uVBStart = xBuffer.m_uVBSize;
			xBufferStrip.m_uIBStart = xBuffer.m_uIBSize;
			
			// DLH - Update Buffer
			xBuffer.m_uIBSize = uBufferIBSizeNew;
			xBuffer.m_uVBSize = uBufferVBSizeNew;
			
			uBufferCount = Asura_Maths::Max( uBufferCount, uBufferIndex + 1);
			break;
		}
	}

	if( uBufferCount < xArray_xBufferDescs.Size())
	{
		xArray_xBufferDescs.Shrink( uBufferCount);
	}

	u_int uVBCount = xArray_xBufferDescs.Size();
	u_int uIBCount = uVBCount;

	// Allocate space for all the strips
	s_uTotalNumberOfStrips = uNumberOfStrips;
	s_pxStrips = new Asura_Direct3D_EnvironmentRenderer_Strip[ s_uTotalNumberOfStrips ];

	// KJFL - 04 May 2005
	// For ease of coding - create a vertex/index buffer per strip!
	// Later will try to combine based on vertex size etc, upto 65536 indices etc.

	// Allocate array of pointers to vertex buffers & index buffers
	s_uTotalNumberOfIndexBuffers	= uIBCount;
	s_uTotalNumberOfVertexBuffers	= uVBCount;

	s_ppxIndexBuffer				= new Direct3D_IndexBuffer* [s_uTotalNumberOfIndexBuffers];
	s_ppxVertexBuffer				= new Direct3D_VertexBuffer* [s_uTotalNumberOfVertexBuffers];
	
	s_ppusIndexData					= new uint16* [s_uTotalNumberOfIndexBuffers];
	memset( s_ppusIndexData, 0, sizeof( *s_ppusIndexData) * s_uTotalNumberOfIndexBuffers);
	
	s_pusIndexData					= new uint16[ uTotalIndices];
	memset( s_pusIndexData, ~0, sizeof( *s_pusIndexData) * uTotalIndices);
	
	// DLH - Create Vertex and Index Buffers
	for( u_int uVBIndex = 0; uVBIndex < uVBCount; uVBIndex += 1)
	{
		Tools_Export_ExportEnvironment_SM3_BufferDesc& xBuffer = xArray_xBufferDescs[ uVBIndex];
		const u_int uVertexBufferSize	= xBuffer.m_uVBSize * sizeof(Asura_Direct3D_EnvironmentRenderer_Vertex);
		
		xResult = Asura_Direct3D::D3DDevice->CreateVertexBuffer
		( 
			uVertexBufferSize,
			D3DUSAGE_WRITEONLY,
			0,
			D3DPOOL_MANAGED,
			&s_ppxVertexBuffer[ uVBIndex],
			0
		);
		ASURA_D3D_OK( xResult);
	}

	{
		uint16* pusIndexData = s_pusIndexData;
		for( u_int uIBIndex = 0; uIBIndex < uIBCount; uIBIndex += 1)
		{
			Tools_Export_ExportEnvironment_SM3_BufferDesc& xBuffer = xArray_xBufferDescs[ uIBIndex];
			const u_int uIndexBufferSize = xBuffer.m_uIBSize * sizeof( uint16);
			
			xResult = Asura_Direct3D::D3DDevice->CreateIndexBuffer
			(
				uIndexBufferSize,
				D3DUSAGE_WRITEONLY,
				D3DFMT_INDEX16,
				D3DPOOL_MANAGED,
				&s_ppxIndexBuffer[ uIBIndex],
				0
			);
			ASURA_D3D_OK( xResult);
			
			s_ppusIndexData[ uIBIndex] = pusIndexData;
			pusIndexData += xBuffer.m_uIBSize;
		}
	}

	// VISIBILITY
	Asura_Bounding_Box* pxVisibilityBBs = new Asura_Bounding_Box[ s_uTotalNumberOfStrips ];


	// scan through each strip, and set up data
	for ( u_int u=0; u<uNumberOfStrips; ++u )
	{
		const Tools_EnvironmentProcessor_Strip& xToolsStrip = pxToolsStrips[u];

		Asura_Direct3D_EnvironmentRenderer_Strip& xStrip = s_pxStrips[u];
		Tools_Export_ExportEnvironment_SM3_Strip& xBufferStrip = xArray_xStrips[ u];

		pxVisibilityBBs[u].Reset();


		// set basic properties of the strip: material, module & number of tris
		xStrip.m_auMaterialResponseHashID[0] = xToolsStrip.m_auMaterialResponseHashID[0];
		xStrip.m_auMaterialResponseHashID[1] = xToolsStrip.m_auMaterialResponseHashID[1];
		

		xStrip.m_uModule = xToolsStrip.m_uModule;
		xStrip.m_uNumberOfPrimitives = xToolsStrip.m_uNumberOfTriangles;
		xStrip.m_uNumberOfVertices = xToolsStrip.m_uNumberOfVertices;

		// set vertex & index buffer indices, and the offsets into those buffers
		xStrip.m_uStartVertex = xBufferStrip.m_uVBStart;
		xStrip.m_uVertexBuffer = xBufferStrip.m_uBufferIndex; 

		xStrip.m_uStartIndex = xBufferStrip.m_uIBStart;
		xStrip.m_uIndexBuffer = xBufferStrip.m_uBufferIndex;

		xStrip.m_xBB.Reset();

		// Do we have any vertices?

		if (xToolsStrip.m_uNumberOfVertices > 0)
		{
			const u_int									uNumVertices		= xToolsStrip.m_uNumberOfVertices;

			//const u_int									uVertexBufferSize	= uNumVertices * sizeof(Asura_Direct3D_EnvironmentRenderer_Vertex);

			Asura_Direct3D_EnvironmentRenderer_Vertex*	pxVertexDataBase = NULL;

			// ...and lock them so we can write to them...

			Direct3D_VertexBuffer* pxVB = s_ppxVertexBuffer[ xStrip.m_uVertexBuffer];

			if( NULL != pxVB)
			{
				xResult = pxVB->Lock( 0, 0,	reinterpret_cast<void**>(&pxVertexDataBase), 0);
				if( ASURA_D3D_OK( xResult))
				{
					Asura_Direct3D_EnvironmentRenderer_Vertex*	pxVertexData = pxVertexDataBase + xStrip.m_uStartVertex;


					// Copy across the vertex data...

					for (u_int v = 0; v < uNumVertices; v++)
					{
						const Tools_EnvironmentProcessor_Vertex&	xToolsVertex	= xToolsStrip.m_pxVertices [v];


						// Set the position data for this vertex...

						pxVertexData->SetPosition	(xToolsVertex.m_xPosition);


						// Set the diffuse colour and texture co-ordinate data for this vertex...

						pxVertexData->SetDiffuseRGBA	(xToolsVertex.m_uDiffuseRGBA);

						pxVertexData->SetTextureCoord	(0,	xToolsVertex.m_fU,	xToolsVertex.m_fV);
						pxVertexData->SetTextureCoord	(1,	xToolsVertex.m_fU2,	xToolsVertex.m_fV2);


						// Set the tangent-space data for this vertex...

						pxVertexData->SetNormal		(xToolsVertex.m_xNormal);
						pxVertexData->SetTangentBinormal	(xToolsVertex.m_xTangent, xToolsVertex.m_xBinormal);
						pxVertexData->SetTangentBinormal2	(xToolsVertex.m_xTangent2, xToolsVertex.m_xBinormal2);


						// Set the spherical harmonic lighting data...

						pxVertexData->SetDiffuseSH_Red		(xToolsVertex.m_xDiffuseSH.m_afRedCoefficients);
						pxVertexData->SetDiffuseSH_Green	(xToolsVertex.m_xDiffuseSH.m_afGreenCoefficients);
						pxVertexData->SetDiffuseSH_Blue		(xToolsVertex.m_xDiffuseSH.m_afBlueCoefficients);


						// Step onto the next vertex...

						pxVertexData++;



						// Set the visibility bounding-box of this strip...

						pxVisibilityBBs [u].Set	(xToolsVertex.m_xPosition);

						xStrip.m_xBB = pxVisibilityBBs[u];
					}

					
					// Writing finished, unlock the buffer...
					pxVB->Unlock	();
				}
			}

		}

		// Do we have any indices?

		if ( xToolsStrip.m_uNumberOfIndices )
		{
			// lock the buffer so we can write to it
			Direct3D_IndexBuffer* pxIB = s_ppxIndexBuffer[ xStrip.m_uIndexBuffer];
			
			if( NULL != pxIB)
			{
				uint16* pusIndexBufferBase = NULL;
				xResult = pxIB->Lock
				(
					0,
					0, 
					(void**)&pusIndexBufferBase,
					0
				);
				
				if( ASURA_D3D_OK( xResult))
				{
					
					uint16* pusIndexBuffer = pusIndexBufferBase + xStrip.m_uStartIndex;
					uint16* pusIndexData = s_ppusIndexData[ xStrip.m_uIndexBuffer] + xStrip.m_uStartIndex;
					
					const u_int uVertexIndexOffset = xStrip.m_uStartVertex;
					
					// copy across the indices
					for ( u_int v=0; v<xToolsStrip.m_uNumberOfIndices; ++v )
					{
						u_int uIndex = xToolsStrip.m_pusIndices[v] + uVertexIndexOffset;
						pusIndexData[v] = pusIndexBuffer[v] = (uint16)uIndex;
					}
					
					// writing finished, unlock buffer
					pxIB->Unlock();
				}
			}
		}
	}

	//s_bUsingPreloadedData = true;

	Tools_EnvironmentProcessor::Shutdown();

	Asura_Vector_3* pxVisibilityBBCentres = new Asura_Vector_3[s_uTotalNumberOfStrips];

	for ( u_int u=0; u<uNumberOfStrips; ++u )
	{
		pxVisibilityBBCentres[u] = pxVisibilityBBs[u].CalculateCentre();
	}

//	Tools_LBAABB_Tree::CreateRunTimeTree( g_pxVisibilityTree, s_uTotalNumberOfStrips, pxVisibilityBBs, pxVisibilityBBCentres );

	delete [] pxVisibilityBBCentres;
	delete [] pxVisibilityBBs;
	

	s_bVertexAndIndexBuffersCreatedInternally = true;
	s_bStripsValid = true;
}


// class to hold data about strips and their materials, an array of which is to be sorted
struct Tools_StripSortDesc
{
	public:

		// callback for qsort
		static int SortCallback( const void* p1, const void* p2 )
		{
			const Tools_StripSortDesc *pxStrip1 = (const Tools_StripSortDesc*)p1;
			const Tools_StripSortDesc *pxStrip2 = (const Tools_StripSortDesc*)p2;

			for(int iMatSlot=0; iMatSlot<TOOLS_NUM_MATERIALRESPONSE_PER_POLY; iMatSlot++)
			{
				int iCompare = Tools_MaterialResponse_System::CompareMaterials( pxStrip1->m_auMaterialResponseHashID[iMatSlot], pxStrip2->m_auMaterialResponseHashID[iMatSlot] );
				if(iCompare) return iCompare;
			}
			return 0;
		}

		Asura_Hash_ID	m_auMaterialResponseHashID[TOOLS_NUM_MATERIALRESPONSE_PER_POLY];
		u_int			m_uModule;
		u_int			m_uPacket;
};


void Asura_Direct3D_EnvironmentRenderer::Tools_CreateData( u_int uNumberOfModules )
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_EnvironmentRenderer::Tools_CreateData( u_int uNumberOfModules )" );

	// Release any existing data
	Reset();
	
	
	// find out how many strips there are
	s_uTotalNumberOfStrips = 0;

	for ( u_int u=0; u<uNumberOfModules; ++u )
	{
		const Tools_RenderModule& xModule = Tools_RenderModule_System::GetRenderModule( u );
		s_uTotalNumberOfStrips += xModule.GetNumberOfPackets();
	}


	// We want to order the strips based on their materials
	
	Tools_StripSortDesc* pxSortedStrips = new Tools_StripSortDesc[ s_uTotalNumberOfStrips ];

	// Want to know how many indices there are total
	
	u_int uTotalIndexCount = 0;
	
	// scan through each module, and set up sorted strip data
	{
		u_int uStrip = 0;
	
		for ( u_int u=0; u<uNumberOfModules; ++u )
		{
			const Tools_RenderModule& xModule = Tools_RenderModule_System::GetRenderModule( u );

			for ( u_int v=0; v<xModule.GetNumberOfPackets(); ++v )
			{
				const Tools_RenderModule_Polygon_Packet& xPacket = xModule.GetPacketList()[v];

				Tools_StripSortDesc &xSortedStrip = pxSortedStrips[ uStrip ];
				for(int iMatSlot=0; iMatSlot<TOOLS_NUM_MATERIALRESPONSE_PER_POLY; iMatSlot++)
				{
					const Asura_Hash_ID xHashID = xPacket.m_auMaterialResponseHashID[iMatSlot];
					xSortedStrip.m_auMaterialResponseHashID[iMatSlot] = xHashID;
#if defined( ASURA_ASSERT)
					if(xHashID)
					{
						const Tools_MaterialResponse* const pxMaterial = Tools_MaterialResponse_System::GetMaterialByHashID( xHashID );
						if( NULL == pxMaterial)
						{
							const Asura_Mesh* pxModuleMesh = Asura_Module_System::GetMeshForModule( u);
							const char* pszModuleName = (NULL != pxModuleMesh) ? pxModuleMesh->Name.GetString() : "UNKNOWN";
							#ifdef ASURA_TOOLS
							Asura_Assertf( false, ("Material %s is missing on module '%s'", Tools_MaterialResponse_System::GetMissingMaterialDescription( xHashID), pszModuleName));
							#else
							Asura_Assertf( false, ("Material (0x%08X) is missing on module '%s'", u_int( xHashID), pszModuleName));
							#endif
						}
					}
#endif
				}
				xSortedStrip.m_uModule						= u;
				xSortedStrip.m_uPacket						= v;
				
				uTotalIndexCount += xPacket.m_uNumberOfIndices;

				++uStrip;
				
			}
		}
	}

	// Now sort. After this we can use the pxSortedStrips array to find the Nth strip
	qsort( pxSortedStrips, s_uTotalNumberOfStrips, sizeof(Tools_StripSortDesc), Tools_StripSortDesc::SortCallback );

	
	// Allocate space for all the strips
	s_pxStrips = new Asura_Direct3D_EnvironmentRenderer_Strip[ s_uTotalNumberOfStrips ];

	
	// Allocate array of pointers to vertex buffers & index buffers
	s_uTotalNumberOfIndexBuffers	= uNumberOfModules;
	s_uTotalNumberOfVertexBuffers	= uNumberOfModules;

	s_ppxIndexBuffer				= new Direct3D_IndexBuffer* [s_uTotalNumberOfIndexBuffers];

	s_ppxVertexBuffer				= new Direct3D_VertexBuffer* [s_uTotalNumberOfVertexBuffers];

	// Allocate index Data memory
	s_ppusIndexData					= new uint16*[ s_uTotalNumberOfIndexBuffers];
	memset( Asura_Direct3D_EnvironmentRenderer::s_ppusIndexData, 0, sizeof(uint16*) * s_uTotalNumberOfIndexBuffers);

	// VISIBILITY
	Asura_Bounding_Box* pxVisibilityBBs = new Asura_Bounding_Box[ s_uTotalNumberOfStrips ];
	Asura_Vector_3* pxVisibilityBBCentres = new Asura_Vector_3[s_uTotalNumberOfStrips];


	// scan through each strip, setting vertex/index data & BBs for the visiblity system

	for ( u_int u=0; u<s_uTotalNumberOfStrips; ++u )
	{
        const Tools_StripSortDesc& xNextSortedStrip = pxSortedStrips[u];

		const Tools_RenderModule& xModule = Tools_RenderModule_System::GetRenderModule( xNextSortedStrip.m_uModule );
		const Tools_RenderModule_Polygon_Packet& xPacket = xModule.GetPacketList()[ xNextSortedStrip.m_uPacket ];

		
		Asura_Direct3D_EnvironmentRenderer_Strip& xStrip = s_pxStrips[ u ];

		// set basic properties of the strip: material, module & number of tris
		xStrip.m_auMaterialResponseHashID[0] = xPacket.m_auMaterialResponseHashID[0];
		xStrip.m_auMaterialResponseHashID[1] = xPacket.m_auMaterialResponseHashID[1];
		
		// module - will remove soon
		xStrip.m_uModule = xNextSortedStrip.m_uModule;
		
		xStrip.m_uNumberOfPrimitives = xPacket.m_uNumberOfIndices / 3;
		xStrip.m_uNumberOfVertices = xPacket.m_uEndVertex;

		// set vertex & index buffer indices
		xStrip.m_uVertexBuffer = xNextSortedStrip.m_uModule; 
		xStrip.m_uIndexBuffer = xNextSortedStrip.m_uModule;

		// set start index / vertex
		xStrip.m_uStartVertex = xPacket.m_uStartVertex;
		xStrip.m_uStartIndex  = xPacket.m_uStartIndex;

		// bounding box
		xStrip.m_xBB = xPacket.m_xBB;

		pxVisibilityBBs[u] = xPacket.m_xBB;
		pxVisibilityBBCentres[u] = pxVisibilityBBs[u].CalculateCentre();

		s_ppxVertexBuffer [xStrip.m_uVertexBuffer]				= xModule.GetVertexBuffer ();

		s_ppxIndexBuffer [xStrip.m_uIndexBuffer]				= xModule.GetIndexBuffer ();
		s_ppusIndexData [xStrip.m_uIndexBuffer]					= xModule.GetIndexBufferData ();
	}


//	Tools_LBAABB_Tree::CreateRunTimeTree( g_pxVisibilityTree, s_uTotalNumberOfStrips, pxVisibilityBBs, pxVisibilityBBCentres );

	delete [] pxVisibilityBBCentres;
	delete [] pxVisibilityBBs;

	delete [] pxSortedStrips;
	
	s_bVertexAndIndexBuffersCreatedInternally = false;
	s_bStripsValid = true;
}

#endif // defined ASURA_TOOLS

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_EnvironmentRenderer::SetStreams
// Author       : Andy Tate
// Date         : 7-Jun-2005
//
// Description  : Sets the appropriate vertex/index streams for the specified
//				  environment strip.
//
// Parameters   : A reference to the strip to render.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_EnvironmentRenderer::SetStreams (const Asura_Direct3D_EnvironmentRenderer_Strip& xStrip)
{
	const u_int	uVertexBuffer	= xStrip.m_uVertexBuffer;


	// Set the vertex-buffer streams and the index buffer...

	Asura_Direct3D::D3DDevice->SetStreamSource	(0,	s_ppxVertexBuffer [uVertexBuffer],				0,	sizeof(Asura_Direct3D_EnvironmentRenderer_Vertex));

	Asura_Direct3D::D3DDevice->SetIndices		(s_ppxIndexBuffer [uVertexBuffer]);
}

/// End of function 'Asura_Direct3D_EnvironmentRenderer::SetStreams'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Direct3D_EnvironmentRenderer::SetStreams
// Author       : Andy Tate
// Date         : 7-Jun-2005
//
// Description  : Sets the appropriate vertex/index streams for the specified
//				  Direct3D effect.
//
// Parameters   : A reference to the effect to use, along with pointers to the
//				  vertex and index buffers and the vertex buffer stride.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Asura_Direct3D_EnvironmentRenderer::SetStreams (const Asura_Direct3D_Effect& xEffect,
													 Direct3D_VertexBuffer* pxVertexBuffer,
													 const u_int uVertexStride,
													 Direct3D_IndexBuffer* pxIndexBuffer)
{
	// Set the vertex-buffer streams and the index buffer...

	Asura_Direct3D::D3DDevice->SetStreamSource	(0,	pxVertexBuffer,	0,	uVertexStride);
	Asura_Direct3D::D3DDevice->SetIndices		(pxIndexBuffer);
}

/// End of function 'Asura_Direct3D_EnvironmentRenderer::SetStreams'. ///


u_int Asura_Environment_Renderer::Platform_GetNumberOfStrips( void )
{
	return Asura_Direct3D_EnvironmentRenderer::s_uTotalNumberOfStrips;
}

void Asura_Environment_Renderer::Platform_PrefetchStrip(u_int uStrip)
{
	const Asura_Direct3D_EnvironmentRenderer_Strip&	xStrip	= Asura_Direct3D_EnvironmentRenderer::s_pxStrips[uStrip];

	Asura_Mem::PreFetchData(&xStrip, sizeof(xStrip));
}

const Asura_Bounding_Box& Asura_Environment_Renderer::Platform_GetBoundingBox( u_int uStrip )
{
	return Asura_Direct3D_EnvironmentRenderer::s_pxStrips[uStrip].m_xBB;
}

const Asura_MaterialResponse* Asura_Environment_Renderer::Platform_GetMaterial( u_int uStrip, u_int uPass )
{
	const Asura_Hash_ID uMaterialHash = Asura_Direct3D_EnvironmentRenderer::s_pxStrips[uStrip].m_auMaterialResponseHashID[ uPass ];
	const Asura_MaterialResponse* pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash );

	return pxMaterial;
}

u_int Asura_Environment_Renderer::Platform_GetModuleIndex( u_int uStrip )
{
	return Asura_Direct3D_EnvironmentRenderer::s_pxStrips[uStrip].m_uModule;
}

#endif	// defined(ASURA_TARGET_SM3)
/// End of file. ///
