//
// Asura_Direct3D_Fog.cpp
//
// Created by RJF - 18 August 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined( ASURA_TARGET_SM3)

// This file's header.
#include "Asura_Direct3D_Fog.h"

// Asura Independent
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Render.h"

// Asura PC
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Direct3D_EffectParameterBlock.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_StoreRenderTargets.h"

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

const u_int uFOG_TEXTURE_SIZE = 128;


const Direct3D_VertexElement axVERTEX_DECLARATION[] =
{
	{ 0, 0, D3DDECLTYPE_SHORT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
		D3DDECL_END()
};

const u_int uVERTEX_SIZE = 4;

const short asFOG_VERTICES[] =
{
	-1,  1,
	 1,  1,
	 1, -1,
	-1, -1,
};



/******************************************************************************/
/* Global Variables                                                           */
/******************************************************************************/


static Direct3D_VertexBuffer* s_pxFogVB = NULL;
static Direct3D_VertexDeclaration* s_pxFogVD = NULL;

/******************************************************************************/
/* Asura_Direct3D_Fog Static Class Functions                                  */
/******************************************************************************/

Asura_Direct3D_Effect*				 Asura_Direct3D_Fog::s_pxEffect		= NULL;
Asura_Direct3D_EffectParameterBlock* Asura_Direct3D_Fog::s_pxParamBlock = NULL;
Direct3D_Texture*					 Asura_Direct3D_Fog::s_pxFogTexture = NULL;

void Asura_Direct3D_Fog::Initialise()
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}

	s_pxEffect = Asura_Direct3D_Effects_System::GetEffect("Fog");
	Asura_Assert(s_pxEffect, "Could not find fog shader");
	if(!s_pxEffect) return;

	UpdateForNewParameters();

	// Create vertex buffer for fog rendering
	Asura_Direct3D::D3DDevice->CreateVertexBuffer(sizeof( asFOG_VERTICES ), D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, &s_pxFogVB, NULL);

	short* psVertices; 
	if (SUCCEEDED(s_pxFogVB->Lock(0, 0, reinterpret_cast<void**>(&psVertices), 0)))
	{
		memcpy( psVertices, asFOG_VERTICES, sizeof( asFOG_VERTICES ) );
		s_pxFogVB->Unlock();
	} else {
		Asura_Assert(0, "Failed to lock vertex buffer");
	}

	// Create vertex declaration
	Asura_Direct3D::D3DDevice->CreateVertexDeclaration( axVERTEX_DECLARATION, &s_pxFogVD );
	Asura_Assert( s_pxFogVD, "Unable to create fog vertex declaration." );	
}	// End Initialise fn.

//---------------------------------------------------------

void Asura_Direct3D_Fog::UpdateForNewParameters()
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}
	
	
	// Now update the lookup texture

	if (s_pxFogTexture==NULL)
	{
#ifdef ASURA_ASSERT
		HRESULT hr = 
#endif // #ifdef ASURA_ASSERT
			Asura_Direct3D::D3DDevice->CreateTexture(uFOG_TEXTURE_SIZE, 1, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &s_pxFogTexture, NULL);

		Asura_Assert(hr==D3D_OK, "Failed to create fog lookup texture!");
		
	}

	if (!s_pxFogTexture )
	{
		return;
	}

	SCOPEDPIXEVENT(( 0, L"Update fog" ));

	
	D3DLOCKED_RECT xLockedRect = {0};
	s_pxFogTexture->LockRect( 0, &xLockedRect, NULL, 0 );
	Asura_Assert( xLockedRect.pBits, "Failed to lock fog lookup texture." );

	uint32* const puTexels = static_cast< uint32* >( xLockedRect.pBits );

	if (puTexels)
	{	
		float fDistance = Asura_Fog::GetFogNearPlane();
		const float fFogFarPlane =   Asura_Fog::GetFogFarPlane();
		const float fDelta = (fFogFarPlane - Asura_Fog::GetFogNearPlane()) / static_cast< float >( uFOG_TEXTURE_SIZE - 1 );

		for( u_int u = 0; u < uFOG_TEXTURE_SIZE; ++u )
		{
			// Need to clamp this, as peaks in the graph can exceed 1 / drop below 0
			uint32 uARGB = (static_cast< uint32 >( Asura_Maths::ClampToRange(Asura_Fog::GetFogDensityAtDistance( fDistance ), 0.0f, 1.0f) * 255.0f ))<<24;
			puTexels[u] = uARGB;
			fDistance += fDelta;
		}

		s_pxFogTexture->UnlockRect(0);
	}


	// Then (re)create the parameter block
	//
	// Probably not the best way to do this... but that's what it was doing already, so I'm not changing it for now...
	// If we're updating fog parameters per-frame (fading between fog setups), we probably want to avoid the 
	// deleting/recreating/searching-by-name here...

	delete s_pxParamBlock;

	s_pxParamBlock = new Asura_Direct3D_EffectParameterBlock();

	Asura_Direct3D_RenderTarget* pxRenderTarget = Asura_Direct3D::GetDeferredRenderingTarget();
	float fRenderTargetDimensions[2] = {(float)pxRenderTarget->GetWidth(), (float)pxRenderTarget->GetHeight()};

	s_pxParamBlock->AddStaticParameter("g_xFogColour",				Asura_Fog::GetFogColour());
	s_pxParamBlock->AddStaticParameter("g_fFogNearPlane",			Asura_Fog::GetFogNearPlane());
	s_pxParamBlock->AddStaticParameter("g_fOneOverFogVolumeDepth",	Asura_Fog::GetOneOverFogPlaneSeparation());
	s_pxParamBlock->AddStaticParameter("g_fDepthTextureDimensions",	fRenderTargetDimensions, 2);
	s_pxParamBlock->AddStaticParameter("g_fFarClipPlane",			Asura_Camera::GetFarClip());
	s_pxParamBlock->AddStaticParameter("g_xFogTexture",				GetTexture());

	if(s_pxEffect)
	{
		s_pxParamBlock->Bind(s_pxEffect);
	}
}	// End Update for New Parameters fn.

//---------------------------------------------------------

void Asura_Direct3D_Fog::Shutdown()
{
	if (s_pxFogTexture)
	{
		s_pxFogTexture->Release();
		s_pxFogTexture=NULL;
	}

	delete s_pxParamBlock;
	s_pxParamBlock = NULL;

	if(s_pxFogVD)
	{
		s_pxFogVD->Release(); 
		s_pxFogVD = NULL;
	}

	if(s_pxFogVB)
	{
		s_pxFogVB->Release(); 
		s_pxFogVB = NULL;
	}
}	// End Shutdown fn.

//---------------------------------------------------------

void Asura_Direct3D_Fog::Reset()
{
	// MGH 17/1/2008: Delete this between levels to avoid memory fragmentation issues.
	delete s_pxParamBlock;
	s_pxParamBlock = NULL;

	// (SJB) Restart the system, so it's in the same state as it was at the end of Initialise ().
	UpdateForNewParameters ();
	
}	// End Reset fn.

//---------------------------------------------------------

void Asura_Direct3D_Fog::Render()
{
	if( !Asura_Fog::IsFogRenderable())
	{
		return;
	}

	if (!s_pxFogTexture )
	{
		return;
	}

	if (!s_pxParamBlock)
	{
		Asura_Assert(0, "No fog effect parameter block.");
		return;
	}

	SCOPEDPIXEVENT((0xff888888, L"Fog"));

	Asura_Direct3D_StoreRenderTargets xStoreRenderTargets;
	
	xStoreRenderTargets.StoreRenderTargets();

	Asura_Direct3D_RenderTarget* pxRenderTarget = Asura_Direct3D::GetDeferredRenderingTarget();
	
	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestZBufferEnabled( false );
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );
	Asura_Render::CurrentStates.RequestStencilEnabled( false );

	// (AAM - 01.09.2006) Need to preserve these, they're set in the effect
	DWORD uMinFilter, uMagFilter, uMipFilter, uAddressU, uAddressV;
	{
		SCOPEDPIXEVENT(( 0xFF888888, L"Preserve sampler state" ));

		Asura_Direct3D::D3DDevice->GetSamplerState( 0, D3DSAMP_MINFILTER, &uMinFilter );
		Asura_Direct3D::D3DDevice->GetSamplerState( 0, D3DSAMP_MAGFILTER, &uMagFilter );
		Asura_Direct3D::D3DDevice->GetSamplerState( 0, D3DSAMP_MIPFILTER, &uMipFilter );
		Asura_Direct3D::D3DDevice->GetSamplerState( 0, D3DSAMP_ADDRESSU, &uAddressU );
		Asura_Direct3D::D3DDevice->GetSamplerState( 0, D3DSAMP_ADDRESSV, &uAddressV );
	}

	s_pxEffect->Begin();
	if(!s_pxEffect->IsD3DEffectValid()) return;

	if(!s_pxParamBlock->IsBound()) s_pxParamBlock->Bind(s_pxEffect);

	{
		SCOPEDPIXEVENT(( 0xFF888888, L"Set vertices and declaration" ));

		Asura_Direct3D::D3DDevice->SetStreamSource(0, s_pxFogVB, 0, sizeof(short) * 2);
		Asura_Direct3D::D3DDevice->SetVertexDeclaration(s_pxFogVD);
	}

	s_pxParamBlock->Apply();
	
	Direct3D_Surface* pxRTFrontSurface = pxRenderTarget->GetSurface( ASURA_DIRECT3D_DEFFEDRENDERTARGET_FRONT_INDEX);

	Asura_Direct3D::ClearRenderTargets( 1);
	Asura_Direct3D::D3DDevice->SetRenderTarget( 0, pxRTFrontSurface);
	
	s_pxEffect->SetParameterTextureByHash("g_xDepthTexture", 0x73BB9798, pxRenderTarget->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX));

	s_pxEffect->GetD3DEffect()->BeginPass(0);
	Asura_Direct3D::D3DDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);
	s_pxEffect->GetD3DEffect()->EndPass();

	s_pxEffect->End();
	Asura_Direct3D_Effects_System::FlushEffects	();
	s_pxEffect->SetParameterTextureByHash("g_xDepthTexture", 0x73BB9798, NULL);
	s_pxEffect->GetD3DEffect()->CommitChanges();

	{
		SCOPEDPIXEVENT(( 0xFF888888, L"Restore sampler state" ));

		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MINFILTER, uMinFilter );
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, uMagFilter );
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MIPFILTER, uMipFilter );
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_ADDRESSU, uAddressU );
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_ADDRESSV, uAddressV );
	}

	// (AAM - 23.10.2006) Need to invalidate the cached texture state, since
	// the effect will have overwritten it
	Asura_Render::CurrentStates.RequestTextureIndex( 0, ASURA_TEXTURE_INVALID );
	xStoreRenderTargets.RestoreRenderTargets();
}






// Asura_Direct3D_FogEffectParameters class
// Used for adding fogging to existing shaders, using fog.fxh. Currently used for PFX

void Asura_Direct3D_FogEffectParameters::Apply(bool bEnabled, ASURA_TRANSLUCENCY_MODE eTranslucency)
{
	if (m_pxEffect)
	{		
		if (!bEnabled)
		{
			m_pxEffect->GetD3DEffect()->SetBool(m_xFogEnabledHandle, bEnabled);
		}
		else
		{
			m_pxEffect->GetD3DEffect()->SetBool(m_xFogEnabledHandle, Asura_Fog::IsFogEnabled());

			if (Asura_Fog::IsFogEnabled())
			{
				Direct3D_Effect *pxD3DEffect = m_pxEffect->GetD3DEffect();

				pxD3DEffect->SetTexture(m_xTextureHandle, Asura_Direct3D_Fog::GetTexture());

				const D3DXVECTOR4 xParams(Asura_Fog::GetFogNearPlane(),
					Asura_Fog::GetOneOverFogPlaneSeparation(), 0.0f, 0.0f);

				pxD3DEffect->SetVector(m_xFogParamsHandle, &xParams);

				D3DXVECTOR4 xCol;

				switch (eTranslucency)
				{
				case ASURA_TRANSLUCENCY_GLOWING:
				case ASURA_TRANSLUCENCY_INVCOLOUR:
					// Additive-blended materials fog out to black, not the fog colour
					xCol.x = xCol.y = xCol.z = 0;
					break;

				default:
					xCol.x = Asura_Fog::GetFogColour().m_fR;
					xCol.y = Asura_Fog::GetFogColour().m_fG;
					xCol.z = Asura_Fog::GetFogColour().m_fB;
					break;
				}

				pxD3DEffect->SetVector(m_xColourHandle, &xCol);					
			}
		}			
	}
};


void Asura_Direct3D_FogEffectParameters::Capture(Asura_Direct3D_Effect *pxEffect)
{
	m_pxEffect = pxEffect;

	if (pxEffect && pxEffect->GetD3DEffect())
	{
		Direct3D_Effect* const pxD3DEffect = pxEffect->GetD3DEffect();

		m_xFogEnabledHandle = pxD3DEffect->GetParameterByName( NULL, "g_bFogEnabled" );
		m_xTextureHandle    = pxD3DEffect->GetParameterByName( NULL, "g_xFogTexture" );
		m_xFogParamsHandle  = pxD3DEffect->GetParameterByName( NULL, "g_xFogParams" );
		m_xColourHandle     = pxD3DEffect->GetParameterByName( NULL, "g_xFogColour" );
	}
};


#endif //defined( ASURA_TARGET_SM3)
