//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE        :  Asura_Direct3D_RenderTargetManager.h
//
//  CREATED     :  Richard J. Fine  (24.3.2005)
//
//  Copyright (C) 2005 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_DIRECT3D_RENDERTARGETMANAGER_H__
#define __ASURA_DIRECT3D_RENDERTARGETMANAGER_H__

#include "Rendering/Asura_Direct3D.h"

#include "Core/Asura_LinkList.h"

class Asura_Direct3D_RenderTargetManager;

class Asura_Direct3D_RenderTarget
{
	friend class Asura_Direct3D_RenderTargetManager;

public:
	inline Direct3D_Surface* GetSurface(u_int uRTIndex = 0) { return m_apxSurfaces[uRTIndex]; }
	inline Direct3D_Texture* GetTexture(u_int uRTIndex = 0) { return m_apxTextures[uRTIndex]; }
	inline Direct3D_Surface* GetDepthStencilSurface() { return m_pxDepthStencilSurface; }
	
	inline u_int GetMRTCount() { return m_uNumRenderTargets; }

	inline u_int GetWidth() { return m_uActualWidth; }
	inline u_int GetHeight() { return m_uActualHeight; }

	void Bind();
		

	inline void MarkAsUnused(void)
	{
		Asura_Assert(this->m_bIsInUse, "Attempting to release render target that isn't in use!");
		this->m_bIsInUse = false;
	}

	
protected:
	static const u_int s_uMAX_NUM_RENDER_TARGETS = 4;

	Direct3D_Surface* m_apxSurfaces[s_uMAX_NUM_RENDER_TARGETS];
	Direct3D_Texture* m_apxTextures[s_uMAX_NUM_RENDER_TARGETS];

	Direct3D_Surface* m_pxDepthStencilSurface;
	u_int m_uActualWidth, m_uActualHeight;

	u_int m_uNumRenderTargets;

	float m_fWidth;
	float m_fHeight;

	D3DFORMAT m_axColourFmt[ s_uMAX_NUM_RENDER_TARGETS ];
	D3DFORMAT m_xDepthFmt;
	bool m_bBackbufferRelativeSize;

	// DLH - 17 July 2006 12:07:56 - Use this to see if some code has a RT in use. 
	bool m_bIsInUse;

	// (AS - 12/10/2007) Added this to avoid creating a new depthstencil buffer when the default one works fine
	// if set to true then the depth buffer is set to be whatever is the currently asisgned one at time of creation
	bool m_bIsUsingExistingDepthStencilBuffer;

	Asura_Direct3D_RenderTarget() : m_pxDepthStencilSurface(0), 
									m_uActualWidth(0), 
									m_uActualHeight(0), 
									m_uNumRenderTargets(0),
									m_fWidth(0),
									m_fHeight(0),
									m_bBackbufferRelativeSize(false),
									m_bIsInUse(false),
									m_bIsUsingExistingDepthStencilBuffer(false)
	{
		Clear();
	}
	~Asura_Direct3D_RenderTarget()
	{
		ReleaseD3DResources();
	}

	// Clear the arrays of pointers.
	void Clear();

	HRESULT CreateD3DResources();
	void ReleaseD3DResources();
};

class Asura_Direct3D_RenderTargetManager
{
public:
	
	static void Initialise();
	static void Reset();
	static void Shutdown();

	static void PreD3DReset();
	static void PostD3DReset();

	static Asura_Direct3D_RenderTarget* CreateRenderTarget(float fWidth, float fHeight, D3DFORMAT colourFmt, D3DFORMAT depthFmt, bool bBackbufferRelativeSize, u_int uRTCount = 1, bool bUseExistingDepthStencilBuffer = false );
	static Asura_Direct3D_RenderTarget* CreateRenderTarget(float fWidth, float fHeight, D3DFORMAT* peColourFmt, D3DFORMAT depthFmt, bool bBackbufferRelativeSize, u_int uRTCount, bool bUseExistingDepthStencilBuffer = false);
	static void DestroyRenderTarget(Asura_Direct3D_RenderTarget* pxRenderTarget);

	typedef void (*ResetCallback)();
	static void AddResetCallbacks(ResetCallback pfnPreReset, ResetCallback pfnPostReset);
	static void RemoveResetCallbacks(ResetCallback pfnPreReset, ResetCallback pfnPostReset);
	
	static Asura_Direct3D_RenderTarget* FindUnusedRenderTarget( float fWidth, float fHeight, D3DFORMAT colourFmt, D3DFORMAT depthFmt, bool bBackbufferRelativeSize, u_int uRTCount = 1, bool bAcceptLargerDimensions = true);

private:
	typedef Asura_LinkList<Asura_Direct3D_RenderTarget*> RenderTargetList;
	typedef Asura_ListElt<Asura_Direct3D_RenderTarget*> RenderTargetListElt;

	static RenderTargetList s_xRenderTargetList;

	static Asura_LinkList<ResetCallback> s_xPreResetCallbacks;
	static Asura_LinkList<ResetCallback> s_xPostResetCallbacks;
};

#endif//__ASURA_DIRECT3D_RENDERTARGETMANAGER_H__
