//////////////////////////////////////////////////////////////////////////////
//
// Asura_Direct3D_SkinRenderer.h
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#if !defined __ASURA_DIRECT3D_SKINRENDERER_H__
#define __ASURA_DIRECT3D_SKINRENDERER_H__

#if defined(ASURA_TARGET_SM3)

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Rendering/Asura_Direct3D.h"

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_Animation;
class Asura_Hierarchy_Skin;
class Asura_Shadow;

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Direct3D_SkinStrip
//
// Holds info for a strip in a skin
//
////////////////////////////////////////////////////////////////////////////////

struct Asura_Direct3D_SkinStrip
{
	Asura_Hash_ID	m_auMaterials[2];
	u_int			m_uNumIndices;
	int				m_iElementID;
	u_int			m_uNumWeights;
	u_int			m_uBoneMap;
};

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Direct3D_Skin
//
// Stores a renderable skin mesh
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Direct3D_Skin
{
	friend class Asura_Chunk_ResourceFile_PC_Skin;

public:

	struct BoneMap
	{
		u_int  m_uSize;
		u_int* m_puEntries;

		BoneMap() : m_uSize(0), m_puEntries(0) {}
		~BoneMap() {delete[] m_puEntries;}
	};

	Asura_Direct3D_Skin();
	~Asura_Direct3D_Skin();

	Direct3D_IndexBuffer*	GetIB() const						{ return m_pxIB; }
	Direct3D_VertexBuffer*	GetVB() const						{ return m_pxVB; }

	u_int					GetNumStrips() const				{ return m_uNumStrips; }
	const Asura_Direct3D_SkinStrip& GetStrip( const int u ) const	{ return m_pxStrips[u]; }

	u_int					GetNumVertices() const				{ return m_uNumVertices; }
	u_int					GetNumTriangles() const				{ return m_uNumTriangles; }

	bool					IsAnyMaterialTransparent() const;

	bool					IsUsingTriLists() const				{ return m_bTriLists; }

	// BoneMaps
	// Gets the number of strips in this smooth skin
	u_int					GetNumBoneMaps() const				{ return m_uNumBoneMaps; }
	// Sets the number of strips, trashing the existing bone maps in the process
	void					SetNumBoneMaps( const u_int uNumBoneMaps );
	// Gets the strip at the given index
	BoneMap&				GetBoneMap(u_int uMapIndex)			{ Asura_Assert(uMapIndex < m_uNumBoneMaps, "Invalid Bonemap index"); return m_pxBoneMaps[uMapIndex]; }
	const BoneMap&			GetBoneMap(u_int uMapIndex) const	{ Asura_Assert(uMapIndex < m_uNumBoneMaps, "Invalid Bonemap index"); return m_pxBoneMaps[uMapIndex]; }

protected:

	Direct3D_IndexBuffer*	m_pxIB;
	Direct3D_VertexBuffer*	m_pxVB;

	Asura_Direct3D_SkinStrip*	m_pxStrips;
	u_int					m_uNumStrips;
	u_int					m_uNumTriangles;
	u_int					m_uNumVertices;

	BoneMap*				m_pxBoneMaps;
	u_int					m_uNumBoneMaps;
	
	bool					m_bTriLists;
};

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Direct3D_SkinRenderer
//
// Manager and renderer for PC skinned meshes
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Direct3D_SkinRenderer
{
public:

	static void Initialise();
	static void Reset();
	static void Shutdown();

	// This can be removed when the old renderers are deleted
	static bool IsEnabled();

	static void ResetCount();

	static Asura_Direct3D_Skin* CreateSkin( const Asura_Hash_ID uHash, bool& bAlreadyExisted );
	static const Asura_Direct3D_Skin* GetSkin( const Asura_Hash_ID uHash );
	static Direct3D_VertexDeclaration* GetVertexDeclaration();

	static void RenderSkin( const Asura_Animation& xAnimation,
		const Asura_Hierarchy_Skin& xHierarchySkin, Asura_AMatrix4x4_InParam xWorldMat,
		const Asura_AMatrix4x4* const pxBoneMats, const Asura_Shadow* const pxShadow = NULL );

	static const u_int uVERTEX_SIZE;

#if defined ASURA_TOOLS
	static Asura_Direct3D_Skin*	Tools_InitialiseSkinFromRawData(
		const Asura_Hierarchy_Skin* const pxConstSkin );
#endif
};

#endif //#if defined(ASURA_TARGET_SM3)

#endif // !defined __ASURA_DIRECT3D_SKINRENDERER_H__
