//
// Asura_Direct3D_Skybox.cpp
//
// Created by RJF - 21 August 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Asura_Direct3D_Skybox.h"

// Asura Independent
#include "Camera/Asura_Camera.h"
#include "Core/Asura_TextureManagement.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Skybox.h"

// Asura PC
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_EffectParameterBlock.h"

Asura_Direct3D_Effect*		Asura_Direct3D_Skybox::s_pxEffect	= NULL;
Direct3D_VertexBuffer*		Asura_Direct3D_Skybox::s_pxVB		= NULL;
Direct3D_VertexDeclaration* Asura_Direct3D_Skybox::s_pxVD		= NULL;

int	Asura_Direct3D_Skybox::s_aiTextureIDs[6] = { ASURA_TEXTURE_INVALID, 
												 ASURA_TEXTURE_INVALID, 
												 ASURA_TEXTURE_INVALID, 
												 ASURA_TEXTURE_INVALID, 
												 ASURA_TEXTURE_INVALID,
												 ASURA_TEXTURE_INVALID };

Asura_Direct3D_EffectParameterBlock* Asura_Direct3D_Skybox::s_pxEPB = NULL;

#if defined( ASURA_TARGET_SM3)

float Asura_Direct3D_Skybox::s_afSkyboxColour[ 3 ];
float Asura_Direct3D_Skybox::s_afSunDir1[ 3 ];
float Asura_Direct3D_Skybox::s_afSunDir2[ 3 ];
float Asura_Direct3D_Skybox::s_afHazeColour1[ 3 ];
float Asura_Direct3D_Skybox::s_fHazeStrength1;
float Asura_Direct3D_Skybox::s_afHazeColour2[ 3 ];
float Asura_Direct3D_Skybox::s_fHazeStrength2;
float Asura_Direct3D_Skybox::s_afSunColour1[ 3 ];
float Asura_Direct3D_Skybox::s_fSunStrength1;
float Asura_Direct3D_Skybox::s_afSunColour2[ 3 ];
float Asura_Direct3D_Skybox::s_fSunStrength2;
bool Asura_Direct3D_Skybox::s_b1MirrorBelowHorizon;
bool Asura_Direct3D_Skybox::s_b2MirrorBelowHorizon;
bool Asura_Direct3D_Skybox::s_bDisableHaze;
bool Asura_Direct3D_Skybox::s_bDisable2Haze;
bool Asura_Direct3D_Skybox::s_bDisableSun;
bool Asura_Direct3D_Skybox::s_bDisable2Sun;
D3DXMATRIX	Asura_Direct3D_Skybox::s_xFrustumToCamDir;
float Asura_Direct3D_Skybox::s_fDistance;

#endif

Asura_RGBA	Asura_Direct3D_Skybox::s_xSkyboxColour(1.f, 1.f, 1.f, 1.f);
Asura_RGBA	Asura_Direct3D_Skybox::s_xFogColour(0.f, 0.f, 0.f, 0.f);
float		Asura_Direct3D_Skybox::s_fFogBlendFactor = 0.f;
bool		Asura_Direct3D_Skybox::s_bFlipTextureVertical = false;
D3DXMATRIX	Asura_Direct3D_Skybox::s_xFaceTransform;


static const float         afVERTEX_DATA[] = {  
												// Bottom
												 -1.f,  1.f, -1.f,	1.f, 0.f,
												 1.f,  1.f, -1.f,	0.f, 0.f,
												 -1.f,  1.f,  1.f,	1.f, 1.f,
												 1.f,  1.f,  1.f,	0.f, 1.f,

												// Front
												-1.f, -1.f, -1.f,	1.f, 0.f,
												-1.f,  1.f, -1.f,	1.f, 1.f,
												1.f, -1.f, -1.f,	0.f, 0.f,
												1.f,  1.f, -1.f,	0.f, 1.f,

												// Back
												-1.f, -1.f,  1.f,	0.f, 0.f,
												-1.f,  1.f,  1.f,	0.f, 1.f,
												 1.f, -1.f,  1.f,	1.f, 0.f,
												 1.f,  1.f,  1.f,	1.f, 1.f,

											    // Left
												1.f, -1.f, -1.f,	1.f, 0.f,
												1.f, -1.f,  1.f,	0.f, 0.f,
												1.f,  1.f, -1.f,	1.f, 1.f,
												1.f,  1.f,  1.f,	0.f, 1.f,

											    // Right
												-1.f, -1.f, -1.f,	0.f, 0.f,
												-1.f, -1.f,  1.f,	1.f, 0.f,
												-1.f,  1.f, -1.f,	0.f, 1.f,
												-1.f,  1.f,  1.f,	1.f, 1.f,

												// Top
												-1.f, -1.f, -1.f,	1.f, 1.f,
												1.f, -1.f, -1.f,	0.f, 1.f,
												-1.f, -1.f,  1.f,	1.f, 0.f,
												1.f, -1.f,  1.f,	0.f, 0.f,
											 };

void Asura_Direct3D_Skybox::Initialise()
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}
	Asura_Direct3D::D3DDevice->CreateVertexBuffer(sizeof(afVERTEX_DATA), D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, &s_pxVB, NULL);
	Asura_Assert(s_pxVB, "Couldn't create skybox vertex buffer"); if(!s_pxVB) return;
	void* pvVertices = NULL; 
	if (SUCCEEDED(s_pxVB->Lock(0, 0, &pvVertices, 0)))
	{
		memcpy(pvVertices, afVERTEX_DATA, sizeof(afVERTEX_DATA));
		s_pxVB->Unlock();
	} else {
		Asura_Assert(0, "Couldn't lock skybox vertex buffer"); 
		return;
	}

	Direct3D_VertexElement axDecl[] = {
		{	0,	0,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },
		{	0,	12,	D3DDECLTYPE_FLOAT2,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	0 },
		D3DDECL_END()
	};
	Asura_Direct3D::D3DDevice->CreateVertexDeclaration(axDecl, &s_pxVD);
	Asura_Assert(s_pxVD, "Couldn't create skybox vertex decl");

	s_pxEPB = new Asura_Direct3D_EffectParameterBlock();
#if defined( ASURA_TARGET_SM3)
	s_pxEPB->AddDynamicParameter<float>	( "g_xSkyboxColour",			s_afSkyboxColour, 3 );
	s_pxEPB->AddDynamicParameter<float>	( "g_xSunDir1",					s_afSunDir1, 3 );
	s_pxEPB->AddDynamicParameter<float>	( "g_xSunDir2",					s_afSunDir2, 3 );
	s_pxEPB->AddDynamicParameter<float>	( "g_xHazeColour1",				s_afHazeColour1, 3 );
	s_pxEPB->AddDynamicParameter<float>	( "g_xHazeColour2",				s_afHazeColour2, 3 );
	s_pxEPB->AddDynamicParameter<float>	( "g_xSunColour1",				s_afSunColour1, 3 );
	s_pxEPB->AddDynamicParameter<float>	( "g_xSunColour2",				s_afSunColour2, 3 );
	s_pxEPB->AddDynamicParameter<float>	( "g_xHazeGradientPower1",		&s_fHazeStrength1 );
	s_pxEPB->AddDynamicParameter<float>	( "g_xHazeGradientPower2",		&s_fHazeStrength2 );
	s_pxEPB->AddDynamicParameter<float>	( "g_xSunGradientPower1",		&s_fSunStrength1 );
	s_pxEPB->AddDynamicParameter<float>	( "g_xSunGradientPower2",		&s_fSunStrength2 );

	s_pxEPB->AddDynamicParameter<bool>	( "g_bDisableHaze",				&s_bDisableHaze );
	s_pxEPB->AddDynamicParameter<bool>	( "g_bDisable2Haze",			&s_bDisable2Haze );
	s_pxEPB->AddDynamicParameter<bool>	( "g_bDisableSun",				&s_bDisableSun );
	s_pxEPB->AddDynamicParameter<bool>	( "g_bDisable2Sun",				&s_bDisable2Sun );

	s_pxEPB->AddDynamicParameter<bool>	( "g_bMirrorBelowHorizon",		&s_b1MirrorBelowHorizon );
	s_pxEPB->AddDynamicParameter<bool>	( "g_b2MirrorBelowHorizon",		&s_b2MirrorBelowHorizon );

	s_pxEPB->AddDynamicParameter<D3DXMATRIX>("g_xFrustumToCamDir",		&s_xFrustumToCamDir);
#endif
	s_pxEPB->AddDynamicParameter		( "SkyboxColour",				&s_xSkyboxColour);
	s_pxEPB->AddDynamicParameter		( "FogColour",					&s_xFogColour);
	s_pxEPB->AddDynamicParameter		( "FogBlendFactor",				&s_fFogBlendFactor);
	s_pxEPB->AddDynamicParameter		( "g_xDistance",				&s_fDistance);
	s_pxEPB->AddDynamicParameter		( "FlipTextureVertical",		&s_bFlipTextureVertical);


	s_pxEPB->AddDynamicParameter<D3DXMATRIX>("g_xWorldViewProjMatrix",	&s_xFaceTransform);
}

void Asura_Direct3D_Skybox::Reset()
{
	for(int i = 0; i < 6; ++i)
	{
		s_aiTextureIDs[i] = ASURA_TEXTURE_INVALID;
	}
}

void Asura_Direct3D_Skybox::Shutdown()
{
	if(s_pxVB) s_pxVB->Release();
	s_pxVB = NULL;

	if(s_pxVD) s_pxVD->Release();
	s_pxVD = NULL;

	delete s_pxEPB; s_pxEPB = NULL;
}

void Asura_Direct3D_Skybox::LoadTextures()
{
	for(int iSkyboxFace = 0; iSkyboxFace < 6; ++iSkyboxFace)
	{
		if(s_aiTextureIDs[iSkyboxFace] == ASURA_TEXTURE_INVALID)
		{
			const Asura_String& xName = Asura_Skybox::GetTexture(iSkyboxFace);
			if(xName.IsNull())
			{
				s_aiTextureIDs[iSkyboxFace] = ASURA_TEXTURE_NONE;
			}
			else
			{
				s_aiTextureIDs[iSkyboxFace] = Asura_Texture_Management::AddTexture(xName);
			}
		}
	}
	Asura_Texture_Management::LoadTextures();
	
	// Clean up any failed loads
	for(int iSkyboxFace = 0; iSkyboxFace < 6; ++iSkyboxFace)
	{
		if (!Asura_Texture_Management::IsTextureLoaded(s_aiTextureIDs[iSkyboxFace]))
		{
#ifndef ASURA_TOOLS
			Asura_Texture_Management::DestroyTexture(s_aiTextureIDs[iSkyboxFace]);
#endif
			s_aiTextureIDs[iSkyboxFace] = ASURA_TEXTURE_NONE;
		}
	}
}

void Asura_Direct3D_Skybox::Render()
{
	s_pxEffect = Asura_Direct3D_Effects_System::GetEffect("Skybox");
	Asura_Assert(s_pxEffect, "Couldn't get skybox effect");
	if(!s_pxEffect) return;

	SCOPEDPIXEVENT((0xff88ff88, L"Skybox"));

	// Ensure that we've got a valid camera...
	Asura_Render::ApplyCameraSettings();

	if(!s_pxEPB->IsBound()) s_pxEPB->Bind(s_pxEffect);

#if defined( ASURA_TARGET_SM3)
	if( Asura_Skybox::GetUseNewSkyboxShader() )
	{
		Asura_Render::CurrentStates.RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
		Asura_Render::CurrentStates.RequestAlphaTest( false );
		Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
		Asura_Render::CurrentStates.RequestZBufferEnabled( false );
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );

#ifdef ASURA_TOOLS
		Asura_Skybox::GetSkyboxColour( s_afSkyboxColour, true );
		Asura_Skybox::GetHaze( s_afHazeColour1, s_afHazeColour2, s_bDisableHaze, s_bDisable2Haze, s_b1MirrorBelowHorizon, s_b2MirrorBelowHorizon, true );
		Asura_Skybox::GetSun( s_afSunColour1, s_afSunColour2, s_bDisableSun, s_bDisable2Sun, true );
		Asura_Skybox::GetHazeStrength( &s_fHazeStrength1, &s_fHazeStrength2, true );
		Asura_Skybox::GetSunSize( &s_fSunStrength1, &s_fSunStrength2, true );
		Asura_Skybox::GetSunDir( s_afSunDir1, s_afSunDir2, true );
#else
		Asura_Skybox::GetSkyboxColour( s_afSkyboxColour );
		Asura_Skybox::GetHaze( s_afHazeColour1, s_afHazeColour2, s_bDisableHaze, s_bDisable2Haze, s_b1MirrorBelowHorizon, s_b2MirrorBelowHorizon );
		Asura_Skybox::GetSun( s_afSunColour1, s_afSunColour2, s_bDisableSun, s_bDisable2Sun );
		Asura_Skybox::GetHazeStrength( &s_fHazeStrength1, &s_fHazeStrength2 );
		Asura_Skybox::GetSunSize( &s_fSunStrength1, &s_fSunStrength2 );
		Asura_Skybox::GetSunDir( s_afSunDir1, s_afSunDir2 );
#endif

		// Screen-space vertex shader (should make it same as PS3, but there's a glitch in my math somewhere so I'll use
		// the simple approach for now)
		D3DXMATRIX xView = Asura_Direct3D::GetViewMatrix();
		const D3DXMATRIX & xProj = Asura_Direct3D::GetProjectionMatrix(); 

		// Remove camera translation so that when we unproject we simply get camera->vertex direction 
		xView._41 = 0.0f;
		xView._42 = 0.0f;
		xView._43 = 0.0f;

		D3DXMATRIX xTempMat = xView * xProj;

		float fDet;
		D3DXMatrixIdentity( &s_xFrustumToCamDir );
		D3DXMatrixInverse( &s_xFrustumToCamDir, &fDet, &xTempMat );
		
		s_fDistance = Asura_Camera::GetFarClip() + 500.f;

		s_pxEffect->Begin( false, 1 );
		if(!s_pxEffect->IsD3DEffectValid()) return;

		s_pxEPB->Apply();
		Asura_Direct3D::D3DDevice->SetVertexDeclaration( Asura_Direct3D::GetFullScreenVD() );
		Asura_Direct3D::D3DDevice->SetStreamSource( 0, Asura_Direct3D::GetFullScreenVB(), 0, sizeof( short ) * 2 );
		s_pxEffect->BeginPass(0);
		s_pxEffect->GetD3DEffect()->CommitChanges();
		Asura_Direct3D::D3DDevice->DrawPrimitive( D3DPT_TRIANGLEFAN, 0, 2 );
		s_pxEffect->EndPass();
		s_pxEffect->End();
	}
	else
#endif
	{
		LoadTextures();

		Asura_Render::CurrentStates.RequestAlphaTest( false );
		Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
		Asura_Render::CurrentStates.RequestZBufferEnabled( false );
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled( false );

		s_xSkyboxColour.m_fR = Asura_Skybox::GetR() / 255.f;
		s_xSkyboxColour.m_fG = Asura_Skybox::GetG() / 255.f;
		s_xSkyboxColour.m_fB = Asura_Skybox::GetB() / 255.f;

		s_xFogColour = Asura_Fog::GetFogColour();

		s_fFogBlendFactor = (Asura_Fog::IsFogRenderable()) ? Asura_Fog::GetSkyboxFogBlendFactor() : 0.f;

		D3DXMATRIX xYRotation; Asura_Angle fRotationAngle; Asura_Skybox::GetOrientationAroundYAxis(fRotationAngle);
		D3DXMatrixRotationY(&xYRotation, fRotationAngle);

		D3DXMATRIX xCameraTranslation;
		const Asura_Vector_3* pxCameraPos = Asura_Camera::GetPosition();
		D3DXMatrixTranslation( &xCameraTranslation, pxCameraPos->x, pxCameraPos->y, pxCameraPos->z );

		s_xFaceTransform = xYRotation * xCameraTranslation * Asura_Direct3D::GetViewProjectionMatrix();

		s_pxEffect->Begin();
		if(!s_pxEffect->IsD3DEffectValid()) return;

		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP );
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP );

		const bool bSRGB = Asura_Direct3D::UseGammaCorrection();
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_SRGBTEXTURE, bSRGB );
		
		Asura_Render::CurrentStates.ResetTextureIndexToLast( 1);

		Asura_Direct3D::D3DDevice->SetStreamSource(0, s_pxVB, 0, sizeof(float) * 5);
		Asura_Direct3D::D3DDevice->SetVertexDeclaration(s_pxVD);

		for(int iSkyboxFace = 0; iSkyboxFace < 6; ++iSkyboxFace)
		{
			SCOPEDPIXEVENT((0xff88ff88, L"Face %i", iSkyboxFace));

			s_bFlipTextureVertical = false;
			int iTextureID = s_aiTextureIDs[iSkyboxFace];

			if(iSkyboxFace == Asura_Skybox::ASURA_SKYBOX_BK && Asura_Skybox::GetBackTextureIsFrontUpsideDown())
			{
				s_bFlipTextureVertical = true;
				iTextureID = s_aiTextureIDs[Asura_Skybox::ASURA_SKYBOX_FR];
			}
			if(iSkyboxFace == Asura_Skybox::ASURA_SKYBOX_RT && Asura_Skybox::GetRightTextureIsLeftUpsideDown())
			{
				s_bFlipTextureVertical = true;
				iTextureID = s_aiTextureIDs[Asura_Skybox::ASURA_SKYBOX_LF];
			}

			Asura_Render::CurrentStates.RequestTextureIndex(0, iTextureID);

			s_pxEPB->Apply();

			s_pxEffect->GetD3DEffect()->BeginPass(0);
			Asura_Direct3D::D3DDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, iSkyboxFace * 4, 2);
			s_pxEffect->GetD3DEffect()->EndPass();
		}

		s_pxEffect->End();

		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_SRGBTEXTURE, FALSE );
	}

}