//*******************************************************************
//
// File: Asura_Direct3D_StoreRenderTargets.h
//
// Created: 22 June 2007 DLH
//
// Description: 
//
//*******************************************************************

#ifndef __ASURA_DIRECT3D_STORERENDERTARGETS_H__
#define __ASURA_DIRECT3D_STORERENDERTARGETS_H__

// other libraries
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_Direct3D_UserOptions.h"

// internal dependencies

// forward declarations

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Class Definitions
//*******************************************************************

class Asura_Direct3D_StoreRenderTargets
{
public:
	Asura_Direct3D_StoreRenderTargets();
	~Asura_Direct3D_StoreRenderTargets();
	
	void Clear();
	void StoreRenderTargets();
	void RestoreRenderTargets();
	
	bool IsD3DDeviceSame() const;
	
private:
	static const u_int s_uSurfaceCountMax = 8;
	u_int m_uSurfaceCount;
	Direct3D_Surface* m_apxSurface[ s_uSurfaceCountMax]; 
};

//*******************************************************************
// Inline Function Definitions
//*******************************************************************

inline Asura_Direct3D_StoreRenderTargets::Asura_Direct3D_StoreRenderTargets()
{
	this->m_uSurfaceCount = 0;
	for( u_int uRTIndex = 0; uRTIndex < s_uSurfaceCountMax; uRTIndex += 1)
	{
		this->m_apxSurface[ uRTIndex] = NULL;
	}
}

inline Asura_Direct3D_StoreRenderTargets::~Asura_Direct3D_StoreRenderTargets()
{
	this->Clear();
}

inline void Asura_Direct3D_StoreRenderTargets::Clear()
{
	for( u_int uRTIndex = 0; uRTIndex < s_uSurfaceCountMax; uRTIndex += 1)
	{
		SAFE_RELEASE( this->m_apxSurface[ uRTIndex]);
	}
}

inline void Asura_Direct3D_StoreRenderTargets::StoreRenderTargets()
{
	this->Clear();
	
	SCOPEDPIXEVENT(( 0xFF006000, L"Store render targets" ));

	u_int uSurfaceCount = Asura_Direct3D_Caps::GetMaxNumberOfRenderTargets();
	
	Asura_Assertf( uSurfaceCount < s_uSurfaceCountMax, ("%d Possible Render Targets useable. Only %d coped with.", uSurfaceCount, s_uSurfaceCountMax));
	// DLH - 14 February 2007 12:31:18 - Stops array over runs
	uSurfaceCount = Asura_Maths::Min( uSurfaceCount, s_uSurfaceCountMax);
	
	this->m_uSurfaceCount = uSurfaceCount;

	HRESULT xResult;

	for ( u_int uSurfaceIndex = 0; uSurfaceIndex < uSurfaceCount; uSurfaceIndex += 1)
	{
		SCOPEDPIXEVENT(( 0xFF004000, L"Render target %u", uSurfaceIndex ));

		xResult = Asura_Direct3D::D3DDevice->GetRenderTarget( uSurfaceIndex, &(this->m_apxSurface[ uSurfaceIndex]));
//		ASURA_D3D_OK( xResult);
	}
}

inline void Asura_Direct3D_StoreRenderTargets::RestoreRenderTargets()
{
	SCOPEDPIXEVENT(( 0xFF006000, L"Restore render targets" ));

	HRESULT xResult;

	// DLH - 14 February 2007 12:33:53 - Restore all the RenderTargets after the first one
	for ( u_int uSurfaceIndex = 1; uSurfaceIndex < this->m_uSurfaceCount; uSurfaceIndex += 1)
	{
		SCOPEDPIXEVENT(( 0xFF004000, L"Render target %u", uSurfaceIndex ));

		Direct3D_Surface* pxSurface = m_apxSurface[ uSurfaceIndex];
		
		xResult = Asura_Direct3D::D3DDevice->SetRenderTarget( uSurfaceIndex, pxSurface);
		ASURA_D3D_OK( xResult);
	}
}

inline bool Asura_Direct3D_StoreRenderTargets::IsD3DDeviceSame() const
{
	u_int uSurfaceCount = this->m_uSurfaceCount;

	HRESULT xResult;

	for ( u_int uSurfaceIndex = 0; uSurfaceIndex < uSurfaceCount; uSurfaceIndex += 1)
	{
		SCOPEDPIXEVENT(( 0xFF004000, L"Render target %u", uSurfaceIndex ));

		

		Direct3D_Surface* pxSurface = NULL;
		xResult = Asura_Direct3D::D3DDevice->GetRenderTarget( uSurfaceIndex, &pxSurface);
//		ASURA_D3D_OK( xResult);
		const bool bIsSame = (pxSurface == this->m_apxSurface[ uSurfaceIndex]);
		SAFE_RELEASE( pxSurface);
		
		if( false == bIsSame)
		{
			return( false);
		}
	}
	
	return( true);
}




//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif // __ASURA_DIRECT3D_STORERENDERTARGETS_H__
