//
// Asura_Direct3D_UserOptions.h
//
// Created by KJL - 03/09/2003 13:44:37
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_DIRECT3D_USEROPTIONS_H__
#define __ASURA_DIRECT3D_USEROPTIONS_H__

#include "Core/Asura_String.h"
#include "Core/Asura_List.h"
#include "Rendering/Asura_Direct3D.h"

class Asura_Direct3D_UserOptions
{
	public:

//		enum { DEFAULT_BACKBUFFER_WIDTH = 1280, DEFAULT_BACKBUFFER_HEIGHT = 720 };
		enum TEXTURE_DETAIL	{ TEXTURE_DETAIL_VERY_LOW = 0, TEXTURE_DETAIL_LOW, TEXTURE_DETAIL_MEDIUM, TEXTURE_DETAIL_HIGH };

		enum SHADOW_SETTING	{ SHADOW_OFF = 0, SHADOW_PLANAR	};

		struct ResolutionInfo
		{
			ResolutionInfo(D3DDISPLAYMODE& xModeData) : m_xModeData(xModeData) { }
			D3DDISPLAYMODE m_xModeData;
			Asura_Char m_szDisplayName[30];
		};
	
		enum ANTIALIASING_SETTING
		{
			ANTIALIASING_OFF = 0,
			ANTIALIASING_2X,
			ANTIALIASING_4X,
			ANTIALIASING_6X,
			ANTIALIASING_8X,
            ANTIALIASING_NUM_SETTINGS
		};
		
		struct Options
		{
		public:
			Options();
			Options(const Options& xOther);
			Options& operator =(const Options& xOther);
			~Options();

			u_int						m_uDefaultWidth;
			u_int						m_uDefaultHeight;

			bool						m_bTrilinearFilteringOn;
			TEXTURE_DETAIL				m_eTextureDetail;
			float						m_fGammaSetting;
			u_int						m_uAnisotropySetting;
			SHADOW_SETTING				m_eSmoothSkinShadowSetting;
			SHADOW_SETTING				m_eObjectHierarchyShadowSetting;
			SHADOW_SETTING				m_eObjectShadowSetting;
#ifdef ASURA_TARGET_SM3
			bool						m_bFilterShadows;
#endif
#ifdef ASURA_SUPPORT_MULTIPLE_MONITORS
			bool						m_bMultiheadEnabled;
#endif

			u_int						m_uAdapterIndex;
			D3DFORMAT					m_eDepthStencilFormat;
			D3DFORMAT					m_eBackBufferFormat;
			D3DFORMAT					m_eFrontBufferFormat;
			u_int						m_uBackBufferCount;
			D3DMULTISAMPLE_TYPE			m_eMultiSampleType;
			u_int						m_uMultiSampleQuality;
			D3DSWAPEFFECT				m_eSwapEffect;
			u_int						m_uRefreshRate;
			u_int						m_uPresentationInterval;
			D3DDEVTYPE					m_eDevType;
			
			D3DFORMAT					m_eRenderTargetBufferFormat;
			D3DFORMAT					m_eRenderTargetDepthFormat;
			
			bool						m_bMultiThreadingEnabled;

			bool Validate();

			u_int						GetBackBufferWidth		( int iScreenIndex = 0) const;
			u_int						GetBackBufferHeight		( int iScreenIndex = 0) const;
			void						SetBackBufferWidth		( u_int dwWidth, int iScreenIndex = 0, bool bForceWriteToWindowedSettings = false, bool bForceWriteToFullscreenSettings = false );
			void						SetBackBufferHeight		( u_int dwWidth, int iScreenIndex = 0, bool bForceWriteToWindowedSettings = false, bool bForceWriteToFullscreenSettings = false );
			void						SetBackBufferDimensions	( u_int dwWidth, u_int dwHeight, int iScreenIndex = 0 );

			void DeleteAllBackBufferDimensions();

		private:
			struct ScreenResolution
			{
				ScreenResolution(int iScreen, u_int uWidth, u_int uHeight) : m_iScreen(iScreen), m_uWidth(uWidth), m_uHeight(uHeight) { }
				int m_iScreen;
				u_int m_uWidth;
				u_int m_uHeight;
			};
			Asura_List<ScreenResolution> m_xWindowedScreenResolutions;
			Asura_List<ScreenResolution> m_xFullscreenScreenResolutions;
		};

		///////////////////////////////////////////////////////////////////////////////
		// (SB - 18/02/2008) - These two structures are used by GetTextureScalingInfo
		// They are designed to be expanded if new features are needed as long as the
		// defaults are setup correctly other projects should be unaffected by
		// additions
		///////////////////////////////////////////////////////////////////////////////

		struct TexInfo
		{
		public:
			// Zeroed by default - NULL pxFilename will always return "don't scale"
			TexInfo( const Asura_String* pxFilename = NULL, u_int uFlags = 0 )
			:	m_pxFilename( pxFilename )
			,	m_uFlags( uFlags ) { }

			const Asura_String* m_pxFilename;
			u_int m_uFlags;
		};

		struct TexScalingInfo
		{
		public:
			// By default don't scale
			TexScalingInfo( bool bScale = false, u_int uMaxLevels = 100 )
			:	m_bScale( bScale )
			,	m_uMaxLevel( uMaxLevels ) { }

			bool m_bScale;
			u_int m_uMaxLevel;
		};

		///////////////////////////////////////////////////////////////////////////////
		// (SB - 19/02/2008) These are used to store sets of advanced settings
		///////////////////////////////////////////////////////////////////////////////

		enum BASIC_SETTING
		{
			BASIC_SETTING_OFF = 0,
			BASIC_SETTING_LOW = 0,
			BASIC_SETTING_MEDIUM = 1,
			BASIC_SETTING_HIGH = 2,
			BASIC_SETTING_CUSTOM = 3
		};

		struct TextureDetailSetting
		{
		public:
			TextureDetailSetting( BASIC_SETTING xSetting = BASIC_SETTING_HIGH );
			BASIC_SETTING GetBasicSetting( void );
			void ApplySettings( void );
			#ifdef ASURA_PC_SUPPORT_WINDOWS_REGISTRY
			void WriteSettingsToRegistry(  const char* const pszSubKey);
			void ReadSettingsFromRegistry(  const char* const pszSubKey);
			#endif

			bool operator==( const TextureDetailSetting& xOther );

			u_int m_eTexDetail;
			bool m_bTrilinearFiltering;
			u_int m_uAnisotropicFiltering;
		private:
			void Project_InitialiseSettings( BASIC_SETTING xSetting );
			bool Project_ApplySettings( void );
		};

		struct ShaderQualitySetting
		{
		public:
			ShaderQualitySetting( BASIC_SETTING xSetting = BASIC_SETTING_HIGH );
			BASIC_SETTING GetBasicSetting( void );
			void ApplySettings( void );
			#ifdef ASURA_PC_SUPPORT_WINDOWS_REGISTRY
			void WriteSettingsToRegistry(  const char* const pszSubKey);
			void ReadSettingsFromRegistry(  const char* const pszSubKey);
			#endif

			bool operator==( const ShaderQualitySetting& xOther );

			// No settings defined yet
		private:
			void Project_InitialiseSettings( BASIC_SETTING xSetting );
			bool Project_ApplySettings( void );
		};

		struct ShadowQualitySetting
		{
		public:
			ShadowQualitySetting( BASIC_SETTING xSetting = BASIC_SETTING_HIGH );
			BASIC_SETTING GetBasicSetting( void );
			void ApplySettings( void );
			#ifdef ASURA_PC_SUPPORT_WINDOWS_REGISTRY
			void WriteSettingsToRegistry(  const char* const pszSubKey);
			void ReadSettingsFromRegistry(  const char* const pszSubKey);
			#endif

			bool operator==( const ShadowQualitySetting& xOther );

			bool m_bShadowsEnabled;
			bool m_bFilterShadows;
			bool m_bBlobShadowsEnabled; // Blob shadows not implemented yet!
		private:
			void Project_InitialiseSettings( BASIC_SETTING xSetting );
			bool Project_ApplySettings( void );
		};

		struct VisualDetailSetting
		{
		public:
			VisualDetailSetting( BASIC_SETTING xSetting = BASIC_SETTING_HIGH );
			BASIC_SETTING GetBasicSetting( void );
			void ApplySettings( void );
			#ifdef ASURA_PC_SUPPORT_WINDOWS_REGISTRY
			void WriteSettingsToRegistry(  const char* const pszSubKey);
			void ReadSettingsFromRegistry(  const char* const pszSubKey);
			#endif

			bool operator==( const VisualDetailSetting& xOther );

			bool m_bSimplifyFSFX;
			// PFX
			// Other effects?
		private:
			void Project_InitialiseSettings( BASIC_SETTING xSetting );
			bool Project_ApplySettings( void );
		};

		// Pending values for advanced and basic settings
		static u_int s_uTextureDetailBasic;
		static u_int s_uShaderQualityBasic;
		static u_int s_uShadowQualityBasic;
		static u_int s_uVisualDetailBasic;

		static TextureDetailSetting s_xTextureDetail;
		static ShaderQualitySetting s_xShaderQuality;
		static ShadowQualitySetting s_xShadowQuality;
		static VisualDetailSetting s_xVisualDetail;

		// DVS interface for the menu system to use
		static void SetupDVSInterface( void );
		//static void DetectSensibleDefaults( void );
		static bool ConsoleCmd_ApplyBasicSettings( bool bHelp );
		static bool ConsoleCmd_ApplyAdvancedSettings( bool bHelp );

		///////////////////////////////////////////////////////////////////////////////
		//
		///////////////////////////////////////////////////////////////////////////////

		// KJL - 05/09/2003 11:37:19 - scan through loaded textures and ensure Direct3D knows of the required texture resolutions
		static void ApplyTextureDetailSettings( void );

		// KJL - 10/09/2003 11:15:42 - access fns to set settings
		static void SetTrilinearFiltering( const bool bOn );
		static void SetTextureDetail( const enum TEXTURE_DETAIL eSetting );
		static void SetAntiAliasingSetting( const enum ANTIALIASING_SETTING eSetting );
		static void SetGammaSetting( const float fGamma );
		static void SetAnisotropicFilteringSetting( const u_int uSetting );
		static void SetSmoothSkinShadows( const enum SHADOW_SETTING eSetting );
		static void SetObjectHierarchyShadows( const enum SHADOW_SETTING eSetting );
		static void SetObjectShadows( const enum SHADOW_SETTING eSetting );
#ifdef ASURA_TARGET_SM3
		static void SetFilterShadows( const bool bOn );
#endif
#ifdef ASURA_SUPPORT_MULTIPLE_MONITORS
		static void SetMultiheadEnabled( const bool bEnabled );
#endif		
		inline static void SetAdapterIndex( u_int uAdapterIndex)						{ s_xCurrentOptions.m_uAdapterIndex				= uAdapterIndex;			}
		inline static void SetFrontBufferFormat( D3DFORMAT eFrontBufferFormat)			{ s_xCurrentOptions.m_eFrontBufferFormat		= eFrontBufferFormat;		}
		inline static void SetBackBufferFormat( D3DFORMAT eBackBufferFormat)			{ s_xCurrentOptions.m_eBackBufferFormat			= eBackBufferFormat;		}
		inline static void SetDepthStencilFormat( D3DFORMAT eDepthStencilFormat)		{ s_xCurrentOptions.m_eDepthStencilFormat		= eDepthStencilFormat;		}
		inline static void SetBackBufferCount( u_int uBackBufferCount)					{ s_xCurrentOptions.m_uBackBufferCount			= uBackBufferCount;			}
		inline static void SetMultiSampleType( D3DMULTISAMPLE_TYPE eMultiSampleType)	{ s_xCurrentOptions.m_eMultiSampleType			= eMultiSampleType;			}
		inline static void SetMultiSampleQuality( DWORD uMultiSampleQuality)			{ s_xCurrentOptions.m_uMultiSampleQuality		= uMultiSampleQuality;		}
		inline static void SetSwapEffect( D3DSWAPEFFECT eSwapEffect)					{ s_xCurrentOptions.m_eSwapEffect				= eSwapEffect;				}
		inline static void SetRefreshRate( const u_int uRefreshRate)					{ s_xCurrentOptions.m_uRefreshRate				= uRefreshRate;				}
		inline static void SetPresentationInterval( const u_int uPresentationInterval)	{ s_xCurrentOptions.m_uPresentationInterval		= uPresentationInterval;	}
		inline static void SetDevType( const D3DDEVTYPE eDevType)						{ s_xCurrentOptions.m_eDevType					= eDevType;					}

		inline static void SetRenderTargetBufferFormat( const D3DFORMAT eRenderTargetBufferFormat)	{ s_xCurrentOptions.m_eRenderTargetBufferFormat		= eRenderTargetBufferFormat;	}
		inline static void SetRenderTargetDepthFormat( const D3DFORMAT eRenderTargetDepthFormat)	{ s_xCurrentOptions.m_eRenderTargetDepthFormat		= eRenderTargetDepthFormat;		}
		
		inline static void SetMultiThreadingEnable( const bool bMultiThreadingEnabled)				{ s_xCurrentOptions.m_bMultiThreadingEnabled = bMultiThreadingEnabled;	}


		// KJL - 10/09/2003 11:15:53 - access fns to query settings
		static bool							IsTrilinearFilteringOn( void )		{ return s_xCurrentOptions.m_bTrilinearFilteringOn;			}
		static bool							IsAnisotropicFilteringOn( void )	{ return (s_xCurrentOptions.m_uAnisotropySetting>1);		}
		static enum TEXTURE_DETAIL			GetTextureDetailSetting( void )		{ return s_xCurrentOptions.m_eTextureDetail;				}
		static enum ANTIALIASING_SETTING	GetAntiAliasingSetting( void );
		static float						GetGammaSetting( void )				{ return s_xCurrentOptions.m_fGammaSetting;					}
		static u_int						GetAnisotropySetting( void )		{ return s_xCurrentOptions.m_uAnisotropySetting;			}
		static enum SHADOW_SETTING			GetSmoothSkinShadows( void )		{ return s_xCurrentOptions.m_eSmoothSkinShadowSetting;		}
		static enum SHADOW_SETTING			GetObjectHierarchyShadows( void )	{ return s_xCurrentOptions.m_eObjectHierarchyShadowSetting;	}
		static enum SHADOW_SETTING			GetObjectShadows( void )			{ return s_xCurrentOptions.m_eObjectShadowSetting;			}
#ifdef ASURA_TARGET_SM3
		static bool							GetFilterShadows( void )			{ return s_xCurrentOptions.m_bFilterShadows;				}
#endif
#ifdef ASURA_SUPPORT_MULTIPLE_MONITORS
		static bool							GetMultiheadEnabled( void )			{ return s_xCurrentOptions.m_bMultiheadEnabled;				}
#endif

		// (SB - 18/02/2008) This replaces the old TextureIsScaleable and is a lot more flexible and scaleable
		static TexScalingInfo GetTextureScalingInfo( const TexInfo xTextureInfo ) { return Project_GetTextureScalingInfo( xTextureInfo ); }
		
		inline static u_int					GetAdapterIndex( void)				{ return( s_xCurrentOptions.m_uAdapterIndex);				}
		inline static D3DFORMAT				GetFrontBufferFormat( void)			{ return( s_xCurrentOptions.m_eFrontBufferFormat);			}
		inline static D3DFORMAT				GetBackBufferFormat( void)			{ return( s_xCurrentOptions.m_eBackBufferFormat);			}
		inline static D3DFORMAT				GetDepthStencilFormat( void)		{ return( s_xCurrentOptions.m_eDepthStencilFormat);			}
		inline static u_int					GetBackBufferCount( void)			{ return( s_xCurrentOptions.m_uBackBufferCount);			}
		static D3DMULTISAMPLE_TYPE			GetMultiSampleType( void);
		static DWORD						GetMultiSampleQuality( void);
		inline static D3DSWAPEFFECT			GetSwapEffect( void)				{ return( s_xCurrentOptions.m_eSwapEffect);					}
		inline static u_int					GetRefreshRate( void)				{ return( s_xCurrentOptions.m_uRefreshRate);				}
		inline static u_int					GetPresentationInterval( void)		{ return( s_xCurrentOptions.m_uPresentationInterval);		}
		inline static D3DDEVTYPE			GetDevType( void)					{ return( s_xCurrentOptions.m_eDevType);					}
		
		inline static D3DFORMAT				GetRenderTargetBufferFormat()		{ return( s_xCurrentOptions.m_eRenderTargetBufferFormat);	}
		inline static D3DFORMAT				GetRenderTargetDepthFormat()		{ return( s_xCurrentOptions.m_eRenderTargetDepthFormat);	}
		
		inline static bool					GetMultiThreadingEnable()			{ return( s_xCurrentOptions.m_bMultiThreadingEnabled);	}

		// KJL - 10/09/2003 10:11:24 - capability tests to determine UI ranges
		static bool							IsAntiAliasingSettingAvailable( const enum ANTIALIASING_SETTING eSetting );
		static u_int						GetMaxAnisotropySetting( void );

		static bool							BuildPresentParameters( D3DPRESENT_PARAMETERS& xParameters, int iScreenIndex = 0);
		
		inline static Options&				GetCurrentOptions() { return s_xCurrentOptions; }

		static bool							ValidateOptions( Options& xOptions);
		static bool							GetAllOptions( Options& xOptions);
		static bool							SetAllOptions( Options& xOptions);
		
		// Pass -1 to get resolutions for the default adapter
		static const Asura_List<ResolutionInfo>*	GetSupportedResolutions(int iAdapterIndex = -1, bool bForceRebuild = false);
		
		// Pass -1 to clear everything
		static void							ClearResolutionCache(int iAdapterIndex = -1);

		#ifdef ASURA_PC_SUPPORT_WINDOWS_REGISTRY
		static void							WriteSettingsToRegistry( const char* const pszSubKey);
		static void							ReadSettingsFromRegistry( const char* const pszSubKey);
		#endif
	private:

		static TexScalingInfo Project_GetTextureScalingInfo( const TexInfo xTextureInfo );

		static Options						s_xCurrentOptions;

		// Display mode enumeration
		struct ResolutionInfoList
		{
			int m_iAdapterIndex;
			Asura_List<ResolutionInfo> m_xModes;
		};
		static Asura_List<ResolutionInfoList> s_xEnumDisplayModes;
};


#endif//__ASURA_DIRECT3D_USEROPTIONS_H__
