// KJL - Tuesday January 9 2001 15:27:50
// 
// Asura_PlatformRender.cpp

#include "Core/Asura.h"

#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Viewport.h"
#include "Mesh/Asura_Mesh_System.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_Windows.h"

#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Debug_Timers.h"

#include "Rendering/Asura_PlatformRenderEnvironment.h"
#if defined (ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
#include "Rendering/Asura_Direct3D_SmoothSkinRenderer.h"
#endif
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
#include "Rendering/Asura_PlatformRenderCharacter.h"
#endif
#include "Rendering/Asura_PlatformRenderObject.h"
#include "Rendering/Asura_Platform_RenderObjectHierarchy.h"
#include "Rendering/Asura_Direct3D_ShadowBuffer.h"

#if defined (ASURA_TARGET_SM3)
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_EnvironmentRenderer.h"
#include "Rendering/Asura_Fog.h"
#endif

#if defined (ASURA_TARGET_SM3)
#include "Rendering/Asura_Direct3D_Fog.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#endif

#include "Shadows/Asura_Shadow_System.h"

#if defined( ASURA_USE_CARPET)
	#include "Carpet/Asura_Direct3D_Carpet_Render.h"
	#include "Carpet/Asura_Direct3D_Carpet_Game_Render.h"
#endif
#if defined( ASURA_USE_BOTANICALS)
	#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"
#endif

#include "Debugging/Asura_CPUProfile.h"
#ifdef ASURA_USE_PROFILING_SYSTEM
	#include "Debugging/Asura_CPUProfile_Renderer.h"
#endif

#include "Rendering/Asura_Direct3D_HDR.h"
#include "Rendering/Asura_Direct3D_SkinRenderer.h"


//How close should the viewport max z be to prevent the player model clipping into stuff
const float fASURA_PC_PLAYER_RENDER_FAR_Z = 0.001f;

void Asura_Render::Platform_Initialise(void)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Render::Platform_Initialise");
	if(!Asura_Direct3D::Initialise())
	{
		// KJL - 18/09/2003 12:04:03 - if we can't initialise Direct3D, try to exit gracefully
		Asura_Direct3D::Project_HandleInitialisationFailure();
	}
	
#ifdef ASURA_TOOLS
	Asura_Mesh_System::PlatformInitialiseRenderMeshes();
#endif

#if defined (ASURA_TARGET_SM3)
	Asura_Direct3D_Effects_System::Initialise();
	Asura_Direct3D_EnvironmentRenderer::Initialise();
#endif
	
	Asura_Direct3D_EnvRenderer::Initialise();
#if defined (ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
	Asura_Direct3D_SmoothSkinRenderer::Initialise();
#endif

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
	Asura_Direct3D_CharacterRenderer::Initialise();
#endif

	Asura_Direct3D_ObjectRenderer::Initialise();
	Asura_Direct3D_ObjectHierarchyRenderer::Initialise();

#if defined(ASURA_TARGET_SM3)
	Asura_Direct3D_SkinRenderer::Initialise();
#endif


#if defined( ASURA_USE_CARPET)
	Asura_Direct3D_Carpet_Render::Initialise();
	Asura_Direct3D_Carpet_Game_Render::Initialise();
#endif

#if defined( ASURA_USE_BOTANICALS)
	Asura_Direct3D_Botanicals_Game_Render::Initialise();
#endif

#if defined(ASURA_TARGET_SM3)
	Asura_Direct3D_Fog::Initialise();
#endif

#if defined(ASURA_USE_HDR)
	Asura_Direct3D_HDR::Initialise();
#endif

	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__TOTALFRAME, "total frame");
}


void Asura_Render::Platform_EarlyShutdown(void)
{
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__TOTALFRAME);
}

void Asura_Render::Platform_LateShutdown(void)
{
#if defined(ASURA_USE_HDR)
	Asura_Direct3D_HDR::Shutdown();
#endif

#if defined(ASURA_TARGET_SM3)
	Asura_Direct3D_Fog::Shutdown();
#endif

#if defined( ASURA_USE_BOTANICALS)
	Asura_Direct3D_Botanicals_Game_Render::ShutDown();
#endif

#if defined( ASURA_USE_CARPET)
	Asura_Direct3D_Carpet_Game_Render::ShutDown();
	Asura_Direct3D_Carpet_Render::ShutDown();
#endif

#if defined(ASURA_TARGET_SM3)
	Asura_Direct3D_SkinRenderer::Shutdown();
#endif

	Asura_Direct3D_ObjectHierarchyRenderer::Shutdown();
	Asura_Direct3D_ObjectRenderer::Shutdown();

#if defined( ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER ) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
	Asura_Direct3D_SmoothSkinRenderer::Shutdown();
#endif

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
	Asura_Direct3D_CharacterRenderer::Shutdown();
#endif

	Asura_Direct3D_EnvRenderer::Shutdown();
#if defined (ASURA_TARGET_SM3)
	Asura_Direct3D_Effects_System::Shutdown();
	Asura_Direct3D_EnvironmentRenderer::Shutdown();
#endif

#ifdef ASURA_TOOLS
	Asura_Mesh_System::PlatformShutdownRenderMeshes();
#endif
	Asura_Direct3D::Shutdown();
}

void Asura_Render::Platform_Reset(void)
{
	Asura_Direct3D::D3DDevice->EvictManagedResources();

#ifdef ASURA_TOOLS
	Asura_Mesh_System::PlatformShutdownRenderMeshes();
	Asura_Mesh_System::PlatformInitialiseRenderMeshes();
#endif
	
#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
	Asura_Direct3D_SmoothSkinRenderer::Reset();
#endif

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
	Asura_Direct3D_CharacterRenderer::Reset();
#endif

	Asura_Direct3D_ObjectRenderer::Reset();
	Asura_Direct3D_ObjectHierarchyRenderer::Reset();
	Asura_Direct3D_EnvRenderer::Reset();

#if defined(ASURA_TARGET_SM3)
	Asura_Direct3D_SkinRenderer::Reset();
#endif

#if defined (ASURA_TARGET_SM3)
	Asura_Direct3D_Effects_System::Reset();
	Asura_Direct3D_EnvironmentRenderer::Reset();
#endif

#if defined(ASURA_TARGET_SM3)
	Asura_Direct3D_Fog::Reset();
#endif
}

void Asura_Render::Platform_ApplyCameraSettings(void)
{
	// NRM 19/11/02 - temporary fix to batching of overlays
	if(Asura_Camera::GetNumberOfCameras() > 1)
	{
		Asura_Render::FlushDisplay();
	}
	Asura_Direct3D::SetViewingArea();
}

void Asura_Render::Platform_AllowFullScreenRendering(void)
{
	SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Allow full-screen rendering" ));
	Asura_Direct3D::ResetClippingRectangle();
}

#ifdef ASURA_TOOLS
void Asura_Render::Tools_SetClippingRectangle(float fX,float fY,float fWidth,float fHeight)
{
	Asura_Direct3D::SetClippingRectangle(fX,fY,fWidth,fHeight);
}

bool Asura_Render::Tools_GetClippingRectangle(float& fX,float& fY,float& fWidth,float& fHeight)
{
	return Asura_Direct3D::GetClippingRectangle(fX,fY,fWidth,fHeight);
}
#endif

#ifdef ASURA_TARGET_SM3
void Asura_Render::Platform_BeginSkyboxRendering(void)
{
	//TODO: This needs to be implemented to support rendering the sky late
}
void Asura_Render::Platform_EndSkyboxRendering(void)
{
	//TODO: This needs to be implemented to support rendering the sky late
}
#endif


void Asura_Render::Platform_ClearZBuffer(void)
{
	Asura_Direct3D::ClearZBuffer();
}


void Asura_Render::Platform_BeginFrame(void)
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}

	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	
	Asura_Direct3D::BeginFrame();	

#if defined(ASURA_TARGET_SM3)
    // (MH - 25.05.2006) moved this here out of Environment Renderer (and it was also done in object hierarchy renderer)
	Asura_Direct3D_Effects_System::SetSharedParameters();
#endif

#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
	{
		static bool	s_bHiResScreenshotModeEnabledLastFrame	= false;
		static bool	s_bMipMapDisabledLastFrame	= false;

		if(s_bMipMapDisabledLastFrame != s_bDisableMipMaps)
		{
			union
			{
				float	m_fValue;
				DWORD	m_dwValue;

			} xMipMapBias;

			// Have we just enabled hi-res mode?
			xMipMapBias.m_fValue	= (s_bDisableMipMaps) ? -16.0f : 0.0f;

			// According to the DX documentation, values between -16.0 and +15.0 are valid for
			// the mip-map biasing, negative numbers reduce the mip-mapping whilst positive 
			// numbers increase it (thus effectively reducing texture resolution whilst rendering)...
			for(u_int u = 0; u < 4; u++)
			{
				Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_MIPMAPLODBIAS, xMipMapBias.m_dwValue);
			}
			s_bMipMapDisabledLastFrame = s_bDisableMipMaps;
		}


		// Only bother doing this if we've changed state...
		if(s_bHiResScreenshotModeEnabledLastFrame != s_bEnableHiResScreenshotMode)
		{

			// Update the variable...
			s_bHiResScreenshotModeEnabledLastFrame	= s_bEnableHiResScreenshotMode;

			Asura_Shadow_System::SetHiResShadowMode(s_bEnableHiResScreenshotMode);
		}
	}
#endif	//defined(ASURA_USE_SCREENSHOT_SYSTEM)
}

void Asura_Render::Platform_EndFrame(void)
{
	ASURA_CPU_PROFILE__STOP(ASURA_CPU_PROFILE__TOTALFRAME);
	
#ifdef ASURA_USE_PROFILING_SYSTEM
  	Asura_CPUProfile_Renderer::Render(true);
  	Asura_CPUProfile_Renderer::ClearValues();
#endif

	// should really start total frame marker from after VSync...
	ASURA_CPU_PROFILE__START_LABEL(ASURA_CPU_PROFILE__TOTALFRAME, "total frame");

	FlushDisplay();
	Asura_Direct3D::EndFrame();


}

void Asura_Render::Platform_FlushDisplay(void)
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}
	
	//Asura_Assert(0,"flush!");
	Asura_Direct3D::FlushFrame();
	//Asura_Assert(0,"flushed!");
}

bool Asura_Render::Platform_SetVideoMode(int width, int height, bool fsaa)
{
	bool bRet = Asura_Direct3D::SetVideoMode(width, height, 0, fsaa);

	// Adam A. Brown (7.12.2004 14:36:20) : The call to set video mode might
	// result in a different resolution from the one asked for. So we need to
	// get the actual size for the viewport and window.
	/*u_int uActualWidth = Asura_Direct3D::GetPresentParameters().BackBufferWidth;
	u_int uActualHeight = Asura_Direct3D::GetPresentParameters().BackBufferHeight;

	Asura_Viewport::Set(uActualWidth, uActualHeight);*/
	//Asura_Windows::SetWindowSize(uActualWidth, uActualHeight);

	return bRet;
}

#ifdef ASURA_TOOLS
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Viewport.h"
#include "Tools_System.h"
#include "Misc/Tools_Render_Options.h"
#endif

void Asura_Overlay_Quad::Platform_Render()
{
#ifdef ASURA_TOOLS
	Asura_Overlay_Quad xBackup = *this;
	if( Tools_Render_Options::IsOverlayScalingEnabled() )
	{
		const Asura_ViewingArea* pxViewingArea = Asura_Camera::GetViewingArea();
		const float	fMinX = pxViewingArea->m_fXOffset * Asura_Viewport::GetWidth();
		const float	fMinY = pxViewingArea->m_fYOffset * Asura_Viewport::GetHeight();

		// Apply the scale
		const float	fScale = Tools_Render_Options::GetOverlayScale();
		for( u_int v=0; v<4; ++v ) 
		{	
			X[v] *= fScale;
			Y[v] *= fScale;
		}

		// Add the viewing-area offsets...
		for( u_int v=0; v<4; ++v )
		{ 
			X[v] += fMinX;
			Y[v] += fMinY;
		}
	}
#endif	

	const Asura_Vector_2& xSubPixelOffset = Asura_Render::GetSubPixelOffset();
	for( u_int v=0; v<4; ++v )
	{
		X[v] += xSubPixelOffset.x;
		Y[v] += xSubPixelOffset.y;
	}

	//Asura_Assert(0,"add quad to batch");
	Asura_Direct3D::AddQuadToBatch(this);

#ifdef ASURA_TOOLS
	*this = xBackup;
#endif 
}

static Direct3D_Surface* SplashScreenSurface=0;

void Asura_Render::Platform_InitialiseSplashScreen(const char* szName)
{
	//if (SplashScreenSurface)
	//{
	//	SplashScreenSurface->Release();
	//}
	//Asura_Direct3D::D3DDevice->CreateImageSurface(Asura_Direct3D::d3dpp.BackBufferWidth,Asura_Direct3D::d3dpp.BackBufferHeight,Asura_Direct3D::d3dpp.BackBufferFormat,&SplashScreenSurface);

	//extern void LoadSplashScreen(Direct3D_Surface* surface, const char* szName);
	//LoadSplashScreen(SplashScreenSurface,szName);
}

#ifdef ASURA_TARGET_SM3

void Asura_Render::Platform_BeginMRT()
{
	Asura_Direct3D::BeginDeferredRendering();
}

void Asura_Render::Platform_EndMRT()
{
	// DMR 30/01/07
	// This fixes the disappearing projectors on AvP PC, and replaces an old
	// game-side hack that I had in SS2, that fixed disappearing PFX...
	// 
	// The PFX bug was only ever seen on XP x64 machines - it looked like
	// a SetRenderTarget was being ignored by D3D, but calling it again worked...
	Asura_Direct3D::GetDeferredRenderingTarget()->Bind();
}

#endif // def ASURA_TARGET_SM3

void Asura_Render::Platform_ShutdownSplashScreen(void)
{
	//if (SplashScreenSurface)
	//{
	//	SplashScreenSurface->Release();
	//	SplashScreenSurface = 0;
	//}
}

void Asura_Render::Platform_SplashScreenRender(void)
{
	//Asura_Assert(SplashScreenSurface,"No splash screen to render...");
	//if (SplashScreenSurface)
	//{
	//	Direct3D_Surface* dest;
	//	Asura_Direct3D::D3DDevice->GetRenderTarget(&dest);
	//	Asura_Direct3D::D3DDevice->CopyRects(SplashScreenSurface,0,0,dest,0);
	//	dest->Release();
	//}
}

void Asura_Render::Platform_RenderDynamicShadowsOntoEnvironment()
{
	Asura_Direct3D_ShadowBufferSystem::FinishedCollectingShadows();
}

void Asura_Render::Platform_PrepareToRenderFirstPersonMesh ()
{
	Asura_Direct3D::SetViewportMinZ(0.0f);
	Asura_Direct3D::SetViewportMaxZ(fASURA_PC_PLAYER_RENDER_FAR_Z);
	Asura_Direct3D::SetViewingArea();
}


void Asura_Render::Platform_FinishRenderFirstPersonMesh ()
{
	Asura_Direct3D::SetViewportMinZ(0.0f);
	Asura_Direct3D::SetViewportMaxZ(1.0f);
	Asura_Direct3D::SetViewingArea();
}


#ifdef ASURA_USE_HDR

void Asura_Render::Platform_ApplyToneMapping()
{
	Asura_Direct3D_HDR::ApplyToneMapping();
}

#endif // ASURA_USE_HDR

//void RenderIcon(Asura_Vector_3& position, int icon, float size = 0.5f){}

void RenderLogo(void){}


#include "rendering/Asura_Font_System.h"

float Asura_Font_System::GetPlatformYScaleFactor(void)
{
	return 1.0f;
}

//eof
