/**
*
* \file Asura_PlatformRenderModules.cpp
*
* Created by Kevin Lea - 14/03/2002 15:07:56
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

// Asura headers
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_PlatformRenderEnvironment.h"

#if defined (ASURA_TARGET_SM3)
#include "Rendering/Asura_Direct3D_EnvironmentRenderer.h"
#endif

// Tools headers
#ifdef ASURA_TOOLS
#include "Tools_System.h"
#endif

void Asura_Module_Handler::Platform_BeginRender(void)
{

}

void Asura_Module_Handler::Platform_Render(void)
{
#ifndef ASURA_TOOLS

	#if defined (ASURA_TARGET_SM3)

		Asura_Direct3D_EnvironmentRenderer::Render( false );

	#else
	
		Asura_Direct3D_EnvRenderer::Render();

	#endif

#else
	#if defined (ASURA_TARGET_SM3)
	
		if(!Tools_System::IsActive())
		{
			Asura_Direct3D_EnvironmentRenderer::Render( false );
		}
	
	#else

		// If we have quit to game, then we want to use the Asura_Direct3D_EnvRenderer renderer
		// First we have to generate the meshes for it though
		if(!Tools_System::IsActive() && !Asura_Direct3D_EnvRenderer::IsUsingPrecomputedData())
		{
			Asura_Direct3D_EnvRenderer::GenerateTriLists();
		}

		// KJL - 14/04/2003 15:45:09 - Tools builds have two options:
		// use the preloaded strips or make them up on the fly
		if (Asura_Direct3D_EnvRenderer::IsUsingPrecomputedData())
		{
			Asura_Direct3D_EnvRenderer::Render();
		}
	#endif

	else
	{
		// KJL - 14/04/2003 15:45:39 - a potential problem is if the environment is not exported
		// put neither is any mesh data - check for this and just return
		if(!m_pxModuleStorage[0].Mesh) { return; }

		/// Kevin Lea - 14/03/2002 15:10:46 - still the old sequential style of rendering cameras at the moment
		
		extern void RenderMesh(Asura_Mesh* pxMesh, const Asura_Vector_3& xPosition,int iModuleIndex); 

		/// Kevin Lea - 14/03/2002 15:13:00 - this bool needs kicking out
		/// basically it's there to make sure the whole environment gets preprocessed on the first frame
		/// this won't be required with a proper level export
		extern bool ForceRedrawAll;

		Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CW);

		for(int i=0; i<Asura_Camera::GetNumberOfCameras(); i++)
		{
			Asura_Camera::SetCameraIndex(i);

			if (Asura_Camera::IsCameraActive())
			{
				Asura_Render::ApplyCameraSettings();

				if (ForceRedrawAll)
				{
					for (int iModule = 0; iModule < m_iNumberOfModules; ++iModule)
					{
						Asura_Module& xModule = m_pxModuleStorage[iModule];
						RenderMesh(xModule.Mesh, xModule.Position, iModule);
					}
					ForceRedrawAll=false;
				}
				else
				{
					for (int iModule = 0; iModule < m_iNumberOfModules; ++iModule)
					{
						if(IsModuleVisible(iModule))
						{
							Asura_Module& xModule = m_pxModuleStorage[iModule];
					
							RenderMesh(xModule.Mesh, xModule.Position, iModule);
						}
					}
				}
			}
		}

	//	if (Asura_Render::PatchesOn&&!Asura_Render::StaticSubdivision) EndPatchOutput();
	}
#endif
}


#ifdef ASURA_TARGET_SM3
// This is to be called when the deferred lighting has been done, but before specialFX 
// rendering, for rendering of translucency and water.
void Asura_Module_Handler::Platform_RenderLate(void)
{
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_Direct3D_EnvironmentRenderer::Render( true );
		Asura_Direct3D_EnvironmentRenderer::RenderWater();
	#endif
}
#endif

void Asura_Module_Handler::Platform_EndRender(void)
{
}

void Asura_Module_Handler::Platform_Initialise(void)
{
}

void Asura_Module_Handler::Platform_Shutdown(void)
{
}
