/**
*
* \file Asura_PlatformRenderWater.cpp
*
* Created by Robert Dibley - 16/11/2004 14:18:36
*
* Copyright (C) 2004 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "Rendering/Asura_Water.h"

#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Weather_System.h"

#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_PlatformRenderWater.h"

#include "Core/Asura_TextureManagement.h"

#ifdef ASURA_USE_WATER

static const unsigned int uWATER_FUNCTION_RESERVED_CONSTANTS = 29;

static float afConstants[uWATER_FUNCTION_RESERVED_CONSTANTS][4];

static Direct3D_Texture* s_pxBlackSpot = NULL;

static void WINAPI BlackSpotFiller(D3DXVECTOR4* pOut, const D3DXVECTOR2* pTexCoord, const D3DXVECTOR2* pTexelSize, LPVOID pData)
{
	*pOut = D3DXVECTOR4(0, 0, 0, 0);
}

void Asura_Water_System::Platform_Initialise(void)
{
	HRESULT hr;
	hr = D3DXCreateTexture(Asura_Direct3D::D3DDevice, 1, 1, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &s_pxBlackSpot);
	Asura_Assert(SUCCEEDED(hr), "Failed to create 1x1 black spot texture");

	D3DXFillTexture(s_pxBlackSpot, BlackSpotFiller, NULL);
}

void Asura_Water_System::Platform_Reset( void )
{
}

void Asura_Water_System::Platform_Shutdown(void)
{
	if(s_pxBlackSpot)
	{
		s_pxBlackSpot->Release();
		s_pxBlackSpot = NULL;
	}
}

void Asura_Water_System::Platform_BeginRender(void)
{
	// set up the vertex shader constants
	D3DPERFCALL(BeginEvent(D3DCOLOR_ARGB(0xff, 0x00, 0x00, 0xff), L"Water rendering"));

	D3DXMATRIX xMat;
	
	Asura_Direct3D::D3DDevice->GetTransform(D3DTS_VIEW,		&xMat);
	D3DXMatrixTranspose(reinterpret_cast<D3DXMATRIX*>(&afConstants[ 0][ 0]), &xMat);

	Asura_Direct3D::D3DDevice->GetTransform(D3DTS_PROJECTION,	&xMat);
	D3DXMatrixTranspose(reinterpret_cast<D3DXMATRIX*>(&afConstants[ 4][ 0]), &xMat);

	// gather the data we need to build constants with
	Asura_Angle fCameraHeading;
	fCameraHeading = Asura_Maths::ArcTangent( Asura_Camera::GetOrientation()->e13, 
											  Asura_Camera::GetOrientation()->e33);
	const Asura_Vector_3* pxCamPosition = Asura_Camera::GetPosition();
	Asura_Vector_3 xSunDirection;
	if(s_pxController->m_bUseLevelSun)
	{
		xSunDirection = Asura_Weather_System::GetSunPos();
	}
	else
	{
		xSunDirection = s_pxController->m_afSunDir;
	}
	Asura_Matrix_3x3 xSunMatrix;
	Asura_Maths::OrientationFromDirection(xSunMatrix, xSunDirection);

	Asura_Vector_2 xCenterpoint = Asura_Fog::GetGraphFogPoint();

	afConstants[ 8][0] = s_pxController->m_fXScale1;
	afConstants[ 8][1] = s_pxController->m_fXScale2;
	afConstants[ 8][2] = s_pxController->m_fXScale3;
	afConstants[ 8][3] = s_pxController->m_fSunBrightness / 128.0f;
	afConstants[ 9][0] = s_pxController->m_fYScale1;
	afConstants[ 9][1] = s_pxController->m_fYScale2;
	afConstants[ 9][2] = s_pxController->m_fYScale3;
	afConstants[ 9][3] = 0.f; //s_pxController->m_fThreshold; // Subdivision threshold not required on Xbox
	afConstants[10][0] = s_pxController->m_fZScale1;
	afConstants[10][1] = s_pxController->m_fZScale2;
	afConstants[10][2] = s_pxController->m_fZScale3;
	afConstants[10][3] = s_pxController->m_fAlphaLevel / 128.0f;
	afConstants[11][0] = 1.f / s_pxController->m_fFalloff1;
	afConstants[11][1] = 1.f / s_pxController->m_fFalloff2;
	afConstants[11][2] = 1.f / s_pxController->m_fFalloff3;
	afConstants[11][3] = s_pxController->m_fSunYScale;
	afConstants[12][0] = s_fWaterCycleTime1;
	afConstants[12][1] = s_fWaterCycleTime2;
	afConstants[12][2] = s_fWaterCycleTime3;
	afConstants[12][3] = 0.f;
	afConstants[13][0] = 1.f;
	afConstants[13][1] = 0.f;
	afConstants[13][2] = -1.f;
	afConstants[13][3] = 1.f / (2.f * Asura_Maths::Pi);
	afConstants[14][0] = 1.5f;
	afConstants[14][1] = 0.5f;
	afConstants[14][2] = 3.f;
	afConstants[14][3] = 2.f;
	/*afConstants[15][0] = 3.f;
	afConstants[15][1] = -3.f;
	afConstants[15][2] = 0.5f;
	afConstants[15][3] = 0.f;*/
	afConstants[16][0] = s_pxController->m_fHorizonBrightness;
	afConstants[16][1] = s_pxController->m_fHorizonYScale;
	afConstants[16][2] = s_pxController->m_fHorizonVOffset;
	afConstants[16][3] = s_pxController->m_fHorizonVScale;
	afConstants[17][0] = pxCamPosition->x;
	afConstants[17][1] = pxCamPosition->y;
	afConstants[17][2] = pxCamPosition->z;
	afConstants[17][3] = fCameraHeading / (2.0f * Asura_Maths::Pi);
	Asura_Maths::SinCos(fCameraHeading, afConstants[18][0], afConstants[18][1]);
	afConstants[18][2] = -s_pxController->m_fLightScale * s_pxController->m_fLightRescale;
	afConstants[18][3] = s_pxController->m_fLightOffset * s_pxController->m_fLightRescale;
	afConstants[19][0] = xSunMatrix.e11 * s_pxController->m_afTexScale[0];
	afConstants[19][1] = xSunMatrix.e21 * s_pxController->m_afTexScale[0];
	afConstants[19][2] = xSunMatrix.e31 * s_pxController->m_afTexScale[0];
	afConstants[19][3] = s_pxController->m_afTexOffset[0];
	afConstants[20][0] = xSunMatrix.e12 * s_pxController->m_afTexScale[1];
	afConstants[20][1] = xSunMatrix.e22 * s_pxController->m_afTexScale[1];
	afConstants[20][2] = xSunMatrix.e32 * s_pxController->m_afTexScale[1];
	afConstants[20][3] = s_pxController->m_afTexOffset[1];
	afConstants[21][0] = xSunDirection.x;
	afConstants[21][1] = xSunDirection.y;
	afConstants[21][2] = xSunDirection.z;
	afConstants[21][3] = 0.f;
	afConstants[22][0] = 1.f / (s_pxController->m_fSunlightFalloffFar - s_pxController->m_fSunlightFalloffNear);
	afConstants[22][1] = afConstants[22][0] * -s_pxController->m_fSunlightFalloffNear;
	afConstants[22][2] = s_afBasePassUVOffset[0];
	afConstants[22][3] = s_afBasePassUVOffset[1];

	if(Asura_Fog::IsFogRenderable())
	{
		afConstants[23][0] = Asura_Fog::GetOneOverFogPlaneSeparation();
		afConstants[23][1] = -Asura_Fog::GetFogNearPlane() * Asura_Fog::GetOneOverFogPlaneSeparation();

		if(Asura_Fog::IsGraphFogEnabled())
		{
			afConstants[23][2] = xCenterpoint.x;
			afConstants[23][3] = 1.0f;
		
			afConstants[24][0] = 1.0f / xCenterpoint.x;
			afConstants[24][1] = afConstants[24][2] = afConstants[24][3] = 0.0f;
			afConstants[25][0] = 1.0f / (1.0f - xCenterpoint.x);
			afConstants[25][1] = -xCenterpoint.x / (1.0f - xCenterpoint.x);
			afConstants[25][2] = afConstants[25][3] = 0.0f;

			float xValue0, xValue1, xValue2, xValue3;
			xValue0 = -xCenterpoint.y;
			xValue1 = 0.f;
			xValue2 = xCenterpoint.y;
			xValue3 = Asura_Fog::GetFogValueAtFarPlane();

			afConstants[26][0] =  0.5f * ( xValue0 * -1.0f + xValue1 *  3.0f + xValue2 * -3.0f + xValue3 *  1.0f );
			afConstants[26][1] =  0.5f * ( xValue0 *  2.0f + xValue1 * -5.0f + xValue2 *  4.0f + xValue3 * -1.0f );
			afConstants[26][2] =  0.5f * ( xValue0 * -1.0f +                   xValue2 *  1.0f                   );
			afConstants[26][3] =  xValue1;

			xValue0 = 0.f;
			xValue1 = xCenterpoint.y;
			xValue2 = Asura_Fog::GetFogValueAtFarPlane();
			xValue3 = 2.f * Asura_Fog::GetFogValueAtFarPlane() - xCenterpoint.y;

			afConstants[27][0] =  0.5f * ( xValue0 * -1.0f + xValue1 *  3.0f + xValue2 * -3.0f + xValue3 *  1.0f );
			afConstants[27][1] =  0.5f * ( xValue0 *  2.0f + xValue1 * -5.0f + xValue2 *  4.0f + xValue3 * -1.0f );
			afConstants[27][2] =  0.5f * ( xValue0 * -1.0f +                   xValue2 *  1.0f                   );
			afConstants[27][3] =  xValue1;
		}
		else
		{
			afConstants[23][2] = 2.0f;
			afConstants[24][0] = 1.0f;
			afConstants[24][1] = afConstants[24][2] = afConstants[24][3] = 0.0f;
			afConstants[26][0] = afConstants[26][1] = afConstants[26][3] = 0.0f;
			afConstants[26][2] = Asura_Fog::GetFogValueAtFarPlane();	// map from 0..1 to 0..farValue
		}
		afConstants[28][0] = Asura_Fog::GetFogColour().m_fR;
		afConstants[28][1] = Asura_Fog::GetFogColour().m_fG;
		afConstants[28][2] = Asura_Fog::GetFogColour().m_fB;

		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGENABLE, TRUE);						
	}
	else
	{
		ZeroMemory(&afConstants[23][0], sizeof(float) * 4 * 5);
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGENABLE, FALSE);
	}

	Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(0, &afConstants[0][0], uWATER_FUNCTION_RESERVED_CONSTANTS);

	if ( Asura_Direct3D_Caps::GetHardwareVertexShaderVersion() > 1.1f )
		Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_WATER);
	else
		Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_WATER_VS11);


	if(Asura_Direct3D_Caps::GetHardwarePixelShaderVersion() < fWATER_SYSTEM_REQUIRED_PIXEL_SHADER)
	{
		Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_WATER_FALLBACK);
	}
	else
	{
		if(s_pxController->m_bBlendHorizon)
		{
			Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_WATER_BLENDED);
		}
		else
		{
			Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_WATER);
		}
	}
	float afPixelConstants[4][4];
	afPixelConstants[0][0] = s_pxController->m_fHorizonBrightness / 128.f;
	afPixelConstants[0][1] = 0.0f;
	afPixelConstants[0][2] = 0.0f;
	afPixelConstants[0][3] = 0.0f;
	afPixelConstants[1][0] = s_pxController->m_fSunBrightness / 128.f;
	afPixelConstants[1][1] = 0.0f;
	afPixelConstants[1][2] = 0.0f;
	afPixelConstants[1][3] = 0.0f;
	afPixelConstants[2][0] = 0.0f;
	afPixelConstants[2][1] = 0.0f;
	afPixelConstants[2][2] = 0.0f;
	afPixelConstants[2][3] = 1.0f;
	afPixelConstants[3][0] = s_pxController->m_fHorizonFresnelScale;
	afPixelConstants[3][1] = 0.0f;
	afPixelConstants[3][2] = 0.0f;
	afPixelConstants[3][3] = 0.0f;
	Asura_Direct3D::D3DDevice->SetPixelShaderConstantF(0, &afPixelConstants[0][0], 4);

	Asura_Render::CurrentStates.RequestTextureIndex(0, s_pxController->m_aiTextureID[0]);
	if(s_pxController->m_bHorizonReflection)
	{
		Asura_Render::CurrentStates.RequestTextureIndex(1, s_pxController->m_aiTextureID[1]);
	}
	else
	{
		Asura_Render::CurrentStates.RequestTextureIndex(1, ASURA_TEXTURE_NONE);
		Asura_Direct3D::D3DDevice->SetTexture(1, s_pxBlackSpot);
	}
	if(s_pxController->m_bSunReflection)
	{
		Asura_Render::CurrentStates.RequestTextureIndex(3, s_pxController->m_aiTextureID[2]);
	}
	else
	{
		Asura_Render::CurrentStates.RequestTextureIndex(3, ASURA_TEXTURE_NONE);
		Asura_Direct3D::D3DDevice->SetTexture(3, s_pxBlackSpot);
	}

	// (DH) 20:Feb:2006 Need to do this otherwise "alphas < 0.6" don't show up
	//
	Asura_Render::CurrentStates.RequestAlphaTest(	false				);
	Asura_Render::CurrentStates.RequestCullMode(	ASURA_CULLMODE_NONE	);


	// no translucency mode for what we want to do...
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	if(s_pxController->m_bTransparency)
	{
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_SRCALPHA);
	}

	// make the sun texture wrap
	Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
	Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);

	// make the sun texture clamped
	Asura_Direct3D::D3DDevice->SetSamplerState(3, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
	Asura_Direct3D::D3DDevice->SetSamplerState(3, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
	Asura_Direct3D::D3DDevice->SetSamplerState(3, D3DSAMP_ADDRESSW, D3DTADDRESS_CLAMP);

	if( Asura_Render::GetWireframe() )
	{
		Asura_Render::CurrentStates.RequestWireFrameMode( true);
		Asura_Render::CurrentStates.RequestTextureOpMode( ASURA_TEXTUREOP_WHITE);
	}

	Asura_Render::CurrentStates.RequestZBufferEnabled( true );
	Asura_Render::CurrentStates.RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( true );
}

void Asura_Water_System::Platform_Render(Asura_Platform_Water_Packet *pPacket)
{
	HRESULT xResult;
	const u_int uBatchDrawMax = 3 * (0x10000 / 3);
	u_int uBatchRemaining = pPacket->m_uVertexCount;
	u_int uBatchOffset = 0;
	
	while( 0 < uBatchRemaining)
	{
		const u_int uBatchDraw = min( uBatchRemaining, uBatchDrawMax);
		Asura_Assert( uBatchDraw <= uBatchRemaining, "Draw Batch is too big!");
		Asura_Assert( (uBatchOffset + uBatchDraw) <= pPacket->m_uVertexCount, "Draw Batch Offset is too big!");
		
		Asura_Direct3D::D3DDevice->SetStreamSource(0, pPacket->m_pxVertices, uBatchOffset * sizeof(Asura_PC_Water_Vertex), sizeof(Asura_PC_Water_Vertex));

		xResult = Asura_Direct3D::D3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, uBatchDraw / 3);
		ASURA_D3D_OK( xResult);
		
		uBatchRemaining -= uBatchDraw;
		uBatchOffset += uBatchDraw;
	}
	
	Asura_Assert( uBatchOffset == pPacket->m_uVertexCount, "Draw Batch Offset misc calc!");
}

void Asura_Water_System::Platform_EndRender(void)
{ 
	Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_NONE);

	if(!s_pxController->m_bHorizonReflection)
	{
		Asura_Direct3D::D3DDevice->SetTexture(1, NULL);
	}
	if(!s_pxController->m_bSunReflection)
	{
		Asura_Direct3D::D3DDevice->SetTexture(3, NULL);
	}

	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_FOGENABLE, FALSE);

	// set the texture addressing back to wrap
	Asura_Direct3D::D3DDevice->SetSamplerState(3, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
	Asura_Direct3D::D3DDevice->SetSamplerState(3, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);
	Asura_Direct3D::D3DDevice->SetSamplerState(3, D3DSAMP_ADDRESSW, D3DTADDRESS_WRAP);

	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);

	D3DPERFCALL(EndEvent());
}

/*******************************************************************************/
/* Asura_Platform_Water_Packet                                                 */
/*******************************************************************************/

Asura_Platform_Water_Packet::Asura_Platform_Water_Packet()
: m_uVertexCount(0), m_uVerticesWritten(0), m_pxVertices(0), m_pxNextVertexToWrite(0)
{
	
}

Asura_Platform_Water_Packet::~Asura_Platform_Water_Packet()
{
	m_uVertexCount = 0;
	Reset();
}

void Asura_Platform_Water_Packet::Reset()
{
	if(m_pxVertices)
	{
		if(m_pxNextVertexToWrite != NULL) m_pxVertices->Unlock();

		m_pxVertices->Release();
		m_pxVertices = NULL;
	}

	m_pxNextVertexToWrite = NULL;
	m_uVerticesWritten = 0;

	if(m_uVertexCount > 0)
	{
		Asura_Direct3D::D3DDevice->CreateVertexBuffer(sizeof(Asura_PC_Water_Vertex) * m_uVertexCount, 0, 0, D3DPOOL_MANAGED, &m_pxVertices, NULL);
		Asura_Assert(m_pxVertices, "Could not create water vertex buffer");
		
		if(m_pxVertices)
		{
			m_pxVertices->Lock(0, 0, reinterpret_cast<void**>(&m_pxNextVertexToWrite), 0);
		}
	}
}

void Asura_Platform_Water_Packet::SetVertexCount(u_int uVertexCount)
{
	m_uVertexCount = uVertexCount;
	Reset();
}

void Asura_Platform_Water_Packet::AddVertex(float fX, float fY, float fZ, float fR, float fG, float fB, float fU, float fV)
{
	Asura_Assert(m_pxNextVertexToWrite, "Water packet is not in a writable state!");
	if(!m_pxNextVertexToWrite) return;

	m_pxNextVertexToWrite->m_xPosition.x = fX;
	m_pxNextVertexToWrite->m_xPosition.y = fY;
	m_pxNextVertexToWrite->m_xPosition.z = fZ;
	m_pxNextVertexToWrite->m_xColour = (Asura_RGBA(fR/255.f, fG/255.f, fB/255.f, 1.0f)).ToUIntARGB();
	m_pxNextVertexToWrite->m_afTexCoords[0] = fU;
	m_pxNextVertexToWrite->m_afTexCoords[1] = fV;

	++m_uVerticesWritten;

	++m_pxNextVertexToWrite;
}

void Asura_Platform_Water_Packet::Close()
{
	Asura_Assert(m_uVerticesWritten == m_uVertexCount, "The number of vertices written isn't what you told me it'd be!");
	if(m_pxVertices && m_pxNextVertexToWrite) 
	{
		m_pxVertices->Unlock();
		m_pxNextVertexToWrite = NULL;
	}
}

#endif
