//
// Asura_PlatformRender_MultiElementSkin.cpp
//
// Created by KJL - 05/11/2003 17:31:43
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_PlatformRender_MultiElementSkin.h"
#include "Rendering/Asura_PlatformRender_MultiElementSkin_Internal.h"


#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_NameTree.h"
#include "Debugging/Asura_Assert.h"

#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_Direct3D_Shaders.h"

#include "Light/Asura_Photon.h"
#include "Core/ASura_Timers.h"



#ifdef ASURA_TOOLS

#include "MeshProcessing/Tools_MultiElementSkin_Processor.h"
#include "Mesh/Asura_Mesh.h"
const Asura_Direct3D_MultiElementSkin* InitialiseSkinFromRawData(const Asura_Hierarchy_Skin* pxSkin)
{
	if (!pxSkin) return 0;

	Asura_Mesh* pxMesh = pxSkin->GetMesh();

	if (!pxMesh) return 0;
	
	Tools_MultiElementSkin_Processor::Initialise(pxMesh,pxSkin,false);
	
	const Tools_MultiElementSkin_Stripified* pxStripifiedSkin = Tools_MultiElementSkin_Processor::GetStripifiedSkin();

	Asura_Direct3D_MultiElementSkin* pxMES =	Asura_Direct3D_MultiElementSkin_Renderer::AddSkin
												(
													pxSkin->Name.GetString(),
													pxStripifiedSkin->m_uTotalNumberOfVertices,
													pxStripifiedSkin->m_uTotalNumberOfIndices,
													pxStripifiedSkin->m_uNumberOfElementStrips
												);
	
	if (pxMES)
	{
		// lock, fill in data and unlock
		Asura_Direct3D_MultiElementSkin_Vertex* pxVertexData;
		pxMES->m_pxVertexBuffer->Lock
		(
			0,
			sizeof(Asura_Direct3D_MultiElementSkin_Vertex)*pxStripifiedSkin->m_uTotalNumberOfVertices,
			(BYTE**)&pxVertexData,
			0
		);
			
		unsigned short* pusIndexData;
		pxMES->m_pxIndexBuffer->Lock
		(
			0,
			sizeof(u_short)*pxStripifiedSkin->m_uTotalNumberOfIndices, 
			(BYTE**)&pusIndexData,
			0
		);

		for (u_int u=0; u<pxStripifiedSkin->m_uTotalNumberOfVertices; u++)
		{
			pxVertexData[u].X = pxStripifiedSkin->m_pxVertices[u].m_xPosition.x;
			pxVertexData[u].Y = pxStripifiedSkin->m_pxVertices[u].m_xPosition.y;
			pxVertexData[u].Z = pxStripifiedSkin->m_pxVertices[u].m_xPosition.z;

			pxVertexData[u].NX = pxStripifiedSkin->m_pxVertices[u].m_xNormal.x;
			pxVertexData[u].NY = pxStripifiedSkin->m_pxVertices[u].m_xNormal.y;
			pxVertexData[u].NZ = pxStripifiedSkin->m_pxVertices[u].m_xNormal.z;
			
			pxVertexData[u].U = pxStripifiedSkin->m_pxVertices[u].m_fU;
			pxVertexData[u].V = pxStripifiedSkin->m_pxVertices[u].m_fV;

			for (u_int v=0; v<4; v++)
			{
				pxVertexData[u].Weight[v] = pxStripifiedSkin->m_pxVertices[u].m_afWeight[v];
				pxVertexData[u].Index[v] = (float)(pxStripifiedSkin->m_pxVertices[u].m_acIndex[v]*3);
			}




		}
		
		for (u_int u=0; u<pxStripifiedSkin->m_uTotalNumberOfIndices; u++)
		{
			pusIndexData[u] = (unsigned short) pxStripifiedSkin->m_puIndices[u];
		}
	
		pxMES->m_pxIndexBuffer->Unlock();
		pxMES->m_pxVertexBuffer->Unlock();

	}


	for (u_int u=0; u<pxStripifiedSkin->m_uNumberOfElementStrips; u++)
	{
		pxMES->m_pxElements[u].m_iMaterialID = pxSkin->GetMesh()->TextureDataPtr[0].m_iMaterialID;
		pxMES->m_pxElements[u].m_uFirstIndexInStrip = pxStripifiedSkin->m_pxElementStrip[u].m_uFirstIndexInStrip;
		pxMES->m_pxElements[u].m_uFirstVertexInStrip = pxStripifiedSkin->m_pxElementStrip[u].m_uFirstIndexInStrip;
		pxMES->m_pxElements[u].m_uNumberOfIndicesInStrip = pxStripifiedSkin->m_pxElementStrip[u].m_uNumberOfIndicesInStrip;
		pxMES->m_pxElements[u].m_uNumberOfVerticesInStrip = pxStripifiedSkin->m_pxElementStrip[u].m_uNumberOfVerticesInStrip;
	}


	Tools_MultiElementSkin_Processor::Shutdown();


	return pxMES;
}
#endif 


static const u_int g_uMaximumNumberOfSkins = 64;

class Asura_Direct3D_MultiElementSkin**	Asura_Direct3D_MultiElementSkin_Renderer::s_ppxSkinStorage;
u_int									Asura_Direct3D_MultiElementSkin_Renderer::s_uNumberOfSkinsStored;
class Asura_NameTree*					Asura_Direct3D_MultiElementSkin_Renderer::s_pxNameTree;

void Asura_Direct3D_MultiElementSkin_Renderer::Initialise(void)
{
	s_pxNameTree = new Asura_NameTree;

	s_ppxSkinStorage = new Asura_Direct3D_MultiElementSkin*[g_uMaximumNumberOfSkins];
	s_uNumberOfSkinsStored = 0;
}

void Asura_Direct3D_MultiElementSkin_Renderer::Shutdown(void)
{
	for (u_int u=0; u<s_uNumberOfSkinsStored; u++)
	{
		delete s_ppxSkinStorage[u];
	}

	delete [] s_ppxSkinStorage;

	s_uNumberOfSkinsStored = 0;
	
	delete s_pxNameTree;
	s_pxNameTree = 0;
}

void Asura_Direct3D_MultiElementSkin_Renderer::Render(const Asura_Hierarchy_Skin* pxSkin, const Asura_Bounding_Box& xLightingBB, const float fAlpha)
{
	const Asura_Direct3D_MultiElementSkin* pxCharacter = GetSkin(pxSkin);

	if (!pxCharacter)
	{
		return; 
	}
		
//	Asura_Render::NumberOfTrianglesRendered += pxCharacter->m_uNumberOfTriangles;

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_4BONES3LIGHTS);

	Asura_Direct3D::D3DDevice->SetStreamSource(0,pxCharacter->m_pxVertexBuffer,sizeof(Asura_Direct3D_MultiElementSkin_Vertex));
	Asura_Direct3D::D3DDevice->SetIndices(pxCharacter->m_pxIndexBuffer,0);

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CW);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);

	Asura_Render::CurrentStates.RequestAlphaTest(false);


	
	{
		D3DXMATRIX mat;
		D3DXMATRIX matView, matProj;

		Asura_Direct3D::D3DDevice->GetTransform( D3DTS_VIEW,       &matView );
		Asura_Direct3D::D3DDevice->GetTransform( D3DTS_PROJECTION, &matProj );

        D3DXMatrixMultiply( &mat, &matView, &matProj );
        D3DXMatrixTranspose( &mat, &mat );

        Asura_Direct3D::D3DDevice->SetVertexShaderConstant(  0, &mat,  4 );
		{
			Asura_Vector_3 xObjectCentre = xLightingBB.CalculateCentre();
			
			float faLights[4*8];

			Asura_Photon_System::MakePhotonListForBoundingBox(xLightingBB,xObjectCentre,true,true,3);
			

			u_int u;

			float* pfLightData = faLights;

			for (u=0; u<Asura_Photon_System::NumberOfLocalPhotons; u++)
			{
				Asura_Vector_3 xV = Asura_Photon_System::LocalPhotons[u].Photon->GetPosition() - xObjectCentre;

				float fScale = 1.0f - (xV.Magnitude()/Asura_Photon_System::LocalPhotons[u].Photon->GetRange());

				if (fScale < 0) fScale = 0.0f;
				
				xV.Normalise();

				*pfLightData++ = xV.x;
				*pfLightData++ = xV.y;
				*pfLightData++ = xV.z;
				*pfLightData++ = 0.0f;
	
				*pfLightData++ = fScale * Asura_Photon_System::LocalPhotons[u].Photon->GetRedComponent();
				*pfLightData++ = fScale * Asura_Photon_System::LocalPhotons[u].Photon->GetGreenComponent();
				*pfLightData++ = fScale * Asura_Photon_System::LocalPhotons[u].Photon->GetBlueComponent();
				*pfLightData++ = 0.0f;
			}
			for ( ; u<3; u++)
			{
				*pfLightData++ = 0.0f;
				*pfLightData++ = 0.0f;
				*pfLightData++ = 0.0f;
				*pfLightData++ = 0.0f;

				*pfLightData++ = 0.0f;
				*pfLightData++ = 0.0f;
				*pfLightData++ = 0.0f;
				*pfLightData++ = 0.0f;
			}
		
			*pfLightData++ = 0.0f;
			*pfLightData++ = 0.0f;
			*pfLightData++ = 0.0f;
			*pfLightData++ = 1.0f;

			Asura_Direct3D::D3DDevice->SetVertexShaderConstant( 4, &faLights, 7 );
		}

		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS_2X);
	
		int c = 12;
		for (int b=1; b<pxSkin->NumberOfBones; b++)
		{
			FLOAT v1[4] = {Asura_Hierarchy_System::s_axBoneOri[b].e11,Asura_Hierarchy_System::s_axBoneOri[b].e12,Asura_Hierarchy_System::s_axBoneOri[b].e13,Asura_Hierarchy_System::s_axBonePos[b].x};
			Asura_Direct3D::D3DDevice->SetVertexShaderConstant( c++, &v1, 1);

			FLOAT v2[4] = {Asura_Hierarchy_System::s_axBoneOri[b].e21,Asura_Hierarchy_System::s_axBoneOri[b].e22,Asura_Hierarchy_System::s_axBoneOri[b].e23,Asura_Hierarchy_System::s_axBonePos[b].y};
			Asura_Direct3D::D3DDevice->SetVertexShaderConstant( c++, &v2, 1);

			FLOAT v3[4] = {Asura_Hierarchy_System::s_axBoneOri[b].e31,Asura_Hierarchy_System::s_axBoneOri[b].e32,Asura_Hierarchy_System::s_axBoneOri[b].e33,Asura_Hierarchy_System::s_axBonePos[b].z};
			Asura_Direct3D::D3DDevice->SetVertexShaderConstant( c++, &v3, 1);
		}
		
	
	}

	u_int uSkip = ((u_int)(Asura_Timers::GetLocalUnscaledTime()*1.0f))%(pxCharacter->m_uNumberOfElements);
	for (u_int u=0; u<pxCharacter->m_uNumberOfElements ; u++)
	{
//		if (u==uSkip) continue;

		Asura_Direct3D_MultiElementSkin_Element* pxElement = &pxCharacter->m_pxElements[u];
		


		Asura_Render::CurrentStates.RequestTextureIndex(0, Asura_Material_System::GetTextureID(pxElement->m_iMaterialID) );

		Asura_Direct3D::D3DDevice->DrawIndexedPrimitive
		(
			D3DPT_TRIANGLESTRIP,
			0,										//	pxElement->m_uFirstVertexInStrip,		//
			pxCharacter->m_uTotalNumberOfVertices,//pxElement->m_uNumberOfVerticesInStrip,	//
			pxElement->m_uFirstIndexInStrip,
			pxElement->m_uNumberOfIndicesInStrip-2
		);
	}
}

Asura_Direct3D_MultiElementSkin* Asura_Direct3D_MultiElementSkin_Renderer::AddSkin(const char* szName, const u_int uNumberOfVertices, const u_int uNumberOfIndices, const u_int uNumberOfElements)
{
	// KJL - 06/11/2003 10:39:16 - allocate a new skin for this name.
	// Returns 0 if there is no more space or if a skin of that name already exists,
	// with the appropriate asserts

	Asura_Assert( szName, "Invalid name");
	if (!szName) return 0;


	int iIndex;
	if (!s_pxNameTree->FindNodeIndexByName(szName,true,iIndex))
	{
		if ( s_uNumberOfSkinsStored>=g_uMaximumNumberOfSkins )
		{
			Asura_Assert( false, "Maximum number of skins exceeded!\nPlease abort now." );
			return 0;
		}

		s_ppxSkinStorage[iIndex] = new Asura_Direct3D_MultiElementSkin(uNumberOfVertices, uNumberOfIndices, uNumberOfElements);
		s_uNumberOfSkinsStored++;
	}
	else
	{
		Asura_Assertf( false, ("A skin of this name <%s> has already been loaded!", szName) )

		return 0;
	}

	return s_ppxSkinStorage[iIndex];
}

bool Asura_Direct3D_MultiElementSkin_Renderer::DeleteSkin(const Asura_Hierarchy_Skin* pxSkin)
{
	if(!pxSkin) return false;

	int iIndex;
	if(s_pxNameTree->DeleteNodeByName(pxSkin->Name,iIndex))
	{
		delete s_ppxSkinStorage[iIndex];
		
		s_ppxSkinStorage[iIndex] = s_ppxSkinStorage[s_uNumberOfSkinsStored-1];
		s_ppxSkinStorage[s_uNumberOfSkinsStored-1] = 0;
		s_uNumberOfSkinsStored--;

		return true;
	}

	return false;
}

const Asura_Direct3D_MultiElementSkin* Asura_Direct3D_MultiElementSkin_Renderer::GetSkin(const Asura_Hierarchy_Skin* pxSkin)
{
	// KJL - 06/11/2003 12:24:17 - get strip for given name; asserts and returns 0 if name not found

	Asura_Assert(pxSkin,"Invalid skin pointer");
	if (!pxSkin) return 0;

	int iIndex;
	if (!s_pxNameTree->FindNodeIndexByName(pxSkin->Name,false,iIndex))
	{
		// KJL - 06/11/2003 12:24:59 - character wasn't found i.e. no character skin of that name has
		// been loaded in. Now, if this is a tools build there's a chance that there'll be an Asura_Mesh 
		// and the associated data around to make a skin ourselves. If this isn't a tools build just
		// return NULL and the character won't be drawn (assuming all the functions that call this handle
		// the case gracefully) - we'll assert so people know there's a problem.

		#ifdef ASURA_TOOLS
		{
			const Asura_Direct3D_MultiElementSkin* pxStrip = InitialiseSkinFromRawData(pxSkin);
			Asura_Assertf(pxStrip, ("A character skin that wasn't loaded as part of an export\ncouldn't be made from existing files either. Skin name: <%s>",pxSkin->Name.GetString()));

			return pxStrip;
		}
		#else
		{
			Asura_Assertf(false, ("No skin for character: <%s>", pxSkin->Name.GetString()) );
			return 0;
		}
		#endif
	}

	return s_ppxSkinStorage[iIndex];

}





// KJL - 06/11/2003 11:07:33 - classes used internally



Asura_Direct3D_MultiElementSkin::Asura_Direct3D_MultiElementSkin(const u_int uNumberOfVertices, const u_int uNumberOfIndices, const u_int uNumberOfElements)
{
	m_uTotalNumberOfVertices	= uNumberOfVertices;
	m_uTotalNumberOfIndices		= uNumberOfIndices;
	m_uNumberOfElements			= uNumberOfElements;

	// default DX buffer ptrs to 0 in case they can't be created
	m_pxVertexBuffer = 0;
	m_pxIndexBuffer  = 0;

	// check DirectX hardware support etc and set flags accordingly
	int flags = D3DUSAGE_WRITEONLY;
	if (!Asura_Direct3D_Caps::HasHardwareVertexShaders())
	{
		flags |= D3DUSAGE_SOFTWAREPROCESSING;
	}

	// Create vertex and index buffers for the skin
	Asura_Direct3D::D3DDevice->CreateVertexBuffer
	(
		m_uTotalNumberOfVertices*sizeof(Asura_Direct3D_MultiElementSkin_Vertex),
		flags,
		0, // no vertex format defn required when using vertex shaders
		D3DPOOL_MANAGED,
		&m_pxVertexBuffer
	);


	Asura_Direct3D::D3DDevice->CreateIndexBuffer
	(
		m_uTotalNumberOfIndices*sizeof(unsigned short),
		flags,
		D3DFMT_INDEX16,
		D3DPOOL_MANAGED,
		&m_pxIndexBuffer
	);
	
	m_pxElements = new Asura_Direct3D_MultiElementSkin_Element[uNumberOfElements];
}

Asura_Direct3D_MultiElementSkin::~Asura_Direct3D_MultiElementSkin()
{
	if (m_pxVertexBuffer)
	{
		m_pxVertexBuffer->Release();
	}
	if (m_pxIndexBuffer)
	{
		m_pxIndexBuffer->Release();
	}

	delete [] m_pxElements;
}
