/**
*
* \file Asura_Platform_MovieToTextureStream.cpp
*
* Created by Kevin Lea - 19/08/2002 17:34:07
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "Core/Asura_String.h"
#include "Debugging/Asura_Logfile.h"
#include "Rendering/Asura_MovieToTexture_System.h"

// Asura_Direct3D.h needs to be included before Asura_Platform_MovieToTextureStream.h (which includes Bink.h)
// Bink defines f32 which clashes with DirectX includes
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Platform_MovieToTextureStream.h"

/// Kevin Lea - 20/08/2002 12:39:11 - evil extern. Need to tidy that part of texture management
extern Direct3D_BaseTexture* evilg_apxTextureStorage[];

#if defined( _WIN64)
	#undef BinkSoundUseDirectSound
	#define BinkSoundUseDirectSound( _A)
	
	#undef BinkGoto
	#define BinkGoto( _A, _B, _C)
	
	#undef BinkWait
	#define BinkWait( _A) (false)
	
	#undef BinkDoFrame
	#define BinkDoFrame( _A)
	
	#undef BinkNextFrame
	#define BinkNextFrame( _A)
	
	#undef BinkOpen
	#define BinkOpen( _A, _B) (NULL)
	
	#undef BinkClose
	#define BinkClose( _A)
	
	#undef BinkPause
	#define BinkPause( _A, _B)
	
	#undef BinkSetVolume
	#define BinkSetVolume( _A, _B, _C)
	
	#undef BinkShouldSkip
	#define BinkShouldSkip( _A) (false)
	
	#undef BinkSetSoundTrack
	#define BinkSetSoundTrack( _A, _B)
	
	#undef BinkCopyToBuffer
	#define BinkCopyToBuffer( _1, _2, _3, _4, _5, _6, _7)
#endif



Asura_Platform_MovieToTextureStream::Asura_Platform_MovieToTextureStream(int iTextureIndex, const char* pxMovieName, bool* pbOK, bool bLooping, bool bHoldAtEnd)
: m_bLooping(bLooping)
, m_bHoldAtEnd(bHoldAtEnd)
, m_bReadyToBeDestroyed(false)
{
	m_iTextureIndex = iTextureIndex;	

	m_pxBinkHandle = BinkOpen(pxMovieName, 0);//BINKPRELOADALL);

	if (!m_pxBinkHandle) 
	{
		Asura_Assertf(false, ("failed to open bink file %s\n", pxMovieName));
		return;
	}

	if (evilg_apxTextureStorage[m_iTextureIndex])
	{
		evilg_apxTextureStorage[m_iTextureIndex]->Release();
	}

	Direct3D_Texture* pxTexture;
	Asura_Direct3D::D3DDevice->CreateTexture(m_pxBinkHandle->Width,m_pxBinkHandle->Height,1,0,D3DFMT_X8R8G8B8,D3DPOOL_MANAGED,&pxTexture,0);
	evilg_apxTextureStorage[m_iTextureIndex] = pxTexture;

	*pbOK = true;
}

Asura_Platform_MovieToTextureStream::~Asura_Platform_MovieToTextureStream()
{
	if (!m_pxBinkHandle) return;

	BinkClose(m_pxBinkHandle);

	if (evilg_apxTextureStorage[m_iTextureIndex])
	{
		evilg_apxTextureStorage[m_iTextureIndex]->Release();
		evilg_apxTextureStorage[m_iTextureIndex] = NULL;
	}
}

void Asura_Platform_MovieToTextureStream::Update(void)
{
	if (!m_pxBinkHandle) return;

	if (!BinkWait(m_pxBinkHandle))
	{
		// Can't LockRect() a BaseTexture
		Direct3D_Texture* pxTexture2D;
		void** const ppTexture2D = reinterpret_cast< void** >( &pxTexture2D );

		if( SUCCEEDED( evilg_apxTextureStorage[ m_iTextureIndex ]->QueryInterface( IID_IDirect3DTexture9, ppTexture2D ) ) )
		{
			D3DLOCKED_RECT rect;

			if(pxTexture2D->LockRect(0,&rect,0,0) == D3D_OK)
			{		
				BinkDoFrame(m_pxBinkHandle);
				BinkCopyToBuffer(m_pxBinkHandle,rect.pBits,rect.Pitch,m_pxBinkHandle->Height,0,0,BINKSURFACE32);

				pxTexture2D->UnlockRect(0);
			}
			else
			{
				Asura_Assert(false, "Lock failed");
			}

			pxTexture2D->Release();
		}

		if((m_pxBinkHandle->FrameNum != m_pxBinkHandle->Frames) || m_bLooping)
		{
			BinkNextFrame(m_pxBinkHandle);
		}
		else if(!m_bHoldAtEnd)
		{
			m_bReadyToBeDestroyed = true;
		}
		
	}

}

/**** Functions for the PC Version of the MovieToTexture_System ****/

void Asura_MovieToTexture_System::Platform_Initialise( void )
{
}

void Asura_MovieToTexture_System::Platform_Shutdown( void )
{
}

void Asura_MovieToTexture_System::Platform_Reset( void )
{
}

void Asura_MovieToTexture_System::Platform_Update( void )
{
	if (s_pxHandleList)
	{
		/// Kevin Lea - 25/09/2002 14:55:10 - independent-esque code: this code be moved to the Update 
		/// method if other platforms are okay with it.
		Asura_Tracking_Handle_List::Iterator xIterator = s_pxHandleList->GetIterator();
		for(xIterator.Begin(); !xIterator.End(); xIterator.Next())
		{
			Asura_Handle iHandle = xIterator.GetCurrentHandle();

			s_ppxStreamStorage[iHandle]->Update();
		}
	}
}
