//*******************************************************************
//
// File: Asura_D3D_PS_Cubic.cpp              
//
// Created: 23 Feb 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_FSFX)

// main header
#include "Rendering/Asura_Direct3D_Shaders.h"

// other libraries

// internal dependencies

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************

//*******************************************************************
// Local Data
//*******************************************************************

static const char* const g_szPS_CUBIC_CODE =

"ps.1.1 \n"

// y = a + b*x + c*x*x + d*x*x*x
//// The Coeffs supplied are half scale 

// Expected constants:
// c0 - 0.5 * coeff a
// c1 - 0.5 * coeff b
// c2 - 0.5 * coeff c
// c3 - 0.5 * coeff d

"tex t0 \n"

// acc = a + b*x 
"mad r0, t0, c1, c0 \n"
// r1 = x*x
"mul r1, t0, t0 \n"
// acc += c*x*x
"mad r0, r1, c2, r0 \n"
// r1 = x*x*x
"mul r1, r1, t0 \n"
// acc += d*x*x*x
"mad_x2 r0, r1, c3, r0 \n"

;

// Register the shader with Asura

const Asura_Direct3D_PixelShader g_xPS_CUBIC_SHADER( ASURA_D3D_PS_CUBIC, g_szPS_CUBIC_CODE);

static const char* const g_szPS_QUADRATIC_CODE =

"ps.1.1 \n"

// y = a + b*x + c*x*x
//// The Coeffs supplied are half scale 

// Expected constants:
// c0 - 0.5 * coeff a
// c1 - 0.5 * coeff b
// c2 - 0.5 * coeff c

"tex t0 \n"

// acc = a + b*x 
"mad r0, t0, c1, c0 \n"
// r1 = x*x
"mul r1, t0, t0 \n"
// acc += c*x*x
"mad_x2 r0, r1, c2, r0 \n"

;

// Register the shader with Asura

const Asura_Direct3D_PixelShader g_xPS_QUADRATIC_SHADER( ASURA_D3D_PS_QUADRATIC, g_szPS_QUADRATIC_CODE);

static const char* const g_szPS_LINEAR_CODE =

"ps.1.1 \n"

// y = a + b*x
//// The Coeffs supplied are half scale 

// Expected constants:
// c0 - 0.5 * coeff a
// c1 - 0.5 * coeff b

"tex t0 \n"

// acc = a + b*x 
"mad_x2 r0, t0, c1, c0 \n"

;

// Register the shader with Asura

const Asura_Direct3D_PixelShader g_xPS_LINEAR_SHADER( ASURA_D3D_PS_LINEAR, g_szPS_LINEAR_CODE);


//*******************************************************************
// Function Definitions
//*******************************************************************


//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_FSFX)
