//
// Asura_D3D_VS_Env_HighEnd.cpp
//
// Created by VGP - 10Mar2006
//
// Copyright (C) 2006 Rebellion (Strangelite), All Rights Reserved.
//
// Vertex shaders for HighEnd environment renderer
//

#include "Core/Asura.h"

#ifdef ASURA_USE_SPECULAR_MAPS

#include "Rendering/Asura_Direct3D_Shaders.h"

static Direct3D_VertexElement auVSD[] =
{
	{	0,	0,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },
	{	0, 12,	D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_NORMAL,	0 },
	{	0, 16,	D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_COLOR,		0 },
	{	0, 20,	D3DDECLTYPE_SHORT2,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	0 },
	{	0, 24,	D3DDECLTYPE_SHORT2,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	1 },
	{	0, 28,	D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	2 },
	{	0, 32,	D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_NORMAL,	1 },
	{	0, 36,	D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_NORMAL,	2 },
	D3DDECL_END()
};


/////////////////////////////////////////////////////////////
//
// Fog
//
/////////////////////////////////////////////////////////////

// Linear fog
//
//	 Used Registers:
//		r0
//
//	 Expected constants:
//		c9      :	Linear fog depth constants
//
#define FOG_LINEAR2(pos, out)					\
	"mad r0.w, -" pos ".w, c9.x, c9.y	\n"		\
	"min r0.w, r0.w, c9.z				\n"		\
	"max " out ", r0.w, c9.w			\n"

#define FOG_LINEAR(pos)							\
	FOG_LINEAR2(pos, "oFog")


// Graph fog
//
//	 Used Registers:
//		r0, r1, a0
//
//	 Expected constants:
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//
#define FOG_GRAPH2(pos, out)														\
	/* c9.x: 1 / (fardistance - neardistance)								*/		\
	/* c9.y: (-neardistance / (fardistance - neardistance))					*/		\
	"mad r1.z, " pos ".w, c9.x, c9.y			\n"									\
	"max r1.z, r1.z, c14.x						\n"									\
	"min r1.z, r1.z, c14.y						\n"									\
																					\
	/* so r1.z is now in the [0..1] range, 1 being farthest and 0 being nearest.*/	\
	/* decide which part of the spline we're in									*/	\
	/* a 0.0 in r1.x means we want c+0, a 1.0 means we want c+1					*/	\
	"sge r0.x, r1.z, c9.z						\n"									\
	"mov a0.x, r0.x								\n"									\
																					\
	/* rescale r1.z for this part of the spline									*/	\
	"mad r1.z, r1.z, c[10+a0.x].x, c[10+a0.x].y	\n"									\
																					\
	/* we want to build (r1.z^3, r1.z^2, r1.z, 1) for easy calc					*/	\
	"mov r1.w, c9.w								\n"									\
	"mul r1.y, r1.z, r1.z						\n"									\
	"mul r1.x, r1.y, r1.z						\n"									\
																					\
	/* dp4 to generate the output value											*/	\
	"dp4 r1, r1, c[12+a0.x]						\n"									\
	"add " out ", c14.y, -r1.x					\n"

#define FOG_GRAPH(pos)																\
	FOG_GRAPH2(pos, "oFog")
	
// Normal Map Vector Transform
//
//	 Used Registers:
//		r1, r2
//
//	 Expected constants:
//		c94     :	Normal decompression values
//
#define 	VECTOR_TO_TANGENTSPACE( i_direction, o_direction, axisU, axisV, axisN)	\
	/* Caclulate Viewvector in tangent space */										\
	"mad r1, "axisU", c94.xxxx, c94.yyyy		\n"		/* Decompress Tangent U */	\
	"dp3 r2.x, r1, "i_direction" \n"												\
	"mad r1, "axisV", c94.xxxx, c94.yyyy		\n"		/* Decompress Tangent V	*/	\
	"dp3 r2.y, r1, "i_direction" \n"												\
	"mad r1, "axisN", c94.xxxx, c94.yyyy		\n"		/* Decompress Normal */		\
	"dp3 r2.z, r1, "i_direction" \n"												\
	/* Normalise Tangent Space View Vector */										\
	"dp3 r1.x, r2.xyz, r2.xyz \n"												\
	"rsq r1.x, r1.x \n"															\
	"mul "o_direction", r2.xyz, r1.x \n"												

/////////////////////////////////////////////////////////////
//
// Ambient lighting + diffuse textures
//
/////////////////////////////////////////////////////////////

// Ambient_Diffuse2_NoFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Diffuse2_NoFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"

	// Transform position by ViewProj
	"m4x4 oPos, v0, c0					\n"

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT1.xy, r0.xy, c8.zw			\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_DIFFUSE2_NOFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_DIFFUSE2_NOFOG,
	szCODE_Ambient_Diffuse2_NoFog,
	auVSD);

// Ambient_Diffuse2_LinearFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c09     :	Linear fog depth constants
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Diffuse2_LinearFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_LINEAR("r0")

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT1.xy, r0.xy, c8.zw			\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_DIFFUSE2_LINEARFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_DIFFUSE2_LINEARFOG,
	szCODE_Ambient_Diffuse2_LinearFog,
	auVSD);

// Ambient_Diffuse2_GraphFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Diffuse2_GraphFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_GRAPH("r0")

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT1.xy, r0.xy, c8.zw			\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_DIFFUSE2_GRAPHFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_DIFFUSE2_GRAPHFOG,
	szCODE_Ambient_Diffuse2_GraphFog,
	auVSD);


/////////////////////////////////////////////////////////////
//
// Ambient lighting + Normal + diffuse textures
//
/////////////////////////////////////////////////////////////

// Ambient_Normal_Diffuse2_NoFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c15     :	Camera position
//
//		c93     :	Bias compression
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Normal_Diffuse2_NoFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"
	"dcl_normal0	v5					\n"
	"dcl_normal1	v6					\n"
	"dcl_normal2	v7					\n"

	// Transform position by ViewProj
	"m4x4 oPos, v0, c0					\n"

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"add oT1.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT2.xy, r0.xy, c8.zw			\n"
	
	// Calculate view vector (in world space)
	"add r0.xyz, c15, -v0					\n"		// View vector = camera pos - vertex pos
	
	VECTOR_TO_TANGENTSPACE( "r0", "r2", "v6", "v7", "v5")

	// Compress the Normal in to a 0 to 1 range
	"mad oT3.xyz, r2.xyz, c93.x, c93.x \n"
	" \n"

};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_NOFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_NOFOG,
	szCODE_Ambient_Normal_Diffuse2_NoFog,
	auVSD);

// Ambient_Normal_Diffuse2_LinearFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c09     :	Linear fog depth constants
//
//		c93     :	Bias compression
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Normal_Diffuse2_LinearFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"
	"dcl_normal0	v5					\n"
	"dcl_normal1	v6					\n"
	"dcl_normal2	v7					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_LINEAR("r0")

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"add oT1.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT2.xy, r0.xy, c8.zw			\n"
	
	// Calculate view vector (in world space)
	"add r0.xyz, c15, -v0					\n"		// View vector = camera pos - vertex pos
	
	VECTOR_TO_TANGENTSPACE( "r0", "r2", "v6", "v7", "v5")

	// Compress the Normal in to a 0 to 1 range
	"mad oT3.xyz, r2.xyz, c93.x, c93.x \n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_LINEARFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_LINEARFOG,
	szCODE_Ambient_Normal_Diffuse2_LinearFog,
	auVSD);

// Ambient_Normal_Diffuse2_GraphFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//
//		c93     :	Bias compression
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Normal_Diffuse2_GraphFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"
	"dcl_normal0	v5					\n"
	"dcl_normal1	v6					\n"
	"dcl_normal2	v7					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_GRAPH("r0")

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"add oT1.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT2.xy, r0.xy, c8.zw			\n"
	
	// Calculate view vector (in world space)
	"add r0.xyz, c15, -v0					\n"		// View vector = camera pos - vertex pos
	
	VECTOR_TO_TANGENTSPACE( "r0", "r2", "v6", "v7", "v5")

	// Compress the Normal in to a 0 to 1 range
	"mad oT3.xyz, r2.xyz, c93.x, c93.x \n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_GRAPHFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_GRAPHFOG,
	szCODE_Ambient_Normal_Diffuse2_GraphFog,
	auVSD);


/////////////////////////////////////////////////////////////
//
// Ambient lighting + diffuse & specular textures
//
/////////////////////////////////////////////////////////////

// Ambient_Diffuse2_Specular_NoFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c15     :	Camera position
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Diffuse2_Specular_NoFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_normal		v1					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"
	"dcl_texcoord2	v5					\n"

	// Transform position by ViewProj
	"m4x4 oPos, v0, c0					\n"

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT1.xy, r0.xy, c8.zw			\n"

	// Calculate view vector (in world space)
	"add oT2, c15, -v0					\n"		// View vector = camera pos - vertex pos

	// Output vertex normal (in world space)
	"mad oT3, v1, c94.xxxx, c94.yyyy	\n"

	// Decompress light direction (in world space)
	"mad oT4, v5, c94.xxxw, c94.yyyz	\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_DIFFUSE2_SPECULAR_NOFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_DIFFUSE2_SPECULAR_NOFOG,
	szCODE_Ambient_Diffuse2_Specular_NoFog,
	auVSD);

// Ambient_Diffuse2_Specular_LinearFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c09     :	Linear fog depth constants
//		c15     :	Camera position
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Diffuse2_Specular_LinearFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_normal		v1					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"
	"dcl_texcoord2	v5					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_LINEAR("r0")

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT1.xy, r0.xy, c8.zw			\n"

	// Calculate view vector (in world space)
	"add oT2, c15, -v0					\n"		// View vector = camera pos - vertex pos

	// Output vertex normal (in world space)
	"mad oT3, v1, c94.xxxx, c94.yyyy	\n"

	// Decompress light direction (in world space)
	"mad oT4, v5, c94.xxxw, c94.yyyz	\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_DIFFUSE2_SPECULAR_LINEARFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_DIFFUSE2_SPECULAR_LINEARFOG,
	szCODE_Ambient_Diffuse2_Specular_LinearFog,
	auVSD);

// Ambient_Diffuse2_Specular_GraphFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//		c15     :	Camera position
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Diffuse2_Specular_GraphFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_normal		v1					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"
	"dcl_texcoord2	v5					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_GRAPH("r0")

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT1.xy, r0.xy, c8.zw			\n"

	// Calculate view vector (in world space)
	"add oT2, c15, -v0					\n"		// View vector = camera pos - vertex pos

	// Output vertex normal (in world space)
	"mad oT3, v1, c94.xxxx, c94.yyyy	\n"

	// Decompress light direction (in world space)
	"mad oT4, v5, c94.xxxw, c94.yyyz	\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_DIFFUSE2_SPECULAR_GRAPHFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_DIFFUSE2_SPECULAR_GRAPHFOG,
	szCODE_Ambient_Diffuse2_Specular_GraphFog,
	auVSD);

/////////////////////////////////////////////////////////////
//
// Ambient lighting + normal + diffuse & specular textures
//
/////////////////////////////////////////////////////////////

// Ambient_Normal_Diffuse2_Specular_NoFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c15     :	Camera position
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Normal_Diffuse2_Specular_NoFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_normal0	v1					\n"
	"dcl_color0	v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"
	"dcl_texcoord2	v5					\n"
	"dcl_normal1	v6					\n"
	"dcl_normal2	v7					\n"

	// Transform position by ViewProj
	"m4x4 oPos, v0, c0					\n"

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT1.xy, r0.xy, c8.zw			\n"

	// Calculate view vector (in world space)
	"add r0, c15, -v0					\n"		// View vector = camera pos - vertex pos
	VECTOR_TO_TANGENTSPACE( "r0", "oT2", "v6", "v7", "v1")

	// Output vertex normal (in world space)
//	"mad oT3, v1, c94.xxxx, c94.yyyy	\n"

	// Decompress light direction (in world space)
	"mad r0, v5, c94.xxxw, c94.yyyz	\n"
	VECTOR_TO_TANGENTSPACE( "r0", "oT3", "v6", "v7", "v1")
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_SPECULAR_NOFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_SPECULAR_NOFOG,
	szCODE_Ambient_Normal_Diffuse2_Specular_NoFog,
	auVSD);

// Ambient_Normal_Diffuse2_Specular_LinearFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c09     :	Linear fog depth constants
//		c15     :	Camera position
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Normal_Diffuse2_Specular_LinearFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_normal0		v1					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"
	"dcl_texcoord2	v5					\n"
	"dcl_normal1	v6					\n"
	"dcl_normal2	v7					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_LINEAR("r0")

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT1.xy, r0.xy, c8.zw			\n"

	// Calculate view vector (in world space)
	"add r0, c15, -v0					\n"		// View vector = camera pos - vertex pos
	VECTOR_TO_TANGENTSPACE( "r0", "oT2", "v6", "v7", "v1")

	// Output vertex normal (in world space)
//	"mad oT3, v1, c94.xxxx, c94.yyyy	\n"

	// Decompress light direction (in world space)
	"mad r0, v5, c94.xxxw, c94.yyyz	\n"
	VECTOR_TO_TANGENTSPACE( "r0", "oT3", "v6", "v7", "v1")
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_SPECULAR_LINEARFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_SPECULAR_LINEARFOG,
	szCODE_Ambient_Normal_Diffuse2_Specular_LinearFog,
	auVSD);

// Ambient_Diffuse2_Specular_GraphFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//		c15     :	Camera position
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Ambient_Normal_Diffuse2_Specular_GraphFog =
{
	"vs.1.1								\n"
	"dcl_position0	v0					\n"
	"dcl_normal		v1					\n"
	"dcl_color0		v2					\n"
	"dcl_texcoord0	v3					\n"
	"dcl_texcoord1	v4					\n"
	"dcl_texcoord2	v5					\n"
	"dcl_normal1	v6					\n"
	"dcl_normal2	v7					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_GRAPH("r0")

	// Output ambient lighting
	"mov oD0, v2						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy	\n" // decompress
	"add oT0.xy, r0.xy, c8.xy			\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy	\n" // decompress
	"add oT1.xy, r0.xy, c8.zw			\n"

	// Calculate view vector (in world space)
	"add r0, c15, -v0					\n"		// View vector = camera pos - vertex pos
	VECTOR_TO_TANGENTSPACE( "r0", "oT2", "v6", "v7", "v1")

	// Output vertex normal (in world space)
//	"mad oT3, v1, c94.xxxx, c94.yyyy	\n"

	// Decompress light direction (in world space)
	"mad r0, v5, c94.xxxw, c94.yyyz	\n"
	VECTOR_TO_TANGENTSPACE( "r0", "oT3", "v6", "v7", "v1")
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_SPECULAR_GRAPHFOG(
	ASURA_D3D_VS_ENV_HIGHEND_AMBIENT_NORMAL_DIFFUSE2_SPECULAR_GRAPHFOG,
	szCODE_Ambient_Normal_Diffuse2_Specular_GraphFog,
	auVSD);

/////////////////////////////////////////////////////////////
//
// Dynamic lighting + diffuse textures
//
/////////////////////////////////////////////////////////////

// Dynamic_Diffuse2_NoFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c16     :	Light position
//		c17     :	Light color
//		c18-c21 :	Projection matrix 1
//		c22-c25 :	Projection matrix 2
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Dynamic_Diffuse2_NoFog =
{
	"vs.1.1									\n"
	"dcl_position0	v0						\n"
	"dcl_normal		v1						\n"
	"dcl_color0		v2						\n"
	"dcl_texcoord0	v3						\n"
	"dcl_texcoord1	v4						\n"

	// Transform position by ViewProj
	"m4x4 oPos, v0, c0						\n"
	
	// DLH - Set the oD1.a to 1.0 as the pixel shader wants it
	"mov oD1.a, v0.w							\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy				\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT1.xy, r0.xy, c8.zw				\n"

	// Lighting
	"add r0.xyz, c16.xyz, -v0.xyz			\n"		// Light vector
	"dp3 r1.w, r0.xyz, r0.xyz				\n"		//
	"rsq r1.w, r1.w							\n"		// Normalize
	"mul r0.xyz, r0.xyz, r1.w				\n"		//

	// Dot with vertex normal
	"mad r1.xyz, v1.xyz, c94.xxxx, c94.yyyy \n"		// Decompress
	"dp3 r0.x, r1.xyz, r0.xyz				\n"
	"max r0.x, r0.x, c95.z					\n"		// Clamp to [0,1]

	// Multiply by light color to get diffuse value, and add emissive
//	"mad oD0.rgb, r0.xxxx, c17, c17			\n"
	"mul oD0.rgb, r0.xxxx, c17			\n"
	"mov oD0.a, v2.a						\n"

	// Texcoords - CameraSpacePosition
	"m4x4 r0, v0, c4						\n"
	"m4x3 oT2.xyz, r0, c18					\n"
	"m4x3 oT3.xyz, r0, c22					\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_DYNAMIC_DIFFUSE2_NOFOG(
	ASURA_D3D_VS_ENV_HIGHEND_DYNAMIC_DIFFUSE2_NOFOG,
	szCODE_Dynamic_Diffuse2_NoFog,
	auVSD);

// Dynamic_Diffuse2_LinearFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c09     :	Linear fog depth constants
//		c16     :	Light position
//		c17     :	Light color
//		c18-c21 :	Projection matrix 1
//		c22-c25 :	Projection matrix 2
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Dynamic_Diffuse2_LinearFog =
{
	"vs.1.1									\n"
	"dcl_position0	v0						\n"
	"dcl_normal		v1						\n"
	"dcl_color0		v2						\n"
	"dcl_texcoord0	v3						\n"
	"dcl_texcoord1	v4						\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_LINEAR2("r0", "oD1.a")

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy				\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT1.xy, r0.xy, c8.zw				\n"

	// Lighting
	"add r0.xyz, c16.xyz, -v0.xyz			\n"		// Light vector
	"dp3 r1.w, r0.xyz, r0.xyz				\n"		//
	"rsq r1.w, r1.w							\n"		// Normalize
	"mul r0.xyz, r0.xyz, r1.w				\n"		//

	// Dot with vertex normal
	"mad r1.xyz, v1.xyz, c94.xxxx, c94.yyyy \n"		// Decompress
	"dp3 r0.x, r1.xyz, r0.xyz				\n"
	"max r0.x, r0.x, c95.z					\n"		// Clamp to [0,1]

	// Multiply by light color to get diffuse value, and add emissive
//	"mad oD0.rgb, r0.xxxx, c17, c17			\n"
	"mul oD0.rgb, r0.xxxx, c17			\n"
	"mov oD0.a, v2.a						\n"

	// Texcoords - CameraSpacePosition
	"m4x4 r0, v0, c4						\n"
	"m4x3 oT2.xyz, r0, c18					\n"
	"m4x3 oT3.xyz, r0, c22					\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_DYNAMIC_DIFFUSE2_LINEARFOG(
	ASURA_D3D_VS_ENV_HIGHEND_DYNAMIC_DIFFUSE2_LINEARFOG,
	szCODE_Dynamic_Diffuse2_LinearFog,
	auVSD);

// Dynamic_Diffuse2_GraphFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//		c16     :	Light position
//		c17     :	Light color
//		c18-c21 :	Projection matrix 1
//		c22-c25 :	Projection matrix 2
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Dynamic_Diffuse2_GraphFog =
{
	"vs.1.1									\n"
	"dcl_position0	v0						\n"
	"dcl_normal		v1						\n"
	"dcl_color0		v2						\n"
	"dcl_texcoord0	v3						\n"
	"dcl_texcoord1	v4						\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_GRAPH2("r0", "oD1.a")

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy				\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT1.xy, r0.xy, c8.zw				\n"

	// Lighting
	"add r0.xyz, c16.xyz, -v0.xyz			\n"		// Light vector
	"dp3 r1.w, r0.xyz, r0.xyz				\n"		//
	"rsq r1.w, r1.w							\n"		// Normalize
	"mul r0.xyz, r0.xyz, r1.w				\n"		//

	// Dot with vertex normal
	"mad r1.xyz, v1.xyz, c94.xxxx, c94.yyyy \n"		// Decompress
	"dp3 r0.x, r1.xyz, r0.xyz				\n"
	"max r0.x, r0.x, c95.z					\n"		// Clamp to [0,1]

	// Multiply by light color to get diffuse value, and add emissive
//	"mad oD0.rgb, r0.xxxx, c17, c17			\n"
	"mul oD0.rgb, r0.xxxx, c17			\n"
	"mov oD0.a, v2.a						\n"

	// Texcoords - CameraSpacePosition
	"m4x4 r0, v0, c4						\n"
	"m4x3 oT2.xyz, r0, c18					\n"
	"m4x3 oT3.xyz, r0, c22					\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_DYNAMIC_DIFFUSE2_GRAPHFOG(
	ASURA_D3D_VS_ENV_HIGHEND_DYNAMIC_DIFFUSE2_GRAPHFOG,
	szCODE_Dynamic_Diffuse2_GraphFog,
	auVSD);

/////////////////////////////////////////////////////////////
//
// Dynamic lighting + normal diffuse textures
//
/////////////////////////////////////////////////////////////

// Dynamic_Normal_Diffuse2_NoFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c16     :	Light position
//		c17     :	Light color
//		c18-c21 :	Projection matrix 1
//		c22-c25 :	Projection matrix 2
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Dynamic_Normal_Diffuse2_NoFog =
{
	"vs.1.1									\n"
	"dcl_position0	v0						\n"
	"dcl_normal		v1						\n"
	"dcl_color0		v2						\n"
	"dcl_texcoord0	v3						\n"
	"dcl_texcoord1	v4						\n"
	"dcl_normal1	v5					\n"
	"dcl_normal2	v6					\n"

	// Transform position by ViewProj
	"m4x4 oPos, v0, c0						\n"
	
	// DLH - Set the oD1.a to 1.0 as the pixel shader wants it
	"mov oD1.a, v0.w							\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy				\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT1.xy, r0.xy, c8.zw				\n"

	// Lighting
	"add r0.xyz, c16.xyz, -v0.xyz			\n"		// Light vector
	VECTOR_TO_TANGENTSPACE( "r0", "oT4", "v5", "v6", "v1")
	
	// Dot with vertex normal
//	"mad r1.xyz, v1.xyz, c94.xxxx, c94.yyyy \n"		// Decompress
//	"dp3 r0.x, r1.xyz, r0.xyz				\n"
//	"max r0.x, r0.x, c95.z					\n"		// Clamp to [0,1]
	
	// Multiply by light color to get diffuse value, and add emissive
//	"mad oD0.rgb, r0.xxxx, c17, c17			\n"
	"mov oD0.rgb, c17			\n"
	"mov oD0.a, v2.a						\n"

	// Texcoords - CameraSpacePosition
	"m4x4 r0, v0, c4						\n"
	"m4x3 oT2.xyz, r0, c18					\n"
	"m4x3 oT3.xyz, r0, c22					\n"
	
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_NOFOG(
	ASURA_D3D_VS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_NOFOG,
	szCODE_Dynamic_Normal_Diffuse2_NoFog,
	auVSD);

// Dynamic_Normal_Diffuse2_LinearFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c09     :	Linear fog depth constants
//		c16     :	Light position
//		c17     :	Light color
//		c18-c21 :	Projection matrix 1
//		c22-c25 :	Projection matrix 2
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Dynamic_Normal_Diffuse2_LinearFog =
{
	"vs.1.1									\n"
	"dcl_position0	v0						\n"
	"dcl_normal		v1						\n"
	"dcl_color0		v2						\n"
	"dcl_texcoord0	v3						\n"
	"dcl_texcoord1	v4						\n"
	"dcl_normal1	v5					\n"
	"dcl_normal2	v6					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_LINEAR2("r0", "r1.x")

	// Multiply by light color to get diffuse value, and add emissive
	"mul oD0.rgb, r1.x, c17			\n"
	"mov oD0.a, v2.a						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy				\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT1.xy, r0.xy, c8.zw				\n"

	// Lighting
	"add r0.xyz, c16.xyz, -v0.xyz			\n"		// Light vector
	VECTOR_TO_TANGENTSPACE( "r0", "oT4", "v5", "v6", "v1")
	
	// Texcoords - CameraSpacePosition
	"m4x4 r0, v0, c4						\n"
	"m4x3 oT2.xyz, r0, c18					\n"
	"m4x3 oT3.xyz, r0, c22					\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_LINEARFOG(
	ASURA_D3D_VS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_LINEARFOG,
	szCODE_Dynamic_Normal_Diffuse2_LinearFog,
	auVSD);

// Dynamic_Normal_Diffuse2_GraphFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//		c16     :	Light position
//		c17     :	Light color
//		c18-c21 :	Projection matrix 1
//		c22-c25 :	Projection matrix 2
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Dynamic_Normal_Diffuse2_GraphFog =
{
	"vs.1.1									\n"
	"dcl_position0	v0						\n"
	"dcl_normal		v1						\n"
	"dcl_color0		v2						\n"
	"dcl_texcoord0	v3						\n"
	"dcl_texcoord1	v4						\n"
	"dcl_normal1	v5					\n"
	"dcl_normal2	v6					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_GRAPH2("r0", "r1.x")

	// Multiply by light color to get diffuse value, and add emissive
	"mul oD0.rgb, r1.x, c17			\n"
	"mov oD0.a, v2.a						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy				\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT1.xy, r0.xy, c8.zw				\n"

	// Lighting
	"add r0.xyz, c16.xyz, -v0.xyz			\n"		// Light vector
	VECTOR_TO_TANGENTSPACE( "r0", "oT4", "v5", "v6", "v1")

	// Texcoords - CameraSpacePosition
	"m4x4 r0, v0, c4						\n"
	"m4x3 oT2.xyz, r0, c18					\n"
	"m4x3 oT3.xyz, r0, c22					\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_GRAPHFOG(
	ASURA_D3D_VS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_GRAPHFOG,
	szCODE_Dynamic_Normal_Diffuse2_GraphFog,
	auVSD);

/////////////////////////////////////////////////////////////
//
// Dynamic lighting + normal diffuse textures + specular textures
//
/////////////////////////////////////////////////////////////

// Dynamic_Normal_Diffuse2_Specular_NoFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c15		:	Camera Position
//		c16     :	Light position
//		c17     :	Light color
//		c18-c21 :	Projection matrix 1
//		c22-c25 :	Projection matrix 2
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Dynamic_Normal_Diffuse2_Specular_NoFog =
{
	"vs.1.1									\n"
	"dcl_position0	v0						\n"
	"dcl_normal		v1						\n"
	"dcl_color0		v2						\n"
	"dcl_texcoord0	v3						\n"
	"dcl_texcoord1	v4						\n"
	"dcl_normal1	v5					\n"
	"dcl_normal2	v6					\n"

	// Transform position by ViewProj
	"m4x4 oPos, v0, c0						\n"
	
	// DLH - Set the oD1.a to 1.0 as the pixel shader wants it
	"mov oD1.a, v0.w							\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy				\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT1.xy, r0.xy, c8.zw				\n"

	// Lighting
	"add r0.xyz, c16.xyz, -v0.xyz			\n"		// Light vector
	VECTOR_TO_TANGENTSPACE( "r0", "oT4", "v5", "v6", "v1")
	
	// View
	"add r0.xyz, c15.xyz, -v0.xyz			\n"		// View vector
	VECTOR_TO_TANGENTSPACE( "r0", "oT5", "v5", "v6", "v1")
	
	// Dot with vertex normal
//	"mad r1.xyz, v1.xyz, c94.xxxx, c94.yyyy \n"		// Decompress
//	"dp3 r0.x, r1.xyz, r0.xyz				\n"
//	"max r0.x, r0.x, c95.z					\n"		// Clamp to [0,1]
	
	// Multiply by light color to get diffuse value, and add emissive
//	"mad oD0.rgb, r0.xxxx, c17, c17			\n"
	"mov oD0.rgb, c17			\n"
	"mov oD0.a, v2.a						\n"

	// Texcoords - CameraSpacePosition
	"m4x4 r0, v0, c4						\n"
	"m4x3 oT2.xyz, r0, c18					\n"
	"m4x3 oT3.xyz, r0, c22					\n"
	
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_SPECULAR_NOFOG(
	ASURA_D3D_VS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_SPECULAR_NOFOG,
	szCODE_Dynamic_Normal_Diffuse2_Specular_NoFog,
	auVSD);

// Dynamic_Normal_Diffuse2_LinearFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c09     :	Linear fog depth constants
//		c15		:	Camera Position
//		c16     :	Light position
//		c17     :	Light color
//		c18-c21 :	Projection matrix 1
//		c22-c25 :	Projection matrix 2
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Dynamic_Normal_Diffuse2_Specular_LinearFog =
{
	"vs.1.1									\n"
	"dcl_position0	v0						\n"
	"dcl_normal		v1						\n"
	"dcl_color0		v2						\n"
	"dcl_texcoord0	v3						\n"
	"dcl_texcoord1	v4						\n"
	"dcl_normal1	v5					\n"
	"dcl_normal2	v6					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_LINEAR2("r0", "r1.x")

	// Multiply by light color to get diffuse value, and add emissive
	"mul oD0.rgb, r1.x, c17			\n"
	"mov oD0.a, v2.a						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy				\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT1.xy, r0.xy, c8.zw				\n"

	// Lighting
	"add r0.xyz, c16.xyz, -v0.xyz			\n"		// Light vector
	VECTOR_TO_TANGENTSPACE( "r0", "oT4", "v5", "v6", "v1")
	
	// View
	"add r0.xyz, c15.xyz, -v0.xyz			\n"		// View vector
	VECTOR_TO_TANGENTSPACE( "r0", "oT5", "v5", "v6", "v1")
	
	// Texcoords - CameraSpacePosition
	"m4x4 r0, v0, c4						\n"
	"m4x3 oT2.xyz, r0, c18					\n"
	"m4x3 oT3.xyz, r0, c22					\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_SPECULAR_LINEARFOG(
	ASURA_D3D_VS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_SPECULAR_LINEARFOG,
	szCODE_Dynamic_Normal_Diffuse2_Specular_LinearFog,
	auVSD);

// Dynamic_Normal_Diffuse2_GraphFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//		c15		:	Camera Position
//		c16     :	Light position
//		c17     :	Light color
//		c18-c21 :	Projection matrix 1
//		c22-c25 :	Projection matrix 2
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Dynamic_Normal_Diffuse2_Specular_GraphFog =
{
	"vs.1.1									\n"
	"dcl_position0	v0						\n"
	"dcl_normal		v1						\n"
	"dcl_color0		v2						\n"
	"dcl_texcoord0	v3						\n"
	"dcl_texcoord1	v4						\n"
	"dcl_normal1	v5					\n"
	"dcl_normal2	v6					\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0					\n"
	"mov oPos, r0						\n"

	// Fog
	FOG_GRAPH2("r0", "r1.x")

	// Multiply by light color to get diffuse value, and add emissive
	"mul oD0.rgb, r1.x, c17			\n"
	"mov oD0.a, v2.a						\n"

	// Calculate texcoords
	"mad r0.xy, v3.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy				\n"
	"mad r0.xy, v4.xy, c95.xx, c95.yy		\n"		// Decompress
	"add oT1.xy, r0.xy, c8.zw				\n"

	// Lighting
	"add r0.xyz, c16.xyz, -v0.xyz			\n"		// Light vector
	VECTOR_TO_TANGENTSPACE( "r0", "oT4", "v5", "v6", "v1")

	// View
	"add r0.xyz, c15.xyz, -v0.xyz			\n"		// View vector
	VECTOR_TO_TANGENTSPACE( "r0", "oT5", "v5", "v6", "v1")
	
	// Texcoords - CameraSpacePosition
	"m4x4 r0, v0, c4						\n"
	"m4x3 oT2.xyz, r0, c18					\n"
	"m4x3 oT3.xyz, r0, c22					\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_SPECULAR_GRAPHFOG(
	ASURA_D3D_VS_ENV_HIGHEND_DYNAMIC_NORMAL_DIFFUSE2_SPECULAR_GRAPHFOG,
	szCODE_Dynamic_Normal_Diffuse2_Specular_GraphFog,
	auVSD);

/////////////////////////////////////////////////////////////
//
// Reflective
//
/////////////////////////////////////////////////////////////

// Reflective_NoFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c16-c19 :	Reflection matrix
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Reflective_NoFog =
{
	"vs.1.1										\n"
	"dcl_position0	v0							\n"
	"dcl_normal		v1							\n"
	"dcl_color0		v2							\n"
	"dcl_texcoord0	v3							\n"

	// Transform position by ViewProj
	"m4x4 oPos, v0, c0							\n"

	// Texcoord0
	"mad r0.xy, v3.xy, c95.xx, c95.yy			\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy					\n"

	// Calculate view vector (in camera space)
	"m4x3 r0, v0, c4							\n"
	"dp3 r1.x, r0, r0							\n"		//
	"rsq r1.x, r1.x								\n"		// Normalize
	"mul r0.xyz, r0.xyz, r1.x					\n"		//

	// Calculate vertex normal (in camera space)
	"mad r2.xyz, v1.xyz, c94.xxxx, c94.yyyy		\n"		// Decompress
	"m3x3 r1, r2, c4							\n"		// Put in view space
	"dp3 r2.x, r1, r1							\n"		//
	"rsq r2.x, r2.x								\n"		// Normalize
	"mul r1.xyz, r1.xyz, r2.x					\n"		//

	// Reflection vector is D - 2(D.N)N
	"dp3 r2.x, r0.xyz, r1.xyz					\n"		// D.N
	"add r2.x, r2.x, r2.x						\n"		// 2(D.N)
	"mul r2.xyz, r2.xxx, r1.xyz					\n"		// 2(D.N)N
	"add r2.xyz, r0.xyz, -r2.xyz				\n"		// D-2(D.N)N
	"mov r2.w, c94.w							\n"		// fix up the w component to be 1

	// texmatrix
	"dp4 oT1.x, r2, c16							\n"
	"dp4 oT1.y, r2, c17							\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_REFLECTIVE_NOFOG(
	ASURA_D3D_VS_ENV_HIGHEND_REFLECTIVE_NOFOG,
	szCODE_Reflective_NoFog,
	auVSD);

// Reflective_LinearFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c09     :	Linear fog depth constants
//		c16-c19 :	Reflection matrix
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Reflective_LinearFog =
{
	"vs.1.1										\n"
	"dcl_position0	v0							\n"
	"dcl_normal		v1							\n"
	"dcl_color0		v2							\n"
	"dcl_texcoord0	v3							\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0							\n"
	"mov oPos, r0								\n"

	// Fog
	FOG_LINEAR("r0")

	// Texcoord0
	"mad r0.xy, v3.xy, c95.xx, c95.yy			\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy					\n"

	// Calculate view vector (in camera space)
	"m4x3 r0, v0, c4							\n"
	"dp3 r1.x, r0, r0							\n"		//
	"rsq r1.x, r1.x								\n"		// Normalize
	"mul r0.xyz, r0.xyz, r1.x					\n"		//

	// Calculate vertex normal (in camera space)
	"mad r2.xyz, v1.xyz, c94.xxxx, c94.yyyy		\n"		// Decompress
	"m3x3 r1, r2, c4							\n"		// Put in view space
	"dp3 r2.x, r1, r1							\n"		//
	"rsq r2.x, r2.x								\n"		// Normalize
	"mul r1.xyz, r1.xyz, r2.x					\n"		//

	// Reflection vector is D - 2(D.N)N
	"dp3 r2.x, r0.xyz, r1.xyz					\n"		// D.N
	"add r2.x, r2.x, r2.x						\n"		// 2(D.N)
	"mul r2.xyz, r2.xxx, r1.xyz					\n"		// 2(D.N)N
	"add r2.xyz, r0.xyz, -r2.xyz				\n"		// D-2(D.N)N
	"mov r2.w, c94.w							\n"		// fix up the w component to be 1

	// texmatrix
	"dp4 oT1.x, r2, c16							\n"
	"dp4 oT1.y, r2, c17							\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_REFLECTIVE_LINEARFOG(
	ASURA_D3D_VS_ENV_HIGHEND_REFLECTIVE_LINEARFOG,
	szCODE_Reflective_LinearFog,
	auVSD);

// Reflective_GraphFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//		c16-c19 :	Reflection matrix
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Reflective_GraphFog =
{
	"vs.1.1										\n"
	"dcl_position0	v0							\n"
	"dcl_normal		v1							\n"
	"dcl_color0		v2							\n"
	"dcl_texcoord0	v3							\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0							\n"
	"mov oPos, r0								\n"

	// Fog
	FOG_GRAPH("r0")

	// Texcoord0
	"mad r0.xy, v3.xy, c95.xx, c95.yy			\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy					\n"

	// Calculate view vector (in camera space)
	"m4x3 r0, v0, c4							\n"
	"dp3 r1.x, r0, r0							\n"		//
	"rsq r1.x, r1.x								\n"		// Normalize
	"mul r0.xyz, r0.xyz, r1.x					\n"		//

	// Calculate vertex normal (in camera space)
	"mad r2.xyz, v1.xyz, c94.xxxx, c94.yyyy		\n"		// Decompress
	"m3x3 r1, r2, c4							\n"		// Put in view space
	"dp3 r2.x, r1, r1							\n"		//
	"rsq r2.x, r2.x								\n"		// Normalize
	"mul r1.xyz, r1.xyz, r2.x					\n"		//

	// Reflection vector is D - 2(D.N)N
	"dp3 r2.x, r0.xyz, r1.xyz					\n"		// D.N
	"add r2.x, r2.x, r2.x						\n"		// 2(D.N)
	"mul r2.xyz, r2.xxx, r1.xyz					\n"		// 2(D.N)N
	"add r2.xyz, r0.xyz, -r2.xyz				\n"		// D-2(D.N)N
	"mov r2.w, c94.w							\n"		// fix up the w component to be 1

	// texmatrix
	"dp4 oT1.x, r2, c16							\n"
	"dp4 oT1.y, r2, c17							\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_REFLECTIVE_GRAPHFOG(
	ASURA_D3D_VS_ENV_HIGHEND_REFLECTIVE_GRAPHFOG,
	szCODE_Reflective_GraphFog,
	auVSD);

/////////////////////////////////////////////////////////////
//
// Reflective Normal
//
/////////////////////////////////////////////////////////////

// Reflective_Normal_NoFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c16-c19 :	Reflection matrix
//		c93     :	Bias compression
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Reflective_Normal_NoFog =
{
	"vs.1.1										\n"
	"dcl_position0	v0							\n"
	"dcl_normal0		v1							\n"
	"dcl_color0		v2							\n"
	"dcl_texcoord0	v3							\n"
	"dcl_normal1	v4					\n"
	"dcl_normal2	v5					\n"

	// Transform position by ViewProj
	"m4x4 oPos, v0, c0							\n"

	// Texcoord0
	"mad r0.xy, v3.xy, c95.xx, c95.yy			\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy					\n"

	// Calculate view vector (in camera space)
//	"m4x3 r0, v0, c4							\n"
	"add r0.xyz, c15.xyz, -v0.xyz			\n"		// View vector
	VECTOR_TO_TANGENTSPACE( "r0", "oT1", "v4", "v5", "v1")
	
	"mov r0.x, c4.x;	\n"
	"mov r0.y, c5.x;	\n"
	"mov r0.z, c6.x;	\n"
	"mov r0.w, v0.w;	\n"
	VECTOR_TO_TANGENTSPACE( "r0", "r4", "v4", "v5", "v1")
	"mov r0.x, c4.y;	\n"
	"mov r0.y, c5.y;	\n"
	"mov r0.z, c6.y;	\n"
	"mov r0.w, v0.w;	\n"
	VECTOR_TO_TANGENTSPACE( "r0", "r5", "v4", "v5", "v1")
	"mov r0.x, c4.z;	\n"
	"mov r0.y, c5.z;	\n"
	"mov r0.z, c6.z;	\n"
	"mov r0.w, v0.w;	\n"
	VECTOR_TO_TANGENTSPACE( "r0", "r6", "v4", "v5", "v1")
	
	"mad oT2.xyzw, r4, c93.xxxw, c93.wwwy; \n"
	"mad oT3.xyzw, r5, c93.xxxw, c93.wwwy; \n"

	"mov r0, c4; \n"
	"mov r0, c93.xwwx; \n"
	VECTOR_TO_TANGENTSPACE( "r0", "r0", "v4", "v5", "v1")
	"mad oT2, r0, c93.xxxw, c93.wwww; \n"
	
	"mov r0, c5; \n"
	"mov r0, c93.wxwx; \n"
	VECTOR_TO_TANGENTSPACE( "r0", "r0", "v4", "v5", "v1")
	"mad oT3, r0, c93.xxxw, c93.wwww; \n"

#if 0
	// Calculate vertex normal (in camera space)
	"mad r2.xyz, v1.xyz, c94.xxxx, c94.yyyy		\n"		// Decompress
	"m3x3 r1, r2, c4							\n"		// Put in view space
	"dp3 r2.x, r1, r1							\n"		//
	"rsq r2.x, r2.x								\n"		// Normalize
	"mul r1.xyz, r1.xyz, r2.x					\n"		//

	// Reflection vector is D - 2(D.N)N
	"dp3 r2.x, r0.xyz, r1.xyz					\n"		// D.N
	"add r2.x, r2.x, r2.x						\n"		// 2(D.N)
	"mul r2.xyz, r2.xxx, r1.xyz					\n"		// 2(D.N)N
	"add r2.xyz, r0.xyz, -r2.xyz				\n"		// D-2(D.N)N
	"mov r2.w, c94.w							\n"		// fix up the w component to be 1

	// texmatrix
	"dp4 oT1.x, r2, c16							\n"
	"dp4 oT1.y, r2, c17							\n"
#endif
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_REFLECTIVE_NORMAL_NOFOG(
	ASURA_D3D_VS_ENV_HIGHEND_REFLECTIVE_NORMAL_NOFOG,
	szCODE_Reflective_Normal_NoFog,
	auVSD);

// Reflective_LinearFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c09     :	Linear fog depth constants
//		c16-c19 :	Reflection matrix
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Reflective_Normal_LinearFog =
{
	"vs.1.1										\n"
	"dcl_position0	v0							\n"
	"dcl_normal		v1							\n"
	"dcl_color0		v2							\n"
	"dcl_texcoord0	v3							\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0							\n"
	"mov oPos, r0								\n"

	// Fog
	FOG_LINEAR("r0")

	// Texcoord0
	"mad r0.xy, v3.xy, c95.xx, c95.yy			\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy					\n"

	// Calculate view vector (in camera space)
	"m4x3 r0, v0, c4							\n"
	"dp3 r1.x, r0, r0							\n"		//
	"rsq r1.x, r1.x								\n"		// Normalize
	"mul r0.xyz, r0.xyz, r1.x					\n"		//

	// Calculate vertex normal (in camera space)
	"mad r2.xyz, v1.xyz, c94.xxxx, c94.yyyy		\n"		// Decompress
	"m3x3 r1, r2, c4							\n"		// Put in view space
	"dp3 r2.x, r1, r1							\n"		//
	"rsq r2.x, r2.x								\n"		// Normalize
	"mul r1.xyz, r1.xyz, r2.x					\n"		//

	// Reflection vector is D - 2(D.N)N
	"dp3 r2.x, r0.xyz, r1.xyz					\n"		// D.N
	"add r2.x, r2.x, r2.x						\n"		// 2(D.N)
	"mul r2.xyz, r2.xxx, r1.xyz					\n"		// 2(D.N)N
	"add r2.xyz, r0.xyz, -r2.xyz				\n"		// D-2(D.N)N
	"mov r2.w, c94.w							\n"		// fix up the w component to be 1

	// texmatrix
	"dp4 oT1.x, r2, c16							\n"
	"dp4 oT1.y, r2, c17							\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_REFLECTIVE_NORMAL_LINEARFOG(
	ASURA_D3D_VS_ENV_HIGHEND_REFLECTIVE_NORMAL_LINEARFOG,
	szCODE_Reflective_Normal_LinearFog,
	auVSD);

// Reflective_GraphFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c04-c07 :	View matrix
//		c08     :	Tex coord offsets
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//		c16-c19 :	Reflection matrix
//		c95     :	Texcoord decompression values
//
const char* const szCODE_Reflective_Normal_GraphFog =
{
	"vs.1.1										\n"
	"dcl_position0	v0							\n"
	"dcl_normal		v1							\n"
	"dcl_color0		v2							\n"
	"dcl_texcoord0	v3							\n"

	// Transform position by ViewProj
	"m4x4 r0, v0, c0							\n"
	"mov oPos, r0								\n"

	// Fog
	FOG_GRAPH("r0")

	// Texcoord0
	"mad r0.xy, v3.xy, c95.xx, c95.yy			\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy					\n"

	// Calculate view vector (in camera space)
	"m4x3 r0, v0, c4							\n"
	"dp3 r1.x, r0, r0							\n"		//
	"rsq r1.x, r1.x								\n"		// Normalize
	"mul r0.xyz, r0.xyz, r1.x					\n"		//

	// Calculate vertex normal (in camera space)
	"mad r2.xyz, v1.xyz, c94.xxxx, c94.yyyy		\n"		// Decompress
	"m3x3 r1, r2, c4							\n"		// Put in view space
	"dp3 r2.x, r1, r1							\n"		//
	"rsq r2.x, r2.x								\n"		// Normalize
	"mul r1.xyz, r1.xyz, r2.x					\n"		//

	// Reflection vector is D - 2(D.N)N
	"dp3 r2.x, r0.xyz, r1.xyz					\n"		// D.N
	"add r2.x, r2.x, r2.x						\n"		// 2(D.N)
	"mul r2.xyz, r2.xxx, r1.xyz					\n"		// 2(D.N)N
	"add r2.xyz, r0.xyz, -r2.xyz				\n"		// D-2(D.N)N
	"mov r2.w, c94.w							\n"		// fix up the w component to be 1

	// texmatrix
	"dp4 oT1.x, r2, c16							\n"
	"dp4 oT1.y, r2, c17							\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_REFLECTIVE_NORMAL_GRAPHFOG(
	ASURA_D3D_VS_ENV_HIGHEND_REFLECTIVE_NORMAL_GRAPHFOG,
	szCODE_Reflective_Normal_GraphFog,
	auVSD);

/////////////////////////////////////////////////////////////
//
// Z Pass
//
/////////////////////////////////////////////////////////////

//
// ZOnly
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c95     :	Texcoord decompression values
//
const char* const szCODE_ZOnly =
{
	"vs.1.1										\n"
	"dcl_position0	v0							\n"
	"dcl_texcoord0	v3							\n"

	// Transform position by ViewProj
	"m4x4 oPos, v0, c0							\n"

	// Texcoord0
	"mad r0.xy, v3.xy, c95.xx, c95.yy			\n"		// Decompress
	"add oT0.xy, r0.xy, c8.xy					\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_ZONLY(
	ASURA_D3D_VS_ENV_HIGHEND_ZONLY,
	szCODE_ZOnly,
	auVSD);

/////////////////////////////////////////////////////////////
//
// Flowing Water
//
/////////////////////////////////////////////////////////////

//
// FlowingWater_NoFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c15     :	Camera position
//		c16.x    :	Constant k0 [= (amp u) sin (phase u)]
//		c16.y    :	Constant k1 [= (amp v) sin (phase v)]
//		c16.z    :	8 pi
//		c17     :	(pi, 1/2, 2 pi, 1 / (2 pi)		(Constants for wrap into [-Pi, Pi])
//		c18     :	(1, -1/(2!), 1/(4!), -1/(6!)	(Constants for cos Taylor series expansion)
//		c19     :	(1, -1/(3!), 1/(5!), -1/(7!)	(Constants for sin Taylor series expansion)
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_FlowingWater_NoFog =
{
	// y coord is perturbed by k0 cos (2 pi u) + k1 sin (8 pi v)
	"vs.1.1									\n"

	"dcl_position	v0 						\n"
	"dcl_normal		v1 						\n"
	"dcl_color		v2 						\n"
	"dcl_texcoord0	v3 						\n"

	// Get the position into r3 so we can modify the y-coord later
	"mov r3, v0								\n"

	// decompress texcoords into r4
	"mad r4.xy, v3.xy, c95.xx, c95.yy		\n"

	// Get 2 pi * u tex coord into r1.x
	"mul r1.x, c17.z, r4.x					\n"

	// Wrap it to [-Pi, Pi]
	"mad  r0.x,  r1.x, c17.w,  c17.y		\n"
	"expp r0.y,  r0.x						\n"
	"mad  r0.x,  r0.y, c17.z, -c17.x		\n"

	// Get the cos of it into r0.x
	"dst  r2.xy, r0.x, r0.x					\n"	// generate 1, (r0.x)^2, .. (r0.x)^6
	"mul  r2.z,  r2.y, r2.y					\n"
	"mul  r2.w,  r2.y, r2.z					\n"
	"dp4  r0.x,  r2,   c18					\n" // compute cos(r0.x)

	// Scale it by k0 and add to y-coord
	"mad r3.y, c16.x, r0.x, r3.y				\n"

	// Get 8 pi * v tex coord into r1.x
	"mul r1.x, c16.z, r4.y					\n"

	// Wrap it to [-Pi, Pi]
	"mad  r0.x,  r1.x, c17.w,  c17.y		\n"
	"expp r0.y,  r0.x						\n"
	"mad  r0.x,  r0.y, c17.z, -c17.x		\n"

	// Get the sin of it into r0.y
	"dst  r2.xy, r0.x, r0.x					\n" // generate 1, (r0.x)^2, .. (r0.x)^6
	"mul  r2.z,  r2.y, r2.y					\n"
	"mul  r2.w,  r2.y, r2.z					\n"
	"mul  r0,    r2,   r0.x					\n" // generate r0.x, (r0.x)^3, .., (r0.x)^7 
	"dp4  r0.y,  r0,   c19					\n" // compute sin(r0.x) 

	// Scale it by k1 and add to y-coord
	"mad r3.y, c16.y, r0.y, r3.y				\n"

	// Shove into output registers, shifting tex coords
	"m4x4 oPos, r3, c0						\n"
	"mov oD0, v2							\n"
	"add oT0.xy, r4.xy, c8.xy				\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_FLOWINGWATER_NOFOG(
	ASURA_D3D_VS_ENV_HIGHEND_FLOWINGWATER_NOFOG,
	szCODE_FlowingWater_NoFog,
	auVSD);

//
// FlowingWater_LinearFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c09     :	Linear fog depth constants
//		c15     :	Camera position
//		c16.x    :	Constant k0 [= (amp u) sin (phase u)]
//		c16.y    :	Constant k1 [= (amp v) sin (phase v)]
//		c16.z    :	8 pi
//		c17     :	(pi, 1/2, 2 pi, 1 / (2 pi)		(Constants for wrap into [-Pi, Pi])
//		c18     :	(1, -1/(2!), 1/(4!), -1/(6!)	(Constants for cos Taylor series expansion)
//		c19     :	(1, -1/(3!), 1/(5!), -1/(7!)	(Constants for sin Taylor series expansion)
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_FlowingWater_LinearFog =
{
	// y coord is perturbed by k0 cos (2 pi u) + k1 sin (8 pi v)
	"vs.1.1									\n"

	"dcl_position	v0 						\n"
	"dcl_normal		v1 						\n"
	"dcl_color		v2 						\n"
	"dcl_texcoord0	v3 						\n"

	// Get the position into r3 so we can modify the y-coord later
	"mov r3, v0								\n"

	// decompress texcoords into r4
	"mad r4.xy, v3.xy, c95.xx, c95.yy		\n"

	// Get 2 pi * u tex coord into r1.x
	"mul r1.x, c17.z, r4.x					\n"

	// Wrap it to [-Pi, Pi]
	"mad  r0.x,  r1.x, c17.w,  c17.y		\n"
	"expp r0.y,  r0.x						\n"
	"mad  r0.x,  r0.y, c17.z, -c17.x		\n"

	// Get the cos of it into r0.x
	"dst  r2.xy, r0.x, r0.x					\n"	// generate 1, (r0.x)^2, .. (r0.x)^6
	"mul  r2.z,  r2.y, r2.y					\n"
	"mul  r2.w,  r2.y, r2.z					\n"
	"dp4  r0.x,  r2,   c18					\n" // compute cos(r0.x)

	// Scale it by k0 and add to y-coord
	"mad r3.y, c16.x, r0.x, r3.y				\n"

	// Get 8 pi * v tex coord into r1.x
	"mul r1.x, c16.z, r4.y					\n"

	// Wrap it to [-Pi, Pi]
	"mad  r0.x,  r1.x, c17.w,  c17.y		\n"
	"expp r0.y,  r0.x						\n"
	"mad  r0.x,  r0.y, c17.z, -c17.x		\n"

	// Get the sin of it into r0.y
	"dst  r2.xy, r0.x, r0.x					\n" // generate 1, (r0.x)^2, .. (r0.x)^6
	"mul  r2.z,  r2.y, r2.y					\n"
	"mul  r2.w,  r2.y, r2.z					\n"
	"mul  r0,    r2,   r0.x					\n" // generate r0.x, (r0.x)^3, .., (r0.x)^7 
	"dp4  r0.y,  r0,   c19					\n" // compute sin(r0.x) 

	// Scale it by k1 and add to y-coord
	"mad r3.y, c16.y, r0.y, r3.y				\n"

	// Shove into output registers, shifting tex coords
	"m4x4 r0, r3, c0						\n"
	FOG_LINEAR("r0")
	"mov oPos, r0							\n"
	"mov oD0, v2							\n"
	"add oT0.xy, r4.xy, c8.xy				\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_FLOWINGWATER_LINEARFOG(
	ASURA_D3D_VS_ENV_HIGHEND_FLOWINGWATER_LINEARFOG,
	szCODE_FlowingWater_LinearFog,
	auVSD);

//
// FlowingWater_GraphFog
//
//	 Expected constants:
//		c00-c03 :	ViewProj matrix
//		c08     :	Tex coord offsets
//		c09     :	Fog values (xyz)
//		c10     :	Scaling values to map full-range value to first-part-of-spline
//		c11     :	Scaling values to map full-range value to second-part-of-spline
//		c12     :	Fog abcv1 vector for points between the near plane and centerpoint
//		c13     :	Fog abcv1 vector for points between centerpoint and far plane
//		c14     :	0.0, 1.0 (xy)
//		c16.x    :	Constant k0 [= (amp u) sin (phase u)]
//		c16.y    :	Constant k1 [= (amp v) sin (phase v)]
//		c16.z    :	8 pi
//		c17     :	(pi, 1/2, 2 pi, 1 / (2 pi)		(Constants for wrap into [-Pi, Pi])
//		c18     :	(1, -1/(2!), 1/(4!), -1/(6!)	(Constants for cos Taylor series expansion)
//		c19     :	(1, -1/(3!), 1/(5!), -1/(7!)	(Constants for sin Taylor series expansion)
//		c94     :	Normal decompression values
//		c95     :	Texcoord decompression values
//
const char* const szCODE_FlowingWater_GraphFog =
{
	// y coord is perturbed by k0 cos (2 pi u) + k1 sin (8 pi v)
	"vs.1.1									\n"

	"dcl_position	v0 						\n"
	"dcl_normal		v1 						\n"
	"dcl_color		v2 						\n"
	"dcl_texcoord0	v3 						\n"

	// Get the position into r3 so we can modify the y-coord later
	"mov r3, v0								\n"

	// decompress texcoords into r4
	"mad r4.xy, v3.xy, c95.xx, c95.yy		\n"

	// Get 2 pi * u tex coord into r1.x
	"mul r1.x, c17.z, r4.x					\n"

	// Wrap it to [-Pi, Pi]
	"mad  r0.x,  r1.x, c17.w,  c17.y		\n"
	"expp r0.y,  r0.x						\n"
	"mad  r0.x,  r0.y, c17.z, -c17.x		\n"

	// Get the cos of it into r0.x
	"dst  r2.xy, r0.x, r0.x					\n"	// generate 1, (r0.x)^2, .. (r0.x)^6
	"mul  r2.z,  r2.y, r2.y					\n"
	"mul  r2.w,  r2.y, r2.z					\n"
	"dp4  r0.x,  r2,   c18					\n" // compute cos(r0.x)

	// Scale it by k0 and add to y-coord
	"mad r3.y, c16.x, r0.x, r3.y				\n"

	// Get 8 pi * v tex coord into r1.x
	"mul r1.x, c16.z, r4.y					\n"

	// Wrap it to [-Pi, Pi]
	"mad  r0.x,  r1.x, c17.w,  c17.y		\n"
	"expp r0.y,  r0.x						\n"
	"mad  r0.x,  r0.y, c17.z, -c17.x		\n"

	// Get the sin of it into r0.y
	"dst  r2.xy, r0.x, r0.x					\n" // generate 1, (r0.x)^2, .. (r0.x)^6
	"mul  r2.z,  r2.y, r2.y					\n"
	"mul  r2.w,  r2.y, r2.z					\n"
	"mul  r0,    r2,   r0.x					\n" // generate r0.x, (r0.x)^3, .., (r0.x)^7 
	"dp4  r0.y,  r0,   c19					\n" // compute sin(r0.x) 

	// Scale it by k1 and add to y-coord
	"mad r3.y, c16.y, r0.y, r3.y				\n"

	// Shove into output registers, shifting tex coords
	"m4x4 r0, r3, c0						\n"
	FOG_GRAPH("r0")
	"mov oPos, r0							\n"
	"mov oD0, v2							\n"
	"add oT0.xy, r4.xy, c8.xy				\n"
};

const Asura_Direct3D_VertexShader xVS_ENV_HIGHEND_FLOWINGWATER_GRAPHFOG(
	ASURA_D3D_VS_ENV_HIGHEND_FLOWINGWATER_GRAPHFOG,
	szCODE_FlowingWater_GraphFog,
	auVSD);




#endif // ASURA_USE_SPECULAR_MAPS
