#include "Core/Asura.h"

#include "Rendering/Asura_Direct3D_Shaders.h"

// Vertex stream declaration for environment vertices
/*
const DWORD auVSD[] =
{
	D3DVSD_STREAM(0),
		D3DVSD_REG(0, D3DVSDT_FLOAT3),		// Position
		D3DVSD_REG(1, D3DVSDT_FLOAT3),		// Normal
		D3DVSD_REG(2, D3DVSDT_D3DCOLOR),	// Color
		D3DVSD_REG(3, D3DVSDT_FLOAT2),		// Texcoord 1
		D3DVSD_REG(4, D3DVSDT_FLOAT2),		// Texcoord 2
	D3DVSD_END()
};
*/
static Direct3D_VertexElement auVSD[] =
{
	{	0,	0,	D3DDECLTYPE_FLOAT3,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },
	{	0, 12,	D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_NORMAL,	0 },
	{	0, 16,	D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_COLOR,		0 },
	{	0, 20,	D3DDECLTYPE_FLOAT2,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	0 },
	{	0, 28,	D3DDECLTYPE_FLOAT2,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	1 },
	D3DDECL_END()
};

const char* const szVS_ENVIRONMENT_LINEARFOG_CODE =

// Environment linear fog shader
// Expected constants:
//  c0-c3: ViewProj matrix
//  c4   : White
//  c5   : Linear fog depth constants

"vs.1.1 \n"
"dcl_position0	v0 \n"

// Do the position
"m4x4 r1, v0, c0 \n"
"mov oPos, r1 \n"
// Fog
"mov oD0, c4 \n"
"mad r0.w, r1.w, c5.x, c5.y \n"
"max r0.w, r0.w, c5.z \n"
"min oD0.w, r0.w, c5.w \n";

// Register the shader with Asura

const Asura_Direct3D_VertexShader xVS_ENVIRONMENT_LINEARFOG_SHADER(ASURA_D3D_VS_ENVIRONMENT_LINEARFOG, szVS_ENVIRONMENT_LINEARFOG_CODE, auVSD);

const char* const szVS_ENVIRONMENT_GRAPHFOG_CODE =

// Environment graph fog shader
// Expected constants:
//  c0-c3: ViewProj matrix
//  c4   : White
//  c5:    Fog values (xyz)
//  c6:    Scaling values to map full-range value to first-part-of-spline
//  c7:    Scaling values to map full-range value to second-part-of-spline
//  c8:    Fog abcv1 vector for points between the near plane and centerpoint
//  c9:    Fog abcv1 vector for points between centerpoint and far plane
// c10:    0.0, 1.0 (xy)

"vs.1.1 \n"
"dcl_position0	v0 \n"

// Do the position
"m4x4 r2, v0, c0 \n"
"mov oPos, r2 \n"
// Fog
"mov oD0, c4 \n"

// our distance from the camera (in metres) is in r12.w (oPos.w)
// c4.x: 1 / (fardistance - neardistance)
// c4.y: (-neardistance / (fardistance - neardistance))
"mad r1.z, r2.w, c5.x, c5.y \n"
"max r1.z, r1.z, c10.x \n"
"min r1.z, r1.z, c10.y \n"
// so r1.z is now in the [0..1] range, 1 being farthest and 0 being nearest.
// decide which part of the spline we're in
// a 0.0 in r1.x means we want c+0, a 1.0 means we want c+1
"sge r0.x, r1.z, c5.z \n"
"mov a0.x, r0.x \n"
// rescale r1.z for this part of the spline
"mad r1.z, r1.z, c[6+a0.x].x, c[6+a0.x].y \n"
// we want to build (r1.z^3, r1.z^2, r1.z, 1) for easy calc
"mov r1.w, c5.w \n"
"mul r1.y, r1.z, r1.z \n"
"mul r1.x, r1.y, r1.z \n"
// dp4 to generate the output value
"dp4 oD0.w, r1, c[8+a0.x] \n";

// Register the shader with Asura

const Asura_Direct3D_VertexShader xVS_ENVIRONMENT_GRAPHFOG_SHADER(ASURA_D3D_VS_ENVIRONMENT_GRAPHFOG, szVS_ENVIRONMENT_GRAPHFOG_CODE, auVSD);

const char* const szVS_ENVIRONMENT_LIGHTING_CODE =

// Environment shader
// Emulates the fixed function pipeline
// Expected constants:
//  c00-c03: ViewProj matrix
//  c04-c07: View matrix
//  c08-c11: Tex0 matrix
//  c12-c15: Tex1 matrix
//  c16:     Light position
//  c17:     Light color
//  c18:	 Zero

"vs.1.1 \n"
"dcl_position0	v0 \n"
"dcl_normal		v1 \n"

// Do the position
"m4x4 oPos, v0, c0 \n"
// Lighting - vector from light to this vertex into r0
"sub r0.xyz, c16.xyz, v0.xyz \n"
// normalise r0
"dp3 r1.w, r0.xyz, r0.xyz \n"
"rsq r1.w, r1.w \n"
"mul r0.xyz, r0.xyz, r1.w \n"
// decompress and dot with normal
"mad r1.xyz, v1.xyz, c94.xxxx, c94.yyyy \n"
"dp3 r0.x, r1.xyz, r0.xyz \n"
// multiply by light color to get diffuse value, and add emissive
//"mad r0, r0.xxxx, c17, c17 \n"
"mul r0, r0.xxxx, c17 \n"
// clamp the result to 0.0f
"max oD0, r0, c18 \n"
// Texcoords - CameraSpacePosition
"m4x4 r0, v0, c4 \n"
"m4x3 oT0.xyz, r0, c8 \n"
"m4x3 oT1.xyz, r0, c12 \n";

// Register the shader with Asura

const Asura_Direct3D_VertexShader xVS_ENVIRONMENT_LIGHTING_SHADER(ASURA_D3D_VS_ENVIRONMENT_LIGHTING, szVS_ENVIRONMENT_LIGHTING_CODE, auVSD);

const char* const szVS_ENVIRONMENT_AMBIENT_CODE =

// Environment Ambient shader
// Emulates the fixed function pipeline, no lighting
// Expected constants:
//  c0-c3: ViewProj matrix

"vs.1.1 \n"
"dcl_position0	v0 \n"
"dcl_color0		v2 \n"
"dcl_texcoord	v3 \n"
// Do the position
"m4x4 oPos, v0, c0 \n"
// Diffuse
"mov oD0, v2 \n"
// Texcoords
"add oT0.xy, v3.xy, c4.xy \n";
//"mov oT1.w, r12.w \n";

// Register the shader with Asura

const Asura_Direct3D_VertexShader xVS_ENVIRONMENT_AMBIENT_SHADER(ASURA_D3D_VS_ENVIRONMENT_AMBIENT, szVS_ENVIRONMENT_AMBIENT_CODE, auVSD);

const char* const szVS_ENVIRONMENT_TEXTURE_CODE =

// Environment Texture shader
// Emulates the fixed function pipeline, no lighting, textures
// Expected constants:
//  c0-c3:  ViewProj matrix
//  c8-c11: Tex0 matrix
//  c12-c15:  Tex1 matrix

"vs.1.1 \n"
"dcl_position0	v0 \n"
"dcl_color0		v2 \n"
"dcl_texcoord0	v3 \n"
"dcl_texcoord1	v4 \n"
// Do the position
"m4x4 oPos, v0, c0 \n"

// Diffuse
"mov oD0, v2 \n"

// Texcoords
"m4x4 oT0, v3, c8 \n"
"m4x4 oT1, v4, c12 \n";
//"mov oT1, v4 \n";

// Register the shader with Asura

const Asura_Direct3D_VertexShader xVS_ENVIRONMENT_TEXTURE_SHADER(ASURA_D3D_VS_ENVIRONMENT_TEXTURES, szVS_ENVIRONMENT_TEXTURE_CODE, auVSD);

const char* const szVS_ENVIRONMENT_TEXTURE_DUPLICATECOORDS_CODE =

// Same as the one above but UV0 input is duplicated for both stages 0 and 1

"vs.1.1 \n"
"dcl_position0	v0 \n"
"dcl_color0		v2 \n"
"dcl_texcoord0	v3 \n"
// Do the position
"m4x4 oPos, v0, c0 \n"

// Diffuse
"mov oD0, v2 \n"

// Texcoords
"m4x4 oT0, v3, c8 \n"
"m4x4 oT1, v3, c12 \n";

// Register the shader with Asura

const Asura_Direct3D_VertexShader xVS_ENVIRONMENT_TEXTURE_DUPLICATECOORDS_SHADER(ASURA_D3D_VS_ENVIRONMENT_TEXTURE_DUPLICATECOORDS, szVS_ENVIRONMENT_TEXTURE_DUPLICATECOORDS_CODE, auVSD);

const char* const szVS_ENVIRONMENT_REFLECTIVE_CODE =

// Environment Reflective shader
// Emulates the fixed function pipeline, no lighting, generates camera space reflection texcoord1
// Expected constants:
//  c0-c3: ViewProj matrix
//  c4-c7: View matrix
//  c8:    Constants (2.0f, 1.0f, *, *)
//  c9-13  Refl matrix

"vs.1.1 \n"
"dcl_position0	v0 \n"
"dcl_normal		v1 \n"
"dcl_color0		v2 \n"
"dcl_texcoord0	v3 \n"
//"dcl_texcoord1	v4 \n"
// Do the position
"m4x4 oPos, v0, c0 \n"

// Diffuse
"mov oD0, v2 \n"

// Texcoord0
"m4x4 oT0, v3, c14 \n"

// Texcoord1 is camera-space reflection vector pushed through a transform matrix
// get viewspace vector to this vertex
"m4x3 r0, v0, c4 \n"
"dp3 r1.x, r0, r0 \n"
"rsq r1.x, r1.x \n"
"mul r0.xyz, r0.xyz, r1.x \n"

// Calc normal vector into r1
// decompress normal first
"mad r2.xyz, v1.xyz, c94.xxxx, c94.yyyy \n"
"m3x3 r1, r2, c4 \n"
// renormalize
"dp3 r2.x, r1, r1 \n"
"rsq r2.x, r2.x \n"
"mul r1.xyz, r1.xyz, r2.x \n"

// Reflection vector is D - 2(D.N)N
"dp3 r2.x, r0.xyz, r1.xyz \n" //D.N
"add r2.x, r2.x, r2.x \n" //2(D.N)
"mul r2.xyz, r2.xxx, r1.xyz \n" // 2(D.N)N
"add r2.xyz, r0.xyz, -r2.xyz \n" // D-2(D.N)N

// fix up the w component to be 1
"mov r2.w, c8.y \n"

// texmatrix
"dp4 oT1.x, r2, c9 \n"
"dp4 oT1.y, r2, c10 \n";

// Register the shader with Asura

const Asura_Direct3D_VertexShader xVS_ENVIRONMENT_REFLECTIVE_SHADER(ASURA_D3D_VS_ENVIRONMENT_REFLECTIVE, szVS_ENVIRONMENT_REFLECTIVE_CODE, auVSD);

const char* const szVS_ENVIRONMENT_SHADOW_CODE =

// Environment Shadow shader
// Produces regular vertex coords in oPos, but shadowspace texcoords
// Expected constants:
//  c0-c3: ViewProj matrix
//  c4-c7: Shadow matrix
//	c8	 : -fFalloffPerMetre, 1.f + fFalloffPerMetre*fBaseHeight, -0.1f, initialAlpha
//  c9   : 1.0f, 0.0f, 0.0f, 0.0f

"vs.1.1 \n"
"dcl_position0 v0 \n"
"dcl_normal    v1 \n"

// Do the position
"m4x4 oPos, v0, c0 \n"

// Diffuse
"mad r0.xyz, v1.xyz, c94.xxxx, c94.yyyy \n"
"slt r0, r0.y, c8.z \n"
"mul oD0.xyz, c8.www, r0 \n"
"mov oD0.w, c9.x \n"

// Texcoord3
"mad oT3.xyzw, v0.yyyy, c8.xxxx, c8.yyyy \n"

// Texcoords
"dp4 r0.x, v0, c4 \n"
"dp4 r0.y, v0, c5 \n"
"dp4 r0.z, v0, c6 \n"

"dp4 r0.w, v0, c7 \n"

"slt r1.w, c0, c0 \n"
"max r0.w, r0.w, r1.w \n"

"mov oT2, r0 \n"

// perspective divide
"rcp r1.w, r0.w \n"
"mul r0, r0, r1.w \n"

// copy to t0, and 1- into t1
"mov oT0.xyz, r0.xy \n"
"add oT1.xyz, c9.xx, -r0.xy \n";

// Register the shader with Asura

const Asura_Direct3D_VertexShader xVS_ENVIRONMENT_SHADOW_SHADER(ASURA_D3D_VS_ENVIRONMENT_SHADOWS, szVS_ENVIRONMENT_SHADOW_CODE, auVSD);
