//
// Asura_D3D_VS_IDRender.cpp
//
// Created by SB - 07 November 2006
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#if defined(ASURA_USE_LINE_RENDERING)

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_Direct3D_Shaders.h"

// Uses same vertex declaration as ASURA_D3D_VS_SKIN_ORDER2SH_4BONES:
static Direct3D_VertexElement g_axCharacter_Vertex_Decl[] = 
{
	{0,  0,  D3DDECLTYPE_FLOAT3,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION,		0},
	{0,  12, D3DDECLTYPE_FLOAT3,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,			0},
	{0,  24, D3DDECLTYPE_FLOAT2,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD,		0},
	{0,  32, D3DDECLTYPE_FLOAT4,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BLENDINDICES,	0},
	{0,	 48, D3DDECLTYPE_FLOAT4,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_BLENDWEIGHT,	0},

	D3DDECL_END()
};

static char* g_szVertexShader =
{
	"vs.2.0 \n"

	"dcl_position		v0 \n"
	"dcl_normal			v1 \n"
	"dcl_texcoord		v2 \n"
	"dcl_blendindices	v3 \n"
	"dcl_blendweight	v4 \n"

	// get index of first bone					
	"mova a0, v3 \n"
	// rotate and translate the vertex by the first bone's transform
	"dp4 r1.x, c[a0.x+12], v0 \n"
	"dp4 r1.y, c[a0.x+13], v0 \n"
	"dp4 r1.z, c[a0.x+14], v0 \n"
	// scale the result by the weighting
	"mul r1.xyz, r1, v4.x \n"

	// now rotate the normal by the first bone (ignore translation)
	"dp3 r3.x, c[a0.x+12], v1 \n"
	"dp3 r3.y, c[a0.x+13], v1 \n"
	"dp3 r3.z, c[a0.x+14], v1 \n"
	// set r3.w to 1.0f to allow for dp4 processing
	"mov r3.w, v0.w	\n"
				
	// rotate and translate the vertex by the second bone's transform,
	// then accumulate result taking weighting into account
	//"mov a0.x, v3.y \n"
	"dp4 r2.x, c[a0.y+12], v0 \n"
	"dp4 r2.y, c[a0.y+13], v0 \n"
	"dp4 r2.z, c[a0.y+14], v0 \n"
	"mad r1.xyz, r2, v4.y, r1 \n"
												
	// rotate and translate the vertex by the third bone's transform,
	// then accumulate result taking weighting into account
	//"mov a0.x, v3.z \n"
	"dp4 r2.x, c[a0.z+12], v0 \n"
	"dp4 r2.y, c[a0.z+13], v0 \n"
	"dp4 r2.z, c[a0.z+14], v0 \n"
	"mad r1.xyz, r2, v4.z, r1 \n"
												
	// rotate and translate the vertex by the forth bone's transform,
	// then accumulate result taking weighting into account
	//"mov a0.x, v3.w \n"
	"dp4 r2.x, c[a0.w+12], v0 \n"
	"dp4 r2.y, c[a0.w+13], v0 \n"
	"dp4 r2.z, c[a0.w+14], v0 \n"
	"mad r1.xyz, r2, v4.w, r1 \n"

	// touch up r1.w so it's 1.0f
	"mov r1.w, v0.w \n"

	// lighting
	// calculate colour from harmonic, and drag across alpha value
	"dp4 oD0.x, r3, c[4] \n"
	"dp4 oD0.y, r3, c[5] \n"
	"dp4 oD0.z, r3, c[6] \n"
	"mov oD0.w, c[7].w \n"

	// fog
	"mov oFog, c[7].y \n"

	// output uv co-ords (applying UV offsets)
	"add oT0, v2, c11 \n"
	"add oT1, v2, c11 \n"

	// output normal in spare uv channel
	"mov oT2, v1 \n"

    // Transform to screen space and write to output position register.
    "m4x4 oPos, r1, c0 \n"
};

static Asura_Direct3D_VertexShader xShader( ASURA_D3D_VS_IDRENDER_SKIN_4BONES, g_szVertexShader, g_axCharacter_Vertex_Decl, 2.0f );

// Uses same vertex declaraction as ASURA_D3D_VS_OBJECT_ORDER2SH
static Direct3D_VertexElement g_axObject_Order2SH_Vertex_Decl[] = 
{
	{0,  0,  D3DDECLTYPE_FLOAT3,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION,	0},
	{0,  12, D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,		0},
	{0,  16, D3DDECLTYPE_SHORT2,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD,	0},

	D3DDECL_END()
};

static Asura_Direct3D_VertexShader Shader(ASURA_D3D_VS_IDRENDER_OBJECT,

	"vs.1.1 \n"

	"dcl_position	v0 \n"
	"dcl_normal		v1 \n"
	"dcl_texcoord0	v2 \n"

	// rotate the vertex and normal into world space ready for lighting
	"dp4 r1.x, v0, c[7]	\n"
	"dp4 r1.y, v0, c[8]	\n"
	"dp4 r1.z, v0, c[9]	\n"
	"dp4 r1.w, v0, c[10] \n"

	// set the normal's w component to 1.0 to enable dp4 processing
	"mad r0.xyz, v1.xyz, c94.xxx, c94.yyy \n" // decompress normal
	"dp3 r3.x, r0, c[7]	\n"
	"dp3 r3.y, r0, c[8]	\n"
	"dp3 r3.z, r0, c[9]	\n"
	"mov r3.w, v0.w	\n"
				
	// copy over the uv co-ords into the first and second stages, adding
	// in the uv animation offset for this object
	"mad r0.xy, v2.xy, c95.xx, c95.yy \n" // decompress
	"add oT0.xy, r0, c[19] \n"
	"add oT1.xy, r0, c[19] \n"

	// calculate colour from harmonic
	"dp4 oD0.x, r3, c[11]	\n"
	"dp4 oD0.y, r3, c[12]	\n"
	"dp4 oD0.z, r3, c[13]	\n"
	"mov oD0.w, c[20].w \n"
	
	// move the vertex into viewspace (unfortunately you can't read from
	// oPos, so we have to have to use another register, and copy it into
	// oPos later).
	"dp4 r2.x, r1, c[0] \n"
	"dp4 r2.y, r1, c[1] \n"
	"dp4 r2.z, r1, c[2] \n"
	"dp4 r2.w, r1, c[3] \n"

	// move precomputed fog value through
	"mov oFog, c[20].x \n"

	// Output the viewspace position
	"mov oPos, r2 \n"

	// output normal in spare uv channel and colour
	"mov oD0, v1 \n"
	"mov oT2, v1 \n"

	, g_axObject_Order2SH_Vertex_Decl);

#endif