//
// Asura_D3D_VS_Object_Order2SH.cpp
//
// Created by KJFL - 2004-06-18 17:13:18
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Rendering/Asura_Direct3D_Shaders.h"


static Direct3D_VertexElement g_axObject_Order2SH_VertexWithAlpha_Decl[] = 
{
	{0,  0,  D3DDECLTYPE_FLOAT3,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION,	0},
	{0,  12, D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,		0},
	{0,  16, D3DDECLTYPE_SHORT2,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD,	0},
	{0,  20, D3DDECLTYPE_FLOAT1,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD,	1},

	D3DDECL_END()
};

static Direct3D_VertexElement g_axObject_Order2SH_Vertex_Decl[] = 
{
	{0,  0,  D3DDECLTYPE_FLOAT3,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION,	0},
	{0,  12, D3DDECLTYPE_D3DCOLOR,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,		0},
	{0,  16, D3DDECLTYPE_SHORT2,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD,	0},

	D3DDECL_END()
};

static Asura_Direct3D_VertexShader ShaderWithAlpha(ASURA_D3D_VS_OBJECT_ORDER2SH_WITHALPHA,

	"vs.1.1 \n"

	"dcl_position	v0 \n"
	"dcl_normal		v1 \n"
	"dcl_texcoord0	v2 \n"
	"dcl_texcoord1	v3 \n" // alpha	

	// rotate the vertex and normal into world space ready for lighting
	"dp4 r1.x, v0, c[7]	\n"
	"dp4 r1.y, v0, c[8]	\n"
	"dp4 r1.z, v0, c[9]	\n"
	"dp4 r1.w, v0, c[10] \n"

	// set the normal's w component to 1.0 to enable dp4 processing
	"mad r0.xyz, v1.xyz, c94.xxx, c94.yyy \n" // decompress normal
	"dp3 r3.x, r0, c[7]	\n"
	"dp3 r3.y, r0, c[8]	\n"
	"dp3 r3.z, r0, c[9]	\n"
	"mov r3.w, v0.w	\n"
				
	// copy over the uv co-ords into the first and second stages, adding
	// in the uv animation offset for this object
	"mad r0.xy, v2.xy, c95.xx, c95.yy \n" // decompress
	"add oT0.xy, r0, c[19] \n"
	"add oT1.xy, r0, c[19] \n"

	// calculate colour from harmonic
	"dp4 oD0.x, r3, c[11]	\n"
	"dp4 oD0.y, r3, c[12]	\n"
	"dp4 oD0.z, r3, c[13]	\n"
	"mul oD0.w, v3.x, c[20].w \n"
	
	// move the vertex into viewspace (unfortunately you can't read from
	// oPos, so we have to have to use another register, and copy it into
	// oPos later).
	"dp4 r2.x, r1, c[0] \n"
	"dp4 r2.y, r1, c[1] \n"
	"dp4 r2.z, r1, c[2] \n"
	"dp4 r2.w, r1, c[3] \n"

	// move precomputed fog value through
	"mov oFog, c[20].x \n"

	// Output the viewspace position
	"mov oPos, r2 \n"

	, g_axObject_Order2SH_VertexWithAlpha_Decl);

static Asura_Direct3D_VertexShader Shader(ASURA_D3D_VS_OBJECT_ORDER2SH,

	"vs.1.1 \n"

	"dcl_position	v0 \n"
	"dcl_normal		v1 \n"
	"dcl_texcoord0	v2 \n"

	// rotate the vertex and normal into world space ready for lighting
	"dp4 r1.x, v0, c[7]	\n"
	"dp4 r1.y, v0, c[8]	\n"
	"dp4 r1.z, v0, c[9]	\n"
	"dp4 r1.w, v0, c[10] \n"

	// set the normal's w component to 1.0 to enable dp4 processing
	"mad r0.xyz, v1.xyz, c94.xxx, c94.yyy \n" // decompress normal
	"dp3 r3.x, r0, c[7]	\n"
	"dp3 r3.y, r0, c[8]	\n"
	"dp3 r3.z, r0, c[9]	\n"
	"mov r3.w, v0.w	\n"
				
	// copy over the uv co-ords into the first and second stages, adding
	// in the uv animation offset for this object
	"mad r0.xy, v2.xy, c95.xx, c95.yy \n" // decompress
	"add oT0.xy, r0, c[19] \n"
	"add oT1.xy, r0, c[19] \n"

	// calculate colour from harmonic
	"dp4 oD0.x, r3, c[11]	\n"
	"dp4 oD0.y, r3, c[12]	\n"
	"dp4 oD0.z, r3, c[13]	\n"
	"mov oD0.w, c[20].w \n"
	
	// move the vertex into viewspace (unfortunately you can't read from
	// oPos, so we have to have to use another register, and copy it into
	// oPos later).
	"dp4 r2.x, r1, c[0] \n"
	"dp4 r2.y, r1, c[1] \n"
	"dp4 r2.z, r1, c[2] \n"
	"dp4 r2.w, r1, c[3] \n"

	// move precomputed fog value through
	"mov oFog, c[20].x \n"

	// Output the viewspace position
	"mov oPos, r2 \n"

	, g_axObject_Order2SH_Vertex_Decl);

static char* g_szObject_Order2SH_Reflective_VertexShaderCode=
{
	"vs.1.1 \n"

	"dcl_position	v0 \n"
	"dcl_normal		v1 \n"
	"dcl_texcoord0	v2 \n"

	// rotate the vertex and normal into world space ready for lighting
	"dp4 r1.x, v0, c[7]	\n"
	"dp4 r1.y, v0, c[8]	\n"
	"dp4 r1.z, v0, c[9]	\n"
	"dp4 r1.w, v0, c[10] \n"

	// set the normal's w component to 1.0 to enable dp4 processing
	"mad r0.xyz, v1.xyz, c94.xxx, c94.yyy \n" // decompress normal
	"dp3 r3.x, r0, c[7]	\n"
	"dp3 r3.y, r0, c[8]	\n"
	"dp3 r3.z, r0, c[9]	\n"
	"mov r3.w, v0.w	\n"
				
	// copy over the uv co-ords into the first and second stages, adding
	// in the uv animation offset for this object
	"mad r0.xy, v2.xy, c95.xx, c95.yy \n" // decompress
	"add oT0.xy, r0, c[19] \n"

	// calculate colour from harmonic
	"dp4 oD0.x, r3, c[11] \n"
	"dp4 oD0.y, r3, c[12] \n"
	"dp4 oD0.z, r3, c[13] \n"
	"mov oD0.w, v0.w \n"
	
	// move the vertex into viewspace (unfortunately you can't read from
	// oPos, so we have to have to use another register, and copy it into
	// oPos later).
	"dp4 r2.x, r1, c[0] \n"
	"dp4 r2.y, r1, c[1] \n"
	"dp4 r2.z, r1, c[2] \n"
	"dp4 r2.w, r1, c[3] \n"

	// Output the viewspace position
	"mov oPos, r2 \n"

	// move precomputed fog value through
	"mov oFog, c[20].x \n"

	// reflection map coordinates
	// push vertex through view matrix and normalise to get cameraspace camera->vertex vector
	"m4x3 r2, r1, c[21] \n"
	"dp3 r4.x, r2, r2 \n"
	"rsq r4.x, r4.x \n"
	"mul r2.xyz, r2.xyz, r4.x \n"

	// push vertex normal through into cameraspace as well
	"m3x3 r4, r3, c[21] \n"
	// renormalise (necessary?)
	"dp3 r5.x, r4, r4 \n"
	"rsq r5.x, r5.x \n"
	"mul r4.xyz, r4.xyz, r5.x \n"

	// reflection vector is D - 2(D.N)N
	"dp3 r5.x, r2, r4 \n"		// D.N
	"add r5.x, r5.x, r5.x \n"   // 2(D.N)
	"mul r5.xyz, r4.xyz, r5.xxx \n" // 2(D.N)N
	"sub r5.xyz, r2.xyz, r5.xyz \n" // D - 2(D.N)N

	"mov r5.w, v0.w \n"

	// push it through the transform matrix
	"dp4 oT1.x, r5, c[24] \n"
	"dp4 oT1.y, r5, c[25] \n"
};

static Asura_Direct3D_VertexShader ReflectiveShader(ASURA_D3D_VS_OBJECT_REFLECTIVE, g_szObject_Order2SH_Reflective_VertexShaderCode, g_axObject_Order2SH_Vertex_Decl);

static char* g_szObject_Order2SH_ReflectiveWithAlpha_VertexShaderCode=
{
	"vs.1.1 \n"

	"dcl_position	v0 \n"
	"dcl_normal		v1 \n"
	"dcl_texcoord0	v2 \n"
	"dcl_texcoord1	v3 \n" // alpha	

	// rotate the vertex and normal into world space ready for lighting
	"dp4 r1.x, v0, c[7]	\n"
	"dp4 r1.y, v0, c[8]	\n"
	"dp4 r1.z, v0, c[9]	\n"
	"dp4 r1.w, v0, c[10] \n"

	// set the normal's w component to 1.0 to enable dp4 processing
	"mad r0.xyz, v1.xyz, c94.xxx, c94.yyy \n" // decompress normal
	"dp3 r3.x, r0, c[7]	\n"
	"dp3 r3.y, r0, c[8]	\n"
	"dp3 r3.z, r0, c[9]	\n"
	"mov r3.w, v0.w	\n"
				
	// copy over the uv co-ords into the first and second stages, adding
	// in the uv animation offset for this object
	"mad r0.xy, v2.xy, c95.xx, c95.yy \n" // decompress
	"add oT0.xy, r0, c[19] \n"

	// calculate colour from harmonic
	"dp4 oD0.x, r3, c[11] \n"
	"dp4 oD0.y, r3, c[12] \n"
	"dp4 oD0.z, r3, c[13] \n"
	"mul oD0.w, v3.x, c[20].w \n"
	
	// move the vertex into viewspace (unfortunately you can't read from
	// oPos, so we have to have to use another register, and copy it into
	// oPos later).
	"dp4 r2.x, r1, c[0] \n"
	"dp4 r2.y, r1, c[1] \n"
	"dp4 r2.z, r1, c[2] \n"
	"dp4 r2.w, r1, c[3] \n"

	// Output the viewspace position
	"mov oPos, r2 \n"

	// move precomputed fog value through
	"mov oFog, c[20].x \n"

	// reflection map coordinates
	// push vertex through view matrix and normalise to get cameraspace camera->vertex vector
	"m4x3 r2, r1, c[21] \n"
	"dp3 r4.x, r2, r2 \n"
	"rsq r4.x, r4.x \n"
	"mul r2.xyz, r2.xyz, r4.x \n"

	// push vertex normal through into cameraspace as well
	"m3x3 r4, r3, c[21] \n"
	// renormalise (necessary?)
	"dp3 r5.x, r4, r4 \n"
	"rsq r5.x, r5.x \n"
	"mul r4.xyz, r4.xyz, r5.x \n"

	// reflection vector is D - 2(D.N)N
	"dp3 r5.x, r2, r4 \n"		// D.N
	"add r5.x, r5.x, r5.x \n"   // 2(D.N)
	"mul r5.xyz, r4.xyz, r5.xxx \n" // 2(D.N)N
	"sub r5.xyz, r2.xyz, r5.xyz \n" // D - 2(D.N)N

	"mov r5.w, v0.w \n"

	// push it through the transform matrix
	"dp4 oT1.x, r5, c[24] \n"
	"dp4 oT1.y, r5, c[25] \n"
};

static Asura_Direct3D_VertexShader ReflectiveWithAlphaShader(ASURA_D3D_VS_OBJECT_REFLECTIVE_WITHALPHA, g_szObject_Order2SH_ReflectiveWithAlpha_VertexShaderCode, g_axObject_Order2SH_VertexWithAlpha_Decl);


static char* g_szObject_Hologram_VertexShaderCode=
{
	"vs.1.1 \n"

	"dcl_position	v0 \n"
	"dcl_normal		v1 \n"
	"dcl_texcoord0	v2 \n"
	"dcl_texcoord1	v3 \n" // alpha	

	// rotate the vertex and normal into world space ready for lighting
	"dp4 r1.x, v0, c[7]	\n"
	"dp4 r1.y, v0, c[8]	\n"
	"dp4 r1.z, v0, c[9]	\n"
	"dp4 r1.w, v0, c[10] \n"

	// set the normal's w component to 1.0 to enable dp4 processing
	"mad r0.xyz, v1.xyz, c94.xxx, c94.yyy \n" // decompress normal
	"dp3 r3.x, r0, c[7]	\n"
	"dp3 r3.y, r0, c[8]	\n"
	"dp3 r3.z, r0, c[9]	\n"
	"mov r3.w, v0.w	\n"
				
	// copy over the uv co-ords into the first and second stages, adding
	// in the uv animation offset for this object
	"mad r0.xy, v2.xy, c95.xx, c95.yy \n" // decompress
	"add oT0.xy, r0, c[19] \n"
	"add oT1.xy, r0, c[19] \n"

	// copy through colour
	"mov oD0, c[11] \n"
	"mul oD0.w, v3.x, c[11].w \n"

	// Hologram distortion
	"m4x4 r2, r1, c[12] \n"

	// viewproj transform
	"m4x4 oPos, r2, c[0] \n"

	// move precomputed fog value through
	"mov oFog, c[20].x \n"
};

static Asura_Direct3D_VertexShader HologramShader(ASURA_D3D_VS_OBJECT_HOLOGRAM, g_szObject_Hologram_VertexShaderCode, g_axObject_Order2SH_VertexWithAlpha_Decl);

static Direct3D_VertexElement g_axObject_Order2SH_Vertex_Uncompressed_Decl[] = 
{
	{0,  0,  D3DDECLTYPE_FLOAT3,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION,	0},
	{0,  12, D3DDECLTYPE_FLOAT3,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,		0},
	{0,  24, D3DDECLTYPE_FLOAT2,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD,	0},

	D3DDECL_END()
};

static Asura_Direct3D_VertexShader UncompressedShader(ASURA_D3D_VS_OBJECT_ORDER2SH_UNCOMPRESSED,

	"vs.1.1 \n"

	"dcl_position	v0 \n"
	"dcl_normal		v1 \n"
	"dcl_texcoord0	v2 \n"

	// rotate the vertex and normal into world space ready for lighting
	"dp4 r1.x, v0, c[7]	\n"
	"dp4 r1.y, v0, c[8]	\n"
	"dp4 r1.z, v0, c[9]	\n"
	"dp4 r1.w, v0, c[10] \n"

	// set the normal's w component to 1.0 to enable dp4 processing
	"dp3 r3.x, v1, c[7]	\n"
	"dp3 r3.y, v1, c[8]	\n"
	"dp3 r3.z, v1, c[9]	\n"
	"mov r3.w, v0.w	\n"
				
	// copy over the uv co-ords into the first and second stages, adding
	// in the uv animation offset for this object
	"add oT0.xy, v2, c[19] \n"
	"add oT1.xy, v2, c[19] \n"

	// calculate colour from harmonic
	"dp4 oD0.x, r3, c[11]	\n"
	"dp4 oD0.y, r3, c[12]	\n"
	"dp4 oD0.z, r3, c[13]	\n"
	"mov oD0.w, c[20].w \n"
	
	// move the vertex into viewspace (unfortunately you can't read from
	// oPos, so we have to have to use another register, and copy it into
	// oPos later).
	"dp4 r2.x, r1, c[0] \n"
	"dp4 r2.y, r1, c[1] \n"
	"dp4 r2.z, r1, c[2] \n"
	"dp4 r2.w, r1, c[3] \n"

	// move precomputed fog value through
	"mov oFog, c[20].x \n"

	// Output the viewspace position
	"mov oPos, r2 \n"

	, g_axObject_Order2SH_Vertex_Uncompressed_Decl);

