
#include "Bot_Shared.fxh"

//------------------------------------
struct VS_Input
{
	// DLH - 23 October 2006 15:11:09 - Mesh Data
	float4	xPosition					: POSITION0;
	float3	xNormal						: NORMAL0;
	float3	xTangent					: TANGENT0;
	float4	xBinormal					: BINORMAL0;	// DLH - XYZ: Binormal, W: Bone Weighting
	float4	axUV0UV1					: TEXCOORD0;
    float4 	xDiffuse					: COLOR0;
 
	// DLH - 23 October 2006 15:11:20 - Instance Data
	float4	xTransform0					: POSITION1;
	float4	xTransform1					: POSITION2;
	float4	xTransform2					: POSITION3;
    float4 	xRedSH						: COLOR1;
    float4 	xGreenSH					: COLOR2;
    float4 	xBlueSH						: COLOR3;
	float4	xBone						: POSITION4;
}; 
   
const float g_fPrelightingScale = 1.0f;
const float3 g_xPrelightingColourFilter = float3( 1.0f, 1.0f, 1.0f);

//------------------------------------
VS_Output VS_Bot_3D_Prelighting_Pass (VS_Input IN)
{
	VS_Output xOut;
 
    // Calculate the vertex position...
    float4 xPositionObject = IN.xPosition;
    {
		float fBoneWeight = IN.xBinormal.w;
		xPositionObject.xyz = Bot_CalcNewBonedPosition( xPositionObject.xyz, fBoneWeight, IN.xBone.xyz);
    }
       
    float4 xPositionWorld = xPositionObject;
      
    xPositionWorld.x = dot( xPositionObject.xyzw, IN.xTransform0.xyzw);
    xPositionWorld.y = dot( xPositionObject.xyzw, IN.xTransform1.xyzw);
    xPositionWorld.z = dot( xPositionObject.xyzw, IN.xTransform2.xyzw);
     
	xOut.m_xPosition = mul( xPositionWorld, g_xViewProjectionMatrix);
       
	xOut.m_xColour = IN.xDiffuse;

    // Calculate the vertex texture-coordinates...
   	xOut.m_xUVPair.xy = CalculateTextureCoordinate (IN.axUV0UV1.xy);
   	xOut.m_xUVPair.zw = CalculateTextureCoordinate (IN.axUV0UV1.wz);
   	
   	float3 xWorldNormal = RotateVectorIntoWorldSpace( IN.xNormal.xyz, IN.xTransform0, IN.xTransform1, IN.xTransform2);
   	float3 xWorldTangent = RotateVectorIntoWorldSpace( IN.xTangent.xyz, IN.xTransform0, IN.xTransform1, IN.xTransform2);
   	float3 xWorldBinormal = RotateVectorIntoWorldSpace( IN.xBinormal.xyz, IN.xTransform0, IN.xTransform1, IN.xTransform2);
    	
	// Light it - from front and back, then lerp
	
#if defined( ASURA_TOOLS)
	if (g_bDisableLighting )
	{
		xOut.m_xColour.rgb = 0.5f;
		xOut.m_xBackColour.rgb = 0.5f;
	}
	else
#endif	// defined( ASURA_TOOLS)
	{
		half3 xColour, xBackColour;		
		half4 xNormal = half4(xWorldNormal,1);						
		xOut.m_xColour.rgb = SHLookup( xNormal, IN.xRedSH, IN.xGreenSH, IN.xBlueSH );
		
		xNormal.rgb=-xNormal.rgb;	
		xOut.m_xBackColour.rgb =  SHLookup( xNormal, IN.xRedSH, IN.xGreenSH, IN.xBlueSH );
	}	
	
	xOut.m_afAlphaCut = IN.xBone.w; 

	// (AAM 13.02.2007) - Used to output data in tangent space, but need world space for
	// reflectivity transforms  

	xOut.m_xTangent = xWorldTangent;
	xOut.m_xBinormal = xWorldBinormal;
	xOut.m_xNormal = xWorldNormal;

	xOut.m_xCameraDirDistance.xyz = g_xCameraPosition.xyz - xPositionWorld;
	xOut.m_xCameraDirDistance.w = xOut.m_xPosition.w;

	return xOut;
}



//-----------------------------------
technique Bot_3D_Prelighting_Pass
{
    pass p0
    {
		VertexShader	= compile vs_3_0 VS_Bot_3D_Prelighting_Pass();
		PixelShader		= compile ps_3_0 PS_Bot_3D_Prelighting_Pass();
    }
}
 