#include "Shared.fxh"
#include "MaterialResponse_Shared.fxh"

struct VS_Input_Static
{
	// DLH - 23 October 2006 15:11:09 - Mesh Data
	float4	xPosition					: POSITION0;
    float3  xUp						    : NORMAL0;
    float4 	xRedSH						: COLOR0;
    float4 	xGreenSH					: COLOR1;
    float4 	xBlueSH						: COLOR2;
    float4	xCorner						: COLOR3;
};

struct VS_Input_Bone
{
	// DLH - 23 October 2006 15:11:09 - Mesh Data
	float4	xPosition					: POSITION0;
    float3  xUp						    : NORMAL0;
    float4 	xRedSH						: COLOR0;
    float4 	xGreenSH					: COLOR1;
    float4 	xBlueSH						: COLOR2;
    float4	xCorner						: COLOR3;
    float3	xBone						: POSITION1;
};

struct VS_Output_Imposter
{
    float4	m_xPosition					: POSITION0;

	float4	axUV0UV1					: TEXCOORD0;
	float4  afAlphaCut					: TEXCOORD1;

	float4 m_xRedSH						: TEXCOORD2;
	float4 m_xGreenSH					: TEXCOORD3;
	float4 m_xBlueSH					: TEXCOORD4;
	
	float4 m_xCameraDirDistance			: TEXCOORD5;
	
	float3 m_xTangent0           : TANGENT0;
	float3 m_xBinormal0          : BINORMAL0;
	float3 m_xTangent1           : TANGENT1;
	float3 m_xBinormal1          : BINORMAL1;
//	float3 m_xNormal            : NORMAL;
	
};

struct PS_Input_Imposter
{
	float4	axUV0UV1					: TEXCOORD0;
	float4  m_afAlphaCut				: TEXCOORD1;

	float4 m_xRedSH						: TEXCOORD2;
	float4 m_xGreenSH					: TEXCOORD3;
	float4 m_xBlueSH					: TEXCOORD4;
	
	float4 m_xCameraDirDistance			: TEXCOORD5;
	
	float3 m_xTangent0           : TANGENT0;
	float3 m_xBinormal0          : BINORMAL0;
	float3 m_xTangent1           : TANGENT1;
	float3 m_xBinormal1          : BINORMAL1;
//	float3 m_xNormal            : NORMAL;
};

struct ImposterData
{
	float4 m_xAlphaFadeGraph;
	float4 m_xAlphaRotateGraph;
	float4 m_axCornerXY[4];
	float4 m_axUV[2][4];
	float4 m_xTangent[2];
	float4 m_xBinormal[2];
};

ImposterData g_xImposterData;

float4	g_xCameraRightVector;
float4	g_xCameraUpVector;
float4	g_xCameraForwardsVector;

float3 RotateVectorIntoWorldSpace (float3 xVector,
							   float3 xTransform0,
							   float3 xTransform1,
							   float3 xTransform2)
{
	float3	xRotatedVector;

	// Rotate the x, y and z components into tangent-space...
	xRotatedVector.x	= dot(xVector, xTransform0);
	xRotatedVector.y	= dot(xVector, xTransform1);
	xRotatedVector.z	= dot(xVector, xTransform2);
	
	xRotatedVector = normalize( xRotatedVector);

	// Return the rotated vector...
	return	(xRotatedVector);
}

float GetInstanceScale( float3 xUp)
{
	return( length( xUp.xyz));
}

float3 Bot_CalcNewBonedPosition( float3 xOldPosition, float fBoneWeight, float3 xBone)
{
	float3 xNewPosition = xOldPosition + (fBoneWeight * xBone);
	
	return( xNewPosition);
}

DeferredOutputPixel PS_Bot_Imposter1_Prelighting_Pass(PS_Input_Imposter xIn)
{
   	float4 xSurfaceNormal = -g_xCameraForwardsVector;

	const float4 xUVPair = xIn.axUV0UV1;
	const float fDualTextureLerp = 0;

	const float3x3 xTangentToWorldMatrix = float3x3( normalize( xIn.m_xTangent0 ),
													 normalize( xIn.m_xBinormal0 ),
													 normalize( xSurfaceNormal.xyz) );
	const float4 xWorldSpaceNormal = NormalLookup( xUVPair, fDualTextureLerp, xTangentToWorldMatrix, xTangentToWorldMatrix);




	// -----------------------------------------------------------------
	// Diffuse-only SH prelighting 
	// -----------------------------------------------------------------		
	const float fMaterialsTextureY = g_xMaterialProperties.x;
	half3 xAlbedo;
	const float3 xAlbedoTimes2 = AlbedoLookup( xUVPair,
											   0.0,
											   xWorldSpaceNormal.xyz,
											   normalize( xIn.m_xCameraDirDistance.xyz ),
											   xAlbedo );
	half4 xColour;		
	
#if defined( ASURA_TOOLS)
	if (g_bDisableLighting)
	{
		xColour.rgb = xAlbedo;
	}
	else
#endif	// defined( ASURA_TOOLS)
	{
		xColour.r	= dot (xWorldSpaceNormal, xIn.m_xRedSH);
		xColour.g	= dot (xWorldSpaceNormal, xIn.m_xGreenSH);
		xColour.b	= dot (xWorldSpaceNormal, xIn.m_xBlueSH);
		xColour.rgb = max (xColour.rgb, 0) * xAlbedoTimes2 + SelfIlluminationLookup( xUVPair, 0.0, xAlbedo );
	}
	

	xColour.a = TransparencyLookup( xUVPair, 0.0 );
	
	xColour.a = xColour.a - xIn.m_afAlphaCut.x;

	half hSpecular = SpecularLookup( xUVPair, 0.0 );
	
	DeferredOutputPixel xOut = Deferred_FillOutputPixel(
										xColour,
										xWorldSpaceNormal.xyz, 
										xIn.m_xCameraDirDistance.w,
										xAlbedo,
										hSpecular,
										fMaterialsTextureY
										);										
	return( xOut);
}

DeferredOutputPixel PS_Bot_Imposter2_Prelighting_Pass(PS_Input_Imposter xIn)
{
   	float4 xSurfaceNormal = -g_xCameraForwardsVector;
   	xSurfaceNormal.xyz = normalize( xSurfaceNormal.xyz);

	const float4 xUVPair = xIn.axUV0UV1;
	const float fDualTextureLerp = xIn.m_afAlphaCut.z;

	const float3x3 xTangentToWorldMatrix1 = float3x3( normalize( xIn.m_xTangent0 ),
													  normalize( xIn.m_xBinormal0 ),
													  xSurfaceNormal.xyz );

	const float3x3 xTangentToWorldMatrix2 = float3x3( normalize( xIn.m_xTangent1 ),
													  normalize( xIn.m_xBinormal1 ),
													  xSurfaceNormal.xyz );

	float4 xWorldSpaceNormal;

	xWorldSpaceNormal = NormalLookup( xUVPair, fDualTextureLerp, xTangentToWorldMatrix1, xTangentToWorldMatrix2);
	xWorldSpaceNormal.w = 1.0;

	// -----------------------------------------------------------------
	// Diffuse-only SH prelighting 
	// -----------------------------------------------------------------		
	const float fMaterialsTextureY = g_xMaterialProperties.x;
	half3 xAlbedo;
	const float3 xAlbedoTimes2 = AlbedoLookup( xUVPair,
											   fDualTextureLerp,
											   xWorldSpaceNormal.xyz,
											   normalize( xIn.m_xCameraDirDistance.xyz ),
											   xAlbedo );
	half4 xColour;

#if defined( ASURA_TOOLS)
	if (g_bDisableLighting)
	{
		xColour.rgb = xAlbedo;
	}
	else
#endif // defined( ASURA_TOOLS)
	{
		xColour.r	= dot (xWorldSpaceNormal, xIn.m_xRedSH);
		xColour.g	= dot (xWorldSpaceNormal, xIn.m_xGreenSH);
		xColour.b	= dot (xWorldSpaceNormal, xIn.m_xBlueSH);
		xColour.rgb = max (xColour.rgb, 0) * xAlbedoTimes2 + SelfIlluminationLookup( xUVPair, fDualTextureLerp, xAlbedo );
	}

	xColour.a = TransparencyLookup( xUVPair, fDualTextureLerp );
	
	xColour.a = xColour.a - xIn.m_afAlphaCut.x;

	half hSpecular = SpecularLookup( xUVPair, fDualTextureLerp );
	
	DeferredOutputPixel xOut = Deferred_FillOutputPixel(
										xColour,
										xWorldSpaceNormal.xyz, 
										xIn.m_xCameraDirDistance.w,
										xAlbedo,
										hSpecular,
										fMaterialsTextureY
										);	
	return( xOut);
}

VS_Output_Imposter VS_Bot_Imposter_Bone(VS_Input_Bone IN, const bool bDoDualTexture)
{
	VS_Output_Imposter	xOut;

    // Calculate the vertex position...
    float fCornerIndex = IN.xCorner.x;
    float2 xCorner = g_xImposterData.m_axCornerXY[ fCornerIndex].xy;
    
    float4 xPositionWorld = IN.xPosition;

	float fInstanceScale = GetInstanceScale( IN.xUp);
	
    // Calculate the vertex texture-coordinates...
   	xOut.axUV0UV1.xy	= g_xImposterData.m_axUV[0][ fCornerIndex].xy;
   	xOut.axUV0UV1.zw	= g_xImposterData.m_axUV[1][ fCornerIndex].xy;
   	
	xOut.m_xTangent0 = RotateVectorIntoWorldSpace( -g_xImposterData.m_xTangent[0], -g_xCameraRightVector, g_xCameraUpVector, g_xCameraForwardsVector);
	xOut.m_xBinormal0 = RotateVectorIntoWorldSpace( g_xImposterData.m_xBinormal[0], -g_xCameraRightVector, g_xCameraUpVector, g_xCameraForwardsVector);;
	if( true == bDoDualTexture)
	{
		xOut.m_xTangent1 = RotateVectorIntoWorldSpace( -g_xImposterData.m_xTangent[1], -g_xCameraRightVector, g_xCameraUpVector, g_xCameraForwardsVector);
		xOut.m_xBinormal1 = RotateVectorIntoWorldSpace( g_xImposterData.m_xBinormal[1], -g_xCameraRightVector, g_xCameraUpVector, g_xCameraForwardsVector);
	}
	else
	{
		xOut.m_xTangent1 = xOut.m_xTangent0;
		xOut.m_xBinormal1 = xOut.m_xBinormal0;
	}
   	
   	xOut.m_xRedSH = IN.xRedSH;
   	xOut.m_xGreenSH = IN.xGreenSH;
   	xOut.m_xBlueSH = IN.xBlueSH;
   	
	float4 xDirectionToCamera = g_xCameraPosition - xPositionWorld;
	xDirectionToCamera.w = 1.0f;

	float fDistanceToCamera = length( xDirectionToCamera.xyz);
	xDirectionToCamera.xyz = normalize( xDirectionToCamera.xyz);

	float fFadeDistance = (fDistanceToCamera * g_xImposterData.m_xAlphaFadeGraph.x) + g_xImposterData.m_xAlphaFadeGraph.y;
	if( true == bDoDualTexture)
	{
		float fFadeDistanceClamped = max( 0.5f, fFadeDistance);
		float fFadeAngle1 = g_xImposterData.m_xAlphaFadeGraph.z;
		float fFadeAngle2 = g_xImposterData.m_xAlphaFadeGraph.w;
		//xOut.afAlphaCut.x = saturate( fFadeDistanceClamped + fFadeAngle1);
		//xOut.afAlphaCut.y = saturate( fFadeDistanceClamped + fFadeAngle2);
		xOut.afAlphaCut.xy = saturate( fFadeDistanceClamped);
		xOut.afAlphaCut.zw = 1 - saturate( g_xImposterData.m_xAlphaRotateGraph.x);
	}
	else
	{
		fFadeDistance += g_xImposterData.m_xAlphaFadeGraph.z;
		
		float fFadeAngle1 = g_xImposterData.m_xAlphaFadeGraph.z;
		
		xOut.afAlphaCut = max( 0.5f, fFadeDistance) + fFadeAngle1;
	}
	
	float fScale = fFadeDistance <= 1.0f;
	fScale *= fInstanceScale;

	float3 xUp = normalize( IN.xUp);
	float4 xPositionCorner = xPositionWorld;
	float2 xCornerScaled = fScale * xCorner.xy;
	float3 xCornerOffset = g_xCameraRightVector.xyz * xCornerScaled.x;
	xCornerOffset.xyz += xUp * xCornerScaled.y;

	{
		float fBoneWeight = g_xImposterData.m_axCornerXY[ fCornerIndex].z;
		xCornerOffset.xyz = Bot_CalcNewBonedPosition( xCornerOffset.xyz, fBoneWeight, IN.xBone.xyz);
	}
	
	xPositionCorner.xyz += xCornerOffset;
    xOut.m_xPosition	= mul(xPositionCorner, g_xViewProjectionMatrix);

	xOut.m_xCameraDirDistance.xyz = g_xCameraPosition.xyz - xPositionCorner;
	xOut.m_xCameraDirDistance.w = xOut.m_xPosition.w;

    // Return the vertex...
    return( xOut);
}

VS_Output_Imposter VS_Bot_Imposter( VS_Input_Static IN, const bool bDoDualTexture)
{
	VS_Output_Imposter	xOut;

    // Calculate the vertex position...
    float fCornerIndex = IN.xCorner.x;
    float2 xCorner = g_xImposterData.m_axCornerXY[ fCornerIndex].xy;
    
    float4 xPositionWorld = IN.xPosition;

	float fInstanceScale = GetInstanceScale( IN.xUp);
	
    // Calculate the vertex texture-coordinates...
   	xOut.axUV0UV1.xy	= g_xImposterData.m_axUV[0][ fCornerIndex].xy;
   	xOut.axUV0UV1.zw	= g_xImposterData.m_axUV[1][ fCornerIndex].xy;
   	
	xOut.m_xTangent0 = RotateVectorIntoWorldSpace( -g_xImposterData.m_xTangent[0], -g_xCameraRightVector, g_xCameraUpVector, g_xCameraForwardsVector);
	xOut.m_xBinormal0 = RotateVectorIntoWorldSpace( g_xImposterData.m_xBinormal[0], -g_xCameraRightVector, g_xCameraUpVector, g_xCameraForwardsVector);
	if( true == bDoDualTexture)
	{
		xOut.m_xTangent1 = RotateVectorIntoWorldSpace( -g_xImposterData.m_xTangent[1], -g_xCameraRightVector, g_xCameraUpVector, g_xCameraForwardsVector);
		xOut.m_xBinormal1 = RotateVectorIntoWorldSpace( g_xImposterData.m_xBinormal[1], -g_xCameraRightVector, g_xCameraUpVector, g_xCameraForwardsVector);
	}
	else
	{
		xOut.m_xTangent1 = xOut.m_xTangent0;
		xOut.m_xBinormal1 = xOut.m_xBinormal0;
	}
   	
   	xOut.m_xRedSH = IN.xRedSH;
   	xOut.m_xGreenSH = IN.xGreenSH;
   	xOut.m_xBlueSH = IN.xBlueSH;
   	
	float4 xDirectionToCamera = g_xCameraPosition - xPositionWorld;
	xDirectionToCamera.w = 1.0f;

	float fDistanceToCamera = length( xDirectionToCamera.xyz);
	xDirectionToCamera.xyz = normalize( xDirectionToCamera.xyz);

	float fFadeDistance = (fDistanceToCamera * g_xImposterData.m_xAlphaFadeGraph.x) + g_xImposterData.m_xAlphaFadeGraph.y;
	if( true == bDoDualTexture)
	{
		float fFadeDistanceClamped = max( 0.5f, fFadeDistance);
		float fFadeAngle1 = g_xImposterData.m_xAlphaFadeGraph.z;
		float fFadeAngle2 = g_xImposterData.m_xAlphaFadeGraph.w;
		//xOut.afAlphaCut.x = saturate( fFadeDistanceClamped + fFadeAngle1);
		//xOut.afAlphaCut.y = saturate( fFadeDistanceClamped + fFadeAngle2);
		xOut.afAlphaCut.xy = saturate( fFadeDistanceClamped);
		xOut.afAlphaCut.zw = saturate( g_xImposterData.m_xAlphaRotateGraph.x);
	}
	else
	{
		fFadeDistance += g_xImposterData.m_xAlphaFadeGraph.z;
		
		float fFadeAngle1 = g_xImposterData.m_xAlphaFadeGraph.z;
		
		xOut.afAlphaCut = max( 0.5f, fFadeDistance) + fFadeAngle1;
	}
	
	float fScale = fFadeDistance <= 1.0f;
	fScale *= fInstanceScale;

	float3 xUp = normalize( IN.xUp);
	float4 xPositionCorner = xPositionWorld;
	float2 xCornerScaled = fScale * xCorner.xy;
	float3 xCornerOffset = g_xCameraRightVector.xyz * xCornerScaled.x;
	xCornerOffset.xyz += xUp * xCornerScaled.y;

	xPositionCorner.xyz += xCornerOffset;
    xOut.m_xPosition	= mul( xPositionCorner, g_xViewProjectionMatrix);

	xOut.m_xCameraDirDistance.xyz = g_xCameraPosition.xyz - xPositionCorner;
	xOut.m_xCameraDirDistance.w = xOut.m_xPosition.w;

    // Return the vertex...
    return( xOut);
}
