
// Lighting defines...

#define MAX_DYNAMIC_LIGHTS	1


// Lighting variables...

shared	bool	g_bUsePrelighting;
shared	int		g_iNumberOfLights;

shared	float3	g_axLightPosition [MAX_DYNAMIC_LIGHTS];
shared	float	g_afOneOverLightRange [MAX_DYNAMIC_LIGHTS];
shared	float3	g_axLightColour [MAX_DYNAMIC_LIGHTS];


// Specular lighting variables...

shared 	bool	g_bUseSpecularLighting		= true;

shared	float3	g_xSpecularHighlightColour	= (1.0, 1.0, 1.0);
shared	float	g_fSpecularReflectionPower	= 16;


// Light calculation functions...

float3 CalculateLightStrength (int iIndex,
							   float4 xPosition)
{
	float3	xDifference	= g_axLightPosition [iIndex] - xPosition;

	// Return the light-direction...
	return	(xDifference * g_afOneOverLightRange [iIndex]);
}


float CalculateLightAttenuation (float3 xLightStrength)
{
	float	fStrengthSquared	= dot (xLightStrength, xLightStrength);
	
	// Return the attenuated value...
	return	(1.0f - saturate(fStrengthSquared));
}


float3 CalculateLightRGB (float fAttenuation,
						  float3 xLightColour,
						  float3 xLightDirection,
						  float3 xNormal)
{
	float	fDotProduct	= saturate (dot(xLightDirection, xNormal));
	float3	xColour		= xLightColour * fAttenuation * fDotProduct;

 	// Return the light-colour...
	return	(xColour);
}


float3 CalculateLightSpecularRGB (float fAttenuation,
								  float3 xLightColour,
								  float3 xLightDirection,
								  float3 xNormal,
								  float fScalar,
								  float3 xEyeDirection)
{
	float3	xHalfwayVector	= normalize (xLightDirection + xEyeDirection);
	float	fDotProduct		= saturate (dot(xHalfwayVector, xNormal));
	float	fPower			= pow (fDotProduct, g_fSpecularReflectionPower);
	float3	xColour			= g_xSpecularHighlightColour * xLightColour * fPower * fAttenuation * fScalar;

 	// Return the specular-colour...
	return	(xColour);
}


// Diffuse dynamic light colour calculation...

float4 CalculateDiffuseDynamicLightColour (int iIndex,
										   float3 xLightStrength,
										   float3 xNormal)
{
	float	fAttenuation	= CalculateLightAttenuation (xLightStrength);
	float3	xLightDirection	= normalize (xLightStrength);

	float4	xColour;

	// Calculate the colour for this light...
	xColour.rgb	= CalculateLightRGB (fAttenuation, g_axLightColour [iIndex], xLightDirection, xNormal);
	xColour.a	= 1.0;
	
	// Return the colour...
	return	(xColour);
}


// Specular dynamic light colour calculation...

float4 CalculateSpecularDynamicLightColour (int iIndex,
											float fSpecularScale,
											float3 xEyeDistance,
											float3 xLightStrength,
											float3 xNormal)
{
	float3	xEyeDirection	= normalize (xEyeDistance);
	float3	xLightDirection	= normalize (xLightStrength);
	float	fAttenuation	= CalculateLightAttenuation (xLightStrength);

	float4	xColour;

	// Accumulate the colour for this light...
	xColour.rgb	= CalculateLightSpecularRGB (fAttenuation, g_axLightColour [iIndex], xLightDirection, xNormal, 1.0, xEyeDirection);
	xColour.a	= fSpecularScale;
	
	// Return the colour...
	return	(xColour);
}


/// End of file. ///
