// Includes must be this way round here
#include "Shared.fxh"
#include "MaterialResponse_Shared.fxh"

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

;
/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 m_xPosition    : POSITION;
};

struct OutputVertex
{
	float4 m_xPosition : POSITION;
	float  m_fLightspaceDistance : TEXCOORD0;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex VShader( const InputVertex xIn, uniform bool g_bCubicMapShadow)
{
	OutputVertex xOut;

	xOut.m_xPosition = mul( xIn.m_xPosition, g_xViewProjectionMatrix );
	if (g_bCubicMapShadow)
	{
		xOut.m_fLightspaceDistance = (length(xIn.m_xPosition - g_xCameraPosition) - g_fShadowFrustumNearPlaneDistance) * g_fOneOverShadowFrustumPlaneSeparation;
	}
	else
	{
		xOut.m_fLightspaceDistance = (xOut.m_xPosition.w - g_fShadowFrustumNearPlaneDistance) * g_fOneOverShadowFrustumPlaneSeparation;
	}
	return xOut;
}


float4 PShader( const OutputVertex xIn ) : COLOR
{
	return xIn.m_fLightspaceDistance.xxxx;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		VertexShader= compile vs_3_0 VShader(false);
		PixelShader = compile ps_3_0 PShader();
	}
}
technique
{
	pass
	{
		VertexShader= compile vs_3_0 VShader(true);
		PixelShader = compile ps_3_0 PShader();
	}
}
