/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const texture g_xTexture;
const sampler g_xTextureSampler : register(s0) = sampler_state
{
	Texture = <g_xTexture>;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
   	MINFILTER	= LINEAR;
   	MAGFILTER	= LINEAR;

};

const texture g_xAccumTexture;
const sampler g_xAccumTextureSampler : register(s1) = sampler_state
{	
	Texture = <g_xAccumTexture>;
	AddressU	= CLAMP;
	AddressV	= CLAMP;
    MINFILTER	= LINEAR;
    MAGFILTER	= LINEAR;
};

const texture g_xDepthTexture;
const sampler g_xDepthTextureSampler : register(s2) = sampler_state
{
	Texture = <g_xDepthTexture>;
	AddressU = CLAMP;
	AddressV = CLAMP;
	MinFilter = POINT;		
	MagFilter = POINT;
};


const float2 g_xUVOffset          = float2(0.05f,0.0f);
const bool   g_bBlur 		  = true;
const bool   g_bExtractBrightBits = true;
const float  g_fSaturation	  = 0.5f;
const float4 g_xOutputColour	  = float4(1,1,1,1);
const float4 g_xInputThreshold	  = float4(0,0,0,0);
const float  g_fInputScale        = 1.0f;
const bool   g_bAccumulate        = false;
const float  g_fAccumFade         = 1.0f;
const float2 g_xSecondaryUVOffset = float2(0,0);
const bool	 g_bExtractNearDepthOfField = false;
const float4 g_xOneOverScreenDimensions;
const float4 g_xFocalDistances;

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float2 m_xPosition   : POSITION;
	float2 m_axUVs	     : TEXCOORD0;
};

struct OutputVertex
{
	float4 m_xPosition   : POSITION;
	float2 m_axUVs[5]    : TEXCOORD0;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex VShader( const InputVertex xIn)
{
	OutputVertex xOut;

	xOut.m_xPosition = float4(xIn.m_xPosition.x,xIn.m_xPosition.y,0,1);


	xOut.m_axUVs[0].xy = xIn.m_axUVs;
	xOut.m_axUVs[1].xy = xIn.m_axUVs + ( -1.5f  * g_xUVOffset);
	xOut.m_axUVs[2].xy = xIn.m_axUVs + ( -0.5f  * g_xUVOffset);
	xOut.m_axUVs[3].xy = xIn.m_axUVs + (  0.5f  * g_xUVOffset);
	xOut.m_axUVs[4].xy = xIn.m_axUVs + (  1.5f  * g_xUVOffset);
	return xOut;
}


float4 PShader( const OutputVertex xIn, const float2 xScreenPos : VPOS  ) : COLOR
{
	float4 xCol;

	if (g_bBlur)
	{
		xCol = 0.25f * 	   (tex2D(g_xTextureSampler, xIn.m_axUVs[1])+
				      		tex2D(g_xTextureSampler, xIn.m_axUVs[2])+
			      			tex2D(g_xTextureSampler, xIn.m_axUVs[3])+
		  		      		tex2D(g_xTextureSampler, xIn.m_axUVs[4]));

		xCol.a *= g_xFocalDistances.z; // 'bleed control' value for the alpha, for the DoF effect only;
	}
	else
	{
		xCol = tex2D(g_xTextureSampler, xIn.m_axUVs[0]);
		xCol.a= 1.0f;
	}	


	if (g_bExtractBrightBits)
	{
	     xCol = saturate(xCol-g_xInputThreshold);	     
	     xCol *= g_fInputScale;
 
	     float fMono = 0.3333f * (xCol.r+xCol.g+xCol.b);
		
	     xCol = lerp(float4(fMono,fMono,fMono,1), xCol, g_fSaturation);             	     
	     
	     if (g_bAccumulate)
	     {						
			xCol += saturate(tex2D(g_xAccumTextureSampler, xIn.m_axUVs[0] + g_xSecondaryUVOffset) * (1-g_fAccumFade));						
	     }	
	}
	else
	{
	     xCol *= g_xOutputColour;
	}
	
	
	if (g_bExtractNearDepthOfField)
	{
		const float2 xNormalizedCoords = (xScreenPos.xy + float2(0.5, 0.5)) * g_xOneOverScreenDimensions.xy;	
		float fDepth = abs(tex2D(g_xDepthTextureSampler, xNormalizedCoords).z);
		
		float fBlurNear = saturate(1.0 - (fDepth-g_xFocalDistances.x) / (g_xFocalDistances.y-g_xFocalDistances.x));
		xCol.a = fBlurNear;
	}


	return saturate(xCol);
}


/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}