
// Normal-map related variables...

shared bool     g_bHasNormalMaps;
shared	bool	g_bDualPassNormalMap;


//------------------------------------


// This one has been updated to work with 2-component normal maps, as it's used for decals

float4 CalculateNormal4 (sampler xSampler,
						 float2 xUV)
{
	float4	xNormal	= tex2D (xSampler, xUV);
		
	xNormal.xy = xNormal.wy * 2.0h - 1.0h;
	// Our normal maps are output upside-down. TODO: do this in a preprocessing step
	xNormal.y = -xNormal.y;
	xNormal.z = sqrt( 1.0h - dot( xNormal.xy, xNormal.xy ) );
	xNormal.w = 1.0h;
	xNormal.xyz = normalize(xNormal.xyz);
	return	(xNormal);
}


// These following functions won't work since the normal map format changed (to use 2-component normal maps)
// I think they're only used by 'dead' shaders from the non-MR renderer 

float3 CalculateNormal3 (sampler xSampler,
						 float2 xUV)
{
	float3	xNormal	= tex2D (xSampler, xUV);
	
	// Current range is 0..1, change it to -1..1...
	xNormal	*= 2.0;
	xNormal	-= 1.0;
	xNormal = normalize(xNormal);

	xNormal.y = -xNormal.y;
	// Return the normal...
	return	(xNormal);
}


float3 CalculateNormal3_DualMap (sampler xSampler1, sampler xSampler2,
						 			float2 xUV1, float2 xUV2, float fLerp)
{
	float3	xNormal1	= tex2D (xSampler1, xUV1);
	float3	xNormal2	= tex2D (xSampler2, xUV2);
	
	float3 xNormal = lerp(xNormal2, xNormal1, fLerp);
	
	// Current range is 0..1, change it to -1..1...
	xNormal	-= 0.5;
	xNormal = normalize(xNormal);
	xNormal.y = -xNormal.y;
	// Return the normal...
	return	(xNormal);
}

float4 CalculateNormal4_DualMap (sampler xSampler1, sampler xSampler2,
						 			float2 xUV1, float2 xUV2, float fLerp)
{
	float4	xNormal1	= tex2D (xSampler1, xUV1);
	float4	xNormal2	= tex2D (xSampler2, xUV2);

	float4 xNormal = lerp(xNormal2, xNormal1, fLerp);
	
	// Current range is 0..1, change it to -1..1...
	xNormal	-= 0.5;
	xNormal.w	 = 1.0;
	xNormal.xyz = normalize(xNormal.xyz);
	xNormal.y = -xNormal.y;
	// Return the normal...
	return	(xNormal);
}

float CalculateDisplacement (sampler xSampler,
							 float2 xUV)
{
	// Extract the alpha from the texture...
	return	(tex2D(xSampler, xUV).a);
}


/// End of file. ///
