
//------------------------------------
#include "Shared.fxh"
#include "Deferred_Shared.fxh"

#include "DynamicLights.fxh"
#include "NormalMap.fxh"
#include "Objects.fxh"
#include "Reflective.fxh"
#include "SHLights.fxh"

bool g_bSelfIlluminating = false;

//------------------------------------
struct VS_Input
{
	float3	xPosition	: POSITION0;
	float3	xNormal		: NORMAL;

	float2	xUV			: TEXCOORD0;
	float	fAlpha		: TEXCOORD1;
};

struct VS_Output
{
    float4	xPosition		: POSITION0;
    float4 	xDiffuse		: COLOR0;

	float2	xUV				: TEXCOORD0;
    
    float3	xEyeDistance	: TEXCOORD2;
    
    float4	xNormalDepth	: TEXCOORD3;
};

//------------------------------------
sampler axSampler [(2*MAX_TEXTURE_COORDS) + 1] =
{
	sampler_state
	{
    	AddressU	= WRAP;
    	AddressV	= WRAP;
    	AddressW	= CLAMP;
    	MIPFILTER	= LINEAR;
    	MINFILTER	= LINEAR;
    	MAGFILTER	= LINEAR;
	},

	sampler_state
	{
    	AddressU	= WRAP;
    	AddressV	= WRAP;
    	AddressW	= CLAMP;
    	MIPFILTER	= LINEAR;
    	MINFILTER	= LINEAR;
    	MAGFILTER	= LINEAR;
	},

	sampler_state
	{
    	AddressU	= WRAP;
    	AddressV	= WRAP;
    	AddressW	= CLAMP;
    	MIPFILTER	= LINEAR;
    	MINFILTER	= LINEAR;
    	MAGFILTER	= LINEAR;
	},

	sampler_state
	{
    	AddressU	= WRAP;
    	AddressV	= WRAP;
    	AddressW	= CLAMP;
    	MIPFILTER	= LINEAR;
    	MINFILTER	= LINEAR;
    	MAGFILTER	= LINEAR;
	},

	sampler_state
	{
    	AddressU	= WRAP;
    	AddressV	= WRAP;
    	AddressW	= CLAMP;
    	MIPFILTER	= LINEAR;
    	MINFILTER	= LINEAR;
    	MAGFILTER	= LINEAR;
	},
};

//------------------------------------
VS_Output VS_Object_TextureAndLight(VS_Input IN)
{
    float4		xPos = float4(IN.xPosition, 1);
    
	VS_Output	OUT;

    // Calculate the vertex position...
    OUT.xPosition		= mul(xPos, g_xWorldMatrix);
    OUT.xDiffuse.rgb	= 1.0f;
    OUT.xDiffuse.a		= IN.fAlpha;
    
    OUT.xEyeDistance	= g_xCameraPosition - OUT.xPosition;
    
    OUT.xPosition		= mul(OUT.xPosition, g_xViewProjectionMatrix);

    // Calculate the vertex texture-coordinates...
	OUT.xUV	= CalculateTextureCoordinate (IN.xUV);

	// Calculate the normal...
	OUT.xNormalDepth.xyz	= mul(IN.xNormal, g_xWorldMatrix);
	OUT.xNormalDepth.w		= OUT.xPosition.w;

    // Return the vertex...
    return	(OUT);
}

//------------------------------------
DeferredOutputPixel PS_Object_TextureAndLight(VS_Output IN)
{
	float4		xWorldNormal = IN.xNormalDepth;

	xWorldNormal.xyz = normalize(xWorldNormal.xyz);
	xWorldNormal.w = 1.0;

	half4 xOutputRGBA;
	
	// Calculate the diffuse spherical-harmonic lighting...
	if (g_bObjectUseLighting)
	{
		xOutputRGBA	= CalculateDiffuseSHLightColour (g_xObjectRedSH, g_xObjectGreenSH, g_xObjectBlueSH, xWorldNormal);
	}
	else
	{
		xOutputRGBA	= float4(0.5f, 0.5f, 0.5f, 1.0f);
	}

	// Multiply by texture colour
	float4 xTexColour	= tex2D (axSampler [0], IN.xUV);

	if (g_bReflectiveTexture)
	{
		float3 xEyeDirection = normalize(IN.xEyeDistance);
		
		xTexColour.rgb += CalculateReflectiveColour_SphereMap(xEyeDirection, xWorldNormal.xyz, xTexColour, axSampler[4]);
	}

	if (g_bSelfIlluminating)
	{
		xOutputRGBA.rgb += xTexColour.rgb * xTexColour.a * 2.0f;
	}


	float fSpecScale;
	if (g_bObjectTextureTransparancy)
	{
		fSpecScale = 1;
		xTexColour.a *= g_fObjectAlpha;
	}
	else
	{
		fSpecScale = xTexColour.a;
		xTexColour.a = g_fObjectAlpha;
	}

	xOutputRGBA *= xTexColour;
	xOutputRGBA *= IN.xDiffuse;

	// Calculate the specular colour...
	if (g_bObjectSpecular && g_bObjectUseLighting)
	{
		float4 xSpecColour = CalculateSpecularSHLightColour (g_xObjectRedSH, g_xObjectGreenSH, g_xObjectBlueSH, xWorldNormal, fSpecScale, IN.xEyeDistance, g_xObjectBrightestLightDirection);
		xSpecColour	*= 2;
		xOutputRGBA.rgb += xSpecColour.rgb * fSpecScale;
	}
	
	half3 xOutputAlbedo = xTexColour;
	half hOutputSpecular = xTexColour.a;
	
	// Return the data...
	DeferredOutputPixel xOut = Deferred_FillOutputPixel( xOutputRGBA, xWorldNormal, IN.xNormalDepth.w, xOutputAlbedo, hOutputSpecular, 0);
	
	return( xOut);
}

//-----------------------------------
technique Object_TextureAndLight
{
    pass p0
    {
		VertexShader	= compile vs_3_0 VS_Object_TextureAndLight();
		PixelShader		= compile ps_3_0 PS_Object_TextureAndLight();
    }
}
