// This is intended for Tools only, to render the front face of a cubemap
// through the Asura overlay quad system

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const texture g_xDataTexture;
const sampler g_xDataTextureSampler = sampler_state
{
	Texture = <g_xDataTexture>;
	AddressU = CLAMP;
	AddressV = CLAMP;	
	MagFilter = POINT;
	MinFilter = POINT;
	MipFilter = POINT;
};

bool g_bShowRed;
bool g_bShowGreen;
bool g_bShowBlue;
bool g_bShowLum;

float g_fXScale;

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct Vertex
{
	float4 m_xPosition  : POSITION;
	float2 m_xTexcoords : TEXCOORD;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

Vertex VShader( const Vertex xIn )
{
	// Would prefer to not have a vertex shader and use transformed input
	// vertices, but that's not the way the Asura overlay quad system will pass
	// the input coordinates

	Vertex xOut = xIn;

	// Touch up the z and w values
	xOut.m_xPosition.z = 0.5f;
	xOut.m_xPosition.w = 1.0f;

	return xOut;
}

float4 PShader( const Vertex xIn ) : COLOR
{
	// Only 2D texcoords come through from the overlay quad system. We assume the input
	// coords run from [0,0] to [1,1]
	
	float4 xValues = tex1D(g_xDataTextureSampler, xIn.m_xTexcoords.x);
	float4 xValues2 = tex1D(g_xDataTextureSampler, xIn.m_xTexcoords.x - (1.0f / 256.0f));
	
	float fY = 1 - xIn.m_xTexcoords.yyyy;
	float fY2 = fY - (2.0f / 256.0f);

	float4 xOutput = step(fY.xxxx, xValues) * step(xValues2, fY.xxxx);
	xOutput = max(step(xValues, fY.xxxx) *  step(fY.xxxx, xValues2), xOutput);
	xOutput = max(step(fY2.xxxx, xValues) * step(xValues, fY.xxxx), xOutput);

	float4 xColour = float4((g_bShowLum ? xOutput.a : 0).xxx * 0.5f, 1.0f); 
	xOutput.a = 0.0f;
	
	xColour.rgb += xOutput.aab * 0.5f * g_bShowBlue;
	xColour.rgb += xOutput.aga * 0.5f * g_bShowGreen;
	xColour.rgb += xOutput.raa * 0.5f * g_bShowRed;

	float fX1 = (xIn.m_xTexcoords.x) * g_fXScale * 4.0f;
	float fX2 = (xIn.m_xTexcoords.x + 1.0f / 256.0f) * g_fXScale * 4.0f;

	xColour.rgb += (floor(fX1) != floor(fX2)) * (1.0f - ((g_fXScale - 1.0f) / 3.0f)) * 0.4f;

	fX1 = (xIn.m_xTexcoords.x) * g_fXScale;
	fX2 = (xIn.m_xTexcoords.x + 1.0f / 256.0f) * g_fXScale;
	
	xColour.rgb += (floor(fX1) != floor(fX2)) * 0.4f;

		
	return float4(saturate(xColour.rgb), 1);
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}