
// Light direction calculation from spherical harmonic...

float3 CalculateBrightestSHLightDirection (float4 xRedSH,
										   float4 xGreenSH,
										   float4 xBlueSH)
{
	float3	xBrightestLightDirection;
	
	// Calculate the brightest light direction from the spherical harmonic lighting...
	xBrightestLightDirection.x	= xRedSH.x + xGreenSH.x + xBlueSH.x;
	xBrightestLightDirection.y	= xRedSH.y + xGreenSH.y + xBlueSH.y;
	xBrightestLightDirection.z	= xRedSH.z + xGreenSH.z + xBlueSH.z;
	
	// Return the light direction...
	return	(xBrightestLightDirection);
}


// Diffuse dynamic light colour calculation...

float4 CalculateDiffuseSHLightColour (float4 xRedSH,
									  float4 xGreenSH,
									  float4 xBlueSH,
									  float4 xNormal)
{
	float4	xColour;

	// Apply the spherical harmonic lighting...
	xColour.r	= dot (xNormal, xRedSH);
	xColour.g	= dot (xNormal, xGreenSH);
	xColour.b	= dot (xNormal, xBlueSH);

	xColour.rgb = max (xColour.rgb, 0);
	xColour.a	= 1.0;
	
	// Move into HDR...
	xColour.rgb	= CalculateHighDynamicRangeRGB (xColour.rgb);

	// Return the colour...
	return	(xColour);
}


// Specular dynamic light colour calculation...

float4 CalculateSpecularSHLightColour (float4 xRedSH,
									   float4 xGreenSH,
									   float4 xBlueSH,
									   float4 xNormal,
									   float fSpecularScale,
									   float3 xEyeDistance,
									   float3 xBrightestLightDirection)
{
	float4 xLight;
	float3	xLightDirection	= normalize (xBrightestLightDirection);
	
	xLight.rgb = xLightDirection;
	xLight.w = 1.0f;
	
	float4	xDiffuseColour	= CalculateDiffuseSHLightColour (xRedSH, xGreenSH, xBlueSH, xLight);
	
	float3	xEyeDirection	= normalize (xEyeDistance);
	
	
	float4	xColour;

	// Calculate the specular colour...
	xColour.rgb	= CalculateLightSpecularRGB (1.0, xDiffuseColour, xLightDirection, xNormal.xyz, 1.0, xEyeDirection);
	xColour.a	= fSpecularScale;
	
	// Return the colour...
	return	(xColour);
}


/// End of file. ///
