///////////////////////////////////////////////////////////////////////////////
// This code is ported from Ben's PS3 version and is probably not perfectly
// same as the original, although it should behave almost in the same way.
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

#include "Shared.fxh"
#include "Deferred_Shared.fxh"

float2            g_xRTTextureResolution;
float2            g_xRTPixelSize;

// x = fn/(f-n); y = f/(f-n); z = n; where n = near clip, f = far clip, corresponding to the depth buffer being read.
float4            g_xDepthProjection;

float4            g_xParams;	         // x = sample distance, y = threshold factor, z = 1 if first pass 0 if not

texture           g_xDepthBufferTexture;
sampler           g_xDepthBufferSampler
{
	Texture = <g_xDepthBufferTexture>;
	MagFilter = POINT; 	
	MinFilter = POINT; 
	MipFilter = POINT; 	
	AddressU = CLAMP; 
	AddressV = CLAMP;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float3	PackFloatToR8G8B8( float fValue )	// value must be in [0, 1]
{
	fValue = fValue * 0.5 + 0.5;
	//[0,1]
	
 	fValue = (int)(fValue*16777215.999);
	//[0,16777215]int
	
  	float3 xOut = float3( fValue, fValue, fValue ) / float3( 65536, 256, 1 );
	//[0,255,65280+255,1677600+255]x3 ~int
	
	xOut = floor(xOut);
	//[0,255,65280+255,1677600+255]x3 int
	
	xOut /= 256.0;
	xOut = frac(xOut);
	//[0,1)x3
	
	xOut *= 256.0/255.0;	//very important, convert [0,1) to [0,1], this is why we did /256 not /255 a moment ago.
	//[0,1]x3
	
	return xOut;
}

// this will retrieve depth data and convert it into [0, 1] to emulate PS3 version.
float ConvertDepthBufferValueSRT( float4 xBufferNormal )
{
	float3 xNormal;
	float fScreenDepth;
	Deferred_GetNormalDepth( xBufferNormal, xNormal, fScreenDepth );
	
	// return the z-buffer depth to match PS3 version
	return g_xDepthProjection.y - g_xDepthProjection.x / fScreenDepth;
}

float ConvertDepthBufferValuePS3Style( float3 xDepthBufferValue )
{
	//read actual depth texture as RGB (A is stencil) - BW I remapped this from the actual memory layout which would have ARG = depth, B = stencil, this remapping is free & easier to work with.
	float3 depthColor = xDepthBufferValue.rgb;
	depthColor *= 255.0f;
	depthColor = round(depthColor);
	float depth = dot( depthColor, float3( 65536, 256, 1 ) );
	depth *= 1.0f/16777215.0;
	
	return depth*2.0f - 1.0f; //BW depth values are in the range [-1,1], though at the time of writing our projection is to [0,1] (so we're only using 23-bits of the buffer)
}

// retrieve 4 depth buffer values from rgb values, without correcting it into being a view-space distance
float4 Convert4DepthBufferValues( float3 xDepthBufferValue0, float3 xDepthBufferValue1, float3 xDepthBufferValue2, float3 xDepthBufferValue3 )
{
	// do the simple already tested version for now
	return float4(	ConvertDepthBufferValuePS3Style( xDepthBufferValue0 ),
					ConvertDepthBufferValuePS3Style( xDepthBufferValue1 ),
					ConvertDepthBufferValuePS3Style( xDepthBufferValue2 ),
					ConvertDepthBufferValuePS3Style( xDepthBufferValue3 ) );

	// this below is the PS3 version function code - might be faster, and might not, see comments.
/*
// for comparison:
//return float4(ConvertDepthBufferValue(xDepthBufferValue0),ConvertDepthBufferValue(xDepthBufferValue1),ConvertDepthBufferValue(xDepthBufferValue2),ConvertDepthBufferValue(xDepthBufferValue3));
//... which proves it's not actually any faster, and the transposed version is actually slower! grr.

// 	float4 xInR = float4( xDepthBufferValue0.r, xDepthBufferValue1.r, xDepthBufferValue2.r, xDepthBufferValue3.r );
// 	float4 xInG = float4( xDepthBufferValue0.g, xDepthBufferValue1.g, xDepthBufferValue2.g, xDepthBufferValue3.g );
// 	float4 xInB = float4( xDepthBufferValue0.b, xDepthBufferValue1.b, xDepthBufferValue2.b, xDepthBufferValue3.b );
// 
// 	xInR *= 255;
// 	xInG *= 255;
// 	xInB *= 255;
// 
// 	xInR = round(xInR);
// 	xInG = round(xInG);
// 	xInB = round(xInB);
// 	
// 	float4 xDepth = xInR*65536;
// 	xDepth += xInG*256;
// 	xDepth += xInB*1;

// original:	
	float3 xTemp;
	float4 xDepth;
	xTemp = round( xDepthBufferValue0*255 );
	xDepth.x = dot( xTemp, float3( 65536, 256, 1 ) );
	xTemp = round( xDepthBufferValue1*255 );
	xDepth.y = dot( xTemp, float3( 65536, 256, 1 ) );
	xTemp = round( xDepthBufferValue2*255 );
	xDepth.z = dot( xTemp, float3( 65536, 256, 1 ) );
	xTemp = round( xDepthBufferValue3*255 );
	xDepth.w = dot( xTemp, float3( 65536, 256, 1 ) );
	
	xDepth *= 1.0f/16777215.0;

	return xDepth*2-1;
	*/
}

void	SmartDepthBlur( float4 xInDepths, float4 xInConfidences, out float fOutDepth, out float fOutConfidence )
{

// 	// average together the samples, if they're within a certain threshold
// 	// annoyingly these samples are 1/w, but the threshold is in z.
// 	float2 fTemp = min( xInDepths.xy, xInDepths.zw );
// 	float fClosest = min( fTemp.x, fTemp.y );
// 
// 	float4 d1 = 1.0f/ (g_xDepthProjection.yyyy - xInDepths);
// 	float4 d2 = 1.0f/(g_xDepthProjection.yyyy - fClosest.xxxx); // closest sample depth
// 	d2 += g_xParams.yyyy; // distance beyond which we want to ignore samples
// 	float4 xFlags = saturate( 0.5f + ( d2 - d1 ) / (1.0f/g_xDepthProjection.xxxx) ); // convert to metres and fade over a specific range.
// 	xFlags *= xInConfidences;
// 
// 	float fDenom = dot( xFlags, float4(1,1,1,1) );
// 	float4 xWeights = xFlags / fDenom;	 // average
// 	
// 	float fDepth = dot ( xInDepths, xWeights );
// 	
// 	float fConfidence = dot( xInConfidences, xFlags ) / 4.0f; // so samples that fail the test have 0 confidence


	// coverage:
	// use whichever depth region occupies the most samples, fade confidence as they approach 50/50

	// pick values that are close enough to each other (within threshold)
	// choose the group of values closest to us unless we have more confidence in some other values
	// i.e. pick the highest confidence, using closest as a tie-breaker.
	// obviously if they're all close enough we use the average, otherwise we use the average of the in-range vals.

	// mask values of equal-highest confidence
 	float2 xTemp = max( xInConfidences.xy, xInConfidences.zw );
	float fMaxConf = max( xTemp.x, xTemp.y ) - 0.001f; //small threshold value, for precision (possibly not needed) //TODO: check this
	float4 xIgnoreMask = (xInConfidences >= fMaxConf.xxxx) ? float4(0, 0, 0, 0) : float4(1, 1, 1, 1); // 0 = use this value, 1 = ignore this value
	
	// pick closest of these
	float4 xMaskedVals = xInDepths + xIgnoreMask; // depth vals are in [0,1] so adding 1 to them guarantees they won't be closest
	xTemp = min( xMaskedVals.xy, xMaskedVals.zw );
	float fClosest = min( xTemp.x, xTemp.y );
	
	// construct xFlags from all vals within threshold of that
 	float4 d1 = 1.0f / ( g_xDepthProjection.yyyy - xInDepths );
 	float4 d2 = 1.0f / ( g_xDepthProjection.yyyy - fClosest.xxxx ); //note: we don't use g_xDepthProjection.x so these values are not in metres, but they are linear, this is fine since we only compare to g_xParams.y which was pre-multiplied appropriately
	float4 xDepth = abs( d1-d2 );
//	float4 xFlags = ( xDepth < g_xParams.yyyy*dot(xInConfidences,float4(0.25)) ) ? float4(1) : float4(0);
//now with fading:
	float4 xFlags = saturate( 0.5f + (1.0f - xDepth/g_xParams.yyyy)/0.4f ); // 0.5 when xDepth = threshold, 0 when xDepth = threshold*1.2, 1 when xDepth = threshold*0.8
//this doesn't have much impact.
	
	// mul by confidence
	xFlags *= xInConfidences;
	float fFlagMag = dot( xFlags, float4(1, 1, 1, 1) );
	
	// new confidence = 1111.flags / 4;
	fOutConfidence = fFlagMag / 4.0f;
	
	// new val = vals.flags / 1111.flags;
	fOutDepth = dot( xFlags, xInDepths ) / fFlagMag;
}

float4 DepthDownsampleFromRTPS( const float2 xScreenVPos : VPOS ) : COLOR
{
	// smart filter:
	//   sample 4 points (which may themselves be sampled from 4 points, etc.)
	//   compute a point that represents the occlusion of this area when viewed from points outside the area
	//   (if it's relevant) we can assume those points are about 1/3rd to 1/2 the width of this area away, approximately horizontally & vertically
	//     - at the time of writing precise positions of those points haven't been decided (it's likely something that will be tweaked for best results).

	// since we're downsampling to half size texture, the pixel we write to is twice the size of the pixel we read from,
	// so adjust fSamplerPixelSize accordingly.
	float2 fSamplerPixelSize = g_xRTPixelSize * 0.5;
	
	// get sampler pixel pos and move it to the pixel center
	float2 xScreenUV = xScreenVPos / g_xRTTextureResolution;
	xScreenUV += fSamplerPixelSize * 0.5;
	
	float2 xTexCoords = xScreenVPos;
	
	float2 texSamplingOffset = float2( 0.0, 0.0 ); //float2( -0.5, -0.5 );

	float4 xDepthBufferValues[4] =
		{
			// just for testing, sample 4 neighbouring pixels
			/*
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 0.0, 0.0 )  ),
 			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 1.0, 0.0 )  ),
 			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 0.0, 1.0 )  ),
 			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 1.0, 1.0 )  )
*/
			
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + 2 * xTexCoords + float2(-1,-1) * g_xParams.x ) ),
 			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + 2 * xTexCoords + float2( 1,-1) * g_xParams.x ) ),
 			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + 2 * xTexCoords + float2(-1, 1) * g_xParams.x ) ),
 			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + 2 * xTexCoords + float2( 1, 1) * g_xParams.x ) )
 			
		};
	
	
	float4 xDepthSamples = 
	float4(
			ConvertDepthBufferValueSRT( xDepthBufferValues[0] ), 
			ConvertDepthBufferValueSRT( xDepthBufferValues[1] ),
			ConvertDepthBufferValueSRT( xDepthBufferValues[2] ),
			ConvertDepthBufferValueSRT( xDepthBufferValues[3] )
		);
	
	float fConfidence;
	float fDepth;
	
	SmartDepthBlur( xDepthSamples, float4(1, 1, 1, 1), fDepth, fConfidence );

	float3 xOut = PackFloatToR8G8B8( fDepth );

	// output depth as RGB (depth buffer style) and confidence as alpha	
	return float4( xOut, fConfidence );
}

float4 DepthBlur16PS( float2 xScreenVPos : VPOS ) : COLOR
{
	// since we're downsampling to half size texture, the pixel we write to is twice the size of the pixel we read from,
	// so adjust fSamplerPixelSize accordingly.
	float2 fSamplerPixelSize = g_xRTPixelSize;
	
	// get sampler pixel pos and move it to the pixel center
	float2 xScreenUV = xScreenVPos / g_xRTTextureResolution;
	xScreenUV += fSamplerPixelSize * 0.5;
	
	//return tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2(0.0, 0.0) );

	// <*> // float2 xTexCoords = xScreenVPos;
	float2 xTexCoords = xScreenVPos;

	float2 texSamplingOffset = float2( 1.0, 1.0 );

	float4 xDepthBufferValues[16] =
		{
			// just for testing, sample 16 neighbouring pixels
			/*
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2(-1, -1) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 0, -1) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 1, -1) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 2, -1) ),
																						
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2(-1,  0) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 0,  0) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 1,  0) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 2,  0) ),
																						
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2(-1,  1) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 0,  1) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 1,  1) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 2,  1) ),
																						
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2(-1,  2) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 0,  2) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 1,  2) ),
			tex2D( g_xDepthBufferSampler, xScreenUV + fSamplerPixelSize * float2( 2,  2) ),
		*/
			
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2(-1,-1)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2(-3,-1)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2(-1,-3)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2(-3,-3)*g_xParams.x ) ),

			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2( 1,-1)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2( 3,-1)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2( 1,-3)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2( 3,-3)*g_xParams.x ) ),

			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2(-1, 1)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2(-3, 1)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2(-1, 3)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2(-3, 3)*g_xParams.x ) ),

			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2( 1, 1)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2( 3, 1)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2( 1, 3)*g_xParams.x ) ),
			tex2D( g_xDepthBufferSampler, fSamplerPixelSize * ( texSamplingOffset + xTexCoords + float2( 3, 3)*g_xParams.x ) )
			
		};

	float4 axDepthSamples[4] = {
			Convert4DepthBufferValues( xDepthBufferValues[0].rgb, xDepthBufferValues[4].rgb, xDepthBufferValues[8].rgb, xDepthBufferValues[12].rgb ),
			Convert4DepthBufferValues( xDepthBufferValues[1].rgb, xDepthBufferValues[5].rgb, xDepthBufferValues[9].rgb, xDepthBufferValues[13].rgb ),
			Convert4DepthBufferValues( xDepthBufferValues[2].rgb, xDepthBufferValues[6].rgb, xDepthBufferValues[10].rgb, xDepthBufferValues[14].rgb ),
			Convert4DepthBufferValues( xDepthBufferValues[3].rgb, xDepthBufferValues[7].rgb, xDepthBufferValues[11].rgb, xDepthBufferValues[15].rgb )
		};
	// not float4x4 because Cg doesn't support component-wise maths on that in quite the same way.
	
	float4 axConfidenceSamples[4] = {
			float4( xDepthBufferValues[0].a, xDepthBufferValues[4].a, xDepthBufferValues[8].a, xDepthBufferValues[12].a ),
			float4( xDepthBufferValues[1].a, xDepthBufferValues[5].a, xDepthBufferValues[9].a, xDepthBufferValues[13].a ),
			float4( xDepthBufferValues[2].a, xDepthBufferValues[6].a, xDepthBufferValues[10].a, xDepthBufferValues[14].a ),
			float4( xDepthBufferValues[3].a, xDepthBufferValues[7].a, xDepthBufferValues[11].a, xDepthBufferValues[15].a )
		};

// to get same results should do each group of 4 first, then re-vectorise and do a 5th pass on that.
// for efficiency I've cunningly arranged each group of 4 adjacent pixels in x,y,z, and w respectively, so we can do all the first 4 at once

	float4 xTemp1 = max( axConfidenceSamples[0], axConfidenceSamples[1] );
	float4 xTemp2 = max( axConfidenceSamples[2], axConfidenceSamples[3] );
	float4 xMaxConf	= max( xTemp1, xTemp2 ) - 0.001f; //small threshold value, for precision (possibly not needed)

	// mask everything with that confidence
	float4 axMask[4] = {
		(axConfidenceSamples[0] >= xMaxConf) ? float4(0,0,0,0) : float4(1,1,1,1),	// 0 = use this value, 1 = ignore this val
		(axConfidenceSamples[1] >= xMaxConf) ? float4(0,0,0,0) : float4(1,1,1,1),
		(axConfidenceSamples[2] >= xMaxConf) ? float4(0,0,0,0) : float4(1,1,1,1),
		(axConfidenceSamples[3] >= xMaxConf) ? float4(0,0,0,0) : float4(1,1,1,1)
	};
	
	// pick the closest of those most confident values
	xTemp1 = min( axDepthSamples[0]+axMask[0], axDepthSamples[1]+axMask[1] ); // depth vals are in [0,1] so adding 1 to them guarantees they won't be closest 
	xTemp2 = min( axDepthSamples[2]+axMask[2], axDepthSamples[3]+axMask[3] );
	float4 xClosestMaskedDepth = min( xTemp1, xTemp2 );

	// mask everything close to that closest confident value depth (including values with lower confidence)
 	float4 xClosestLinear = 1.0f / ( g_xDepthProjection.yyyy - xClosestMaskedDepth );
 	float4 xLowerBound = max( g_xDepthProjection.zzzz, xClosestLinear - g_xParams.yyyy ); //clamp to near plane, the maths goes wrong if we try to use something closer than that.
 	float4 xUpperBound = max( g_xDepthProjection.zzzz, xClosestLinear + g_xParams.yyyy );
 	xLowerBound = g_xDepthProjection.yyyy - 1.0f / xLowerBound;
 	xUpperBound = g_xDepthProjection.yyyy - 1.0f / xUpperBound;

 	float4 xMid = (xLowerBound+xUpperBound) / 2.0f;
 	float4 xRange = abs(xUpperBound - xLowerBound)/2.0f;

	float4 axFlags[4] = {
		(abs(axDepthSamples[0] - xMid) < xRange) ? float4(1,1,1,1) : float4(0,0,0,0),
		(abs(axDepthSamples[1] - xMid) < xRange) ? float4(1,1,1,1) : float4(0,0,0,0),
		(abs(axDepthSamples[2] - xMid) < xRange) ? float4(1,1,1,1) : float4(0,0,0,0),
		(abs(axDepthSamples[3] - xMid) < xRange) ? float4(1,1,1,1) : float4(0,0,0,0),
	};
// the working for ^ :
// abs(depthdiff)<param
// -param < 1/(dpy-d) - c < param
// //I *do* need to check both signs - majority rules
// c-param < 1/(dpy-d) < c+param
// //know 1/(p-d) >= 0 (it's proportional to viewspace depth)
// 1/(c-param) - dpy > -d > 1/(c+param) - dpy
// dpy - 1/(c-param) < d < dpy - 1/(c+param)
// => 2 vals to check against

	axFlags[0] *= axConfidenceSamples[0];
	axFlags[1] *= axConfidenceSamples[1];
	axFlags[2] *= axConfidenceSamples[2];
	axFlags[3] *= axConfidenceSamples[3];
	
	float4 xFlagMags = axFlags[0]+axFlags[1]+axFlags[2]+axFlags[3];
	
	// average them, weighted by confidence
	float4 xDepths = axDepthSamples[0]*axFlags[0] / xFlagMags;
	xDepths += axDepthSamples[1]*axFlags[1] / xFlagMags;
	xDepths += axDepthSamples[2]*axFlags[2] / xFlagMags;
	xDepths += axDepthSamples[3]*axFlags[3] / xFlagMags;

	float4 xConfidences = xFlagMags / 4.0f;		 // this has already been mulled by the sample confidences
	

	// coverage:
	// use whichever depth region occupies the most samples, fade confidence as they approach 50/50

	// pick values that are close enough to each other (within threshold)
	// choose the group of values closest to us unless we have more confidence in some other values
	// i.e. pick the highest confidence, using closest as a tie-breaker.
	// obviously if they're all close enough we use the average, otherwise we use the average of the in-range vals.

	// mask values of equal-highest confidence
 	float2 xTemp = max( xConfidences.xy, xConfidences.zw );
	float fMaxConf = max( xTemp.x, xTemp.y ) - 0.001f; //small threshold value, for precision (possibly not needed)
	float4 xIgnoreMask = (xConfidences >= fMaxConf.xxxx) ? float4(0,0,0,0) : float4(1,1,1,1); // 0 = use this value, 1 = ignore this value
	
	// pick closest of these
	float4 xMaskedVals = xDepths + xIgnoreMask; // depth vals are in [0,1] so adding 1 to them guarantees they won't be closest
	xTemp = min( xMaskedVals.xy, xMaskedVals.zw );
	float fClosest = min( xTemp.x, xTemp.y );
	
	// construct xFlags from all vals within threshold of that
 	float4 d1 = 1.0f / ( g_xDepthProjection.yyyy - xDepths );
 	float4 d2 = 1.0f / ( g_xDepthProjection.yyyy - fClosest.xxxx ); //note: we don't use g_xDepthProjection.x so these values are not in metres, but they are linear, this is fine since we only compare to g_xParams.y which was pre-multiplied appropriately
	float4 xDepthDiff = abs( d1-d2 );
	float4 xFlags = ( xDepthDiff < g_xParams.yyyy ) ? float4(1,1,1,1) : float4(0,0,0,0);
//now with fading:
//	float4 xFlags = saturate( 0.5f + (1.0f - xDepthDiff/g_xParams.yyyy)/0.4f ); // 0.5 when xDepth = threshold, 0 when xDepth = threshold*1.2, 1 when xDepth = threshold*0.8
//this doesn't have much impact.
	
	// mul by confidence
	xFlags *= xConfidences;
	float fFlagMag = dot( xFlags, float4(1,1,1,1) );
	
	// new confidence = 1111.flags / 4;
	float fConfidence = fFlagMag / 4.0f;
	
	// new val = vals.flags / 1111.flags;
	float fDepth = dot( xFlags, xDepths ) / fFlagMag;


	// pack it back into RGB8
	float3 xOut = PackFloatToR8G8B8( fDepth );

	// output depth as RGB (depth buffer style) and confidence as alpha	
	return float4( xOut, fConfidence );
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


// <*> const sampler	g_xNormalMaterialBufferSampler;		//:register( s0 );
// <*> const sampler	g_xDepthBufferSampler;				//:register( s2 );

texture           g_xDeferredRT_NI_Texture;
sampler           g_xDeferredRT_NI_Sampler
{
	Texture = <g_xDeferredRT_NI_Texture>;
	MagFilter = POINT; MinFilter = POINT; MipFilter = POINT; AddressU = CLAMP; AddressV = CLAMP;
};

texture			g_axDepthDownsampleTexture0;
sampler			g_axDepthDownsampleSampler0
{	
	Texture = <g_axDepthDownsampleTexture0>; 
	MagFilter = POINT; MinFilter = POINT; MipFilter = POINT; AddressU = CLAMP; AddressV = CLAMP;
};

texture			g_axDepthDownsampleTexture1;
sampler			g_axDepthDownsampleSampler1
{	
	Texture = <g_axDepthDownsampleTexture1>; 
	MagFilter = POINT; MinFilter = POINT; MipFilter = POINT; AddressU = CLAMP; AddressV = CLAMP;
};
/*
texture			g_axDepthDownsampleTexture2;
sampler			g_axDepthDownsampleSampler2
{	
	Texture = <g_axDepthDownsampleTexture2>; 
	MagFilter = POINT; MinFilter = POINT; MipFilter = POINT; AddressU = CLAMP; AddressV = CLAMP;
};
*/

float4x4		g_xViewMatrix;

float4			g_xTwoOverScreen;		// dimensions of screen, in pixels. Used to convert VPOS pixel shader input into normalised coordinates.
float4			g_xCameraTanHalfFOV;	// Slopes of camera frustum. Used to reconstruct viewspace positions from screen coordinates and depth.
float2			g_xDepthAdjustment;

float4			g_xBrightnessContrast;

float4			g_xDistanceFade;

float4			g_axSamplePositions[2]; // = { float4( -1,1, -2,6 ), float4( -6,26, 0,0 ) }; (for testing)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// used for debugging only
/*
const float4x4 g_xInvViewMatrix;
const float    g_fTwoOverScreenWidth;						// Dimensions of screen, in pixels. Used to convert VPOS pixel shader input into normalised coordinates.
const float    g_fTwoOverScreenHeight;
const float    g_fCameraTanHalfHorizFOV;					// Slopes of camera frustum. Used to reconstruct viewspace positions from screen coordinates and depth.
const float    g_fCameraTanHalfVertFOV;
*/
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Convert z-buffer depth to distance from camera view plane
// ( distance from camera view is (worldPos * viewProj).w, z-buffer depth is ((worldPos * viewProj).z / (worldPos * viewProj).w) )
float DepthToDistanceFromViewPlane( float fDepth )
{
	// invert the projection & perspective. i.e.
	// Z = b-a*z; W=z; depth = Z/W;
	// so z = a/(b-depth)
 	return g_xDepthAdjustment.x / (g_xDepthAdjustment.y - fDepth);
}

float ps3style_GetDepth( float3 xDepthBufferValue )
{
	float fUnpackedZ = ConvertDepthBufferValuePS3Style( xDepthBufferValue );
	
	return DepthToDistanceFromViewPlane( fUnpackedZ );
}

// nabbed this from the lighting code, should put it in the material response header or one of the other headers
float4 ScreenToViewPosition( float2 xScreenVPos, float fScreenDepth )
{
	//xScreenVPos -= float2( 0.5, 0.5 );
	
	float4 xRetVal = float4(	g_xCameraTanHalfFOV.xy * fScreenDepth * ( xScreenVPos * g_xTwoOverScreen.xy - float2( 1.0f, 1.0f ) ), fScreenDepth, 1.0f );
	xRetVal.y *= -1;
	return xRetVal;
}

// This can be useful for testing (to perform calculations in world space instead of view space)
// (in order for this to work you'll have to enable setting of variables in Asura_Direct3D_AmbientOcclusion.cpp, ApplyShadows )
/*
float4 ScreenToWorldPosition( float2 xDepthTexturePixelPos, float fDepth )
{
	const float4 xViewSpacePosition = float4(	g_fCameraTanHalfHorizFOV * fDepth * (xDepthTexturePixelPos.x * g_fTwoOverScreenWidth  - 1.0),
													g_fCameraTanHalfVertFOV  * fDepth * -(xDepthTexturePixelPos.y * g_fTwoOverScreenHeight - 1.0),
													fDepth, 1.0);
	
	float4 xWorldPosition = mul( xViewSpacePosition, g_xInvViewMatrix );
	return xWorldPosition;
}
*/

float CalcOcclusion( float3 xBasePos, float3 xBaseNormal, float3 xSamplePos, float fSampleConfidence )
{
	float3 xDiff = xSamplePos - xBasePos;
	
	float fLength = sqrt( dot( xDiff.xyz,xDiff.xyz ) );

	float fResult = saturate( dot( xBaseNormal, xDiff.xyz ) / fLength ); // we *do* need all this precision & floats.
 		
	float fReduction = 1.0;

//		// reduce more on distant samples to hide the hard-edges resulting from the limited blur range in the data
//		float fPixelRange = 2*sqrt(dot(xSamp,xSamp)); // distance in pixels to fade to
//		float fMaxDist = g_xCameraTanHalfFOV.x * fDepth * fPixelRange * g_xTwoOverScreen.x; // equivalent world-space distance
//		// clamp max dist to give more consistent results at different depths
//		//fMaxDist = min( fMaxDist, 1.0f );
	float fMaxDist = g_xDistanceFade.x * fSampleConfidence; //^ is just not getting nice results, < is surprisingly effective
	float fFadeRange = 0.2f; // as a proportion of 1.0
	fReduction *= saturate( (1.0f - fLength / fMaxDist ) / fFadeRange );

	// reduce the significance of this sample
	// since position is not linearly related to occlusion angle we might want to tweak this curve a bit (so long as it's cheaper than an acos)
	fReduction = pow( fReduction, 1 );

	return fResult * fReduction;
}

float4 SampleOcclusion( float2 xScreenVPos, float fBaseDepth, float4 xBaseViewPos, float3 xBaseNormal, float2 xSamp, sampler xDepthSampler, int iLevel, out float4 xConfidence )
{
	float2 fDepthSamplerPixelSize = g_xRTPixelSize * 2.0;
	
	float fScale = 2;

	// take 4 samples around the base pos
	float2 axSamplePos[4] = { xSamp, float2( xSamp.y, -xSamp.x ), -xSamp, float2( -xSamp.y, xSamp.x ) };
	
	float4 xResult;
	
	float2 xRoundedBasePos = round( xScreenVPos / fScale ); // round to closest
	
	xConfidence = float4( 0, 0, 0, 0 );
	
	[unroll] for ( int i=0; i < 4; i++ )
	{
		float fDepth;

		float2 xActualPos, xTextureCoord;
		
		if ( iLevel == 1 )
		{
			// sample the first downsampled buffer, pixel centres are at the centre of the 4 pixels they represent, /2
			xTextureCoord = xRoundedBasePos + axSamplePos[i]/fScale;
		}
		else
		{
			// sample a buffer blurred from one of the same size, which has a 1/2 pixel offset in the -ve direction.
			xTextureCoord = xRoundedBasePos + axSamplePos[i]/fScale - float2(0.5, 0.5);
		}
		xActualPos = xRoundedBasePos*fScale + axSamplePos[i]; // this will be innaccurate with sample pos's that don't deal properly with the downsampled buffer (e.g. non-int ones or odd ones when iLevel>1)
		
		
		// <*> // float4 xTexel = h4tex2D( g_axDepthDownsampleSampler[iLevel-1], xTextureCoord );
		// <*> // fDepth = Deferred_GetDepth( xTexel );
		float4 xTexel = tex2D( xDepthSampler, fDepthSamplerPixelSize * xTextureCoord );
		fDepth = ps3style_GetDepth( xTexel.rgb );

		// convert this value to an amount of occlusion, allowing for perspective distortion
 		float4 xAdjPos = ScreenToViewPosition( xActualPos, fDepth );
 		//float4 xAdjPos = ScreenToWorldPosition( xActualPos, fDepth );
 		
		xConfidence[i]	= xTexel.a;
 		xResult[i]		= CalcOcclusion( xBaseViewPos.xyz, xBaseNormal, xAdjPos.xyz, xTexel.a );
	}
	
	//xResult.yzw = 0.0;
	//xConfidence.yzw = 0.0;
	
	return xResult;
}

float4 OcclusionPS( float2 xScreenVPos : VPOS ) : COLOR
{
	//xScreenVPos = round( xScreenVPos / 2 ) * 2;

	float2 xScreenUV = (xScreenVPos + float2(0.5, 0.5)) / g_xRTTextureResolution;
	
	//float smpdpth = ps3style_GetDepth( tex2D( g_axDepthDownsampleSampler0, xScreenUV ) );
	
// 	//elaborate comparison between depth buffer and blurred depth buffer, it helped find some serious errors
// 	if( xScreenVPos.x < 256 )
// 		return Deferred_GetDepth( h4tex2D( g_axDepthDownsampleSampler[1], xScreenVPos ) );
// 	else if ( xScreenVPos.x < 512 )
// 		return h4tex2D( g_axDepthDownsampleSampler[1], xScreenVPos/2 );
// 	else if ( xScreenVPos.x < 768 )
// 		return 64*abs(h4tex2D( g_axDepthDownsampleSampler[2], xScreenVPos )-h4tex2D( g_xDepthBufferSampler, xScreenVPos ));
// 	else if ( xScreenVPos.x < 1024 )
// 		return h4tex2D( g_xDepthBufferSampler, xScreenVPos );
// 	else
// 		return Deferred_GetDepth( h4tex2D( g_xDepthBufferSampler, xScreenVPos ) );

// return float4(	h4tex2D( g_axDepthDownsampleSampler[0], xScreenVPos ).a,
// 				h4tex2D( g_axDepthDownsampleSampler[1], xScreenVPos ).a,
// 				h4tex2D( g_axDepthDownsampleSampler[2], xScreenVPos ).a,
// 				1 );


 	// <*> // float4 xNormalDepth = GetNormalDepth(xScreenVPos);
 	// <*> // float fDepth = xNormalDepth.w;
	float3 xNormal;
	float fDepth;
	Deferred_GetNormalDepth( tex2D( g_xDeferredRT_NI_Sampler, xScreenUV ), xNormal, fDepth );
 	// (normals and depths are ok here, tested)
 	
 	// transform normal into view space
 	
 	// to do the occlusion in world space (useful for testing) disable this line, and use ScreenToWorldPosition instead of ScreenToViewPosition
 	xNormal = mul( xNormal, g_xViewMatrix );


	// get viewspace position of this pixel
 	float4 xPosition = ScreenToViewPosition( xScreenVPos, fDepth );
 	//float4 xPosition = ScreenToWorldPosition( xScreenVPos + float2( 0.5, 0.5 ), fDepth );
 	
 	const float fSampDist = 1;//3.0;

	float4 axOcclusion[3] = { float4(0, 0, 0, 0), float4(0, 0, 0, 0), float4(0, 0, 0, 0) };
	float4 axConfidence[3] = { float4(0, 0, 0, 0), float4(0, 0, 0, 0), float4(0, 0, 0, 0) };
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	#define NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS	(2) //corresponding to declaration in Asura_Direct3D_AmbientOcclusion.cpp

	axOcclusion[0] = SampleOcclusion( xScreenVPos, fDepth, xPosition, xNormal, g_axSamplePositions[0].xy, g_axDepthDownsampleSampler0, 1, axConfidence[0] );
#if ( NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS > 1 )
	axOcclusion[1] = SampleOcclusion( xScreenVPos, fDepth, xPosition, xNormal, g_axSamplePositions[0].zw, g_axDepthDownsampleSampler1, 2, axConfidence[1] );
#if ( NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS > 2 )
	axOcclusion[2] = SampleOcclusion( xScreenVPos, fDepth, xPosition, xNormal, g_axSamplePositions[1].xy, g_axDepthDownsampleSampler2, 3, axConfidence[2] );
#endif
#endif
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

 	// reduce occlusion by confidence
 	// lerp towards nearer occlusion, to simulate a small number of peaks poking over the nearer things
	axOcclusion[0] *= axConfidence[0];
	axOcclusion[1] = lerp( axOcclusion[0], axOcclusion[1], axConfidence[1] );
	axOcclusion[2] = lerp( axOcclusion[1], axOcclusion[2], axConfidence[2] );
	
	float4 xOcclusion = axOcclusion[0];
#if ( NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS > 1 )
	xOcclusion = max( xOcclusion, axOcclusion[1] );
#if ( NUM_AMBIENT_OCCLUSION_SHADER_SAMPLERS > 2 )
	xOcclusion = max( xOcclusion, axOcclusion[2] );
#endif
#endif

	// combine occlusion
	xOcclusion = acos( xOcclusion );	//this value is 0 for vertical (fully occluded) and pi/2 for horizontal (not occluded in this direction)
	
	// assume each direction represents approximate occlusion of a 90 degree region, so just average them together
	// might be better to simulate some overlap (to fake the regions that aren't sampled), e.g. for 180 degrees you add the opposing ones xz+=yw then multiply the results vis = x*z
	float fVisibility = dot( xOcclusion, float4(1 / 6.283185, 1 / 6.283185, 1 / 6.283185, 1 / 6.283185) );
	
	// adjust & clamp the effect
	// - to remove artefacts on curved surfaces (which self-shadow since the normals don't match the actual shape)
	// - and to artificially deepen shadows
	fVisibility = saturate(fVisibility*g_xBrightnessContrast.x + g_xBrightnessContrast.y);

//return float4( fVisibility * (axConfidence[2].xyz * axConfidence[2].w+0)*2, 1 ); //neat CMYK debug mode.
//if ( fVisibility > 0.95 ) return float4 ( 1 - frac( fVisibility.xxx * float3(4,16,1)/0.05f ) ,1 ); // debug - emphasize errors in near-unoccluded surfaces (to pick up on flat surface self-occlusion artefacts)
//	return float4( fVisibility.xxx*3, 1 );	 //better brightness (silly HDR doesn't adapt well)
	return float4( fVisibility.xxx, 1 );
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/*
texture           g_xSourceTexture;
sampler           g_xSourceSampler
{
	Texture = <g_xSourceTexture>;
	MagFilter = POINT;	MinFilter = POINT;	MipFilter = POINT;	AddressU = CLAMP;	AddressV = CLAMP;
};

///////////////////////////////////////////////////////////////////////////////
// used for debugging purposes only
float4 JustCopyPS( const float2 xScreenVPos : VPOS ) : COLOR
{
	float2 xScreenUV = (xScreenVPos + float2(0.5, 0.5)) / g_xRTTextureResolution;
	
	//float4 xRetVal = tex2D( g_xSourceSampler, xScreenUV );
	float4 xRetVal = tex2D( g_axDepthDownsampleSampler1, xScreenUV );
	
	xRetVal.xyz = float3( 0.5 + sin( ps3style_GetDepth( xRetVal.xyz ).xxx * 2 ) * 0.5 );
	//xRetVal.xyz = float3( xRetVal.www );
	clip( -1 );
	
	xRetVal.w = 1;
	
	return xRetVal;
}
*/
///////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique SSAO
{
	// pass 0
	pass DepthDownsampleFromRT
	{
		VertexShader = NULL;
		PixelShader = compile ps_3_0 DepthDownsampleFromRTPS( );
	}
	
	// pass 1
	pass DepthBlur
	{
		VertexShader = NULL;
		PixelShader = compile ps_3_0 DepthBlur16PS();
	}

	// pass 2
	pass Occlusion
	{
		VertexShader = NULL;
		PixelShader = compile ps_3_0 OcclusionPS();
	}

	/*
	// debugging only
	// pass 3
	pass DrawDebugInfo
	{
		VertexShader = NULL;
		PixelShader = compile ps_3_0 JustCopyPS();
	}
	*/
}
