
// Texture-related variables...

#define MAX_TEXTURE_COORDS	2

shared	bool	g_bDualPassTexture;
//shared	bool	g_bSelfIlluminatingTexture;

shared	float4	g_xTextureCoordOffsetAndScale	= {0.0, 0.0, 1.0f, 1.0f};


// Camera-related variables...

shared 	float4	g_xCameraPosition	: CameraPosition;


// Time-related variables...

shared 	float	g_fCurrentTime	: Time;


// View-related variables...

shared 	float4x4	g_xViewProjectionMatrix	: ViewProjection;


// High Dynamic Range stuff

shared 	float 	g_fHighDynamicRangeExposureScale	= 4.0f;

// Shadowing

// Shadow frustum near/far values, used to map shadow depth textures into the appropriate range
shared float	g_fShadowFrustumNearPlaneDistance;
shared float	g_fOneOverShadowFrustumPlaneSeparation;

#if defined( ASURA_TOOLS )

	// If these move, update Asura_Direct3D_MaterialResponse::SetMaterialConstants(). We ensure
	// they're bound only for pixel shader targets, since otherwise the compiler pushes the bones
	// for the skinning vertex shaders up too high for most graphics cards (even though these
	// parameters aren't referenced by those shaders...)

	shared bool g_bDisableLighting : register( ps, b0 ) : register( ps, c222 ) = false;
	shared bool g_bShowPerPixelNormals : register( ps, b1 ) : register( ps, c223 ) = false;

#endif

shared bool g_bUseGammaCorrection;

// Texture co-ordinate calculation function...

float2 CalculateTextureCoordinate (float2 xUV)
{
	// Simply scale the existing co-ordinates and add the offset...
	return	((xUV * g_xTextureCoordOffsetAndScale.zw) + g_xTextureCoordOffsetAndScale.xy);
}


// Tangent-space rotation function...

float3 RotateIntoTangentSpace (float3 xVector,
							   float3 xTangent,
							   float3 xBinormal,
							   float3 xNormal)
{
	float3	xRotatedVector;

	// Rotate the x, y and z components into tangent-space...
	xRotatedVector.x	= dot(xVector, xTangent);
	xRotatedVector.y	= dot(xVector, xBinormal);
	xRotatedVector.z	= dot(xVector, xNormal);

	// Return the rotated vector...
	return	(xRotatedVector);
}

float3 RotateFromTangentSpace (float3 xVector,
							  float3 xTangent,
							  float3 xBinormal,
							  float3 xNormal)
{
	float3	xRotatedVector;

	// Rotate the x, y and z components into tangent-space...
	xRotatedVector.xyz	= xVector.x * xTangent;
	xRotatedVector.xyz	+= xVector.y * xBinormal;
	xRotatedVector.xyz	+= xVector.z * xNormal;

	// Return the rotated vector...
	return	(xRotatedVector);
}

// High-dynamic range calculation function...

float3 CalculateHighDynamicRangeRGB (float3 xColour)
{
	// Move High Dynamic Range lighting into display range...
	xColour	*= g_fHighDynamicRangeExposureScale;
	xColour	 = (xColour) / (xColour + 1.0);
	
	// Return the colour...
	return	(xColour);
}


/// End of file. ///
