/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

#include "Shared.fxh"
#include "Deferred_Shared.fxh"

const float4x4	g_xWorldViewProjMatrix;
const float4x4	g_xFrustumToCamDir;

const float3	g_xSkyboxColour			= float3( 0.0f, 0.0f, 0.3f );
const float3	g_xSunDir1				= float3( 0.1f, 0.8f, 1.0f );
const float3	g_xSunDir2				= float3( 0.1f, 0.8f, 1.0f );

const float		g_xHazeGradientPower1	= 1.0f;
const float3	g_xHazeColour1			= float3( 0.7f, 0.8f, 0.9f );
const float		g_xHazeGradientPower2	= 1.0f;
const float3	g_xHazeColour2			= float3( 1.0f, 1.0f, 1.0f );

const float		g_xSunGradientPower1	= 50.0f;
const float3	g_xSunColour1			= float3( 2.0f, 1.8f, 1.5f );

const float		g_xSunGradientPower2	= 250.0f;
const float3	g_xSunColour2			= float3( 1.0f, 0.8f, 0.5f );

const bool		g_bDisableHaze			= false;
const bool		g_bDisable2Haze			= false;

const bool		g_bDisableSun			= false;
const bool		g_bDisable2Sun			= false;

const bool		g_bMirrorBelowHorizon	= false;
const bool		g_b2MirrorBelowHorizon	= false;

const sampler BaseTexture : register(s0);

const float4 SkyboxColour;
const float4 FogColour;			
const float  FogBlendFactor;

const bool   FlipTextureVertical;

// Distance in metres to output to the normal-depth RT. Should be
// beyond the far clip plane so the fog pass works correctly
const float g_xDistance;

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float3 Position : POSITION0;
	float2 UV       : TEXCOORD0;
};

struct InputVertexT1
{
	float4 Position : POSITION0;
};

struct OutputVertex
{
	float4 Position : POSITION0;
	float2 UV       : TEXCOORD0;
};

struct OutputVertexT1
{
	float4 m_xPosition	: POSITION0;
	float3 m_xCameraDir	: TEXCOORD1;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float3 CalculateYColour( float3 xCameraDir, float3 xInColour, float3 xDestColour, float fPow, bool bMirrorBelowHorizon )
{
	float fCamDir	= bMirrorBelowHorizon ? abs( xCameraDir.y ) : max( xCameraDir.y, 0.0f );

	// (AAM - 24.03.2009) This was a saturate, but that breaks the March 2009 HLSL compiler. See the
	// readme for that release for details. It can go back to a saturate when that's fixed
	// (AS - 01.04.2009) So the clamp is a warning on Nov08, saturate is an error on March09
	// Novemebers version number handling seems a bit dodgy but this should hopefulyl work
#if D3DX_VERSION >= 0xA29 // March 09
	float xT1		= clamp( 1.0f - fCamDir, 0.00001f, 1.0f );
#else
	float xT1		= saturate( 1.0f - fCamDir);
#endif 
	float xTmp1		= pow( xT1, fPow );

	return lerp( xInColour, xDestColour, xTmp1 );
}

float3 CalculateSun( float fDot, float3 xInColour, float3 xDestColour, float fPow )
{
	float xTmp2	= pow( fDot, fPow );
	
	return lerp( xInColour, xDestColour, xTmp2 );
}

OutputVertex VShader( const InputVertex In )
{
	OutputVertex Out;
	
	Out.Position = mul( float4( In.Position, 1.0 ), g_xWorldViewProjMatrix );
	Out.UV = In.UV;
	if(FlipTextureVertical) Out.UV.y = 1.f - Out.UV.y;
	
	return Out;
}

DeferredOutputPixel PShader( const OutputVertex In )
{
	float4 xBaseTexture = tex2D( BaseTexture, In.UV );
	half4 xOutputRGBA = lerp( xBaseTexture * SkyboxColour, FogColour, FogBlendFactor);
	float3 xOutputNormal = float3( 0.0, 0.0, -1.0);
	float fOutputDepth = g_xDistance;
	half3 xOutputAlbedo = xBaseTexture;
	half hOutputSpecular = xBaseTexture.a;
	
	// Return the data...
	DeferredOutputPixel xOut = Deferred_FillOutputPixel( xOutputRGBA, xOutputNormal, fOutputDepth, xOutputAlbedo, hOutputSpecular, 0);
	
	return( xOut);
}

OutputVertexT1 VShaderT1( const float4 xIn : POSITION )
{
	OutputVertexT1 xOut;

	xOut.m_xPosition = float4( xIn.xy, 1.0f, 1.0f );						// render at maximum depth value (z/w = 1)
	xOut.m_xCameraDir = -mul( xOut.m_xPosition, g_xFrustumToCamDir );			// not actually world matrix, just reusing the name!

	return xOut;
}

DeferredOutputPixel PShaderT1 ( const OutputVertexT1 xIn )
{
	DeferredOutputPixel xOut;

	float3 xNorm		= xIn.m_xCameraDir / sqrt( dot( xIn.m_xCameraDir, xIn.m_xCameraDir ) );
	
	float3 xHazeColour1, xHazeColour2;
	float3 xSunColour1, xSunColour2;

	if( g_bDisableHaze )
	{
		xHazeColour2 = g_xSkyboxColour;
	}
	else
	{
		if( g_bDisable2Haze )
		{
			xHazeColour2	= CalculateYColour( xNorm, g_xSkyboxColour.rgb, g_xHazeColour1.rgb, g_xHazeGradientPower1, g_bMirrorBelowHorizon );
		}
		else
		{
			xHazeColour1	= CalculateYColour( xNorm, g_xSkyboxColour.rgb, g_xHazeColour1.rgb, g_xHazeGradientPower1, g_bMirrorBelowHorizon );
			xHazeColour2	= CalculateYColour( xNorm, xHazeColour1, g_xHazeColour2.rgb, g_xHazeGradientPower2, g_b2MirrorBelowHorizon );
		}
	}
	
	if( g_bDisableSun )
	{
		xSunColour2	= xHazeColour2;
	}
	else
	{
		if( g_bDisable2Sun )
		{
			float fDot	= saturate( dot( xNorm, g_xSunDir1 ) * 0.5f + 0.5f );
			xSunColour2	= CalculateSun( fDot, xHazeColour2, g_xSunColour1, g_xSunGradientPower1 );
		}
		else
		{
			float fDot	= saturate( dot( xNorm, g_xSunDir1 ) * 0.5f + 0.5f );
			xSunColour1	= CalculateSun( fDot, xHazeColour2, g_xSunColour1, g_xSunGradientPower1 );
			fDot		= saturate( dot( xNorm, g_xSunDir2 ) * 0.5f + 0.5f );
			xSunColour2	= CalculateSun( fDot, xSunColour1, g_xSunColour2, g_xSunGradientPower2 );
		}
	}
	
	float4 xColour		= float4( xSunColour2, 1.0f );

	xOut = Deferred_FillOutputPixel( xColour,
									 half3( 0.0h, 0.0h, -1.0h ),	//WorldSpaceNormal
									 g_xDistance,					//Distance
									 half3( 0.0h, 0.0h, 0.0h ),		//Albedo
									 0.0h,							// Specular
									 0.0h );						// Material v-coord

	return xOut;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique t0
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}

technique t1
{
	pass
	{
		VertexShader = compile vs_3_0 VShaderT1();
		PixelShader = compile ps_3_0 PShaderT1();
	}
}