//////////////////////////////////////////////////////////////////////////////
//
// Asura_PC_ShadowBuffer.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_TARGET_SM3)

#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_EffectParameterBlock.h"
#include "Rendering/Asura_Direct3D_EnvironmentRenderer.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_UserOptions.h"
#include "Rendering/Asura_DeferredShading.h"


#include "Shadows/Asura_Shadow_System.h"
#include "Shadows/Asura_PC_ShadowBuffer.h"

bool Asura_PC_ShadowBuffer::m_bNvidiaDepthMap = false;


Asura_PC_ShadowBuffer::Asura_PC_ShadowBuffer(u_int uTextureSize, Direct3D_Surface* pxDepthSurface) 
: m_uTextureSize(uTextureSize), m_pxOtherSurface(pxDepthSurface)
{

	D3DFORMAT eFormat;
	u_int uUsage;

	if (IsNvidiaDepthMap())
	{
		eFormat = D3DFMT_D24S8;
		uUsage = D3DUSAGE_DEPTHSTENCIL;
	} 
	else 
	{
		eFormat = D3DFMT_R32F;
		uUsage = D3DUSAGE_RENDERTARGET;
	}

	Asura_Direct3D::D3DDevice->CreateTexture(uTextureSize,uTextureSize, 1, uUsage, eFormat, D3DPOOL_DEFAULT, &m_pxShadowTexture, NULL);
	Asura_Assertf(m_pxShadowTexture, ("Unable to create shadow texture"));

	if (m_pxShadowTexture)
	{
		m_pxShadowTexture->GetSurfaceLevel(0, &m_pxShadowTextureSurface);
		Asura_Assert(m_pxShadowTextureSurface, "Unable to get surface of shadow texture");
	} 
	else 
	{
		m_pxShadowTextureSurface = NULL;
	}

	if (m_pxOtherSurface)
	{
		m_pxOtherSurface->AddRef();
	}
}


Asura_PC_ShadowBuffer::~Asura_PC_ShadowBuffer()
{
	SAFE_RELEASE(m_pxOtherSurface);
	SAFE_RELEASE(m_pxShadowTextureSurface);
	SAFE_RELEASE(m_pxShadowTexture);
}

void Asura_PC_ShadowBuffer::Bind() const
{
	Direct3D_Surface* pxFrontSurface = GetFrontSurface();
	Direct3D_Surface* pxDepthSurface = GetDepthSurface();

	if (pxFrontSurface)
	{
		Asura_Direct3D::D3DDevice->SetRenderTarget(0, pxFrontSurface);
	}
	if (pxDepthSurface)
	{
		Asura_Direct3D::D3DDevice->SetDepthStencilSurface(pxDepthSurface);
	}
	Asura_Direct3D::D3DDevice->SetRenderTarget(1, NULL);
	Asura_Direct3D::D3DDevice->SetRenderTarget(2, NULL);
	Asura_Direct3D::D3DDevice->SetRenderTarget(3, NULL);
}

#endif