//////////////////////////////////////////////////////////////////////////////
//
// Asura_PC_ReplaceableSounds.cpp
//
// Created by AAM - 07/01/2004 12:41:17
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if !defined ASURA_USE_AUDIO_SYSTEM

#include "Sound/Asura_PC_ReplaceableSounds.h"

#include "Core/Asura_PlatformBasics.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Sound/Asura_PC_Sound.h"
#include "Sound/Asura_ProjectSoundIDs.h"
#include "Sound/Asura_Sound.h"

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

/******************************************************************************/
/* Static Class Data                                                          */
/******************************************************************************/

Asura_PC_ReplaceableSoundData* Asura_PC_ReplaceableSounds::s_pxReplaceableSoundData = NULL;
char Asura_PC_ReplaceableSounds::s_szReplaceableSoundFile[MAX_PATH] = "";

/******************************************************************************/
/* Class Methods                                                              */
/******************************************************************************/

Asura_PC_ReplaceableSoundData::Asura_PC_ReplaceableSoundData() :
	m_uByteOffset( ASURA_PC_SOUND_INVALID_BYTE_OFFSET ),
	m_uFileSize( 0 ),
	m_bInMemory( false )
	#ifdef ASURA_TOOLS
	, m_iSoundID( ASURA_SID_NO_SOUND )
	#endif
{

}

// Initialise the replaceable sounds system
// returns true for success, false for failure
bool Asura_PC_ReplaceableSounds::Initialise( u_int uiNumEntries, const char* szFilename )
{
	Asura_Assert( strlen( s_szReplaceableSoundFile ) == 0, "Replaceable sounds loaded twice?");

	Asura_PrintfToBfr( s_szReplaceableSoundFile, "%s", MAX_PATH - 1, true, szFilename );

	// TBD couldn't we have this as an array? Would simplify things.
	s_pxReplaceableSoundData = new Asura_PC_ReplaceableSoundData[ASURA_MAX_NUMBER_OF_LOADED_SOUNDS];

	return true;
}

// shut down the replaceable sounds system
void Asura_PC_ReplaceableSounds::Shutdown()
{
	// moved here from Asura_PCSound_System::Shutdown()
	delete[] s_pxReplaceableSoundData;
	s_pxReplaceableSoundData = NULL;
}

// reset the replaceable sounds system
void Asura_PC_ReplaceableSounds::Reset()
{
	// moved here from Asura_PCSound_System::Reset()
	// Unload any replaceable sounds which were loaded into memory

	if ( s_pxReplaceableSoundData )
	{
		for ( u_int u = 0; u < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS; ++u )
		{
			if ( s_pxReplaceableSoundData[u].m_bInMemory )
			{
				Asura_PC_Sound::UnloadSound( u );
				s_pxReplaceableSoundData[u].m_bInMemory = false;
			}
		}
	}
}

// load a replaceable sound
// returns true for success, false for failure
bool Asura_PC_ReplaceableSounds::LoadReplaceableSound( int iLoadedSoundID )
{
	// moved here from Asura_PCSound_System::LoadReplaceableSound()

	if ( !s_pxReplaceableSoundData )
	{
		Asura_Assert( s_pxReplaceableSoundData, "Replaceable sounds are not initialised." );
		return false;
	}

	// Asura_Assert( !s_pxReplaceableSoundData[iLoadedSoundIndex].m_bInMemory, "sound already in memory" );
	// If already loaded, just say the load attempt was succesful. RAR 16/11/04
	if ( s_pxReplaceableSoundData[iLoadedSoundID].m_bInMemory )
	{
		return true;
	}

	u_int uByteOffset = s_pxReplaceableSoundData[iLoadedSoundID].m_uByteOffset;
	u_int uFileSize = s_pxReplaceableSoundData[iLoadedSoundID].m_uFileSize;
	Asura_Assert( uByteOffset != ASURA_PC_SOUND_INVALID_BYTE_OFFSET, "Invalid replaceable sound." );

	if ( uByteOffset == ASURA_PC_SOUND_INVALID_BYTE_OFFSET )
	{
		return false;
	}

	Asura_Assert( uFileSize != 0, "Invalid replaceable sound size." );

	// (MH - 15.04.2003) maybe we should just keep this file open...
	Asura_Handle iFileHandle = Asura_FileAccess::Open( s_szReplaceableSoundFile );
	Asura_Assert( iFileHandle != ASURA_INVALID_HANDLE, "Failed to open file for replaceable sound." );

	if ( iFileHandle == ASURA_INVALID_HANDLE )
	{
		return false;
	}

	// Skip past the data we don't want
	Asura_FileAccess::SkipData( iFileHandle, uByteOffset );
	bool bOK = Asura_PC_Sound::LoadSoundFromResource( iFileHandle, uFileSize, iLoadedSoundID );

	if ( bOK )
	{
		s_pxReplaceableSoundData[iLoadedSoundID].m_bInMemory = true;
	}

	Asura_FileAccess::Close( iFileHandle );

	return bOK;
}

// replace a sound
// returns true for success, false for failure
bool Asura_PC_ReplaceableSounds::ReplaceSound( int iExistingSoundLoadedID, int iNewSoundLoadedID )
{
	// moved here from Asura_PCSound_System::ReplaceSound()

	if ( !s_pxReplaceableSoundData )
	{
		Asura_Assert( s_pxReplaceableSoundData, "Replaceable sounds are not initialised." );
		return false;
	}

	Asura_Assert( s_pxReplaceableSoundData[iExistingSoundLoadedID].m_bInMemory, "Replaced sound not in memory." );
	Asura_Assert( !s_pxReplaceableSoundData[iNewSoundLoadedID].m_bInMemory, "Sound to replace with is already in memory." );

	u_int uByteOffset = s_pxReplaceableSoundData[iNewSoundLoadedID].m_uByteOffset;

	#ifdef ASURA_ASSERT
		u_int uFileSize = 
	#endif
	s_pxReplaceableSoundData[iNewSoundLoadedID].m_uFileSize;
	Asura_Assert( uByteOffset != ASURA_PC_SOUND_INVALID_BYTE_OFFSET, "Invalid replaceable sound." );

	if ( uByteOffset == ASURA_PC_SOUND_INVALID_BYTE_OFFSET )
	{
		return false;
	}

	Asura_Assert(uFileSize != 0, "Invalid replaceable sound size.");

	bool bOK = Asura_PC_Sound::UnloadSound( iExistingSoundLoadedID );

	if ( bOK )
	{
		s_pxReplaceableSoundData[iExistingSoundLoadedID].m_bInMemory = false;
		bOK = LoadReplaceableSound( iNewSoundLoadedID );
	}

	return bOK;
}

// Add info about a replaceable sound to the list
// returns true for success, false for failure
bool Asura_PC_ReplaceableSounds::AddReplaceableSoundInfo( int iSoundId, u_int uiByteOffset, u_int uiSize )
{
	if ( !uiSize )
	{
		// error when sound was added to resource file
		return false;
	}

	int iLoadedIndex = Asura_Sound_System::SoundLoadedByPlatformCode( static_cast<Asura_Sound_ID>( iSoundId ) );

	if ( iLoadedIndex < 0 )
	{
		// sound already loaded
		return false;
	}

	s_pxReplaceableSoundData[iLoadedIndex].m_uByteOffset = uiByteOffset;
	s_pxReplaceableSoundData[iLoadedIndex].m_uFileSize = uiSize;
	#ifdef ASURA_TOOLS
		s_pxReplaceableSoundData[iLoadedIndex].m_iSoundID = iSoundId;
	#endif

	return true;
}

#ifdef ASURA_TOOLS
void Asura_PC_ReplaceableSounds::AddReplaceableSoundName( u_int uByteOffset, const char* pszFilename )
{
	if ( s_pxReplaceableSoundData )
	{
		// Only have the file position available here, so there isn't any convenient way of getting
		// the sound index, other than by searching through the list. However once the first is found
		// any futher ones should (hopefully) come in ascending order. RAR 08/12/2004

		static int s_iLastLoadedIndex = -1;
		int iLoadedIndex = s_iLastLoadedIndex + 1;

		for ( int i = 0; i < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS; ++i, ++iLoadedIndex )
		{
			if ( iLoadedIndex >= ASURA_MAX_NUMBER_OF_LOADED_SOUNDS )
			{
				iLoadedIndex = 0;
			}

			if ( uByteOffset == s_pxReplaceableSoundData[iLoadedIndex].m_uByteOffset )
			{
				// Sound name belongs to this sound
				s_iLastLoadedIndex = iLoadedIndex;
				Asura_Sound_System::SetSoundFilename( iLoadedIndex,
					static_cast<Asura_Sound_ID>( s_pxReplaceableSoundData[iLoadedIndex].m_iSoundID ), pszFilename );

				return;
			}
		}
	}
}
#endif //ASURA_TOOLS

#endif // !defined ASURA_USE_AUDIO_SYSTEM
