/**
*
* \file Asura_PlatformSound.cpp
*
* Created by Kevin Lea - 25/04/2002 13:04:50
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#if !defined(ASURA_USE_AUDIO_SYSTEM)

#include "Debugging/Asura_Logfile.h"

#include "Sound/Asura_PC_Sound.h"
#include "Sound/Asura_PC_ReplaceableSounds.h"
#include "Sound/Asura_PlatformStreamSound.h"

// T Y P E S /////////////////////////////////////////////////////////////////

// G L O B A L S /////////////////////////////////////////////////////////////

// (MH - 07.04.2003) fairly arbitrary limit given that at the moment
// these sounds are all resident. When they are eventually implemented 
// as dynamically loaded, typically each active one will be replaced 
// when it is finished and this will involve harddisk usage. To limit 
// this, use this number.
static const int ASURA_PC_SOUND_MAX_ACTIVE_REPLACEABLE_SOUNDS = 4;

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

int Asura_Sound_System::PlatformInitialise()
{
	s_bUseHardware3dPan = true;

	Asura_PlatformStreamSound::Initialise();
	
	if ( !Asura_PC_Sound::Initialise() )
	{
		// KJL - 18/09/2003 12:04:03 - if we can't initialise, try to exit gracefully
		Asura_PC_Sound::Project_HandleInitialisationFailure();
		
	}

    return ASURA_PC_SOUND_MAX_NUMBER_OF_ACTIVE_SOUNDS;
}

void Asura_Sound_System::PlatformShutdown()
{
	Asura_PC_Sound::Shutdown();
	Asura_PlatformStreamSound::Shutdown();
	Asura_PC_ReplaceableSounds::Shutdown();
}

// (MH - 03.07.2002) may want to check that sound memory available at reset point
// is always the same.
void Asura_Sound_System::PlatformReset()
{
	Asura_PC_Sound::Reset();
	Asura_PC_ReplaceableSounds::Reset();
}

void Asura_Sound_System::PlatformSetResetPoint()
{

}

bool Asura_Sound_System::PlatformLoadSound( const char* szFilename, int iLoadedSoundID )
{
	Asura_Assert( iLoadedSoundID >= 0 && iLoadedSoundID < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid loaded sound index" );
	return Asura_PC_Sound::LoadSound( szFilename, iLoadedSoundID );
}

#ifdef ASURA_TOOLS
void Asura_Sound_System::PlatformForceSoundToBeLoaded (int iLoadedSoundID )
{
	Asura_Assert( iLoadedSoundID >= 0 && iLoadedSoundID < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid loaded sound index" );
    return Asura_PC_Sound::ForceSoundToBeLoaded( iLoadedSoundID );
}

void Asura_Sound_System::PlatformReloadSound( int iLoadedSoundID )
{
	Asura_Assert( iLoadedSoundID >= 0 && iLoadedSoundID < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid loaded sound index" );
    return Asura_PC_Sound::ReloadSound( iLoadedSoundID);
}
#endif //ASURA_TOOLS


bool Asura_Sound_System::PlatformLoadSoundFromResource( Asura_Handle iFileHandle, int iFileSize, int iLoadedSoundID )
{
	Asura_Assert( iLoadedSoundID >= 0 && iLoadedSoundID < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid loaded sound index" );
    return Asura_PC_Sound::LoadSoundFromResource( iFileHandle, iFileSize, iLoadedSoundID );
}

bool Asura_Sound_System::PlatformLoadReplaceableSound( int iLoadedSoundID )
{
	Asura_Assert( iLoadedSoundID >= 0 && iLoadedSoundID < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid loaded sound index" );
    return Asura_PC_ReplaceableSounds::LoadReplaceableSound( iLoadedSoundID );
}

bool Asura_Sound_System::PlatformReplaceSound( int iExistingSoundLoadedID, int iNewSoundLoadedID )
{
	Asura_Assert( iExistingSoundLoadedID >= 0 && iExistingSoundLoadedID < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid loaded sound index" );
	Asura_Assert( iNewSoundLoadedID >= 0 && iNewSoundLoadedID < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid loaded sound index" );

    return Asura_PC_ReplaceableSounds::ReplaceSound( iExistingSoundLoadedID, iNewSoundLoadedID );
}

int Asura_Sound_System::Platform_GetMaxActiveReplaceableSounds()
{
	return ASURA_PC_SOUND_MAX_ACTIVE_REPLACEABLE_SOUNDS;
}

bool Asura_Sound_System::PlatformUnloadSound( int iLoadedSoundID )
{
	Asura_Assert( iLoadedSoundID >= 0 && iLoadedSoundID < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid loaded sound index" );
    return Asura_PC_Sound::UnloadSound( iLoadedSoundID );
}

bool Asura_Sound_System::PlatformCreateSound( int iLoadedSoundID, Asura_Handle iSoundHandle, bool bUseEQ, bool bUseHardware3dPan )
{
	Asura_Assert( iLoadedSoundID >= 0 && iLoadedSoundID < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid loaded sound index" );
	return Asura_PC_Sound::CreateSound( iLoadedSoundID, iSoundHandle, bUseHardware3dPan );
}

bool Asura_Sound_System::PlatformPlaySound( Asura_Handle iSoundHandle, bool bLooping )
{
	Asura_Assert( IsValidPlayingSoundHandle( iSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::PlaySound( iSoundHandle, bLooping );
}

bool Asura_Sound_System::PlatformStopSound( Asura_Handle iSoundHandle )
{
	Asura_Assert( IsValidPlayingSoundHandle( iSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::StopSound( iSoundHandle );
}

bool Asura_Sound_System::PlatformDestroySound(Asura_Handle iSoundHandle)
{
	Asura_Assert( IsValidPlayingSoundHandle( iSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::DestroySound( iSoundHandle );
}

void Asura_Sound_System::PlatformUpdate()
{
	Asura_Assert( s_iNumSplitScreens == 1, "split-screen not allowed on PC" );
		
	Asura_PC_Sound::SetListener( s_axSoundCentre[0], s_axSoundOrientation[0] );

	Asura_PC_Sound::Update();

	TraversePlayingSoundHandles( Asura_PC_Sound::UpdateSound );
	
	if ( Asura_PC_Sound::HasReverbChanged() )
	{
		TraversePlayingSoundHandles( Asura_PC_Sound::UpdateReverb );
		Asura_PC_Sound::SetReverbChanged( false );
	}

#if _DEBUG
	TraversePlayingSoundHandles( Asura_PC_Sound::CheckSound );
#endif

    Asura_PlatformStreamSound::Update();
}

bool Asura_Sound_System::PlatformSoundIsPlaying( Asura_Handle iPlayingSoundHandle )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SoundIsPlaying( iPlayingSoundHandle );
}

bool Asura_Sound_System::PlatformSoundIsPreparing( Asura_Handle iPlayingSoundHandle )
{
	//No on-demand sounds on PC
	return false;
}

bool Asura_Sound_System::PlatformSetSoundVolumePanAndFade( Asura_Handle iPlayingSoundHandle, float fVolume, float fPan, float fFade )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );

    bool bOK = Asura_PC_Sound::SetVolume( iPlayingSoundHandle, fVolume );

	if ( bOK )
	{
		bOK = Asura_PC_Sound::SetPan( iPlayingSoundHandle, fPan );
	}

    return bOK;
}

bool Asura_Sound_System::Platform_SetSoundWetDryVolume (Asura_Handle iPlayingSoundHandle, float fWetVolume, float fDryVolume)
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SetWetDryVolume( iPlayingSoundHandle, fWetVolume, fDryVolume );
}

bool Asura_Sound_System::Platform_HardwarePanSetSoundVolume( Asura_Handle iPlayingSoundHandle, float fVolume )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
    return Asura_PC_Sound::SetVolume( iPlayingSoundHandle, fVolume );
}

bool Asura_Sound_System::Platform_HardwarePanSetSoundPosition( Asura_Handle iPlayingSoundHandle, const Asura_Vector_3& xPosition )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SetSoundPosition( iPlayingSoundHandle, xPosition );
}

bool Asura_Sound_System::Platform_HardwarePanSetSoundSpatial( Asura_Handle iPlayingSoundHandle, bool bIsSpatial )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SetSpatial( iPlayingSoundHandle, bIsSpatial );
}

bool Asura_Sound_System::Platform_HardwarePanSetSoundRadii( Asura_Handle iPlayingSoundHandle, float fInner, float fOuter )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SetSoundRadii( iPlayingSoundHandle, fInner, fOuter );
}

bool Asura_Sound_System::Platform_HardwarePanSetSoundForcedPanAndFade( Asura_Handle iPlayingSoundHandle, float fPan, float fFade )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SetSoundForcedPanAndFade( iPlayingSoundHandle, fPan, fFade );
}

bool Asura_Sound_System::PlatformSetSoundPitch( Asura_Handle iPlayingSoundHandle, float fPitch )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SetPitch( iPlayingSoundHandle, fPitch );
}

bool Asura_Sound_System::Platform_SetSoundUsesReverb( Asura_Handle iPlayingSoundHandle, bool bUseReverb )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SetUseReverb( iPlayingSoundHandle, bUseReverb );
}

bool Asura_Sound_System::Platform_SetSoundUsesDistanceFilter( Asura_Handle iPlayingSoundHandle, bool bUseDistanceFilter )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SetUseDistanceFilter( iPlayingSoundHandle, bUseDistanceFilter );
}

bool Asura_Sound_System::Platform_SetOcclusionLPF(const Asura_Handle iPlayingSoundHandle, const float fOcclusion)
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SetOcclusionLPF( iPlayingSoundHandle, fOcclusion );
}

bool Asura_Sound_System::PlatformPauseSound( Asura_Handle iPlayingSoundHandle )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::PauseSound( iPlayingSoundHandle );
}

bool Asura_Sound_System::PlatformUnpauseSound( Asura_Handle iPlayingSoundHandle )
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::UnpauseSound( iPlayingSoundHandle );
}

#ifdef ASURA_TOOLS
float Asura_Sound_System::PlatformGetSoundLength( int iLoadedSoundID )
{
	Asura_Assert( iLoadedSoundID >= 0 && iLoadedSoundID < ASURA_MAX_NUMBER_OF_LOADED_SOUNDS, "invalid loaded sound index" );

    return Asura_PC_Sound::GetSoundLength( iLoadedSoundID );
}
#endif //ASURA_TOOLS

// S T R E A M I N G   S O U N D S ///////////////////////////////////////////

bool Asura_Sound_System::Platform_CreateStreamingSound( Asura_Handle iHandle, const char* pszFilename)
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return false;
	}
	
    return Asura_PlatformStreamSound::Create( iHandle, pszFilename);
}


void Asura_Sound_System::Platform_PlayStreamingSound( Asura_Handle iHandle, bool bLooping, bool bPreload )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return;
	}
	
    Asura_PlatformStreamSound::Play( iHandle, bLooping );
}

void Asura_Sound_System::Platform_StopStreamingSound( Asura_Handle iHandle )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return;
	}
	
    Asura_PlatformStreamSound::Stop( iHandle );
}

void Asura_Sound_System::Platform_PauseStreamingSound( Asura_Handle iHandle )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return;
	}
    
    Asura_PlatformStreamSound::Pause( iHandle );
}

void Asura_Sound_System::Platform_ResumeStreamingSound( Asura_Handle iHandle )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return;
	}
    
    Asura_PlatformStreamSound::Resume( iHandle );
}

void Asura_Sound_System::Platform_ReleaseStreamingSound( Asura_Handle iHandle )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return;
	}
	
    Asura_PlatformStreamSound::Release( iHandle );
}

bool Asura_Sound_System::Platform_StreamingSoundIsReady( Asura_Handle iHandle )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return false;
	}

    // (MH - 30.11.2002) todo
    return true;
}


void Asura_Sound_System::Platform_StartPreloadedStreamingSound( Asura_Handle iHandle )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return;
	}
}

void Asura_Sound_System::Platform_SetStreamingSoundOutput( Asura_Handle iHandle, SoundOutputType eOutputType )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return;
	}

	// (MH - 12.12.2002) todo
}

bool Asura_Sound_System::Platform_StreamingSoundIsFinished( Asura_Handle iHandle )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return false;
	}

	return Asura_PlatformStreamSound::IsFinished( iHandle );
}

float Asura_Sound_System::Platform_GetStreamingSoundPosition( Asura_Handle iHandle )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return false;
	}

	return Asura_PlatformStreamSound::GetPosition( iHandle );
}

float Asura_Sound_System::Platform_GetStreamingSoundLength( Asura_Handle iHandle )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return false;
	}

	return Asura_PlatformStreamSound::GetLength( iHandle );
}

#ifdef ASURA_TOOLS
void Asura_Sound_System::Platform_SetStreamingSoundPosition( Asura_Handle iHandle, float fTime )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return;
	}

	Asura_PlatformStreamSound::SetPosition( iHandle, fTime );

}
#endif //ASURA_TOOLS

void Asura_Sound_System::Platform_SetReverb( enum ReverbType eType, float fReverbAmount )
{
    Asura_PC_Sound::SetReverb( eType, fReverbAmount );
}

int Asura_Sound_System::Platform_GetMaxStreamingSounds()
{
    return ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS;
}

void Asura_Sound_System::Platform_EnsureAllStreamingSoundsAreStopped()
{
	// todo
}

void Asura_Sound_System::Platform_SetStreamingSoundVolume( Asura_Handle iHandle, float fVolume )
{
    Asura_Assert( iHandle >= 0 && iHandle < ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS, "invalid handle" );
    if ( iHandle < 0 || iHandle >= ASURA_PC_MAX_CONCURRENT_SOUNDSTREAMS )
	{
		return;
	}

    Asura_Assert( fVolume >= 0.0f && fVolume <= 1.0f, "invalid volume" );
    
    Asura_PlatformStreamSound::SetVolume( iHandle, fVolume );
}

int Asura_Sound_System::Platform_GetReplaceableSoundSize( int iLoadedSoundID )
{
    return 0;
}

int Asura_Sound_System::Platform_GetMaxSoundsWithEQ()
{
	// return number of voices which may have EQ applied. Generally this 
	// will be the same return value as PlatformInitialise() but on PS2 only
	// half the voices can have EQ.
	return ASURA_PC_SOUND_MAX_NUMBER_OF_ACTIVE_SOUNDS;
}

void Asura_Sound_System::Platform_SetEQType( Asura_Handle iHandle, EQType eType, float fLevel )
{

}

bool Asura_Sound_System::Platform_SetSoundDistanceFilterGain (Asura_Handle iPlayingSoundHandle, float fNormalisedGain)
{
	Asura_Assert( IsValidPlayingSoundHandle( iPlayingSoundHandle ), "invalid playing sound handle" );
	return Asura_PC_Sound::SetDistanceFilterGain( iPlayingSoundHandle, fNormalisedGain );
}

#endif	//!defined(ASURA_USE_AUDIO_SYSTEM)
