//
// Asura_PlatformSkybox.cpp
//
// Created by Kevin Lea - 02/10/2001 12:51:00
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "SpecialFX/Asura_Skybox.h"

#include "Core/Asura_TextureManagement.h"
#include "Rendering/Asura_Direct3D.h"
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Render.h"

#include "Rendering/Asura_Direct3D_Shaders.h"

#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Fog.h"

#if defined(ASURA_TARGET_SM3)

#include "Rendering/Asura_Direct3D_Skybox.h"

void Asura_Skybox::Platform_DoRenderSetup()
{
	Asura_Direct3D_Skybox::Reset();
}

void Asura_Skybox::Platform_Initialise()
{
	Asura_Direct3D_Skybox::Initialise();
}

void Asura_Skybox::Platform_Render()
{
	Asura_Direct3D_Skybox::Render();
}

void Asura_Skybox::Platform_Reset()
{
	Asura_Direct3D_Skybox::Reset();
}

bool Asura_Skybox::Platform_ShouldModifyFogForSkyboxObjects()
{
	return false;
}

void Asura_Skybox::Platform_Shutdown()
{
	Asura_Direct3D_Skybox::Shutdown();
}

#else

static int g_aiSkyBoxTextureIndex[6] = { ASURA_TEXTURE_NONE };
static int g_iFractalTexture = ASURA_TEXTURE_NONE;
static int g_iFractalTexture2 = ASURA_TEXTURE_NONE;

static Asura_Vector_3 g_axPositions[] = 
{
	Asura_Vector_3(-1.f,+1.f,-1.f), // 0
	Asura_Vector_3(-1.f,+1.f,+1.f), // 1
	Asura_Vector_3(+1.f,+1.f,+1.f), // 2
	Asura_Vector_3(+1.f,+1.f,-1.f), // 3
	Asura_Vector_3(-1.f,-1.f,-1.f), // 4
	Asura_Vector_3(-1.f,-1.f,+1.f), // 5
	Asura_Vector_3(+1.f,-1.f,+1.f), // 6
	Asura_Vector_3(+1.f,-1.f,-1.f), // 7
	Asura_Vector_3(+1.f, 0.f,+1.f), // 8
	Asura_Vector_3(+1.f, 0.f,-1.f), // 9
	Asura_Vector_3(-1.f, 0.f,-1.f), //10
	Asura_Vector_3(-1.f, 0.f,+1.f), //11
};

static u_int g_aauBasicFaces[6][4] =
{
	{1,0,3,2}, // ASURA_SKYBOX_DN
	{0,4,7,3}, // ASURA_SKYBOX_FR
	{2,6,5,1}, // ASURA_SKYBOX_BK
	{3,7,6,2}, // ASURA_SKYBOX_LF
	{1,5,4,0}, // ASURA_SKYBOX_RT
	{4,5,6,7}, // ASURA_SKYBOX_UP
};

static u_int g_aauMirroredFaces[6][6] =
{
	{ 0, 1, 2, 3, 0, 0}, // ASURA_SKYBOX_DN (note that this is flipped from the other version)
	{10, 4, 7, 9, 3, 0}, // ASURA_SKYBOX_FR
	{ 8, 6, 5,11, 1, 2}, // ASURA_SKYBOX_BK
	{ 9, 7, 6, 8, 2, 3}, // ASURA_SKYBOX_LF
	{11, 5, 4,10, 0, 1}, // ASURA_SKYBOX_RT
	{ 4, 5, 6, 7, 0, 0}, // ASURA_SKYBOX_UP
};

#define D3DFVF_SKYBOXVERTEX (D3DFVF_XYZ|D3DFVF_SPECULAR|D3DFVF_DIFFUSE|D3DFVF_TEX1)
struct D3DSkyboxVertex
{
	float m_fX, m_fY, m_fZ;
	u_int m_uDiffuse;
	u_int m_uSpecular;
	float m_fU;
	float m_fV;
};

static Direct3D_VertexBuffer* g_apxVertexBuffers[6] = { NULL };
static u_int g_auVertexCounts[6] = { 0 };

void Asura_Skybox::Platform_DoRenderSetup(void)
{
	{
		for (int i=0; i<6; i++)
		{
			g_aiSkyBoxTextureIndex[i] = Asura_Texture_Management::AddTexture(GetTexture(i));
		}
		// AJG - 19/02/2004 - I don't really like this, these could be in the array above if we extend it,
		// but since the original fractal texture was kept separate, for clarity I guess, I'll do the same for the second
		g_iFractalTexture = Asura_Texture_Management::AddTexture(GetTexture(ASURA_SKYBOX_FRACTALSKY));
		g_iFractalTexture2 = Asura_Texture_Management::AddTexture(GetTexture(ASURA_SKYBOX_FRACTALSKY_2));
		Asura_Texture_Management::LoadTextures();

		// (MH - 05.09.2003) cleanup any failed texture loads, so LoadTextures doesn't
		// keep on trying to find them.
		// RAR 13/06/2006 - Disabled cleanup in tools , because it can cause problems if other
		//things are using the same texture ,and need to be able to get hold of the name.
		for (int i=0; i<6; i++)
		{
			if (!Asura_Texture_Management::IsTextureLoaded(g_aiSkyBoxTextureIndex[i]))
			{
#ifndef ASURA_TOOLS
				Asura_Texture_Management::DestroyTexture(g_aiSkyBoxTextureIndex[i]);
#endif
				g_aiSkyBoxTextureIndex[i] = ASURA_TEXTURE_NONE;
			}
		}
		if (!Asura_Texture_Management::IsTextureLoaded(g_iFractalTexture))
		{
#ifndef ASURA_TOOLS
			Asura_Texture_Management::DestroyTexture(g_iFractalTexture);
#endif
			g_iFractalTexture = ASURA_TEXTURE_NONE;
		}
		if (!Asura_Texture_Management::IsTextureLoaded(g_iFractalTexture2))
		{
#ifndef ASURA_TOOLS
			Asura_Texture_Management::DestroyTexture(g_iFractalTexture2);
#endif
			g_iFractalTexture2 = ASURA_TEXTURE_NONE;
		}
	}
	// Create vertex buffers.
	const u_int uNumBasicVertices = 4;
	const u_int uNumMirroredVertices = 6;

	const float fFogScale = Asura_Fog::IsFogRenderable() ? Asura_Fog::GetSkyboxFogBlendFactor() : 0.0f;
	const float fSkyboxScale = 1.0f - fFogScale;
	const u_int uColour = 0xff000000
		+ (Asura_Maths::FloatToInt(fSkyboxScale * s_fRed)<<16)
		+ (Asura_Maths::FloatToInt(fSkyboxScale * s_fGreen)<<8)
		+ (Asura_Maths::FloatToInt(fSkyboxScale * s_fBlue));
	const Asura_RGBA xFogColour = Asura_Fog::GetFogColour() * fFogScale;
	const u_int uSpecular = xFogColour.ToUIntARGB();

	for(u_int uFace = 0; uFace < 6; ++uFace)
	{
		const bool bUseMirroredVertices = s_bMirrorBelowHorizon ? (uFace != ASURA_SKYBOX_DN && uFace != ASURA_SKYBOX_UP) : false;
		const u_int uNumVertices = g_auVertexCounts[uFace] = bUseMirroredVertices ? uNumMirroredVertices : uNumBasicVertices;
		if(g_apxVertexBuffers[uFace])
		{
			g_apxVertexBuffers[uFace]->Release();
		}
		Asura_Direct3D::D3DDevice->CreateVertexBuffer
		(
			uNumVertices * sizeof(D3DSkyboxVertex),
			0, //D3DUSAGE_WRITEONLY,
			D3DFVF_SKYBOXVERTEX,
			D3DPOOL_MANAGED,
			&g_apxVertexBuffers[uFace],
			0
		);
		D3DSkyboxVertex* pxVertexdata = NULL;
		g_apxVertexBuffers[uFace]->Lock(0,sizeof(D3DSkyboxVertex) * uNumVertices,(void**)&pxVertexdata, 0);

		const bool bFlipped = (uFace == ASURA_SKYBOX_BK && GetBackTextureIsFrontUpsideDown()) ||
							  (uFace == ASURA_SKYBOX_RT && GetRightTextureIsLeftUpsideDown());

		for(u_int u = 0; u < uNumVertices; ++u)
		{
			const int iIndex = s_bMirrorBelowHorizon ? g_aauMirroredFaces[uFace][u] : g_aauBasicFaces[uFace][u];
			const Asura_Vector_3& xPos = g_axPositions[iIndex];
			pxVertexdata[u].m_fX = xPos.x;
			pxVertexdata[u].m_fY = (bFlipped && !s_bMirrorBelowHorizon) ? -xPos.y : xPos.y;
			pxVertexdata[u].m_fZ = xPos.z;
			pxVertexdata[u].m_uDiffuse = uColour;
			pxVertexdata[u].m_uSpecular = uSpecular;
		}

		if(bUseMirroredVertices)
		{
			const float fV = bFlipped ? 1.f : 0.f;
			pxVertexdata[0].m_fU = 1.f; pxVertexdata[0].m_fV = .5f;
			pxVertexdata[1].m_fU = 1.f; pxVertexdata[1].m_fV = fV;
			pxVertexdata[2].m_fU = 0.f; pxVertexdata[2].m_fV = fV;
			pxVertexdata[3].m_fU = 0.f; pxVertexdata[3].m_fV = .5f;
			pxVertexdata[4].m_fU = 0.f; pxVertexdata[4].m_fV = fV;
			pxVertexdata[5].m_fU = 1.f; pxVertexdata[5].m_fV = fV;
		}
		else
		{
			pxVertexdata[0].m_fU = 1.f; pxVertexdata[0].m_fV = 1.f;
			pxVertexdata[1].m_fU = 1.f; pxVertexdata[1].m_fV = 0.f;
			pxVertexdata[2].m_fU = 0.f; pxVertexdata[2].m_fV = 0.f;
			pxVertexdata[3].m_fU = 0.f; pxVertexdata[3].m_fV = 1.f;
		}
		g_apxVertexBuffers[uFace]->Unlock();
	}
}

void Asura_Skybox::Platform_Reset(void)
{
	Platform_Shutdown();
}

void Asura_Skybox::Platform_Initialise(void)
{
}

void Asura_Skybox::Platform_Render(void)
{
	D3DPERFCALL(BeginEvent(0xff889944, L"Skybox"));

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

#ifdef ASURA_USE_SKYSPHERE
	if(s_bDrawClouds)
	{
		// Kevin Lea - 03/10/2001 11:31:22 - ah, clouds. A bit messy at the moment - a lot depends on how much
		// control needs to be given to designers (cloud cover, colours, speed, etc.)
		void RenderSkySphere(void);
		RenderSkySphere();
	}
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
#else
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
#endif
	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_SKYBOXVERTEX);
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP );
	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP );
	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_SPECULARENABLE, TRUE);

	for (int f=0; f<6; f++)
	{
		Asura_Render::CurrentStates.RequestTextureIndex(0,g_aiSkyBoxTextureIndex[f]);
		Asura_Direct3D::D3DDevice->SetStreamSource(0, g_apxVertexBuffers[f], 0, sizeof(D3DSkyboxVertex));
		for(int i=0; i<Asura_Camera::GetNumberOfCameras(); i++)
		{
			Asura_Camera::SetCameraIndex(i);

			if (Asura_Camera::IsCameraActive())
			{
				Asura_Render::ApplyCameraSettings();

				const Asura_Vector_3* pxCamPos = Asura_Camera::GetPosition();
				D3DXMATRIX xTranslation, xRotation;
				D3DXMatrixTranslation( &xTranslation, pxCamPos->x, pxCamPos->y, pxCamPos->z );
				D3DXMatrixRotationY(&xRotation, s_fOrientationAroundYAxis);
				xRotation *= xTranslation;
				Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD, &xRotation );
				Asura_Direct3D::D3DDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, g_auVertexCounts[f] - 2);
			}
		}
	}
	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
	Asura_Direct3D::D3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );

	D3DXMATRIX matWorld;
	D3DXMatrixTranslation( &matWorld, 0, 0, 0 );
	Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );

	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);


	Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_SPECULARENABLE, FALSE);

	D3DPERFCALL(EndEvent());
}

void Asura_Skybox::Platform_Shutdown(void)
{
	for(u_int uFace = 0; uFace < 6; ++uFace)
	{
		if(g_apxVertexBuffers[uFace])
		{
			g_apxVertexBuffers[uFace]->Release();
			g_apxVertexBuffers[uFace] = NULL;
		}
	}
}

bool Asura_Skybox::Platform_ShouldModifyFogForSkyboxObjects( void )
{
	return true;
}

#ifdef ASURA_USE_SKYSPHERE

// Kevin Lea - 05/07/2001 10:15:39 - sky
#include "SpecialFX/Asura_Sphere.h"
#include "Core/Asura_Timers.h"
extern TRI_FACE SphereFace[SPHERE_FACES];
Asura_Vector_3 SphereRotatedVertex[SPHERE_VERTICES];

struct SkyVertexFormat
{

	float X;
	float Y;
	float Z;

	union
	{
		unsigned int Colour;
		struct Colour_Components ColourComponents;
	};

	float U0;
	float V0;
	float U1;
	float V1;
};
#define SKY_VERTEX_DESCRIPTION (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX2)

SkyVertexFormat SkyVertices[3];

void FixVertexWrap(float& v0, float& v1, float& v2)
{
	float d1 = v0-v1;
	float d2 = v0-v2;
	float d3 = v1-v2;

	float ad1=d1,ad2=d2,ad3=d3;
	int i1=0,i2=0,i3=0;

	if (ad1<0) ad1=-ad1;
	if (ad2<0) ad2=-ad2;
	if (ad3<0) ad3=-ad3;

	if (ad1>=(SPHERE_TEXTURE_WRAP/1.5f))
	{
		if (d1>0) i2=1;
		else i1=1;
	}
	if (ad2>=(SPHERE_TEXTURE_WRAP/1.5f))
	{
		if (d2>0) i3=1;
		else i1=1; 
	}
	if (ad3>=(SPHERE_TEXTURE_WRAP/1.5f))
	{
		if (d3>0) i3=1; 
		else i2=1; 
	}

	if(i1) v0+=SPHERE_TEXTURE_WRAP;
	if(i2) v1+=SPHERE_TEXTURE_WRAP;
	if(i3) v2+=SPHERE_TEXTURE_WRAP;

}




extern Direct3D_VertexBuffer* RenderMeshVertexBuffer;
extern Direct3D_VertexBuffer* RenderMeshVertexBuffer2;
extern Direct3D_IndexBuffer* RenderMeshIndexBuffer;
extern void InitialiseRMVB(void);

void RenderSkySphere(void)
{
	
	InitialiseRMVB();

	float alpha = Asura_Maths::Sine( Asura_Timers::GetLocalScaledTime() / 16.0f );

	alpha = 255.0f*alpha*alpha;

	unsigned colour = (((unsigned int)alpha)<<24)+0xffffff;

	
	{
	   	int f;
		Asura_Vector_3 *vSphere = SphereRotatedVertex;

		for (f=0; f<SPHERE_VERTICES; f++)
		{	
			vSphere->x = SphereVertex[f].x*4;
			vSphere->z = SphereVertex[f].y*4;
			vSphere->y = -SphereVertex[f].z+0.75f;//+0.75f;
			int a = 0;//GetSin((RipplePhase[f])&4095)/64;
			if (a<0) a = -a;
			
			float l= 10.0f;

			*vSphere = *vSphere*l;

	   //		*vSphere *= 1.0f;
		//   	TranslatePointIntoViewspace(vSphere);
			vSphere++;
		}
 
	//	Render_SkyStars();	
	//	Render_MoonBlackOut();

//			Render_Moon();
		int iTextureToUse = Asura_Skybox::GetIsUsingSecondClouds() ? g_iFractalTexture2 : g_iFractalTexture;

		Asura_Render::CurrentStates.RequestTextureIndex(0,iTextureToUse);
		Asura_Render::CurrentStates.RequestTextureIndex(1,iTextureToUse);

		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_MODULATE);
		Asura_Direct3D::D3DDevice->SetTextureStageState(1,D3DTSS_COLOROP,   D3DTOP_ADD);

	
		int numberOfIndices = (SPHERE_FACES*3)/2;
		int numberOfFaces = SPHERE_FACES/2;
	
		{
			D3DGenericVertex* vertexData;
			D3DGenericVertex* vertexData2;
			unsigned short* indexData;

			RenderMeshIndexBuffer->Lock(0,sizeof(unsigned short)*(numberOfIndices),(void**)&indexData,D3DLOCK_DISCARD);
			RenderMeshVertexBuffer->Lock(0,sizeof(D3DGenericVertex)*(numberOfIndices),(void**)&vertexData,D3DLOCK_DISCARD);
			RenderMeshVertexBuffer2->Lock(0,sizeof(D3DGenericVertex)*(numberOfIndices),(void**)&vertexData2,D3DLOCK_DISCARD);
	
			int i=0;
			int indexNumber = 0;

			float phase1 = Asura_Timers::GetLocalScaledTime() / 64.0f;  ///2.0f;
			float phase2 = Asura_Timers::GetLocalScaledTime() / 128.0f; ///2.0f;
			for (int p=0; p<numberOfFaces; p++)
			{	
				int v;

				for (v=0; v<3; v++) 
				{
					int n = SphereFace[p].v[v];
				
					vertexData[v].X = SphereRotatedVertex[n].x;
					vertexData[v].Y = SphereRotatedVertex[n].y;
					vertexData[v].Z = SphereRotatedVertex[n].z;
					vertexData[v].Diffuse = 0xffffffff;
				
	  				vertexData[v].U = SphereVertex[n].U;
					vertexData[v].V = SphereVertex[n].V;

					vertexData[v].U2 = SphereVertex[n].U;
					vertexData[v].V2 = SphereVertex[n].V;
				
					vertexData2[v].X = SphereRotatedVertex[n].x;
					vertexData2[v].Y = SphereRotatedVertex[n].y;
					vertexData2[v].Z = SphereRotatedVertex[n].z;
					vertexData2[v].Diffuse = colour;
				
	  				vertexData2[v].U = SphereVertex[n].U;
					vertexData2[v].V = SphereVertex[n].V;

					vertexData2[v].U2 = SphereVertex[n].U;
					vertexData2[v].V2 = SphereVertex[n].V;
				}

				FixVertexWrap(vertexData[0].V,vertexData[1].V,vertexData[2].V);
				FixVertexWrap(vertexData[0].V2,vertexData[1].V2,vertexData[2].V2);
				FixVertexWrap(vertexData2[0].V,vertexData2[1].V,vertexData2[2].V);
				FixVertexWrap(vertexData2[0].V2,vertexData2[1].V2,vertexData2[2].V2);
	
				for (v=0; v<3; v++) 
				{
					vertexData[v].U = (vertexData[v].U)+phase2;
					vertexData[v].V = (vertexData[v].V)-phase1;

					vertexData[v].U2 = (vertexData[v].U2*2)-phase1;
					vertexData[v].V2 = (vertexData[v].V2*2)-phase2;
				
	  				vertexData2[v].U = (vertexData2[v].U)+phase1;
					vertexData2[v].V = (vertexData2[v].V);

					vertexData2[v].U2 = (vertexData2[v].U2*2)+phase2;
					vertexData2[v].V2 = (vertexData2[v].V2*2)+phase2;
				}
				vertexData+=3;
				vertexData2+=3;
				
				indexData[i++] = (unsigned short)(indexNumber+0);
				indexData[i++] = (unsigned short)(indexNumber+1);
				indexData[i++] = (unsigned short)(indexNumber+2);
				indexNumber+=3;
				
			}
			
			RenderMeshIndexBuffer->Unlock(); 
			RenderMeshVertexBuffer->Unlock();
			RenderMeshVertexBuffer2->Unlock();
		}	
		Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
		Asura_Direct3D::D3DDevice->SetSamplerState(1, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );
	
		
		Asura_Render::CurrentStates.RequestDynamicLights(false);
		Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);
		
		Asura_Direct3D::D3DDevice->SetIndices(RenderMeshIndexBuffer);

		for(int i=0; i<Asura_Camera::GetNumberOfCameras(); i++)
		{
			Asura_Camera::SetCameraIndex(i);

			if (Asura_Camera::IsCameraActive())
			{
				Asura_Render::ApplyCameraSettings();
				{
					const Asura_Vector_3* cameraPosition = Asura_Camera::GetPosition();
					D3DXMATRIX matWorld;
					D3DXMatrixTranslation( &matWorld, cameraPosition->x, cameraPosition->y, cameraPosition->z );
					Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &matWorld );
				}

				Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
				Asura_Direct3D::D3DDevice->SetStreamSource(0,RenderMeshVertexBuffer,0,sizeof(D3DGenericVertex));
				Asura_Direct3D::D3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST,0,0,numberOfIndices,0,numberOfFaces);

				Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
				Asura_Direct3D::D3DDevice->SetStreamSource(0,RenderMeshVertexBuffer2,0,sizeof(D3DGenericVertex));
				Asura_Direct3D::D3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST,0,0,numberOfIndices,0,numberOfFaces);
			}
		}

	}

}

#endif

#endif
