/**
*
* \file Asura_Platform_Corona.cpp
*
* Created by Kevin Lea - 27/01/2003 18:43:57
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "SpecialFX/Asura_Corona_System.h"

#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_StoreRenderTargets.h"
#include "Rendering/Asura_Render.h"
#include "Core/Asura_TextureManagement.h"
#include "Rendering/Asura_Direct3D_Shaders.h"


#ifndef ASURA_TARGET_SM3

void Asura_Corona_System::Platform_Initialise()
{
}

void Asura_Corona_System::Platform_Shutdown()
{
}

void Asura_Corona_System::Platform_RenderStart(void)
{
	D3DPERFCALL(BeginEvent(0xffff8899, L"Coronas"));

	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, 0 );

	Asura_Direct3D_Shaders::RequestPixelShader( ASURA_D3D_PS_NONE );
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);

	
	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);
		
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
	Asura_Render::CurrentStates.RequestAlphaTest(false);
}

void Asura_Corona_System::Platform_RenderCorona(const Asura_Corona* pxCorona, u_int uCamVisFlags)
{
	/// Kevin Lea - 27/01/2003 15:21:02 - add corona to a system of batch buffers based on texture
	/// to be later rendered by RenderEnd() (or on an ongoing basis if buffers fill up etc.


	/// Kevin Lea - 27/01/2003 15:24:57 - hacky first-pass renderer to test independent & tools code
	/// NOT INTENDED AS EXAMPLE CODE
	Asura_Render::CurrentStates.RequestTextureIndex(0,pxCorona->m_iTextureIndex);

	float fSize = pxCorona->m_fRadius;
	
	for(int i=0; i<Asura_Camera::GetNumberOfCameras(); i++)
	{
		if(!(uCamVisFlags & (1<<i))) continue;
		
		Asura_Camera::SetCameraIndex(i);
		Asura_Render::ApplyCameraSettings();


		const Asura_Vector_3*	pxCameraPosition = Asura_Camera::GetPosition();
		const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();

		Asura_Vector_3 xDeltaX = Asura_Vector_3(pxCameraOrientation->e11,pxCameraOrientation->e21,pxCameraOrientation->e31)*fSize;
		Asura_Vector_3 xDeltaY = Asura_Vector_3(pxCameraOrientation->e12,pxCameraOrientation->e22,pxCameraOrientation->e32)*fSize;

		Asura_Vector_3 xVertex[4];
		xVertex[0] = pxCorona->m_xPosition - xDeltaX - xDeltaY;
		xVertex[1] = pxCorona->m_xPosition + xDeltaX - xDeltaY;
		xVertex[2] = pxCorona->m_xPosition + xDeltaX + xDeltaY;
		xVertex[3] = pxCorona->m_xPosition - xDeltaX + xDeltaY; 

		float fLightScale = 255.0f; // KJL - 21/07/2003 15:03:24 - this is set to default for (m_bUseDirectionEffect==false)

		Asura_Vector_3 xDelta =  *pxCameraPosition - pxCorona->m_xPosition ;
		float fDistance = xDelta.Magnitude();
		
		if(pxCorona->m_bUseDirectionEffect)
		{
			if (fDistance>0.0f)
			{
				fLightScale = (pxCorona->m_xDirection * xDelta) / fDistance;
			
				if (fLightScale<0)
				{
					fLightScale = 64.0f;
				}
				else
				{
					fLightScale = 191.0f*fLightScale+64.0f;
				}
			}
			else
			{
				fLightScale = 0.0f;
			}
		}

		if( !pxCorona->m_bDoNotFadeOutWithDistance )
		{
			float fDistanceScale = fDistance - ASURA_CORONA_FADE_DIST_BEGIN;
			
			if (fDistanceScale > 0.0f)
			{
				fDistanceScale *= ( 1.0f / (ASURA_CORONA_FADE_DIST_END - ASURA_CORONA_FADE_DIST_BEGIN) );

				fLightScale *= (1.0f - fDistanceScale);
			}
		}

		const float fNearFadeDistance = pxCorona->GetNearFadeDistance();
		if(fNearFadeDistance)
		{
			Asura_Vector_3 xZDir = Asura_Vector_3(pxCameraOrientation->e13,pxCameraOrientation->e23,pxCameraOrientation->e33);
			float fZDistance = -(xDelta*xZDir);

			if(fZDistance<fNearFadeDistance)
			{
				fLightScale *= fZDistance/fNearFadeDistance;
				if(fLightScale<0) fLightScale = 0;
			}
		}

		unsigned int uColour = 0xff000000	
						 + (Asura_Maths::FloatToInt(fLightScale*pxCorona->m_fRed)<<16)
						 + (Asura_Maths::FloatToInt(fLightScale*pxCorona->m_fGreen)<<8)
						 + (Asura_Maths::FloatToInt(fLightScale*pxCorona->m_fBlue));

		{
			D3DGenericVertex v[4];
			
			v[0].U = 0.0f;
			v[0].V = 0.0f;

			v[1].U = 1.0f;
			v[1].V = 0.0f;

			v[2].U = 1.0f;
			v[2].V = 1.0f;

			v[3].U = 0.0f;
			v[3].V = 1.0f;

			for (int i=0; i<4; i++)
			{
				v[i].X = xVertex[i].x;
				v[i].Y = xVertex[i].y;
				v[i].Z = xVertex[i].z;

				v[i].Diffuse = uColour;
			}
			
			Asura_Direct3D::D3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, (LPVOID)v, sizeof(D3DGenericVertex) );

		}	
	}

}

void Asura_Corona_System::Platform_RenderEnd(void)
{
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );
	D3DPERFCALL(EndEvent());
}



#else	// ASURA_TARGET_SM3

// DMR 11/07/07 - New SM3 corona renderer - with soft particle support and some batching

#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"

const int iASURA_MAX_BATCHABLE_CORONA_TEXTURES	   = 8;
const int iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE = 32;

// Similar to D3DGenericVertex but with HDR colours
struct CoronaVertex
{
	float X,Y,Z;
	class Asura_Vector_3 Normal;
	float Diffuse[3];
	float U;
	float V;
	float U2;
	float V2;
};

class Asura_Direct3D_Corona_System
{
public:
	static void		Initialise();
	static void		Shutdown();
	static void		RenderStart();
	static void		RenderEnd();
	static void		RenderCorona(const Asura_Corona* pxCorona, u_int uCamVisFlags);	

	static void		OnEffectLoaded(Asura_Direct3D_Effect* pxEffect);
	static void		OnEffectUnloaded(Asura_Direct3D_Effect* pxEffect);
	static void		CaptureEffect(void);

private:
	static void		FlushBatch(int iTextureSlot);
	static float	CalcLightScale(const Asura_Corona *pxCorona, const Asura_Vector_3 *pxCameraPosition, const Asura_Vector_3 &xShiftForward, const Asura_Matrix_3x3 *pxCameraOrientation);
	
	static int				   s_aiBatchedTextures  [iASURA_MAX_BATCHABLE_CORONA_TEXTURES];
	static int				   s_aiNumBatchedCoronas[iASURA_MAX_BATCHABLE_CORONA_TEXTURES];
	static const Asura_Corona* s_apxBatchedCoronas  [iASURA_MAX_BATCHABLE_CORONA_TEXTURES][iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE];
	static uint32			   s_auBatchedVisFlags	[iASURA_MAX_BATCHABLE_CORONA_TEXTURES][iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE];
	static CoronaVertex		   s_axVertices			[iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE*4];
	static uint16			   s_au16Indices		[iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE*6];
		
	static Asura_Direct3D_Effect*	   s_pxCoronaEffect;
	static Direct3D_VertexDeclaration* s_pxCoronaVertexDecl;

	static D3DXHANDLE s_xTextureHandle;
	static D3DXHANDLE s_xDepthTextureHandle;
	static D3DXHANDLE s_xOneOverScreenDimensionsHandle;
	
	static Asura_Direct3D_StoreRenderTargets s_xStoredRenderTargets;
};


int						Asura_Direct3D_Corona_System::s_aiBatchedTextures  [iASURA_MAX_BATCHABLE_CORONA_TEXTURES];
int						Asura_Direct3D_Corona_System::s_aiNumBatchedCoronas[iASURA_MAX_BATCHABLE_CORONA_TEXTURES];
const Asura_Corona*		Asura_Direct3D_Corona_System::s_apxBatchedCoronas  [iASURA_MAX_BATCHABLE_CORONA_TEXTURES][iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE];
uint32					Asura_Direct3D_Corona_System::s_auBatchedVisFlags  [iASURA_MAX_BATCHABLE_CORONA_TEXTURES][iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE];
CoronaVertex			Asura_Direct3D_Corona_System::s_axVertices		   [iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE*4];
uint16					Asura_Direct3D_Corona_System::s_au16Indices		   [iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE*6];

Direct3D_VertexDeclaration*	Asura_Direct3D_Corona_System::s_pxCoronaVertexDecl = 0;
Asura_Direct3D_Effect*		Asura_Direct3D_Corona_System::s_pxCoronaEffect = 0;

D3DXHANDLE					Asura_Direct3D_Corona_System::s_xTextureHandle = 0;
D3DXHANDLE					Asura_Direct3D_Corona_System::s_xDepthTextureHandle = 0;
D3DXHANDLE					Asura_Direct3D_Corona_System::s_xOneOverScreenDimensionsHandle = 0;

Asura_Direct3D_StoreRenderTargets Asura_Direct3D_Corona_System::s_xStoredRenderTargets;


// All platform calls passed straight to Asura_Direct3D_CoronaSystem
void Asura_Corona_System::Platform_Initialise()														{ Asura_Direct3D_Corona_System::Initialise();							}
void Asura_Corona_System::Platform_Shutdown()														{ Asura_Direct3D_Corona_System::Shutdown();								}
void Asura_Corona_System::Platform_RenderStart(void)												{ Asura_Direct3D_Corona_System::RenderStart();							}
void Asura_Corona_System::Platform_RenderEnd(void)													{ Asura_Direct3D_Corona_System::RenderEnd();							}
void Asura_Corona_System::Platform_RenderCorona(const Asura_Corona* pxCorona, u_int uCamVisFlags)	{ Asura_Direct3D_Corona_System::RenderCorona(pxCorona, uCamVisFlags);   }



// ---------------------------------------------------------------------------------
// Initialise() - Clear the batches, init the shader
// ---------------------------------------------------------------------------------

void Asura_Direct3D_Corona_System::Initialise()
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}
	
	for(int i=0; i<iASURA_MAX_BATCHABLE_CORONA_TEXTURES; i++)
	{
		s_aiNumBatchedCoronas[i] = 0;
		s_aiBatchedTextures[i] = -1;

		for(int j=0; j<iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE; j++)
		{
			s_apxBatchedCoronas[i][j]=0;
		}
	}

	for(int i=0; i<iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE; i++)
	{
		// Indices remain constant
		s_au16Indices[i*6+0] = static_cast<uint16>((i*4)+0);
		s_au16Indices[i*6+1] = static_cast<uint16>((i*4)+1);
		s_au16Indices[i*6+2] = static_cast<uint16>((i*4)+2);
		s_au16Indices[i*6+3] = static_cast<uint16>((i*4)+2);
		s_au16Indices[i*6+4] = static_cast<uint16>((i*4)+3);
		s_au16Indices[i*6+5] = static_cast<uint16>((i*4)+0);
		
		// UVs are also constant
		s_axVertices[i*4+0].U = 0.0f;
		s_axVertices[i*4+0].V = 0.0f;
		s_axVertices[i*4+1].U = 1.0f;
		s_axVertices[i*4+1].V = 0.0f;
		s_axVertices[i*4+2].U = 1.0f;
		s_axVertices[i*4+2].V = 1.0f;
		s_axVertices[i*4+3].U = 0.0f;
		s_axVertices[i*4+3].V = 1.0f;
	}	
	
	Direct3D_VertexElement auVSD[] =
	{
		{	0,	0,	D3DDECLTYPE_FLOAT3,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },
		{	0, 12,	D3DDECLTYPE_FLOAT3,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_NORMAL,	0 },
		{	0, 24,	D3DDECLTYPE_FLOAT3,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_COLOR,		0 },
		{	0, 36,	D3DDECLTYPE_FLOAT4,	D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_TEXCOORD,	0 },
		D3DDECL_END()
	};
	
	Asura_Direct3D::D3DDevice->CreateVertexDeclaration( auVSD, &(s_pxCoronaVertexDecl) );

	CaptureEffect();
}


// ---------------------------------------------------------------------------------
// Shutdown()
// ---------------------------------------------------------------------------------

void Asura_Direct3D_Corona_System::Shutdown()
{
	SAFE_RELEASE( s_pxCoronaVertexDecl);
}


// ------------------------------------------------------------------------------------
// Shader loading + parameter getting
// ------------------------------------------------------------------------------------

void Asura_Direct3D_Corona_System::CaptureEffect(void)
{
	#define LOADPARAMETER(codeName, shaderName) { codeName = s_pxCoronaEffect->GetD3DEffect()->GetParameterByName(NULL, shaderName); Asura_Assert(codeName, "Could not get shader parameter '" shaderName "'."); }

	s_pxCoronaEffect = Asura_Direct3D_Effects_System::GetEffect("coronas");

	if (s_pxCoronaEffect)
	{	
#if ASURA_LOAD_EFFECT_FILES == 1
		s_pxCoronaEffect->SetOnLoadCallback(Asura_Direct3D_Corona_System::OnEffectLoaded);
		s_pxCoronaEffect->SetOnUnloadCallback(Asura_Direct3D_Corona_System::OnEffectUnloaded);
#endif

		LOADPARAMETER(s_xTextureHandle					, "g_xTexture" );
		LOADPARAMETER(s_xDepthTextureHandle				, "g_xDepthTexture" );
		LOADPARAMETER(s_xOneOverScreenDimensionsHandle  , "g_xOneOverScreenDimensions" );
	}
}

#if ASURA_LOAD_EFFECT_FILES == 1
void Asura_Direct3D_Corona_System::OnEffectLoaded(Asura_Direct3D_Effect* pxEffect)
{
	Asura_Direct3D_Corona_System::CaptureEffect();
}

void Asura_Direct3D_Corona_System::OnEffectUnloaded(Asura_Direct3D_Effect* pxEffect)
{
	Asura_Direct3D_Corona_System::CaptureEffect();
}
#endif


// ---------------------------------------------------------------------------------
// RenderStart() - set up all constant renderstates/shader params
// ---------------------------------------------------------------------------------

void Asura_Direct3D_Corona_System::RenderStart()
{
	D3DPERFCALL(BeginEvent(0xffff8899, L"Coronas"));
	
	// DLH - Store off and clear the present render targets from the D3DDevice
	s_xStoredRenderTargets.StoreRenderTargets();
	Asura_Direct3D::ClearRenderTargets( 1);

	// Setup initial renderstates
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
	Asura_Render::CurrentStates.RequestAlphaTest(false);

	Asura_Direct3D::D3DDevice->SetVertexDeclaration(s_pxCoronaVertexDecl);
	Asura_Direct3D_Effects_System::BeginEffect(*s_pxCoronaEffect);
	
	// Constant shader params
	D3DXVECTOR4 xVec;
	xVec.x = 1.0f / Asura_Viewport::GetWidth();
	xVec.y = 1.0f / Asura_Viewport::GetHeight();
	s_pxCoronaEffect->GetD3DEffect()->SetVector(s_xOneOverScreenDimensionsHandle, &xVec);

	s_pxCoronaEffect->GetD3DEffect()->SetTexture(s_xDepthTextureHandle, Asura_Direct3D::GetDeferredRenderingTarget()->GetTexture(  ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX));
}



// ---------------------------------------------------------------------------------
// RenderEnd() - Flush all batches, then restore all renderstates
// ---------------------------------------------------------------------------------

void Asura_Direct3D_Corona_System::RenderEnd()
{
	for(int i=0; i<iASURA_MAX_BATCHABLE_CORONA_TEXTURES; i++)
	{	
		FlushBatch(i);
	}
	
	Asura_Direct3D_Effects_System::FlushEffects();
	Asura_Direct3D_Shaders::RequestPixelShader( ASURA_D3D_PS_NONE );
	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);

	s_pxCoronaEffect->GetD3DEffect()->SetTexture(s_xDepthTextureHandle, NULL);

	// DLH - Restore the render target states when RenderStart was called
	s_xStoredRenderTargets.RestoreRenderTargets();
	s_xStoredRenderTargets.Clear();
	
	D3DPERFCALL(EndEvent());
}


// ---------------------------------------------------------------------------------
// RenderCorona() - This queues the corona for batched rendering - but will
//					flush a batch if there isn't space to add the new corona
// ---------------------------------------------------------------------------------

void Asura_Direct3D_Corona_System::RenderCorona(const Asura_Corona* pxCorona, u_int uCamVisFlags)
{
	
	if (pxCorona->m_iTextureIndex<0)
	{
		// Invalid texture
		return;
	}

	// Do we have a batch for this texture?
	int iTextureSlot = -1;
	for(int i=0; i<iASURA_MAX_BATCHABLE_CORONA_TEXTURES; i++)
	{							
		if (s_aiBatchedTextures[i] == pxCorona->m_iTextureIndex)
		{
			iTextureSlot = i;
		}
	}

	if (iTextureSlot>=0)
	{
		// If we've got a batch, but it's full, we need to flush it
		if (s_aiNumBatchedCoronas[iTextureSlot]>=iASURA_MAX_BATCHABLE_CORONAS_PER_TEXTURE)
		{
			FlushBatch(iTextureSlot);
		}		
	}
	else
	{
		// No - Do we have an empty slot for a new texture batch?		
		int iBestToFlush  = 0;		
		int iBiggestBatch = 0;		

		for(int i=0; i<iASURA_MAX_BATCHABLE_CORONA_TEXTURES; i++)
		{					
			// Find the first empty slot, if we have one
			if (s_aiBatchedTextures[i]<0)
			{
				if (iTextureSlot<0)
				{
					iTextureSlot = i;
				}				
			}

			// Update best candidate to flush (the one with the biggest batch already)
			if (s_aiNumBatchedCoronas[i]>iBiggestBatch)
			{
				iBiggestBatch = s_aiNumBatchedCoronas[i];
				iBestToFlush = i;
			}
		}

		if (iTextureSlot<0)
		{
			FlushBatch(iBestToFlush);
			iTextureSlot = iBestToFlush;
		}
	}
	
	s_aiBatchedTextures  [iTextureSlot] = pxCorona->m_iTextureIndex;	
	s_apxBatchedCoronas  [iTextureSlot][s_aiNumBatchedCoronas[iTextureSlot]] = pxCorona;	
	s_auBatchedVisFlags  [iTextureSlot][s_aiNumBatchedCoronas[iTextureSlot]] = uCamVisFlags;
	s_aiNumBatchedCoronas[iTextureSlot]++;
}




// ---------------------------------------------------------------------------------
// CalcLightScale() - Calculate the intensity of the corona - handle the distance fade
//                    and direction effect
// ---------------------------------------------------------------------------------

float Asura_Direct3D_Corona_System::CalcLightScale(const Asura_Corona *pxCorona, const Asura_Vector_3 *pxCameraPosition,
												   const Asura_Vector_3 &xShiftForward, const Asura_Matrix_3x3 *pxCameraOrientation)
{
	float fLightScale = 255.0f; 

	Asura_Vector_3 xDelta =  *pxCameraPosition - (pxCorona->m_xPosition + xShiftForward);
	float fDistance = xDelta.Magnitude();

	if(pxCorona->m_bUseDirectionEffect)
	{
		if (fDistance>0.0f)
		{
			fLightScale = (pxCorona->m_xDirection * xDelta) / fDistance;

			if (fLightScale<0)
			{
				fLightScale = 64.0f;
			}
			else
			{
				fLightScale = 191.0f*fLightScale+64.0f;
			}
		}
		else
		{
			fLightScale = 0.0f;
		}
	}

	if( !pxCorona->m_bDoNotFadeOutWithDistance )
	{
		float fDistanceScale = fDistance - ASURA_CORONA_FADE_DIST_BEGIN;

		if (fDistanceScale > 0.0f)
		{
			fDistanceScale *= ( 1.0f / (ASURA_CORONA_FADE_DIST_END - ASURA_CORONA_FADE_DIST_BEGIN) );

			fLightScale *= (1.0f - fDistanceScale);
		}
	}

	const float fNearFadeDistance = pxCorona->GetNearFadeDistance();
	if(fNearFadeDistance)
	{
		Asura_Vector_3 xZDir = Asura_Vector_3(pxCameraOrientation->e13,pxCameraOrientation->e23,pxCameraOrientation->e33);
		float fZDistance = -(xDelta*xZDir);

		if(fZDistance<fNearFadeDistance)
		{
			fLightScale *= fZDistance/fNearFadeDistance;
			if(fLightScale<0) fLightScale = 0;
		}
	}

	return fLightScale;
}


// ---------------------------------------------------------------------------------
// FlushBatch() - Render a batch of coronas that share the same texture
// ---------------------------------------------------------------------------------

void Asura_Direct3D_Corona_System::FlushBatch(int iTextureSlot)
{
	// Have we got something to render?
	if (s_aiBatchedTextures[iTextureSlot]>=0 && s_aiNumBatchedCoronas[iTextureSlot]>0)
	{
		// Set the texture
		Direct3D_BaseTexture* pxTexture = Asura_Direct3D::GetDirect3DTextureByIndex(s_aiBatchedTextures[iTextureSlot]);
		Asura_Assert( NULL != pxTexture, "Corona Texture not found");
		if( NULL != pxTexture)
		{
			s_pxCoronaEffect->GetD3DEffect()->SetTexture( s_xTextureHandle, pxTexture );
		}
		
		for(int iCamera=0; iCamera<Asura_Camera::GetNumberOfCameras(); iCamera++)
		{			
			Asura_Camera::SetCameraIndex(iCamera);
			Asura_Render::ApplyCameraSettings();

			const Asura_Vector_3*	pxCameraPosition = Asura_Camera::GetPosition();
			const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();
			Asura_Vector_3 xDeltaX = Asura_Vector_3(pxCameraOrientation->e11,pxCameraOrientation->e21,pxCameraOrientation->e31);
			Asura_Vector_3 xDeltaY = Asura_Vector_3(pxCameraOrientation->e12,pxCameraOrientation->e22,pxCameraOrientation->e32);
			
			// 2 passes through the coronas - first the soft ones, then the non-soft ones?
			for(int iPass=0; iPass<2; iPass++)
			{
				int iNumCoronas = 0;
				int iNumBatched = s_aiNumBatchedCoronas[iTextureSlot];

				CoronaVertex *pxVtx = &s_axVertices[0];				

				for(int i=0; i<iNumBatched; i++)
				{
					const Asura_Corona *pxCorona = s_apxBatchedCoronas[iTextureSlot][i];					

					Asura_Vector_3 xForward = pxCorona->m_xPosition - *pxCameraPosition;
					xForward.Normalise();
					
					if (iPass==0 && pxCorona->m_fVolumetricDepthScale<=0.0f || 
						iPass==1 && pxCorona->m_fVolumetricDepthScale>0.0f)
					{
						if((s_auBatchedVisFlags[iTextureSlot][i] & (1<<iCamera))) 
						{
							Asura_Vector_3 xShiftForward =- ( xForward * pxCorona->m_fRadius * pxCorona->m_fVolumetricDepthScale );

							// Calculate the colour
							float fLightScale = CalcLightScale(pxCorona, pxCameraPosition, xShiftForward, pxCameraOrientation);

							if (fLightScale>0.0f)
							{															
								// Calculate the billboarded vert positions
								float fSize = pxCorona->m_fRadius;
								float afDeltaXScale[4] = {-fSize, fSize, fSize,-fSize};
								float afDeltaYScale[4] = {-fSize,-fSize, fSize, fSize};

								for(int i=0; i<4; i++)
								{
									pxVtx->X = pxCorona->m_xPosition.x + (afDeltaXScale[i] * xDeltaX.x) + (afDeltaYScale[i] * xDeltaY.x) + xShiftForward.x;
									pxVtx->Y = pxCorona->m_xPosition.y + (afDeltaXScale[i] * xDeltaX.y) + (afDeltaYScale[i] * xDeltaY.y) + xShiftForward.y;
									pxVtx->Z = pxCorona->m_xPosition.z + (afDeltaXScale[i] * xDeltaX.z) + (afDeltaYScale[i] * xDeltaY.z) + xShiftForward.z;
									pxVtx->Diffuse[0] = fLightScale * pxCorona->m_fRed / 255.0f;
									pxVtx->Diffuse[1] = fLightScale * pxCorona->m_fGreen / 255.0f;
									pxVtx->Diffuse[2] = fLightScale * pxCorona->m_fBlue / 255.0f;
									pxVtx->U2 = fSize * pxCorona->m_fVolumetricDepthScale * 2.0f;
									pxVtx++;
								}

								// Add the corona to the VB
								iNumCoronas++;
							}
						}						
					}				
				}

				// Render 'em now
				if (iNumCoronas>0)
				{

					s_pxCoronaEffect->GetD3DEffect()->BeginPass(iPass);
					s_pxCoronaEffect->GetD3DEffect()->CommitChanges();

					Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP(D3DPT_TRIANGLELIST, 0, iNumCoronas*4, iNumCoronas*2, s_au16Indices, D3DFMT_INDEX16, (LPVOID)s_axVertices, sizeof(CoronaVertex));

					s_pxCoronaEffect->GetD3DEffect()->EndPass();
				}
			}
		}

		// This is enabled in the effect, so undo that now
		for( u_int u = 0; u < 16; ++u )
		{
			Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_SRGBTEXTURE, FALSE );
		}
	}

	s_aiBatchedTextures[iTextureSlot]   = -1;
	s_aiNumBatchedCoronas[iTextureSlot] = 0;
}




#endif // ASURA_TARGET_SM3