///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Platform_Decal_SM3.h
// Author : Dave Reed
// Date   : 20-Aug-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef _ASURA_PLATFORM_DECAL_SM3_H_
#define _ASURA_PLATFORM_DECAL_SM3_H_

#ifdef ASURA_TARGET_SM3

#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "SpecialFX/Asura_Decal_Template.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"


// ------------------------------------------------------------------------------------
// Vertex Formats
// ------------------------------------------------------------------------------------

class D3D_DecalVertex
{
public:

	float	fX,fY,fZ;
	float	fU,fV;	
	float	fCreationTime;
	float	fAlpha;
	float	afNormal[3];
	float	afTangent[3];
	float	afBinormal[3];	
	float	afSHRed[4];
	float	afSHGreen[4];
	float	afSHBlue[4];
};

// ------------------------------------------------------------------------------------
// Shader Parameters object
// ------------------------------------------------------------------------------------

class Asura_Direct3D_Decal_ShaderParameters
{	
public:
	// (We rely on shared params being set for WVP matrix and camera pos)
	D3DXHANDLE m_xFadeValues;	
	D3DXHANDLE m_xUVScroll;
	D3DXHANDLE m_xPrelightingColourFilter;

	void Get(Asura_Direct3D_Effect *pxEffect)
	{
		m_xFadeValues	  = pxEffect->GetD3DEffect()->GetParameterByName(NULL, "g_xFadeValues");
		m_xUVScroll       = pxEffect->GetD3DEffect()->GetParameterByName(NULL, "g_xUVScroll");
		m_xPrelightingColourFilter       = pxEffect->GetD3DEffect()->GetParameterByName(NULL, "g_xPrelightingColourFilter");
	}
};




// ------------------------------------------------------------------------------------
// Asura_Direct3D_DecalBuffer 
// This stores all decals for one material type (which may be more than one decal template)
// Has a circular buffer of verts, and an index buffer that gets shifted down when full
// ------------------------------------------------------------------------------------
class Asura_Direct3D_DecalBuffer : public Asura_DecalBuffer
{
	friend class Asura_Decal_Handler;

public:

	Asura_Direct3D_DecalBuffer(const Asura_Decal_Template* pxDecalTemplate);
	~Asura_Direct3D_DecalBuffer();

	void AddStrip(const Asura_Decal_Vertex* pxStrip, const u_int uNumberOfVertices, const Asura_Decal_Template* pxDecalTemplate, const Asura_RGBA& xDiffuseRGBA);
	void Render(void);
	
	class Decal
	{
	public:
		int		iFirstIndex;
		int		iNumIndices;		
		float	fCreationTime;
	};

private:

	int							m_iMaxDecals;				// Max decals in this buffer
	int							m_iDecalMaterialIndex;		// Material used by this buffer	
	Decal*						m_apxDecals;				// Decal structs (circular buffer)
	int							m_iNextVertex;				// Next vertex to use (verts = another circular buffer)
	int							m_iNumVerts;
	D3D_DecalVertex*			m_apxVerts;					// Verts  
	int							m_iNumIndices;
	uint16*						m_apxIndices;				// Indices 
	int							m_iFirstDecal;				// First decal struct in use
	int							m_iNumDecals;				// Num decals active	
	int							m_iNumDecalTris;			// Num decal triangles active	
	Asura_Direct3D_DecalBuffer* m_pxNextDecalBuffer;
};



// ----------------------------------------------------------------------------
// Asura_Direct3D_StaticDecalBuffer
// ----------------------------------------------------------------------------

class Asura_Direct3D_StaticDecalBuffer
{
public:

	Asura_Direct3D_StaticDecalBuffer()
	{
		m_pxIndexBuffer = NULL;
		m_pxVertexBuffer = NULL;
	}

	~Asura_Direct3D_StaticDecalBuffer()
	{
		if (m_pxIndexBuffer)
		{
			m_pxIndexBuffer->Release();
			m_pxIndexBuffer = NULL;
		}

		if (m_pxVertexBuffer)
		{
			m_pxVertexBuffer->Release();
			m_pxVertexBuffer = NULL;
		}
	}


	int							m_iModuleIndex;
	uint32						m_uDecalMaterialHash;
	int							m_iDecalMaterialIndex;
	int							m_iNumVertices;
	int							m_iNumIndices;
	Asura_Bounding_Box			m_xAABB;

	Direct3D_IndexBuffer*		m_pxIndexBuffer;
	Direct3D_VertexBuffer*		m_pxVertexBuffer;
};


// ----------------------------------------------------------------------------
// Asura_Platform_Decal_System_SM3
// ----------------------------------------------------------------------------

class Asura_Platform_Decal_System_SM3
{

public:
	static void Initialise(void);
	static void Shutdown(void);
	static void Reset(void);
	static void RenderStaticDecals(void);

	static void CalculateTangentAndBinormal( const  Asura_Vector_3* pxPositions,
											 const  Asura_Vector_2* pxTextureCoords,
													Asura_Vector_3& rxTangent,
													Asura_Vector_3& rxBinormal );


	static Asura_Direct3D_Effect*				 GetEffect(void)		{return  s_pxDecalEffect; };
	static Asura_Direct3D_Decal_ShaderParameters*GetEffectParameters()  {return &s_xShaderParams; };
	static Direct3D_VertexDeclaration*			 GetVertexDecl()		{return  s_pxDecalVertexDecl;};
	static Direct3D_VertexDeclaration*			 GetVertexDeclSimple()	{return  s_pxDecalVertexSimpleDecl;};

	static void OnEffectLoaded(Asura_Direct3D_Effect* pxEffect);
	static void OnEffectUnloaded(Asura_Direct3D_Effect* pxEffect);
	static void CaptureEffect(void);

	static void									 AllocateStaticDecalBuffers(int iNumBuffers);
	static Asura_Direct3D_StaticDecalBuffer*	 GetStaticDecalBuffer(int iIndex);

	static void PrepareRenderStates(void);
	static void RestoreRenderStates(void);

	static void SetupMaterial(int iPass, const Asura_Decal_Material *pxMtl, float *pfFadeValues);


private:


	// The declaration objects
	static Direct3D_VertexDeclaration*			 s_pxDecalVertexDecl;
	static Direct3D_VertexDeclaration*			 s_pxDecalVertexSimpleDecl;

	// The shader
	static Asura_Direct3D_Effect*				 s_pxDecalEffect;

	// Its parameters
	static Asura_Direct3D_Decal_ShaderParameters s_xShaderParams;

	// Static decal buffers
	static int									 s_iNumStaticDecalBuffers;
	static Asura_Direct3D_StaticDecalBuffer*	 s_pxStaticDecalBuffers;

};

#endif // ASURA_TARGET_SM3

#endif // _ASURA_PLATFORM_DECAL_SM3_H_

