//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_Glow_System.cpp
//
// Created by AAM - 09/06/2004 17:41:23
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "SpecialFX/Asura_Glow_System.h"

#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_Caps.h"
#include "Rendering/Asura_Render.h"

/*******************************************************************************/
/* Types                                                                       */
/*******************************************************************************/

struct Asura_Glow_System_PC
{

	struct SimpleVertex
	{
		float m_fX, m_fY, m_fZ;
		float m_fU, m_fV;
	};

	/*******************************************************************************/
	/* Static globals                                                              */
	/*******************************************************************************/

	static Direct3D_Texture* s_pxGlowTexture;
	static Direct3D_Texture* s_pxHBlurTexture;

	static Direct3D_Surface* s_pxGlowSurface;
	static Direct3D_Surface* s_pxHBlurSurface;

	static Direct3D_VertexBuffer* s_pxQuadVB;

	static bool				 s_bGlowIsInitialised;

	static u_int			 s_uUsingBackBufferWidth;
	static u_int			 s_uUsingBackBufferHeight;
	static u_int			 s_uUsingRenderTargetWidth;
	static u_int			 s_uUsingRenderTargetHeight;

	static float			 s_afScaledHBlurOffsets[4][4];
	static float			 s_afScaledVBlurOffsets[4][4];
	static float			 s_afGlowColorParameter[3][4];

	/*******************************************************************************/
	/* Local function prototypes                                                   */
	/*******************************************************************************/

	static void ExtractGlow(Direct3D_Texture* pxSource);
	static void ApplyBlur();
	static void RenderGlow();
	
	static bool CreateRenderTargets(u_int uRenderTargetWidth, u_int uRenderTargetHeight);
	static void ReleaseRenderTargets();

	static void GenerateScaledOffsets();

};

Direct3D_Texture*		Asura_Glow_System_PC::s_pxGlowTexture				= NULL;
Direct3D_Texture*		Asura_Glow_System_PC::s_pxHBlurTexture				= NULL;

Direct3D_Surface*		Asura_Glow_System_PC::s_pxGlowSurface				= NULL;
Direct3D_Surface*		Asura_Glow_System_PC::s_pxHBlurSurface				= NULL;

Direct3D_VertexBuffer*	Asura_Glow_System_PC::s_pxQuadVB					= NULL;

bool					Asura_Glow_System_PC::s_bGlowIsInitialised			= false;

u_int					Asura_Glow_System_PC::s_uUsingBackBufferWidth		= 0;
u_int					Asura_Glow_System_PC::s_uUsingBackBufferHeight		= 0;
u_int					Asura_Glow_System_PC::s_uUsingRenderTargetWidth		= 0;
u_int					Asura_Glow_System_PC::s_uUsingRenderTargetHeight	= 0;

float					Asura_Glow_System_PC::s_afScaledHBlurOffsets[4][4];
float					Asura_Glow_System_PC::s_afScaledVBlurOffsets[4][4];
float					Asura_Glow_System_PC::s_afGlowColorParameter[3][4];

/*******************************************************************************/
/* Constants                                                                   */
/*******************************************************************************/

// Sampling offsets used by the horizontal and vertical blurs

const float afHBLUR_TEXCOORD_OFFSETS[4][4] =
{
	{ -3.0f, 0.5f, 0.0f, 0.0f },
	{ -1.0f, 0.5f, 0.0f, 0.0f },
	{  1.0f, 0.5f, 0.0f, 0.0f },
	{  3.0f, 0.5f, 0.0f, 0.0f },
};

const float afVBLUR_TEXCOORD_OFFSETS[4][4] =
{
	{ 0.5f, -3.0f, 0.0f, 0.0f },
	{ 0.5f, -1.0f, 0.0f, 0.0f },
	{ 0.5f,  1.0f, 0.0f, 0.0f },
	{ 0.5f,  3.0f, 0.0f, 0.0f },
};

// Filter weights
const float afFILTER_WEIGHTS[4][4] =
{
	{ 0.1f, 0.1f, 0.1f, 0.1f },
	{ 0.4f, 0.4f, 0.4f, 0.4f },
	{ 0.4f, 0.4f, 0.4f, 0.4f },
	{ 0.1f, 0.1f, 0.1f, 0.1f },
};

// Vertex data for the quad buffer

const Asura_Glow_System_PC::SimpleVertex axQUAD_VERTEX_DATA[4] =
{
	{ -1.0f,  1.0f,  0.0f,  0.0f,  0.0f },
	{  1.0f,  1.0f,  0.0f,  1.0f,  0.0f },
	{  1.0f, -1.0f,  0.0f,  1.0f,  1.0f },
	{ -1.0f, -1.0f,  0.0f,  0.0f,  1.0f },
};

const u_int				 uDEFAULT_RENDER_TARGET_WIDTH	= 128;
const u_int				 uDEFAULT_RENDER_TARGET_HEIGHT	= 128;



/*******************************************************************************/
/* Asura_Glow_System class functions                                           */
/*******************************************************************************/

//////////////////////////////////////////////////////
// Asura_Glow_System::Platform_Initialise()
//
// Parameters:    None
// Return value:  Success flag
//
// Initialises the glow system
//////////////////////////////////////////////////////
bool Asura_Glow_System::Platform_Initialise()
{
	// Make damn sure we're shut down
	Platform_Shutdown();

	Asura_Assert(!Asura_Glow_System_PC::s_bGlowIsInitialised, "Glow system trying to initialise twice!");

	// We need PS1.1 to run the effect (i.e. hardware pixel shaders).
	if( !Asura_Direct3D_Caps::HasHardwarePixelShaders() ) return false;

	Asura_Glow_System_PC::s_uUsingBackBufferWidth = Asura_Direct3D::GetPresentParameters().BackBufferWidth;
	Asura_Glow_System_PC::s_uUsingBackBufferHeight = Asura_Direct3D::GetPresentParameters().BackBufferHeight;

	if( !Asura_Glow_System_PC::CreateRenderTargets(uDEFAULT_RENDER_TARGET_WIDTH, uDEFAULT_RENDER_TARGET_HEIGHT) ) return false;

	Asura_Glow_System_PC::GenerateScaledOffsets();

	// Create the Quad vertex buffer
	if(FAILED(Asura_Direct3D::D3DDevice->CreateVertexBuffer(sizeof(axQUAD_VERTEX_DATA), D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, &Asura_Glow_System_PC::s_pxQuadVB, NULL)))
	{
		Platform_Shutdown();
		return false;
	}

	Asura_Glow_System_PC::SimpleVertex* pxVertex = NULL;
	if (SUCCEEDED(Asura_Glow_System_PC::s_pxQuadVB->Lock(0, 0, reinterpret_cast<void**>(&pxVertex), 0)))
	{
		memcpy(pxVertex, axQUAD_VERTEX_DATA, sizeof(axQUAD_VERTEX_DATA));
		Asura_Glow_System_PC::s_pxQuadVB->Unlock();
	} else {
		Asura_Assert(0, "Failed to lock VB");
	}

	Asura_Glow_System_PC::s_bGlowIsInitialised = true;

	return true;
}

//////////////////////////////////////////////////////
// Asura_Glow_System::Platform_Shutdown()
//
// Parameters:    None
// Return value:  None
//
// Shuts down the glow system
//////////////////////////////////////////////////////
void Asura_Glow_System::Platform_Shutdown() 
{
	if(Asura_Glow_System_PC::s_pxQuadVB)
	{
		Asura_Glow_System_PC::s_pxQuadVB->Release();
		Asura_Glow_System_PC::s_pxQuadVB = NULL;
	}
	Asura_Glow_System_PC::ReleaseRenderTargets();
	Asura_Glow_System_PC::s_bGlowIsInitialised = false;
}

//////////////////////////////////////////////////////
// Asura_Glow_System::Platform_PreReset()
//
// Parameters:    None
// Return value:  None
//
// Called before Reset() to let us release POOL_DEFAULT resources
//////////////////////////////////////////////////////
void Asura_Glow_System::Platform_PreReset()
{
	Asura_Glow_System_PC::ReleaseRenderTargets();
}

//////////////////////////////////////////////////////
// Asura_Glow_System::Platform_PostReset()
//
// Parameters:    None
// Return value:  None
//
// Called after Reset() to let us reacquire POOL_DEFAULT resources
//////////////////////////////////////////////////////
void Asura_Glow_System::Platform_PostReset() 
{
	// use g_bGlowIsInitialised to tell us whether the glow was on before the reset
	if ( Asura_Glow_System_PC::s_bGlowIsInitialised )
	{
		Platform_Initialise();
	}
}

//////////////////////////////////////////////////////
// Asura_Glow_System::Platform_Render()
//
// Parameters:    None
// Return value:  None
//
// Renders the full-screen glows
//////////////////////////////////////////////////////

DEBUGVAR float dbg_fGlowValueLowerLimit =  64.f / 255.f; // below this gets no glow
DEBUGVAR float dbg_fGlowValueUpperLimit = 255.f / 255.f; // above this gets full glow

void Asura_Glow_System::Platform_Render(float fRed, float fGreen, float fBlue, float fLevel) 
{
	HRESULT xResult;
	
	if(!Asura_Glow_System_PC::s_bGlowIsInitialised) return;

	D3DPERFCALL(BeginEvent(0xff885937, L"Glow"));

	if(!Asura_Direct3D_BackBufferCapture::HasCapturedThisFrame())
	{
		// Nobody's captured yet. So we do it.
		Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();
	}
	Asura_Render::AllowFullScreenRendering();

	// Check that the texture's ok.
	if(Asura_Direct3D_BackBufferCapture::IsCaptureDirty()) return; // Don't render anything if texture is dirty

	const float fGlowScale = 1.f / (dbg_fGlowValueUpperLimit - dbg_fGlowValueLowerLimit);
	const float fGlowOffset = -dbg_fGlowValueLowerLimit / (dbg_fGlowValueUpperLimit - dbg_fGlowValueLowerLimit);

	Asura_Glow_System_PC::s_afGlowColorParameter[0][0] = 0.296875;
	Asura_Glow_System_PC::s_afGlowColorParameter[0][1] = 0.59375;
	Asura_Glow_System_PC::s_afGlowColorParameter[0][2] = 0.109375;
	Asura_Glow_System_PC::s_afGlowColorParameter[0][3] = 0.0f;
	Asura_Glow_System_PC::s_afGlowColorParameter[1][0] = fGlowScale / 2.f; // ps1.1 constant registers are [-1..1] range
	Asura_Glow_System_PC::s_afGlowColorParameter[1][1] = fGlowScale / 2.f; // but fGlowScale is usually going to be > 1
	Asura_Glow_System_PC::s_afGlowColorParameter[1][2] = fGlowScale / 2.f; // so we'll scale it up in the shader
	Asura_Glow_System_PC::s_afGlowColorParameter[1][3] = 0.f;
	Asura_Glow_System_PC::s_afGlowColorParameter[2][0] = fGlowOffset / 2.f;
	Asura_Glow_System_PC::s_afGlowColorParameter[2][1] = fGlowOffset / 2.f;
	Asura_Glow_System_PC::s_afGlowColorParameter[2][2] = fGlowOffset / 2.f;
	Asura_Glow_System_PC::s_afGlowColorParameter[2][3] = 0.f;

	DWORD auOldAddressU[4], auOldAddressV[4];

	{
		SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Save previous state" ));

		// Set up texture addressing
		for ( u_int u = 0; u < 4; ++u )
		{
			Asura_Direct3D::D3DDevice->GetSamplerState(u, D3DSAMP_ADDRESSU, &auOldAddressU[u]);
			Asura_Direct3D::D3DDevice->GetSamplerState(u, D3DSAMP_ADDRESSV, &auOldAddressV[u]);

			Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
			Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
		}
	}

	// Set up some render states

	Asura_Render::CurrentStates.RequestAlphaTest(false);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);

		// I'm not totally sure why we set these two states instead of just turning off the Z-buffer,
		// apparently it's a workaround for other parts of the engine or something.
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_ALWAYS);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	Direct3D_Texture* pxCaptureTexture = Asura_Direct3D_BackBufferCapture::GetCaptureTexture();
	Asura_Assert(pxCaptureTexture, "Capture texture is NULL");

	Direct3D_Surface* pxOldDepthBuffer  = NULL;

	// Save out the old render target and depth buffer, to restore at the end
	// DLH - 14 February 2007 12:33:53 - Store and Clear all the RenderTargets after the first one
	const u_int uSurfaceCountMax = 8;
	Direct3D_Surface* apxSurface[ uSurfaceCountMax] = {NULL}; 
	u_int uSurfaceCount = Asura_Direct3D_Caps::GetMaxNumberOfRenderTargets();
	
	Asura_Assertf( uSurfaceCount < uSurfaceCountMax, ("%d Possible Render Targets useable. Only %d coped with.", uSurfaceCount, uSurfaceCountMax));
	// DLH - 14 February 2007 12:31:18 - Stops array over runs
	uSurfaceCount = Asura_Maths::Min( uSurfaceCount, uSurfaceCountMax);
	
	for ( u_int uSurfaceIndex = 0; uSurfaceIndex < uSurfaceCount; uSurfaceIndex += 1)
	{
		xResult = Asura_Direct3D::D3DDevice->GetRenderTarget( uSurfaceIndex, &(apxSurface[ uSurfaceIndex]));
		//ASURA_D3D_OK( xResult);
	}

	for ( u_int uSurfaceIndex = 1; uSurfaceIndex < uSurfaceCount; uSurfaceIndex += 1)
	{
		xResult = Asura_Direct3D::D3DDevice->SetRenderTarget( uSurfaceIndex, NULL);
		ASURA_D3D_OK( xResult);
	}

	Asura_Direct3D::D3DDevice->GetDepthStencilSurface(&pxOldDepthBuffer);
	Asura_Direct3D::D3DDevice->SetDepthStencilSurface(NULL);

	#if defined ASURA_TARGET_SM3
		// Ignore sRGB for now
		Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_SRGBWRITEENABLE, FALSE );
	#endif

	// Extract our glow texture from the scene.

	Asura_Glow_System_PC::ExtractGlow(pxCaptureTexture);

	// Blur our glow texture.

	Asura_Glow_System_PC::ApplyBlur();

	// Restore the render target and depth buffer (so they can have the glow effect applied)

	Asura_Direct3D::D3DDevice->SetDepthStencilSurface(pxOldDepthBuffer);
	if(pxOldDepthBuffer)pxOldDepthBuffer->Release();

	// DLH - 14 February 2007 12:33:53 - Restore all the RenderTargets after the first one
	for ( u_int uSurfaceIndex = 0; uSurfaceIndex < uSurfaceCount; uSurfaceIndex += 1)
	{
		Direct3D_Surface* pxSurface = apxSurface[ uSurfaceIndex];
		
		xResult = Asura_Direct3D::D3DDevice->SetRenderTarget( uSurfaceIndex, pxSurface);
		ASURA_D3D_OK( xResult);
		if( NULL != pxSurface)
		{
			pxSurface->Release();
		}
	}

	// Blend the glow texture back over the scene.

	Asura_Glow_System_PC::RenderGlow();

	{
		SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Restore previous state" ));

		// Clean up a few things
		for ( u_int u = 0; u < 4; ++u )
		{
			Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_ADDRESSU, auOldAddressU[u]);
			Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_ADDRESSV, auOldAddressV[u]);
		}

		Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);

		#if defined ASURA_TARGET_SM3
			const bool bSRGB = Asura_Direct3D::UseGammaCorrection();
			Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_SRGBWRITEENABLE, bSRGB );
		#endif
	}

	D3DPERFCALL(EndEvent());
}

/*******************************************************************************/
/* Local function implementations                                              */
/*******************************************************************************/

//////////////////////////////////////////////////////
// CreateRenderTargets
//
// Parameters:    Width and height of render targets
// Return value:  Bool to indicate success
//
// Creates the glow effect's intermediary render target textures
//////////////////////////////////////////////////////
bool Asura_Glow_System_PC::CreateRenderTargets(u_int uRenderTargetWidth, u_int uRenderTargetHeight)
{
	HRESULT hr;

	hr = Asura_Direct3D::D3DDevice->CreateTexture(uRenderTargetWidth, uRenderTargetHeight, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &s_pxGlowTexture, NULL);
	if ( FAILED( hr ) || !s_pxGlowTexture ) return false;

	hr = Asura_Direct3D::D3DDevice->CreateTexture(uRenderTargetWidth, uRenderTargetHeight, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &s_pxHBlurTexture, NULL);
	if ( FAILED ( hr ) || !s_pxHBlurTexture ) return false;

	hr = s_pxGlowTexture->GetSurfaceLevel( 0, &s_pxGlowSurface );
	if ( FAILED ( hr ) || !s_pxGlowSurface ) return false;

	hr = s_pxHBlurTexture->GetSurfaceLevel( 0, &s_pxHBlurSurface );
	if ( FAILED ( hr ) || !s_pxHBlurSurface ) return false;

	s_uUsingRenderTargetWidth = uRenderTargetWidth;
	s_uUsingRenderTargetHeight = uRenderTargetHeight;

	return true;
}

//////////////////////////////////////////////////////
// ReleaseRenderTargets
//
// Parameters:    None
// Return value:  None
//
// Releases the POOL_DEFAULT resources created by CreateRenderTargets
//////////////////////////////////////////////////////
void Asura_Glow_System_PC::ReleaseRenderTargets()
{
	
	if ( s_pxHBlurSurface )
	{
		s_pxHBlurSurface->Release();
		s_pxHBlurSurface = NULL;
	}
	if ( s_pxGlowSurface )
	{
		s_pxGlowSurface->Release();
		s_pxGlowSurface = NULL;
	}
	if ( s_pxHBlurTexture )
	{
		s_pxHBlurTexture->Release();
		s_pxHBlurTexture = NULL;
	}
	if ( s_pxGlowTexture )
	{
		s_pxGlowTexture->Release();
		s_pxGlowTexture = NULL;
	}
}

//////////////////////////////////////////////////////
// GenerateScaledOffsets
//
// Parameters:    None
// Return value:  None
//
// Scales the texel offsets for the pixel shader to account for the size of the backbuffer
//////////////////////////////////////////////////////
void Asura_Glow_System_PC::GenerateScaledOffsets()
{
	for ( u_int u = 0; u < 4; ++u )
	{
		s_afScaledHBlurOffsets[u][0] = (afHBLUR_TEXCOORD_OFFSETS[u][0] + 0.5f) / static_cast<float>(s_uUsingRenderTargetWidth);
		s_afScaledHBlurOffsets[u][1] = (afHBLUR_TEXCOORD_OFFSETS[u][1] + 0.5f) / static_cast<float>(s_uUsingRenderTargetHeight);
		s_afScaledVBlurOffsets[u][0] = (afVBLUR_TEXCOORD_OFFSETS[u][0] + 0.5f) / static_cast<float>(s_uUsingRenderTargetWidth);
		s_afScaledVBlurOffsets[u][1] = (afVBLUR_TEXCOORD_OFFSETS[u][1] + 0.5f) / static_cast<float>(s_uUsingRenderTargetHeight);
		s_afScaledHBlurOffsets[u][2] = s_afScaledHBlurOffsets[u][3] = 0.0f;
		s_afScaledVBlurOffsets[u][2] = s_afScaledVBlurOffsets[u][3] = 0.0f;
	}
}

//////////////////////////////////////////////////////
// ExtractGlow()
//
// Parameters:    Interface to source image (capture)
// Return value:  None
//
// Generates a glow texture from the source image
//////////////////////////////////////////////////////
void Asura_Glow_System_PC::ExtractGlow(Direct3D_Texture* pxSource)
{
	SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Extract glowing areas" ));

	Asura_Assert(pxSource, "Null source texture passed to ExtractGlow");

	Asura_Direct3D::D3DDevice->SetRenderTarget(0, s_pxGlowSurface);

	// Save out the current texture
	Direct3D_BaseTexture* pxOldTexture;
	Asura_Direct3D::D3DDevice->GetTexture(0, &pxOldTexture);

	Asura_Direct3D::D3DDevice->SetTexture(0, pxSource);

	// Draw the quad
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_EXTRACTGLOW);
	Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_EXTRACTGLOW);
	Asura_Direct3D::D3DDevice->SetPixelShaderConstantF(0, &s_afGlowColorParameter[0][0], 3);

	Asura_Direct3D::D3DDevice->SetStreamSource(0, s_pxQuadVB, 0, sizeof(SimpleVertex));
	Asura_Direct3D::D3DDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);

	// Done, clean up

	Asura_Direct3D::D3DDevice->SetTexture(0, pxOldTexture);
	if(pxOldTexture)
	{
		pxOldTexture->Release();
	}
}

//////////////////////////////////////////////////////
// ApplyBlur()
//
// Parameters:    None
// Return value:  None
//
// Blurs the glow texture
//////////////////////////////////////////////////////
void Asura_Glow_System_PC::ApplyBlur()
{
	SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Apply blur" ));

	Direct3D_BaseTexture* pxOldTexture[4];
	DWORD auMinFilter[4], auMagFilter[4];

	{
		SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Save existing state" ));

		// Save out the current texture
		for ( u_int u = 0; u < 4; ++u )
		{
			Asura_Direct3D::D3DDevice->GetTexture(u, &pxOldTexture[u]);
			Asura_Direct3D::D3DDevice->SetTexture(u, s_pxGlowTexture);

			Asura_Direct3D::D3DDevice->GetSamplerState(u, D3DSAMP_MINFILTER, &auMinFilter[u]);
			Asura_Direct3D::D3DDevice->GetSamplerState(u, D3DSAMP_MAGFILTER, &auMagFilter[u]);
			Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
			Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
		}
	}

	{
		SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Horizontal blur" ));

		Asura_Direct3D::D3DDevice->SetRenderTarget(0, s_pxHBlurSurface);

		Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_APPLYBLUR);
		Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_APPLYBLUR);
		Asura_Direct3D::D3DDevice->SetPixelShaderConstantF(0, &afFILTER_WEIGHTS[0][0], 4);
		Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(0, &s_afScaledHBlurOffsets[0][0], 4);

		Asura_Direct3D::D3DDevice->SetStreamSource(0, s_pxQuadVB, 0, sizeof(SimpleVertex));
		Asura_Direct3D::D3DDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);
	}

	{
		SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Vertical blur" ));

		Asura_Direct3D::D3DDevice->SetRenderTarget(0, s_pxGlowSurface);
		for ( u_int u = 0; u < 4; ++u )
		{
			Asura_Direct3D::D3DDevice->SetTexture(u, s_pxHBlurTexture);
		}

		Asura_Direct3D::D3DDevice->SetVertexShaderConstantF(0, &s_afScaledVBlurOffsets[0][0], 4);

		Asura_Direct3D::D3DDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);
	}

	{
		SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Restore previous state" ));

		// Done, clean up
		for ( u_int u = 0; u < 4; ++u )
		{
			Asura_Direct3D::D3DDevice->SetTexture(u, pxOldTexture[u]);
			if(pxOldTexture[u])
			{
				pxOldTexture[u]->Release();
			}
			Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_MINFILTER, auMinFilter[u]);
			Asura_Direct3D::D3DDevice->SetSamplerState(u, D3DSAMP_MAGFILTER, auMagFilter[u]);
		}
	}
}

//////////////////////////////////////////////////////
// RenderGlow()
//
// Parameters:    None
// Return value:  None
//
// Renders the glow over the scene
//////////////////////////////////////////////////////
void Asura_Glow_System_PC::RenderGlow()
{
	SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Render glow onto scene" ) );

	// Save out the current texture
	Direct3D_BaseTexture* pxOldTexture;
	Asura_Direct3D::D3DDevice->GetTexture(0, &pxOldTexture);
	Asura_Direct3D::D3DDevice->SetTexture(0, s_pxGlowTexture);

	Asura_Direct3D_Shaders::RequestVertexShader(ASURA_D3D_VS_EXTRACTGLOW);
	Asura_Direct3D_Shaders::RequestPixelShader(ASURA_D3D_PS_NONE);
	
	Asura_Direct3D::D3DDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE2X );
	Asura_Direct3D::D3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_TEXTURE     );
	Asura_Direct3D::D3DDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TFACTOR     );

	Asura_Direct3D::D3DDevice->SetTextureStageState( 1, D3DTSS_COLOROP,   D3DTOP_DISABLE    );

	u_char ucLevel = static_cast< u_char >( Asura_Maths::FloatToInt_Chop(Asura_Glow_System::GetGlowLevel() * 192.f) & 0xff );
	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_TEXTUREFACTOR, (ucLevel) | (ucLevel<<8) | (ucLevel<<16) | (ucLevel<<24) );

	Asura_Render::CurrentStates.RequestTextureIndex(1, ASURA_TEXTURE_NONE);
	Asura_Render::CurrentStates.RequestTextureIndex(2, ASURA_TEXTURE_NONE);
	Asura_Render::CurrentStates.RequestTextureIndex(3, ASURA_TEXTURE_NONE);

	DWORD uOldAlphaBlend, uOldSrcBlend, uOldDestBlend;

	{
		SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Save existing state" ));

		// There is no Asura blend op for additive blending, so we do it ourselves.
		Asura_Direct3D::D3DDevice->GetRenderState(D3DRS_ALPHABLENDENABLE,	&uOldAlphaBlend			);
		Asura_Direct3D::D3DDevice->GetRenderState(D3DRS_SRCBLEND,			&uOldSrcBlend			);
		Asura_Direct3D::D3DDevice->GetRenderState(D3DRS_DESTBLEND,			&uOldDestBlend			);

		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,	TRUE					);
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_SRCBLEND,			D3DBLEND_INVDESTCOLOR	);
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_DESTBLEND,			D3DBLEND_ONE			);
	}

	Asura_Direct3D::D3DDevice->SetStreamSource(0, s_pxQuadVB, 0, sizeof(SimpleVertex));
	Asura_Direct3D::D3DDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);

	{
		SCOPEDPIXEVENT(( 0xFFFFFFFF, L"Restore previous state" ));

		// Done, clean up
		Asura_Direct3D::D3DDevice->SetTexture(0, pxOldTexture);
		if(pxOldTexture)
		{
			pxOldTexture->Release();
		}
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,	uOldAlphaBlend			);
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_SRCBLEND,			uOldSrcBlend			);
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_DESTBLEND,			uOldDestBlend			);
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_WHITE);
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_OPAQUE);
	}
}
