
//
// Asura_Platform_PFX_FastPath.cpp
//
// Created by DMR 15/11/06
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
// Faster renderer for most PFX types, using hardware instancing
//

#include "Core/Asura.h"


#ifdef ASURA_USE_FASTPATH_PFX

// This file's header.
#include "SpecialFX/Asura_Platform_PFX_System_FastPath.h"

// Asura platform-independent headers
#include "Camera/Asura_Camera.h"
#include "Core/Asura_HalfFloat.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Debugging/Asura_Logfile.h"
#include "Interface/Asura_CommandConsole.h"
#include "Light/Asura_GI_Solution.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "SpecialFX/PFX/Asura_PFX_Classes.h"
#include "SpecialFX/PFX/Asura_PFX_Effect.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "SpecialFX/PFX/Asura_PFX_Templates.h"

// Asura PC headers
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_Fog.h"

// Tools headers
#ifdef ASURA_TOOLS
#include "Rendering/Asura_Text_Output.h"
#include "Particles/Tools_Mode_PFX.h"
#endif

#if defined( ASURA_TOOLS)
#include "Misc/Tools_Render_Options.h"
#endif

bool								 Asura_Platform_PFX_System_FastPath::s_bIsEnabled = true;
Direct3D_VertexDeclaration*		     Asura_Platform_PFX_System_FastPath::s_pxPFXVertexDecl = NULL;
Asura_Direct3D_Effect*			     Asura_Platform_PFX_System_FastPath::s_pxPFXEffect = NULL;

D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xTextureHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xDepthTextureHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xFirstInstanceHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xUseLightingHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xCrossfadeTextureFramesHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xZReadEnableHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xXDeltaHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xYDeltaHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xLightColourHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xVolumetricDepthScaleHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xPosOffsetAndSizeScaleHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xNearFarFadeHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xTwoOverScreenWidthHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xTwoOverScreenHeightHandle = 0;
D3DXHANDLE							 Asura_Platform_PFX_System_FastPath::s_xIsVolumetricHandle = 0;

static Asura_Direct3D_FogEffectParameters   g_xFogEffectParameters;

Direct3D_VertexBuffer			    *Asura_Platform_PFX_System_FastPath::s_pxVertexBuffer = NULL;
Direct3D_VertexBuffer				*Asura_Platform_PFX_System_FastPath::s_pxVertexDataBuffer = NULL;
Direct3D_IndexBuffer				*Asura_Platform_PFX_System_FastPath::s_pxIndexBuffer = NULL;

uint32							 	 Asura_Platform_PFX_System_FastPath::s_uFastPathParticlesRenderedThisFrame;
float								 Asura_Platform_PFX_System_FastPath::s_fApproxScreenfullsRenderedThisFrame = 0.0f;
bool								 Asura_Platform_PFX_System_FastPath::s_bVertexBufferLocked = false;
bool								 Asura_Platform_PFX_System_FastPath::s_bVertexBufferOverflow = false;
int								 	 Asura_Platform_PFX_System_FastPath::s_iCurrentVertexBufferPosition = 0;
Asura_Platform_PFXInstanceData	    *Asura_Platform_PFX_System_FastPath::s_pxCurrentVertexBuffer=0;



static bool							 g_bPFXOutlinesEnabled = false;


// Decls/Effects/Handles for the new (fast path) instanced PFX renderer
// ------------------------------------------------------------------------------------------

// This is for the geometry data. Just 4 verts that will be instanced.
//
// This vertex format looks bloated - but the cost of converting floats->ints and floats->halfs 
// was very severe - at the moment, this seems to give better results.
//
const Direct3D_VertexElement axINSTANCED_VERTEX_DECLARATION[] =
{
	// VERTEX STREAM
	{ 0,  0, D3DDECLTYPE_FLOAT1,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 4 },	// Vertex Index

	// INSTANCE STREAM
	{ 1,  0, D3DDECLTYPE_FLOAT4,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },	// Position (Orientation in W)
	{ 1, 16, D3DDECLTYPE_FLOAT4,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },	// UVset 0 (U0,V0,U1,V1)
	{ 1, 32, D3DDECLTYPE_FLOAT4,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 1 },  // UVset 1 (U0,V0,U1,V1)
	{ 1, 48, D3DDECLTYPE_FLOAT2,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 2 },	// Size
	{ 1, 56, D3DDECLTYPE_FLOAT2,	D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 3 },	// TextureAlignment and CrossfadeBlendWeight
	{ 1, 64, D3DDECLTYPE_FLOAT4,    D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },	// Colour+Alpha (without near-far fadeout)

	D3DDECL_END()
};



void Asura_Platform_PFX_System_FastPath::CaptureEffect(void)
{
	#define LOADPARAMETER(codeName, shaderName) { codeName = s_pxPFXEffect->GetD3DEffect()->GetParameterByName(NULL, shaderName); Asura_Assert(codeName, "Could not get shader parameter '" shaderName "'."); }
	
	s_pxPFXEffect = Asura_Direct3D_Effects_System::GetEffect("PFX_FastPath");	

	if (!s_pxPFXEffect)
	{
		return;
	}

#if ASURA_LOAD_EFFECT_FILES == 1
	s_pxPFXEffect->SetOnLoadCallback(OnEffectLoaded);
	s_pxPFXEffect->SetOnUnloadCallback(OnEffectUnloaded);
#endif	

	LOADPARAMETER(s_xTextureHandle				 , "g_xTexture" );
	LOADPARAMETER(s_xDepthTextureHandle			 , "g_xDepthTexture" );
	LOADPARAMETER(s_xUseLightingHandle			 , "g_bUseLighting" );
	LOADPARAMETER(s_xFirstInstanceHandle		 , "g_fFirstInstance" );
	LOADPARAMETER(s_xCrossfadeTextureFramesHandle, "g_bCrossfadeTextureFrames" );
	LOADPARAMETER(s_xZReadEnableHandle			 , "g_bZReadEnable" );	
	LOADPARAMETER(s_xXDeltaHandle			 	 , "g_xXDelta" );
	LOADPARAMETER(s_xYDeltaHandle				 , "g_xYDelta" );
	LOADPARAMETER(s_xNearFarFadeHandle			 , "g_xNearFarFade" );
	LOADPARAMETER(s_xLightColourHandle			 , "g_xLightColour" );
	LOADPARAMETER(s_xVolumetricDepthScaleHandle	 , "g_fVolumetricDepthScale" );
	LOADPARAMETER(s_xPosOffsetAndSizeScaleHandle , "g_xPosOffsetAndSizeScale" );
	LOADPARAMETER(s_xNearFarFadeHandle			 , "g_xNearFarFade" );
	LOADPARAMETER(s_xTwoOverScreenWidthHandle	 , "g_fTwoOverScreenWidth");
	LOADPARAMETER(s_xTwoOverScreenHeightHandle	 , "g_fTwoOverScreenHeight");
	LOADPARAMETER(s_xIsVolumetricHandle			 , "g_bVolumetric");	

	g_xFogEffectParameters.Capture(s_pxPFXEffect);
}


void Asura_Platform_PFX_System_FastPath::Initialise( void )
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}
	
	HRESULT xResult = Asura_Direct3D::D3DDevice->CreateVertexDeclaration( axINSTANCED_VERTEX_DECLARATION, &s_pxPFXVertexDecl );
	(void) xResult;
	Asura_Assert( SUCCEEDED( xResult ) && s_pxPFXVertexDecl, "Failed to create PFX (fast path) vertex declaration." );	

	CaptureEffect();
	
	PreD3DReset();
	PostD3DReset();

#if defined ASURA_SUPPORT_COMMAND_CONSOLE
	Asura_CommandConsole::AddVar( L"Render.PFX.EnableFastPath", &s_bIsEnabled, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );

#if defined ASURA_TOOLS && defined ASURA_TOOLS_SHOW_PFX_COST
	Asura_CommandConsole::AddVar( L"Debug.PFXPerformance", &g_bPFXOutlinesEnabled, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif
#endif
}


#if ASURA_LOAD_EFFECT_FILES == 1
void Asura_Platform_PFX_System_FastPath::OnEffectLoaded(Asura_Direct3D_Effect* pxEffect)
{
	CaptureEffect();
}

void Asura_Platform_PFX_System_FastPath::OnEffectUnloaded(Asura_Direct3D_Effect* pxEffect)
{
	s_pxPFXEffect = NULL;	
}
#endif

void Asura_Platform_PFX_System_FastPath::Shutdown( void )
{
	if (s_pxPFXVertexDecl)
	{	
		s_pxPFXVertexDecl->Release();
		s_pxPFXVertexDecl = NULL;
	}
	
	// Effect system destroys the shader
	s_pxPFXEffect = NULL;

	PreD3DReset();
}

void Asura_Platform_PFX_System_FastPath::PreD3DReset( void)
{
	if (s_pxVertexBuffer)
	{
		s_pxVertexBuffer->Release();
		s_pxVertexBuffer = NULL;
	}

	if (s_pxIndexBuffer)
	{
		s_pxIndexBuffer->Release();
		s_pxIndexBuffer=NULL;
	}

	if (s_pxVertexDataBuffer)
	{
		s_pxVertexDataBuffer->Release();
		s_pxVertexDataBuffer=NULL;
	}
}

void Asura_Platform_PFX_System_FastPath::PostD3DReset( void)
{
	HRESULT xResult;

	Asura_Assert( NULL == s_pxVertexBuffer, "VB already created");	
	xResult = Asura_Direct3D::D3DDevice->CreateVertexBuffer(sizeof(Asura_Platform_PFXInstanceData)*VERTEXBUFFER_SIZE, D3DUSAGE_DYNAMIC | D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, &s_pxVertexBuffer, NULL);
	Asura_Assert( SUCCEEDED( xResult ) && s_pxVertexBuffer, "Failed to create PFX (fast path) vertex buffer." );
	
	s_bVertexBufferLocked			= false;

	// Create and fill the buffers for the vertex data for each instance. These
	// don't change.

	// The vertex data buffer
	Asura_Assert( NULL == s_pxVertexDataBuffer, "VB already created");	
	xResult = Asura_Direct3D::D3DDevice->CreateVertexBuffer(4*sizeof(Asura_Platform_PFX_Vertex), D3DUSAGE_WRITEONLY,0, D3DPOOL_DEFAULT,&s_pxVertexDataBuffer,NULL);
	Asura_Assert( SUCCEEDED( xResult ) , "Failed to create PFX (fast path) vertex buffer." );

	Asura_Platform_PFX_Vertex *pxData = NULL;
	xResult = s_pxVertexDataBuffer->Lock(0,0, (void**)&pxData,0);
	Asura_Assert( SUCCEEDED( xResult ) , "Failed to lock PFX (fast path) vertex buffer." );
	if( SUCCEEDED( xResult ) && NULL != pxData)
	{
		for(int i=0;i<4;i++)
		{
			pxData[i].fVertexIndex = (float)i;
		}
		s_pxVertexDataBuffer->Unlock();
	}

	// The index data buffer
	Asura_Assert( NULL == s_pxIndexBuffer, "IB already created");	
	xResult = Asura_Direct3D::D3DDevice->CreateIndexBuffer(6*2, D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_DEFAULT,&s_pxIndexBuffer,NULL);
	Asura_Assert( SUCCEEDED( xResult ) , "Failed to create PFX (fast path) index buffer." );

	// 360 has quad lists - on PC we've got to use tri lists. Which is fine as we
	// can only do hardware instancing with indexed data, anyway.
	uint16 *pxIndexData;
	xResult = s_pxIndexBuffer->Lock(0,0, (void**)&pxIndexData,0);
	Asura_Assert( SUCCEEDED( xResult ) , "Failed to lock PFX (fast path) index buffer." );
	if( SUCCEEDED( xResult ) && NULL != pxIndexData)
	{
		pxIndexData[0] = 0;
		pxIndexData[1] = 1;
		pxIndexData[2] = 3;
		pxIndexData[3] = 1;
		pxIndexData[4] = 2;
		pxIndexData[5] = 3;
		s_pxIndexBuffer->Unlock();	
	}
}

void Asura_Platform_PFX_System_FastPath::UpdateInstanceBuffersForNewFrame()
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}

	SCOPEDPIXEVENT(( D3DCOLOR_XRGB( 0, 0, 0 ), L"PFX: Update instance buffers for new frame" ));

	if (s_bVertexBufferLocked)
	{
		s_bVertexBufferLocked=false;
		s_pxVertexBuffer->Unlock();		
	}
	
	Asura_Direct3D::D3DDevice->SetStreamSource(0,NULL,0,0);

	s_iCurrentVertexBufferPosition = 0;

	HRESULT xResult = s_pxVertexBuffer->Lock(0,0,(void **)&s_pxCurrentVertexBuffer,D3DLOCK_DISCARD);

	// JAT 19/01/07 fix a warning when asserts are turned off
	//(void)xResult;

	Asura_Assertf( SUCCEEDED( xResult ) && s_pxCurrentVertexBuffer, ("Failed to lock PFX (fast path) vertex buffer (0x%08x).",  xResult)  );

	// SB (08/04/08) Fix occasional crashes when buffer is not lockable
	if ( SUCCEEDED( xResult ) && s_pxCurrentVertexBuffer )
	{
		s_bVertexBufferLocked	= true;
		s_uFastPathParticlesRenderedThisFrame = 0;
		s_fApproxScreenfullsRenderedThisFrame = 0.0f;
		s_bVertexBufferOverflow = false;
	}
	else
	{
		s_pxCurrentVertexBuffer = NULL;
		s_bVertexBufferLocked = false;
		s_bVertexBufferOverflow = false;
		s_uFastPathParticlesRenderedThisFrame = 0;
		s_fApproxScreenfullsRenderedThisFrame = 0.0f;
	}
}



void *Asura_Platform_PFX_System_FastPath::GetInstanceBufferArea(uint32 uNumParticles)
{
	if( NULL == s_pxCurrentVertexBuffer)
	{
		return( NULL);
	}
	
	if (s_iCurrentVertexBufferPosition+uNumParticles<VERTEXBUFFER_SIZE)
	{
		int iPos = s_iCurrentVertexBufferPosition;
		s_iCurrentVertexBufferPosition+=uNumParticles;
		return (void *)&s_pxCurrentVertexBuffer[iPos];
	}
	else
	{		
		s_bVertexBufferOverflow = true;
	}

	return NULL;
}



void Asura_Platform_PFX_System_FastPath::RenderParticles(				
	const  Asura_PFX_Effect *pxEffect,
	const  Asura_PFX_Particle* pxParticles,
	u_int  uNumParticles,
	const  Asura_PFX_Particle_Template* pxTemplate,
	const  Asura_RGBA& xLightColour,
	const  uint16 usPFXFlags,														 
	const  uint16 usCameraFlags, 
	void* pPlatformRenderData
	)
{	

	if (!s_pxPFXEffect)
	{
		CaptureEffect();
	}

	if (!s_pxPFXEffect)
	{
		return;
	}
	if (!s_pxCurrentVertexBuffer)
	{
		return;
	}
	
	if (s_bVertexBufferLocked)
	{
		s_bVertexBufferLocked=false;
		s_pxVertexBuffer->Unlock();		
	}		


	// Is this effect visible from any cameras? If not, return now before we start setting renderstates
	int iCamCount = 0;
	for( int iCam=0; iCam<Asura_Camera::GetNumberOfCameras(); ++iCam )
	{
		if((usCameraFlags & (1<<iCam)))
		{
			iCamCount++;
		}
	}

	if (iCamCount==0)
	{
		return;
	}


	Asura_Platform_PFXInstanceData *pxCurrentPos = (Asura_Platform_PFXInstanceData *)pPlatformRenderData;

	uint32 uFirstInstance = pxCurrentPos-s_pxCurrentVertexBuffer;
		
	// Set the blending mode
	if ( usPFXFlags & ASURA_PFX_EFFECT_WATER_DISTORTER )
	{
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);		
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_ALPHA);
	}
	else
	{
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
		Asura_Render::CurrentStates.RequestTranslucencyMode(pxTemplate->m_eTranslucency);
	}
	

	// SHADER CONSTANT SETUP - Could be optimized by batching these
	// ---------------------------------------------------------------------------------
	
	Direct3D_BaseTexture* pxTexture = Asura_Direct3D::GetDirect3DTextureByIndex(pxTemplate->m_iTextureHandle);
	
	Asura_Assert( NULL != pxTexture, "Particle Texture not found");

	if( NULL != pxTexture)
	{
		s_pxPFXEffect->GetD3DEffect()->SetTexture( s_xTextureHandle, pxTexture );
	}

	bool bAlwaysOnTop = (usPFXFlags & ASURA_PFX_EFFECT_ALWAYS_ON_TOP) != 0;

		
	s_pxPFXEffect->GetD3DEffect()->SetBool( s_xZReadEnableHandle, !bAlwaysOnTop );
	s_pxPFXEffect->GetD3DEffect()->SetBool( s_xUseLightingHandle, pxTemplate->m_bUseLighting );
	s_pxPFXEffect->GetD3DEffect()->SetBool( s_xCrossfadeTextureFramesHandle, pxTemplate->m_bCrossfadeTextureFrames );

	// Set the depth-normal texture
	s_pxPFXEffect->GetD3DEffect()->SetTexture(s_xDepthTextureHandle, Asura_Direct3D::GetDeferredRenderingTarget()->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX));
	
	if(pxTemplate->m_bUseLighting)
	{
		s_pxPFXEffect->GetD3DEffect()->SetFloatArray(s_xLightColourHandle,(float *)&xLightColour,4);
	}
	else
	{
		float afOne[4] = {1.0f,1.0f,1.0f,1.0f};
		s_pxPFXEffect->GetD3DEffect()->SetFloatArray(s_xLightColourHandle,afOne,4);
	}	

	s_pxPFXEffect->GetD3DEffect()->SetFloat(s_xVolumetricDepthScaleHandle,pxTemplate->m_fVolumetricDepthScale);	

	s_pxPFXEffect->GetD3DEffect()->SetFloat(s_xTwoOverScreenWidthHandle,  (2.0f / Asura_Viewport::GetWidth()));
	s_pxPFXEffect->GetD3DEffect()->SetFloat(s_xTwoOverScreenHeightHandle, (2.0f / Asura_Viewport::GetHeight()));


	// Apply the fogging parameters
	g_xFogEffectParameters.Apply(pxTemplate->m_bUseFogging, pxTemplate->m_eTranslucency );	 


	for( int iCam=0; iCam<Asura_Camera::GetNumberOfCameras(); ++iCam )
	{
		if(!(usCameraFlags & (1<<iCam))) continue;

		Asura_Camera::SetCameraIndex( iCam );		
		if(Asura_Camera::IsCameraActive())
		{
			Asura_Render::ApplyCameraSettings();

			// Cull if all particles are faded out.
			// A bit late really, as we've already done the update and VB filling...
			if (pxTemplate->m_bFarCameraFade)
			{	
				Asura_Vector_3 xCentre = pxEffect->GetBoundingBox().CalculateCentre();
				Asura_Vector_3 xCam = *Asura_Camera::GetPosition();			
				xCentre = xCentre - xCam; 
				float fR = pxEffect->GetBoundingRadius();

				if (xCentre.MagnitudeSquared() > 
					(pxTemplate->m_fFarZeroDistance+fR)*(pxTemplate->m_fFarZeroDistance+fR))
				{			
					// Effect is completely faded out in it's current position
					continue;
				}						
			}


			// Camera-specific shader constants
			// ------------------------------------------------------------------------------------------

			float axPositionOffsetAndSizeScale[4];
			if ( usPFXFlags & ASURA_PFX_EFFECT_SKYBOX )
			{
				const	Asura_Vector_3 *pxCameraPos	= Asura_Camera::GetPosition();
				axPositionOffsetAndSizeScale[0] = pxCameraPos->x;
				axPositionOffsetAndSizeScale[1] = pxCameraPos->y;
				axPositionOffsetAndSizeScale[2] = pxCameraPos->z;
			}
			else
			{
				axPositionOffsetAndSizeScale[0] =
				axPositionOffsetAndSizeScale[1] =
				axPositionOffsetAndSizeScale[2] = 0.0f;
			}

			if( pxEffect && pxTemplate->m_bUseSizeScale )
			{		
				axPositionOffsetAndSizeScale[3] = pxEffect->GetSizeScale();
			}
			else
			{
				axPositionOffsetAndSizeScale[3] = 1.0f;
			}

			s_pxPFXEffect->GetD3DEffect()->SetFloatArray(s_xPosOffsetAndSizeScaleHandle, (float *)axPositionOffsetAndSizeScale, 4);

			float axNearFarFade[4];
			if(pxTemplate->m_bNearCameraFade || pxTemplate->m_bFarCameraFade)
			{
				axNearFarFade[0] = pxTemplate->m_fNearFadeDistance;
				axNearFarFade[1] = pxTemplate->m_fNearZeroDistance;
				axNearFarFade[2] = pxTemplate->m_fFarFadeDistance;
				axNearFarFade[3] = pxTemplate->m_fFarZeroDistance;
			}
			else
			{
				axNearFarFade[0] = -10000.0f;
				axNearFarFade[1] = -10001.0f;
				axNearFarFade[2] = 10000.0f;
				axNearFarFade[3] = 10001.0f;
			}

			s_pxPFXEffect->GetD3DEffect()->SetFloatArray(s_xNearFarFadeHandle, (float *)axNearFarFade, 4);


			Asura_Vector_3		xDeltaX;
			Asura_Vector_3		xDeltaY;
			const Asura_Matrix_3x3* pxOrientation = pxEffect->GetOrientation();

			if(pxTemplate->m_bAlignEffectX && pxOrientation)
			{
				xDeltaX = Asura_Vector_3(pxOrientation->e12, pxOrientation->e22, pxOrientation->e32);
				xDeltaY = Asura_Vector_3(pxOrientation->e13, pxOrientation->e23, pxOrientation->e33);
			}
			else if(pxTemplate->m_bAlignEffectY && pxOrientation)
			{
				xDeltaX = Asura_Vector_3(pxOrientation->e11, pxOrientation->e21, pxOrientation->e31);
				xDeltaY = Asura_Vector_3(pxOrientation->e13, pxOrientation->e23, pxOrientation->e33);
			}
			else if(pxTemplate->m_bAlignEffectZ && pxOrientation)
			{
				xDeltaX = Asura_Vector_3(pxOrientation->e11, pxOrientation->e21, pxOrientation->e31);
				xDeltaY = Asura_Vector_3(pxOrientation->e12, pxOrientation->e22, pxOrientation->e32);
			}
			else if(pxTemplate->m_bBillboardJustY && pxOrientation)
			{
				const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();
				xDeltaX = Asura_Vector_3(pxCameraOrientation->e11,pxCameraOrientation->e21,pxCameraOrientation->e31);
				xDeltaY = Asura_Vector_3(pxOrientation->e13, pxOrientation->e23, pxOrientation->e33);
			}
			else if(pxTemplate->m_bBillboardJustX && pxOrientation)
			{
				const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();
				xDeltaX = Asura_Vector_3(pxOrientation->e12, pxOrientation->e22, pxOrientation->e32);
				xDeltaY = Asura_Vector_3(pxCameraOrientation->e12,pxCameraOrientation->e22,pxCameraOrientation->e32);
			}
			else
			{
				// Normal particles - screen-aligned
				const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();
				xDeltaX = Asura_Vector_3(pxCameraOrientation->e11,pxCameraOrientation->e21,pxCameraOrientation->e31);
				xDeltaY = Asura_Vector_3(pxCameraOrientation->e12,pxCameraOrientation->e22,pxCameraOrientation->e32);
			}

			// Volumetric on/off?
			s_pxPFXEffect->GetD3DEffect()->SetBool(s_xIsVolumetricHandle, (pxTemplate->m_fVolumetricDepthScale>0.0f));

			s_pxPFXEffect->GetD3DEffect()->SetFloatArray(s_xXDeltaHandle,(float *)&xDeltaX,3);
			s_pxPFXEffect->GetD3DEffect()->SetFloatArray(s_xYDeltaHandle,(float *)&xDeltaY,3);

			// Set up the geometry data stream
			Asura_Direct3D::D3DDevice->SetStreamSourceFreq(0,(D3DSTREAMSOURCE_INDEXEDDATA | uNumParticles));
			Asura_Direct3D::D3DDevice->SetStreamSource(0, s_pxVertexDataBuffer, 0, sizeof(Asura_Platform_PFX_Vertex));

			// Set up the instance data stream
			Asura_Direct3D::D3DDevice->SetStreamSourceFreq(1, (D3DSTREAMSOURCE_INSTANCEDATA | 1ul) );
			Asura_Direct3D::D3DDevice->SetStreamSource(1, s_pxVertexBuffer, sizeof(Asura_Platform_PFXInstanceData) * uFirstInstance , sizeof(Asura_Platform_PFXInstanceData));

			// Set the instance stream
			Asura_Direct3D::D3DDevice->SetIndices(s_pxIndexBuffer);

			// Set the decl
			Asura_Direct3D::D3DDevice->SetVertexDeclaration(s_pxPFXVertexDecl);

			// Make sure the depth-normal buffer is no longer set as a target, as we're using it as a source
			Asura_Direct3D::ClearRenderTargets( 1);

#if defined( ASURA_TOOLS)		
			if (!Tools_Render_Options::GetWireframeMode()==TOOLS_WIREFRAME_ON)
#endif
			{		
				Asura_Direct3D_Effects_System::BeginEffect(*s_pxPFXEffect);
				Asura_Assert(s_pxPFXEffect->IsD3DEffectValid(), "PFX (fast path) effect invalid");
				if (s_pxPFXEffect->IsD3DEffectValid())
				{
					s_pxPFXEffect->GetD3DEffect()->BeginPass(0);
					s_pxPFXEffect->GetD3DEffect()->CommitChanges();

					Asura_Direct3D::D3DDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0, 0, 4, 0, 2 );


					s_pxPFXEffect->GetD3DEffect()->EndPass();

					// Flush any effects we may still be using
					Asura_Direct3D_Effects_System::FlushEffects	();
				}
			}


#if defined( ASURA_TOOLS)		
			if (Tools_Render_Options::GetWireframeMode()==TOOLS_WIREFRAME_ON || Tools_Render_Options::GetWireframeMode()==TOOLS_WIREFRAME_MIXED || g_bPFXOutlinesEnabled)
			{
				Asura_Direct3D_Effects_System::BeginEffect(*s_pxPFXEffect);
				Asura_Assert(s_pxPFXEffect->IsD3DEffectValid(), "PFX (fast path) effect invalid");
				if (s_pxPFXEffect->IsD3DEffectValid())
				{
					Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);		
					Asura_Render::CurrentStates.RequestWireFrameMode(true);
					s_pxPFXEffect->GetD3DEffect()->SetTexture( s_xTextureHandle, Asura_Direct3D::GetUntexturedTexture());

					s_pxPFXEffect->GetD3DEffect()->BeginPass(0);
					s_pxPFXEffect->GetD3DEffect()->CommitChanges();

					Asura_Direct3D::D3DDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0, 0, 4, 0, 2 );


					s_pxPFXEffect->GetD3DEffect()->EndPass();

					Asura_Render::CurrentStates.RequestWireFrameMode(false);				

					// Flush any effects we may still be using
					Asura_Direct3D_Effects_System::FlushEffects	();
				}
			}
#endif


#ifdef ASURA_TOOLS					
			Tools_Mode_PFX::AddToParticlesRendered(uNumParticles);
#endif
			Asura_Render::AddToParticlesRendered( uNumParticles );		
			s_uFastPathParticlesRenderedThisFrame += uNumParticles;		
		}
	}  // for each camera	


	// Reset renderstates.	
	s_pxPFXEffect->GetD3DEffect()->SetTexture(s_xDepthTextureHandle, 0);

	if ( usPFXFlags & ASURA_PFX_EFFECT_WATER_DISTORTER )
	{
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA);
	}

	// Optimization - only do this after rendering all fast-path effects, instead of per-effect?

	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);		
	Asura_Direct3D::D3DDevice->SetStreamSourceFreq(0,1);
	Asura_Direct3D::D3DDevice->SetStreamSource(0,NULL,0,0);
	Asura_Direct3D::D3DDevice->SetStreamSourceFreq(1,1);
	Asura_Direct3D::D3DDevice->SetStreamSource(1,NULL,0,0);
	Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);

	Asura_Render::CurrentStates.RequestAlphaTest( false );

	for( u_int u = 0; u < 16; ++u )
	{
		// The .fx file changes these, for sampling from the depth texture - so restore them to
		// defaults
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_SRGBTEXTURE, FALSE );
	}
}





// UpdateInstanceBuffer - This has been separated out of the render call, as it can be 
// run multithreaded
// ------------------------------------------------------------------------------------------				

void Asura_Platform_PFX_System_FastPath::UpdateInstanceBuffer(const Asura_PFX_Particle_Template *pxTemplate, 
															  const Asura_PFX_Particle			*pxParticles, 
															  Asura_Platform_PFXInstanceData	*pxParticleInstanceData, 
															  uint32 uNumParticles)
{		
	if( 0 == uNumParticles)
	{
		return;
	}

	Asura_Assert( NULL != pxTemplate, "Template not setup");
	if( NULL == pxTemplate)
	{
		return;
	}
	
	Asura_Assert( NULL != pxParticles, "Particles not setup");
	if( NULL == pxParticles)
	{
		return;
	}
	
	Asura_Assert( NULL != pxParticleInstanceData, "Particles Instance Data not setup");
	if( NULL == pxParticleInstanceData)
	{
		return;
	}
	
	SCOPEDPIXEVENT((0xff8800, L"Asura_Platform_PFX_System_FastPath::UpdateInstanceBuffer"));
	
	for(u_int uParticle = 0 ; uParticle < uNumParticles ; ++uParticle)
	{
		const	Asura_PFX_Particle*	pxParticle = &pxParticles[uParticle];
		Asura_Assert( NULL != pxParticle, "Particle not setup");
		
		Asura_Vector_3		xPosition	= pxParticle->m_xPosition;

		Asura_RGBA xColour = pxTemplate->m_xColour.GetValueAtTime_Discrete(pxParticle->m_fAge);			

		float fSize = pxTemplate->m_xSize.GetValueAtTime_Discrete(pxParticle->m_fAge);

		fSize *= (1.f - pxParticle->m_fSeed * pxTemplate->m_xSizeVariation.GetValueAtTime_Discrete(pxParticle->m_fAge));


		// All of this lot just calculates fTheta - the rotation angle
		float fTheta = 0.0f;
		if(pxTemplate->m_bFixedRotation)
		{
			fTheta = Asura_Maths::DegToRad(pxTemplate->m_fFixedRotation);
			if(pxTemplate->m_uNumFixedRotations > 1)
			{
				int iRotation = Asura_Maths::FloatToInt_Chop(pxParticle->m_fSeed * pxTemplate->m_uNumFixedRotations);
				fTheta += (iRotation * Asura_Maths::Pi * 2.f) / pxTemplate->m_uNumFixedRotations;
			}
		}
		else 
		{
			// Need a rotation
			if(pxTemplate->m_bUseRandomRotation)
			{
				fTheta = ((Asura_Timers::GetGameTimeScale() == 0.0f) ? 0.0f : Asura_Random::GetFloat()) * Asura_Maths::Pi * 2.0f;
			}
			else
			{
#ifdef ASURA_TOOLS
				fTheta = (pxParticle->m_fSeed + pxTemplate->m_xRotation.GetAreaUnderGraph(pxParticle->m_fAge)) * Asura_Maths::Pi * 2.f;
#else
				fTheta = (pxParticle->m_fSeed + pxTemplate->m_xRotation.GetValueAtTime_Discrete(pxParticle->m_fAge)) * Asura_Maths::Pi * 2.f;
#endif
			}

			if(pxTemplate->m_bMirrorRotation)
			{
				const int* piRandom = reinterpret_cast<const int*>(&pxParticle->m_fSeed);
				if(*piRandom & 1)
				{
					fTheta = -fTheta;
				}
			}
		}


		// are we changing texture orient?
		u_int uTexOrient = 0;
		float fCrossFadeBlendWeight = 0.0f;

		if(pxTemplate->m_bRandomTextureOrient)
		{
			uTexOrient = Asura_Maths::FloatToInt_Chop(pxParticle->m_fSeed * 4.f);
		}

		// Get the texture frame(s) for this particle
		const Asura_PFX_TextureFrame* pxTextureFrame;
		const Asura_PFX_TextureFrame* pxTextureFrame2;
		if (pxTemplate->m_bCrossfadeTextureFrames)
		{
			pxTemplate->GetTextureFramesForCrossfadeAtTime(pxParticle->m_fAge, pxTextureFrame, pxTextureFrame2, fCrossFadeBlendWeight);
		}
		else
		{
			fCrossFadeBlendWeight = 0.0f;
			pxTextureFrame = pxTemplate->GetTextureFrameAtTime(pxParticle->m_fAge);
			pxTextureFrame2 = pxTextureFrame; 
		}


		pxParticleInstanceData->X = xPosition.x;
		pxParticleInstanceData->Y = xPosition.y;
		pxParticleInstanceData->Z = xPosition.z;
		pxParticleInstanceData->Orientation = fTheta;
		pxParticleInstanceData->TextureAlignment = (float)uTexOrient;		

		pxParticleInstanceData->Width  = fSize;
		pxParticleInstanceData->Height = fSize;

		if ( pxTemplate->m_bUseUVAsAspect )
		{
			const float fDU = pxTextureFrame->m_afTextureUVs[ 2 ] - pxTextureFrame->m_afTextureUVs[ 0 ];
			const float fDV = pxTextureFrame->m_afTextureUVs[ 3 ] - pxTextureFrame->m_afTextureUVs[ 1 ];
			const float fAspect = ( fDV ) ? fDU / fDV : 1.0f;
			pxParticleInstanceData->Width *= fAspect;
		}

		for(int i=0;i<4;i++)
		{				
			pxParticleInstanceData->UVSet0[i] = pxTextureFrame->m_afTextureUVs[i];
			pxParticleInstanceData->UVSet1[i] = pxTextureFrame2->m_afTextureUVs[i];
		}

		pxParticleInstanceData->CrossFadeBlendWeight = fCrossFadeBlendWeight;
		pxParticleInstanceData->Diffuse[0] = xColour.m_fR;
		pxParticleInstanceData->Diffuse[1] = xColour.m_fG;
		pxParticleInstanceData->Diffuse[2] = xColour.m_fB;
		pxParticleInstanceData->Diffuse[3] = xColour.m_fA;

#ifdef ASURA_TOOLS
#ifdef ASURA_TOOLS_SHOW_PFX_COST
		// Only do this for billboarded PFX
		if (pxParticleInstanceData->Diffuse[3]>0 && !(pxTemplate->m_bAlignEffectX || pxTemplate->m_bAlignEffectY || pxTemplate->m_bAlignEffectZ))
		{	
			// Calculate screen-space area of this particle
			Asura_Vector_3 xP1, xP2;

			const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();
			Asura_Vector_3 xRight = Asura_Vector_3(pxCameraOrientation->e11,pxCameraOrientation->e21,pxCameraOrientation->e31);
			Asura_Vector_3 xUp    = Asura_Vector_3(pxCameraOrientation->e12,pxCameraOrientation->e22,pxCameraOrientation->e32);

			Asura_Vector_3 xDelta = (xRight * fSize) + (xUp * fSize);

			xP1 = xPosition - xDelta;
			xP2 = xPosition + xDelta;			
					
			D3DXVECTOR4 xVIn1,xVIn2;
			D3DXVECTOR4 xV1,xV2;
			xVIn1.x = xP1.x;	xVIn1.y = xP1.y;	xVIn1.z = xP1.z;	xVIn1.w = 1.0f;
			xVIn2.x = xP2.x;	xVIn2.y = xP2.y;	xVIn2.z = xP2.z;	xVIn2.w = 1.0f;
			D3DXVec4Transform(&xV1,&xVIn1,&Asura_Direct3D::GetViewProjectionMatrix());
			D3DXVec4Transform(&xV2,&xVIn2,&Asura_Direct3D::GetViewProjectionMatrix());			
			xP1.x = xV1.x;		xP1.y = xV1.y;		xP1.z = xV1.z;
			xP2.x = xV2.x;		xP2.y = xV2.y;		xP2.z = xV2.z;

			if (xP1.z > 0.0f)
			{			
				xP1 = xP1 * (1.0f/xV1.w);
				xP2 = xP2 * (1.0f/xV2.w);				
		
				// Handle the distance fade
				if (xV1.z>pxTemplate->m_fNearZeroDistance && xV1.z<pxTemplate->m_fFarZeroDistance)
				{		
					float fAlpha = 1.0f;
					if (xV1.z<pxTemplate->m_fNearFadeDistance)
					{
						fAlpha = (xV1.z-pxTemplate->m_fNearZeroDistance) / (pxTemplate->m_fNearFadeDistance-pxTemplate->m_fNearZeroDistance);
					}

					if (xV1.z>pxTemplate->m_fFarFadeDistance)
					{
						fAlpha = 1.0f - (xV1.z-pxTemplate->m_fFarFadeDistance) / (pxTemplate->m_fFarZeroDistance-pxTemplate->m_fFarFadeDistance);
					}

					float fArea = 0.25f*(xP2.x-xP1.x)*-(xP2.y-xP1.y);

					if (fAlpha>=0.01f)
					{
						s_fApproxScreenfullsRenderedThisFrame+=fArea;				
					}										
				}
			}
		}
#endif
#endif

		pxParticleInstanceData++;
	}
}



void Asura_Platform_PFX_System_FastPath::RenderStats(void)
{
#ifdef ASURA_TOOLS
#ifdef ASURA_TOOLS_SHOW_PFX_COST
	if (g_bPFXOutlinesEnabled)
	{
		Asura_Text_Output::Print(70,1,0xffffffff,0,"Approx PFX Cost: %0.02f Screenfills",Asura_Platform_PFX_System_FastPath::GetNumFastPathParticleScreenfullsRenderedThisFrame());
	}
#endif
#endif
}



#endif // ASURA_USE_FASTPATH_PFX

