//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_SkinTypes.h
//
// Copyright (c) 2002-2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef _MSC_VER
    #pragma once
#endif

#ifndef ___ASURA_PS2_SKINTYPES_H___
#define ___ASURA_PS2_SKINTYPES_H___ 1

// T Y P E S /////////////////////////////////////////////////////////////////

//
// Smooth skin data format.
//

// Overview:
//
//     skin hdr
//         material hdr 0
//             sub element hdrs
//         ...
//         material hdr N-1
//         <padding to QW boundary>
//         strip data chain for material 0's first sub element
//         ...
//         strip data chain for material N-1's last sub element
//
// The types are assumed to be of compatible sizes, such that no padding is
// required other than before the first strip data chain.

struct Asura_PS2_SmoothSkin_SkinHdr
{
    u_short usNumMaterials;
    u_short usNumElements;
    u_short usMaxVertsPerStrip;
    u_short usQWOffsetToFirstStripDataChain;
//  int aiMaterialIDs[usNumMaterials];
};

struct Asura_PS2_SmoothSkin_MaterialHdr
{
    u_int uNumElementsUsingThisMaterial;
};

struct Asura_PS2_SmoothSkin_SubElementHdr
{
    u_short usElementIdx;
    u_short usTriCount;
    u_int   uByteOffsetToNextChain; // Must be a multiple of 16.
};

// Material ordering - simplified version of the env stuff.
enum ASURA_PS2_SMOOTH_SKIN_RENDERPASS
{
    ASURA_PS2_SMOOTH_SKIN_RENDERPASS_FIRST,
    ASURA_PS2_SMOOTH_SKIN_RENDERPASS_OPAQUE = ASURA_PS2_SMOOTH_SKIN_RENDERPASS_FIRST,
    ASURA_PS2_SMOOTH_SKIN_RENDERPASS_ON_OFF_TRANSPARENCY,
    ASURA_PS2_SMOOTH_SKIN_RENDERPASS_TRANSLUCENT,
    ASURA_PS2_SMOOTH_SKIN_RENDERPASS_ADDITIVE_ALPHA,
    ASURA_PS2_NUM_SMOOTH_SKIN_RENDERPASSES,
};

// Strip data format:
//
// The strip data needs to be unpacked to the beginning of the current VU
// input buffer (as determined by the current VIF1 double buffering state.)
// The unpacked data for each vertex is expected to look like this:
//
//     /---------------------------------------------------------------\
//     | nrm x         | nrm y         | nrm z         | ADC           |  V4_5 format (effectively 1:0:4 with a bias for xyz)
//     |---------------------------------------------------------------|
//     | bone idx 0    | bone idx 1    | bone idx 2    | bone idx 3    |  integer
//     |---------------------------------------------------------------|
//     | bone weight 0 | bone weight 1 | bone weight 2 | bone weight 3 |  1:0:15 fixed point
//     |---------------------------------------------------------------|
//     | pos x         | pos y         | pos z         | [don't care]  |  1:3:12 fixed point
//     |---------------------------------------------------------------|
//     | texel s       | texel t       | [don't care]  | [don't care]  |  1:3:12 fixed point
//     \---------------------------------------------------------------/
//
// After unpack, the ITOPS register should be set with the vertex count and
// finally an MSCNT VIF code should be issued.

//
// Hierarchical skin file format.
//

// Chunk format overview for a hierarchical skin with h hierarchy sections:
//
//      skin name
//
//      skin header (Asura_PS2_HierSkin_SkinHdr)
//
//      section material counts (array of u_short)
//
//      section material headers (array of Asura_PS2_HierSkin_SectionMatHdr)
//
//      padding to 128-byte alignment, relative to start of skin header
//
//      each section material's DMA chain
//      (each chain will be a multiple of 8QWs in size)

struct Asura_PS2_HierSkin_SectionMatHdr;

struct Asura_PS2_HierSkin_SkinHdr
{
    u_int                             uAllMaterialFlags;    // Bitwise ORing of all material flags in the skin.
    short                             sTextureID;           // Setup at runtime.
    u_short                           usSectionCount;
    Asura_PS2_HierSkin_SectionMatHdr* pxSectionMatHdrStart; // Converted from byte offset to ptr at load time.
    u_int                             uStripDataStart;      // Converted from QW offset to address at load time.
};

struct Asura_PS2_HierSkin_SectionMatHdr
{
    u_short usTriCount;
    short   sMaterialID;
    u_int   uByteOffsetToNextChain;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_SKINTYPES_H___
