//*******************************************************************
//
// File: Asura_PS2_Carpet_Mesh_Shared.h
//
// Created: 14 March 2006 by Jake Turner
//
// Description: 
// Classes and structures used to hold PS2 Carpet Mesh data
// This file is shared and used by the PC TOOL and PS2 platform code
// The tools export data in these structure and the PS2 uses them at runtime
//
//*******************************************************************

#ifdef ASURA_USE_CARPET

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __ASURA_PS2_CARPET_MESH_SHARED_H__
#define __ASURA_PS2_CARPET_MESH_SHARED_H__

//////////////////////////////////////////////////////////////////////////////
//
// The PS2 Carpet mesh data format Version 3 is:
//
//     1 * Asura_PS2_Carpet_Mesh_Header
//
//     uNumStyles * Asura_PS2_Carpet_Mesh_Style_Header
//
//     uTotalNumModuleHdrs * Asura_PS2_Carpet_Mesh_Module_Header
//
//     uTotalNumModulesHdrs * <dma chain>
//         (totalling uTotalDMAChainSize altogther)
//
//     uNumStyles * Asura_PS2_Carpet_Style_DMA_Data
//
//////////////////////////////////////////////////////////////////////////////

#define ASURA_PS2_CARPET_MESH_CHUNK_VERSION (4)

union Asura_PS2_Carpet_QW
{
	short         i16s[8];
	u_short       u16s[8];
	float         f32s[4];
	int           i32s[4];
	u_int         u32s[4];
};

struct Asura_PS2_Carpet_Mesh_Header
{
	u_int uNumStyles;
	u_int uTotalNumTris;
    u_int uTotalNumModuleHdrs;
    u_int uTotalDMAChainSize;
	u_int uTotalHeadersSizeInBytes; // Excluding this header itself.
};

struct Asura_PS2_Carpet_Mesh_Style_Header
{
	u_short usStyleID;
	u_short usNumModules;
	u_int uNumTris;
};

struct Asura_PS2_Carpet_Mesh_Module_Header
{
	u_short usModuleID;
	u_short usExpectedNumBillboards; // Total number of billboards we expect VU1 to generate when rendering the source tri for this module.
    u_int   uDMAChainSizeOrPtr;      // At runtime this is patched up to be an address for CALLing.
    u_int   uSeed;
};

struct Asura_PS2_Carpet_Triangle_DMA_Data
{
    u_int   uUnpackXYZW;
    u_short ausXYZWs[3 * 4]; // XYZ, density (which is actually vert density pre-multiplied by the style density)
    u_int   uUnpackRGBA;
    u_char  aucRGBAs[3 * 4];
    u_int   uMSCNT;
};

// Don't see much point in making a structure showing the actual
// layout because it makes it very hard to use from C++ code
// Much easier to fill out a generic qword array
struct Asura_PS2_Carpet_Style_DMA_Data
{
	Asura_PS2_Carpet_QW xRETtag; // contains the UNPACK command
	// 16 QW's of
	//       x            y            z            w
	//  sequence[0]   movement[0]    width[0]    height[0]
	//  sequence[1]   movement[1]    width[1]    height[1]
	//  sequence[2]   movement[2]    width[2]    height[2]
	//  sequence[3]   movement[3]    width[3]    height[3]
	//  sequence[4]   movement[4]    width[4]    height[4]
	//  sequence[5]   movement[5]    width[5]    height[5]
	//  sequence[6]   movement[6]    width[6]    height[6]
	//  sequence[7]   movement[7]    width[7]    height[7]
	//  sequence[8]   movement[8]   minScale[0]    style density
	//  sequence[9]   movement[9]   minScale[1]    fLoDFadeNear
	// sequence[10]  movement[10]   minScale[2]    fLoDFadeFar
	// sequence[11]  movement[11]   minScale[3]    fK0
	// sequence[12]  movement[12]   minScale[4]    fScale0
	// sequence[13]  movement[13]   minScale[5]    fK1
	// sequence[14]  movement[14]   minScale[6]    fScale1
	// sequence[15]  movement[15]   minScale[7]    fGlobalFade
	//
	// 8 QW's of
	//    x       y        z      w
	//   u0[0]   v0[0]   u1[0]   v1[0]
	//   u0[1]   v0[1]   u1[1]   v1[1]
	//   u0[2]   v0[2]   u1[2]   v1[2]
	//   u0[3]   v0[3]   u1[3]   v1[3]
	//   u0[4]   v0[4]   u1[4]   v1[4]
	//   u0[5]   v0[5]   u1[5]   v1[5]
	//   u0[6]   v0[6]   u1[6]   v1[6]
	//   u0[7]   v0[7]   u1[7]   v1[7]
	Asura_PS2_Carpet_QW xData[16+8];

    int iTextureID; // Actually a hash in the exported data, fixed up on load.
    int pad[3];
};

#endif //  #ifndef __ASURA_PS2_CARPET_MESH_SHARED_H__

#endif // #ifdef ASURA_USE_CARPET