////////////////////////////////////////////////////////////////////////////// //
// Asura_Chunk_ResourceFile_PS2.cpp
//
// By: Morgan Parry.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Independent.
#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile.h"

#include "Core/Asura_Handle.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"

// PS2.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2.h"
#if defined( ASURA_USE_BOTANICALS )
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_Botanicals.h"
#endif
#if defined( ASURA_USE_CARPET)
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_Carpet.h"
#endif
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_IRX.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_SkinLODThresholds.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_StrippedEnv.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_StrippedHierSkin.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_StrippedObject.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_StrippedSkybox.h"
#ifdef ASURA_USE_SKYSPHERE
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_StrippedSkysphere.h"
#endif
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_StrippedSmoothSkin.h"
#ifdef ASURA_PS2_SUPPORT_PARTIALLY_STREAMED_SOUNDS
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_PartiallyStreamedSounds.h"
#endif
#ifdef ASURA_PS2_SUPPORT_DLLS
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_Dll.h"
#endif
#ifdef ASURA_PS2_ENABLE_DNAS
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_DnasAuthData.h"
#endif


// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool Asura_Chunk_ResourceFile::Platform_Process( Asura_Handle hFile, const char* szOriginalFileName )
{
    bool bProcessedEntireChunk;

    // (MP - 16.09.2002) Note that I've omitted the semi-colon at the end of
    // this macro; this stops Lint complaining.
    #define PROCESS( type, class_for_type ) \
        case type: \
            bProcessedEntireChunk = (reinterpret_cast<class_for_type*>(this))->Process( hFile ); \
            break
    #define PROCESS_WITH_FILENAME( type, class_for_type ) \
        case type: \
            bProcessedEntireChunk = (reinterpret_cast<class_for_type*>(this))->Process( hFile, szOriginalFileName ); \
            break

    switch( this->SubType )
    {
        // Process known subtypes.
        PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_IRX,                     Asura_Chunk_ResourceFile_PS2_IRX                     );
        PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_SKINLODTHRESHOLDS,       Asura_Chunk_ResourceFile_PS2_SkinLODThresholds       );
        PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_STRIPPEDENV,             Asura_Chunk_ResourceFile_PS2_StrippedEnv             );
        PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_STRIPPEDHIERSKIN,        Asura_Chunk_ResourceFile_PS2_StrippedHierSkin        );
        PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_STRIPPEDOBJECT,          Asura_Chunk_ResourceFile_PS2_StrippedObject          );
        PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_STRIPPEDSKYBOX,          Asura_Chunk_ResourceFile_PS2_StrippedSkybox          );
#ifdef ASURA_USE_SKYSPHERE
        PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_STRIPPEDSKYSPHERE,       Asura_Chunk_ResourceFile_PS2_StrippedSkysphere       );
#endif
        PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_STRIPPEDSMOOTHSKIN,      Asura_Chunk_ResourceFile_PS2_StrippedSmoothSkin      );
#ifdef ASURA_PS2_SUPPORT_PARTIALLY_STREAMED_SOUNDS
        PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_PARTIALLYSTREAMEDSOUNDS, Asura_Chunk_ResourceFile_PS2_PartiallyStreamedSounds );
#endif
#ifdef ASURA_PS2_SUPPORT_DLLS
		PROCESS_WITH_FILENAME( ASURA_RESOURCEFILE_TYPE_PS2_DLL,		  Asura_Chunk_ResourceFile_PS2_Dll					   );
#endif
#ifdef ASURA_PS2_ENABLE_DNAS
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_DNASAUTHDATA,			  Asura_Chunk_ResourceFile_PS2_DnasAuthData			   );
#endif
#if defined( ASURA_USE_CARPET )
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_CARPET,				  Asura_Chunk_ResourceFile_PS2_Carpet				   );
#endif
#if defined( ASURA_USE_BOTANICALS )
		PROCESS( ASURA_RESOURCEFILE_TYPE_PS2_BOTANICALS,			  Asura_Chunk_ResourceFile_PS2_Botanicals			   );
#endif


        // Deal with unrecognised subtypes.
        default:
            Asura_Assert( false, "Unsupported PS2 resource file chunk type." );
            Asura_FileAccess::SkipData( hFile, this->FileSize );
            bProcessedEntireChunk = true;
            break;
    }

    #undef PROCESS
    #undef PROCESS_WITH_FILENAME

    return bProcessedEntireChunk;
}
