//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_ResourceFile_PS2_StrippedObject.cpp
//
// Created by Richard M. Smith - 17 July 2002
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PS2_StrippedObject.h"

// Independent.
#include "Core/Asura_Handle.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_FileAccess.h"

// PS2.
#include "Rendering/Asura_PlatformRenderObject.h"
#include "Rendering/Asura_PS2_Object_System.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool Asura_Chunk_ResourceFile_PS2_StrippedObject::Process( Asura_Handle iFileHandle )
{
    if( iASURA_PS2_STRIPPEDOBJECT_CUR_VERSION_NUMBER != Version )
    {
	    Asura_Assert( false, "This PS2 object render data cannot be used. Please re-export the relevant data or update your code." );
        return false;
    }

	u_int uBytesAlreadyRead = 0;

    // Read in name.
    u_int uSizeOfNameInInts;
    const Asura_Char* szName = Asura_FileAccess::ReadString( iFileHandle, NULL, &uSizeOfNameInInts );
	uBytesAlreadyRead += uSizeOfNameInInts << 2;

    Asura_PS2_Shape_ShapeHdr xShapeHdr;
    Asura_FileAccess::ReadData( iFileHandle, &xShapeHdr, sizeof(Asura_PS2_Shape_ShapeHdr) );
	uBytesAlreadyRead += sizeof(Asura_PS2_Shape_ShapeHdr);

	// Allocate the object
	Asura_PS2_StrippedObject* pxNewPS2Obj = Asura_PS2_Object_System::CreatePS2Object( szName, xShapeHdr.fLODThreshold );
    if( !pxNewPS2Obj )
    {
	    Asura_Assert( false, "Failed to create object. Out of memory? Or too many LODs?" );
        return false;
    }

	// Create the object
	return pxNewPS2Obj->LoadStrippedObject( iFileHandle, this->FileSize - uBytesAlreadyRead, xShapeHdr );
}
