//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_Chunk_Loaders.cpp
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Engine independent.
#include "ChunkSystem/Asura_Chunk_Loaders.h"

// Engine platform-specific.
#include "Core/Asura_PlatformBasics.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

// PJC (15.01.2008) The use of SPR for loads multithreaded/vsynched to run 
// concurrently with rendering can cause issues relating to data stored in 
// the SPR being overwritten
#ifdef ASURA_PS2_CHUNKLOADING_DISABLE_SPR

char* Asura_ChunkLoading_System::Platform_GetTempChunkMemory( u_int uBytesReqd )
{
	return new char[uBytesReqd];
}

void Asura_ChunkLoading_System::Platform_ReturnTempChunkMemory( char* pcTempMem )
{
	delete[] pcTempMem;
}

#else

char* Asura_ChunkLoading_System::Platform_GetTempChunkMemory( u_int uBytesReqd )
{
	// Currently all of the places that use this only require small bits of
	// memory, say < 4KB, so we should be fine using some SPR.
	Asura_Assert( uBytesReqd <= 16 * 1024, "" );

	return reinterpret_cast<char*>( Asura_PS2_SPR_Alloc(0, (uBytesReqd + 15) >> 4) );
}

void Asura_ChunkLoading_System::Platform_ReturnTempChunkMemory( char* pcTempMem )
{
	Asura_PS2_SPR_Dealloc( reinterpret_cast<Asura_PS2_QW*>(pcTempMem) );
}

#endif