/**
*
* \file Asura_PlatformCore.cpp
*
* Created by Kevin Lea - 27/11/2001 12:11:07
*
* Copyright (C) 2001 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

// Standard platform-specific.
#include <libcdvd.h>
#include <libsdr.h>
#include <libsn.h>
#include <sifdev.h>
#include <sifrpc.h>
#include <wchar.h>
#if defined( ASURA_PS2_EE_ONLY_EH )
#include <eekernel.h>
#endif // #if ASURA_PS2_EE_ONLY_EH

// Engine independent.
#include "Core/Asura_Core.h"
#include "Core/Asura_PlatformBasics.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#include "Sound/Asura_Sound.h"

// Engine platform-specific.
#include "Animation/Asura_PS2_Hierarchy.h"
#include "Core/Asura_PS2_IRX.h"
#include "Core/Asura_PS2_OS.h"
#include "Core/Asura_PS2_SystemInfo.h"
#include "Debugging/Asura_ExceptionHandler.h"
#include "FileAccess/Asura_PS2_TOCCache.h"
//#include "FMV/Asura_PS2_Mpeg.h"
#include "Input/Asura_PS2_Input.h"
#include "MemoryCard/Asura_PS2_MemoryCard.h"
#include "Profiling/Asura_PS2_Profiling.h"
#include "ProjectSettings/Asura_PS2_ProjectControllerDefines.h"
#include "Rendering/Asura_PS2_Object_System.h"
#include "Rendering/Asura_PS2_ShadowBuffer.h"
#include "Sound/Asura_PS2_Multistream.h"
#if ASURA_PS2_SUPPORT_EYETOY
#include "Eyetoy/Asura_PS2_Eyetoy.h"
#endif
#ifdef ASURA_PS2_USE_SHELLMCUI_MEMORYCARD_SYSTEM
#include "ShellMCUI/Asura_PS2_ShellMCUI.h"
#endif

// F U N C T I O N   D E C L A R A T I O N S /////////////////////////////////
extern "C" {
	extern int sceChk01(u_int *stat);
};

// D E F I N E S /////////////////////////////////////////////////////////////
#if defined(ASURA_PS2_CDEMUL) || defined(ASURA_PS2_ENABLE_DNAS)
#define ASURA_PS2_NEEDTOINITCD 1
#else
#define ASURA_PS2_NEEDTOINITCD 0
#endif


// G L O B A L S /////////////////////////////////////////////////////////////
#if _DEBUG
DEBUGVAR bool dbg_bUseDebugIRXs = true;
#else
DEBUGVAR bool dbg_bUseDebugIRXs = false;
#endif

static bool g_bHACKAlwaysFalse = false;

// F U N C T I O N   I M P L E M E N T A T I O N S ///////////////////////////
static void InitIOPModules()
{
// (MH - 10.05.2002) in order to support AtWinMon, if we want to use the old code
// which loads modules directly from host0/cdrom one at a time some kind of buffer
// needs to be created with ASURA_PS2_FILEROOT and each module name added to it.

    Asura_PS2_OS::LoadIRXModule(SIO2MAN, 0, NULL);
    Asura_PS2_OS::LoadIRXModule(PADMAN, 0, NULL);
#if ASURA_PS2_SUPPORT_MULTITAP
    Asura_PS2_OS::LoadIRXModule(MTAPMAN, 0, NULL);
#endif
    Asura_PS2_OS::LoadIRXModule(_MOD_SD, 0, NULL);

    if (dbg_bUseDebugIRXs)
    {
		// (MH - 21.07.2003) always load dbgstream.irx from host0
        Asura_PS2_OS::LoadIRXModule(_MOD_STREAM_DBG, 0, NULL, true); 
    }
    else
    {
        Asura_PS2_OS::LoadIRXModule(_MOD_STREAM, 0, NULL); 
    }

	Asura_PS2_OS::LoadIRXModule(SDRDRV_MODULE, 0, NULL);

//    // Load usb driver
#ifdef ASURA_PS2_LOAD_USBD_FROM_HOST0 
	bool bForceUSBModulesFromHost0 = true;
#else
	bool bForceUSBModulesFromHost0 = false;
#endif

#if ASURA_PS2_SUPPORT_KEYBOARD || ASURA_PS2_SUPPORT_MOUSE || ASURA_PS2_SUPPORT_GUNCON2  || ASURA_PS2_SUPPORT_EYETOY || defined(ASURA_PS2_SUPPORT_USB_HEADSET)// only needed if supporting one of these devices
    // Build the arguments
	// (AW - 25.11.2004) - Changed arguments. These are apparently recommended by Sony
	// and give a 16.25kb IOP RAM saving over our previous arguments.
    char szUSBOptions[] = "hub=1\0dev=11\0ed=32\0gtd=64\0itd=64\0ioreq=128\0";//"dev=8\0hub=2\0port=4\0";
    Asura_PS2_OS::LoadIRXModule(USBD_MODULE, sizeof(szUSBOptions)+1, szUSBOptions, bForceUSBModulesFromHost0);
#endif

    // Load usb keyboard and mouse driver
#if ASURA_PS2_SUPPORT_KEYBOARD || ASURA_PS2_SUPPORT_MOUSE 

    // Build the arguments
    char szLogitechOptions[64];
    char* pcCurrentArg = szLogitechOptions;
    int iArgSize = 0;
    // Thread priority
    sprintf(pcCurrentArg, "thpri=%i", ASURA_PS2_LGKBM_THREAD_PRIORITY);
    iArgSize += strlen(pcCurrentArg)+1;
    pcCurrentArg += strlen(pcCurrentArg)+1;

    // Max keyboards
    sprintf(pcCurrentArg, "MaxKbd=%i",  ASURA_PS2_MAX_KEYBOARDS);
    iArgSize += strlen(pcCurrentArg)+1;
    pcCurrentArg += strlen(pcCurrentArg)+1;
    
    // Max mice
    sprintf(pcCurrentArg, "MaxMice=%i", ASURA_PS2_MAX_MICE);
    iArgSize += strlen(pcCurrentArg)+1;

    Asura_PS2_OS::LoadIRXModule(USB_LGKBM_MODULE, iArgSize, szLogitechOptions, bForceUSBModulesFromHost0);
#endif

#ifdef ASURA_PS2_SUPPORT_USB_HEADSET
	// (AW - 23.03.2005) - Default parameters except maxstream is 32kb instead of 64kb.
	char szHeadsetOptions[] = "thpri=48\0rpc=1\0maxstream=32768\0";
	Asura_PS2_OS::LoadIRXModule(USB_LGAUD_MODULE, sizeof(szHeadsetOptions)+1, szHeadsetOptions);
#endif //#ifdef ASURA_PS2_SUPPORT_USB_HEADSET

#ifndef ASURA_PS2_DEMODISK
	// load memory card drivers
    Asura_PS2_OS::LoadIRXModule(MCMAN, 0, NULL);
    Asura_PS2_OS::LoadIRXModule(MCSERV, 0, NULL);
#endif//#ifndef ASURA_PS2_DEMODISK

#if ASURA_PS2_SUPPORT_GUNCON2
    Asura_PS2_OS::LoadIRXModule(IRX_GUNCON2_FILENAME, 0, NULL);
#endif

#if ASURA_PS2_SUPPORT_EYETOY
    Asura_PS2_Eyetoy::LoadIRXModule();
#endif

	// Load CD/DVD streaming module .. for FMV streaming
	Asura_PS2_OS::LoadIRXModule(CDVD_STREAM, 0, NULL);

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	// install dev9.irx and the poweroff thread to provide consistent behaviour
	// regardless of whether you play online or not. 
	int iResult;
	Asura_PS2_OS::LoadIRXModule("\\IRX\\NET\\DEV9.IRX", 0, NULL, false, 0, &iResult);
	Asura_PS2_OS::SetNetworkAdaptorConnected(iResult == RESIDENT_END);
    Asura_PS2_OS::PreparePowerOff();
#endif
	
    sceSdRemoteInit();
    sceSdRemote(1, rSdInit, SD_INIT_COLD);
    
#ifdef ASURA_PS2_CDEMUL
    sceCdDiskReady(0);
#endif

}

void Asura_Core::PlatformInitialise(void)
{
    // A compiler option is required in order to make wide characters just
    // two bytes in size. By default they're four.
    Asura_Assert( sizeof(Asura_UnicodeChar) == 2, "Incorrect size of wide characters - add -fshort-wchar to your compiler options." );
	Asura_Assert( wcslen(L"1234") == 4, "incorrect version of wcslen" );

#if _DEBUG
    // In debug, devkits must be reset with bit 0 of the EE parameters on, to
    // enable a fix for an EE memory controller bug.
    {
        const u_int* puBootParams = snGetEEBootParams();
        if( puBootParams && !(puBootParams[EEPARAM_MEM_CONTROLLER_FIX_OFFSET] & EEPARAM_MEM_CONTROLLER_FIX_MASK) )
        {
            while( true ) // Force them to fix the problem.
            {
                Asura_Assert( false, "You haven't reset your devkit with bit 0 of the EE parameter high." );
            }
        }
    }
#endif
    
#if _DEBUG
    Asura_PS2_DebugSPR::Initialise();
#endif

    // (MP - 08.01.2002) Note that this needs to be done before any file accesses.
	// (PSJE - 16.12.2002) Startup sequce updated to match sony recomended order - (See: sce.dev.prog.cddvd - sceCdSearchFile weirdness when Rebooting IOP)

	// replace IOP default module
#ifdef ASURA_PS2_CDEMUL
    sceSifInitRpc(0);
    sceCdInit(SCECdINIT);
#if !_DEBUG	// don't reboot IOP if debugging.
    while (!sceSifRebootIop (IOPRP)); /* Reboot IOP, replace modules */
    while( !sceSifSyncIop() );        /* Wait for end */
#endif // ndef _DEBUG
#endif // ASURA_PS2_CDEMUL

#if !defined(ASURA_PS2_CDEMUL) || _DEBUG
	// (MH - 26.03.2004) for Tuner compatability, try to make sure neither gets stripped
	if (g_bHACKAlwaysFalse)
	{
		sceSifRebootIop("");
		sceSifSyncIop();
	}
#endif

    sceSifInitRpc(0);
    sceSifLoadFileReset();
    sceSifInitIopHeap();
    sceFsReset();
#if ASURA_PS2_NEEDTOINITCD
    sceCdInit(SCECdINIT);
    switch( sceCdGetDiskType() )
    {
        case SCECdPS2DVD:
            Asura_OutputToDebugger(( "Disk type: DVD.\n" ));
            sceCdMmode( SCECdDVD );
            break;

        default:
            Asura_OutputToDebugger(( "Disk type: CD.\n" ));
            sceCdMmode( SCECdCD );
            break;  
    }
#endif

#if ASURA_PS2_USE_TOC_CACHE
	// Read CD/DVD TOC
	Asura_PS2_TOCCache::ReadTOC();
#endif

// (MP - 12.06.2003) This stopped being necessary with ProDG v2.2.
//#if (!defined(ASURA_PS2_CDEMUL)) && (_DEBUG || ASURA_PS2_ENABLE_PROFILING)
//    // MH: this sets up some VU debugging extensions
//    if( !snDebugInit() )
//    {
//        printf( "ERROR: Call to snDebugInit() failed.\n" );
//    }
//#endif // #if (!defined(ASURA_PS2_CDEMUL)) && (_DEBUG || ASURA_PS2_ENABLE_PROFILING)

    // MH: load IRXs
    InitIOPModules();

#if 0 // library integrity check
	int ret;
	u_int stat;
	ret = sceChk01( &stat );

	Asura_OutputToDebugger_Retail(("sceChk01 ret:%d stat:%x\n", ret, stat));

    if (0 <= ret) 
	{
        if (0 != stat) 
		{
			/* library consistency check failed, take countermeasures */
			Asura_OutputToDebugger_Retail(("library consistency check failed\n"));
			ASURA_PS2_DEBUG_BREAK();
        }
	}
    else 
	{
        Asura_OutputToDebugger_Retail(("***** sceChk01 Command Fail *****\n"));
    }
#endif
		
	Asura_PS2_Input::PreInitialise();

    Asura_PS2_OS::Initialise();
#ifndef ASURA_PS2_DEMODISK

	#ifdef ASURA_PS2_USE_SHELLMCUI_MEMORYCARD_SYSTEM
		Asura_PS2_ShellMCUI::Initialise();
	#else
		Asura_PS2_MemoryCard_System::Initialise();
	#endif

#endif

#if ASURA_PS2_ENABLE_PROFILING
    if( Asura_PS2_OS::IsTOOL() || Asura_PS2_OS::IsTestKit() )
    {
        Asura_PS2_ProfilingSystem::Initialise( Asura_PS2_OS::IsTOOL() );
    }
#endif

    if( !Asura_PS2_Multistream::Initialise(ASURA_MAX_NUMBER_OF_LOADED_SOUNDS) )
    {
        Asura_Assert( false, "Failed to initialise multistream." );
    }

    // MH 28.9.01 -- install EE exception handler
#ifdef ASURA_PS2_USE_EXCEPTION_HANDLER
    // if ( !Asura_PS2_OS::IsTOOL() )
	// JAT 100506 - changed to test if the debugger is running so can get exceptions on TOOL's e.g. run ELF via target manager
    if ( !snIsDebuggerRunning() )
    {
#if ASURA_PS2_EE_ONLY_EH
		::SetDebugHandler(  1, Asura_ExceptionHandler );
		::SetDebugHandler(  2, Asura_ExceptionHandler );
		::SetDebugHandler(  3, Asura_ExceptionHandler );
		::SetDebugHandler(  4, Asura_ExceptionHandler );
		::SetDebugHandler(  5, Asura_ExceptionHandler );
		::SetDebugHandler(  6, Asura_ExceptionHandler );
		::SetDebugHandler(  7, Asura_ExceptionHandler );
		if ( !snIsDebuggerRunning() )
		{
//			::SetDebugHandler(  8, Asura_ExceptionHandler );
//			::SetDebugHandler(  9, Asura_ExceptionHandler );
		}
		::SetDebugHandler( 10, Asura_ExceptionHandler );
		::SetDebugHandler( 11, Asura_ExceptionHandler );
		::SetDebugHandler( 12, Asura_ExceptionHandler );
		::SetDebugHandler( 13, Asura_ExceptionHandler );
#else
        sceExcepSetDebugEEHandler(EEExceptionHandler);
        sceExcepSetDebugIOPHandler(IOP_EXCEPTION_HANDLER_MODULE,IOPExceptionHandler, &iop);
#endif
    }
#endif

// (RMS) Addition - PS2 Object System (similar to Asura_Mesh_System)
	Asura_PS2_Object_System::Initialise();
    Asura_PS2_SystemInfo::Initialise();
}

void Asura_Core::PlatformShutdown(void)
{
    Asura_PS2_Multistream::Shutdown();

//#if ASURA_PS2_USE_NEW_FMV_STUFF
//  Asura_PS2_MpegSystem::Shutdown();
//#endif

	Asura_PS2_ShadowBufferSystem::Destroy_ModuleData();
    
    Asura_PS2_Object_System::Shutdown();

#if !defined(ASURA_PS2_CDEMUL) && ASURA_PS2_ENABLE_PROFILING
    Asura_PS2_ProfilingSystem::Shutdown();
#endif
	
#ifndef ASURA_PS2_DEMODISK

	#ifdef ASURA_PS2_USE_SHELLMCUI_MEMORYCARD_SYSTEM
		Asura_PS2_ShellMCUI::Shutdown();
	#else
		Asura_PS2_MemoryCard_System::Shutdown();
	#endif

#endif
    Asura_PS2_OS::Shutdown();

#if _DEBUG
    Asura_PS2_DebugSPR::Shutdown();
#endif

#if ASURA_PS2_SUPPORT_EYETOY
    Asura_PS2_Eyetoy::Shutdown();
#endif
	
#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	Asura_PS2_OS::UndoPowerOff();
#endif
}

void Asura_Core::PlatformReset(void)
{
	Asura_PS2_ShadowBufferSystem::Destroy_ModuleData();
	Asura_PS2_Object_System::Reset();
    Asura_PS2_OS::Reset();
}

bool Asura_Core::Platform_IsMainThread(void)
{
	return Asura_PS2_OS::IsMainThread();
}
