//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE        :  Asura_Platform_Time_System.cpp
//
//  CREATED     :  Andrew McCabe  (13.1.2004)
//
//  Copyright (C) 2004 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// independent includes
#include "Core/Asura_Time_System.h"

// ps2 includes
#include "Core/Asura_PS2_SystemInfo.h"

// library includes
#include <time.h>

//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
//
// Asura_Time_System
//
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//
// Platform_GetCurrentDateAndTime
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Time_System::Platform_GetCurrentDateAndTime()
{
	tm xDateAndTime;

	Asura_PS2_SystemInfo::GetSystemDateAndTime(&xDateAndTime);

	s_xTime.m_ucSecond = static_cast<u_char>(xDateAndTime.tm_sec);
	s_xTime.m_ucMinute = static_cast<u_char>(xDateAndTime.tm_min);
	s_xTime.m_ucHour = static_cast<u_char>(xDateAndTime.tm_hour);
	s_xTime.m_ucDayOfMonth = static_cast<u_char>(xDateAndTime.tm_mday-1);	//Asura dates are zero-based.
	s_xTime.m_ucMonth = static_cast<u_char>(xDateAndTime.tm_mon-1);		

	// Unfortunately the year information got from the CD clock on the PS2 goes from 0-99.
	// Thank you Sony. The docs show this is intended to cover the years from 2000 to 2099.
	// So I guess we'll just add 2000 to the year value.

	s_xTime.m_usYear = static_cast<u_short>(xDateAndTime.tm_year+2000);

	s_xTime.m_ucDayOfWeek = static_cast<u_char>(GetDayOfWeek(s_xTime));
}


// in minutes, where UTC = local time + bias 
int Asura_Time_System::Platform_GetTimeZoneBias()
{
	return Asura_PS2_SystemInfo::GetTimeZone();
}

//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
//
// Asura_Time
//
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//
// Platform_FillFromLibCTime
//
////////////////////////////////////////////////////////////////////////////////

void Asura_Time::Platform_FillFromLibCTime(u_int uLibCTime)
{
	time_t xTimeT = uLibCTime;
	const tm* pxLibCTimeBlock = gmtime(&xTimeT);
	m_usYear = pxLibCTimeBlock->tm_year + 1900;
	m_ucMonth = pxLibCTimeBlock->tm_mon;
	m_ucDayOfMonth = pxLibCTimeBlock->tm_mday - 1;
	m_ucHour = pxLibCTimeBlock->tm_hour;
	m_ucMinute = pxLibCTimeBlock->tm_min;
	m_ucSecond = pxLibCTimeBlock->tm_sec;
}
