//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_DemoSystem.cpp
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     Dmeo system for SCEE demos
//
// (AW - 14.12.2004) - Updated for new SCEA/SCEE combined demo SDK
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_PS2_DEMODISK

#include "Asura_PS2_DemoSystem.h"

#include <stdio.h>

#include "Core/Asura_Maths.h"
#include "Core/Asura_UserInput.h"
#include "Input/Asura_PlatformInput.h"

Asura_PS2_DemoSystem::DemoParms Asura_PS2_DemoSystem::s_xDemoParms;
float Asura_PS2_DemoSystem::s_fInactivityTimer;
float Asura_PS2_DemoSystem::s_fMaxInactivity;
float Asura_PS2_DemoSystem::s_fGameplayTimer;
float Asura_PS2_DemoSystem::s_fMaxGameplay;
bool  Asura_PS2_DemoSystem::s_bGamePlayStarted = false;
float Asura_PS2_DemoSystem::s_fPressButtonTimer = 0.0f;
sceDemoEndReason Asura_PS2_DemoSystem::s_eReasonForEnding = static_cast<sceDemoEndReason>(-1);

const float fBUTTONPRESSTIMEOUT = 1.0f;

static void simpleatoi(char *a,unsigned short *i)
{
    int c;

    *i = 0;

    for (c = 0;a[c] != 0;c++) 
    {
        *i = (*i * 10) + a[c] - '0';
    }
}

void Asura_PS2_DemoSystem::Initialise(int argc, char **argv)
{
    printf("argc = %d.\n", argc);
    for (int i = 1; i < argc; i++)
    {
        u_short usTemp;
        simpleatoi(argv[i], &usTemp);
        printf("argv[%d] = %d.\n", i, usTemp);
    }

	// Reset timer
	s_fInactivityTimer	= 0.0f;
	s_fGameplayTimer	= 0.0f;

	// Start the demo and get all the parameters from the program that launches it
	// (AW - 14.12.2004) - Note that we currently ignore network parameters.
	sceDemoStart(argc, argv,
				 &s_xDemoParms.usLanguage,
				 &s_xDemoParms.usAspect,
				 &s_xDemoParms.usPlaymode,
				 &s_xDemoParms.usToInactive,
				 &s_xDemoParms.usToGameplay,
				 NULL,
				 NULL,
				 NULL);

	if (s_xDemoParms.usPlaymode == SCE_DEMO_PLAYMODE_ONLINE_PLAYABLE)
	{
		s_xDemoParms.usPlaymode = SCE_DEMO_PLAYMODE_PLAYABLE;
	}
	else if (s_xDemoParms.usPlaymode == SCE_DEMO_PLAYMODE_ONLINE_ATTRACT)
	{
		s_xDemoParms.usPlaymode = SCE_DEMO_PLAYMODE_ATTRACT;
	}
    
	// Set the max inactivity
	if (argc >= 6)
    {
        s_fMaxInactivity = static_cast<float>(s_xDemoParms.usToInactive);
	    s_fMaxGameplay   = static_cast<float>(s_xDemoParms.usToGameplay);
    }
    else
    {
        printf("argc = %d, using default timeouts.\n", argc);
	    s_fMaxInactivity = 30.0f;
	    s_fMaxGameplay   = 4.0f * 60.0f;
    }
    printf("Max inactivity = %f. Max gameplay = %f.\n", Asura_Maths::FtoD(s_fMaxInactivity), Asura_Maths::FtoD(s_fMaxGameplay));
    printf("Playmode = %d.\n", s_xDemoParms.usPlaymode);
}

void Asura_PS2_DemoSystem::Shutdown()
{
	Asura_Assert(s_eReasonForEnding != -1, "Reason for ending demo not set");
	if (s_eReasonForEnding == -1)
	{
		// Set to a valid entry.
		s_eReasonForEnding = SCE_DEMO_ENDREASON_PLAYABLE_QUIT;
	}
	printf("Reason for ending = %d", s_eReasonForEnding);
	sceDemoEnd(s_eReasonForEnding);
}

void Asura_PS2_DemoSystem::Update(float fFrameTime, float fGameTime)
{
	// Gameplay timeout
	if(s_bGamePlayStarted)
	{
		s_fGameplayTimer += fGameTime;
	}

    if (s_xDemoParms.usPlaymode == SCE_DEMO_PLAYMODE_PLAYABLE)
    {
	    // Inactivity timeout
		if(Asura_UserInput::HasAnyKeyBeenPressed() || Asura_UserInput::HasAnyAxisBeenMoved())
	    {
		    s_fInactivityTimer = 0.0f;
	    }
	    else
	    {
		    s_fInactivityTimer += fFrameTime;
	    }
    }

	// Start + select pressed
	if(Asura_UserInput::KeyPressed(ASURA_PS2_KEYID_PAD1_START) &&
		Asura_UserInput::KeyPressed(ASURA_PS2_KEYID_PAD1_SELECT))
	{
		s_fPressButtonTimer += fFrameTime;
	}
	else
	{
		s_fPressButtonTimer = 0.0f;
	}
}

void Asura_PS2_DemoSystem::SetEndReason(sceDemoEndReason eReasonForEnding)
{
	s_eReasonForEnding = eReasonForEnding;
}


u_short Asura_PS2_DemoSystem::GetLanguage(void)
{
	return s_xDemoParms.usLanguage;
}

u_short Asura_PS2_DemoSystem::GetAspectRatio(void)
{
	return s_xDemoParms.usAspect;
}

u_short Asura_PS2_DemoSystem::GetPlaymode(void)
{
	return s_xDemoParms.usPlaymode;
}

void Asura_PS2_DemoSystem::SetPlaymode(u_short uPlaymode)
{
	s_xDemoParms.usPlaymode = uPlaymode;
}

void  Asura_PS2_DemoSystem::ResetGameplayTimer(void)
{
	s_fGameplayTimer = 0.0f;
	s_bGamePlayStarted = true;
}

bool  Asura_PS2_DemoSystem::GoToAttractMode(void)
{
	if(s_fInactivityTimer >= s_fMaxInactivity)
	{
		s_fInactivityTimer = 0.0f;
		return true;
	}
	return false;
}

bool  Asura_PS2_DemoSystem::GameplayTimerExpired(void)
{
	if(s_fGameplayTimer >= s_fMaxGameplay)
	{
		s_bGamePlayStarted = false;
		return true;
	}
	return false;
}

bool  Asura_PS2_DemoSystem::IsQuitRequested(void)
{
	if(s_fPressButtonTimer >= fBUTTONPRESSTIMEOUT)
	{
		return true;
	}
	return false;
}

void Asura_PS2_DemoSystem::FakeInput()
{
    s_fInactivityTimer = 0.0f;
}

#endif // #ifdef ASURA_PS2_DEMODISK
