//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_Eyetoy.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     PS2 Eyetoy support
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_EYETOY_H___
#define ___ASURA_PS2_EYETOY_H___ 1

#include "core/Asura_PlatformBasics.h"
#include "core/Asura_Maths.h"

class Asura_PS2_Eyetoy_Rectangle
{
    public:
        Asura_PS2_Eyetoy_Rectangle(void)
        {
            m_iStartX = 0;
            m_iStartY = 0;
            m_iWidth  = 0;
            m_iHeight = 0;
        }

        Asura_PS2_Eyetoy_Rectangle(int iStartX, int iStartY, int iWidth, int iHeight)
        {
            m_iStartX = iStartX;
            m_iStartY = iStartY;
            m_iWidth  = iWidth;
            m_iHeight = iHeight;
        }

        int m_iStartX;
        int m_iStartY;
        int m_iWidth;
        int m_iHeight;
};

#define USE_RGB16 (1)
typedef u_int   pixel_t;

class Asura_PS2_Eyetoy
{
    public:
        static void Initialise(void);
        static void Shutdown(void);
        static void Update(void);

        static void SetResolutionAndFPS(int iResolution, u_int uFrameRate);
        static int  GetResolution(void);
        static int  GetFPS(void);
        static bool Available(void) {return s_bCameraAvailable;}
        static pixel_t* GetImageData(void);

        static void DisplayFrame(float fAlpha);

        static void StartCapture(void);
        static void StopCapture(void);
        static bool Capturing(void){return (s_bCameraAvailable && s_bAttemptStreaming);}

        // set the previous frame as the background frame
        static void SetBackgroundImage(void);
        // delete the background
        static void SubtractBackground(const int iTolerance);
        // make the image less mpeg blocky - not implemented (yet!)
        static void SetDeblock(bool bDeblock){s_bDeblock = bDeblock;}
        // loop through the image and adjust the alpha byte
        static void AdjustAlpha(u_char uAlphaValue);
        // adjust the alpha byte in a region of the image, nb width *MUST* be a multiple of 4
        static void AdjustAlpha(const Asura_PS2_Eyetoy_Rectangle& xImageArea, u_char uAlphaValue);
        // set the reference image for the patch matching
        static void  SetReferenceImage(int iReferenceImageNumber);    // whole image
        static void  SetReferenceImage(int iReferenceImageNumber, const Asura_PS2_Eyetoy_Rectangle& xImageArea); // sub area of image
        static float CompareWithReferenceImage(int iReferenceImageNumber);

        static bool  ReferenceImageSet(int iReferenceImageNumber);
        static void  LoadIRXModule(void);

    private:
        static bool         s_bCameraAvailable;
        static bool         s_bFreezeFlag;
        static bool         s_bAttemptStreaming;
        static bool         s_bDeblock;

        static void ClearBuffer(void);
        static void UpdateSony(void);
        static void UpdateLogitech(void);
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_EYETOY_H___
