/* Copyright (C) 2002 Sony Computer Entertainment America */
/*        All Rights Reserved.  SCEA Confidential         */

#ifndef SCEIMAGE_CONVERT_H
#define SCEIMAGE_CONVERT_H

// convert from blocks generated from .mpg file format to grayscale image
void unpack_mpg(void *packed, sceImage16& luminance);

// convert from blocks generated from .mpg file format to R,G,B images
void unpackRGB_mpg(void *packed, sceImage16& r, sceImage16& g, sceImage16& b);
void unpackRGB_mpg(void *packed, sceImage8& r, sceImage8& g, sceImage8& b);

// convert from blocks generated from .ipu file format to R,G,B images
void unpackIpuRGB32(void *packed, sceImage16& r, sceImage16& g, sceImage16& b);
void unpackIpuRGB32(void *packed, sceImage8& r, sceImage8& g, sceImage8& b);
void unpackIpuRGB32(void *src, sceImageRGBA& rgba);
void unpackIpuRGB16(void *src, sceImageRGB16& rgb);

// convert from packed rgba format to R,G,B images
void unpackRGB(void *src, sceImage16& r, sceImage16& g, sceImage16& b);
void unpackRGB(void *src, sceImage8& r, sceImage8& g, sceImage8& b);

// replicate grayscale image in RGBA32 format.
void pack(const sceImage16& luminance, sceImageRGBA& packed);
void pack(const sceImage8& luminance, sceImageRGBA& packed);

void unpack(sceImageRGBA& _rgba, sceImage8& _mono, int floor=20);
void unpack(sceImageRGBA& _rgba, sceImage16& _mono, int floor=20);

// pack R,G,B images into RGBA32 format.  A is set to 128.
void packRGB(const sceImage16& _srcr, const sceImage16& _srcg, const sceImage16& _srcb, sceImageRGBA& _dest);
void packRGB(const sceImage8& _srcr, const sceImage8& _srcg, const sceImage8& _srcb, sceImageRGBA& _dest);

// pack R,G,B,A images into RGBA32 format
void packRGBA(const sceImage16& _srcr, const sceImage16& _srcg, const sceImage16& _srcb, const sceImage16& _srca, sceImageRGBA& _dest);
void packRGBA(const sceImage8& _srcr, const sceImage8& _srcg, const sceImage8& _srcb, const sceImage8& _srca, sceImageRGBA& _dest);

void packYCbCr(const sceImage16& _srcy, const sceImage16& _srcu, const sceImage16& _srcv, sceImageRGBA& _dest);
void packYCbCr(const sceImage8& _srcy, const sceImage8& _srcu, const sceImage8& _srcv, sceImageRGBA& _dest);

void packYCbCrA(const sceImage16& _srcy, const sceImage16& _srcu, const sceImage16& _srcv, const sceImage16& _srca, sceImageRGBA& _dest);
void packYCbCrA(const sceImage8& _srcy, const sceImage8& _srcu, const sceImage8& _srcv, const sceImage8& _srca, sceImageRGBA& _dest);

void packYUVA(const sceImage16& _srcy, const sceImage16& _srcu, const sceImage16& _srcv, const sceImage16& _srca, sceImageRGBA& _dest);

void packYUV(const sceImage16& _srcy, const sceImage16& _srcu, const sceImage16& _srcv, sceImageRGBA& _dest);
void packYUV(const sceImage8& _srcy, const sceImage8& _srcu, const sceImage8& _srcv, sceImageRGBA& _dest);

void pack(const sceImage8& _srcy, const sceImage8& _srcu, 
	  const sceImage8& _srcv, sceImageRGBA& _dest, int input_type=0);

void pack(const sceImage16& _srcy, const sceImage16& _srcu, 
	  const sceImage16& _srcv, sceImageRGBA& _dest, int input_type=0);


// The following are VERY special-purpose (for some of my demos)

// pack R,G,B images, but pixels where A is not zero are made grayscale
void packRGBcolorize(const sceImage16& _srcr, const sceImage16& _srcg, const sceImage16& _srcb, const sceImage16& _srca, sceImageRGBA& _dest);

// alter rgb images such that saturation is maximum and luminance 
// is .5 (preserve hue)
void justHue(sceImage16& r, sceImage16& g, sceImage16& b);

// alter rgb images as above, then filter out pixels with different hue
// than the pixel at image_loc; if image_loc<0, filter against hue0
void filterHue(sceImage16& r, sceImage16& g, sceImage16& b, int image_loc, float hue0=1.0f);


// convert from blocks generated from .mpg file format to Y,U,V images
void unpackIpuRAW16(void *packed, sceImage16& y, sceImage16& u, sceImage16& v);
void unpackIpuRAW16(void *packed, sceImage8& y, sceImage8& u, sceImage8& v);
#endif

