/* Copyright (C) 2002 Sony Computer Entertainment America */
/*        All Rights Reserved.  SCEA Confidential         */

#ifndef SCEIMAGE_MATCH_H
#define SCEIMAGE_MATCH_H

// the two routines below can be used to find the best match of
// a reference image region to a region in another image.  The
// match score is computed from a sum of absolute differences between
// image regions.

// size of the patch to be matched
#define PATCHW 16 // this must be a multiple of 8
#define PATCHH 12

// copies data from _i1 to refpatch, (0,0) in refpatch will be filled with
// (x,y) in _i1.  refpatch is resized to be PATCHWxPATCHH.
void constructReferencePatch(const sceImage& _i1, int x, int y, sceImage& refpatch);

// matches refpatch to _i1 such that 0,0 of refpatch is searched for at
// (x-searchx...x+searchx, y-searchy...y+searchy)
// the subpixel resulting offset is returned in (dx,dy), and the confidence
// of this result is returned in conf
void matchPatch(const sceImage& refpatch, const sceImage& _i1, int x, int y, float&dx, float&dy, float&conf, int searchx=8, int searchy=8);

#endif
