
#include <eekernel.h>
#include <eeregs.h>
#include <libgraph.h>
#include <stdio.h>
#include <sifdev.h>
#include <sifrpc.h>
#include <string.h>

#include "SCE_USB_Guncon2.h"

#define SSIZE 0x20
static sceSifClientData cdUsbGun;
//static u_int sdata[SSIZE/sizeof(u_int)] __attribute__((aligned(64)));
static u_char sdata[SSIZE] __attribute__((aligned(64)));
//static u_int rdata[SSIZE/sizeof(u_int)] __attribute__((aligned(64)));

static nm_TimerRegAdr_t	nm_TimerRegAdr[] = {
	{ T0_COUNT, T0_MODE, T0_COMP, T0_HOLD },
	{ T1_COUNT, T1_MODE, T1_COMP, T1_HOLD },
	//{ T2_COUNT, T2_MODE, T2_COMP, T2_HOLD },
	//{ T3_COUNT, T3_MODE, T3_COMP, T3_HOLD },
};

static int	nm_MeterTimerNo = 0;
static int 	time0,time1 ;

/*---------------------------------------------
  Function Name	: nmUsbGunInit
  function     	: RPC register
  Input Data	: none
  Output Data   : none
  Return Value	: 0(Standard termination),not0(Irregular termination)
----------------------------------------------*/
int nmUsbGunInit(void)
{
	unsigned int i;
	
	sceSifInitRpc(0);

	while( 1 ) {
		if (sceSifBindRpc( &cdUsbGun, TC2_RPC_USB_GUN, 0) < 0) {
    #ifdef _DEBUG
			printf("bind errr\n");
    #endif
			while( 1 );
		}
		if (cdUsbGun.serve != 0) break;
		i = 0x10000;
		while(i --);
	}
	
	return 0;
}

/*---------------------------------------------
  Function Name	: nmUsbGunGetInfo
  function     	: Obtaining Guncon connection information
  Input Data	: none
  Output Data   : info(Guncon connection information)
                  *Requires 64byte alignmented buffer of SSIZE
                   info[0]: Maximum number of connections possible m
                   info[1]:Current number of connection
		   info[2]:Connection information of Gun No.0 (1 when connected)
                     :
                   info[m+1]:Connection information of Gun No.(m-1) (1 when connected)
  Return Value	: 0(Regular termination),not0(Irregular termination)
----------------------------------------------*/
int nmUsbGunGetInfo(u_char *info)
{
	int ret;
        
	ret = sceSifCheckStatRpc((sceSifRpcData*)&cdUsbGun) ;
	if (ret) 
    {
    #ifdef _DEBUG
        printf ("not complete get info!!\n") ;
    #endif
        return 0;
    }

	ret = sceSifCallRpc(&cdUsbGun, 1, SIF_RPCM_NOWAIT,
	              &sdata[0],SSIZE, info, SSIZE, 0, 0);
	if (ret != 0) { return ret; }
	
	return 0;
}

/*---------------------------------------------
  Function Name	: nmUsbGunRead
  function     	: Lead for Gun data
  Input Data	: no(Gun No) * Not device ID
  Output Data   : data(Gun data) 
                  *Requires 64byte alignmented buffer of SSIZE
		   data[0]:Gun data size n (0 when blank)
		   data[1]:SW0
		   data[2]:SW1
                   data[3]:X(LOWER8bit)
		   data[4]:X(UPPER8bit)
		   data[5]:Y(LOWER8bit)
		   data[6]:Y(UPPER8bit)
  Return Value	: 0(Regular termination),not0(Irregular termination)
----------------------------------------------*/
int nmUsbGunRead(u_int no,char *data)
{
	int ret;
	
	sdata[0] = no;
	
	ret = sceSifCheckStatRpc((sceSifRpcData*)&cdUsbGun) ;
	if (ret) 
    {
    #ifdef _DEBUG
        printf ("not complete read data!!\n") ;
    #endif
        return 0;
    }

	ret = sceSifCallRpc(&cdUsbGun, 2, SIF_RPCM_NOWAIT,
	              &sdata[0],SSIZE, data, SSIZE, 0, 0);
	if (ret != 0) { return ret; }
	
	return 0;
}

/*---------------------------------------------
  Function Name	: nmUsbGunGetLocation
  function     	: Obtaining Gun connection location
  Input Data	: no(Gun No) * Not device ID.
  Output Data   : location(Gun connection location)
                  *Requires 64byte alignmented buffer of SSIZE
		   location[0]:Console Port no. (0 when doesn't exist)
		   location[1]:HUB1 Port number (0 when not existing)
		   location[2]:HUB2 Port number (0 when not existing)
		   location[3]:HUB3 Port number (0 when not existing)
		   location[4]:HUB4 Port number (0 when not existing)
		   location[5]:HUB5 Port number (0 when not existing)
		   location[6]:ɂO
  Return Value	: 0(Regular termination),not0(Irregular termination)
----------------------------------------------*/
int nmUsbGunGetLocation(int no,u_char *location)
{
	int ret;
	
	sdata[0] = no;
        
	ret = sceSifCheckStatRpc((sceSifRpcData*)&cdUsbGun) ;
	if (ret) 
    {
    #ifdef _DEBUG
        printf ("not complete get location!!\n") ;
    #endif
        return 0;
    }

	ret = sceSifCallRpc(&cdUsbGun, 3, SIF_RPCM_NOWAIT,
	              &sdata[0],SSIZE, location, SSIZE, 0, 0);
	if (ret != 0) { return ret; }
	
	return 0;
}

/*---------------------------------------------
  Function Name	: nmUsbGunGetConfig
  function     	: Obtain Guncon configuration information
----------------------------------------------*/
int nmUsbGunGetConfig(u_char *config)
{
	int ret;

	ret = sceSifCallRpc(&cdUsbGun, 4, SIF_RPCM_NOWAIT,
	              &sdata[0],SSIZE, config, SSIZE, 0, 0);
	if (ret != 0) { return ret; }
	
	return 0;
}
/*---------------------------------------------
  Function Name	: nmUsbTransCtrl
  function     	: Guncon data output
  Input Data	: no(Gun No) *Not Device ID 
                  data(Gun Data) 
                   *Requires 64byte alignmented buffer of SSIZE
                   data[0]:X Offset(LOWER8bit)
		   data[1]:X Offset(UPPER8bit)
		   data[2]:Y Offset(LOWER8bit)
		   data[3]:Y Offset(UPPER8bit)
		   data[4]:NoUse
		   data[5]:HDS(1bit)
  Output Data   : none
  Return Value	: 0(Regular termination),not0(Irregular termination)
----------------------------------------------*/
int nmUsbTransCtrl(int no,u_char *data)
{
	int ret;

	sdata[0] = no;
	memcpy(&sdata[1],data,6) ;
        
	ret = sceSifCallRpc(&cdUsbGun, 5, SIF_RPCM_NOWAIT,
	              &sdata[0],SSIZE, data, SSIZE, 0, 0);
	if (ret != 0) { return ret; }
	
	return 0;
}
/*---------------------------------------------
  Function Name	: nmUsbGunSync
  function     	: Wait until executing command finishes 
  Input Data	: mode(0:block 1:non-block)
  Output Data   : none
  Return Value	: 0(End),not0(in execution)
----------------------------------------------*/
int nmUsbGunSync(int mode) 
{
        int i;

	switch(mode)
	{
		case 0:
			i = 0;
			while(sceSifCheckStatRpc((sceSifRpcData *) &cdUsbGun) != 0){
				i ++;
//				if ((i%1000)==0) printf("loop cnt %d \n",i );
			}
			return 0;
			break;
		case 1:
		return sceSifCheckStatRpc((sceSifRpcData *) &cdUsbGun);
	}
	return 0 ;
}

void nmPadTimerReset(void)
{
	int	no=nm_MeterTimerNo ;

	time0=*nm_TimerRegAdr[no].count ;
}

void nmPadWaitTimer(int time)
{
	int	no=nm_MeterTimerNo ;
	int	i ;

	while(1) {
		time1=*nm_TimerRegAdr[no].count ;
		if (((time1-time0)&0xffff)>=time) break ;
		for (i=0;i<0x3ff;i++) ;		// dummy loop(wait)
	}
}
