//////////////////////////////////////////////////////////////////////////////
//
// SCE_USB_Guncon2.h
//
// Created by Namco.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Header for USB library for Guncon2
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___SCE_USB_GUNCON2_H___
#define ___SCE_USB_GUNCON2_H___ 1

extern "C"
{

#define TC2_RPC_USB_GUN		0x76510600

typedef struct nm_TimerRegAdr_tag {
	volatile u_int	*count;
	volatile u_int	*mode;
	volatile u_int	*comp;
	volatile u_int	*hold;
} nm_TimerRegAdr_t;

/*---------------------------------------------
  Function Name	: nmUsbGunInit
  function     	: RPC register
  Input Data	: none
  Output Data   : none
  Return Value	: 0(Standard termination),not0(Irregular termination)
----------------------------------------------*/
int nmUsbGunInit(void);

/*---------------------------------------------
  Function Name	: nmUsbGunGetInfo
  function     	: Obtaining Guncon connection information
  Input Data	: none
  Output Data   : info(Guncon connection information)
                  *Requires 64byte alignmented buffer of SSIZE
                   info[0]: Maximum number of connections possible m
                   info[1]:Current number of connection
		   info[2]:Connection information of Gun No.0 (1 when connected)
                     :
                   info[m+1]:Connection information of Gun No.(m-1) (1 when connected)
  Return Value	: 0(Regular termination),not0(Irregular termination)
----------------------------------------------*/
int nmUsbGunGetInfo(unsigned char *info);

/*---------------------------------------------
  Function Name	: nmUsbGunRead
  function     	: Lead for Gun data
  Input Data	: no(Gun No) * Not device ID
  Output Data   : data(Gun data) 
                  *Requires 64byte alignmented buffer of SSIZE
		   data[0]:Gun data size n (0 when blank)
		   data[1]:SW0
		   data[2]:SW1
                   data[3]:X(LOWER8bit)
		   data[4]:X(UPPER8bit)
		   data[5]:Y(LOWER8bit)
		   data[6]:Y(UPPER8bit)
  Return Value	: 0(Regular termination),not0(Irregular termination)
----------------------------------------------*/
int nmUsbGunRead(unsigned int no,char *data);

/*---------------------------------------------
  Function Name	: nmUsbGunGetLocation
  function     	: Obtaining Gun connection location
  Input Data	: no(Gun No) * Not device ID.
  Output Data   : location(Gun connection location)
                  *Requires 64byte alignmented buffer of SSIZE
		   location[0]:Console Port no. (0 when doesn't exist)
		   location[1]:HUB1 Port number (0 when not existing)
		   location[2]:HUB2 Port number (0 when not existing)
		   location[3]:HUB3 Port number (0 when not existing)
		   location[4]:HUB4 Port number (0 when not existing)
		   location[5]:HUB5 Port number (0 when not existing)
		   location[6]:ɂO
  Return Value	: 0(Regular termination),not0(Irregular termination)
----------------------------------------------*/
int nmUsbGunGetLocation(int no,unsigned char *location);

/*---------------------------------------------
  Function Name	: nmUsbGunGetConfig
  function     	: Obtain Guncon configuration information
----------------------------------------------*/
int nmUsbGunGetConfig(unsigned char *config) ;

/*---------------------------------------------
  Function Name	: nmUsbTransCtrl
  function     	: Guncon data output
  Input Data	: no(Gun No) *Not Device ID 
                  data(Gun Data) 
                   *Requires 64byte alignmented buffer of SSIZE
                   data[0]:X Offset(LOWER8bit)
		   data[1]:X Offset(UPPER8bit)
		   data[2]:Y Offset(LOWER8bit)
		   data[3]:Y Offset(UPPER8bit)
		   data[4]:NoUse
		   data[5]:HDS(1bit)
  Output Data   : none
  Return Value	: 0(Regular termination),not0(Irregular termination)
----------------------------------------------*/
int nmUsbGunSync(int mode) ;

/*---------------------------------------------
  Function Name	: nmUsbGunSync
  function     	: Wait until executing command finishes 
  Input Data	: mode(0:block 1:non-block)
  Output Data   : none
  Return Value	: 0(End),not0(in execution)
----------------------------------------------*/
int nmUsbTransCtrl(int no,unsigned char *data) ;

void nmPadTimerReset(void) ;
void nmPadWaitTimer(int time) ;

}

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___SCE_USB_GUNCON2_H___
