//////////////////////////////////////////////////////////////////////////////
//
// Asura_HIDToAscii.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     HID to Ascii conversion array
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_HIDTOASCII_H___
#define ___ASURA_HIDTOASCII_H___ 1

#include "Input/Logitech/hidkbcode.h"

const u_short HID_TO_ASCII[] =
{
    '\0',         // HK_NO_KEY           0x00
    '\0',         // HK_ERR_ROLL_OVER    0x01
    '\0',         // HK_ERR_POST_FAIL    0x02
    '\0',         // HK_ERR_UNDEFINED    0x03

    'a',          // HK_A                0x04
    'b',          // HK_B                0x05
    'c',          // HK_C                0x06
    'd',          // HK_D                0x07
    'e',          // HK_E                0x08
    'f',          // HK_F                0x09
    'g',          // HK_G                0x0a
    'h',          // HK_H                0x0b
    'i',          // HK_I                0x0c
    'j',          // HK_J                0x0d
    'k',          // HK_K                0x0e
    'l',          // HK_L                0x0f
    'm',          // HK_M                0x10
    'n',          // HK_N                0x11
    'o',          // HK_O                0x12
    'p',          // HK_P                0x13
    'q',          // HK_Q                0x14
    'r',          // HK_R                0x15
    's',          // HK_S                0x16
    't',          // HK_T                0x17
    'u',          // HK_U                0x18
    'v',          // HK_V                0x19
    'w',          // HK_W                0x1A
    'x',          // HK_X                0x1B
    'y',          // HK_Y                0x1C
    'z',          // HK_Z                0x1D
    '1',          // HK_1                0x1E
    '2',          // HK_2                0x1F
    '3',          // HK_3                0x20
    '4',          // HK_4                0x21
    '5',          // HK_5                0x22
    '6',          // HK_6                0x23
    '7',          // HK_7                0x24
    '8',          // HK_8                0x25
    '9',          // HK_9                0x26
    '0',          // HK_0                0x27

    13,           // HK_ENTER            0x28
    27,           // HK_ESCAPE           0x29
    8,            // HK_BACKSPACE        0x2A
    '\t',         // HK_TAB              0x2B
    32,           // HK_SPACEBAR         0x2C

    '-',          // HK_MINUS            0x2D
    '=',          // HK_EQUAL            0x2E
    '[',          // HK_SQ_OPEN_BRACKET  0x2F
    ']',          // HK_SQ_CLOSE_BRACKET 0x30
    '\\',         // HK_BACK_SLASH       0x31
    '#',          // HK_NON_HASH         0x32
    ';',          // HK_SEMICOLON        0x33
    '\'',         // HK_QUOTE            0x34
    '`',          // HK_GRAVE_ACCENT     0x35
    ',',          // HK_COMMA            0x36
    '.',          // HK_DOT              0x37
    '/',          // HK_FORWARD_SLASH    0x38

// All charactors upto FORWARD_SLASH has corresponding ASCII codes.
};

const u_short ASURA_END_OF_HID_TO_ASCII = HK_FORWARD_SLASH;

// Keys that don't have Ascii values but which convert to Asura_KEY_IDs

const ASURA_KEYID HID_TO_KEY_ID[] = 
{
    ASURA_KEYID_CAPSLOCK,       //HK_CAPS_LOCK        0x39
    ASURA_KEYID_F1,             //HK_F1               0x3A
    ASURA_KEYID_F2,             //HK_F2               0x3B
    ASURA_KEYID_F3,             //HK_F3               0x3C
    ASURA_KEYID_F4,             //HK_F4               0x3D
    ASURA_KEYID_F5,             //HK_F5               0x3E
    ASURA_KEYID_F6,             //HK_F6               0x3F
    ASURA_KEYID_F7,             //HK_F7               0x40
    ASURA_KEYID_F8,             //HK_F8               0x41
    ASURA_KEYID_F9,             //HK_F9               0x42
    ASURA_KEYID_F10,            //HK_F10              0x43
    ASURA_KEYID_F11,            //HK_F11              0x44
    ASURA_KEYID_F12,            //HK_F12              0x45
    ASURA_KEYID_VOID,           //HK_PRINT_SCR        0x46
    ASURA_KEYID_SCROLLOK,       //HK_SCROLL           0x47
    ASURA_KEYID_PAUSE,          //HK_PAUSE            0x48
    ASURA_KEYID_INS,            //HK_INSERT           0x49
    ASURA_KEYID_HOME,           //HK_HOME             0x4A
    ASURA_KEYID_PAGEUP,         //HK_PAGE_UP	      0x4b
                                
    ASURA_KEYID_DEL,            //HK_DELETE           0x4C
    ASURA_KEYID_END,            //HK_END              0x4D
    ASURA_KEYID_PAGEDOWN,       //HK_PAGE_DOWN		  0x4e
                                
    ASURA_KEYID_RIGHT,          //HK_RIGHT_ARROW      0x4F
    ASURA_KEYID_LEFT,           //HK_LEFT_ARROW       0x50
    ASURA_KEYID_DOWN,           //HK_DOWN_ARROW       0x51
    ASURA_KEYID_UP,             //HK_UP_ARROW         0x52
                                
    ASURA_KEYID_NUMLOCK,        //HK_KP_NUMLOCK       0x53

    ASURA_KEYID_NUMPADDIVIDE,   //HK_KP_SLASH         0x54
    ASURA_KEYID_NUMPADMULTIPLY, //HK_KP_STAR          0x55
    ASURA_KEYID_NUMPADSUB,      //HK_KP_MINUS         0x56
    ASURA_KEYID_NUMPADADD,      //HK_KP_PLUS          0x57
    ASURA_KEYID_NUMPADENTER,    //HK_KP_ENTER         0x58
    ASURA_KEYID_NUMPAD1,        //HK_KP_END_1         0x59
    ASURA_KEYID_NUMPAD2,        //HK_KP_DOWN_ARROW_2  0x5A
    ASURA_KEYID_NUMPAD3,        //HK_KP_PAGE_DOWN_3   0x5B
    ASURA_KEYID_NUMPAD4,        //HK_KP_LEFT_ARROW_4  0x5C
    ASURA_KEYID_NUMPAD5,        //HK_KP_5             0x5D
    ASURA_KEYID_NUMPAD6,        //HK_KP_RIGHT_ARROW_6 0x5E
    ASURA_KEYID_NUMPAD7,        //HK_KP_HOME_7        0x5F
    ASURA_KEYID_NUMPAD8,        //HK_KP_UP_ARROW_8    0x60
    ASURA_KEYID_NUMPAD9,        //HK_KP_PAGE_UP_9     0x61
    ASURA_KEYID_NUMPAD0,        //HK_KP_INSERT_0      0x62
    ASURA_KEYID_NUMPADDEL,      //HK_KP_DEL_DOT       0x63
    (ASURA_KEYID)('\\'),        //HK_NON_US_SLASH     0x64
    ASURA_KEYID_VOID,           //HK_KP_EQUAL         0x67
};

const u_short ASURA_END_OF_HID_TO_ASURA_KEY_ID = HK_KP_EQUAL;

const ASURA_KEYID PAD_HID_TO_KEY_ID_NUMLOCK [] =
{
    ASURA_KEYID_NUMPADDIVIDE,   //HK_KP_SLASH         0x54
    ASURA_KEYID_NUMPADMULTIPLY, //HK_KP_STAR          0x55
    ASURA_KEYID_NUMPADSUB,      //HK_KP_MINUS         0x56
    ASURA_KEYID_NUMPADADD,      //HK_KP_PLUS          0x57
    ASURA_KEYID_NUMPADENTER,    //HK_KP_ENTER         0x58
    ASURA_KEYID_END,            //HK_KP_END_1         0x59
    ASURA_KEYID_DOWN,           //HK_KP_DOWN_ARROW_2  0x5A
    ASURA_KEYID_PAGEDOWN,       //HK_KP_PAGE_DOWN_3   0x5B
    ASURA_KEYID_LEFT,           //HK_KP_LEFT_ARROW_4  0x5C
    ASURA_KEYID_NUMPAD5,        //HK_KP_5             0x5D
    ASURA_KEYID_RIGHT,          //HK_KP_RIGHT_ARROW_6 0x5E
    ASURA_KEYID_HOME,           //HK_KP_HOME_7        0x5F
    ASURA_KEYID_UP,             //HK_KP_UP_ARROW_8    0x60
    ASURA_KEYID_PAGEUP,         //HK_KP_PAGE_UP_9     0x61
    ASURA_KEYID_INS,            //HK_KP_INSERT_0      0x62
    ASURA_KEYID_DEL,            //HK_KP_DEL_DOT       0x63
};

const u_short ASURA_START_OF_NUM_PAD_HID = HK_KP_SLASH;
const u_short ASURA_END_OF_NUM_PAD_HID   = HK_KP_DOT;

// Convert modifier keys (ctrl, shift, alt etc.)

const u_short ASURA_START_OF_HID_MOD_KEY = HK_LEFT_CTRL;
const u_short ASURA_END_OF_HID_MOD_KEY   = HK_RIGHT_GUI;

const ASURA_KEYID HID_TO_MOD_KEY_ID [] =
{
    ASURA_KEYID_LEFTCTRL,       //HK_LEFT_CTRL        0xE0
    ASURA_KEYID_LEFTSHIFT,      //HK_LEFT_SHIFT       0xE1
    ASURA_KEYID_LEFTALT,        //HK_LEFT_ALT         0xE2
    ASURA_KEYID_LWIN,           //HK_LEFT_GUI         0xE3
    ASURA_KEYID_RIGHTCTRL,      //HK_RIGHT_CTRL       0xE4
    ASURA_KEYID_RIGHTSHIFT,     //HK_RIGHT_SHIFT      0xE5
    ASURA_KEYID_RIGHTALT,       //HK_RIGHT_ALT        0xE6
    ASURA_KEYID_RWIN,           //HK_RIGHT_GUI        0xE7
};

int GetAsuraKeyIDFromHID(u_short usHID, bool bNumLock)
{
    // Check range
    if(usHID <= ASURA_END_OF_HID_TO_ASCII)
    {
        return HID_TO_ASCII[usHID];
    }
    else if(usHID >= ASURA_START_OF_NUM_PAD_HID &&
            usHID <= ASURA_END_OF_NUM_PAD_HID &&
            !bNumLock)
    {
        return PAD_HID_TO_KEY_ID_NUMLOCK[usHID - ASURA_START_OF_NUM_PAD_HID];
    }
    else if(usHID <= ASURA_END_OF_HID_TO_ASURA_KEY_ID)
    {
        return HID_TO_KEY_ID[usHID - ASURA_END_OF_HID_TO_ASCII - 1];
    }
    else if(usHID >= ASURA_START_OF_HID_MOD_KEY && usHID <= ASURA_END_OF_HID_MOD_KEY)
    {
        return HID_TO_MOD_KEY_ID[usHID - ASURA_START_OF_HID_MOD_KEY];
    }
    return ASURA_KEYID_VOID;
}

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_HIDTOASCII_H___
