//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_CommandConsole.cpp
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     PS2-specific implementations of command console functionality.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE

// Engine independent.
#include "Core/Asura_TextureFlags.h"
#include "Debugging/Asura_ScreenShot_System.h"
#include "Input/Asura_ControlMap.h"
#include "Interface/Asura_CommandConsole.h"

// Engine platform-specific.
#include "Debugging/Asura_PS2_DebugInfo.h"
#include "Input/Asura_PlatformInput.h"

// C O N S T A N T S /////////////////////////////////////////////////////////
static const Asura_UnicodeChar* CONSOLE_CMDNAME_DEBUGINFO = L"PS2DebugInfo";

// S T A T I C   F U N C T I O N S ///////////////////////////////////////////
static bool ConsoleCmd_DebugInfo( bool bHelp )
{
#ifndef ASURA_PS2_SONY_SUBMISSION
    if (bHelp)
    {
        Asura_CommandConsole::Printf( L"Toggles debug info display" );
    }
    else
    {
        Asura_PS2_DebugInfo::EnableDisplay( !Asura_PS2_DebugInfo::IsDisplayEnabled() );
    }
#endif //#ifndef ASURA_PS2_SONY_SUBMISSION
    return true;
}

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

void Asura_CommandConsole::Platform_Initialise( void )
{
    SetBorderX(7.5f);
    SetFontScale(0.55f);

    SetTexture( ASURA_TEXTURE_NONE );
    SetBackgroundColour( 0xFF000000 );

    AddCmd( CONSOLE_CMDNAME_DEBUGINFO,  ConsoleCmd_DebugInfo,     REMOVE_AT_SHUTDOWN );

// ARC - 08 March 2006 15:19:02 - Already called in Asura_CommandConsole::Initialise()
    //Asura_ScreenShot_System::AddConsoleCommands();
}

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM

void Asura_DebugVariableSystem::Platform_InitialiseDefaultControlMapping( void )
{
    s_xControlMap.AddKeyMapping( KEYMAP_MOVERIGHT,              ASURA_PS2_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_PRESS );
    s_xControlMap.AddKeyMapping( KEYMAP_MOVELEFT,               ASURA_PS2_KEYID_PAD1_LEFT,  Asura_ControlMapping::ON_PRESS );
    s_xControlMap.AddKeyMapping( KEYMAP_MOVEUP,                 ASURA_PS2_KEYID_PAD1_UP,    Asura_ControlMapping::ON_PRESS );
    s_xControlMap.AddKeyMapping( KEYMAP_MOVEDOWN,               ASURA_PS2_KEYID_PAD1_DOWN,  Asura_ControlMapping::ON_PRESS );
    s_xControlMap.AddKeyMapping( KEYMAP_INCREASEVALUE,          ASURA_PS2_KEYID_PAD1_R1,    Asura_ControlMapping::ON_PRESS );
    s_xControlMap.AddKeyMapping( KEYMAP_DECREASEVALUE,          ASURA_PS2_KEYID_PAD1_R2,    Asura_ControlMapping::ON_PRESS );
    s_xControlMap.AddKeyMapping( KEYMAP_INCREASEVALUEINCREMENT, ASURA_PS2_KEYID_PAD1_L1,    Asura_ControlMapping::ON_PRESS );
    s_xControlMap.AddKeyMapping( KEYMAP_DECREASEVALUEINCREMENT, ASURA_PS2_KEYID_PAD1_L2,    Asura_ControlMapping::ON_PRESS );

#ifdef ASURA_USE_CONTROLMAP_MANAGER
	s_xControlMap.AddKeyMapping( KEYMAP_CLOSE,					ASURA_PS2_KEYID_PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
#endif 

}

#endif

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifdef ASURA_SUPPORT_COMMAND_CONSOLE
