//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_MemoryCard_Error.h
//
// Created by Neil Massey.
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Error message system for memory card
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_MEMORYCARD_ERROR_H___
#define ___ASURA_PS2_MEMORYCARD_ERROR_H___ 1

enum
{
	ASURA_PS2_MEMORY_CARD_SUCCEED = 0,						// congratulations
	ASURA_PS2_MEMORY_CARD_ERROR_NO_CARD,
	ASURA_PS2_MEMORY_CARD_ERROR_NOT_FORMATTED,
	ASURA_PS2_MEMORY_CARD_ERROR_FULL,					// no space
	ASURA_PS2_MEMORY_CARD_ERROR_FILE_NOT_FOUND,
	ASURA_PS2_MEMORY_CARD_ERROR_PATH_NOT_FOUND,
	ASURA_PS2_MEMORY_CARD_ERROR_FILE_IN_USE,
	ASURA_PS2_MEMORY_CARD_ERROR_FILE_ALREADY_OPEN,		// file handler already used
	ASURA_PS2_MEMORY_CARD_ERROR_FILE_NOT_OPENED,		// file handler not found
	ASURA_PS2_MEMORY_CARD_ERROR_FILE_CORRUPT,
	ASURA_PS2_MEMORY_CARD_ERROR_FILE_LIMIT_REACHED,		// number of open files
	ASURA_PS2_MEMORY_CARD_ERROR_DIRECTORY_NOT_EMPTY,
	ASURA_PS2_MEMORY_CARD_ERROR_DIRECTORY_DESTROYED,	// the entry with the same name as the directory could not be found
	ASURA_PS2_MEMORY_CARD_ERROR_DIRECTORY_ALREADY_EXISTS,
    ASURA_PS2_MEMORY_CARD_ERROR_DIRECTORY_NOT_FOUND,
	ASURA_PS2_MEMORY_CARD_ERROR_CARD_CHANGED,
	ASURA_PS2_MEMORY_CARD_ERROR_WRITE_FAILED,
	ASURA_PS2_MEMORY_CARD_ERROR_READ_FAILED,
};


class Asura_PS2_MemoryCard_ErrorSystem
{
	friend class Asura_PS2_MemoryCard_SubSystem;
	friend class Asura_PS2_MemoryCard_System;

public:
	static u_int GetError(void);

private:
	static void ProcessError(const int iFunction, const int iError);
	static void ProcessCardInfoError(const int iError);
	static void ProcessOpenError(const int iError);
	static void ProcessCloseError(const int iError);
	static void ProcessSeekError(const int iError);
	static void ProcessReadError(const int iError);
	static void ProcessWriteError(const int iError);
	static void ProcessFlushError(const int iError);
	static void ProcessMkDirError(const int iError);
	static void ProcessChDirError(const int iError);
	static void ProcessGetDirError(const int iError);
	static void ProcessFileInfoError(const int iError);
	static void ProcessDeleteError(const int iError);
	static void ProcessFormatError(const int iError);
	static void ProcessUnformatError(const int iError);
	static void ProcessEntSpaceError(const int iError);
	static void ProcessRenameError(const int iError);
	static void ProcessChgPriorError(const int iError);

	static u_int s_uCurrentError;
	static u_int s_uErrorFrameStamp;		// only interested in recent errors

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_MEMORYCARD_ERROR_H___
