//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_MemoryCard_Shell.cpp
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     Shell for memory card to help with TRC
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "MemoryCard/Asura_PS2_MemoryCard_Shell.h"

// Stdlib independent.
#include <string.h>
#include <stdio.h>

// Engine independent
#include "Core/Asura_Maths.h"
#include "Core/Asura_Handle.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Logfile.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Sound/Asura_Sound.h"

// Engine platform dependent
#include "MemoryCard/Asura_PS2_MemoryCard_Error.h"
#include "Sound/Asura_PS2_Multistream.h"
#include "Core/Asura_UserInput.h"
#include "Input/Asura_PlatformInput.h"
#include "Core/Asura_Checksum.h"

/*************************** S T A T I C   V A R I A B L E S ***************************/

const float fWAIT_TIMER = 1.0f;
const float fFORMAT_COMPLETION_TIME = 4.5f;
//const float fREAD_SPEED  = 325.864448f; // KB/s Approx. read speed (from sce.dev.prog.peripherals.memorycard newsgroup)
//const float fWRITE_SPEED = 117.776384f; // KB/s Approx. write speed

Asura_PS2_MemoryCard_Shell::Phase            Asura_PS2_MemoryCard_Shell::s_eCurrentPhase     =   PHASE_INVALID;
Asura_PS2_MemoryCard_Shell::Command          Asura_PS2_MemoryCard_Shell::s_eEntryCommand     =   CMD_INVALID;
Asura_PS2_MemoryCard_Shell::State            Asura_PS2_MemoryCard_Shell::s_eCurrentState     =   STATE_INVALID;
Asura_PS2_MemoryCard_Shell::DisplayMessage   Asura_PS2_MemoryCard_Shell::s_eCurrentMessage   =   MSG_INVALID;
Asura_PS2_MemoryCard_Shell::CommandStatus    Asura_PS2_MemoryCard_Shell::s_eCurrentCommandStatus =  CMD_STATUS_INVALID;
Asura_PS2_MemoryCard_Shell::GetInputMessage  Asura_PS2_MemoryCard_Shell::s_eCurrentGetInputMessage = MSG_GI_NONE;

int     Asura_PS2_MemoryCard_Shell::s_iCurrentPort = 0;
int     Asura_PS2_MemoryCard_Shell::s_iCurrentSlot = 0;
float   Asura_PS2_MemoryCard_Shell::s_fWaitTimer = -1.0f;
int     Asura_PS2_MemoryCard_Shell::s_iDataSize = 0;
const void*   Asura_PS2_MemoryCard_Shell::s_pSaveDataPointer = NULL;
void*   Asura_PS2_MemoryCard_Shell::s_pLoadDataPointer = NULL;
int     Asura_PS2_MemoryCard_Shell::s_iIconFileSize = 0;
char    Asura_PS2_MemoryCard_Shell::s_szDirList[512] = "\0";
char*   Asura_PS2_MemoryCard_Shell::s_szSaveListOutput = Asura_PS2_MemoryCard_Shell::s_szLocalSaveList;
char    Asura_PS2_MemoryCard_Shell::s_szLocalSaveList[512];
float   Asura_PS2_MemoryCard_Shell::s_fProgressOfCurrentOperation = 0.0f;
bool    Asura_PS2_MemoryCard_Shell::s_bCreateIconAndDir = false;
bool    Asura_PS2_MemoryCard_Shell::s_bFinishedSuccessfully = false;
bool    Asura_PS2_MemoryCard_Shell::s_bCardChanged = false;
char    Asura_PS2_MemoryCard_Shell::s_szFileName[32] = "\0";
char    Asura_PS2_MemoryCard_Shell::s_szProductName[32] = "\0";
char    Asura_PS2_MemoryCard_Shell::s_szProductCode[32] = "\0";
bool    Asura_PS2_MemoryCard_Shell::s_bDisableAutosave = false;
int     Asura_PS2_MemoryCard_Shell::s_iCheckDataSize = 0;
int     Asura_PS2_MemoryCard_Shell::s_iCancelSound = ASURA_SID_NO_SOUND;
int     Asura_PS2_MemoryCard_Shell::s_iRetrySound  = ASURA_SID_NO_SOUND;
float   Asura_PS2_MemoryCard_Shell::s_fMenuSoundVolume = 0.5f;
int     Asura_PS2_MemoryCard_Shell::s_iOverrideDataSize = -1;
bool    Asura_PS2_MemoryCard_Shell::s_bSaveStarted = false;

struct  Asura_PS2_MemoryCard_Shell::BatchInfo   Asura_PS2_MemoryCard_Shell::s_xBatchInformation =
{
    false,          //bool bAutomaticallyOverwrite;
    false,          //bool bInBatchOperation;
    false,          //bool bLastOperationInBatch;
    false,          //bool bFirstOperationInBatch;
    false,          //bool bSaveWasCancelledInBatch;
    false,          //bool bUserConfirmedCardUnavailable;
    false,          //bool bUserConfirmedCardUnformatted;
    false,          //bool bUserConfirmedNoRoom;
    false,          //bool bUserCancelledOverwrite;
    false,          //bool bSilentlyDelete;
    false,          //bool bSetAsOverwrite
};

/************************* M E M B E R   F U N C T I O N S ************************/
#ifndef ASURA_PS2_DEMODISK

#ifdef ASURA_LOGFILES_ENABLED
	#define MEMCARD_LOGFILENAME "Logfiles\\MemCard.txt"
	DEBUGVAR bool dbg_bEnableMemCardLogging = false;
#endif

void Asura_PS2_MemoryCard_Shell::Initialise(void)
{
    s_eCurrentPhase     =   PHASE_INVALID;
    s_eEntryCommand     =   CMD_INVALID;
    s_eCurrentState     =   STATE_INVALID;
    s_eCurrentMessage   =   MSG_INVALID;
    s_eCurrentCommandStatus = CMD_STATUS_INVALID;
    s_bDisableAutosave  = false;
    Asura_PS2_MemoryCard_System::SetSaveNumber(1);

#ifdef ASURA_LOGFILES_ENABLED
	Asura_SimpleLogfileClear((MEMCARD_LOGFILENAME));
	Asura_SimpleLogfileOutput((MEMCARD_LOGFILENAME, "----------------------------------\n"));
	Asura_SimpleLogfileOutput((MEMCARD_LOGFILENAME, "- Asura_PS2_MemoryCard_Shell.cpp -\n"));
	Asura_SimpleLogfileOutput((MEMCARD_LOGFILENAME, "----------------------------------\n"));
	Asura_SimpleLogfileOutput((MEMCARD_LOGFILENAME, "\n"));
#endif
}

#ifdef ASURA_LOGFILES_ENABLED

Asura_Char* Asura_PS2_MemoryCard_Shell::GetStateDescription(State eState)
{
	Asura_Char* szText = NULL;
	switch(eState)
	{            
		case STATE_RUNNING:
			szText = "RUNNING";
			break;

		case STATE_ERROR:
			szText = "ERROR";
			break;
            
		case STATE_FINISHED:
			szText = "FINISHED";
			break;
				
		default:
		case STATE_INVALID:
			szText = "INVALID";
			break;
	}

	return szText;
}

Asura_Char* Asura_PS2_MemoryCard_Shell::GetPhaseDescription(Phase ePhase)
{
	Asura_Char* szText = NULL;

	switch(ePhase)
	{
		case PHASE_CHECK_FOR_CARD: szText = "CHECK_FOR_CARD"; break;
		case PHASE_CHECK_FOR_ROOM: szText = "CHECK_FOR_ROOM"; break;
		case PHASE_CHECK_FORMATTED: szText = "CHECK_FORMATTED"; break;
		case PHASE_CHECK_FOR_SAVE: szText = "CHECK_FOR_SAVE"; break;
		case PHASE_CHECK_FOR_SUB_SAVE: szText = "CHECK_FOR_SUB_SAVE"; break;
		case PHASE_FORMATTING: szText = "FORMATTING"; break;
		case PHASE_SAVING: szText = "SAVING"; break;
		case PHASE_LOADING: szText = "LOADING"; break;
		case PHASE_GET_SAVELIST: szText = "GET_SAVELIST"; break;
		case PHASE_DELETING: szText = "DELETING"; break;
		case PHASE_DELETING_SAVE_DIRECTORY: szText = "DELETING_SAVE_DIRECTORY"; break;
		case PHASE_PROMPT_CREATE_SAVE: szText = "PROMPT_CREATE_SAVE"; break;
		case PHASE_PROMPT_CARD_CHANGED: szText = "PROMPT_CARD_CHANGED"; break;
		case PHASE_PROMPT_NO_CARD: szText = "PROMPT_NO_CARD"; break;
		case PHASE_PROMPT_OVERWRITE_SUB_SAVE: szText = "PROMPT_OVERWRITE_SUB_SAVE"; break;
		case PHASE_PROMPT_SAVE_DOES_NOT_EXIST: szText = "PROMPT_SAVE_DOES_NOT_EXIST"; break;
		case PHASE_PROMPT_DELETE_CONFIRM: szText = "PROMPT_DELETE_CONFIRM"; break;
		case PHASE_FORMAT_FAILED: szText = "FORMAT_FAILED"; break;
		case PHASE_SAVE_FAILED: szText = "SAVE_FAILED"; break;
		case PHASE_LOAD_FAILED: szText = "LOAD_FAILED"; break;
		case PHASE_DELETE_FAILED: szText = "DELETE FAILED"; break;
		case PHASE_FINISHED: szText = "FINISHED"; break;
		default:
		case PHASE_INVALID: szText = "INVALID"; break;
	}

	return szText;
}

#endif

// SNH: added these two to allow logging
void Asura_PS2_MemoryCard_Shell::SetState(State eState)
{
#ifdef ASURA_LOGFILES_ENABLED
	if(dbg_bEnableMemCardLogging && eState != s_eCurrentState)
	{
		Asura_SimpleLogfileOutput((MEMCARD_LOGFILENAME, "State: %s -> %s\n", GetStateDescription(s_eCurrentState), GetStateDescription(eState)));
	}
#endif

	s_eCurrentState = eState;
}
void Asura_PS2_MemoryCard_Shell::SetPhase(Phase ePhase)
{
#ifdef ASURA_LOGFILES_ENABLED
	if(dbg_bEnableMemCardLogging && ePhase != s_eCurrentPhase)
	{
		Asura_SimpleLogfileOutput((MEMCARD_LOGFILENAME, "Phase: %s -> %s\n", GetPhaseDescription(s_eCurrentPhase), GetPhaseDescription(ePhase)));
	}
#endif
	
	s_eCurrentPhase = ePhase;
}

Asura_PS2_MemoryCard_Shell::PollResult Asura_PS2_MemoryCard_Shell::PollCard(void)
{
    Asura_PS2_MemoryCard_System::Update();

    if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
    {
        return POLL_BUSY;
    }

    PollResult ePollResult = POLL_OK;
    if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_ERROR_NO_CARD ||
        !Asura_PS2_MemoryCard_System::Available(s_iCurrentPort, s_iCurrentSlot))
    {
        ePollResult = POLL_NO_CARD;
    }
    else if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_ERROR_CARD_CHANGED)
    {
        ePollResult = POLL_CARD_CHANGED;
    }
    else if(!Asura_PS2_MemoryCard_System::Formatted(s_iCurrentPort, s_iCurrentSlot))
    {
        ePollResult = POLL_CARD_UNFORMATTED;
    }
    
    Asura_PS2_MemoryCard_System::Rescan(s_iCurrentPort, s_iCurrentSlot);
    return ePollResult;
}

/********************************** U P D A T E ***********************************/

void Asura_PS2_MemoryCard_Shell::CheckForCard(void)
{
    // Check for cards being swapped out etc...
    if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
    {
        return;
    }
    switch(Asura_PS2_MemoryCard_ErrorSystem::GetError())
    {
        case ASURA_PS2_MEMORY_CARD_ERROR_NO_CARD:
        {
            switch(s_eCurrentPhase)
            {
                case PHASE_CHECK_FOR_CARD:
                {
                    switch(s_eEntryCommand)
                    {
                        case CMD_START_SAVE:
                        case CMD_START_AUTOSAVE_MAIN:
                        case CMD_START_SAVE_MAIN:
                            if(s_bSaveStarted)
                            {
                                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
								SetPhase(PHASE_SAVE_FAILED);
                            }
                        break;
                        default:{}
                    }
                }
                break;
                case PHASE_FINISHED:
                {
                }
                break;
                case PHASE_FORMATTING:
                case PHASE_FORMAT_FAILED:
                {
                    s_fWaitTimer = 0.0f;
                }
                break;
                default:
                {
                    s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
					SetPhase(PHASE_PROMPT_NO_CARD);
                    switch(s_eEntryCommand)
                    {                        
                        case CMD_START_SAVE:
                        case CMD_START_AUTOSAVE_MAIN:
                        case CMD_START_SAVE_MAIN:
                            if(s_bSaveStarted)
                            {
								SetPhase(PHASE_SAVE_FAILED);
                            }
                        break;
                        default:{}
                    }

                    if(s_xBatchInformation.bInBatchOperation && 
                        !s_xBatchInformation.bFirstOperationInBatch)
                    {
                        switch(s_eEntryCommand)
                        {
                            case CMD_START_LOAD:
                            case CMD_START_AUTOLOAD_MAIN:
                            case CMD_START_LOAD_MAIN:
								SetPhase(PHASE_LOAD_FAILED);
                            break;
                            
                            case CMD_START_SAVE:
                            case CMD_START_AUTOSAVE_MAIN:
                            case CMD_START_SAVE_MAIN:                   
								SetPhase(PHASE_SAVE_FAILED);
                            break;

                            case CMD_START_DELETE:
                            case CMD_START_DELETE_SAVE_DIRECTORY:
								SetPhase(PHASE_DELETE_FAILED);
                            break;

                            default:
                            {
                                Asura_Assert(0, "Unknown entry command");
                            }
                        }
                    }
                }
            }
        }
        break;
        case ASURA_PS2_MEMORY_CARD_ERROR_CARD_CHANGED:
        {
            s_bCardChanged = true;
        }
        break;
        case ASURA_PS2_MEMORY_CARD_ERROR_FILE_NOT_FOUND:
        {
            if((s_eEntryCommand == CMD_START_DELETE ||
                s_eEntryCommand == CMD_START_DELETE_SAVE_DIRECTORY))
            {
                SetState(STATE_ERROR);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
				SetPhase(PHASE_DELETE_FAILED);
            }
        }
        break;
        default:
        {}
    }
}

void Asura_PS2_MemoryCard_Shell::Update(void)
{
    Asura_PS2_MemoryCard_System::Update();

    s_eCurrentGetInputMessage = MSG_GI_NONE;

    CheckForCard();

    s_fWaitTimer -= Asura_Timers::GetFrameTime();

    if(s_fWaitTimer >= 0.0f)
    {
        return;
    }

    switch(s_eCurrentPhase)
    {
        // check phases
        case PHASE_CHECK_FOR_CARD:
            ProcessCheckForCard();
        break;
        case PHASE_CHECK_FOR_ROOM:
            ProcessCheckForRoom();
        break;
        case PHASE_CHECK_FORMATTED:
            ProcessCheckFormatted();
        break;
        case PHASE_CHECK_FOR_SAVE:
            ProcessCheckForSave();
        break;
        case PHASE_CHECK_FOR_SUB_SAVE:
            ProcessCheckForSubSave();
        break;

        // "do" phases
        case PHASE_FORMATTING:
            ProcessFormatting();
        break;
        case PHASE_SAVING:
            ProcessSaving();
        break;
        case PHASE_LOADING:
            ProcessLoading();
        break;
        case PHASE_DELETING:
            ProcessDeleting();
        break;
        case PHASE_DELETING_SAVE_DIRECTORY:
            ProcessDeletingSaveDirectory();
        break;
        case PHASE_GET_SAVELIST:
            ProcessGetSaveList();
        break;

        // prompt phases
        case PHASE_PROMPT_CREATE_SAVE:
            ProcessPromptCreateSave();
        break;
        case PHASE_PROMPT_CARD_CHANGED:
            ProcessPromptCardChanged();
        break;
        case PHASE_PROMPT_OVERWRITE_SUB_SAVE:
            ProcessPromptOverwriteSubSave();
        break;
        case PHASE_PROMPT_NO_CARD:
            ProcessPromptNoCard();
        break;
        case PHASE_PROMPT_SAVE_DOES_NOT_EXIST:
            ProcessPromptSaveDoesNotExist();
        break;
        case PHASE_PROMPT_DELETE_CONFIRM:
            ProcessPromptDelete();
        break;

        // Failure phases
        case PHASE_FORMAT_FAILED:
            ProcessFormatFailed();
        break;
        case PHASE_SAVE_FAILED:
            ProcessSaveFailed();
        break;
        case PHASE_LOAD_FAILED:
            ProcessLoadFailed();
        break;
        case PHASE_DELETE_FAILED:
            ProcessDeleteFailed();
        break;

        case PHASE_FINISHED:
            NowFinished();
        break;
        case PHASE_INVALID:
            Asura_Assert(0, "Invalid phase");
        break;
    }
}

/******************************** S H U T D O W N *********************************/

void Asura_PS2_MemoryCard_Shell::Shutdown(void)
{
	SetPhase(PHASE_INVALID);
    s_eEntryCommand     =   CMD_INVALID;
    SetState(STATE_INVALID);
    s_eCurrentMessage   =   MSG_INVALID;
    s_eCurrentCommandStatus = CMD_STATUS_INVALID;
    s_eCurrentGetInputMessage = MSG_GI_NONE;
}

/************************ S E T   P R O D U C T   N A M E *************************/

void Asura_PS2_MemoryCard_Shell::SetProductName(const char* szProductName)
{
    Asura_Assert(szProductName, "No product name");
    strcpy(s_szProductName, szProductName);
}

void Asura_PS2_MemoryCard_Shell::SetProductCode(const char* szProductCode)
{
    Asura_Assert(szProductCode, "No product code");
    strcpy(s_szProductCode, szProductCode);
    Asura_PS2_MemoryCard_System::SetProductCode(szProductCode);
}

/********************** C H E C K   C A R D   O N   B O O T ***********************/

void Asura_PS2_MemoryCard_Shell::CheckCardOnBoot(void)
{
    IssueCommand(CMD_CHECK_ON_BOOT);
}

/******************************** S A V E   M A I N ********************************/

void Asura_PS2_MemoryCard_Shell::SaveMain(const void* pData, const int iDataSize)
{
    Asura_Assert(pData, "No save data");
    Asura_Assert(iDataSize > 0, "Zero data size");

    if(s_xBatchInformation.bInBatchOperation)
    {
        // Check to see whether the user has already confirmed that the card is unformatted in this batch
        bool bFormat = Asura_PS2_MemoryCard_System::Formatted(s_iCurrentPort, s_iCurrentSlot);
        if(!bFormat && 
            s_xBatchInformation.bUserConfirmedCardUnformatted)
        {
            return;
        }
    }
    s_pSaveDataPointer = pData;
    s_iDataSize = iDataSize;
    IssueCommand(CMD_START_SAVE_MAIN);
}

/******************************** A U T O S A V E   M A I N ********************************/

void Asura_PS2_MemoryCard_Shell::AutoSaveMain(const void* pData, const int iDataSize)
{
    Asura_Assert(pData, "No save data");
    Asura_Assert(iDataSize > 0, "Zero data size");

    if(s_xBatchInformation.bInBatchOperation)
    {
        // Check to see whether the user has already confirmed that the card is unformatted in this batch
        bool bFormat = Asura_PS2_MemoryCard_System::Formatted(s_iCurrentPort, s_iCurrentSlot);
        if(!bFormat && 
            s_xBatchInformation.bUserConfirmedCardUnformatted)
        {
            return;
        }
    }
    s_pSaveDataPointer = pData;
    s_iDataSize = iDataSize;
    IssueCommand(CMD_START_AUTOSAVE_MAIN);
}

/******************************** S A V E ********************************/

void Asura_PS2_MemoryCard_Shell::Save(const char* szSaveName, const void* pData, const int iDataSize)
{
    Asura_Assert(szSaveName, "No save name");
    Asura_Assert(pData, "No save data");
    Asura_Assert(iDataSize > 0, "Zero data size");

    // Explicitly requested save so this doesn't apply
    if(s_xBatchInformation.bInBatchOperation)
    {
        // Check to see whether the user has already confirmed that the card is unformatted in this batch
        bool bFormat = Asura_PS2_MemoryCard_System::Formatted(s_iCurrentPort, s_iCurrentSlot);
        if(!bFormat && 
            s_xBatchInformation.bUserConfirmedCardUnformatted)
        {
            return;
        }
    }
    else
    {
        s_xBatchInformation.bUserConfirmedCardUnavailable = false;
        s_xBatchInformation.bUserConfirmedCardUnformatted = false;
    }
    s_pSaveDataPointer = pData;
    s_iDataSize = iDataSize;
    strcpy(s_szFileName, szSaveName);
    IssueCommand(CMD_START_SAVE);
}

/******************************** G E T   S A V E   L I S T ********************************/

void Asura_PS2_MemoryCard_Shell::GetSaveNameList(char* szSaveNameList)
{
    szSaveNameList[0] = 0;
#ifndef _DEBUG
    szSaveNameList[1] = '\0';
#else
    strcpy(&(szSaveNameList[1]), "No saves found\0");
#endif
    s_szSaveListOutput = szSaveNameList;
    IssueCommand(CMD_START_GET_SAVELIST);
}

/******************************** A U T O L O A D   M A I N ********************************/

void Asura_PS2_MemoryCard_Shell::AutoLoadMain(void* pData, const int iDataSize)
{
    Asura_Assert(pData, "No load data");
    Asura_Assert(iDataSize > 0, "Zero data size");

    s_pLoadDataPointer = pData;
    s_iDataSize = iDataSize;
    if(!s_xBatchInformation.bUserConfirmedCardUnavailable)
    {
        IssueCommand(CMD_START_AUTOLOAD_MAIN);
    }
}

/******************************** L O A D   M A I N ********************************/
void Asura_PS2_MemoryCard_Shell::LoadMain(void* pData, const int iDataSize)
{
    Asura_Assert(pData, "No load data");
    Asura_Assert(iDataSize > 0, "Zero data size");

    // Explicitly requested load so this doesn't apply
    s_xBatchInformation.bUserConfirmedCardUnavailable = false;
    s_pLoadDataPointer = pData;
    s_iDataSize = iDataSize;
    IssueCommand(CMD_START_LOAD_MAIN);
}

/******************************** L O A D ********************************/

void Asura_PS2_MemoryCard_Shell::Load(const char* szLoadName, void* pData, const int iDataSize)
{
    Asura_Assert(szLoadName, "No load name");
    Asura_Assert(pData, "No load data");
    Asura_Assert(iDataSize > 0, "Zero data size");

    // Explicitly requested load so this doesn't apply
    s_xBatchInformation.bUserConfirmedCardUnavailable = false;
    s_pLoadDataPointer = pData;
    s_iDataSize = iDataSize;
    strcpy(s_szFileName, szLoadName);
    IssueCommand(CMD_START_LOAD);
}

/******************************** D E L E T E ********************************/

void Asura_PS2_MemoryCard_Shell::Delete(const char* szDelName)
{
    Asura_Assert(szDelName, "No delete name");

    strcpy(s_szFileName, szDelName);
    IssueCommand(CMD_START_DELETE);
}

void Asura_PS2_MemoryCard_Shell::DeleteSaveDirectory(void)
{
    IssueCommand(CMD_START_DELETE_SAVE_DIRECTORY);
}

/******************************** I S S U E   C O M M A N D ********************************/

void Asura_PS2_MemoryCard_Shell::SetMessageToCheckingForCard(bool bCheckIfInBatch)
{
    // (NRM - 19.05.2003) check for batching messages
    if(!bCheckIfInBatch)
    {
        s_eCurrentMessage = MSG_CHECKING_CARD;
        s_eCurrentGetInputMessage = MSG_GI_NONE;
    }
    else if(!s_xBatchInformation.bInBatchOperation || 
             s_xBatchInformation.bFirstOperationInBatch)
    {
        s_eCurrentMessage = MSG_CHECKING_CARD;
        s_eCurrentGetInputMessage = MSG_GI_NONE;
    }
}


void Asura_PS2_MemoryCard_Shell::IssueCommand(const Command eCommand)
{
    s_eEntryCommand = eCommand;
    // Decide on entry point for command
    switch(eCommand)
    {
        case CMD_START_SAVE_MAIN:
        case CMD_START_AUTOSAVE_MAIN:
        case CMD_START_SAVE:
        {
            // One cancel should cancel all saves
            if(s_xBatchInformation.bInBatchOperation && 
                (s_xBatchInformation.bSaveWasCancelledInBatch ||
                  s_xBatchInformation.bUserConfirmedNoRoom ||
                    s_xBatchInformation.bUserConfirmedCardUnavailable))
            {
                return;
            }
        }
        // Fall thru is intentional
        case CMD_CHECK_ON_BOOT:
        case CMD_START_LOAD:
        case CMD_START_AUTOLOAD_MAIN:
        case CMD_START_LOAD_MAIN:
        case CMD_START_GET_SAVELIST:
        case CMD_START_DELETE:
        case CMD_START_DELETE_SAVE_DIRECTORY:
        {
			SetPhase(PHASE_CHECK_FOR_CARD);
            s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
            SetMessageToCheckingForCard();
			SetState(STATE_RUNNING);
        }
        break;
           

        default:
        {
            Asura_Assert(0, "Invalid or unknown command");
        }
    }
}

/**************************** C R E A T E   D E F A U L T   I C O N ****************************/

void Asura_PS2_MemoryCard_Shell::CreateDefaultIcon(const char* szIconName)
{
    // (NRM - 19.03.2003) quick way to create an icon
	const int aiBgcolor[4][4] = 
	{	
		{ 0x80,    0,    0, 0 },
		{    0, 0x80,    0, 0 },
		{    0,    0, 0x80, 0 },
		{ 0x80, 0x80, 0x80, 0 },
	};
	const Asura_Vector_3 axLightdir[3]= 
	{
		Asura_Vector_3(0.5f, 0.5f, 0.5f),
		Asura_Vector_3(0.0f,-0.4f,-0.1f),
		Asura_Vector_3(-0.5f,-0.5f, 0.5f),
	};
	static Asura_Vector_3 axLightcol[3]= 
	{
		Asura_Vector_3(0.25f, 0.25f, 0.25f),
		Asura_Vector_3(0.25f, 0.25f, 0.25f),
		Asura_Vector_3(0.25f, 0.25f, 0.25f),
	};
	static float aiAmbient[4] = { 0.50, 0.50, 0.50, 0.00 };			
	// Background colour
	for(int i=0; i<4; ++i)
	{
		Asura_PS2_MemoryCard_System::s_xIcon.SetBackgroundColour(
			static_cast<Asura_PS2_MemoryCard_Icon::Asura_PS2_MemoryCard_Icon_BackgroundPosition>(i),
			aiBgcolor[i][0],
			aiBgcolor[i][1],
			aiBgcolor[i][2]);
	}
	
	// Light position
	for(int i=0; i<3; ++i)
	{
		Asura_PS2_MemoryCard_System::s_xIcon.SetLightSource(i, 
															axLightdir[i], 
															axLightcol[i]);
	}
	Asura_PS2_MemoryCard_System::s_xIcon.SetAmbient(aiAmbient[0], aiAmbient[1], aiAmbient[2]);
	Asura_PS2_MemoryCard_System::s_xIcon.SetBackgroundTransparency(0x00);

    // Get the size of the icon
    Asura_Handle iFileID = Asura_FileAccess::Open(szIconName);
    if(iFileID == ASURA_INVALID_HANDLE)
    {
        Asura_Assert(0, "Cannot open memory card icon file - memory card save will be non-TRC compliant\n");
    }
    else
    {
        u_int uFileSize = Asura_PS2_Multistream_FastLoad::GetFileSize(iFileID);
        Asura_FileAccess::Close(iFileID);
        Asura_PS2_MemoryCard_System::s_xIcon.SetFileName(ASURA_PS2_MEMORYCARD_ICON_FILE,
                                                         szIconName,
                                                         uFileSize);
        // Adjustment of file size according to tech note
        s_iIconFileSize = uFileSize;
    }
}

/******************************** N O W   F I N I S H E D ********************************/

void Asura_PS2_MemoryCard_Shell::NowFinished(void)
{
    if(s_bFinishedSuccessfully || s_eEntryCommand == CMD_CHECK_ON_BOOT)
    {
        if(s_eEntryCommand == CMD_CHECK_ON_BOOT)
        {
            if(!s_bFinishedSuccessfully)
            {
                s_xBatchInformation.bUserConfirmedCardUnavailable = true;
            }
        }
		SetPhase(PHASE_INVALID);
        s_eEntryCommand   = CMD_INVALID;
        s_eCurrentCommandStatus = CMD_STATUS_INVALID;
        s_eCurrentGetInputMessage = MSG_GI_NONE;
		SetState(STATE_FINISHED);

        // (NRM - 19.05.2003) don't reset the message if in a batch
        if(!s_xBatchInformation.bInBatchOperation || 
            s_xBatchInformation.bLastOperationInBatch)
        {
            s_eCurrentMessage = MSG_NONE;
            s_xBatchInformation.bSetAsOverwrite = false;
        }   
        if(s_xBatchInformation.bUserConfirmedCardUnformatted)
        {
            s_bFinishedSuccessfully = false;
        }
    }
    else
    {
        // Sort the message out
        switch(s_eEntryCommand)
        {
            case CMD_START_SAVE:
            case CMD_START_AUTOSAVE_MAIN:
            case CMD_START_SAVE_MAIN:
				// SNH: added message for cancelled format
				if(s_xBatchInformation.bUserConfirmedCardUnformatted)
				{
					s_eCurrentMessage = MSG_ERR_RETRY_FORMAT;
				}
				else
				{
					s_eCurrentMessage = s_xBatchInformation.bSetAsOverwrite ? MSG_ERR_RETRY_SAVE
                                                                        : MSG_ERR_RETRY_OVERWRITE;
				}
                s_eCurrentGetInputMessage = MSG_GI_RETRY;
            break;

            case CMD_START_GET_SAVELIST:
            case CMD_START_AUTOLOAD_MAIN:
            case CMD_START_LOAD:
            case CMD_START_LOAD_MAIN:
                s_eCurrentMessage = MSG_ERR_RETRY_LOAD;
                s_eCurrentGetInputMessage = MSG_GI_RETRY;
            break;
            case CMD_START_DELETE:
            case CMD_START_DELETE_SAVE_DIRECTORY:
                s_eCurrentMessage = MSG_ERR_RETRY_DELETE;
                s_eCurrentGetInputMessage = MSG_GI_RETRY;
            break;
            default:
                Asura_Assert(0, "Unexpected entry command");
        }
        UserInput eUI = GetUserInput();
        // (NRM - 25.03.2003) logic for this is reversed - may change depending on subsequent TRC submissions
        // look at it as "retry saving"
        if(eUI == UI_YES)
        {
			SetPhase(PHASE_CHECK_FOR_CARD);
            s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
            // (NRM - 19.05.2003) don't call SetMessageToCheckingForCard() as we always want to display this
            // after user input
            s_eCurrentMessage = MSG_CHECKING_CARD;
            s_eCurrentGetInputMessage = MSG_GI_NONE;
            switch(s_eEntryCommand)
            {
                case CMD_START_SAVE:
                case CMD_START_AUTOSAVE_MAIN:
                case CMD_START_SAVE_MAIN:
                    s_xBatchInformation.bSaveWasCancelledInBatch = false;
                break;
				
				// SNH: reset this?
				case CMD_START_LOAD_MAIN:
					s_xBatchInformation.bUserConfirmedCardUnavailable = false;
					break;
					
                default:{}
            }
            s_xBatchInformation.bUserConfirmedCardUnformatted = false;
        }
        else if(eUI == UI_NO)
        {
            // Check if autosave should be disabled
            s_bDisableAutosave = true;

			SetPhase(PHASE_INVALID);
            s_eEntryCommand   = CMD_INVALID;
            s_eCurrentCommandStatus = CMD_STATUS_INVALID;
            s_eCurrentMessage = MSG_NONE;
            s_eCurrentGetInputMessage = MSG_GI_NONE;
			SetState(STATE_FINISHED);
            s_xBatchInformation.bSetAsOverwrite = false;
        }
    }
}

/******************************** G E T   U S E R   I N P U T ********************************/

Asura_PS2_MemoryCard_Shell::UserInput Asura_PS2_MemoryCard_Shell::GetUserInput(void)
{
    if(Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_PAD1_CROSS))
    {
        if(s_eCurrentGetInputMessage != MSG_GI_FORMAT && s_eCurrentGetInputMessage != MSG_GI_DELETE)
        {
            s_eCurrentGetInputMessage = MSG_GI_NONE;
        }
        return UI_ACCEPT;
    }
    else if(Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_PAD1_TRIANGLE))
    {
        s_eCurrentGetInputMessage = MSG_GI_NONE;
        return UI_CANCEL;
    }
    else if(Asura_UserInput::DebouncedKeyPressed(ASURA_PS2_KEYID_PAD1_CIRCLE))
    {
        if(s_eCurrentGetInputMessage == MSG_GI_FORMAT)
        {
            s_eCurrentGetInputMessage = MSG_GI_NONE;
        }
        return UI_FORMAT;
    }
    return UI_DO_NOTHING;
}

/************************ P R O C E S S   C H E C K   F O R   C A R D ************************/

void Asura_PS2_MemoryCard_Shell::ProcessCheckForCard(void)
{
    // Check if command has been issued yet
    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
        {
            s_bSaveStarted = false;
            // Check for sync before issuing command
            if(Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                Asura_PS2_MemoryCard_System::Rescan(s_iCurrentPort, s_iCurrentSlot);
                s_eCurrentCommandStatus = CMD_STATUS_WAIT;
				SetState(STATE_RUNNING);
                SetMessageToCheckingForCard();
            }
        }
        break;

        case CMD_STATUS_WAIT:
        {
            // Process previous frames error
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }

            if(s_eCurrentState == STATE_ERROR)
            {
                switch(s_eEntryCommand)
                {
                    case CMD_CHECK_ON_BOOT:
                    {
                        s_eCurrentMessage = MSG_ERR_NO_CARD_ON_BOOT;
                    }
                    break;
                    case CMD_START_SAVE:
                    case CMD_START_AUTOSAVE_MAIN:
                    case CMD_START_SAVE_MAIN:
                    {
                        s_eCurrentMessage = MSG_ERR_NO_CARD_SAVE;
                    }
                    break;
                    default:
                    {
                        s_eCurrentMessage = MSG_ERR_NO_CARD_LOAD;
                    }
                }
                s_eCurrentGetInputMessage = MSG_GI_RETRY;
                // Check for retry on user input
                UserInput eUI = GetUserInput();
                if(eUI == UI_RETRY)
                {
                    PlayRetrySound();
                    s_eCurrentCommandStatus = CMD_STATUS_ISSUE; // Reissue the command
                    s_xBatchInformation.bUserConfirmedCardUnavailable = false;
                }
                else if(eUI == UI_CANCEL)
                {   
                    PlayCancelSound();
                    s_bFinishedSuccessfully = false;
					SetPhase(PHASE_FINISHED);
                    s_xBatchInformation.bUserConfirmedCardUnavailable = true;
                }
            }
            else
            {
                // Check that the card was found
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE; // Set up next command
                int iError = Asura_PS2_MemoryCard_ErrorSystem::GetError();
                if(iError == ASURA_PS2_MEMORY_CARD_SUCCEED ||
                   iError == ASURA_PS2_MEMORY_CARD_ERROR_CARD_CHANGED)
                {
					// Set next phase                   
					SetPhase(PHASE_CHECK_FORMATTED);
                    // Delay any message change
                    s_fWaitTimer = fWAIT_TIMER;
                }
                else
                {
                    //s_eCurrentState = STATE_ERROR;
					SetState(STATE_ERROR);
                    switch(s_eEntryCommand)
                    {
                        case CMD_START_AUTOSAVE_MAIN:
                        {
                            // Exit if autosave disabled
                            if(s_bDisableAutosave)
                            {
                                s_bFinishedSuccessfully = true;
								SetPhase(PHASE_FINISHED);
                            }
                            else
                            {
                                s_bFinishedSuccessfully = false;
                                if(s_bSaveStarted)
                                {
									SetPhase(PHASE_SAVE_FAILED);
                                }
                                else
                                {
									SetPhase(PHASE_PROMPT_NO_CARD);
                                }
                            }
                        }
                        break;
                        case CMD_START_SAVE:
                        case CMD_START_SAVE_MAIN:
                        {
                            if(s_bSaveStarted)
                            {
								SetPhase(PHASE_SAVE_FAILED);
                            }
                            else
                            {
								SetPhase(PHASE_PROMPT_NO_CARD);
                            }
                        }
                        break;

                        case CMD_START_AUTOLOAD_MAIN:
                        case CMD_START_LOAD_MAIN:
                        case CMD_START_LOAD:
                        {
                            if(s_xBatchInformation.bUserConfirmedCardUnavailable ||
                                 (s_eEntryCommand == CMD_START_AUTOLOAD_MAIN && s_bDisableAutosave))
                            {
                                s_bFinishedSuccessfully = true;
								SetPhase(PHASE_FINISHED);
                            }
                            else
                            {
                                s_bFinishedSuccessfully = false;
								SetPhase(PHASE_PROMPT_NO_CARD);
                            }
                        }
                        break;
                        case CMD_CHECK_ON_BOOT:
                        {
                            if(iError == ASURA_PS2_MEMORY_CARD_ERROR_NO_CARD)
                            {
                                s_bFinishedSuccessfully = false;                        
								SetPhase(PHASE_PROMPT_NO_CARD);
                            }
                            else
                            {
                                s_bFinishedSuccessfully = true;                           
								SetPhase(PHASE_FINISHED);
                            }
                        }
                        break;
                        default:
                        {
                            s_bFinishedSuccessfully = false;
							SetPhase(PHASE_PROMPT_NO_CARD);
                        }
                    }
                }
            }
        }
        break;

        default:
            Asura_Assert(0, "Invalid command status");
    }
}

/************************ P R O C E S S   C H E C K   F O R M A T T E D ************************/

void Asura_PS2_MemoryCard_Shell::ProcessCheckFormatted(void)
{
    // No need to issue a command as the formatted status already got by CheckForCard
    bool bFormat = Asura_PS2_MemoryCard_System::Formatted(s_iCurrentPort, s_iCurrentSlot);
    if(!bFormat)
    {
        if(s_xBatchInformation.bUserConfirmedCardUnformatted)
        {
            s_bFinishedSuccessfully = true;
			SetPhase(PHASE_FINISHED);
            return;
        }

        switch(s_eEntryCommand)
        {
            case CMD_CHECK_ON_BOOT:
                s_bFinishedSuccessfully = true;
				SetPhase(PHASE_FINISHED);
                // (NRM - 20.03.2003) TRC stipulates to treat an unformatted card as an empty card
            break;
            case CMD_START_SAVE:
            case CMD_START_AUTOSAVE_MAIN:
            case CMD_START_SAVE_MAIN:
            {
                s_eCurrentMessage = MSG_ERR_NO_FORMAT;
                s_eCurrentGetInputMessage = MSG_GI_FORMAT;
				SetState(STATE_ERROR);

                UserInput eUI = GetUserInput();
                if(eUI == UI_FORMAT)
                {
                    PlayRetrySound();
					SetPhase(PHASE_FORMATTING);
                    s_eCurrentCommandStatus = CMD_STATUS_ISSUE; // Reissue the command
                    s_xBatchInformation.bUserConfirmedCardUnformatted = false;
                }
                else if(eUI == UI_CANCEL)
                {
                    PlayCancelSound();
                    s_xBatchInformation.bUserConfirmedCardUnformatted = true;
                    s_bFinishedSuccessfully = false;
					SetPhase(PHASE_FINISHED);
                }
            }
            break;
            case CMD_START_GET_SAVELIST:
            {
                // No save games if not formatted
                s_szSaveListOutput[0] = 0;
                s_szSaveListOutput[1] = '\0';
                s_bFinishedSuccessfully = true;
				SetPhase(PHASE_FINISHED);
            }
            break;
            case CMD_START_LOAD:
            case CMD_START_LOAD_MAIN:
            case CMD_START_DELETE:
            case CMD_START_DELETE_SAVE_DIRECTORY:
            {
                s_eCurrentMessage = MSG_ERR_NO_SAVES;
				SetState(STATE_ERROR);
                s_eCurrentGetInputMessage = MSG_GI_RETRY;

                UserInput eUI = GetUserInput();
                if(eUI == UI_RETRY)
                {
                    PlayRetrySound();
					SetPhase(PHASE_CHECK_FOR_CARD);
                    s_eCurrentCommandStatus = CMD_STATUS_ISSUE; // Reissue the command
                }
                else if(eUI == UI_CANCEL)
                {
                    PlayCancelSound();
                    s_bFinishedSuccessfully = false;
					SetPhase(PHASE_FINISHED);
                }
            }
            break;
            case CMD_START_AUTOLOAD_MAIN:
            {
                s_bFinishedSuccessfully = true;
				SetPhase(PHASE_FINISHED);
            }
            break;
            default:
                Asura_Assert(0,"Invalid entry command");
        }
    }
    else
    {
        s_eCurrentCommandStatus = CMD_STATUS_ISSUE; // Set up next command
        switch(s_eEntryCommand)
        {
            case CMD_START_GET_SAVELIST:
            {
                s_szSaveListOutput[0] = 0;
                s_szSaveListOutput[1] = '\0';
				SetPhase(PHASE_GET_SAVELIST);
            }
            break;
            case CMD_START_LOAD:
            case CMD_START_LOAD_MAIN:
            case CMD_START_AUTOLOAD_MAIN:
            {
				SetPhase(PHASE_LOADING);
            }
            break;
            case CMD_START_DELETE:
            case CMD_START_DELETE_SAVE_DIRECTORY:
            {
                if(s_xBatchInformation.bSilentlyDelete)
                {
                    //s_eCurrentPhase = s_eEntryCommand == CMD_START_DELETE ? PHASE_DELETING
                    //                                                      : PHASE_DELETING_SAVE_DIRECTORY;
					SetPhase((s_eEntryCommand == CMD_START_DELETE) ? PHASE_DELETING : PHASE_DELETING_SAVE_DIRECTORY);
                    s_eCurrentCommandStatus = CMD_STATUS_ISSUE; // Reissue the command
                }
                else
                {
					SetPhase(PHASE_PROMPT_DELETE_CONFIRM);
                }
            }
            break;
            default:
            {
                s_szSaveListOutput[0] = 0;
                s_szSaveListOutput[1] = '\0';
                // Set next phase                   
				SetPhase(PHASE_CHECK_FOR_ROOM);
            }
        }
    }
}

/************************ P R O C E S S   C H E C K   F O R   R O O M  *************************/

void Asura_PS2_MemoryCard_Shell::ProcessCheckForRoom(void)
{
    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
        {
            // Need to get the directory list here so we know the size of any (possibly corrupt) project save
			SetState(STATE_RUNNING);
            s_eCurrentCommandStatus = CMD_STATUS_CARD_CHECK;
            SetMessageToCheckingForCard();
            s_fWaitTimer = fWAIT_TIMER;
            Asura_PS2_MemoryCard_System::SetSaveName(s_szProductName);
	        Asura_PS2_MemoryCard_System::ChangeToProductDir(s_iCurrentPort, s_iCurrentSlot);
        }
        break;

        case CMD_STATUS_CARD_CHECK:
        {
            if(Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                s_szDirList[0] = 0;
                s_szDirList[1] = '\0';
                if(Asura_PS2_MemoryCard_ErrorSystem::GetError() != ASURA_PS2_MEMORY_CARD_ERROR_DIRECTORY_NOT_FOUND)
                {
	                Asura_PS2_MemoryCard_System::GetDirList(s_iCurrentPort, s_iCurrentSlot, s_szDirList);
                }
                s_eCurrentCommandStatus = CMD_STATUS_WAIT;
            }
        }
        break;

        case CMD_STATUS_WAIT:
        {
            // Check for error from previous frame
            if(s_eCurrentState == STATE_ERROR)
            {
                s_eCurrentMessage   = MSG_ERR_NO_ROOM;
                s_eCurrentGetInputMessage = MSG_GI_RETRY;
                // Check for retry on user input
                UserInput eUI = GetUserInput();
                if(eUI == UI_RETRY)
                {
                    PlayRetrySound();
					SetPhase(PHASE_CHECK_FOR_CARD);
                    s_eCurrentCommandStatus = CMD_STATUS_ISSUE; // Reissue the command
                    SetMessageToCheckingForCard(false);
                }
                else if(eUI == UI_CANCEL)
                {
                    PlayCancelSound();                  
                    s_bFinishedSuccessfully = false;
					SetPhase(PHASE_FINISHED);
                    s_xBatchInformation.bUserConfirmedNoRoom = true;
                    return;
                }
                break;
            }

            // Check for sync before doing any error checking
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                break;
            }
            // Calculate size of any (possibly corrupt) project files
	        int iSizeOffset = 0;
            if(strlen(s_szDirList) > 0)
            {
                iSizeOffset = Asura_PS2_MemoryCard_System::GetProjectFileSize();// convert to KB
            }
	        int iFreeSpace  = Asura_PS2_MemoryCard_System::GetFreeSpace(s_iCurrentPort, s_iCurrentSlot);
            // Check against predicted size (by user) - sizes from memory card are in KB
            // (NRM - 02.07.2003) adjustment of icon cluster size due to tech note
            // (NRM - 02.07.2003) s_iCheckDataSize is correct through function SetDataSize
            int iIconClusterSize = Asura_Maths::FloatToInt_Chop((s_iIconFileSize + 1023.f) / 1024.f) * 1024;

            bool bFreeSpace;
            
            if(s_iOverrideDataSize != -1 && // (NRM - 18.07.2003) value if not -1 will have been set
                iSizeOffset == 0)           // (NRM - 21.07.2003) we have enough free space if one save made
            {
                bFreeSpace = (iFreeSpace + iSizeOffset >= s_iOverrideDataSize);
            }
            else
            {
                // (NRM - 05.08.2003) > to make sure that we know if we are going to occupy another cluster
                bFreeSpace = (iFreeSpace + iSizeOffset >= iIconClusterSize + s_iCheckDataSize);
            }

            if(s_eEntryCommand == CMD_CHECK_ON_BOOT)
            {
                bFreeSpace |= (iSizeOffset != 0);
            }

	        if(!bFreeSpace)
	        {
				SetState(STATE_ERROR);
	        }
	        else
	        {
                // go to next phase - depends on entry command
                switch(s_eEntryCommand)
                {
                    case CMD_CHECK_ON_BOOT:
                    {
                        s_bFinishedSuccessfully = true;
						SetPhase(PHASE_FINISHED);
                    }
                    break;
                    case CMD_START_SAVE:
                    case CMD_START_AUTOSAVE_MAIN:
                    case CMD_START_SAVE_MAIN:
                    {
                        s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
						SetPhase(PHASE_CHECK_FOR_SAVE);
                        SetMessageToCheckingForCard();
                    }
                    break;
                    default:
                        Asura_Assert(0, "Unexpected entry command");
                }
	        }
        }
        break;

        default:
            Asura_Assert(0, "Invalid command status");
    }
}

/************************ P R O C E S S   F O R M A T T I N G *************************/

void Asura_PS2_MemoryCard_Shell::ProcessFormatting(void)
{
    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
        {
            // First check that the card is still unformatted so as to not format a card that has been switched in
			SetState(STATE_RUNNING);
    		Asura_PS2_MemoryCard_System::Rescan(s_iCurrentPort, s_iCurrentSlot);
            s_fWaitTimer = fWAIT_TIMER;
            s_eCurrentCommandStatus = CMD_STATUS_CARD_CHECK;
            s_eCurrentMessage = MSG_CHECKING_CARD;
            s_eCurrentGetInputMessage = MSG_GI_NONE;            
        }
        break;

        case CMD_STATUS_CARD_CHECK:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
    		if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_ERROR_NO_CARD)
	    	{
				SetPhase(PHASE_PROMPT_NO_CARD);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
		    }
		    else if(Asura_PS2_MemoryCard_System::Formatted(s_iCurrentPort, s_iCurrentSlot))
		    {
                s_bCardChanged = true;
				SetPhase(PHASE_CHECK_FOR_ROOM);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
		    }
		    else
    		{
    			Asura_PS2_MemoryCard_System::Format(s_iCurrentPort, s_iCurrentSlot);
                s_fProgressOfCurrentOperation = 0.0f;
                s_eCurrentMessage = MSG_FORMATTING;
                s_eCurrentCommandStatus = CMD_STATUS_WAIT;
            }
        }
        break;

        case CMD_STATUS_WAIT:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                // update progress time
                s_fProgressOfCurrentOperation += (1.0f / fFORMAT_COMPLETION_TIME) * Asura_Timers::GetFrameTime();
                return;
            }
            // check for error and goto next stage
            s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
            if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_SUCCEED)
            {	
                s_bCreateIconAndDir = true;
				SetPhase(PHASE_SAVING);
                s_eCurrentMessage   = MSG_FORMAT_SUCCESSFUL;
                s_fWaitTimer        = fWAIT_TIMER;
            }
            else
            {
				SetState(STATE_ERROR);
				SetPhase(PHASE_FORMAT_FAILED);
            }
        }
        break;

        default:
            Asura_Assert(0, "Invalid command status");
    }    
}

/****************************** W A I T   F O R   I N P U T ******************************/

void Asura_PS2_MemoryCard_Shell::WaitForInput(const DisplayMessage eMessage, const Phase eNextPhase)
{
    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
        {
			SetState(STATE_ERROR);
            s_eCurrentMessage = eMessage;
            // Check for retry on user input
            UserInput eUI = GetUserInput();
            if(eUI == UI_RETRY)
            {
                PlayRetrySound();
                s_eCurrentCommandStatus = CMD_STATUS_WAIT;
                switch(s_eCurrentMessage)
                {
					case MSG_ERR_CARD_CHANGED:
						s_bCardChanged = false;		// deliberate fallthrough...
                    case MSG_ERR_SAVE_EXISTS:
                    case MSG_ERR_CONFIRM_OVERWRITE:
                        s_xBatchInformation.bUserCancelledOverwrite = false;
                        s_xBatchInformation.bSetAsOverwrite = true;
                        s_xBatchInformation.bAutomaticallyOverwrite = true;
                    break;
                    default:
                    {
                    } 
                }

                // (NRM - 19.05.2003) Don't set message using SetMessageToCheckingForCard()
                s_eCurrentMessage = MSG_CHECKING_CARD;
                s_eCurrentGetInputMessage = MSG_GI_NONE;
                s_fWaitTimer = fWAIT_TIMER;
                Asura_PS2_MemoryCard_System::Rescan(s_iCurrentPort, s_iCurrentSlot);
            }
            else if(eUI == UI_CANCEL)
            {
                PlayCancelSound();
                s_bFinishedSuccessfully = false;
				SetPhase(PHASE_FINISHED);
                s_eCurrentGetInputMessage = MSG_GI_NONE;
                if(s_xBatchInformation.bInBatchOperation)
                {
                    s_xBatchInformation.bSaveWasCancelledInBatch = true;
                }
                switch(s_eCurrentMessage)
                {
                    case MSG_ERR_NO_CARD_LOAD:
                    case MSG_ERR_NO_CARD_SAVE:
                    case MSG_ERR_NO_CARD_ON_BOOT:
                        s_xBatchInformation.bUserConfirmedCardUnavailable = true;
                    break;
                    case MSG_ERR_SAVE_EXISTS:
                        s_xBatchInformation.bUserCancelledOverwrite = true;
                    break;
                    case MSG_ERR_DELETE_FAILED:
                        s_xBatchInformation.bInBatchOperation      = false;
                        s_xBatchInformation.bFirstOperationInBatch = false;                       
                        s_xBatchInformation.bLastOperationInBatch  = false;
                    break;
                    case MSG_ERR_CARD_CHANGED:
                        s_bCardChanged = false;
                    break;
                    default:
                    {
                    }
                }
            }
        }
        break;
        case CMD_STATUS_WAIT:
        {
            if(Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
				SetState(STATE_RUNNING);
				SetPhase(eNextPhase);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
                s_eCurrentGetInputMessage = MSG_GI_NONE;
            }
        }
        break;
        default:
            Asura_Assert(0, "Invalid command status");
    }
}

/************************ P R O C E S S   C A R D   C H A N G E D ************************/

void Asura_PS2_MemoryCard_Shell::ProcessPromptCardChanged(void)
{
    s_eCurrentGetInputMessage = MSG_GI_YES_NO;
    WaitForInput(MSG_ERR_CARD_CHANGED, PHASE_CHECK_FOR_CARD);
}

/************************ P R O C E S S   C R E A T E   S A V E *************************/

void Asura_PS2_MemoryCard_Shell::ProcessPromptCreateSave(void)
{
    s_eCurrentGetInputMessage = MSG_GI_YES_NO;
    WaitForInput(MSG_ERR_NO_SAVE_CREATE, PHASE_SAVING);
}

/***************** P R O C E S S   O V E R W R I T E   S U B   S A V E ******************/

void Asura_PS2_MemoryCard_Shell::ProcessPromptOverwriteSubSave(void)
{
    s_eCurrentGetInputMessage = MSG_GI_YES_NO;
    WaitForInput(s_xBatchInformation.bSetAsOverwrite ? MSG_ERR_CONFIRM_OVERWRITE
                                                     : MSG_ERR_SAVE_EXISTS, 
                 PHASE_SAVING);
}

void Asura_PS2_MemoryCard_Shell::ProcessPromptNoCard(void)
{
    s_bFinishedSuccessfully = false;
    if(s_xBatchInformation.bUserConfirmedCardUnavailable)
    {
		SetPhase(PHASE_FINISHED);
        s_bFinishedSuccessfully = true;
    }
    else
    {
        s_eCurrentGetInputMessage = MSG_GI_RETRY;
        switch(s_eEntryCommand)
        {
            case CMD_CHECK_ON_BOOT:
            {
                s_eCurrentMessage = MSG_ERR_NO_CARD_ON_BOOT;
            }
            break;
            case CMD_START_SAVE:
            case CMD_START_AUTOSAVE_MAIN:
            case CMD_START_SAVE_MAIN:
            {
                s_eCurrentMessage = MSG_ERR_NO_CARD_SAVE;
            }
            break;
            default:
            {
                s_eCurrentMessage = MSG_ERR_NO_CARD_LOAD;
            }
        }
        WaitForInput(s_eCurrentMessage, PHASE_CHECK_FOR_CARD);
    }
}

/********************* P R O C E S S   C H E C K   F O R   S A V E *********************/

void Asura_PS2_MemoryCard_Shell::ProcessCheckForSave(void)
{
    // Try to change to project's save dir
    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
			SetState(STATE_RUNNING);
            Asura_PS2_MemoryCard_System::SetSaveName(s_szProductName);
        	Asura_PS2_MemoryCard_System::ChangeToProductDir(s_iCurrentPort, s_iCurrentSlot);
            s_eCurrentCommandStatus = CMD_STATUS_WAIT;
        break;

        case CMD_STATUS_WAIT:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }

            bool bSaveExists = (Asura_PS2_MemoryCard_ErrorSystem::GetError() != ASURA_PS2_MEMORY_CARD_ERROR_DIRECTORY_NOT_FOUND);

            // (NRM - 16.06.2003) check whether the card has been changed or not
            s_bCreateIconAndDir = !bSaveExists;
            s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
            if(s_bCardChanged)
            {
           //   if(bSaveExists)
                {
					SetPhase(PHASE_PROMPT_CARD_CHANGED);
                    return;
                }
           //     else
           //   {
           //      // s_bCardChanged = false;
           //   }
            }
			
			// SNH double-check save exists - find the primary file in this directory.
			char szPrimaryName[32];
            Asura_PS2_MemoryCard_System::GetPrimaryName(szPrimaryName);
            if(s_eEntryCommand == CMD_START_SAVE && bSaveExists && !FileExists(szPrimaryName))
			{
				bSaveExists = false;
				s_bCreateIconAndDir = true;
			}

            // (NRM - 16.06.2003) have to check whether the lower level of saves exist
            if(bSaveExists)
            {                    
                if(s_eEntryCommand == CMD_START_SAVE &&
                    !s_xBatchInformation.bAutomaticallyOverwrite ||
                     s_xBatchInformation.bUserCancelledOverwrite ||
                     s_xBatchInformation.bSetAsOverwrite)
                {
					SetPhase(PHASE_CHECK_FOR_SUB_SAVE);
                }
                else
                {
					SetPhase(PHASE_SAVING);
                }
            }
            else
            {
				SetPhase(PHASE_PROMPT_CREATE_SAVE);
            }
        }
        break;
        default:
            Asura_Assert(0, "Invalid command status");
    }
}

/****************** P R O C E S S   C H E C K   F O R   S U B S A V E ******************/

void Asura_PS2_MemoryCard_Shell::ProcessCheckForSubSave(void)
{
    // check whether or not the save is going to be overwritten for non-autosave saves
    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
        {
			SetState(STATE_RUNNING);
            Asura_PS2_MemoryCard_System::SetSaveName(s_szProductName);
        	Asura_PS2_MemoryCard_System::ChangeToProductDir(s_iCurrentPort, s_iCurrentSlot);
            s_szDirList[0] = '\0';
	        Asura_PS2_MemoryCard_System::GetDirList(s_iCurrentPort, s_iCurrentSlot, s_szDirList);
            s_eCurrentCommandStatus = CMD_STATUS_WAIT;
        }
        break;
        case CMD_STATUS_WAIT:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
            char* pcCurrentString = &(s_szDirList[1]);
            s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
            for(int i=0; i<static_cast<int>(s_szDirList[0]); ++i)
            {
                if(strcmp(pcCurrentString, s_szFileName) == 0)
                {
                    // file already exists so want to prompt user to overwrite it
                    //s_eCurrentPhase = (s_xBatchInformation.bAutomaticallyOverwrite && 
                    //                    !s_xBatchInformation.bUserCancelledOverwrite)
                    //                   ? PHASE_SAVING : PHASE_PROMPT_OVERWRITE_SUB_SAVE;
					SetPhase((s_xBatchInformation.bAutomaticallyOverwrite && 
                                        !s_xBatchInformation.bUserCancelledOverwrite)
										? PHASE_SAVING : PHASE_PROMPT_OVERWRITE_SUB_SAVE);

                    // (NRM - 19.05.2003) this means that the user will have to set automatically overwrite 
                    // every time a save is called - which is good.
                    s_xBatchInformation.bAutomaticallyOverwrite = false;
                    return;
                }
                pcCurrentString += (strlen(pcCurrentString) + 1);
            }
			SetPhase(PHASE_SAVING);
        }
        break;
        default:
            Asura_Assert(0, "Invalid command status");
    }
}

/************************ P R O C E S S   S A V I N G ************************/

void Asura_PS2_MemoryCard_Shell::ProcessSaving(void)
{
    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
        {
            s_bSaveStarted = true;
			SetState(STATE_RUNNING);
    		Asura_PS2_MemoryCard_System::Rescan(s_iCurrentPort, s_iCurrentSlot);
            s_fWaitTimer = fWAIT_TIMER;
            s_eCurrentCommandStatus = CMD_STATUS_CARD_CHECK;
            if(!s_xBatchInformation.bInBatchOperation ||
                s_xBatchInformation.bFirstOperationInBatch ||
                 s_bCardChanged)
            {
                s_eCurrentMessage = MSG_CHECKING_CARD;
            }
            s_eCurrentGetInputMessage = MSG_GI_NONE;            
        }
        break;

        case CMD_STATUS_CARD_CHECK:
        {
            if(s_bCardChanged)
            {
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
				SetPhase(PHASE_PROMPT_CARD_CHANGED);
                return;
            }
			SetState(STATE_RUNNING);
            // Issue AFTER sync
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
            // Check card is formatted
            if(!Asura_PS2_MemoryCard_System::Formatted(s_iCurrentPort, s_iCurrentSlot))
            {
				SetPhase(PHASE_CHECK_FORMATTED);
                return;
            }
            // Create save directory and icon if not already there
            Asura_PS2_MemoryCard_System::SetBrowserName(s_szProductName);
            if(s_bCreateIconAndDir)
            {
                Asura_PS2_MemoryCard_System::SetSaveName(s_szProductName);
            	Asura_PS2_MemoryCard_System::WriteIcon(s_iCurrentPort, s_iCurrentSlot);
            }
            // Determine whether to write the data as primary(autosave) or secondary(non-autosave)
            if(s_eEntryCommand == CMD_START_AUTOSAVE_MAIN ||
                s_eEntryCommand == CMD_START_SAVE_MAIN)
            {
                Asura_PS2_MemoryCard_System::SetSaveName(s_szProductName);
                Asura_PS2_MemoryCard_System::WritePrimary(s_iCurrentPort, s_iCurrentSlot, 
                                                          s_pSaveDataPointer, s_iDataSize, 
                                                          true);
            }
            else
            {
                Asura_PS2_MemoryCard_System::SetSaveName(s_szFileName);
                Asura_PS2_MemoryCard_System::WriteSecondary(s_iCurrentPort, s_iCurrentSlot, 
                                                          s_pSaveDataPointer,
                                                          s_szFileName,
                                                          s_iDataSize,
                                                          true);
            }
            // Calculate percentage change per second
            float fTotalSaveSize = static_cast<float>(s_iDataSize);
            if(s_bCreateIconAndDir)
            {
                fTotalSaveSize += static_cast<float>(s_iIconFileSize);      // in Bytes
            }
            if(s_xBatchInformation.bSetAsOverwrite)
            {
                s_eCurrentMessage = MSG_OVERWRITING;
            }
            else
            {
                s_eCurrentMessage = MSG_SAVING;
            }
            s_eCurrentCommandStatus = CMD_STATUS_WAIT;
            s_fProgressOfCurrentOperation = 0.0f;
        }
        break;
        case CMD_STATUS_WAIT:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                s_fProgressOfCurrentOperation += 0.02f;
                return;
            }
            if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_SUCCEED)
            {
                s_fWaitTimer      = fWAIT_TIMER;
                s_bFinishedSuccessfully = true;
				SetPhase(PHASE_FINISHED);
                if(!s_xBatchInformation.bInBatchOperation || 
                     s_xBatchInformation.bLastOperationInBatch)
                {
                    s_eCurrentMessage = s_xBatchInformation.bSetAsOverwrite ? MSG_OVERWRITE_SUCCESSFUL
                                                                            : MSG_SAVE_SUCCESSFUL;
                }
                s_bDisableAutosave = false;         // got a card so autosave
            }
            else
            {
				SetState(STATE_ERROR);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
				SetPhase(PHASE_SAVE_FAILED);
            }
            s_xBatchInformation.bFirstOperationInBatch = false;
            if(!s_xBatchInformation.bInBatchOperation || 
                 s_xBatchInformation.bLastOperationInBatch)
            {
                s_bSaveStarted = false;
            }
        }
        break;
        default:
            Asura_Assert(0, "Invalid command status");
    }
}

/************************ F I L E   E X I S T S ************************/

bool Asura_PS2_MemoryCard_Shell::FileExists(const char* szFileName)
{
    char* pcCurrentString = &(s_szDirList[1]);
    for(int i=0; i<s_szDirList[0]; ++i)
    {
        if(strcmp(pcCurrentString, szFileName) == 0)
        {
            return true;
        }
        pcCurrentString += strlen(pcCurrentString) + 1;
    }
    return false;
}

/************************ P R O C E S S   L O A D I N G ************************/

void Asura_PS2_MemoryCard_Shell::ProcessLoading(void)
{
    static u_char aucReadInChecksum[Asura_Checksum::CHECKSUM_LENGTH];

    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
        {
            s_bCardChanged = false;
			SetState(STATE_RUNNING);
            Asura_PS2_MemoryCard_System::SetSaveName(s_szProductName);
			Asura_PS2_MemoryCard_System::ChangeToProductDir(s_iCurrentPort, s_iCurrentSlot);
			s_eCurrentCommandStatus = CMD_STATUS_WAIT_ON_CHANGE_DIRECTORY;
        }
        break;

		case CMD_STATUS_WAIT_ON_CHANGE_DIRECTORY:
		{
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
			// check to see if the directory exists - if it doesn't then no save exists
			if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_ERROR_DIRECTORY_NOT_FOUND)
			{
				switch(s_eEntryCommand)
				{
					case CMD_START_LOAD:
						SetState(STATE_ERROR);
						s_bFinishedSuccessfully = false;
						SetPhase(PHASE_PROMPT_SAVE_DOES_NOT_EXIST);
						s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
					break;

					case CMD_START_LOAD_MAIN:
						SetState(STATE_ERROR);
						s_bFinishedSuccessfully = false;
						SetPhase(PHASE_PROMPT_SAVE_DOES_NOT_EXIST);
						s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
					break;

					case CMD_START_AUTOLOAD_MAIN:
						s_bFinishedSuccessfully = true;
						SetPhase(PHASE_FINISHED);
						s_eCurrentCommandStatus = CMD_STATUS_ISSUE;                                        
					break;

					default:
					{
					}
				}
			}
			else
			{
				s_szDirList[0] = '\0';
				Asura_PS2_MemoryCard_System::GetDirList(s_iCurrentPort, s_iCurrentSlot, s_szDirList, false);
				SetMessageToCheckingForCard();
				s_eCurrentCommandStatus = CMD_STATUS_CARD_CHECK;
			}
		}
		break;

        case CMD_STATUS_CARD_CHECK:
        {
            // Wait for get dir to complete to see if save exists

            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
            if(s_eEntryCommand == CMD_START_LOAD)
            {
                if(FileExists(s_szFileName))
                {
                    s_fWaitTimer = fWAIT_TIMER;
			        Asura_PS2_MemoryCard_System::Read(s_iCurrentPort, s_iCurrentSlot,
                                                      s_szFileName,
					    					          s_pLoadDataPointer, 
						    				          s_iDataSize,
                                                      aucReadInChecksum);
                    s_eCurrentMessage = MSG_LOADING;
                    s_eCurrentCommandStatus = CMD_STATUS_WAIT;
                }
                else
                {
					SetState(STATE_ERROR);
                    s_bFinishedSuccessfully = false;
					SetPhase(PHASE_PROMPT_SAVE_DOES_NOT_EXIST);
                    s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
                }
            }
            else
            {
                char szPrimaryName[32];
                Asura_PS2_MemoryCard_System::GetPrimaryName(szPrimaryName);
                if(FileExists(szPrimaryName))
                {
                    s_fWaitTimer = fWAIT_TIMER;
			        Asura_PS2_MemoryCard_System::ReadPrimary(s_iCurrentPort, s_iCurrentSlot,
					    						             s_pLoadDataPointer, 
						    					             s_iDataSize,
                                                             aucReadInChecksum);
                    s_eCurrentMessage = MSG_LOADING;
                    s_eCurrentCommandStatus = CMD_STATUS_WAIT;
                }
                else if(s_eEntryCommand == CMD_START_LOAD_MAIN)
                {
                    s_eCurrentMessage = MSG_ERR_NO_SAVES;
					SetState(STATE_ERROR);
                    s_eCurrentGetInputMessage = MSG_GI_RETRY;

                    UserInput eUI = GetUserInput();
                    if(eUI == UI_RETRY)
                    {
                        PlayRetrySound();
						SetPhase(PHASE_CHECK_FOR_CARD);
                        s_eCurrentCommandStatus = CMD_STATUS_ISSUE; // Reissue the command
                    }
                    else if(eUI == UI_CANCEL)
                    {
                        PlayCancelSound();
                        s_bFinishedSuccessfully = false;
						SetPhase(PHASE_FINISHED);
                    }
                }
                else
                {
					SetState(STATE_ERROR);
                    s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
					SetPhase(PHASE_LOAD_FAILED);
                    s_eCurrentMessage = MSG_ERR_SAVE_CORRUPT;
                }
            }            
        }
        break;
        case CMD_STATUS_WAIT:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                s_fProgressOfCurrentOperation += 0.02f;
                return;
            }
            if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_SUCCEED)
            {
                // (NRM - 06.08.2003) checksum check
                u_char aucCalculatedChecksum[16];
                Asura_Checksum::EncodeFromBuffer(reinterpret_cast<const u_char*>(s_pLoadDataPointer), s_iDataSize, aucCalculatedChecksum);
                bool bChecksumMatch = true;
                for(int i=0; i<Asura_Checksum::CHECKSUM_LENGTH; ++i)
                {
                    bChecksumMatch &= (aucReadInChecksum[i] == aucCalculatedChecksum[i]);
                }

                if(bChecksumMatch)
                {
                    s_fWaitTimer      = fWAIT_TIMER;
                    s_bFinishedSuccessfully = true;
					SetPhase(PHASE_FINISHED);
                    // (NRM - 19.05.2003) only display successfull message if not in a batch or the last op in the batch
                    if(!s_xBatchInformation.bInBatchOperation || 
                         s_xBatchInformation.bLastOperationInBatch)
                    {
                        s_eCurrentMessage = MSG_LOAD_SUCCESSFUL;
                    }
                }
                else
                {   
					SetState(STATE_ERROR);
                    s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
					SetPhase(PHASE_LOAD_FAILED);
                    s_eCurrentMessage = MSG_ERR_SAVE_CORRUPT;
                }
                s_bDisableAutosave = false;         // got a card so autosave
            }
            else
            {   
				SetState(STATE_ERROR);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
				SetPhase(PHASE_LOAD_FAILED);
            }
            s_xBatchInformation.bFirstOperationInBatch = false;
        }
        break;
        default:
            Asura_Assert(0, "Unexpected command status");
    }
}
/************************ P R O C E S S   G E T   S A V E   L I S T ************************/

void Asura_PS2_MemoryCard_Shell::ProcessGetSaveList(void)
{
    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
        {
			SetState(STATE_RUNNING);
            // Issue AFTER sync
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
            // Issue command
            Asura_PS2_MemoryCard_System::SetSaveName(s_szProductName);
        	Asura_PS2_MemoryCard_System::ChangeToProductDir(s_iCurrentPort, s_iCurrentSlot);
            SetMessageToCheckingForCard();
            s_eCurrentCommandStatus = CMD_STATUS_CARD_CHECK;            
        }
        break;
        case CMD_STATUS_CARD_CHECK:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
            s_fWaitTimer = fWAIT_TIMER;
            s_szDirList[0] = 0;
            s_szDirList[1] = '\0';
            if(Asura_PS2_MemoryCard_ErrorSystem::GetError() != ASURA_PS2_MEMORY_CARD_ERROR_DIRECTORY_NOT_FOUND)
            {
	            Asura_PS2_MemoryCard_System::GetDirList(s_iCurrentPort, s_iCurrentSlot, s_szDirList);
                s_eCurrentCommandStatus = CMD_STATUS_WAIT;
            }
            else
            {
                // no saves found but finished successfully
                s_bFinishedSuccessfully = true;
				SetPhase(PHASE_FINISHED);
            }
        }
        break;
        case CMD_STATUS_WAIT:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
            // Copy across the filenames that don't match : ".", "..", "icon.sys", 
            // "*.ico", or containing the product code
            // First byte is number of saves - minus the five above
            s_szSaveListOutput[0] = s_szDirList[0] - 5;
            char* pcCurrentString = &(s_szDirList[1]);
            char* pcOutputString   = &(s_szSaveListOutput[1]);
            for(int i=0; i<s_szDirList[0]; ++i)
            {
                // (NRM - 26.03.2003) Gadzooks - surely there must be a better way?
                if(strcmp(pcCurrentString, ".")         != 0 &&
                    strcmp(pcCurrentString, "..")       != 0 &&
                    strcmp(pcCurrentString, "icon.sys") != 0 &&
                    strstr(pcCurrentString, ".ico")     == NULL &&
                    strstr(pcCurrentString, s_szProductCode) == NULL)
                {
                    strcpy(pcOutputString, pcCurrentString);
                    pcOutputString += strlen(pcCurrentString) + 1;
                }
                pcCurrentString += strlen(pcCurrentString) + 1;
            }
            s_bFinishedSuccessfully = true;
			SetPhase(PHASE_FINISHED);
            s_xBatchInformation.bFirstOperationInBatch = false;
        }
        break;
        default:
            Asura_Assert(0, "Invalid command status");
    }
}

/************************ P R O C E S S   F O R M A T   F A I L E D ************************/

void Asura_PS2_MemoryCard_Shell::ProcessFormatFailed(void)
{
    s_eCurrentGetInputMessage = MSG_GI_RETRY;
    WaitForInput(MSG_ERR_FORMAT_FAILED, PHASE_CHECK_FOR_CARD);
}

/************************** P R O C E S S   S A V E   F A I L E D **************************/

void Asura_PS2_MemoryCard_Shell::ProcessSaveFailed(void)
{
    s_eCurrentGetInputMessage = MSG_GI_RETRY;
    WaitForInput(s_xBatchInformation.bSetAsOverwrite ? MSG_ERR_OVERWRITE_FAILED
                                                     : MSG_ERR_SAVE_FAILED, 
                 PHASE_CHECK_FOR_CARD);
}

/************************** P R O C E S S   L O A D   F A I L E D **************************/


void Asura_PS2_MemoryCard_Shell::ProcessLoadFailed(void)
{
    s_eCurrentGetInputMessage = MSG_GI_RETRY;
    WaitForInput(s_eCurrentMessage == MSG_ERR_SAVE_CORRUPT ? MSG_ERR_SAVE_CORRUPT 
                                                           : MSG_ERR_LOAD_FAILED, PHASE_CHECK_FOR_CARD);
}

/********** P R O C E S S   P R O M P T   S A V E   D O E S   N O T   E X I S T ************/

void Asura_PS2_MemoryCard_Shell::ProcessPromptSaveDoesNotExist(void)
{
    s_eCurrentGetInputMessage = MSG_GI_RETRY;
    WaitForInput(MSG_ERR_FILE_DOES_NOT_EXIST, PHASE_CHECK_FOR_CARD);
}

/********** P R O C E S S   D E L E T I N G **********/

void Asura_PS2_MemoryCard_Shell::ProcessDeleting(void)
{
    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
        {
            // First check that the card is still present so as to not delete from a card that has been switched in
			SetState(STATE_RUNNING);
    		Asura_PS2_MemoryCard_System::Rescan(s_iCurrentPort, s_iCurrentSlot);
            SetMessageToCheckingForCard();
            s_fWaitTimer = fWAIT_TIMER;
            s_eCurrentCommandStatus = CMD_STATUS_CARD_CHECK;
        }
        break;

        case CMD_STATUS_CARD_CHECK:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
            s_fWaitTimer        = fWAIT_TIMER;
    		if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_ERROR_NO_CARD)
	    	{
				SetPhase(PHASE_CHECK_FOR_CARD);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
		    }
		    else if(s_bCardChanged)
		    {
				SetPhase(PHASE_PROMPT_CARD_CHANGED);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
		    }
		    else
    		{
                Asura_PS2_MemoryCard_System::ChangeToProductDir(s_iCurrentPort, s_iCurrentSlot);
    			Asura_PS2_MemoryCard_System::Delete(s_iCurrentPort, s_iCurrentSlot, s_szFileName);
                s_fProgressOfCurrentOperation = 0.0f;
                s_eCurrentMessage = MSG_DELETING;
                s_eCurrentCommandStatus = CMD_STATUS_WAIT;
            }
        }
        break;

        case CMD_STATUS_WAIT:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
            // check for error and goto next stage
            s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
            s_fWaitTimer        = fWAIT_TIMER;
            if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_SUCCEED)
            {
                s_bCreateIconAndDir = true;
				SetPhase(PHASE_FINISHED);
                if(!s_xBatchInformation.bInBatchOperation || 
                     s_xBatchInformation.bLastOperationInBatch)
                {
                    s_eCurrentMessage = s_xBatchInformation.bSetAsOverwrite ? MSG_OVERWRITE_SUCCESSFUL
                                                                            : MSG_SAVE_SUCCESSFUL;
                }
            }
            else
            {
				SetState(STATE_ERROR);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
				SetPhase(PHASE_DELETE_FAILED);
            }
            s_xBatchInformation.bFirstOperationInBatch = false;
            // (NRM - 11.08.2003) set silently delete to be false as well - the user must call this every time they
            // call delete - which I like!
            s_xBatchInformation.bSilentlyDelete = false;
        }
        break;

        default:
            Asura_Assert(0, "Invalid command status");
    }    
}

void Asura_PS2_MemoryCard_Shell::ProcessDeletingSaveDirectory(void)
{
    enum SubPhase {PS_GETTING_DIR_LIST, PS_DELETING_DIR_CONTENTS, PS_DELETING_DIR};
    static SubPhase eSubPhase = PS_GETTING_DIR_LIST;
    switch(s_eCurrentCommandStatus)
    {
        case CMD_STATUS_ISSUE:
        {
            // First check that the card is still present so as to not delete from a card that has been switched in
			SetState(STATE_RUNNING);
    		Asura_PS2_MemoryCard_System::Rescan(s_iCurrentPort, s_iCurrentSlot);
            SetMessageToCheckingForCard();
            s_fWaitTimer = fWAIT_TIMER;
            s_eCurrentCommandStatus = CMD_STATUS_CARD_CHECK;
        }
        break;

        case CMD_STATUS_CARD_CHECK:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }
            s_fWaitTimer        = fWAIT_TIMER;
    		if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_ERROR_NO_CARD)
	    	{
				SetPhase(PHASE_CHECK_FOR_CARD);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
		    }
		    else if(s_bCardChanged)
		    {
				SetPhase(PHASE_PROMPT_CARD_CHANGED);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
		    }
		    else
    		{
                s_eCurrentMessage = MSG_DELETING;
                switch(eSubPhase)
                {
                    case PS_GETTING_DIR_LIST:
                    {
                        // Get the directory list
                        Asura_PS2_MemoryCard_System::ChangeToProductDir(s_iCurrentPort, s_iCurrentSlot);
                        Asura_PS2_MemoryCard_System::GetDirList(s_iCurrentPort, s_iCurrentSlot, s_szDirList);
                        eSubPhase = PS_DELETING_DIR_CONTENTS;
                    }
                    break;
                    case PS_DELETING_DIR_CONTENTS:
                    {
                        // delete everything out of the directory
                        char* pcCurrentString = &(s_szDirList[1]);
                        for(int i=0; i<Asura_PS2_MemoryCard_System::GetNumberOfEntries(); ++i)
                        {
                            if(!(strcmp(pcCurrentString, ".") == 0 ||
                                 strcmp(pcCurrentString, "..") == 0))
                            {
            			        Asura_PS2_MemoryCard_System::Delete(s_iCurrentPort, s_iCurrentSlot, pcCurrentString);                                
                            }
                            pcCurrentString += (strlen(pcCurrentString) + 1);
                            eSubPhase = PS_DELETING_DIR;
                        }
                    }
                    break;
                    case PS_DELETING_DIR:
                    {
                        // change to root directory
                        static char szPrimaryName[22];
                        Asura_PS2_MemoryCard_System::GetPrimaryName(szPrimaryName);
                        Asura_PS2_MemoryCard_System::ChangeDir(s_iCurrentPort, s_iCurrentSlot, "/");
                        // delete the directory
    			        Asura_PS2_MemoryCard_System::Delete(s_iCurrentPort, s_iCurrentSlot, szPrimaryName);
                        s_fProgressOfCurrentOperation = 0.0f;
                        s_eCurrentCommandStatus = CMD_STATUS_WAIT;
                    }
                    break;
                }
            }
        }
        break;
        case CMD_STATUS_WAIT:
        {
            if(!Asura_PS2_MemoryCard_System::Sync(ASURA_PS2_MEMORYCARD_SYNC_NOWAIT))
            {
                return;
            }

            if(Asura_PS2_MemoryCard_ErrorSystem::GetError() == ASURA_PS2_MEMORY_CARD_SUCCEED)
            {
                s_fWaitTimer      = fWAIT_TIMER;
                s_bFinishedSuccessfully = true;
				SetPhase(PHASE_FINISHED);
                if(!s_xBatchInformation.bInBatchOperation || 
                     s_xBatchInformation.bLastOperationInBatch)
                {
                    s_eCurrentMessage = MSG_DELETE_SUCCESSFUL;
                }
                s_bDisableAutosave = false;         // got a card so autosave
            }
            else
            {
				SetState(STATE_ERROR);
                s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
				SetPhase(PHASE_DELETE_FAILED);
            }
            s_xBatchInformation.bFirstOperationInBatch = false;
            // (NRM - 11.08.2003) set silently delete to be false as well - the user must call this every time they
            // call delete - which I like!
            s_xBatchInformation.bSilentlyDelete = false;
        }
        break;
        default:
            Asura_Assert(0, "Invalid command status");
    }
}

/********** P R O C E S S   P R O M P T D E L E T E **********/

void Asura_PS2_MemoryCard_Shell::ProcessPromptDelete(void)
{
    s_eCurrentGetInputMessage = MSG_GI_DELETE;
    s_eCurrentMessage         = s_eEntryCommand == CMD_START_DELETE ? MSG_ERR_CONFIRM_DELETE 
                                                                    : MSG_ERR_CONFIRM_DELETE_SAVE_DIRECTORY;
	SetState(STATE_ERROR);

    UserInput eUI = GetUserInput();
	if(s_bCardChanged)
	{
		SetPhase(PHASE_PROMPT_CARD_CHANGED);
        s_eCurrentCommandStatus = CMD_STATUS_ISSUE;
        return;
	}

    if(eUI == UI_DELETE)
    {
        PlayRetrySound();       //Ok, so it's just the same as retry
        //s_eCurrentPhase = s_eEntryCommand == CMD_START_DELETE ? PHASE_DELETING
        //                                                      : PHASE_DELETING_SAVE_DIRECTORY;
		SetPhase(s_eEntryCommand == CMD_START_DELETE ? PHASE_DELETING
			                                                    : PHASE_DELETING_SAVE_DIRECTORY);
        s_eCurrentCommandStatus = CMD_STATUS_ISSUE; // Reissue the command
        s_bFinishedSuccessfully = true;
        // (NRM - 30.06.2003) reset the first operation as this is the REAL first operation
        if(s_xBatchInformation.bInBatchOperation)
        {
            s_xBatchInformation.bFirstOperationInBatch = true;
        }
    }
    else if(eUI == UI_CANCEL)
    {
        PlayCancelSound();
        s_bFinishedSuccessfully = false;
		SetPhase(PHASE_FINISHED);
    }
}

/********** P R O C E S S   D E L E T E   F A I L E D **********/

void Asura_PS2_MemoryCard_Shell::ProcessDeleteFailed(void)
{
    s_eCurrentGetInputMessage = MSG_GI_RETRY;
    WaitForInput(MSG_ERR_DELETE_FAILED, PHASE_CHECK_FOR_CARD);
}

#endif // !ASURA_PS2_DEMODISK

int Asura_PS2_MemoryCard_Shell::SetDataSize(const int* aiDataSize, const int iNumberOfFiles)
{
    // iNumberOfFiles + 3.f as +1.f from tech note, +1.f for icon.sys. +1.f for icon itself
    s_iCheckDataSize = 0;
    for(int i=0; i<iNumberOfFiles; ++i)
    {
        s_iCheckDataSize += Asura_Maths::FloatToInt_Chop((aiDataSize[i] + 1023.f) / 1024.f) * 1024;
    }

    s_iCheckDataSize += Asura_Maths::FloatToInt_Chop((iICON_SYS_SIZE + 1023.f) / 1024.f) * 1024
                      + Asura_Maths::FloatToInt_Chop(((iNumberOfFiles + 3.f) / 2.f) + 2.f) * 1024;	// conv into bytes
    return s_iCheckDataSize;
}

void Asura_PS2_MemoryCard_Shell::PlayRetrySound(void)
{
    if(static_cast<Asura_Sound_ID>(s_iRetrySound) == ASURA_SID_NO_SOUND)
        return;

    Asura_Sound_System::Play2DSound(static_cast<Asura_Sound_ID>(s_iRetrySound), s_fMenuSoundVolume);
}

void Asura_PS2_MemoryCard_Shell::PlayCancelSound(void)
{
    if(static_cast<Asura_Sound_ID>(s_iCancelSound) == ASURA_SID_NO_SOUND)
        return;

    Asura_Sound_System::Play2DSound(static_cast<Asura_Sound_ID>(s_iCancelSound), s_fMenuSoundVolume);
}
