//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_MemoryCard_Shell_UnitTest.cpp
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     Unit test for memory card shell system - do NOT add to your projects
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "MemoryCard/Asura_PS2_MemoryCard_Shell_UnitTest.h"

#include "MemoryCard/Asura_PS2_MemoryCard_Shell.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"

#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Font_System.h"

#include "Core/Asura_Maths.h"

char Asura_PS2_MemoryCard_Shell_UnitTest::s_szSaveNameList[512] = "\0";

const char* aszTRCMessages[] =
{
    /* MSG_CHECKING_CARD*/          "Checking memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1.  Do not remove memory card (8MB) (for PlayStation2), reset, or switch off the console.",
    /* MSG_LOADING */               "Loading data.  Do not remove memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1, reset, or switch off the console.",   
    /* MSG_SAVING */                "Saving data.  Do not remove memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1,  reset, or switch off the console.",
    /* MSG_AUTOSAVING*/             "Autosaving data.  Do not remove memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1, reset, or switch off the console.",
    /* MSG_FORMATTING*/             "Formatting memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1.  Do not remove memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1, reset, or switch off the console.",
    /* MSG_LOAD_SUCCESSFUL*/        "Load successful",
    /* MSG_SAVE_SUCCESSFUL*/        "Save successful",
    /* MSG_FORMAT_SUCCESSFUL*/      "Format succesful",
    /* MSG_ERR_NO_CARD*/            "No memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1.\nX - Retry, TRI - Cancel",
    /* MSG_ERR_NO_CARD_ON_BOOT*/    "Caution! If you wish to save your game data, insert a memory card (8MB) (for PlayStation2) with at least 136KB of free space or press TRI button to continue without saving.\nX - Retry, TRI - Cancel",
    /* MSG_ERR_NO_FORMAT*/          "Memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1 is unformatted.  Format memory card (8MB) (for PlayStation2)?\nO - Yes, TRI - No",
    /* MSG_ERR_NO_SAVE_CREATE*/     "Gunfighter II uses an Autosave feature.  Do you wish to create a save game on the memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1?  This will use 136KB.\nX - Yes, TRI - NO",
    /* MSG_ERR_NO_SAVES*/           "No Gunfighter II saves exist on the memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1.\nX - Retry, TRI - Cancel",
    /* MSG_ERR_NO_ROOM*/            "Insufficient space on memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1.  Gunfighter II requires 136KB of free space to save data.\nX - Retry, TRI - Cancel",
    /* MSG_ERR_FILE_DOES_NOT_EXIST*/"Requested file does not exist on memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1.\nX - Retry, TRI - Cancel",
    /* MSG_ERR_FORMAT_FAILED*/      "Format failed!  Check memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1 and please try again.\nX - Retry, TRI - Cancel",
    /* MSG_ERR_LOAD_FAILED*/        "Load failed!  Check memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1 and please try again.\nX - Retry, TRI - Cancel",
    /* MSG_ERR_SAVE_FAILED*/        "Save failed!  Check memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1 and please try again.\nX - Retry, TRI - Cancel",
    /* MSG_ERR_SAVE_EXISTS*/        "A save of that name already exists on memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1.  Overwrite this save data?\nX - Yes, TRI - No",
    /* MSG_ERR_CARD_CHANGED*/       "Data on memory card (8MB) (for PlayStation2) in MEMORY CARD slot 1 has changed.  Overwrite previous Gunfighter II data?\nX - Yes, TRI - No",
    /* MSG_ERR_RETRY_SAVE*/         "Caution!  Cancel save?\nX - No, TRI - Yes",
    /* MSG_ERR_RETRY_LOAD*/         "Caution!  Cancel load?\nX - No, TRI - Yes",
    /* MSG_NONE*/                   "",
    /* MSG_INVALID*/                "INVALID.  I N V A L I D",
};

struct
{
    int iVersionNumber;
    char ucData[128];
} xSaveData;

void Asura_PS2_MemoryCard_Shell_UnitTest::Initialise(void)
{
    Asura_PS2_MemoryCard_Shell::Initialise();
    Asura_PS2_MemoryCard_Shell::CreateDefaultIcon("MCIcon\\guns.ico");
    Asura_PS2_MemoryCard_Shell::SetProductCode("SLES-54321");
    Asura_PS2_MemoryCard_Shell::SetProductName("Dredd");
    s_szSaveNameList[0] = '\0';
    Asura_Timers::Initialise();
    xSaveData.ucData[0] = 'N';
    xSaveData.ucData[1] = 'E';
    xSaveData.ucData[2] = 'I';
    xSaveData.ucData[3] = 'L';
    xSaveData.ucData[4] = '\0';
}

DEBUGVAR float fScale = .7f;
DEBUGVAR float fLineSpaceRatio = 1.f;
DEBUGVAR float fFooterY = .86f;
DEBUGVAR float fFooterScale = .5f;
DEBUGVAR float dbg_fOffsetY = -.03f;

void Asura_PS2_MemoryCard_Shell_UnitTest::Update(void)
{
    Asura_PS2_MemoryCard_Shell::Update();
    Asura_UserInput::Update();
    Asura_Render::BeginFrame();

    Asura_PS2_MemoryCard_Shell::DisplayMessage eMessage = Asura_PS2_MemoryCard_Shell::GetCurrentMessage();

	const int iFormat   = ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_WORDWRAP |
                          ASURA_TEXTFORMAT_CENTREJUSTIFIED  | ASURA_TEXTFORMAT_VERTICALCENTREJUSTIFIED;
    static Asura_Bounding_Rectangle s_xTextRect(0.2f, 0.3f, 0.8f, 0.7f);
	Asura_Font_System::Print(s_xTextRect, 0.f, dbg_fOffsetY, fLineSpaceRatio, fScale, 0xFFFFFFFF, iFormat, 
							 aszTRCMessages[eMessage]);
    

    Asura_Render::EndFrame();
    Asura_Timers::HandlePrimaryTimer();
}

void Asura_PS2_MemoryCard_Shell_UnitTest::RunCheckCardAtBootTest(void)
{
    Asura_PS2_MemoryCard_Shell::CheckCardOnBoot();
}

void Asura_PS2_MemoryCard_Shell_UnitTest::RunSaveTest(void)
{
    Asura_PS2_MemoryCard_Shell::Save("Dredd Player 1", &xSaveData, sizeof(xSaveData));
}

void Asura_PS2_MemoryCard_Shell_UnitTest::RunLoadTest(void)
{
    Asura_PS2_MemoryCard_Shell::GetSaveNameList(s_szSaveNameList);
    while(!Asura_PS2_MemoryCard_Shell::Finished())
    {
        Asura_PS2_MemoryCard_Shell_UnitTest::Update();
    }
    if(s_szSaveNameList[0] != 0)
    {
        char* szSaveName = &(s_szSaveNameList[1]);
        Asura_PS2_MemoryCard_Shell::Load(szSaveName, &xSaveData, sizeof(xSaveData));
    }
}

void Asura_PS2_MemoryCard_Shell_UnitTest::RunAutoSaveTest(void)
{
    Asura_PS2_MemoryCard_Shell::AutosaveMain(&xSaveData, sizeof(xSaveData));
}

void Asura_PS2_MemoryCard_Shell_UnitTest::RunSaveMainTest(void)
{
    Asura_PS2_MemoryCard_Shell::SaveMain(&xSaveData, sizeof(xSaveData));
}

void Asura_PS2_MemoryCard_Shell_UnitTest::RunGetSaveListTest(void)
{
    Asura_PS2_MemoryCard_Shell::GetSaveNameList(s_szSaveNameList);
}

void Asura_PS2_MemoryCard_Shell_UnitTest::RunLoadMainTest(void)
{
    Asura_PS2_MemoryCard_Shell::LoadMain(&xSaveData, sizeof(xSaveData));
}

void Asura_PS2_MemoryCard_Shell_UnitTest::RunAutoLoadTest(void)
{
    Asura_PS2_MemoryCard_Shell::AutoLoadMain(&xSaveData, sizeof(xSaveData));
}

void Asura_PS2_MemoryCard_Shell_UnitTest::Run(void)
{
    Asura_PS2_MemoryCard_Shell_UnitTest::Initialise();
    Asura_PS2_MemoryCard_Shell_UnitTest::RunCheckCardAtBootTest();
    while(!Asura_PS2_MemoryCard_Shell::Finished())
    {
        Asura_PS2_MemoryCard_Shell_UnitTest::Update();
    }
    Asura_PS2_MemoryCard_Shell_UnitTest::RunAutoSaveTest();
    while(!Asura_PS2_MemoryCard_Shell::Finished())
    {
        Asura_PS2_MemoryCard_Shell_UnitTest::Update();
    }

    Asura_PS2_MemoryCard_Shell_UnitTest::RunSaveTest();
    while(!Asura_PS2_MemoryCard_Shell::Finished())
    {
        Asura_PS2_MemoryCard_Shell_UnitTest::Update();
    }

    Asura_PS2_MemoryCard_Shell_UnitTest::RunAutoSaveTest();
    while(!Asura_PS2_MemoryCard_Shell::Finished())
    {
        Asura_PS2_MemoryCard_Shell_UnitTest::Update();
    }

    memset(&xSaveData, 0, sizeof(xSaveData));
    Asura_PS2_MemoryCard_Shell_UnitTest::RunAutoLoadTest();
    while(!Asura_PS2_MemoryCard_Shell::Finished())
    {
        Asura_PS2_MemoryCard_Shell_UnitTest::Update();
    }

    printf(xSaveData.ucData);

    memset(&xSaveData, 0, sizeof(xSaveData));
    Asura_PS2_MemoryCard_Shell_UnitTest::RunLoadMainTest();
    while(!Asura_PS2_MemoryCard_Shell::Finished())
    {
        Asura_PS2_MemoryCard_Shell_UnitTest::Update();
    }
    printf(xSaveData.ucData);

    Asura_PS2_MemoryCard_Shell::GetSaveNameList(s_szSaveNameList);
    while(!Asura_PS2_MemoryCard_Shell::Finished())
    {
        Asura_PS2_MemoryCard_Shell_UnitTest::Update();
    }
    printf("%s\n",&(s_szSaveNameList[1]));
    Asura_PS2_MemoryCard_Shell_UnitTest::RunLoadTest();
    while(!Asura_PS2_MemoryCard_Shell::Finished())
    {
        Asura_PS2_MemoryCard_Shell_UnitTest::Update();
    }
}
