//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_eznetcnf.h
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_EZNETCNF_H___
#define ___ASURA_PS2_EZNETCNF_H___ 1

#include "Core/Asura_LocalisedText.h"

// middleware
#include "Sony/eznetcnf/include/eznetcnf.h"

class Asura_PS2_EZNetCnf
{

public:

	enum NetState
	{
		NetState_None,
		NetState_LoadingIRXs,				// loading eznetcnf IRXs from CD
		NetState_WaitingForMemCardState,	// checking for MC present
		NetState_GettingConfigs,			// memory card is being read
		NetState_DisplayingConfigs,			// waiting to be told 
		NetState_Initialising,				// waiting for selected config to be initialised
		NetState_Connected,					// everything's OK
		NetState_Error,
	};

	// todo - check each of these error conditions
	enum NetCnfError
	{
		Error_None,
		Error_LoadingIRXsFailed,		// probably CD tray open
		Error_NoConfigsOnMemoryCard,	// display appropriate message
		Error_MagicError,				// created on different PS2 - display special message
		Error_NoMemoryCardInSlot1,		// display appropriate message
		Error_MemoryCardNotFormatted,	// display appropriate message
		Error_CorruptedData,			// display appropriate message
		Error_HardwareNotConnected,		// display appropriate message, using GetConfigType
		Error_HardwareNotSupported,		// display appropriate message, using GetConfigType
		Error_CableUnplugged,			// display appropriate message
		Error_ConnectionFailed,			// display appropriate message
		Error_AuthorisationError,		// special case of connection failed.
		Error_ConnectionTimedOut,		// display appropriate message
		Error_LoadingNetworkIRXsFailed,	// probably CD tray open
	};

	static void BeginNetCnfSelection( void );	// call this to begin checking MC1
	static void RestartNetCnfSelection( void );	// call this to begin checking MC1 again
	static bool UpdateNetCnfSelection( void );	// call this to run through the state machine
	static void FinishNetCnfSelection( void );	// call this when finished selection, whether successful or not
	
    static bool IsConnectionActive( void ) { return s_bHasConnection; }
	static bool IsCablePluggedIn( void );	// once connection is established, find out if cable is connected
	static bool IsConnectionOK( void );		// once connection is established, find out if anything has gone wrong

    static void ShutdownNetworkConnection( void ) { if( IsConnectionActive() ) { ShutdownNetworkConnection_Internal(); } }	

	static const char* GetStatusMessage( void );		// return localised message - TODO, replace const char* with enum value
	static NetState GetState( void ) { return s_eState; }
	static NetCnfError GetError( void ) { return s_eError; }
	static u_int GetNumConfigs( void );
	static const char* GetConfigName( u_int uConfigNumber );	// name user gave this config
	static const char* GetConfigType( u_int uConfigNumber );	// config type (e.g. SCE/Ethernet (Network Adaptor))
	static bool ConfigIsPresent( u_int uConfigNumber );		// used to filter out empty slots - display <empty> or just nothing
	static bool ConfigIsSelectable( u_int uConfigNumber );	// used to filter out invalid entries - should be greyed-out
	static void ChooseConfig( u_int uConfigNumber ) { InitConfig(uConfigNumber); }
	static u_int GetDefaultConfig( void );				// get the config number of the default config.
	
	static bool CanUseLANBroadcast( void ) { return s_bCanUseLANBroadcast; }

    // function which gives an example of usage for reference, and can be used
    // to quickly implement config selection for prototypes that don't have menus.
    // if bAutoChooseDefault is set to true, the default config is selected.
    // Set to false if you want to test using the DEBUGVAR dbg_uHACKConfigChoice
    static void FakeConfigSelectionMenu(bool bAutoChooseDefault = true);

	static bool IsPPPoE();

private:

	enum MemCardState
	{
		MemCard_OK,
		MemCard_NotPresent,
		MemCard_NotFormatted,
		MemCard_Unknown,
		MemCard_Recheck,
	};

	// internal update function which calls ezNetCtlGetStatus() once every few seconds
	static void UpdateConnectedStatus( void );

	static bool LoadEENetCore( void );
	static bool LoadEENetSMAP( void );
	static bool EENetStartSpecific(struct sceNetcnfifData* pxData);
	static bool ShutdownEENet( void );

	static void SetError( NetCnfError eError );
	static void LoadNetCnfAndNetCtlIRXs( void );
	static void GetConfigs( void );
	static void InitConfig( u_int uConfigNumber );
	static void UpdateInitialisingState ( void );

	static void UpdateMemoryCard( void );
	static void UnloadNetCnfAndNetCtlIRXs( void );

    static void ShutdownNetworkConnection_Internal( void );

	static NetState s_eState;
	static NetCnfError s_eError;
	static u_int s_uNumConfigs;		// will generally be either 0 or 6

	// mem card state
	static MemCardState s_eMemoryCardState;
	
	// status and data for mem card library function call
	static int s_iMemoryCardIsFormatted;
	static bool s_bMemoryCardCheckInProgress;

	static bool s_bCanUseLANBroadcast;
	static bool s_bHasConnection;
	static bool s_bNetCnfAndNetCtlIRXsLoaded;

	static char s_szMsg[kStrBufferSize];
	static u_int s_uTimerValue;

	static bool s_bIsPPPoE;
	static bool s_bAssumePPPoEConnectionDead;
};


// Menu code should call BeginNetCnfSelection() to begin selection process.
// call UpdateNetCnfSelection() every frame and check the return value. If 
// false, display an appropriate error message using GetError().
// When state is NetState_DisplayingConfigs display each config using 
// GetNumConfigs() and GetConfigName(). NB config numbers start at 1, not 0.
// On selection of a config, call ChooseConfig() with the relevant number.
// While the state is NetState_Initialising, keep displaying a message, until 
// state is NetState_Connected or an error occurs.
// Then call FinishNetCnfSelection().
//
// Error handling: if an error occurs during processing, display a dialog 
// to retry/cancel. If retry is selected, call RestartNetCnfSelection, else 
// call FinishNetCnfSelection().

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_EZNETCNF_H___
