//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_Profiling.cpp
//
// By: Morgan Parry
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"                          // Global header.

// This file's header.
#include "Profiling/Asura_PS2_Profiling.h"

// Stdlib platform-specific.
#include <eekernel.h>
#include <eestruct.h>
#include <eeregs.h>
#include <libgraph.h>
#if ASURA_PS2_SN_PROFILER
#include <eetypes.h>
#include <libsn.h>
#include <Sony\PS2\TunerDev\SNTuner.h>
#include <sifdev.h> 
#include <stdio.h>
#endif

// Engine platform-specific.
#include "Core/Asura_PS2_DMA.h"						// For dma tags
#include "Core/Asura_PS2_Timers.h"					// for EE cycles/s
#include "Rendering/Asura_PS2_DeferredRendering.h"	// For GetVifBuffer()
#include "Rendering/Asura_PS2_GS.h"					// For ScreenWidth and refresh rate

// M A C R O S ///////////////////////////////////////////////////////////////

#if ASURA_PS2_DRAW_PROFILE_BARS
#define FRAME_SMOOTHING	(6)
#endif

// C O N S T A N T S /////////////////////////////////////////////////////////

#if ASURA_PS2_SN_PROFILER
static const char* szPROFILING_MODULE_FILENAME = "host0:~\\usr\\local\\sce\\iop\\modules\\SNProfil.irx";
#endif

// G L O B A L S /////////////////////////////////////////////////////////////

#if ASURA_PS2_DRAW_PROFILE_BARS

DEBUGVAR bool dbg_bEnableVifProfiling = true;
DEBUGVAR bool dbg_bEnableEEProfiling  = true;

DEBUGVAR bool dbg_bVifProfilingOptimiseOutRedundantMarkers = true;

static Asura_PS2_QW* g_pxLastVifEnd;
static u_int         g_uMarkForLastVifEnd;

// These need to be volatile since we're fiddling with them inside interrupts.
static volatile u_int g_auVIFMarkerStartTimes[ASURA_PS2_PROFILE__MARKER_QTY];
static volatile u_int g_auVIFMarkerTimes[ASURA_PS2_PROFILE__MARKER_QTY];

u_int g_auEEMarkerStartTimes[ASURA_PS2_PROFILE__MARKER_QTY];
u_int g_auEEMarkerTimes[ASURA_PS2_PROFILE__MARKER_QTY];

int g_iVIF1InteruptHandlerID = -1;

int   g_aiFrameSmoothing[2];
u_int g_uCurrentFrame;

#define MAX_NUM_MARKER_COLOURS 32

// Must be a power of 2
ASURA_STATIC_ASSERT( ( MAX_NUM_MARKER_COLOURS & ( MAX_NUM_MARKER_COLOURS - 1 ) ) == 0 );

/* JAT 13/07/06 - old PS2 colours - now moved to using the same colours as in Asura_CPUProfile_Renderer.cpp
							   0x00FFFFFF,   // 0  - white - ASURA_PS2_PROFILE__TOTAL_FRAME_MARKER ASURA_PS2_PROFILE__USER_MARKER_2
                               0x00FF0000,   // 1 - blue - ASURA_PS2_PROFILE__SKYSPHERE_MARKER  ASURA_PS2_PROFILE__USER_MARKER_1
                               0x0000FF00,   // 2 - green - ASURA_PS2_PROFILE__SKYBOX_MARKER  
                               0x000000FF,   // 3 - red - ASURA_PS2_PROFILE__ENVIRONMENT_MARKER
                               0x00FFFF00,   // 4 - cyan - ASURA_PS2_PROFILE__OBJECT_MARKER
                               0x0000FFFF,   // 5- yellow - ASURA_PS2_PROFILE__HIERACHY_MARKER
                               0x00FF00FF,   // 6 - magenta - ASURA_PS2_PROFILE__SKINNED_MARKER
                               0x00000000,   // 7 - black - ASURA_PS2_PROFILE__PARTICLE_MARKER
							   0x0020FFA0,   // 8 - lime green - ASURA_PS2_PROFILE__CORONA_MARKER
                               0x002580FF,   // 9 - orange - ASURA_PS2_PROFILE__PROJ_SHADOW_MARKER
                               0x00A07040,   // 10 - soft blue - ASURA_PS2_PROFILE__WATER_MARKER
                               0x009055FF,   // 11 - pink - ASURA_PS2_PROFILE__CARPET_MARKER
                               0x00FF5590,   // 12 - purple - ASURA_PS2_PROFILE__BOTANICALS_MARKER
                               0x00FF8000,   // 13 - bright blue - ASURA_PS2_PROFILE__USER_MARKER_5
                               0x00A0C0FF,   // 14 - bright pink - ASURA_PS2_PROFILE__USER_MARKER_4
                               0x00889A00,   // 15 - greeny blue - ASURA_PS2_PROFILE__USER_MARKER_3
													};
*/
// JAT 13/07/06 - moved to using the same colours as in Asura_CPUProfile_Renderer.cpp
// Also changed the renderer to use PC format colours
// Note these are PC colours so ARGB not ABGR (PS2)
u_int g_auMarkerColours[MAX_NUM_MARKER_COLOURS] =  { 
0xff604020, 0xff2222ff, 0xff22ff22, 0xffff2222, 0xff22ffff, 0xffffff22, 0xffff22ff, 0xff222222,
0xffffffff, 0xff5555dd, 0xff55dd55, 0xffdd5555, 0xff55dddd, 0xffdddd55, 0xffdd55dd, 0xff555555,
0xffdddddd, 0xff2222ff, 0xff22ff22, 0xffff2222, 0xff22ffff, 0xffffff22, 0xffff22ff, 0xff222222,
0xffffffff, 0xff5555dd, 0xff55dd55, 0xffdd5555, 0xff55dddd, 0xffdddd55, 0xffdd55dd, 0xff555555,
													};

#if _DEBUG
static bool g_abVIFMarkerSet[ASURA_PS2_PROFILE__MARKER_QTY];
#endif

#endif // #if ASURA_PS2_DRAW_PROFILE_BARS

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

#if ASURA_PS2_SN_PROFILER
    u_long128 Asura_PS2_ProfilingSystem::s_auSNProfilingData[Asura_PS2_ProfilingSystem::NUM_SN_PROFILING_DATA_QWORDS];
bool	  Asura_PS2_ProfilingSystem::s_bEnableSNProfiling = true;
bool	  Asura_PS2_ProfilingSystem::s_bSNProfilerEnabled = true;
#endif

//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Asura_PS2_ProfilingSystem::Initialise().
// PARAMETERS   None.
// RETURNS      True iff successful.
// DESCRIPTION  Initialises the profiling module.
// AUTHOR       Morgan Parry - 16.10.2001.
//
//////////////////////////////////////////////////////////////////////////////
bool Asura_PS2_ProfilingSystem::Initialise( bool bSNProfiler )
{
    bool bRetVal = true;
#if ASURA_PS2_SN_PROFILER
	if ( bSNProfiler )
	{
		if( sceSifLoadModule(szPROFILING_MODULE_FILENAME, 0, NULL) < 0 )
		{
			printf( "Couldn't load profiling module.\n" );
			bRetVal = false;
		}
		else if( snProfInit(_4KHZ, s_auSNProfilingData, sizeof(s_auSNProfilingData)) )
		{
			printf( "Profiler initialisation failed.\n" );
			bRetVal = false;
		}
	}
#endif

#if ASURA_PS2_DRAW_PROFILE_BARS
	// Add the VIF1 Interrupt handler for timing drawing
	g_iVIF1InteruptHandlerID = AddIntcHandler( INTC_VIF1, VIF1_InteruptHandler, 0 );
	EnableIntc( INTC_VIF1 );
	g_uCurrentFrame = 0;
    g_pxLastVifEnd = NULL;

#if _DEBUG
    // Slight hack - due to our slightly odd frame flow, in particular doing
    // the frame- or z-buffer clearing at the end of EndFrame() rather than
    // the start of BeginFrame(), we won't have this set correctly in the
    // first frame.
    g_abVIFMarkerSet[ASURA_PS2_PROFILE__TOTAL_FRAME_MARKER] = true;
#endif
#endif // #if ASURA_PS2_DRAW_PROFILE_BARS

    return bRetVal;
}
// End function Asura_PS2_ProfilingSystem::Initialise().

//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Asura_PS2_ProfilingSystem::Shutdown().
// PARAMETERS   None.
// RETURNS      None.
// DESCRIPTION  Shuts down the profiling module.
// AUTHOR       Morgan Parry - 16.10.2001.
//
//////////////////////////////////////////////////////////////////////////////
void Asura_PS2_ProfilingSystem::Shutdown( void )
{
#if ASURA_PS2_DRAW_PROFILE_BARS
	// Remove our interrupt handler
	DisableIntc( INTC_VIF1 );
	RemoveIntcHandler( INTC_VIF1, g_iVIF1InteruptHandlerID );
    g_iVIF1InteruptHandlerID = -1;
#endif
}
// End function Asura_PS2_ProfilingSystem::Shutdown().

//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Asura_PS2_ProfilingSystem::Update().
// PARAMETERS   None.
// RETURNS      None.
// DESCRIPTION  Checks and enables/disables Profiler interrupts, so in the 
//				debugger you can switch off profiler interrupts for PA grabs
// AUTHOR       Pete Ellacott - 23.10.2003.
//
//////////////////////////////////////////////////////////////////////////////
void Asura_PS2_ProfilingSystem::Update()
{
#if ASURA_PS2_SN_PROFILER
	if ( s_bEnableSNProfiling != s_bSNProfilerEnabled)
	{
		if ( s_bEnableSNProfiling )
		{
			_snProfEnableInt();
		}
		else
		{
			_snProfDisableInt();
		}
		s_bSNProfilerEnabled = s_bEnableSNProfiling;
	}
#endif

#if ASURA_PS2_DRAW_PROFILE_BARS
    g_pxLastVifEnd = NULL;
#endif
}

#if ASURA_PS2_DRAW_PROFILE_BARS
//////////////////////////////////////////////////////////////////////////////
// FUNCTION     Asura_PS2_ProfilingSystem::AddVIFStart
// PARAMETERS   Marker identifier.
// DESCRIPTION  Adds a DMA packet containing a VIF1_MARK( uMarkerNumber ) and a VIF1_NOP[i]
//				for the VIF1 interupt handler to find and time
// AUTHOR       Pete Ellacott - 07.03.2003.
//////////////////////////////////////////////////////////////////////////////
void Asura_PS2_ProfilingSystem::AddVIFStart( u_int uMarkerID )
{
    if( !dbg_bEnableVifProfiling ) { return; }

    Asura_Assert( -1 != g_iVIF1InteruptHandlerID, "" );
    Asura_Assert( uMarkerID < ASURA_PS2_PROFILE__MARKER_QTY, "" );

#if _DEBUG
    Asura_Assert( !g_abVIFMarkerSet[uMarkerID], "" );
    g_abVIFMarkerSet[uMarkerID] = true;

    // In fact, I don't think we want any other marker set at the same time,
    // apart from the total frame marker. The linear way we display them
    // isn't really going to show when one's also incorporated in another.
    for( u_int u = 1; u < ASURA_PS2_PROFILE__MARKER_QTY; ++u )
    {
        if( u != uMarkerID ) { Asura_Assert( !g_abVIFMarkerSet[u], "" ); }
    }
#endif

    // If we've only just written out an end for the same marker, just
    // cancel that instead.
    if( g_pxLastVifEnd == Asura_PS2_DeferredRendering::GetCurVifBfrPtr() - 1 && g_uMarkForLastVifEnd == uMarkerID )
    {
        Asura_PS2_DeferredRendering::ReturnVifBuffer( g_pxLastVifEnd, 1 );
        g_pxLastVifEnd = NULL;
        return;
    }

    Asura_PS2_DmaTag* pxTag = reinterpret_cast<Asura_PS2_DmaTag*>( Asura_PS2_DeferredRendering::GetVifBuffer(1) );
    pxTag->Set( 0,
                Asura_PS2_DmaTag::PCE_NOCHANGE,
                Asura_PS2_DmaTag::ID_CNT,
                0,
                false,
                0,
                SCE_VIF1_SET_MARK(uMarkerID | ASURA_PS2_PROFILE__START_MARK_MASK, false), // Record what we are profiling.
                SCE_VIF1_SET_NOP(true) );                                             // Cause an interrupt.
}

//////////////////////////////////////////////////////////////////////////////
// FUNCTION     Asura_PS2_ProfilingSystem::AddVIFEnd
// PARAMETERS   Marker identifier.
// DESCRIPTION  Adds a DMA packet containing a VIF1_MARK( uMarkerNumber ) and a VIF1_NOP[i]
//				for the VIF1 interupt handler to find and time
// AUTHOR       Pete Ellacott - 07.03.2003.
//////////////////////////////////////////////////////////////////////////////
void Asura_PS2_ProfilingSystem::AddVIFEnd( u_int uMarkerID )
{
    if( !dbg_bEnableVifProfiling ) { return; }

    Asura_Assert( -1 != g_iVIF1InteruptHandlerID, "" );
	Asura_Assert( uMarkerID < ASURA_PS2_PROFILE__MARKER_QTY, "" );

#if _DEBUG
    Asura_Assert( g_abVIFMarkerSet[uMarkerID], "" );
    g_abVIFMarkerSet[uMarkerID] = false;
#endif

    Asura_PS2_QW* pxOut = Asura_PS2_DeferredRendering::GetVifBuffer( 1 );

    (reinterpret_cast<Asura_PS2_DmaTag*>(pxOut))->Set( 0,
                                                       Asura_PS2_DmaTag::PCE_NOCHANGE,
                                                       Asura_PS2_DmaTag::ID_CNT,
                                                       0,
                                                       false,
                                                       0,
                                                       SCE_VIF1_SET_MARK(uMarkerID | ASURA_PS2_PROFILE__STOP_MARK_MASK, false), // Record what we were profiling.
                                                       SCE_VIF1_SET_NOP(true) );                                            // Cause an interrupt.

    if( dbg_bVifProfilingOptimiseOutRedundantMarkers )
    {
        g_pxLastVifEnd       = pxOut;
        g_uMarkForLastVifEnd = uMarkerID;
    }
}

//////////////////////////////////////////////////////////////////////////////
// FUNCTION     Asura_PS2_ProfilingSystem::AddEEStart
// PARAMETERS   Marker identifier.
// DESCRIPTION  Records the start tiem of EE marker 'uMarker'
// AUTHOR       Pete Ellacott - 07.03.2003.
//////////////////////////////////////////////////////////////////////////////
void Asura_PS2_ProfilingSystem::AddEEStart( u_int uMarkerID )
{
    if( !dbg_bEnableEEProfiling ) { return; }

	// Records the start time of this EE marker
	Asura_Assert( uMarkerID < ASURA_PS2_PROFILE__MARKER_QTY, "" );
	asm __volatile__( "mfc0 %0, $9" : "=r" (g_auEEMarkerStartTimes[uMarkerID]) );
}

//////////////////////////////////////////////////////////////////////////////
// FUNCTION     Asura_PS2_ProfilingSystem::AddEEStart
// PARAMETERS   Marker identifier.
// DESCRIPTION  Records the end time of EE marker 'uMarker'
// AUTHOR       Pete Ellacott - 07.03.2003.
//////////////////////////////////////////////////////////////////////////////
void Asura_PS2_ProfilingSystem::AddEEEnd( u_int uMarkerID )
{
    if( !dbg_bEnableEEProfiling ) { return; }

	// We are stopping a timer
	Asura_Assert( uMarkerID < ASURA_PS2_PROFILE__MARKER_QTY, "" );
	u_int uTime;
	asm __volatile__( "mfc0 %0, $9" : "=r" (uTime) );
	if ( uTime < g_auEEMarkerStartTimes[uMarkerID] )
	{
		g_auEEMarkerTimes[uMarkerID] += uTime + (0xffffffff - g_auEEMarkerStartTimes[uMarkerID]);
	}
	else
	{
		g_auEEMarkerTimes[uMarkerID] += (uTime - g_auEEMarkerStartTimes[uMarkerID]);
	}
#if ASURA_PS2_SN_PROFILER
	snStopMarker(uMarkerID);
#endif
}

void Asura_PS2_ProfilingSystem::RegisterLabel(u_int uMarkerID, const char* szLabel)
{
#if ASURA_PS2_SN_PROFILER
	snStartMarker(uMarkerID, szLabel);
#endif
}

static bool g_bDrawProfileBars = true;

//////////////////////////////////////////////////////////////////////////////
// FUNCTION     Asura_PS2_ProfilingSystem::Render
// PARAMETERS   N/A
// DESCRIPTION  Renders the results of the frames timing
// AUTHOR       Pete Ellacott - 07.03.2003.
//////////////////////////////////////////////////////////////////////////////
void Asura_PS2_ProfilingSystem::Render()
{
    if( !g_bDrawProfileBars ) { return; }

	// Render the profile bar
    Asura_PS2_QW* const pxBase = Asura_PS2_SPR_Alloc( 0, 1024 );	// We don't know how much we'll need yet
	Asura_PS2_QW* pxOut = pxBase;

	// Create a DMA tag - fill in actual dma tag at the end when we know how much we wrote
//	pxOut->u32s[0] = PY_DMA_END | uNLoop+1;
	pxOut->u32s[1] = 0;
	pxOut->u32s[2] = SCE_VIF1_SET_NOP(0);
//	pxOut->u32s[3] = SCE_VIF1_SET_DIRECT( uNLoop+1, 0 );
	pxOut++;

	// Leave space for the gif tag - again filled in when we know howmuch data we wrote
	pxOut->u64s[1] = SCE_GIF_PACKED_AD;
	pxOut++;

    // Setup Test for Z always passes and Alpha always fails and only updates the framebuffer (to disable z writes)
    pxOut->u64s[0] = SCE_GS_SET_TEST_1(	1,					// ATE   = ON
										0,					// ATST  = ALL FAIL
										0,					// AREF  = N/A
										1,					// AFAIL = Frame buffer only (no z writes)
										0,					// DATE  = OFF
										0,					// DATM  = N/A
										1,					// ZTE   = ON
										SCE_GS_ZALWAYS );	// ZTST  = ALWAYS
    pxOut->u64s[1] = SCE_GS_TEST_1;
    pxOut++;

	// Set blend equation
	pxOut->u64s[0] = SCE_GS_SET_ALPHA( 0, 1, 0, 1, 0 );
	pxOut->u64s[1] = SCE_GS_ALPHA_1;
	pxOut++;

	// Calculate frame scales ect
	const int	iXOFF = (ASURA_PS2_SCREEN_CENTRE_X - (Asura_PS2_GS::GetScreenWidth()>>1))<<4;
	const int	iYOFF = (ASURA_PS2_SCREEN_CENTRE_Y<<4) - (Asura_PS2_GS::GetScreenHeight()<<3);
	const float PROFILE_BAR_WIDTH		= 400.0f;
	const float PROFILE_BAR_X			= (Asura_PS2_GS::GetScreenWidth() - PROFILE_BAR_WIDTH) * 0.5f;
	const int	PROFILE_BAR_TOP			= iYOFF + (10<<4);
	const int	PROFILE_BAR_VIF_TOP		= PROFILE_BAR_TOP+(2<<4);
	const int	PROFILE_BAR_VIF_BOTTOM	= PROFILE_BAR_VIF_TOP+(4<<4);
	const int	PROFILE_BAR_EE_TOP		= PROFILE_BAR_VIF_BOTTOM+(2<<4);
	const int	PROFILE_BAR_EE_BOTTOM	= PROFILE_BAR_EE_TOP+(4<<4);
	const int	PROFILE_BAR_BOTTOM		= PROFILE_BAR_EE_BOTTOM+(2<<4);
	
	const float fCyclesPerFrame = ASURA_PS2_EE_CYCLES_PER_SEC / Asura_PS2_GS::GetRefreshRate();;
	
	// How many frames did it take us to draw everything
	const float fFrames	  = ((float)g_auVIFMarkerTimes[0])/fCyclesPerFrame;
	const float fEEFrames = ((float)g_auEEMarkerTimes[0])/fCyclesPerFrame;
	int iFramesToDraw = 1 + ((fFrames>fEEFrames) ? (int)(fFrames) : (int)(fEEFrames));

	// Frame smoothing
	if ( iFramesToDraw > g_aiFrameSmoothing[g_uCurrentFrame>>FRAME_SMOOTHING] )
	{
		g_aiFrameSmoothing[g_uCurrentFrame>>FRAME_SMOOTHING] = iFramesToDraw;
	}
	for ( u_int u = 0; u<2; u++ )
	{
		if ( g_aiFrameSmoothing[u] > iFramesToDraw )	iFramesToDraw = g_aiFrameSmoothing[u];
	}
	g_uCurrentFrame++;
	g_uCurrentFrame &= ~(1<<(FRAME_SMOOTHING+1));
	if ((g_uCurrentFrame&~(1<<FRAME_SMOOTHING))==0)
	{	// Reset frame smoothing
		g_aiFrameSmoothing[g_uCurrentFrame>>FRAME_SMOOTHING] = 0;
	}

	// Lock frame scale to even numbers of frames
//	iFramesToDraw = (iFramesToDraw+1)&(~1);

	// Limit the number of frames we draw - due to the size of the scratch pad...
	if ( iFramesToDraw > 10 )	iFramesToDraw = 10;

	// Calculate the resolution we are drawing at
	const float fTotalCyclesToDraw = iFramesToDraw * fCyclesPerFrame;

	// Draw the frames
    int iX		 = iXOFF + (int)(PROFILE_BAR_X*16.0f);
	int iXStep	 =		   (int)((PROFILE_BAR_WIDTH*16.0f)/iFramesToDraw);

	for ( int i = 0; i<iFramesToDraw; i++ )
	{
		// Set the colour
		if( i & 1 )
		{
            pxOut->u64s[0] = SCE_GS_SET_RGBAQ( 128, 128, 128, 96, 0x3F800000 );
        }
        else
        {
            pxOut->u64s[0] = SCE_GS_SET_RGBAQ( 96, 96, 96, 96, 0x3F800000 );
        }

		pxOut->u64s[1] = SCE_GS_RGBAQ;
		pxOut++;

	    // Draw a frame
        pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_TOP, 0 );
        pxOut->u64s[1] = SCE_GS_XYZ2;
        pxOut++;

		iX += iXStep;

        pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_BOTTOM, 0 );
		pxOut->u64s[1] = SCE_GS_XYZ2;
        pxOut++;
    }

    // Frame markers.
    {
        pxOut->u64s[0] = SCE_GS_SET_RGBAQ( 255, 255, 255, 128, 0x3F800000 );
        pxOut->u64s[1] = SCE_GS_RGBAQ;
        ++pxOut;

        iX = iXOFF + (int)(PROFILE_BAR_X*16.0f);
        for( int i = 0; i < iFramesToDraw; ++i )
        {
            iX += iXStep;

            pxOut->u64s[0] = SCE_GS_SET_XYZ( iX - (1 << 4), PROFILE_BAR_TOP - (1 << 4), 0 );
            pxOut->u64s[1] = SCE_GS_XYZ2;
            ++pxOut;
            pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_TOP, 0 );
            pxOut->u64s[1] = SCE_GS_XYZ2;
            ++pxOut;
        }
    }

	// Draw the timers
    iX = iXOFF + (int)(PROFILE_BAR_X*16.0f);
	for ( u_int u = 1; u<ASURA_PS2_PROFILE__MARKER_QTY; u++ )
	{
		if( !g_auVIFMarkerTimes[u] )
		{
			// unused timer
			continue;
		}

		int iXStep	 = (int)((PROFILE_BAR_WIDTH * g_auVIFMarkerTimes[u] * 16.0f) / fTotalCyclesToDraw);
	
		const u_int uPCcolour = g_auMarkerColours[u & ( MAX_NUM_MARKER_COLOURS - 1 ) ];
		// JAT 13/07/06 swap around to PS2 format from PC format 
		pxOut->u64s[0] = SCE_GS_SET_RGBAQ( ((uPCcolour>> 16)&0xff), ((uPCcolour>> 8)&0xff), ((uPCcolour>>0)&0xff), 128, 0x3F800000 );
		pxOut->u64s[1] = SCE_GS_RGBAQ;
		pxOut++;
        
		pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_VIF_TOP, 0 );
        pxOut->u64s[1] = SCE_GS_XYZ2;
        pxOut++;

		iX += iXStep;
        
		pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_VIF_BOTTOM, 0 );
		pxOut->u64s[1] = SCE_GS_XYZ2;
        pxOut++;
    }

	// Draw the remaining VIF frame time we used
	pxOut->u64s[0] = SCE_GS_SET_RGBAQ( 64, 64, 0, 128, 0x3F800000 );
	pxOut->u64s[1] = SCE_GS_RGBAQ;
	pxOut++;

    pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_VIF_TOP, 0 );
	pxOut->u64s[1] = SCE_GS_XYZ2;
	pxOut++;

	iX = iXOFF + (int)((PROFILE_BAR_X*16.0f) + ((PROFILE_BAR_WIDTH * g_auVIFMarkerTimes[0] * 16.0f)/fTotalCyclesToDraw));

	pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_VIF_BOTTOM, 0 );
	pxOut->u64s[1] = SCE_GS_XYZ2;
	pxOut++;

	// Draw the EE timers
    iX = iXOFF + (int)(PROFILE_BAR_X*16.0f);
	for ( u_int u = 1; u<ASURA_PS2_PROFILE__MARKER_QTY; u++ )
	{
		if( !g_auEEMarkerTimes[u] )
		{
			// unused timer
			continue;
		}

		int iXStep	 = (int)((PROFILE_BAR_WIDTH * g_auEEMarkerTimes[u] * 16.0f) / fTotalCyclesToDraw);
	
		const u_int uPCcolour = g_auMarkerColours[u & ( MAX_NUM_MARKER_COLOURS - 1 ) ];
		// JAT 13/07/06 swap around to PS2 format from PC format 
		pxOut->u64s[0] = SCE_GS_SET_RGBAQ( ((uPCcolour>> 16)&0xff), ((uPCcolour>> 8)&0xff), ((uPCcolour>>0)&0xff), 128, 0x3F800000 );
		pxOut->u64s[1] = SCE_GS_RGBAQ;
		pxOut++;
        
		pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_EE_TOP, 0 );
        pxOut->u64s[1] = SCE_GS_XYZ2;
        pxOut++;

		iX += iXStep;
        
		pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_EE_BOTTOM, 0 );
		pxOut->u64s[1] = SCE_GS_XYZ2;
        pxOut++;
    }

	// Draw the remaining EE frame time we used
	pxOut->u64s[0] = SCE_GS_SET_RGBAQ( 64, 64, 0, 128, 0x3F800000 );
	pxOut->u64s[1] = SCE_GS_RGBAQ;
	pxOut++;

    pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_EE_TOP, 0 );
	pxOut->u64s[1] = SCE_GS_XYZ2;
	pxOut++;

	iX = iXOFF + (int)((PROFILE_BAR_X*16.0f) + ((PROFILE_BAR_WIDTH * g_auEEMarkerTimes[0] * 16.0f)/fTotalCyclesToDraw));

	pxOut->u64s[0] = SCE_GS_SET_XYZ( iX, PROFILE_BAR_EE_BOTTOM, 0 );
	pxOut->u64s[1] = SCE_GS_XYZ2;
	pxOut++;

	// Restore pixel test to : disable alpha test and enable depth test
    pxOut->u64s[0] = SCE_GS_SET_TEST_1(	0,					// ATE   = ON
										0,					// ATST  = ALL FAIL
										0,					// AREF  = N/A
										0,					// AFAIL = Frame buffer only (no z writes)
										0,					// DATE  = OFF
										0,					// DATM  = N/A
										1,					// ZTE   = ON
										SCE_GS_ZGEQUAL );	// ZTST  = ALWAYS
    pxOut->u64s[1] = SCE_GS_TEST_1;
    ++pxOut;

    // Calculate how big the dma packet is and update the dma tag
	const u_int uDmaPacketSize = (pxOut - pxBase);
	pxBase[0].u32s[0] = PY_DMA_END | (uDmaPacketSize-1);
	pxBase[0].u32s[3] = SCE_VIF1_SET_DIRECT( (uDmaPacketSize-1), 0 );

	// And the gif tag
	pxBase[1].u64s[0] = SCE_GIF_SET_TAG( uDmaPacketSize-2,		// NLOOP - Sneaked camera count in here !!!!!!
										 1,						// EOP (End Of Packet) - Added by microcode
										 1,						// PRE (PRIM field Enable)
										 SCE_GS_SET_PRIM( SCE_GS_PRIM_SPRITE,		// PRIM Type
														  1,				// IIP (Flat/Gouraud Shading)
														  0,				// TME (Texture Mapping Enabled)
														  0,				// FGE (Fogging Enabled)
														  1,				// ABE (Alpha Blending Enabled)
														  0,				// AA1 (1 Pass Anti-Aliasing Enabled)
														  0,				// FST (Method of specifying Texture Coords)
														  0,				// CTXT (Context)
														  0 ),				// FIX (Fragment Value Control)
										 SCE_GIF_PACKED,			// FLG (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
										 1 );						// NREG (Number of registers)

	// Make sure nothing is drawing at the moment
    Asura_PS2_BetterGsSyncPath( 0, 0 );

	// Fire of the packet
	DPUT_D1_QWC(0);
	// source address
	DPUT_D1_TADR((u_int)pxBase | ASURA_PS2_DMA_SPR_FLAG);   // when transferring from the scratchpad, set the top bit to 1.
	// for polling of completion
	DPUT_D_PCR(D_PCR_CPC1_M);
	DPUT_D_STAT(D_STAT_CIS1_M);

	asm __volatile__("sync.l");

	// start actual kick
	DPUT_D1_CHCR(1  // from memory
		| (1 << 2)  // chain mode
		| (1 << 6)  // transfer tag on
		| (1 << 8));    // dma start

	asm __volatile__("sync.l");
	asm __volatile__("sync.p");

	// Wait for completion.
	asm __volatile__( " _AsuraPS2Profiling_Render_WaitForDMA:         " );
	asm __volatile__( "     bc0f _AsuraPS2Profiling_Render_WaitForDMA " );
	asm __volatile__( "     nop                               " );

	Asura_PS2_SPR_Dealloc( pxBase );

	// Reset all our timers ready for next frame
	for ( u_int u = 0; u<ASURA_PS2_PROFILE__MARKER_QTY; u++ )
	{
		g_auVIFMarkerTimes[u] = 0;
		g_auEEMarkerTimes[u]  = 0;
	
		g_auVIFMarkerStartTimes[u] = 0;
		g_auEEMarkerStartTimes[u] = 0;
	}
}

//////////////////////////////////////////////////////////////////////////////
// FUNCTION     Asura_PS2_ProfilingSystem::VIF1_InteruptHandler
// PARAMETERS   int cause = the cause of the interrupt
// DESCRIPTION  Handles the interrupt generated by the Timing markers we added 
//				to the draw list
// AUTHOR       Pete Ellacott - 07.03.2003.
//////////////////////////////////////////////////////////////////////////////
int Asura_PS2_ProfilingSystem::VIF1_InteruptHandler( int cause )
{
	// Check if the last buffer just finished drawing
	if ( cause == INTC_VIF1) 
	{
		// Make sure this interrupt is caused by an interrupt and not some error
		u_int status = DGET_VIF1_STAT();
		if ( status & 0x40 )
		{
			// Read the VIF1_MARK Value so we can decide which marker we have
			const u_int uMark = DGET_VIF1_MARK();

			if( uMark & ASURA_PS2_PROFILE__START_MARK_MASK )
			{
				// We are starting a timer
				const u_int uMarker = uMark & (~ASURA_PS2_PROFILE__START_MARK_MASK);
				asm __volatile__( "mfc0 %0, $9" : "=r" (g_auVIFMarkerStartTimes[uMarker]) );
			}
			else if( uMark & ASURA_PS2_PROFILE__STOP_MARK_MASK )
			{
				// We are stopping a timer
				const u_int uMarker = uMark & (~ASURA_PS2_PROFILE__STOP_MARK_MASK);

                u_int uTime;
				asm __volatile__( "mfc0 %0, $9" : "=r" (uTime) );
				if ( uTime < g_auVIFMarkerStartTimes[uMarker] )
				{
                    // Wrapped.
					g_auVIFMarkerTimes[uMarker] += uTime + (0xffffffff - g_auVIFMarkerStartTimes[uMarker]);
				}
				else
				{
					g_auVIFMarkerTimes[uMarker] += (uTime - g_auVIFMarkerStartTimes[uMarker]);
				}
			}
	    }
	}

	// Restart VIF 1
	DPUT_VIF1_FBRST( VIF1_FBRST_STC_M );

	// We're not greedy, let the other handlers have thier say too
	asm volatile("sync.l; ei; sync.p");
	return 1;
}
#endif // #if ASURA_PS2_DRAW_PROFILE_BARS
