//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_2D_Graphics.h
//
// Copyright (c) 2002-2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_2D_GRAPHICS_H___
#define ___ASURA_PS2_2D_GRAPHICS_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Rendering/Asura_Render.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

static const int ASURA_PS2_MAX_SPRITES_IN_BATCH       = 64;
static const int ASURA_PS2_MAX_ROTATED_QUADS_IN_BATCH = 4;
static const int ASURA_PS2_MAX_GOURAUD_QUADS_IN_BATCH = 8;
static const int ASURA_PS2_MAX_ZSPRITES_IN_BATCH      = 32;
static const int ASURA_PS2_MAX_ZQUADS_IN_BATCH        = 4;

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

union Asura_PS2_QW;

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PS2_2D_Graphics
{

public:
    static void Initialise();
    static void Shutdown();
    static void AddQuad( const class Asura_Overlay_Quad* quad );

    static void Flush();

    // quad batch is a collection of sprites (for HUD type stuff) with the 
    // same rendering attributes (texture and blend mode)
    struct BatchSprite
    {
        int iTextureIndex;
        ASURA_TRANSLUCENCY_MODE eTranslucencyMode;

        float X[2];
        float Y[2];
        float U[2];
        float V[2];

        u_int uARGB;
    };

    struct RotatedBatchQuad
    {
        int iTextureIndex;
        ASURA_TRANSLUCENCY_MODE eTranslucencyMode;

        float X[4];
        float Y[4];
        float U[4];
        float V[4];

        u_int uARGB;
    };

    struct GouraudBatchQuad
    {
        int iTextureIndex;
        ASURA_TRANSLUCENCY_MODE eTranslucencyMode;

        float X[4];
        float Y[4];
        float U[4];
        float V[4];

        u_int auARGBs[4];
    };
    
    struct ZSortedBatchSprite
    {
        int  iTextureIndex;
        ASURA_TRANSLUCENCY_MODE eTranslucencyMode;

        float X[2];
        float Y[2];
        float U[2];
        float V[2];
        float fViewspaceZ;

        u_int uARGB;
    };

    struct ZSortedBatchQuad
    {
        int  iTextureIndex;
        ASURA_TRANSLUCENCY_MODE eTranslucencyMode;

        float X[4];
        float Y[4];
        float U[4];
        float V[4];
        float fViewspaceZ;

        u_int uARGB;
    };

    struct QuadBatch
    {
        int iTexID;
        ASURA_TRANSLUCENCY_MODE eBlendMode;
        int iNumSpritesInBatch;
        int iNumRotatedQuadsInBatch;
        int iNumGouraudQuadsInBatch;
        int iNumZSpritesInBatch;
        int iNumZQuadsInBatch;
        BatchSprite         a_xSprites[ASURA_PS2_MAX_SPRITES_IN_BATCH];
        RotatedBatchQuad    a_xRotatedQuads[ASURA_PS2_MAX_ROTATED_QUADS_IN_BATCH];
        GouraudBatchQuad    a_xGouraudQuads[ASURA_PS2_MAX_GOURAUD_QUADS_IN_BATCH];
        ZSortedBatchSprite  a_xZSprites[ASURA_PS2_MAX_ZSPRITES_IN_BATCH];
        ZSortedBatchQuad    a_xZQuads[ASURA_PS2_MAX_ZQUADS_IN_BATCH];
    };
    
#ifdef ASURA_PS2_MAX_2D_LINES_PER_FRAME
    struct Line2D
    {
        float afX[2];
        float afY[2];
        u_int uARGB;
    };
#endif

    // NRM 22-08-02 added these platform dependent functions to add very specific quads to the batches
    // Had to make the structs for the sprites and quads public though
    static void AddSprite       (const BatchSprite* pxSprite);
    static void AddZSprite      (const ZSortedBatchSprite* pxZSprite);
    static void AddRotatedQuad  (const RotatedBatchQuad* pxRotQuad);
    static void AddZQuad        (const ZSortedBatchQuad* pxZQuad);
    static void AddGouraudQuad  (const GouraudBatchQuad* pxGouraudQuad);

    // Set of helper functions for writing out the draw lists for all the
    // sprite / quad types.
    static void WriteRGBAQs( const u_int* puARGBs, u_int uNumARGBs,
                             Asura_PS2_QW* pxOut, u_int uStride,
                             bool bTextured );
    static void SetupClampAndUVs( const float* pfSs, const float* pfTs, u_int uNumCoordinateSets,
                                  float fTexWidth, float fTexHeight,
                                  u_int* puUVOut, u_int uUVStride,
                                  Asura_PS2_QW* pxCLAMPOut );
    static void WriteXYZs( const float* pfXs, const float* pfYs, float fZ, u_int uNumCoordinateSets,
                           int iTransX, int iTransY,
                           Asura_PS2_QW* pxOut, u_int uStride,
                           bool bScaleForWidescreen = true );
    
    // For use by Asura_PS2_GS at the end of a frame.
    static void CheckForTexDisplayRequest( void );

#ifdef ASURA_PS2_MAX_2D_LINES_PER_FRAME
    static Line2D* GetFreeLine( void );
#endif
    
    static u_int GetNumQWsForSpriteStrips( u_int uWidth, bool bNeedUVs, u_int uStripWidth = 32, u_int uLog2StripWidth = 5, u_int uStride = 0 );
    static Asura_PS2_QW* WriteSpriteStrips( Asura_PS2_QW* pxOut,
                                            u_int uX, u_int uY, u_int uWidth, u_int uHeight,
                                            bool bNeedUVs, u_int uU0 = 0, u_int uV0 = 0, u_int uDU = 0, u_int uDV = 0, bool bCentreUVs = false,
                                            u_int uStripWidth = 32, u_int uLog2StripWidth = 5, u_int uStride = 0,
                                            u_int uSpriteZ = 0, u_int uUOffset = 0, u_int uVOffset = 0 );

    static float CentreCoordinateOnTexel( float fCoord, u_int uDimensionSize );
	static u_int GetNumQWsForSpriteStripsWithSTs( u_int uWidth, u_int uStripWidth = 32, u_int uLog2StripWidth = 5, u_int uStride = 0 );
	static Asura_PS2_QW* WriteSpriteStripsWithSTs( Asura_PS2_QW* pxOut,
												   u_int uX, u_int uY, u_int uWidth, u_int uHeight,
												   float fS, float fT, float fTextureW, float fTextureH, 
												   u_int uTextureWidthTexels, u_int uTextureHeightTexels,
												   bool bCentreSTs = true, u_int uStripWidth = 32, u_int uLog2StripWidth = 5, 
												   u_int uStride = 0, u_int uSpriteZ = 0);
private:

    // NRM 22-08-02 cut up the 2-d rendering to make it more modular (and easy to see what's going on!)
    // Add 2D graphics to current batches
    static bool AddSpriteToThisBatch(QuadBatch* pxBatch, const BatchSprite * pxSprite);
    static bool AddRotatedQuadToThisBatch(QuadBatch* pxBatch, const RotatedBatchQuad * pxRotQuad);
    static bool AddGouraudQuadToThisBatch(QuadBatch* pxBatch, const GouraudBatchQuad* pxGouraudQuadToAdd);
    static bool AddZSpriteToThisBatch(QuadBatch* pxBatch, const ZSortedBatchSprite * pxZSprite);
    static bool AddZQuadToThisBatch(QuadBatch* pxBatch, const ZSortedBatchQuad *pxZQuad);

    // Add 2D graphics to new 
    static void AddSpriteToNewBatch(const BatchSprite* pxSprite);
    static void AddRotatedQuadToNewBatch(const RotatedBatchQuad* pxRotQuad);
    static void AddGouraudQuadToNewBatch(const GouraudBatchQuad* pxGouraudQuad);
    static void AddZSpriteToNewBatch(const ZSortedBatchSprite * pxZSprite);
    static void AddZQuadToNewBatch(const ZSortedBatchQuad* pxZQuad);
    static void AddZTestPacket(Asura_PS2_QW*& pxOut, bool bZTest);

    static bool CreateNewBatch(QuadBatch*& pxNewBatch);
    static int  FindBatch( int iTexID, ASURA_TRANSLUCENCY_MODE eBlendMode, int iFreeType, QuadBatch*& pxBatch );
    static void SwapBatches( u_int uStartIndex, QuadBatch* pxBatch );

    // Functions to convert the 
    static void QuadToSprite     (const class Asura_Overlay_Quad * quad, struct BatchSprite* pxSpriteToAdd);
    static void QuadToRotatedQuad(const class Asura_Overlay_Quad * quad, struct RotatedBatchQuad* pxRotQuadToAdd);
    static void QuadToGouraudQuad(const class Asura_Overlay_Quad * quad, struct GouraudBatchQuad* pxGouraudQuadToAdd);

    static void DrawSprites( const QuadBatch* pBatch, Asura_PS2_QW*& pxVoidOut, int iTransX, int iTransY );
    static void DrawRotatedQuads( const QuadBatch* pBatch, Asura_PS2_QW*& pxVoidOut, int iTransX, int iTransY );
    static void DrawGouraudQuads( const QuadBatch* pBatch, Asura_PS2_QW*& pxVoidOut, int iTransX, int iTransY );
    static void DrawZSprites( const QuadBatch* pBatch, Asura_PS2_QW*& pxVoidOut, int iTransX, int iTransY );
    static void DrawZQuads( const QuadBatch* pBatch, Asura_PS2_QW*& pxVoidOut, int iTransX, int iTransY );
    static void CreateRenderStateForQuads( const QuadBatch* pBatch, Asura_PS2_QW*& pxOut );

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_2D_GRAPHICS_H___
