/////////////////////////////////////////////////////////////////////////////
//
// Extneded Particled Renderer.
//
//	Currently only a simple quad renderer with all the particle
//	Effects being done on the EE.  If EE time is too much, the
//	calculation can be moved to vu1
//
//	A post process stage of fogging is optionally carried out as required.
//
//			Memory layout
//		  -----------------
//
//    0    /---------------------------\ 
//         |		  FOG TABLE			|     256 QWs
//   256   |---------------------------|
//         |		ALPHA NORMAL		|     1 QWs
//   256   |---------------------------|
//         |		ALPHA NORMAL		|     1 QWs
//   257   |---------------------------|
//         |	    ALPHA GLOWING		|	  1 QWs
//   258   |---------------------------|
//         |		Drawing Giftag		|	  1 QWs 
//   259   |---------------------------|
//         |  World to Screen Matrix   |	  4 QWs
//   263   |---------------------------|
//         |   World to Clip Matrix    |	  4 QWs
//   267   |---------------------------|
//         |		    Unused			|	 35 QWs
//   302   |---------------------------|
//         |	    Input Buffer 1		|	100	QWs
//   402   |---------------------------|
//			|	    Input Buffer 2		|	100 QWs
//   502   |---------------------------|
//         |	   Output Buffer 1		|	261 QWs
//   763   |---------------------------|
//         |	   Output Buffer 2		|	261 QWs
//  1024   \---------------------------/ 
//
//
//	Output data (per particle) = ALPHA,ST,RGBA,XYZ,ST,RGBA,XYZ,ST,RGBA,XYZ,ST,RGBA,XYZ = 13
//	Output buffer size = 1 + NP * 13
//
//	Input data per particle = RGBA+XYZ+XDELTA+YDELTA+ST = 5
// Input buffer size = NP * 5
//
//	Free Space = 1024 - 267 = 757
//
//	757 = 2*(1 + 13NP + 5NP) = 2 + 30NP
// 755 = 36NP
// NP = 20 particles per buffer
//
// Input buffer size  = 5NP = 100 qws
//	Output buffer size = 1 + 13NP = 261 qws
//
/////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_EXTENDEDPARTICLEDEFINES_H__INCLUDED___
#define ___ASURA_PS2_EXTENDEDPARTICLEDEFINES_H__INCLUDED___

// -=- Maximum Quantities -=-
#define ASURA_PS2_EXPART__MAX_PARTICLES_PER_PACKET		(   20)
// -=- Masks -=-
#define ASURA_PS2_EXPART__MASK__ADDITIVE				( 0x08)
// -=- Data Addresses -=-
#define ASURA_PS2_EXPART__ADDR__FOGTABLE				(    0)
#define	ASURA_PS2_EXPART__ADDR__ALPHANORMAL				(  256)
#define	ASURA_PS2_EXPART__ADDR__ALPHAGLOWING			(  257)
#define	ASURA_PS2_EXPART__ADDR__GIFTAG					(  258)
#define	ASURA_PS2_EXPART__ADDR__WORLD2SCREEN			(  259)
#define	ASURA_PS2_EXPART__ADDR__WORLD2CLIP				(  263)
#define ASURA_PS2_EXPART__ADDR__INPUT_BUFFER1			(  302)
#define ASURA_PS2_EXPART__ADDR__INPUT_BUFFER2			(  402)
#define ASURA_PS2_EXPART__ADDR__OUTPUT_BUFFER1			(  502)
#define ASURA_PS2_EXPART__ADDR__OUTPUT_BUFFER2			(  763)
// Data area sizes
#define ASURA_PS2_EXPART__SIZE__INPUT_BUFFER			( ASURA_PS2_EXPART__ADDR__INPUT_BUFFER2 - ASURA_PS2_EXPART__ADDR__INPUT_BUFFER1 )
#define ASURA_PS2_EXPART__SIZE__OUTPUT_BUFFER			( ASURA_PS2_EXPART__ADDR__OUTPUT_BUFFER2 - ASURA_PS2_EXPART__ADDR__OUTPUT_BUFFER1 )

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_EXTENDEDPARTICLEDEFINES_H__INCLUDED___
