/////////////////////////////////////////////////////////////////////////////////////////
//
//	FILE		Asura_PS2_GS.H
//
//	CREATOR		Kevin Lea (21 March 2001)
//
/////////////////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef	___ASURA_PS2_GS_H___
#define	___ASURA_PS2_GS_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

#include <libgraph.h>                       // For sceGsDBuff struct.

#include "Camera/Asura_Camera.h"					// for max cameras
#include "Debugging/Asura_Assert.h"

// M A C R O S ///////////////////////////////////////////////////////////////

#define ASURA_PS2_FRAMEBFR_WIDTH         ( 4096 )
#define ASURA_PS2_FRAMEBFR_HEIGHT        ( 4096 )

#define ASURA_PS2_NUM_SCR_PIXELS         ( Asura_PS2_GS::GetScreenWidth() * Asura_PS2_GS::GetScreenHeight() )

// Centre co-ordinates of screen in framebuffer.
#define ASURA_PS2_SCREEN_CENTRE_X        ( ASURA_PS2_FRAMEBFR_WIDTH >> 1 )
#define ASURA_PS2_SCREEN_CENTRE_Y        ( ASURA_PS2_FRAMEBFR_HEIGHT >> 1 )

// Buffer byte depths.
#define ASURA_PS2_DISPBFR_BYTE_DEPTH     ( Asura_PS2_GS::GetDispBfrBitDepth() >> 3 )
#define ASURA_PS2_DRAWBFR_BYTE_DEPTH     ( Asura_PS2_GS::GetDrawBfrBitDepth() >> 3 )
#define ASURA_PS2_ZBFR_BYTE_DEPTH        ( Asura_PS2_GS::GetZBfrBitDepth()    >> 3 )

// Buffer sizes in words.
#define ASURA_PS2_DISPBFR_NUM_WORDS      ( Asura_PS2_GS::GetDispBfrBitDepth() == 24 ? ASURA_PS2_NUM_SCR_PIXELS : ((ASURA_PS2_DISPBFR_BYTE_DEPTH * ASURA_PS2_NUM_SCR_PIXELS) >> 2) )
#define ASURA_PS2_DRAWBFR_NUM_WORDS      ( Asura_PS2_GS::GetDrawBfrBitDepth() == 24 ? ASURA_PS2_NUM_SCR_PIXELS : ((ASURA_PS2_DRAWBFR_BYTE_DEPTH * ASURA_PS2_NUM_SCR_PIXELS) >> 2) )
#define ASURA_PS2_ZBFR_NUM_WORDS         ( Asura_PS2_GS::GetZBfrBitDepth()    == 24 ? ASURA_PS2_NUM_SCR_PIXELS : ((ASURA_PS2_ZBFR_BYTE_DEPTH    * ASURA_PS2_NUM_SCR_PIXELS) >> 2) )

// Buffer sizes in units of 2048 words.
#define ASURA_PS2_DISPBFR_SIZE_2KB       ( (ASURA_PS2_DISPBFR_NUM_WORDS + 2047) >> 11 )
#define ASURA_PS2_DRAWBFR_SIZE_2KB       ( (ASURA_PS2_DRAWBFR_NUM_WORDS + 2047) >> 11 )
#define ASURA_PS2_ZBFR_SIZE_2KB          ( (ASURA_PS2_ZBFR_NUM_WORDS    + 2047) >> 11 )

// Base pointers as word address / 2048.
#define ASURA_PS2_FRAMEBFR_0_BASE_PTR    ( 0 )                                                          // Disp buffer if not flipping.
#define ASURA_PS2_FRAMEBFR_1_BASE_PTR    ( ASURA_PS2_FRAMEBFR_0_BASE_PTR + ASURA_PS2_DISPBFR_SIZE_2KB ) // Draw          "            .
#define ASURA_PS2_ZBFR_DEFAULT_BASE_PTR  ( ASURA_PS2_FRAMEBFR_1_BASE_PTR + ASURA_PS2_DRAWBFR_SIZE_2KB )

// Texture buffer base pointer and size.
#define ASURA_PS2_TEXBFR_NUM_KB          ( 1024 )
#define ASURA_PS2_TEXBFR_BASE_PTR        ( (4096 - ASURA_PS2_TEXBFR_NUM_KB) << 2 )
#define ASURA_PS2_TEXBFR_NUM_WORDS       ( ASURA_PS2_TEXBFR_NUM_KB << 8 )
#define ASURA_PS2_TEXBFR_NUM_64W_UNITS   ( ASURA_PS2_TEXBFR_NUM_WORDS >> 6 )

// T Y P E S /////////////////////////////////////////////////////////////////

enum Asura_PS2_VideoMode
{
	ASURA_PS2_VIDEOMODE_USEDEFAULT = -1,
	ASURA_PS2_VIDEOMODE_PAL,
	ASURA_PS2_VIDEOMODE_NTSC,
	ASURA_PS2_VIDEOMODE_PROGRESSIVESCAN,
};

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

union Asura_PS2_QW;

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PS2_GS
{

    public:

        // T Y P E S /////////////////////////////////////////////////////////

        struct ViewportParams
        {
            short sOriginX, sOriginY;       // Pixel offset from screen origin.
            short sWidth, sHeight;
            short sPrimTransX, sPrimTransY; // In GS 12:4 fixed point units.
        };

        // C O N S T A N T S /////////////////////////////////////////////////

        enum
        {
            NUM_QWS_FOR_VIEWPORT_SETUP             = 5, // Buffer size required for SetupViewport().
            NUM_QWS_FOR_VIEWPORT_SETUP_PER_CONTEXT = 2, // Buffer size required for GetViewportSetupGifCmds(), per context.
        };

        enum RenderContext
        {
            CONTEXT_1,
            CONTEXT_2,
            CONTEXTS_1_AND_2,
        };

        // C L A S S   M E T H O D S /////////////////////////////////////////

	    static void Initialise( void );
	    static void Shutdown( void );

	    static void SwapScreenBuffers( bool bPutDrawEnv );

#ifndef ASURA_PS2_SONY_SUBMISSION
        // Debug/dev bits.
        static void EnableDrawToFrontBfr( bool bOn );
		static void ShowAlphaChannel( void );
        static void ShowZBuffer( void );
#endif

        static void SetClearDrawBufferOnSwap(bool bEnable);
        static bool GetClearDrawBufferOnSwap(void) { return s_bClearDrawBufferOnSwap; }
		static void SetClearZBufferOnSwap(bool bEnable);
        static bool GetClearZBufferOnSwap(void) { return s_bClearZBufferOnSwap; }
		static void SetClearDrawBufferColour( u_int uColour )	{ s_uClearColour = uColour;	}

	    static void BeginFrame(void);
	    static void EndFrame(void);
        static void SetEndFrameCallback( void(*pfnCallback)(void) );

        static inline u_int GetScreenWidth( void )     { return s_sScreenWidth;      }
        static inline u_int GetScreenHeight( void )    { return s_sScreenHeight;     }
        static inline u_int GetDispBfrBitDepth( void ) { return s_usDispBfrBitDepth; }

        static inline u_int GetDrawBfrBitDepth( void ) { return s_usDrawBfrBitDepth; }
        static void SetDrawBfrBitDepth( u_short usNewBitDepth );

        static inline u_short GetZBfrBitDepth( void ) { return s_usZBfrBitDepth; }
        static void SetZBfrBitDepth( u_short usNewBitDepth );

        static inline u_short GetZBfrBasePtr( void ) { return s_usZBfrBasePtr; }
        static void SetZBfrBasePtr( u_short usNewZBP );
        
		static void ClearVRAM( void );
		static void ClearVRAMColour( const u_int uABGR );
		static void ClearZBuffer( void );

        static void FillDrawBuffer( u_int uABGR = 0, bool bClearZBuffer = true );

        // multiple viewports
		static void AllowFullScreenRendering(void);
        static bool IsFullScreenRendering() { return s_bFullScreenRendering; }
		static void SetupViewport( int iCamID );
        static Asura_PS2_QW* GetViewportSetupGifCmds( int iCamID, RenderContext eContext, Asura_PS2_QW* pxOut );
        static inline int GetViewportWidth( u_int uCam ) { return s_axViewportParams[uCam].sWidth; }
        static inline int GetViewportHeight( u_int uCam ) { return s_axViewportParams[uCam].sHeight; }
        static const ViewportParams& GetViewportParams( u_int uCam ) { return s_axViewportParams[uCam]; }
        static const ViewportParams& GetViewportParamsForFullScreenRendering( void ) { return s_axViewportParams[ASURA_MAX_NO_OF_CAMERAS]; }

		static inline const sceGsDBuffDc& GetDBuffInfo( void ) { return s_xDoubleBuffer; }
        static void SetDBuffInfo( const sceGsDBuffDc& xNewDBuffInfo );
        static u_int GetCurDispBuff( void );
		static inline u_int GetCurDrawBuff( void )	{ return 1 - GetCurDispBuff(); }

		static inline u_int GetDispBfrFBP( void ) { return s_xDoubleBuffer.disp[GetCurDispBuff()].dispfb.FBP; }
		static inline u_int GetDrawBfrFBP( void ) { return s_xDoubleBuffer.disp[GetCurDrawBuff()].dispfb.FBP; }

        static u_short GetDispBfrPSM( void );
        static u_short GetDrawBfrPSM( void );
        static u_short GetZBfrPSM( void ) { return TranslateZBfrBitDepthToPSM( s_usZBfrBitDepth ); }
        static u_short TranslateZBfrBitDepthToPSM( u_short usBitDepth );

        static void SetMotionBlur(bool on_off);
        static void SetMotionBlurValue( u_char ucValue );
        
        static void SetFramerateTarget( float fTarget ); // Passing 0 disables.
		static float GetTargetFramerate() { return s_fTargetFrameRate; }

        static void SetEnableVSync(bool bEnable) { s_bEnableVsync = bEnable; }
        static bool GetEnableVSync() { return s_bEnableVsync; }

		static void SetRGBAFrameMask(unsigned int uFM);
		static unsigned int GetRGBAFrameMask()			{ return s_uRGBAFrameMask;	}

		// Special Slow clear screen independant of deferred rendering chains for use by AsuraAssert
		static void DebugClearFrame(  u_char ucR=0, u_char ucG=0, u_char ucB=16, u_char ucA=255 );

        // functions for use by screen adjust menus
        static void SetScreenOffsets(int iX, int iY);
        static void GetScreenOffsets(int& iX, int& iY);
        static void GetScreenOffsetRanges(int& iMinX, int& iMinY, int& iMaxX, int& iMaxY);
        static void GetScreenOffsetDefaults(int& iDefaultX, int& iDefaultY);

		//uCameras is a bitfield specifying which camera's to cache for
		//non-camera specific parameters will only be updated when all cameras are selected for update
        static void CacheViewportParams( u_int uCameras = UINT_MAX );

        static void ChangeVideoMode( Asura_PS2_VideoMode eVideoMode = ASURA_PS2_VIDEOMODE_USEDEFAULT, 
									 int iWidth = -1, int iHeight = -1,
                                     int iFrontBfrBitDepth = -1, bool bFrameMode = false );
        static void SetAllowDrawBufferToBeFixed( bool bAllow ) { s_bAllowDisplayBufferToBeFixed = bAllow; }
        static void SetAutoScreenBufferSwapFlag( bool bAuto = true ) { s_bAutoScreenBufferSwap = bAuto; }

		// function to test whether extra lines are being used for PAL display.
		// (MH - 23.07.2004) note that progressive scan is NTSC
		static bool VideoModeIsPal() { return s_eVideoMode == ASURA_PS2_VIDEOMODE_PAL; }
		static float GetRefreshRate() { return VideoModeIsPal() ? 50.0f : 60.0f; }
        static inline const char* GetMovieFormat( void ) { return( VideoModeIsPal() ? "pal" : "ntsc" ); }
		static Asura_PS2_VideoMode GetVideoMode() { return s_eVideoMode; }

        static void SetInitialiseDoubleBufferCallback( void(*pfnCallback)(sceGsDBuffDc*) );

    private:

        // C L A S S   M E T H O D S /////////////////////////////////////////

        // ie. are we blitting from the back buffer to the front buffer at
        // the end of the frame, rather than just switching between them?
        static bool IsDisplayBufferFixed( void ) { return( s_bAllowDisplayBufferToBeFixed && 16 == GetDispBfrBitDepth() && 32 == GetDrawBfrBitDepth() ); }

        static void DoMotionBlurOrBackToFrontBlit( bool bBlitOnly );

        static void InitialiseDoubleBuffer();
        static void PutDispEnvExplicit();

        static Asura_PS2_VideoMode Project_GetDefaultVideoMode( void );
        static void Project_GetDefaultScreenParamsForVideoMode( Asura_PS2_VideoMode eVideoMode, u_int* puWidth, u_int* puHeight, u_int* puFrontBfrBitDepth, u_int* puDrawBfrBitDepth );

        // C L A S S   S T A T E /////////////////////////////////////////////

	    static short s_sScreenHeight;
	    static short s_sScreenWidth;

	    static u_short s_usDispBfrBitDepth;
	    static u_short s_usDrawBfrBitDepth;
	    static u_short s_usZBfrBitDepth;
        static u_short s_usZBfrBasePtr;     // ZBP, ie. word address / 2048

        static bool s_bFullScreenRendering;
        static ViewportParams s_axViewportParams[ASURA_MAX_NO_OF_CAMERAS + 1];

	    static sceGsDBuffDc s_xDoubleBuffer;
        static unsigned s_uCurDispBuffer;
        static bool s_bAllowDisplayBufferToBeFixed;
        static bool s_bAutoScreenBufferSwap;

        static float s_fTargetFrameRate;

        static bool  s_bClearDrawBufferOnSwap;
		static bool  s_bClearZBufferOnSwap;
		static u_int s_uClearColour;

        static bool s_bEnableVsync;

		static unsigned int s_uRGBAFrameMask;

		static Asura_PS2_VideoMode s_eVideoMode;
};

struct Asura_PS2_GifTag
{

    // C O N S T A N T S /////////////////////////////////////////////////////

    enum
    {
        MAX_NLOOP = 32767,
    };

    // F U N C T I O N S /////////////////////////////////////////////////////

    inline void SetExceptRegs( u_int uNLOOP, bool bEOP, bool bPRE, u_int uPRIM, u_int uFLG, u_int uNREG )
    {
        uTagPart = SCE_GIF_SET_TAG( uNLOOP, bEOP, bPRE, uPRIM, uFLG, uNREG );
    }

    inline void Set( u_int uNLOOP, bool bEOP, bool bPRE, u_int uPRIM, u_int uFLG, u_int uNREG )
    {
        SetExceptRegs( uNLOOP, bEOP, bPRE, uPRIM, uFLG, uNREG );
        uAllRegs = 0;
    }

    inline void Set( u_int uNLOOP, 
                     bool  bEOP,
                     bool  bPRE,
                     u_int uPRIM,
                     u_int uFLG,
                     u_int uNREG,
                     u_int uReg0,
                     u_int uReg1  = 0,
                     u_int uReg2  = 0,
                     u_int uReg3  = 0,
                     u_int uReg4  = 0,
                     u_int uReg5  = 0,
                     u_int uReg6  = 0,
                     u_int uReg7  = 0,
                     u_int uReg8  = 0,
                     u_int uReg9  = 0,
                     u_int uReg10 = 0,
                     u_int uReg11 = 0,
                     u_int uReg12 = 0,
                     u_int uReg13 = 0,
                     u_int uReg14 = 0,
                     u_int uReg15 = 0 )
    {
        SetExceptRegs( uNLOOP, bEOP, bPRE, uPRIM, uFLG, uNREG );

        uAllRegs = (static_cast<u_long>(uReg0 ) <<  0)
                 | (static_cast<u_long>(uReg1 ) <<  4)
                 | (static_cast<u_long>(uReg2 ) <<  8)
                 | (static_cast<u_long>(uReg3 ) << 12)
                 | (static_cast<u_long>(uReg4 ) << 16)
                 | (static_cast<u_long>(uReg5 ) << 20)
                 | (static_cast<u_long>(uReg6 ) << 24)
                 | (static_cast<u_long>(uReg7 ) << 28)
                 | (static_cast<u_long>(uReg8 ) << 32)
                 | (static_cast<u_long>(uReg9 ) << 36)
                 | (static_cast<u_long>(uReg10) << 40)
                 | (static_cast<u_long>(uReg11) << 44)
                 | (static_cast<u_long>(uReg12) << 48)
                 | (static_cast<u_long>(uReg13) << 52)
                 | (static_cast<u_long>(uReg14) << 56)
                 | (static_cast<u_long>(uReg15) << 60);
    }

    // D A T A ///////////////////////////////////////////////////////////////

    union
    {
        struct
        {
            u_long uNLOOP   : 15,
                   bEOP     :  1,
	               _padding : 30,
                   bPRE     :  1,
                   uPRIM    : 11,
                   uFLG     :  2,
                   uNREG    :  4;
	        u_long uReg0    :  4,
	               uReg1    :  4,
	               uReg2    :  4,
	               uReg3    :  4,
	               uReg4    :  4,
	               uReg5    :  4,
	               uReg6    :  4,
	               uReg7    :  4,
	               uReg8    :  4,
	               uReg9    :  4,
	               uReg10   :  4,
	               uReg11   :  4,
	               uReg12   :  4,
	               uReg13   :  4,
	               uReg14   :  4,
	               uReg15   :  4;
        };

        u_long128 u128;

        u_long auDoubleWords[2];

        struct
        {
            u_long uTagPart;
            u_long uAllRegs;
        };
    };

};

///////////////
// FUNCTIONS //
///////////////

int Asura_PS2_BetterGsSyncPath( int iMode, unsigned short uTimeout );

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_GS_H___

