//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_Lighting.cpp
//
// Copyright (c) 2001-2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_PS2_Lighting.h"

// Engine independent.
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Camera/Asura_Camera.h"
#include "Debugging/Asura_Assert.h"
#include "Light/Asura_Light.h"
#include "Light/Asura_Photon.h"

// Engine platform-specific.
#include "Rendering/Asura_PlatformRenderCharacter.h"

// G L O B A L S /////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fAmbient = 0.1f;
/*DEBUGVAR*/ static bool  dbg_bDisableSmoothSkinSpecularPass = false;
sceVu0FVECTOR g_CharacterAmbientColour = {0.0f, 0.0f, 0.0f, 128.0f};//rgb of environment lighting value. 128 is a hack to get base alpha value of each vertex 
Asura_PS2_VU_Light g_CharacterTransformedLights[ASURA_PS2_MAX_LIGHTS_FOR_VU];
u_int g_CharacterNumLocalLights;

sceVu0FVECTOR g_CharacterSpecularLightPosition;
sceVu0FVECTOR g_CharacterSpecularColour;

// F U N C T I O N   I M P L E M E N T A T I O N S ///////////////////////////

void Asura_PS2_SetupCharacterLighting( const Asura_Animation* pxAnim, const Asura_Vector_3& xBBCentre, float* pfLightLevel )
{
	g_CharacterAmbientColour[0] = 1.0f;
	g_CharacterAmbientColour[1] = 1.0f;
	g_CharacterAmbientColour[2] = 1.0f;
    
    for( u_int u = 0; u < ASURA_PS2_MAX_LIGHTS_FOR_VU; ++u )
    {
        g_CharacterTransformedLights[u].colour[0] = 0.0f;
        g_CharacterTransformedLights[u].colour[1] = 0.0f;
        g_CharacterTransformedLights[u].colour[2] = 0.0f;
        g_CharacterTransformedLights[u].colour[3] = 1.0f;
    }

    if( pfLightLevel ) { *pfLightLevel = 1.0f; }
}

#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL

DEBUGVAR float dbg_LightAlpha = 127.0f;// this can't be 128 or transparency on characters doesn't work.
// The reason is that if the shiny map has alpha = 128, the opaque pass ends up with alpha of 128 on just
// those regions with full shininess. This is the same as the the alpha on the environment, so the
// highlights don't get drawn.
DEBUGVAR float dbg_SpecularAlpha = 128.0f;
// (MH - 12.08.2002) hack for disabling graphic novel
void Asura_PS2_SetupCharacterLighting_GraphicNovel( const Asura_Animation* pxAnim, const Asura_Vector_3& xBBCentre, float* pfLightLevel )
{
    pxAnim->UpdateLighting();
	
    // (MH - 12.08.2002) hack to prevent non specular-lit characters from appearaing black
    if (dbg_bDisableSmoothSkinSpecularPass)
    {
        // directional light from camera
		const Asura_Photon_Sampling_Result* pxResult = Asura_Photon_System::SamplePhotonList( xBBCentre );
        Asura_Vector_3 xDir = xBBCentre - *Asura_Camera::GetPosition();
        xDir.Normalise();

        if( !Asura_Hierarchy_System::s_bBonesInWorldspace )
        {
            xDir = Asura_Hierarchy_System::s_pxOri->ApplyTranspose( xDir );
        }

        g_CharacterTransformedLights[0].direction[0] = xDir.x;
        g_CharacterTransformedLights[0].direction[1] = xDir.y;
        g_CharacterTransformedLights[0].direction[2] = xDir.z;
        g_CharacterTransformedLights[0].colour[0] = Asura_Maths::Min( 255.0f, pxResult->m_fDiffuseR * 128.0f );
        g_CharacterTransformedLights[0].colour[1] = Asura_Maths::Min( 255.0f, pxResult->m_fDiffuseG * 128.0f );
        g_CharacterTransformedLights[0].colour[2] = Asura_Maths::Min( 255.0f, pxResult->m_fDiffuseB * 128.0f );
        g_CharacterTransformedLights[0].colour[3] = dbg_LightAlpha;
    }
    else
    {
	    // transformed light array; currently only position offset applies
		g_CharacterNumLocalLights = Asura_Photon_System::NumberOfLocalPhotons;

		// (MH - 04.10.2002) added ambient debugvar so characters are not completely black
		// (AW - 26.02.2004) - I haven't updated the GN-renderers on PC or XBox so I haven't updated 
		// this code to use Asura_Lighting_System::s_fEntityAmbient# either.
		g_CharacterAmbientColour[0] = dbg_fAmbient;
		g_CharacterAmbientColour[1] = dbg_fAmbient;
		g_CharacterAmbientColour[2] = dbg_fAmbient;

        float fCumulativeLightLevel = 0.0f;

		for ( u_int u=0; u<g_CharacterNumLocalLights; u++)
		{
			const Asura_Photon	 *pxPhoton		  = Asura_Photon_System::LocalPhotons[u].Photon;
			const Asura_Vector_3& xPhotonPosition = pxPhoton->GetPosition();
	        
			Asura_Vector_3 xLightToCharacter = xBBCentre - xPhotonPosition;
			float fAttenuation = Asura_Maths::Max( 0.0f, 1.0f - xLightToCharacter.Magnitude()/pxPhoton->GetRange() );
	        Asura_Assert( fAttenuation >=0.0f && fAttenuation  <= 1.0f, "light attenuation out of bounds" );

            // (MP - 15.05.2003) Normal range for lights is [0, 0.5]. Anything
            // over 0.5 is oversaturated. Deal with this on the EE and then in
            // VU1 microcode we can use the 0->1 and 1+ ranges respectively,
            // which makes more sense from a PS2 point of view.
            fAttenuation *= 2.0f;

			if ( u==0 )
			{
                Asura_Vector_3 xPos = xPhotonPosition;

                if( !Asura_Hierarchy_System::s_bBonesInWorldspace )
                {
                    xPos -= *Asura_Hierarchy_System::s_pxPos;
                    xPos = Asura_Hierarchy_System::s_pxOri->ApplyTranspose( xPos );
                }

		        // First light is used for specular
				g_CharacterSpecularLightPosition[0] = xPos.x;
				g_CharacterSpecularLightPosition[1] = xPos.y;
				g_CharacterSpecularLightPosition[2] = xPos.z;
				g_CharacterSpecularLightPosition[3] = 1.0f;

				g_CharacterSpecularColour[0] = Asura_Maths::Min( 255.0f, fAttenuation * pxPhoton->GetRedComponent()   * 128.0f );
				g_CharacterSpecularColour[1] = Asura_Maths::Min( 255.0f, fAttenuation * pxPhoton->GetGreenComponent() * 128.0f );
				g_CharacterSpecularColour[2] = Asura_Maths::Min( 255.0f, fAttenuation * pxPhoton->GetBlueComponent()  * 128.0f );
				g_CharacterSpecularColour[3] = dbg_SpecularAlpha;
			}

            if( !Asura_Hierarchy_System::s_bBonesInWorldspace )
            {
                xLightToCharacter = Asura_Hierarchy_System::s_pxOri->ApplyTranspose( xLightToCharacter );
            }
			g_CharacterTransformedLights[u].direction[0] = xLightToCharacter.x;
	        g_CharacterTransformedLights[u].direction[1] = xLightToCharacter.y;
			g_CharacterTransformedLights[u].direction[2] = xLightToCharacter.z;
			g_CharacterTransformedLights[u].direction[3] = 1.0f;
            
			g_CharacterTransformedLights[u].colour[0] = fAttenuation * pxPhoton->GetRedComponent();
			g_CharacterTransformedLights[u].colour[1] = fAttenuation * pxPhoton->GetGreenComponent();
			g_CharacterTransformedLights[u].colour[2] = fAttenuation * pxPhoton->GetBlueComponent();
			g_CharacterTransformedLights[u].colour[3] = 1.0f;

            fCumulativeLightLevel += (1.0f / 3.0f) * (   g_CharacterTransformedLights[u].colour[0]
                                                       + g_CharacterTransformedLights[u].colour[1]
                                                       + g_CharacterTransformedLights[u].colour[2] );
		}
    
        if( pfLightLevel ) { *pfLightLevel = Asura_Maths::Min( fCumulativeLightLevel, 2.0f ); }
    
		for ( u_int u = g_CharacterNumLocalLights; u < ASURA_PS2_MAX_LIGHTS_FOR_VU; u++ )
		{
			g_CharacterTransformedLights[u].colour[0] = 0.0f;
			g_CharacterTransformedLights[u].colour[1] = 0.0f;
			g_CharacterTransformedLights[u].colour[2] = 0.0f;
			g_CharacterTransformedLights[u].colour[3] = 1.0f;
		}
    }
}

#endif // #ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
