//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS2_WaterRenderer.h
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Description:
//
//     EE side code for rendering a water mesh based on a very small data set
//     All the work is to be done on the vector units
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS2_WATERRENDERER_H___
#define ___ASURA_PS2_WATERRENDERER_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Rendering/Asura_Water.h"

// Engine platform-specific.

// G L O B A L S /////////////////////////////////////////////////////////////

// Subroutines on the VU.
extern u_int VU1_WATER_INIT[]                   __attribute__(( section(".vudata") ));
extern u_int VU1_WATER_RENDER[]                 __attribute__(( section(".vudata") ));
extern u_int VU1_WATER_SWITCH_CAM_DATA_BFR[]    __attribute__(( section(".vudata") ));
extern u_int VU1_WATER_SWITCH_RENDERSTATE_BFR[] __attribute__(( section(".vudata") ));
extern u_int VU1_WATER_LOAD_MESH_FLAGS[]        __attribute__(( section(".vudata") ));
extern u_int VU1_WATER_LOAD_CAM_FLAGS[]         __attribute__(( section(".vudata") ));
extern u_int VU1_WATER_NOP[]			        __attribute__(( section(".vudata") ));

// M A C R O S ///////////////////////////////////////////////////////////////

#define ASURA_PS2_WATER_SPR_QWS_RESERVED (0x30)

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PS2_WaterRenderer
{

    public:

        // C L A S S   M E T H O D S /////////////////////////////////////////

        //
        // General setup stuff.
        //

        static void Initialise( void );
        static void Reset( void );
        static void Shutdown( void );

        //
        // Rendering.
        //

        static void SetRenderstate( u_int uMaterialFlags );
        static void SetTexture( int iTexID, int iMMAG, int iMMIN, u_int uBuffer, int iPass = 0 );

        // Startup.

        static void Begin( void );

        // Data transfer and actual rendering.

        static void RenderMesh( u_int uREFAddr, u_int uNumQWs, u_int uNumVertices );
		static void SetupYManipulations( float fT1, float fT2, float fT3, const Asura_Water_System::Water_Controls* pxControls );

        // Tidy up.

        static void End( void );

        //
        // Misc.
        //

        // Only valid between Begin() and End().
        static u_int GetNumPassesPerTri( void );
        
        static bool IsMaterialReflective( u_int uMaterialFlags );

};
// End class Asura_PS2_WaterRenderer.



// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS2_WATERRENDERER_H___
