//////////////////////////////////////////////////////////////////////////////
//
// Asura_PlatformRenderCharacter.cpp
//
// By: Mike Healey
//
// Copyright (c) 2001 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_PlatformRenderCharacter.h"

// Independant
#include "Animation/Asura_Hierarchy.h"
#include "Camera/Asura_Camera.h"   // for specular
#include "Core/Asura_Material_System.h"
#include "Core/Asura_TextureManagement.h"
#include "Light/Asura_Light.h"
#include "Light/Asura_Photon.h"
#include "Rendering/Asura_Render.h"

// PS2
#include "Animation/Asura_PS2_CharDefines.h"
#include "Animation/Asura_PS2_Hierarchy.h"
#include "Animation/Asura_PS2_SkinTypes.h"
#include "Core/Asura_PlatformBasics.h"
#include "Core/Asura_PS2_DMA.h"
#include "Core/Asura_PS2_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Profiling/Asura_PS2_Profiling.h"
#include "Rendering/Asura_PlatformRendering.h"
#include "Rendering/Asura_PlatformRenderStates.h"
#include "Rendering/Asura_PlatformTextureManagement.h"
#include "Rendering/Asura_PS2_DeferredRendering.h"
#include "Rendering/Asura_PS2_GS.h"
#include "Rendering/Asura_PS2_Lighting.h"
#include "Rendering/Asura_PS2_RenderingOptions.h"
#include "Rendering/Asura_PS2_Shadows.h"
#include "Rendering/Asura_PS2_ShadowBuffer.h"
#include "Rendering/Asura_PS2_VU1.h"

// D E F I N E S /////////////////////////////////////////////////////////////

#define SKINANDRENDER_MAX_NUMBER_OF_CAMERAS				4
#define SKINANDRENDER_MAX_SHADOWCASTING_LIGHTS			1			// Most of the code should handle upto 4  shadow casting lights, but at prsent the light is projected straight down, so it's only worth having one
#define SKINANDRENDER_DATA_PER_SHADOWCASTING_LIGHT		5

#define SHADOW_SIZE	(256) //(64)

#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
	#define ENABLE_TRANSPARENCY_ON_CHARACTERS_WITH_GRAPHIC_NOVEL (0)
#endif

// C O N S T A N T S /////////////////////////////////////////////////////////

#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
static const u_int TEX_BFR_FOR_BASE   = 0;
static const u_int TEX_BFR_FOR_GN     = 1;
static const u_int TEX_BFR_FOR_SHADOW = 2;
#else
static const u_int TEX_BFR_FOR_BASE   = Asura_PS2_TextureManagement::BFR_AUTO;
static const u_int TEX_BFR_FOR_SHADOW = 2;
#endif

// G L O B A L S /////////////////////////////////////////////////////////////

//DEBUGVAR Asura_Vector_3 dbg_CharacterMaxLightingValue(255.0f, 255.0f, 255.0f);
//DEBUGVAR float dbg_MaxDistForShadowCastingLight = 10.0f;
//DEBUGVAR float dbg_MaxDistForShadowCastingLightSquared = dbg_MaxDistForShadowCastingLight*dbg_MaxDistForShadowCastingLight;
//DEBUGVAR float dbg_fForceShadowAlpha = 0.0f;

//DEBUGVAR float dbg_fMinLightLevelForShadows = 0.1f;
//DEBUGVAR float dbg_fMaxLightLevelForShadows = 1.25f;

DEBUGVAR bool dbg_bCharBilinear  = true;
DEBUGVAR bool dbg_bCharTrilinear = false;
#define CHAR_MMAG (dbg_bCharBilinear ? SCE_GS_LINEAR : SCE_GS_NEAREST)
#define CHAR_MMIN (dbg_bCharTrilinear ? SCE_GS_LINEAR_MIPMAP_LINEAR : (dbg_bCharBilinear ? SCE_GS_LINEAR_MIPMAP_NEAREST : SCE_GS_NEAREST_MIPMAP_NEAREST))

DEBUGVAR bool dbg_bDisplayCharShadowBufferRGB   = false;
DEBUGVAR bool dbg_bDisplayCharShadowBufferAlpha = false;

// P R O T O T Y P E S   A N D   I N L I N E   F U N C T I O N S /////////////

static void SkinAndRender_AddShadowData( const Asura_Vector_3& xBSCentre, float fBSRadius, Asura_PS2_QW* pxShadowRenderDataBase );

#if 0
static void SkinAndRender_DrawShadow( u_int uBoneQty,
                                      const Asura_Vector_3& xShadowPos, float fShadowSize,
                                      float fShadowAlpha,
									  bool bGraphicsNovel,
                                      u_int uCamVisFlags );
#endif

static Asura_PS2_QW* CreateBones( Asura_PS2_QW* pxOut, u_int uBoneCount )
{
    Asura_Assert( pxOut, "" );
    for( u_int u = 0; u < uBoneCount; pxOut +=4, ++u )
    {
        Asura_PS2_VU1::SetupPS2MatFromAsuraData( pxOut->f32s, Asura_Hierarchy_System::s_axBoneOri[u], Asura_Hierarchy_System::s_axBonePos[u] );
    }
	return pxOut;
}

// F U N C T I O N S /////////////////////////////////////////////////////////

#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
static int g_iHighlightTextureID = -2;
static const char* HIGHLIGHT_TEXTURE_NAME = "\\specialfx\\specularshader32.tm2";
static int g_iDreddTextureID;

static void InitDreddCharacterRenderTextures( int iMainTextureID, bool bMainTextureUploaded )
{
    Asura_PS2_TextureManagement::SetupTextureEnvironment( bMainTextureUploaded, ASURA_PS2_TEXREGFLAG_TEXFLUSH );

    if( g_iHighlightTextureID == -2 )
    {
        g_iHighlightTextureID = Asura_Texture_Management::AddTexture( HIGHLIGHT_TEXTURE_NAME );

        // (MP - 06.08.2003) Shouldn't be any need to load this separately
        // anymore. Include it in your main graphics archive because it might
        // as well be permanently resident.
        Asura_Assert( Asura_Texture_Management::IsTextureLoaded(g_iHighlightTextureID), "" );
    }

	bool bHilightTextureUploaded = Asura_PS2_TextureManagement::UploadTexture( g_iHighlightTextureID, TEX_BFR_FOR_GN );

    Asura_PS2_TextureManagement::SetupTextureEnvironment( bHilightTextureUploaded, ASURA_PS2_TEXREGFLAG_TEXFLUSH );

    // remember the main texture ID for later
    g_iDreddTextureID = iMainTextureID;
}
#endif//#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL

static void ConcatenateModelToWorldTransform( sceVu0FMATRIX xIn, sceVu0FMATRIX xOut )
{
    if( Asura_Hierarchy_System::s_bBonesInWorldspace )
    {
        sceVu0CopyMatrix( xOut, xIn );
    }
    else
    {
        sceVu0FMATRIX xModelToWorld;
        Asura_PS2_VU1::SetupPS2MatFromAsuraData( &xModelToWorld[0][0], *Asura_Hierarchy_System::s_pxOri, *Asura_Hierarchy_System::s_pxPos );
        sceVu0MulMatrix( xOut, xIn, xModelToWorld );
    }
}

void SetupCharacterCoordinateMatrices( sceVu0FMATRIX xModelToScreen, sceVu0FMATRIX xModelToClip )
{
    sceVu0FMATRIX xModelToView;
    ConcatenateModelToWorldTransform( Asura_PS2_VU1::GetWorldToViewMat(), xModelToView );

    sceVu0MulMatrix( xModelToScreen, Asura_PS2_VU1::GetViewToScreenMat(),        xModelToView );
    sceVu0MulMatrix( xModelToClip,   Asura_PS2_VU1::GetViewToClipMatGuardband(), xModelToView ); 
}

void SetupCharacterLightMatrices( sceVu0FMATRIX xLightColour, sceVu0FMATRIX xLightNormal )
{
	sceVu0NormalLightMatrix( xLightNormal, g_CharacterTransformedLights[0].direction, 
										   g_CharacterTransformedLights[1].direction, 
										   g_CharacterTransformedLights[2].direction );

	sceVu0LightColorMatrix( xLightColour, g_CharacterTransformedLights[0].colour, 
										  g_CharacterTransformedLights[1].colour, 
										  g_CharacterTransformedLights[2].colour, 
										  g_CharacterAmbientColour );
}

#if _DEBUG
	static float g_HackLightDist = 1.5f;
	static float g_HackAngleXZ = 0.0f;
	static float g_HackLightSpeedXZ = Asura_Maths::Deg180;
	static float g_HackAngleY = 0.0f;
	static float g_HackLightSpeedY = 0.0f;
	static bool dbg_TestSpecular = false;
	static bool dbg_bInvertSpecularHalfwayVector = false;
#endif

#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
	DEBUGVAR int dbg_CharAlphaComp = 3;
	DEBUGVAR bool dbg_bEnableDreddTransparency = false; //true;
	DEBUGVAR bool dbg_bEnableGraphicNovelHighlights = true;
#endif


//////////////////////////////////////////////////////////////////
////////////////// S K I N   A N D   R E N D E R /////////////////
//////////////////////////////////////////////////////////////////

// Setup character rendering data for use with VU1 Skin And Render rendering
static void SkinAndRender_CreateDrawlist( const u_int uBoneQty,
                                          const bool bGraphicsNovel,
                                          const bool bShadow,
                                          const u_int uCamVisFlags,
                                          const u_int uAlpha,
                                          const u_int uSkinDrawlistAddress,
                                          const Asura_Vector_3& xBSCentre,
                                          float fBSRadius )
{
    int			  i;
	int			  j;
//	u_int		  u;
    sceVu0FMATRIX xCharacterLocal2Screen;
    sceVu0FMATRIX xCharacterLocal2Clip;
    sceVu0FMATRIX xLightColour;
    sceVu0FMATRIX xLightNormal;
    Asura_PS2_QW* pxOut;
	Asura_PS2_QW* pxRenderData;
    u_int		  uBoneQwords	 = Asura_Maths::Max((int)(uBoneQty<<2),(int)ASURA_PS2_SKINNING__QWS_CONSTANTS_GRAPHICS_NOVEL);
	const u_int   uXYZReg		 = SCE_GS_XYZ2;
	const u_int   uPrimType		 = (Asura_Render::GetWireframe()) ? SCE_GS_PRIM_LINESTRIP : SCE_GS_PRIM_TRISTRIP;
	const u_int	  uDataPerCamera = (bGraphicsNovel) ? 12 : 10;

	// Sort out the lights
    SetupCharacterLightMatrices( xLightColour, xLightNormal );

	// Calculate how many cameras we are going to be rendering
	u_int uCameraQty = 0;
	for( int iCam = 0; iCam < Asura_Camera::GetNumberOfCameras(); ++iCam )
	{
		if( uCamVisFlags & (1 << iCam) ) { ++uCameraQty; }
	}

	Asura_Assert( uCameraQty, "Shouldn't have even called this function if no cameras can see this smooth skin." );

    // Lets make some room to build the setup draw list - enough room for the maximum number of cameras and shadows
	const u_int uPreCameraQwords = 1 + (bGraphicsNovel ? ASURA_PS2_SKINNING__QWS_CONSTANTS_GRAPHICS_NOVEL : ASURA_PS2_SKINNING__QWS_CONSTANTS);	// Number of QWs of data before the camera data
	const u_int uPostShadowQwords = 2 + uBoneQwords;					// Number of QWs of data after the shadows
	const u_int uTotalQwords = uPreCameraQwords
							 + (uCameraQty * uDataPerCamera ) +								// Data per camera
                             + (bShadow ? SKINANDRENDER_DATA_PER_SHADOWCASTING_LIGHT : 0)
							 + uPostShadowQwords;											// Other data (GIF tags etc)

    // Calculate buffer Sizes
	const u_int uVertsPerBuffer = ASURA_PS2_MAX_VERTICES_PER_SMOOTH_SKIN_STRIP( uBoneQty );
	const u_int uOutputBufferSize = 1 + (uVertsPerBuffer * ASURA_PS2_SKINNING__QWS_PER_OUTPUT_VERTEX); // Needs to be a multiple of 2
	const u_int uInputBufferSize  = (uVertsPerBuffer * ASURA_PS2_SKINNING__QWS_PER_INPUT_VERTEX);
	const u_int uInputBufferStart = uBoneQwords + (uOutputBufferSize<<1);

    Asura_PS2_QW* const pxBase = Asura_PS2_DeferredRendering::GetVifBuffer( uTotalQwords );
	pxRenderData = pxBase + uPreCameraQwords;

	// Add the per camera data
    for( int iCam = 0; iCam < Asura_Camera::GetNumberOfCameras(); ++iCam )
	{
		if( uCamVisFlags & (1 << iCam) )
		{
			Asura_Camera::SetCameraIndex( iCam );

			// Recalculate the projection and clipping matricies
			SetupCharacterCoordinateMatrices( xCharacterLocal2Screen, xCharacterLocal2Clip );

            {
                const u_long128* puSrc;
                
                puSrc = reinterpret_cast<u_long128*>( xCharacterLocal2Screen );
                for( u_int u = 4; u--; ) { (pxRenderData++)->u128 = *puSrc++; }

                puSrc = reinterpret_cast<u_long128*>( xCharacterLocal2Clip );
                for( u_int u = 4; u--; ) { (pxRenderData++)->u128 = *puSrc++; }
            }
            
#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
			if ( bGraphicsNovel )
			{
				const Asura_Vector_3 xLightPosition(g_CharacterSpecularLightPosition[0], g_CharacterSpecularLightPosition[1], g_CharacterSpecularLightPosition[2]);
			
				Asura_Vector_3 xToEye = *Asura_Camera::GetPosition();
                if( !Asura_Hierarchy_System::s_bBonesInWorldspace )
                {
                    xToEye = Asura_Hierarchy_System::s_pxOri->ApplyTranspose( xToEye - *Asura_Hierarchy_System::s_pxPos );
                }
				xToEye -= xBSCentre;
				xToEye.Normalise();
				
				Asura_Vector_3 xToLight = xLightPosition - xBSCentre;
				xToLight.Normalise();
				
				Asura_Vector_3 xSpecularHalfVector = xToLight + xToEye;
				xSpecularHalfVector.Normalise();

				//;	1Qwds	Camera - Specular Eye Vector
				pxRenderData->f32s[0] = xToEye.x;
				pxRenderData->f32s[1] = xToEye.y;
				pxRenderData->f32s[2] = xToEye.z;
				pxRenderData->f32s[3] = 1.0f;
				pxRenderData++;

				//;	1Qwds	Camera - Specular Half Vector
				pxRenderData->f32s[0] = xSpecularHalfVector.x;
				pxRenderData->f32s[1] = xSpecularHalfVector.y;
				pxRenderData->f32s[2] = xSpecularHalfVector.z;
				pxRenderData->f32s[3] = 1.0f;
				pxRenderData++;

				//;	1Qwds	Camera 2 Scissor Value
				//;	1Qwds	Camera 2 XYOffset Value
				pxRenderData = Asura_PS2_GS::GetViewportSetupGifCmds( Asura_Camera::GetCameraIndex(), Asura_PS2_GS::CONTEXT_1, pxRenderData );
			}
			else
#endif//#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
			{
				//;	1Qwds	Camera 2 Scissor Value
				//;	1Qwds	Camera 2 XYOffset Value
				pxRenderData = Asura_PS2_GS::GetViewportSetupGifCmds( Asura_Camera::GetCameraIndex(), Asura_PS2_GS::CONTEXT_1, pxRenderData );
			}
		}
	}

	if( bShadow )
	{
		SkinAndRender_AddShadowData( xBSCentre, fBSRadius, pxRenderData );
	    pxRenderData += SKINANDRENDER_DATA_PER_SHADOWCASTING_LIGHT;
	}

	// How much data re we going to transfer
	const u_int uQwordsToTransfer = ( uPreCameraQwords - 1 ) +
									( uCameraQty * uDataPerCamera ) + 
									(bShadow ? SKINANDRENDER_DATA_PER_SHADOWCASTING_LIGHT : 0);
	
#if _DEBUG
    const Asura_PS2_QW* pxEnd = pxBase + 1 + uQwordsToTransfer + uPostShadowQwords;
#endif
	pxOut = pxBase;

#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
;//;    (BoneBuffer - 21)   Pass 1 TEST
;//;	(BoneBuffer - 20)	Pass 1 Setup Gif Tag
;//;	(BoneBuffer - 19)	Pass 1 MIPTBP1
;//;	(BoneBuffer - 18)	Pass 1 TEX1
;//;	(BoneBuffer - 17)	Pass 1 TEX0
;//;	(BoneBuffer - 16)	Pass 2 Setup Gif Tag
;//;	(BoneBuffer - 15)	Pass 2 Setup TEX1
;//;	(BoneBuffer - 14)	Pass 2 Setup TEX0
;//;	(BoneBuffer - 13)	Pass 2 Setup TEST
;//;	(BoneBuffer - 12)	Shadow Setup Gif Tag
;//;	(BoneBuffer - 11)	Shadow RGBAQ register
;//;	(BoneBuffer - 10)	Pass 1 Drawing Gif Tag - Camera count in NLOOP (x component)
;//;	(BoneBuffer -  9)	Pass 2 Drawing Gif Tag - Shadow casting light count in NLOOP (x component)
;//;	(BoneBuffer -  8)	Shadow Drawing Gif Tag
;//;	(BoneBuffer -  7)	Light Normals (X Components )
;//;	(BoneBuffer -  6)	Light Normals (Y Components )
;//;	(BoneBuffer -  5)	Light Normals (Y Components )
;//;	(BoneBuffer -  4)	Light Colour 0
;//;	(BoneBuffer -  3)	Light Colour 1
;//;	(BoneBuffer -  2)	Light Colour 2
;//;	(BoneBuffer -  1)	Specular Colour
;//;	(BoneBuffer +  0)	Camera 1 Screeen Matrix
;//;	(BoneBuffer +  4)	Camera 1 Clipping Matrix
;//;	(BoneBuffer +  8)	Camera 1 Specular Eye Vector
;//;	(BoneBuffer +  9)	Camera 1 Specular Half Vector
;//;	(BoneBuffer + 10)	Camera 1 Scissor Value
;//;	(BoneBuffer + 11)	Camera 1 XYOffset Value

    // Setup VU Microcodes Constant Data
    if (bGraphicsNovel)
	{	//; See VU1_Skinning_Dredd.vcl for complete memory map
	    pxOut->u32s[0] = PY_DMA_CALL | uTotalQwords-1;													// Call the skin drawlist
		pxOut->u32s[1] = uSkinDrawlistAddress;															// null address, since not REF tag
		pxOut->u32s[2] = SCE_VIF1_SET_STCYCL(4, 4, 0);													// CL == WL, i.e. write 1 qword every cycle. 
		pxOut->u32s[3] = SCE_VIF1_SET_UNPACK( (uBoneQwords-ASURA_PS2_SKINNING__QWS_CONSTANTS_GRAPHICS_NOVEL), uQwordsToTransfer, CMD_V4_32, 0);	// Put the camera and light data after the bones
		pxOut++;

		// Pass 1 Test register
#if ENABLE_TRANSPARENCY_ON_CHARACTERS_WITH_GRAPHIC_NOVEL
		pxOut->u64s[0] = SCE_GS_SET_TEST_1( 0,							// ATE   Alpha test enable
											SCE_GS_ALPHA_GREATER,		// ATST  Alpha test method
											dbg_CharAlphaComp,			// AREF  Alpha Comparison
											SCE_GS_AFAIL_KEEP,			// AFAIL When failed
											0,							// DATE  Destination alpha test
											0,							// DATM  Dest. Alpha test mode
											1,							// ZTE   Depth Test enable
											SCE_GS_ZGEQUAL );			// ZTST  Depth Test Method
#else
		pxOut->u64s[0] = SCE_GS_SET_TEST_1( 0,							// ATE   Alpha test enable
											0,	                        // ATST  Alpha test method
											0,			                // AREF  Alpha Comparison
											0,			                // AFAIL When failed
											0,							// DATE  Destination alpha test
											0,							// DATM  Dest. Alpha test mode
											1,							// ZTE   Depth Test enable
											SCE_GS_ZGEQUAL );			// ZTST  Depth Test Method
#endif
		pxOut->u64s[1] = SCE_GS_TEST_1;
		pxOut++;

		// Pass 1 Setup Gif Tag
        // Gif tag for camera and first pass setup - (SCISSOR, XYOFFSET, MIPTBP_1, TEX1_1, TEX0_1, TEST_1)
		pxOut->u64s[0] = SCE_GIF_SET_TAG( 6,				// NLOOP 
										  1,				// EOP  (End Of Packet)
										  NULL,				// PRE  (PRIM field Enable)
										  NULL,				// PRIM
										  SCE_GIF_PACKED,	// FLG  (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
										  1 );				// NREG (Number of registers)
		pxOut->u64s[1] = SCE_GIF_PACKED_AD;
		pxOut++;

        // Pass 1 MIPTBP1
        // (MP - 20.10.2003) Send this before TEX1 so that it will get
        // overwritten by the hardware if we're enabling MTBA. This saves us
        // making the NLOOP in the GIF tag conditional on whether or not the
        // GS can automatically calculate the TBP values. The recommended
        // order is TEX1-TEX0-MIPTBP1 but this seems to work ok and it's only
        // a hack for Dredd PS2 NTSC US anyway.
        Asura_PS2_TextureManagement::GetMIPTBP1Setting( &pxOut->u64s[0], g_iDreddTextureID, TEX_BFR_FOR_BASE );
        pxOut->u64s[1] = SCE_GS_MIPTBP1_1;
        ++pxOut;

		// Pass 1 Tex1 register
        Asura_PS2_TextureManagement::GetTEX1Setting( &pxOut->u64s[0], g_iDreddTextureID, CHAR_MMAG, CHAR_MMIN );
		pxOut->u64s[1] = SCE_GS_TEX1_1;
        ++pxOut;
        
		// Pass 1 Tex0 register
		Asura_PS2_TextureManagement::GetTEX0Setting( &pxOut->u64s[0], g_iDreddTextureID, TEX_BFR_FOR_BASE );
		pxOut->u64s[1] = SCE_GS_TEX0_1;
		pxOut++;

		// Pass 2 Setup Gif Tag - (TEX0_1, TEX1_1, TEST_1)
		pxOut->u64s[0] = SCE_GIF_SET_TAG( 3,							// NLOOP 
										  1,							// EOP  (End Of Packet)
										  NULL,							// PRE  (PRIM field Enable)
										  NULL,							// PRIM
										  SCE_GIF_PACKED,				// FLG  (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
										  1 );							// NREG (Number of registers)
		pxOut->u64s[1] = SCE_GIF_PACKED_AD;//0x0eL
		pxOut++;

		// Pass 2 Setup TEX1
        Asura_PS2_TextureManagement::GetTEX1Setting( &pxOut->u64s[0], g_iHighlightTextureID, CHAR_MMAG, CHAR_MMIN );
		pxOut->u64s[1] = SCE_GS_TEX1_1;
		pxOut++;

		// Pass 2 Setup TEX0
		Asura_PS2_TextureManagement::GetTEX0Setting( &pxOut->u64s[0], g_iHighlightTextureID, TEX_BFR_FOR_GN );
		pxOut->u64s[1] = SCE_GS_TEX0_1;
		pxOut++;

		// Pass 2 Setup TEST
		pxOut->u64s[0] = SCE_GS_SET_TEST_1( 0,								// ATE   Alpha test enable
											0,								// ATST  Alpha test method
											0,								// AREF  Alpha Comparison
											0,								// AFAIL When failed
											dbg_bEnableDreddTransparency,	// DATE  Destination alpha test
											0,								// DATM  Dest. Alpha test mode
											1,								// ZTE   Depth Test enable
											SCE_GS_ZGEQUAL );				// ZTST  Depth Test Method
		pxOut->u64s[1] = SCE_GS_TEST_1;
		pxOut++;

		// Shadow Setup Gif Tag
		pxOut->u64s[0] = SCE_GIF_SET_TAG( 2,				// NLOOP 
										  1,				// EOP  (End Of Packet)
										  NULL,				// PRE  (PRIM field Enable)
										  NULL,				// PRIM
										  SCE_GIF_PACKED,	// FLG  (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
										  1 );				// NREG (Number of registers)
		pxOut->u64s[1] = SCE_GIF_PACKED_AD;
		pxOut++;

		// Shadow RGBAQ register - white
		pxOut->u64s[0] = SCE_GS_SET_RGBAQ( 255, 255, 255, 128, 0x3F800000 );
		pxOut->u64s[1] = SCE_GS_RGBAQ;
		pxOut++;

		// Pass 1 Drawing Gif Tag - (ST, RGBAQ, XYZ)
        pxOut->u64s[0] = SCE_GIF_SET_TAG( uCameraQty,					// NLOOP - Sneaked camera count in here !!!!!!
										  0,							// EOP (End Of Packet) - Added by microcode
										  1,							// PRE (PRIM field Enable)
										  SCE_GS_SET_PRIM( uPrimType,		// PRIM Type
														   1,				// IIP (Flat/Gouraud Shading)
														   1,				// TME (Texture Mapping Enabled)
														   0,				// FGE (Fogging Enabled)
														   0,				// ABE (Alpha Blending Enabled)
														   0,				// AA1 (1 Pass Anti-Aliasing Enabled)
														   0,				// FST (Method of specifying Texture Coords)
														   0,				// CTXT (Context)
														   0 ),				// FIX (Fragment Value Control)
										  0,							// FLG (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
										  3 );							// NREG (Number of registers)
        pxOut->u64s[1] = (uXYZReg << 8) | (SCE_GS_RGBAQ << 4) | (SCE_GS_ST << 0);
		pxOut++;
        
		// Pass 2 Drawing Gif Tag - (ST, RGBAQ, XYZ)
		pxOut->u64s[0] = SCE_GIF_SET_TAG( bShadow,			// NLOOP - Sneak in shadow count !!!!!!
										  0,								// EOP (End Of Packet) - Added in the microcode
										  1,								// PRE (PRIM field Enable)
										  SCE_GS_SET_PRIM( uPrimType,							// PRIM Type
														   1,									// IIP (Flat/Gouraud Shading)
														   1,									// TME (Texture Mapping Enabled)
														   0,									// FGE (Fogging Enabled)
														   dbg_bEnableGraphicNovelHighlights,	// ABE (Alpha Blending Enabled)
														   0,									// AA1 (1 Pass Anti-Aliasing Enabled)
														   0,									// FST (Method of specifying Texture Coords)
														   0,									// CTXT (Context)
															   0 ),									// FIX (Fragment Value Control)
										  0,								// FLG (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
										  3 );								// NREG (Number of registers)
        pxOut->u64s[1] = (uXYZReg << 8) | (SCE_GS_RGBAQ << 4) | (SCE_GS_ST << 0);
		pxOut++;

		// Shadow Drawing Gif Tag
        {
            const u_int uPRIM = Asura_PS2_Shadows::Proj_GetPRIMSettingForRenderToBfr();

            pxOut->u64s[0] = SCE_GIF_SET_TAG( uOutputBufferSize, // NLOOP - Sneak output buffer size in here.
								              false,             // EOP   - Set by microcode.
								              true,              // PRE
								              uPRIM,
								              SCE_GIF_PACKED,    // FLG
								              1 );               // NREG
            pxOut->u64s[1] = SCE_GS_XYZ2;
            ++pxOut;
        }

		// Light Directions
		for (i = 0; i < 3; i++)
		{
			for (j = 0; j < 4; j++)
			{
				pxOut->f32s[j] = xLightNormal[i][j];
			}
			pxOut++;
		}

		//;	0x076	Light Colours
		for (i = 0; i < 3; i++)
		{
			for (j = 0; j < 4; j++)
			{
				pxOut->f32s[j] = xLightColour[i][j];
			}
			pxOut++;
		}

		// Specular Colour
        for (i = 0; i < 4; i++)
        {
            pxOut->u32s[i] = static_cast<int>(g_CharacterSpecularColour[i]);
        }
		pxOut++;


		//; Camera data and Shadow data already in the draw list
	}
	else	// !bGraphicsNovel
#endif//#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
	{	//; See VU1_Skinning.vcl for complete memory map
		pxOut->u32s[0] = PY_DMA_CALL | uTotalQwords-1;													// Call the skin drawlist
		pxOut->u32s[1] = uSkinDrawlistAddress;													// null address, since not REF tag
		pxOut->u32s[2] = SCE_VIF1_SET_STCYCL(4, 4, 0);											// CL == WL, i.e. write 1 qword every cycle. 
		pxOut->u32s[3] = SCE_VIF1_SET_UNPACK( (uBoneQwords-ASURA_PS2_SKINNING__QWS_CONSTANTS), uQwordsToTransfer, CMD_V4_32, 0);	// Put the camera and light data after the bones
		pxOut++;

		// Drawing Gif Tag
        {
            const u_int uABE = ( !bGraphicsNovel || 128u != uAlpha );

		    pxOut->u64s[0] = SCE_GIF_SET_TAG( uCameraQty,			// NLOOP - Sneaked camera count in here !!!!!!
										      0,					// EOP (End Of Packet) - Gets added in micro code
										      1,					// PRE (PRIM field Enable)
										      SCE_GS_SET_PRIM( uPrimType,				// PRIM Type
														       1,						// IIP (Flat/Gouraud Shading)
														       1,						// TME (Texture Mapping Enabled)
														       0,						// FGE (Fogging Enabled)
														       uABE,			// ABE (Alpha Blending Enabled)
														       0,						// AA1 (1 Pass Anti-Aliasing Enabled)
														       0,						// FST (Method of specifying Texture Coords)
														       0,						// CTXT (Context)
														       0),						// FIX (Fragment Value Control)
										      0,					// FLG (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
										      3); 					// NREG (Number of registers)
		    pxOut->u64s[1]= (uXYZReg << 8) | (SCE_GS_RGBAQ << 4) | (SCE_GS_ST << 0);
		    pxOut++;
        }

		//;	0x071	Light Colour Scale + Output buffer size
//		pxOut->f32s[0] = dbg_CharacterMaxLightingValue.x;
//		pxOut->f32s[1] = dbg_CharacterMaxLightingValue.y;
//		pxOut->f32s[2] = dbg_CharacterMaxLightingValue.z;
		pxOut->u32s[3] = uOutputBufferSize;
		pxOut++;
		
		//;	0x072	Light Directions
		for (i = 0; i < 3; i++)
		{
			for (j = 0; j < 4; j++)
			{
				pxOut->f32s[j] = xLightNormal[i][j];
			}
			pxOut++;
		}

		//; 0x75	Characters Alpha value
//		pxOut->u32s[0] = 0; Unused
//		pxOut->u32s[1] = 0; Unused
//		pxOut->u32s[2] = 0; Unused
		pxOut->u32s[3] = uAlpha;
		pxOut++;

		//;	0x076	Light Colours
		for (i = 0; i < 4; i++)
		{
			for (j = 0; j < 4; j++)
			{
				pxOut->f32s[j] = xLightColour[i][j];
			}
			pxOut++;
		}

		//;	0x07a	Camera Setup Gif Tag
        // Gif tag for camera setup - (SCISSOR, XYOFFSET)
		pxOut->u64s[0] = SCE_GIF_SET_TAG( 2,				// NLOOP 
										  1,				// EOP  (End Of Packet)
										  NULL,				// PRE  (PRIM field Enable)
										  NULL,				// PRIM
										  SCE_GIF_PACKED,	// FLG  (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
										  1 );				// NREG (Number of registers)
		pxOut->u64s[1] = SCE_GIF_PACKED_AD;
		pxOut++;

		{
            //;    0x07b    Shadow Drawing Gif Tag
            {
                const u_int uPRIM = Asura_PS2_Shadows::Proj_GetRenderToBfrPRIMSetting();

                pxOut->u64s[0] = SCE_GIF_SET_TAG( bShadow, // NLOOP - sneak in shadow count.
                                                  false,   // EOP   - put in by microcode.
                                                  true,    // PRE
                                                  uPRIM,   // PRIM
                                                  0,       // FLG
                                                  1 );     // NREG
                pxOut->u64s[1] = SCE_GS_XYZ2;
                ++pxOut;
            }

			//;	0x07c	Shadow Setup Gif Tag
			pxOut->u64s[0] = SCE_GIF_SET_TAG( 2,				// NLOOP 
											  1,				// EOP  (End Of Packet)
											  NULL,				// PRE  (PRIM field Enable)
											  NULL,				// PRIM
											  SCE_GIF_PACKED,	// FLG  (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
											  1 );				// NREG (Number of registers)
			pxOut->u64s[1] = SCE_GIF_PACKED_AD;
			pxOut++;

			//;	0x07d	Shadow RGBAQ register - white
			pxOut->u64s[0] = SCE_GS_SET_RGBAQ( 255, 255, 255, 128, 0x3F800000 );
			pxOut->u64s[1] = SCE_GS_RGBAQ;
			pxOut++;
		}
		//; Camera data and Shadow data already in the draw list
	}

    // Setup the VU double buffer
	pxRenderData->u32s[0] = SCE_VIF1_SET_BASE( uInputBufferStart, 0 );					// setup double buffers to start after the bone data
    pxRenderData->u32s[1] = SCE_VIF1_SET_OFFSET( uInputBufferSize, 0 );					// The buffers are the size of one input buffer + 1 output buffer
	// Start the microcode, to load the constants, then upload the bone data
	pxRenderData->u32s[2] = SCE_VIF1_SET_ITOP(uBoneQwords, 0);						// Put the first output buffer address into ITOPS
    pxRenderData->u32s[3] = SCE_VIF1_SET_MSCAL(0, 0);								// start program execution at 0 (2nd arg => no irq)
	pxRenderData++;

	pxRenderData->u32s[0] = SCE_VIF1_SET_NOP(0);
	pxRenderData->u32s[1] = SCE_VIF1_SET_NOP(0);
    pxRenderData->u32s[2] = SCE_VIF1_SET_FLUSHE(0);									// wait for the constants to be loaded
    pxRenderData->u32s[3] = SCE_VIF1_SET_UNPACK( 0, uBoneQwords, CMD_V4_32, 0);		// address in VU mem
	pxRenderData++;

	// Create the bones and add them to the draw list
	CreateBones( pxRenderData, uBoneQty );
	pxRenderData += uBoneQwords;

#if _DEBUG
    Asura_Assert(pxEnd == pxRenderData, "Vif1 Render Pack wrong length !!!");
#endif
}

// (MH - 31.07.2002) this is temporarily accessible as a global so we can see non-GraphicNovel
// rendering without rewriting the renderer
void Asura_PS2_RenderSmoothSkin( const Asura_Animation* pxAnim, const Asura_PS2_SmoothSkin_SkinHdr* pxSkinHdr,
                                 u_int uBoneQty,
                                 float fAlpha,
                                 u_int uCamVisFlags, u_int uRenderFlags,
                                 const Asura_Vector_3& xBSCentre, float fBSRadius )
{
//	ASURA_PS2_PROFILE__ADD_VIF_START( ASURA_PS2_PROFILE__SKINNED_MARKER );

    const int iMaterialID = (reinterpret_cast<const int*>(pxSkinHdr + 1))[0];

    // Set texture buffer layout.
#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
    Asura_PS2_TextureManagement::SetBufferLayout( ASURA_PS2_TEXBFRLAYOUT_2_MEDIUM_1_SMALL, false );     
#else
    Asura_PS2_TextureManagement::SetBufferLayout( ASURA_PS2_TEXBFRLAYOUT_2_MEDIUM_1_SMALL, true, 0, 1 );     
#endif
    
	// renderstates
    Asura_PlatformRenderStates::CreateDefaultRenderStatePacket( Asura_PlatformRenderStates::CONTEXT_1 );
    
#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
    if( Asura_PS2_RenderingOptions::GetOption(Asura_PS2_RenderingOptions::OPTION_USE_GN_SMOOTH_SKIN_RENDERER) )
    {
		// Hilite blend for second pass of dredd character
		Asura_PlatformRenderStates::SetAlphaBlend( SCE_GS_ALPHA_CS, SCE_GS_ALPHA_ZERO, SCE_GS_ALPHA_AD, SCE_GS_ALPHA_CD, 0 );
		Asura_PlatformRenderStates::SetTextureClamp( true );
	}
	else
#endif
    if( 1.0f != fAlpha || Asura_Material_System::IsMaterialTransparent(iMaterialID) )
	{
        // Transparent material - setup test to avoid z buffer writes on transparent pixels
//		Asura_PlatformRenderStates::SetPixelTest( true, true, Asura_PlatformRenderStates::CONTEXT_1, 128, SCE_GS_ALPHA_GEQUAL, SCE_GS_AFAIL_FRAME );
		// To keep in line with the pc :o(
		Asura_PlatformRenderStates::SetPixelTest( true, true, 1 );
	}
    else
    {
		Asura_PlatformRenderStates::SetAlphaBlend( SCE_GS_ALPHA_ZERO, SCE_GS_ALPHA_ZERO, SCE_GS_ALPHA_FIX, SCE_GS_ALPHA_CS, 0 );
    }

    Asura_PlatformRenderStates::ApplyRenderStates();
    
    // upload texture here (assume just one texture)
    const int iTexID = Asura_Material_System::GetTextureID( iMaterialID );
    bool bTextureUploaded = Asura_PS2_TextureManagement::UploadTexture( iTexID, TEX_BFR_FOR_BASE );

#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
    InitDreddCharacterRenderTextures( iTexID, bTextureUploaded );
#else
    Asura_PS2_TextureManagement::SetupTextureEnvironment( bTextureUploaded,
                                                          ASURA_PS2_TEXREGFLAG_ALL,
                                                          iTexID,
                                                          TEX_BFR_FOR_BASE,
                                                          CHAR_MMAG,
                                                          CHAR_MMIN );
#endif

    bool  bShadow      = false;
    float fShadowAlpha = 0.0f;
    float fMinDistFromCam;

    if( !(uRenderFlags & ASURA_HIERARCHY_RENDER_NO_SHADOW)
        &&
        Asura_PS2_RenderingOptions::GetOption(Asura_PS2_RenderingOptions::OPTION_ENABLE_SMOOTH_SKIN_SHADOWS) )
    {
        fShadowAlpha = Asura_PS2_Shadows::CalcAlphaForMinDistFromCam( xBSCentre, fBSRadius, uCamVisFlags, &fMinDistFromCam );
        if( fShadowAlpha > 0.0f )
        {
            bShadow = true;
        }
    }

	// Set up the characters lights
    float fLightLevel;
    Asura_PS2_ProjectSpecific_SetupSmoothSkinLighting( pxAnim, xBSCentre, (bShadow ? &fLightLevel : NULL) );

    Asura_Vector_3 xShadowPos;
    if( bShadow )
    {
        fShadowAlpha *= fLightLevel; //Asura_PS2_Shadows::AdjustLightLevelForShadows( fLightLevel );

        xShadowPos.x = xBSCentre.x;
        xShadowPos.z = xBSCentre.z;
        
//		replaced call below because I don't want to modify alpha any more
//        Asura_PS2_Shadows::Proj_GetShadowYAndModifyAlphaForAnim( pxAnim, &xShadowPos.y, &fShadowAlpha );
        Asura_PS2_Shadows::Proj_GetShadowYAndHeightForAnim( pxAnim, &xShadowPos.y, &fShadowAlpha );

        xShadowPos.y += ASURA_PS2_SHADOW_OFFSET_Y;
    }

	// having decided that we need a shadow, we now need to ensure there is a buffer available !
	if ( bShadow )
	{
		// Setup shadow buffer for this anim
		Asura_PS2_ShadowBuffer* pxShadowBuffer = Asura_PS2_ShadowBufferSystem::GetShadowBufferForAnim(pxAnim, xBSCentre, fBSRadius);
		Asura_PS2_ShadowBuffer::s_pxCurrentShadowBuffer = pxShadowBuffer;

		// fail gracefully
		if (!pxShadowBuffer) 
		{ 
			bShadow = false; 
		}
		else
		{
			pxShadowBuffer->SetLightLevel(fShadowAlpha);
		}

	}

	// Set up the renderer
//#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
//  Asura_PS2_VU1::RENDERER eRenderer;
//  if( Asura_PS2_RenderingOptions::GetOption(Asura_PS2_RenderingOptions::OPTION_USE_GN_SMOOTH_SKIN_RENDERER) )
//  {
//		eRenderer = Asura_PS2_VU1::RENDERER_SKINNING_GRAPHIC_NOVEL;
//	}
//  else
//  {
//		eRenderer = Asura_PS2_VU1::RENDERER_SKINNING;
//	}
//#else
    Asura_PS2_VU1::RENDERER eRenderer = Asura_PS2_VU1::RENDERER_SKINNING;
//#endif

    Asura_PS2_VU1::LoadRenderer( eRenderer );

	// And the renderers data
    bool bGraphicsNovel;
//#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
//  bGraphicsNovel = Asura_PS2_RenderingOptions::GetOption( Asura_PS2_RenderingOptions::OPTION_USE_GN_SMOOTH_SKIN_RENDERER );
//#else
    bGraphicsNovel = false;
//#endif

    if( bShadow )
    {
//  	Asura_PS2_TextureManagement::InvalidateTextureBuffer( TEX_BFR_FOR_SHADOW );
//        Asura_PS2_Shadows::Proj_SetBfrParamsForBS( fBSRadius, fMinDistFromCam );
	    Asura_PS2_ShadowBuffer::s_pxCurrentShadowBuffer->JustSetupRenderToBfr( SCE_VIF1_SET_FLUSH(false) );
    }

    // (MP - 24.06.2004) Quick hackery to get old renderer processing new
    // data format, for debugging purposes. Obviously this will render
    // everything using a standard material but it's still helpful for
    // verification, etc.
    {
        const u_int uNumMaterials = pxSkinHdr->usNumMaterials;
        const int*  piMaterialIDs = reinterpret_cast<const int*>( pxSkinHdr + 1);

        const Asura_PS2_SmoothSkin_MaterialHdr* pxMaterialHdr = reinterpret_cast<const Asura_PS2_SmoothSkin_MaterialHdr*>( piMaterialIDs + uNumMaterials );

        u_int uChainAddr = reinterpret_cast<u_int>(pxSkinHdr) + (pxSkinHdr->usQWOffsetToFirstStripDataChain << 4);

        for( u_int u = uNumMaterials; u--; )
        {
            const Asura_PS2_SmoothSkin_SubElementHdr* pxSubElementHdr = reinterpret_cast<const Asura_PS2_SmoothSkin_SubElementHdr*>( pxMaterialHdr + 1 );
            for( u_int v = pxMaterialHdr->uNumElementsUsingThisMaterial; v--; )
            {
	            SkinAndRender_CreateDrawlist( uBoneQty,
                                              bGraphicsNovel,
                                              bShadow,
                                              uCamVisFlags,
                                              Asura_Maths::FloatToInt(128.0f * fAlpha),
                                              uChainAddr,
                                              xBSCentre, fBSRadius );

	            // Add VIF_FLUSH to wait for the last packet to draw
	            Asura_PS2_QW *pxFlush = Asura_PS2_DeferredRendering::GetVifBuffer(1);
	            pxFlush->u32s[0] = PY_DMA_CNT | 0;
	            pxFlush->u32s[1] = 0;
	            pxFlush->u32s[2] = SCE_VIF1_SET_FLUSH(0);
	            pxFlush->u32s[3] = SCE_VIF1_SET_NOP(0);

                uChainAddr += pxSubElementHdr->uByteOffsetToNextChain;
                ++pxSubElementHdr;
            }

            pxMaterialHdr = reinterpret_cast<const Asura_PS2_SmoothSkin_MaterialHdr*>( pxSubElementHdr );
        }
    }

#if 0
	// Draw the shadow that we have created, if necessary.
	if( bShadow )
    {
		SkinAndRender_DrawShadow( uBoneQty, xShadowPos, fBSRadius, fShadowAlpha, bGraphicsNovel, uCamVisFlags );
    }
#endif

	// release the shadow buffer if used
	Asura_PS2_ShadowBuffer::s_pxCurrentShadowBuffer = NULL;

//	ASURA_PS2_PROFILE__ADD_VIF_END( ASURA_PS2_PROFILE__SKINNED_MARKER );
}

// Setup shadow buffers for all the lights that are casting shadows on this character
// Returns the number of shadow casting light
void SkinAndRender_AddShadowData( const Asura_Vector_3& xBSCentre, float fBSRadius, Asura_PS2_QW* pxOut )
{
//  sceVu0FVECTOR xMeshCentre = { xBSCentre.x, xBSCentre.y, xBSCentre.z, 1.0f };
    sceVu0FVECTOR xLightPos   = { xBSCentre.x, xBSCentre.y + ASURA_PS2_SHADOW_CASTING_LIGHT_OFFSET_Y, xBSCentre.z, 1.0f };
    
//	// distant lights do not work well - the inaccuracies when creating the drop shadow
//  // matrix, and when calculating the size of the back buffer cause the shadow to dart 
//  // around, so I'm limiting shadow casting light distance to a shortish distance away.
//  sceVu0FVECTOR xObjectToLight;
//  sceVu0SubVector( xObjectToLight, xLightPos, xMeshCentre );
//  float fLightToObjectDistanceSqd = sceVu0InnerProduct( xObjectToLight, xObjectToLight );
//  if ( fLightToObjectDistanceSqd > dbg_MaxDistForShadowCastingLightSquared )
//  {
//      sceVu0Normalize( xObjectToLight, xObjectToLight );
//      sceVu0ScaleVector( xObjectToLight, xObjectToLight, dbg_MaxDistForShadowCastingLight );
//      sceVu0AddVector( xLightPos, xMeshCentre, xObjectToLight );
//  }

	Asura_PS2_QW* const pxBase = pxOut;

    // Used to put FRAME_2 here.
    (pxOut++)->u64s[1] = SCE_GS_NOP;

    if( Asura_Hierarchy_System::s_bBonesInWorldspace )
    {
		Asura_PS2_ShadowBuffer::s_pxCurrentShadowBuffer->Proj_GetShadowProjMat( NULL, &pxOut->f32s[0] );
    }
    else
    {
        sceVu0FMATRIX xO2W;
        Asura_PS2_VU1::SetupPS2MatFromAsuraData( &xO2W[0][0], *Asura_Hierarchy_System::s_pxOri, *Asura_Hierarchy_System::s_pxPos );
        Asura_PS2_ShadowBuffer::s_pxCurrentShadowBuffer->Proj_GetShadowProjMat( &xO2W[0][0], &pxOut->f32s[0] );
    }
    pxOut += 4;

    Asura_Assert( static_cast<u_int>(pxOut - pxBase) == SKINANDRENDER_DATA_PER_SHADOWCASTING_LIGHT, "" );
}

#if 0 // no longer rendering shadows as flat quads using main renderers
// Render the shadow buffer back into each of the cameras
static void SkinAndRender_DrawShadow( u_int uBoneQty,
                                      const Asura_Vector_3& xShadowPos, float fShadowSize,
                                      float fShadowAlpha,
									  bool bGraphicsNovel,
                                      u_int uCamVisFlags )
{
//  Asura_PS2_Shadows::Proj_PrepareBfrForUseAsTexture( SCE_VIF1_SET_NOP(false) );

    if( dbg_bDisplayCharShadowBufferRGB || dbg_bDisplayCharShadowBufferAlpha )
    {
        Asura_PS2_Shadows::Proj_ShowShadowBfr( 128, 128, 256, 256, dbg_bDisplayCharShadowBufferAlpha, true );
    }
    
	// Start up the quad renderer - it only need a GIF tag and the output buffer size
    u_int uQwords = 4;

	Asura_PS2_QW* const pxStartupBase = Asura_PS2_DeferredRendering::GetVifBuffer( uQwords );
	const u_int uDmaTag = PY_DMA_CNT;
	Asura_PS2_QW *pxOut = pxStartupBase;

	// Calculate output buffer Sizes
	const u_int uVertsPerBuffer = ASURA_PS2_MAX_VERTICES_PER_SMOOTH_SKIN_STRIP( uBoneQty );
	const u_int uOutputBufferSize = (1 + (uVertsPerBuffer * ASURA_PS2_SKINNING__QWS_PER_OUTPUT_VERTEX)) >> 1; // Output buffer size is 1/2 for quad renderer (double buffering takes place

	// DMA Tag
	pxOut->u32s[0] = uDmaTag | uQwords-1;
	pxOut->u32s[1] = 0;
	pxOut->u32s[2] = SCE_VIF1_SET_STCYCL(4, 4, 0);					// CL == WL, i.e. write 1 qword every cycle. 
	pxOut->u32s[3] = SCE_VIF1_SET_UNPACK( 0, 2, CMD_V4_32, 0 );		// Unpack the quads gif tag at 0 in vu memory
	pxOut++;

	// Setup GIF tag.
	pxOut->u64s[0] = SCE_GIF_SET_TAG( 1,                  // NLOOP 
									  1,                  // EOP (End Of Packet)
									  1,                  // PRE (PRIM field Enable)
									  SCE_GS_SET_PRIM( SCE_GS_PRIM_TRISTRIP,   // PRIM (Type of drawing primitive)
													   0,                      // IIP (Flat Shading/Gouraud Shading)
													   1,                      // TME (Texture Mapping Enabled)
													   0,                      // FGE (Fogging Enabled)
													   1,                      // ABE (Alpha Blending Enabled)
													   0,                      // AA1 (1 Pass Anti-Aliasing Enabled)
													   0,                      // FST (Method of specifying Texture Coords)
													   0,                      // CTXT (Context)
													   0 ),                    // FIX (Fragment Value Control)
									  0,                  // FLG (Data format, 0 = PACKED, 1 = REGLIST, 2 = IMAGE)
									  14 );                // NREG (Number of registers)
	pxOut->u32s[2] = (SCE_GIF_PACKED_AD<< 0) | (SCE_GIF_PACKED_AD<< 4) |		// Frame and Scissor
					 (SCE_GS_XYZ2<<16) | (SCE_GS_RGBAQ<<12) | (SCE_GS_ST<< 8) | 
					 (SCE_GS_XYZ2<<28) | (SCE_GS_RGBAQ<<24) | (SCE_GS_ST<<20);
	pxOut->u32s[3] = (SCE_GS_XYZ2<< 8) | (SCE_GS_RGBAQ<< 4) | (SCE_GS_ST<< 0) | 
					 (SCE_GS_XYZ2<<20) | (SCE_GS_RGBAQ<<16) | (SCE_GS_ST<<12);
	pxOut++;

	pxOut->u32s[0] = 0;
	pxOut->u32s[1] = 0;
	pxOut->u32s[2] = 0;
	pxOut->u32s[3] = uOutputBufferSize;
	pxOut++;

	// Fire it up
#ifdef ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
	if (bGraphicsNovel)
	{
		extern u_int VU1_Micro_Skinning_GraphicNovel_QuadRender[];
		pxOut->u32s[0] = SCE_VIF1_SET_MSCAL((reinterpret_cast<u_int>(VU1_Micro_Skinning_GraphicNovel_QuadRender)>>3), false );
	}
	else
#endif	//ASURA_PS2_SUPPORT_GRAPHIC_NOVEL
	{
		extern u_int VU1_Micro_Skinning_QuadRender[];
		pxOut->u32s[0] = SCE_VIF1_SET_MSCAL((reinterpret_cast<u_int>(VU1_Micro_Skinning_QuadRender)>>3), false );
	}

	pxOut->u32s[1] = 0;		// VIF_NOP
	pxOut->u32s[2] = 0;		// VIF_NOP
	pxOut->u32s[3] = 0;		// VIF_NOP
	pxOut++;

    const u_int uNumCamDataQWs = 9 * Asura_Camera::GetNumberOfCameras();

    // (MP - 31.07.2003) To deal with the fact that we may be working from
    // modelspace, we need to send new world->screen and world->clip matrices,
    // since the ones already on the VU may in fact be model->screen and
    // model->clip respectively. Just place these at the beginning of the
    // input buffer. While we're at it, send the RGBAs; these are constant
    // across the quad but the alpha changes per camera according to distance.
    // (MP - 18.02.2004) Actually, alpha is constant across all cameras now.
    // Easier to leave renderer as it is for the time being though.
    {
		Asura_PS2_QW* pxOut = Asura_PS2_DeferredRendering::GetVifBuffer( 1 + uNumCamDataQWs );

        (reinterpret_cast<Asura_PS2_DmaTag*>(pxOut++))->Set( uNumCamDataQWs,
                                                             Asura_PS2_DmaTag::PCE_NOCHANGE,
                                                             Asura_PS2_DmaTag::ID_CNT,
                                                             false,
                                                             0,
                                                             false,
                                                             SCE_VIF1_SET_NOP(false),
                                                             SCE_VIF1_SET_UNPACKR(0, uNumCamDataQWs, Asura_PS2_VU1::VIFPK_V4_32, false) );

        const u_int uShadowAlpha = Asura_Maths::FloatToInt( fShadowAlpha );
        for( int iCam = 0; iCam < Asura_Camera::GetNumberOfCameras(); ++iCam )
        {
            if( !(uCamVisFlags & (1 << iCam)) ) { continue; }

            const u_long128* puSrc;

            puSrc = reinterpret_cast<const u_long128*>( Asura_PS2_VU1::GetWorldToScreenMat(iCam) );
            for( u_int u = 4; u--; ) { (pxOut++)->u128 = *puSrc++; }

            puSrc = reinterpret_cast<const u_long128*>( Asura_PS2_VU1::GetWorldToClipMatGuardband(iCam) );
            for( u_int u = 4; u--; ) { (pxOut++)->u128 = *puSrc++; }

            // NB: RGB is irrelevant since our blend is Cd *= (1 - As).
            (pxOut++)->u32s[3] = uShadowAlpha;
        }
    }

    uQwords = 7;
    pxOut = Asura_PS2_DeferredRendering::GetVifBuffer( uQwords );

    (reinterpret_cast<Asura_PS2_DmaTag*>(pxOut++))->Set( uQwords - 1,
                                                         Asura_PS2_DmaTag::PCE_NOCHANGE,
                                                         Asura_PS2_DmaTag::ID_CNT,
                                                         false,
                                                         0,
                                                         false,
                                                         SCE_VIF1_SET_NOP(false),
                                                         SCE_VIF1_SET_DIRECT(uQwords - 1, false) );

    // Set GIF tag
    {
        Asura_PS2_GifTag* pxGifTag = reinterpret_cast<Asura_PS2_GifTag*>( pxOut++ );
        pxGifTag->SetExceptRegs( uQwords - 2, true, false, 0, SCE_GIF_PACKED, 1 );
        pxGifTag->uAllRegs = SCE_GIF_PACKED_AD;
    }
    // Texture setup.
    Asura_PS2_Shadows::Proj_GetTEX1andTEX0( &((pxOut + 0)->u64s[0]), &((pxOut + 1)->u64s[0]) );
    (pxOut++)->u64s[1] = SCE_GS_TEX1_1;
    (pxOut++)->u64s[1] = SCE_GS_TEX0_1;
    // Clamp to borders.
    pxOut->u64s[0] = SCE_GS_SET_CLAMP( SCE_GS_CLAMP, SCE_GS_CLAMP, 0, 0, 0, 0 );
    pxOut->u64s[1] = SCE_GS_CLAMP_1;
    ++pxOut;
    // Alpha blend: Cd *= (1 - As).
    pxOut->u64s[0] = SCE_GS_SET_ALPHA( SCE_GS_ALPHA_ZERO, SCE_GS_ALPHA_CD, SCE_GS_ALPHA_AS, SCE_GS_ALPHA_CD, 0 );
    pxOut->u64s[1] = SCE_GS_ALPHA_1;
    ++pxOut;
    // Test z but don't write.
    pxOut->u64s[0] = SCE_GS_SET_TEST( true, SCE_GS_ALPHA_NEVER, 0, SCE_GS_AFAIL_FB_ONLY, 0, 0, true, SCE_GS_ZGEQUAL );
    pxOut->u64s[1] = SCE_GS_TEST_1;
    ++pxOut;

	uQwords = 10;
	pxOut = Asura_PS2_DeferredRendering::GetVifBuffer( uQwords );

    (reinterpret_cast<Asura_PS2_DmaTag*>(pxOut++))->Set( uQwords - 1,
                                                         Asura_PS2_DmaTag::PCE_NOCHANGE,
                                                         Asura_PS2_DmaTag::ID_CNT,
                                                         false,
                                                         0,
                                                         false,
                                                         SCE_VIF1_SET_NOP(false),
                                                         SCE_VIF1_SET_UNPACKR(9 * ASURA_MAX_NO_OF_CAMERAS, 8, CMD_V4_32, false) );

    // TL
    pxOut->f32s[0] = 0.0f;
    pxOut->f32s[1] = 0.0f;
    pxOut->f32s[2] = 1.0f;
    ++pxOut;
    pxOut->f32s[0] = xShadowPos.x - fShadowSize;
    pxOut->f32s[1] = xShadowPos.y;
    pxOut->f32s[2] = xShadowPos.z - fShadowSize;
    ++pxOut;
    // TR
    pxOut->f32s[0] = 1.0f;
    pxOut->f32s[1] = 0.0f;
    pxOut->f32s[2] = 1.0f;
    ++pxOut;
    pxOut->f32s[0] = xShadowPos.x + fShadowSize;
    pxOut->f32s[1] = xShadowPos.y;
    pxOut->f32s[2] = xShadowPos.z - fShadowSize;
    ++pxOut;
    // BL
    pxOut->f32s[0] = 0.0f;
    pxOut->f32s[1] = 1.0f;
    pxOut->f32s[2] = 1.0f;
    ++pxOut;
    pxOut->f32s[0] = xShadowPos.x - fShadowSize;
    pxOut->f32s[1] = xShadowPos.y;
    pxOut->f32s[2] = xShadowPos.z + fShadowSize;
    ++pxOut;
    // BR
    pxOut->f32s[0] = 1.0f;
    pxOut->f32s[1] = 1.0f;
    pxOut->f32s[2] = 1.0f;
    ++pxOut;
    pxOut->f32s[0] = xShadowPos.x + fShadowSize;
    pxOut->f32s[1] = xShadowPos.y;
    pxOut->f32s[2] = xShadowPos.z + fShadowSize;
    ++pxOut;

    // Now finish up
    pxOut->u32s[0] = SCE_VIF1_SET_FLUSH( false );
    pxOut->u32s[1] = SCE_VIF1_SET_MSCNT( false );
    pxOut->u32s[2] = SCE_VIF1_SET_NOP( false );
    pxOut->u32s[3] = SCE_VIF1_SET_FLUSH( false );
    ++pxOut;

    Asura_PS2_Render::UpdateTriStatsFromVisFlags( 2, 1, uCamVisFlags );
}
#endif
