//////////////////////////////////////////////////////////////////////////////
//
// Asura_PlatformRenderGlass.cpp
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Engine independent.
#include "Camera/Asura_Camera.h"
#include "Entity/Glass/Asura_ClientEntity_Glass.h"
#include "Light/Asura_Photon.h"
#include "SpecialFX/Asura_Shard_Effect.h"
#include "SpecialFX/Asura_Shard_System.h"

// Engine platform-specific.
#include "Core/Asura_PS2_DMA.h"
#include "Rendering/Asura_PlatformRendering.h"
#include "Rendering/Asura_PlatformRenderStates.h"
#include "Rendering/Asura_PlatformTextureManagement.h"
#include "Rendering/Asura_PS2_DeferredRendering.h"
#include "Rendering/Asura_PS2_GlassDefines.h"
#include "Rendering/Asura_PS2_GS.h"
#include "Rendering/Asura_PS2_RenderingOptions.h"
#include "Rendering/Asura_PS2_VU1.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int MAX_LIGHTS       = 4;
static const float TEX_MATRIX_SCALE = 0.25f;

// G L O B A L S /////////////////////////////////////////////////////////////

extern u_int VU1_GlassRender_BeginBatch[] __attribute__(( section(".vudata") ));

// P R O T O T Y P E S   A N D   I N L I N E   F U N C T I O N S /////////////

void PlatformGlass_AddLightData( const Asura_Bounding_Box& xBoundingBox, Asura_PS2_QW* pxOut );

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// Asura_Shard_Effect.
//

void Asura_Shard_Effect::Platform_Render(unsigned uCamVisFlags)
{
#if 0

    if( !Asura_PS2_RenderingOptions::GetOption(Asura_PS2_RenderingOptions::OPTION_DRAW_GLASS) ) { return; }

    Asura_PS2_VU1::LoadRenderer( Asura_PS2_VU1::RENDERER_GLASS );

    const bool bTexturedGlass = ( ASURA_TEXTURE_NONE != m_iTextureIndex );
    if( bTexturedGlass )
    {
        // The glass is textured, so setup the buffer layout
        Asura_PS2_TextureManagement::SetBufferLayout( ASURA_PS2_TEXBFRLAYOUT_2_LARGE );

        // Upload the texture
        bool bTextureUploaded = Asura_PS2_TextureManagement::UploadTexture( m_iTextureIndex );
        
        // And setup the texture environment
        Asura_PS2_TextureManagement::SetupTextureEnvironment( bTextureUploaded,
                                                              ASURA_PS2_TEXREGFLAG_ALL,
                                                              m_iTextureIndex,
                                                              Asura_PS2_TextureManagement::BFR_AUTO,
                                                              SCE_GS_LINEAR,
                                                              SCE_GS_LINEAR_MIPMAP_NEAREST );
    }

    const u_int uGlassColour = (m_uColour & 0x00FFFFFFu) | (Asura_Maths::FloatToInt((m_uColour >> 24) * (1.0f - m_fAlpha)) << 24);

    Asura_PlatformRenderStates::CreateDefaultRenderStatePacket( Asura_PlatformRenderStates::CONTEXT_1 );
	// (AW - 21.05.2004) - Texture clamping disabled.
    //Asura_PlatformRenderStates::SetTextureClamp( true );
    Asura_PlatformRenderStates::SetZBufferWriteEnabled( false );
    Asura_PlatformRenderStates::ApplyRenderStates();

    // How many batches of shards are we going to up load ?
    const u_int uBatchCount = (m_iNumberOfActiveShards + (ASURA_PS2_GLASS_MAX_TRIS_PER_BATCH - 1)) / ASURA_PS2_GLASS_MAX_TRIS_PER_BATCH;

    u_int uNumActiveCams = 0;
    for( int iCam = 0; iCam < Asura_Camera::GetNumberOfCameras(); ++iCam )
    {
        if( uCamVisFlags & (1 << iCam) ) { ++uNumActiveCams; }
    }

    const u_int uSetupQWs = 1                                                 // GIF tag.
                          + uNumActiveCams * ASURA_PS2_GLASS_NUM_QWS_PER_CAM; // Cam data.

    const u_int uPerBatchOverhead = 1  // UNPACK at start and then ITOP, MSCAL and either NOP or FLUSH at the end.
                                  + 1; // Drawing GIF tag

    const u_int uTotalDmaQWs = uSetupQWs
                             + 1                                                          // MSCAL for setup microprogram and input buffer setup.
                             + 1 + ASURA_PS2_GLASS_NUM_QWS_LIGHT_DATA                     // Lighting data (and its UNPACK.)
                             + uBatchCount * uPerBatchOverhead
                             + m_iNumberOfActiveShards * ASURA_PS2_GLASS_NUM_QWS_PER_TRI; // Shard data.

    const u_int uTotalQWsOfVifBuffer = 1 + uTotalDmaQWs;

    Asura_PS2_QW* const pxBase = Asura_PS2_DeferredRendering::GetVifBuffer( uTotalQWsOfVifBuffer );
    Asura_PS2_QW* pxOut = pxBase;

    (reinterpret_cast<Asura_PS2_DmaTag*>(pxOut++))->Set( uTotalDmaQWs,
                                                         Asura_PS2_DmaTag::PCE_NOCHANGE,
                                                         Asura_PS2_DmaTag::ID_CNT,
                                                         false,
                                                         0,
                                                         false,
                                                         SCE_VIF1_SET_STCYCL(1, 1, false),
                                                         SCE_VIF1_SET_UNPACK(0, uSetupQWs, CMD_V4_32, 0) );

    (reinterpret_cast<Asura_PS2_GifTag*>(pxOut++))->Set( 2,                   // NLOOP
                                                         false,               // EOP
                                                         false,               // PRE
                                                         0,                   // PRIM
                                                         SCE_GIF_PACKED,      // FLG
                                                         1,                   // NREG
                                                         SCE_GIF_PACKED_AD ); // REG0

    for( int iCam = 0; iCam < Asura_Camera::GetNumberOfCameras(); ++iCam )
    {
        if( !(uCamVisFlags & (1 << iCam)) ) { continue; }

        Asura_Camera::SetCameraIndex( iCam );

        sceVu0CopyMatrix( &pxOut->Vu0FVec, Asura_PS2_VU1::GetWorldToScreenMat() );
        pxOut += 4;
        // Since we're not clipping, we can send the guardband version of the
        // world->clip matrix. It's possible larger shards will occasionally
        // need clipping for absolute correctness; using the larger clipping
        // bounds here will make that a bit less obvious.
        sceVu0CopyMatrix( &pxOut->Vu0FVec, Asura_PS2_VU1::GetWorldToClipMatGuardband() );
        pxOut += 4;
        // And we can just skip writing the clip->screen matrix.
        pxOut += 4;

        pxOut = Asura_PS2_GS::GetViewportSetupGifCmds( iCam, Asura_PS2_GS::CONTEXT_1, pxOut );

        const Asura_Vector_3* pxCamPos = Asura_Camera::GetPosition();
        pxOut->f32s[0] = pxCamPos->x;
        pxOut->f32s[1] = pxCamPos->y;
        pxOut->f32s[2] = pxCamPos->z;
        ++pxOut;

        if( IsUsingEnvironmentMap() )
        {
            const Asura_Matrix_3x3* pxCamOri = Asura_Camera::GetOrientation();
            
            pxOut->f32s[0] = pxCamOri->e11 * TEX_MATRIX_SCALE;
            pxOut->f32s[1] = pxCamOri->e12;
            pxOut->f32s[2] = pxCamOri->e13;
            ++pxOut;
            
            pxOut->f32s[0] = pxCamOri->e21;
            pxOut->f32s[1] = pxCamOri->e22 * TEX_MATRIX_SCALE;
            pxOut->f32s[2] = pxCamOri->e23;
            ++pxOut;
        }
        else
        {
            (pxOut++)->u128 = 0;
            (pxOut++)->u128 = 0;
        }
    }

    // Unpack for light data. Just stick it in input buffer 0.
    for( u_int u = 0; u < 3; ++u ) { pxOut->u32s[u] = SCE_VIF1_SET_NOP( false ); }
    (pxOut++)->u32s[3] = SCE_VIF1_SET_UNPACK( ASURA_PS2_GLASS_ADDR_INPUT_BUFFER_0, ASURA_PS2_GLASS_NUM_QWS_LIGHT_DATA, Asura_PS2_VU1::VIFPK_V4_32, false );

    // Leave space for the light data to be added at the end.
    Asura_PS2_QW* const pxLightData = pxOut;
    pxOut += ASURA_PS2_GLASS_NUM_QWS_LIGHT_DATA;

    // Set the camera quantity, start the setup microcode and setup the
    // input buffer layout.
    pxOut->u32s[0] = SCE_VIF1_SET_ITOP( uNumActiveCams, false );
    pxOut->u32s[1] = SCE_VIF1_SET_MSCAL( 0, false );
    pxOut->u32s[2] = SCE_VIF1_SET_BASE( ASURA_PS2_GLASS_ADDR_INPUT_BUFFER_0, false );
    pxOut->u32s[3] = SCE_VIF1_SET_OFFSET( ASURA_PS2_GLASS_SIZE_INPUT_BUFFER, false );
    ++pxOut;

    // Make the batches equal size
    u_int uShardsPerBatch = m_iNumberOfActiveShards / uBatchCount;

    // Integer division rounds down so it's possible we won't get all the
    // shards uploaded in the expect number of batches. Solution: increase
    // the shards-per-batch count by 1.
    if( uShardsPerBatch < ASURA_PS2_GLASS_MAX_TRIS_PER_BATCH )
    {
        ++uShardsPerBatch;
    }

    // Keep track of how many shards remain.
    u_int uShardsRemaining = m_iNumberOfActiveShards;

    Asura_Bounding_Box xShardLightingBB;
    xShardLightingBB.Reset();

    Asura_PS2_GifTag xDrawGIFTag;

    {
        const u_int uPRIM = SCE_GS_SET_PRIM( SCE_GS_PRIM_TRISTRIP, // PRIM
                                             1,                    // IIP
                                             bTexturedGlass,       // TME
                                             false,                // FGE
                                             true,                 // ABE
                                             false,                // AA1
                                             0,                    // FST
                                             0,                    // CTXT
                                             0 );                  // FIX

        xDrawGIFTag.Set( 0,              // NLOOP (setup per batch below)
                         true,           // EOP
                         true,           // PRE
                         uPRIM,          // PRIM
                         SCE_GIF_PACKED, // FLG
                         9,              // NREG
                         SCE_GS_ST,      // REG0
                         SCE_GS_RGBAQ,   // REG1
                         SCE_GS_XYZ2,    // REG2
                         SCE_GS_ST,      // REG3
                         SCE_GS_RGBAQ,   // REG4
                         SCE_GS_XYZ2,    // REG5
                         SCE_GS_ST,      // REG6
                         SCE_GS_RGBAQ,   // REG7
                         SCE_GS_XYZ2 );  // REG8
    }

    const Asura_Shard* const * ppxShards = m_apxShards;
    while( uShardsRemaining )
    {
        // Work out how many shards we are sending in this batch.
        u_int uShardsInThisBatch = ( uShardsRemaining < ASURA_PS2_GLASS_MAX_TRIS_PER_BATCH ? uShardsRemaining : uShardsPerBatch );
        uShardsRemaining -= uShardsInThisBatch;

        const u_int uUnpackQWs = 1 + (ASURA_PS2_GLASS_NUM_QWS_PER_TRI * uShardsInThisBatch);
        pxOut->u32s[0] = SCE_VIF1_SET_UNPACKR( 0, uUnpackQWs, CMD_V4_32, false ); 

        xDrawGIFTag.uNLOOP = uShardsInThisBatch;
        
        {
            const u_int* puSrc  = reinterpret_cast<const u_int*>( &xDrawGIFTag );
            for( u_int u = 1; u < 5; ++u ) { pxOut->u32s[u] = *puSrc++; }
            ++pxOut;
        }

        // Add each shard.
        for( u_int u = uShardsInThisBatch; u--; )
        {
            const Asura_Shard* pxShard = *ppxShards++;

            // Calculate a normal for this shard.
            Asura_Vector_3 xNorm = (pxShard->m_axVertices[1] - pxShard->m_axVertices[0]) ^ (pxShard->m_axVertices[2] - pxShard->m_axVertices[0]);
            xNorm.Normalise();

            pxOut->f32s[1] = xNorm.x;
            pxOut->f32s[2] = xNorm.y;
            pxOut->f32s[3] = xNorm.z;
            ++pxOut;
            pxOut->f32s[0] = 1.0f; // Normal's w is specular scale.
            
            const float fColourRange = 0.5f / 255.0f;
            pxOut->f32s[1] = ((uGlassColour >> 16) & 0xFF) * fColourRange; // R
            pxOut->f32s[2] = ((uGlassColour >>  8) & 0xFF) * fColourRange; // G
            pxOut->f32s[3] = ((uGlassColour >>  0) & 0xFF) * fColourRange; // B
            ++pxOut;
            pxOut->f32s[0] = ((uGlassColour >> 24) & 0xFF) * fColourRange; // A
            
            // Add this shard to the BB. Note that we're only adding 1 vertex
            // from each shard but this should be okay provided glass shards
            // aren't too large.
            xShardLightingBB.Set( pxShard->m_axVertices[0] );

            for( u_int uVtx = 0; uVtx < 3; ++uVtx )
            {
                pxOut->f32s[1] = pxShard->m_axVertices[uVtx].x;
                pxOut->f32s[2] = pxShard->m_axVertices[uVtx].y;
                pxOut->f32s[3] = pxShard->m_axVertices[uVtx].z;
                ++pxOut;
                pxOut->u32s[0] = ( 2 == uVtx ? 0 : 0x8000 );
                pxOut->f32s[1] = pxShard->m_afU[uVtx];
                pxOut->f32s[2] = pxShard->m_afV[uVtx];
                pxOut->f32s[3] = 1.0f;
                ++pxOut;
            }
        }
        
        pxOut->u32s[1] = SCE_VIF1_SET_ITOP( 3 | ASURA_PS2_GLASS_FLAG_NO_CLIPPING_REQD, false );
        pxOut->u32s[2] = SCE_VIF1_SET_MSCAL( reinterpret_cast<u_int>(VU1_GlassRender_BeginBatch) >> 3, false );

        // Flush after the final batch to make sure it's finished rendering.
        pxOut->u32s[3] = ( uShardsRemaining ? SCE_VIF1_SET_NOP(false) : SCE_VIF1_SET_FLUSH(false) );
        ++pxOut;
    }

    // Assert here rather than after filling in the light data since the ptrs
    // will be the same and this will keep us in scope.
    Asura_Assert( static_cast<u_int>(pxOut - pxBase) == uTotalQWsOfVifBuffer, "" );

    // Finally fill in the light data.
    PlatformGlass_AddLightData( xShardLightingBB, pxLightData );

    Asura_PS2_Render::UpdateTriStatsFromVisFlags( m_iNumberOfActiveShards, 1, uCamVisFlags );

#endif
}

//
// Asura_ClientEntity_Glass.
//

void Asura_ClientEntity_Glass::Platform_Render(unsigned uCamVisFlags)
{
#if 0

    if( !Asura_PS2_RenderingOptions::GetOption(Asura_PS2_RenderingOptions::OPTION_DRAW_GLASS) ) { return; }

    Asura_PS2_VU1::LoadRenderer( Asura_PS2_VU1::RENDERER_GLASS );

    const bool bTexturedGlass = ( ASURA_TEXTURE_NONE != m_iTextureIndex );
    if( bTexturedGlass )
    {
        // The glass is textured, so setup the buffer layout
        Asura_PS2_TextureManagement::SetBufferLayout( ASURA_PS2_TEXBFRLAYOUT_2_LARGE );

        // Upload the texture
        bool bTextureUploaded = Asura_PS2_TextureManagement::UploadTexture( m_iTextureIndex );
        
        // And setup the texture environment
        Asura_PS2_TextureManagement::SetupTextureEnvironment( bTextureUploaded,
                                                              ASURA_PS2_TEXREGFLAG_ALL,
                                                              m_iTextureIndex,
                                                              Asura_PS2_TextureManagement::BFR_AUTO,
                                                              SCE_GS_LINEAR,
                                                              SCE_GS_LINEAR_MIPMAP_NEAREST );
    }

    Asura_PlatformRenderStates::CreateDefaultRenderStatePacket( Asura_PlatformRenderStates::CONTEXT_1 );
	// (AW - 21.05.2004) - Texture clamping disabled.
    //Asura_PlatformRenderStates::SetTextureClamp( true );
    Asura_PlatformRenderStates::SetZBufferWriteEnabled( false );
    Asura_PlatformRenderStates::ApplyRenderStates();

    u_int uNumActiveCams = 0;
    for( int iCam = 0; iCam < Asura_Camera::GetNumberOfCameras(); ++iCam )
    {
        if( uCamVisFlags & (1 << iCam) ) { ++uNumActiveCams; }
    }

    const u_int uSetupQWs = 1                                                 // GIF tag.
                          + uNumActiveCams * ASURA_PS2_GLASS_NUM_QWS_PER_CAM; // Cam data.

    const u_int uPaneDataQWs = 1      // Drawing GIF tag.
                             + 2      // Normal and RGBA for pane.
                             + 2 * 4; // XYZ and STQ per vertex.

    const u_int uTotalDmaQWs = uSetupQWs
                             + 1 + ASURA_PS2_GLASS_NUM_QWS_LIGHT_DATA // Lighting data (and its UNPACK.)
                             + 1                                      // MSCAL for setup microprogram and input buffer setup.
                             + 1 + uPaneDataQWs                       // Pane data plus extra QW for UNPACK at start and then MSCAL, etc. at the end.
                             + 1 + 1;                                 // UNPACK and clipped strip GIF tag.

    const u_int uTotalQWsOfVifBuffer = 1 + uTotalDmaQWs;

    Asura_PS2_QW* const pxBase = Asura_PS2_DeferredRendering::GetVifBuffer( uTotalQWsOfVifBuffer );
    Asura_PS2_QW* pxOut = pxBase;

    (reinterpret_cast<Asura_PS2_DmaTag*>(pxOut++))->Set( uTotalDmaQWs,
                                                         Asura_PS2_DmaTag::PCE_NOCHANGE,
                                                         Asura_PS2_DmaTag::ID_CNT,
                                                         false,
                                                         0,
                                                         false,
                                                         SCE_VIF1_SET_STCYCL(1, 1, false),
                                                         SCE_VIF1_SET_UNPACK(0, uSetupQWs, CMD_V4_32, 0) );

    (reinterpret_cast<Asura_PS2_GifTag*>(pxOut++))->Set( 2,                   // NLOOP
                                                         false,               // EOP
                                                         false,               // PRE
                                                         0,                   // PRIM
                                                         SCE_GIF_PACKED,      // FLG
                                                         1,                   // NREG
                                                         SCE_GIF_PACKED_AD ); // REG0

    for( int iCam = 0; iCam < Asura_Camera::GetNumberOfCameras(); ++iCam )
    {
        if( !(uCamVisFlags & (1 << iCam)) ) { continue; }

        Asura_Camera::SetCameraIndex( iCam );

        sceVu0CopyMatrix( &pxOut->Vu0FVec, Asura_PS2_VU1::GetWorldToScreenMat() );
        pxOut += 4;
        sceVu0CopyMatrix( &pxOut->Vu0FVec, Asura_PS2_VU1::GetWorldToClipMatScreen() );
        pxOut += 4;
        sceVu0CopyMatrix( &pxOut->Vu0FVec, Asura_PS2_VU1::GetClipToScreenMat() );
        pxOut += 4;

        pxOut = Asura_PS2_GS::GetViewportSetupGifCmds( iCam, Asura_PS2_GS::CONTEXT_1, pxOut );

        const Asura_Vector_3* pxCamPos = Asura_Camera::GetPosition();
        pxOut->f32s[0] = pxCamPos->x;
        pxOut->f32s[1] = pxCamPos->y;
        pxOut->f32s[2] = pxCamPos->z;
        ++pxOut;

        if( m_bUseEnvironmentMap )
        {
            Asura_Matrix_3x3 xMat = *Asura_Camera::GetOrientation();
            xMat.Transpose();

            xMat = xMat * m_xPane.Orientation;
            pxOut->f32s[0] = xMat.e11 * TEX_MATRIX_SCALE;
            pxOut->f32s[1] = xMat.e21;
            pxOut->f32s[2] = xMat.e31;
            ++pxOut;
            
            pxOut->f32s[0] = xMat.e12;
            pxOut->f32s[1] = xMat.e22 * TEX_MATRIX_SCALE;
            pxOut->f32s[2] = xMat.e32;
            ++pxOut;
        }
        else
        {
            (pxOut++)->u128 = 0;
            (pxOut++)->u128 = 0;
        }
    }

    // Unpack for light data. Just stick it in input buffer 0.
    for( u_int u = 0; u < 3; ++u ) { pxOut->u32s[u] = SCE_VIF1_SET_NOP( false ); }
    (pxOut++)->u32s[3] = SCE_VIF1_SET_UNPACK( ASURA_PS2_GLASS_ADDR_INPUT_BUFFER_0, ASURA_PS2_GLASS_NUM_QWS_LIGHT_DATA, Asura_PS2_VU1::VIFPK_V4_32, false );

    // Leave space for the light data to be added at the end.
    Asura_PS2_QW* const pxLightData = pxOut;
    pxOut += ASURA_PS2_GLASS_NUM_QWS_LIGHT_DATA;

    // Set the camera quantity, start the setup microcode and setup the
    // input buffer layout.
    pxOut->u32s[0] = SCE_VIF1_SET_ITOP( uNumActiveCams, false );
    pxOut->u32s[1] = SCE_VIF1_SET_MSCAL( 0, false );
    pxOut->u32s[2] = SCE_VIF1_SET_BASE( ASURA_PS2_GLASS_ADDR_INPUT_BUFFER_0, false );
    pxOut->u32s[3] = SCE_VIF1_SET_OFFSET( ASURA_PS2_GLASS_SIZE_INPUT_BUFFER, false );
    ++pxOut;

    Asura_Bounding_Box xLightingBB;
    xLightingBB.Reset();

    pxOut->u32s[0] = SCE_VIF1_SET_UNPACKR( 0, uPaneDataQWs, CMD_V4_32, false ); 

    // Drawing GIF tag.
    {
        Asura_PS2_GifTag xDrawGIFTag;

        const u_int uPRIM = SCE_GS_SET_PRIM( SCE_GS_PRIM_TRISTRIP, // PRIM
                                             1,                    // IIP
                                             bTexturedGlass,       // TME
                                             false,                // FGE
                                             true,                 // ABE
                                             false,                // AA1
                                             0,                    // FST
                                             0,                    // CTXT
                                             0 );                  // FIX

        xDrawGIFTag.Set( 1,              // NLOOP
                         true,           // EOP
                         true,           // PRE
                         uPRIM,          // PRIM
                         SCE_GIF_PACKED, // FLG
                         12,             // NREG
                         SCE_GS_ST,      // REG0
                         SCE_GS_RGBAQ,   // REG1
                         SCE_GS_XYZ2,    // REG2
                         SCE_GS_ST,      // REG3
                         SCE_GS_RGBAQ,   // REG4
                         SCE_GS_XYZ2,    // REG5
                         SCE_GS_ST,      // REG6
                         SCE_GS_RGBAQ,   // REG7
                         SCE_GS_XYZ2,    // REG8
                         SCE_GS_ST,      // REG9
                         SCE_GS_RGBAQ,   // REG10
                         SCE_GS_XYZ2 );  // REG11

        const u_int* puSrc  = reinterpret_cast<const u_int*>( &xDrawGIFTag );
        for( u_int u = 1; u < 5; ++u ) { pxOut->u32s[u] = *puSrc++; }
        ++pxOut;
    }

    // Normal.
    pxOut->f32s[1] = m_xPane.Orientation.e13;
    pxOut->f32s[2] = m_xPane.Orientation.e23;
    pxOut->f32s[3] = m_xPane.Orientation.e33;
    ++pxOut;
    pxOut->f32s[0] = 1.0f; // Normal's w is specular scale.
            
    const float fColourRange = 0.5f / 255.0f;
    pxOut->f32s[1] = ((m_uColour >> 16) & 0xFF) * fColourRange; // R
    pxOut->f32s[2] = ((m_uColour >>  8) & 0xFF) * fColourRange; // G
    pxOut->f32s[3] = ((m_uColour >>  0) & 0xFF) * fColourRange; // B
    ++pxOut;
    pxOut->f32s[0] = ((m_uColour >> 24) & 0xFF) * fColourRange; // A

    // (MP - 15.07.2003) NB: Currently CalculateCorners() generates vertices
    // in the correct order for a tristrip. I'm assuming it'll stay that way.
	Asura_Vector_3 axCorners[4];
	m_xPane.CalculateCorners( axCorners );

    for( u_int uCorner = 0; uCorner < 4; ++uCorner )
    {
        pxOut->f32s[1] = axCorners[uCorner].x;
        pxOut->f32s[2] = axCorners[uCorner].y;
        pxOut->f32s[3] = axCorners[uCorner].z;
        ++pxOut;
        pxOut->u32s[0] = ( uCorner < 2 ? 0x8000 : 0 );
        pxOut->f32s[1] = m_afU[uCorner];
        pxOut->f32s[2] = m_afV[uCorner];
        pxOut->f32s[3] = 1.0f;
        ++pxOut;

        xLightingBB.Set( axCorners[uCorner] );
    }
    
    pxOut->u32s[1] = SCE_VIF1_SET_UNPACK( ASURA_PS2_GLASS_ADDR_CLIPPED_STRIP_GIF_TAG, 1, Asura_PS2_VU1::VIFPK_V4_32, false );

    {
        const u_int uPRIM = SCE_GS_SET_PRIM( SCE_GS_PRIM_TRISTRIP, // PRIM
                                             1,                    // IIP
                                             bTexturedGlass,       // TME
                                             false,                // FGE
                                             true,                 // ABE
                                             false,                // AA1
                                             0,                    // FST
                                             0,                    // CTXT
                                             0 );                  // FIX

        Asura_PS2_GifTag xClipDrawGIFTag;
        xClipDrawGIFTag.Set( 0,              // NLOOP (setup in microcode)
                             true,           // EOP
                             true,           // PRE
                             uPRIM,          // PRIM
                             SCE_GIF_PACKED, // FLG
                             3,              // NREG
                             SCE_GS_ST,      // REG0
                             SCE_GS_RGBAQ,   // REG1
                             SCE_GS_XYZ2 );  // REG2

        const u_int* puSrc = reinterpret_cast<const u_int*>( &xClipDrawGIFTag );
        for( u_int u = 2; u < 6; ++u ) { pxOut->u32s[u] = *puSrc++; }
        ++pxOut;
    }
    
    pxOut->u32s[2] = SCE_VIF1_SET_ITOP( 4, false );
    pxOut->u32s[3] = SCE_VIF1_SET_MSCAL( reinterpret_cast<u_int>(VU1_GlassRender_BeginBatch) >> 3, false );
    ++pxOut;
    pxOut->u32s[0] = SCE_VIF1_SET_FLUSH( false );
    for( u_int u = 1; u < 4; ++u ) { pxOut->u32s[u] = SCE_VIF1_SET_NOP( false ); }
    ++pxOut;

    // Assert here rather than after filling in the light data since the ptrs
    // will be the same and this will keep us in scope.
    Asura_Assert( static_cast<u_int>(pxOut - pxBase) == uTotalQWsOfVifBuffer, "" );

    // Finally fill in the light data.
    PlatformGlass_AddLightData( xLightingBB, pxLightData );

    Asura_PS2_Render::UpdateTriStatsFromVisFlags( 2, 1, uCamVisFlags );

#endif
}

// F U N C T I O N   I M P L E M E N T A T I O N S ///////////////////////////

void PlatformGlass_AddLightData( const Asura_Bounding_Box &xBoundingBox, Asura_PS2_QW* const pxOut )
{
#if 0

    Asura_Assert( false, "TODO - update this renderer for new lighting system" );
    
    const Asura_Vector_3 xCentre = xBoundingBox.CalculateCentre();
    Asura_Photon_System::MakePhotonListForBoundingBox( xBoundingBox, xCentre, true, true, MAX_LIGHTS );

    const u_int uLightQty = Asura_Photon_System::NumberOfLocalPhotons;
    Asura_Assert( uLightQty <= MAX_LIGHTS, "" );

    // Now loop over the lights and fill in the required data.
    for( u_int u = 0; u < uLightQty; ++u )
    {
        const Asura_Local_Photon& xLocalPhoton = Asura_Photon_System::LocalPhotons[u];

        Asura_Vector_3 xLightVector = xCentre - xLocalPhoton.GetPosition();
        float fDist = xLightVector.Magnitude();
        xLightVector /= fDist;

        pxOut[0].f32s[u] = xLightVector.x;
        pxOut[1].f32s[u] = xLightVector.y;
        pxOut[2].f32s[u] = xLightVector.z;

        const float fBrightness = Asura_Maths::Max( 0.0f, 1.0f - (fDist / xLocalPhoton.GetRange()) );
        pxOut[3 + u].f32s[0] = xLocalPhoton.GetRedComponent()   * fBrightness;
        pxOut[3 + u].f32s[1] = xLocalPhoton.GetGreenComponent() * fBrightness;
        pxOut[3 + u].f32s[2] = xLocalPhoton.GetBlueComponent()  * fBrightness;
        pxOut[3 + u].f32s[3] = 0.0f;
    }

    // If we have less than max lights then just set the spare ones to black.
    for( u_int u = uLightQty; u < MAX_LIGHTS; ++u )
    {
        pxOut[0].f32s[u] = 0.0f;
        pxOut[1].f32s[u] = 0.0f;
        pxOut[2].f32s[u] = 0.0f;

        pxOut[3 + u].u128 = 0;
    }

    Asura_Assert( 3 + MAX_LIGHTS == ASURA_PS2_GLASS_NUM_QWS_LIGHT_DATA, "" );

#endif
}
