//////////////////////////////////////////////////////////////////////////////
//
// Asura_PlatformRendering.h
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PLATFORMRENDERING_H___
#define ___ASURA_PLATFORMRENDERING_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Render.h"

// M A C R O S ///////////////////////////////////////////////////////////////

#define ASURA_PS2_COLLECT_RENDER_STATS         (!defined(ASURA_PS2_SONY_SUBMISSION))
#define ASURA_PS2_COLLECT_DETAILED_LIGHT_STATS (0) //(ASURA_PS2_COLLECT_RENDER_STATS)

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PS2_Render
{

    public:

        // T Y P E S /////////////////////////////////////////////////////////

        enum TRICOUNT
        {
            TRICOUNT_ENV,
            TRICOUNT_HIER,
            TRICOUNT_OBJ,
            TRICOUNT_SMOOTH_SKIN,
            TRICOUNT_WATER,
			TRICOUNT_SHADOW,
			TRICOUNT_BOTANICALS,
			TRICOUNT_CARPET,
            TRICOUNT_NUM_TYPES,
        };

        // C L A S S   M E T H O D S /////////////////////////////////////////

        static void ResetForNewFrame( void );

        static void UpdateLightStats( u_int uNumLights, bool bDirectional, u_int uNumTrisAffected )
        {
#if ASURA_PS2_COLLECT_DETAILED_LIGHT_STATS
            (bDirectional ? s_auTrisPerDirLightCount : s_auTrisPerPointLightCount)[uNumLights] += uNumTrisAffected;
#endif
        }

        static void UpdateTriStats( u_int uNumTris, u_int uNumPasses )
        {
#if ASURA_PS2_COLLECT_RENDER_STATS
            Asura_Render::AddToTrianglesRendered( uNumTris, uNumPasses );
#endif
        }

        static void UpdateTriStats( u_int uNumTris, u_int uNumPasses, TRICOUNT eTriCount )
        {
#if ASURA_PS2_COLLECT_RENDER_STATS
            Asura_Render::AddToTrianglesRendered( uNumTris, uNumPasses );
            s_auTriPassCounts[eTriCount] += uNumTris * uNumPasses;
#endif
        }

        static void UpdateTriStatsFromVisFlags( u_int uNumTrisPerCam, u_int uNumPasses, u_int uCamVisFlags )
        {
#if ASURA_PS2_COLLECT_RENDER_STATS
            u_int uNumTris = 0;
            for( u_int u = ASURA_MAX_NO_OF_CAMERAS; u; )
            {
                if( uCamVisFlags & (1 << --u) ) { uNumTris += uNumTrisPerCam; }
            }

            UpdateTriStats( uNumTris, uNumPasses );
#endif
        }

#if ASURA_PS2_COLLECT_RENDER_STATS
        static u_int GetTriPassCount( TRICOUNT eTriCount ) { return s_auTriPassCounts[eTriCount]; }

        static void SetNoClipModulesRatio( float fRatio ) { s_fNoClipModulesRatio = fRatio; }
        static float GetNoClipModulesRatio( void ) { return s_fNoClipModulesRatio; }
#endif

    private:
        
        // C L A S S   S T A T E /////////////////////////////////////////////

#if ASURA_PS2_COLLECT_RENDER_STATS
        static u_int s_auTriPassCounts[TRICOUNT_NUM_TYPES];
        static float s_fNoClipModulesRatio;
#endif

#if ASURA_PS2_COLLECT_DETAILED_LIGHT_STATS
        static u_int s_auTrisPerDirLightCount[3];
        static u_int s_auTrisPerPointLightCount[8];
#endif
        
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PLATFORMRENDERING_H___
