//////////////////////////////////////////////////////////////////////////////
//
// VU1_Coronas.vcl
//
// (MP - 17.05.2003) Rewritten for new corona system, including optimisation
// and a bunch of work to bring it up to date with how other renderers work.
// Obviously still heavily based on MH's original microcode, which was in
// turn heavily based on VU1_ParticleRenderer.vcl
//
// See Asura_PS2_CoronaDefines.h for memory layout, etc.
//
//////////////////////////////////////////////////////////////////////////////

// V C L   S E T U P /////////////////////////////////////////////////////////

.init_vf_all
.init_vi_all

.syntax new

--enter
--endenter

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Asura_PS2_CoronaDefines.h"
#include "vcl_sml.i"

// M A C R O S ///////////////////////////////////////////////////////////////

.macro SwitchOutputBuffer pCurDblBfr
	IADDIU iTmp, VI00, ASURA_PS2_CORONA_OUTPUT_BUFFERS_SIZE
    ISUBIU \pCurDblBfr, \pCurDblBfr, ASURA_PS2_CORONA_OUTPUT_BUFFERS_BASE
    ISUB   \pCurDblBfr, iTmp, \pCurDblBfr
    IADDIU \pCurDblBfr, \pCurDblBfr, ASURA_PS2_CORONA_OUTPUT_BUFFERS_BASE
.endm

// M I C R O C O D E /////////////////////////////////////////////////////////

INITIALISE_VU_DATA:

    FCSET 0

    // Setup a couple of useful registers. Firstly, we need 0x8000 in an
    // int register for setting ADC bits.
	IADDIU i0x8000, VI00,    0x7fff
	IADDIU i0x8000, i0x8000, 1
    // Also, having 1 in an int register is useful for masking later.
    IADDIU iOne, VI00, 1

    // For now, we use the entirety of each corona texture so just setup UVs
    // here for them all.
	ADD.xy   fTLUV, VF00, VF00
	ADDw.z   fTLUV, VF00, VF00
	ADD.y    fTRUV, VF00, VF00
	ADDw.xz  fTRUV, VF00, VF00
	ADD.x    fBLUV, VF00, VF00
	ADDw.yz  fBLUV, VF00, VF00
	ADDw.xyz fBRUV, VF00, VF00

INITIALISE_CAMERAS:
    
    IADDIU pCurOutputBfr, VI00,          ASURA_PS2_CORONA_OUTPUT_BUFFERS_BASE
    IADDIU pOutput,       pCurOutputBfr, 0

    // Get camera count.
	XITOP iNumCams
    
    // Our GIF tags will (temporarily) be in the beginning of the input area.
    IADDIU pGIFTags, VI00, ASURA_PS2_CORONA_INPUT_BUFFERS_BASE
    // Read them in.
    LQ fViewportGIFTag, 0(pGIFTags)
    LQ fQuadGIFTag,     1(pGIFTags)
    // And copy to both output buffers.
    SQ fViewportGIFTag, 0(pOutput)
    SQ fViewportGIFTag, (ASURA_PS2_CORONA_OUTPUT_BUFFERS_SIZE)(pOutput)
    SQ fQuadGIFTag,     3(pOutput)
    SQ fQuadGIFTag,     (ASURA_PS2_CORONA_OUTPUT_BUFFERS_SIZE + 3)(pOutput)

    --cont

PROCESS_BATCH:

    XITOP iNumCoronas

INITIALISE_INPUT_PROCESSING:

    IADDIU pCamData,  VI00, ASURA_PS2_CORONA_CAM_DATA_BASE
    IADD   iCamsLeft, VI00, iNumCams

    // We start our mask at 2 rather than 1 since bit 0 is used for the
    // should-fade-out-with-distance flag.
    LOI 2
    ADDi.x fCamNotVisFlagMask, VF00, I

CAMERA_LOOP:

    // Reset input ptr for this camera.
    XTOP pInput

	// Copy mask for this camera from FP reg.
    FTOI0.x fCamNotVisFlagMask, fCamNotVisFlagMask
	MTIR    iCamNotVisFlagMask, fCamNotVisFlagMask[x]
    ITOF0.x fCamNotVisFlagMask, fCamNotVisFlagMask

	// Get FP reg ready for next one.
    LOI 2
    MULi.x fCamNotVisFlagMask, fCamNotVisFlagMask, I

    IADDIU iNumCoronasLeft, iNumCoronas, 0

    // Load this camera's data.
    LQ         fCamViewportSetting1, 0(pCamData)
    LQ         fCamViewportSetting2, 1(pCamData)
    MatrixLoad mW2S,                 2, pCamData
    MatrixLoad mW2C,                 6, pCamData
    LQ.xyz     fCamPos,              10(pCamData)
    LQ.xyz     fCoronaAxisX,         11(pCamData)
    LQ.xyz     fCoronaAxisY,         12(pCamData)

    // Copy viewport setup QWs to output.
    SQ fCamViewportSetting1, 1(pOutput)
    SQ fCamViewportSetting2, 2(pOutput)
    
    // Move output ptr to after second GIF tag.
    IADDIU pOutput, pOutput, (1 + ASURA_PS2_CORONA_NUM_QWS_PER_VIEWPORT + 1)

    // Slightly quicker way of applying deltas...
    SUB.xyz fCoronaDeltaTL, VF00,           fCoronaAxisX
    SUB.xyz fCoronaDeltaTL, fCoronaDeltaTL, fCoronaAxisY
    ADD.xyz fCoronaDeltaTR, VF00,           fCoronaAxisX
    SUB.xyz fCoronaDeltaTR, fCoronaDeltaTR, fCoronaAxisY

CORONA_LOOP:

	--LoopCS 1, 0

    // This tells VCL the minimum number of cycles we expect to see in the
    // main loop. This speeds up compile time since it then doesn't bother
    // trying to find solutions with less cycles. This needs to be kept up to
    // date with changes to the code below; if you modify anything, comment
    // this out and see what the loop actually comes out to.
    --LoopAbs 51

    // Load worldspace position and the corona size (which will be in w) and
    // also its RGBA and some flags. Note that we hardcode alpha to 128 so
    // there's no need to upload that.
    LQ fWSPosAndSize,       0(pInput)
    LQ fRGBAndVariousFlags, 1(pInput)

    // We'll need those flags in an int register.
    MTIR iCamNotVisAndFadeoutFlags, fRGBAndVariousFlags[w]

    // Direction effect. We'll calculate the colour scaling factor anyway, to
    // avoid a branch, and then multiply by the w component of this register.
    // This is assumed to contain 128.0 if the dir effect should be used and
    // 0.0 otherwise.
    LQ fDirAndUseDirFlag, 2(pInput)
    // Get corona->cam dir and distance.
    SUB.xyz  fToCam, fCamPos, fWSPosAndSize
    ELENG    P, fToCam
    MFP.w    fToCamLen, P
    ERCPR    P, fToCamLen[w] // NB: Using DIV instead here produces a longer main loop, despite ERCPR being slower.
    MFP.y    fOOToCamLen, P
    MULy.xyz fToCam, fToCam, fOOToCamLen[y]
    // Dot with corona dir.
    MUL.xyz fToCamDotDir, fToCam, fDirAndUseDirFlag
    ADDy.x  fToCamDotDir, fToCamDotDir, fToCamDotDir[y]
    ADDz.x  fToCamDotDir, fToCamDotDir, fToCamDotDir[z]
    // Clamp to [0, ...)
    MAX.x   fToCamDotDir, fToCamDotDir, VF00
    // Calculate colour scale. The PC uses (191 * dot + 64) (post-clamping)
    // so we'll use half that.
    LOI     95.5
    MULi.x  fColourScale, fToCamDotDir, I
    LOI     32
    ADDi.x  fColourScale, fColourScale, I
    // Use either this factor or 128, if we're not using the direction effect.
    MAXw.x  fColourScale, fColourScale, fDirAndUseDirFlag[w]
    // Modulate RGB.
    MULx.xyz fRGBAOut, fRGBAndVariousFlags, fColourScale[x]
    // Better clamp to 255, I guess.
    LOI 255
    MINi.xyz fRGBAOut, fRGBAOut, I

    // We may also want to scale alpha according to distance.
    LOI ASURA_PS2_CORONA_FADE_DIST_BEGIN
    SUBi.w fAlphaScale, fToCamLen, I
    // Clamp to [0, ...)
    MAXx.w fAlphaScale, fAlphaScale, VF00[x]
    // Mask off everything except for the should-fade-with-distance flag.
    IAND iFadeWithDistFlag, iCamNotVisAndFadeoutFlags, iOne
    // That will now be either 0 or 1. Move this to a FP register. Stick it
    // in the z component since we're not using that much for scalar values
    // and it'll probably help with register pressure.
    MFIR.z  fFadeWithDistMul, iFadeWithDistFlag
    ITOF0.z fFadeWithDistMul, fFadeWithDistMul
    // Scale is then (1 - fademul * (dist - begin) / (end - begin)). We
    // assume dist is < end here since otherwise we'll have set the not vis
    // flag for this camera on the CPU and we'll set ADC bits below anyway.
    LOI (1.0 / (ASURA_PS2_CORONA_FADE_DIST_END - ASURA_PS2_CORONA_FADE_DIST_BEGIN))
    MULi.w fAlphaScale, fAlphaScale, I
    MULz.w fAlphaScale, fAlphaScale, fFadeWithDistMul[z]
    SUB.w fAlphaScale, VF00, fAlphaScale
    // Multiply by 128 to get into GS range and copy to output register.
    // (No need to clamp.)
    LOI 128
    MULi.w fRGBAOut, fAlphaScale, I

	// Convert RGBA and copy to output.
	FTOI0 fRGBAOut, fRGBAOut
	SQ    fRGBAOut, 0(pOutput)

	// Copy STQs to output.
	SQ.xyz fTLUV, 1(pOutput)
    SQ.xyz fTRUV, 3(pOutput)
    SQ.xyz fBLUV, 5(pOutput)
    SQ.xyz fBRUV, 7(pOutput)

    ADDA.xyz ACC, fWSPosAndSize, VF00
    // 0 - TL
    MADDw.xyz fWSV0, fCoronaDeltaTL, fWSPosAndSize[w]
    // 3 - BR
    MSUBw.xyz fWSV3, fCoronaDeltaTL, fWSPosAndSize[w]

    // NB: We only clip the TL and BR vertices since we're working with
    // camera-aligned billboards. Similarly, we calculate the TR and BL
    // vertex positions by permuting the TL and BR vertices' x and y
    // components.

    // 0 - TL.
    // Clip.
    MatrixMultiplyVertexW1 fCSPos, mW2C, fWSV0
    CLIPw.xyz              fCSPos, fCSPos[w]
    // Perspective-project and convert for GS.
    MatrixMultiplyVertexW1 fTLSSPos, mW2S, fWSV0
    VertexPersCorr         fTLSSPos, fTLSSPos
    // First two vertices need ADC bit set to prevent the GS trying to
    // continue a previous trifan / tristrip.
	MFIR.w fTLSSPos, i0x8000
    // Convert and copy to output.
    FTOI4.xyz fTLSSPos, fTLSSPos
    SQ        fTLSSPos, 2(pOutput)

    // 3 - BR.
    // Clip.
    MatrixMultiplyVertexW1 fCSPos, mW2C, fWSV3
    CLIPw.xyz              fCSPos, fCSPos[w]
    // Perspective-project and convert for GS.
    MatrixMultiplyVertexW1 fBRSSPos, mW2S, fWSV3
    VertexPersCorr         fBRSSPos, fBRSSPos
    // Convert and copy to output.
    FTOI4.xyz fBRSSPos, fBRSSPos
    SQ.xyz    fBRSSPos, 8(pOutput) // NB: We'll sort out w below.

    // 1 - TR.
    // Calculate SS pos.
    MOVE.xz fTRSSPos, fBRSSPos
    MOVE.yw fTRSSPos, fTLSSPos // This will copy ADC bit in w as well.
    // Copy to output.
    SQ fTRSSPos, 4(pOutput)

    // 2 - BL.
    // Calculate SS pos.
    MOVE.x  fBLSSPos, fTLSSPos
    MOVE.yz fBLSSPos, fBRSSPos
    // Copy to output.
    SQ.xyz fBLSSPos, 6(pOutput) // NB: We'll sort out w below.

    // Apply not-vis flag mask for this camera.
    IAND iNotVisFlag, iCamNotVisAndFadeoutFlags, iCamNotVisFlagMask

    // Check clip flags for TL and BR vertices and set ADC for remaining
    // vertices of the quad if any bits are high.
    FCAND VI01, 0xFFF
    // OR on the not vis flag for this camera as well.
    IOR iCull, VI01, iNotVisFlag
    // Calculate value for w component. The iCull register will now be non-
    // zero iff either any vertices are outside the guardband or the corona
    // isn't visible to this camera. Adding a non-zero value to 0x7FFF will
    // set the ADC bit (provided the value isn't too large to wrap but we
    // know it will be no greater than 8.)
    IADDIU iADC, iCull, 0x7FFF
    // Bear in mind that we don't care about the lower bits of w since we're
    // not doing vertex fog, only whether or not bit 15 is set, so we can
    // now copy that straight into the last two vertices' w components.
    ISW.w iADC, 6(pOutput)w
    ISW.w iADC, 8(pOutput)w

    // Update output ptr.
    IADDIU pOutput, pOutput, ASURA_PS2_CORONA_NUM_QWS_PER_OUTPUT_CORONA

    // Update input ptr.
    IADDIU pInput, pInput, ASURA_PS2_CORONA_NUM_QWS_PER_INPUT_CORONA

    // Any coronas left?
    IADDI iNumCoronasLeft, iNumCoronasLeft, -1
    IBNE  iNumCoronasLeft, VI00, CORONA_LOOP

    // Fix up NLOOP in the GIF tag.
    IOR iEOPAndNLOOP, i0x8000, iNumCoronas
    // Copy to output.
    ISW.x iEOPAndNLOOP, (1 + ASURA_PS2_CORONA_NUM_QWS_PER_VIEWPORT)(pCurOutputBfr)

    // Send this batch to the GS.
    XGKICK pCurOutputBfr

    // Switch output buffer.
    SwitchOutputBuffer pCurOutputBfr
    IADDIU pOutput, pCurOutputBfr, 0

    // Move on to next camera's data.
    IADDIU pCamData, pCamData, ASURA_PS2_CORONA_NUM_QWS_PER_CAM

    // Any cameras left?
    IADDI iCamsLeft, iCamsLeft, -1
    IBNE  iCamsLeft, VI00, CAMERA_LOOP

    --cont
    B PROCESS_BATCH

.end
