//////////////////////////////////////////////////////////////////////////////
//
// VU1_GlassRender.vcl
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// Description:
//
//     See Asura_PS2_GlassDefines.h for memory layout details.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "vcl_sml.i"
#include "Asura_PS2_GlassDefines.h"

// M A C R O S ///////////////////////////////////////////////////////////////

// Which planes need clipping against. These match the VU1 clipping bits.
#define CLIP_PLANE_RIGHT  (0x001)
#define CLIP_PLANE_LEFT   (0x002)
#define CLIP_PLANE_BOTTOM (0x004)
#define CLIP_PLANE_TOP    (0x008)
#define CLIP_PLANE_FAR    (0x010)
#define CLIP_PLANE_NEAR   (0x020)

.macro SpecularLighting fRGBAOut, fXYZ, fCamPos, fDiffuse, fRGBA, fReflectionXs, fReflectionYs, fReflectionZs, fNormal
    // Calculate Camera Normal...
    SUB.xyz     fCameraNormal, \fXYZ, \fCamPos
    ERLENG      P, fCameraNormal        // This should be done in parallel and the final dot product scaled back
    MFP.w       fCameraNormal, P
    MULw.xyz    fCameraNormal, fCameraNormal, fCameraNormal
    // Camera Normal dot Reflection
    MULAx.xyzw  ACC,      \fReflectionXs, fCameraNormal
    MADDAy.xyzw ACC,      \fReflectionYs, fCameraNormal
    MADDz.xyzw  fVdotE,   \fReflectionZs, fCameraNormal
    // Clamp to [0, 1].
    MAXx.xyzw   fVdotE,   fVdotE, VF00
    // Scale up the speculars; ^16 should do.
    MUL.xyzw    fVdotE,   fVdotE,        fVdotE        // ^2
    MUL.xyzw    fVdotE,   fVdotE,        fVdotE        // ^4
    MUL.xyzw    fVdotE,   fVdotE,        fVdotE        // ^8
    MUL.xyzw    fVdotE,   fVdotE,        fVdotE        // ^16
    MULw.xyzw   fVdotE,   fVdotE,        \fNormal
    // Add specular-ness to diffuse. Note that we deliberately modify alpha
    // here as well so that the glass becomes more opaque the nearer the view
    // vector gets to its plane.
    ADDx.xyzw   fLight,   \fDiffuse,     fVdotE
    ADDy.xyzw   fLight,   fLight,        fVdotE
    ADDz.xyzw   fLight,   fLight,        fVdotE
    ADDw.xyzw   fLight,   fLight,        fVdotE
    // Scale colour by light values.
    MUL.xyzw \fRGBAOut, fLight, \fRGBA
    // Scale colour and clamp to [0, 255].
    LOI 255.0
    MULi.xyzw  \fRGBAOut, \fRGBAOut, I
    MINIi.xyzw \fRGBAOut, \fRGBAOut, I
.endm

.macro SwitchOutputBuffers iCurBfr
    ISUBIU iOffset,       \iCurBfr,      ASURA_PS2_GLASS_ADDR_OUTPUT_BUFFER_0
    IADDIU iSizeOfBuffer, VI00,          ASURA_PS2_GLASS_SIZE_OUTPUT_BUFFER
    ISUB   iOffset,       iSizeOfBuffer, iOffset
    IADDIU \iCurBfr,      iOffset,       ASURA_PS2_GLASS_ADDR_OUTPUT_BUFFER_0
.endm

// V C L   S E T U P /////////////////////////////////////////////////////////

.syntax new

.init_vf_all
.init_vi_all

.global VU1_GlassRender_BeginBatch

// C O D E ///////////////////////////////////////////////////////////////////

START:

    --enter
    --endenter

    XITOP iNumCams
    // Setup the output buffer pointers.
    IADDIU  pThisOutputBuffer, VI00, ASURA_PS2_GLASS_ADDR_OUTPUT_BUFFER_0
    IADDIU  pNextOutputBuffer, VI00, ASURA_PS2_GLASS_ADDR_OUTPUT_BUFFER_1
    // Load lighting data for this batch. This will just get crammed into 
    // input buffer 0.
    IADDIU pLightDataIn, VI00, ASURA_PS2_GLASS_ADDR_INPUT_BUFFER_0
    LQ     fLightNormalXs, 0(pLightDataIn)
    LQ     fLightNormalYs, 1(pLightDataIn)
    LQ     fLightNormalZs, 2(pLightDataIn)
    LQ     fLightColour1,  3(pLightDataIn) // NB: We only use xyz for these colour
    LQ     fLightColour2,  4(pLightDataIn) // registers but we need to load in all
    LQ     fLightColour3,  5(pLightDataIn) // four components since otherwise VCL
    LQ     fLightColour4,  6(pLightDataIn) // complains about the out_vfs below.
    // Copy GS register setup GIF tag to output buffers. It'll remain
    // unchanged for the duration of this renderer.
    LQ fRegGIFTag, ASURA_PS2_GLASS_ADDR_CAM_SETUP_GIF_TAG(VI00)
    SQ fRegGIFTag, 0(pThisOutputBuffer)
    SQ fRegGIFTag, 0(pNextOutputBuffer)

    --exit
    out_vi pThisOutputBuffer(VI02)
    out_vi iNumCams(VI03)

    out_vf fLightNormalXs(VF01)
    out_vf fLightNormalYs(VF02)
    out_vf fLightNormalZs(VF03)
    out_vf fLightColour1(VF04)
    out_vf fLightColour2(VF05)
    out_vf fLightColour3(VF06)
    out_vf fLightColour4(VF07)
    --endexit

VU1_GlassRender_BeginBatch:

    --enter
    in_vi pThisOutputBuffer(VI02)
    in_vi iNumCams(VI03)

    in_vf fLightNormalXs(VF01)
    in_vf fLightNormalYs(VF02)
    in_vf fLightNormalZs(VF03)
    in_vf fLightColour1(VF04)
    in_vf fLightColour2(VF05)
    in_vf fLightColour3(VF06)
    in_vf fLightColour4(VF07)
    --endenter

    XTOP  pInputBuffer
    // Shut VCL up...
    FCSET 0
    // Setup camera loop state.
    IADD   iNumCamsLeft, VI00, iNumCams
    IADDIU pCamData,     VI00, ASURA_PS2_GLASS_ADDR_CAM_DATA

    CAM_LOOP:

        --LoopCS 1, 0

        // Load cam data.
        MatrixLoad mW2S, 0, pCamData
        MatrixLoad mW2C, 4, pCamData
        // And GS register values.
        LQ fSCISSOR,  12(pCamData)
        LQ fXYOFFSET, 13(pCamData)
        // And the position and texture matrices.
        LQ.xyz fCamPos,  14(pCamData)
        LQ.xyz fTexMat0, 15(pCamData)
        LQ.xyz fTexMat1, 16(pCamData)
        // And the GIF tag. This is obviously constant but we're running
        // short on registers
        LQ fDrawingGIFTag, 0(pInputBuffer)
        // Copy GS register values and drawing GIF tag to output buffer.
        SQ fSCISSOR,       1(pThisOutputBuffer)
        SQ fXYOFFSET,      2(pThisOutputBuffer)
        SQ fDrawingGIFTag, 3(pThisOutputBuffer)
        // Read in poly count. Obviously this is constant across all cameras
        // but we're short on registers.
        ILW.x iNumPolysLeft, 0(pInputBuffer)x
        // Mask off EOP bit.
        IADDIU iNLOOPMask,     VI00,          0x7FFF
        IAND   iNumPolysLeft,  iNumPolysLeft, iNLOOPMask
        // Setup other stuff for loop.
        IADDIU pIn,           pInputBuffer,      1
        IADDIU pOut,          pThisOutputBuffer, 4
        IADDIU pClipOut,      VI00,              ASURA_PS2_GLASS_ADDR_CLIPPER_INPUT_BUFFER
        IADDIU iAnyClipped,   VI00,              0

        POLY_LOOP:

            --LoopCS    1,0

            // Load the normal and colour.
            LQ fPolyNormal, 0(pIn)
            LQ fPolyRGBA,   1(pIn)
            // Calculate L.N (for all 4 lights simultaneously.)
            MULAx.xyzw  ACC,    fLightNormalXs, fPolyNormal
            MADDAy.xyzw ACC,    fLightNormalYs, fPolyNormal
            MADDz.xyzw  fLdotN, fLightNormalZs, fPolyNormal
            // Calculate 2(L.N) for specular
            ADD.xyzw f2LdotN, fLdotN, fLdotN
            // Get absolute value of L.N becuase glass is two-sided and so
            // should be lit from either side
            ABS.xyzw    fLdotN,        fLdotN
            // Calculate the light reflection vectors: 2(L.N)*N - L.
            MULx.xyzw fReflectionXs, f2LdotN,       fPolyNormal
            MULy.xyzw fReflectionYs, f2LdotN,       fPolyNormal
            MULz.xyzw fReflectionZs, f2LdotN,       fPolyNormal
            SUB.xyzw  fReflectionXs, fReflectionXs, fLightNormalXs
            SUB.xyzw  fReflectionYs, fReflectionYs, fLightNormalYs
            SUB.xyzw  fReflectionZs, fReflectionZs, fLightNormalZs
            // Diffuse.
            SUBAx.xyz  ACC,      VF00,          VF00
            MADDAx.xyz ACC,      fLightColour1, fLdotN
            MADDAy.xyz ACC,      fLightColour2, fLdotN
            MADDAz.xyz ACC,      fLightColour3, fLdotN
            MADDw.xyz  fDiffuse, fLightColour4, fLdotN
            // Fix its alpha to 1.
            ADDx.w fDiffuse, VF00, VF00
            // Calculate UV offset for environment mapping. Note that if
            // environment mapping is switched off then this will just
            // evaluate to (0, 0).
            MUL.xyz fSOffset,  fTexMat0, fPolyNormal
            ADDz.x  fSOffset,  fSOffset, fSOffset
            ADDy.x  fSTOffset, fSOffset, fSOffset
            MUL.xyz fTOffset,  fTexMat1, fPolyNormal
            ADDz.y  fTOffset,  fTOffset, fTOffset
            ADDx.y  fSTOffset, fTOffset, fTOffset
            // Setup stuff for vertex loop.
            IADDIU pIn, pIn, 2
            // Extract poly-per-vertex value. This is constant for the batch
            // but we're running short on registers.
            XITOP iXITOP
            IADDIU iMask, VI00, ASURA_PS2_GLASS_MASK_VERTICES_PER_POLY
            IAND iNumVertsLeft, iXITOP, iMask
            
            VERTEX_LOOP:

                --LoopCS 3, 0

                // Load vertex data.
                LQ.xyz fXYZ, 0(pIn)
                ILW.w  iADC, 0(pIn)
                LQ.xyz fSTQ, 1(pIn)
                // Evaluate lighting.
                SpecularLighting fRGBA, fXYZ, fCamPos, fDiffuse, fPolyRGBA, fReflectionXs, fReflectionYs, fReflectionZs, fPolyNormal
                // Apply ST offset.
                ADD.xy fSTQ, fSTQ, fSTOffset
                // Transform to screenspace.
                MatrixMultiplyVertexW1 fScrnXYZ, mW2S, fXYZ
                // Perspective divide.
                DIV      Q,        VF00[w],  fScrnXYZ[w]
                MULq.xyz fScrnXYZ, fScrnXYZ, Q
                MULq.xyz fSTQOut,  fSTQ,     Q
                // Convert position and RGBA to GS format.
                FTOI4.xyz fGSScrnXYZ, fScrnXYZ
                FTOI0     fGSRGBA,    fRGBA
                // Transform to clip space.
                MatrixMultiplyVertexW1 fClipXYZ, mW2C, fXYZ
                // Apply clipping judgement.
                CLIPw.xyz fClipXYZ,  fClipXYZ
                FCGET     iClipFlags
                FCAND     VI01,      0x3FFFF
                // Keep a record of whether or not we clipped any vertices.
                IOR iAnyClipped, iAnyClipped, VI01
                // Construct clipping ADC flag and then OR with input ADC.
                IADDIU iClipADC, VI01, 0x7FFF
                IOR    iADCOut,  iADC, iClipADC
                // Output vertex.
                SQ.xyz fSTQOut,    0(pOut) 
                SQ     fGSRGBA,    1(pOut)
                SQ.xyz fGSScrnXYZ, 2(pOut)
                ISW.w  iADCOut,    2(pOut)w
                // Store some information in case we end up clipping a tri
                // using this vertex.
                SQ     fClipXYZ,   0(pClipOut)
                SQ.xy  fSTQ,       1(pClipOut)
                ISW.z  iClipFlags, 1(pClipOut)z
                ISW.w  iADC,       1(pClipOut)w
                SQ     fRGBA,      2(pClipOut)
                // Update loop state.
                IADDIU pIn,           pIn,           2
                IADDIU pOut,          pOut,          3
                IADDIU pClipOut,      pClipOut,      3
                ISUBIU iNumVertsLeft, iNumVertsLeft, 1
                // Are we done yet?
                IBNE iNumVertsLeft, VI00, VERTEX_LOOP

            // Update loop state.
            IADDI iNumPolysLeft, iNumPolysLeft, -1
            // Are we done yet?
            IBNE iNumPolysLeft, VI00, POLY_LOOP

        // Ok, send that all off to the GS.
        XGKICK pThisOutputBuffer
        // Any polys clipped?
        IBEQ   iAnyClipped, VI00,   SKIP_CLIPPING
        // Clipping required at all?
        IADDIU iNoClipFlag, VI00,   ASURA_PS2_GLASS_FLAG_NO_CLIPPING_REQD
        XITOP  iXITOP
        IAND   iNoClipFlag, iXITOP, iNoClipFlag
        IBNE   iNoClipFlag, VI00,   SKIP_CLIPPING
        // Ok, guess we need to clip. First we need to record the number of
        // vertices in this batch in a register.
        ILW.x  iNLOOP, 3(pThisOutputBuffer)
        // Mask off EOP bit.
        IADDIU iNLOOPMask, VI00,   0x7FFF
        IAND   iNLOOP,     iNLOOP, iNLOOPMask
        // Extract poly-per-vertex value. This is constant for the batch
        // but we're running short on integer registers.
        XITOP iXITOP
        IADDIU iMask, VI00, ASURA_PS2_GLASS_MASK_VERTICES_PER_POLY
        IAND iNumVerticesPerPoly, iXITOP, iMask
        // The NLOOP value is actually the number of polys so we need to
        // multiply by the number of vertices per poly.
        IADDIU iNumVertices, VI00,                0
        IADDIU iNumAddsLeft, iNumVerticesPerPoly, 0
        VERTEX_COUNT_MULTIPLY_LOOP:
            IADD   iNumVertices, iNumVertices, iNLOOP
            ISUBIU iNumAddsLeft, iNumAddsLeft, 1
            IBNE   iNumAddsLeft, VI00,         VERTEX_COUNT_MULTIPLY_LOOP
        // Switch output buffers and jump to clipping code.
        SwitchOutputBuffers pThisOutputBuffer
        B CLIP_TRIANGLES

    SKIP_CLIPPING:

        SwitchOutputBuffers pThisOutputBuffer
        B END_OF_CAM_LOOP
            
    DONE_CLIPPING:

        // Anything in buffer that needs rendering? (NB: Could kick anyway
        // but want to avoid swapping output buffers if possible.)
        ILW.x  iNLOOP,     (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 3)(pThisOutputBuffer)
        IADDIU iNLOOPMask, VI00,   0x7FFF
        IAND   iNLOOP,     iNLOOP, iNLOOPMask
        IBEQ   iNLOOP,     VI00,   END_OF_CAM_LOOP
        // Ok, we have something to render.
        XGKICK              pThisOutputBuffer
        SwitchOutputBuffers pThisOutputBuffer

    END_OF_CAM_LOOP:

        // Update loop state.
        IADDIU pCamData,     pCamData,     ASURA_PS2_GLASS_NUM_QWS_PER_CAM
        ISUBIU iNumCamsLeft, iNumCamsLeft, 1
        // Are we done yet?
        IBNE iNumCamsLeft, VI00, CAM_LOOP

FINISHED_BATCH:

    // (MP - 15.07.2003) NB: Don't do --cont here. If we do, the --exit stuff
    // below gets ignored. There's no need to anyway since we always just
    // MSCAL from VU1_GlassRender_BeginBatch above.

    --exit
    out_vi pThisOutputBuffer(VI02)
    out_vi iNumCams(VI03)

    out_vf fLightNormalXs(VF01)
    out_vf fLightNormalYs(VF02)
    out_vf fLightNormalZs(VF03)
    out_vf fLightColour1(VF04)
    out_vf fLightColour2(VF05)
    out_vf fLightColour3(VF06)
    out_vf fLightColour4(VF07)
    --endexit

CLIP_TRIANGLES:

    --barrier

    // Load GS register settings and GIF tag.
    LQ fSCISSOR,            12(pCamData)
    LQ fXYOFFSET,           13(pCamData)
    LQ fClippedStripGIFTag, ASURA_PS2_GLASS_ADDR_CLIPPED_STRIP_GIF_TAG(VI00)
    // And copy to output buffer.
    SQ fSCISSOR,            (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 1)(pThisOutputBuffer)
    SQ fXYOFFSET,           (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 2)(pThisOutputBuffer)
    SQ fClippedStripGIFTag, (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 3)(pThisOutputBuffer)
    // Setup the output pointers, where the final strips go.
    IADDIU pOut, pThisOutputBuffer, (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 4)
	// Setup input ptr and clip mask.
    IADDIU pIn,       VI00, ASURA_PS2_GLASS_ADDR_CLIPPER_INPUT_BUFFER
    IADDIU iClipMask, VI00, 0x3F
	// Get clipping results for vertex 0.
	LQ.xyzw		vClipPos,		0(pIn)
	CLIPw.xyz	vClipPos,		vClipPos
	ILW.z		iClipFlag2,		1(pIn)
	IAND		iClipFlag2,		iClipFlag2,		iClipMask
	// And for vertex 1.
	LQ.xyzw		vClipPos,		3(pIn)
	CLIPw.xyz	vClipPos,		vClipPos
	ILW.z		iClipFlag1,		4(pIn)
	IAND		iClipFlag1,		iClipFlag1,		iClipMask
	// Calculate half of first clipping test
	IAND		iClipAll21,		iClipFlag2,		iClipFlag1
    // Jump to vertex 2.
    IADDIU pIn,          pIn,          6
    ISUBIU iNumVertices, iNumVertices, 2

FIND_CLIP_TRIANGLE_LOOP:

        --LoopCS 1, 0
            
        IBEQ iNumVertices, VI00, DONE_CLIPPING
	    // Get clipping results for vertex N
	    LQ.xyzw		vClipPos,	0(pIn)
	    ILW.z		iClipFlag0,	1(pIn)
	    IAND		iClipFlag0,	iClipFlag0,	iClipMask
	    // Calculate second half of clipping test
	    IAND		iClipAll10,	iClipFlag1,	iClipFlag0
	    IAND		iClipAll,	iClipAll21,	iClipAll10
	    // Check for none clipped
	    CLIPw.xyz	vClipPos,	vClipPos
	    // Move clipping flags/result along 1
	    IADD		iClipFlag1,	VI00,		iClipFlag0
	    IADD		iClipAll21,	VI00,		iClipAll10
        // Update loop state.
        IADDIU pIn,          pIn,          3
        ISUBIU iNumVertices, iNumVertices, 1
        // Check if we found a triangle that need clipping
	    FCAND		VI01,		0x3FFFF
	    IBEQ		VI01,		VI00,	FIND_CLIP_TRIANGLE_LOOP	// The triangle is complete unclipped
	    IBNE		iClipAll,	VI00,	FIND_CLIP_TRIANGLE_LOOP	// The triangle is complete clipped to at least one plane
	    // Check for degenracy
	    ILW.w		iDegenerate, -2(pIn)
	    IBNE		iDegenerate, VI00,	FIND_CLIP_TRIANGLE_LOOP	// Degenerate vert.. if this triangle needs clipping it'll be picked up later

    // This triangle needs clipping. Store some int registers in a float
    // register to help VCL out with allocation.
    MFIR.x fStack0, pIn
    MFIR.y fStack0, iNumVertices
    MFIR.z fStack0, iClipFlag1
    MFIR.w fStack0, iClipAll21
    MFIR.x fStack1, iNumCams
    MFIR.y fStack1, pInputBuffer

    --barrier

	// Setup the clipping variables
    ISUBIU pClipSrc,     pIn,               9                                          // First vertex in clipped triangle is 3 vertices back from pIn
    IADDIU pClipDst,     pThisOutputBuffer, ASURA_PS2_GLASS_OFFSET_CLIPPER_WORKSPACE_0 // Work buffer swap is done using:  pSrc = pDst
    IADDIU pClipMid,     pClipDst,          ASURA_PS2_GLASS_SIZE_CLIPPER_WORKSPACE     // pDst = Buffer1-pSrc+Buffer0 : so if pSrc = Buffer1 : pDst = Buffer1-Buffer1+Buffer0 = Buffer0
    IADD   pClipMid,     pClipMid,          pClipDst                                   // if pSrc = Buffer0 : pDst = Buffer1-Buffer0+Buffer0 = Buffer1.  so we keep Buffer0+Buffer1 then just subtract pSrc
	IADDIU iSrcPointQty, VI00,              3

	//----------------------------------
	// Clip to the near plane (Z -ve)
	//----------------------------------
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_NEAR		// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_FAR_PLANE
	SUBw.xw	fPlaneSign,	VF00, VF00							// Clear destination vertex count and set plane sign to -ve
//	IADDIU	pInterpolate, VI00, INTERPOLATE_Z				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 0							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	ISUB	pClipDst, pClipMid, pClipSrc					// destination pointer
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters for next clip
	MTIR	iSrcPointQty, fPlaneSign[w]
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//----------------------------------
	// Clip to the far plane (Z +ve)
	//----------------------------------
CLIP_FAR_PLANE:
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_FAR		// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_TOP_PLANE
	ADDw.x	fPlaneSign,	VF00, VF00							// Set plane sig to +ve
	SUB.w	fPlaneSign, VF00, VF00							// Clear vertex count
//	IADDIU	pInterpolate, VI00, INTERPOLATE_Z				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 0							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	ISUB	pClipDst, pClipMid, pClipSrc					// destination pointer
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters for next clip
	MTIR	iSrcPointQty, fPlaneSign[w]
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//----------------------------------
	// Clip to the top plane (Y -ve)
	//----------------------------------
CLIP_TOP_PLANE:
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_TOP		// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_BOTTOM_PLANE
	SUBw.xw	fPlaneSign,	VF00, VF00							// Clear destination vertex count and set plane sign to -ve
//	IADDIU	pInterpolate, VI00, INTERPOLATE_Y				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 1							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	ISUB	pClipDst, pClipMid, pClipSrc					// destination pointer
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters for next clip
	MTIR	iSrcPointQty, fPlaneSign[w]
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//----------------------------------
	// Clip to the bottom plane (Y +ve)
	//----------------------------------
CLIP_BOTTOM_PLANE:
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_BOTTOM	// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_LEFT_PLANE
	ADDw.x	fPlaneSign,	VF00, VF00							// Set plane sign to +ve
	SUB.w	fPlaneSign, VF00, VF00							// Clear destination vertex counter
//	IADDIU	pInterpolate, VI00, INTERPOLATE_Y				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 1							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	ISUB	pClipDst, pClipMid, pClipSrc					// destination pointer
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters for next clip
	MTIR	iSrcPointQty, fPlaneSign[w]
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//----------------------------------
	// Clip to the left plane (X -ve)
	//----------------------------------
CLIP_LEFT_PLANE:
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_LEFT		// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_RIGHT_PLANE
	SUBw.xw	fPlaneSign,	VF00, VF00							// Set plane sign to -ve and clear destination vertex count
//	IADDIU	pInterpolate, VI00, INTERPOLATE_X				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 2							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	ISUB	pClipDst, pClipMid, pClipSrc					// destination pointer
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters for next clip
	MTIR	iSrcPointQty, fPlaneSign[w]
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//----------------------------------
	// Clip to the right plane (X +ve)
	//----------------------------------
CLIP_RIGHT_PLANE:
	IADDIU	iCurrentClipMask, VI00, CLIP_PLANE_RIGHT	// Setup the current clip mask
//	IAND	iTemp,	iClipPlanes, iCurrentClipMask
//	IBEQ	iTemp,	VI00,	CLIP_FINISHED_PLANES
	ADDw.x	fPlaneSign,	VF00, VF00							// Set plane sign to +ve
	SUB.w	fPlaneSign, VF00, VF00							// Clear destination vertex count
//	IADDIU	pInterpolate, VI00, INTERPOLATE_X				// No VCL JUMP TABLE SUPPORT :o(
	IADDIU	pInterpolate, VI00, 2							// No VCL JUMP TABLE SUPPORT :o(
	BAL		pRtA, CLIP_POINTS								// Clip the points
	IADDIU	pClipSrc, pClipDst, 0							// Setup the source pointer,
	FTOI0.w	fPlaneSign, fPlaneSign							//	and counters
	MTIR	iSrcPointQty, fPlaneSign[w]				
	IBEQ	iSrcPointQty, VI00,	CLIP_FINISHED_THIS_TRIANGLE	// All the points have been clipped away...

	//-----------------------------------------------
	// Finished clipping - Project in to screen space
	//-----------------------------------------------
CLIP_FINISHED_PLANES:
    // Check there is enough space in the output buffer for this many more
    // vertices.
    ISUB   iTemp, pOut,  pThisOutputBuffer
    ISUBIU iTemp, iTemp, (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 3)   // now contains (3 * num verts output so far)
    IADD   iTemp, iTemp, iSrcPointQty                                         // \ 
    IADD   iTemp, iTemp, iSrcPointQty                                         //  |- + (3 * num verts in this strip)
    IADD   iTemp, iTemp, iSrcPointQty                                         // /
    ISUBIU iTemp, iTemp, (ASURA_PS2_GLASS_MAX_VERTICES_IN_CLIPPER_OUTPUT * 3) // now < 0 if there is space for this strip
    IBLTZ  iTemp, CLIP_ENOUGH_SPACE_IN_OUTPUT
    // Not enough space for this strip so kick what we have and swap buffers.
    XGKICK              pThisOutputBuffer
    SwitchOutputBuffers pThisOutputBuffer
    // Load GS register settings and GIF tag.
    LQ fSCISSOR,            12(pCamData)
    LQ fXYOFFSET,           13(pCamData)
    LQ fClippedStripGIFTag, ASURA_PS2_GLASS_ADDR_CLIPPED_STRIP_GIF_TAG(VI00)
    // And copy to output buffer.
    SQ fSCISSOR,            (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 1)(pThisOutputBuffer)
    SQ fXYOFFSET,           (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 2)(pThisOutputBuffer)
    SQ fClippedStripGIFTag, (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 3)(pThisOutputBuffer)
    // Reset output ptr.
    IADDIU pOut, pThisOutputBuffer, (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 4)
	
CLIP_ENOUGH_SPACE_IN_OUTPUT:
    // Increment NLOOP count in GIF tag.
    ILW.x iTemp, (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 3)(pThisOutputBuffer)
    IADD  iTemp, iTemp,	iSrcPointQty
    ISW.x iTemp, (ASURA_PS2_GLASS_OFFSET_CLIPPER_OUTPUT_BUFFER + 3)(pThisOutputBuffer)
	// Setup the input pointers
	IADD	pClipSrcE, iSrcPointQty, iSrcPointQty
	IADD	pClipSrcE, pClipSrcE, iSrcPointQty
	IADD	pClipSrcE, pClipSrcE, pClipSrc
	ISUBIU	pClipSrcE, pClipSrcE, 3					// Last vertex in fan
	// Load the clip to screen matrix
    MatrixLoad mC2S, 8, pCamData
	// Set Q component of vSTQ to 1.0
	ADDw.z	vSTQ,	VF00,	VF00
	// Set degeneracy flag for first two output verts
	IADDIU	iADC,	VI00,	0x6000
    // Offset value for RGBAs to get correct rounding.
    LOI 0.5

CLIP_OUTPUT_STRIP_LOOP

	    --LoopCS 2, 0

	    // Load vertex from the start
	    LQ.xyzw	vClipPos,	0(pClipSrc)
	    LQ.xy	vSTQ,		1(pClipSrc)
		LQ.xyzw	vRGBA,		2(pClipSrc)
		IADDIU	pClipSrc, pClipSrc,	3
	    // Transform clip space position to screenspace.
	    MatrixMultiplyVertex vScrnPos, mC2S, vClipPos
	    // Calculate 1/w.
		DIV Q, VF00[w], vScrnPos[w]
		// Perspective divide.
		MULq.xyz vScrnPos, vScrnPos, Q
		MULq.xyz vTexOut,  vSTQ,     Q
		// Convert XYZ to GS format.
        FTOI4.xyz vScrnPos, vScrnPos
        // And RGBA. Note that the interpolated alpha can be slightly out,
        // even when the alpha values its interpolating between are 128 and
        // 128, so we need to add on a little bit to stop alpha 127 being
        // written when it should be alpha 128 (due to alpha test limits,
        // etc.)
        ADDi.w vRGBA, vRGBA, I
        FTOI0  vRGBA, vRGBA
		// Output.
		SQ.xyz	vTexOut,  0(pOut)
		SQ.xyzw	vRGBA,	  1(pOut)
		SQ.xyz	vScrnPos, 2(pOut)
		IADDIU	pOut,	pOut,	3
		// Set the ADC bit (for degenerates)
		IADD	iADC,	iADC,	iADC
		ISW.w	iADC,	-1(pOut)
		// Check if we've finished yet
		ISUB	iTemp,	pClipSrcE,	pClipSrc
		IBLTZ 	iTemp,	CLIP_FINISHED_OUTPUT_THIS_TRIANGLE
		// Load vertex from the end
	    LQ.xyzw	vClipPos,	0(pClipSrcE)
	    LQ.xy	vSTQ,		1(pClipSrcE)
		LQ.xyzw	vRGBA,		2(pClipSrcE)
		ISUBIU	pClipSrcE,	pClipSrcE,	3
	    // Transform clip space position to screenspace.
	    MatrixMultiplyVertex vScrnPos, mC2S, vClipPos
	    // Calculate 1/w.
		DIV Q, VF00[w], vScrnPos[w]
		// Perspective divide.
		MULq.xyz	vScrnPos, vScrnPos, Q
		MULq.xyz	vTexOut,  vSTQ,     Q
		// Convert XYZ to GS format.
        FTOI4.xyz vScrnPos, vScrnPos
        // And RGBA. See comment above.
        ADDi.w vRGBA, vRGBA, I
        FTOI0  vRGBA, vRGBA
		// Output.
		SQ.xyz	vTexOut,  0(pOut)
		SQ.xyzw	vRGBA,	  1(pOut)
		SQ.xyz	vScrnPos, 2(pOut)
		IADDIU	pOut,	pOut,	3
		// Set the ADC bit (for degenerates)
		IADD	iADC,	iADC,	iADC
		ISW.w	iADC,	-1(pOut)
		// Check if we've finished yet
		ISUB	iTemp,	pClipSrcE,	pClipSrc
		IBGEZ 	iTemp,	CLIP_OUTPUT_STRIP_LOOP

CLIP_FINISHED_OUTPUT_THIS_TRIANGLE:
CLIP_FINISHED_THIS_TRIANGLE:
    // Retrieve int registers.
    MTIR pIn,                 fStack0[x]
    MTIR iNumVertices,        fStack0[y]
    MTIR iClipFlag1,          fStack0[z]
    MTIR iClipAll21,          fStack0[w]
    MTIR iNumCams,            fStack1[x]
    MTIR pInputBuffer,        fStack1[y]
    // Reset clip mask.
    IADDIU iClipMask, VI00, 0x3F
    // Carry out previous 2 clips again so the clip find triangle loop has the correct clip history
    LQ.xyzw   vClipPos, -6(pIn)
    CLIPw.xyz vClipPos, vClipPos
    LQ.xyzw   vClipPos, -3(pIn)
    CLIPw.xyz vClipPos, vClipPos

	B FIND_CLIP_TRIANGLE_LOOP

--barrier

CLIP_POINTS:
    // Stick some more integer stuff on the "stack" to try and relieve
    // register pressure...
    MFIR.x fStack2, pClipMid
    MFIR.y fStack2, pOut
    MFIR.z fStack2, pThisOutputBuffer
	// Keep track of the start of the input buffer so we can add the first vertex to the end of the list at the end
	IADDIU	pClipSrcStart, pClipSrc, 0
	// Create an output pointer
	IADDIU	pClipOut, pClipDst, 0
    // Load the first vertex from the buffer
    LQ.xyzw	vNVert,   0(pClipSrc)
    LQ.xy	vNSTQ,    1(pClipSrc)
    LQ.xyzw	vNColour, 2(pClipSrc)
    IADDIU	pClipSrc, pClipSrc, 3
	// Check if it's in or out
	CLIPw.xyz	vNVert, vNVert[w]
	FCGET		VI01
	IAND		iTemp, VI01, iCurrentClipMask
	IBEQ		iTemp, VI00, CLIP_CURRENT_IN

CLIP_CURRENT_OUT:
	// Current point is out
	// Anymore points ?
	ISUBIU	iSrcPointQty, iSrcPointQty, 1
	IBEQ	iSrcPointQty, VI00, CLIP_POINTS_END
	// Move next point to current point
	MOVE.xyzw	vCVert,		vNVert
	MOVE.xy		vCSTQ,		vNSTQ
	MOVE.xyzw	vCColour,	vNColour
    // Load the next point
    LQ.xyzw vNVert,		0(pClipSrc)
    LQ.xy   vNSTQ,		1(pClipSrc)
    LQ.xyzw vNColour,	2(pClipSrc)
    IADDIU  pClipSrc,	pClipSrc,	3
	// Check if it's in or out
	CLIPw.xyz	vNVert, vNVert[w]
	FCGET		VI01
	IAND		iTemp, VI01, iCurrentClipMask
	IBEQ		iTemp, VI00, CLIP_OUT_IN
CLIP_OUT_OUT:
	// Next point
	B	CLIP_CURRENT_OUT
CLIP_OUT_IN:
	// Adding 1 to destination point count
	ADD.w	fPlaneSign, fPlaneSign, VF00
	// Write INTERPOLATED - Setup retrun address to jump to the next point
//	IADDIU	pRtA2, VI00, CLIP_CURRENT_IN
//	JR		pInterpolate
	// Hack until VCL support JumpTables or I re write it in VSM
	BAL	pRtA2,	INTERPOLATE
	B	CLIP_CURRENT_IN

CLIP_CURRENT_IN:
	// Current point is in
	// Anymore points ?
	ISUBIU	iSrcPointQty, iSrcPointQty, 1
	IBEQ	iSrcPointQty, VI00, CLIP_POINTS_END
	// Move next point to current point
	MOVE.xyzw	vCVert,		vNVert
	MOVE.xy		vCSTQ,		vNSTQ
	MOVE.xyzw	vCColour,	vNColour
	// Load the next point
	LQ.xyzw	vNVert,		0(pClipSrc)
	LQ.xy	vNSTQ,		1(pClipSrc)
	LQ.xyzw	vNColour,	2(pClipSrc)
	IADDIU	pClipSrc,	pClipSrc,	3
	// Check if it's in or out
	CLIPw.xyz	vNVert, vNVert[w]
	FCGET		VI01
	IAND		iTemp, VI01, iCurrentClipMask
	IBNE		iTemp, VI00, CLIP_IN_OUT
CLIP_IN_IN:
	// Adding 1 to destination point count
	ADD.w	fPlaneSign, fPlaneSign, VF00
	// Write CURRENT
	SQ.xyzw	vCVert,		0(pClipOut)
	SQ.xy	vCSTQ,		1(pClipOut)
	SQ.xyzw	vCColour,	2(pClipOut)
	IADDIU	pClipOut,	pClipOut,	3
	// Next point
	B	CLIP_CURRENT_IN
CLIP_IN_OUT:
	// Adding 2 to destination point count
	ADD.w	fPlaneSign, fPlaneSign, VF00
	ADD.w	fPlaneSign, fPlaneSign, VF00
	// Write CURRENT
	SQ.xyzw	vCVert,		0(pClipOut)
	SQ.xy	vCSTQ,		1(pClipOut)
	SQ.xyzw	vCColour,	2(pClipOut)
	IADDIU	pClipOut,	pClipOut,	3
	// Write INTERPOLATED - Setup retrun address to jump to the next point
//	IADDIU	pRtA2, VI00, CLIP_CURRENT_OUT
//	JR		pInterpolate
	// Hack until VCL support JumpTables or I re write it in VSM
	BAL	pRtA2,	INTERPOLATE
	B	CLIP_CURRENT_OUT

CLIP_POINTS_END:
    // Check if we've done the final line yet (last->first)
    IBNE pClipSrcStart, VI00, CLIP_POINT_LAST_SEGMENT
    // Pop stuff back off the "stack".
    MTIR pClipMid,          fStack2[x]
    MTIR pOut,              fStack2[y]
    MTIR pThisOutputBuffer, fStack2[z]
    // And we're done.
    JR	pRtA

CLIP_POINT_LAST_SEGMENT
	// Get the start of the input buffer and clear the input buffer pointer so we know we've finished
	IADDIU	pClipSrc, pClipSrcStart, 0
	IADDIU	pClipSrcStart, VI00, 0
	// One final segment to do
	IADDIU	iSrcPointQty, iSrcPointQty, 2
	// Check if the last clip was in or out
	FCGET	VI01
	IAND	iTemp, VI01, iCurrentClipMask
	IBEQ	iTemp, VI00, CLIP_CURRENT_IN
	B		CLIP_CURRENT_OUT

--barrier

	//		 |-------L-------|
	//		C ----------|---- N
	//					P
	//		 |----A-----l--B-|
	//					a
	//					n
	//					e
	//
	//	we need to find the ratio R = |A/L|

	// Hack until VCL support JumpTables or I re write it in VSM
INTERPOLATE:
	IBEQ	pInterpolate, VI00, INTERPOLATE_Z
	ISUBIU	iInt, pInterpolate, 1
	IBEQ	iInt, VI00, INTERPOLATE_Y
	// Interpolate the current line segment against the clip plane
INTERPOLATE_X:
	// Get distance from clip plane to current (taking into account -/+ clip plane)		(-A in above diagram)
	ADDAx.x	ACC, VF00, vCVert						; ACC.x = CVert.z
	MSUBw.x fC2Plane, fPlaneSign, vCVert			; fC2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	// Get distance from next to clip plane (taking into account -/+ clip plane)			(B in above diagram)
	ADDAx.x	ACC, VF00, vNVert						; ACC.x = Vert.z
	MSUBw.x fN2Plane, fPlaneSign, vNVert			; fN2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	B	INTERPOLATE_COMMON
INTERPOLATE_Y:
	// Get distance from clip plane to current (taking into account -/+ clip plane)		(-A in above diagram)
	ADDAy.x	ACC, VF00, vCVert						; ACC.x = CVert.z
	MSUBw.x fC2Plane, fPlaneSign, vCVert			; fC2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	// Get distance from next to clip plane (taking into account -/+ clip plane)			(B in above diagram)
	ADDAy.x	ACC, VF00, vNVert						; ACC.x = Vert.z
	MSUBw.x fN2Plane, fPlaneSign, vNVert			; fN2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	B	INTERPOLATE_COMMON
INTERPOLATE_Z:
	// Get distance from clip plane to current (taking into account -/+ clip plane)		(-A in above diagram)
	ADDAz.x	ACC, VF00, vCVert						; ACC.x = CVert.z
	MSUBw.x fC2Plane, fPlaneSign, vCVert			; fC2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	// Get distance from next to clip plane (taking into account -/+ clip plane)			(B in above diagram)
	ADDAz.x	ACC, VF00, vNVert						; ACC.x = Vert.z
	MSUBw.x fN2Plane, fPlaneSign, vNVert			; fN2Plane.x = (CVert.z) - (fPlaneSign, CVert.w)
	//B	INTERPOLATE_COMMON
INTERPOLATE_COMMON:
	// Calculate L = B + A = B - -A
	SUB.x	fDiff, fN2Plane, fC2Plane
	// Calculate R = |A/L|
	DIV		Q, fC2Plane[x], fDiff[x]
	ADDq.x	fR, vf00, Q
	ABS.x	fR, fR
	// Create a new vertex from the old verts and the ratio (New = Current + (Next-Current)*R)
	// XYZ
	ADDAx.xyzw	ACC,		vCVert,		VF00		// = Current
	MADDAx.xyzw	ACC,		vNVert,		fR			//	= Current + (Next*R)
	MSUBx.xyzw	vNewVert,	vCVert,		fR			//	= Current + (Next*R) - (Current*R)
	SQ.xyzw		vNewVert,	0(pClipOut)
	// ST
	ADDAx.xy	ACC,		vCSTQ,		VF00		// = Current
	MADDAx.xy	ACC,		vNSTQ,		fR			//	= Current + (Next*R)
	MSUBx.xy	vNewSTQ,	vCSTQ,		fR			//	= Current + (Next*R) - (Current*R)
	SQ.xy		vNewSTQ,	1(pClipOut)
	// RGBA
	ADDAx.xyzw	ACC,		vCColour,	VF00		// = Current
	MADDAx.xyzw	ACC,		vNColour,	fR			//	= Current + (Next*R)
	MSUBx.xyzw	vNewColour,	vCColour,	fR			//	= Current + (Next*R) - (Current*R)
	SQ.xyzw		vNewColour,	2(pClipOut)
	// Done
	IADDIU		pClipOut,	pClipOut,	3
	JR	pRtA2
    
.end
