//////////////////////////////////////////////////////////////////////////////
//
// VU1_Water.vcl
//
// Copyright (c) 2003 Rebellion. All rights reserved.
//
// See Asura_PS2_WATERDefines.h for memory map.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Asura_PS2_WaterDefines.h"
#include "vcl_sml.i"
#include "VU1_CommonMacros.i"

#define EDGE_01_MASK	0x80
#define EDGE_12_MASK	0x40
#define EDGE_20_MASK	0x20
#define EDGE_TEST_BITS	( EDGE_01_MASK | EDGE_12_MASK | EDGE_20_MASK )

#define EDGE_01_FORCE_MASK  ( EDGE_12_MASK | EDGE_20_MASK )
#define EDGE_12_FORCE_MASK  ( EDGE_01_MASK | EDGE_20_MASK )
#define EDGE_20_FORCE_MASK  ( EDGE_12_MASK | EDGE_01_MASK )
#define EDGES_UNSPLIT       ( EDGE_01_MASK | EDGE_12_MASK | EDGE_20_MASK )

// re-enable this define if you want to be able to disable all rendering kicks in order to determine whether 
// any slow bits are slow because of rendering or VU processing.
//define ALLOW_KICK_CONTROL

#ifdef ALLOW_KICK_CONTROL
.macro KickMacro reg
	ILW.y	iMeshFlags, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)
	IADDIU	iMask, VI00, ASURA_PS2_WATER__FLAG__SKIP_KICKS
	IAND	iMask, iMask, iMeshFlags
	IBGTZ   iMask, SKIP_KICK\@

	XGKICK  \reg

SKIP_KICK\@:
.endm
#else
.macro KickMacro reg
	XGKICK \reg
.endm
#endif

// put this define in if debugging to find out what kind of polys are being generated
//define COUNT_POLYS
#ifdef COUNT_POLYS
.macro CountOne type, sub=
.aif "\type" eq "input"
	LQ.x	fCounter, ASURA_PS2_WATER__ADDR__VI_STACK_3(VI00)
	.aif "\sub" eq "sub"
		SUBw.x  fCounter, fCounter, VF00
	.aelse
		ADDw.x  fCounter, fCounter, VF00
	.aendi
	SQ.x	fCounter, ASURA_PS2_WATER__ADDR__VI_STACK_3(VI00)
.aelse
 .aif "\type" eq "reject"
	LQ.y	fCounter, ASURA_PS2_WATER__ADDR__VI_STACK_3(VI00)
	.aif "\sub" eq "sub"
		SUBw.y  fCounter, fCounter, VF00
	.aelse
		ADDw.y  fCounter, fCounter, VF00
	.aendi
	SQ.y	fCounter, ASURA_PS2_WATER__ADDR__VI_STACK_3(VI00)
 .aelse
   .aif "\type" eq "output"
	LQ.z	fCounter, ASURA_PS2_WATER__ADDR__VI_STACK_3(VI00)
	.aif "\sub" eq "sub"
		SUBw.z  fCounter, fCounter, VF00
	.aelse
		ADDw.z  fCounter, fCounter, VF00
	.aendi
	SQ.z	fCounter, ASURA_PS2_WATER__ADDR__VI_STACK_3(VI00)
   .aelse
    .aif "\type" eq "far"
 	 LQ.w	fCounter, ASURA_PS2_WATER__ADDR__VI_STACK_3(VI00)
	 .aif "\sub" eq "sub"
		 SUBw.w  fCounter, fCounter, VF00
	 .aelse
		 ADDw.w  fCounter, fCounter, VF00
	 .aendi
	 SQ.w	fCounter, ASURA_PS2_WATER__ADDR__VI_STACK_3(VI00)
    .aendi
   .aendi
 .aendi
.aendi
.endm
#endif

// V C L   S E T U P /////////////////////////////////////////////////////////

.syntax new

.init_vf_all
.init_vi_all

.global VU1_WATER_SWITCH_CAM_DATA_BFR
.global VU1_WATER_SWITCH_RENDERSTATE_BFR
.global VU1_WATER_LOAD_MESH_FLAGS
.global VU1_WATER_LOAD_CAM_FLAGS
.global VU1_WATER_NOP
.global VU1_WATER_INIT 
.global VU1_WATER_RENDER 

// M A C R O S ///////////////////////////////////////////////////////////////

#define POLYNOMIAL_PEAK_POINT	-0.2113248
;#define POST_RENDER_BFC

.macro Fast_SwitchBfr_Constants pCur, first_bfr, second_bfr
	IADDIU _pFirst, VI00, \first_bfr + \second_bfr
	ISUB	\pCur, _pFirst, \pCur
.endm

.macro IXOR	out, in1, in2
	IOR		iTemp, \in1, \in2
	IAND	iTemp2, \in1, \in2
	ISUB	\out, iTemp, iTemp2
.endm

.macro frac_space_presets
	// preload the multipliers and position them
	LQ			fXScales, ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS(VI00)
	LQ			fZScales, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+1)(VI00)
	LQ			fTScales, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+2)(VI00)
	LQ			fYScales, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+3)(VI00)
	LQ			fFScales, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+4)(VI00)

	LOI		33554432.0	// make special constant register for conversion to integer
	MAXi	ftoiconst1, VF00, I
	LOI		8388608.0	// make special constant register for conversion to integer
	MAXi	ftoiconst2, VF00, I
	LOI		8
	MAXi	f8888, VF00, I
.endm

.macro compute_frac_space	out, inputx, inputz, cpt
	MULAx.\cpt		ACC,	fXScales, 	\inputx[x]
	MADDAz.\cpt		ACC,	fZScales,	\inputz[z]
	MADDw.\cpt		\out,	fTScales,	VF00[w]			// total is ax+bz+t for two different waves

	// first stage - bring components into small range - target +/- 8
	// has to be done like this because -ve and positive numbers behave differently
	// so we add a very large number which loses all the accuracy, so that adding original number back in
	// gives us something closer to zero but with fractional accuracy retained
	SUBA.\cpt		ACC,	VF00,		\out
	MADDAw.\cpt		ACC,	ftoiconst1, VF00[w]
	MSUBAw.\cpt		ACC,	ftoiconst1, VF00[w]
	MADDAw.\cpt		ACC,	\out,		VF00[w]

	// then we offset the number to ensure it is always positive
	MADDw.\cpt		\out,	f8888,		VF00[w]

	// note needs to come out of ACC here in order to do later subtraction
	// but we can keep it in ACC as well for speed, so we do same offset again
	MADDAw.\cpt		ACC,	f8888,		VF00[w]
	MADDAw.\cpt		ACC,	ftoiconst2, VF00[w]
	MSUBAw.\cpt		ACC,	ftoiconst2, VF00[w]
	MSUBw.\cpt		\out,	\out,		VF00[w]
	// this leaves -frac(x) because it is effectively int(x)-x
.endm

.macro refix_frac_space		out, input, cpt
	ADDAw.\cpt		ACC,    ftoiconst2, \input
	MSUBAw.\cpt		ACC,	ftoiconst2, VF00[w]
	MSUBw.\cpt		\out,	\input,		VF00[w]
	SUB.\cpt		\out,	VF00,		\out
.endm

.macro compute_polynomial outreg, inreg, cpt
	MUL.\cpt		fSquare,	\inreg,		\inreg		// get squared result
	LOI				-0.5
	MULAi.\cpt		ACC,		\inreg,		I			// ACC = 0.5 x
	LOI				1.5
	MSUBAi.\cpt		ACC,		fSquare,	I			// ACC -= 1.5 x^2
	MSUB.\cpt		\outreg,	fSquare,	\inreg		// ACC += x^3
	// note we are using the polynomial for -x as our input is assumed (and usually) negative
.endm

// C O D E ///////////////////////////////////////////////////////////////////

--enter
--endenter


VU1_WATER_INIT:

    // Setup initial values of the input ptrs that are controlled from EE.
    IADDIU pCamDataBfr,     VI00, ASURA_PS2_WATER__ADDR__PER_OBJ_CAM_DATA_BFR_0
    IADDIU pRenderstateBfr, VI00, ASURA_PS2_WATER__ADDR__RENDERSTATE_BFR_0
    // Push onto integer register stack, to help relieve register pressure.
    ISW.y pCamDataBfr,     ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
    ISW.w pRenderstateBfr, ASURA_PS2_WATER__ADDR__VI_STACK_5(VI00)
    // Reset stored cam flags so we recognise they've changed the first time
    // they're uploaded.
    ISW.w VI00, ASURA_PS2_WATER__ADDR__VI_STACK_1(VI00)

#ifdef COUNT_POLYS
	SUB		fCounters, VF00, VF00
	SQ		fCounters, ASURA_PS2_WATER__ADDR__VI_STACK_3(VI00)	// counter for tracking
#endif

	B VU1_WATER_NOP
//	--cont

//	XTOP dummy
//	IBEQ dummy, VI00, VU1_WATER_RENDER

--enter
--endenter

VU1_WATER_LOAD_MESH_FLAGS:

    // Put this first to force VCL to avoid overwriting the register
    // containing iMeshFlags in the ...SWITCH... stuff below. See the (*)
    // comment above.
    XITOP iMeshFlags
	ISW.y iMeshFlags,  ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)
    --cont

--enter
--endenter

VU1_WATER_LOAD_CAM_FLAGS:

    XITOP iCamFlags
    ILW.w iOldVisAndClipFlags, ASURA_PS2_WATER__ADDR__VI_STACK_1(VI00)
    // Extract components.
    IADDIU iMask, VI00, ASURA_PS2_WATER__FLAG__VIS_FOR_MORE_THAN_ONE_CAM
    IAND iMoreThanOneCamFlag, iCamFlags, iMask
    IADDIU iMask, VI00, ASURA_PS2_WATER__MASK__CAM_VIS_AND_CLIP_FLAGS
    IAND iVisAndClipFlags, iCamFlags, iMask
    // Store to stack.
    ISW.x iMoreThanOneCamFlag, ASURA_PS2_WATER__ADDR__VI_STACK_5(VI00)
    ISW.w iVisAndClipFlags,    ASURA_PS2_WATER__ADDR__VI_STACK_1(VI00)
    // Have the vis flags changed?
    IADDIU iMask, VI00, ASURA_PS2_WATER__MASK__CAM_VIS_FLAGS
    IAND iOldVisFlags, iOldVisAndClipFlags, iMask
    IAND iNewVisFlags, iVisAndClipFlags, iMask
    IBEQ iOldVisFlags, iNewVisFlags, VU1_WATER_FINISHED_SETTING_UP_CAM_FLAGS
    // Yes, they've changed so force RS to be sent next time. Note that
    // writing any non-zero value will do here.
    ISW.w iMask, ASURA_PS2_WATER__ADDR__VI_STACK_4(VI00)

VU1_WATER_FINISHED_SETTING_UP_CAM_FLAGS:

    --cont

--enter
--endenter

VU1_WATER_SWITCH_CAM_DATA_BFR:

    ILW.y pCamDataBfr, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
    Fast_SwitchBfr_Constants pCamDataBfr, ASURA_PS2_WATER__ADDR__PER_OBJ_CAM_DATA_BFR_0, ASURA_PS2_WATER__ADDR__PER_OBJ_CAM_DATA_BFR_1
    ISW.y pCamDataBfr, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
    --cont

--enter
--endenter

VU1_WATER_SWITCH_RENDERSTATE_BFR:

    // Swap ptrs.
    ILW.w pRenderstateBfr, ASURA_PS2_WATER__ADDR__VI_STACK_5(VI00)
    Fast_SwitchBfr_Constants pRenderstateBfr, ASURA_PS2_WATER__ADDR__RENDERSTATE_BFR_0, ASURA_PS2_WATER__ADDR__RENDERSTATE_BFR_1
    ISW.w pRenderstateBfr, ASURA_PS2_WATER__ADDR__VI_STACK_5(VI00)
    // Set need-to-send-RS flag for next strip. Writing any non-zero value
    // will do.
    ISW.w pRenderstateBfr, ASURA_PS2_WATER__ADDR__VI_STACK_4(VI00)
    --cont

VU1_WATER_NOP:	// dummy nothing

	--cont

VU1_WATER_RENDER:

// first do stuff which only needs doing one time - ie first camera

    // Setup output buffer ptr.etc
    IADDIU pOutputBuffer, VI00, ASURA_PS2_WATER__ADDR__GEOM_OUTPUT_BFR_0
    IADDIU pVertOut, pOutputBuffer, ASURA_PS2_WATER__OFFSET__GEOM_OUT_FIRST_VERTEX
	IADDIU pVertExtra, VI00, ASURA_PS2_WATER__ADDR__ST_OVERWRITE_BFR
	IADD	iVertexQty, VI00, VI00

    // Retrieve main cam data bfr ptr from stack.
    ILW.y pCamDataBfr,   ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
    // Reset cam data ptrs.
    IADDIU pCurConstCamData, VI00, ASURA_PS2_WATER__ADDR__CONST_CAM_DATA
    IADDIU pCurPerObjCamData, pCamDataBfr, 0

    // Reset remaining cam flags.
    ILW.w iRemainingCamFlags, ASURA_PS2_WATER__ADDR__VI_STACK_1(VI00)
    ISW.x iRemainingCamFlags, ASURA_PS2_WATER__ADDR__VI_STACK_4(VI00)

	// keep copy of camera pointer so it can be released
	ISW.w pCurConstCamData, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)  
	ISW.z pCurPerObjCamData, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)  

	// calculate maximum possible offset for clipping purposes
	LOI POLYNOMIAL_PEAK_POINT
	MULi.w	fMaxOffset, VF00, I
	compute_polynomial fMaxOffset, fMaxOffset, w

	LQ			fYScales, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+3)(VI00)
	ABS.xyz		fYScales, fYScales
	MULAx.w		ACC, fMaxOffset, fYScales[x]
	MADDy.w		fMaxOffset, fMaxOffset, fYScales[y]		// add two waves (not three) since thats what we do when rendering!

	// ideally we should also multiply by 1/cos(fov/2) to account for the way this will be used in adjusting clipping space
	// however tests show that this wouldn't help with performance at all.
	LOI	2.0
	MUL.w	fMaxOffset, fMaxOffset, I
	ABS.w	fMaxOffset, fMaxOffset

VU1_WATER_RENDER_NEXT_CAMERA:

// first of all decide whether we are processing this camera at all, or if not find one we are

	// reload to save register pressue
    ILW.x iRemainingCamFlags, ASURA_PS2_WATER__ADDR__VI_STACK_4(VI00)

    // Prepare mask value for loop and setup first iteration test.
    IADDIU iVisBitMask, VI00, 0x80
    IAND iRenderForThisCam, iRemainingCamFlags, iVisBitMask
    // Any cameras left to render this strip for?
    IBEQ iRemainingCamFlags, VI00, VU1_WATER_NOP	// finished all so go back and wait for new data
    // Reload the vertex count and the light and cam ptrs. Could do this once
    // per strip, rather than per camera, but this helps with register
    // pressure.
    ILW.z pCurPerObjCamData, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
    ILW.w pCurConstCamData,  ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)

    VU1_WATER_FIND_NEXT_CAM_LOOP:

        // Render for this cam?
        IBNE iRenderForThisCam, VI00, VU1_WATER_FOUND_NEXT_CAM
        // No. Shift flags up 2.
        IADD iRemainingCamFlags, iRemainingCamFlags, iRemainingCamFlags
        IADD iRemainingCamFlags, iRemainingCamFlags, iRemainingCamFlags
        // Setup next iteration test.
        IAND iRenderForThisCam, iRemainingCamFlags, iVisBitMask

		// prevent possible fatal crash bug - if the flags are wrong coming in, they can put us in infinite loop
		IBEQ iRemainingCamFlags, VI00, VU1_WATER_NOP	// finished all so go back and wait for new data

        // And move ptrs on.
        IADDIU pCurPerObjCamData, pCurPerObjCamData, ASURA_PS2_WATER__OFFSET__CAM_PER_OBJ_NEXT_CAM
        IADDIU pCurConstCamData,  pCurConstCamData,  ASURA_PS2_WATER__OFFSET__CONST_CAM_NEXT_CAM
        // Go on to check for next cam.
        B VU1_WATER_FIND_NEXT_CAM_LOOP

VU1_WATER_FOUND_NEXT_CAM:

	// now store the camera pointers etc ready for next time
    ISW.x iRemainingCamFlags, ASURA_PS2_WATER__ADDR__VI_STACK_4(VI00)
	ISW.z pCurPerObjCamData, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
	ISW.w pCurConstCamData,  ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)

// now do stuff which will need restarting for a new camera

	// this needs refreshing because it gets trashed as we process
	XTOP	data
	XITOP	count
	ISW.w	data, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)
	ISW.x	count, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)

	// reset vertex stack to start
	IADDIU  pVertStack, VI00, ASURA_PS2_WATER__ADDR__VERT_STACK
	FCSET 0

	// load matrices for current camera (which may change !)
    MatrixLoad mO2GBC, ASURA_PS2_WATER__OFFSET__CAM_PER_OBJ_O2GBC_MAT, pCurPerObjCamData
    LQ fGBC2SScale, ASURA_PS2_WATER__OFFSET__CONST_CAM_GBC2S_SCALE(pCurConstCamData)
    LQ fGBC2STrans, ASURA_PS2_WATER__OFFSET__CONST_CAM_GBC2S_TRANS(pCurConstCamData)

// NOTE : currently single pass so just copied in RS code from GSM renderer
// will need to implement properly later - but will only have to happen at each kick I think.
// might have to modify storage to account for later passes though !

VU1_WATER_NEXT_TRI:
	BAL	retAddress, VU1_WATER_GET_TRIANGLE	// note this doesn't change the addresses - so we can decide whether to add tri (3) or vert (strip)
	IADDIU iTrisOnStack, VI00, 1			// used to track completion
	IADDIU iNextTriToProcess, VI00, ASURA_PS2_WATER__ADDR__VERT_STACK		// used to track current work point during recursion
#ifdef COUNT_POLYS
	CountOne input
#endif

#define MANUAL_OPT
	// remove the above define and all code that is within it below to see what the code looked like before I
	// rearranged it massively to squeeze a few more cycles out.
#ifndef MANUAL_OPT
#define AUTO_OPT
#endif

TRI_LOOP_POINT:
	// TEST_EDGES assumes that the 3 points are already loaded in fOut0, 1 and 2
	// and so when we do subdivision we will have to create all the data on the stack and then reload it for testing
	BAL retAddress, TEST_EDGES				// gets test results for whether each edge needs subdivision, and the number of edges which need subdividing

	// Test vertices to see whether this is clipped - if so then abandon
	ILW.w	iClip0, 0(iNextTriToProcess)		// NOTE : these are all pre-masked to 0xfff
	ILW.w	iClip1, 4(iNextTriToProcess)
	ILW.w	iClip2, 8(iNextTriToProcess)

#ifdef MANUAL_OPT
	IADDIU	iTemp, VI00, 0xBC0	// clip mask for later, to use up spare calcs - note no longer checking far plane (+z)
#endif
#ifdef MANUAL_OPT
	IADDIU	iTemp2, VI00, EDGES_UNSPLIT	// moved up from way down to use what would otherwise always be a stall
#endif

#ifdef MANUAL_OPT
	// do some bits early - to optimize branches
	LQ.z		ftemp, 1(iNextTriToProcess)	// subdivision count stored in UV.z
	LOI		10.0 // limited number of subdivisions
#endif

	IAND	iAllClip, iClip0, iClip1
	IAND	iAllClip, iAllClip, iClip2

#ifdef MANUAL_OPT
	IOR		iNoClip, iClip0, iClip1
#endif
	// now if clip0 is non zero then the whole thing is off one side or another
	// NOTE : this is equally true if it is outside the screen clip, and conveniently it will 
	// get thrown out in that case too !!!
	IBNE	iAllClip, VI00, REJECT_THIS_TRI	

#ifdef MANUAL_OPT
	IOR		iNoClip, iNoClip, iClip2
#endif
#ifdef MANUAL_OPT
	SUBi.z	ftemp, ftemp, I
#endif

	// if all edges would need splitting anyway, we may as well subdivide it and not bother with lots more tests
	IBEQ	iEdges, VI00, CLIPPED_SUBDIVIDE	
		
	// also, if NO part is GBC clipped, then we don't really need to waste any more time - we can just
	// process it as if there is nothing to worry about
#ifdef AUTO_OPT
	IOR		iNoClip, iClip0, iClip1
#endif
#ifdef AUTO_OPT
	IOR		iNoClip, iNoClip, iClip2
#endif
	IAND	iNoClip, iNoClip, iTemp

#ifdef MANUAL_OPT
	// start off by masking off the screen clip bits as we only care about the GBC clip cases
	IAND	iClip0, iClip0, iTemp
#endif

	IBEQ	iNoClip, VI00, NOT_GBC_CLIPPED

	// ok, we now know that there are some vertices which are off the guard band, so we MUST subdivide 
	// any edge which crosses from in to outside GBC

#ifdef AUTO_OPT
	IADDIU	iTemp, VI00, 0xBC0	// clip mask for later, to use up spare calcs - note no longer checking far plane (+z)
#endif

#ifdef AUTO_OPT
	// start off by masking off the screen clip bits as we only care about the GBC clip cases
	IAND	iClip0, iClip0, iTemp
#endif
	IAND	iClip1, iClip1, iTemp
	IAND	iClip2, iClip2, iTemp

	// now compare them all together with each other
	ISUB	iClip0, iClip0, iClip1
	ISUB	iClip1, iClip1, iClip2
	IADD	iClip2, iClip1, iClip0	;// (A-B) + (B-C) = (A-C)


#ifdef MANUAL_OPT
	IADDIU	iMask, VI00, EDGE_01_FORCE_MASK  	// mask out bit 1 (out of 1, 2 and 3)
#endif
	// now if each case is non zero then that implies the edge needs splitting
	IBEQ	iClip0, VI00, EDGE_01_IS_OK

#ifdef AUTO_OPT
	IADDIU	iMask, VI00, EDGE_01_FORCE_MASK  	// mask out bit 1 (out of 1, 2 and 3)
#endif
	IAND	iEdges, iEdges, iMask

EDGE_01_IS_OK:

#ifdef MANUAL_OPT
	IADDIU	iMask, VI00, EDGE_12_FORCE_MASK  	// mask out bit 2 (out of 1, 2 and 3)
#endif
	IBEQ	iClip1, VI00, EDGE_12_IS_OK

#ifdef AUTO_OPT
	IADDIU	iMask, VI00, EDGE_12_FORCE_MASK  	// mask out bit 2 (out of 1, 2 and 3)
#endif
	IAND	iEdges, iEdges, iMask

EDGE_12_IS_OK:

#ifdef MANUAL_OPT
	IADDIU	iMask, VI00, EDGE_20_FORCE_MASK  	// mask out bit 3 (out of 1, 2 and 3)
#endif
	IBEQ	iClip2, VI00, CLIPPED_SUBDIVIDE		// note :: this was EDGE_20_IS_OK but might as well save a branch

#ifdef AUTO_OPT
	IADDIU	iMask, VI00, EDGE_20_FORCE_MASK  	// mask out bit 3 (out of 1, 2 and 3)
#endif
	IAND	iEdges, iEdges, iMask

EDGE_20_IS_OK:

	// otherwise we subdivide and continue - I think ?
	B CLIPPED_SUBDIVIDE

NOT_GBC_CLIPPED:

#ifdef AUTO_OPT
	IADDIU	iTemp2, VI00, EDGES_UNSPLIT
#endif
	IBEQ	iEdges, iTemp2, JUST_RENDER_ME

CLIPPED_SUBDIVIDE:

	// check there is still room left for this - otherwise just abandon
	ISUBIU	iStackSpace, iNextTriToProcess, ASURA_PS2_WATER__ADDR__VERT_STACK_FULL

#ifdef AUTO_OPT
	// do some bits early - to optimize branches
	LQ.z		ftemp, 1(iNextTriToProcess)	// subdivision count stored in UV.z
	LOI		10.0 // limited number of subdivisions
#endif

#ifdef MANUAL_OPT
	FSAND	iTemp, 0x2
#endif
	IBGTZ	iStackSpace, OUT_OF_SPACE_RENDER_ME

	// first we'll check for subdivision limiting
#ifdef AUTO_OPT
	SUBi.z	ftemp, ftemp, I
#endif
#ifdef AUTO_OPT
	FSAND	iTemp, 0x2
#endif
	IBEQ	iTemp, VI00, OUT_OF_SPACE_RENDER_ME	// render triangle if hit subdivision limit


	// TRIANGLE_SUBDIVIDE will just generate 2, 3 or 4 triangles on the stack, over-writing the current one
	BAL retAddress, TRIANGLE_SUBDIVIDE
	B POST_RENDER_CONTINUE

OUT_OF_SPACE_RENDER_ME:

JUST_RENDER_ME:
	// call kick code if needed

	// check if buffer is too full
	IADDIU	iSpaceLeft, pOutputBuffer, (ASURA_PS2_WATER__SIZE__OUTPUT_BFR - 10)
	ISUB	iSpaceLeft, iSpaceLeft, pVertOut

	IBGTZ	iSpaceLeft, STILL_ROOM


	BAL		retAddress, JUST_KICK_IT

STILL_ROOM:

	// will need these presets soon, so load them now
	frac_space_presets	

.macro PreRender

	    ILW.z pCurPerObjCamData,   ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)

		LQ			fCamPos, ASURA_PS2_WATER__OFFSET__CAM_PER_OBJ_CAM_POS( pCurPerObjCamData )
		MatrixLoad mTex, ASURA_PS2_WATER__OFFSET__CAM_PER_OBJ_TEX_MAT, pCurPerObjCamData
		LQ	fTexRot, ASURA_PS2_WATER__OFFSET__CAM_PER_OBJ_TEX_ROTATE( pCurPerObjCamData );

.endm
.macro RenderOneVert offset, adc, output
	LQ		viewPos, (\offset + 3)(iNextTriToProcess)
	LQ		fFScales, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+4)(VI00)
	LQ.xy	fHorizon, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+5)(VI00)
	LQ.z	fHorizon, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+7)(VI00)

	ADDw.xyz	fFalloff, VF00, VF00			// = 1,1,1
	ADDAw.xyz	ACC, fFalloff, VF00				// = 2,2,2
	MSUBw.xyz	fFalloff, fFScales, viewPos[w]	// = (2 - Z/f)  where f is falloff distance for each channel

	LQ		fWorldPos, (\offset)(iNextTriToProcess)
	LQ		RGB, (\offset + 2)(iNextTriToProcess)
	LQ		UV,  (\offset + 1)(iNextTriToProcess)

	IADDIU    iNegativeXYZ, VI00, 0xE0
	FMAND	  iTest, iNegativeXYZ
	IBEQ	  iTest, iNegativeXYZ, ZERO_HEIGHT\@

	compute_frac_space	fInterim, fWorldPos, fWorldPos, xyz
	compute_polynomial fHeights, fInterim, xyz

	//apply falloff scales (clamping first)
	MINIw.xyz	fFalloff, fFalloff, VF00[w]
	MAXx.xyz	fFalloff, fFalloff, VF00[x]
	MUL.xyz		fHeights, fHeights, fFalloff

	// now we modify the fWorldPos = but in a copy as it hasn't been stored yet

	MUL.xyz		fHeights, fHeights, fYScales		// scale up all 3 waves
	ADDx.y		fHeights, fHeights, fHeights		// sum the two waves we are using for the geometry
	ADD.y		fWorldPos, fWorldPos, fHeights		// final position with wave form added

    IADDIU iMask, VI00, ASURA_PS2_WATER__MASK__MULTIPASS
	ILW.y	iMeshFlags, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)
    IAND iMultiPass, iMeshFlags, iMask
    // Go off and do other passes if necessary.
    IBEQ iMultiPass, VI00, POST_PASS_2_COMPUTE\@


//####################################################################################
//##    Stuff which is required for all extra passes                                ##
//####################################################################################

	// using Martins directly computed formula for the normal - required for both kinds of extra pass, so shared code
	LOI	3.0
	MULi.xyz		fThree, fInterim, I
	LOI 0.5
	ADDAi.xyz		ACC, VF00, I		// = 0.5 across all three
	MADDAw.xyz		ACC, fThree, VF00	// - 3 x frac space
	MADD.xyz		fThree, fThree, fInterim	// plus 3 x frace space squared

	MUL.xyz			fThree, fThree, fFalloff	// scale down for fall off

	// now the actual normal requires sum of these applied to the component scales
	MUL.xyz			fXBits, fThree, fXScales
	MUL.xyz			fZBits, fThree, fZScales

	MUL.xyz			fXBits, fXBits, fYScales	// applying vertical scale in world to horizontal scale in normal
	MUL.xyz			fZBits, fZBits, fYScales

	ADDy.x			fXBits, fXBits, fXBits
	ADDy.z			fZBits, fZBits, fZBits

	ADDz.x			fNormal, fXBits, fXBits
	ADDx.z			fNormal, fZBits, fZBits
	SUBw.y			fNormal, VF00,   VF00

	// normalise fNormal - note both normals are now flattened 
	MULz.y          fNormalPass2, fNormal, fHorizon
	MOVE.xz         fNormalPass2, fNormal
	VectorNormalize fNormalPass2, fNormalPass2

	MUL.y			fNormal, fNormal, fHorizon
	VectorNormalize fNormal, fNormal

	// compute view vector - fWorldPos - fcamPos
        // Convert position to camera relative       .// NOTE : not overwriting fWorldPos
		SUB		fPos, fWorldPos, fCamPos

        // Normalise the view space position. This is then just a direction
        // from the camera.
        VectorNormalize fDir, fPos
        // Calculate 2(D.N).
        VectorDotProduct fDot, fDir, fNormalPass2
        VectorDotProduct fDotP1, fDir, fNormal

		B NOT_ZERO_HEIGHT\@
ZERO_HEIGHT\@:
		SUB		fPos, fWorldPos, fCamPos
        VectorNormalize fDir, fPos

		// all normals are 0,-1,0, so we can bypass dot products
		SUBy.x	fDot, VF00, fDir
		SUBy.x  fDotP1, VF00, fDir	// yes, its the same, but so what, it keeps the code simple!
		MOVE.xyz fHeights, VF00
		MOVE.xz fNormalPass2, VF00
		SUBw.y	fNormalPass2, VF00, VF00
		MOVE.xz fNormal, VF00
		SUBw.y	fNormal, VF00, VF00
	
		IADDIU iMask, VI00, ASURA_PS2_WATER__MASK__MULTIPASS
		ILW.y	iMeshFlags, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)
		IAND iMultiPass, iMeshFlags, iMask
		// Go off and do other passes if necessary.
		IBEQ iMultiPass, VI00, POST_PASS_2_COMPUTE\@

NOT_ZERO_HEIGHT\@:
		// incidental calculation of fresnel term - approximated as :
		// R0 + (1-dot)^2 - we should then cube this for pass 2 !
		// this has well distributed error levels which never exceed 3% of total light strength
		// note : couldn't generate two fresnel terms (which is what I should really do) because there isn't 
		// anywhere to store them.  so instead we take a rough approximation (max error appx +/-10 in 8 bit value)

#define fChosenDot  fDotP1

		ABS.x	fDot2, fChosenDot
		SUBx.w	fChosenDot, VF00, fDot2	// fDot.w = 1 - dot product 
		MUL.w	fFresnel, fChosenDot, fChosenDot	// (1-dot)^2

		// put fresnel term in over the output texture co-ordinate data for recovery in multi-pass render stage
		ADDx.w	UV, fFresnel, VF00	

//####################################################################################
//##    Stuff which is only required for pass one                                   ##
//####################################################################################

	// take reflection vector and convert to appropriate space by offset and scale

	// revised plan uses input data as follows :

	// from per-camera data : sin, cos, xOffset
	// from overall controller : y offset, y scale
	// constant value : 1/(2 * pi) for x scale

	// note x and y calculations are all done in z and w because this is going to be stored this way
	// in order not to conflict with pass 2

	// load horizon controls data
	// x = brightness, y = normal y scale, z = V offset, w = V scale
	LQ.zw	fTexScale, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+5)(VI00)

        ADD.x fDotP1, fDotP1, fDotP1
        // Calculate R = D - 2(D.N)N
        MULx.xyz fR, fNormal, fDotP1
        SUB.xyz  fR, fDir, fR

	MULz.w	fTexOff, VF00, fTexScale
	ADDw.z	fTexOff, VF00, fTexRot[w]

	LOI		0.1591549	// 1/2pi
	ADDi.z	fTexScale, VF00, I
	
	MULy.w	fST, VF00, fR
	MULAx.z	ACC, fTexRot, fR[x]		// ACC.z = fR.x * cos
	MSUBx.z fST, fR, fTexRot[x]		// fST.z = (fR.x * cos) - (fR.z * sin)

	ADDA.wz	ACC, VF00, fTexOff		// add offsets 
	MADD.wz	fST, fST, fTexScale		// and then scaled texture co-ords from reflection vector

	SQ.wz   fST, (pVertExtra)
	

//####################################################################################
//##    Stuff which is only required for pass two                                   ##
//####################################################################################
POST_PASS_1_COMPUTE\@:

        ADD.x fDot, fDot, fDot
        // Calculate R = D - 2(D.N)N
        MULx.xyz fR2, fNormalPass2, fDot
        SUB.xyz  fR2, fDir, fR2

        // Apply texture coordinate matrix.
        MatrixMultiplyVertexW1 fST, mTex, fR2

        // Is the vertex between the camera and the sun? Negative dot product
        // => it's not - the dir is from the camera, not to it, but we negate
        // the sun direction to compensate. If it's not between them, force
        // the ST to {-10,-10}, which we'll never get otherwise.
        LQ.xyz fNegSunDir, ASURA_PS2_WATER__ADDR__NEG_SUN_DIR(VI00)
        VectorDotProduct fCamSunDot, fDir, fNegSunDir
        IADDIU iXSignMask, VI00, 0x80
        FMAND iIsNegative, iXSignMask
        IBEQ iIsNegative, VI00, PASS_2_DO_NOT_KILL_SUN\@
        LOI -10.0
        ADDi.xy fST, VF00, I

PASS_2_DO_NOT_KILL_SUN\@:

        SQ.xy fST, (pVertExtra)

POST_PASS_2_COMPUTE\@:

	IADDIU pVertExtra, pVertExtra, 1	// having done one or other pass, we move the pointer on

	// now transform the newly offset vertex
	MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fWorldPos

	// Scale and translate to screenspace.
    MULA  ACC,   fGBCPos,     fGBC2SScale
    MADDw viewPos, fGBC2STrans, fGBCPos[w]
    // Perspective divide XYZ and STs.
    DIV      Q,     VF00[w], viewPos[w]

    // Perform clipping judgement.
    CLIPw.xyz fGBCPos, fGBCPos

	// also perform clipping in Screen Space !!!
	MUL fSCPos, fGBCPos, fGBC2SCScale
	CLIPw.xyz fSCPos, fSCPos
	FCGET	\output

    MULq.xyz viewPos, viewPos,   Q
	ADDw.z   UV, VF00, VF00				// UV.z needs to be Q afterwards, so 1.0 now
	MULq.xyz  UV, UV, Q

	MUL.y		fHeights, fHeights, fYScales[w]
	ADD.y		fHeights, fHeights, fFScales[w]

//	SQ.w		RGB, -1(pVertExtra)		// stuff the overall RGB scale into storage for use by the sunlight
// The above has been replaced with fresnel term for better looking reflection

	// now apply this value to the colour
	MULy.xyz	RGB, RGB, fHeights[y]	// ranges from 1 to appx 1.25 I think

	// and set transparency too - lighter means more solid :
	
	MUL.w	RGB, fTScales, VF00		// Alpha set to constant from parameters

	LOI 255
	MINIi	RGB, RGB, I	// prevent overflow !!!!
	MAX.xyz	RGB, RGB, VF00	// and underflow

	// get various bits into right scale
    FTOI4.xyz viewPos, viewPos
    FTOI0     RGB, RGB

	// Generate ADC bit to make wireframe look better
.aif \adc eq 1
	IADDIU	iTemp, VI00, 0x7fff
	IADDIU	iTemp, iTemp, \adc
	MFIR.w	viewPos, iTemp
.aelse
    MFIR.w  viewPos, VI00
.aendi
	// Store 3 verts to output buffer
    SQ      UV,		0(pVertOut)
    SQ      RGB,	1(pVertOut)
    SQ		viewPos,  2(pVertOut)
	IADDIU pVertOut, pVertOut, 3
	IADDIU iVertexQty, iVertexQty, 1
.endm

	PreRender
--barrier
	RenderOneVert 0, 1, iClip0
--barrier
	RenderOneVert 4, 0, iClip1
--barrier
	RenderOneVert 8, 0, iClip2
--barrier

#ifdef COUNT_POLYS
	CountOne output
#endif

	IAND	iAllClip, iClip0, iClip1
	IAND	iAllClip, iAllClip, iClip2

	// now if iAllclip is non zero then the whole thing is off one side or another
	// NOTE : this is equally true if it is outside the screen clip, and conveniently it will 
	// get thrown out in that case too !!!
	IBNE	iAllClip, VI00, DROP_THIS_TRI

	// also, if NO part is GBC clipped, then we don't really need to waste any more time - we can just
	// process it as if there is nothing to worry about

	IOR		iAllClip, iClip0, iClip1
	IOR		iAllClip, iAllClip, iClip2
	IADDIU	iTemp, VI00, 0xfc0
	IAND	iAllClip, iAllClip, iTemp

	IBEQ	iAllClip, VI00, THIS_TRI_IS_DONE

DROP_THIS_TRI:
#ifdef COUNT_POLYS
	CountOne output, sub
	CountOne reject
#endif
	ISUBIU pVertOut, pVertOut, 9
	ISUBIU iVertexQty, iVertexQty, 3
	ISUBIU	pVertExtra, pVertExtra, 3

#ifdef POST_RENDER_BFC

	// need to do cross product to find out if back facing
	SUB.xyz	view0, view0, view1
	SUB.xyz	view1, view1, view2

	OPMULA	ACC, view0,view1
	OPMSUB	view0, view0, view1

	//if view0.z is negative then we are backfacing
	IADDIU	iMask, VI00, 0x20	// negative Z
	FMAND	iMask, iMask
	IBEQ	iMask, VI00, THIS_TRI_IS_DONE

	ISUBIU pVertOut, pVertOut, 9
	ISUBIU iVertexQty, iVertexQty, 3
	ISUBIU	pVertExtra, pVertExtra, 3
#endif

REJECT_THIS_TRI:

#ifdef COUNT_POLYS
	CountOne reject
#endif

THIS_TRI_IS_DONE:	
	// decrement tris in stack
	ISUBIU iTrisOnStack, iTrisOnStack, 1
	ISUBIU iNextTriToProcess, iNextTriToProcess,12		// stored in 4 QW per vert buffer

POST_RENDER_CONTINUE:

	// loop if there aren't any more triangles to process from the subdivision
	IBEQ	iTrisOnStack, VI00, DONE_THIS_SET

	// if there are, then we must load the next set of verts and continue processing
	LQ fOut0,  3(iNextTriToProcess)
	LQ fOut1,  7(iNextTriToProcess)
	LQ fOut2, 11(iNextTriToProcess)

	B TRI_LOOP_POINT

DONE_THIS_SET:

	ILW.w	data, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)
	ILW.x	count, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)

	IADDIU  data, data, 6	// one triangle, 3 verts, 2 QW each
	ISUBIU  count, count, 1

	ISW.w	data, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)
	ISW.x	count, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)

	IBNE	count, VI00, VU1_WATER_NEXT_TRI	// we are done if our count hits zero

	IBEQ	iVertexQty, VI00, DO_NEXT_CAM_STUFF		// if we are done, and nothing left to render, just TRY FOR ANOTHER CAMERA!

	BAL retAddress, JUST_KICK_IT						// otherwise, render it !
	
	// now that we have detected that this section is complete, check if any more cameras are needed.

DO_NEXT_CAM_STUFF:
    // Mask top two bits off cam flags so that next time we run the
    // VU1_GSM_FIND_NEXT_CAM_LOOP we'll be forced to move ptrs on, etc.
    ILW.x iRemainingCamFlags, ASURA_PS2_WATER__ADDR__VI_STACK_4(VI00)
    IADDIU iMask, VI00, 0x3F
    IAND iRemainingCamFlags, iRemainingCamFlags, iMask
	ISW.x iRemainingCamFlags, ASURA_PS2_WATER__ADDR__VI_STACK_4(VI00)
    // And on to the next camera.

	B VU1_WATER_RENDER_NEXT_CAMERA
//	B VU1_WATER_NOP


VU1_WATER_GET_TRIANGLE:

	ILW.w	data, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)

	// prepare local copies of pointers - leave it to the calling end to decide how far to move things
	IADD		localData, data, VI00
	IADD		localStack, pVertStack, VI00

	LQ.xy		fUVOffset, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+7)(VI00)

.macro processVertex outreg
	// load data
	LQI		fWorldPos, (localData++)
	LQI		colour,	  (localData++)

	// rearrange
	MOVE.x  fUV, colour			// copy RGB.x into U
	ADDw.y  fUV, VF00, fWorldPos	// copy worldpos.w into V
	MR32.xyz colour, colour		// roll colour round to right place
	MAX.zw	fUV, VF00, VF00			// put 0.0f, 1.0f into UV.w and z
//	LOI 100.0	// temporarily put fixed alpha 255 into alpha channel
//	MAXi.w  colour, VF00, I		// note this will be replaced by a parameter probably

	// apply constant UV offset
	ADD.xy  fUV, fUV, fUVOffset

	// NOTE : subdivision is performed without applying wave offsets, to avoid flickery changes as the waves move
	// Transform to GB clip space.
    MatrixMultiplyVertexW1 fGBCPos, mO2GBC, fWorldPos

    // Perform clipping judgement.
    CLIPw.xyz fGBCPos, fGBCPos

	// also perform clipping in Screen Space !!!
	MUL fSCPos, fGBCPos, fGBC2SCScale
	ADD.w	fSCPos, fSCPos, fMaxOffset		// note generous excess allowed to ensure we never clip something which could get on screen
	CLIPw.xyz fSCPos, fSCPos

    FCGET iClipFlags			// no need to mask as I want 12 bits

	MFIR.w fWorldPos, iClipFlags		// store clip flags back into worldpos
        	
	// Scale and translate to screenspace.
    MULA  ACC,   fGBCPos,     fGBC2SScale
    MADDw \outreg, fGBC2STrans, fGBCPos[w]
    // Perspective divide XYZ and STs.
    DIV      Q,     VF00[w], \outreg[w]

    MULq.xyz \outreg, \outreg,   Q
	MULq.w   fUV, fUV, Q

	// store data
	SQI		fWorldPos, (localStack++)
	SQI		fUV,	   (localStack++)
	SQI		colour,   (localStack++)
	SQI		\outreg,  (localStack++)	

.endm

    // Load GBC->SC scale vector.in preparation for screen clip testing
    ILW.w pCurConstCamData, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
    LQ fGBC2SCScale, ASURA_PS2_WATER__OFFSET__CONST_CAM_GBC2SC_SCALE(pCurConstCamData)

	processVertex	fOut0
	processVertex	fOut1
	processVertex	fOut2

	JR	retAddress

--barrier

TEST_EDGES:
	// subdivision threshold value 
	LQ.w	thresh, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+1)(VI00)

#if CAMERA_SPECIFIC_FOV_THRESHOLD
	// bias threshold for current camera zoom
    ILW.z pCurPerObjCamData, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
	LQ.y	fFOVThresh, ASURA_PS2_WATER__OFFSET__CAM_PER_OBJ_TEX_ROTATE( pCurPerObjCamData );
	MULy.w  thresh, thresh, fFOVThresh
#endif

	LQ.y	fTemp, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+6)(VI00)	// y scale for subdivision

	SUB.xy	fDist01, fOut0, fOut1
	SUB.xy	fDist12, fOut1, fOut2
	SUB.xy	fDist20, fOut2, fOut0

	// idea for consideration - rearrange to get xxx, yyy rather than xy,xy,xy

	ADDy.xz	fTemp, VF00, fTemp	//	duplicate Y scale value for three parts

	// combine rearranging and scaling of Y components into a single process
	MULy.x	f3Y, fTemp, fDist01
	MUL.y	f3Y, fTemp, fDist12
	MULy.z  f3Y, fTemp, fDist20

	ADDx.z  fDist01, VF00, fDist20
	ADDx.y  fDist01, VF00, fDist12


	// do the dot products
	SUBAw.xyz	ACC, VF00, thresh
	MADDA.xyz	ACC, fDist01, fDist01	// swapped x and y parts to assist scheduling
	MADD.xyz	fOut, f3Y, f3Y
	
	IADDIU	iEdgeTest, VI00, EDGE_TEST_BITS	
	FMAND   iEdges, iEdgeTest		// test for negatives in X Y and Z

	// so we are left with iEdges = 8 for edge 20, plus 4 for edge 12, plus 2 for edge 01

	JR	retAddress

--barrier

JUST_KICK_IT:
	// out of space so need to kick - but do render state first

VU1_WATER_RENDER_PASS_0:

    // Retrieve flag and ptrs from stack.
    ILW.w iNeedToSendRS, ASURA_PS2_WATER__ADDR__VI_STACK_4(VI00)
    ILW.w pRSInputBfr,   ASURA_PS2_WATER__ADDR__VI_STACK_5(VI00)
    // Reset need-to-send-RS flag for next time. For the default value, use
    // the can-more-than-camera-see-this-mesh flag value; we'll need to send
    // RS every time if there is more than one camera involved.
    ILW.x iMoreThanOneCamFlag, ASURA_PS2_WATER__ADDR__VI_STACK_5(VI00)
    ISW.w iMoreThanOneCamFlag, ASURA_PS2_WATER__ADDR__VI_STACK_4(VI00)
    // Setup RS output ptr. May not need it, of course.
    IADDIU pRSOutputBfr, VI00, ASURA_PS2_WATER__ADDR__RS_OUTPUT_BFR
    // Is previous renderstate still ok?
//    IBEQ iNeedToSendRS, VI00, VU1_WATER_RENDER_PASS_0_KICK

VU1_WATER_RENDER_PASS_0_RS:

    ILW.w pCurConstCamData, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
    // Build base pass camera and renderstate setup list.
    LQ fSetupGIFTag, (ASURA_PS2_WATER__ADDR__GIF_TAGS + ASURA_PS2_WATER__OFFSET__GIF_TAGS_GEOM_SETUP)(VI00)

    LQ fSCISSOR,     ASURA_PS2_WATER__OFFSET__CONST_CAM_SCISSOR(pCurConstCamData)
    LQ fXYOFFSET,    ASURA_PS2_WATER__OFFSET__CONST_CAM_XYOFFSET(pCurConstCamData)
    LQ fALPHA,       ASURA_PS2_WATER__OFFSET__RS_IN_BASE_ALPHA(pRSInputBfr)
    LQ fTEX1,        ASURA_PS2_WATER__OFFSET__RS_IN_BASE_TEX1(pRSInputBfr)
    LQ fTEX0,        ASURA_PS2_WATER__OFFSET__RS_IN_BASE_TEX0(pRSInputBfr)
    LQ fMIPTBP1,     ASURA_PS2_WATER__OFFSET__RS_IN_BASE_MIPTBP1(pRSInputBfr)
    LQ fTEST,        ASURA_PS2_WATER__OFFSET__RS_IN_BASE_TEST(pRSInputBfr)

	SUB.xyw		fCLAMP, VF00, VF00
	IADDIU		iCLAMP, VI00, 8
	MFIR.z		fCLAMP, iCLAMP

    SQ fSetupGIFTag, ASURA_PS2_WATER__OFFSET__RS_OUT_0_SETUP_GIF_TAG(pRSOutputBfr)
    SQ fSCISSOR,     ASURA_PS2_WATER__OFFSET__RS_OUT_0_SCISSOR(pRSOutputBfr)
    SQ fXYOFFSET,    ASURA_PS2_WATER__OFFSET__RS_OUT_0_XYOFFSET(pRSOutputBfr)
    SQ fALPHA,       ASURA_PS2_WATER__OFFSET__RS_OUT_0_ALPHA(pRSOutputBfr)
    SQ fTEX1,        ASURA_PS2_WATER__OFFSET__RS_OUT_0_TEX1(pRSOutputBfr)
    SQ fMIPTBP1,     ASURA_PS2_WATER__OFFSET__RS_OUT_0_MIPTBP1(pRSOutputBfr)
    SQ fTEX0,        ASURA_PS2_WATER__OFFSET__RS_OUT_0_TEX0(pRSOutputBfr)
    SQ fTEST,        ASURA_PS2_WATER__OFFSET__RS_OUT_0_TEST(pRSOutputBfr)

	SQ fCLAMP,		 ASURA_PS2_WATER__OFFSET__RS_OUT_0_CLAMP(pRSOutputBfr)

    // And send it.
    KickMacro pRSOutputBfr

VU1_WATER_RENDER_PASS_0_KICK:

	// fill in GIF tag (minus EOP and NLOOP)
    LQ.yzw fDrawGIFTag, (ASURA_PS2_WATER__ADDR__GIF_TAGS + ASURA_PS2_WATER__OFFSET__GIF_TAGS_GEOM_DRAW)(VI00)
    SQ.yzw fDrawGIFTag, ASURA_PS2_WATER__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pOutputBuffer)

	// fill in number of verts
    IADDIU iNLOOPandEOP, iVertexQty,   0x7FFF
    IADDIU iNLOOPandEOP, iNLOOPandEOP, 1
    ISW.x  iNLOOPandEOP, ASURA_PS2_WATER__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pOutputBuffer)

	// copy buffer pointer to allow delayed kick
	IADD	pOldBuffer, pOutputBuffer, VI00

	// swap buffers
	IADDIU	pOutputBuffer, VI00, ASURA_PS2_WATER__ADDR__GEOM_OUTPUT_BFR_0 + ASURA_PS2_WATER__ADDR__GEOM_OUTPUT_BFR_1
	ISUB	pOutputBuffer, pOutputBuffer, pOldBuffer
	IADDIU	pVertOut, pOutputBuffer, ASURA_PS2_WATER__OFFSET__GEOM_OUT_FIRST_VERTEX

	// check whether we need to perform multi-pass kicking 
    IADDIU iMask, VI00, ASURA_PS2_WATER__MASK__MULTIPASS
	ILW.y	iMeshFlags, ASURA_PS2_WATER__ADDR__VI_STACK_0(VI00)
    IAND iMultipass, iMeshFlags, iMask
    // And send base pass.
	KickMacro pOldBuffer
    // Go off and do other passes if necessary.
    IBEQ iMultipass, VI00, POST_PASS_2_KICK

--barrier
    IADDIU iMask, VI00, ASURA_PS2_WATER__MASK__PASS_1
    IAND iMultipass, iMeshFlags, iMask

    IBEQ iMultipass, VI00, VU1_WATER_PASS_2_SETUP

VU1_WATER_PASS_1_SETUP:
    ILW.w	pRSInputBfr,   ASURA_PS2_WATER__ADDR__VI_STACK_5(VI00)
    IADDIU	pRSOutputBfr, VI00, ASURA_PS2_WATER__ADDR__RS_OUTPUT_BFR
	LQ		fXScales, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS)(VI00)

	// load horizon controls data
	// x = brightness, y = normal y scale, z = V offset, w = V scale
	LQ.x	fHorizon, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+5)(VI00)

    // Build the ST overwrite pass setup list.
	IADDIU		iTemp, VI00, ASURA_PS2_WATER__FLAG__SWAP_BLEND_HORIZON
	IAND		iTemp, iTemp, iMeshFlags
	IADDIU		pAlpha, pRSInputBfr, ASURA_PS2_WATER__OFFSET__RS_IN_SUN_ALPHA
	ISUB		pAlpha, pAlpha, iTemp
    LQ fALPHA,    0(pAlpha)

    LQ fTEX1,     ASURA_PS2_WATER__OFFSET__RS_IN_HORIZON_TEX1(pRSInputBfr)
    LQ fTEX0,     ASURA_PS2_WATER__OFFSET__RS_IN_HORIZON_TEX0(pRSInputBfr)
    LQ fMIPTBP1,  ASURA_PS2_WATER__OFFSET__RS_IN_HORIZON_MIPTBP1(pRSInputBfr)
    SQ fALPHA,    ASURA_PS2_WATER__OFFSET__RS_OUT_1_ALPHA(pRSOutputBfr)
    SQ fTEX1,     ASURA_PS2_WATER__OFFSET__RS_OUT_1_TEX1(pRSOutputBfr)
    SQ fMIPTBP1,  ASURA_PS2_WATER__OFFSET__RS_OUT_1_MIPTBP1(pRSOutputBfr)
    SQ fTEX0,     ASURA_PS2_WATER__OFFSET__RS_OUT_1_TEX0(pRSOutputBfr)
    // Fix up NLOOP and EOP.
	IADDIU iNLOOPandEOP, VI00, 0x7fff
    IADDIU iNLOOPandEOP, iNLOOPandEOP, (ASURA_PS2_WATER__RS_OUT_1_NLOOP + 1)
    ISW.x iNLOOPandEOP, ASURA_PS2_WATER__OFFSET__RS_OUT_1_SETUP_GIF_TAG(pRSOutputBfr)

	IADDIU		iCLAMP, VI00, 4		// this is CLAMP V direction only
	ISW.x		iCLAMP,	ASURA_PS2_WATER__OFFSET__RS_OUT_1_CLAMP(pRSOutputBfr)

	// copy giftag from other buffer
	LQ	fDrawGIFTag, ASURA_PS2_WATER__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pOldBuffer)
	SQ  fDrawGIFTag, ASURA_PS2_WATER__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pOutputBuffer)

    KickMacro pRSOutputBfr

	// this whole section needs replacing, but first I need to write the code to create the reflected UVs
	// plus I need to find somewhere to store the fuckers as I have already used 3 of my registers
	    IADDIU  pOverwriteSTs, VI00, ASURA_PS2_WATER__ADDR__ST_OVERWRITE_BFR
		IADDIU	pOldBuffer, pOldBuffer, 1
		IADD	pOverwriteSTEnd, pOverwriteSTs, iVertexQty

		MAXx	fRGBA, VF00, fHorizon[x]	// make grey based on brightness required for horizon reflections
//		FTOI0	fRGBA, fRGBA

		LQ.w	fScaleFresnel, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+7)(VI00)	// scale factor to allow more transparency in blend mode horizon

	// loop over output and copy it, filling in new UV's as we go
VU1_WATER_PASS_1_LOOP;
        LQ.zw   fSTQ,     0(pOldBuffer)
        LQI.zw  fSTQ2,     (pOverwriteSTs++)
		ADDz.x  fSTQ, VF00, fSTQ2
		ADDw.y  fSTQ, VF00, fSTQ2

        //LQI     fRGBA,    (pOldBuffer++)
        LQ     fScrnPos, 2(pOldBuffer)
		IADDIU	pOldBuffer, pOldBuffer, 3		// note had to switch to this approach as we are skipping RGBA - oops

		MUL.w	fRGBA, fSTQ, fScaleFresnel	// use recorded fresnel times scale value to vary reflection power by putting it in alpha
		FTOI0	fRGBA2, fRGBA				// note creating a copy for the integer version so we still have the base version

        // Perspective divide texture coordinates.
        MULz.xy fSTQ, fSTQ, fSTQ[z]
        // Copy to new output buffer.
        SQI		fSTQ,     (pVertOut++)	// w required for next pass - its the fresnel factor, which was in STQ.w in pass 0  and so needs to be after pass 1 for pass 2 code to work !
        SQI		fRGBA2,   (pVertOut++)	// w required now for alpha as using alpha scaled additive to insert fresnel
        SQI     fScrnPos, (pVertOut++)

        // Done yet?
        IBNE pOverwriteSTs, pOverwriteSTEnd, VU1_WATER_PASS_1_LOOP

--barrier
		// note : not bothering to delay XGKICK because we aren't frigging with nloop etc
		KickMacro pOutputBuffer	

		// copy buffer pointer to allow delayed kick
		IADD	pOldBuffer, pOutputBuffer, VI00

		// swap buffers
		IADDIU	pOutputBuffer, VI00, ASURA_PS2_WATER__ADDR__GEOM_OUTPUT_BFR_0 + ASURA_PS2_WATER__ADDR__GEOM_OUTPUT_BFR_1
		ISUB	pOutputBuffer, pOutputBuffer, pOldBuffer
		IADDIU	pVertOut, pOutputBuffer, ASURA_PS2_WATER__OFFSET__GEOM_OUT_FIRST_VERTEX

VU1_WATER_PASS_2_SETUP:
    IADDIU iMask, VI00, ASURA_PS2_WATER__MASK__PASS_2
    IAND iMultipass, iMeshFlags, iMask

    IBEQ iMultipass, VI00, POST_PASS_2_KICK

		
	ILW.w pRSInputBfr,   ASURA_PS2_WATER__ADDR__VI_STACK_5(VI00)
    IADDIU pRSOutputBfr, VI00, ASURA_PS2_WATER__ADDR__RS_OUTPUT_BFR
	LQ			fXScales, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS)(VI00)

    // Build the ST overwrite pass setup list.
    LQ fALPHA,    ASURA_PS2_WATER__OFFSET__RS_IN_SUN_ALPHA(pRSInputBfr)
    LQ fTEX1,     ASURA_PS2_WATER__OFFSET__RS_IN_SUN_TEX1(pRSInputBfr)
    LQ fTEX0,     ASURA_PS2_WATER__OFFSET__RS_IN_SUN_TEX0(pRSInputBfr)
    LQ fMIPTBP1,  ASURA_PS2_WATER__OFFSET__RS_IN_SUN_MIPTBP1(pRSInputBfr)
    SQ fALPHA,    ASURA_PS2_WATER__OFFSET__RS_OUT_2_ALPHA(pRSOutputBfr)
    SQ fTEX1,     ASURA_PS2_WATER__OFFSET__RS_OUT_2_TEX1(pRSOutputBfr)
    SQ fMIPTBP1,  ASURA_PS2_WATER__OFFSET__RS_OUT_2_MIPTBP1(pRSOutputBfr)
    SQ fTEX0,     ASURA_PS2_WATER__OFFSET__RS_OUT_2_TEX0(pRSOutputBfr)
    // Fix up NLOOP and EOP.
	IADDIU iNLOOPandEOP, VI00, 0x7fff
    IADDIU iNLOOPandEOP, iNLOOPandEOP, (ASURA_PS2_WATER__RS_OUT_2_NLOOP + 1)
    ISW.x iNLOOPandEOP, ASURA_PS2_WATER__OFFSET__RS_OUT_2_SETUP_GIF_TAG(pRSOutputBfr)

	IADDIU		iCLAMP, VI00, 5		// this is CLAMP U and V directions
	ISW.x		iCLAMP,	ASURA_PS2_WATER__OFFSET__RS_OUT_1_CLAMP(pRSOutputBfr)
	
	// copy giftag from other buffer
	LQ	fDrawGIFTag, ASURA_PS2_WATER__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pOldBuffer)
	SQ  fDrawGIFTag, ASURA_PS2_WATER__OFFSET__GEOM_OUT_DRAW_GIF_TAG(pOutputBuffer)

    KickMacro pRSOutputBfr

	    IADDIU  pOverwriteSTs, VI00, ASURA_PS2_WATER__ADDR__ST_OVERWRITE_BFR
		IADDIU	pOldBuffer, pOldBuffer, 1
		IADD	pOverwriteSTEnd, pOverwriteSTs, iVertexQty

		MAXw	fRGBA, VF00, fXScales[w]	// make grey based on brightness required for reflections
//		FTOI0	fRGBA, fRGBA

	// loop over output and copy it, filling in new UV's as we go
VU1_WATER_PASS_2_LOOP:
		LQ.zw	fSunFallOff, (ASURA_PS2_WATER__ADDR__Y_SCALE_AND_TRANS_PARAMS+6)(VI00)	// sun fall off factors (generated from near and far)
        LQ.zw   fSTQ,     0(pOldBuffer)

		MUL.w	fSTQ, fSTQ, fSTQ				// square previous fresnel factor to make it more like the real thing (should be cubed really !)

        LQI.xy  fSTQ,     (pOverwriteSTs++)
        //LQI     fRGBA,    (pOldBuffer++)
        LQ     fScrnPos, 2(pOldBuffer)
		IADDIU	pOldBuffer, pOldBuffer, 3		// note had to switch to this approach as we are skipping RGBA - oops

		MUL.z	fSunFallOff, fSunFallOff, fSTQ
		SUBw.z	fSunFallOff, fSunFallOff, fSunFallOff
		MINIw.z	fSunFallOff, fSunFallOff, VF00[w]	// ensure no more than 1
		MAX.z   fSunFallOff, fSunFallOff, VF00[z]	// and no less than 0

		LOI		128		
		MULi.w	fRGBA2, fSTQ, I			// use recorded fresnel value to vary reflection power by putting it in alpha

		MULz.xyz fRGBA2, fRGBA, fSunFallOff[z]	// note creating a copy for the integer version so we still have the base version


        // If the ST value is {-10,-10} then the sun pass should be killed,
        // since the vertex is not between the camera and the sun.
        LOI 10.0
        ADD.xy fAlphaScale, fSTQ, I
        // That's now positive iff it should *not* be killed, otherwise it's
        // 0. Modify the value so that it remains at 0 if it's already 0
        // and else goes to 1. This trick is stolen from sce.dev.prog.vu.
        LOI 0x7f800000                            // Max valid float exponent
        MULi.x fAlphaScale, fAlphaScale, I        // Scale - smallest positive float becomes +4
        MINIw.x fAlphaScale, fAlphaScale, VF00[w] // Clamp to <= 1
        MAXx.x fAlphaScale, fAlphaScale, VF00[x]  // Clamp to <= 0
        // Scale the alpha by that, so that it goes to 0 iff the sun pass
        // should be killed.
        MULx.w fRGBA2, fRGBA2, fAlphaScale

		FTOI0	fRGBA2, fRGBA2

        // Perspective divide texture coordinates.
        MULz.xy fSTQ, fSTQ, fSTQ[z]		// STQ[z] is 1 / real world Z
        // Copy to new output buffer.
        SQI.xyz fSTQ,     (pVertOut++)
        SQI		fRGBA2,   (pVertOut++)
        SQI     fScrnPos, (pVertOut++)

        // Done yet?
        IBNE pOverwriteSTs, pOverwriteSTEnd, VU1_WATER_PASS_2_LOOP

--barrier
		// note : not bothering to delay XGKICK because we aren't frigging with nloop etc
		KickMacro pOutputBuffer	

		// copy buffer pointer to allow delayed kick
		IADD	pOldBuffer, pOutputBuffer, VI00

		// swap buffers
		IADDIU	pOutputBuffer, VI00, ASURA_PS2_WATER__ADDR__GEOM_OUTPUT_BFR_0 + ASURA_PS2_WATER__ADDR__GEOM_OUTPUT_BFR_1
		ISUB	pOutputBuffer, pOutputBuffer, pOldBuffer
		IADDIU	pVertOut, pOutputBuffer, ASURA_PS2_WATER__OFFSET__GEOM_OUT_FIRST_VERTEX

POST_PASS_2_KICK:
	// reset counters - saved until here so that I can keep using the count for other passes !
	IADD	iVertexQty, VI00, VI00

	// reset pointer for second pass data too, to stop it trashing everything
	IADDIU pVertExtra, VI00, ASURA_PS2_WATER__ADDR__ST_OVERWRITE_BFR

	JR retAddress

--barrier

// MACRO for use by subdivision code - takes a subdivision generated vertex, applies the oscillation,
// transforms it, and puts the Q value in the UV thing too !


.macro TransAndPersp outreg, inreg, outUV

    // Transform to GB clip space.
    MatrixMultiplyVertexW1 fGBCPos, mO2GBC, \inreg

    // Perform clipping judgement.
    CLIPw.xyz fGBCPos, fGBCPos

	// also perform clipping in Screen Space !!!
	MUL fSCPos, fGBCPos, fGBC2SCScale
	ADD.w	fSCPos, fSCPos, fMaxOffset
	CLIPw.xyz fSCPos, fSCPos

    FCGET iClipFlags	// no need to mask, I want 12 bits

	MFIR.w \inreg,   iClipFlags

    // Scale and translate to screenspace.
    MULA  ACC,   fGBCPos,     fGBC2SScale
    MADDw \outreg, fGBC2STrans, fGBCPos[w]
    // Perspective divide XYZ and STs.
    DIV      Q,     VF00[w], \outreg[w]

    MULq.xyz \outreg, \outreg,   Q
	MULq.w	 \outUV, \outUV, Q
.endm


	// Note : assumes iNextTriToProcess is pointing at the current triangle as a POINTER, not just index
TRIANGLE_SUBDIVIDE:

	// new improved subdivision system
	// needs to allow for 3 scenarios :
	// 1: only subdivide 1 edge, and output two triangles
	// 2: subdivide 2 edges, and output 3 triangles
	// 3: subdivide all 3 edges and output 4 triangles (already done of course !)

	// start off by deciding how many edges to subdivide !!!!!
//	IBEQ	iSplitEdges, VI00, SUBDIVIDE_3_EDGES	// special case - if nothing to divide it must be a clipped one

//	IADDIU	iCount3, VI00, 6
//	IBEQ	iSplitEdges, iCount3, SUBDIVIDE_3_EDGES

//	IADDIU	iCount2, VI00, 4
//	IBEQ	iSplitEdges, iCount2, SUBDIVIDE_2_EDGES

	IBEQ	iEdges, VI00, SUBDIVIDE_3_EDGES

	IADDIU	iTemp, VI00, EDGES_UNSPLIT	// none split
	IBEQ	iEdges, iTemp, SUBDIVIDE_3_EDGES

	// single edge means power of two, which we can detect ! - and is still true for new system !
	ISUBIU	iTemp, iEdges, 1
	IAND	iTemp, iTemp, iEdges

	IBEQ	iTemp, VI00, SUBDIVIDE_2_EDGES


// Fall through to case 1, since we cannot possibly get here with any other value

// *********************
// **     CASE 1      **
// *********************
SUBDIVIDE_1_EDGE:
	// find out which edge needs splitting, and then ensure that it is loaded as 
	// vertex 0-1, so that we know where to split

	IADDIU	iVert0, iNextTriToProcess, 0
	IADDIU	iVert1, iNextTriToProcess, 4
	IADDIU	iVert2, iNextTriToProcess, 8

	IADDIU	iTemp, VI00, EDGE_01_FORCE_MASK		// 12 is 8 + 4 = edges 12 and 20 are not split, so 01 is
	IBEQ	iEdges, iTemp, ALREADY01

	IADDIU	iTemp, VI00, EDGE_12_FORCE_MASK		// 10 is 8 + 2 = edges 01 and 20 are not split, so 12 is
	IBEQ    iEdges, iTemp, ITS12

MUST_BE_20:
	IADDIU  iVert0, iNextTriToProcess, 8
	IADDIU  iVert1, iNextTriToProcess, 0
	IADDIU  iVert2, iNextTriToProcess, 4
	B ALREADY01

ITS12:
	IADDIU  iVert0, iNextTriToProcess, 4
	IADDIU  iVert1, iNextTriToProcess, 8
	IADDIU  iVert2, iNextTriToProcess, 0

ALREADY01:

	LQ		world0, 0(iVert0):memgroup1
	LQ		world1, 0(iVert1):memgroup1
	LQ		world2, 0(iVert2):memgroup1
	LQ		view0,  3(iVert0):memgroup2
	LQ		view1,  3(iVert1):memgroup2
	LQ		view2,  3(iVert2):memgroup2

	// prepare UV.w for storage of Q values in W and subdiv count in Z
	MAXw.zw	UVA, VF00, VF00

	// compute half vectors for world space - have to be transformed for view space
	LOI 5e-1
	MULA	ACC,	world0, I
	MADD	worldA,	world1, I

    // Load GBC->SC scale vector.
    ILW.w pCurConstCamData, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
    LQ fGBC2SCScale, ASURA_PS2_WATER__OFFSET__CONST_CAM_GBC2SC_SCALE(pCurConstCamData)

	// transform half vectors for view space
	TransAndPersp viewA, worldA, UVA

	// store results in multiple locations as required
	// WORLD first
	SQ	world0, (0+ 0)(iNextTriToProcess):memgroup1
	SQ	worldA, (0+ 4)(iNextTriToProcess):memgroup1
	SQ	world2, (0+ 8)(iNextTriToProcess):memgroup1

	SQ	world1, (0+12)(iNextTriToProcess)
	SQ	world2, (0+16)(iNextTriToProcess)
	SQ	worldA, (0+20)(iNextTriToProcess)

	SQ	view0, (3+ 0)(iNextTriToProcess):memgroup2
	SQ	viewA, (3+ 4)(iNextTriToProcess):memgroup2
	SQ	view2, (3+ 8)(iNextTriToProcess):memgroup2

	SQ	view1, (3+12)(iNextTriToProcess)
	SQ	view2, (3+16)(iNextTriToProcess)
	SQ	viewA, (3+20)(iNextTriToProcess)

	//########################################################
	//stage 2: doing the same thing for the UV and RGB values
	//########################################################

	// load RGB and UV for 3 verts

	LQ		RGB0, 2(iVert0):memgroup3
	LQ		RGB1, 2(iVert1):memgroup3
	LQ		RGB2, 2(iVert2):memgroup3
	LQ		UV0,  1(iVert0):memgroup4
	LQ		UV1,  1(iVert1):memgroup4
	LQ		UV2,  1(iVert2):memgroup4

	// compute half vectors for RGB and UV
	LOI 0.5
	MULA	ACC,  RGB0, I
	MADD	RGBA, RGB1, I

	// subdivision counter increment
	ADDw.z	UV0, UV0, VF00
	ADDw.z	UV1, UV1, VF00
	ADDw.z	UV2, UV2, VF00

	MULA.xyz	ACC, UV0, I
	MADD.xyz	UVA, UV1, I

	// store results in multiple locations as required
	// RGB first
	SQ	RGB0, (2+ 0)(iNextTriToProcess):memgroup3
	SQ	RGBA, (2+ 4)(iNextTriToProcess):memgroup3
	SQ	RGB2, (2+ 8)(iNextTriToProcess):memgroup3

	SQ	RGB1, (2+12)(iNextTriToProcess)
	SQ	RGB2, (2+16)(iNextTriToProcess)
	SQ	RGBA, (2+20)(iNextTriToProcess)

	SQ	UV0, (1+ 0)(iNextTriToProcess):memgroup4	// increased subdivision count has to be stored
	SQ	UVA, (1+ 4)(iNextTriToProcess):memgroup4
	SQ	UV2, (1+ 8)(iNextTriToProcess):memgroup4

	SQ	UV1, (1+12)(iNextTriToProcess)
	SQ	UV2, (1+16)(iNextTriToProcess)
	SQ	UVA, (1+20)(iNextTriToProcess)

	IADDIU iNextTriToProcess, iNextTriToProcess, 12
	IADDIU iTrisOnStack, iTrisOnStack, 1	// note only adding 1 because we are overwriting one !

	JR	retAddress

--barrier

// *********************
// **     CASE 2      **
// *********************
SUBDIVIDE_2_EDGES:

	// find out which edge doesn't need splitting, and then ensure that it is loaded as 
	// vertex 2-0, so that we know where to split

	IADDIU	iVert0, iNextTriToProcess, 0
	IADDIU	iVert1, iNextTriToProcess, 4
	IADDIU	iVert2, iNextTriToProcess, 8

	IADDIU	iTemp, VI00, EDGE_20_MASK		// 8 = edge 20 is not split
	IBEQ	iEdges, iTemp, ALREADY20

	IADDIU	iTemp, VI00, EDGE_01_MASK		// 2 = edge 01 is not split
	IBEQ    iEdges, iTemp, ITS01

MUST_BE_12:
	IADDIU  iVert0, iNextTriToProcess, 8
	IADDIU  iVert1, iNextTriToProcess, 0
	IADDIU  iVert2, iNextTriToProcess, 4
	B ALREADY20

ITS01:
	IADDIU  iVert0, iNextTriToProcess, 4
	IADDIU  iVert1, iNextTriToProcess, 8
	IADDIU  iVert2, iNextTriToProcess, 0

ALREADY20:

	LQ		world0, 0(iVert0):memgroup5
	LQ		world1, 0(iVert1):memgroup5
	LQ		world2, 0(iVert2):memgroup5
	LQ		view0,  3(iVert0):memgroup6
	LQ		view1,  3(iVert1):memgroup6
	LQ		view2,  3(iVert2):memgroup6

	// before we go any further I need to know which edge is longer, 01 or 12, as it effects 
	// which way I subdivide the triangle
	// Please don't ask me to explain the following piece of maths, it requires a lot of equation jiggling
	// to prove that it works
	SUB.xy		fT1, view0, view2
	ADDA.xy		ACC, view0, view2
	LOI	2.0
	MSUBi.xy	fT2,  view1, I

	MUL.xy		fT2, fT2, fT1
	ADDy.x		VF00, fT2, fT2

	FSAND		iLeftLonger, 0x2	// this will be 0 or 2 depending on which side is longer

	
	// prepare UV.w for storage of Q values in W and 1.0f in Z
	MAXw.zw	UVA, VF00, VF00
	MAXw.zw	UVB, VF00, VF00

	// compute half vectors for world space - have to be transformed for view space
	LOI 5e-1
	MULA	ACC,	world1, I	// heheheh, this just gets better and better when I use the MAC
	MADD	worldA, world0, I
	MADD	worldB, world2, I

    // Load GBC->SC scale vector.
    ILW.w pCurConstCamData, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
    LQ fGBC2SCScale, ASURA_PS2_WATER__OFFSET__CONST_CAM_GBC2SC_SCALE(pCurConstCamData)

	// transform half vectors for view space
	TransAndPersp viewA, worldA, UVA
	TransAndPersp viewB, worldB, UVB

	// store results in multiple locations as required
	// WORLD first

	SQ	world1, 0(iNextTriToProcess):memgroup5
	SQ	worldB, 4(iNextTriToProcess):memgroup5
	SQ	worldA, 8(iNextTriToProcess):memgroup5
	
	SQ	world2, 12(iNextTriToProcess)
	SQ	world0, 16(iNextTriToProcess)
	// skip one for a sec

	SQ	worldA, 24(iNextTriToProcess)
	SQ	worldB, 28(iNextTriToProcess)
	// skip another one

	// Now VIEW
	SQ	view1, (3+0)(iNextTriToProcess):memgroup6
	SQ	viewB, (3+4)(iNextTriToProcess):memgroup6
	SQ	viewA, (3+8)(iNextTriToProcess):memgroup6

	SQ	view2, (3+12)(iNextTriToProcess)
	SQ	view0, (3+16)(iNextTriToProcess)
	// skip one

	SQ	viewA, (3+24)(iNextTriToProcess)
	SQ	viewB, (3+28)(iNextTriToProcess)
	// skip another

	// now do both the skipped ones

	IBNE	iLeftLonger, VI00, OTHERSIDE_WV

	SQ	worldA, 20(iNextTriToProcess)
	SQ	world2, 32(iNextTriToProcess)
	SQ	viewA, (3+20)(iNextTriToProcess)
	SQ	view2, (3+32)(iNextTriToProcess)
	B SKIP_OTHERSIDE_WV

OTHERSIDE_WV:
	
	SQ	worldB, 20(iNextTriToProcess)
	SQ	world0, 32(iNextTriToProcess)
	SQ	viewB, (3+20)(iNextTriToProcess)
	SQ	view0, (3+32)(iNextTriToProcess)

SKIP_OTHERSIDE_WV:

	//########################################################
	//stage 2: doing the same thing for the UV and RGB values
	//########################################################

	// load RGB and UV for 3 verts

	LQ		RGB0, 2(iVert0):memgroup7
	LQ		RGB1, 2(iVert1):memgroup7
	LQ		RGB2, 2(iVert2):memgroup7
	LQ		UV0,  1(iVert0):memgroup8
	LQ		UV1,  1(iVert1):memgroup8
	LQ		UV2,  1(iVert2):memgroup8

	// compute half vectors for RGB and UV
	LOI 0.5
	MULA	ACC, RGB1, I
	MADD	RGBA, RGB0, I
	MADD	RGBB, RGB2, I

	// subdivision counter increment
	ADDw.z	UV0, UV0, VF00
	ADDw.z	UV1, UV1, VF00
	ADDw.z	UV2, UV2, VF00

	MULA.xyz	ACC, UV1, I
	MADD.xyz	UVA, UV0, I
	MADD.xyz	UVB, UV2, I   

	// store results in multiple locations as required
	// RGB first
	SQ	RGB1, (2+0)(iNextTriToProcess):memgroup7
	SQ	RGBB, (2+4)(iNextTriToProcess):memgroup7
	SQ	RGBA, (2+8)(iNextTriToProcess):memgroup7

	SQ	RGB2, (2+12)(iNextTriToProcess)
	SQ	RGB0, (2+16)(iNextTriToProcess)
	// skip one

	SQ	RGBA, (2+24)(iNextTriToProcess)
	SQ	RGBB, (2+28)(iNextTriToProcess)
	// skip another

	// Now UV
	SQ	UV1, (1+0)(iNextTriToProcess):memgroup8
	SQ	UVB, (1+4)(iNextTriToProcess):memgroup8
	SQ	UVA, (1+8)(iNextTriToProcess):memgroup8

	SQ	UV2, (1+12)(iNextTriToProcess)
	SQ	UV0, (1+16)(iNextTriToProcess)
	// skip one

	SQ	UVA, (1+24)(iNextTriToProcess)
	SQ	UVB, (1+28)(iNextTriToProcess)
	// skip another

// now do both the skipped ones
	IBNE	iLeftLonger, VI00, OTHERSIDE_RU

	SQ	RGBA,(2+20)(iNextTriToProcess)
	SQ	RGB2,(2+32)(iNextTriToProcess)
	SQ	UVA, (1+20)(iNextTriToProcess)
	SQ	UV2, (1+32)(iNextTriToProcess)
	B SKIP_OTHERSIDE_RU

OTHERSIDE_RU:
	
	SQ	RGBB,(2+20)(iNextTriToProcess)
	SQ	RGB0,(2+32)(iNextTriToProcess)
	SQ	UVB, (1+20)(iNextTriToProcess)
	SQ	UV0, (1+32)(iNextTriToProcess)

SKIP_OTHERSIDE_RU:



	IADDIU iNextTriToProcess, iNextTriToProcess, 24
	IADDIU iTrisOnStack, iTrisOnStack, 2	// note only adding 2 because we are overwriting one !

	JR	retAddress

--barrier

// *********************
// **     CASE 3      **
// *********************
	// stage 1 : processing world and view space co-ordinates
	// load world and trans for 3 verts
SUBDIVIDE_3_EDGES:

	LQ		world0, 0(iNextTriToProcess):memgroup9
	LQ		world1, 4(iNextTriToProcess):memgroup9
	LQ		world2, 8(iNextTriToProcess):memgroup9
	LQ		view0,  3(iNextTriToProcess):memgroupA
	LQ		view1,  7(iNextTriToProcess):memgroupA
	LQ		view2, 11(iNextTriToProcess):memgroupA

	// prepare UV.w for storage of Q values in W and 1.0f in Z
	MAXw.zw	UVA, VF00, VF00
	MAXw.zw	UVB, VF00, VF00
	MAXw.zw	UVC, VF00, VF00

	// compute half vectors for world space - have to be transformed for view space
	LOI 5e-1
	MULA	ACC,	world1, I
	MADD	worldA, world0, I
	MADD	worldB, world2, I
	MULA	ACC,	world2, I
	MADD	worldC, world0, I

    // Load GBC->SC scale vector.
    ILW.w pCurConstCamData, ASURA_PS2_WATER__ADDR__VI_STACK_2(VI00)
    LQ fGBC2SCScale, ASURA_PS2_WATER__OFFSET__CONST_CAM_GBC2SC_SCALE(pCurConstCamData)

	// transform half vectors for view space
	TransAndPersp viewA, worldA, UVA
	TransAndPersp viewB, worldB, UVB
	TransAndPersp viewC, worldC, UVC

	// store results in multiple locations as required
	// WORLD first
	SQ	world1, 12(iNextTriToProcess)
	SQ	world2, 24(iNextTriToProcess)

	SQ	worldA, 4(iNextTriToProcess):memgroup9
	SQ	worldA, 20(iNextTriToProcess)
	SQ	worldA, 36(iNextTriToProcess)

	SQ	worldB, 16(iNextTriToProcess)
	SQ	worldB, 32(iNextTriToProcess)
	SQ	worldB, 40(iNextTriToProcess)

	SQ	worldC, 8(iNextTriToProcess):memgroup9
	SQ	worldC, 28(iNextTriToProcess)
	SQ	worldC, 44(iNextTriToProcess)

	SQ	view1, (3+12)(iNextTriToProcess)
	SQ	view2, (3+24)(iNextTriToProcess)

	SQ	viewA, (3+4)(iNextTriToProcess):memgroupA
	SQ	viewA, (3+20)(iNextTriToProcess)
	SQ	viewA, (3+36)(iNextTriToProcess)

	SQ	viewB, (3+16)(iNextTriToProcess)
	SQ	viewB, (3+32)(iNextTriToProcess)
	SQ	viewB, (3+40)(iNextTriToProcess)

	SQ	viewC, (3+8)(iNextTriToProcess):memgroupA
	SQ	viewC, (3+28)(iNextTriToProcess)
	SQ	viewC, (3+44)(iNextTriToProcess)

	//########################################################
	//stage 2: doing the same thing for the UV and RGB values
	//########################################################

	// load RGB and UV for 3 verts

	LQ		RGB0, 2(iNextTriToProcess):memgroupB
	LQ		RGB1, 6(iNextTriToProcess):memgroupB
	LQ		RGB2,10(iNextTriToProcess):memgroupB
	LQ		UV0,  1(iNextTriToProcess):memgroupC
	LQ		UV1,  5(iNextTriToProcess):memgroupC
	LQ		UV2,  9(iNextTriToProcess):memgroupC

	// compute half vectors for RGB and UV
	LOI 0.5
	MULA	ACC,  RGB1, I
	MADD	RGBA, RGB0, I
	MADD	RGBB, RGB2, I
	MULA	ACC,  RGB2, I
	MADD	RGBC, RGB0, I

	// subdivision counter increment
	ADDw.z	UV0, UV0, VF00
	ADDw.z	UV1, UV1, VF00
	ADDw.z	UV2, UV2, VF00

	MULA.xyz	ACC, UV1, I
	MADD.xyz	UVA, UV0, I
	MADD.xyz	UVB, UV2, I
	MULA.xyz	ACC, UV2, I
	MADD.xyz	UVC, UV0, I

	// store results in multiple locations as required
	// RGB first
	SQ	RGB0, (2+0)(iNextTriToProcess):memgroupB
	SQ	RGB1, (2+12)(iNextTriToProcess)
	SQ	RGB2, (2+24)(iNextTriToProcess)

	SQ	RGBA, (2+4)(iNextTriToProcess):memgroupB
	SQ	RGBA, (2+20)(iNextTriToProcess)

	SQ	RGBB, (2+16)(iNextTriToProcess)
	SQ	RGBB, (2+32)(iNextTriToProcess)

	SQ	RGBC, (2+8)(iNextTriToProcess):memgroupB
	SQ	RGBC, (2+28)(iNextTriToProcess)

	SQ	RGBA, (2+36)(iNextTriToProcess)
	SQ	RGBB, (2+40)(iNextTriToProcess)
	SQ	RGBC, (2+44)(iNextTriToProcess)

	SQ	UV0, (1+0 )(iNextTriToProcess):memgroupC	// increased subdivision count has to be stored
	SQ	UV1, (1+12)(iNextTriToProcess)
	SQ	UV2, (1+24)(iNextTriToProcess)

	SQ	UVA, (1+4 )(iNextTriToProcess):memgroupC
	SQ	UVA, (1+20)(iNextTriToProcess)
	SQ	UVA, (1+36)(iNextTriToProcess)

	SQ	UVB, (1+16)(iNextTriToProcess)
	SQ	UVB, (1+32)(iNextTriToProcess)
	SQ	UVB, (1+40)(iNextTriToProcess)

	SQ	UVC, (1+8 )(iNextTriToProcess):memgroupC
	SQ	UVC, (1+28)(iNextTriToProcess)
	SQ	UVC, (1+44)(iNextTriToProcess)

	IADDIU iNextTriToProcess, iNextTriToProcess, 36
	IADDIU iTrisOnStack, iTrisOnStack, 3	// note only adding 3 because we are overwriting one !

	JR	retAddress

.end

