//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_BotanicalsLOD.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Botanicals/Asura_PS3_Botanicals.h"
#include "Rendering/Asura_PS3_RenderingStates.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_LODData::Asura_PS3_Botanicals_LODData
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
Asura_PS3_Botanicals_LODData::Asura_PS3_Botanicals_LODData()
: m_eType( TYPE_BOTANICALS_GEOMETRY_2SIDED )
, m_fMinDistance( 0.0f )
, m_fMinFullAlphaDistance( 0.0f )
, m_fMaxFullAlphaDistance( 0.0f )
, m_fMaxDistance( 0.0f )
, m_uNumVisibleInstances( 0 )
, m_pxGeometryLODs( NULL )
{
}
/// End of function 'Asura_PS3_Botanicals_LODData::Asura_PS3_Botanicals_LODData'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_LODData::~Asura_PS3_Botanicals_LODData
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
Asura_PS3_Botanicals_LODData::~Asura_PS3_Botanicals_LODData()
{
	switch( m_eType )
	{
	case TYPE_BOTANICALS_GEOMETRY_1SIDED:
	case TYPE_BOTANICALS_GEOMETRY_2SIDED:
		{
			delete m_pxGeometryLODs;
			break;
		}
	case TYPE_BOTANICALS_BILLBOARDS:
		{
			delete m_pxBillboardLODs;
			break;
		}
	case TYPE_BOTANICALS_IMPOSTER:
		{
			delete m_pxImposterLODs;
			break;
		}
	default:
		{
			Asura_Assert( false, "Unknown botanicals element type." );
			break;
		}
	}
	m_pxGeometryLODs = NULL;
}
/// End of function 'Asura_PS3_Botanicals_LODData::~Asura_PS3_Botanicals_LODData'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_LODData::Render
// Author       : Catalin Arsenescu
// Date         : 06-September-2007
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Botanicals_LODData::Render( u_int uRenderStartInstance, u_int uNumVisibleInstances ) const
{
	Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0123_ALL );
	switch(m_eType)
	{
	case TYPE_BOTANICALS_GEOMETRY_2SIDED:
		{
			ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Botanicals TYPE_BOTANICALS_GEOMETRY_2SIDED Start" );
			Asura_PS3_RenderingStates::RequestCullMode(ASURA_CULLMODE_NONE);
			m_pxGeometryLODs->Render( uRenderStartInstance, uNumVisibleInstances );
			ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

			break;
		}
	case TYPE_BOTANICALS_GEOMETRY_1SIDED:
		{
			ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Botanicals TYPE_BOTANICALS_GEOMETRY_1SIDED Start" );
			Asura_PS3_RenderingStates::RequestCullMode(ASURA_CULLMODE_CW);
			m_pxGeometryLODs->Render( uRenderStartInstance, uNumVisibleInstances );
			ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

			break;
		}
	case TYPE_BOTANICALS_BILLBOARDS:
		{
			ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Botanicals TYPE_BOTANICALS_BILLBOARDS Start" );
			Asura_PS3_RenderingStates::RequestCullMode(ASURA_CULLMODE_CCW);
			m_pxBillboardLODs->Render( uRenderStartInstance, uNumVisibleInstances );
			ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

			break;
		}
	case TYPE_BOTANICALS_IMPOSTER:
		{
			ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Botanicals TYPE_BOTANICALS_IMPOSTER Start" );
			Asura_PS3_RenderingStates::RequestCullMode(ASURA_CULLMODE_CW);
			m_pxImposterLODs->Render( uRenderStartInstance, uNumVisibleInstances );
			ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

			break;
		}
	default:
		{
			break;
		}
	}
}
/// End of function 'Asura_PS3_Botanicals_LODData::Render'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_LODData::Initialise
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Botanicals_LODData::Initialise(Asura_PS3_Botanicals_Mesh_Type eType)
{
	m_eType = eType;

	switch( m_eType )
	{
	case TYPE_BOTANICALS_GEOMETRY_1SIDED:
	case TYPE_BOTANICALS_GEOMETRY_2SIDED:
		{
			m_pxGeometryLODs = new Asura_PS3_Botanicals_LOD_Geometry();
			break;
		}
	case TYPE_BOTANICALS_BILLBOARDS:
		{
			m_pxBillboardLODs = new Asura_PS3_Botanicals_LOD_Billboards();
			break;
		}
	case TYPE_BOTANICALS_IMPOSTER:
		{
			m_pxImposterLODs = new Asura_PS3_Botanicals_LOD_Imposter();
			break;
		}
	default:
		{
			Asura_Assert( false, "Unknown botanicals element type." );
			break;
		}
	}
}
/// End of function 'Asura_PS3_Botanicals_LODData::Initialise'. ///
