//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_BotanicalsLODGeometry.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Botanicals/Asura_PS3_Botanicals.h"

// Asura Platform-independent
#include "Camera/Asura_Camera.h"
#include "Core/Memory/Asura_Mem.h"
#include "Core/Asura_MaterialFlags.h"
#include "Core/Asura_Material_System.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Interface/Asura_CommandConsole.h"

// Asura PS3
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_RenderingOptions.h"
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"

#include "Rendering/ShadersDefinition/VD/BotanicalsInstace_VD.h"
#include "Rendering/ShadersDefinition/VD/BotanicalsGeometry_VD.h"
#include "Rendering/ShadersDefinition/VP/BotanicalsGeometry_VP.h"

static bool s_bUseIndexedPrimitive	= true;

static Asura_PS3_VertexBufferDeclaration g_axVertexDeclarationGeometry[] =
{
	//	m_szName, m_uOffset, m_uType, m_uNumberOfCoords, m_ucAttributeIndex
	{ "xIn.m_xPosition",		 0,	CELL_GCM_VERTEX_F,		3, 2 }, //ATTR2
	{ "xIn.m_xUV",				12,	CELL_GCM_VERTEX_SF,		2, 3 }, //ATTR3
	{ "xIn.m_xNormal",			16,	CELL_GCM_VERTEX_CMP,	1, 4 }, //ATTR4
	{ "xIn.m_xTangent",			20,	CELL_GCM_VERTEX_CMP,	1, 5 }, //ATTR5
	{ "xIn.m_xBinormal",		24,	CELL_GCM_VERTEX_CMP,	1, 6 }, //ATTR6
	{ "xIn.m_fBoneWeight",		28,	CELL_GCM_VERTEX_F,		1, 7 }, //ATTR7
};

static Asura_PS3_GCM_IndexBuffer s_xIB;

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_LOD_Geometry::Asura_PS3_Botanicals_LOD_Billboards
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
Asura_PS3_Botanicals_LOD_Geometry::Asura_PS3_Botanicals_LOD_Geometry()
: m_uMaterialResponseHashID( ASURA_HASH_ID_UNSET )
, m_uNumIndices( 0 )
, m_uNumTriangles( 0 )
, m_uNumVertices( 0 )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	const Asura_UnicodeChar* const szVar = L"Render.Botanical.IndexedPrimitive";
	if (!Asura_CommandConsole::VarExists(szVar))
	{
		Asura_CommandConsole::AddVar( szVar, &s_bUseIndexedPrimitive, Asura_CommandConsole::REMOVE_AT_RESET );
	}
#endif
}
/// End of function 'Asura_PS3_Botanicals_LOD_Geometry::Asura_PS3_Botanicals_LOD_Geometry'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_LOD_Geometry::~Asura_PS3_Botanicals_LOD_Geometry
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
Asura_PS3_Botanicals_LOD_Geometry::~Asura_PS3_Botanicals_LOD_Geometry()
{
	m_xVB.Shutdown();
	m_xIB.Shutdown();

	m_uNumIndices	= 0;
	m_uNumTriangles	= 0;
	m_uNumVertices	= 0;
}
/// End of function 'Asura_PS3_Botanicals_LOD_Geometry::~Asura_PS3_Botanicals_LOD_Geometry'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_LOD_Geometry::Render
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Botanicals_LOD_Geometry::Render( u_int uRenderStartInstance, u_int uNumVisibleInstances )
{
	Asura_PS3_GCM_VertexShader* g_pxVertexShader = Asura_PS3_Botanicals_System::GetBotanicalsGeometryVP();
	Asura_PS3_FragmentShaderCopy* g_pxFragmentShader = Asura_PS3_GCM_ShadersLibrary::GetResettableFP( static_cast<Asura_Hash_ID> ( ASURA_SHADER_HASH_BOTANICALS ), m_uFPHashID );
	if( !g_pxFragmentShader ) return;

	const Asura_Vector_3* const pxCamPos = Asura_Camera::GetPosition();

	float afCamPos[ 4 ]	= { pxCamPos->x, pxCamPos->y, pxCamPos->z, 1.0f };

	Asura_PS3_DeferredRendering::Bind();
	g_pxVertexShader->Bind();

	const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( m_uMaterialResponseHashID );
	Asura_TextureAnimationOutput xTexAnim1;
	Asura_TextureAnimation_System::GetAnimationState( pxMaterial1->GetTextureAnim(), NULL, &xTexAnim1 );
	const float afTexCoordOffset[4] = { xTexAnim1.m_fUOffset, xTexAnim1.m_fVOffset, 0.0f, 0.0f };
	const Asura_AMatrix4x4& xViewProjectionMatrix = Asura_PS3_Graphics::GetViewProjectionMatrix();
	g_pxVertexShader->SetUniformParameter( VSG_ViewProjectionMatrix, xViewProjectionMatrix.GetElements() );
	g_pxVertexShader->SetUniformParameter( VSG_CameraPosition, afCamPos );
	g_pxVertexShader->SetUniformParameter( VSG_UV_Offsets, &afTexCoordOffset[ 0 ] );

	Asura_PS3_Botanicals_System::SetMaterial( m_uMaterialResponseHashID, g_pxFragmentShader );
	Asura_PS3_RenderingStates::RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( true );
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::SetBlendEnableMrt( false, false, false );
	Asura_PS3_RenderingStates::SetAlphaBlending( false );

	g_pxFragmentShader->Bind();
	u_short uiStride = m_uNumIndices;
	u_int uiNumVertices = m_uNumIndices;
	cellGcmSetFrequencyDividerOperationUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(),
		CELL_GCM_FREQUENCY_MODULO << 2 |
		CELL_GCM_FREQUENCY_MODULO << 3 |
		CELL_GCM_FREQUENCY_MODULO << 4 |
		CELL_GCM_FREQUENCY_MODULO << 5 |
		CELL_GCM_FREQUENCY_MODULO << 6 |
		CELL_GCM_FREQUENCY_MODULO << 7 );

	if( s_bUseIndexedPrimitive )
	{
		u_int uiSize = uNumVisibleInstances * uiNumVertices;
		s_xIB.InitialiseFromBuffer( uiSize );
		u_short* pxOIndexData	= static_cast<u_short*>( m_xIB.LockForWriting() );

		if( s_xIB.Is32BitIndex() )
		{
			u_int* pxIndexData	= static_cast<u_int*>( s_xIB.LockForWriting() );
			for( u_int i = 0, k = 0; i < uNumVisibleInstances; ++i )
			{
				u_int l = i * uiNumVertices;
				for( u_int j = 0; j < m_uNumIndices; ++j, ++k )
				{
					if( pxOIndexData[ j ] == 0x0000FFFF )
					{
						pxIndexData[ k ] = 0xFFFFFFFF;
					}
					else
					{
						pxIndexData[ k ] = l + pxOIndexData[ j ];
					}
				}
			}
		}
		else
		{
			u_short* pxIndexData	= static_cast<u_short*>( s_xIB.LockForWriting() );
			for( u_int i = 0, k = 0; i < uNumVisibleInstances; ++i )
			{
				u_int l = i * uiNumVertices;
				for( u_int j = 0; j < m_uNumIndices; ++j, ++k )
				{
					if( pxOIndexData[ j ] == 0x0000FFFF )
					{
						pxIndexData[ k ] = 0x0000FFFF;
					}
					else
					{
						pxIndexData[ k ] = l + pxOIndexData[ j ];
					}
				}
			}
		}

		Asura_PS3_Botanicals_System::BindInstanceData( uRenderStartInstance, uiStride );
		bool abKeep[16] = {false, false, false, false, false, false, false, false, true, true, true, true, true, true, true, true};
		m_xVB.Bind( g_axVertexDeclarationGeometry, VD_GEOMETRY_ELEMENTS, uiStride, true, abKeep );
		s_xIB.RenderTriangleStrip( uiSize );
		Asura_PS3_Graphics::Flush();
	}
	else
	{
		for( u_int i = 0; i < uNumVisibleInstances; ++i )
		{
			Asura_PS3_Botanicals_System::BindInstanceData( uRenderStartInstance + i, uiStride );
//			bool abKeep[16] = {false, false, false, false, false, false, false, false, true, true, true, true, true, true, true, true};
// (MP - 22.04.2009) TODO: Suspect this call is supposed to be m_xVB.Bind( g_axVertexDeclarationGeometry, VD_GEOMETRY_ELEMENTS, 1, true, abKeep )
// Certainly the arguments are the wrong way round, though they happen to work out (since uiStride will presumably be non-zero and therefore true and of course true gets converted to 1).
			m_xVB.Bind( g_axVertexDeclarationGeometry, VD_GEOMETRY_ELEMENTS, true, uiStride );

			m_xIB.RenderTriangleStrip( uiNumVertices );
			Asura_PS3_Graphics::Flush();
		}
	}

	cellGcmSetFrequencyDividerOperationUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), 0 );
}
/// End of function 'Asura_PS3_Botanicals_LOD_Billboards::Render'. ///
