//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_BotanicalsSystem.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Botanicals/Asura_PS3_Botanicals.h"

// Asura Platform-independent
#include "Camera/Asura_Camera.h"
#include "Core/Memory/Asura_Mem.h"
#include "Core/Asura_MaterialFlags.h"
#include "Core/Asura_Material_System.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"
#include "Core/Asura_MaterialResponse_System.h"

// Asura PS3
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"

//Vertex declaration
#include "Rendering/ShadersDefinition/VD/BotanicalsInstace_VD.h"
#include "Rendering/ShadersDefinition/VD/BotanicalsBillboards_VD.h"
#include "Rendering/ShadersDefinition/VD/BotanicalsImposters_VD.h"
#include "Rendering/ShadersDefinition/VD/BotanicalsGeometry_VD.h"

#include "Rendering/ShadersDefinition/VP/BotanicalsBillboards_VP.h"
#include "Rendering/ShadersDefinition/VP/BotanicalsImposters_VP.h"
#include "Rendering/ShadersDefinition/VP/BotanicalsGeometry_VP.h"

#include "Rendering/ShadersDefinition/FP/Botanicals_FP.h"

/******************************************************************************/
/* Asura_PS3_Botanicals_System Static Class Data                          */
/******************************************************************************/

// Instance buffer
Asura_PS3_GCM_VertexBuffer			Asura_PS3_Botanicals_System::s_axInstanceVBs[NUM_INSTANCE_BUFFERS];
int									Asura_PS3_Botanicals_System::s_iCurrentInstanceBuffer			= 0;

// Shape and template lists

Asura_PS3_Botanicals_Shape*			Asura_PS3_Botanicals_System::s_pxShapes							= NULL;
Asura_PS3_Botanicals_Template*		Asura_PS3_Botanicals_System::s_pxTemplates						= NULL;

u_int								Asura_PS3_Botanicals_System::s_uNumShapes						= 0;
u_int								Asura_PS3_Botanicals_System::s_uNumTemplates					= 0;

static Asura_PS3_GCM_VertexShader*			g_pxGeometryVertexShader	= NULL;
static Asura_PS3_GCM_VertexShader*			g_pxBillboardsVertexShader	= NULL;
static Asura_PS3_GCM_VertexShader*			g_pxImpostersVertexShader	= NULL;

//*******************************************************************
// Local Data
//*******************************************************************

static Asura_PS3_VertexBufferDeclaration g_axVertexDeclarationBotanicalInstances [] =
{
	//	m_szName, m_uOffset, m_uType, m_uNumberOfCoords, m_ucAttributeIndex
	{ "xIn.m_xInstanceMatrix0",	 0,	CELL_GCM_VERTEX_SF,	4,  8 }, //ATTR08
	{ "xIn.m_xInstanceMatrix1",	 8,	CELL_GCM_VERTEX_SF,	4,  9 }, //ATTR09
	{ "xIn.m_xInstanceMatrix2",	16,	CELL_GCM_VERTEX_SF,	4, 10 }, //ATTR10

	{ "xIn.m_xRedSH",			24,	CELL_GCM_VERTEX_SF,	4, 11 }, //ATTR11
	{ "xIn.m_xGreenSH",			32,	CELL_GCM_VERTEX_SF,	4, 12 }, //ATTR12
	{ "xIn.m_xBlueSH",			40,	CELL_GCM_VERTEX_SF,	4, 13 }, //ATTR13

	{ "xIn.m_xAlphaScale",		48,	CELL_GCM_VERTEX_F,	2, 14 }, //ATTR14

	{ "xIn.m_xBoneOffset",		56,	CELL_GCM_VERTEX_F,	3, 15 }, //ATTR15
};


Asura_PS3_Botanicals_System s_xPS3_Botanicals_System;

//*******************************************************************
// Function Definitions
//*******************************************************************

Asura_PS3_GCM_VertexShader* Asura_PS3_Botanicals_System::GetBotanicalsGeometryVP( void )
{
	return g_pxGeometryVertexShader;
}

Asura_PS3_GCM_VertexShader* Asura_PS3_Botanicals_System::GetBotanicalsBillboardVP( void )
{
	return g_pxBillboardsVertexShader;
}

Asura_PS3_GCM_VertexShader* Asura_PS3_Botanicals_System::GetBotanicalsImpostersVP( void )
{
	return g_pxImpostersVertexShader;
}

void Asura_PS3_Botanicals_System::BindInstanceData( u_int uRenderStartInstance, u_short usStride )
{
	s_axInstanceVBs[ s_iCurrentInstanceBuffer ].BindFromVertex( g_axVertexDeclarationBotanicalInstances, VDBI_ELEMENTS, uRenderStartInstance, usStride, false );
}

void Asura_PS3_Botanicals_System::SetMaterial( Asura_Hash_ID uMaterialHash, Asura_PS3_FragmentShaderCopy* pxFragmentShader )
{
	const Asura_Hash_ID uMaterialHash1 = uMaterialHash;
	const Asura_MaterialResponse* const pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );
	const Asura_MaterialResponse* const pxMaterial2 = NULL;

	Asura_PS3_MaterialResponse_System::SetEffectParameters( pxMaterial1, pxMaterial2, NULL, pxFragmentShader );
	Asura_PS3_MaterialResponse_System::SetBlendStates( pxMaterial1 );
	
	//BW moved this from inside SetBlendStates, because character and environment renderers don't want this being changed
	if ( pxMaterial1->GetBlendMode() == Asura_MaterialResponse::BLEND_MODE_OPAQUE )
	{
		Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( true );
	}
	else
	{
		Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( false );
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::GetBGRManager
// Author       : Catalin Arsenescu
// Date         : 03-August-2007
////////////////////////////////////////////////////////////////////////////////
Asura_BGR_Manager& Asura_BGR_Manager::GetBGRManager( void)
{
	return( s_xPS3_Botanicals_System);
}
/// End of function 'Asura_PS3_Botanicals_System::GetBGRManager'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::GetInstanceHeaderCount
// Author       : Catalin Arsenescu
// Date         : 03-August-2007
////////////////////////////////////////////////////////////////////////////////
u_int Asura_PS3_Botanicals_System::GetInstanceHeaderCount( void) const
{
	return( s_uNumTemplates);
}
/// End of function 'Asura_PS3_Botanicals_System::GetInstanceHeaderCount'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::GetInstanceHeader
// Author       : Catalin Arsenescu
// Date         : 03-August-2007
////////////////////////////////////////////////////////////////////////////////
const Asura_BGR_Instances_Header* Asura_PS3_Botanicals_System::GetInstanceHeader( const u_int uInstanceHeaderIndex) const
{
	Asura_Assertf( uInstanceHeaderIndex < static_cast< u_int>( this->s_uNumTemplates), ("Instance Header %d is out of %d range.", uInstanceHeaderIndex, this->s_uNumTemplates));
	const Asura_BGR_Instances_Header* const pxInstanceHeader = s_pxTemplates + uInstanceHeaderIndex;
	return( pxInstanceHeader);
}
/// End of function 'Asura_PS3_Botanicals_System::GetInstanceHeader'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::GetInstanceHeader
// Author       : Catalin Arsenescu
// Date         : 03-August-2007
////////////////////////////////////////////////////////////////////////////////
Asura_BGR_Instances_Header* Asura_PS3_Botanicals_System::GetInstanceHeader( const u_int uInstanceHeaderIndex)
{
	Asura_Assertf( uInstanceHeaderIndex < static_cast< u_int>( this->s_uNumTemplates), ("Instance Header %d is out of %d range.", uInstanceHeaderIndex, this->s_uNumTemplates));
	Asura_BGR_Instances_Header* pxInstanceHeader = s_pxTemplates + uInstanceHeaderIndex;
	return( pxInstanceHeader);
}
/// End of function 'Asura_PS3_Botanicals_System::GetInstanceHeader'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::GetType
// Author       : Catalin Arsenescu
// Date         : 03-August-2007
////////////////////////////////////////////////////////////////////////////////
Asura_BGR_Type* Asura_PS3_Botanicals_System::GetType( const Asura_BGR_TypeID xTypeID)
{
	Asura_Assertf( xTypeID.GetID() < s_uNumShapes, ("xTypeID %d is out of %D range.", xTypeID.GetID(), s_uNumShapes));
	Asura_BGR_Type* pxType = this->s_pxShapes + xTypeID.GetID();

	return( pxType);
}
/// End of function 'Asura_PS3_Botanicals_System::GetType'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::Initialise
// Author       : Catalin Arsenescu
// Date         : 07-August-2007
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Botanicals_System::Initialise( void )
{
	g_pxGeometryVertexShader	= new Asura_PS3_VP_BotanicalGeometry();
	g_pxBillboardsVertexShader	= new Asura_PS3_VP_BotanicalBillboard();
	g_pxImpostersVertexShader	= new Asura_PS3_VP_BotanicalImposter();

}
/// End of function 'Asura_PS3_Botanicals_System::Initialise'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::Reset
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Botanicals_System::Reset( void )
{
	for( int i = 0; i < NUM_INSTANCE_BUFFERS; ++i )
	{	
		s_axInstanceVBs[i].Shutdown();
	}
	delete[] s_pxShapes;
	delete[] s_pxTemplates;

	s_pxShapes = NULL;
	s_pxTemplates = NULL;

	s_uNumShapes = 0;
	s_uNumTemplates = 0;
}
/// End of function 'Asura_PS3_Botanicals_System::Reset'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::Shutdown
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Botanicals_System::Shutdown( void )
{
	Reset();

	delete g_pxGeometryVertexShader;
	delete g_pxBillboardsVertexShader;
	delete g_pxImpostersVertexShader;

	g_pxGeometryVertexShader	= NULL;
	g_pxBillboardsVertexShader	= NULL;
	g_pxImpostersVertexShader	= NULL;
}
/// End of function 'Asura_PS3_Botanicals_System::Shutdown'. ///
/*
////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::Update
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Botanicals_System::Update( const float fTime )
{
}
/// End of function 'Asura_PS3_Botanicals_System::Update'. ///
*/
////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::Render
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Botanicals_System::Render( const float fFade )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE)
	if( false == Asura_BGR_Manager::s_bDebugVarShow)
	{
		return;
	}
#endif

    ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__BOTANICALS, "Botanicals" );

	CalculateLODs();

	u_int uRenderStartInstance = 0;

	// Sort render by template
	struct LODCache
	{
	public:
		const Asura_PS3_Botanicals_Element* m_pxElement;
		u_int m_uLODIndex;
		float m_fDistance;
		u_int m_uRenderStartInstance;
	};
	const u_int uLODCacheSizeMax = 256;
	LODCache axLODCache[ uLODCacheSizeMax];
	u_int uLODCacheSize = 0;

	int iTemplateOffsetToNextBase = 1;
	for( u_int u = 0; u < s_uNumTemplates; u += iTemplateOffsetToNextBase)
	{
		const Asura_PS3_Botanicals_Template& xCurrTemplateBase = s_pxTemplates[u];

		iTemplateOffsetToNextBase = xCurrTemplateBase.m_iOffsetToNextBase;

		const Asura_PS3_Botanicals_Shape* const pxCurrShape = xCurrTemplateBase.m_pxShape;
		if( !pxCurrShape ) continue;

		// Loop over each shape element

		for( int i = 0; i < pxCurrShape->m_iNumElements; ++i )
		{
			const Asura_PS3_Botanicals_Element& xCurrElement = pxCurrShape->m_pxElements[i];

			// Render as many of each level of detail as are in range

//			float fDistanceMin = -1e30;

			for( int j = xCurrElement.m_iNumLODs - 1; 0 <= j; --j )
			{
				const Asura_PS3_Botanicals_LODData& xCurrLODData = xCurrElement.m_pxLODData[j];

//				Asura_Assert( fDistanceMin <= xCurrLODData.m_fMinDistance, "LOD Data Out of Order");
//				fDistanceMin = Asura_Maths::Max( fDistanceMin, xCurrLODData.m_fMinDistance);

				if( xCurrLODData.m_uNumVisibleInstances == 0 ) 
				{
					continue;
				}

				Asura_Assert( uLODCacheSize < uLODCacheSizeMax, "Not enough LOD cache space");
				if( uLODCacheSizeMax <= uLODCacheSize)
				{
					break;
				}

				{
					LODCache& xLODCache = axLODCache[ uLODCacheSize];
					xLODCache.m_pxElement = &xCurrElement;
					xLODCache.m_uLODIndex = j;
					xLODCache.m_fDistance = xCurrLODData.m_fMaxDistance;
					xLODCache.m_uRenderStartInstance = uRenderStartInstance;
				}
				uLODCacheSize += 1;
				uRenderStartInstance += xCurrLODData.m_uNumVisibleInstances;
			}
		}
	}

	// DLH - 11 May 2007 11:12:42 - Sort required LODs into depth order to allow the Z buffer earn its keep
	// DLH - 11 May 2007 11:13:09 - Nasty Bubble sort, please improve if you get time.
	if( 1 < uLODCacheSize)
	{
		for( u_int uTemplateIndexA = 0; uTemplateIndexA < (uLODCacheSize - 1); uTemplateIndexA += 1)
		{
			for( u_int uTemplateIndexB = (uTemplateIndexA + 1); uTemplateIndexB < (uLODCacheSize); uTemplateIndexB += 1)
			{
				LODCache& xTemplateA = axLODCache[ uTemplateIndexA];
				LODCache& xTemplateB = axLODCache[ uTemplateIndexB];

				int eTypeA = xTemplateA.m_pxElement->m_eType;
				int eTypeB = xTemplateB.m_pxElement->m_eType;

				if( eTypeA < eTypeB)
				{
					continue;
				}
				if( eTypeA == eTypeB)
				{
					if( xTemplateA.m_fDistance < xTemplateB.m_fDistance)
					{
						continue;
					}
				}
				LODCache xTemplateTemp = xTemplateA;
				xTemplateA = xTemplateB;
				xTemplateB = xTemplateTemp;
			}
		}
	}

	for( u_int uLODCacheIndex = 0; uLODCacheIndex < uLODCacheSize; ++uLODCacheIndex )
	{
		LODCache& xLODCache = axLODCache[ uLODCacheIndex];
		const Asura_PS3_Botanicals_Element& xCurrElement = *xLODCache.m_pxElement;
		const u_int uLod = xLODCache.m_uLODIndex;
		const Asura_PS3_Botanicals_LODData& xCurrLODData = xCurrElement.m_pxLODData[ uLod ];
		const u_int uLODRenderStartInstance = xLODCache.m_uRenderStartInstance;

		xCurrLODData.Render( uLODRenderStartInstance, xCurrLODData.m_uNumVisibleInstances );
	}

	// Double-buffering: Swap buffers
	s_iCurrentInstanceBuffer = ( s_iCurrentInstanceBuffer + 1 ) % NUM_INSTANCE_BUFFERS;

    ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__BOTANICALS );
}
/// End of function 'Asura_PS3_Botanicals_System::Render'. ///

////////////////////////////////////////////////////////////////////////////////
// Class		: Asura_PS3_Botanicals_InstanceCache
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
class Asura_PS3_Botanicals_InstanceCache
{
public:
	float m_fDistance;
	const Asura_PS3_Botanicals_Template* m_pxCurrTemplate;
	const Asura_PS3_Botanicals_Instance* m_pxCurrInstance;
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	Asura_BGR_DynamicInstance* m_pxDynamicInstanceData;
#endif
};
/// End of class 'Asura_PS3_Botanicals_InstanceCache'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::CalculateLODsCache
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
Asura_PS3_Botanicals_Instance::RenderData* Asura_PS3_Botanicals_System::CalculateLODsCache( Asura_PS3_Botanicals_Element& xCurrElement,
                                                                                            const Asura_PS3_Botanicals_InstanceCache* paxInstanceCache,
                                                                                            const u_int uInstanceCacheMax,
                                                                                            Asura_PS3_Botanicals_Instance::RenderData* pxCurrRenderData,
                                                                                            Asura_PS3_Botanicals_Instance::RenderData* const pxRenderDataEnd )
{
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	const Asura_BGR_Section_ID xCurrElementSectionID = xCurrElement.m_xSectionID;
	const Asura_BGR_Version_ID xCurrElementVersionID = xCurrElement.m_xVersionID;
#endif
	float fDistanceMin = -1e30;

	for( int j = xCurrElement.m_iNumLODs - 1; 0 <= j; --j )
	{
		Asura_PS3_Botanicals_LODData& xCurrLODData = xCurrElement.m_pxLODData[j];
		xCurrLODData.m_uNumVisibleInstances = 0;

		if (fDistanceMin > xCurrLODData.m_fMinDistance && uInstanceCacheMax > 0)
		{	
			// Assert now gives name of offending template
			Asura_Assertf( false, ("LOD Data out of order in botanical template '%s'. Please correct the data!", paxInstanceCache[0].m_pxCurrTemplate->m_xName.GetString()));
		}
		fDistanceMin = Asura_Maths::Max( fDistanceMin, xCurrLODData.m_fMinDistance);

		for( u_int uInstanceCacheIndex = 0; uInstanceCacheIndex < uInstanceCacheMax; uInstanceCacheIndex += 1)
		{
            if( pxCurrRenderData >= pxRenderDataEnd )
            {
                Asura_Assert( false, "Rendering too many botanicals this frame!" );
                return pxCurrRenderData;
            }

			const Asura_PS3_Botanicals_InstanceCache& xInstanceCache = paxInstanceCache[ uInstanceCacheIndex];

#if (defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) \
	||  (defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
			const Asura_PS3_Botanicals_Template& xCurrTemplate = *(xInstanceCache.m_pxCurrTemplate);
#endif

			{
				const Asura_PS3_Botanicals_Instance& xCurrInstance = *(xInstanceCache.m_pxCurrInstance);

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
				Asura_BGR_DynamicInstance* pxDynamicInstanceData = xInstanceCache.m_pxDynamicInstanceData;
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
				if( true == xCurrTemplate.HasDynamicSectionVersionData() && NULL != pxDynamicInstanceData)
				{
					const Asura_BGR_Version_ID xInstanceVersionID = xCurrTemplate.GetDynamicSectionVersion( pxDynamicInstanceData, xCurrElementSectionID);
					if( xCurrElementVersionID != xInstanceVersionID)
					{
						continue;
					}
				}
#endif
				float fDist = xInstanceCache.m_fDistance;

				// Imposters don't have a draw-distance limit (no module system to limit them...)
				if (xCurrElement.m_eType == TYPE_BOTANICALS_IMPOSTER)
				{					
					if (fDist > xCurrLODData.m_fMaxDistance )
					{
						fDist = xCurrLODData.m_fMaxDistance;
					}
				}

				if( xCurrLODData.m_fMinDistance <= fDist && fDist <= xCurrLODData.m_fMaxDistance )
				{
					// Calculate the fade value. This isn't a simple alpha blend; we fade botanicals
					// in by increasing their alpha level against the alpha test, combined with a
					// speckled texture alpha channel

					float fAlpha = 0.0f;
					if( fDist < xCurrLODData.m_fMinFullAlphaDistance )
					{
						fAlpha = (fDist - xCurrLODData.m_fMinDistance) / (xCurrLODData.m_fMinFullAlphaDistance - xCurrLODData.m_fMinDistance);
						fAlpha = 1.0f - 0.5f * fAlpha;
					}
					else if( fDist <= xCurrLODData.m_fMaxFullAlphaDistance )
					{
						fAlpha = 0.5f;
					}
					else
					{
						fAlpha = (xCurrLODData.m_fMaxDistance - fDist) / (xCurrLODData.m_fMaxDistance - xCurrLODData.m_fMaxFullAlphaDistance);
						fAlpha = 1.0f - 0.5f * fAlpha;
					}

					// Create the data in normal memory first, then copy the whole thing to write-combined
					Asura_PS3_Botanicals_Instance::RenderData xInstanceRenderData;
					Asura_Mem::Copy( &xInstanceRenderData, &xCurrInstance.m_xRenderData, sizeof( xCurrInstance.m_xRenderData ) );

					xInstanceRenderData.m_fAlpha = fAlpha;

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE) && 1 == ASURA_BOTANICAL_SUPPORT_INTERACTIVE
					if( true == xCurrTemplate.HasDynamicInteractiveData())
					{
						const Asura_BGR_InteractiveBasic* pxInteractiveBasic = xCurrTemplate.GetDynamicInteractiveData( xInstanceCache.m_pxDynamicInstanceData);
						xInstanceRenderData.m_afBone[0] = pxInteractiveBasic->m_afPosition[0];
						xInstanceRenderData.m_afBone[1] = pxInteractiveBasic->m_afPosition[1];
						xInstanceRenderData.m_afBone[2] = pxInteractiveBasic->m_afPosition[2];
					}
#endif						

					// N.B.: since vertex buffers are allocated in write-combined memory (i.e. uncached), we can't
					// use the faster XMemCpy(), which is what we'll get if we use Asura_Mem::Copy()
					memcpy( pxCurrRenderData, &xInstanceRenderData, sizeof( xInstanceRenderData ) );

					++pxCurrRenderData;
					++xCurrLODData.m_uNumVisibleInstances;
				}
			}
		}
	}

    return pxCurrRenderData;
}
/// End of function 'Asura_PS3_Botanicals_System::CalculateLODsCache'. ///

float Asura_PS3_Botanicals_System_Distance_Scale = 1.0f;

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_Botanicals_System::CalculateLODs
// Author       : Catalin Arsenescu
// Date         : 02-August-2007
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_Botanicals_System::CalculateLODs( void )
{
    Asura_PS3_Botanicals_Instance::RenderData* const pxRenderDataStart = static_cast<Asura_PS3_Botanicals_Instance::RenderData*>( s_axInstanceVBs[s_iCurrentInstanceBuffer].LockForWriting() );
    Asura_PS3_Botanicals_Instance::RenderData* pxCurRenderData = pxRenderDataStart;
	Asura_PS3_Botanicals_Instance::RenderData* const pxRenderDataEnd = pxRenderDataStart + ASURA_BOTANICALS_MAX_ELEMENT_INSTANCES_PER_FRAME;

	// Don't assert here since there won't be an instance buffer if there aren't any botanicals in the level
	if( !pxCurRenderData ) return;

	Asura_Vector_3 xCameraPosition = *Asura_Camera::GetPosition();

	int iTemplateOffsetToNextBase = 1;
	for( u_int u = 0; u < s_uNumTemplates; u += iTemplateOffsetToNextBase)
	{
		Asura_PS3_Botanicals_Template& xCurrTemplateBase = s_pxTemplates[u];

		iTemplateOffsetToNextBase = xCurrTemplateBase.m_iOffsetToNextBase;

		if( !xCurrTemplateBase.m_pxShape ) continue;

		Asura_PS3_Botanicals_Shape& xCurrShape = *xCurrTemplateBase.m_pxShape;

		float fShapeRadius = xCurrShape.GetBoundingRadius();

		const u_int uInstanceCacheMax = 2*1024;
		static Asura_PS3_Botanicals_InstanceCache axInstanceCache[ uInstanceCacheMax];

		u_int uInstanceCacheIndex = 0;

		{
			struct TemplateCache
			{
			public:
				u_int m_uTemplateIndex;
				float m_fDistSqd;
			};
			const u_int uTemlateCacheSizeMax = 50;
			TemplateCache axTemplateCache[ uTemlateCacheSizeMax];
			u_int uTemplateCacheSize = 0;

			{
				for( int iTemplateOffset = 0; iTemplateOffset < iTemplateOffsetToNextBase; iTemplateOffset += 1)	
				{
					const u_int uTemplateIndex = u + iTemplateOffset;
					Asura_PS3_Botanicals_Template& xCurrTemplate = s_pxTemplates[ uTemplateIndex];

					Asura_Assert( xCurrTemplateBase.m_pxShape == xCurrTemplate.m_pxShape, "Current Template does not match Base template type");

#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
					if(!xCurrTemplate.IsVisibleFromCurrentCamera())
					{
						continue;
					}

					const Asura_Bounding_Box&	xTemplateBoundingBox	= xCurrTemplate.GetBoundingBox();
					const Asura_Vector_3		xDirection(xTemplateBoundingBox.CalculateCentre() - xCameraPosition);
					const float					fDistSqd				= xDirection.MagnitudeSquared();
#else
					{
						const Asura_PS3_Botanicals_Instance& xCurrInstance = xCurrTemplate.m_pxInstances[0];
						const int iModuleIndex = xCurrInstance.m_sModuleIndex;

						if( (0 <= iModuleIndex && iModuleIndex < Asura_Module_System::GetNumberOfModules()) && !Asura_Module_System::IsModuleVisible(iModuleIndex) )//IsModuleLinkedInForCurrentCamera(iModuleIndex) )
						{
							continue;
						}
					}

					float fDistSqd = 0;

					{
						const Asura_Bounding_Box& xTemplateBoundingBox = xCurrTemplate.GetBoundingBox();
						if( !Asura_Camera::IsAABBInFrustum( xTemplateBoundingBox) ) 
						{
							continue;
						}

						const Asura_Vector_3 xDirection = xTemplateBoundingBox.CalculateCentre() - xCameraPosition;
						fDistSqd = xDirection.MagnitudeSquared();
					}
#endif	//defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)

					Asura_Assert( uTemplateCacheSize < uTemlateCacheSizeMax, "Not enough Temple cache space");
					if( uTemlateCacheSizeMax <= uTemplateCacheSize)
					{
						break;
					}

					{
						TemplateCache& xTemplateCache = axTemplateCache[ uTemplateCacheSize];
						xTemplateCache.m_uTemplateIndex = uTemplateIndex;
						xTemplateCache.m_fDistSqd = fDistSqd;
					}
					uTemplateCacheSize += 1;
				}
			}

			// DLH - 11 May 2007 11:12:42 - Sort required Modules into depth order to allow the Z buffer earn its keep
			// DLH - 11 May 2007 11:13:09 - Nasty Bubble sort, please improve if you get time.

			if( 1 < uTemplateCacheSize)
			{
				for( u_int uTemplateIndexA = 0; uTemplateIndexA < (uTemplateCacheSize - 1); uTemplateIndexA += 1)
				{
					for( u_int uTemplateIndexB = (uTemplateIndexA + 1); uTemplateIndexB < (uTemplateCacheSize); uTemplateIndexB += 1)
					{
						TemplateCache& xTemplateA = axTemplateCache[ uTemplateIndexA];
						TemplateCache& xTemplateB = axTemplateCache[ uTemplateIndexB];

						if( xTemplateA.m_fDistSqd < xTemplateB.m_fDistSqd )
						{
							continue;
						}
						TemplateCache xTemplateTemp = xTemplateA;
						xTemplateA = xTemplateB;
						xTemplateB = xTemplateTemp;
					}
				}
			}

			{
				for( u_int uTemplateCacheIndex = 0; uTemplateCacheIndex < uTemplateCacheSize; uTemplateCacheIndex += 1)
				{
					u_int uTemplateIndex = axTemplateCache[ uTemplateCacheIndex].m_uTemplateIndex;
					Asura_PS3_Botanicals_Template& xCurrTemplate = s_pxTemplates[ uTemplateIndex];

					for( int k = 0; k < xCurrTemplate.m_iNumInstances; ++k )
					{
						const Asura_PS3_Botanicals_Instance& xCurrInstance = xCurrTemplate.m_pxInstances[k];

						// Do frustum test
						const float fBoundingSphereRadius = fShapeRadius * xCurrInstance.m_xRenderData.m_fScale;
						Asura_Vector_3 xBoundingSphereCentre = xCurrInstance.m_xPosition;

#if defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)
						if(!xCurrTemplate.m_pxVisibilityQueries[k].IsSphereVisibleFromCurrentCamera(xBoundingSphereCentre, fBoundingSphereRadius))
						{
							continue;
						}
#else
						if( !Asura_Camera::IsSphereInFrustum( xBoundingSphereCentre, fBoundingSphereRadius ) ) 
						{
							continue;
						}
#endif	//defined(ASURA_USE_NEW_VISIBILITY_SYSTEM)

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
						Asura_BGR_DynamicInstance* pxDynamicInstanceData = xCurrTemplate.GetDynamicInstanceData( k);
#endif
						// Check that we're in range
						// DLH - This could be changed for a dp4 plane against position to remove teh square root!

						Asura_Vector_3 xInstanceDisplacement = xCurrInstance.m_xPosition;
						xInstanceDisplacement -= *Asura_Camera::GetPosition();

						float fDist = xInstanceDisplacement.Magnitude();
						fDist *= Asura_PS3_Botanicals_System_Distance_Scale;

						if( uInstanceCacheMax <= uInstanceCacheIndex)
						{
							Asura_Assert(false, "To many instances to cache");
							break;
						}

						{
							Asura_PS3_Botanicals_InstanceCache& xInstanceCache = axInstanceCache[ uInstanceCacheIndex];

							xInstanceCache.m_fDistance = fDist;
							xInstanceCache.m_pxCurrTemplate = &xCurrTemplate;
							xInstanceCache.m_pxCurrInstance = &xCurrInstance;

#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
							xInstanceCache.m_pxDynamicInstanceData = pxDynamicInstanceData;
#endif

							//BW set visible flag, used next frame to check if it needs to animate
							//BW this could lead to a 1-frame jump in animation when botanicals first become visible, if that's the case move the visibity or animation code
							xCurrInstance.m_bVisible = true;

							uInstanceCacheIndex += 1;
						}
					}
				}
			}
		}

		for( int i = 0; i < xCurrShape.m_iNumElements; ++i )
		{
			Asura_PS3_Botanicals_Element& xCurrElement = xCurrShape.m_pxElements[i];
			pxCurRenderData = CalculateLODsCache( xCurrElement, axInstanceCache, uInstanceCacheIndex, pxCurRenderData, pxRenderDataEnd );
		}
	}

//  static int iMaxInstances = 0;
//  iMaxInstances = Asura_Maths::Max( iMaxInstances, pxCurRenderData - pxRenderDataStart );
}
