//
// Asura_Chunk_Loaders.cpp
//
// Created by Kevin Lea - 12/06/2001 11:18:56
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"

// KJFL - 2004-07-20 - NOTE:
//
// Please add new Chunk headers to Asura_Chunk_Definitions.h
//

#include "ChunkSystem/Asura_Chunk_Definitions.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_Huffman.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Menus/Asura_ProgressBar.h"
#include "Rendering/Asura_Viewport.h"

#ifdef ASURA_TOOLS
#include "Load&Save/Tools_LoadedFileList.h"
#endif

// G L O B A L S /////////////////////////////////////////////////////////////

DEBUGVAR bool dbg_bLogSucessfulAsuraFileLoads = false;
DEBUGVAR bool dbg_bLogFailedAsuraFileLoads = true;
DEBUGVAR bool dbg_bPrintSuccessfulAsuraFileLoads = false;
DEBUGVAR bool dbg_bPrintFailedAsuraFileLoads = true;

#if defined(ASURA_LOGFILES_ENABLED)
static const char* FILELOADING_LOGFILE = "logfiles\\loads.txt";
#endif	// defined(ASURA_LOGFILES_ENABLED)

// D E F I N E S /////////////////////////////////////////////////////////////

// (MP - 13.09.2002) The slightly strange format of this case is to prevent
// Lint complaining about the empty statement between cases caused by the
// CHUNK_PROCESS() calls in the switch() statement below having semi-colons.
#define CHUNK_PROCESS(chunkID, chunkClass)	\
case (chunkID):	\
{	\
	xStream.ReadChunk_GetNextHeader(acChunkBuffer, sizeof(chunkClass)); \
	class chunkClass *chunk2 = (chunkClass *)pxChunkHeader;	\
	bChunkReadCompletely = chunk2->Process(iHandle);	\
}	\
break

#define SKIP_KNOWN_CHUNK( chunk_id ) \
    case (chunk_id): bReadInChunkHeader = false; break

// G L O B A L   F U N C T I O N S ///////////////////////////////////////////
static ASURA_CHUNK_PARSER_FUNCTION ProjectSpecificChunkParser = 0;
void SetProjectSpecificChunkParser(ASURA_CHUNK_PARSER_FUNCTION function)
{
	ProjectSpecificChunkParser = function;
}

bool LoadAsuraFile(const char* szName)
{
    return Asura_ChunkLoading_System::LoadAsuraFile(szName);
}
    
    
// C L A S S E S /////////////////////////////////////////////////////////////
#ifdef ASURA_TOOLS
Asura_Chunk_Header::Asura_Chunk_Header(enum ASURA_CHUNKID id, int s, int v, int f)
{
	ID = id;
	ASURA_CHUNK_VERIFY(ID, int);
	Size = s;
	ASURA_CHUNK_VERIFY(Size, int);
	Version = v;
	ASURA_CHUNK_VERIFY(Version, int);
	Flags = f;
	ASURA_CHUNK_VERIFY(Flags, int);
}
#endif

bool Asura_Chunk_Padding::Process(Asura_Handle iFileHandle)
{
	Asura_FileAccess::SkipData(iFileHandle, Size - sizeof(Asura_Chunk_Padding));
	return true;
}

#ifdef ASURA_TOOLS
void Asura_Chunk_Padding::Create(Asura_SaveBlock* pxSaveBlock, int iNumBytes)
{
	Asura_Assert(iNumBytes > 0, "Zero sized padding?");

	int iChunkSize = sizeof(Asura_Chunk_Padding) + iNumBytes;
	Asura_Chunk_Padding* pxChunk = (Asura_Chunk_Padding*)pxSaveBlock->GetAccessPointer(iChunkSize);

    //char* const pcDataStart = reinterpret_cast<char*>( pxChunk );

	pxChunk->ID = ASURA_CHUNK_PADDING;
	ASURA_CHUNK_VERIFY(pxChunk->ID, int);
	pxChunk->Size = iChunkSize;
	ASURA_CHUNK_VERIFY(pxChunk->Size, int);
	pxChunk->Version = 0; 
	ASURA_CHUNK_VERIFY(pxChunk->Version, int);
	pxChunk->Flags = 0;
	ASURA_CHUNK_VERIFY(pxChunk->Flags, int);

	char* pcChunkBuffer = (char*)(pxChunk+1);
	Asura_Assert(iNumBytes < 16, "Do you really want to pad more than a QW");
	// Fill with junk.
	while (iNumBytes--)
	{
		ASURA_CHUNK_FILL(pcChunkBuffer, 0xCD, u_char);
	}

}
#endif


Asura_ProgressDisplay* Asura_ChunkLoading_System::s_pxProgressDisplay = NULL;
float Asura_ChunkLoading_System::s_fLoadingBarInitialValue;
float Asura_ChunkLoading_System::s_fLoadingBarEndValue;
void (*Asura_ChunkLoading_System::s_pfnRender)();  // function to call at each render step (if NULL, clears the screen)
float Asura_ChunkLoading_System::s_fUncompressedFileSize;
int Asura_ChunkLoading_System::s_iRecursionLevel = -1;

#ifdef ASURA_TOOLS
bool (*Asura_ChunkLoading_System::s_pfnChunkLoadFilter)(enum ASURA_CHUNKID eChunkID) = 0;
#endif

void Asura_ChunkLoading_System::UseLoadingBar( Asura_ProgressDisplay* pxProgressDisplay, float fEndValue, void (*pfnRender)() )
{
    s_pxProgressDisplay = pxProgressDisplay;
    s_fLoadingBarInitialValue = pxProgressDisplay->GetProgress();
    Asura_Assert(s_fLoadingBarInitialValue >= 0.0f && s_fLoadingBarInitialValue <= 1.0f, "invalid initial value");
    Asura_Assert(fEndValue >= 0.0f && fEndValue <= 1.0f, "invalid end value");
    s_fLoadingBarEndValue = fEndValue;
    s_pfnRender = pfnRender;
}

void Asura_ChunkLoading_System::KillLoadingBar()
{
    s_pxProgressDisplay = NULL;
    s_pfnRender = NULL;
}


bool Asura_ChunkLoading_System::LoadAsuraFile(const char* szName, u_char* pucChecksumOutput /* = NULL */)
{

	#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	char szFilename[260];
	Asura_FileAccess::PlatformGetNetworkFilename(szName,szFilename,260);
	szName = szFilename;
	#endif //ASURA_PC_ALLOW_LOAD_FROM_NETWORK

	s_iRecursionLevel++;
	if (s_pxProgressDisplay && s_iRecursionLevel==0) s_fLoadingBarInitialValue = s_pxProgressDisplay->GetProgress();
	
	bool bComputeChecksum = pucChecksumOutput != NULL;
	Asura_Handle iHandle = Asura_FileAccess::Open(szName, bComputeChecksum);

	if (iHandle==ASURA_INVALID_HANDLE) 
	{
		if (dbg_bLogFailedAsuraFileLoads)
		{
			Asura_Logfile_Retail( (FILELOADING_LOGFILE, "failed to load %s\n", szName) );
		}
        if (dbg_bPrintFailedAsuraFileLoads)
        {
            Asura_OutputToDebugger(("failed to load %s\n", szName));
        }
		s_iRecursionLevel--;
		return false;
	}
	else 
    {
        if (dbg_bLogSucessfulAsuraFileLoads)
	    {
		    Asura_Logfile( (FILELOADING_LOGFILE, "loaded %s\n", szName) );
	    }
        if (dbg_bPrintSuccessfulAsuraFileLoads)
        {
            Asura_OutputToDebugger(("loaded %s\n", szName));
        }
    }
	
	bool bOkay = LoadAsuraFileMain(iHandle);

	Asura_FileAccess::Close( iHandle, pucChecksumOutput );
	s_iRecursionLevel--;
	return bOkay;
}

bool Asura_ChunkLoading_System::LoadAsuraFile(Asura_Handle iFileHandle)
{
	s_iRecursionLevel++;
	if (s_pxProgressDisplay && s_iRecursionLevel==0) s_fLoadingBarInitialValue = s_pxProgressDisplay->GetProgress();

	bool bOkay = LoadAsuraFileMain(iFileHandle);
	s_iRecursionLevel--;

	return bOkay;
}

bool Asura_ChunkLoading_System::LoadAsuraFileMain(Asura_Handle iHandle)
{
	Asura_Assert(iHandle != ASURA_INVALID_HANDLE,"");

	char acChunkBuffer[512];

#ifdef ASURA_TOOLS
	Tools_LoadedFileList::AddFile(Asura_FileAccess::GetFilename(iHandle));
#endif

#if ASURA_LOG_FILE_CONTENTS
	static bool s_bCleared = false;
	if(!s_bCleared)
	{
		s_bCleared = true;
		Asura_SimpleLogfileClear(ASURA_FILE_CONTENTS_LOGFILE);
	}
	Asura_SimpleLogfileOutput((ASURA_FILE_CONTENTS_LOGFILE,"\n\n%s\n",Asura_FileAccess::GetFilename(iHandle)));
#endif

	// Kevin Lea - 07/09/2001 11:56:01 - check id
    {
        if( !Asura_FileAccess::ReadData(iHandle, acChunkBuffer, ASURA_FILE_ID_NUM_CHARS) )
        {
		    return false;
        }

	    // 1) make sure it's an Asura file
	    if( strnicmp(acChunkBuffer, ASURA_UNCOMPRESSED_FILE_ID, ASURA_UNCOMPRESSED_FILE_ID_ACTUAL_CHARS) )
	    {
		    // Kevin Lea - 07/09/2001 12:41:46 - not an Asura file
		    Asura_Assertf( false, ("\"%s\" isn't an Asura file.", Asura_FileAccess::GetFilename(iHandle)) );
            return false;
	    }

	    // 2) see if it's compressed
	    if( !strnicmp(acChunkBuffer, ASURA_COMPRESSED_FILE_ID, ASURA_COMPRESSED_FILE_ID_ACTUAL_CHARS) )
	    {
            Asura_Huffman_Decoder& xDecoder = Asura_FileAccess::SetCompressed( iHandle );

            // Kevin Lea - 07/09/2001 16:17:42 - the Asura id exists in the compressed file too, so skip it
            xDecoder.Decompress( reinterpret_cast<u_char*>(acChunkBuffer), ASURA_FILE_ID_NUM_CHARS );
	    }
    }

	// Create stream from this file
	Asura_Chunk_File_Stream xStream(iHandle);

	while(xStream.ReadChunk_GetNextID(acChunkBuffer))
	{
		// KOR - note that only the chunk ID is valid at this point
		Asura_Chunk_Header* pxChunkHeader = reinterpret_cast<Asura_Chunk_Header*>( acChunkBuffer );

		//Asura_OutputToDebugger	(("ChunkID = %c%c%c%c\n", pxChunkHeader->TextID [0], pxChunkHeader->TextID [1], pxChunkHeader->TextID [2], pxChunkHeader->TextID [3]));


		//bChunkReadCompletely takes the return value from the Process function. Used to determine
		//whether to assert if it turns out that the file position hasn't been advanded to the end of the chunk
		//RAR 08/11/02
		bool bChunkReadCompletely = false;
        bool bReadInChunkHeader   = true;
#ifdef ASURA_TOOLS
		if(s_pfnChunkLoadFilter && !s_pfnChunkLoadFilter(pxChunkHeader->ID))
		{
			//skip this chunk
			bReadInChunkHeader = false;
		}
		else
#endif
		{
			switch(pxChunkHeader->ID)
			{ 
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_MESH_VERTICES,Asura_Chunk_Mesh_Vertices);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_POLYGONS,Asura_Chunk_Mesh_Polygons);
#ifdef ASURA_USE_SAVED_MESH_COLOURS
				CHUNK_PROCESS(ASURA_CHUNK_MESH_COLOURS,Asura_Chunk_Mesh_Colours);
#endif
				CHUNK_PROCESS(ASURA_CHUNK_MESH_NORMALS,Asura_Chunk_Mesh_Normals);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_TEXTUREINFO,Asura_Chunk_Mesh_TextureInfo);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_SMOOTHINGGROUP,Asura_Chunk_Mesh_SmoothingGroup);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_POLYGONFLAGS,Asura_Chunk_Mesh_PolygonFlags);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_LOD,Asura_Chunk_Mesh_LOD);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_MISC,Asura_Chunk_Mesh_Misc);
				CHUNK_PROCESS(ASURA_CHUNK_MESH_COLLMESHGENDATA,Asura_Chunk_Mesh_CollMeshGenData);
#ifndef ASURAIMPORT_EXPORTS
				CHUNK_PROCESS(ASURA_CHUNK_ENTITY_NEXTGUID,Asura_Chunk_Entity_NextGuid);
				CHUNK_PROCESS( ASURA_CHUNK_ENVIRONMENTALLIGHTTRANSPORT, Asura_Chunk_EnvironmentalLightTransport );
				CHUNK_PROCESS( ASURA_CHUNK_GLOBALILLUMINATION, Asura_Chunk_GlobalIllumination );
				CHUNK_PROCESS( ASURA_CHUNK_POLYGONLIGHTS, Asura_Chunk_PolygonLights );
#endif // ASURAIMPORT_EXPORTS
#else // ASURA_TOOLS
				SKIP_KNOWN_CHUNK( ASURA_CHUNK_ENTITY_NEXTGUID );
#endif // !ASURA_TOOLS

				CHUNK_PROCESS(ASURA_CHUNK_ENVIRONMENT_MODULELIST,Asura_Chunk_Environment_ModuleList);
				CHUNK_PROCESS(ASURA_CHUNK_TEXTURENAMES,Asura_Chunk_TextureNames);
				CHUNK_PROCESS(ASURA_CHUNK_TEXTUREFLAGS,Asura_Chunk_TextureFlags);
				CHUNK_PROCESS(ASURA_CHUNK_MATERIAL,Asura_Chunk_Material);
				CHUNK_PROCESS(ASURA_CHUNK_MATERIALNAMES,Asura_Chunk_MaterialNames);
				CHUNK_PROCESS(ASURA_CHUNK_TEXTUREANIMATION,Asura_Chunk_TextureAnimation);

//				CHUNK_PROCESS(ASURA_CHUNK_STARTPOSITION,Asura_Chunk_StartPosition);

#ifndef ASURAIMPORT_EXPORTS
				CHUNK_PROCESS(ASURA_CHUNK_ENVIRONMENT_LBAABBTREES, Asura_Chunk_EnvironmentLBAABBTrees);
#endif // ASURAIMPORT_EXPORTS

				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_SKIN,Asura_Chunk_Hierarchy_Skin);
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_SKINLOD,Asura_Chunk_Hierarchy_SkinLOD);
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_MARKERPOINTS,Asura_Chunk_Hierarchy_MarkerPoints);
				CHUNK_PROCESS(ASURA_CHUNK_SHAPEDATA, Asura_Chunk_ShapeData);

#ifdef ASURA_TOOLS
	            CHUNK_PROCESS( ASURA_CHUNK_HIERARCHY_MANUALSECTIONBBS, Asura_Chunk_Hierarchy_ManualSectionBBs );
#endif //ASURA_TOOLS

#ifndef ASURAIMPORT_EXPORTS
#ifdef ASURA_DEVELOPMENT_PHYSICS
				CHUNK_PROCESS( ASURA_CHUNK_PHYSICSENVIRONMENT, Asura_Chunk_PhysicsEnvironment );
#endif
				CHUNK_PROCESS(ASURA_CHUNK_TILEDTEXTURE,Asura_Chunk_TiledTexture);
				CHUNK_PROCESS(ASURA_CHUNK_SHAPE,Asura_Chunk_Shape);
				CHUNK_PROCESS(ASURA_CHUNK_FRAGMENT,Asura_Chunk_Fragment);
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_COMPRESSEDANIM,Asura_Chunk_Hierarchy_CompressedAnim);
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_ANIM,Asura_Chunk_Hierarchy_Anim);
				CHUNK_PROCESS( ASURA_CHUNK_GI_SOLUTION, Asura_Chunk_GI_Solution );
	
/***
				CHUNK_PROCESS(ASURA_CHUNK_LIGHTS,Asura_Chunk_Lights);
				CHUNK_PROCESS(ASURA_CHUNK_CORONA,Asura_Chunk_Corona);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_LIGHTSCALE,Asura_Chunk_LightScale);
#endif // ASURA_TOOLS

***/
	            CHUNK_PROCESS( ASURA_CHUNK_HIERARCHY_SECTIONBBS, Asura_Chunk_Hierarchy_SectionBBs );
	            
				CHUNK_PROCESS(ASURA_CHUNK_PHONONS,Asura_Chunk_Phonons);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_PHONON_TEMPLATEGUIDS,Asura_Chunk_Phonon_TemplateGuids);
#endif // ASURA_TOOLS

#ifdef ASURA_TOOLS
	            CHUNK_PROCESS(ASURA_CHUNK_SOUNDNAMES,Asura_Chunk_SoundNames);
#endif
/***
	            CHUNK_PROCESS(ASURA_CHUNK_STREAMINGBACKGROUNDSOUND,Asura_Chunk_StreamingBackgroundSound);
	            CHUNK_PROCESS(ASURA_CHUNK_METAMUSIC,Asura_Chunk_MetaMusic);
				CHUNK_PROCESS(ASURA_CHUNK_MODULEREVERB,Asura_Chunk_ModuleReverb);
				CHUNK_PROCESS(ASURA_CHUNK_LIGHTVISIBILITIES,Asura_Chunk_LightVisibilities);

				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHY_SOUNDS,Asura_Chunk_Hierarchy_Sounds);
				CHUNK_PROCESS(ASURA_CHUNK_HIERARCHYBODYPART,Asura_Chunk_HierarchyBodyPart);
				CHUNK_PROCESS(ASURA_CHUNK_DEATHANIMFILENAMES,Asura_Chunk_DeathAnimFilenames);
***/
				
				//
				CHUNK_PROCESS(ASURA_CHUNK_SKYBOX,Asura_Chunk_SkyBox);
				CHUNK_PROCESS(ASURA_CHUNK_NAVIGATION,Asura_Chunk_Navigation);

				CHUNK_PROCESS(ASURA_CHUNK_RESOURCEFILE,Asura_Chunk_ResourceFile);
	            CHUNK_PROCESS(ASURA_CHUNK_RESOURCEFILELIST, Asura_Chunk_ResourceFileList);

				CHUNK_PROCESS(ASURA_CHUNK_CUTSCENE,Asura_Chunk_Cutscene);
				CHUNK_PROCESS(ASURA_CHUNK_CUTSCENEACTOR,Asura_Chunk_Actor);
				CHUNK_PROCESS(ASURA_CHUNK_CUTSCENETRACK,Asura_Chunk_CutsceneTrack);
				CHUNK_PROCESS(ASURA_CHUNK_CUTSCENEEVENT,Asura_Chunk_CutsceneEvent);
				CHUNK_PROCESS(ASURA_CHUNK_CUTSCENEATTACHMENT,Asura_Chunk_CutsceneAttachment);

				CHUNK_PROCESS(ASURA_CHUNK_MODULELINKING,Asura_Chunk_ModuleLinking);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_MODULENAMES,Asura_Chunk_ModuleNames);
#endif // ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_ENTITY,Asura_Chunk_Entity);
//				CHUNK_PROCESS(ASURA_CHUNK_ENTITY_TOOLSDATA,Asura_Chunk_Entity_ToolsData);
#ifdef ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_ENTITY_SECONDARYFILE,Asura_Chunk_Entity_SecondaryFile);
#endif // ASURA_TOOLS
				CHUNK_PROCESS(ASURA_CHUNK_STATICMESSAGES,Asura_Chunk_StaticMessages);
				CHUNK_PROCESS(ASURA_CHUNK_ANIMATIONTABLE,Asura_Chunk_AnimationTable);
				CHUNK_PROCESS(ASURA_CHUNK_FONT,Asura_Chunk_Font);
/***
				CHUNK_PROCESS(ASURA_CHUNK_LOCALTEXT,Asura_Chunk_LocalText);
				CHUNK_PROCESS(ASURA_CHUNK_HASHEDLOCALISEDTEXT,Asura_Chunk_HashedLocalisedText);
				CHUNK_PROCESS(ASURA_CHUNK_LOCALTEXT_PAGE,Asura_Chunk_LocalText_Page);
#ifdef ASURA_USE_ORIGINAL_PARTICLE_SYSTEM
				CHUNK_PROCESS(ASURA_CHUNK_PARTICLETEMPLATES, Asura_Chunk_ParticleTemplates);
#endif //ASURA_USE_ORIGINAL_PARTICLE_SYSTEM
#ifdef ASURA_USE_EXTENDED_PARTICLES
				CHUNK_PROCESS(ASURA_CHUNK_EXTENDED_PARTICLE_GENERATOR_TEMPLATE, Asura_Chunk_Extended_Particle_Generator_Template);
#endif // ASURA_USE_EXTENDED_PARTICLES
#ifdef ASURA_USE_EXPLOSIONS
				CHUNK_PROCESS(ASURA_CHUNK_EXPLOSION_TEMPLATE, Asura_Chunk_Explosion_Template);
#endif // ASURA_USE_EXPLOSIONS
#ifdef ASURA_USE_BILLBOARD_PLANTS
				CHUNK_PROCESS(ASURA_CHUNK_BILLBOARDPLANTS_TEMPLATES, Asura_Chunk_BillboardPlant_Template);
				CHUNK_PROCESS(ASURA_CHUNK_BILLBOARDPLANTS_BATCHES, Asura_Chunk_BillboardPlant_Batches);
#endif // ASURA_USE_BILLBOARD_PLANTS
***/
#ifdef ASURA_SUPPORT_WEATHER_SYSTEM
/***
				CHUNK_PROCESS(ASURA_CHUNK_WEATHERSYSTEM, Asura_Chunk_WeatherSystem);
***/
                CHUNK_PROCESS( ASURA_CHUNK_FOG, Asura_Chunk_Fog );
#endif // ASURA_SUPPORT_WEATHER_SYSTEM
/***
#ifdef ASURA_EXTENDED_PARTICLE_USERAIN
				CHUNK_PROCESS(ASURA_CHUNK_EXTENDED_PARTICLE_RAIN_SYSTEM, Asura_Chunk_Extended_Particle_Rain_System);
#endif // ASURA_EXTENDED_PARTICLE_USERAIN
#ifdef ASURA_SUPPORT_MULTIPLAYER_COMMS
				CHUNK_PROCESS(ASURA_CHUNK_MPCOMMS_BANNEDLIST, Asura_Chunk_MPComms_BannedList );
#endif // ASURA_SUPPORT_MULTIPLAYER_COMMS
				CHUNK_PROCESS(ASURA_CHUNK_SHAPESOUND,Asura_Chunk_ShapeSound);
***/
	            CHUNK_PROCESS(ASURA_CHUNK_FILEINFO,Asura_Chunk_FileInfo);			
	            CHUNK_PROCESS(ASURA_CHUNK_RENDERSETTINGS,Asura_Chunk_RenderSettings);			
/***
				CHUNK_PROCESS(ASURA_CHUNK_PADDING, Asura_Chunk_Padding);
***/
				CHUNK_PROCESS(ASURA_CHUNK_PFX_PCLE_TEMP, Asura_Chunk_PFX_ParticleTemp);
				CHUNK_PROCESS(ASURA_CHUNK_PFX_SOURCE_TEMP, Asura_Chunk_PFX_SourceTemp);
				CHUNK_PROCESS(ASURA_CHUNK_PFX_EFFECT_TEMP, Asura_Chunk_PFX_EffectTemp);
/***
#ifdef ASURA_USE_WATER
				CHUNK_PROCESS(ASURA_CHUNK_WATERPACKET, Asura_Chunk_WaterPacket);
				CHUNK_PROCESS(ASURA_CHUNK_WATERCONTROL, Asura_Chunk_WaterControl);
#endif //ASURA_USE_WATER

***/

#ifdef ASURA_USE_VISIBILITY_SYSTEM
				CHUNK_PROCESS( ASURA_CHUNK_VISIBILITY, Asura_Chunk_Visibility );
#endif

#endif // !ASURAIMPORT_EXPORTS
				
	            case ASURA_CHUNK_VOID:
	                // Can't happen; handled by IsValid() call above.
	                Asura_Assert( false, "" );
	                //lint -fallthrough
				
				default:
				{
					if(ProjectSpecificChunkParser)
					{
						if(ProjectSpecificChunkParser(pxChunkHeader,xStream))
						{
							bChunkReadCompletely = true;
							break;
						}
					}
					#ifndef ASURAIMPORT_EXPORTS
					Asura_Assertf( false, ("ERROR: Unknown chunk type '%c%c%c%c'.", pxChunkHeader->TextID[0], pxChunkHeader->TextID[1], pxChunkHeader->TextID[2], pxChunkHeader->TextID[3]) );
					#endif

	                Asura_OutputToDebugger_Retail(("Error: Unknown chunk type! '%c%c%c%c'.\n", pxChunkHeader->TextID[0],pxChunkHeader->TextID[1],pxChunkHeader->TextID[2],pxChunkHeader->TextID[3]));

	                bReadInChunkHeader = false;
					break;
				}
			}
		}

        if( !bReadInChunkHeader )
        {
			// KOR - read the rest of the header to get the chunk size
			xStream.ReadChunk_GetNextHeader(acChunkBuffer, sizeof(Asura_Chunk_Header));
        }
		// KOR - finish off the current chunk. The stream performs verification and skipping if necessary
		xStream.ReadChunk_End(bChunkReadCompletely);

        if (s_pxProgressDisplay && s_iRecursionLevel==0) UpdateLoadingBar(iHandle);
	}

    if (s_pxProgressDisplay && s_iRecursionLevel==0)
    {
        UpdateLoadingBar(iHandle);
    }

	return true;
}


void Asura_ChunkLoading_System::UpdateLoadingBar( Asura_Handle iFileHandle )
{
    UpdateLoadingBarWithForcedFilePosition( Asura_FileAccess::GetFilePosition(iFileHandle) );
}

void Asura_ChunkLoading_System::UpdateLoadingBarWithForcedFilePosition( u_int uForcedFilePos )
{
	if (!s_pxProgressDisplay) return;

    float fProgress = uForcedFilePos / s_fUncompressedFileSize;
    fProgress *= (s_fLoadingBarEndValue - s_fLoadingBarInitialValue);
    fProgress += s_fLoadingBarInitialValue;
    Asura_Assert(fProgress >= 0.0f && fProgress <= 1.0f, "invalid progress");
    if (fProgress < 0.0f) fProgress = 0.0f;
    if (fProgress > 1.0f) fProgress = 1.0f;

    if (s_pxProgressDisplay->Update(fProgress))
    {
        Asura_Render::BeginFrame();
        if( s_pfnRender ) { s_pfnRender(); }
        s_pxProgressDisplay->Render();
        Asura_Render::EndFrame();
    }
}

void Asura_ChunkLoading_System::SetFileSize(u_int uFileSize)
{
    s_fUncompressedFileSize = static_cast<float>(uFileSize);
}

///////////////////////////////////////////////////////////////////////////////
// Asura_Chunk_FileSize
///////////////////////////////////////////////////////////////////////////////
static const int ASURA_CHUNK_FILEINFO_VERSION = 0;
bool Asura_Chunk_FileInfo::Process(Asura_Handle iFileHandle) const
{
	Asura_Assert(Version<=ASURA_CHUNK_FILEINFO_VERSION,"Unknown chunk version");

	u_int uFileSize;
	ASURA_CHUNK_FILEEXTRACT(iFileHandle, uFileSize, u_int);
	if(Asura_ChunkLoading_System::GetRecursionLevel() == 0)
	{
    	Asura_ChunkLoading_System::SetFileSize(uFileSize);
	}

    return true;
}

#ifdef ASURA_TOOLS
void Asura_Chunk_FileInfo::Create(class Asura_SaveBlock* pxSaveBlock, u_int uFileSize)
{
	char* pcBuffer = reinterpret_cast<char*>(pxSaveBlock->GetAccessPointer(GetChunkSize()));
    Create(pcBuffer, uFileSize);
}

void Asura_Chunk_FileInfo::Create(char* pcBuffer, u_int uFileSize)
{
#if 0
	Asura_Chunk_FileInfo* pxChunk = reinterpret_cast<Asura_Chunk_FileInfo*>(pcBuffer);
    pxChunk->ID = ASURA_CHUNK_FILEINFO;
	ASURA_CHUNK_VERIFY(pxChunk->ID, int);
	pxChunk->Size = GetChunkSize();
	ASURA_CHUNK_VERIFY(pxChunk->Size, int);
	pxChunk->Version = ASURA_CHUNK_FILEINFO_VERSION;
	ASURA_CHUNK_VERIFY(pxChunk->Version, int);
	pxChunk->Flags = 0;
	ASURA_CHUNK_VERIFY(pxChunk->Flags, int);

	ASURA_CHUNK_VERIFY(uFileSize, u_int);
    pcBuffer = (char*)(pxChunk+1);
    ASURA_CHUNK_FILL(pcBuffer, uFileSize, u_int);
#else
	Asura_Chunk_Memory_Stream	xStream (uFileSize, pcBuffer);


	// Write the data...

	xStream.WriteChunk_Start	(ASURA_CHUNK_FILEINFO, ASURA_CHUNK_FILEINFO_VERSION, 0);

	xStream << uFileSize;

	xStream.WriteChunk_End	();
#endif
}
#endif

int Asura_Chunk_FileInfo::GetChunkSize()
{
	int iChunkSize = sizeof(Asura_Chunk_FileInfo);
    ASURA_CHUNK_SIZE(iChunkSize, uFileSize, u_int);
    return iChunkSize;
}

