//
// Asura_Platform_Atomic.h
//
// Created by KJFL - 28 April 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifndef __ASURA_PLATFORM_ATOMIC_H__
#define __ASURA_PLATFORM_ATOMIC_H__

#include <cell/atomic.h>

template< class T>
inline T* Asura_Atomic::CompareAndSwapPointer( T* volatile &ptPointer, T* ptCompare, T* ptSet)
{
	uint32_t* puPointer = (uint32_t*)(&ptPointer);

	uint32_t uCompare = (uint32_t)( ptCompare );
	uint32_t uSet = (uint32_t)( ptSet );

	uint32_t uOriginal = cellAtomicCompareAndSwap32( puPointer, uCompare, uSet );

	return (T*)(uOriginal);
}

template< class T>
inline T* Asura_Atomic::ExchangePointer( T* volatile &ptPointer, T* ptNew )
{
	uint32_t* puPointer = (uint32_t*)(&ptPointer);

	uint32_t uSet = (uint32_t)( ptNew );

	uint32_t uOriginal = cellAtomicStore32( puPointer, uSet );

	return (T*)(uOriginal);
}

inline u_int Asura_Atomic::CompareAndSwap( u_int volatile& uPointer, const u_int uCompare, const u_int uSet)
{
	// need to use a const cast to remove the volatile
	uint32_t* puPointer = const_cast< uint32_t* >(&uPointer);

	u_int uOriginal = cellAtomicCompareAndSwap32( puPointer, uCompare, uSet );
	
	return( uOriginal);
}

inline int Asura_Atomic::CompareAndSwap( int volatile& iPointer, const int iCompare, const int iSet)
{
	uint32_t* puPointer = ( uint32_t* )(&iPointer);
	
	int iOriginal = cellAtomicCompareAndSwap32( puPointer, iCompare, iSet);
	
	return( iOriginal);
}

inline int Asura_Atomic::Increment( u_int volatile& uVariable)
{
	// need to use a const cast to remove the volatile
	// Need to +1 because Asura_Atomic wants the new value, and 
	// PS3 OS returns old value.
	uint32_t* puVariable = const_cast< uint32_t* >(&uVariable);
	return( cellAtomicIncr32( puVariable ) + 1 );
}

inline int Asura_Atomic::Decrement( u_int volatile& uVariable)
{
	// need to use a const cast to remove the volatile
	// Need to -1 because Asura_Atomic wants the new value, and 
	// PS3 OS returns old value.uint32_t* puVariable = const_cast< uint32_t*>(&uVariable);
	uint32_t* puVariable = const_cast< uint32_t* >(&uVariable);
	return( cellAtomicDecr32( puVariable ) - 1 );
}

inline int Asura_Atomic::Increment( int volatile& iVariable)
{
	// Need to +1 because Asura_Atomic wants the new value, and 
	// PS3 OS returns old value.
	uint32_t* puVariable = (uint32_t*)&iVariable;
	return( cellAtomicIncr32( puVariable ) + 1 );
}

inline int Asura_Atomic::Decrement( int volatile& iVariable)
{	
	// Need to -1 because Asura_Atomic wants the new value, and 
	// PS3 OS returns old value.
	uint32_t* puVariable = (uint32_t*)&iVariable;
	return( cellAtomicDecr32( puVariable ) - 1 );
}

#endif // __ASURA_PLATFORM_ATOMIC_H__

