//////////////////////////////////////////////////////////////////////////////
//
// Asura_Platform_Core.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Stdlib independent.
#include <stdio.h>
#include <time.h>

// Stdlib platform-specific.
#include <sys/ppu_thread.h>
#include <sysutil/sysutil_common.h>
#include <sdk_version.h>
#include <spu_printf.h>

// Engine independent.
#include "Core/Asura_Core.h"
#include "Core/Asura_Maths.h"
#include "Debugging/Asura_Assert.h"
#ifdef ASURA_USE_PROFILING_SYSTEM
#include "Debugging/Asura_CPUProfile_Renderer.h"
#endif

// Engine platform-specific.
#include "Core/Asura_PlatformBasics.h"
#include "Debugging/Asura_PS3_ExceptionHandler.h"
#ifdef ASURA_SUPPORT_LOBBY
#include "Network/Online/Asura_PS3_NP_Basic.h"
#endif
#include "Utility/Asura_PS3_DiscBootGame.h"
#include "Utility/Asura_PS3_GameContent.h"
#include "Utility/Asura_PS3_SaveLoad.h"

// M A C R O S ///////////////////////////////////////////////////////////////

#if CELL_SDK_VERSION < 0x210001
#error ****************************************************
#error *   PS3 code must be built with SDK 210 or later   *   
#error ****************************************************
#endif

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int uSPU_PRINTF_SERVICE_PRIORITY = 100;

// G L O B A L S /////////////////////////////////////////////////////////////

static sys_ppu_thread_t g_xMainThreadID;

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

void Asura_Core::PlatformInitialise (void)
{
    // Store main thread ID for future reference.
    sys_ppu_thread_get_id( &g_xMainThreadID );

	// init the RNG with a seed using time()
	Asura_Random::SetSeed(static_cast<u_int>(time(NULL)));

	// TODO - we may need extra defines to decide if this is a disc boot or hdd boot game.
	Asura_PS3_DiscBootGame::Initialise();

	// NY - 07 April 2009 - always init this, as it loads the sysutil_game module, which is effectively mandatory
	Asura_PS3_GameContent::Initialise();

#ifdef ASURA_PS3_SUPPORT_EXCEPTION_HANDLER
	Asura_PS3_ExceptionHandler::Initialise(); //BW maybe a bit late, put earlier if poss
#endif

    // Somewhat redundant since 082, when 2 byte wchars became the default,
    // but let's check anyway. The only way this should be incorrect these
    // days is if -fno-short-wchar has been set in the compiler options.
    ASURA_STATIC_ASSERT( sizeof(Asura_UnicodeChar) == 2 );

#ifdef ASURA_USE_PROFILING_SYSTEM
    Asura_CPUProfile_Renderer::Initialise();
#endif

#ifndef ASURA_NO_DEBUG_OUTPUT
	int iRetCode = spu_printf_initialize( uSPU_PRINTF_SERVICE_PRIORITY, NULL );
    if( iRetCode != CELL_OK && iRetCode != EBUSY )
    {
	    Asura_Assertf( false, ("spu_printf_initialize() failed with code 0x%08X", iRetCode) );
    }
#endif

#ifdef ASURA_PS3_SAVEGAMES
	Asura_PS3_SaveLoad::Initialise();
#endif // #ifdef ASURA_PS3_SAVEGAMES

#ifdef ASURA_SUPPORT_LOBBY
	Asura_PS3_NP_Basic::AddConsoleCmdsAndVars();
#endif
}

void Asura_Core::PlatformShutdown (void)
{
#ifndef ASURA_NO_DEBUG_OUTPUT
	int iRetCode = spu_printf_finalize();
    if( iRetCode != CELL_OK )
    {
	    Asura_Assertf( false, ("spu_printf_finalize() failed with code 0x%08X", iRetCode) );
    }
#endif

#ifdef ASURA_USE_PROFILING_SYSTEM
    Asura_CPUProfile_Renderer::Shutdown();
#endif

#ifdef ASURA_PS3_SUPPORT_EXCEPTION_HANDLER
	Asura_PS3_ExceptionHandler::Shutdown();
#endif

#ifdef ASURA_PS3_SAVEGAMES
	Asura_PS3_SaveLoad::Shutdown();
#endif // #ifdef ASURA_PS3_SAVEGAMES

	Asura_PS3_GameContent::Shutdown();

	// TODO - we may need extra defines to decide if this is a disc-boot or hdd-boot game.
	Asura_PS3_DiscBootGame::Shutdown();
}

void Asura_Core::PlatformReset (void)
{
}

bool Asura_Core::Platform_IsMainThread (void)
{
    sys_ppu_thread_t xThreadID;
    sys_ppu_thread_get_id(&xThreadID);
    return  (g_xMainThreadID == xThreadID);
}

// F U N C T I O N   I M P L E M E N T A T I O N S ///////////////////////////

#ifndef __SNC__

void std::abort( void )
{
    // The standard behaviour of abort() is to just stop the program. This
    // isn't very useful since we can't then debug. Let's break into the
    // debugger if possible.
	printf("std:abort called\n");
    ASURA_PLATFORM_DEBUGBREAK();

    // Can't return.
    while( true ) { }
}

#endif
