///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_PS3_FMV.h
// Author    : Philip Hall
// Created   : 18/10/2007
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined __ASURA_PS3_FMV_H__
#define __ASURA_PS3_FMV_H__

// Asura Headers
#include "Rendering/Asura_Overlay_Quad.h"
#include "Core/Asura_Timers.h"

// Asura PS3 Headers
#include "FileAccess/Asura_PS3_FileSystem.h"
#include "FMV/Asura_PS3_FMV_dmux.h"				// Private dmux object
#include "FMV/Asura_PS3_FMV_adec.h"				// Private audio decoding object
#include "FMV/Asura_PS3_FMV_vdec.h"				// Private video decoding object
#include "Rendering/Asura_PS3_TextureManagement.h"



////PGH/////////////////////////////////////////
// Playback flags                             
////////////////////////////////////////////////
enum Asura_PS3_FMV_Flags
{
	ASURA_PS3_FMV_NOAUDIO   = 0x0001, 
	ASURA_PS3_FMV_ALLOWSKIP = 0x0002,
	ASURA_PS3_FMV_ASYNC     = 0x0004,
	ASURA_PS3_FMV_LOOP		= 0x0008,
	ASURA_PS3_FMV_DEFAULT   = ASURA_PS3_FMV_ALLOWSKIP,
};


////PGH/////////////////////////////////////////
// Subtitle rendering callback
////////////////////////////////////////////////
typedef void (*Asura_PS3_SubtitleCallback)(float fTime, void* pArg);


////PGH/////////////////////////////////////////
// PS3 FMV playback
////////////////////////////////////////////////
class Asura_PS3_FMV
{
protected:
	Asura_PS3_FMV(const char* szFilename, int iAudioStream, u_int uFlags, Asura_PS3_SubtitleCallback pfnCallback=NULL);
	~Asura_PS3_FMV();
	
	bool Initialise();
	void Shutdown();
	u_int Update();
	
public:
	void Begin();
	void End();

	static bool Play(const char* szFilename, int iAudioStream=0, u_int uFlags=ASURA_PS3_FMV_DEFAULT, Asura_PS3_SubtitleCallback pfnCallback=NULL);

private:
	void  ThreadDMUX();
	void  ThreadADEC();
	void  ThreadVDEC();
	void  RenderAudio();
	void  RenderVideo();

	bool  InitialiseAudio();
	bool  InitialiseVideo();

	void  DecodeThread();
	void  PlaybackThread();

	static void DecodeThreadProxy(uint64_t arg) { reinterpret_cast<Asura_PS3_FMV*>(arg)->DecodeThread(); }
	static void PlaybackThreadProxy(uint64_t arg) { reinterpret_cast<Asura_PS3_FMV*>(arg)->PlaybackThread(); }

private:
	u_int                     m_uFlags;
	int                       m_iAudioStream;
	int                       m_iVideoTimeout;
	char                      m_szFilename[ASURA_PS3_MAX_PATH];
	sys_ppu_thread_t          m_uDecodeThreadID;
	
	// Decoding components
	Asura_PS3_FMV_Streamer	  m_xStreamer;
	Asura_PS3_FMV_dmux        m_xDMux;
	Asura_PS3_FMV_adec        m_xADec;
	Asura_PS3_FMV_vdec        m_xVDec;
	Asura_PS3_FMV_Control     m_xControl;
	SPamfVideoStreamInfo      m_xVideoInfo;
	SPamfAudioStreamInfo      m_xAudioInfo;
	Asura_PS3_FMV_ESInfo*     m_pAudioInfo;
	Asura_PS3_FMV_ESInfo*     m_pVideoInfo;
	int                       m_iDMuxLookahead; // Lookahead for dmux (sometimes it needs multiple blocks)
	int                       m_iDMuxTime;
	bool                      m_bAudioStarted;

	// Video rendering
	Asura_Overlay_Quad			m_xVideoQuad;
	Asura_PS3_Texture			m_axVideoTexture[ 2 ];
	void*						m_paxVideoFrame[ 2 ];
	int							m_iCurrentTexture;
	Asura_TickCount				m_xStartTicks;
	Asura_PS3_SubtitleCallback	m_pfnSubtitleCallback;
	void*						m_pSubtitleArg;
	bool						m_bSkipping;
};



#endif

