///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_PS3_FMV_Streamer.h
// Author    : Philip Hall
// Created   : 19/10/2007
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined __ASURA_PS3_FMV_STREAMER_H__
#define __ASURA_PS3_FMV_STREAMER_H__

// Asura headers
#include "Core/Asura_RingBuffer.h"

// Asura PS3 headers
#include "FileAccess/Asura_PS3_FileSystem.h"
#include "Core/Asura_PS3_Mutex.h"
#include "Core/Asura_PS3_Semaphore.h"
#include "FMV/Asura_PS3_FMV_Control.h"


#define STMIN_BANK_SIZE	  (1024*512)  // 512kb seems quite reasonable
#define STMIN_BANK_PITCH  ( 128 * (1 + (/128)) )

struct Asura_PS3_StreamBlock
{
	u_char acData[STMIN_BANK_SIZE];
	u_int  uSize;
	bool   bStart;
};


// Stream buffer ring
typedef Asura_RingBuffer<Asura_PS3_StreamBlock, ASURA_SIZE_TO_128_BYTE(sizeof(Asura_PS3_StreamBlock))>  Asura_PS3_StreamBuffer;


////PGH/////////////////////////////////////////
// Threaded, banked streaming class     
////////////////////////////////////////////////
class Asura_PS3_FMV_Streamer
{
public:
	Asura_PS3_FMV_Streamer();
	~Asura_PS3_FMV_Streamer();

	bool Initialise(Asura_PS3_FMV_Control* pControl, const char* szFilename, off64_t fileOffset, off64_t streamSize);
	bool Open();
	void Close();

	bool Begin();
	int  End();
	void Abort();
		
	bool Lock(u_int uTimeoutUS)   { return m_xBufferLock.Lock(uTimeoutUS); }
	void Unlock()                 { m_xBufferLock.Unlock(); }
	Asura_PS3_Semaphore& Signal() { return m_xReadySignal; } 
	
	Asura_PS3_StreamBuffer& Buffer() { return m_xReadBuffer; }
	
private:
	void Thread();
	static void ThreadProxy(uint64_t arg) { reinterpret_cast<Asura_PS3_FMV_Streamer*>(arg)->Thread(); }

private:
	Asura_PS3_FMV_Control* m_pControl;
	Asura_PS3_Mutex        m_xBufferLock;
	Asura_PS3_Semaphore    m_xReadySignal;
	
	// Threaded reading
	volatile bool          m_bFinished;
	sys_ppu_thread_t       m_threadId;
	off64_t                m_fileOffset;
	off64_t                m_streamSize;
	uint64_t               m_uTotalBytes;

	// File buffering
	char                         m_szFilename[ASURA_PS3_MAX_PATH];
	Asura_PS3_FileSystem::Handle m_StreamHandle;
	Asura_PS3_StreamBuffer       m_xReadBuffer;
};



#endif










