///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_PS3_FMV_dmux.h
// Author    : Philip Hall
// Created   : 18/10/2007
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined __ASURA_PS3_FMV_DMUX_H__
#define __ASURA_PS3_FMV_DMUX_H__

// Asura PS3 headers
#include "Core/Asura_PS3_Mutex.h"
#include "FMV/Asura_PS3_FMV_Control.h"
#include "FMV/Asura_PS3_FMV_Streamer.h"

#include <cell/codec.h>


enum Asura_PS3_FMV_DecodeResult
{
	ASURA_PS3_FMV_DECODE_OK = 0x00,
	ASURA_PS3_FMV_DECODE_BUSY = 0x01,
	ASURA_PS3_FMV_DECODE_EMPTY = 0x02,
};


////PGH/////////////////////////////////////////
// Structures taken from PAMF sample (pamf_dmux)
////////////////////////////////////////////////
union UDmuxEsSpecInfo 
{
	CellDmuxPamfEsSpecificInfoM2v			m2v;
	CellDmuxPamfEsSpecificInfoAvc			avc;
	CellDmuxPamfEsSpecificInfoLpcm			lpcm;
	CellDmuxPamfEsSpecificInfoAc3			ac3;
	CellDmuxPamfEsSpecificInfoAtrac3plus	at3plus;
};

struct SPamfVideoStreamInfo
{
	bool                bPresent;
	uint8_t				uType;
	CellDmuxEsFilterId	esFilterId;
	UDmuxEsSpecInfo		esSpecInfo;
	uint32_t			uWidth;
	uint32_t			uHeight;
	uint32_t			vdecSpuNum;
	uint32_t			bufferNum;
	const char*			szType;
};

struct SPamfAudioStreamInfo
{
	bool                bPresent;
	uint8_t				uType;
	CellDmuxEsFilterId	esFilterId;
	UDmuxEsSpecInfo		esSpecInfo;
	uint32_t			uChannels;
	uint32_t			bufferNum;
	uint32_t			auSample;
	const char*			szType;
};



////PGH/////////////////////////////////////////
// DMUX elementary stream information         
////////////////////////////////////////////////
class Asura_PS3_FMV_ESInfo
{
private:
	Asura_PS3_Mutex     xLock;
public:
	uint8_t				uStreamType;
	CellDmuxEsHandle	xHandle;
	UDmuxEsSpecInfo		xSpecInfo;
	uint8_t*			pWorkMemory;
	Asura_PS3_Semaphore xAUDone;
	Asura_PS3_Semaphore xReady;
	volatile int        iDMUXFoundAUCount;
	
	Asura_PS3_FMV_ESInfo()
	{
		uStreamType  = 0;  
		xHandle     = NULL;
		pWorkMemory = NULL;
		iDMUXFoundAUCount = 0;
	}

	~Asura_PS3_FMV_ESInfo()
	{
		if ( pWorkMemory != NULL )
		{
			Asura_Deallocate(pWorkMemory);
		}
	}
	
	bool Create(size_t uMemorySize)
	{
		pWorkMemory = reinterpret_cast<uint8_t*>( Asura_Allocate(uMemorySize) );
		if (pWorkMemory == NULL)
		{
			Asura_OutputToDebugger(("[Asura_PS3_FMV_ESInfo::Create] - Unable to allocate working memory\n"));
			return false;
		}
		return true;		
	}
	
	void Destroy()
	{
		xAUDone.Destroy();
		xReady.Destroy();
		xLock.Destroy();

		if ( pWorkMemory != NULL )
		{
			Asura_Deallocate(pWorkMemory);
			pWorkMemory = NULL;
		}
	}

	bool Lock(u_int uTimeoutUS=0)
	{
		return xLock.Lock(uTimeoutUS);
	}

	bool Unlock() 
	{
		return xLock.Unlock();
	}
};


////PGH/////////////////////////////////////////
// libdmux wrapper
////////////////////////////////////////////////
class Asura_PS3_FMV_dmux
{
public:
	Asura_PS3_FMV_dmux();
	~Asura_PS3_FMV_dmux();
	
	void Initialise(Asura_PS3_FMV_Control* pControl, uint32_t uESCount);
	bool Open();
	void Close();

	bool StartProcessing(void* pData, size_t uSize, bool bFirst);
	bool IsBusy();
	int  ProcessedCount();
	void AdjustProcessedCount(int iCount);
	
	Asura_PS3_FMV_ESInfo* EnableEs(uint8_t streamType, CellDmuxEsFilterId* pEsFilterId, UDmuxEsSpecInfo* pEsSpecInfo);
	void         DisableEs(Asura_PS3_FMV_ESInfo* pDmuxEsInfo);

protected:
	static uint32_t ESCallback(CellDmuxHandle handle, CellDmuxEsHandle esHandle, const CellDmuxEsMsg* pDmuxEsMsg, void* cbArg);
	static uint32_t Callback(CellDmuxHandle handle, const CellDmuxMsg* pDmuxMsg, void* callbackArg);
	
private:
	Asura_PS3_FMV_Control* m_pControl;
	Asura_PS3_Mutex        m_xLock;
	volatile bool          m_bDmuxBusy;
	volatile int           m_iProcessedCount;
	int                    m_iRunningFrameCount;

	CellDmuxType           m_xType;
	CellDmuxHandle         m_handle;
	size_t                 m_uMemSize;
	uint8_t*               m_pWorkMemory;
	uint32_t               m_uESCount;
	Asura_PS3_FMV_ESInfo*  m_axESInfo;
};


#endif




