///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_PS3_FMV_vdec.h
// Author    : Philip Hall
// Created   : 18/10/2007
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#if !defined __ASURA_PS3_FMV_VDEC_H__
#define __ASURA_PS3_FMV_VDEC_H__

// Asura PS3 headers
#include "Core/Asura_PS3_Mutex.h"
#include "FMV/Asura_PS3_FMV_dmux.h"

#include <cell/codec.h>
#include <cell/codec/vdec.h>



////PGH/////////////////////////////////////////
// Video storage marker structure
////////////////////////////////////////////////
struct Asura_PS3_VideoBlock
{
	u_int  uSize;
	void  *pFrame;
	CellVdecTimeStamp xTimestamp;
};

// Video buffer ring
typedef Asura_RingBuffer<Asura_PS3_VideoBlock>  Asura_PS3_VideoBuffer;


////PGH/////////////////////////////////////////
// libvdec wrapper
////////////////////////////////////////////////
class Asura_PS3_FMV_vdec
{
public:
	Asura_PS3_FMV_vdec();
	~Asura_PS3_FMV_vdec();

	void Initialise(Asura_PS3_FMV_Control* pControl);
	bool Open(Asura_PS3_FMV_ESInfo* pEsInfo, u_int uWidth, u_int uHeight, u_int uSPUCount);
	void Close();

	Asura_PS3_FMV_DecodeResult StartProcessing();

	Asura_PS3_VideoBuffer& Buffer() { return m_xVideoBuffer; }

protected:
	uint32_t Callback(CellVdecHandle handle, CellVdecMsgType msgType, int32_t msgData);

	static uint32_t CallbackProxy(CellAdecHandle handle, CellVdecMsgType msgType, int32_t msgData, void *cbArg)
		{ return reinterpret_cast<Asura_PS3_FMV_vdec*>(cbArg)->Callback(handle, msgType, msgData); }

	bool GetNextFrame();
	u_int GetFrameRate(const CellVdecPicItem *pPicItem );

private:
	Asura_PS3_FMV_Control*  m_pControl;
	Asura_PS3_FMV_ESInfo*   m_pESInfo;
	const CellDmuxAuInfo*   m_pAUInfo;
	uint16_t                m_uPRXModule;
	CellVdecType            m_decType;
	CellVdecHandle          m_decHandle;
	size_t                  m_uMemSize;
	void*                   m_pWorkMemory;
	int                     m_iRunningFrameCount;

	// Video buffering
	Asura_PS3_Mutex        m_xBufferLock;
	Asura_PS3_VideoBuffer  m_xVideoBuffer;
	u_char*                m_pVideoBufferMemory;
	u_int                  m_uFrameSizeBytes;
};


#endif




