//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined ASURA_USE_FSFX

#include "FSFX/Asura_PS3_FSFX.h"

// Asura Platform-independent
#include "FSFX/Asura_FSFX_Component.h"
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Camera/Asura_Camera.h"
#include "Interface/Asura_CommandConsole.h"

// Asura PS3
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_MaterialResponse_System.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_GCM_VertexBuffer.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"

// Misc

//Shaders definition

#include "Rendering/ShadersDefinition/vd/Fog_VD.h"
#include "Rendering/ShadersDefinition/vd/FSFX_Noise_VD.h"

#include "Rendering/ShadersDefinition/vp/Fog_VP.h"
#include "Rendering/ShadersDefinition/vp/FSFX_RadialBlur_VP.h"
#include "Rendering/ShadersDefinition/vp/FSFX_Noise_VP.h"
#include "Rendering/ShadersDefinition/vp/FSFX_SM30_Bloom_VP.h"

#include "Rendering/ShadersDefinition/fp/FSFX_BrightnessContrast_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_ColourMap_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_DesatTint_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_Gamma_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_RadialBlur_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_Noise_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_SM30_Bloom_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_FishEye_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_DoubleVision_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_SM30_DepthOfField_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_Bloom_FP.h"


/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

/******************************************************************************/
/* Global Variables                                                           */
/******************************************************************************/
static Asura_PS3_GCM_VertexShader*		g_pxVertexShaderSimple	= NULL;
static Asura_PS3_GCM_VertexShader*		g_pxVertexShaderRB		= NULL;
static Asura_PS3_GCM_VertexShader*		g_pxVertexShaderN		= NULL;
static Asura_PS3_GCM_VertexShader*		g_pxVertexShaderB30		= NULL;

static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderBC	= NULL;
static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderCM	= NULL;
static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderDT	= NULL;
static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderG		= NULL;
static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderRB	= NULL;
static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderN		= NULL;
static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderB30	= NULL;
static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderDOF30	= NULL;
static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderFE	= NULL;
static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderDV	= NULL;
static Asura_PS3_FragmentShaderCopy*	g_pxFragmentShaderB		= NULL;

static u_int g_uBloomRTWidth = 0;
static u_int g_uBloomRTHeight = 0;

static Asura_PS3_Texture				g_xAdvBloomTexture;
static Asura_PS3_Texture				g_xAdvBloomTempTexture;
static Asura_PS3_Texture				g_xBloomTexture;

static Asura_PS3_GCM_VertexBuffer		g_xVertexBufferSimple;
static Asura_PS3_GCM_VertexBuffer		g_xVertexBufferNoise;
static Asura_PS3_GCM_VertexBuffer		g_xVertexBufferFishEye;
static Asura_PS3_GCM_VertexBuffer		g_xVertexBufferDoubleVision;

#define FSFX_BC				 1
#define FSFX_CM				 2
#define FSFX_DT				 3
#define FSFX_G				 4
#define FSFX_RB				 5
#define FSFX_N				 6

#define FSFX_B30_0000		 7
#define FSFX_B30_0100		 8
#define FSFX_B30_0110		 9
#define FSFX_B30_1000		10
#define FSFX_B30_1100		11
#define FSFX_B30_1110		12

#define FSFX_B30_0001		13
#define FSFX_B30_0101		14
#define FSFX_B30_0111		15
#define FSFX_B30_1001		16
#define FSFX_B30_1101		17
#define FSFX_B30_1111		18

#define FSFX_FE0			19
#define FSFX_FE1			20
#define FSFX_FE2			21
#define FSFX_FE3			22

#define FSFX_DOF30			23

#define FSFX_DV				24

#define FSFX_B0				25
#define FSFX_B1				26
#define FSFX_B2				27
#define FSFX_B3				28

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
static bool s_bBrightnessContrast	= true;
static bool s_bColourMap			= true;
static bool s_bDesatTint			= true;
static bool s_bGamma				= true;
static bool s_bRadialBlur			= true;
static bool s_bNoise				= true;
static bool s_bBloom				= true;
static bool s_bSM30_Bloom			= true;
static bool s_bFishEye				= true;
static bool s_bDoubleVision			= true;
static bool s_bDepthOfField			= true;
#endif
////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::Initialise()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::Initialise( void )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	Asura_CommandConsole::AddVar( L"Render.FSFX.BrightnessContrast",	&s_bBrightnessContrast,	Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.FSFX.ColourMap",				&s_bColourMap,			Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.FSFX.DesatTint",				&s_bDesatTint,			Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.FSFX.Gamma",					&s_bGamma,				Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.FSFX.RadialBlur",			&s_bRadialBlur,			Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.FSFX.Noise",					&s_bNoise,				Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.FSFX.Bloom",					&s_bBloom,				Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.FSFX.SM30_Bloom",			&s_bSM30_Bloom,			Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.FSFX.FishEye",				&s_bFishEye,			Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.FSFX.DoubleVision",			&s_bDoubleVision,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"Render.FSFX.DepthOfField",			&s_bDepthOfField,		Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif
	g_pxVertexShaderSimple	= new Asura_PS3_VP_Fog();
	g_pxVertexShaderRB		= new Asura_PS3_VP_RadialBlur();
	g_pxVertexShaderN		= new Asura_PS3_VP_Noise();
	g_pxVertexShaderB30		= new Asura_PS3_VP_SM30_Bloom();

	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_BC, &_binary_PS3_FSFX_BrightnessContrast_FP_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_BrightnessContrast_FP_bin_end - (unsigned char *)&_binary_PS3_FSFX_BrightnessContrast_FP_bin_start ), new Asura_PS3_FSFX_BC_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_CM, &_binary_PS3_FSFX_ColourMap_FP_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_ColourMap_FP_bin_end - (unsigned char *)&_binary_PS3_FSFX_ColourMap_FP_bin_start ), new Asura_PS3_FSFX_CM_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_DT, &_binary_PS3_FSFX_DesatTint_FP_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_DesatTint_FP_bin_end - (unsigned char *)&_binary_PS3_FSFX_DesatTint_FP_bin_start ), new Asura_PS3_FSFX_DT_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_G, &_binary_PS3_FSFX_Gamma_FP_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_Gamma_FP_bin_end - (unsigned char *)&_binary_PS3_FSFX_Gamma_FP_bin_start ), new Asura_PS3_FSFX_G_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_RB, &_binary_PS3_FSFX_RadialBlur_FP_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_RadialBlur_FP_bin_end - (unsigned char *)&_binary_PS3_FSFX_RadialBlur_FP_bin_start ), new Asura_PS3_FSFX_RB_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_N, &_binary_PS3_FSFX_Noise_FP_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_Noise_FP_bin_end - (unsigned char *)&_binary_PS3_FSFX_Noise_FP_bin_start ), new Asura_PS3_FSFX_N_Handles, 1, true );

	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_0000, &_binary_PS3_FSFX_SM30_Bloom_FP0000_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0000_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0000_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_0100, &_binary_PS3_FSFX_SM30_Bloom_FP0100_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0100_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0100_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_0110, &_binary_PS3_FSFX_SM30_Bloom_FP0110_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0110_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0110_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_1000, &_binary_PS3_FSFX_SM30_Bloom_FP1000_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1000_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1000_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_1100, &_binary_PS3_FSFX_SM30_Bloom_FP1100_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1100_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1100_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_1110, &_binary_PS3_FSFX_SM30_Bloom_FP1110_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1110_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1110_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );

	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_0001, &_binary_PS3_FSFX_SM30_Bloom_FP0001_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0001_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0001_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_0101, &_binary_PS3_FSFX_SM30_Bloom_FP0101_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0101_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0101_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_0111, &_binary_PS3_FSFX_SM30_Bloom_FP0111_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0111_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP0111_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_1001, &_binary_PS3_FSFX_SM30_Bloom_FP1001_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1001_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1001_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_1101, &_binary_PS3_FSFX_SM30_Bloom_FP1101_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1101_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1101_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B30_1111, &_binary_PS3_FSFX_SM30_Bloom_FP1111_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1111_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_Bloom_FP1111_bin_start ), new Asura_PS3_FSFX_SM30Bloom_Handles, 1, true );

	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_FE0, &_binary_PS3_FSFX_FishEye_FP0_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_FishEye_FP0_bin_end - (unsigned char *)&_binary_PS3_FSFX_FishEye_FP0_bin_start ), new Asura_PS3_FSFX_FE_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_FE1, &_binary_PS3_FSFX_FishEye_FP1_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_FishEye_FP1_bin_end - (unsigned char *)&_binary_PS3_FSFX_FishEye_FP1_bin_start ), new Asura_PS3_FSFX_FE_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_FE2, &_binary_PS3_FSFX_FishEye_FP2_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_FishEye_FP2_bin_end - (unsigned char *)&_binary_PS3_FSFX_FishEye_FP2_bin_start ), new Asura_PS3_FSFX_FE_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_FE3, &_binary_PS3_FSFX_FishEye_FP3_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_FishEye_FP3_bin_end - (unsigned char *)&_binary_PS3_FSFX_FishEye_FP3_bin_start ), new Asura_PS3_FSFX_FE_Handles, 1, true );

	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_DOF30, &_binary_PS3_FSFX_SM30_DepthOfField_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_SM30_DepthOfField_bin_end - (unsigned char *)&_binary_PS3_FSFX_SM30_DepthOfField_bin_start ), new Asura_PS3_FSFX_SM30DepthOfField_Handles, 1, true );

	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_DV, &_binary_PS3_FSFX_DoubleVision_FP_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_DoubleVision_FP_bin_end - (unsigned char *)&_binary_PS3_FSFX_DoubleVision_FP_bin_start ), new Asura_PS3_FSFX_FE_Handles, 1, true );

	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B0, &_binary_PS3_FSFX_Bloom_FP0_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_Bloom_FP0_bin_end - (unsigned char *)&_binary_PS3_FSFX_Bloom_FP0_bin_start ), new Asura_PS3_FSFX_B_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B1, &_binary_PS3_FSFX_Bloom_FP1_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_Bloom_FP1_bin_end - (unsigned char *)&_binary_PS3_FSFX_Bloom_FP1_bin_start ), new Asura_PS3_FSFX_B_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B2, &_binary_PS3_FSFX_Bloom_FP2_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_Bloom_FP2_bin_end - (unsigned char *)&_binary_PS3_FSFX_Bloom_FP2_bin_start ), new Asura_PS3_FSFX_B_Handles, 1, true );
	Asura_PS3_GCM_ShadersLibrary::AddFP( ASURA_SHADER_HASH_FSFX, FSFX_B3, &_binary_PS3_FSFX_Bloom_FP3_bin_start, ( (unsigned char *)&_binary_PS3_FSFX_Bloom_FP3_bin_end - (unsigned char *)&_binary_PS3_FSFX_Bloom_FP3_bin_start ), new Asura_PS3_FSFX_B_Handles, 1, true );

	g_pxVertexShaderSimple->InitVertexBufferDeclaration( g_axVertexDeclaration, VD_ELEMENTS );
	g_pxVertexShaderN->InitVertexBufferDeclaration( g_axVertexDeclarationN, VDN_ELEMENTS );

	g_uBloomRTWidth = 320;
	g_uBloomRTHeight = 180;

	//could create bloom RT here, but I'll just use a small chunk of the other one to save vram
//BW WHAT?S
	g_xBloomTexture.CreateTexture( g_uBloomRTWidth, g_uBloomRTHeight, ASURA_PS3_TEXTUREFORMAT_8888_RGBA );
	g_xAdvBloomTexture.CreateTexture( Asura_PS3_Graphics::GetScreenWidth() >> 1, Asura_PS3_Graphics::GetScreenHeight() >> 1, ASURA_PS3_TEXTUREFORMAT_8888_RGBA );
	g_xAdvBloomTempTexture.CreateTexture( Asura_PS3_Graphics::GetScreenWidth() >> 1, Asura_PS3_Graphics::GetScreenHeight() >> 1, ASURA_PS3_TEXTUREFORMAT_8888_RGBA );

	g_xVertexBufferSimple.Initialise( FOG_VERTEX_SIZE, 4, CELL_GCM_LOCATION_MAIN );
	g_xVertexBufferNoise.Initialise( NOISE_VERTEX_SIZE, 4, CELL_GCM_LOCATION_MAIN );
	g_xVertexBufferFishEye.Initialise( NOISE_VERTEX_SIZE, 4, CELL_GCM_LOCATION_MAIN );
	g_xVertexBufferDoubleVision.Initialise( NOISE_VERTEX_SIZE, 4, CELL_GCM_LOCATION_MAIN );

	float* pfVertexData = ( float* )g_xVertexBufferSimple.LockForWriting();
	pfVertexData[ 0 ] = -1; pfVertexData[ 1 ] =  1; //Top left
	pfVertexData[ 2 ] =  1; pfVertexData[ 3 ] =  1; //Top right
	pfVertexData[ 4 ] =  1; pfVertexData[ 5 ] = -1; //Bottom right
	pfVertexData[ 6 ] = -1; pfVertexData[ 7 ] = -1; //Bottom left
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::Reset()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::Reset( void )
{

}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::Shutdown()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::Shutdown( void )
{
	g_xAdvBloomTexture.Destroy();
	g_xAdvBloomTempTexture.Destroy();
	g_xVertexBufferSimple.Shutdown();
	g_xVertexBufferNoise.Shutdown();
	g_xVertexBufferFishEye.Shutdown();
	g_xVertexBufferDoubleVision.Shutdown();

	delete g_pxVertexShaderSimple;
	delete g_pxVertexShaderRB;
	delete g_pxVertexShaderN;
	delete g_pxVertexShaderB30;

	g_pxVertexShaderSimple	= NULL;
	g_pxVertexShaderRB		= NULL;
	g_pxVertexShaderN		= NULL;
	g_pxVertexShaderB30		= NULL;

	g_pxFragmentShaderBC	= NULL;
	g_pxFragmentShaderCM	= NULL;
	g_pxFragmentShaderDT	= NULL;
	g_pxFragmentShaderG		= NULL;
	g_pxFragmentShaderRB	= NULL;
	g_pxFragmentShaderN		= NULL;
	g_pxFragmentShaderB30	= NULL;
	g_pxFragmentShaderFE	= NULL;
	g_pxFragmentShaderDV	= NULL;
	g_pxFragmentShaderB		= NULL;
}

void PrepareRenderTarget( void )
{
	Asura_PS3_DeferredRendering::GetTempFrameBuffer()->Bind();
}

void RestoreViewport( void )
{
	Asura_PS3_DeferredRendering::BindAsSingleRT();
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::BrightnessAndContrast()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::BrightnessAndContrast( float* pfCoeffs )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bBrightnessContrast ) return;
#endif
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "BrightnessAndContrast" );
	Asura_PS3_DeferredRendering::ResolveBackBuffer();

	// Pixel shader will output fog colour with fog factor as alpha
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );

	// Taken these settings out of the effect, as they were screwing up the cached states
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);
	g_pxFragmentShaderBC = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_BC );

	RestoreViewport();

	g_pxFragmentShaderBC->SetFloats ( FPBC_Coefficients, 2, pfCoeffs );
	g_pxFragmentShaderBC->SetTexture( FPBC_InitialImage, Asura_PS3_DeferredRendering::GetResolvedTexture(), false );

	g_pxVertexShaderSimple->Bind();
	g_pxFragmentShaderBC->Bind();
	g_xVertexBufferSimple.Bind( g_axVertexDeclaration, VD_ELEMENTS );
	g_xVertexBufferSimple.RenderQuads( 4 );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::DesaturateAndTint()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::DesaturateAndTint( float* pfTintStrength )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bDesatTint ) return;
#endif
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "DesaturateAndTint" );
	Asura_PS3_DeferredRendering::ResolveBackBuffer();

	// Pixel shader will output fog colour with fog factor as alpha
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_NONE );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );

	// Taken these settings out of the effect, as they were screwing up the cached states
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	RestoreViewport();

	g_pxFragmentShaderDT = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_DT );
	g_pxFragmentShaderDT->SetFloats ( FPDT_TintColourStrength, 4, pfTintStrength );
	g_pxFragmentShaderDT->SetTexture( FPDT_InitialImage, Asura_PS3_DeferredRendering::GetResolvedTexture(), false );

	g_pxVertexShaderSimple->Bind();
	g_pxFragmentShaderDT->Bind();
	g_xVertexBufferSimple.Bind( g_axVertexDeclaration, VD_ELEMENTS );
	g_xVertexBufferSimple.RenderQuads( 4 );
	Asura_PS3_Graphics::Flush();
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::Noise()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::Noise( float* pfUV, int iTextureIndex, float* pfEffecStrength )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bNoise ) return;
#endif
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Noise" );

	RestoreViewport();

	float* pfVertexData = ( float* )g_xVertexBufferNoise.LockForWriting();
	pfVertexData[  0 ] = -1; pfVertexData[  1 ] =  1; //Top left Pos
	pfVertexData[  2 ] = pfUV[ 0 ]; pfVertexData[  3 ] =  pfUV[ 1 ]; //Top left UV
	pfVertexData[  4 ] =  1; pfVertexData[  5 ] =  1; //Top right Pos
	pfVertexData[  6 ] =  pfUV[ 0 ] + pfUV[ 2 ]; pfVertexData[  7 ] =  pfUV[ 1 ]; //Top right UV
	pfVertexData[  8 ] =  1; pfVertexData[  9 ] = -1; //Bottom right Pos
	pfVertexData[ 10 ] =  pfUV[ 0 ] + pfUV[ 2 ]; pfVertexData[ 11 ] = pfUV[ 1 ] + pfUV[ 3 ]; //Bottom right UV
	pfVertexData[ 12 ] = -1; pfVertexData[ 13 ] = -1; //Bottom left Pos
	pfVertexData[ 14 ] = pfUV[ 0 ]; pfVertexData[ 15 ] = pfUV[ 1 ] + pfUV[ 3 ]; //Bottom left UV
	g_xVertexBufferNoise.Unlock();


	// Pixel shader will output fog colour with fog factor as alpha
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_UGH );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );

	// Taken these settings out of the effect, as they were screwing up the cached states
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	g_pxFragmentShaderN = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_N );
	g_pxFragmentShaderN->SetFloats ( FPN_Colour, 1, pfEffecStrength );
	g_pxFragmentShaderN->SetTexture( FPN_NoiseTexture, iTextureIndex );

	g_pxVertexShaderN->Bind();
	g_pxFragmentShaderN->Bind();
	g_xVertexBufferNoise.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
	g_xVertexBufferNoise.RenderQuadsInline( 4, pfVertexData );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::GaussianBlur()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::GaussianBlur( const float fIntensity )
{

}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::RadialBlur()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::RadialBlur( float* pfSampleWeights, const float* pxVertexTransforms )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bRadialBlur ) return;
#endif
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "RadialBlur" );
	Asura_PS3_DeferredRendering::ResolveBackBuffer();

	// Pixel shader will output fog colour with fog factor as alpha
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );

	// Taken these settings out of the effect, as they were screwing up the cached states
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	RestoreViewport();

	g_pxFragmentShaderRB = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_RB );
	g_pxFragmentShaderRB->SetFloats ( FPRB_SampleWeights, 4, pfSampleWeights );
	g_pxFragmentShaderRB->SetTexture( FPRB_Texture, Asura_PS3_DeferredRendering::GetResolvedTexture(), true );

	g_pxVertexShaderRB->Bind();
	g_pxVertexShaderRB->SetUniformParameter( VPRB_VertexTransforms, pxVertexTransforms, 4 );
	g_pxFragmentShaderRB->Bind();
	g_xVertexBufferSimple.Bind( g_axVertexDeclaration, VD_ELEMENTS );
	g_xVertexBufferSimple.RenderQuads( 4 );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );
}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::DepthOfField()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::DepthOfField( const float fIntensity )
{

}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::Bloom()
//
// PS3 implementation of the "inferior" (not SM30) bloom, apparently the artists like it better.
//
// Author: Ben Weston (based on Xbox360 version)
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::Bloom( const float fIntensity, const float* afTint, const float fThreshold )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bBloom ) return;
#endif
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Bloom" );

	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Extract Bloom" );
	Asura_PS3_RenderingStates::RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	const float fScale = 1.0f / (1.0f - 0.25f);
	const float fBias = 0.25f / (1.0f - 0.25f);
	const float fLevel = fThreshold * 0.75f / 100.0f;

	const float fScreenWidth = static_cast< float >( Asura_Viewport::GetWidth() );
	const float fScreenHeight = static_cast< float >( Asura_Viewport::GetHeight() );
	const float fTexWidth = static_cast< float >( g_uBloomRTWidth );
	const float fTexHeight = static_cast< float >( g_uBloomRTHeight );
	const float afRcpScreenSizes[] = {	fScreenWidth / fTexWidth,
										fScreenHeight / fTexHeight,
										fTexWidth / fScreenWidth,
										fTexHeight / fScreenHeight };

	Asura_PS3_DeferredRendering::ResolveBackBuffer();
	//PrepareRenderTarget();
	Asura_PS3_DeferredRendering::GetTempFrameBuffer()->Bind();

	g_pxVertexShaderSimple->Bind();

	g_pxFragmentShaderB = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_B0 );
	g_pxFragmentShaderB->SetTexture( FPB_InitialImage, Asura_PS3_DeferredRendering::GetResolvedTexture(), false );
	g_pxFragmentShaderB->SetFloats ( FPBLOOM_BloomTint, 3, afTint );
	g_pxFragmentShaderB->SetFloats ( FPBLOOM_BloomScale, 1, &fScale );
	g_pxFragmentShaderB->SetFloats ( FPBLOOM_BloomBias, 1, &fBias );
	g_pxFragmentShaderB->SetFloats ( FPBLOOM_BloomLevel, 1, &fLevel );
	g_pxFragmentShaderB->SetFloats ( FPBLOOM_BloomRcpSizes, 4, afRcpScreenSizes );
	g_pxFragmentShaderB->Bind();

	// Extract glowing parts of the source image
	g_xVertexBufferSimple.Bind( g_axVertexDeclaration, VD_ELEMENTS );
	g_xVertexBufferSimple.RenderQuads( 4 );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );


	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Horizontal Blur" );

	Asura_PS3_DeferredRendering::GetTempFrameBuffer()->ResolveBackBuffer( &g_xBloomTexture );
	Asura_PS3_DeferredRendering::GetTempFrameBuffer()->Bind();


	// Blur the glow horizontally
	g_pxFragmentShaderB = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_B1 );
	g_pxFragmentShaderB->SetTexture( FPB_BlurImage, &g_xBloomTexture, false );
	g_pxFragmentShaderB->Bind();
	g_xVertexBufferSimple.RenderQuads( 4 );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Vertical Blur" );


	Asura_PS3_DeferredRendering::GetTempFrameBuffer()->ResolveBackBuffer( &g_xBloomTexture );
	Asura_PS3_DeferredRendering::GetTempFrameBuffer()->Bind();

	// Then blur it vertically
	g_pxFragmentShaderB = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_B2 );
	g_pxFragmentShaderB->SetTexture( FPB_BlurImage, &g_xBloomTexture, false );
	g_pxFragmentShaderB->Bind();
	g_xVertexBufferSimple.RenderQuads( 4 );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );


	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Apply Bloom" );

	Asura_PS3_DeferredRendering::GetTempFrameBuffer()->ResolveBackBuffer( &g_xBloomTexture );
	RestoreViewport();


	// Finally, add it on top of the scene
	g_pxFragmentShaderB = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_B3 );
	g_pxFragmentShaderB->SetTexture( FPB_BlurImage, &g_xBloomTexture, false );
	g_pxFragmentShaderB->SetFloats ( FPBLOOM_BloomLevel, 1, &fLevel );
	g_pxFragmentShaderB->SetFloats ( FPBLOOM_BloomRcpSizes, 4, afRcpScreenSizes );
	g_pxFragmentShaderB->Bind();

	Asura_PS3_RenderingStates::RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);

	g_xVertexBufferSimple.RenderQuads( 4 );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );


	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::Gamma()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::Gamma( float* pfCoef0, float* pfCoef1, float* pfCoef2, float* pfCoef3 )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bGamma ) return;
#endif
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Gamma" );

	Asura_PS3_DeferredRendering::ResolveBackBuffer();

	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	RestoreViewport();

	g_pxFragmentShaderG = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_G );
	g_pxFragmentShaderG->SetFloats ( FPG_Coefficients0, 3, pfCoef0 );
	g_pxFragmentShaderG->SetFloats ( FPG_Coefficients1, 3, pfCoef1 );
	g_pxFragmentShaderG->SetFloats ( FPG_Coefficients2, 3, pfCoef2 );
	g_pxFragmentShaderG->SetFloats ( FPG_Coefficients3, 3, pfCoef3 );
	g_pxFragmentShaderG->SetTexture( FPG_InitialImage, Asura_PS3_DeferredRendering::GetResolvedTexture(), false );

	g_pxVertexShaderSimple->Bind();
	g_pxFragmentShaderG->Bind();
	g_xVertexBufferSimple.Bind( g_axVertexDeclaration, VD_ELEMENTS );
	g_xVertexBufferSimple.RenderQuads( 4 );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::Test()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::Test( const float fIntensity )
{

}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::SM30_Bloom()
//
////////////////////////////////////////////////////////////////////////////////

void Asura_PS3_FSFX::SM30_Bloom( const float fIntensity, float* afInputThreshold, float* afAdvBloom, bool m_bAccumulate, float m_iBlurPasses, float m_fBlurRadius, float m_fROutputCol, float m_fGOutputCol, float m_fBOutputCol )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bSM30_Bloom ) return;
#endif
	if( fIntensity == 0.0f ) return;
	float* pfVertexData = ( float* )g_xVertexBufferNoise.LockForWriting();
	pfVertexData[  0 ] = -1.0f;	pfVertexData[  1 ] =  1.0f; //Top left Pos
	pfVertexData[  2 ] =  0.0f;	pfVertexData[  3 ] =  1.0f; //Top left UV
	pfVertexData[  4 ] =  1.0f;	pfVertexData[  5 ] =  1.0f; //Top right Pos
	pfVertexData[  6 ] =  1.0f;	pfVertexData[  7 ] =  1.0f; //Top right UV
	pfVertexData[  8 ] =  1.0f;	pfVertexData[  9 ] = -1.0f; //Bottom right Pos
	pfVertexData[ 10 ] =  1.0f;	pfVertexData[ 11 ] =  0.0f; //Bottom right UV
	pfVertexData[ 12 ] = -1.0f;	pfVertexData[ 13 ] = -1.0f; //Bottom left Pos
	pfVertexData[ 14 ] =  0.0f;	pfVertexData[ 15 ] =  0.0f; //Bottom left UV
	g_xVertexBufferNoise.Unlock();

	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "SM30 Bloom" );


	bool bStencil = Asura_PS3_RenderingStates::GetStencilTestEnabled();
	Asura_PS3_RenderingStates::SetStencilTestEnabled(false);

	Asura_PS3_DeferredRendering::ResolveBackBuffer();
	PrepareRenderTarget();

	Asura_PS3_RenderingStates::RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	float afUVOffsets[ 4 ] = { 0.0f, 0.0f, 0.0f, 0.0f };

	g_pxVertexShaderB30->Bind();
	g_pxVertexShaderB30->SetUniformParameter( VPB30_UVOffset, afUVOffsets );

	u_int uHash = FSFX_B30_0100;

	if( m_bAccumulate )
	{
		uHash = FSFX_B30_0110;
	}
	const float afUVCorrection[] = { 1.0f / Asura_PS3_Graphics::GetScreenWidth(), 1.0f / Asura_PS3_Graphics::GetScreenHeight() };
	g_pxFragmentShaderB30 = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, uHash );
	g_pxFragmentShaderB30->SetFloats( FPB30_InputThreshold,	3, afInputThreshold );
	g_pxFragmentShaderB30->SetFloats( FPB30_Saturation,		1, &afAdvBloom[ 0 ] );
	g_pxFragmentShaderB30->SetFloats( FPB30_InputScale,		1, &afAdvBloom[ 1 ] );
	g_pxFragmentShaderB30->SetFloats( FPB30_AccumFade,		1, &afAdvBloom[ 2 ] );
	g_pxFragmentShaderB30->SetFloats( FPB30_UVCorrection,	2, afUVCorrection );
	{
		float afOutputColour[ 3 ] = { 1.0f, 1.0f, 1.0f };
		g_pxFragmentShaderB30->SetFloats( FPB30_OutputColour, 3, afOutputColour );
	}

	g_pxFragmentShaderB30->SetTexture( FPB30_Texture, Asura_PS3_DeferredRendering::GetResolvedTexture(), true );
	g_pxFragmentShaderB30->SetTexture( FPB30_AccumTexture, &g_xAdvBloomTexture, true );

	g_pxFragmentShaderB30->Bind();
	g_xVertexBufferNoise.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
	g_xVertexBufferNoise.RenderQuadsInline( 4, pfVertexData );

	Asura_PS3_DeferredRendering::GetTempFrameBuffer()->ResolveBackBuffer( &g_xAdvBloomTexture );
	PrepareRenderTarget();

	Asura_PS3_RenderingStates::RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	for( int iPass = 0; iPass < m_iBlurPasses; ++iPass )
	{
		for( int iDir = 0; iDir < 2; ++iDir )
		{
			PrepareRenderTarget();
			Asura_PS3_RenderingStates::RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
			Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
			Asura_PS3_RenderingStates::RequestAlphaTest( false );
			Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
			Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);
			g_pxFragmentShaderB30->Bind();
			g_xVertexBufferNoise.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
			if (iDir==0)
			{
				// Horiz blur
				float afHorizUVOffsets[ 4 ] = { 0.0025f * m_fBlurRadius, 0.0f, 0.0f, 0.0f };
				g_pxVertexShaderB30->SetUniformParameter( VPB30_UVOffset, afHorizUVOffsets );
			}
			else
			{
				// Vert blur
				float afVertUVOffsets[ 4 ] = { 0.0f, 0.0025f * m_fBlurRadius, 0.0f, 0.0f };
				g_pxVertexShaderB30->SetUniformParameter( VPB30_UVOffset, afVertUVOffsets );
			}

			uHash = FSFX_B30_1000;
			g_pxFragmentShaderB30 = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, uHash );

			float afOutputColour[ 3 ] = { 1.0f, 1.0f, 1.0f };
			float afFocalDistances[ 3 ] = { 1.0f, 1.0f, 1.0f };
			g_pxFragmentShaderB30->SetFloats( FPB30_OutputColour, 3, afOutputColour );
			g_pxFragmentShaderB30->SetFloats( FPB30_FocalDistances,	3, afFocalDistances );
			g_pxFragmentShaderB30->SetTexture( FPB30_Texture, &g_xAdvBloomTexture, true );

			// Draw it
			g_pxFragmentShaderB30->Bind();
			g_xVertexBufferNoise.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
			g_xVertexBufferNoise.RenderQuadsInline( 4, pfVertexData );

			// Remember to resolve it!
			Asura_PS3_DeferredRendering::GetTempFrameBuffer()->ResolveBackBuffer( &g_xAdvBloomTexture );
		}
	}

	RestoreViewport();
	Asura_PS3_RenderingStates::RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);

	Asura_PS3_RenderingStates::SetStencilTestEnabled(bStencil);

	afUVOffsets[ 0 ] = 0.0f;
	afUVOffsets[ 1 ] = 0.0f;
	float afOutputColour[ 3 ] = { m_fROutputCol * fIntensity, m_fGOutputCol * fIntensity, m_fBOutputCol * fIntensity };

	g_pxVertexShaderB30->SetUniformParameter( VPB30_UVOffset, afUVOffsets );

	uHash = FSFX_B30_0000;
	g_pxFragmentShaderB30 = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, uHash );
	
	g_pxFragmentShaderB30->SetFloats ( FPB30_OutputColour, 3, afOutputColour );
	g_pxFragmentShaderB30->SetTexture( FPB30_Texture, &g_xAdvBloomTexture, true );

	// Draw it
	g_pxFragmentShaderB30->Bind();
	g_xVertexBufferNoise.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
	g_xVertexBufferNoise.RenderQuadsInline( 4, pfVertexData );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}

void Asura_PS3_FSFX::ColourMap( const float fIntensity, int iTextureIndex )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bColourMap ) return;
#endif
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "ColourMap" );

	Asura_PS3_DeferredRendering::ResolveBackBuffer();

	float fInt = fIntensity;
	// Pixel shader will output fog colour with fog factor as alpha
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );

	// Taken these settings out of the effect, as they were screwing up the cached states
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	RestoreViewport();

	g_pxFragmentShaderCM = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_CM );

	g_pxFragmentShaderCM->SetFloats ( FPCM_Intensity, 1, &fInt );
	g_pxFragmentShaderCM->SetTexture( FPCM_InitialImage, Asura_PS3_DeferredRendering::GetResolvedTexture(), false );
	g_pxFragmentShaderCM->SetTexture( FPCM_ColourMap, iTextureIndex );

	g_pxVertexShaderSimple->Bind();
	g_pxFragmentShaderCM->Bind();
	g_xVertexBufferSimple.Bind( g_axVertexDeclaration, VD_ELEMENTS );
	g_xVertexBufferSimple.RenderQuads( 4 );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}

void Asura_PS3_FSFX::SM30_FishEye( float* pfData1, float* pfData2  )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bFishEye ) return;
#endif
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "SM30_FishEye" );

	const Asura_ViewingArea *pxViewingArea = Asura_Camera::GetViewingArea();
	const float fSqrt2 = 0.70710678118654752440084436210485f;	//1.0f / 1.4142135623730950488016887242097f; // 1.0f / sqrt(2)

	const int		iFishEyeType =  ( int )pfData2[ 0 ];	//	Eliminates any possibility of LoadHitStore

	float fPosLeft = -1.0f + pxViewingArea->m_fXOffset * 2.0f;
	float fPosRight = fPosLeft + pxViewingArea->m_fWidth * 2.0f;
	float fPosBottom = -1.0f + pxViewingArea->m_fYOffset * 2.0f;
	float fPosTop = fPosBottom + pxViewingArea->m_fHeight * 2.0f;
	float fUVLeft = -fSqrt2 + pxViewingArea->m_fXOffset * 2.0f * fSqrt2;
	float fUVRight = fUVLeft + pxViewingArea->m_fWidth * 2.0f * fSqrt2;
	float fUVBottom = -fSqrt2 + pxViewingArea->m_fYOffset * 2.0f * fSqrt2;
	float fUVTop = fUVBottom + pxViewingArea->m_fHeight * 2.0f * fSqrt2;
	float* pfVertexData = ( float* )g_xVertexBufferFishEye.LockForWriting();
	pfVertexData[ 0 ] = fPosLeft;	pfVertexData[ 1 ] = fPosTop;	pfVertexData[ 2 ] = fUVLeft;	pfVertexData[ 3 ] = fUVBottom;
	pfVertexData[ 4 ] = fPosRight;	pfVertexData[ 5 ] = fPosTop;	pfVertexData[ 6 ] = fUVRight;	pfVertexData[ 7 ] = fUVBottom;
	pfVertexData[ 8 ] = fPosRight;	pfVertexData[ 9 ] = fPosBottom;	pfVertexData[ 10] = fUVRight;	pfVertexData[ 11] = fUVTop;
	pfVertexData[ 12] = fPosLeft;	pfVertexData[ 13] = fPosBottom;	pfVertexData[ 14] = fUVLeft;	pfVertexData[ 15] = fUVTop;
	g_xVertexBufferFishEye.Unlock();

	Asura_PS3_DeferredRendering::ResolveBackBuffer();
	RestoreViewport();

	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	const float	fDataZOver2		= pfData1[2] / 2.0f;
	float		pfData3[ 4 ]	= { ( 1 / ( Asura_Maths::Tangent( pfData1[2] ) ) ), Asura_Maths::Sine( fDataZOver2 ), Asura_Maths::Tangent( fDataZOver2 ), 0 };

	g_pxFragmentShaderFE = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, ( FSFX_FE0 + iFishEyeType ) );

	g_pxFragmentShaderFE->SetFloats( FPFE_Data1, 4, pfData1 );
	g_pxFragmentShaderFE->SetFloats( FPFE_Data2, 4, pfData3 );
	g_pxFragmentShaderFE->SetTexture( FPFE_TextureSampler, Asura_PS3_DeferredRendering::GetResolvedTexture(), true );

	g_pxFragmentShaderFE->Bind();
	g_pxVertexShaderN->Bind();
	g_xVertexBufferFishEye.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
	g_xVertexBufferFishEye.RenderQuads( 4 );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}

////////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX::SM30_DoubleVision()
//
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_FSFX::SM30_DoubleVision( float* pfData1, float* pfData2  )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bDoubleVision ) return;
#endif
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "SM30_DoubleVision" );


	const float fPosLeft = -1.0f;
	const float fPosRight = 1.0f;
	const float fPosTop = -1.0f;
	const float fPosBottom = 1.0f;
	const float fUVLeft = pfData2[0];
	const float fUVRight = pfData2[1];
	const float fUVTop = pfData2[3];
	const float fUVBottom = pfData2[2];
	float* pfVertexData = ( float* )g_xVertexBufferDoubleVision.LockForWriting();
	pfVertexData[ 0 ] = fPosLeft;	pfVertexData[ 1 ] = fPosTop;	pfVertexData[ 2 ] = fUVLeft;	pfVertexData[ 3 ] = fUVBottom;
	pfVertexData[ 4 ] = fPosRight;	pfVertexData[ 5 ] = fPosTop;	pfVertexData[ 6 ] = fUVRight;	pfVertexData[ 7 ] = fUVBottom;
	pfVertexData[ 8 ] = fPosRight;	pfVertexData[ 9 ] = fPosBottom;	pfVertexData[ 10] = fUVRight;	pfVertexData[ 11] = fUVTop;
	pfVertexData[ 12] = fPosLeft;	pfVertexData[ 13] = fPosBottom;	pfVertexData[ 14] = fUVLeft;	pfVertexData[ 15] = fUVTop;
	g_xVertexBufferDoubleVision.Unlock();

	Asura_PS3_DeferredRendering::ResolveBackBuffer();
	RestoreViewport();

	g_pxFragmentShaderDV = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_DV );

	g_pxFragmentShaderDV->Bind();
	g_pxVertexShaderN->Bind();

	g_pxFragmentShaderDV->SetFloats ( FPDV_Data1, 4, pfData1 );
	g_pxFragmentShaderDV->SetTexture( FPDV_TextureSampler, Asura_PS3_DeferredRendering::GetResolvedTexture(), true );

	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( false );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_NONE );

	g_xVertexBufferDoubleVision.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
	g_xVertexBufferDoubleVision.RenderQuads( 4 );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}

Asura_PS3_Texture* Asura_PS3_FSFX::GetBlurredBuffer( void )
{
	return &g_xAdvBloomTempTexture;
}

void Asura_PS3_FSFX::UpdateBlurredBuffer( int iBlurPasses, float fBlurRadius, float fSaturation, bool bDepthOfFieldMode,  float fDist0, float fDist1, float fBleedScale )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "UpdateBlurredBuffer" );

	const float fViewWidth		= static_cast<float> ( Asura_Viewport::GetWidth());
	const float fViewHeight		= static_cast<float> ( Asura_Viewport::GetHeight());
//	const float fBufferWidth	= fViewWidth / 2.0f;
//	const float fBufferHeight	= fViewHeight / 2.0f;
	float fHalfTexelU = 0;//0.5f/fViewWidth;  //BW On PS3 the sub-pixel position doesn't need adjusting (unlike xbox360)
	float fHalfTexelV = 0;//0.5f/fViewHeight; //BW this is because the interpolated pixel positions correspond to the centre of the pixel (at .5,.5) so if you read UV from there you get a nice value

	float* pfVertexData = ( float* )g_xVertexBufferNoise.LockForWriting();
	pfVertexData[  0 ] = -1.0f;	pfVertexData[  1 ] =  1.0f; //Top left Pos
	pfVertexData[  2 ] =  0.0f+fHalfTexelU;	pfVertexData[  3 ] =  1.0f+fHalfTexelV; //Top left UV
	pfVertexData[  4 ] =  1.0f;	pfVertexData[  5 ] =  1.0f; //Top right Pos
	pfVertexData[  6 ] =  1.0f+fHalfTexelU;	pfVertexData[  7 ] =  1.0f+fHalfTexelV; //Top right UV
	pfVertexData[  8 ] =  1.0f;	pfVertexData[  9 ] = -1.0f; //Bottom right Pos
	pfVertexData[ 10 ] =  1.0f+fHalfTexelU;	pfVertexData[ 11 ] =  0.0f+fHalfTexelV; //Bottom right UV
	pfVertexData[ 12 ] = -1.0f;	pfVertexData[ 13 ] = -1.0f; //Bottom left Pos
	pfVertexData[ 14 ] =  0.0f+fHalfTexelU;	pfVertexData[ 15 ] =  0.0f+fHalfTexelV; //Bottom left UV
	g_xVertexBufferNoise.Unlock();

	Asura_PS3_DeferredRendering::ResolveBackBuffer();
	PrepareRenderTarget();

	Asura_PS3_RenderingStates::RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	float afUVOffsets[ 4 ] = { 0.0f, 0.0f, 0.0f, 0.0f };

	g_pxVertexShaderB30->Bind();
	g_pxVertexShaderB30->SetUniformParameter( VPB30_UVOffset, afUVOffsets );

	u_int uHash;

	if( fSaturation < 1.0f )
	{
		if( bDepthOfFieldMode )
		{
			uHash = FSFX_B30_0101;
		}
		else
		{
			uHash = FSFX_B30_0100;
		}
	}
	else
	{
		if( bDepthOfFieldMode )
		{
			uHash = FSFX_B30_0001;
		}
		else
		{
			uHash = FSFX_B30_0000;
		}
	}

	float afOutputColour[ 3 ]	= { 1.0f, 1.0f, 1.0f };
	float afInputThreshold[ 3 ]	= { 0.0f, 0.0f, 0.0f };
	float afAdvBloom[ 3 ]		= { fSaturation, 1.0f, 0.0f };
	float afFocalDistances[ 3 ] = { fDist0, fDist1, fBleedScale };
	const float afUVCorrection[] = { 0.5f / Asura_PS3_Graphics::GetScreenWidth(), 0.5f / Asura_PS3_Graphics::GetScreenHeight() };
	g_pxFragmentShaderB30 = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, uHash );
	g_pxFragmentShaderB30->SetFloats( FPB30_OutputColour,	4, afOutputColour );
	g_pxFragmentShaderB30->SetFloats( FPB30_InputThreshold,	4, afInputThreshold );
	g_pxFragmentShaderB30->SetFloats( FPB30_Saturation,		1, &afAdvBloom[ 0 ] );
	g_pxFragmentShaderB30->SetFloats( FPB30_InputScale,		1, &afAdvBloom[ 1 ] );
	g_pxFragmentShaderB30->SetFloats( FPB30_AccumFade,		1, &afAdvBloom[ 2 ] );
	g_pxFragmentShaderB30->SetFloats( FPB30_UVCorrection,	2, afUVCorrection );
	if( s_bDepthOfField )
	{
		Asura_PS3_MaterialResponse_System::SetDepthParameters( g_pxFragmentShaderB30, FPB30_DepthAdjustment );
		g_pxFragmentShaderB30->SetTexture( FPB30_DepthTexture, Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->GetZBufferAsTexture(), false );
		g_pxFragmentShaderB30->SetFloats( FPB30_FocalDistances,	4, afFocalDistances );
	}

	g_pxFragmentShaderB30->SetTexture( FPB30_Texture, Asura_PS3_DeferredRendering::GetResolvedTexture(), true );

	g_pxFragmentShaderB30->Bind();

	// Update the UVs
	fHalfTexelU = 0;//0.5f/fBufferWidth;
	fHalfTexelV = 0;//0.5f/fBufferHeight;

	pfVertexData = ( float* )g_xVertexBufferNoise.LockForWriting();
	pfVertexData[  2 ] =  0.0f+fHalfTexelU;	pfVertexData[  3 ] =  1.0f+fHalfTexelV; //Top left UV
	pfVertexData[  6 ] =  1.0f+fHalfTexelU;	pfVertexData[  7 ] =  1.0f+fHalfTexelV; //Top right UV
	pfVertexData[ 10 ] =  1.0f+fHalfTexelU;	pfVertexData[ 11 ] =  0.0f+fHalfTexelV; //Bottom right UV
	pfVertexData[ 14 ] =  0.0f+fHalfTexelU;	pfVertexData[ 15 ] =  0.0f+fHalfTexelV; //Bottom left UV
	g_xVertexBufferNoise.Unlock();

	g_xVertexBufferNoise.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
	g_xVertexBufferNoise.RenderQuadsInline( 4, pfVertexData );

	Asura_PS3_DeferredRendering::GetTempFrameBuffer()->ResolveBackBuffer( &g_xAdvBloomTempTexture );
	PrepareRenderTarget();
	Asura_PS3_RenderingStates::RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	for( int iPass = 0; iPass < iBlurPasses; ++iPass )
	{
		for( int iDir = 0; iDir < 2; ++iDir )
		{
			PrepareRenderTarget();
			Asura_PS3_RenderingStates::RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
			Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
			Asura_PS3_RenderingStates::RequestAlphaTest( false );
			Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
			Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);
			g_pxFragmentShaderB30->Bind();
			g_xVertexBufferNoise.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
			if (iDir==0)
			{
				// Horiz blur
				float afHorizUVOffsets[ 4 ] = { 0.0025f * fBlurRadius, 0.0f, 0.0f, 0.0f };
				g_pxVertexShaderB30->SetUniformParameter( VPB30_UVOffset, afHorizUVOffsets );
			}
			else
			{
				// Vert blur
				float afVertUVOffsets[ 4 ] = { 0.0f, 0.0025f * fBlurRadius, 0.0f, 0.0f };
				g_pxVertexShaderB30->SetUniformParameter( VPB30_UVOffset, afVertUVOffsets );
			}

			uHash = FSFX_B30_1000;
			if( bDepthOfFieldMode )
			{
				uHash = FSFX_B30_1001;
			}

			g_pxFragmentShaderB30 = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, uHash );

			g_pxFragmentShaderB30->SetFloats( FPB30_FocalDistances,	4, afFocalDistances );

			Asura_PS3_MaterialResponse_System::SetDepthParameters( g_pxFragmentShaderB30, FPB30_DepthAdjustment );
			g_pxFragmentShaderB30->SetTexture( FPB30_DepthTexture, Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->GetZBufferAsTexture(), false );

			float afFragmentShaderOutputColour[ 4 ] = { 1.0f, 1.0f, 1.0f, 1.0f };
			g_pxFragmentShaderB30->SetFloats( FPB30_OutputColour, 4, afFragmentShaderOutputColour );
			g_pxFragmentShaderB30->SetTexture( FPB30_Texture, &g_xAdvBloomTempTexture, true );

			// Draw it
			g_pxFragmentShaderB30->Bind();
			g_xVertexBufferNoise.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
			g_xVertexBufferNoise.RenderQuadsInline( 4, pfVertexData );

			// Remember to resolve it!
			Asura_PS3_DeferredRendering::GetTempFrameBuffer()->ResolveBackBuffer( &g_xAdvBloomTempTexture );
		}
	}

	RestoreViewport();
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}

void Asura_PS3_FSFX::SM30_DepthOfField( const float fIntensity, int iBlurPasses, float fBlurRadius, float fBlurDesaturate, float* afDistances, float fBlendScale )
{
#if defined( ASURA_SUPPORT_COMMAND_CONSOLE )
	if( !s_bDepthOfField ) return;
#endif
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "SM30_DepthOfField" );

	bool bStencil = Asura_PS3_RenderingStates::GetStencilTestEnabled();
	Asura_PS3_RenderingStates::SetStencilTestEnabled(false);

	Asura_PS3_FSFX::UpdateBlurredBuffer( iBlurPasses, fBlurRadius, fBlurDesaturate, true, afDistances[ 0 ], afDistances [ 1 ], fBlendScale );
	Asura_PS3_RenderingStates::SetAlphaBlending(true);
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	RestoreViewport();

	g_pxFragmentShaderDOF30 = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_FSFX, FSFX_DOF30 );
	float fEffectStrength = fIntensity;
	
	Asura_PS3_MaterialResponse_System::SetDepthParameters( g_pxFragmentShaderDOF30, FPDOF30_DepthAdjustment );
	g_pxFragmentShaderDOF30->SetFloats ( FPDOF30_FocalDistances, 4, afDistances );
	g_pxFragmentShaderDOF30->SetFloats ( FPDOF30_EffectStrength, 1, &fEffectStrength );
	g_pxFragmentShaderDOF30->SetTexture( FPDOF30_BlurredTexture, Asura_PS3_FSFX::GetBlurredBuffer(), false );
	g_pxFragmentShaderDOF30->SetTexture( FPDOF30_DepthTexture, Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->GetZBufferAsTexture(), false );

	Asura_PS3_RenderingStates::SetStencilTestEnabled(bStencil);

	g_pxVertexShaderSimple->Bind();
	g_pxFragmentShaderDOF30->Bind();
	g_xVertexBufferSimple.Bind( g_axVertexDeclaration, VD_ELEMENTS );
	g_xVertexBufferSimple.RenderQuads( 4 );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );

}

#endif // defined ASURA_USE_FSFX
