//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_FSFX.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined ASURA_USE_FSFX

#include "FSFX/Asura_PS3_FSFX.h"

// Asura Platform-independent
#include "Core/Asura_Timers.h"			// For Double Vision effect
#include "FSFX/Asura_FSFX_Component.h"
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Camera/Asura_Camera.h"

// Asura PS3
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_GCM_VertexBuffer.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"
#include "Rendering/Asura_PS3_TextureManagement.h"


int Asura_FSFX_Template::Platform_GetPlatformFlag()
{
	return Asura_FSFX_Component::FSFX_PLATFORM_PS3;
}

void Asura_FSFX_System::Platform_Initialise()	{ Asura_PS3_FSFX::Initialise();	}
void Asura_FSFX_System::Platform_Reset()		{ Asura_PS3_FSFX::Reset();		}
void Asura_FSFX_System::Platform_Shutdown()		{ Asura_PS3_FSFX::Shutdown();	}
void Asura_FSFX_System::Platform_BeginRender()	{}
void Asura_FSFX_System::Platform_EndRender()	{ Asura_PS3_RenderingStates::SetStencilTestEnabled(false); }

void Asura_FSFX_Component_BrightnessAndContrast::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: Brightness & Contrast" );

	const float fBrightness = fIntensity * this->m_fBrightness / 100.0f;
	const float fScale = (fIntensity * this->m_fContrast / 100.0f) + 1.0f;

	const float fTerm0 = Asura_Maths::ClampToRange( fBrightness  + (0.5f - (0.5f * fScale)), 0.0f, 2.0f);
	const float fTerm1 = Asura_Maths::ClampToRange(fScale, 0.0f, 2.0f);



	float afCoef[ 2 ] = { fTerm0, fTerm1 };

	Asura_PS3_FSFX::BrightnessAndContrast( afCoef );

    ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_DesaturateAndTint::Platform_Render( const float fIntensity )
{
    ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: Desaturate & Tint" );

	float afTintStrength[ 4 ] = { m_fIntensityR / 100.0f, m_fIntensityG / 100.0f, m_fIntensityB / 100.0f, m_fStrength * fIntensity / 100.0f };

	Asura_PS3_FSFX::DesaturateAndTint( afTintStrength );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_Noise::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: Noise" );

	float fUV[ 4 ];
	float fEffectStrength = 0.5f * fIntensity * m_fAmount / 100.0f;

	fUV[ 0 ] = Asura_Random::GetFloat( 0.0f, 1.0f );
	fUV[ 1 ] = Asura_Random::GetFloat( 0.0f, 1.0f );
	fUV[ 2 ] = (Asura_Random::GetFloat( 0.0f, 1.0f ) <= 0.5f ? 1.0f : -1.0f) * static_cast< float >( m_uTextureTileX );
	fUV[ 3 ] = (Asura_Random::GetFloat( 0.0f, 1.0f ) <= 0.5f ? 1.0f : -1.0f) * static_cast< float >( m_uTextureTileY );

	Asura_PS3_FSFX::Noise( fUV, m_iTextureID, &fEffectStrength );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_GaussianBlur::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: Gaussian Blur" );

	Asura_PS3_FSFX::GaussianBlur( fIntensity );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_RadialBlur::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: Radial Blur" );

	float afSampleWeights[4];

	afSampleWeights[0] = 1.0f - 0.75f * fIntensity * m_fStrength / 100.0f;
	afSampleWeights[3] = 0.25f * (1.0f - afSampleWeights[0]);
	afSampleWeights[1] = 0.5f * (1.0f - afSampleWeights[0] - afSampleWeights[3]);
	afSampleWeights[2] = afSampleWeights[1];

	float fBlurBufferHeight = ( float )Asura_PS3_Graphics::GetScreenHeight();

	Asura_AMatrix4x4 axTransforms[ 4 ];

	// First transform is a simple scale and bias from [-1,1] space into [0,1]
	//Translate and scale 0.5, 0.5
	axTransforms[ 0 ].SetColumn0(Asura_AVector4( 0.5f, 0.0f, 0.0f, 0.0f ) );
	axTransforms[ 0 ].SetColumn1(Asura_AVector4( 0.0f, 0.5f, 0.0f, 0.0f ) );
	axTransforms[ 0 ].SetColumn2(Asura_AVector4( 0.0f, 0.0f, 1.0f, 0.0f ) );
	axTransforms[ 0 ].SetColumn3(Asura_AVector4( 0.5f, 0.5f, 0.0f, 1.0f ) );

	const float fSize = m_fSize / 4.0f;
	const float fCentreX = m_fCentreX / 100.0f;
	const float fCentreY = m_fCentreY / 100.0f;
	const float afScaleFactors[ 3 ] = { 1.0f, 3.0f, 5.0f };

	for( u_int u = 1; u < 4; ++u )
	{
		const float xScaleX[ 2 ]		= { 1.0f - afScaleFactors[ u - 1 ] * fSize / fBlurBufferHeight, 1.0f };
		const float xTranslateX[ 2 ]	= { ( 1.0f - xScaleX[ 0 ] ) * fCentreX, 0.0f };
		axTransforms[ u ].SetColumn0(Asura_AVector4( xScaleX[ 0 ], 0.0f, 0.0f, 0.0f ) );
		axTransforms[ u ].SetColumn1(Asura_AVector4( 0.0f, xScaleX[ 1 ], 0.0f, 0.0f ) );
		axTransforms[ u ].SetColumn2(Asura_AVector4( 0.0f, 0.0f, 1.0f, 0.0f ) );
		axTransforms[ u ].SetColumn3(Asura_AVector4( xTranslateX[ 0 ], xTranslateX[ 1 ], 0.0f, 1.0f ) );

		axTransforms[ u ] = axTransforms[ u ] * axTransforms[ 0 ];
	}

	Asura_PS3_FSFX::RadialBlur( afSampleWeights, axTransforms[0].GetElements() );

	for( u_int u = 1; u < 4; ++u )
	{
		const float xScaleY[ 2 ]		= { 1.0f, 1.0f - afScaleFactors[ u - 1 ] * fSize / fBlurBufferHeight };
		const float xTranslateY[ 2 ]	= { 0.0f, ( 1.0f - xScaleY[ 1 ] ) * fCentreY };
		axTransforms[ u ].SetColumn0(Asura_AVector4( xScaleY[ 0 ], 0.0f, 0.0f, 0.0f ) );
		axTransforms[ u ].SetColumn1(Asura_AVector4( 0.0f, xScaleY[ 1 ], 0.0f, 0.0f ) );
		axTransforms[ u ].SetColumn2(Asura_AVector4( 0.0f, 0.0f, 1.0f, 0.0f ) );
		axTransforms[ u ].SetColumn3(Asura_AVector4( xTranslateY[ 0 ], xTranslateY[ 1 ], 0.0f, 1.0f ) );

		axTransforms[ u ] = axTransforms[ u ] * axTransforms[ 0 ];
	}

	Asura_PS3_FSFX::RadialBlur( afSampleWeights, axTransforms[0].GetElements() );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_DepthOfField::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: Depth of Field" );

	Asura_PS3_FSFX::DepthOfField( fIntensity );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_Bloom::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: Bloom" );

	const float afTint[3] =
	{
		m_fIntensityR * fIntensity / 100.0f,
		m_fIntensityG * fIntensity / 100.0f,
		m_fIntensityB * fIntensity / 100.0f,
	};

	Asura_PS3_FSFX::Bloom( fIntensity, afTint, m_fThreshold );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}


static void RenderGamma( const float fR, const float fG, const float fB )
{
	const float fTangentDelta = 0.1f;

	const Asura_Vector_3 xy0( 0.0f, 0.0f, 0.0f);
	const Asura_Vector_3 xy1( pow( fTangentDelta, 1.0f / fR ),		pow( fTangentDelta, 1.0f / fG ),		pow( fTangentDelta, 1.0f / fB ));
	const Asura_Vector_3 xy2( pow( 1.0f-fTangentDelta, 1.0f / fR ),	pow( 1.0f-fTangentDelta, 1.0f / fG ),	pow( 1.0f-fTangentDelta, 1.0f / fB ));
	const Asura_Vector_3 xy3( 1.0f, 1.0f, 1.0f);

	// DLH - 23 February 2006 12:38:16 - Cubic Hermit Spline
	const Asura_Vector_3 xp0 = xy0;
	const Asura_Vector_3 xp1 = xy3;
	const Asura_Vector_3 xm0 = (xy1 - xy0) / fTangentDelta;
	const Asura_Vector_3 xm1 = (xy3 - xy2) / fTangentDelta;
	const Asura_Vector_3 xt3 = (xp0 * 2.0f) + (xm0 * 1.0f) + (xp1 * -2.0f) + (xm1 * 1.0f);
	const Asura_Vector_3 xt2 = (xp0 * -3.0f) + (xm0 * -2.0f) + (xp1 * +3.0f) + (xm1 * -1.0f);
	const Asura_Vector_3 xt1 = (xm0);
	const Asura_Vector_3 xt0 = (xp0);

	float afCoef0[ 3 ] = { xt0.x, xt0.y, xt0.z };
	float afCoef1[ 3 ] = { xt1.x, xt1.y, xt1.z };
	float afCoef2[ 3 ] = { xt2.x, xt2.y, xt2.z };
	float afCoef3[ 3 ] = { xt3.x, xt3.y, xt3.z };

	Asura_PS3_FSFX::Gamma( afCoef0, afCoef1, afCoef2, afCoef3 );
}

void Asura_FSFX_Component_Gamma::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: Gamma" );

	RenderGamma( 1.0f - fIntensity + fIntensity * m_fAmount,
		1.0f - fIntensity + fIntensity * m_fAmount,
		1.0f - fIntensity + fIntensity * m_fAmount );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_RGBGamma::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: RGB Gamma" );

	RenderGamma( 1.0f - fIntensity + fIntensity * m_fAmountR,
		1.0f - fIntensity + fIntensity * m_fAmountG,
		1.0f - fIntensity + fIntensity * m_fAmountB );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_Test::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: Test" );

	Asura_PS3_FSFX::Test( fIntensity );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_SM30_Bloom::Platform_Render( const float fIntensity )
{
	if ( fIntensity == 0.0f )
	{
		return;
	}

	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: SM30 Bloom" );

	float afInputThreshold[ 3 ] = { m_fRBloomThreshold, m_fGBloomThreshold, m_fBBloomThreshold };
	float afAdvBloom[ 3 ] = { m_fSaturation, m_fInputScale, 1.0f - m_fAccumFade };

	Asura_PS3_FSFX::SM30_Bloom( fIntensity, afInputThreshold, afAdvBloom, m_bAccumulate, m_iBlurPasses, m_fBlurRadius, m_fROutputCol, m_fGOutputCol, m_fBOutputCol );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_ColourMap::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: Colour Map" );

	Asura_PS3_FSFX::ColourMap( fIntensity, m_iColourMapIndex );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_SM30_DepthOfField::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: SM30 Depth of Field" );

	Asura_PS3_FSFX::SM30_DepthOfField( fIntensity, m_iBlurPasses, m_fBlurRadius, 1.0f - m_fBlurDesaturate, m_afDistances, m_fBleedScale );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_SM30_FishEye::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: SM30 Fish Eye" );

	const float fWidth = static_cast< float >( Asura_Viewport::GetWidth() );
	const float fHeight = static_cast< float >( Asura_Viewport::GetHeight() );
	float fScaleX, fScaleY;

	if (fWidth > fHeight)
	{
		fScaleX = m_fXScale;
		fScaleY = m_fYScale * fHeight / fWidth;
	}
	else
	{
		fScaleX = m_fXScale * fWidth / fHeight;
		fScaleY = m_fYScale;
	}

	float fDynamicIntensity = fIntensity;
	if ( m_uFlags & DynamicSine )
	{
		// Calculate sine value and clip
		const float fTime = Asura_Timers::GetLocalScaledTime();
		const float fSineClipped = Asura_Maths::Max( m_fSineClip, Asura_Maths::Sine( fTime * m_fSpeed ) / 2.0f + 0.5f );

		// Work out max value after clipping
		float fScaleValue = 1.0f;
		if ( m_fSineClip != 1.0f )
		{
			fScaleValue = 1.0f / ( 1.0f - m_fSineClip );
		}

		// Normalise sine back up to fill 0 to 1 range
		const float fParameter = ( fSineClipped - m_fSineClip ) * fScaleValue;

		// Lerp and multiply by intensity
		fDynamicIntensity *= Asura_Maths::Lerp( m_fMinMultiplier, m_fMaxMultiplier, fParameter );
	}

	float faData1[ 4 ]= { fScaleX, fScaleY, m_fFov * 0.5f, m_fMultiplier * fDynamicIntensity }, faData2[ 2 ] = { (float)m_eType, 1.0f };

	Asura_PS3_FSFX::SM30_FishEye( faData1, faData2 );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_SM30_FishEye::Platform_Initialise()
{
}
void Asura_FSFX_Component_SM30_FishEye::Platform_Reset()
{
}
void Asura_FSFX_Component_SM30_FishEye::Platform_Shutdown()
{
}

void Asura_FSFX_Component_SM30_DoubleVision::Platform_Render( const float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: SM30 Double Vision" );

	const float fTime = Asura_Timers::GetLocalScaledTime();

	// Interpolate from zero to set parameters
	const float fXOffset1 = m_fOrigXOffset1 * fIntensity;
	const float fYOffset1 = m_fOrigYOffset1 * fIntensity;
	const float fXScale1 = m_fOrigXScale1 * fIntensity;
	const float fYScale1 = m_fOrigYScale1 * fIntensity;
	const float fXOffset2 = m_fOrigXOffset2 * fIntensity;
	const float fYOffset2 = m_fOrigYOffset2 * fIntensity;
	const float fXScale2 = m_fOrigXScale2 * fIntensity;
	const float fYScale2 = m_fOrigYScale2 * fIntensity;

	const float fXMultiplier = 1.0f / 100.0f;
	const float fYMultiplier = 1.0f / 100.0f;

	// Apply phase and speed scale to time
	const float fTime1 = fTime * m_fSpeed1 + m_fPhase1;
	const float fTime2 = fTime * m_fSpeed2 + m_fPhase2;

	float fXDist1;
	float fYDist1;
	float fXDist2;
	float fYDist2;

	Asura_Maths::SinCos( fTime1, fXDist1, fYDist1 );
	Asura_Maths::SinCos( fTime2, fXDist2, fYDist2 );
	fXDist1 *= fXScale1;
	fYDist1 *= fYScale1;
	fXDist2 *= fXScale2;
	fYDist2 *= fYScale2;

	const float fXTotalOffset1 = (fXOffset1 + fXDist1) * fXMultiplier;
	const float fYTotalOffset1 = (fYOffset1 + fYDist1) * fYMultiplier;
	const float fXTotalOffset2 = (fXOffset2 + fXDist2) * fXMultiplier;
	const float fYTotalOffset2 = (fYOffset2 + fYDist2) * fYMultiplier;

	const float fXMaxOffset = Asura_Maths::Max( Asura_Maths::Abs(fXOffset1) + Asura_Maths::Abs(fXDist1) , Asura_Maths::Abs(fXOffset2) + Asura_Maths::Abs(fXDist2) ) * fXMultiplier;
	const float fYMaxOffset = Asura_Maths::Max( Asura_Maths::Abs(fYOffset1) + Asura_Maths::Abs(fYDist1) , Asura_Maths::Abs(fYOffset2) + Asura_Maths::Abs(fYDist2) ) * fYMultiplier;
	const float fXUVScale = 1.0f / (1.0f + fXMaxOffset );
	const float fYUVScale = 1.0f / (1.0f + fYMaxOffset );

	const float fUVLeft = 1.0f - fXUVScale;
	const float fUVRight = fXUVScale;
	const float fUVTop = 1.0f - fYUVScale;
	const float fUVBottom = fYUVScale;

	// Create float vectors to pass to the shader
	float afData1[ 4 ]= { fXTotalOffset1, fYTotalOffset1, fXTotalOffset2, fYTotalOffset2 };
	float afData2[ 4 ] = { fUVLeft, fUVRight, fUVTop, fUVBottom };

	Asura_PS3_FSFX::SM30_DoubleVision( afData1, afData2 );

	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_SM30_DoubleVision::Platform_Initialise()
{
}
void Asura_FSFX_Component_SM30_DoubleVision::Platform_Reset()
{
}
void Asura_FSFX_Component_SM30_DoubleVision::Platform_Shutdown()
{
}

void Asura_FSFX_Component_SM30_MotionBlur::Platform_Render( const float fIntensity )
{
	//ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__FSFX, "FSFX: SM30 Motion Blur" );
	//
	//Asura_PS3_FSFX::SM30_MotionBlur( fIntensity );
	//
	//ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__FSFX );
}

void Asura_FSFX_Component_SM30_MotionBlur::Platform_Initialise()
{
}

void Asura_FSFX_Component_SM30_MotionBlur::Platform_Reset()
{
}

void Asura_FSFX_Component_SM30_MotionBlur::Platform_Shutdown()
{
}

#endif // defined ASURA_USE_FSFX
