// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_Platform_GUIMenuInput.cpp												//
//																						//
//	Description: this handles the platform specific menu control system					//
//																						//
//	Created: Damian Furlong - 01/05/07														//
//																						//
//	Copyright (C) 2007 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"
#include "Input/Asura_ControlMap.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_PS3_RenderingStates.h"

#include "Input/Asura_PlatformInput.h"


#define ASURA_GUIMENU_MAX_CONTROLLERS	1

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_InitInputMappings( void)
{
	if( s_pxMenuControlMaps == NULL)
	{
		s_pxMenuControlMaps = new Asura_ControlMap[ASURA_GUIMENU_MAX_CONTROLLERS];

		s_pxMenuControlMaps[0].CreateMappings( ASURA_GUIMENU_CONTROL_MAX);

#ifdef ASURA_USE_NEW_INPUT
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_UP, ASURA_PS3_KEYID_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PS3_KEYID_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PS3_KEYID_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PS3_KEYID_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_AXIS_H, ASURA_PS3_AXISID_LSTICK_H, Asura_ControlMapping::ON_PRESS, 1.0f );
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_AXIS_V, ASURA_PS3_AXISID_LSTICK_V, Asura_ControlMapping::ON_PRESS, -1.0f );

		int iAssignment = CELL_SYSUTIL_ENTER_BUTTON_ASSIGN_CROSS;
#ifdef ASURA_ASSERT
		const int iReturnValue = 
#endif
		cellSysutilGetSystemParamInt( CELL_SYSUTIL_SYSTEMPARAM_ID_ENTER_BUTTON_ASSIGN, &iAssignment );
		Asura_Assert(iReturnValue == CELL_OK, "PS3 enter button assignment lookup failed" );
		if (iAssignment == CELL_SYSUTIL_ENTER_BUTTON_ASSIGN_CROSS)
		{
			s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_PS3_KEYID_CROSS, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_PS3_KEYID_CROSS, Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_PS3_KEYID_CIRCLE, Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_PS3_KEYID_CIRCLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		}
		else
		{
			s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_PS3_KEYID_CIRCLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_PS3_KEYID_CIRCLE, Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_PS3_KEYID_CROSS, Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_PS3_KEYID_CROSS, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		}

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, ASURA_PS3_KEYID_SQUARE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, ASURA_PS3_KEYID_TRIANGLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, ASURA_PS3_KEYID_R3, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4, ASURA_PS3_KEYID_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, ASURA_PS3_KEYID_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6, ASURA_PS3_KEYID_L1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7, ASURA_PS3_KEYID_R1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8, ASURA_PS3_KEYID_L2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_PS3_KEYID_R2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_PS3_KEYID_SQUARE, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_PS3_KEYID_TRIANGLE, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD, ASURA_PS3_KEYID_R3, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD, ASURA_PS3_KEYID_SELECT, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, ASURA_PS3_KEYID_PAD1_START, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD, ASURA_PS3_KEYID_L1, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD, ASURA_PS3_KEYID_R1, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD, ASURA_PS3_KEYID_L2, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD, ASURA_PS3_KEYID_R2, Asura_ControlMapping::ON_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PS3_KEYID_UP, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PS3_KEYID_DOWN, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PS3_KEYID_LEFT, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PS3_KEYID_RIGHT, Asura_ControlMapping::ON_PRESS);

#else // #ifdef ASURA_USE_NEW_INPUT
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_UP, ASURA_PS3_KEYID_PAD1_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PS3_KEYID_PAD1_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PS3_KEYID_PAD1_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PS3_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_PS3_KEYID_PAD1_CROSS, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, ASURA_PS3_KEYID_PAD1_SQUARE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, ASURA_PS3_KEYID_PAD1_TRIANGLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, ASURA_PS3_KEYID_PAD1_R3, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4, ASURA_PS3_KEYID_PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, ASURA_PS3_KEYID_PAD1_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6, ASURA_PS3_KEYID_PAD1_L1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7, ASURA_PS3_KEYID_PAD1_R1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8, ASURA_PS3_KEYID_PAD1_L2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_PS3_KEYID_PAD1_R2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_PS3_KEYID_PAD1_CIRCLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PS3_KEYID_PAD1_UP, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PS3_KEYID_PAD1_DOWN, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PS3_KEYID_PAD1_LEFT, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PS3_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_PS3_KEYID_PAD1_CROSS, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_PS3_KEYID_PAD1_SQUARE, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_PS3_KEYID_PAD1_TRIANGLE, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD, ASURA_PS3_KEYID_PAD1_R3, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD, ASURA_PS3_KEYID_PAD1_SELECT, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, ASURA_PS3_KEYID_PAD1_START, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD, ASURA_PS3_KEYID_PAD1_L1, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD, ASURA_PS3_KEYID_PAD1_R1, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD, ASURA_PS3_KEYID_PAD1_L2, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD, ASURA_PS3_KEYID_PAD1_R2, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_PS3_KEYID_PAD1_CIRCLE, Asura_ControlMapping::ON_PRESS);
#endif // #ifdef ASURA_USE_NEW_INPUT


#ifdef ASURA_USE_CONTROLMAP_MANAGER
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PS3_PADBUTTONOFFSET_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
#endif 
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_ShutdownInputMappings( void)
{
	u_int u;

	if( s_pxMenuControlMaps)
	{
		for( u=0; u<ASURA_GUIMENU_MAX_CONTROLLERS; u++)
		{
			s_pxMenuControlMaps[u].DeleteMappings();
		}

		delete []s_pxMenuControlMaps;
		s_pxMenuControlMaps = NULL;
	}
}

u_int Asura_GUIMenu_System::Platform_GetNumControlMaps( void)
{
	return ASURA_GUIMENU_MAX_CONTROLLERS;
}

bool Asura_GUIMenu_System::Platform_ShouldUpdateControls()
{
	return true;
}

bool Asura_GUIMenu_System::Platform_ShouldRenderMenuPages()
{
	// on PSP we do some fancy stuff when the system utility is open to 
	// switch to a special background menu page. For now just clear to 
	// black on PS3 if a utility menu is open and intercepting pad input.
	if (Asura_PS3_Input::IsInputIntercepted())
	{
		Asura_Render::Overlay.Colour           = 0xff000000;
		Asura_Render::Overlay.TextureIndex     = ASURA_TEXTURE_NONE;
		Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
		Asura_Render::Overlay.SetRectXY( 0.0f, 0.0f, static_cast< float >( Asura_Viewport::GetWidth() ), 
			static_cast< float >( Asura_Viewport::GetHeight() ) );
		Asura_Render::Overlay.Render();
		Asura_Render::FlushDisplay();
		return false;
	}
	return true;
}

// --------------------------------------------------------------------------------------- 
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
void Asura_GUIMenu_System::Platform_RenderSafetyBorder( void)
{
	if( s_bRenderSafetyBorder)
	{
		float fScreenWidth, fScreenHeight;
		float fSafetyWidth, fSafetyHeight;
		Asura_Vector_2 xTopLeft;
		Asura_Vector_2 xBottomRight;

		fScreenWidth = static_cast<float>(Asura_Viewport::GetWidth());
		fScreenHeight = static_cast<float>(Asura_Viewport::GetHeight());
		fSafetyWidth = fScreenWidth * 0.85f;
		fSafetyHeight = fScreenHeight * 0.85f;

		xTopLeft.x = (fScreenWidth-fSafetyWidth) * 0.5f;
		xTopLeft.y = (fScreenHeight-fSafetyHeight) * 0.5f;
		xBottomRight.x = xTopLeft.x + fSafetyWidth;
		xBottomRight.y = xTopLeft.y + fSafetyHeight;

		// left edge
		Asura_Render::Render2DLine( xTopLeft.x, xTopLeft.y, xTopLeft.x, xBottomRight.y, 0xFFFF0000);
		// right edge
		Asura_Render::Render2DLine( xBottomRight.x, xTopLeft.y, xBottomRight.x, xBottomRight.y, 0xFFFF0000);
		// top edge
		Asura_Render::Render2DLine( xTopLeft.x, xTopLeft.y, xBottomRight.x, xTopLeft.y, 0xFFFF0000);
		// bottom edge
		Asura_Render::Render2DLine( xTopLeft.x, xBottomRight.y, xBottomRight.x, xBottomRight.y, 0xFFFF0000);
	}
}
#endif // ASURA_DEBUG_VARIABLE_SYSTEM
// --------------------------------------------------------------------------------------- 

// (MH - 27.02.2007) text input functions
static bool g_bDoingTextInput = false;
static Asura_UnicodeChar* g_wszTextInputBuffer;
static u_int g_uTextInputBufferMaxChars;
static u_int g_uTextInputCursorPos;
static u_int g_uTextInputFrameNumber;

void Asura_GUIMenu_System::Platform_BeginTextInput( const Asura_UnicodeChar* wszMessage, 
												   Asura_UnicodeChar* wszBuffer, 
												   u_int uBufferMaxChars )
{
	Asura_Assert(!g_bDoingTextInput, "already doing text input");
	g_bDoingTextInput = true;
	g_wszTextInputBuffer = wszBuffer;
	g_uTextInputBufferMaxChars = uBufferMaxChars;
	g_uTextInputCursorPos = uBufferMaxChars;// this indicates "clear on first input"
	g_uTextInputFrameNumber = Asura_Timers::GetFrameNumber();
}

void Asura_GUIMenu_System::Platform_UpdateTextInput()
{
	Asura_UserInput::UpdateInputBuffer( g_wszTextInputBuffer, g_uTextInputBufferMaxChars, g_uTextInputCursorPos, NULL );
}

void Asura_GUIMenu_System::Platform_BeginRender()
{
	Asura_PS3_RenderingStates::RequestAlphaTest(true, 1);
}

void Asura_GUIMenu_System::Platform_RenderTextInput()
{
	// just leave the menu to render it? what about cursors?
}

void Asura_GUIMenu_System::Platform_EndRender()
{
	Asura_PS3_RenderingStates::RequestAlphaTest(false);
}

bool Asura_GUIMenu_System::Platform_IsTextInputFinished()
{
	if (g_uTextInputFrameNumber == Asura_Timers::GetFrameNumber())
	{
		return false;
	}
	return Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_ENTER );
}

void Asura_GUIMenu_System::Platform_EndTextInput()
{
	Asura_Assert(g_bDoingTextInput, "not doing text input");
	g_bDoingTextInput = false;
	g_wszTextInputBuffer = NULL;
	g_uTextInputBufferMaxChars = 0;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_System::Platform_ProcessMenuBackControls( u_int uButtonsPressed, u_int uButtonsTrapped)
{
	bool bRet = false;

	// filters out any buttons presses not required, then we do an equality check
	uButtonsPressed &= (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD);

	if( uButtonsPressed == (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD) )
	{
		if( (uButtonsTrapped&ASURA_GUIMENU_ACTION_BUTTON_CANCEL) == 0)
		{
			bRet = true;
		}
	}

	return bRet;
}

#endif // ASURA_USE_GUI_MENU_SYSTEM
