//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_USBPad.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS3_USBPAD_H___
#define ___ASURA_PS3_USBPAD_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Platform independent
#include "Input/Asura_Controller.h"
#include "Core/Asura_Maths.h"

// Engine platform-specific.
#include "Input/Asura_PlatformInput.h"

// Platform libraries
#include <sdk_version.h>
#include <cell/pad.h>

// M A C R O S ///////////////////////////////////////////////////////////////


#if CELL_SDK_VERSION < 0x250000	// The 'get raw data' functions have been abolished in SDK 250

	#define ASURA_PS3_USBPAD_SUPPORT_MANUAL_TRANSLATION (1)

#endif

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

struct CellPadData;
struct CellPadFilterIIRSos;

#if CELL_SDK_VERSION < 0x250001

	typedef CellPadInfo Asura_PS3_PadInfo;
	
#else

	typedef CellPadInfo2 Asura_PS3_PadInfo;

#endif

// C O N S T S ////////////////////////////////////////////////////////////////

const u_int uNUM_ACCELERATION_SAMPLES = 4;

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PS3_USBPad
#if ASURA_USE_NEW_INPUT
	: public Asura_Controller
#endif //#if ASURA_USE_NEW_INPUT
{

public:
	Asura_PS3_USBPad( u_int uControllerIndex ); 

    void Initialise( u_int uID );
    void Reset( void );
    void Update( const Asura_PS3_PadInfo& xCurPadInfo, u_char* pucButtonOutput, float* pfAxisOutput, float* pfSensorOutput = NULL );

    bool IsConnectedAndSupported( void ) const { return STATUS_CONNECTED == m_eStatus; }
	bool IsSensorSupported( void )		 const { return m_bSensorSupported; }

	// NY - 16 April 2008 - this get's set to true when the PS3 is using the pad input itself, which probably means an overlay is up.
	bool IsInputIntercepted()			 const { return m_bIntercepted; }

	enum ASURA_PS3_USBPAD_SENSOR_FILTER_MODE
	{
		ASURA_PS3_USBPAD_SENSORFILTER_NO_FILTER,
		ASURA_PS3_USBPAD_SENSORFILTER_50_PERCENT,
		ASURA_PS3_USBPAD_SENSORFILTER_20_PERCENT,
		ASURA_PS3_USBPAD_SENSORFILTER_10_PERCENT,
	};

	enum ASURA_PS3_USB_SENSOR_ROCK_STATE
	{
		ASURA_PS3_USB_SENSOR_ROCKSTATE_NONE = 0,
		ASURA_PS3_USB_SENSOR_ROCKSTATE_NEG,
		ASURA_PS3_USB_SENSOR_ROCKSTATE_POS
	};

	void SetSensorFilterMode( const ASURA_PS3_USBPAD_SENSOR_FILTER_MODE uSensorFilterMode ); 
    void SetMotorSpeeds(float fSmall, float fLarge);

	static bool GetAllPadInfo ( Asura_PS3_PadInfo& xPadInfo );

private:

    enum STATUS
    {
        STATUS_CONNECTED,
        STATUS_UNSUPPORTED,  // A pad is connected but it's not supported.
        STATUS_DISCONNECTED,
    };

    static float ConvertToAsuraAxis( int iPS3Value, bool bIsVerticalAxis );
	static float ConvertToAsuraAccelerometer( int iPS3Value ); 
	static float ConvertToAsuraGyroscope( int iPS3Value ); 

    void RefreshStatusWhenConnected( const Asura_PS3_PadInfo& xCurPadInfo );

    void ResetButtonAndAxisStates( void );
    void UpdateButtonAndAxisStates( const CellPadData& xPadData );
	
    void ResetSensorStates( void );
	void UpdateSensorStates( const CellPadData& xPadData ); 

	void UpdateSensorActions( const CellPadData& xPadData);
	int	 DetectSensorSpike( Asura_Vector_3& xSpikeDetection, const Asura_Vector_3& xGradient, u_int uAxis, float fPosSpikeThreshold, float fNegSpikeThreshold);
	u_int DetectRocking( float fCurrentTilt, ASURA_PS3_USB_SENSOR_ROCK_STATE& eRockState, float fPosThreshold, float fNegThreshold);

#if ASURA_PS3_USBPAD_SUPPORT_MANUAL_TRANSLATION
    bool CanManuallyTranslate( const Asura_PS3_PadInfo& xCurPadInfo );
    bool ManuallyTranslate( const Asura_PS3_PadInfo& xCurPadInfo, CellPadData* pxDataOut );
#endif

	bool IsPadConnected	( const Asura_PS3_PadInfo& xInfo ) const;
	bool SetSensorModeOn () const;

	bool IsIntercepted ( const Asura_PS3_PadInfo& xInfo ) const;

	u_int                 m_uID;
	STATUS                m_eStatus;
	u_char                m_aucButtonStates[ASURA_PS3_PADBUTTONOFFSET_NEXT_PAD];
	float                 m_afAxisStates[ASURA_PS3_PADAXISOFFSET_NEXT_PAD];
	bool                  m_bSensorSupported; 
	float                 m_afSensorStates[ASURA_PS3_PADSENSOROFFSET_NEXT_PAD];
	bool                  m_bRumbleSupported;

	float				  m_fRockXTimer;
	ASURA_PS3_USB_SENSOR_ROCK_STATE	 m_eRockState;
	u_int				  m_uNumberOfRocks;

	float				  m_fShakeTimer;
	u_int				  m_uNumberOfSensorSpikes;

	Asura_Vector_3		  m_xIndividualFlickSpikeDetection;
	Asura_Vector_3		  m_xShakeSpikeDetection;
	Asura_Vector_3		  m_axSensorAccSamples[uNUM_ACCELERATION_SAMPLES];
	u_int				  m_uCurrentSensorSample;

	bool                  m_bIntercepted;
	CellPadFilterIIRSos*  m_pxCellPadFilter; 
};
// End class Asura_PS3_USBPad.

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS3_USBPAD_H___
