//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_NP_Room.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     wrapper for creating/joining PSN rooms. Maintains internal list of 
//     NpIds of room members. Although this duplicates the list maintained by
//     the NP system it gives us full control of timing of addition and 
//     removal of players and should be a negligible overhead.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ASURA_PS3_NP_ROOM_H___
#define ASURA_PS3_NP_ROOM_H___ 1

#include <np.h>
#include "Core/Asura_PlatformBasics.h"

enum ASURA_PS3_NP_ROOM_STATE
{
	ASURA_PS3_NP_ROOM_STATE_NONE,
	ASURA_PS3_NP_ROOM_STATE_CREATING,
	ASURA_PS3_NP_ROOM_STATE_IDLE,
	ASURA_PS3_NP_ROOM_STATE_DESTROYED,
	ASURA_PS3_NP_ROOM_STATE_ERROR,
};

class Asura_PS3_NP_Room
{
public:

	Asura_PS3_NP_Room(u_int uMaxPlayers);
	~Asura_PS3_NP_Room();
	
	void HostRoom(u_int uPublicSlots, u_int uPrivateSlots, const SceNpMatching2IntAttr* 
		pxSearchableAttributes, u_int uNumSearchableAttributes);
	void JoinRoom(SceNpMatching2RoomId xRoomID);

	void StartGame(bool bAllowJoinInProgress);

	void InviteFriendToRoom(const Asura_UnicodeChar* wszSubject, const Asura_UnicodeChar* wszBody) const;

	void Update();
	ASURA_PS3_NP_ROOM_STATE GetState() const { return m_eState; }
	int GetErrorCode() const { return m_iErrorCode; }

	SceNpMatching2RoomMemberId GetHostMemberId() const { return m_usHostMemberId; }
	SceNpMatching2RoomMemberId GetLocalMemberID() const { return m_usLocalMemberId; }
	const SceNpMatching2RoomId& GetRoomId() const { return m_xRoomId; }

	//Enable or disable automatic ownership transferal, and set the order in which clients
	//are considered for hosting. Supplying A blank owner list or less than 2 owners will disable.
	bool SetOwnerAutoGrant( const SceNpMatching2RoomMemberId* pusOwners, u_int uNumOwners );
	//Immediately grants ownership to another room member (host only)
	bool GrantOwnership( SceNpMatching2RoomMemberId usNewOwner );

	void AddPlayer( const SceNpId& xNpId );
	void RemovePlayer( const SceNpId& xNpId );
	bool IsPlayerInRoom( const SceNpId& xNpId ) const;
	u_int GetNumPlayers() const { return m_uNumPlayers; }
	u_int GetMaxPlayers() const { return m_uMaxPlayers; }
	const SceNpId& GetPlayer( u_int uIndex ) const;

	static void RegisterRoomCallback();

private:

	// private member functions

	// static functions
	static void RoomRequestCallback(SceNpMatching2ContextId xCtxId, 
		SceNpMatching2RequestId xReqId, 
		SceNpMatching2Event usEvent,
		SceNpMatching2EventKey xEventKey,
		int iErrorCode, 
		size_t uDataSize, 
		void* pArg);

	static void RoomEventCallback(SceNpMatching2ContextId xCtxId, 
		SceNpMatching2RoomId xRoomId,
		SceNpMatching2Event xEvent,
		SceNpMatching2EventKey xEventKey,
		int iErrorCode, 
		size_t uDataSize, 
		void* pArg);

	static void RoomMessageCallback(SceNpMatching2ContextId xCtxId, 
		SceNpMatching2RoomId xRoomId,
		SceNpMatching2RoomMemberId xSrcMemberId,
		SceNpMatching2Event xEvent,
		SceNpMatching2EventKey xEventKey,
		int iErrorCode, 
		size_t uDataSize, 
		void* pArg);

	static void RoomSignalingCallback( SceNpMatching2ContextId xCtxId, 
		SceNpMatching2RoomId xRoomId,
		SceNpMatching2RoomMemberId usPeerMemberId, 
		SceNpMatching2Event usEvent, 
		int iErrorCode, 
		void *arg);

	// member data
	ASURA_PS3_NP_ROOM_STATE m_eState;
	int m_iErrorCode;
	bool m_bIsHost;
	u_int m_uMaxPlayers;
	u_int m_uNumPlayers;
	SceNpId* m_pxPlayers;
	SceNpMatching2RoomMemberId m_usHostMemberId;
	SceNpMatching2RoomMemberId m_usLocalMemberId;
	SceNpMatching2RoomId m_xRoomId;

	// host data
	SceNpMatching2SessionPassword m_xPassword;

	// static class data
	static Asura_PS3_NP_Room* s_pxThis;
};


// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ASURA_PS3_NP_ROOM_H___
