//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_SPURS.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "PPU_SPU_Utils/Asura_PS3_SPURS.h"

// Stdlib platform-specific.
#include <stdlib.h>
#include <stdio.h>

// Engine independent.
#include "Debugging/Asura_Assert.h"   

// Engine platform-specific.


//	TODO:	What is this for? Get rid of
const char _binary_job_notify_bin_start[1] = {'0'};
const char _binary_job_notify_bin_size[1] = {'0'};


//
u_int Asura_PS3_SPURS::m_uNumber_Of_SPUs_To_Use				=	5;

//	Static members

Spurs*	Asura_PS3_SPURS::m_pxSpurs		= NULL;
bool	Asura_PS3_SPURS::m_bInitialised = false;

Asura_PS3_SPURS::Asura_PS3_SPURS()
{
}

Asura_PS3_SPURS::~Asura_PS3_SPURS()
{
}

void	Asura_PS3_SPURS::Initialise( u_int uNumSPUs )
{
	int	iRet = CELL_OK;

	m_pxSpurs = static_cast<Spurs*>( memalign(Spurs::ALIGN, sizeof(Spurs)) );
	iRet = Spurs::initialize( m_pxSpurs, "Asura_SPURS_", m_uNumber_Of_SPUs_To_Use, Spurs::DEFAULT_SPU_THREAD_GROUP_PRIORITY, Spurs::DEFAULT_PPU_THREAD_PRIORITY, Spurs::TYPE_EXCLUSIVE_NON_CONTEXT );
	Asura_Assert( iRet == CELL_OK, "Failed to initialise SPURS" );

	m_bInitialised = true;
}

void	Asura_PS3_SPURS::Shutdown( void )
{
	m_pxSpurs->finalize();
	free(m_pxSpurs);
}

Taskset*	Asura_PS3_SPURS::CreateTaskSet( const char* szSetName, uint64_t uiData )
{
	Asura_Assert( m_bInitialised == true, "Asura_PS3_SPURS not initialised!" );
	int iRet = CELL_OK;

	Taskset* pxTaskset = static_cast<Taskset*>( memalign( Taskset::ALIGN, sizeof(Taskset)) );
	iRet = Taskset::create( pxTaskset, szSetName, m_pxSpurs, uiData );
	Asura_Assert( iRet == CELL_OK, "Failed to create Taskset!" );
	return pxTaskset;
}

Task* 		Asura_PS3_SPURS::CreateTask( Taskset* pxTaskset, const void* pElf, const void* pTaskArg )
{
	Asura_Assert( m_bInitialised == true, "Asura_PS3_SPURS not initialised!" );
	int iRet = CELL_OK;

	Task* pxTask = (Task*)memalign( Task::ALIGN, sizeof(Task) );
	iRet = Task::create( pxTask, pxTaskset,  pElf, (CellSpursTaskArgument*)pTaskArg );
	Asura_Assert( iRet == CELL_OK, "Failed to create task!" );
	return pxTask;
}

Task* 		Asura_PS3_SPURS::CreateTask( Taskset* pxTaskset, const void* pElf, const void* pTaskArg, const void* pContext )
{
	Asura_Assert( m_bInitialised == true, "Asura_PS3_SPURS not initialised!" );
	int iRet = CELL_OK;

	Task* pxTask = (Task*)memalign( Task::ALIGN, sizeof(Task) );
	iRet = Task::create( pxTask, pxTaskset,  pElf, (CellSpursTaskArgument*)pTaskArg, pContext, 1024*1024 );
	Asura_Assert( iRet == CELL_OK, "Failed to create task!" );
	return pxTask;
}

int			Asura_PS3_SPURS::BlockingWaitForTask( Task* pxTask )
{
	int iRet = CELL_OK;
	iRet = pxTask->join();
	return iRet;
}

int		Asura_PS3_SPURS::NonBlockingWaitForTask( Task* pxTask )
{
	int iRet = CELL_OK;
	iRet = pxTask->tryJoin();
	return iRet;
}

void		Asura_PS3_SPURS::ShutdownTask( Task* pxTask )
{
	free( pxTask );
	pxTask = NULL;
}

void		Asura_PS3_SPURS::ShutdownTaskSet( Taskset* pxTaskset )
{
	pxTaskset->shutdown();
	pxTaskset->join();
	free( pxTaskset );
}

//////////////////////////////////////////////////////////////////////////////
//	Asura_PS3_SPURS::Asura_PS3_SPURS_Taskset
//////////////////////////////////////////////////////////////////////////////

Asura_PS3_SPURS::Asura_PS3_SPURS_Taskset::Asura_PS3_SPURS_Taskset( Asura_PS3_SPURS* pxSPURS, const char* szName, uint64_t uiData )
{
	m_pxTaskset = Asura_PS3_SPURS::CreateTaskSet( szName, uiData );
}

Task*	Asura_PS3_SPURS::Asura_PS3_SPURS_Taskset::Begin( Asura_PS3_SPURS* pxSPURS, const void* pElf, const void* pTaskArg/* = NULL*/ )
{
	return Asura_PS3_SPURS::CreateTask( m_pxTaskset, pElf, pTaskArg );
}
Task*	Asura_PS3_SPURS::Asura_PS3_SPURS_Taskset::Begin( Asura_PS3_SPURS* pxSPURS, const void* pElf, const void* pTaskArg, void* pContext )
{
	return Asura_PS3_SPURS::CreateTask( m_pxTaskset, pElf, pTaskArg, pContext );
}

void	Asura_PS3_SPURS::Asura_PS3_SPURS_Taskset::End( Asura_PS3_SPURS* pxSPURS )
{
	Asura_PS3_SPURS::ShutdownTaskSet( m_pxTaskset );
}