// SCE CONFIDENTIAL
// PLAYSTATION(R)3 Programmer Tool Runtime Library 250.001
// Copyright (C) 2008 Sony Computer Entertainment Inc.
// All Rights Reserved.

#ifndef __CELL_UTIL_SPURS_JOBCHAIN_H__
#define __CELL_UTIL_SPURS_JOBCHAIN_H__ 1

#include <cellUtilSpursDefine.h>
#include <cellUtilSpurs.h>
#include <cellUtilSpursJobCommand.h>
#include <cell/spurs/job_chain.h>

__CELL_UTIL_SPURS_BEGIN

class JobChain : public CellSpursJobChain {
	JobChain(const JobChain&);
	JobChain& operator =(const JobChain&);

public:
	static const uint32_t ALIGN = CELL_SPURS_JOBCHAIN_ALIGN;
	JobChain() {}

	static int create(JobChain* jobChain,
					  const char* name,
					  Spurs* spurs,
					  const uint64_t* jobChainEntry,
					  uint16_t sizeJobDescriptor,
					  uint8_t priority = CELL_SPURS_MAX_PRIORITY - 1,
					  uint8_t spuMask = 0xff);

	static int create(JobChain* jobChain,
					  const char* name,
					  Spurs* spurs,
					  const Job::Command* commandList,
					  uint16_t sizeJobDescriptor,
					  uint8_t priority = CELL_SPURS_MAX_PRIORITY - 1,
					  uint8_t spuMask = 0xff)
		{
			return create(jobChain, name, spurs,
						  reinterpret_cast<const uint64_t*>(commandList),
						  sizeJobDescriptor, priority, spuMask);
		}

	static int create(JobChain* jobChain,
					  Spurs* spurs,
					  CellSpursJobChainAttribute *attr)
		{
			return cellSpursCreateJobChainWithAttribute(spurs, jobChain,  attr);
		}

	int run(void)
		{
			return cellSpursRunJobChain(this);
		}
	int shutdown(void)
		{
			return cellSpursShutdownJobChain(this);
		}
	int join(void)
		{
			return cellSpursJoinJobChain(this);
		}

	//
	// Information
	//
	int getError(void** cause)
		{
			return cellSpursJobChainGetError(this, cause);
		}
	int getId(CellSpursWorkloadId* id)
		{
			return cellSpursGetJobChainId(this, id);
		}
	int getSpursAddress(Spurs** spurs)
		{
			return cellSpursJobChainGetSpursAddress(this, (CellSpurs**)spurs);
		}
	int getInfo(CellSpursJobChainInfo* info)
		{
			return cellSpursGetJobChainInfo(this, info);
		}
} __attribute__((aligned(CELL_SPURS_JOBCHAIN_ALIGN)));

__CELL_UTIL_SPURS_END

#endif // __CELL_UTIL_SPURS_JOBCHAIN_H__

// Local Variables:
// mode: C++
// c-file-style: "stroustrup"
// tab-width: 4
// End:
// vim:sw=4:sts=4:ts=4
