// SCE CONFIDENTIAL
// PLAYSTATION(R)3 Programmer Tool Runtime Library 250.001
// Copyright (C) 2007 Sony Computer Entertainment Inc.
// All Rights Reserved.

#ifndef __CELL_UTIL_SPURS_TASK_H__
#define __CELL_UTIL_SPURS_TASK_H__ 1

#include <cellUtilSpursDefine.h>

#include <cell/spurs/task.h>
#include <cell/spurs/task_exit_code.h>

__CELL_UTIL_SPURS_BEGIN

class Taskset;
class TaskElf;

class Task {
	CellSpursTaskExitCode m_exitCode;
	Taskset* m_taskset;
	CellSpursTaskId m_id;
	const void* m_saveBuffer;

	Task(const Task&);
	Task& operator =(const Task&);

public:
	static const uint32_t ALIGN = CELL_SPURS_TASK_EXIT_CODE_ALIGN;
	Task() {}
	//
	// full context task
	//
	static int create(Task* task,
					  Taskset* taskset,
					  const void* elf,
					  const CellSpursTaskArgument& argument,
					  const void* context)
		{
			CellSpursTaskSaveConfig config = {
				const_cast<void*>(context),
				CELL_SPURS_TASK_CONTEXT_SIZE_ALL,
				&gCellSpursTaskLsAll
			};
			return create(task, taskset, elf, &argument, &config);
		}
	static int create(Task* task,
					  Taskset* taskset,
					  const void* elf,
					  const CellSpursTaskArgument* argument,
					  const void* context)
		{
			CellSpursTaskSaveConfig config = {
				const_cast<void*>(context),
				CELL_SPURS_TASK_CONTEXT_SIZE_ALL,
				&gCellSpursTaskLsAll
			};
			return create(task, taskset, elf, argument, &config);
		}
	//
	// no context task
	//
	static int create(Task* task,
					  Taskset* taskset,
					  const void* elf,
					  const CellSpursTaskArgument& argument)
		{
			return create(task, taskset, elf, &argument, (CellSpursTaskSaveConfig*)0);
		}
	static int create(Task* task,
					  Taskset* taskset,
					  const void* elf,
					  const CellSpursTaskArgument* argument)
		{
			return create(task, taskset, elf, argument, (CellSpursTaskSaveConfig*)0);
		}
	//
	// partial context task
	//
	static int create(Task* task,
					  Taskset* taskset,
					  const TaskElf& elf,
					  const CellSpursTaskArgument& argument,
					  const void* saveBuffer,
					  unsigned saveBufferSize)
		{
			return create(task, taskset, elf, &argument, saveBuffer, saveBufferSize);
		}
	static int create(Task* task,
					  Taskset* taskset,
					  const TaskElf& elf,
					  const CellSpursTaskArgument* argument,
					  const void* saveBuffer,
					  unsigned saveBufferSize);
	//
	// generic
	//
	static int create(Task* task,
					  Taskset* taskset,
					  const void* elf,
					  const CellSpursTaskArgument* argument,
					  const CellSpursTaskSaveConfig* config);

	CellSpursTaskId id() const
		{
			return m_id;
		}
	void* saveBuffer() const
		{
			return const_cast<void*>(m_saveBuffer);
		}
	int join(int* value = 0)
		{
			int dummy;
			return cellSpursTaskExitCodeGet(&m_exitCode, value ? value : &dummy);
		}
	int tryJoin(int* value = 0)
		{
			int dummy;
			return cellSpursTaskExitCodeTryGet(&m_exitCode, value ? value : &dummy);
		}
} __attribute__((aligned(CELL_SPURS_TASK_EXIT_CODE_ALIGN)));

__CELL_UTIL_SPURS_END

#endif // __CELL_UTIL_SPURS_TASK_H__

// Local Variables:
// mode: C++
// c-file-style: "stroustrup"
// tab-width: 4
// End:
// vim:sw=4:sts=4:ts=4
