/* SCE CONFIDENTIAL
PLAYSTATION(R)3 Programmer Tool Runtime Library 210.001
* Copyright (C) 2007 Sony Computer Entertainment Inc.
* All Rights Reserved.
*/

#ifndef SPU_THREAD_GROUP_UTILS_H
#define SPU_THREAD_GROUP_UTILS_H



#include <sys/spu_thread_group.h>
#include <sys/spu_thread.h>
#include <sys/spu_utility.h>
#include <sys/spu_image.h>
#include <stdint.h>
#include <stdio.h>

#include <sdk_version.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#if _DEBUG
#define CELL_SRE_PRINTF(...) printf(__VA_ARGS__)
#define CELL_SRE_FPRINTF(fd, ...) fprintf(fd, __VA_ARGS__)
#else
#define CELL_SRE_PRINTF(...)
#define CELL_SRE_FPRINTF(fd, ...)
#endif


typedef struct UtilSpuThreadInfo
{
	sys_spu_thread_attribute_t threadAttr;
	sys_spu_image_t            thread_image;
	sys_spu_thread_argument_t    arg;
} UtilSpuThreadInfo;

typedef struct UtilSpuThreadGroupInfo 
{
	sys_spu_thread_group_t group_id;
	int nThreads;
	sys_spu_thread_t thread_id[8];
	UtilSpuThreadInfo *threads;
} UtilSpuThreadGroupInfo;

typedef struct UtilSpuThreadGroupStatus 
{
	int cause;
	int status;
	int nThreads;
	int threadStatus[8];
} UtilSpuThreadGroupStatus;

int utilInitializeSpuThread(UtilSpuThreadInfo *spuThreadInfo, 	
							const char *name,
							sys_addr_t elf,
							const uint64_t arg[4],
							uint32_t option);


int utilInitializeSpuThreadGroup(UtilSpuThreadGroupInfo *info, 
								 const char *group_name,
								 int priority,
								 int nThreads,
								 UtilSpuThreadInfo *spuThreadInfos); 	
	

int utilStartSpuThreadGroup(const UtilSpuThreadGroupInfo *info);
int utilWaitSpuThreadGroup(const UtilSpuThreadGroupInfo *info, UtilSpuThreadGroupStatus* status);

int utilFinalizeSpuThread(UtilSpuThreadInfo *info);
int utilFinalizeSpuThreadGroup(UtilSpuThreadGroupInfo *info);



typedef struct UtilSpuThreadArg
{
	const char *name;
	sys_addr_t elf;
	uint64_t arg[4];
	uint32_t option;
} UtilSpuThreadArg;

typedef struct UtilSpuThreadGroupArg 
{
	const char *group_name;
	int priority;
	int nThreads;
	UtilSpuThreadArg thread[8];
} UtilSpuThreadGroupArg;

int utilInitializeSpuThreadGroupAll(UtilSpuThreadGroupInfo *info, 
									UtilSpuThreadGroupArg  *arg); 	

int utilFinalizeSpuThreadGroupAll(UtilSpuThreadGroupInfo *info); 	

	





#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* SPU_THREAD_GROUP_UTILS_H */

/*
 * Local Variables:
 * mode: C
 * tab-width: 4
 * End:
 * vim:sw=4:sts=4:ts=4
 */
