//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_ObjectHierarchyRenderer.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_PS3_OBJECTHIERARCHYRENDERER_H___
#define ___ASURA_PS3_OBJECTHIERARCHYRENDERER_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine platform-specific.
#include "Rendering/Asura_PS3_GCM_IndexBuffer.h"
#include "Rendering/Asura_PS3_GCM_VertexBuffer.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Animation;
class Asura_Bounding_Box;
class Asura_Hierarchy_Skin;
class Asura_Matrix_3x3;
class Asura_Shadow;
class Asura_Vector_3;

template< typename T >
class Asura_TemplateHashTree;

// C L A S S E S /////////////////////////////////////////////////////////////

class Asura_PS3_ObjectHierarchyRenderer
{
	friend class Asura_Chunk_ResourceFile_PS3_ObjectHierarchy;
public:
	// Types
	struct Strip
	{
		Asura_Hash_ID	m_uFPHashID;
		Asura_Hash_ID	m_auMaterialResponseHashIDs[2];
		u_int			m_uBoneIndex;
		u_int			m_uStartIndex;
		u_int			m_uNumberOfIndices;
		u_int			m_uNumberOfTriangles;
	};

	class Hierarchy
	{
		friend class Asura_Chunk_ResourceFile_PS3_ObjectHierarchy;
		friend class Asura_PS3_ObjectHierarchyRenderer;
	public:

		// Constructor / destructor
		Hierarchy();
		~Hierarchy();

		// Functions
		u_int GetNumberOfStrips() const { return m_uNumberOfStrips; }
		const Strip& GetStrip( const u_int uStripIndex ) const { return m_pxStrips[ uStripIndex ]; }

	protected:
		Asura_PS3_GCM_IndexBuffer	m_xIndexBuffer;
		Asura_PS3_GCM_VertexBuffer	m_xVertexBuffer;
		Asura_PS3_GCM_IndexBuffer*	m_pxAntiAliasEdges;
		u_int						m_uNumAABuffers;

		u_int						m_uNumberOfStrips;
		Strip*						m_pxStrips;
	};

    static void Initialise( void );
    static void Reset( void );
    static void Shutdown( void );

	// To draw a hierarchical object
	// 1. Call BeginRender()
	// 2. Call Render() for each visible strip of the hierarchical object
	// 3. Call EndRender()

	static void RenderHierarchy( const Asura_Animation& xAnim, const Asura_Hierarchy_Skin& xSkin );
	static void RenderShadow( const Asura_Animation& xAnim, const Asura_Hierarchy_Skin& xSkin, const Asura_Shadow& xShadow );

	static void BeginRender( const Asura_Animation& xAnim, const Asura_Hierarchy_Skin& xSkin );
	static void RenderStrip( const u_int uSectionIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Animation& xAnim );
	static void EndRender();

	static void BeginShadowRender( const Asura_Hierarchy_Skin& xSkin, const Asura_Shadow& xShadow );
	static void RenderShadowStrip( const u_int uStripIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Animation& xAnim, const Asura_Shadow& xShadow );
	static void EndShadowRender();

	static const Asura_TemplateHashTree< Hierarchy >& GetHierarchyTree() { return s_xHierarchyTree; }
	static bool IsHierarchyOpaque( const Asura_Hash_ID uHash );

	static int GetVertexSize( void );

protected:
	// Static functions
	static Hierarchy* CreateHierarchy( const Asura_Hash_ID uHashID );
	// Static data
	static Asura_TemplateHashTree< Hierarchy >	s_xHierarchyTree;
	static const Hierarchy*						s_pxCurrentHierarchy;
	static bool									s_bSetVBData;
};
// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_PS3_OBJECTHIERARCHYRENDERER_H___
