///////////////////////////////////////////////////////////////////////////////
//
// File		: Asura_PS3_RenderingStates.cpp
// Author	: Catalin Arsenescu (11-June-2007)
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Rendering/Asura_PS3_RenderingStates.h"

#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Shaders/Asura_PS3_VertexShader.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"

// Asura independent header-files...
#include "Debugging/Asura_Assert.h"

#include "cell/gcm/gcm_method_data.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////
bool									Asura_PS3_RenderingStates::s_BlendEnableMrt1;
bool									Asura_PS3_RenderingStates::s_BlendEnableMrt2;
bool									Asura_PS3_RenderingStates::s_BlendEnableMrt3;

u_int									Asura_PS3_RenderingStates::s_uColorMaskMRT0;
u_int									Asura_PS3_RenderingStates::s_uColorMaskMRT123;

CellGcmEnum								Asura_PS3_RenderingStates::s_eMinFilter[ ASURA_PS3_MAXIMUM_TEXTURES ];
CellGcmEnum								Asura_PS3_RenderingStates::s_eMagFilter[ ASURA_PS3_MAXIMUM_TEXTURES ];
CellGcmEnum								Asura_PS3_RenderingStates::s_eConvolutionFilter[ ASURA_PS3_MAXIMUM_TEXTURES ];

ASURA_PS3_TEXTURE_OPTIMIZATION			Asura_PS3_RenderingStates::s_eTextureOptimization[ ASURA_PS3_MAXIMUM_TEXTURES ];
bool									Asura_PS3_RenderingStates::s_bUse_sRGBConversion[ ASURA_PS3_MAXIMUM_TEXTURES ];
bool									Asura_PS3_RenderingStates::s_bTextureEnable[ ASURA_PS3_MAXIMUM_TEXTURES ];

CellGcmEnum								Asura_PS3_RenderingStates::s_eTextureAniso[ ASURA_PS3_MAXIMUM_TEXTURES ];


CellGcmEnum								Asura_PS3_RenderingStates::s_eTextureAddressModeS[ ASURA_PS3_MAXIMUM_TEXTURES ];
CellGcmEnum								Asura_PS3_RenderingStates::s_eTextureAddressModeT[ ASURA_PS3_MAXIMUM_TEXTURES ];
CellGcmEnum								Asura_PS3_RenderingStates::s_eTextureAddressModeR[ ASURA_PS3_MAXIMUM_TEXTURES ];

const void*								Asura_PS3_RenderingStates::s_pxTexture[ ASURA_PS3_MAXIMUM_TEXTURES ];
bool									Asura_PS3_RenderingStates::s_abNormalizedCoordinates[ ASURA_PS3_MAXIMUM_TEXTURES ];

u_short									Asura_PS3_RenderingStates::s_usVertexBufferFrequency[ ASURA_PS3_MAXIMUM_VERTEX_BUFFER_ARRAYS ];
u_char									Asura_PS3_RenderingStates::s_ucVertexBufferStride[ ASURA_PS3_MAXIMUM_VERTEX_BUFFER_ARRAYS ];
u_char									Asura_PS3_RenderingStates::s_ucVertexBufferSize[ ASURA_PS3_MAXIMUM_VERTEX_BUFFER_ARRAYS ];
u_char									Asura_PS3_RenderingStates::s_ucVertexBufferType[ ASURA_PS3_MAXIMUM_VERTEX_BUFFER_ARRAYS ];
u_char									Asura_PS3_RenderingStates::s_ucVertexBufferLocation[ ASURA_PS3_MAXIMUM_VERTEX_BUFFER_ARRAYS ];
u_int									Asura_PS3_RenderingStates::s_uiVertexBufferOffset[ ASURA_PS3_MAXIMUM_VERTEX_BUFFER_ARRAYS ];
u_int									Asura_PS3_RenderingStates::s_uiRestartIndex	= 0xFFFFFFFF;

u_int									Asura_PS3_RenderingStates::s_uiTextureOffset[ ASURA_PS3_MAXIMUM_TEXTURES ];
u_int									Asura_PS3_RenderingStates::s_uiTextureFormat[ ASURA_PS3_MAXIMUM_TEXTURES ];
u_int									Asura_PS3_RenderingStates::s_uiTextureAddress[ ASURA_PS3_MAXIMUM_TEXTURES ];
u_int									Asura_PS3_RenderingStates::s_uiTextureControl0[ ASURA_PS3_MAXIMUM_TEXTURES ];
u_int									Asura_PS3_RenderingStates::s_uiTextureControl1[ ASURA_PS3_MAXIMUM_TEXTURES ];
u_int									Asura_PS3_RenderingStates::s_uiTextureControl2[ ASURA_PS3_MAXIMUM_TEXTURES ];
u_int									Asura_PS3_RenderingStates::s_uiTextureControl3[ ASURA_PS3_MAXIMUM_TEXTURES ];
u_int									Asura_PS3_RenderingStates::s_uiTextureFilter[ ASURA_PS3_MAXIMUM_TEXTURES ];
u_int									Asura_PS3_RenderingStates::s_uiTextureImageRect[ ASURA_PS3_MAXIMUM_TEXTURES ];
u_int									Asura_PS3_RenderingStates::s_auTextureBorderColour[ ASURA_PS3_MAXIMUM_TEXTURES ];

const void*								Asura_PS3_RenderingStates::s_pxFragmentShaderConfig;
const void*								Asura_PS3_RenderingStates::s_pxVertexShaderConfig;
const void*								Asura_PS3_RenderingStates::s_pxVertexShaderAddress;

bool									Asura_PS3_RenderingStates::s_bInRecordMode = false;

bool									Asura_PS3_RenderingStates::s_bUseDepthBias = false;

bool									Asura_PS3_RenderingStates::s_bStencilEnabled = false;

bool									Asura_PS3_RenderingStates::s_bWireFrameMode;
bool									Asura_PS3_RenderingStates::s_bZBufferEnabled;
bool									Asura_PS3_RenderingStates::s_bZBufferWriteEnabled;
bool									Asura_PS3_RenderingStates::s_bAlphaBlending;
bool									Asura_PS3_RenderingStates::s_bAlphaTest;
bool									Asura_PS3_RenderingStates::s_bCullFace;
bool									Asura_PS3_RenderingStates::s_bSRGBWriteEnabled;

ASURA_PS3_STENCIL_FUNCTION				Asura_PS3_RenderingStates::s_eStencilFunction = ASURA_PS3_STENCIL_ALWAYS;
ASURA_PS3_STENCIL_OPERATION				Asura_PS3_RenderingStates::s_eStencilFail = ASURA_PS3_STENCIL_OPERATION_KEEP;
ASURA_PS3_STENCIL_OPERATION				Asura_PS3_RenderingStates::s_eDepthFail = ASURA_PS3_STENCIL_OPERATION_KEEP;
ASURA_PS3_STENCIL_OPERATION				Asura_PS3_RenderingStates::s_eDepthPass = ASURA_PS3_STENCIL_OPERATION_KEEP;
u_int									Asura_PS3_RenderingStates::s_uiStencilRef = 0;
u_int									Asura_PS3_RenderingStates::s_uiStencilFunctionMask = 0xFF;
u_int									Asura_PS3_RenderingStates::s_uiStencilMask = 0xFF;

int										Asura_PS3_RenderingStates::s_iAlphaTestRef;

enum ASURA_ZBUFFERCMP_ID				Asura_PS3_RenderingStates::s_eZBufferCompareMode;
enum ASURA_TRANSLUCENCY_MODE			Asura_PS3_RenderingStates::s_eTranslucencyMode;
enum ASURA_CULLMODE_ID					Asura_PS3_RenderingStates::s_eCullMode;

float									Asura_PS3_RenderingStates::s_fDepthBiasFactor = 0.0f;
float									Asura_PS3_RenderingStates::s_fDepthBiasUnits = 0.0f;

int										Asura_PS3_RenderingStates::s_iLineWidth = 8;

CellGcmSurface*							Asura_PS3_RenderingStates::s_pxSurface = NULL;

u_int 									Asura_PS3_RenderingStates::s_uViewportX = 0;
u_int 									Asura_PS3_RenderingStates::s_uViewportY = 0;
u_int 									Asura_PS3_RenderingStates::s_uViewportWidth = 4096;
u_int 									Asura_PS3_RenderingStates::s_uViewportHeight = 4096;
float 									Asura_PS3_RenderingStates::s_fViewportOutputMinZ = 0.0f;
float 									Asura_PS3_RenderingStates::s_fViewportOutputMaxZ = 1.0f;
float 									Asura_PS3_RenderingStates::s_fViewportClipMinZ = 0.0f;
float 									Asura_PS3_RenderingStates::s_fViewportClipMaxZ = 1.0f;
bool									Asura_PS3_RenderingStates::s_bViewportScaleToSurface = false;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::Initialise
// Author		: Catalin Arsenescu (11-June-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::Initialise( void )
{
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::Shutdown
// Author		: Catalin Arsenescu (11-June-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::Shutdown( void )
{
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::Reset
// Author		: Catalin Arsenescu (11-June-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::BeginFrame( void )
{
	int i;

	// RSX resets these render states every frame so update our states to match - see graphics/libgcm-overview section 12.2	
	s_uColorMaskMRT0	= ASURA_PS3_COLOR_MASK_MRT0_ALL;
	s_uColorMaskMRT123	= ASURA_PS3_COLOR_MASK_MRT123_NONE;

	s_BlendEnableMrt1	= false;
	s_BlendEnableMrt2	= false;
	s_BlendEnableMrt3	= false;

	s_bStencilEnabled	= false;
	s_eStencilFunction	= ASURA_PS3_STENCIL_ALWAYS;
	s_eStencilFail		= ASURA_PS3_STENCIL_OPERATION_KEEP;
	s_eDepthFail		= ASURA_PS3_STENCIL_OPERATION_KEEP;
	s_eDepthPass		= ASURA_PS3_STENCIL_OPERATION_KEEP;
	s_uiStencilRef		= 0;
	s_uiStencilFunctionMask	= 0xFF;
	s_uiStencilMask		= 0xFF;
	s_pxSurface			= NULL;
	
	s_uViewportX = 0;
	s_uViewportY = 0;
	s_uViewportWidth = 4096;
	s_uViewportHeight = 4096;
	s_fViewportOutputMinZ = 0.0f;
	s_fViewportOutputMaxZ = 1.0f;
	s_fViewportClipMinZ = 0.0f;
	s_fViewportClipMaxZ = 1.0f;
	s_bViewportScaleToSurface = false; //default scales are for 4096 regardless of actual frame buffer size
	
	// SCULL isn't reset from one frame to the next, there's probably a more efficient way to do this.
	Asura_PS3_Graphics::SwitchStencilCullMode( ASURA_PS3_STENCIL_ALWAYS, 0, 0 );

	s_bWireFrameMode	= false;
	s_bZBufferEnabled	= false;
	s_bZBufferWriteEnabled = true;
	s_bAlphaBlending	= false;
	s_bAlphaTest		= false;
	s_bCullFace			= false;
	s_bSRGBWriteEnabled	= false;
	
	s_iLineWidth		= 8;

	s_iAlphaTestRef		= -1;	

	s_eZBufferCompareMode = ASURA_ZBUFFERCMP_ALWAYS;
	RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
	s_eTranslucencyMode	= ASURA_TRANSLUCENCY_NOT_SET;
	s_eCullMode			= ASURA_CULLMODE_CCW;

	for( i = 0; i < ASURA_PS3_MAXIMUM_TEXTURES; ++i )
	{
		s_bUse_sRGBConversion[ i ]	= false;
		s_eMinFilter[ i ]			= CELL_GCM_TEXTURE_NEAREST_LINEAR;
		s_eMagFilter[ i ]			= CELL_GCM_TEXTURE_LINEAR;
		s_eConvolutionFilter[ i ]	= CELL_GCM_TEXTURE_CONVOLUTION_QUINCUNX;

		s_eTextureOptimization[ i ]	= ASURA_PS3_TEXTURE_OPTIMIZATION_MEDIUM_QUALITY;

		s_bTextureEnable[ i ]		= false;
		s_eTextureAniso[ i ]		= CELL_GCM_TEXTURE_MAX_ANISO_1;

		s_eTextureAddressModeS[ i ]	= CELL_GCM_TEXTURE_WRAP;
		s_eTextureAddressModeT[ i ]	= CELL_GCM_TEXTURE_WRAP;
		s_eTextureAddressModeR[ i ]	= CELL_GCM_TEXTURE_CLAMP_TO_EDGE;

		s_pxTexture[ i ] = NULL;
		s_abNormalizedCoordinates[ i ] = true;

		s_uiTextureOffset[ i ]		= 0xFFFFFFFF;
		s_uiTextureFormat[ i ]		= 0xFFFFFFFF;
		s_uiTextureAddress[ i ]		= 0xFFFFFFFF;
		s_uiTextureControl0[ i ]	= 0xFFFFFFFF;
		s_uiTextureControl1[ i ]	= 0xFFFFFFFF;
		s_uiTextureControl2[ i ]	= 0xFFFFFFFF;
		s_uiTextureControl3[ i ]	= 0xFFFFFFFF;
		s_uiTextureFilter[ i ]		= 0xFFFFFFFF;
		s_uiTextureImageRect[ i ]	= 0xFFFFFFFF;
		s_auTextureBorderColour[i]	= 0;
	}

	for( i = 0; i < ASURA_PS3_MAXIMUM_VERTEX_BUFFER_ARRAYS; ++i )
	{
		s_usVertexBufferFrequency[ i ]	= 0xFFFF;
		s_ucVertexBufferStride[ i ]		= 0xFF;
		s_ucVertexBufferSize[ i ]		= 0xFF;
		s_ucVertexBufferType[ i ]		= 0xFF;
		s_ucVertexBufferLocation[ i ]	= 0xFF;
		s_uiVertexBufferOffset[ i ]		= 0x00000000;
	}

	s_pxFragmentShaderConfig	= NULL;
	s_pxVertexShaderConfig		= NULL;
	s_pxVertexShaderAddress		= NULL;
	s_uiRestartIndex			= 0xFFFFFFFF;

	s_bUseDepthBias = false;
	s_fDepthBiasFactor = 0.0f;
	s_fDepthBiasUnits = 0.0f;

	Asura_PS3_GCM_VertexShader::ResetToDefault();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::SetColorMaskMrt0
// Author		: Catalin Arsenescu (11-June-2007) - Re-write by Ben Weston (4/9/08)
// Input		: ASURA_PS3_COLOR_MASK_MRT0 - or any u_int formatted as specified in that define
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::SetColorMaskMrt0( u_int uColorMask )
{
	if( s_uColorMaskMRT0 == uColorMask && !s_bInRecordMode ) return;

	s_uColorMaskMRT0 = uColorMask;
	
	cellGcmSetColorMaskUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(),
		((uColorMask & ASURA_PS3_COLOR_MASK_MRT0_RED  )?CELL_GCM_COLOR_MASK_R:0 ) |
		((uColorMask & ASURA_PS3_COLOR_MASK_MRT0_GREEN)?CELL_GCM_COLOR_MASK_G:0 ) |
		((uColorMask & ASURA_PS3_COLOR_MASK_MRT0_BLUE )?CELL_GCM_COLOR_MASK_B:0 ) |
		((uColorMask & ASURA_PS3_COLOR_MASK_MRT0_ALPHA)?CELL_GCM_COLOR_MASK_A:0 )
	);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::SetColorMaskMrt123
// Author		: Catalin Arsenescu (11-June-2007) - Re-write by Ben Weston (4/9/08)
// Input		: ASURA_PS3_COLOR_MASK_MRT123 - or any u_int formatted as specified in that define
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::SetColorMaskMrt123( u_int uColorMask )
{
	if( s_uColorMaskMRT123 == uColorMask && !s_bInRecordMode ) return;

	s_uColorMaskMRT123 = uColorMask;

	cellGcmSetColorMaskMrtUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(),	s_uColorMaskMRT123 ); //our enum matches the definitions of CELL_GCM_COLOR_MASK_MRT1_A etc.

	Asura_Assert( CELL_GCM_COLOR_MASK_MRT2_G == 0x0400, "Sanity check failed, have the CELL_GCM_COLOR_MASK_ defines changed in gcm_enum.h?" );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::SetColorMaskMrtState
// Author		: Catalin Arsenescu (11-June-2007) - Re-write by Ben Weston (4/9/08)
// Input		: ASURA_PS3_MRT_COMBINATION - or any u_int formatted as specified in that define
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::SetColorMaskMrtState( u_int uCombination )
{
	SetColorMaskMrt0  ( uCombination&0x000f );
	SetColorMaskMrt123( uCombination&0xfff0 );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::SetBlendEnableMrt
// Author		: Catalin Arsenescu (11-June-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::SetBlendEnableMrt( bool bTarget1, bool bTarget2, bool bTarget3 )
{
	u_int		uMrt1, uMrt2, uMrt3;

	if( s_BlendEnableMrt1 != bTarget1 || s_BlendEnableMrt2 != bTarget2 || s_BlendEnableMrt3 != bTarget3 || s_bInRecordMode )
	{
		s_BlendEnableMrt1 = bTarget1;
		s_BlendEnableMrt2 = bTarget2;
		s_BlendEnableMrt3 = bTarget3;

		if( bTarget1 )
		{
			uMrt1 = CELL_GCM_TRUE;
		}
		else
		{
			uMrt1 = CELL_GCM_FALSE;
		}

		if( bTarget2 )
		{
			uMrt2 = CELL_GCM_TRUE;
		}
		else
		{
			uMrt2 = CELL_GCM_FALSE;
		}

		if( bTarget3 )
		{
			uMrt3 = CELL_GCM_TRUE;
		}
		else
		{
			uMrt3 = CELL_GCM_FALSE;
		}
		//		uMrt3 = CELL_GCM_FALSE;

		cellGcmSetBlendEnableMrtUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), uMrt1, uMrt2, uMrt3 );
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::SetTextureFilter
// Author		: Catalin Arsenescu (11-June-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::SetTextureFilter( u_char ucTextureIndex, CellGcmEnum eMinFilter, CellGcmEnum eMagFilter, CellGcmEnum eConvolutionFilter )
{
	if( eMinFilter == s_eMinFilter[ ucTextureIndex ] && eMagFilter == s_eMagFilter[ ucTextureIndex ] && eConvolutionFilter == s_eConvolutionFilter[ ucTextureIndex ] && !s_bInRecordMode ) return;


	s_eMinFilter[ ucTextureIndex ]			= eMinFilter;
	s_eMagFilter[ ucTextureIndex ]			= eMagFilter;
	s_eConvolutionFilter[ ucTextureIndex ]	= eConvolutionFilter;

	cellGcmSetTextureFilterUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucTextureIndex, 0, eMinFilter, eMagFilter, eConvolutionFilter );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::SetTextureOptimization
// Author		: Catalin Arsenescu (11-June-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::SetTextureOptimization( u_char ucTextureIndex, ASURA_PS3_TEXTURE_OPTIMIZATION eTextureOptimization )
{
	if( eTextureOptimization == s_eTextureOptimization[ ucTextureIndex ] && !s_bInRecordMode ) return;

	s_eTextureOptimization[ ucTextureIndex ] = eTextureOptimization;

	switch( eTextureOptimization )
	{
	case ASURA_PS3_TEXTURE_OPTIMIZATION_LOW_QUALITY:
		{
			cellGcmSetTextureOptimizationUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucTextureIndex, 0x1F, CELL_GCM_TEXTURE_ISO_LOW, CELL_GCM_TEXTURE_ANISO_LOW );
			break;
		}
	case ASURA_PS3_TEXTURE_OPTIMIZATION_MEDIUM_QUALITY:
		{
			cellGcmSetTextureOptimizationUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucTextureIndex, 0x08, CELL_GCM_TEXTURE_ISO_HIGH, CELL_GCM_TEXTURE_ANISO_HIGH );
			break;
		}
	case ASURA_PS3_TEXTURE_OPTIMIZATION_HIGH_QUALITY:
		{
			cellGcmSetTextureOptimizationUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucTextureIndex, 0x00, CELL_GCM_TEXTURE_ISO_HIGH, CELL_GCM_TEXTURE_ANISO_HIGH );
			break;
		}
	default:
		{
			Asura_Assert( false, "Invalid SetTextureOptimization mode!" );
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::SetTextureControl
// Author		: Catalin Arsenescu (11-June-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::SetTextureControl( u_char ucTextureIndex, bool bEnable, CellGcmEnum eTextureAniso )
{
	if( bEnable == s_bTextureEnable[ ucTextureIndex ] && eTextureAniso == s_eTextureAniso[ ucTextureIndex ] && !s_bInRecordMode ) return;

	u_int uEnabled;

	if( bEnable )
	{
		uEnabled = CELL_GCM_TRUE;
	}
	else
	{
		uEnabled = CELL_GCM_FALSE;
	}

	s_eTextureAniso[ ucTextureIndex ]	= eTextureAniso;
	s_bTextureEnable[ ucTextureIndex ]	= bEnable;

	cellGcmSetTextureControlUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucTextureIndex, uEnabled, 0, 12U<<8, eTextureAniso );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_RenderingStates::SetTextureAddressMode
// Author		: Catalin Arsenescu (11-June-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_RenderingStates::SetTextureAddressMode( u_char ucTextureIndex, CellGcmEnum eAddressModeS, CellGcmEnum eAddressModeT, CellGcmEnum eAddressModeR, bool bUse_sRGBConversion )
{
	if( s_eTextureAddressModeS[ ucTextureIndex ] == eAddressModeS && s_eTextureAddressModeT[ ucTextureIndex ] == eAddressModeT && s_eTextureAddressModeR[ ucTextureIndex ] == eAddressModeR && s_bUse_sRGBConversion[ ucTextureIndex ] == bUse_sRGBConversion && !s_bInRecordMode ) return;

	s_eTextureAddressModeS[ ucTextureIndex ]	= eAddressModeS;
	s_eTextureAddressModeT[ ucTextureIndex ]	= eAddressModeT;
	s_eTextureAddressModeR[ ucTextureIndex ]	= eAddressModeR;
	s_bUse_sRGBConversion[ ucTextureIndex ]		= bUse_sRGBConversion;

	if( bUse_sRGBConversion && Asura_PS3_Graphics::UseGammaCorrection() )
	{
		cellGcmSetTextureAddressUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucTextureIndex,  eAddressModeS, eAddressModeT, eAddressModeR, CELL_GCM_TEXTURE_UNSIGNED_REMAP_NORMAL, CELL_GCM_TEXTURE_ZFUNC_LESS, CELL_GCM_TEXTURE_GAMMA_R | CELL_GCM_TEXTURE_GAMMA_G | CELL_GCM_TEXTURE_GAMMA_B );
	}
	else
	{
		cellGcmSetTextureAddressUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucTextureIndex,  eAddressModeS, eAddressModeT, eAddressModeR, CELL_GCM_TEXTURE_UNSIGNED_REMAP_NORMAL, CELL_GCM_TEXTURE_ZFUNC_LESS, 0 );
	}
}


void Asura_PS3_RenderingStates::SetTextureBorderColour( u_char ucTextureIndex, u_int uColour )
{
	if( s_auTextureBorderColour[ ucTextureIndex ] != uColour )
	{
		cellGcmSetTextureBorderColorUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucTextureIndex, uColour );
	}
}


void Asura_PS3_RenderingStates::SetTexture( u_char ucTextureChannel, const CellGcmTexture* pxTexture, bool bNormalizedCoordinates )
{
	if( s_pxTexture[ ucTextureChannel ] == pxTexture && s_abNormalizedCoordinates[ ucTextureChannel ] == bNormalizedCoordinates && !s_bInRecordMode ) return;

	s_pxTexture[ ucTextureChannel ] = pxTexture;
	s_abNormalizedCoordinates[ ucTextureChannel ] = bNormalizedCoordinates;

	if( pxTexture && pxTexture->width > 0 && pxTexture->height > 0 )
	{
		cellGcmSetTextureUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucTextureChannel, pxTexture );
	}
	else
	{
		Asura_PS3_RenderingStates::SetTextureControl( ucTextureChannel, false );
	}
}

Asura_PS3_Texture* Asura_PS3_RenderingStates::RequestTextureIndex( int iTextureChannel, int iTextureIndex )
{
	if( iTextureIndex != ASURA_TEXTURE_INVALID )
	{
		Asura_PS3_Texture* pxTexture = Asura_PS3_TextureManagement::GetTexture( iTextureIndex );
		pxTexture->SetAsTextureIndex( iTextureChannel );
		return pxTexture;
	}
	return NULL;
}

void Asura_PS3_RenderingStates::SetTextureMacro( u_char iTextureChannel, Asura_PS3_Texture* pxTexture, u_int uiTextureAddress, u_int uiTextureControl0, u_int uiTextureControl2, u_int uiTextureFilter )
{
	CellGcmContextData* pxCommandBuffer	= Asura_PS3_Graphics::GetGcmCurrentContext();
	uint32_t* puiCmd					= pxCommandBuffer->current;
	int iNum							= 0;

	puiCmd[ 1 ] = pxTexture->GetTextureOffset();
	puiCmd[ 2 ] = pxTexture->GetTextureFormat();
	puiCmd[ 3 ] = uiTextureAddress;
	puiCmd[ 4 ] = uiTextureControl0;
	puiCmd[ 5 ] = pxTexture->GetTextureControl1();
	puiCmd[ 6 ] = uiTextureFilter;
	puiCmd[ 7 ] = pxTexture->GetTextureImageRect();
	//	puiCmd[ 7 ] = uiTextureControl2;

	if( SetTextureRectNeeded( iTextureChannel, pxTexture->GetTextureImageRect() ) )
	{
		iNum = 7;
		SetTextureFilterNeeded( iTextureChannel, uiTextureFilter );
		SetTextureControl1Needed( iTextureChannel, pxTexture->GetTextureControl1() );
		SetTextureControl0Needed( iTextureChannel, uiTextureControl0 );
		SetTextureAddressNeeded( iTextureChannel, uiTextureAddress );
		SetTextureFormatNeeded( iTextureChannel, pxTexture->GetTextureFormat() );
		SetTextureOffsetNeeded( iTextureChannel, pxTexture->GetTextureOffset() );
	}
	else if( SetTextureFilterNeeded( iTextureChannel, uiTextureFilter ) )
	{
		iNum = 6;
		SetTextureControl1Needed( iTextureChannel, pxTexture->GetTextureControl1() );
		SetTextureControl0Needed( iTextureChannel, uiTextureControl0 );
		SetTextureAddressNeeded( iTextureChannel, uiTextureAddress );
		SetTextureFormatNeeded( iTextureChannel, pxTexture->GetTextureFormat() );
		SetTextureOffsetNeeded( iTextureChannel, pxTexture->GetTextureOffset() );
	}
	else if( SetTextureControl1Needed( iTextureChannel, pxTexture->GetTextureControl1() ) )
	{
		iNum = 5;
		SetTextureControl0Needed( iTextureChannel, uiTextureControl0 );
		SetTextureAddressNeeded( iTextureChannel, uiTextureAddress );
		SetTextureFormatNeeded( iTextureChannel, pxTexture->GetTextureFormat() );
		SetTextureOffsetNeeded( iTextureChannel, pxTexture->GetTextureOffset() );
	}
	else if( SetTextureControl0Needed( iTextureChannel, uiTextureControl0 ) )
	{
		iNum = 4;
		SetTextureAddressNeeded( iTextureChannel, uiTextureAddress );
		SetTextureFormatNeeded( iTextureChannel, pxTexture->GetTextureFormat() );
		SetTextureOffsetNeeded( iTextureChannel, pxTexture->GetTextureOffset() );
	}
	else if( SetTextureAddressNeeded( iTextureChannel, uiTextureAddress ) )
	{
		iNum = 3;
		SetTextureFormatNeeded( iTextureChannel, pxTexture->GetTextureFormat() );
		SetTextureOffsetNeeded( iTextureChannel, pxTexture->GetTextureOffset() );
	}
	else if( SetTextureFormatNeeded( iTextureChannel, pxTexture->GetTextureFormat() ) )
	{
		iNum = 2;
		SetTextureOffsetNeeded( iTextureChannel, pxTexture->GetTextureOffset() );
	}
	else if( SetTextureOffsetNeeded( iTextureChannel, pxTexture->GetTextureOffset() ) )
	{
		iNum = 1;
	}

	if( iNum > 0 )
	{
		puiCmd[ 0 ] = CELL_GCM_METHOD_HEADER_TEXTURE_OFFSET( iTextureChannel, iNum );
		++iNum;
	}

	if( SetTextureControl3Needed( iTextureChannel, pxTexture->GetTextureControl3() ) )
	{
		puiCmd[ iNum ] = CELL_GCM_METHOD_HEADER_TEXTURE_CONTROL3( iTextureChannel, 1 );
		++iNum;
		puiCmd[ iNum ] = pxTexture->GetTextureControl3();
		++iNum;
	}

	if( SetTextureControl2Needed( iTextureChannel, uiTextureControl2 ) )
	{
		puiCmd[ iNum ] = CELL_GCM_METHOD_HEADER_TEXTURE_CONTROL2( iTextureChannel, 1 );
		++iNum;
		puiCmd[ iNum ] = uiTextureControl2;
		++iNum;
	}

	if( iNum > 0 )
	{
		pxCommandBuffer->current += iNum;
	}
}

void Asura_PS3_RenderingStates::ClearTexture( u_char ucTextureIndex )
{
	s_pxTexture[ ucTextureIndex ] = NULL;
}

void Asura_PS3_RenderingStates::SetVertexDataArray ( u_char ucIndex, u_short usFrequency, u_char ucStride, u_char ucSize, u_char ucType, u_char ucLocation, u_int uiOffset)
{
	if( s_usVertexBufferFrequency[ ucIndex ]	== usFrequency	&&
		s_ucVertexBufferStride[ ucIndex ]		== ucStride		&&
		s_ucVertexBufferSize[ ucIndex ]			== ucSize		&&
		s_ucVertexBufferType[ ucIndex ]			== ucType		&&
		s_ucVertexBufferLocation[ ucIndex ]		== ucLocation	&&
		s_uiVertexBufferOffset[ ucIndex ]		== uiOffset		&& !s_bInRecordMode ) return;

	if(	s_usVertexBufferFrequency[ ucIndex ]	!= usFrequency	||
		s_ucVertexBufferStride[ ucIndex ]		!= ucStride		||
		s_ucVertexBufferSize[ ucIndex ]			!= ucSize		||
		s_ucVertexBufferType[ ucIndex ]			!= ucType )
	{
		if(	s_ucVertexBufferLocation[ ucIndex ]	!= ucLocation	||
			s_uiVertexBufferOffset[ ucIndex ]	!= uiOffset )
		{
			cellGcmSetVertexDataArrayUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucIndex, usFrequency, ucStride, ucSize, ucType, ucLocation, uiOffset );
		}
		else
		{
			cellGcmSetVertexDataArrayFormatUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucIndex, usFrequency, ucStride, ucSize, ucType );
		}
	}
	else
	{
		cellGcmSetVertexDataArrayOffsetUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ucIndex, ucLocation, uiOffset );
	}

	s_usVertexBufferFrequency[ ucIndex ]	= usFrequency;
	s_ucVertexBufferStride[ ucIndex ]		= ucStride;
	s_ucVertexBufferSize[ ucIndex ]			= ucSize;
	s_ucVertexBufferType[ ucIndex ]			= ucType;
	s_ucVertexBufferLocation[ ucIndex ]		= ucLocation;
	s_uiVertexBufferOffset[ ucIndex ]		= uiOffset;
}

bool Asura_PS3_RenderingStates::SetVertexAttributeNeeded( u_char ucIndex, u_short usFrequency, u_char ucStride, u_char ucSize, u_char ucType )
{
	if(	s_usVertexBufferFrequency[ ucIndex ]	!= usFrequency	||
		s_ucVertexBufferStride[ ucIndex ]		!= ucStride		||
		s_ucVertexBufferSize[ ucIndex ]			!= ucSize		||
		s_ucVertexBufferType[ ucIndex ]			!= ucType )
	{
		s_usVertexBufferFrequency[ ucIndex ]	= usFrequency;
		s_ucVertexBufferStride[ ucIndex ]		= ucStride;
		s_ucVertexBufferSize[ ucIndex ]			= ucSize;
		s_ucVertexBufferType[ ucIndex ]			= ucType;

		return true;
	}
	return false;
}

bool Asura_PS3_RenderingStates::SetVertexOffsetNeeded( u_char ucIndex, u_char ucLocation, u_int uiOffset )
{
	if(	s_ucVertexBufferLocation[ ucIndex ]	!= ucLocation	||
		s_uiVertexBufferOffset[ ucIndex ]	!= uiOffset )
	{
		s_ucVertexBufferLocation[ ucIndex ]	= ucLocation;
		s_uiVertexBufferOffset[ ucIndex ]	= uiOffset;

		return true;
	}
	return false;
}


void Asura_PS3_RenderingStates::SetVertexShader( const void* pxVertexShaderConfig, const void* pxVertexShaderAddress, Asura_PS3_GCM_VertexShader* pxVertexShader )
{
	if( s_pxVertexShaderConfig == pxVertexShaderConfig && s_pxVertexShaderAddress == pxVertexShaderAddress && !s_bInRecordMode ) return;

	s_pxVertexShaderAddress	= pxVertexShaderAddress;
	s_pxVertexShaderConfig	= pxVertexShaderConfig;

#if 0 //No need to use GCM Macro (default code use the same approach)

	if( Asura_PS3_Graphics::UseGCmMacros() )
	{
		CellGcmContextData* pxCommandBuffer = Asura_PS3_Graphics::GetGcmCurrentContext();
		CellCgbVertexProgramConfiguration* pxVertexConfig = ( CellCgbVertexProgramConfiguration* )pxVertexShaderConfig;
		uint32_t* puiCmd = pxCommandBuffer->current;
		const uint32_t* puiRawData = ( const uint32_t* )pxVertexShaderAddress;
		int iInstCount = static_cast<int>( pxVertexConfig->instructionCount );
		uint32_t uiInstIndex = pxVertexConfig->instructionSlot;

		puiCmd[ 0 ] = CELL_GCM_METHOD( CELL_GCM_NV4097_SET_TRANSFORM_PROGRAM_LOAD, 2 );
		puiCmd[ 1 ] = CELL_GCM_METHOD_DATA_VERTEX_PROGRAM_LOAD( uiInstIndex );
		puiCmd[ 2 ] = CELL_GCM_METHOD_DATA_VERTEX_PROGRAM_START( uiInstIndex );
		pxCommandBuffer->current += 3;

		while( iInstCount > 0 )
		{
			int iInstToTransfer;
			if( iInstCount >= 8 )
			{
				iInstToTransfer = 8;
			}
			else
			{
				iInstToTransfer = iInstCount;
			}
			int iLoop = iInstToTransfer * 4;

			puiCmd = pxCommandBuffer->current;
			puiCmd[ 0 ] = CELL_GCM_METHOD( CELL_GCM_NV4097_SET_TRANSFORM_PROGRAM, iLoop );
			for( int i = 0; i < iLoop; ++i )
			{
				puiCmd[ i + 1 ] = CELL_GCM_METHOD_DATA_VERTEX_PROGRAM( puiRawData[ i ] );
			}
			puiRawData += iLoop;
			iInstCount -= iInstToTransfer;
			pxCommandBuffer->current += iLoop + 1;
		}

		puiCmd = pxCommandBuffer->current;
		if( pxVertexConfig->registerCount <= 32 )
		{
			puiCmd[ 0 ] = CELL_GCM_METHOD( CELL_GCM_NV4097_SET_TRANSFORM_TIMEOUT, 1 );
			puiCmd[ 1 ] = CELL_GCM_METHOD_DATA_VERTEX_PROGRAM_TIMEOUT( 0xFFFF, 32 );
		}
		else
		{
			puiCmd[ 0 ] = CELL_GCM_METHOD( CELL_GCM_NV4097_SET_TRANSFORM_TIMEOUT, 1 );
			puiCmd[ 1 ] = CELL_GCM_METHOD_DATA_VERTEX_PROGRAM_TIMEOUT( 0xFFFF, 48 );
		}
		pxCommandBuffer->current += 2;
		cellGcmSetVertexAttribInputMaskUnsafeInline( pxCommandBuffer, pxVertexConfig->attributeInputMask );
	}
	else
#else
	{
		cellGcmSetVertexProgramLoadUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ( CellCgbVertexProgramConfiguration* )pxVertexShaderConfig, pxVertexShaderAddress );
	}
#endif
	pxVertexShader->LoadVertexProgramConstants();
}

void Asura_PS3_RenderingStates::SetFragmentShader( const void* pxFragmentShaderConfig, bool bFPInMainMemory )
{
	if( s_pxFragmentShaderConfig == pxFragmentShaderConfig && !s_bInRecordMode ) return;

	s_pxFragmentShaderConfig = pxFragmentShaderConfig;

	if( bFPInMainMemory )
	{
		cellGcmSetFragmentProgramLoadLocationUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ( CellCgbFragmentProgramConfiguration* )pxFragmentShaderConfig, CELL_GCM_LOCATION_MAIN );
	}
	else
	{
		cellGcmSetFragmentProgramLoadUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ( CellCgbFragmentProgramConfiguration* )pxFragmentShaderConfig );
	}
}

void Asura_PS3_RenderingStates::SetUseDepthBias( bool bEnabled )
{
	if( bEnabled == s_bUseDepthBias && !s_bInRecordMode  ) return;

	s_bUseDepthBias = bEnabled;
	cellGcmSetPolygonOffsetFillEnableUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), bEnabled ? CELL_GCM_TRUE : CELL_GCM_FALSE );
}

void Asura_PS3_RenderingStates::SetDepthBias( float fUnits, float fFactor )
{
	if( fUnits == s_fDepthBiasUnits && fFactor == s_fDepthBiasFactor && !s_bInRecordMode  ) return;

	s_fDepthBiasUnits = fUnits;
	s_fDepthBiasFactor = fFactor;

	cellGcmSetPolygonOffsetUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), fFactor, fUnits );
}


static bool SurfaceHasStencil( const CellGcmSurface* pxSurface )
{
	return pxSurface->depthFormat == CELL_GCM_SURFACE_Z24S8 //z format has stencil
		&& !(pxSurface->depthOffset == 0 && pxSurface->depthPitch == 64); //flag to indicate uninitialised buffer
}


void Asura_PS3_RenderingStates::SetStencilTestEnabled( bool bEnable )
{
	if ( s_pxSurface==NULL || !SurfaceHasStencil(s_pxSurface) )
	{
		Asura_Assert(!bEnable, "Cannot enable stencil without stencil buffer bound.");		//seriously, the GPU crashes
 		Asura_Assert(!s_bStencilEnabled, "Stencil sanity check failed! changed surfaces with it enabled! oh heck.");
		s_bStencilEnabled = false;
		return;
	}

	if( s_bStencilEnabled != bEnable )
	{
		s_bStencilEnabled = bEnable;
		cellGcmSetStencilTestEnableUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), s_bStencilEnabled ? CELL_GCM_TRUE : CELL_GCM_FALSE );
	}
}

void Asura_PS3_RenderingStates::SetStencilMask( u_int uMask )
{
	if( s_uiStencilMask != uMask )
	{
		s_uiStencilMask = uMask;
		cellGcmSetStencilMaskUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), s_uiStencilMask );
	}
}

void Asura_PS3_RenderingStates::SetStencilFunction( ASURA_PS3_STENCIL_FUNCTION eStencilFunction, u_int uiRef, u_int uiMask )
{
	if( eStencilFunction != s_eStencilFunction || s_uiStencilRef != uiRef || s_uiStencilFunctionMask != uiMask )
	{
		s_eStencilFunction	= eStencilFunction;
		s_uiStencilRef		= uiRef;
		s_uiStencilFunctionMask		= uiMask;

		cellGcmSetStencilFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ( u_int )s_eStencilFunction, s_uiStencilRef, s_uiStencilFunctionMask );
	}
}

void Asura_PS3_RenderingStates::SetStencilOperation( ASURA_PS3_STENCIL_OPERATION eStencilFail, ASURA_PS3_STENCIL_OPERATION eDepthFail, ASURA_PS3_STENCIL_OPERATION eDepthPass )
{
	if( s_eStencilFail != eStencilFail || s_eDepthFail != eDepthFail || s_eDepthPass != eDepthPass )
	{
		s_eStencilFail	= eStencilFail;
		s_eDepthFail	= eDepthFail;
		s_eDepthPass	= eDepthPass;

		cellGcmSetStencilOpUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), ( u_int )s_eStencilFail, ( u_int )s_eDepthFail, ( u_int )s_eDepthPass );
	}
}


void Asura_PS3_RenderingStates::SetSurface( CellGcmSurface *pxSurface )
{
	if( s_pxSurface != pxSurface )
	{
		//BW fix a crash caused by dirty render state after FSFX. Generally we don't need to clean up render states 'cause they all get reset, but this one makes it crash
		if ( !SurfaceHasStencil(pxSurface) ) SetStencilTestEnabled( false );

		s_pxSurface = pxSurface;
		//		cellGcmSetSurfaceInline( Asura_PS3_Graphics::GetGcmCurrentContext(), s_pxSurface );
		cellGcmSetSurfaceWindowInline( Asura_PS3_Graphics::GetGcmCurrentContext(), s_pxSurface, CELL_GCM_WINDOW_ORIGIN_TOP, CELL_GCM_WINDOW_PIXEL_CENTER_HALF );
	}
}


void Asura_PS3_RenderingStates::SetViewport( u_int uX, u_int uY, u_int uWidth, u_int uHeight,
											  float fViewportOutputMinZ, float fViewportOutputMaxZ,
											  float fViewportClipMinZ, float fViewportClipMaxZ,
											  bool bScaleToSurface )
{
	if ( s_uViewportX == uX && s_uViewportY == uY && s_uViewportWidth == uWidth && s_uViewportHeight == uHeight &&
		 s_fViewportOutputMinZ == fViewportOutputMinZ && s_fViewportOutputMaxZ == fViewportOutputMaxZ && s_fViewportClipMinZ == fViewportClipMinZ && s_fViewportClipMaxZ == fViewportClipMaxZ &&
	     s_bViewportScaleToSurface == bScaleToSurface )
	{
		return;
	}

	s_uViewportX      = uX;
    s_uViewportY      = uY;
    s_uViewportWidth  = uWidth;
    s_uViewportHeight = uHeight;

    s_fViewportOutputMinZ = fViewportOutputMinZ;
    s_fViewportOutputMaxZ = fViewportOutputMaxZ;
    s_fViewportClipMinZ = fViewportClipMinZ;
    s_fViewportClipMaxZ = fViewportClipMaxZ;
    
    s_bViewportScaleToSurface = bScaleToSurface;
    
    float afViewportScales[4], afViewportOffsets[4];
    

	if ( bScaleToSurface )
	{
		// set scales for the whole buffer, but still scissor to the desired viewport
		//BW it may be better to adjust the WPOS window when we use SetSurface, so WPOS in pixel shaders uses coordinates of the whole buffer, not the viewport.
		//BW but this "overrun" system is working and has fixed all present issues, so just bear this in mind for future problems.
		u_int uTargetWidth;
		u_int uTargetHeight;
		if ( s_pxSurface )
		{
			uTargetWidth = s_pxSurface->width;
			uTargetHeight= s_pxSurface->height;
		}
		else
		{
			// use hardware default values to avoid a crash
			uTargetWidth = 4096;
			uTargetHeight= 4096;
		}
		afViewportScales[0] = uTargetWidth * 0.5f;
		afViewportScales[1] = uTargetHeight * 0.5f;
		afViewportScales[2] = ( fViewportOutputMaxZ - fViewportOutputMinZ ) * 0.5f;
		afViewportScales[3] = 0.0f;
		afViewportOffsets[0] = 0 + afViewportScales[0];
		afViewportOffsets[1] = 0 + afViewportScales[1];
		afViewportOffsets[2] = ( fViewportOutputMaxZ + fViewportOutputMinZ ) * 0.5f;
		afViewportOffsets[3] = 0.0f;
	}
	else
	{
		// set scales for the viewport area
		afViewportScales[0] = s_uViewportWidth * 0.5f;
		afViewportScales[1] = s_uViewportHeight * 0.5f;
		afViewportScales[2] = ( fViewportOutputMaxZ - fViewportOutputMinZ ) * 0.5f;
		afViewportScales[3] = 0.0f;
		afViewportOffsets[0] = s_uViewportX + afViewportScales[0];
		afViewportOffsets[1] = s_uViewportY + afViewportScales[1];
		afViewportOffsets[2] = ( fViewportOutputMaxZ + fViewportOutputMinZ ) * 0.5f;
		afViewportOffsets[3] = 0.0f;
	}

	cellGcmSetViewportUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(),
                                    s_uViewportX, s_uViewportY,
                                    s_uViewportWidth, s_uViewportHeight,
                                    s_fViewportClipMinZ, s_fViewportClipMaxZ,
                                    afViewportScales,
                                    afViewportOffsets );

	cellGcmSetScissorUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(),
                                   s_uViewportX, s_uViewportY,
                                   s_uViewportWidth, s_uViewportHeight );
}


void Asura_PS3_RenderingStates::RequestWireFrameMode( bool bWireFrameMode )
{
	if( bWireFrameMode != s_bWireFrameMode )
	{
		s_bWireFrameMode = bWireFrameMode;
		if( s_bWireFrameMode )
		{
			cellGcmSetFrontPolygonModeUnsafeInline	( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_POLYGON_MODE_LINE );
			cellGcmSetBackPolygonModeUnsafeInline		( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_POLYGON_MODE_LINE );
		}
		else
		{
			cellGcmSetFrontPolygonModeUnsafeInline	( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_POLYGON_MODE_FILL );
			cellGcmSetBackPolygonModeUnsafeInline		( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_POLYGON_MODE_FILL );
		}
	}
}

void Asura_PS3_RenderingStates::RequestZBufferEnabled( bool bZBufferEnabled )
{
	if( bZBufferEnabled != s_bZBufferEnabled )
	{
		s_bZBufferEnabled = bZBufferEnabled;
		cellGcmSetDepthTestEnableUnsafeInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), s_bZBufferEnabled ? CELL_GCM_TRUE : CELL_GCM_FALSE );
	}
}

void Asura_PS3_RenderingStates::SetSRGBWriteEnabled( bool bEnabled )
{
	if( bEnabled != s_bSRGBWriteEnabled )
	{
		s_bSRGBWriteEnabled = bEnabled;
		cellGcmSetFragmentProgramGammaEnableUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), s_bSRGBWriteEnabled ? CELL_GCM_TRUE : CELL_GCM_FALSE );
	}
}

void Asura_PS3_RenderingStates::RequestZBufferCompareMode( enum ASURA_ZBUFFERCMP_ID eMode )
{
	if( s_eZBufferCompareMode != eMode )
	{
		s_eZBufferCompareMode = eMode;

		switch( s_eZBufferCompareMode )
		{
		case ASURA_ZBUFFERCMP_LESSEQUAL:
			{
				cellGcmSetDepthFuncUnsafeInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_LEQUAL );
				break;		
			}
		case ASURA_ZBUFFERCMP_ALWAYS:
			{
				cellGcmSetDepthFuncUnsafeInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_ALWAYS );
				break;		
			}
		case ASURA_ZBUFFERCMP_EQUALS:
			{
				cellGcmSetDepthFuncUnsafeInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_EQUAL );
				break;
			}
		case ASURA_ZBUFFERCMP_GREATER:
			{
				cellGcmSetDepthFuncUnsafeInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_GREATER );
				break;		
			}
		}		// End of switch.
	}
}

void Asura_PS3_RenderingStates::RequestZBufferWriteEnabled( bool bZBufferWriteEnabled )
{
	if( s_bZBufferWriteEnabled != bZBufferWriteEnabled )
	{
		s_bZBufferWriteEnabled = bZBufferWriteEnabled;
		cellGcmSetDepthMaskUnsafeInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), s_bZBufferWriteEnabled ? CELL_GCM_TRUE : CELL_GCM_FALSE );
	}
}

void Asura_PS3_RenderingStates::SetAlphaBlending( bool bAlphaBlending )
{
	if( s_bAlphaBlending != bAlphaBlending )
	{
		s_bAlphaBlending = bAlphaBlending;
		cellGcmSetBlendEnableUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), s_bAlphaBlending ? CELL_GCM_TRUE : CELL_GCM_FALSE );
	}
}

void Asura_PS3_RenderingStates::RequestTranslucencyMode( enum ASURA_TRANSLUCENCY_MODE eMode )
{
	// do this before the if, since it is an independent state.
	const bool	 bAlphaBlending	= !( ASURA_TRANSLUCENCY_OFF == eMode || ASURA_TRANSLUCENCY_NOT_SET == eMode );
	SetAlphaBlending( bAlphaBlending );

	if( s_eTranslucencyMode != eMode )
	{
		s_eTranslucencyMode = eMode;

		// Which translucency mode do we wish to set?
		switch( s_eTranslucencyMode )
		{
		case ASURA_TRANSLUCENCY_OFF:
			{
				break;
			}
		case ASURA_TRANSLUCENCY_NORMAL:
			{
				cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA );
				break;
			}
		case ASURA_TRANSLUCENCY_COLOUR:
			{
				cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_DST_COLOR, CELL_GCM_ZERO, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA );
				break;
			}
		case ASURA_TRANSLUCENCY_DOUBLECOLOUR:
			{
				cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_DST_COLOR, CELL_GCM_SRC_COLOR, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA );
				break;
			}
		case ASURA_TRANSLUCENCY_INVCOLOUR:
			{
				cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_ZERO, CELL_GCM_ONE_MINUS_SRC_COLOR, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA );
				break;
			}
		case ASURA_TRANSLUCENCY_GLOWING:
			{
				cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_SRC_ALPHA, CELL_GCM_ONE, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA );
				break;
			}
		case ASURA_TRANSLUCENCY_DARKENINGCOLOUR:
			{
				cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_ONE_MINUS_DST_COLOR, CELL_GCM_ZERO, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA );
				break;
			}
		case ASURA_TRANSLUCENCY_JUSTSETZ:
			{
				cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_ZERO, CELL_GCM_ONE, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA );
				break;
			}
		case ASURA_TRANSLUCENCY_ALPHACOMPARE:
			{
				cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_DST_ALPHA, CELL_GCM_ZERO, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA );
				break;
			}
		case ASURA_TRANSLUCENCY_UGH:
			{
				cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_DST_COLOR, CELL_GCM_ONE, CELL_GCM_SRC_ALPHA, CELL_GCM_ONE_MINUS_SRC_ALPHA );
				break;
			}
		default:
			{
				break;
			}
		}		// End of switch.
	}
}

void Asura_PS3_RenderingStates::RequestAlphaTest( bool bAlphaTest, int iRefValue )
{
	SetAlphaTestRefValue( iRefValue );
	if( bAlphaTest != s_bAlphaTest )
	{
		if( bAlphaTest )
		{
			if( Asura_PS3_DeferredRendering::IsBindAsSingleRT() )
			{
				cellGcmSetAlphaTestEnableUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_TRUE );
			}
			else
			{
				Asura_Assert( false, "Alpha testing is not available when using multiple render targets" );
			}
		}
		else
		{
			cellGcmSetAlphaTestEnableUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_FALSE );
		}
		s_bAlphaTest = bAlphaTest;
	}
}

void Asura_PS3_RenderingStates::SetAlphaTestRefValue( int iRefValue )
{
	if( iRefValue != s_iAlphaTestRef )
	{
		s_iAlphaTestRef = iRefValue;
		if( s_iAlphaTestRef == 0)
		{
			cellGcmSetAlphaFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_ALWAYS, s_iAlphaTestRef );
		}
		else
		{
			cellGcmSetAlphaFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_GREATER, s_iAlphaTestRef );
		}
	}
}

void Asura_PS3_RenderingStates::SetCullFace( bool bCullFace )
{
	if( s_bCullFace != bCullFace )
	{
		s_bCullFace = bCullFace;
		cellGcmSetCullFaceEnableUnsafeInline ( Asura_PS3_Graphics::GetGcmCurrentContext(), s_bCullFace ? CELL_GCM_TRUE : CELL_GCM_FALSE );
	}
}

void Asura_PS3_RenderingStates::RequestCullMode( enum ASURA_CULLMODE_ID eMode )
{
	if( s_eCullMode != eMode )
	{
		s_eCullMode = eMode;
		switch( eMode )
		{
		case ASURA_CULLMODE_CW:
			{
				// Ensure that cull-mode is enabled...
				SetCullFace( true );
				// Are we to mirror the image?
				if (Asura_Render::ShouldRenderMirrored())
				{
					cellGcmSetCullFaceUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_FRONT );
				}
				else
				{
					cellGcmSetCullFaceUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_BACK );
				}
				break;
			}
		case ASURA_CULLMODE_CCW:
			{
				// Ensure that cull-mode is enabled...
				SetCullFace( true );

				// Are we to mirror the image?
				if (Asura_Render::ShouldRenderMirrored())
				{
					cellGcmSetCullFaceUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_BACK );
				}
				else
				{
					cellGcmSetCullFaceUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_FRONT );
				}
				break;
			}
		default:
			{
				// Diable cull-mode...
				SetCullFace( false );
				break;
			}
		}		// End of switch.
	}
}

void Asura_PS3_RenderingStates::SetRestartIndex( u_int uiRestartIndex )
{
	if( uiRestartIndex != s_uiRestartIndex )
	{
		s_uiRestartIndex = uiRestartIndex;
		cellGcmSetRestartIndexUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), s_uiRestartIndex );
	}
}

bool Asura_PS3_RenderingStates::SetTextureOffsetNeeded( u_char ucTextureIndex, u_int uTextureOffset )
{
	if( s_uiTextureOffset[ ucTextureIndex ] == uTextureOffset ) return false;
	s_uiTextureOffset[ ucTextureIndex ] = uTextureOffset;
	return true;
}

bool Asura_PS3_RenderingStates::SetTextureFormatNeeded( u_char ucTextureIndex, u_int uTextureFormat )
{
	if( s_uiTextureFormat[ ucTextureIndex ] == uTextureFormat ) return false;
	s_uiTextureFormat[ ucTextureIndex ] = uTextureFormat;
	return true;
}

bool Asura_PS3_RenderingStates::SetTextureAddressNeeded( u_char ucTextureIndex, u_int uTextureAddress )
{
	if( s_uiTextureAddress[ ucTextureIndex ] == uTextureAddress ) return false;
	s_uiTextureAddress[ ucTextureIndex ] = uTextureAddress;
	return true;
}

bool Asura_PS3_RenderingStates::SetTextureControl0Needed( u_char ucTextureIndex, u_int uTextureControl0 )
{
	if( s_uiTextureControl0[ ucTextureIndex ] == uTextureControl0 ) return false;
	s_uiTextureControl0[ ucTextureIndex ] = uTextureControl0;
	return true;
}

bool Asura_PS3_RenderingStates::SetTextureControl1Needed( u_char ucTextureIndex, u_int uTextureControl1 )
{
	if( s_uiTextureControl1[ ucTextureIndex ] == uTextureControl1 ) return false;
	s_uiTextureControl1[ ucTextureIndex ] = uTextureControl1;
	return true;
}

bool Asura_PS3_RenderingStates::SetTextureControl2Needed( u_char ucTextureIndex, u_int uTextureControl2 )
{
	if( s_uiTextureControl2[ ucTextureIndex ] == uTextureControl2 ) return false;
	s_uiTextureControl2[ ucTextureIndex ] = uTextureControl2;
	return true;
}

bool Asura_PS3_RenderingStates::SetTextureControl3Needed( u_char ucTextureIndex, u_int uTextureControl3 )
{
	if( s_uiTextureControl3[ ucTextureIndex ] == uTextureControl3 ) return false;
	s_uiTextureControl3[ ucTextureIndex ] = uTextureControl3;
	return true;
}

bool Asura_PS3_RenderingStates::SetTextureFilterNeeded( u_char ucTextureIndex, u_int uTextureFilter )
{
	if( s_uiTextureFilter[ ucTextureIndex ] == uTextureFilter ) return false;
	s_uiTextureFilter[ ucTextureIndex ] = uTextureFilter;
	return true;
}

bool Asura_PS3_RenderingStates::SetTextureRectNeeded( u_char ucTextureIndex, u_int uTextureRect )
{
	if( s_uiTextureImageRect[ ucTextureIndex ] == uTextureRect ) return false;
	s_uiTextureImageRect[ ucTextureIndex ] = uTextureRect;
	return true;
}

void Asura_PS3_RenderingStates::SetLineWidth( float fLineWidth )
{
	int iLineWidth = (int)(fLineWidth*8.0f);//line width is set using a fixed-point 0:6:3
	if ( s_iLineWidth != iLineWidth )
	{
		s_iLineWidth = iLineWidth;
		cellGcmSetLineWidthUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), iLineWidth );
	}
}
