///////////////////////////////////////////////////////////////////////////////
//
// File		: Asura_PS3_VideoMemoryManager.cpp
// Author	: Catalin Arsenescu (03-April-2007)
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include <sdk_version.h>

#if CELL_SDK_VERSION < 0x240000
#include <memory.h>
#else
#include <sys/memory.h>
#endif 

#include <stdlib.h>

// This file's header...
#include "Rendering/Asura_PS3_VideoMemoryManager.h"

// Asura independent header-files...
#include "Debugging/Asura_Assert.h"
#include "Core/Memory/Asura_MemAllocator.h"
#include "Core/Asura_Platform_MemoryManagement.h"

////////////////////////////////////////////////////////////////////////////////
// constants
////////////////////////////////////////////////////////////////////////////////
#ifndef ASURA_PS3_INITIAL_VRAM_ALLOC_RECORDS
	#define ASURA_PS3_INITIAL_VRAM_ALLOC_RECORDS 10000
#endif //#ifndef ASURA_PS3_INITIAL_VRAM_ALLOC_RECORDS

static const u_int uASURA_PS3_VRAM_MIN_ALIGNMENT = 128;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////
Asura_MemAllocator* Asura_PS3_VideoMemoryManager::s_pxMemAllocatorLocal = NULL;
unsigned char Asura_PS3_VideoMemoryManager::s_ucLocation				= 0xff;

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_VideoMemoryManager::Initialise
// Author		: Catalin Arsenescu (03-April-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_VideoMemoryManager::Initialise( void* pLocalAddress, u_int uLocalMemorySize )
{
	s_pxMemAllocatorLocal = new Asura_MemAllocator(pLocalAddress, uLocalMemorySize, uASURA_PS3_VRAM_MIN_ALIGNMENT, ASURA_PS3_INITIAL_VRAM_ALLOC_RECORDS);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_VideoMemoryManager::Shutdown
// Author		: Catalin Arsenescu (03-April-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_VideoMemoryManager::Shutdown( void )
{
	delete s_pxMemAllocatorLocal;

	s_pxMemAllocatorLocal	= NULL;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_VideoMemoryManager::AllocLocal
// Author		: Catalin Arsenescu (03-April-2007)
////////////////////////////////////////////////////////////////////////////////

#if defined(ASURA_MEMORYMANAGEMENT_ENABLED)
void* Asura_PS3_VideoMemoryManager::AllocLocal( const u_int uSize, const u_int uAlignment, const char* szFile, const u_int uLine, const u_int uReturnAddr )
{
	if (s_pxMemAllocatorLocal)
	{
		return s_pxMemAllocatorLocal->Allocate( uSize, uAlignment );
	}
	return 0;
}
#else
void* Asura_PS3_VideoMemoryManager::AllocLocal( const u_int uSize, const u_int uAlignment )
{
	if (s_pxMemAllocatorLocal)
	{
		return s_pxMemAllocatorLocal->Allocate( uSize, uAlignment );
	}
	return 0;
}
#endif	//defined(ASURA_MEMORYMANAGEMENT_ENABLED)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_VideoMemoryManager::FreeLocal
// Author		: Catalin Arsenescu (03-April-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_VideoMemoryManager::FreeLocal( void* pAddress )
{	
	if (s_pxMemAllocatorLocal)
	{
		return s_pxMemAllocatorLocal->Free( pAddress );
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_VideoMemoryManager::FreeMain
// Author		: Catalin Arsenescu (03-April-2007)
////////////////////////////////////////////////////////////////////////////////
void Asura_PS3_VideoMemoryManager::FreeMain( void* pAddress )
{
	return free( pAddress );
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_VideoMemoryManager::GetFreeMain
// Author		: Catalin Arsenescu (03-April-2007)
////////////////////////////////////////////////////////////////////////////////
u_int Asura_PS3_VideoMemoryManager::GetFreeMain( void )
{
	return Asura_PS3_GetFreeMainMemory();
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_PS3_VideoMemoryManager::GetFreeLocal
// Author		: Catalin Arsenescu (03-April-2007)
////////////////////////////////////////////////////////////////////////////////
u_int Asura_PS3_VideoMemoryManager::GetFreeLocal( void )
{
	u_int uFree = s_pxMemAllocatorLocal->GetFreeSpace();
	return uFree;
}
