///////////////////////////////////////////////////////////////////////////////
//
// File		: Asura_PS3_VideoMemoryManager.h
// Author	: Catalin Arsenescu (03-April-2007)
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PS3_VIDE_MEMORY_MANAGER_H__
#define __ASURA_PS3_VIDE_MEMORY_MANAGER_H__

// I N C L U D E S ///////////////////////////////////////////////////////////
#include <stdlib.h>

#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_PS3_Graphics.h"

class Asura_MemAllocator;


// C L A S S E S /////////////////////////////////////////////////////////////
class Asura_PS3_VideoMemoryManager
{
public:
	//The first two parameters can be obtain using cellGcmGetConfiguration()
	//All the addresses must be aligned to 1 mega byte
	static void		Initialise	( void* pLocalAddress, u_int uLocalMemorySize );
    static void		Shutdown	( void );

#if defined(ASURA_MEMORYMANAGEMENT_ENABLED)
	static void*	AllocLocal	( const u_int uSize, const u_int uAlignment, const char* szFile, const u_int uLine, const u_int uReturnAddr );
#else
	static void*	AllocLocal	( const u_int uSize, const u_int uAlignment );
#endif	//defined(ASURA_MEMORYMANAGEMENT_ENABLED)

	static void		FreeLocal	( void* pAddress );
	static void		FreeMain	( void* pAddress );

	static u_int	GetFreeMain	( void );
	static u_int	GetFreeLocal( void );

	//	PAZ - override the memory pool to use for textures and geometry - either VRAM or main memory.
	//	An example...
	//		We load a menu to display whilst loading a level, which allocates 5MB of VRAM
	//		We then load a level, which uses up the remaining VRAM and some main memory
	//		At this point, we free off the menu, which gives us the 5MB of VRAM back
	//		However, this VRAM is never used as we have already loaded all of our game textures and geometry...
	//		...and 5MB of main memory has been used for textures that could of otherwise been stored in VRAM
	//
	//	However, if you override so that the menu textures are loaded into main memory, this memory can be reused once freed.
	//
	static void		UseLocalLocation( void )	{ s_ucLocation = CELL_GCM_LOCATION_LOCAL; }
	static void		UseMainLocation( void )		{ s_ucLocation = CELL_GCM_LOCATION_MAIN; }
	static void		UseDefaultLocation( void )	{ s_ucLocation = 0xff; }
	static bool		OverrideLocation( unsigned char &ucLocation )
	{ 
		if( ( s_ucLocation == CELL_GCM_LOCATION_LOCAL ) || ( s_ucLocation == CELL_GCM_LOCATION_MAIN ) )
		{
			ucLocation = s_ucLocation;
			return( true );
		}
		return( false );
	}
	
	static bool		IsVramAddress( void* pxAddress ) {
		// On PS3 VRAM addresses are in 0xC0000000 to 0xCFFFFFFF
		u_int uA = reinterpret_cast<uint32_t>(pxAddress);
		u_int uB = uA>>28;
		return (uB == 0xC);
	}
	
protected:
private:
	static Asura_MemAllocator*	s_pxMemAllocatorLocal;
	static unsigned char s_ucLocation;

	Asura_PS3_VideoMemoryManager();
	~Asura_PS3_VideoMemoryManager();
};

// Macros used to hide the file/line storage...
#if defined(ASURA_MEMORYMANAGEMENT_ENABLED)
#define ASURA_PS3_VIDEOMEMORY_ALLOCLOCAL(uSize, uAlignment)	Asura_PS3_VideoMemoryManager::AllocLocal(uSize, uAlignment, __FILE__, __LINE__, ASURA_PLATFORM_MEMORYMANAGEMENT_GETRETURNADDR())
#else
#define ASURA_PS3_VIDEOMEMORY_ALLOCLOCAL(uSize, uAlignment)	Asura_PS3_VideoMemoryManager::AllocLocal(uSize, uAlignment)
#endif	//defined(ASURA_MEMORYMANAGEMENT_ENABLED)
#define ASURA_PS3_VIDEOMEMORY_ALLOCMAIN(uSize, uAlignment)	memalign( uAlignment, uSize ) // don't call Asura_AllocateAlign, because that cannot support the large alignments needed for frame buffers. In most cases memalign is mapped to that anyway.

#endif //#ifndef __ASURA_PS3_VIDE_MEMORY_MANAGER_H__
