//
// Asura_PS3_Shaders.cpp
//
// Created by KJFL - 22 August 2005
// Modified by Catalin Arsenescu - 29 March 2007
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Engine independent.
#include "Core/Asura_Multithread.h"
#include "Debugging/Asura_Assert.h"

// Engine platform-specific.
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_RenderingStates.h"

static void TreeReset( Asura_Hash_ID uHash, Asura_PS3_FP_List** pxFP_List )
{
	delete (*pxFP_List);
	(*pxFP_List) = NULL;
}

static void TreeEndFrame( Asura_Hash_ID uHash, Asura_PS3_FP_List** pxFP_List )
{
	(*pxFP_List)->EndFrame();
}

void Asura_PS3_FP_List_Tree::Shutdown( void )
{
	m_xTree.Traverse( TreeReset );
	m_xTree.Reset();
}

void Asura_PS3_FP_List_Tree::EndFrame( void )
{
	m_xTree.Traverse( TreeEndFrame );
}

Asura_PS3_FragmentShaderCopy* Asura_PS3_FP_List_Tree::GetShader( Asura_Hash_ID uHash )
{
	Asura_PS3_FP_List** fpList = m_xTree.GetDataByHash( uHash );

	if( fpList )
	{
		return ( *fpList )->GetShader();
	}

	Asura_Assertf( false, ( "Shader hash ID 0x%08x not found!", uHash ) );
	return NULL;
}
