
///////////////////////////////////////////////////////////////////////////////
//
// File   : SFX_PFX.fx
// Author : Catalin Arsenescu
// Date   : 02-June-2008
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include "Shaders/Project_ShaderConfiguration.fxh"
#include "MaterialResponse_Shared.ps3FXH"

////////////////////////////////////////////////////////////////////////////////
// Uniform variables...
////////////////////////////////////////////////////////////////////////////////

#ifdef USE_SOFT_PARTICLE
const sampler2D g_xDepthBufferSampler				: register( s2 );
#endif

const sampler2D g_xDiffuseMap						: register( s3 );

#ifdef USE_HAZE
const sampler2D	g_xHeatHazeCaptureTexture			: register( s4 );
const float2	g_xOneOverScreenDimensions;
#endif

////////////////////////////////////////////////////////////////////////////////
// Input structure...
////////////////////////////////////////////////////////////////////////////////

struct VS_Output
{
	float4	UV_Pair		: TEXCOORD0;
	float3	FrameInfo	: TEXCOORD1;
	float4	Diffuse		: TEXCOORD2;
};

////////////////////////////////////////////////////////////////////////////////
// Main function...
////////////////////////////////////////////////////////////////////////////////

half4 main ( VS_Output IN
#if defined USE_SOFT_PARTICLE || defined USE_HAZE
			, float2 xScreenPos : WPOS
#endif
			) : COLOR0
{
	half4 xColour;
    half4 xTexture1	= h4tex2D( g_xDiffuseMap, IN.UV_Pair.xy );

#ifdef USE_CROSS_FADE_TEXTURE
    half4 xTexture2	= h4tex2D( g_xDiffuseMap, IN.UV_Pair.zw );
    half4 xTexture	= lerp( xTexture2, xTexture1, IN.FrameInfo.x );
#else
    half4 xTexture	= xTexture1;
#endif

#ifdef USE_HAZE
	xColour	= xTexture;
	xColour.a *= IN.Diffuse.a;
	const half2 xNormalizedCoords = ( xScreenPos.xy ) * g_xOneOverScreenDimensions.xy;

	const half2 xOffset = ( IN.Diffuse.rg * ( xColour.rg - 0.5h ) ) / ( 10.0h + IN.FrameInfo.y );

	// Do the offset lookup from the frame capture
	xColour.rgb = 0.25h *
				  (
				  GetScreenColour( g_xHeatHazeCaptureTexture, xNormalizedCoords + 0.3h * xOffset ).rgb +
				  GetScreenColour( g_xHeatHazeCaptureTexture, xNormalizedCoords + 0.6h * xOffset ).rgb +
				  GetScreenColour( g_xHeatHazeCaptureTexture, xNormalizedCoords + 0.8h * xOffset ).rgb +
				  GetScreenColour( g_xHeatHazeCaptureTexture, xNormalizedCoords +        xOffset ).rgb
				  );
#else
    // Calculate the output colour...
    xColour = xTexture * IN.Diffuse;
#endif

#ifdef USE_SOFT_PARTICLE
	const half	xDepth		= Deferred_GetDepth( h4tex2D( g_xDepthBufferSampler, xScreenPos ) );
	const float fSoftValue	= min( abs( xDepth - IN.FrameInfo.z ) / IN.FrameInfo.y, 1.0f );
	xColour.a *= fSoftValue;
#endif

	return ApplyFog( xColour, IN.FrameInfo.z );
}

/// End of file. ///
