///////////////////////////////////////////////////////////////////////////////
//
// File      : Asura_PS3_Sound_Loader.h
// Author    : Philip Hall
// Created   : 11/09/2007
//
// Copyright (C) 2007 Rebellion. All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __ASURA_PS3_SOUND_LOADER_H__
#define __ASURA_PS3_SOUND_LOADER_H__

#ifndef ASURA_USE_AUDIO_SYSTEM

// Asura Headers
#include "Debugging/Asura_Assert.h"

// Asura PS3 Headers
#include "FileAccess/Asura_PS3_FileSystem.h"


////PGH/////////////////////////////////////////
// Sample loader sound information           
////////////////////////////////////////////////
// (MH - 27.03.2008) reduced size as mush as I could. With 2750 loaded sounds, 
// and the structure using 36 bytes, that came to nearly 99kB! Now 8 bytes => more like 22kB total.
class Asura_PS3_Sound_Info
{
public:
	Asura_PS3_Sound_Info()
	{
		m_usMP3FrameSize = 0;	
		m_usSampleRate = 0;		
		m_uSampleFormat = 0;	
		m_uChannelCount = 0;	
		m_uTotalDataBytes = 0;
	}
	
	int GetSamplesPerMP3Frame() const { return m_bSamplesPerMP3FrameIs576 ? 576 : 1152; }
	void SetSamplesPerMP3Frame(int iValue) 
	{ 
		Asura_Assert(iValue == 1152 || iValue == 576, "unexpected samples per MP3 frame");
		m_bSamplesPerMP3FrameIs576 = static_cast<u_short>(iValue == 576);
	}

	int GetMP3FrameSize() const { return static_cast<int>(m_usMP3FrameSize); }
	void SetMP3FrameSize(int iValue)
	{
		Asura_Assert(iValue >= 0 && iValue < (1 << 15), "unexpected MP3FrameSize");
		m_usMP3FrameSize = static_cast<u_short>(iValue);
	}

	int GetSampleRate() const { return static_cast<int>(m_usSampleRate); }
	void SetSampleRate(int iValue)
	{
		Asura_Assert(iValue >= 0 && iValue < 65536, "unexpected sample rate");
		m_usSampleRate = static_cast<u_short>(iValue);
	}

	int GetSampleFormat() const { return static_cast<int>(m_uSampleFormat); }
	void SetSampleFormat(int iValue)
	{
		Asura_Assert(iValue >= 0 && iValue < (1<<3), "unexpected sample format");
		m_uSampleFormat = static_cast<u_int>(iValue);
	}

	int GetSampleBits() const { return 16; }
	void SetSampleBits(int iValue) { Asura_Assert(iValue == 16, "unexpected bits per sample"); }

	int GetChannelCount() const { return static_cast<int>(m_uChannelCount); }
	void SetChannelCount(int iValue)
	{
		Asura_Assert(iValue >= 0 && iValue < (1<<3), "unexpected channel count");
		m_uChannelCount = static_cast<u_int>(iValue);
	}

	int GetLoopOffset() const { return 0; }
	void SetLoopOffset(int iValue) { Asura_Assert(iValue == 0, "unexpected loop offset"); }

	int GetDataOffset() const { return 64; }
	void SetDataOffset(int iValue) { Asura_Assert(iValue == 64, "unexpected data offset"); }

	int GetTotalDataBytes() const { return static_cast<int>(m_uTotalDataBytes); }
	void SetTotalDataBytes(int iValue)
	{
		Asura_Assert(iValue >= 0 && iValue < (1<<26), "unexpected channel count");
		m_uTotalDataBytes = static_cast<u_int>(iValue);
	}

private:

	u_short m_bSamplesPerMP3FrameIs576;	// 576 or 1152.
	u_short	m_usMP3FrameSize:15;	// packet size. dependent on bit-rate. Typically around 420.
	u_short	m_usSampleRate;			// 0-48000 (or a bit more once we've accounted for looping MP3s adjusting sample rate)
	u_int	m_uSampleFormat:3;		// 0-7
	//int	m_iSampleBits;			// always 16 (could have 8-bit WAVs I suppose...)
	u_int	m_uChannelCount:3;		// 0-7
	//int	m_iLoopOffset;			// always 0 at the moment? almost certainly < 65536
	//int	m_iDataOffset;			// probably always 64 for MSF (could be 48 for VAG)
	u_int	m_uTotalDataBytes:26;
};


////////////////////////////////////////////
// PS3 Wave loader (mostly generic)
////////////////////////////////////////////
class Asura_PS3_Sound_Loader
{
public:
	static bool LoadAuto(char* pcBuffer, int iBufferSize, char*& pSamples, Asura_PS3_Sound_Info& xSoundInfo);
	static bool LoadFileInfoAuto(Asura_PS3_FileSystem::Handle xFileHandle, Asura_PS3_Sound_Info& xSoundInfo);
	
	static bool LoadMSFInfo(void* pBuffer, int iBufferSize, Asura_PS3_Sound_Info& xSoundInfo);
// (MH - 27.03.2008) only supporting MSF now.
//	static bool LoadWAVInfo(char* pcBuffer, int iBufferSize, Asura_PS3_Sound_Info& xSoundInfo);
//	static bool LoadVAGInfo(char* pcBuffer, int iBufferSize, Asura_PS3_Sound_Info& xSoundInfo);

private:
	static int  FindChunkInData(const char* pcData, int iDataSize, const char *szChunkName);
};


#endif // #ifndef ASURA_USE_AUDIO_SYSTEM

#endif

//eof
