//////////////////////////////////////////////////////////////////////////////
//
// Asura_PS3_MsgBox.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     Wrapper Class for PS3 Utilities Message display
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Utility/Asura_PS3_MsgBox.h"

#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"

bool Asura_PS3_MsgBox::s_bDisplaying = false;
bool Asura_PS3_MsgBox::s_bLastQuestionAnswer = false;
char Asura_PS3_MsgBox::s_szMessage[CELL_MSGDIALOG_STRING_SIZE] = "";

bool Asura_PS3_MsgBox::DisplaySonyError(u_int uErrorCode)
{
	if (s_bDisplaying)
	{
		return false;
	}
	if (cellMsgDialogOpenErrorCode(uErrorCode, CallBack, 0, 0) == 0)
	{
		s_bDisplaying = true;
	}
	return s_bDisplaying;
}

bool Asura_PS3_MsgBox::DisplayMessage(const Asura_UnicodeChar* wszMessage, bool bOKButton /*= true*/, START_SOUND eSound /*= START_SOUND_NORMAL*/)
{
	if (s_bDisplaying)
	{
		return false;
	}
	u_int uFlags = CELL_MSGDIALOG_TYPE_BG_INVISIBLE
		| CELL_MSGDIALOG_TYPE_PROGRESSBAR_NONE;
	if (bOKButton)
	{
		uFlags |=  CELL_MSGDIALOG_TYPE_BUTTON_TYPE_OK
			| CELL_MSGDIALOG_TYPE_DISABLE_CANCEL_ON
			| CELL_MSGDIALOG_TYPE_DEFAULT_CURSOR_OK;
	}
	else
	{
		uFlags |=  CELL_MSGDIALOG_TYPE_BUTTON_TYPE_NONE
			| CELL_MSGDIALOG_TYPE_DISABLE_CANCEL_OFF
			| CELL_MSGDIALOG_TYPE_DEFAULT_CURSOR_NONE;
	}
	switch (eSound)
	{
		case START_SOUND_NONE:
		{
			uFlags |= CELL_MSGDIALOG_TYPE_SE_TYPE_NORMAL | CELL_MSGDIALOG_TYPE_SE_MUTE_ON;
			break;
		}
		case START_SOUND_NORMAL:
		{
			uFlags |= CELL_MSGDIALOG_TYPE_SE_TYPE_NORMAL | CELL_MSGDIALOG_TYPE_SE_MUTE_OFF;
			break;
		}
		case START_SOUND_ERROR:
		{
			uFlags |= CELL_MSGDIALOG_TYPE_SE_TYPE_ERROR | CELL_MSGDIALOG_TYPE_SE_MUTE_OFF;
			break;
		}
	}
	Asura_UnicodeToUTF8(wszMessage, s_szMessage, CELL_MSGDIALOG_STRING_SIZE);
	if (cellMsgDialogOpen2(uFlags, s_szMessage, CallBack, 0, 0) == 0)
	{
		s_bDisplaying = true;
	}

	return s_bDisplaying;
}

bool Asura_PS3_MsgBox::DisplayQuestion(const Asura_UnicodeChar* wszMessage, bool bHighlightYes /*= false*/, START_SOUND eSound /*= START_SOUND_NORMAL*/)
{
	if (s_bDisplaying)
	{
		return false;
	}
	u_int uFlags = CELL_MSGDIALOG_TYPE_BG_INVISIBLE
		| CELL_MSGDIALOG_TYPE_BUTTON_TYPE_YESNO
		| CELL_MSGDIALOG_TYPE_DISABLE_CANCEL_ON
		| CELL_MSGDIALOG_TYPE_PROGRESSBAR_NONE;
	if (bHighlightYes)
	{
		uFlags |= CELL_MSGDIALOG_TYPE_DEFAULT_CURSOR_YES;
	}
	else
	{
		uFlags |= CELL_MSGDIALOG_TYPE_DEFAULT_CURSOR_NO;
	}

	switch (eSound)
	{
		case START_SOUND_NONE:
		{
			uFlags |= CELL_MSGDIALOG_TYPE_SE_TYPE_NORMAL | CELL_MSGDIALOG_TYPE_SE_MUTE_ON;
			break;
		}
		case START_SOUND_NORMAL:
		{
			uFlags |= CELL_MSGDIALOG_TYPE_SE_TYPE_NORMAL | CELL_MSGDIALOG_TYPE_SE_MUTE_OFF;
			break;
		}
		case START_SOUND_ERROR:
		{
			uFlags |= CELL_MSGDIALOG_TYPE_SE_TYPE_ERROR | CELL_MSGDIALOG_TYPE_SE_MUTE_OFF;
			break;
		}
	}
	Asura_UnicodeToUTF8(wszMessage, s_szMessage, CELL_MSGDIALOG_STRING_SIZE);
	if (cellMsgDialogOpen2(uFlags, s_szMessage, CallBack, &s_bLastQuestionAnswer, 0) == 0)
	{
		s_bLastQuestionAnswer = false;
		s_bDisplaying = true;
	}

	return s_bDisplaying;
}

void Asura_PS3_MsgBox::CallBack(int iButtonType, void* pUserData)
{
	Asura_Assert(s_bDisplaying, "callback fired with no message displaying?");
	s_bDisplaying = false;
	if (pUserData)
	{
		bool* pbAnswer = static_cast<bool*>(pUserData);
		switch (iButtonType)
		{
			case CELL_MSGDIALOG_BUTTON_YES:
			{
				*pbAnswer = true;
				break;
			}
			case CELL_MSGDIALOG_BUTTON_NO:
			{
				*pbAnswer = false;
				break;
			}
			default:
			{
				Asura_Assert(0, "Unexpected button");
				break;
			}
		}
	}
}
