//
// Asura_Platform_Visibility_System.cpp
//
// Created by KJFL - 06 May 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#if defined(ASURA_USE_VISIBILITY_SYSTEM)

// Stdlib platform-specific.

// Engine independent.
#include "Visibility/Asura_Visibility_System.h"

// Engine platform-specific.
#include "Rendering/Asura_PS3_AABBRenderer.h"
#include "Rendering/Asura_PS3_EnvironmentRenderer.h"


class Asura_PS3_OcclusionQuery
{
	public:

		static Asura_PS3_OcclusionQuery* Create( u_int uNodeIndex )
		{
			Asura_PS3_OcclusionQuery* pxOccQuery = new Asura_PS3_OcclusionQuery;

			GLenum eError = glGetError();
			Asura_Assert( eError == GL_NO_ERROR, "error before glGenQueriesARB");

			glGenQueriesARB( 1, &(pxOccQuery->m_uQueryID) );
			
			eError = glGetError();
			Asura_Assert( eError == GL_NO_ERROR, "error in glGenQueriesARB");
			if (eError != GL_NO_ERROR)
			{
				delete pxOccQuery;
				return NULL;
			}

			pxOccQuery->m_uNodeIndex = uNodeIndex;
			pxOccQuery->m_pxNext = 0;

			return pxOccQuery;
		}

		~Asura_PS3_OcclusionQuery()
		{
			glDeleteQueriesARB( 1, &(m_uQueryID) );
		}

		void Start( void )
		{
			glBeginQueryARB( GL_SAMPLES_PASSED_ARB, m_uQueryID );
		}

		void End( void )
		{
			glEndQueryARB( GL_SAMPLES_PASSED_ARB );
		}

		int GetResult( bool bForceFlush )
		{
			int iResultAvailable = GL_FALSE;
			int iNumberOfPixelsDrawn = -1;
            
			// see if the result of the query is available
			glGetQueryObjectivARB( m_uQueryID, GL_QUERY_RESULT_AVAILABLE_ARB, &iResultAvailable );

			// Ask for the result if it's is available, or if we're happy to force a flush (and probable stall)

		//	if ( iResultAvailable == GL_FALSE && bForceFlush )
		//	{
		//		return 0;
		//	}

			if ( ( iResultAvailable == GL_TRUE ) || bForceFlush ) 
			{
				glGetQueryObjectivARB( m_uQueryID, GL_QUERY_RESULT_ARB, &iNumberOfPixelsDrawn );
			}

			return iNumberOfPixelsDrawn;
		}

	
		Asura_PS3_OcclusionQuery*	m_pxNext;

		GLuint						m_uQueryID;
		u_int						m_uNodeIndex;

};


	
// G L O B A L S /////////////////////////////////////////////////////////////
Asura_PS3_OcclusionQuery* g_pxActiveQueries = 0;
Asura_PS3_OcclusionQuery* g_pxEndOfQueue = 0;
u_int g_uNumberOfActiveQueries = 0;


// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////
void Asura_Visibility_System::Platform_RenderStripZOnly( u_int uStripID )
{
	Asura_PS3_EnvironmentRenderer::RenderStripZOnly( uStripID );
}

/* Function not yet implemented on all platforms
void Asura_Visibility_System::Platform_RenderZPassOfVisibleStrips( void )
{
	Asura_PS3_EnvironmentRenderer::RenderZPassForStrips();
}
*/

void Asura_Visibility_System::Platform_IssueOcclusionQuery( u_int uNodeIndex, const Asura_Bounding_Box& xBB )
{
	Asura_PS3_OcclusionQuery* pxQuery = Asura_PS3_OcclusionQuery::Create( uNodeIndex );

    if (!pxQuery) return;

	pxQuery->Start();
	Asura_PS3_AABBRenderer::Render( xBB );
	pxQuery->End();

#if 0
	pxQuery->m_pxNext = g_pxActiveQueries;
	g_pxActiveQueries = pxQuery;

	g_uNumberOfActiveQueries++;
#else
	g_pxEndOfQueue = g_pxActiveQueries;
	if ( g_pxEndOfQueue )
	{
		while (g_pxEndOfQueue->m_pxNext)
		{
			g_pxEndOfQueue = g_pxEndOfQueue->m_pxNext;
		}

		g_pxEndOfQueue->m_pxNext = pxQuery;
		g_pxEndOfQueue = pxQuery;
	}
	else
	{
		g_pxActiveQueries = pxQuery;
	}

	g_uNumberOfActiveQueries++;
#endif

}

void Asura_Visibility_System::Platform_IssueOcclusionQuery( u_int uNodeIndex, u_int uStripID )
{
	Asura_PS3_OcclusionQuery* pxQuery = Asura_PS3_OcclusionQuery::Create( uNodeIndex );

    if (!pxQuery) return;

	pxQuery->Start();
	Asura_PS3_EnvironmentRenderer::RenderStripZOnly( uStripID );
	pxQuery->End();

#if 0
	pxQuery->m_pxNext = g_pxActiveQueries;
	g_pxActiveQueries = pxQuery;

	g_uNumberOfActiveQueries++;
#else
	g_pxEndOfQueue = g_pxActiveQueries;
	if ( g_pxEndOfQueue )
	{
		while (g_pxEndOfQueue->m_pxNext)
		{
			g_pxEndOfQueue = g_pxEndOfQueue->m_pxNext;
		}

		g_pxEndOfQueue->m_pxNext = pxQuery;
		g_pxEndOfQueue = pxQuery;
	}
	else
	{
		g_pxActiveQueries = pxQuery;
	}

	g_uNumberOfActiveQueries++;
#endif
}

u_int Asura_Visibility_System::Platform_GetNumberOfOutstandingOcclusionQueries( void )
{
	return g_uNumberOfActiveQueries;
}

bool Asura_Visibility_System::Platform_GetOcclusionResults( u_int& uNodeIndex, bool bForceFlush )
{
	Asura_PS3_OcclusionQuery** ppxActiveQueries = &g_pxActiveQueries;

	while (*ppxActiveQueries)
	{
		Asura_PS3_OcclusionQuery* pxThisQuery = (*ppxActiveQueries);

		int iNumberOfPixelsDrawn = pxThisQuery->GetResult( bForceFlush );
		
		// a negative iNumberOfPixelsDrawn means the query hasn't completed yet.
		if ( iNumberOfPixelsDrawn >= 0 )
		{
			uNodeIndex = pxThisQuery->m_uNodeIndex;

			*ppxActiveQueries = (pxThisQuery->m_pxNext);

			delete pxThisQuery;

			--g_uNumberOfActiveQueries;

			if ( iNumberOfPixelsDrawn > 0 )
			{
				return true;
			}
		}
		
		if (*ppxActiveQueries)
		{
			ppxActiveQueries = &((*ppxActiveQueries)->m_pxNext);
		}
		
	}

	return false;
}

#endif	// defined(ASURA_USE_VISIBILITY_SYSTEM)


/// End of file. ///
