#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PS3_DECAL_SYSTEM_H__
#define __ASURA_PS3_DECAL_SYSTEM_H__

#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "SpecialFX/Asura_Decal_Template.h"

#include "Rendering/Asura_PS3_GCM_IndexBuffer.h"
#include "Rendering/Asura_PS3_GCM_VertexBuffer.h"
#include "Core/Asura_HalfFloat.h"

class Asura_PS3_FragmentShaderCopy;


// ----------------------------------------------------------------------------
// Vertex Formats - with and without support for normal mapping
// ----------------------------------------------------------------------------

class Asura_PS3_Decal_Vertex
{
public:
	float			fX,fY,fZ;
	float			fU,fV;	
	float			fCreationTime;
	float			fAlpha;
	uint32			uNormal;
	uint32			uTangent;
	uint32		    uBinormal;
	Asura_HalfFloat xSHRed[4];
	Asura_HalfFloat  xSHGreen[4];
	Asura_HalfFloat  xSHBlue[4];
};



// ----------------------------------------------------------------------------
// Asura_PS3_DecalBuffer
// ----------------------------------------------------------------------------

class Asura_PS3_DecalBuffer : public Asura_DecalBuffer
{
	friend class Asura_Decal_Handler;

public:

	// These are created the first time a decal of this material type is created
	// This isn't ideal - but we can't pre-alloc them as we don't know how many 
	// decal templates there are.
	Asura_PS3_DecalBuffer(const Asura_Decal_Template* pxDecalTemplate);
	~Asura_PS3_DecalBuffer();

	void AddStrip(const Asura_Decal_Vertex* pxStrip, const u_int uNumberOfVertices, const Asura_Decal_Template* pxDecalTemplate, const Asura_RGBA& xRGBA );
	void Render(void);

private:

	class Decal
	{
	public:
		int		iFirstIndex;
		int		iNumIndices;		
		float	fCreationTime;
	};

	int							 m_iMaxDecals;				// Max decals in this buffer
	int							 m_iDecalMaterialIndex;		// Material used by this buffer	
	Decal*						 m_apxDecals;				// Decal structs (circular buffer)
	int							 m_iNextVertex;				// Next vertex to use (verts = another circular buffer)
	int							 m_iNumVerts;
	int							 m_iNumIndices;
	uint16*						 m_apxIndices;				// Indices 
	int							 m_iFirstDecal;				// First decal struct in use
	int							 m_iNumDecals;				// Num decals active	
	int							 m_iNumDecalTris;			// Num decal triangles active
	bool						 m_bIndexBufferInvalid;		// Index buffer contents need updating

	Asura_PS3_GCM_VertexBuffer	 m_pxVB;					// Vertex buffer
	Asura_PS3_GCM_IndexBuffer	 m_apxIB[2];				// Double-buffered index buffer
	int							 m_iCurrentIndexBuffer;	

	Asura_PS3_DecalBuffer*   m_pxNextDecalBuffer;
};

// ----------------------------------------------------------------------------
// Asura_PS3_StaticDecalBuffer
// ----------------------------------------------------------------------------

class Asura_PS3_StaticDecalBuffer
{
public:

	int							m_iModuleIndex;
	uint32						m_uDecalMaterialHash;
	int							m_iDecalMaterialIndex;
	int							m_iNumVertices;
	int							m_iNumIndices;
	bool						m_bHasTangentFrames;
	Asura_Bounding_Box			m_xAABB;
	
	Asura_PS3_GCM_IndexBuffer	m_xIndexBuffer;
	Asura_PS3_GCM_VertexBuffer	m_xVertexBuffer;
};

// ----------------------------------------------------------------------------
// Asura_PS3_Decal_System
// ----------------------------------------------------------------------------

class Asura_PS3_DecalSystem
{

public:
	static void Initialise(void);
	static void Shutdown(void);
	static void Reset(void);
#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
	static void RenderStaticDecals(void);
#endif

	static void CalculateTangentAndBinormal( const  Asura_Vector_3* pxPositions,
											 const  Asura_Vector_2* pxTextureCoords,
													Asura_Vector_3& rxTangent,
													Asura_Vector_3& rxBinormal );


	static void									 AllocateStaticDecalBuffers(int iNumBuffers);
	static Asura_PS3_StaticDecalBuffer*			 GetStaticDecalBuffer(int iIndex);

	static void PrepareRenderStates(void);
	static void RestoreRenderStates(void);

	static void SetupMaterial( int iPass, const Asura_Decal_Material *pxMtl);
	static void SetVertexShader( bool bLitShader, const Asura_Decal_Material *pxMtl, float *pfFadeValues );
private:

	// Static decal buffers
	static int									 s_iNumStaticDecalBuffers;
	static Asura_PS3_StaticDecalBuffer*			 s_pxStaticDecalBuffers;
};

#endif //#ifndef __ASURA_PS3_DECAL_SYSTEM_H__
