///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_PSP_BotanicalsElement.h
// Author : Catalin Arsenescu
// Date   : 09-March-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////
#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif //ASURA_USE_PRAGMA_ONCE

#ifdef ASURA_USE_BOTANICALS
#ifndef __ASURA_PSP_BOTANICALS_ELEMENT__
#define __ASURA_PSP_BOTANICALS_ELEMENT__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////
#include "Botanicals/Asura_Botanical_Game_Render_Types.h"
#include "Botanicals/Asura_PSP_Botanicals_Shared.h"
#include "Rendering/Asura_PSP_BotanicalRenderer_Internal.h"

class Asura_PSP_BotanicalsTemplate;
class Asura_PSP_BotanicalsShape;

extern const unsigned int g_uASURA_PSP_BOTANICALRENDER_BILLBOARD_BUFFER_SIZE;

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_PSP_Botanicals_LOD_Geometry
// Author       : Catalin Arsenescu
// Date         : 13-March-2007
////////////////////////////////////////////////////////////////////////////////
class Asura_PSP_Botanicals_LOD_Geometry
{
public:
//Public functions
	Asura_PSP_Botanicals_LOD_Geometry();
	~Asura_PSP_Botanicals_LOD_Geometry();

	void Render(void);

//Public members
	int									m_iMaterial;
	// Geometry size
	u_int								m_uNumIndices;
	u_int								m_uNumTriangles;
	u_int								m_uNumVertices;

	// Draw array calls get pre-generated now
	u_int								m_uDrawArrayDisplayList[7];
	Asura_PSP_BotanicalStrip_Vertex*	m_pxVertices;
	u_short*							m_pusIndices;

};

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_PSP_Botanicals_LOD_Billboards
// Author       : Catalin Arsenescu
// Date         : 13-March-2007
////////////////////////////////////////////////////////////////////////////////
class Asura_PSP_Botanicals_LOD_Billboards
{
public:
//Public functions
	Asura_PSP_Botanicals_LOD_Billboards();
	~Asura_PSP_Botanicals_LOD_Billboards();

	void Render(Asura_Vector_3& xInstancePosition, float fInstanceScale);

//Public members
	int										m_iMaterial;
	// Geometry size
	u_int									m_uNumberOfRectangles;
	u_int									m_uNumberOfVertices;

	// Data needed to generate billboards in real time
	Asura_PSP_BotanicalBillboard_Data*		m_pxVertices;
	Asura_Vector_3*							m_pxPositions; 
	Asura_Vector_3*							m_pxNormals; 
};

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_PSP_Botanicals_LOD_Imposters
// Author       : Catalin Arsenescu
// Date         : 13-March-2007
////////////////////////////////////////////////////////////////////////////////
class Asura_PSP_Botanicals_LOD_Imposters
{
public:
//Public functions
	Asura_PSP_Botanicals_LOD_Imposters();
	~Asura_PSP_Botanicals_LOD_Imposters();

	void Render(Asura_Vector_3& xInstancePosition, float fInstanceScale, float fFadeValue);

//Public members
	int										m_iMaterial;
	int										m_iNumAngles;

	Asura_PSP_BotanicalsImposter_Data*		m_pxAngles;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_PSP_BotanicalsLODData
// Author       : Catalin Arsenescu
// Date         : 13-March-2007
////////////////////////////////////////////////////////////////////////////////
class Asura_PSP_BotanicalsLODData
{
public:
//Public functions
	Asura_PSP_BotanicalsLODData();
	~Asura_PSP_BotanicalsLODData();

	void Render(Asura_Vector_3 xInstancePosition, float fDistance, float fInstanceScale);
	void Initialise(Asura_PSP_Botanicals_Mesh_Type eType);

//Public members
	// Geometry type
	Asura_PSP_Botanicals_Mesh_Type				m_eMeshType;

	float										m_fMinDistance;
	float										m_fMinFullAlphaDistance;
	float										m_fMaxFullAlphaDistance;
	float										m_fMaxDistance;

	u_int										m_uNumVisibleInstances; // Calculated at runtime

	union
	{
		Asura_PSP_Botanicals_LOD_Geometry*		m_pxGeometryLODs;
		Asura_PSP_Botanicals_LOD_Billboards*	m_pxBillboardLODs;
		Asura_PSP_Botanicals_LOD_Imposters*		m_pxImpostersLODs;
	};
};

////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_PSP_BotanicalsElement
// Author       : Catalin Arsenescu
// Date         : 13-March-2007
////////////////////////////////////////////////////////////////////////////////
class Asura_PSP_BotanicalsElement
{
public:
//Public functions
	Asura_PSP_BotanicalsElement();
	~Asura_PSP_BotanicalsElement();

	void	Render	(Asura_PSP_BotanicalsTemplate*	pxTemplate
					,Asura_PSP_BotanicalsShape*		pxShape
					,Asura_Vector_3					xInstancePosition
					,int							iInstanceID
					,float							fDistance
					,float							fInstanceScale
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
					,Asura_BGR_DynamicInstance*		pxDynamicInstanceData
#endif
					);

//Public members
	float										m_fCentre;
	float										m_fRadius;

	int											m_iNumLODs;
	Asura_PSP_BotanicalsLODData*				m_pxLODData;
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	Asura_BGR_Section_ID						m_xSectionID;
	Asura_BGR_Version_ID						m_xVersionID;
#endif
};

#endif //__ASURA_PSP_BOTANICALS_ELELEMENT__
#endif //ASURA_USE_BOTANICALS
