//
// Asura_Chunk_ResourceFile_PSP_Environment.cpp
//
// Created by KJFL - Wednesday, February 04, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PSP_Environment.h"

#include "Core/Asura_Material_System.h"
#include "Core/Memory/Asura_ResourceAllocator.h"
#include "FileAccess/Asura_FileAccess.h"

#include "Rendering/Asura_PSP_RenderEnvironment.h"
#include "Rendering/Asura_PSP_VertexFormat.h"

void Asura_Chunk_ResourceFile_PSP_Environment::Process( Asura_Handle hFile )
{
	Asura_ResourcePushType(ASURA_RESOURCE_ENVIRONMENT_MESH);
	{
		Asura_PSP_RenderEnvironment_Zone&	xZone	= Asura_PSP_RenderEnvironment::GetCurrentZone();

		u_int uTotalNumberOfModules;
		u_int uTotalNumberOfStrips;
		u_int uTotalNumberOfDualPassStrips;
		u_int uTotalNumberOfVertices;
		u_int uTotalNumberOfIndices;

		// KJFL - 2004/02/06 12:31:00 - load in basic parameters of environment
		Asura_FileAccess::ReadData(hFile, &uTotalNumberOfModules, sizeof(u_int));
		Asura_FileAccess::ReadData(hFile, &uTotalNumberOfStrips, sizeof(u_int));
		Asura_FileAccess::ReadData(hFile, &uTotalNumberOfDualPassStrips, sizeof(u_int));
		Asura_FileAccess::ReadData(hFile, &uTotalNumberOfVertices, sizeof(u_int));
		Asura_FileAccess::ReadData(hFile, &uTotalNumberOfIndices, sizeof(u_int));

		xZone.m_uNumberOfModules = uTotalNumberOfModules;

#ifndef PSP_NEW_ENV_RENDER
		// KJFL - 2004/02/06 12:30:39 - allocate and load in module descriptions
		Asura_ResourceAllocateArray(xZone.m_pxModules, uTotalNumberOfModules);
		Asura_FileAccess::ReadData(hFile, xZone.m_pxModules, sizeof(Asura_PSP_RenderEnvironment_Module) * uTotalNumberOfModules);

		Asura_ChunkLoading_System::YieldBackgroundProcessing();
#else
		Asura_ResourceAllocateArray(xZone.m_pxModuleLightingCache, uTotalNumberOfModules);
		Asura_ResourceAllocateArray(xZone.m_puModuleClipFlags, uTotalNumberOfModules);
		Asura_ResourceAllocateArray(xZone.m_puReflectedModuleClipFlags, uTotalNumberOfModules);
#endif

		xZone.m_uTotalNumberOfStrips			= uTotalNumberOfStrips;
		xZone.m_uTotalNumberOfDualPassStrips	= uTotalNumberOfDualPassStrips;

		// KJFL - 2004/02/06 12:30:49 - allocate and load in strip descriptions
		Asura_ResourceAllocateArray(xZone.m_pxStrips, (uTotalNumberOfStrips + uTotalNumberOfDualPassStrips));
		for(u_int uStrip = 0; uStrip < (uTotalNumberOfStrips + uTotalNumberOfDualPassStrips); ++uStrip)
		{
			Asura_FileAccess::ReadData(hFile, (void*)&xZone.m_pxStrips[uStrip], sizeof(Asura_PSP_RenderEnvironment_Strip) - sizeof(Asura_Bounding_Box) - 3 * sizeof(u_int));
			// (SB - 06/02/07) If loading old version data and fix up strips by assigning correct vertex formats
			if (this->Version < 1)
			{
				if (uStrip < uTotalNumberOfStrips)
				{
					// First pass vertex format
					xZone.m_pxStrips[uStrip].m_usVertexFormat = SCEGU_TEXTURE_USHORT|SCEGU_COLOR_PF5650|SCEGU_NORMAL_SHORT|SCEGU_VERTEX_FLOAT;
				}
				else
				{
					// Second pass vertex format
					xZone.m_pxStrips[uStrip].m_usVertexFormat = SCEGU_TEXTURE_USHORT|SCEGU_COLOR_PF4444|SCEGU_NORMAL_SHORT|SCEGU_VERTEX_FLOAT;
				}
			}
#ifdef DEBUG
			else
			{
				// Check format is same across all first pass and across all second pass strips
				if (uStrip != 0 && uStrip != uTotalNumberOfStrips) // Skip test on first strip of each type
				{
					Asura_Assert(xZone.m_pxStrips[uStrip].m_usVertexFormat == xZone.m_pxStrips[uStrip - 1].m_usVertexFormat, "Vertex format must be the same across all first pass strips");
				}
			}
#endif

			Asura_ChunkLoading_System::YieldBackgroundProcessing();
		}

		// (SB - 19/02/07) Store first and second pass vertex formats
		{
			xZone.m_usFirstPassFormat = xZone.m_pxStrips[0].m_usVertexFormat;
			xZone.m_xFirstPassVertexSize = Asura_PSP_VertexFormat::CalculateVertexSize(xZone.m_usFirstPassFormat);
			xZone.m_xFirstPassVertexPosOffset = (Asura_PSP_VertexFormat::GetSizeOfTexCoord(xZone.m_usFirstPassFormat) * 2) + Asura_PSP_VertexFormat::GetSizeOfColour(xZone.m_usFirstPassFormat) + (Asura_PSP_VertexFormat::GetSizeOfNormalComponent(xZone.m_usFirstPassFormat) * 3);
			if (xZone.m_uTotalNumberOfDualPassStrips > 0)
			{
				xZone.m_usSecondPassFormat = xZone.m_pxStrips[xZone.m_uTotalNumberOfStrips].m_usVertexFormat;
				xZone.m_xSecondPassVertexSize = Asura_PSP_VertexFormat::CalculateVertexSize(xZone.m_usSecondPassFormat);
				xZone.m_xSecondPassVertexPosOffset = (Asura_PSP_VertexFormat::GetSizeOfTexCoord(xZone.m_usSecondPassFormat) * 2) + Asura_PSP_VertexFormat::GetSizeOfColour(xZone.m_usSecondPassFormat) + (Asura_PSP_VertexFormat::GetSizeOfNormalComponent(xZone.m_usSecondPassFormat) * 3);
			}
			else
			{
				xZone.m_usSecondPassFormat = 0;
				xZone.m_xSecondPassVertexSize = 0;
				xZone.m_xSecondPassVertexPosOffset = 0;
			}
		}

		// KJFL - 2004/02/06 12:30:13 - go through strips and patch up material IDs.
		for (u_int u=0; u<(uTotalNumberOfStrips + uTotalNumberOfDualPassStrips); u++)
		{
			xZone.m_pxStrips[u].m_iMaterialIndex = Asura_Material_System::ConvertIndex(xZone.m_pxStrips[u].m_iMaterialIndex);
		}

		Asura_ChunkLoading_System::YieldBackgroundProcessing();

		{
			// (SB - 19/01/07) Allocate and load in all the vertices taking account of custom vertex format sizes per strip
			unsigned int uTotalVertexSize = 0;
			// Calculate total space used by vertices and generate offsets to vertex buffer for each strip
			for (u_int i = 0; i < (uTotalNumberOfStrips + uTotalNumberOfDualPassStrips); i++)
			{
				uTotalVertexSize += xZone.m_pxStrips[i].m_uNumberOfIndices * Asura_PSP_VertexFormat::CalculateVertexSize(xZone.m_pxStrips[i].m_usVertexFormat);
			}
			Asura_ResourceAllocateArray(xZone.m_pucVertexBuffer, uTotalVertexSize);
			Asura_FileAccess::ReadData(hFile, xZone.m_pucVertexBuffer, uTotalVertexSize);
		}

		Asura_ChunkLoading_System::YieldBackgroundProcessing();

		// Check if we have an indexed environment
		if(uTotalNumberOfIndices)
		{
			// KJFL - 2004/02/06 12:35:58 - allocate and load in all the indices
			Asura_ResourceAllocateArray(xZone.m_pusIndexBuffer, uTotalNumberOfIndices);
			Asura_FileAccess::ReadData(hFile, xZone.m_pusIndexBuffer, sizeof(u_short)*uTotalNumberOfIndices);

			Asura_ChunkLoading_System::YieldBackgroundProcessing();
		}
		else
		{
			xZone.m_pusIndexBuffer = 0;
		}

		// KOR - temporary bounding box generation. This will be done by the exporter if it proves useful
		for(u_int uStrip = 0; uStrip < (uTotalNumberOfStrips + uTotalNumberOfDualPassStrips); ++uStrip)
		{
			// Get the current strip
			Asura_PSP_RenderEnvironment_Strip& xStrip = xZone.m_pxStrips[uStrip];

			// Calculate the number of strips if we split out through degenerate triangles
			xZone.CalculateNumberOfStrips( xStrip ); 

			// Get the pointer to the first vertex
			const unsigned char* pucVertex;
			xStrip.m_xBB.Reset();
			// Iterate through vertices adding them to the BB
			for(u_int uVertex = 0; uVertex < xStrip.m_uNumberOfIndices; ++uVertex)
			{
				pucVertex = xZone.GetVertexPtr(xStrip, uVertex);
				Asura_Vector_3 xPos = Asura_PSP_VertexFormat::GetPositionFromVertex(pucVertex, xStrip.m_usVertexFormat);
				xStrip.m_xBB.Set(xPos);
			}

			Asura_ChunkLoading_System::YieldBackgroundProcessing();
		}
	}
	Asura_ResourcePopType();
}
