//
// Asura_Chunk_ResourceFile_PSP_Object.cpp
//
// Created by Anthony Gregson - 22/06/04
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PSP_Object.h"

#include "Core/Asura_Hash.h"
#include "Core/Asura_Material_System.h"
#include "FileAccess/Asura_FileAccess.h"

#include "Rendering/Asura_PSP_ObjectRenderer.h"
#include "Rendering/Asura_PSP_ObjectRenderer_Internal.h"


void Asura_Chunk_ResourceFile_PSP_Object::Process( Asura_Handle hFile )
{
	Asura_ResourcePushType(ASURA_RESOURCE_OBJECT_MESH);
	{
		Asura_Hash_ID uHash;
		u_int uNumberOfVertices;
		u_int uNumberOfIndices;
		u_int uNumberOfTriangles;
		int iMaterialID;

		Asura_FileAccess::ReadData( hFile, &uHash, sizeof(Asura_Hash_ID) );

		Asura_FileAccess::ReadData( hFile, &uNumberOfTriangles, sizeof(u_int) );
		Asura_FileAccess::ReadData( hFile, &uNumberOfVertices, sizeof(u_int) );
		Asura_FileAccess::ReadData( hFile, &uNumberOfIndices, sizeof(u_int) );
		Asura_FileAccess::ReadData( hFile, &iMaterialID, sizeof(int) );

		Asura_PSP_ObjectRender_Handler&	xHandler	= Asura_PSP_ObjectRenderer::GetCurrentHandler();
		Asura_PSP_ObjectStrip*			pxObject	= xHandler.AddObjectStrip( uHash );

		if (pxObject)
		{
			pxObject->m_uNumberOfTriangles = uNumberOfTriangles;
			pxObject->m_uNumberOfIndices = uNumberOfIndices;
			pxObject->m_uNumberOfVertices = uNumberOfVertices;
			pxObject->m_iMaterial = Asura_Material_System::ConvertIndex(iMaterialID);

			// allocate space for vertices & indices
			Asura_ResourceAllocateArray(pxObject->m_pxVertices, uNumberOfVertices);
			Asura_ResourceAllocateArray(pxObject->m_pusIndices, uNumberOfIndices);

			// AJG - 22/06/04 - TODO - not checking for failure of memory allocations
            if( uNumberOfVertices )
            {
			    Asura_FileAccess::ReadData(hFile, pxObject->m_pxVertices, sizeof(Asura_PSP_ObjectStrip_Vertex)*uNumberOfVertices);
            }
			if(uNumberOfIndices > 0)
			{
				Asura_FileAccess::ReadData(hFile, pxObject->m_pusIndices, sizeof(u_short)*uNumberOfIndices);
			}

			// Next LOD distance and mesh
			Asura_FileAccess::ReadData(hFile, &pxObject->m_fMinDistance, sizeof(float));
			Asura_FileAccess::ReadData(hFile, &pxObject->m_uLODHash, sizeof(Asura_Hash_ID));
		}
		else
		{
			// we've ran out of strip space or the object already existed
			// either way, let's skip the data

			Asura_FileAccess::SkipData( hFile, sizeof(Asura_PSP_ObjectStrip_Vertex)*uNumberOfVertices );
			if(uNumberOfIndices > 0)
			{
				Asura_FileAccess::SkipData( hFile, sizeof(u_short)*uNumberOfIndices );
			}

			// Also LOD hash and distance
			Asura_FileAccess::SkipData(hFile, sizeof(float));
			Asura_FileAccess::SkipData(hFile, sizeof(Asura_Hash_ID));
		}
	}
	Asura_ResourcePopType();
}
