//
// Asura_Chunk_ResourceFile_PSP_ObjectHierarchy.cpp
//
// Created by KJFL - Thursday, April 08, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PSP_ObjectHierarchy.h"

#include "Core/Asura_Material_System.h"
#include "Core/Asura_Hash.h"
#include "FileAccess/Asura_FileAccess.h"

#include "Rendering/Asura_PSP_ObjectHierarchyRenderer.h"
#include "Rendering/Asura_PSP_ObjectHierarchyRenderer_Internal.h"


void Asura_Chunk_ResourceFile_PSP_ObjectHierarchy::Process( Asura_Handle hFile )
{
	Asura_ResourcePushType(ASURA_RESOURCE_OBJECT_HIERARCHY_MESH);
	{
		Asura_Hash_ID uHash;
		u_int uTotalNumberOfVertices;
		u_int uTotalNumberOfIndices;
		u_int uNumberOfStrips;

		Asura_FileAccess::ReadData( hFile, &uHash, sizeof(Asura_Hash_ID) );

		Asura_FileAccess::ReadData( hFile, &uNumberOfStrips, sizeof(u_int) );
		Asura_FileAccess::ReadData( hFile, &uTotalNumberOfVertices, sizeof(u_int) );
		Asura_FileAccess::ReadData( hFile, &uTotalNumberOfIndices, sizeof(u_int) );

		Asura_PSP_ObjectHierarchyRender_Handler&	xHandler	= Asura_PSP_ObjectHierarchyRenderer::GetCurrentHandler();
		Asura_PSP_ObjectHierarchy*					pxObjHier	= xHandler.AddHierarchy( uHash );

		const bool bColoursReplaceNormals = (Flags & ColoursReplaceNormals) && Version > 0;
		const bool bShortVerts = (Flags & VertsUseShorts) && Version > 1;
	
		size_t xVertexSize;

		if (bShortVerts)
		{
			if (bColoursReplaceNormals)
			{
				xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_ShortVertex_Colour);
			}
			else
			{
				xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_ShortVertex_Normal);
			}
		}
		else
		{
			if (bColoursReplaceNormals)
			{
				xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_Vertex_Colour);
			}
			else
			{
				xVertexSize = sizeof(Asura_PSP_ObjectHierarchy_Vertex_Normal);
			}
		}

		if (pxObjHier)
		{
			pxObjHier->m_bColoursReplaceNormals = bColoursReplaceNormals;
			pxObjHier->m_bShortVerts = bShortVerts;
			pxObjHier->m_uNumberOfStrips = uNumberOfStrips;
			pxObjHier->m_uTotalNumberOfIndices = uTotalNumberOfIndices;
			pxObjHier->m_uTotalNumberOfVertices = uTotalNumberOfVertices;

			// allocate space for strips, vertices & indices
			Asura_ResourceAllocateArray(pxObjHier->m_pxHierarchyStrips, uNumberOfStrips);
			Asura_ResourceAllocateArray(pxObjHier->m_pucVertices, uTotalNumberOfVertices * xVertexSize);
			Asura_ResourceAllocateArray(pxObjHier->m_pusIndices, uTotalNumberOfIndices);

			// KJFL - Thursday, April 08, 2004 - TODO - not checking for failure of memory allocations

			Asura_FileAccess::ReadData( hFile, pxObjHier->m_pxHierarchyStrips, sizeof(Asura_PSP_ObjectHierarchy_Strip) * uNumberOfStrips );
            if( uTotalNumberOfVertices )
            {
			    Asura_FileAccess::ReadData( hFile, pxObjHier->m_pucVertices, xVertexSize * uTotalNumberOfVertices );
			}

			if(uTotalNumberOfIndices > 0)
			{
				Asura_FileAccess::ReadData( hFile, pxObjHier->m_pusIndices, sizeof(u_short) * uTotalNumberOfIndices );
			}

			// now go through strips and patch up material ids
			for (u_int u=0; u<uNumberOfStrips; u++)
			{
				pxObjHier->m_pxHierarchyStrips[u].m_iMaterialIndex = Asura_Material_System::ConvertIndex( pxObjHier->m_pxHierarchyStrips[u].m_iMaterialIndex );
			}
		}
		else
		{
			// we've ran out of strip space or the hierarchy already existed
			// either way, let's skip the data

			Asura_FileAccess::SkipData( hFile, sizeof(Asura_PSP_ObjectHierarchy_Strip) * uNumberOfStrips );
			Asura_FileAccess::SkipData( hFile, xVertexSize * uTotalNumberOfVertices );
			if(uTotalNumberOfIndices > 0)
			{
				Asura_FileAccess::SkipData( hFile, sizeof(u_short)*uTotalNumberOfIndices );
			}
		}
	}
	Asura_ResourcePopType();
}
