// Asura_PSP_Core.cpp
//
// Created by KJFL - Wednesday, March 31, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.

#include <stdio.h>
#include <kernel.h>
#if !defined ASURA_PSP_SONY_SUBMISSION
	#if defined __SNC__
		#include <sys/unistd.h>
		#include <libsn.h>
	#endif
	#if defined __GNUC__ && !defined __SNC__
		#include <sys/features.h>
		#include <newlib.h>
	#endif
    #include <powersvc.h>
#endif
#include <devkit_version.h>
#if (SCE_DEVKIT_VERSION >= 0x03010110)
#include <utility/utility_module.h>
#else //#if (SCE_DEVKIT_VERSION >= 0x03010110)
#include <utility/utility_netmodule.h>
#endif //#else //#if (SCE_DEVKIT_VERSION >= 0x03010110)

#include "Core/Asura.h"
#include "Core/Asura_Core.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_PSP_KernelManagement.h"
#include "Debugging/Asura_Assert.h"
#include "FileAccess/Asura_PSP_FileIO.h"
#ifdef ASURA_PSP_PROFILING
#include "Debugging/Asura_PSP_Profiling.h"
#endif


#if (SCE_DEVKIT_VERSION >= 0x02070000) && (SCE_DEVKIT_VERSION < 0x03010110)
#include <utility/utility_avmodule.h>
#endif

// D E F I N E S /////////////////////////////////////////////////////////////
#ifndef PROGRAMMER_INITIALS
#define PROGRAMMER_INITIALS "unknown"
#endif

// G L O B A L S /////////////////////////////////////////////////////////////

// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////

void Asura_Core::PlatformInitialise(void)
{
#if !defined ASURA_PSP_SONY_SUBMISSION
	{
		#if !defined PROGRAMMER_INITIALS
			#warning PROGRAMMER_INITIALS is not defined
			#define PROGRAMMER_INITIALS "(unknown)"
		#endif
		#if !defined ASURA_BUILD_APP
			#warning ASURA_BUILD_APP is not defined
			#define ASURA_BUILD_APP "(missing)"
		#endif
		#if !defined ASURA_BUILD_TYPE
			#warning ASURA_BUILD_TYPE is not defined
			#define ASURA_BUILD_TYPE "(missing)"
		#endif
		#if !defined ASURA_BUILD_TIME
			#if defined __TIMESTAMP__
				#define ASURA_BUILD_TIME __TIMESTAMP__
			#elif defined __DATE__ && defined __TIME
				#define ASURA_BUILD_TIME __DATE__ " " __TIME__
			#else
				#define ASURA_BUILD_TIME "(not available)"
			#endif
		#endif

		static const char* const Asura_PSP_Core_App
			= "Application: '" ASURA_BUILD_APP "', " PROGRAMMER_INITIALS
				", '" ASURA_BUILD_TYPE "', " ASURA_BUILD_TIME;

		sceKernelPrintf("Application '%s'\n    Built by %s, type '%s' time %s\n",
			ASURA_BUILD_APP,PROGRAMMER_INITIALS,
			ASURA_BUILD_TYPE,ASURA_BUILD_TIME);
		
		#if defined NDEBUG
			#define ASURA_PSP_CORE_NDEBUG	1
		#else
			#define ASURA_PSP_CORE_NDEBUG	0
		#endif
		#if defined _DEBUG
			#define ASURA_PSP_CORE__DEBUG	1
		#else
			#define ASURA_PSP_CORE__DEBUG	0
		#endif

		sceKernelPrintf("    Flags: 'NDEBUG': %s, '_DEBUG': %s\n",
			ASURA_PSP_CORE_NDEBUG ? "defined" : "not defined",
			ASURA_PSP_CORE__DEBUG ? "defined" : "not defined");

		#if defined ASURA_PSP_ARTIST_BUILD
			#define ASURA_PSP_CORE_ART		1
		#else
			#define ASURA_PSP_CORE_ART		0
		#endif
		#if defined ASURA_EUROPEAN_BUILD
			#define ASURA_PSP_CORE_EU		1
		#else
			#define ASURA_PSP_CORE_EU		0
		#endif
		#if defined ASURA_JAPANESE_BUILD
			#define ASURA_PSP_CORE_JP		1
		#else
			#define ASURA_PSP_CORE_JP		0
		#endif

		sceKernelPrintf("    Build flags: artist: %s, european: %s, japanese: %s\n",
			ASURA_PSP_CORE_ART ? "defined" : "not defined",
			ASURA_PSP_CORE_EU ? "defined" : "not defined",
			ASURA_PSP_CORE_JP ? "defined" : "not defined");

		#if !defined ASURA_BUILD_DEFS
			#define ASURA_BUILD_DEFS		"(not available)"
		#endif
		#if !defined ASURA_BUILD_OPTS
			#define ASURA_BUILD_OPTS		"(not available)"
		#endif

		static const char* const Asura_PSP_Core_Flags
			 = "Defines: '" ASURA_BUILD_DEFS "'"
			   "\nOptions: '" ASURA_BUILD_OPTS "'";

		sceKernelPrintf("    Defines: %s\n    Options: %s\n",
			ASURA_BUILD_DEFS,ASURA_BUILD_OPTS);
	}
#endif

#if !defined ASURA_PSP_SONY_SUBMISSION
	{
		fputs("    Compiler:",stdout);

		fputs(" STDC",stdout);
		#if defined __STDC__
			sceKernelPrintf(" %lu",__STDC__);			
		#else
			fputs(" (unknown)",stdout);
		#endif

		#if defined __STDC_VERSION__
			sceKernelPrintf(" version %lu",__STDC_VERSION__);			
		#else
			fputs(" version (unknown)",stdout);
		#endif

		#if defined __SNC__
			static const char* const Asura_PSP_Core_Compiler
				= "Compiler: SNC";

			#if !defined __GNUC__
				sceKernelPrintf(", SNC %u\n",__SNC__);				
			#else
				sceKernelPrintf(", SNC %u (GNUC %u)\n",__SNC__,__GNUC__);
			#endif
		#elif defined __MWERKS__
			static const char* const Asura_PSP_Core_Compiler
				= "Compiler: CW";

			fputs(", CW",stdout);
		#elif defined __GNUC__ && !defined __SNC__
			#define ASURA_PSP_CORE_GNUC			__GNUC__
			#if defined __GNUC_MINOR__
				#define ASURA_PSP_CORE_GNUC_M	__GNUC_MINOR__
			#else
				#define ASURA_PSP_CORE_GNUC_M	0
			#endif
			#if defined __GNUC_PATCHLEVEL__
				#define ASURA_PSP_CORE_GNUC_P	__GNUC_PATCHLEVEL__
			#else
				#define ASURA_PSP_CORE_GNUC_P	0
			#endif
			#if defined __GXX_ABI_VERSION
				#define ASURA_PSP_CORE_GNUC_A	__GXX_ABI_VERSION
			#else
				#define ASURA_PSP_CORE_GNUC_A	0
			#endif
			#if defined __psp_gcc__
				#define ASURA_PSP_CORE_GNUC_S	__psp_gcc__
			#else
				#define ASURA_PSP_CORE_GNUC_S	0
			#endif
			#if defined __FINITE_MATH_ONLY__
				#define ASURA_PSP_CORE_GNUC_F	"on"
			#else
				#define ASURA_PSP_CORE_GNUC_F	"off"
			#endif
			#if defined __vfpu_enabled__
				#define ASURA_PSP_CORE_GNUC_V	"on"
			#else
				#define ASURA_PSP_CORE_GNUC_V	"off"
			#endif
			#if !defined __VERSION__
				#define __VERSION__				"(missing)"
			#endif

			static const char* const Asura_PSP_Core_Compiler
				= "Compiler: GCC version " __VERSION__;

			sceKernelPrintf(", GCC %d.%d.%d C++ ABI %d\n",
				ASURA_PSP_CORE_GNUC,ASURA_PSP_CORE_GNUC_M,
				ASURA_PSP_CORE_GNUC_P,ASURA_PSP_CORE_GNUC_A);

			sceKernelPrintf("        Sony level %d, version %s"
					"; only finite math %s, VFPU %s\n",
				ASURA_PSP_CORE_GNUC,__VERSION__,
				ASURA_PSP_CORE_GNUC_F,ASURA_PSP_CORE_GNUC_V);

				#elif defined _MSC_VER
			#define ASURA_PSP_CORE_MSC_VER		_MSC_VER
			#if defined _MT
				#define ASURA_PSP_CORE_MSC_MT	"on"
			#else
				#define ASURA_PSP_CORE_MSC_MT	"off"
			#endif
			#if !defined _M_IX86
				#define ASURA_PSP_CORE_MSC_86	_M_IX86
			#else
				#define ASURA_PSP_CORE_MSC_86	0
			#endif

			static const char* const Asura_PSP_Core_Compiler
				= "Compiler: MSC, multithreading " ASURA_PSP_CORE_MSC_MT;
			sceKernelPrintf(" MSC version 0x%x; multithreading %s, optimization 0x%x\n",
				_MSC_VER,ASURA_PSP_CORE_MSC_MT,ASURA_PSP_CORE_MSC_86);
			
		#else
			static const char* const Asura_PSP_Core_Compiler
				= "Compiler: unknown"

			fputs(", not SNC nor GCC nor CW\n",stdout);
		#endif
	}
#endif

#if !defined ASURA_PSP_SONY_SUBMISSION
	{
		fputs("    Runtime libraries:",stdout);

		#if defined SCE_DEVKIT_VERSION
			sceKernelPrintf(" devkit %x",SCE_DEVKIT_VERSION);			
		#else
			fputs(" devkit (unknown)",stdout);
		#endif

		#if defined _NEWLIB_VERSION
			sceKernelPrintf(", newlib %s",_NEWLIB_VERSION);			
		#else
			fputs(", newlib (unknown)",stdout);
		#endif

		#if defined _POSIX_VERSION
			sceKernelPrintf(", POSIX %lu",_POSIX_VERSION);			
		#else
			fputs(", POSIX (not defined)",stdout);
		#endif

		#if defined __SCE__
			sceKernelPrintf(", SCE %lu",__SCE__);			
		#else
			fputs(", SCE (not defined)",stdout);
		#endif

		#if defined __psp__
			sceKernelPrintf(", psp %x\n",__psp__);			
		#else
			fputs(", psp (not defined)\n",stdout);
		#endif

		#if defined __SNC__
			for (char **p = &__SN_Libsn_version[0]; *p != 0; p++)
			{
				sceKernelPrintf("        libsn: '%s'\n",*p);				
			}
		#endif
	}
#endif

#if !defined ASURA_PSP_SONY_SUBMISSION
	sceKernelPrintf( "    CPU clocked at %iMHz\n", scePowerGetCpuClockFrequencyInt() );    
#endif

	Asura_PSP_KernelManagement::StoreMainThreadID();
	Asura_PSP_KernelManagement::Initialise();

#if (SCE_DEVKIT_VERSION >= 0x03010110)
	SceUID uID;

	sceUtilityLoadModule(SCE_UTILITY_MODULE_AV_AVCODEC);
	sceUtilityLoadModule(SCE_UTILITY_MODULE_AV_SASCORE);
											
	sceUtilityLoadModule(SCE_UTILITY_MODULE_AV_LIBATRAC3PLUS);
											
#ifdef ASURA_SUPPORT_PSP_MPEG				
	sceUtilityLoadModule(SCE_UTILITY_MODULE_AV_MPEG);
#endif

#elif (SCE_DEVKIT_VERSION >= 0x02070000) //not #if (SCE_DEVKIT_VERSION >= 0x03010110)
	SceUID uID;

	sceUtilityLoadAvModule(SCE_UTILITY_AV_MODULE_AVCODEC);
	sceUtilityLoadAvModule(SCE_UTILITY_AV_MODULE_SASCORE);

	sceUtilityLoadAvModule(SCE_UTILITY_AV_MODULE_LIBATRAC3PLUS);

#ifdef ASURA_SUPPORT_PSP_MPEG
	sceUtilityLoadAvModule(SCE_UTILITY_AV_MODULE_MPEG);
#endif

#else // not #if (SCE_DEVKIT_VERSION >= 0x02070000)

	SceUID uID;
	SceKernelLMOption kMode;
	SceKernelLMOption uMode;

	kMode.size = sizeof(kMode);
	kMode.mpidtext = SCE_KERNEL_PRIMARY_KERNEL_PARTITION;
	kMode.mpiddata = SCE_KERNEL_PRIMARY_KERNEL_PARTITION;
	kMode.position = SCE_KERNEL_LMWO_POS_Low;
	kMode.access   = SCE_KERNEL_LMWO_ACCESS_Noseek;

	uMode.size = sizeof(uMode);
    uMode.mpidtext = SCE_KERNEL_PRIMARY_USER_PARTITION;
    uMode.mpiddata = SCE_KERNEL_PRIMARY_USER_PARTITION;
    uMode.position = SCE_KERNEL_LMWO_POS_Low;
    uMode.access   = SCE_KERNEL_LMWO_ACCESS_Noseek;

	Asura_PSP_FileIO::LoadStartModule(&uID,"kmodule/audiocodec.prx",&kMode);
	Asura_PSP_FileIO::LoadStartModule(&uID,"kmodule/sc_sascore.prx",&kMode);
	Asura_PSP_FileIO::LoadStartModule(&uID,"module/libatrac3plus.prx",&uMode);

#ifdef ASURA_SUPPORT_PSP_MPEG
	Asura_PSP_FileIO::LoadStartModule(&uID,"kmodule/videocodec.prx",&kMode);
	Asura_PSP_FileIO::LoadStartModule(&uID,"kmodule/mpegbase.prx",&kMode);
	Asura_PSP_FileIO::LoadStartModule(&uID,"module/mpeg.prx",&uMode);
#endif

#endif // #else not #if (SCE_DEVKIT_VERSION >= 0x02070000)

#if defined(ASURA_SUPPORT_EXTERNAL_NETWORK) || defined(ASURA_PSP_GAMESHARING)
	int iCode = 0;

#if (SCE_DEVKIT_VERSION >= 0x03010110)
	iCode = sceUtilityLoadModule(SCE_UTILITY_MODULE_NET_COMMON);
	if (iCode < 0)
	{
		sceKernelPrintf("sceUtilityLoadModule(SCE_UTILITY_MODULE_NET_COMMON) failed."
			 " iCode: 0x%x\n",iCode);		
	}

	iCode = sceUtilityLoadModule(SCE_UTILITY_MODULE_NET_ADHOC);
	if (iCode < 0)
	{
		sceKernelPrintf("sceUtilityLoadModule(SCE_UTILITY_MODULE_NET_ADHOC) failed."
			 " iCode: 0x%x\n",iCode);
	}

#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	iCode = sceUtilityLoadModule(SCE_UTILITY_MODULE_NET_INET);
	if (iCode < 0)
	{
		sceKernelPrintf("sceUtilityLoadModule(SCE_UTILITY_MODULE_NET_INET) failed."
			 " iCode: 0x%x\n",iCode);
	}
#endif // #ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
#else //#if (SCE_DEVKIT_VERSION >= 0x03010110)
	iCode = sceUtilityLoadNetModule(SCE_UTILITY_NET_MODULE_COMMON);
	if (iCode < 0)
	{
		sceKernelPrintf("sceUtilityLoadNetModule(SCE_UTILITY_NET_MODULE_COMMON) failed."
			 " iCode: 0x%x\n",iCode);		
	}

	iCode = sceUtilityLoadNetModule(SCE_UTILITY_NET_MODULE_ADHOC);
	if (iCode < 0)
	{
		sceKernelPrintf("sceUtilityLoadNetModule(SCE_UTILITY_NET_MODULE_ADHOC) failed."
			 " iCode: 0x%x\n",iCode);
	}

#ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
	iCode = sceUtilityLoadNetModule(SCE_UTILITY_NET_MODULE_INET);
	if (iCode < 0)
	{
		sceKernelPrintf("sceUtilityLoadNetModule(SCE_UTILITY_NET_MODULE_INET) failed."
			 " iCode: 0x%x\n",iCode);
	}
#endif // #ifdef ASURA_PSP_SUPPORT_INFRASTRUCTURE_NETWORK
#endif //#else //#if (SCE_DEVKIT_VERSION >= 0x03010110)

#endif

	const float fThreeQuarters = 0.75f;
	const float fOneAndAQuarter = 1.25f;
	Asura_Assert(Asura_Maths::FloatToInt_Chop(fThreeQuarters) == 0, "FloatToInt_Chop error");
	Asura_Assert(Asura_Maths::FloatToInt_Chop(-fThreeQuarters) == 0, "FloatToInt_Chop error");
	Asura_Assert(Asura_Maths::FloatToInt_Chop(fOneAndAQuarter) == 1, "FloatToInt_Chop error");
	Asura_Assert(Asura_Maths::FloatToInt_Chop(-fOneAndAQuarter) == -1, "FloatToInt_Chop error");
	Asura_Assert(Asura_Maths::FloatToInt_Near(fThreeQuarters) == 1, "FloatToInt_Near error");
	Asura_Assert(Asura_Maths::FloatToInt_Near(-fThreeQuarters) == -1, "FloatToInt_Near error");
	Asura_Assert(Asura_Maths::FloatToInt_Near(fOneAndAQuarter) == 1, "FloatToInt_Near error");
	Asura_Assert(Asura_Maths::FloatToInt_Near(-fOneAndAQuarter) == -1, "FloatToInt_Near error");

#ifdef ASURA_PSP_PROFILING
	Asura_PSP_Profiling::Initialise();
#endif
}

void Asura_Core::PlatformShutdown(void)
{
#ifdef ASURA_PSP_PROFILING
	Asura_PSP_Profiling::Shutdown();
#endif

	Asura_PSP_KernelManagement::Shutdown();
}

void Asura_Core::PlatformReset(void)
{
}

bool Asura_Core::Platform_IsMainThread(void)
{
	return Asura_PSP_KernelManagement::GetMainThreadID() == sceKernelGetThreadId();
}
