
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Handler_System_Callbacks_PSP.cpp
// Author : Paul Cardy
// Date   : 01-Feb-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_MULTIPLE_HANDLERS)

// This file's header...
#include "Core/Handlers/Asura_Handler_System_Callbacks_PSP.h"

// Asura independent header-files...
#include "Core/Handlers/Asura_Handler_System.h"

// Asura platform header-files...
#include "Rendering/Asura_PSP_CharacterRenderer.h"
#include "Rendering/Asura_PSP_RenderEnvironment.h"
#include "Rendering/Asura_PSP_ObjectHierarchyRenderer.h"
#include "Rendering/Asura_PSP_ObjectRenderer.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

// Creating a single instance forces registration with the "Asura_Handler_System"...
static Asura_Handler_System_Callbacks_PSP	s_xPlatformHandlerSystemCallbacks;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PSP::Asura_Handler_System_Callbacks_PSP
// Author       : Paul Cardy
// Date         : 01-Feb-2008
////////////////////////////////////////////////////////////////////////////////

Asura_Handler_System_Callbacks_PSP::Asura_Handler_System_Callbacks_PSP(void)
{
	Asura_Handler_System::RegisterPlatformCallbacks(this);
}

/// End of function 'Asura_Handler_System_Callbacks_PSP::Asura_Handler_System_Callbacks_PSP'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PSP::CreateHandlerData
// Author       : Paul Cardy
// Date         : 01-Feb-2008
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System_Callbacks_PSP::CreateHandlerData(u_int& uHandlerIndex,
														  const Asura_Hash_ID uHandlerHash,
														  const u_int uDataIndex,
														  const char* szName,
														  const Asura_Hash_ID uParentHandlerHash)
{
	const u_int	uIndex	= Asura_Handler_System::GetIndexWithGroup(uHandlerIndex);

	switch(uIndex)
	{
		case 0:
		{
			Asura_PSP_CharacterRenderer::CreateAndSetHandler( uHandlerHash, uParentHandlerHash );
			uHandlerIndex++;
			return(true);
		}

		case 1:
		{
			Asura_PSP_ObjectRenderer::CreateAndSetHandler( uHandlerHash, uParentHandlerHash );
			uHandlerIndex++;
			return(true);
		}

		case 2:
		{
			Asura_PSP_ObjectHierarchyRenderer::CreateAndSetHandler( uHandlerHash, uParentHandlerHash );
			uHandlerIndex++;
			return(false);
		}

		default:
		{
			Asura_Assert(false, "Unexpected index for sub-system handler to be created.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System_Callbacks_PSP::CreateHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PSP::DestroyHandlerData
// Author       : Paul Cardy
// Date         : 01-Feb-2008
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System_Callbacks_PSP::DestroyHandlerData(u_int& uHandlerIndex,
														   const Asura_Hash_ID uHandlerHash,
														   const u_int uDataIndex)
{
	const u_int	uIndex	= Asura_Handler_System::GetIndexWithGroup(uHandlerIndex);

	switch(uIndex)
	{
		case 0:
		{
			Asura_PSP_CharacterRenderer::DestroyHandler( uHandlerHash );
			uHandlerIndex++;
			return(true);
		}

		case 1:
		{
			Asura_PSP_ObjectRenderer::DestroyHandler( uHandlerHash );
			uHandlerIndex++;
			return(true);
		}

		case 2:
		{
			Asura_PSP_ObjectHierarchyRenderer::DestroyHandler( uHandlerHash );
			uHandlerIndex++;
			return(false);
		}

		default:
		{
			Asura_Assert(false, "Unexpected index for sub-system handler to be destroyed.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System_Callbacks_PSP::DestroyHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PSP::ReparentAllHandlerData
// Author       : Paul Cardy
// Date         : 01-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System_Callbacks_PSP::ReparentAllHandlerData(const Asura_Hash_ID uHandlerHash,
															   const Asura_Hash_ID uParentHandlerHash)
{
	// Set all the render data...
	Asura_PSP_CharacterRenderer::ReparentHandler( uHandlerHash, uParentHandlerHash );
	Asura_PSP_ObjectRenderer::ReparentHandler( uHandlerHash, uParentHandlerHash );
	Asura_PSP_ObjectHierarchyRenderer::ReparentHandler( uHandlerHash, uParentHandlerHash );
}

/// End of function 'Asura_Handler_System_Callbacks_PSP::ReparentAllHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PSP::SetAllHandlerData
// Author       : Paul Cardy
// Date         : 01-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System_Callbacks_PSP::SetAllHandlerData(const Asura_Hash_ID uHandlerHash,
														  const u_int uDataIndex)
{
	// SetAll the render data...
	Asura_PSP_CharacterRenderer::SetCurrentHandler( uHandlerHash );
	Asura_PSP_ObjectRenderer::SetCurrentHandler( uHandlerHash );
	Asura_PSP_ObjectHierarchyRenderer::SetCurrentHandler( uHandlerHash );
}

/// End of function 'Asura_Handler_System_Callbacks_PSP::SetAllHandlerData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PSP::CreateZoneData
// Author       : Paul Cardy
// Date         : 01-Feb-2008
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System_Callbacks_PSP::CreateZoneData(u_int& uHandlerIndex,
													   const Asura_Hash_ID uHandlerHash,
													   const u_int uDataIndex,
													   const char* szName,
													   const Asura_Hash_ID uParentHandlerHash,
													   const Asura_Hash_ID uZoneHash,
													   const bool bAllowEnvironment)
{
	const u_int	uIndex	= Asura_Handler_System::GetIndexWithGroup(uHandlerIndex);

	switch(uIndex)
	{
		case 0:
		{
			if(bAllowEnvironment)
			{
#if defined(ASURA_USE_STREAMING)
				Asura_PSP_RenderEnvironment::CreateAndSetZone(uHandlerHash);
#endif
			}
			return(false);
		}

		default:
		{
			Asura_Assert(false, "Unexpected index for sub-system handler to be created.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System_Callbacks_PSP::CreateZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PSP::DestroyZoneData
// Author       : Paul Cardy
// Date         : 01-Feb-2008
////////////////////////////////////////////////////////////////////////////////

bool Asura_Handler_System_Callbacks_PSP::DestroyZoneData(u_int& uHandlerIndex,
														const Asura_Hash_ID uHandlerHash,
														const u_int uDataIndex,
														const bool bAllowEnvironment)
{
	const u_int	uIndex	= Asura_Handler_System::GetIndexWithGroup(uHandlerIndex);

	switch(uIndex)
	{
		case 0:
		{
			if(bAllowEnvironment)
			{
#ifdef ASURA_USE_STREAMING
				Asura_PSP_RenderEnvironment::DestroyZone( uHandlerHash );
#else
				Asura_PSP_RenderEnvironment::Reset();
#endif
			}
			return(false);
		}

		default:
		{
			Asura_Assert(false, "Unexpected index for sub-system handler to be destroyed.");
			return(false);
		}
	}		// End of switch.
}

/// End of function 'Asura_Handler_System_Callbacks_PSP::DestroyZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PSP::ReparentAllZoneData
// Author       : Paul Cardy
// Date         : 01-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System_Callbacks_PSP::ReparentAllZoneData(const Asura_Hash_ID uHandlerHash,
															const Asura_Hash_ID uParentHandlerHash)
{
}

/// End of function 'Asura_Handler_System_Callbacks_PSP::ReparentAllZoneData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PSP::SetAllZoneData
// Author       : Paul Cardy
// Date         : 01-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System_Callbacks_PSP::SetAllZoneData(const Asura_Hash_ID uHandlerHash,
													   const u_int uDataIndex,
													   const bool bAllowEnvironment)
{
	if(bAllowEnvironment)
	{
#if defined(ASURA_USE_STREAMING)
		Asura_PSP_RenderEnvironment::SetCurrentZone(uHandlerHash);
#endif	//defined(ASURA_USE_STREAMING)
	}
}

/// End of function 'Asura_Handler_System_Callbacks_PSP::SetAllZoneData'. ///


#if defined(ASURA_LOGFILES_ENABLED)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Handler_System_Callbacks_PSP::CreateReport
// Author       : Paul Cardy
// Date         : 01-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void Asura_Handler_System_Callbacks_PSP::CreateReport(const Asura_Hash_ID uHandlerHash,
													 Asura_Stream& xOutputStream)
{
	Asura_PSP_ObjectRenderer::WriteHandlerReport( uHandlerHash, xOutputStream );
	Asura_PSP_ObjectHierarchyRenderer::WriteHandlerReport( uHandlerHash, xOutputStream );
	Asura_PSP_CharacterRenderer::WriteHandlerReport( uHandlerHash, xOutputStream );
}

/// End of function 'Asura_Handler_System_Callbacks_PSP::CreateReport'. ///

#endif	//defined(ASURA_LOGFILES_ENABLED)

#endif	//defined(ASURA_USE_MULTIPLE_HANDLERS)


/// End of file. ///
