//
// Asura_PSP_ScreenLog.cpp
//
// Created by KOR - 10/03/2005 14:31:19
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Debugging/Asura_PSP_ScreenLog.h"
#include "ProjectSettings/Asura_PSPConfiguration.h"

// Asura headers
#include "Rendering/Asura_PSP_GU.h"
#include "Rendering/Asura_PSP_MotionBlur.h"
#include "Rendering/Asura_Render.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Timers.h"

// PSP headers
#include<kernel.h>
#include<stdarg.h>
#include<stdio.h>
#include<thread.h>

static const int SCREENLOG_OUTPUT_BUFFER_SIZE = 1024;

bool Asura_PSP_ScreenLog::s_bHadFatalError = false;

#ifdef ASURA_PSP_ENABLE_SCREENLOG

void Asura_PSP_ScreenLog::Log(const char* szString, ...)
{
	// Don't bother if we're in a frame
	if(Asura_Render::IsRendering()) return;
	static int s_iRow = 0;
	int iNumChars;
	// Print string to buffer
	char szBuffer[SCREENLOG_OUTPUT_BUFFER_SIZE];

	va_list xArgs;
	va_start( xArgs, szString );
    Asura_PrintfToBfr( szBuffer, szString, xArgs, SCREENLOG_OUTPUT_BUFFER_SIZE, true );
	va_end( xArgs );

	// Render string
	Asura_Render::BeginFrame();
	GU_DebugPrint(8,s_iRow, 0x00ffffff, szBuffer);
	// Save settings
	const bool bMotionBlurWasEnabled = Asura_PSP_MotionBlur::IsEnabled();
	const float fTimeScale = Asura_Timers::GetGameTimeScale();
	// Set settings
	Asura_PSP_MotionBlur::Enable(true);
	Asura_Timers::SetGameTimeScale(0.f);
	Asura_Render::EndFrame();
	// Restore settings
	Asura_PSP_MotionBlur::Enable(bMotionBlurWasEnabled);
	Asura_Timers::SetGameTimeScale(fTimeScale);
	// Wait
	sceKernelDelayThread(500000);
	// Move down to next line
	s_iRow += 10;
	if(s_iRow > 260) s_iRow = 0;
}

#else//ASURA_PSP_ENABLE_SCREENLOG

void Asura_PSP_ScreenLog::Log(const char* szString, ...)
{
}

#endif// else ASURA_PSP_ENABLE_SCREENLOG

void Asura_PSP_ScreenLog::FatalError(const char* szString, ...)
{
	s_bHadFatalError = true;
	int iNumChars;
	// Print string to buffer
	char szBuffer[SCREENLOG_OUTPUT_BUFFER_SIZE];

	va_list xArgs;
	va_start( xArgs, szString );
    Asura_PrintfToBfr( szBuffer, szString, xArgs, SCREENLOG_OUTPUT_BUFFER_SIZE, true );
	va_end( xArgs );

	// Kill last frame
	if(Asura_Render::IsRendering()) Asura_Render::EndFrame();

	bool bTrue = true;
	while(bTrue)
	{
		// Start a new one
		Asura_Render::BeginFrame();

		GU_ClearColor(0x000000ff);
		GU_Clear(SCEGU_CLEAR_ALL);
		// Render string
		GU_DebugPrint(8,0, 0x00ffffff, szBuffer);

		Asura_Render::EndFrame();
		// Wait
		//sceKernelDelayThread(1000000);


		// If there's no debugger running then breaking is a really bad idea.
		// currently SNIsDebuggerRunning is broken so do it based on whether it's a DEBUG build.
#if _DEBUG
		bool bCanBreak = true;//( 0 != SNIsDebuggerRunning() );
#else
		bool bCanBreak = false;
#endif
		if(bCanBreak)
		{
			// Do another frame in case we're syncing on begin instead of end.
			Asura_Render::BeginFrame();
			GU_ClearColor(0x000000ff);
			GU_Clear(SCEGU_CLEAR_ALL);
			// Render string
			GU_DebugPrint(8,0, 0x00ffffff, szBuffer);
			Asura_Render::EndFrame();
			// Now break so we can see what caused the error in the debugger.
			asm("break 1"); asm("nop");
		}
	}
}
