//
// Asura_PSP_FileIO.h
//
// Created by KOR - 13/04/2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PSP_FILEIO_H__
#define __ASURA_PSP_FILEIO_H__

#include <kerneltypes.h>
#include "Core/Asura_Timers.h"

class Asura_PSP_FileIO
{
public:
	static int		Initialise();
	static int		Shutdown();
	static bool		DiscExists(void);
	static bool		FileExists(const char* szFilePath);
	static int		Open(const char *filepath, int flag, SceMode mode, const bool bAllowRetry = true);
	static int		Getstat(const char* szFilePath, struct SceIoStat* pxStat);
	static int		GetFileSize(const char* szFilePath);
	static int		Read(SceUID fd, void *pvReadBuffer, SceSize readsize, const bool bAllowRetry = true);
	static SceOff	Lseek(SceUID fd, SceOff offset, int whence);
	static int		Close(SceUID fd);
	static SceUID	LoadModule(const char *fileName, int flag, const struct SceKernelLMOption *opt);
	static int		LoadStartModule(SceUID *mid, const char *fileName, const struct SceKernelLMOption *opt);
	static int		UnloadStopModule(SceUID* piModuleID);
	
	static void		Suspend(void);
	static void		Resume(void);

	static bool		IsSuspended(void);

	// wrappers for sceIoDopen/sceIoDread/sceIoDclose
	static SceUID	OpenDir(const char* szDirName, const bool bAllowRetry = true);
	static int		ReadDir(SceUID fd, struct SceIoDirent* pxContents, const bool bAllowRetry = true);
	static int		CloseDir(SceUID fd);

	// GetFullPath converts a simple filename e.g. "envs/level.psp" to a fully-qualified
	// uppercase one e.g. "HOST0:ENVS/LEVEL.PSP" or "DISC0:/PSP_GAME/USRDIR/ENVS/LEVEL.PSP".
	// It returns a pointer to a static buffer so you must use the filename before calling
	// the function again.
	static void GetFullPath(const char* szFilename, const u_int uMaxPathLength, char* szFullPath, bool bForceMemStickIfUMD = false);
	// in debug builds, default is to force FMV files from MS0...
	static void GetFullPathForFMV(const char* szFilename, const u_int uMaxPathLength, char* szFullPath);

	// SK 27.06.07 - in non UMD builds this allows you to change whether the fmvs are played from
	// a memory stick or straight from the host.
	// true = memory stick, false = host.
	static void SetReadFMVFromMemStick( bool bTF);

private:

	static int		WaitSemaphore( int iLine );
#if defined ASURA_PSP_UMD && !defined(ASURA_PSP_ARTIST_BUILD)
	static int		WaitSemaphoreAndDriveReady( bool bAllowRetry, int iLine );
#endif
	static int		SignalSemaphore();
	static int		PollSemaphore();
	static int		WaitSemaphoreCB();

	static SceUID			s_iSemaphoreId;
	static Asura_TickCount	s_xLastOperationTicks;

#if defined(ASURA_PSP_UMD) && defined(ASURA_PSP_KEEP_UMD_SPINNING)
	static int				s_iKeepUMDSpinningThreadID;

	static void		KeepUMDSpinning(void);
	static int		KeepUMDSpinningThread(SceSize args, void *argp);
#endif	// defined(ASURA_PSP_UMD) && defined(ASURA_PSP_KEEP_UMD_SPINNING)
};

#endif//__ASURA_PSP_FILEIO_H__
