// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_Platform_GUIMenuInput.cpp												//
//																						//
//	Description: this handles the platform specific menu control system					//
//				 ps2 version.															//
//																						//
//	Created: Steve Keen - 02/01/07														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
// Asura headers
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"

#include "Input/Asura_PlatformInput.h"
#include "Input/Asura_ControlMap.h"

#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"
#include "Rendering/Asura_Render.h"

// PSP headers
#include "SCEUtility/Asura_PSP_SCEUtilityWrapper.h"


#define ASURA_GUIMENU_MAX_CONTROLLERS	1

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_InitInputMappings( void)
{
	if( s_pxMenuControlMaps == NULL)
	{
		s_pxMenuControlMaps = new Asura_ControlMap[ASURA_GUIMENU_MAX_CONTROLLERS];

		s_pxMenuControlMaps[0].CreateMappings( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_MAX);

		// -- This is a fix for the dodgy mapping on my pad --
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_UP, ASURA_PSP_KEYID_PAD1_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN, ASURA_PSP_KEYID_PAD1_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT, ASURA_PSP_KEYID_PAD1_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_PSP_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_PSP_KEYID_PAD1_CROSS, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, ASURA_PSP_KEYID_PAD1_SQUARE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, ASURA_PSP_KEYID_PAD1_TRIANGLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	//	s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, ASURA_PSP_KEYID_PAD1_R3, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4, ASURA_PSP_KEYID_PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5, ASURA_PSP_KEYID_PAD1_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6, ASURA_PSP_KEYID_PAD1_L1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7, ASURA_PSP_KEYID_PAD1_R1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	//	s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8, ASURA_PC_KEYID_PS2PAD1_L2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	//	s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_PC_KEYID_PS2PAD1_R2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_PSP_KEYID_PAD1_CIRCLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_PSP_KEYID_PAD1_UP, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_PSP_KEYID_PAD1_DOWN, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_PSP_KEYID_PAD1_LEFT, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_PSP_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_PSP_KEYID_PAD1_CROSS, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_PSP_KEYID_PAD1_SQUARE, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_PSP_KEYID_PAD1_TRIANGLE, Asura_ControlMapping::ON_PRESS);
	//	s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD, ASURA_PSP_KEYID_PAD1_R3, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD, ASURA_PSP_KEYID_PAD1_SELECT, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD, ASURA_PSP_KEYID_PAD1_START, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD, ASURA_PSP_KEYID_PAD1_L1, Asura_ControlMapping::ON_PRESS);
		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD, ASURA_PSP_KEYID_PAD1_R1, Asura_ControlMapping::ON_PRESS);
	//	s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD, ASURA_PC_KEYID_PS2PAD1_L2, Asura_ControlMapping::ON_PRESS);
	//	s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD, ASURA_PC_KEYID_PS2PAD1_R2, Asura_ControlMapping::ON_PRESS);

		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_PSP_KEYID_PAD1_CIRCLE, Asura_ControlMapping::ON_PRESS);

#ifdef ASURA_USE_CONTROLMAP_MANAGER
//		s_pxMenuControlMaps[0].AddKeyMapping( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PSP_KEYID_PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
		s_pxMenuControlMaps[0].AddHoldKeyMapping(Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_PSP_KEYID_PAD1_SELECT, 1.0f );
#endif 
	}
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_ShutdownInputMappings( void)
{
	u_int u;

	if( s_pxMenuControlMaps)
	{
		for( u=0; u<ASURA_GUIMENU_MAX_CONTROLLERS; u++)
		{
			s_pxMenuControlMaps[u].DeleteMappings();
		}

		delete []s_pxMenuControlMaps;
		s_pxMenuControlMaps = NULL;
	}
}

// --------------------------------------------------------------------------------------- 

u_int Asura_GUIMenu_System::Platform_GetNumControlMaps( void)
{
	return ASURA_GUIMENU_MAX_CONTROLLERS;
}

// --------------------------------------------------------------------------------------- 


bool Asura_GUIMenu_System::Platform_ShouldUpdateControls()
{
	return !Asura_PSP_SCEUtilityWrapper::ShouldHaveFocus();
}

bool Asura_GUIMenu_System::Platform_ShouldRenderMenuPages()
{
	static Asura_Hash_ID uUtilityBackgroundOpen = ASURA_HASH_ID_UNSET;
	static Asura_Hash_ID uPreviouslyFocusedPage = ASURA_HASH_ID_UNSET;
	static const Asura_Hash_ID& uUtilityBackground = Asura_PSP_SCEUtilityWrapper::GetBackgroundMenuPage();
	
	if (Asura_PSP_SCEUtilityWrapper::ShouldHaveFocus())
	{
		if (uUtilityBackground != ASURA_HASH_ID_UNSET)
		{
			if (uUtilityBackgroundOpen == ASURA_HASH_ID_UNSET)
			{
				//attempt to open
				uPreviouslyFocusedPage = Asura_GUIMenu_System::GetFocusedMenuHashID();
				
				bool bPageOpened = Asura_GUIMenu_System::OpenMenuPage( uUtilityBackground );
				Asura_Assertf(bPageOpened,("Can't open sce utility page background menu page 0x%08x!", uUtilityBackground));

				uUtilityBackgroundOpen =  (bPageOpened) ? uUtilityBackground : ASURA_HASH_ID_UNSET;
				
				if (!bPageOpened) Asura_PSP_SCEUtilityWrapper::SetBackgroundMenuPage( ASURA_HASH_ID_UNSET );
			}
			else //Skip the first render
			{
				return true;
			}
		}
		
		// default - render black quad so the Utility menu is nice and clear.
		Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
		Asura_Render::Overlay.SetRectXY(0.0f, 0.0f, 480.0f, 272.0f);
		Asura_Render::Overlay.Colour = 0xff000000;
		Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
		Asura_Render::Overlay.Render();
		Asura_Render::FlushDisplay();
		return false;
	}
	else if (uUtilityBackgroundOpen)
	{
		Asura_GUIMenu_System::CloseMenuPage( uUtilityBackgroundOpen );
		Asura_GUIMenu_System::SetFocusedMenuByID( uPreviouslyFocusedPage );
		uUtilityBackgroundOpen = ASURA_HASH_ID_UNSET;
		uPreviouslyFocusedPage = ASURA_HASH_ID_UNSET;
	}

	return true;
}

// --------------------------------------------------------------------------------------- 

// (MH - 27.02.2007) text input functions
static bool g_bDoingTextInput = false;
static Asura_UnicodeChar* g_wszInitialBuffer;
static Asura_UnicodeChar* g_wszOutputBuffer;
void Asura_GUIMenu_System::Platform_BeginTextInput( const Asura_UnicodeChar* wszMessage, 
												    Asura_UnicodeChar* wszBuffer, 
													u_int uBufferMaxChars )
{
	Asura_Assert(!g_bDoingTextInput, "already doing text input");
	g_bDoingTextInput = true;
	Asura_Assert(!g_wszOutputBuffer, "memory leak");
	g_wszInitialBuffer = wszBuffer;
	g_wszOutputBuffer = new Asura_UnicodeChar[uBufferMaxChars + 1];
	wcscpy(g_wszOutputBuffer, wszBuffer);
	Asura_PSP_SCEUtilityWrapper::InitialiseOnScreenKeyboard(wszMessage, 
		g_wszInitialBuffer, g_wszOutputBuffer, uBufferMaxChars + 1, 
		Asura_PSP_SCEUtilityWrapper::LATIN_ALPHANUMERIC);
}

void Asura_GUIMenu_System::Platform_UpdateTextInput()
{
	Asura_PSP_SCEUtilityWrapper::BeginFrame();
	Asura_PSP_SCEUtilityWrapper::Update();
	Asura_PSP_SCEUtilityWrapper::EndFrame();
}

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
void Asura_GUIMenu_System::Platform_RenderSafetyBorder()
{
}
#endif // ASURA_DEBUG_VARIABLE_SYSTEM

void Asura_GUIMenu_System::Platform_BeginRender()
{
}

void Asura_GUIMenu_System::Platform_RenderTextInput()
{
	Asura_PSP_SCEUtilityWrapper::Render();
}

void Asura_GUIMenu_System::Platform_EndRender()
{
}

bool Asura_GUIMenu_System::Platform_IsTextInputFinished()
{
	return Asura_PSP_SCEUtilityWrapper::GetState() != Asura_PSP_SCEUtilityWrapper::COMMAND_IN_PROGRESS;
}

bool Asura_GUIMenu_System::Platform_IsTextInputCancelled()
{
	return Asura_PSP_SCEUtilityWrapper::GetState() == Asura_PSP_SCEUtilityWrapper::COMMAND_CANCELLED;
}


void Asura_GUIMenu_System::Platform_EndTextInput()
{
	Asura_Assert(g_bDoingTextInput, "not doing text input");
	g_bDoingTextInput = false;
	wcscpy(g_wszInitialBuffer, g_wszOutputBuffer);

	delete[] g_wszOutputBuffer;
	g_wszOutputBuffer = NULL;
	g_wszInitialBuffer = NULL;
}

// -------------------------------------------------------------------------------------------------

bool Asura_GUIMenu_System::Platform_ProcessMenuBackControls( u_int uButtonsPressed, u_int uButtonsTrapped)
{
	bool bRet = false;

	// filters out any buttons presses not required, then we do an equality check
	uButtonsPressed &= (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD);

	if( uButtonsPressed == (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD) )
	{
		if( (uButtonsTrapped&ASURA_GUIMENU_ACTION_BUTTON_CANCEL) == 0)
		{
			bRet = true;
		}
	}

	return bRet;
}

// --------------------------------------------------------------------------------------- 

#endif // ASURA_USE_GUI_MENU_SYSTEM
