//
// Asura_PSP_MoviePlayback_Internal.h
//
// Created by KJFL - 13 April 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PSP_MOVIEPLAYBACK_INTERNAL_H__
#define __ASURA_PSP_MOVIEPLAYBACK_INTERNAL_H__

#include <psptypes.h>
#include <kernel.h>
#include <libmpeg.h>
#include "Core/Asura_Multithread.h"
#include "Rendering/Asura_PSP_Graphics.h"

class Asura_PSP_Movie
{
	public:

		typedef void (*Asura_RenderFMVSubtitleCB)(const float fTime);

		Asura_PSP_Movie(void (*pfnRenderSubtitleCB)(const float fTime),
										float fTimeBeforeSkipAllowed,
										float fAudioVolume);
		~Asura_PSP_Movie();

		bool Initialise( const char* const szFilename, int iAudioStream, int iVideoStream, bool bStreamShouldBePresent ); // If bool is false, skip any asserts or warnings about stream id out of range.
		void Play();
		bool Update();
		void Shutdown( void );
		void DeallocateVolatileMemory();

		void Finish();

		bool Finished() const {return m_bFinished;}

		// "SimpleAudio" decoding...
		bool DecodeAudio(short* psCurrentBuffer, float& fVolume);

	private:

		// ParseHeader1 does whatever parsing we can do without having initialised the mpeg object
		// and assumes the mutex is unlocked (so it can do file access without unlocking it).
		bool ParseHeader1( const char* pcHeader );
		// ParseHeader2 does the remaining parsing (but the mutex is locked so must be unlocked for fileaccess).
		bool ParseHeader2( const char* pcHeader );

		void ReadStream( void );

		bool DecodeVideo( void );
		bool DecodeVideoEnd( void );

		bool DecodeAudio( void );
		bool DecodeAudioEnd( void );

		bool IsADecodeAudioBufferFree( void );
		SceUChar8* GetNextDecodedAudioBuffer( void );


		// callback to handle reading of data
		static SceInt32 RingbufferCallBack( SceVoid* pBuf, SceInt32 iNum, SceVoid* pCallbackData );

		SceMpeg				m_xMpeg;
		SceMpegRingbuffer	m_xRingbuffer;

        // handle to file we'll be streaming from
		Asura_Handle		m_iFileID;	 

		// where the data stream actually starts
		SceUInt32			m_uStreamOffset;

		// total size of data
		SceUInt32			m_uStreamSize;

		// total read in so far (not necessary decoded, just loaded)
		SceUInt32			m_uBytesRead;

		// one stream for video, one for audio
		SceMpegStream*		m_pxVideoStream;
		SceMpegStream*		m_pxAudioStream;
		SceMpegAu			m_xAvcAu;
		SceMpegAu			m_xAtracAu;
		SceMpegHandle		m_pxAvcEsBuf;

		// audio decode and play buffers
		Asura_Mutex			m_xMutex;
		SceUChar8**			m_ppucDecodedAudioBuffers;
		int					m_iAudioInitFlag;
		int					m_iNextAudioDecodeBuffer;
		int					m_iNextAudioPlayBuffer;
		volatile int		m_iNumberOfFullAudioBuffers;

		// LOCKSEMA

		// SceUID				m_iLockSema;

		bool				m_bKillAudioThread;
		bool				m_bUsedSASMemory;
		bool				m_bCallsceMpegInitOK;
		bool				m_bCallsceMpegCreateOK;

		char*				m_pcPSPStream;
		char*				m_pcWorkBuffer;
		SceChar8*			m_pcAudioBuffer;
		bool				m_bScreenClearEnabledStore;
		volatile bool		m_bFinished;
		bool				m_bDoneInitialSeek;

		void (*m_pfnRenderSubtitleCB)(const float fTime);
		float m_fTimeBeforeSkipAllowed;
		float m_fAudioVolume;
		int m_iStartVSyncCount;
		int m_iFramesDecoded;

		SceInt32			m_iNumAudioStreams;
		SceInt32			m_iNumVideoStreams;
};
#endif//__ASURA_PSP_MOVIEPLAYBACK_INTERNAL_H__
