//
// Asura_PFX_Adhoc.h
//
// Created by KOR - 22/02/2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PSP_ADHOC_H__
#define __ASURA_PSP_ADHOC_H__

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#include "Core/Asura_Multithread.h"

#include <pspnet_adhocctl.h>

enum ASURA_PSP_ADHOC_STATE
{
	ASURA_PSP_ADHOC_UNINITIALISED,
	ASURA_PSP_ADHOC_DISCONNECTED,
	ASURA_PSP_ADHOC_CONNECTING,
	ASURA_PSP_ADHOC_CONNECTED,
	ASURA_PSP_ADHOC_DISCONNECTING,
	ASURA_PSP_ADHOC_SCANNING,
	ASURA_PSP_ADHOC_CREATING,
	ASURA_PSP_ADHOC_JOINING,
	ASURA_PSP_ADHOC_WLAN_SWITCH_OFF,
};

// TODO - update diagram
// ---------------------------------
// | ASURA_PSP_ADHOC_UNINITIALISED |
// ---------------------------------
//        |             ^
//    Initialise()      |
//        |         Shutdown()
//        V             |
// ---------------------------------				-----------------------------
// | ASURA_PSP_ADHOC_DISCONNECTED  |-----Scan()---->| ASURA_PSP_ADHOC_SCANNING  |
// |                               |<----Update()---|                           |
// ---------------------------------  |				-----------------------------
//                |                   |
//  Create() or Connect() or Join()   |
//                |                   |
//                V                   |
// ---------------------------------  |
// | ASURA_PSP_ADHOC_CONNECTING    |  |
// ---------------------------------  |
//                |                   |
//             Update()            Update()
//                |                   |
//                V                   |
// ---------------------------------  |
// | ASURA_PSP_ADHOC_CONNECTED     |  |
// ---------------------------------  |
//                |                   |
//           Disconnect()             |
//                |                   |
//                V                   |
// ---------------------------------  |
// | ASURA_PSP_ADHOC_DISCONNECTING |--/
// ---------------------------------
// 
// If at any point the wireless LAN switch is moved to the off position, the state will
// change to ASURA_PSP_ADHOC_WLAN_SWITCH_OFF. When they return it to the on position,
// the state will change to ASURA_PSP_ADHOC_DISCONNECTED

class Asura_PSP_Adhoc
{
public:
	static void Initialise();
	static void Shutdown();

	static void Suspend(void);
	static void Resume(void);

	static ASURA_PSP_ADHOC_STATE GetState()
	{
		if (s_iSemaphoreID != -1)
		{
			WaitSemaphore();
			ASURA_PSP_ADHOC_STATE eRet = s_eState;
			SignalSemaphore();
			return eRet;
		}
		else
		{
			return s_eState;
		}
	}
	static bool IsWLANSwitchedOff()	{ return (GetState() == ASURA_PSP_ADHOC_WLAN_SWITCH_OFF); }
	static bool IsInitialised()		{ return (GetState() != ASURA_PSP_ADHOC_UNINITIALISED); }
	static void Update();
	static void Connect(const char* szGroupName);
	static bool IsConnecting()		{ return GetState() == ASURA_PSP_ADHOC_CONNECTING; }
	static bool IsConnected()		{ return GetState() == ASURA_PSP_ADHOC_CONNECTED; }
	static void Disconnect();
	static bool IsDisconnecting()	{ return GetState() == ASURA_PSP_ADHOC_DISCONNECTING; }
	static bool IsDisconnected()	{ return GetState() == ASURA_PSP_ADHOC_DISCONNECTED; }
	static void Scan();
	static bool	IsScanning()		{ return GetState() == ASURA_PSP_ADHOC_SCANNING; }
	static void Create(const char* szGroupName);
	static bool Join(int iGroup);
	static int	GetNumScanResults();
	static const char* GetScanResult(int iResult);

	// Semaphore
	static int	PollSemaphore();
	static int	WaitSemaphore();
	static int	WaitSemaphoreCB();
	static int	SignalSemaphore();

private:
	static void Handler(int iEvent, int iError, void *pvArg);
	// Static members
	static ASURA_PSP_ADHOC_STATE	s_eState;
	static SceNetAdhocctlAdhocId	s_xAdhocID;
	static int						s_iHandlerID;
	static int						s_iNumScanResults;
	static SceNetAdhocctlScanInfo*	s_pxScanResults;
	static int						s_iSemaphoreID;

	static Asura_Mutex				s_xAdhocMutex;
};

#endif//ASURA_SUPPORT_EXTERNAL_NETWORK

#endif//__ASURA_PSP_ADHOC_H__
