//
// Asura_Platform_NetSocket.h
//
// PSP version
//
// Created by KOR - 15.02.2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __ASURA_PLATFORM_NETSOCKET_H__
#define __ASURA_PLATFORM_NETSOCKET_H__

#ifndef __GSSOCKET_H__

// PSP library headers
#include <errno.h>
#include <pspnet.h>
#include <pspnet_inet.h>
#include <pspnet/sys/socket.h>
#include <pspnet/sys/select.h>
#include <pspnet/sys/time.h>

#define SOCKET_ERROR (-1)
#define INVALID_SOCKET (-1)
#define INADDR_NONE 0xffffffff

// PSP sockets compatability, cribbed from GameSpy header.
// These define the WSAxxx to match ones in <errno.h>.
// Note that in adhoc we don't get any API errors anyway.
#define WSAEWOULDBLOCK      EWOULDBLOCK             
#define WSAEINPROGRESS      EINPROGRESS             
#define WSAEALREADY         EALREADY                
#define WSAENOTSOCK         ENOTSOCK                
#define WSAEDESTADDRREQ     EDESTADDRREQ            
#define WSAEMSGSIZE         EMSGSIZE                
#define WSAEPROTOTYPE       EPROTOTYPE              
#define WSAENOPROTOOPT      ENOPROTOOPT             
#define WSAEPROTONOSUPPORT  EPROTONOSUPPORT         
#define WSAESOCKTNOSUPPORT  ESOCKTNOSUPPORT         
#define WSAEOPNOTSUPP       EOPNOTSUPP              
#define WSAEPFNOSUPPORT     EPFNOSUPPORT            
#define WSAEAFNOSUPPORT     EAFNOSUPPORT            
#define WSAEADDRINUSE       EADDRINUSE              
#define WSAEADDRNOTAVAIL    EADDRNOTAVAIL           
#define WSAENETDOWN         ENETDOWN                
#define WSAENETUNREACH      ENETUNREACH             
#define WSAENETRESET        ENETRESET               
#define WSAECONNABORTED     ECONNABORTED            
#define WSAECONNRESET       ECONNRESET              
#define WSAENOBUFS          ENOBUFS                 
#define WSAEISCONN          EISCONN                 
#define WSAENOTCONN         ENOTCONN                
#define WSAESHUTDOWN        ESHUTDOWN               
#define WSAETOOMANYREFS     ETOOMANYREFS            
#define WSAETIMEDOUT        ETIMEDOUT               
#define WSAECONNREFUSED     ECONNREFUSED            
#define WSAELOOP            ELOOP                   
#define WSAENAMETOOLONG     ENAMETOOLONG            
#define WSAEHOSTDOWN        EHOSTDOWN               
#define WSAEHOSTUNREACH     EHOSTUNREACH            
#define WSAENOTEMPTY        ENOTEMPTY               
#define WSAEPROCLIM         EPROCLIM                
#define WSAEUSERS           EUSERS                  
#define WSAEDQUOT           EDQUOT                  
#define WSAESTALE           ESTALE                  
#define WSAEREMOTE          EREMOTE
#define WSAEINVAL           EINVAL

#define AF_INET     SCE_NET_INET_AF_INET
#define SOCK_STREAM SCE_NET_INET_SOCK_STREAM
#define SOCK_DGRAM  SCE_NET_INET_SOCK_DGRAM
#define INADDR_ANY  SCE_NET_INET_INADDR_ANY
#define SOL_SOCKET  SCE_NET_INET_SOL_SOCKET
#define SO_SNDBUF   SCE_NET_INET_SO_SNDBUF
#define SO_RCVBUF   SCE_NET_INET_SO_RCVBUF
#define SO_NBIO     SCE_NET_INET_SO_NBIO
#define SO_BROADCAST SCE_NET_INET_SO_BROADCAST
#define SO_KEEPALIVE SCE_NET_INET_SO_KEEPALIVE
#define SO_REUSEADDR SCE_NET_INET_SO_REUSEADDR

#define IPPROTO_TCP SCE_NET_INET_IPPROTO_TCP // protocol defined by SOCK_STREAM
#define IPPROTO_UDP SCE_NET_INET_IPPROTO_UDP // protocol defined by SOCK_DGRAM


// structures
#define in_addr     SceNetInetInAddr
#define sockaddr_in	SceNetInetSockaddrIn
#define sockaddr    SceNetInetSockaddr

// Remove FD types set in sys/types.h
// Replace with types in pspnet/sys/select.h
#if defined(_SYS_TYPES_H) && defined(FD_SET)
	#undef fd_set
	#undef FD_SET
	#undef FD_CLR
	#undef FD_ZERO
	#undef timeval
#endif
#define fd_set  SceNetInetFdSet
#define timeval SceNetInetTimeval
#define FD_SET  SceNetInetFD_SET
#define FD_CLR  SceNetInetFD_CLR
#define FD_ZERO SceNetInetFD_ZERO

// functions
#define htonl		sceNetHtonl
#define ntohl		sceNetNtohl
#define htons		sceNetHtons
#define ntohs		sceNetNtohs
#define socket      sceNetInetSocket
#define shutdown    sceNetInetShutdown
#define closesocket sceNetInetClose

#define setsockopt					  sceNetInetSetsockopt
#define getsockopt(s, l, on, ov, ol)  sceNetInetGetsockopt(s, l, on, ov, (SceNetInetSocklen_t *)ol)

#define bind			sceNetInetBind
#define select			sceNetInetSelect

#define connect			sceNetInetConnect
#define listen			sceNetInetListen
#define accept(s,a,l)	sceNetInetAccept(s, a, (SceNetInetSocklen_t *)l)

#define send		sceNetInetSend  
#define recv		sceNetInetRecv
#define sendto		sceNetInetSendto 
#define recvfrom(s, b, l, f, fr, fl)	sceNetInetRecvfrom(s, b, l, f, fr, (SceNetInetSocklen_t *)fl)


#define inet_addr   sceNetInetInetAddr

#define getsockname(s,n,l) sceNetInetGetsockname(s, n, (SceNetInetSocklen_t *)l)

#define WSAGetLastError(s) sceNetInetGetErrno()

typedef int SOCKET;
typedef struct sockaddr    SOCKADDR;
typedef struct sockaddr_in SOCKADDR_IN;
typedef struct in_addr     IN_ADDR;
typedef struct hostent     HOSTENT;
typedef struct timeval     TIMEVAL;

#endif // #ifndef __GSSOCKET_H__


// Asura_NetSocket platform defines

// On the Xbox, use the VDP protocol which allows for combined game and voice data
// (game data is sent encrypted, voice unencrypted)
// On other platforms, we will want to use IPPROTO_UDP
#define ASURA_NETSOCKET_PROTOCOL IPPROTO_UDP

// On XBox the IP+VDP header size ranges from 44 to 51, so say 48 on average.
// On PC/PS2 IP+UDP header size is 28 bytes
// KOR - TODO - find out the actual packet header size...
#define ASURA_NETSOCKET_PROTOCOL_PACKETHEADERSIZE (28)

// 3658 is the IANA-registered PS2 port. 1000 should be used by Xbox as it gives the smallest packet header.
// KOR - TODO - find out the actual port we want to use...
#define ASURA_NETSOCKET_DEFAULTPORT (3658)

// Handle message crashes if we don't copy the header out of the message to access its data
#define ASURA_NETSOCKET_PLATFORM_ALIGN_HEADER (1)

#ifdef ASURA_NETWORK_USE_BROADCAST_FOR_GAME_DATA
// this should only be defined for platforms where it makes sense to spam
// the entire network with game data, e.g. PSP in AdHoc mode where every 
// machine on the local network is actaully in the game.
// Assigned using operator= which sets all bytes of the address so just one byte.
#define ASURA_NETADDRESS_BROADCAST (0xff)
#endif

//typedef SceNetEtherAddr Asura_NetAddress;
class Asura_NetAddress
{
public:
	Asura_NetAddress() {};
	// ctor from u_char is used to set broadcast address
	explicit Asura_NetAddress(u_char ucValue)
	{
		*this = ucValue;
	}
	// ctor from u_int is used to set to 0, and to construct from BSD address
	explicit Asura_NetAddress(u_int uValue)
	{
		*this = uValue;
	}
	explicit Asura_NetAddress(const SceNetEtherAddr& xAddress)
	{
		*this = xAddress;
	}
	Asura_NetAddress& operator = (u_char ucValue)
	{
		m_xAdhocAddress.data[0] = m_xAdhocAddress.data[1] = m_xAdhocAddress.data[2] = 
			m_xAdhocAddress.data[3] = m_xAdhocAddress.data[4] = m_xAdhocAddress.data[5] = ucValue;
		return *this;
	}
	Asura_NetAddress& operator = (u_int uBSDAddress)
	{
		m_uBSDAddress = uBSDAddress;
		m_xAdhocAddress.data[4] = 0;
		m_xAdhocAddress.data[5] = 0;
		return *this;
	}
	Asura_NetAddress& operator = (const SceNetEtherAddr& xAddress)
	{
		m_xAdhocAddress.data[0] = xAddress.data[0];
		m_xAdhocAddress.data[1] = xAddress.data[1];
		m_xAdhocAddress.data[2] = xAddress.data[2];
		m_xAdhocAddress.data[3] = xAddress.data[3];
		m_xAdhocAddress.data[4] = xAddress.data[4];
		m_xAdhocAddress.data[5] = xAddress.data[5];
		return *this;
	}
	bool operator == (const Asura_NetAddress& xAddress) const
	{
		return  (m_xAdhocAddress.data[0] == xAddress.m_xAdhocAddress.data[0]) &&
				(m_xAdhocAddress.data[1] == xAddress.m_xAdhocAddress.data[1]) &&
				(m_xAdhocAddress.data[2] == xAddress.m_xAdhocAddress.data[2]) &&
				(m_xAdhocAddress.data[3] == xAddress.m_xAdhocAddress.data[3]) &&
				(m_xAdhocAddress.data[4] == xAddress.m_xAdhocAddress.data[4]) &&
				(m_xAdhocAddress.data[5] == xAddress.m_xAdhocAddress.data[5]);
	}

	const SceNetEtherAddr& GetAdhocAddress() const { return m_xAdhocAddress; }
	u_int GetBSDAddress() const { return m_uBSDAddress; }

private:

	union
	{
		SceNetEtherAddr m_xAdhocAddress;
		u_int			m_uBSDAddress;
	};
};

#endif//__ASURA_PLATFORM_NETSOCKET_H__
