//
// Asura_PSP_CharacterRenderer.h
//
// Created by KJFL - Tuesday, March 02, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PSP_CHARACTERRENDERER_H__
#define __ASURA_PSP_CHARACTERRENDERER_H__

#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_HashTree.h"

#include "Asura_PSP_Graphics.h" // For Asura_PSP_Matrix4
#include "Rendering/Asura_PSP_CharacterRenderer_Internal.h"

class Asura_Stream;

class Asura_PSP_CharacterRender_Handler
{
	friend class Asura_Hierarchy_System;
	friend class Asura_PSP_CharacterRenderer;

public:
	// Public functions...
	Asura_PSP_CharacterRender_Handler(void)	{}
//	~Asura_PSP_CharacterRender_Handler()	{}

	// Reset function...
	void						Reset				(void);

	// Called by the chunkloaders
	// Returns 0 if there is no more space or if a strip of that hash already exists (in any handler), with the appropriate asserts
	Asura_PSP_SmoothSkin*		AddCharacter		(Asura_Hash_ID uHash);

	// get strip for given name; asserts and returns 0 if name not found
	const Asura_PSP_SmoothSkin*	GetCharacter		(const Asura_Hierarchy_Skin* pxSkin) const	{ return(m_xHashTree.GetDataByHash(pxSkin->m_uNameHashID)); }

	// Gets the material used by the skin.
	int							GetMaterialIndex	(const Asura_Hierarchy_Skin* pxSkin) const;

private:
	// Private variables...
	Asura_TemplateHashTree<Asura_PSP_SmoothSkin>	m_xHashTree;
};

class Asura_PSP_CharacterRenderer
{
	friend class Asura_Hierarchy_System;
	friend class Asura_PSP_CharacterRender_Handler;

public:
	typedef void FnBeginRendering(Asura_PSP_Matrix4 *xViewMatrix, Asura_PSP_Matrix4* xObjectOrientation);
	typedef int  FnSetRenderingStates(const Asura_PSP_SmoothSkin_Group& xGroup, const float fAlpha, const int iTextureOverride );
	typedef void FnEndRendering(void);


	static 	void 								Initialise			(void);
	static 	void 								Reset				(void);
	static 	void 								Shutdown			(void);

	static	int									SetDefaultRenderingStates(const Asura_PSP_SmoothSkin_Group& xGroup, const float fAlpha, const int iTextureOverride );
	static 	void								Render				(const Asura_Hierarchy_Skin* pxSkin, const Asura_Bounding_Box& xLightingBB, const float fAlpha, const int iTextureOverride );

#ifdef ASURA_HIERARCHY_QUAD
	static void	SetViewMatrix( const Asura_Matrix_3x3& rxOri, const Asura_Vector_3& rxPos );
#endif

	// Gets the material used by the skin.
	static	int									GetMaterialIndex	(const Asura_Hierarchy_Skin* pxSkin)	{ return(GetCurrentHandler().GetMaterialIndex(pxSkin)); }

#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)
	// Handler functions...
	static	void								ReparentHandler		(Asura_PSP_CharacterRender_Handler& xSourceHandler, const Asura_Hash_ID uNewParentHash);
	static	void								ReparentHandler		(const Asura_Hash_ID uSourceHash, const Asura_Hash_ID uNewParentHash);	

	static	Asura_PSP_CharacterRender_Handler*	CreateHandler		(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash);
	static	bool								DestroyHandler		(const Asura_Hash_ID uHash);
	static	void								WriteHandlerReport	(u_int uHandler, Asura_Stream& xLog);

	static	void								CreateAndSetHandler	(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash)	{ s_pxCurrentHandler = CreateHandler(uHash, uParentHandlerHash); }
	static	void								SetCurrentHandler	(Asura_PSP_CharacterRender_Handler* pxHandler)						{ s_pxCurrentHandler = pxHandler; }
	static	void								SetCurrentHandler	(const Asura_Hash_ID uHash)											{ s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHash); }
	static	Asura_PSP_CharacterRender_Handler&	GetCurrentHandler	(void)																{ return(*s_pxCurrentHandler); }
	static	Asura_PSP_CharacterRender_Handler*	GetHandler			(const Asura_Hash_ID uHash)											{ return(s_xHandlerTree.GetDataByHash(uHash)); }
#else
	// Handler functions...
	static	Asura_PSP_CharacterRender_Handler&	GetCurrentHandler	(void)																{ return(s_xHandler); }
#endif

	static void inline DrawArray(unsigned int uPrimType, unsigned int uVertexFormat, unsigned int uVertices, void* pIndices, void* pBuffer); 

	static void SetBeginRenderingCharacterCallback ( FnBeginRendering* pxFunction )
	{
		s_pxBeginRenderingCharacter = pxFunction;
	}

	static void SetRenderingStatesRenderCharacterCallback ( FnSetRenderingStates* pxFunction )
	{
		s_pxSetRenderingStatesCharacter = pxFunction;
	}

	static void SetEndRenderingCharacterCallback ( FnEndRendering* pxFunction )
	{
		s_pxEndRenderingCharacter = pxFunction;
	}
	
private:
#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)
	// Private variables...
	static	Asura_PSP_CharacterRender_Handler*							s_pxCurrentHandler;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_PSP_CharacterRender_Handler>	s_xHandlerTree;			// Tree of all of the handlers available.
#else
	// Private variables...
	static	Asura_PSP_CharacterRender_Handler							s_xHandler;				// Single static-instance of a zone.
#endif

#ifdef ASURA_HIERARCHY_QUAD
	static	Asura_AMatrix4x4											s_xView;
	static	Asura_AMatrix4x4											s_axBonePosAndOri[ ASURA_MAX_NO_OF_BONES_PER_MODEL ];
#else
	static	Asura_PSP_Matrix4											s_axBonePosAndOri[ASURA_MAX_NO_OF_BONES_PER_MODEL];
#endif

	static	int															s_iLastMaterial;

	static	FnBeginRendering*											s_pxBeginRenderingCharacter;
	static	FnSetRenderingStates*										s_pxSetRenderingStatesCharacter;
	static	FnEndRendering*												s_pxEndRenderingCharacter;
};

#endif//__ASURA_PSP_CHARACTERRENDERER_H__
