//
// Asura_PSP_ObjectHierarchyRenderer.h
//
// Created by KJFL - Thursday, April 08, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_PSP_OBJECTHIERARCHYRENDERER_H__
#define __ASURA_PSP_OBJECTHIERARCHYRENDERER_H__

#include "Core/Asura_HashTree.h"
#include "Core/Asura_Maths.h"
#include "Rendering/Asura_PSP_ObjectHierarchyRenderer_Internal.h"
#include "Rendering/Asura_PSP_Graphics.h"

class Asura_Animation;
class Asura_Hierarchy_Skin;
class Asura_Stream;

class Asura_PSP_ObjectHierarchyRender_Handler
{
	friend class Asura_PSP_ObjectHierarchyRenderer;
	friend class Asura_Chunk_ResourceFile_PSP_ObjectHierarchy;

public:
	// Public functions...
	Asura_PSP_ObjectHierarchyRender_Handler  (void)	{}
//	~Asura_PSP_ObjectHierarchyRender_Handler ()		{}

	void								Reset				(void);

	//! Gets the material used by the skin.
	int									GetMaterialIndex	(const Asura_Hierarchy_Skin* pxSkin) const;


private:
	// Private variables...
	Asura_TemplateHashTree<Asura_PSP_ObjectHierarchy>	m_xHashTree;

	// Private functions...
	Asura_PSP_ObjectHierarchy*			AddHierarchy		(const Asura_Hash_ID uHash);
	const Asura_PSP_ObjectHierarchy*	GetHierarchy		(const Asura_Hash_ID uHash) const	{ return(m_xHashTree.GetDataByHash(uHash)); }
};


class Asura_PSP_ObjectHierarchyRenderer
{
	friend class Asura_PSP_ObjectHierarchyRender_Handler;

public:
	typedef void FnBeginRendering(void);
	typedef bool FnSetRenderingStates(const Asura_Animation* pxAnim, Asura_PSP_Matrix4 *xViewMatrix, Asura_PSP_Matrix4* xObjectOrientation, const Asura_PSP_ObjectHierarchy_Strip& xStrip, const float fAlpha, const int iTextureOverride );
	typedef void FnEndRendering(void);

	//! One-time initialisation function, called (indirectly) by Asura_Core::Initialise()
	static void Initialise(void);
	//! Removes all object hierarchies, without deleting hash tree and storage array
	static void Reset();
	//! One-time shutdown function, called (indirectly) by Asura_Core::Shutdown()
	static void Shutdown(void);

	//! BeginRender() should be called once per hierarchy before the Render() calls
	static void BeginRender( const class Asura_Animation* pxAnim, const class Asura_Hierarchy_Skin* pxSkin );
	//! Draws one object from an object hierarchy.
	//! All Render() calls for an hierarchy should be surrounded by just one BeginRender/EndRender pairing.

	static bool SetDefaultRenderingStates(const Asura_Animation* pxAnim, const Asura_PSP_ObjectHierarchy_Strip& xStrip, const float fAlpha, const int iTextureOverride ); //Returns true if material has the flag AsuraMaterialFlag_SelfIlluminate on

#ifndef ASURA_HIERARCHY_QUAD
	static void Render( const class Asura_Animation* pxAnim, const class Asura_Hierarchy_Skin* pxSkin, const int iSectionDataIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation,
						const int iTextureIDOverride );
#else
	static void	SetViewMatrix( const Asura_Matrix_3x3& rxOri, const Asura_Vector_3& rxPos );
	static void Render(	const class Asura_Animation* pxAnim, const class Asura_Hierarchy_Skin* pxSkin, const int iSectionDataIndex, Asura_AMatrix4x4& rxTransform,
						const int iTextureIDOverride );
#endif

	//! EndRender() should be called once per hierarchy after Render() calls are completed
	static void EndRender( void );

	//! BeginShadowRender() should be called once per hierarchy before the RenderShadow() calls
	static void BeginShadowRender( const class Asura_Hierarchy_Skin* pxSkin, const Asura_Vector_3& xShadowPosition );
	//! Draws one object from an object hierarchy.
	//! All RenderShadow calls for an hierarchy should be surrounded by just one BeginShadowRender/EndShadowRender pairing.
	static void RenderShadow( const int iSectionDataIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation );
	//! EndShadowRender() should be called once per hierarchy after RenderShadow() calls are completed
	static void EndShadowRender( void );

	static int GetMaterialIndex( const class Asura_Hierarchy_Skin* pxSkin ) { return(GetCurrentHandler().GetMaterialIndex(pxSkin)); }

	static void SetBeginRenderingHierarchyCallback ( FnBeginRendering* pxFunction )
	{
		s_pxBeginRenderingHierarchy = pxFunction;
	}

	static void SetRenderingStatesRenderHierarchyCallback ( FnSetRenderingStates* pxFunction )
	{
		s_pxSetRenderingStatesHierarchy = pxFunction;
	}

	static void SetEndRenderingHierarchyCallback ( FnEndRendering* pxFunction )
	{
		s_pxEndRenderingHierarchy = pxFunction;
	}

#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)
	// Handler functions...
	static	void										ReparentHandler		(Asura_PSP_ObjectHierarchyRender_Handler& xSourceHandler, const Asura_Hash_ID uNewParentHash);
	static	void										ReparentHandler		(const Asura_Hash_ID uSourceHash, const Asura_Hash_ID uNewParentHash);	

	static	Asura_PSP_ObjectHierarchyRender_Handler*	CreateHandler		(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash);
	static	bool										DestroyHandler		(const Asura_Hash_ID uHash);
	static	void										WriteHandlerReport	(u_int uHandler, Asura_Stream& xLog);

	static	void										CreateAndSetHandler	(const Asura_Hash_ID uHash, const Asura_Hash_ID uParentHandlerHash)	{ s_pxCurrentHandler = CreateHandler(uHash, uParentHandlerHash); }
	static	void										SetCurrentHandler	(Asura_PSP_ObjectHierarchyRender_Handler* pxHandler)				{ s_pxCurrentHandler = pxHandler; }
	static	void										SetCurrentHandler	(const Asura_Hash_ID uHash)											{ s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHash); }
	static	Asura_PSP_ObjectHierarchyRender_Handler&	GetCurrentHandler	(void)																{ return(*s_pxCurrentHandler); }
	static	Asura_PSP_ObjectHierarchyRender_Handler*	GetHandler			(const Asura_Hash_ID uHash)											{ return(s_xHandlerTree.GetDataByHash(uHash)); }
#else
	// Handler functions...
	static	Asura_PSP_ObjectHierarchyRender_Handler&	GetCurrentHandler	(void)																{ return(s_xHandler); }
#endif

private:
#if defined(ASURA_USE_MULTIPLE_HANDLERS) || defined(ASURA_USE_STREAMING)
	// Private variables...
	static	Asura_PSP_ObjectHierarchyRender_Handler*						s_pxCurrentHandler;		// Currently active handler.
	static	Asura_TemplateHashTree<Asura_PSP_ObjectHierarchyRender_Handler>	s_xHandlerTree;			// Tree of all of the handlers available.
#else
	// Private variables...
	static	Asura_PSP_ObjectHierarchyRender_Handler							s_xHandler;				// Single static-instance of a zone.
#endif

	// data on current status of drawing
	static	bool															s_bNormalRendering;
	static	bool															s_bShadowRendering;
	static	const Asura_PSP_ObjectHierarchy*								s_pxCurrentHierarchyObject;

#ifdef ASURA_HIERARCHY_QUAD
	static Asura_AMatrix4x4	s_xView;
#endif
	static	FnBeginRendering*												s_pxBeginRenderingHierarchy;
	static	FnSetRenderingStates*											s_pxSetRenderingStatesHierarchy;
	static	FnEndRendering*													s_pxEndRenderingHierarchy;
};



#endif//__ASURA_PSP_OBJECTHIERARCHYRENDERER_H__
