//
// Asura_PSP_OverlayRenderer.cpp
//
// Created by KJFL - Thursday, April 08, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_Render.h"

#include "Core/Asura_Maths.h"
#include "Debugging/Asura_ScreenShot_System.h"
#include "ProjectSettings/Asura_PSPConfiguration.h"
#include "Rendering/Asura_PSP_Graphics.h"
#include "Rendering/Asura_PSP_RenderStates.h"
#include "Rendering/Asura_PSP_TextureManagement.h"
#include "Core/Asura_String.h"

#include <kernel.h>

// D E F I N E S /////////////////////////////////////////////////////////////

// non-rotated
#if !defined(ASURA_PSP_MAX_QUADS_PER_FRAME)
#define ASURA_PSP_MAX_QUADS_PER_FRAME	512
#endif	// !defined(ASURA_PSP_MAX_QUADS_PER_FRAME)

// rotated
#if !defined(ASURA_PSP_MAX_RQUADS_PER_FRAME)
#define ASURA_PSP_MAX_RQUADS_PER_FRAME	(ASURA_PSP_MAX_QUADS_PER_FRAME / 8)
#endif	// !defined(ASURA_PSP_MAX_RQUADS_PER_FRAME)

// gouraud
#if !defined(ASURA_PSP_MAX_GQUADS_PER_FRAME)
#define ASURA_PSP_MAX_GQUADS_PER_FRAME	(ASURA_PSP_MAX_QUADS_PER_FRAME / 16)
#endif	// !defined(ASURA_PSP_MAX_GQUADS_PER_FRAME)

#define ASURA_PSP_QUAD_BUFFER_SIZE		(ASURA_PSP_MAX_QUADS_PER_FRAME * 4)
#define ASURA_PSP_RQUAD_BUFFER_SIZE		(ASURA_PSP_MAX_RQUADS_PER_FRAME * 4)
#define ASURA_PSP_GQUAD_BUFFER_SIZE		(ASURA_PSP_MAX_GQUADS_PER_FRAME * 4)

#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
DEBUGVAR bool dbg_bForceHiResOverlays = false;
#endif

// C L A S S E S /////////////////////////////////////////////////////////////
#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
class Asura_PSP_Overlay_VertexHiResScreenshot
{
	public:
		
		// order of member determined by PSP GPU
		// data sizes determined by THROUGH mode - do not change
		float			m_fU;
		float			m_fV;

		u_int			m_uDiffuse;

		Asura_Vector_3	m_xPosition;
};
#endif	// defined(ASURA_USE_SCREENSHOT_SYSTEM)

class Asura_PSP_Overlay_VertexGouraudShaded
{
	public:
		
		// order of member determined by PSP GPU
		// data sizes determined by THROUGH mode - do not change
		u_short			m_usU;
		u_short			m_usV;
		u_int			m_uDiffuse;
		short			m_asPosition[3];

		// 14 bytes (rounded to 16)
};

class Asura_PSP_Overlay_VertexFlatShaded
{
	public:
		
		// order of member determined by PSP GPU
		// data sizes determined by THROUGH mode - do not change
		u_short			m_usU;
		u_short			m_usV;
		short			m_asPosition[3];

		// 10 bytes (rounded to 12)
};
// TODO: can we pack vertices together?

// I N T E R N A L   F U N C T I O N   P R O T O T Y P E S ///////////////////
static void Asura_PSP_RenderOverlayQuadForHiResScreenshot(const Asura_Overlay_Quad& xQuad);

// G L O B A L   D A T A /////////////////////////////////////////////////////
#ifdef ASURA_PSP_DEFERREDRENDER
	#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
		static Asura_PSP_Overlay_VertexHiResScreenshot g_axVerticesHiRes[ 2 ][ASURA_PSP_QUAD_BUFFER_SIZE] __attribute__ ((aligned(16)));
	#endif
	static Asura_PSP_Overlay_VertexFlatShaded g_axVerticesFS[ 2 ][ASURA_PSP_QUAD_BUFFER_SIZE] __attribute__ ((aligned(16)));
	static Asura_PSP_Overlay_VertexFlatShaded g_axVerticesRotFS[ 2 ][ASURA_PSP_RQUAD_BUFFER_SIZE] __attribute__ ((aligned(16)));
	static Asura_PSP_Overlay_VertexGouraudShaded g_axVerticesGS[ 2 ][ASURA_PSP_GQUAD_BUFFER_SIZE] __attribute__ ((aligned(16)));
#else
	#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
		static Asura_PSP_Overlay_VertexHiResScreenshot g_axVerticesHiRes[ 1 ][ASURA_PSP_QUAD_BUFFER_SIZE] __attribute__ ((aligned(16)));
	#endif
	static Asura_PSP_Overlay_VertexFlatShaded g_axVerticesFS[ 1 ][ASURA_PSP_QUAD_BUFFER_SIZE] __attribute__ ((aligned(16)));
	static Asura_PSP_Overlay_VertexFlatShaded g_axVerticesRotFS[ 1 ][ASURA_PSP_RQUAD_BUFFER_SIZE] __attribute__ ((aligned(16)));
	static Asura_PSP_Overlay_VertexGouraudShaded g_axVerticesGS[ 1 ][ASURA_PSP_GQUAD_BUFFER_SIZE] __attribute__ ((aligned(16)));
#endif

#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
static u_int g_uOffsetHiRes = 0;
#endif
static u_int g_uOffsetFS = 0;
static u_int g_uOffsetRotFS = 0;
static u_int g_uOffsetGS = 0;

// F U N C T I O N S /////////////////////////////////////////////////////////
void FlushQuads()
{
#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
	g_uOffsetHiRes = 0;
#endif
	g_uOffsetFS = 0;
	g_uOffsetRotFS = 0;
	g_uOffsetGS = 0;
}

static void RenderQuadNonRotatedFlatShaded( const Asura_Overlay_Quad& xQuad )
{
	Asura_Assert(g_uOffsetFS < ASURA_PSP_QUAD_BUFFER_SIZE - 1, "Too many overlay quads this frame");
	if (g_uOffsetFS >= ASURA_PSP_QUAD_BUFFER_SIZE) return;

	// PSP rectangle primitives can't handle UVs that don't match up with XYs. To fix this 
	// assert you need to set m_bNonRotated = false; after calling SetRectXY() in the calling code.
	Asura_Assert( xQuad.TextureIndex == ASURA_TEXTURE_NONE || 
				 (xQuad.U[0] <= xQuad.U[2] && xQuad.X[0] <= xQuad.X[2] && 
				  xQuad.V[0] <= xQuad.V[2] && xQuad.Y[0] <= xQuad.Y[2]), 
				 "non-rotated quads must be top-left to bottom-right with non-rotated UVs" );

	float fTextureWidth = 0.0f;
	float fTextureHeight = 0.0f;

	// KJFL - 01 April 2005 - unfortunately PSP through mode uses UV
	// coords that are in texels, so we need info on the texture size
	int iTextureIndex = xQuad.TextureIndex;
	if ( iTextureIndex != ASURA_TEXTURE_NONE )
	{
		extern Asura_PSP_Texture* g_apxTextureStorage[];
	
		Asura_PSP_Texture*& pxTexture = g_apxTextureStorage[iTextureIndex];

		if (pxTexture)
		{
			fTextureWidth = (float)pxTexture->m_usWidth;
			fTextureHeight = (float)pxTexture->m_usHeight;
		}
	}

	Asura_PSP_Overlay_VertexFlatShaded* const pxBaseVert = &g_axVerticesFS[ Asura_PSP_Graphics::GetCurrentDisplayList() ][g_uOffsetFS];

	Asura_PSP_Overlay_VertexFlatShaded* pxVert = pxBaseVert;
	// top left
	{
		const u_int uU = Asura_Maths::Min(Asura_Maths::FloatToInt_Near(xQuad.U[0]*fTextureWidth), 65535);
		const u_int uV = Asura_Maths::Min(Asura_Maths::FloatToInt_Near(xQuad.V[0]*fTextureHeight), 65535);

		pxVert->m_usU = static_cast<u_short>(uU);
		pxVert->m_usV = static_cast<u_short>(uV);

		pxVert->m_asPosition[0] = Asura_Maths::FloatToInt_Near(xQuad.X[0]);
		pxVert->m_asPosition[1] = Asura_Maths::FloatToInt_Near(xQuad.Y[0]);
		pxVert->m_asPosition[2] = 32767; // 32767 seems to be near plane
	}

	++pxVert;
	// bottom-right
	{
		const u_int uU = Asura_Maths::Min(Asura_Maths::FloatToInt_Near(xQuad.U[2]*fTextureWidth), 65535);
		const u_int uV = Asura_Maths::Min(Asura_Maths::FloatToInt_Near(xQuad.V[2]*fTextureHeight), 65535);

		pxVert->m_usU = static_cast<u_short>(uU);
		pxVert->m_usV = static_cast<u_short>(uV);

		pxVert->m_asPosition[0] = Asura_Maths::FloatToInt_Near(xQuad.X[2]);
		pxVert->m_asPosition[1] = Asura_Maths::FloatToInt_Near(xQuad.Y[2]);
		pxVert->m_asPosition[2] = 32767; // 32767 seems to be near plane
	}


	// KJFL - 2004-02-10 16:24:42 - Red and blue appear to be swapped on the PSP vertex format
	u_int uRGBA		= xQuad.Colour;
	u_int uRed		= (uRGBA&0x00ff0000)>>16;
	u_int uAlpha	= (uRGBA&0xff000000);
	u_int uGreen	= (uRGBA&0x0000ff00);
	u_int uBlue		= (uRGBA&0x000000ff)<<16;

	u_int uPSP_RGBA = (uRed) + (uGreen) + (uBlue) + (uAlpha);

	// JG - 03/01/07: Replaced GU_Color with another function we keep trace of its state, also invalidated 
	// the material type internal state so any other standard material type will override this settings
	//GU_Color(uPSP_RGBA); -> Replaced by
	Asura_PSP_Graphics::SetMaterialType( ASURA_PSP_GRAPHICS_MATERIALTYPE_MANUAL );
	GU_Material(SCEGU_AMBIENT, uPSP_RGBA);// Model ambient color

	sceKernelDcacheWritebackRange((const char*)(pxBaseVert), 2 * sizeof(Asura_PSP_Overlay_VertexFlatShaded));

	GU_DrawArray
	(
		SCEGU_PRIM_RECTANGLES,
		SCEGU_TEXTURE_USHORT|SCEGU_VERTEX_SHORT|SCEGU_THROUGH,
		2,
		0,
		pxBaseVert
	);

	g_uOffsetFS += 2;

}
static void RenderQuadRotatedFlatShaded( const Asura_Overlay_Quad& xQuad )
{
	Asura_Assert(g_uOffsetRotFS < ASURA_PSP_RQUAD_BUFFER_SIZE - 3, "Too many rotated overlay quads this frame");
	if (g_uOffsetRotFS >= ASURA_PSP_RQUAD_BUFFER_SIZE) return;

	float fTextureWidth = 0.0f;
	float fTextureHeight = 0.0f;

	// KJFL - 01 April 2005 - unfortunately PSP through mode uses UV
	// coords that are in texels, so we need info on the texture size
	int iTextureIndex = xQuad.TextureIndex;
	if ( iTextureIndex != ASURA_TEXTURE_NONE )
	{
		extern Asura_PSP_Texture* g_apxTextureStorage[];
	
		Asura_PSP_Texture*& pxTexture = g_apxTextureStorage[iTextureIndex];

		if (pxTexture)
		{
			fTextureWidth = (float)pxTexture->m_usWidth;
			fTextureHeight = (float)pxTexture->m_usHeight;
		}
	}

	Asura_PSP_Overlay_VertexFlatShaded* const pxBaseVert = &g_axVerticesRotFS[ Asura_PSP_Graphics::GetCurrentDisplayList() ][g_uOffsetRotFS];

	Asura_PSP_Overlay_VertexFlatShaded* pxVert = pxBaseVert;
	
  	u_int auSwitchedIndices[4] = {0, 3, 2, 1};
	
	for (u_int uVertexIndex = 0; uVertexIndex < 4; ++uVertexIndex)
	{
  		u_int uSourceIndex = auSwitchedIndices[uVertexIndex];
		const u_int uU = Asura_Maths::Min(Asura_Maths::FloatToInt_Near(xQuad.U[uSourceIndex]*fTextureWidth), 65535);
		const u_int uV = Asura_Maths::Min(Asura_Maths::FloatToInt_Near(xQuad.V[uSourceIndex]*fTextureHeight), 65535);

		pxVert->m_usU = static_cast<u_short>(uU);
		pxVert->m_usV = static_cast<u_short>(uV);

		pxVert->m_asPosition[0] = Asura_Maths::FloatToInt_Near(xQuad.X[uSourceIndex]);
		pxVert->m_asPosition[1] = Asura_Maths::FloatToInt_Near(xQuad.Y[uSourceIndex]);
		pxVert->m_asPosition[2] = 32767; // 32767 seems to be near plane

		++pxVert;
	}

	// KJFL - 2004-02-10 16:24:42 - Red and blue appear to be swapped on the PSP vertex format
	u_int uRGBA		= xQuad.Colour;
	u_int uRed		= (uRGBA&0x00ff0000)>>16;
	u_int uAlpha	= (uRGBA&0xff000000);
	u_int uGreen	= (uRGBA&0x0000ff00);
	u_int uBlue		= (uRGBA&0x000000ff)<<16;

	u_int uPSP_RGBA = (uRed) + (uGreen) + (uBlue) + (uAlpha);
	
	// JG - 03/01/07: Replaced GU_Color with another function we keep trace of its state, also invalidated 
	// the material type internal state so any other standard material type will override this settings
	//GU_Color(uPSP_RGBA); -> Replaced by
	Asura_PSP_Graphics::SetMaterialType( ASURA_PSP_GRAPHICS_MATERIALTYPE_MANUAL );
	GU_Material(SCEGU_AMBIENT, uPSP_RGBA);// Model ambient color
	
	sceKernelDcacheWritebackRange((const char*)(pxBaseVert), 4 * sizeof(Asura_PSP_Overlay_VertexFlatShaded));

	GU_DrawArray
	(
		SCEGU_PRIM_TRIANGLE_FAN,
		SCEGU_TEXTURE_USHORT|SCEGU_VERTEX_SHORT|SCEGU_THROUGH,
		4,
		0,
		pxBaseVert
	);

	g_uOffsetRotFS += 4;
}
static void RenderQuadGouraudShaded( const Asura_Overlay_Quad& xQuad )
{
	Asura_Assert(g_uOffsetGS < ASURA_PSP_GQUAD_BUFFER_SIZE - 3, "Too many gouraud overlay quads this frame");
	if (g_uOffsetGS >= ASURA_PSP_GQUAD_BUFFER_SIZE) return;

	float fTextureWidth = 0.0f;
	float fTextureHeight = 0.0f;

	// KJFL - 01 April 2005 - unfortunately PSP through mode uses UV
	// coords that are in texels, so we need info on the texture size
	int iTextureIndex = xQuad.TextureIndex;
	if ( iTextureIndex != ASURA_TEXTURE_NONE )
	{
		extern Asura_PSP_Texture* g_apxTextureStorage[];
	
		Asura_PSP_Texture*& pxTexture = g_apxTextureStorage[iTextureIndex];

		if (pxTexture)
		{
			fTextureWidth = (float)pxTexture->m_usWidth;
			fTextureHeight = (float)pxTexture->m_usHeight;
		}
	}

	Asura_PSP_Overlay_VertexGouraudShaded* const pxBaseVert = &g_axVerticesGS[ Asura_PSP_Graphics::GetCurrentDisplayList() ][g_uOffsetGS];

	Asura_PSP_Overlay_VertexGouraudShaded* pxVert = pxBaseVert;
	
  	u_int auSwitchedIndices[4] = {0, 3, 2, 1};
	
	for (u_int uVertexIndex = 0; uVertexIndex < 4; ++uVertexIndex)
	{
  		u_int uSourceIndex = auSwitchedIndices[uVertexIndex];

		pxVert->m_usU = static_cast<u_short>(Asura_Maths::Min(Asura_Maths::FloatToInt_Near(xQuad.U[uSourceIndex]*fTextureWidth), 65535));
		pxVert->m_usV = static_cast<u_short>(Asura_Maths::Min(Asura_Maths::FloatToInt_Near(xQuad.V[uSourceIndex]*fTextureHeight), 65535));

  		// KJFL - 2004-02-10 16:24:42 - Red and blue appear to be swapped on the PSP vertex format
  		const u_int uRGBA				= xQuad.GouraudColour[uSourceIndex];
  		const u_int uRed				= (uRGBA&0x00ff0000)>>16;
  		const u_int uGreenAndAlpha		= (uRGBA&0xff00ff00);
  		const u_int uBlue				= (uRGBA&0x000000ff)<<16;
  	
  		pxVert->m_uDiffuse = (uRed) + (uGreenAndAlpha) + (uBlue);

		pxVert->m_asPosition[0] = Asura_Maths::FloatToInt_Near(xQuad.X[uSourceIndex]);
		pxVert->m_asPosition[1] = Asura_Maths::FloatToInt_Near(xQuad.Y[uSourceIndex]);
		pxVert->m_asPosition[2] = 32767; // 32767 seems to be near plane

		++pxVert;
	}

    // (MH - 30.03.2007) need to set a material type or we will get different behaviour depending
	// on what was rendered before. I think the settings for environment are correct here.
	Asura_PSP_Graphics::SetMaterialType( ASURA_PSP_GRAPHICS_MATERIALTYPE_ENV);

	sceKernelDcacheWritebackRange((const char*)(pxBaseVert), 4 * sizeof(Asura_PSP_Overlay_VertexGouraudShaded));

	GU_DrawArray
	(
		SCEGU_PRIM_TRIANGLE_FAN,
		SCEGU_TEXTURE_USHORT|SCEGU_COLOR_PF8888|SCEGU_VERTEX_SHORT|SCEGU_THROUGH,
		4,
		0,
		pxBaseVert
	);

	g_uOffsetGS += 4;
}

void Asura_Overlay_Quad::Platform_Render()
{
#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
	if (Asura_ScreenShot_System::IsTakingHiResScreenshot() || dbg_bForceHiResOverlays)
	{
		Asura_PSP_RenderOverlayQuadForHiResScreenshot( *this );
		return;
	}
#endif

	Asura_PSP_Graphics::SetUseColourDoubling(false);
	Asura_PSP_Graphics::SetUseLighting( false );
	
	// set renderstates
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_PSP_TextureManagement::RequestTextureIndex( this->TextureIndex );
	Asura_Render::CurrentStates.RequestTranslucencyMode( this->TranslucencyMode );
	Asura_Render::CurrentStates.RequestZBufferEnabled(false);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

	// If we claim to be non-rotated, check co-ordinate ordering: PSP doesn't support flipped coords as non-rotated quads
	if( m_bNonRotated && (U[0] > U[2] || X[0] > X[2] ||	V[0] > V[2] || Y[0] > Y[2]) )
	{
		m_bNonRotated = false;
	}

	if (m_bGouraudShaded)
	{
		RenderQuadGouraudShaded( *this );
	}
	else
	{
		if (m_bNonRotated)
		{
			RenderQuadNonRotatedFlatShaded( *this );
		}
		else
		{
			RenderQuadRotatedFlatShaded( *this );
		}
	}

	// restore renderstates
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
}

#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
static void Asura_PSP_RenderOverlayQuadForHiResScreenshot(const Asura_Overlay_Quad& xQuad)
{
	Asura_Assert(g_uOffsetHiRes < ASURA_PSP_QUAD_BUFFER_SIZE, "Too many overlay quads this frame");
	if(g_uOffsetHiRes >= ASURA_PSP_QUAD_BUFFER_SIZE) return;

	Asura_PSP_Graphics::SetUseColourDoubling(false);
	Asura_PSP_Graphics::SetUseLighting( false );

	u_int uPSP_RGBA = 0;
	if (!xQuad.m_bGouraudShaded)
	{
		// KJFL - 2004-02-10 16:24:42 - Red and blue appear to be swapped on the PSP vertex format
		u_int uRGBA		= xQuad.Colour;
		u_int uRed		= (uRGBA&0x00ff0000)>>16;
		u_int uAlpha	= (uRGBA&0xff000000);
		u_int uGreen	= (uRGBA&0x0000ff00);
		u_int uBlue		= (uRGBA&0x000000ff)<<16;
		// KOR halve colours since we're going to double whole framebuffer
//		uRed	= (uRed   / 2);
//		uGreen	= (uGreen / 2)&0x0000ff00;
//		uBlue	= (uBlue  / 2)&0x00ff0000;
	
		uPSP_RGBA = (uRed) + (uGreen) + (uBlue) + (uAlpha);
	}

	for (u_int uVertexIndex = 0; uVertexIndex < 4; uVertexIndex++)
	{
		// KOR - 24/8/2004 - using rects instead of triangle fans
		// So we need the top left (index 0 = 0 * 2) and bottom right (index 2 = 1 * 2)
		//const u_int uSourceIndex = uVertexIndex * 2;
		// KOR - 1/10/2004 - this doesn't work for rotated UVs, unfortunately.
		// Perhaps a special separate renderer would be better. For now, back to trifan
		u_int uSourceIndex;
		// Hacktastic - Flips the ordering of vertices as follows:
		// 
		// uVertexIndex:    uSourceIndex:
		//     0-----1         0-----3
		//     |     |   --\   |     |
		//     |     |   --/   |     |
		//     3-----2         1-----2
		//
		switch(uVertexIndex)
		{
			default://fallthrough to fix warning
			case 0: uSourceIndex = 0; break;
			case 1: uSourceIndex = 3; break;
			case 2: uSourceIndex = 2; break;
			case 3: uSourceIndex = 1; break;
		}

		//const u_int uSourceIndex = uVertexIndex;


		// Set the texture-coords directly...

		g_axVerticesHiRes[ Asura_PSP_Graphics::GetCurrentDisplayList() ][uVertexIndex + g_uOffsetHiRes].m_fU = xQuad.U [uSourceIndex];
		g_axVerticesHiRes[ Asura_PSP_Graphics::GetCurrentDisplayList() ][uVertexIndex + g_uOffsetHiRes].m_fV = xQuad.V [uSourceIndex];

		// Set the vertex coords directly...
		g_axVerticesHiRes[ Asura_PSP_Graphics::GetCurrentDisplayList() ][uVertexIndex + g_uOffsetHiRes].m_xPosition.x	= xQuad.X [uSourceIndex];
		g_axVerticesHiRes[ Asura_PSP_Graphics::GetCurrentDisplayList() ][uVertexIndex + g_uOffsetHiRes].m_xPosition.y	= xQuad.Y [uSourceIndex];
		g_axVerticesHiRes[ Asura_PSP_Graphics::GetCurrentDisplayList() ][uVertexIndex + g_uOffsetHiRes].m_xPosition.z	= -1.0f;

		if (xQuad.m_bGouraudShaded)
		{
			// KJFL - 2004-02-10 16:24:42 - Red and blue appear to be swapped on the PSP vertex format
			u_int uRGBA				= xQuad.GouraudColour[uSourceIndex];
			u_int uRed				= (uRGBA&0x00ff0000)>>16;
			u_int uGreenAndAlpha	= (uRGBA&0xff00ff00);
			u_int uBlue				= (uRGBA&0x000000ff)<<16;
		
			uPSP_RGBA = (uRed) + (uGreenAndAlpha) + (uBlue);
		}

		g_axVerticesHiRes[ Asura_PSP_Graphics::GetCurrentDisplayList() ][uVertexIndex + g_uOffsetHiRes].m_uDiffuse = uPSP_RGBA;
	}

	const Asura_Vector_2&	xSubPixelOffset	= Asura_Render::GetSubPixelOffset ();
	const ScePspFVector3	xTranslation	= {xSubPixelOffset.x / (SCEGU_SCR_WIDTH / 2), -xSubPixelOffset.y / (SCEGU_SCR_HEIGHT / 2), 0.0f};


	// Initialise the screenshot matrices...

	GUM_MatrixMode	(SCEGU_MATRIX_WORLD);
	GUM_PushMatrix	();
	GUM_LoadIdentity	();

	GUM_MatrixMode	(SCEGU_MATRIX_VIEW);
	GUM_PushMatrix	();
	GUM_LoadIdentity	();

	GUM_MatrixMode	(SCEGU_MATRIX_PROJECTION);
	GUM_PushMatrix	();
	GUM_LoadIdentity	();
	GUM_Translate		(&xTranslation);
	GUM_Ortho			(0.0f, SCEGU_SCR_WIDTH, SCEGU_SCR_HEIGHT, 0.0f, 0.1f, 1000.0f);

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestTextureIndex( 0, xQuad.TextureIndex );
	Asura_Render::CurrentStates.RequestTranslucencyMode( xQuad.TranslucencyMode );
	Asura_Render::CurrentStates.RequestZBufferEnabled(false);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	//GU_Disable(SCEGU_DEPTH_TEST);

    // (MH - 30.03.2007) need to set a material type or we will get different behaviour depending
	// on what was rendered before. I think the settings for environment are correct here. 	
	Asura_PSP_Graphics::SetMaterialType( ASURA_PSP_GRAPHICS_MATERIALTYPE_ENV );

	sceKernelDcacheWritebackRange((const char*)(&g_axVerticesHiRes[ Asura_PSP_Graphics::GetCurrentDisplayList() ][g_uOffsetHiRes]), 4 * sizeof(Asura_PSP_Overlay_VertexHiResScreenshot));

	GUM_DrawArray
	(
		SCEGU_PRIM_TRIANGLE_FAN,
		SCEGU_TEXTURE_FLOAT|SCEGU_COLOR_PF8888|SCEGU_VERTEX_FLOAT,
		4,
		0,
		&g_axVerticesHiRes[ Asura_PSP_Graphics::GetCurrentDisplayList() ][g_uOffsetHiRes]
	);


	// Restore the matrices...

	GUM_MatrixMode	(SCEGU_MATRIX_PROJECTION);
	GUM_PopMatrix		();

	GUM_MatrixMode	(SCEGU_MATRIX_VIEW);
	GUM_PopMatrix		();

	GUM_MatrixMode	(SCEGU_MATRIX_WORLD);
	GUM_PopMatrix		();

	g_uOffsetHiRes += 4;
	//GU_Enable(SCEGU_DEPTH_TEST);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
}
#endif //#if defined(ASURA_USE_SCREENSHOT_SYSTEM)
