//
// Asura_PSP_RenderParticles.cpp
//
// Created by KJFL - Wednesday, March 31, 2004
// First-pass implementation by Andrew Weinkove - 16/04/04. Based on PC implementation.
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "SpecialFX/Asura_Particle.h"
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Render.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"

#include "Rendering/Asura_PSP_Graphics.h"

#include <kernel.h>

void Asura_Particle_System::Platform_BeginRender(bool bHeatHaze)
{
	static int s_iParticleTexture = ASURA_TEXTURE_NONE;
	if (s_iParticleTexture == ASURA_TEXTURE_NONE)
	{
		bool bWasNew = false;
		s_iParticleTexture = Asura_Texture_Management::AddTexture("\\specialfx\\partclfx.tga", &bWasNew);
		Asura_Assert(!bWasNew, "Particle texture should have been loaded already");
	}		

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestTextureIndex(0,s_iParticleTexture);
	Asura_PSP_Graphics::SetMaterialType( ASURA_PSP_GRAPHICS_MATERIALTYPE_ENV );
}

void Asura_Particle_System::Platform_Render(void)
{

}

void Asura_Particle_System::Platform_EndRender(void)
{
//	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
//	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CCW);
//	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
}


struct Asura_PSP_RenderParticle_Vertex
{
	// fully 32bit - 24 bytes
	float			m_fU;
	float			m_fV;
	unsigned		m_uColour;
	float			m_afPosition[3];
};

// KOR - 30/09/2004 - Dodgy buffer. Yuck. To be replaced with proper class when I get the time (unify with beam renderer)
#define ASURA_PSP_MAX_PCLES_PER_FRAME (1024)
#define ASURA_PSP_PARTICLE_BUFFER_SIZE (ASURA_PSP_MAX_PCLES_PER_FRAME * 6)

static Asura_PSP_RenderParticle_Vertex g_axVertices[ASURA_PSP_PARTICLE_BUFFER_SIZE] __attribute__ ((aligned(16)));
static u_int g_uOffset = 0;

void FlushParticles()
{
	g_uOffset = 0;
}

const int iASURA_PSP_PARTICLE_VERTEX_FORMAT = SCEGU_TEXTURE_FLOAT | SCEGU_VERTEX_FLOAT | SCEGU_COLOR_PF8888;

void RenderParticles(const Asura_Particle* const * ppxParticles, int iNumParticles, const Asura_Particle_Template* pxParticleTemplate)
{
	if(!iNumParticles) return;

	u_int uInitialOffset = g_uOffset;

	Asura_PSP_Graphics::SetUseColourDoubling(false);
	Asura_Vector_3 axVertex[4];

	const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();
	const Asura_Vector_3 xDX = Asura_Vector_3(pxCameraOrientation->e11, pxCameraOrientation->e21, pxCameraOrientation->e31);
	const Asura_Vector_3 xDY = Asura_Vector_3(pxCameraOrientation->e11, pxCameraOrientation->e21, pxCameraOrientation->e31);

	Asura_Render::CurrentStates.RequestTranslucencyMode(pxParticleTemplate->TranslucencyMode);

	int iParticle;
	for(iParticle = 0; iParticle < iNumParticles; ++iParticle)
	{
		Asura_Assert(g_uOffset < ASURA_PSP_PARTICLE_BUFFER_SIZE, "Too many particles this frame");
		if(g_uOffset >= ASURA_PSP_PARTICLE_BUFFER_SIZE) break;

		const Asura_Particle* pxParticlePtr = ppxParticles[iParticle];

		Asura_Assert(pxParticlePtr->Position.x < 6000.0f && pxParticlePtr->Position.x > -6000.0f,"position problem");
		Asura_Assert(pxParticlePtr->Position.y < 6000.0f && pxParticlePtr->Position.y > -6000.0f,"position problem");
		Asura_Assert(pxParticlePtr->Position.z < 6000.0f && pxParticlePtr->Position.z > -6000.0f,"position problem");
		Asura_Assert(pxParticlePtr->Size < 50.0f && pxParticlePtr->Size > -50.0f,"size problems?");

		const Asura_Vector_3 xDeltaX = xDX * pxParticlePtr->Size;
		const Asura_Vector_3 xDeltaY = xDY * pxParticlePtr->Size;
		/// Kevin Lea - 27/03/2002 11:43:15 - code to draw particle stretched along its path
		/// ucky and needs rewriting
		if (pxParticleTemplate->UseMotionBlur)
		{
			Asura_Vector_3 xBeta;

			xBeta = pxParticlePtr->Position - pxParticlePtr->PrevPosition;
			if( xBeta.MagnitudeSquared() > (0.1f*0.1f) )
			{
				xBeta.Normalise();
				xBeta *= 0.1f;
			}
				
			Asura_Assert(pxParticlePtr->PrevPosition.x < 6000.0f && pxParticlePtr->PrevPosition.x > -6000.0f,"prev position problem");
			Asura_Assert(pxParticlePtr->PrevPosition.y < 6000.0f && pxParticlePtr->PrevPosition.y > -6000.0f,"prev position problem");
			Asura_Assert(pxParticlePtr->PrevPosition.z < 6000.0f && pxParticlePtr->PrevPosition.z > -6000.0f,"prev position problem");

			if (Asura_Maths::Abs(xBeta * xDeltaX) > Asura_Maths::Abs(xBeta * xDeltaY))
			{
				if (xBeta * xDeltaX > 0)
				{
					axVertex[0] = pxParticlePtr->PrevPosition - xBeta - xDeltaX - xDeltaY; 
					axVertex[1] = pxParticlePtr->Position + xBeta + xDeltaX - xDeltaY;
					axVertex[2] = pxParticlePtr->Position + xBeta + xDeltaX + xDeltaY;
					axVertex[3] = pxParticlePtr->PrevPosition - xBeta - xDeltaX + xDeltaY;
				}
				else
				{
					axVertex[0] = pxParticlePtr->Position + xBeta - xDeltaX - xDeltaY; 
					axVertex[1] = pxParticlePtr->PrevPosition- xBeta + xDeltaX - xDeltaY;
					axVertex[2] = pxParticlePtr->PrevPosition- xBeta + xDeltaX + xDeltaY;
					axVertex[3] = pxParticlePtr->Position + xBeta - xDeltaX + xDeltaY;
				}
			}
			else
			{
				if (xBeta * xDeltaY > 0)
				{
					axVertex[0] = pxParticlePtr->PrevPosition - xBeta - xDeltaX - xDeltaY; 
					axVertex[1] = pxParticlePtr->PrevPosition - xBeta + xDeltaX - xDeltaY;
					axVertex[2] = pxParticlePtr->Position + xBeta + xDeltaX + xDeltaY;
					axVertex[3] = pxParticlePtr->Position + xBeta - xDeltaX + xDeltaY;
				}
				else
				{
					axVertex[0] = pxParticlePtr->Position + xBeta - xDeltaX - xDeltaY; 
					axVertex[1] = pxParticlePtr->Position + xBeta + xDeltaX - xDeltaY;
					axVertex[2] = pxParticlePtr->PrevPosition - xBeta + xDeltaX + xDeltaY;
					axVertex[3] = pxParticlePtr->PrevPosition - xBeta - xDeltaX + xDeltaY;
				}
			}
		}
		else 
		{
			float fTheta;

			if (pxParticleTemplate->UseRandomRotation)
			{
				fTheta = ((Asura_Timers::GetGameTimeScale() == 0.0f) ? 0.0f : Asura_Random::GetFloat()) * Asura_Maths::Pi * 2.0f;	
			}
			else
			{
				fTheta = pxParticlePtr->fRotation;
			}
			
			if (fTheta != 0.0f)
			{
				float fSin = Asura_Maths::Sine(fTheta) * pxParticlePtr->Size;
				float fCos = Asura_Maths::Cosine(fTheta) * pxParticlePtr->Size;;
			
				Asura_Vector_3 xCosDeltaX = Asura_Vector_3(pxCameraOrientation->e11, pxCameraOrientation->e21, pxCameraOrientation->e31) * fCos;
				Asura_Vector_3 xSinDeltaX = Asura_Vector_3(pxCameraOrientation->e11, pxCameraOrientation->e21, pxCameraOrientation->e31) * fSin;
				Asura_Vector_3 xCosDeltaY = Asura_Vector_3(pxCameraOrientation->e12, pxCameraOrientation->e22, pxCameraOrientation->e32) * fCos;
				Asura_Vector_3 xSinDeltaY = Asura_Vector_3(pxCameraOrientation->e12, pxCameraOrientation->e22, pxCameraOrientation->e32) * fSin;
				
				axVertex[0] = pxParticlePtr->Position + (-xCosDeltaX + xSinDeltaX) + (-xSinDeltaY - xCosDeltaY);		
				axVertex[1] = pxParticlePtr->Position + ( xCosDeltaX + xSinDeltaX) + ( xSinDeltaY - xCosDeltaY);
				axVertex[2] = pxParticlePtr->Position + ( xCosDeltaX - xSinDeltaX) + ( xSinDeltaY + xCosDeltaY);
				axVertex[3] = pxParticlePtr->Position + (-xCosDeltaX - xSinDeltaX) + (-xSinDeltaY + xCosDeltaY);
			}
			else
			{
				axVertex[0] = pxParticlePtr->Position - xDeltaX - xDeltaY; 
				axVertex[1] = pxParticlePtr->Position + xDeltaX - xDeltaY;
				axVertex[2] = pxParticlePtr->Position + xDeltaX + xDeltaY;
				axVertex[3] = pxParticlePtr->Position - xDeltaX + xDeltaY;
			}
		}


		g_axVertices[g_uOffset + 0].m_fU = pxParticleTemplate->TopLeftU;
		g_axVertices[g_uOffset + 0].m_fV = pxParticleTemplate->TopLeftV;

		g_axVertices[g_uOffset + 1].m_fU = pxParticleTemplate->BottomRightU;
		g_axVertices[g_uOffset + 1].m_fV = pxParticleTemplate->TopLeftV;

		g_axVertices[g_uOffset + 2].m_fU = pxParticleTemplate->BottomRightU;
		g_axVertices[g_uOffset + 2].m_fV = pxParticleTemplate->BottomRightV;

		g_axVertices[g_uOffset + 3].m_fU = pxParticleTemplate->BottomRightU;
		g_axVertices[g_uOffset + 3].m_fV = pxParticleTemplate->TopLeftV;

		g_axVertices[g_uOffset + 4].m_fU = pxParticleTemplate->BottomRightU;
		g_axVertices[g_uOffset + 4].m_fV = pxParticleTemplate->BottomRightV;

		g_axVertices[g_uOffset + 5].m_fU = pxParticleTemplate->TopLeftU;
		g_axVertices[g_uOffset + 5].m_fV = pxParticleTemplate->BottomRightV;

		// Colour is in ABGR format
		unsigned int uColour;
		uColour = Asura_Maths::FloatToInt(pxParticlePtr->Colour.Alpha * 255.0f);
		uColour <<=8;
		uColour += Asura_Maths::FloatToInt(pxParticlePtr->Colour.Blue * 128.0f);
		uColour <<=8;
		uColour += Asura_Maths::FloatToInt(pxParticlePtr->Colour.Green * 128.0f);
		uColour <<=8;
		uColour += Asura_Maths::FloatToInt(pxParticlePtr->Colour.Red * 128.0f);

		for (int i = 0; i < 6; ++i)
		{
			g_axVertices[g_uOffset + i].m_afPosition[0] = axVertex[i].x;
			g_axVertices[g_uOffset + i].m_afPosition[1] = axVertex[i].y;
			g_axVertices[g_uOffset + i].m_afPosition[2] = axVertex[i].z;

			g_axVertices[g_uOffset + i].m_uColour = uColour;
		}
		g_uOffset += 6;
	}
	//sceKernelDcacheWritebackAll();
	sceKernelDcacheWritebackRange((const char*)(&g_axVertices[uInitialOffset]), 6 * iParticle * sizeof(Asura_PSP_RenderParticle_Vertex));
	
	sceGumDrawArray(SCEGU_PRIM_TRIANGLES, 
					iASURA_PSP_PARTICLE_VERTEX_FORMAT,
					6 * iParticle, 
					NULL, 
					g_axVertices + uInitialOffset);

#if defined ASURA_RENDER_TTYPES
	Asura_Render::AddToTrianglesRendered(2 * iNumParticles,1,
		ASURA_RENDER_TTYPE_PFX);
#else
	Asura_Render::AddToTrianglesRendered(2 * iNumParticles);
#endif
}
