//
// Asura_PSP_RenderShards.cpp
//
// Created by KJFL - Wednesday, March 31, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Camera/Asura_Camera.h"
#include "Core/Asura_TextureManagement.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Shard_System.h"

#include "Rendering/Asura_PSP_Graphics.h"
#include "Rendering/Asura_PSP_TextureManagement.h"
#include <kernel.h>

struct Asura_PSP_RenderShard_Vertex
{
	// fully 32bit - 24 bytes
	float			m_fU;
	float			m_fV;
	unsigned		m_uColour;
	float			m_afPosition[3];
};


const int iASURA_PSP_MAX_NUMBER_SHARD_VERTICES = 768; // Max of 256 shards. (Uses 24kB!)
const int iASURA_PSP_SHARD_VERTEX_FORMAT = SCEGU_TEXTURE_FLOAT | SCEGU_VERTEX_FLOAT | SCEGU_COLOR_PF8888;
static Asura_PSP_RenderShard_Vertex s_apxShardVertices[iASURA_PSP_MAX_NUMBER_SHARD_VERTICES] __attribute__ ((aligned(16)));

static u_int s_uShardVerticesUsedThisBatch = 0;
static u_int s_uShardVerticesOffsetForThisBatch = 0;
static int s_iCurrentTextureIndex = ASURA_TEXTURE_NONE;

static void FlushShards()
{
	if (s_uShardVerticesUsedThisBatch)
	{
		sceKernelDcacheWritebackAll();
		
		GUM_DrawArray
		(
			SCEGU_PRIM_TRIANGLES, 
			iASURA_PSP_SHARD_VERTEX_FORMAT,
			s_uShardVerticesUsedThisBatch, 
			NULL, 
			&s_apxShardVertices[s_uShardVerticesOffsetForThisBatch]
		);
		s_uShardVerticesOffsetForThisBatch += s_uShardVerticesUsedThisBatch;
		s_uShardVerticesUsedThisBatch = 0;
	}
}

void Asura_Shard_System::Platform_BeginRender(void)
{
	// Setup camera settings.
	Asura_Assert(Asura_Camera::GetNumberOfCameras() == 1, "Glass renderer assumes 1 camera");
	Asura_Camera::SetCameraIndex(0);
	Asura_Render::ApplyCameraSettings();

	// Setup render states.
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_PSP_Graphics::SetMaterialType( ASURA_PSP_GRAPHICS_MATERIALTYPE_ENV );
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_PSP_Graphics::SetUseColourDoubling(false);
	Asura_PSP_Graphics::SetUseLighting(false);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);

	s_uShardVerticesUsedThisBatch = 0;
	s_uShardVerticesOffsetForThisBatch = 0;
	s_iCurrentTextureIndex = ASURA_TEXTURE_NONE;
}

void Asura_Shard_System::Platform_EndRender(void)
{
	FlushShards();
}

// (AW - 07/10/05) - Shard renderer. Rather inefficient, but hopefully we won't be drawing very many.
void RenderShard(Asura_Shard *pxShardPtr, int iTextureIndex, u_int uColour)
{
	if (s_uShardVerticesUsedThisBatch + s_uShardVerticesOffsetForThisBatch + 3 > iASURA_PSP_MAX_NUMBER_SHARD_VERTICES)
	{
		Asura_Assert(false, "Too many shards to render in one frame!");
		return;
	}
	if (!s_uShardVerticesUsedThisBatch)
	{
		s_iCurrentTextureIndex = iTextureIndex;
		Asura_PSP_TextureManagement::RequestTextureIndex(iTextureIndex);
	}
	else if (iTextureIndex != s_iCurrentTextureIndex)
	{
		FlushShards();
		s_iCurrentTextureIndex = iTextureIndex;
		Asura_PSP_TextureManagement::RequestTextureIndex(iTextureIndex);
	}
	Asura_PSP_RenderShard_Vertex* pxShardVertices = &s_apxShardVertices[s_uShardVerticesUsedThisBatch + s_uShardVerticesOffsetForThisBatch];
	
	u_int uColourABGR = ((uColour & 0x00ff0000) >> 16) | ((uColour & 0x000000ff) << 16) | (uColour & 0xff00ff00);


	for (int i=0;i<3;i++)
	{
		pxShardVertices[i].m_fU  = pxShardPtr->m_afU[i];
		pxShardVertices[i].m_fV  = pxShardPtr->m_afV[i];
		pxShardVertices[i].m_afPosition[0]  = pxShardPtr->m_axVertices[i].x;
		pxShardVertices[i].m_afPosition[1]  = pxShardPtr->m_axVertices[i].y;
		pxShardVertices[i].m_afPosition[2]  = pxShardPtr->m_axVertices[i].z;
		pxShardVertices[i].m_uColour = uColourABGR;
	}

	// Update counters
	s_uShardVerticesUsedThisBatch += 3;
#if defined ASURA_RENDER_TTYPES
	Asura_Render::AddToTrianglesRendered(1,1,
		ASURA_RENDER_TTYPE_OTHER);
#else
	Asura_Render::AddToTrianglesRendered(1);
#endif
}
