// Tools_PSP_VertexFormat.cpp
//
// Created by SB - 18/01/2007
//
// Copyright (C) 2007 Rebellion (Derby), All Rights Reserved.

// Asura includes
#include "Core/Asura.h"
#include "Core/Asura_Maths.h"

// This file's header
// (SB - 15/02/07) Having to use a relative path for this to compile in both PSP and Tools builds
#include "../PSP/Rendering/Asura_PSP_VertexFormat.h"

// Return the size of a vertex with a given format
unsigned int Asura_PSP_VertexFormat::CalculateVertexSize(const unsigned short usVertexFormat)
{
	return
		GetSizeOfTexCoord(usVertexFormat) * 2 
		+ GetSizeOfColour(usVertexFormat)
		+ GetSizeOfNormalComponent(usVertexFormat) * 3
		+ GetSizeOfVertexComponent(usVertexFormat) * 3;
}

// Return the size of a texcoord component in a given format
unsigned int Asura_PSP_VertexFormat::GetSizeOfTexCoord(const unsigned short usVertexFormat)
{
	// Texture coordinate size
	const unsigned short usTexCoordFormat = usVertexFormat & ASURA_SCEGU_TEXTURE_MASK;
	if (usTexCoordFormat == ASURA_SCEGU_TEXTURE_UBYTE) // Unsigned 8 bit
	{
		return 1;
	}
	else if (usTexCoordFormat == ASURA_SCEGU_TEXTURE_USHORT) // Unsigned 16 bit
	{
		return 2;
	}
	else if (usTexCoordFormat == ASURA_SCEGU_TEXTURE_FLOAT) // Float
	{
		return 4;
	}
	return 0;
}

// Return the size of the colour channel of a given format
unsigned int Asura_PSP_VertexFormat::GetSizeOfColour(const unsigned short usVertexFormat)
{
	// Colour value
	const unsigned short usColourFormat = usVertexFormat & ASURA_SCEGU_COLOR_MASK;
	if (usColourFormat == ASURA_SCEGU_COLOR_PF5650) // 16 bit
	{
		return 2;
	}
	else if (usColourFormat == ASURA_SCEGU_COLOR_PF5551) // 16 bit
	{
		return 2;
	}
	else if (usColourFormat == ASURA_SCEGU_COLOR_PF4444) // 16 bit
	{
		return 2;
	}
	else if (usColourFormat == ASURA_SCEGU_COLOR_PF8888) // 32 bit
	{
		return 4;
	}
	return 0;
}

// Return the size of a normal component for a given format
unsigned int Asura_PSP_VertexFormat::GetSizeOfNormalComponent(const unsigned short usVertexFormat)
{
	// Normal vector
	const unsigned short usNormalFormat = usVertexFormat & ASURA_SCEGU_NORMAL_MASK;
	if (usNormalFormat == ASURA_SCEGU_NORMAL_BYTE) // Signed 8 bit
	{
		return 1;
	}
	else if (usNormalFormat == ASURA_SCEGU_NORMAL_SHORT) // Signed 16 bit
	{
		return 2;
	}
	else if (usNormalFormat == ASURA_SCEGU_NORMAL_FLOAT) // Floating point
	{
		return 4;
	}
	return 0;
}

// Return the size of a vertex component for a given format
unsigned int Asura_PSP_VertexFormat::GetSizeOfVertexComponent(const unsigned short usVertexFormat)
{
	// Vertex vector
	const unsigned short usVtxFormat = usVertexFormat & ASURA_SCEGU_VERTEX_MASK;
	if (usVtxFormat == ASURA_SCEGU_VERTEX_BYTE) // Signed 8 bit
	{
		return 1;
	}
	else if (usVtxFormat == ASURA_SCEGU_VERTEX_SHORT) // Signed 16 bit
	{
		return 2;
	}
	else if (usVtxFormat == ASURA_SCEGU_VERTEX_FLOAT) // Float
	{
		return 4;
	}
	return 0;
}

// Return a given component of the vertex from the specified format
Asura_PSP_VertexFormat::ShortVector2& Asura_PSP_VertexFormat::GetTexCoordsFromVertex(const unsigned char* pucVertex, const unsigned short usVertexFormat)
{
	static ShortVector2 xOut;
	switch (usVertexFormat)
	{
		// Default format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF5650|ASURA_SCEGU_NORMAL_SHORT|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_Default_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_Default_Vertex*>(pucVertex);
			xOut.x = pxVertex->m_usU;
			xOut.y = pxVertex->m_usV;
			break;
		}
		// Second pass format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF4444|ASURA_SCEGU_NORMAL_SHORT|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_SecondPass_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_SecondPass_Vertex*>(pucVertex);
			xOut.x = pxVertex->m_usU;
			xOut.y = pxVertex->m_usV;
			break;
		}
		// 32bit colour, no normals format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF8888|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_RGBA8888NoNormal_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_RGBA8888NoNormal_Vertex*>(pucVertex);
			xOut.x = pxVertex->m_usU;
			xOut.y = pxVertex->m_usV;
			break;
		}
		default:
		{
			Asura_Assert(false, "Unsupported vertex format!");
			xOut.x = 0;
			xOut.y = 0;
			break;
		}
	}
	return xOut;
}

int Asura_PSP_VertexFormat::GetColourFromVertex(const unsigned char* pucVertex, const unsigned short usVertexFormat)
{
	switch (usVertexFormat)
	{
		// Default format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF5650|ASURA_SCEGU_NORMAL_SHORT|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_Default_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_Default_Vertex*>(pucVertex);
			const unsigned char r = (unsigned char)(pxVertex->m_usRGB565 & 0x0000001f) << 3;
			const unsigned char g = (unsigned char)((pxVertex->m_usRGB565 & 0x000007e0) >> (11 - 6 - 2));
			const unsigned char b = (unsigned char)((pxVertex->m_usRGB565 & 0x0000f800) >> (16 - 5 - 3));
			return (0xff000000 | (b << 16) | (g << 8) | (r));
		}
		// Second pass format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF4444|ASURA_SCEGU_NORMAL_SHORT|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_SecondPass_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_SecondPass_Vertex*>(pucVertex);
			const unsigned char r = (unsigned char)((pxVertex->m_usRGBA4444 & 0x0000000f) * 17); // 17 = 255 / 15
			const unsigned char g = (unsigned char)(((pxVertex->m_usRGBA4444 & 0x000000f0) >> 4) * 17); // 17 = 255 / 15
			const unsigned char b = (unsigned char)(((pxVertex->m_usRGBA4444 & 0x00000f00) >> 8) * 17); // 17 = 255 / 15
			const unsigned char a = (unsigned char)(((pxVertex->m_usRGBA4444 & 0x0000f000) >> 12) * 17); // 17 = 255 / 15
			return ((a << 24) | (b << 16) | (g << 8) | (r));
		}
		// 32bit colour, no normals format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF8888|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_RGBA8888NoNormal_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_RGBA8888NoNormal_Vertex*>(pucVertex);
			return pxVertex->m_uRGBA8888;
		}
		default:
		{
			Asura_Assert(false, "Unsupported vertex format!");
			break;
		}
	}
	return 0xffffffff;
}

Asura_PSP_VertexFormat::ShortVector3& Asura_PSP_VertexFormat::GetNormalFromVertex(const unsigned char* pucVertex, const unsigned short usVertexFormat)
{
	static ShortVector3 xOut;
	switch (usVertexFormat)
	{
		// Default format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF5650|ASURA_SCEGU_NORMAL_SHORT|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_Default_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_Default_Vertex*>(pucVertex);
			xOut.x = pxVertex->m_asNormal[0];
			xOut.y = pxVertex->m_asNormal[1];
			xOut.z = pxVertex->m_asNormal[2];
			break;
		}
		// Second pass format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF4444|ASURA_SCEGU_NORMAL_SHORT|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_SecondPass_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_SecondPass_Vertex*>(pucVertex);
			xOut.x = pxVertex->m_asNormal[0];
			xOut.y = pxVertex->m_asNormal[1];
			xOut.z = pxVertex->m_asNormal[2];
			break;
		}
		// 32bit colour, no normals format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF8888|ASURA_SCEGU_VERTEX_FLOAT:
		{
			xOut.x = 0;
			xOut.y = 0;
			xOut.z = 0;
			break;
		}
		default:
		{
			Asura_Assert(false, "Unsupported vertex format!");
			xOut.x = 0;
			xOut.y = 0;
			xOut.z = 0;
			break;
		}
	}
	return xOut;
}

Asura_Vector_3& Asura_PSP_VertexFormat::GetPositionFromVertex(const unsigned char* pucVertex, const unsigned short usVertexFormat)
{
	static Asura_Vector_3 xOut;
	switch (usVertexFormat)
	{
		// Default format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF5650|ASURA_SCEGU_NORMAL_SHORT|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_Default_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_Default_Vertex*>(pucVertex);
			xOut.x = pxVertex->m_afPosition[0];
			xOut.y = pxVertex->m_afPosition[1];
			xOut.z = pxVertex->m_afPosition[2];
			break;
		}
		// Second pass format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF4444|ASURA_SCEGU_NORMAL_SHORT|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_SecondPass_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_SecondPass_Vertex*>(pucVertex);
			xOut.x = pxVertex->m_afPosition[0];
			xOut.y = pxVertex->m_afPosition[1];
			xOut.z = pxVertex->m_afPosition[2];
			break;
		}
		// 32bit colour, no normals format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF8888|ASURA_SCEGU_VERTEX_FLOAT:
		{
			const Asura_PSP_RGBA8888NoNormal_Vertex* pxVertex = reinterpret_cast<const Asura_PSP_RGBA8888NoNormal_Vertex*>(pucVertex);
			xOut.x = pxVertex->m_afPosition[0];
			xOut.y = pxVertex->m_afPosition[1];
			xOut.z = pxVertex->m_afPosition[2];
			break;
		}
		default:
		{
			Asura_Assert(false, "Unsupported vertex format!");
			xOut.x = 0;
			xOut.y = 0;
			xOut.z = 0;
			break;
		}
	}
	return xOut;
}

Asura_Vector_3& Asura_PSP_VertexFormat::GetPositionFromVertex(const unsigned char* pucVertex, const unsigned short usVertexFormat, const size_t xVertexPosOffset)
{
	static Asura_Vector_3 xOut;
	const unsigned char* pucVertexPos = pucVertex + xVertexPosOffset;
	Asura_Assert((usVertexFormat & ASURA_SCEGU_VERTEX_MASK) == ASURA_SCEGU_VERTEX_FLOAT, "Currently only floating point vertex positions are supported");
	const float *pfVertex = reinterpret_cast<const float*>(pucVertexPos);
	xOut.x = pfVertex[0];
	xOut.y = pfVertex[1];
	xOut.z = pfVertex[2];
	return xOut;
}

void Asura_PSP_VertexFormat::SetColourInVertex(unsigned char* pucVertex, const unsigned int uColour, const unsigned short usVertexFormat)
{	
	switch (usVertexFormat)
	{
		// Default format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF5650|ASURA_SCEGU_NORMAL_SHORT|ASURA_SCEGU_VERTEX_FLOAT:
		{
			Asura_PSP_Default_Vertex* pxVertex = reinterpret_cast<Asura_PSP_Default_Vertex*>(pucVertex);
			Asura_PSP_VertexFormat::ConvertVertexColourToSpecifiedFormat(reinterpret_cast<unsigned char*>(&pxVertex->m_usRGB565), uColour, usVertexFormat);
			//pxVertex->m_usRGB565 = Asura_PSP_VertexFormat::ConvertVertexColourToSpecifiedFormat(uColour, usVertexFormat).usData;
			break;
		}
		// Second pass format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF4444|ASURA_SCEGU_NORMAL_SHORT|ASURA_SCEGU_VERTEX_FLOAT:
		{
			Asura_PSP_SecondPass_Vertex* pxVertex = reinterpret_cast<Asura_PSP_SecondPass_Vertex*>(pucVertex);
			Asura_PSP_VertexFormat::ConvertVertexColourToSpecifiedFormat(reinterpret_cast<unsigned char*>(&pxVertex->m_usRGBA4444), uColour, usVertexFormat);
			//pxVertex->m_usRGBA4444 = Asura_PSP_VertexFormat::ConvertVertexColourToSpecifiedFormat(uColour, usVertexFormat).usData;
			break;
		}
		// 32bit colour, no normals format
		case ASURA_SCEGU_TEXTURE_USHORT|ASURA_SCEGU_COLOR_PF8888|ASURA_SCEGU_VERTEX_FLOAT:
		{
			Asura_PSP_RGBA8888NoNormal_Vertex* pxVertex = reinterpret_cast<Asura_PSP_RGBA8888NoNormal_Vertex*>(pucVertex);
			Asura_PSP_VertexFormat::ConvertVertexColourToSpecifiedFormat(reinterpret_cast<unsigned char*>(&pxVertex->m_uRGBA8888), uColour, usVertexFormat);
			//pxVertex->m_uRGBA8888 = Asura_PSP_VertexFormat::ConvertVertexColourToSpecifiedFormat(uColour, usVertexFormat).uData;
			break;
		}
		default:
		{
			Asura_Assert(false, "Unsupported vertex format!");
			break;
		}
	}
}

void Asura_PSP_VertexFormat::ConvertTexcoordToSpecifiedFormat(unsigned char* pucOut, float fCoord, const unsigned short usVertexFormat)
{
	// Texture coordinate size
	const unsigned short usTexCoordFormat = usVertexFormat & ASURA_SCEGU_TEXTURE_MASK;
	(*reinterpret_cast<unsigned int*>(pucOut)) = 0; // Clear output buffer
	if (usTexCoordFormat == ASURA_SCEGU_TEXTURE_UBYTE) // Unsigned 8 bit
	{
		// This won't allow for textures to wrap
		pucOut[0] = static_cast<unsigned char>(Asura_Maths::FloatToInt(fCoord * 256.0f));
	}
	else if (usTexCoordFormat == ASURA_SCEGU_TEXTURE_USHORT) // Unsigned 16 bit
	{
		(*reinterpret_cast<unsigned short*>(pucOut)) = static_cast<unsigned short>(Asura_Maths::FloatToInt(fCoord * (32768.0f / 128.0f)));
	}
	else if (usTexCoordFormat == ASURA_SCEGU_TEXTURE_FLOAT) // Float
	{
		(*reinterpret_cast<float*>(pucOut)) = fCoord;
	}
}

// This also swaps order of colour channels to convert from tools to PSP
void Asura_PSP_VertexFormat::ConvertVertexColourToSpecifiedFormat(unsigned char* pucOut, unsigned int uInput, const unsigned short usVertexFormat)
{
	// Colour value
	const unsigned short usColourFormat = usVertexFormat & ASURA_SCEGU_COLOR_MASK;
	(*reinterpret_cast<unsigned int*>(pucOut)) = 0; // Clear output buffer
	if (usColourFormat == ASURA_SCEGU_COLOR_PF5650) // 16 bit
	{
		const unsigned char r = (unsigned char)((uInput & 0x000000ff) >> (8 - 5));
		const unsigned char g = (unsigned char)((uInput & 0x0000ff00) >> (16 - 6));
		const unsigned char b = (unsigned char)((uInput & 0x00ff0000) >> (24 - 5));
#ifdef ASURA_TOOLS
		(*reinterpret_cast<unsigned short*>(pucOut)) = ((r << 11) | (g << 5) | (b));
#else
		(*reinterpret_cast<unsigned short*>(pucOut)) = ((b << 11) | (g << 5) | (r));
#endif
	}
	else if (usColourFormat == ASURA_SCEGU_COLOR_PF5551) // 16 bit
	{
		const unsigned char r = (unsigned char)((uInput & 0x000000ff) >> (8 - 5));
		const unsigned char g = (unsigned char)((uInput & 0x0000ff00) >> (16 - 5));
		const unsigned char b = (unsigned char)((uInput & 0x00ff0000) >> (24 - 5));
		const unsigned char a = (unsigned char)((uInput & 0xff000000) >> (32 - 1));
#ifdef ASURA_TOOLS
		(*reinterpret_cast<unsigned short*>(pucOut)) = ((a << 15) | (r << 11) | (g << 6) | (b));
#else
		(*reinterpret_cast<unsigned short*>(pucOut)) = ((a << 15) | (b << 11) | (g << 6) | (r));
#endif
	}
	else if (usColourFormat == ASURA_SCEGU_COLOR_PF4444) // 16 bit
	{
		const unsigned char r = (unsigned char)((uInput & 0x000000ff) >> (8 - 4));
		const unsigned char g = (unsigned char)((uInput & 0x0000ff00) >> (16 - 4));
		const unsigned char b = (unsigned char)((uInput & 0x00ff0000) >> (24 - 4));
		const unsigned char a = (unsigned char)((uInput & 0xff000000) >> (32 - 4));
#ifdef ASURA_TOOLS
		(*reinterpret_cast<unsigned short*>(pucOut)) = ((a << 12) | (r << 8) | (g << 4) | (b));
#else
		(*reinterpret_cast<unsigned short*>(pucOut)) = ((a << 12) | (b << 8) | (g << 4) | (r));
#endif
	}
	else if (usColourFormat == ASURA_SCEGU_COLOR_PF8888) // 32 bit
	{
		const unsigned char r = (unsigned char)(uInput & 0x000000ff);
		const unsigned char g = (unsigned char)((uInput & 0x0000ff00) >> 8);
		const unsigned char b = (unsigned char)((uInput & 0x00ff0000) >> 16);
		const unsigned char a = (unsigned char)((uInput & 0xff000000) >> 24);
#ifdef ASURA_TOOLS
		(*reinterpret_cast<unsigned int*>(pucOut)) = ((a << 24) | (r << 16) | (g << 8) | (b));
#else
		(*reinterpret_cast<unsigned int*>(pucOut)) = ((a << 24) | (b << 16) | (g << 8) | (r));
#endif
	}
}

void Asura_PSP_VertexFormat::ConvertNormalComponentToSpecifiedFormat(unsigned char* pucOut, float fComponent, const unsigned short usVertexFormat)
{
	// Normal vector
	const unsigned short usNormalFormat = usVertexFormat & ASURA_SCEGU_NORMAL_MASK;
	(*reinterpret_cast<unsigned int*>(pucOut)) = 0; // Clear output buffer
	if (usNormalFormat == ASURA_SCEGU_NORMAL_BYTE) // Signed 8 bit
	{
		(*reinterpret_cast<char*>(pucOut)) = static_cast<char>(Asura_Maths::FloatToInt(fComponent * 128.0f));
	}
	else if (usNormalFormat == ASURA_SCEGU_NORMAL_SHORT) // Signed 16 bit
	{
		(*reinterpret_cast<short*>(pucOut)) = static_cast<short>(Asura_Maths::FloatToInt(fComponent * 16384.0f));
	}
	else if (usNormalFormat == ASURA_SCEGU_NORMAL_FLOAT) // Floating point
	{
		(*reinterpret_cast<float*>(pucOut)) = fComponent;
	}
}

void Asura_PSP_VertexFormat::ConvertVertexComponentToSpecifiedFormat(unsigned char* pucOut, float fComponent, const unsigned short usVertexFormat)
{
	// Vertex vector
	const unsigned short usVtxFormat = usVertexFormat & ASURA_SCEGU_VERTEX_MASK;
	(*reinterpret_cast<unsigned int*>(pucOut)) = 0; // Clear output buffer
	if (usVtxFormat == ASURA_SCEGU_VERTEX_BYTE) // Signed 8 bit
	{
		(*reinterpret_cast<char*>(pucOut)) = static_cast<char>(Asura_Maths::FloatToInt(fComponent * 128.0f));
	}
	else if (usVtxFormat == ASURA_SCEGU_VERTEX_SHORT) // Signed 16 bit
	{
		(*reinterpret_cast<short*>(pucOut)) = static_cast<short>(Asura_Maths::FloatToInt(fComponent * 16384.0f));
	}
	else if (usVtxFormat == ASURA_SCEGU_VERTEX_FLOAT) // Float
	{
		(*reinterpret_cast<float*>(pucOut)) = fComponent;
	}
}

/*
bool Asura_PSP_VertexFormat::PositionsAreEqual(const unsigned char* pucFirst, const unsigned char* pucSecond, const unsigned short usVertexFormat)
{
	const unsigned char* pucFirstInput = ((unsigned char*)(pucFirst + GetSizeOfTexCoord(usVertexFormat) * 2 + GetSizeOfColour(usVertexFormat) + GetSizeOfNormalComponent(usVertexFormat) * 3));
	const unsigned char* pucSecondInput = ((unsigned char*)(pucSecond + GetSizeOfTexCoord(usVertexFormat) * 2 + GetSizeOfColour(usVertexFormat) + GetSizeOfNormalComponent(usVertexFormat) * 3));
	
	for (u_int i = 0; i < GetSizeOfVertexComponent(usVertexFormat) * 3; i++)
	{
		if (pucFirstInput[i] != pucSecondInput[i])
		{
			return false;
		}
	}

	return true;
}
*/

bool Asura_PSP_VertexFormat::PositionsAreEqual(const unsigned char* pucFirst, const unsigned char* pucSecond, const size_t xVertexPosOffset, const size_t xVertexPosComponentSize)
{
	const unsigned char* pucFirstInput = pucFirst + xVertexPosOffset;
	const unsigned char* pucSecondInput = pucSecond + xVertexPosOffset;
	
	for (u_int i = 0; i < xVertexPosComponentSize * 3; i++)
	{
		if (pucFirstInput[i] != pucSecondInput[i])
		{
			return false;
		}
	}
	return true;
}
