//
// Asura_Platform_Render.cpp
//
// Created by KJFL - Wednesday, February 04, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "ProjectSettings/Asura_PSPConfiguration.h"

// This file's header.
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Font_System.h"

// Asura headers
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Timers.h"
#include "Entity/Asura_Entity_System.h"
#include "Network/Asura_Network.h"
#include "Physics/Asura_Physics.h"
#include "Sound/Asura_Sound.h"

// PSP headers
#include "Debugging/Asura_PSP_GPIO.h"
#include "Rendering/Asura_PSP_VRAMManagement.h"
#include "Rendering/Asura_PSP_Graphics.h"
#include "Rendering/Asura_PSP_CharacterRenderer.h"
#include "Rendering/Asura_PSP_ObjectHierarchyRenderer.h"
#include "Rendering/Asura_PSP_ObjectRenderer.h"
#include "Rendering/Asura_PSP_RenderEnvironment.h"
#include "Rendering/Asura_PSP_TextureManagement.h"

#include <powersvc.h>

// KOR - 07/02/2006 - definitions/declarations for line rendering
#ifdef ASURA_PSP_ENABLE_LINE_RENDERING
// Vertex definition
static const u_int uASURA_LINE_VERTEX_DEFINITION = SCEGU_COLOR_PF8888 | SCEGU_VERTEX_FLOAT;
static const u_int uASURA_2DLINE_VERTEX_DEFINITION = SCEGU_COLOR_PF8888 | SCEGU_VERTEX_SHORT | SCEGU_THROUGH;

struct Asura_PSP_3D_Line_Vertex
{
	u_int			m_uDiffuseRGBA;
	float			m_afPosition[3];
};

struct Asura_PSP_2D_Line_Vertex
{
	u_int			m_uDiffuseRGBA;
	u_short			m_ausPosition[3];
};

// Number of lines we can render in a frame
#define ASURA_PSP_MAX_LINES (512)
#define ASURA_PSP_MAX_LINE_VERTICES (ASURA_PSP_MAX_LINES * 2)

// Buffer for vertices
static Asura_PSP_3D_Line_Vertex g_axLineVertices[ASURA_PSP_MAX_LINE_VERTICES];
static u_int g_uCurrentLineVertex = 0;

static Asura_PSP_2D_Line_Vertex g_ax2DLineVertices[ASURA_PSP_MAX_LINE_VERTICES];
static u_int g_uCurrent2DLineVertex = 0;
#endif//def ASURA_PSP_ENABLE_LINE_RENDERING

float Asura_Font_System::GetPlatformYScaleFactor(void)
{
	return 1.0f;
}

void Asura_Render::Platform_Initialise(void)
{
	Asura_PSP_VRAMManagement::Initialise(); 

	Asura_PSP_Graphics::Initialise();

	Asura_PSP_RenderEnvironment::Initialise();
	Asura_PSP_CharacterRenderer::Initialise();
	Asura_PSP_ObjectHierarchyRenderer::Initialise();
	Asura_PSP_ObjectRenderer::Initialise();
}

void Asura_Render::Platform_EarlyShutdown(void) {}

void Asura_Render::Platform_LateShutdown(void)
{
	Asura_PSP_CharacterRenderer::Shutdown();
	Asura_PSP_ObjectHierarchyRenderer::Shutdown();
	Asura_PSP_ObjectRenderer::Shutdown();
	Asura_PSP_RenderEnvironment::Shutdown();
	Asura_PSP_Graphics::Shutdown();
}

void Asura_Render::Platform_Reset(void)
{
	Asura_PSP_CharacterRenderer::Reset();
	Asura_PSP_ObjectHierarchyRenderer::Reset();
	Asura_PSP_ObjectRenderer::Reset();
	Asura_PSP_RenderEnvironment::Reset();
}


void Asura_Render::Platform_ApplyCameraSettings(void)
{
	Asura_Assert(IsRendering(), "Calling graphics function outside Begin/End frame");
	/*
	// KOR - TODO - is this needed on PSP? Not doing any multiple camera PSP games yet.
	// NRM 19/11/02 - temporary fix to batching of overlays
	if(Asura_Camera::GetNumberOfCameras() > 1)
	{
		Asura_Render::FlushDisplay();
	}
	*/
	const Asura_ViewingArea& xView = *Asura_Camera::GetViewingArea();

	GU_Scissor(xView.m_fXOffset * SCEGU_SCR_WIDTH, xView.m_fYOffset * SCEGU_SCR_HEIGHT, xView.m_fWidth * SCEGU_SCR_WIDTH, xView.m_fHeight * SCEGU_SCR_HEIGHT);

	GU_Viewport(2048, 2048, xView.m_fWidth * SCEGU_SCR_WIDTH, xView.m_fHeight * SCEGU_SCR_HEIGHT);
	GU_Offset(SCEGU_SCR_OFFSETX - SCEGU_SCR_WIDTH * (xView.m_fXOffset + xView.m_fWidth / 2.0f - 0.5f), SCEGU_SCR_OFFSETY - SCEGU_SCR_HEIGHT * (xView.m_fYOffset + xView.m_fHeight / 2.0f - 0.5f));

	Asura_PSP_Graphics::SetupPSPMatFromAsuraData(*Asura_Camera::GetOrientation(), *Asura_Camera::GetPosition());
}

void Asura_Render::Platform_AllowFullScreenRendering(void)
{
	
}


void Asura_Render::Platform_ClearZBuffer(void)
{
	Asura_Assert(IsRendering(), "Calling graphics function outside Begin/End frame");

	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	GU_Clear(SCEGU_CLEAR_DEPTH);
}


void Asura_Render::Platform_BeginFrame(void)
{
	Asura_PSP_Graphics::BeginFrame();
	//Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	//Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	//Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	
	//Asura_Direct3D::BeginFrame();	
#ifdef ASURA_PSP_ENABLE_LINE_RENDERING
	g_uCurrentLineVertex = 0;
	g_uCurrent2DLineVertex = 0;
#endif//def ASURA_PSP_ENABLE_LINE_RENDERING
}

#ifndef ASURA_PSP_SONY_SUBMISSION
	/* 'static' because it got to be the number for previous frame */
#if !defined(ASURA_RENDER_TTYPES)
	static u_int g_uTris = 0;
#endif	// !defined(ASURA_RENDER_TTYPES)
	#if 0
		/* Apparently the same as the tri count */
		static u_int g_uPass = 0;
	#endif
#endif

/*
	For standardcolor values and names:

		http://www.skillbank.co.uk/webeng/colours.htm
*/

#if defined(PSP_TEST_GUARDBAND_CLIPPING)
DEBUGVAR bool dbg_bShowClippingStats = false;
#endif	// defined(PSP_TEST_GUARDBAND_CLIPPING)

void Asura_Render::Platform_EndFrame(void)
{
#ifndef ASURA_PSP_SONY_SUBMISSION
	// (AW - 27/10/05) - Render various pieces of useful info. Based on
	// the state of the little switches on the front of the kits.

	/* Start from left margin */
	static const u_int uXOffset = 8;
	/* Start of first line and line height */
	static const u_int uYStep = 10;
	u_int uYOffset = 60;

	GU_DebugBegin();

	char szBuffer[200];
	if (Asura_PSP_GPIO::GetGPI(ASURA_PSP_GPI_FPS))
	{
		const float fFPS = Asura_Timers::GetSmoothedFrameRate() + 0.05f;
		const u_int uColour =
			(fFPS < 20.f) ? 0x000000ff /* red */
			: (fFPS < 30.f) ? 0x0000ffff /* yellow */
			: /*(fFPS >= 30.f) ?*/ 0x0000ff00 /* green */;

		sprintf(szBuffer, "FPS: %3.1f",fFPS);
		GU_DebugPrint(uXOffset,uYOffset,uColour,szBuffer); 
		uYOffset += uYStep;

		const Asura_Vector_3*	pxCamPos	= Asura_Camera::GetPosition();
		sprintf(szBuffer, "Cam Pos: (%3.1f, %3.1f, %3.1f)", pxCamPos->GetX(), pxCamPos->GetY(), pxCamPos->GetZ()); 
		GU_DebugPrint(uXOffset, uYOffset, 0xffffffff, szBuffer);
		uYOffset += uYStep;

		// QA requested this was added to the 'easily readable' bits as they often need to check the data
		const struct mallinfo xMallInfo = mallinfo();
		sprintf(szBuffer, "Mem: %iKB, %i blks.\n", xMallInfo.fordblks / 1024, xMallInfo.ordblks );
		GU_DebugPrint(uXOffset, uYOffset, 0xffffffff, szBuffer);
		uYOffset += uYStep;
	}

	if (Asura_PSP_GPIO::GetGPI(ASURA_PSP_GPI_DEBUG_INFO))
	{
		const unsigned uColour = 0x00ff00ff; /* fuchsia */
	#if defined ASURA_RENDER_TTYPES
		sprintf(szBuffer, "Tr: %5u En %5u+%-4u Se %5u Hi %4u Sk %4u",
			Asura_Render::GetNumberOfTrianglesRendered(),
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_ENV),
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_ENVGLUE),
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_SELF),
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_HIER),
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_SKIN)
		);
		GU_DebugPrint(uXOffset, uYOffset, uColour, szBuffer);
		uYOffset += uYStep;
		sprintf(szBuffer, "          Ob %2u Gl %2u Wa %2u Sh %2u Pa %2u Ot %3u",
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_OBJ),
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_GLS),
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_WAT),
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_SHA),
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_PFX),
			Asura_Render::GetNumberOfTrianglesRendered(ASURA_RENDER_TTYPE_OTHER)
		);
		GU_DebugPrint(uXOffset, uYOffset, uColour, szBuffer);
		uYOffset += uYStep;

		sprintf(szBuffer, "Hi %2u Sk %2u Sh %2u Pa %3u",
			Asura_Render::GetHierarchiesRenderedThisFrame(),
			Asura_Render::GetSmoothSkinsRenderedThisFrame(),
			Asura_Render::GetShapesRenderedThisFrame(),
			Asura_Render::GetParticlesRenderedThisFrame()
		);
	#else
		sprintf(szBuffer, "Tr: %5u; Hi %2u Sk %2u Sh %2u Pa %3u",
			g_uTris,
			Asura_Render::GetHierarchiesRenderedThisFrame(),
			Asura_Render::GetSmoothSkinsRenderedThisFrame(),
			Asura_Render::GetShapesRenderedThisFrame(),
			Asura_Render::GetParticlesRenderedThisFrame()
		);
	#endif
		GU_DebugPrint(uXOffset, uYOffset, uColour, szBuffer);
		uYOffset += uYStep;

		sprintf(szBuffer, "CPU Speed: %umhz", scePowerGetCpuClockFrequencyInt());
		GU_DebugPrint(uXOffset, uYOffset, 0xffffffff, szBuffer); 
		uYOffset += uYStep;
		
#ifdef ASURA_PSP_DISPLAY_LIST_SIZE
		// overridden in Asura_PSPConfiguration.h
		const int iDisplayListSize = ASURA_PSP_DISPLAY_LIST_SIZE;
#else
		// use default value from Asura_PSP_Graphics.h
		const int iDisplayListSize = ASURA_PSP_DEFAULT_DISPLAY_LIST_SIZE;
#endif
		sprintf(szBuffer, "Max Display List: %ikB (%ikB Limit).\n", Asura_PSP_Graphics::GetMaxDisplayList() / 1024, iDisplayListSize / 1024);
		GU_DebugPrint(uXOffset, uYOffset, 0xffffffff, szBuffer);
		uYOffset += uYStep;

		const u_int uPlayingSounds = Asura_Sound_System::GetNumberOfPlayingSounds();
		const u_int uSoundColour = (uPlayingSounds < 16) ? 0xffffffff : 0xff0000ff; // go red when we hit 16
		sprintf(szBuffer, "Playing sounds: %2i", uPlayingSounds);
		GU_DebugPrint(uXOffset,uYOffset,uSoundColour,szBuffer); 
		uYOffset += uYStep;

		if(Asura_Network::IsExternalNetworkActive())
		{
			sprintf(szBuffer, "Latency: %1.1fs", Asura_Network::GetLatency());
			GU_DebugPrint(uXOffset, uYOffset, 0xffffffff, szBuffer);
			uYOffset += uYStep;

			sprintf(szBuffer, "Packet loss: %02.1f%%", Asura_Network::GetPacketLossPercent());
			GU_DebugPrint(uXOffset, uYOffset, 0xffffffff, szBuffer); 
			uYOffset += uYStep;

			u_int uSent, uReceived;
			Asura_Network::GetNetworkBandwidthUsage(uSent, uReceived);

			sprintf(szBuffer, "Sent: %ubps", uSent);
			GU_DebugPrint(uXOffset, uYOffset, 0xffffffff, szBuffer); 
			uYOffset += uYStep;

			sprintf(szBuffer, "Received: %ubps", uReceived);
			GU_DebugPrint(uXOffset, uYOffset, 0xffffffff, szBuffer); 
			uYOffset += uYStep;
		}
	}

	if (Asura_PSP_GPIO::GetGPI(ASURA_PSP_GPI_QA))
	{
		// ARC - 04 January 2007 17:02:22 - Can only do this if ASURA_PSP_PROFILING is defined (Simpsons doesn't have it defined)
#if defined(ASURA_PSP_PROFILING)
		sprintf(szBuffer, "Updating Server Entities: %d", Asura_Entity_System::GetNumServerEntitiesUpdating()); 
		GU_DebugPrint(uXOffset, uYOffset, 0xff00ffff, szBuffer);
		uYOffset += uYStep;

		sprintf(szBuffer, "Updating Client Entities: %d", Asura_Entity_System::GetNumClientEntitiesUpdating()); 
		GU_DebugPrint(uXOffset, uYOffset, 0xff00ffff, szBuffer);
		uYOffset += uYStep;

		sprintf(szBuffer, "Renderable Entities: %d", Asura_Entity_System::GetNumRenderableEntities()); 
		GU_DebugPrint(uXOffset, uYOffset, 0xff00ffff, szBuffer);
		uYOffset += uYStep;
#endif // ASURA_PSP_PROFILING

#ifdef ASURA_PHYSICS_ENABLE_DEBUG_INFO
		sprintf(szBuffer, "physics: %u actors, %u vehicles", Asura_Physics_System::GetNumActiveActors(), 
			Asura_Physics_System::GetNumActiveVehicleActors()); 
		GU_DebugPrint(uXOffset, uYOffset, 0xff00ffff, szBuffer);
		uYOffset += uYStep;
#endif
	}

#if defined(PSP_TEST_GUARDBAND_CLIPPING)
	if(dbg_bShowClippingStats)
	{
		u_int uColour = 0x0000ffff;
		Asura_PSP_RenderEnvironment_Zone& xZone = Asura_PSP_RenderEnvironment::GetCurrentZone();
		sprintf(szBuffer, "Module tests: %u", xZone.m_uNumModuleTests);
		GU_DebugPrint(8,uYOffset, uColour, szBuffer); uYOffset += uYStep;
		sprintf(szBuffer, "Module accepts: %u", xZone.m_uNumModuleAccepts);
		GU_DebugPrint(8,uYOffset, uColour, szBuffer); uYOffset += uYStep;
		sprintf(szBuffer, "Strip tests: %u", xZone.m_uNumStripTests);
		GU_DebugPrint(8,uYOffset, uColour, szBuffer); uYOffset += uYStep;
		sprintf(szBuffer, "Strip accepts: %u", xZone.m_uNumStripAccepts);
		GU_DebugPrint(8,uYOffset, uColour, szBuffer); uYOffset += uYStep;
		sprintf(szBuffer, "Strip behinds: %u", xZone.m_uNumStripBehinds);
		GU_DebugPrint(8,uYOffset, uColour, szBuffer); uYOffset += uYStep;
		sprintf(szBuffer, "Triangle tests: %u", xZone.m_uNumTriangleTests);
		GU_DebugPrint(8,uYOffset, uColour, szBuffer); uYOffset += uYStep;
		sprintf(szBuffer, "Triangle accepts: %u", xZone.m_uNumTriangleAccepts);
		GU_DebugPrint(8,uYOffset, uColour, szBuffer); uYOffset += uYStep;
		sprintf(szBuffer, "Max strip length: %u", xZone.m_uMaxStripLength);
		GU_DebugPrint(8,uYOffset, uColour, szBuffer); uYOffset += uYStep;
		xZone.m_uNumModuleTests = 0;
		xZone.m_uNumStripTests = 0;
		xZone.m_uNumTriangleTests = 0;
		xZone.m_uNumModuleAccepts = 0;
		xZone.m_uNumStripBehinds = 0;
		xZone.m_uNumStripAccepts = 0;
		xZone.m_uNumTriangleAccepts = 0;
		xZone.m_uMaxStripLength = 0;
	}
#endif	// defined(PSP_TEST_GUARDBAND_CLIPPING)

	GU_DebugEnd();

#endif //#ifndef ASURA_PSP_SONY_SUBMISSION

	Asura_PSP_Graphics::EndFrame();
	//FlushDisplay();
	//Asura_Direct3D::EndFrame();
}

void Asura_Render::Platform_FlushDisplay(void)
{
	// (MH - 19.12.2006) - overlay renderer seems to be interfering with entity rendering.
	// Since this function gets called after rendering text, it fixes the problem.
	Asura_PSP_Graphics::InvalidateMaterialType();

#ifndef ASURA_PSP_SONY_SUBMISSION
	/*
		The number of triangles rendered must be obtained _after_ the
		end of a frame, so here to be displayed in the next frame.
		Thanks to RichardR for the hint.
	*/
#if !defined(ASURA_RENDER_TTYPES)
    g_uTris   = Asura_Render::GetNumberOfTrianglesRendered();
#endif	// !defined(ASURA_RENDER_TTYPES)
	#if 0
		/* Apparently the same as the tri count */
		g_uPass   = Asura_Render::GetNumTriPassesRendered();
	#endif
#endif
}

bool Asura_Render::Platform_SetVideoMode(int width, int height, bool fsaa)
{
	// Can't change resolution on PSP
	return true;
}

void Asura_Render::Platform_InitialiseSplashScreen(const char* szName)
{	
}

void Asura_Render::Platform_ShutdownSplashScreen(void)
{	
}

void Asura_Render::Platform_SplashScreenRender(void)
{	
}

#ifdef ASURA_PSP_ENABLE_LINE_RENDERING
void Asura_Render::Platform_RenderLine(const Asura_Vector_3& xP1, const Asura_Vector_3& xP2, u_int uColour)
{
    Asura_Assert( g_uCurrentLineVertex + 1 < ASURA_PSP_MAX_LINE_VERTICES, "Too many lines this frame" );
    if( g_uCurrentLineVertex + 1 >= ASURA_PSP_MAX_LINE_VERTICES ) { return; }

	Asura_PSP_Graphics::InvalidateMaterialType();
	Asura_Render::CurrentStates.ResetAll();

	// Use vertex colours
	Asura_PSP_Graphics::SetMaterialType( ASURA_PSP_GRAPHICS_MATERIALTYPE_ENV );
	Asura_PSP_Graphics::SetUseLighting( false );
	Asura_PSP_Graphics::SetUseColourDoubling( false );
	CurrentStates.RequestAlphaTest( true );
    CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
	CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
	CurrentStates.RequestZBufferEnabled( true );
	CurrentStates.RequestZBufferWriteEnabled( false );
	Asura_PSP_TextureManagement::RequestTextureIndex( ASURA_TEXTURE_INVALID );

	// convert ARGB to ABGR
	const u_int uRed	= (uColour&0x00ff0000)>>16;
	const u_int uAlpha	= (uColour&0xff000000);
	const u_int uGreen	= (uColour&0x0000ff00);
	const u_int uBlue	= (uColour&0x000000ff)<<16;
	uColour = (uRed) | (uGreen) | (uBlue) | (uAlpha);

	// Set up the vertex properties
	const u_int uV0 = g_uCurrentLineVertex;
	const u_int uV1 = g_uCurrentLineVertex + 1;
	g_axLineVertices[uV0].m_afPosition[0] = xP1.x;
	g_axLineVertices[uV0].m_afPosition[1] = xP1.y;
	g_axLineVertices[uV0].m_afPosition[2] = xP1.z;
	g_axLineVertices[uV0].m_uDiffuseRGBA = uColour;
	g_axLineVertices[uV1].m_afPosition[0] = xP2.x;
	g_axLineVertices[uV1].m_afPosition[1] = xP2.y;
	g_axLineVertices[uV1].m_afPosition[2] = xP2.z;
	g_axLineVertices[uV1].m_uDiffuseRGBA = uColour;

	// Flush the part of the cache we want to render
	sceKernelDcacheWritebackRange(&g_axLineVertices[g_uCurrentLineVertex], sizeof(Asura_PSP_3D_Line_Vertex) * 2);

	GUM_DrawArray
	(
		SCEGU_PRIM_LINES,
		uASURA_LINE_VERTEX_DEFINITION,
		2,
		NULL,
		&g_axLineVertices[g_uCurrentLineVertex]
	);
	g_uCurrentLineVertex += 2;
}

void Asura_Render::Platform_Render2DLine(const Asura_Vector_2& xPoint1, const Asura_Vector_2& xPoint2, u_int uColour)
{
	Asura_Assert( g_uCurrent2DLineVertex + 1 < ASURA_PSP_MAX_LINE_VERTICES, "Too many lines this frame" );
	if( g_uCurrent2DLineVertex + 1 >= ASURA_PSP_MAX_LINE_VERTICES ) { return; }

	Asura_PSP_Graphics::InvalidateMaterialType();
	Asura_Render::CurrentStates.ResetAll();

	// Use vertex colours
	Asura_PSP_Graphics::SetMaterialType( ASURA_PSP_GRAPHICS_MATERIALTYPE_ENV );
	Asura_PSP_Graphics::SetUseLighting( false );
	Asura_PSP_Graphics::SetUseColourDoubling( false );
	CurrentStates.RequestAlphaTest( true );
	CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
	CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );

	CurrentStates.RequestZBufferEnabled( false );
	CurrentStates.RequestZBufferWriteEnabled( false );

	Asura_PSP_TextureManagement::RequestTextureIndex( ASURA_TEXTURE_INVALID );

	// convert ARGB to ABGR
	const u_int uRed	= (uColour&0x00ff0000)>>16;
	const u_int uAlpha	= (uColour&0xff000000);
	const u_int uGreen	= (uColour&0x0000ff00);
	const u_int uBlue	= (uColour&0x000000ff)<<16;
	uColour = (uRed) | (uGreen) | (uBlue) | (uAlpha);

	// Set up the vertex properties
	const u_int uV0 = g_uCurrent2DLineVertex;
	const u_int uV1 = g_uCurrent2DLineVertex + 1;

	g_ax2DLineVertices[uV0].m_ausPosition[0] = xPoint1.x;
	g_ax2DLineVertices[uV0].m_ausPosition[1] = xPoint1.y;
	g_ax2DLineVertices[uV0].m_ausPosition[2] = 32767; // 32767 seems to be near plane
	g_ax2DLineVertices[uV0].m_uDiffuseRGBA = uColour;

	g_ax2DLineVertices[uV1].m_ausPosition[0] = xPoint2.x;
	g_ax2DLineVertices[uV1].m_ausPosition[1] = xPoint2.y;
	g_ax2DLineVertices[uV1].m_ausPosition[2] = 32767; // 32767 seems to be near plane
	g_ax2DLineVertices[uV1].m_uDiffuseRGBA = uColour;

	// Flush the part of the cache we want to render
	sceKernelDcacheWritebackRange(&g_ax2DLineVertices[g_uCurrent2DLineVertex], sizeof(Asura_PSP_2D_Line_Vertex) * 2);

	GUM_DrawArray
		(
		SCEGU_PRIM_LINES,
		uASURA_2DLINE_VERTEX_DEFINITION,//SCEGU_COLOR_PF8888|SCEGU_VERTEX_SHORT|SCEGU_THROUGH
		2,
		NULL,
		&g_ax2DLineVertices[g_uCurrent2DLineVertex]
	);
	g_uCurrent2DLineVertex += 2;

}

#else//def ASURA_PSP_ENABLE_LINE_RENDERING
// Dummy implementation for when line rendering is disabled
void Asura_Render::Platform_RenderLine(const Asura_Vector_3& xP1, const Asura_Vector_3& xP2, u_int uColour)
{
	Asura_Assert(0, "Line rendering will not work on PSP unless ASURA_PSP_ENABLE_LINE_RENDERING is defined (32KB buffer cost).");
}

void Asura_Render::Platform_Render2DLine(const Asura_Vector_2& xPoint1, const Asura_Vector_2& xPoint2, u_int uColour)
{
	Asura_Assert(0, "Line rendering will not work on PSP unless ASURA_PSP_ENABLE_LINE_RENDERING is defined (32KB buffer cost).");
}

#endif//else def ASURA_PSP_ENABLE_LINE_RENDERING


void Asura_Render::Platform_RenderDynamicShadowsOntoEnvironment()
{
}


void Asura_Render::Platform_PrepareToRenderFirstPersonMesh ()
{
	Asura_Assert(false,"Special rendering settings for the first person mesh are not supported on this platform");
}


void Asura_Render::Platform_FinishRenderFirstPersonMesh ()
{
	//Do nothing
}


/// End of file. ///
