//
// Asura_Platform_Shape.cpp
//
// Created by KJFL - Tuesday, March 30, 2004
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Shapes/Asura_Platform_Shape.h"

#include "Rendering/Asura_Render.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Material_System.h"

#include "Rendering/Asura_PSP_ObjectRenderer.h"

#include "Debugging/Asura_PSP_Profiling.h"

/////////////////////////////////////////////////////////////////////////////////////////
void Asura_Shape_Instance::Platform_SetAmbientARGB( const unsigned int uARGB )
{
	// PC implementation - just copy
	m_uAmbientARGB = uARGB;
	m_uFlags &= ~ASURA_SHAPE_INSTANCE_USES_DEFAULT_AMBIENT;
}

/////////////////////////////////////////////////////////////////////////////////////////
// FUNCTION		Asura_Shape_Instance::Platform_SetAmbientRGB
// PARAMETERS	components of colour in four 8 bit value (RGBA)
// RETURNS		None
// DESCRIPTION	
// AUTHOR		Neil Massey (13th Sept. 2002)
/////////////////////////////////////////////////////////////////////////////////////////

void Asura_Shape_Instance::Platform_SetAmbientRGB( const unsigned char ucR, const unsigned char ucG, const unsigned char ucB, const unsigned char ucA)
{
	// Just generate as a bit field for PC
	m_uAmbientARGB = (ucA << 24) | (ucR << 16) | (ucG << 8) | ucB;
	m_uFlags &= ~ASURA_SHAPE_INSTANCE_USES_DEFAULT_AMBIENT;
}

/////////////////////////////////////////////////////////////////////////////////////////
// FUNCTION		Asura_Shape_Instance::Platform_SetDefaultAmbientRGB
// PARAMETERS	None
// RETURNS		None
// DESCRIPTION	
// AUTHOR		Richard M. Smith (9 August 2002)
// MODIFIED     Mike Healey (26.9.02) now calling the appropriate function to set up
//                          ambient to correct values for each platform.
// MODIFIED		Andrew Weinkove (01.03.04) Now just set a flag to say it uses default ambient
/////////////////////////////////////////////////////////////////////////////////////////
void Asura_Shape_Instance::Platform_SetDefaultAmbientRGB( void )
{
	m_uFlags |= ASURA_SHAPE_INSTANCE_USES_DEFAULT_AMBIENT;
} 


int Asura_Shape::Platform_GetMaterialIdx( void ) const
{
    return Asura_PSP_ObjectRenderer::GetMaterialIndex( *this );
}

void Asura_Shape::Platform_Render( const Asura_Shape_Instance* pxShapeInst, u_int uAmbientARGB, u_int uCamVisFlags ) const
{


#ifdef ASURA_PSP_PROFILING
		Asura_PSP_Profiling::Signal( Asura_PSP_Profiling::ASURA_PSP_SIGNAL_ENTITY_START ); 
#endif // #ifdef ASURA_PSP_PROFILING

		Asura_Guid guid = ASURA_GUID_UNREGISTERED;
		for( int iCam = Asura_Camera::GetNumberOfCameras(); iCam--; )
		{
			Asura_Camera::SetCameraIndex( iCam );
			if (Asura_Camera::IsCameraActive() && 
				((1 << iCam) & uCamVisFlags) &&
				(Asura_Camera::GetEntityToIgnore() != guid) )
			{
				Asura_Render::ApplyCameraSettings();

				Asura_PSP_ObjectRenderer::Render( this, pxShapeInst, uAmbientARGB );
			}
		}

#ifdef ASURA_PSP_PROFILING
		Asura_PSP_Profiling::Signal( Asura_PSP_Profiling::ASURA_PSP_SIGNAL_ENTITY_END ); 
#endif // #ifdef ASURA_PSP_PROFILING

}

void Asura_Shape::Platform_RenderShadow( const Asura_Shape_Instance* pxShapeInst, const Asura_Shadow& xShadow, u_int uCamVisFlags ) const
{
}

// AJG - 23/06/04 - Render data constructor. Skip the filename string. Copied, comment and all, from Gamecube constructor.
Asura_Platform_Shape_RenderData::Asura_Platform_Shape_RenderData( const char*& pcChunkBuffer )
{
	// Ooooh! Magic number!
	pcChunkBuffer += 260;
}

bool Asura_Shape::Platform_IsOpaque( void ) const
{
	return !Asura_Material_System::IsMaterialTransparent(Platform_GetMaterialIdx());
}
