/**
*
* \file Asura_PlatformSound.cpp
*
* Created by Kevin Lea - 25/04/2002 13:04:50
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

// This file's header
#include "Sound/Asura_Sound.h"

// Asura includes
#include "Sound/Asura_StreamingBackgroundSound_System.h"

// Asura PSP includes
#include "Sound/Asura_PSP_Atrac.h"
#include "Sound/Asura_PSP_SAS.h"
#include "Sound/Asura_PSP_Sound.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////
int Asura_Sound_System::PlatformInitialise(void)
{
	Asura_PSP_Sound::Initialise();
	return uASURA_PSP_MAX_VOICES;
}

void Asura_Sound_System::PlatformShutdown(void)
{
	Asura_PSP_Sound::Shutdown();
}

// (MH - 03.07.2002) may want to check that sound memory available at reset point
// is always the same.
void Asura_Sound_System::PlatformReset(void)
{
	Asura_PSP_Sound::Reset();
}

void Asura_Sound_System::PlatformSetResetPoint(void)
{
	// Make all currently loaded sounds loaded permanently. May not be necessary on PSP.
	Asura_PSP_Sound::SetResetPoint();
}

void Asura_Sound_System::PlatformUpdate()
{}

/// File handling /////////////////////////////////////////////////////////////////////////////////

bool Asura_Sound_System::PlatformLoadSound( const char* szFilename, const int iLoadedSoundID)
{
	// KOR - do we need this?
	return false;
}

bool Asura_Sound_System::PlatformLoadSoundFromResource(Asura_Handle iFileHandle, int iFileSize, int iLoadedSoundID)
{
	return Asura_PSP_SAS::LoadSoundFromResource(iFileHandle, iFileSize, iLoadedSoundID);
}

bool Asura_Sound_System::PlatformUnloadSound(const int iLoadedSoundID)
{
	if (Asura_PSP_Sound::IsResident(iLoadedSoundID))
	{
		return Asura_PSP_SAS::UnloadSound(iLoadedSoundID);
	}
	return true;
}

/// Basic sound playing functionality /////////////////////////////////////////////////////////////

bool Asura_Sound_System::PlatformCreateSound(int iLoadedSoundID, Asura_Handle iSoundHandle, bool bUseEQ, bool bUseHardware3dPan)
{
	return Asura_PSP_Sound::CreateSound(iLoadedSoundID, iSoundHandle);
}

bool Asura_Sound_System::PlatformPlaySound(Asura_Handle iSoundHandle, bool bLooping)
{
	return Asura_PSP_Sound::PlaySound(iSoundHandle, bLooping);
}

bool Asura_Sound_System::PlatformSoundIsPlaying(Asura_Handle iSoundHandle)
{
	return Asura_PSP_Sound::IsSoundPlaying(iSoundHandle);
}

bool Asura_Sound_System::PlatformSoundIsPreparing( Asura_Handle iPlayingSoundHandle )
{
	return false;
}

bool Asura_Sound_System::PlatformStopSound(Asura_Handle iSoundHandle)
{
	return Asura_PSP_Sound::StopSound(iSoundHandle);
}

bool Asura_Sound_System::PlatformDestroySound(Asura_Handle iSoundHandle)
{
	return Asura_PSP_Sound::DestroySound(iSoundHandle);
}

bool Asura_Sound_System::PlatformPauseSound(Asura_Handle iSoundHandle)
{
	return Asura_PSP_Sound::PauseSound(iSoundHandle);
}

bool Asura_Sound_System::PlatformUnpauseSound(Asura_Handle iSoundHandle)
{
	return Asura_PSP_Sound::UnpauseSound(iSoundHandle);
}

/// Sound set property methods ////////////////////////////////////////////////////////////////////

bool Asura_Sound_System::PlatformSetSoundVolumePanAndFade(Asura_Handle iSoundHandle, float fVolume, float fPan, float fFade)
{
	return Asura_PSP_Sound::SetSoundVolumePanAndFade(iSoundHandle, fVolume, fPan, fFade);
}

bool Asura_Sound_System::PlatformSetSoundPitch(Asura_Handle iSoundHandle, float fPitch)
{
	return Asura_PSP_Sound::SetSoundPitch(iSoundHandle, fPitch);
}

void Asura_Sound_System::Platform_SetReverb(enum ReverbType eType, float fReverbAmount)
{
}

/// Replaceable sounds ////////////////////////////////////////////////////////////////////////////

bool Asura_Sound_System::PlatformLoadReplaceableSound(const int iLoadedSoundID)
{
	return false;
}

bool Asura_Sound_System::PlatformReplaceSound(const int iExistingSoundLoadedID, const int iNewSoundLoadedID)
{
	return false;
}

int Asura_Sound_System::Platform_GetMaxActiveReplaceableSounds()
{
	// KOR - 23/09/2004 - arbitrary limit until we work out replaceable sounds on the PSP
	return 4;
}

int Asura_Sound_System::Platform_GetReplaceableSoundSize(const int iLoadedSoundID)
{
    return 0;
}

// Hardware panning is currently PC-only (it's the only way under DirectSound to access surround speakers)
// All other platforms allow us to specify speaker volumes directly (or if using Dolby PL2 have some concept of front and back as well as pan).
bool Asura_Sound_System::Platform_HardwarePanSetSoundVolume(Asura_Handle iSoundHandle, float fVolume)
{
	Asura_Assert(false, "this platform did not expect to use hardware 3d pan");
	return true;
}

bool Asura_Sound_System::Platform_HardwarePanSetSoundPosition(Asura_Handle iSoundHandle, const Asura_Vector_3& xPosition)
{
	Asura_Assert(false, "this platform did not expect to use hardware 3d pan");
	return true;
}

bool Asura_Sound_System::Platform_HardwarePanSetSoundSpatial(Asura_Handle iSoundHandle, bool bIsSpatial)
{
	Asura_Assert(false, "this platform did not expect to use hardware 3d pan");
	return true;
}

bool Asura_Sound_System::Platform_HardwarePanSetSoundRadii(Asura_Handle iSoundHandle, float fInner, float fOuter)
{
	Asura_Assert(false, "this platform did not expect to use hardware 3d pan");
	return true;
}

bool Asura_Sound_System::Platform_HardwarePanSetSoundForcedPanAndFade(Asura_Handle iSoundHandle, float fPan, float fFade)
{
	Asura_Assert(false, "this platform did not expect to use hardware 3d pan");
	return true;
}

/// Streaming sounds //////////////////////////////////////////////////////////////////////////////

bool Asura_Sound_System::Platform_CreateStreamingSound( Asura_Handle iHandle, const char* pszFilename)	
{
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");

	// Fix the metamusic extensions
	char szFixedFilename[256];
	strncpy(szFixedFilename, pszFilename, 256); 

	const int iLength = strlen(szFixedFilename); 
	strcpy(&(szFixedFilename[iLength-4]), ".AT3");

	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	// JAT 0 = data offset, -1 = means play the whole file
	return Asura_PSP_Atrac::Create( xMusicSoundHandle, szFixedFilename, 0, -1);
}

void Asura_Sound_System::Platform_PlayStreamingSound(Asura_Handle iHandle, bool bLooping, bool bPreload) 
{
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	// As far as I know, we will preload anyway when we create the streaming sound
	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	Asura_PSP_Atrac::Play( xMusicSoundHandle, bLooping );
}

void Asura_Sound_System::Platform_StopStreamingSound( Asura_Handle iHandle ) 
{
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	if ( Asura_PSP_Atrac::IsPlaying( xMusicSoundHandle ) == true )
	{
		Asura_PSP_Atrac::Stop( xMusicSoundHandle );
	}
}

void Asura_Sound_System::Platform_PauseStreamingSound(Asura_Handle iHandle) 
{
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	Asura_PSP_Atrac::PauseTrack( xMusicSoundHandle, true );
}

void Asura_Sound_System::Platform_ResumeStreamingSound(Asura_Handle iHandle) 
{
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	Asura_PSP_Atrac::PauseTrack( xMusicSoundHandle, false );
}

void Asura_Sound_System::Platform_ReleaseStreamingSound(Asura_Handle iHandle) 
{
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	Asura_PSP_Atrac::Release( xMusicSoundHandle );
}

bool Asura_Sound_System::Platform_StreamingSoundIsReady(Asura_Handle iHandle) 
{	
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	// KOR - 31/03/2006 - on PSP, length is initialised as negative,
	// and is set to the correct value once the the sound is ready
	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	return (Asura_PSP_Atrac::GetLength( xMusicSoundHandle ) > 0.f);
}

void Asura_Sound_System::Platform_StartPreloadedStreamingSound(Asura_Handle iHandle) 
{
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	// Right now, everything is preloaded. Do nothing
}

void Asura_Sound_System::Platform_SetStreamingSoundOutput(Asura_Handle iHandle, SoundOutputType eOutputType) 
{
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	// No support for this at the moment
}

bool Asura_Sound_System::Platform_StreamingSoundIsFinished(Asura_Handle iHandle) 
{ 
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	return !Asura_PSP_Atrac::IsPlaying( xMusicSoundHandle );
}

float Asura_Sound_System::Platform_GetStreamingSoundPosition(Asura_Handle iHandle) 
{ 
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	return Asura_PSP_Atrac::GetPosition( xMusicSoundHandle );
}
float Asura_Sound_System::Platform_GetStreamingSoundLength(Asura_Handle iHandle) 
{ 
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	return Asura_PSP_Atrac::GetLength( xMusicSoundHandle );
}

int Asura_Sound_System::Platform_GetMaxStreamingSounds() 
{ 
	// KOR - 12/05/2006 - streaming more than one sound off UMD would be nasty.
	return 1; 
}

void Asura_Sound_System::Platform_EnsureAllStreamingSoundsAreStopped() 
{
	// Make sure all our streams are stopped
	Asura_PSP_Atrac::StopAll();
}

void Asura_Sound_System::Platform_SetStreamingSoundVolume(Asura_Handle iHandle, float fVolume) 
{
	Asura_Assert(iHandle == 0, "Cannot cope with more than one stream on PSP");
	const Asura_Handle xMusicSoundHandle = Asura_PSP_Sound::GetMusicSoundHandle();
	Asura_PSP_Atrac::SetVolume( xMusicSoundHandle, fVolume );
}

bool Asura_Sound_System::Platform_SetSoundUsesReverb( Asura_Handle iPlayingSoundHandle, bool bUseReverb )
{
	return true;
}

bool Asura_Sound_System::Platform_SetSoundUsesDistanceFilter( Asura_Handle iPlayingSoundHandle, bool bUseFilter )
{
	return true;
}

bool Asura_Sound_System::Platform_SetSoundDistanceFilterGain (Asura_Handle iPlayingSoundHandle, float fNormalisedGain)
{
	//Platform does not use this function
	return true;
}

bool Asura_Sound_System::Platform_SetSoundWetDryVolume(Asura_Handle iPlayingSoundHandle, float fWetVolume, float fDryVolume)
{
	//TODO: Write me please
	return true;
}

bool Asura_Sound_System::Platform_SetOcclusionLPF(const Asura_Handle iPlayingSoundHandle, const float fOcclusion)
{
	//TODO: Write me please
	return true;
}


/// Asura_StreamingBackgroundSound_System /////////////////////////////////////////////////////////
// (not really worth giving its own file
///////////////////////////////////////////////////////////////////////////////////////////////////

int Asura_Sound_System::Platform_GetMaxSoundsWithEQ(void)
{
	// return number of voices which may have EQ applied. Generally this 
	// will be the same return value as PlatformInitialise() but on PS2 only
	// half the voices can have EQ.
	return 0;
}

void Asura_Sound_System::Platform_SetEQType(Asura_Handle iHandle, EQType eType, float fLevel)
{
}

