//
// Asura_Platform_ShadowDecal.cpp
//
// Created by KJFL - 15 March 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "SpecialFX/Asura_ShadowDecal_System.h"
#include "SpecialFX/Asura_ShadowDecal_System_Internal.h"

#include "Camera/Asura_Camera.h"

#include "ProjectSettings/Asura_PSPConfiguration.h"
#include "Rendering/Asura_PSP_Fog.h"
#include "Rendering/Asura_PSP_Graphics.h"
#include "Rendering/Asura_PSP_RenderStates.h"
#include "Rendering/Asura_Render.h"

#include "Core/Asura_TextureManagement.h"
#include <kernel.h>


#if !defined(ASURA_PSP_MAX_SHADOW_DECAL_VERTS)
#define	ASURA_PSP_MAX_SHADOW_DECAL_VERTS	2048
#endif	// !defined(ASURA_PSP_MAX_SHADOW_DECAL_VERTS)

static const int iASURA_PSP_DECAL_VERTEX_FORMAT = SCEGU_TEXTURE_FLOAT | SCEGU_VERTEX_FLOAT;


struct Asura_PSP_RenderDecal_Vertex
{
	// fully 32bit - 20 bytes
	float			m_fU;
	float			m_fV;
	float			m_afPosition[3];

	inline bool operator != ( const Asura_PSP_RenderDecal_Vertex& xVertex) const
	{
	    return (m_afPosition[0] != xVertex.m_afPosition[0] ||
				m_afPosition[1] != xVertex.m_afPosition[1] ||
				m_afPosition[2] != xVertex.m_afPosition[2] ||
				m_fU != xVertex.m_fU ||
				m_fV != xVertex.m_fV);
	}
};


class Asura_PSP_ShadowDecalBuffer
{
public:
	Asura_PSP_ShadowDecalBuffer()
	{
		m_uMaxNumberOfIndices = ASURA_PSP_MAX_SHADOW_DECAL_VERTS - 24;
		m_uNumberOfVerticesToDraw = 0;
	
		m_uCurrentIndex = 0;
		m_uMaxIndex = 0;
	}

	~Asura_PSP_ShadowDecalBuffer()
	{
	}

	void Reset( void )
	{
		m_uNumberOfVerticesToDraw = 0;
		m_uCurrentIndex = 0;
		m_uMaxIndex = 0;
	}

	bool IsEmpty( void ) const
	{
		return (m_uNumberOfVerticesToDraw==0);
	}


	void AddStrip(const Asura_ShadowDecal_Vertex* pxStrip, const u_int uNumberOfVertices)
	{
		if (m_uCurrentIndex + uNumberOfVertices+2 > m_uMaxNumberOfIndices)
		{
			m_uCurrentIndex = 0;
		}

		unsigned uFirstVertexIndex = m_uCurrentIndex;
		m_uCurrentIndex++;
		// Copy all of the vertices.
		for(unsigned i=0; i<uNumberOfVertices; i++)
		{
			unsigned v = i + m_uCurrentIndex;
			m_axVertices[v].m_afPosition[0] = pxStrip[i].m_xPosition.x;
			m_axVertices[v].m_afPosition[1] = pxStrip[i].m_xPosition.y;
			m_axVertices[v].m_afPosition[2] = pxStrip[i].m_xPosition.z;
			m_axVertices[v].m_fU = pxStrip[i].m_fU;
			m_axVertices[v].m_fV = pxStrip[i].m_fV;
		}
		m_uCurrentIndex += uNumberOfVertices;
		// First vertex is copied twice to allow seperate decals to be drawn as a single strip.
		m_axVertices[uFirstVertexIndex] = m_axVertices[uFirstVertexIndex + 1];
		// Last vertex is copied twice to allow seperate decals to be drawn as a single strip.
		m_axVertices[m_uCurrentIndex] = m_axVertices[m_uCurrentIndex - 1];
		m_uCurrentIndex++;
			
		// KJL - 29/04/2003 10:32:10 - if wrapped we have to tidy up any decals we've half-overwritten
		if (m_uCurrentIndex<m_uMaxIndex)
		{
			unsigned uTemp = m_uCurrentIndex;
			while (m_axVertices[uTemp] != m_axVertices[uTemp+1] && 
					uTemp < m_uMaxIndex - 1)
			{
				m_axVertices[uTemp] = m_axVertices[uTemp-1];
				uTemp++;
			}
		}
		else
		{
			m_uMaxIndex = m_uCurrentIndex;
		}
		
		if (m_uCurrentIndex>m_uNumberOfVerticesToDraw)
		{
			m_uNumberOfVerticesToDraw = m_uCurrentIndex;
		}
	}

	void Render(void)
	{
		sceKernelDcacheWritebackRange( m_axVertices, m_uNumberOfVerticesToDraw * sizeof(Asura_PSP_RenderDecal_Vertex) );

		GUM_DrawArray(SCEGU_PRIM_TRIANGLE_STRIP, 
						iASURA_PSP_DECAL_VERTEX_FORMAT,
						m_uNumberOfVerticesToDraw, 
						NULL, 
						m_axVertices);

	#if defined ASURA_RENDER_TTYPES
		Asura_Render::AddToTrianglesRendered(m_uNumberOfVerticesToDraw - 2,1,
			ASURA_RENDER_TTYPE_SHA);
	#else
		Asura_Render::AddToTrianglesRendered(m_uNumberOfVerticesToDraw - 2);
	#endif

	}

private:
	Asura_PSP_RenderDecal_Vertex m_axVertices[ASURA_PSP_MAX_SHADOW_DECAL_VERTS];
	unsigned m_uMaxNumberOfIndices;
	unsigned m_uNumberOfVerticesToDraw;
	unsigned m_uCurrentIndex;
	u_int m_uMaxIndex;

};
static Asura_PSP_ShadowDecalBuffer g_xShadowDecalBuffer;
static int g_iShadowTexture = 1;

void Asura_ShadowDecal_System::Platform_Initialise(void)
{
	g_xShadowDecalBuffer.Reset();

	g_iShadowTexture = ASURA_TEXTURE_NONE;
	Asura_Texture_Management::LoadTextures();
}

void Asura_ShadowDecal_System::Platform_Reset(void)
{
	Platform_Initialise();
}

void Asura_ShadowDecal_System::Platform_Shutdown(void)
{
	g_xShadowDecalBuffer.Reset();
}

void Asura_ShadowDecal_System::Platform_AddStrip(const Asura_ShadowDecal_Vertex* pxStrip, u_int uNumVertices )
{
	g_xShadowDecalBuffer.AddStrip(pxStrip, uNumVertices);
}

DEBUGVAR bool dbg_bASURA_PSP_DEPTHOFFSETWORKS = true;
DEBUGVAR int dbg_iShadowDecalDepthOffset = -100; 

void Asura_ShadowDecal_System::Platform_Render(void)
{
	if (g_xShadowDecalBuffer.IsEmpty()) return;

	if ( g_iShadowTexture == ASURA_TEXTURE_NONE)
	{
		g_iShadowTexture = Asura_Texture_Management::GetTextureIndex ("\\specialfx\\blobshadow.tga");
	}

	// KJFL - 28 February 2005 - disable vertex fog if it's on.
	Asura_PSP_Fog::TurnOffFogIfNecessary();

	if (dbg_bASURA_PSP_DEPTHOFFSETWORKS)
	{
		Asura_PSP_RenderStates::RequestDepthOffset(dbg_iShadowDecalDepthOffset);
	}
	else
	{
		Asura_PSP_Graphics::SetupPSPProjectionMatrix(0.010f);
	}
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestTextureIndex(0,g_iShadowTexture);
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
	Asura_PSP_Graphics::SetUseColourDoubling(false);
	Asura_PSP_Graphics::SetUseLighting(false);
	Asura_PSP_Graphics::SetMaterialType( ASURA_PSP_GRAPHICS_MATERIALTYPE_ENTITY );

	g_xShadowDecalBuffer.Render();

	if (dbg_bASURA_PSP_DEPTHOFFSETWORKS)
	{
		Asura_PSP_RenderStates::RequestDepthOffset(0);	// Restore the depth offset.
	}
	else
	{
		Asura_PSP_Graphics::SetupPSPProjectionMatrix(0.000f);
	}
	
	g_xShadowDecalBuffer.Reset();
}
