
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Platform_Streaming_System.cpp
// Author : Andy Tate
// Date   : 9-May-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// RJT NOTE: Required for GCC builds as the file list generation doesn't account for VC project exclusion from build
#ifdef ASURA_USE_STREAMING

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Streaming/Asura_Streaming_System.h"

// Asura independent header-files...

// Asura platform header-files...
#include "Core/Asura_PSP_VolatileMemoryManagement.h"

// Game independent header-files...

// Game platform header-files...

// Tools header-files...


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

static Asura_Handle	g_uVolatileMemoryHandle	= ASURA_INVALID_HANDLE;
static u_char*		g_pucVolatileMemory		= NULL;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AllocateVolatileMemory
// Author       : Andy Tate
// Date         : 18-May-2006
////////////////////////////////////////////////////////////////////////////////

static void AllocateVolatileMemory (u_char* pucDataStart)
{
	g_pucVolatileMemory	= pucDataStart;
}

/// End of function 'AllocateVolatileMemory'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : DeallocateVolatileMemory
// Author       : Andy Tate
// Date         : 18-May-2006
////////////////////////////////////////////////////////////////////////////////

static void DeallocateVolatileMemory (u_char* pucDataStart)
{
	g_pucVolatileMemory	= NULL;
}

/// End of function 'DeallocateVolatileMemory'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_System::Platform_AllocateFileCacheMemory
// Author       : Andy Tate
// Date         : 9-May-2006
////////////////////////////////////////////////////////////////////////////////

u_char* Asura_Streaming_System::Platform_AllocateFileCacheMemory (const u_int uSize)
{
	g_uVolatileMemoryHandle = Asura_PSP_VolatileMemoryManagement::Allocate(uSize,AllocateVolatileMemory,DeallocateVolatileMemory);
	Asura_Assert((g_uVolatileMemoryHandle != ASURA_INVALID_HANDLE), "Failed to allocate PSP volatile memory for streaming file-cache");
	return(g_pucVolatileMemory);
}

/// End of function 'Asura_Streaming_System::Platform_AllocateFileCacheMemory'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_Streaming_System::Platform_DeallocateFileCacheMemory
// Author       : Andy Tate
// Date         : 9-May-2006
////////////////////////////////////////////////////////////////////////////////

void Asura_Streaming_System::Platform_DeallocateFileCacheMemory (u_char* pucData)
{
	Asura_PSP_VolatileMemoryManagement::Deallocate(g_uVolatileMemoryHandle);
}

/// End of function 'Asura_Streaming_System::Platform_DeallocateFileCacheMemory'. ///



////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

#endif	// ASURA_USE_STREAMING

/// End of file. ///
