//
// Asura_Platform_Timers.cpp
//
// Created by KOR - 09 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Core/Asura_Timers.h"

// Asura headers
#include "Debugging/Asura_Assert.h"

// Library headers
#include <revolution/os.h>

static Asura_TickCount g_uLastTickCount = 0;
const static float g_fFrequency = (float)OSSecondsToTicks(1.0f);
const static float g_fTickLength = 1.f / g_fFrequency;

static inline Asura_TickCount GetCurrentTickCount()
{	
	return OSGetTick();
}

void Asura_Timers::Platform_Initialise()
{}

void Asura_Timers::Platform_Reset()
{
	g_uLastTickCount = GetCurrentTickCount();
}


void Asura_Timers::Platform_Shutdown()
{}


void Asura_Timers::Platform_Update()
{
}

u_int Asura_Timers::Platform_GetTicksThisFrame()
{
	const Asura_TickCount uNewTickCount = GetCurrentTickCount();
	const Asura_TickCount uTicksThisFrame = OSDiffTick(uNewTickCount, g_uLastTickCount);

	g_uLastTickCount = uNewTickCount;

	return uTicksThisFrame;
}

u_int Asura_Timers::Platform_GetTicksInFrame()
{
	const Asura_TickCount uNewTickCount = GetCurrentTickCount();
	return OSDiffTick(uNewTickCount, g_uLastTickCount);
}

float Asura_Timers::Platform_GetTickFrequency()
{
	return g_fFrequency;
}


float Asura_Timers::Platform_GetDefaultFrameRate()
{
	return 60.0f;
}

Asura_TickCount Asura_Timers::Platform_GetCurTicks()
{
	return GetCurrentTickCount();
}

Asura_TickCount Asura_Timers::Platform_GetElapsedTicks(Asura_TickCount xStartTicks)
{
	return OSDiffTick(GetCurrentTickCount(), xStartTicks);
}

float Asura_Timers::Platform_TicksToSeconds(Asura_TickCount xTicks)
{
	// Can't use OSTicksToSeconds as it is an integer operation.
	return static_cast<float>(xTicks) * g_fTickLength;
}

Asura_TickCount Asura_Timers::Platform_SecondsToTicks(float fSeconds)
{
	return OSSecondsToTicks(fSeconds);
}


/// End of file. ///
