//
// Asura_Platform_FSFX_Component.cpp (Wii)
//
// Created by SW 26/02/09
// 
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
// Implement FSFX system on the Wii
//

#include "Core/Asura.h"

#ifdef ASURA_USE_FSFX

#include "FSFX/Asura_FSFX_Component.h"
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"

#include "SpecialFX/Asura_GX_DOF.h"
#include "SpecialFX/Asura_GX_Glow.h"

///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_System platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_System::Platform_Initialise()
{
	Asura_GX_Glow::Initialise();
	Asura_GX_DOF::Initialise();
}

void Asura_FSFX_System::Platform_Reset()
{
}

void Asura_FSFX_System::Platform_Shutdown()
{
	Asura_GX_DOF::Shutdown();
	Asura_GX_Glow::Shutdown();
}

void Asura_FSFX_System::Platform_BeginRender(void)
{
}

void Asura_FSFX_System::Platform_EndRender(void)
{
}


///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Template platform specific
///////////////////////////////////////////////////////////////////////////////////////


int Asura_FSFX_Template::Platform_GetPlatformFlag()
{
	return Asura_FSFX_Component::FSFX_PLATFORM_WII;
}


///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Component_Test platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_Component_Test::Platform_Render( const float fIntensity )
{
}


///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Component_BrightnessAndContrast platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_Component_BrightnessAndContrast::Platform_Render( const float fIntensity )
{
	Asura_Assert( false, "BrightnessAndContrast FSFX not implemented");
}


///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Component_DesaturateAndTint platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_Component_DesaturateAndTint::Platform_Render( const float fIntensity )
{
	Asura_Assert( false, "DesaturateAndTint FSFX not implemented");
}


///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Component_Noise platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_Component_Noise::Platform_Render( const float fIntensity )
{
	Asura_Assert( false, "Component noise FSFX not implemented");
}


///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Component_GaussianBlur platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_Component_GaussianBlur::Platform_Render( const float fIntensity )
{
	Asura_Assert( false, "Gaussian blur FSFX not implemented");
}


///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Component_RadialBlur platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_Component_RadialBlur::Platform_Render( const float fIntensity )
{
	Asura_Assert( false, "Radial blur FSFX not implemented");
}


///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Component_DepthOfField platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_Component_DepthOfField::Platform_Render( const float fIntensity )
{
	float fStrength = m_fStrength * fIntensity * 0.01f; // Copied this from the PS2 version, so no idea where the magic number comes from
	Asura_GX_DOF::Render( m_fNearBlurZ, m_fFarBlurZ, m_fSize * 0.5f, fStrength );
}

///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Component_Bloom platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_Component_Bloom::Platform_Render( const float fIntensity )
{
	Asura_GX_Glow::Render();
}


///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Component_Gamma platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_Component_Gamma::Platform_Render( const float fIntensity )
{
	Asura_Assert( false, "Gamma FSFX not implemented");
}


///////////////////////////////////////////////////////////////////////////////////////
// Asura_FSFX_Component_RGBGamma platform specific
///////////////////////////////////////////////////////////////////////////////////////


void Asura_FSFX_Component_RGBGamma::Platform_Render( const float fIntensity )
{
	Asura_Assert( false, "RGBGamma FSFX not implemented");
}


#endif // ASURA_USE_FSFX
