//
// Asura_PlatformFileAccessUtils.cpp
//
// Created by KOR - 09 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "FileAccess/Asura_PlatformFileAccessUtils.h"

// asura
#include "Input/Asura_Rumble_System.h"
#include "Rendering/Asura_Render.h"
#include "Sound/Asura_Sound.h"

// Asura Wii
#include "HomeButton/Asura_Wii_HomeButtonMenu.h"
#include "Input/Asura_Wii_Input.h"
#include "Rendering/Asura_GX.h"
#include "Rendering/Asura_Wii_ROMFont.h"


#include <dolphin/pad.h>

bool Asura_PlatformFileAccessUtils::s_bAllowHomeButton = false;

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////


#ifdef ASURA_USE_BINK
bool Asura_PlatformFileAccessUtils::CheckDiscStatus(HBINK pxBinkHandle /* = 0 */)
#else
bool Asura_PlatformFileAccessUtils::CheckDiscStatus()
#endif
{
	ASURA_WII_DISC_ERROR eError = GetDiscError();
    if (eError != ASURA_WII_DISC_ERROR_NONE)
    {
		if (Asura_Rumble_System::IsInitialised())
		{
			Asura_Rumble_System::PauseAllRumbling();
			Asura_Rumble_System::Update();

		}
		// Pause all sounds
		Asura_Sound_System::PauseAllSystems();

#ifdef ASURA_USE_BINK
		if(pxBinkHandle)
		{
			BinkSetSoundOnOff(pxBinkHandle, 0);
		}
#endif
		const bool bHomeMenuInitialised = Asura_Wii_HomeButtonMenu::IsInitialised() && Asura_GX::ReadyToDraw();
		bool bHomeMenuWasAllowed = false;
		if(bHomeMenuInitialised && s_bAllowHomeButton)
		{
			bHomeMenuWasAllowed = Asura_Wii_HomeButtonMenu::IsAllowed();
			Asura_Wii_HomeButtonMenu::Allow(true);
		}

		Asura_Wii_ROMFont::SetSize(24, 0);

		// Stop any other threads rendering.
		Asura_GX::LockMutex();

		OSReport("Disc error detected.\n");
		bool bPrevClrScr = Asura_Render::GetEnableScreenClear();
		Asura_Render::SetEnableScreenClear(true);
		Asura_Render::BeginFrame();
		Asura_Render::EndFrame();
		while(eError != ASURA_WII_DISC_ERROR_NONE || DVDGetDriveStatus() == DVD_STATE_BUSY || Asura_Wii_HomeButtonMenu::IsActive())
        {
			if(bHomeMenuInitialised)
			{
				Asura_Input::Update();
			}
            Asura_Render::BeginFrame();
			const char* szErrorStr = PlatformProject_GetDiscErrorString(eError);
			Asura_Wii_ROMFont::RenderString(50, 175, 0, szErrorStr);
			if(bHomeMenuInitialised)
			{
				Asura_Wii_HomeButtonMenu::Render();
			}			
            Asura_Render::EndFrame();
			if(DVDGetDriveStatus() != DVD_STATE_BUSY)
			{
				eError = GetDiscError();
			}
        }
		// Clear the screen (both buffers)
		Asura_Render::BeginFrame();
		//DEMORFPuts(50, 175, 0, "Post-error frame 1");
		Asura_Render::EndFrame();
		Asura_Render::BeginFrame();
		//DEMORFPuts(50, 175, 0, "Post-error frame 2");
		Asura_Render::EndFrame();
		Asura_Render::SetEnableScreenClear(bPrevClrScr);

		Asura_GX::UnlockMutex();
		
		OSReport("Disc error resolved.\n");
		Asura_Sound_System::UnpauseAllSystems();
		Asura_Rumble_System::UnpauseAllRumbling();
		if(bHomeMenuInitialised  && s_bAllowHomeButton)
		{
			Asura_Wii_HomeButtonMenu::Allow(bHomeMenuWasAllowed);
		}

		return true;
    }
	return false;
}

ASURA_WII_DISC_ERROR Asura_PlatformFileAccessUtils::GetDiscError()
{
	switch(DVDGetDriveStatus())
	{
		case DVD_STATE_FATAL_ERROR:	
		{
			// wii_disc_error_fatal
			return ASURA_WII_DISC_ERROR_RETRY;
		}	
		case DVD_STATE_NO_DISK:
		{
			// wii_disc_error_no_disc
			return ASURA_WII_DISC_ERROR_NO_DISK;
		}
		case DVD_STATE_WRONG_DISK:
		{
			// wii_disc_error_wrong_disc
			return ASURA_WII_DISC_ERROR_WRONG_DISK;
		}
		case DVD_STATE_RETRY:
		{
			// wii_disc_error_cannot_be_read
			return ASURA_WII_DISC_ERROR_RETRY;
		}
		
		case DVD_STATE_END:
		case DVD_STATE_BUSY:
		case DVD_STATE_WAITING:
		case DVD_STATE_COVER_CLOSED:
		case DVD_STATE_COVER_OPEN:
		case DVD_STATE_MOTOR_STOPPED:
		case DVD_STATE_PAUSING:
		case DVD_STATE_IGNORED:
		case DVD_STATE_CANCELED:
		case DVD_STATE_NO_INPUT:
		{
			return ASURA_WII_DISC_ERROR_NONE;
		}
		default:
		{
			Asura_Assert(0, "Unexpected DVD state");
			return ASURA_WII_DISC_ERROR_NONE;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
