// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_Platform_GUIMenuInput.cpp												//
//																						//
//	Description: this handles the platform specific menu control system					//
//																						//
//	Created: Steve Keen - 16/11/06														//
//																						//
//	Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Core/Asura_Timers.h"

#include "Input/Asura_Wii_Input.h"
#include "Input/Asura_ControlMap.h"

#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"

#ifdef ASURA_USE_GUI_OSK
#include "GUIMenu/Asura_GUIMenu_OnScreenKeyboard.h"
#endif

//#include "GUIMenu/Asura_GUIMenu_Includes.h"

static void WII_SetupMenuControlMap( Asura_ControlMap* pxControlMap, u_int uButtonOffsetID);

#define ASURA_GUIMENU_MAX_CONTROLLERS	4

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_InitInputMappings( void)
{
	u_int u;

	if( s_pxMenuControlMaps == NULL)
	{
		s_pxMenuControlMaps = new Asura_ControlMap[ASURA_GUIMENU_MAX_CONTROLLERS];

		for( u=0; u<ASURA_GUIMENU_MAX_CONTROLLERS; u++)
		{
			WII_SetupMenuControlMap( &s_pxMenuControlMaps[u], u);
		}
	}
}

//void Asura_GUIMenu_System::Platform_InitInputMappings( void)
//{
//	s_xMenuControlMap.CreateMappings( ASURA_GUIMENU_CONTROL_MAX);
//
//#ifdef ASURA_USE_NEW_INPUT
//	s_xMenuControlMap.SetControllerIndex(0);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_UP, ASURA_WII_INPUT_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN, ASURA_WII_INPUT_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT, ASURA_WII_INPUT_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_WII_INPUT_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_WII_INPUT_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, ASURA_WII_INPUT_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, ASURA_WII_INPUT_2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, ASURA_WII_KEYID_PAD1_R3, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_WII_INPUT_B, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_WII_INPUT_UP, Asura_ControlMapping::ON_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_WII_INPUT_DOWN, Asura_ControlMapping::ON_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_WII_INPUT_LEFT, Asura_ControlMapping::ON_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_WII_INPUT_RIGHT, Asura_ControlMapping::ON_PRESS);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_WII_INPUT_A, Asura_ControlMapping::ON_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_WII_INPUT_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_WII_INPUT_2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_WII_INPUT_B, Asura_ControlMapping::ON_PRESS);
//
//#ifdef ASURA_USE_CONTROLMAP_MANAGER
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_WII_INPUT_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
//#endif 
//
//#else
//
//	// -- This is a fix for the dodgy mapping on my pad --
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_UP, ASURA_KEYID_WII_PAD1_BUTTON_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN, ASURA_KEYID_WII_PAD1_BUTTON_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT, ASURA_KEYID_WII_PAD1_BUTTON_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT, ASURA_KEYID_WII_PAD1_BUTTON_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_KEYID_WII_PAD1_BUTTON_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, ASURA_KEYID_WII_PAD1_BUTTON_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, ASURA_KEYID_WII_PAD1_BUTTON_2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, ASURA_WII_KEYID_PAD1_R3, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL, ASURA_KEYID_WII_PAD1_BUTTON_B, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_KEYID_WII_PAD1_BUTTON_UP, Asura_ControlMapping::ON_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_KEYID_WII_PAD1_BUTTON_DOWN, Asura_ControlMapping::ON_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_KEYID_WII_PAD1_BUTTON_LEFT, Asura_ControlMapping::ON_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_KEYID_WII_PAD1_BUTTON_RIGHT, Asura_ControlMapping::ON_PRESS);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_KEYID_WII_PAD1_BUTTON_A, Asura_ControlMapping::ON_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_KEYID_WII_PAD1_BUTTON_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_KEYID_WII_PAD1_BUTTON_2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
//
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_KEYID_WII_PAD1_BUTTON_B, Asura_ControlMapping::ON_PRESS);
//
//#ifdef ASURA_USE_CONTROLMAP_MANAGER
//	s_xMenuControlMap.AddKeyMapping( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_KEYID_WII_PAD1_BUTTON_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
//#endif 
//
//#endif
//}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_ShutdownInputMappings( void)
{
	u_int u;

	if( s_pxMenuControlMaps)
	{
		for( u=0; u<ASURA_GUIMENU_MAX_CONTROLLERS; u++)
		{
			s_pxMenuControlMaps[u].DeleteMappings();
		}

		delete []s_pxMenuControlMaps;
		s_pxMenuControlMaps = NULL;
	}
}

// --------------------------------------------------------------------------------------- 

u_int Asura_GUIMenu_System::Platform_GetNumControlMaps( void)
{
	return ASURA_GUIMENU_MAX_CONTROLLERS;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_System::Platform_ShouldUpdateControls()
{
	return true;
}

bool Asura_GUIMenu_System::Platform_ShouldRenderMenuPages()
{
	return true;
}

// --------------------------------------------------------------------------------------- 

bool Asura_GUIMenu_System::Platform_ProcessMenuBackControls( u_int uButtonsPressed, u_int uButtonsTrapped)
{
	bool bRet = false;
	u_int uButtonTest = uButtonsPressed;
	u_int uButtonMask = 0xffffffff - (ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT9 | ASURA_GUIMENU_ACTION_BUTTON_ALT_ACCEPT9_HELD);
		
	uButtonTest &= uButtonMask;
	uButtonsPressed = uButtonTest;

	// filters out any buttons presses not required, then we do an equality check
	uButtonsPressed &= (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD);

	if( uButtonsPressed == (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD) )
	{
		if( (uButtonsTrapped&ASURA_GUIMENU_ACTION_BUTTON_CANCEL) == 0)
		{
			bRet = true;
		}
	}

	return bRet;
}

// --------------------------------------------------------------------------------------- 

// ARC - 01 March 2007 12:56:27 - Added stub functions for Wii
static bool g_bDoingTextInput = false;
static Asura_UnicodeChar* g_wszTextInputBuffer;
static u_int g_uTextInputBufferMaxChars;
static u_int g_uTextInputCursorPos;
static u_int g_uTextInputFrameNumber;

void Asura_GUIMenu_System::Platform_BeginTextInput( const Asura_UnicodeChar* wszMessage, 
												   Asura_UnicodeChar* wszBuffer, 
												   u_int uBufferMaxChars )
{
#ifdef ASURA_USE_GUI_OSK
	Asura_GUIMenu_OnScreenKeyboard::Initialise( wszBuffer, uBufferMaxChars);
#else
	Asura_Assert(!g_bDoingTextInput, "already doing text input");
	g_bDoingTextInput = true;
	g_wszTextInputBuffer = wszBuffer;
	g_uTextInputBufferMaxChars = uBufferMaxChars;
	g_uTextInputCursorPos = uBufferMaxChars;// this indicates "clear on first input"
	g_uTextInputFrameNumber = Asura_Timers::GetFrameNumber();
#endif // ASURA_USE_GUI_OSK
}

void Asura_GUIMenu_System::Platform_UpdateTextInput()
{
#ifdef ASURA_USE_GUI_OSK
	Asura_GUIMenu_OnScreenKeyboard::Update();
#endif

	//Asura_UserInput::UpdateInputBuffer( g_wszTextInputBuffer, g_uTextInputBufferMaxChars, g_uTextInputCursorPos, NULL );
}

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
void Asura_GUIMenu_System::Platform_RenderSafetyBorder()
{
}
#endif // ASURA_DEBUG_VARIABLE_SYSTEM

void Asura_GUIMenu_System::Platform_BeginRender()
{
}

void Asura_GUIMenu_System::Platform_RenderTextInput()
{
	// just leave the menu to render it? what about cursors?
}

void Asura_GUIMenu_System::Platform_EndRender()
{
}

bool Asura_GUIMenu_System::Platform_IsTextInputFinished()
{
#ifdef ASURA_USE_GUI_OSK
	bool bRet = false;

	switch( Asura_GUIMenu_OnScreenKeyboard::GetExitType())
	{
	case ASURA_GUIMENU_OSK_DONE:
	case ASURA_GUIMENU_OSK_CANCEL:
		bRet = true;
		break;

	case ASURA_GUIMENU_OSK_NOEXIT:
	default:
		break;
	}

	return bRet;
#else

	//if (g_uTextInputFrameNumber == Asura_Timers::GetFrameNumber())
	//{
	//	return false;
	//}
	//return Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_ENTER );
	return true;
#endif // ASURA_USE_GUI_OSK
}

bool Asura_GUIMenu_System::Platform_IsTextInputCancelled()
{
#ifdef ASURA_USE_GUI_OSK
	return (Asura_GUIMenu_OnScreenKeyboard::GetExitType() == ASURA_GUIMENU_OSK_CANCEL);
#else
	return false;
#endif 
}

void Asura_GUIMenu_System::Platform_EndTextInput()
{
#ifdef ASURA_USE_GUI_OSK
	Asura_GUIMenu_OnScreenKeyboard::Shutdown();
#else
	Asura_Assert(g_bDoingTextInput, "not doing text input");
	g_bDoingTextInput = false;
	g_wszTextInputBuffer = NULL;
	g_uTextInputBufferMaxChars = 0;
#endif // ASURA_USE_GUI_OSK
}

// --------------------------------------------------------------------------------------- 

void WII_SetupMenuControlMap( Asura_ControlMap* pxControlMap, u_int uControllerIndex)
{
	if( pxControlMap)
	{
		pxControlMap->CreateMappings( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_MAX);

#ifdef ASURA_USE_NEW_INPUT
		pxControlMap->SetControllerIndex( uControllerIndex);

#ifdef ASURA_USE_CURSOR
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_CLICK, ASURA_WII_INPUT_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_CLICK_HELD, ASURA_WII_INPUT_A, Asura_ControlMapping::ON_PRESS);
#else // #ifdef ASURA_USE_CURSOR
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_UP, ASURA_WII_INPUT_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DOWN, ASURA_WII_INPUT_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_LEFT, ASURA_WII_INPUT_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_RIGHT, ASURA_WII_INPUT_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_UP, ASURA_WII_INPUT_NUNCHUCK_STICK_U, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DOWN, ASURA_WII_INPUT_NUNCHUCK_STICK_D, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_LEFT, ASURA_WII_INPUT_NUNCHUCK_STICK_L, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_RIGHT, ASURA_WII_INPUT_NUNCHUCK_STICK_R, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_WII_INPUT_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_WII_INPUT_UP, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_WII_INPUT_DOWN, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_WII_INPUT_LEFT, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_WII_INPUT_RIGHT, Asura_ControlMapping::ON_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_WII_INPUT_NUNCHUCK_STICK_U, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_WII_INPUT_NUNCHUCK_STICK_D, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_WII_INPUT_NUNCHUCK_STICK_L, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_WII_INPUT_NUNCHUCK_STICK_R, Asura_ControlMapping::ON_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_WII_INPUT_A, Asura_ControlMapping::ON_PRESS);
#endif // #ifdef ASURA_USE_CURSOR

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, ASURA_WII_INPUT_2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, ASURA_WII_INPUT_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		// KOR - we need any key to skip the home menu. Therefore use one of the alt accepts.
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_B, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_C, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_Z, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_PLUS, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_MINUS, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT9, ASURA_WII_INPUT_2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_CANCEL, ASURA_WII_INPUT_B, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_WII_INPUT_2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_WII_INPUT_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_WII_INPUT_B, Asura_ControlMapping::ON_PRESS);

#ifdef ASURA_USE_CONTROLMAP_MANAGER
//		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_WII_INPUT_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
		pxControlMap->AddHoldKeyMapping(Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_WII_INPUT_1, 1.0f );
#endif 

#else

		// -- This is a fix for the dodgy mapping on my pad --
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_UP, ASURA_KEYID_WII_PAD1_BUTTON_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DOWN, ASURA_KEYID_WII_PAD1_BUTTON_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_LEFT, ASURA_KEYID_WII_PAD1_BUTTON_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_RIGHT, ASURA_KEYID_WII_PAD1_BUTTON_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ACCEPT, ASURA_KEYID_WII_PAD1_BUTTON_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT1, ASURA_KEYID_WII_PAD1_BUTTON_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT2, ASURA_KEYID_WII_PAD1_BUTTON_2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		//	pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT3, ASURA_WII_KEYID_PAD1_R3, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_CANCEL, ASURA_KEYID_WII_PAD1_BUTTON_B, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_UP_HELD, ASURA_KEYID_WII_PAD1_BUTTON_UP, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DOWN_HELD, ASURA_KEYID_WII_PAD1_BUTTON_DOWN, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_LEFT_HELD, ASURA_KEYID_WII_PAD1_BUTTON_LEFT, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_RIGHT_HELD, ASURA_KEYID_WII_PAD1_BUTTON_RIGHT, Asura_ControlMapping::ON_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ACCEPT_HELD, ASURA_KEYID_WII_PAD1_BUTTON_A, Asura_ControlMapping::ON_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD, ASURA_KEYID_WII_PAD1_BUTTON_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD, ASURA_KEYID_WII_PAD1_BUTTON_2, Asura_ControlMapping::ON_DEBOUNCED_PRESS);

		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_CANCEL_HELD, ASURA_KEYID_WII_PAD1_BUTTON_B, Asura_ControlMapping::ON_PRESS);

#ifdef ASURA_USE_CONTROLMAP_MANAGER
		pxControlMap->AddKeyMapping( Asura_GUIMenu_System::ASURA_GUIMENU_CONTROL_DVS_ACTIVATE, ASURA_KEYID_WII_PAD1_BUTTON_1, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
#endif 

#endif
	}
}

#endif // ASURA_USE_GUI_MENU_SYSTEM
