//
// Asura_Wii_HomeButtonMenu.h
//
// Created by OF - 21 March 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_WII_HOMEBUTTONMENU_H_
#define __ASURA_WII_HOMEBUTTONMENU_H_

#include <revolution/gx.h>
#include <revolution/kpad.h>
#include <revolution/mem/allocator.h>
#include <revolution/sc.h>
#include <revolution/mtx.h>
#include <revolution/vi.h>
#include <revolution/tpl.h>

#include <revolution/hbm/HBMBase.h>

class Asura_Wii_HomeButtonMenu
{
public:
	static void	Initialise();
	static void	InitialiseSound();
	static void	Shutdown();

	// If it is not allowed, the ban icon will be shown when the button is pressed
	static bool	IsAllowed();
	static void	Allow( bool bAllow );
	// If it is not enabled, nothing at will happen when the button is pressed.
	static void SetEnabled(bool bEnabled);

	static bool	IsActive();

	static void	Activate();
	static void Deactivate();

	static void	ProcessController();
	static void	Render();

	static bool IsInitialised() { return s_bInitialised; }
private:
	static void ActivateBanIcon();

	static void	InitControllerData( HBMControllerData* g_xHbmControllerData );

	static void	InitHomeButtonInfo( HBMDataInfo* pxHbmInfo );
	static void	ShutDownHomeButtonInfo( HBMDataInfo* pxHbmInfo );

	static void	InitSound();
	static void	ShutDownSound();

	static void	InitialiseDisallowedIcon( TPLPalette** pxIcon );
	static void	ShutDownDisallowedIcon( TPLPalette* xIcon );

	static u_char*	OpenAllocAndReadFileHelper( const char* pszFilename, u_int* puFileSize );

	static void	RenderMenu();
	static void	DrawBanIcon( u_char alpha );
	static void	PreShutdown();

	static bool ConsoleCmd_Allow(bool bAllow, bool bHelp, bool bInterrogate);
	static bool ConsoleCmd_Enable(bool bEnabled, bool bHelp, bool bInterrogate);

	static void ClearScreen();

	enum
	{
		eCS_HBM_INACTIVE		= 0x0000,
		eCS_HBM_FADING_IN		= 0x0001,
		eCS_HBM_FADING_OUT		= 0x0002,
		eCS_HBM_ACTIVE			= 0x0003,

		eCS_HBM_ALLOWEDMASK		= 0x7FFF,

		eCS_HBM_DISALLOWED		= 0x8000,
	};

	enum
	{
		eICON_OFF,	
		eICON_FADING_IN,
		eICON_PAUSED,
		eICON_FADING_OUT,
	};

	static HBMControllerData	m_xHbmControllerData;
	static HBMDataInfo			m_xHbmDataInfo;

	static u_int				m_uState;

	static void*				m_pSoundFile;
	static void*				m_pSoundMemory;

	static OSTick				m_xBanIconTime;
	static u_int				m_uBanIconMode;

	static TPLPalette*			m_xIconTpl;
	static bool					s_bInitialised;
	static bool					s_bEnabled;
};
#endif
