//
// Asura_GX_DisplayList.h
//
// Created by AS - 12 July 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_GX_DISPLAYLIST_H__
#define __ASURA_GX_DISPLAYLIST_H__

#include "Core/Memory/Asura_Mem.h"
#include "Debugging/Asura_Assert.h"

#include <revolution/gx.h>

class Asura_Chunk_Stream;
class Asura_GX_DisplayListBlock;

class Asura_GX_DisplayList
{
	unsigned int m_puSize;
	void *m_pvMem;

	friend class Asura_GX_DisplayListBlock;
	void Initialise(void *pMem, unsigned int uSize)
	{
		m_puSize = (uSize);
		m_pvMem = (pMem);
	}
public:
	static u_int CalculateSize(unsigned int uNumVerts, unsigned int uVertSize)
	{
		return 3 + uNumVerts * uVertSize;
	}

	class Iterator;
	friend class Iterator;

	Asura_GX_DisplayList(void *pMem, unsigned int uSize) : m_puSize(uSize), m_pvMem(pMem)
	{

	}

	Asura_GX_DisplayList() : m_puSize(0), m_pvMem(NULL)
	{

	}
	~Asura_GX_DisplayList()
	{
	}
	void operator=(const Asura_GX_DisplayList&other)
	{
		Initialise(other.m_pvMem, other.m_puSize);
	}
	Asura_GX_DisplayList(const Asura_GX_DisplayList&other)
	{
		Initialise(other.m_pvMem, other.m_puSize);
	}

	Iterator GetIterator();

	void Render() const 
	{
		if (m_pvMem)
		{
			Asura_Assert((m_puSize & 31) == 0, "Display List size not aligned?");
			GXCallDisplayList(m_pvMem, m_puSize);
		}
	}

	u_short GetNumberVerts() const 
	{
		// nasty horrible hack but only needed to update tris rendered debug info
		return *(u_short*)(((char*)m_pvMem)+1);
	}


	unsigned int GetSize() const 
	{
		return m_puSize;
	}

	bool IsValid() const 
	{
		return m_pvMem != NULL;
	}
};


class Asura_GX_DisplayList::Iterator
{
	u_char *m_pu8DisplayListMem;

	friend class Asura_GX_DisplayList;
	Iterator(void *pvDisplayListMem) : m_pu8DisplayListMem((u_char *)pvDisplayListMem)
	{

	}

public:

	template <typename T> void AddToken(T token)
	{
		*(T*)m_pu8DisplayListMem = token;
		m_pu8DisplayListMem += sizeof(T);
	}
	void ReadFromFile(Asura_Handle fileHandle, u_int uSize);
	void ReadFromStream(Asura_Chunk_Stream& xStream, u_int uSize);
	void Finalize(Asura_GX_DisplayList &xDisplayList, bool bPad);
};	

inline Asura_GX_DisplayList::Iterator Asura_GX_DisplayList::GetIterator()
{
	return Asura_GX_DisplayList::Iterator(m_pvMem);
}


class Asura_GX_DisplayListBlock
{
	Asura_GX_DisplayList m_xDisplayList;

	Asura_GX_DisplayListBlock(Asura_GX_DisplayListBlock&)
	{

	}
	void operator=(Asura_GX_DisplayListBlock&)
	{

	}
	void* GetMem() 
	{
		return m_xDisplayList.m_pvMem;
	}
public:
	Asura_GX_DisplayListBlock()
	{

	}
	~Asura_GX_DisplayListBlock()
	{
		Release();
	}
	class Iterator;
	friend class Iterator;
	Iterator GetIterator();


	void Initialise(unsigned int uSize, bool bPad=true);
	void Release();

	const Asura_GX_DisplayList &DisplayList() const {return m_xDisplayList;}
	Asura_GX_DisplayList &DisplayList() {return m_xDisplayList;}

	unsigned int GetSize() const 
	{
		return m_xDisplayList.GetSize();
	}

};

class Asura_GX_DisplayListBlock::Iterator
{
	u_char *m_pu8DisplayListMem;

	friend class Asura_GX_DisplayListBlock;
	Iterator(void *pvDisplayListMem) : m_pu8DisplayListMem((u_char *)pvDisplayListMem)
	{

	}

public:
	void Finalize(Asura_GX_DisplayListBlock &xDisplayList);
	Asura_GX_DisplayList GetDisplayList(unsigned int uSize, bool bPad = true)
	{
		u_int uAlignedSize = uSize + (bPad ? 32 + (uSize &31) : 0);
		Asura_GX_DisplayList xDisplayList(m_pu8DisplayListMem, uAlignedSize);
		m_pu8DisplayListMem += uAlignedSize;
		return xDisplayList;
	}

};

inline Asura_GX_DisplayListBlock::Iterator Asura_GX_DisplayListBlock::GetIterator()
{
	return Asura_GX_DisplayListBlock::Iterator(m_xDisplayList.m_pvMem);
}


#endif//__ASURA_GX_DISPLAYLIST_H__