//
// Asura_GX_Environment.cpp
//
// Created by KOR - 10 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_GX_Environment.h"

// Asura headers
#include "Animation/Asura_TextureAnimation.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_TextureManagement.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Light/Asura_Photon.h"
#include "Mesh/Asura_CollisionMesh.h"
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Water.h"
#include "SpecialFX/Asura_Weather_System.h"

// Platform headers
#include "Core/Asura_Wii_MEM2.h"
#include "Rendering/Asura_GX.h"
#include "SpecialFX/Asura_GX_Fog.h"

// Library headers
#include <revolution/gx.h>
#include <revolution/base/PPCArch.h>

// Defines
#define ASURA_CHUNK_ENV_CURRENT_VERSION (1)
#define ASURA_GX_MAX_ENV_LIGHTS (8)

unsigned int			Asura_GX_Environment::s_uNumberOfMeshes = 0;
Asura_Vector_3*			Asura_GX_Environment::s_pxAllPositions = NULL;
short*					Asura_GX_Environment::s_psAllTexCoords = NULL;
Asura_GX_Env_Mesh*		Asura_GX_Environment::s_pxMeshStorage = NULL;
bool					Asura_GX_Environment::s_bInitialised = false;
unsigned int			Asura_GX_Environment::s_uNormalTableEntries = 0;
Asura_Vector_3*			Asura_GX_Environment::s_pxNormalTable = NULL;
int						Asura_GX_Environment::s_iAttenuationMap = ASURA_TEXTURE_NONE;
Asura_Vector_3			Asura_GX_Environment::s_xMeshPos(0.f, 0.f, 0.f);
unsigned int			Asura_GX_Environment::s_uExportFlags = 0;
void*					Asura_GX_Environment::s_pvDisplayListMem = NULL;
u_int					Asura_GX_Environment::s_uDisplayListMaxSize = 0;

template <typename T> void InitialiseAndReadArray(Asura_Chunk_Stream& xStream, T **pDest, T * pSource, u_int *puCurrentPosition, u_int uNumEntries, u_int uItemSize = sizeof(T))
{
	if (uNumEntries)
	{
		*pDest = pSource + (puCurrentPosition ? *puCurrentPosition : 0);
		xStream.Read(uItemSize * uNumEntries, *pDest);
		if (puCurrentPosition)
		{
			*puCurrentPosition += uNumEntries;
		}
	}
	else
	{
		pDest = NULL;
	}
}

void Asura_GX_Environment::ReadNewEnvironmentFormat(Asura_Handle iFileHandle)
{
	Asura_Chunk_File_Stream xStream(iFileHandle);
	// 		u_int uNumModules;
	xStream >> s_uNumberOfMeshes;
	s_pxMeshStorage = new Asura_GX_Env_Mesh[s_uNumberOfMeshes];

	// 		u_int uFlags;
	xStream >> s_uExportFlags;

	Asura_Assert((s_uExportFlags & EXPORT_FLAG_USE_DISPLAY_LISTS) != 0, "Loading of pre-display list environment versions not supported");
	// Set up the environment vertex description according to the bytes per colour for this environment
	GXSetVtxAttrFmt(GX_VTXFMT0, GX_VA_CLR0, GX_CLR_RGBA, (s_uExportFlags & EXPORT_FLAG_VERTEXCOLOURRGBA6) ? GX_RGBA6 : GX_RGB8, 0);

	// 		u_int uNumPositions; //= sum(Modules.uNumPositions);
	u_int uTotalPositions;
	xStream >> uTotalPositions;
	s_pxAllPositions = (Asura_Vector_3*)ASURA_WII_MEM2_ALLOC(uTotalPositions * sizeof(Asura_Vector_3));

	// 		u_int uNumVerticies; //= sum(Modules.uNumVerticies);
	u_int uTotalVertices;
	xStream >> uTotalVertices;

	u_int uTotalTexcoords;
	xStream >> uTotalTexcoords;

	// 		u_int uNumNormals;  // if (uFlags & EXPORT_FLAG_NORMALS)
	if(s_uExportFlags & EXPORT_FLAG_NORMALS)
	{
		xStream >> s_uNormalTableEntries;
	}

	// 		u_int uNumCompressedIndicies;  // if (!(uFlags & EXPORT_FLAG_USE_DISPLAY_LISTS))
	// 		u_int uTotalDisplayListSize;  // if (uFlags & EXPORT_FLAG_USE_DISPLAY_LISTS)
	xStream >> s_uDisplayListMaxSize;
	s_pvDisplayListMem = ASURA_WII_MEM2_ALLOC( s_uDisplayListMaxSize);
	s_psAllTexCoords = (short*)ASURA_WII_MEM2_ALLOC(uTotalTexcoords * 2 * sizeof(short));

	if(s_uExportFlags & EXPORT_FLAG_NORMALS)
	{
		s_pxNormalTable = (Asura_Vector_3*)ASURA_WII_MEM2_ALLOC(s_uNormalTableEntries * sizeof(Asura_Vector_3));
		// 		float fNormals[uNumNormals * 3]; // if (uFlags & EXPORT_FLAG_NORMALS)
		xStream.Read(s_uNormalTableEntries * sizeof(Asura_Vector_3), s_pxNormalTable);
	}


	unsigned int uCurrentPosition = 0;
	unsigned int uCurrentTexcoord = 0;
	unsigned int uCurrentCmpIndex = 0;
	unsigned int uDisplayListMemory = 0;
	for(u_int uModule = 0; uModule < s_uNumberOfMeshes; ++uModule)
	{
		Asura_GX_Env_Mesh* pxMesh = s_pxMeshStorage + uModule;

		// 			u_int uNumPositions;
		xStream >> pxMesh->m_uNumberOfPositions;
		// 			u_int uNumVerticies;
		xStream >> pxMesh->m_uNumberOfTexcoords;
		// 			u_int uNumStrips;
		xStream >> pxMesh->m_uNumberOfStrips;
		// 			Asura_Bounding_Box xBB;
		xStream >> pxMesh->m_xBB;

		// 			float fPositions[uNumPositions * 3];
		InitialiseAndReadArray(xStream, &pxMesh->m_pxPositions, s_pxAllPositions, &uCurrentPosition, pxMesh->m_uNumberOfPositions, sizeof(Asura_Vector_3));

		// 			short sUV[uNumVerticies*2]; // 6.10 format
		InitialiseAndReadArray(xStream, &pxMesh->m_psTexCoords, s_psAllTexCoords + uCurrentTexcoord * 2 , NULL, pxMesh->m_uNumberOfTexcoords, 2 * sizeof(short));

		uCurrentTexcoord += pxMesh->m_uNumberOfTexcoords;

		pxMesh->m_pxStrips = new Asura_GX_Env_Tristrip[pxMesh->m_uNumberOfStrips];
		for(u_int uStrip = 0; uStrip < pxMesh->m_uNumberOfStrips; ++uStrip)
		{
			// 				u_int uStripFlags;
			// 				u_int uMatID;
			// 				u_int uNumIndicies;
			// 				u_int uNumCompressedIndicies; // if (!(uFlags & EXPORT_FLAG_USE_DISPLAY_LISTS))
			// 				u_short usCompressedIndicies[uNumCompressedIndicies]; // if (!(uFlags & EXPORT_FLAG_USE_DISPLAY_LISTS))
			// 				u_int uDisplayListSize; // if (uFlags & EXPORT_FLAG_USE_DISPLAY_LISTS)
			// 			    char cDisplayList[uDisplayListSize] // if (uFlags & EXPORT_FLAG_USE_DISPLAY_LISTS)
			Asura_GX_Env_Tristrip* pxStrip = pxMesh->m_pxStrips + uStrip;

			xStream >> pxStrip->m_uStripFlags;
			xStream >> pxStrip->m_iMaterialID;
			pxStrip->m_iMaterialID = Asura_Material_System::ConvertIndex(pxStrip->m_iMaterialID);
			xStream >> pxStrip->m_uNumberOfIndices;
			xStream >> pxStrip->m_uDisplayListSize;
			InitialiseAndReadArray(xStream , &pxStrip->m_pucDisplayList, (unsigned char*)s_pvDisplayListMem, &uDisplayListMemory, pxStrip->m_uDisplayListSize, 1);
			pxMesh->m_bMeshHasTransparentStrips |= ( Asura_Material_System::GetMaterialFlags( pxStrip->m_iMaterialID ) & AsuraMaterialFlag_Transparent ) ? 1 : 0;
		}
	}
	Asura_Assertf(uCurrentTexcoord == uTotalTexcoords, ("Texcoord size mismatch: expected %u, got %u", uTotalTexcoords, uCurrentTexcoord));
	Asura_Assertf(uDisplayListMemory == s_uDisplayListMaxSize, ("Display list memory mismatch: %d != %d", uDisplayListMemory, s_uDisplayListMaxSize));
}

bool Asura_GX_Environment::Initialise(Asura_Handle xFileHandle)
{
	Asura_Assert(!s_bInitialised, "Environment store already initialised.");
	if(s_bInitialised)
	{
		Shutdown();
	}

	s_iAttenuationMap = Asura_Texture_Management::GetTextureIndex("\\sphereshade.tpl");

	u_int uVersionNumber;
	Asura_FileAccess::ReadData(xFileHandle, &uVersionNumber, sizeof(int));
	Asura_Assert(uVersionNumber == ASURA_CHUNK_ENV_CURRENT_VERSION, "Unsupported version number.");

	ReadNewEnvironmentFormat(xFileHandle);
	s_bInitialised = true;

	return true;
}

void Asura_GX_Environment::Update()
{
	// Probably won't need updating 
}

void Asura_GX_Environment::Reset()
{
	if(s_bInitialised)
	{
		Shutdown();
	}
}

void Asura_GX_Environment::Shutdown()
{
	Asura_Assert(s_bInitialised, "Env. store not currently initialised so cannot shut down");
	if(!s_bInitialised)
	{
		return;
	}
	ASURA_WII_MEM2_FREE(s_pxNormalTable);
	ASURA_WII_MEM2_FREE(s_pxAllPositions);
	ASURA_WII_MEM2_FREE(s_psAllTexCoords);
	ASURA_WII_MEM2_FREE(s_pvDisplayListMem);
	delete [] s_pxMeshStorage;

	s_uNumberOfMeshes = 0;
	s_bInitialised = false;
}

void Asura_GX_Environment::RenderSetup(bool bInverted /* = false */)
{
	const GXColor xBlack = {0x00, 0x00, 0x00, 0x00};
	const GXColor xWhite = {0xff, 0xff, 0xff, 0xff};
	const GXColor xGrey  = {0x40, 0x40, 0x40, 0x00};

	// Set up materials
	GXSetChanCtrl(GX_COLOR0,FALSE, GX_SRC_REG, GX_SRC_VTX, 0, GX_DF_NONE, GX_AF_NONE);
	GXSetChanCtrl(GX_ALPHA0,FALSE, GX_SRC_REG, GX_SRC_VTX, 0, GX_DF_NONE, GX_AF_NONE);
	GXSetChanAmbColor(GX_COLOR0A0, xBlack);
	GXSetChanMatColor(GX_COLOR0A0, xWhite);
	if(bInverted)
	{
		GXSetChanCtrl(GX_COLOR1,FALSE, GX_SRC_REG, GX_SRC_REG, 0, GX_DF_NONE, GX_AF_NONE);
		GXSetChanCtrl(GX_ALPHA1,FALSE, GX_SRC_REG, GX_SRC_REG, 0, GX_DF_NONE, GX_AF_NONE);
		GXSetChanAmbColor(GX_COLOR1A1, xBlack);
		GXSetChanMatColor(GX_COLOR1A1, xBlack);
		GXSetNumChans(1);
	}
	else
	{
		GXSetChanCtrl(GX_COLOR1, TRUE, GX_SRC_REG, GX_SRC_REG, GX_LIGHT1 | GX_LIGHT2 | GX_LIGHT3, GX_DF_SIGN, GX_AF_NONE);
		GXSetChanCtrl(GX_ALPHA1,FALSE, GX_SRC_REG, GX_SRC_REG, 0, GX_DF_NONE, GX_AF_NONE);
		// set up ambient color
		GXSetChanAmbColor(GX_COLOR1A1, xGrey);
		GXSetChanMatColor(GX_COLOR1A1, xWhite);
		GXSetNumChans(2);
	}

	Asura_Render::CurrentStates.RequestCullMode(bInverted ? ASURA_CULLMODE_CW : ASURA_CULLMODE_CCW);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
	GXClearVtxDesc();
	GXSetVtxDesc(GX_VA_POS, GX_INDEX16);
	if (s_uExportFlags & EXPORT_FLAG_NORMALS)
	{
		GXSetVtxDesc(GX_VA_NRM, GX_INDEX8);
	}
	GXSetVtxDesc(GX_VA_CLR0, GX_DIRECT);
	GXSetVtxDesc(GX_VA_TEX0, GX_INDEX16);

	Asura_GX::SetDisableTranslucency(true);
}

DEBUGVAR int dbg_iModuleToRender = -1;
void Asura_GX_Environment::Render(bool bInverted /* = false */)
{
	RenderSetup(bInverted);
	const u_int uNumCameras = Asura_Camera::GetNumberOfCameras();
	if(bInverted)
	{
		const Asura_RGBA& xFogColour = Asura_Fog::GetFogColour();
		GXColor xGXFogColour;
		Asura_GX::ARGBToGXColor(xFogColour.ToUIntARGB(true), xGXFogColour);
		const float fFogNear = Asura_Fog::GetFogNearPlane();
		const float fFogFar = Asura_Fog::GetFogFarPlane();
		float fModifiedNear = fFogNear;
		float fModifiedFar = fFogFar;
		if(Asura_Fog::IsGraphFogEnabled())
		{
			const Asura_Vector_2& xFogMidPoint = Asura_Fog::GetGraphFogPoint();
			if(xFogMidPoint.y == 0.f)
			{
				// Push near plane out
				fModifiedNear = fFogNear + (fFogFar - fFogNear) * xFogMidPoint.x;
			}
		}
		const float fValueAtFarPlane = Asura_Fog::GetFogValueAtFarPlane();
		if(0.f < fValueAtFarPlane)
		{
			// Push far plane out
			fModifiedFar /= fValueAtFarPlane;
		}
		for(int i = 0; i < uNumCameras; ++i)
		{
			if (Asura_Camera::IsCameraActive(i))
			{
				Asura_Camera::SetCameraIndex(i);
				Asura_Render::ApplyCameraSettings();

				const float fWaterHeight = Asura_Water_System::GetVisibleWaterYPosition();
				Mtx xNewPosMtx, xInvert;
				Vec xPos = { 0.f, fWaterHeight, 0.f };
				Vec xNrm = { 0.f, 1.f, 0.f };
				MTXReflect(xInvert, &xPos, &xNrm);
				const Mtx& xViewMtx = Asura_GX::GetViewMatrix();
				MTXConcat(xViewMtx, xInvert, xNewPosMtx);
				GXLoadPosMtxImm(xNewPosMtx, GX_PNMTX0);

				const u_int uNumModules = Asura_Module_System::GetNumberOfModules();
				for (u_int uModule = 0; uModule < uNumModules; ++uModule)
				{
					if(dbg_iModuleToRender != -1 && dbg_iModuleToRender != uModule)
					{
						continue;
					}
					if(!Asura_Module_System::IsModuleLinkedInForCurrentCamera(uModule))
					{
						continue;
					}
					Asura_Bounding_Box xAABB = Asura_Module_System::GetBoundingBoxOfModule( uModule );
					xAABB.Displace( Asura_Module_System::GetPositionOfModule( uModule ) );
					// Make sure it isn't entirely underwater.
					if(xAABB.MinY > fWaterHeight)
					{
						continue;
					}
					// Reflect BB in water plane
					const float fMinY = xAABB.MinY;
					xAABB.MinY = 2.f * fWaterHeight - xAABB.MaxY;
					xAABB.MaxY = 2.f * fWaterHeight - fMinY;
					if(Asura_Camera::IsAABBInFrustum(xAABB))
					{
						const Asura_Vector_3& xModuleCentre = Asura_Module_System::GetCentreOfModule(uModule);
						const float fDist = (xModuleCentre - *Asura_Camera::GetPosition()).Magnitude();
						if(fDist > fFogFar)
						{
							// Prevent fog becoming too opaque at distance
							Asura_GX_Fog::SetHardwareFogFixed(fValueAtFarPlane, fDist, xGXFogColour);
						}
						else
						{
							Asura_GX_Fog::SetHardwareFogLinear(fModifiedNear, fModifiedFar, xGXFogColour);
						}
						s_pxMeshStorage[uModule].RenderSimple();
					}
				}
			}
		}
	}
	else
	{
		for(int i = 0; i < uNumCameras; ++i)
		{
			if (Asura_Camera::IsCameraActive(i))
			{
				Asura_Camera::SetCameraIndex(i);
				Asura_Render::ApplyCameraSettings();

				const u_int uNumModules = Asura_Module_System::GetNumberOfModules();
				for (u_int uModule = 0; uModule < uNumModules; ++uModule)
				{
					if(dbg_iModuleToRender != -1 && dbg_iModuleToRender != uModule)
					{
						continue;
					}
					if(Asura_Module_System::IsModuleVisible(uModule))
					{
						s_pxMeshStorage[uModule].Render();
					}
				}
			}
		}
	}
	Asura_GX::SetDisableTranslucency(false);
}

Asura_GX_Env_Mesh::Asura_GX_Env_Mesh()
: m_uNumberOfPositions(0)
, m_uNumberOfTexcoords(0)
, m_uNumberOfStrips(0)
, m_pxPositions(NULL)
, m_psTexCoords(NULL)
, m_pxStrips(NULL)
, m_ucFlags(0)
{
	m_xBB.Reset();
}

Asura_GX_Env_Mesh::~Asura_GX_Env_Mesh()
{
	delete [] m_pxStrips;
}

DEBUGVAR float dbg_fLightShininess = 16.f;
DEBUGVAR bool dbg_bNoLights = false;
DEBUGVAR bool dbg_bSpecularDirFromSunPos = true;
DEBUGVAR Asura_Vector_3 dbg_xSpecularDir(0.f, 1.f, 0.f);
DEBUGVAR Asura_Vector_3 dbg_xLightVector(0.f, 0.f, -1.f);
DEBUGVAR bool dbg_bNormalAttenuation = true;

const GXLightID g_aeLightEnums[3] = { GX_LIGHT1, GX_LIGHT2, GX_LIGHT3 };
const GXTevKColorID g_aeKColours[3] = { GX_KCOLOR1, GX_KCOLOR2, GX_KCOLOR3 };
const GXTevSwapSel g_aeSwapIDs[3] = { GX_TEV_SWAP1, GX_TEV_SWAP2, GX_TEV_SWAP3 };
const GXTevKColorSel g_aeKColorIDs[3] = { GX_TEV_KCSEL_K1, GX_TEV_KCSEL_K2, GX_TEV_KCSEL_K3 };
const GXTexMtx g_aaeTexMtxIDs[3][2] =
{
	{ GX_TEXMTX1, GX_TEXMTX2 },
	{ GX_TEXMTX3, GX_TEXMTX4 },
	{ GX_TEXMTX5, GX_TEXMTX6 },
};
const GXTexCoordID g_aaeTexCoordIDs[3][2] = 
{
	{ GX_TEXCOORD0, GX_TEXCOORD1 },
	{ GX_TEXCOORD2, GX_TEXCOORD3 },
	{ GX_TEXCOORD4, GX_TEXCOORD5 },
};

DEBUGVAR bool dbg_bTest = false;
void Asura_GX_Env_Mesh::SetupLighting() const
{
	const GXColor xBlack = {0x00, 0x00, 0x00, 0x00};
	const GXColor xWhite = {0xff, 0xff, 0xff, 0xff};
	const GXColor xGrey  = {0x40, 0x40, 0x40, 0x40};

	const Asura_Vector_3 xBBCentre = m_xBB.CalculateCentre();

	Asura_Vector_3 xSunDir = dbg_xSpecularDir;
	if(dbg_bSpecularDirFromSunPos)
	{
		const Asura_Vector_3& xSunPos = Asura_Weather_System::GetSunPos();
		xSunDir = xBBCentre - xSunPos;
		xSunDir.Normalise();
	}

	// Set up a specular light just in case we need it.
	GXLightObj xSpecLight;
	GXInitLightColor(&xSpecLight, xWhite);
	Asura_Vector_3 xViewspaceSunDir;
	const Mtx& xViewMatrix = Asura_GX::GetViewMatrix();
	MTXMultVecSR(xViewMatrix, (VecPtr)&xSunDir, (VecPtr)&xViewspaceSunDir);
	Asura_Vector_3 xHalfAngle = (xViewspaceSunDir + Asura_Vector_3(0.f, 0.f, 1.f)) * 0.5f;
	xHalfAngle.Normalise();
	// This is a nasty hack, becase for some reason Wii hardware clamps off the specular highlight based on N.L
	// Therefore, set the light direction to be the opposite of the half angle.
	const Asura_Vector_3 xMinusHalfAngle = -xHalfAngle;
	GXInitSpecularDirHAv(&xSpecLight, (VecPtr)&xMinusHalfAngle, (VecPtr)&xHalfAngle);
	GXInitLightShininess(&xSpecLight, dbg_fLightShininess);

	GXLoadLightObjImm(&xSpecLight, GX_LIGHT0);

	Asura_Photon_System::MakePhotonListForBoundingBox(m_xBB, xBBCentre,false,true,3);

	if(dbg_bNoLights)
	{
		Asura_Photon_System::NumberOfLocalPhotons = 0;
	}

	if(!Asura_Photon_System::NumberOfLocalPhotons)
	{
		GXSetChanCtrl(GX_COLOR1,FALSE, GX_SRC_REG, GX_SRC_REG, 0, GX_DF_SIGN, GX_AF_NONE);
		GXSetNumChans(1);
		return;
	}
	GXSetNumChans(2);

	GXLightObj axLights[3];
	GXSetTevSwapModeTable(GX_TEV_SWAP1, GX_CH_RED, GX_CH_RED, GX_CH_RED, GX_CH_RED);
	GXSetTevSwapModeTable(GX_TEV_SWAP2, GX_CH_GREEN, GX_CH_GREEN, GX_CH_GREEN, GX_CH_GREEN);
	GXSetTevSwapModeTable(GX_TEV_SWAP3, GX_CH_BLUE, GX_CH_BLUE, GX_CH_BLUE, GX_CH_BLUE);
		
	GXColor axRGB[3] =
	{ {0x80, 0x00, 0x00, 0x00}, 
	  {0x00, 0x80, 0x00, 0x00}, 
	  {0x00, 0x00, 0x80, 0x00}, };

	u_int u;
	u_int uLightFlags = 0;
	// Fake per-pixel lighting, up to three lights
	for (u = 0; u < Asura_Photon_System::NumberOfLocalPhotons; u++)
	{
		const Asura_Local_Photon& xPhoton = Asura_Photon_System::LocalPhotons[u];
		const Asura_Vector_3& xLightPos = xPhoton.GetPosition();
		GXColor xLightColour;
		xLightColour.r = static_cast<char>(0xff * Asura_Maths::ClampToRange(Asura_Photon_System::LocalPhotons[u].GetRedComponent(), 0.f, 1.f));
		xLightColour.g = static_cast<char>(0xff * Asura_Maths::ClampToRange(Asura_Photon_System::LocalPhotons[u].GetGreenComponent(), 0.f, 1.f));
		xLightColour.b = static_cast<char>(0xff * Asura_Maths::ClampToRange(Asura_Photon_System::LocalPhotons[u].GetBlueComponent(), 0.f, 1.f));
		xLightColour.a = 0xff;

		Vec xViewSpaceLightPos;
		const Mtx& xViewMatrix = Asura_GX::GetViewMatrix();
		MTXMultVec(xViewMatrix, (VecPtr)&xLightPos, (VecPtr)&xViewSpaceLightPos);
		GXInitLightColor(&axLights[u], axRGB[u]);
		GXInitLightPosv(&axLights[u], &xViewSpaceLightPos);
		GXInitLightDir(&axLights[u], 1.0f, 0.0f, 0.0f);
		GXInitLightAttnA(&axLights[u], 1.0f, 0.0f, 0.0f);
		GXInitLightDistAttn(&axLights[u], 1.0f, 0.0f, GX_DA_OFF);
		GXLoadLightObjImm(&axLights[u], g_aeLightEnums[u]);
		GXSetTevKColor(g_aeKColours[u], xLightColour);
		uLightFlags |= g_aeLightEnums[u];

		const Asura_Vector_3 xRelativeLightPos = xLightPos - Asura_GX_Environment::s_xMeshPos;
		const float fRange = xPhoton.GetRange();
		const float fTemp = 0.5f / fRange;
		const float afTexMatrixXZ[2][4] =
		{
			{ fTemp,	0.f, 0.f,	0.5f - xRelativeLightPos.x * fTemp},
			{ 0.f,		0.f, fTemp, 0.5f - xRelativeLightPos.z * fTemp}
		};
		const float afTexMatrixY[2][4] =
		{
			{ 0.f,	fTemp,	0.f, 0.5f - xRelativeLightPos.y * fTemp},
			{ 0.f,	0.f,	0.f, 0.5f}
		};

		GXTexMtx eTexMtx0 = g_aaeTexMtxIDs[u][0];
		GXTexMtx eTexMtx1 = g_aaeTexMtxIDs[u][1];
		GXTexCoordID eTexCoord0 = g_aaeTexCoordIDs[u][0];
		GXTexCoordID eTexCoord1 = g_aaeTexCoordIDs[u][1];
		GXTevStageID eStage0 = static_cast<GXTevStageID>(GX_TEVSTAGE0 + u * 3);
		GXTevStageID eStage1 = static_cast<GXTevStageID>(GX_TEVSTAGE1 + u * 3);
		GXTevStageID eStage2 = static_cast<GXTevStageID>(GX_TEVSTAGE2 + u * 3);

		GXLoadTexMtxImm(afTexMatrixXZ, eTexMtx0, GX_MTX2x4);
		GXLoadTexMtxImm(afTexMatrixY, eTexMtx1, GX_MTX2x4);
		

		GXSetTexCoordGen(eTexCoord0, GX_TG_MTX2x4, GX_TG_POS, eTexMtx0);
		GXSetTexCoordGen(eTexCoord1, GX_TG_MTX2x4, GX_TG_POS, eTexMtx1);

		GXSetTevOrder(eStage0, eTexCoord0, GX_TEXMAP3, GX_COLOR1A1);
		GXSetTevOrder(eStage1, eTexCoord1, GX_TEXMAP3, GX_COLOR_NULL);
		if(u)
		{
			GXSetTevOrder(eStage2, GX_TEXCOORD_NULL, GX_TEXMAP_NULL, GX_COLOR_NULL);
		}
		else
		{
			GXSetTevOrder(eStage2, GX_TEXCOORD_NULL, GX_TEXMAP_NULL, GX_COLOR0A0);
		}

		GXSetTevSwapMode(eStage0, g_aeSwapIDs[u], GX_TEV_SWAP0);
		GXSetTevSwapMode(eStage1, GX_TEV_SWAP0, GX_TEV_SWAP0);
		GXSetTevSwapMode(eStage2, GX_TEV_SWAP0, GX_TEV_SWAP0);

		// Modulate x-z distance attenuation with rasterised lighting from this channel (giving normal attenuation)
		GXSetTevColorIn(eStage0, GX_CC_ZERO, GX_CC_TEXC, GX_CC_RASC, GX_CC_ZERO);
		GXSetTevColorOp(eStage0, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_2, GX_ENABLE, GX_TEVREG0);

		// Modulate previous value with y distance attenuation
		GXSetTevColorIn(eStage1, GX_CC_ZERO, GX_CC_TEXC, GX_CC_C0, GX_CC_ZERO);
		GXSetTevColorOp(eStage1, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_ENABLE, GX_TEVREG0);

		// Modulate previous value with light colour, then add to final light value.
		GXSetTevKColorSel(eStage2, g_aeKColorIDs[u]);
		if(u)
		{
			GXSetTevColorIn(eStage2, GX_CC_ZERO, GX_CC_KONST, GX_CC_C0, GX_CC_CPREV);
		}
		else
		{
			GXSetTevColorIn(eStage2, GX_CC_ZERO, GX_CC_KONST, GX_CC_C0, GX_CC_RASC);
		}
		GXSetTevColorOp(eStage2, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_ENABLE, GX_TEVPREV);
	}
	if(dbg_bNormalAttenuation)
	{
		GXSetChanCtrl(GX_COLOR1, TRUE, GX_SRC_REG, GX_SRC_REG, uLightFlags, GX_DF_SIGN, GX_AF_NONE);
	}
	else
	{
		GXSetChanCtrl(GX_COLOR1,FALSE, GX_SRC_REG, GX_SRC_REG, 0, GX_DF_NONE, GX_AF_NONE);
	}
	Asura_Render::CurrentStates.RequestTextureIndex(3, Asura_GX_Environment::s_iAttenuationMap);
}

void Asura_GX_Env_Mesh::SetupArrays() const
{
	GXSetArray(GX_VA_POS, m_pxPositions, sizeof(Asura_Vector_3));
	Asura_GX_Environment::s_xMeshPos.Zero();
	const u_int uExportFlags = Asura_GX_Environment::s_uExportFlags;
	if (uExportFlags & Asura_GX_Environment::EXPORT_FLAG_NORMALS)
	{
		GXSetArray(GX_VA_NRM, (char*)Asura_GX_Environment::s_pxNormalTable, sizeof(Asura_Vector_3));
	}
	GXSetArray(GX_VA_TEX0, (char*)m_psTexCoords, sizeof(short) * 2);
	GXSetArray(GX_VA_TEX1, (char*)m_psTexCoords, sizeof(short) * 2);
}

void Asura_GX_Env_Mesh::Render() const
{
	SetupLighting();

	SetupArrays();

	const u_int uTotalTevStages = Asura_Photon_System::NumberOfLocalPhotons * 3;
	const u_int uTotalTexGens = Asura_Photon_System::NumberOfLocalPhotons * 2;
	for(u_int uStrip = 0; uStrip < m_uNumberOfStrips; uStrip++)
	{
		m_pxStrips[uStrip].BeginRender(uTotalTevStages, uTotalTexGens);
		m_pxStrips[uStrip].Render();
	}
}

void Asura_GX_Env_Mesh::RenderSimple() const
{
	SetupArrays();

	for(u_int uStrip = 0; uStrip < m_uNumberOfStrips; uStrip++)
	{
		m_pxStrips[uStrip].BeginRenderSimple();
		m_pxStrips[uStrip].Render();
	}
}

void Asura_GX_Env_Tristrip::BeginRender(u_int uBaseTevStages, u_int uBaseTexGens) const
{
	const int iSecondaryTexture = Asura_Material_System::GetTextureID(m_iMaterialID, 1);
	if(iSecondaryTexture != ASURA_TEXTURE_NONE)
	{
		GXSetVtxDesc(GX_VA_TEX1, GX_INDEX16);
	}
	else
	{
		GXSetVtxDesc(GX_VA_TEX1, GX_NONE);
	}
	Asura_GX::SetupMaterial(m_iMaterialID, uBaseTevStages, uBaseTexGens);
	if(Asura_Material_System::IsMaterialReflective(m_iMaterialID))
	{
		GXSetChanCtrl(GX_ALPHA1, TRUE, GX_SRC_REG, GX_SRC_REG, GX_LIGHT0, GX_DF_CLAMP, GX_AF_SPEC);
		GXSetNumChans(2);
	}
	else
	{
		GXSetChanCtrl(GX_ALPHA1,FALSE, GX_SRC_REG, GX_SRC_REG, 0, GX_DF_NONE, GX_AF_NONE);
		GXSetNumChans(uBaseTevStages ? 2 : 1);
	}
}

void Asura_GX_Env_Tristrip::BeginRenderSimple() const
{
	const int iSecondaryTexture = Asura_Material_System::GetTextureID(m_iMaterialID, 1);
	if(iSecondaryTexture != ASURA_TEXTURE_NONE)
	{
		GXSetVtxDesc(GX_VA_TEX1, GX_INDEX16);
	}
	else
	{
		GXSetVtxDesc(GX_VA_TEX1, GX_NONE);
	}
	Asura_GX::SetupMaterial(m_iMaterialID, 0, 0);
	if(Asura_Material_System::IsMaterialReflective(m_iMaterialID))
	{
		GXSetChanCtrl(GX_ALPHA1,FALSE, GX_SRC_REG, GX_SRC_REG, 0, GX_DF_NONE, GX_AF_NONE);
		GXSetNumChans(2);
	}
	else
	{
		GXSetNumChans(1);
	}
}

void Asura_GX_Env_Tristrip::Render(bool bAddToStats) const
{
	GXCallDisplayList(m_pucDisplayList, m_uDisplayListSize);
	if (bAddToStats)
	{
		Asura_Render::AddToTrianglesRendered(m_uNumberOfIndices - 2, 1, ASURA_RENDER_TTYPE_ENV);
	}
}
