//
// Asura_GX_Shadows.cpp
//
// Created by KOR - 10 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_GX_Shadows.h"

#if 01

// Asura includes
#include "Core/Asura_TextureManagement.h"
#include "Camera/Asura_Camera.h"
//#include "Dynamics/Asura_Dynamics.h"
#include "Light/Asura_Photon.h"
#include "Physics/Asura_Physics_Environment.h"
#include "Physics/Asura_Physics_EnvironmentCache.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "SpecialFX/Asura_Weather_System.h"

// Platform includes
#include "Rendering/Asura_GX.h"

#include <revolution/gx.h>

int						Asura_GX_ShadowRenderer::s_iShadowTexture = -1;
unsigned int			Asura_GX_ShadowRenderer::s_uNumShadows;
//Asura_Vector_3			Asura_GX_ShadowRenderer::s_xBasePos;
Asura_GX_Shadow_Bone	Asura_GX_ShadowRenderer::s_axShadowBones[ASURA_GX_MAX_SHADOWS];

u_int ClipPolyToPlane(Asura_Decal_Vertex* pxInVertices, u_int uNumberOfVertices, Asura_Decal_Vertex* pxOutVertices, Asura_Vector_3& xPlaneNormal, float fPlaneConstant);

void Asura_GX_ShadowRenderer::BeginRender()
{
	s_uNumShadows = 0;
}

DEBUGVAR bool dbg_bUseSunPos = false;
DEBUGVAR bool dbg_bUseAvgLocalLightPos = false;

void Asura_GX_ShadowRenderer::AddShadow(const Asura_Vector_3& xPosition1, const Asura_Vector_3& xPosition2, float fSize, float fAlpha /* = 0.375 */)
{
	Asura_Assert(s_uNumShadows < ASURA_GX_MAX_SHADOWS, "Too many shadows for this character.");
	if(s_uNumShadows >= ASURA_GX_MAX_SHADOWS) return;
	Asura_GX_Shadow_Bone* pxShadowBone = &s_axShadowBones[s_uNumShadows];
	pxShadowBone->m_xPosition1 = xPosition1;
	pxShadowBone->m_xPosition2 = xPosition2;
	pxShadowBone->m_fSize = fSize;
	pxShadowBone->m_fAlpha = fAlpha;
	s_uNumShadows++;
}


static Asura_GX_Shadow_Vertex	g_aaxVertexBuffer[ASURA_MAX_SHADOW_FANS][9];
static unsigned int				g_auNumVertices[ASURA_MAX_SHADOW_FANS];
void Asura_GX_ShadowRenderer::EndRender(unsigned int uCamVisFlags, bool bBufferedRender, const Asura_Vector_3* pxBasePos /* = NULL */)
{
	GXColor xGrey = {0x60, 0x60, 0x60, 0xff};
	GXColor xWhite = {0xff, 0xff, 0xff, 0xff};
	if(s_iShadowTexture == -1)
	{
		s_iShadowTexture = Asura_Texture_Management::GetTextureIndex("\\graphicnovel\\shadow.bmp");
	}
	GXClearVtxDesc();
	GXSetVtxDesc(GX_VA_POS, GX_DIRECT);
	GXSetVtxDesc(GX_VA_CLR0, GX_DIRECT);
	GXSetVtxDesc(GX_VA_TEX0, GX_DIRECT);
	GXSetNumChans(1);
	GXSetNumTexGens(1);
	GXSetNumTevStages(1);

	GXSetTevSwapMode(GX_TEVSTAGE0, GX_TEV_SWAP0, GX_TEV_SWAP0);
	GXSetTexCoordGen(GX_TEXCOORD0, GX_TG_MTX2x4, GX_TG_TEX0, GX_IDENTITY);
	GXSetTevOrder( GX_TEVSTAGE0, GX_TEXCOORD0, GX_TEXMAP0, GX_COLOR0A0);
	GXSetTevOp(GX_TEVSTAGE0, GX_MODULATE);

	GXSetChanCtrl(GX_COLOR0, GX_ENABLE, GX_SRC_VTX, GX_SRC_REG, 0, GX_DF_CLAMP, GX_AF_NONE);
	GXSetChanAmbColor(GX_COLOR0A0, xGrey);
	GXSetChanMatColor(GX_COLOR0A0, xWhite);	
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_INVCOLOUR);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestTextureIndex(0,s_iShadowTexture);

	Asura_Bounding_Box xOverallBB;
	Asura_GX_Shadow	axShadowBuffer[ASURA_GX_MAX_SHADOWS];
	Asura_Vector_3 xUp( 0.0f,-1.0f, 0.0f);
	if(!s_uNumShadows) return;
	float fBaseHeight;
	if(pxBasePos)
	{
		fBaseHeight = pxBasePos->y;
	}
	else
	{
		fBaseHeight = -Asura_Maths::LargeFloat;
		for(u_int uShadow = 0; uShadow < s_uNumShadows; uShadow++)
		{
			fBaseHeight = Asura_Maths::Max(s_axShadowBones[uShadow].m_xPosition1.y, fBaseHeight);
			fBaseHeight = Asura_Maths::Max(s_axShadowBones[uShadow].m_xPosition2.y, fBaseHeight);
		}
		fBaseHeight += 0.1f;
	}

	Asura_Vector_3 xDirectionFromLight(0.0f, 1.0f, 0.0f);
	xOverallBB.Reset();
	for(u_int uShadow = 0; uShadow < s_uNumShadows; uShadow++)
	{
		Asura_GX_Shadow_Bone* pxShadowBone = &s_axShadowBones[uShadow];
		Asura_GX_Shadow* pxShadow = &axShadowBuffer[uShadow];
#if 0
		if(Asura_Weather_System::IsSunGlareEnabled() && dbg_bUseSunPos)
		{
			xDirectionFromLight = s_xBasePos - Asura_Weather_System::GetSunPos();
		}
		if(dbg_bUseAvgLocalLightPos)
		{
			Asura_Bounding_Box xBB;
			xBB.Set(s_xBasePos);
			Asura_Photon_System::MakePhotonListForBoundingBox(xBB, xBB.CalculateCentre(), true, false);
			Asura_Photon_System::CalculateAverageLocalLightPos(xDirectionFromLight);
			xDirectionFromLight = s_xBasePos - xDirectionFromLight;
		}
#endif
		// Project position down to base position.
		xDirectionFromLight.Normalise();
		if(fBaseHeight - pxShadowBone->m_xPosition1.y < 0.0f) return;
		if(fBaseHeight - pxShadowBone->m_xPosition2.y < 0.0f) return;
		pxShadowBone->m_xPosition1 = pxShadowBone->m_xPosition1 + xDirectionFromLight * ((fBaseHeight - pxShadowBone->m_xPosition1.y)/xDirectionFromLight.y);
		pxShadowBone->m_xPosition2 = pxShadowBone->m_xPosition2 + xDirectionFromLight * ((fBaseHeight - pxShadowBone->m_xPosition2.y)/xDirectionFromLight.y) ;

		Asura_Vector_3 xDisp = pxShadowBone->m_xPosition2 - pxShadowBone->m_xPosition1;
		pxShadow->m_xPosition = (pxShadowBone->m_xPosition2 + pxShadowBone->m_xPosition1) * 0.5f;
		float fLength = xDisp.Magnitude();
		xDisp.Normalise();
		bool bRenderBlob = false;
		if(fLength < pxShadowBone->m_fSize)
		{
			bRenderBlob = true;
		}
		
		// Okay, extreme cheekiness here - steal some of the decal code:
		pxShadow->m_xUOffset = Asura_Vector_3(-1.0f, 0.0f, 0.0f);
		pxShadow->m_xVOffset = Asura_Vector_3( 0.0f, 0.0f, 1.0f);
		pxShadow->m_xNOffset = Asura_Vector_3( 0.0f,-1.0f, 0.0f);
		if(!bRenderBlob)
		{
			pxShadow->m_xUOffset = xDisp;
			pxShadow->m_xVOffset = pxShadow->m_xUOffset ^ pxShadow->m_xNOffset;
		}

		float fHeight	=	pxShadowBone->m_fSize;
		float fWidth	=	pxShadowBone->m_fSize;
		pxShadow->m_fDepth	=	1.0f;
		if(!bRenderBlob)
		{
			fWidth	=	fLength;
		}

		pxShadow->m_xUOffset *= fWidth;
		pxShadow->m_xVOffset *= fHeight;
		pxShadow->m_xNOffset *= pxShadow->m_fDepth;

		pxShadow->m_fInvHeight = 1.0f / fHeight;
		pxShadow->m_fInvWidth = 1.0f / fWidth;

		{
			pxShadow->m_xBB.Reset();
		
			Asura_Vector_3 xVertex1 = - pxShadow->m_xUOffset - pxShadow->m_xVOffset + pxShadow->m_xNOffset; 
			Asura_Vector_3 xVertex2 =   pxShadow->m_xUOffset - pxShadow->m_xVOffset + pxShadow->m_xNOffset; 
			Asura_Vector_3 xVertex3 = - pxShadow->m_xUOffset - pxShadow->m_xVOffset - pxShadow->m_xNOffset; 
			Asura_Vector_3 xVertex4 =   pxShadow->m_xUOffset - pxShadow->m_xVOffset - pxShadow->m_xNOffset; 
			Asura_Vector_3 xVertex5 = - pxShadow->m_xUOffset + pxShadow->m_xVOffset + pxShadow->m_xNOffset; 
			Asura_Vector_3 xVertex6 =   pxShadow->m_xUOffset + pxShadow->m_xVOffset + pxShadow->m_xNOffset; 
			Asura_Vector_3 xVertex7 = - pxShadow->m_xUOffset + pxShadow->m_xVOffset - pxShadow->m_xNOffset; 
			Asura_Vector_3 xVertex8 =   pxShadow->m_xUOffset + pxShadow->m_xVOffset - pxShadow->m_xNOffset; 

			pxShadow->m_xBB.Set(xVertex1);
			pxShadow->m_xBB.Set(xVertex2);
			pxShadow->m_xBB.Set(xVertex3);
			pxShadow->m_xBB.Set(xVertex4);
			pxShadow->m_xBB.Set(xVertex5);
			pxShadow->m_xBB.Set(xVertex6);
			pxShadow->m_xBB.Set(xVertex7);
			pxShadow->m_xBB.Set(xVertex8);

			pxShadow->m_xBB.Displace(pxShadow->m_xPosition);
		}

		pxShadow->m_xUOffset.Normalise();
		pxShadow->m_xVOffset.Normalise();
		pxShadow->m_xNOffset.Normalise();

		pxShadow->m_fAlpha = pxShadowBone->m_fAlpha;

		xOverallBB.Set(pxShadow->m_xBB);
	}

    //u_int uNumberOfTris = Asura_Dynamics_System::FindVisibleCollisionPolysInBoundingBox(xOverallBB);
	//if(!uNumberOfTris) return;

	//const Asura_Collision_Poly* pxTriangle = Asura_Dynamics_System::GetCollisionPolysArray();

	// make an environment cache of the polys where the poly
	Asura_Physics_EnvironmentCache xCache;

	// if we can see it, add a decal to it.
	xCache.SetFlagsOfPolysToIgnore( AsuraPolygonFlag_DontRender );
	xCache.SetFacesOnly( true );
	xCache.Update( xOverallBB );

	const Asura_Physics_EnvironmentCache_Face* pxFace = xCache.GetFaceList();

	Asura_Bounding_Box xTriBB;
	if(bBufferedRender)
	{
		//Asura_Assert(uNumberOfTris * s_uNumShadows < ASURA_MAX_SHADOW_FANS, "Too many polys to shadow");
		unsigned int uNumberOfFans = 0;
 		//for (u_int u=0; u<uNumberOfTris; u++, pxTriangle++)
		for (; pxFace!=NULL; pxFace = pxFace->m_pxNext )
		{
			const Asura_Vector_3 xV0(Asura_Physics_Environment::GetFaceVertex( pxFace->m_uFaceID, 0 ));
			const Asura_Vector_3 xV1(Asura_Physics_Environment::GetFaceVertex( pxFace->m_uFaceID, 1 ));
			const Asura_Vector_3 xV2(Asura_Physics_Environment::GetFaceVertex( pxFace->m_uFaceID, 2 ));

			Asura_Vector_3 xNormal = ((xV1 - xV0)^(xV2 - xV0));

			xNormal.Normalise();
			
			if (xUp * xNormal<=-0.02f) continue;
			xTriBB.Reset();
			xTriBB.Set(xV0);
			xTriBB.Set(xV1);
			xTriBB.Set(xV2);
			Asura_Decal_Vertex xVertexBufferA[9];
			Asura_Decal_Vertex xVertexBufferB[9];

			u_int uNumberOfVertices = 3;

			xVertexBufferA[0].m_xPosition = xV0;
			xVertexBufferA[1].m_xPosition = xV1;
			xVertexBufferA[2].m_xPosition = xV2;

			// Bottom
			uNumberOfVertices = ClipPolyToPlane( xVertexBufferA, uNumberOfVertices, xVertexBufferB, xUp, 1.0f + fBaseHeight);
			// Not quite perfected this yet
			//uNumberOfVertices = ClipPolyToHeight( xVertexBufferA, uNumberOfVertices, xVertexBufferB, true, 1.0f + s_xBasePos.y);
			if (uNumberOfVertices==0) continue;
			// Top
			Asura_Vector_3 xPlaneNormal = -xUp;
			uNumberOfVertices = ClipPolyToPlane( xVertexBufferB, uNumberOfVertices, xVertexBufferA, xPlaneNormal, 1.0f - fBaseHeight);
			//uNumberOfVertices = ClipPolyToHeight( xVertexBufferB, uNumberOfVertices, xVertexBufferA, false, 1.0f - s_xBasePos.y);
			if (uNumberOfVertices==0) continue;

			for(u_int uShadow = 0; uShadow < s_uNumShadows; uShadow++)
			{
				if(uNumberOfFans >= ASURA_MAX_SHADOW_FANS) break;
				Asura_GX_Shadow* pxShadow = &axShadowBuffer[uShadow];
				if(!pxShadow->m_xBB.Intersects(xTriBB)) continue;
				Asura_GX_Shadow_Vertex* pxVertex = g_aaxVertexBuffer[uNumberOfFans];
				for (u_int v=0; v<uNumberOfVertices; v++)
				{
					pxVertex->m_xPosition = xVertexBufferA[v].m_xPosition;
					// Set the colour
					Asura_Vector_3 xDisp = xVertexBufferA[v].m_xPosition - pxShadow->m_xPosition;

					pxVertex->m_fU = (0.5f * (xDisp * pxShadow->m_xUOffset) * pxShadow->m_fInvWidth + 0.5f);
					pxVertex->m_fV = (0.5f * (xDisp * pxShadow->m_xVOffset) * pxShadow->m_fInvHeight + 0.5f);

					Asura_RGBA xColour;
					xColour.m_fA = pxShadow->m_fAlpha * (1.0f - Asura_Maths::Abs(xDisp * pxShadow->m_xNOffset));
					xColour.m_fR = xColour.m_fA;
					xColour.m_fG = xColour.m_fA;
					xColour.m_fB = xColour.m_fA;
					pxVertex->m_uColour = xColour.ToUIntRGBA(true);
					pxVertex++;
				}
				g_auNumVertices[uNumberOfFans] = uNumberOfVertices;
				uNumberOfFans++;
			}
		}
		if(uNumberOfFans)
		for(int i=0; i<Asura_Camera::GetNumberOfCameras(); i++)
		{
			if(uCamVisFlags & (1 << i))
			{
				Asura_Camera::SetCameraIndex(i);
			}
			else
			{
				continue;
			}

			if (Asura_Camera::IsCameraActive())
			{
				Asura_Render::ApplyCameraSettings();

				// Set up projection matrix to shift z towards camera
				Mtx44 xProjMatrix;
				MTXCopy(Asura_GX::GetProjMatrix(), xProjMatrix);
				MTXRowCol(xProjMatrix, 2, 2) = 0.0f;
				GXSetProjection(xProjMatrix, GX_PERSPECTIVE);

				for(unsigned int uFan = 0; uFan < uNumberOfFans; uFan++)
				{
					// Render it straight out
					GXBegin(GX_TRIANGLEFAN, GX_VTXFMT3, g_auNumVertices[uFan]);
					{
						Asura_GX_Shadow_Vertex* pxVertex = g_aaxVertexBuffer[uFan];
						for(unsigned int u = 0; u < g_auNumVertices[uFan]; u++)
						{
							GXPosition3f32(pxVertex->m_xPosition.x, pxVertex->m_xPosition.y, pxVertex->m_xPosition.z);
							GXColor1u32(pxVertex->m_uColour);
							GXTexCoord2f32(pxVertex->m_fU, pxVertex->m_fV);
							pxVertex++;
						}
					}
					GXEnd();
				}
				GXSetProjection(Asura_GX::GetProjMatrix(), GX_PERSPECTIVE);
			}
		}
	}
	else
	{
		for(int i=0; i<Asura_Camera::GetNumberOfCameras(); i++)
		{
			if(uCamVisFlags & (1 << i))
			{
				Asura_Camera::SetCameraIndex(i);
			}
			else
			{
				continue;
			}

			if (Asura_Camera::IsCameraActive())
			{
				Asura_Render::ApplyCameraSettings();

				// Set up projection matrix to shift z towards camera
				Mtx44 xProjMatrix;
				MTXCopy(Asura_GX::GetProjMatrix(), xProjMatrix);
				MTXRowCol(xProjMatrix, 2, 2) = 0.0f;
				GXSetProjection(xProjMatrix, GX_PERSPECTIVE);

 				//for (u_int u=0; u<uNumberOfTris; u++, pxTriangle++)
				for (; pxFace!=NULL; pxFace = pxFace->m_pxNext )
				{
					const Asura_Vector_3 xV0(Asura_Physics_Environment::GetFaceVertex( pxFace->m_uFaceID, 0 ));
					const Asura_Vector_3 xV1(Asura_Physics_Environment::GetFaceVertex( pxFace->m_uFaceID, 1 ));
					const Asura_Vector_3 xV2(Asura_Physics_Environment::GetFaceVertex( pxFace->m_uFaceID, 2 ));

					Asura_Vector_3 xNormal = ((xV1 - xV0)^(xV2 - xV0));

					xNormal.Normalise();

					if (xUp * xNormal<=-0.02f) continue;
					xTriBB.Reset();
					xTriBB.Set(xV0);
					xTriBB.Set(xV1);
					xTriBB.Set(xV2);
					Asura_Decal_Vertex xVertexBufferA[9];
					Asura_Decal_Vertex xVertexBufferB[9];

					u_int uNumberOfVertices = 3;

					xVertexBufferA[0].m_xPosition = xV0;
					xVertexBufferA[1].m_xPosition = xV1;
					xVertexBufferA[2].m_xPosition = xV2;

					// Bottom
					uNumberOfVertices = ClipPolyToPlane( xVertexBufferA, uNumberOfVertices, xVertexBufferB, xUp, 1.0f + fBaseHeight);
					if (uNumberOfVertices==0) continue;
					// Top
					Asura_Vector_3 xPlaneNormal = -xUp;
					uNumberOfVertices = ClipPolyToPlane( xVertexBufferB, uNumberOfVertices, xVertexBufferA, xPlaneNormal, 1.0f - fBaseHeight);

					if (uNumberOfVertices==0) continue;
					Asura_GX_Shadow_Vertex axVertexBuffer[9];
					for(u_int uShadow = 0; uShadow < s_uNumShadows; uShadow++)
					{
						Asura_GX_Shadow* pxShadow = &axShadowBuffer[uShadow];
						if(!pxShadow->m_xBB.Intersects(xTriBB)) continue;
						// Render it straight out
						GXBegin(GX_TRIANGLEFAN, GX_VTXFMT3, uNumberOfVertices);
						for (u_int v=0; v<uNumberOfVertices; v++)
						{
							// Set the colour
							Asura_Vector_3 xDisp = xVertexBufferA[v].m_xPosition - pxShadow->m_xPosition;
							Asura_RGBA xColour;
							xColour.m_fA = pxShadow->m_fAlpha * (1.0f - Asura_Maths::Abs(xDisp * pxShadow->m_xNOffset));
							xColour.m_fR = xColour.m_fA;
							xColour.m_fG = xColour.m_fA;
							xColour.m_fB = xColour.m_fA;
							GXPosition3f32(xVertexBufferA[v].m_xPosition.x, xVertexBufferA[v].m_xPosition.y, xVertexBufferA[v].m_xPosition.z);
							GXColor1u32(xColour.ToUIntRGBA(true));
							GXTexCoord2f32((0.5f * (xDisp * pxShadow->m_xUOffset) * pxShadow->m_fInvWidth + 0.5f), (0.5f * (xDisp * pxShadow->m_xVOffset) * pxShadow->m_fInvHeight + 0.5f));
						}
						GXEnd();
					}
				}
			}
		}
		GXSetProjection(Asura_GX::GetProjMatrix(), GX_PERSPECTIVE);
	}
}

u_int ClipPolyToPlane(Asura_Decal_Vertex* pxInVertices, u_int uNumberOfVertices, Asura_Decal_Vertex* pxOutVertices, Asura_Vector_3& xPlaneNormal, float fPlaneConstant)
{
	u_int uNewVertexCount = 0;

	float fDot[9];

	{
		bool bAllNegative = true;
		for (u_int u=0; u<uNumberOfVertices; u++)
		{
			fDot[u] = pxInVertices[u].m_xPosition*xPlaneNormal + fPlaneConstant;
			if (fDot[u]>=0.0f)
			{
				bAllNegative = false;
			}
		}

		/// Kevin Lea - 14/11/2002 10:49:43 - early exit if all vertices would be clipped
		if (bAllNegative) return 0;
	}

	for (u_int u=0; u<uNumberOfVertices; u++)
	{
		u_int v = (u!=0) ? u-1 : uNumberOfVertices-1; // previous vertex

		if (fDot[u]<0.0f)
		{
			if (fDot[v]>=0.0f)
			{
				// Current vertex is on negative side of plane,
				// but previous vertex is on positive side.
				Asura_Vector_3 xEdge = pxInVertices[v].m_xPosition - pxInVertices[u].m_xPosition;
				float f = xEdge * xPlaneNormal;

				Asura_Assert( f!=0.0f, "edge parallel to plane");

				float fLambda = fDot[v] / f;

				// Asura_Assertf( fLambda >= -0.0001f && fLambda <= 1.0001f, ("Out of bounds: %f",Asura_Maths::FtoD(fLambda)));
				if (fLambda < 0.0f) fLambda = 0.0f;
				if (fLambda > 1.0f) fLambda = 1.0f;

				pxOutVertices[uNewVertexCount].m_xPosition = pxInVertices[v].m_xPosition - xEdge * fLambda;
				uNewVertexCount++;	
			}
		}
		else
		{
			if (fDot[v]<0.0f)
			{
				// Current vertex is on positive side of plane,
				// but previous vertex is on negative side.
				const Asura_Vector_3 xEdge = pxInVertices[u].m_xPosition - pxInVertices[v].m_xPosition;
				float f = xEdge * xPlaneNormal;

				Asura_Assert( f!=0.0f, "edge parallel to plane");

				float fLambda = fDot[u] / f;

				// Asura_Assertf( fLambda >= -0.0001f && fLambda <= 1.0001f, ("Out of bounds: %f",Asura_Maths::FtoD(fLambda)));
				if (fLambda < 0.0f) fLambda = 0.0f;
				if (fLambda > 1.0f) fLambda = 1.0f;

				pxOutVertices[uNewVertexCount].m_xPosition = pxInVertices[u].m_xPosition - xEdge * fLambda;
				uNewVertexCount++;	
			}

			/// Kevin Lea - 14/11/2002 11:22:04 - include current vertex
			pxOutVertices[uNewVertexCount].m_xPosition = pxInVertices[u].m_xPosition;
			uNewVertexCount++;
		}
	}


	return uNewVertexCount;
}

#if 0
static int ClipPolyToHeight(Asura_Decal_Vertex* pxInVertices, u_int uNumberOfVertices, Asura_Decal_Vertex* pxOutVertices, bool bUp, float fPlaneConstant)
{
	u_int uNewVertexCount = 0;

	float fDot[9];

	{
		bool bAllNegative = true;
		for (u_int u=0; u<uNumberOfVertices; u++)
		{
			if(bUp)
			{
				fDot[u] = -pxInVertices[u].m_xPosition.y + fPlaneConstant;
			}
			else
			{
				fDot[u] = pxInVertices[u].m_xPosition.y + fPlaneConstant;
			}
			if (fDot[u]>=0.0f)
			{
				bAllNegative = false;
			}
		}

		/// Kevin Lea - 14/11/2002 10:49:43 - early exit if all vertices would be clipped
		if (bAllNegative) return 0;
	}

	for (u_int u=0; u<uNumberOfVertices; u++)
	{
		u_int v = (u!=0) ? u-1 : uNumberOfVertices-1; // previous vertex
		
		if (fDot[u]<0.0f)
		{
			if (fDot[v]>=0.0f)
			{
				// Current vertex is on negative side of plane,
				// but previous vertex is on positive side.
				if(bUp)
				{
					float f = pxInVertices[u].m_xPosition.y - pxInVertices[v].m_xPosition.y;
					float fLambda = fDot[v] / f;
					pxOutVertices[uNewVertexCount].m_xPosition = pxInVertices[v].m_xPosition;
					pxOutVertices[uNewVertexCount].m_xPosition.y += f * fLambda;
				}
				else
				{
					float f = pxInVertices[v].m_xPosition.y - pxInVertices[u].m_xPosition.y;
					float fLambda = fDot[v] / f;
					pxOutVertices[uNewVertexCount].m_xPosition = pxInVertices[v].m_xPosition;
					pxOutVertices[uNewVertexCount].m_xPosition.y -= f * fLambda;
				}				
				uNewVertexCount++;	
			}
		}
		else
		{
			if (fDot[v]<0.0f)
			{
				// Current vertex is on positive side of plane,
				// but previous vertex is on negative side.
				if(bUp)
				{
					float f = pxInVertices[v].m_xPosition.y - pxInVertices[u].m_xPosition.y;
					float fLambda = fDot[u] / f;
					pxOutVertices[uNewVertexCount].m_xPosition = pxInVertices[u].m_xPosition;
					pxOutVertices[uNewVertexCount].m_xPosition.y += f * fLambda;
				}
				else
				{
					float f = pxInVertices[u].m_xPosition.y - pxInVertices[v].m_xPosition.y;
					float fLambda = fDot[u] / f;
					pxOutVertices[uNewVertexCount].m_xPosition = pxInVertices[u].m_xPosition;
					pxOutVertices[uNewVertexCount].m_xPosition.y -= f * fLambda;
				}				
				uNewVertexCount++;	
			}

			/// Kevin Lea - 14/11/2002 11:22:04 - include current vertex
			pxOutVertices[uNewVertexCount].m_xPosition = pxInVertices[u].m_xPosition;
			uNewVertexCount++;
		}
	}
	return uNewVertexCount;
}

#endif

#endif

//
// Asura_ShadowBuffer_System
//

#include "Rendering/Asura_ShadowBuffer.h"

void Asura_ShadowBuffer_System::PreAllocateBuffer( const Asura_Animation* pxAnim, BUFFERSIZE eBufferSize, float fScale )
{
}

void Asura_ShadowBuffer_System::PreProcessLevelData( void )
{
}
