//
// Asura_GX_SmoothSkin.cpp
//
// Created by KOR - 10 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Rendering/Asura_GX_SmoothSkin.h"

// Asura
#include "Animation/Asura_Hierarchy.h"
#include "Animation/Asura_TextureAnimation.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Material_System.h"
#include "Core/Asura_TextureManagement.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Rendering/Asura_Render.h"

// Asura Wii
#include "ChunkSystem/Asura_Platform_Chunk_ResourceFile.h"
#include "Core/Asura_Wii_MEM2.h"
#include "Rendering/Asura_GX.h"

// Library headers
#include <revolution/gx.h>
#include <revolution/os.h>
#include <revolution/base/PPCArch.h>

// Static members
Asura_TemplateHashTree<Asura_GX_SmoothSkin>	Asura_GX_SmoothSkinRenderer::s_xHashTree;


#ifdef USE_SMOOTH_SKIN_ADAPTIVE_RENDERBUFFER
u_int Asura_GX_SmoothSkin_RenderBuffer::s_uRenderVertexSize;
#endif
Asura_GX_SmoothSkin_Output_Vertex* Asura_GX_SmoothSkin_RenderBuffer::s_pxRenderVertices = NULL;
unsigned int Asura_GX_SmoothSkin_RenderBuffer::s_auRenderVertexPtr[uASURA_WII_MAX_SMOOTHSKINS];
volatile u_int Asura_GX_SmoothSkin_RenderBuffer::s_pxRenderVertexGPUIndex;
u_int Asura_GX_SmoothSkin_RenderBuffer::s_pxRenderVertexCPUIndex;

void Asura_GX_SmoothSkin_RenderBuffer::Initialise()
{
#ifndef USE_SMOOTH_SKIN_ADAPTIVE_RENDERBUFFER
#ifdef _DEBUG
	// Stick it in MEM2 to free up some MEM1.
	s_pxRenderVertices = ASURA_WII_MEM2_ALLOC_ARRAY(Asura_GX_SmoothSkin_Output_Vertex, uASURA_WII_MAX_SMOOTHSKIN_VERTICES);
#else
	s_pxRenderVertices = new Asura_GX_SmoothSkin_Output_Vertex[uASURA_WII_MAX_SMOOTHSKIN_VERTICES];
#endif
#endif//ndef USE_SMOOTH_SKIN_ADAPTIVE_RENDERBUFFER
}

void Asura_GX_SmoothSkin_RenderBuffer::Shutdown()
{
#ifdef _DEBUG
	// Stick it in MEM2 to free up some MEM1.
	ASURA_WII_MEM2_FREE(s_pxRenderVertices);
#else
	delete [] s_pxRenderVertices;
#endif

}

void Asura_GX_SmoothSkin_RenderBuffer::IncrementGPUPtr()
{
	s_pxRenderVertexGPUIndex = (s_pxRenderVertexGPUIndex + 1) % uASURA_WII_MAX_SMOOTHSKINS;
}
void Asura_GX_SmoothSkin_RenderBuffer::PostCreateVerts()
{
	u_int uNextPtr = (s_pxRenderVertexCPUIndex + 1) % uASURA_WII_MAX_SMOOTHSKINS;
	u_int uCurrent = s_auRenderVertexPtr[s_pxRenderVertexCPUIndex];
	u_int uNext = s_auRenderVertexPtr[uNextPtr];

	DCStoreRangeNoSync( &s_pxRenderVertices[ uCurrent ], (uNext - uCurrent) * sizeof( Asura_GX_SmoothSkin_Output_Vertex ) );
}

void Asura_GX_SmoothSkin_RenderBuffer::PostRenderVerts()
{
	s_pxRenderVertexCPUIndex = (s_pxRenderVertexCPUIndex + 1) % uASURA_WII_MAX_SMOOTHSKINS;
	GXPixModeSync();
	Asura_GX::AddDrawSyncCallback(Asura_GX::GPU_CALLBACK_SMOOTHSKIN_SYNC);
	GXPixModeSync();
}

Asura_GX_SmoothSkin_Output_Vertex *Asura_GX_SmoothSkin_RenderBuffer::GetRenderVertices()
{
	return &s_pxRenderVertices[s_auRenderVertexPtr[s_pxRenderVertexCPUIndex]];
}
void Asura_GX_SmoothSkin_RenderBuffer::AllocateRenderVertices(u_int uNumVerts)
{
#ifdef USE_SMOOTH_SKIN_ADAPTIVE_RENDERBUFFER
	if (s_pxRenderVertices == NULL)
		s_pxRenderVertices = new Asura_GX_SmoothSkin_Output_Vertex[s_uRenderVertexSize = uASURA_WII_MAX_SMOOTHSKIN_VERTICES];
#endif

	u_int uCurrent = s_auRenderVertexPtr[s_pxRenderVertexCPUIndex];
	u_int uGPU = s_auRenderVertexPtr[s_pxRenderVertexGPUIndex];
	u_int uUsed;

	if (uGPU > uCurrent)
		uUsed = s_uRenderVertexSize - uGPU + uCurrent;
	else
		uUsed = uCurrent - uGPU;

	uUsed += uNumVerts;

	u_int uStart = uCurrent;

	if (uCurrent + uNumVerts > s_uRenderVertexSize)
	{

		s_auRenderVertexPtr[s_pxRenderVertexCPUIndex] = 0;
		uCurrent = 0;


	} 
	if (s_pxRenderVertexCPUIndex != s_pxRenderVertexGPUIndex &&	((uGPU >= uCurrent && uGPU < uCurrent + uNumVerts) || (uStart != uCurrent && uGPU >= uStart)))
	{
#ifdef USE_SMOOTH_SKIN_ADAPTIVE_RENDERBUFFER
		GXPixModeSync();
		Asura_GX::AddDrawSyncCallback(Asura_GX::GPU_CALLBACK_SMOOTHSKIN_SYNC);
		GXFlush();


		while (s_pxRenderVertexGPUIndex != (s_pxRenderVertexCPUIndex+1))
			OSYieldThread();


		s_pxRenderVertexGPUIndex = 0;
		s_pxRenderVertexCPUIndex = 0;
		s_auRenderVertexPtr[s_pxRenderVertexCPUIndex] = 0;
		uCurrent = 0;

		delete [] s_pxRenderVertices;
		s_pxRenderVertices = new Asura_GX_SmoothSkin_Output_Vertex[s_uRenderVertexSize = (s_uRenderVertexSize * 3) / 2]; 
#else
		while (s_pxRenderVertexCPUIndex != s_pxRenderVertexGPUIndex && 
			((s_auRenderVertexPtr[s_pxRenderVertexGPUIndex] >= uCurrent && s_auRenderVertexPtr[s_pxRenderVertexGPUIndex] <= uCurrent + uNumVerts) ||
			(uStart != uCurrent && s_auRenderVertexPtr[s_pxRenderVertexGPUIndex] >= uStart)))
			OSYieldThread();
#endif
	}
	u_int uNextPtr = (s_pxRenderVertexCPUIndex + 1) % uASURA_WII_MAX_SMOOTHSKINS;
	s_auRenderVertexPtr[uNextPtr] = uCurrent + uNumVerts;
}
void Asura_GX_SmoothSkin_RenderBuffer::SetupArrays(bool bIncludeNormals)
{
	PPCSync();
	Asura_GX_SmoothSkin_Output_Vertex *pRenderVerts = GetRenderVertices();
	GXSetArray( GX_VA_POS, &pRenderVerts->m_xPosition, sizeof( Asura_GX_SmoothSkin_Output_Vertex ) );
	if (bIncludeNormals)
	{
		GXSetArray( GX_VA_NRM, &pRenderVerts->m_xNormal, sizeof( Asura_GX_SmoothSkin_Output_Vertex ) );
	}
}

static Mtx g_xBones[ASURA_MAX_NO_OF_BONES_PER_MODEL];

void Asura_GX_SmoothSkinRenderer::Initialise()
{
	Asura_GX_SmoothSkin_RenderBuffer::Initialise();
}

void Asura_GX_SmoothSkinRenderer::Update()
{
	// Nothing to update, I think
}

void Asura_GX_SmoothSkinRenderer::Reset()
{
	s_xHashTree.Reset();
}

void Asura_GX_SmoothSkinRenderer::Shutdown()
{
	Asura_GX_SmoothSkin_RenderBuffer::Shutdown();
	s_xHashTree.Reset();
}

bool Asura_GX_SmoothSkinRenderer::AddSmoothSkin(Asura_Handle fileHandle, u_int uChunkVersion, u_int uChunkFlags)
{
	Asura_Hash_ID uHashID;
	Asura_FileAccess::ReadData(fileHandle, &uHashID, sizeof(Asura_Hash_ID));

	Asura_GX_SmoothSkin* pxSkin = s_xHashTree.AddNode(uHashID);
	pxSkin->LoadFromChunk(fileHandle, uChunkVersion, uChunkFlags);

	return (pxSkin != NULL);
}

int Asura_GX_SmoothSkinRenderer::GetFirstMaterialIndex(Asura_Hash_ID uHashID)
{
	const Asura_GX_SmoothSkin* pxSkin = GetSkin(uHashID);
	if(pxSkin)
	{
		return pxSkin->GetMaterialID(0);
	}
	Asura_Assert(0, "Could not find character.");
	return ASURA_MATERIAL_NONE;
}

const Asura_GX_SmoothSkin* Asura_GX_SmoothSkinRenderer::GetSkin(Asura_Hash_ID uHashID)
{
	return s_xHashTree.GetDataByHash(uHashID);
}

void Asura_GX_SmoothSkinRenderer::RenderSmoothSkin(Asura_Hash_ID uHashID, u_int uCamVisFlags, int iTexIDOverride, float fAlpha /* = 1.f */)
{
	const Asura_GX_SmoothSkin* pxSkin = GetSkin(uHashID);
	Asura_Assert(pxSkin, "Could not find character to render.");
	if(pxSkin)
	{
		pxSkin->BeginRender(uCamVisFlags);
		if(fAlpha == 1.f)
		{
			pxSkin->Render(uCamVisFlags, iTexIDOverride);
		}
		else
		{
			pxSkin->Render(uCamVisFlags, iTexIDOverride, true);
			pxSkin->Render(uCamVisFlags, iTexIDOverride, false, fAlpha);
		}
		Asura_GX_SmoothSkin_RenderBuffer::PostRenderVerts();
	}
}

bool Asura_GX_SmoothSkinRenderer::IsSkinOpaque(Asura_Hash_ID uHashID)
{
	const Asura_GX_SmoothSkin* pxSkin = GetSkin(uHashID);
	if(pxSkin)
	{
		const u_int uNumMaterials = pxSkin->GetNumberOfMaterials();
		for(u_int u = 0; u < uNumMaterials; ++u)
		{
			if(Asura_Material_System::IsMaterialTransparent(pxSkin->GetMaterialID(u)))
			{
				return false;
			}
		}
	}
	return true;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////


Asura_GX_SmoothSkin::Asura_GX_SmoothSkin()
: m_uNumberOfVertices(0)
, m_uNumberOfBoneInfos(0)
, m_uNumberOfMaterials(0)
, m_uColourDepth(0)
, m_pxVertices(NULL)
, m_pxTexInfo(NULL)
, m_pxBoneInfo(NULL)
, m_pxMaterials(NULL)
, m_pucColours(NULL)
, m_bIncludeNormals(false)
{
}

void Asura_GX_SmoothSkin::LoadFromStream(Asura_Chunk_Stream& xStream, u_int uChunkVersion, u_int uChunkFlags)
{
	Asura_Assert(uChunkVersion >= 4, "Old skin version loading not supported.");

	m_bIncludeNormals = (uChunkFlags & ASURA_WII_SMOOTHSKIN_CHUNK_FLAG_HAS_NORMALS != 0);

	xStream >> m_uNumberOfVertices;
	xStream >> m_uNumberOfBoneInfos;
	xStream >> m_uNumberOfMaterials;

	Asura_Assert(m_uNumberOfVertices, "No vertices for skin");
	Asura_Assert(m_uNumberOfBoneInfos, "No bone info for skin");
	Asura_Assert(m_uNumberOfMaterials, "No materials for skin");
	m_pxMaterials = new Asura_GX_SmoothSkin_Material[m_uNumberOfMaterials];

	for(u_int uMaterial = 0; uMaterial < m_uNumberOfMaterials; ++uMaterial)
	{
		u_int uDisplayListSize = 0;
		Asura_GX_SmoothSkin_Material& xMaterial = m_pxMaterials[uMaterial];
		Asura_GX_DisplayListBlock& xDisplayListBlock = xMaterial.m_xDisplayListBlock;
		xStream >> xMaterial.m_iMaterialID;
		xStream >> uDisplayListSize;
		xStream >> xMaterial.m_uNumTris;

		// Convert ID
		xMaterial.m_iMaterialID = Asura_Material_System::ConvertIndex(xMaterial.m_iMaterialID);

		// Initialise display list
		xDisplayListBlock.Initialise(uDisplayListSize, false);
		Asura_GX_DisplayList::Iterator xIter = xDisplayListBlock.DisplayList().GetIterator();
		xIter.ReadFromStream(xStream, xDisplayListBlock.GetSize());
		xIter.Finalize(xDisplayListBlock.DisplayList(), false);
	}

	xStream >> m_uColourDepth;
	if(m_uColourDepth)
	{
		m_pucColours = new u_char[m_uColourDepth * m_uNumberOfVertices];
		xStream.Read(m_uColourDepth * m_uNumberOfVertices, m_pucColours);
	}
	else
	{
		m_pucColours = NULL;
	}

	// Keep this in MEM1 as we convert it on the CPU.
	m_pxVertices = new Asura_GX_SmoothSkin_Vertex[m_uNumberOfVertices];
	m_pxBoneInfo = new Asura_GX_SmoothSkin_BoneInfo[m_uNumberOfBoneInfos];
	// This could probably do with being moved to MEM2
	m_pxTexInfo  = new Asura_GX_SmoothSkin_TexInfo[m_uNumberOfVertices];
	if(m_bIncludeNormals)
	{
		xStream.Read(sizeof(Asura_GX_SmoothSkin_Vertex) * m_uNumberOfVertices, m_pxVertices);
	}
	else
	{
		for(unsigned int uIndex = 0; uIndex < m_uNumberOfVertices; ++uIndex )
		{
			xStream.Read(sizeof(u_short) * 3, &m_pxVertices[uIndex].m_sX);
		}
	}
	xStream.Read(sizeof(Asura_GX_SmoothSkin_TexInfo) * m_uNumberOfVertices, m_pxTexInfo);
	xStream.Read(sizeof(Asura_GX_SmoothSkin_BoneInfo) * m_uNumberOfBoneInfos, m_pxBoneInfo);
}

void Asura_GX_SmoothSkin::LoadFromChunk(Asura_Handle fileHandle, u_int uChunkVersion, u_int uChunkFlags)
{
	Asura_Chunk_File_Stream xStream(fileHandle);
	LoadFromStream(xStream, uChunkVersion, uChunkFlags);
}

Asura_GX_SmoothSkin::~Asura_GX_SmoothSkin()
{
	delete[] m_pxVertices;
	delete[] m_pxTexInfo;
	delete[] m_pxBoneInfo;
	delete[] m_pxMaterials;
	delete[] m_pucColours;
}

void Asura_GX_SmoothSkin::CalculateBoneOffsets() const
{
	for (int b=0; b<Asura_Hierarchy_System::s_pxSkin->NumberOfBones; b++)
	{
		Asura_Hierarchy_System::s_axBoneOri[b] =
            Asura_Hierarchy_System::GetWorldspaceBoneOri(b) * Asura_Hierarchy_System::s_pxSkin->BoneRestOrientation[b];
		Asura_Hierarchy_System::s_axBonePos[b] =
            Asura_Hierarchy_System::GetWorldspaceBonePos(b) - Asura_Hierarchy_System::s_pxSkin->BoneRestPosition[b]*Asura_Hierarchy_System::s_axBoneOri[b];

		Asura_GX::OriPosToMTX(Asura_Hierarchy_System::s_axBoneOri[b], Asura_Hierarchy_System::s_axBonePos[b], g_xBones[b]);
	}
}

void Asura_GX_SmoothSkin::CalculateAndConvertVertexPositions( const bool bRenderSimple ) const
{
	Asura_Assert(m_uNumberOfVertices < uASURA_WII_MAX_SMOOTHSKIN_VERTICES, "Number of character vertices exceeds size of intermediate skin renderer storage.");

    // set GQR6 to convert s16 -> float with divide by 1024
    OSSetGQR6( OS_GQR_S16, 10 );
    // set GQR7 to convert s8 -> float with divide by 64
    OSSetGQR7( OS_GQR_S8,  6 );

	float         fWeight;
	Vec           xTempVector;
    unsigned char ucTotalWeight;
    VecPtr        pxWorldPosition;
	Vec           xSkinPosition;
	Vec           xSkinNormal;
    Mtx*          pxBoneMatrix;
    unsigned char ucBoneInfoWeight;

#ifdef ASURA_WII_INLINE_ASM_MATHS
    volatile register float fTempReg;
#endif

	const unsigned int uVertexLimit = m_uNumberOfVertices;

	Asura_GX_SmoothSkin_RenderBuffer::AllocateRenderVertices(uVertexLimit);
	Asura_GX_SmoothSkin_BoneInfo*      pxBoneInfo     = m_pxBoneInfo;
	Asura_GX_SmoothSkin_Vertex*        pxSkinVertex   = m_pxVertices;
	Asura_GX_SmoothSkin_Output_Vertex* pxRenderVertex = Asura_GX_SmoothSkin_RenderBuffer::GetRenderVertices();

	for ( unsigned int uVertex = 0; uVertex < uVertexLimit; ++uVertex )
	{
		// Get skin position, re-cast and scale to float
#ifdef ASURA_WII_INLINE_ASM_MATHS
        asm __volatile__ ( "psq_l  %0, 0(%2), 0, 6 \n"           \
                           "psq_st %1, 0(%3), 0, 0   "           \
                           : "=f"( fTempReg )                    \
                           : "0"( fTempReg ), "b"( &pxSkinVertex->m_sX ), "b"( &xSkinPosition.x ) );
        asm ( "psq_l %0, 0(%1), 1, 6" : "=f"( xSkinPosition.z ) : "b"( &pxSkinVertex->m_sZ ) );
#else
		OSs16tof32(&pxSkinVertex->m_sX, &xSkinPosition.x);
		OSs16tof32(&pxSkinVertex->m_sY, &xSkinPosition.y);
		OSs16tof32(&pxSkinVertex->m_sZ, &xSkinPosition.z);
		VECScale(&xSkinPosition, &xSkinPosition, 0.0009765625f); // 1/1024
#endif
		// Get skin normal, re-cast and scale to float
		if (m_bIncludeNormals)
		{
#ifdef ASURA_WII_INLINE_ASM_MATHS
			asm __volatile__ ( "psq_l  %0, 0(%2), 0, 7 \n"           \
							"psq_st %1, 0(%3), 0, 0   "           \
							: "=f"( fTempReg )                    \
							: "0"( fTempReg ), "b"( &pxSkinVertex->m_cNX ), "b"( &xSkinNormal.x ) );
			asm ( "psq_l %0, 0(%1), 1, 7" : "=f"( xSkinNormal.z ) : "b"( &pxSkinVertex->m_cNZ ) );
#else
			OSs8tof32(&pxSkinVertex->m_cNX, &xSkinNormal.x);
			OSs8tof32(&pxSkinVertex->m_cNY, &xSkinNormal.y);
			OSs8tof32(&pxSkinVertex->m_cNZ, &xSkinNormal.z);
			VECScale(&xSkinNormal, &xSkinNormal, 0.015625f); // 1/64
#endif
		}

		// Get pointer to world position
		pxWorldPosition = (Vec*)&pxRenderVertex->m_xPosition;

        pxBoneMatrix     = &g_xBones[ pxBoneInfo->m_ucBoneIndex ];
        ucBoneInfoWeight = pxBoneInfo->m_ucBoneWeight;

		// Calculate normal
		if (m_bIncludeNormals)
		{
			MTXMultVecSR(*pxBoneMatrix, &xSkinNormal, (Vec*)&pxRenderVertex->m_xNormal); 
		}

		MTXMultVec(*pxBoneMatrix, &xSkinPosition, pxWorldPosition);

		// Check if there is more than one bone for this vertex
		if(ucBoneInfoWeight < 0xff)
		{
			if(!bRenderSimple)
			{
				// Scale first bone result by weight
				// Convert weight back into a float and scale
				OSu8tof32(&ucBoneInfoWeight, &fWeight);
				VECScale(pxWorldPosition, pxWorldPosition, fWeight * 0.00392157f);  // 1/255

			    // Tally weight until we reach one.
			    ucTotalWeight = ucBoneInfoWeight;
			    do
			    {
				    ++pxBoneInfo;
				    ucTotalWeight += pxBoneInfo->m_ucBoneWeight;

					MTXMultVec(g_xBones[pxBoneInfo->m_ucBoneIndex], &xSkinPosition, &xTempVector);

					// Convert weight back into a float and scale
					OSu8tof32(&pxBoneInfo->m_ucBoneWeight, &fWeight);
					VECScale(&xTempVector, &xTempVector, fWeight * 0.00392157f);    // 1/255
					VECAdd(pxWorldPosition, &xTempVector, pxWorldPosition);
			    } while (ucTotalWeight < 0xff);
			}
            else
            {
			    // Tally weight until we reach one.
			    ucTotalWeight = ucBoneInfoWeight;
			    do
			    {
				    ++pxBoneInfo;
				    ucTotalWeight += pxBoneInfo->m_ucBoneWeight;
			    } while (ucTotalWeight < 0xff);
            }
		}

		++pxBoneInfo;
        ++pxSkinVertex;
        ++pxRenderVertex;
	}
	Asura_GX_SmoothSkin_RenderBuffer::PostCreateVerts();
}

DEBUGVAR float dbg_fSimpleRenderDistance = 20.f;

void Asura_GX_SmoothSkin::BeginRender(u_int uCamVisFlags) const
{
	Asura_GX::Set3DProjection(true);

	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_CCW);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);

	// CPU skinning
	CalculateBoneOffsets();
	CalculateAndConvertVertexPositions( ShouldRenderSimple(uCamVisFlags) );

	// Setup arrays
	GXInvalidateVtxCache();
	GXClearVtxDesc();
	GXSetVtxDesc( GX_VA_POS,  GX_INDEX16 );
	if (m_bIncludeNormals)
	{
		GXSetVtxDesc( GX_VA_NRM,  GX_INDEX16 );
	}
	else
	{
		GXSetVtxDesc( GX_VA_NRM,  GX_NONE );
	}
	GXSetVtxDesc( GX_VA_TEX0, GX_INDEX16 );
	SetupArrays();
}

bool Asura_GX_SmoothSkin::ShouldRenderSimple(u_int uCamVisFlags) const
{
	// LOD calculation

	// Calculate closest camera
	bool bRenderSimple = true;
	for( int iCam = Asura_Camera::GetNumberOfCameras(); iCam--; )
	{
		if(Asura_Camera::IsCameraActive(iCam) && ((1 << iCam) & uCamVisFlags) )
		{
			Asura_Camera::SetCameraIndex( iCam );
			const Asura_Vector_3* pxCamPos = Asura_Camera::GetPosition();
			const float fDistance = (*pxCamPos - *Asura_Hierarchy_System::s_pxPos).Magnitude() * Asura_Camera::GetLODMetric(iCam);
			if(fDistance < dbg_fSimpleRenderDistance)
			{
				bRenderSimple = false;
			}
		}
	}
	return bRenderSimple;
}

void Asura_GX_SmoothSkin::SetupArrays() const
{
	Asura_GX_SmoothSkin_RenderBuffer::SetupArrays(m_bIncludeNormals	);
	GXSetArray( GX_VA_TEX0, (char*)m_pxTexInfo, sizeof( Asura_GX_SmoothSkin_TexInfo ) );
	if(m_pucColours)
	{
		GXSetArray( GX_VA_CLR0, (char*)m_pucColours, m_uColourDepth );
	}
}

void Asura_GX_SmoothSkin::Render(u_int uCamVisFlags, int iTexIDOverride /* = -1 */, bool bDepthOnly /* = false */, float fAlpha /* = 1.f */) const
{	
	if(bDepthOnly)
	{
		Asura_GX::SetupDepthOnly();
	}
	for(u_int uMaterial = 0; uMaterial < m_uNumberOfMaterials; ++uMaterial)
	{
		const Asura_GX_SmoothSkin_Material& xSkinMaterial = m_pxMaterials[uMaterial];
		const int iMaterialID = xSkinMaterial.m_iMaterialID;
		if(!bDepthOnly)
		{
			Asura_GX::SetupMaterial(iMaterialID, 0, 0, iTexIDOverride, fAlpha);
		}

		for( int iCam = Asura_Camera::GetNumberOfCameras(); iCam--; )
		{
			if (Asura_Camera::IsCameraActive(iCam) && ((1 << iCam) & uCamVisFlags))
			{
				Asura_Camera::SetCameraIndex( iCam );
				Asura_Render::ApplyCameraSettings();

				if(!bDepthOnly)
				{
					Asura_GX::SetUpEntityLights(*Asura_Hierarchy_System::s_pxPos);
					Asura_GX::SetUpEntitySpecular(*Asura_Hierarchy_System::s_pxPos, iMaterialID);
				}
				xSkinMaterial.m_xDisplayListBlock.DisplayList().Render();
				Asura_Render::AddToTrianglesRendered(xSkinMaterial.m_uNumTris, 1, ASURA_RENDER_TTYPE_SKIN);
			}
		}
	}
}
