//
// Asura_GX_TextureManagement.h
//
// Created by KOR - 09 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_GX_TEXTUREMANAGEMENT_H__
#define __ASURA_GX_TEXTUREMANAGEMENT_H__

// For loading texture palette files
#include <revolution/tpl.h>
#include "Core/Asura_TextureManagement.h"
#include "Rendering/Asura_RenderStates.h"

// KOR - 19/09/2003 21:00:44 - Had to switch this around, because of confusion over bitmap byte order
enum Asura_RGB_Channel
{
	Asura_Channel_Blue = 0,
	Asura_Channel_Green = 1,
	Asura_Channel_Red = 2,
};

// Texture can either have a pointer to a texture object
// or an index to another texture and a channel within the RGBtexture
// Not both! (but can have neither)
struct Asura_GX_AlphaMap
{
	GXTexObj*			m_pxTextureObject;
	int					m_iIndex;
	Asura_RGB_Channel	m_eChannel;
};

const u_int uASURA_GX_ALPHA_TEXTURE_INDEX_OFFSET = 4;

class Asura_GX_Texture_Management
{
	friend class Asura_Texture_Management;
public:
	static bool InitialiseAlphaMapping(Asura_Handle fileHandle);

	static void SetTextureIndex(int iTextureIndex, int iLoadedIndex);

	// Version of SetTextureIndex that over-rides the colour with a TLUT.
	// Will only work with intensity textures - using a S3TC will probably do Bad Things.
	static void SetTextureIndexTLUT(int iTextureIndex, int iLoadedIndex, GXTlutObj& xTlut);

	// Version of SetTextureIndex that turns off filtering on the texture
	static void SetTextureIndexUnfiltered(int iTextureIndex, int iLoadedIndex);

	static GXTexMapID GetAlphaTexMapID(int iTextureIndex = 0) { return static_cast<GXTexMapID>(GX_TEXMAP0 + iTextureIndex + uASURA_GX_ALPHA_TEXTURE_INDEX_OFFSET); }

	static bool IsColourTextureLoaded(int iTextureIndex = 0) { return s_aucTextureChannelsLoaded[iTextureIndex]; }
	static bool IsAlphaTextureLoaded(int iTextureIndex = 0) { return s_aucTextureChannelsLoaded[iTextureIndex] > 1; }
	static bool IsAlphaTextureDifferentRes(int iTextureIndex = 0) { return s_abAlphaTextureDifferentRes[iTextureIndex]; }

	static void SwizzleTexture(const u_char* pucSource, u_char* pucTarget, u_int uWidth, u_int uHeight);
private:
	// Helpers for custom SetTextureIndex
	static bool GetTLUTTexObjFromTPL( TPLPalettePtr pxTPL, GXTexObj *pxTexObj, GXTlutObj *pxTlutObj, GXTlut xTlut, u_int uIndex );
	static bool GetUnfilteredTexObjFromTPL( TPLPalettePtr pxTPL, GXTexObj *pxTexObj, u_int uIndex );
	static void LoadAlphaMap(int iTextureIndex, int iLoadedIndex);

	// Static members
	static GXTexObj*			s_apxTextureStorage[ASURA_MAX_NO_OF_TEXTURES];
	static Asura_GX_AlphaMap	s_axAlphaStorage[ASURA_MAX_NO_OF_TEXTURES];
	static TPLPalettePtr		s_apxTexturePaletteStorage[ASURA_MAX_NO_OF_TEXTURES];
	static u_char				s_aucTextureChannelsLoaded[MAX_TEXTURES_PER_POLYGON];
	static bool					s_abAlphaTextureDifferentRes[MAX_TEXTURES_PER_POLYGON];
};

#endif//__ASURA_GX_TEXTUREMANAGEMENT_H__