//
// Asura_AX.h
//
// Created by KOR - 10 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __ASURA_AX_H__
#define __ASURA_AX_H__

// Project
#include "Sound/Asura_ProjectSoundIDs.h"

// Library headers
#include <revolution/os.h>
#include <revolution/ax.h>

// TODO - work out max practical voices on Wii.
#define ASURA_WII_MAX_NUMBER_OF_ACTIVE_SOUNDS 48

// Test code to allow more voices by having less mix updates.
//#define ASURA_WII_DONT_UPDATE_SHORT_SOUNDS (1)

struct Asura_AX_LoadedSound
{
	DSPADPCM	m_xADPCMHeader;
	char*		m_pcADPCMData;	
};

struct Asura_AX_ActiveSound
{
	AXVPB*	m_pxAXVPB;
	u_int	m_uLoadedSoundIndex;
	bool	m_bIsPaused;
#ifdef ASURA_WII_DONT_UPDATE_SHORT_SOUNDS
	bool	m_bShouldUpdate;
#endif
};

class Asura_AX
{
public:
	static void Initialise();
	static void Shutdown();
	static void Update();
	static void Reset();
	static bool LoadSound(const char* filename, int index);
	static bool LoadSoundFromResource(Asura_Handle iFileHandle, int iFileSize, Asura_Handle iLoadedSoundIndex);
	static bool UnloadSound(Asura_Handle iSoundHandle);
	static bool IsSoundLoaded(Asura_Handle iSoundHandle);
	static bool CreateSound(Asura_Handle iActiveSoundHandle, Asura_Handle iLoadedSoundHandle);
	static bool PlaySound(Asura_Handle iSoundHandle, bool looping);
	static bool PauseSound(Asura_Handle iSoundHandle);
	static bool UnpauseSound(Asura_Handle iSoundHandle);
	static bool StopSound(Asura_Handle iSoundHandle);
	static bool DestroySound(Asura_Handle iSoundHandle);
	static bool IsSoundPlaying(Asura_Handle iSoundHandle);
	static bool SetVolume(Asura_Handle iSoundHandle, float fVolume);
	static bool SetPitch(Asura_Handle iSoundHandle, float fPitch);
	static bool SetPan(Asura_Handle iSoundHandle, float fPan);
	static bool SetFade(Asura_Handle iSoundHandle, float fFade);
	static void SetSoundFXVolume(float fVolume);
	static int  GetSoundFXVolume() { return s_iSoundFXVolume; }
	static void	VoiceDropCallback(void* pxAXPB);
	static u_int GetZeroBuffer() { return s_uZeroBuffer; }
	
	// Helpers for conversion between linear volume and decibels
	static float LinearVolumeToDecibels(float fLinear);
	static float DecibelsToLinearVolume(float fDecibels);
private:

	static Asura_AX_ActiveSound s_axActiveSounds[ASURA_WII_MAX_NUMBER_OF_ACTIVE_SOUNDS];
	static Asura_AX_LoadedSound* s_apxLoadedSounds[ASURA_MAX_NUMBER_OF_LOADED_SOUNDS];
	static char*		s_pcZeroBuffer;
	static u_int		s_uZeroBuffer;
	static int			s_iSoundFXVolume;
	static bool			s_bInitialised;
};


#endif//__ASURA_AX_H__