//
// Asura_GX_Fog.cpp
//
// Created by KOR - 29 November 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Asura_GX_Fog.h"

// Asura
#include "Rendering/Asura_Fog.h"
#include "Rendering/Asura_Render.h"

// Wii
#include "Rendering/Asura_GX.h"
#include "Rendering/Asura_GX_TextureManagement.h"

// Library headers
#include <revolution/gx.h>


// Asura_Fog platform implementation //////////////////////////////////////////////////////////////

void Asura_Fog::Platform_UpdateForNewParameters( void )
{
	Asura_GX_Fog::RequestUpdateForNewParameters();
}

void Asura_Fog::Platform_Render( void )
{
}

// Asura_GX_Fog static members and constants //////////////////////////////////////////////////////

u_short*	Asura_GX_Fog::s_pusZBuffer = NULL;
u_char*		Asura_GX_Fog::s_pucZMappingBuffer = NULL;
GXTexObj	Asura_GX_Fog::s_xZTexObj;
GXTexObj	Asura_GX_Fog::s_xZMappingTexObj;
bool		Asura_GX_Fog::s_bNeedsUpdate = true;

// Utility functions //////////////////////////////////////////////////////////////////////////////
static inline u_char FloatToUChar(float f)
{
	return static_cast<u_char>(Asura_Maths::ClampToRange(f, 0.f, 1.f) * 0xff);
}

// Asura_GX_Fog implementation ////////////////////////////////////////////////////////////////////

void Asura_GX_Fog::Initialise()
{
	// Allocate buffers
	const int iScreenWidth = Asura_GX::GetScreenWidth();
	const int iScreenHeight = Asura_GX::GetScreenHeight(); 
	// could use OSRoundUp32B instead?
	const int iImageHeight = iScreenHeight + ((iScreenHeight & 31) ? 32 - (iScreenHeight & 31) : 0);
	// could put these in MEM2?
	s_pusZBuffer	= static_cast<u_short*>	(Asura_AllocateAlign(iScreenWidth * iImageHeight * 2,ASURA_ALIGN_32_BYTE));
	s_pucZMappingBuffer = static_cast<u_char*>	(Asura_AllocateAlign(uASURA_GX_FOG_ZMAPPING_SIZE * uASURA_GX_FOG_ZMAPPING_SIZE * sizeof(u_char),ASURA_ALIGN_32_BYTE));
	Asura_Assert( s_pusZBuffer,		"Failed to allocate fog buffer");
	Asura_Assert( s_pucZMappingBuffer,"Failed to allocate fog buffer");
	GXInitTexObj( &s_xZTexObj,		s_pusZBuffer,	iScreenWidth, iScreenHeight, GX_TF_IA8,		GX_CLAMP, GX_CLAMP, GX_DISABLE );
	GXInitTexObj( &s_xZMappingTexObj,s_pucZMappingBuffer, uASURA_GX_FOG_ZMAPPING_SIZE, uASURA_GX_FOG_ZMAPPING_SIZE, GX_TF_I8,	GX_CLAMP, GX_CLAMP, GX_DISABLE );
	GXInitTexObjLOD( &s_xZTexObj,		GX_NEAR, GX_NEAR, 0.0f, 0.0f, 0.0f, GX_DISABLE, GX_DISABLE, GX_ANISO_1 );
	GXInitTexObjLOD( &s_xZMappingTexObj,GX_NEAR, GX_NEAR, 0.0f, 0.0f, 0.0f, GX_DISABLE, GX_DISABLE, GX_ANISO_1 );
}

void Asura_GX_Fog::Shutdown()
{
	// Release buffers
	Asura_DeallocateAlign(s_pusZBuffer);
	Asura_DeallocateAlign(s_pucZMappingBuffer);
}

DEBUGVAR bool dbg_bReportValues = false;

void Asura_GX_Fog::UpdateForNewParameters()
{
	// Recalculate palette based on graph

	// No point doing anything if fog is disabled.
	if(!Asura_Fog::IsFogEnabled()) { return; }

	// Fill out mapping object into temporary buffer.
	u_char* pucTempBuffer = static_cast<u_char*>	(Asura_AllocateAlign(256 * 256 * sizeof(u_char),ASURA_ALIGN_32_BYTE));
	for(u_int uY = 0; uY < uASURA_GX_FOG_ZMAPPING_SIZE; ++uY)
	{
		for(u_int uX = 0; uX < uASURA_GX_FOG_ZMAPPING_SIZE; ++uX)
		{
			const u_short usScreenZ = uY * uASURA_GX_FOG_ZMAPPING_SIZE + uX;
			const float fScreenZ = static_cast<float>(0xffff - usScreenZ) * (1.f / 0xffff);
			const float fWorldZ = (fASURA_GX_NEAR_CLIP * fASURA_GX_FAR_CLIP) / ((fASURA_GX_FAR_CLIP - fASURA_GX_NEAR_CLIP) * fScreenZ + fASURA_GX_NEAR_CLIP);
			const u_char ucValue = FloatToUChar(Asura_Fog::GetFogDensityAtDistance(fWorldZ));
			pucTempBuffer[uY * uASURA_GX_FOG_ZMAPPING_SIZE + uX] = ucValue;
			if(dbg_bReportValues && !(usScreenZ % 100))
			{
				OSReport("usScreenZ: %u,\tfScreenZ: %f,\tfWorldZ: %f,\tucValue: %u\n", usScreenZ, fScreenZ, fWorldZ, ucValue);
			}
		}
	}
	pucTempBuffer[uASURA_GX_FOG_ZMAPPING_SIZE * uASURA_GX_FOG_ZMAPPING_SIZE - 1] = FloatToUChar(Asura_Fog::GetSkyboxFogBlendFactor());

	// Now we need to swizzle it
	Asura_GX_Texture_Management::SwizzleTexture(pucTempBuffer, s_pucZMappingBuffer, uASURA_GX_FOG_ZMAPPING_SIZE, uASURA_GX_FOG_ZMAPPING_SIZE);
	// Done with the temporary buffer now.
	Asura_DeallocateAlign(pucTempBuffer);
}

void Asura_GX_Fog::Render()
{
	if(!Asura_Fog::IsFogEnabled()) { return; }

	if(s_bNeedsUpdate)
	{
		UpdateForNewParameters();
		s_bNeedsUpdate = false;
	}


	const int iScreenWidth = Asura_GX::GetScreenWidth();
	const int iScreenHeight = Asura_GX::GetScreenHeight();

	Asura_Render::AllowFullScreenRendering();

	// Copy Z buffer to texture
	const GXRenderModeObj& xRenderMode = Asura_GX::GetRenderModeObject();

	// Disable the filter as it's doing nasty things to the exact pixel values I'd like.
	GXSetCopyFilter(GX_FALSE, xRenderMode.sample_pattern, GX_FALSE, xRenderMode.vfilter);

	// Copy out eFB z plane as I8 texture format
	GXSetTexCopySrc( 0, 0, iScreenWidth, iScreenHeight );
	GXSetTexCopyDst(iScreenWidth, iScreenHeight, GX_TF_Z16, GX_DISABLE );
	GXCopyTex( s_pusZBuffer, GX_DISABLE );

	// Re-enable filter
	GXSetCopyFilter(GX_FALSE, xRenderMode.sample_pattern, GX_TRUE, xRenderMode.vfilter);

	// Be sure texture is completely copied out
	GXPixModeSync();

	// flush texture cache
	GXInvalidateTexAll();

	// Render fullscreen quad using texture and graph palette
	GXLoadTexObj( &s_xZMappingTexObj,		GX_TEXMAP3 );
	GXLoadTexObj( &s_xZTexObj,		GX_TEXMAP0 );


	GXSetNumChans(0);


	GXSetNumTexGens(1);
	GXSetTexCoordGen(GX_TEXCOORD0, GX_TG_MTX2x4, GX_TG_TEX0, GX_IDENTITY);
	
	GXSetNumTevStages(1);

	GXSetTevSwapMode(GX_TEVSTAGE0, GX_TEV_SWAP0, GX_TEV_SWAP0);
	GXSetTevSwapModeTable(GX_TEV_SWAP0, GX_CH_RED, GX_CH_GREEN, GX_CH_BLUE, GX_CH_ALPHA);

	GXSetNumIndStages(1);
	GXSetIndTexOrder(GX_INDTEXSTAGE0, GX_TEXCOORD0, GX_TEXMAP0);
	GXSetTevIndirect(GX_TEVSTAGE0, GX_INDTEXSTAGE0, GX_ITF_8, GX_ITB_NONE, GX_ITM_0, GX_ITW_0, GX_ITW_0, GX_FALSE, GX_FALSE, GX_ITBA_OFF);
	// Note that this is multiplied by 2^1 using the exponent parameter in GXSetIndTexMtx
	const float afIndTexMtx[2][3] =
	{
		{ .5f, 0.f, 0.f },
		{ 0.f, .5f, 0.f },
	};
	GXSetIndTexMtx(GX_ITM_0, afIndTexMtx, 1);
	// Load in Z as alpha, add to mask loaded in stage 3
	GXSetTevOrder( GX_TEVSTAGE0, GX_TEXCOORD_NULL, GX_TEXMAP3, GX_COLOR_NULL );

	// Fog colour
	const Asura_RGBA& xRGBAFogColour = Asura_Fog::GetFogColour();
	GXColor xGXFogColour;
	xGXFogColour.r = FloatToUChar(xRGBAFogColour.m_fR);
	xGXFogColour.g = FloatToUChar(xRGBAFogColour.m_fG);
	xGXFogColour.b = FloatToUChar(xRGBAFogColour.m_fB);
	xGXFogColour.a = 0xff;
	GXSetTevKColor(GX_KCOLOR0, xGXFogColour);
	GXSetTevKColorSel(GX_TEVSTAGE0, GX_TEV_KCSEL_K0);	
	GXSetTevColorIn( GX_TEVSTAGE0, GX_CC_ZERO, GX_CC_ZERO, GX_CC_ZERO, GX_CC_KONST);
	GXSetTevColorOp( GX_TEVSTAGE0, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_ENABLE, GX_TEVPREV );
	// Just use indirect texture alpha
	GXSetTevAlphaIn( GX_TEVSTAGE0, GX_CA_ZERO, GX_CA_ZERO, GX_CA_ZERO, GX_CA_TEXA );
	GXSetTevAlphaOp( GX_TEVSTAGE0, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_ENABLE, GX_TEVPREV );

	Asura_GX::Set3DProjection(false);
	Asura_Render::CurrentStates.RequestAlphaTest(true, 0);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
	Asura_Render::CurrentStates.RequestZBufferEnabled(false);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

#ifdef ASURA_USE_SCREENSHOT_SYSTEM
	// Make sure we are working in exact screenspace.
	GXSetViewport(0.f, 0.f, static_cast<float>(iScreenWidth), static_cast<float>(iScreenHeight), 0.f, 1.f);
#endif

	// vertex descriptor settings
	GXClearVtxDesc( );
	GXSetVtxDesc( GX_VA_POS, GX_DIRECT );
	GXSetVtxDesc( GX_VA_TEX0, GX_DIRECT );

	// Vertices
	GXBegin(GX_QUADS, GX_VTXFMT2, 4);
		GXPosition2f32(0.0F, 0.0F);
		GXTexCoord2f32(0.0F, 0.0F);
		GXPosition2f32((f32) iScreenWidth, 0.0F);
		GXTexCoord2f32(1.0F, 0.0F);
		GXPosition2f32((f32) iScreenWidth, (f32)iScreenHeight);
		GXTexCoord2f32(1.0F, 1.0F);
		GXPosition2f32(0.0F, (f32)iScreenHeight);
		GXTexCoord2f32(0.0F, 1.0F);
	GXEnd();


	GXSetNumIndStages(0);
	GXSetTevDirect( GX_TEVSTAGE0 );
	Asura_GX::Set3DProjection(true);
	Asura_GX::SetZBufferMode(true, ASURA_ZBUFFERCMP_LESSEQUAL, true);
	Asura_Render::CurrentStates.RequestTextureIndex(0, ASURA_TEXTURE_NONE);
	Asura_Render::CurrentStates.RequestTextureIndex(3, ASURA_TEXTURE_NONE);

#ifdef ASURA_USE_SCREENSHOT_SYSTEM
	// Set the viewport correctly again
	Asura_GX::Set3DProjection(false, true);
#endif
}

void Asura_GX_Fog::DisableHardwareFog()
{
	const GXColor xBlack = { 0, 0, 0, 0 };
	GXSetFog(GX_FOG_NONE, 0.f, 1.f, 0.f, 1.f, xBlack);
}

DEBUGVAR float dbg_fFudgeValue = 1000000.f;
void Asura_GX_Fog::SetHardwareFogFixed(float fAlpha, float fDistance, GXColor& xColour)
{
	if(fAlpha > 0.75f)
	{
		// Don't use the fudge, it doesn't seem to work very well.
		GXSetFog(GX_FOG_PERSP_LIN, 0.f, fDistance / fAlpha, fASURA_GX_NEAR_CLIP, fASURA_GX_FAR_CLIP, xColour);
	}
	else
	{
		// Calculate the slope so it is as shallow as possible, with the correct value at the desired distance
		GXSetFog(GX_FOG_PERSP_LIN, -dbg_fFudgeValue, (dbg_fFudgeValue + fDistance) / fAlpha, fASURA_GX_NEAR_CLIP, fASURA_GX_FAR_CLIP, xColour);
	}
}

void Asura_GX_Fog::SetHardwareFogLinear(float fNear, float fFar, GXColor& xColour)
{
	GXSetFog(GX_FOG_PERSP_LIN, fNear, fFar, fASURA_GX_NEAR_CLIP, fASURA_GX_FAR_CLIP, xColour);
}
