//
// Asura_Platform_Corona.cpp
//
// Created by KOR - 10 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "SpecialFX/Asura_Corona_System.h"

// Asura headers
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Render.h"

// Platform headers
#include "Rendering/Asura_GX.h"
#include "Rendering/Asura_GX_TextureManagement.h"

// Library headers
#include <revolution/gx.h>

void Asura_Corona_System::Platform_Initialise() {}
void Asura_Corona_System::Platform_Shutdown() {}

void Asura_Corona_System::Platform_RenderStart(void)
{
	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);			
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_GLOWING);
	Asura_Render::CurrentStates.RequestAlphaTest(false);

	// No lighting
	GXSetChanCtrl(GX_COLOR0A0, GX_DISABLE, GX_SRC_REG, GX_SRC_VTX, GX_LIGHT_NULL, GX_DF_CLAMP, GX_AF_NONE);
	
	GXClearVtxDesc();
	GXSetVtxDesc(GX_VA_POS, GX_DIRECT);
	GXSetVtxDesc(GX_VA_CLR0, GX_DIRECT);
	GXSetVtxDesc(GX_VA_TEX0, GX_DIRECT);
	// Initialize lighting, texgen, and tev parameters
	GXSetNumChans(1);

	// Use Unit Matrix in Texture Coordinate Generation
	GXSetTexCoordGen(GX_TEXCOORD0, GX_TG_MTX2x4, GX_TG_TEX0, GX_IDENTITY);
	GXSetTexCoordGen(GX_TEXCOORD1, GX_TG_MTX2x4, GX_TG_TEX0, GX_IDENTITY);
	// Number of Texture Coordinates to be Generated
	GXSetNumTexGens(1);
	// Channel Color Used
	GXSetNumChans(1);

	GXSetNumTevStages(1);
	GXSetTevOp(GX_TEVSTAGE0, GX_MODULATE);
	GXSetTevOrder(GX_TEVSTAGE0, GX_TEXCOORD0, GX_TEXMAP0, GX_COLOR0A0);
	GXSetTevSwapMode(GX_TEVSTAGE0, GX_TEV_SWAP0, GX_TEV_SWAP0);
#ifdef ASURA_ASSERT
	Asura_GX::SetRenderStatesValid(true);
#endif
}

void Asura_Corona_System::Platform_RenderCorona(const Asura_Corona* pxCorona, u_int uCamVisFlags)
{
	/// Kevin Lea - 27/01/2003 15:21:02 - add corona to a system of batch buffers based on texture
	/// to be later rendered by RenderEnd() (or on an ongoing basis if buffers fill up etc.


	/// Kevin Lea - 27/01/2003 15:24:57 - hacky first-pass renderer to test independent & tools code
	/// NOT INTENDED AS EXAMPLE CODE
	// KOR - 21/08/2003 16:12:28 - Don't have any other reference, so **** it, using as example code.
	
	Asura_Render::CurrentStates.RequestTextureIndex(0,pxCorona->m_iTextureIndex);
	
	if(!Asura_GX_Texture_Management::IsColourTextureLoaded())
	{
		Asura_Assert(0, "Could not find corona texture.");
#ifdef ASURA_ASSERT
		if(!Asura_GX::GetRenderStatesValid())
		{
			Platform_RenderStart();
		}
#endif
		return;
	}
	float fSize = pxCorona->m_fRadius;
	
	for(int i=0; i<Asura_Camera::GetNumberOfCameras(); i++)
	{
		if(!(uCamVisFlags & (1<<i))) continue;
		
		Asura_Camera::SetCameraIndex(i);
		Asura_Render::ApplyCameraSettings();


		const Asura_Vector_3*	pxCameraPosition = Asura_Camera::GetPosition();
		const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();

		Asura_Vector_3 xDeltaX = Asura_Vector_3(pxCameraOrientation->e11,pxCameraOrientation->e21,pxCameraOrientation->e31)*fSize;
		Asura_Vector_3 xDeltaY = Asura_Vector_3(pxCameraOrientation->e12,pxCameraOrientation->e22,pxCameraOrientation->e32)*fSize;

		Asura_Vector_3 xVertex[4];
		xVertex[3] = pxCorona->m_xPosition - xDeltaX - xDeltaY; 
		xVertex[2] = pxCorona->m_xPosition + xDeltaX - xDeltaY;
		xVertex[1] = pxCorona->m_xPosition + xDeltaX + xDeltaY;
		xVertex[0] = pxCorona->m_xPosition - xDeltaX + xDeltaY;

		float fLightScale = 255.0f; // KJL - 21/07/2003 15:03:24 - this is set to default for (m_bUseDirectionEffect==false)

		Asura_Vector_3 xDelta =  *pxCameraPosition - pxCorona->m_xPosition ;
		float fDistance = xDelta.Magnitude();

		if(pxCorona->m_bUseDirectionEffect)
		{
			if (fDistance>0.0f)
			{
				fLightScale = (pxCorona->m_xDirection * xDelta) / fDistance;

				if (fLightScale<0)
				{
					fLightScale = 64.0f;
				}
				else
				{
					fLightScale = 191.0f*fLightScale+64.0f;
				}
			}
			else
			{
				fLightScale = 0.0f;
			}
		}

		if( !pxCorona->m_bDoNotFadeOutWithDistance )
		{
			float fDistanceScale = fDistance - ASURA_CORONA_FADE_DIST_BEGIN;

			if (fDistanceScale > 0.0f)
			{
				fDistanceScale *= ( 1.0f / (ASURA_CORONA_FADE_DIST_END - ASURA_CORONA_FADE_DIST_BEGIN) );

				fLightScale *= (1.0f - fDistanceScale);
			}
		}

		const float fNearFadeDistance = pxCorona->GetNearFadeDistance();
		if(fNearFadeDistance)
		{
			Asura_Vector_3 xZDir = Asura_Vector_3(pxCameraOrientation->e13,pxCameraOrientation->e23,pxCameraOrientation->e33);
			float fZDistance = -(xDelta*xZDir);

			if(fZDistance<fNearFadeDistance)
			{
				fLightScale *= fZDistance/fNearFadeDistance;
				if(fLightScale<0) fLightScale = 0;
			}
		}

		unsigned int uColour = 0x000000ff	
						 + (Asura_Maths::FloatToInt(fLightScale*pxCorona->m_fRed)<<24)
						 + (Asura_Maths::FloatToInt(fLightScale*pxCorona->m_fGreen)<<16)
						 + (Asura_Maths::FloatToInt(fLightScale*pxCorona->m_fBlue)<<8);
		
		GXBegin(GX_TRIANGLEFAN, GX_VTXFMT3, 4);
			GXPosition3f32(xVertex[0].x, xVertex[0].y, xVertex[0].z);
			GXColor1u32(uColour);
			GXTexCoord2f32(0.0f, 1.0f);

			GXPosition3f32(xVertex[1].x, xVertex[1].y, xVertex[1].z);
			GXColor1u32(uColour);
			GXTexCoord2f32(1.0f, 1.0f);

			GXPosition3f32(xVertex[2].x, xVertex[2].y, xVertex[2].z);
			GXColor1u32(uColour);
			GXTexCoord2f32(1.0f, 0.0f);

			GXPosition3f32(xVertex[3].x, xVertex[3].y, xVertex[3].z);
			GXColor1u32(uColour);
			GXTexCoord2f32(0.0f, 0.0f);
		GXEnd();
		Asura_Render::AddToTrianglesRendered(2, 1, ASURA_RENDER_TTYPE_OTHER);
	}
}

void Asura_Corona_System::Platform_RenderEnd(void)
{
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
}
