//
// Asura_Platform_Decal.cpp
//
// Created by KOR - 10 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "SpecialFX/Asura_Decal_System.h"

#include "Camera/Asura_Camera.h"
//#include "Core/Asura_TextureManagement.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Decal_Template.h"

// Platform headers
#include "Rendering/Asura_GX.h"

// Library headers
#include <revolution/gx.h>
#include <revolution/mtx.h>

class Asura_GX_DecalVertex
{
public:
	float X,Y,Z;
	float U,V;

	inline bool operator == ( const Asura_GX_DecalVertex& v) const
	{
	    return (X == v.X && Y == v.Y && Z == v.Z
			 && U == v.U && V == v.V);
	}
};

class Asura_GX_DecalBuffer
{

	public:

	Asura_GX_DecalBuffer(const Asura_Decal_Template* pxDecalTemplate)
	{
		m_uMaxNumberOfIndices = 1000;
		m_uVertexStripIndex = 0;
		m_uNumberOfStripTris = 0;
	
		m_uCurrentIndex = 0;
		m_uMaxIndex = 0;

		m_pxDecalTemplate = pxDecalTemplate;
		m_pxNextDecalBuffer = 0;
	}

	~Asura_GX_DecalBuffer()
	{
		delete m_pxNextDecalBuffer;
	}

	void AddStrip(const Asura_Decal_Vertex* pxStrip, const u_int uNumberOfVertices)
	{
		if (m_uCurrentIndex + uNumberOfVertices+2 > m_uMaxNumberOfIndices)
		{
			m_uCurrentIndex = 0;
		}

		m_axVertices[m_uCurrentIndex].X = pxStrip[0].m_xPosition.x;
		m_axVertices[m_uCurrentIndex].Y = pxStrip[0].m_xPosition.y;
		m_axVertices[m_uCurrentIndex].Z = pxStrip[0].m_xPosition.z;
		m_axVertices[m_uCurrentIndex].U = pxStrip[0].m_fU;
		m_axVertices[m_uCurrentIndex].V = pxStrip[0].m_fV;
		m_uCurrentIndex++;

		for(u_int i = 0; i < uNumberOfVertices; i++)
		{
			u_int v = i + m_uCurrentIndex;
			m_axVertices[v].X = pxStrip[i].m_xPosition.x;
			m_axVertices[v].Y = pxStrip[i].m_xPosition.y;
			m_axVertices[v].Z = pxStrip[i].m_xPosition.z;
			m_axVertices[v].U = pxStrip[i].m_fU;
			m_axVertices[v].V = pxStrip[i].m_fV;
		}
		
		m_uCurrentIndex += uNumberOfVertices;
		
		m_axVertices[m_uCurrentIndex].X = pxStrip[uNumberOfVertices-1].m_xPosition.x;
		m_axVertices[m_uCurrentIndex].Y = pxStrip[uNumberOfVertices-1].m_xPosition.y;
		m_axVertices[m_uCurrentIndex].Z = pxStrip[uNumberOfVertices-1].m_xPosition.z;
		m_axVertices[m_uCurrentIndex].U = pxStrip[uNumberOfVertices-1].m_fU;
		m_axVertices[m_uCurrentIndex].V = pxStrip[uNumberOfVertices-1].m_fV;
		m_uCurrentIndex++;
			
		// KJL - 29/04/2003 10:32:10 - if wrapped we have to tidy up any decals we've half-overwritten
		if (m_uCurrentIndex < m_uMaxIndex)
		{
			u_int u = m_uCurrentIndex;

			while (!(m_axVertices[u] == m_axVertices[u+1]) && u < m_uMaxIndex-1)
			{
				m_axVertices[u].X = m_axVertices[u-1].X;
				m_axVertices[u].Y = m_axVertices[u-1].Y;
				m_axVertices[u].Z = m_axVertices[u-1].Z;
				m_axVertices[u].U = m_axVertices[u-1].U;			
				m_axVertices[u].V = m_axVertices[u-1].V;
				u++;
			}
		}
		else
		{
			m_uMaxIndex = m_uCurrentIndex;
		}
		
		if (m_uCurrentIndex>m_uNumberOfStripTris+2)
		{
			m_uNumberOfStripTris = m_uCurrentIndex - 2;
		}
	}

	void Render(void)
	{
		Asura_Render::CurrentStates.RequestTextureIndex(0,m_pxDecalTemplate->m_iTextureIndex);

		for(int i=0; i<Asura_Camera::GetNumberOfCameras(); i++)
		{
			Asura_Camera::SetCameraIndex(i);

			if (Asura_Camera::IsCameraActive())
			{
				Asura_Render::ApplyCameraSettings();

				// Set up projection matrix to shift z towards camera
				//float fEpsilon = -1.0f;
				const Mtx44& xDefaultProjMatrix = Asura_GX::GetProjMatrix();
				Mtx44 xProjMatrix;
				MTXCopy(xDefaultProjMatrix, xProjMatrix);
				//MTXRowCol(xProjMatrix, 2, 2) *= 1.0f + fEpsilon;
				MTXRowCol(xProjMatrix, 2, 2) = 0.0f;
				GXSetProjection(xProjMatrix, GX_PERSPECTIVE);

				GXBegin(GX_TRIANGLESTRIP, GX_VTXFMT3, m_uNumberOfStripTris + 2);
				{
					for(unsigned int u = 0; u < m_uNumberOfStripTris + 2; u++)
					{
						GXPosition3f32(m_axVertices[u].X, m_axVertices[u].Y, m_axVertices[u].Z);
						GXTexCoord2f32(m_axVertices[u].U, m_axVertices[u].V);
					}
				}
				GXEnd();

				Asura_Render::AddToTrianglesRendered(m_uNumberOfStripTris, 1, ASURA_RENDER_TTYPE_OTHER);
				GXSetProjection(xDefaultProjMatrix, GX_PERSPECTIVE);
			}
		}

		if (m_pxNextDecalBuffer)
		{
			m_pxNextDecalBuffer->Render();
		}
	}

	Asura_GX_DecalVertex m_axVertices[1024];
	u_int m_uMaxNumberOfIndices;
	u_int m_uVertexStripIndex;
	u_int m_uNumberOfStripTris;

	u_int m_uCurrentIndex;
	u_int m_uMaxIndex;

	const Asura_Decal_Template* m_pxDecalTemplate;

	Asura_GX_DecalBuffer* m_pxNextDecalBuffer;
};

Asura_GX_DecalBuffer* g_pxFirstDecalBuffer;

void Asura_Decal_System::Platform_Initialise(void)
{
	g_pxFirstDecalBuffer = 0;
}

void Asura_Decal_System::Platform_Reset(void)
{
	delete g_pxFirstDecalBuffer;
	g_pxFirstDecalBuffer = 0;
}

void Asura_Decal_System::Platform_Shutdown(void)
{
	delete g_pxFirstDecalBuffer;
	g_pxFirstDecalBuffer = 0;
}

/// Kevin Lea - 19/11/2002 17:07:34 - test renderer for proof of concept: (very slow!)

void Asura_Decal_Handler::Platform_AddStrip(const Asura_Decal_Vertex* pxStrip, const u_int uNumberOfVertices, const Asura_RGBA& xDiffuseRGBA, const Asura_Decal_Template* pxDecalTemplate)
{
	// TODO - use colour
	Asura_GX_DecalBuffer* pxDecalBuffer = g_pxFirstDecalBuffer;
		
	while (pxDecalBuffer)
	{
		if (pxDecalBuffer->m_pxDecalTemplate == pxDecalTemplate)
		{
			pxDecalBuffer->AddStrip(pxStrip,uNumberOfVertices);
			return;
		}
		pxDecalBuffer = pxDecalBuffer->m_pxNextDecalBuffer;
	}

	pxDecalBuffer = new Asura_GX_DecalBuffer(pxDecalTemplate);
	pxDecalBuffer->m_pxNextDecalBuffer = g_pxFirstDecalBuffer;
	g_pxFirstDecalBuffer = pxDecalBuffer;

	pxDecalBuffer->AddStrip(pxStrip,uNumberOfVertices);
	
}

void Asura_Decal_Handler::Platform_Render(void)
{
	if (g_pxFirstDecalBuffer == 0) return;

	GXClearVtxDesc();
	GXSetVtxDesc(GX_VA_POS, GX_DIRECT);
	GXSetVtxDesc(GX_VA_TEX0, GX_DIRECT);
	GXSetNumChans(1);
	GXSetNumTexGens(1);
	GXSetNumTevStages(1);
	
	GXSetTevSwapMode(GX_TEVSTAGE0, GX_TEV_SWAP0, GX_TEV_SWAP0);
	GXSetTexCoordGen(GX_TEXCOORD0, GX_TG_MTX2x4, GX_TG_TEX0, GX_IDENTITY);
	GXSetTevOrder( GX_TEVSTAGE0, GX_TEXCOORD0, GX_TEXMAP0, GX_COLOR0A0);
	GXSetTevOp(GX_TEVSTAGE0, GX_DECAL);
	
	Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_DOUBLECOLOUR);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);

	g_pxFirstDecalBuffer->Render();

	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
}