//
// Asura_Platform_PFX.cpp
//
// Created by KOR - 10 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "SpecialFX/PFX/Asura_PFX_Effect.h"

// Asura headers
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Timers.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/PFX/Asura_PFX_Classes.h"
#include "SpecialFX/PFX/Asura_PFX_Templates.h"

// Wii headers
#include "Rendering/Asura_GX.h"
#include "Rendering/Asura_GX_TextureManagement.h"

// Library headers
#include <revolution/gx.h>

#ifndef ASURA_PFX_SYSTEM_SUSPICIOUS_POSITION_THRESHOLD
    // The position beyond which a particle position is considered to be suspicious.
    #define	ASURA_PFX_SYSTEM_SUSPICIOUS_POSITION_THRESHOLD  ( 10000.0f )
#endif 

void Asura_PFX_System::Platform_Initialise( void )
{
}

void Asura_PFX_System::Platform_Reset( void )
{
}

void Asura_PFX_System::Platform_Shutdown( void )
{
}

DEBUGVAR bool dbg_bDisablePFX = false;
void Asura_PFX_System::Platform_BeginRender()
{
	if(dbg_bDisablePFX) { return; }

	Asura_Render::CurrentStates.RequestZBufferEnabled(true);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
	Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
	Asura_Render::CurrentStates.RequestAlphaTest(true, 1);
	if(s_eCurrentRenderType == ASURA_PFX_RENDER_SKYBOX)
	{
		// Use offset to ensure that skybox is always rendered at farthest Z.
		GXSetZScaleOffset(1.f, 1.f);
	}

	// No lighting
	GXSetChanCtrl(GX_COLOR0A0, GX_DISABLE, GX_SRC_REG, GX_SRC_VTX, GX_LIGHT_NULL, GX_DF_CLAMP, GX_AF_NONE);

	GXClearVtxDesc();
	GXSetVtxDesc(GX_VA_POS, GX_DIRECT);
	GXSetVtxDesc(GX_VA_CLR0, GX_DIRECT);
	GXSetVtxDesc(GX_VA_TEX0, GX_DIRECT);

	GXSetTevSwapMode(GX_TEVSTAGE0, GX_TEV_SWAP0, GX_TEV_SWAP0);
	// Use Unit Matrix in Texture Coordinate Generation
	GXSetTexCoordGen(GX_TEXCOORD0, GX_TG_MTX2x4, GX_TG_TEX0, GX_IDENTITY);
	GXSetTexCoordGen(GX_TEXCOORD1, GX_TG_MTX2x4, GX_TG_TEX0, GX_IDENTITY);
	// Number of Texture Coordinates to be Generated
	GXSetNumTexGens(1);
	// Channel Color Used
	GXSetNumChans(1);
	// Number of TEV Stages Used
	GXSetNumTevStages(2);

	// Stage 0 hands over the CMPR Texture Colors to the subsequent stage without modification.
	GXSetTevOrder( GX_TEVSTAGE0, GX_TEXCOORD0, GX_TEXMAP0, GX_COLOR0A0);
	// Input = Tex colour, raster colour
	GXSetTevColorIn( GX_TEVSTAGE0, GX_CC_ZERO, GX_CC_TEXC, GX_CC_RASC, GX_CC_ZERO );
	// Output Color = Tex colour * raster colour
	GXSetTevColorOp( GX_TEVSTAGE0, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_ENABLE, GX_TEVPREV );
	// Input = 0
	GXSetTevAlphaIn( GX_TEVSTAGE0, GX_CA_ZERO, GX_CA_ZERO, GX_CA_ZERO, GX_CA_ZERO );
	// Output = 0
	GXSetTevAlphaOp( GX_TEVSTAGE0, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_ENABLE, GX_TEVPREV );

	GXSetTevSwapMode(GX_TEVSTAGE1, GX_TEV_SWAP0, GX_TEV_SWAP1);
	// Stage 1 outputs the colors it receives from the previous stage unchanged, and 
	// substitutes the alphas with the input from the I8 texture * raster alpha
	GXSetTevOrder( GX_TEVSTAGE1, GX_TEXCOORD0, Asura_GX_Texture_Management::GetAlphaTexMapID(), GX_COLOR0A0);
	// Input = Previous Stage Output Color
	GXSetTevColorIn( GX_TEVSTAGE1, GX_CC_ZERO, GX_CC_ZERO, GX_CC_ZERO, GX_CC_CPREV );
	// Output Color = Previous Stage Output Color
	GXSetTevColorOp( GX_TEVSTAGE1, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_ENABLE, GX_TEVPREV );
	// Input = I8 Texture Alpha
	GXSetTevAlphaIn( GX_TEVSTAGE1, GX_CA_ZERO, GX_CA_TEXA, GX_CA_RASA, GX_CA_ZERO );
	// Output Alpha = I8 Texture Alpha
	GXSetTevAlphaOp( GX_TEVSTAGE1, GX_TEV_ADD, GX_TB_ZERO, GX_CS_SCALE_1, GX_ENABLE, GX_TEVPREV );
#ifdef ASURA_ASSERT
	Asura_GX::SetRenderStatesValid(true);
#endif
}

void Asura_PFX_System::Platform_EndRender()
{
	// Reset z offset.
	GXSetZScaleOffset(1.f, 0.f);
}

bool Asura_PFX_System::Platform_SupportsHeatHaze()
{
	return false;
}



void Asura_PFX_Effect::Platform_RenderParticles( const Asura_PFX_Particle* pxParticles,
                                                 u_int uNumParticles,
                                                 const Asura_PFX_Particle_Template* pxTemplate,
                                                 const Asura_RGBA& xLightColour ) const
{
	if(dbg_bDisablePFX) { return; }
	// It is valid to pass in zero particles (source could just have started or ended. Simply return)
	if(!uNumParticles) return;

	Asura_Assert(pxParticles, "Null particle pointer given");
	Asura_Assert(pxTemplate, "Null particle template pointer given");

	if(!pxTemplate || !pxParticles) return;

#ifdef ASURA_ASSERT
	if(!Asura_GX::GetRenderStatesValid())
	{
		Asura_PFX_System::Platform_BeginRender();
	}
#endif

	Asura_Render::CurrentStates.RequestTextureIndex( 0, pxTemplate->m_iTextureHandle );

	if(m_usPFXFlags & ASURA_PFX_EFFECT_ALWAYS_ON_TOP)
	{
		Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_ALWAYS);
	}
	else
	{
		Asura_Render::CurrentStates.RequestZBufferCompareMode(ASURA_ZBUFFERCMP_LESSEQUAL);
	}
	Asura_Render::CurrentStates.RequestTranslucencyMode(pxTemplate->m_eTranslucency);
	Asura_Render::CurrentStates.RequestZBufferWriteEnabled(pxTemplate->m_eTranslucency == ASURA_TRANSLUCENCY_OFF);

	Asura_Vector_3 axVertices[4];

	for( int iCam=0; iCam<Asura_Camera::GetNumberOfCameras(); ++iCam )
	{
		if(!Asura_Camera::IsCameraActive(iCam) || !(m_ucCameraFlags & (1<<iCam))) { continue; }

		Asura_Camera::SetCameraIndex(iCam);
		Asura_Render::ApplyCameraSettings();

		const Asura_Vector_3* pxCameraPos = Asura_Camera::GetPosition();
		Asura_Vector_3 axPositions[4];
		Asura_Vector_2 axTexCoords0[4];
		Asura_Vector_2 axTexCoords1[4];

		for(u_int uParticle = 0 ; uParticle < uNumParticles; ++uParticle)
		{
			const	Asura_PFX_Particle*	pxParticle = &pxParticles[uParticle];
					Asura_Vector_3		xPosition	= pxParticle->m_xPosition;

			if ( m_usPFXFlags & ASURA_PFX_EFFECT_SKYBOX ) xPosition += *pxCameraPos;
        
			Asura_RGBA xColour = pxTemplate->m_xColour.GetValueAtTime_Discrete(pxParticle->m_fAge);
	        if(pxTemplate->m_bUseLighting)
			{
				xColour.m_fR *= xLightColour.m_fR;
				xColour.m_fG *= xLightColour.m_fG;
				xColour.m_fB *= xLightColour.m_fB;
			}

			if(pxTemplate->m_bNearCameraFade || pxTemplate->m_bFarCameraFade)
			{
				float fDist = (*pxCameraPos - xPosition).Magnitude();
					//fade when near camera
				if(fDist < pxTemplate->m_fNearFadeDistance && pxTemplate->m_bNearCameraFade)
				{
					if( (pxTemplate->m_fNearFadeDistance - pxTemplate->m_fNearZeroDistance) == 0.0f )
					{ 
						xColour.m_fA = 0.0f;
					}
					else
					{
						xColour.m_fA *= (fDist - pxTemplate->m_fNearZeroDistance) / (pxTemplate->m_fNearFadeDistance - pxTemplate->m_fNearZeroDistance);
					}					
				}
				//fade when far from camera
				if(fDist > pxTemplate->m_fFarFadeDistance && pxTemplate->m_bFarCameraFade)
				{
					if( (pxTemplate->m_fFarFadeDistance - pxTemplate->m_fFarZeroDistance) == 0.0f )
					{
						xColour.m_fA = 0.0f;
					}
					else
					{
						xColour.m_fA *= (fDist - pxTemplate->m_fFarZeroDistance) / (pxTemplate->m_fFarFadeDistance - pxTemplate->m_fFarZeroDistance);
					}					
				}
			}
			// KOR - 15/11/2005 - early out on zero alpha
			if(xColour.m_fA <= 0.f)
			{
				continue;
			}

			float fSize = pxTemplate->m_xSize.GetValueAtTime_Discrete(pxParticle->m_fAge);
			if( pxTemplate->m_bUseSizeScale )
			{
				fSize *= GetSizeScale();
			}

			fSize *= (1.f - pxParticle->m_fSeed * pxTemplate->m_xSizeVariation.GetValueAtTime_Discrete(pxParticle->m_fAge));
			const float fAgeInSeconds = pxParticle->m_fAge * pxTemplate->GetLifetimeFromSeed(pxParticle->m_fSeed);
			const float fLength = Asura_Maths::Min(pxTemplate->m_fBlurLength, fAgeInSeconds);
			const Asura_Vector_3 xPrevPosition = xPosition - (pxParticle->m_xVelocity * fLength);

			Asura_Vector_3 xDeltaX;
			Asura_Vector_3 xDeltaY;

			float fTheta;

			if(pxTemplate->m_bFixedRotation)
			{
				fTheta = Asura_Maths::DegToRad(pxTemplate->m_fFixedRotation);
				if(pxTemplate->m_uNumFixedRotations > 1)
				{
					int iRotation = Asura_Maths::FloatToInt_Chop(pxParticle->m_fSeed * pxTemplate->m_uNumFixedRotations);
					fTheta += (iRotation * Asura_Maths::Pi * 2.f) / pxTemplate->m_uNumFixedRotations;
				}
			}
			else if(pxTemplate->m_b3DRotation || !pxTemplate->m_bUseMotionBlur)
			{
				// Need a rotation
				if(pxTemplate->m_bUseRandomRotation)
				{
					fTheta = ((Asura_Timers::GetGameTimeScale() == 0.0f) ? 0.0f : GetRNG()->GetFloat()) * Asura_Maths::Pi * 2.0f;
				}
				else
				{
	#ifdef ASURA_TOOLS
					fTheta = (pxParticle->m_fSeed + pxTemplate->m_xRotation.GetAreaUnderGraph(pxParticle->m_fAge)) * Asura_Maths::Pi * 2.f;
	#else
					fTheta = (pxParticle->m_fSeed + pxTemplate->m_xRotation.GetValueAtTime_Discrete(pxParticle->m_fAge)) * Asura_Maths::Pi * 2.f;
	#endif
				}
			
				if(pxTemplate->m_bMirrorRotation)
				{
					const int* piRandom = reinterpret_cast<const int*>(&pxParticle->m_fSeed);
					if(*piRandom % 2)
					{
						fTheta = -fTheta;
					}
				}
			}

			const Asura_Matrix_3x3* pxOrientation = GetOrientation();
			if(pxTemplate->m_b3DRotation)
			{
				// KOR - 9/9/2004 - not optimal, and possibly not the way we want to do 3D rotation
				// (intended for effects such as glass shards or scattering debris). Need artist feedback to improve.
				const Asura_Euler_3 xEuler(fTheta, fTheta, 0.f);
				Asura_Matrix_3x3 xMatrix;
				xEuler.ToMatrix(xMatrix);
				xDeltaX = Asura_Vector_3(xMatrix.e11, xMatrix.e21, xMatrix.e31) * fSize;
				xDeltaY = Asura_Vector_3(xMatrix.e12, xMatrix.e22, xMatrix.e32) * fSize;
			}
			else if(pxTemplate->m_bAlignEffectX && pxOrientation)
			{
				xDeltaX = Asura_Vector_3(pxOrientation->e12, pxOrientation->e22, pxOrientation->e32) * fSize;
				xDeltaY = Asura_Vector_3(pxOrientation->e13, pxOrientation->e23, pxOrientation->e33) * fSize;
			}
			else if(pxTemplate->m_bAlignEffectY && pxOrientation)
			{
				xDeltaX = Asura_Vector_3(pxOrientation->e11, pxOrientation->e21, pxOrientation->e31) * fSize;
				xDeltaY = Asura_Vector_3(pxOrientation->e13, pxOrientation->e23, pxOrientation->e33) * fSize;
			}
			else if(pxTemplate->m_bAlignEffectZ && pxOrientation)
			{
				xDeltaX = Asura_Vector_3(pxOrientation->e11, pxOrientation->e21, pxOrientation->e31) * fSize;
				xDeltaY = Asura_Vector_3(pxOrientation->e12, pxOrientation->e22, pxOrientation->e32) * fSize;
			}
			else if(pxTemplate->m_bBillboardJustY && pxOrientation)
			{
				const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();
				xDeltaX = Asura_Vector_3(pxCameraOrientation->e11,pxCameraOrientation->e21,pxCameraOrientation->e31) * fSize;
				xDeltaY = Asura_Vector_3(pxOrientation->e13, pxOrientation->e23, pxOrientation->e33) * fSize;
			}
			else if(pxTemplate->m_bBillboardJustX && pxOrientation)
			{
				const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();
				xDeltaX = Asura_Vector_3(pxOrientation->e12, pxOrientation->e22, pxOrientation->e32) * fSize;
				xDeltaY = Asura_Vector_3(pxCameraOrientation->e12,pxCameraOrientation->e22,pxCameraOrientation->e32) * fSize;
			}
			else
			{
				const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();
				xDeltaX = Asura_Vector_3(pxCameraOrientation->e11,pxCameraOrientation->e21,pxCameraOrientation->e31) * fSize;
				xDeltaY = Asura_Vector_3(pxCameraOrientation->e12,pxCameraOrientation->e22,pxCameraOrientation->e32) * fSize;
			}

			// JGP - moved the getting of frames to here

			
				// get the texture frame(s) for this particle
				const Asura_PFX_TextureFrame* pxTextureFrame;
				const Asura_PFX_TextureFrame* pxTextureFrame2;
				float fWeight = 1.f;
				if (pxTemplate->m_bCrossfadeTextureFrames)
				{
					pxTemplate->GetTextureFramesForCrossfadeAtTime(pxParticle->m_fAge, pxTextureFrame, pxTextureFrame2, fWeight);
					// weight needs to be replicated across RGBA
					//u_int uWeight = Asura_Maths::FloatToInt(fWeight * 255.0f);
					//uWeight = (uWeight << 24) | (uWeight << 16) | (uWeight << 8) | (uWeight << 0);
					// set the amount of blending between texture frames for this particle
				}
				else
				{
					pxTextureFrame = pxTemplate->GetTextureFrameAtTime(pxParticle->m_fAge);
					pxTextureFrame2 = NULL;
				}

			// JGP aspect ratio from UV

			if ( pxTemplate->m_bUseUVAsAspect )
			{
				
				const float fDU = pxTextureFrame->m_afTextureUVs[ 2 ] - pxTextureFrame->m_afTextureUVs[ 0 ];
				const float fDV = pxTextureFrame->m_afTextureUVs[ 3 ] - pxTextureFrame->m_afTextureUVs[ 1 ];
				const float fAspect = ( fDV ) ? fDU / fDV : 1.0f;
				xDeltaX *= fAspect;
			}

			/// Kevin Lea - 27/03/2002 11:43:15 - code to draw particle stretched along its path
			/// ucky and needs rewriting
			if (pxTemplate->m_bUseMotionBlur)
			{
				Asura_Vector_3 xBeta;

				xBeta = xPosition - xPrevPosition;
				if( xBeta.MagnitudeSquared() > (0.1f*0.1f) )
				{
					xBeta.Normalise();
					xBeta*=0.1f;
				}
				
				if (Asura_Maths::Abs(xBeta * xDeltaX) > Asura_Maths::Abs(xBeta*xDeltaY))
				{
					if (xBeta * xDeltaX > 0)
					{
						axVertices[0] = xPrevPosition	- xBeta - xDeltaX - xDeltaY; 
						axVertices[1] = xPosition		+ xBeta + xDeltaX - xDeltaY;
						axVertices[2] = xPosition		+ xBeta + xDeltaX + xDeltaY;
						axVertices[3] = xPrevPosition	- xBeta - xDeltaX + xDeltaY;
					}
					else
					{
						axVertices[0] = xPosition		+ xBeta - xDeltaX - xDeltaY; 
						axVertices[1] = xPrevPosition	- xBeta + xDeltaX - xDeltaY;
						axVertices[2] = xPrevPosition	- xBeta + xDeltaX + xDeltaY;
						axVertices[3] = xPosition		+ xBeta - xDeltaX + xDeltaY;
					}
				}
				else
				{
					if (xBeta * xDeltaY > 0)
					{
						axVertices[0] = xPrevPosition	- xBeta - xDeltaX - xDeltaY; 
						axVertices[1] = xPrevPosition	- xBeta + xDeltaX - xDeltaY;
						axVertices[2] = xPosition		+ xBeta + xDeltaX + xDeltaY;
						axVertices[3] = xPosition		+ xBeta - xDeltaX + xDeltaY;
					}
					else
					{
						axVertices[0] = xPosition		+ xBeta - xDeltaX - xDeltaY; 
						axVertices[1] = xPosition		+ xBeta + xDeltaX - xDeltaY;
						axVertices[2] = xPrevPosition	- xBeta + xDeltaX + xDeltaY;
						axVertices[3] = xPrevPosition	- xBeta - xDeltaX + xDeltaY;
					}
				}
			}
			else 
			{
				if (fTheta!=0.f)
				{
					const float fSin = Asura_Maths::Sine(fTheta);
					const float fCos = Asura_Maths::Cosine(fTheta);
		
					Asura_Vector_3 xCosDeltaX = xDeltaX * fCos;
					Asura_Vector_3 xSinDeltaX = xDeltaX * fSin;
					Asura_Vector_3 xCosDeltaY = xDeltaY * fCos;
					Asura_Vector_3 xSinDeltaY = xDeltaY * fSin;
				
					axVertices[0] = xPosition + (-xCosDeltaX + xSinDeltaX) + (-xSinDeltaY - xCosDeltaY);		
					axVertices[1] = xPosition + ( xCosDeltaX + xSinDeltaX) + ( xSinDeltaY - xCosDeltaY);
					axVertices[2] = xPosition + ( xCosDeltaX - xSinDeltaX) + ( xSinDeltaY + xCosDeltaY);
					axVertices[3] = xPosition + (-xCosDeltaX - xSinDeltaX) + (-xSinDeltaY + xCosDeltaY);
				}
				else
				{
					axVertices[0] = xPosition - xDeltaX - xDeltaY; 
					axVertices[1] = xPosition + xDeltaX - xDeltaY;
					axVertices[2] = xPosition + xDeltaX + xDeltaY;
					axVertices[3] = xPosition - xDeltaX + xDeltaY;
				}
			}


			// are we changing texture orient?
			u_int uTexOrient = 0;
			if(pxTemplate->m_bRandomTextureOrient)
			{
				uTexOrient = Asura_Maths::FloatToInt_Chop(pxParticle->m_fSeed * 4.f);
			}

			switch(uTexOrient)
			{
				case 0:
				case 1:
					// Normal
					axTexCoords0[0].x = axTexCoords0[3].x	= pxTextureFrame->m_afTextureUVs[0];
					axTexCoords0[1].x = axTexCoords0[2].x	= pxTextureFrame->m_afTextureUVs[2];
					if (pxTextureFrame2)
					{
						axTexCoords1[0].x = axTexCoords1[3].x	= pxTextureFrame2->m_afTextureUVs[0];
						axTexCoords1[1].x = axTexCoords1[2].x	= pxTextureFrame2->m_afTextureUVs[2];
					}
					break;
				case 2:
				case 3:
				default:
					// Mirrored
					axTexCoords0[0].x = axTexCoords0[3].x	= pxTextureFrame->m_afTextureUVs[2];
					axTexCoords0[1].x = axTexCoords0[2].x	= pxTextureFrame->m_afTextureUVs[0];
					if (pxTextureFrame2)
					{
						axTexCoords1[0].x = axTexCoords1[3].x	= pxTextureFrame2->m_afTextureUVs[2];
						axTexCoords1[1].x = axTexCoords1[2].x	= pxTextureFrame2->m_afTextureUVs[0];
					}
					break;
			}
			switch(uTexOrient)
			{
				case 0:
				case 3:
					// Normal
					axTexCoords0[0].y = axTexCoords0[1].y	= pxTextureFrame->m_afTextureUVs[1];
					axTexCoords0[2].y = axTexCoords0[3].y	= pxTextureFrame->m_afTextureUVs[3];
					if (pxTextureFrame2)
					{
						axTexCoords1[0].y = axTexCoords1[1].y	= pxTextureFrame2->m_afTextureUVs[1];
						axTexCoords1[2].y = axTexCoords1[3].y	= pxTextureFrame2->m_afTextureUVs[3];
					}
					break;
				case 1:
				case 2:
				default:
					// Mirrored
					axTexCoords0[0].y = axTexCoords0[1].y	= pxTextureFrame->m_afTextureUVs[3];
					axTexCoords0[2].y = axTexCoords0[3].y	= pxTextureFrame->m_afTextureUVs[1];
					if (pxTextureFrame2)
					{
						axTexCoords1[0].y = axTexCoords1[1].y	= pxTextureFrame2->m_afTextureUVs[3];
						axTexCoords1[2].y = axTexCoords1[3].y	= pxTextureFrame2->m_afTextureUVs[1];
					}
					break;
			}

			Asura_RGBA xColour0 = xColour;
			xColour0.m_fA *= fWeight;
			const unsigned int uDiffuse0 = xColour0.ToUIntRGBA(true);

			GXBegin(GX_TRIANGLEFAN, GX_VTXFMT3, 4);
				GXPosition3f32(axVertices[0].x, axVertices[0].y, axVertices[0].z);
				GXColor1u32(uDiffuse0);
				GXTexCoord2f32(axTexCoords0[0].x, axTexCoords0[0].y);

				GXPosition3f32(axVertices[1].x, axVertices[1].y, axVertices[1].z);
				GXColor1u32(uDiffuse0);
				GXTexCoord2f32(axTexCoords0[1].x, axTexCoords0[1].y);

				GXPosition3f32(axVertices[2].x, axVertices[2].y, axVertices[2].z);
				GXColor1u32(uDiffuse0);
				GXTexCoord2f32(axTexCoords0[2].x, axTexCoords0[2].y);

				GXPosition3f32(axVertices[3].x, axVertices[3].y, axVertices[3].z);
				GXColor1u32(uDiffuse0);
				GXTexCoord2f32(axTexCoords0[3].x, axTexCoords0[3].y);
			GXEnd();
			if (pxTextureFrame2)
			{
				Asura_RGBA xColour1 = xColour;
				xColour1.m_fA *= (1.f - fWeight);
				const unsigned int uDiffuse1 = xColour1.ToUIntRGBA(true);

				GXBegin(GX_TRIANGLEFAN, GX_VTXFMT3, 4);
				GXPosition3f32(axVertices[0].x, axVertices[0].y, axVertices[0].z);
				GXColor1u32(uDiffuse1);
				GXTexCoord2f32(axTexCoords1[0].x, axTexCoords1[0].y);

				GXPosition3f32(axVertices[1].x, axVertices[1].y, axVertices[1].z);
				GXColor1u32(uDiffuse1);
				GXTexCoord2f32(axTexCoords1[1].x, axTexCoords1[1].y);

				GXPosition3f32(axVertices[2].x, axVertices[2].y, axVertices[2].z);
				GXColor1u32(uDiffuse1);
				GXTexCoord2f32(axTexCoords1[2].x, axTexCoords1[2].y);

				GXPosition3f32(axVertices[3].x, axVertices[3].y, axVertices[3].z);
				GXColor1u32(uDiffuse1);
				GXTexCoord2f32(axTexCoords1[3].x, axTexCoords1[3].y);
				GXEnd();
			}

		}// end particle loop
		Asura_Render::AddToParticlesRendered(uNumParticles);
		Asura_Render::AddToTrianglesRendered(2 * uNumParticles, 1, ASURA_RENDER_TTYPE_PFX);
	}// end camera loop

	// Don't leave this in!
	GXFlush();
}
