//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_ResourceFile_Xbox360_Character.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_ResourceFile_Xbox360_Character.h"

// Asura Platform-independent
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_Material_System.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_CharacterRenderer.h"
#include "Rendering/Asura_Xbox360_IndexBuffer.h"
#include "Rendering/Asura_Xbox360_VertexBuffer.h"

// Misc
#include <xtl.h>

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

const u_int uVERTEX_SIZE = 48;

/******************************************************************************/
/* Asura_Chunk_ResourceFile_Xbox360_Character Class Functions                 */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_ResourceFile_Xbox360_Character::Process()
//
// Processes an Xbox 360 character chunk
//
// Parameters:		szFilename - Filename of the character
//					xStream - Reference to stream from which to read
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////
 
bool Asura_Chunk_ResourceFile_Xbox360_Character::Process( const char* const szFilename, Asura_Chunk_Stream& xStream )
{
	if( Version != iASURA_CHUNK_XBOX360_CHARACTER_VERSION )
	{
		Asura_Assertf( false, ("Invalid character chunk version (%d) encountered. Should be %d. You may need new code, or to re-export the level.",
							   Version,
							   iASURA_CHUNK_XBOX360_CHARACTER_VERSION) );
		return(false);
	}

	Asura_Hash_ID uNameHash;
	xStream >> uNameHash;

	u_int uNumStrips;
	xStream >> uNumStrips;

	Asura_Xbox360_CharacterRenderer_Skin* const pxCharacter = Asura_Xbox360_CharacterRenderer::CreateCharacter( uNameHash, uNumStrips );
	if( !pxCharacter )
	{
		return(false);
	}

	u_int uNumVertices, uNumIndices;
	xStream >> uNumVertices;
	xStream >> uNumIndices;

	xStream >> pxCharacter->m_uNumTriangles;
	xStream >> pxCharacter->m_uNumBoneMaps;
	if (pxCharacter->m_uNumBoneMaps)
	{
		pxCharacter->m_pxBoneMaps = new Asura_Xbox360_CharacterRenderer_Skin::BoneMap[pxCharacter->m_uNumBoneMaps];
	} else {
		pxCharacter->m_pxBoneMaps = NULL;

	}

	xStream.Read( sizeof( Asura_Xbox360_CharacterRenderer_Strip ) * pxCharacter->m_uNumStrips, pxCharacter->m_pxStrips );

	// Vertices

	Asura_Xbox360_VertexBuffer& xVertexBuffer = pxCharacter->m_xVertexBuffer;

	xVertexBuffer.CreateStatic( uVERTEX_SIZE, uNumVertices );

	void* const pVBData = xVertexBuffer.Lock();
	xStream.Read( uNumVertices * uVERTEX_SIZE, pVBData );
	xVertexBuffer.Unlock();

	// Indices

	Asura_Xbox360_IndexBuffer& xIndexBuffer = pxCharacter->m_xIndexBuffer;

	xIndexBuffer.CreateStatic( sizeof( u_short ) * uNumIndices );

	void* const pIBData = xIndexBuffer.Lock();
	xStream.Read( uNumIndices * sizeof( u_short ), pIBData );
	xIndexBuffer.Unlock();

	for(u_int uBoneIndex = 0; uBoneIndex < pxCharacter->GetNumBoneMaps(); ++uBoneIndex)
	{
		Asura_Xbox360_CharacterRenderer_Skin::BoneMap &xBoneMap = pxCharacter->GetBoneMap(uBoneIndex);
		xStream.Read(sizeof(u_int), &xBoneMap.m_uSize);
		xBoneMap.m_puEntries = new u_int[xBoneMap.m_uSize];
		xStream.Read(sizeof(u_int) * xBoneMap.m_uSize, xBoneMap.m_puEntries);
	}

	return(true);
}
