//////////////////////////////////////////////////////////////////////////////
//
// Asura_Chunk_ResourceFile_Xbox360_Object.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_ResourceFile_Xbox360_Object.h"

// Asura Platform-independent
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_Material_System.h"
#include "Debugging/Asura_Assert.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_IndexBuffer.h"
#include "Rendering/Asura_Xbox360_ObjectRenderer.h"
#include "Rendering/Asura_Xbox360_VertexBuffer.h"

// Misc
#include <xtl.h>

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

const u_int uVERTEX_SIZE = 36;


/******************************************************************************/
/* Asura_Chunk_ResourceFile_Xbox360_Object Class Functions                    */
/******************************************************************************/

bool Asura_Chunk_ResourceFile_Xbox360_Object::Process( const char* const szFilename, Asura_Chunk_Stream& xStream )
{
	Asura_Hash_ID uNameHash;
	xStream >> uNameHash;

	Asura_Xbox360_ObjectRenderer::Object* const pxObject = Asura_Xbox360_ObjectRenderer::CreateObject( uNameHash );
	if( !pxObject )
	{
		Asura_Assertf( false, ("Failed to create object for %s.", szFilename) );
		return(false);
	}
                                                                                                                                                                                                                                                                                                                                                                                                  
	xStream >> pxObject->m_uNumberOfStrips;

	pxObject->m_pxStrips = new Asura_Xbox360_ObjectRenderer::Strip[ pxObject->m_uNumberOfStrips ];
	if( !pxObject->m_pxStrips )
	{
		Asura_Assertf( false, ("Unable to allocate object strips for %s.", szFilename) );
		return(false);
	}

	xStream.Read( pxObject->m_uNumberOfStrips * sizeof( Asura_Xbox360_ObjectRenderer::Strip ), pxObject->m_pxStrips );

	u_int uNumVertices;
	xStream >> uNumVertices;

	if( uNumVertices > 0 )
	{
		Asura_Xbox360_VertexBuffer& xVertexBuffer = pxObject->m_xVertexBuffer;

		xVertexBuffer.CreateStatic( uVERTEX_SIZE, uNumVertices );

		void* const pData = xVertexBuffer.Lock();
		xStream.Read( uNumVertices * uVERTEX_SIZE, pData );
		xVertexBuffer.Unlock();
	}

	u_int uNumIndices;
	xStream >> uNumIndices;

	if( uNumIndices > 0 )
	{
		Asura_Xbox360_IndexBuffer& xIndexBuffer = pxObject->m_xIndexBuffer;

		xIndexBuffer.CreateStatic( sizeof( u_short ) * uNumIndices );

		void* const pData = xIndexBuffer.Lock();
		xStream.Read( uNumIndices * sizeof( u_short ), pData );
		xIndexBuffer.Unlock();
	}

	return(true);
}
