//
// Asura_Chunk_ResourceFile_Xbox360_ObjectHierarchy.cpp
//
// Created by KJFL - 10 November 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "ChunkSystem/Asura_Chunk_ResourceFile_Xbox360_ObjectHierarchy.h"

// Asura Platform-independent
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Material_System.h"
#include "Rendering/Asura_Xbox360_ObjectHierarchyRenderer.h"

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

const u_int uVERTEX_SIZE = 36;

/******************************************************************************/
/* Asura_Chunk_ResourceFile_Xbox360_ObjectHierarchy Class Functions           */
/******************************************************************************/

bool Asura_Chunk_ResourceFile_Xbox360_ObjectHierarchy::Process( const char* szFilename,	Asura_Chunk_Stream& xStream )
{
	Asura_Hash_ID	xNameHashID;
	u_int			uTotalNumberOfStrips;
	u_int			uTotalNumberOfVertices;
	u_int			uTotalNumberOfIndices;
	
	// first get the unique hash id for this hierarchy
	xStream >> xNameHashID;

	// attempt to create a new hierarchy based on the hash id
	Asura_Xbox360_ObjectHierarchyRenderer::Hierarchy* pxHierarchy = Asura_Xbox360_ObjectHierarchyRenderer::CreateHierarchy( xNameHashID ); 
	
	// is this invalid?
	if ( !pxHierarchy )
	{
		// Out of memory, or hierarchy already exists. Either way there should have been an assert already,
		// and if we just return then the chunk loaders will skip the remaining data (with another assert).
		return(false);
	}

	// stream in the hierarchy description
	xStream >> uTotalNumberOfStrips;
	xStream >> uTotalNumberOfVertices;
	xStream >> uTotalNumberOfIndices;

	pxHierarchy->m_uNumberOfStrips = uTotalNumberOfStrips;

    // Some hierarchies, or sections of them, won't have any vertices or
    // indices, since artists sometimes use empty ones for faking up other
    // things. In this case, it shouldn't have any strips either.
    Asura_Assert( (uTotalNumberOfStrips == 0) == (uTotalNumberOfVertices == 0), "" );
    Asura_Assert( (uTotalNumberOfStrips == 0) == (uTotalNumberOfIndices == 0), "" );

    // And so we can just stop at this point.
    if( !uTotalNumberOfStrips )
	{
		return(false);
	}

	// we now need to allocate enough strip descriptors, and then read in the data for them
	pxHierarchy->m_pxStrips	= new Asura_Xbox360_ObjectHierarchyRenderer::Strip[ uTotalNumberOfStrips ];
	if( !pxHierarchy->m_pxStrips )
	{
		Asura_Assert( false, "Failed to allocate object hierarchy strips." );
		return(false);
	}

	xStream.Read( sizeof( Asura_Xbox360_ObjectHierarchyRenderer::Strip ) * uTotalNumberOfStrips, pxHierarchy->m_pxStrips );

	// setup vertex buffer and stream in the data (in one big chunk)
	if( uTotalNumberOfVertices > 0)
	{
		Asura_Xbox360_VertexBuffer& xVertexBuffer = pxHierarchy->m_xVertexBuffer;

		// generate the vertex buffer
		const u_int uVertexBufferSize = uTotalNumberOfVertices * uVERTEX_SIZE;
		xVertexBuffer.CreateStatic( uVERTEX_SIZE, uTotalNumberOfVertices );

		// lock the buffer
		void* pBuffer = xVertexBuffer.Lock();
		
		// read in data
		xStream.Read( uVertexBufferSize, pBuffer );

		// unlock the buffer
		xVertexBuffer.Unlock();
	}

	// set up index buffer and stream in the data (in one big chunk)
	if( uTotalNumberOfIndices > 0 )
	{
		Asura_Xbox360_IndexBuffer& xIndexBuffer	= pxHierarchy->m_xIndexBuffer;

		// generate the index buffer
		const u_int uIndexBufferSize = uTotalNumberOfIndices * sizeof(short);
		xIndexBuffer.CreateStatic( uIndexBufferSize );
		
		// lock the buffer
		void* pBuffer = xIndexBuffer.Lock();
		
		// read in data
		xStream.Read( uIndexBufferSize, pBuffer );

		// unlock the buffer
		xIndexBuffer.Unlock();
	}

	return(true);
}
