///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Platform_MemoryManagement.h
// Author : Kevin Floyer-Lea
// Date   : 20-Jun-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#if !defined __ASURA_PLATFORM_MEMORYMANAGEMENT_H__
#define __ASURA_PLATFORM_MEMORYMANAGEMENT_H__

/******************************************************************************/
/* Macros                                                                     */
/******************************************************************************/

#define ASURA_MEMORYMANAGEMENT_PLATFORM_INIT_WHEN_DISABLED	0
#define ASURA_MEMORYMANAGEMENT_INIT_OBJ_IN_PLATFORM_CODE	1

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

#if !defined ASURA_MEMORYMANAGEMENT_ENABLED

// Externs for the Xbox360 specific functions...

extern void* Asura_PC_AllocateAlign( unsigned int bytes, enum ASURA_ALIGN_ID alignment );
extern void Asura_PC_DeallocateAlign( void* address );

// Inline-independent functions which call the Xbox360 specific ones...

inline void* Asura_AllocateAlign( unsigned int bytes, enum ASURA_ALIGN_ID alignment )
{
	return Asura_PC_AllocateAlign( bytes, alignment );
}

inline void Asura_DeallocateAlign( void* address )
{
	Asura_PC_DeallocateAlign( address );
}

#endif // !defined ASURA_MEMORYMANAGEMENT_ENABLED



////////////////////////////////////////////////////////////////////////////////
// Class        : Asura_Xbox360_MemoryManagement
// Author       : Andy Tate
// Date         : 18-Dec-2008
//
// Description  : Static class used to provide Xbox360 specific memory functions.
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_MemoryManagement
{
public:
	// Public enumerated-types...
	enum MemoryType
	{
		MT_READ_ONLY,
		MT_READ_WRITE,
		MT_READ_WRITE_COMBINE,

		MAX_MEMORY_TYPES
	};

	// Public functions...
	static void*	AllocatePhysical(const MemoryType eMemoryType, const u_int uNumBytes, const u_int uXAllocAlignment /*= XALLOC_PHYSICAL_ALIGNMENT_DEFAULT*/);
	static void		FreePhysical(const MemoryType eMemoryType, void* pData, const u_int uXAllocAlignment /*= XALLOC_PHYSICAL_ALIGNMENT_DEFAULT*/);


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...

	// Private functions...
	Asura_Xbox360_MemoryManagement(void);
	Asura_Xbox360_MemoryManagement(const Asura_Xbox360_MemoryManagement& xOther);
	~Asura_Xbox360_MemoryManagement();

	Asura_Xbox360_MemoryManagement&	operator=(const Asura_Xbox360_MemoryManagement& xOther);

};   // End of class 'Asura_Xbox360_MemoryManagement'.


#endif // !defined __ASURA_PLATFORM_MEMORYMANAGEMENT_H__
