//Asura_XBox360_Notifications.cpp
//Copyright (C) 2005 Rebellion, All Rights Reserved.
//Thomas Young - 27/09/07

//Main Asura Header
#include "Core/Asura.h"

//My header
#include "Core/Asura_XBox360_Notifications.h"

//Other Asura Headers
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Network/Asura_NetLinkManager.h"
#include "Network/Asura_Network.h"
#ifdef ASURA_SUPPORT_LOBBY
#include "Network/Online/Asura_FriendsList.h"
#endif
#include "Rewards/Asura_Rewards_System.h"

// Asura Xbox 360 headers
#include "Core/Asura_Xbox360_OS.h"
#include "Core/Asura_Xbox360_Profile_System.h"

//External Headers
#include <Xam.h>

HANDLE Asura_XBox360_Notifications::s_pNotificationHandle = 0;
#ifdef ASURA_SUPPORT_LOBBY
XINVITE_INFO Asura_XBox360_Notifications::s_xInviteInfo;
bool Asura_XBox360_Notifications::s_bGameInvitePending;
#endif

//Make a static list of notifications (there are a known number)
static Asura_Notification_Structure g_axFunctionList[] =  
{
	//System notifications
	{ XN_SYS_UI,						0, 0 },
	{ XN_SYS_SIGNINCHANGED,				0, 0 },
	{ XN_SYS_STORAGEDEVICESCHANGED,		0, 0 },
	{ XN_SYS_PROFILESETTINGCHANGED,		0, 0 },
	{ XN_SYS_MUTELISTCHANGED,			0, 0 },
	{ XN_SYS_INPUTDEVICESCHANGED,		0, 0 },
	{ XN_SYS_INPUTDEVICECONFIGCHANGED,	0, 0 },

	//Live notifications
	{ XN_LIVE_CONNECTIONCHANGED,		0, 0 },
	{ XN_LIVE_INVITE_ACCEPTED,			0, 0 },
	{ XN_LIVE_LINK_STATE_CHANGED,		0, 0 },
	{ XN_LIVE_CONTENT_INSTALLED,		0, 0 },
	{ XN_LIVE_MEMBERSHIP_PURCHASED,		0, 0 },
	{ XN_LIVE_VOICECHAT_AWAY,			0, 0 },
	{ XN_LIVE_PRESENCE_CHANGED,			0, 0 },

	//Friends Notifications
	{ XN_FRIENDS_PRESENCE_CHANGED,		0, 0 },
	{ XN_FRIENDS_FRIEND_ADDED,			0, 0 },
	{ XN_FRIENDS_FRIEND_REMOVED,		0, 0 },

	//Custom Notifications
	{ XN_CUSTOM_ACTIONPRESSED,			0, 0 },
	{ XN_CUSTOM_GAMERCARD,				0, 0 },
	 
	//Xbox Media Player Notifications
	{ XN_XMP_STATECHANGED,				0, 0 },
	{ XN_XMP_PLAYBACKBEHAVIORCHANGED,	0, 0 },
	{ XN_XMP_PLAYBACKCONTROLLERCHANGED,	0, 0 },

	{ 0 , 0 , 0 }
};

void Asura_XBox360_Notifications::Initialise ()
{
	//Set up a notification listener
	s_pNotificationHandle = 0;
	s_pNotificationHandle = XNotifyCreateListener(XNOTIFY_ALL);
	Asura_Assert(s_pNotificationHandle,"Failed to create notification listener");
	
	//Register the default callbacks
	RegisterNotificationCallback(XN_SYS_UI, Asura_XBox360_Notifications::Default_XN_SYS_UI);

	//Register the system callbacks
	RegisterSystemNotificationCallback(XN_SYS_UI, Asura_XBox360_Notifications::System_XN_SYS_UI);
	RegisterSystemNotificationCallback(XN_SYS_SIGNINCHANGED, Asura_XBox360_Notifications::System_XN_SYS_SIGNINCHANGED);
	RegisterSystemNotificationCallback(XN_SYS_STORAGEDEVICESCHANGED, System_XN_SYS_STORAGEDEVICESCHANGED);
	#ifdef ASURA_SUPPORT_LOBBY
	RegisterSystemNotificationCallback(XN_FRIENDS_FRIEND_ADDED, Asura_XBox360_Notifications::System_XN_FRIENDS_FRIEND_ADDED);
	RegisterSystemNotificationCallback(XN_FRIENDS_FRIEND_REMOVED, Asura_XBox360_Notifications::System_XN_FRIENDS_FRIEND_REMOVED);
	RegisterSystemNotificationCallback(XN_LIVE_INVITE_ACCEPTED, Asura_XBox360_Notifications::System_XN_LIVE_INVITE_ACCEPTED);
	#endif
	RegisterSystemNotificationCallback(XN_LIVE_CONNECTIONCHANGED, Asura_XBox360_Notifications::System_XN_LIVE_CONNECTIONCHANGED);
	RegisterSystemNotificationCallback(XN_LIVE_LINK_STATE_CHANGED, Asura_XBox360_Notifications::System_XN_LIVE_LINK_STATE_CHANGED);
}

void Asura_XBox360_Notifications::Update ()
{
	DWORD	uNotificationID = 0;
	u_long	uParameter = 0;
	//loop all notifications
	Asura_Notification_Structure* pxItem = g_axFunctionList;
	while(pxItem->uID)
	{		
		uNotificationID = pxItem->uID;
		//note: we get and ignore notifications if they don't have an associated function (we don't want to leave them piling up in the listener)
		while(XNotifyGetNext(s_pNotificationHandle,uNotificationID,&uNotificationID,&uParameter))
		{
			if(pxItem->pfnSystemFunction)
			{
				pxItem->pfnSystemFunction(uParameter);
			}
			if(pxItem->pfnProjectFunction)	
			{
				pxItem->pfnProjectFunction(uParameter);
			}
		}
		++pxItem;
	}
}

void Asura_XBox360_Notifications::Shutdown ()
{
	if(!CloseHandle(s_pNotificationHandle))
	{
		Asura_Assert(true, "Failed to close notification handle");
	}
	
}
void Asura_XBox360_Notifications::RegisterNotificationCallback (DWORD uID, Asura_Notify_Function	pfnFunction)
{
	Asura_Notification_Structure*	pxItem = g_axFunctionList;
	//loop and find this ID in our static list
	while(pxItem->uID)
	{
		if(pxItem->uID == uID) //found the function we need to replace
		{
			pxItem->pfnProjectFunction = pfnFunction;
			break;
		}
		++pxItem;
	}
	Asura_Assert(pxItem->uID,"Tried to register a notification callback with an invalid ID");
}

void Asura_XBox360_Notifications::RegisterSystemNotificationCallback (DWORD uID, Asura_Notify_Function pfnFunction)
{
	Asura_Notification_Structure*	pxItem = g_axFunctionList;
	//loop and find this ID in our static list
	while(pxItem->uID)
	{
		if(pxItem->uID == uID) //found the function we need to replace
		{
			pxItem->pfnSystemFunction = pfnFunction;
			break;
		}
		++pxItem;
	}
	Asura_Assert(pxItem->uID,"Tried to register a notification callback with an invalid ID");
}

void	Asura_XBox360_Notifications::ClearNotificationCallback	(DWORD uID)
{
	Asura_Notification_Structure* pxItem = g_axFunctionList;
	//loop and find this ID in our static list
	while(pxItem->uID)
	{
		if(pxItem->uID == uID) //found the function we need to replace
		{
			pxItem->pfnProjectFunction = 0;
			break;
		}
		++pxItem;
	}
	Asura_Assert(pxItem->uID,"Tried to clear a notification callback with an invalid ID");
}

void Asura_XBox360_Notifications::Flush ()
{
	DWORD	uNotificationID = 0;
	u_long	uParameter = 0;
	//loop all notifications
	Asura_Notification_Structure* pxItem = g_axFunctionList;
	while(pxItem->uID)
	{		
		uNotificationID = pxItem->uID;
		while(XNotifyGetNext(s_pNotificationHandle,uNotificationID,&uNotificationID,&uParameter)) 
		{
			//do nothing with the notification
		}
		++pxItem;
	}
}

//Default implementations follow...

void Asura_XBox360_Notifications::Default_XN_SYS_UI (u_long uData)
{
	//set to TRUE if the UI is about to be displayed and FALSE if it is about to be hidden.
	if(uData)
	{
		//Don't pause if the network is active because we are in a multiplayer game
		if(!Asura_Network::IsExternalNetworkActive())
		{
			Asura_Timers::SetSystemPause(true);	
		}		
	}
	else
	{
		Asura_Timers::SetSystemPause(false);
	}
}

//System implementations follow...

void Asura_XBox360_Notifications::System_XN_SYS_UI (u_long uData)
{
	//set to TRUE if the UI is about to be displayed and FALSE if it is about to be hidden.
	if(uData)
	{
		//Do stuff
		Asura_Xbox360_Profile_System::SetUIOpen( true );
		Asura_Xbox360_OS::SetSystemUIOpen(true);
	}
	else
	{
		Asura_Xbox360_Profile_System::SetUIOpen( false );
		Asura_Xbox360_OS::SetSystemUIOpen(false);

		#ifdef ASURA_SUPPORT_LOBBY
		// if the friends GUI was flagged as active, it must have just closed.
		if (Asura_FriendsList::IsFriendsGUIActive())
		{
			Asura_FriendsList::FriendsGUIDeactivated();
		}
		#endif
	}
}

void Asura_XBox360_Notifications::System_XN_SYS_SIGNINCHANGED( u_long uData )
{
#ifdef ASURA_USE_REWARDS
	Asura_Rewards_System::Reset();
#endif // #ifdef ASURA_USE_REWARDS
}

void Asura_XBox360_Notifications::System_XN_SYS_STORAGEDEVICESCHANGED( u_long uData )
{
	Asura_Xbox360_Profile_System::StorageDevicesChanged ();
}


#ifdef ASURA_SUPPORT_LOBBY
void Asura_XBox360_Notifications::System_XN_FRIENDS_FRIEND_ADDED(u_long uData)
{
	Asura_FriendsList::Refresh();
}

void Asura_XBox360_Notifications::System_XN_FRIENDS_FRIEND_REMOVED(u_long uData)
{
	Asura_FriendsList::Refresh();
}

void Asura_XBox360_Notifications::System_XN_LIVE_INVITE_ACCEPTED(u_long uData)
{
	HRESULT dwError = XInviteGetAcceptedInfo( uData, &s_xInviteInfo );
	if (dwError == ERROR_SUCCESS)
	{
		s_bGameInvitePending = true;
	}
}

const XINVITE_INFO& Asura_XBox360_Notifications::GetGameInviteDetails()
{
	Asura_Assert(s_bGameInvitePending, "should check IsGameInvitePending() first");
	s_bGameInvitePending = false;
	return s_xInviteInfo;
}
#endif


void Asura_XBox360_Notifications::System_XN_LIVE_CONNECTIONCHANGED( u_long uData )
{
	#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	HRESULT xResult = static_cast<HRESULT>( uData );
	
	if( xResult != XONLINE_S_LOGON_CONNECTION_ESTABLISHED )
	{
		Asura_NetLinkManager::DropConnection();
	}
	#endif
}


void Asura_XBox360_Notifications::System_XN_LIVE_LINK_STATE_CHANGED( u_long uData )
{
	#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	BOOL bConnected = static_cast<BOOL>( uData );

	if( bConnected != TRUE )
	{
		Asura_NetLinkManager::DropConnection();
	}
	#endif
}


//eof
