//*******************************************************************
//
// File: Asura_XBox360_VectorMaths.h
//
// Created: 21 December 2006 DLH
//
// Description: 
//
//*******************************************************************

#ifndef __ASURA_XBOX360_VECTORMATHS_H__
#define __ASURA_XBOX360_VECTORMATHS_H__

#if defined(ASURA_USE_PLATFORM_VECTORS)

// Asura Platform-independent
#include "Core/Asura_HalfFloat.h"

//*******************************************************************
// Defines
//*******************************************************************

// Asura_AVector3...
#define Asura_AVector3_InParam		const Asura_AVector3
#define Asura_AVector3_OutParam		Asura_AVector3&
#define Asura_AVector3_InOutParam	Asura_AVector3&

// Asura_AMatrix3x3...
#define Asura_AMatrix3x3_InParam	const Asura_AMatrix3x3&
#define Asura_AMatrix3x3_OutParam	Asura_AMatrix3x3&
#define Asura_AMatrix3x3_InOutParam	Asura_AMatrix3x3&

// Asura_AVector4...
#define Asura_AVector4_InParam		const Asura_AVector4
#define Asura_AVector4_OutParam		Asura_AVector4&
#define Asura_AVector4_InOutParam	Asura_AVector4&

// Asura_AMatrix4x4...
#define Asura_AMatrix4x4_InParam	const Asura_AMatrix4x4&
#define Asura_AMatrix4x4_OutParam	Asura_AMatrix4x4&
#define Asura_AMatrix4x4_InOutParam	Asura_AMatrix4x4&


//*******************************************************************
// Class Definitions
//*******************************************************************

//////////////////////////////////////////////////////////////////////////
// Asura_AVector3
//////////////////////////////////////////////////////////////////////////

// ART - 18/07/2007 - Removed for the time being as its causing an
// internal compiler error in current builds...
//__declspec(passinreg) 
// KJFL - 11 February 2009 - added the passinreg declspec back in!
__declspec(passinreg) class Asura_AVector3
{
public:
	// Constructors...
	Asura_AVector3(void);
	Asura_AVector3(const float fX, const float fY, const float fZ);
	//Asura_AVector3(Asura_AVector3_InParam xOther);		// Not necessary due to register class declaration.
	explicit Asura_AVector3(const Asura_Vector_3& xOther);

	// Zero function...
	void Zero(void);

	// Selection functions...
	void SelectGE(Asura_AVector3_InParam xLHS, Asura_AVector3_InParam xRHS, Asura_AVector3_InParam xTrue, Asura_AVector3_InParam xFalse);
	void SelectGT(Asura_AVector3_InParam xLHS, Asura_AVector3_InParam xRHS, Asura_AVector3_InParam xTrue, Asura_AVector3_InParam xFalse);
	void SelectLE(Asura_AVector3_InParam xLHS, Asura_AVector3_InParam xRHS, Asura_AVector3_InParam xTrue, Asura_AVector3_InParam xFalse);
	void SelectLT(Asura_AVector3_InParam xLHS, Asura_AVector3_InParam xRHS, Asura_AVector3_InParam xTrue, Asura_AVector3_InParam xFalse);
	void SelectEQ(Asura_AVector3_InParam xLHS, Asura_AVector3_InParam xRHS, Asura_AVector3_InParam xTrue, Asura_AVector3_InParam xFalse);
	void SelectNE(Asura_AVector3_InParam xLHS, Asura_AVector3_InParam xRHS, Asura_AVector3_InParam xTrue, Asura_AVector3_InParam xFalse);

	// Load/store functions...
	void LoadAligned(const float* pfFloats);
	void LoadUnaligned(const float*	pfFloats);
	void StoreAligned(float* pfFloats) const;
	void StoreUnaligned(float* pfFloats) const;
	void StoreAlignedHalf(Asura_HalfFloat* pxHalves) const;

	// Manipulation functions...
	float MagnitudeSquared(void) const;
	float Magnitude(void) const;

	void Normalise(void);
	float NormaliseReturningMagnitude(void);

	void ApproximateNormalise(void);
	float ApproximateNormaliseReturningMagnitude(void);

	void Lerp(Asura_AVector3_InParam xTo, const float fWeight);
	void Lerp(Asura_AVector3_InParam xFrom, Asura_AVector3_InParam xTo, const float fWeight);

	// Minimum/maximum functions...
	void Max(Asura_AVector3_InParam xV1, Asura_AVector3_InParam xV2);
	void Min(Asura_AVector3_InParam xV1, Asura_AVector3_InParam xV2);

	// Assignment operators...
	//Asura_AVector3_OutParam operator=(Asura_AVector3_InParam xOther);		// Not necessary due to register class declaration.
	void operator=(const Asura_Vector_3& xOther);

	void operator+=(Asura_AVector3_InParam xOther);
	void operator-=(Asura_AVector3_InParam xOther);
	void operator*=(const float fScale);
	void operator/=(const float fScale);

	void operator*=(Asura_AVector3_InParam xOther);

	// Unary operators...
	Asura_AVector3 operator-(void) const;

	// Binary operators...
	Asura_AVector3 operator+(Asura_AVector3_InParam xOther) const;
	Asura_AVector3 operator-(Asura_AVector3_InParam xOther) const;
	Asura_AVector3 operator*(const float fScale) const;
	Asura_AVector3 operator/(const float fScale) const;
	Asura_AVector3 operator^(Asura_AVector3_InParam xOther) const;

	float operator*(Asura_AVector3_InParam xOther) const;

	// Comparison operators...
	bool ApproximatelyEquals(Asura_AVector3_InParam xOther, const float fEpsilon = 1.0e-6f) const;

	bool operator==(Asura_AVector3_InParam xOther) const;
	bool operator!=(Asura_AVector3_InParam xOther) const;

	bool operator<(Asura_AVector3_InParam xOther) const;
	bool operator<=(Asura_AVector3_InParam xOther) const;
	bool operator>=(Asura_AVector3_InParam xOther) const;
	bool operator>(Asura_AVector3_InParam xOther) const;

	// Properties...
	float GetX(void) const;
	float GetY(void) const;
	float GetZ(void) const;
	void SetX(const float fX);
	void SetY(const float fY);
	void SetZ(const float fZ);

	float GetElement(const u_int uIndex) const;
	void SetElement(const u_int uIndex, const float fScale);

private:
	friend class Asura_AMatrix3x3;

	XMVECTOR	m_xVector4;		// Use the vector-intrinsics Microsoft have written for us.
};


//////////////////////////////////////////////////////////////////////////
// Asura_AMatrix3x3
//////////////////////////////////////////////////////////////////////////
// Note that in Asura_Matrix_3x3 the data is stored as 3 rows. For efficiency when using 
// vector instructions it's better for us to store 3 columns. Since the old 3x3 matrix
// use e11 to e33 I'm keeping the convention of starting named columns at 1 rather than 0.
//////////////////////////////////////////////////////////////////////////
class Asura_AMatrix3x3
{
public:
    Asura_AMatrix3x3(void);
    explicit Asura_AMatrix3x3(const Asura_Matrix_3x3& xMat3);
    Asura_AMatrix3x3(const float fE11, const float fE12, const float fE13,
					 const float fE21, const float fE22, const float fE23,
					 const float fE31, const float fE32, const float fE33);
	Asura_AMatrix3x3(const Asura_AVector3& xCol0, const Asura_AVector3& xCol1, const Asura_AVector3& xCol2);

	void ToMat3(Asura_Matrix_3x3& xMat3) const;
	void FromMat3(const Asura_Matrix_3x3& xMat3);

	Asura_AVector3 GetColumn0(void) const;
	Asura_AVector3 GetColumn1(void) const;
	Asura_AVector3 GetColumn2(void) const;
	Asura_AVector3 GetColumn(const u_int uIndex) const;

	Asura_AVector3 GetRow0(void) const;
	Asura_AVector3 GetRow1(void) const;
	Asura_AVector3 GetRow2(void) const;
	Asura_AVector3 GetRow(const u_int uIndex) const;

	float GetElement(const u_int uRow, const u_int uCol) const;
    
	void SetColumn0(Asura_AVector3_InParam xCol0);
	void SetColumn1(Asura_AVector3_InParam xCol1);
	void SetColumn2(Asura_AVector3_InParam xCol2);
	void SetColumn(const u_int uIndex, Asura_AVector3_InParam xColumn);

	void SetRow0(Asura_AVector3_InParam xRow0);
	void SetRow1(Asura_AVector3_InParam xRow1);
	void SetRow2(Asura_AVector3_InParam xRow2);
	void SetRow(const u_int uIndex, Asura_AVector3_InParam xRow);

	void SetElement(const u_int uRow, const u_int uCol, const float fValue);

	// Tilde operator to make a skew-symmetric matrix from a vector
	void Tilde(Asura_AVector3_InParam xVector);

	// Try to make the matrix orthogonal by forcing the axes represented by the matrix
	// columns to be a) unit-length and b) perpendicular.
	void Orthogonalise(void);

	// matrix-matrix per component addition - used in integration
	void Add(const Asura_AMatrix3x3& xM);

	Asura_AVector3 Apply(Asura_AVector3_InParam xInput) const;
	void Apply(Asura_AMatrix3x3_InParam xInput, Asura_AMatrix3x3_OutParam xOutput) const;

	void Transpose(void);
    Asura_AVector3 ApplyTranspose(Asura_AVector3_InParam xVec) const;

	void Identity(void);
	void Zero(void);

	Asura_AMatrix3x3 operator*(const float fScale) const;

	bool operator==(Asura_AMatrix3x3_InParam xMat) const;
	bool operator!=(Asura_AMatrix3x3_InParam xMat) const;

private:
	Asura_AVector3	m_axColumns[3];
};


//////////////////////////////////////////////////////////////////////////
// Asura_SoA_Float
//////////////////////////////////////////////////////////////////////////
class Asura_SoA_Float
{
	float m_afValues[4];

public:

	Asura_SoA_Float() {}
	Asura_SoA_Float(float fSplat)
	{
		m_afValues[0] = fSplat;
		m_afValues[1] = fSplat;
		m_afValues[2] = fSplat;
		m_afValues[3] = fSplat;
	}
	Asura_SoA_Float(float f0, float f1, float f2, float f3)
	{
		m_afValues[0] = f0;
		m_afValues[1] = f1;
		m_afValues[2] = f2;
		m_afValues[3] = f3;
	}
	__forceinline void operator= (const Asura_SoA_Float& xRHS)
	{
		m_afValues[0] = xRHS.m_afValues[0];
		m_afValues[1] = xRHS.m_afValues[1];
		m_afValues[2] = xRHS.m_afValues[2];
		m_afValues[3] = xRHS.m_afValues[3];
	}
	Asura_SoA_Float(const Asura_SoA_Float& xRHS)
	{
		m_afValues[0] = xRHS.m_afValues[0];
		m_afValues[1] = xRHS.m_afValues[1];
		m_afValues[2] = xRHS.m_afValues[2];
		m_afValues[3] = xRHS.m_afValues[3];
	}

	__forceinline Asura_SoA_Float operator+ (const Asura_SoA_Float& xRHS) const
	{
		return Asura_SoA_Float(m_afValues[0] + xRHS.m_afValues[0],
								m_afValues[1] + xRHS.m_afValues[1],
								m_afValues[2] + xRHS.m_afValues[2],
								m_afValues[3] + xRHS.m_afValues[3]);
	}

	__forceinline Asura_SoA_Float operator* (const Asura_SoA_Float& xRHS) const
	{
		return Asura_SoA_Float(m_afValues[0] * xRHS.m_afValues[0],
								m_afValues[1] * xRHS.m_afValues[1],
								m_afValues[2] * xRHS.m_afValues[2],
								m_afValues[3] * xRHS.m_afValues[3]);
	}

	__forceinline bool operator== (const Asura_SoA_Float& xRHS) const
	{
		return	m_afValues[0] == xRHS.m_afValues[0] && 
				m_afValues[1] == xRHS.m_afValues[1] && 
				m_afValues[2] == xRHS.m_afValues[2] && 
				m_afValues[3] == xRHS.m_afValues[3];
	}
	__forceinline bool operator!= (const Asura_SoA_Float& xRHS) const
	{
		return	m_afValues[0] != xRHS.m_afValues[0] || 
				m_afValues[1] != xRHS.m_afValues[1] || 
				m_afValues[2] != xRHS.m_afValues[2] || 
				m_afValues[3] != xRHS.m_afValues[3];
	}

	friend class Asura_SoA_Vector_3;
};


//////////////////////////////////////////////////////////////////////////
// Asura_SoA_Vector_3
//////////////////////////////////////////////////////////////////////////
class Asura_SoA_Vector_3
{
	Asura_SoA_Float m_xX;
	Asura_SoA_Float m_xY;
	Asura_SoA_Float m_xZ;

public:

	Asura_SoA_Vector_3() {}
	Asura_SoA_Vector_3(const Asura_SoA_Float& xX, const Asura_SoA_Float& xY, const Asura_SoA_Float& xZ) 
	{
		m_xX = xX;
		m_xY = xY;
		m_xZ = xZ;
	}
	Asura_SoA_Vector_3(const Asura_SoA_Vector_3& xRHS)
	{
		m_xX = xRHS.m_xX;
		m_xY = xRHS.m_xY;
		m_xZ = xRHS.m_xZ;
	}
	void operator= (const Asura_SoA_Vector_3& xRHS)
	{
		m_xX = xRHS.m_xX;
		m_xY = xRHS.m_xY;
		m_xZ = xRHS.m_xZ;
	}

	// create new SoA floats from members
	__forceinline Asura_SoA_Float GetXXXX() const { return m_xX; }
	__forceinline Asura_SoA_Float GetYYYY() const { return m_xY; }
	__forceinline Asura_SoA_Float GetZZZZ() const { return m_xZ; }

	// dot product
	__forceinline Asura_SoA_Float operator* (const Asura_SoA_Vector_3& xRHS) const
	{
//		return	m_xX * xRHS.m_xX + 
//				m_xY * xRHS.m_xY + 
//				m_xZ * xRHS.m_xZ;
		float fOut0, fOut1, fOut2, fOut3;

		float fX0 = m_xX.m_afValues[0];
		float fX1 = xRHS.m_xX.m_afValues[0];
		float fY0 = m_xY.m_afValues[0];
		float fY1 = xRHS.m_xY.m_afValues[0];
		float fZ0 = m_xZ.m_afValues[0];
		float fZ1 = xRHS.m_xZ.m_afValues[0];
		float fX2 = m_xX.m_afValues[1];
		float fX3 = xRHS.m_xX.m_afValues[1];
		float fY2 = m_xY.m_afValues[1];
		float fY3 = xRHS.m_xY.m_afValues[1];
		float fZ2 = m_xZ.m_afValues[1];
		float fZ3 = xRHS.m_xZ.m_afValues[1];

		fOut0 = fX0 * fX1 + fY0 * fY1 + fZ0 * fZ1;
		fOut1 = fX2 * fX3 + fY2 * fY3 + fZ2 * fZ3;

		fX0 = m_xX.m_afValues[2];
		fX1 = xRHS.m_xX.m_afValues[2];
		fY0 = m_xY.m_afValues[2];
		fY1 = xRHS.m_xY.m_afValues[2];
		fZ0 = m_xZ.m_afValues[2];
		fZ1 = xRHS.m_xZ.m_afValues[2];
		fX2 = m_xX.m_afValues[3];
		fX3 = xRHS.m_xX.m_afValues[3];
		fY2 = m_xY.m_afValues[3];
		fY3 = xRHS.m_xY.m_afValues[3];
		fZ2 = m_xZ.m_afValues[3];
		fZ3 = xRHS.m_xZ.m_afValues[3];

		fOut2 = fX0 * fX1 + fY0 * fY1 + fZ0 * fZ1;
		fOut3 = fX2 * fX3 + fY2 * fY3 + fZ2 * fZ3;

		return Asura_SoA_Float(fOut0, fOut1, fOut2, fOut3);
	}

	// transform by matrix
  	__forceinline const Asura_SoA_Vector_3 operator* (Asura_AMatrix3x3_InParam xMat) const
	{
		const Asura_AVector3 xCol0 = xMat.GetColumn0();
		// splat M11 to all elements of xM11, etc.
		Asura_SoA_Float xM11(xCol0.GetX());
		Asura_SoA_Float xM21(xCol0.GetY());
		Asura_SoA_Float xM31(xCol0.GetZ());

		Asura_SoA_Float xOutX = m_xX * xM11;
		Asura_SoA_Float xOutY = m_xX * xM21;
		Asura_SoA_Float xOutZ = m_xX * xM31;

		const Asura_AVector3 xCol1 = xMat.GetColumn1();
		Asura_SoA_Float xM12(xCol1.GetX());
		Asura_SoA_Float xM22(xCol1.GetY());
		Asura_SoA_Float xM32(xCol1.GetZ());

		xOutX = m_xY * xM12 + xOutX;
		xOutY = m_xY * xM22 + xOutY;
		xOutZ = m_xY * xM32 + xOutZ;

		const Asura_AVector3 xCol2 = xMat.GetColumn2();
		Asura_SoA_Float xM13(xCol2.GetX());
		Asura_SoA_Float xM23(xCol2.GetY());
		Asura_SoA_Float xM33(xCol2.GetZ());

		xOutX = m_xZ * xM13 + xOutX;
		xOutY = m_xZ * xM23 + xOutY;
		xOutZ = m_xZ * xM33 + xOutZ;

		return Asura_SoA_Vector_3(xOutX, xOutY, xOutZ);
	}
};


//////////////////////////////////////////////////////////////////////////
// Asura_AVector4
//////////////////////////////////////////////////////////////////////////
__declspec(passinreg) class Asura_AVector4
{
	friend class Asura_AMatrix4x4;

public:
	// Constructors...
	Asura_AVector4(void);
	Asura_AVector4(const float fX, const float fY, const float fZ, const float fW);
	//Asura_AVector4(const Asura_AVector4& xOther);		// Not necessary due to register class declaration.
	Asura_AVector4(const Asura_AVector3& xOther, const float fW);
	Asura_AVector4(const Asura_Vector_3& xOther, const float fW);

	// Zero functions...
	void ZeroW0(void);
	void ZeroW1(void);

	// Selection functions...
	void SelectGE(Asura_AVector4_InParam xLHS, Asura_AVector4_InParam xRHS, Asura_AVector4_InParam xTrue, Asura_AVector4_InParam xFalse);
	void SelectGT(Asura_AVector4_InParam xLHS, Asura_AVector4_InParam xRHS, Asura_AVector4_InParam xTrue, Asura_AVector4_InParam xFalse);
	void SelectLE(Asura_AVector4_InParam xLHS, Asura_AVector4_InParam xRHS, Asura_AVector4_InParam xTrue, Asura_AVector4_InParam xFalse);
	void SelectLT(Asura_AVector4_InParam xLHS, Asura_AVector4_InParam xRHS, Asura_AVector4_InParam xTrue, Asura_AVector4_InParam xFalse);
	void SelectEQ(Asura_AVector4_InParam xLHS, Asura_AVector4_InParam xRHS, Asura_AVector4_InParam xTrue, Asura_AVector4_InParam xFalse);
	void SelectNE(Asura_AVector4_InParam xLHS, Asura_AVector4_InParam xRHS, Asura_AVector4_InParam xTrue, Asura_AVector4_InParam xFalse);

	// Load/store functions...
	void LoadAligned(const float* pfFloats);
	void LoadUnaligned(const float* pfFloats);
	void StoreAligned(float* pfFloats) const;
	void StoreUnaligned(float* pfFloats) const;
	void StoreAlignedHalf(Asura_HalfFloat* pxHalves) const;

	// Manipulation functions...
	Asura_AVector4 ComponentwiseProduct(Asura_AVector4_InParam xOther) const;
	float DotProduct3(Asura_AVector4_InParam xOther) const;
	float DotProduct4(Asura_AVector4_InParam xOther) const;

	float MagnitudeSquared(void) const;
	float Magnitude(void) const;

	void Normalise(void);
	float NormaliseReturningMagnitude(void);
	void ApproximateNormalise(void);
	float ApproximateNormaliseReturningMagnitude(void);

	void Lerp(Asura_AVector4_InParam xTo, const float fWeight);
	void Lerp(Asura_AVector4_InParam xFrom, Asura_AVector4_InParam xTo, const float fWeight);

	// Assignment operators...
	//Asura_AVector4& operator=(Asura_AVector4_InParam xOther);		// Not necessary due to register class declaration.

	void operator+=(Asura_AVector4_InParam xOther);
	void operator-=(Asura_AVector4_InParam xOther);

	void operator*=(const float fScale);
	void operator/=(const float fScale);

	// Unary operators...
	Asura_AVector4 operator-(void) const;

	// Binary operators...
	Asura_AVector4 operator+(Asura_AVector4_InParam xOther) const;
	Asura_AVector4 operator-(Asura_AVector4_InParam xOther) const;
	Asura_AVector4 operator^(Asura_AVector4_InParam xOther) const;

	float operator*(Asura_AVector4_InParam xOther) const;

	Asura_AVector4 operator*(const float fScale) const;
	Asura_AVector4 operator/(const float fScale) const;

	// Comparison operators...
	bool ApproximatelyEquals(Asura_AVector4_InParam xOther, const float fEpsilon = 1.0e-6f) const;

	bool operator==(Asura_AVector4_InParam xOther) const;
	bool operator!=(Asura_AVector4_InParam xOther) const;

	bool operator<(Asura_AVector4_InParam xOther) const;
	bool operator<=(Asura_AVector4_InParam xOther) const;
	bool operator>=(Asura_AVector4_InParam xOther) const;
	bool operator>(Asura_AVector4_InParam xOther) const;

	// Minimum/maximum functions...
	void Max(Asura_AVector4_InParam xV1, Asura_AVector4_InParam xV2);
	void Min(Asura_AVector4_InParam xV1, Asura_AVector4_InParam xV2);

	// Properties...
	float GetX(void) const;
	float GetY(void) const;
	float GetZ(void) const;
	float GetW(void) const;

	void SetX(const float fX);
	void SetY(const float fY);
	void SetZ(const float fZ);
	void SetW(const float fW);

	float GetElement(const u_int uIndex) const;
	void SetElement(const u_int uIndex, const float fValue);
	void SetElements(const float fX, const float fY, const float fZ, const float fW);

private:
	XMVECTOR	m_xVector4;		// Use the vector-intrinsics Microsoft have written for us.
};


//////////////////////////////////////////////////////////////////////////
// Asura_AMatrix4x4
//////////////////////////////////////////////////////////////////////////
class Asura_AMatrix4x4
{
public:
	// Constructors/initializers
	Asura_AMatrix4x4(void);
	Asura_AMatrix4x4(const float fE11, const float fE12, const float fE13, const float fE14,
					 const float fE21, const float fE22, const float fE23, const float fE24,
					 const float fE31, const float fE32, const float fE33, const float fE34,
					 const float fE41, const float fE42, const float fE43, const float fE44);
	Asura_AMatrix4x4(const Asura_AMatrix4x4& xOther);
	Asura_AMatrix4x4(const Asura_Matrix_3x3& xRotation, const Asura_Vector_3& xPosition);
	Asura_AMatrix4x4(const Asura_Quat& xRotation, const Asura_Vector_3& xPosition);

	// Constant functions...
	void Identity(void);
	void Zero(void);

	// Orientation/translation functions...
	void ToMatrix(Asura_Matrix_3x3& xOrientation) const;
	void FromMatrix(const Asura_Matrix_3x3& xOrientation);

	Asura_Vector_3 GetTranslation() const;
	void SetTranslation(const Asura_Vector_3& xTranslation);

	Asura_AVector3 GetATranslation() const;
	void SetTranslation(Asura_AVector3_InParam xTranslation);

	Asura_AVector4 GetA4Translation() const;
	void SetTranslation(Asura_AVector4_InParam xTranslation);

	// Manipulation functions...
	Asura_AVector4 Apply(Asura_AVector4_InParam xInput) const;
	void ApplyToArray(const Asura_AVector4* pxInput, const u_int uNumElements, Asura_AVector4* pxOutput) const;

	void Apply(Asura_AMatrix4x4_InParam xInput, Asura_AMatrix4x4_OutParam xOutput) const;
	void ApplyToArray(const Asura_AMatrix4x4* pxInput, const u_int uNumElements, Asura_AMatrix4x4* pxOutput) const;

	float Determinant(void) const;
	float Cofactor(const u_int uRow, const u_int uCol) const;

	void Invert(void);
	void InvertOrthogonal(void);
	void Transpose(void);

	void Invert(Asura_AMatrix4x4_OutParam xOther) const;
	void InvertOrthogonal(Asura_AMatrix4x4_OutParam xOther) const;
	void Transpose(Asura_AMatrix4x4_OutParam xOther) const;

	// Unary operators
	Asura_AMatrix4x4 operator-(void) const;

	// Binary operators
	Asura_AMatrix4x4 operator*(const float fScale) const;
	Asura_AMatrix4x4 operator/(const float fScale) const;

	// Assignment operators
	void operator=(Asura_AMatrix4x4_InParam xOther);
	void operator*= (const float fScale);
	void operator/= (const float fScale);

	// Conditional operators
	bool operator==(Asura_AMatrix4x4_InParam xOther) const;
	bool ApproximatelyEquals(Asura_AMatrix4x4_InParam xOther, const float fEpsilon = 1.0e-6f) const;
	bool operator!=(Asura_AMatrix4x4_InParam xOther) const;

	// Elementary properties...
	float GetElement(const u_int uRow, const u_int uCol) const;
	void SetElement(const u_int uRow, const u_int uCol, const float fValue);

	Asura_AVector4 GetColumn(const u_int uColumnIndex) const;
	Asura_AVector4 GetColumn0() const;
	Asura_AVector4 GetColumn1() const;
	Asura_AVector4 GetColumn2() const;
	Asura_AVector4 GetColumn3() const;

	void SetColumn(const u_int uColumnIndex, Asura_AVector4_InParam xColumn);
	void SetColumn0(Asura_AVector4_InParam xCol0);
	void SetColumn1(Asura_AVector4_InParam xCol1);
	void SetColumn2(Asura_AVector4_InParam xCol2);
	void SetColumn3(Asura_AVector4_InParam xCol3);

	Asura_AVector4 GetRow(const u_int uRowIndex) const;
	Asura_AVector4 GetRow0() const;
	Asura_AVector4 GetRow1() const;
	Asura_AVector4 GetRow2() const;
	Asura_AVector4 GetRow3() const;

	void SetRow(const u_int uRowIndex, Asura_AVector4_InParam xRow);
	void SetRow0(Asura_AVector4_InParam xRow0);
	void SetRow1(Asura_AVector4_InParam xRow1);
	void SetRow2(Asura_AVector4_InParam xRow2);
	void SetRow3(Asura_AVector4_InParam xRow3);

	void SetColumns(Asura_AVector4_InParam xCol0, Asura_AVector4_InParam xCol1, Asura_AVector4_InParam xCol2, Asura_AVector4_InParam xCol3);
	void SetRows(Asura_AVector4_InParam xRow0, Asura_AVector4_InParam xRow1, Asura_AVector4_InParam xRow2, Asura_AVector4_InParam xRow3);

	// Load/store functions...
	void LoadColsAligned(const float* pfFloats);
	void LoadRowsAligned(const float* pfFloats) ;
	void LoadColsUnaligned(const float* pfFloats);
	void LoadRowsUnaligned(const float* pfFloats);

	void StoreColsAligned(float* pfFloats) const;
	void StoreRowsAligned(float* pfFloats) const;
	void StoreColsUnaligned(float* pfFloats) const;
	void StoreRowsUnaligned(float* pfFloats) const;

private:
	Asura_AVector4	m_axColumns[4];	// Array of column-vectors.
};


//////////////////////////////////////////////////////////////////////////
// Inline Function Definitions
//////////////////////////////////////////////////////////////////////////
__forceinline Asura_AVector3 operator*(Asura_AVector3_InParam xVec, Asura_AMatrix3x3_InParam xMat)
{
	return xMat.Apply( xVec );
}

__forceinline Asura_AVector3 operator*(Asura_AVector3_InParam xVec, const Asura_Matrix_3x3& xMat)
{
	const Asura_AMatrix3x3	xTemp(xMat);
	return xTemp.Apply(xVec);
}

__forceinline Asura_AMatrix3x3 operator*(Asura_AMatrix3x3_InParam xM1, Asura_AMatrix3x3_InParam xM2)
{
	Asura_AMatrix3x3	xResult;

	xM2.Apply(xM1, xResult);
	return(xResult);
}

// scalar * 4x1 Vector - to match the member function in Asura_AVector4
__forceinline Asura_AVector4 operator*(const float fScale, Asura_AVector4_InParam xOther)
{
	return(xOther * fScale);
}

__forceinline Asura_AVector4 operator*(Asura_AVector4_InParam xVec, Asura_AMatrix4x4_InParam xMat)
{
	return xMat.Apply(xVec);
}

// To match the backwards vector-matrix multiply, we do a backwards matrix-vector multiply too.
__forceinline Asura_AMatrix4x4 operator*(Asura_AMatrix4x4_InParam xM1, Asura_AMatrix4x4_InParam xM2)
{
	Asura_AMatrix4x4	xResult;

	xM2.Apply(xM1, xResult);
	return(xResult);
}


//////////////////////////////////////////////////////////////////////////
// Asura_Quat
//////////////////////////////////////////////////////////////////////////
__forceinline void Asura_Quat::ToMatrix(Asura_AMatrix3x3_OutParam m) const
{
	float s = 2.0f;///Magnitude();      

	float xs = x*s;
	float ys = y*s;
	float zs = z*s;

	float wx = w*xs;
	float wy = w*ys;
	float wz = w*zs;

	float xx = x*xs;
	float xy = x*ys;
	float xz = x*zs;

	float yy = y*ys;
	float yz = y*zs;
	float zz = z*zs;

	// Implementation from Graphics Gems IV
	m.SetColumn0(Asura_AVector3(1.0f - (yy+zz), xy + wz, xz - wy));
	m.SetColumn1(Asura_AVector3(xy - wz, 1.0f - (xx + zz), yz + wx));
	m.SetColumn2(Asura_AVector3(xz + wy, yz - wx, 1.0f - (xx+yy)));
}

__forceinline void Asura_Quat::FromMatrix(const Asura_AMatrix3x3& xM4)
{
	Asura_Matrix_3x3 xM3;
	xM4.ToMat3(xM3);
	FromMatrix(xM3);
}

__forceinline Asura_AVector3 Asura_Quat::RotateVector(const Asura_AVector3& p) const
{
	//-----------------------------------------------------
//
	// This function rotates a vector p by the quaternion q
	// The formula is:
	//                  p' = (q)(p)(q*)
//
	// but we calculate it using a vector formula which is an
	// expansion of this that Kev worked out.
	// for q = (v,w)
	// v' = 2w(v^p) + (2(v.p)v + (w^2-v^2)p
//
	//--------------------------------------------------------

	const Asura_AVector3 v(x,y,z); // the vector part of q
	Asura_AVector3 section1, section2, section3;

	float vMagSq = v.MagnitudeSquared();

	section1 = v^p;
	section1 *= (2*w);

	section2 = v;
	section2 *= (2*(v*p));

	section3 = p;
	section3 *= (w*w - vMagSq);

	return section1+section2+section3;
}

__forceinline void Asura_Quat::FromRotation(Asura_Angle angle, const Asura_AVector3& axis)
{
	Asura_Assert(Asura_Maths::Abs(axis.MagnitudeSquared() - 1.0f) < 0.0001f, "axis should be normalised");

	angle *= 0.5f;
	float s;
	Asura_Maths::SinCos(angle, s, w);
	x = axis.GetX() * s;
	y = axis.GetY() * s;
	z = axis.GetZ() * s;
}


//////////////////////////////////////////////////////////////////////////
// Asura_Vector_3
//////////////////////////////////////////////////////////////////////////
inline Asura_Vector_3::Asura_Vector_3(const Asura_AVector3& xOther)
{
	xOther.StoreUnaligned(&x);
}

inline void Asura_Vector_3::operator=(const Asura_AVector3& xOther)
{
	xOther.StoreUnaligned(&x);
}


//////////////////////////////////////////////////////////////////////////
// Asura_AVector3
//////////////////////////////////////////////////////////////////////////
inline Asura_AVector3::Asura_AVector3(void)
{
}

inline Asura_AVector3::Asura_AVector3(const float fX, const float fY, const float fZ)
{
	const XMVECTOR	xTemp	= {fX, fY, fZ, 0.0f};

	m_xVector4	= xTemp;
}

inline Asura_AVector3::Asura_AVector3(const Asura_Vector_3& xOther)
{
	const XMVECTOR	xTemp	= {xOther.GetX(), xOther.GetY(), xOther.GetZ(), 0.0f};

	m_xVector4	= xTemp;
}

inline void Asura_AVector3::Zero(void)
{
	m_xVector4	= XMVectorZero();
}

inline void Asura_AVector3::LoadAligned(const float* pfFloats)
{
	m_xVector4	= XMLoadVector3A(pfFloats);
}

inline void Asura_AVector3::LoadUnaligned(const float* pfFloats)
{
	m_xVector4	= XMLoadVector3(pfFloats);
}

inline void Asura_AVector3::StoreAligned(float* pfFloats) const
{
	XMStoreVector3A(pfFloats, m_xVector4);
}

inline void Asura_AVector3::StoreUnaligned(float* pfFloats) const
{
	XMStoreVector3(pfFloats, m_xVector4);
}

inline void Asura_AVector3::StoreAlignedHalf(Asura_HalfFloat* pxHalves) const
{
	// NB can't use XMStoreHalf4 here because we must only write out 3 components
	pxHalves[0] = m_xVector4.x;
	pxHalves[1] = m_xVector4.y;
	pxHalves[2] = m_xVector4.z;
}

inline float Asura_AVector3::MagnitudeSquared(void) const
{
	const XMVECTOR	xMagnitudeSquared	= XMVector3Dot(m_xVector4, m_xVector4);

	return(xMagnitudeSquared.x);
}

inline float Asura_AVector3::Magnitude(void) const
{
	const XMVECTOR	xMagnitude	= XMVector3Length(m_xVector4);

	return(xMagnitude.x);
}

inline void Asura_AVector3::Normalise(void)
{
	m_xVector4	= XMVector3Normalize(m_xVector4);
}

inline float Asura_AVector3::NormaliseReturningMagnitude(void)
{
	const XMVECTOR	xMagnitude	= XMVector3Length(m_xVector4);

	if(xMagnitude.x == 0.0f)
	{
		return(0.0f);
	}
	else
	{
		const XMVECTOR	xReciprocalMagnitude	= XMVectorReciprocal(xMagnitude);

		m_xVector4	= XMVectorMultiply(m_xVector4, xReciprocalMagnitude);
		return(xMagnitude.x);
	}
}

inline void Asura_AVector3::ApproximateNormalise(void)
{
	m_xVector4	= XMVector3NormalizeEst(m_xVector4);
}

inline float Asura_AVector3::ApproximateNormaliseReturningMagnitude(void)
{
	const XMVECTOR	xMagnitude	= XMVector3LengthEst(m_xVector4);

	if(xMagnitude.x == 0.0f)
	{
		return(0.0f);
	}
	else
	{
		const XMVECTOR	xReciprocalMagnitude	= XMVectorReciprocalEst(xMagnitude);

		m_xVector4	= XMVectorMultiply(m_xVector4, xReciprocalMagnitude);
		return(xMagnitude.x);
	}
}

inline void Asura_AVector3::Lerp(Asura_AVector3_InParam xTo, const float fWeight)
{
	m_xVector4	= XMVectorLerp(m_xVector4, xTo.m_xVector4, fWeight);
}

inline void Asura_AVector3::Lerp(Asura_AVector3_InParam xFrom, Asura_AVector3_InParam xTo, const float fWeight)
{
	m_xVector4	= XMVectorLerp(xFrom.m_xVector4, xTo.m_xVector4, fWeight);
}

inline void Asura_AVector3::Max(Asura_AVector3_InParam xV1, Asura_AVector3_InParam xV2)
{
	m_xVector4	= XMVectorMax(xV1.m_xVector4, xV2.m_xVector4);
}

inline void Asura_AVector3::Min(Asura_AVector3_InParam xV1, Asura_AVector3_InParam xV2)
{
	m_xVector4	= XMVectorMin(xV1.m_xVector4, xV2.m_xVector4);
}

inline void Asura_AVector3::operator=(const Asura_Vector_3& xOther)
{
	const Asura_AVector3	xTemp(xOther);

	m_xVector4	= xTemp.m_xVector4;
}

inline void Asura_AVector3::operator+=(Asura_AVector3_InParam xOther)
{
	m_xVector4	= XMVectorAdd(m_xVector4, xOther.m_xVector4);
}

inline void Asura_AVector3::operator-=(Asura_AVector3_InParam xOther)
{
	m_xVector4	= XMVectorSubtract(m_xVector4, xOther.m_xVector4);
}

inline void Asura_AVector3::operator*=(const float fScale)
{
	m_xVector4	= XMVectorScale(m_xVector4, fScale);
}

inline void Asura_AVector3::operator/=(const float fScale)
{
	const XMVECTOR	xScale				= XMVectorReplicate(fScale);
	const XMVECTOR	xReciprocalScale	= XMVectorReciprocal(xScale);

	m_xVector4	= XMVectorMultiply(m_xVector4, xReciprocalScale);
}

inline void Asura_AVector3::operator*=(Asura_AVector3_InParam xOther)
{
	m_xVector4	= XMVectorMultiply(m_xVector4, xOther.m_xVector4);
}

inline Asura_AVector3 Asura_AVector3::operator-(void) const
{
	Asura_AVector3	xResult;

	xResult.m_xVector4	= XMVectorNegate(m_xVector4);
	return(xResult);
}

inline Asura_AVector3 Asura_AVector3::operator+(Asura_AVector3_InParam xOther) const
{
	Asura_AVector3	xResult;

	xResult.m_xVector4	= XMVectorAdd(m_xVector4, xOther.m_xVector4);
	return(xResult);
}

inline Asura_AVector3 Asura_AVector3::operator-(Asura_AVector3_InParam xOther) const
{
	Asura_AVector3	xResult;

	xResult.m_xVector4	= XMVectorSubtract(m_xVector4, xOther.m_xVector4);
	return(xResult);
}

inline Asura_AVector3 Asura_AVector3::operator*(const float fScale) const
{
	Asura_AVector3	xResult;

	xResult.m_xVector4	= XMVectorScale(m_xVector4, fScale);
	return(xResult);
}

inline Asura_AVector3 Asura_AVector3::operator/(const float fScale) const
{
	const XMVECTOR	xScale				= XMVectorReplicate(fScale);
	const XMVECTOR	xReciprocalScale	= XMVectorReciprocal(xScale);
	Asura_AVector3	xResult;

	xResult.m_xVector4	= XMVectorMultiply(m_xVector4, xReciprocalScale);
	return(xResult);
}

inline Asura_AVector3 Asura_AVector3::operator^(Asura_AVector3_InParam xOther) const
{
	Asura_AVector3	xResult;

	xResult.m_xVector4	= XMVector3Cross(m_xVector4, xOther.m_xVector4);
	return(xResult);
}

inline float Asura_AVector3::operator*(Asura_AVector3_InParam xOther) const
{
	const XMVECTOR	xDot	= XMVector3Dot(m_xVector4, xOther.m_xVector4);

	return(xDot.x);
}

inline bool Asura_AVector3::ApproximatelyEquals(Asura_AVector3_InParam xOther, const float fEpsilon) const
{
	const XMVECTOR	xDifference	= XMVectorSubtract(m_xVector4, xOther.m_xVector4);
	const XMVECTOR	xResult		= XMVector3Dot(xDifference, xDifference);

	return(xResult.x < fEpsilon);
}

inline bool Asura_AVector3::operator==(Asura_AVector3_InParam xOther) const
{
	return(XMVector3Equal(m_xVector4, xOther.m_xVector4) == TRUE);
}

inline bool Asura_AVector3::operator!=(Asura_AVector3_InParam xOther) const
{
	return(XMVector3NotEqual(m_xVector4, xOther.m_xVector4) == TRUE);
}

inline bool Asura_AVector3::operator<(Asura_AVector3_InParam xOther) const
{
	return(XMVector3Less(m_xVector4, xOther.m_xVector4) == TRUE);
}

inline bool Asura_AVector3::operator<=(Asura_AVector3_InParam xOther) const
{
	return(XMVector3LessOrEqual(m_xVector4, xOther.m_xVector4) == TRUE);
}

inline bool Asura_AVector3::operator>=(Asura_AVector3_InParam xOther) const
{
	return(XMVector3GreaterOrEqual(m_xVector4, xOther.m_xVector4) == TRUE);
}

inline bool Asura_AVector3::operator>(Asura_AVector3_InParam xOther) const
{
	return(XMVector3Greater(m_xVector4, xOther.m_xVector4) == TRUE);
}

inline float Asura_AVector3::GetX(void) const
{
	return(m_xVector4.x);
}

inline float Asura_AVector3::GetY(void) const
{
	return(m_xVector4.y);
}

inline float Asura_AVector3::GetZ(void) const
{
	return(m_xVector4.z);
}

inline float Asura_AVector3::GetElement(const u_int uIndex) const 
{ 
	return(m_xVector4.v[uIndex]);
}

inline void Asura_AVector3::SetX(const float fX)
{
	m_xVector4.x = fX;
}

inline void Asura_AVector3::SetY(const float fY)
{
	m_xVector4.y = fY;
}

inline void Asura_AVector3::SetZ(const float fZ)
{
	m_xVector4.z = fZ;
}

inline void Asura_AVector3::SetElement(const u_int uIndex, const float fValue)
{
	m_xVector4.v[uIndex]	= fValue;
}


//////////////////////////////////////////////////////////////////////////
// Asura_AMatrix3x3
//////////////////////////////////////////////////////////////////////////
inline Asura_AMatrix3x3::Asura_AMatrix3x3(void)
{
}

inline Asura_AMatrix3x3::Asura_AMatrix3x3(const Asura_Matrix_3x3& xMat3)
{
	FromMat3(xMat3);
}

inline Asura_AMatrix3x3::Asura_AMatrix3x3(const float fE11, const float fE12, const float fE13,
										  const float fE21, const float fE22, const float fE23,
										  const float fE31, const float fE32, const float fE33)
{
	m_axColumns[0]	= Asura_AVector3(fE11, fE21, fE31);
	m_axColumns[1]	= Asura_AVector3(fE12, fE22, fE32);
	m_axColumns[2]	= Asura_AVector3(fE13, fE23, fE33);
}

inline Asura_AMatrix3x3::Asura_AMatrix3x3(const Asura_AVector3& xCol0, const Asura_AVector3& xCol1, const Asura_AVector3& xCol2)
{
	m_axColumns[0] = xCol0;
	m_axColumns[1] = xCol1;
	m_axColumns[2] = xCol2;
}

inline void Asura_AMatrix3x3::ToMat3(Asura_Matrix_3x3& xMat3) const
{
	xMat3.e11 = GetElement(0,0);
	xMat3.e12 = GetElement(0,1);
	xMat3.e13 = GetElement(0,2);
	xMat3.e21 = GetElement(1,0);
	xMat3.e22 = GetElement(1,1);
	xMat3.e23 = GetElement(1,2);
	xMat3.e31 = GetElement(2,0);
	xMat3.e32 = GetElement(2,1);
	xMat3.e33 = GetElement(2,2);
}

inline void Asura_AMatrix3x3::FromMat3(const Asura_Matrix_3x3& xMat3)
{
	m_axColumns[0] = Asura_AVector3(xMat3.e11, xMat3.e21, xMat3.e31);
	m_axColumns[1] = Asura_AVector3(xMat3.e12, xMat3.e22, xMat3.e32);
	m_axColumns[2] = Asura_AVector3(xMat3.e13, xMat3.e23, xMat3.e33);
}

inline Asura_AVector3 Asura_AMatrix3x3::GetColumn0(void) const
{
	return(m_axColumns[0]);
}

inline Asura_AVector3 Asura_AMatrix3x3::GetColumn1(void) const
{
	return(m_axColumns[1]);
}

inline Asura_AVector3 Asura_AMatrix3x3::GetColumn2(void) const
{
	return(m_axColumns[2]);
}

inline Asura_AVector3 Asura_AMatrix3x3::GetColumn(const u_int uIndex) const
{
	return(m_axColumns[uIndex]);
}

inline Asura_AVector3 Asura_AMatrix3x3::GetRow0(void) const
{
	return(Asura_AVector3(m_axColumns[0].GetX(), m_axColumns[1].GetX(), m_axColumns[2].GetX()));
}

inline Asura_AVector3 Asura_AMatrix3x3::GetRow1(void) const
{
	return(Asura_AVector3(m_axColumns[0].GetY(), m_axColumns[1].GetY(), m_axColumns[2].GetY()));
}

inline Asura_AVector3 Asura_AMatrix3x3::GetRow2(void) const
{
	return(Asura_AVector3(m_axColumns[0].GetZ(), m_axColumns[1].GetZ(), m_axColumns[2].GetZ()));
}

inline Asura_AVector3 Asura_AMatrix3x3::GetRow(const u_int uIndex) const
{
	return(Asura_AVector3(m_axColumns[0].GetElement(uIndex), m_axColumns[1].GetElement(uIndex), m_axColumns[2].GetElement(uIndex)));
}

inline float Asura_AMatrix3x3::GetElement(const u_int uRow, const u_int uCol) const
{
	return(GetColumn(uCol).GetElement(uRow));
}

inline void Asura_AMatrix3x3::SetColumn0(Asura_AVector3_InParam xCol0)
{
	m_axColumns[0]	= xCol0;
}

inline void Asura_AMatrix3x3::SetColumn1(Asura_AVector3_InParam xCol1)
{
	m_axColumns[1]	= xCol1;
}

inline void Asura_AMatrix3x3::SetColumn2(Asura_AVector3_InParam xCol2)
{
	m_axColumns[2]	= xCol2;
}

inline void Asura_AMatrix3x3::SetColumn(const u_int uIndex, Asura_AVector3_InParam xColumn)
{
	m_axColumns[uIndex]	= xColumn;
}

inline void Asura_AMatrix3x3::SetRow0(Asura_AVector3_InParam xRow0)
{
	m_axColumns[0].SetX(xRow0.GetX());
	m_axColumns[1].SetX(xRow0.GetY());
	m_axColumns[2].SetX(xRow0.GetZ());
}

inline void Asura_AMatrix3x3::SetRow1(Asura_AVector3_InParam xRow1)
{
	m_axColumns[0].SetY(xRow1.GetX());
	m_axColumns[1].SetY(xRow1.GetY());
	m_axColumns[2].SetY(xRow1.GetZ());
}

inline void Asura_AMatrix3x3::SetRow2(Asura_AVector3_InParam xRow2)
{
	m_axColumns[0].SetZ(xRow2.GetX());
	m_axColumns[1].SetZ(xRow2.GetY());
	m_axColumns[2].SetZ(xRow2.GetZ());
}

inline void Asura_AMatrix3x3::SetRow(const u_int uIndex, Asura_AVector3_InParam xRow)
{
	m_axColumns[0].SetElement(uIndex, xRow.GetX());
	m_axColumns[1].SetElement(uIndex, xRow.GetY());
	m_axColumns[2].SetElement(uIndex, xRow.GetZ());
}

inline void Asura_AMatrix3x3::SetElement(const u_int uRow, const u_int uCol, const float fValue)
{
	m_axColumns[uCol].SetElement(uRow, fValue);
}

inline void Asura_AMatrix3x3::Tilde(Asura_AVector3_InParam xVector)
{
	m_axColumns[0] = Asura_AVector3(0.0f,				+xVector.GetZ(),	-xVector.GetY());
	m_axColumns[1] = Asura_AVector3(-xVector.GetZ(),	0.0f,				+xVector.GetX());
	m_axColumns[2] = Asura_AVector3(+xVector.GetY(),	-xVector.GetX(),	0.0f			);
}

inline void Asura_AMatrix3x3::Orthogonalise(void)
{
	// normalise "down" column
	m_axColumns[1].Normalise();

	// make "forwards" column perpendicular to this, and normalise
	m_axColumns[2] -= m_axColumns[1] * (m_axColumns[2] * m_axColumns[1]);
	m_axColumns[2].Normalise();

	// and now the "right" column must be perpendicular to both of these: x = + y^z
	m_axColumns[0] = m_axColumns[1] ^ m_axColumns[2];

}

inline void Asura_AMatrix3x3::Add(const Asura_AMatrix3x3& xM) 
{
	m_axColumns[0] += xM.m_axColumns[0];
	m_axColumns[1] += xM.m_axColumns[1];
	m_axColumns[2] += xM.m_axColumns[2];
}

inline Asura_AVector3 Asura_AMatrix3x3::Apply(Asura_AVector3_InParam xInput) const
{
	// KJFL - 21 July 2008 - this function was worryingly coming up on profiles,
	// so here's a native vector version.

	const XMVECTOR xX = XMVectorSplatX( xInput.m_xVector4);
	const XMVECTOR xY = XMVectorSplatY( xInput.m_xVector4);
	const XMVECTOR xZ = XMVectorSplatZ( xInput.m_xVector4);
	
	XMVECTOR xResult;
	xResult = XMVectorMultiply( GetColumn0().m_xVector4, xX );
	xResult = XMVectorMultiplyAdd( GetColumn1().m_xVector4, xY, xResult );
	xResult = XMVectorMultiplyAdd( GetColumn2().m_xVector4, xZ, xResult );
	Asura_AVector3 xOutput;
	xOutput.m_xVector4 = xResult;
	return xOutput;
}

inline void Asura_AMatrix3x3::Apply(Asura_AMatrix3x3_InParam xInput, Asura_AMatrix3x3_OutParam xOutput) const
{
	xOutput = Asura_AMatrix3x3(Apply(xInput.m_axColumns[0]),Apply(xInput.m_axColumns[1]),Apply(xInput.m_axColumns[2]));
}

inline void Asura_AMatrix3x3::Transpose(void)
{
	float f = m_axColumns[0].GetY();
	m_axColumns[0].SetY(m_axColumns[1].GetX());
	m_axColumns[1].SetX(f);

	f = m_axColumns[0].GetZ();
	m_axColumns[0].SetZ(m_axColumns[2].GetX());
	m_axColumns[2].SetX(f);

	f = m_axColumns[1].GetZ();
	m_axColumns[1].SetZ(m_axColumns[2].GetY());
	m_axColumns[2].SetY(f);
}

inline Asura_AVector3 Asura_AMatrix3x3::ApplyTranspose(Asura_AVector3_InParam xVec) const
{
	return(Asura_AVector3((xVec * m_axColumns[0]), (xVec * m_axColumns[1]), (xVec * m_axColumns[2])));
}

inline void Asura_AMatrix3x3::Identity(void)
{
	m_axColumns[0] = Asura_AVector3(1.0f, 0.0f, 0.0f);
	m_axColumns[1] = Asura_AVector3(0.0f, 1.0f, 0.0f);
	m_axColumns[2] = Asura_AVector3(0.0f, 0.0f, 1.0f);
}

inline void Asura_AMatrix3x3::Zero(void)
{
	m_axColumns[0].Zero();
	m_axColumns[1].Zero();
	m_axColumns[2].Zero();
}

inline Asura_AMatrix3x3 Asura_AMatrix3x3::operator*(const float fScale) const
{
	Asura_AMatrix3x3	xResult((m_axColumns[0] * fScale), (m_axColumns[1] * fScale), (m_axColumns[2] * fScale));

	return(xResult);
}

inline bool Asura_AMatrix3x3::operator==(Asura_AMatrix3x3_InParam xMat) const
{
	return((m_axColumns[0] == xMat.m_axColumns[0]) && (m_axColumns[1] == xMat.m_axColumns[1]) && (m_axColumns[2] == xMat.m_axColumns[2]));
}

inline bool Asura_AMatrix3x3::operator!=(Asura_AMatrix3x3_InParam xMat) const
{
	return(!(*this==xMat));
}


//////////////////////////////////////////////////////////////////////////
// Asura_AVector4
//////////////////////////////////////////////////////////////////////////
__forceinline Asura_AVector4::Asura_AVector4(void)
{
}

__forceinline Asura_AVector4::Asura_AVector4(const float fX, const float fY, const float fZ, const float fW)
{
	const XMVECTOR	xTemp	= {fX, fY, fZ, fW};

	m_xVector4	= xTemp;
}

__forceinline Asura_AVector4::Asura_AVector4(const Asura_AVector3& xVec, const float fW)
{
	const XMVECTOR	xTemp	= {xVec.GetX(), xVec.GetY(), xVec.GetZ(), fW};

	m_xVector4	= xTemp;
}

__forceinline Asura_AVector4::Asura_AVector4(const Asura_Vector_3& xVec, const float fW)
{
	const XMVECTOR	xTemp	= {xVec.GetX(), xVec.GetY(), xVec.GetZ(), fW};

	m_xVector4	= xTemp;
}

__forceinline void Asura_AVector4::LoadAligned(const float* pfFloats)
{
	m_xVector4	= XMLoadVector4A(pfFloats);
}

__forceinline void Asura_AVector4::LoadUnaligned(const float* pfFloats)
{
	m_xVector4	= XMLoadVector4(pfFloats);
}

__forceinline void Asura_AVector4::StoreAligned(float* pfFloats) const
{
	XMStoreVector4A(pfFloats, m_xVector4);
}

__forceinline void Asura_AVector4::StoreUnaligned(float* pfFloats) const
{
	XMStoreVector4(pfFloats, m_xVector4);
}

__forceinline void Asura_AVector4::StoreAlignedHalf(Asura_HalfFloat* pxHalves) const
{
	XMStoreHalf4(reinterpret_cast<XMHALF4*>(pxHalves), m_xVector4);
}

__forceinline void Asura_AVector4::ZeroW0(void)
{
	m_xVector4	= XMVectorZero();
}

__forceinline void Asura_AVector4::ZeroW1(void)
{
	m_xVector4 = XMVectorZero();
	m_xVector4.w = 1.0f;
}

__forceinline void Asura_AVector4::operator+=(Asura_AVector4_InParam xOther)
{
	m_xVector4	= XMVectorAdd(m_xVector4, xOther.m_xVector4);
}

__forceinline void Asura_AVector4::operator-=(Asura_AVector4_InParam xOther)
{
	m_xVector4	= XMVectorSubtract(m_xVector4, xOther.m_xVector4);
}

__forceinline void Asura_AVector4::operator*=(const float fScale)
{
	m_xVector4	= XMVectorScale(m_xVector4, fScale);
}

__forceinline void Asura_AVector4::operator/=(const float fScale)
{
	const XMVECTOR	xScale				= XMVectorReplicate(fScale);
	const XMVECTOR	xReciprocalScale	= XMVectorReciprocal(xScale);

	m_xVector4	= XMVectorMultiply(m_xVector4, xReciprocalScale);
}

__forceinline void Asura_AVector4::Normalise(void)
{
	m_xVector4	= XMVector4Normalize(m_xVector4);
}

__forceinline float Asura_AVector4::NormaliseReturningMagnitude(void)
{
	const XMVECTOR	xMagnitude	= XMVector4Length(m_xVector4);

	if(xMagnitude.x == 0.0f)
	{
		return(0.0f);
	}
	else
	{
		const XMVECTOR	xReciprocalMagnitude	= XMVectorReciprocal(xMagnitude);

		m_xVector4	= XMVectorMultiply(m_xVector4, xReciprocalMagnitude);
		return(xMagnitude.x);
	}
}

__forceinline void Asura_AVector4::ApproximateNormalise(void)
{
	m_xVector4	= XMVector4NormalizeEst(m_xVector4);
}

__forceinline float Asura_AVector4::ApproximateNormaliseReturningMagnitude(void)
{
	const XMVECTOR	xMagnitude	= XMVector4LengthEst(m_xVector4);

	if(xMagnitude.x == 0.0f)
	{
		return(0.0f);
	}
	else
	{
		const XMVECTOR	xReciprocalMagnitude	= XMVectorReciprocalEst(xMagnitude);

		m_xVector4	= XMVectorMultiply(m_xVector4, xReciprocalMagnitude);
		return(xMagnitude.x);
	}
}

__forceinline float Asura_AVector4::MagnitudeSquared(void) const
{
	const XMVECTOR	xMagnitudeSqaured	= XMVector4Dot(m_xVector4, m_xVector4);

	return(xMagnitudeSqaured.x);
}

__forceinline float Asura_AVector4::Magnitude(void) const
{
	const XMVECTOR	xMagnitude	= XMVector4Length(m_xVector4);

	return(xMagnitude.x);
}

__forceinline Asura_AVector4 Asura_AVector4::ComponentwiseProduct(Asura_AVector4_InParam xOther) const
{
	Asura_AVector4	xResult;

	xResult.m_xVector4	= XMVectorMultiply(m_xVector4, xOther.m_xVector4);
	return(xResult);
}

__forceinline float Asura_AVector4::DotProduct3(Asura_AVector4_InParam xOther) const
{
	const XMVECTOR	xResult	= XMVector3Dot(m_xVector4, xOther.m_xVector4);

	return(xResult.x);
}

__forceinline float Asura_AVector4::DotProduct4(Asura_AVector4_InParam xOther) const
{
	const XMVECTOR	xResult	= XMVector4Dot(m_xVector4, xOther.m_xVector4);

	return(xResult.x);
}

__forceinline void Asura_AVector4::Lerp(Asura_AVector4_InParam xTo, const float fWeight)
{
	m_xVector4	= XMVectorLerp(m_xVector4, xTo.m_xVector4, fWeight);
}

__forceinline void Asura_AVector4::Lerp(Asura_AVector4_InParam xFrom, Asura_AVector4_InParam xTo, const float fWeight)
{
	m_xVector4	= XMVectorLerp(xFrom.m_xVector4, xTo.m_xVector4, fWeight);
}

__forceinline Asura_AVector4 Asura_AVector4::operator-(void) const
{
	Asura_AVector4	xReturn;

	xReturn.m_xVector4	= XMVectorNegate(m_xVector4);
	return(xReturn);
}

__forceinline Asura_AVector4 Asura_AVector4::operator+(Asura_AVector4_InParam xOther) const
{
	Asura_AVector4	xReturn;

	xReturn.m_xVector4	= XMVectorAdd(m_xVector4, xOther.m_xVector4);
	return(xReturn);
}

__forceinline Asura_AVector4 Asura_AVector4::operator-(Asura_AVector4_InParam xOther) const
{
	Asura_AVector4	xReturn;

	xReturn.m_xVector4	= XMVectorSubtract(m_xVector4, xOther.m_xVector4);
	return(xReturn);
}

__forceinline Asura_AVector4 Asura_AVector4::operator*(const float fScale) const
{
	Asura_AVector4	xReturn;

	xReturn.m_xVector4	= XMVectorScale(m_xVector4, fScale);
	return(xReturn);
}

__forceinline Asura_AVector4 Asura_AVector4::operator/(const float fScale) const
{
	const XMVECTOR	xScale		= XMVectorReplicate(fScale);
	const XMVECTOR	xReciprocal	= XMVectorReciprocal(xScale);
	Asura_AVector4	xReturn;

	xReturn.m_xVector4	= XMVectorMultiply(m_xVector4, xReciprocal);
	return(xReturn);
}

__forceinline float Asura_AVector4::operator*(Asura_AVector4_InParam xOther) const
{
	const XMVECTOR	xResult	= XMVector4Dot(m_xVector4, xOther.m_xVector4);

	return(xResult.x);
}

__forceinline Asura_AVector4 Asura_AVector4::operator^(Asura_AVector4_InParam xOther) const
{
	Asura_AVector4	xResult;

	xResult.m_xVector4	= XMVector3Cross(m_xVector4, xOther.m_xVector4);
	return(xResult);	
}

__forceinline bool Asura_AVector4::ApproximatelyEquals(Asura_AVector4_InParam xOther, const float fEpsilon) const
{
	const XMVECTOR	xDifference	= XMVectorSubtract(m_xVector4, xOther.m_xVector4);
	const XMVECTOR	xResult		= XMVector4Dot(xDifference, xDifference);

	return(xResult.x < fEpsilon);
}

__forceinline bool Asura_AVector4::operator==(Asura_AVector4_InParam xOther) const
{
	return(XMVector4Equal(m_xVector4, xOther.m_xVector4) == TRUE);
}

__forceinline bool Asura_AVector4::operator!=(Asura_AVector4_InParam xOther) const
{
	return(XMVector4NotEqual(m_xVector4, xOther.m_xVector4) == TRUE);
}

__forceinline bool Asura_AVector4::operator<(Asura_AVector4_InParam xOther) const
{
	return(XMVector4Less(m_xVector4, xOther.m_xVector4) == TRUE);
}

__forceinline bool Asura_AVector4::operator<=(Asura_AVector4_InParam xOther) const
{
	return(XMVector4LessOrEqual(m_xVector4, xOther.m_xVector4) == TRUE);
}

__forceinline bool Asura_AVector4::operator>=(Asura_AVector4_InParam xOther) const
{
	return(XMVector4GreaterOrEqual(m_xVector4, xOther.m_xVector4) == TRUE);
}

__forceinline bool Asura_AVector4::operator>(Asura_AVector4_InParam xOther) const
{
	return(XMVector4Greater(m_xVector4, xOther.m_xVector4) == TRUE);
}

__forceinline void Asura_AVector4::Max(Asura_AVector4_InParam xV1, Asura_AVector4_InParam xV2)
{
	m_xVector4	= XMVectorMax(xV1.m_xVector4, xV2.m_xVector4);
}

__forceinline void Asura_AVector4::Min(Asura_AVector4_InParam xV1, Asura_AVector4_InParam xV2)
{
	m_xVector4	= XMVectorMin(xV1.m_xVector4, xV2.m_xVector4);
}

__forceinline float Asura_AVector4::GetX(void) const
{
	return(m_xVector4.x);
}

__forceinline float Asura_AVector4::GetY(void) const
{
	return(m_xVector4.y);
}

__forceinline float Asura_AVector4::GetZ(void) const
{
	return(m_xVector4.z);
}

__forceinline float Asura_AVector4::GetW(void) const
{
	return(m_xVector4.w);
}

__forceinline float Asura_AVector4::GetElement(const u_int uIndex) const 
{ 
	return(m_xVector4.v[uIndex]);
}

__forceinline void Asura_AVector4::SetX(const float fX)
{
	m_xVector4.x = fX;
}

__forceinline void Asura_AVector4::SetY(const float fY)
{
	m_xVector4.y = fY;
}

__forceinline void Asura_AVector4::SetZ(const float fZ)
{
	m_xVector4.z = fZ;
}

__forceinline void Asura_AVector4::SetW(const float fW)
{
	m_xVector4.w = fW;
}

__forceinline void Asura_AVector4::SetElement(const u_int uIndex, const float fValue) 
{
	m_xVector4.v[uIndex]	= fValue;
}

__forceinline void Asura_AVector4::SetElements(const float fX, const float fY, const float fZ, const float fW)
{
	const XMVECTOR	xTemp	= {fX, fY, fZ, fW};

	m_xVector4	= xTemp;
}


//////////////////////////////////////////////////////////////////////////
// Asura_AMatrix4x4
//////////////////////////////////////////////////////////////////////////
__forceinline Asura_AMatrix4x4::Asura_AMatrix4x4(void)
{
}

__forceinline Asura_AMatrix4x4::Asura_AMatrix4x4(const float fE11, const float fE12, const float fE13, const float fE14,
												 const float fE21, const float fE22, const float fE23, const float fE24,
												 const float fE31, const float fE32, const float fE33, const float fE34,
												 const float fE41, const float fE42, const float fE43, const float fE44)
{
	m_axColumns[0].SetElements(fE11, fE21, fE31, fE41);
	m_axColumns[1].SetElements(fE12, fE22, fE32, fE42);
	m_axColumns[2].SetElements(fE13, fE23, fE33, fE43);
	m_axColumns[3].SetElements(fE14, fE24, fE34, fE44);
}

__forceinline Asura_AMatrix4x4::Asura_AMatrix4x4(const Asura_AMatrix4x4& xOther)
{
	m_axColumns[0]	= xOther.m_axColumns[0];
	m_axColumns[1]	= xOther.m_axColumns[1];
	m_axColumns[2]	= xOther.m_axColumns[2];
	m_axColumns[3]	= xOther.m_axColumns[3];
}

__forceinline Asura_AMatrix4x4::Asura_AMatrix4x4(const Asura_Matrix_3x3& xRotation, const Asura_Vector_3& xPosition)
{
	m_axColumns[0].SetElements(xRotation.e11,		xRotation.e21,		xRotation.e31,		0.0f);
	m_axColumns[1].SetElements(xRotation.e12,		xRotation.e22,		xRotation.e32,		0.0f);
	m_axColumns[2].SetElements(xRotation.e13,		xRotation.e23,		xRotation.e33,		0.0f);
	m_axColumns[3].SetElements(xPosition.GetX(),	xPosition.GetY(),	xPosition.GetZ(),	1.0f);
}

__forceinline Asura_AMatrix4x4::Asura_AMatrix4x4(const Asura_Quat& xRotation, const Asura_Vector_3& xPosition)
{
	const float	fS	= 2.0f;	//xRotation.Magnitude();

	const float	fXS	= xRotation.x * fS;
	const float	fYS	= xRotation.y * fS;
	const float	fZS	= xRotation.z * fS;

	const float	fWX	= xRotation.w * fXS;
	const float	fWY	= xRotation.w * fYS;
	const float	fWZ	= xRotation.w * fZS;

	const float	fXX	= xRotation.x * fXS;
	const float	fXY	= xRotation.x * fYS;
	const float	fXZ	= xRotation.x * fZS;

	const float	fYY	= xRotation.y * fYS;
	const float	fYZ	= xRotation.y * fZS;
	const float	fZZ	= xRotation.z * fZS;

	// Implementation from Graphics Gems IV...
	m_axColumns[0].SetElements(1.0f - (fYY + fZZ),	fXY + fWZ,			fXZ - fWY,			0.0f);
	m_axColumns[1].SetElements(fXY - fWZ,			1.0f - (fXX + fZZ),	fYZ + fWX,			0.0f);
	m_axColumns[2].SetElements(fXZ + fWY,			fYZ - fWX,			1.0f - (fXX + fYY),	0.0f);
	m_axColumns[3].SetElements(xPosition.GetX(),	xPosition.GetY(),	xPosition.GetZ(),	1.0f);
}

__forceinline void Asura_AMatrix4x4::SetColumns(Asura_AVector4_InParam xCol0, Asura_AVector4_InParam xCol1, Asura_AVector4_InParam xCol2, Asura_AVector4_InParam xCol3)
{
	m_axColumns[0]	= xCol0;
	m_axColumns[1]	= xCol1;
	m_axColumns[2]	= xCol2;
	m_axColumns[3]	= xCol3;
}

__forceinline void Asura_AMatrix4x4::SetRows(Asura_AVector4_InParam xRow0, Asura_AVector4_InParam xRow1, Asura_AVector4_InParam xRow2, Asura_AVector4_InParam xRow3)
{
	SetRow0(xRow0);
	SetRow1(xRow1);
	SetRow2(xRow2);
	SetRow3(xRow3);
}

__forceinline void Asura_AMatrix4x4::Identity(void)
{
	m_axColumns[0].SetElements(1.0f, 0.0f, 0.0f, 0.0f);
	m_axColumns[1].SetElements(0.0f, 1.0f, 0.0f, 0.0f);
	m_axColumns[2].SetElements(0.0f, 0.0f, 1.0f, 0.0f);
	m_axColumns[3].ZeroW1();
}

__forceinline void Asura_AMatrix4x4::Zero(void)
{
	m_axColumns[0].ZeroW0();
	m_axColumns[1].ZeroW0();
	m_axColumns[2].ZeroW0();
	m_axColumns[3].ZeroW0();
}

__forceinline void Asura_AMatrix4x4::ToMatrix(Asura_Matrix_3x3& xOrientation) const
{
	xOrientation.e11	= m_axColumns[0].GetX();
	xOrientation.e21	= m_axColumns[0].GetY();
	xOrientation.e31	= m_axColumns[0].GetZ();

	xOrientation.e12	= m_axColumns[1].GetX();
	xOrientation.e22	= m_axColumns[1].GetY();
	xOrientation.e32	= m_axColumns[1].GetZ();

	xOrientation.e13	= m_axColumns[2].GetX();
	xOrientation.e23	= m_axColumns[2].GetY();
	xOrientation.e33	= m_axColumns[2].GetZ();
}

__forceinline void Asura_AMatrix4x4::FromMatrix(const Asura_Matrix_3x3& xOrientation)
{
	m_axColumns[0].SetElements(xOrientation.e11, xOrientation.e21, xOrientation.e31, 0.0f);
	m_axColumns[1].SetElements(xOrientation.e12, xOrientation.e22, xOrientation.e32, 0.0f);
	m_axColumns[2].SetElements(xOrientation.e13, xOrientation.e23, xOrientation.e33, 0.0f);
	m_axColumns[3].ZeroW1();
}

__forceinline Asura_Vector_3 Asura_AMatrix4x4::GetTranslation() const
{
	return Asura_Vector_3(m_axColumns[3].GetX(),m_axColumns[3].GetY(),m_axColumns[3].GetZ());
}

__forceinline void Asura_AMatrix4x4::SetTranslation(const Asura_Vector_3& xTranslation)
{
	m_axColumns[3]	= Asura_AVector4(xTranslation, 1.0f);
}

__forceinline Asura_AVector3 Asura_AMatrix4x4::GetATranslation() const
{
	return Asura_AVector3(m_axColumns[3].GetX(),m_axColumns[3].GetY(),m_axColumns[3].GetZ());
}

__forceinline void Asura_AMatrix4x4::SetTranslation(Asura_AVector3_InParam xTranslation)
{
	m_axColumns[3].SetElements(xTranslation.GetX(), xTranslation.GetY(), xTranslation.GetZ(), 1.0f);
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::GetA4Translation() const
{
	return m_axColumns[3];
}

__forceinline void Asura_AMatrix4x4::SetTranslation(Asura_AVector4_InParam xTranslation)
{
	m_axColumns[3]	= xTranslation;
}

__forceinline Asura_AMatrix4x4 Asura_AMatrix4x4::operator-(void) const
{
	Asura_AMatrix4x4	xTemp;

	xTemp.SetColumns(-m_axColumns[0], -m_axColumns[1], -m_axColumns[2], -m_axColumns[3]);
	return(xTemp);
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::Apply(Asura_AVector4_InParam xInput) const
{
	Asura_AVector4	axRows[4];

	// Extract the data...
	axRows[0] = GetRow0();
	axRows[1] = GetRow1();
	axRows[2] = GetRow2();
	axRows[3] = GetRow3();

	// Set each element of the result in turn...
	return Asura_AVector4((axRows[0] * xInput), (axRows[1] * xInput), (axRows[2] * xInput), (axRows[3] * xInput));
}

__forceinline void Asura_AMatrix4x4::ApplyToArray(const Asura_AVector4* pxInput, const u_int uNumElements, Asura_AVector4* pxOutput) const
{
	Asura_AVector4	axRows[4];

	// Extract the data...
	axRows[0] = GetRow0();
	axRows[1] = GetRow1();
	axRows[2] = GetRow2();
	axRows[3] = GetRow3();

	for(u_int u = 0; u < uNumElements; u++, pxInput++, pxOutput++)
	{
		// Set each element of the result in turn...
		pxOutput->SetElements((axRows[0] * *pxInput), (axRows[1] * *pxInput), (axRows[2] * *pxInput), (axRows[3] * *pxInput));
	}
}

__forceinline void Asura_AMatrix4x4::Apply(Asura_AMatrix4x4_InParam xInput, Asura_AMatrix4x4_OutParam xOutput) const
{
	const __vector4	xA1(xInput.m_axColumns[0].m_xVector4);
	const __vector4	xA2(xInput.m_axColumns[1].m_xVector4);
	const __vector4	xA3(xInput.m_axColumns[2].m_xVector4);
	const __vector4	xA4(xInput.m_axColumns[3].m_xVector4);

	const __vector4	xB1(m_axColumns[0].m_xVector4);
	const __vector4	xB2(m_axColumns[1].m_xVector4);
	const __vector4	xB3(m_axColumns[2].m_xVector4);
	const __vector4	xB4(m_axColumns[3].m_xVector4);

	__vector4		xC1;
	__vector4		xC2;
	__vector4		xC3;
	__vector4		xC4;

	// Do the first scalar x vector multiply for each row...
	xC1	= __vmulfp(__vspltw(xA1, 0), xB1);
	xC2	= __vmulfp(__vspltw(xA2, 0), xB1);
	xC3	= __vmulfp(__vspltw(xA3, 0), xB1);
	xC4	= __vmulfp(__vspltw(xA4, 0), xB1);

	// xAccumulate in the second scalar x vector multiply for each row...
	xC1	= __vmaddfp(__vspltw(xA1, 1), xB2, xC1);
	xC2	= __vmaddfp(__vspltw(xA2, 1), xB2, xC2);
	xC3	= __vmaddfp(__vspltw(xA3, 1), xB2, xC3);
	xC4	= __vmaddfp(__vspltw(xA4, 1), xB2, xC4);

	// xAccumulate in the third scalar x vector multiply for each row...
	xC1	= __vmaddfp(__vspltw(xA1, 2), xB3, xC1);
	xC2	= __vmaddfp(__vspltw(xA2, 2), xB3, xC2);
	xC3	= __vmaddfp(__vspltw(xA3, 2), xB3, xC3);
	xC4	= __vmaddfp(__vspltw(xA4, 2), xB3, xC4);

	// xAccumulate in the fourth scalar x vector multiply for each row...
	xC1	= __vmaddfp(__vspltw(xA1, 3), xB4, xC1);
	xC2	= __vmaddfp(__vspltw(xA2, 3), xB4, xC2);
	xC3	= __vmaddfp(__vspltw(xA3, 3), xB4, xC3);
	xC4	= __vmaddfp(__vspltw(xA4, 3), xB4, xC4);

	// Store out the result...
	xOutput.m_axColumns[0].m_xVector4	= xC1;
	xOutput.m_axColumns[1].m_xVector4	= xC2;
	xOutput.m_axColumns[2].m_xVector4	= xC3;
	xOutput.m_axColumns[3].m_xVector4	= xC4;
}

__forceinline void Asura_AMatrix4x4::ApplyToArray(const Asura_AMatrix4x4* pxInput, const u_int uNumElements, Asura_AMatrix4x4* pxOutput) const
{
	Asura_AVector4	axRows[4];

	// Extract the row-data...
	axRows[0] = GetRow0();
	axRows[1] = GetRow1();
	axRows[2] = GetRow2();
	axRows[3] = GetRow3();

	for(u_int u = 0; u < uNumElements; u++, pxInput++, pxOutput++)
	{
		Asura_AVector4	axColumns[4];

		// Extract the column-data...
		axColumns[0] = pxInput->GetColumn0();
		axColumns[1] = pxInput->GetColumn1();
		axColumns[2] = pxInput->GetColumn2();
		axColumns[3] = pxInput->GetColumn3();

		// Set each element of the result in turn...
		pxOutput->SetColumn0(Asura_AVector4(axColumns[0] * axRows[0], axColumns[0] * axRows[1], axColumns[0] * axRows[2], axColumns[0] * axRows[3]));
		pxOutput->SetColumn1(Asura_AVector4(axColumns[1] * axRows[0], axColumns[1] * axRows[1], axColumns[1] * axRows[2], axColumns[1] * axRows[3]));
		pxOutput->SetColumn2(Asura_AVector4(axColumns[2] * axRows[0], axColumns[2] * axRows[1], axColumns[2] * axRows[2], axColumns[2] * axRows[3]));
		pxOutput->SetColumn3(Asura_AVector4(axColumns[3] * axRows[0], axColumns[3] * axRows[1], axColumns[3] * axRows[2], axColumns[3] * axRows[3]));
	}
}

__forceinline float Asura_AMatrix4x4::Cofactor(const u_int uRow, const u_int uCol) const
{
	static const u_int	aauIndices[4][3]	= { {1, 2, 3}, {0, 2, 3}, {0, 1, 3}, {0, 1, 2} };
	const u_int*		puRowsToUse			= aauIndices[uRow];
	const u_int*		puColsToUse			= aauIndices[uCol];

	// Calculate the determinant of the sub-matrix produced by ignoring row uRow and column uCol
	const float	fA	= GetElement(puRowsToUse[0], puColsToUse[0]);
	const float	fB	= GetElement(puRowsToUse[0], puColsToUse[1]);
	const float	fC	= GetElement(puRowsToUse[0], puColsToUse[2]);

	const float	fD	= GetElement(puRowsToUse[1], puColsToUse[0]);
	const float	fE	= GetElement(puRowsToUse[1], puColsToUse[1]);
	const float	fF	= GetElement(puRowsToUse[1], puColsToUse[2]);

	const float	fG	= GetElement(puRowsToUse[2], puColsToUse[0]);
	const float	fH	= GetElement(puRowsToUse[2], puColsToUse[1]);
	const float	fI	= GetElement(puRowsToUse[2], puColsToUse[2]);

	return( ((fA * fE * fI) + (fD * fH * fC) + (fG * fB * fF)) -
			((fA * fH * fF) + (fD * fB * fI) + (fG * fE * fC)) );
}

__forceinline float Asura_AMatrix4x4::Determinant(void) const
{
	const float	fElement00	= GetElement(0, 0);
	const float	fElement01	= GetElement(0, 1);
	const float	fElement02	= GetElement(0, 2);
	const float	fElement03	= GetElement(0, 3);

	const float fCofactor00	= Cofactor(0, 0);
	const float fCofactor01	= Cofactor(0, 1);
	const float fCofactor02	= Cofactor(0, 2);
	const float fCofactor03	= Cofactor(0, 3);

	return((fElement00 * fCofactor00) - (fElement01 * fCofactor01) + (fElement02 * fCofactor02) - (fElement03 * fCofactor03));
}

__forceinline void Asura_AMatrix4x4::Invert(Asura_AMatrix4x4_OutParam xOther) const
{
	const float			fDeterminant		= Determinant();
	const float			fOneOverDeterminant	= 1.0f / fDeterminant;
	Asura_AMatrix4x4	xResult;

	// Calculate using the adjoint matrix approach...
	Asura_Assert(fDeterminant != 0.f, "Trying to calculate the inverse of a non-invertable matrix");
	for(int iRow = 0; iRow < 4; ++iRow)
	{
		for(int iCol = 0; iCol < 4; ++iCol)
		{
			const float	fSign		= (((iRow + iCol) & 1) == 0) ? 1.0f : -1.0f;
			const float	fCofactor	= Cofactor(iRow, iCol);	// note flipped uCol/uRow

			xOther.SetElement(iCol, iRow, fSign * fCofactor * fOneOverDeterminant);
		}
	}
}

__forceinline void Asura_AMatrix4x4::Invert(void)
{
	const Asura_AMatrix4x4	xTemp(*this);

	xTemp.Invert(*this);
}

__forceinline void Asura_AMatrix4x4::InvertOrthogonal(Asura_AMatrix4x4_OutParam xOther) const
{
	// Create the transposed orientation matrix...
	xOther.m_axColumns[0].SetElements(m_axColumns[0].GetX(), m_axColumns[1].GetX(), m_axColumns[2].GetX(), 0.0f);
	xOther.m_axColumns[1].SetElements(m_axColumns[0].GetY(), m_axColumns[1].GetY(), m_axColumns[2].GetY(), 0.0f);
	xOther.m_axColumns[2].SetElements(m_axColumns[0].GetZ(), m_axColumns[1].GetZ(), m_axColumns[2].GetZ(), 0.0f);
	xOther.m_axColumns[3].ZeroW1();

	// ...apply the transpose to the position and subtract this from the final translation...
	xOther.m_axColumns[3]	-= m_axColumns[3] * xOther;
	xOther.m_axColumns[3].SetW(1.0f);
}

__forceinline void Asura_AMatrix4x4::InvertOrthogonal(void)
{
	const Asura_AMatrix4x4	xTemp(*this);

	xTemp.InvertOrthogonal(*this);
}

__forceinline void Asura_AMatrix4x4::Transpose(Asura_AMatrix4x4_OutParam xOther) const
{
	xOther.SetRows(m_axColumns[0], m_axColumns[1], m_axColumns[2], m_axColumns[3]);
}

__forceinline void Asura_AMatrix4x4::Transpose(void)
{
	const Asura_AMatrix4x4	xTemp(*this);

	xTemp.Transpose(*this);
}

__forceinline Asura_AMatrix4x4 Asura_AMatrix4x4::operator*(const float fScale) const
{
	Asura_AMatrix4x4	xTemp(*this);

	xTemp.m_axColumns[0]	*= fScale;
	xTemp.m_axColumns[1]	*= fScale;
	xTemp.m_axColumns[2]	*= fScale;
	xTemp.m_axColumns[3]	*= fScale;
	return(xTemp);
}

__forceinline Asura_AMatrix4x4 Asura_AMatrix4x4::operator/(const float fScale) const
{
	const float			fOneOverScale	= 1.0f / fScale;
	Asura_AMatrix4x4	xTemp(*this);

	xTemp.m_axColumns[0]	*= fOneOverScale;
	xTemp.m_axColumns[1]	*= fOneOverScale;
	xTemp.m_axColumns[2]	*= fOneOverScale;
	xTemp.m_axColumns[3]	*= fOneOverScale;
	return(xTemp);
}

__forceinline void Asura_AMatrix4x4::operator=(Asura_AMatrix4x4_InParam xM)
{
	m_axColumns[0]	= xM.m_axColumns[0];
	m_axColumns[1]	= xM.m_axColumns[1];
	m_axColumns[2]	= xM.m_axColumns[2];
	m_axColumns[3]	= xM.m_axColumns[3];
}

__forceinline void Asura_AMatrix4x4::operator*=(const float fScale)
{
	m_axColumns[0]	*= fScale;
	m_axColumns[1]	*= fScale;
	m_axColumns[2]	*= fScale;
	m_axColumns[3]	*= fScale;
}

__forceinline void Asura_AMatrix4x4::operator/=(const float fScale)
{
	const float	fOneOverScale	= 1.0f / fScale;

	m_axColumns[0]	*= fOneOverScale;
	m_axColumns[1]	*= fOneOverScale;
	m_axColumns[2]	*= fOneOverScale;
	m_axColumns[3]	*= fOneOverScale;
}

__forceinline bool Asura_AMatrix4x4::operator==(Asura_AMatrix4x4_InParam xOther) const
{
	return (m_axColumns[0] == xOther.m_axColumns[0])
		&& (m_axColumns[1] == xOther.m_axColumns[1])
		&& (m_axColumns[2] == xOther.m_axColumns[2])
		&& (m_axColumns[3] == xOther.m_axColumns[3]);
}

__forceinline bool Asura_AMatrix4x4::ApproximatelyEquals(Asura_AMatrix4x4_InParam xOther, const float fEpsilon) const
{
	return (m_axColumns[0].ApproximatelyEquals(xOther.m_axColumns[0], fEpsilon))
		&& (m_axColumns[1].ApproximatelyEquals(xOther.m_axColumns[1], fEpsilon))
		&& (m_axColumns[2].ApproximatelyEquals(xOther.m_axColumns[2], fEpsilon))
		&& (m_axColumns[3].ApproximatelyEquals(xOther.m_axColumns[3], fEpsilon));
}

__forceinline bool Asura_AMatrix4x4::operator!=(Asura_AMatrix4x4_InParam xOther) const
{
	return (m_axColumns[0] != xOther.m_axColumns[0])
		|| (m_axColumns[1] != xOther.m_axColumns[1])
		|| (m_axColumns[2] != xOther.m_axColumns[2])
		|| (m_axColumns[3] != xOther.m_axColumns[3]);
}

__forceinline float Asura_AMatrix4x4::GetElement(const u_int uRow, const u_int uCol) const
{
	Asura_Assert(uRow <= 3, "Invalid matrix row index passed to Asura_AMatrix4x4::GetElement");
	Asura_Assert(uCol <= 3, "Invalid matrix column index passed to Asura_AMatrix4x4::GetElement");
	return(m_axColumns[uCol].GetElement(uRow));
}

__forceinline void Asura_AMatrix4x4::SetElement(const u_int uRow, const u_int uCol, const float fValue)
{ 
	Asura_Assert(uRow <= 3, "Invalid matrix row index passed to Asura_AMatrix4x4::SetElement");
	Asura_Assert(uCol <= 3, "Invalid matrix column index passed to Asura_AMatrix4x4::SetElement");
	m_axColumns[uCol].SetElement(uRow, fValue);
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::GetColumn(const u_int uColumnIndex) const
{
	Asura_Assert(uColumnIndex <= 3, "Invalid matrix column index passed to Asura_AMatrix4x4::GetColumn");
	return m_axColumns[uColumnIndex];
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::GetColumn0() const
{
	return m_axColumns[0];
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::GetColumn1() const
{
	return m_axColumns[1];
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::GetColumn2() const
{
	return m_axColumns[2];
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::GetColumn3() const
{
	return m_axColumns[3];
}

__forceinline void Asura_AMatrix4x4::SetColumn(const u_int uColumnIndex, Asura_AVector4_InParam xColumn)
{
	Asura_Assert(uColumnIndex <= 3, "Invalid matrix column index passed to Asura_AMatrix4x4::SetColumn");
	m_axColumns[uColumnIndex] = xColumn;
}

__forceinline void Asura_AMatrix4x4::SetColumn0(Asura_AVector4_InParam xCol0)
{
	m_axColumns[0] = xCol0;
}

__forceinline void Asura_AMatrix4x4::SetColumn1(Asura_AVector4_InParam xCol1)
{
	m_axColumns[1] = xCol1;
}

__forceinline void Asura_AMatrix4x4::SetColumn2(Asura_AVector4_InParam xCol2)
{
	m_axColumns[2] = xCol2;
}

__forceinline void Asura_AMatrix4x4::SetColumn3(Asura_AVector4_InParam xCol3)
{
	m_axColumns[3] = xCol3;
}


__forceinline Asura_AVector4 Asura_AMatrix4x4::GetRow(const u_int uRowIndex) const
{
	Asura_Assert(uRowIndex <= 3, "Invalid matrix row index passed to Asura_AMatrix4x4::GetRow");
	return Asura_AVector4(m_axColumns[0].GetElement(uRowIndex), m_axColumns[1].GetElement(uRowIndex),
						  m_axColumns[2].GetElement(uRowIndex), m_axColumns[3].GetElement(uRowIndex));
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::GetRow0() const
{
	return Asura_AVector4(m_axColumns[0].GetElement(0), m_axColumns[1].GetElement(0),
						  m_axColumns[2].GetElement(0), m_axColumns[3].GetElement(0));
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::GetRow1() const
{
	return Asura_AVector4(m_axColumns[0].GetElement(1), m_axColumns[1].GetElement(1),
						  m_axColumns[2].GetElement(1), m_axColumns[3].GetElement(1));
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::GetRow2() const
{
	return Asura_AVector4(m_axColumns[0].GetElement(2), m_axColumns[1].GetElement(2),
						  m_axColumns[2].GetElement(2), m_axColumns[3].GetElement(2));
}

__forceinline Asura_AVector4 Asura_AMatrix4x4::GetRow3() const
{
	return Asura_AVector4(m_axColumns[0].GetElement(3), m_axColumns[1].GetElement(3),
						  m_axColumns[2].GetElement(3), m_axColumns[3].GetElement(3));
}
__forceinline void Asura_AMatrix4x4::SetRow(const u_int uRowIndex, Asura_AVector4_InParam xRow)
{
	Asura_Assert(uRowIndex <= 3, "Invalid matrix row index passed to Asura_AMatrix4x4::SetRow");
	m_axColumns[0].SetElement(uRowIndex, xRow.GetElement(0));
	m_axColumns[1].SetElement(uRowIndex, xRow.GetElement(1));
	m_axColumns[2].SetElement(uRowIndex, xRow.GetElement(2));
	m_axColumns[3].SetElement(uRowIndex, xRow.GetElement(3));
}

__forceinline void Asura_AMatrix4x4::SetRow0(Asura_AVector4_InParam xRow0)
{
	m_axColumns[0].SetElement(0, xRow0.GetElement(0));
	m_axColumns[1].SetElement(0, xRow0.GetElement(1));
	m_axColumns[2].SetElement(0, xRow0.GetElement(2));
	m_axColumns[3].SetElement(0, xRow0.GetElement(3));
}

__forceinline void Asura_AMatrix4x4::SetRow1(Asura_AVector4_InParam xRow1)
{
	m_axColumns[0].SetElement(1, xRow1.GetElement(0));
	m_axColumns[1].SetElement(1, xRow1.GetElement(1));
	m_axColumns[2].SetElement(1, xRow1.GetElement(2));
	m_axColumns[3].SetElement(1, xRow1.GetElement(3));
}

__forceinline void Asura_AMatrix4x4::SetRow2(Asura_AVector4_InParam xRow2)
{
	m_axColumns[0].SetElement(2, xRow2.GetElement(0));
	m_axColumns[1].SetElement(2, xRow2.GetElement(1));
	m_axColumns[2].SetElement(2, xRow2.GetElement(2));
	m_axColumns[3].SetElement(2, xRow2.GetElement(3));
}

__forceinline void Asura_AMatrix4x4::SetRow3(Asura_AVector4_InParam xRow3)
{
	m_axColumns[0].SetElement(3, xRow3.GetElement(0));
	m_axColumns[1].SetElement(3, xRow3.GetElement(1));
	m_axColumns[2].SetElement(3, xRow3.GetElement(2));
	m_axColumns[3].SetElement(3, xRow3.GetElement(3));
}

__forceinline void Asura_AMatrix4x4::LoadColsAligned(const float* pfFloats)
{
	m_axColumns[0].LoadAligned(&pfFloats[0]);
	m_axColumns[1].LoadAligned(&pfFloats[4]);
	m_axColumns[2].LoadAligned(&pfFloats[8]);
	m_axColumns[3].LoadAligned(&pfFloats[12]);
}

__forceinline void Asura_AMatrix4x4::LoadRowsAligned(const float* pfFloats) 
{
	LoadColsAligned(pfFloats);
	Transpose();
}

__forceinline void Asura_AMatrix4x4::LoadColsUnaligned(const float* pfFloats)
{
	m_axColumns[0].LoadUnaligned(&pfFloats[0]);
	m_axColumns[1].LoadUnaligned(&pfFloats[4]);
	m_axColumns[2].LoadUnaligned(&pfFloats[8]);
	m_axColumns[3].LoadUnaligned(&pfFloats[12]);
}

__forceinline void Asura_AMatrix4x4::LoadRowsUnaligned(const float* pfFloats)
{
	LoadColsUnaligned(pfFloats);
	Transpose();
}

__forceinline void Asura_AMatrix4x4::StoreColsAligned(float* pfFloats) const
{
	m_axColumns[0].StoreAligned(&pfFloats[0]);
	m_axColumns[1].StoreAligned(&pfFloats[4]);
	m_axColumns[2].StoreAligned(&pfFloats[8]);
	m_axColumns[3].StoreAligned(&pfFloats[12]);
}

__forceinline void Asura_AMatrix4x4::StoreRowsAligned(float* pfFloats) const
{
	Asura_AMatrix4x4	xTemp;

	Transpose(xTemp);
	xTemp.StoreColsAligned(pfFloats);
}

__forceinline void Asura_AMatrix4x4::StoreColsUnaligned(float* pfFloats) const
{
	m_axColumns[0].StoreUnaligned(&pfFloats[0]);
	m_axColumns[1].StoreUnaligned(&pfFloats[4]);
	m_axColumns[2].StoreUnaligned(&pfFloats[8]);
	m_axColumns[3].StoreUnaligned(&pfFloats[12]);
}

__forceinline void Asura_AMatrix4x4::StoreRowsUnaligned(float* pfFloats) const
{
	Asura_AMatrix4x4	xTemp;

	Transpose(xTemp);
	xTemp.StoreColsUnaligned(pfFloats);
}

//*******************************************************************
//
// (c) 2006 Rebellion
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //ASURA_USE_PLATFORM_VECTORS

#endif // __ASURA_XBOX360_VECTORMATHS_H__
