
///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Platform_ScreenShot_System.cpp
// Author : Kevin Floyer-Lea
// Date   : 8-Sep-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_SCREENSHOT_SYSTEM

// This file's header...
#include "Debugging/Asura_ScreenShot_System.h"

// Asura independent header-files...
#include "Core/Asura_Stream.h"
#include "Core/Memory/Asura_Mem.h"
#include "Rendering/Asura_Font_System.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Rendering/Asura_Render.h"

// Asura platform header-files...
#include "Rendering/Asura_Xbox360_Graphics.h"
#include "Input/Asura_PlatformInput.h"

//external
#include <XGraphics.h>

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

void Asura_ScreenShot_System::Platform_RenderProgressBar (const Asura_Vector_2& xPosition, const Asura_Vector_2& xSize, const u_int uARGB)
{
	// Prepare the debug quad...
	Asura_Render::Overlay.TextureIndex		= ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.TranslucencyMode	= ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.Colour			= uARGB;

	Asura_Render::Overlay.SetRectXY	(xPosition.x, xPosition.y, xSize.x, xSize.y);

	// Render the quad...
	Asura_Render::Overlay.Render			();	
}

void Asura_ScreenShot_System::Platform_GetScreenDimensions (Asura_IntVector_2* pxPosition, Asura_IntVector_2* pxSize, u_int* puFormat)
{	
	if (pxPosition)
	{
		pxPosition->iX	= 0;
		pxPosition->iY	= 0;
	}
	if (pxSize)
	{		
		pxSize->iX	= Asura_Xbox360_Graphics::GetScreenWidth ();
		pxSize->iY	= Asura_Xbox360_Graphics::GetScreenHeight ();
	}
	if (puFormat)
	{
		//This doesnt actually get used...
		*puFormat	= 0;
	}
}

void Asura_ScreenShot_System::Platform_WriteScreenAs24Bit (Asura_Stream& xStream, const Asura_IntVector_2& xPosition, const Asura_IntVector_2& xSize, const u_int uFormat)
{
	//In the interests of consistancy this function doesnt actually use any of the paramaters passed in other than xStream
	//the paramaters are just grabbed from a call to Platform_GetScreenDimensions externally

	HRESULT				hResult;
	
	Direct3D_Texture*	pxSourceTexture;
	Direct3D_Texture*	pxDestTexture;
	D3DLOCKED_RECT		xSourceRect;
	D3DLOCKED_RECT		xDestRect;

	const int				iWidth	= Asura_Xbox360_Graphics::GetScreenWidth ();
	const int				iHeight = Asura_Xbox360_Graphics::GetScreenHeight ();

	// Ensure that we've finished rendering...
	Asura_Render::FlushDisplay	();

	Direct3D_Device* pDevice = Asura_Xbox360_Graphics::GetDevice();

	//Get the source texture and lock its bits	
	pxSourceTexture = Asura_Xbox360_Graphics::GetFrontBuffer();

	hResult	= pxSourceTexture->LockRect(0, &xSourceRect, NULL, D3DLOCK_READONLY);
	if (FAILED(hResult))
	{
		Asura_Assert	(false, "Failed to lock texture for screenshot");
		return;
	}

	//Make a destination texture and lock its bits
	pDevice->CreateTexture( iWidth, iHeight, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &pxDestTexture, NULL );
	
	hResult	= pxDestTexture->LockRect(0, &xDestRect, NULL, 0);
	if (FAILED(hResult))
	{
		Asura_Assert	(false, "Failed to lock texture for screenshot");
		return;
	}
	unsigned char*	pucTemp	= static_cast<unsigned char *>(xDestRect.pBits);
	pucTemp[0]=0;

	POINT xPoint = {0,0};
	RECT xRect;
	xRect.top = 0;
	xRect.left = 0;
	xRect.bottom = iHeight;
	xRect.right = iWidth;
	
	//Now we need to untile the texture because otherwise the data will not be linear		
	DWORD uFlags = 0;
	if(XGIsBorderTexture(pxDestTexture))
	{
		uFlags |= XGTILE_BORDER;
	}
	if(!XGIsPackedTexture(pxDestTexture))
	{
		uFlags |= XGTILE_NONPACKED;
	}
	XGUntileTextureLevel(iWidth,iHeight,0,XGGetGpuFormat(D3DFMT_A8R8G8B8),uFlags,xDestRect.pBits,xDestRect.Pitch,&xPoint,xSourceRect.pBits,&xRect);
	
	//Now read all the bits
	const u_int				uBytesPerLine	= iWidth * 3;

	const unsigned char*	puSourceBuf	= static_cast<unsigned char *>(xDestRect.pBits) + (xDestRect.Pitch * (iHeight - 1));
	unsigned char*			pucOutputBuffer	= new unsigned char [uBytesPerLine];
	if(!pucOutputBuffer)
	{
		Asura_Assert(false,"Failed to allocate memory for screenshot");
	}
	else
	{
		for (int y = 0; y < iHeight; y++)
		{
			const unsigned int*	puSource	= reinterpret_cast<const unsigned int*>(puSourceBuf);
			unsigned char*		pucDest		= pucOutputBuffer;

			for (int x = 0; x < iWidth; x++)
			{
				const unsigned int	uColour	= puSource[x];

				pucDest[x*3]	= static_cast<unsigned char>((uColour & 0xFF000000) >> 24);		// R
				pucDest[x*3+1]	= static_cast<unsigned char>((uColour & 0x00FF0000) >> 16);		// G					
				pucDest[x*3+2]	= static_cast<unsigned char>((uColour & 0x0000FF00) >> 8);		// B
			}
			xStream.WriteData	(uBytesPerLine, pucOutputBuffer);
			puSourceBuf	-= xDestRect.Pitch;
		}
		delete []	(pucOutputBuffer);
	}
	//Clean up	
	pxSourceTexture->UnlockRect(0);
	pxDestTexture->UnlockRect (0);
	pxDestTexture->Release();
}

void Asura_ScreenShot_System::Platform_AddScreenToFPBuffer (const Asura_IntVector_2& xPosition, const Asura_IntVector_2& xSize, const u_int uFormat, float* pfFPBuffer)
{
	//In the interests of consistancy this function doesnt actually use any of the paramaters passed in other than xStream
	//the paramaters are just grabbed from a call to Platform_GetScreenDimensions externally

	HRESULT				hResult;

	Direct3D_Texture*	pxSourceTexture;
	Direct3D_Texture*	pxDestTexture;
	D3DLOCKED_RECT		xSourceRect;
	D3DLOCKED_RECT		xDestRect;

	const int				iWidth	= Asura_Xbox360_Graphics::GetScreenWidth ();
	const int				iHeight = Asura_Xbox360_Graphics::GetScreenHeight ();

	// Ensure that we've finished rendering...
	Asura_Render::FlushDisplay	();

	Direct3D_Device* pDevice = Asura_Xbox360_Graphics::GetDevice();

	//Get the source texture and lock its bits	
	pxSourceTexture = Asura_Xbox360_Graphics::GetFrontBuffer();
	//pDevice->CreateTexture( iWidth, iHeight, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &pxSourceTexture, NULL );
	//pDevice->Resolve( D3DRESOLVE_RENDERTARGET0, NULL, pxSourceTexture, NULL, 0, 0, NULL, 1.0f, 0, NULL );
	
	hResult	= pxSourceTexture->LockRect(0, &xSourceRect, NULL, D3DLOCK_READONLY);
	if (FAILED(hResult))
	{
		Asura_Assert	(false, "Failed to lock texture for screenshot");
		return;
	}

	//Make a destination texture and lock its bits
	pDevice->CreateTexture( iWidth, iHeight, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &pxDestTexture, NULL );

	hResult	= pxDestTexture->LockRect(0, &xDestRect, NULL, 0);
	if (FAILED(hResult))
	{
		Asura_Assert	(false, "Failed to lock texture for screenshot");
		return;
	}
	unsigned char*	pucTemp	= static_cast<unsigned char *>(xDestRect.pBits);
	pucTemp[0]=0;

	POINT xPoint = {0,0};
	RECT xRect;
	xRect.top = 0;
	xRect.left = 0;
	xRect.bottom = iHeight;
	xRect.right = iWidth;

	//Now we need to untile the texture because otherwise the data will not be linear		
	DWORD uFlags = 0;
	if(XGIsBorderTexture(pxDestTexture))
	{
		uFlags |= XGTILE_BORDER;
	}
	if(!XGIsPackedTexture(pxDestTexture))
	{
		uFlags |= XGTILE_NONPACKED;
	}
	XGUntileTextureLevel(iWidth,iHeight,0,XGGetGpuFormat(D3DFMT_A8R8G8B8),uFlags,xDestRect.pBits,xDestRect.Pitch,&xPoint,xSourceRect.pBits,&xRect);

	//Now read all the bits
	const unsigned char*	puSourceBuf	= static_cast<unsigned char *>(xDestRect.pBits) + (xDestRect.Pitch * (iHeight - 1));
	
	for (int y = 0; y < iHeight; y++)
	{
		const unsigned int*	puSource	= reinterpret_cast<const unsigned int*>(puSourceBuf);

		for (int x = 0; x < iWidth; x++)
		{
			const unsigned int	uColour	= puSource[x];

			*(pfFPBuffer++)	+= static_cast<float>((uColour & 0xFF000000) >> 24);		// R
			*(pfFPBuffer++)	+= static_cast<float>((uColour & 0x00FF0000) >> 16);		// G					
			*(pfFPBuffer++)	+= static_cast<float>((uColour & 0x0000FF00) >> 8);		// B
		}
		puSourceBuf	-= xDestRect.Pitch;
	}
	//Clean up
	pxSourceTexture->UnlockRect(0);
	pxDestTexture->UnlockRect (0);
	pxDestTexture->Release();
}

void Asura_ScreenShot_System::Platform_BeginHiResRender (void)
{
	s_iAcceptKey		= ASURA_XBOX360_KEYID_PAD1_START;
	s_aiCancelKey[0]	= ASURA_XBOX360_KEYID_PAD1_X;
}

void Asura_ScreenShot_System::Platform_EndHiResRender (void)
{	
	//nothing to do here	
}

int Asura_ScreenShot_System::Platform_TakeScreenShot (const char* szFilename)
{
	Asura_IntVector_2	xScreenPosition;
	Asura_IntVector_2	xScreenSize;
	u_int				uScreenFormat;	
	int					iRet = 0;
	
	Platform_GetScreenDimensions	(&xScreenPosition, &xScreenSize, &uScreenFormat);

	iRet = BMPTakeScreenShot(szFilename,  xScreenPosition, xScreenSize, uScreenFormat);
		
	return iRet;
}

void Asura_ScreenShot_System::Platform_NotTakingScreenShot (void)
{
	//nothing to do here
}

float* Asura_ScreenShot_System::Platform_AllocateFloatBuffer (u_int uNumFloats)
{
	float* pfMem = 0;
	pfMem = new float [uNumFloats];
	Asura_Assert(pfMem,"Failed to allocate memory for screenshot float buffer");
	return pfMem;
}

void Asura_ScreenShot_System::Platform_FreeFloatBuffer (float* pfBuffer)
{
	delete []	(pfBuffer);
}

void Asura_ScreenShot_System::Platform_RenderPrompt (void)
{	
	char acBuffer[256];
	Asura_PrintfToBfr( acBuffer, "Press 'Start' to take a screenshot, 'X' to cancel", 256, true);
	Asura_Font_System::SetFont(0);
	const u_int uFormat = ASURA_TEXTFORMAT_RELATIVEPOSITION | ASURA_TEXTFORMAT_FORMATSTRING | ASURA_TEXTFORMAT_DROPSHADOWS;
	Asura_Font_System::Print(0.1f, 0.5f, 0.8f, 0xffffffff, uFormat, acBuffer);	
}

#endif // #ifdef ASURA_USE_SCREENSHOT_SYSTEM

//eof
