// ------------------------------------------------------------------------------------ //
//																						//
//	File: Asura_Platform_GUIMenuInput.cpp												//
//																						//
//	Description: this handles the platform specific menu control system					//
//																						//
//	Created: Damian Furlong - 01/05/07														//
//																						//
//	Copyright (C) 2007 Rebellion (Derby), All Rights Reserved.							//
// ------------------------------------------------------------------------------------ //

#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "Core/Asura_Timers.h"
#include "Input/Asura_PlatformInput.h"
#include "Input/Asura_ControlMap.h"
#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

#define ASURA_GUIMENU_MAX_CONTROLLERS	XUSER_MAX_COUNT
#if ASURA_GUIMENU_MAX_CONTROLLERS != 4
	#error Expected 4 controllers on Xbox360
#endif

static ASURA_XBOX360_KEYID OffsetKeyIDForPad (ASURA_XBOX360_KEYID xKeyID, u_int uPadNumber)
{
	//Asura_Assert( (uPadNumber > 0) && (uPadNumber < 5), "Converting a key using an invalud pad number");
	return static_cast<ASURA_XBOX360_KEYID>(xKeyID + uPadNumber * uASURA_XBOX360_NUM_BUTTONS_IN_PAD);
}

static ASURA_XBOX360_AXISID OffsetAxisIDForPad (ASURA_XBOX360_AXISID xAxisID, u_int uPadNumber)
{
	//Asura_Assert( (uPadNumber > 0) && (uPadNumber < 5), "Converting an axis using an invalud pad number");
	return static_cast<ASURA_XBOX360_AXISID>(xAxisID + uPadNumber * uASURA_XBOX360_NUM_AXES_IN_PAD);
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_InitInputMappings( void)
{
	if( s_pxMenuControlMaps == NULL)
	{
		//TY 31.03.08 - Add mappings for each controller
		//TCR # 033 - Players must be able to start the game from the initial interactive screen or attract mode with any supported controller recognized by the console
		s_pxMenuControlMaps = new Asura_ControlMap[ASURA_GUIMENU_MAX_CONTROLLERS];

		for(u_int uController = 0; uController < ASURA_GUIMENU_MAX_CONTROLLERS; ++uController)
		{
			s_pxMenuControlMaps[uController].CreateMappings( ASURA_GUIMENU_CONTROL_MAX);

			// -- This is a fix for the dodgy mapping on my pad --
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_UP				, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_UP	, uController), Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN				, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_DOWN	, uController), Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT				, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_LEFT	, uController), Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT				, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_RIGHT	, uController), Asura_ControlMapping::ON_DEBOUNCED_PRESS);

			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT			, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_A		, uController), Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_START	, uController), Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL			, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_B		, uController), Asura_ControlMapping::ON_DEBOUNCED_PRESS);

			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_X		, uController), Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_Y		, uController), Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_R_THUMB, uController),Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_BACK, uController),	Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_L_BUMPER, uController),Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_R_BUMPER, uController),Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_L_TRIGGER, uController),Asura_ControlMapping::ON_DEBOUNCED_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_R_TRIGGER, uController),Asura_ControlMapping::ON_DEBOUNCED_PRESS);

			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_UP_HELD			, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_UP		, uController), Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD			, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_DOWN		, uController), Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD			, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_LEFT		, uController), Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_RIGHT		, uController), Asura_ControlMapping::ON_PRESS);

			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ACCEPT_HELD		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_A		, uController), Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_CANCEL_HELD		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_B		, uController), Asura_ControlMapping::ON_PRESS);

			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT1_HELD	, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_X		, uController), Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT2_HELD	, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_Y		, uController), Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT3_HELD	, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_R_THUMB, uController),Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT4_HELD	, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_BACK, uController),	Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT5_HELD	, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_START	, uController), Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT6_HELD	, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_L_BUMPER, uController),Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT7_HELD	, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_R_BUMPER, uController),Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT8_HELD	, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_L_TRIGGER, uController),Asura_ControlMapping::ON_PRESS);
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_ALT_ACCEPT9_HELD	, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_R_TRIGGER, uController),Asura_ControlMapping::ON_PRESS);

			#ifdef ASURA_USE_CONTROLMAP_MANAGER
			s_pxMenuControlMaps[uController].AddKeyMapping( ASURA_GUIMENU_CONTROL_DVS_ACTIVATE		, OffsetKeyIDForPad( ASURA_XBOX360_KEYID_PAD1_BACK	, uController), Asura_ControlMapping::ON_DEBOUNCED_PRESS );
			#endif 

			// Added Axis movement to allow navigation and value changes using the analogue sticks.
			s_pxMenuControlMaps[uController].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_UP			, OffsetAxisIDForPad( ASURA_XBOX360_AXISID_PAD1_LSTICK_V, uController), 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f, -1);
			s_pxMenuControlMaps[uController].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_UP_HELD		, OffsetAxisIDForPad( ASURA_XBOX360_AXISID_PAD1_LSTICK_V, uController), 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, 0.5f, -1);

			s_pxMenuControlMaps[uController].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_DOWN			, OffsetAxisIDForPad( ASURA_XBOX360_AXISID_PAD1_LSTICK_V, uController), 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f, 1);
			s_pxMenuControlMaps[uController].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_DOWN_HELD		, OffsetAxisIDForPad( ASURA_XBOX360_AXISID_PAD1_LSTICK_V, uController), 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, -0.5f, 1);

			s_pxMenuControlMaps[uController].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_LEFT			, OffsetAxisIDForPad( ASURA_XBOX360_AXISID_PAD1_LSTICK_H, uController), 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f, 1);
			s_pxMenuControlMaps[uController].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_LEFT_HELD		, OffsetAxisIDForPad( ASURA_XBOX360_AXISID_PAD1_LSTICK_H, uController), 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, -0.5f, 1);

			s_pxMenuControlMaps[uController].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_RIGHT			, OffsetAxisIDForPad( ASURA_XBOX360_AXISID_PAD1_LSTICK_H, uController), 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f, -1);
			s_pxMenuControlMaps[uController].AddTapAxisMapping( ASURA_GUIMENU_CONTROL_RIGHT_HELD	, OffsetAxisIDForPad( ASURA_XBOX360_AXISID_PAD1_LSTICK_H, uController), 1, Asura_ControlMapping::ON_PRESS, 1.0f, 0.1f, 0.5f, -1);
		}
	}
}

// --------------------------------------------------------------------------------------- 

u_int Asura_GUIMenu_System::Platform_GetNumControlMaps( void)
{
	return ASURA_GUIMENU_MAX_CONTROLLERS;
}

// --------------------------------------------------------------------------------------- 

void Asura_GUIMenu_System::Platform_ShutdownInputMappings( void)
{
	u_int u;

	if( s_pxMenuControlMaps)
	{
		for( u=0; u<ASURA_GUIMENU_MAX_CONTROLLERS; u++)
		{
			s_pxMenuControlMaps[u].DeleteMappings();
		}

		delete []s_pxMenuControlMaps;
		s_pxMenuControlMaps = NULL;
	}
}

bool Asura_GUIMenu_System::Platform_ShouldUpdateControls()
{
	return true;
}

bool Asura_GUIMenu_System::Platform_ShouldRenderMenuPages()
{
	return true;
}

// --------------------------------------------------------------------------------------- 
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
void Asura_GUIMenu_System::Platform_RenderSafetyBorder( void)
{
	if( s_bRenderSafetyBorder)
	{
		float fScreenWidth, fScreenHeight;
		float fSafetyWidth, fSafetyHeight;
		Asura_Vector_2 xTopLeft;
		Asura_Vector_2 xBottomRight;

		// calc & draw 90% title safe
		fScreenWidth = static_cast<float>(Asura_Viewport::GetWidth());
		fScreenHeight = static_cast<float>(Asura_Viewport::GetHeight());
		fSafetyWidth = fScreenWidth * 0.9f;
		fSafetyHeight = fScreenHeight * 0.9f;

		xTopLeft.x = (fScreenWidth-fSafetyWidth) * 0.5f;
		xTopLeft.y = (fScreenHeight-fSafetyHeight) * 0.5f;
		xBottomRight.x = xTopLeft.x + fSafetyWidth;
		xBottomRight.y = xTopLeft.y + fSafetyHeight;

		// left edge
		Asura_Render::Render2DLine( xTopLeft.x, xTopLeft.y, xTopLeft.x, xBottomRight.y, 0xFFFF0000);
		// right edge
		Asura_Render::Render2DLine( xBottomRight.x, xTopLeft.y, xBottomRight.x, xBottomRight.y, 0xFFFF0000);
		// top edge
		Asura_Render::Render2DLine( xTopLeft.x, xTopLeft.y, xBottomRight.x, xTopLeft.y, 0xFFFF0000);
		// bottom edge
		Asura_Render::Render2DLine( xTopLeft.x, xBottomRight.y, xBottomRight.x, xBottomRight.y, 0xFFFF0000);

		// calc & draw 80% critical text safe
		fScreenWidth = static_cast<float>(Asura_Viewport::GetWidth());
		fScreenHeight = static_cast<float>(Asura_Viewport::GetHeight());
		fSafetyWidth = fScreenWidth * 0.8f;
		fSafetyHeight = fScreenHeight * 0.8f;

		xTopLeft.x = (fScreenWidth-fSafetyWidth) * 0.5f;
		xTopLeft.y = (fScreenHeight-fSafetyHeight) * 0.5f;
		xBottomRight.x = xTopLeft.x + fSafetyWidth;
		xBottomRight.y = xTopLeft.y + fSafetyHeight;

		// left edge
		Asura_Render::Render2DLine( xTopLeft.x, xTopLeft.y, xTopLeft.x, xBottomRight.y, 0xFFFF0000);
		// right edge
		Asura_Render::Render2DLine( xBottomRight.x, xTopLeft.y, xBottomRight.x, xBottomRight.y, 0xFFFF0000);
		// top edge
		Asura_Render::Render2DLine( xTopLeft.x, xTopLeft.y, xBottomRight.x, xTopLeft.y, 0xFFFF0000);
		// bottom edge
		Asura_Render::Render2DLine( xTopLeft.x, xBottomRight.y, xBottomRight.x, xBottomRight.y, 0xFFFF0000);
	}
}
#endif // ASURA_DEBUG_VARIABLE_SYSTEM
// --------------------------------------------------------------------------------------- 

// (MH - 27.02.2007) text input functions
static bool g_bDoingTextInput = false;
static Asura_UnicodeChar* g_wszTextInputBuffer;
static u_int g_uTextInputBufferMaxChars;
static u_int g_uTextInputCursorPos;
static u_int g_uTextInputFrameNumber;

void Asura_GUIMenu_System::Platform_BeginTextInput( const Asura_UnicodeChar* wszMessage, 
												   Asura_UnicodeChar* wszBuffer, 
												   u_int uBufferMaxChars )
{
	Asura_Assert(!g_bDoingTextInput, "already doing text input");
	g_bDoingTextInput = true;
	g_wszTextInputBuffer = wszBuffer;
	g_uTextInputBufferMaxChars = uBufferMaxChars;
	g_uTextInputCursorPos = uBufferMaxChars;// this indicates "clear on first input"
	g_uTextInputFrameNumber = Asura_Timers::GetFrameNumber();
}

void Asura_GUIMenu_System::Platform_UpdateTextInput()
{
	Asura_UserInput::UpdateInputBuffer( g_wszTextInputBuffer, g_uTextInputBufferMaxChars, g_uTextInputCursorPos, NULL );
}

void Asura_GUIMenu_System::Platform_BeginRender()
{
#if defined(ASURA_XBOX360_ENABLE_PIX)
	PIXBeginNamedEvent(D3DCOLOR_XRGB(0xFF, 0xFF, 0xFF), "GUIMenu");
#endif	//defined(ASURA_XBOX360_ENABLE_PIX)

	Asura_Render::CurrentStates.RequestAlphaTest(true, 1);
}

void Asura_GUIMenu_System::Platform_RenderTextInput()
{
	// just leave the menu to render it? what about cursors?
}

void Asura_GUIMenu_System::Platform_EndRender()
{
	Asura_Render::CurrentStates.RequestAlphaTest(false);

#if defined(ASURA_XBOX360_ENABLE_PIX)
	PIXEndNamedEvent();
#endif	//defined(ASURA_XBOX360_ENABLE_PIX)
}

bool Asura_GUIMenu_System::Platform_IsTextInputFinished()
{
	if (g_uTextInputFrameNumber == Asura_Timers::GetFrameNumber())
	{
		return false;
	}
	return Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_ENTER );
}

void Asura_GUIMenu_System::Platform_EndTextInput()
{
	Asura_Assert(g_bDoingTextInput, "not doing text input");
	g_bDoingTextInput = false;
	g_wszTextInputBuffer = NULL;
	g_uTextInputBufferMaxChars = 0;
}

// -------------------------------------------------------------------------------------------------

bool Asura_GUIMenu_System::Platform_ProcessMenuBackControls( u_int uButtonsPressed, u_int uButtonsTrapped )
{
	bool bRet = false;

	// filters out any buttons presses not required, then we do an equality check
	uButtonsPressed &= (ASURA_GUIMENU_ACTION_BUTTON_CANCEL|ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD);

	if ( uButtonsPressed == ( ASURA_GUIMENU_ACTION_BUTTON_CANCEL | ASURA_GUIMENU_ACTION_BUTTON_CANCEL_HELD ) )
	{
		if ( ( uButtonsTrapped & ASURA_GUIMENU_ACTION_BUTTON_CANCEL ) == 0 )
		{
			bRet = true;
		}
	}
	return bRet;
}

// --------------------------------------------------------------------------------------- 

#endif // ASURA_USE_GUI_MENU_SYSTEM
