// Asura_Platform_Rumble.cpp
// TY - 21.02.08
// Copyright (c) 2006 Rebellion. All rights reserved.

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header
#include "Input/Asura_Platform_Rumble.h"

// Optionally include file contents
#if (defined ASURA_SUPPORT_CONTROLLER_RUMBLE) && (ASURA_SUPPORT_CONTROLLER_RUMBLE != 0)

// Asura headers
#include "Debugging/Asura_Assert.h"
#include "Input/Asura_Rumble_System.h"


// I N I T I A L I S A T I O N ///////////////////////////////////////////////

Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_EXPLOSION = NULL;
Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_SINGLESHOT = NULL;
Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_HEARTBEAT = NULL;
Asura_Platform_Rumble* Asura_Rumble_System::ASURA_STANDARD_RUMBLE_HIT = NULL;

// C L A S S   D E C L A R A T I O N S //////////////////////////////////////

class Xbox360_Rumble_Capability
{
	public:
	
		bool m_bCanRumble;
};

// G L O B A L   V A R I A B L E S ///////////////////////////////////////////

static Xbox360_Rumble_Capability g_axDeviceCapabilites[XUSER_MAX_COUNT];

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

void Asura_Rumble_System::Platform_Initialise()
{
	//Check our rumble capabilities
	for(u_int uUser = 0; uUser < XUSER_MAX_COUNT ; ++uUser)
	{
		XINPUT_CAPABILITIES xCapabilities;
		g_axDeviceCapabilites[uUser].m_bCanRumble = false;
		DWORD uResult = XInputGetCapabilities( uUser, XINPUT_FLAG_GAMEPAD, &xCapabilities );
		Asura_Assertf( ( uResult == ERROR_SUCCESS ) || ( uResult == ERROR_DEVICE_NOT_CONNECTED ) , ("Failed to get controller capabilites, error %u", uResult));
		if(		(uResult == ERROR_SUCCESS )
			&&	( xCapabilities.Vibration.wLeftMotorSpeed	> 0 )		//The api is a bit vague about detecting vibration, but presumably all controllers have it?
			&&	( xCapabilities.Vibration.wRightMotorSpeed	> 0 ) )
		{
			g_axDeviceCapabilites[uUser].m_bCanRumble	= true;
		}
	}
	

	// Shot
	Asura_Handle iSingleShotSlowMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 1.0f, 0.0f, 2.5f, 1);
	Asura_Handle iSingleShotRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_SINGLESHOT = s_pxDynamicPlatformRumbleList->GetPointer(iSingleShotRumble);
    ASURA_STANDARD_RUMBLE_SINGLESHOT->m_apxMotorRumbles[PLATFORM_RUMBLE_MOTORS_SMALL] = NULL;
	ASURA_STANDARD_RUMBLE_SINGLESHOT->m_apxMotorRumbles[PLATFORM_RUMBLE_MOTORS_LARGE] = Asura_Rumble::GetRumblePointer(iSingleShotSlowMotor);
	
	ASURA_STANDARD_RUMBLE_HIT = ASURA_STANDARD_RUMBLE_SINGLESHOT;
	ASURA_STANDARD_RUMBLE_EXPLOSION = ASURA_STANDARD_RUMBLE_SINGLESHOT;
	ASURA_STANDARD_RUMBLE_HEARTBEAT = ASURA_STANDARD_RUMBLE_SINGLESHOT;
	
	// Hit	
	Asura_Handle iHitSlowMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_LINEAR, RUMBLE_ADJUST_REVERSE, 0.75f, 0.0f, 1.0f, 1);
	Asura_Handle iHitRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_HIT = s_pxDynamicPlatformRumbleList->GetPointer(iHitRumble);
    ASURA_STANDARD_RUMBLE_HIT->m_apxMotorRumbles[PLATFORM_RUMBLE_MOTORS_SMALL] = NULL;
	ASURA_STANDARD_RUMBLE_HIT->m_apxMotorRumbles[PLATFORM_RUMBLE_MOTORS_LARGE] = Asura_Rumble::GetRumblePointer(iHitSlowMotor);

	// Explosion
	Asura_Handle iExplosionFastMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 1.0f, 0.0f, 2.5f, 1);
	Asura_Rumble::AddRumbleStep(iExplosionFastMotor, RUMBLE_BASE_LINEAR, RUMBLE_ADJUST_REVERSE, 1.0f, 0.0f, 1.0f, 1);
	Asura_Handle iExplosionSlowMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 1.0f, 0.0f, 2.5f, 1);
	Asura_Rumble::AddRumbleStep(iExplosionSlowMotor, RUMBLE_BASE_LINEAR, RUMBLE_ADJUST_REVERSE, 1.0f, 0.0f, 0.5f, 1);
	Asura_Handle iExplosionRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_EXPLOSION = s_pxDynamicPlatformRumbleList->GetPointer(iExplosionRumble);
	ASURA_STANDARD_RUMBLE_EXPLOSION->m_apxMotorRumbles[PLATFORM_RUMBLE_MOTORS_SMALL] = Asura_Rumble::GetRumblePointer(iExplosionFastMotor);
	ASURA_STANDARD_RUMBLE_EXPLOSION->m_apxMotorRumbles[PLATFORM_RUMBLE_MOTORS_LARGE] = Asura_Rumble::GetRumblePointer(iExplosionSlowMotor);

	// Heartbeat
	Asura_Handle iHeartbeatSlowMotor = Asura_Rumble::CreateAsuraRumble(RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 0.0f, 0.0f, 5.0f, 1);
	Asura_Rumble::AddRumbleStep(iHeartbeatSlowMotor, RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 0.3f, 0.0f, 2.0f, 1);
	Asura_Rumble::AddRumbleStep(iHeartbeatSlowMotor, RUMBLE_BASE_CONSTANT, RUMBLE_ADJUST_NONE, 0.0f, 0.0f, 1.0f, 1);
	Asura_Handle iHeartbeatRumble = s_pxDynamicPlatformRumbleList->Allocate();
	ASURA_STANDARD_RUMBLE_HEARTBEAT = s_pxDynamicPlatformRumbleList->GetPointer(iHeartbeatRumble);
    ASURA_STANDARD_RUMBLE_HEARTBEAT->m_apxMotorRumbles[PLATFORM_RUMBLE_MOTORS_SMALL] = NULL;
	ASURA_STANDARD_RUMBLE_HEARTBEAT->m_apxMotorRumbles[PLATFORM_RUMBLE_MOTORS_LARGE] = Asura_Rumble::GetRumblePointer(iHeartbeatSlowMotor);
}


void Asura_Rumble_System::Platform_Shutdown()
{
	Platform_StopAllRumbles();
}


void Asura_Rumble_System::Platform_Reset()
{
	Platform_StopAllRumbles();
}


void Asura_Rumble_System::Platform_PlayCurrentRumbles()
{
	for(u_int uUser = 0; uUser < XUSER_MAX_COUNT ; ++uUser)
	{
		#ifdef ASURA_ASSERT
		float fMotorSmall = s_fCurrentMotorMagnitudes[uUser][PLATFORM_RUMBLE_MOTORS_SMALL];
		float fMotorLarge = s_fCurrentMotorMagnitudes[uUser][PLATFORM_RUMBLE_MOTORS_LARGE];
		Asura_Assertf( ( fMotorSmall >= 0.0f ) && ( fMotorSmall <= 1.0f ), ( "Expected a motor magnitude between 0 and 1 for vibration (got %.2f)", fMotorSmall ) );
		Asura_Assertf( ( fMotorLarge >= 0.0f ) && ( fMotorLarge <= 1.0f ), ( "Expected a motor magnitude between 0 and 1 for vibration (got %.2f)", fMotorSmall ) );
		#endif

		if ( g_axDeviceCapabilites[uUser].m_bCanRumble )
		{
			XINPUT_VIBRATION xVibration;
			xVibration.wLeftMotorSpeed	= static_cast< WORD >( s_fCurrentMotorMagnitudes[uUser][PLATFORM_RUMBLE_MOTORS_SMALL] * 65535.0f );
			xVibration.wRightMotorSpeed = static_cast< WORD >( s_fCurrentMotorMagnitudes[uUser][PLATFORM_RUMBLE_MOTORS_LARGE] * 65535.0f );

			#ifdef ASURA_ASSERT
			DWORD uResult = XInputSetState ( uUser, &xVibration );
			//its not that unlikely Xinput will be busy, we dont really care since we set this so often
			if( ( uResult != ERROR_DEVICE_NOT_CONNECTED ) && ( uResult != ERROR_BUSY ) ) 
			{
				Asura_Assertf( uResult == ERROR_SUCCESS, ( "Failed to set vibration on controller %u, error %u", uUser, uResult ) );
			}
			#else
			XInputSetState ( uUser, &xVibration );
			#endif
		}
    }
}


void Asura_Rumble_System::Platform_StopAllRumbles()
{
	for(u_int uUser = 0; uUser < XUSER_MAX_COUNT ; ++uUser)
	{
		if(g_axDeviceCapabilites[uUser].m_bCanRumble)
		{
			XINPUT_VIBRATION xVibration;
			xVibration.wLeftMotorSpeed = 0;
			xVibration.wRightMotorSpeed = 0;
			DWORD uResult = ERROR_BUSY;
			//Naughty busy loop, should be okay unless xinput bombs somehow
			while(uResult == ERROR_BUSY)
			{
				uResult = XInputSetState ( uUser, &xVibration );
				#ifdef ASURA_ASSERT
				if( ( uResult != ERROR_DEVICE_NOT_CONNECTED ) && ( uResult != ERROR_BUSY ) )
				{
					Asura_Assertf( uResult == ERROR_SUCCESS, ( "Failed to stop vibration on controller %u, error %u", uUser, uResult ) );
				}
				#endif
			}
		}
		
		
	}
}


#endif // #if (defined ASURA_SUPPORT_CONTROLLER_RUMBLE) && (ASURA_SUPPORT_CONTROLLER_RUMBLE != 0) 
