//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_DLC_System.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT

#include "Core/Asura_Xbox360_OS.h"
#include "Core/Asura_Xbox360_Profile_System.h"
#include "Network/Asura_Xbox360_DLC_System.h"

#define DLC_DRIVE_NAME_PREFIX "DLC"

int Asura_Xbox360_DLC_System::s_iNumVirtualDrives = 0;

void Asura_Xbox360_DLC_System::Initialise()
{
}

void Asura_Xbox360_DLC_System::Shutdown()
{
}

void Asura_Xbox360_DLC_System::UpdateDownloadedFiles(Asura_TemplateHashTree<Asura_DLC_DownloadedFileData*>& xDLCFiles)
{
	// The independent function will have cleared the data out already, so just fill it.
	// For now we won't try to select users and drives here, we'll let other things set that up. If they aren't set up, we can't detect any files.
	if (Asura_Xbox360_Profile_System::HasValidGamerProfile() && Asura_Xbox360_Profile_System::HasValidStorageDevice())
	{
		u_int uActivePlayer = Asura_Xbox360_OS::GetActivePlayerIndex();
		int iStorageDevice = Asura_Xbox360_Profile_System::GetActiveStorageDevice();

		DWORD dwBufferSize;
		HANDLE xEnumHandle;

		// TODO - we should probably find out how many items of DLC are actually possible rather than just creating a big value. This needs ~300Kb.
		u_int uRet = XContentCreateEnumerator(uActivePlayer, iStorageDevice, XCONTENTTYPE_MARKETPLACE, 0, 1000, &dwBufferSize, &xEnumHandle);
		Asura_Assertf(uRet == ERROR_SUCCESS, ("XContentCreateEnumerator failed with error code 0x%08x.", uRet));

		if (uRet == ERROR_SUCCESS)
		{
			if (dwBufferSize > 0)
			{
				void* pBuffer = Asura_Allocate(dwBufferSize);
				Asura_Assert(pBuffer, "Failed to allocate memory for content enumeration.");

				if (pBuffer)
				{
					DWORD dwItems;
					uRet = XEnumerate(xEnumHandle, pBuffer, dwBufferSize, &dwItems, NULL);
					Asura_Assertf((uRet == ERROR_SUCCESS) || (uRet == ERROR_NO_MORE_FILES), ("XEnumerate failed with error code 0x%08x.", uRet));

					if (uRet == ERROR_SUCCESS)
					{
						// Loop through the enumeration we've created, mount each item to a virtual drive and store details about each file within.
						const XCONTENT_DATA* pxContentArray = static_cast<XCONTENT_DATA*>(pBuffer);
						char acDriveNameBuffer[8];
						for (u_int u=0; u<dwItems; ++u)
						{
							const XCONTENT_DATA& xData = pxContentArray[u];
							DWORD dwLicenseMask;
							Asura_PrintfToBfr(acDriveNameBuffer, DLC_DRIVE_NAME_PREFIX "%03d", 8, true, s_iNumVirtualDrives);

							uRet = XContentCreate(uActivePlayer, acDriveNameBuffer, &xData, XCONTENTFLAG_OPENEXISTING, NULL, &dwLicenseMask, NULL);
							Asura_Assertf(uRet == ERROR_SUCCESS, ("XContentCreate for DLC %S failed with error code 0x%08x.", xData.szDisplayName, uRet));

							if (uRet == ERROR_SUCCESS)
							{
								AddFilesInCurrentPackageToList(dwLicenseMask, xDLCFiles);
								++s_iNumVirtualDrives;
							}
						}
					}

					Asura_Deallocate(pBuffer);
				}
			}

			CloseHandle(xEnumHandle);
		}
	}
}

bool Asura_Xbox360_DLC_System::IsNewContentAvailable()
{
	return false;
}

void Asura_Xbox360_DLC_System::OpenStore()
{
}

void Asura_Xbox360_DLC_System::FillFullPath(const char* szFilename, char* szFullPath)
{
	const Asura_Xbox360_DLC_DownloadedFileData* pxData = static_cast<const Asura_Xbox360_DLC_DownloadedFileData*>(Asura_DLC_System::GetDownloadedFileData(szFilename));
	if (!pxData)
	{
		Asura_PrintfToBfr(szFullPath, "GAME:\\%s", MAX_PATH, true, szFilename);
	}
	else
	{
		Asura_PrintfToBfr(szFullPath, DLC_DRIVE_NAME_PREFIX "%03d:\\%s", MAX_PATH, true, pxData->GetDriveID(), szFilename);
	}
}

void Asura_Xbox360_DLC_System::FillDriveList(Asura_List<Asura_String>& xOutputList)
{
	char acTempBuffer[16];
	for (int i=0; i<s_iNumVirtualDrives; ++i)
	{
		Asura_PrintfToBfr(acTempBuffer, DLC_DRIVE_NAME_PREFIX "%03d:\\", 16, true, i);
		xOutputList.AddToList(new Asura_String(acTempBuffer));
	}
}

void Asura_Xbox360_DLC_System::ClearDownloadedFileData()
{
	// Close all open virtual drives.
	char acDriveBuffer[8];
	for (int i=0; i<s_iNumVirtualDrives; ++i)
	{
		Asura_PrintfToBfr(acDriveBuffer, DLC_DRIVE_NAME_PREFIX "%03d", 8, true, i);
#ifdef ASURA_ASSERT
		u_int uRet =
#endif //#ifdef ASURA_ASSERT
			XContentClose(acDriveBuffer, NULL);

		Asura_Assertf(uRet == ERROR_SUCCESS, ("Failed to close DLC drive %d.", i));
	}
}

void Asura_Xbox360_DLC_System::AddFilesInCurrentPackageToList(u_int uPackageRestrictions, Asura_TemplateHashTree<Asura_DLC_DownloadedFileData*>& xDLCFiles)
{
	char acFullPath[MAX_PATH];

	// Fill the list of directories.
	Asura_List<Asura_String> xDirList;

	Asura_PrintfToBfr(acFullPath, DLC_DRIVE_NAME_PREFIX "%03d:", MAX_PATH, true, s_iNumVirtualDrives);
	xDirList.AddToList(new Asura_String(acFullPath));

	Asura_List_ReverseIt<Asura_String> xIt( &xDirList );

	WIN32_FIND_DATA xFindData;

	const char* szRelativePath = &(acFullPath[8]); // The relative path starts just beyond the first slash. That's not in the buffer yet, but it will be by the time we make use of this pointer.

	for( int iDir = 0; iDir < xDirList.GetNumEntries(); ++iDir ) // Same as in fileaccess, but we know there's exactly 1 entry in the list at the start.
	{
		char* szFilenameStart;

		//copy the search string into a buffer for buidling the filenames , 
		//and work out where the filename part should be copied
		{
			if( iDir )
			{
				xIt.Next();
			}

			const Asura_String& xNext = *xIt.GetCurrent();
			int iLen = xNext.GetLength();
			Asura_PrintfToBfr(acFullPath, "%s\\*.*", MAX_PATH, true, xNext.GetString());

			szFilenameStart = &(acFullPath[iLen + 1]);
		}

		HANDLE hFindFile = ::FindFirstFile(acFullPath, &xFindData);
		if (INVALID_HANDLE_VALUE != hFindFile)
		{
			do
			{	
				if (xFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				{
					// Directory. Add it to the list of directories still to search.
					// Ignore the "." and ".." directories.
					if( '.' == xFindData.cFileName[0] )
					{
						if( !xFindData.cFileName[1] || ('.' == xFindData.cFileName[1] && !xFindData.cFileName[2]) )
						{
							continue;
						}
					}

					strcpy(szFilenameStart, xFindData.cFileName);
					xDirList.AddToList(new Asura_String(acFullPath));
				}
				else
				{
					// Not a directory, hence a file. Add it to the list of DLC files.
					strcpy(szFilenameStart, xFindData.cFileName);
					Asura_Xbox360_DLC_DownloadedFileData* pxNewData = new Asura_Xbox360_DLC_DownloadedFileData(uPackageRestrictions, s_iNumVirtualDrives);
					if (pxNewData)
					{
						xDLCFiles.AddNode(Asura_GetHashID(szRelativePath), pxNewData);
					}
				}
			}
			while (::FindNextFile(hFindFile, &xFindData));

			::FindClose(hFindFile);
		}
	}

	//delete strings from xDirList
	for( Asura_List_It<Asura_String> xIt(&xDirList); !xIt.Done(); xIt.Next() )
	{
		delete (xIt.GetCurrent());
	}
}

#endif //#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT
