//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_DLC_System.h
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
// Description: Xbox360-specific class for handling DLC files. Interfaces with
//				the Live! Marketplace and handles the XContent structures
//				stored on this machine in order to provide a way to access
//				the files stored within them.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_XBOX360_DLC_SYSTEM_H___
#define ___ASURA_XBOX360_DLC_SYSTEM_H___ 1

#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT

#include "Core/Asura_HashTree.h"
#include "Core/Asura_List.h"
#include "Core/Asura_String.h"
#include "Network/Asura_DLC_System.h"

// Xbox360 variant of downloaded file data - includes the drive ID the file has been mounted on.
class Asura_Xbox360_DLC_DownloadedFileData : public Asura_DLC_DownloadedFileData
{
public:
	Asura_Xbox360_DLC_DownloadedFileData(u_int uUsageRestrictions, int iDriveID)
		: Asura_DLC_DownloadedFileData(uUsageRestrictions)
		, m_iDriveID(iDriveID)
	{
	}

	int GetDriveID() const { return m_iDriveID; }

private:
	u_int m_iDriveID;
};

class Asura_Xbox360_DLC_System
{
public:
	static void Initialise();
	static void Shutdown();

	static void UpdateDownloadedFiles(Asura_TemplateHashTree<Asura_DLC_DownloadedFileData*>& xDLCFiles);

	static bool IsNewContentAvailable();
	static void OpenStore();

	static void FillFullPath(const char* szFilename, char* szFullPath); // Requires a buffer of at least MAX_PATH.
	static void FillDriveList(Asura_List<Asura_String>& xOutputList);

private:
	static void ClearDownloadedFileData();

	static void AddFilesInCurrentPackageToList(u_int uPackageRestrictions, Asura_TemplateHashTree<Asura_DLC_DownloadedFileData*>& xDLCFiles); // Uses the current s_iNumVirtualDrives to work out the path, so can only be called during UpdateDownloadedFiles.

	static int s_iNumVirtualDrives;

	friend class Asura_DLC_System; // Let the main DLC system call the clear function.
};

#endif //#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT

#endif // #ifndef ___ASURA_XBOX360_DLC_SYSTEM_H___
