//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_LiveContextManager.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Network/Online/Asura_Xbox360_LiveContextManager.h"

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////
class Asura_Xbox360_LiveContextManagerMapping
{
public:
	Asura_Xbox360_LiveContextManagerMapping();

	u_int m_uContextOrPropertyID;
	bool m_bIsProperty;
};

// S T A T I C   C L A S S   D A T A /////////////////////////////////////////
Asura_Xbox360_LiveContextManagerMapping* Asura_Xbox360_LiveContextManager::s_pxMapping;
u_int Asura_Xbox360_LiveContextManager::s_uMaxMappings;

// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////
Asura_Xbox360_LiveContextManagerMapping::Asura_Xbox360_LiveContextManagerMapping()
{
	m_uContextOrPropertyID = uASURA_XBOX360_INVALID_CONTEXT_MAPPING;
	m_bIsProperty = false;
}

void Asura_Xbox360_LiveContextManager::Initialise( u_int uNumMappings )
{
	Asura_Assert(!s_pxMapping, "already initialised");
	s_pxMapping = new Asura_Xbox360_LiveContextManagerMapping[uNumMappings];
	s_uMaxMappings = uNumMappings;
}

void Asura_Xbox360_LiveContextManager::Shutdown()
{
	delete[] s_pxMapping;
	s_pxMapping = NULL;
	s_uMaxMappings = 0;
}

void Asura_Xbox360_LiveContextManager::AddMapping( u_int uAsuraLobbyAttribute, 
												  u_int uContextOrPropertyID, 
												  bool bIsProperty )
{
	Asura_Assert(uContextOrPropertyID != uASURA_XBOX360_INVALID_CONTEXT_MAPPING, "unexpected value for context ID");
	Asura_Assert(uAsuraLobbyAttribute < s_uMaxMappings, "cannot add mapping");
	if (uAsuraLobbyAttribute < s_uMaxMappings)
	{
		s_pxMapping[uAsuraLobbyAttribute].m_uContextOrPropertyID = uContextOrPropertyID;
		s_pxMapping[uAsuraLobbyAttribute].m_bIsProperty = bIsProperty;
	}
}

u_int Asura_Xbox360_LiveContextManager::GetMapping( u_int uAsuraLobbyAttribute, bool& bIsProperty )
{
	Asura_Assert(uAsuraLobbyAttribute < s_uMaxMappings, "invalid mapping");
	if (uAsuraLobbyAttribute < s_uMaxMappings)
	{
		Asura_Assert(s_pxMapping[uAsuraLobbyAttribute].m_uContextOrPropertyID != 
			uASURA_XBOX360_INVALID_CONTEXT_MAPPING, "mapping hasn't been set");
		bIsProperty = s_pxMapping[uAsuraLobbyAttribute].m_bIsProperty;
		return s_pxMapping[uAsuraLobbyAttribute].m_uContextOrPropertyID;
	}
	bIsProperty = false;
	return uASURA_XBOX360_INVALID_CONTEXT_MAPPING;
}
