
//
// Asura_Xbox360_DeferredShading.h
//
// Created by DMR 13/02/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
// Handles deferred lighting passes
//

#ifndef __ASURA_XBOX360_DEFERREDSHADING_H__
#define __ASURA_XBOX360_DEFERREDSHADING_H__

#include "Rendering/Asura_Xbox360_Effect_System.h"
#include "Rendering/Asura_Xbox360_Graphics.h"

class Asura_Photon;
class Asura_Vector_3;
class Asura_Xbox360_IndexBuffer;
class Asura_Xbox360_VertexBuffer;

class Asura_Xbox360_DeferredShading
{
	friend class Asura_DeferredShading;
public:

	static void Initialise(void);
	static void Shutdown(void);
	static void Reset(void);
	static void RenderPhotons(bool bWithDarkeningShadows, bool bWithRealShadows, bool bWithNoShadows);
	static void RenderDebugOutput(void);

	static void SetShaderParameters(Asura_Photon *pxPhoton, Asura_Vector_3 &xMeshPos, float fMeshScale, float fDistFade);
	static void DebugRenderDeferredBuffers(void);
	static void RenderPhotons_Callback( Asura_Photon* pxPhoton, void* pData );
	static void RenderPhotonOutlines_Callback( Asura_Photon* pxPhoton, void* pData );

	static Asura_Xbox360_VertexBuffer*		   s_apxSphereVB[4];
	static int								   s_aiSphereNumVerts[4];

private:
	static Asura_Xbox360_Effect* 			   s_pxDeferredPhotonEffect;
	static Asura_Xbox360_Effect* 			   s_pxRenderNormalBufferEffect;
	static Direct3D_VertexDeclaration*		   s_pxDeferredPhotonVertexDeclaration;

	static Asura_Xbox360_VertexBuffer*		   s_apxIndexedSphereVB[4];
	static int								   s_aiIndexedSphereNumVerts[4];
	static Asura_Xbox360_IndexBuffer*		   s_apxIndexedSphereIB[4];
	static int								   s_aiIndexedSphereNumIndices[4];

	static Asura_Xbox360_VertexBuffer*		   s_apxIndexedSphereSegmentVB[4];
	static int								   s_aiIndexedSphereSegmentNumVerts[4];
	static Asura_Xbox360_IndexBuffer*		   s_apxIndexedSphereSegmentIB[4];
	static int								   s_aiIndexedSphereSegmentNumIndices[4];

	static Asura_Xbox360_Effect_System::Handle s_xMeshScaleAndPosHandle;
	static Asura_Xbox360_Effect_System::Handle s_xNormalDepthBufferHandle;
	static Asura_Xbox360_Effect_System::Handle s_xNormalBufferHandle;
	static Asura_Xbox360_Effect_System::Handle s_xDepthMtlBufferHandle;
	static Asura_Xbox360_Effect_System::Handle s_xAlbedoBufferHandle;
	static Asura_Xbox360_Effect_System::Handle s_xInvViewMatrixHandle;
	static Asura_Xbox360_Effect_System::Handle s_xMaterialsTextureHandle;
	static Asura_Xbox360_Effect_System::Handle s_xCameraTanHalfFOVHandle;
	static Asura_Xbox360_Effect_System::Handle s_xTwoOverScreenDimensionsHandle;
	static Asura_Xbox360_Effect_System::Handle s_xLightDirectionHandle;
	static Asura_Xbox360_Effect_System::Handle s_xLightPosScaleHandle;
	static Asura_Xbox360_Effect_System::Handle s_xLightColourHandle;
	static Asura_Xbox360_Effect_System::Handle s_xFalloffHandle;
	static Asura_Xbox360_Effect_System::Handle s_xLimitAnglesHandle;
	static Asura_Xbox360_Effect_System::Handle s_xUseSpecularMapHandle;
	static Asura_Xbox360_Effect_System::Handle s_xProjCubeTextureHandle;
	static Asura_Xbox360_Effect_System::Handle s_xProjCubeBlurredTextureHandle;
	static Asura_Xbox360_Effect_System::Handle s_xProjCubeLerpDistHandle;
	static Asura_Xbox360_Effect_System::Handle s_xLightOrientationHandle;

	static void RenderPhotonPrimitivies(const Asura_Vector_3* pxCuboidVerts, const int iSphereLOD, const bool bUseAngleLimits);
};

#endif	// __ASURA_XBOX360_DEFERREDSHADING_H__
