//
// Asura_Xbox360_EnvironmentRenderer.h
//
// Created by KJFL - 22 June 2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#if defined ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#if !defined __ASURA_XBOX360_ENVIRONMENTRENDERER_H__
#define __ASURA_XBOX360_ENVIRONMENTRENDERER_H__


#define ASURA_MAX_VISIBLE_WATER_STRIPS 128


/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura Platform-independent
#include "Core/Asura_Maths.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_Graphics.h"

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_Xbox360_IndexBuffer;
class Asura_Xbox360_VertexBuffer;

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_EnvironmentRenderer_Strip
//
// Minimal description of a strip - chunk data matches order and size
// so all strips can be loaded in one go
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_EnvironmentRenderer_Strip
{
public:

	u_int				m_uNumberOfVertices;
	u_int				m_uNumberOfIndices;
	u_int				m_uBaseVertexIndex;
	u_int				m_uStartIndex;
	Asura_Hash_ID		m_auMaterialResponseHashIDs[2];
	int					m_iModuleIndex;
	u_int				m_uNumberOfTriangles;
	u_int				m_uAntiAliasEdgeIdxStart;
	u_int				m_uAntiAliasEdgeIdxNum;
	Asura_Bounding_Box	m_xBoundingBox;
};

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_EnvironmentRenderer
//
// The environment renderer itself
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_EnvironmentRenderer
{ 
	friend class Asura_Chunk_ResourceFile_Xbox360_Environment;

public:

	static void Initialise();
	static void Reset();
	static void Shutdown();

	static void Render( const bool bTransparentStrips );

	static void RenderWater(void);
	static void RenderToShadow(float fNear, float fFar, bool bCubeMap);

	static void RenderToShadowCubemap_Initialise( float fNear, float fFar );
	static void RenderToShadowCubemap( u_char* pxStripVisibilities );

	static Direct3D_VertexDeclaration* GetVD();
	static u_int GetNumStrips() { return s_uNumberOfStrips; }
	static const Asura_Xbox360_IndexBuffer& GetIB() { return s_xIB; }
	static const Asura_Xbox360_VertexBuffer& GetVB( const u_int u ) { return s_pxVBs[u]; }

	static const Asura_Xbox360_EnvironmentRenderer_Strip& GetStrip( const u_int u )
	{
		return s_pxStrips[u];
	}

	static void RenderGeometry();

	static void RenderAABB( const Asura_Bounding_Box& xBB );
	static void RenderStripZOnly( const u_int uStripID );

private:

	friend class Asura_Environment_Renderer;

	static void CreateAABBBuffers();
	static void QueueWaterStrip(int iStripIdx);

	static u_int											s_uNumberOfStrips;
	static Asura_Xbox360_EnvironmentRenderer_Strip*			s_pxStrips;

	static void*											s_pVertexData;
	static Asura_Xbox360_VertexBuffer*						s_pxVBs;
	static Asura_Xbox360_IndexBuffer						s_xIB;

	static Asura_Xbox360_IndexBuffer						s_xAntiAliasEdges;

	static int												s_aiQueuedWaterStrips[ASURA_MAX_VISIBLE_WATER_STRIPS];		
	static int												s_iNumQueuedWaterStrips;				
};

#endif // !defined __ASURA_XBOX360_ENVIRONMENTRENDERER_H__
