//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_IndexBuffer.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Rendering/Asura_Xbox360_IndexBuffer.h"

// Asura Xbox 360
#include "Rendering/Asura_Xbox360_Graphics.h"

// Misc
#include <xtl.h>

/******************************************************************************/
/* Asura_Xbox360_IndexBuffer Class Functions                                  */
/******************************************************************************/

Asura_Xbox360_IndexBuffer::Asura_Xbox360_IndexBuffer()
	: m_pxBuffer( NULL )
{
}

Asura_Xbox360_IndexBuffer::~Asura_Xbox360_IndexBuffer()
{
	Destroy();
}

// Create a static (i.e. write to once) index buffer, of the given size. Other methods to cope with
// more 'usage' options/combinations e.g. dynamic, patches, etc. may be required later.
void Asura_Xbox360_IndexBuffer::CreateStatic( const u_int uSizeInBytes )
{
	Asura_Assert( !m_pxBuffer, "Index buffer being created twice." );

	// Usage virtually ignored on 360
	Asura_Xbox360_Graphics::GetDevice()->CreateIndexBuffer( uSizeInBytes, 0, D3DFMT_INDEX16, D3DPOOL_DEFAULT, &m_pxBuffer, NULL );

	Asura_Assert( m_pxBuffer, "Failed to create index buffer. This is normally an 'Out oF Memory' condition" );
}

void Asura_Xbox360_IndexBuffer::Destroy()
{
	if( m_pxBuffer )
	{
		m_pxBuffer->Release();
		m_pxBuffer = NULL;
	}
}

void* Asura_Xbox360_IndexBuffer::Lock()
{
	void* pData = NULL;

	if( m_pxBuffer )
	{
		m_pxBuffer->Lock( 0, 0, &pData, 0 );
		Asura_Assert( pData, "Failed to lock index buffer." );
	}

	return pData;
}

void Asura_Xbox360_IndexBuffer::Unlock()
{
	if( m_pxBuffer )
	{
		m_pxBuffer->Unlock();
	}
}

// set this vertex buffer as the active one for rendering
void Asura_Xbox360_IndexBuffer::Bind() const
{
	Asura_Xbox360_Graphics::GetDevice()->SetIndices( m_pxBuffer );
}

bool Asura_Xbox360_IndexBuffer::IsEmpty() const
{
	return !m_pxBuffer;
}

bool Asura_Xbox360_IndexBuffer::GetDesc( D3DINDEXBUFFER_DESC* pxDesc )
{
	if( m_pxBuffer == NULL )
		return false;

	m_pxBuffer->GetDesc( pxDesc );

	return true;
}
