//////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_SkinRenderer.h
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __ASURA_XBOX360_SKINRENDERER_H__
#define __ASURA_XBOX360_SKINRENDERER_H__

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Rendering/Asura_Xbox360_IndexBuffer.h"
#include "Rendering/Asura_Xbox360_VertexBuffer.h"

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

class Asura_Animation;
class Asura_Hierarchy_Skin;
class Asura_Shadow;

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_SkinStrip
//
// Holds info for a strip in a skin
//
////////////////////////////////////////////////////////////////////////////////

struct Asura_Xbox360_SkinStrip
{
	Asura_Hash_ID m_auMaterials[2];
	u_int         m_uNumIndices;
	int           m_iElementID;
	u_int         m_uNumWeights;
};

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_Skin
//
// Stores a renderable skin mesh
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_Skin
{
	friend class Asura_Chunk_ResourceFile_Xbox360_Skin;

public:

	Asura_Xbox360_Skin();
	~Asura_Xbox360_Skin();

	const Asura_Xbox360_IndexBuffer&  GetIB() const { return m_xIB; }
	const Asura_Xbox360_VertexBuffer& GetVB() const { return m_xVB; }

	u_int GetNumStrips() const { return m_uNumStrips; }
	const Asura_Xbox360_SkinStrip& GetStrip( const int u ) const { return m_pxStrips[u]; }

	u_int GetNumTriangles() const { return m_uNumTriangles; }

	bool IsAnyMaterialTransparent() const;

protected:

	Asura_Xbox360_IndexBuffer  m_xIB;
	Asura_Xbox360_VertexBuffer m_xVB;

	Asura_Xbox360_SkinStrip* m_pxStrips;
	u_int                    m_uNumStrips;
	u_int                    m_uNumTriangles;
};

////////////////////////////////////////////////////////////////////////////////
//
// Asura_Xbox360_SkinRenderer
//
// Manager and renderer for 360 skinned meshes
//
////////////////////////////////////////////////////////////////////////////////

class Asura_Xbox360_SkinRenderer
{
public:

	static void Initialise();
	static void Reset();
	static void Shutdown();

	// This can be removed when the old renderers are deleted
	static bool IsEnabled();

	static void ResetCount();
	static void ResetBoneBuffer();

	static void OwnRegisters();
	static void DisownRegisters();

	static Asura_Xbox360_Skin* CreateSkin( const Asura_Hash_ID uHash, bool& bAlreadyExisted );
	static const Asura_Xbox360_Skin* GetSkin( const Asura_Hash_ID uHash );
	static Direct3D_VertexDeclaration* GetVertexDeclaration();

	static void RenderSkin( const Asura_Animation& xAnimation,
		const Asura_Hierarchy_Skin& xHierarchySkin, Asura_AMatrix4x4_InParam xWorldMat,
		const Asura_AMatrix4x4* const pxBoneMats, const Asura_Shadow* const pxShadow = NULL );

	static bool CopyBones( const Asura_AMatrix4x4* const pxBoneMats, const u_int uNumBones,
		const Asura_Animation& xAnim );

	static const u_int uVERTEX_SIZE;
};

#endif // !defined __ASURA_XBOX360_SKINRENDERER_H__
