/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float4x4 ViewProj;

const sampler BaseTexture = sampler_state
{
	AddressU = CLAMP;
	AddressV = CLAMP;
};

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 Position : POSITION0;
	float4 Colour   : COLOR0;
	float2 UV       : TEXCOORD0;
};

struct OutputVertex
{
	float4 Position : POSITION0;
	float4 Colour   : COLOR0;
	float2 UV       : TEXCOORD0;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex VShader( const InputVertex In )
{
	OutputVertex Out;

	Out.Position = mul( In.Position, ViewProj );
	Out.Colour = In.Colour;
	Out.UV = In.UV;

	return Out;
}

float4 PShader( const OutputVertex In ) : COLOR0
{
	return tex2D( BaseTexture, In.UV ) * In.Colour;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = TRUE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}