#include "Deferred_Shared.fxh"
#include "Shared.fxh"

const shared sampler g_xTexture : register( s0);

struct Asura_Direct3D_CGR_VSConstants_Wave
{
	float4 m_afWave;
};

struct Asura_Direct3D_CGR_VSConstants_Base
{
	float4 m_afTransform[4];
	float4 m_afBillboardRight;
	float4 m_afBillboardUp;
	float4 m_afCameraPlane;
	float4 m_afConstants2;
	float4 m_afCorners[4];
	
	Asura_Direct3D_CGR_VSConstants_Wave m_axWaves[16];

};

Asura_Direct3D_CGR_VSConstants_Base xConstants : register( c0);

struct Asura_Direct3D_CGR_VSConstants_Style
{
//	float m_afWidth, m_afHeight, m_afFadeStart, m_afFadeScale;
	float4 m_afDetails;
	float4 m_afUV;
};

float4	g_xMovementScale : register( c35);

Asura_Direct3D_CGR_VSConstants_Style xStyles[ 20] : register( c36);

struct vertexInput
{
    float4 xPosition	: POSITION0;
    float4 xColour0		: COLOR0;
    float4 xColour1		: COLOR1;    
};

struct vertexOutput
{
    float4 xPosition		: POSITION0;
	float4 xNormalDepth		: NORMAL;		// Contains world-space normal in xyz, and distance from near plane in w
    float2 xUV				: TEXCOORD0;
	float4 xColour			: TEXCOORD1;	// Colour and Alpha
	float3 xNormal			: TEXCOORD2;	// world space
};

vertexOutput VS_TransformDiffuse(vertexInput IN) 
{
    vertexOutput OUT;
    
    // DLH - Get the Corner Info
    float fCornerIndex = xConstants.m_afConstants2.x * IN.xColour1.z;
    float4 xCorner = xConstants.m_afCorners[ fCornerIndex];
    
    float fStyleIndex = xConstants.m_afConstants2.x * IN.xColour1.x;
    Asura_Direct3D_CGR_VSConstants_Style xStyle = xStyles[ fStyleIndex];
    
    float fWindIndex = xConstants.m_afConstants2.x * IN.xColour1.y;
    Asura_Direct3D_CGR_VSConstants_Wave xWind = xConstants.m_axWaves[ fWindIndex];
    
    float fDistance = dot( IN.xPosition.xyzw, xConstants.m_afCameraPlane.xyzw);
    float4 xFade = min( xConstants.m_afBillboardUp.w, (fDistance * xStyle.m_afDetails.w) + xStyle.m_afDetails.z);
    
    xCorner.xy *= IN.xColour1.ww;
    xCorner.x *= xStyle.m_afDetails.x;
    xCorner.y *= xStyle.m_afDetails.y;
    
    float4 xPosition;
    xPosition = IN.xPosition;
    xPosition.xyz += xConstants.m_afBillboardRight.xyz * xCorner.x;
    xPosition.xyz += xConstants.m_afBillboardUp.xyz * xCorner.y;
    xPosition.xyz += xWind.m_afWave.xyz * xCorner.y * g_xMovementScale.x;

   	OUT.xPosition.x = dot( xPosition.xyzw, xConstants.m_afTransform[0]);
   	OUT.xPosition.y = dot( xPosition.xyzw, xConstants.m_afTransform[1]);
   	OUT.xPosition.z = dot( xPosition.xyzw, xConstants.m_afTransform[2]);
   	OUT.xPosition.w = dot( xPosition.xyzw, xConstants.m_afTransform[3]);
	
	OUT.xUV.xy = (xCorner.zw * xStyle.m_afUV.zw) + xStyle.m_afUV.xy;
	
	OUT.xNormalDepth.xyz = cross( xConstants.m_afBillboardRight, xConstants.m_afBillboardUp);
	OUT.xNormalDepth.w = OUT.xPosition.w;
	
	OUT.xColour.xyz = 2.0f * IN.xColour0.xyz;
	OUT.xColour.w = xConstants.m_afBillboardRight.w * xFade.x;
   
    // Normal - artificially pointing upwards a bit
	OUT.xNormal.xyz = normalize(float3( 0.0, -1.0, 0.0 ) + normalize(g_xCameraPosition.xyz - IN.xPosition));
	
    return OUT;
}


DeferredOutputPixel PShader_Carpet_Prelighting( const vertexOutput xIn, float fFace:VFace )
{
	float2 xUV = xIn.xUV.xy;

	// Read the diffuse
	float4 xTextureColour;
	asm
	{
		tfetch2D xTextureColour, xUV, g_xTexture, MinFilter=linear, MagFilter=linear, MipFilter=linear
	};

	float fAlphaOut = xTextureColour.w - (1 - xIn.xColour.w);

	clip( fAlphaOut);

	// Get the normal, and xform it into world space
	half4 xColour;
	xColour.rgb = xTextureColour.rgb * xIn.xColour.rgb;
	xColour.a = fAlphaOut;

	DeferredOutputPixel xOut;

	xOut = Deferred_FillOutputPixel( xColour,
									 xIn.xNormal,
									 xIn.xNormalDepth.w,	// Distance
									 xTextureColour.rgb,	// Albedo
									 0.0,					// Specular mask
									 0.0f );				// Material v-coord

	return xOut;
}

technique
{
	pass
	{
		VertexShader = compile vs_3_0 VS_TransformDiffuse();
		PixelShader = compile ps_3_0 PShader_Carpet_Prelighting();
	}
}