/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

// Hack to replace albedo buffer alpha channel output (which usually contains
// specular mask) with transparency value, causing colour to blend into albedo.
// Needed for decal-type polygons on walls; should be removed when transparency
// is properly implemented

#if defined( NOT_EFFECT )

	#if !defined( BLEND_INTO_ALBEDO )
		#define BLEND_INTO_ALBEDO false
	#endif

	static bool g_bBlendIntoAlbedo = BLEND_INTO_ALBEDO;

#else // !defined( NOT_EFFECT )

	bool g_bBlendIntoAlbedo = false;

#endif // !defined( NOT_EFFECT )

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct DeferredOutputPixel
{
	half4 m_xBufferColour         : COLOR0;
	half4 m_xBufferNormal		  : COLOR1;
	half4 m_xBufferAlbedo		  : COLOR2;
	half4 m_xBufferDepthMtl		  : COLOR3;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

// NB, the layout differs between PC and 360

DeferredOutputPixel Deferred_FillOutputPixel( half4 xRGBA, float3 xNormal, float fDistance, half3 xAlbedo, half hSpecular, float fMaterial)
{
	DeferredOutputPixel xOut;
	xOut.m_xBufferColour     = xRGBA;
	xOut.m_xBufferNormal.xyz = (xNormal * 0.5f) + 0.5f;
	xOut.m_xBufferAlbedo.rgb = xAlbedo;
	xOut.m_xBufferAlbedo.a   = g_bBlendIntoAlbedo ? xRGBA.a : hSpecular;
	xOut.m_xBufferDepthMtl.r = fDistance;
	xOut.m_xBufferDepthMtl.g = fMaterial;
	
	xOut.m_xBufferDepthMtl.ba = xOut.m_xBufferNormal.a = 1;
	
	return( xOut);
}

void Deferred_GetNormalDepthAlbedoSpecularMaterial( half4 xBufferNormal, half4 xBufferAlbedo, half4 xBufferDepthMtl, out half3 xOutputNormal, out float fOutputDistance, out half3 xOutputAlbedo, out half hOutputSpecular, out float fMaterial)
{
	xOutputNormal   = (xBufferNormal.xyz * 2.0f) - 1.0f;
	
	fOutputDistance = xBufferDepthMtl.r;
	fMaterial       = xBufferDepthMtl.g;
	xOutputAlbedo   = xBufferAlbedo.xyz;
	hOutputSpecular = xBufferAlbedo.w;	
}

void Deferred_GetNormalDepthAlbedoSpecular( half4 xBufferNormal, half4 xBufferAlbedo, half4 xBufferDepthMtl,  out half3 xOutputNormal, out float fOutputDistance, out half3 xOutputAlbedo, out half hOutputSpecular)
{
	xOutputNormal   = (xBufferNormal.xyz * 2.0f) - 1.0f;

	fOutputDistance = xBufferDepthMtl.r;	
	xOutputAlbedo   = xBufferAlbedo.xyz;
	hOutputSpecular = xBufferAlbedo.w;	
}

void Deferred_GetNormalDepthMaterial( half4 xBufferNormal, half4 xBufferAlbedo, half4 xBufferDepthMtl,  out half3 xOutputNormal, out float fOutputDistance, out float fMaterial)
{
	xOutputNormal   = (xBufferNormal.xyz * 2.0f) - 1.0f;

	fOutputDistance = xBufferDepthMtl.r;	
	fMaterial       = xBufferDepthMtl.g;
}

void Deferred_GetAlbedoSpecular( half4 xBufferNormal, half4 xBufferAlbedo, half4 xBufferDepthMtl, out half3 xOutputAlbedo, out half hOutputSpecular)
{
	xOutputAlbedo   = xBufferAlbedo.xyz;
	hOutputSpecular = xBufferAlbedo.w;	
}


void Deferred_GetNormalDepth( half4 xBufferNormal, half4 xBufferDepthMtl,  out half3 xOutputNormal, out float fOutputDistance)
{
	xOutputNormal   = (xBufferNormal.xyz * 2.0f) - 1.0f;
	xOutputNormal.xyz = normalize( xOutputNormal.xyz);

	fOutputDistance = xBufferDepthMtl.r;	
}




