/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float3 g_xExtractTint;
const float g_fExtractScale;
const float g_fExtractBias;

const float g_fApplyLevel;

// (1/extract target width, 1/extract target height, 1/screen width, 1/screen height)
const float4 g_xRcpScreenSizes;

static const float g_afBlurWeights[7] = { 0.05f, 0.1f, 0.21f, 0.28f, 0.21f, 0.1f, 0.05f };

const sampler2D g_xBlurImage = sampler_state
{
	AddressU = CLAMP;
	AddressV = CLAMP;
};

shared const sampler2D g_xInitialImage = sampler_state
{
	AddressU = CLAMP;
	AddressV = CLAMP;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float4 VShader( const float4 xIn : POSITION ) : POSITION
{
	return xIn;
}

float4 PShader_Extract( float2 xScreenPos : VPOS ) : COLOR
{
	// Input viewport co-ords are with respect to the bloom texture
	xScreenPos *= g_xRcpScreenSizes.xy;

	float4 xInputColour;
	asm
	{
		tfetch2D xInputColour, xScreenPos.xy, g_xInitialImage, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled, OffsetX=1.0, OffsetY=1.0
	};

	const float fTintedBrightness = dot( xInputColour.rgb, g_xExtractTint );
	const float fRescaledBrightness = max( 0.0, fTintedBrightness * g_fExtractScale + g_fExtractBias );

	return fRescaledBrightness * fRescaledBrightness * fRescaledBrightness;
}

float4 PShader_HBlur( const float2 xScreenPos : VPOS ) : COLOR
{
	float4 xSample0, xSample1, xSample2, xSample3, xSample4, xSample5, xSample6;
	asm
	{
		tfetch2D xSample0, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=-2.5, OffsetY=0.5
		tfetch2D xSample1, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=-1.5, OffsetY=0.5
		tfetch2D xSample2, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=-0.5, OffsetY=0.5
		tfetch2D xSample3, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=0.5
		tfetch2D xSample4, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=1.5, OffsetY=0.5
		tfetch2D xSample5, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=2.5, OffsetY=0.5
		tfetch2D xSample6, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=3.5, OffsetY=0.5
	};

	return xSample0 * g_afBlurWeights[0]
		 + xSample1 * g_afBlurWeights[1]
		 + xSample2 * g_afBlurWeights[2]
		 + xSample3 * g_afBlurWeights[3]
		 + xSample4 * g_afBlurWeights[4]
		 + xSample5 * g_afBlurWeights[5]
		 + xSample6 * g_afBlurWeights[6];
}

float4 PShader_VBlur( const float2 xScreenPos : VPOS ) : COLOR
{
	float4 xSample0, xSample1, xSample2, xSample3, xSample4, xSample5, xSample6;
	asm
	{
		tfetch2D xSample0, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=-2.5
		tfetch2D xSample1, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=-1.5
		tfetch2D xSample2, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=-0.5
		tfetch2D xSample3, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=0.5
		tfetch2D xSample4, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=1.5
		tfetch2D xSample5, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=2.5
		tfetch2D xSample6, xScreenPos.xy, g_xBlurImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=3.5
	};

	return xSample0 * g_afBlurWeights[0]
		 + xSample1 * g_afBlurWeights[1]
		 + xSample2 * g_afBlurWeights[2]
		 + xSample3 * g_afBlurWeights[3]
		 + xSample4 * g_afBlurWeights[4]
		 + xSample5 * g_afBlurWeights[5]
		 + xSample6 * g_afBlurWeights[6];
}

float4 PShader_Apply( float2 xScreenPos : VPOS ) : COLOR
{
	// Input viewport co-ords are full-frame now
	xScreenPos *= g_xRcpScreenSizes.zw;

	float4 xBlurredColour;
	asm
	{
		tfetch2D xBlurredColour, xScreenPos.xy, g_xBlurImage, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=0.5
	};

	return xBlurredColour * g_fApplyLevel;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Extract();
	}

	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_HBlur();
	}

	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_VBlur();
	}

	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader_Apply();
	}
}