/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

shared const sampler2D g_xInitialImage = sampler_state
{
	AddressU = CLAMP;
	AddressV = CLAMP;
};

const sampler1D g_xColourMap = sampler_state
{
	AddressU = CLAMP;
};

const float g_fIntensity;

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

// Luminance = 0.27r + 0.67g + 0.06b
static half3 xLUMINANCE_CONSTANTS = { 0.27h, 0.67h, 0.06h };

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float4 VShader( const float4 xIn : POSITION ) : POSITION
{
	return xIn;
}

float4 PShader( const float2 xScreenPos : VPOS ) : COLOR
{
	float4 xInitialColour;
	asm
	{
		tfetch2D xInitialColour, xScreenPos.xy, g_xInitialImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled
	};

	// No need to saturate here, use clamp addressing
	const half hLuminance = dot( xInitialColour.rgb, xLUMINANCE_CONSTANTS );

	half4 xFinalColour;
	asm
	{
		tfetch1D xFinalColour, hLuminance, g_xColourMap, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled
	};

	xFinalColour = lerp( xInitialColour, xFinalColour, g_fIntensity );

	return half4( xFinalColour.rgb, 1.0h );
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = FALSE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}