/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float3 g_xTintColour;
const float g_fStrength;

shared const sampler2D g_xInitialImage = sampler_state
{
	AddressU = CLAMP;
	AddressV = CLAMP;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float4 VShader( const float4 xIn : POSITION ) : POSITION
{
	return xIn;
}

float4 PShader( const float2 xScreenPos : VPOS ) : COLOR
{
	float4 xInitialColour;
	asm
	{
		tfetch2D xInitialColour, xScreenPos, g_xInitialImage, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled
	};
	// Gamma to linear space
	xInitialColour.rgb = pow(xInitialColour.rgb, 2.2);

	const float3 xThirds = { 1.0 / 3.0, 1.0 / 3.0, 1.0 / 3.0 };
	const float fBrightness = dot( xInitialColour.rgb, xThirds );

	float4 xOutput;
	xOutput.rgb = lerp( xInitialColour, fBrightness * g_xTintColour, g_fStrength );
	xOutput.a = xInitialColour.a;

	// Linear to gamma space
	xOutput.rgb = pow(xOutput.rgb, 1.0f/2.2);

	return xOutput;
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}