/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const sampler2D g_xTexture = sampler_state
{
	AddressU = CLAMP;
	AddressV = CLAMP;
};

const float4x2 g_axVertexTransforms[4];
const float g_afSampleWeights[4];

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct OutputVertex
{
	float4 m_xPosition   : POSITION;
	float4 m_axUVPair[2] : TEXCOORD0; // Four pairs packed into two iterators
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

OutputVertex VShader( const float4 xIn : POSITION )
{
	OutputVertex xOut;

	xOut.m_xPosition = xIn;

	// Each vertex transform is a scale and offset for the blur, followed by a
	// mapping from [-1,1] space into [0,1]

	xOut.m_axUVPair[0].xy = mul( xIn, g_axVertexTransforms[0] );
	xOut.m_axUVPair[0].zw = mul( xIn, g_axVertexTransforms[1] );
	xOut.m_axUVPair[1].xy = mul( xIn, g_axVertexTransforms[2] );
	xOut.m_axUVPair[1].zw = mul( xIn, g_axVertexTransforms[3] );

	return xOut;
}

float4 PShader( const OutputVertex xIn ) : COLOR
{
	// Can't access arrays in inline asm
	const float2 xUV0 = xIn.m_axUVPair[0].xy;
	const float2 xUV1 = xIn.m_axUVPair[0].zw;
	const float2 xUV2 = xIn.m_axUVPair[1].xy;
	const float2 xUV3 = xIn.m_axUVPair[1].zw;

	float4 xSample0, xSample1, xSample2, xSample3;
	asm
	{
		// Don't forget to offset by half a texel, following sampling rules
		tfetch2D xSample0, xUV0, g_xTexture, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=0.5
		tfetch2D xSample1, xUV1, g_xTexture, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=0.5
		tfetch2D xSample2, xUV2, g_xTexture, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=0.5
		tfetch2D xSample3, xUV3, g_xTexture, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled, OffsetX=0.5, OffsetY=0.5
	};

	return xSample0 * g_afSampleWeights[0]
		 + xSample1 * g_afSampleWeights[1]
		 + xSample2 * g_afSampleWeights[2]
		 + xSample3 * g_afSampleWeights[3];
}

/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}