/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const texture g_xBlurredTexture;
const sampler g_xBlurredTextureSampler : register(s0) = sampler_state
{
	Texture = <g_xBlurredTexture>;
	AddressU	= CLAMP;
	AddressV	= CLAMP;

};

const texture g_xDepthTexture;
const sampler g_xDepthTextureSampler : register(s1) = sampler_state
{
	Texture = <g_xDepthTexture>;
	AddressU = CLAMP;
	AddressV = CLAMP;
};
			  
const float   g_fEffectStrength;
const float4  g_xFocalDistances;

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float4 VShader( const float4 xIn : POSITION ) : POSITION
{
	return xIn;
}

float4 PShader( const float2 xScreenPos : VPOS ) : COLOR
{
	float4 xCol;
	
	float2 xScaledScreenPos = 0.5f * xScreenPos;
	
	asm
	{
		tfetch2D xCol, xScaledScreenPos.xy, g_xBlurredTextureSampler, UnnormalizedTextureCoords=true, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled
	};


	float4 xDepth;
	asm
	{
		tfetch2D xDepth, xScreenPos.xy, g_xDepthTextureSampler, UnnormalizedTextureCoords=true, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled
	};


	float fBlurNear = saturate(1.0 - (xDepth.r-g_xFocalDistances.x) / (g_xFocalDistances.y-g_xFocalDistances.x));
	float fBlurFar = saturate((xDepth.r - g_xFocalDistances.z) / (g_xFocalDistances.w-g_xFocalDistances.z));
	
	xCol.a = saturate(max(max(fBlurNear,xCol.a), fBlurFar)); 	
	
	xCol.a = xCol.a * g_fEffectStrength;

	return saturate(xCol);
}



/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}