
//------------------------------------
#define	MAX_PARALLAX_MAP_ITERATIONS	2

//------------------------------------
float	g_fParallaxMapScale	= 0.04;
float	g_fParallaxMapBias	= -0.02;

//------------------------------------
float CalculateLineIntersectionParametric (float fH0,
										   float fH1)
{
	/////////////////////////////////////////////////////////////////////
	// Uses line intersection calculation between a 2D representation of
	// the eye vector (line a) and a line with the same x values, but
	// the y values equalling the heights of the 2 samples (line b).
	//
	// The parametric point of intersection for these lines is solved:
	//
	//         (x4 - x3)(y1 - y3) - (y4 - y3)(x1 - x3)
	// t(a) = -----------------------------------------
	//         (x4 - y3)(x2 - x1) - (x4 - x3)(y2 - y1)
	// OR:
	//         (x2 - x1)(y1 - y3) - (y2 - y1)(x1 - x3)
	// t(b) = -----------------------------------------
	//         (y4 - y3)(x2 - x1) - (x4 - x3)(y2 - y1)
	//
	// Where:
	// ~~~~~~
	// Line a: P1 = (0, 0),  P2 = (x, h0)
	// Line b: P3 = (0, h0), P4 = (x, h1)
	//
	// Reducing either of these equations leaves us with the following
	// equation for the parametric value between the 2 lines:
	//
	//         -h0
	// t = -----------
	//      h1 - 2.h0
	/////////////////////////////////////////////////////////////////////
	
	float	fNumerator		= -fH0;
	float	fDenominator	= fH1 - (2 * fH0);
	float	fParametric		= saturate (fNumerator / fDenominator);

	// Return the result of the intersections...
	return	(fParametric);
}

/// End of file. ///
