// QuadOverlayRenderer.fx
#include <Shared.fxh>

//------------------------------------
const float4	g_xFlatColour;

//------------------------------------
struct VS_FlatColour_Input
{
	float2 Position		: POSITION0;
	float2 UV			: TEXCOORD0;
};

struct VS_FlatColour_Output
{
	float4 Position		: POSITION0;
	float2 UV			: TEXCOORD0;
};

struct VS_GouraudColour_Input
{
	float2 Position		: POSITION0;
	float2 UV			: TEXCOORD0;
	float4 Colour		: COLOR;
};

struct VS_GouraudColour_Output
{
	float4 Position		: POSITION0;
	float2 UV			: TEXCOORD0;
	float4 Colour		: COLOR;
};

struct PS_Output
{
	float4 Colour		: COLOR;
};

uniform sampler xSampler = sampler_state
{
	AddressU = WRAP;
	AddressV = WRAP;
};

//------------------------------------
VS_FlatColour_Output VS_FlatColour_Overlay(VS_FlatColour_Input IN) 
{
	VS_FlatColour_Output OUT;

	// Mainly a straight pass-through job, but
	// we want to make z & w represent a polygon
	// right in front of the camera, i.e. a z
	// distance of zero. This makes w = 1.0f.

	OUT.Position.xy = IN.Position.xy;
	
	OUT.Position.z = 0.0f;
	OUT.Position.w = 1.0f;

	OUT.UV = IN.UV;
	
	return OUT;
}

PS_Output PS_FlatColour_Overlay (VS_FlatColour_Output IN)
{
	PS_Output    OUT;

	// take the texture and modulate it by the diffuse colour
	OUT.Colour = TexFetch2D_LinearLinearLinear ( xSampler, IN.UV ) * g_xFlatColour;

	return OUT;
}

//------------------------------------
VS_GouraudColour_Output VS_GouraudColour_Overlay(VS_GouraudColour_Input IN) 
{
	VS_GouraudColour_Output OUT;

	// Mainly a straight pass-through job, but
	// we want to make z & w represent a polygon
	// right in front of the camera, i.e. a z
	// distance of zero. This makes w = 1.0f.

	OUT.Position.xy = IN.Position.xy;
	
	OUT.Position.z = 0.0f;
	OUT.Position.w = 1.0f;

	OUT.UV = IN.UV;
	
	OUT.Colour.rgba = IN.Colour.rgba;
	
	return OUT;
}

PS_Output PS_GouraudColour_Overlay (VS_GouraudColour_Output IN)
{
	PS_Output    OUT;

	// take the texture and modulate it by the diffuse colour
	OUT.Colour = TexFetch2D_LinearLinearLinear ( xSampler, IN.UV ) * IN.Colour;

	return OUT;
}


//-----------------------------------
technique OverlayQuad_FlatColour
{
	pass Pass1
	{
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_2_0 VS_FlatColour_Overlay();
		PixelShader = compile ps_2_0 PS_FlatColour_Overlay();
	}
}

technique OverlayQuad_GouraudColour
{
	pass Pass1
	{
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_2_0 VS_GouraudColour_Overlay();
		PixelShader = compile ps_2_0 PS_GouraudColour_Overlay();
	}
}