// QuadOverlayRenderer.fx

//------------------------------------
struct VS_Input
{
	float2 Position		: POSITION0;
	float2 UV			: TEXCOORD0;
};

struct VS_Output
{
	float4 Position		: POSITION0;
	float2 UV			: TEXCOORD0;
};

struct PS_Output
{
	float4 Colour		: COLOR;
};

uniform sampler xSampler = sampler_state
{
	AddressU = WRAP;
	AddressV = WRAP;
};

//------------------------------------
VS_Output VS_Overlay(VS_Input IN) 
{
	VS_Output OUT;

	// Mainly a straight pass-through job, but
	// we want to make z & w represent a polygon
	// right in front of the camera, i.e. a z
	// distance of zero. This makes w = 1.0f.

	OUT.Position.xy = IN.Position.xy;
	
	OUT.Position.z = 0.0f;
	OUT.Position.w = 1.0f;

	OUT.UV = IN.UV;
	
	return OUT;
}
float4 fColours[8] = {
	{0.0f,0.0f,0.0f, 1.0f},
	
	{1.0f,0.0f,0.0f, 1.0f},
	{0.0f,1.0f,0.0f, 1.0f},
	{0.0f,0.0f,1.0f, 1.0f},
	
	{0.0f,1.0f,1.0f, 1.0f},
	{1.0f,1.0f,0.0f, 1.0f},
	{1.0f,0.0f,1.0f, 1.0f},
	
	{1.0f,1.0f,1.0f, 1.0f},
};

//------------------------------------
PS_Output PS_Overlay (VS_Output IN)
{
	PS_Output    OUT;
	float fDepth = clamp(tex2D ( xSampler, IN.UV ).x,0,1) * 8.0f;
	
	if (fDepth < 1.0f)
		OUT.Colour = lerp(fColours[0], fColours[1], fDepth);
	else if (fDepth < 2.0f)
		OUT.Colour = lerp(fColours[1], fColours[2], fDepth - 1.0f);
	else if (fDepth < 3.0f)
		OUT.Colour = lerp(fColours[2], fColours[3], fDepth - 2.0f);
	else if (fDepth < 4.0f)
		OUT.Colour = lerp(fColours[3], fColours[4], fDepth - 3.0f);
	else if (fDepth < 5.0f)
		OUT.Colour = lerp(fColours[4], fColours[5], fDepth - 4.0f);
	else if (fDepth < 6.0f)
		OUT.Colour = lerp(fColours[5], fColours[6], fDepth - 5.0f);
	else if (fDepth < 7.0f)
		OUT.Colour = lerp(fColours[6], fColours[7], fDepth - 6.0f);
	else 
		OUT.Colour = lerp(fColours[7], fColours[0], fDepth - 7.0f);


	return OUT;
}


//-----------------------------------
technique OverlayQuad
{
	pass Pass1
	{
		AlphaTestEnable = FALSE;
		CullMode = NONE;
		ZEnable = FALSE;
		ZWriteEnable = FALSE;

		VertexShader = compile vs_2_0 VS_Overlay();

		PixelShader = compile ps_2_0 PS_Overlay();
	}
}