// Camera-related variables...

shared 	float4	g_xCameraPosition	: CameraPosition;


// View-related variables...

shared 	float4x4	g_xViewProjectionMatrix	: ViewProjection;

shared 	float2	g_xVPOSOffsetVector;


// High Dynamic Range stuff

shared 	float 	g_fHighDynamicRangeExposureScale	= 4.0f;

// Tangent-space rotation function...

float3 RotateIntoTangentSpace (float3 xVector,
							   float3 xTangent,
							   float3 xBinormal,
							   float3 xNormal)
{
	float3	xRotatedVector;

	// Rotate the x, y and z components into tangent-space...
	xRotatedVector.x	= dot(xVector, xTangent);
	xRotatedVector.y	= dot(xVector, xBinormal);
	xRotatedVector.z	= dot(xVector, xNormal);

	// Return the rotated vector...
	return	(xRotatedVector);
}


// High-dynamic range calculation function...

float3 CalculateHighDynamicRangeRGB (float3 xColour)
{
	// Move High Dynamic Range lighting into display range...
	xColour	*= g_fHighDynamicRangeExposureScale;
	xColour	 = (xColour) / (xColour + 1.0);
	
	// Return the colour...
	return	(xColour);
}

//Naming convention for texture fetch functions are 
// TexFetch<type><return type>_<MinFilter><MagFilter><Mipfilter>

float4 TexFetchCube_LinearLinearPoint(samplerCUBE xSampler, float3 xUV)
{
	float4 fRet;
	const float f1_5 = 1.5f;
	asm
	{
		cube fRet, xUV.zzxy, xUV.yxz
		rcp fRet.z, fRet_abs.z
		mad fRet.xy, fRet, fRet.zzzw, f1_5

		tfetchCube fRet, fRet.yxw, xSampler, UnnormalizedTextureCoords=false, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled
	};
	return fRet;
}

float4 TexFetch2D_PointPointPoint(sampler2D xSampler, float2 xUV)
{
	float4 fRet;
	asm
	{
		tfetch2D fRet, xUV, xSampler, UnnormalizedTextureCoords=false, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled
	};
	return fRet;
}

float4 TexFetch2D_PointPointNone(sampler2D xSampler, float2 xUV)
{
	float4 fRet;
	asm
	{
		tfetch2D fRet, xUV, xSampler, UseComputedLOD=false, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled
	};
	return fRet;
}

float4 TexFetch2D_LinearLinearNone(sampler2D xSampler, float2 xUV)
{
	float4 fRet;
	asm
	{
		tfetch2D fRet, xUV, xSampler, UseComputedLOD=false, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled
	};
	return fRet;
}

float4 TexFetch2D_LinearLinearLinear(sampler2D xSampler, float2 xUV)
{
	float4 fRet;
	asm
	{
		tfetch2D fRet, xUV, xSampler, MinFilter=linear, MagFilter=linear, MipFilter=linear, AnisoFilter=disabled
	};
	return fRet;
}


float4 TexFetch2DProj_LinearLinearNone(sampler2D xSampler, float4 xUV)
{
	float4 fRet;
	xUV.xy /= xUV.w;
	asm
	{
		tfetch2D fRet, xUV, xSampler, UseComputedLOD=false, MinFilter=linear, MagFilter=linear, MipFilter=point, AnisoFilter=disabled
	};
	return fRet;
}

float4 TexFetch1D_PointPointPoint(sampler2D xSampler, float xUV)
{
	float4 fRet;
	asm
	{
		tfetch1D fRet, xUV, xSampler, UnnormalizedTextureCoords=false, MinFilter=point, MagFilter=point, MipFilter=point, AnisoFilter=disabled
	};
	return fRet;
}


/// End of file. ///
