/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

float2   g_xShadowFrustum : register(c4); // (x = near plane, y = 1 / (far plane - near plane)
float4x4 g_xWorldViewProj : register(c8);

int g_iNumBoneWeightings  : register(i0);

/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 m_xPosition     : POSITION;
	float4 m_xBlendIndices : BLENDINDICES;
	float4 m_xBlendWeights : BLENDWEIGHT;
};

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float3x4 FetchBone( const int fIndex )
{
	float4 xBoneMat0, xBoneMat1, xBoneMat2;

	// Bones stored in semantics POSITION1-3. Ignore the last row, it's always (0 0 0 1)

	asm
	{
		vfetch xBoneMat0, fIndex, position1, UseTextureCache=true
		vfetch xBoneMat1, fIndex, position2, UseTextureCache=true
		vfetch xBoneMat2, fIndex, position3, UseTextureCache=true
	};

	// Per-bone texturing blend factors ("damage values") are packed into one of the matrix
	// elements. We can reconstruct the row (assuming they're orthogonal) with a cross product
	xBoneMat2.xyz = cross( xBoneMat0.xyz, xBoneMat1.xyz );

	return float3x4( xBoneMat0, xBoneMat1, xBoneMat2 );
}

float4 main( const InputVertex xIn, out float fDepth : TEXCOORD ) : POSITION
{
	// Accumulate the blended skinning matrix

	float3x4 xBoneMat =
	{
		{ 0.0f, 0.0f, 0.0f, 0.0f },
		{ 0.0f, 0.0f, 0.0f, 0.0f },
		{ 0.0f, 0.0f, 0.0f, 0.0f },
	};

	for( int i = 0; i < g_iNumBoneWeightings; ++i )
	{
		const float fWeight = xIn.m_xBlendWeights[i];

		[branch]
		if( fWeight > 0.0f )
		{
			const float3x4 xMat = FetchBone( xIn.m_xBlendIndices[i] );
			xBoneMat += xMat * fWeight;
		}
	}

	const float4 xPos = float4( mul( xBoneMat, xIn.m_xPosition ), 1.0f );
	const float4 xOutPos = mul( g_xWorldViewProj, xPos );

	fDepth = (xOutPos.w - g_xShadowFrustum.x) * g_xShadowFrustum.y;

	return xOutPos;
}