

///////////////////////////////////////////////////////////////////////////////
//
// File   : Asura_Platform_Decal_System.h
// Author : Dave Reed
// Date   : 15-Aug-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////


#include "Rendering/Asura_Colour.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Core/Asura_HalfFloat.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Decal_System.h"
#include "SpecialFX/Asura_Decal_Template.h"
#include "Rendering/Asura_Xbox360_Effect_System.h"
#include "Rendering/Asura_Xbox360_Graphics.h"
#include "Rendering/Asura_Xbox360_TextureManagement.h"
#include "Rendering/Asura_Xbox360_IndexBuffer.h"
#include "Rendering/Asura_Xbox360_VertexBuffer.h"
#include "Rendering/Asura_Xbox360_MaterialResponse_System.h"


// ----------------------------------------------------------------------------
// Vertex Formats - with and without support for normal mapping
// ----------------------------------------------------------------------------

class Asura_Xbox360_Decal_Vertex
{
public:

	float			fX,fY,fZ;
	float			fU,fV;	
	float			fCreationTime;
	float			fAlpha;
	uint32			uNormal;
	uint32			uTangent;
	uint32		    uBinormal;
	Asura_HalfFloat	afSHRed[4];
	Asura_HalfFloat	afSHGreen[4];
	Asura_HalfFloat	afSHBlue[4];
};


// ----------------------------------------------------------------------------
// ShaderParameters object
// ----------------------------------------------------------------------------

class Asura_Xbox360_Decal_ShaderParameters
{	
public:
	Asura_Xbox360_MaterialResponse_Handles m_xMaterialResponseHandles;
	// (We rely on shared params being set for WVP matrix and camera pos)
	Asura_Xbox360_Effect_System::Handle m_xFadeValues;	
	Asura_Xbox360_Effect_System::Handle m_xUVScroll;
	Asura_Xbox360_Effect_System::Handle m_xPrelightingColourFilter;

	void Get(const Asura_Xbox360_Effect *pxEffect)
	{

		m_xFadeValues	  = pxEffect->GetParameterHandle("g_xFadeValues");
		m_xUVScroll		  = pxEffect->GetParameterHandle("g_xUVScroll");
		m_xPrelightingColourFilter		  = pxEffect->GetParameterHandle("g_xPrelightingColourFilter");

		m_xMaterialResponseHandles.InitialiseFromEffect( *pxEffect );
	}
	void SetMRParameters(Asura_Xbox360_Effect *pxEffect, const Asura_MaterialResponse* const pxMaterial)
	{
		Asura_Xbox360_MaterialResponse_System::SetEffectParameters( pxMaterial, NULL, NULL, m_xMaterialResponseHandles, *pxEffect );
		Asura_Xbox360_MaterialResponse_System::SetSHParameters( pxMaterial, m_xMaterialResponseHandles, *pxEffect );
		Asura_Xbox360_MaterialResponse_System::SetFogParameters( false, pxMaterial, m_xMaterialResponseHandles, *pxEffect );

		Asura_RGBA xPrelightingColour = pxMaterial->GetAmbientColour();
		xPrelightingColour *= pxMaterial->GetAmbientContribution();

		float afPrelightingColourVector[4] = {xPrelightingColour.m_fR, xPrelightingColour.m_fG, xPrelightingColour.m_fB, 0.0f };

		pxEffect->GetInterface()->SetVectorF(m_xPrelightingColourFilter, afPrelightingColourVector);


	}
};



// ----------------------------------------------------------------------------
// Asura_Xbox360_DecalBuffer
// ----------------------------------------------------------------------------

class Asura_Xbox360_DecalBuffer : public Asura_DecalBuffer
{
	friend class Asura_Decal_Handler;

public:

	// These are created the first time a decal of this material type is created
	// This isn't ideal - but we can't pre-alloc them as we don't know how many 
	// decal templates there are.
	Asura_Xbox360_DecalBuffer(const Asura_Decal_Template* pxDecalTemplate);
	~Asura_Xbox360_DecalBuffer();

	void AddStrip(const Asura_Decal_Vertex* pxStrip, const u_int uNumberOfVertices, const Asura_Decal_Template* pxDecalTemplate, const Asura_RGBA& xRGBA );
	void Render(void);

private:

	class Decal
	{
	public:
		int		iFirstIndex;
		int		iNumIndices;		
		float	fCreationTime;
	};

	int							 m_iMaxDecals;				// Max decals in this buffer
	int							 m_iDecalMaterialIndex;		// Material used by this buffer	
	Decal*						 m_apxDecals;				// Decal structs (circular buffer)
	int							 m_iNextVertex;				// Next vertex to use (verts = another circular buffer)
	int							 m_iNumVerts;
	int							 m_iNumIndices;
	uint16*						 m_apxIndices;				// Indices 
	int							 m_iFirstDecal;				// First decal struct in use
	int							 m_iNumDecals;				// Num decals active	
	int							 m_iNumDecalTris;			// Num decal triangles active
	bool						 m_bIndexBufferInvalid;		// Index buffer contents need updating

	IDirect3DVertexBuffer9*		 m_pxVB;					// Vertex buffer
	IDirect3DIndexBuffer9*		 m_apxIB[2];				// Double-buffered index buffer
	int							 m_iCurrentIndexBuffer;	

	Asura_Xbox360_DecalBuffer*   m_pxNextDecalBuffer;
};


// ----------------------------------------------------------------------------
// Asura_Xbox360_StaticDecalBuffer
// ----------------------------------------------------------------------------

class Asura_Xbox360_StaticDecalBuffer
{
public:

	int							m_iModuleIndex;
	uint32						m_uDecalMaterialHash;
	int							m_iDecalMaterialIndex;
	int							m_iNumVertices;
	int							m_iNumIndices;
	Asura_Bounding_Box			m_xAABB;
	
	Asura_Xbox360_IndexBuffer	m_xIndexBuffer;
	Asura_Xbox360_VertexBuffer	m_xVertexBuffer;
};


// ----------------------------------------------------------------------------
// Asura_Xbox360_Decal_System
// ----------------------------------------------------------------------------

class Asura_Xbox360_Decal_System
{

public:
	static void Initialise(void);
	static void Shutdown(void);
	static void Reset(void);
	static void RenderStaticDecals(void);

	static void CalculateTangentAndBinormal( const  Asura_Vector_3* pxPositions,
											 const  Asura_Vector_2* pxTextureCoords,
													Asura_Vector_3& rxTangent,
													Asura_Vector_3& rxBinormal );


	static Asura_Xbox360_Effect*				 GetEffect(void)		{return  s_pxDecalEffect; };
	static Asura_Xbox360_Decal_ShaderParameters* GetEffectParameters()  {return &s_xShaderParams; };
	static Direct3D_VertexDeclaration*			 GetVertexDecl()		{return  s_pxDecalVertexDecl;};


	static void									 AllocateStaticDecalBuffers(int iNumBuffers);
	static Asura_Xbox360_StaticDecalBuffer*		 GetStaticDecalBuffer(int iIndex);


	static void PrepareRenderStates(void);
	static void RestoreRenderStates(void);

	static void SetupMaterial(int iPass, const Asura_Decal_Material *pxMtl, float *pfFadeValues);

private:

	// The declaration objects
	static Direct3D_VertexDeclaration*			 s_pxDecalVertexDecl;

	// The shader
	static Asura_Xbox360_Effect*				 s_pxDecalEffect;

	// Its parameters
	static Asura_Xbox360_Decal_ShaderParameters  s_xShaderParams;

	// Static decal buffers
	static int									 s_iNumStaticDecalBuffers;
	static Asura_Xbox360_StaticDecalBuffer*		 s_pxStaticDecalBuffers;
};



