// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS



template <typename T, typename U> inline
bdTernaryTree<T,U>::bdTernaryTree()
: m_root(BD_NULL)
{
	// empty
}

template <typename T, typename U> inline
bdTernaryTree<T,U>::~bdTernaryTree()
{
	const bdTernaryTree<T,U>::Position root = reinterpret_cast<bdTernaryTree<T,U>::Position>(m_root);
	deleteTree(root);
	m_root = BD_NULL;
}

template <typename T, typename U> inline
typename bdTernaryTree<T,U>::Position bdTernaryTree<T,U>::search(const U *ptr,
																 const bdUInt len) const
{
	bdTernaryTree<T,U>::Node *p  = m_root;
	bdUInt index = 0;
	while ((index<len) && p)
	{
		const U &keyElement = ptr[index];
		const U &splitval = p->m_splitval;

		if (keyElement < splitval)
		{
			p = p->m_lo;
		}
		else if (keyElement == splitval)
		{
			if (index == (len - 1))
			{
				break;
			}
			p = p->m_eq;
			index++;
		}
		else
		{
			p = p->m_hi;
		}
    }

	return (reinterpret_cast<bdTernaryTree<T,U>::Position>(p));
}

template <typename T, typename U> inline
bdBool bdTernaryTree<T,U>::get(const U *ptr, const bdUInt len, 
							   T &data) const
{
	return (get(search(ptr, len), data));
}

template <typename T, typename U> inline
bdBool bdTernaryTree<T,U>::get(const Position pos,
							   T &data) const
{
	const bdTernaryTree<T,U>::Node *const node = reinterpret_cast<const bdTernaryTree<T,U>::Node*>(pos);

	if (node)
	{
		if (node->m_data)
		{
			data = *(node->m_data);
			return true;
		}
	}

	return false;
}

template <typename T, typename U> inline
bdBool bdTernaryTree<T,U>::insert(const U *ptr,
								  const bdUInt len,
								  const T &data)
{
	bdTernaryTree<T,U>::Node **pp      = &m_root;
	bdTernaryTree<T,U>::Node  *p       = *pp;
	bdTernaryTree<T,U>::Node  *parent  = BD_NULL;
	bdUInt index = 0;

	while (p && (index < len))
	{
		const U &keyElement = ptr[index];
		const U &splitval = p->m_splitval;

		if (keyElement == splitval)
		{
			parent = p;
			pp     = &(p->m_eq);
			p      = p->m_eq;
			index++;
		}
		else if (keyElement > splitval)
		{
			parent = p;
			pp     = &(p->m_hi);
			p      = p->m_hi;
		}
		else
		{
			parent = p;
			pp     = &(p->m_lo);
			p      = p->m_lo;
		}
	}

	while (index < len)
	{
		p = new bdTernaryTree<T,U>::Node(ptr[index], parent);
		*pp  = p;
		parent = p;
		pp = &(p->m_eq);
		index++;
	}

	p = parent;

	if (p->m_data)
	{
		// already data for this key
		return false;
	}
	else
	{
		p->m_data = new T(data);
		return true;
	}
}

template <typename T, typename U> inline
bdBool bdTernaryTree<T,U>::remove(const U *ptr, 
								  const bdUInt len)
{
	return remove(search(ptr, len));
}

template <typename T, typename U> inline
bdBool bdTernaryTree<T,U>::remove(const Position pos)
{
	bdTernaryTree<T,U>::Node *node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
	bdBool hasData = false;

	// check if this node contains data
	if (node)
	{
		if (node->m_data)
		{
			delete node->m_data;
			node->m_data = BD_NULL;
			hasData = true;
		}

		cleanupNode(reinterpret_cast<bdTernaryTree<T,U>::Position &>(node));
	}

	return hasData;
}

template <typename T, typename U> inline
bdBool bdTernaryTree<T,U>::replace(const U *ptr,
								   bdUInt len,
								   T &oldData,
								   const T &newData)
{
	return replace(search(ptr, len), oldData, newData);
}

template <typename T, typename U> inline
bdBool bdTernaryTree<T,U>::replace(const Position postition,
								   T &oldData,
								   const T &newData)
{
	Node *const node = reinterpret_cast<Node*>(postition);

	if (node)
	{
		if (node->m_data)
		{
			// copy the old data then delete it
			oldData = *(node->m_data);
			delete node->m_data;

			T *const newDataCopy = new T(newData);
			node->m_data = newDataCopy;
			return true;
		}
	}

	return false;
}

template <typename T, typename U> inline
void bdTernaryTree<T,U>::deleteTree(const Position pos)
{
	if(pos)
	{
		bdTernaryTree<T,U>::Node *node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
		if (node->m_lo)
		{
			const Position pos = reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_lo);
			deleteTree(pos);
			node->m_lo = BD_NULL;
		}

		if (node->m_eq)
		{
			const Position pos = reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_eq);
			deleteTree(pos);
			node->m_eq = BD_NULL;
		}

		if (node->m_hi)
		{
			Position pos = reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_hi);
			deleteTree(pos);
			node->m_hi = BD_NULL;
		}

		delete node;
		
		if (node == m_root)
		{
			m_root = BD_NULL;
		}
	}
}

template <typename T, typename U> inline
typename bdTernaryTree<T,U>::Position bdTernaryTree<T,U>::getRoot() const
{
	const bdTernaryTree<T,U>::Position root = reinterpret_cast<bdTernaryTree<T,U>::Position>(m_root);
	return root;
}

template <typename T, typename U> inline
typename bdTernaryTree<T,U>::Position bdTernaryTree<T,U>::getParent(const Position pos) const
{
	const bdTernaryTree<T,U>::Node *const node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
	return reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_parent);
}

template <typename T, typename U> inline
typename bdTernaryTree<T,U>::Position bdTernaryTree<T,U>::getEqNode(const Position pos) const
{
	const bdTernaryTree<T,U>::Node *const node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
	return reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_eq);
}

template <typename T, typename U> inline
typename bdTernaryTree<T,U>::Position bdTernaryTree<T,U>::getLoNode(const Position pos) const
{
	const bdTernaryTree<T,U>::Node *const node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
	return reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_lo);
}

template <typename T, typename U> inline
typename bdTernaryTree<T,U>::Position bdTernaryTree<T,U>::getHiNode(const Position pos) const
{
	const bdTernaryTree<T,U>::Node *const node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
	return reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_hi);
}

template <typename T, typename U> inline
void bdTernaryTree<T,U>::print() const
{
	bdFastArray<bdFastArray<bdTernaryTree<T,U>::Position>*> array;
	getAllNodes(array);
	bdUInt nodeCount     = 0;
	bdUInt dataNodeCount = 0;
	for (bdUInt i=0; i<array.getSize(); i++)
	{
		for (bdUInt j=0; j<array[i]->getSize(); j++)
		{
			bdTernaryTree<T,U>::Position pos;
			array[i]->get(j, pos);
			bdTernaryTree<T,U>::Node *node;
			node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
			if (node->m_data)
			{
				bdPrintf ("*");
				dataNodeCount++;
			}
			else
			{
				bdPrintf ("o");
			}
			nodeCount++;
		}
		bdPrintf ("\n");
	}
	bdPrintf ("Ternary tree with %d nodes, %d with data\n",nodeCount,dataNodeCount);
	for (bdUInt j=0; j<array.getSize(); j++)
	{
		delete (array[j]);
	}
}

template <typename T, typename U> inline
void bdTernaryTree<T,U>::getAllDataNodes(bdFastArray<Position> &array, 
										 const Position pos) const
{
	bdTernaryTree<T,U>::Node *node;
	if (pos == BD_NULL)
	{
		node = m_root;
	}
	else
	{
		node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
	}
	if (node == BD_NULL)
	{
		return;
	}
	if (node->m_data)
	{
		array.pushBack(reinterpret_cast<bdTernaryTree<T,U>::Position>(node));
	}
	if (node->m_lo)
	{
		getAllDataNodes(array, reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_lo));
	}
	if (node->m_eq)
	{
		getAllDataNodes(array, reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_eq));
	}
	if (node->m_hi)
	{
		getAllDataNodes(array, reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_hi));
	}
}

template <typename T, typename U> inline
void bdTernaryTree<T,U>::getKey(const Position pos, bdFastArray<U> &key) const
{
	bdTernaryTree<T,U>::Node *node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
	BD_ASSERT(node != BD_NULL,"bdTernaryTree::getKey, node must not be null");
	
	recursiveGetKey(node, key);
	key.pushBack(node->m_splitval);
}

template <typename T, typename U> inline
void bdTernaryTree<T,U>::cleanupNode(const Position pos)
{
	bdTernaryTree<T,U>::Node *node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
	BD_ASSERT(node != BD_NULL,"bdTernaryTree::cleanupNode, node must not be null");

	if ((node->m_lo == BD_NULL) &&
		(node->m_eq == BD_NULL) &&
		(node->m_hi == BD_NULL) &&
		(node->m_data == BD_NULL))
	{
		bdTernaryTree<T,U>::Node *const parent = node->m_parent;

		if (parent == BD_NULL)
		{
			BD_ASSERT (node == m_root,
					   "bdTernaryTree::cleanupNode, root must be the only node without parent.");

			m_root = BD_NULL;
		}
		else if (parent->m_lo == node)
		{
			parent->m_lo = BD_NULL;
		}
		else if (parent->m_eq == node)
		{
			parent->m_eq = BD_NULL;
		}
		else if (parent->m_hi == node)
		{
			parent->m_hi = BD_NULL;
		}
		else
		{
			BD_ASSERT (false, "bdTernaryTree::cleanupNode, inconsistent tree.");
		}

		if (parent)
		{
			cleanupNode(reinterpret_cast<const bdTernaryTree<T,U>::Position>(parent));
		}

		delete node;
	}
}

template <typename T, typename U> inline
void bdTernaryTree<T,U>::getAllNodes(bdFastArray<bdFastArray<Position>*> &array, 
									 const Position pos, 
									 const bdUInt y) const
{
	bdTernaryTree<T,U>::Node *node;
	if (pos == BD_NULL)
	{
		if(m_root)
		{
			node = m_root;
		}
		else
		{
			return;
		}
	}
	else
	{
		node = reinterpret_cast<bdTernaryTree<T,U>::Node*>(pos);
	}

	while (array.getSize()<=y)
	{
		array.pushBack(new bdFastArray<Position>());
	}
	(array[y])->pushBack(reinterpret_cast<bdTernaryTree<T,U>::Position>(node));
	if (node->m_lo)
	{
		getAllNodes(array, reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_lo), y + 1);
	}
	if (node->m_eq)
	{
		getAllNodes(array, reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_eq), y + 1);
	}
	if (node->m_hi)
	{
		getAllNodes(array, reinterpret_cast<bdTernaryTree<T,U>::Position>(node->m_hi), y + 1);
	}
}

template <typename T, typename U> inline
void bdTernaryTree<T,U>::recursiveGetKey(Node* node, bdFastArray<U> &key) const
{
	if (node->m_parent == BD_NULL)
	{
		return;
	}

	recursiveGetKey(node->m_parent, key);

	if (node->m_parent->m_eq == node)
	{
		key.pushBack(node->m_parent->m_splitval);
		return;
	}
}

