// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: WIN32 PS2 UNIX OSX PSP-INFRA PS3
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdCore/bdCrypto/bdECCKey.h>

#include <bdCore/bdUtilities/bdTrulyRandom.h>

#define BD_LOG_LEVEL "ecckey"

bdECCKey::bdECCKey()
: m_madeKey(false)
{
	ltc_mp = ltm_desc;
}

bdECCKey::~bdECCKey()
{
	if (m_madeKey)
	{
		ecc_free(&m_key);
		m_madeKey = false;
	}
}

bdBool bdECCKey::create()
{
	bdInt error;
	prng_state prng;

	if (register_prng(&yarrow_desc) == -1)
	{
		return false;
	}

	const bdUInt size = 128;
	bdUByte8 random[size];
	bdTrulyRandom::getInstance()->getRandomUByte8(random, size);

	if ((error = yarrow_start(&prng)) != CRYPT_OK)
	{
		BD_ERR(BD_LOG_LEVEL, "Start error %s.", error_to_string(error));
		return false;
	}

	if ((error = yarrow_add_entropy(random, size, &prng)) != CRYPT_OK)
	{
		BD_ERR(BD_LOG_LEVEL, "Add_entropy error %s.", error_to_string(error));
		return false;
	}

	if ((error = yarrow_ready(&prng)) != CRYPT_OK)
	{
		BD_ERR(BD_LOG_LEVEL, "Ready error %s.", error_to_string(error));
		return false;
	}

	if ((error = ecc_make_key(&prng, find_prng("yarrow"), BD_ECC_KEY_SIZE, &m_key)) != CRYPT_OK)
	{
		BD_ASSERT(g_assertFalse, "Unable to create private key.");
		return false;
	}

	m_madeKey = true;

	return true;
}

bdBool bdECCKey::importKey(const bdUByte8 *const pubKey, bdUInt keylen)
{
	const bdInt error = ecc_import(pubKey, keylen, &m_key);

	if (error != CRYPT_OK)
	{
		BD_WARN(BD_LOG_LEVEL, "Unable to import public key.");
		return false;
	}

	m_madeKey = true;

	return true;
}

bdBool bdECCKey::generateSecret(bdECCKey &other, bdUByte8 secret[BD_ECC_SHARED_SECRET_SIZE])
{
	unsigned long tmpSize = BD_ECC_SHARED_SECRET_SIZE;

	const bdInt error = ecc_shared_secret(&m_key, &(other.m_key), secret, &tmpSize);
	if (error != CRYPT_OK)
	{
		BD_WARN(BD_LOG_LEVEL, "Unable to generate secret key. Error: %s", error_to_string(error));
		return false;
	}

	// if the shared secret is less than the buffer size just fill the rest with
	// zeros.
	if(tmpSize < BD_ECC_SHARED_SECRET_SIZE)
	{
		bdMemset(secret + tmpSize, 0, BD_ECC_SHARED_SECRET_SIZE - tmpSize);
	}

	BD_ASSERT(tmpSize <= BD_ECC_SHARED_SECRET_SIZE, 
		"Shared secret wrong size.");

	return true;
}

bdBool bdECCKey::exportKey(bdUByte8 *pubKey, bdUInt &keylen)
{
	unsigned long tmpKeylen = keylen;
	bdInt error = ecc_export(pubKey, &tmpKeylen, PK_PUBLIC, &m_key);
	if (error != CRYPT_OK)
	{
		BD_ASSERT(g_assertFalse, "Unable to export public key.");
		return false;
	}
	keylen = tmpKeylen;

	if (keylen < BD_ECC_EXPORTED_KEY_SIZE)
	{
		bdMemset(pubKey + keylen, 0, BD_ECC_EXPORTED_KEY_SIZE - keylen);
	}

	BD_ASSERT(keylen <= BD_ECC_EXPORTED_KEY_SIZE, "Wrong length!");

	keylen = BD_ECC_EXPORTED_KEY_SIZE;

	return true;

}
