// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdCore/bdLog/bdLog.h>
#include <bdCore/bdLog/bdLogChannel.h>

BD_REGISTER_SINGLETON(bdLogImpl)

bdBool g_assertFalse = false;

void bdLogImpl::subscribe(const bdNChar8 *channelName, 
						  bdLogSubscriber *const subscriber)
{
	if(!m_root)
	{
		setRoot(BD_LOG_ROOT);
	}

	if(bdStrstr(channelName, BD_LOG_ROOT) != channelName)
	{
		m_root->addSubscriber(channelName, channelName, subscriber);
	}
}

void bdLogImpl::unsubscribe(const bdNChar8 *channelName, 
							bdLogSubscriber *const subscriber)
{
	if(bdStrstr(channelName, BD_LOG_ROOT) != channelName)
	{
		m_root->removeSubscriber(channelName, channelName, subscriber);
	}
}

void bdLogImpl::unsubscribeAll(bdLogSubscriber *const subscriber)
{
	m_root->removeSubscriber(subscriber);
}

// protected:
void bdLogImpl::setRoot(const bdNChar8 *const channelName)
{
	delete m_root;
	m_root = new bdLogChannel(channelName);
}

void bdLogImpl::log(const bdNChar8 *const file, 
					const bdNChar8 *const function, 
					const bdUInt line, 
					const bdNChar8 *const channelName, 
					const bdNChar8 *const msg)
{
	if(!m_root)
	{
		setRoot(BD_LOG_ROOT);
	}

	const bdUWord len = strlen(BD_LOG_ROOT);

	m_root->log(file, function, line, channelName + len + 1, msg);
}


bdLogImpl::bdLogImpl()
: m_root(BD_NULL)
{
}

bdLogImpl::~bdLogImpl()
{
	delete m_root;
}

bdMessageProxy::bdMessageProxy(const bdNChar8 *const file, 
							   const bdNChar8 *const function, 
							   const bdUInt line, 
							   const bdNChar8 *const baseChannel)
: m_file(file),
  m_function(function),
  m_line(line),
  m_baseChannel(baseChannel)
{
};

void bdMessageProxy::log(const bdNChar8 *const channelName, 
						 const bdNChar8 *const format, 
						 ...) const
{
	bdNChar8 fullChannelName[150];
	bdStrlcpy(fullChannelName, m_baseChannel, 150);
	bdStrlcat(fullChannelName, channelName, 150);
	
	const bdUInt bufferSize = 1024;
	bdNChar8 buffer[bufferSize];

	bdVaList args;
	bdVaStart(args, format);

	bdVsnprintf(buffer, bufferSize, format, args);

	bdVaEnd(args);

	bdLog::getInstance()->log(m_file, m_function, m_line, fullChannelName, buffer);			
}
