// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#ifndef BD_LOG_H
#define BD_LOG_H

#include <bdCore/bdMemory/bdMemory.h>
#include <bdCore/bdLog/bdLogConfig.h>

class bdMessageProxy
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS;		

		bdMessageProxy(const bdNChar8 *const file, 
			const bdNChar8* const function, 
			const bdUInt line, 
			const bdNChar8 *const baseChannel);

		void log(const bdNChar8 *const channelName, 
				 const bdNChar8 *const format, 
			...) const;
	public:

		const bdNChar8 *m_file;
		const bdNChar8 *m_function;
		bdUInt m_line;
		const bdNChar8 *m_baseChannel;
};

extern bdBool g_assertFalse;

#if defined BD_COMPILER_MSVC7 || defined BD_COMPILER_MSVC8
#	define BD_BREAK() DebugBreak()
#else
#	define BD_BREAK() bdAbort()
#endif

#ifdef __FUNCSIG__
#	define BD_FUNCSIG __FUNCSIG__
#else
#	define BD_FUNCSIG ""
#endif // __FUNCSIG__

#define BD_INFO \
	bdMessageProxy(__FILE__, BD_FUNCSIG, __LINE__, BD_LOG_ROOT "/" BD_LOG_INFO "/").log

#define BD_WARN \
	bdMessageProxy(__FILE__, BD_FUNCSIG, __LINE__, BD_LOG_ROOT "/" BD_LOG_WARN "/").log

#define BD_ERR \
	bdMessageProxy(__FILE__ ,BD_FUNCSIG, __LINE__, BD_LOG_ROOT "/" BD_LOG_ERR "/").log

#ifdef BD_THROW_ASSERTS
#define BD_ASSERT(a, m)																				\
	do																								\
	{																								\
		if(!(a))																					\
		{																							\
			bdMessageProxy(__FILE__, BD_FUNCSIG, __LINE__, BD_LOG_ROOT "/" BD_LOG_ERR).log("", m);	\
			BD_BREAK();																				\
		}																							\
	} while (g_assertFalse)
#else // BD_THROW_ASSERTS
#define BD_ASSERT(a, m)																				\
	do																								\
	{																								\
		if(!(a))																					\
		{																							\
			bdMessageProxy(__FILE__, BD_FUNCSIG, __LINE__, BD_LOG_ROOT "/" BD_LOG_ERR).log("", m);	\
		}																							\
	} while (g_assertFalse)
#endif // BD_THROW_ASSERTS


#include <bdPlatform/bdPlatformError/bdPlatformError.h>
#include <bdCore/bdContainers/bdSingleton.h>

class bdLogSubscriber;
class bdLogChannel;

class bdLogImpl
{
	public:		

		BD_DECLARE_NEW_AND_DELETE_OPERATORS;

		void subscribe(const bdNChar8 *channelName, 
					   bdLogSubscriber *const subscriber);

		void unsubscribe(const bdNChar8 *channelName, 
						 bdLogSubscriber *const subscriber);

		void unsubscribeAll(bdLogSubscriber *const subscriber);

	protected:

		/// bdSingleton<bdErrorImpl> is granted friendship so that it can 
		/// construct an instances of this class.
		friend class bdSingleton<bdLogImpl>;

		friend class bdMessageProxy;

		void setRoot(const bdNChar8 *const channelName);

		void log(const bdNChar8 *const file, 
				 const bdNChar8 *const function, 
				 const bdUInt line, 
				 const bdNChar8 *const channelName, 
				 const bdNChar8 *const msg);

		/// Default constructor.
		/// This is declared as protected so that non-singleton instance of 
		/// this class cannot be declared.
		bdLogImpl();

		/// Destructor.
		virtual ~bdLogImpl();

	protected:

		bdLogChannel *m_root;
};

/// This class provides a point of access to the singleton instance.
/// Implementation details can be found in bdErrorImpl.
class bdLog : public bdSingleton<bdLogImpl>
{
	BD_DECLARE_NEW_AND_DELETE_OPERATORS
		protected:
			bdLog();
};

#endif // BD_LOG_H
