// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdCore/bdLog/bdLogSubscriber.h>

bdLogSubscriber::bdLogSubscriber()
{
#ifdef BD_PLATFORM_UNIX
	bdMemset(m_processName, 0, sizeof(m_processName));
#endif // BD_PLATFORM_UNIX
}
 
bdLogSubscriber::~bdLogSubscriber()
{
}

void bdLogSubscriber::publish(const bdNChar8 *const fullChannelName, 
							  const bdNChar8 *const file, 
							  const bdNChar8 *const function, 
							  const bdUInt line, 
							  const bdNChar8 *const msg) 
{
	const bdNChar8 *pos = bdStrrchr(file, '\\');
	bdUWord fileOffset = 0;

#if defined BD_PLATFORM_UNIX
	bdUWord lenProcName = bdStrlen(m_processName);

	if (lenProcName > 0)
	{
		openlog(m_processName,BD_LOG_UNIXSYSLOG_FLAGS,BD_LOG_UNIXSYSLOG_FACILITY);
	}
	else
	{
		openlog("bdLog",BD_LOG_UNIXSYSLOG_FLAGS,BD_LOG_UNIXSYSLOG_FACILITY);
	}
#endif
	
	if(pos)
	{
		fileOffset = static_cast<bdUWord>(pos - file) + 1;
	}

	if(bdStrstr(fullChannelName, BD_LOG_INFO) == fullChannelName)
	{
#if defined BD_PLATFORM_XBOX
		bdNChar8 string[2048];
		bdSnprintf(string, sizeof(string), "%s(%u): %s\n\tINFO: %s\n", file + fileOffset, line, function, msg);		
		OutputDebugString(string);
#elif defined BD_PLATFORM_UNIX
		syslog(LOG_INFO,"%s(%u): INFO:%s", file + fileOffset, line, msg);
#else
		bdPrintf("%s(%u): %s\n", file + fileOffset, line, msg);
#endif
	}
	else if(bdStrstr(fullChannelName, BD_LOG_WARN) == fullChannelName)
	{
#if defined BD_PLATFORM_XBOX
		bdNChar8 string[2048];		
		bdSnprintf(string, sizeof(string), "%s(%u): %s\n\tWARNING: %s\n", file + fileOffset, line, function, msg);
		OutputDebugString(string);
#elif defined BD_PLATFORM_UNIX		
		syslog(LOG_WARNING,"%s(%u): %sWARNING: %s\n", file + fileOffset, line, function, msg);
#else
		bdPrintf("%s(%u): %s\n\tWARNING: %s\n", file + fileOffset, line, function, msg);
#endif
	}
	else if(bdStrstr(fullChannelName, BD_LOG_ERR) == fullChannelName)
	{
#if defined BD_PLATFORM_XBOX
		bdNChar8 string[2048];
		bdSnprintf(string, sizeof(string), "%s(%u): %s\n\tERROR: %s\n", file + fileOffset, line, function, msg);
		OutputDebugString(string);
#elif defined BD_PLATFORM_UNIX		
		syslog(LOG_ERR, "%s(%u): %sERROR: %s\n", file + fileOffset, line, function, msg);
#else
		bdPrintf("%s(%u): %s\n\tERROR: %s\n", file + fileOffset, line, function, msg);
#endif
	}
	else
	{
#if defined BD_PLATFORM_XBOX
		OutputDebugString("bdLogSubscriber::publish: invalid channel name!\n");
#else
		bdPrintf("bdLogSubscriber::publish: invalid channel name!\n");
#endif
		BD_BREAK();
	}

#if defined BD_PLATFORM_UNIX
	closelog();
#endif

}

#ifdef BD_PLATFORM_UNIX
void bdLogSubscriber::setProcessName(const bdNChar8 *const processName)
{
	bdStrlcpy(m_processName, processName, sizeof(m_processName));
	// bdPrintf("%s %s\n","m_processName->",m_processName);

}
#endif // BD_PLATFORM_UNIX
