// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdPlatform/bdPlatformMemory/bdPlatformMemory.h>
#include <bdCore/bdLog/bdLog.h>
#include <bdCore/bdMemory/bdAlignedOffsetMemory.h>


inline 
void* BD_CALL bdMallocMemory::allocate(const bdUWord size)
{
	const bdUWord totalSize = size + sizeof(bdMemoryChainElement);

	// use 8 byte alignment even for basic malloc
	bdMemoryChainElement *link =  reinterpret_cast<bdMemoryChainElement*>(bdAlignedOffsetMalloc(totalSize, 8, sizeof(bdMemoryChainElement)));

	return recordMemory(link, size, false);
}

inline 
void BD_CALL bdMallocMemory::deallocate(void *p)
{
	if(p)
	{
		bdMemoryChainElement *link =  reinterpret_cast<bdMemoryChainElement*>(p) - 1;

		BD_ASSERT(link->m_aligned == false, "Memory block allocated as aligned "
											"but is beign deallocated with bdMallocMemory::deallocate. "
											"Use bdMallocMemory::alignedDeallocate instead.");

		eraseMemory(link);
		bdAlignedOffsetFree(link);
	}
}

inline 
void* BD_CALL bdMallocMemory::reallocate(void *p,
										 const bdUWord size)
{	
	if(p)
	{
		bdMemoryChainElement *link =  reinterpret_cast<bdMemoryChainElement*>(p) - 1;
		const bdUWord origSize = link->m_size + sizeof(bdMemoryChainElement);
		eraseMemory(link);

		const bdUWord totalSize = size + sizeof(bdMemoryChainElement);

		// use 8 byte alignment even for basic realloc
		link = reinterpret_cast<bdMemoryChainElement*>(bdAlignedOffsetRealloc(link, origSize, totalSize, 8, sizeof(bdMemoryChainElement)));

		return recordMemory(link, size, false);
	}
	else
	{
		return allocate(size);
	}
}

inline 
void* BD_CALL bdMallocMemory::alignedAllocate(const bdUWord size, 
											  const bdUWord align)
{
	const bdUWord totalSize = size + sizeof(bdMemoryChainElement);

	// use bdAlignedOffsetMalloc here to ensure the returned pointer is
	// correctly aligned rather than the address of the link.
	bdMemoryChainElement *link =  reinterpret_cast<bdMemoryChainElement*>(bdAlignedOffsetMalloc(totalSize, align, sizeof(bdMemoryChainElement)));

	return recordMemory(link, size, true);
}

inline 
void BD_CALL bdMallocMemory::alignedDeallocate(void *p)
{
	if(p)
	{
		bdMemoryChainElement *link =  reinterpret_cast<bdMemoryChainElement*>(p) - 1;
		
		BD_ASSERT(link->m_aligned == true, "Memory block allocated unaligned "
										   "but is beign deallocated with bdMallocMemory::alignedDeallocate. "
										   "Use bdMallocMemory::deallocate instead.");

		eraseMemory(link);

		bdAlignedOffsetFree(link);
	}
}

inline 
void* BD_CALL bdMallocMemory::alignedReallocate(void *p,
												const bdUWord size,
												const bdUWord align)
{
	if(p)
	{
		bdMemoryChainElement *link =  reinterpret_cast<bdMemoryChainElement*>(p) - 1;
		const bdUWord origSize = link->m_size + sizeof(bdMemoryChainElement);
		eraseMemory(link);

		const bdUWord totalSize = size + sizeof(bdMemoryChainElement);

		link = reinterpret_cast<bdMemoryChainElement*>(bdAlignedOffsetRealloc(link, origSize, totalSize, align, sizeof(bdMemoryChainElement)));

		return recordMemory(link, size, true);
	}
	else
	{
		return alignedAllocate(size, align);
	}
}

