// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

// PURPOSE: A platform independent timing class interface.

#ifndef BD_SHORT_TIMER_H
#define BD_SHORT_TIMER_H

#include <bdCore/bdMemory/bdMemory.h>

/// A platform independent timing class interface.
/// A class that times the duration of operations and events.
/// \note This timer is small (only contains a 32 bit integer) but can
/// only record short periods of time. It is guaranteed to be able 
/// to record a time period of up to 10 seconds.
class bdShortTimer
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		/// Default constructor
		bdShortTimer();

		/// Start the stopwatch running.
		/// The current timings are not cleared before starting the timer.
		/// To clear the current timings call reset().
		void start();

		/// Clear the current timings.
		/// The time values stored in the stopwatch are reset to zero.
		void reset();

		/// Get the elapsed time between the call to start() and now. 
		/// \return		A float in the form of 'a.b' where 'a' is the second
		///				count and 'b' is the fraction of the second.
		bdFloat32 getElapsedTimeInSeconds() const;

	protected:

		/// start time.
		bdUInt32 m_start;
};

#endif // BD_SHORT_TIMER_H
