// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include "Simulation.h"
#include <bdLobby/bdKeyArchive/bdKeyArchive.h>

#define TITLE_ID (0x12F7)

#define USERNAME "keyarchiveuser"
#define PASSWORD "kauserpass"
#define USERID (7241693198847328901ULL)
#define INITIAL_GAME_TYPE (1)
#define CHANGED_GAME_TYPE (2)

Simulation::Simulation()
: m_taskStatus(READY),
  m_task(START_NETWORK),
  m_changedScreen(true),
  m_authGetHostByName(BD_NULL),
  m_authService(BD_NULL),
  m_lsgGetHostByName(BD_NULL),
  m_remoteTask(BD_NULL),
  m_pumpLobbyConnection(false),
  m_lobbyErrorCode(BD_NO_ERROR)
{
}

Simulation::~Simulation()
{
}

bdBool Simulation::init(bdInt32 ,
						bdNChar8* [])
{
	bdBool ok = true;

	// start logging
	// we want to hear about all warnings and errors
	bdLogImpl *log = bdLog::getInstance();
	log->subscribe("warn", &m_logSubscriber);
	log->subscribe("err", &m_logSubscriber);
	log->subscribe("info", &m_logSubscriber);

	return ok;
}

bdBool Simulation::step(const bdFloat32)
{
	const bdBool quitKey = getScreen().wasKeyPressed(BD_QUIT);

	TaskStatus oldTaskStatus = m_taskStatus;

	if(m_taskStatus == READY)
	{
		startTask();
	}

	if(m_taskStatus == PENDING || m_taskStatus == DONE || m_taskStatus == FAILED)
	{
		// pump even if the status is DONE or FAILED to read out any results
		// and close any remote task handles
		pumpTask();
	}

	if(m_taskStatus == DONE || m_taskStatus == FAILED || m_taskStatus == SKIPPED)
	{
		// incase the task fails or completes in a single loop
		if(oldTaskStatus == READY)
		{
			oldTaskStatus = m_taskStatus;
		}

		displayTaskResult();
	}

	m_changedScreen = (oldTaskStatus != m_taskStatus);

	if(m_pumpLobbyConnection)
	{
		bdBool lostConnection = true;
		bdLobbyServiceImpl *const lobbyService = bdLobbyService::getInstance();
		if(lobbyService)
		{
			const bdLobbyConnection::Status status = lobbyService->getStatus();
			switch(status)
			{
				case bdConnection::BD_NOT_CONNECTED:
				case bdConnection::BD_CONNECTING:
				case bdConnection::BD_DISCONNECTING:
				case bdConnection::BD_DISCONNECTED:
				{
					lostConnection = true;
				}
				break;
				case bdConnection::BD_CONNECTED:
				{
					lostConnection = false;
				}
				break;
			}
		}

		if(lostConnection)
		{
			// set up for a reconnect
			m_pumpLobbyConnection = false;
			m_task = STOP_NETWORK;
			m_taskStatus = READY;
			m_changedScreen = true;
			CloseRemoteTask(&m_remoteTask);
			m_lastTaskDescription = "Lost connection to lobby.";
		}
	}

	getScreen().printToScreen(m_screenText);

	if(m_changedScreen)
	{
		m_screenText = "";
	}

	return !quitKey;
}

void Simulation::quit()
{
	bdLogImpl *log = bdLog::getInstance();
	log->unsubscribe("warn", &m_logSubscriber);
	log->unsubscribe("err", &m_logSubscriber);
	log->unsubscribe("info", &m_logSubscriber);
	
	stopNetwork();

	delete m_authGetHostByName;
	delete m_authService;
	delete m_lsgGetHostByName;
	
	if(m_remoteTask)
	{
		m_remoteTask->close();
	}
}

void Simulation::startTask()
{	
	const bdBool skipKey = getScreen().wasKeyPressed(SKIP_KEY);

	if (skipKey)
	{
		m_taskStatus = SKIPPED;
	}
	else
	{
		const bdBool startKey = getScreen().wasKeyPressed(START_KEY);
		bdString taskDescription("Unknown task");

		switch(m_task)
		{
			case START_NETWORK:
			{
				startNetworkDescription(taskDescription);
				if(startKey)
				{
					startNetwork();
				}
			}
			break;
			case CHECK_DNS:
			{
				checkDNSDescription(taskDescription);
				if(startKey)
				{
					startCheckDNS();
				}
			}
			break;
			case CREATE_ACCOUNT:
			{
				createAccountDescription(taskDescription);
				if(startKey)
				{
					startCreateAccount();
				}
			}
			break;
			case AUTHENTICATE_ACCOUNT:
			{
				authenticateAccountDescription(taskDescription);
				if(startKey)
				{
					startAuthenticateAccount();
				}
			}
			break;
			case LOBBY_CONNECT:
			{
				lobbyServiceConnectDescription(taskDescription);
				if(startKey)
				{
					startLobbyServiceConnect();
				}
			}
			break;
			
	
			case READ_PAIRS:
			{
				readKeyPairsDescription(taskDescription);
				if(startKey)
				{
					startReadKeyPairs();
				}
			}
			break;
	
			case READ_ALL_PAIRS:
			{
				readAllKeyPairsDescription(taskDescription);
				if(startKey)
				{
					startReadAllKeyPairs();
				}
			}
			break;

			case USER_WRITE_PAIRS:
			{
				writeKeyPairsDescription(taskDescription);
				if(startKey)
				{
					startWriteKeyPairs();
				}
			}
			break;
			case USER_DELETE_PAIRS:
			{
				deleteKeyPairsDescription(taskDescription);
				if(startKey)
				{
					startDeleteKeyPairs();
				}
			}
			break;
			case USER_DELETE_ALL_USERS_PAIRS:
			{
				deleteUsersKeyPairsDescription(taskDescription);
				if(startKey)
				{
					startDeleteUsersKeyPairs();
				}
			}
			break;

			case LOBBY_DISCONNECT:
			{
				lobbyServiceDisconnectDescription(taskDescription);
				if(startKey)
				{
					startLobbyServiceDisconnect();
				}
			}
			break;
			case STOP_NETWORK:
			{
				stopNetworkDescription(taskDescription);
				if(startKey)
				{
					stopNetwork();
				}
			}
			break;
		}
		
		if(m_changedScreen)
		{
			m_screenText += taskDescription;
			m_screenText += "\n" START_TEXT;
			m_screenText += "\n" SKIP_TEXT;

			if(m_lastTaskDescription.getLength())
			{
				m_screenText += "\n\n";
				m_screenText += m_lastTaskDescription;				
			}
		}
	}
}

void Simulation::pumpTask()
{
	bdString taskDescription("Unknown task");

	switch(m_task)
	{
		case START_NETWORK:
		{
			startNetworkDescription(taskDescription);
			pumpStartNetwork();
		}
		break;
		case CHECK_DNS:
		{
			checkDNSDescription(taskDescription);
			pumpCheckDNS();
		}
		break;
		case CREATE_ACCOUNT:
		{
			createAccountDescription(taskDescription);
			pumpCreateAccount();
		}
		break;
		case AUTHENTICATE_ACCOUNT:
		{
			authenticateAccountDescription(taskDescription);
			pumpAuthenticateAccount();
		}
		break;
		case LOBBY_CONNECT:
		{
			lobbyServiceConnectDescription(taskDescription);
			pumpLobbyServiceConnect();
		}
		break;
		
		
		case READ_PAIRS:
		{
			readKeyPairsDescription(taskDescription);
			pumpReadKeyPairs();
		}
		break;		
	
		case READ_ALL_PAIRS:
		{
			readAllKeyPairsDescription(taskDescription);
			pumpReadAllKeyPairs();
		}
		break;
		
		case USER_WRITE_PAIRS:
		{
			writeKeyPairsDescription(taskDescription);
			pumpReadAllKeyPairs();
		}
		break;
		case USER_DELETE_PAIRS:
		{
			deleteKeyPairsDescription(taskDescription);
			pumpDeleteKeyPairs();
		}
		break;
		case USER_DELETE_ALL_USERS_PAIRS:
		{
			deleteUsersKeyPairsDescription(taskDescription);
			pumpDeleteUsersKeyPairs();
		}
		break;

		case LOBBY_DISCONNECT:
		{
			lobbyServiceDisconnectDescription(taskDescription);
			pumpLobbyServiceDisconnect();
		}
		break;
		case STOP_NETWORK:
		{
			// nothing to to here
		}
		break;
	}

	if(m_changedScreen)
	{
		m_screenText += taskDescription;
	}
}

void Simulation::displayTaskResult()
{
	bdString taskDescription("Unknown task result");

	Task nextTaskIfDone = m_task;
	Task nextTaskIfFailed = m_task;
	Task nextTaskIfSkipped = m_task;

	bdBool displayLobbyError = true;

	switch(m_task)
	{
		case START_NETWORK:
		{
			startNetworkDescription(m_lastTaskDescription);
			nextTaskIfDone = CHECK_DNS;
			nextTaskIfFailed = START_NETWORK;
			nextTaskIfSkipped = START_NETWORK;
			displayLobbyError = false;
		}
		break;
		case CHECK_DNS:
		{
			checkDNSDescription(m_lastTaskDescription);
			nextTaskIfDone = CREATE_ACCOUNT;
			nextTaskIfFailed = CHECK_DNS;
			nextTaskIfSkipped = CREATE_ACCOUNT;
			displayLobbyError = false;
		}
		break;
		case CREATE_ACCOUNT:
		{
			createAccountDescription(m_lastTaskDescription);
			checkAuthServiceError();
			nextTaskIfDone = AUTHENTICATE_ACCOUNT;
			nextTaskIfFailed = CREATE_ACCOUNT;
			nextTaskIfSkipped = AUTHENTICATE_ACCOUNT;			
		}
		break;
		case AUTHENTICATE_ACCOUNT:
		{
			authenticateAccountDescription(m_lastTaskDescription);
			checkAuthServiceError();
			nextTaskIfDone = LOBBY_CONNECT;
			nextTaskIfFailed = AUTHENTICATE_ACCOUNT;
			nextTaskIfSkipped = LOBBY_CONNECT;			
		}
		break;		
		case LOBBY_CONNECT:
		{
			lobbyServiceConnectDescription(m_lastTaskDescription);
			nextTaskIfDone = USER_WRITE_PAIRS;
			nextTaskIfFailed = LOBBY_CONNECT;
			nextTaskIfSkipped = USER_WRITE_PAIRS;
		}
		break;
	
		case USER_WRITE_PAIRS:
		{
			writeKeyPairsDescription(m_lastTaskDescription);
			checkKeyArchiveError();
			nextTaskIfDone = READ_PAIRS;
			nextTaskIfFailed = READ_PAIRS;
			nextTaskIfSkipped = READ_PAIRS;
		}
		break;
		case READ_PAIRS:
		{
			readKeyPairsDescription(m_lastTaskDescription);
			checkKeyArchiveError();
			nextTaskIfDone = READ_ALL_PAIRS;
			nextTaskIfFailed = READ_PAIRS;
			nextTaskIfSkipped = READ_ALL_PAIRS;
		}
		break;
		case READ_ALL_PAIRS:
		{
			readAllKeyPairsDescription(m_lastTaskDescription);
			checkKeyArchiveError();
			nextTaskIfDone = USER_DELETE_PAIRS;
			nextTaskIfFailed = READ_ALL_PAIRS;
			nextTaskIfSkipped = USER_DELETE_PAIRS;
		}
		break;
	
		
		case USER_DELETE_PAIRS:
		{
			deleteKeyPairsDescription(m_lastTaskDescription);
			checkKeyArchiveError();
			nextTaskIfDone = USER_DELETE_ALL_USERS_PAIRS;
			nextTaskIfFailed = USER_DELETE_PAIRS;
			nextTaskIfSkipped = USER_DELETE_ALL_USERS_PAIRS;
		}
		break;
		case USER_DELETE_ALL_USERS_PAIRS:
		{
			deleteUsersKeyPairsDescription(m_lastTaskDescription);
			checkKeyArchiveError();
			nextTaskIfDone = LOBBY_DISCONNECT;
			nextTaskIfFailed = USER_DELETE_ALL_USERS_PAIRS;
			nextTaskIfSkipped = LOBBY_DISCONNECT;
		}
		break;
		

		case LOBBY_DISCONNECT:
		{
			lobbyServiceDisconnectDescription(m_lastTaskDescription);			
			nextTaskIfDone = STOP_NETWORK;
			nextTaskIfFailed = LOBBY_DISCONNECT;
			nextTaskIfSkipped = STOP_NETWORK;
		}
		break;
		case STOP_NETWORK:
		{
			stopNetworkDescription(m_lastTaskDescription);
			nextTaskIfDone = START_NETWORK;
			nextTaskIfFailed = STOP_NETWORK;
			nextTaskIfSkipped = CREATE_ACCOUNT;
		}
		break;
	}

	switch(m_taskStatus)
	{
		case READY:
		case PENDING:
		{
			BD_WARN(BD_LOG_LEVEL, "Unexpected task status.");
		}
		break;
		case DONE:
		{
			m_task = nextTaskIfDone;
		}
		break;
		case FAILED:
		{
			m_task = nextTaskIfFailed;
		}
		break;
		case SKIPPED:
		{
			m_task = nextTaskIfSkipped;
		}
		break;
	}

	// if task uses sets a lobby error code and has failed
	// display the error description
	if(displayLobbyError && (m_taskStatus == FAILED))
	{
		bdNChar8 buf[32];
		LobbyErrorCodeToString(m_lobbyErrorCode, buf, sizeof(buf));
		m_lastTaskDescription += "\nError: ";
		m_lastTaskDescription += buf;
		m_lobbyErrorCode = BD_NO_ERROR;
	}

	m_taskStatus = READY;
}

void Simulation::startNetworkDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"Start Network",				// READY
											"Starting Network",				// PENDING
											"Started Network",				// DONE
											"Starting Network Failed",		// FAILED
											"Starting Network Skipped"};	// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);
}

void Simulation::startNetwork()
{
	bdNetImpl *const net = bdNet::getInstance();
	bdNetStartParams params;
	params.m_onlineGame = false;

	// platform specific startup parameters
	bdPlatformParams platformParams;
	//TODO: check the return value here
	bdExamplePlatformInit::init(params, &platformParams);
	
	if(net->start(params))
	{
		m_taskStatus = PENDING;
	}
	else
	{
		m_taskStatus = FAILED;
	}
}

void Simulation::pumpStartNetwork()
{
	bdNetImpl *const net = bdNet::getInstance();
	net->pump();

	const bdNetImpl::bdNetStatus networkStatus = net->getStatus();
	switch(networkStatus)
	{
		case bdNetImpl::BD_NET_BIND_FAILED:
		case bdNetImpl::BD_NET_ONLINE_FAILED:
		case bdNetImpl::BD_NET_INIT_FAILED:
		case bdNetImpl::BD_NET_STOPPED:
		{
			// error conditions
			m_taskStatus = FAILED;
		}
		break;
		case bdNetImpl::BD_NET_PENDING:
		{
			// wait
			m_taskStatus = PENDING;
		}
		break;
		case bdNetImpl::BD_NET_DONE:
		{
			// complete
			m_taskStatus = DONE;
		}
	}
}

void Simulation::checkDNSDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"Start DNS Lookup",			// READY
											"Performing DNS Lookup",	// PENDING
											"DNS Lookup Succeeded",		// DONE
											"DNS Lookup Failed",		// FAILED
											"DNS Lookup Skipped"};	// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);

	if(m_taskStatus == DONE)
	{
		bdNChar8 addrString[BD_INET_ADDR_STRING_SIZE];
		m_authAddr.toString(addrString, sizeof(addrString));
		description +=  "\n " AUTH_ADDRESS ":";
		description += addrString;
		description +=  "\n";

		m_lsgAddr.toString(addrString, sizeof(addrString));
		description +=  " " LSG_ADDRESS ":";
		description += addrString;
	}

}		

void Simulation::startCheckDNS()
{
	const TaskStatus authDNSStatus = StartGetHostByName(AUTH_ADDRESS, &m_authGetHostByName, m_authAddr);
	const TaskStatus lsgDNSStatus = StartGetHostByName(LSG_ADDRESS, &m_lsgGetHostByName, m_lsgAddr);
	if((authDNSStatus == FAILED) || (lsgDNSStatus == FAILED))
	{
		m_taskStatus = FAILED;
	}

	if((authDNSStatus == PENDING) || (lsgDNSStatus == PENDING))
	{
		m_taskStatus = PENDING;
	}

	if((authDNSStatus == DONE) && (lsgDNSStatus == DONE))
	{
		m_taskStatus = DONE;
	}
}

void Simulation::pumpCheckDNS()
{
	TaskStatus authDNSStatus = DONE;
	TaskStatus lsgDNSStatus = DONE;

	if(m_authGetHostByName)
	{
		authDNSStatus = PumpGetHostByName(&m_authGetHostByName, m_authAddr);
	}

	if(m_lsgGetHostByName)
	{
		lsgDNSStatus = PumpGetHostByName(&m_lsgGetHostByName, m_lsgAddr);
	}

	if((authDNSStatus == FAILED) || (lsgDNSStatus == FAILED))
	{
		m_taskStatus = FAILED;
	}

	if((authDNSStatus == PENDING) || (lsgDNSStatus == PENDING))
	{
		m_taskStatus = PENDING;
	}

	if((authDNSStatus == DONE) && (lsgDNSStatus == DONE))
	{
		m_taskStatus = DONE;
	}
}

bdAuthService* Simulation::getAuthService()
{
	if(m_authService == BD_NULL)
	{
		const bdAddr authAddr(m_authAddr, AUTH_PORT);
		m_authService = new bdAuthService(TITLE_ID, authAddr);
	}

	return m_authService;
}


const bdAuthInfo *Simulation::getAuthInfo()
{
	if(m_authService != BD_NULL)
	{
		return m_authService->getAuthInfo();
	}

	return BD_NULL;
}


void Simulation::checkAuthServiceError()
{
	bdAuthService *const authService = getAuthService();
	if(authService)
	{
		m_lobbyErrorCode = authService->getErrorCode();
	}
}

void Simulation::pumpAuthServiceTask()
{
	// fail safe
	m_taskStatus = FAILED;

	bdAuthService *const authService = getAuthService();
	if(authService)
	{
		m_taskStatus = PENDING;
		const bdAuthService::bdStatus status = authService->getStatus();
		if(status == bdAuthService::BD_READY)
		{			
			const bdLobbyErrorCode lobbyErrorCode = authService->getErrorCode();
			if(lobbyErrorCode == BD_AUTH_NO_ERROR)
			{
				m_taskStatus = DONE;
			}
			else
			{
				m_taskStatus = FAILED;
			}
		}
	}
}

void Simulation::createAccountDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"Create Account (" USERNAME "/" PASSWORD ")",	// READY
											"Creating Account",								// PENDING
											"Created Account",								// DONE
											"Create Account Failed",						// FAILED
											"Create Account Skipped"};						// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);
}

void Simulation::startCreateAccount()
{
	// fail safe
	m_taskStatus = FAILED;

	bdAuthService *const authService = getAuthService();
	if(authService)
	{
		if(authService->createAccount(USERNAME, PASSWORD))
		{
			m_taskStatus = PENDING;
		}
	}
}

void Simulation::pumpCreateAccount()
{
	pumpAuthServiceTask();
}

void Simulation::authenticateAccountDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"Authenticate Account (" USERNAME "/" PASSWORD ")",	// READY
											"Authenticating Account",									// PENDING
											"Authenticated Account",									// DONE
											"Authenticate Account Failed",								// FAILED
											"Authenticate Account Skipped"};							// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);
}

void Simulation::startAuthenticateAccount()
{
	// fail safe
	m_taskStatus = FAILED;

	bdAuthService *const authService = getAuthService();
	if(authService)
	{
		if(authService->authorizeAccount(USERNAME, PASSWORD))
		{
			m_taskStatus = PENDING;
		}
	}
}

void Simulation::pumpAuthenticateAccount()
{
	pumpAuthServiceTask();
}

void Simulation::lobbyServiceConnectDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"Connect to Lobby Service",				// READY
											"Connecting to Lobby Service",			// PENDING
											"Connected to Lobby Service",			// DONE
											"Connect to Lobby Service Failed",		// FAILED
											"Connect to Lobby Service Skipped"};	// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);
}

void Simulation::startLobbyServiceConnect()
{
	// fail safe
	m_taskStatus = FAILED;

	bdLobbyServiceImpl *const lobbyService = bdLobbyService::getInstance();
	if(lobbyService)
	{
		const bdAddr lsgAddr(m_lsgAddr, LSG_PORT);
		const bdBool connecting = lobbyService->connect(lsgAddr, getAuthInfo());
		if(connecting)
		{
			m_taskStatus = PENDING;
		}
		// no need to keep the bdAuthService instance after we are connected to the lobby
		delete m_authService;
		m_authService = BD_NULL;
	}
}

void Simulation::pumpLobbyServiceConnect()
{
	// fail safe
	m_taskStatus = FAILED;

	m_pumpLobbyConnection = false;

	bdLobbyServiceImpl *const lobbyService = bdLobbyService::getInstance();
	if(lobbyService)
	{
		const bdLobbyConnection::Status connectionStatus = lobbyService->getStatus();
		switch(connectionStatus)
		{			
			case bdConnection::BD_NOT_CONNECTED:
			case bdConnection::BD_DISCONNECTING:
			case bdConnection::BD_DISCONNECTED:
			{
				m_taskStatus = FAILED;
			}
			break;
			case bdConnection::BD_CONNECTING:
			{
				m_taskStatus = PENDING;
			}
			break;
			case bdConnection::BD_CONNECTED:
			{
				m_taskStatus = DONE;
				m_pumpLobbyConnection = true;
			}
			break;			
		}
	}
}

void Simulation::checkKeyArchiveError()
{
	const bdKeyArchive *const keyArchive = GetKeyArchive();	
	if(keyArchive)
	{
		m_lobbyErrorCode = keyArchive->getErrorCode();
	}
}

void Simulation::printDictList(bdString& string) const
{
	/*
	const bdUInt numDicts = m_foundDicts.getSize();
	bdNChar8 buf[32];
	bdSnprintf(buf, sizeof(buf), "%u dicts(s) found:\n", numDicts);
	string += buf;
	m_foundDicts.print(string);
	*/
}




void Simulation::readKeyPairsDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"Read KeyPairs",			// READY
											"Reading KeyPairs",		// PENDING
											"Read KeyPairs Complete",	// DONE
											"Read KeyPairs Failed",	// FAILED
											"Read KeyPairs Skipped"};	// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);
	
	if(m_taskStatus == DONE)
	{
		description += "\n";
		printDictList(description);
	}
}
void Simulation::startReadKeyPairs()
{
	// fail safe
	m_taskStatus = FAILED;

	bdKeyArchive *const keyArchive = GetKeyArchive();
	if(keyArchive)
	{
		bdKeyPairDict keyPairDict;		
		keyPairDict.put(bdArchiveKey(USERID, false, "client keyName1 3"), bdArchiveValue(0));
	
		// only reads keypairs which were written by users
		bdBool readDedicated = false;
		m_remoteTask = keyArchive->read(keyPairDict, readDedicated);
		m_taskStatus = CheckRemoteTaskStatus(m_remoteTask);
	}
}
void Simulation::pumpReadKeyPairs()
{
	m_taskStatus = CheckRemoteTaskStatus(m_remoteTask);

	if(m_taskStatus == DONE)
	{
		// fail safe
		m_taskStatus = FAILED;

		bdKeyArchive *const keyArchive = GetKeyArchive();
		if(keyArchive)
		{
			// just a local instance, throwing away after results are obtained.
			// could also keep this instance around for later use.
			bdKeyPairDict keyPairDict;	
			const bdBool taskHandled = keyArchive->handleTask(m_remoteTask, &keyPairDict);
			if(taskHandled)
			{
				m_taskStatus = DONE;

				bdUInt numEntityIDs = keyPairDict.getSize();
				bdPrintf("total number of keypairs = %d", numEntityIDs);
				
				if (numEntityIDs > 0)
				{
					bdKeyPairDict::Iterator it;
					it = keyPairDict.getIterator();
					while (it)
					{
						bdArchiveKey &key = keyPairDict.getKey(it);
						bdArchiveValue &value = keyPairDict.getValue(it);

						bdPrintf("entityID = %I64u, isTeam = %d", key.getEntityID(), key.getIsTeam());
						switch(value.getValueType())
						{
							case bdArchiveValue::BD_INT32:
							{
								bdInt32 val = 0;
								value.getValue(val);
								bdPrintf("\tKeyName(%s), Value(%i)\n", key.getKeyName().getBuffer(), val);
							}
							break;
							case bdArchiveValue::BD_INT64:
							{
								bdInt64 val = 0;
								value.getValue(val);
								bdPrintf("\tKeyName(%s), Value(%I64i)\n", key.getKeyName().getBuffer(), val);
							}
							break;
							case bdArchiveValue::BD_FLOAT64:
							{
								bdFloat64 val = 0;
								value.getValue(val);
								bdPrintf("\tKeyName(%s)\tvalue(%lf)\n", key.getKeyName().getBuffer(), val);
							}
							break;
							case bdArchiveValue::BD_NONE:
							{
								// Nothing to do here.
								bdPrintf("\tKeyName(%s)\tvalue(!! NONE!!)\n", key.getKeyName().getBuffer());
							}
							break;
						}
						keyPairDict.next(it);
					}
					keyPairDict.releaseIterator(it);
				}
			}
		}
	}

	if((m_taskStatus == DONE) || (m_taskStatus == FAILED))
	{
		CloseRemoteTask(&m_remoteTask);
	}
}



void Simulation::readAllKeyPairsDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"Read all of an entityIDs keyPairs",			// READY
											"Reading all of an entityIDs keyPairs",		// PENDING
											"Done reading all of an entityIDs keyPairs",			// DONE
											"Read all of an entityIDs keyPairs Failed",		// FAILED
											"Read all of an entityIDs keyPairs Skipped"};	// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);

	if(m_taskStatus == DONE)
	{
		description += "\n";
		printDictList(description);
	}
}

void Simulation::startReadAllKeyPairs()
{
	// fail safe
	m_taskStatus = FAILED;
	
	bdKeyArchive *const keyArchive = GetKeyArchive();
	if(keyArchive)
	{
		bdOnlineUserID userID = USERID;
		bdBool isTeam = false;
		bdBool isDedicated = false;
		m_remoteTask = keyArchive->readAll(userID, isTeam, isDedicated);
		m_taskStatus = CheckRemoteTaskStatus(m_remoteTask);
	}
}

void Simulation::pumpReadAllKeyPairs()
{	
	m_taskStatus = CheckRemoteTaskStatus(m_remoteTask);

	if(m_taskStatus == DONE)
	{
		// fail safe
		m_taskStatus = FAILED;

		bdKeyArchive *const keyArchive = GetKeyArchive();
		if(keyArchive)
		{
			bdKeyPairDict keyPairDict;
			const bdBool taskHandled = keyArchive->handleTask(m_remoteTask, &keyPairDict);
			if(taskHandled)
			{
				m_taskStatus = DONE;
				bdUInt numEntityIDs = keyPairDict.getSize();
				bdPrintf("total number of keypairs = %d", numEntityIDs);
				
				if (numEntityIDs > 0)
				{
					bdKeyPairDict::Iterator it;
					it = keyPairDict.getIterator();
					while (it)
					{
						bdArchiveKey &key = keyPairDict.getKey(it);
						bdArchiveValue &value = keyPairDict.getValue(it);

						bdPrintf("entityID(%I64u), isTeam(%d)", key.getEntityID(), key.getIsTeam());
						switch(value.getValueType())
						{
							case bdArchiveValue::BD_INT32:
							{
								bdInt32 val = 0;
								value.getValue(val);
								bdPrintf("\tKeyName(%s), Value(%i)\n", key.getKeyName().getBuffer(), val);
							}
							break;
							case bdArchiveValue::BD_INT64:
							{
								bdInt64 val = 0;
								value.getValue(val);
								bdPrintf("\tKeyName(%s), Value(%I64i)\n", key.getKeyName().getBuffer(), val);
							}
							break;
							case bdArchiveValue::BD_FLOAT64:
							{
								bdFloat64 val = 0;
								value.getValue(val);
								bdPrintf("\tKeyName(%s)\tvalue(%lf)\n", key.getKeyName().getBuffer(), val);
							}
							break;
							case bdArchiveValue::BD_NONE:
							{
								// Nothing to do here.
								bdPrintf("\tKeyName(%s)\tvalue(!!NONE!!)\n", key.getKeyName().getBuffer());
							}
							break;
						}
						keyPairDict.next(it);
					}
					keyPairDict.releaseIterator(it);
				}
			}
		}
	}

	if((m_taskStatus == DONE) || (m_taskStatus == FAILED))
	{
		CloseRemoteTask(&m_remoteTask);
	}
}


// user writes own KeyPairs
void Simulation::writeKeyPairsDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"User - write KeyPairs",			// READY
											"User - writing KeyPairs",			// PENDING
											"User - wrote KeyPairs",			// DONE
											"User - write KeyPairs Failed",		// FAILED
											"User - write KeyPairs Skipped"};	// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);
}
void Simulation::startWriteKeyPairs()
{
	// fail safe
	m_taskStatus = FAILED;

	bdNetImpl *const net = bdNet::getInstance();
	bdKeyArchive *const keyArchive = GetKeyArchive();

	if(keyArchive && net)
	{
		bdKeyPairDict keyPairDict;
		
		keyPairDict.put(bdArchiveKey(USERID, false, "client keyName1 1"), bdArchiveValue(111111));
		keyPairDict.put(bdArchiveKey(USERID, false, "client keyName1 2"), bdArchiveValue(222222));
		keyPairDict.put(bdArchiveKey(USERID, false, "client keyName1 3"), bdArchiveValue(333333));

		keyPairDict.put(bdArchiveKey(USERID, false, "client keyName2 1"), bdArchiveValue(111111));
		keyPairDict.put(bdArchiveKey(USERID, false, "client keyName2 2"), bdArchiveValue(222222));
		bdFloat64 pi_in_the_sky = 3.141592654;
		keyPairDict.put(bdArchiveKey(USERID, false, "client keyName2 3"), bdArchiveValue(pi_in_the_sky));

		// send to the lobby service
		m_remoteTask = keyArchive->write(keyPairDict);
		m_taskStatus = CheckRemoteTaskStatus(m_remoteTask);		
	}
}
void Simulation::pumpWriteKeyPairs()
{
	m_taskStatus = CheckRemoteTaskStatus(m_remoteTask);

	if(m_taskStatus == DONE)
	{
		// fail safe
		m_taskStatus = FAILED;

		bdKeyArchive *const keyArchive = GetKeyArchive();
		if(keyArchive)
		{
			const bdBool taskHandled = keyArchive->handleTask(m_remoteTask);
			if(taskHandled)
			{
				m_taskStatus = DONE;
			}
		}
	}

	if((m_taskStatus == DONE) || (m_taskStatus == FAILED))
	{
		CloseRemoteTask(&m_remoteTask);
	}
}

void Simulation::deleteKeyPairsDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"User - Delete KeyPairs",			// READY
											"User - Deleting KeyPairs",			// PENDING
											"User - Deleted KeyPairs",			// DONE
											"User - Delete KeyPairs Failed",	// FAILED
											"User - Delete KeyPairs Skipped"};	// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);
}

void Simulation::startDeleteKeyPairs()
{
	// fail safe
	m_taskStatus = FAILED;

	// send a keyPairDict instead of an array of bdArchiveKeys
	bdKeyArchive *const keyArchive = GetKeyArchive();

	if (keyArchive)
	{
		bdKeyPairDict keyPairDict;		
		keyPairDict.put(bdArchiveKey(USERID, false, "client keyName1 1"), bdArchiveValue(0));
		keyPairDict.put(bdArchiveKey(USERID, false, "client keyName1 2"), bdArchiveValue(0));
		keyPairDict.put(bdArchiveKey(USERID, false, "client keyName1 3"), bdArchiveValue(0));

		m_remoteTask = keyArchive->remove(keyPairDict);
		m_taskStatus = CheckRemoteTaskStatus(m_remoteTask);
	}
}

void Simulation::pumpDeleteKeyPairs()
{
	m_taskStatus = CheckRemoteTaskStatus(m_remoteTask);

	if(m_taskStatus == DONE)
	{
		// fail safe
		m_taskStatus = FAILED;

		bdKeyArchive *const keyArchive = GetKeyArchive();
		if(keyArchive)
		{
			const bdBool taskHandled = keyArchive->handleTask(m_remoteTask);
			if(taskHandled)
			{
				m_taskStatus = DONE;
			}
		}
	}

	if((m_taskStatus == DONE) || (m_taskStatus == FAILED))
	{
		CloseRemoteTask(&m_remoteTask);
	}
}

void Simulation::deleteUsersKeyPairsDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"User - Delete All Users KeyPairs",			// READY
											"User - Deleting All Users KeyPairs",			// PENDING
											"User - Deleted All Users KeyPairs",			// DONE
											"User - Delete All Users KeyPairs Failed",	// FAILED
											"User - Delete All Users KeyPairs Skipped"};	// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);
}
void Simulation::startDeleteUsersKeyPairs()
{
	// fail safe
	m_taskStatus = FAILED;

	// send a keyPairDict instead of an array of bdArchiveKeys
	bdKeyArchive *const keyArchive = GetKeyArchive();

	if (keyArchive)
	{
		m_remoteTask = keyArchive->removeAll(USERID, false);
		m_taskStatus = CheckRemoteTaskStatus(m_remoteTask);
	}
}
void Simulation::pumpDeleteUsersKeyPairs()
{
	m_taskStatus = CheckRemoteTaskStatus(m_remoteTask);

	if(m_taskStatus == DONE)
	{
		// fail safe
		m_taskStatus = FAILED;

		bdKeyArchive *const keyArchive = GetKeyArchive();
		if(keyArchive)
		{
			const bdBool taskHandled = keyArchive->handleTask(m_remoteTask);
			if(taskHandled)
			{
				m_taskStatus = DONE;
			}
		}
	}

	if((m_taskStatus == DONE) || (m_taskStatus == FAILED))
	{
		CloseRemoteTask(&m_remoteTask);
	}
}

void Simulation::lobbyServiceDisconnectDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"Disconnect from Lobby Service",			// READY
											"Disconnecting from Lobby Service",			// PENDING
											"Disconnected from Lobby Service",			// DONE
											"Disconnect from Lobby Service Failed",		// FAILED
											"Disconnect from Lobby Service Skipped"};	// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);
}

void Simulation::startLobbyServiceDisconnect()
{
	// fail safe
	m_taskStatus = FAILED;

	bdLobbyServiceImpl *const lobbyService = bdLobbyService::getInstance();
	if(lobbyService)
	{
		lobbyService->disconnect();
		m_taskStatus = PENDING;
	}
	
}

void Simulation::pumpLobbyServiceDisconnect()
{
	// fail safe
	m_taskStatus = FAILED;

	bdLobbyServiceImpl *const lobbyService = bdLobbyService::getInstance();
	if(lobbyService)
	{
		const bdLobbyConnection::Status status = lobbyService->getStatus();
		switch(status)
		{
			case bdConnection::BD_DISCONNECTED:
			case bdConnection::BD_NOT_CONNECTED:
			{
				m_taskStatus = DONE;
			}
			break;
			case bdConnection::BD_CONNECTING:
			case bdConnection::BD_CONNECTED:
			{
				m_taskStatus = FAILED;
			}
			break;
			case bdConnection::BD_DISCONNECTING:
			{
				m_taskStatus = PENDING;
			}
			break;
		}
	}
}

void Simulation::stopNetworkDescription(bdString& description)
{
	const bdNChar8 *const descriptions[] = {"Stop Network",					// READY
											"Stopping Network",				// PENDING
											"Stopped Network",				// DONE
											"Stopping Network Failed",		// FAILED
											"Stopping Network Skipped"};	// SKIPPED

	const bdUInt numDescriptions = sizeof(descriptions) / sizeof(bdNChar8*);
	IndexStrings(descriptions, numDescriptions, m_taskStatus, description);
}

void Simulation::stopNetwork()
{
	bdNetImpl *const net = bdNet::getInstance();
	net->stop();
	if(net->getStatus() == bdNetImpl::BD_NET_STOPPED)
	{
		bdExamplePlatformInit::quit();
		m_taskStatus = DONE;
	}
	else
	{
		m_taskStatus = FAILED;
	}
}
