// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include "LobbyStrings.h"
#include <bdCore/bdCore.h>
#include <bdLobby/bdMessageStore/bdLobbyMessage.h>
#include <bdLobby/bdMessageStore/bdMessageTypes/bdLobbyMessageTypes.h>
#include <bdLobby/bdMessageStore/bdMessageTypes/bdChannelChatMessage.h>
#include <bdLobby/bdMessageStore/bdMessageTypes/bdFriendCustomMessage.h>
#include <bdLobby/bdMessageStore/bdMessageTypes/bdTeamCustomMessage.h>

#include <bdLobby/bdFriends/bdGetFriendsResult.h>
#include <bdLobby/bdFriends/bdGetBlockListResult.h>
#include <bdLobby/bdFriends/bdGetGroupNamesResult.h>
#include <bdLobby/bdFriends/bdGetGroupedFriendsResult.h>
#include <bdLobby/bdFriends/bdGetProposalListResult.h>
#include <bdLobby/bdTeams/bdCheckTeamMembershipResult.h>
#include <bdLobby/bdTeams/bdGetTeamMembersResult.h>
#include <bdLobby/bdTeams/bdCreateTeamResult.h>
#include <bdLobby/bdTeams/bdGetTeamsProposalListResult.h>
#include <bdLobby/bdChannelChat/bdGetChannelMemberNamesResult.h>
#include <bdLobby/bdChannelChat/bdGetChannelMembersResult.h>
#include <bdLobby/bdChannelChat/bdGetChannelsResult.h>
#include <bdLobby/bdStorage/bdGetFileResult.h>
#include <bdLobby/bdStorage/bdListFilesResult.h>
#include <bdLobby/bdStorage/bdUploadFileResult.h>
#include <bdLobby/bdMatchMaking/bdSessionCreateResult.h>
#include <bdLobby/bdStats/bdStatsQueryResult.h>

LobbyStrings::LobbyStrings()
{
	m_errorsHashmap.put(0,"BD_NO_ERROR");

	// Client-side-only errors
	m_errorsHashmap.put(1,"BD_TOO_MANY_TASKS");
	m_errorsHashmap.put(2,"BD_NOT_CONNECTED");	
	m_errorsHashmap.put(3,"BD_SEND_FAILED");

		// General lobby service gateway errors
	m_errorsHashmap.put(100,"BD_MALFORMED_TASK_HEADER");
	m_errorsHashmap.put(101,"BD_ACCESS_DENIED");
	m_errorsHashmap.put(102,"BD_EXCEPTION_IN_DB");
	m_errorsHashmap.put(103,"BD_NO_RESULTS");
	m_errorsHashmap.put(104,"BD_INVALID_ROW");
	m_errorsHashmap.put(105,"BD_INVALID_FIELD");
	m_errorsHashmap.put(106,"BD_PARAM_PARSE_ERROR");
	m_errorsHashmap.put(107,"BD_PARAM_MISMATCHED_TYPE");
	m_errorsHashmap.put(108,"BD_SERVICE_NOT_AVAILABLE");
	m_errorsHashmap.put(109,"BD_CONNECTION_RESET");

		// Teams service errors
	m_errorsHashmap.put(300,"BD_MEMBER_NO_PROPOSAL");
	m_errorsHashmap.put(301,"BD_TEAMNAME_ALREADY_EXISTS");
	m_errorsHashmap.put(302,"BD_NO_MULTIPLE_TEAM_MEMBERSHIP");
	m_errorsHashmap.put(303,"BD_NO_MULTIPLE_TEAM_OWNERSHIP");
	m_errorsHashmap.put(304,"BD_NOT_A_TEAM_MEMBER");
	m_errorsHashmap.put(305,"BD_NOT_A_TEAM_ADMIN");
	m_errorsHashmap.put(306,"BD_INVALID_TEAM_ID");
	m_errorsHashmap.put(307,"BD_NO_INFO_EXISTS");
	m_errorsHashmap.put(308,"BD_TEAM_MEMBER_ALREADY_EXISTS");
	m_errorsHashmap.put(309,"BD_MEMBER_NOT_INVITED");

		// Stats service errors
	m_errorsHashmap.put(400,"BD_S_INVALID_LEADERBOARD_ID");	
	m_errorsHashmap.put(401,"BD_S_INVALID_QUERY_ID");
	m_errorsHashmap.put(402,"BD_S_NO_SUCH_STATS_QUERY_IN_DB");
	m_errorsHashmap.put(403,"BD_S_NOT_ALLOWED_TO_USE_NULL_HERE");
	m_errorsHashmap.put(404,"BD_S_NO_STATS_IN_LEADERBOARD");
	m_errorsHashmap.put(405,"BD_BAD_TEAMID");

		// Matchmaking service errors
	m_errorsHashmap.put(500,"BD_INVALID_TITLE_ID");
	m_errorsHashmap.put(501,"BD_INVALID_USER_ID");
	m_errorsHashmap.put(502,"BD_INVALID_QUERY_ID");
	m_errorsHashmap.put(503,"BD_NO_ENTRY_TO_UPDATE");

		// Messaging service errors
	m_errorsHashmap.put(600,"BD_NO_MSGS");

		// Authentication service errors
	m_errorsHashmap.put(700,"BD_AUTH_NO_ERROR");
	m_errorsHashmap.put(701,"BD_AUTH_BAD_REQUEST");
	m_errorsHashmap.put(702,"BD_AUTH_SERVER_CONFIG_ERROR");
	m_errorsHashmap.put(703,"BD_AUTH_BAD_TITLE_ID");
	m_errorsHashmap.put(704,"BD_AUTH_BAD_ACCOUNT");
	m_errorsHashmap.put(705,"BD_AUTH_ILLEGAL_OPERATION");
	m_errorsHashmap.put(706,"BD_AUTH_INCORRECT_LICENSE_CODE");
	m_errorsHashmap.put(707,"BD_AUTH_CREATE_USERNAME_EXISTS");
	m_errorsHashmap.put(708,"BD_AUTH_CREATE_USERNAME_ILLEGAL");
	m_errorsHashmap.put(709,"BD_AUTH_CREATE_USERNAME_VULGAR");
	m_errorsHashmap.put(710,"BD_AUTH_CREATE_MAX_ACC_EXCEEDED");
	m_errorsHashmap.put(711,"BD_AUTH_MIGRATE_NOT_SUPPORTED");
	m_errorsHashmap.put(712,"BD_AUTH_TITLE_DISABLED");
	m_errorsHashmap.put(713,"BD_AUTH_ACCOUNT_EXPIRED");
	m_errorsHashmap.put(714,"BD_AUTH_ACCOUNT_LOCKED");
	m_errorsHashmap.put(715,"BD_AUTH_UNKNOWN_ERROR");
	m_errorsHashmap.put(716,"BD_AUTH_INCORRECT_PASSWORD");

		// Profile service errors
	m_errorsHashmap.put(800,"BD_PROFILE_EXISTS");

		// Friends service errors
	m_errorsHashmap.put(900,"BD_FRIENDSHIP_NOT_REQUSTED");
	m_errorsHashmap.put(901,"BD_NOT_A_FRIEND");
	m_errorsHashmap.put(902,"BD_NOT_INVITED_TO_SESSION");
	m_errorsHashmap.put(903,"BD_SELF_FRIENDSHIP_NOT_ALLOWED");
	m_errorsHashmap.put(904,"BD_FRIENDSHIP_EXISTS");
	m_errorsHashmap.put(905,"BD_PENDING_FRIENDSHIP_EXISTS");	
	m_errorsHashmap.put(906,"BD_USERID_BANNED");  
	m_errorsHashmap.put(907,"BD_NO_PROFILE_MATCHES_USERID");
	m_errorsHashmap.put(908,"BD_SELF_BLOCK_NOT_ALLOWED");
	
	// Storage service errors
	m_errorsHashmap.put(1000,"BD_NO_FILE");
	m_errorsHashmap.put(1001,"BD_PERMISSION_DENIED");
	m_errorsHashmap.put(1002,"BD_FILESIZE_LIMIT_EXCEEDED");

	m_errorsHashmap.put(1100,"BD_CHANNEL_EXISTS");
	m_errorsHashmap.put(1101,"BD_CHANNEL_DOES_NOT_EXIST");
	m_errorsHashmap.put(1102,"BD_CHANNEL_ALREADY_SUBSCRIBED");
	m_errorsHashmap.put(1103,"BD_CHANNEL_NOT_SUBSCRIBED");

	m_errorsHashmap.put(1104,"BD_MAX_ERROR_CODE");

	// Lobby messaging types
	m_messsageHashmap.put(1,"BD_FRIENDSHIP_PROPOSAL_MSG");
	m_messsageHashmap.put(2,"BD_FRIENDSHIP_ACCEPT_MSG");
	m_messsageHashmap.put(3,"BD_FRIENDSHIP_REJECT_MSG");
	m_messsageHashmap.put(4,"BD_FRIENDSHIP_REVOKE_MSG");
	m_messsageHashmap.put(5,"BD_FRIEND_SESSION_INVITATION_MSG");
	m_messsageHashmap.put(6,"BD_FRIEND_SESSION_ACCEPT_MSG");
	m_messsageHashmap.put(7,"BD_FRIEND_SESSION_REJECT_MSG");
	m_messsageHashmap.put(8,"BD_FRIEND_CUSTOM_MSG");
	m_messsageHashmap.put(9,"BD_FRIEND_CONNECTED_MSG");
	m_messsageHashmap.put(10,"BD_FRIEND_DISCONNECTED_MSG");
	m_messsageHashmap.put(11,"BD_FRIEND_BLOCK_MSG");
	m_messsageHashmap.put(12,"BD_FRIEND_UNBLOCK_MSG");

	m_messsageHashmap.put(13,"BD_TEAM_INVITATION_MSG");
	m_messsageHashmap.put(14,"BD_TEAM_ACCEPT_MSG");
	m_messsageHashmap.put(15,"BD_TEAM_REJECT_MSG");
	m_messsageHashmap.put(16,"BD_TEAM_LEAVE_MSG");
	m_messsageHashmap.put(17,"BD_TEAM_PROMOTION_MSG");
	m_messsageHashmap.put(18,"BD_TEAM_KICK_MSG");
	m_messsageHashmap.put(19,"BD_TEAM_SESSION_INVITATION_MSG");
	m_messsageHashmap.put(20,"BD_TEAM_SESSION_ACCEPT_MSG");
	m_messsageHashmap.put(21,"BD_TEAM_SESSION_REJECT_MSG");
	m_messsageHashmap.put(22,"BD_TEAM_MEMBER_CUSTOM_MSG");
	m_messsageHashmap.put(23,"BD_TEAM_ADMIN_CUSTOM_MSG");
	m_messsageHashmap.put(24,"BD_TEAM_MEMBER_CONNECTED_MSG");
	m_messsageHashmap.put(25,"BD_TEAM_MEMBER_DISCONNECTED_MSG");
	m_messsageHashmap.put(26,"BD_TEAM_DISSOLVED");
	m_messsageHashmap.put(27,"BD_TEAMNAME_UPDATED");
	m_messsageHashmap.put(28,"BD_TEAM_PROMOTION_TO_OWNER_MSG");

	m_messsageHashmap.put(29,"BD_MULTIPLE_LOGONS");

	m_messsageHashmap.put(30,"BD_CHANNEL_CHAT_BROADCAST_MSG");
	m_messsageHashmap.put(31,"BD_CHANNEL_CHAT_WHISPER_MSG");
	m_messsageHashmap.put(32,"BD_CHANNEL_USER_SUBSCRIBE_MSG");
	m_messsageHashmap.put(33,"BD_CHANNEL_USER_UNSUBSCRIBE_MSG");

	m_messsageHashmap.put(34,"BD_FRIENDSHIP_PROPOSAL_CANCEL_MSG");
	m_messsageHashmap.put(35,"BD_FRIEND_STATUS_ONLINE");
	m_messsageHashmap.put(36,"BD_PROFILE_INFO_UPDATED");

	bdSnprintf(m_lastError, 1, "%s", "\n");

}

LobbyStrings::~LobbyStrings()
{
	m_errorsHashmap.clear();
	m_messsageHashmap.clear();
}

bdBool LobbyStrings::formatErrorString(bdUInt32 errorCode)
{
	bdNChar8 *dest;
	bdBool success = m_errorsHashmap.get(errorCode,dest);

	if (success)
	{
		bdSnprintf(m_lastError, bdStrlen(dest)+1, "%s", dest);
	}
	else
	{
		bdSnprintf(m_lastError, 1, "%s", "\n");
	}
	
	return success; 
}

bdNChar8* LobbyStrings::getLastError()
{
	return m_lastError;
}

bdBool LobbyStrings::formatMessageString(bdUInt32 messageCode)
{
	bdNChar8 *dest;
	bdBool success = m_messsageHashmap.get(messageCode,dest);

	if (success)
	{
		bdSnprintf(m_lastMessage, bdStrlen(dest)+1, "%s", dest);	
	}
	else
	{
		bdSnprintf(m_lastMessage, 1, "%s", "\n");
	}

	return success; 
}

bdBool LobbyStrings::formatMessageDesc(bdLobbyMessageRef lobbyMessage)
{
	bdUInt msgType = lobbyMessage->m_msgType;

	bdNChar8 *msgTypeStr;
	bdBool success = m_messsageHashmap.get(msgType,msgTypeStr);
	
	if (success == false)
	{
		return false;
	}

	if ( msgType == 8 )
	{
		bdFriendCustomMessage* msg = static_cast<bdFriendCustomMessage*>(&*lobbyMessage);

		bdSnprintf(m_lastMessage, 256, "\nFrom: %s \nType: %s IsRead: %s\nPayload: %s",
			msg->m_originatorName,
			msgTypeStr,
			(msg->m_isRead ? "true" : "false"),
			msg->m_data->getData());
	}
	else if (msgType >= 22 && msgType <= 23)
	{
		bdTeamCustomMessage<0>* msg = static_cast<bdTeamCustomMessage<0>*>(&*lobbyMessage);
		bdSnprintf(m_lastMessage, 256, "\nFrom: %s Team: %s \nType: %s IsRead: %s\nPayload: %s",
			msg->m_originatorName,
			msg->m_teamName,
			msgTypeStr,
			(msg->m_isRead ? "true" : "false"),
			msg->m_data->getData());
	}
	else if ( msgType >= 1 && msgType <= 12 )
	{
		bdUserNotificationMessage<0>* msg = static_cast<bdUserNotificationMessage<0>*>(&*lobbyMessage);
		bdSnprintf(m_lastMessage, 256, "\nFrom: %s \nType: %s IsRead: %s",
					msg->m_originatorName,
					msgTypeStr,
					(msg->m_isRead ? "true" : "false"));
	}
	else if (msgType >= 13 && msgType <= 28)
	{
		bdTeamNotificationMessage<0>* msg = static_cast<bdTeamNotificationMessage<0>*>(&*lobbyMessage);
		bdSnprintf(m_lastMessage, 256, "\nFrom: %s Team: %s \nType: %s IsRead: %s",
					msg->m_originatorName,
					msg->m_teamName,
					msgTypeStr,
					(msg->m_isRead ? "true" : "false"));
	}
	else if (msgType >= 32 && msgType <= 33)
	{
		bdChannelNotificationMessage<0>* msg = static_cast<bdChannelNotificationMessage<0>*>(&*lobbyMessage);
		bdSnprintf(m_lastMessage, 256, "\nFrom: %s Channel: %s \nType: %s IsRead: %s",
			msg->m_originatorName,
			msg->m_channelName,
			msgTypeStr,
			(msg->m_isRead ? "true" : "false"));
	}
	else if (msgType >= 30 && msgType <= 31)
	{
		bdChannelChatMessage<0>* msg = static_cast<bdChannelChatMessage<0>*>(&*lobbyMessage);

		bdSnprintf(m_lastMessage, 256, "\nFrom: %s Channel: %s \nType: %s IsRead: %s\nPayload: %s",
			msg->m_originatorName,
			msg->m_channelName,
			msgTypeStr,
			(msg->m_isRead ? "true" : "false"),
			msg->m_data->getData());
	}

	return true;
}

bdBool LobbyStrings::formatMessageBriefDesc(bdLobbyMessageRef lobbyMessage)
{
	bdUInt msgType = lobbyMessage->m_msgType;

	bdNChar8 *msgTypeStr;
	bdBool success = m_messsageHashmap.get(msgType,msgTypeStr);

	if (success == false)
	{
		return false;
	}

	if ( msgType == 8 )
	{
		bdFriendCustomMessage* msg = static_cast<bdFriendCustomMessage*>(&*lobbyMessage);

		bdNChar8 messagePayload[18];
		bdSnprintf(messagePayload, 18, "%s", msg->m_data->getData());

		bdSnprintf(m_lastMessage, 256, "\n%s(%s) %s %s...",
			msgTypeStr,
			msg->m_originatorName,
			(msg->m_isRead ? "Old" : "New"),
			messagePayload);
	}
	else if (msgType >= 22 && msgType <= 23)
	{
		bdTeamCustomMessage<0>* msg = static_cast<bdTeamCustomMessage<0>*>(&*lobbyMessage);
		
		bdNChar8 messagePayload[18];
		bdSnprintf(messagePayload, 18, "%s", msg->m_data->getData());
		
		bdSnprintf(m_lastMessage, 256, "\n%s(%s,%s) %s %s...",
			msgTypeStr,
			msg->m_originatorName,
			msg->m_teamName,
			(msg->m_isRead ? "Old" : "New"),
			messagePayload);
	}
	else if ( msgType >= 1 && msgType <= 12 )
	{
		bdUserNotificationMessage<0>* msg = static_cast<bdUserNotificationMessage<0>*>(&*lobbyMessage);
		bdSnprintf(m_lastMessage, 256, "\n%s(%s) %s",
			msgTypeStr,
			msg->m_originatorName,
			(msg->m_isRead ? "Old" : "New"));
	}
	else if (msgType >= 13 && msgType <= 28)
	{
		bdTeamNotificationMessage<0>* msg = static_cast<bdTeamNotificationMessage<0>*>(&*lobbyMessage);
		bdSnprintf(m_lastMessage, 256, "\n%s(%s,%s) %s",
			msgTypeStr,
			msg->m_originatorName,
			msg->m_teamName,
			(msg->m_isRead ? "Old" : "New"));
	}
	else if (msgType >= 32 && msgType <= 33)
	{
		bdChannelNotificationMessage<0>* msg = static_cast<bdChannelNotificationMessage<0>*>(&*lobbyMessage);
		bdSnprintf(m_lastMessage, 256, "\n%s(%s,%s) %s",
			msgTypeStr,
			msg->m_originatorName,
			msg->m_channelName,
			(msg->m_isRead ? "Old" : "New"));
	}
	else if (msgType >= 30 && msgType <= 31)
	{
		bdChannelChatMessage<0>* msg = static_cast<bdChannelChatMessage<0>*>(&*lobbyMessage);

		bdNChar8 messagePayload[18];
		bdSnprintf(messagePayload, 18, "%s", msg->m_data->getData());

		bdSnprintf(m_lastMessage, 256, "\n%s(%s,%s) %s %s...",
			msgTypeStr,
			msg->m_originatorName,
			msg->m_channelName,
			(msg->m_isRead ? "Old" : "New"),
			messagePayload);
	}

	return true;
}

void LobbyStrings::appendFriendsText(const bdGetFriendsResult& friendsList, bdString* screenText)
{
	for (int i=friendsList.m_friends.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nFriend: %s Online: %i",
			friendsList.m_friends[i].m_userName,
			friendsList.m_friends[i].m_isOnline);

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}

}

void LobbyStrings::appendGroupedFriendsText(const bdGetGroupedFriendsResult& friendsList, bdString* screenText)
{
	for (int i=friendsList.m_friends.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nFriend: %s Online: %i GroupID: %i",
			friendsList.m_friends[i].m_userName,
			friendsList.m_friends[i].m_isOnline,
			friendsList.m_friends[i].m_groupID);

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}

/// Appends to the supplied string
void LobbyStrings::appendBlockListText(const bdGetBlockListResult& blockList, bdString* screenText)
{
	for (int i=blockList.m_blockList.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nUserName: %s BlockType: %i",
			blockList.m_blockList[i].m_userName,
			blockList.m_blockList[i].m_blockLevel);

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}

/// Appends to the supplied string
void LobbyStrings::appendProposalListText(const bdGetProposalListResult& proposalList, bdString* screenText)
{
	for (int i=proposalList.m_proposalList.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nPotential Friend: %s",
			proposalList.m_proposalList[i].m_userName);

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}


/// Appends to the supplied string
void LobbyStrings::appendTeamProposalListText(const bdGetTeamsProposalListResult& proposalList, bdString* screenText)
{
	for (int i=proposalList.m_proposalList.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nPotential Team Member: %s",
			proposalList.m_proposalList[i].m_userName);

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}

void LobbyStrings::appendGroupNamesText(const bdGetGroupNamesResult& groupList, bdString* screenText)
{
	for (int i=groupList.m_friendgroups.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nGroupID: %i GroupName: %s",
			groupList.m_friendgroups[i].m_groupID,
			groupList.m_friendgroups[i].m_groupName);

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}

/// Appends to the supplied string
void LobbyStrings::appendTeamMembersText(const bdGetTeamMembersResult& membersList, bdString* screenText)
{
	for (int i=membersList.m_members.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nMember: %s Status: %i Online: %i",
			membersList.m_members[i].m_userName,
			membersList.m_members[i].m_status,
			membersList.m_members[i].m_isOnline);

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}

/// Appends to the supplied string
void LobbyStrings::appendTeamMembershipText(const bdCheckTeamMembershipResult& membershipList, bdString* screenText)
{
	for (int i=membershipList.m_teams.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nTeam: %s Status: %i",
			membershipList.m_teams[i].m_teamName,
			membershipList.m_teams[i].m_memberStatus);

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}

/// Appends to the supplied string
void LobbyStrings::appendTeamCreationText(const bdCreateTeamResult& creationResult, bdString* screenText)
{
	bdNChar8 buffer[128];
	int size = bdSnprintf(buffer, 128, "\nTeamID: %i",
		creationResult.m_teamID);

	bdNChar8 *buffer2 = new bdNChar8[size+1];
	bdStrlcpy(buffer2, buffer, size+1);

	*screenText += buffer2;
	delete buffer2;
}

/// Appends to the supplied string
void LobbyStrings::appendChannelsText(const bdGetChannelsResult& channelsList, bdString* screenText)
{
	for (int i=channelsList.m_channelList.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nName: %s Type: %i\nNumMembers: %i MaxMembers: %i",
			channelsList.m_channelList[i].m_channelName,
			channelsList.m_channelList[i].m_channelType,
			channelsList.m_channelList[i].m_numSubscribers,
			channelsList.m_channelList[i].m_maxSubscribers);

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}

/// Appends to the supplied string
void LobbyStrings::appendChannelMembersText(const bdGetChannelMembersResult& membershipList, bdString* screenText)
{
	for (int i=membershipList.m_memberList.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nMember: %s Type: %i",
			membershipList.m_memberList[i].m_userName,
			membershipList.m_memberList[i].m_userType);

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}

/// Appends to the supplied string
void  LobbyStrings::appendListFilesText(const bdListFilesResult& fileslist, bdString* screenText)
{
	for (int i=fileslist.m_fileHeaders.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nOwnerType: %i Visibility: %i\nName: %s Size: %i",
			fileslist.m_fileHeaders[i]->getOwnerType(),
			fileslist.m_fileHeaders[i]->getVisibility(),
			fileslist.m_fileHeaders[i]->getFileName(),
			fileslist.m_fileHeaders[i]->getFileSize());

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}

/// Appends to the supplied string
void  LobbyStrings::appendGetFileText(const bdGetFileResult& file, bdString* screenText)
{
	for (int i=file.m_files.getSize()-1;i>=0;i--)
	{
		bdNChar8 buffer[128];
		int size = bdSnprintf(buffer, 128, "\nOwnerType: %i Visibility: %i\nName: %s Size: %i",
			file.m_files[i]->getOwnerType(),
			file.m_files[i]->getVisibility(),
			file.m_files[i]->getFileName(),
			file.m_files[i]->getFileSize());

		bdNChar8 *buffer2 = new bdNChar8[size+1];
		bdStrlcpy(buffer2, buffer, size+1);

		*screenText += buffer2;
		delete buffer2;
	}
}

/// Appends to the supplied string
void  LobbyStrings::appendUploadFileText(const bdUploadFileResult& file, bdString* screenText)
{
	bdNChar8 buffer[128];
	int size = bdSnprintf(buffer, 128, "\nFileID: %i",
		file.m_fileID);

	bdNChar8 *buffer2 = new bdNChar8[size+1];
	bdStrlcpy(buffer2, buffer, size+1);

	*screenText += buffer2;
	delete buffer2;
}

/// Appends to the supplied string
void  LobbyStrings::appendSessionText(const bdSessionCreateResult& session, bdString* screenText)
{
	bdNChar8 buffer[128];
	int size = bdSnprintf(buffer, 128, "\nSecurityID: %s SecurityKey: %s",
		session.m_secID.ab,
		session.m_secKey.ab);

	bdNChar8 *buffer2 = new bdNChar8[size+1];
	bdStrlcpy(buffer2, buffer, size+1);

	*screenText += buffer2;
	delete buffer2;
}

/// Appends to the supplied string
void  LobbyStrings::appendStatsQueryText(const bdStatsQueryResult& stats, bdString* screenText)
{
	bdNChar8 buffer[128];
	int size = bdSnprintf(buffer, 128, "\nTotal Leaderboard Entries: %i",
		stats.m_totalLeaderboardEntries);

	bdNChar8 *buffer2 = new bdNChar8[size+1];
	bdStrlcpy(buffer2, buffer, size+1);

	*screenText += buffer2;
	delete buffer2;
}

void LobbyStrings::appendMessages(bdMessageStore* msgManager, bdString* screenText)
{
	int numMessages = msgManager->getNumMessages(); 
	if (numMessages > 0)
	{
		for (int i=numMessages-1;i>=0;i--)
		{
			bdLobbyMessageRef msg = msgManager->getAt(i);

			formatMessageBriefDesc(msg);

			*screenText += getLastMessage();

		}
		*screenText += "\n";
	}
}

void LobbyStrings::consumeMessages(bdMessageStore* msgManager, bdString* screenText)
{
	int numMessages = msgManager->getNumMessages(); 
	if (numMessages > 0)
	{
		for (int i=numMessages-1;i>=0;i--)
		{
			bdLobbyMessageRef msg = msgManager->getAt(i);

			formatMessageDesc(msg);

			*screenText += "\n";
			*screenText += getLastMessage();

			msgManager->removeAt(i);
		}
	}
}

void LobbyStrings::consumeMessage(bdMessageStore* msgManager, bdString* screenText)
{
	int numMessages = msgManager->getNumMessages(); 
	if (numMessages > 0)
	{
		bdLobbyMessageRef msg = msgManager->getAt(0);

		formatMessageDesc(msg);

		*screenText += "\n";
		*screenText += getLastMessage();

		msgManager->removeAt(0);
	}
}

bdNChar8* LobbyStrings::getLastMessage()
{
	return m_lastMessage;
}

