// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdLobby/bdChannelChat/bdChannelChat.h>
#include <bdLobby/bdRemoteTaskManager/bdRemoteTaskManager.h>

#define BD_LOG_LEVEL	("lobby/channelChat")

bdChannelChat::bdChannelChat(bdRemoteTaskManager *const remoteTaskManager)
: m_errorCode(BD_NO_ERROR),
  m_remoteTaskManager(remoteTaskManager)
{
}

bdChannelChat::~bdChannelChat()
{
	m_remoteTaskManager = BD_NULL;
}

bdRemoteTask *bdChannelChat::subscribeToChannel(bdUByte8 /*controllerIndex*/, 
												bdUInt64 channelID)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(channelID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_CHANNEL_CHAT_SERVICE,
		BD_CHANNEL_CHAT_SUBSCRIBE,
		buffer);

	return remoteTask;
}

bdRemoteTask* bdChannelChat::unsubscribeToChannel(bdUByte8 /*controllerIndex*/, 
												  bdUInt64 channelID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(channelID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
											 BD_CHANNEL_CHAT_SERVICE,
											 BD_CHANNEL_CHAT_UNSUBSCRIBE,
											 buffer);
	return remoteTask;
}

bdRemoteTask* bdChannelChat::getChannelList(bdUByte8 /*controllerIndex*/)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												 BD_CHANNEL_CHAT_SERVICE,
												 BD_CHANNEL_CHAT_DISCOVER,
												 buffer);
	return remoteTask;
}

bdRemoteTask *bdChannelChat::sendToChannel(bdUByte8 /*controllerIndex*/, 
										   bdUInt64 channelID,
										   bdByteBuffer& payload)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(channelID);
	buffer.writeBlob(payload.getData(),payload.getSize());

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												 BD_CHANNEL_CHAT_SERVICE,
												 BD_CHANNEL_CHAT_SEND,
												 buffer);

	return remoteTask;
}

bdRemoteTask *bdChannelChat::whisperToChannelMember(bdUByte8 /*controllerIndex*/, 
													bdUInt64 channelID,
													bdOnlineUserID userID,
													bdByteBuffer& payload)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(channelID);
	buffer.writeUInt64(userID);
	buffer.writeBlob(payload.getData(),payload.getSize());

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_CHANNEL_CHAT_SERVICE,
		BD_CHANNEL_CHAT_WHISPER,
		buffer);

	return remoteTask;
}

bdRemoteTask *bdChannelChat::getChannelMembers(bdUByte8 /*controllerIndex*/, 
											   bdUInt64 channelID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(channelID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_CHANNEL_CHAT_SERVICE,
		BD_CHANNEL_CHAT_GET_MEMBERS,
		buffer);

	return remoteTask;
}

bdBool bdChannelChat::handleTask(const bdRemoteTask *remoteTask, 
							 bdQueryResult *queryResult)
{
	bdBool ok;
	bdBitBufferRef buffer = remoteTask->getResults();
	bdUInt32 taskErrorCode = BD_MAX_ERROR_CODE;
	ok = buffer->readUInt32(taskErrorCode);
	if (ok)
	{
		m_errorCode = static_cast<bdLobbyErrorCode>(taskErrorCode);
		if (m_errorCode == BD_NO_ERROR)
		{
			bdUByte8 operation = 0; 
			ok = buffer->readUByte8(operation);

			if(ok)
			{
				switch (operation)
				{
					case BD_CHANNEL_CHAT_SUBSCRIBE:
					{
						if (queryResult)
						{
							ok = queryResult->handleResult(1, buffer);
						}
					}
					break;
					case BD_CHANNEL_CHAT_DISCOVER:
					case BD_CHANNEL_CHAT_GET_MEMBERS:
					{
						if (queryResult)
						{
							bdUInt32 numReplies = 0;
							ok = buffer->readUInt32(numReplies);
							if (ok && numReplies > 0)
							{
								ok = queryResult->handleResult(numReplies, buffer);
							}
						}
						break;
					}
					default:
					{
					}
					break;
				}
			}
		}
#if defined BD_DEBUG
		else
		{
			bdUInt64 transactionID;
			bdUInt64 connectionID;

			connectionID = m_remoteTaskManager->getConnectionID();
			transactionID = remoteTask->getTransactionID();

			// here we originally called bdRemoteTaskManager::obtainTransactionID
			BD_WARN(BD_LOG_LEVEL, "Transaction ID " BD_UINT_64_FORMAT ":" BD_UINT_64_FORMAT " failed.",
				connectionID, transactionID);
		}
#endif // BD_DEBUG
	}
	return (m_errorCode == BD_NO_ERROR && ok);
}

bdLobbyErrorCode bdChannelChat::getErrorCode() const
{
	return m_errorCode;
}


