// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdLobby/bdFriends/bdFriends.h>
#include <bdLobby/bdRemoteTaskManager/bdRemoteTaskManager.h>
#include <bdLobby/bdFriends/bdGetFriendsResult.h>

#define BD_LOG_LEVEL	("lobby/friends")

bdFriends::bdFriends(bdRemoteTaskManager *const remoteTaskManager)
: m_errorCode(BD_NO_ERROR),
  m_remoteTaskManager(remoteTaskManager)
{
	
}

bdFriends::~bdFriends()
{
	m_remoteTaskManager = BD_NULL;
}

bdRemoteTask *bdFriends::proposeFriendship(bdUByte8 /*controllerIndex*/, 
										   bdOnlineUserID friendID,
										   bdByteBuffer * const attachment)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(friendID);

	if ( attachment != BD_NULL )
	{
		buffer.writeBlob(attachment->getData(),attachment->getSize());
	}

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
											 BD_FRIENDS_MANAGEMENT_SERVICE,
											 BD_FRIEND_PROPOSE_FRIENDSHIP,
											 buffer);

	return remoteTask;
}

bdRemoteTask *bdFriends::withdrawProposal(bdUByte8 /*controllerIndex*/, 
										   bdOnlineUserID friendID)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(friendID);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_FRIENDS_MANAGEMENT_SERVICE,
		BD_FRIEND_WITHDRAW_PROPOSAL,
		buffer);

	return remoteTask;
}

bdRemoteTask *bdFriends::getProposalList(bdUByte8 /*controllerIndex*/)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_FRIENDS_MANAGEMENT_SERVICE,
												BD_FRIEND_GET_PROPOSALS,
												buffer);

	return remoteTask;
}

bdRemoteTask* bdFriends::acceptProposal(bdUByte8 /*controllerIndex*/, 
										bdOnlineUserID proposer)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(proposer);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_FRIENDS_MANAGEMENT_SERVICE,
												BD_FRIEND_ACCEPT_PROPOSAL,
												buffer);
	return remoteTask;
}

bdRemoteTask* bdFriends::rejectProposal(bdUByte8 /*controllerIndex*/, 
										bdOnlineUserID proposer)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(proposer);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												 BD_FRIENDS_MANAGEMENT_SERVICE,
												 BD_FRIEND_REJECT_PROPOSAL,
												 buffer);
	return remoteTask;
}

bdRemoteTask *bdFriends::removeFriend(bdUByte8 /*controllerIndex*/, 
									  bdOnlineUserID friendID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(friendID);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												 BD_FRIENDS_MANAGEMENT_SERVICE,
												 BD_FRIEND_REVOKE_FRIENDSHIP,
												 buffer);

	return remoteTask;
}

bdRemoteTask *bdFriends::getFriends(bdUByte8 /*controllerIndex*/)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
											 BD_FRIENDS_MANAGEMENT_SERVICE,
											 BD_FRIEND_GET_FRIENDS,
											 buffer);

	return remoteTask;
}

bdRemoteTask *bdFriends::inviteFriend(bdUByte8 /*controllerIndex*/, 
									  bdOnlineUserID friendID,
									  const bdSecurityID &securityID,
									  bdByteBuffer * const attachment)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(friendID);
	buffer.writeBlob(securityID.ab, BD_SECURITY_ID_LENGTH);

	if ( attachment != BD_NULL )
	{
		buffer.writeBlob(attachment->getData(),attachment->getSize());
	}

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_FRIENDS_MANAGEMENT_SERVICE,
												BD_FRIEND_INVITE_TO_SESSION,
												buffer);

	return remoteTask;
}

bdRemoteTask* bdFriends::acceptInvitation(bdUByte8 /*controllerIndex*/, 
										  bdOnlineUserID inviter)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(inviter);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
											 BD_FRIENDS_MANAGEMENT_SERVICE,
											 BD_FRIEND_ACCEPT_INVITE,
											 buffer);
	return remoteTask;
}

bdRemoteTask* bdFriends::rejectInvitation(bdUByte8 /*controllerIndex*/, 
										  bdOnlineUserID inviter)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(inviter);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												 BD_FRIENDS_MANAGEMENT_SERVICE,
												 BD_FRIEND_REJECT_INVITE,
												 buffer);
	return remoteTask;
}

bdRemoteTask *bdFriends::setBlockedStatus(bdUByte8 /*controllerIndex*/, 
										  bdOnlineUserID userID,
										  bdBlockLevel blockLevel)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(userID);
	const bdUByte8 tempBlockLevel = static_cast<bdUByte8>(blockLevel);
	buffer.writeUByte8(tempBlockLevel);
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
											 BD_FRIENDS_MANAGEMENT_SERVICE,
											 BD_FRIEND_SET_BLOCKED_STATUS,
											 buffer);
	return remoteTask;
}

bdRemoteTask *bdFriends::getBlockedList(bdUByte8 /*controllerIndex*/)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_FRIENDS_MANAGEMENT_SERVICE,
												BD_FRIEND_GET_BLOCKED,
												buffer);

	return remoteTask;
}

bdRemoteTask *bdFriends::sendFriendMessage(bdUByte8 /*controllerIndex*/, 
										   bdOnlineUserID friendId,
										   bdByteBuffer& payload,
										   bdMessageDeliveryMethod deliveryMethod)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(friendId);

	buffer.writeUInt32(static_cast<bdUInt32>(deliveryMethod));

	buffer.writeBlob(payload.getData(),payload.getSize());

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_FRIENDS_MANAGEMENT_SERVICE,
												BD_FRIEND_SEND_FRIEND_MSG,
												buffer);

	return remoteTask;
}

bdRemoteTask *bdFriends::setFriendGroup(bdUByte8 /*controllerIndex*/, 
									    bdOnlineUserID friendID,
										bdUByte8 groupID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);

	buffer.writeUByte8(0);
	buffer.writeUInt64(friendID);
	buffer.writeUByte8(groupID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_FRIENDS_MANAGEMENT_SERVICE,
												BD_FRIEND_SET_GROUP,
												buffer);

	return remoteTask;
}

bdRemoteTask *bdFriends::setGroupName(bdUByte8 /*controllerIndex*/, 
									  bdUByte8 groupID,
									  const bdNChar8 *groupName)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);

	buffer.writeUByte8(0);
	buffer.writeUByte8(groupID);
	buffer.writeString(groupName,bdStrnlen(groupName,BD_MAX_GROUPNAME_LENGTH));

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_FRIENDS_MANAGEMENT_SERVICE,
												BD_FRIEND_SET_GROUP_NAME,
												buffer);

	return remoteTask;
}

bdRemoteTask *bdFriends::getGroupNames(bdUByte8 /*controllerIndex*/)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);

	buffer.writeUByte8(0);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_FRIENDS_MANAGEMENT_SERVICE,
												BD_FRIEND_GET_GROUPS,
												buffer);

	return remoteTask;
}

bdRemoteTask *bdFriends::getGroupedFriends(bdUByte8 /*controllerIndex*/)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);

	buffer.writeUByte8(0);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_FRIENDS_MANAGEMENT_SERVICE,
												BD_FRIEND_GET_GROUPED_FRIENDS,
												buffer);

	return remoteTask;
}

bdRemoteTask *bdFriends::deleteGroupName(bdUByte8 /*controllerIndex*/, 
										 bdUByte8 groupID)
{
	bdRemoteTask *remoteTask = BD_NULL;
	bdBitBuffer buffer(0, true);

	buffer.writeUByte8(0);
	buffer.writeUByte8(groupID);
	
	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
		BD_FRIENDS_MANAGEMENT_SERVICE,
		BD_FRIEND_DELETE_GROUP_NAME,
		buffer);

	return remoteTask;
}

bdBool bdFriends::handleTask(const bdRemoteTask *remoteTask, 
							 bdQueryResult *queryResult)
{
	bdBool ok;
	bdBitBufferRef buffer = remoteTask->getResults();
	bdUInt32 taskErrorCode = BD_MAX_ERROR_CODE;
	ok = buffer->readUInt32(taskErrorCode);
	if (ok)
	{
		m_errorCode = static_cast<bdLobbyErrorCode>(taskErrorCode);
		if (m_errorCode == BD_NO_ERROR)
		{
			bdUByte8 operation = 0; 
			ok = buffer->readUByte8(operation);

			if(ok)
			{
				switch (operation)
				{
					case BD_FRIEND_GET_FRIENDS:
					case BD_FRIEND_GET_GROUPED_FRIENDS:
					case BD_FRIEND_GET_GROUPS:
					case BD_FRIEND_GET_BLOCKED:
					case BD_FRIEND_GET_PROPOSALS:
					{
						if (queryResult)
						{
							bdUInt32 numReplies = 0;
							ok = buffer->readUInt32(numReplies);
							if (ok && numReplies > 0)
							{
								ok = queryResult->handleResult(numReplies, buffer);
							}
						}
						break;
					}
					default:
					{
					}
					break;
				}
			}
		}
#if defined BD_DEBUG
		else
		{
			bdUInt64 transactionID;
			bdUInt64 connectionID;

			connectionID = m_remoteTaskManager->getConnectionID();
			transactionID = remoteTask->getTransactionID();

			// here we originally called bdRemoteTaskManager::obtainTransactionID
			BD_WARN(BD_LOG_LEVEL, "Transaction ID " BD_UINT_64_FORMAT ":" BD_UINT_64_FORMAT " failed.",
				connectionID, transactionID);
		}
#endif // BD_DEBUG	
	}
	return (m_errorCode == BD_NO_ERROR && ok);
}

bdLobbyErrorCode bdFriends::getErrorCode() const
{
	return m_errorCode;
}


