// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

// PURPOSE:	

#ifndef BD_ARCHIVE_VALUE_H
#define BD_ARCHIVE_VALUE_H

#include <bdCore/bdCore.h>
#include <bdLobby/bdLobbyCommon/bdLobbyServicesCommon.h>

#define BD_MAX_STRING_VALUE_LEN 512

/// Contains a single 'value' for a keyPair.
/// The value's data type can be:
/// bdFloat64 (precision not guaranteed, may write and read the same value & the value may be out.
/// bdInt64
/// bdInt32
class bdArchiveValue
{
	public:
		
		enum bdArchiveUpdateType
		{
		   BD_REPLACE,
		   BD_ADD
		};

		enum ValueType
		{
			BD_NONE,
			BD_FLOAT64,
			BD_INT32,
			BD_INT64
		};

		bdArchiveValue();
		~bdArchiveValue();

		bdArchiveValue(const bdFloat64 value, bdArchiveUpdateType updateType = BD_REPLACE);
		bdArchiveValue(const bdInt32 value, bdArchiveUpdateType updateType = BD_REPLACE);
		bdArchiveValue(const bdInt64 value, bdArchiveUpdateType updateType = BD_REPLACE);

		void setValue(const bdFloat64 value, bdArchiveUpdateType updateType = BD_REPLACE);
		void setValue(const bdInt32 value, bdArchiveUpdateType updateType = BD_REPLACE);
		void setValue(const bdInt64 value, bdArchiveUpdateType updateType = BD_REPLACE);

		bdBool getValue(bdFloat64 &value);
		bdBool getValue(bdInt32 &value);
		bdBool getValue(bdInt64 &value);

		bdArchiveUpdateType getUpdateType();
		ValueType getValueType();

		bdBool hasValue() const;

		void serialize(bdBitBufferRef buffer) const;
		
		bdBool deserialize(bdBitBufferRef buffer);
	
	protected:

		union
		{
			bdFloat64 m_f64Value;
			bdInt32 m_i32Value;
			bdInt64 m_i64Value;
		};

		ValueType m_valueType;
		bdArchiveUpdateType m_updateType;
};

#endif //BD_ARCHIVE_VALUE_H
