// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

// PURPOSE:


#include <bdLobby/bdLobbyService.h>

#include <bdLobby/bdAuthTicket.h>
#include <bdLobby/bdRemoteTaskManager/bdRemoteTaskManager.h>
#include <bdLobby/bdMatchMaking/bdMatchMaking.h>
#include <bdLobby/bdStats/bdStats.h>
#include <bdLobby/bdProfiles/bdProfiles.h>
#include <bdLobby/bdTeams/bdTeams.h>
#include <bdLobby/bdFriends/bdFriends.h>
#include <bdLobby/bdStorage/bdStorage.h>
#include <bdLobby/bdChannelChat/bdChannelChat.h>
#include <bdLobby/bdTitleUtilities/bdTitleUtilities.h>
#include <bdLobby/bdKeyArchive/bdKeyArchive.h>


#define BD_LOG_LEVEL "lobbyservice"

BD_REGISTER_SINGLETON(bdLobbyServiceImpl)

#if (!defined BD_PLATFORM_XBOX) && (!defined BD_PLATFORM_XENON)
bdBool bdLobbyServiceImpl::connect(bdAddr lobbyServiceAddr, 
                                   const bdAuthInfo *authInfo,
								   const bdBool useEncryption)
{
	m_encryptedConnection = useEncryption;

	BD_ASSERT(authInfo != BD_NULL, "No authentication info specified");
	bdBool ok = false;
	bdCommonAddrRef lsgAddr;
	m_authInfo = *authInfo;
	lsgAddr = new bdCommonAddr(lobbyServiceAddr);

	if (m_lobbyConnection.isNull())
	{
		m_lobbyConnection = new bdLobbyConnection( lsgAddr, this);
		ok = m_lobbyConnection->connect(m_authInfo);
		
	}	
	return ok;
}

bdBool bdLobbyServiceImpl::connect(bdAddr lobbyServiceAddr, 
								   const bdAuthInfo *authInfo,
								   void *sendBuffer,
								   bdUWord sendBufSize,
								   void *recvBuffer, 
								   bdUWord recvBufSize,
								   const bdBool useEncryption )
{
	m_encryptedConnection = useEncryption;
	BD_ASSERT(authInfo != BD_NULL, "No authentication info specified");
	m_authInfo = *authInfo;
	bdBool ok = false;
	bdCommonAddrRef lsgAddr = new bdCommonAddr(lobbyServiceAddr);
	if (m_lobbyConnection.isNull())
	{
		m_lobbyConnection = new bdLobbyConnection( lsgAddr, sendBuffer, sendBufSize, recvBuffer, recvBufSize, this);
		ok = m_lobbyConnection->connect(m_authInfo);

	}	
	return ok;
}

#else
#if (defined BD_PLATFORM_XBOX) && (!defined BD_PLATFORM_XENON)
bdBool bdLobbyServiceImpl::connect(	bdCommonAddrRef lobbyServiceAddr, 
									bdSecurityID &id,
									const bdAuthInfo *authInfo)
{
	m_encryptedConnection = false;
	BD_ASSERT(authInfo != BD_NULL, "No authentication info specified");
	m_authInfo = *authInfo;
	bdBool result = false;
	if (m_lobbyConnection.isNull())
	{
		m_lobbyConnection = new bdLobbyConnection( lobbyServiceAddr, id, this);
		result = m_lobbyConnection->connect(m_authInfo);

	}
	return result;
}
#elif defined BD_PLATFORM_XENON
bdBool bdLobbyServiceImpl::connect(	bdAddr lobbyServiceAddr, 
									bdSecurityID &id,
									const bdAuthInfo *authInfo)
{
	m_encryptedConnection = false;
	BD_ASSERT(authInfo != BD_NULL, "No authentication info specified");
	m_authInfo = *authInfo;
	bdBool result = false;
	if (m_lobbyConnection.isNull())
	{
		m_lobbyConnection = new bdLobbyConnection( lobbyServiceAddr, id, this);
		result = m_lobbyConnection->connect(m_authInfo);

	}
	return result;
}
#endif
#endif

void bdLobbyServiceImpl::disconnect()
{
	if(m_lobbyConnection.notNull())
	{
		m_lobbyConnection->disconnect();
	}

	m_lobbyConnection = BD_NULL;
	
	if (m_taskManager)
	{
		delete m_taskManager;
		m_taskManager = BD_NULL;
	}

	if (m_profiles)
	{
		delete m_profiles;
		m_profiles = BD_NULL;
	}
	if (m_matchMaking)
	{
		delete m_matchMaking;
		m_matchMaking = BD_NULL;
	}
	if (m_statsManager)
	{
		delete m_statsManager;
		m_statsManager = BD_NULL;
	}
	if (m_teamsManager)
	{
		delete m_teamsManager;
		m_teamsManager = BD_NULL;
	}
	if (m_friendsManager)
	{
		delete m_friendsManager;
		m_friendsManager = BD_NULL;
	}
	if (m_storageManager)
	{
		delete m_storageManager;
		m_storageManager = BD_NULL;
	}
	if (m_channelChatManager)
	{
		delete m_channelChatManager;
		m_channelChatManager = BD_NULL;
	}
	if (m_titleUtilitiesManager)
	{
		delete m_titleUtilitiesManager;
		m_titleUtilitiesManager = BD_NULL;
	}
	m_titleID = 0;
}


bdLobbyConnection::Status bdLobbyServiceImpl::getStatus()
{
	
	if (!m_lobbyConnection)
	{
		return bdLobbyConnection::BD_NOT_CONNECTED;
	}
	bdBitBufferRef message;
	bdUByte8 messageType  = 0;
	while (m_lobbyConnection->getMessageToDispatch(messageType, message))
	{
		
		switch (messageType)
		{
			case BD_LOBBY_SERVICE_TASK_REPLY:
			{
				BD_INFO(BD_LOG_LEVEL, "Received message of type: BD_LOBBY_SERVICE_TASK_REPLY");

				if(m_taskManager)
				{
					m_taskManager->handleTaskReply(message);
				}	
				else
				{
					BD_WARN(BD_LOG_LEVEL, "No task manager.");
				}
			}
			break;
			case BD_LOBBY_SERVICE_PUSH_MESSAGE:
			{
				BD_INFO(BD_LOG_LEVEL, "Received message of type: BD_LOBBY_SERVICE_PUSH_MESSAGE");

				if(m_messagingMode == BD_RETAIN_PUSH_MESSAGES)
				{
					bdMessageStore *const messageStore = getMessageStore();
					if(messageStore)
					{
						messageStore->createMessage(message);
					}
					else
					{
						BD_WARN(BD_LOG_LEVEL, "No message store.");
					}
				}
			}
			break;
			case BD_LOBBY_ERROR_RESULT:
			{
				//bdBitBufferRef payload = message->getPayload();
				bdUInt32 errorCode;
				if (message->readUInt32(errorCode))
				{
					m_errorCode = errorCode;
					BD_INFO(BD_LOG_LEVEL, "Received LSG error: %u", errorCode);
				}
				else
				{
					BD_WARN(BD_LOG_LEVEL, "Received LSG error message, but cannot read out errorCode.");
				}
			}
			break;
			case BD_LOBBY_CONNECTION_ID:
				{
					//bdBitBufferRef payload = message->getPayload();
					bdUInt64 connectionID;
					if (message->readUInt64(connectionID))
					{
						if (m_taskManager)
						{
							BD_INFO(BD_LOG_LEVEL, "Received LSG connection ID:" BD_UINT_64_FORMAT, connectionID);
							m_taskManager->m_connectionID = connectionID;
						}
						else
						{
							BD_WARN(BD_LOG_LEVEL, "No task manager to assign connection ID to.");
						}
					}
					else
					{
						BD_WARN(BD_LOG_LEVEL, "Received LSG connection ID header, but couldn't read ID.");
					}
				}
				break;
			default:
			{
				BD_WARN(BD_LOG_LEVEL, "Received unknown message type: %u.", messageType);
			}
		}		
	}

	return m_lobbyConnection->getStatus();
}

bdProfiles* bdLobbyServiceImpl::getProfilesManager()
{
	if (!m_lobbyConnection || (m_lobbyConnection->getStatus() != bdLobbyConnection::BD_CONNECTED))
	{
		BD_WARN(BD_LOG_LEVEL, "Not connected.");
		return BD_NULL;
	}
	if (!m_profiles)
	{
		m_profiles = new bdProfiles(m_taskManager);
	}

	return m_profiles;
}

bdMessageStore* bdLobbyServiceImpl::getMessageStore()
{
	return &m_messageStore;
}


bdMatchMaking* bdLobbyServiceImpl::getMatchMaking()
{
	if (!m_lobbyConnection || (m_lobbyConnection->getStatus() != bdLobbyConnection::BD_CONNECTED))
	{
		BD_WARN(BD_LOG_LEVEL, "Not connected.");
		return BD_NULL;
	}
	if (!m_matchMaking)
	{
		m_matchMaking = new bdMatchMaking(m_taskManager);
	}

	return m_matchMaking;
}
bdStats* bdLobbyServiceImpl::getStats()
{
	if (!m_lobbyConnection || (m_lobbyConnection->getStatus() != bdLobbyConnection::BD_CONNECTED))
	{
		BD_WARN(BD_LOG_LEVEL, "Not connected.");
		return BD_NULL;
	}
	if (!m_statsManager)
	{
		m_statsManager = new bdStats(m_taskManager);
	}

	return m_statsManager;
}

bdTeams* bdLobbyServiceImpl::getTeams()
{
	if (!m_lobbyConnection || (m_lobbyConnection->getStatus() != bdLobbyConnection::BD_CONNECTED))
	{
		BD_WARN(BD_LOG_LEVEL, "Not connected.");
		return BD_NULL;
	}
	if (!m_teamsManager)
	{
		m_teamsManager = new bdTeams(m_taskManager);
	}

	return m_teamsManager;
}

bdFriends* bdLobbyServiceImpl::getFriends()
{
	if (!m_lobbyConnection || (m_lobbyConnection->getStatus() != bdLobbyConnection::BD_CONNECTED))
	{
		BD_WARN(BD_LOG_LEVEL, "Not connected.");
		return BD_NULL;
	}
	if (!m_friendsManager)
	{
		m_friendsManager = new bdFriends(m_taskManager);
	}

	return m_friendsManager;
}


bdStorage* bdLobbyServiceImpl::getStorage()
{
	if (!m_lobbyConnection || (m_lobbyConnection->getStatus() != bdLobbyConnection::BD_CONNECTED))
	{
		BD_WARN(BD_LOG_LEVEL, "Not connected.");
		return BD_NULL;
	}
	if (!m_storageManager)
	{
		m_storageManager = new bdStorage(m_taskManager);
	}

	return m_storageManager;
}

bdChannelChat* bdLobbyServiceImpl::getChannelChat()
{
	if (!m_lobbyConnection || (m_lobbyConnection->getStatus() != bdLobbyConnection::BD_CONNECTED))
	{
		BD_WARN(BD_LOG_LEVEL, "Not connected.");
		return BD_NULL;
	}
	if (!m_channelChatManager)
	{
		m_channelChatManager = new bdChannelChat(m_taskManager);
	}

	return m_channelChatManager;
}


bdTitleUtilities* bdLobbyServiceImpl::getTitleUtilities()
{
	if (!m_lobbyConnection || (m_lobbyConnection->getStatus() != bdLobbyConnection::BD_CONNECTED))
	{
		BD_WARN(BD_LOG_LEVEL, "Not connected.");
		return BD_NULL;
	}
	if (!m_titleUtilitiesManager)
	{
		m_titleUtilitiesManager = new bdTitleUtilities(m_taskManager);
	}

	return m_titleUtilitiesManager;
}

bdKeyArchive* bdLobbyServiceImpl::getKeyArchive()
{
	if (!m_lobbyConnection || (m_lobbyConnection->getStatus() != bdLobbyConnection::BD_CONNECTED))
	{
		BD_WARN(BD_LOG_LEVEL, "Not connected.");
		return BD_NULL;
	}
	if (!m_keyArchive)
	{
		m_keyArchive = new bdKeyArchive(m_taskManager);
	}

	return m_keyArchive;
}

void bdLobbyServiceImpl::setMessagingMode(bdLobbyServiceImpl::bdMessagingMode messagingMode)
{
	m_messagingMode = messagingMode;
}


bdRemoteTaskManager* bdLobbyServiceImpl::getTaskManager()
{
	return m_taskManager;
}

bdLobbyServiceImpl::bdLobbyServiceImpl()
: m_taskManager(BD_NULL),
  m_profiles(BD_NULL),
  m_messagingMode(BD_RETAIN_PUSH_MESSAGES),
  m_matchMaking(BD_NULL),
  m_statsManager(BD_NULL),
  m_friendsManager(BD_NULL),
  m_teamsManager(BD_NULL),
  m_storageManager(BD_NULL),
  m_channelChatManager(BD_NULL),
  m_titleUtilitiesManager(BD_NULL),
  m_keyArchive(BD_NULL),
  m_titleID(0),
  m_errorCode(BD_NO_ERROR)
{
}


bdLobbyServiceImpl::~bdLobbyServiceImpl()
{
	if (m_taskManager)
	{
		delete m_taskManager;
	}
	if (m_profiles)
	{
		delete m_profiles;
	}
	if (m_matchMaking)
	{
		delete m_matchMaking;
	}
	if (m_statsManager)
	{
		delete m_statsManager;
	}
	if (m_friendsManager)
	{
		delete m_friendsManager;
	}
	if (m_teamsManager)
	{
		delete m_teamsManager;
	}
	if (m_storageManager)
	{
		delete m_storageManager;
	}
	if (m_channelChatManager)
	{
		delete m_channelChatManager;
	}
	if (m_titleUtilitiesManager)
	{
		delete m_titleUtilitiesManager;
	}
	if (m_keyArchive)
	{
		delete m_keyArchive;
	}
}

void bdLobbyServiceImpl::onConnect(bdLobbyConnectionRef connection)
{
	// Send client specifications (protocol version and receive buffer size) as a separate message
	bdByteBuffer byteBuffer(8);

	byteBuffer.write((bdUInt32)BD_PROTOCOL_VERSION);
	byteBuffer.write(connection->getReceiveBufferSize());
	connection->sendRaw(byteBuffer.getData(), 8);


	// Once we are connected we register our title id

	bdUByte8 serviceID = BD_AUTH_SERVICE_ID;
	bdBitBufferRef buffer = new bdBitBuffer(&serviceID, 8, true); //msg->getPayload();
	buffer->setTypeCheck(false);
	buffer->writeBool(true);
	buffer->setTypeCheck(true);

	// All platforms care about the title id
	buffer->writeUInt32(m_authInfo.m_titleID);

//#ifndef BD_PLATFORM_XBOX
//	buffer->writeUInt32(m_authInfo.m_IVSeed);
//#endif

	if (m_encryptedConnection)
	{
		buffer->writeUInt32(m_authInfo.m_IVSeed);
	}
	else
	{
		buffer->writeUInt32(0);
	}
	buffer->writeBits(m_authInfo.m_data, BD_NUM_BYTES_TO_NUM_BITS(BD_AUTH_TICKET_SIZE));

	connection->send(buffer->getData(), buffer->getDataSize(), static_cast<bdBool>(BD_SEND_PLAINTEXT));

#ifndef BD_PLATFORM_XBOX
	connection->setSessionKey(m_authInfo.m_sessionKey);
#endif  //#ifndef BD_PLATFORM_XBOX

    // create a task manager
	m_taskManager = new bdRemoteTaskManager(connection, m_encryptedConnection);
	m_messageStore.setRemoteTaskManager(m_taskManager);

	BD_INFO(BD_LOG_LEVEL, "Connected to MatchMaking Service.");
}


void bdLobbyServiceImpl::onDisconnect(bdLobbyConnectionRef /*connection*/)
{
	BD_INFO(BD_LOG_LEVEL, "Disconnected from MatchMaking Service.");
	m_messageStore.setRemoteTaskManager(BD_NULL);
}

bdLobbyService::bdLobbyService()
{
}
