// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdLobby/bdMatchMaking/bdMatchMakingInfo.h>

#include <bdNet/bdNet.h>

#define BD_LOG_LEVEL "lobby/matchmakinginfo"

bdMatchMakingInfo::bdMatchMakingInfo()
: m_hostAddr(0),
  m_publicOpen(0),
  m_publicFilled(0),
  m_privateOpen(0),
  m_privateFilled(0)
{
	m_hostAddr = new bdCommonAddr();
}

bdMatchMakingInfo::bdMatchMakingInfo(const bdCommonAddrRef hostAddr,
									 const bdUByte8 publicOpen,
									 const bdUByte8 privateOpen,
									 const bdUByte8 publicFilled,
									 const bdUByte8 privateFilled)
: m_hostAddr(hostAddr),
  m_publicOpen(publicOpen),
  m_publicFilled(publicFilled),
  m_privateOpen(privateOpen),
  m_privateFilled(privateFilled)
{
}

bdMatchMakingInfo::~bdMatchMakingInfo()
{
}

void bdMatchMakingInfo::serialize(bdBitBuffer &buffer)
{
	// serialize the hostAddr into a byte array
	bdUByte8 hostAddr[BD_COMMON_ADDR_SERIALIZED_SIZE];
	m_hostAddr->serialize(hostAddr);
	buffer.writeBlob(hostAddr, BD_COMMON_ADDR_SERIALIZED_SIZE);

	// secID
	buffer.writeBlob(m_secID.ab, BD_SECURITY_ID_LENGTH);

	// secKey
	buffer.writeBlob(m_secKey.ab, BD_SECURITY_KEY_LENGTH);

	buffer.writeInt32(m_publicOpen);
	buffer.writeInt32(m_publicFilled);
	buffer.writeInt32(m_privateOpen);
	buffer.writeInt32(m_privateFilled);
}

bdBool bdMatchMakingInfo::deserialize(bdBitBufferRef buffer)
{
	bdUInt length = BD_COMMON_ADDR_SERIALIZED_SIZE;
	bdUByte8 addrBlob[BD_COMMON_ADDR_SERIALIZED_SIZE];

	bdBool ok = buffer->readBlob(addrBlob, length);

	if (length == BD_COMMON_ADDR_SERIALIZED_SIZE)
	{
		/*if (m_hostAddr->m_data == BD_NULL)
		{
			m_hostAddr->m_data = new bdUByte8[BD_COMMON_ADDR_SERIALIZED_SIZE];
		}*/
		ok = ok && m_hostAddr->deserialize(bdNet::getInstance()->getLocalCommonAddr(), addrBlob);
	}
	else if (ok)
	{
		BD_ERR(BD_LOG_LEVEL, "addr returned length != BD_COMMON_ADDR_SERIALIZED_SIZE, ");
		BD_INFO(BD_LOG_LEVEL, "Above error most likely caused by addr being from different version of BitDemon...");
	}

	length = BD_SECURITY_ID_LENGTH;
	ok = ok && buffer->readBlob(m_secID.ab, length);
	length = BD_SECURITY_KEY_LENGTH;
	ok = ok && buffer->readBlob(m_secKey.ab, length);

	bdInt32 intValue = 0;
	ok = ok && buffer->readInt32(intValue);
	m_publicOpen = static_cast<bdUByte8>(intValue);

	ok = ok && buffer->readInt32(intValue);
	m_publicFilled = static_cast<bdUByte8>(intValue);

	ok = ok && buffer->readInt32(intValue);
	m_privateOpen = static_cast<bdUByte8>(intValue);

	ok = ok && buffer->readInt32(intValue);
	m_privateFilled = static_cast<bdUByte8>(intValue);

	if (!ok)
	{
		BD_ERR(BD_LOG_LEVEL, "Deserialization failed");
	}
	return ok;
}

const bdSecurityID& bdMatchMakingInfo::getSecurityID() const
{
	return m_secID;
}
void bdMatchMakingInfo::setSecurityID(const bdSecurityID& secID)
{
	m_secID = secID;
}

const bdSecurityKey& bdMatchMakingInfo::getSecurityKey() const
{
	return m_secKey;
}
void bdMatchMakingInfo::setSecurityKey(const bdSecurityKey& secKey)
{
	m_secKey = secKey;
}

bdCommonAddrRef bdMatchMakingInfo::getHostAddr() const
{
	return m_hostAddr;
}
void bdMatchMakingInfo::setHostAddr(const bdCommonAddrRef hostAddr)
{
	m_hostAddr = hostAddr;
}

bdUByte8 bdMatchMakingInfo::getPublicOpen() const
{
	return m_publicOpen;
}
void bdMatchMakingInfo::setPublicOpen(const bdUByte8 publicOpen)
{
	m_publicOpen = publicOpen;
}

bdUByte8 bdMatchMakingInfo::getPrivateOpen() const
{
	return m_privateOpen;
}
void bdMatchMakingInfo::setPrivateOpen(const bdUByte8 privateOpen)
{
	m_privateOpen = privateOpen;
}

bdUByte8 bdMatchMakingInfo::getPublicFilled() const
{
	return m_publicFilled;
}
void bdMatchMakingInfo::setPublicFilled(const bdUByte8 publicFilled)
{
	m_publicFilled = publicFilled;
}
void bdMatchMakingInfo::setPrivateFilled(const bdUByte8 privateFilled)
{
	m_privateFilled = privateFilled;
}
bdUByte8 bdMatchMakingInfo::getPrivateFilled() const
{
	return m_privateFilled;
}
