// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

// Purpose: This wraps a single user's stats on a single leaderboard.

#ifndef BD_STATS_SUBMISSION_H
#define BD_STATS_SUBMISSION_H

#include <bdCore/bdCore.h>
#include <bdLobby/bdLobbyCommon/bdQueryParams.h>
#include <bdLobby/bdLobbyCommon/bdQueryResult.h>
#include <bdLobby/bdLobbyCommon/bdLobbyServicesCommon.h>
#include <bdLobby/bdStats/bdStatsInfo.h>
#include <bdLobby/bdStats/bdStats.h>

BD_REFERENCE(bdStatsSubmission);

/// Statistics submission for use with bdStats::writeMultipleStats().
/// All fields of this object should be correctly filled.
class bdStatsSubmission
{
	public:
		BD_DECLARE_NEW_AND_DELETE_OPERATORS;

		/// Default constructor.
		bdStatsSubmission();

		/// Default destructor.
		~bdStatsSubmission();

		/// Initializer.
		/// \param leaderboardID	The ID of the leaderboard this entry should be written to.
		/// \param writeType		Write type to apply to this entry
		/// \param statsInfo		An appropriate statistics submission. Make sure to fill in
		///							bdStatsInfo::m_entityID: either a bdOnlineUserID or a bdTeamID,
		///							depending on the type of the leaderboard.
		bdStatsSubmission(const bdLeaderBoardID leaderboardID,
						  const bdStats::bdWriteType writeType,
						  bdStatsInfo* const statsInfo);

		void setLeaderboardID(const bdLeaderBoardID newLeaderboardID);
		bdLeaderBoardID getLeaderboardID() const;

		void setWriteType(const bdStats::bdWriteType newWriteType);
		bdStats::bdWriteType getWriteType() const;

		void setStatsInfo(bdStatsInfo* const newStatsInfo);
		bdStatsInfo* getStatsInfo() const;

		/// Serialization function for use by bdStats class.
		bdBool serialize(bdBitBuffer &buffer);

		/// Deserialization function, not implemented.
		bdBool deserialize(bdBitBufferRef buffer);


	protected:
		bdLeaderBoardID m_leaderboardID;
		bdStats::bdWriteType m_writeType;
		bdStatsInfo* m_statsInfo;
};


#endif // BD_STATS_SUBMISSION_H

