// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

// PURPOSE: Container class for files to be uploaded to or downloaded from the 
//			storage service.

#ifndef BD_LOBBY_FILE_H
#define BD_LOBBY_FILE_H

#include <bdCore/bdCore.h>
#include <bdLobby/bdStorage/bdLobbyFileHeader.h>

BD_REFERENCE(bdLobbyFile);

/// Container class for files to be uploaded to or downloaded from the storage 
/// service.
/// This class inherits accessors and mutators for manipulating the file
/// meta-data.
class bdLobbyFile : public bdLobbyFileHeader
{
	public:
		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		/// Constructor.
		/// If you are preparing a file for upload, use this constructor to 
		/// specify the required size. This constructor allocates memory to 
		/// contain the file data (set later via bdLobbyFile::setData).
		/// \param	size[in]	The size of the file which this instance will
		///						contain.
		bdLobbyFile(bdUInt size);

		/// Destructor.
		~bdLobbyFile();

		/// Deserializes a bit buffer into this instance. This is used when 
		/// a file is received from the storage service.
		/// \param	buffer[in]	A bit buffer containing the file data.
		/// \return	
		///			- True, if deserialization was successful.
		///			- False, if deserialization was unsuccessful.
		bdBool deserialize(bdBitBufferRef buffer);

		/// Set the file contents.
		/// This function copies the specified memory into this file, which will
		/// be deallocated when this instance is deleted.
		/// \param	data[in]	A pointer to the start of the file.
		/// \param	size[in]	The size of the file.
		void setData(bdUByte8* data,
					 bdUInt32 size);

		/// Get a pointer to the file contents.
		/// \return		A pointer to the file contents.
		const void *getData() const;
		
		/// Get a pointer to the file contents.
		/// \return		A pointer to the file contents.
		void *getData();

	protected:

		void *m_data;
		bdBool m_deallocate;
};

#endif // BD_LOBBY_FILE_H
