// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS
#include <bdLobby/bdStorage/bdStorage.h>
#include <bdLobby/bdRemoteTaskManager/bdRemoteTaskManager.h>

#define BD_LOG_LEVEL	("lobby/storage")

bdStorage::bdStorage(bdRemoteTaskManager *const remoteTaskManager)
: m_errorCode(BD_NO_ERROR),
  m_remoteTaskManager(remoteTaskManager)
{
	
}

bdStorage::~bdStorage()
{
	m_remoteTaskManager = BD_NULL;
}


bdRemoteTask *bdStorage::uploadFile(bdUByte8 /*controllerIndex*/, 
									const bdNChar8 *fileName, 
									const bdLobbyFile* fileData,
									bdLobbyFileHeader::bdVisibility fileVisibility, 
									bdLobbyFileHeader::bdOwnerType ownerType)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeBool(ownerType == bdLobbyFileHeader::BD_OWNER_PUBLISHER);
	buffer.writeString(fileName, bdStrnlen(fileName, BD_MAX_FILE_NAME_LENGTH));
	buffer.writeBool(fileVisibility == bdLobbyFileHeader::BD_VISIBLE_PRIVATE);
	buffer.writeBlob(fileData->getData(), fileData->getFileSize());

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
											    BD_STORAGE_SERVICE,
											    BD_STORAGE_UPLOAD,
											    buffer);
	return remoteTask;
}

bdRemoteTask *bdStorage::updateFile(bdUByte8 /*controllerIndex*/, 
									bdUInt64 fileID, 
									bdLobbyFile* fileData)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(fileID);
	buffer.writeBlob(fileData->getData(), fileData->getFileSize());

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_STORAGE_SERVICE,
												BD_STORAGE_UPDATE,
												buffer);

	return remoteTask;
}

bdRemoteTask *bdStorage::updateFileProperties(bdUByte8 /*controllerIndex*/, 
											  bdUInt64 fileID, 
											  const bdNChar8 *fileName, 
											  bdLobbyFileHeader::bdVisibility fileVisibility)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(fileID);
	buffer.writeString(fileName, bdStrnlen(fileName, BD_MAX_FILE_NAME_LENGTH));
	buffer.writeBool(fileVisibility == bdLobbyFileHeader::BD_VISIBLE_PRIVATE);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												 BD_STORAGE_SERVICE,
												 BD_STORAGE_UPDATE_PROPS,
												 buffer);
	return remoteTask;
}

bdRemoteTask *bdStorage::removeFile(bdUByte8 /*controllerIndex*/, 
									bdUInt64 fileID)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(fileID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_STORAGE_SERVICE,
												BD_STORAGE_REMOVE,
												buffer);
	return remoteTask;
}

bdRemoteTask *bdStorage::getFile(bdUByte8 /*controllerIndex*/, 
								 bdUInt64 fileID)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(fileID);

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
											 	 BD_STORAGE_SERVICE,
												 BD_STORAGE_GETFILE,
												 buffer);
	return remoteTask;
}

bdRemoteTask *bdStorage::listFilesByOwner(bdUByte8 /*controllerIndex*/, 
										  bdOnlineUserID ownerID,
										  bdUInt32 startDate,
										  bdUInt16 maxNumResults,
										  const bdNChar8 *fileName)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt64(ownerID);
	buffer.writeUInt32(startDate);
	buffer.writeUInt16(maxNumResults);

	if (fileName)
	{
		buffer.writeString(fileName);
	}

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_STORAGE_SERVICE,
												BD_STORAGE_LISTBYOWNER,
												buffer);

	return remoteTask;
}

bdRemoteTask *bdStorage::listAllPublisherFiles(bdUByte8 /*controllerIndex*/,
											   bdUInt32 startDate,
											   bdUInt16 maxNumResults,
											   const bdNChar8 *fileName)
{
	bdRemoteTask *remoteTask = BD_NULL;

	bdBitBuffer buffer(0, true);
	buffer.writeUByte8(0);
	buffer.writeUInt32(startDate);
	buffer.writeUInt16(maxNumResults);

	if (fileName)
	{
		buffer.writeString(fileName);
	}

	m_errorCode = m_remoteTaskManager->startTask(&remoteTask,
												BD_STORAGE_SERVICE,
												BD_STORAGE_LISTALLPUBFILES,
												buffer);
	return remoteTask;
}

bdBool bdStorage::handleTask(const bdRemoteTask *remoteTask, 
							 bdQueryResult *queryResult)
{
	bdBool ok;
	bdBitBufferRef buffer = remoteTask->getResults();
	bdUInt32 taskErrorCode = BD_MAX_ERROR_CODE;
	ok = buffer->readUInt32(taskErrorCode);
	if (ok)
	{
		m_errorCode = static_cast<bdLobbyErrorCode>(taskErrorCode);
		if (m_errorCode == BD_NO_ERROR)
		{
			bdUByte8 operation; 
			ok = buffer->readUByte8(operation);
			if(ok)
			{
				switch (operation)
				{
					case BD_STORAGE_LISTALLPUBFILES:
					case BD_STORAGE_LISTBYOWNER:
					{
						if (queryResult)
						{
							bdUInt numReplies = 0;
							ok = buffer->readUInt32(numReplies);
							if (ok && numReplies > 0)
							{
								ok = queryResult->handleResult(numReplies, buffer);
							}
						}
					}
					break;
					case BD_STORAGE_UPLOAD:
					case BD_STORAGE_GETFILE:
					{
						if (queryResult)
						{
							ok = queryResult->handleResult(1, buffer);
						}
					}
					break;
					default:
					{
					}
					break;
				}
			}
		}
#if defined BD_DEBUG
		else
		{
			bdUInt64 transactionID;
			bdUInt64 connectionID;

			connectionID = m_remoteTaskManager->getConnectionID();
			transactionID = remoteTask->getTransactionID();

			// here we originally called bdRemoteTaskManager::obtainTransactionID
			BD_WARN(BD_LOG_LEVEL, "Transaction ID " BD_UINT_64_FORMAT ":" BD_UINT_64_FORMAT " failed.",
				connectionID, transactionID);
		}
#endif // BD_DEBUG		
	}
	return (m_errorCode == BD_NO_ERROR && ok);
}

bdLobbyErrorCode bdStorage::getErrorCode() const
{
	return m_errorCode;
}
