// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: PS2
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdCore/bdCore.h>
#include <bdCore/bdThread/bdThread.h>
#include <bdNet/bdGetHostByName.h>
#include <bdPlatform/bdPlatformSocket/bdPlatformSocket.h>

bdGetHostByName::bdGetHostByName()
:m_status(BD_READY),
 m_timeout(0),
 m_numAddresses(0)
{
	m_timer.reset();
	m_thread = new bdThread(this);
}

bdGetHostByName::~bdGetHostByName()
{
	bdInt threadID = m_thread->getThreadHandle();
	m_thread->cleanup();
	bdPlatformSocket::unregisterThread(threadID);
}

bdBool bdGetHostByName::start(const bdNChar8 *hostname, bdFloat32 timeout)
{
	m_timeout = timeout;
	bdBool success = false;
	if (m_status != BD_PENDING)
	{	
		m_status = BD_PENDING;
		m_timer.start();
		success = m_thread->start((char*)hostname, bdStrlen(hostname)+1);
		if (!success)
		{
			m_status = BD_FAILED;
		}
	}
	return success;
}

void bdGetHostByName::cancel()
{
	if (m_status == BD_PENDING)
	{
		bdPlatformSocket::abortGetHostByName();
	}
}

bdGetHostByName::bdStatus  bdGetHostByName::getStatus()
{
	if (m_status == BD_PENDING)
	{
		if (m_timer.getElapsedTimeInSeconds() > m_timeout)
		{
			cancel();
//			m_status = BD_READY;
		}
	}
	return m_status;
}

bdUInt bdGetHostByName::getNumAddresses()
{
	return m_numAddresses;
}

bdInAddr bdGetHostByName::getAddressAt(bdUInt index)
{
	return m_adresses[index];
}

bdUInt bdGetHostByName::run(void* args)
{
	m_numAddresses = bdPlatformSocket::getHostByName((char*)args, m_adresses, BD_MAX_NUM_ADDRESSES);
	if (m_numAddresses > 0)
	{
		m_status = BD_READY;
	}
	else
	{
		m_status = BD_FAILED;
	}
	return 0;
}
