// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

/// Purpose: A platform independent wrapper for performing Domain Name Server lookups

#ifndef BD_GET_HOST_BY_NAME_H
#define BD_GET_HOST_BY_NAME_H

#include <bdCore/bdSocket/bdInetAddr.h>
#include <bdCore/bdTiming/bdStopwatch.h>
#include <bdCore/bdThread/bdThread.h>

#define BD_MAX_NUM_ADDRESSES (4u)

class bdGetHostByName : public bdRunnable
{
	public:

		enum bdStatus
		{
			BD_READY,
			BD_PENDING,
			BD_FAILED
		};

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		bdGetHostByName();

		~bdGetHostByName();

		/// Starts the DNS lookup. This call creates a 
		/// separate worker thread, call getStatus() to retrieve the status
		/// of the name lookup
		/// \param hostname[in] The name of the host that needs to be resolved
		/// \param timeout[in] The time in seconds after which this request will timeout
		/// \return - True: success
		///			- False: failed
		bdBool start(const bdNChar8 *hostname, bdFloat32 timeout);

		/// Cancels a progressing name lookup
		void cancel();

		/// Gets the current status of the name lookup
		bdGetHostByName::bdStatus  getStatus();

		/// Gets the actual number of IP addresses that were returned by the DNS server
		/// \return The number of IP addresses returned by the DNS server
		bdUInt getNumAddresses();

		/// Gets a specific address from the result set
		/// \param index[in]: The index of the address to be returned (0 .. getNumAddresses() -1)
		/// \return  the InAddr of the specified address
		bdInAddr getAddressAt(bdUInt index);

	protected:

		virtual bdUInt run(void* args);

		bdStopwatch m_timer;

		bdStatus m_status;

		bdThread *m_thread;

		bdFloat32 m_timeout;

		bdInAddr m_adresses[BD_MAX_NUM_ADDRESSES];

		bdUInt m_numAddresses;

		bdDNSHandle m_handle;

};

#endif  //BD_GET_HOST_BY_NAME_H
