// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: WIN32 PS2 UNIX OSX PSP-INFRA PS3
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdSocket/bdAddressMap.h>

BD_REGISTER_SINGLETON(bdAddressMapImpl)

#define BD_LOG_LEVEL "bdSocket/addrmap"

bdAddressMapImpl::bdAddressMapImpl()
{
}

bdBool bdAddressMapImpl::commonAddrToAddr(bdCommonAddrRef ca, const bdSecurityID &id, bdAddrHandleRef &addrHandle)
{
	bdAddrHandleRef newHandle;

	bdCommonAddrRefWrapper wrapper;
	wrapper.m_ca = ca;

	if(!m_caToAddr.get(wrapper, newHandle))
	{
		newHandle = new bdAddrHandle(ca, id);

		m_caToAddr.put(wrapper, newHandle);
	}
	else if(newHandle->getStatus() != bdAddrHandle::BD_ADDR_RESOLVED)
	{
		newHandle->setStatus(bdAddrHandle::BD_ADDR_NOT_RESOLVED);
	}

	// TODO: verify this check, the sum could by chance equal zero
	// check the all bdSecurityID are initialised with zeros.
	// Would a memcmp or a newHandle->m_secID == bdSecurityID(); be simpler?
#if defined(BD_DEBUG)
	bdUInt val = 0;
	for(bdUInt i = 0; i < sizeof(bdSecurityID); i++)
	{
		val += newHandle->m_secID.ab[i];
	}
	BD_ASSERT(val > 0, "Invalid security ID");
#endif

	addrHandle = newHandle;

	return true;
}

bdBool bdAddressMapImpl::addrToCommonAddr(const bdAddrHandleRef &addrHandle, bdCommonAddrRef &ca, bdSecurityID &id) const
{
	bdBool success = true;

	ca = addrHandle->m_commonAddr;
	bdMemcpy(id.ab, addrHandle->m_secID.ab, BD_SECURITY_ID_LENGTH);

	if(ca.isNull())
	{
		success = false;

		BD_ERR(BD_LOG_LEVEL, "Failed to get common addr from addr handle.");
	}

	return success;
}

bdBool bdAddressMapImpl::unregisterAddr(bdAddrHandleRef &addrHandle)
{
	// remove common address from map
	bdCommonAddrRefWrapper wrapper;
	wrapper.m_ca = addrHandle->m_commonAddr;
	m_caToAddr.remove(wrapper);

	// set the handle to unresolved.
	addrHandle->m_commonAddr = BD_NULL;
	addrHandle->m_status = bdAddrHandle::BD_ADDR_UNRESOLVED;
	bdMemset(addrHandle->m_secID.ab, 0, BD_SECURITY_ID_LENGTH);

	return true;
}

bdBool bdAddressMapImpl::getTitleCommonAddr(bdCommonAddrRef &ca)
{
	ca = m_me;
	return m_me.notNull();
}

void bdAddressMapImpl::setTitleCommonAddr(bdCommonAddrRef ca)
{
	m_me = ca;
}

bdUWord bdAddressMapImpl::addrToString(const bdAddrHandleRef &addrHandle, 
									   bdNChar8 *const str, 
									   const bdUWord size) const
{
	bdUWord strLength = 0;

	if(addrHandle->m_status == bdAddrHandle::BD_ADDR_RESOLVED)
	{
		strLength = addrHandle->m_realAddr.toString(str, size);
	}
	else
	{
		// check if it's loopback
		if(addrHandle->m_commonAddr && addrHandle->m_commonAddr->isLoopback()) 	
		{
			const bdNChar8 *const loopbackStr = "loopback";
			bdStrlcpy(str, loopbackStr, size);

			// return the desired string length (even is strLength > size)
			strLength = bdStrlen(loopbackStr);
		}
		else
		{
			const bdNChar8 *const notResolvedStr = "not resolved";
			bdStrlcpy(str, notResolvedStr, size);

			// return the desired string length (even is strLength > size)
			strLength = bdStrlen(notResolvedStr);			
		}
	}

	return strLength;
}

bdBool bdAddressMapImpl::getAddressMapping(const bdAddr &realAddr, bdAddrHandleRef &addrHandle) const
{
	bdBool ok = false;

	bdHashMap<bdCommonAddrRefWrapper, bdAddrHandleRef, bdCommonAddrRefWrapper>::Iterator it = m_caToAddr.getIterator();

	while(it && !ok)
	{
		const bdAddrHandleRef &tmpAddrHandle = m_caToAddr.getValue(it);

		if(tmpAddrHandle->getRealAddr() == realAddr)
		{
			ok = true;
			addrHandle = tmpAddrHandle;
		}

		m_caToAddr.next(it);
	}

	m_caToAddr.releaseIterator(it);

	return ok;
}

void bdAddressMapImpl::remove(bdCommonAddrRef ca)
{
	// remove common address from map
	bdCommonAddrRefWrapper wrapper;
	wrapper.m_ca = ca;
	m_caToAddr.remove(wrapper);
}

void bdAddressMapImpl::clear()
{
	m_caToAddr.clear();
}
