// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: WIN32 PS2 UNIX OSX PSP-INFRA PS3
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#include <bdSocket/bdDTLS/bdPacket/bdDTLSCookieEcho.h>

bdDTLSCookieEcho::bdDTLSCookieEcho()
{
}

bdDTLSCookieEcho::bdDTLSCookieEcho(bdUInt16 vtag,
								   const bdDTLSInitAck &cookie,
								   const bdUByte8 ca[BD_COMMON_ADDR_SERIALIZED_SIZE],
								   const bdUByte8 secID[BD_SECURITY_ID_LENGTH],
								   const bdUByte8 key[BD_ECC_EXPORTED_KEY_SIZE])
: bdDTLSHeader(BD_DTLS_COOKIE_ECHO, vtag, 0),
m_cookie(cookie)
{
	bdMemcpy(m_ca, ca, BD_COMMON_ADDR_SERIALIZED_SIZE);
	bdMemcpy(m_secID, secID, BD_SECURITY_ID_LENGTH);
	bdMemcpy(m_ECCKey, key, BD_ECC_EXPORTED_KEY_SIZE);
}

bdDTLSCookieEcho::~bdDTLSCookieEcho()
{
}

bdBool bdDTLSCookieEcho::serialize(void *data,
								   const bdUInt size,
								   const bdUInt offset,
								   bdUInt &newOffset) const
{
	bdBool status = true;
	newOffset = offset;

	status = status && bdDTLSHeader::serialize(data, size, newOffset, newOffset);
	status = status && m_cookie.serialize(data, size, newOffset, newOffset);
	status = status && bdBytePacker::appendBuffer(data, size, newOffset,
			newOffset, m_ca, BD_COMMON_ADDR_SERIALIZED_SIZE);
	status = status && bdBytePacker::appendBuffer(data, size, newOffset,
			newOffset, m_secID, BD_SECURITY_ID_LENGTH);
	status = status && bdBytePacker::appendBuffer(data, size, newOffset,
			newOffset, m_ECCKey, BD_ECC_EXPORTED_KEY_SIZE);

	if (!status)
	{
		newOffset = offset;
	}
	return status;
}

bdBool bdDTLSCookieEcho::deserialize(const void* data,
									 const bdUInt size,
									 const bdUInt offset,
									 bdUInt &newOffset)
{
	bdBool status = true;
	newOffset = offset;

	status = status && bdDTLSHeader::deserialize(data, size, newOffset, newOffset);
	status = status && m_cookie.deserialize(data, size, newOffset, newOffset);
	status = status && bdBytePacker::removeBuffer(data, size, newOffset,
			newOffset, m_ca, BD_COMMON_ADDR_SERIALIZED_SIZE);
	status = status && bdBytePacker::removeBuffer(data, size, newOffset,
			newOffset, m_secID, BD_SECURITY_ID_LENGTH);
	status = status && bdBytePacker::removeBuffer(data, size, newOffset,
			newOffset, m_ECCKey, BD_ECC_EXPORTED_KEY_SIZE);

	if (!status)
	{
		newOffset = offset;
	}
	return status;
}

const bdDTLSInitAck& bdDTLSCookieEcho::getCookie() const
{
	return m_cookie;
}

const bdUByte8* bdDTLSCookieEcho::getCa() const
{
	return m_ca;
}

const bdUByte8*  bdDTLSCookieEcho::getSecID() const
{
	return m_secID;
}

const bdUByte8*  bdDTLSCookieEcho::getECCKey() const
{
	return m_ECCKey;
}


