// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: WIN32 PS2 UNIX OSX PSP-INFRA PS3
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

// PURPOSE: A class that implements client functionality of the STUN protocol
// (Simple Traversal of UDP Through NATs).

#ifndef BD_NAT_TYPE_DISCOVERY_CLIENT_H
#define BD_NAT_TYPE_DISCOVERY_CLIENT_H

#include <bdCore/bdCore.h>

class bdNATTypeDiscoveryPacketReply;

/// A class that implements client functionality of the STUN protocol
/// (Simple Traversal of UDP Through NATs).
class bdNATTypeDiscoveryClient
{

	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		bdNATTypeDiscoveryClient(const bdInetAddr &clientAddr, const bdAddr &serverAddr);

		/// Virtual destructor
		virtual ~bdNATTypeDiscoveryClient();

		bdBool startDiscovery();

		bdBool isPending() const;

		bdBool wasSuccess() const;

		/// Return a descriptor describing the discovered NAT.
		/// \return a descriptor describing the discovered NAT.
		const bdNATType& getNATType() const;

		void pump();

	protected:

		/// Empty default constructor for the benefit of child classes.
		bdNATTypeDiscoveryClient();

		void sendForTest1();

		void sendForTest2();

		void sendForTest3();

		void handleResponse(const bdAddr &addr, const bdNATTypeDiscoveryPacketReply &reply);

	protected:

		enum bdNATTypeDiscoveryClientState
		{
			BD_NTDCS_NOT_RUNNING = 0,
			BD_NTDCS_RUN_TEST_1,
			BD_NTDCS_RUN_TEST_2,
			BD_NTDCS_RUN_TEST_3,
			BD_NTDCS_FINI
		};

	protected:

		bdDefaultSocket* m_socket;

		// The address at which the STUN server is located
		bdAddr m_serverAddr1;

		// The alternate address used by the STUN server
		bdAddr m_serverAddr2;

		bdAddr m_mappedAddr;

		bdNATType m_NATtype;

		bdBool m_success;

		bdNATTypeDiscoveryClientState m_state;

		bdUByte8 m_resends;

		bdStopwatch m_timer;
};


#endif // BD_NAT_TYPE_DISCOVERY_CLIENT_H
