// DWBS ------------------------------------------------------------------ DWBS
//
// PLATFORM		: ALL
// PRODUCT		: BIT_DEMON
// VISIBILITY	: PUBLIC
//
// DWBS ------------------------------------------------------------------ DWBS

#ifndef BD_DW_SOCKET_CONFIG_H
#define BD_DW_SOCKET_CONFIG_H

#include <bdCore/bdCore.h>

enum bdSocketStatus
{
	BD_SOCKET_IDLE,
	BD_SOCKET_PENDING,
	BD_SOCKET_CONNECTED,
	BD_SOCKET_LOST
};

/// Timeouts for the socket-layer association
#if defined BD_PLATFORM_XBOX
	// On XBox, underlying security associations timeout at around 2 minutes.
	#define	BD_SOCKET_TIMEOUT		(120.0f)
#elif defined BD_PLATFORM_PSP_ADHOC
	// On PSP Adhoc, underlying encrypted transport is not connection-oriented.
	#define BD_SOCKET_TIMEOUT		(60.0f * 60.0f * 24.0f)
#else // !XBOX && !PSP_ADHOC
	// On PC, set timeout to 30 seconds in release. To aid debugging, set to 30
	// minutes in debug.
	#if defined BD_RELEASE
		// The association timeout is 30 seconds
		#define BD_SOCKET_TIMEOUT	(30.f)
	#else
		// The association timeout is 30 minutes
		#define BD_SOCKET_TIMEOUT	(30.f * 60.f)
	#endif // RELEASE
#endif // !XBOX && !PSP_ADHOC

#if defined BD_PLATFORM_PSP_ADHOC

typedef bdAddr bdAddrHandle;
typedef bdAddrHandle bdAddrHandleRef; 

class bdAddrHandleHashmapWrapper
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		inline bdAddrHandleHashmapWrapper()
		{
		};

		inline bdAddrHandleHashmapWrapper(bdAddrHandleRef handle)
		: m_handle(handle)
		{
		}

		bdBool operator == (const bdAddrHandleHashmapWrapper &other) const
		{
			return m_handle.getHash() == other.m_handle.getHash();
		}

		inline bdUInt getHash(const bdAddrHandleHashmapWrapper &addr) const 
		{
			return addr.m_handle.getHash();
		}

		bdAddrHandleRef m_handle;
};


#elif !defined BD_PLATFORM_XBOX

BD_REFERENCE(bdAddrHandle);

class bdAddrHandle : public bdReferencable
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS;

		bdAddrHandle()
		: m_status(BD_ADDR_NOT_CONSTRUCTED)
		{
		}

		bdAddrHandle(bdCommonAddrRef ca, const bdSecurityID &secID)
		: m_status(BD_ADDR_NOT_RESOLVED),
		  m_commonAddr(ca)
		{
			bdMemcpy(m_secID.ab, secID.ab, BD_SECURITY_ID_LENGTH);

			m_handle = bdTrulyRandom::getInstance()->getRandomUInt();
		}

		enum bdAddrHandleStatus
		{
			BD_ADDR_NOT_CONSTRUCTED,
			BD_ADDR_NOT_RESOLVED,
			BD_ADDR_ERROR,
			BD_ADDR_RESOLVED,
			BD_ADDR_UNRESOLVED
		};

		bdBool operator == (const bdAddrHandle &other) const
		{
			return m_handle == other.m_handle;
		}

		void setRealAddr(const bdAddr &realAddr)
		{
			m_realAddr = realAddr;
			m_status = BD_ADDR_RESOLVED;
		}

		const bdAddr &getRealAddr() const
		{
			return m_realAddr;
		}

		void setStatus(const bdAddrHandleStatus &status)
		{
			m_status = status;
		}

		bdAddrHandleStatus getStatus()
		{
			return m_status;
		}

		bdAddrHandleStatus m_status;

		bdAddr m_realAddr;

		bdCommonAddrRef m_commonAddr;

		bdSecurityID m_secID;

		bdUInt32 m_handle;

	protected:
		
};

class bdAddrHandleHashmapWrapper
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		inline bdAddrHandleHashmapWrapper()
		{
		};

		inline bdAddrHandleHashmapWrapper(bdAddrHandleRef handle)
		: m_handle(handle)
		{
		}

		bdBool operator == (const bdAddrHandleHashmapWrapper &other) const
		{
			return m_handle->m_handle == (other.m_handle->m_handle);
		}

		inline bdUInt getHash(const bdAddrHandleHashmapWrapper &addr) const 
		{
			return addr.m_handle->m_handle;
		}

		bdAddrHandleRef m_handle;
};

#else // xbox/xenon

BD_REFERENCE(bdAddrHandle);

class bdAddrHandle : public bdReferencable
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		bdBool operator == (const bdAddrHandleRef &other) const
		{
			return m_addr.s_addr == other->m_addr.s_addr;
		}

		in_addr m_addr;

		bdPort m_port;
};

class bdAddrHandleHashmapWrapper
{
	public:

		BD_DECLARE_NEW_AND_DELETE_OPERATORS

		inline bdAddrHandleHashmapWrapper()
		{
		};

		inline bdAddrHandleHashmapWrapper(bdAddrHandleRef handle)
		: m_handle(handle)
		{
		}

		bdBool operator == (const bdAddrHandleHashmapWrapper &other) const
		{
			return m_handle->operator ==(other.m_handle);
		}

		inline bdUInt getHash(const bdAddrHandleHashmapWrapper &addr) const 
		{
			return addr.m_handle->m_addr.s_addr;
		}

		bdAddrHandleRef m_handle;
};

#endif

#endif // BD_DW_SOCKET_CONFIG_H
