

//-----------------------------------------------------------------------------
//            shellDebug - simple scalable font and profiler bars
//
// Header file
//
// $Workfile: shellDebug.h $
// $Author: Mdean $
// $Date: 6/11/03 11:08 $
// $Revision: 4 $
//
// Copyright (c) 2002-2003 Sony Computer Entertainment Europe.
// All Rights Reserved.
//
//-----------------------------------------------------------------------------

#ifndef _SHELLDEBUG_H
#define _SHELLDEBUG_H

//-----------------------------------------------------------------------------
// T Y P E D E F S
//-----------------------------------------------------------------------------

typedef enum 
{
	SHELLDEBUG_ALIGN_LEFT         = 0, 
	SHELLDEBUG_ALIGN_RIGHT        = 1, 
	SHELLDEBUG_ALIGN_CENTRE       = 2,
	SHELLDEBUG_ALIGN_LEFT_BELOW   = 4,
	SHELLDEBUG_ALIGN_RIGHT_BELOW  = 5,
	SHELLDEBUG_ALIGN_CENTRE_BELOW = 6
} EShellDebugTextAlign;

typedef enum
{
	SHELLDEBUG_BLACK   = 128,
	SHELLDEBUG_BLUE    = 129,
	SHELLDEBUG_RED     = 130,
	SHELLDEBUG_MAGENTA = 131,
	SHELLDEBUG_GREEN   = 132,
	SHELLDEBUG_CYAN    = 133,
	SHELLDEBUG_YELLOW  = 134,
	SHELLDEBUG_WHITE   = 135
} EShellDebugTextColour;

typedef enum
{
	SHELLDEBUG_BAR = 0,
	SHELLDEBUG_PIE = 1,
	SHELLDEBUG_BAR_AND_TEXT = 2,
	SHELLDEBUG_PIE_AND_TEXT = 3
} EShellDebugProfilerType;

typedef struct _ShellDebugProfile
{
	float lastValue;
	float scale;
	float x;
	float y;
	u_int z;
	float w;
	float h;
	EShellDebugProfilerType iType;
	u_long bgColour;
	u_long fontColour;
	float  fontScaleX;
	float  fontScaleY;
} ShellDebugProfile;

typedef struct _ShellDebugText
{
	int   currentX;
	int   currentY;
	float coordScaleX;
	float coordScaleY;
	float drawOffsetX;
	float drawOffsetY;
	float fontScaleX;
	float fontScaleY;
	float charsWide;
	float charsHigh;
	float windowW;
	float windowH;
	u_long colour;
	u_int z;
} ShellDebugFontInfo;

typedef struct _ShellDebugConsoleWin
{
	char pText[65536];
	int   numLines; 
	int   column;
	ShellDebugFontInfo *pTextInfo;
} ShellDebugConsole;


//-----------------------------------------------------------------------------
// P R O T O T Y P E S
//-----------------------------------------------------------------------------

#if defined(__LANGUAGE_C_PLUS_PLUS)||defined(__cplusplus)||defined(c_plusplus)
extern "C" {
#endif

void shellDebugInit(int tex_addr, int clut_addr);
void shellDebugSelectFontTexture(int transparentBG);

// Basic debug font
void shellDebugFontInit(ShellDebugFontInfo *pInfo, int windowW, int windowH, int offsetX, int offsetY, u_int z );
void shellDebugFontSetCoordScale(ShellDebugFontInfo *pInfo, float coordScaleX, float coordScaleY);
void shellDebugFontSetSize(ShellDebugFontInfo *pInfo, float fontScaleX, float fontScaleY);
void shellDebugFontPrintAt(ShellDebugFontInfo *pInfo, float x, float y, EShellDebugTextAlign align, char const* const string, ... );
void shellDebugFontDrawBg(ShellDebugFontInfo *pInfo, u_long colour);
void shellDebugFontSetColour(ShellDebugFontInfo *pInfo, u_long colour);

// Console text (uses debug font)
void shellDebugConsoleInit(ShellDebugConsole *pConsole, ShellDebugFontInfo *pTexInfo);
void shellDebugConsoleClear(ShellDebugConsole *pConsole);
void shellDebugConsolePrintf(ShellDebugConsole *pConsole, char const* const string, ...);
void shellDebugConsoleScroll(ShellDebugConsole *pConsole, int numLines);
void shellDebugConsoleDraw(ShellDebugConsole *pConsole);

// Profiler bars
void shellDebugProfilerInit(ShellDebugProfile *pProfile, 
						   EShellDebugProfilerType type, float scale,
						   int x, int y, u_int z, 
						   int width, int height, 
						   u_long bgColour);
void shellDebugProfilerSetFont(ShellDebugProfile *pProfile, float scaleX, float scaleY, u_long colour);
void shellDebugProfilerReset(ShellDebugProfile *pProfile);
void shellDebugProfilerAddValue(ShellDebugProfile *pProfile, float value, u_long colour);

// VRAM viewer
void shellDebugViewVRAM(int y, int width, int pixelFormat, u_char alpha);
void shellDebugClearVRAM(int format, u_char r, u_char g, u_char b, u_char a);


#if defined(__LANGUAGE_C_PLUS_PLUS)||defined(__cplusplus)||defined(c_plusplus)
}
#endif

#endif // _SHELLDEBUG_H
