///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//#include "../gsCommon.h"
#include "../gsPlatformSocket.h"


///////////////////////////////////////////////////////////////////////////////
#include "../../nonport.h"

// C O N S T A N T S /////////////////////////////////////////////////////////
#define uMAX_DNS_NAME_LEN (123)
#define uMAX_CACHED_ENTRIES (16)	// (123+1+4)*16 = 2kbytes

// T Y P E S /////////////////////////////////////////////////////////////////
typedef struct
{
	char acName[uMAX_DNS_NAME_LEN + 1];
	u_int uIP;
} CachedDNSEntry;

// G L O B A L S /////////////////////////////////////////////////////////////
static CachedDNSEntry g_axDNSEntries[uMAX_CACHED_ENTRIES];
static u_int g_uNumCachedDNSEntries;
static struct hostent g_xHostEntReturn;
static u_int g_uIPReturn;
static char* g_pcAddressReturn[2] = { (char*)(&g_uIPReturn), 0 };
static u_int g_bPrintCachedDNSAddresses = 0;

// F U N C T I O N S ///////////////////////////////////////////////////////// 
struct hostent* rebGetHostByName( const char* szHost )
{
	u_int uCachedEntry = 0xffffffff;
	struct hostent* pxHostEnt = NULL;
	u_int u;

	// check for bad parameter
	if (!szHost)
	{
		return NULL;
	}

	// check cached entries - DNS is case-insensitive
	for (u = 0; u < g_uNumCachedDNSEntries; ++u)
	{
		if ( 0 == stricmp( szHost, g_axDNSEntries[u].acName ) )
		{
			uCachedEntry = u;
			break;
		}
	}


	if (0xffffffff == uCachedEntry)
	{
		// entry wasn't cached - do a lookup.
		pxHostEnt = gsSocketGetHostByName( szHost );
		if (!pxHostEnt)
		{
			return NULL;
		}

		if (g_uNumCachedDNSEntries < uMAX_CACHED_ENTRIES)
		{
			// cache this entry
			uCachedEntry = g_uNumCachedDNSEntries++;

			strncpy( g_axDNSEntries[uCachedEntry].acName, szHost, uMAX_DNS_NAME_LEN );
			g_axDNSEntries[uCachedEntry].uIP = *(u_int*)(pxHostEnt->h_addr_list[0]);

			if (g_bPrintCachedDNSAddresses)
			{
				u_int uIP = g_axDNSEntries[uCachedEntry].uIP;
				sceKernelPrintf("cached IP address for %s (%d.%d.%d.%d)\n", szHost, ((uIP >> 0) & 0xff), ((uIP >> 8) & 0xff), ((uIP >> 16) & 0xff), ((uIP >> 24) & 0xff) );
			}
		}
	}


	if (uCachedEntry < uMAX_CACHED_ENTRIES)
	{
		// set return value
		g_uIPReturn = g_axDNSEntries[uCachedEntry].uIP;

		// fill out returned struct
		g_xHostEntReturn.h_name = g_axDNSEntries[uCachedEntry].acName;
		g_xHostEntReturn.h_aliases = NULL;
		g_xHostEntReturn.h_addrtype = AF_INET;
		g_xHostEntReturn.h_length = 4;
		g_xHostEntReturn.h_addr_list = g_pcAddressReturn;
		return &g_xHostEntReturn;
	}

	return pxHostEnt;
}
///////////////////////////////////////////////////////////////////////////////
#define MAX_IPS  5
struct hostent* gsSocketGetHostByName(const char* name)
{
	int result = 0;
	int resolverID = 0;
	char buf[1024]; // PSP documentation recommends 1024
	struct in_addr ip;
	
	static struct hostent ahostent;
	static char * aliases = NULL;
	static char * ipPtrs[MAX_IPS + 1];
	static unsigned int ips[MAX_IPS];

	ahostent.h_name = "";
	ahostent.h_aliases = &aliases;
	ahostent.h_addrtype = AF_INET;
	ahostent.h_addr_list = ipPtrs;
	
	result = sceNetResolverCreate(&resolverID, buf, sizeof(buf));
	if (result < 0)
		return NULL;  

	// this will block until completed
	result = sceNetResolverStartNtoA(resolverID, name, &ip, GSI_RESOLVER_TIMEOUT, GSI_RESOLVER_RETRY);
	sceNetResolverDelete(resolverID); // delete right away, result is stored in ip
	if (result < 0)
		return NULL;

	ahostent.h_length = sizeof(struct in_addr);
	memcpy(&ips[0], &ip, sizeof(struct in_addr));
	ahostent.h_addr_list[0] = (char*)&ips[0];
	ahostent.h_addr_list[1] = NULL;

	gsDebugFormat(GSIDebugCat_Common, GSIDebugType_Network, GSIDebugLevel_Notice,
		"gsSocketGetHostByName: %s = %s\r\n", name, sceNetInetInetNtop(SCE_NET_INET_AF_INET, &ip, buf, sizeof(buf)));
	
	return &ahostent;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
const char* gsSocketInetNtoa(struct in_addr in)
{
	static char buf[sizeof("XXX.XXX.XXX.XXX")];
	sceNetInetInetNtop(SCE_NET_INET_AF_INET, &in, buf, sizeof(buf));
	return buf;
}

// (MH - 07.11.2008) added from 2.06 SDKs
HOSTENT * getlocalhost(void)
{
#define MAX_IPS  5
	static HOSTENT localhost;
	static char * aliases = NULL;
	static char * ipPtrs[MAX_IPS + 1];
	static unsigned int ips[MAX_IPS];
	int result = 0;
	gsi_u32 addr;

	static union SceNetApctlInfo info;

	localhost.h_name = "localhost";
	localhost.h_aliases = &aliases;
	localhost.h_addrtype = AF_INET;
	localhost.h_length = 0;
	localhost.h_addr_list = (char **)ipPtrs;
	ipPtrs[0] = (char *)&ips[0];
	ipPtrs[1] = NULL;
	ips[0] = 0;

	result = sceNetApctlGetInfo(SCE_NET_APCTL_INFO_IP_ADDRESS, &info);
	if (result < 0)
	{
		printf("getlocalhost sceNetApctlGetInfo returned %d\r\n", result);
		return NULL;
	}

	// fill in the hostent structure
	addr = inet_addr(info.ip_address); // NBO
	memcpy(&ips[0], &addr, sizeof(addr)); // still NBO
	localhost.h_length = (gsi_u16)sizeof(addr);

	return &localhost;
}

