
#ifndef _MMessage
#define _MMessage
//
//-
// ==========================================================================
// Copyright (C) 1995 - 2006 Autodesk, Inc., and/or its licensors.  All 
// rights reserved.
// 
// The coded instructions, statements, computer programs, and/or related 
// material (collectively the "Data") in these files contain unpublished 
// information proprietary to Autodesk, Inc. ("Autodesk") and/or its 
// licensors,  which is protected by U.S. and Canadian federal copyright law 
// and by international treaties.
// 
// The Data may not be disclosed or distributed to third parties or be 
// copied or duplicated, in whole or in part, without the prior written 
// consent of Autodesk.
// 
// The copyright notices in the Software and this entire statement, 
// including the above license grant, this restriction and the following 
// disclaimer, must be included in all copies of the Software, in whole 
// or in part, and all derivative works of the Software, unless such copies 
// or derivative works are solely in the form of machine-executable object 
// code generated by a source language processor.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND. 
// AUTODESK DOES NOT MAKE AND HEREBY DISCLAIMS ANY EXPRESS OR IMPLIED 
// WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
// NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, 
// OR ARISING FROM A COURSE OF DEALING, USAGE, OR TRADE PRACTICE. IN NO 
// EVENT WILL AUTODESK AND/OR ITS LICENSORS BE LIABLE FOR ANY LOST 
// REVENUES, DATA, OR PROFITS, OR SPECIAL, DIRECT, INDIRECT, OR 
// CONSEQUENTIAL DAMAGES, EVEN IF AUTODESK AND/OR ITS LICENSORS HAS 
// BEEN ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES. 
// ==========================================================================
//+
//
// CLASS:    MMessage
//
// *****************************************************************************
//
// CLASS DESCRIPTION (MMessage)
//
// This is the base class for message callbacks.
// 
// There are three types of messages in maya that are exposed through the
// API and they are the following:
// scene messages, dependency graph messages, and dependency node messages.
// can be registered for these messages using the addCallback
// methods of the MSceneMessage, MDGMessage, and MNodeMessage classes.
// 
// When a callback is added a number or id is returned. This id is used to
// keep track of the callback and is necessary for removing it.
// A callback id with the value 'NULL' represents an invalid callback.
// Use the removeCallback member function of this class for removing a
// callback.
//
// It is the user's responsibility to keep track of the callback id's
// and remove all callbacks for a plug-in when it is unloaded.
//
// *****************************************************************************

#if defined __cplusplus

// *****************************************************************************

// INCLUDED HEADER FILES


#include <maya/MStatus.h>

// *****************************************************************************

// DECLARATIONS

typedef MUintPtrSz MCallbackId;

// Declaration for the callback table
typedef struct MMessageNode {
	void* fClientPtr;
	void* fServerPtr;
	void* fSubClientPtr;
	MCallbackId fId;
	bool isValid;	// Unused
} * MMessageNodePtr;

// *****************************************************************************

// CLASS DECLARATION (MMessage)

class MIntArray;
class MObject;
class MCallbackIdArray;
class MFileObject;
class MUintArray;
class MString;
class MPlug;
class MDagPath;
class MObject;
class MTime;
class MDGModifier;
class MObjectArray;

/// Message base class. (OpenMaya) (OpenMaya.py)
/**
*/
#ifdef _WIN32
#pragma warning(disable: 4522)
#endif // _WIN32

class OPENMAYA_EXPORT MMessage
{ 
public:	
	// Function pointer typedefs
	typedef void (*MBasicFunction)( void* clientData );
	typedef void (*MElapsedTimeFunction)( float elapsedTime, float lastTime, void* clientData );
	typedef void (*MCheckFunction)( bool* retCode, void* clientData );
	typedef void (*MCheckFileFunction)( bool* retCode, MFileObject& file, void* clientData );
	typedef void (*MComponentFunction)(MUintArray componentIds[], unsigned int count,void *clientData);
	typedef void (*MNodeFunction)(MObject& node,void *clientData);
	typedef void (*MStringFunction)( const MString &,void* clientData );
	typedef void (*MStateFunction)( bool state,void* clientData );
	typedef void (*MTimeFunction)( MTime& time,void* clientData );
	typedef void (*MPlugFunction)( MPlug& srcPlug, MPlug& destPlug,bool made,void* clientData );
	typedef void (*MNodePlugFunction)( MObject& node,MPlug& plug,void* clientData );
	typedef void (*MNodeStringFunction)( MObject & node,const MString & prevName,void* clientData );
	typedef void (*MParentChildFunction)( MDagPath &child, MDagPath &parent, void * clientData );
	typedef void (*MModifierFunction)( MDGModifier& modifier, void* clientData );
	typedef void (*MNodeModifierFunction)( MObject &node, MDGModifier& modifier, void* clientData );
	typedef void (*MObjArray)(MObjectArray &editedCurves,void *clientData);
	typedef void (*MNodeObjArray)(MObject &animNode,MObjectArray &editedKeys,void *clientData);
	typedef void (*MStringNode)( const MString &panelName, MObject &camera, void* clientData );
											 
	///
	static MStatus  removeCallback( MCallbackId id );
	///
	static MStatus  removeCallbacks( MCallbackIdArray &ids );
	///
	static MCallbackId	currentCallbackId( MStatus* ReturnStatus = NULL );
	///
	static MStatus	nodeCallbacks( MObject& node, MCallbackIdArray& ids );

	///				
	static void setRegisteringCallableScript();
	///
	static bool registeringCallableScript();	

BEGIN_NO_SCRIPT_SUPPORT:

	/// OBSOLETE and NO SCRIPT SUPPORT
	static MStatus  removeCallbacks( MIntArray &ids );
	/// OBSOLETE and NO SCRIPT SUPPORT
	static MStatus	nodeCallbacks( MObject& node, MIntArray& ids );

END_NO_SCRIPT_SUPPORT:


protected:
    static void addNode( MMessageNodePtr node ); 
    static void removeNode( MMessageNodePtr node );

private:
	static const char* 		className();
	static bool fRegisteringCallableScript;	
};

#ifdef _WIN32
#pragma warning(default: 4522)
#endif // _WIN32

// *****************************************************************************

#endif /* __cplusplus */
#endif /* _MMessage */
