#ifndef _MTimerMessage
#define _MTimerMessage
//
//-
// ==========================================================================
// Copyright (C) 1995 - 2006 Autodesk, Inc., and/or its licensors.  All 
// rights reserved.
// 
// The coded instructions, statements, computer programs, and/or related 
// material (collectively the "Data") in these files contain unpublished 
// information proprietary to Autodesk, Inc. ("Autodesk") and/or its 
// licensors,  which is protected by U.S. and Canadian federal copyright law 
// and by international treaties.
// 
// The Data may not be disclosed or distributed to third parties or be 
// copied or duplicated, in whole or in part, without the prior written 
// consent of Autodesk.
// 
// The copyright notices in the Software and this entire statement, 
// including the above license grant, this restriction and the following 
// disclaimer, must be included in all copies of the Software, in whole 
// or in part, and all derivative works of the Software, unless such copies 
// or derivative works are solely in the form of machine-executable object 
// code generated by a source language processor.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND. 
// AUTODESK DOES NOT MAKE AND HEREBY DISCLAIMS ANY EXPRESS OR IMPLIED 
// WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
// NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, 
// OR ARISING FROM A COURSE OF DEALING, USAGE, OR TRADE PRACTICE. IN NO 
// EVENT WILL AUTODESK AND/OR ITS LICENSORS BE LIABLE FOR ANY LOST 
// REVENUES, DATA, OR PROFITS, OR SPECIAL, DIRECT, INDIRECT, OR 
// CONSEQUENTIAL DAMAGES, EVEN IF AUTODESK AND/OR ITS LICENSORS HAS 
// BEEN ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES. 
// ==========================================================================
//+
//
// CLASS:    MTimerMessage
//
// ****************************************************************************
//
// CLASS DESCRIPTION (MTimerMessage)
//
// This class is used to register callbacks that are based on a fixed
// time interval.  Unlike other callbacks, a timer callback is
// executed with the highest priority within Maya and take precidence
// over *all* other Maya events.  Use with care as it is possible to
// construct a callback that will prevent any other part of Maya from
// running.
//
// If the execution time of a callback exceeds its period/2 the timer
// will be reset.  This will give Maya 1 period in which to do other
// things.
//
// To remove a callback use MMessage::removeCallback.
// All callbacks that are registered by a plug-in must be removed by that
// plug-in when it is unloaded. Failure to do so will result in a fatal error.
//
// ****************************************************************************

#if defined __cplusplus

// INCLUDED HEADER FILES


#include <maya/MMessage.h>

// ****************************************************************************
//
// DECLARATIONS
//
// ****************************************************************************
//
// CLASS DECLARATION (MTimerMessage)

#ifdef _WIN32
#pragma warning(disable: 4522)
#endif // _WIN32

/// Register callbacks that are based on a fixed time interval. (OpenMaya) (OpenMaya.py)
class OPENMAYA_EXPORT MTimerMessage : public MMessage
{ 
public:
	typedef void (*sleepCallbackPtr)();
	///
	static MCallbackId	addTimerCallback(
								float period,
								MMessage::MElapsedTimeFunction func,
								void * clientData = NULL,
								MStatus * ReturnStatus = NULL );
	///
	static MStatus setSleepCallback( sleepCallbackPtr sleepFunc );
	///
	static MTimerMessage::sleepCallbackPtr sleepCallback( );

private: 
	static const char* className();
};

#ifdef _WIN32
#pragma warning(default: 4522)
#endif // _WIN32

// ****************************************************************************

#endif /* __cplusplus */
#endif /* _MTimerMessage */
