#ifndef _MUintArray
#define _MUintArray
//
//-
// ==========================================================================
// Copyright (C) 1995 - 2006 Autodesk, Inc., and/or its licensors.  All 
// rights reserved.
// 
// The coded instructions, statements, computer programs, and/or related 
// material (collectively the "Data") in these files contain unpublished 
// information proprietary to Autodesk, Inc. ("Autodesk") and/or its 
// licensors,  which is protected by U.S. and Canadian federal copyright law 
// and by international treaties.
// 
// The Data may not be disclosed or distributed to third parties or be 
// copied or duplicated, in whole or in part, without the prior written 
// consent of Autodesk.
// 
// The copyright notices in the Software and this entire statement, 
// including the above license grant, this restriction and the following 
// disclaimer, must be included in all copies of the Software, in whole 
// or in part, and all derivative works of the Software, unless such copies 
// or derivative works are solely in the form of machine-executable object 
// code generated by a source language processor.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND. 
// AUTODESK DOES NOT MAKE AND HEREBY DISCLAIMS ANY EXPRESS OR IMPLIED 
// WARRANTIES INCLUDING, BUT NOT LIMITED TO, THE WARRANTIES OF 
// NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, 
// OR ARISING FROM A COURSE OF DEALING, USAGE, OR TRADE PRACTICE. IN NO 
// EVENT WILL AUTODESK AND/OR ITS LICENSORS BE LIABLE FOR ANY LOST 
// REVENUES, DATA, OR PROFITS, OR SPECIAL, DIRECT, INDIRECT, OR 
// CONSEQUENTIAL DAMAGES, EVEN IF AUTODESK AND/OR ITS LICENSORS HAS 
// BEEN ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES. 
// ==========================================================================
//+
//
// CLASS:    MUintArray
//
// *****************************************************************************
//
// CLASS DESCRIPTION (MUintArray)
//
//	This class implements an array of unsigned integers.  Common convenience functions
//	are available, and the implementation is compatible with the internal
//	Maya implementation so that it can be passed efficiently between plugins
//	and internal maya data structures.
//
// *****************************************************************************

#if defined __cplusplus

// *****************************************************************************

// INCLUDED HEADER FILES



#include <maya/MTypes.h>
#include <maya/MStatus.h>

typedef unsigned int MUint;

// *****************************************************************************

// DECLARATIONS

// *****************************************************************************

// CLASS DECLARATION (MUintArray)

/// Array of integers data type. (OpenMaya) (OpenMaya.py)
/**
  Implement an array of integers data type.
*/
#ifdef _WIN32
#pragma warning(disable: 4522)
#endif // _WIN32

class OPENMAYA_EXPORT MUintArray  
{

public:
 	///
					MUintArray();
 	///
					MUintArray( const MUintArray& other );
 	///
					MUintArray( const unsigned int src[], unsigned int count );
	///
					MUintArray( unsigned int initialSize, 
							   unsigned int initialValue = 0 );
	///
					~MUintArray();
 	///
 	MUintArray &     operator=( const MUintArray & other );
	///
	MStatus			set( unsigned int element, unsigned int index );
 	///
	MStatus			setLength( unsigned int length );
	///
 	unsigned int        length() const;
 	///
 	MStatus			remove( unsigned int index );
 	///
 	MStatus			insert( unsigned int element, unsigned int index );
 	///
 	MStatus			append( unsigned int element );
 	///
 	MStatus         copy( const MUintArray& source );
 	///
 	MStatus		 	clear();
	///
	MStatus			get( unsigned int[] ) const;
	///
	void			setSizeIncrement ( unsigned int newIncrement );
	///
	unsigned int		sizeIncrement () const;

BEGIN_NO_SCRIPT_SUPPORT:
	
	/// NO SCRIPT SUPPORT
	unsigned int	operator[]( unsigned int index ) const;
	
	///	NO SCRIPT SUPPORT
 	unsigned int&	 operator[]( unsigned int index );
	
	///	NO SCRIPT SUPPORT
	friend OPENMAYA_EXPORT std::ostream &operator<<(std::ostream &os, 
											   const MUintArray &array);

END_NO_SCRIPT_SUPPORT:

#if defined(SWIG)
	swigTPythonObjectConverter(MUint);
	swigExtendAPIArray(MUintArray,MUint);
#endif

protected:
// No protected members

private:
	MUintArray( void* );
	friend class MPolyMessage;
 	void* arr;
	const unsigned int* debugPeekValue;
	bool   own;
	static const char* className();
  	void	syncDebugPeekValue();          
};

#ifdef _WIN32
#pragma warning(default: 4522)
#endif // _WIN32

// *****************************************************************************
#endif /* __cplusplus */
#endif /* _MUintArray */
