/*
* SCE CONFIDENTIAL
* PSP(TM) Graphic Library Samples
* Copyright (C) 2004-2006 Sony Computer Entertainment Europe
* All Rights Reserved.
*
* sceePspGum - sceePspGum.h
*   matrix library header file
*
*   Date            Design      Log
*  ----------------------------------------------------
*   2006-01-06      srubens     version 1.6.1
*   2005-12-12      srubens     version 1.6.0
*   2005-10-31      srubens     version 1.5.0
*   2005-09-27      srubens     version 1.4.1
*   2005-09-23      srubens     version 1.4.0
*   2005-08-23      srubens     version 1.3.0
*   2005-08-12      srubens     version 1.2.1b
*   2005-07-29      srubens     version 1.2.0
*   2005-07-21      srubens     version 1.1.1
*   2005-07-20      srubens     version 1.1.0
*   2005-04-05      srubens     version 1.0.0
*   2004-04-23      srubens     initial implementation
*/

// ------------------------------------------------------------------------------
// sceePspGum library
// Copyright (C) 2004-2006 Sony Computer Entertainment Europe.
// All Rights Reserved.
// ------------------------------------------------------------------------------
// FILE:			$File: sceePspGum.h $
// FIRST VERSION:	23/04/2004
// FIRST AUTHOR:	Sebastien Rubens
// LAST AUTHOR:		$Author: srubens  $
// VERSION:			$DateTime:  $
// REVISION:		$Revision:  $
// CHANGE:			$Change:  $
//
// DESCRIPTION:
//	This library is a libgum-like library with many improvements.
// ------------------------------------------------------------------------------


#ifndef	SCEEPSPGUM_H
#define	SCEEPSPGUM_H
// ------------------------------------------------------------------------------
// COMPILATION FLAGS
// ------------------------------------------------------------------------------
// TRC library usage check
#include <devkit_version.h>
#if		((SCE_DEVKIT_VERSION&0xFFFFFF00) < 0x02080000)
#error You are not authorized to use this version of the libraries for submission to SCE QA after the 9th of November 2006.
#endif


// ------------------------------------------------------------------------------
// INCLUDES
// ------------------------------------------------------------------------------
#include <psptypes.h>
#include "sceeTypes.h"
#include "sceePspGu.h"


namespace scee {
namespace Psp {
namespace Gum {
using namespace scee::Psp::Gu;
// ------------------------------------------------------------------------------
// TYPEDEF (ENUM, STRUCT...)
// ------------------------------------------------------------------------------
typedef enum
{
	VIEW_AND_WORLD		=	0,
	MODEL_TO_VIEW		=	1
}	EPrecision;


// ------------------------------------------------------------------------------
// FUNCTIONS
// ------------------------------------------------------------------------------
// Initialization functions
void			SetMatrixStack(	ScePspFMatrix4*			m,
								u32						proj,
								u32						view,
								u32						world,
								u32						tex );
// Matrix Manipulation functions
void			MatrixMode(		EMatrixType				mode );
void			LoadIdentity(	void );
void			LoadMatrix(		const ScePspFMatrix4*	m );
void			StoreMatrix(	ScePspFMatrix4*			m );
void			PushMatrix(		void );
void			PopMatrix(		void );
// Coordinates transformation function
void			MultMatrix(		const ScePspFMatrix4*	m );
void			RotateX(		f32						angle );
void			RotateY(		f32						angle );
void			RotateZ(		f32						angle );
void			RotateXYZ(		const ScePspFVector3*	vec );
void			RotateZYX(		const ScePspFVector3*	vec );
void			Translate(		const ScePspFVector3*	vec );
void			Scale(			const ScePspFVector3*	vec );
void			LookAt(			const ScePspFVector3*	eye,
							const ScePspFVector3*	center,
								const ScePspFVector3*	up );
void			Invert43(		const ScePspFMatrix4*	m = 0 );
// Projection functions
void			Ortho(			f32						left,
								f32						right,
								f32						bottom,
								f32						top,
								f32						r_near,
								f32						r_far );
void			Frustum(		f32						left,
								f32						right,
								f32						bottom,
								f32						top,
								f32						zNear,
								f32						zFar );
void			Perspective(	f32						fovy,
								f32						aspect,
								f32						r_near,
								f32						r_far );
// Projection matrix for texture mapping
void			TexOrtho(		f32						left,
								f32						right,
								f32						bottom,
								f32						top,
								f32						scaleS = 1.0f,
								f32						scaleT = 1.0f,
								f32						transS = 0.0f,
								f32						transT = 0.0f );
void			TexFrustum(		f32						left,
								f32						right,
								f32						bottom,
								f32						top,
								f32						zNear,
								f32						scaleS = 1.0f,
								f32						scaleT = 1.0f,
								f32						transS = 0.0f,
								f32						transT = 0.0f );
void			TexPerspective(	f32						fovy,
								f32						aspect,
								f32						scaleS = 1.0f,
								f32						scaleT = 1.0f,
								f32						transS = 0.0f,
								f32						transT = 0.0f );
// Point projection
ScePspFVector4*	Project(		ScePspFVector4*			pDest,
								const ScePspFVector3*	pSour );
// Function to synchronize with sceePspGu
void			SetMode(		EPrecision				mode );
void			SetCurMatrix(	void );
void			UpdateMatrix(	void );
// Draw with Matrix update
void			DrawArray(		EPrimitiveType			prim,
								VertexFormat			type,
								u16						count,
								const void*				i,
								const void*				p );
void			DrawArrayN(		EPrimitiveType			prim,
								VertexFormat			type,
								u16						n_verts,
								u32						n_prims,
								const void*				i,
								const void*				p );
void			DrawBezier(		VertexFormat			type,
								u8						ucount,
								u8						vcount,
								const void*				i,
								const void*				p );
void			DrawSpline(		VertexFormat			type,
								u8						ucount,
								u8						vcount,
								ESplineParameter		uflag,
								ESplineParameter		vflag,
								const void*				i,
								const void*				p );
// Context functions
u32				GetContextSize(	void );
void			StoreContext(	void*					pCtx );
void			LoadContext(	const void*				pCtx );


// ------------------------------------------------------------------------------
}	// namespace Gum
}	// namespace Psp
}	// namespace scee


// ------------------------------------------------------------------------------
// EXTRA LIBGU FUNCTIONS
// ------------------------------------------------------------------------------
#if		defined(__cplusplus)
extern "C" {
#endif	// defined(__cplusplus)

void	sceGumTexOrtho(			f32						left,
								f32						right,
								f32						bottom,
								f32						top,
								f32						scaleS = 1.0f,
								f32						scaleT = 1.0f,
								f32						transS = 0.0f,
								f32						transT = 0.0f );
void	sceGumTexFrustum(		f32						left,
								f32						right,
								f32						bottom,
								f32						top,
								f32						zNear,
								f32						scaleS = 1.0f,
								f32						scaleT = 1.0f,
								f32						transS = 0.0f,
								f32						transT = 0.0f );
void	sceGumTexPerspective(	f32						fovy,
								f32						aspect,
								f32						scaleS = 1.0f,
								f32						scaleT = 1.0f,
								f32						transS = 0.0f,
								f32						transT = 0.0f );

#if		defined(__cplusplus)
}	// extern "C"
#endif	// defined(__cplusplus)


// ------------------------------------------------------------------------------
#endif	// SCEEPSPGUM_H
