/*
* SCE CONFIDENTIAL
* PSP(TM) Graphic Library Samples
* Copyright (C) 2004-2006 Sony Computer Entertainment Europe
* All Rights Reserved.
*
* sceePspGuu - sceePspGuu.h
* library header file
*
* Date  Design Log
* ----------------------------------------------------
* 2004-04-23 srubens  initial implementation
*/

// ------------------------------------------------------------------------------
// sceePspGuu library
// Copyright (C) 2004-2006 Sony Computer Entertainment Europe.
// All Rights Reserved.
// ------------------------------------------------------------------------------
// FILE:			$File: sceePspGu.h $
// FIRST VERSION:	23/04/2004
// FIRST AUTHOR:	Sebastien Rubens
// LAST AUTHOR:		$Author: srubens $
// VERSION:			$DateTime: $
// REVISION:		$Revision: $
// CHANGE:			$Change: $
//
// DESCRIPTION:
//	This library is an additional utilities functions library to libgu.
// ------------------------------------------------------------------------------


#ifndef	SCEEPSPGUU_H
#define	SCEEPSPGUU_H
// ------------------------------------------------------------------------------
// COMPILATION FLAGS
// ------------------------------------------------------------------------------
// TRC library usage check
#include <devkit_version.h>
#if		((SCE_DEVKIT_VERSION&0xFFFFFF00) < 0x02080000)
#error You are not authorized to use this version of the libraries for submission to SCE QA after the 9th of November 2006.
#endif


// ------------------------------------------------------------------------------
// INCLUDES
// ------------------------------------------------------------------------------
#include <psptypes.h>
#include "sceeTypes.h"
#include "sceePspGu.h"


namespace scee {
namespace Psp {
namespace Guu {
using namespace scee::Psp::Gu;
// ------------------------------------------------------------------------------
// TYPEDEFS
// ------------------------------------------------------------------------------
typedef enum
{
	TSLOPE_TYPE_OVER_BLUR	=	0,
	TSLOPE_TYPE_UNDER_BLUR	=	1,
	TSLOPE_TYPE_MEAN_BLUR	=	2
}	ETSlopeType;

typedef struct
{
	f32		primMipType[3];
	f32		varBlur;
	u32		nbTriangles;
}	TSlopeInfo;

typedef struct
{
	s32		align;
	s32		weightOff[8];
	s32		textureOff;
	s32		colorOff;
	s32		normalOff;
	s32		vertexOff;
	s32		vertexSize;
	s32		totalSize;
}	VertexFormatInfo;


// ------------------------------------------------------------------------------
// FUNCTIONS
// ------------------------------------------------------------------------------
// VertexFormat related functions
s32		GetVertexAlign(		VertexFormat			vf );
void	GetVertexInfos(		VertexFormatInfo*		pVFI,
							VertexFormat			vf );

// ------------------------------------------------------------------------------
// Drawing functions
void	DrawArray(			EPrimitiveType			prim,
							VertexFormat			vf,
							u32						count,
							void* const				i,
							void* const				p );

// ------------------------------------------------------------------------------
// TSlope related functions
void	ComputeTsi(			TSlopeInfo*				pTsi,
							u32						nbTsi,
							EPrimitiveType			prim,
							VertexFormat			vf,
							u16						count,
							const void*				i,
							const void*				p );
void	MergeTsi(			TSlopeInfo*				pDestTsi,
							const TSlopeInfo*		pSourTsi,
							u32						nbTsi );
f32		ComputeTSlope(		const TSlopeInfo*		pTsi,
							f32						textureWidth,
							f32						textureHeight,
							ETSlopeType				tst );

// ------------------------------------------------------------------------------
// CSlope related functions
f32		ComputeCSlope(		const ScePspFMatrix4*	pMatrix,
							f32						screenWidth,
							f32						screenHeight );


// ------------------------------------------------------------------------------
}	// namespace Guu
}	// namespace Psp
}	// namespace scee
#endif	// SCEEPSPGUU_H
