/*
[SCE CONFIDENTIAL]
$PSDocId$
                  Copyright (C) 2002-4 Sony Computer Entertainment America
                                                     All Rights Reserved
//  ------------------------------------------------------------------------

File: internal.h

//  ------------------------------------------------------------------------
*/

#ifndef	_EZNETCNF_INTERNAL_H
#define	_EZNETCNF_INTERNAL_H


#if defined(__R5900__) || defined(__ee__)
    #define __EE__
#endif  // __EE__

#ifdef	__EE__
    #define printf scePrintf
    //#define snprintf sceSnprintf
#endif	// __EE__


/*  --------------------------------------------------------
 *  compile-time options/optimizations
 *
 *  most of these options affect both the EE and IOP library.
 *  recompile both sides after a change.
 *  --------------------------------------------------------
 */

    // ENABLE for dprintf TTY messages
#define DEBUG                   1

    // ENABLE to dump sceNetCnfEnv and sceNetcnfifData properties
    // for conversion verification
#define DUMP_CONFIGURATION      0

    // ENABLE to dump ifstat internals for i18n verification
#define DUMP_IFSTAT             0

    // DISABLE to have eznetctl.c dynamically allocate and free
    // the temporary sceNetcnfifData_t used by SetProperties().
    // also affects write_env.c::sceNetcnfifWriteEnv() to make 
    // deep-copies of sceNetcnfifData_t's source string buffers.
#define STATIC_NETCNFIF_DATA    0

    // DISABLE to strip out modem-specific code
#define SUPPORT_MODEM           1


/*  --------------------------------------------------------
 *  constants
 *  --------------------------------------------------------
 */

#define kVersion            0x0204

#define kModName_ezNetCnf   "eznetcnf"
#ifdef INET_VERSION
#define kModName_ezNetCtl   "eznetctl_inet"
#else
#define kModName_ezNetCtl   "eznetctl_eenet"
#endif  // INET_VERSION

/*
 *  SIF RPC magic numbers (aka SIFNUM)
 *  used with sceSifRegisterRpc() and sceSifBindRpc()
 */
#define kRPCId_ezNetCnf     0x75499128
#define kRPCId_ezNetCtl     0x75488909

/*
 *  SIF RPC buffer size (keep to 64 byte alignment)
 */
#define kRPCBufferSize      (256 + 128)

// for syntactic convenience, define a RPC request structure
typedef struct {
    int  args[4];
    char buf1[kStrBufferSize];
    char buf2[kRPCBufferSize - kStrBufferSize - 4*sizeof(int)];
} rpcRequest_t;

/*
 *  SIF DMA are performed in 16-byte blocks from IOP -> EE.
 *  enforce padding in both DMA directions.
 */
#define kDMAWait        0
#define kDMANoWait      1
#define kDMABlockSize   16
#define DMA_PAD(size)   (((size) + (kDMABlockSize-1)) & ~(kDMABlockSize-1))

/*
 *  Maximum number of interfaces to scan, for active I/F.
 */
#define kMaxIfnum       4


/*  --------------------------------------------------------
 *  DEBUG sensitive preprocessor macros
 *  --------------------------------------------------------
 */

/*
 *  use dprintf() for non-essential debug messages only.
 */
#if (DEBUG)
    #ifdef  __GNUC__
        #define dprintf(args...) {printf("%s> ", __FUNCTION__); printf(args);}
    #else
        // Ben - changed for CW (C99 syntax)
        #define dprintf(...) {printf("%s> ", __FUNCTION__); printf(__VA_ARGS__);}
    #endif  // __GNUC__
#else   // !DEBUG
    #define dprintf(args...)
#endif  // DEBUG

/*
 *  use ASSERT_ALIGN to verify data address alignment
 */
#if (DEBUG)
#define ASSERT_DMA(addr, align) \
if ((u_int)(addr) % align) {    \
    dprintf("DMA address 0x%x not %d-byte aligned\n", (u_int)(addr), align); \
    return (-SCE_EINVAL);       \
}
#else   // !DEBUG
#define ASSERT_DMA(addr, align)
#endif	// DEBUG

/*
 *  use ASSERT_RPC to verify RPC running status and transfer size
 */
#if (DEBUG)
#define ASSERT_RPC(size)        \
if (!m_rpcRunning) {            \
    dprintf("RPC service not running\n"); \
    return (-1);                \
}                               \
if (size > kRPCBufferSize) {    \
    dprintf("RPC transfer size 0x%x exceeds buffer 0x%x\n", size, kRPCBufferSize); \
    return (-SCE_EINVAL);       \
}
#else   // !DEBUG
#define ASSERT_RPC(size)
#endif	// DEBUG


/*  --------------------------------------------------------
 *  RPC function IDs
 *  --------------------------------------------------------
 */

/*
 *  eznetcnf RPC function ids
 */
typedef enum {
    eCnfGetCount = 0,
    eCnfGetCombinationList,
    eCnfGetProperties,

    eCnfEnd = 0x1ffff           // for enum size consistency
} ezNetCnfFunctionId;

/*
 *  eznetctl RPC function ids
 */
typedef enum {
    eCtlGetPropertiesAddress = 0,
    eCtlSetProperties,
    eCtlSetCombination,
    
    eCtlGetStatus,
    eCtlWaitEvent,
    eCtlUpInterface,
    eCtlDownInterface,

    eCtlPreloadModemCnf,
    eCtlUnloadModemCnf,

    eCtlEnd = 0x1ffff           // for enum size consistency
} ezNetCtlFunctionId;


#endif	// _EZNETCNF_INTERNAL_H

