//////////////////////////////////////////////////////////////////////////////
//
// AVP_GS2_AnimPackage.cpp
//
// Copyright (C) 2009 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "AnimTemplates/AVP_GS2_AnimPackage.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"

// avp
#include "Inventory/AVP_Inventory_ItemTypes.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Gamescene_AnimTemplate_System
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Gamescene_AnimTemplate_System : public Axon_Gamescene_AnimTemplate_System
{
public:
	// Could use a mempool if we wanted.
	virtual Axon_GS2_AnimPackage* CreateAnimPackage() { return new AVP_GS2_AnimPackage; }
	virtual void DestroyAnimPackage( Axon_GS2_AnimPackage* pxPackage ) { delete pxPackage; }
};

Axon_Gamescene_AnimTemplate_System* Axon_Gamescene_AnimTemplate_System::Project_CreateAnimTemplateSystem()
{
	return new AVP_Gamescene_AnimTemplate_System;
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_GS2_AnimMapping
//
//////////////////////////////////////////////////////////////////////////////

void AVP_GS2_AnimMapping::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_auFactors[ AVP_GS2_MF_SKIN ];
		xStream >> m_auFactors[ AVP_GS2_MF_WEAPON ];
	}
	else
	{
		Asura_Assertf( false, ( "AVP_GS2_AnimMapping::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void AVP_GS2_AnimMapping::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_auFactors[ AVP_GS2_MF_SKIN ];
	xStream << m_auFactors[ AVP_GS2_MF_WEAPON ];
}

bool AVP_GS2_AnimMapping::Matches( const void* pMappingData ) const
{
	const AVP_GS2_AnimMappingData* pxMappingData = reinterpret_cast< const AVP_GS2_AnimMappingData* >( pMappingData );

	if ( m_auFactors[ AVP_GS2_MF_SKIN ] != ASURA_HASH_ID_UNSET &&
		pxMappingData->m_auFactorValues[ AVP_GS2_MF_SKIN ] != m_auFactors[ AVP_GS2_MF_SKIN ] )
	{
		return false;
	}
	if ( m_auFactors[ AVP_GS2_MF_WEAPON ] != AVP_WMF_X &&
		pxMappingData->m_auFactorValues[ AVP_GS2_MF_WEAPON ] != m_auFactors[ AVP_GS2_MF_WEAPON ] )
	{
		return false;
	}

	return true;
}

//////////////////////////////////////////////////////////////////////////////
//
// AVP_GS2_AnimPackage
//
//////////////////////////////////////////////////////////////////////////////

AVP_GS2_AnimPackage::AVP_GS2_AnimPackage()
: m_uFinalPosture( 0 )
, m_uFinalPose( 0 )
{
}

void AVP_GS2_AnimPackage::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_uFinalPosture;
		xStream >> m_uFinalPose;
	}
	else
	{
		Asura_Assertf( false, ( "AVP_GS2_AnimPackage::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

#ifdef ASURA_TOOLS
void AVP_GS2_AnimPackage::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_uFinalPosture;
	xStream << m_uFinalPose;
}
#endif

Axon_GS2_AnimMapping* AVP_GS2_AnimPackage::CreateAnimMapping( bool bSetToDefaults )
{
	// Any anim mapping factors must start off with the "don't care" value initially.
	AVP_GS2_AnimMapping* pxMapping = new AVP_GS2_AnimMapping;
#ifdef ASURA_TOOLS
	if ( bSetToDefaults )
	{
		pxMapping->GetFactor( AVP_GS2_MF_SKIN )   = ASURA_HASH_ID_UNSET;
		pxMapping->GetFactor( AVP_GS2_MF_WEAPON ) = AVP_WMF_X;
	}
#else
	Asura_Assert( !bSetToDefaults, "AVP_GS2_AnimPackage::CreateAnimMapping - can't set to defaults outside of tools code" );
#endif
	return pxMapping;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
