//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_AVP3Alien_Wander.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour\BehaviourTypes\AVP3Alien\AVP_Behaviour_AVP3Alien_Wander.h"

// asura
#include "AI/Asura_AI_Navigation.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/AVP3Alien/AVP_Behaviour_AVP3Alien_Movement.h"
#include "Core/BrainTypes/Axon_Project_Brain.h"

// avp
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"

// avp axon
#include "Core/AVP_Navigation.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_AVP3Alien_Wander
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_AVP3Alien_Wander::AVP_Behaviour_AVP3Alien_Wander( Axon_Project_Brain* pxBrain )
: PARENT( pxBrain, AXON_BEHAVIOUR_TYPE_AVP3ALIEN_WANDER )
, m_eCurrentState( WAITING )
, m_fTimeInState( 0.0f )
{
}

void AVP_Behaviour_AVP3Alien_Wander::Deactivate()
{
    // Destroy any children.
    Axon_Behaviour* pxChildBehaviour = GetChild();
    while ( pxChildBehaviour )
    {
        UnlinkChild( pxChildBehaviour );
        GetProjectBrain()->DestroyBehaviour( pxChildBehaviour );
        pxChildBehaviour = GetChild();
    }

    PARENT::Deactivate();
}

void AVP_Behaviour_AVP3Alien_Wander::Update( const float fTimeslice )
{
    PARENT::Update( fTimeslice );

    m_fTimeInState += fTimeslice;

    switch ( m_eCurrentState )
    {
        case WAITING:
        {
            if ( m_fTimeInState > 2.0f )
            {
                SwitchToState( GETTING_TO_POSITION );
            }
            break;
        }

        case GETTING_TO_POSITION:
        {
            AVP_Behaviour_AVP3Alien_Movement* pxGTPBehaviour =
                static_cast< AVP_Behaviour_AVP3Alien_Movement* >( GetChildWithType( this, AXON_BEHAVIOUR_TYPE_AVP3ALIEN_MOVEMENT ) );
            if ( !pxGTPBehaviour || pxGTPBehaviour->IsInError() || pxGTPBehaviour->IsFinished() )
            {
                SwitchToState( WAITING );
            }

            break;
        }
    }
}

#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
    "WAITING",	
    "GETTING TO POSITION"
};

void AVP_Behaviour_AVP3Alien_Wander::RenderDebugInfo()
{
    PARENT::RenderDebugInfo();

    Asura_Font_System::Print( m_pxBrain->GetDebugOutputPos().x, m_pxBrain->GetDebugOutputPos().y, 0.5f, 0xffffffff,
        ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "%s", s_aszStateStrings[ m_eCurrentState ] );
    m_pxBrain->GetDebugOutputPos().y += 10.0f;
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

//static bool s_bToggle = false;

void AVP_Behaviour_AVP3Alien_Wander::SwitchToState( const State eNextState )
{
    // Do any work we need to exit our current state.
    switch ( m_eCurrentState )
    {
        case GETTING_TO_POSITION:
        {
            DestroyChildWithType( AXON_BEHAVIOUR_TYPE_AVP3ALIEN_MOVEMENT );
            break;
        }

        case WAITING:
        {
            break;
        }
    }

    // Do any work we need to change into our next state.
    m_eCurrentState = eNextState;
    m_fTimeInState  = 0.0f;
    switch ( m_eCurrentState )
    {
        case GETTING_TO_POSITION:
        {
            const AVP_Navigation::NavData& xNavData = GetProjectBrain()->GetNavData();

            // Pick a valid random waypoint.
            Asura_Vector_3 xDestination = Asura_Maths::ZeroVector3;
            do
            {
                const u_short usRandomWaypoint = static_cast< u_short >( Asura_Random::GetIntU( 0, Asura_AI_Navigation_System::GetNumberOfWaypoints() - 1 ) );

                u_short usNearestWaypoint = ASURA_INVALID_WAYPOINT_ID;
                AVP_Navigation::TestParams xTestParams;
                xTestParams.m_uPathfinderFlags = xNavData.m_uPathfinderFlags;
                xTestParams.m_ucZone           = xNavData.m_ucCurrentZone;
                if ( AVP_Navigation::GetNearestWaypointID( Asura_AI_Navigation_System::GetWaypoint( usRandomWaypoint )->m_xPosition, usNearestWaypoint, xTestParams ) )
                {
                    xDestination = Asura_AI_Navigation_System::GetWaypoint( usNearestWaypoint )->m_xPosition;
                }
            } while ( xDestination == Asura_Maths::ZeroVector3 );

            Asura_Vector_3 xRandomVector;
            xRandomVector.RandomDirectionSeeded();
            xDestination += xRandomVector * Asura_Random::GetFloat( 1.0f, 5.0f );

            AVP_Navigation::TestParams xTestParams;
            xTestParams.m_uPathfinderFlags = xNavData.m_uPathfinderFlags;
            xTestParams.m_ucZone           = xNavData.m_ucCurrentZone;
            AVP_Navigation::ResolveToValidPosition( xDestination, xTestParams );

            //if ( s_bToggle )
            //{
            //    xDestination = Asura_AI_Navigation_System::GetWaypoint( 587 )->m_xPosition;
            //}
            //else
            //{
            //    xDestination = Asura_AI_Navigation_System::GetWaypoint( 515 )->m_xPosition;
            //}
            //s_bToggle = !s_bToggle;

            AVP_Behaviour_AVP3Alien_Movement* pxGTPBehaviour =
                static_cast< AVP_Behaviour_AVP3Alien_Movement* >( CreateChildWithType( AXON_BEHAVIOUR_TYPE_AVP3ALIEN_MOVEMENT ) );
            if ( pxGTPBehaviour && pxGTPBehaviour->GetToPosition( xDestination ) )
            {
            }
            else
            {
                SwitchToState( WAITING );
            }
            break;
        }

        case WAITING:
        {
            break;
        }
    }
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
