//////////////////////////////////////////////////////////////////////////////
//
// AVP_BehaviourTree_Element.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BEHAVIOURTREE_ELEMENT_H__
#define __AVP_BEHAVIOURTREE_ELEMENT_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Behaviour/Axon_Project_Behaviour.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// AVP_BehaviourTree_Element
//
//////////////////////////////////////////////////////////////////////////////

class AVP_BehaviourTree_Element : public Axon_Project_Behaviour
{
    typedef Axon_Project_Behaviour PARENT;

public:
    AVP_BehaviourTree_Element( Axon_Project_Brain* pxBrain, u_int uType );
	virtual ~AVP_BehaviourTree_Element() = 0;

    virtual void Deactivate();

	void SetPriority( u_int uPriority ) { m_uPriority = uPriority; }
	u_int GetPriority( void ) const { return m_uPriority; }

	void SetProbability( float fProbability ) { m_fProbability = Asura_Maths::Min( Asura_Maths::Max( fProbability, 0.0f ), 1.0f ); }
	float GetProbability( void ) const { return m_fProbability; }

	virtual void UpdateRunningConditions( const float fTimeslice ) {}
	virtual void EvaluateRunningConditions() { m_bRunningConditionsMet = true; }
	bool RunningConditionsMet( void ) { return m_bRunningConditionsMet; }

	// Used to force the active child to be a specific (already existing/linked in/etc) behaviour.
	void  SetActiveChild( AVP_BehaviourTree_Element* pxActiveChild );
	u_int GetActiveChildType( void ) const { return m_pxActiveChild ? m_pxActiveChild->GetType() : static_cast<u_int>(AXON_BEHAVIOUR_TYPE_UNSET); }

protected:
	bool	m_bRunningConditionsMet;
	bool	m_bEvaluateChildren;
	bool	m_bIsAtomic;

	virtual void PerFrameUpdate( const float fTimeslice );
	virtual void Update( const float fTimeslice );

	virtual void CreateChildBehaviours() {}

private:
	AVP_BehaviourTree_Element* m_pxActiveChild;
	bool m_bChildBehavioursCreated;

	u_int m_uPriority; // The higher this number the higher the priority
	float m_fProbability; // From all valid behaviours with the same (highest) priority, the next behaviour is randomly choosen based on this value - range is [0.0f, 1.0f] for clarity
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BEHAVIOURTREE_ELEMENT_H__
