//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Shared_GoTo.cpp
//
// Created by Adam Law - 19 April 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Core
#include "Core/Asura.h"

// This files header
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Shared_GoTo.h"

// AVP specific
#include "Actor/Vertebrates/Human/AVP_Human_Description.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Entity/AVP_Entity_Classification.h"

// Axon specific
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/Axon_Project_Brain.h"


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_Shared_GoTo::AVP_Behaviour_Shared_GoTo
// Author       : Adam Law
// Date         : 19-Apr-2007
////////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Shared_GoTo::AVP_Behaviour_Shared_GoTo( Axon_Project_Brain* pxBrain )
: PARENT(pxBrain, AXON_BEHAVIOUR_TYPE_ACTOR_GOTO)
, m_pxGoToPoint(NULL)
, m_eUpdateState(US_UNSET)
//, m_pxController(NULL)
{	
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_Shared_GoTo::~AVP_Behaviour_Shared_GoTo
// Author       : Adam Law
// Date         : 05-Apr-2007
////////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Shared_GoTo::~AVP_Behaviour_Shared_GoTo()
{
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_Shared_GoTo::Activate
// Author       : Adam Law
// Date         : 19-Apr-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Behaviour_Shared_GoTo::Activate()
{
    bool bOk = PARENT::Activate();

	// Figure out if we are dealing with a Human or Alien
	m_pxActor = m_pxBrain->GetActor();
	u_int uActorType = m_pxActor->GetClassification();
	
	if ( !m_pxActor ) {
		return false;
	}

	if ((uActorType == AVP_ENTITYCLASS_ALIEN) || (uActorType == AVP_ENTITYCLASS_ALIENFACEHUGGER) || (uActorType == AVP_ENTITYCLASS_HUMAN))
	{	
		m_bIsAlien = uActorType != AVP_ENTITYCLASS_HUMAN ? true : false;
		m_eUpdateState = US_GET_GO_TO;	
	}
	else
	{			
		Asura_Assertf( false, ("AVP_Behaviour_Shared_GoTo - Invalid actor type %d", uActorType ));
		return false;
	}   
	if (m_bIsAlien) 
	{
		m_pxBrain->RequestAnimation(AVP_Alien_Description::ALIEN_ANIM_SPRINT);
	}
	else
	{
		m_pxBrain->RequestAnimation(AVP_Human_Description::MARINE_WALK_ANIMATION_HASH);
	}

	return bOk;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_Shared_GoTo::Deactivate
// Author       : Adam Law
// Date         : 13-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Behaviour_Shared_GoTo::Deactivate()
{   
    PARENT::Deactivate();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_Shared_GoTo::Update
// Author       : Adam Law
// Date         : 19-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Behaviour_Shared_GoTo::Update( const float fTimeslice ) 
{
	PARENT::Update( fTimeslice );	
	if ( IsInError() || IsFinished() ) { return; }

	switch ( m_eUpdateState )
    {   
		case US_GET_GO_TO:
		{
			// Find the Go to Point for the actor to head towards
			if ( GetGoToPoint() ) 
			{
				SetDestination( m_pxGoToPoint->GetPosition(), fTimeslice );
				m_eUpdateState = US_NAV_ROUTE;
			}
			else
			{
				Asura_Assert(false, "Unable to retrieve Go to Point");
				SetInError();
			}
		} break;

		case US_FINISHED:
		{				
			SwitchAway();
		} break;
				
		case US_NAV_ROUTE:
		{
		} break;
		
		case US_UNSET:
		default:
		{
			StopMoving();			
			Asura_Assertf( false, ("AVP_Behaviour_Shared_GoTo::Update - Invalid state (%d)", m_eUpdateState ));
			SetInError(); 
		} break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_Shared_GoTo::PerFrameUpdate
// Author       : Adam Law
// Date         : 19-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Behaviour_Shared_GoTo::PerFrameUpdate( const float fTimeslice)
{	
	PARENT::Update( fTimeslice );
	if ( IsInError() || IsFinished() ) { return; }
	
	switch ( m_eUpdateState )
    {	
		case US_NAV_ROUTE:
		{
			OrientAlongRoute(fTimeslice);

			// If we have reached the Go to Point
			if ( AtTargetPosition(m_pxActor->GetPosition(), m_pxGoToPoint->GetPosition()) )
			{
				m_eUpdateState = US_FINISHED;				
			}
		} break;
	
		default:
		{
		} break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_Shared_GoTo::GetGoToPoint
// Author       : Adam Law
// Date         : 19-Apr-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_Behaviour_Shared_GoTo::GetGoToPoint()
{
#if 0
	// Get & store our destination point
	AVP_ServerEntity_SpawnPoint* pxSpawnPoint = AVP_ServerEntity_SpawnPoint::FindEntity(m_pxActor->GetControllerID());
	Asura_Guid GoToGUID = pxSpawnPoint->GetLinkedStdGoToPoint();	
	m_pxGoToPoint = AVP_ServerEntity_GoToPoint::FindEntity(GoToGUID);


	return m_pxGoToPoint != NULL ? true : false;
#endif
	return false;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_Shared_GoTo::SetDestination
// Author       : Adam Law
// Date         : 19-Apr-2007
////////////////////////////////////////////////////////////////////////////////

const bool AVP_Behaviour_Shared_GoTo::SetDestination( const Asura_Vector_3& vTarget, const float fTimeslice )
{
	// Use A* to calculate a route to our destination
	if ( !GetRouteTo(vTarget) )
    {
		StopMoving();
		SetInError();
		return false;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_Shared_GoTo::SwitchAway
// Author       : Adam Law
// Date         : 19-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Behaviour_Shared_GoTo::SwitchAway()
{
	StopMoving();
	SetFinished();	
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_Shared_GoTo::StopMoving
// Author       : Adam Law
// Date         : 19-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Behaviour_Shared_GoTo::StopMoving()
{
	if (m_bIsAlien)
	{		
		m_pxBrain->RequestAnimation(AVP_Alien_Description::ALIEN_ANIM_SNEAK_REST);		
	}
	else
	{
		m_pxBrain->RequestAnimation(AVP_Human_Description::MARINE_REST_ANIMATION_HASH);
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	"GO_TO - US_UNSET",	
	"GO_TO - GET_GO_TO",
	"GO_TO - NAV_ROUTE",
	"GO_TO - FINISHED"
};

void AVP_Behaviour_Shared_GoTo::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( m_pxBrain->GetDebugOutputPos().x, m_pxBrain->GetDebugOutputPos().y, 0.5f, 0xffffffff,
							  ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "%s", s_aszStateStrings[ m_eUpdateState ] );
							  m_pxBrain->GetDebugOutputPos().y += 10.0f;
}
#endif
