//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Attack.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BEHAVIOUR_ALIEN_ATTACK_H__
#define __AVP_BEHAVIOUR_ALIEN_ATTACK_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Behaviour/Axon_Project_Behaviour.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Message;
class AVP_Brain_Actor_Alien;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Attack
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Behaviour_Alien_Attack : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:
	AVP_Behaviour_Alien_Attack( AVP_Brain_Actor_Alien* pxBrain );

protected:
	virtual bool Activate();
	virtual void Deactivate();
	virtual void PerFrameUpdate( const float fTimeslice );
	virtual bool HandleMessage( const Asura_Message* pxMessage );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

private:
	enum State
	{
		UNSET,
		RUNNING_IN,
		REQUESTED_JUMP_ATTACK,
		STARTED_JUMP_ATTACK,
		REQUESTED_STAND_ATTACK,
		STARTED_STAND_ATTACK,
		STARTED_LOOP_STAND_ATTACK,
		CYCLING_ROUND,
		COUNTERED,
		FINISHED,
		NUM_STATES
	};

	AVP_Brain_Actor_Alien* GetAlienBrain() const;
	void RequestMelee( const u_int uMeleeType );
	void SwitchToState( const State eState );
	bool EndOfRouteAllowsStandAttack() const;
	void SetupStandSlashAttack();
	void SetupJumpSlashAttack();

private:
	u_int m_uMeleeType;
	Asura_Hash_ID m_uInitialMeleeMoveHash;
	enum MeleeHandedness
	{
		NONE,
		LEFT,
		RIGHT
	} m_eMeleeHandedness;
	float m_fAttackRangeMin;
	float m_fAttackRangeMax;
	u_int m_uNumRemainingAttacks;
	u_int m_uNumRemainingCyclesRound;

#ifndef ASURA_NO_DEBUG_OUTPUT
	static const char* s_aszStateStrings[ NUM_STATES ];
#endif
};

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_JumpAttack
// Hardwired jump attack for the SPSC.
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Behaviour_Alien_JumpAttack : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:
	AVP_Behaviour_Alien_JumpAttack( AVP_Brain_Actor_Alien* pxBrain );

protected:
	virtual bool Activate();
	virtual void Deactivate();
	virtual void PerFrameUpdate( const float fTimeslice );
	virtual bool HandleMessage( const Asura_Message* pxMessage );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

private:
	enum State
	{
		UNSET,
		RUNNING_IN,
		STARTED_JUMP_ATTACK,
		COUNTERED,
		FINISHED,
		NUM_STATES
	};

	AVP_Brain_Actor_Alien* GetAlienBrain() const;
	void RequestJumpAttackMelee();
	void SwitchToState( const State eState );

private:


#ifndef ASURA_NO_DEBUG_OUTPUT
	static const char* s_aszStateStrings[ NUM_STATES ];
#endif
};


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_StandAttack
// Hardwired jump attack for the SPSC.
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Behaviour_Alien_StandAttack : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:
	AVP_Behaviour_Alien_StandAttack( AVP_Brain_Actor_Alien* pxBrain );

protected:
	virtual bool Activate();
	virtual void Deactivate();
	virtual void PerFrameUpdate( const float fTimeslice );
	virtual bool HandleMessage( const Asura_Message* pxMessage );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

private:
	enum State
	{
		UNSET,
		RUNNING_IN,
		STARTED_STAND_ATTACK,
		STARTED_LOOP_STAND_ATTACK,
		COUNTERED,
		FINISHED,
		NUM_STATES
	};

	AVP_Brain_Actor_Alien* GetAlienBrain() const;
	void RequestMelee( const u_int uMeleeType );
	void SwitchToState( const State eState );

private:
	u_int m_uMeleeType;
	enum MeleeHandedness
	{
		NONE,
		LEFT,
		RIGHT
	} m_eMeleeHandedness;
	u_int m_uNumRemainingAttacks;

#ifndef ASURA_NO_DEBUG_OUTPUT
	static const char* s_aszStateStrings[ NUM_STATES ];
#endif
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BEHAVIOUR_ALIEN_GRAPPLE_H__
