//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_CycleRoundImmediate.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_CycleRoundImmediate.h"

// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Movement.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_CycleRoundImmediate
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Alien_CycleRoundImmediate::AVP_Behaviour_Alien_CycleRoundImmediate( Axon_Project_Brain* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_ALIEN_CYCLEROUNDIMMEDIATE )
, m_xThreatPositionAtPathStart(Asura_Maths::ZeroAVector3)
{
	SetCurrentState( UNSET );
}

bool AVP_Behaviour_Alien_CycleRoundImmediate::Activate()
{
	SetCurrentState( UNSET );
	 return PARENT::Activate();
}

void AVP_Behaviour_Alien_CycleRoundImmediate::Deactivate()
{
	PARENT::Deactivate();
}

void AVP_Behaviour_Alien_CycleRoundImmediate::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );
	
	if ( fTimeslice == 0.0f ) { return; }

	UpdateStateTimer( fTimeslice );

	if (!GetAlienBrain()->CanPlanMove())
	{
		return;
	}

	switch ( GetCurrentState() )
	{
	case UNSET:
		{
			const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetCurrentTargetKnowledge();
			const AVP_AlienBehaviourData& xBehaviourData = GetAlienBrain()->GetBehaviourData();
			if (pxTargetKnowledge)
			{
				m_xThreatPositionAtPathStart = pxTargetKnowledge->GetPosition();
				AVP_Navigation::AlienAwayFromFunctor xFunctor(pxTargetKnowledge->GetPosition(), xBehaviourData.m_fIdealDistanceMax);
				if ( GetAlienBrain()->GetMovement().GetAwayFromPosition( xFunctor, AVP_Alien_Movement::EMM_MEDIUM ) )
				{
					SwitchToState(TURNING);
				}
				else
				{
					SetInError(true);
					SwitchToState(FINISHED);
				}
			}
			else
			{
				SwitchToState(FINISHED);
			}
		}
		break;
	case TURNING:
		{
			const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetCurrentTargetKnowledge();
			AVP_Alien_Movement& xMovement = GetAlienBrain()->GetMovement();
			if ( xMovement.IsInError() )
			{
				SetInError(true);
				SwitchToState(FINISHED);
			}
			else if ( !pxTargetKnowledge || xMovement.IsFinished() )
			{
				SwitchToState(FINISHED);
			}
			else if ( GetAlienBrain()->ShouldRefreshPath((pxTargetKnowledge->GetPosition()- m_xThreatPositionAtPathStart).MagnitudeSquared() ) )
			{
				m_xThreatPositionAtPathStart = pxTargetKnowledge->GetPosition();
				const AVP_AlienBehaviourData& xBehaviourData = GetAlienBrain()->GetBehaviourData();
				AVP_Navigation::AlienAwayFromFunctor xFunctor(pxTargetKnowledge->GetPosition(), xBehaviourData.m_fIdealDistanceMax);
				xMovement.GetAwayFromPosition( xFunctor, AVP_Alien_Movement::EMM_MEDIUM );
			}
		}
		break;
	case FINISHED:
	default:
		break;
	}

}

#ifndef ASURA_NO_DEBUG_OUTPUT
const char* AVP_Behaviour_Alien_CycleRoundImmediate::s_aszStateStrings[] =
{
	"UNSET",
	"TURNING",
	"FINISHED"
};

void AVP_Behaviour_Alien_CycleRoundImmediate::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, "Alien Cycle Round Immediate: %s", s_aszStateStrings[ GetCurrentState() ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

AVP_Brain_Actor_Alien* AVP_Behaviour_Alien_CycleRoundImmediate::GetAlienBrain() const
{
	return static_cast< AVP_Brain_Actor_Alien* >( GetBrain() );
}

void AVP_Behaviour_Alien_CycleRoundImmediate::SwitchToState( int iState )
{
	switch ( GetCurrentState() )
	{
	case UNSET:
	case TURNING:
	case FINISHED:
		break;
	}

	SetCurrentState( iState );
	
	switch ( GetCurrentState() )
	{
	case UNSET:
	case TURNING:
		break;
	case FINISHED:
		SetFinished();
	default:
		break;
	}
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
