//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Default.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Default.h"

// asura
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "Network/Asura_Message.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Movement.h"
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Stalk.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"
#include "Moves/AVP_Moves.h"

// avp
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Alien.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_Default
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Alien_Default::AVP_Behaviour_Alien_Default( Axon_Project_Brain* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_ALIEN_DEFAULT )
, m_uAttackBehaviourType( AXON_BEHAVIOUR_TYPE_UNSET )
{
	SetCurrentState( UNSET );
}

bool AVP_Behaviour_Alien_Default::HandleMessage( const Asura_Message* pxMessage )
{
	if ( GetCurrentState() == GRAPPLE_PUPPET )
	{
		return PARENT::HandleMessage( pxMessage );
	}

	switch ( pxMessage->GetMsgID() )
	{
		case AVP_MSGID_MOVESTART_SERVER:
		case ASURA_MSG_MOVE_START:
		{
			const AVP_Msg_Move* pxMoveMsg = static_cast< const AVP_Msg_Move* >( pxMessage->GetData() );
			if ( pxMoveMsg->GetHasTargetEntity() && pxMoveMsg->GetTargetEntityIsPuppet() )
			{
				const AVP_Msg_MoveWithTargetEntity* pxMWTEMsg = static_cast< const AVP_Msg_MoveWithTargetEntity* >( pxMoveMsg );
				if ( pxMWTEMsg->GetTargetGuid() == GetBrain()->GetOwnerGuid() )
				{
					SwitchToState( GRAPPLE_PUPPET );
					return true;
				}
			}

			break;
		}

		//## - todo - this stuff should move to the movement behaviour, so that higher level behaviours are unaware of it
		case AVP_MSGID_INCOMING_MELEE_STARTED:
		{
			SwitchToState( BEING_MELEED );
			return true;
		}

		case AVP_MSGID_INCOMING_MELEE_FINISHED:
		{
			if ( GetCurrentState() == BEING_MELEED )
			{
				SwitchToState( STALKING, GetAlienBrain()->GetCurrentTargetGuid() );
				return true;
			}

			break;
		}
	}

	return PARENT::HandleMessage( pxMessage );
}

void AVP_Behaviour_Alien_Default::Deactivate()
{
	SwitchToState( UNSET );
	PARENT::Deactivate();
}

void AVP_Behaviour_Alien_Default::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	if ( fTimeslice == 0.0f ) { return; }
	UpdateStateTimer( fTimeslice );

	const AVP_AlienBehaviourData& xBehaviourData = GetAlienBrain()->GetBehaviourData();

	switch ( GetCurrentState() )
	{
		case UNSET:
		{
			//const AVP_AlienTeam_Knowledge* pxBestTarget = GetAlienBrain()->ChooseNewTarget();
			//if ( pxBestTarget )
			//{
			//	GetAlienBrain()->StartedTargeting( pxBestTarget->GetGuid() );
			//	SwitchToState( TESTING );
			//}

			if ( xBehaviourData.m_iBehaviour == AVP_AlienBehaviourData::DISAPPEAR )
			{
				SwitchToState( DISAPPEARING );
			}
			else
			{
				// Pick our best threat.
				const AVP_AlienTeam_Knowledge* pxBestTarget = GetAlienBrain()->ChooseNewTarget();
				if ( pxBestTarget )
				{
					SwitchToState( STALKING, pxBestTarget->GetGuid() );
				}
				//else
				//{
				//	SwitchToState( WANDERING );
				//}
			}

			break;
		}

		case TESTING:
		{
			break;
		}

		case WANDERING:
		{
			// Not implemented yet.
			SwitchToState( UNSET );
			break;
		}

		case STALKING:
		{
			const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetCurrentTargetKnowledge();
			if ( !pxTargetKnowledge )
			{
				SwitchToState( UNSET );
			}
			else if ( xBehaviourData.m_iBehaviour == AVP_AlienBehaviourData::DISAPPEAR )
			{
				SwitchToState( DISAPPEARING );
			}
			else
			{
				// If we are in range and allowed to attack then we should.
				const AVP_Behaviour_Alien_Stalk* pxStalkChild =
					static_cast< const AVP_Behaviour_Alien_Stalk* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_STALK ) );
				if ( !pxStalkChild || pxStalkChild->IsInError() )
				{
					SwitchToState( UNSET );
				}
				else if ( pxStalkChild->CanFinish() )
				{
					if ( AVP_AR_NEAR < GetAlienBrain()->GetCurrentTargetRange() && GetAlienBrain()->GetCurrentTargetRange() < AVP_AR_FAR )
					{
						if ( GetAlienBrain()->CanAttackCurrentTarget() )
						{
							SwitchToState( ATTACKING, GetAlienBrain()->GetCurrentTargetGuid() );
						}
					}
				}
			}

			break;
		}

		case ATTACKING:
		{
			const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetAlienBrain()->GetCurrentTargetKnowledge();
			if ( !pxTargetKnowledge )
			{
				SwitchToState( UNSET );
			}
			else if ( xBehaviourData.m_iBehaviour == AVP_AlienBehaviourData::DISAPPEAR )
			{
				SwitchToState( DISAPPEARING );
			}
			else
			{
				// If we've finished attacking we should cycle.
				const Axon_Behaviour* pxAttackChild = GetChildWithType( this, m_uAttackBehaviourType );
				if ( !pxAttackChild || pxAttackChild->IsFinished() || pxAttackChild->IsInError() )
				{
					if ( xBehaviourData.m_iPostAttackAction == AVP_AlienBehaviourData::CYCLE_ROUND_DELAYED )
					{
						SwitchToState( DISAPPEARING );
					}
					else //if ( xBehaviourData.m_iPostAttackAction == AVP_AlienBehaviourData::CYCLE_ROUND_IMMEDIATE )
					{
						// We have the opportunity to switch threats now.
						const AVP_AlienTeam_Knowledge* pxBestTarget = GetAlienBrain()->ChooseNewTarget();
						if ( pxBestTarget )
						{
							SwitchToState( STALKING, pxBestTarget->GetGuid() );
						}
						else
						{
							SwitchToState( UNSET );
						}
					}
				}
			}

			break;
		}

		case BEING_MELEED:
		{
			// Hardcoded timeout.
			if ( GetStateTimer() > 5.0f )
			{
				SwitchToState( STALKING, GetAlienBrain()->GetCurrentTargetGuid() );
			}
			break;
		}

		case GRAPPLE_PUPPET:
		{
			const Axon_Behaviour* pxGrapplePuppetChild = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_GRAPPLEPUPPET );
			if ( !pxGrapplePuppetChild || pxGrapplePuppetChild->IsFinished() || pxGrapplePuppetChild->IsInError() )
			{
				SwitchToState( STALKING, GetAlienBrain()->GetCurrentTargetGuid() );
			}
			break;
		}

		case DISAPPEARING:
		{
			// If we've finished cycling we should stalk again.
			const Axon_Behaviour* pxCyclingChild = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIEN_CYCLEROUNDDELAYED );
			if ( !pxCyclingChild || pxCyclingChild->IsFinished() || pxCyclingChild->IsInError() )
			{
				// We have the opportunity to switch threats now.
				const AVP_AlienTeam_Knowledge* pxBestTarget = GetAlienBrain()->ChooseNewTarget();
				if ( pxBestTarget )
				{
					SwitchToState( STALKING, pxBestTarget->GetGuid() );
				}
				else
				{
					SwitchToState( UNSET );
				}
			}
			break;
		}

		default:
			break;
	}
}

void AVP_Behaviour_Alien_Default::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
	}
	else
	{
		Asura_Assertf( false, ( "AVP_Behaviour_Alien_Default::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void AVP_Behaviour_Alien_Default::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );
}

#ifndef ASURA_NO_DEBUG_OUTPUT
const char* AVP_Behaviour_Alien_Default::s_aszStateStrings[] =
{
	"UNSET",
	"TESTING",
	"WANDERING",
	"STALKING",
	"ATTACKING",
	"BEING_MELEED",
	"GRAPPLE_PUPPET",
	"DISAPPEARING",
	"TERMINAL_FAILURE"
};

void AVP_Behaviour_Alien_Default::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Alien Default: %s", s_aszStateStrings[ GetCurrentState() ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

void AVP_Behaviour_Alien_Default::SwitchToState( int iNewState, Asura_Guid uGuid /*= ASURA_GUID_UNREGISTERED*/ )
{
	switch ( GetCurrentState() )
	{
		case TESTING:
		{
			DestroyChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_MOVEBETWEENCOVER );
			break;
		}

		case STALKING:
		{
			DestroyChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_STALK );
			break;
		}

		case ATTACKING:
		{
			DestroyChildWithType( m_uAttackBehaviourType );
			m_uAttackBehaviourType = AXON_BEHAVIOUR_TYPE_UNSET;

			// Unregister as attacking our target.
			GetAlienBrain()->StoppedAttackingCurrentTarget();
			break;
		}

		case BEING_MELEED:
		{
			break;
		}

		case GRAPPLE_PUPPET:
		{
			DestroyChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_GRAPPLEPUPPET );
			break;
		}

		case DISAPPEARING:
		{
			DestroyChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_CYCLEROUNDDELAYED );
			break;
		}

		case UNSET:
		default:
			break;
	}

	SetCurrentState( iNewState );
	switch ( GetCurrentState() )
	{
		case TESTING:
		{
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_MOVEBETWEENCOVER );
			break;
		}

		case STALKING:
		{
			// Register as targeting this guy.
			Asura_Assert( uGuid != ASURA_GUID_UNREGISTERED, "AVP_Behaviour_Alien_Default::SwitchToState - can't stalk an unregistered guid" );
			GetAlienBrain()->StartedTargeting( uGuid );
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_STALK );
			break;
		}

		case ATTACKING:
		{
			// Register as attacking our target.
			GetAlienBrain()->StartedAttackingCurrentTarget();

			const float fJumpVsStandFraction = GetAlienBrain()->GetAlienSquad()->GetJumpVsStandFraction();
			if ( Asura_Maths::Abs( fJumpVsStandFraction - 0.0f ) < 0.001f )
			{
				m_uAttackBehaviourType = AVP_BEHAVIOUR_TYPE_ALIEN_JUMPATTACK;
			}
			else if ( Asura_Maths::Abs( fJumpVsStandFraction - 1.0f ) < 0.001f )
			{
				m_uAttackBehaviourType = AVP_BEHAVIOUR_TYPE_ALIEN_STANDATTACK;
			}
			else
			{
				m_uAttackBehaviourType = AVP_BEHAVIOUR_TYPE_ALIEN_ATTACK;
			}

			CreateChildWithType( m_uAttackBehaviourType );
			break;
		}

		case BEING_MELEED:
		{
			GetAlienBrain()->GetMovement().Stop();
			break;
		}

		case GRAPPLE_PUPPET:
		{
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_GRAPPLEPUPPET );
			break;
		}

		case DISAPPEARING:
		{
			GetAlienBrain()->StoppedAttackingCurrentTarget();
			GetAlienBrain()->StoppedTargetingCurrentTarget();
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIEN_CYCLEROUNDDELAYED );
			break;
		}

		case UNSET:
		default:
		{
			GetAlienBrain()->StoppedAttackingCurrentTarget();
			GetAlienBrain()->StoppedTargetingCurrentTarget();
			break;
		}
	}
}

AVP_Brain_Actor_Alien* AVP_Behaviour_Alien_Default::GetAlienBrain() const
{
	return static_cast< AVP_Brain_Actor_Alien* >( GetBrain() );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
