//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_HideInShadows.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BEHAVIOUR_ALIEN_HIDEINSHADOWS_H__
#define __AVP_BEHAVIOUR_ALIEN_HIDEINSHADOWS_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_LinkList.h"
#include "Network/Asura_Guid.h"

// axon
#include "Behaviour/Axon_Project_Behaviour.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Message;
class AVP_Brain_Actor_Alien;
struct AVP_AlienTeam_Knowledge;
//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Alien_HideInShadows
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Behaviour_Alien_HideInShadows : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:
	AVP_Behaviour_Alien_HideInShadows( AVP_Brain_Actor_Alien* pxBrain );

protected:
	virtual bool Activate();
	virtual void Deactivate();
	virtual void PerFrameUpdate( const float fTimeslice );
	virtual bool HandleMessage( const Asura_Message* pxMessage );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

private:
	enum State
	{
		UNSET,
		MOVING,
		HIDE_IN_SHADOWS,
		HISS_AND_ATTACK,
		HISS_AND_REHIDE,
		ATTACK,
		WANDER,
		FINISHED,
		NUM_STATES
	};


	AVP_Brain_Actor_Alien* GetAlienBrain() const;
	void SwitchToState( const State eState );

	void PlanPathToShadow(bool bUseVantagePoint = true);
	void UpdateMoving(float fTimeSlice);
	void UpdateHideInShadows(float fTimeSlice);
	void UpdateWander();

	void UpdateHiddenStatus(float fTimeSlice);
	bool TookDamage();
	bool NeedToRehideAfterATtack() const;
	bool IsInAdajcentGPV(Asura_Guid uTargetGPV) const;
	bool TooCloseToAnyTarget() const;

	bool SelectVPDestination();
	bool SelectNonVPDestination();

	void LateHandleMessage();	
	void RequestHiss();

private:

	enum 
	{
		MAX_CANDIDATES = 32,
		MAX_GVP = 8
	};

	u_int m_uLightGeneration;
	float m_fTimeTotallyVisible;
	float m_fTimeTotallyInvisible;
	float m_fTimeVisibleByMarineLight;
	float m_fTimeSinceVisibleByMarineLight;

	float m_fMinTimeTotallyInvisible;
	float m_fMaxDistanceSq;
	float m_fLastHealth;
	bool m_bNeedToChooseToRehideAfterAttack;
	float m_fTimeTillStopAllowed;
	float m_fTimeToStayHiddenInShadows;
	Asura_Vector_3 m_xTargetPositionDuringPlan;
	Asura_Guid m_uCurrentVP;
	bool m_bDestinationIsVP;
	Asura_Vector_3 m_xDestination;
	u_short m_usDestinationWP;
	Asura_LinkList< int > m_xDelayedMsgList;
	Asura_Guid m_uPlayerGuid;
	float m_fSprintTimer;

};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BEHAVIOUR_ALIEN_HIDEINSHADOWS_H__
