//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Movement_Test.cpp
//
// Copyright (C) 2009 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Movement_Test.h"

// asura
#include "AI/Asura_Nav_Interface.h"
#include "Debugging/Asura_DebugLineRender.h"
#include "Debugging/Asura_Profiling_System.h"
#include "Debugging/Asura_Logfile.h"

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/AVP_Navigation.h"
#include "Core/BrainTypes/AVP_Brain_Actor.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Alien.h"

// avp
#include "Actor/Vertebrates/Alien/AVP_Alien_AIAnimStateMachine.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

//class AVP_Nav_TestParams : public Asura_Navigation::TestParams
//{
//public:
//	virtual bool IsValidWaypoint( const Asura_AI_NavVertex& xWaypoint ) const
//	{
//		if ( !Asura_Navigation::TestParams::IsValidWaypoint( xWaypoint ) )
//		{
//			return false;
//		}
//
//		if ( ( xWaypoint.GetFlags() & AVP_Navigation::AVP_AI_NAVVERTEX_FLAG_ALIEN ) == 0 )
//		{
//			return false;
//		}
//
//		return true;
//	}
//};

//class RememberBFF : public Asura_Navigation::BreadthFirstFunctor
//{
//public:
//	const Asura_LinkList< u_int >& GetWPIDs() const { return m_xWPIDs; }
//
//protected:
//	virtual bool VisitNode( const u_int uWPID, const Asura_AI_NavVertex& xWP )
//	{
//		m_xWPIDs.PushBack( uWPID );
//		return false;
//	}
//
//private:
//	Asura_LinkList< u_int > m_xWPIDs;
//};
//
//static RememberBFF s_xRBFF;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Movement_Test
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Movement_Test::AVP_Behaviour_Movement_Test( AVP_Brain_Actor* pxActorBrain )
: PARENT( pxActorBrain )
, m_iState( WAIT )
, m_fSpeed( 0.0f )
{
}

bool AVP_Behaviour_Movement_Test::StartPathTraversal()
{
	const bool bParentOk = PARENT::StartPathTraversal();

	if ( m_iState == WAIT )
	{
		if ( m_fSpeed < 0.33f )
		{
			AVP_Alien_AIAnimStateMachine* pxVertStateHandler = static_cast< AVP_Alien_AIAnimStateMachine* >( GetAlienBrain()->GetActor()->GetAIAnimStateMachine() );
			pxVertStateHandler->RequestMoveSlow();
		}
		else
		{
			AVP_Alien_AIAnimStateMachine* pxVertStateHandler = static_cast< AVP_Alien_AIAnimStateMachine* >( GetAlienBrain()->GetActor()->GetAIAnimStateMachine() );
			pxVertStateHandler->RequestMoveMedium();
		}
		m_iState = WALK_ROUTE;
	}

	return bParentOk;
}

void AVP_Behaviour_Movement_Test::SetSpeed( const float fSpeed )
{
	m_fSpeed = fSpeed;

	if ( m_iState == WALK_ROUTE )
	{
		if ( m_fSpeed < 0.33f )
		{
			AVP_Alien_AIAnimStateMachine* pxVertStateHandler = static_cast< AVP_Alien_AIAnimStateMachine* >( GetAlienBrain()->GetActor()->GetAIAnimStateMachine() );
			pxVertStateHandler->RequestMoveSlow();
		}
		else
		{
			AVP_Alien_AIAnimStateMachine* pxVertStateHandler = static_cast< AVP_Alien_AIAnimStateMachine* >( GetAlienBrain()->GetActor()->GetAIAnimStateMachine() );
			pxVertStateHandler->RequestMoveMedium();
		}
	}
}

Axon_Behaviour_Movement::PathTraversalState AVP_Behaviour_Movement_Test::GetPathTraversalState()
{
	if ( m_iState == WALK_ROUTE )
	{
		return PTS_MOVING;
	}
	else
	{
		return PTS_AT_DESTINATION;
	}
}

bool AVP_Behaviour_Movement_Test::StopPathTraversal()
{
	const bool bParentOk = PARENT::StopPathTraversal();

	if ( m_iState == WALK_ROUTE )
	{
		AVP_Alien_AIAnimStateMachine* pxVertStateHandler = static_cast< AVP_Alien_AIAnimStateMachine* >( GetAlienBrain()->GetActor()->GetAIAnimStateMachine() );
		pxVertStateHandler->RequestIdle();
		m_iState = WAIT;
	}

	return bParentOk;
}

void AVP_Behaviour_Movement_Test::SetOwnerOrientation( const Asura_Matrix_3x3& xOrientation )
{
	const Asura_Vector_3 xForwards( xOrientation.e13, xOrientation.e23, xOrientation.e33 );

	GetAlienBrain()->SetMovementDirection( xForwards );
	GetAlienBrain()->SetLookAtPosition( xForwards );
	GetAlienBrain()->SetAimDirection( xForwards );
}

void AVP_Behaviour_Movement_Test::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	if ( IsFinished() || IsInError() ) { return; }

	if ( m_iState == WALK_ROUTE )
	{
		const Asura_Vector_3& xOwnerPos = GetOwnerPosition();

		Asura_Vector_3 xMovementVec;
		bool bAtDest;
		if ( CalculateMovementVector( xMovementVec, bAtDest ) )
		{
			Asura_DebugLineRender::AddLine( xOwnerPos, xOwnerPos + xMovementVec, 0xffffffff );

			xMovementVec.Normalise();

			GetAlienBrain()->SetMovementDirection( xMovementVec );
			GetAlienBrain()->SetLookAtPosition( xOwnerPos + xMovementVec * 10.0f );
			GetAlienBrain()->SetAimDirection( xMovementVec );
		}
		else
		{
			SetInError();
			SetFinished();
		}

		if ( bAtDest )
		{
			AVP_Alien_AIAnimStateMachine* pxVertStateHandler = static_cast< AVP_Alien_AIAnimStateMachine* >( GetAlienBrain()->GetActor()->GetAIAnimStateMachine() );
			pxVertStateHandler->RequestIdle();

			m_iState = WAIT;
			SetFinished();
		}
	}

#ifdef _DEBUG
	Asura_Navigation::Path::RenderPath( GetPath().GetPathPoints(), 0xffffffff );
	Asura_Navigation::Path::RenderPathWPs( GetPath().GetOriginalPathPoints(), 0xffff0000 );
#endif
}

AVP_Brain_Actor_Alien* AVP_Behaviour_Movement_Test::GetAlienBrain() const
{
	return static_cast< AVP_Brain_Actor_Alien* >( GetBrain() );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
