///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Behaviour_AlienQueen_ForcedRoar.cpp
// Author : Callum Hope
// Date   : 31/3/09
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// INCLUDES

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/AlienQueen/AVP_Behaviour_AlienQueen_ForcedRoar.h"

// asura
//#include "AI/Asura_AI_Navigation.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Debugging/Asura_DebugLineRender.h"
#include "Rendering/Asura_Font_System.h"
#endif

// avp
#include "Actor/Vertebrates/AlienQueen/AVP_AlienQueen_AIAnimStateMachine.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AlienQueen/AVP_ServerActor_AlienQueen.h"
#include "Moves/Vertebrates/AlienQueen/AVP_AlienQueen_MovesMelee.h"

// axon
#include "Behaviour/BehaviourTypes/AlienQueen/AVP_Behaviour_AlienQueen_Movement.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_AlienQueen.h"

///////////////////////////////////////////////////////////////////////////////
// CLASS FUNCTIONS
// AVP_Behaviour_AlienQueen_ForcedRoar

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_AlienQueen_ForcedRoar - Constructor
// Author       : Callum Hope CAH
// Date         : 6/05/09
// Scope		: Public
////////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_AlienQueen_ForcedRoar::AVP_Behaviour_AlienQueen_ForcedRoar( AVP_Brain_Actor_AlienQueen* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_ALIENQUEEN_FORCEDROAR )
, m_eCurrentState( INIT )
, m_fTimeInState( 0.0f )
, m_uTargetGuid( ASURA_GUID_UNREGISTERED )
, m_xLastUpdatePosition( Asura_Maths::ZeroVector3 )
{
	// default for this is Fast. 
	GetAlienQueenBrain()->SetMovementMode( AVP_Navigation::MM_FAST );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_AlienQueen_ForcedRoar::Deactivate
// Author       : Callum Hope CAH
// Date         : 6/05/09
// Scope		: Virtual Protected
//
// Desc			: anything that needs closed down when this behaviour is killed. 
////////////////////////////////////////////////////////////////////////////////

void AVP_Behaviour_AlienQueen_ForcedRoar::Deactivate()
{
	// Destroy any children if we have any
	Axon_Behaviour* pxChildBehaviour = GetChild();
	while( pxChildBehaviour )
	{
		UnlinkChild( pxChildBehaviour );
		GetProjectBrain()->DestroyBehaviour( pxChildBehaviour );
		pxChildBehaviour = GetChild();
	}

	PARENT::Deactivate();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_AlienQueen_ForcedRoar::HandleMessage
// Author       : Callum Hope CAH
// Date         : 6/05/09
// Scope		: Virtual Public
////////////////////////////////////////////////////////////////////////////////

bool AVP_Behaviour_AlienQueen_ForcedRoar::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		////////////////////
		case ASURA_MSG_MOVE_START:
		////////////////////
		{
			if( m_eCurrentState == REQUEST_ROAR )
			{
				const AVP_Msg_Move* pxData = static_cast< const AVP_Msg_Move* >( pxMessage->GetData() );
				if( AVP_AlienQueen_Move_Melee_Stationary_Roar::GetStaticHash() == pxData->GetMoveID() )
				{
					SwitchToState( STARTED_ROAR );
					return true;
				}
			}
			break;
		}
		////////////////////
		case ASURA_MSG_MOVECHAINCOMPLETE:
		////////////////////
		{
			const Asura_Msg_MoveChainComplete* pxData = static_cast< const Asura_Msg_MoveChainComplete* >( pxMessage->GetData() );

			if ( pxData && pxData->GetInitialMoveID() == AVP_AlienQueen_Move_Melee_Stationary_Roar::GetStaticHash() )
			{
				SwitchToState( FINISHED );
				return true;
			}
			break;
		}
		////////////////////
		default:
		////////////////////
		{
			break;
		}
	}
	return PARENT::HandleMessage( pxMessage );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_AlienQueen_ForcedRoar::Update
// Author       : Callum Hope CAH
// Date         : 6/05/09
// Scope		: Virtual Protected
//
// Desc			: Update behaviour dependant on the current state. 
////////////////////////////////////////////////////////////////////////////////

void AVP_Behaviour_AlienQueen_ForcedRoar::Update( const float fTimeslice )
{
	PARENT::Update( fTimeslice );

	m_fTimeInState += fTimeslice;

	switch( m_eCurrentState )
	{
		////////////////////
		case INIT:
		////////////////////
		{
			// get rid of anything we are currently doing no exceptions. 
			GetAlienQueenBrain()->GetClientActor()->GetMovementSystem()->AbortCurrentMove();
			GetAlienQueenBrain()->GetActor()->GetAIAnimStateMachine()->RequestIdle();

			if( m_uTargetGuid == ASURA_GUID_UNREGISTERED )
			{
				// if we dont have a target to roar at then dont worry about moving, just carry on the the roar request. 
				SwitchToState( REQUEST_ROAR );
			}
			else
			{
				SwitchToState( MOVE_TO_POSITION );
			}
			break;
		}
		////////////////////
		case MOVE_TO_POSITION:
		////////////////////
		{
			AVP_Behaviour_AlienQueen_Movement* pxBehaviour = static_cast< AVP_Behaviour_AlienQueen_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIENQUEEN_MOVEMENT ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
			{
				// if the movement is finished or in error then continue to the next state 
				SwitchToState( REQUEST_ROAR );
			}
			break;
		}
		////////////////////
		case REQUEST_ROAR:
		////////////////////
		{
			// Hardcoded timeout in case Move never starts
			if ( GetStateTimer() > 1.0f )
			{
				SetInError( true );
				SwitchToState( FINISHED );
			}
			break;
		}
		////////////////////
		case STARTED_ROAR:
		////////////////////
		{
			break;
		}
		////////////////////
		case FINISHED:
		default:
		////////////////////
		{
			break;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_AlienQueen_ForcedRoar::GetAlienQueenBrain
// Author       : Callum Hope CAH
// Date         : 6/05/09
// Scope		: Private
//
// Desc			: returns a casted Axon_Brain to allow the queen functionality quickly.
////////////////////////////////////////////////////////////////////////////////

AVP_Brain_Actor_AlienQueen* AVP_Behaviour_AlienQueen_ForcedRoar::GetAlienQueenBrain() const
{
	return static_cast<AVP_Brain_Actor_AlienQueen*>( GetProjectBrain() );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_AlienQueen_ForcedRoar::SwitchToState
// Author       : Callum Hope CAH
// Date         : 6/05/09
// Scope		: Private
//
// Desc			: Closes one state and starts another, in that order. 
////////////////////////////////////////////////////////////////////////////////

void AVP_Behaviour_AlienQueen_ForcedRoar::SwitchToState( const State eNextState )
{
	// Do any work we need to exit our current state.
	switch( m_eCurrentState )
	{
		////////////////////
		case INIT:
		////////////////////
		{
			break;
		}
		////////////////////
		case MOVE_TO_POSITION:
		////////////////////
		{
			if( m_uTargetGuid != ASURA_GUID_UNREGISTERED )
			{
				// make sure we finish at leate looking in the right direction if possible. 
				AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( m_uTargetGuid );

				if( pxActor )
				{
					Asura_Vector_3 xLookDirection = pxActor->GetPosition() - GetAlienQueenBrain()->GetActor()->GetPosition();
					xLookDirection.Normalise();

					Asura_Matrix_3x3 xQnOri = GetAlienQueenBrain()->GetActor()->GetOrientation();
					Asura_Vector_3 xQueenDirection = Asura_Vector_3( xQnOri.e13, xQnOri.e23, xQnOri.e33 );
					xQueenDirection.Normalise();

					float fDotProduct = xQueenDirection * xLookDirection;

					// look at target as long as they are withink 30 degrees or out forward orientation. 
					if( fDotProduct > GetAlienQueenBrain()->GetMaxHeadTurnCosine() )
					{
						Asura_Vector_3 xChestPosition( Asura_Maths::ZeroVector3 );

						pxActor->GetChestPosition( xChestPosition );

						GetAlienQueenBrain()->AimAtPosition( xChestPosition );
						GetAlienQueenBrain()->SetLookAtPosition( xChestPosition );
					}
					else
					{
						// if the player is not in front make sure the head is not off on some tangent from a previous job. 
						GetAlienQueenBrain()->LookAndAimForward();
					}
				}
			}

			DestroyChildWithType( AVP_BEHAVIOUR_TYPE_ALIENQUEEN_MOVEMENT );

			break;
		}
		////////////////////
		case REQUEST_ROAR:
		////////////////////
		{
			break;
		}
		////////////////////
		case STARTED_ROAR:
		////////////////////
		{
			break;
		}
		////////////////////
		case FINISHED:
		default:
		////////////////////
		{
			break;
		}
	}

	// Do any work we need to change into our next state.
	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;

	switch( m_eCurrentState )
	{
		////////////////////
		case INIT:
		////////////////////
		{
			break;
		}
		////////////////////
		case MOVE_TO_POSITION:
		////////////////////
		{
			if( m_uTargetGuid == ASURA_GUID_UNREGISTERED )
			{
				// if we dont have a target to roar at then dont worry about moving, just carry on the the roar request. 
				SwitchToState( REQUEST_ROAR );
			}

			if( ! FindNewDestinationAndPath() )
			{
				// if the route finding didnt work or was not needed then go onto the roar request state. 
				SwitchToState( REQUEST_ROAR );
			}

			break;
		}
		////////////////////
		case REQUEST_ROAR:
		////////////////////
		{
			RequestStaticRoarMove();
			break;
		}
		////////////////////
		case STARTED_ROAR:
		////////////////////
		{
			break;
		}
		////////////////////
		case FINISHED:
		default:
		////////////////////
		{
			SetFinished( true );
			break;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_AlienQueen_ForcedRoar::RequestStaticRoarMove
// Author       : Callum Hope CAH
// Date         : 6/05/09
// Scope		: Private
//
// Desc			: Subroutein to set up the specific Melee move in a MOVE_START message and send it. 
////////////////////////////////////////////////////////////////////////////////

void AVP_Behaviour_AlienQueen_ForcedRoar::RequestStaticRoarMove()
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Msg_Melee_Move ) );
	if ( pxMessage != NULL )
	{
		AVP_Msg_Melee_Move* pxData = static_cast< AVP_Msg_Melee_Move* >( pxMessage->GetData() );

		pxMessage->SetFromGuid( GetAlienQueenBrain()->GetOwnerGuid() );
		pxMessage->SetToGuid( GetAlienQueenBrain()->GetOwnerGuid() );

		pxData->ClearFlags();
		pxData->SetIgnoredByOwner( false );
		pxData->SetMoveID( AVP_AlienQueen_Move_Melee_Stationary_Roar::GetStaticHash() );
		pxData->SetVariation( 0 );
		pxData->SetTargetGuid( m_uTargetGuid );
		pxData->SetTargetEntityIsPuppet( false );
		
		Asura_Network::Send( pxMessage );
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Behaviour_AlienQueen_ForcedRoar::FindNewDestinationAndPath
// Author       : Callum Hope CAH
// Date         : 6/05/09
// Scope		: Private
//
// Desc			: If the queen needs to move in order to face the roar target this finds the best way to do it and sends her on her way
//				  This funciton will also cancel any movement destination currently existing as a child of this behaviour. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_Behaviour_AlienQueen_ForcedRoar::FindNewDestinationAndPath()
{
	AVP_Navigation::NavData& xNavData = GetAlienQueenBrain()->GetNavData();

	if( m_uTargetGuid != ASURA_GUID_UNREGISTERED )
	{
		AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( m_uTargetGuid );

		if( pxActor )
		{
			// get the position of the target of Roar the direction it is in and hence the distance to it from the queen before normalising it. 
			Asura_Vector_3 xDestination = pxActor->GetPosition();
			Asura_Vector_3 xDirection = ( xDestination - GetAlienQueenBrain()->GetQueenActor()->GetPosition() );
			float fDistanceToTarget = xDirection.Magnitude();
			xDirection.Normalise();

			// now change the destination to be only a few meters away from the queen inthe correct direction. ( if possible )
			if( fDistanceToTarget > 5.0f )
			{
				xDestination = GetAlienQueenBrain()->GetQueenActor()->GetPosition() + ( xDirection * 5.0f );
			}
			else
			{
				// else we are too close to attempt a turn, so return fail. 
				return false;
			}

			// set up test params based on the queens nav data and find the neares possible waypoint to our desired destination. 
			Asura_Vector_3 xNearestWpDestination = xDestination;
			AVP_Navigation::TestParams xTestParams;
			xTestParams.m_uPathfinderFlags = xNavData.m_uPathfinderFlags;
			xTestParams.m_usZone = xNavData.m_usCurrentZone;
			AVP_Navigation::ResolveToValidPosition( xNearestWpDestination, xTestParams );

			// do a check to make sure the nearest possible position found is not very far away. 
			if( ( xNearestWpDestination - xDestination ).MagnitudeSquared() > 40.0f )
			{
				return false;
			}

			// check if we already have a move behaviour child, if not make one.  
			AVP_Behaviour_AlienQueen_Movement* pxBehaviour = static_cast< AVP_Behaviour_AlienQueen_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_ALIENQUEEN_MOVEMENT ) );
			if( ! pxBehaviour )
			{
				pxBehaviour = static_cast<AVP_Behaviour_AlienQueen_Movement*>( CreateChildWithType( AVP_BEHAVIOUR_TYPE_ALIENQUEEN_MOVEMENT ) );
			}

			// If still no behaviour then we have issues. 
			if( ! pxBehaviour )
			{
				m_bIsInError = true;
				return false;
			}

			// if we cant find a route to the position or we are already there
			if( ! pxBehaviour->GetToPosition( xNearestWpDestination ) || pxBehaviour->IsFinished() )
			{
				return false;
			}

			m_xLastUpdatePosition = xNearestWpDestination;
			return true;
		}
	}
	return false;	
}

#ifndef ASURA_NO_DEBUG_OUTPUT

static const char* s_aszStateStrings[] =
{
	"INIT",
	"MOVE_TO_POSITION",
	"REQUEST_ROAR",
	"STARTED_ROAR",
	"FINISHED"
};

DEBUGVAR Asura_Vector_3 s_xOffSetUp = Asura_Vector_3( 0.0f, -0.5f, 0.0f );

void AVP_Behaviour_AlienQueen_ForcedRoar::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "AlienQueen Forced Roar: %s", s_aszStateStrings[ m_eCurrentState ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

	// render the a line to and the position of the last destination for the roar to happen at. 
	if( m_eCurrentState == MOVE_TO_POSITION )
	{
		Asura_DebugLineRender::AddLine( GetAlienQueenBrain()->GetQueenActor()->GetPosition() + s_xOffSetUp, m_xLastUpdatePosition, 0x5500ffff );
		Asura_DebugLineRender::AddSphere( m_xLastUpdatePosition, 0.5f, 0x5500ffff, 8, 4 );
	}
}

#endif //ASURA_NO_DEBUG_OUTPUT
