///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Behaviour_AlienQueen_Stunned.cpp
// Author : Callum Hope
// Date   : 31/3/09
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// INCLUDES

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/AlienQueen/AVP_Behaviour_AlienQueen_Stunned.h"

// asura
#include "AI/Asura_AI_Navigation.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// avp
#include "Actor/Vertebrates/AlienQueen/AVP_AlienQueen_AIAnimStateMachine.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h" 
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Moves/Vertebrates/AlienQueen/AVP_AlienQueen_MovesMelee.h"

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_AlienQueen.h"

///////////////////////////////////////////////////////////////////////////////
// CLASS FUNCTIONS
// AVP_Behaviour_AlienQueen_Stunned

AVP_Behaviour_AlienQueen_Stunned::AVP_Behaviour_AlienQueen_Stunned( AVP_Brain_Actor_AlienQueen* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_ALIENQUEEN_STUNNED )
, m_eCurrentState( INIT )
, m_fTimeInState( 0.0f )
{
	// default for this is slow. 
	GetAlienQueenBrain()->SetMovementMode( AVP_Navigation::MM_SLOW );
}

void AVP_Behaviour_AlienQueen_Stunned::Deactivate()
{
	// Destroy any children if we have any
	Axon_Behaviour* pxChildBehaviour = GetChild();
	while( pxChildBehaviour )
	{
		UnlinkChild( pxChildBehaviour );
		GetProjectBrain()->DestroyBehaviour( pxChildBehaviour );
		pxChildBehaviour = GetChild();
	}

	PARENT::Deactivate();
}

bool AVP_Behaviour_AlienQueen_Stunned::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		////////////////////
		case ASURA_MSG_MOVE_START:
		////////////////////
		{
			if( m_eCurrentState == REQUEST_STUNNED )
			{
				const AVP_Msg_Move* pxData = static_cast< const AVP_Msg_Move* >( pxMessage->GetData() );
				if( AVP_AlienQueen_Move_Stunned_Stumble::GetStaticHash() == pxData->GetMoveID() )
				{
					SwitchToState( STARTED_STUNNED );
					return true;
				}
			}
			break;
		}
		////////////////////
		case ASURA_MSG_MOVECHAINCOMPLETE:
		////////////////////
		{
			const Asura_Msg_MoveChainComplete* pxData = static_cast< const Asura_Msg_MoveChainComplete* >( pxMessage->GetData() );

			if ( pxData && pxData->GetInitialMoveID() == AVP_AlienQueen_Move_Stunned_Stumble::GetStaticHash() )
			{
				SwitchToState( FINISHED );
				return true;
			}
			break;
		}
		////////////////////
		default:
		////////////////////
		{
			break;
		}
	}
	return PARENT::HandleMessage( pxMessage );
}

void AVP_Behaviour_AlienQueen_Stunned::Update( const float fTimeslice )
{
	PARENT::Update( fTimeslice );

	m_fTimeInState += fTimeslice;

	switch( m_eCurrentState )
	{
		////////////////////
		case INIT:
		////////////////////
		{
			// get rid of anything we are currently doing no exceptions. 
			GetAlienQueenBrain()->GetClientActor()->GetMovementSystem()->AbortCurrentMove();
			GetAlienQueenBrain()->GetActor()->GetAIAnimStateMachine()->RequestIdle();
			SwitchToState( REQUEST_STUNNED );
			break;
		}
		////////////////////
		case REQUEST_STUNNED:
		////////////////////
		{
			// Hardcoded timeout in case Move never starts
			if ( GetStateTimer() > 1.0f )
			{
				SetInError( true );
				SwitchToState( FINISHED );
			}
			break;
		}
		////////////////////
		case STARTED_STUNNED:
		////////////////////
		{
			break;
		}
		////////////////////
		case FINISHED:
		default:
		////////////////////
		{
			break;
		}
	}
}

AVP_Brain_Actor_AlienQueen* AVP_Behaviour_AlienQueen_Stunned::GetAlienQueenBrain() const
{
	return static_cast< AVP_Brain_Actor_AlienQueen* >( GetProjectBrain() );
}

void AVP_Behaviour_AlienQueen_Stunned::SwitchToState( const State eNextState )
{
	// Do any work we need to exit our current state.
	switch( m_eCurrentState )
	{
		////////////////////
		case INIT:
		////////////////////
		{
			break;
		}
		////////////////////
		case REQUEST_STUNNED:
		////////////////////
		{
			break;
		}
		////////////////////
		case STARTED_STUNNED:
		////////////////////
		{
			// TODO, make sure that if the move is still playing it is aborted. 
			break;
		}
		////////////////////
		case FINISHED:
		default:
		////////////////////
		{
			break;
		}
	}

	// Do any work we need to change into our next state.
	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;

	switch( m_eCurrentState )
	{
		////////////////////
		case INIT:
		////////////////////
		{
			break;
		}
		////////////////////
		case REQUEST_STUNNED:
		////////////////////
		{
			RequestStunnedStumbleMove();
			break;
		}
		////////////////////
		case STARTED_STUNNED:
		////////////////////
		{
			break;
		}
		////////////////////
		case FINISHED:
		default:
		////////////////////
		{
			SetFinished( true );
			break;
		}
	}
}

void AVP_Behaviour_AlienQueen_Stunned::RequestStunnedStumbleMove()
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Msg_Move ) );
	if ( pxMessage != NULL )
	{
		AVP_Msg_Move* pxData = static_cast< AVP_Msg_Move* >( pxMessage->GetData() );

		pxMessage->SetFromGuid( GetAlienQueenBrain()->GetOwnerGuid() );
		pxMessage->SetToGuid( GetAlienQueenBrain()->GetOwnerGuid() );

		pxData->ClearFlags();
		pxData->SetIgnoredByOwner( false );
		pxData->SetMoveID( AVP_AlienQueen_Move_Stunned_Stumble::GetStaticHash() );

		Asura_Network::Send( pxMessage );
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT

static const char* s_aszStateStrings[] =
{
	"INIT",
	"REQUEST_STUNNED",
	"STARTED_STUNNED",
	"FINISHED"
};

void AVP_Behaviour_AlienQueen_Stunned::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "AlienQueen Stunned: %s", s_aszStateStrings[ m_eCurrentState ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}

#endif //ASURA_NO_DEBUG_OUTPUT
