//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger_HideInShadows.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_HideInShadows.h"

// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Debugging/Asura_DebugLineRender.h"
#include "Rendering/Asura_Font_System.h"
#endif
// avp
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AIAnimStateMachine.h"
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_MeleeCounter_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Plane.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/Damage/AVP_EntityDamage_Request.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"


// axon
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_AttackPrey.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Movement.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_MoveToPrey.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_ReturnHome.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Wander.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/AxonAVP_Messages.h"
#include "Core/AVP_Navigation_Types.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "Core/BrainTypes/AVP_Brain_Actor_FaceHugger.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"


//////////////////////////////////////////////////////////////////////////////
//
// AVP_FaceHuggerShadowsAroundTargetFunctor - used to find a point away from a target in shadows
//
//////////////////////////////////////////////////////////////////////////////

class AVP_FaceHuggerShadowsAroundTargetFunctor : public AVP_Navigation::BreadthFirstSearch
{
public:
	struct WPInfo
	{
		u_short m_usWPID;
		float m_fDistanceFromCenterSq;
		float m_fDistanceFromStartSq;
		float m_fScore;
	};


	virtual bool LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags );
	virtual bool VisitNode( const int iWaypointIndex );

	void AddGPV(const AVP_ServerEntity_GamePlayVolume* pxGPV);
	void Filter(float fRangeBand);
	u_short GetWPID() const;

	AVP_FaceHuggerShadowsAroundTargetFunctor( const Asura_Vector_3& xCenter, const Asura_Vector_3& xStart, u_short usWPID, const float fMaxSearchRange, float fMinRangeFromCenter, float fMinRangeFromStart ) 
		: AVP_Navigation::BreadthFirstSearch(usWPID, fMaxSearchRange)
		, m_usNumGPV(0)
		, m_usNumWPInfo(0)
		, m_fMinRangeFromCenterSq(fMinRangeFromCenter * fMinRangeFromCenter)
		, m_fMinRangeFromStartSq(fMinRangeFromStart * fMinRangeFromStart)
		, m_xCenter(xCenter)
		, m_xStart(xStart)
		, m_fMinRangeFromCenterSqFound(Asura_Maths::LargeFloat)
		, m_fTotalScore(0.0f)
	{}

	enum { MAX_GPV = 8 };

protected:

	u_short m_usNumGPV;
	const AVP_ServerEntity_GamePlayVolume* m_apxGPV[MAX_GPV];

	enum { MAX_WPID = 64 };
	u_short m_usNumWPInfo;
	WPInfo m_axWPInfo[MAX_WPID];
	const float m_fMinRangeFromCenterSq;
	const float m_fMinRangeFromStartSq;
	const Asura_Vector_3 m_xCenter;
	const Asura_Vector_3 m_xStart;
	float m_fMinRangeFromCenterSqFound;
	float m_fTotalScore;

	typedef AVP_Navigation::BreadthFirstSearch PARENT;
};

bool AVP_FaceHuggerShadowsAroundTargetFunctor::LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags )
{
	return AVP_Navigation::AlienLinkFlagsTest( pxLink, uPathfinderFlags ) && !(pxLink->GetFlags() & AVP_AI_CONNECTION_FLAG_GS2);
}

bool AVP_FaceHuggerShadowsAroundTargetFunctor::VisitNode( const int iWaypointIndex )
{
	const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );
	Asura_Assert(pxWP , "Way point not found during search");
	if (!pxWP)
	{
		return false;
	}

	float fDistanceFromCenterSq = (pxWP->GetPosition()-m_xCenter).MagnitudeSquared();

	if ( fDistanceFromCenterSq < m_fMinRangeFromCenterSq )
	{
		return false;
	}

	float fDistanceFromStartSq = (pxWP->GetPosition()-m_xStart).MagnitudeSquared();

	const float fLuminaceFactor = AVP_Waypoint_Manager::GetWPLuminance( iWaypointIndex );
	if (fLuminaceFactor > AVP_Waypoint_Manager::GetDefaultMaxHideLuminance())
	{
		return false;
	}

	if (m_usNumGPV != 0)
	{
		bool bFound = false;
		for (u_short u = 0; u < m_usNumGPV && !bFound; ++u)
		{
			if (m_apxGPV[u]->Contains(pxWP->GetPosition()))
			{
				bFound = true;
			}
		}

		if (!bFound)
		{
			return false;
		}
	}

	if (m_usNumWPInfo < MAX_WPID)
	{
		m_axWPInfo[m_usNumWPInfo].m_usWPID = static_cast<u_short>(iWaypointIndex);
		m_axWPInfo[m_usNumWPInfo].m_fDistanceFromCenterSq = fDistanceFromCenterSq;
		m_axWPInfo[m_usNumWPInfo].m_fDistanceFromStartSq = fDistanceFromStartSq;
		++m_usNumWPInfo;
		if (m_fMinRangeFromCenterSqFound > fDistanceFromCenterSq)
		{
			m_fMinRangeFromCenterSqFound = fDistanceFromCenterSq;
		}
	}

	return false;
}

void AVP_FaceHuggerShadowsAroundTargetFunctor::AddGPV(const AVP_ServerEntity_GamePlayVolume* pxGPV)
{
	if (m_usNumGPV < MAX_GPV)
	{
		m_apxGPV[m_usNumGPV] = pxGPV;
		++m_usNumGPV;
	}
}

DEBUGVAR float dbg_fGoodCloseToCenterPointScore = 5.0f;
DEBUGVAR float dbg_fGoodCloseToStartPointScore = 2.0f;
void AVP_FaceHuggerShadowsAroundTargetFunctor::Filter(float fRangeBand)
{
	float fMaxDistanceFromCenterSq = Asura_Maths::Sqrt(m_fMinRangeFromCenterSqFound) + fRangeBand;
	fMaxDistanceFromCenterSq *= fMaxDistanceFromCenterSq;

	m_fTotalScore = 0.0f;

	for (u_short u = 0; u < m_usNumWPInfo; ++u)
	{
		m_axWPInfo[u].m_fScore = 0.0f;
		if (m_axWPInfo[u].m_fDistanceFromCenterSq  < fMaxDistanceFromCenterSq)
		{
			m_axWPInfo[u].m_fScore += dbg_fGoodCloseToCenterPointScore * ( fMaxDistanceFromCenterSq - m_axWPInfo[u].m_fDistanceFromCenterSq) / (fMaxDistanceFromCenterSq - m_fMinRangeFromCenterSq) ;
		}
		else
		{
			m_axWPInfo[u].m_fScore += 0.0f;
		}

		if (m_axWPInfo[u].m_fDistanceFromStartSq  < m_fMinRangeFromStartSq)
		{
			m_axWPInfo[u].m_fScore += 0.0f;
		}
		else
		{
			m_axWPInfo[u].m_fScore += dbg_fGoodCloseToStartPointScore;
		}

		m_fTotalScore += m_axWPInfo[u].m_fScore;

	}
}

u_short AVP_FaceHuggerShadowsAroundTargetFunctor::GetWPID() const
{

	if (m_fTotalScore == 0.0f)
	{
		return ASURA_INVALID_WAYPOINT_ID;
	}

	float fDestScore = Asura_Random::GetFloat(0.0f, m_fTotalScore - 1e-6f);
	

	for (u_short u = 0; u < m_usNumWPInfo; ++u)
	{
		if (m_axWPInfo[u].m_fScore == 0.0f)
		{
			continue;
		}
		
		if (fDestScore < m_axWPInfo[u].m_fScore)
		{
			return m_axWPInfo[u].m_usWPID;
		}

		fDestScore -= m_axWPInfo[u].m_fScore;
	}

	return ASURA_INVALID_WAYPOINT_ID;

}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_FaceHuggerMoveAvoidingTargetInShadowsFunctor
//
//////////////////////////////////////////////////////////////////////////////
DEBUGVAR float dbg_fCloseMultiplier = 100.0f;
DEBUGVAR float dbg_fTransitionMultiplier = 0.5f;

class AVP_FaceHuggerMoveAvoidingTargetInShadowsFunctor : public AVP_Navigation::FaceHuggerAStarFunctor 
{
public:

	virtual float EstimateTraversalCost( const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags );
	virtual float EstimateCostToGoal( const int iWaypointIndex );

	AVP_FaceHuggerMoveAvoidingTargetInShadowsFunctor(const Asura_Vector_3& xCenter, float fMinDistance) 
		: AVP_Navigation::FaceHuggerAStarFunctor() 
		, m_xCenter(xCenter)
		, m_fMinDistanceSq(fMinDistance * fMinDistance) 
	{}
private:
	Asura_Vector_3 m_xCenter;
	float m_fMinDistanceSq;

	typedef AVP_Navigation::FaceHuggerAStarFunctor PARENT;
};

float AVP_FaceHuggerMoveAvoidingTargetInShadowsFunctor::EstimateTraversalCost( const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags )
{
	const float fBaseCost = PARENT::EstimateTraversalCost( iWaypointIndex, iLinkIndex, uPathfinderFlags );

	if (fBaseCost < 0.0f)
	{
		return -1.0f;
	}

	const Asura_AI_NavVertex* pxFromWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );
	const Asura_AI_NavVertex* pxToWP   = Asura_AI_Navigation_System::GetWaypoint( pxFromWP->m_pxConnections[ iLinkIndex ].m_usIndex );


	// care a bit but not reallyt.
	float fMismatchedAgilityCost = 0.0f;

	if ( pxToWP->IsWallWaypoint() ) // hate to be on the wall 
	{
		fMismatchedAgilityCost = fBaseCost / 4.0f;
	}
	else if ( pxToWP->IsCeilingWaypoint() ) // hate even more to be on the ceiling
	{
		fMismatchedAgilityCost = fBaseCost / 2.0f;
	}

	// Add a cost for transition links to stop unnecessary hopping on and off walls, etc.
	float fTransitionCost = 0.0f;
	const u_short usLinkFlags = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex )->m_pxConnections[ iLinkIndex ].m_usFlags;
	if ( AVP_Navigation::IsTransitionLink( usLinkFlags ) )
	{
		fTransitionCost = fBaseCost * dbg_fTransitionMultiplier;
	}

	float fCloseToTargetCost = 0.0f;

	Asura_Vector_3 xNearestPoint;
	Asura_Maths::GetNearestPointOnLineSegment(pxFromWP->GetPosition(), pxToWP->GetPosition(), m_xCenter, xNearestPoint);
	float fDistanceSq = (m_xCenter - xNearestPoint).MagnitudeSquared();

	if (  fDistanceSq < m_fMinDistanceSq ) 
	{
		fCloseToTargetCost = fBaseCost * dbg_fCloseMultiplier * (m_fMinDistanceSq - fDistanceSq)/m_fMinDistanceSq;
	}

	// hate lite
	float fPrevLuminace = AVP_Waypoint_Manager::GetWPLuminance( iWaypointIndex );
	float fLuminace = AVP_Waypoint_Manager::GetWPLuminance( pxFromWP->m_pxConnections[ iLinkIndex ].m_usIndex );
	float fLuminaceFactor = 1.0f;

	if (fLuminace < AVP_Waypoint_Manager::GetDefaultMaxHideLuminance()) // moving into shadow
	{
		if (fPrevLuminace < AVP_Waypoint_Manager::GetDefaultMaxHideLuminance()) // was already in shadow
		{
			fLuminaceFactor = fPrevLuminace > fLuminace ? 0.1f : 0.5f;
		}
		else // moving away from the light
		{
			fLuminaceFactor = 0.0f;
		}

	}
	else // moving into the light
	{
		if (fPrevLuminace < AVP_Waypoint_Manager::GetDefaultMaxHideLuminance()) // was already in shadow
		{
			fLuminaceFactor = 10.0f;
		}
		else // was already in the light 
		{
			fLuminaceFactor = fPrevLuminace > fLuminace ? 5.0f : 8.0f;
		}

	}


	return ( fLuminaceFactor * (fBaseCost + fMismatchedAgilityCost + fTransitionCost ) + fCloseToTargetCost);

}

float AVP_FaceHuggerMoveAvoidingTargetInShadowsFunctor::EstimateCostToGoal( const int iWaypointIndex )
{
	float fLuminace = AVP_Waypoint_Manager::GetWPLuminance( iWaypointIndex );

	if (fLuminace > AVP_Waypoint_Manager::GetDefaultMaxHideLuminance())
	{
		return PARENT::EstimateCostToGoal(iWaypointIndex);
	}
	else
	{
		return 0.5f * PARENT::EstimateCostToGoal(iWaypointIndex);
	}

}


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger_HideInShadows
//
//////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fIgnoreTargetTime = 1.0f;
DEBUGVAR float dbg_fRangeBand = 6.0f;
DEBUGVAR float dbg_fForgetMarineLightTime = 0.5f;

DEBUGVAR float dbg_fMaxTimeSinceLastFindTarget = 0.5f;
DEBUGVAR float dbg_fTimeBetweenIdle = 4.0f;
DEBUGVAR float dbg_fTimeInIdle = 1.0f;

DEBUGVAR float dbg_fLargeArriveRange = 3.0f;

DEBUGVAR float dbg_fMinTimeToStayIdleInShadows = 5.0f;
DEBUGVAR float dbg_fMaxTimeToStayIdleInShadows = 10.0f;
DEBUGVAR float dbg_fTimeTillStopAllowedAfterAttack = 3.0f;
DEBUGVAR float dbg_fMaxTimeTillStopAllowed = 0.0f; // for now don't force movement after in shadow since the env is very bright and finding dark areas is hard.
DEBUGVAR float dbg_fMinMoveTimeDuringRehide = 3.0f;
DEBUGVAR float dbg_fMinMoveDistance = 7.0f;
DEBUGVAR float dbg_fMaxMoveForReplanSq = 4.0f;


AVP_Behaviour_FaceHugger_HideInShadows::AVP_Behaviour_FaceHugger_HideInShadows( AVP_Brain_Actor_FaceHugger* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_FACEHUGGER_HIDE_IN_SHADOWS )
, m_fTimeSinceLastFindTarget(0.0f)
, m_uLightGeneration(0)
, m_fTimeTotallyVisible(0.0f)
, m_fTimeTotallyInvisible(0.0f)
, m_fTimeVisibleByMarineLight(0.0f)
, m_fTimeSinceVisibleByMarineLight(0.0f)
, m_fMaxDistanceSq(0.0f)
, m_fLastHealth(pxBrain->GetActor()->GetHealth())
, m_fTimeTillStopAllowed(0.0f)
, m_fTimeToStayHiddenInShadows(0.0f)
, m_xTargetPositionDuringPlan(Asura_Maths::ZeroVector3)
{
	SetCurrentState( UNSET );
}

bool AVP_Behaviour_FaceHugger_HideInShadows::Activate()
{
	return PARENT::Activate();
}

void AVP_Behaviour_FaceHugger_HideInShadows::Deactivate()
{
	SwitchToState( UNSET );
	PARENT::Deactivate();
}

void AVP_Behaviour_FaceHugger_HideInShadows::Update( const float fTimeslice )
{
	PARENT::Update( fTimeslice );

	if (!GetFaceHuggerBrain()->CanPlanMove())
	{
		return;
	}

	switch ( GetCurrentState() )
	{
	case UNSET:
		SwitchToState(IDLE);
		break;
	case IDLE:
		if (!GetFaceHuggerBrain()->FarFromSpawn() && m_fTimeSinceLastFindTarget > dbg_fMaxTimeSinceLastFindTarget)
		{		
			m_fTimeSinceLastFindTarget = 0.0f;
			if (GetFaceHuggerBrain()->FindTarget(false))
			{
				SwitchToState( MOVE_IN_SHADOWS );
			}
		}
		else if (GetStateTimer() > dbg_fTimeInIdle)
		{
			SwitchToState(WANDER);
		}		
		break;
	case WANDER:
		if ( m_fTimeSinceLastFindTarget > dbg_fMaxTimeSinceLastFindTarget )
		{
			m_fTimeSinceLastFindTarget = 0.0f;
			if (GetFaceHuggerBrain()->FindTarget(false) )
			{				
				SwitchToState( MOVE_IN_SHADOWS );
			}
			else if (GetStateTimer() > dbg_fTimeBetweenIdle )
			{
				SwitchToState(IDLE);
			}
		}
		break;
	case MOVE_IN_SHADOWS:
		break;
	case HIDE_IN_SHADOWS:
		break;
	case MOVE_TO_PREY:
		if(!GetFaceHuggerBrain()->IsTargetInPGVolumes() && GetFaceHuggerBrain()->WasTargetLost() )
		{
			SwitchToState(RETURN_HOME);
		}
		else if (m_fTimeSinceLastFindTarget > dbg_fMaxTimeSinceLastFindTarget || !GetFaceHuggerBrain()->IsTargetValid() )
		{
			const AVP_AlienTeam_Knowledge* pxTarget = GetFaceHuggerBrain()->GetTarget();
			Asura_Guid uLastGuid = pxTarget ? pxTarget->GetGuid() : ASURA_GUID_UNREGISTERED;
			m_fTimeSinceLastFindTarget = 0.0f;
			GetFaceHuggerBrain()->FindTarget(false);
			const AVP_AlienTeam_Knowledge* pxNewTarget = GetFaceHuggerBrain()->GetTarget();
			Asura_Guid uNewGuid = pxNewTarget ? pxNewTarget->GetGuid() : ASURA_GUID_UNREGISTERED;
			if (uNewGuid != ASURA_GUID_UNREGISTERED && uNewGuid != uLastGuid)
			{
				SwitchToState(MOVE_TO_PREY);
			}	
		}
		break;
	case ATTACKING:
		break;
	case RETURN_HOME:
		AVP_Behaviour_FaceHugger_ReturnHome* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_ReturnHome* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_RETURN_HOME ) );
		if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
		{
			SwitchToState(IDLE);
		}
		else if (GetFaceHuggerBrain()->IsTargetVisible())
		{
			SwitchToState(MOVE_IN_SHADOWS); // MOVE_TO_PREY
		}
		else if (m_fTimeSinceLastFindTarget > dbg_fMaxTimeSinceLastFindTarget)
		{
			const AVP_AlienTeam_Knowledge* pxTarget = GetFaceHuggerBrain()->GetTarget();
			Asura_Guid uLastGuid = pxTarget ? pxTarget->GetGuid() : ASURA_GUID_UNREGISTERED;
			m_fTimeSinceLastFindTarget = 0.0f;
			GetFaceHuggerBrain()->FindTarget(false);
			const AVP_AlienTeam_Knowledge* pxNewTarget = GetFaceHuggerBrain()->GetTarget();
			Asura_Guid uNewGuid = pxNewTarget ? pxNewTarget->GetGuid() : ASURA_GUID_UNREGISTERED;
			if (uNewGuid != ASURA_GUID_UNREGISTERED && uNewGuid != uLastGuid)
			{
				SwitchToState(MOVE_IN_SHADOWS); // MOVE_TO_PREY
			}	
		}
		break;
	}
}

void AVP_Behaviour_FaceHugger_HideInShadows::PerFrameUpdate( const float fTimeslice )
{
	PARENT::Update( fTimeslice );

	UpdateStateTimer(fTimeslice);
	m_fTimeSinceLastFindTarget += fTimeslice;

	UpdateHiddenStatus(fTimeslice);
	m_fTimeTillStopAllowed = Asura_Maths::Min(m_fTimeTillStopAllowed, dbg_fMaxTimeTillStopAllowed);
	
	if (!GetFaceHuggerBrain()->CanPlanMove())
	{
		return;
	}

	switch ( GetCurrentState() )
	{
	case UNSET:
		break;
	case IDLE:
		break;
	case WANDER:
		{
			AVP_Behaviour_FaceHugger_Wander* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Wander* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_WANDER ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
			{
				SwitchToState( IDLE );
			}
		}
		break;
	case MOVE_IN_SHADOWS:
		UpdateMovingInShadows(fTimeslice);
		break;
	case HIDE_IN_SHADOWS:
		UpdateHideInShadows(fTimeslice);
		break;
	case MOVE_TO_PREY:
		{
			AVP_Behaviour_FaceHugger_MoveToPrey* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_MoveToPrey* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVETOPREY ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
			{
				SwitchToState(RETURN_HOME);
			}
			else if ( GetFaceHuggerBrain()->ShouldAttack() )
			{
				SwitchToState(ATTACKING);
			}
		}
		break;
	case ATTACKING:
		{
			AVP_Behaviour_FaceHugger_AttackPrey* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_AttackPrey* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACKPREY ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
			{
				if(!GetFaceHuggerBrain()->IsTargetInPGVolumes() && GetFaceHuggerBrain()->WasTargetLost())
				{
					SwitchToState(RETURN_HOME);
				}
				else if (GetFaceHuggerBrain()->GetBehaviourData().m_fChaneToRehideAfterAttackFraction == 0.0f)
				{
					SwitchToState(MOVE_TO_PREY); 
				}
				else if (GetFaceHuggerBrain()->GetBehaviourData().m_fChaneToRehideAfterAttackFraction == 1.0f)
				{
					SwitchToState(MOVE_IN_SHADOWS); 
				}
				else if (Asura_Random::GetFloat(0.0f, 1.0f) > GetFaceHuggerBrain()->GetBehaviourData().m_fChaneToRehideAfterAttackFraction)
				{
					SwitchToState(MOVE_TO_PREY); 
				}
				else
				{					
					SwitchToState(MOVE_IN_SHADOWS); 
				}
			}
		}
		break;
	case RETURN_HOME:
		break;
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT

static const char* s_aszStateStrings[] =
{
	"UNSET",
	"IDLE",
	"WANDER",
	"MOVE_IN_SHADOWS",
	"HIDE_IN_SHADOWS",
	"MOVE_TO_PREY",
	"ATTACK",
	"RETURN_HOME"
};

void AVP_Behaviour_FaceHugger_HideInShadows::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();	
	float fTime = 0.0f;

	char cHiddenValue = ' ';

	if (m_fTimeTotallyInvisible > 0)
	{
		cHiddenValue = 'I';
		fTime = m_fTimeTotallyInvisible;
	}

	if (m_fTimeTotallyVisible > 0)
	{
		cHiddenValue = 'V';
		fTime = m_fTimeTotallyVisible;
	}


	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "HideInShadows: %s  %c %.3f %.3f", s_aszStateStrings[ GetCurrentState() ], 
		cHiddenValue, fTime, m_fTimeVisibleByMarineLight );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

}
#endif

AVP_Brain_Actor_FaceHugger* AVP_Behaviour_FaceHugger_HideInShadows::GetFaceHuggerBrain() const
{
	return static_cast< AVP_Brain_Actor_FaceHugger* >( GetBrain() );
}

void AVP_Behaviour_FaceHugger_HideInShadows::SwitchToState( const State eState )
{
	switch ( GetCurrentState() )
	{
	case UNSET:
		break;
	case IDLE:
		break;
	case WANDER:
		DestroyAllChildren();
		break;
	case MOVE_IN_SHADOWS:
		DestroyAllChildren();
		break;
	case HIDE_IN_SHADOWS:
		break;
	case MOVE_TO_PREY:
		DestroyAllChildren();
		break;
	case ATTACKING:
		GetFaceHuggerBrain()->SetNearbyActorGuidToIgnore(ASURA_GUID_UNREGISTERED);
		DestroyAllChildren();
		m_fTimeTillStopAllowed += dbg_fTimeTillStopAllowedAfterAttack;
		break;
	case RETURN_HOME:
		DestroyAllChildren();
		break;
	default:
		Asura_Assert(false, "Unexpected state");
		break;
	}

	SetCurrentState( eState );
	ResetStateTimer();
	switch ( GetCurrentState() )
	{
	case UNSET:
		GetFaceHuggerBrain()->MarkTargetNotTargeted();
		break;
	case IDLE:
		GetFaceHuggerBrain()->MarkTargetNotTargeted();
		break;
	case WANDER:
		{
			GetFaceHuggerBrain()->MarkTargetNotTargeted();
			AVP_Behaviour_FaceHugger_Wander * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Wander* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_WANDER ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished())
			{
				SwitchToState( IDLE );
			}
			else
			{
				pxBehaviour->SetAvoidLight();
				pxBehaviour->SetCenterPoint(GetFaceHuggerBrain()->GetSpawnPoint());
			}
		}
		break;
	case MOVE_IN_SHADOWS:
		PlanPathToShadow();
		break;
	case HIDE_IN_SHADOWS:
		{
			m_fTimeToStayHiddenInShadows = Asura_Random::GetFloat(dbg_fMinTimeToStayIdleInShadows, dbg_fMaxTimeToStayIdleInShadows);
			AVP_FaceHugger_AIAnimStateMachine* pxVertStateHandler = static_cast< AVP_FaceHugger_AIAnimStateMachine* >( GetFaceHuggerBrain()->GetActor()->GetAIAnimStateMachine() );
			pxVertStateHandler->RequestIdle();
		}
		break;
	case MOVE_TO_PREY:
		{
			AVP_Behaviour_FaceHugger_MoveToPrey * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_MoveToPrey* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVETOPREY ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished())
			{
				SwitchToState( RETURN_HOME );
			}

		}
		break;
	case ATTACKING:
		{
			GetFaceHuggerBrain()->SetNearbyActorGuidToIgnore(GetFaceHuggerBrain()->GetTarget()->GetGuid());
			AVP_Behaviour_FaceHugger_AttackPrey * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_AttackPrey* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACKPREY ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished())
			{
				SwitchToState( RETURN_HOME );
			}
		}
		break;
	case RETURN_HOME:
		{
			AVP_Behaviour_FaceHugger_ReturnHome * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_ReturnHome* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_RETURN_HOME ) );
			if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished())
			{
				SwitchToState( IDLE );
			}
		}
		break;
	}
}

void AVP_Behaviour_FaceHugger_HideInShadows::PlanPathToShadow()
{
	const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetFaceHuggerBrain()->GetTarget();
	if (!pxTargetKnowledge)
	{		
		GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fIgnoreTargetTime);
		SwitchToState(IDLE);
		return;
	}

	m_xTargetPositionDuringPlan = pxTargetKnowledge->GetPosition();

	AVP_Navigation::NavData xNavData;
	GetFaceHuggerBrain()->GetNavData().CopyConfigTo(xNavData);

	xNavData.m_xStartPosition = m_xTargetPositionDuringPlan;
	const AVP_Navigation::TestParams xTestParams( xNavData.m_uPathfinderFlags, ASURA_NAVVERTEX_MAX_ZONE_ID );

	if ( !AVP_Navigation::GetNearestWaypointID( m_xTargetPositionDuringPlan, xNavData.m_usCurrentWPID, xTestParams ) )
	{
		GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fIgnoreTargetTime);
		SwitchToState(IDLE);
		return;
	}

	AVP_FaceHuggerShadowsAroundTargetFunctor xShadowAroundTargetFunctor(m_xTargetPositionDuringPlan, GetFaceHuggerBrain()->GetActor()->GetPosition(), xNavData.m_usCurrentWPID, GetFaceHuggerBrain()->GetBehaviourData().m_fMinHideDistance + dbg_fRangeBand * 2.0f, GetFaceHuggerBrain()->GetBehaviourData().m_fMinHideDistance, dbg_fMinMoveDistance );
	const AVP_ServerEntity_GamePlayVolume* pxTargetGPV = AVP_ServerEntity_GamePlayVolume::FindEntity(pxTargetKnowledge->m_uCurrentGPV);
	if (pxTargetGPV)
	{
		xShadowAroundTargetFunctor.AddGPV(pxTargetGPV);
		for (u_short u = 0; u < AVP_FaceHuggerShadowsAroundTargetFunctor::MAX_GPV-1 && u < pxTargetGPV->GetNumLinkedGamePlayVolumes(); ++u)
		{
			const AVP_ServerEntity_GamePlayVolume* pxAdjGPV = AVP_ServerEntity_GamePlayVolume::FindEntity(pxTargetGPV->GetLinkedGamePlayVolumeGuid(u));
			if (pxAdjGPV)
			{
				xShadowAroundTargetFunctor.AddGPV(pxAdjGPV);
			}
		}
	}

	AVP_Navigation::FindRouteAwayFrom( xNavData, &xShadowAroundTargetFunctor, false );

	xShadowAroundTargetFunctor.Filter(dbg_fRangeBand);
	u_short usWPID = xShadowAroundTargetFunctor.GetWPID();
	if (usWPID == ASURA_INVALID_WAYPOINT_ID) // can not find a place to hide around so attack instead
	{
		SwitchToState(MOVE_TO_PREY);
		return;
	}


	const Asura_Vector_3 xDestination = 
		Asura_AI_Navigation_System::GetWaypoint( usWPID )->GetPosition();

	m_fMaxDistanceSq = (xDestination - m_xTargetPositionDuringPlan).Magnitude() + dbg_fRangeBand;
	m_fMaxDistanceSq *= m_fMaxDistanceSq;
	AVP_FaceHuggerMoveAvoidingTargetInShadowsFunctor xMoveInShadoesFunctor(m_xTargetPositionDuringPlan, GetFaceHuggerBrain()->GetBehaviourData().m_fMinHideDistance);

	AVP_Behaviour_FaceHugger_Movement * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
	if ( pxBehaviour && pxBehaviour->GetToPosition( xDestination, xMoveInShadoesFunctor, dbg_fLargeArriveRange, false  ) )
	{
		pxBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_FAST);
		pxBehaviour->SetAvoidLight();
	}
	else
	{
		GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fIgnoreTargetTime);
		SwitchToState(IDLE);
		return;
	}

	m_uLightGeneration = AVP_ServerEntity_Team_Alien::GetInstance()->GetLightGeneration();
}

void AVP_Behaviour_FaceHugger_HideInShadows::UpdateMovingInShadows(float fTimeslice)
{
	const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetFaceHuggerBrain()->GetTarget();
	if (!pxTargetKnowledge)
	{
		GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fIgnoreTargetTime);
		SwitchToState(IDLE);
		return;
	}

	if (TookDamage())
	{
		SwitchToState(MOVE_TO_PREY); 
		return;
	}

	u_int uLightGeneration = AVP_ServerEntity_Team_Alien::GetInstance()->GetLightGeneration();

	AVP_Behaviour_FaceHugger_Movement * pxMoveBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
	if ( !pxMoveBehaviour )
	{
		SwitchToState(MOVE_IN_SHADOWS);
	}

	float fDistanceToTargetSq = (pxTargetKnowledge->GetPosition() - GetFaceHuggerBrain()->GetActor()->GetPosition()).MagnitudeSquared();
	
	bool bInRange = fDistanceToTargetSq < m_fMaxDistanceSq && fDistanceToTargetSq > GetFaceHuggerBrain()->GetBehaviourData().m_fMinHideDistance * GetFaceHuggerBrain()->GetBehaviourData().m_fMinHideDistance;

	if ( m_fTimeTotallyInvisible >  0.0f)
	{
		if ( bInRange && IsInAdajcentGPV(pxTargetKnowledge->m_uCurrentGPV) && (m_fTimeTillStopAllowed == 0.0f || pxMoveBehaviour->IsFinished()))
		{
			SwitchToState(HIDE_IN_SHADOWS);
			return;
		}
		else if (bInRange)
		{
			pxMoveBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_MEDIUM);
		}
		else
		{
			pxMoveBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_FAST);
		}
	}
	else if (m_fTimeTotallyVisible >  0.0f )
	{
		if ( bInRange && GetFaceHuggerBrain()->GetBehaviourData().m_bAttackIfInMarineLight && m_fTimeVisibleByMarineLight >  GetFaceHuggerBrain()->GetBehaviourData().m_fTimeInLightBeforeAttack)
		{
			SwitchToState(MOVE_TO_PREY); 
			return;
		}
		else
		{
			pxMoveBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_FAST);
		}
	}
	else // not totally hidden and not totally visible
	{
		pxMoveBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_MEDIUM);
	}
	
	if (uLightGeneration != m_uLightGeneration || pxMoveBehaviour->IsInError() || pxMoveBehaviour->IsFinished() || ((pxTargetKnowledge->GetPosition()- m_xTargetPositionDuringPlan).MagnitudeSquared() > dbg_fMaxMoveForReplanSq))
	{
		SwitchToState(MOVE_IN_SHADOWS);
	}
}

void AVP_Behaviour_FaceHugger_HideInShadows::UpdateHideInShadows(float fTimeslice)
{
	const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetFaceHuggerBrain()->GetTarget();
	if (!pxTargetKnowledge)
	{
		GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fIgnoreTargetTime);
		SwitchToState(IDLE);
		return;
	}

	if (TookDamage())
	{
		SwitchToState(MOVE_TO_PREY); 
		return;
	}

	float fDistanceToTargetSq = (pxTargetKnowledge->GetPosition() - GetFaceHuggerBrain()->GetActor()->GetPosition()).MagnitudeSquared();

	bool bTooFar = fDistanceToTargetSq > m_fMaxDistanceSq;
	bool bTooClose = fDistanceToTargetSq < GetFaceHuggerBrain()->GetBehaviourData().m_fMinHideDistance * GetFaceHuggerBrain()->GetBehaviourData().m_fMinHideDistance; 

	if ( m_fTimeTotallyInvisible >  0.0f)
	{
		if (bTooFar || !IsInAdajcentGPV(pxTargetKnowledge->m_uCurrentGPV))
		{
			SwitchToState(MOVE_IN_SHADOWS);	
			return;
		}
		else if (bTooClose)
		{
			if (GetFaceHuggerBrain()->GetBehaviourData().m_fChaneToRehideIfCloseFraction == 0.0f)
			{
				SwitchToState(MOVE_TO_PREY); 
				return;
			}
			else if (GetFaceHuggerBrain()->GetBehaviourData().m_fChaneToRehideIfCloseFraction == 1.0f)
			{
				SwitchToState(MOVE_IN_SHADOWS);
				return;
			}
			else if (Asura_Random::GetFloat(0.0f, 1.0f) < GetFaceHuggerBrain()->GetBehaviourData().m_fChaneToRehideIfCloseFraction)
			{				
				SwitchToState(MOVE_IN_SHADOWS);	
				return;
			}
			else 
			{
				SwitchToState(MOVE_TO_PREY); 
				return;
			}
		}
		else if (GetStateTimer() > m_fTimeToStayHiddenInShadows) 
		{
			m_fTimeTillStopAllowed += dbg_fMinMoveTimeDuringRehide;
			SwitchToState(MOVE_IN_SHADOWS); // should re-hide
		}
		else
		{
			// do nothing ...
		}

	}
	else if ( m_fTimeVisibleByMarineLight > 0.0f )
	{
		if (GetFaceHuggerBrain()->GetBehaviourData().m_fChaneToRehideIfInLightFraction == 0.0f)
		{
			SwitchToState(MOVE_TO_PREY); 
			return;
		}
		else if (GetFaceHuggerBrain()->GetBehaviourData().m_fChaneToRehideIfInLightFraction == 1.0f)
		{
			SwitchToState(MOVE_IN_SHADOWS);
			return;
		}
		else if (Asura_Random::GetFloat(0.0f, 1.0f) < GetFaceHuggerBrain()->GetBehaviourData().m_fChaneToRehideIfInLightFraction)
		{				
			SwitchToState(MOVE_IN_SHADOWS);	
			return;
		}
		else 
		{
			SwitchToState(MOVE_TO_PREY); 
			return;
		}
	}
	else // should move
	{
		SwitchToState(MOVE_IN_SHADOWS);
		return;
	}
}

DEBUGVAR float dbg_fTourchMinMultiplier = 1.0f;
DEBUGVAR float dbg_fTourchMaxMultiplier = 1.2f;
DEBUGVAR float dbg_fTourchDelta = 1.0f;
void AVP_Behaviour_FaceHugger_HideInShadows::UpdateHiddenStatus(float fTimeSlice) 
{
	const float fLuminaceFactor = AVP_Waypoint_Manager::GetWPLuminance( GetFaceHuggerBrain()->GetNavData().m_usCurrentWPID );
	
	bool bTotallyHiddenFromMarineLight = true;

	const Asura_Vector_3 & xOurPosition = GetFaceHuggerBrain()->GetActor()->GetPosition();
	const Asura_Vector_3 xOurNormal( -GetFaceHuggerBrain()->GetActor()->GetCurrentGravityDirection() );


	const u_int uNumLights = AVP_ServerEntity_Team_Alien::GetInstance()->GetNumLights();
	for ( u_int uLight = 0; uLight < uNumLights; ++uLight )
	{
		const AVP_ServerEntity_Team_Alien::LightInfo& xLightInfo = AVP_ServerEntity_Team_Alien::GetInstance()->GetLightInfo( uLight );

		if (xLightInfo.m_eType == AVP_ServerEntity_Team_Alien::LightInfo::IDLE_FLARE ) // ignore idle flares - they already taken into account in the way points
		{
			continue;
		}

		const float fDistanceFromSourceSq = (xOurPosition - xLightInfo.m_xSourcePosition).MagnitudeSquared();
		if (fDistanceFromSourceSq > xLightInfo.m_fCutoffDistance * xLightInfo.m_fCutoffDistance)
		{
			continue;
		}

		Asura_Vector_3 xLightPlaneUp = xLightInfo.m_xPosition - xLightInfo.m_xSourcePosition;
		xLightPlaneUp.Normalise();
		AVP_Plane xLightPlane(xLightInfo.m_xPosition, xLightPlaneUp);
		Asura_Vector_3 xOurPositionInLightPlane;
		xLightPlane.CalculateClosestPoint(xOurPosition, xOurPositionInLightPlane);

		Asura_Vector_3 xFromVector = xOurPositionInLightPlane - xLightInfo.m_xPosition;
		float fDistanceFromLightSq = xFromVector.MagnitudeSquared();
		float fMinDistanceSq = ((xLightInfo.m_fRadius * dbg_fTourchMinMultiplier) * (xLightInfo.m_fRadius * dbg_fTourchMinMultiplier));
		if (fDistanceFromLightSq <  fMinDistanceSq)
		{
			m_fTimeVisibleByMarineLight += fTimeSlice;
			m_fTimeSinceVisibleByMarineLight = 0.0f;
			m_fTimeTotallyVisible += fTimeSlice;
			m_fTimeTotallyInvisible = 0.0f;
			return;
		}
		else if (fDistanceFromLightSq < (xLightInfo.m_fRadius * dbg_fTourchMaxMultiplier + dbg_fTourchDelta) * (xLightInfo.m_fRadius * dbg_fTourchMaxMultiplier + dbg_fTourchDelta) )
		{
			bTotallyHiddenFromMarineLight = false;
		}
	}

	if (!bTotallyHiddenFromMarineLight)
	{
		m_fTimeVisibleByMarineLight += fTimeSlice;
		m_fTimeSinceVisibleByMarineLight = 0.0f;
	}
	else
	{
		m_fTimeSinceVisibleByMarineLight += fTimeSlice;
		if (m_fTimeSinceVisibleByMarineLight > dbg_fForgetMarineLightTime)
		{
			m_fTimeVisibleByMarineLight = 0.0f;
		}
	}


	if (fLuminaceFactor > AVP_Waypoint_Manager::GetDefaultMaxHideLuminance() )
	{
		m_fTimeTillStopAllowed += 0.5f * fTimeSlice;
		m_fTimeTotallyVisible += fTimeSlice;
		m_fTimeTotallyInvisible = 0.0f;
		return;
	}

	if (fLuminaceFactor > AVP_Waypoint_Manager::GetDefaultMinHideLuminance() ) // && fLuminaceFactor < dbg_fMaxShadowValue
	{
		m_fTimeTillStopAllowed += 0.25f * fTimeSlice ;
		m_fTimeTotallyVisible = 0.0f;
		m_fTimeTotallyInvisible = 0.0f;
		return;
	}

	if (!bTotallyHiddenFromMarineLight)
	{
		m_fTimeTotallyInvisible = 0.0f;
	}
	else
	{
		m_fTimeTillStopAllowed = m_fTimeTillStopAllowed < fTimeSlice ? 0.0f : m_fTimeTillStopAllowed - fTimeSlice;
		m_fTimeTotallyInvisible += fTimeSlice;
		m_fTimeTotallyVisible = 0.0f;

	}
}

bool AVP_Behaviour_FaceHugger_HideInShadows::TookDamage()
{
	bool bResult = GetFaceHuggerBrain()->GetActor()->GetHealth() < m_fLastHealth;
	m_fLastHealth = GetFaceHuggerBrain()->GetActor()->GetHealth();
	return bResult;
}

bool AVP_Behaviour_FaceHugger_HideInShadows::IsInAdajcentGPV(Asura_Guid uTargetGPV) const
{
	Asura_Guid uMyGPV = GetFaceHuggerBrain()->GetCurrentGPV();
	if (uMyGPV == ASURA_GUID_UNREGISTERED)
	{
		return true;
	}

	if (uMyGPV == uTargetGPV)
	{
		return true;
	}

	const AVP_ServerEntity_GamePlayVolume* pxTargetGPV = AVP_ServerEntity_GamePlayVolume::FindEntity(uTargetGPV);
	if (pxTargetGPV == NULL)
	{
		return true;
	}

	for (u_short u = 0; u < pxTargetGPV->GetNumLinkedGamePlayVolumes(); ++u)
	{
		if (uMyGPV == pxTargetGPV->GetLinkedGamePlayVolumeGuid(u))
		{
			return true;
		}
	}

	return false;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
