//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger_MoveToPrey.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_MoveToPrey.h"

// asura
#include "AI/Asura_AI_Navigation.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// avp
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AIAnimStateMachine.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_FaceHugger.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"

// axon
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Movement.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/AVP_Navigation.h"
#include "Core/BrainTypes/AVP_Brain_Actor_FaceHugger.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_FaceHuggerMoveAway
// Class that calculate cost and Gaol achievement for A* algorithm trying to move away in any direction
//
//////////////////////////////////////////////////////////////////////////////

class AVP_FaceHuggerMoveAway : public AVP_Navigation::AStarFunctorBase
{
public:
	AVP_FaceHuggerMoveAway(Asura_Vector_3 xCenter, float fDistance);
	virtual float EstimateCostToGoal( const int iWaypointIndex );
	virtual float EstimateTraversalCost( const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags);
	virtual bool IsGoalAchieved( const int iWaypointIndex);
	virtual bool LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags );
	const Asura_Vector_3& GetDestination() const;
private:
	Asura_Vector_3 m_xCenter;
	float m_fDistanceSquared;
	Asura_Vector_3 m_xDestination;
};

AVP_FaceHuggerMoveAway::AVP_FaceHuggerMoveAway(Asura_Vector_3 xCenter, float fDistance)
: m_xCenter(xCenter)
, m_fDistanceSquared(fDistance*fDistance)
{
}

float AVP_FaceHuggerMoveAway::EstimateCostToGoal( const int iWaypointIndex )
{
	const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );

	float fDistanceSquared = (pxWP->GetPosition() - m_xCenter).MagnitudeSquared();

	if( fDistanceSquared > m_fDistanceSquared)
	{
		return 0.0f;
	}
	else
	{
		return 50.0f * (fDistanceSquared - m_fDistanceSquared);
	}	
}


float AVP_FaceHuggerMoveAway::EstimateTraversalCost( const int iWaypointIndex, const int iLinkIndex, const u_int uPathfinderFlags)
{
	const Asura_AI_NavVertex* pxFromWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );

	return Asura_AI_Navigation_System::GetWaypointWeight(static_cast<u_short>(pxFromWP->GetLink(iLinkIndex).GetIndex())) * 1.0f;
}

bool AVP_FaceHuggerMoveAway::IsGoalAchieved( const int iWaypointIndex)
{
	const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointIndex );

	if (pxWP->GetFlags() & AVP_Navigation::AVP_AI_NAVVERTEX_FLAG_GS2 ) // GS2 are not valid candidate
	{
		return false;
	}

	float xDistance = (pxWP->GetPosition() - m_xCenter).MagnitudeSquared();

	bool bSucceed =  (xDistance > m_fDistanceSquared);

	if (bSucceed)
	{
		m_xDestination = pxWP->GetPosition();
	}

	return bSucceed;
}

bool AVP_FaceHuggerMoveAway::LinkFlagsTest( const Asura_AI_NavConnection* pxLink, const u_int uPathfinderFlags )
{
	return AlienLinkFlagsTest( pxLink, uPathfinderFlags );
}

const Asura_Vector_3& AVP_FaceHuggerMoveAway::GetDestination() const
{
	return m_xDestination;
}



//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger_MoveToPrey
//
//////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fIgnoreTargetTime = 2.0f;

AVP_Behaviour_FaceHugger_MoveToPrey::AVP_Behaviour_FaceHugger_MoveToPrey( AVP_Brain_Actor_FaceHugger* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVETOPREY )
, m_eCurrentState(CLOSE_DISTANCE)
, m_fTimeInState( 0.0f )
, m_xTargetPosition(0.0f, 0.0f, 0.0f)
, m_xDestination(0.0f, 0.0f, 0.0f)
, m_bTryToMoveCCW(false)
{
	GetFaceHuggerBrain()->SetMovementMode(AVP_Navigation::MM_FAST);
}

void AVP_Behaviour_FaceHugger_MoveToPrey::Deactivate()
{
	// Destroy any children.
	Axon_Behaviour* pxChildBehaviour = GetChild();
	while ( pxChildBehaviour )
	{
		UnlinkChild( pxChildBehaviour );
		GetProjectBrain()->DestroyBehaviour( pxChildBehaviour );
		pxChildBehaviour = GetChild();
	}

	PARENT::Deactivate();
}



DEBUGVAR float g_fFaceHuggerTooCloseDistanceSquared = 2.0f * 2.0f;
DEBUGVAR float g_fFaceHuggerTooFarDeltaDistance = 3.0f;

void AVP_Behaviour_FaceHugger_MoveToPrey::Update( const float fTimeslice )
{
	PARENT::Update( fTimeslice );

	m_fTimeInState += fTimeslice;

	if (!GetFaceHuggerBrain()->CanPlanMove())
	{
		return;
	}

	switch ( m_eCurrentState )
	{
	case FINISHED:
		break;
	case CLOSE_DISTANCE:
		{
			AVP_Behaviour_FaceHugger_Movement* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
			const AVP_AlienTeam_Knowledge* pxTarget = GetFaceHuggerBrain()->GetTarget();
			if(!pxTarget)
			{
				SwitchToState(FINISHED);
			} 
			else 
			{
				float fAvoidanceDistance = GetFaceHuggerBrain()->GetBehaviourData().m_fAvoidanceDistance;
				float fDistanceToTargetSquared = (GetFaceHuggerBrain()->GetActor()->GetPosition()-pxTarget->GetPosition()).MagnitudeSquared();
				if (fDistanceToTargetSquared < g_fFaceHuggerTooCloseDistanceSquared ) 
				{
					SwitchToState( RUN_AWAY );
				}
				else if( fDistanceToTargetSquared < fAvoidanceDistance * fAvoidanceDistance )
				{
					SwitchToState(ROAM_AROUND_PREY);
				}
				else if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
				{
					SwitchToState( CLOSE_DISTANCE );
				}
			}
		}
		break;
	case RUN_AWAY:
		{
			AVP_Behaviour_FaceHugger_Movement* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
			const AVP_AlienTeam_Knowledge* pxTarget = GetFaceHuggerBrain()->GetTarget();
			if(!pxTarget)
			{
				SwitchToState(FINISHED);
			} 
			else 
			{
				float fAvoidanceDistance = GetFaceHuggerBrain()->GetBehaviourData().m_fAvoidanceDistance;
				float fDistanceToTargetSquared = (GetFaceHuggerBrain()->GetActor()->GetPosition()-pxTarget->GetPosition()).MagnitudeSquared();

				if( fDistanceToTargetSquared > (fAvoidanceDistance * fAvoidanceDistance ) )
				{
					SwitchToState(ROAM_AROUND_PREY);
				}
				else if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() )
				{
					SwitchToState( RUN_AWAY );
				}
			}
		}
	break;
	case ROAM_AROUND_PREY:
		{
			AVP_Behaviour_FaceHugger_Movement* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
			const AVP_AlienTeam_Knowledge* pxTarget = GetFaceHuggerBrain()->GetTarget();
			if(!pxTarget)
			{
				SwitchToState(FINISHED);
			} 
			else 
			{
				float fAvoidanceDistance = GetFaceHuggerBrain()->GetBehaviourData().m_fAvoidanceDistance;
				float fDistanceToTargetSquared = (GetFaceHuggerBrain()->GetActor()->GetPosition()-pxTarget->GetPosition()).MagnitudeSquared();
				if ( !pxBehaviour || pxBehaviour->IsInError() || pxBehaviour->IsFinished() || (m_xTargetPosition-pxTarget->GetPosition()).MagnitudeSquared() > 1.0f*1.0f )
				{
					if (fDistanceToTargetSquared < g_fFaceHuggerTooCloseDistanceSquared ) 
					{
						SwitchToState( RUN_AWAY );
					}
					else if ( fDistanceToTargetSquared > (fAvoidanceDistance + g_fFaceHuggerTooFarDeltaDistance) * (fAvoidanceDistance + g_fFaceHuggerTooFarDeltaDistance))
					{
						SwitchToState( CLOSE_DISTANCE );
					}				
					else
					{
						SwitchToState( ROAM_AROUND_PREY );
					}
				}
			}
		}
		break;
	}
}


#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	"CLOSE DISTANCE",
	"RUN AWAY",
	"ROAM AROUND PREY",
	"FINISHED"
};

void AVP_Behaviour_FaceHugger_MoveToPrey::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "FaceHugger MoveToPrey: %s", s_aszStateStrings[ m_eCurrentState ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

void AVP_Behaviour_FaceHugger_MoveToPrey::SwitchToState( const State eNextState )
{
	// Do any work we need to exit our current state.
	switch ( m_eCurrentState )
	{
	case FINISHED:
		Asura_Assert(false, "Not suppose to leave FINISHED state");
		break;
	case CLOSE_DISTANCE:
	case ROAM_AROUND_PREY:
	case RUN_AWAY:
		DestroyChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT );
		break;
	}

	// Do any work we need to change into our next state.
	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;
	switch ( m_eCurrentState )
	{
	case FINISHED:
		SetFinished(true);
		break;
	case CLOSE_DISTANCE:
		InitiateCloseDistance();
		break;
	case RUN_AWAY:
		InitiateRunAway();
		break;
	case ROAM_AROUND_PREY:
		InitiateRoamAroundPrey();
		break;
	}
}


AVP_Brain_Actor_FaceHugger* AVP_Behaviour_FaceHugger_MoveToPrey::GetFaceHuggerBrain() const
{
	return static_cast< AVP_Brain_Actor_FaceHugger* >( GetProjectBrain() );
}


bool AVP_Behaviour_FaceHugger_MoveToPrey::ChooseRoamDestination(float fAvoidanceDistance, bool bFront)
{
	const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetFaceHuggerBrain()->GetTarget();

	if (!pxTargetKnowledge)
	{
		return false;
	}

	const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(pxTargetKnowledge->GetGuid());


	Asura_Assert(pxTarget, "Target Server Entity Not found");
	if (!pxTarget)
	{
		return false;
	}

	m_xTargetPosition = pxTarget->GetPosition();
	m_xTargetAimDirection = pxTarget->GetAimDirection();

	// copy on purpose since we going to mess with it 
	// but dont hold a copy to the route since a new one is being build
	AVP_Navigation::NavData xNavData;
	GetFaceHuggerBrain()->GetNavData().CopyConfigTo(xNavData);

	float fRotationAngle[3];
	if (bFront)
	{
		if(Asura_Random::GetInt(0,1) == 0)
		{
			fRotationAngle[0] = AVP_Brain_Actor_FaceHugger::s_fFrontalAngle;
			fRotationAngle[1] = -AVP_Brain_Actor_FaceHugger::s_fFrontalAngle;
			fRotationAngle[2] = 0;
		} 
		else
		{
			fRotationAngle[0] = -AVP_Brain_Actor_FaceHugger::s_fFrontalAngle;
			fRotationAngle[1] = AVP_Brain_Actor_FaceHugger::s_fFrontalAngle;
			fRotationAngle[2] = 0;
		}
	}
	else
	{
		if(Asura_Random::GetInt(0,1) == 0)
		{
			fRotationAngle[0] = Asura_Maths::DegToRad(180.0f) + AVP_Brain_Actor_FaceHugger::s_fFrontalAngle;
			fRotationAngle[1] = Asura_Maths::DegToRad(180.0f) - AVP_Brain_Actor_FaceHugger::s_fFrontalAngle;
			fRotationAngle[2] = Asura_Maths::DegToRad(180.0f) ;
		}
		else
		{
			fRotationAngle[0] = Asura_Maths::DegToRad(180.0f) - AVP_Brain_Actor_FaceHugger::s_fFrontalAngle;
			fRotationAngle[1] = Asura_Maths::DegToRad(180.0f) + AVP_Brain_Actor_FaceHugger::s_fFrontalAngle;
			fRotationAngle[2] = Asura_Maths::DegToRad(180.0f) ;
		}
	}


	m_bTryToMoveCCW = !m_bTryToMoveCCW;

	bool bFound = false;
	const AVP_Navigation::TestParams xTestParams( xNavData.m_uPathfinderFlags, xNavData.m_usCurrentZone );
	xNavData.m_xStartPosition = m_xTargetPosition;
	if ( !AVP_Navigation::GetNearestWaypointID( xNavData.m_xStartPosition, xNavData.m_usCurrentWPID, xTestParams ) )
	{
		SetInError();
		//++m_fConsecRouteFindFailureCount;
		SwitchToState( FINISHED );
		return false;
	}

	for (int index = 0; index < 4 && !bFound; ++index)
	{
		Asura_Vector_3 xDirection = m_xTargetAimDirection; 
		xDirection.Normalise();

		Asura_Quat xRotation;
		Asura_Vector_3 xHorizontal(0.0f, 1.0f, 0.0f);
		xRotation.FromRotation(fRotationAngle[index], xHorizontal);
		xDirection = xRotation.RotateVector(xDirection);

		xNavData.ResetRoute();
		xNavData.m_xDestinationPosition = m_xTargetPosition + xDirection * fAvoidanceDistance;

		if ( !AVP_Navigation::GetNearestWaypointID( xNavData.m_xDestinationPosition, xNavData.m_usDestinationWPID, xTestParams ) )
		{
			continue;
		}

		AVP_Navigation::FaceHuggerAwayFromInDirFunctor xCalc(m_xTargetPosition, GetFaceHuggerBrain()->GetBehaviourData().m_fAvoidanceDistance, xDirection);
		if ( AVP_Navigation::FindRoute( xNavData, &xCalc, false) )
		{
			Asura_AI_NavVertex* pxNewWP = Asura_AI_Navigation_System::GetWaypoint( xNavData.m_pusRoute[xNavData.m_usRouteLength-1] );
			if (pxNewWP)
			{
				Asura_Vector_3 xNewDestination = pxNewWP->GetPosition();
				if ( (xNewDestination - m_xDestination).MagnitudeSquared() > 1.0f * 1.0f)
				{
					m_xDestination = xNewDestination;
					return true;
				}
			}
		}
	}
	return false;
}


bool AVP_Behaviour_FaceHugger_MoveToPrey::ChooseRunAwayDestination()
{
	const AVP_AlienTeam_Knowledge* pxTargetKnowledge = GetFaceHuggerBrain()->GetTarget();

	if (!pxTargetKnowledge)
	{
		return false;
	}

	const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(pxTargetKnowledge->GetGuid());


	Asura_Assert(pxTarget, "Target Server Entity Not found");
	if (!pxTarget)
	{
		return false;
	}

	m_xTargetPosition = pxTarget->GetPosition();

	// copy on purpose since we going to mess with it 
	// but dont hold a copy to the route since a new one is being build
	AVP_Navigation::NavData xNavData;
	GetFaceHuggerBrain()->GetNavData().CopyConfigTo(xNavData);
	xNavData.m_pusRoute = NULL;
	xNavData.ResetRoute();

	xNavData.m_xStartPosition = m_xTargetPosition;
	const AVP_Navigation::TestParams xTestParams( xNavData.m_uPathfinderFlags, xNavData.m_usCurrentZone );
	if ( !AVP_Navigation::GetNearestWaypointID( xNavData.m_xStartPosition, xNavData.m_usCurrentWPID, xTestParams ) )
	{
		return false;
	}
	xNavData.m_xDestinationPosition = xNavData.m_xStartPosition; // just to avoid zone problems since the destination is not even used.
	xNavData.m_usDestinationWPID= xNavData.m_usCurrentWPID;

	AVP_FaceHuggerMoveAway xCalc(m_xTargetPosition, GetFaceHuggerBrain()->GetBehaviourData().m_fAvoidanceDistance);
	if(AVP_Navigation::FindRouteAwayFrom(xNavData, &xCalc, false))
	{
		m_xDestination = Asura_AI_Navigation_System::GetWaypoint( xNavData.m_pusRoute[xNavData.m_usRouteLength-1] )->GetPosition();
		return true;
	}
	else
	{
		return false;
	}
}

void AVP_Behaviour_FaceHugger_MoveToPrey::InitiateCloseDistance()
{
	const AVP_AlienTeam_Knowledge* pxTarget = GetFaceHuggerBrain()->GetTarget();
	if (!pxTarget)
	{
		SetInError(true);
		SwitchToState(FINISHED);
		return;
	}

	m_xDestination = pxTarget->GetPosition();

	AVP_Navigation::NavData& xNavData = GetFaceHuggerBrain()->GetNavData();

	AVP_Navigation::TestParams xTestParams;
	xTestParams.m_uPathfinderFlags = xNavData.m_uPathfinderFlags;
	xTestParams.m_usZone           = xNavData.m_usCurrentZone;
	AVP_Navigation::ResolveToValidPosition( m_xDestination, xTestParams );

	AVP_Behaviour_FaceHugger_Movement * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
	if ( pxBehaviour && pxBehaviour->GetToPosition( m_xDestination ) )
	{
		pxBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_FAST);
	}
	else
	{
		GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fIgnoreTargetTime);
		SetInError(true);
		SwitchToState(FINISHED);
	}

}


void AVP_Behaviour_FaceHugger_MoveToPrey::InitiateRunAway()
{
	bool bFound = false;
	AVP_Behaviour_FaceHugger_Movement * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
	if ( !pxBehaviour )
	{
		SetInError(true);
		SwitchToState(FINISHED);
	}
	else
	{
		pxBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_FAST);
		for (int i = 0; i < 4 && bFound == false; ++i)
		{
			if(ChooseRunAwayDestination())
			{
				if (pxBehaviour->GetToPosition( m_xDestination) )
				{
					bFound = true;
				}
			}
		}
		if (bFound == false)
		{
			DestroyChildWithType(AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT);
			GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fIgnoreTargetTime);
			SetInError(true);
			SwitchToState(FINISHED);
		}
	}
}

void AVP_Behaviour_FaceHugger_MoveToPrey::InitiateRoamAroundPrey()
{
	AVP_Behaviour_FaceHugger_Movement * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
	if ( !pxBehaviour )
	{
		SetInError(true);
		SwitchToState(FINISHED);
	}
	else
	{
		pxBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_FAST);
		float fAvoidanceDistance = Asura_Random::GetFloat(AVP_Brain_Actor_FaceHugger::s_fFaceHuggerMinimumComfortDistance, GetFaceHuggerBrain()->GetBehaviourData().m_fAvoidanceDistance + 1.0f);
		bool bFound = false;
		for (int iRetry = 0; iRetry < 3 && !bFound && fAvoidanceDistance > AVP_Brain_Actor_FaceHugger::s_fFaceHuggerMinimumComfortDistance; ++iRetry)
		{
			if(ChooseRoamDestination(fAvoidanceDistance, true) && 
				pxBehaviour->GetToPosition( m_xDestination, m_xTargetPosition, fAvoidanceDistance, fAvoidanceDistance + 1.0f, m_xTargetAimDirection ))
			{
				bFound = true;
			} 
			fAvoidanceDistance -= 4.0f;
		}

		fAvoidanceDistance = Asura_Random::GetFloat(AVP_Brain_Actor_FaceHugger::s_fFaceHuggerMinimumComfortDistance, GetFaceHuggerBrain()->GetBehaviourData().m_fAvoidanceDistance + 1.0f);
		for (int iRetry = 0; iRetry < 3 && !bFound && fAvoidanceDistance > AVP_Brain_Actor_FaceHugger::s_fFaceHuggerMinimumComfortDistance; ++iRetry)
		{
			if(ChooseRoamDestination(fAvoidanceDistance, false) && 
				pxBehaviour->GetToPosition( m_xDestination, m_xTargetPosition, fAvoidanceDistance, fAvoidanceDistance + 1.0f, m_xTargetAimDirection ))
			{
				bFound = true;
			} 
			fAvoidanceDistance -= 4.0f;
		}

		if (!bFound)
		{
			GetFaceHuggerBrain()->IgnoreCurrentTarget(dbg_fIgnoreTargetTime);
			SetInError(true);
			SwitchToState(FINISHED);
		}
	}

}




//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
