//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger_ReturnHome.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_ReturnHome.h"

// asura
#include "AI/Asura_AI_Navigation.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// avp
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AIAnimStateMachine.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"

// axon
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Movement.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/AVP_Navigation.h"
#include "Core/BrainTypes/AVP_Brain_Actor_FaceHugger.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_FaceHugger_ReturnHome
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_FaceHugger_ReturnHome::AVP_Behaviour_FaceHugger_ReturnHome( AVP_Brain_Actor_FaceHugger* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_FACEHUGGER_RETURN_HOME )
, m_eCurrentState( UNSET )
, m_fTimeInState( 0.0f )
{
	GetFaceHuggerBrain()->SetMovementMode(AVP_Navigation::MM_FAST);
}

void AVP_Behaviour_FaceHugger_ReturnHome::Deactivate()
{
	// Destroy any children.
	Axon_Behaviour* pxChildBehaviour = GetChild();
	while ( pxChildBehaviour )
	{
		UnlinkChild( pxChildBehaviour );
		GetProjectBrain()->DestroyBehaviour( pxChildBehaviour );
		pxChildBehaviour = GetChild();
	}

	PARENT::Deactivate();
}

void AVP_Behaviour_FaceHugger_ReturnHome::Update( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	m_fTimeInState += fTimeslice;

	if (!GetFaceHuggerBrain()->CanPlanMove())
	{
		return;
	}


	switch ( m_eCurrentState )
	{
	case UNSET:
		SwitchToState(GOING_HOME);
		break;
	case FINISHED:
		break;
	case GOING_HOME:
		{
			AVP_Behaviour_FaceHugger_Movement* pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
			if (!pxBehaviour )
			{
				SwitchToState(FINISHED);
			}
			else if ( pxBehaviour->IsInError() )
			{
				SwitchToState(GOING_HOME);
			}
			else if ( pxBehaviour->IsFinished() )
			{
				SwitchToState(FINISHED);
			}

			break;
		}
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	"UNSET",
	"GOING HOME",
	"FINISHED"
};

void AVP_Behaviour_FaceHugger_ReturnHome::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "FaceHugger ReturnHome: %s", s_aszStateStrings[ m_eCurrentState ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

void AVP_Behaviour_FaceHugger_ReturnHome::SwitchToState( const State eNextState )
{
	// Do any work we need to exit our current state.
	switch ( m_eCurrentState )
	{
	case UNSET:
		break;
	case GOING_HOME:
		{
			DestroyChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT );
			break;
		}
	default:
	case FINISHED:
		Asura_Assert(false, "unexpected state");
		break;
	}

	// Do any work we need to change into our next state.
	m_eCurrentState = eNextState;
	m_fTimeInState  = 0.0f;
	switch ( m_eCurrentState )
	{
	case UNSET:
		Asura_Assert(false, "unexpected state");
		break;
	case FINISHED:
		SetFinished(true);
		break;
	case GOING_HOME:
		{

			AVP_Navigation::NavData& xNavData = GetFaceHuggerBrain()->GetNavData();
			Asura_Vector_3 xHome = GetFaceHuggerBrain()->GetSpawnPoint();

			AVP_Navigation::TestParams xTestParams;
			xTestParams.m_uPathfinderFlags = xNavData.m_uPathfinderFlags;
			xTestParams.m_usZone           = xNavData.m_usCurrentZone;
			AVP_Navigation::ResolveToValidPosition( xHome, xTestParams );

			AVP_Behaviour_FaceHugger_Movement * pxBehaviour = static_cast< AVP_Behaviour_FaceHugger_Movement* >( CreateChildWithType( AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT ) );
			if ( pxBehaviour && pxBehaviour->GetToPosition( xHome ) )
			{
				pxBehaviour->SetMovementMode(AVP_Behaviour_FaceHugger_Movement::EMM_MEDIUM);
			}
			else
			{
				m_bIsInError = true;
				SwitchToState(FINISHED);
			}
			break;
		}
	}
}

AVP_Brain_Actor_FaceHugger* AVP_Behaviour_FaceHugger_ReturnHome::GetFaceHuggerBrain() const
{
	return static_cast< AVP_Brain_Actor_FaceHugger* >( GetProjectBrain() );
}



//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
