//-------------------------------------------------------------------------------------------------
//	File	:	AVP_Behaviour_FaceHugger_SearchForPrey.cpp
//	Created	:	29 / 06 / 07
//	Author	:	Gaz Thomas
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//	INCLUDES
//-------------------------------------------------------------------------------------------------
//	Core
#include "Core/Asura.h"
#include "Core/Asura_Maths.h"
//	This files header
#include "Behaviour\BehaviourTypes\FaceHugger\AVP_Behaviour_FaceHugger_SearchForPrey.h"
// Asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"	
#endif
//	AVP
#include "Actor/Vertebrates/Alien/AVP_FaceHugger_Description.h"
//	Axon
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/Axon_Project_Brain.h"
#include "Knowledge/SenseTypes/AVP_Sense_Vision.h"

//-------------------------------------------------------------------------------------------------
//	Local Defines
//-------------------------------------------------------------------------------------------------
#define FACE_HUGGER_SEARCH_FOR_PREY_RUN_THRESHOLD			0.6f
//-------------------------------------------------------------------------------------------------
#define FACE_HUGGER_MIN_SEARCH_WAIT_TIME					1.0f
#define FACE_HUGGER_MAX_SEARCH_WAIT_TIME					8.0f
//-------------------------------------------------------------------------------------------------
#define FACE_HUGGER_MIN_SEEK_WAIT_TIME						5.0f
#define FACE_HUGGER_MAX_SEEK_WAIT_TIME						15.0f
//-------------------------------------------------------------------------------------------------
#define	FACE_HUGGER_FACE_NEW_DIRECTION_TIMEOUT				2.0f
//-------------------------------------------------------------------------------------------------
#define	FACE_HUGGER_RUN_ORIENT_SPEED						37.5f
#define	FACE_HUGGER_WALK_ORIENT_SPEED						75.0f
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::AVP_Behaviour_FaceHugger_SearchForPrey
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//-------------------------------------------------------------------------------------------------
AVP_Behaviour_FaceHugger_SearchForPrey::AVP_Behaviour_FaceHugger_SearchForPrey( Axon_Project_Brain* pxBrain )
: PARENT( pxBrain, AXON_BEHAVIOUR_TYPE_FACEHUGGER_SEARCH_FOR_PREY )
, m_uUpdateState( US_UNSET )
{	
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::~AVP_Behaviour_FaceHugger_SearchForPrey
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
AVP_Behaviour_FaceHugger_SearchForPrey::~AVP_Behaviour_FaceHugger_SearchForPrey() 
{
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::Activate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
bool AVP_Behaviour_FaceHugger_SearchForPrey::Activate()
{
	bool bOk = PARENT::Activate();

	//	Zero out our move to position
	m_xSeekMoveToPos				= Asura_Vector_3(0.0f, 0.0f, 0.0f);
	//	Set up a time-out time to indicate how long to stay in Search state
	m_fSearchTimeoutTime			= Asura_Random::GetFloat(FACE_HUGGER_MIN_SEARCH_WAIT_TIME, FACE_HUGGER_MAX_SEARCH_WAIT_TIME);
	m_fSearchTimer					= 0.0f;
	//	Set up a time-out time to indicate how long to stay in Seek state
	m_fSeekTimeoutTime				= Asura_Random::GetFloat(FACE_HUGGER_MIN_SEEK_WAIT_TIME, FACE_HUGGER_MAX_SEEK_WAIT_TIME);
	m_fSeekTimer					= 0.0f;
	//	Set up a time-out time to indicate how long to stay in Face New Direction State
	m_fFaceNewDirTimer				= 0.0f;		
	// Initialise the ChooseNew Direction States variables
	m_bNewFacingDirectionCalculated = false;
	m_xPositionToFace				= Asura_Vector_3(0.0f, 0.0f, 0.0f);

	//	Set the update state
	SetUpdateState(US_SEARCH);

	return bOk;
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::Deactivate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_SearchForPrey::Deactivate()
{   
	PARENT::Deactivate();
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::PerFrameUpdate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_SearchForPrey::PerFrameUpdate(const float fTimeslice)
{
	//	Check we aren't in an error state, or that we haven't finished the behaviour
	if (IsInError() || IsFinished())
		return;

	switch ( m_uUpdateState )
	{
		case US_SEARCH:
			Search_PerFrameUpdate(fTimeslice);						return;
		case US_FIND_SEEK_POINT:
			FindSeekPoint_PerFrameUpdate(fTimeslice);				return;
		case US_SEEK:
			Seek_PerFrameUpdate(fTimeslice);						return;
		case US_FACE_NEW_DIRECTTION:
			ChooseNewFacingDirection_PerFrameUpdate(fTimeslice);	return;
		default:
			SetInError();return;
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::Search_PerFrameUpdate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_SearchForPrey::Search_PerFrameUpdate(const float fTimeslice)
{
	//	Request an idle Animation, it Should be be animating already but just incase request one
	m_pxBrain->RequestAnimation(AVP_FaceHugger_Description::FACEHUGGER_ANIM_IDLE);

	//	Increase the Search Timer
	m_fSearchTimer += fTimeslice;

	//	If our timer has passed our random search time-out time, go looking for an enemy
	if (m_fSearchTimer >= m_fSearchTimeoutTime)
	{
		//	Reset The Search Timer to zero
		m_fSearchTimer = 0.0f;
		//	Set up a time-out time to indicate how long to stay in Search state
		m_fSearchTimeoutTime	= Asura_Random::GetFloat(FACE_HUGGER_MIN_SEARCH_WAIT_TIME, FACE_HUGGER_MAX_SEARCH_WAIT_TIME);
		//	Switch State
		SetUpdateState(US_FIND_SEEK_POINT);
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::FindSeekPoint_PerFrameUpdate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_SearchForPrey::FindSeekPoint_PerFrameUpdate(const float fTimeslice)
{
	//	Request an idle Animation, it Should be be animating already but just incase request one
	m_pxBrain->RequestAnimation(AVP_FaceHugger_Description::FACEHUGGER_ANIM_IDLE);

	if (FindSeekPosition())
	{
		AVP_Behaviour_FaceHugger* pxParent = static_cast<AVP_Behaviour_FaceHugger*>(m_pxParentBehaviour);

		//	Calculate a Route to the Seek Position
		if (!GetRouteTo(m_xSeekMoveToPos))
		{
			//	We couldn't find a route so lets choose a new facing direction
			SetUpdateState(US_FACE_NEW_DIRECTTION);
		}

		//	Choose if the Face Hugger should walk or run to this position
		if (Asura_Random::GetFloat() >= FACE_HUGGER_SEARCH_FOR_PREY_RUN_THRESHOLD)
		{
			//	Validate the pointer
			if (pxParent)
			{
				//	Always walk if the Face hugger is not on the floor
				if (!pxParent->IsOnFloor())
				{
					m_bShouldWalk = true;
				}
				else
				{
					m_bShouldWalk = false;
				}
			}
		}
		else
		{
			m_bShouldWalk = true;
		}

		//	Switch State
		SetUpdateState(US_SEEK);
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::Seek_PerFrameUpdate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_SearchForPrey::Seek_PerFrameUpdate(const float fTimeslice)
{
	//	Request a movement animation
	if (m_bShouldWalk)
		m_pxBrain->RequestAnimation(AVP_FaceHugger_Description::FACEHUGGER_ANIM_WALK);
	else
		m_pxBrain->RequestAnimation(AVP_FaceHugger_Description::FACEHUGGER_ANIM_RUN);

	//	Move Along the route
	OrientAlongRoute(fTimeslice);

	//	Increase the Search Timer
	m_fSeekTimer += fTimeslice;

	if ((m_fSeekTimer >= m_fSeekTimeoutTime) || AtTargetPosition(m_pxBrain->GetActor()->GetPosition(), m_xSeekMoveToPos))
	{
		//	Reset The Seek Timer to zero
		m_fSeekTimer = 0.0f;
		//	Set up a time-out time to indicate how long to stay in Seek state
		m_fSeekTimeoutTime		= Asura_Random::GetFloat(FACE_HUGGER_MIN_SEEK_WAIT_TIME, FACE_HUGGER_MAX_SEEK_WAIT_TIME);
		//	Switch State
		SetUpdateState(US_FACE_NEW_DIRECTTION);
	}
}
//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::ChooseNewFacingDirection_PerFrameUpdate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------

void AVP_Behaviour_FaceHugger_SearchForPrey::ChooseNewFacingDirection_PerFrameUpdate(const float fTimeslice)
{
	Asura_Matrix_3x3	xActorOrientation		= m_pxBrain->GetActor()->GetOrientation();
	Asura_Vector_3		xActorUp				= Asura_Vector_3(xActorOrientation.e12, xActorOrientation.e22, xActorOrientation.e32);
	Asura_Vector_3		xActorForward			= Asura_Vector_3(xActorOrientation.e13, xActorOrientation.e23, xActorOrientation.e33);
	Asura_Vector_3		xNewFacingDirection		= Asura_Vector_3(0.0f, 0.0f, 0.0f);
	Asura_Vector_3		xPositionToFace			= Asura_Vector_3(0.0f, 0.0f, 0.0f);
	Asura_Quat			xQuat;

	//	Call a movement Animation (same as before to make it look consistent to do the orientation!
	if (m_bShouldWalk)
	{
		m_pxBrain->RequestAnimation(AVP_FaceHugger_Description::FACEHUGGER_ANIM_WALK);
	}
	else
	{
		m_pxBrain->RequestAnimation(AVP_FaceHugger_Description::FACEHUGGER_ANIM_RUN);
	}

	// Only calculate this once per run through of the state
	if (!m_bNewFacingDirectionCalculated)
	{
		//	Lets get a random angle between Zero degrees and 360 degrees
		Asura_Angle fAngle = Asura_Random::GetFloat(0.0f, Asura_Maths::Deg360);

		//	Normalise our vectors before we attempt to use them
		xActorUp.Normalise();
		xActorForward.Normalise();
		//	Get the quat from the Actors Orientation Matrix
		xQuat.FromMatrix(xActorOrientation);
		//	Rotate the vector around the Actors up vector (Local Y Axis)
		xQuat.FromRotation(fAngle, xActorUp);
		//	Now rotate the forward vector and store the result in the new facing direction vector
		xQuat.RotateVector(xActorForward, xNewFacingDirection);
		//	Calculate a position in the new direction we calculated
		m_xPositionToFace = m_pxBrain->GetActor()->GetPosition() + (xNewFacingDirection * 10.0f);
		//	We have chose a position to face towards now so lets not do it again until the next time we are in this state
		m_bNewFacingDirectionCalculated = true;
	}

	//	Call Orient to that Direction!
	if (m_bShouldWalk)
	{
		//	Increased Orientation time or it snaps to that direction!
		//	Also Orient Slower as we are walking
		OrientToPosition(&m_xPositionToFace, (fTimeslice * FACE_HUGGER_WALK_ORIENT_SPEED)); 
	}
	else
	{
		//	Increased Orientation time or it snaps to that direction!
		//	Also Orient Slower as we are Running, but orient faster than walking to reduce the distance travelled
		OrientToPosition(&m_xPositionToFace, (fTimeslice * FACE_HUGGER_RUN_ORIENT_SPEED));
	}

	//	Calculate the direction to the position we need to face
	Asura_Vector_3	xDirectionToFacingPosition = m_xPositionToFace - m_pxBrain->GetActor()->GetPosition();
	xDirectionToFacingPosition.Normalise();

	//	Check we are facing roughly to the position we want to be facing towards
	if (xActorForward.ApproximatelyEquals(xDirectionToFacingPosition, 0.02f))
	{
		//	Reset this states variables as we are about to switch out of it
		m_bNewFacingDirectionCalculated = false;
		m_xPositionToFace = Asura_Vector_3(0.0f, 0.0f, 0.0f);
		SetUpdateState(US_SEARCH);
	}

	//	Update our Timeout timer
	m_fFaceNewDirTimer += fTimeslice;

	//	A time-out in case we never get to face our new direction
	if (m_fFaceNewDirTimer >= FACE_HUGGER_FACE_NEW_DIRECTION_TIMEOUT)
	{
		m_fFaceNewDirTimer = 0.0f;
		SetUpdateState(US_SEARCH);
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::PerFrameUpdate
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
bool AVP_Behaviour_FaceHugger_SearchForPrey::FindSeekPosition()
{
	// Get The actors Forward Direction;
	Asura_Matrix_3x3	xOrientMatrix	= m_pxBrain->GetActor()->GetOrientation();
	Asura_Vector_3		xForwardVector	= Asura_Vector_3(xOrientMatrix.e13, xOrientMatrix.e23, xOrientMatrix.e33);
	xForwardVector.Normalise();

	const Asura_Vector_3 xSeekMovePos = m_pxBrain->GetActor()->GetPosition() + (xForwardVector * 10.0f);

	if (AVP_Sense_Vision::LOSCheckOnPos(m_pxBrain->GetActor(), xSeekMovePos))
	{
		m_xSeekMoveToPos = xSeekMovePos;
		return true;
	}

	return false;
}

#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	"CHILD STATE - UNSET",
	"CHILD STATE - SEARCH",
	"CHILD STATE - FIND SEEK POINT",
	"CHILD STATE - SEEK",
	"CHILD STATE - FACE NEW DIRECTTION"
};

//--------------------------------------------------------------------------------------
//	Function	:	AVP_Behaviour_FaceHugger_SearchForPrey::RenderDebugInfo
//	Created		:	29 / 06 / 07
//	Author		:	Gaz Thomas
//--------------------------------------------------------------------------------------
void AVP_Behaviour_FaceHugger_SearchForPrey::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		m_pxBrain->GetDebugOutputPos().x, 
		m_pxBrain->GetDebugOutputPos().y, 
		0.5f, 
		0xff0000ff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
		"%s", s_aszStateStrings[ m_uUpdateState ] 
	);
	m_pxBrain->GetDebugOutputPos().y += 10.0f;
}
#endif
