//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_GS2Node_Human_CoreNodeWithGunFire.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Core
#include "Core/Asura.h"

// This files header
#include "Behaviour/BehaviourTypes/Human/AVP_Behaviour_GS2Node_Human_CoreNodeWithGunFire.h"

// Asura
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationController_Animation.h"

// Axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"

// AVP
#include "Animation/AVP_AnimFlags.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Moves/AVP_Moves.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/AVP_InventoryItem_SubType.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"


//////////////////////////////////////////////////////////////////////////////
// Function Implementations
//////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
AVP_Behaviour_GS2Node_Human_CoreNodeWithGunFire::AVP_Behaviour_GS2Node_Human_CoreNodeWithGunFire( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( AXON_BEHAVIOUR_TYPE_HUMAN_GS2NODE_ACTOR_WITH_GUNFIRE, pxBrain )
{

}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Human_CoreNodeWithGunFire::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream( xStream );
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Human_CoreNodeWithGunFire::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Human_CoreNodeWithGunFire::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	AVP_ServerActor_Human* pxServerActor = GetMarineBrain()->GetHumanActor();
	AVP_ClientActor_Human* pxClientActor = AVP_ClientActor_Human::FindEntity( GetBrain()->GetOwnerGuid() );
	
	if ( !pxClientActor )
	{
		return;
	}

	const Asura_AnimationController_Animation*	pxAACA				= NULL;
	const AVP_Move*								pxCurrentGSMove		= NULL;

	if ( pxClientActor->GetMovementSystem() && pxClientActor->GetMovementSystem()->GetCurrentMove() && pxClientActor->GetMovementSystem()->GetCurrentMove()->GetHash() == AVP_Move_GS2RigidAnim::GetStaticHash() )
	{
		pxCurrentGSMove = static_cast< const AVP_Move* >( pxClientActor->GetMovementSystem()->GetCurrentMove() );
	}

	if ( pxCurrentGSMove && pxCurrentGSMove->GetAVPAnimationData() )
	{
		pxAACA = pxCurrentGSMove->GetAVPAnimationData()->GetAnimationController();
	}
	else
	{
		const Asura_Animation* const pxClientAnimation = pxClientActor->GetShapeInstance().GetAnimation();

		if ( pxClientAnimation )
		{
			pxAACA = pxClientAnimation->GetControllerAnimationFromHash( pxClientActor->GetCurrentGamesceneAnimControllerHash() );
		}
	}

	if ( pxAACA )
	{
		u_int uFlags = pxAACA->GetFlagsForLastFrame();

		if ( uFlags & AVP_ANIMFLAG_ACTION_1 )
		{
			// fire gun!
			AVP_Inventory_System*		pxInventorySystem	= pxServerActor->GetInventorySystem();
			AVP_InventoryItem_Weapon*	pxCurrentWeapon		= static_cast<AVP_InventoryItem_Weapon*>(pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL);

			if (pxCurrentWeapon != NULL)
			{
				if (pxCurrentWeapon->GetSubType(0)->GetCurrentQuantity() > 0)
				{
					if ( pxCurrentWeapon->IsIdling( true ) )
					{
						pxCurrentWeapon->RequestBeginUsage( AVP_InventoryWeapon_RangedWeapon::UT_N_BULLET_BURST_PRIMARY );
					}
				}
			}
		}
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
AVP_Brain_Actor_Marine* AVP_Behaviour_GS2Node_Human_CoreNodeWithGunFire::GetMarineBrain( void ) const
{
	return static_cast< AVP_Brain_Actor_Marine* >( GetBrain() );
}

/// END OF FILE