//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Civilian_Alert.cpp
// Created		: 04 / 03 / 2008
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------
// Core
#include "Core/Asura.h"

// This Files Header
#include "Behaviour/BehaviourTypes/Human/Civilian/AVP_Behaviour_Civilian_Alert.h"

// Asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// AVP
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Civilian/AVP_ServerActor_Civilian.h"

// AVP Axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Human/AVP_Behaviour_Human_Movement.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Civilian.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
AVP_Behaviour_Civilian_Alert::AVP_Behaviour_Civilian_Alert(AVP_Brain_Actor_Civilian* pxBrain)
: PARENT(pxBrain, AVP_BEHAVIOUR_TYPE_CIVILIAN_ALERT)
, m_eUpdateState(MAKE_DECISION)
, m_pxCivilianBrain(pxBrain)
, m_uAlertGuid( ASURA_GUID_UNREGISTERED )
, m_xAlertPosition( Asura_Maths::ZeroVector3 )
{

}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
bool AVP_Behaviour_Civilian_Alert::Activate(void)
{
	const bool bParentOk = PARENT::Activate();

	m_uAlertGuid		= GetCivilianBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge().GetGuid();
	m_xAlertPosition	= GetCivilianBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge().m_xPosition;

	return bParentOk;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Alert::Deactivate(void)
{
	if ( GetCivilianBrain()->IsActive() )
	{
		if ( GetChildWithType(this, AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT ) )
		{
			if ( GetCivilianBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_NONE )
			{
				AVP_Behaviour_Human_Movement* pxHumanMovement = static_cast<AVP_Behaviour_Human_Movement*>( GetChildWithType(this, AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT ) );
				pxHumanMovement->SetStopAtEnd( false );
			}
		}
	}

	DestroyAllChildren();

	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Alert::PerFrameUpdate(const float fTimeslice)
{
	bool bHaveThreat	= (GetCivilianBrain()->GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid() != ASURA_GUID_UNREGISTERED) ? true : false;

	// If we have a threat then go back to high level and it will make us panic.
	if (bHaveThreat) 
	{
		if ( GetChildWithType(this, AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT ) )
		{
			if ( GetCivilianBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_NONE )
			{
				AVP_Behaviour_Human_Movement* pxHumanMovement = static_cast<AVP_Behaviour_Human_Movement*>( GetChildWithType(this, AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT ) );
				pxHumanMovement->SetStopAtEnd( false );
			}
		}

		SetFinished(); 
		return;
	}

	// If we don't have an alert any more finish the behaviour
	if ( ( m_uAlertGuid == ASURA_GUID_UNREGISTERED && m_xAlertPosition == Asura_Maths::ZeroVector3) || GetCivilianBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS )
	{
		SetFinished(); 
		return;
	}

	AVP_ServerEntity_Actor*		pxAlertActor	= AVP_ServerEntity_Actor::FindEntity( m_uAlertGuid );
	AVP_ServerActor_Civilian*	pxOurActor		= GetCivilianBrain()->GetCivilianActor();

	switch (GetUpdateState())
	{
		//--------------------------------------
		case MAKE_DECISION:
		{
			float fDistanceSq = (pxAlertActor->GetPosition() - pxOurActor->GetPosition()).MagnitudeSquared();

			if (fDistanceSq < 225.0f)
			{
				SetUpdateState(TURN_TO_FACE);
			}
			else
			{
				SetUpdateState(INVESTIGATE);
			}
		}break;
		//--------------------------------------
		case TURN_TO_FACE:
		{
			if ( pxAlertActor )
			{
				LookAtAlert(pxAlertActor);
			}
			else
			{
				GetCivilianBrain()->SetLookAtPosition( m_xAlertPosition );
				GetCivilianBrain()->AimAtPosition( m_xAlertPosition );
			}

		}break;
		//--------------------------------------
		case INVESTIGATE:
		{
			AVP_Behaviour_Human_Movement* pxMoveBehaviour = (AVP_Behaviour_Human_Movement*)GetChildWithType(this, AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT);

			if (pxMoveBehaviour)
			{
				if (pxMoveBehaviour->IsFinished())
				{
					if ( GetCivilianBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_NONE )
					{
						AVP_Behaviour_Human_Movement* pxHumanMovement = static_cast<AVP_Behaviour_Human_Movement*>( GetChildWithType(this, AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT ) );
						pxHumanMovement->SetStopAtEnd( false );
					}
					
					DestroyChildWithType(AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT);
					SetUpdateState(TURN_TO_FACE);
				}
			}
			else
			{
				if ( pxAlertActor )
				{
					CreateMovementBehaviour(pxAlertActor);
				}
				else
				{
					CreateMovementBehaviour();
				}
			}
		}
		//--------------------------------------
		case UNSET:
		default:
		{

		}break;
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Alert::LookAtAlert(AVP_ServerEntity_Actor* pxAlertActor)
{
	AVP_ServerActor_Civilian* pxOurActor = GetCivilianBrain()->GetCivilianActor();

	// Get The chest Height Central Position;
	Asura_Vector_3 xOurChestPos = Asura_Maths::ZeroVector3;
	pxOurActor->GetChestPosition(xOurChestPos);
	xOurChestPos.x = pxOurActor->GetPosition().x;
	xOurChestPos.z = pxOurActor->GetPosition().z;

	// Get The chest Height Central Position;
	Asura_Vector_3 xAlertChestPos = Asura_Maths::ZeroVector3;
	pxAlertActor->GetChestPosition(xAlertChestPos);
	xAlertChestPos.x = pxAlertActor->GetPosition().x;
	xAlertChestPos.z = pxAlertActor->GetPosition().z;

	// calculate The Direction
	Asura_Vector_3 xDirtectionToAlert = (xAlertChestPos - xOurChestPos);
	xDirtectionToAlert.Normalise();

	// Set The Actual Directions
	GetCivilianBrain()->SetAimDirection(xDirtectionToAlert);
	GetCivilianBrain()->SetLookAtPosition(xAlertChestPos);
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Alert::CreateMovementBehaviour(AVP_ServerEntity_Actor* pxAlertActor)
{
	AVP_Behaviour_Human_Movement* pxHumanBehaviour = (AVP_Behaviour_Human_Movement*)CreateChildWithType(AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT);

	if (pxHumanBehaviour)
	{
		pxHumanBehaviour->GetToPosition(pxAlertActor->GetPosition(), AVP_Navigation::MM_SLOW, true);
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Alert::CreateMovementBehaviour( )
{
	AVP_Behaviour_Human_Movement* pxHumanBehaviour = (AVP_Behaviour_Human_Movement*)CreateChildWithType(AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT);

	if (pxHumanBehaviour)
	{
		pxHumanBehaviour->GetToPosition(m_xAlertPosition, AVP_Navigation::MM_SLOW, true);
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[AVP_Behaviour_Civilian_Alert::MAX_STATES] =
{
	"ALERT - UNSET",
	"ALERT - MAKE_DECISION",
	"ALERT - TURN_TO_FACE",
	"ALERT - INVESTIGATE",
	"ALERT - FINISHED",
};

void AVP_Behaviour_Civilian_Alert::RenderDebugInfo(void)
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif
