//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Civilian_Attack.cpp
// Created		: 04 / 03 / 2008
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------
// Core
#include "Core/Asura.h"

// This Files Header
#include "Behaviour/BehaviourTypes/Human/Civilian/AVP_Behaviour_Civilian_Attack.h"

// Asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// AVP
#include "Entity/Locations/ShootAt/AVP_ServerEntity_ShootAt.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Civilian/AVP_ServerActor_Civilian.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/Items/Weapons/AVP_RangedWeapon_SubType.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"

// AVP Axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Human/AVP_Behaviour_Human_Movement.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Civilian.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Tactical/AVP_Tactical_System.h"

DEBUGVAR float dbg_fMinShootTime		= 0.05f;
DEBUGVAR float dbg_fMaxShootTime		= 0.1f;

DEBUGVAR float dbg_fMinDontShootTime	= 1.0f;
DEBUGVAR float dbg_fMaxDontShootTime	= 1.5f;

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
AVP_Behaviour_Civilian_Attack::AVP_Behaviour_Civilian_Attack(AVP_Brain_Actor_Civilian* pxBrain)
: PARENT(pxBrain, AVP_BEHAVIOUR_TYPE_CIVILIAN_ATTACK)
, m_eUpdateState( CHECK_TO_SHOOT )
, m_pxCivilianBrain(pxBrain)
, m_fShootTimer( 0.0f )
, m_fDontShootTimer( 0.0f )
, m_uEnemyGuid( ASURA_GUID_UNREGISTERED )
, m_bReloading( false )
{

}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
bool AVP_Behaviour_Civilian_Attack::Activate(void)
{
	const bool bParentOk = PARENT::Activate();
	// Place code below these calls

	GetOurActor()->SetAiming( true );

	// Place code above these calls
	return bParentOk;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Attack::Deactivate(void)
{
	if ( GetCivilianBrain()->IsActive() )
	{
		GetOurActor()->SetAiming( false );
	}

	// Place code above these calls
	DestroyAllChildren();
	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Attack::PerFrameUpdate(const float fTimeslice)
{
	Asura_Guid uThreatGuid = GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid();

	if ( uThreatGuid == ASURA_GUID_UNREGISTERED )
	{
		SetFinished();
		return;
	}
	Asura_Vector_3 xThreatPos( Asura_Maths::ZeroVector3 );

	if ( ( GetCivilianBrain()->GetKnowledgeSystem()->ShootAtScriptedTarget() && !GetCivilianBrain()->GetKnowledgeSystem()->ScriptedTargetIsShootAtPoint() ) || ( !GetCivilianBrain()->GetKnowledgeSystem()->ShootAtScriptedTarget() ) )
	{
		AVP_ServerEntity_Actor* pxThreatActor = AVP_ServerEntity_Actor::FindEntity( uThreatGuid );

		if ( !pxThreatActor )
		{
			SetFinished();
			return;
		}
		else
		{
			pxThreatActor->GetChestPosition( xThreatPos );
		}
	}
	else
	{
		AVP_ServerEntity_ShootAt* pxShootAt = AVP_ServerEntity_ShootAt::FindEntity( uThreatGuid );

		if ( !pxShootAt )
		{
			SetFinished();
			return;
		}
		else
		{
			xThreatPos = pxShootAt->GetPosition();
		}
	}

	GetOurActor()->SetAiming( true );

	switch ( GetUpdateState() )
	{
		//----------------------------------------------
		case CHECK_TO_SHOOT:
		{
			if ( CheckForFire() )
			{
				SwitchToState( SHOOT );
			}
			
		}break;
		//----------------------------------------------
		case SHOOT:
		{
			bool bFiring = false;

			// If we don't have a threat no point actually shooting
			if (GetCivilianBrain()->GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid() != ASURA_GUID_UNREGISTERED)
			{
				bFiring = true;

				if ( !GetCivilianBrain()->GetKnowledgeSystem()->ShootAtScriptedTarget() || ( GetCivilianBrain()->GetKnowledgeSystem()->ShootAtScriptedTarget() && !GetCivilianBrain()->GetKnowledgeSystem()->ScriptedTargetIsShootAtPoint() ) )
				{
					AVP_ServerEntity_Actor* pxThreatActor = AVP_ServerEntity_Actor::FindEntity(GetCivilianBrain()->GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid());

					if (pxThreatActor)
					{
						if (pxThreatActor->IsDead())
						{
							bFiring = false;
						}
					}
				}
			}

			AVP_ServerEntity_Actor*			pxOurActor			= GetCivilianBrain()->GetCivilianActor();

			bool bOverride = false;
			bFiring = false;
			AVP_Inventory_System*		pxInventorySystem	= pxOurActor->GetInventorySystem();
			AVP_InventoryItem_Weapon*	pxCurrentWeapon		= static_cast<AVP_InventoryItem_Weapon*>(pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL);

			if (pxCurrentWeapon != NULL)
			{
				if ( pxCurrentWeapon->IsIdling( false ) && m_fDontShootTimer > 0.0f )
				{
					bOverride = true;
					m_fDontShootTimer -= fTimeslice;
				}
				else if ( pxCurrentWeapon->IsIdling( false ) )
				{
					if ( CheckForFire() && m_fDontShootTimer <= 0.0f )
					{
						bFiring = true;
					}

					bOverride = true;
				}
				else
				{
					if ( !CheckForFire() )
					{
						bFiring = false;
					}
					else
					{
						bFiring = false;
						bOverride = true;
					}
				}
			}

			if ( !bFiring && bOverride == false )
			{
				StopShooting();
				SetUpdateState( CHECK_TO_SHOOT );
			}
			else if ( bFiring )
			{
				Shoot();
			}

			if (GetCivilianBrain()->GetTacticalSystem()->IsReloadingWeapon() == true)
			{
				SetUpdateState(RELOAD);
			}

		}break;
		//----------------------------------------------
		case RELOAD:
		{
			if ( GetTacticalSystem()->IsReloadingWeapon() == false )
			{
				m_fDontShootTimer = 0.0f;
				SwitchToState( CHECK_TO_SHOOT );
			}
		}
		//----------------------------------------------
		default:
		{

		}break;
	}

	if ( ( GetTacticalSystem()->IsReloadingWeapon() == true ) && ( m_bReloading == false ) )
	{
		SwitchToState( RELOAD );
	}

	GetCivilianBrain()->AimAtPosition( xThreatPos );
	GetCivilianBrain()->SetLookAtPosition( xThreatPos );
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Attack::SwitchToState( AttackUpdateState eState )
{
	switch ( GetUpdateState() )
	{
		//----------------------------------------------
		case CHECK_TO_SHOOT:
		{
		}break;
		//----------------------------------------------
		case SHOOT:
		{
		}break;
		//----------------------------------------------
		case WAIT_TO_SHOOT:
		{
		}break;
		//----------------------------------------------
		case RELOAD:
		{
			m_bReloading = false;

		}break;
		//----------------------------------------------
		default:
		{

		}break;
	}

	SetUpdateState( eState );

	switch ( eState )
	{
		//----------------------------------------------
		case CHECK_TO_SHOOT:
		{
		}break;
		//----------------------------------------------
		case SHOOT:
		{
			m_fShootTimer = Asura_Random::GetFloat( dbg_fMinShootTime, dbg_fMaxShootTime );
		}break;
		//----------------------------------------------
		case WAIT_TO_SHOOT:
		{
			m_fDontShootTimer = Asura_Random::GetFloat( dbg_fMinDontShootTime, dbg_fMaxDontShootTime );
			StopShooting();
		}break;
		//----------------------------------------------
		case RELOAD:
		{
			m_bReloading = true;

		}break;
		//----------------------------------------------
		default:
		{

		}break;
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
bool AVP_Behaviour_Civilian_Attack::CheckForFire( void )
{
	const AVP_ServerActor_Civilian* pxOurActor	= GetCivilianBrain()->GetCivilianActor();

	if (pxOurActor)
	{
		float fWeaponAmmo = GetWeaponAmmo();

		if (fWeaponAmmo > 0.0f)
		{
			Asura_Vector_3		xHeadPos = Asura_Maths::ZeroVector3;

			// Get our actors marker point for the Head
			pxOurActor->GetHeadPosition(xHeadPos);

			// Ensure we have a threat
			if (GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid() != ASURA_GUID_UNREGISTERED)
			{
				Asura_Vector_3 xThreatPos = Asura_Maths::ZeroVector3;
				GetCivilianBrain()->GetThreatPos( xThreatPos );

				if ( ( GetKnowledgeSystem()->ShootAtScriptedTarget() && !GetKnowledgeSystem()->ScriptedTargetIsShootAtPoint() ) || ( !GetKnowledgeSystem()->ShootAtScriptedTarget() ) )
				{
					AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid() );

					if ( pxActor )
					{
						pxActor->GetChestPosition( xThreatPos );
					}
				}

				Asura_Vector_3		xGunPos = Asura_Maths::ZeroVector3;
				Asura_Matrix_3x3	xGunOri;

				// Get the Pulse rifles muzzle marker point
				if (GetWeaponMuzzlePositionAndOrientation(&xGunPos, &xGunOri))
				{
					float			fWeaponMaxRange		= GetWeaponsMaxRange();
					Asura_Vector_3	xGunFwd(xGunOri.e13, xGunOri.e23, xGunOri.e33);
					Asura_Vector_3	xTestDirection( xThreatPos - xGunPos );
					xGunPos = xGunPos - xGunFwd;

					float fDistanceToThreat = xTestDirection.Normalise();

					// Do a LOS test
					const Asura_LOS_InputData	xInput = Asura_LOS_InputData(xGunPos, xGunFwd, pxOurActor->GetGuid(), fDistanceToThreat, false);
					const Asura_LOS_OutputData*	pxOutput = Asura_LOS::TestAgainstEverything(&xInput);

					AVP_ServerEntity_Actor* pxHitActor = AVP_ServerEntity_Actor::FindEntity( pxOutput->m_uHitEntity );

					if ( pxHitActor )
					{
						if ( AVP_Tactical_System::GetAllegiance( *pxOurActor, *pxHitActor ) != AVP_ALLEGIANCE_ENEMY )
						{
							m_uEnemyGuid = ASURA_GUID_UNREGISTERED;
							return false;
						}

						m_uEnemyGuid = pxHitActor->GetGuid();
					}
					else
					{
						if ( fWeaponMaxRange < fDistanceToThreat )
						{
							m_uEnemyGuid = ASURA_GUID_UNREGISTERED;
							return false;
						}
					}
					
					Asura_Vector_3 xTestPos(xGunPos + ( xGunFwd * fWeaponMaxRange ) );

					// Need to check if the gun is inside a friendly collision box.
					if ( IsWeaponPositionInsideFriendlyBoundingBox(xGunPos) || AllyToCloseToFire(xGunPos, xTestPos ) )
					{
						m_uEnemyGuid = ASURA_GUID_UNREGISTERED;
						return false;
					}

					float fPercentage = ( pxOutput->m_fDistanceToIntersection / fDistanceToThreat );

					if ( fPercentage > 0.99f )
					{
						float fDot = xTestDirection * xGunFwd;

						if ( fDot > 0.95f )
						{
							return true;
						}
					}
				}
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Weapon :: AllyToCloseToFire
//---------------------------------------------------------------------------
bool AVP_Behaviour_Civilian_Attack::AllyToCloseToFire( const Asura_Vector_3 &xStart, const Asura_Vector_3& xEnd )
{
	const AVP_ServerActor_Human* pxOurActor	= (AVP_ServerActor_Human*)GetCivilianBrain()->GetCivilianActor();

	for ( Asura_Hashed_List_It< AVP_ServerActor_Human > it( AVP_ServerActor_Human::GetList() ); !it.Done(); it.Next() )
	{
		AVP_ServerActor_Human* pxHuman = it.GetCurrent();

		if ( pxHuman && !pxHuman->IsDead() && pxHuman->IsEntityActive() && ( pxHuman != pxOurActor ) )
		{
			if ( AVP_Tactical_System::GetAllegiance(*pxOurActor, *pxHuman) == AVP_ALLEGIANCE_FRIEND )
			{
				Asura_Vector_3			xNearestPoint = Asura_Maths::ZeroVector3;
				const Asura_Vector_3&	xHumanPos = pxHuman->GetPosition();

				Asura_Maths::GetNearestPointOnLineSegment( xStart, xEnd, pxHuman->GetPosition(), xNearestPoint );

				float fDistanceSq = ( xHumanPos - xNearestPoint ).MagnitudeSquared();
				float fDistFromStartsq = ( xHumanPos - xStart ).MagnitudeSquared();

				if ( fDistanceSq < 2.5f && fDistFromStartsq > 4.0f )
				{
					return true;
				}
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Weapon :: DoWeHitFriendlyBeforeEnemy
//---------------------------------------------------------------------------
bool AVP_Behaviour_Civilian_Attack::IsWeaponPositionInsideFriendlyBoundingBox(const Asura_Vector_3& xTestPos)
{
	const AVP_ServerActor_Human* pxOurActor	= (AVP_ServerActor_Human*)GetCivilianBrain()->GetCivilianActor();

	// Iterate through all humans
	for (Asura_Hashed_List_It<AVP_ServerActor_Human> xNodeIter(AVP_ServerActor_Human::GetList()); !xNodeIter.Done(); xNodeIter.Next())
	{
		AVP_ServerActor_Human* pxHuman = xNodeIter.GetCurrent();

		if ( pxHuman && ( pxHuman != pxOurActor ) )
		{
			if (!pxHuman->IsDead() && pxHuman->IsEntityActive())
			{
				if (pxHuman->GetPhysicsObject())
				{
					if (pxHuman->GetPhysicsObject()->GetBoundingBox().Contains(xTestPos))
					{
						return true;
					}
				}
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
float AVP_Behaviour_Civilian_Attack::GetWeaponAmmo(void)
{
	const AVP_ServerActor_Civilian*	pxHuman	= GetCivilianBrain()->GetCivilianActor();

	if (pxHuman)
	{
		const AVP_Inventory_System*	pxInventorySystem	= pxHuman->GetInventorySystem();
		const AVP_Inventory_Item*	pxCurrentWeapon		= pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL;

		if (pxCurrentWeapon != NULL)
		{
			const AVP_InventoryItem_SubType* pxSubType = pxCurrentWeapon->GetSubType(0);

			if (pxSubType != NULL)
			{
				return pxSubType->GetCurrentQuantity();
			}
		}
	}

	return 0.0f;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
bool AVP_Behaviour_Civilian_Attack::GetWeaponMuzzlePositionAndOrientation(Asura_Vector_3* xPosition, Asura_Matrix_3x3* xOrientation)
{
	const AVP_ServerActor_Civilian*	pxHuman	= GetCivilianBrain()->GetCivilianActor();

	if (pxHuman)
	{
		return pxHuman->GetWorldSpaceMarkerPoint(0x0001A7D2, xPosition, xOrientation); // Asura_GetHashID("muz")
	}

	return false;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Attack::CreateMovementBehaviour(AVP_ServerEntity_Actor* pxAttackActor)
{
	AVP_Behaviour_Human_Movement* pxHumanBehaviour = (AVP_Behaviour_Human_Movement*)CreateChildWithType(AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT);

	if (pxHumanBehaviour)
	{
		pxHumanBehaviour->GetToPosition(pxAttackActor->GetPosition(), AVP_Navigation::MM_SLOW, true);
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Attack::Shoot(void)
{
	AVP_ServerActor_Civilian*	pxOurActor	= GetCivilianBrain()->GetCivilianActor();

	if (pxOurActor)
	{
		AVP_ClientEntity_Actor* pxOurClientActor = AVP_ClientEntity_Actor::FindEntity(pxOurActor->GetGuid());

		if (pxOurClientActor)
		{
			AVP_Inventory_System*			pxInventorySystem	= pxOurActor->GetInventorySystem();
			AVP_InventoryItem_Weapon*		pxCurrentWeapon		= static_cast<AVP_InventoryItem_Weapon*>(pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL);

			if (pxCurrentWeapon != NULL)
			{
				AVP_RangedWeapon_SubType* pxSubType = static_cast<AVP_RangedWeapon_SubType*>( pxCurrentWeapon->GetSubType( 0 ) );

				if ( pxSubType->GetCurrentQuantity() > 0 )
				{
					if ( pxCurrentWeapon->IsIdling( true ) )
					{
						pxCurrentWeapon->RequestBeginUsage( AVP_InventoryWeapon_RangedWeapon::UT_N_BULLET_BURST_PRIMARY );
						m_fDontShootTimer = Asura_Random::GetFloat( pxSubType->GetBurstFireAIMinGap(), pxSubType->GetBurstFireAIMaxGap() );
					}
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Attack::StopShooting(void)
{
	AVP_ServerActor_Civilian*	pxOurActor	= GetCivilianBrain()->GetCivilianActor();

	if (pxOurActor)
	{
		AVP_ClientEntity_Actor* pxOurClientActor = AVP_ClientEntity_Actor::FindEntity( pxOurActor->GetGuid() );

		if (pxOurClientActor)
		{
			AVP_Inventory_System*	pxInventorySystem	= pxOurActor->GetInventorySystem();
			AVP_Inventory_Item*		pxCurrentWeapon		= pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL;

			if ( ( pxCurrentWeapon != NULL ) && pxCurrentWeapon->IsUsing( true ) && pxCurrentWeapon->GetCurrentUsageType() != AVP_InventoryWeapon_RangedWeapon::UT_RELOAD_ALL )
			{
				pxCurrentWeapon->RequestEndUsage(AVP_InventoryWeapon_RangedWeapon::UT_N_BULLET_BURST_PRIMARY );
			}
		}
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
float AVP_Behaviour_Civilian_Attack::GetWeaponsMaxRange(void)
{
	const AVP_ServerActor_Civilian*	pxOurActor	= GetCivilianBrain()->GetCivilianActor();

	if (pxOurActor)
	{
		const AVP_Inventory_System*	pxInventorySystem	= pxOurActor->GetInventorySystem();
		const AVP_Inventory_Item*	pxCurrentWeapon		= pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL;

		if (pxCurrentWeapon != NULL)
		{
			const AVP_RangedWeapon_SubType*	pxSubType	= static_cast<const AVP_RangedWeapon_SubType*>(pxCurrentWeapon->GetSubType(0));

			if (pxSubType != NULL)
			{
				return(pxSubType->GetMaxRange());
			}
		}
	}

	return 1.0f;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[AVP_Behaviour_Civilian_Attack::MAX_STATES] =
{
	"ATTACK - UNSET",
	"ATTACK - CHECK_TO_SHOOT",
	"ATTACK - SHOOT",
	"ATTACK - WAIT_TO_SHOOT",
	"ATTACK - RELOAD",
};

void AVP_Behaviour_Civilian_Attack::RenderDebugInfo(void)
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0xffff0000,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
		"FT : %0.2f", m_fDontShootTimer
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif
