//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Civilian_Patrol.cpp
// Created		: 04 / 03 / 2008
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------
// Core
#include "Core/Asura.h"

// This Files Header
#include "Behaviour/BehaviourTypes/Human/Civilian/AVP_Behaviour_Civilian_Patrol.h"

// Asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// AVP
#include "Entity/Locations/GuardZone/AVP_ServerNode_GuardZone.h"
#include "Entity/Locations/OrientedNode/AVP_ServerNode_OrientedNode.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Civilian/AVP_ServerActor_Civilian.h"

// AVP Axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Human/AVP_Behaviour_Human_Movement.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Civilian.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
AVP_Behaviour_Civilian_Patrol::AVP_Behaviour_Civilian_Patrol(AVP_Brain_Actor_Civilian* pxBrain)
: PARENT(pxBrain, AVP_BEHAVIOUR_TYPE_CIVILIAN_PATROL)
, m_eUpdateState(SELECT_GUARD_ZONE)
, m_pxCivilianBrain(pxBrain)
, m_uGuardZoneGuid(ASURA_GUID_UNREGISTERED)
, m_fWaitTime(0.0f)
, m_iCurrentPatrolIndex(-1)
, m_uCurrentPatrolGuid(ASURA_GUID_UNREGISTERED)
, m_uPreviousPatrolGuid(ASURA_GUID_UNREGISTERED)
{
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
bool AVP_Behaviour_Civilian_Patrol::Activate(void)
{
	const bool bParentOk = PARENT::Activate();

	return bParentOk;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Patrol::Deactivate(void)
{
	DestroyAllChildren( false );
	PARENT::Deactivate();

	if (m_uCurrentPatrolGuid != ASURA_GUID_UNREGISTERED)
	{
		AVP_ServerNode_OrientedNode* pxPatrolPoint = AVP_ServerNode_OrientedNode::FindEntity(m_uCurrentPatrolGuid);

		if (pxPatrolPoint)
		{
			pxPatrolPoint->SetIsOccupied(false);
		}
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Patrol::PerFrameUpdate(const float fTimeslice)
{
	AVP_ServerActor_Civilian*	pxOurActor	= GetCivilianBrain()->GetCivilianActor();
	bool						bHaveThreat	= (GetCivilianBrain()->GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid() != ASURA_GUID_UNREGISTERED) ? true : false;
	bool						bHaveAlert	= (GetCivilianBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge().GetGuid() != ASURA_GUID_UNREGISTERED) ? true : false;

	// if we have an alert or a threat set finished
	if ((bHaveThreat) || (bHaveAlert)) {SetFinished(); return;}

	switch (GetUpdateState())
	{
		//--------------------------------------
		case SELECT_GUARD_ZONE:
		{
			if (SelectGuardZone())
			{
				SetUpdateState(SELECT_PATROL_POINT);
			}
			else
			{
				SetFinished();
				return;
			}
		}break;
		//--------------------------------------
		case SELECT_PATROL_POINT:
		{
			AVP_ServerNode_GuardZone* pxGuardZone = AVP_ServerNode_GuardZone::FindEntity(m_uGuardZoneGuid);

			if (pxGuardZone)
			{
				AVP_ServerNode_OrientedNode* pxPatrolPoint = NULL;

				switch (pxGuardZone->GetPatrolType())
				{
					//--------------------------------------------------------------
					case AVP_ServerNode_GuardZone::GUARD_ZONE_PATROL_TYPE_LOOPING:
					{
						if (m_uCurrentPatrolGuid == ASURA_GUID_UNREGISTERED)
						{
							pxPatrolPoint = pxGuardZone->GetNearestGuardPoint(pxOurActor->GetPosition());
						}
						else
						{
							pxPatrolPoint = pxGuardZone->GetNextLoopingGuardPoint(m_uCurrentPatrolGuid);
						}

					}break;
					//--------------------------------------------------------------
					case AVP_ServerNode_GuardZone::GUARD_ZONE_PATROL_TYPE_PING_PONG:
					{
						if (m_uCurrentPatrolGuid == ASURA_GUID_UNREGISTERED)
						{
							pxPatrolPoint = pxGuardZone->GetNearestGuardPoint(pxOurActor->GetPosition());
						}
						else
						{
							pxPatrolPoint = pxGuardZone->GetNextPingPongGuardPoint(m_uCurrentPatrolGuid);
						}
					}break;
					//--------------------------------------------------------------
					case AVP_ServerNode_GuardZone::GUARD_ZONE_PATROL_TYPE_RANDOM:
					default:
					{
						pxPatrolPoint = pxGuardZone->GetRandomGuardPoint();
					}break;
					//--------------------------------------------------------------
				}

				if (pxPatrolPoint)
				{
					// Set the old point as not occupied
					if ((pxPatrolPoint->GetGuid() != m_uCurrentPatrolGuid) && (m_uCurrentPatrolGuid != ASURA_GUID_UNREGISTERED))
					{
						AVP_ServerNode_OrientedNode* pxOldPoint = AVP_ServerNode_OrientedNode::FindEntity(m_uCurrentPatrolGuid);

						if (pxOldPoint)
						{
							pxOldPoint->SetIsOccupied(false);
						}
					}

					pxPatrolPoint->SetIsOccupied(true);
					m_uCurrentPatrolGuid = pxPatrolPoint->GetGuid();
					CreateMovementBehaviour(pxPatrolPoint->GetPosition());
					SetUpdateState(MOVE_TO_PATROL_POINT);
				}
				else
				{
					SetUpdateState(WAIT_AT_PATROL_POINT);
				}
			}
			else
			{
				SetFinished();
				return;
			}
		}break;
		//--------------------------------------
		case MOVE_TO_PATROL_POINT:
		{
			AVP_Behaviour_Human_Movement* pxHumanBehaviour = (AVP_Behaviour_Human_Movement*)GetChildWithType(this, AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT);
			if (pxHumanBehaviour)
			{
				if (pxHumanBehaviour->IsFinished())
				{
					AVP_ServerNode_OrientedNode* pxPatrolPoint = AVP_ServerNode_OrientedNode::FindEntity(m_uCurrentPatrolGuid);
					m_fWaitTime = pxPatrolPoint ? ( pxPatrolPoint->GetWaitTime() + pxPatrolPoint->GetWaitTimeRandomOffset() ) : 0.0f;
					DestroyChildWithType(AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT);
					SetUpdateState(WAIT_AT_PATROL_POINT);
				}
			}
			else
			{
				SetUpdateState(SELECT_PATROL_POINT);
			}
		}break;
		//--------------------------------------
		case WAIT_AT_PATROL_POINT:
		{
			SetLookAtFromNode();

			m_fWaitTime -= fTimeslice;
			if ( m_fWaitTime < 0.0f )
			{
				SetUpdateState(SELECT_PATROL_POINT);
			}
		}break;
		//--------------------------------------
		case UNSET:
		default:
		{

		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Civilian_Patrol :: SetLookAtFromNode
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Patrol::SetLookAtFromNode(void)
{
	if (m_uCurrentPatrolGuid != ASURA_GUID_UNREGISTERED)
	{
		AVP_ServerNode_OrientedNode*	pxNode		= GetPatrolPoint();
		Asura_Matrix_3x3				xNodeOri;
		pxNode->GetOrientation( xNodeOri );
		Asura_Vector_3					xLookAtDir	= Asura_Vector_3(xNodeOri.e13, xNodeOri.e23, xNodeOri.e33);

		// Set The Actual Directions
		GetCivilianBrain()->SetAimDirection(xLookAtDir);
		GetCivilianBrain()->SetLookAtPosition(GetCivilianBrain()->GetActor()->GetPosition() + xLookAtDir * 10.0f);
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
bool AVP_Behaviour_Civilian_Patrol::SelectGuardZone(void)
{
	AVP_ServerActor_Civilian* pxOurActor = GetCivilianBrain()->GetCivilianActor();

	if (pxOurActor->GetGuardZoneGuid() != ASURA_GUID_UNREGISTERED)
	{
		m_uGuardZoneGuid = pxOurActor->GetGuardZoneGuid();
		return true;
	}
	else
	{
		return false;
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
AVP_ServerNode_OrientedNode* AVP_Behaviour_Civilian_Patrol::GetPatrolPoint(void)
{
	return AVP_ServerNode_OrientedNode::FindEntity(m_uCurrentPatrolGuid);
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
AVP_ServerNode_GuardZone* AVP_Behaviour_Civilian_Patrol::GetGuardZone(void)
{
	return AVP_ServerNode_GuardZone::FindEntity(m_uGuardZoneGuid);
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void AVP_Behaviour_Civilian_Patrol::CreateMovementBehaviour(const Asura_Vector_3& xGoToPoint)
{
	AVP_Behaviour_Human_Movement* pxHumanBehaviour = (AVP_Behaviour_Human_Movement*)CreateChildWithType(AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT);

	if (pxHumanBehaviour)
	{
		pxHumanBehaviour->GetToPosition(xGoToPoint, AVP_Navigation::MM_SLOW, true);
	}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[AVP_Behaviour_Civilian_Patrol::MAX_STATES] =
{
	"PATROL - UNSET",
	"PATROL - SELECT_GUARD_ZONE",
	"PATROL - SELECT_PATROL_POINT",
	"PATROL - MOVE_TO_PATROL_POINT",
	"PATROL - WAIT_AT_PATROL_POINT",
};

void AVP_Behaviour_Civilian_Patrol::RenderDebugInfo(void)
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif
