//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Civilian_Patrol.h
// Created		: 04 / 03 / 2008
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// BEGIN FILE WRAPPER ///////////////////////////////////////
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_BEHAVIOUR_CIVILIAN_PATROL_H__
#define __AVP_BEHAVIOUR_CIVILIAN_PATROL_H__ 1
// END FILE WRAPPER ///////////////////////////////////////////

// INCLUDES ///////////////////////////////////////////////////////////
#include "Behaviour/Axon_Project_Behaviour.h"

// FORWARD DECLARATIONS ///////////////////////////////////
class AVP_Brain_Actor_Civilian;
class AVP_ServerNode_OrientedNode;
class AVP_ServerNode_GuardZone;

//---------------------------------------------------------------------------
// Class		: AVP_Behaviour_Civilian_Patrol
//---------------------------------------------------------------------------
class AVP_Behaviour_Civilian_Patrol : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:

	// Patrol Update State Enum
	enum PatrolUpdateState
	{
		UNSET,
		SELECT_GUARD_ZONE,
		SELECT_PATROL_POINT,
		MOVE_TO_PATROL_POINT,
		WAIT_AT_PATROL_POINT,
		MAX_STATES
	};

	// Construction / Deconstruction
	AVP_Behaviour_Civilian_Patrol						(AVP_Brain_Actor_Civilian* pxBrain);
	virtual ~AVP_Behaviour_Civilian_Patrol				(void){}

	// Public Member Virtual Functions...
	virtual bool 					Activate			(void);
	virtual void 					Deactivate			(void);
	virtual void					PerFrameUpdate		(const float fTimeslice);

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void					RenderDebugInfo		(void);
#endif

	// Public Member Functions...
	PatrolUpdateState				GetUpdateState		(void) const					{return m_eUpdateState;}
	AVP_Brain_Actor_Civilian*		GetCivilianBrain	(void)							{return m_pxCivilianBrain;}

protected:

	void							SetUpdateState			(PatrolUpdateState eState)	{m_eUpdateState = eState;}
	void							CreateMovementBehaviour	(const Asura_Vector_3& xGoToPoint);
	bool							SelectGuardZone			(void);

	AVP_ServerNode_OrientedNode*	GetPatrolPoint			(void);
	AVP_ServerNode_GuardZone*		GetGuardZone			(void);
	void							SetLookAtFromNode		(void);

private:

	// Private Member Variables...
	PatrolUpdateState				m_eUpdateState;
	AVP_Brain_Actor_Civilian*		m_pxCivilianBrain;
	Asura_Guid						m_uGuardZoneGuid;
	float							m_fWaitTime;
	int								m_iCurrentPatrolIndex;
	Asura_Guid						m_uCurrentPatrolGuid;
	Asura_Guid						m_uPreviousPatrolGuid;

	enum { CURRENT_CHUNK_VERSION = 0 };

};

#endif //__AVP_BEHAVIOUR_CIVILIAN_PATROL_H__

/// End Of File 'AVP_Behaviour_Civilian_Patrol.h'
