//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_GS2Node_Marine_Combat.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Core
#include "Core/Asura.h"

// This files header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_GS2Node_Marine_Combat.h"

// Asura Includes
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Locations/Spline/GamesceneSpline/Asura_ServerNode_GS2Spline.h"
#include "Network/Asura_Message.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// Tools Includes
#ifdef ASURA_TOOLS
#include "Entity/Locations/Tools_Entity_Node.h"
#include "Interface/Asura_Dialog.h"
#include "Load&Save/Tools_SaveAsura.h"
#endif

// AVP
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateMachine.h"
#include "Animation/AVP_Animation_BodyRig.h"
#include "Behaviour/AVP_BehaviourParams.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Weapon.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Entity/AVP_Messages.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/Locations/LookAt/AVP_ServerEntity_LookAt.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Tactical/AVP_Tactical_System.h"

// axon
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Actor.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"



//////////////////////////////////////////////////////////////////////////////
// Function Implementations
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
TOOLS_DECLARE_SHARED_PARAM( bool,         0xa8163c2b, AVPGS2MarCmbt_bCrouched );			// Asura_GetHashID( "AVPGS2MarCmbt_bCrouched" )
TOOLS_DECLARE_SHARED_PARAM( bool,         0xc60b8232, AVPGS2MarCmbt_bExitOnTimeOut );		// Asura_GetHashID( "AVPGS2MarCmbt_bExitOnTimeOut" )
TOOLS_DECLARE_SHARED_PARAM( float,        0xe72f6e4b, AVPGS2MarCmbt_fTimeOut );				// Asura_GetHashID( "AVPGS2MarCmbt_fTimeOut" )
#endif

DEBUGVAR float s_fAimLerpMod = 0.25f;
DEBUGVAR float s_fMinLook = 0.4f;
DEBUGVAR float s_fMaxLook = 1.0f;


//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
AVP_Behaviour_GS2Node_Marine_Combat::AVP_Behaviour_GS2Node_Marine_Combat( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( AVP_BEHAVIOUR_TYPE_GS2_MARINE_NODE_COMBAT, pxBrain )
, m_bCrouched( false )
, m_bExitOnTimeOut( false )
, m_fTimeOut( 0.0f )
, m_fTimeOutTimer( 0.0f )
, m_bExitCalled( false )
, m_uCurrentGS2NodeGuid( ASURA_GUID_UNREGISTERED )
, m_uGS2LinkGuid( ASURA_GUID_UNREGISTERED )
, m_bAlertSet( false )
, m_fAlertTimer( 0.0f )
, m_xAlertPosition( Asura_Maths::ZeroVector3 )
, m_fLookTimeOut( 0.0f )
, m_uLookAtGuid( ASURA_GUID_UNREGISTERED )
, m_fScanTimer( 0.0f )
{
	m_fLookTimeOut = Asura_Random::GetFloat( s_fMinLook, s_fMaxLook );
}

bool AVP_Behaviour_GS2Node_Marine_Combat::HasNextGuidControl()
{
	return !m_bExitCalled || m_uGS2LinkGuid != ASURA_GUID_UNREGISTERED;
}


//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
Asura_Guid AVP_Behaviour_GS2Node_Marine_Combat::GetNextGamesceneEntityGuid()
{
	LeaveNode();
	return m_uGS2LinkGuid;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
bool AVP_Behaviour_GS2Node_Marine_Combat::Activate( void )
{
	AVP_ServerActor_Human*	pxOurActor	= GetMarineBrain()->GetHumanActor();
	const bool				bParentOk	= PARENT::Activate();

	if ( GetMarineBrain()->IsActive() )
	{
		GetMarineBrain()->SetIsAllowedToExitGS2OnKnowledge(false);
	}

	GetMarineBrain()->GetTacticalSystem()->SetIsOnGameScene(false);

	// Create The Weapon Behaviour
	CreateChildWithType(AVP_BEHAVIOUR_TYPE_MARINE_WEAPON);

	// Tell The AI Anim State Machine that we aren't controlling the animation
	pxOurActor->GetAIAnimStateMachine()->RequestEndGamescene();

	if ( GetGSActorBehaviour() )
	{
		// Find out Gamescene Node
		m_uCurrentGS2NodeGuid = GetGSActorBehaviour()->GetCurrentEntityGuid();
	}

	if ( m_uCurrentGS2NodeGuid != ASURA_GUID_UNREGISTERED )
	{
		AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity(m_uCurrentGS2NodeGuid);

		if ( pxGS2Node )
		{
			// It is going to teleport us to the position and orientation of the node but we want to stop that
			Asura_Matrix_3x3 xNewOri;
			pxGS2Node->GetOrientation( xNewOri );

			Asura_Vector_3 xFwd( xNewOri.e13, 0.0f, xNewOri.e33 );

			Asura_Maths::OrientationFromDirection( xNewOri, xFwd );

			if ( GetMarineBrain()->GetTacticalSystem()->IsOnCombatGS2() )
			{
				pxOurActor->TeleportTo( pxGS2Node->GetPosition(), pxOurActor->GetOrientation(), true );
			}
			else
			{
				pxOurActor->TeleportTo( pxGS2Node->GetPosition(), xNewOri, true );
			}
			
			AVP_Actor_PhysicsObjectHandler* pxPhysicsHandler = pxOurActor->GetPhysicsObjectHandler();

			if ( pxPhysicsHandler )
			{
				pxPhysicsHandler->DisableExternalControlPhysics( AVP_PhysicsObjectHandler::ECREQ_GAMESCENE );
			}

			AVP_Human_AIAnimStateMachine* pxStateMachine = ( AVP_Human_AIAnimStateMachine* )pxOurActor->GetAIAnimStateMachine();

			if ( pxStateMachine )
			{
				pxStateMachine->RequestIdle();
			}
		}
	}

	return bParentOk;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_Combat::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << static_cast<int>(CURRENT_CHUNK_VERSION);

	PARENT::WriteToChunkStream(xStream);

	xStream << m_uCurrentGS2NodeGuid;
	
	if ( m_bCrouched == 0 )
	{
		xStream << 0;
	}
	else
	{
		xStream << 1;
	}

	if ( m_bExitOnTimeOut == 0 )
	{
		xStream << 0;
	}
	else
	{
		xStream << 1;
	}

	xStream << m_fTimeOut;
	xStream << m_fTimeOutTimer;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_Combat::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream(xStream);

	if ( iVersion >= 0 )
	{
		xStream >> m_uCurrentGS2NodeGuid;

		if (HasActivateBeenCalled())
		{
			AVP_ServerActor_Human*	pxOurActor	= GetMarineBrain()->GetHumanActor();

			if ( GetMarineBrain()->IsActive() )
			{
				GetMarineBrain()->SetIsAllowedToExitGS2OnKnowledge(false);
			}

			// Create The Weapon Behaviour
			CreateChildWithType(AVP_BEHAVIOUR_TYPE_MARINE_WEAPON);

			if ( m_uCurrentGS2NodeGuid != ASURA_GUID_UNREGISTERED )
			{
				AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity(m_uCurrentGS2NodeGuid);

				if ( pxGS2Node )
				{
					// It is going to teleport us to the position and orientation of the node but we want to stop that
					Asura_Matrix_3x3 xNewOri;
					Asura_Vector_3 xFwd( 0.0f, 0.0f, 1.0f );
					Asura_Maths::OrientationFromDirection( xNewOri, xFwd );
					pxOurActor->TeleportTo( pxGS2Node->GetPosition(), xNewOri, true );

					AVP_Actor_PhysicsObjectHandler* pxPhysicsHandler = pxOurActor->GetPhysicsObjectHandler();

					if ( pxPhysicsHandler )
					{
						// GBT :: Sort of a hack.....
						// Because the system backs up the last set Physics type. I have to force it to actor before enabling external control, -
						// then disable external control. so that when it disables it again later it goes to the correct type... ridiculous!
						pxPhysicsHandler->GetPhysicsObject()->ChangeType(ASURA_PHYSICS_OBJECT_ACTOR);
						pxPhysicsHandler->EnableExternalControlPhysics( AVP_PhysicsObjectHandler::ECREQ_GAMESCENE );
						pxPhysicsHandler->DisableExternalControlPhysics( AVP_PhysicsObjectHandler::ECREQ_GAMESCENE );
					}
				}
			}
		}
	}

	if ( iVersion >= CURRENT_CHUNK_VERSION )
	{
		u_int uBoolValue = 0;
		xStream >> uBoolValue;

		if ( uBoolValue == 0 )
		{
			m_bCrouched = false;
		}
		else
		{
			m_bCrouched = true;
		}

		xStream >> uBoolValue;

		if ( uBoolValue == 0 )
		{
			m_bExitOnTimeOut = false;
		}
		else
		{
			m_bExitOnTimeOut = true;
		}

		xStream >> m_fTimeOut;
		xStream >> m_fTimeOutTimer;
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_Combat::Deactivate( void )
{
	DestroyAllChildren();

	if ( GetMarineBrain()->IsActive() )
	{
		GetMarineBrain()->SetIsAllowedToExitGS2OnKnowledge(true);

		GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( 1.0f );

		if (GetMarineBrain()->GetTacticalSystem()->IsPerformingAction())
		{
			GetMarineBrain()->GetTacticalSystem()->SetIsOnGameScene(true);
		}

		AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();

		if ( HasNextGuidControl() )
		{
			AVP_Actor_PhysicsObjectHandler* pxPhysicsHandler = pxOurActor->GetPhysicsObjectHandler();

			if ( pxPhysicsHandler )
			{
				pxPhysicsHandler->EnableExternalControlPhysics( AVP_PhysicsObjectHandler::ECREQ_GAMESCENE );
			}
		}
	}

	PARENT::Deactivate();
}

void AVP_Behaviour_GS2Node_Marine_Combat::PostLoadInitialise()
{
	PARENT::PostLoadInitialise();

	// Unfortunately the anim state machine is not created until PLI time.
	// Tell The AI Anim State Machine that we aren't controlling the animation
	GetMarineBrain()->GetHumanActor()->GetAIAnimStateMachine()->RequestEndGamescene();
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_Combat::PerFrameUpdate( const float fTimeslice )
{
	// If our time out has run out and some how are still in this behaviour the set finished and return.
	if ( m_bExitCalled )
	{
		SetFinished();
		return;
	}

	// Make sure we are always aiming in this behaviour
	if ( !GetMarineBrain()->GetHumanActor()->IsAiming() )
	{
		GetMarineBrain()->GetHumanActor()->SetAiming( true );
	}

	
	// Check for time if this nodes has the exit on time out condition set.
	if ( m_bExitOnTimeOut )
	{
		m_fTimeOutTimer += fTimeslice;

		if ( m_fTimeOutTimer >= m_fTimeOut )
		{
			m_bExitCalled = true;
			LeaveNode();
			return;
		}
	}

	// If we have a threat then look at it otherwise look in the direction of the nodes orientation
	if ( GetMarineBrain()->DoesHaveThreat() )
	{
		m_bAlertSet = false;
		m_fAlertTimer = 0.0f;
		m_xAlertPosition = Asura_Maths::ZeroVector3;
		GetMarineBrain()->AimAtThreat();
		GetMarineBrain()->LookAtThreat();
		GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( 1.0f );
	}
	else if ( GetMarineBrain()->DoesHaveAlert() || ( m_bAlertSet && m_fAlertTimer > 0.0f ) )
	{
		if ( !m_bAlertSet )
		{
			m_bAlertSet = true;
			m_fAlertTimer = 4.0f;
			m_xAlertPosition = GetMarineBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge().m_xPosition;
		}

		m_fAlertTimer -= fTimeslice;

		GetMarineBrain()->AimAtPosition( m_xAlertPosition );
		GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( 1.0f );
	}
	else if ( GetMarineBrain()->GetKnowledgeSystem()->DoesHaveReconKnowledge() )
	{
		GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( 1.0f );
		m_bExitCalled = true;
	}
	else
	{
		GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( s_fAimLerpMod );
		m_bAlertSet = false;
		m_fAlertTimer = 0.0f;
		m_xAlertPosition = Asura_Maths::ZeroVector3;
		//LookInNodesDirection();
		DoLooking(fTimeslice);
	}

	AVP_ClientEntity_Actor* pxClientActor = AVP_ClientEntity_Actor::FindEntity( GetMarineBrain()->GetOwnerGuid() );
	// If we should be crouched then crouch!

	if ( !pxClientActor->GetAnimationHandler()->GetBodyRig()->IsTurning() )
	{
		if ( m_bCrouched )
		{
			GetMarineBrain()->Crouch();
		}
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_Combat::HandleParams( const Axon_BehaviourParams* pxParams )
{
	PARENT::HandleParams( pxParams );

	if ( pxParams->GetParamsType() == AVP_BP_TYPE_MARINE_COMBAT )
	{
		const AVP_BehaviourParams_Marine_Combat* pxMrnCmbtParams = static_cast< const AVP_BehaviourParams_Marine_Combat* >( pxParams );

		m_bCrouched			= pxMrnCmbtParams->m_bCrouched;
		m_bExitOnTimeOut	= pxMrnCmbtParams->m_bExitOnTimeOut;
		m_fTimeOut			= pxMrnCmbtParams->m_fTimeOut;
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_Combat::LeaveNode(void)
{
	m_uGS2LinkGuid = ASURA_GUID_UNREGISTERED;
	SetFinished();

	AVP_ServerNode_GS2Node* pxCurrentNode = AVP_ServerNode_GS2Node::FindEntity( m_uCurrentGS2NodeGuid );
	if ( !pxCurrentNode )
	{
		SetInError();
		return;
	}

	u_int uNumFreeNodes = 0;
	for ( u_int u = 0; u < pxCurrentNode->GetNumberLinks(); ++u )
	{
		const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxCurrentNode->GetLink( u ) );
		if ( !pxLink )
		{
			SetInError();
			return;
		}
		if ( pxLink->IsOccupied() )
		{
			continue;
		}

		const Asura_ServerNode_GS2Node* pxNextNode = Asura_ServerNode_GS2Node::FindEntity( pxLink->GetTargetGuid() );
		if ( !pxNextNode )
		{
			SetInError();
			return;
		}
		if ( pxNextNode->IsOccupied() || pxNextNode->HasPendingActors() )
		{
			continue;
		}

		++uNumFreeNodes;
	}
	if ( uNumFreeNodes == 0 )
	{
		return;
	}

	// Pick a free node at random.
	const u_int uRandomLink = Asura_Random::GetIntU( 0, uNumFreeNodes - 1 );
	uNumFreeNodes = 0;
	for ( u_int u = 0; u < pxCurrentNode->GetNumberLinks(); ++u )
	{
		const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxCurrentNode->GetLink( u ) );
		if ( !pxLink )
		{
			SetInError();
			return;
		}
		if ( pxLink->IsOccupied() )
		{
			continue;
		}

		const Asura_ServerNode_GS2Node* pxNextNode = Asura_ServerNode_GS2Node::FindEntity( pxLink->GetTargetGuid() );
		if ( !pxNextNode )
		{
			SetInError();
			return;
		}
		if ( pxNextNode->IsOccupied() || pxNextNode->HasPendingActors() )
		{
			continue;
		}

		if ( uNumFreeNodes == uRandomLink )
		{
			m_uGS2LinkGuid = pxLink->GetGuid();
			return;
		}
		++uNumFreeNodes;
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
bool AVP_Behaviour_GS2Node_Marine_Combat::HandleMessage( const Asura_Message* pxMessage )
{
	switch (pxMessage->GetMsgID())
	{
		case ASURA_MSG_AXON_GS2_LEAVE_NODE:
		{
			LeaveNode();
			return true;
		}

		case AVP_MSGID_PERFORM_AI_OBJECTIVE_ACTION:
		{
			const AVP_Message_PerformObjectiveAction* pxMsgData = static_cast<const AVP_Message_PerformObjectiveAction*>(pxMessage->GetData());

			if (pxMsgData)
			{
				if (pxMsgData->GetActionGuid() != ASURA_GUID_UNREGISTERED)
				{
					GetMarineBrain()->GetHumanActor()->SetActionGamesceneGuid( pxMsgData->GetActionGuid() );
				}
			}
			return true;
		}

		default:
		{
			return PARENT::HandleMessage(pxMessage);
		}
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_Combat::LookInNodesDirection()
{
	AVP_ServerNode_GS2Node*		pxGS2Node	= AVP_ServerNode_GS2Node::FindEntity( m_uCurrentGS2NodeGuid );

	if ( pxGS2Node )
	{
		Asura_Matrix_3x3 xOri;
		pxGS2Node->GetOrientation( xOri );
		Asura_Vector_3 xFwd( xOri.e13, xOri.e23, xOri.e33 );
		GetMarineBrain()->SetAimDirection( xFwd );
		GetMarineBrain()->SetLookAtPosition( GetMarineBrain()->GetActor()->GetPosition() + xFwd * 10.0f );
		return;
	}

	GetMarineBrain()->AimForward();
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_Combat::DoLooking(const float fTimeslice)
{
// 	AVP_ServerActor_Human*				pxOurActor = GetMarineBrain()->GetHumanActor();
// 	
// 	AVP_ServerEntity_GamePlayVolume*	pxGPVolume = pxOurActor->GetCurrentGamePlayVolume();
// 
// 	if ( pxGPVolume )
// 	{
// 		u_int uNumGPVolumes = pxGPVolume->GetNumLookAtPointGuids() + 1;
// 
// 		AVP_ServerEntity_GamePlayVolume* pxTestVolume = NULL;
// 
// 		for (u_int u = 0; u < uNumGPVolumes; ++u )
// 		{
// 			if ( u == 0 )
// 				pxTestVolume = pxGPVolume;
// 			else
// 				pxTestVolume = AVP_ServerEntity_GamePlayVolume::FindEntity( pxGPVolume->GetLinkedGamePlayVolumeGuid( u - 1 ) );
// 
// 			if ( pxTestVolume )
// 			{
// 				pxTestVolume->GetNumLookAtPointGuids();
// 			}
// 		}
// 	}

	AVP_ServerActor_Human*				pxOurActor = GetMarineBrain()->GetHumanActor();
	AVP_ServerEntity_GamePlayVolume*	pxGPVolume = pxOurActor->GetCurrentGamePlayVolume();

	if ( pxGPVolume )
	{
		if ( pxGPVolume->GetNumLookAtPointGuids() > 0 )
		{
			float fDot = 0.0f;

			if ( m_uLookAtGuid != ASURA_GUID_UNREGISTERED )
			{
				Asura_Vector_3 xAimDir = GetMarineBrain()->GetAimDirection();
				Asura_Vector_3 xMuzPos( Asura_Maths::ZeroVector3 );
				
				pxOurActor->GetWorldSpaceMarkerPoint( 0x0001A7D2/*muz*/, &xMuzPos, NULL );

				AVP_ServerEntity_LookAt* pxCurrentLookAt = AVP_ServerEntity_LookAt::FindEntity( m_uLookAtGuid );

				if ( pxCurrentLookAt )
				{
					Asura_Vector_3 xDirToLookAt( pxCurrentLookAt->GetPosition() - xMuzPos );
					xDirToLookAt.Normalise();

					fDot = xDirToLookAt * xAimDir;
				}
			}

			if ( fDot > 0.999f || m_uLookAtGuid == ASURA_GUID_UNREGISTERED )
			{
				m_fScanTimer += fTimeslice;

				if ( m_fScanTimer > m_fLookTimeOut )
				{
					AVP_ServerEntity_LookAt*	pxLookAt		= NULL;
					u_int						uLookAtIndex	= 0;

					if ( pxGPVolume->GetNumLookAtPointGuids() > 1 )
					{
						uLookAtIndex = Asura_Random::GetIntU( 0, pxGPVolume->GetNumLookAtPointGuids() - 1 );
					}

					pxLookAt = AVP_ServerEntity_LookAt::FindEntity( pxGPVolume->GetLookAtPointGuid( uLookAtIndex ) );

					if ( pxLookAt )
					{
						Asura_Vector_3 xHeadPos( Asura_Maths::ZeroVector3 );
						pxOurActor->GetHeadPosition( xHeadPos );

						Asura_Vector_3 xToLookAt = pxLookAt->GetPosition() - xHeadPos;
						float fDistanceToLookAt = xToLookAt.Normalise();

						// Do a LOS test
						const Asura_LOS_InputData	xInput = Asura_LOS_InputData( xHeadPos, xToLookAt, pxOurActor->GetGuid(), fDistanceToLookAt, false);
						const Asura_LOS_OutputData*	pxOutput = Asura_LOS::TestAgainstEnvironment( &xInput );

						if ( pxOutput->m_fDistanceToIntersection == fDistanceToLookAt )
						{
							// look at it!
							GetMarineBrain()->SetLookAtPosition( xHeadPos + xToLookAt );
							GetMarineBrain()->SetAimDirection( xToLookAt );
							GetMarineBrain()->GetTacticalSystem()->LetRouteSetLookAt( false );
							GetMarineBrain()->GetTacticalSystem()->SetForceAiming( true );

							m_fScanTimer = 0.0f;
							m_fLookTimeOut = Asura_Random::GetFloat( s_fMinLook, s_fMaxLook );
							m_uLookAtGuid = pxLookAt->GetGuid();
						}
						else
						{
							LookInNodesDirection();
						}
					}
				}
			}
		}
		else
		{
			LookInNodesDirection();
		}
	}


}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
AVP_Brain_Actor_Marine* AVP_Behaviour_GS2Node_Marine_Combat::GetMarineBrain( void ) const
{
	return static_cast< AVP_Brain_Actor_Marine* >( GetBrain() );
}

DEBUGVAR bool s_bRenderLookInfo = true;

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Behaviour_GS2Node_Marine_Combat::RenderDebugInfo( void )
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetDbgOutputPosX(), GetDbgOutputPosY(), 0.5f, 0xffffffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED, "AVP MARINE NODE COMBAT" );

	GetMarineBrain()->GetDebugOutputPos().y += 10.0f;

	if ( m_bExitOnTimeOut )
	{
		float fPrintValue = m_fTimeOut - m_fTimeOutTimer;

		Asura_Font_System::Print( GetDbgOutputPosX(), GetDbgOutputPosY(), 0.5f, 0xffffffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Timeout Left : %0.2f", fPrintValue );

		GetMarineBrain()->GetDebugOutputPos().y += 10.0f;
	}

	if ( s_bRenderLookInfo )
	{
		Asura_Font_System::Print( GetDbgOutputPosX(), GetDbgOutputPosY(), 0.5f, 0xffffffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Look At Guid = %u", m_uLookAtGuid );
		GetMarineBrain()->GetDebugOutputPos().y += 10.0f;

		Asura_Font_System::Print( GetDbgOutputPosX(), GetDbgOutputPosY(), 0.5f, 0xffffffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Look At TimeOut = %0.2f", m_fLookTimeOut );
		GetMarineBrain()->GetDebugOutputPos().y += 10.0f;

		Asura_Font_System::Print( GetDbgOutputPosX(), GetDbgOutputPosY(), 0.5f, 0xffffffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Look At Timer = %0.2f", m_fScanTimer );
		GetMarineBrain()->GetDebugOutputPos().y += 10.0f;
	}
}
#endif

bool AVP_BehaviourParams_Marine_Combat::s_bTimeOutChanged = false;

//////////////////////////////////////////////////////////////////////////////
AVP_BehaviourParams_Marine_Combat::AVP_BehaviourParams_Marine_Combat( const u_int uBehaviourType )
: PARENT( AVP_BP_TYPE_MARINE_COMBAT, uBehaviourType )
, m_bCrouched( false )
, m_bExitOnTimeOut( false )
, m_fTimeOut( 1.0f )
, m_ucCrouched( 0 )
, m_ucExitOnTimeOut( 0 )
{

}

//////////////////////////////////////////////////////////////////////////////
bool AVP_BehaviourParams_Marine_Combat::IsBaseOrDerivativeOf( const u_int uParamsType ) const
{
	return ( uParamsType == AVP_BP_TYPE_MARINE_COMBAT ) ? true : PARENT::IsBaseOrDerivativeOf( uParamsType );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_Combat::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( iVersion == 0 )
	{
		xStream >> m_ucCrouched;

		if (m_ucCrouched == 1)
		{
			m_bCrouched = true;
		}
		else
		{
			m_bCrouched = false;
		}

		m_ucExitOnTimeOut = false;
		m_bExitOnTimeOut = false;
		m_fTimeOut = 1.0f;
	}
	else if ( iVersion == 1 )
	{
		xStream >> m_ucCrouched;

		if (m_ucCrouched == 1)
		{
			m_bCrouched = true;
		}
		else
		{
			m_bCrouched = false;
		}

		xStream >> m_ucExitOnTimeOut;

		if (m_ucExitOnTimeOut == 1)
		{
			m_bExitOnTimeOut = true;
		}
		else
		{
			m_bExitOnTimeOut = false;
		}

		xStream >> m_fTimeOut;

	}
	else if ( iVersion == 2 )
	{
		xStream >> m_bCrouched;
		xStream >> m_bExitOnTimeOut;
		xStream >> m_fTimeOut;
	}
	else
	{
		Asura_Assertf( false, ( "Axon_BehaviourParams_GS2AttractorEnRoute::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_Combat::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << CURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_bCrouched;
	xStream << m_bExitOnTimeOut;
	xStream << m_fTimeOut;
}

#ifdef ASURA_TOOLS
//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_Combat::DoAddOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset )
{
	AddOverlayControls( pxTabsDialog, xOffset );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_Combat::AddOverlayControls(Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls( pxTabsDialog, xOffset );

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_SharedTickbox_Dialog("Crouch", xOffset, &g_xParam_AVPGS2MarCmbt_bCrouched);
	pxDialog->SetHelpText("Select this to have the marine crouch at this node.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Exit On Timeout", xOffset, &g_xParam_AVPGS2MarCmbt_bExitOnTimeOut);
	pxDialog->SetHelpText("Select this to have the marine exit this node on a time out.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if (m_bExitOnTimeOut)
	{
		pxDialog = new Asura_Numerical_Dialog("TimeOut", xOffset, &g_xParam_AVPGS2MarCmbt_fTimeOut, 0.5f, 360.0f , 0.1f, "%.1f");
		pxDialog->SetHelpText("When this time is hit the behaviour will terminate.");
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void AVP_BehaviourParams_Marine_Combat::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( s_bTimeOutChanged )
	{
		s_bTimeOutChanged = false;
		Tools_SaveAsura::SetEntityModified();
		GetToolsNode()->RequestRefreshOverlay();
	}
}

bool AVP_BehaviourParams_Marine_Combat::GetParameter( const Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch ( uParamID )
	{
		case g_uHash_AVPGS2MarCmbt_bCrouched:
		{
			pxParam->SetBool( m_bCrouched );
			return true;
		}break;

		case g_uHash_AVPGS2MarCmbt_bExitOnTimeOut:
		{
			pxParam->SetBool( m_bExitOnTimeOut );
			return true;

		}break;

		case g_uHash_AVPGS2MarCmbt_fTimeOut:
		{
			pxParam->SetFloat( m_fTimeOut );
			return true;
		}break;

		default:
		{
			return PARENT::GetParameter( uParamID, pxParam );
		}break;
	}
}

bool AVP_BehaviourParams_Marine_Combat::SetParameter( const Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	switch ( uParamID )
	{
		case g_uHash_AVPGS2MarCmbt_bCrouched:
		{
			m_bCrouched = pxParam->GetBool();
			Tools_SaveAsura::SetEntityModified();
			return true;
		}break;

		case g_uHash_AVPGS2MarCmbt_bExitOnTimeOut:
		{
			m_bExitOnTimeOut = pxParam->GetBool();
			s_bTimeOutChanged = true;
			return true;

		}break;

		case g_uHash_AVPGS2MarCmbt_fTimeOut:
		{
			m_fTimeOut = pxParam->GetFloat();
			Tools_SaveAsura::SetEntityModified();
			return true;
		}break;

		default:
		{
			return PARENT::SetParameter( uParamID, pxParam );
		}break;
	}
}

//////////////////////////////////////////////////////////////////////////////
Axon_BehaviourParams* AVP_BehaviourParams_Marine_Combat::Clone() const
{
	return new AVP_BehaviourParams_Marine_Combat( *this );
}
#endif

/// END OF FILE
