//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_GS2Node_Marine_LookAt.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Core
#include "Core/Asura.h"

// This files header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_GS2Node_Marine_LookAt.h"

// Asura Includes
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Locations/Spline/GamesceneSpline/Asura_ServerNode_GS2Spline.h"
#include "Network/Asura_Message.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// Tools Includes
#ifdef ASURA_TOOLS
#include "Entity/Locations/LookAt/ToolsAVP_Entity_LookAt.h"
#include "Entity/Locations/Tools_Entity_Node.h"
#include "Entity/PhysicalObject/Actor/ToolsAVP_Entity_Actor.h"
#include "Interface/Asura_Dialog.h"
#include "Load&Save/Tools_SaveAsura.h"

#endif

// AVP
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateMachine.h"
#include "Animation/AVP_Animation_BodyRig.h"
#include "Behaviour/AVP_BehaviourParams.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Weapon.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Entity/AVP_Messages.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/Locations/LookAt/AVP_ServerEntity_LookAt.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Tactical/AVP_Tactical_System.h"

// axon
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Actor.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"

//////////////////////////////////////////////////////////////////////////////
// Function Implementations
//////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
AVP_Behaviour_GS2Node_Marine_LookAt::AVP_Behaviour_GS2Node_Marine_LookAt( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( AVP_BEHAVIOUR_TYPE_GS2_MARINE_NODE_LOOKAT, pxBrain )
, m_uCurrentGS2NodeGuid( ASURA_GUID_UNREGISTERED )
, m_uGS2LinkGuid( ASURA_GUID_UNREGISTERED )
, m_bExitCalled( false )
, m_bLookAtPlayer( false )
, m_uLookAtEntityGuid( ASURA_GUID_UNREGISTERED )
, m_bExitOnTimeOut( false )
, m_fTimeOut( 0.0f )
, m_fTimeOutTimer( 0.0f )
, m_bLookAtActor( false )
{

}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
bool AVP_Behaviour_GS2Node_Marine_LookAt::HasNextGuidControl()
{
	return !m_bExitCalled || m_uGS2LinkGuid != ASURA_GUID_UNREGISTERED;
}


//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
Asura_Guid AVP_Behaviour_GS2Node_Marine_LookAt::GetNextGamesceneEntityGuid()
{
	LeaveNode();
	return m_uGS2LinkGuid;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
bool AVP_Behaviour_GS2Node_Marine_LookAt::Activate( void )
{
	AVP_ServerActor_Human*	pxOurActor	= GetMarineBrain()->GetHumanActor();
	const bool				bParentOk	= PARENT::Activate();

	if ( GetMarineBrain()->IsActive() )
	{
		GetMarineBrain()->SetIsAllowedToExitGS2OnKnowledge(true);
	}

	GetMarineBrain()->GetTacticalSystem()->SetIsOnGameScene(false);

	// Tell The AI Anim State Machine that we aren't controlling the animation
	pxOurActor->GetAIAnimStateMachine()->RequestEndGamescene();

	if ( GetGSActorBehaviour() )
	{
		// Find out Gamescene Node
		m_uCurrentGS2NodeGuid = GetGSActorBehaviour()->GetCurrentEntityGuid();
	}

	if ( m_uCurrentGS2NodeGuid != ASURA_GUID_UNREGISTERED )
	{
		AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity(m_uCurrentGS2NodeGuid);

		if ( pxGS2Node )
		{
			AVP_Actor_PhysicsObjectHandler* pxPhysicsHandler = pxOurActor->GetPhysicsObjectHandler();

			if ( pxPhysicsHandler )
			{
				pxPhysicsHandler->DisableExternalControlPhysics( AVP_PhysicsObjectHandler::ECREQ_GAMESCENE );
			}

			AVP_Human_AIAnimStateMachine* pxStateMachine = ( AVP_Human_AIAnimStateMachine* )pxOurActor->GetAIAnimStateMachine();

			if ( pxStateMachine )
			{
				pxStateMachine->RequestIdle();
			}
		}

		SetLookAtData();
	}

	return bParentOk;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_LookAt::SetLookAtData()
{
	if ( !m_bLookAtPlayer )
	{
		AVP_ServerEntity_LookAt* pxLookAt = AVP_ServerEntity_LookAt::FindEntity( m_uLookAtEntityGuid );

		if ( pxLookAt )
		{
			m_bLookAtActor = false;
		}
		else
		{
			AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( m_uLookAtEntityGuid );
			
			if ( pxActor )
			{
				m_bLookAtActor = true;
			}
		}
	}
	else
	{
		AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( 1000 );

		if ( pxPlayerInfo )
		{
			m_bLookAtActor = true;
			m_uLookAtEntityGuid = pxPlayerInfo->GetPuppetGuid();
		}
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_LookAt::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << static_cast<int>(CURRENT_CHUNK_VERSION);

	PARENT::WriteToChunkStream(xStream);

	xStream << m_uCurrentGS2NodeGuid;
	xStream << m_bLookAtPlayer;
	xStream << m_uLookAtEntityGuid;
	xStream << m_bExitOnTimeOut;
	xStream << m_fTimeOutTimer;
	xStream << m_fTimeOut;
	
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_LookAt::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream(xStream);

	if ( iVersion >= 0 )
	{
		xStream >> m_uCurrentGS2NodeGuid;

		if (HasActivateBeenCalled())
		{
			AVP_ServerActor_Human*	pxOurActor	= GetMarineBrain()->GetHumanActor();

			if ( GetMarineBrain()->IsActive() )
			{
				GetMarineBrain()->SetIsAllowedToExitGS2OnKnowledge(true);
			}

			if ( m_uCurrentGS2NodeGuid != ASURA_GUID_UNREGISTERED )
			{
				AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity(m_uCurrentGS2NodeGuid);

				if ( pxGS2Node )
				{
					AVP_Actor_PhysicsObjectHandler* pxPhysicsHandler = pxOurActor->GetPhysicsObjectHandler();

					if ( pxPhysicsHandler )
					{
						// GBT :: Sort of a hack.....
						// Because the system backs up the last set Physics type. I have to force it to actor before enabling external control, -
						// then disable external control. so that when it disables it again later it goes to the correct type... ridiculous!
						pxPhysicsHandler->GetPhysicsObject()->ChangeType(ASURA_PHYSICS_OBJECT_ACTOR);
						pxPhysicsHandler->EnableExternalControlPhysics( AVP_PhysicsObjectHandler::ECREQ_GAMESCENE );
						pxPhysicsHandler->DisableExternalControlPhysics( AVP_PhysicsObjectHandler::ECREQ_GAMESCENE );
					}
				}
			}
		}
	}

	if ( iVersion >= CURRENT_CHUNK_VERSION )
	{
		xStream >> m_bLookAtPlayer;
		xStream >> m_uLookAtEntityGuid;
		xStream >> m_bExitOnTimeOut;
		xStream >> m_fTimeOutTimer;
		xStream >> m_fTimeOut;

		SetLookAtData();
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_LookAt::Deactivate( void )
{
	DestroyAllChildren();

	if ( GetMarineBrain()->IsActive() )
	{
		GetMarineBrain()->SetIsAllowedToExitGS2OnKnowledge(true);

		if (GetMarineBrain()->GetTacticalSystem()->IsPerformingAction())
		{
			GetMarineBrain()->GetTacticalSystem()->SetIsOnGameScene(true);
		}

		AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();

		if ( HasNextGuidControl() )
		{
			AVP_Actor_PhysicsObjectHandler* pxPhysicsHandler = pxOurActor->GetPhysicsObjectHandler();

			if ( pxPhysicsHandler )
			{
				pxPhysicsHandler->EnableExternalControlPhysics( AVP_PhysicsObjectHandler::ECREQ_GAMESCENE );
			}
		}
	}

	PARENT::Deactivate();
}

void AVP_Behaviour_GS2Node_Marine_LookAt::PostLoadInitialise()
{
	PARENT::PostLoadInitialise();

	// Unfortunately the anim state machine is not created until PLI time.
	// Tell The AI Anim State Machine that we aren't controlling the animation
	GetMarineBrain()->GetHumanActor()->GetAIAnimStateMachine()->RequestEndGamescene();
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_LookAt::PerFrameUpdate( const float fTimeslice )
{
	// If our time out has run out and some how are still in this behaviour the set finished and return.
	if ( m_bExitCalled )
	{
		SetFinished();
		return;
	}

	// Make sure we are always aiming in this behaviour
	if ( GetMarineBrain()->GetHumanActor()->IsAiming() )
	{
		GetMarineBrain()->GetHumanActor()->SetAiming( false );
	}

	AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity( m_uCurrentGS2NodeGuid );
	bool bAllowedToLeaveOnThreatInfo = true;

	if ( pxGS2Node )
	{
		bAllowedToLeaveOnThreatInfo = pxGS2Node->ShouldExitGamesceneOnKnowledge();
	}

	// If we have a threat then look at it otherwise look in the direction of the nodes orientation
	if ( GetMarineBrain()->DoesHaveThreat() && bAllowedToLeaveOnThreatInfo )
	{
		m_bExitCalled = true;
	}
	else if ( GetMarineBrain()->DoesHaveAlert() && bAllowedToLeaveOnThreatInfo )
	{
		m_bExitCalled = true;
	}
	else
	{
		LookAtEntity();
	}

		// Check for time if this nodes has the exit on time out condition set.
	if ( m_bExitOnTimeOut )
	{
		m_fTimeOutTimer += fTimeslice;

		if ( m_fTimeOutTimer >= m_fTimeOut )
		{
			m_bExitCalled = true;
			LeaveNode();
			return;
		}
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_LookAt::LookAtEntity( void )
{
	if ( !m_bLookAtActor )
	{
		AVP_ServerEntity_LookAt* pxLookAt = AVP_ServerEntity_LookAt::FindEntity( m_uLookAtEntityGuid );

		if ( pxLookAt )
		{
			GetMarineBrain()->AimAtPosition( pxLookAt->GetPosition() );
			GetMarineBrain()->SetLookAtPosition( pxLookAt->GetPosition() );
		}
	}
	else
	{
		AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( m_uLookAtEntityGuid );

		if ( pxActor )
		{
			Asura_Vector_3 xHeadPosition( Asura_Maths::ZeroVector3 );
			Asura_Vector_3 xChestPosition( Asura_Maths::ZeroVector3 );

			pxActor->GetHeadPosition( xHeadPosition );
			pxActor->GetChestPosition( xChestPosition );

			GetMarineBrain()->AimAtPosition( xChestPosition );
			GetMarineBrain()->SetLookAtPosition( xHeadPosition );
		}
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_LookAt::HandleParams( const Axon_BehaviourParams* pxParams )
{
	PARENT::HandleParams( pxParams );

	if ( pxParams->GetParamsType() == AVP_BP_TYPE_MARINE_LOOKAT )
	{
		const AVP_BehaviourParams_Marine_LookAt* pxMrnlookAtParams = static_cast< const AVP_BehaviourParams_Marine_LookAt* >( pxParams );

		m_bLookAtPlayer		= pxMrnlookAtParams->m_bLookAtPlayer;
		m_uLookAtEntityGuid	= pxMrnlookAtParams->m_uLookAtEntityGuid;
		m_bExitOnTimeOut	= pxMrnlookAtParams->m_bExitOnTimeOut;
		m_fTimeOut			= pxMrnlookAtParams->m_fTimeOut;
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
void AVP_Behaviour_GS2Node_Marine_LookAt::LeaveNode(void)
{
	m_uGS2LinkGuid = ASURA_GUID_UNREGISTERED;
	SetFinished();

	AVP_ServerNode_GS2Node* pxCurrentNode = AVP_ServerNode_GS2Node::FindEntity( m_uCurrentGS2NodeGuid );
	if ( !pxCurrentNode )
	{
		SetInError();
		return;
	}

	u_int uNumFreeNodes = 0;
	for ( u_int u = 0; u < pxCurrentNode->GetNumberLinks(); ++u )
	{
		const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxCurrentNode->GetLink( u ) );
		if ( !pxLink )
		{
			SetInError();
			return;
		}
		if ( pxLink->IsOccupied() )
		{
			continue;
		}

		const Asura_ServerNode_GS2Node* pxNextNode = Asura_ServerNode_GS2Node::FindEntity( pxLink->GetTargetGuid() );
		if ( !pxNextNode )
		{
			SetInError();
			return;
		}
		if ( pxNextNode->IsOccupied() || pxNextNode->HasPendingActors() )
		{
			continue;
		}

		++uNumFreeNodes;
	}
	if ( uNumFreeNodes == 0 )
	{
		return;
	}

	// Pick a free node at random.
	const u_int uRandomLink = Asura_Random::GetIntU( 0, uNumFreeNodes - 1 );
	uNumFreeNodes = 0;
	for ( u_int u = 0; u < pxCurrentNode->GetNumberLinks(); ++u )
	{
		const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxCurrentNode->GetLink( u ) );
		if ( !pxLink )
		{
			SetInError();
			return;
		}
		if ( pxLink->IsOccupied() )
		{
			continue;
		}

		const Asura_ServerNode_GS2Node* pxNextNode = Asura_ServerNode_GS2Node::FindEntity( pxLink->GetTargetGuid() );
		if ( !pxNextNode )
		{
			SetInError();
			return;
		}
		if ( pxNextNode->IsOccupied() || pxNextNode->HasPendingActors() )
		{
			continue;
		}

		if ( uNumFreeNodes == uRandomLink )
		{
			m_uGS2LinkGuid = pxLink->GetGuid();
			return;
		}
		++uNumFreeNodes;
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
bool AVP_Behaviour_GS2Node_Marine_LookAt::HandleMessage( const Asura_Message* pxMessage )
{
	switch (pxMessage->GetMsgID())
	{
		case ASURA_MSG_AXON_GS2_LEAVE_NODE:
		{
			LeaveNode();
			return true;
		}

		case AVP_MSGID_PERFORM_AI_OBJECTIVE_ACTION:
		{
			const AVP_Message_PerformObjectiveAction* pxMsgData = static_cast<const AVP_Message_PerformObjectiveAction*>(pxMessage->GetData());

			if (pxMsgData)
			{
				if (pxMsgData->GetActionGuid() != ASURA_GUID_UNREGISTERED)
				{
					GetMarineBrain()->GetHumanActor()->SetActionGamesceneGuid( pxMsgData->GetActionGuid() );
				}
			}
			return true;
		}

		default:
		{
			return PARENT::HandleMessage(pxMessage);
		}
	}
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
AVP_Brain_Actor_Marine* AVP_Behaviour_GS2Node_Marine_LookAt::GetMarineBrain( void ) const
{
	return static_cast< AVP_Brain_Actor_Marine* >( GetBrain() );
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Behaviour_GS2Node_Marine_LookAt::RenderDebugInfo( void )
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetDbgOutputPosX(), GetDbgOutputPosY(), 0.5f, 0xffffffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED, "AVP MARINE NODE LOOK AT" );

	GetMarineBrain()->GetDebugOutputPos().y += 10.0f;

	if ( m_bExitOnTimeOut )
	{
		float fPrintValue = m_fTimeOut - m_fTimeOutTimer;

		Asura_Font_System::Print( GetDbgOutputPosX(), GetDbgOutputPosY(), 0.5f, 0xffffffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "Timeout Left : %0.2f", fPrintValue );

		GetMarineBrain()->GetDebugOutputPos().y += 10.0f;
	}
}
#endif

#ifdef ASURA_TOOLS
TOOLS_DECLARE_SHARED_PARAM( bool,			0xc085d5e9, GS2LookAt_bExitOnTimeOut );			// Asura_GetHashID( "GS2LookAt_bExitOnTimeOut" )
TOOLS_DECLARE_SHARED_PARAM( float,			0x12049742, GS2LookAt_fTimeOut );				// Asura_GetHashID( "GS2LookAt_fTimeOut" )
TOOLS_DECLARE_SHARED_PARAM( bool,			0xb0e7cd2e, GS2LookAt_bLookAtPlayer );			// Asura_GetHashID( "GS2LookAt_bLookAtPlayer" )
TOOLS_DECLARE_SHARED_PARAM( Asura_Guid,		0x583c8ecc, GS2LookAt_uLookAtEntityGuid );		// Asura_GetHashID( "GS2LookAt_uLookAtEntityGuid" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String,	0x99821ded, GS2LookAt_sDummyString );			// Asura_GetHashID( "GS2LookAt_sDummyString" )
#endif

bool									AVP_BehaviourParams_Marine_LookAt::s_bTimeOutChanged		= false;
bool									AVP_BehaviourParams_Marine_LookAt::s_bLookAtDataChanged		= false;

//////////////////////////////////////////////////////////////////////////////
AVP_BehaviourParams_Marine_LookAt::AVP_BehaviourParams_Marine_LookAt( const u_int uBehaviourType )
: PARENT( AVP_BP_TYPE_MARINE_LOOKAT, uBehaviourType )
, m_bLookAtPlayer( false )
, m_uLookAtEntityGuid( ASURA_GUID_UNREGISTERED )
, m_bExitOnTimeOut( false )
, m_fTimeOut( 1.0f )
{

}

bool AVP_BehaviourParams_Marine_LookAt::IsBaseOrDerivativeOf( const u_int uParamsType ) const
{
	return ( uParamsType == AVP_BP_TYPE_MARINE_LOOKAT ) ? true : PARENT::IsBaseOrDerivativeOf( uParamsType );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_LookAt::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream( xStream );

	xStream >> m_bLookAtPlayer;
	xStream >> m_uLookAtEntityGuid;
	xStream >> m_bExitOnTimeOut;
	xStream >> m_fTimeOut;
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_LookAt::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << CURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_bLookAtPlayer;
	xStream << m_uLookAtEntityGuid;
	xStream << m_bExitOnTimeOut;
	xStream << m_fTimeOut;
}

#ifdef ASURA_TOOLS
//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_LookAt::DoAddOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset )
{
	AddOverlayControls( pxTabsDialog, xOffset );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_LookAt::AddOverlayControls(Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls( pxTabsDialog, xOffset );

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_SharedTickbox_Dialog( "Exit On Timeout", xOffset, &g_xParam_GS2LookAt_bExitOnTimeOut );
	pxDialog->SetHelpText("Select this to have the marine exit this node on a time out.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( m_bExitOnTimeOut )
	{
		pxDialog = new Asura_Numerical_Dialog( "TimeOut", xOffset, &g_xParam_GS2LookAt_fTimeOut, 0.5f, 360.0f , 0.1f, "%.1f" );
		pxDialog->SetHelpText("When this time is hit the behaviour will terminate.");
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	pxDialog = new Asura_SharedTickbox_Dialog( "Look At Player", xOffset, &g_xParam_GS2LookAt_bLookAtPlayer );
	pxDialog->SetHelpText("Select this to have the marine Look at the player while on this node.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( !m_bLookAtPlayer )
	{
		pxDialog = new Asura_Function_Dialog( "Choose Entity To look At", xOffset, SelectLookAt );
		((Asura_Function_Dialog*)pxDialog)->SetUseSharedDialogColour( true );
		pxDialog->SetHelpText("Allows selection of the Look At Entity to be used.");
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		if ( m_uLookAtEntityGuid != ASURA_GUID_UNREGISTERED )
		{
			ToolsAVP_Entity_Actor* pxActor = ( ToolsAVP_Entity_Actor* )ToolsAVP_Entity_PhysicalObject::FindEntity( m_uLookAtEntityGuid );
				
			if ( pxActor )
			{
				pxDialog->UpdateLabel( pxActor->GetName() );
			}
			else
			{
				ToolsAVP_Entity_LookAt* pxLookAt = ToolsAVP_Entity_LookAt::FindEntity( m_uLookAtEntityGuid );

				if ( pxLookAt )
				{
					pxDialog->UpdateLabel( pxLookAt->GetName() );
				}
			}
		}

		pxDialog = new Asura_Function_Dialog("Clear Look At Entity", xOffset, ClearLookAt);
		((Asura_Function_Dialog*)pxDialog)->SetUseSharedDialogColour( true );
		pxDialog->SetHelpText("Clears the selection of the Look At Entity being used.");
		pxTabsDialog->AddDialog(pxDialog);
	}
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_LookAt::SelectLookAt( void )
{
	ToolsAVP_LookAtChooseObject* pxChooseObject = new ToolsAVP_LookAtChooseObject( SetSelectedLookAt );
	Tools_Object_System::ChooseObject( "Pick the LookAt Entity.", pxChooseObject );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_LookAt::SetSelectedLookAt( Asura_Guid uGuid )
{
	g_xParam_GS2LookAt_uLookAtEntityGuid.SetValue( uGuid );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_LookAt::ClearLookAt( void )
{
	g_xParam_GS2LookAt_uLookAtEntityGuid.SetValue( ASURA_GUID_UNREGISTERED );
}

//////////////////////////////////////////////////////////////////////////////
void AVP_BehaviourParams_Marine_LookAt::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( s_bTimeOutChanged )
	{
		s_bTimeOutChanged = false;
		Tools_SaveAsura::SetEntityModified();
		GetToolsNode()->RequestRefreshOverlay();
	}

	if ( s_bLookAtDataChanged )
	{
		s_bLookAtDataChanged = false;
		Tools_SaveAsura::SetEntityModified();
		GetToolsNode()->RequestRefreshOverlay();
	}
}

//////////////////////////////////////////////////////////////////////////////
Axon_BehaviourParams* AVP_BehaviourParams_Marine_LookAt::Clone() const
{
	return new AVP_BehaviourParams_Marine_LookAt( *this );
}

//////////////////////////////////////////////////////////////////////////////
bool AVP_BehaviourParams_Marine_LookAt::GetParameter( const Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch ( uParamID )
	{
		case g_uHash_GS2LookAt_bExitOnTimeOut:
		{
			pxParam->SetBool( m_bExitOnTimeOut );
			return true;
		}break;

		case g_uHash_GS2LookAt_fTimeOut:
		{
			pxParam->SetFloat( m_fTimeOut );
			return true;

		}break;

		case g_uHash_GS2LookAt_bLookAtPlayer:
		{
			pxParam->SetBool( m_bLookAtPlayer );
			return true;
		}break;

		case g_uHash_GS2LookAt_uLookAtEntityGuid:
		{
			pxParam->SetUInt( ( u_int )m_uLookAtEntityGuid );
			return true;
		}break;
			
		default:
		{
			return PARENT::GetParameter( uParamID, pxParam );
		}break;
	}
}

//////////////////////////////////////////////////////////////////////////////
bool AVP_BehaviourParams_Marine_LookAt::SetParameter( const Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	switch ( uParamID )
	{
		case g_uHash_GS2LookAt_bExitOnTimeOut:
		{
			m_bExitOnTimeOut = pxParam->GetBool();
			s_bTimeOutChanged = true;
			return true;
		}break;

		case g_uHash_GS2LookAt_fTimeOut:
		{
			m_fTimeOut = pxParam->GetFloat();
			s_bTimeOutChanged = true;
			return true;

		}break;

		case g_uHash_GS2LookAt_bLookAtPlayer:
		{
			m_bLookAtPlayer = pxParam->GetBool();
			s_bLookAtDataChanged = true;
			return true;
		}break;

		case g_uHash_GS2LookAt_uLookAtEntityGuid:
		{
			m_uLookAtEntityGuid = ( Asura_Guid )pxParam->GetUInt();
			s_bLookAtDataChanged = true;
			return true;
		}break;
			
		default:
		{
			return PARENT::SetParameter( uParamID, pxParam );
		}break;
	}
}

#endif

/// END OF FILE