//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_GS2Node_Marine_LookAt.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_BEHAVIOUR_GS2NODE_MARINE_LOOKAT_H__
#define __AVP_BEHAVIOUR_GS2NODE_MARINE_LOOKAT_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// axon
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Interface.h"
#include "Behaviour/Axon_BehaviourParams.h"

// Avp Axon
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"

#ifdef ASURA_TOOLS
#include "ToolsAVP_ChooseObjects.h"
#endif

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_GS2Node_Marine_LookAt
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Behaviour_GS2Node_Marine_LookAt : public Axon_Behaviour_GS2Interface
{
	typedef Axon_Behaviour_GS2Interface PARENT;

public:
	AVP_Behaviour_GS2Node_Marine_LookAt( AVP_Brain_Actor_Marine* pxBrain );

	virtual bool       HasNextGuidControl();
	virtual Asura_Guid GetNextGamesceneEntityGuid();

protected:

	virtual bool			Activate( void );
	virtual void			Deactivate( void );
	virtual void			PostLoadInitialise();
	virtual void			PerFrameUpdate( const float fTimeslice );
	virtual void			HandleParams( const Axon_BehaviourParams* pxParams );
	virtual bool			HandleMessage( const Asura_Message* pxMessage );
	virtual void			WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual void			ReadFromChunkStream( Asura_Chunk_Stream& xStream );

	void LeaveNode( void );
	void LookAtEntity( void );
	void SetLookAtData( void );

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo( void );
#endif

private:

	enum { CURRENT_CHUNK_VERSION = 1 };

	AVP_Brain_Actor_Marine*	GetMarineBrain( void ) const;
#ifndef ASURA_NO_DEBUG_OUTPUT
	float					GetDbgOutputPosX() const		{ return GetMarineBrain()->GetDebugOutputPos().x; }
	float					GetDbgOutputPosY() const		{ return GetMarineBrain()->GetDebugOutputPos().y; }
#endif

	Asura_Guid		m_uCurrentGS2NodeGuid;
	Asura_Guid		m_uGS2LinkGuid;
	bool			m_bExitCalled;
	bool			m_bLookAtPlayer;
	Asura_Guid		m_uLookAtEntityGuid;
	bool			m_bExitOnTimeOut;
	float			m_fTimeOut;
	float			m_fTimeOutTimer;

	bool			m_bLookAtActor;

	static const u_int uCURRENT_CHUNK_VERSION = 0;
};

//////////////////////////////////////////////////////////////////////////////
//
// AVP_BehaviourParams_Marine_LookAt
//
//////////////////////////////////////////////////////////////////////////////

class AVP_BehaviourParams_Marine_LookAt : public Axon_BehaviourParams
{
	typedef Axon_BehaviourParams PARENT;

public:
	AVP_BehaviourParams_Marine_LookAt( const u_int uBehaviourType );

	virtual bool IsBaseOrDerivativeOf( const u_int uParamsType ) const;

#ifdef ASURA_TOOLS
	void DoAddOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset );
#endif

	bool			m_bLookAtPlayer;
	Asura_Guid		m_uLookAtEntityGuid;
	bool			m_bExitOnTimeOut;
	float			m_fTimeOut;

protected:
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

#ifdef ASURA_TOOLS
	virtual void AddOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset );
	virtual void UpdateWhileSelected();
	virtual bool GetParameter( const Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const;
	virtual bool SetParameter( const Asura_Hash_ID uParamID, const Tools_Parameter* pxParam );
	virtual Axon_BehaviourParams* Clone() const;
#endif

private:

	// Statics
	static bool									s_bTimeOutChanged;
	static bool									s_bLookAtDataChanged;

	static void SelectLookAt( void );
	static void SetSelectedLookAt( Asura_Guid uGuid );
	static void ClearLookAt( void );

	enum { CURRENT_CHUNK_VERSION = 1 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BEHAVIOUR_GS2NODE_MARINE_LOOKAT_H__
