#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_GS2Spline.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BEHAVIOUR_GS2SPLINE_MARINE_COMBAT_H__
#define __AVP_BEHAVIOUR_GS2SPLINE_MARINE_COMBAT_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// axon
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Interface.h"
#include "Behaviour/Axon_BehaviourParams.h"

// Avp Axon
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"

//------------------------------------------------------------------------------
//	AVP_Behaviour_GS2Spline_Marine_Combat
//------------------------------------------------------------------------------
class AVP_Behaviour_GS2Spline_Marine_Combat : public Axon_Behaviour_GS2Interface
{
	typedef Axon_Behaviour_GS2Interface PARENT;

public:
	AVP_Behaviour_GS2Spline_Marine_Combat( AVP_Brain_Actor_Marine* pxBrain );
	
	virtual bool       HasNextGuidControl();
	virtual Asura_Guid GetNextGamesceneEntityGuid();

	enum UpdateState
	{
		UNSET,
		LOOK_N_AIM_DOWN_ROUTE,
		LOOK_N_AIM_AT_THREAT,
		LOOK_N_AIM_AT_ALERT,
		FINISHING,
		MAX_STATES,
	};

protected:

	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual void HandleParams( const Axon_BehaviourParams* pxParams );
	virtual bool Activate();
	virtual void Deactivate();
	virtual void PerFrameUpdate( const float fTimeslice );

	#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo( void );
	#endif

	void LeaveNode();
	
	float GetDistanceToStop(void);
	Asura_Hash_ID GetMountGamesceneAnimBlendTemplateBlueprintHash(void);
	void SendMountGamesceneMessage();
	void SwitchToState( UpdateState eState );

private:

	enum { CURRENT_CHUNK_VERSION = 1 };

	Asura_Guid		m_uGS2NodeGuid;
	bool			m_bWaiting;
	UpdateState		m_eUpdateState;
	bool			m_bStoppingAtDestination;

	AVP_Brain_Actor_Marine*	GetMarineBrain( void ) const;

	#ifndef ASURA_NO_DEBUG_OUTPUT
	float					GetDbgOutputPosX() const		{ return GetMarineBrain()->GetDebugOutputPos().x; }
	float					GetDbgOutputPosY() const		{ return GetMarineBrain()->GetDebugOutputPos().y; }
	#endif

};

//------------------------------------------------------------------------------
//	AVP_BehaviourParams_GS2Spline_Marine_Combat
//------------------------------------------------------------------------------
class AVP_BehaviourParams_GS2Spline_Marine_Combat : public Axon_BehaviourParams
{
	typedef Axon_BehaviourParams PARENT;

public:

	AVP_BehaviourParams_GS2Spline_Marine_Combat( const u_int uBehaviourType );

	virtual bool IsBaseOrDerivativeOf( const u_int uParamsType ) const;

	#ifdef ASURA_TOOLS
	void DoAddOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset );
	#endif

protected:
	
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	#ifdef ASURA_TOOLS
	virtual void AddOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset );
	virtual Axon_BehaviourParams* Clone() const;
	virtual bool GetParameter( const Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const;
	virtual bool SetParameter( const Asura_Hash_ID uParamID, const Tools_Parameter* pxParam );
	void UpdateWhileSelected();
	#endif

private:

	enum { CURRENT_CHUNK_VERSION = 1 };
};
#endif // __AVP_BEHAVIOUR_GS2SPLINE_MARINE_COMBAT_H__
#endif // ASURA_USE_AXON