//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_Alert.cpp
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Alert.h"
// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#include "Debugging/Asura_DebugLineRender.h"
#endif
// axon
// AVP
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/MotionTrackerBlip/AVP_ServerEntity_MotionTrackerBlip.h"
// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Investigate.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Civilian.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Core/AVP_PositionEvaluation.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Tactical/AVP_Tactical_System.h"

//---------------------------------------------------------------------------------
//*********************************************************************************
//---------------------------------------------------------------------------------
// Class		: AVP_Behaviour_Marine_Alert
// Description	: 
//---------------------------------------------------------------------------------
//*********************************************************************************
//---------------------------------------------------------------------------------

Asura_Guid AVP_Behaviour_Marine_Alert::s_uLastAlertGuidForDialogue = ASURA_GUID_UNREGISTERED;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Alert :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_Alert::AVP_Behaviour_Marine_Alert(AVP_Brain_Actor_Marine* pxBrain)
: PARENT(pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_ALERT)
, m_eUpdateState(UNSET)
, m_pxMarineBrain(pxBrain)
, m_fAlertTimer(0.0f)
, m_xLastHeardPosition(Asura_Maths::ZeroVector3)
, m_bInvestigator(false)
, m_uSquadControllerGuid(ASURA_GUID_UNREGISTERED)
, m_bNewLookSet(false)
, m_uAlertGuid(ASURA_GUID_UNREGISTERED)
, m_fLookTimeOut( 0.0f )
, m_bIsInvestigator( false )
, m_uSoundType( 0 )
, m_uSquadGuid( ASURA_GUID_UNREGISTERED )
, m_bHasBeenAiming( false )
, m_bDialogueUsed( false )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Alert :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Alert::Activate()
{
	const bool bParentOk = PARENT::Activate();

	GetMarineBrain()->GetTacticalSystem()->UseAcurateDistanceCheck(false);
	GetMarineBrain()->GetTacticalSystem()->LetRouteSetLookAt(false);
	GetMarineBrain()->GetTacticalSystem()->StopMoving();

	m_uAlertGuid = GetMarineBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge().GetGuid();

	if ( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() != AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS )
	{
		GetMarineBrain()->GetKnowledgeSystem()->SetAlertLevel( AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS );
	}
	
	GetMarineBrain()->GetKnowledgeSystem()->ResetAlertTimer();

	GetActionToPerfromFromSoundType();

	m_xLastHeardPosition = GetMarineBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge().m_xPosition;
	
	if ( ( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS ) || 
		( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS ) )
	{
		AVP_ServerEntity_Actor* pxAlertActor = AVP_ServerEntity_Actor::FindEntity( m_uAlertGuid );
		
		if ( pxAlertActor )
		{
			float fDistanceSq = ( pxAlertActor->GetPosition() - GetMarineBrain()->GetHumanActor()->GetPosition() ).MagnitudeSquared();

			if ( fDistanceSq > 4.0f && GetUpdateState() == SCAN_AREA )
			{

				GetMarineBrain()->GetHumanActor()->PlayDialogue(AVP_ServerActor_Human::AVP_DLG_HUMAN_PASSIVE_TO_ALERT);
				pxAlertActor->GetChestPosition( m_xLastHeardPosition );
			}
		}
	}

	m_uSquadGuid = GetMarineBrain()->GetHumanActor()->GetInitialSquadGuid();

	GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( 2.0f );
	GetMarineBrain()->GetTacticalSystem()->SetIsAlert( true );

	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Alert :: Activate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Alert::Deactivate()
{
	if (GetMarineBrain()->IsActive())
	{
		GetMarineBrain()->GetTacticalSystem()->UseAcurateDistanceCheck(false);
		GetMarineBrain()->GetTacticalSystem()->LetRouteSetLookAt(false);
		GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier( 1.0f );
		GetMarineBrain()->GetHumanActor()->SetAiming( false );
		GetMarineBrain()->GetTacticalSystem()->SetIsAlert( false );
	}

	if ( m_bIsInvestigator && m_uSquadGuid != ASURA_GUID_UNREGISTERED )
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity( m_uSquadGuid );

		if ( pxSquad )
		{
			pxSquad->SetHasInvestigator( false );
		}
	}

	DestroyAllChildren();

	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Alert :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Alert::PerFrameUpdate(const float fTimeslice)
{
	// If we have a threat set the behaviour as finished
	if ( GetMarineBrain()->GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid() != ASURA_GUID_UNREGISTERED )
	{
		if ( GetUpdateState() == TURN_TO_FACE )
		{
			GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_SCARED_REACTION );
		}

		SetFinished();
		return;
	}

	if ( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS )
	{
		if ( AVP_Behaviour_Marine_Investigate::GetInvestigatorGuid() != GetMarineBrain()->GetOwnerGuid() )
		{
			SetFinished();
			return;
		}
	}

	if ( GetUpdateState() != INVESTIGATE )
	{
		if ( GetMarineBrain()->GetKnowledgeSystem()->DoesHaveReconKnowledge() )
		{
			GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_KEEP_FORMATION ); // RECON FAILURE
		}
	}

	// Increase the alert timer
	IncreaseAlertTimer(fTimeslice);

	switch ( GetUpdateState() )
	{
		//-----------------------
		case INVESTIGATE:
		{
			GetMarineBrain()->GetTacticalSystem()->SetForceAiming( true );
			m_bHasBeenAiming = true;

			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_INVESTIGATE ) )
			{
				if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_INVESTIGATE )->IsFinished() )
				{
					SetFinished();
					return;
				}
			}

		}break;
		//-----------------------
		case IGNORE_SOUND:
		{
			GetMarineBrain()->GetTacticalSystem()->SetForceAiming( m_bHasBeenAiming );

			Asura_Guid uAlertGuid = GetMarineBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge().GetGuid();

			if ( uAlertGuid != ASURA_GUID_UNREGISTERED )
			{
				GetMarineBrain()->GetKnowledgeSystem()->DeleteCurrentAlertKnowledge(uAlertGuid);
			}

			SetFinished();
			return;
		}
		//-----------------------
		case TURN_TO_FACE:
		case TURN_TO_FACE_MT_BLIP:
		{
			GetMarineBrain()->GetTacticalSystem()->SetForceAiming( true );
			m_bHasBeenAiming = true;

			SetLookDirection(fTimeslice);

			m_fAlertTimer += fTimeslice;

			if ( m_fAlertTimer > 4.0f )
			{
				SetFinished();
			}

		}break;
		//-----------------------
		case TURN_TO_FACE_POSITION:
		{
			GetMarineBrain()->GetTacticalSystem()->SetForceAiming( true );
			GetMarineBrain()->SetLookAtPosition( m_xLastHeardPosition );
			GetMarineBrain()->AimAtPosition( m_xLastHeardPosition );

			if ( !m_bDialogueUsed )
			{
				Asura_Vector_3 xDirToMine( m_xLastHeardPosition - GetMarineBrain()->GetHumanActor()->GetPosition() );
				float fDistanceToMine = xDirToMine.Normalise();

				if ( fDistanceToMine < 20.0f )
				{
					float fDot = xDirToMine * GetMarineBrain()->GetAimDirection();

					if ( fDot > 0.8f )
					{
						m_bDialogueUsed = true;
						GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_SCARED_REACTION );
					}
				}
			}

			m_fAlertTimer += fTimeslice;

			if ( m_fAlertTimer > 4.0f )
			{
				SetFinished();
			}

		}break;
		//-----------------------
		case SCAN_AREA:
		{
			GetMarineBrain()->GetTacticalSystem()->SetForceAiming( m_bHasBeenAiming );

			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );

			if ( m_uSoundType != GetMarineBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge().m_uTypeInformation )
			{
				GetActionToPerfromFromSoundType();
			}

		}break;
		//-----------------------
		case TAKE_COVER:
		{
			GetMarineBrain()->GetTacticalSystem()->SetForceAiming( true );
			m_bHasBeenAiming = true;

			// The trickiest one!
			// take cover but take cover from who as we don't always have where it.
			AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( m_uAlertGuid );
			u_short					uOurClassification	  = GetMarineBrain()->GetHumanActor()->GetClassification();

			if ( pxActor )
			{
				bool bUseAdditionalInfo = false;

				if ( pxActor )
				{
					if ( pxActor->GetClassification() == AVP_ENTITYCLASS_MARINE && uOurClassification == AVP_ENTITYCLASS_MARINE )
					{
						bUseAdditionalInfo = true;
					}
					else if ( pxActor->GetClassification() == AVP_ENTITYCLASS_ANDROID && uOurClassification == AVP_ENTITYCLASS_ANDROID )
					{
						bUseAdditionalInfo = true;
					}
				}

				if ( !pxActor || bUseAdditionalInfo )
				{
					Asura_Guid uAlertGuid = GetMarineBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge().GetGuid();

					if (uAlertGuid != ASURA_GUID_UNREGISTERED)
					{
						pxActor = AVP_ServerEntity_Actor::FindEntity( GetMarineBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge().m_uProjectData );

						if ( !pxActor )
						{
							SetUpdateState( IGNORE_SOUND );
							return;
						}
					}
				}
			}
			else
			{
				SetUpdateState( IGNORE_SOUND );
				return;
			}

			if ( !pxActor )
			{
				SetUpdateState( IGNORE_SOUND );
				return;
			}

			if ( AVP_Tactical_System::GetAllegiance( *(GetMarineBrain()->GetHumanActor()), *pxActor ) == AVP_ALLEGIANCE_ENEMY )
			{
				Asura_Guid uCombatZoneGuid = GetMarineBrain()->GetCombatZoneGuid();

				// An enemy! Woo we know what to do here!
				AVP_Navigation::NavData&			xNavData = GetMarineBrain()->GetNavData();
				AVP_Navigation::AVP_PosEvalParams	xPosEvalParams( GetMarineBrain()->GetOwnerGuid(), xNavData.m_usCurrentWPID, 30.0f, 0.0f, 0.0f, uCombatZoneGuid );
				AVP_Navigation::PosEvalFunctor		xPosEval( xNavData.m_usCurrentWPID, 25.0f, xPosEvalParams, ScoreWaypoints );
				AVP_Navigation::SearchWaypoints( AVP_Navigation::AVP_AI_PATHFINDER_FLAG_HUMANOID, &xPosEval );
				FindBestPosition( &xPosEval );
				SetUpdateState( MOVE_TO_COVER );
				m_fLookTimer = 0.0f;
			}
			else
			{
				SetUpdateState(IGNORE_SOUND);
			}
		}break;
		//-----------------------
		case MOVE_TO_COVER:
		{
			GetMarineBrain()->GetTacticalSystem()->SetForceAiming( true );
			m_bHasBeenAiming = true;

			if ( GetMarineBrain()->GetTacticalSystem()->HasArrivedAtDestination() || GetMarineBrain()->GetTacticalSystem()->DidRouteFindFail() )
			{
				m_fLookTimer += fTimeslice;

				if ( m_fLookTimer > 2.0f )
				{
					m_fLookTimer = 0.0f;
					SetUpdateState( SCAN_AREA );
				}
			}
			else
			{
				GetMarineBrain()->GetTacticalSystem()->SetMovementSpeed( AVP_Navigation::MM_MEDIUM );
			}

			GetMarineBrain()->SetLookAtPosition( m_xLastHeardPosition );
			GetMarineBrain()->AimAtPosition( m_xLastHeardPosition );
		}break;
		//-----------------------
		case REGOGNISE_FRIENDLIES_THREAT:
		{
			GetMarineBrain()->GetTacticalSystem()->SetForceAiming( true );
			m_bHasBeenAiming = true;

			const Axon_Knowledge&	xAlertKnowledge			= GetMarineBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge();
			u_short					usAlertClassification	= static_cast< u_short >( xAlertKnowledge.m_uSourceClassification );
			const Asura_Entity_Server*	pxThem				= Asura_Entity_Server::FindEntity( xAlertKnowledge.GetGuid() ) ;

			if ( pxThem && AVP_Tactical_System::GetAllegiance( *(GetMarineBrain()->GetHumanActor()), *pxThem ) == AVP_ALLEGIANCE_FRIEND)
			{
				AVP_ServerActor_Human* pxFriendActor = AVP_ServerActor_Human::FindEntity( xAlertKnowledge.GetGuid() );

				if ( pxFriendActor && ( usAlertClassification == AVP_ENTITYCLASS_ANDROID || usAlertClassification == AVP_ENTITYCLASS_MARINE ) )
				{
					AVP_Brain_Actor_Marine* pxMarineBrain = ( AVP_Brain_Actor_Marine* )pxFriendActor->GetActorBrain();

					if ( pxMarineBrain && pxMarineBrain->GetKnowledgeSystem() )
					{
						const Axon_Knowledge& xThreat = pxMarineBrain->GetKnowledgeSystem()->GetBestThreatKnowledge();
						GetMarineBrain()->GetKnowledgeSystem()->AddKnowledgeIfNeeded( xThreat );
					}
				}
				else if ( pxFriendActor && usAlertClassification == AVP_ENTITYCLASS_CIVILIAN )
				{
					AVP_Brain_Actor_Civilian* pxCivBrain = ( AVP_Brain_Actor_Civilian* )pxFriendActor->GetActorBrain();

					if ( pxCivBrain && pxCivBrain->GetKnowledgeSystem() )
					{
						const Axon_Knowledge& xThreat = pxCivBrain->GetKnowledgeSystem()->GetBestThreatKnowledge();
						GetMarineBrain()->GetKnowledgeSystem()->AddKnowledgeIfNeeded( xThreat );
					}
				}
			}
			else
			{
				SetUpdateState(TURN_TO_FACE);
			}
		}break;
		//-----------------------
		default:
		{

		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Alert :: FindBestPosition
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Alert::FindBestPosition( AVP_Navigation::PosEvalFunctor* pxPosEval )
{
	u_int uBestWaypointID = pxPosEval->GetBestWaypointID();

	if ( uBestWaypointID != ASURA_INVALID_WAYPOINT_ID )
	{
		Asura_AI_NavVertex*		pxWaypoint	= Asura_AI_Navigation_System::GetWaypoint( uBestWaypointID );
		AVP_Tactical_System*	pxTactical	= GetMarineBrain()->GetTacticalSystem();
		
		if ( pxWaypoint )
		{
			pxTactical->SetDestination( pxWaypoint->m_xPosition );
			pxTactical->SetAtDestinationRange( ONE_METER );
			pxTactical->SetMovementSpeed( AVP_Navigation::MM_MEDIUM );
			pxTactical->SetShouldStopAtDestination( true );
		}
		else
		{
			SetUpdateState( SCAN_AREA );
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Alert :: FindBestPosition
//---------------------------------------------------------------------------
u_int AVP_Behaviour_Marine_Alert::ScoreWaypoints( AVP_Navigation::AVP_PosEvalParams xPosEvalParams )
{
	AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity( xPosEvalParams.m_uActorGuid );

	u_int uScore = 0;

	// this should always be valid as he has just requested a position evaluation
	if ( !pxHuman ) { return uScore; }

	// Waypoint Proximity
	uScore += AVP_PositionEvaluation::GetProximityScore( pxHuman, xPosEvalParams.m_uTestWPID, xPosEvalParams.m_fEvaluationRange );

	// We don't want LOF to the Alert because we don't want the Alert to see threat to be able to see us
	uScore += AVP_PositionEvaluation::GetWaypointNoLOFToAlertScore( pxHuman, xPosEvalParams.m_uTestWPID, 40);

	// Waypoint Occupied by ally
	uScore += AVP_PositionEvaluation::GetHumanoidPositionOccupiedByAllyScore( pxHuman, xPosEvalParams.m_uTestWPID, 20 );

	// Waypoint Has a cover point that provides cover from primary threat
	uScore += AVP_PositionEvaluation::GetPositionProvidesCoverFromPrimaryThreat( pxHuman, xPosEvalParams.m_uTestWPID, 20 );
	
	if ( pxHuman->GetInitialSquadGuid() != ASURA_GUID_UNREGISTERED )
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity( pxHuman->GetInitialSquadGuid() );

		if ( ( !pxSquad ) || ( pxSquad->GetSquadLeaderGuid() == pxHuman->GetGuid() ) ) { return uScore; }

		// Waypoint Has a cover point that provides cover from primary threat
		uScore += AVP_PositionEvaluation::GetPositionCloseToSquadLeaderScore( pxHuman, xPosEvalParams.m_uTestWPID, 20);
	}

	return uScore;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Alert :: GetActionToPerfromFromSoundType
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Alert::GetActionToPerfromFromSoundType(void)
{
	const Axon_Knowledge&	xAlertKnowledge			= GetMarineBrain()->GetKnowledgeSystem()->GetBestAlertKnowledge();
	AVP_ServerActor_Human*	pxOurActor				= GetMarineBrain()->GetHumanActor();
	const Asura_Entity_Server*	pxThem				= Asura_Entity_Server::FindEntity( xAlertKnowledge.GetGuid() );

	m_uSoundType = xAlertKnowledge.m_uTypeInformation;

	switch (m_uSoundType)
	{
		//--------------------------------------------------------
		case AVP_SOUND_TYPE_SCREAM:
		{
			SetUpdateState( TAKE_COVER );
			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
		}break;
		//--------------------------------------------------------
		case AVP_SOUND_TYPE_PRED_ROAR:
		case AVP_SOUND_TYPE_ALIEN_ROAR:
		{
			m_uAlertGuid = xAlertKnowledge.GetGuid();
			SetUpdateState( TURN_TO_FACE );
			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
		}break;
		//--------------------------------------------------------
		case AVP_SOUND_TYPE_HUMAN_HURT:
		{
			AVP_ServerEntity_Actor* pxDamageActor = NULL;

			if ( xAlertKnowledge.GetGuid() == pxOurActor->GetGuid() )
			{
				pxDamageActor = AVP_ServerEntity_Actor::FindEntity( pxOurActor->GetGuidToLastDamageUs() );
			}
			else
			{
				AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity( xAlertKnowledge.GetGuid() );

				if ( pxHuman )
				{
					pxDamageActor = AVP_ServerEntity_Actor::FindEntity( pxHuman->GetGuidToLastDamageUs() );
				}
			}

			float fDistanceSq = 0.0f;

			if ( pxDamageActor )
			{
				fDistanceSq = ( pxDamageActor->GetPosition() - pxOurActor->GetPosition() ).MagnitudeSquared();
			}
			else
			{
				fDistanceSq = ( xAlertKnowledge.m_xPosition - pxOurActor->GetPosition() ).MagnitudeSquared();
			}

			float fSightRangeSq = GetMarineBrain()->GetKnowledgeSystem()->GetVisionRange();
			fSightRangeSq *= fSightRangeSq;

			if ( fDistanceSq < fSightRangeSq )
			{
				if ( pxDamageActor )
				{
					m_uAlertGuid = pxDamageActor->GetGuid();
					SetUpdateState( TURN_TO_FACE );
					GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
				}
				else
				{
					m_uAlertGuid = ASURA_GUID_UNREGISTERED;
					SetUpdateState( SCAN_AREA );
					GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
				}
			}
			else
			{
				if ( pxDamageActor )
				{
					m_uAlertGuid = pxDamageActor->GetGuid();
					SetUpdateState( TAKE_COVER );
					GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
				}
				else
				{
					m_uAlertGuid = ASURA_GUID_UNREGISTERED;
					SetUpdateState( SCAN_AREA );
					GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );
				}
			}
			
		}break;
		//--------------------------------------------------------
		case AVP_SOUND_TYPE_MOTION_TRACKER_BLIP:
		{
			m_uAlertGuid = xAlertKnowledge.GetGuid();
			SetUpdateState( TURN_TO_FACE_MT_BLIP );
			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
		}break;
		//--------------------------------------------------------
		case AXON_SOUND_TYPE_FOOTSTEP:
		case AVP_SOUND_TYPE_JUMP_LANDING:
		case AVP_SOUND_TYPE_MELEE_ATTACK:
		case AVP_SOUND_TYPE_RELOAD:
		{
			if ( pxThem && AVP_Tactical_System::GetAllegiance( *pxOurActor, *pxThem ) == AVP_ALLEGIANCE_ENEMY)
			{
				m_uAlertGuid = pxThem->GetGuid();
				SetUpdateState( TURN_TO_FACE );
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
			}
			else
			{
				SetUpdateState( IGNORE_SOUND );
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );
			}

		}break;
		//--------------------------------------------------------
		case AVP_SOUND_TYPE_MINE_TRIGGER:
		{
			SetUpdateState( TURN_TO_FACE_POSITION );
			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );

		}break;
		//--------------------------------------------------------
		case AXON_SOUND_TYPE_GUNFIRE:
		{
			if ( GetMarineBrain()->GetOwnerGuid() == xAlertKnowledge.GetGuid() )
			{
				SetUpdateState( IGNORE_SOUND );
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );
			}
			else
			{
				if ( pxThem && AVP_Tactical_System::GetAllegiance(*pxOurActor, *pxThem) == AVP_ALLEGIANCE_ENEMY)
				{
					SetUpdateState( TAKE_COVER );
					GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
				}
				else
				{
					SetUpdateState( REGOGNISE_FRIENDLIES_THREAT );
					GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
				}
			}
		}break;
		//--------------------------------------------------------
		case AXON_SOUND_TYPE_RICOCHET:
		case AXON_SOUND_TYPE_EXPLOSION:
		{
			GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_SCARED_REACTION );

			if ( pxThem && AVP_Tactical_System::GetAllegiance( *pxOurActor, *pxThem) == AVP_ALLEGIANCE_ENEMY)
			{
				SetUpdateState( TAKE_COVER );
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
			}
			else
			{
				SetUpdateState( IGNORE_SOUND );
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );
			}

		}break;
		//--------------------------------------------------------
		case AVP_SOUND_TYPE_DISTRACTION:
		{
			AVP_ServerEntity_Squad_Marine* pxSquad = GetMarineBrain()->GetMarineSquad();

			if ( pxSquad )
			{
				if ( AVP_Behaviour_Marine_Investigate::GetInvestigatorGuid() != ASURA_GUID_UNREGISTERED )
				{
					m_uAlertGuid = xAlertKnowledge.GetGuid();
					SetUpdateState( TURN_TO_FACE );
					return;
				}
				else
				{
					if ( GetMarineBrain()->GetOwnerGuid() == pxSquad->GetSquadLeaderGuid() )
					{
						pxSquad->ChooseNewSquadLeader( GetMarineBrain()->GetOwnerGuid() );
					}

					pxSquad->SetHasInvestigator( true );
					m_bIsInvestigator = true;
				}
			}

			if ( !GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_INVESTIGATE ) )
			{
				CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_INVESTIGATE );
			}

			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
			SetUpdateState( INVESTIGATE );

		}break;
		//--------------------------------------------------------
		default:
		{
			SetUpdateState( IGNORE_SOUND );
			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );

		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Alert :: SetLastHeardDirection
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Alert::SetLookDirection(const float fTimeslice)
{
	AVP_ServerActor_Human*	pxOurActor = GetMarineBrain()->GetHumanActor();

	switch ( GetUpdateState() )
	{
		case IGNORE_SOUND:
		{
			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );
			GetMarineBrain()->GetTacticalSystem()->SetForceAiming( false );
			SetFinished();
			return;
		}

		case TURN_TO_FACE_MT_BLIP:
		{
			AVP_ServerEntity_MotionTrackerBlip*		pxMTBlip = AVP_ServerEntity_MotionTrackerBlip::FindEntity(m_uAlertGuid);

			if ( pxMTBlip )
			{
				Asura_Vector_3				xChestPos	= Asura_Maths::ZeroVector3;
				pxOurActor->GetChestPosition(xChestPos);
				Asura_Vector_3				xDirection	= pxMTBlip->GetPosition() - xChestPos;
				Asura_Vector_3				xLookAt		= Asura_Maths::ZeroVector3;
				
				xDirection.Normalise();
				xLookAt = xChestPos + xDirection;
				GetMarineBrain()->AimAtPosition(xLookAt);

				GetMarineBrain()->GetTacticalSystem()->SetForceAiming( true );
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
			}
			else
			{
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );
				GetMarineBrain()->GetTacticalSystem()->SetForceAiming( false );
				SetFinished();
				return;
			}
		}break;

		case TURN_TO_FACE:
		{
			AVP_ServerEntity_Actor*		pxAttacker			= AVP_ServerEntity_Actor::FindEntity(m_uAlertGuid);

			if ( ( pxAttacker ) && ( pxAttacker->GetGuid() != pxOurActor->GetGuid() ) )
			{
				Asura_Vector_3				xDirection	= pxAttacker->GetPosition() - pxOurActor->GetPosition();
				Asura_Vector_3				xChestPos	= Asura_Maths::ZeroVector3;
				Asura_Vector_3				xLookAt		= Asura_Maths::ZeroVector3;
				pxOurActor->GetChestPosition(xChestPos);
				xDirection.Normalise();
				xLookAt = xChestPos + xDirection;
				GetMarineBrain()->AimAtPosition(xLookAt);

				GetMarineBrain()->GetTacticalSystem()->SetForceAiming( true );
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
			}
			else
			{
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );
				GetMarineBrain()->GetTacticalSystem()->SetForceAiming( false );
				SetFinished();
				return;
			}

		}break;

		case MOVE_TO_COVER:
		case SCAN_AREA:
		{
			GetMarineBrain()->GetTacticalSystem()->SetForceAiming( false );
			GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );

		}break;

		default:
		{
		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Alert :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT

static const char* s_aszAlertActionStrings[AVP_Behaviour_Marine_Alert::MAX_STATES] = 
{
	"ALERT - UNSET",
	"ALERT - IGNORE_SOUND",
	"ALERT - TURN_TO_FACE",
	"ALERT - TURN_TO_FACE_MT_BLIP",
	"ALERT - SCAN_AREA",
	"ALERT - TAKE_COVER",
	"ALERT - MOVE_TO_COVER",
	"ALERT - REGOGNISE_FRIENDLIES_THREAT",
};

void AVP_Behaviour_Marine_Alert::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		AVP_Brain_Actor_Marine::s_uTemporaryChildDebugColour,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszAlertActionStrings[m_eUpdateState]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_Alert.cpp'
