//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_AttackSelector.cpp
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_AttackSelector.h"
// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif
// axon
// AVP
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"

// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Core/BrainTypes/AVP_Brain_Squad_Marine.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"


//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_AttackSelector :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_AttackSelector::AVP_Behaviour_Marine_AttackSelector( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SELECTOR )
, m_eUpdateState( UNSET )
, m_pxMarineBrain( pxBrain )
, m_uWeaponType( AVP_WEAPON_BEHAVIOUR_TYPE_NONE )
, m_bIsAssaulter( false )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_AttackSelector :: Deconstructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_AttackSelector::~AVP_Behaviour_Marine_AttackSelector( void )
{
	
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_AttackSelector :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_AttackSelector::Activate( void )
{
	const bool bParentOk = PARENT::Activate();
	// Put Code Below This Comment

	m_uWeaponType = GetMarineBrain()->GetCurrentWeaponType();

	m_fLastHealth = GetOurActor()->GetHealth() / GetOurActor()->GetInitialHealth();

	// Put Code Above This Comment
	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_AttackSelector :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_AttackSelector::Deactivate( void )
{
	if ( m_bIsAssaulter )
	{
		AVP_ServerEntity_Squad_Marine* pxSquad = GetMarineBrain()->GetMarineSquad();

		if ( pxSquad )
		{
			AVP_Brain_Squad_Marine*	pxSquadBrain = pxSquad->GetMarineSquadBrain();

			if ( pxSquadBrain && pxSquadBrain->IsActive() )
			{
				pxSquadBrain->DealocateAssaulter();
			}
		}
	}

	if ( GetMarineBrain()->IsActive() )
	{
		GetOurActor()->SetFormationRole( AVP_ServerActor_Human::FORMATION_ROLE_NONE );
	}

	// Put Code Above This Comment
	DestroyAllChildren();
	PARENT::Deactivate();
}

DEBUGVAR float dbg_fPistolThreatRangeSq			= 10.0f * 10.0f;
DEBUGVAR float dbg_fPulseRifleThreatRangeSq		= 20.0f * 20.0f;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_AttackSelector :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_AttackSelector::PerFrameUpdate( const float fTimeslice )
{
	AVP_ServerEntity_Squad_Marine*	pxSquad						= GetMarineBrain()->GetMarineSquad();	// Verified by brain

	if ( !pxSquad )
	{
		SetFinished();
		return;
	}

	AVP_Brain_Squad_Marine*			pxSquadBrain				= pxSquad->GetMarineSquadBrain();		// Only exists if the squad exists
	u_int							uNumThreats					= pxSquadBrain->GetNumSquadThreats();
	u_int							uNumMembers					= pxSquad->GetNumSquadMembers();
	float							fCurrentHealth				= GetOurActor()->GetHealth();
	float							fInitialHealth				= GetOurActor()->GetInitialHealth();
	float							fHealth						= fCurrentHealth / fInitialHealth;
	float							fAvgThreatDistanceSq		= GetAvgThreatDistanceSq();
	float							fAssaultWeaponPercentage	= ((float) pxSquadBrain->GetNumMembersWithAssaultWeapons() / (float)pxSquad->GetNumSquadMembers() );

	if ( !pxSquadBrain )
	{
		SetFinished();
		return;
	}

	if ( pxSquad->GetSquadCombatState() == AVP_ServerEntity_Squad_Marine::SQUAD_COMBAT_STATE_STANDGROUND )
	{
		if ( GetMarineBrain()->GetCurrentWeaponType() == AVP_WEAPON_BEHAVIOUR_TYPE_SMARTGUN )
		{
			SwitchToState( ATTACK_SMARTGUN );
		}
		else
		{
			// STAND GROUND
			SwitchToState( ATTACK_STANDGROUND );
		}
	}
	else
	{
		switch ( GetWeaponType() )
		{
			//---------------------------------------------------------------------------

			case AVP_WEAPON_BEHAVIOUR_TYPE_PISTOL:
			{
				// We have a pistol - we want to fall back, or advance if we have overwhelming force
				if ( fHealth > 0.5f )
				{
					if ( uNumMembers > uNumThreats )
					{
						if ( fAssaultWeaponPercentage <= 0.2f && AreWeSuitableToBeTheAssaulter() )
						{
							SwitchToState( ATTACK_ADVANCE );
						}
						else
						{
							SwitchToState( ATTACK_SUPPORT );
						}
					}
					else
					{
						if ( fAvgThreatDistanceSq > dbg_fPistolThreatRangeSq )
						{
							SwitchToState( ATTACK_SUPPORT );
						}
						else
						{
							SwitchToState( ATTACK_FALLBACK );
						}
					}
				}
				else
				{
					if ( fAvgThreatDistanceSq > dbg_fPistolThreatRangeSq )
					{
						SwitchToState( ATTACK_SUPPORT );
					}
					else
					{
						SwitchToState( ATTACK_FALLBACK );
					}
				}

			}break;

			//---------------------------------------------------------------------------

			case AVP_WEAPON_BEHAVIOUR_TYPE_PULSERIFLE:
			{
				// we want to support unless no one else has assault weapons - then assault
				if ( fHealth > 0.45f && !m_bIsAssaulter )
				{
					if ( uNumMembers >= uNumThreats )
					{
						if ( ( ( fAssaultWeaponPercentage <= 0.2f && AreWeSuitableToBeTheAssaulter() ) || m_bIsAssaulter ) && pxSquad->GetNumSquadMembers() > 1 )
						{
							if ( !m_bIsAssaulter )
							{
								pxSquadBrain->AssignAssaulter();
								m_bIsAssaulter = true;
							}

							SwitchToState( ATTACK_ADVANCE );
						}
						else
						{
							if ( m_bIsAssaulter )
							{
								pxSquadBrain->DealocateAssaulter();
								m_bIsAssaulter = false;
							}

							SwitchToState( ATTACK_SUPPORT );
						}
					}
					else
					{
						if ( m_bIsAssaulter )
						{
							pxSquadBrain->DealocateAssaulter();
							m_bIsAssaulter = false;
						}

						if ( fAvgThreatDistanceSq > dbg_fPulseRifleThreatRangeSq  ||  pxSquad->GetNumSquadMembers() == 1 )
						{
							SwitchToState( ATTACK_SUPPORT );
						}
						else
						{
							SwitchToState( ATTACK_FALLBACK );
						}
					}
				}
				else
				{
					if ( m_bIsAssaulter && fHealth >= m_fLastHealth )
					{
						return;
					}

					if ( m_bIsAssaulter )
					{
						pxSquadBrain->DealocateAssaulter();
						m_bIsAssaulter = false;
					}

					if ( fAvgThreatDistanceSq > dbg_fPulseRifleThreatRangeSq  ||  pxSquad->GetNumSquadMembers() == 1 )
					{
						SwitchToState( ATTACK_SUPPORT );
					}
					else
					{
						SwitchToState( ATTACK_FALLBACK );
					}
				}

			}break;

			//---------------------------------------------------------------------------

			case AVP_WEAPON_BEHAVIOUR_TYPE_SHOTGUN:
			case AVP_WEAPON_BEHAVIOUR_TYPE_FLAMETHROWER:
			{
				AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();

				float fLastDamageTime = pxOurActor->GetTimeSinceLastDamage();

				if ( fLastDamageTime != 0.0f && fLastDamageTime > 5.0f )
				{
					SwitchToState( ATTACK_ADVANCE );
				}
				else
				{
					SwitchToState( ATTACK_FALLBACK );
				}

			}break;

			//---------------------------------------------------------------------------

			case AVP_WEAPON_BEHAVIOUR_TYPE_SMARTGUN:
			{
				SwitchToState( ATTACK_SMARTGUN );
			}break;

			//---------------------------------------------------------------------------

			case AVP_WEAPON_BEHAVIOUR_TYPE_SCOPED_RIFLE:
			{
				// do androids squads only contain scoped rifles?
				if ( fHealth > 0.5f )
				{
					SwitchToState( ATTACK_SUPPORT );
				}
				else
				{
					if ( fAvgThreatDistanceSq > dbg_fPulseRifleThreatRangeSq )
					{
						SwitchToState( ATTACK_SUPPORT );
					}
					else
					{
						SwitchToState( ATTACK_FALLBACK );
					}
				}

			}break;

			//---------------------------------------------------------------------------

			case AVP_WEAPON_BEHAVIOUR_TYPE_NONE:
			default:
			{
				SwitchToState( ATTACK_SUPPORT );

			}break;

			//---------------------------------------------------------------------------
		}
	}

	m_fLastHealth = GetOurActor()->GetHealth() / GetOurActor()->GetInitialHealth();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_AttackSelector :: AreWeSuitableToBeTheAssaulter
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_AttackSelector::AreWeSuitableToBeTheAssaulter()
{
	AVP_ServerActor_Human*			pxOurActor	= GetOurActor(); // verified by brain
	AVP_ServerEntity_Squad_Marine*	pxSquad		= GetMarineBrain()->GetMarineSquad();	// Verified by brain
	Asura_Vector_3					xThreatPos	= Asura_Maths::ZeroVector3; GetMarineBrain()->GetThreatPos( xThreatPos );

	float fOurDistanceToThreatSq = ( xThreatPos - pxOurActor->GetPosition() ).MagnitudeSquared();

	u_int uNumMembersMoreSuitable = 0;

	for ( u_int u = 0; u < pxSquad->GetNumSquadMembers(); ++u )
	{
		AVP_ServerActor_Human* pxMember = AVP_ServerActor_Human::FindEntity( pxSquad->GetMemberGuid( u ) );

		if ( pxMember && pxMember->IsEntityActive() && !pxMember->IsDead() && pxMember->GetFormationRole() != AVP_ServerActor_Human::FORMATION_ROLE_ASSAULT )
		{
			float fDistanceToThreatSq = (xThreatPos - pxMember->GetPosition() ).MagnitudeSquared();

			if ( fDistanceToThreatSq < fOurDistanceToThreatSq )
			{
				++uNumMembersMoreSuitable;
			}
		}
	}

 	if ( uNumMembersMoreSuitable == 0 )
	{
		return true;
	}
	else
	{
		return false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_AttackSelector :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_AttackSelector::SwitchToState( UpdateState eState )
{
	switch (GetUpdateState())
	{
		//---------------------------------------------------------------------------
		// ATTACK_ADVANCE
		//---------------------------------------------------------------------------
		case ATTACK_ADVANCE:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_ADVANCE );

			if ( pxBehaviour && eState == ATTACK_ADVANCE )
			{
				return;
			}
			else if ( pxBehaviour && eState != ATTACK_ADVANCE )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_ADVANCE );
			}

		}break;
		//---------------------------------------------------------------------------
		// ATTACK_FALLBACK
		//---------------------------------------------------------------------------
		case ATTACK_FALLBACK:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_FALLBACK );

			if ( pxBehaviour && eState == ATTACK_FALLBACK )
			{
				return;
			}
			else if ( pxBehaviour && eState != ATTACK_FALLBACK )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_FALLBACK );
			}

		}break;
		//---------------------------------------------------------------------------
		// ATTACK_SMARTGUN
		//---------------------------------------------------------------------------
		case ATTACK_SMARTGUN:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SMARTGUN );

			if ( pxBehaviour && eState == ATTACK_SMARTGUN )
			{
				return;
			}
			else if ( pxBehaviour && eState != ATTACK_SMARTGUN )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SMARTGUN );
			}

		}break;
		//---------------------------------------------------------------------------
		// ATTACK_SUPPORT
		//---------------------------------------------------------------------------
		case ATTACK_SUPPORT:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SUPPORT );

			if ( pxBehaviour && eState == ATTACK_SUPPORT )
			{
				return;
			}
			else if ( pxBehaviour && eState != ATTACK_SUPPORT )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SUPPORT );
			}

		}break;
		//---------------------------------------------------------------------------
		// ATTACK_STANDGROUND
		//---------------------------------------------------------------------------
		case ATTACK_STANDGROUND:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_STANDGROUND );

			if ( pxBehaviour && eState == ATTACK_STANDGROUND )
			{
				return;
			}
			else if ( pxBehaviour && eState != ATTACK_STANDGROUND )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_STANDGROUND );
			}

		}break;
		//---------------------------------------------------------------------------
		// ATTACK_TRAVELLING
		//---------------------------------------------------------------------------
		case ATTACK_TRAVELLING:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_TRAVELLING );

			if ( pxBehaviour && eState == ATTACK_TRAVELLING )
			{
				return;
			}
			else if ( pxBehaviour && eState != ATTACK_TRAVELLING )
			{
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_TRAVELLING );
			}

		}break;
		//---------------------------------------------------------------------------
		case UNSET:
		default:
		{

		}break;
		//---------------------------------------------------------------------------
	}

	AVP_ServerActor_Human*	pxOurActor = GetOurActor();

	SetUpdateState( eState );

	switch (GetUpdateState())
	{
		//---------------------------------------------------------------------------
		// ATTACK_ADVANCE
		//---------------------------------------------------------------------------
		case ATTACK_ADVANCE:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_ADVANCE );

			if ( !pxBehaviour )
			{
				CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_ADVANCE );
			}

			pxOurActor->SetFormationRole( AVP_ServerActor_Human::FORMATION_ROLE_ASSAULT );

		}break;
		//---------------------------------------------------------------------------
		// ATTACK_FALLBACK
		//---------------------------------------------------------------------------
		case ATTACK_FALLBACK:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_FALLBACK );

			if ( !pxBehaviour )
			{
				CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_FALLBACK );
			}

			pxOurActor->SetFormationRole( AVP_ServerActor_Human::FORMATION_ROLE_FALLBACK );

		}break;
		//---------------------------------------------------------------------------
		// ATTACK_SMARTGUN
		//---------------------------------------------------------------------------
		case ATTACK_SMARTGUN:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SMARTGUN );

			if ( !pxBehaviour )
			{
				CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SMARTGUN );
			}

			pxOurActor->SetFormationRole( AVP_ServerActor_Human::FORMATION_ROLE_SMARTGUN );

		}break;
		//---------------------------------------------------------------------------
		// ATTACK_SUPPORT
		//---------------------------------------------------------------------------
		case ATTACK_SUPPORT:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SUPPORT );

			if ( !pxBehaviour )
			{
				CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SUPPORT );
			}

			pxOurActor->SetFormationRole( AVP_ServerActor_Human::FORMATION_ROLE_SUPPORT );

		}break;
		//---------------------------------------------------------------------------
		// ATTACK_STANDGROUND
		//---------------------------------------------------------------------------
		case ATTACK_STANDGROUND:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_STANDGROUND );

			if ( !pxBehaviour )
			{
				CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_STANDGROUND );
			}

			pxOurActor->SetFormationRole( AVP_ServerActor_Human::FORMATION_ROLE_STANDGROUND );

		}break;

		//---------------------------------------------------------------------------
		// ATTACK_TRAVELLING
		//---------------------------------------------------------------------------
		case ATTACK_TRAVELLING:
		{
			Axon_Behaviour* pxBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_TRAVELLING );

			if ( !pxBehaviour )
			{
				CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_TRAVELLING );
			}

			pxOurActor->SetFormationRole( AVP_ServerActor_Human::FORMATION_ROLE_TRAVELLING );

		}break;
		//---------------------------------------------------------------------------
		case UNSET:
		default:
		{

		}
		//---------------------------------------------------------------------------
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_AttackSelector :: GetAvgThreatDistance
//---------------------------------------------------------------------------
float AVP_Behaviour_Marine_AttackSelector::GetAvgThreatDistanceSq( void )
{
	float fDistanceSq = ( GetOurActor()->GetPosition() - GetKnowledgeSystem()->GetBestThreatKnowledge().m_xPosition ).MagnitudeSquared();

	for ( u_int u = 0; u < GetKnowledgeSystem()->GetNumSecondaryThreats(); ++u )
	{
		fDistanceSq += ( GetOurActor()->GetPosition() - GetKnowledgeSystem()->GetSecondaryThreatKnowledge( u ).m_xPosition ).MagnitudeSquared();
	}

	if ( fDistanceSq > 0.0f )
	{
		fDistanceSq = fDistanceSq / (float) (GetKnowledgeSystem()->GetNumSecondaryThreats() + 1 );
	}

	return fDistanceSq;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_AttackSelector :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[ AVP_Behaviour_Marine_AttackSelector::MAX_STATES ] =
{
	"ATTACK_UNSET",
	"ATTACK_ADVANCE",
	"ATTACK_FALLBACK",
	"ATTACK_SUPPORT",
	"ATTACK_STANDGROUND",
	"ATTACK_SMARTGUN",
	"ATTACK_TRAVELLING",
};

void AVP_Behaviour_Marine_AttackSelector::RenderDebugInfo( void )
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0xffffff00,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_AttackSelector.cpp'
