//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_Attack_Support.cpp
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Attack_Support.h"
// asura
#include "Entity/Locations/OrientedNode/CoverpointNode/Asura_ServerNode_Coverpoint.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// axon
// AVP
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Tactical/AVP_Tactical_System.h"
#include "Core/AVP_PositionEvaluation.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"

#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_InCover.h"

DEBUGVAR float dbg_fLookTimeout = 0.1f;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_Attack_Support::AVP_Behaviour_Marine_Attack_Support( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SUPPORT )
, m_pxMarineBrain( pxBrain )
, m_eUpdateState( SUPPORT_CHOOSE )
, m_fTimer( 0.0f )
, m_uPrimaryThreatGuid( ASURA_GUID_UNREGISTERED )
, m_uBestCoverPoint( ASURA_GUID_UNREGISTERED )
, m_fAimTimer( dbg_fLookTimeout )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: Deconstructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_Attack_Support::~AVP_Behaviour_Marine_Attack_Support( void )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Attack_Support::Activate( void )
{
	const bool bParentOk = PARENT::Activate();
	// Put Code Below This Comment

	AVP_ServerEntity_Actor* pxThreat = AVP_ServerEntity_Actor::FindEntity( GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid() );

	if ( pxThreat && pxThreat->IsCloaked( true ) )
	{
		float fDistanceSq = ( pxThreat->GetPosition() - GetMarineBrain()->GetHumanActor()->GetPosition() ).MagnitudeSquared();

		if (fDistanceSq < 100.0f)
		{
			GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_SCARED_REACTION );
		}
	}
	else
	{
		GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_PROVIDING_COEVRING_FIRE );
	}

	m_uPrimaryThreatGuid = GetMarineBrain()->GetPrimaryThreatGuid();

	// Put Code Above This Comment
	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Attack_Support::Deactivate( void )
{
	// Put Code Above This Comment
	DestroyAllChildren();
	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Attack_Support::PerFrameUpdate( const float fTimeslice )
{
	switch ( GetUpdateState() )
	{
		//---------------------------------------------------------------------------

		case SUPPORT_CHOOSE:
		{
			if ( m_fTimer <= 0.0f )
			{
				AVP_Navigation::NavData&	xNavData		= GetMarineBrain()->GetNavData();
				Asura_Guid					uCombatZoneGuid	= GetMarineBrain()->GetCombatZoneGuid();

				AVP_Navigation::TestParams			xTestParams( AVP_Navigation::AVP_AI_PATHFINDER_FLAG_HUMANOID, xNavData.m_usCurrentZone );
				AVP_Navigation::AVP_PosEvalParams	xPosEvalParams( GetMarineBrain()->GetOwnerGuid(), xNavData.m_usCurrentWPID, 15.0f, 15.0f, 30.0f, uCombatZoneGuid );
				AVP_Navigation::PosEvalFunctor		pxPosEval( xNavData.m_usCurrentWPID, 15.0f, xPosEvalParams, EvaluationFunction );
				AVP_Navigation::SearchWaypoints( AVP_Navigation::AVP_AI_PATHFINDER_FLAG_HUMANOID, &pxPosEval );

				if ( FindBestWaypoint( &pxPosEval ) )
				{
					SwitchToState( SUPPORT_MOVE );

					GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_PROVIDING_COEVRING_FIRE );
				}
				else
				{
					m_fTimer = Asura_Random::GetFloat( 3.7f, 5.2f );
				}
			}
// 			else
// 			{
// 				CheckForCover();
// 			}

			m_fTimer -= fTimeslice;

		}break;

		//---------------------------------------------------------------------------

		case SUPPORT_MOVE:
		{
			if ( GetTacticalSystem()->HasArrivedAtDestination() )
			{
				m_fTimer -= fTimeslice;

				if ( m_fTimer <= 0.0f )
				{
					SwitchToState( SUPPORT_CHOOSE );
				}
			}

		}break;

		//---------------------------------------------------------------------------

		case SUPPORT_MOVE_TO_COVER:
		{
			AVP_ServerActor_Human*			pxOurActor		= GetMarineBrain()->GetHumanActor();
			Asura_ServerNode_Coverpoint*	pxCoverPoint	= Asura_ServerNode_Coverpoint::FindEntity( m_uBestCoverPoint );
			Asura_Vector_3					xThreatPos		= Asura_Maths::ZeroVector3;
			GetMarineBrain()->GetThreatPos( xThreatPos );

			if ( !pxCoverPoint || !pxCoverPoint->ProvidesCoverFrom( xThreatPos ) )
			{
				SwitchToState( SUPPORT_CHOOSE );
				return;
			}

			float fDistanceToCoverSq = ( pxCoverPoint->GetPosition() - pxOurActor->GetPosition() ).MagnitudeSquared();

			if ( fDistanceToCoverSq <= 3.0f )
			{
				SwitchToState( SUPPORT_COVER );
			}

			if ( GetTacticalSystem()->DidRouteFindFail() )
			{
				SwitchToState( SUPPORT_CHOOSE );
			}

			// Always Look At The Threat
			GetMarineBrain()->AimAtThreat();
			GetMarineBrain()->LookAtThreat();
			GetTacticalSystem()->SetMovementSpeed( AVP_Navigation::MM_MEDIUM );

		}break;

		//---------------------------------------------------------------------------

		case SUPPORT_COVER:
		{
			// If our cover is no longer valid then switch to position evaluation
			if (m_uBestCoverPoint == ASURA_GUID_UNREGISTERED)
			{
				SwitchToState( SUPPORT_CHOOSE );
				return;
			}

			// Check that the cover behaviour hasn't finished
			Axon_Behaviour* pxCoverBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_INCOVER );
			if ( ( pxCoverBehaviour && pxCoverBehaviour->IsFinished() ) || ( !pxCoverBehaviour ) )
			{
				SwitchToState( SUPPORT_CHOOSE );
				return;
			}

			Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( m_uBestCoverPoint );
			Asura_Vector_3					xThreatPos		= Asura_Maths::ZeroVector3;
			GetMarineBrain()->GetThreatPos( xThreatPos );

			// If our Cover no longer provides cover then switch to position evaluation
			if ( pxCoverPoint )
			{
				Asura_Vector_3 xDirToThreat = ( xThreatPos - pxCoverPoint->GetPosition() );
				float fDistanceSq = xDirToThreat.MagnitudeSquared();

				if ( fDistanceSq < 64.0f )
				{
					SwitchToState( SUPPORT_CHOOSE );
					return;
				}

				if ( !pxCoverPoint->ProvidesCoverFrom( xThreatPos ) )
				{
					SwitchToState( SUPPORT_CHOOSE );
					return;
				}
			}

		}break;

		//---------------------------------------------------------------------------

		case UNSET:
		default:
		{
		}break;
	}
	
	// Choose wehere to look here
	GetMarineBrain()->AimAtThreat();
	GetMarineBrain()->LookAtThreat();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: CheckForCover
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Attack_Support::LookInThreatsArea(const float fTimeslice)
{
	if ( m_fAimTimer >= dbg_fLookTimeout )
	{
		if (GetTacticalSystem() && GetKnowledgeSystem())
		{
			Asura_Vector_3 xThreatPos( Asura_Maths::ZeroVector3 );
			GetMarineBrain()->GetThreatPos( xThreatPos );

			const Asura_Vector_3& xOurPos = GetOurActor()->GetPosition();

			float fDistance = ( xOurPos - xThreatPos ).Magnitude();
			float fAimOffsetXZ = fDistance * 0.04f;
			float fAimOffsety = fDistance * 0.04f;

#if 0
			Asura_Vector_3 xThreatPos( Asura_Maths::ZeroVector3 );

			GetMarineBrain()->GetThreatPos( xThreatPos );
#endif
			xThreatPos.x += Asura_Random::GetFloat( -fAimOffsetXZ, fAimOffsetXZ );
			xThreatPos.y += Asura_Random::GetFloat( -fAimOffsety, fAimOffsety );
			xThreatPos.z += Asura_Random::GetFloat( -fAimOffsetXZ, fAimOffsetXZ );

			GetMarineBrain()->AimAtPosition( xThreatPos );

		}
	}

	m_fAimTimer -= fTimeslice;

	if ( m_fAimTimer <= 0.0f )
	{
		m_fAimTimer = Asura_Random::GetFloat(0.3f, 0.7f);
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: CheckForCover
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Attack_Support::CheckForCover()
{
	AVP_ServerActor_Human*				pxOurActor			= GetOurActor(); // Validated every frame by brain
	AVP_ServerEntity_GamePlayVolume*	pxOurVolume			= pxOurActor->GetCurrentGamePlayVolume();
	Asura_Vector_3						xThreatPos			= Asura_Maths::ZeroVector3; GetMarineBrain()->GetThreatPos( xThreatPos );
	float								fBestDistance		= Asura_Maths::LargeFloat;
	Asura_ServerNode_Coverpoint*		pxBestCoverPoint	= NULL;
	
	if ( !pxOurVolume ) { return; }

	for ( u_int u = 0; u < pxOurVolume->GetNumCoverPointGuids(); ++u )
	{
		Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( pxOurVolume->GetCoverPointGuid( u ) );
		if ( !pxCoverPoint || pxCoverPoint->IsOccupied() || !pxCoverPoint->IsEntityActive() ) { continue; }

		Asura_Vector_3 xOurDirectionToCoverPoint( pxCoverPoint->GetPosition() - pxOurActor->GetPosition() );
		float fDistanceFromUsSq = xOurDirectionToCoverPoint.MagnitudeSquared();
		if ( fDistanceFromUsSq < 64.0f ) { continue; } // 8 meters

		Asura_Vector_3 xThreatDirectionToCoverPoint( pxCoverPoint->GetPosition() - xThreatPos );
		float fDistanceFromThreatSq = xThreatDirectionToCoverPoint.MagnitudeSquared();
		if ( fDistanceFromThreatSq < 25.0f ) { continue; } // 5 meters

		if ( !pxCoverPoint->ProvidesCoverFrom( xThreatPos) ) { continue; }

		float fDot = xOurDirectionToCoverPoint * xThreatDirectionToCoverPoint;

		if (fDot > 0) { continue; }

		if ( fDistanceFromUsSq < fBestDistance )
		{
			fBestDistance = fDistanceFromUsSq;
			pxBestCoverPoint = pxCoverPoint;
		}
	}


	if ( pxBestCoverPoint )
	{
		m_uBestCoverPoint = pxBestCoverPoint->GetGuid();
		GetTacticalSystem()->SetDestination( pxBestCoverPoint->GetPosition() );
		GetTacticalSystem()->LetRouteSetLookAt( false );
		GetTacticalSystem()->SetAtDestinationRange( ONE_METER );
		GetTacticalSystem()->SetMovementSpeed( AVP_Navigation::MM_MEDIUM );
		GetTacticalSystem()->OverrideSlowDownForThisDestination();
		SwitchToState( SUPPORT_MOVE_TO_COVER );
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: FindBestWaypoint
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Attack_Support::FindBestWaypoint( AVP_Navigation::PosEvalFunctor* pxPosEval )
{
	u_int uBestWaypointID = pxPosEval->GetBestWaypointID();

	if ( uBestWaypointID != ASURA_INVALID_WAYPOINT_ID )
	{
		Asura_AI_NavVertex*	pxWaypoint	= Asura_AI_Navigation_System::GetWaypoint( uBestWaypointID );
		AVP_Tactical_System* pxTactical	= GetMarineBrain()->GetTacticalSystem();

		if ( pxWaypoint )
		{
			if ( DoesWaypointContainCoverAndItsNotOccupied( uBestWaypointID ) )
			{
				if ( m_uBestCoverPoint != ASURA_GUID_UNREGISTERED )
				{
					Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( m_uBestCoverPoint );
					if ( pxCoverPoint )
					{
						pxTactical->SetDestination( pxCoverPoint->GetPosition() );
						pxTactical->SetAtDestinationRange( ONE_METER );
						pxTactical->SetMovementSpeed( AVP_Navigation::MM_MEDIUM );
						pxTactical->LetRouteSetLookAt( false );
						pxTactical->OverrideSlowDownForThisDestination();
						SwitchToState( SUPPORT_MOVE_TO_COVER );
						return false;
					}
				}
			}

			float fDistanceSq = ( GetOurActor()->GetPosition() - pxWaypoint->m_xPosition ).MagnitudeSquared();

			if ( fDistanceSq > 9.0f )
			{
				pxTactical->SetDestination( pxWaypoint->m_xPosition );
				pxTactical->SetAtDestinationRange( ONE_METER );
				pxTactical->SetMovementSpeed( AVP_Navigation::MM_SLOW );
				pxTactical->LetRouteSetLookAt( false );
				return true;
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: DoesWaypointContainCoverAndItsNotOccupied
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Attack_Support::DoesWaypointContainCoverAndItsNotOccupied( u_int uWaypointID )
{
	// Ensure we have a threat
	if ( !GetMarineBrain()->DoesHaveThreat() ) { return false; }

	// Ensure Our Waypoint Is Valid
	Asura_AI_NavVertex*					pxWaypoint	= Asura_AI_Navigation_System::GetWaypoint( uWaypointID );
	if ( !pxWaypoint ) { return false; }

	// Ensure Our Volume Is Valid
	AVP_ServerEntity_GamePlayVolume*	pxVolume	= AVP_ServerEntity_GamePlayVolume::GetVolumeThatContainsPosition( pxWaypoint->GetPosition() );
	if ( ( !pxVolume ) || ( pxVolume->GetNumCoverPointGuids() == 0 ) ) { return false; }

	Asura_Vector_3 xThreatPos = Asura_Maths::ZeroVector3;
	GetMarineBrain()->GetThreatPos( xThreatPos );

	// Reset the best cover point
	m_uBestCoverPoint = ASURA_GUID_UNREGISTERED;

	// Iterate around all cover points in the waypoints gameplay volume
	for ( u_int u = 0; u < pxVolume->GetNumCoverPointGuids(); ++u )
	{
		Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( pxVolume->GetCoverPointGuid( u ) );
		if ( !pxCoverPoint || !pxCoverPoint->IsEntityActive() || pxCoverPoint->IsOccupied() ) { continue; }

		// is the cover point in our waypoint
		if ( Asura_AI_Navigation_System::IsPositionInWaypoint( pxCoverPoint->GetPosition(), uWaypointID ) )
		{
			Asura_Vector_3 xDirToThreat = ( xThreatPos - pxCoverPoint->GetPosition() );
			float fDistanceSq = xDirToThreat.MagnitudeSquared();

			if ( fDistanceSq < 64.0f )
			{
				continue;
			}

			// Does it provide cover from our threat?
			if ( pxCoverPoint->ProvidesCoverFrom( xThreatPos ) )
			{
				// wow we have one!
				m_uBestCoverPoint = pxCoverPoint->GetGuid();
				break;
			}
		}
	}

	if ( m_uBestCoverPoint != ASURA_GUID_UNREGISTERED )
	{
		return true;
	}
	else
	{
		return false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: GetAttackingScoreForWaypoint
//---------------------------------------------------------------------------
u_int AVP_Behaviour_Marine_Attack_Support::EvaluationFunction( AVP_Navigation::AVP_PosEvalParams xPosEvalParams )
{
	AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity( xPosEvalParams.m_uActorGuid );

	u_int uScore = 0;

	// this should always be valid as he has just requested a position evaluation
	if ( !pxHuman ) { return uScore; }

	// Waypoint Proximity
	//uScore += AVP_PositionEvaluation::GetProximityScore( pxHuman, xPosEvalParams.m_uTestWPID, xPosEvalParams.m_fEvaluationRange * 2.0f );

	AVP_Brain_Actor_Marine* pxBrain = (AVP_Brain_Actor_Marine*)pxHuman->GetActorBrain();

	if ( pxBrain )
	{
		if ( !pxBrain->GetKnowledgeSystem()->ShootAtScriptedTarget() || ( pxBrain->GetKnowledgeSystem()->ShootAtScriptedTarget() && !pxBrain->GetKnowledgeSystem()->ScriptedTargetIsShootAtPoint() ) )
		{
			// Line Of Fire To The Primary Threat
			uScore += AVP_PositionEvaluation::GetWaypointLOFtoPrimaryThreatScore( pxHuman, xPosEvalParams.m_uTestWPID, 40 );
		}
		else
		{
			uScore += AVP_PositionEvaluation::GetWaypointLOFToScriptedTarget( pxHuman, xPosEvalParams.m_uTestWPID, 40 );
		}
	}

	// Preferred Fighting Range
	uScore += AVP_PositionEvaluation::GetPreferredFightingRangeScore( pxHuman, xPosEvalParams.m_uTestWPID, xPosEvalParams.m_fMinFightingRange, xPosEvalParams.m_fMaxFightingRange, 40 );

	// Waypoint Occupied by ally
	uScore += AVP_PositionEvaluation::GetHumanoidPositionOccupiedByAllyScore( pxHuman, xPosEvalParams.m_uTestWPID, 30 );

	// Waypoint Gives no LOF from Secondary Threats
	uScore += AVP_PositionEvaluation::GetHumanoidPositionGivesCoverFromSecondaryThreatsScore( pxHuman, xPosEvalParams.m_uTestWPID, 30 );

	// Waypoint Has a cover point that provides cover from primary threat
	uScore += AVP_PositionEvaluation::GetPositionProvidesCoverFromPrimaryThreat( pxHuman, xPosEvalParams.m_uTestWPID, 60 );

	return uScore;

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Attack_Support::SwitchToState( UpdateState eState )
{
	switch (GetUpdateState())
	{
		//---------------------------------------------------------------------------
		case SUPPORT_MOVE_TO_COVER:
		{
			if (eState != SUPPORT_COVER)
			{
				Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( m_uBestCoverPoint );
				if ( pxCoverPoint )
				{
					pxCoverPoint->SetIsOccupied( false );
				}
			}
		}break;
		//---------------------------------------------------------------------------
		case SUPPORT_COVER:
		{
			DestroyChildWithType(AVP_BEHAVIOUR_TYPE_MARINE_INCOVER);
			
			m_uBestCoverPoint = ASURA_GUID_UNREGISTERED;

			Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( m_uBestCoverPoint );
			if ( pxCoverPoint )
			{
				pxCoverPoint->SetIsOccupied( false );
			}

		}break;
		//---------------------------------------------------------------------------
		case UNSET:
		default:
		{

		}
	}

	SetUpdateState( eState );

	switch (GetUpdateState())
	{
		//---------------------------------------------------------------------------
		case SUPPORT_MOVE_TO_COVER:
		{
			// Create the cover behaviour and set the cover point
			Asura_ServerNode_Coverpoint* pxCoverPoint = Asura_ServerNode_Coverpoint::FindEntity( m_uBestCoverPoint );
			if ( pxCoverPoint )
			{
				pxCoverPoint->SetIsOccupied( true );
			}

		}break;
		//---------------------------------------------------------------------------
		case SUPPORT_COVER:
		{
			// Create the cover behaviour and set the cover point
			AVP_Behaviour_Marine_InCover* pxCoverBehaviour = ( AVP_Behaviour_Marine_InCover* )CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_INCOVER );

			if ( pxCoverBehaviour )
			{
				pxCoverBehaviour->SetCoverPoint( m_uBestCoverPoint );
			}

		}break;
		//---------------------------------------------------------------------------
		case SUPPORT_CHOOSE:
		{
			m_fTimer = 0.0f;
		}break;
		//---------------------------------------------------------------------------
		case SUPPORT_MOVE:
		{
			m_fTimer = Asura_Random::GetFloat( 3.7f, 5.2f );
		}break;
		//---------------------------------------------------------------------------
		case UNSET:
		default:
		{
			
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Attack_Support :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[ AVP_Behaviour_Marine_Attack_Support::MAX_STATES ] =
{
	"ATTACK_SUPPORT - UNSET",
	"ATTACK_SUPPORT - CHOOSE",
	"ATTACK_SUPPORT - MOVE",
	"ATTACK_SUPPORT - MOVE TO COVER",
	"ATTACK_SUPPORT - COVER",
};

void AVP_Behaviour_Marine_Attack_Support::RenderDebugInfo( void )
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0XFFFF00FF,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_Attack_Support.cpp'
