//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_HandleObjective.h
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// BEGIN FILE WRAPPER ///////////////////////////////////////
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_BEHAVIOUR_MARINE_HandleObjective_H__
#define __AVP_BEHAVIOUR_MARINE_HandleObjective_H__ 1
// END FILE WRAPPER ///////////////////////////////////////////

// INCLUDES ///////////////////////////////////////////////////////////
//AXON
#include "Behaviour/Axon_Project_Behaviour.h"

// AVP
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"

// FORWARD DECLARATIONS ///////////////////////////////////
class AVP_Tactical_System;
class AVP_Knowledge_System_Actor;

//---------------------------------------------------------------------------
// Class		: AVP_Behaviour_Marine_HandleObjective
// Description	: 
//---------------------------------------------------------------------------
class AVP_Behaviour_Marine_HandleObjective : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:

	// InCover Update State Enum
	enum UpdateState
	{
		UNSET,
		MOVING_TO_OBJECTIVE,
		WAIT_TO_PERFROM_ACTION,
		MOVE_TO_ACTION,
		IN_MOVE,
		PERFROM_ACTION,
		ACTION_COMPLETE,
		WAIT_FOR_PLAYER,
		WAIT_FOR_OBJECTIVE_COMPLETE,
		FORMATION_ADVANCE,
		FORMATION_RETREAT,
		CORPSE_REACTION,
		RESTART,
		MAX_STATES
	};

	enum Posiitons
	{
		LEAD,
		REAR_GUARD,
		MIDDLE_MAN,
		MAX_POSITIONS
	};

	// Construction / Deconstruction
	AVP_Behaviour_Marine_HandleObjective( AVP_Brain_Actor_Marine* pxBrain );
	virtual ~AVP_Behaviour_Marine_HandleObjective( void );

	// Public Member Virtual Functions...
	virtual bool 					Activate( void );
	virtual void 					Deactivate( void );
	virtual void					PerFrameUpdate( const float fTimeslice );
	virtual bool					HandleMessage( const Asura_Message* pxMessage );
	#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void					RenderDebugInfo( void );
	#endif

protected:

	// Public Member Data Access...
	UpdateState						GetUpdateState( void ) const			{ return m_eUpdateState; }
	AVP_Brain_Actor_Marine*			GetMarineBrain( void )					{ return m_pxMarineBrain; }
	AVP_Tactical_System*			GetTacticalSystem( void )				{ return m_pxMarineBrain->GetTacticalSystem(); }
	AVP_Knowledge_System_Actor*		GetKnowledgeSystem( void )				{ return m_pxMarineBrain->GetKnowledgeSystem(); }
	AVP_ServerActor_Human*			GetOurActor( void )						{ return m_pxMarineBrain->GetHumanActor(); }
	AVP_ServerEntity_Squad_Marine*	GetSquad( void )						{ return AVP_ServerEntity_Squad_Marine::FindEntity( GetOurActor()->GetInitialSquadGuid() ); }
	bool							DoesHaveThreat( void )					{ return m_pxMarineBrain->DoesHaveThreat(); }
	bool							DoesHaveAlert( void )					{ return m_pxMarineBrain->DoesHaveAlert(); }

	void							SetUpdateState( UpdateState eState )	{ m_eUpdateState = eState; }

	// Public Member Functions...
	void							SwitchToState( UpdateState eState );
	bool							ValidateObjective( void );
	void							CheckObjectiveAction( void );
	void							SetRouteToObjective( bool bIgnoreSettingRouteLookAt = false, bool bSetOverrideMoverDelay = false );
	void							RequestAction( void );
	void							SetRouteToActionGameScene( void );
	void							SendMountGamesceneMessage( void );
	Asura_Hash_ID					GetMountGamesceneAnimBlendTemplateBlueprintHash( void );
	void							CheckForCompletion( void );
	void							CheckForEarlyOut( void );
	void							CheckForAimingAtThreat( void );
	bool							WaitForPlayer( void  );
	void							FormationPositionEvaluation( const float fTimeslice );
	AVP_ServerActor_Human*			GetSquadLeader( void );
	void							FindBestWaypoint( AVP_Navigation::PosEvalFunctor* pxPosEval );
	static u_int					GetFormationScoreForWaypoint( AVP_Navigation::AVP_PosEvalParams xPosEvalParams );
	AVP_ServerActor_Human*			GetPlayer( void );
	AVP_ServerActor_Human*			GetClosestSquadMemberToPlayer( void );
	void							ScanArea( const float fTimeslice );
	float							GetDistanceToStop( void );

	bool							CheckForCorpseResponse( const float fTimeSlice );

	bool IsLeadMarine( void );
	bool IsRearGuard( void );
	void ScanAreaInFront( const float fTimeslice );
	void ScanAreaBehind( const float fTimeslice );
	void ScanAnyLookAtNodes( const float fTimeslice );

	void TestForPause( void );

private:

	// Private Member Variables...
	AVP_Brain_Actor_Marine*			m_pxMarineBrain;
	UpdateState						m_eUpdateState;
	Asura_Guid						m_uObjectiveGuid;
	Asura_Guid						m_uActionGuid;
	bool							m_bHasStopped;
	float							m_fWaitForPlayerDistance;
	float							m_fPosEvalUpdateTimer;
	bool							m_bIsOwnActorsObjective;
	bool							m_bPlayerArrived;
	Asura_Vector_3					m_xPosOnRoute;
	Asura_Guid						m_uLeadMarine;
	u_int							m_uPosition;
	u_int							m_uRearGuardCounter;
	float							m_fLookTimeOut;
	bool							m_bDirectionLeft;
	bool							m_bOverrideMovementMode;
	float							m_fScanTimer;
	float							m_fCorpseReactionTimer;
	float							m_fCorpseCheckTimer;

	enum { CURRENT_CHUNK_VERSION = 1 };
};

#endif //__AVP_BEHAVIOUR_MARINE_HandleObjective_H__

/// End Of File 'AVP_Behaviour_Marine_HandleObjective.h'
