//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_HideFromCloakedPredator.h
// Created		: 02 / 07 / 2008
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// BEGIN FILE WRAPPER ///////////////////////////////////////
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_BEHAVIOUR_MARINE_HIDEFROMCLOAKEDPREDATOR_H__
#define __AVP_BEHAVIOUR_MARINE_HIDEFROMCLOAKEDPREDATOR_H__ 1
// END FILE WRAPPER ///////////////////////////////////////////

// INCLUDES ///////////////////////////////////////////////////////////
#include "Behaviour/Axon_Project_Behaviour.h"

#include "Core/AVP_PositionEvaluation.h"

#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"


// FORWARD DECLARATIONS ///////////////////////////////////
class AVP_Brain_Actor_Marine;
class AVP_ServerActor_Human;
class AVP_ServerEntity_Squad_Marine;
class AVP_Tactical_System;
class AVP_Knowledge_System_Actor;

//---------------------------------------------------------------------------
// Class		: AVP_Behaviour_Marine_HideFromCloakedPredator
// Description	: 
//---------------------------------------------------------------------------
class AVP_Behaviour_Marine_HideFromCloakedPredator : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:

	// InCover Update State Enum
	enum UpdateState
	{
		UNSET,
		POSITION_EVALUATION,
		MOVE_TO_HIDE_POSITION,
		AT_HIDE_POSITION,
		MAX_STATES
	};

	// Construction / Deconstruction
	AVP_Behaviour_Marine_HideFromCloakedPredator( AVP_Brain_Actor_Marine* pxBrain );
	virtual ~AVP_Behaviour_Marine_HideFromCloakedPredator( void );

	// Public Member Virtual Functions...
	virtual bool 					Activate( void );
	virtual void 					Deactivate( void );
	virtual void					PerFrameUpdate( const float fTimeslice );
	#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void					RenderDebugInfo( void );
	#endif

	// Public Member Functions...
	void SetLastSeenPosition(const Asura_Vector_3 xPosition) { m_xLastSeenPosition = xPosition; }

protected:

	// Protected Member Functions...
	UpdateState						GetUpdateState( void ) const					{return m_eUpdateState;}
	AVP_Brain_Actor_Marine*			GetMarineBrain( void )							{return m_pxMarineBrain;}
	AVP_ServerActor_Human*			GetOurActor( void ) const						{ return ( m_pxMarineBrain->GetHumanActor() ); }
	AVP_Tactical_System*			GetTacticalSystem( void )						{ return ( m_pxMarineBrain->GetTacticalSystem() ); }
	AVP_Knowledge_System_Actor*		GetKnowledgeSystem( void )						{ return ( m_pxMarineBrain->GetKnowledgeSystem() ); }
	AVP_ServerEntity_Squad_Marine*	GetSquad( void ) const							{ return ( m_pxMarineBrain->GetMarineSquad() ); }
	bool							DoesHaveThreat( void )							{ return ( m_pxMarineBrain->DoesHaveThreat() ); }
	bool							DoesHaveAlert( void )							{ return ( m_pxMarineBrain->DoesHaveAlert() ); }
	void							SetUpdateState( UpdateState eState )			{ m_eUpdateState = eState; }

	void							SwitchToState( UpdateState eState );
	static u_int					GetHideFromPredatorScoreForWaypoint( AVP_Navigation::AVP_PosEvalParams xPosEvalParams );
	void							FindBestWaypoint( AVP_Navigation::PosEvalFunctor* pxPosEval );
	void							AimAroundLastSeenPos( void );
	void							FireAtAimPoint( void );
	void							StopShooting( void );
	void							DoAimingAndFiring( void );
	

private:

	// Private Member Variables...
	UpdateState						m_eUpdateState;
	AVP_Brain_Actor_Marine*			m_pxMarineBrain;
	Asura_Vector_3					m_xLastSeenPosition;
	float							m_fShootTimer;
	float							m_fShootTimeOut;
	float							m_fStopShootTimeOut;
	float							m_fAimTimer;
	float							m_fHideTimer;
	float							m_fHideTimout;

	enum { CURRENT_CHUNK_VERSION = 1 };
};

#endif //__AVP_BEHAVIOUR_MARINE_HIDEFROMCLOAKEDPREDATOR_H__

/// End Of File 'AVP_Behaviour_Marine_HideFromCloakedPredator.h'
