//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_MeleeCounter.cpp
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_MeleeCounter.h"
// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif
// axon
// AVP
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"

#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Actor/Vertebrates/AVP_Vertebrate_MeleeCounter_System.h"


//---------------------------------------------------------------------------------
//*********************************************************************************
//---------------------------------------------------------------------------------
// Class		: AVP_Behaviour_Marine_MeleeCounter
// Description	: 
//---------------------------------------------------------------------------------
//*********************************************************************************
//---------------------------------------------------------------------------------

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleeCounter :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_MeleeCounter::AVP_Behaviour_Marine_MeleeCounter(AVP_Brain_Actor_Marine* pxBrain)
: PARENT(pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_MELEE_COUNTER)
, m_pxMarineBrain(pxBrain)
, m_eUpdateState(UNSET)
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleeCounter :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_MeleeCounter::Activate()
{
	const bool bParentOk = PARENT::Activate();

	AVP_Knowledge_System_Actor* pxKnowledgeSystem	= GetMarineBrain()->GetKnowledgeSystem();
	Asura_Guid					uThreatGuid			= pxKnowledgeSystem->GetBestThreatKnowledge().GetGuid();

	Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_REQUEST_MELEE, sizeof(AVP_Message_Vertebrate_RequestMelee));
	
	if (pxMessage)
	{
		AVP_Message_Vertebrate_RequestMelee* pxData = static_cast<AVP_Message_Vertebrate_RequestMelee*>(pxMessage->GetData());

		pxData->SetType(AVP_MELEEATTACK_TYPE_COUNTER);
		pxData->SetTargetGuid(uThreatGuid);

		pxMessage->SetToGuid(GetMarineBrain()->GetOwnerGuid());
		pxMessage->SetFromGuid(GetMarineBrain()->GetOwnerGuid());
		Asura_Network::Send(pxMessage);
	}

	if ( Asura_Random::GetFloat(0.0f, 1.0f) > 0.5f )
	{
		GetMarineBrain()->GetHumanActor()->PlayDialogue( AVP_ServerActor_Human::AVP_DLG_HUMAN_SCARED_REACTION );
	}

	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleeCounter :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_MeleeCounter::Deactivate()
{
	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleeCounter :: Pause
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_MeleeCounter::Pause()
{
	PARENT::Pause();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleeCounter :: Unpause
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_MeleeCounter::Unpause()
{
	PARENT::Unpause();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleeCounter :: Update
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_MeleeCounter::Update(const float fTimeslice)
{
	if ((!GetMarineBrain()) || (!GetMarineBrain()->GetHumanActor()))
	{
		return;
	}

	switch (GetUpdateState())
	{
		case UNSET:
		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleeCounter :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_MeleeCounter::PerFrameUpdate(const float fTimeslice)
{
	if ((!GetMarineBrain()) || (!GetMarineBrain()->GetHumanActor()))
	{
		return;
	}

	switch (GetUpdateState())
	{
		case UNSET:
		default:
		{

		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_MeleeCounter :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[AVP_Behaviour_Marine_MeleeCounter::MAX_STATES] =
{
	"MELLE_COUNTER - UNSET",
};

void AVP_Behaviour_Marine_MeleeCounter::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		AVP_Brain_Actor_Marine::s_uTemporaryChildDebugColour,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		s_aszStateStrings[GetUpdateState()]
	);

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_MeleeCounter.cpp'
