//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_SquadLeader.cpp
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// INCLUDES
//---------------------------------------------------------------------------

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_SquadLeader.h"

// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// AVP
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Entity/Locations/Objective/Marine/AVP_ServerEntity_Objective_Marine.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/AVP_Messages.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_PredatorCloak.h"

// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Alert.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_HideFromCloakedPredator.h"
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Investigate.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Knowledge/SenseTypes/AVP_Sense_Vision_Actor.h"
#include "Tactical/AVP_Tactical_System.h"

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_SquadLeader::AVP_Behaviour_Marine_SquadLeader( AVP_Brain_Actor_Marine* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_SQUADLEADER )
, m_eUpdateState( IDLE )
, m_pxMarineBrain( pxBrain )
, m_uLastObjectiveGuid( ASURA_GUID_UNREGISTERED )
, m_bThreatIsCloakedPred( false )
, m_uCloakedPredGuid( ASURA_GUID_UNREGISTERED )
, m_xCloakedPredLastKnownPosition( Asura_Maths::ZeroVector3 )
, m_bNewLookSet( false )
, m_fLookTimer( 0.0f )
, m_fLookTimeout( 0.0f )
, m_bDirection( false )
, m_uLastRouteSegment( 0 )
{
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: Destructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_SquadLeader::~AVP_Behaviour_Marine_SquadLeader( void )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_SquadLeader::Activate()
{
	const bool bParentOk = PARENT::Activate();
	// Put Code Below This Comment

	// Put Code Above This Comment
	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_SquadLeader::Deactivate()
{
	// Put Code Above This Comment
	DestroyAllChildren( false );
	PARENT::Deactivate();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_SquadLeader::PerFrameUpdate(const float fTimeslice)
{
	// First of All Check we are still the squad leader
	if ( !AreWeStillSquadLeader() )
	{
		SetFinished();
		return;
	}

	bool				 bHaveThreat	= DoesHaveThreat();
	bool				 bHaveAlert		= DoesHaveAlert();

	CheckForObjective( bHaveThreat, bHaveAlert );

	switch ( GetUpdateState() )
	{
		//---------------------------------------------------------------------------
		case IDLE:
		{
			if ( GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS )
			{
				GetMarineBrain()->GetTacticalSystem()->SetForceAiming( false );
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );

				if ( bHaveThreat )
				{
					SwitchToState( ATTACK );
				}
				else if ( bHaveAlert )
				{
					SwitchToState( ALERT );
				}
			}
			else
			{
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );

				if ( bHaveThreat )
				{
					GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
					SwitchToState( ATTACK );
				}
				else if ( bHaveAlert )
				{
					SwitchToState( ALERT );
				}
			}

			if ( GetMarineBrain()->GetLookAtGuid() != ASURA_GUID_UNREGISTERED )
			{
				GetMarineBrain()->LookAt_LookAtEntity();
			}
			else
			{
				GetMarineBrain()->LookForward();
				GetMarineBrain()->AimForward();
			}

		}break;
		//---------------------------------------------------------------------------
		case ATTACK:
		{
			if ( !bHaveThreat && GetKnowledgeSystem()->GetBestThreatKnowledge().m_uTypeInformation != AVP_Sense_Vision::TYPE_INFO_CLOAKED_CANT_SEE )
			{
				if ( ( m_bThreatIsCloakedPred ) && ( m_uCloakedPredGuid != ASURA_GUID_UNREGISTERED ) )
				{
					AVP_ServerActor_Predator* pxPredator = AVP_ServerActor_Predator::FindEntity( m_uCloakedPredGuid );

					if ( pxPredator && !pxPredator->IsDead() && pxPredator->IsEntityActive() )
					{
						SwitchToState( HIDE_FROM_CLOAKED_PRED );
					}
					else
					{
						SwitchToState( IDLE );
					}
				}
				else
				{
					// we don't have a threat any more kill the attack behaviour
					SwitchToState( IDLE );
				}
			}
			else
			{
				if ( GetKnowledgeSystem()->GetBestThreatKnowledge().m_uSourceClassification == AVP_ENTITYCLASS_PREDATOR && GetKnowledgeSystem()->GetBestThreatKnowledge().m_uTypeInformation == AVP_Sense_Vision::TYPE_INFO_CLOAKED_CANT_SEE)
				{
					AVP_ServerActor_Predator* pxPredator = AVP_ServerActor_Predator::FindEntity( GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid() );

					if ( pxPredator )
					{
						AVP_InventoryItem_Cloak* pxCloak = static_cast< AVP_InventoryItem_Cloak* >( pxPredator->GetInventorySystem()->GetItem( uAVP_INVENTORY_ITEMHASH_CLOAK ) );

						if ( pxCloak && pxCloak->IsUsing( false ) )
						{
							Asura_Vector_3 xPredChest	= Asura_Maths::ZeroVector3;
							Asura_Vector_3 xOurEyes		= Asura_Maths::ZeroVector3;

							GetOurActor()->GetHeadPosition( xOurEyes );
							pxPredator->GetChestPosition( xPredChest );

							Asura_Vector_3 xDirection = xPredChest - xOurEyes;
							float fDistance = xDirection.Normalise();

							const Asura_LOS_InputData	xInput = Asura_LOS_InputData(xOurEyes, xDirection, GetOurActor()->GetGuid(), fDistance, false);
							const Asura_LOS_OutputData*	pxOutput = Asura_LOS::TestAgainstEnvironment(&xInput);

							if (pxOutput->m_fDistanceToIntersection >= fDistance)
							{
								m_bThreatIsCloakedPred = true;
								m_uCloakedPredGuid = GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid();
								m_xCloakedPredLastKnownPosition = xPredChest;

								if ((fDistance * fDistance) >= GetKnowledgeSystem()->GetCloakedVisibleRangeSq())
								{
									SwitchToState( HIDE_FROM_CLOAKED_PRED );
								}
							}
							else
							{
								m_uCloakedPredGuid = ASURA_GUID_UNREGISTERED;
								m_bThreatIsCloakedPred = false;
								m_xCloakedPredLastKnownPosition = Asura_Maths::ZeroVector3;
							}
						}
					}
				}
				else
				{
					m_uCloakedPredGuid = ASURA_GUID_UNREGISTERED;
					m_bThreatIsCloakedPred = false;
					m_xCloakedPredLastKnownPosition = Asura_Maths::ZeroVector3;
				}
			}
		}break;
		//---------------------------------------------------------------------------
		case HIDE_FROM_CLOAKED_PRED:
		{
			if ( bHaveThreat && ( GetKnowledgeSystem()->GetBestThreatKnowledge().m_uSourceClassification != AVP_ENTITYCLASS_PREDATOR || GetKnowledgeSystem()->GetBestThreatKnowledge().m_uTypeInformation == AVP_Sense_Vision::TYPE_INFO_LASER ) )
			{
				SwitchToState( ATTACK );
			}
			else 
			{
				Axon_Behaviour* pxHideBehaviour = GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_HIDEFROMCLOAKEDPREDATOR );

				if ( pxHideBehaviour && pxHideBehaviour->IsFinished() )
				{
					SwitchToState( IDLE );
				}
			}

		}break;
		//---------------------------------------------------------------------------
		case ALERT:
		{
			if ( bHaveThreat )
			{
				SwitchToState( ATTACK );
			}
			else if (GetChildWithType(this, AVP_BEHAVIOUR_TYPE_MARINE_ALERT)->IsFinished())
			{
				AVP_Behaviour_Marine_Alert* pxAlertBehaviour = ( AVP_Behaviour_Marine_Alert* )GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ALERT );

				if ( pxAlertBehaviour )
				{
					bool bScan = false;

					if ( pxAlertBehaviour->GetUpdateState() == AVP_Behaviour_Marine_Alert::SCAN_AREA )
					{
						bScan = true;
					}

					DestroyChildWithType(AVP_BEHAVIOUR_TYPE_MARINE_ALERT);

					if ( bScan )
					{
						SwitchToState( BE_ALERT );
					}
					else
					{
						SwitchToState( IDLE );
					}
				}
			}
		}break;
		//---------------------------------------------------------------------------
		case BE_ALERT:
		{
			//LookAround( fTimeslice );

			if ( bHaveThreat )
			{
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
				SwitchToState( ATTACK );
			}
			else if ( bHaveAlert )
			{
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
				SwitchToState( ALERT );
			}
			else if ( GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS )
			{
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( false );
				SwitchToState( IDLE );
			}
			else
			{
				GetMarineBrain()->GetTacticalSystem()->SetIsAllowedToReact( true );
			}

		}break;
		//---------------------------------------------------------------------------
		default:
		{
			Asura_Assert( false, "AVP_Behaviour_Marine_SquadLeader :: PerFrameUpdate - Update State Is Invalid Please Report To Mantis And Assign To Gaz" );
		}break;
		//---------------------------------------------------------------------------
	}
}


//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: DoesSquadHaveValidObjective
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_SquadLeader::DoesSquadHaveValidObjective()
{
	AVP_ServerActor_Human*			pxOurActor = GetMarineBrain()->GetHumanActor();
	AVP_ServerEntity_Squad_Marine*	pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity(pxOurActor->GetInitialSquadGuid());

	if ( pxSquad && pxSquad->IsSquadSetup() )
	{
		if ( pxSquad->IsPlayersSquad() )
		{
			if ( pxSquad->GetCurrentObjective() && !GetMarineBrain()->DoesHaveThreat() && !GetMarineBrain()->DoesHaveAlert() && 
				( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS || pxSquad->ShouldIgnoreAlertState() ) &&
				pxSquad->GetCurrentObjectiveState() != AVP_ServerNode_Objective_Marine::NOT_STARTED &&
				pxSquad->GetCurrentObjectiveState() != AVP_ServerNode_Objective_Marine::MOVING_TO_OBJECTIVE )
			{
				return true;
			}
		}
		else
		{
			if ( pxSquad->GetCurrentObjective() && !GetMarineBrain()->DoesHaveThreat() && !GetMarineBrain()->DoesHaveAlert() && 
				( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS || pxSquad->ShouldIgnoreAlertState() ) &&
				pxSquad->GetCurrentObjectiveState() != AVP_ServerNode_Objective_Marine::COMPLETE )
			{
				return true;
			}

		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: LookInAGoodDirection
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_SquadLeader::LookInAGoodDirection( void )
{
	AVP_Navigation::NavData& xNavData = GetMarineBrain()->GetNavData();

	Asura_Vector_3 xLookDirection(0.0f, 0.0f, 0.0f);

	AVP_Waypoint_Manager::GetMaxLOFDirection(xNavData.m_usCurrentWPID, xLookDirection);

	if (xLookDirection != Asura_Maths::ZeroVector3)
	{
		GetMarineBrain()->SetAimDirection(xLookDirection);
		GetMarineBrain()->SetLookAtPosition( GetMarineBrain()->GetActor()->GetPosition() + xLookDirection * 10.0f );
	}
	else
	{
		GetMarineBrain()->AimForward();
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: SwitchToState
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_SquadLeader::SwitchToState( SLUpdateState eState )
{
	switch ( GetUpdateState() )
	{
		case ATTACK:
		{
			if ( GetMarineBrain()->GetMarineSquad() )
			{
				if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SELECTOR ) )
					DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SELECTOR );
			}
			else
			{
				if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ATTACK ) )
					DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK );
			}
		}break;

		case ALERT:
		{
			if ( GetChildWithType( this, AVP_BEHAVIOUR_TYPE_MARINE_ALERT ) )
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ALERT );
		}break;

		case HIDE_FROM_CLOAKED_PRED:
		{
			if ( GetChildWithType(this, AVP_BEHAVIOUR_TYPE_MARINE_HIDEFROMCLOAKEDPREDATOR ) )
				DestroyChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_HIDEFROMCLOAKEDPREDATOR );

		}break;

		case BE_ALERT:
		{
			GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier(1.0f);

		}break;

		default:
		{
		}break;
	}

	SetUpdateState( eState );

	switch ( eState )
	{
		//---------------------------------------------------------------------------
		case BE_ALERT:
		{
			m_bNewLookSet = false;
			m_fLookTimer = 0.0f;
		}break;
		//---------------------------------------------------------------------------
		case IDLE:
		{
			
		}break;
		//---------------------------------------------------------------------------
		case ATTACK:
		{
			if ( GetMarineBrain()->GetMarineSquad() )
			{
				// need to create an attack behaviour here
				CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK_SELECTOR );
			}
			else
			{
				CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ATTACK );
			}

		}break;
		//---------------------------------------------------------------------------
		case HIDE_FROM_CLOAKED_PRED:
		{
			AVP_Behaviour_Marine_HideFromCloakedPredator* pxBehaviour = (AVP_Behaviour_Marine_HideFromCloakedPredator*)CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_HIDEFROMCLOAKEDPREDATOR );

			if ( pxBehaviour )
			{
				pxBehaviour->SetLastSeenPosition( m_xCloakedPredLastKnownPosition );
			}
		}break;
		//---------------------------------------------------------------------------
		case ALERT:
		{
			// need to create an alert behaviour here
			CreateChildWithType( AVP_BEHAVIOUR_TYPE_MARINE_ALERT );
		}break;
		//---------------------------------------------------------------------------
		default:
		{
			Asura_Assert( false, "AVP_Behaviour_Marine_SquadLeader :: SwitchToState - Switch To State Is Invalid Please Report To Mantis And Assign To Gaz" );
		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: LookAround
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_SquadLeader::CheckForObjective( bool bHasThreat, bool bHasAlert )
{

	if ( GetOurActor()->GetCurrentObjective() )
	{
		bool bShouldExitOnObjectiveState = false;

		// If we are NOT a players squad we should be moving towards objectives and playing actions.
		if ( GetOurActor()->GetCurrentObjectiveState() != AVP_ServerNode_Objective_Marine::COMPLETE )
		{
			bShouldExitOnObjectiveState = true;
		}

		bool bExitNoThreat = false;

		// We are not engaging threats we want to keep moving towards the objective
		if ( GetOurActor()->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION )
		{
			bExitNoThreat = true;
		}
		else
		{
			// we do not have a threat or an alert so move to objective
			if	( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS )
			{
				if ( !bHasAlert && !bHasThreat )
				{
					bExitNoThreat = true;
				}
			}
		}

		if ( bShouldExitOnObjectiveState && bExitNoThreat )
		{
			SetFinished();
		}

		return;
	}

	AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

	if ( !pxSquad || !pxSquad->GetCurrentObjective() ) { return; }

	bool bShouldExitOnObjectiveState = false;

	// If we are a players squad we do not move to objectives in the normal way however we still want to perfrom actions.
	if ( pxSquad->IsPlayersSquad() && 
		(pxSquad->GetCurrentObjectiveState() == AVP_ServerNode_Objective_Marine::ARRIVED_AT_OBJECTIVE ||
		pxSquad->GetCurrentObjectiveState() == AVP_ServerNode_Objective_Marine::WAITING_FOR_COMPLETION) )
	{
		bShouldExitOnObjectiveState = true;
	}

	// If we are NOT a players squad we should be moving towards objectives and playing actions.
	if ( !pxSquad->IsPlayersSquad() &&
		pxSquad->GetCurrentObjectiveState() != AVP_ServerNode_Objective_Marine::COMPLETE )
	{
		bShouldExitOnObjectiveState = true;
	}

	bool bExitNoThreat = false;

	// We are not engaging threats we want to keep moving towards the objective
	if ( pxSquad->GetCurrentObjectiveThreatAction() == AVP_ServerNode_Objective_Marine::NO_ACTION )
	{
		bExitNoThreat = true;
	}
	else
	{
		// we do not have a threat or an alert so move to objective
		if	( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS || pxSquad->ShouldIgnoreAlertState() )
		{
			if ( !bHasAlert && !bHasThreat )
			{
				bExitNoThreat = true;
			}
		}
	}

	if ( bShouldExitOnObjectiveState && bExitNoThreat && AVP_Behaviour_Marine_Investigate::GetInvestigatorGuid() != GetMarineBrain()->GetOwnerGuid() )
	{
		SetFinished();
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: LookAround
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_SquadLeader::LookAround(const float fTimeslice)
{
	GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier(0.5f);

	AVP_ServerActor_Human* pxOurActor = GetOurActor();

	if ( GetMarineBrain()->DoesHaveThreat() )
	{
		GetMarineBrain()->LookAtThreat();
		return;
	}
	
	if ( !m_bNewLookSet )
	{
		// we don't know where the sound came from so scan the area.
		const Asura_Matrix_3x3&		xOurOri		= pxOurActor->GetOrientation();
		Asura_Vector_3				xFwd		= Asura_Vector_3( xOurOri.e13, xOurOri.e23, xOurOri.e33 );
		Asura_Vector_3				xUp			= Asura_Vector_3( xOurOri.e12, xOurOri.e22, xOurOri.e32 );
		Asura_Vector_3				xDirection	= Asura_Maths::ZeroVector3;
		Asura_Vector_3				xLookAt		= Asura_Maths::ZeroVector3;
		Asura_Vector_3				xChestPos	= Asura_Maths::ZeroVector3;
		Asura_Angle					fDeg45		= Asura_Maths::Deg90 / 2.0f;
		Asura_Angle					fAngle		= Asura_Random::GetFloat( Asura_Maths::Deg90, Asura_Maths::Deg90 + fDeg45 );
		Asura_Quat					xQuat;

		if (Asura_Random::GetIntU(0, 1))
			fAngle = -fAngle;

		pxOurActor->GetChestPosition( xChestPos );
		xQuat.FromMatrix( xOurOri );
		xQuat.FromRotation( fAngle, xUp );
		xDirection = xQuat.RotateVector( xFwd);
		xDirection.Normalise();
		xLookAt = xChestPos + xDirection;
		GetMarineBrain()->AimAtPosition( xLookAt );

		m_bNewLookSet = true;
		m_fLookTimer = 0.0f;
		m_fLookTimeout = Asura_Random::GetFloat( 1.9f, 2.5f );
	}
	else
	{
		m_fLookTimer += fTimeslice;

		if ( m_fLookTimer > 2.5f )
		{
			m_bNewLookSet = false;
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: LookScan
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_SquadLeader::LookScan(const float fTimeslice)
{
	GetMarineBrain()->GetTacticalSystem()->SetAimLerpModifier(0.5f);

	AVP_ServerActor_Human* pxOurActor = GetMarineBrain()->GetHumanActor();

	AVP_Navigation::NavData& xNavData = GetMarineBrain()->GetNavData();

	if (xNavData.m_usRouteSegmentIndex != m_uLastRouteSegment && xNavData.m_usRouteSegmentIndex != 0)
	{
		if ( ( xNavData.m_usRouteSegmentIndex + 1 ) < xNavData.m_usRouteLength )
		{
			Asura_AI_NavVertex* pxNextWp = Asura_AI_Navigation_System::GetWaypoint( xNavData.m_pusRoute[xNavData.m_usRouteSegmentIndex + 1] );
			Asura_AI_NavVertex* pxThisWp = Asura_AI_Navigation_System::GetWaypoint( xNavData.m_pusRoute[xNavData.m_usRouteSegmentIndex] );
			Asura_AI_NavVertex* pxLastWp = Asura_AI_Navigation_System::GetWaypoint( xNavData.m_pusRoute[xNavData.m_usRouteSegmentIndex - 1] );

			if ( pxNextWp && pxThisWp && pxNextWp )
			{
				Asura_Vector_3 xDirection1( pxThisWp->m_xPosition - pxLastWp->m_xPosition );
				Asura_Vector_3 xDirection2( pxLastWp->m_xPosition - pxThisWp->m_xPosition );

				float fDot = xDirection1 * xDirection2;

				if (fDot < 0.995)
				{
					m_bNewLookSet = false;

					if (m_bDirection)
						m_bDirection = false;
					else
						m_bDirection = true;
				}
			}
		}
	}

	m_uLastRouteSegment = xNavData.m_usRouteSegmentIndex;

	if ( !m_bNewLookSet )
	{
		// we don't know where the sound came from so scan the area.
		const Asura_Matrix_3x3&		xOurOri			= pxOurActor->GetOrientation();
		Asura_Vector_3				xMoveDir		= GetMarineBrain()->GetMovementDirection();

		xMoveDir.y = 0.0f;
		
		Asura_Vector_3				xUp				= Asura_Vector_3( xOurOri.e12, xOurOri.e22, xOurOri.e32 );
		Asura_Vector_3				xDirection		= Asura_Maths::ZeroVector3;
		Asura_Vector_3				xLookAt			= Asura_Maths::ZeroVector3;
		Asura_Vector_3				xChestPos		= Asura_Maths::ZeroVector3;
		Asura_Angle					fDeg45			= Asura_Maths::Deg90 / 2.0f;
		Asura_Quat					xQuat;

		if ( m_bDirection == true )
		{
			fDeg45 = -fDeg45;
			m_bDirection = false;
		}
		else
		{
			m_bDirection = true;
		}

		xQuat.FromRotation( fDeg45, xUp );
		xDirection = xQuat.RotateVector( xMoveDir);
		xDirection.Normalise();
		GetMarineBrain()->SetAimDirection( xDirection );
		GetMarineBrain()->SetLookAtPosition( pxOurActor->GetPosition() + xDirection * 10.0f );

		m_bNewLookSet = true;
		m_fLookTimer = 0.0f;
		m_fLookTimeout = Asura_Random::GetFloat( 1.0f, 2.0f );
	}
	else
	{
		m_fLookTimer += fTimeslice;

		if ( m_fLookTimer > m_fLookTimeout )
		{
			m_bNewLookSet = false;
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: AreWeStillSquadLeader
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_SquadLeader::AreWeStillSquadLeader() const
{
	// If we don't have a squad Guid then we are in a squad on our own.
	if ( GetOurActor()->GetInitialSquadGuid() == ASURA_GUID_UNREGISTERED ) { return true; }

	AVP_ServerEntity_Squad_Marine* pxSquad = GetSquad();

	// If we have no squad its in the default squad
	if ( !pxSquad )	{ return true; }

	// Check the squad leaders guid is our guid
	if ( pxSquad->GetSquadLeaderGuid() == GetOurActor()->GetGuid() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_SquadLeader :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszMarineStateStrings[AVP_Behaviour_Marine_SquadLeader::MAX_STATES] =
{
	"MARINE LEADER - UNSET",
	"MARINE LEADER - IDLE",
	"MARINE LEADER - ATTACK",
	"MARINE LEADER - ALERT",
	"MARINE LEADER - HIDE_FROM_CLOAKED_PRED",
	"MARINE LEADER - BE_ALERT",
};

static const char* s_aszAndroidStateStrings[AVP_Behaviour_Marine_SquadLeader::MAX_STATES] =
{
	"ANDROID LEADER - UNSET",
	"ANDROID LEADER - IDLE",
	"ANDROID LEADER - ATTACK",
	"ANDROID LEADER - ALERT",
	"ANDROID LEADER - HIDE_FROM_CLOAKED_PRED",
	"ANDROID LEADER - BE_ALERT",
};

static const char* s_aszLoneMarineStateStrings[AVP_Behaviour_Marine_SquadLeader::MAX_STATES] =
{
	"LONE MARINE - UNSET",
	"LONE MARINE - IDLE",
	"LONE MARINE - ATTACK",
	"LONE MARINE - ALERT",
	"LONE MARINE - HIDE_FROM_CLOAKED_PRED",
	"LONE MARINE - BE_ALERT",
};

void AVP_Behaviour_Marine_SquadLeader::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	bool bIsAndroid = false;

	if (GetMarineBrain()->GetHumanActor()->GetClassification() == AVP_ENTITYCLASS_ANDROID)
	{
		bIsAndroid = true;
	}

	if (!GetSquad())
	{
		Asura_Font_System::Print
		(
			GetProjectBrain()->GetDebugOutputPos().x,
			GetProjectBrain()->GetDebugOutputPos().y,
			0.5f, 
			AVP_Brain_Actor_Marine::s_uTemporaryLeadersChildDebugColour,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
			s_aszLoneMarineStateStrings[GetUpdateState()]
		);
	}
	else
	{
		if (!bIsAndroid)
		{
			Asura_Font_System::Print
			(
				GetProjectBrain()->GetDebugOutputPos().x,
				GetProjectBrain()->GetDebugOutputPos().y,
				0.5f, 
				0xff00ff00,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
				s_aszMarineStateStrings[GetUpdateState()]
			);
		}
		else
		{
			Asura_Font_System::Print
			(
				GetProjectBrain()->GetDebugOutputPos().x,
				GetProjectBrain()->GetDebugOutputPos().y,
				0.5f, 
				0xffff0000,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
				s_aszAndroidStateStrings[GetUpdateState()]
			);
		}
	}

	// Offset the debug output position for other behaviours to render there own text
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_SquadLeader.cpp'
