//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_SquadLeader.h
// Created		: 
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// BEGIN FILE WRAPPER
//---------------------------------------------------------------------------
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_BEHAVIOUR_MARINE_SQUADLEADER_H__
#define __AVP_BEHAVIOUR_MARINE_SQUADLEADER_H__ 1
//---------------------------------------------------------------------------
// END FILE WRAPPER
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// INCLUDES
//---------------------------------------------------------------------------

// Asura

// Axon
#include "Behaviour/Axon_Project_Behaviour.h"

// AVP
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"

// AVP Axon
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"

// FORWARD DECLARATIONS ///////////////////////////////////
class AVP_ServerActor_Human;
class AVP_ServerEntity_Squad_Marine;
class AVP_Knowledge_System_Actor;
class AVP_Tactical_System;

//---------------------------------------------------------------------------
// Class		: AVP_Behaviour_Marine_SquadLeader
// Description	: 
//---------------------------------------------------------------------------
class AVP_Behaviour_Marine_SquadLeader : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:

	// InCover Update State Enum
	enum SLUpdateState
	{
		UNSET,
		IDLE,
		ATTACK,
		ALERT,
		HIDE_FROM_CLOAKED_PRED,
		BE_ALERT,
		MAX_STATES
	};

	// Construction / Deconstruction
	AVP_Behaviour_Marine_SquadLeader( AVP_Brain_Actor_Marine* pxBrain );
	virtual ~AVP_Behaviour_Marine_SquadLeader( void );

	// Public Member Functions...
	SLUpdateState					GetUpdateState( void ) const					{ return m_eUpdateState; }
	AVP_Brain_Actor_Marine*			GetMarineBrain( void )							{ return m_pxMarineBrain; }

protected:

	// Protected Virtual Functions...
	virtual bool 					Activate( void );
	virtual void 					Deactivate( void );
	virtual void					PerFrameUpdate( const float fTimeslice );
	#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void					RenderDebugInfo( void );
	#endif

	// Protected Functions...
	void							SetUpdateState( SLUpdateState eState )			{ m_eUpdateState = eState; }
	void							SwitchToState( SLUpdateState eState );
	AVP_ServerActor_Human*			GetOurActor( void ) const						{ return ( m_pxMarineBrain->GetHumanActor() ); }
	AVP_Tactical_System*			GetTacticalSystem( void )						{ return ( m_pxMarineBrain->GetTacticalSystem() ); }
	AVP_Knowledge_System_Actor*		GetKnowledgeSystem( void )						{ return ( m_pxMarineBrain->GetKnowledgeSystem() ); }
	AVP_ServerEntity_Squad_Marine*	GetSquad( void ) const							{ return ( m_pxMarineBrain->GetMarineSquad() ); }	
	bool							DoesHaveThreat( void )							{ return ( m_pxMarineBrain->DoesHaveThreat() ); }
	bool							DoesHaveAlert( void )							{ return ( m_pxMarineBrain->DoesHaveAlert() ); }
	
	bool							AreWeStillSquadLeader( void ) const;
	void							LookInAGoodDirection( void );
	void							LookAround( const float fTimeslice );
	void							LookScan( const float fTimeslice );

	void							CheckForObjective( bool bHasThreat, bool bHasAlert );
	bool							DoesSquadHaveValidObjective();

	

private:

	// Private Member Variables...
	SLUpdateState					m_eUpdateState;
	AVP_Brain_Actor_Marine*			m_pxMarineBrain;
	Asura_Guid						m_uLastObjectiveGuid;
	bool							m_bThreatIsCloakedPred;
	Asura_Guid						m_uCloakedPredGuid;
	Asura_Vector_3					m_xCloakedPredLastKnownPosition;
	bool							m_bNewLookSet;
	float							m_fLookTimer;
	float							m_fLookTimeout;
	bool							m_bDirection;
	u_int							m_uLastRouteSegment;

	enum { CURRENT_CHUNK_VERSION = 0 };
};

#endif //__AVP_BEHAVIOUR_MARINE_SQUADLEADER_H__

/// End Of File 'AVP_Behaviour_Marine_SquadLeader.h'
