//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_Travel.cpp
// Created		: 30 / 10 / 07
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// INCLUDES ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"
// header
#include "Behaviour/BehaviourTypes/Human/Marine/AVP_Behaviour_Marine_Travel.h"
// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#include "Debugging/Asura_DebugLineRender.h"
#endif
// axon
// AVP
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateMachine.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
// AVP axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Knowledge/SenseTypes/AVP_Sense_Vision_Actor.h"
#include "Tactical/AVP_Tactical_System.h"

//---------------------------------------------------------------------------
//***************************************************************************
//---------------------------------------------------------------------------
// Class		: AVP_Behaviour_Marine_Travel
// Description	: Function Implementations for the AVP_Beahviour_Marine_Travel class
//---------------------------------------------------------------------------
//***************************************************************************
//---------------------------------------------------------------------------

float	AVP_Behaviour_Marine_Travel::s_fLookAheadDistance			= 1.0f;		// 1.0f meters
float	AVP_Behaviour_Marine_Travel::s_fAtDestinationRangeSq		= 0.5625f;	// 0.75f meters
float	AVP_Behaviour_Marine_Travel::s_fMinRouteLengthSq			= 4.0f;	// 0.75f meters
u_char	AVP_Behaviour_Marine_Travel::s_ucWaypointWeightAdjustment	= 1;

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: Constructor
//---------------------------------------------------------------------------
AVP_Behaviour_Marine_Travel::AVP_Behaviour_Marine_Travel(AVP_Brain_Actor_Marine* pxBrain)
: PARENT(pxBrain, AVP_BEHAVIOUR_TYPE_MARINE_TRAVEL)
, m_pxMarineBrain(pxBrain)
, m_eUpdateState(UNSET)
, m_bNewDestinationSet(false)
, m_bRouteEnded(false)
, m_bForceMovementDirection(false)
, m_fNearbyUpdateTimer(0.0f)
, m_fNearbyBaseRange(0.0f)
, m_uNumNearbyGuids(0)
, m_xNearestPointOnRoute(Asura_Maths::ZeroVector3)
, m_xPositionOnRoute(Asura_Maths::ZeroVector3)
, m_xPreviousPositionOnRoute(Asura_Maths::ZeroVector3)
, m_fSmoothedPORVelocity(0.0f)
, m_xNearestPositionOnRoute(Asura_Maths::ZeroVector3)
, m_bSpeedAltered(false)
, m_xFollowPoint(Asura_Maths::ZeroVector3)
, m_xSpeedAdjustmentTrackingPoint(Asura_Maths::ZeroVector3)
, m_xAvoidancePosition(Asura_Maths::ZeroVector3)
, m_bShouldStop(false)
, m_bWeightsAltered(false)
, m_pxWaypointsBackup(NULL)
, m_uNumWaypointsAdjusted(0)
, m_fSpeedDiffTimer( 0.0f )
, m_fLengthRemainingOnLink( 0.0f )
, m_fPhysicsNotSupportedTimer( 0.0f )
, m_bFindRoute( false )
, m_fRouteDelayTimer( 0.0f )
, m_fRouteStopTimer( 0.0f )
, m_bStopWithDelayTriggered( false )
, m_bWasAggressive( false )
, m_bTurnRequested( false )
, m_fTurnTimer( 0.0f )
{

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: Activate
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Travel::Activate()
{
	const bool bParentOk = PARENT::Activate();

	u_int uMoveSpeed = AVP_Navigation::MM_MEDIUM;
	GetMarineBrain()->GetTacticalSystem()->SetMovementSpeed(uMoveSpeed);
#ifndef ASURA_NO_DEBUG_OUTPUT
	GetMarineBrain()->SetRenderRoute(false);
#endif

	return bParentOk;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: Deactivate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::Deactivate()
{
	PARENT::Deactivate();

	DestroyAllChildren();

	delete [] m_pxWaypointsBackup;
	m_pxWaypointsBackup = NULL;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: Pause
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::Pause()
{
	PARENT::Pause();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: Unpause
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::Unpause()
{
	PARENT::Unpause();
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: Update
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::Update(const float fTimeslice)
{
	if ((!GetMarineBrain()) || (!GetMarineBrain()->GetHumanActor()))
	{
		return;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: PerFrameUpdate
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::PerFrameUpdate(const float fTimeslice)
{
	AVP_Tactical_System* pxTactical = GetMarineBrain()->GetTacticalSystem();

	if (pxTactical->IsInGrapple() || pxTactical->IsOnGameScene() || pxTactical->IsUsingCover())
	{
		return;
	}

	CheckForNewRoute(fTimeslice);

	switch (GetUpdateState())
	{
		//---------------------------------------------------------------------------
		case MOVING:
		{
			GetMarineBrain()->GetTacticalSystem()->SetIsMoving( true );

			DoMoving(fTimeslice);
		}break;
		//---------------------------------------------------------------------------
		case IDLE:
		{
			GetMarineBrain()->GetTacticalSystem()->SetIsMoving( false );

			UpdateCurrentWaypoint();

			SetStateIdle();

// 			if ( GetMarineBrain()->GetTacticalSystem()->HasArrivedAtDestination() == false )
// 			{
// 				GetMarineBrain()->GetTacticalSystem()->SetHasArrivedAtDestination( true );
// 			}

		}break;
		//---------------------------------------------------------------------------
		case UNSET:
		default:
		{
			SetStateIdle();
		}break;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: CheckForNewRoute
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::CheckForNewRoute(const float fTimeslice)
{
	AVP_Tactical_System* pxTacticalSystem = GetMarineBrain()->GetTacticalSystem();

	// If we have a new destination set the state to find a route
	if (pxTacticalSystem->HasNewDestinationBeenSet())
	{
		if ( !pxTacticalSystem->IsStartMovementDelayOverriden() )
		{
			m_fRouteDelayTimer = Asura_Random::GetFloat(0.0f, 1.5f);
		}
		else
		{
			m_fRouteDelayTimer = 0.0f;
		}

		pxTacticalSystem->NewDestinationProcessed();
		m_bFindRoute = true;
	}

	if (m_bFindRoute)
	{
		m_fRouteDelayTimer -= fTimeslice;

		if ( m_fRouteDelayTimer <= 0.0f )
		{
			m_fRouteDelayTimer = 0.0f;
			pxTacticalSystem->CancelStartMovementDelayOverride();

			// update the current waypoint just before we attempt to find a route
			UpdateCurrentWaypoint();

			// Find a route to the desired destination position
			if (CalculateNewRoute())
			{
				AVP_Navigation::NavData& xNavData = GetMarineBrain()->GetNavData();

				if (xNavData.m_pusRoute)
				{
					m_bForceMovementDirection = true;
					pxTacticalSystem->CancelStopMoving();
					CalculateAndSetMovementDirection(fTimeslice);
					SetStateMoving();
					m_bFindRoute = false;
					return;
				}
			}

			if ( GetMarineBrain()->GetHumanActor()->IsMoving() )
			{
				SetStateIdle();
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: HasArrivedAtDestination
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Travel::HasArrivedAtDestination(void)
{
	if (GetUpdateState() == IDLE)
	{
		return true;
	}
	else
	{
		return false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: DoMoving
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::DoMoving(const float fTimeslice)
{
	if ( GetMarineBrain()->IsInjured() )
	{
		return;
	}

	if ( !GetMarineBrain()->GetNavData().m_pusRoute )
	{
		SetUpdateState( IDLE );
		return;
	}

	AVP_Tactical_System* pxTactical = GetMarineBrain()->GetTacticalSystem();

	// Update the current waypoint while we are moving
	UpdateCurrentWaypoint();

	// Update the Nearby Guid Array
	UpdateNearbyGuidData(fTimeslice);

	// Update the navigation data
	UpdateNavData();

	if ( !pxTactical->HasArrivedAtDestination() )
	{
		// Calculate and set the movement direction for the Animation State Machine
		CalculateAndSetMovementDirection( fTimeslice );

		// Alter the speed if we need to so we don't wander off route if we are moving to fast for a sharp corner
		AlterSpeedIfNeeded();

		// Set the movement speed
		RequestMovement();
	}

	if ( !m_bFindRoute )
	{
		// Check if we have arrived at our desired destination
		if ( CheckForArrivalAtDestination() )
		{
			pxTactical->SetHasArrivedAtDestination(true);

			if ( pxTactical->ShouldStopAtDestination() )
			{
				SetStateIdle();
			}
		}
	}

	CalculateLengthRemainingOnLink();

	if ( AreWeStuck(fTimeslice) )
	{
		if ( CalculateNewRoute() )
		{
			m_bForceMovementDirection = true;
			GetMarineBrain()->GetTacticalSystem()->CancelStopMoving();
			CalculateAndSetMovementDirection( fTimeslice );
			SetStateMoving();
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: AlterSpeedIfNeeded
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::AlterSpeedIfNeeded(void)
{
	AVP_Tactical_System*			pxTactical			= GetMarineBrain()->GetTacticalSystem();

	if (!m_bRouteEnded && !pxTactical->IsSpeedAdjustmentOverriden() )
	{
		AVP_ServerActor_Human*			pxOurActor			= GetMarineBrain()->GetHumanActor();
		Asura_Vector_3					xOurPos				= pxOurActor->GetPosition();
		AVP_Navigation::NavData&		xNavData			= GetMarineBrain()->GetNavData();
		AVP_Human_AIAnimStateMachine*	pxHumanStateMachine	= static_cast<AVP_Human_AIAnimStateMachine*>(pxOurActor->GetAIAnimStateMachine());

		if (pxTactical->GetMovementSpeed() == AVP_Navigation::MM_FAST || m_bSpeedAltered)
		{
			const Asura_Vector_3&		xCurrentPosition	= GetMarineBrain()->GetHumanActor()->GetPosition();
			u_short						uNearestSegmentIndex;

			AVP_Navigation::FindNearestPointOnRoute(xNavData, xCurrentPosition, 5.0f, m_xSpeedAdjustmentTrackingPoint, uNearestSegmentIndex);

			if ((xNavData.m_usRouteSegmentIndex + 1) < xNavData.m_usRouteLength)
			{
				Asura_AI_NavVertex*	pxCurrentWaypoint			= Asura_AI_Navigation_System::GetWaypoint(xNavData.m_pusRoute[xNavData.m_usRouteSegmentIndex]);
				Asura_Vector_3		xSpeedPointToCurrentWP		= pxCurrentWaypoint->m_xPosition - m_xSpeedAdjustmentTrackingPoint;
				Asura_Vector_3		xFollowPointToCurrentWp		= pxCurrentWaypoint->m_xPosition - m_xFollowPoint;

				xSpeedPointToCurrentWP.Normalise();
				xFollowPointToCurrentWp.Normalise();

				float fDot = xSpeedPointToCurrentWP * xFollowPointToCurrentWp;

				if ((fDot < 0.975f) && (fDot >= 0.0f))
				{
					m_bSpeedAltered = true;

					if (pxHumanStateMachine->IsMoveFast())
					{
						pxTactical->SetMovementSpeed(AVP_Navigation::MM_MEDIUM);
					}
				}
				else
				{
					Asura_Vector_3 xCurrentPosToCurrentWp = pxCurrentWaypoint->m_xPosition - m_xPositionOnRoute;

					xCurrentPosToCurrentWp.Normalise();

					fDot = xFollowPointToCurrentWp * xCurrentPosToCurrentWp;

					if ((fDot < 0.975f) && (fDot >= 0.0f))
					{
						m_bSpeedAltered = true;

						if (pxHumanStateMachine->IsMoveFast())
						{
							pxTactical->SetMovementSpeed(AVP_Navigation::MM_MEDIUM);
						}
					}
					else
					{
						if (!pxHumanStateMachine->IsMoveFast())
						{
							pxTactical->SetMovementSpeed(AVP_Navigation::MM_FAST);
						}
					}
				}
			}
			else 
			{
				if (m_bSpeedAltered)
				{
					m_bSpeedAltered = false;
					pxTactical->SetMovementSpeed(AVP_Navigation::MM_FAST);
				}
			}

			return;
		}
		
	}
	
	m_xSpeedAdjustmentTrackingPoint = m_xPositionOnRoute;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: UpdateNavData
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::UpdateNavData(void)
{
	const Asura_Vector_3&		xCurrentPosition	= GetMarineBrain()->GetHumanActor()->GetPosition();
	AVP_Navigation::NavData&	xNavData			= GetMarineBrain()->GetNavData();

	m_xPreviousPositionOnRoute = m_xPositionOnRoute;

	AVP_Navigation::FindNearestPointOnRoute(xNavData, xCurrentPosition, 0.0f, m_xPositionOnRoute, xNavData.m_usRouteSegmentIndex );
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: AreWeStuck
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Travel::AreWeStuck(const float fTimeslice)
{
	// Simplest check is whether we've been at this too long.
	const float fAnimSpeed = GetMarineBrain()->GetAnimationMovementSpeed();
	m_fLengthRemainingOnLink -= fTimeslice * fAnimSpeed;
	if ( m_fLengthRemainingOnLink < 0.0f )
	{
		return true;
	}

	// More complicated check is if our actual speed differs by too much from our animation speed for too long.
	// Also, check that we are actually moving (and there's not been an anim problem).
	const float fActualSpeed = GetMarineBrain()->GetHumanActor()->GetVelocity().Magnitude();

	// In very low frame rate situations ( mainly console debug builds ) -
	// both of these can be zero causing the stuck detection to kick in every frame
	if ( fAnimSpeed == 0.0f && fActualSpeed == 0.0f )
	{
		return false;
	}

	if ( Asura_Maths::Abs( fAnimSpeed - fActualSpeed ) > fAnimSpeed * 0.25f || fAnimSpeed < 0.01f )
	{
		m_fSpeedDiffTimer += fTimeslice;
	}
	else
	{
		m_fSpeedDiffTimer = 0.0f;
	}
	if ( m_fSpeedDiffTimer > 2.5f )
	{
		return true;
	}

	return false;
}

void AVP_Behaviour_Marine_Travel::CalculateLengthRemainingOnLink()
{
	const AVP_Navigation::NavData& xNavData = GetMarineBrain()->GetNavData();

	if ( xNavData.m_usRouteLength > 1 )
	{
		const bool bFirstLink = ( xNavData.m_usRouteSegmentIndex == 0 );

		const Asura_AI_NavVertex* pxFromWP = Asura_AI_Navigation_System::GetWaypoint( xNavData.m_pusRoute[ xNavData.m_usRouteSegmentIndex ] );
		const Asura_AI_NavVertex* pxToWP   = Asura_AI_Navigation_System::GetWaypoint( xNavData.m_pusRoute[ xNavData.m_usRouteSegmentIndex + 1 ] );

		m_fLengthRemainingOnLink  = ( pxToWP->m_xPosition - pxFromWP->m_xPosition ).Magnitude();
		m_fLengthRemainingOnLink += ( bFirstLink ? 15.0f : 5.0f );
	}
	else
	{
		m_fLengthRemainingOnLink = ( xNavData.m_xDestinationPosition - GetMarineBrain()->GetHumanActor()->GetPosition() ).Magnitude() + 5.0f;
	}
}


//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: CalculateAndSetMovementDirection
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::ForceMovementDirectionIfNeeded(const Asura_Vector_3& xNewMoveDir)
{
//	const Asura_Vector_3& xCurrMoveDir = GetMarineBrain()->GetMovementDirection();
//	float fDot = xCurrMoveDir * xNewMoveDir;
//	if (fDot <= 0.0f)
	{
		GetMarineBrain()->ForceMovementDirection(xNewMoveDir);
		m_bForceMovementDirection = false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: DoPhysicsNonSupportedMovement
//---------------------------------------------------------------------------
Asura_Vector_3 AVP_Behaviour_Marine_Travel::DoPhysicsNonSupportedMovement(void)
{
	// We are in the air and so we can't turn! set movement direction to be the actors fwd vector.
	const Asura_Matrix_3x3&	xActorOri = GetMarineBrain()->GetHumanActor()->GetOrientation();
	Asura_Vector_3			xActorFwd = Asura_Vector_3(xActorOri.e13, xActorOri.e23, xActorOri.e33);

	return UpdateMovment(xActorFwd);
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: UpdateMovemnt
//---------------------------------------------------------------------------
Asura_Vector_3 AVP_Behaviour_Marine_Travel::UpdateMovment(Asura_Vector_3 xDirection)
{
	xDirection.Normalise();

	if (m_bForceMovementDirection)
	{
		ForceMovementDirectionIfNeeded(xDirection);
	}

	GetMarineBrain()->SetMovementDirection(xDirection);

	return xDirection;
}



//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: VerifyCollisionAvoidance
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::VerifyCollisionAvoidance(Asura_Vector_3 xAvoidanceSteering)
{
	const static float			sfFastDist		= 3.5f;
	const static float			sfMedDist		= 2.5f;
	const static float			sfSlowDist		= 1.75f;
	AVP_ServerActor_Human*		pxOurActor		= GetMarineBrain()->GetHumanActor();
	AVP_Tactical_System*		pxTacical		= GetMarineBrain()->GetTacticalSystem();
	Asura_Vector_3				xDirection		= xAvoidanceSteering;
	xDirection.Normalise();

	switch (pxTacical->GetMovementSpeed())
	{
		case AVP_Navigation::MM_FAST:
		{
			m_xAvoidancePosition = pxOurActor->GetPosition() + (xDirection * sfFastDist);
		}break;
		case AVP_Navigation::MM_SLOW:
		{
			m_xAvoidancePosition = pxOurActor->GetPosition() + (xDirection * sfMedDist);
		}break;
		case AVP_Navigation::MM_MEDIUM:
		default:
		{
			m_xAvoidancePosition = pxOurActor->GetPosition() + (xDirection * sfSlowDist);
		}break;
		
	}
	
// 	if (!Asura_AI_Navigation_System::IsPositionInWaypoint(m_xAvoidancePosition))
// 	{
// 		RequestIdle();
// 		m_bShouldStop = true;
// 	}
// 	else
	{
		m_bShouldStop = false;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: PositionInLastWaypoint
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Travel::PositionInLastWaypoint(u_short usNearestRouteIndex)
{
	const AVP_Navigation::NavData& xNavData = GetMarineBrain()->GetNavData();

	if ( usNearestRouteIndex == ( xNavData.m_usRouteLength - 2 ) )
	{
		if ( Asura_AI_Navigation_System::IsPositionInWaypoint( m_xPositionOnRoute, xNavData.m_pusRoute[usNearestRouteIndex + 1] ) )
		{
			return true;
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: CalculateAndSetMovementDirection
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::CalculateAndSetMovementDirection(const float fTimeStep)
{
	AVP_ServerActor_Human*			pxOurActor			= GetMarineBrain()->GetHumanActor();
	float							fLookAheadDistance	= GetDistanceToStop();

	if ( fLookAheadDistance == 0.0f )
	{
		// Temporary until the Data has been setup correctly for the Anim State Machine Function GetDistanceRequired.
		AVP_Human_AIAnimStateMachine*	pxHumanStateMachine	= static_cast<AVP_Human_AIAnimStateMachine*>(pxOurActor->GetAIAnimStateMachine());

		fLookAheadDistance = 2.5f;

		if (((pxHumanStateMachine) && pxHumanStateMachine->IsMoveFast()) || m_bSpeedAltered)
		{
			fLookAheadDistance = 2.5f;
		}
	}

	Asura_Vector_3 xLookAtDir = Asura_Maths::ZeroVector3;

	// Is our physics object supported by the floor or another surface?
	if (!AreWeOnASurface())
	{
		m_fPhysicsNotSupportedTimer += fTimeStep;

		if (m_fPhysicsNotSupportedTimer >= 0.3f)
		{
			//xLookAtDir = DoPhysicsNonSupportedMovement();
		}
	}
	else
	{
		m_fPhysicsNotSupportedTimer = 0.0f;

		AVP_Navigation::NavData&	xNavData					= GetMarineBrain()->GetNavData();
		Asura_Vector_3				xMoveDirection				= Asura_Maths::ZeroVector3;
		bool						bPosInLastWaypointOrBeyond	= false;
		bool						bStillDoCollAvoidance		= true;
		u_short						usNearestRouteIndex;

		// Returns false if on the route, returns true if is passed the last waypoint and just needs to get the direction to its destination
		if (!AVP_Navigation::FindNearestPointOnRoute(xNavData, m_xPositionOnRoute, fLookAheadDistance, m_xFollowPoint, usNearestRouteIndex))
		{
			if (PositionInLastWaypoint(usNearestRouteIndex))
			{
				float fDistToDestSq = ( pxOurActor->GetPosition() - GetMarineBrain()->GetTacticalSystem()->GetDestination() ).MagnitudeSquared();

				if ( fDistToDestSq < GetMarineBrain()->GetTacticalSystem()->GetAtDestinationRange() * 2.0f )
				{
					bStillDoCollAvoidance = true;
				}
				
				bPosInLastWaypointOrBeyond = true;

				Asura_Vector_3	xToDest( xNavData.m_xDestinationPosition - pxOurActor->GetPosition() );
				float			fDistanceToDest = xToDest.Normalise();
				
				m_xFollowPoint = pxOurActor->GetPosition() + xToDest * fLookAheadDistance;

				if ( fDistanceToDest <= fLookAheadDistance )
				{
					m_xFollowPoint = xNavData.m_xDestinationPosition;
				}
			}
		}
		else
		{
			bPosInLastWaypointOrBeyond = true;

			Asura_Vector_3	xToDest( xNavData.m_xDestinationPosition - pxOurActor->GetPosition() );
			float			fDistanceToDest = xToDest.Normalise();
			
			m_xFollowPoint = pxOurActor->GetPosition() + xToDest * fLookAheadDistance;

			if ( fDistanceToDest <= fLookAheadDistance )
			{
				m_xFollowPoint = xNavData.m_xDestinationPosition;
			}
		} 
		
		if (!bPosInLastWaypointOrBeyond)
		{
			Asura_Vector_3 xCollisionAvoidanceSteering;

			xMoveDirection = m_xFollowPoint - pxOurActor->GetPosition();
			xMoveDirection.Normalise();

			// Collision Avoidance
			CalculateCollisionAvoidanceSteering(xMoveDirection, xCollisionAvoidanceSteering);

			// If the collision avoidance is a zero vector then there is no collision imminent
			if (xCollisionAvoidanceSteering != Asura_Maths::ZeroVector3)
			{
				float fCollisionSteeringLength = xCollisionAvoidanceSteering.MagnitudeSquared();
				const static float sfCollisionSteeringLengthSq = 0.5f;

				if (fCollisionSteeringLength > sfCollisionSteeringLengthSq)
				{
					VerifyCollisionAvoidance(xCollisionAvoidanceSteering);	
				}
				else
				{
					m_xAvoidancePosition = Asura_Maths::ZeroVector3;
					m_bShouldStop = false;
				}

				xMoveDirection = (xMoveDirection + xCollisionAvoidanceSteering) * 0.5f;
			}
			else
			{
				m_xAvoidancePosition = Asura_Maths::ZeroVector3;
				m_bShouldStop = false;
			}
		}
		else
		{
			Asura_Vector_3 xCollisionAvoidanceSteering;

			// Set our movement direction based on the direction directly to our destination position
			xMoveDirection = xNavData.m_xDestinationPosition - pxOurActor->GetPosition();
			xMoveDirection.Normalise();

			if (bStillDoCollAvoidance)
			{
				// Collision Avoidance
				CalculateCollisionAvoidanceSteering(xMoveDirection, xCollisionAvoidanceSteering);

				// If the collision avoidance is a zero vector then there is no collision imminent
				if (xCollisionAvoidanceSteering != Asura_Maths::ZeroVector3)
				{
					float fCollisionSteeringLength = xCollisionAvoidanceSteering.MagnitudeSquared();
					const static float sfCollisionSteeringLengthSq = 0.005f;

					if (fCollisionSteeringLength > sfCollisionSteeringLengthSq)
					{
						VerifyCollisionAvoidance(xCollisionAvoidanceSteering);	
					}
					else
					{
						m_xAvoidancePosition = Asura_Maths::ZeroVector3;
						m_bShouldStop = false;
					}

					xMoveDirection = (xMoveDirection + xCollisionAvoidanceSteering) * 0.5f;
				}
				else
				{
					m_xAvoidancePosition = Asura_Maths::ZeroVector3;
					m_bShouldStop = false;
				}
			}

			if (!m_bRouteEnded)
			{
				m_bRouteEnded = true;
			}

			m_xAvoidancePosition = Asura_Maths::ZeroVector3;
			m_bShouldStop = false;
		}
		
		if ( xMoveDirection != Asura_Maths::ZeroVector3 )
		{
			xLookAtDir = UpdateMovment(xMoveDirection);

			// Does the route control where the actor is looking?
			if (GetMarineBrain()->GetTacticalSystem()->DoesRouteSetLookAt())
			{
				xLookAtDir.y = 0.0f;

				if ( xLookAtDir != Asura_Maths::ZeroVector3 )
				{
					GetMarineBrain()->SetAimDirection( xLookAtDir );
					GetMarineBrain()->SetLookAtPosition( GetMarineBrain()->GetActor()->GetPosition() + xLookAtDir * 10.0f );
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: SetLookAt
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::SetLookAt(Asura_Vector_3 xLookAtDir)
{
	Asura_Vector_3 xLookAt = GetMarineBrain()->GetHumanActor()->GetPosition() + xLookAtDir;
	xLookAt.y -= 1.2f;
	GetMarineBrain()->AimAtPosition(xLookAt);
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: UpdateNearbyGuidData
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::UpdateNearbyGuidData(const float fTimeslice)
{
	static const float s_fUpdatePeriod = 0.125f;

	m_fNearbyUpdateTimer -= fTimeslice;

	if (m_fNearbyUpdateTimer <= 0.0f)
	{
		FindNearbyGuidData();

		m_fNearbyUpdateTimer += s_fUpdatePeriod;
	}
	else
	{
		// We only need update the data if we've not (re)found it this update.
		const Asura_Vector_3& xOurPosition = GetMarineBrain()->GetHumanActor()->GetPosition();

		for (u_int i = 0; i < m_uNumNearbyGuids; i += 1)
		{
			const AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity(m_axNearbyGuidData[i].m_uGuid);
			
			if (pxActor)
			{
				m_axNearbyGuidData[i].m_xToGuid		= pxActor->GetPosition() - xOurPosition;
				m_axNearbyGuidData[i].m_fRangeSq	= m_axNearbyGuidData[i].m_xToGuid.MagnitudeSquared();
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: FindNearByGuidData
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::FindNearbyGuidData(void)
{
	const AVP_ServerActor_Human*	pxOurServerActor	= GetMarineBrain()->GetHumanActor();
	const Asura_Vector_3&			xOurPosition		= pxOurServerActor->GetPosition();
	const Asura_Vector_3&			xOurVelocity		= pxOurServerActor->GetVelocity();
	float fNearbyRange;

	if (m_bShouldStop)
	{
		fNearbyRange = m_fNearbyBaseRange + 6.0f * 2.5f;
	}
	else
	{
		// Give us the base range plus 2 seconds of leeway.
		fNearbyRange = m_fNearbyBaseRange + xOurVelocity.Magnitude() * 2.5f;
	}

	// I'm sure there's a better way of doing this, but this'll do for testing purposes.
	m_uNumNearbyGuids = 0;

	for (Asura_Hashed_List_ConstIt<AVP_ServerEntity_Actor> xActorIter(AVP_ServerEntity_Actor::GetList()); !xActorIter.Done() && m_uNumNearbyGuids < MAX_NEARBY_GUIDS; xActorIter.Next())
	{
		const AVP_ServerEntity_Actor* pxActor = xActorIter.GetCurrent();

		// if the actor pointer isn't valid, or if it is our actor or the actor is dead or inactive then move onto the next actor
		if ((!pxActor) || (pxActor->GetGuid() == pxOurServerActor->GetGuid()) || (pxActor->IsDead()) || (!pxActor->IsEntityActive()))
		{
			continue;
		}

		Asura_Vector_3 xToActor			= pxActor->GetPosition() - xOurPosition;
		float          fActorDistSq		= xToActor.MagnitudeSquared();

		if (fActorDistSq < (fNearbyRange * fNearbyRange))
		{
			Asura_Vector_3	xMoveDirection	= GetMarineBrain()->GetMovementDirection();
			float			fDot			= xMoveDirection * xToActor;

			if (fDot > 0.0f)
			{
				m_axNearbyGuidData[m_uNumNearbyGuids].m_uGuid		= pxActor->GetGuid();
				m_axNearbyGuidData[m_uNumNearbyGuids].m_xToGuid		= xToActor;
				m_axNearbyGuidData[m_uNumNearbyGuids].m_fRangeSq	= fActorDistSq;
				++m_uNumNearbyGuids;
			}
		}
	}
}

float 					s_fMBCollisionAvoidOverallScalar     		= 1.2f;
int   					s_iMBCollisionAvoidRangePower        		= 2;
float 					s_fMBCollisionAvoidLocalScalar       		= 2.0f;
float 					s_fMBCollisionAvoidRangeBias         		= 0.0f;
float 					s_fMBCollisionAvoidMinimumAngleDeg   		= 135.0f; // was 45.0f;
bool  					s_bMBCollisionAvoidDoLocalAngleHack  		= false;
bool  					s_bMBCollisionAvoidDoGlobalAngleHack 		= true;
Asura_Guid				s_uMBFocusGuid								= ASURA_GUID_UNREGISTERED;
static u_int			s_uMBNumAvoids								= 0;
static Asura_Vector_3	s_axMBAvoidVectors[8];
static Asura_Vector_3	s_axMBAvoidPositions[8];

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: CalculateCollisionAvoidanceSteering
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::CalculateCollisionAvoidanceSteering(Asura_Vector_3 &xFollowPoint, Asura_Vector_3 &xSteering)
{
	if (s_uMBFocusGuid == GetProjectBrain()->GetOwnerGuid())
	{
		s_uMBNumAvoids = 0;
	}

	AVP_ServerActor_Human*	pxOurHuman = GetMarineBrain()->GetHumanActor();
	const Asura_Matrix_3x3& xOurOrientation = pxOurHuman->GetOrientation();
	const Asura_Vector_3    xOurNormal( -pxOurHuman->GetCurrentGravityDirection() );
	Asura_Vector_3			xNormalisedFollowPointSteering = xFollowPoint;

	xNormalisedFollowPointSteering.Normalise();

	xSteering = Asura_Maths::ZeroVector3;

	for (u_int uNearbyGuid = 0; uNearbyGuid < m_uNumNearbyGuids; ++uNearbyGuid)
	{
		float fRangeSqrt = Asura_Maths::Sqrt(m_axNearbyGuidData[uNearbyGuid].m_fRangeSq);

		const float    fRange     = Asura_Maths::Max(fRangeSqrt + s_fMBCollisionAvoidRangeBias, 1.0f );
		Asura_Vector_3 xFromVector = -m_axNearbyGuidData[uNearbyGuid].m_xToGuid;

		// We need to resolve the from vector into our plane using our normal.
		Asura_Vector_3 xPlaneVector = xFromVector ^ xOurNormal;
		xPlaneVector = xOurNormal ^ xPlaneVector;
		xPlaneVector.Normalise();

		if (s_bMBCollisionAvoidDoLocalAngleHack)
		{
			// Make sure the from vector meets the follow point steering at a minimum angle,
			// so that we don't end up with sudden movements when meeting an obstacle head on.
			const float fMinimumAngle    = Asura_Maths::DegToRad(s_fMBCollisionAvoidMinimumAngleDeg);
			const float fMinimumAngleDot = Asura_Maths::Cosine(fMinimumAngle);
			const float fDot = -xPlaneVector * xNormalisedFollowPointSteering;
			if (fDot > fMinimumAngleDot)
			{
				// Work out the direction of the rotation using the plane vector in our local space.
				const Asura_Vector_3 xLocalPlaneVector = xOurOrientation.ApplyTranspose(xPlaneVector);

				// Rotate our backward direction by the required amount (in the required direction).
				const Asura_Vector_3 xOurUp(xOurOrientation.e12, xOurOrientation.e22, xOurOrientation.e32);
				Asura_Quat xRotationQuat;
				xRotationQuat.FromRotation(-Asura_Maths::Sign(xLocalPlaneVector.x) * fMinimumAngle, xOurUp);
				xPlaneVector = xRotationQuat.RotateVector(-xNormalisedFollowPointSteering);
			}
		}

		xSteering += xPlaneVector * (s_fMBCollisionAvoidLocalScalar / static_cast<float>(pow(fRange, s_iMBCollisionAvoidRangePower)));

		if (s_uMBFocusGuid == GetProjectBrain()->GetOwnerGuid())
		{
			s_axMBAvoidVectors[s_uMBNumAvoids] = xPlaneVector * (s_fMBCollisionAvoidLocalScalar / static_cast<float>(pow(fRange, s_iMBCollisionAvoidRangePower)));
			s_axMBAvoidPositions[s_uMBNumAvoids] = AVP_ServerEntity_Actor::FindEntity(m_axNearbyGuidData[uNearbyGuid].m_uGuid)->GetPosition();
			++s_uMBNumAvoids;
		}
	}

	if (m_uNumNearbyGuids > 0)
	{
		xSteering /= static_cast<float>(m_uNumNearbyGuids);
	}
	
	xSteering *= s_fMBCollisionAvoidOverallScalar;

	if (s_bMBCollisionAvoidDoGlobalAngleHack)
	{
		Asura_Vector_3 xGlobalPlaneVector = xSteering;
		float          fGlobalPlaneVector = xGlobalPlaneVector.Normalise();

		// Make sure the from vector meets the follow point steering at a minimum angle,
		// so that we don't end up with sudden movements when meeting an obstacle head on.
		const float fMinimumAngle    = Asura_Maths::DegToRad(s_fMBCollisionAvoidMinimumAngleDeg);
		const float fMinimumAngleDot = Asura_Maths::Cosine(fMinimumAngle);
		const float fDot = -xGlobalPlaneVector * xNormalisedFollowPointSteering;
		if (fDot > fMinimumAngleDot)
		{
			// Work out the direction of the rotation using the plane vector in our local space.
			const Asura_Vector_3 xLocalPlaneVector = xOurOrientation.ApplyTranspose(xGlobalPlaneVector);

			// Rotate our backward direction by the required amount (in the required direction).
			const Asura_Vector_3 xOurUp(xOurOrientation.e12, xOurOrientation.e22, xOurOrientation.e32);
			Asura_Quat xRotationQuat;
			xRotationQuat.FromRotation(-Asura_Maths::Sign(xLocalPlaneVector.x) * fMinimumAngle, xOurUp);
			xGlobalPlaneVector = xRotationQuat.RotateVector(-xNormalisedFollowPointSteering);
		}

		xGlobalPlaneVector *= fGlobalPlaneVector;
		xSteering = xGlobalPlaneVector;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: ResetMovementDirection
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::ResetMovementDirection(void)
{
	AVP_ServerActor_Human*	pxOurServerActor = GetMarineBrain()->GetHumanActor();

	if (pxOurServerActor)
	{
		GetMarineBrain()->SetMovementDirection(Asura_Maths::ZeroVector3);
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: AreWeOnASurface
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Travel::AreWeOnASurface(void)
{
	AVP_ServerActor_Human*	pxOurServerActor = GetMarineBrain()->GetHumanActor();

	if (pxOurServerActor)
	{
		// Only orient if we are in contact with a surface (looks a bit funny if we turn in mid air)	
		AVP_Actor_PhysicsObjectHandler* pxObjHandler;
		pxObjHandler = pxOurServerActor->GetPhysicsObjectHandler();

		if (pxObjHandler)
		{
			Asura_Physics_Object* pxObject = pxObjHandler->GetPhysicsObject();		

			if (pxObject && !pxObject->IsSupported())
			{
				return false;
			}
			else
			{
				return true;
			}
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: CheckForArrivalAtDestination
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Travel::CheckForArrivalAtDestination(void)
{
	AVP_Tactical_System*		pxTacticalSystem		= GetMarineBrain()->GetTacticalSystem();
	AVP_ServerActor_Human*		pxOurServerActor		= GetMarineBrain()->GetHumanActor();
	AVP_Navigation::NavData&	xNavData				= GetMarineBrain()->GetNavData();

	if (pxTacticalSystem->isUsingAcurateDistanceCheck())
	{
		u_short uStartWP = 0;

		if (xNavData.m_usRouteSegmentIndex < xNavData.m_usRouteLength - 1)
		{
			Asura_Vector_3	xNearestWaypointPos			= Asura_AI_Navigation_System::GetWaypoint(xNavData.m_pusRoute[xNavData.m_usRouteSegmentIndex])->m_xPosition;
			Asura_Vector_3	xNextWaypointPos			= Asura_AI_Navigation_System::GetWaypoint(xNavData.m_pusRoute[xNavData.m_usRouteSegmentIndex + 1])->m_xPosition;
			Asura_Vector_3	xActorToNearestDirection	= xNearestWaypointPos - pxOurServerActor->GetPosition();
			Asura_Vector_3	xActorToNextDirection		= xNextWaypointPos - pxOurServerActor->GetPosition();
			float			fActorDot					= xActorToNearestDirection * xActorToNextDirection;

			if (fActorDot < 0.0f)
			{
				uStartWP = xNavData.m_usRouteSegmentIndex + 1;
			}
		}
		else
		{
			uStartWP = xNavData.m_usRouteSegmentIndex;
		}

		float fDistanceToRouteEnd = 0.0f;

		if ( GetMarineBrain()->GetTacticalSystem()->IsPerformingAction() )
		{
			fDistanceToRouteEnd = AVP_Navigation::CalculateRouteDistance(xNavData, pxOurServerActor->GetPosition(), uStartWP);
		}
		else
		{
			fDistanceToRouteEnd = AVP_Navigation::CalculateRouteDistance(xNavData, m_xFollowPoint, uStartWP);
		}

		if ( pxTacticalSystem->GetAtDestinationRange() <= 1.0f )
		{
			if ( ( fDistanceToRouteEnd * fDistanceToRouteEnd ) < 0.5f )
			{
				pxTacticalSystem->DisableSlowdownOverride();
				return true;
			}
		}
		else
		{
			if ( ( fDistanceToRouteEnd * fDistanceToRouteEnd ) < pxTacticalSystem->GetAtDestinationRange() )
			{
				pxTacticalSystem->DisableSlowdownOverride();
				return true;
			}
		}
	}
	else
	{
		float fDistcanceToDestinationSq = 0.0f;

		if ( GetMarineBrain()->GetTacticalSystem()->IsPerformingAction() )
		{
			fDistcanceToDestinationSq = (pxOurServerActor->GetPosition() - xNavData.m_xDestinationPosition).MagnitudeSquared();
		}
		else
		{
			fDistcanceToDestinationSq = (m_xFollowPoint - xNavData.m_xDestinationPosition).MagnitudeSquared();
		}

		if ( pxTacticalSystem->GetAtDestinationRange() <= 1.0f )
		{
			if ( fDistcanceToDestinationSq < 0.5f )
			{
				pxTacticalSystem->DisableSlowdownOverride();
				return true;
			}
			else
			{
				fDistcanceToDestinationSq = (pxOurServerActor->GetPosition() - xNavData.m_xDestinationPosition).MagnitudeSquared();

				if ( fDistcanceToDestinationSq < 0.5f )
				{
					pxTacticalSystem->DisableSlowdownOverride();
					return true;
				}
			}
		}
		else
		{
			if ( fDistcanceToDestinationSq < pxTacticalSystem->GetAtDestinationRange() )
			{
				pxTacticalSystem->DisableSlowdownOverride();
				return true;
			}
			else
			{
				fDistcanceToDestinationSq = (pxOurServerActor->GetPosition() - xNavData.m_xDestinationPosition).MagnitudeSquared();

				if ( fDistcanceToDestinationSq < pxTacticalSystem->GetAtDestinationRange() )
				{
					pxTacticalSystem->DisableSlowdownOverride();
					return true;
				}
			}
		}
	}

	if (pxTacticalSystem->DoWeNeedToStopMoving())
	{
		pxTacticalSystem->DisableSlowdownOverride();
		pxTacticalSystem->CancelStopMoving();
		m_bFindRoute = false;
		return true;
	}

	if (pxTacticalSystem->DoWeNeedToStopMovingWithDelay() && m_bStopWithDelayTriggered == false)
	{
		m_fRouteStopTimer = Asura_Random::GetFloat( 0.7f, 1.2f );
		m_bStopWithDelayTriggered = true;
		m_bFindRoute = false;
		return false;
	}

	if ( m_fRouteStopTimer > 0.0f )
	{
		m_fRouteStopTimer -= Asura_Timers::GetFrameTime() * Asura_Timers::GetGameTimeScale();

		if ( m_fRouteStopTimer <= 0.0f )
		{
			m_fRouteStopTimer = 0.0f;
			pxTacticalSystem->CancelStopMovingWithDelay();
			pxTacticalSystem->DisableSlowdownOverride();
			m_bStopWithDelayTriggered = false;
			return true;
		}
	}

	return false;
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: CalculateNewRoute
//---------------------------------------------------------------------------
bool AVP_Behaviour_Marine_Travel::CalculateNewRoute(void)
{
	AVP_Tactical_System*	pxTacticalSystem = GetMarineBrain()->GetTacticalSystem();
	AVP_ServerActor_Human*	pxOurServerActor = GetMarineBrain()->GetHumanActor();

	AVP_Navigation::NavData&	xNavData		= GetMarineBrain()->GetNavData();
	AVP_Navigation::TestParams	xTestParams;

	// Fill The navigation data with our desired route information
	xNavData.m_xStartPosition					= pxOurServerActor->GetPosition();
	xNavData.m_xDestinationPosition				= pxTacticalSystem->GetDestination();

	// Fill the test against parameters with the route finding needed information
	xTestParams.m_uPathfinderFlags				= xNavData.m_uPathfinderFlags;
	xTestParams.m_usZone						= xNavData.m_usCurrentZone;
	xTestParams.m_xNormal						= -pxOurServerActor->GetCurrentGravityDirection();

	AVP_Navigation::ResolveToValidPosition(xNavData.m_xDestinationPosition, xTestParams);

	if (pxOurServerActor->GetPosition().ApproximatelyEquals(xNavData.m_xDestinationPosition, 1.0f))
	{
		pxTacticalSystem->SetRouteFindFail(true);
		return false;
	}

//		float fDistanceSq = (xNavData.m_xStartPosition - xNavData.m_xDestinationPosition).MagnitudeSquared();

	// Feed the resolved position back into the tactical system
	pxTacticalSystem->UpdateDestination(xNavData.m_xDestinationPosition);
	m_bFindRoute = false;

	// Find the waypoint id closest to the destination position
	if (!AVP_Navigation::GetNearestWaypointID(xNavData.m_xDestinationPosition, xNavData.m_usDestinationWPID, xTestParams))
	{
		pxTacticalSystem->SetRouteFindFail(true);
		return false;
	}

	if (!Asura_AI_Navigation_System::IsPositionInWaypoint(xNavData.m_xDestinationPosition, xNavData.m_usDestinationWPID))
	{
		pxTacticalSystem->SetRouteFindFail(true);
		return false;
	}

	// Clear out the old route and find the new one.
	xNavData.ResetRoute();

	//if ( !pxTacticalSystem->IsWeightAdjustmentOverriden() )
	{
		// adjust the waypoint weights so that we avoid identical routes.
		AdjustWaypointWeights();
	}

	Asura_Guid uThreatGuid = GetMarineBrain()->GetPrimaryThreatGuid();
	bool bSuccess = false;

	float fWaypointWeightFactor = Asura_AI_Navigation_System::GetWaypointWeightFactor();

	Asura_AI_Navigation_System::SetWaypointWeightFactor( 1.0f );

	if ( uThreatGuid != ASURA_GUID_UNREGISTERED )
	{
		AVP_ServerEntity_Actor* pxThreatActor = AVP_ServerEntity_Actor::FindEntity( uThreatGuid );

		if ( pxThreatActor && pxThreatActor->GetClassification() != AVP_ENTITYCLASS_ALIEN && 
			( pxOurServerActor->GetFormationRole() == AVP_ServerActor_Human::FORMATION_ROLE_ASSAULT || 
			pxOurServerActor->GetFormationRole() == AVP_ServerActor_Human::FORMATION_ROLE_SUPPORT ) )
		{
			AVP_Navigation::HumanoidCombatAStarFunctor xCombatFunc( pxThreatActor->GetPosition() );
			bSuccess = AVP_Navigation::FindRoute(xNavData, &xCombatFunc );
		}
		else
		{
			bSuccess = AVP_Navigation::FindRoute(xNavData);
		}
	}
	else
	{
		bSuccess = AVP_Navigation::FindRoute(xNavData);
	}

	Asura_AI_Navigation_System::SetWaypointWeightFactor( fWaypointWeightFactor );

	// Find a route!
	if ( !bSuccess )
	{
		//if ( !pxTacticalSystem->IsWeightAdjustmentOverriden() )
		{
			// restore the waypoint weights
			RestoreWaypointWeights();
		}

		pxTacticalSystem->SetRouteFindFail(true);
		return false;
	}
	else
	{
		//if ( !pxTacticalSystem->IsWeightAdjustmentOverriden() )
		{
			// restore the waypoint weights
			RestoreWaypointWeights();
		}

		// We have found a route, tell the tactical system, reset the route ended flag and return true!
		pxTacticalSystem->SetRouteFindFail(false);
		pxTacticalSystem->SetHasArrivedAtDestination(false);
		m_fSmoothedPORVelocity = 5.0f;
		m_bRouteEnded = false;
		return true;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: UpdateCurrentWaypointID
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::AdjustWaypointWeights(void)
{
	AVP_ServerActor_Human*			pxHuman = GetMarineBrain()->GetHumanActor();
	AVP_ServerEntity_Squad_Marine*	pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity(pxHuman->GetInitialSquadGuid());

	// If we are not in a squad don't adjust weights
	if (!pxSquad)
	{
		m_bWeightsAltered = false;
		return;
	}

	m_uNumWaypointsAdjusted = 0;

	for (u_int u = 0; u < pxSquad->GetNumSquadMembers(); ++u)
	{
		// Ignore any NULL, Dead and Inactive actors also ignore ourselves.
		AVP_ServerActor_Human* pxMember = AVP_ServerActor_Human::FindEntity(pxSquad->GetMemberGuid(u));

		if (!pxMember || pxMember->GetGuid() == pxHuman->GetGuid() || pxHuman->IsDead() || !pxHuman->IsEntityActive())
			continue;

		if ( pxHuman->GetClassification() == AVP_ENTITYCLASS_MARINE && pxMember->GetClassification() == AVP_ENTITYCLASS_ANDROID )
		{
			continue;
		}

		if ( pxHuman->GetClassification() == AVP_ENTITYCLASS_ANDROID && pxMember->GetClassification() == AVP_ENTITYCLASS_MARINE )
		{
			continue;
		}

		AVP_Brain_Actor* pxActorBrain = pxMember->GetActorBrain();

		if (!pxActorBrain)
			continue;

		AVP_Navigation::NavData& xNavData = pxActorBrain->GetNavData();

		// Count how many waypoints we need to adjust
		m_uNumWaypointsAdjusted += xNavData.m_usRouteLength - xNavData.m_usRouteSegmentIndex;
	}

	if ( m_uNumWaypointsAdjusted > 0 )
	{
		m_pxWaypointsBackup = new u_short[m_uNumWaypointsAdjusted];

		u_int uNumWaypointsProcessed = 0;

		for (u_int u = 0; u < pxSquad->GetNumSquadMembers(); ++u)
		{
			// Ignore any NULL, Dead and Inactive actors also ignore ourselves.
			AVP_ServerActor_Human* pxMember = AVP_ServerActor_Human::FindEntity(pxSquad->GetMemberGuid(u));

			if (!pxMember || pxMember->GetGuid() == pxHuman->GetGuid() || pxHuman->IsDead() || !pxHuman->IsEntityActive())
				continue;

			if ( pxHuman->GetClassification() == AVP_ENTITYCLASS_MARINE && pxMember->GetClassification() == AVP_ENTITYCLASS_ANDROID )
			{
				continue;
			}

			if ( pxHuman->GetClassification() == AVP_ENTITYCLASS_ANDROID && pxMember->GetClassification() == AVP_ENTITYCLASS_MARINE )
			{
				continue;
			}

			AVP_Brain_Actor* pxActorBrain = pxMember->GetActorBrain();

			if (!pxActorBrain)
				continue;

			AVP_Navigation::NavData& xNavData = pxActorBrain->GetNavData();

			// Loop around all waypoints we haven't reached yet in the route
			for (u_int v = xNavData.m_usRouteSegmentIndex; v < xNavData.m_usRouteLength; ++v)
			{
				u_short uWaypointID = xNavData.m_pusRoute[v];

				m_pxWaypointsBackup[uNumWaypointsProcessed] = uWaypointID;
				++uNumWaypointsProcessed;
				Asura_AI_Navigation_System::AddWaypointWeight(uWaypointID, s_ucWaypointWeightAdjustment);
				
				if (!m_bWeightsAltered)
				{
					m_bWeightsAltered = true;
				}
			}
		}

		if ( !m_bWeightsAltered )
		{
			delete [] m_pxWaypointsBackup;
			m_pxWaypointsBackup = NULL;
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: UpdateCurrentWaypointID
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::RestoreWaypointWeights(void)
{
	if (m_bWeightsAltered)
	{
		// Restore waypoint weights we have adjusted
		for (u_int u = 0; u < m_uNumWaypointsAdjusted; ++u)
		{
			Asura_AI_Navigation_System::RemoveWaypointWeight(m_pxWaypointsBackup[u], s_ucWaypointWeightAdjustment);
		}

		// Delete our array
		delete [] m_pxWaypointsBackup;
		m_pxWaypointsBackup = NULL;
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: UpdateCurrentWaypointID
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::UpdateCurrentWaypoint(void)
{
	AVP_ServerActor_Human* pxOurServerActor = GetMarineBrain()->GetHumanActor();

	if (pxOurServerActor)
	{
		AVP_Navigation::NavData& xNavData	= GetMarineBrain()->GetNavData();
		xNavData.m_xStartPosition			= pxOurServerActor->GetPosition();

		AVP_Navigation::TestParams xTestParams;
		xTestParams.m_uPathfinderFlags = xNavData.m_uPathfinderFlags;
		xTestParams.m_usZone           = xNavData.m_usCurrentZone;

		AVP_Navigation::UpdateCurrentWaypointID(pxOurServerActor->GetPosition(), xNavData.m_usCurrentWPID, xTestParams);
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: GetDistanceToStop
//---------------------------------------------------------------------------
float AVP_Behaviour_Marine_Travel::GetDistanceToStop(void)
{
	AVP_ServerActor_Human&			xOurActor			= *GetMarineBrain()->GetHumanActor();
	AVP_Tactical_System*			pxTactical			= GetMarineBrain()->GetTacticalSystem();
	AVP_Human_AIAnimStateMachine*	pxHumanStateMachine	= static_cast<AVP_Human_AIAnimStateMachine*>(xOurActor.GetAIAnimStateMachine());

	if (pxTactical->ShouldMoveCrouched())
	{
		return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::HUMAN_STATE_CROUCH_MOVE, AVP_Human_AIAnimStateMachine::HUMAN_STATE_CROUCH_IDLE);
	}
	else
	{
		switch(pxTactical->GetMovementSpeed())
		{
			case AVP_Navigation::MM_SLOW:
			{
				return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_SLOW, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE);
			}

			case AVP_Navigation::MM_FAST:
			{
				return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_FAST, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE);
			}

			default:
			case AVP_Navigation::MM_MEDIUM:
			{
				return pxHumanStateMachine->GetDistanceRequired(AVP_Human_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_MEDIUM, AVP_Human_AIAnimStateMachine::ACTOR_STATE_IDLE);
			}
		}
	}

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: SetStateIdle
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::SetStateIdle()
{	
	if ( GetMarineBrain()->IsInjured() )
	{
		return;
	}

	bool bIdleRequested = false;
	
	AVP_ServerActor_Human* pxOurServerActor = GetMarineBrain()->GetHumanActor();

 	if ( pxOurServerActor )
	{
		AVP_Human_AIAnimStateMachine* pxHumanStateMachine = static_cast< AVP_Human_AIAnimStateMachine* >( pxOurServerActor->GetAIAnimStateMachine() );

		if ( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS )
		{
			m_bWasAggressive = false;

			if ( GetMarineBrain()->GetKnowledgeSystem()->EverBeenAggressive() && !pxHumanStateMachine->IsCrouchIdle() && !pxHumanStateMachine->IsAlertIdle() )
			{
				bIdleRequested = true;

				if ( pxHumanStateMachine->IsMovingSlowState() || pxHumanStateMachine->IsMovingMediumState() || pxHumanStateMachine->IsMovingFastState() )
				{
					pxHumanStateMachine->RequestAlertIdle();
				}
				else if ( !pxHumanStateMachine->IsAlertIdle() && !pxHumanStateMachine->IsMovingSlowState() && !pxHumanStateMachine->IsMovingMediumState() && !pxHumanStateMachine->IsMovingFastState() )
				{
					pxHumanStateMachine->ForceState( AVP_Human_AIAnimStateMachine::HUMAN_STATE_ALERT_IDLE );
				}
			}
			else if ( !GetMarineBrain()->GetKnowledgeSystem()->EverBeenAggressive() && !pxHumanStateMachine->IsIdle() && !pxHumanStateMachine->IsCrouchIdle() )
			{
				bIdleRequested = true;
				RequestIdle();
				return;
			}

			m_bTurnRequested = false;
		}
		else if ( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS )
		{
			if ( GetMarineBrain()->GetTacticalSystem()->IsAlert() || m_bWasAggressive || GetMarineBrain()->GetTacticalSystem()->IsAllowedToReact() )
			{
				if ( GetMarineBrain()->GetTacticalSystem()->IsAllowedToReact() == false )
				{
					bIdleRequested = true;
					pxHumanStateMachine->RequestIdle();
				}
				else if ( !pxHumanStateMachine->IsAlertIdle() && !pxHumanStateMachine->IsCrouchIdle() && GetMarineBrain()->GetTacticalSystem()->IsAllowedToReact() == true )
				{
					bIdleRequested = true;
					pxHumanStateMachine->RequestAlertIdle();
					return;
				}
			}

			m_bTurnRequested = false;
		}
		else if ( GetMarineBrain()->GetKnowledgeSystem()->GetAlertLevel() == AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS )
		{
			m_bWasAggressive = true;

			if ( GetMarineBrain()->DoesHaveThreat() && GetMarineBrain()->GetKnowledgeSystem()->GetBestThreatKnowledge().m_uTypeInformation == AVP_Sense_Vision::TYPE_INFO_CAN_SEE )
			{
				bIdleRequested = true;
				pxHumanStateMachine->ForceState( AVP_Actor_AIAnimStateMachine::ACTOR_STATE_IDLE );
			}
			else if ( !pxHumanStateMachine->IsAggressiveIdle() && !pxHumanStateMachine->IsCrouchIdle() && GetMarineBrain()->GetTacticalSystem()->IsAllowedToReact() == true && !GetMarineBrain()->DoesHaveThreat() )
			{
				bIdleRequested = true;
				pxHumanStateMachine->RequestAggressiveIdle();
				m_bTurnRequested = false;
			}
			else if ( !pxHumanStateMachine->IsAggressiveIdle() && !pxHumanStateMachine->IsCrouchIdle() && GetMarineBrain()->DoesHaveThreat() && GetMarineBrain()->GetKnowledgeSystem()->GetBestThreatKnowledge().m_uTypeInformation == AVP_Sense_Vision::TYPE_INFO_CANT_SEE )
			{
				bIdleRequested = true;
				pxHumanStateMachine->RequestAggressiveIdle();
				m_bTurnRequested = false;
			}
			else
			{
				bIdleRequested = true;
				RequestIdle();
			}
		}
		else
		{
			if ( !pxHumanStateMachine->IsIdle() && !pxHumanStateMachine->IsCrouchIdle() )
			{
				bIdleRequested = true;
				RequestIdle();
				return;
			}
		}

		if ( !bIdleRequested && ( pxHumanStateMachine->IsMoveSlow() || pxHumanStateMachine->IsMoveMedium() || pxHumanStateMachine->IsMoveFast() ) )
		{
			RequestIdle();
		}

		SetUpdateState(IDLE);
	}
}
//-----------------------------------------------
Asura_Angle AVP_Behaviour_Marine_Travel::CalculateAngleToTurn()
{
	AVP_ServerActor_Human*		pxOurServerActor	= GetMarineBrain()->GetHumanActor();
	const Asura_Matrix_3x3&		xOri				= pxOurServerActor->GetOrientation();
	Asura_Vector_3				xFwdDir( xOri.e13, xOri.e23, xOri.e33 );
	Asura_Vector_3				xUpDir( xOri.e12, xOri.e22, xOri.e32 );
	Asura_Vector_3				xAimDir( GetMarineBrain()->GetAimDirection() );

	float fDot = xFwdDir * xAimDir;

	return Asura_Maths::ArcCosine( fDot );


// 	Asura_Vector_3		xDirToAimPoint			= m_xAimFor - m_xTurretPivotPos;
// 	float				fDistOrigToPlane		= 0.0f;
// 	float				fDistToPlane			= 0.0f;
// 	bool				bIsToTheLeft;
// 
// 	//	Normalise our direction vector
// 	xDirToAimPoint.Normalise();
// 	//	Making an imaginary plane along the thanks forward vector, with normal set to the left vector
// 	fDistOrigToPlane = m_xTurretPivotLeft * m_xTurretPivotPos;
// 	//	Calculate the distance to the plane - this will tell us if we are left or right of the Helicopters forward vector
// 	fDistToPlane = fDistOrigToPlane - (m_xTurretPivotLeft * m_xAimFor);
// 
// 	//	Is the actor to the left or right?
// 	if (fDistToPlane <= 0.0f)
// 		bIsToTheLeft = true;
// 	else
// 		bIsToTheLeft = false;
// 
// 	//	Ignore the Y component
// 	Asura_Vector_3 xTempCannonForwards = m_xTurretPivotForward;
// 	xTempCannonForwards.y = 0.0f;
// 	xDirToAimPoint.y = 0.0f;
// 
// 	//	Always normalise after altering direction vectors
// 	xTempCannonForwards.Normalise();
// 	xDirToAimPoint.Normalise();
// 
// 	//	Calculate the cos of the angle between the pivot forward vector and the direction too the aim for position
// 	Asura_Angle fCosOfAngleToAimTo = (xTempCannonForwards * xDirToAimPoint);
// 
// 	//	The cos of angle value won't be linear, and we need it linear
// 	m_fTime = Asura_Maths::ArcCosine(fCosOfAngleToAimTo) / Asura_Maths::Deg180;

}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: SetStateMoving
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::SetStateMoving()
{
	RequestMovement();
	SetUpdateState(MOVING);
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: RequestIdle
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::RequestMovement()
{
	AVP_ServerActor_Human*			pxOurServerActor	= GetMarineBrain()->GetHumanActor();
	AVP_Tactical_System*			pxTactical			= GetMarineBrain()->GetTacticalSystem();
	AVP_Human_AIAnimStateMachine*	pxHumanStateMachine	= static_cast< AVP_Human_AIAnimStateMachine* >( pxOurServerActor->GetAIAnimStateMachine() );

	if (pxTactical->IsUsingCover())
	{
		return;
	}

	AVP_ClientActor_Human* pxHuman = AVP_ClientActor_Human::FindEntity( pxOurServerActor->GetGuid() );

	if ( pxHuman )
	{
		if ( pxHuman->GetMovementSystem() )
		{
			if ( pxHuman->GetMovementSystem()->GetCurrentMove() )
			{
				return;
			}
		}
	}

	if (!m_bShouldStop)
	{
		if (pxTactical->ShouldMoveCrouched())
		{
			pxHumanStateMachine->RequestCrouchMove();
		}
		else
		{
			// check the distance to our destination
			float fDistanceSq = (pxOurServerActor->GetPosition() - pxTactical->GetDestination()).MagnitudeSquared();

			if ( ( fDistanceSq < 36.0f && !pxTactical->IsSlowdownOverridden() ) || ( m_bStopWithDelayTriggered ) )
			{
				pxHumanStateMachine->RequestMoveSlow();
			}
			else if ( ( pxTactical->GetMovementSpeed() == AVP_Navigation::MM_FAST ) && ( fDistanceSq < 36.0f ) )
			{
				pxTactical->SetMovementSpeed(AVP_Navigation::MM_MEDIUM);
			}
			else
			{
				switch( pxTactical->GetMovementSpeed() )
				{
					case AVP_Navigation::MM_SLOW:
					{
						if (!pxHumanStateMachine->IsMoveSlow())
						{
							pxHumanStateMachine->RequestMoveSlow();
						}
						break;
					}

					default:
					case AVP_Navigation::MM_MEDIUM:
					{
						if (!pxHumanStateMachine->IsMoveMedium())
						{
							pxHumanStateMachine->RequestMoveMedium();
						}
						break;
					}

					case AVP_Navigation::MM_FAST:
					{
						if (!pxHumanStateMachine->IsMoveFast())
						{
							pxHumanStateMachine->RequestMoveFast();
						}
						break;
					}
				}
			}
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: RequestIdle
//---------------------------------------------------------------------------
void AVP_Behaviour_Marine_Travel::RequestIdle()
{
	AVP_ServerActor_Human*	pxOurServerActor	= GetMarineBrain()->GetHumanActor();
	AVP_Tactical_System*	pxTactical			= GetMarineBrain()->GetTacticalSystem();

	if (!pxTactical->IsUsingCover())
	{
		if (!pxOurServerActor->GetAIAnimStateMachine()->IsIdle())
		{
			pxOurServerActor->GetAIAnimStateMachine()->RequestIdle();
		}

		if (m_bSpeedAltered)
		{
			m_bSpeedAltered = false;
		}
	}
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Marine_Travel :: RenderDebugInfo
//---------------------------------------------------------------------------
#ifndef ASURA_NO_DEBUG_OUTPUT


DEBUGVAR bool g_bRenderTravelInfo = true;
DEBUGVAR bool g_RenderSpeedAdjustment = true;

void AVP_Behaviour_Marine_Travel::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	if (GetMarineBrain()->GetTacticalSystem()->IsOnGameScene())
	{
		return;
	}

	if (g_bRenderTravelInfo)
	{
		if (GetUpdateState() == MOVING)
		{
			// Route Traversal Positions
			Asura_DebugLineRender::AddSphere(m_xPositionOnRoute,				0.5f, 0xffffffff, 3, 2);

			Asura_Vector_3 xOutputPos = Asura_Maths::ZeroVector3;

			if (g_RenderSpeedAdjustment)
			{
				if (m_xFollowPoint != Asura_Maths::ZeroVector3)
				{
					xOutputPos = m_xFollowPoint;

					if (Asura_Camera::ProjectWorldCoordsToScreenSpace(xOutputPos, true) || xOutputPos.z > 0.0f)
					{
						Asura_DebugLineRender::AddSphere(m_xFollowPoint, 0.5f, 0xffffff00, 3, 2);
						Asura_Font_System::Print(xOutputPos.x, xOutputPos.y, 0.5f, 0xffffff00, ASURA_TEXTFORMAT_CENTREJUSTIFIED, "Follow Point");
					}
				}

				if ( m_xSpeedAdjustmentTrackingPoint != Asura_Maths::ZeroVector3 && m_xSpeedAdjustmentTrackingPoint != m_xPositionOnRoute )
				{
					xOutputPos = m_xSpeedAdjustmentTrackingPoint;

					if (Asura_Camera::ProjectWorldCoordsToScreenSpace(xOutputPos, true) || xOutputPos.z > 0.0f)
					{
						Asura_DebugLineRender::AddSphere(m_xSpeedAdjustmentTrackingPoint, 0.5f, 0xff00ffff, 3, 2);
						Asura_Font_System::Print(xOutputPos.x, xOutputPos.y, 0.5f, 0xff00ffff, ASURA_TEXTFORMAT_CENTREJUSTIFIED, "Speed Adjustment Point");
					}
				}

				if (m_xAvoidancePosition != Asura_Maths::ZeroVector3)
				{
					xOutputPos = m_xAvoidancePosition;

					if (Asura_Camera::ProjectWorldCoordsToScreenSpace(xOutputPos, true) || xOutputPos.z > 0.0f)
					{
						Asura_DebugLineRender::AddSphere(m_xAvoidancePosition, 0.5f, 0xffff0000, 3, 2);
						Asura_Font_System::Print(xOutputPos.x, xOutputPos.y, 0.5f, 0xffff0000, ASURA_TEXTFORMAT_CENTREJUSTIFIED, "Collision Avoidance Point");
					}
				}

				const AVP_Navigation::NavData &xNavData = GetMarineBrain()->GetNavData();

				if ( xNavData.m_xDestinationPosition != Asura_Maths::ZeroVector3 )
				{
					Asura_DebugLineRender::AddSphere(xNavData.m_xDestinationPosition, 0.5f, 0xffff0000, 5, 5);
				}
			}
		}

		u_int uState = GetMarineBrain()->GetHumanActor()->GetAIAnimStateMachine()->GetCurrentState();

		Asura_Font_System::Print
		(
			GetProjectBrain()->GetDebugOutputPos().x,
			GetProjectBrain()->GetDebugOutputPos().y,
			0.5f, 
			0xffff8888,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
			AVP_Human_AIAnimStateMachine::GetStateDescriptionStatic(uState)
		);

		GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
	}

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0xffff8888,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
		"SpeedDiffTimer = %0.2f", m_fSpeedDiffTimer
	);

	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
}
#endif

/// End Of File 'AVP_behaviour_Marine_Travel.cpp'
