//---------------------------------------------------------------------------
// File			: AVP_Behaviour_Marine_Travel.h
// Created		: 30 / 10 / 07
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//---------------------------------------------------------------------------

// BEGIN FILE WRAPPER ///////////////////////////////////////
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_BEHAVIOUR_MARINE_TRAVEL_H__
#define __AVP_BEHAVIOUR_MARINE_TRAVEL_H__ 1
// END FILE WRAPPER ///////////////////////////////////////////

// INCLUDES ///////////////////////////////////////////////////////////
#include "Behaviour/Axon_Project_Behaviour.h"
#include "Core/AVP_Navigation.h"


// FORWARD DECLARATIONS ///////////////////////////////////
class AVP_Brain_Actor_Marine;
class AVP_Behaviour_Marine;

//---------------------------------------------------------------------------------------------------
// Class		: AVP_Behaviour_Marine_Travel
// Description	: Child Behaviour for the Marine AI Actors to make them move about the environment
//---------------------------------------------------------------------------------------------------
class AVP_Behaviour_Marine_Travel : public Axon_Project_Behaviour
{
	typedef Axon_Project_Behaviour PARENT;

public:

	// Update State Enum
	enum TravelUpdateState
	{
		UNSET,
		MOVING,
		IDLE,
		MAX_STATES
	};

	// Construction / Deconstruction
	AVP_Behaviour_Marine_Travel										(AVP_Brain_Actor_Marine* pxBrain);
	virtual ~AVP_Behaviour_Marine_Travel							(void)								{}

	// Public Member Functions...
	TravelUpdateState			GetUpdateState						(void) const						{return m_eUpdateState;}
	bool						HasArrivedAtDestination				(void);

protected:
	virtual bool 				Activate							(void);
	virtual void 				Deactivate							(void);
	virtual void 				Pause								(void);
	virtual void 				Unpause								(void);
	virtual void 				Update								(const float fTimeslice);
	virtual void				PerFrameUpdate						(const float fTimeslice);
#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void				RenderDebugInfo						(void);
#endif

	void CalculateLengthRemainingOnLink();

	enum {MAX_NEARBY_GUIDS = 8};

	struct NearbyGuidData
	{
		Asura_Guid     m_uGuid;
		Asura_Vector_3 m_xToGuid;
		float          m_fRangeSq;

	};


	// Protected Data Access Functions
	AVP_Brain_Actor_Marine*		GetMarineBrain						(void)								{return m_pxMarineBrain;}
	bool						HasNewDestinationBeenRequested		(void)								{return m_bNewDestinationSet;}
	void						NewDestinationHasBeenRequested		(void)								{m_bNewDestinationSet = true;}
	void						NewDestinationRouteFound			(void)								{m_bNewDestinationSet = false;}
	void						SetUpdateState						(TravelUpdateState eNewState)		{m_eUpdateState = eNewState;}

	// Protected Member Functions...
	bool						CalculateNewRoute					(void);
	void						UpdateCurrentWaypoint				(void);
	bool						CheckForArrivalAtDestination		(void);
	void						CalculateAndSetMovementDirection	(const float fTimeStep);
	void						RequestIdle							(void);
	void						RequestMovement						(void);
	bool						AreWeOnASurface						(void);
	void						CheckForNewRoute					(const float fTimeslice);
	void						DoMoving							(const float fTimeslice);
	bool						AreWeStuck							(const float fTimeslice);
	void						SetStateIdle						(void);
	void						SetStateMoving						(void);
	void						ResetMovementDirection				(void);
	void						CalculateCollisionAvoidanceSteering	(Asura_Vector_3 &xFollowPoint, Asura_Vector_3 &xSteering);
	void						UpdateNearbyGuidData				(const float fTimeslice);
	void						FindNearbyGuidData					(void);
	void						ForceMovementDirectionIfNeeded		(const Asura_Vector_3& xNewMoveDir);
	void						CheckForSquadLeaderArrival			(void);
	Asura_Vector_3				DoPhysicsNonSupportedMovement		(void);
	Asura_Vector_3				UpdateMovment						(Asura_Vector_3 xDirection);
	void						SetLookAt							(Asura_Vector_3 xLookAtDir);
	void						UpdateNavData						(void);
	float						GetDistanceToStop					(void);
	void						AlterSpeedIfNeeded					(void);
	void						VerifyCollisionAvoidance			(Asura_Vector_3 xAvoidanceSteering);
	bool						PositionInLastWaypoint				(u_short usNearestRouteIndex);
	void						AdjustWaypointWeights				(void);
	void						RestoreWaypointWeights				(void);
	Asura_Angle					CalculateAngleToTurn				(void);
	
	// Protected Member Variables...
	AVP_Brain_Actor_Marine*		m_pxMarineBrain;
	TravelUpdateState			m_eUpdateState;
	Asura_Vector_3				m_xDestination;
	bool						m_bNewDestinationSet;
	bool						m_bRouteEnded;
	bool						m_bForceMovementDirection;
	
	// Actor avoidance variables
	float						m_fNearbyUpdateTimer;						// Nearby guid stuff for collision avoidance test.
	float						m_fNearbyBaseRange;							// The nearby range is based on speed and the character's bounding box dimensions.
	NearbyGuidData				m_axNearbyGuidData[MAX_NEARBY_GUIDS];
	u_int						m_uNumNearbyGuids;
	Asura_Vector_3				m_xNearestPointOnRoute;

	Asura_Vector_3				m_xPositionOnRoute;
	Asura_Vector_3				m_xPreviousPositionOnRoute;
	float						m_fSmoothedPORVelocity;
	Asura_Vector_3				m_xNearestPositionOnRoute;
	bool						m_bSpeedAltered;
	Asura_Vector_3				m_xFollowPoint;
	Asura_Vector_3				m_xSpeedAdjustmentTrackingPoint;
	Asura_Vector_3				m_xAvoidancePosition;
	bool						m_bShouldStop;
	bool						m_bWeightsAltered;

	u_short*					m_pxWaypointsBackup;
	u_int						m_uNumWaypointsAdjusted;
	float						m_fSpeedDiffTimer;
	float						m_fLengthRemainingOnLink;
	float						m_fPhysicsNotSupportedTimer;

	bool						m_bFindRoute;
	float						m_fRouteDelayTimer;
	float						m_fRouteStopTimer;
	bool						m_bStopWithDelayTriggered;

	bool						m_bWasAggressive;
	bool						m_bTurnRequested;
	float						m_fTurnTimer;

	static float				s_fLookAheadDistance;
	static float				s_fAtDestinationRangeSq;
	static float				s_fMinRouteLengthSq;
	static u_char				s_ucWaypointWeightAdjustment;

};

#endif //__AVP_BEHAVIOUR_MARINE_TRAVEL_H__

/// End Of File 'AVP_Behaviour_Marine.h'
