//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predalien.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BEHAVIOUR_PREDALIEN_H__
#define __AVP_BEHAVIOUR_PREDALIEN_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// avp axon
#include "Behaviour/Axon_Project_Behaviour.h"

// asura
#include "Core/Asura_LinkList.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Brain_Actor_Predalien;
class AVP_ServerNode_VantagePoint_Predalien;
class AVP_ServerActor_Vertebrate;
class AVP_Behaviour_Predalien_Movement;

namespace AVP_Navigation
{
	class AStarFunctorBase;
}
//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predalien
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Behaviour_Predalien : public Axon_Project_Behaviour
{
    typedef Axon_Project_Behaviour PARENT;

public:
    AVP_Behaviour_Predalien( AVP_Brain_Actor_Predalien* pxBrain );
	void KnockDown();

protected:
    virtual void Deactivate();
    virtual void PerFrameUpdate( const float fTimeslice );
	virtual bool HandleMessage( const Asura_Message* pxMessage );

#ifndef ASURA_NO_DEBUG_OUTPUT
    virtual void RenderDebugInfo();
#endif

    AVP_Brain_Actor_Predalien* GetPredalienBrain() const;

private:

	
    enum State
    {
        UNSET,
		IDLE,
		RUNNING_IN,
		STAND_CLOSE_MELEE,
		WALK_CLOSE_MELEE,
		TAIL_CLOSE_MELEE,
		LEAP_ATTACK, 
		FLINCHED,
		RUNNING_TO_VANTAGE_POINT,
		AT_VANTAGE_POINT,
		JUMP_DOWN_ATTACK,
		KNOCKED_DOWN,
		BEING_GRAPPLED,
		TEST_BEHAVIOUR,
        NUM_STATES
    };

    void SwitchToState( const State eNextState );

	const AVP_ServerActor_Vertebrate* GetTarget() const;
	Asura_Guid GetTargetGuid() const;
	void PlanMeleePath();
	void PlanVantagePath();
	bool PlanPathToDestination(AVP_Navigation::AStarFunctorBase* xFunctor);
	void UpdateMoveToMelee(float fTimeslice);
	void UpdateMoveToVantagePoint(float fTimeslice);
	void UpdateAtVantagePoint(float fTimeslice);

	void IgnoreTarget();
	void CanMelee(const AVP_ServerEntity_Actor* pxTarget, bool& bFromFront, bool& bFromAnyDirection, bool& bUseTail) const;
	void RequestMelee(Asura_Hash_ID uMeleeType, bool bStopBeforeMelee );
	void RequestKnockedDownMove();
	void AimAtTarget(bool m_bUpdateMoveToo);
	void LateHandleMessage();
	bool CanDoCloseStandMelee(float fDistanceToTarget) const;
	bool CanDoCloseWalkMelee(float fDistanceToTarget, const AVP_Behaviour_Predalien_Movement *pxBehaviour) const;
	bool CanDoLeapAttack(float fDistanceToTarget, const Asura_Vector_3& xToTargetDirection) const;
	bool CanDoJumpDownAttack(const Asura_Vector_3& xTargetPosition) const;
	bool SelectVantagePoint(const Asura_Vector_3& xOurPosition, const Asura_Vector_3& xTargetPosition);
	bool IsVantagePointValid( const Asura_Vector_3& xVPPosition, const Asura_Vector_3& xTargetPosition, bool bIgnoreHorizontalDistance);
	void IgnoreVentagePoint(Asura_Guid uGuid);
	bool IsVentagePointIgnored( Asura_Guid uGuid ) const;
	void UpdateIgnoreList();

private:

	State m_eCurrentState;
    float m_fTimeInState;

	float m_fTimeToIgnoreTarget;
	Asura_Vector_3 m_xDestinationPosition;
	Asura_LinkList< int > m_xDelayedMsgList;
	Asura_Guid m_uVantagePointGuid;
	float m_fTargetHealth;
	float m_fTimeSinceHealthChanged;
	float m_fTimeSinceStalking;
	bool m_bWasKnockedDown;

	struct  IgnoreVPData
	{
		Asura_Guid m_uGuid;
		float m_fTimeToStopIgnore;
	};
	Asura_LinkList<IgnoreVPData> m_xIgnoredVentagePoints;

};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BEHAVIOUR_PREDALIEN_H__
