#if 0
//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predator_PointAndShoot.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Predator/AVP_Behaviour_Predator_PointAndShoot.h"

// asura
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ClientActor_Predator.h"
#include "Actor/AVP_Actor_DynamicStat.h"
#endif

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Predator.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"

// avp
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"

static const Asura_Vector_3 g_xPredatorPlasmaCasterTargetingOffset( 0.0f, -2.1f, 0.0f );

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predator_PointAndShoot
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Predator_PointAndShoot::AVP_Behaviour_Predator_PointAndShoot( AVP_Brain_Actor_Predator* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_PREDATOR_POINTANDSHOOT )
, m_fCrouchPeriod( 0.0f )
, m_fStandPeriod( 0.0f )
, m_eChargeState( DISABLED )
, m_fShootTimer( 0.0f )
, m_xLookDirection( Asura_Maths::ZeroVector3 )
, m_fLookPeriod( GetNewLookPeriod() )
, m_fLookTimer( 0.0f )
, m_uTargetGuid( ASURA_GUID_UNREGISTERED )
, m_xTargetPosition( Asura_Maths::ZeroVector3 )
{
	SetCurrentState( UNSET );
}

bool AVP_Behaviour_Predator_PointAndShoot::HandleMessage( const Asura_Message* pxMessage )
{
	return PARENT::HandleMessage( pxMessage );
}

bool AVP_Behaviour_Predator_PointAndShoot::Activate()
{
	const bool bOk = PARENT::Activate();

	EquipPlasmaCaster();

	return bOk;
}

void AVP_Behaviour_Predator_PointAndShoot::Deactivate()
{
	if ( GetPredatorBrain()->IsActive() )
	{
		GetPredatorBrain()->Decloak();
		GetPredatorBrain()->SetCrouched( false );
		SwitchToChargingState( DISABLED );
	}

	PARENT::Deactivate();
}

void AVP_Behaviour_Predator_PointAndShoot::Update( const float fTimeslice )
{
	PARENT::Update( fTimeslice );

	if ( IsInError() || IsFinished() )
	{
		return;
	}

	switch ( GetCurrentState() )
	{
		case UNSET:
		{
			SwitchToState( STANDING );
			break;
		}

		case STANDING:
		{
			if ( HaveTarget() )
			{
				SwitchToState( TARGETTING );
			}
			else if ( GetStateTimer() > m_fStandPeriod )
			{
				SwitchToState( CROUCHING );
			}

			if ( m_fLookTimer > m_fLookPeriod )
			{
				PickNewLookDirection();
			}

			break;
		}

		case CROUCHING:
		{
			if ( GetStateTimer() > m_fCrouchPeriod )
			{
				if ( HaveTarget() )
				{
					SwitchToState( TARGETTING );
				}
				else
				{
					SwitchToState( STANDING );
				}
			}

			if ( m_fLookTimer > m_fLookPeriod )
			{
				PickNewLookDirection();
			}

			break;
		}

		case TARGETTING:
		{
			if ( !HaveTarget() )
			{
				SwitchToState( STANDING );
			}
			else
			{
				TurnTowardsTarget();
			}

			break;
		}
	}

	switch ( m_eChargeState )
	{
		case CHARGING:
		{
			if ( m_fShootTimer > 2.0f )
			{
				SwitchToChargingState( SHOOT );
			}
			break;
		}

		case SHOOT:
		{
			if ( m_fShootTimer > 2.0f )
			{
				SwitchToChargingState( CHARGING );
			}
			break;
		}

		default:
		case DISABLED:
		{
			break;
		}
	}
}

void AVP_Behaviour_Predator_PointAndShoot::PerFrameUpdate( const float fTimeslice )
{
	PARENT::PerFrameUpdate( fTimeslice );

	UpdateStateTimer( fTimeslice );
	m_fShootTimer += fTimeslice;
	m_fLookTimer += fTimeslice;

	if ( IsInError() || IsFinished() )
	{
		return;
	}

	UpdateTargetDetails();
}

#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	"UNSET",	
	"STANDING",
	"CROUCHING",
	"TARGETTING"
};

static const char* s_aszChargeStateStrings[] =
{
	"DISABLED",
	"CHARGING",
	"SHOOT"
};

void AVP_Behaviour_Predator_PointAndShoot::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "%s: %s",
		s_aszStateStrings[ GetCurrentState() ], s_aszChargeStateStrings[ m_eChargeState ] );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

	const AVP_ClientActor_Predator* pxClient = AVP_ClientActor_Predator::FindEntity(GetPredatorBrain()->GetPredatorActor()->GetGuid());
	const AVP_Actor_DynamicStat* pxEnergy = pxClient->GetDynamicEnergy();

	Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "energy: %.2f", pxEnergy->GetCurrentValue() );
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

	const AVP_Inventory_System* pxSystem = this->GetPredatorBrain()->GetActor()->GetInventorySystem();
	const AVP_Inventory_Group* pxGroup = pxSystem != NULL ? pxSystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	const AVP_InventoryWeapon_RangedWeapon* pxItem = pxGroup != NULL ? static_cast< const AVP_InventoryWeapon_RangedWeapon* >( pxGroup->GetItem( uAVP_INVENTORY_WEAPONHASH_PLASMACASTER ) ) : NULL;
	if ( pxItem != NULL )
	{
		Asura_Font_System::Print( GetProjectBrain()->GetDebugOutputPos().x, GetProjectBrain()->GetDebugOutputPos().y, 0.5f, 0xffffffff,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, "energy in wepaon: %.2f", pxItem->GetCharge() );
		GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
	}
}
#endif

AVP_Brain_Actor_Predator* AVP_Behaviour_Predator_PointAndShoot::GetPredatorBrain() const
{
	return static_cast< AVP_Brain_Actor_Predator* >( GetProjectBrain() );
}

void AVP_Behaviour_Predator_PointAndShoot::UpdateTargetDetails()
{
	m_uTargetGuid = GetPredatorBrain()->GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid();
	if ( m_uTargetGuid != ASURA_GUID_UNREGISTERED )
	{
		const AVP_ServerEntity_Actor* pxActorTarget = AVP_ServerEntity_Actor::FindEntity( m_uTargetGuid );
		if ( pxActorTarget )
		{
			pxActorTarget->GetTargetPosition( m_xTargetPosition );
		}
		else
		{
			m_uTargetGuid = ASURA_GUID_UNREGISTERED;
		}
	}
	else
	{
		Asura_Assert( true, "" );
	}
}

bool AVP_Behaviour_Predator_PointAndShoot::HaveTarget() const
{
	return ( m_uTargetGuid != ASURA_GUID_UNREGISTERED );
}

void AVP_Behaviour_Predator_PointAndShoot::SwitchToState( const State eNextState )
{
	switch ( GetCurrentState() )
	{
		case STANDING:
		{
			break;
		}

		case CROUCHING:
		{
			GetPredatorBrain()->Decloak();
			GetPredatorBrain()->SetCrouched( false );
			break;
		}

		case TARGETTING:
		{
			SwitchToChargingState( DISABLED );
			break;
		}

		case UNSET:
		default:
			break;
	}

	SetCurrentState( eNextState );
	switch ( GetCurrentState() )
	{
		case STANDING:
		{
			m_fStandPeriod = GetNewStandPeriod();
			break;
		}

		case CROUCHING:
		{
			m_fCrouchPeriod = GetNewCrouchPeriod();
			GetPredatorBrain()->Cloak();
			GetPredatorBrain()->SetCrouched( true );
			break;
		}

		case TARGETTING:
		{
			SwitchToChargingState( CHARGING );
			break;
		}

		case UNSET:
		default:
			break;
	}
}

void AVP_Behaviour_Predator_PointAndShoot::SwitchToChargingState( const ChargeState eNextChargeState )
{
	AVP_Inventory_System*		pxInventorySystem	= GetPredatorBrain()->GetActor()->GetInventorySystem();
	AVP_InventoryItem_Weapon*	pxCurrentWeapon		= static_cast<AVP_InventoryItem_Weapon*>(pxInventorySystem ? pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) : NULL);

	switch ( m_eChargeState )
	{
		case CHARGING:
		{
			break;
		}

		case SHOOT:
		{
			break;
		}

		default:
		case DISABLED:
		{
			break;
		}
	}

	m_eChargeState = eNextChargeState;
	m_fShootTimer  = 0.0f;
	switch ( m_eChargeState )
	{
		case CHARGING:
		{
			if ( pxCurrentWeapon )
			{
				pxCurrentWeapon->RequestBeginUsage( AVP_InventoryWeapon_RangedWeapon::UT_CHARGED_SHOT_PRIMARY );
			}
			break;
		}

		case SHOOT:
		{
			if ( pxCurrentWeapon )
			{
				// Usage type required to instruct shooting.
//				pxCurrentWeapon->SetTargetPosition( m_xTargetPosition );
				pxCurrentWeapon->RequestEndUsage( AVP_InventoryWeapon_RangedWeapon::TT_SHOOT );
			}
			break;
		}

		default:
		case DISABLED:
		{
			if ( pxCurrentWeapon )
			{
				// Usage type required to instruct charge dissipation.
				pxCurrentWeapon->RequestEndUsage( AVP_InventoryWeapon_RangedWeapon::TT_ABORT_RESTORE_CHARGE );
			}
			break;
		}
	}
}

void AVP_Behaviour_Predator_PointAndShoot::EquipPlasmaCaster()
{
	AVP_Inventory_System* pxInventorySystem	= GetPredatorBrain()->GetActor()->GetInventorySystem();
	AVP_Inventory_Group*  pxWeaponGroup		= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	if ( pxWeaponGroup )
	{
		pxWeaponGroup->SetCurrentItem( uAVP_INVENTORY_WEAPONHASH_PLASMACASTER, true );
	}
}

void AVP_Behaviour_Predator_PointAndShoot::PickNewLookDirection()
{
	// Pick a new waypoint and look at it.
	const u_int uWP = Asura_Random::GetInt( 0, Asura_AI_Navigation_System::GetNumberOfWaypoints() - 1 );
	const Asura_Vector_3& xPosition = Asura_AI_Navigation_System::GetWaypoint( uWP )->m_xPosition;
	m_xLookDirection = xPosition - GetPredatorBrain()->GetActor()->GetPosition();
	m_xLookDirection.Normalise();

	m_fLookPeriod = GetNewLookPeriod();
	m_fLookTimer  = 0.0f;
	GetPredatorBrain()->SetLookDirection( m_xLookDirection );
}

void AVP_Behaviour_Predator_PointAndShoot::TurnTowardsTarget()
{
	// Ideally we'd turn eye to eye, but this'll do for now.
	Asura_Vector_3 xDirection = m_xTargetPosition - ( GetPredatorBrain()->GetActor()->GetPosition() + g_xPredatorPlasmaCasterTargetingOffset );
	xDirection.Normalise();
	GetPredatorBrain()->SetLookDirection( xDirection );
}

float AVP_Behaviour_Predator_PointAndShoot::GetNewLookPeriod()
{
	static const float s_fLookPeriodMin = 2.0f;
	static const float s_fLookPeriodMax = 10.0f;
	return Asura_Random::GetFloat( s_fLookPeriodMin, s_fLookPeriodMax );
}

float AVP_Behaviour_Predator_PointAndShoot::GetNewStandPeriod()
{
	static const float s_fStandPeriodMin = 5.0f;
	static const float s_fStandPeriodMax = 20.0f;
	return Asura_Random::GetFloat( s_fStandPeriodMin, s_fStandPeriodMax );
}

float AVP_Behaviour_Predator_PointAndShoot::GetNewCrouchPeriod()
{
	static const float s_fCrouchPeriodMin = 5.0f;
	static const float s_fCrouchPeriodMax = 20.0f;
	return Asura_Random::GetFloat( s_fCrouchPeriodMin, s_fCrouchPeriodMax );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // 0