//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predator_Test.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Behaviour/BehaviourTypes/Predator/AVP_Behaviour_Predator_TestBehaviour.h"

// asura
#include "Interface/Asura_CommandConsole.h"
#include "Network/Asura_Network.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Predator.h"

// avp
#include "Actor/Vertebrates/Predator/AVP_Predator_AIAnimStateMachine.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"

DVSVAR bool  dvs_bMoveMedium	= true;
DVSVAR float dvs_fMoveDirAngVel = 90.0f;
DVSVAR float dvs_fAimAngle		= 0.0f;
DVSVAR bool  dvs_bDoCloak		= false;

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
u_int AVP_Behaviour_Predator_TestBehaviour::s_uConsoleVarReferenceCount = 0;
#endif	

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Behaviour_Predator_Test
//
//////////////////////////////////////////////////////////////////////////////

AVP_Behaviour_Predator_TestBehaviour::AVP_Behaviour_Predator_TestBehaviour( AVP_Brain_Actor_Predator* pxBrain )
: PARENT( pxBrain, AVP_BEHAVIOUR_TYPE_PREDATOR_TESTBEHAVIOUR )
, m_pxPredatorBrain( pxBrain )
, m_fMoveDirCurrent( 0.0f )
, m_fCloakTimer( 0.0f )
{
#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	if ( s_uConsoleVarReferenceCount == 0 )
	{
		if ( !Asura_CommandConsole::VarExists( L"AVP.TestBehaviour.Predator.AngVel" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.TestBehaviour.Predator.AngVel", &dvs_fMoveDirAngVel, -180.0f, 180.0f );
		}
		if ( !Asura_CommandConsole::VarExists( L"AVP.TestBehaviour.Predator.Run" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.TestBehaviour.Predator.Run", &dvs_bMoveMedium );
		}
		if ( !Asura_CommandConsole::VarExists( L"AVP.TestBehaviour.Predator.AimAngle" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.TestBehaviour.Predator.AimAngle", &dvs_fAimAngle, -90.0f, 90.0f );
		}
		if ( !Asura_CommandConsole::VarExists( L"AVP.TestBehaviour.Predator.DoTheHoakyCloaky" ) )
		{
			Asura_CommandConsole::AddVar( L"AVP.TestBehaviour.Predator.DoTheHoakyCloaky", &dvs_bDoCloak );
		}
	}
	++s_uConsoleVarReferenceCount;
#endif
}

void AVP_Behaviour_Predator_TestBehaviour::PerFrameUpdate(const float fTimeslice)
{
// 	AVP_Predator_AIAnimStateMachine* pxSM = static_cast< AVP_Predator_AIAnimStateMachine* >( GetPredatorBrain()->GetActor()->GetAIAnimStateMachine() );
// 	if ( pxSM )
// 	{
// 		if ( dvs_bMoveMedium )
// 		{
// 			if ( !pxSM->IsMoveMedium() )
// 			{
// 				pxSM->RequestMoveMedium();
// 			}
// 		}
// 		else
// 		{
// 			if ( !pxSM->IsMoveSlow() )
// 			{
// 				pxSM->RequestMoveSlow();
// 			}
// 		}
// 	}
// 
// 	float fNegAimY, fAimZ;
// 	Asura_Maths::SinCos( Asura_Maths::DegToRad( dvs_fAimAngle ), fNegAimY, fAimZ );
// 	GetPredatorBrain()->SetAimDirection( Asura_Vector_3( 0.0f, -fNegAimY, fAimZ ) );
// 
// 	m_fMoveDirCurrent += Asura_Maths::DegToRad( dvs_fMoveDirAngVel ) * Asura_Timers::GetGameTimeSlice();
// 
// 	float fMoveX, fMoveZ;
// 	Asura_Maths::SinCos( m_fMoveDirCurrent, fMoveX, fMoveZ );
// 	GetPredatorBrain()->SetMovementDirection( Asura_Vector_3( fMoveX, 0.0f, fMoveZ ) );

	if ( dvs_bDoCloak )
	{
		m_fCloakTimer += Asura_Timers::GetGameTimeSlice();
		if ( m_fCloakTimer > 5.0f ) 
		{
			m_fCloakTimer = 0.0f;
			if ( GetPredatorBrain()->IsCloaked() )
			{
				GetPredatorBrain()->Decloak();
			}
			else
			{
				GetPredatorBrain()->Cloak();
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Behaviour_Predator_TestBehaviour::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	Asura_Font_System::Print
	(
		GetProjectBrain()->GetDebugOutputPos().x,
		GetProjectBrain()->GetDebugOutputPos().y,
		0.5f, 
		0xFFFFFF00,
		ASURA_TEXTFORMAT_CENTREJUSTIFIED, 
		"TEST BEHAVIOUR"
	);
	GetProjectBrain()->GetDebugOutputPos().y += 10.0f;

	if ( dvs_bDoCloak )
	{
		Asura_Font_System::Print
		(
			GetProjectBrain()->GetDebugOutputPos().x,
			GetProjectBrain()->GetDebugOutputPos().y,
			0.5f, 
			0xFFFFFF00,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
			"Cloak Timer: %.2f", m_fCloakTimer
		);
		GetProjectBrain()->GetDebugOutputPos().y += 10.0f;
	}

}
#endif
