// ------------------------------------------------------------------------------------------------------------------------------------
// File		: AVP_PositionEvaluation.h
// Created	: 29 / 05 / 08
// Author	: Gareth Thomas
// Copyright (C) 2008 Rebellion. All Rights Reserved.
// ------------------------------------------------------------------------------------------------------------------------------------

// ------------------------------------------------------------------------------------------------------------------------------------
// BEGIN FILE WRAPPER
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_POSITIONEVALUATION_H__
#define __AVP_POSITIONEVALUATION_H__ 1
// END FILE WRAPPER
// ------------------------------------------------------------------------------------------------------------------------------------


// ------------------------------------------------------------------------------------------------------------------------------------
// INCLUDES
// ------------------------------------------------------------------------------------------------------------------------------------
#include "Core/Asura_Maths.h"

#include "Core/AVP_Navigation.h"

// ------------------------------------------------------------------------------------------------------------------------------------
// FORWARD DECLARATIONS
// ------------------------------------------------------------------------------------------------------------------------------------
class AVP_ServerEntity_Actor;
class AVP_ServerActor_Human;
class AVP_ServerActor_Alien;
class AVP_ServerActor_Predator;

// ------------------------------------------------------------------------------------------------------------------------------------
// Class			: AVP_PositionEvaluation
// Description		: Class containing static functions for position evaluation use
// Last modified	: 29 / 05 / 08
// ------------------------------------------------------------------------------------------------------------------------------------
class AVP_PositionEvaluation
{

public:

	static Asura_Guid	GetPrimaryThreatGuid( const AVP_ServerEntity_Actor* pxOurActor );
	static Asura_Guid	GetAlertGuid( const AVP_ServerEntity_Actor* pxOurActor );

	// All Species
	static u_int		GetProximityScore( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const float fEvaluationRange, const float fScoreModifier = 1.0f );
	static u_int		GetProximityToThreatScore( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const float fScoreModifier = 1.0f );
	static u_int		GetWaypointLOFtoPrimaryThreatScore( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const u_int uScoreIfInLOF );
	static u_int		GetWaypointNoLOFToPrimaryThreatScore( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const u_int uScoreIfNotInLOF );
	static u_int		GetWaypointNoLOFToAlertScore( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const u_int uScoreIfNotInLOF );
	static u_int		GetPreferredFightingRangeScore( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const float fMinPrefferedFightingRange, const float fMaxPrefferedFightingRange, const u_int uScoreIfInFiringRange );
	static u_int		GetWaypointNoLOFToPositionScore( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const Asura_Vector_3 xPosition, const u_int uScoreIfNotInLOF );
	static u_int		GetProximityToPositionScore( const u_int uWaypointID, const float fEvaluationRange, const Asura_Vector_3 xPosition, const float fScoreModifier = 1.0f );
	static u_int		GetPositionIsInFrontScore( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const u_int uScore );
	static u_int		GetPositionIsBehindScore( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const u_int uScore );
	static u_int		GetPositionIsInFrontOfSquadScore( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const u_int uScore );
	static u_int		GetPositionIsWaypointNextToWall( const u_int uWaypointID, const u_int uScore );

	// Alien Only
	static u_int		GetAlienPositionOccupiedByAllyScore( const AVP_ServerActor_Alien* pxOurActor, const u_int uWaypointID, const u_int uScoreIfAllyDoesNotOccupyPosition );

	// Marine / Android Only
	static u_int		GetPositionCloseToSquadLeaderScore( const AVP_ServerActor_Human* pxOurActor, const u_int uWaypointID, const u_int uScoreIfInCloseProximity );
	static u_int		GetPositionCloseToSquadLeaderDestinationScore( const AVP_ServerActor_Human* pxOurActor, const u_int uWaypointID, const u_int uScoreIfInCloseProximityToLeadersDestination );
	static u_int		GetHumanoidPositionOccupiedByAllyScore( const AVP_ServerActor_Human* pxOurActor, const u_int uWaypointID, const u_int uScoreIfAllyDoesNotOccupyPosition );
	static u_int		GetHumanoidPositionGivesCoverFromSecondaryThreatsScore( const AVP_ServerActor_Human* pxOurActor, const u_int uWaypointID, const u_int uScoreIfProvidesGoodCover );
	static u_int		GetPositionProvidesCoverFromPrimaryThreat( const AVP_ServerActor_Human* pxOurActor, const u_int uWaypointID, const u_int uScoreIfWaypointProvidesCover );
	static u_int		GetWaypointProvidesCoverFromPosition(const Asura_Vector_3 xPosition, const u_int uWaypointID, const u_int uScoreIfWaypointProvidesCover );
	static u_int		GetWaypointLOFToScriptedTarget( const AVP_ServerActor_Human* pxOurActor, const u_int uWaypointID, const u_int uScoreIfInLOF );
	static u_int		GetWaypointNoLOFToScriptedTarget( const AVP_ServerEntity_Actor* pxOurActor, const u_int uWaypointID, const u_int uScoreIfNotInLOF );
	static u_int		GetWaypointIsBeyondTargetScore( const AVP_ServerActor_Human* pxOurActor, const u_int uWaypointID, const u_int uScore );
	static u_int		GetTraversalCostScore( const AVP_ServerActor_Human* pxOurActor, AVP_Navigation::AVP_PosEvalParams xPosEvalParams, u_int uScore );
	
	// Predator Only

	private:

	static Asura_Guid s_uLastGPVGuid;

};

#endif // __AVP_POSITIONEVALUATION_H__