////////////////////////////////////////////////////////////////////////////////
//
// AVP_Waypoint_Manager.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Core/AVP_Waypoint_Manager.h"

// asura
#include "AI/Asura_AI_Navigation.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_HalfFloat.h"
#include "Core/Asura_List.h"
#include "Core/Asura_LOS.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_Mempool.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/DynamicLight/Asura_ClientEntity_DynamicLight.h"
#include "Entity/DynamicLight/Asura_ServerEntity_DynamicLight.h"
#include "Entity/EntityTemplate/Asura_ServerEntity_Template.h"
#include "Interface/Asura_CommandConsole.h"
#include "Light/Asura_GI_Solution.h"
#include "Light/Asura_Photon.h"
#include "Network/Asura_Message.h"
#include "Projectors/Asura_Projector.h"
#include "Projectors/Asura_Projector_System.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Font_System.h"

// avp
#include "Core/AVP_Navigation.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/AVP_server_Messages.h"
#include "Entity/Container/AVP_ServerEntity_Container.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Marine/AVP_ServerActor_Marine.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_HumanFlare.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"

#ifdef ASURA_TOOLS
#include "Core/Asura_DynamicBVH.h"
#include "Core/Threads/Asura_Task_Array.h"
#include "Waypointing/Tools_Mode_Waypointing.h"
#include "Lighting/Tools_Environment_LOS.h"
#include "Misc/Tools_MessageLog.h"
#include "Misc/Tools_Timed_Messages.h"
#endif


#ifdef ASURA_TOOLS
//////////////////////////////////////////////////////////////////////////////
//
// AVP_DynamicBVH_StaticObjects_Wrapper
// Wrapper class Used for the DynamicBVH LOS test.
//
//////////////////////////////////////////////////////////////////////////////

class AVP_DynamicBVH_StaticObjects_Wrapper
{
public:
	AVP_DynamicBVH_StaticObjects_Wrapper( Asura_ServerEntity_PhysicalObject* pxStaticObject );
	~AVP_DynamicBVH_StaticObjects_Wrapper( void );

	// Entity Wrapper Access Functions
	Asura_Guid		GetGuid() const																			{ return m_pxStaticObject->GetGuid(); }
	bool			LOSTest( const Asura_LOS_InputData* pxInput, Asura_LOS_OutputData* pxOutput ) const		{ return m_pxStaticObject->LOSTest(pxInput, pxOutput); }
	bool			IsDestroyed() const																		{ return m_pxStaticObject->IsDestroyed(); }
	unsigned short	GetClassification(void) const															{ return m_pxStaticObject->GetClassification(); }

	// Entity Wrapper Functions
	void			GetLineOfSightAABB(Asura_Bounding_Box& xAABB);

	static u_int	GetNumInList(void)			{return m_uNumInList;}
	static void		ResetNumInList(void)		{m_uNumInList = 0;}
	static void		IncreaseNumInList(void)		{++m_uNumInList;}

	// Static List Access Functions;
	static Asura_Classified_List<AVP_DynamicBVH_StaticObjects_Wrapper>* GetLineOfSightList()				{ return &s_xLineOfSightList; } 

    static inline void GetAABB( AVP_DynamicBVH_StaticObjects_Wrapper& xWrapper, Asura_Bounding_Box& xAABB ) { xWrapper.GetLineOfSightAABB( xAABB ); }

private:
	Asura_ServerEntity_PhysicalObject*	m_pxStaticObject;
	static u_int m_uNumInList;
	static Asura_Classified_List<AVP_DynamicBVH_StaticObjects_Wrapper> s_xLineOfSightList;
};


class AVP_DynamicBVH_StaticObjects_Wrapper_Iterator : public Asura_Classified_List_It<AVP_DynamicBVH_StaticObjects_Wrapper>
{

public:

	AVP_DynamicBVH_StaticObjects_Wrapper_Iterator( Asura_Classified_List<AVP_DynamicBVH_StaticObjects_Wrapper>* pxList )
        : Asura_Classified_List_It<AVP_DynamicBVH_StaticObjects_Wrapper>( pxList )
    {
    }

    inline void Next( void )
    {
        do
        {
            Asura_Classified_List_It<AVP_DynamicBVH_StaticObjects_Wrapper>::Next();
        }
        while( !Done() && GetCurrent()->IsDestroyed() );
    }

};


Asura_Classified_List<AVP_DynamicBVH_StaticObjects_Wrapper> AVP_DynamicBVH_StaticObjects_Wrapper::s_xLineOfSightList;
static Asura_DynamicBVH<AVP_DynamicBVH_StaticObjects_Wrapper> s_xStaticLineOfSightBVH;
u_int AVP_DynamicBVH_StaticObjects_Wrapper::m_uNumInList = 0;
#endif // ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Waypoint_Manager
//
//////////////////////////////////////////////////////////////////////////////

void AVP_Waypoint_Manager::WaypointLOFData::Reset()
{
	// Force the min distance to cover to be large and the max distance to be small for each direction.
	Asura_Mem::Set( m_ucMinData, 0xff, AVP_Waypoint_Manager::NUM_SECTORS * sizeof( u_char ) );
	Asura_Mem::Set( m_ucMaxData, 0x00, AVP_Waypoint_Manager::NUM_SECTORS * sizeof( u_char ) );
#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
	Asura_Mem::Set( m_usMinWP, 0xff, AVP_Waypoint_Manager::NUM_SECTORS * sizeof( u_short ) );
	Asura_Mem::Set( m_usMaxWP, 0xff, AVP_Waypoint_Manager::NUM_SECTORS * sizeof( u_short ) );
#endif
}

#ifdef ASURA_TOOLS
void AVP_Waypoint_Manager::WaypointLOFData::SetMinLOFValue( const u_int uSector, const float fDistance )
{
	// Min distance is in the range 0 -> 255.
	m_ucMinData[ uSector ] = static_cast< u_char >( Asura_Maths::ClampToRange( Asura_Maths::FloatToInt_Near( fDistance ), 0, 255 ) );
}

void AVP_Waypoint_Manager::WaypointLOFData::SetMaxLOFValue( const u_int uSector, const float fDistance )
{
	// Max distance is in the range 0 -> 255.
	m_ucMaxData[ uSector ] = static_cast< u_char >( Asura_Maths::ClampToRange( Asura_Maths::FloatToInt_Near( fDistance ), 0, 255 ) );
}
#endif

float AVP_Waypoint_Manager::WaypointLOFData::GetMinLOFValue( const u_int uSector ) const
{
	// Min distance is in the range 0 -> 255.
	return ( static_cast< float >( m_ucMinData[ uSector ] ) );
}

float AVP_Waypoint_Manager::WaypointLOFData::GetMaxLOFValue( const u_int uSector ) const
{
	// Max distance is in the range 0 -> 255.
	return ( static_cast< float >( m_ucMaxData[ uSector ] ) );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

u_int   AVP_Waypoint_Manager::s_uNumEntries					= 0;
Asura_HalfFloat*	AVP_Waypoint_Manager::s_pxWaypointLuminance	= NULL;
u_char* AVP_Waypoint_Manager::s_pucWaypointUserFactors		= NULL;
AVP_Waypoint_Manager::WaypointLOFData* AVP_Waypoint_Manager::s_pxWaypointLOFData = NULL;
Asura_LinkList< u_short > AVP_Waypoint_Manager::s_xPendingWPIDList;
u_char*	AVP_Waypoint_Manager::s_pucPendingWPIDMaker =  NULL;
AVP_Waypoint_Manager::ExtraLinksData AVP_Waypoint_Manager::s_xExtraLinkData;

float	AVP_Waypoint_Manager::s_fMinHideLum = 0.05f;
float	AVP_Waypoint_Manager::s_fMaxHideLum = 0.10f;
float	AVP_Waypoint_Manager::s_fMaxHideLumForMarineAI = 0.20f;


#ifdef ASURA_TOOLS
bool*		AVP_Waypoint_Manager::s_bLOFWaypointShouldWriteToLog	= NULL;
Asura_Guid	AVP_Waypoint_Manager::s_uLastGamePlayVolumeGuid			= ASURA_GUID_UNREGISTERED;
#endif

Asura_Hashed_List< AVP_Waypoint_Manager::WaypointAndNodes > AVP_Waypoint_Manager::s_xWaypointNodesLookupList;

#if !defined(ASURA_NO_DEBUG_OUTPUT)
bool AVP_Waypoint_Manager::s_bRenderDebugInfo = false;
bool dbg_bOutputLightCalcInfo = false;
#endif

void AVP_Waypoint_Manager::Initialise()
{
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE) && !defined(ASURA_NO_DEBUG_OUTPUT)
	Asura_CommandConsole::AddVar( L"AVP.AI.RenderLightingInfo", &s_bRenderDebugInfo, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
#endif
}

void AVP_Waypoint_Manager::Update()
{
	// Allow 1/100th of a frame (at 30fps) for updating.
	static const float fAllowedUpdateTime = ( 1.0f / 30.0f ) * 0.01f;
	const float fStarttime = Asura_Timers::GetTimeInFrame();
	const float fEndTime = fStarttime + fAllowedUpdateTime;

	Asura_ListElt<u_short>* pxWPIter;
	bool bChangesMade = false;
	int iWPCount = 0;
	while( Asura_Timers::GetTimeInFrame() < fEndTime && (pxWPIter = s_xPendingWPIDList.Front()) != NULL)
	{
		u_short usWPID = pxWPIter->Data();
		CalculateLuminanceForWaypoint(usWPID);
		s_xPendingWPIDList.PopFront();
		s_pucPendingWPIDMaker[usWPID / 8 ] &= 0xff - ( 1 << (usWPID & 0x7 ));	
		bChangesMade = true;
		++iWPCount;
	}

	if (bChangesMade && s_xPendingWPIDList.Front() == NULL)
	{
		AVP_ServerEntity_Team_Alien* pxAlienTeam = AVP_ServerEntity_Team_Alien::GetInstance();		
		if (pxAlienTeam)
		{
			pxAlienTeam->LightChanged();
		}
	}

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	if (dbg_bOutputLightCalcInfo)
	{
		if(bChangesMade)
		{
			Asura_OutputToDebugger_Retail( ( "%d waypoints updated in %f\n", iWPCount, Asura_Timers::GetTimeInFrame() - fStarttime ) );
			if (s_xPendingWPIDList.Front() == NULL)
			{
				Asura_OutputToDebugger_Retail( ( "DONE waypoints updated\n" ) );
			}
		}
	}
#endif
}

void AVP_Waypoint_Manager::Reset()
{
	Shutdown();
}

void AVP_Waypoint_Manager::Shutdown()
{
	s_uNumEntries = 0;
	delete [] s_pxWaypointLuminance;
	s_pxWaypointLuminance = NULL;
	delete [] s_pucWaypointUserFactors;
	s_pucWaypointUserFactors = NULL;
	delete [] s_pxWaypointLOFData;
	s_pxWaypointLOFData = NULL;
	delete [] s_pucPendingWPIDMaker;
	s_pucPendingWPIDMaker = NULL;

	s_xPendingWPIDList.Clear();

	Asura_Hashed_List_Deleter_It< WaypointAndNodes > xListIter( &s_xWaypointNodesLookupList );
	while ( !xListIter.Done() )
	{
		WaypointAndNodes* pxToDelete = xListIter.GetCurrent();
		xListIter.DeleteCurrent();
		delete [] pxToDelete->m_puGS2NodeGuids;
		delete pxToDelete;
	}
}

void AVP_Waypoint_Manager::HandleMessage( const Asura_Message* pxMessage )
{
	if ( pxMessage->GetMsgID() == AVP_MSGID_LIGHT_CHANGE_EVENT )
	{
		// Handle the static message, which comes from where exactly?
		Asura_Assertf( false, ( "Will doesn't think this message is getting sent any more, if it is, please tell him. Guid %d, Classification %d", pxMessage->GetFromGuid(), pxMessage->GetEntityClassification() ) );
		return;
	}

	if ( pxMessage->GetMsgID() != AVP_MSGID_LIGHT_CHANGE_EVENT_CODE )
	{
		return;
	}

	if ( !s_pxWaypointLuminance )
	{
		return;
	}

	const AVP_Message_LightChange* pxData = static_cast< const AVP_Message_LightChange* >( pxMessage->GetData() );
	Asura_Assert( pxData, "Invalid message data!" );
	if ( !pxData )
	{
		return;
	}

	CollectWPForBoundingBox( pxData->GetOldBB() );
	CollectWPForBoundingBox( pxData->GetNewBB() );

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	if (dbg_bOutputLightCalcInfo)
	{
		Asura_OutputToDebugger_Retail( ( "Light change Event Received\n" ) );
	}
#endif
}

void AVP_Waypoint_Manager::CreateConsoleVars()
{
	Asura_CommandConsole::AddVar( L"AVP.AI.WaypointLighting.DarkLuminance", &s_fMinHideLum, 0.0f, 10.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( L"AVP.AI.WaypointLighting.LightLuminance", &s_fMaxHideLum, 0.0f, 10.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
}

void AVP_Waypoint_Manager::ReadFromChunkStream( Asura_Chunk_Stream& xChunkStream )
{
	u_int uVersion;
	xChunkStream >> uVersion;

	delete [] s_pucPendingWPIDMaker;
	s_pucPendingWPIDMaker = NULL;

	s_xPendingWPIDList.Clear();

	if ( uVersion == 8 )
	{
		xChunkStream >> s_uNumEntries;
		if ( s_uNumEntries )
		{
			s_pxWaypointLuminance = new Asura_HalfFloat[ s_uNumEntries ];
			s_pucWaypointUserFactors     = new u_char[ s_uNumEntries ];
			s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ];
			Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

			Asura_ReadFromStream_Array_Wrapper< Asura_HalfFloat > xLuminanceFactors( s_uNumEntries, s_pxWaypointLuminance );
			xChunkStream >> xLuminanceFactors;

			Asura_ReadFromStream_Array_Wrapper< u_char > xUserFactors( s_uNumEntries, s_pucWaypointUserFactors );
			xChunkStream >> xUserFactors;
		}

		u_int uNumLookupEntries;
		xChunkStream >> uNumLookupEntries;
		for ( u_int u = 0; u < uNumLookupEntries; ++u )
		{
			WaypointAndNodes* pxWPnNode = new WaypointAndNodes;
			xChunkStream >> pxWPnNode->m_usWPID;
			xChunkStream >> pxWPnNode->m_usNumNodes;
			pxWPnNode->m_puGS2NodeGuids = new Asura_Guid[ pxWPnNode->m_usNumNodes ];
			for ( u_short usGuid = 0; usGuid < pxWPnNode->m_usNumNodes; ++usGuid )
			{
				xChunkStream >> pxWPnNode->m_puGS2NodeGuids[ usGuid ];
			}
			s_xWaypointNodesLookupList.AddToList( pxWPnNode );
		}

		if ( s_uNumEntries )
		{
			s_pxWaypointLOFData = new WaypointLOFData[ s_uNumEntries ];
			for ( u_int uWPID = 0; uWPID < s_uNumEntries; ++uWPID )
			{
				for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
				{
					xChunkStream >> s_pxWaypointLOFData[ uWPID ].m_ucMinData[ uDir ];
				}
				for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
				{
					xChunkStream >> s_pxWaypointLOFData[ uWPID ].m_ucMaxData[ uDir ];
				}
			}
		}

		xChunkStream.Skip4BytePadding();
	}
	else if ( uVersion == 7 )
	{
		xChunkStream >> s_uNumEntries;
		if ( s_uNumEntries )
		{
			u_char* pucWaypointLuminanceFactors = new u_char[ s_uNumEntries ];
			s_pucWaypointUserFactors		= new u_char[ s_uNumEntries ];
			s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ];
			Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

			Asura_ReadFromStream_Array_Wrapper< u_char > xLuminanceFactors( s_uNumEntries, pucWaypointLuminanceFactors );
			xChunkStream >> xLuminanceFactors;

			delete[] pucWaypointLuminanceFactors;

			Asura_ReadFromStream_Array_Wrapper< u_char > xUserFactors( s_uNumEntries, s_pucWaypointUserFactors );
			xChunkStream >> xUserFactors;
		}

		u_int uNumLookupEntries;
		xChunkStream >> uNumLookupEntries;
		for ( u_int u = 0; u < uNumLookupEntries; ++u )
		{
			WaypointAndNodes* pxWPnNode = new WaypointAndNodes;
			xChunkStream >> pxWPnNode->m_usWPID;
			xChunkStream >> pxWPnNode->m_usNumNodes;
			pxWPnNode->m_puGS2NodeGuids = new Asura_Guid[ pxWPnNode->m_usNumNodes ];
			for ( u_short usGuid = 0; usGuid < pxWPnNode->m_usNumNodes; ++usGuid )
			{
				xChunkStream >> pxWPnNode->m_puGS2NodeGuids[ usGuid ];
			}
			s_xWaypointNodesLookupList.AddToList( pxWPnNode );
		}

		if ( s_uNumEntries )
		{
			s_pxWaypointLOFData = new WaypointLOFData[ s_uNumEntries ];
			for ( u_int uWPID = 0; uWPID < s_uNumEntries; ++uWPID )
			{
				for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
				{
					xChunkStream >> s_pxWaypointLOFData[ uWPID ].m_ucMinData[ uDir ];
				}
				for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
				{
					xChunkStream >> s_pxWaypointLOFData[ uWPID ].m_ucMaxData[ uDir ];
				}
			}
		}

		xChunkStream.Skip4BytePadding();
	}
	else if ( uVersion == 6 )
	{
		xChunkStream >> s_uNumEntries;
		if ( s_uNumEntries )
		{
			u_char* pucWaypointLuminanceFactors = new u_char[ s_uNumEntries ];
			s_pucWaypointUserFactors      = new u_char[ s_uNumEntries ];
			s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ];
			Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

			Asura_ReadFromStream_Array_Wrapper< u_char > xLuminanceFactors( s_uNumEntries, pucWaypointLuminanceFactors );
			xChunkStream >> xLuminanceFactors;

			delete[] pucWaypointLuminanceFactors;

			Asura_ReadFromStream_Array_Wrapper< u_char > xUserFactors( s_uNumEntries, s_pucWaypointUserFactors );
			xChunkStream >> xUserFactors;
		}

		u_int uNumLookupEntries;
		xChunkStream >> uNumLookupEntries;
		for ( u_int u = 0; u < uNumLookupEntries; ++u )
		{
			WaypointAndNodes* pxWPnNode = new WaypointAndNodes;
			xChunkStream >> pxWPnNode->m_usWPID;
			xChunkStream >> pxWPnNode->m_usNumNodes;
			pxWPnNode->m_puGS2NodeGuids = new Asura_Guid[ pxWPnNode->m_usNumNodes ];
			for ( u_short usGuid = 0; usGuid < pxWPnNode->m_usNumNodes; ++usGuid )
			{
				xChunkStream >> pxWPnNode->m_puGS2NodeGuids[ usGuid ];
			}
			s_xWaypointNodesLookupList.AddToList( pxWPnNode );
		}

		u_int uLOFDataGenerated;
		xChunkStream >> uLOFDataGenerated;

		if ( s_uNumEntries && uLOFDataGenerated == 1 )
		{
			s_pxWaypointLOFData = new WaypointLOFData[ s_uNumEntries ];
			for ( u_int uWPID = 0; uWPID < s_uNumEntries; ++uWPID )
			{
				for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
				{
					xChunkStream >> s_pxWaypointLOFData[ uWPID ].m_ucMinData[ uDir ];
				}
				for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
				{
					xChunkStream >> s_pxWaypointLOFData[ uWPID ].m_ucMaxData[ uDir ];
				}
			}
		}

		xChunkStream.Skip4BytePadding();
	}
#ifdef ASURA_TOOLS
	else if ( uVersion == 5 )
	{
		xChunkStream >> s_uNumEntries;
		if ( s_uNumEntries )
		{
			u_char* pucWaypointLuminanceFactors = new u_char[ s_uNumEntries ];
			s_pucWaypointUserFactors      = new u_char[ s_uNumEntries ];
			s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ];
			Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

			Asura_ReadFromStream_Array_Wrapper< u_char > xLuminanceFactors( s_uNumEntries, pucWaypointLuminanceFactors );
			xChunkStream >> xLuminanceFactors;

			delete[] pucWaypointLuminanceFactors;

			Asura_ReadFromStream_Array_Wrapper< u_char > xUserFactors( s_uNumEntries, s_pucWaypointUserFactors );
			xChunkStream >> xUserFactors;
		}

		u_int uNumLookupEntries;
		xChunkStream >> uNumLookupEntries;
		for ( u_int u = 0; u < uNumLookupEntries; ++u )
		{
			WaypointAndNodes* pxWPnNode = new WaypointAndNodes;
			xChunkStream >> pxWPnNode->m_usWPID;
			xChunkStream >> pxWPnNode->m_usNumNodes;
			pxWPnNode->m_puGS2NodeGuids = new Asura_Guid[ pxWPnNode->m_usNumNodes ];
			for ( u_short usGuid = 0; usGuid < pxWPnNode->m_usNumNodes; ++usGuid )
			{
				xChunkStream >> pxWPnNode->m_puGS2NodeGuids[ usGuid ];
			}
			s_xWaypointNodesLookupList.AddToList( pxWPnNode );
		}

		if ( s_uNumEntries )
		{
			s_pxWaypointLOFData = new WaypointLOFData[ s_uNumEntries ];
			for ( u_int uWPID = 0; uWPID < s_uNumEntries; ++uWPID )
			{
				for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
				{
					xChunkStream >> s_pxWaypointLOFData[ uWPID ].m_ucMinData[ uDir ];
				}
				for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
				{
					xChunkStream >> s_pxWaypointLOFData[ uWPID ].m_ucMaxData[ uDir ];
				}
			}
		}

		xChunkStream.Skip4BytePadding();
	}
	else if ( uVersion == 4 )
	{
		xChunkStream >> s_uNumEntries;
		if ( s_uNumEntries )
		{
			u_char* pucWaypointLuminanceFactors = new u_char[ s_uNumEntries ];
			s_pucWaypointUserFactors      = new u_char[ s_uNumEntries ];
			s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ];
			Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

			Asura_ReadFromStream_Array_Wrapper< u_char > xLuminanceFactors( s_uNumEntries, pucWaypointLuminanceFactors );
			xChunkStream >> xLuminanceFactors;

			delete[] pucWaypointLuminanceFactors;

			Asura_ReadFromStream_Array_Wrapper< u_char > xUserFactors( s_uNumEntries, s_pucWaypointUserFactors );
			xChunkStream >> xUserFactors;
		}

		u_int uNumLookupEntries;
		xChunkStream >> uNumLookupEntries;
		for ( u_int u = 0; u < uNumLookupEntries; ++u )
		{
			WaypointAndNodes* pxWPnNode = new WaypointAndNodes;
			xChunkStream >> pxWPnNode->m_usWPID;
			xChunkStream >> pxWPnNode->m_usNumNodes;
			pxWPnNode->m_puGS2NodeGuids = new Asura_Guid[ pxWPnNode->m_usNumNodes ];
			for ( u_short usGuid = 0; usGuid < pxWPnNode->m_usNumNodes; ++usGuid )
			{
				xChunkStream >> pxWPnNode->m_puGS2NodeGuids[ usGuid ];
			}
			s_xWaypointNodesLookupList.AddToList( pxWPnNode );
		}

		if ( s_uNumEntries )
		{
			// Chuck the old data away.
			s_pxWaypointLOFData = new WaypointLOFData[ s_uNumEntries ];
			for ( u_int uWPID = 0; uWPID < s_uNumEntries; ++uWPID )
			{
				for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
				{
					u_char ucDummy;
					xChunkStream >> ucDummy;
				}
				s_pxWaypointLOFData[ uWPID ].Reset();
			}
		}

		xChunkStream.Skip4BytePadding();
	}
	else if ( uVersion == 3 )
	{
		xChunkStream >> s_uNumEntries;
		if ( s_uNumEntries )
		{
			u_char* pucWaypointLuminanceFactors = new u_char[ s_uNumEntries ];
			s_pucWaypointUserFactors      = new u_char[ s_uNumEntries ];
			s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ];
			Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

			Asura_ReadFromStream_Array_Wrapper< u_char > xLuminanceFactors( s_uNumEntries, pucWaypointLuminanceFactors );
			xChunkStream >> xLuminanceFactors;

			delete[] pucWaypointLuminanceFactors;

			Asura_ReadFromStream_Array_Wrapper< u_char > xUserFactors( s_uNumEntries, s_pucWaypointUserFactors );
			xChunkStream >> xUserFactors;
		}

		u_int uNumLookupEntries;
		xChunkStream >> uNumLookupEntries;
		for ( u_int u = 0; u < uNumLookupEntries; ++u )
		{
			WaypointAndNodes* pxWPnNode = new WaypointAndNodes;
			xChunkStream >> pxWPnNode->m_usWPID;
			xChunkStream >> pxWPnNode->m_usNumNodes;
			pxWPnNode->m_puGS2NodeGuids = new Asura_Guid[ pxWPnNode->m_usNumNodes ];
			for ( u_short usGuid = 0; usGuid < pxWPnNode->m_usNumNodes; ++usGuid )
			{
				xChunkStream >> pxWPnNode->m_puGS2NodeGuids[ usGuid ];
			}
			s_xWaypointNodesLookupList.AddToList( pxWPnNode );
		}

		xChunkStream.Skip4BytePadding();
	}
	else if ( uVersion == 2 )
	{
		xChunkStream >> s_uNumEntries;
		if ( s_uNumEntries )
		{
			u_char* pucWaypointLuminanceFactors = new u_char[ s_uNumEntries ];
			s_pucWaypointUserFactors      = new u_char[ s_uNumEntries ];
			s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ];
			Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

			Asura_ReadFromStream_Array_Wrapper< u_char > xLuminanceFactors( s_uNumEntries, pucWaypointLuminanceFactors );
			xChunkStream >> xLuminanceFactors;

			delete[] pucWaypointLuminanceFactors;

			Asura_ReadFromStream_Array_Wrapper< u_char > xUserFactors( s_uNumEntries, s_pucWaypointUserFactors );
			xChunkStream >> xUserFactors;
		}

		u_int uNumLookupEntries;
		xChunkStream >> uNumLookupEntries;
		for ( u_int u = 0; u < uNumLookupEntries; ++u )
		{
			WaypointAndNodes* pxWPnNode = new WaypointAndNodes;
			xChunkStream >> pxWPnNode->m_usWPID;
			// There was only ever one guid saved in this version.
			pxWPnNode->m_usNumNodes = 1;
			pxWPnNode->m_puGS2NodeGuids = new Asura_Guid[ pxWPnNode->m_usNumNodes ];
			xChunkStream >> pxWPnNode->m_puGS2NodeGuids[ 0 ];
			s_xWaypointNodesLookupList.AddToList( pxWPnNode );
		}

		xChunkStream.Skip4BytePadding();
	}
	else if ( uVersion == 1 )
	{
		xChunkStream >> s_uNumEntries;
		if ( s_uNumEntries )
		{
			u_char* pucWaypointLuminanceFactors = new u_char[ s_uNumEntries ];
			s_pucWaypointUserFactors      = new u_char[ s_uNumEntries ];
			s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ];
			Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

			Asura_ReadFromStream_Array_Wrapper< u_char > xLuminanceFactors( s_uNumEntries, pucWaypointLuminanceFactors );
			xChunkStream >> xLuminanceFactors;

			delete[] pucWaypointLuminanceFactors;

			Asura_ReadFromStream_Array_Wrapper< u_char > xUserFactors( s_uNumEntries, s_pucWaypointUserFactors );
			xChunkStream >> xUserFactors;
		}

		xChunkStream.Skip4BytePadding();
	}
	else if ( uVersion == 0 )
	{
		xChunkStream >> s_uNumEntries;
		if ( s_uNumEntries )
		{
			u_char* pucWaypointLuminanceFactors = new u_char[ s_uNumEntries ];
			s_pucWaypointUserFactors      = new u_char[ s_uNumEntries ];
			s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ];
			Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

			for ( u_int u = 0 ; u < s_uNumEntries; ++u )
			{
				xChunkStream >> pucWaypointLuminanceFactors[ u ];
				xChunkStream >> s_pucWaypointUserFactors[ u ];
			}

			delete[] pucWaypointLuminanceFactors;
		}

		xChunkStream.Skip4BytePadding();
	}
#endif // ASURA_TOOLS
	else
	{
		Asura_Assertf( false, ( "AVP_Waypoint_Manager::ReadFromChunkStream - unhandled chunk version (%d)", uVersion ) );
	}

#ifdef ASURA_TOOLS
	// Ditch all the data from old chunk versions if it's not all valid.
	if ( uVersion < 8 && s_uNumEntries > 0 )
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_WARNING, "Warning: Extra Waypointing Data is invalid, it needs re-generating in AVP Waypointing Mode." );
		DestroyExtraWaypointingData();
	}
#endif
}

float AVP_Waypoint_Manager::GetWPLuminance( const u_int uWPID )
{
	if ( !s_pxWaypointLuminance || !s_pucWaypointUserFactors || uWPID >= s_uNumEntries )
	{
		Asura_Assert( false, "Invalid waypoint lighting data, please recalculate this mission's Extra Waypointing Data in AVP Waypointing Mode in Tools, thank you." );
		return 0.5f;
	}
	else
	{
		return s_pxWaypointLuminance[ uWPID ];
	}
}

bool AVP_Waypoint_Manager::GetGS2NodeGuidsFromWPIndex( const u_short usWPIndex, const Asura_Guid*& puNodeGuids, int& iNumNodesGuids )
{
	const WaypointAndNodes* pxWPnNode = s_xWaypointNodesLookupList.FindEntity( static_cast< Asura_Guid >( usWPIndex ) );
	if ( pxWPnNode )
	{
        puNodeGuids    = pxWPnNode->m_puGS2NodeGuids;
        iNumNodesGuids = pxWPnNode->m_usNumNodes;
		return true;
	}
	else
	{
		return false;
	}
}

#ifdef ASURA_TOOLS
void AVP_Waypoint_Manager::WriteToChunkStream( Asura_Chunk_Stream& xChunkStream )
{
	xChunkStream << uCURRENT_CHUNK_VERSION;

	// Luminance factors.
	xChunkStream << s_uNumEntries;
	if ( s_uNumEntries )
	{
		Asura_WriteToStream_Array_Wrapper< Asura_HalfFloat > xLuminanceFactors( s_uNumEntries, s_pxWaypointLuminance );
		xChunkStream << xLuminanceFactors;

		Asura_WriteToStream_Array_Wrapper< u_char > xUserFactors( s_uNumEntries, s_pucWaypointUserFactors );
		xChunkStream << xUserFactors;
	}

	// Waypoint to GS2 node lookup data.
	Asura_Hashed_List_ConstIt< WaypointAndNodes > xWPnNodeIter( &s_xWaypointNodesLookupList );
	u_int uNumWPNodeEntries = 0;
	while ( !xWPnNodeIter.Done() )
	{
		++uNumWPNodeEntries;
		xWPnNodeIter.Next();
	}

	xChunkStream << uNumWPNodeEntries;
	for ( xWPnNodeIter.Restart(); !xWPnNodeIter.Done(); xWPnNodeIter.Next() )
	{
		const WaypointAndNodes* pxWPnNodes = xWPnNodeIter.GetCurrent();

		xChunkStream << pxWPnNodes->m_usWPID;
		xChunkStream << pxWPnNodes->m_usNumNodes;
		for ( u_short usGuid = 0; usGuid < pxWPnNodes->m_usNumNodes; ++ usGuid )
		{
			xChunkStream << pxWPnNodes->m_puGS2NodeGuids[ usGuid ];
		}
	}

	// Waypoint LOF data.
	for ( u_int uWPID = 0; uWPID < s_uNumEntries; ++uWPID )
	{
		for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
		{
			xChunkStream << s_pxWaypointLOFData[ uWPID ].m_ucMinData[ uDir ];
		}
		for ( u_int uDir = 0; uDir < NUM_SECTORS; ++uDir )
		{
			xChunkStream << s_pxWaypointLOFData[ uWPID ].m_ucMaxData[ uDir ];
		}
	}

	xChunkStream.Write4BytePadding();
}

bool AVP_Waypoint_Manager::ExtraWaypointingDataValid()
{
	//## todo - once the data is on the latest chunk (8) we can ditch all but the first test
	return ( s_uNumEntries > 0 && s_pxWaypointLuminance && s_pucWaypointUserFactors && s_pxWaypointLOFData );
}

void AVP_Waypoint_Manager::DestroyExtraWaypointingData()
{
	s_uNumEntries = 0;
	DestroyWaypointLuminanceFactors();
	DestroyGS2LookupData();
	DestroyLOFLookUpTableData();
}

bool AVP_Waypoint_Manager::CalculateExtraWaypointingData( bool& bTouchedEntityData )
{
	bool bOk = true;
	s_uNumEntries = Asura_AI_Navigation_System::GetNumberOfWaypoints();
	bOk	&= CalculateWaypointLuminanceFactors();
	bOk	&= CalculateGS2LookupData( bTouchedEntityData );
	bOk	&= CalculateLOFLookUpTableData();
	return bOk;
}

void AVP_Waypoint_Manager::DestroyWaypointLuminanceFactors()
{
	delete [] s_pxWaypointLuminance;
	s_pxWaypointLuminance = NULL;
	delete [] s_pucWaypointUserFactors;
	s_pucWaypointUserFactors = NULL;
}

bool AVP_Waypoint_Manager::CalculateWaypointLuminanceFactors()
{
	// This can be called from object mode, so the s_uNumEntries value may not match the number of waypoints if the waypoints are out of date.
	if ( s_uNumEntries != Asura_AI_Navigation_System::GetNumberOfWaypoints() )
	{
		return false;
	}

	DestroyWaypointLuminanceFactors();

	s_pxWaypointLuminance = new Asura_HalfFloat[ s_uNumEntries ];
	if ( !s_pxWaypointLuminance )
	{
		return false;
	}

	s_pucWaypointUserFactors = new u_char[ s_uNumEntries ];
	if ( !s_pucWaypointUserFactors )
	{
		return false;
	}
	Asura_Mem::Set( s_pucWaypointUserFactors, 127, s_uNumEntries * sizeof( u_char ) );

// 	float fMinValue = Asura_Maths::LargeFloat;
// 	float fMaxValue = 0.0f;
	for ( u_int u = 0; u < s_uNumEntries; ++u )
	{
		CalculateLuminanceForWaypoint( static_cast< u_short >( u ) );
// 		if ( s_pxWaypointLuminanceFactors[ u ] < fMinValue )
// 		{
// 			fMinValue = s_pxWaypointLuminanceFactors[ u ];
// 		}
// 		if ( s_pxWaypointLuminanceFactors[ u ] > fMaxValue )
// 		{
// 			fMaxValue = s_pxWaypointLuminanceFactors[ u ];
// 		}
	}
	//Asura_OutputToDebugger_Retail( ( "Waypoint lighting values: max: %f, min: %f\n", fMaxValue, fMinValue ) );

	s_xPendingWPIDList.Clear();
	delete [] s_pucPendingWPIDMaker;
	s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ]; // 1 bit per waypoint
	Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

	return true;
}

void AVP_Waypoint_Manager::ResetUserLuminanceFactors()
{
	delete [] s_pucWaypointUserFactors;
	s_pucWaypointUserFactors = new u_char[ s_uNumEntries ];
	Asura_Mem::Set( s_pucWaypointUserFactors, 127, s_uNumEntries * sizeof( u_char ) );

	s_xPendingWPIDList.Clear();
	delete [] s_pucPendingWPIDMaker;
	s_pucPendingWPIDMaker = new u_char[ s_uNumEntries / 8 + 1 ]; 
	Asura_Mem::Set( s_pucPendingWPIDMaker, 0, (s_uNumEntries / 8 + 1) * sizeof( u_char ) );

}

// void AVP_Waypoint_Manager::IncrementLuminanceFactor( const int iWaypointID, const float fTimeslice )
// {
// 	if ( !s_pucWaypointUserFactors )
// 	{
// 		return;
// 	}
// 
// 	// 0 -> 1 in 8 seconds.
// 	const float fInc = 32.0f * fTimeslice;
// 	if ( GetWPLuminance( iWaypointID ) < 1.0f )
// 	{
// 		s_pucWaypointUserFactors[ iWaypointID ] =
// 			static_cast< u_char >( Asura_Maths::FloatToInt_Near( Asura_Maths::Min( s_pucWaypointUserFactors[ iWaypointID ] + fInc, 255.0f ) ) );
// 	}
// 
// 	const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointID );
// 	for ( u_char uc = 0; uc < pxWP->m_ucNumConnections; ++uc )
// 	{
// 		const int iNextWPID = pxWP->m_pxConnections[ uc ].m_usIndex;
// 		if ( GetWPLuminance( iNextWPID ) < 1.0f )
// 		{
// 			s_pucWaypointUserFactors[ iNextWPID ] =
// 				static_cast< u_char >( Asura_Maths::FloatToInt_Near( Asura_Maths::Min( s_pucWaypointUserFactors[ iNextWPID ] + fInc, 255.0f ) ) );
// 		}
// 	}
// }
// 
// void AVP_Waypoint_Manager::DecrementLuminanceFactor( const int iWaypointID, const float fTimeslice )
// {
// 	if ( !s_pucWaypointUserFactors )
// 	{
// 		return;
// 	}
// 
// 	// 1 -> 0 in 8 seconds.
// 	const float fDec = 32.0f * fTimeslice;
// 	if ( GetWPLuminance( iWaypointID ) > 0.0f )
// 	{
// 		s_pucWaypointUserFactors[ iWaypointID ] =
// 			static_cast< u_char >( Asura_Maths::FloatToInt_Near( Asura_Maths::Max( s_pucWaypointUserFactors[ iWaypointID ] - fDec, 0.0f ) ) );
// 	}
// 
// 	const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( iWaypointID );
// 	for ( u_char uc = 0; uc < pxWP->m_ucNumConnections; ++uc )
// 	{
// 		const int iNextWPID = pxWP->m_pxConnections[ uc ].m_usIndex;
// 		if ( GetWPLuminance( iNextWPID ) > 0.0f )
// 		{
// 			s_pucWaypointUserFactors[ iNextWPID ] =
// 				static_cast< u_char >( Asura_Maths::FloatToInt_Near( Asura_Maths::Max( s_pucWaypointUserFactors[ iNextWPID ] - fDec, 0.0f ) ) );
// 		}
// 	}
// }

void AVP_Waypoint_Manager::DestroyGS2LookupData()
{
	Asura_Hashed_List_Deleter_It< WaypointAndNodes > xListIter( &s_xWaypointNodesLookupList );
	while ( !xListIter.Done() )
	{
		WaypointAndNodes* pxToDelete = xListIter.GetCurrent();
		xListIter.DeleteCurrent();
		delete [] pxToDelete->m_puGS2NodeGuids;
		delete pxToDelete;
	}
}

bool AVP_Waypoint_Manager::CalculateGS2LookupData( bool& bTouchedEntityData )
{
	DestroyGS2LookupData();

	// Iterate through the waypoints and for each GS2 waypoint find the corresponding GS2 nodes.
	for ( u_short usWPID = 0; usWPID < Asura_AI_Navigation_System::GetNumberOfWaypoints(); ++usWPID )
	{
		const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( usWPID );
		if ( !( pxWP->GetFlags() & AVP_Navigation::AVP_AI_NAVVERTEX_FLAG_GS2 ) )
		{
			continue;
		}

		// Find the GS2 nodes with the same pos and ori.
		AVP_ServerNode_GS2Node* apxMatchingNodes[ 128 ];
		u_short					usNumMatchingNodes = 0;
		for ( Asura_Hashed_List_It< AVP_ServerNode_GS2Node > xGS2NodeIter( AVP_ServerNode_GS2Node::GetList() ); !xGS2NodeIter.Done(); xGS2NodeIter.Next() )
		{
			AVP_ServerNode_GS2Node* pxGS2Node = xGS2NodeIter.GetCurrent();
			if ( !pxGS2Node->GetPosition().ApproximatelyEquals( pxWP->m_xPosition ) )
			{
				continue;
			}

			// We have to allow some inaccuracy in the orientation as the waypoint normals are compressed.
			Asura_Matrix_3x3 xNodeOrientM;
			pxGS2Node->GetOrientation( xNodeOrientM );
			const Asura_Vector_3 xNodeUp = Asura_Vector_3( 0.0f, -1.0f, 0.0f ) * xNodeOrientM;
			static const float s_fCos15Degrees = Asura_Maths::Cosine( Asura_Maths::DegToRad( 15.0f ) );
			if ( xNodeUp * pxWP->GetNormal() < s_fCos15Degrees )
			{
				continue;
			}

			if ( usNumMatchingNodes == USHRT_MAX )
			{
				Asura_Assertf( false, ( "AVP_Waypoint_Manager::GenerateGS2LookupData - too many matching nodes for waypoint %u", usWPID ) );
			}

			apxMatchingNodes[ usNumMatchingNodes++ ] = pxGS2Node;
		}

		WaypointAndNodes* pxWPnNodes = new WaypointAndNodes;
        pxWPnNodes->m_usWPID         = usWPID;
        pxWPnNodes->m_usNumNodes     = usNumMatchingNodes;
		pxWPnNodes->m_puGS2NodeGuids = new Asura_Guid[ usNumMatchingNodes ];
		// Copy the matching node guids to the WaypointAndNodes lookup data and also inform each node what its matching waypoint is.
		for ( u_short usMatchingNode = 0; usMatchingNode < usNumMatchingNodes; ++usMatchingNode )
		{
			pxWPnNodes->m_puGS2NodeGuids[ usMatchingNode ] = apxMatchingNodes[ usMatchingNode ]->GetGuid();
			apxMatchingNodes[ usMatchingNode ]->SetWaypointID( usWPID );
			bTouchedEntityData = true;
		}
		s_xWaypointNodesLookupList.AddToList( pxWPnNodes );
	}

	return true;
}

void AVP_Waypoint_Manager::BuildStaticObjectsLOSList()
{
	for ( Asura_Hashed_List_It< Asura_ServerEntity_PhysicalObject > it( Asura_ServerEntity_PhysicalObject::GetList() ); !it.Done(); it.Next() )
	{
		Asura_ServerEntity_PhysicalObject* pxPhysicalObject = it.GetCurrent();

		// If the pointer is valid and its type is of static object add it to our list
		if ( ( pxPhysicalObject ) && ( pxPhysicalObject->GetPhysicalObjectFlags() & ASURA_PHYSICALOBJECT_FLAG_STATIC ) )
		{
			new AVP_DynamicBVH_StaticObjects_Wrapper( pxPhysicalObject );
			
			AVP_DynamicBVH_StaticObjects_Wrapper::IncreaseNumInList();
		}
	}
}

void AVP_Waypoint_Manager::DestroyStaticObjectsLOSList()
{
	for ( Asura_Hashed_List_It< Asura_ServerEntity_PhysicalObject > it( Asura_ServerEntity_PhysicalObject::GetList() ); !it.Done(); it.Next() )
	{
		Asura_ServerEntity_PhysicalObject* pxPhysicalObject = it.GetCurrent();

		// If the pointer is valid and its type is of static object attempt to find it in our list
		if ( ( pxPhysicalObject ) && ( pxPhysicalObject->GetPhysicalObjectFlags() & ASURA_PHYSICALOBJECT_FLAG_STATIC ) )
		{
			AVP_DynamicBVH_StaticObjects_Wrapper* pxWrapperObject = AVP_DynamicBVH_StaticObjects_Wrapper::GetLineOfSightList()->FindEntity( pxPhysicalObject->GetGuid() );

			// If valid destroy it - the destructor will remove it from the list
			if ( pxWrapperObject )
			{
				delete pxWrapperObject;
			}
		}
	}
	
	AVP_DynamicBVH_StaticObjects_Wrapper::ResetNumInList();
}

Asura_Mutex			   AVP_Waypoint_Manager::s_xRequestMutex;

void AVP_Waypoint_Manager::CalcLOFForWaypointCallback( const void* pData )
{
	Asura_Task_Array::TaskDataPacket* pxPacket = ( Asura_Task_Array::TaskDataPacket* )( pData );
	const u_int uBaseWPID = pxPacket->GetArrayStartIndex();
	pxPacket->SignalFinishedWithData();

	// Get the waypoint from the value passed in the packet data
	Asura_AI_NavVertex* pxBaseWaypoint = Asura_AI_Navigation_System::GetWaypoint( uBaseWPID );
	if (!pxBaseWaypoint)
	{
		return;
	}

	// Position to do the LOS test from incorporates average height of Alien/humanoid.
	const Asura_Vector_3 xLOSTestPos = pxBaseWaypoint->GetPosition() + ( pxBaseWaypoint->GetNormal() * 1.2f );

	// Data to store the distances tested
	float afMinDistance[ NUM_SECTORS ];
	float afMaxDistance[ NUM_SECTORS ];
#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
	u_short ausMinWP[ NUM_SECTORS ];
	u_short ausMaxWP[ NUM_SECTORS ];
#endif
	for ( u_int uSector = 0; uSector < NUM_SECTORS; ++uSector )
	{
		afMinDistance[ uSector ] = Asura_Maths::LargeFloat;
		afMaxDistance[ uSector ] = 0.0f;
#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
		ausMinWP[ uSector ] = ASURA_INVALID_WAYPOINT_ID;
		ausMaxWP[ uSector ] = ASURA_INVALID_WAYPOINT_ID;
#endif
	}

	//s_xRequestMutex.Lock();
	AVP_ServerEntity_GamePlayVolume* pxBaseGPVolume = AVP_ServerEntity_GamePlayVolume::Tools_GetVolumeThatContainsPosition( pxBaseWaypoint->GetPosition(), s_uLastGamePlayVolumeGuid );
	//s_xRequestMutex.Unlock();

	if ( !pxBaseGPVolume )
	{
		if (s_bLOFWaypointShouldWriteToLog[ uBaseWPID ] == false )
		{
			s_bLOFWaypointShouldWriteToLog[ uBaseWPID ] = true;
		}

		// Waypoint isn't in a Game Play Volume so set its Min and Max values to 0.0f
		for ( u_int uSector = 0; uSector < NUM_SECTORS; ++uSector )
		{
			s_pxWaypointLOFData[ uBaseWPID ].SetMinLOFValue( uSector, 0.0f );
			s_pxWaypointLOFData[ uBaseWPID ].SetMaxLOFValue( uSector, 0.0f );
		}
		return;
	}

	s_uLastGamePlayVolumeGuid = pxBaseGPVolume->GetGuid();

	// Iterate around every waypoint in the level
	for ( u_int uTestWPID = 0; uTestWPID < s_uNumEntries; ++uTestWPID )
	{
		Asura_AI_NavVertex* pxTestWaypoint = Asura_AI_Navigation_System::GetWaypoint( uTestWPID );
		if ( !pxTestWaypoint || ( uBaseWPID == uTestWPID ) )
		{
			continue;
		}

		//s_xRequestMutex.Lock();
		AVP_ServerEntity_GamePlayVolume* pxTestGPVolume = AVP_ServerEntity_GamePlayVolume::Tools_GetVolumeThatContainsPosition( pxTestWaypoint->GetPosition(), s_uLastGamePlayVolumeGuid );
		//s_xRequestMutex.Unlock();

		if ( !pxTestGPVolume )
		{
			if (s_bLOFWaypointShouldWriteToLog[ uTestWPID ] == false )
			{
				s_bLOFWaypointShouldWriteToLog[ uTestWPID ] = true;
			}
			continue;
		}

		s_uLastGamePlayVolumeGuid = pxTestGPVolume->GetGuid();

		// Test to see if the waypoint is in the base GP volume or linked volumes.
		if ( pxTestGPVolume->GetGuid() != pxBaseGPVolume->GetGuid() )
		{
			bool bFound = false;

			for ( u_int uGPLink = 0; uGPLink < pxBaseGPVolume->GetNumLinkedGamePlayVolumes(); ++uGPLink )
			{
				if ( pxBaseGPVolume->GetLinkedGamePlayVolumeGuid( uGPLink ) == pxTestGPVolume->GetGuid() )
				{
					bFound  = true;
					break;
				}
			}

			if ( !bFound )
			{
				continue;
			}
		}

		// The direction from our base waypoint to the test waypoint
		Asura_Vector_3 xBaseToTestDir = pxTestWaypoint->m_xPosition - xLOSTestPos;
		// We require the distance for LOS testing purposes
		const float	   fDistance	  = xBaseToTestDir.Normalise();

		// Ignore any waypoints not near enough.
		if ( fDistance > GetMaxLOFTestDistance() )
		{
			continue;
		}

		// Do the LOS test taking into account the things we care about
		const float fDistanceToIntersection = EnvironmentAndStaticObjectsLOSTest( xLOSTestPos, xBaseToTestDir, fDistance );

		// Evaluate the result and set data.
		const u_int uSector = GetSectorFromDirection( xBaseToTestDir );
		// Allow a fudge factor of 5cm in the collision test in case the waypoint isn't exactly flush with the polys.
		if ( fDistanceToIntersection < fDistance - 0.05f )
		{
			// We have hit something - update the minimum LOF distance.
			if ( fDistanceToIntersection < afMinDistance[ uSector ] )
			{
				afMinDistance[ uSector ] = fDistanceToIntersection;
#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
				ausMinWP[ uSector ] = static_cast< u_short >( uTestWPID );
#endif
			}
		}

		// We can always update the maximum LOF distance.
		if ( fDistanceToIntersection > afMaxDistance[ uSector ] )
		{
			afMaxDistance[ uSector ] = fDistanceToIntersection;
#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
			ausMaxWP[ uSector ] = static_cast< u_short >( uTestWPID );
#endif
		}
	}

	// Set the Polar LOF data into our Polar LOF data array
	for ( u_int uSector = 0; uSector < NUM_SECTORS; ++uSector )
	{
		// If there were no valid waypoints in the sector the max distance will be unset (0.0f).
		// If this is the case then set it a valid value.
		if ( afMaxDistance[ uSector ] == 0.0f )
		{
			if ( afMinDistance[ uSector ] != Asura_Maths::LargeFloat )
			{
				afMaxDistance[ uSector ] = afMinDistance[ uSector ];
#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
				ausMaxWP[ uSector ] = ausMinWP[ uSector ];
#endif
			}
			else
			{
				afMaxDistance[ uSector ] = pxBaseWaypoint->GetRadius();
			}
		}
		// If we hit nothing in the sector, or there were no valid waypoints in the sector the min distance will be unset (Asura_Maths::LargeFloat).
		// If this is the case then set it to the max value (which will be valid now).
		if ( afMinDistance[ uSector ] == Asura_Maths::LargeFloat )
		{
			afMinDistance[ uSector ] = afMaxDistance[ uSector ];
#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
			ausMinWP[ uSector ] = ausMaxWP[ uSector ];
#endif
		}

		s_pxWaypointLOFData[ uBaseWPID ].SetMinLOFValue( uSector, afMinDistance[ uSector ] );
		s_pxWaypointLOFData[ uBaseWPID ].SetMaxLOFValue( uSector, afMaxDistance[ uSector ] );
#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
		s_pxWaypointLOFData[ uBaseWPID ].SetMinWP( uSector, ausMinWP[ uSector ] );
		s_pxWaypointLOFData[ uBaseWPID ].SetMaxWP( uSector, ausMaxWP[ uSector ] );
#endif
	}
}

void AVP_Waypoint_Manager::VerifyWaypointsForLOF()
{
	s_uLastGamePlayVolumeGuid = ASURA_GUID_UNREGISTERED;
	bool bWrittenAtLeastOneError = false;

	for ( u_int uWPID = 0; uWPID < s_uNumEntries; ++uWPID )
	{
		Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( uWPID );
		
		if ( !pxWP ) { continue; }

		AVP_ServerEntity_GamePlayVolume* pxGPV = AVP_ServerEntity_GamePlayVolume::Tools_GetVolumeThatContainsPosition( pxWP->GetPosition(), s_uLastGamePlayVolumeGuid );

		if ( pxGPV )
		{
			s_uLastGamePlayVolumeGuid = pxGPV->GetGuid();
		}
		else
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::WAYPOINTING, "", "Waypoint %d is not inside a Game Play Volume : Co-ords X: %.2f, Y: %.2f, Z: %.2f", uWPID, pxWP->GetPosition().x, pxWP->GetPosition().y, pxWP->GetPosition().z );

			if ( !bWrittenAtLeastOneError )
			{
				bWrittenAtLeastOneError = true;
			}
		}
	}

	if ( bWrittenAtLeastOneError )
	{
		Tools_MessageLog::ShowOverlay( Tools_MessageLog::WAYPOINTING, "" );
	}
	else
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::WAYPOINTING, "", " No Problems Found For Line Of Fire Data Generation" );
	}
}

void AVP_Waypoint_Manager::DestroyLOFLookUpTableData()
{
	delete [] s_pxWaypointLOFData;
	s_pxWaypointLOFData = NULL;
}

bool AVP_Waypoint_Manager::CalculateLOFLookUpTableData()
{
	DestroyLOFLookUpTableData();

	// Create our data array
	s_pxWaypointLOFData = new WaypointLOFData[ s_uNumEntries ];
	
	// if our array of data isn't valid return false as everything will fail
	if (!s_pxWaypointLOFData)
	{
		return false;
	}

	s_bLOFWaypointShouldWriteToLog = new bool[ s_uNumEntries ];

	for ( u_int u = 0; u < s_uNumEntries; ++u )
	{
		s_bLOFWaypointShouldWriteToLog[u] = false;
	}

	// Ensure the Environmental LOS data is valid
	Tools_Environment_LOS::EnsureDataIsValid();

	//Reset the number in list so we don't have to test against statics if there are not any.
	AVP_DynamicBVH_StaticObjects_Wrapper::ResetNumInList();
	// Create Our Static Objects List
	BuildStaticObjectsLOSList();
	// Create the BVH Tree
    AVP_DynamicBVH_StaticObjects_Wrapper_Iterator xIt( AVP_DynamicBVH_StaticObjects_Wrapper::GetLineOfSightList() );
	s_xStaticLineOfSightBVH.Create<AVP_DynamicBVH_StaticObjects_Wrapper>( xIt );
	// Refit the BVH Tree
	s_xStaticLineOfSightBVH.Refit<AVP_DynamicBVH_StaticObjects_Wrapper>();



	// -----------------------------------------------------------
	// Multi Thread the process
	// -----------------------------------------------------------
	bool bShouldQuit = false;

	// ---------------------------------------------------------------------------------------------------------------------
	// *** BRACES ARE IMPORTANT WHEN THE xTaskArray GOES OUT OF SCOPE THE DESTRUCTOR YIELDS UNTIL ALL TASKS ARE COMPLETE ***
	// ---------------------------------------------------------------------------------------------------------------------
	{
		// setup out Task Manager Array
		Asura_Task_Array xTaskArray( s_uNumEntries, 1, CalcLOFForWaypointCallback );

		// Loop until the task is finished
		do 
		{
			// Call update progress and check fro an abort
			bShouldQuit = Tools_Mode_Waypointing::UpdateProgress( xTaskArray.GetCurrentIndex(), s_uNumEntries, "Calculating Line Of Fire Data..." );

			// Abort the process if the user as aborted
			if ( bShouldQuit ) 
			{
				break;
			}

			Sleep( 100 );

		} 
		while( !xTaskArray.HasFinished() );
	}
	// ---------------------------------------------------------------------------------------------------------------------
	// *** BRACES ARE IMPORTANT WHEN THE xTaskArray GOES OUT OF SCOPE THE DESTRUCTOR YIELDS UNTIL ALL TASKS ARE COMPLETE ***
	// ---------------------------------------------------------------------------------------------------------------------

	// Destroy our list of static objects used in the DynamicBVH LOS Test
	DestroyStaticObjectsLOSList();

	for ( u_int uWPID = 0; uWPID < s_uNumEntries; ++uWPID )
	{
		if ( s_bLOFWaypointShouldWriteToLog[ uWPID ] )
		{
			Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( uWPID );
			Tools_MessageLog::AddMessage( Tools_MessageLog::WAYPOINTING, "", "Waypoint %d is not inside a Game Play Volume : Co-ords X: %.2f, Y: %.2f, Z: %.2f", uWPID, pxWP->GetPosition().x, pxWP->GetPosition().y, pxWP->GetPosition().z );
		}
	}

	if ( s_bLOFWaypointShouldWriteToLog != NULL )
	{
		delete [] s_bLOFWaypointShouldWriteToLog;
		s_bLOFWaypointShouldWriteToLog = NULL;
	}

	// If the process was aborted or failed delete our data array and return false.
	if ( bShouldQuit )
	{
		DestroyLOFLookUpTableData();
		return false;
	}
	else
	{
		return true;
	}
}

const float AVP_Waypoint_Manager::EnvironmentAndStaticObjectsLOSTest( const Asura_Vector_3& xOrigin, const Asura_Vector_3& xDirection, float fDistance )
{
	// Test against the environment.
	Tools_Environment_LOS_Data xInput( xOrigin, xDirection );
	xInput.SetPolyFlagsToIgnore( AsuraPolygonFlag_NoCollisions | AVP_PolygonFlag_IgnoreForPhysicsObjectCollision );
	if ( Tools_Environment_LOS::TestWithRay( xInput ) )
	{
		// If we hit the environment we can reduce the distance we test static objects over.
		fDistance = xInput.GetDistance();
	}

	// Test against static objects.
	if ( AVP_DynamicBVH_StaticObjects_Wrapper::GetNumInList() > 0 )
	{
		// Build input data for the DynamicBVH ray test
		const Asura_LOS_InputData xInputData( xOrigin, xDirection, ASURA_GUID_UNREGISTERED, fDistance );
		Asura_LOS_OutputData	  xNewOutput;
		xNewOutput.Reset( fDistance );
		s_xStaticLineOfSightBVH.TestRay( xInputData, xNewOutput );
		// Modify the distance based on the LOS results.
		fDistance = xNewOutput.m_fDistanceToIntersection;
	}

	return fDistance;
}

void AVP_Waypoint_Manager::GetSectorLimits( const u_int uSector, Asura_Vector_3& xLimitOne, Asura_Vector_3& xLimitTwo )
{
	static const float s_fCosSectorAngle = 0.70710678f; // Asura_Maths::Cosine( Asura_Maths::DegToRad( 45.0f ) );
	static const Asura_Vector_3 s_axSectorLimits[ NUM_SECTORS ] =
	{
		Asura_Vector_3( 0.0f, 0.0f, 1.0f ),
		Asura_Vector_3( s_fCosSectorAngle, 0.0f, s_fCosSectorAngle ),
		Asura_Vector_3( 1.0f, 0.0f, 0.0f ),
		Asura_Vector_3( s_fCosSectorAngle, 0.0f, -s_fCosSectorAngle ),
		Asura_Vector_3( 0.0f, 0.0f, -1.0f ),
		Asura_Vector_3( -s_fCosSectorAngle, 0.0f, -s_fCosSectorAngle ),
		Asura_Vector_3( -1.0f, 0.0f, 0.0f ),
		Asura_Vector_3( -s_fCosSectorAngle, 0.0f, s_fCosSectorAngle )
	};

	xLimitOne = s_axSectorLimits[ uSector ];
	xLimitTwo = s_axSectorLimits[ ( uSector + 1 ) % NUM_SECTORS ];
}

float AVP_Waypoint_Manager::GetMinLOFValue( const u_int uWaypointID, const u_int uSector )
{
	if ( s_pxWaypointLOFData )
	{
		return s_pxWaypointLOFData[ uWaypointID ].GetMinLOFValue( uSector );
	}
	else
	{
		return 0.0f;
	}
}

float AVP_Waypoint_Manager::GetMaxLOFValue( const u_int uWaypointID, const u_int uSector )
{
	if ( s_pxWaypointLOFData )
	{
		return s_pxWaypointLOFData[ uWaypointID ].GetMaxLOFValue( uSector );
	}
	else
	{
		return 0.0f;
	}
}

#ifdef AVP_INCLUDE_LOF_DEBUG_INFO
u_short AVP_Waypoint_Manager::GetMinLOFWPID( const u_int uWaypointID, const u_int uSector )
{
	if ( s_pxWaypointLOFData )
	{
		return s_pxWaypointLOFData[ uWaypointID ].GetMinWP( uSector );
	}
	else
	{
		return ASURA_INVALID_WAYPOINT_ID;
	}
}

u_short AVP_Waypoint_Manager::GetMaxLOFWPID( const u_int uWaypointID, const u_int uSector )
{
	if ( s_pxWaypointLOFData )
	{
		return s_pxWaypointLOFData[ uWaypointID ].GetMaxWP( uSector );
	}
	else
	{
		return ASURA_INVALID_WAYPOINT_ID;
	}
}
#endif // AVP_INCLUDE_LOF_DEBUG_INFO
#endif // ASURA_TOOLS

bool AVP_Waypoint_Manager::HasCoverFrom( const u_int uBaseWPID, const u_int uThreatWPID )
{
	if ( !s_pxWaypointLOFData ) { return false; }

	const Asura_AI_NavVertex* pxBaseWP = Asura_AI_Navigation_System::GetWaypoint( uBaseWPID );
	const Asura_AI_NavVertex* pxThreatWP = Asura_AI_Navigation_System::GetWaypoint( uThreatWPID );
	if ( !pxBaseWP || !pxThreatWP ) { return false; }

	Asura_Vector_3 xBaseToThreat = pxThreatWP->GetPosition() - pxBaseWP->GetPosition();
	const float    fBaseToThreat = xBaseToThreat.Normalise();
	const u_int uSector = GetSectorFromDirection( xBaseToThreat );
	const u_int uSectorDash = GetOppositeSector( uSector );

	return ( s_pxWaypointLOFData[ uBaseWPID ].GetMaxLOFValue( uSector ) < fBaseToThreat ||
		s_pxWaypointLOFData[ uThreatWPID ].GetMaxLOFValue( uSectorDash ) < fBaseToThreat );
}

float AVP_Waypoint_Manager::GetMinLOFValue( const u_int uWaypointID, const Asura_Vector_3& xDirection )
{
	if ( s_pxWaypointLOFData && (uWaypointID != ASURA_INVALID_WAYPOINT_ID) )
	{
		const u_int uSector = GetSectorFromDirection( xDirection );
		return s_pxWaypointLOFData[ uWaypointID ].GetMinLOFValue( uSector );
	}
	else
	{
		return 0.0f;
	}
}

float AVP_Waypoint_Manager::GetMaxLOFValue( const u_int uWaypointID, const Asura_Vector_3& xDirection )
{
	if ( s_pxWaypointLOFData && (uWaypointID != ASURA_INVALID_WAYPOINT_ID) )
	{
		const u_int uSector = GetSectorFromDirection( xDirection );
		return s_pxWaypointLOFData[ uWaypointID ].GetMaxLOFValue( uSector );
	}
	else
	{
		return 0.0f;
	}
}

void AVP_Waypoint_Manager::GetMinLOFDirection( const u_int uWaypointID, Asura_Vector_3& xDirection )
{
	if (uWaypointID == ASURA_INVALID_WAYPOINT_ID)
	{
		xDirection.Zero();
	}
	else if ( s_pxWaypointLOFData )
	{
		float fMin = Asura_Maths::LargeFloat;
		u_int uBestSector = 0;
		for ( u_int uSector = 0; uSector < NUM_SECTORS; ++uSector )
		{
			const float fSectorMin = s_pxWaypointLOFData[ uWaypointID ].GetMinLOFValue( uSector );
			if ( fSectorMin < fMin )
			{
				fMin = fSectorMin;
				uBestSector = uSector;
			}
		}

		CalcDirectionFromSector( uBestSector, xDirection );
	}
}

void AVP_Waypoint_Manager::GetMaxLOFDirection( const u_int uWaypointID, Asura_Vector_3& xDirection )
{
	if (uWaypointID == ASURA_INVALID_WAYPOINT_ID)
	{
		xDirection.Zero();
	}
	else if ( s_pxWaypointLOFData )
	{
		float fMax = -1.0f;
		u_int uBestSector = 0;
		for ( u_int uSector = 0; uSector < NUM_SECTORS; ++uSector )
		{
			const float fSectorMax = s_pxWaypointLOFData[ uWaypointID ].GetMaxLOFValue( uSector );
			if ( fSectorMax > fMax )
			{
				fMax = fSectorMax;
				uBestSector = uSector;
			}
		}

		CalcDirectionFromSector( uBestSector, xDirection );
	}
	else
	{
		Asura_Assert( false, "Invalid waypoint line-of-fire data, please recalculate this mission's Extra Waypointing Data in AVP Waypointing Mode in Tools, thank you." );
		xDirection.Zero();
	}
}


#if !defined(ASURA_NO_DEBUG_OUTPUT)
void AVP_Waypoint_Manager::RenderDebugInfo()
{
	if ( !s_bRenderDebugInfo )
	{
		return;
	}

	Asura_Vector_3 xCameraForward;
	Asura_Camera::GetForwardVector( xCameraForward );

	for ( u_int u = 0; u < s_uNumEntries; ++u )
	{
		const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( u );
		const Asura_Vector_3& xWPNormal = pxWP->GetNormal();
		
		if ( xCameraForward * xWPNormal > 0.0f )
		{
			continue;
		}
		const float fToCameraSq = ( *Asura_Camera::GetPosition() - pxWP->m_xPosition ).MagnitudeSquared();
		if ( fToCameraSq > 2500.0f )
		{
			continue;
		}

		Asura_Vector_3 xOutputPos = pxWP->m_xPosition;
		if ( Asura_Camera::ProjectWorldCoordsToScreenSpace( xOutputPos, true ) && xOutputPos.z >= 0.0f )
		{
			Asura_Font_System::Print( xOutputPos.x, xOutputPos.y, 1.0f, 0xffff0000,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING,
				"%0.2f", AVP_Waypoint_Manager::GetWPLuminance( u ) );
		}
	}
}
#endif

void AVP_Waypoint_Manager::CollectWPForBoundingBox( const Asura_Bounding_Box& xBB )
{
	// Find the waypoints that are affected.
	Asura_AI_Navigation_System::SearchWaypoints( xBB, AddWaypointForLuminanceCalculation );
}

DEBUGVAR float dbg_fGIConst3 = 2.0f;  // This is here to match a legacy x 2 in the material response shader.

bool AVP_Waypoint_Manager::CalculateLuminanceForWaypoint( u_short usWaypointID )
{

	const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( usWaypointID );
	if ( !pxWP ) return false;
	const Asura_Vector_3& xWPNormal = pxWP->GetNormal();

	// First calculate the static luminance.
	Asura_SphericalHarmonics_Order2_RGB xSHResult;
	Asura_GI_Solution::GetStaticIrradiance( xSHResult, pxWP->m_xPosition );
	Asura_RGBA xRGBA;
	xSHResult.EvaluateInDirection( xWPNormal, xRGBA );
	float fLuminance = xRGBA.GetLuminance() * dbg_fGIConst3;

	// Next approximate the extra luminance value from lights/pfx/flares/etc.
	for ( Asura_Hashed_List_ConstIt< Asura_ClientEntity_DynamicLight > xLightIter( Asura_ClientEntity_DynamicLight::GetList() ); !xLightIter.Done(); xLightIter.Next() )
	{
		const Asura_ClientEntity_DynamicLight* pxLight = xLightIter.GetCurrent();
		fLuminance += GetLightLuminance( *pxLight, *pxWP );
	}

	for ( Asura_Hashed_List_ConstIt< AVP_ServerProjectile_HumanFlare > xFlareIter( AVP_ServerProjectile_HumanFlare::GetList() ); !xFlareIter.Done(); xFlareIter.Next() )
	{
		const AVP_ServerProjectile_HumanFlare* pxFlare = xFlareIter.GetCurrent();
		fLuminance += GetFlareLuminance( *pxFlare, *pxWP );
	}

	for ( Asura_Hashed_List_ConstIt< Asura_ServerEntity_Container > xIter( Asura_ServerEntity_Container::GetList() ); !xIter.Done(); xIter.Next() )
	{
		const AVP_ServerEntity_Container* pxContainerEntity = static_cast<const AVP_ServerEntity_Container*>(xIter.GetCurrent());
		fLuminance += GetContainerLuminance( *pxContainerEntity, *pxWP );
	}

	// Now convert our float luminance into a HalfFloat.
	s_pxWaypointLuminance[ usWaypointID ] = Asura_HalfFloat( fLuminance );

	return true;
}

bool AVP_Waypoint_Manager::AddWaypointForLuminanceCalculation( u_short usWaypointID )
{
	if (s_pucPendingWPIDMaker[usWaypointID / 8 ] & ( 1 << (usWaypointID & 0x7 )))
	{
		return true;
	}

	s_pucPendingWPIDMaker[usWaypointID / 8 ] |= ( 1 << (usWaypointID & 0x7 ));
	s_xPendingWPIDList.Push_Back(usWaypointID);

	return true;
}

float AVP_Waypoint_Manager::GetLightLuminance( const Asura_ClientEntity_DynamicLight& xLight, const Asura_AI_NavVertex& xWaypoint )
{
	// Do a rough calculation to determine the luminance.
	if ( !xLight.IsEntityActive() ) { return 0.0f; }

	const Asura_DynamicLight_Parameters& xParams = xLight.GetDynamicLightParams();
	Asura_Vector_3 xToLight = ( xParams.m_xPosition - xWaypoint.m_xPosition );
	const float    fToLight = xToLight.Normalise();
	const Asura_Vector_3& xWPNormal = xWaypoint.GetNormal();

	if ( xParams.m_eType == ASURA_DYNAMICLIGHT_PHOTON )
	{
		if ( xParams.m_uFlags & ASURA_DYNAMICLIGHTFLAG_USEPHOTONLIMITINGANGLES )
		{
			// Assuming m_fLimitingAngleInner == 0.0f for speed and simplicity.
			const Asura_Vector_3 xLightForwards( xParams.m_xOrientation.e13, xParams.m_xOrientation.e23, xParams.m_xOrientation.e33 );
			const float fAngle = Asura_Maths::ArcCosine( xLightForwards * -xToLight );
			const float fLimitingAngleScale = ( 0.5f * xParams.m_fLimitingAngleOuter ) - fAngle;
			if ( fLimitingAngleScale <= 0.0f )
			{
				return 0.0f;
			}

			float fDist = fToLight/xParams.m_fPhotonRange;
			if (fDist < 0.0f || fDist > 1.0f)
			{
				return 0.0f;
			}

			float fAngleScale = xToLight * xWPNormal;
			if ( fAngleScale <= 0.0f )
			{
				const Asura_Vector_3 xTestPoint = xWaypoint.m_xPosition + xWPNormal * 1.5f;
				Asura_Vector_3 xTestToLight = xParams.m_xPosition - xTestPoint;
				float fTestToLight = xTestToLight.Normalise();
				const Asura_LOS_InputData	xLOSInput(xTestPoint, xTestToLight, ASURA_GUID_UNREGISTERED, fTestToLight);
				const Asura_LOS_OutputData*	pxLOSOutput	= Asura_LOS::TestAgainstEverything(&xLOSInput);

				if (pxLOSOutput->m_bHitEnvironment)
				{
					return 0.0f;
				}
			}

			// ignore the angle scale since an object on the point will probably reflect more light
			fAngleScale = 1.0f;			
			float fScale;
			if (xParams.m_fFalloffInnerRatio >= 0.99f)
			{

				fScale = 100.0f;
			}
			else
			{
				fScale = (1.0f / (1.0f - xParams.m_fFalloffInnerRatio));
			}
			float fFallOff = Asura_Maths::Min(1.0f, Asura_Maths::Power(Asura_Maths::Abs((1.0f - fDist) * fScale), xParams.m_fFalloffPower )); 
			const float afLUMINANCE_CONSTANTS[3] = { 0.27f, 0.67f, 0.06f };
			const float fProjRGBLum = ( xParams.m_fBaseR * afLUMINANCE_CONSTANTS[ 0 ] + xParams.m_fBaseG * afLUMINANCE_CONSTANTS[ 1 ] + xParams.m_fBaseB * afLUMINANCE_CONSTANTS[ 2 ] );

			return ( xParams.m_fBaseIntensity * fAngleScale * fProjRGBLum * fFallOff * dbg_fGIConst3);
		}
		else
		{
			float fDist = fToLight/xParams.m_fPhotonRange;
			if (fDist < 0.0f || fDist > 1.0f)
			{
				return 0.0f;
			}

			float fAngleScale = xToLight * xWPNormal;
			if ( fAngleScale <= 0.0f )
			{
				const Asura_Vector_3 xTestPoint = xWaypoint.m_xPosition + xWPNormal * 1.5f;
				Asura_Vector_3 xTestToLight = xParams.m_xPosition - xTestPoint;
				float fTestToLight = xTestToLight.Normalise();
				const Asura_LOS_InputData	xLOSInput(xTestPoint, xTestToLight, ASURA_GUID_UNREGISTERED, fTestToLight);
				const Asura_LOS_OutputData*	pxLOSOutput	= Asura_LOS::TestAgainstEverything(&xLOSInput);

				if (pxLOSOutput->m_bHitEnvironment)
				{
					return 0.0f;
				}
			}

			// ignore the angle scale since an object on the point will probably reflect more light
			fAngleScale = 1.0f;

			float fScale;
			if (xParams.m_fFalloffInnerRatio >= 0.99f)
			{

				fScale = 100.0f;
			}
			else
			{
				fScale = (1.0f / (1.0f - xParams.m_fFalloffInnerRatio));
			}
			float fFallOff = Asura_Maths::Min(1.0f, Asura_Maths::Power(Asura_Maths::Abs((1.0f - fDist) * fScale), xParams.m_fFalloffPower )); 
			const float afLUMINANCE_CONSTANTS[3] = { 0.27f, 0.67f, 0.06f };
			const float fProjRGBLum = ( xParams.m_fBaseR * afLUMINANCE_CONSTANTS[ 0 ] + xParams.m_fBaseG * afLUMINANCE_CONSTANTS[ 1 ] + xParams.m_fBaseB * afLUMINANCE_CONSTANTS[ 2 ] );

			return ( xParams.m_fBaseIntensity * fAngleScale * fProjRGBLum * fFallOff * dbg_fGIConst3 );
		}
	}
	else if ( xParams.m_eType == ASURA_DYNAMICLIGHT_PROJECTOR || xParams.m_eType == ASURA_DYNAMICLIGHT_SPOTLIGHT )
	{
		// We'll have to make some simplifications for these.
		// If these move by too much then we can't take them into account.
		if ( xParams.m_xSwingDist.MagnitudeSquared() > 300.0f )
		{
			return 0.0f;
		}
		if ( xParams.m_xSwingDist.MagnitudeSquared() > 1.0f && !xParams.m_abSwing )
		{
			return 0.0f;
		}
		if ( !xLight.GetProjectorPtr() )
		{
			return 0.0f;
		}

		if ( !xLight.GetProjectorPtr()->PointInFrustum( xWaypoint.m_xPosition ) )
		{
			return 0.0f;
		}

		// Do a rough calculation to determine the luminance.
		const float fIntensity = xLight.GetProjectorPtr()->GetIntensity();
		if ( fIntensity <= 0.0f ) { return 0.0f; }

		Asura_Vector_3 xToLight = ( xLight.GetProjectorPtr()->GetPosition() - xWaypoint.m_xPosition );
		const float    fToLight = xToLight.Normalise();

		float fDist = (fToLight - xLight.GetProjectorPtr()->GetMinRange())/(xLight.GetProjectorPtr()->GetMaxRange() - xLight.GetProjectorPtr()->GetMinRange());
		if (fDist < 0.0f || fDist > 1.0f)
		{
			return 0.0f;
		}

		float fScale = xLight.GetProjectorPtr()->GetFalloffScale();

		float fFallOff = Asura_Maths::Min(1.0f, Asura_Maths::Power(Asura_Maths::Abs((1.0f - fDist) * fScale), xLight.GetProjectorPtr()->GetFalloffPower() )); 
		if (xParams.m_eType == ASURA_DYNAMICLIGHT_SPOTLIGHT && xParams.m_fInnerFoV <  xParams.m_fFoV )
		{
			const Asura_Vector_3 xLightForward(xParams.m_xOrientation.e13, xParams.m_xOrientation.e23, xParams.m_xOrientation.e33); 
			const float fInner = Asura_Maths::Cosine(xParams.m_fInnerFoV/2.0f);
			const float fOuter = Asura_Maths::Cosine(xParams.m_fFoV/2.0f);
			const float fDot = (-xToLight * xLightForward);
			const float fDirFalloff = Asura_Maths::ClampToRange((fOuter - fDot) / (fOuter - fInner), 0.0f, 1.0f);
			fFallOff *= fDirFalloff;
		}

		const float* afProjRGB = xLight.GetProjectorPtr()->GetRGB();
		const float afLUMINANCE_CONSTANTS[3] = { 0.27f, 0.67f, 0.06f };
		const float fProjRGBLum = ( afProjRGB[ 0 ] * afLUMINANCE_CONSTANTS[ 0 ] + afProjRGB[ 1 ] * afLUMINANCE_CONSTANTS[ 1 ] + afProjRGB[ 2 ] * afLUMINANCE_CONSTANTS[ 2 ] );
		return ( fIntensity * fFallOff * fProjRGBLum * 2.0f ); // This 2.0 is because there's one in Asura_Platform_Projector_System.cpp on m_fDistanceFade ( line 515 in PC, 650 on 360, and somewhere mystical on the PS3 ) 
	}
	else if ( xParams.m_eType == ASURA_DYNAMICLIGHT_INFINITE )
	{
		// Check we in the direction of the light.
		const Asura_Vector_3 xLightForwards( xParams.m_xOrientation.e13, xParams.m_xOrientation.e23, xParams.m_xOrientation.e33 );
		if ( xLightForwards * xToLight < 0.0f )
		{
			const float fRangeScale = ( 1.0f - fToLight / xParams.m_fMaxRange );
			if ( fRangeScale <= 0.0f )
			{
				return 0.0f;
			}

			float fAngleScale = xToLight * xWPNormal;
			if ( fAngleScale <= 0.0f )
			{
				const Asura_Vector_3 xTestPoint = xWaypoint.m_xPosition + xWPNormal * 1.5f;
				Asura_Vector_3 xTestToLight = xParams.m_xPosition - xTestPoint;
				float fTestToLight = xTestToLight.Normalise();
				const Asura_LOS_InputData	xLOSInput(xTestPoint, xTestToLight, ASURA_GUID_UNREGISTERED, fTestToLight);
				const Asura_LOS_OutputData*	pxLOSOutput	= Asura_LOS::TestAgainstEverything(&xLOSInput);

				if (pxLOSOutput->m_bHitEnvironment)
				{
					return 0.0f;
				}
			}
			fAngleScale = 1.0f;
			const float afLUMINANCE_CONSTANTS[3] = { 0.27f, 0.67f, 0.06f };
			const float fProjRGBLum = ( xParams.m_fBaseR * afLUMINANCE_CONSTANTS[ 0 ] + xParams.m_fBaseG * afLUMINANCE_CONSTANTS[ 1 ] + xParams.m_fBaseB * afLUMINANCE_CONSTANTS[ 2 ] );

			return ( xParams.m_fBaseIntensity * fProjRGBLum * fAngleScale * fRangeScale );
		}
		else
		{
			return 0.0f;
		}
	}
	else
	{
		return 0.0f;
	}
}

float AVP_Waypoint_Manager::GetProjectorLuminance( const Asura_Projector& xProjector, const Asura_AI_NavVertex& xWaypoint )
{
	// Do a rough calculation to determine the luminance.
	const float fIntensity = xProjector.GetIntensity();
	if ( fIntensity <= 0.0f ) { return 0.0f; }

	Asura_Vector_3 xToLight = ( xProjector.GetPosition() - xWaypoint.m_xPosition );
	const float    fToLight = xToLight.Normalise();

	if ( !xProjector.PointInFrustum( xWaypoint.m_xPosition ) )
	{
		return 0.0f;
	}

	float fDist = (fToLight - xProjector.GetMinRange())/(xProjector.GetMaxRange() - xProjector.GetMinRange());
	if (fDist < 0.0f || fDist > 1.0f)
	{
		return 0.0f;
	}

	float fScale = xProjector.GetFalloffScale();

	float fFallOff = Asura_Maths::Min(1.0f, Asura_Maths::Power(Asura_Maths::Abs((1.0f - fDist) * fScale), xProjector.GetFalloffPower() )); 

	const float* afProjRGB = xProjector.GetRGB();
	const float afLUMINANCE_CONSTANTS[3] = { 0.27f, 0.67f, 0.06f };
	const float fProjRGBLum = ( afProjRGB[ 0 ] * afLUMINANCE_CONSTANTS[ 0 ] + afProjRGB[ 1 ] * afLUMINANCE_CONSTANTS[ 1 ] + afProjRGB[ 2 ] * afLUMINANCE_CONSTANTS[ 2 ] );
	return ( fIntensity * fFallOff * fProjRGBLum * 2.0f ); // This 2.0 is because there's one in Asura_Platform_Projector_System.cpp on m_fDistanceFade ( line 515 in PC, 650 on 360, and somewhere mystical on the PS3 ) 
}

float AVP_Waypoint_Manager::GetFlareLuminance( const AVP_ServerProjectile_HumanFlare& xFlare, const Asura_AI_NavVertex& xWaypoint )
{
	if (xFlare.IsFlareExpired())
	{
		return 0.0f;
	}

	const float fRadius = AVP_ServerProjectile_HumanFlare::GetFlareAILightRadius();
	float fDistance = ( xFlare.GetPosition() - xWaypoint.m_xPosition ).Magnitude();
	if ( fDistance < fRadius)
	{
		return AVP_ServerProjectile_HumanFlare::GetFlareAILightIntensity() * 0.27f * (fRadius - fDistance) / fRadius; // flare is mostly red so multiply by .27 which is the lumanance at red.
	}
	else
	{
		return 0.0f;
	}
}

// Do a rough calculation to determine the luminance.
float AVP_Waypoint_Manager::GetContainerLuminance( const AVP_ServerEntity_Container& xContainerEntity, const Asura_AI_NavVertex& xWaypoint )
{
	if ( !xContainerEntity.IsEntityActive() )  return 0.0f;

	const Asura_Vector_3& xWPPosition = xWaypoint.m_xPosition;
	const Asura_Vector_3& xWPNormal = xWaypoint.GetNormal();
	return xContainerEntity.GetContainerLuminance(xWPPosition, xWPNormal);
}

void AVP_Waypoint_Manager::SetBBFromLight( const Asura_ServerEntity_DynamicLight& xLight, Asura_Bounding_Box& xBB )
{
	switch ( xLight.GetParams().m_eType )
	{
	case ASURA_DYNAMICLIGHT_PHOTON:
		xBB.SetToPoint( xLight.GetParams().m_xPosition );
		xBB.Grow( xLight.GetParams().m_fPhotonRange );
		break;
	case ASURA_DYNAMICLIGHT_PROJECTOR:
	case ASURA_DYNAMICLIGHT_SPOTLIGHT:
		{
			Asura_ClientEntity_DynamicLight *pxLight = Asura_ClientEntity_DynamicLight::FindEntity(xLight.GetGuid());
			Asura_Assert(pxLight, "Light Not Found");
			if (pxLight)
			{
				xBB = pxLight->GetProjectorPtr()->GetBoundingBox();
			}
			else
			{
				xBB.Reset();
			}
		}
		break;
	default:
		Asura_Assert( false, "AVP_Waypoint_Manager::SetBBFromLight - unhandled light type" );
		break;
	}

// 	switch ( xLight.GetParams().m_eType )
// 	{
// 		case ASURA_DYNAMICLIGHT_PHOTON:
// 		{
// 			xBB.SetToPoint( xLight.GetParams().m_xPosition );
// 			xBB.Grow( xLight.GetParams().m_fPhotonRange );
// 			break;
// 		}
// 
// 		case ASURA_DYNAMICLIGHT_PROJECTOR:
// 		case ASURA_DYNAMICLIGHT_SPOTLIGHT:
// 		{
// 			float fMinRange = xLight.GetParams().m_fMinRange;
// 			const float fMaxRange = xLight.GetParams().m_fMaxRange;
// 			Asura_Vector_3 xVertices[ 8 ];
// 			const Asura_Vector_3& xPos = xLight.GetParams().m_xPosition;
// 			const Asura_Matrix_3x3& xOrient = xLight.GetParams().m_xOrientation;
// 			const float fScale = xLight.GetParams().m_fScale;
// 			const float fFoV = xLight.GetParams().m_fFoV;
// 			const float fUVAspect = xLight.GetParams().m_fAspect;
// 
// 			// This code is stolen from Asura_Projector::UpdateVertices().
// 			float fOfsMin, fOfsMax;
// 			if (fMinRange<0.1f)
// 			{
// 				fMinRange = 0.1f;
// 			}
// 			if (xLight.GetParams().m_uFlags & ASURA_DYNAMICLIGHTFLAG_PARALLELPROJECTION)
// 			{
// 				fOfsMin = fScale;	
// 				fOfsMax = fScale;	
// 			}
// 			else
// 			{
// 				fOfsMin = Asura_Maths::Tangent(fFoV*0.5f) * fMinRange;	
// 				fOfsMax = Asura_Maths::Tangent(fFoV*0.5f) * fMaxRange;	
// 			}
// 			const float afScaleX[4] = {-1,1,1,-1};
// 			const float afScaleY[4] = {-1,-1,1,1};
// 			for(int i=0;i<4;i++)
// 			{	
// 				xVertices[i].x   = afScaleX[i]* fOfsMin * fUVAspect;
// 				xVertices[i].y   = afScaleY[i]* fOfsMin;
// 				xVertices[i].z   = fMinRange;
// 				xVertices[i+4].x = afScaleX[i]* fOfsMax * fUVAspect;
// 				xVertices[i+4].y = afScaleY[i]* fOfsMax;
// 				xVertices[i+4].z = fMaxRange;			
// 			}
// 			for(int i=0;i<8;i++)
// 			{
// 				xVertices[i] = xPos + (xVertices[i] * xOrient);
// 				xBB.Set( xVertices[ i ] );
// 			}
// 
// 			break;
// 		}
// 
// 		default:
// 		{
// 			Asura_Assert( false, "AVP_Waypoint_Manager::SetBBFromLight - unhandled light type" );
// 			break;
// 		}
// 	}
}

void AVP_Waypoint_Manager::SetBBFromFlare( const AVP_ServerProjectile_HumanFlare& xFlare, Asura_Bounding_Box& xBB )
{
	xBB.SetToPoint( xFlare.GetPosition() );
	xBB.Grow( AVP_ServerProjectile_HumanFlare::GetFlareAILightRadius() );
}

void AVP_Waypoint_Manager::SetBBFromContainer( const AVP_ServerEntity_Container& xContainerEntity, Asura_Bounding_Box& xBB )
{
	xContainerEntity.SetLightingBB(xBB);
}

u_int AVP_Waypoint_Manager::GetSectorFromDirection( const Asura_Vector_3& xDirection )
{
	// Doesn't matter what these are as long as they're consistent.
	const Asura_Vector_3 xNorth( 0.0f, 0.0f, 1.0f );
	const Asura_Vector_3 xEast( 1.0f, 0.0f, 0.0f );

	//enum Sector
	//{
	//	NORTH_NORTH_EAST,
	//	EAST_NORTH_EAST,
	//	EAST_SOUTH_EAST,
	//	SOUTH_SOUTH_EAST,
	//	SOUTH_SOUTH_WEST,
	//	WEST_SOUTH_WEST,
	//	WEST_NORTH_WEST,
	//	NORTH_NORTH_WEST,
	//	NUM_SECTORS
	//};

	Asura_Vector_3 xFlatDirection( xDirection.x, 0.0f, xDirection.z );
	if ( xFlatDirection.Normalise() > 0.0f )
	{
		const float fCosSectorAngle = 0.70710678f; // Asura_Maths::Cosine( Asura_Maths::DegToRad( 45.0f ) );

		const float fNorthDot = ( xFlatDirection * xNorth );
		const float fEastDot  = ( xFlatDirection * xEast );
		if ( fEastDot > 0.0f )
		{
			if ( fNorthDot > fCosSectorAngle )
			{
				return 0;
			}
			else if ( fNorthDot > 0.0f )
			{
				return 1;
			}
			else if ( fNorthDot > -fCosSectorAngle )
			{
				return 2;
			}
			else
			{
				return 3;
			}
		}
		else if ( fEastDot == 0.0f )
		{
			if ( fNorthDot > 0.0f )
			{
				return 0;
			}
			else
			{
				return 4;
			}
		}
		else
		{
			if ( fNorthDot < -fCosSectorAngle )
			{
				return 4;
			}
			else if ( fNorthDot < 0.0f )
			{
				return 5;
			}
			else if ( fNorthDot < fCosSectorAngle )
			{
				return 6;
			}
			else
			{
				return 7;
			}
		}
	}
	else
	{
		// Direction is straight up or down - can't resolve this.
		return 0;
	}
}

u_int AVP_Waypoint_Manager::GetOppositeSector( const u_int uSector )
{
	return ( ( uSector + NUM_SECTORS / 2 ) % NUM_SECTORS );
}

void AVP_Waypoint_Manager::CalcDirectionFromSector( const u_int uSector, Asura_Vector_3& xDirection )
{
	//enum Sector
	//{
	//	NORTH_NORTH_EAST,
	//	EAST_NORTH_EAST,
	//	EAST_SOUTH_EAST,
	//	SOUTH_SOUTH_EAST,
	//	SOUTH_SOUTH_WEST,
	//	WEST_SOUTH_WEST,
	//	WEST_NORTH_WEST,
	//	NORTH_NORTH_WEST,
	//	NUM_SECTORS
	//};

	static const Asura_Vector_3 s_axSectorMidDirection[] =
	{
		Asura_Vector_3( 0.382683f, 0.0f, 0.923880f ),
		Asura_Vector_3( 0.923880f, 0.0f, 0.382683f ),
		Asura_Vector_3( 0.923880f, 0.0f, -0.382683f ),
		Asura_Vector_3( 0.382683f, 0.0f, -0.923880f ),
		Asura_Vector_3( -0.382683f, 0.0f, -0.923880f ),
		Asura_Vector_3( -0.923880f, 0.0f, -0.382683f ),
		Asura_Vector_3( -0.923880f, 0.0f, 0.382683f ),
		Asura_Vector_3( -0.382683f, 0.0f, 0.923880f )
	};

	xDirection = s_axSectorMidDirection[ uSector ];
}

#ifdef ASURA_TOOLS
// -----------------------------------------------------------------------------------------------------------------------------------------
// AVP_DynamicBVH_StaticObjects_Wrapper	: Constructor
// Last Modified		: 29 / 05 / 08
// -----------------------------------------------------------------------------------------------------------------------------------------
AVP_DynamicBVH_StaticObjects_Wrapper::AVP_DynamicBVH_StaticObjects_Wrapper(Asura_ServerEntity_PhysicalObject* pxStaticObject)
: m_pxStaticObject(pxStaticObject)
{
	s_xLineOfSightList.AddToList(this);
}

// -----------------------------------------------------------------------------------------------------------------------------------------
// AVP_DynamicBVH_StaticObjects_Wrapper	: Deconstructor
// Last Modified		: 29 / 05 / 08
// -----------------------------------------------------------------------------------------------------------------------------------------
AVP_DynamicBVH_StaticObjects_Wrapper::~AVP_DynamicBVH_StaticObjects_Wrapper(void)
{
	s_xLineOfSightList.RemoveFromList(this);
}

// -----------------------------------------------------------------------------------------------------------------------------------------
// AVP_DynamicBVH_StaticObjects_Wrapper	: Deconstructor
// Description							: Gets the BoundingBox from the static object and sets it in the passed in paramater
// Last Modified						: 29 / 05 / 08
// -----------------------------------------------------------------------------------------------------------------------------------------
void AVP_DynamicBVH_StaticObjects_Wrapper::GetLineOfSightAABB(Asura_Bounding_Box& xAABB)		
{
	if (m_pxStaticObject)
	{
		xAABB = *(m_pxStaticObject->GetBB());
	}
}
#endif // ASURA_TOOLS


// extra links data

void AVP_Waypoint_Manager::InitialiseLinkExtraData()
{
	Asura_AI_Navigation_System::SetBlockCallback(AVP_Waypoint_Manager::AvpBlockLinkCallback);
	s_xExtraLinkData.Initialise();
}

void AVP_Waypoint_Manager::ShutdownLinkExtraData()
{
	Asura_AI_Navigation_System::SetBlockCallback(NULL);
	s_xExtraLinkData.Reset();
}

bool AVP_Waypoint_Manager::IsLinkBlockedForHumans(const Asura_AI_NavConnection* pxLink)
{
	return s_xExtraLinkData.IsLinkBlockedForHumans(pxLink);
}

bool AVP_Waypoint_Manager::IsLinkBlockedForAliens(const Asura_AI_NavConnection* pxLink)
{
	return s_xExtraLinkData.IsLinkBlockedForAliens(pxLink);
}

bool AVP_Waypoint_Manager::IsLinkBlockedForPredator(const Asura_AI_NavConnection* pxLink)
{
	return s_xExtraLinkData.IsLinkBlockedForPredator(pxLink);
}

void AVP_Waypoint_Manager::SetBlockParams(bool bHuman, bool bAlien, bool bPredator)
{
	s_xExtraLinkData.SetBlockParams(bHuman, bAlien, bPredator);
}

void AVP_Waypoint_Manager::RestoreBlockParamsDefault()
{
	s_xExtraLinkData.RestoreDefaultBlockParams();
}

void AVP_Waypoint_Manager::AvpBlockLinkCallback(u_int uWaypointIndex, u_int uConnectionIndex, const bool bBlock)
{
	s_xExtraLinkData(uWaypointIndex, uConnectionIndex, bBlock);
}


AVP_Waypoint_Manager::ExtraLinksData::ExtraLinksData()
: m_xLinkDescriptors()
, m_uNumLinks(0)
, m_aucLinksBlockFlags(NULL)
, m_bHuman(true)
, m_bAlien(true)
, m_bPredator(true)
{
}

AVP_Waypoint_Manager::ExtraLinksData::~ExtraLinksData()
{
	Reset();
}

void AVP_Waypoint_Manager::ExtraLinksData::Reset()
{
	m_xLinkDescriptors.Reset();
	delete[] m_aucLinksBlockFlags;
	m_aucLinksBlockFlags = NULL;
}

void AVP_Waypoint_Manager::ExtraLinksData::Initialise()
{
	Reset();

	m_uNumLinks = Asura_AI_Navigation_System::GetNumberOfConnections();
	if (m_uNumLinks > 0)
	{
		m_aucLinksBlockFlags = new u_char[m_uNumLinks];
		Asura_Mem::Set(m_aucLinksBlockFlags, 0, sizeof(u_char) * m_uNumLinks);
	}
}


bool AVP_Waypoint_Manager::ExtraLinksData::IsLinkBlockedForHumans(const Asura_AI_NavConnection* pxLink) const
{
	u_int uLinkIndex = GetLinkIndex(pxLink);
	Asura_Assertf(uLinkIndex < m_uNumLinks, ("IsLinkBlockedForHumans: Link %d is out of range %d", uLinkIndex, m_uNumLinks));
	if (uLinkIndex < m_uNumLinks)
	{
		return (m_aucLinksBlockFlags[uLinkIndex] & BLOCK_HUMAN_FLAG) != 0;
	}

	return true;
}

bool AVP_Waypoint_Manager::ExtraLinksData::IsLinkBlockedForAliens(const Asura_AI_NavConnection* pxLink) const
{
	u_int uLinkIndex = GetLinkIndex(pxLink);
	Asura_Assertf(uLinkIndex < m_uNumLinks, ("IsLinkBlockedForAliens: Link %d is out of range %d", uLinkIndex, m_uNumLinks));
	if (uLinkIndex < m_uNumLinks)
	{
		return (m_aucLinksBlockFlags[uLinkIndex] & BLOCK_ALIEN_FLAG) != 0;
	}

	return true;
}

bool AVP_Waypoint_Manager::ExtraLinksData::IsLinkBlockedForPredator(const Asura_AI_NavConnection* pxLink) const
{
	u_int uLinkIndex = GetLinkIndex(pxLink);
	Asura_Assertf(uLinkIndex < m_uNumLinks, ("IsLinkBlockedForPredator: Link %d is out of range %d", uLinkIndex, m_uNumLinks));
	if (uLinkIndex < m_uNumLinks)
	{
		return (m_aucLinksBlockFlags[uLinkIndex] & BLOCK_PREDATOR_FLAG) != 0;
	}

	return true;
}

void AVP_Waypoint_Manager::ExtraLinksData::operator()(u_int uWaypointIndex, u_int uConnectionIndex, const bool bBlock)
{
	u_int uLinkIndex = GetLinkIndex(uWaypointIndex, uConnectionIndex);

	if (uLinkIndex == ASURA_INVALID_CONNECTION_OFFSET)
	{
		return;
	}


	if (bBlock)
	{
		BlockLink(uLinkIndex);
	}
	else
	{
		UnblockLink(uLinkIndex);
	}
}

void AVP_Waypoint_Manager::ExtraLinksData::SetBlockParams(bool bHuman, bool bAlien, bool bPredator)
{
	m_bHuman = bHuman;
	m_bAlien = bAlien;
	m_bPredator = bPredator;
}

void AVP_Waypoint_Manager::ExtraLinksData::RestoreDefaultBlockParams()
{
	m_bHuman = true;
	m_bAlien = true;
	m_bPredator = true;
}

void AVP_Waypoint_Manager::ExtraLinksData::BlockLink(u_int uLinkIndex)
{	
	if (!(m_bHuman | m_bAlien | m_bPredator))
	{
		return;
	}

	LinkBlockDescriptor* pxDescriptor = m_xLinkDescriptors.GetDataByHash(uLinkIndex);

	if (pxDescriptor) // already have this descriptor - just update the values
	{
		if (m_bHuman)
		{
			if (pxDescriptor->m_ucHumanBlockCount == 0 )
			{
				m_aucLinksBlockFlags[uLinkIndex] |= BLOCK_HUMAN_FLAG;
			}
			++pxDescriptor->m_ucHumanBlockCount;
		}
		if (m_bAlien)
		{
			if (pxDescriptor->m_ucAlienBlockCount == 0 )
			{
				m_aucLinksBlockFlags[uLinkIndex] |= BLOCK_ALIEN_FLAG;
			}
			++pxDescriptor->m_ucAlienBlockCount;
		}
		if (m_bPredator)
		{
			if (pxDescriptor->m_ucPredatorBlockCount == 0 )
			{
				m_aucLinksBlockFlags[uLinkIndex] |= BLOCK_PREDATOR_FLAG;
			}
			++pxDescriptor->m_ucPredatorBlockCount;
		}
	}
	else
	{
		LinkBlockDescriptor xDescriptor;		
		if (m_bHuman)
		{
			m_aucLinksBlockFlags[uLinkIndex] |= BLOCK_HUMAN_FLAG;
			xDescriptor.m_ucHumanBlockCount = 1;
		}
		else
		{
			xDescriptor.m_ucHumanBlockCount = 0;
		}

		if (m_bAlien)
		{
			m_aucLinksBlockFlags[uLinkIndex] |= BLOCK_ALIEN_FLAG;
			xDescriptor.m_ucAlienBlockCount = 1;
		}
		else
		{
			xDescriptor.m_ucAlienBlockCount = 0;
		}

		if (m_bPredator)
		{
			m_aucLinksBlockFlags[uLinkIndex] |= BLOCK_PREDATOR_FLAG;
			xDescriptor.m_ucPredatorBlockCount = 1;
		}
		else
		{
			xDescriptor.m_ucPredatorBlockCount = 0;
		}

		m_xLinkDescriptors.AddNode(uLinkIndex, xDescriptor);
	}
}

void AVP_Waypoint_Manager::ExtraLinksData::UnblockLink(u_int uLinkIndex)
{
	if (!(m_bHuman | m_bAlien | m_bPredator))
	{
		return;
	}

	LinkBlockDescriptor* pxDescriptor = m_xLinkDescriptors.GetDataByHash(uLinkIndex);

	if (pxDescriptor) // already have this descriptor - just update the values
	{
		if (m_bHuman)
		{
			Asura_Assertf(pxDescriptor->m_ucHumanBlockCount > 0, ("UnblockLink: Link %d human count mismatch", uLinkIndex));
			--pxDescriptor->m_ucHumanBlockCount;
			if (pxDescriptor->m_ucHumanBlockCount == 0 )
			{
				m_aucLinksBlockFlags[uLinkIndex] &= 0xff - BLOCK_HUMAN_FLAG;
			}
		}
		if (m_bAlien)
		{
			Asura_Assertf(pxDescriptor->m_ucAlienBlockCount > 0, ("UnblockLink: Link %d alien count mismatch", uLinkIndex));
			--pxDescriptor->m_ucAlienBlockCount;
			if (pxDescriptor->m_ucAlienBlockCount == 0 )
			{
				m_aucLinksBlockFlags[uLinkIndex] &= 0xff - BLOCK_ALIEN_FLAG;
			}
		}
		if (m_bPredator)
		{
			Asura_Assertf(pxDescriptor->m_ucPredatorBlockCount > 0, ("UnblockLink: Link %d alien count mismatch", uLinkIndex));
			--pxDescriptor->m_ucPredatorBlockCount;
			if (pxDescriptor->m_ucPredatorBlockCount == 0 )
			{
				m_aucLinksBlockFlags[uLinkIndex] &= 0xff - BLOCK_PREDATOR_FLAG;
			}
		}

		// if nothing is blocked - delete
		if ( (m_aucLinksBlockFlags[uLinkIndex] & (BLOCK_HUMAN_FLAG | BLOCK_ALIEN_FLAG | BLOCK_PREDATOR_FLAG)) == 0)
		{
			Asura_Assert(pxDescriptor->m_ucHumanBlockCount == 0, "UnblockLink: Descriptor mismatch");
			Asura_Assert(pxDescriptor->m_ucAlienBlockCount == 0, "UnblockLink: Descriptor mismatch");
			Asura_Assert(pxDescriptor->m_ucPredatorBlockCount == 0, "UnblockLink: Descriptor mismatch");
			m_xLinkDescriptors.DeleteNodeByHash(uLinkIndex);
		}
	}
	else
	{

		Asura_Assertf(false, ("Link %d Descriptor expected but not found ", uLinkIndex));
	}
}


u_int AVP_Waypoint_Manager::ExtraLinksData::GetLinkIndex(u_int uWapointID, u_int uLinkNumber) const 
{
	const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint(uWapointID);
	Asura_Assertf(pxWP, ("Waypoint %d not found", uWapointID));
	if (!pxWP)
	{
		return ASURA_INVALID_CONNECTION_OFFSET;
	}

	const Asura_AI_NavConnection* pxLink = &pxWP->GetLink(uLinkNumber);
	Asura_Assertf(pxLink, ("Waypoint %d not found", uWapointID));
	if (!pxLink)
	{
		return ASURA_INVALID_CONNECTION_OFFSET;
	}

	return GetLinkIndex(pxLink);
}

u_int AVP_Waypoint_Manager::ExtraLinksData::GetLinkIndex(const Asura_AI_NavConnection* pxLink) const 
{
	u_int uLinkIndex = Asura_AI_Navigation_System::GetConnectionOffset(pxLink);

	Asura_Assertf(uLinkIndex < m_uNumLinks, ("BlockLink: Link %d is out of range %d", uLinkIndex, m_uNumLinks));
	if (uLinkIndex >= m_uNumLinks)
	{
		return ASURA_INVALID_CONNECTION_OFFSET;
	}

	return uLinkIndex;
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
