//////////////////////////////////////////////////////////////////////////////
//
// AxonAVP_Messages.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AXONAVP_MESSAGES_H__
#define __AXONAVP_MESSAGES_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Chunk_Stream;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Message_SetPreferredTargetType
//
//////////////////////////////////////////////////////////////////////////////

// Designer triggered message for instructing the AI to target a certain type of enemy.
struct AVP_Message_SetPreferredTargetType
{
	enum { CURRENT_VERSION = 0 };
	enum PreferredTargetType
	{
		PTT_NONE,
		PTT_HUMANOID,
		PTT_ALIEN,
		PTT_PREDATOR,
		PTT_END_ENUM
	};

	u_int m_uVersion;
	int   m_iPreferredTargetType;

#ifdef ASURA_TOOLS
	static const char* s_aszTargetTypeStrings[ PTT_END_ENUM ];
#endif
};


struct AVP_Msg_SetAlienSquadBehaviour
{
	u_int m_uVersion;
	int m_iBehaviour;
	Asura_Hash_ID m_uProtoBlueprintHashID;
	float m_fJumpFraction;
	float m_fStandFraction;
	float m_fMinTimeBetweenGrapples;
	float m_fDamageFromBehindMultiplier;
	int m_iMoveInMode;
	float m_fIdealDistanceMin;
	float m_fIdealDistanceMax;
	float m_fAttackDelay;
	int m_iMaxAttackersPerTarget;
	int m_iMaxAttackersTotal;
	int m_iMaxBurstPerTarget;
	int m_iPostAttackAction;
	float m_fRespawnDelay;
	float m_fTimeBetweenAttacks;
	u_int m_uMaxNumberOfChargers;
	bool m_bAttackIfInMarineLight;
	float m_fTimeInLightBeforeAttack;
	float m_fMinHideDistance;
	float m_fChaneToRehideIfCloseFraction;
	float m_fChaneToRehideIfInLightFraction;
	float m_fChaneToRehideAfterAttackFraction;

	static const u_int uCURRENT_VERSION = 7;
};

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Msg_SetAlienSquadBehaviour
//
//////////////////////////////////////////////////////////////////////////////

struct AVP_AlienBehaviourData
{
	// Don't change the order of these - they are saved out to chunks.
	enum Behaviour
	{
		STANDARD,
		GUERILLA,
		ATTACK,
		STALK,
		AMBUSH,
		DISAPPEAR,
		CUSTOM,
		SWARM,
		HIDE,
		DESTROY,
		AGGRESIVE,
		HIDE_IN_SHADOWS,
		NUM_BEHAVIOURS
	};
	enum MoveStyle
	{
		CREEP,
		DIRECT_CREEP,
		CHARGE,
		NUM_MOVE_STYLES
	};
	enum PostAttackAction
	{
		CYCLE_ROUND_IMMEDIATE,
		CYCLE_ROUND_DELAYED,
		DISAPPEAR_PAA,
		NUM_POST_ATTACK_ACTIONS
	};


	int m_iBehaviour;
	Asura_Hash_ID m_uProtoBlueprintHashID;
	float m_fJumpFraction;
	float m_fStandFraction;
	float m_fMinTimeBetweenGrapples;
	float m_fDamageFromBehindMultiplier;
	int m_iMoveInMode;
	float m_fIdealDistanceMin;
	float m_fIdealDistanceMax;
	float m_fAttackDelay;
	int m_iMaxAttackersPerTarget;
	int m_iMaxAttackersTotal;
	int m_iMaxBurstPerTarget;
	int m_iPostAttackAction;
	float m_fRespawnDelay;
	u_int m_uNumHidePoints;
	Asura_Guid* m_pxHidePointGuids;
	float m_fTimeBetweenAttacks;
	u_int m_uMaxNumberOfChargers;
	bool m_bAttackIfInMarineLight;
	float m_fTimeInLightBeforeAttack;
	float m_fMinHideDistance;
	float m_fChaneToRehideIfCloseFraction;
	float m_fChaneToRehideIfInLightFraction;
	float m_fChaneToRehideAfterAttackFraction;


	AVP_AlienBehaviourData()
	: m_iBehaviour( AGGRESIVE )
	, m_uProtoBlueprintHashID(ASURA_HASH_ID_UNSET)
	, m_fJumpFraction(0.5f)
	, m_fStandFraction(0.5f)
	, m_fMinTimeBetweenGrapples(30.0f)
	, m_fDamageFromBehindMultiplier(0.1f)
	, m_iMoveInMode( 0 )
	, m_fIdealDistanceMin( 10.0f )
	, m_fIdealDistanceMax( 25.0f )
	, m_fAttackDelay( 2.5f )
	, m_iMaxAttackersPerTarget( 2 )
	, m_iMaxAttackersTotal( 2 )
	, m_iMaxBurstPerTarget( 2 )
	, m_iPostAttackAction( 0 )
	, m_fRespawnDelay( 10.0f )
	, m_uNumHidePoints( 0 )
	, m_pxHidePointGuids( NULL )
	, m_fTimeBetweenAttacks(1.0f)
	, m_uMaxNumberOfChargers( 5 )
	, m_bAttackIfInMarineLight(false)
	, m_fTimeInLightBeforeAttack(5.0f)
	, m_fMinHideDistance(5.0f)
	, m_fChaneToRehideIfCloseFraction(0.0f)
	, m_fChaneToRehideIfInLightFraction(1.0f)
	, m_fChaneToRehideAfterAttackFraction(0.0f)
	{
	}
	void SetParamsFromBehaviour(); // obsolete!!!
	void SetParamsFromBlueprint();
	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

#ifdef ASURA_TOOLS
	static const char* s_apszBehaviourStrings[ NUM_BEHAVIOURS ];
	static const char* s_apszMoveInModeStrings[ NUM_MOVE_STYLES ];
	static const char* s_apszPostAttackActionStrings[ NUM_POST_ATTACK_ACTIONS ];
#endif

private:
	static const u_int uCURRENT_CHUNK_VERSION = 1007;
};

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Msg_SetFaceHuggerBehaviour
//
//////////////////////////////////////////////////////////////////////////////

struct AVP_FaceHuggerBehaviourData
{
	enum Behaviour
	{
		ATTACK,
		HIDE_IN_SHADOWS,
		NUM_BEHAVIOURS
	};

	int m_iBehaviour;
	float m_fMaxDistanceFromSpawn;
	float m_fAvoidanceDistance;
	u_int m_uMaxTargeters;
	float m_fTimeBetweenAttacks;
	float m_fWanderDistance;	
	bool m_bAttackIfInMarineLight;
	float m_fTimeInLightBeforeAttack;
	float m_fMinHideDistance;
	float m_fChaneToRehideIfCloseFraction;
	float m_fChaneToRehideIfInLightFraction;
	float m_fChaneToRehideAfterAttackFraction;


	AVP_FaceHuggerBehaviourData()
		: m_iBehaviour(ATTACK)
		, m_fMaxDistanceFromSpawn(20.0f)
		, m_fAvoidanceDistance(12.0f)
		, m_uMaxTargeters(3)
		, m_fTimeBetweenAttacks(5.0f)
		, m_fWanderDistance(5.0f)
		, m_bAttackIfInMarineLight(false)
		, m_fTimeInLightBeforeAttack(5.0f)
		, m_fMinHideDistance(5.0f)
		, m_fChaneToRehideIfCloseFraction(0.0f)
		, m_fChaneToRehideIfInLightFraction(1.0f)
		, m_fChaneToRehideAfterAttackFraction(0.0f)

	{
	}

	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

#ifdef ASURA_TOOLS
	static const char* s_apszBehaviourStrings[ NUM_BEHAVIOURS ];
#endif
private:

	static const u_int uCURRENT_CHUNK_VERSION = 1002;
};

struct AVP_Msg_SetFaceHuggerSquadBehaviour
{
	u_int m_uVersion;
	int m_iBehaviour;
	float m_fAvoidanceDistance;
	u_int m_uMaxTargeters;
	float m_fTimeBetweenAttacks;
	float m_fWanderDistance;
	bool m_bAttackIfInMarineLight;
	float m_fTimeInLightBeforeAttack;
	float m_fMinHideDistance;
	float m_fChaneToRehideIfCloseFraction;
	float m_fChaneToRehideIfInLightFraction;
	float m_fChaneToRehideAfterAttackFraction;

	static const u_int uCURRENT_VERSION = 2;
};



//////////////////////////////////////////////////////////////////////////////
//
// AlienQueen Behaviour Data
// Functions and static initialisations in AVP_ServerActor_AlienQueen.cpp
//
//////////////////////////////////////////////////////////////////////////////

struct AVP_AlienQueenBehaviourData
{
	enum BEHAVOIURS
	{
		WANDER,				// Wander around the maze
		STUNNED,			// Stay at position for certain time and play stunned anim. 
		ROARING,			// Roar on spot or at current target. 

		NUM_BEHAVIOURS
	};

	int m_iBehaviour;
	float m_fMaxDistanceForCharge;
	float m_fMinDistanceForCharge;

	float m_fAggressionModifer; // How quickly the alien becomes aggresive, increases chance of going into charge
	float m_fDefenceModifer; // How quickly alien becomes defensive, backs of from player etc. 

	AVP_AlienQueenBehaviourData() 
	: m_iBehaviour( WANDER )
	, m_fMaxDistanceForCharge( 30.0f )
	, m_fMinDistanceForCharge( 20.0f )
	, m_fAggressionModifer( 0.0f )
	, m_fDefenceModifer( 0.0f )
	{

	};

	void SetParamsFromBehaviour();
	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	int  GetAVPBehaviourType() const;

#ifdef ASURA_TOOLS
	static const char* s_apszBehaviourStrings[ NUM_BEHAVIOURS ];
#endif

	static const u_int uCURRENT_CHUNK_VERSION = 1;
};
//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AXONAVP_MESSAGES_H__
