//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_NPC.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Core/BrainTypes/AVP_Brain_Actor.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Camera/Asura_Camera.h"
#include "Debugging/Asura_DebugLineRender.h"
#include "Rendering/Asura_Font_System.h"
#endif
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"

// avp
#include "Actor/Vertebrates/AVP_Vertebrate_AIAnimStateMachine.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Animation/AVP_Animation_BodyRig.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Squad.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"
#include "Moves/AVP_Moves.h"

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_GS2Actor.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_GS2Node_AnimMessages.h"
#include "Behaviour/BehaviourTypes/AVP_Behaviour_GS2Node_RouteTraverse.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Attractor.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Node.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Spline.h"
#include "Core/Axon_Messages.h"

#ifdef ASURA_TOOLS
#include "Tools_System.h"
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor
//
//////////////////////////////////////////////////////////////////////////////

#ifndef ASURA_NO_DEBUG_OUTPUT
// CHANGE THIS TO AN IN GAME GUID TO ONLY RENDER THE DEBUG INFO FOR THAT GUID.
Asura_Guid AVP_Brain_Actor::s_uRenderGuid = ASURA_GUID_UNREGISTERED;
#endif

//----------------------------------------------------------------------------
// AVP_Brain_Actor :: ReadFromChunkStream
//----------------------------------------------------------------------------
void AVP_Brain_Actor::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
    int iVersion;
    xStream >> iVersion;

	if ( iVersion == CURRENT_CHUNK_VERSION || iVersion == 5 )
	{
		xStream >> m_bChunkNavData;

		if ( m_bChunkNavData )
		{
			u_int uMoveMode = 0;		xStream >> uMoveMode;		m_xNavData.m_eMovementMode =		( AVP_Navigation::MovementMode )uMoveMode;
			u_int uRouteAgil = 0;		xStream >> uRouteAgil;		m_xNavData.m_eRouteAgilityRating =	( AVP_Navigation::AgilityRating )uRouteAgil;
			u_int uRouteType = 0;		xStream >> uRouteType;		m_xNavData.m_eRouteType =			( AVP_Navigation::RouteType )uRouteType;
			u_int uTraverseStyle = 0;	xStream >> uTraverseStyle;	m_xNavData.m_eTraverseStyle =		( AVP_Navigation::TraverseStyle )uTraverseStyle;

			xStream >> m_xNavData.m_usCurrentWPID;
			xStream >> m_xNavData.m_usCurrentZone;
			xStream >> m_xNavData.m_usDestinationWPID;
			xStream >> m_xNavData.m_usRouteLength;
			xStream >> m_xNavData.m_usRouteSegmentIndex;
			xStream >> m_xNavData.m_xDestinationPosition;
			xStream >> m_xNavData.m_xStartPosition;

			if ( m_xNavData.m_pusRoute )
			{
				AVP_Navigation::DestroyRouteBuffer( m_xNavData.m_pusRoute );
			}

			m_xNavData.m_pusRoute = AVP_Navigation::CreateRouteBuffer( m_xNavData.m_usRouteLength );

			for ( u_int u = 0; u < m_xNavData.m_usRouteLength; ++u )
			{
				xStream >> m_xNavData.m_pusRoute[ u ];
			}
		}
	}

    PARENT::ReadFromChunkStream(xStream);

	if ( iVersion == CURRENT_CHUNK_VERSION || iVersion == 5 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfieldWrapper( m_uActorFlags );
		xStream >> xBitfieldWrapper;

		// Zero these. We had to save them because of the way the bitfield stuff works but we want them to start fresh each time.
		m_bRequestRegisterWithTeam	 = false;
		m_bRequestUnregisterWithTeam = false;
		m_bRegisteredWithTeam		 = false;

		u_int uNumTPs;
		xStream >> uNumTPs;
		for ( u_int uTP = 0; uTP < uNumTPs; ++uTP )
		{
			AVP_AI_TargetPriorityData xTP;
			xTP.ReadFromChunkStream( xStream );
			m_xTargetPriorityList.Push_Back( xTP );
		}
	}
	else if ( iVersion == 4 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfieldWrapper( m_uActorFlags );
		xStream >> xBitfieldWrapper;

		// Zero these. We had to save them because of the way the bitfield stuff works but we want them to start fresh each time.
		m_bRequestRegisterWithTeam	 = false;
		m_bRequestUnregisterWithTeam = false;
		m_bRegisteredWithTeam		 = false;

		u_int uNumTPs;
		xStream >> uNumTPs;
		for ( u_int uTP = 0; uTP < uNumTPs; ++uTP )
		{
			AVP_AI_TargetPriorityData xTP;
			xTP.ReadFromChunkStream( xStream );
			m_xTargetPriorityList.Push_Back( xTP );
		}

		xStream >> m_bChunkNavData;

		if ( m_bChunkNavData )
		{
			u_int uMoveMode = 0;		xStream >> uMoveMode;		m_xNavData.m_eMovementMode =		( AVP_Navigation::MovementMode )uMoveMode;
			u_int uRouteAgil = 0;		xStream >> uRouteAgil;		m_xNavData.m_eRouteAgilityRating =	( AVP_Navigation::AgilityRating )uRouteAgil;
			u_int uRouteType = 0;		xStream >> uRouteType;		m_xNavData.m_eRouteType =			( AVP_Navigation::RouteType )uRouteType;
			u_int uTraverseStyle = 0;	xStream >> uTraverseStyle;	m_xNavData.m_eTraverseStyle =		( AVP_Navigation::TraverseStyle )uTraverseStyle;

			xStream >> m_xNavData.m_usCurrentWPID;
			xStream >> m_xNavData.m_usCurrentZone;
			xStream >> m_xNavData.m_usDestinationWPID;
			xStream >> m_xNavData.m_usRouteLength;
			xStream >> m_xNavData.m_usRouteSegmentIndex;
			xStream >> m_xNavData.m_xDestinationPosition;
			xStream >> m_xNavData.m_xStartPosition;

			AVP_Navigation::DestroyRouteBuffer( m_xNavData.m_pusRoute );
			m_xNavData.m_pusRoute = AVP_Navigation::CreateRouteBuffer( m_xNavData.m_usRouteLength );

			for ( u_int u = 0; u < m_xNavData.m_usRouteLength; ++u )
			{
				xStream >> m_xNavData.m_pusRoute[ u ];
			}
		}		
	}
    else if ( iVersion == 3 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfieldWrapper( m_uActorFlags );
		xStream >> xBitfieldWrapper;

		// Zero these. We had to save them because of the way the bitfield stuff works but we want them to start fresh each time.
		m_bRequestRegisterWithTeam	 = false;
		m_bRequestUnregisterWithTeam = false;
		m_bRegisteredWithTeam		 = false;

		u_int uNumTPs;
		xStream >> uNumTPs;
		for ( u_int uTP = 0; uTP < uNumTPs; ++uTP )
		{
			AVP_AI_TargetPriorityData xTP;
			xTP.ReadFromChunkStream( xStream );
			m_xTargetPriorityList.Push_Back( xTP );
		}

		m_bChunkNavData = false;
	}
    else if ( iVersion == 2 ) // This version got corrupted for some sentry guns at one point, ending up with a stupid number of target priorities saved. 
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfieldWrapper( m_uActorFlags );
		xStream >> xBitfieldWrapper;

		// Zero these. We had to save them because of the way the bitfield stuff works but we want them to start fresh each time.
		m_bRequestRegisterWithTeam	 = false;
		m_bRequestUnregisterWithTeam = false;
		m_bRegisteredWithTeam		 = false;

		u_int uNumTPs;
		xStream >> uNumTPs;
		// Read this data out, but don't put it into the list.
		for ( u_int uTP = 0; uTP < uNumTPs; ++uTP )
		{
			AVP_AI_TargetPriorityData xTP;
			xTP.ReadFromChunkStream( xStream );
		}
	}
	else if ( iVersion == 1 )
    {
        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfieldWrapper( m_uActorFlags );
        xStream >> xBitfieldWrapper;

		// Zero these. We had to save them because of the way the bitfield stuff works but we want them to start fresh each time.
		m_bRequestRegisterWithTeam	 = false;
		m_bRequestUnregisterWithTeam = false;
		m_bRegisteredWithTeam		 = false;
    }
    else if ( iVersion == 0 )
    {
    }
    else
    {
        Asura_Assertf(false, ("AVP_Brain_Actor::ReadFromChunkStream - unhandled chunk version (%d)", iVersion));
    }
	#ifdef ASURA_TOOLS
	if(iVersion < 6)
	{
		//Fix old data so that the unused bits are zeroed. RAR 28/11/2008
		m_uActorFlags &= 0x1f;
	}
	#endif

}

//----------------------------------------------------------------------------
// AVP_Brain_Actor :: WriteToChunkStream
//----------------------------------------------------------------------------
void AVP_Brain_Actor::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
    xStream << CURRENT_CHUNK_VERSION;

	xStream << m_bChunkNavData;

	if ( m_bChunkNavData )
	{
		u_int uMoveMode =		(u_int)m_xNavData.m_eMovementMode;			xStream << uMoveMode;
		u_int uRouteAgil =		(u_int)m_xNavData.m_eRouteAgilityRating;	xStream << uRouteAgil;
		u_int uRouteType =		(u_int)m_xNavData.m_eRouteType;				xStream << uRouteType;
		u_int uTraverseStyle =	(u_int)m_xNavData.m_eTraverseStyle;			xStream << uTraverseStyle;

		xStream << m_xNavData.m_usCurrentWPID;
		xStream << m_xNavData.m_usCurrentZone;
		xStream << m_xNavData.m_usDestinationWPID;
		xStream << m_xNavData.m_usRouteLength;
		xStream << m_xNavData.m_usRouteSegmentIndex;
		xStream << m_xNavData.m_xDestinationPosition;
		xStream << m_xNavData.m_xStartPosition;

		for ( u_int u = 0; u < m_xNavData.m_usRouteLength; ++u )
		{
			xStream << m_xNavData.m_pusRoute[ u ];
		}
	}

    PARENT::WriteToChunkStream(xStream);

    Asura_WriteToStream_Bitfield_Wrapper< u_int > xBitfieldWrapper( m_uActorFlags );
    xBitfieldWrapper.Add( m_bUseAnimMovement, 1 );
    xBitfieldWrapper.Add( m_bDoPanicMechanicOnGamesceneExit, 1 );
	xBitfieldWrapper.Add( m_bRequestRegisterWithTeam, 1 );
	xBitfieldWrapper.Add( m_bRequestUnregisterWithTeam, 1 );
	xBitfieldWrapper.Add( m_bRegisteredWithTeam, 1 );
	xBitfieldWrapper.Add( m_bUpdated, 1 );
	xBitfieldWrapper.Add( m_bInCurrentWaypoint, 1 );
    xStream << xBitfieldWrapper;

	u_int uNumTPs = m_xTargetPriorityList.Size();
	xStream << uNumTPs;
	for ( Asura_LinkList_ConstIt< AVP_AI_TargetPriorityData > xTPIt( m_xTargetPriorityList ); !xTPIt.Done(); xTPIt.Next() )
	{
		xTPIt.GetCurrent()->WriteToChunkStream( xStream );
	}
}

void AVP_Brain_Actor::PostLoadInitialise()
{
    PARENT::PostLoadInitialise();

	// Base range is 1m out past our maximum extent.
	const AVP_Collision_Descriptor* pxCollDesc = GetActor()->GetDefaultCollisionDescriptor();
	if ( pxCollDesc )
	{
		const Asura_Collision_Object* pxCollObj = pxCollDesc->GetCollisionBox();
		if ( pxCollObj )
		{
			m_fNearbyBaseRange = pxCollObj->GetBoundingRadius() + 1.0f;
		}
	}

    // Establish our movement directions as straight ahead.
	ResetMovementDirection();
	ResetAimDirection();
	ResetLookAtPosition();

    //## Once the orientation handler is moved into the actor we should establish our torso and head orientations as straight ahead here also.

    if ( m_bRequestRegisterWithTeam && RegisterWithTeam() )
    {
        m_bRequestRegisterWithTeam = false;
        m_bRegisteredWithTeam      = true;
    }
    if ( m_bRequestUnregisterWithTeam && UnregisterWithTeam() )
    {
        m_bRequestUnregisterWithTeam = false;
        m_bRegisteredWithTeam        = false;
    }
}

void AVP_Brain_Actor::DoPerFrameUpdate(const float fTimeslice)
{
    if ( m_bRequestRegisterWithTeam && RegisterWithTeam() )
    {
        m_bRequestRegisterWithTeam = false;
        m_bRegisteredWithTeam      = true;
    }
    if ( m_bRequestUnregisterWithTeam && UnregisterWithTeam() )
    {
        m_bRequestUnregisterWithTeam = false;
        m_bRegisteredWithTeam        = false;
    }

	if ( !IsActive() || IsPaused() ) { return; }

	if ( !UpdateClientActorPointer() ) { return; }

	UpdateCurrentWaypointID();

	UpdateNearbyGuidData( fTimeslice );

    PARENT::DoPerFrameUpdate( fTimeslice );

#ifndef ASURA_NO_DEBUG_OUTPUT
	if ( VerifyFailed() ) { return; }
#endif

	UpdateMovementDirection( fTimeslice );
	UpdateAimDirection( fTimeslice );
	UpdateLookAtPosition( fTimeslice );
}

void AVP_Brain_Actor::Shutdown()
{
    // Destroy any route we may still have.
    if ( m_xNavData.m_pusRoute )
    {
        AVP_Navigation::DestroyRouteBuffer( m_xNavData.m_pusRoute );
        m_xNavData.m_pusRoute = NULL;
    }

	// Make sure we unregister with team
	RequestUnregisterWithTeam();

	PARENT::Shutdown();
}

void AVP_Brain_Actor::RequestRegisterWithTeam()
{
    m_bRequestRegisterWithTeam   = false;
    m_bRequestUnregisterWithTeam = false;

    if ( m_bRegisteredWithTeam )
    {
        // Nothing to do.
    }
    else if ( RegisterWithTeam() )
	{
		m_bRegisteredWithTeam = true;
	}
	else
    {
        m_bRequestRegisterWithTeam = true;
    }
}

void AVP_Brain_Actor::RequestUnregisterWithTeam()
{
    m_bRequestRegisterWithTeam   = false;
    m_bRequestUnregisterWithTeam = false;

    if ( !m_bRegisteredWithTeam )
    {
        // Nothing to do.
    }
    else if ( UnregisterWithTeam() )
	{
		m_bRegisteredWithTeam = false;
	}
	else
    {
        m_bRequestUnregisterWithTeam = true;
    }
}

#ifdef ASURA_TOOLS
void AVP_Brain_Actor::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance );

    m_xNavData.m_xStartPosition       = pxInstance->TransformPosition( m_xNavData.m_xStartPosition );
	m_xNavData.m_xDestinationPosition = pxInstance->TransformPosition( m_xNavData.m_xDestinationPosition );
}
#endif

#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Brain_Actor::DoRenderDebugInfo()
{
	bool bRender = true;

	// Render everyone's debug info if we haven't set the render guid, otherwise only render the one we selected.
	if (s_uRenderGuid != ASURA_GUID_UNREGISTERED)
	{
		if (s_uRenderGuid != GetOwnerGuid())
		{
			bRender = false;
		}
	}

	if (bRender)
	{
		if (m_pxOwnerActor->IsDead() || !m_pxOwnerActor->IsEntityActive())
		{
			return;
		}

		if ( m_bRenderRoute )
		{
			AVP_Navigation::RenderRoute( GetNavData() );
		}

		Asura_Vector_3 xOutputPos = m_pxOwnerActor->GetPosition();
		
		if (!Asura_Camera::ProjectWorldCoordsToScreenSpace( xOutputPos, true ) || xOutputPos.z <= 0.0f)
		{
			return;
		}

		m_xDebugOutputPos.x = xOutputPos.x;
		m_xDebugOutputPos.y = xOutputPos.y;

		Asura_Font_System::Print
		(
			m_xDebugOutputPos.x, 
			m_xDebugOutputPos.y, 
			0.5f, 
			0xffffffff,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, 
			"GUID: %u - Health: %.0f", m_pxOwnerActor->GetGuid(), m_pxOwnerActor->GetHealth()
		);
		m_xDebugOutputPos.y += 10.0f;

		PARENT::DoRenderDebugInfo();

		Asura_Vector_3 xEyePosition = GetActor()->GetPosition();
		GetActor()->GetWorldSpaceMarkerPoint( 0xe5a85d04, &xEyePosition, NULL );	// target_2
		Asura_DebugLineRender::AddLine( xEyePosition, m_xCurrentLookAtPosition, 0xff00ff00 );		// green
		Asura_DebugLineRender::AddLine( xEyePosition, xEyePosition + m_xCurrentMovementDirection * 4.0f, 0xffff0000 );	// red
		Asura_DebugLineRender::AddLine( xEyePosition, xEyePosition + m_xCurrentAimDirection * 4.0f, 0xff0000ff );		// blue

		//Asura_Bounding_Box xBB;
		//xBB.SetToPoint( Asura_Maths::ZeroVector3 );
		//xBB.Grow( 0.5f );
		//Asura_DebugLineRender::AddBoundingBox( xBB, GetActor()->GetPosition(), GetActor()->GetOrientation(), 0xffff0000 );
		//Asura_DebugLineRender::AddBoundingBox( xBB, GetClientActor()->GetCurrentPosition(), GetClientActor()->GetCurrentOrientation(), 0xff00ff00 );
		const Asura_AI_NavVertex* pxWP = Asura_AI_Navigation_System::GetWaypoint( GetNavData().m_usCurrentWPID );
		if (pxWP)
		{
			if (IsInCurrentWaypoint())
			{
				Asura_DebugLineRender::AddSphere( pxWP->m_xPosition, 0.5f, 0xff0000ff, 4, 2 );
			}
			else
			{
				Asura_DebugLineRender::AddSphere( pxWP->m_xPosition, 0.5f, 0xffff0000, 4, 2 );
			}
		}
	}
}
#endif

#include "Core/Asura_MemoryManagement_DebugOff.h"
Axon_Behaviour* AVP_Brain_Actor::CreateBehaviour( const u_int uBehaviourType, const Asura_Message* pxMessage /*= NULL*/ )
{
    void* pxMemory = s_pxBehaviourMempool->Allocate();
    if ( !pxMemory )
    {
        Asura_Assert( false, "AVP_Brain_Actor::CreateBehaviour - could not allocate behaviour memory" );
        return NULL;
    }

    Axon_Behaviour* pxBehaviour = NULL;
    switch ( uBehaviourType )
    {
		case AXON_BEHAVIOUR_TYPE_GS2_ACTOR:
		{
			pxBehaviour = new ( pxMemory ) AVP_Behaviour_GS2Actor( this );
			break;
		}

		case AVP_BEHAVIOUR_TYPE_GS2_NODE_ANIMMESSAGES:
		{
			pxBehaviour = new ( pxMemory ) AVP_Behaviour_GS2Node_AnimMessages( this );
			break;
		}

		case AVP_BEHAVIOUR_TYPE_GS2_NODE_ROUTETRAVERSE:
		{
			pxBehaviour = new ( pxMemory ) AVP_Behaviour_GS2Node_RouteTraverse( this );
			break;
		}

		case AXON_BEHAVIOUR_TYPE_GS2_NODE:
		{
			pxBehaviour = new ( pxMemory ) Axon_Behaviour_GS2Node( this );
			break;
		}

		case AXON_BEHAVIOUR_TYPE_GS2_ATTRACTOR_EN_ROUTE:
		{
			pxBehaviour = new ( pxMemory ) Axon_Behaviour_GS2AttractorEnRoute( this );
			break;
		}

		case AXON_BEHAVIOUR_TYPE_GS2_SPLINE:
		{
			pxBehaviour = new ( pxMemory ) Axon_Behaviour_GS2Spline( this );
			break;
		}

        default:
        {
            break;
        }
    }

    if ( !pxBehaviour )
    {
        // Couldn't create the behaviour.
        s_pxBehaviourMempool->Deallocate( pxMemory );
    }
    else if ( pxMessage )
    {
        pxBehaviour->DoHandleMessage( pxMessage, true );
    }

    return pxBehaviour;
}
#include "Core/Asura_MemoryManagement_DebugOn.h"

float AVP_Brain_Actor::GetAnimationMovementSpeed() const
{
	Asura_Animation* pxAnim = m_pxClientOwnerActor->GetShapeInstance().GetAnimation();

	Asura_Vector_3 xLinearVelocity;
	if ( pxAnim && pxAnim->GetVelocity( &xLinearVelocity ) )
	{
		return xLinearVelocity.Magnitude();
	}
	else
	{
		return 0.0f;
	}
}

const Asura_Vector_3& AVP_Brain_Actor::ResetMovementDirection()
{
    const Asura_Matrix_3x3& xOrientation = GetActor()->GetOrientation();
    m_xDesiredMovementDirection.x = xOrientation.e13;
    m_xDesiredMovementDirection.y = xOrientation.e23;
    m_xDesiredMovementDirection.z = xOrientation.e33;
    m_xCurrentMovementDirection = m_xDesiredMovementDirection;
	GetActor()->SetMovementDirection( m_xCurrentMovementDirection );

	Asura_Assert( !m_xCurrentMovementDirection.ApproximatelyEquals( Asura_Maths::ZeroVector3 ), "AVP_Brain_Actor::SetMovementDirection - movement direction should never be zero" );

	return m_xCurrentMovementDirection;
}

const Asura_Vector_3& AVP_Brain_Actor::SetMovementDirection( const Asura_Vector_3& xDirection )
{
	Asura_Assert( !xDirection.ApproximatelyEquals( Asura_Maths::ZeroVector3 ), "AVP_Brain_Actor::SetMovementDirection - movement direction should never be zero" );
	Asura_Assert( xDirection == xDirection, "AVP_Brain_Actor::SetMovementDirection - bad vector" );
	m_xDesiredMovementDirection = xDirection;
	return m_xCurrentMovementDirection;
}

void AVP_Brain_Actor::ForceMovementDirection( const Asura_Vector_3& xMovementDirection )
{
	Asura_Assert( !xMovementDirection.ApproximatelyEquals( Asura_Maths::ZeroVector3 ), "AVP_Brain_Actor::SetMovementDirection - movement direction should never be zero" );
	Asura_Assert( xMovementDirection == xMovementDirection, "AVP_Brain_Actor::ForceMovementDirection - bad vector" );
	const Asura_Matrix_3x3& xOurOrientM = GetActor()->GetOrientation();
	Asura_Vector_3 xDesiredLocalVector = xOurOrientM.ApplyTranspose( xMovementDirection );
	xDesiredLocalVector.y = 0.0f;
	xDesiredLocalVector.Normalise();
	m_xCurrentMovementDirection = xDesiredLocalVector * xOurOrientM;
	m_xDesiredMovementDirection = xDesiredLocalVector * xOurOrientM;
	GetActor()->SetMovementDirection( m_xCurrentMovementDirection );
}

const Asura_Vector_3& AVP_Brain_Actor::ResetAimDirection()
{
	const Asura_Matrix_3x3& xOrientation = GetActor()->GetOrientation();
	m_xDesiredAimDirection.x = xOrientation.e13;
	m_xDesiredAimDirection.y = xOrientation.e23;
	m_xDesiredAimDirection.z = xOrientation.e33;
	m_xCurrentAimDirection = m_xDesiredAimDirection;
	return m_xCurrentAimDirection;
}

const Asura_Vector_3& AVP_Brain_Actor::SetAimDirection( const Asura_Vector_3& xDirection )
{
	Asura_Assert( !xDirection.ApproximatelyEquals( Asura_Maths::ZeroVector3 ), "AVP_Brain_Actor::SetAimDirection - aim direction should never be zero" );
	Asura_Assert( xDirection == xDirection, "AVP_Brain_Actor::SetAimDirection - bad vector" );
	m_xDesiredAimDirection = xDirection;
	return m_xCurrentAimDirection;
}

void AVP_Brain_Actor::ForceAimDirection( const Asura_Vector_3& xAimDirection )
{
	Asura_Assert( xAimDirection == xAimDirection, "AVP_Brain_Actor::ForceAimDirection - bad vector" );
	m_xCurrentAimDirection = xAimDirection;
	m_xDesiredAimDirection = xAimDirection;
}

const Asura_Vector_3& AVP_Brain_Actor::ResetLookAtPosition()
{
	const Asura_Matrix_3x3& xOri = GetActor()->GetOrientation();
	m_xDesiredLookAtPosition = GetActor()->GetPosition() + Asura_Vector_3( xOri.e13, xOri.e23, xOri.e33 ) * 10.0f;
	m_xCurrentLookAtPosition = m_xDesiredLookAtPosition;
	return m_xCurrentLookAtPosition;
}

const Asura_Vector_3& AVP_Brain_Actor::SetLookAtPosition( const Asura_Vector_3& xPos )
{
	Asura_Assert(xPos == xPos, "Trying To Set LookAtPosition to NAN");
	if (xPos == xPos)
	{
		m_xDesiredLookAtPosition = xPos;
	}
	return m_xCurrentLookAtPosition;
}

void AVP_Brain_Actor::ForceLookAtPosition( const Asura_Vector_3& xPos )
{
	Asura_Assert(xPos == xPos, "Trying To Set LookAtPosition to NAN");
	if (xPos == xPos)
	{
		m_xCurrentLookAtPosition =  xPos;
		m_xDesiredLookAtPosition = xPos;
	}
}

u_int AVP_Brain_Actor::GetLargestBehaviourSize()
{
	u_int uLargestBehaviourSize = 0;

	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_GS2Actor );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_GS2Node_AnimMessages );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_GS2Node_RouteTraverse );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, Axon_Behaviour_GS2AttractorEnRoute );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, Axon_Behaviour_GS2Node );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, Axon_Behaviour_GS2Spline );

    return uLargestBehaviourSize;
}

bool AVP_Brain_Actor::WillOrbitPosition( const Asura_Vector_3& xPosition ) const
{
	const Asura_Vector_3& xOurPosition = GetActor()->GetPosition();
	const Asura_Vector_3 xOurNormal( -GetActor()->GetCurrentGravityDirection() );
	const float	fMinTurnRadius = GetTurningCircleRadius();
	const float fMinTurnRadiusSq = fMinTurnRadius * fMinTurnRadius;

	const Asura_Vector_3 xRTurnCircleCentre = xOurPosition + ( m_xCurrentMovementDirection ^ xOurNormal ) * fMinTurnRadius;
	const Asura_Vector_3 xLTurnCircleCentre = xOurPosition + ( xOurNormal ^ m_xCurrentMovementDirection ) * fMinTurnRadius;

	return ( ( xPosition - xRTurnCircleCentre ).MagnitudeSquared() < fMinTurnRadiusSq ||
		( xPosition - xLTurnCircleCentre ).MagnitudeSquared() < fMinTurnRadiusSq );
}

float AVP_Brain_Actor::GetTurningCircleRadius() const
{
	return GetAnimationMovementSpeed() / m_fMaxMoveDirAngularVelocity;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

AVP_Brain_Actor::AVP_Brain_Actor(AVP_ServerEntity_Actor* pxActor)
: PARENT(pxActor->GetGuid())
, m_fNearbyBaseRange( 0.0f )
, m_fNearbyUpdateTimer( 0.0f )
, m_uNumNearbyGuids( 0 )
, m_uNearbyGuidToIgnore( ASURA_GUID_UNREGISTERED )
, m_bNearbyGuidIgnoreDeadActors(true)
, m_pxOwnerActor(pxActor)
, m_pxClientOwnerActor(NULL)
, m_uActorFlags(0)
, m_xCurrentMovementDirection( Asura_Maths::ZeroVector3 )
, m_xDesiredMovementDirection( Asura_Maths::ZeroVector3 )
, m_fMaxMoveDirAngularVelocity( Asura_Maths::Pi )
, m_xCurrentAimDirection( Asura_Maths::ZeroVector3 )
, m_xDesiredAimDirection( Asura_Maths::ZeroVector3 )
, m_fMaxAimDirAngularVelocity( Asura_Maths::Pi )
, m_xCurrentLookAtPosition( Asura_Maths::ZeroVector3 )
, m_xDesiredLookAtPosition( Asura_Maths::ZeroVector3 )
, m_fMaxLookDirAngularVelocity( 4.0f * Asura_Maths::Pi ) // eyes move quick
, m_bIsAllowedToExitGS2OnKnowledge( true )
#ifndef ASURA_NO_DEBUG_OUTPUT
, m_bRenderRoute(false)
#endif
, m_bChunkNavData( false )
{
    m_bUseAnimMovement = true;
}

AVP_Brain_Actor::~AVP_Brain_Actor()
{
	m_xTargetPriorityList.Clear();
}

void AVP_Brain_Actor::OnActivate()
{
	PARENT::OnActivate();
	EstablishNavData();
	UpdateCurrentWaypointID();
}

void AVP_Brain_Actor::OnDeactivate()
{
	RequestUnregisterWithTeam();
	PARENT::OnDeactivate();
}

bool AVP_Brain_Actor::HandleMessage( const Asura_Message* pxMessage )
{
	//switch ( pxMessage->GetMsgID() )
	//{
	//	case ASURA_MSG_MOVE_START:
	//	{
	//		const AVP_Msg_Move* pxData = static_cast< const AVP_Msg_Move* >( pxMessage->GetData() );
	//		Asura_OutputToDebugger( ( "AVP_Brain_Actor::HandleMessage - ASURA_MSG_MOVE_START (0x%08x)\n", pxData->GetMoveID() ) );
	//		break;
	//	}

	//	case ASURA_MSG_MOVE_ABORT:
	//	{
	//		const Asura_Msg_Move* pxData = static_cast< const Asura_Msg_Move* >( pxMessage->GetData() );
	//		Asura_OutputToDebugger( ( "AVP_Brain_Actor::HandleMessage - ASURA_MSG_MOVE_ABORT (0x%08x)\n", pxData->GetMoveID() ) );
	//		break;
	//	}

	//	case ASURA_MSG_MOVE_FINISHED:
	//	{
	//		const Asura_Msg_Move* pxData = static_cast< const Asura_Msg_Move* >( pxMessage->GetData() );
	//		Asura_OutputToDebugger( ( "AVP_Brain_Actor::HandleMessage - ASURA_MSG_MOVE_FINISHED (0x%08x)\n", pxData->GetMoveID() ) );
	//		break;
	//	}

	//	case ASURA_MSG_MOVECHAINCOMPLETE:
	//	{
	//		const Asura_Msg_MoveChainComplete* pxData = static_cast< const Asura_Msg_MoveChainComplete* >( pxMessage->GetData() );
	//		Asura_OutputToDebugger( ( "AVP_Brain_Actor::HandleMessage - ASURA_MSG_MOVECHAINCOMPLETE (0x%08x, 0x%08x)\n",
	//			pxData->GetInitialMoveID(), pxData->GetFinalMoveID() ) );
	//		break;
	//	}

	//	case AVP_MSGID_MOVESTART_SERVER:
	//	{
	//		const AVP_Msg_Move* pxData = static_cast< const AVP_Msg_Move* >( pxMessage->GetData() );
	//		Asura_OutputToDebugger( ( "AVP_Brain_Actor::HandleMessage - AVP_MSGID_MOVESTART_SERVER (0x%08x)\n", pxData->GetMoveID() ) );
	//		break;
	//	}

	//	case AVP_MSGID_INCOMING_MELEE_STARTED:
	//	{
	//		Asura_OutputToDebugger( ( "AVP_Brain_Actor::HandleMessage - AVP_MSGID_INCOMING_MELEE_STARTED\n" ) );
	//		break;
	//	}

	//	case AVP_MSGID_INCOMING_MELEE_FINISHED:
	//	{
	//		Asura_OutputToDebugger( ( "AVP_Brain_Actor::HandleMessage - AVP_MSGID_INCOMING_MELEE_FINISHED\n" ) );
	//		break;
	//	}

	//	case AVP_MSGID_MELEE_ATTACK_HIT:
	//	{
	//		Asura_OutputToDebugger( ( "AVP_Brain_Actor::HandleMessage - AVP_MSGID_MELEE_ATTACK_HIT\n" ) );
	//		break;
	//	}

	//	case AVP_MSGID_COUNTER_MELEE:
	//	{
	//		Asura_OutputToDebugger( ( "AVP_Brain_Actor::HandleMessage - AVP_MSGID_COUNTER_MELEE\n" ) );
	//		break;
	//	}
	//}

    bool bHandledMessage = false;
    
	switch ( pxMessage->GetMsgID() )
	{
		case ASURA_MSG_MOVE_START:
		case ASURA_MSG_MOVE_FINISHED:
		case ASURA_MSG_MOVE_ABORT:
		case ASURA_MSG_MOVECHAINCOMPLETE:
		case AVP_MSGID_MOVESTART_SERVER:
		case AVP_MSGID_REQUEST_HUMANOID_PATROL:
		case AVP_MSGID_ENTITYDAMAGE_APPLY_TO_ENTITY:
		case AVP_MSGID_INCOMING_MELEE_STARTED:
		case AVP_MSGID_INCOMING_MELEE_FINISHED:
		case AVP_MSGID_COUNTER_MELEE:
		case AVP_MSGID_PHYSICS_REORIENTATION_RESET:
		case AVP_MSGID_FLINCH:
		case AVP_MSGID_GRAPPLE_START:
		case AVP_MSGID_MELEE_ATTACK_HIT:
		case AVP_MSGID_MELEE_ATTACK_FAILED_TO_START:
		case AVP_MSGID_GRAPPLE_ATTACKER_UPDATE:
		case AVP_MSGID_GRAPPLE_VICTIM_UPDATE:
		case AVP_MSGID_ACTIVEGRAPPLE_RESULT:
		{
			if ( m_pxFirstBehaviour )
			{
				m_pxFirstBehaviour->DoHandleMessage( pxMessage );
			}
			bHandledMessage = true;
            break;
		}

		case AVP_MSGID_SET_AI_TARGET_PRIORITIES:
		{
			// Register the target priorities.
			const AVP_Msg_SetAITargetPriorities* pxMsgData = static_cast< const AVP_Msg_SetAITargetPriorities* >( pxMessage->GetData() );
			if ( pxMsgData )
			{
				for ( u_int u = 0; u < pxMsgData->m_uNumEntries; ++u )
				{
					const AVP_Msg_SetAITargetPriorities::TPData* pxMsgTPData = &pxMsgData->GetFirstTPDataPtr()[ u ];

					if ( pxMsgTPData->m_iPriority == AVP_AI_TargetPriorityData::UNSET )
					{
						continue;
					}

					// See if we already have an entry that matches.
					AVP_AI_TargetPriorityData* pxTPData = NULL;
					for ( Asura_LinkList_It< AVP_AI_TargetPriorityData > xTPIt( m_xTargetPriorityList ); !xTPIt.Done() && !pxTPData; xTPIt.Next() )
					{
						AVP_AI_TargetPriorityData* pxItTPData = xTPIt.GetCurrent();
						if ( pxItTPData->m_iTargetType == pxMsgTPData->m_iTargetType )
						{
							if ( pxItTPData->m_iTargetType == AVP_AI_TargetPriorityData::SPECIFIC_ENTITY )
							{
								if ( pxItTPData->m_uSpecificGuid == pxMsgTPData->m_uSpecificGuid )
								{
									pxTPData = pxItTPData;
								}
							}
							else
							{
								pxTPData = pxItTPData;
							}
						}
					}

					if ( !pxTPData )
					{
						// Have to add a new one.
						AVP_AI_TargetPriorityData xTP;
						m_xTargetPriorityList.Push_Front( xTP );
						pxTPData = &m_xTargetPriorityList.Front()->Data();
					}

					// Update this entry.
					// If the priority is normal then delete it.
					if ( pxMsgTPData->m_iPriority == AVP_AI_TargetPriorityData::NORMAL )
					{
						m_xTargetPriorityList.Remove( m_xTargetPriorityList.Find( *pxTPData ) );
					}
					else
					{
						pxTPData->m_iPriority = pxMsgTPData->m_iPriority;
						pxTPData->m_iTargetType = pxMsgTPData->m_iTargetType;
						pxTPData->m_iSpecificGuidType = pxMsgTPData->m_iSpecificGuidType;
						pxTPData->m_uSpecificGuid = pxMsgTPData->m_uSpecificGuid;
					}
				}
			}

			bHandledMessage = true;
			break;
		}

		case AVP_MSGID_RESET_AI_TARGET_PRIORITIES:
		{
			m_xTargetPriorityList.Clear();
			bHandledMessage = true;
			break;
		}

		default:
		{
			bHandledMessage = PARENT::HandleMessage( pxMessage );
		}
	}

    // Soak up any messages that we are sent but won't handle under all circumstances.
    switch ( pxMessage->GetMsgID() )
    {
        case AVP_MSGID_ITEM_BEGIN_IDLING:
        case AVP_MSGID_ITEM_BEGIN_USING:
		case AVP_MSGID_SET_PREFERRED_TARGET_TYPE:
		case AVP_MSGID_REQUEST_HUMANOID_PATROL:
        {
            bHandledMessage = true;
            break;
        }
    }

    // Override axon brain not handling messages if there is no first behaviour.
    if ( !m_pxFirstBehaviour )
    {
        bHandledMessage = true;
    }

    return bHandledMessage;
}

void AVP_Brain_Actor::UpdateCurrentWaypointID()
{
	AVP_Navigation::TestParams xTestParams;
	xTestParams.m_uPathfinderFlags     = m_xNavData.m_uPathfinderFlags;
	xTestParams.m_bRejectNonalignedWPs = true;
	xTestParams.m_xNormal              = -GetActor()->GetCurrentGravityDirection();
	xTestParams.m_usZone               = m_xNavData.m_usCurrentZone;
	u_short usCurrentWaypointID = m_xNavData.m_usCurrentWPID;
	AVP_Navigation::UpdateCurrentWaypointID( GetActor()->GetPosition(), usCurrentWaypointID, xTestParams );
	if ( usCurrentWaypointID != ASURA_INVALID_WAYPOINT_ID )
	{
		m_xNavData.m_usCurrentWPID = usCurrentWaypointID;
		const Asura_AI_NavVertex* pxNavVertex = Asura_AI_Navigation_System::GetWaypoint( usCurrentWaypointID );
		if (!pxNavVertex || !pxNavVertex->Contains(GetActor()->GetPosition()))
		{
			m_bInCurrentWaypoint = false;
		}
		else
		{
			m_bInCurrentWaypoint = true;
		}
	}
	else
	{
		m_bInCurrentWaypoint = false;
	}
}

void AVP_Brain_Actor::UpdateMovementDirection( const float fTimeslice )
{
    if ( fTimeslice == 0.0f )
    {
        return;
    }

	// if we're currently in a move then the client will be calculating the directions, so get the client data and use it here.
	const AVP_Move* pxCurrentMove = GetClientActor() ? ( GetClientActor()->GetMovementSystem() ? static_cast< const AVP_Move* >( GetClientActor()->GetMovementSystem()->GetCurrentMove() ) : NULL ) : NULL;
	if ( pxCurrentMove != NULL && pxCurrentMove->GetPositionUsage() != Asura_Move::POSITION_NOT_USED )
	{
		switch ( pxCurrentMove->GetMovementDirectionUsage() )
		{
			case AVP_Move::MOVEMENTDIRECTION_SET_FROM_VELOCITY:
			{
				Asura_Vector_3 xVel;
				GetClientActor()->GetMovementSystem()->GetCurrentMoveVelocity( xVel );
				if (!xVel.ApproximatelyEquals(Asura_Maths::ZeroVector3))
				{
					ForceMovementDirection( xVel );
				}
			}
			break;

			case AVP_Move::MOVEMENTDIRECTION_SET_FROM_ORIENTATION_FORWARDS_VECTOR:
			{
				const Asura_Matrix_3x3& xOri = GetClientActor()->GetCurrentOrientation();
				ForceMovementDirection( Asura_Vector_3( xOri.e13, xOri.e23, xOri.e33 ) );
			}
			break;

			case AVP_Move::MOVEMENTDIRECTION_NOT_USED:
			{
				// Do nothing
			}
			break;

			default:
			{
				Asura_Assertf( false, ( "Unandled case %d", pxCurrentMove->GetMovementDirectionUsage() ) );
			}
			break;
		}
	}
	else
	{
		// We can only update the movement direction if we're moving.
		if ( m_bUseAnimMovement && GetAnimationMovementSpeed() < 0.05f )
		{
			return;
		}

		const Asura_Matrix_3x3& xOurOrientM = GetActor()->GetOrientation();

		Asura_Vector_3 xDesiredLocalVector = xOurOrientM.ApplyTranspose( m_xDesiredMovementDirection );
		Asura_Vector_3 xCurrentLocalVector = xOurOrientM.ApplyTranspose( m_xCurrentMovementDirection );
		// Put these both into our plane.
		xDesiredLocalVector.y = 0.0f;
		if ( !xDesiredLocalVector.ApproximatelyEquals( Asura_Maths::ZeroVector3 ) ){ xDesiredLocalVector.Normalise(); }
		xCurrentLocalVector.y = 0.0f;
		if ( !xCurrentLocalVector.ApproximatelyEquals( Asura_Maths::ZeroVector3 ) ){ xCurrentLocalVector.Normalise(); }

		if ( !xCurrentLocalVector.ApproximatelyEquals( xDesiredLocalVector ) )
		{
			const float fDesPhi		= ( xDesiredLocalVector.GetX() != 0.0f || xDesiredLocalVector.GetZ() != 0.0f ) ? Asura_Maths::ArcTangent( xDesiredLocalVector.GetX(), xDesiredLocalVector.GetZ() ) : 0.0f;
			float fCurPhi			= ( xCurrentLocalVector.GetX() != 0.0f || xCurrentLocalVector.GetZ() != 0.0f ) ? Asura_Maths::ArcTangent( xCurrentLocalVector.GetX(), xCurrentLocalVector.GetZ() ) : 0.0f;

			const float	fMaxAngle = m_fMaxMoveDirAngularVelocity * fTimeslice;

			const float fDeltaPhi = Asura_Maths::ClampToRange( Asura_Maths::RadiansToZeroCentredRange( fDesPhi - fCurPhi ), -fMaxAngle, fMaxAngle );
			fCurPhi += fDeltaPhi;

			Asura_Assert( fCurPhi == fCurPhi, "Bad Phi" );
			if ( fCurPhi == fCurPhi )
			{
				float fAimX, fAimZ;
				Asura_Maths::SinCos( fCurPhi, fAimX, fAimZ );
				xCurrentLocalVector = Asura_Vector_3( fAimX, 0.0f, fAimZ );
	
				// Put this back into world space.
				m_xCurrentMovementDirection = xCurrentLocalVector * xOurOrientM;
			}
		}
	}

	GetActor()->SetMovementDirection( m_xCurrentMovementDirection );
}

DEBUGVAR float dbg_fTurnLimit = Asura_Maths::DegToRad( 80.0f );

void AVP_Brain_Actor::UpdateAimDirection( const float fTimeslice )
{
	if ( fTimeslice == 0.0f )
	{
		return;
	}

	// if we're currently in a move then the client will be calculating the directions, so get the client data and use it here.
	const AVP_Move* pxCurrentMove = GetClientActor() ? ( GetClientActor()->GetMovementSystem() ? static_cast< const AVP_Move* >( GetClientActor()->GetMovementSystem()->GetCurrentMove() ) : NULL ) : NULL;

	if ( pxCurrentMove != NULL && pxCurrentMove->GetAimDirectionUsage() != AVP_ClientEntity_Actor::AIM_DIRECTION_NOT_USED )
	{
		Asura_AVector3 xClientDir;
		GetClientActor()->GetAimDirection( xClientDir );
		ForceAimDirection( Asura_Vector_3( xClientDir ) );
	}
	else
	{
		if ( !m_xCurrentAimDirection.ApproximatelyEquals( m_xDesiredAimDirection ) )
		{
			const Asura_Matrix_3x3& xOurOrientM = GetActor()->GetOrientation();

			Asura_Vector_3 xDesiredLocalVector = xOurOrientM.ApplyTranspose( m_xDesiredAimDirection );
			if ( !xDesiredLocalVector.ApproximatelyEquals( Asura_Maths::ZeroVector3 ) ){ xDesiredLocalVector.Normalise(); }
			Asura_Vector_3 xCurrentLocalVector = xOurOrientM.ApplyTranspose( m_xCurrentAimDirection );
			if ( !xCurrentLocalVector.ApproximatelyEquals( Asura_Maths::ZeroVector3 ) ){ xCurrentLocalVector.Normalise(); }

			const float fDesPhi		= ( xDesiredLocalVector.GetX() != 0.0f || xDesiredLocalVector.GetZ() != 0.0f ) ? Asura_Maths::ArcTangent( xDesiredLocalVector.GetX(), xDesiredLocalVector.GetZ() ) : 0.0f;
			const float fDesTheta	= Asura_Maths::ArcSine( Asura_Maths::ClampToRange( -xDesiredLocalVector.GetY(), -1.0f, 1.0f ) );
			float fCurPhi			= ( xCurrentLocalVector.GetX() != 0.0f || xCurrentLocalVector.GetZ() != 0.0f ) ? Asura_Maths::ArcTangent( xCurrentLocalVector.GetX(), xCurrentLocalVector.GetZ() ) : 0.0f;
			float fCurTheta			= Asura_Maths::ArcSine( Asura_Maths::ClampToRange( -xCurrentLocalVector.GetY(), -1.0f, 1.0f ) );

			float fMaxAngle			= m_fMaxAimDirAngularVelocity * fTimeslice;
			float fDeltaPhi			= Asura_Maths::RadiansToZeroCentredRange( fDesPhi - fCurPhi );

			AVP_Animation_BodyRig* pxBodyRig = GetClientActor()->GetAnimationHandler() ? GetClientActor()->GetAnimationHandler()->GetBodyRig() : NULL;
			if ( pxBodyRig )
			{
				if ( pxBodyRig->IsTurning() )
				{
					fMaxAngle = pxBodyRig->GetTurnAnimAngVel() * fTimeslice * 1.7f; // arbitrary hack values FTW
					if ( fDeltaPhi * fMaxAngle < 0.0f && Asura_Maths::Abs( fDeltaPhi ) > Asura_Maths::Deg90 )
					{
						fDeltaPhi = fMaxAngle;
					}
					fMaxAngle = Asura_Maths::Abs( fMaxAngle );
				}
				else if ( Asura_Maths::Abs( fDesPhi ) > dbg_fTurnLimit )
				{
					SendTurnMessage( fDesPhi );
				}
			}

			const float fDeltaTheta = Asura_Maths::ClampToRange( fDesTheta - fCurTheta, -fMaxAngle, fMaxAngle );
			fCurTheta += fDeltaTheta;

			fDeltaPhi = Asura_Maths::ClampToRange( fDeltaPhi, -fMaxAngle, fMaxAngle );
			fCurPhi += fDeltaPhi;

			fCurPhi = Asura_Maths::ClampToRange( fCurPhi, -Asura_Maths::Deg90, Asura_Maths::Deg90 );

			Asura_Assert( fCurTheta == fCurTheta, "Bad Theta" );
			Asura_Assert( fCurPhi == fCurPhi, "Bad Phi" );
			
			if ( fCurPhi == fCurPhi && fCurTheta == fCurTheta )
			{
				float fAimX, fNegAimY, fAimZ, fCosTheta;
				Asura_Maths::SinCos( fCurPhi, fAimX, fAimZ );
				Asura_Maths::SinCos( fCurTheta, fNegAimY, fCosTheta );
				xCurrentLocalVector = Asura_Vector_3( fCosTheta * fAimX, -fNegAimY, fCosTheta * fAimZ );

				// Put this back into world space.
				m_xCurrentAimDirection = xCurrentLocalVector * xOurOrientM;
			}
		}
	}

	GetActor()->SetAimDirection( m_xCurrentAimDirection );
}

void AVP_Brain_Actor::UpdateLookAtPosition( const float fTimeslice )
{
	if ( fTimeslice == 0.0f )
	{
		return;
	}

	// if we're currently in a move then the client will be calculating the directions, so get the client data and use it here.
	const AVP_Move* pxCurrentMove = GetClientActor() ? ( GetClientActor()->GetMovementSystem() ? static_cast< const AVP_Move* >( GetClientActor()->GetMovementSystem()->GetCurrentMove() ) : NULL ) : NULL;

	if ( pxCurrentMove != NULL && pxCurrentMove->GetLookAtPositionUsage() != AVP_ClientEntity_Actor::LOOK_AT_POSITION_NOT_USED )
	{
		Asura_AVector3 xClientDir;
		GetClientActor()->GetLookAtPosition( xClientDir );
		ForceLookAtPosition( Asura_Vector_3( xClientDir ) );
	}
	else
	{
		Asura_Vector_3 xEyePos;
		GetActor()->GetHeadPosition( xEyePos );
		Asura_Vector_3 xCurrentLookDirection = m_xCurrentLookAtPosition - xEyePos;
		Asura_Vector_3 xDesiredLookDirection = m_xDesiredLookAtPosition - xEyePos;

		if ( !xCurrentLookDirection.ApproximatelyEquals( xDesiredLookDirection ) )
		{
			const Asura_Matrix_3x3& xOurOrientM = GetActor()->GetOrientation();

			Asura_Vector_3 xDesiredLocalVector = xOurOrientM.ApplyTranspose( xDesiredLookDirection );
			Asura_Vector_3 xCurrentLocalVector = xOurOrientM.ApplyTranspose( xCurrentLookDirection );

			float fCurrentLookDirMag = 0.0f;
			if ( !xCurrentLocalVector.ApproximatelyEquals( Asura_Maths::ZeroVector3 ) ){ xCurrentLocalVector.Normalise(); }
			if ( !xDesiredLocalVector.ApproximatelyEquals( Asura_Maths::ZeroVector3 ) ){ fCurrentLookDirMag = xDesiredLocalVector.Normalise(); }

			const float fDesPhi		= ( xDesiredLocalVector.GetX() != 0.0f || xDesiredLocalVector.GetZ() != 0.0f ) ? Asura_Maths::ArcTangent( xDesiredLocalVector.GetX(), xDesiredLocalVector.GetZ() ) : 0.0f;
			const float fDesTheta	= Asura_Maths::ArcSine( Asura_Maths::ClampToRange( -xDesiredLocalVector.GetY(), -1.0f, 1.0f ) );
			float fCurPhi			= ( xCurrentLocalVector.GetX() != 0.0f || xCurrentLocalVector.GetZ() != 0.0f ) ? Asura_Maths::ArcTangent( xCurrentLocalVector.GetX(), xCurrentLocalVector.GetZ() ) : 0.0f;
			float fCurTheta			= Asura_Maths::ArcSine( Asura_Maths::ClampToRange( -xCurrentLocalVector.GetY(), -1.0f, 1.0f ) );

			const float	fMaxAngle = m_fMaxLookDirAngularVelocity * fTimeslice;

			const float fDeltaTheta = Asura_Maths::ClampToRange( fDesTheta - fCurTheta, -fMaxAngle, fMaxAngle );
			fCurTheta += fDeltaTheta;

			const float fDeltaPhi = Asura_Maths::ClampToRange( Asura_Maths::RadiansToZeroCentredRange( fDesPhi - fCurPhi ), -fMaxAngle, fMaxAngle );
			fCurPhi += fDeltaPhi;

			Asura_Assert( fCurTheta == fCurTheta, "Bad Theta" );
			Asura_Assert( fCurPhi == fCurPhi, "Bad Phi" );

			if ( fCurPhi == fCurPhi && fCurTheta == fCurTheta )
			{
				float fLookX, fNegLookY, fLookZ, fCosTheta;
				Asura_Maths::SinCos( fCurPhi, fLookX, fLookZ );
				Asura_Maths::SinCos( fCurTheta, fNegLookY, fCosTheta );
				xCurrentLocalVector = Asura_Vector_3( fCosTheta * fLookX, -fNegLookY, fCosTheta * fLookZ );
	
				// Put this back into world space.
				xCurrentLookDirection = xCurrentLocalVector * xOurOrientM;
				m_xCurrentLookAtPosition = xEyePos + xCurrentLookDirection * fCurrentLookDirMag;
			}
		}
	}

	GetActor()->SetLookAtPosition( m_xCurrentLookAtPosition );
}

#if 0
void AVP_Brain_Actor_Predator::UpdateLook()
{
	switch ( m_eLookMode )
	{
		default:
		case LOOK_IN_MOVE_DIRECTION:
		{
			const Asura_Vector_3& xMoveDir = GetMovementDirection();
			GetPredatorActor()->SetLookDirection( xMoveDir );
			GetPredatorActor()->SetAimDirection( xMoveDir );
			break;
		}

		case LOOK_AT_POSITION:
		{
			Asura_Vector_3 xHeadPosition;
			GetPredatorActor()->GetHeadPosition( xHeadPosition );
			Asura_Vector_3 xToPosition = m_xLookVector - xHeadPosition;
			xToPosition.Normalise();

			GetPredatorActor()->SetLookDirection( xToPosition );
			GetPredatorActor()->SetAimDirection( xToPosition );
			break;
		}

		case LOOK_IN_DIRECTION:
		{
			GetPredatorActor()->SetLookDirection( m_xLookVector );
			GetPredatorActor()->SetAimDirection( m_xLookVector );
			break;
		}
	}
}
#endif


AVP_ServerEntity_Squad* AVP_Brain_Actor::GetSquad() const
{
	return AVP_ServerEntity_Squad::FindEntity( GetActor()->GetInitialSquadGuid() );
}

float AVP_Brain_Actor::GetNearbyRange() const
{
	return  m_fNearbyBaseRange + GetActor()->GetVelocity().Magnitude() * 2.0f;
}

void AVP_Brain_Actor::SendTurnMessage( const float fTurnAngle )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_REQUEST_TURN, sizeof( AVP_Message_Actor_RequestTurn ), GetActor()->GetClassification() );
	if(pxMessage)
	{
		AVP_Message_Actor_RequestTurn* pxData = static_cast< AVP_Message_Actor_RequestTurn* >( pxMessage->GetData() );
		pxData->SetTurnAngle( fTurnAngle );
		pxMessage->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );
		pxMessage->SetFromGuid( GetActor()->GetGuid() );
		Asura_Network::Send( pxMessage );
	}
}

void AVP_Brain_Actor::UpdateBehaviours( const float fTimeslice )
{
	PARENT::UpdateBehaviours( fTimeslice );
	m_bUpdated = true;
}

void AVP_Brain_Actor::CreateGS2BehaviourIfNecessary()
{
	// Create the GS2 behaviour if we are starting on gamescene and if we have never been updated.
	// (If we have been updated then we will have moved already and we don't want to reset back to our initial GS2 node.)
	if ( GetActor()->GetInitialGamesceneGuid() != ASURA_GUID_UNREGISTERED && !m_bUpdated )
	{
		AVP_ServerNode_GS2Node* pxGSNode = AVP_ServerNode_GS2Node::FindEntity( GetActor()->GetInitialGamesceneGuid() );
		if ( pxGSNode )
		{
			Axon_Behaviour_GS2Actor* pxGS2Behaviour = static_cast< Axon_Behaviour_GS2Actor* >( CreateActivateAndPushBehaviour( AXON_BEHAVIOUR_TYPE_GS2_ACTOR ) );
			if ( !pxGS2Behaviour ) { return; }

			#ifdef ASURA_TOOLS
			// In tools, if we are creating our initial behaviours (by being set active or initialising from blueprint) we don't want
			// to go into GS2 fully, as this will cause messages to be sent out. Instead we set a pending guid and go into GS2 fully
			// at PostLoadInitialise time.
			if ( Tools_System::IsActive() )
			{
				pxGS2Behaviour->SetInitialNodeGuid( GetActor()->GetInitialGamesceneGuid() );
			}
			else
			#endif
			{
				// Teleport ourselves to our initial node.
				Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_AXON_GS2_TELEPORT_TO_NODE, sizeof( Axon_Msg_GS2_Teleport_To_Node ) );
				if ( pxMessage )
				{
					Axon_Msg_GS2_Teleport_To_Node* pxData = static_cast< Axon_Msg_GS2_Teleport_To_Node* >( pxMessage->GetData() );
					pxData->m_uNodeGuid = GetActor()->GetInitialGamesceneGuid();
					DoHandleMessage( pxMessage );
					Asura_Network::DestroyMessage( pxMessage );
				}
			}
		}
	}
}

bool AVP_Brain_Actor::CalcTargetingPriority( const int iTargetClassification, const Asura_Guid uTargetGuid, int& iPriority ) const
{
	// See if this guid or type is down as being of a non-default targeting priority.
	const AVP_ServerEntity_Actor* pxTargetActor = AVP_ServerEntity_Actor::FindEntity( uTargetGuid );
	const bool bTargetIsPlayer = ( pxTargetActor && pxTargetActor->IsPlayer() );

	for ( Asura_LinkList_ConstIt< AVP_AI_TargetPriorityData > xTPIter( m_xTargetPriorityList ); !xTPIter.Done(); xTPIter.Next() )
	{
		const AVP_AI_TargetPriorityData* pxTP = xTPIter.GetCurrent();
		switch ( pxTP->m_iTargetType )
		{
			default:
			{
				if ( !bTargetIsPlayer && pxTP->ClassificationMatchesTargetType( iTargetClassification ) )
				{
					iPriority = pxTP->m_iPriority;
					return true;
				}
				else
				{
					break;
				}
			}
			case AVP_AI_TargetPriorityData::PLAYER:
			{
				if ( bTargetIsPlayer )
				{
					iPriority = pxTP->m_iPriority;
					return true;
				}
				else
				{
					break;
				}
			}
			case AVP_AI_TargetPriorityData::SPECIFIC_ENTITY:
			{
				if ( pxTP->m_uSpecificGuid == uTargetGuid )
				{
					iPriority = pxTP->m_iPriority;
					return true;
				}
				else
				{
					break;
				}
			}
		}
	}

	// Also check the squad.
	AVP_ServerEntity_Squad* pxSquad = GetSquad();
	if ( pxSquad && pxSquad->CalcTargetingPriority( iTargetClassification, uTargetGuid, iPriority ) )
	{
		return true;
	}

	return false;
}

#ifndef ASURA_NO_DEBUG_OUTPUT
bool AVP_Brain_Actor::Verify()
{
	Asura_Assert( m_pxOwnerActor, "AVP_Brain_Actor::Verify - no owner pointer" );
	Asura_Assert( m_pxClientOwnerActor, "AVP_Brain_Actor::Verify - no client owner pointer" );
	return ( PARENT::Verify() && m_pxOwnerActor && m_pxClientOwnerActor );
}
#endif

bool AVP_Brain_Actor::UpdateClientActorPointer()
{
	// We need to do this at the start of every frame in case our client entity has been destroyed
	m_pxClientOwnerActor = AVP_ClientEntity_Actor::FindEntity(GetOwnerGuid());
	return ( m_pxClientOwnerActor != NULL );
}

void AVP_Brain_Actor::UpdateNearbyGuidData( const float fTimeslice )
{
	static const float s_fUpdatePeriod = 0.125f;

	const Asura_Vector_3& xOurPosition = GetActor()->GetPosition();
	const float			  fNearbyRange =  GetNearbyRange();

	m_fNearbyUpdateTimer -= fTimeslice;
	if ( m_fNearbyUpdateTimer <= 0.0f )
	{
		m_uNumNearbyGuids = 0;
		for ( Asura_Hashed_List_ConstIt< AVP_ServerEntity_Actor > xActorIter( AVP_ServerEntity_Actor::GetList() );
			!xActorIter.Done() && m_uNumNearbyGuids < MAX_NEARBY_GUIDS;
			xActorIter.Next() )
		{
			const AVP_ServerEntity_Actor* pxActor = xActorIter.GetCurrent();
			if ( !pxActor ||
				pxActor->GetGuid() == GetOwnerGuid() ||
				pxActor->GetGuid() == m_uNearbyGuidToIgnore ||
				( m_bNearbyGuidIgnoreDeadActors && pxActor->IsDead() ) ||
				!pxActor->IsEntityActive() )
			{
				continue;
			}

			Asura_Vector_3 xFromActor = xOurPosition - pxActor->GetPosition();
			float          fFromActor = xFromActor.Normalise();
			if ( fFromActor < fNearbyRange )
			{
				m_axNearbyGuidData[ m_uNumNearbyGuids ].m_uClassification = pxActor->GetClassification();
				m_axNearbyGuidData[ m_uNumNearbyGuids ].m_uGuid     = pxActor->GetGuid();
				m_axNearbyGuidData[ m_uNumNearbyGuids ].m_xFromGuid = xFromActor;
				m_axNearbyGuidData[ m_uNumNearbyGuids ].m_xPosition = pxActor->GetPosition();
				m_axNearbyGuidData[ m_uNumNearbyGuids ].m_xVelocity = pxActor->GetVelocity();
				m_axNearbyGuidData[ m_uNumNearbyGuids ].m_fRange    = fFromActor;
				m_axNearbyGuidData[ m_uNumNearbyGuids ].m_bTakingDamage =
					( 0.0f < pxActor->GetLastDamageInterval() && pxActor->GetLastDamageInterval() < 1.0f );
				++m_uNumNearbyGuids;
			}
		}

		m_fNearbyUpdateTimer += s_fUpdatePeriod;
	}
	else
	{
		// We only need update the data if we've not (re)found it this update.
		for ( u_int uNearbyGuid = 0; uNearbyGuid < m_uNumNearbyGuids; ++uNearbyGuid )
		{
			const AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( m_axNearbyGuidData[ uNearbyGuid ].m_uGuid );
			if ( !pxActor ) { continue; }

			m_axNearbyGuidData[ uNearbyGuid ].m_xFromGuid = xOurPosition - pxActor->GetPosition();
			m_axNearbyGuidData[ uNearbyGuid ].m_fRange    = m_axNearbyGuidData[ uNearbyGuid ].m_xFromGuid.Normalise();
			m_axNearbyGuidData[ uNearbyGuid ].m_bTakingDamage =
				( 0.0f < pxActor->GetLastDamageInterval() && pxActor->GetLastDamageInterval() < 1.0f );
			m_axNearbyGuidData[ uNearbyGuid ].m_xPosition = pxActor->GetPosition();
			m_axNearbyGuidData[ uNearbyGuid ].m_xVelocity = pxActor->GetVelocity();

		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
