//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_NPC.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BRAIN_ACTOR_H__
#define __AVP_BRAIN_ACTOR_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_LinkList.h"

// avp
#include "Entity/Controller/AVP_Controller_Messages.h"

// avp axon
#include "Core/AVP_Navigation.h"
#include "Core/BrainTypes/Axon_Project_Brain.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_ClientEntity_Actor;
class AVP_ServerEntity_Actor;
class AVP_ServerEntity_Squad;
class AVP_Knowledge_System_Actor;
class Tools_EntityTemplateInstance;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Brain_Actor : public Axon_Project_Brain
{
    typedef Axon_Project_Brain PARENT;

public:
	AVP_Brain_Actor( AVP_ServerEntity_Actor* pxActor );
	virtual ~AVP_Brain_Actor();

	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
    virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;
	virtual void DoPerFrameUpdate(const float fTimeslice);
	virtual void PostLoadInitialise();
    virtual void Shutdown();
	
	virtual Asura_Guid GetPrimaryThreatGuid( void ) = 0;
	virtual Asura_Guid GetAlertGuid( void ) = 0;

    void RequestRegisterWithTeam();
    void RequestUnregisterWithTeam();

	void HandleGrapple(void);

#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );
#endif

#ifndef ASURA_NO_DEBUG_OUTPUT
    void SetRenderRoute( const bool bRender ) { m_bRenderRoute = bRender; }
    static void       SetRenderGuid( const Asura_Guid uGuid ) { s_uRenderGuid = uGuid; }
    static Asura_Guid GetRenderGuid() { return s_uRenderGuid; }
    virtual void DoRenderDebugInfo();
#endif

    virtual Axon_Behaviour* CreateBehaviour( const u_int uBehaviourType, const Asura_Message* pxMessage = NULL );

	virtual bool CanFidget() { return false; }


    // Get functions.
    AVP_ServerEntity_Actor*     GetActor()                          const { return m_pxOwnerActor; }
    AVP_ClientEntity_Actor*     GetClientActor()                    const { return m_pxClientOwnerActor; }
    bool                        GetDoPanicMechanicOnGamesceneExit() const { return m_bDoPanicMechanicOnGamesceneExit; }
    AVP_Navigation::NavData&    GetNavData()                              { return m_xNavData; }
	const AVP_Navigation::NavData& GetNavData()                     const { return m_xNavData; }
	float						GetAnimationMovementSpeed() const;

    void SetDoPanicMechanicOnGamesceneExit( bool bDo ) { m_bDoPanicMechanicOnGamesceneExit = bDo; }

    // Both functions return the interpolated movement direction.
    const Asura_Vector_3& ResetMovementDirection();
    const Asura_Vector_3& SetMovementDirection( const Asura_Vector_3& xDirection );
    const Asura_Vector_3& GetMovementDirection() const { return m_xCurrentMovementDirection; }
	const Asura_Vector_3& GetDesiredMovementDirection() const { return m_xDesiredMovementDirection; }

	void ForceMovementDirection( const Asura_Vector_3& xMovementDirection );
	void SetMaxMoveDirAngularVelocity( const float fMaxAV ) { m_fMaxMoveDirAngularVelocity = fMaxAV; }
	float GetMaxMoveDirAngularVelocity() const { return m_fMaxMoveDirAngularVelocity; }

	const Asura_Vector_3& ResetAimDirection();
	const Asura_Vector_3& SetAimDirection( const Asura_Vector_3& xDirection );
	const Asura_Vector_3& GetAimDirection() const { return m_xCurrentAimDirection; }
	const Asura_Vector_3& GetDesiredAimDirection() const { return m_xDesiredMovementDirection; }

	void ForceAimDirection( const Asura_Vector_3& xMovementDirection );
	void SetMaxAimDirAngularVelocity( const float fMaxAV ) { m_fMaxAimDirAngularVelocity = fMaxAV; }
	float GetMaxAimDirAngularVelocity() const { return m_fMaxAimDirAngularVelocity; }

	const Asura_Vector_3& ResetLookAtPosition();
	const Asura_Vector_3& SetLookAtPosition( const Asura_Vector_3& xAtPosition );
	const Asura_Vector_3& GetLookAtPosition() const { return m_xCurrentLookAtPosition; }
	void ForceLookAtPosition( const Asura_Vector_3& xMovementAtPosition );
	void SetMaxLookDirAngularVelocity( const float fMaxAV ) { m_fMaxLookDirAngularVelocity = fMaxAV; }
	float GetMaxLookeDirAngularVelocity() const { return m_fMaxLookDirAngularVelocity; }

	bool IsAllowedToExitGS2OnKnowledge( void )			{ return m_bIsAllowedToExitGS2OnKnowledge; }
	void SetIsAllowedToExitGS2OnKnowledge(bool bVal)	{ m_bIsAllowedToExitGS2OnKnowledge = bVal; }


	void SetNearbyActorGuidToIgnore( const Asura_Guid uGuid ) { m_uNearbyGuidToIgnore = uGuid; }
	void SetNearbyIgnoreDeadActors( const bool bIgnoreDeadActors ) { m_bNearbyGuidIgnoreDeadActors = bIgnoreDeadActors; }

	// Uses the character's current movement direction, speed and turn rate to determine if it will orbit or can reach the given position.
	bool WillOrbitPosition( const Asura_Vector_3& xPosition ) const;
	float GetTurningCircleRadius() const;

	// Returns whether it has a priority for this target.
	bool CalcTargetingPriority( const int iTargetClassification, const Asura_Guid uTargetGuid, int& iPriority ) const;

#if 0
	enum LookMode { LOOK_IN_MOVE_DIRECTION, LOOK_AT_POSITION, LOOK_IN_DIRECTION };
	void SetLook( const LookMode eMode, const Asura_Vector_3& xVector = Asura_Maths::ZeroVector3 );
#endif

	virtual void EstablishNavData() = 0;

	// entry point when GS2 behaviour is activated and deactivated.
	virtual void StartGS2() {}
	virtual void EndGS2() {}

	struct NearbyGuidData
	{
		u_int          m_uClassification;
		Asura_Guid     m_uGuid;
		Asura_Vector_3 m_xFromGuid;
		Asura_Vector_3 m_xPosition;
		Asura_Vector_3 m_xVelocity;
		float          m_fRange;
		union
		{
			u_int m_uFlags;
			struct 
			{
				// Whether the entity has taken damage in the last second.
				u_int m_bTakingDamage : 1;
			};
		};
	};
	u_int				  GetNumNearbyGuids() const { return m_uNumNearbyGuids; }
	const NearbyGuidData* GetNearbyGuidData() const { return m_axNearbyGuidData; }

	static u_int GetLargestBehaviourSize();	

	void SendTurnMessage( const float fTurnAngle );
	bool IsInCurrentWaypoint() const { return m_bInCurrentWaypoint; }

protected:
	virtual void OnActivate();
	virtual void OnDeactivate();
	virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual bool RegisterWithTeam() { return true; }
	virtual bool UnregisterWithTeam() { return true; }
	virtual void UpdateCurrentWaypointID();
	// Interpolate the current movement direction towards the desired.
	void UpdateMovementDirection( const float fTimeslice );
	void UpdateAimDirection( const float fTimeslice );
	void UpdateLookAtPosition( const float fTimeslice );

	AVP_ServerEntity_Squad* GetSquad() const;

	virtual void SetShouldChunkNavData( bool bVal ) { m_bChunkNavData = bVal; }

	virtual float GetNearbyRange() const;

	virtual void UpdateBehaviours( const float fTimeslice );

	void CreateGS2BehaviourIfNecessary();

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
#endif

	// Nearby guid stuff.
	float          m_fNearbyBaseRange;
	float          m_fNearbyUpdateTimer;
	enum { MAX_NEARBY_GUIDS = 16 };
	NearbyGuidData m_axNearbyGuidData[ MAX_NEARBY_GUIDS ];
	u_int          m_uNumNearbyGuids;
	Asura_Guid m_uNearbyGuidToIgnore;
	bool m_bNearbyGuidIgnoreDeadActors;


private:
    bool UpdateClientActorPointer();
	void UpdateNearbyGuidData( const float fTimeslice );

    AVP_ServerEntity_Actor* m_pxOwnerActor;
	AVP_ClientEntity_Actor* m_pxClientOwnerActor;

    union
	{
	    u_int m_uActorFlags;
	    struct
	    {
            u_int m_bUseAnimMovement                : 1, // no longer used
                  m_bDoPanicMechanicOnGamesceneExit : 1,
				  m_bRequestRegisterWithTeam		: 1,
				  m_bRequestUnregisterWithTeam		: 1,
				  m_bRegisteredWithTeam				: 1,
				  m_bUpdated						: 1, // whether this brain has ever been updated (consistent across chunking also)
				  m_bInCurrentWaypoint				: 1; // true if our position is inside our current waypoint.
	    };
	};

	AVP_Navigation::NavData m_xNavData;
	Asura_Vector_3			m_xCurrentMovementDirection;
	Asura_Vector_3			m_xDesiredMovementDirection;
    float                   m_fMaxMoveDirAngularVelocity;
    
	Asura_Vector_3			m_xCurrentAimDirection;
	Asura_Vector_3			m_xDesiredAimDirection;
	float                   m_fMaxAimDirAngularVelocity;

	Asura_Vector_3			m_xCurrentLookAtPosition;
	Asura_Vector_3			m_xDesiredLookAtPosition;
	float                   m_fMaxLookDirAngularVelocity;

	bool					m_bIsAllowedToExitGS2OnKnowledge;

#if 0
	LookMode					m_eLookMode;
#endif

private:
	Asura_LinkList< AVP_AI_TargetPriorityData > m_xTargetPriorityList;

#ifndef ASURA_NO_DEBUG_OUTPUT
    static Asura_Guid		s_uRenderGuid;
    bool					m_bRenderRoute;
#endif

	bool m_bChunkNavData;

private:
    enum { CURRENT_CHUNK_VERSION = 6 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BRAIN_ACTOR_H__
