//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_NPC_Alien.h
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BRAIN_NPC_ALIEN_H__
#define __AVP_BRAIN_NPC_ALIEN_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_LinkList.h"

// axon
#include "Behaviour/BehaviourTypes/Alien/AVP_Behaviour_Alien_Movement.h"
#include "Core/AxonAVP_Messages.h"
#include "Core/BrainTypes/AVP_Brain_Actor.h"

// avp
#include "Entity/Controller/AVP_Controller_Messages.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

struct AVP_AlienTeam_Knowledge;
class Asura_Blueprint;
class AVP_Behaviour_Alien_M2;
class AVP_ServerActor_Alien;
class AVP_ServerEntity_SpawnPoint2;
class AVP_ServerEntity_Squad_Alien;
class AVP_ServerEntity_Team_Alien;

enum AVP_AlienRange
{
	AVP_AR_NEAR,
	AVP_AR_NEARISH,
	AVP_AR_OK,
	AVP_AR_FARISH,
	AVP_AR_FAR,
	NUM_AVP_ALIEN_RANGES
};

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Alien
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Brain_Actor_Alien : public AVP_Brain_Actor
{
	typedef AVP_Brain_Actor PARENT;

public:
	AVP_Brain_Actor_Alien( AVP_ServerActor_Alien* pxAlien );
	virtual ~AVP_Brain_Actor_Alien();

    virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );
	virtual void PostLoadInitialise();
	virtual void DoPerFrameUpdate( const float fTimeslice );
	virtual void Shutdown();

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void DoRenderDebugInfo();
#endif

	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

    virtual Axon_Behaviour* CreateBehaviour( const u_int uBehaviourType, const Asura_Message* pxMessage = NULL );

	virtual Asura_Guid GetPrimaryThreatGuid( void );
	virtual Asura_Guid GetAlertGuid( void );

	virtual void EstablishNavData();
	virtual void StartGS2();
	virtual void EndGS2();

    AVP_ServerActor_Alien* GetAlienActor() const;
	AVP_ClientActor_Alien* GetClientAlienActor() const;
	AVP_ServerEntity_Squad_Alien* GetAlienSquad() const;
	AVP_ServerEntity_Team_Alien* GetAlienTeam() const;

    static u_int GetLargestBehaviourSize();

	// New stuff goes in here as we add it/decide we need it.
public:
	const AVP_AlienBehaviourData& GetBehaviourData() const;

	void IgnoreTarget( const Asura_Guid uTargetGuid, const float fIgnorePeriod );
	const AVP_AlienTeam_Knowledge* ChooseNewTarget();
	AVP_ServerEntity_SpawnPoint2* GetBestExit();
	bool CurrentTargetValid() const { return ( m_uCurrentTargetGuid != ASURA_GUID_UNREGISTERED ); }
	Asura_Guid GetCurrentTargetGuid() const { return m_uCurrentTargetGuid; }
	const AVP_AlienTeam_Knowledge* GetCurrentTargetKnowledge() const;
	u_int GetCurrentTargetWPID() const;
	const Asura_Vector_3& GetDirToCurrentTarget() const { return m_xToCurrentTarget; }
	float GetCurrentTargetDistance() const { return m_fCurrentTargetDistance; }
	const Asura_Vector_3& GetCurrentTargetPosition() const;
	AVP_AlienRange GetCurrentTargetRange() const { return m_eCurrentTargetRange; }
	bool CurrentTargetWithinRange() const;
	bool CurrentTargetWithinAttackFOV() const { return m_bWithinAttackFOV; }
	bool HaveLOSToCurrentTarget();
	bool IsAimedAtByCurrentTarget();
	const AVP_ServerEntity_Actor* GetCurrentTargetActor() const;

	bool CanAttack( const Asura_Guid uTargetGuid ) const;
	bool CanAttackCurrentTarget() const;

	bool CanCharge( const Asura_Guid uTargetGuid ) const;
	bool CanChargeCurrentTarget() const;


	void StartedTargeting( const Asura_Guid uTargetGuid );
	void StoppedTargetingCurrentTarget();
	void StartedAttackingCurrentTarget();
	void StoppedAttackingCurrentTarget();

	void StartedChargingCurrentTarget();
	void StoppedChargingCurrentTarget();


	bool IsInCoverFromTarget();
	bool WaypointIsInCoverFromTarget( const u_int uWPID );

	bool IsPlayingMove() const { return m_bPlayingMove; }

	bool IsInjured() const { return m_bIsInjured; }

	bool IsTightAvoidance() const { return !m_bIsLooseAvoidance; }

	bool NeedToAvoidLight() const;
	// Returns whether anything was calculated.
	bool CalcLightCentroid( Asura_Vector_3& xCentroid ) const;

	void ResolveToOurPlane( const Asura_Vector_3& xVectorIn, Asura_Vector_3& xVectorOut ) const;

	float GetDangerJumpFraction() const { return m_fDangerJumpFraction; }

	bool CanPlanMove() const;
	bool CanUpdateMelee() const;
	void SetFalling(bool bIsFalling);
	void SetSharpTurn(bool bIsSharpTurn);
	void SetJumping(bool bIsJumping);
	void SetGameScene(bool bIsGameScene);
	bool IsFalling() const { return m_bIsFalling; }
	bool IsJumping() const { return m_bIsJumping; }
	bool IsGS2() const { return m_bIsGS2; }
	bool IsSharpTurn() const { return m_bIsSharpTurn; }
	void SetMelee(bool bIsMelee);
	bool IsMelee() const { return m_bIsMelee; }
	
	float GetTimeSinceActivate() { return m_fTimeSinceActivate; }


	AVP_Alien_Movement& GetMovement() { return m_xMovement; }
	const AVP_Alien_Movement& GetMovement() const { return m_xMovement; }

	bool TargetValid() const { return CurrentTargetValid(); }
	bool CurrentTargetWithinRange( const float fRangeMin, const float fRangeMax ) const;


	bool ShouldRefreshPath(float fDeltaDestinationSquared, float fDeltaTargetForwardCos = 1.0f) const; // 1.0f ignore turn  completely

	bool ShouldIgnoreTarget( const Asura_Guid uTargetGuid );

	Asura_Guid GetCurrentGPV() const { return m_uCurrentGPV; }

	float GetInjuredLifeSpan() const { return m_fInjuredLifeSpan; }

	// enums to describe light status.
	enum InLightStatus {  IN_DARK, IN_LIGHT, BETWEEN_LIGHT_AND_DARK };
	enum InMarineLightStatus { MARINE_LIGHT_NO, MARINE_LIGHT_YES, MARINE_LIGHT_CLOSE };
	InLightStatus GetLightStatus() const { return m_eInLightStatus; }
	InMarineLightStatus GetInMarineLight() const { return m_eInMarineLightStatus; }
	bool GetInLightForMarine() const { return m_bInLightForMarine; }

	u_int GetStandAttackMin() const { return m_uMinNumStandAttack; }
	u_int GetStandAttackMax() const { return m_uMaxNumStandAttack; }
	bool GetPreferAttackToJump() const { return m_bPreferAttackToJump; }

protected:
	virtual void OnActivate();
	virtual void OnDeactivate();
	virtual void OnPause();
	virtual void OnUnpause();

	virtual bool HandleMessage( const Asura_Message* pxMessage );

	virtual bool RegisterWithTeam();
    virtual bool UnregisterWithTeam();
	virtual void UpdateBehaviours( const float fTimeslice );
	virtual float GetNearbyRange() const;


#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
#endif

private:
	void UpdateCurrentTargetData();
	void UpdateCurrentGPV();
	int GetTargetingPriority( const int iTargetClassification, const Asura_Guid uTargetGuid );
	void UpdateTargetIgnoreList( const float fTimeslice );
	void UpdateHiddenInShadows();

	static float GetScoreFromPriority( const int iPriority );

private:
	AVP_Alien_Movement m_xMovement;

	// Current target is set/unset when StartedTargeting/StoppedTargeting are called.
	// Behaviours should work off the current target unless they are allowed to update the best/current target.
	Asura_Guid     m_uCurrentTargetGuid;
	AVP_AlienRange m_eCurrentTargetRange;
	Asura_Guid     m_uCurrentGPV;
	u_int m_uLOSToCurrentTargetTestFrame;
	float m_fTimeSinceActivate;
	union
	{
		u_int m_uFlagsNotChunked;
		struct
		{
			u_int m_bWithinAttackFOV        : 1,
                  m_bPlayingMove            : 1,
				  m_bHaveLOSToCurrentTarget : 1,
				  m_bAttackingCurrentTarget : 1,
				  m_bChargingCurrentTarget	: 1,
				  m_bIsJumping				: 1,
				  m_bIsFalling				: 1,
				  m_bIsSharpTurn			: 1,
				  m_bIsGS2					: 1,
				  m_bIsMelee				: 1,
				  m_bLookForDestroyedSquad  : 1,
				  m_bIsLooseAvoidance		: 1,
				  m_bPreferAttackToJump     : 1;
		};
	};
	Asura_Vector_3 m_xToCurrentTarget;
	float		   m_fCurrentTargetDistance;

	union
	{
		u_int m_uFlagsChunked;
		struct
		{
			u_int m_bBehaviourDataValid : 1,
				  m_bIsInjured          : 1;
		};
	};
	AVP_AlienBehaviourData m_xBehaviourData;
	float m_fDangerJumpFraction;
	float m_fInjuredLifeSpan;

	struct TargetIgnoreData
	{
		Asura_Guid m_uGuidToIgnore;
		float	   m_fRemoveTime;

		Asura_Guid GetGuid() const { return m_uGuidToIgnore; }
	};
	Asura_LinkList< TargetIgnoreData > m_xTargetIgnoreList;
	u_int m_uMobility;
	u_int m_uMinNumStandAttack;
	u_int m_uMaxNumStandAttack;


	InLightStatus m_eInLightStatus;
	InMarineLightStatus m_eInMarineLightStatus;
	bool m_bInLightForMarine;



	enum { CURRENT_CHUNK_VERSION = 4 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // __AVP_BRAIN_NPC_ALIEN_H__
