//----------------------------------------------------------------------------
// File			: AVP_Brain_Actor_Civilian.cpp
// Created		: 29 / 10 / 07
// Created By	: Gareth Thomas
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// INCLUDES
//----------------------------------------------------------------------------
// Core
#include "Core/Asura.h"
// This files Header
#include "Core/BrainTypes/AVP_Brain_Actor_Civilian.h"
// Asura
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#include "Core/Asura_Scheduler.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Font_System.h"
#include "Interface/Asura_CommandConsole.h"
#endif

// Axon

// AVP
#include "Actor/Vertebrates/AVP_Vertebrate_AIAnimStateMachine.h"
#include "Actor/Vertebrates/Human/AVP_Human_MoraleHandler.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Locations/ShootAt/AVP_ServerEntity_ShootAt.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Civilian/AVP_ClientActor_Civilian.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Android/AVP_ServerActor_Android.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Civilian/AVP_ServerActor_Civilian.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/AVP_InventoryItem_Messages.h"
#include "Inventory/Items/AVP_InventoryItem_Torch.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Inventory/Items/Weapons/AVP_RangedWeapon_SubType.h"

// AVP Axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/Axon_Project_System.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Tactical/AVP_Tactical_System.h"

// Behaviour Header files includes
#include "Behaviour/BehaviourTypes/Human/AVP_Behaviour_Human_Grapple.h"
#include "Behaviour/BehaviourTypes/Human/AVP_Behaviour_Human_Movement.h"
#include "Behaviour/BehaviourTypes/Human/Civilian/AVP_Behaviour_Civilian.h"
#include "Behaviour/BehaviourTypes/Human/Civilian/AVP_Behaviour_Civilian_Alert.h"
#include "Behaviour/BehaviourTypes/Human/Civilian/AVP_Behaviour_Civilian_Patrol.h"
#include "Behaviour/BehaviourTypes/Human/Civilian/AVP_Behaviour_Civilian_Panic.h"
#include "Behaviour/BehaviourTypes/Human/Civilian/AVP_Behaviour_Civilian_Wander.h"
#include "Behaviour/BehaviourTypes/Human/Civilian/AVP_Behaviour_Civilian_Attack.h"



//----------------------------------------------------------------------------
// Statics
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian - Function Implementations
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: Constructor
//----------------------------------------------------------------------------
AVP_Brain_Actor_Civilian::AVP_Brain_Actor_Civilian(AVP_ServerActor_Civilian* pxCivilian): PARENT(pxCivilian)
, m_iKnowledgeUpdateHandle(ASURA_INVALID_HANDLE)
{
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: Initialise
//----------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::Initialise(void)
{
	PARENT::Initialise();

	m_pxKnowledgeSystem = new AVP_Knowledge_System_Actor(this);

	if (m_pxKnowledgeSystem)
	{
		m_pxKnowledgeSystem->Initialise();
	}

	m_pxTacticalSystem = new AVP_Tactical_System(this);

	SetShouldChunkNavData( true );
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Civilian_Travel :: EstablishNavData
//---------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::PostLoadInitialise()
{
	PARENT::PostLoadInitialise();

	SetShouldChunkNavData( true );
}

//---------------------------------------------------------------------------
// AVP_Behaviour_Civilian_Travel :: EstablishNavData
//---------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::EstablishNavData(void)
{
	AVP_ServerActor_Civilian* pxOurServerActor = GetCivilianActor();

	AVP_Navigation::NavData& xNavData = GetNavData();

	// Set the human path finding flags
	xNavData.m_uPathfinderFlags = AVP_Navigation::AVP_AI_PATHFINDER_FLAG_HUMANOID;
	xNavData.m_eMovementMode = AVP_Navigation::MM_SLOW;

	// Establish our current waypoint ID.
	AVP_Navigation::TestParams xTestParams;
	xTestParams.m_uPathfinderFlags		= xNavData.m_uPathfinderFlags;
	xTestParams.m_bRejectNonalignedWPs	= true;
	xTestParams.m_xNormal				= -pxOurServerActor->GetCurrentGravityDirection();

	const bool bGotWaypointID = AVP_Navigation::GetNearestWaypointID(pxOurServerActor->GetPosition(), xNavData.m_usCurrentWPID, xTestParams);
	if (bGotWaypointID)
	{
		// Establish our current zone.
		xNavData.m_usCurrentZone = Asura_AI_Navigation_System::GetWaypoint(xNavData.m_usCurrentWPID)->m_usZone;
	}

	xNavData.m_fCharacterHeight = 2.0f;
	xNavData.m_fCharacterWidth  = 1.0f;
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: Shutdown
//----------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::Shutdown(void)
{
	delete m_pxTacticalSystem;
	m_pxTacticalSystem = NULL;

	if (m_pxKnowledgeSystem)
	{
		m_pxKnowledgeSystem->Shutdown();
	}

	delete m_pxKnowledgeSystem;
	m_pxKnowledgeSystem = NULL;

	PARENT::Shutdown();
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: Shutdown
//----------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::InitialiseFromBlueprint(const Asura_Blueprint& xBlueprint)
{
	PARENT::InitialiseFromBlueprint(xBlueprint);

	if ( m_pxKnowledgeSystem )
	{
		m_pxKnowledgeSystem->InitialiseFromBlueprint( xBlueprint );
	}

	CleanUpBehaviours();
	if ( IsActive() )
	{
		if (!CreateDefaultBehaviour())
		{
			Asura_Assert(false, "AVP_Brain_Actor_Civilian :: InitialiseFromBlueprint - Could not create / push default marine behaviour");
		}
		CreateGS2BehaviourIfNecessary();
	}

	EstablishNavData();
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: CleanUpBehaviours
//----------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::CleanUpBehaviours()
{
	DestroyAllBehaviours();
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: CreateBeahviour
//----------------------------------------------------------------------------
Axon_Behaviour* AVP_Brain_Actor_Civilian::CreateBehaviour(const u_int uBehaviourType, const Asura_Message* pxMessage)
{
	bool				bIsCivilianBehaviour	= IsCivilianBehaviour(uBehaviourType);
	Axon_Behaviour*		pxBehaviour			= NULL;

	if (bIsCivilianBehaviour)
	{
		pxBehaviour = CreateCivilianBehaviour(uBehaviourType);

		if (pxBehaviour && pxMessage)
		{
			pxBehaviour->DoHandleMessage(pxMessage, true);
		}
	}
	else
	{
		pxBehaviour = PARENT::CreateBehaviour(uBehaviourType, pxMessage);
	}

	return pxBehaviour;
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: GetPrimaryThreatGuid
//----------------------------------------------------------------------------
Asura_Guid AVP_Brain_Actor_Civilian::GetPrimaryThreatGuid()
{
	AVP_Knowledge_System_Actor*	pxKnowledgeSystem = GetKnowledgeSystem();

	if ( !pxKnowledgeSystem ) 
	{
		return ASURA_GUID_UNREGISTERED;
	}
	else
	{
		return  pxKnowledgeSystem->GetBestThreatKnowledge().GetGuid();
	}
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: GetAlertGuid
//----------------------------------------------------------------------------
Asura_Guid AVP_Brain_Actor_Civilian::GetAlertGuid( void )
{
	AVP_Knowledge_System_Actor*	pxKnowledgeSystem = GetKnowledgeSystem();
	
	if ( !pxKnowledgeSystem ) 
	{
		return ASURA_GUID_UNREGISTERED;
	}

	const Axon_Knowledge xAlertInfo = pxKnowledgeSystem->GetBestAlertKnowledge();

	if ( xAlertInfo.m_uSourceGuid != ASURA_GUID_UNREGISTERED )
	{
		AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( xAlertInfo.m_uSourceGuid );

		if ( pxActor )
		{
			return xAlertInfo.m_uSourceGuid;
		}
		else
		{
			pxActor = AVP_ServerEntity_Actor::FindEntity( xAlertInfo.m_uProjectData );

			if ( pxActor )
			{
				return xAlertInfo.m_uProjectData;
			}
		}
	}

	return ASURA_GUID_UNREGISTERED;
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: RegisterCallbacks
//----------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::RegisterCallbacks()
{
	PARENT::RegisterCallbacks();

	if (m_iKnowledgeUpdateHandle == ASURA_INVALID_HANDLE)
	{
		Asura_Scheduler& xKnowledgeUpdateScheduler = Axon_System::GetKnowledgeUpdateScheduler();
		Asura_Callback_MemberFunction_1Arg<void, float, AVP_Brain_Actor_Civilian> xKnowledgeCallback( this, &AVP_Brain_Actor_Civilian::DoKnowledgeUpdate );
		m_iKnowledgeUpdateHandle = xKnowledgeUpdateScheduler.RegisterCallback( 0.5f, &xKnowledgeCallback );
	}
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: UnregisterCallbacks
//----------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::UnregisterCallbacks()
{
	PARENT::UnregisterCallbacks();

	if (m_iKnowledgeUpdateHandle != ASURA_INVALID_HANDLE)
	{
		Axon_System::GetKnowledgeUpdateScheduler().UnregisterCallback( m_iKnowledgeUpdateHandle );
		m_iKnowledgeUpdateHandle = ASURA_INVALID_HANDLE;
	}
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: HandleMessage
//----------------------------------------------------------------------------
bool AVP_Brain_Actor_Civilian::HandleMessage(const Asura_Message* pxMessage)
{
	switch (pxMessage->GetMsgID())
	{
		case AVP_MSGID_ITEM_BEGIN_IDLING:
		{
			const AVP_Message_InventoryItem_BeginIdling*	pxData	= static_cast<const AVP_Message_InventoryItem_BeginIdling*>(pxMessage->GetData());

			if( (pxData != NULL) &&
				(pxData->GetGroupHash() == uAVP_INVENTORY_GROUPHASH_WEAPON) &&
				(pxData->GetUsageType() == AVP_InventoryItem_Weapon::UT_RELOAD_ALL) )
			{
				GetTacticalSystem()->SetReloadingWeapon(false);
			}

			return true;
		}

		case AVP_MSGID_ITEM_BEGIN_USING:
		{
			const AVP_Message_InventoryItem_BeginUsing* pxData = static_cast<const AVP_Message_InventoryItem_BeginUsing*>(pxMessage->GetData());

			if ((pxData) && (pxData->GetGroupHash() == uAVP_INVENTORY_GROUPHASH_WEAPON) && (pxData->GetUsageType()== AVP_InventoryItem_Weapon::UT_RELOAD_ALL) && (!GetTacticalSystem()->IsReloadingWeapon()))
			{
				if (GetKnowledgeSystem())
				{
					if (GetKnowledgeSystem()->GetBestThreatKnowledge().GetGuid() != ASURA_GUID_UNREGISTERED)
					{
						if (GetCivilianActor()->GetMoraleHandler())
						{
							GetCivilianActor()->GetMoraleHandler()->EffectMorale(AVP_Human_MoraleHandler::AVP_MORALE_EFFECTOR_RELOADING_WHILE_UNDER_THREAT);
						}
					}
				}

				GetTacticalSystem()->SetReloadingWeapon(true);
			}

			return true;
		}

		case AVP_MSGID_ATTACK_SCRIPTED_TARGET:
		{
			const AVP_Message_AttackScriptedTarget* pxMsgData = static_cast< const AVP_Message_AttackScriptedTarget*>( pxMessage->GetData() );

			if ( pxMsgData )
			{
				AVP_ServerEntity_ShootAt* pxShootAt = AVP_ServerEntity_ShootAt::FindEntity( pxMsgData->m_uShootAtGuid );

				if ( pxShootAt )
				{
					if ( pxShootAt->GetShootAtPlayer() )
					{
						// Find the player
						// This Needs changing in future as its single player only
						AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( 1000 ); 

						if ( pxPlayerInfo )
						{
							AVP_ServerEntity_Actor* pxPlayer = AVP_ServerEntity_Actor::FindEntity( pxPlayerInfo->GetPuppetGuid() );

							if ( pxPlayer )
							{
								// Shoot at entity!
								GetKnowledgeSystem()->SetScriptedTargetIsShootAtPoint( false );
								GetKnowledgeSystem()->SetScriptedTarget( pxPlayer->GetGuid() );
							}
						}
					}
					else if ( pxShootAt->GetShouldShootAtLinkedEntity() )
					{
						// Shoot at entity!
						GetKnowledgeSystem()->SetScriptedTargetIsShootAtPoint( false );
						GetKnowledgeSystem()->SetScriptedTarget( pxShootAt->GetShootAtEntityGuid() );
					}
					else
					{
						// Shoot at ShootAtPoint!
						GetKnowledgeSystem()->SetScriptedTargetIsShootAtPoint( true );
						GetKnowledgeSystem()->SetScriptedTarget( pxShootAt->GetGuid() );
					}
				}
			}

			return true;

		}

		case AVP_MSGID_STOP_ATTACKING_SCRIPTED_TARGET:
		{
			GetKnowledgeSystem()->StopAttackingScriptedTarget();
			return true;
		}

		case AVP_MSGID_REQUEST_TORCH_ON:
		{
			TorchOn();
			return(true);
		}

		case AVP_MSGID_REQUEST_TORCH_OFF:
		{
			TorchOff();
			return(true);
		}

		default:
		{
			return PARENT::HandleMessage(pxMessage);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Brain_Actor_Marine::IsTorchOn
// Author       : Andy Tate
// Date         : 25-Mar-2008
////////////////////////////////////////////////////////////////////////////////
bool AVP_Brain_Actor_Civilian::IsTorchOn(void) const
{
	const AVP_ServerActor_Human*	pxHuman		= (AVP_ServerActor_Human*)GetCivilianActor();
	const AVP_InventoryItem_Torch*	pxTorchItem	= pxHuman->GetTorchItem();

	if(pxTorchItem == NULL)
	{
		return(false);
	}
	else
	{
		return(pxTorchItem->IsUsing( false ));
	}
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Marine :: TorchOn
//----------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::TorchOn(void)
{
	AVP_ServerActor_Human*		pxHuman		= (AVP_ServerActor_Human*)GetCivilianActor();
	AVP_InventoryItem_Torch*	pxTorchItem	= pxHuman->GetTorchItem();

	if(pxTorchItem != NULL)
	{
		pxTorchItem->RequestBeginUsage(AVP_InventoryItem_Torch::UT_TORCH, AVP_InventoryItem_Torch::TT_AUTOMATIC);
	}
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Marine :: TorchOff
//----------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::TorchOff(void)
{
	AVP_ServerActor_Human*		pxHuman		= (AVP_ServerActor_Human*)GetCivilianActor();
	AVP_InventoryItem_Torch*	pxTorchItem	= pxHuman->GetTorchItem();

	if(pxTorchItem != NULL)
	{
		pxTorchItem->RequestEndUsage(AVP_InventoryItem_Torch::TT_AUTOMATIC);
	}
}


//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: GetNearbyRange
//----------------------------------------------------------------------------

DEBUGVAR float dbg_fSlowRange = 4.0f;
DEBUGVAR float dbg_fMediumRange = 8.0f;
DEBUGVAR float dbg_fFastRange = 10.0f;

float AVP_Brain_Actor_Civilian::GetNearbyRange() const
{
	switch (GetActor()->GetAIAnimStateMachine()->GetTargetState())
	{

	case AVP_Vertebrate_AIAnimStateMachine::ACTOR_STATE_IDLE:
		return 0.0f;
	case AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_SLOW:
		return dbg_fSlowRange;
	default:
	case AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_MEDIUM:
		return dbg_fMediumRange;
	case AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_FAST:
		return dbg_fFastRange;
	}
}


//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: CreateCivilianBeahviour
//----------------------------------------------------------------------------
#include "Core/Asura_MemoryManagement_DebugOff.h"
Axon_Behaviour* AVP_Brain_Actor_Civilian::CreateCivilianBehaviour(u_int uBehaviourType)
{
	void*				pxMemory			= s_pxBehaviourMempool->Allocate();
	Axon_Behaviour*		pxBehaviour			= NULL;

	if (!pxMemory)
	{
		Asura_Assert(false, "Axon_Project_Brain::CreateBehaviour - could not allocate behaviour memory");
		return NULL;
	}

	switch (uBehaviourType)
	{
		//-------------------------------------------------------------------------------------------------------------------------------
		case AVP_BEHAVIOUR_TYPE_CIVILIAN:					pxBehaviour = new (pxMemory) AVP_Behaviour_Civilian(this); break;
		//-------------------------------------------------------------------------------------------------------------------------------
		case AVP_BEHAVIOUR_TYPE_CIVILIAN_ALERT:				pxBehaviour = new (pxMemory) AVP_Behaviour_Civilian_Alert(this); break;
		//-------------------------------------------------------------------------------------------------------------------------------
		case AVP_BEHAVIOUR_TYPE_CIVILIAN_PATROL:			pxBehaviour = new (pxMemory) AVP_Behaviour_Civilian_Patrol(this); break;
		//-------------------------------------------------------------------------------------------------------------------------------
		case AVP_BEHAVIOUR_TYPE_CIVILIAN_PANIC:				pxBehaviour = new (pxMemory) AVP_Behaviour_Civilian_Panic(this); break;
		//-------------------------------------------------------------------------------------------------------------------------------
		case AVP_BEHAVIOUR_TYPE_CIVILIAN_WANDER:			pxBehaviour = new (pxMemory) AVP_Behaviour_Civilian_Wander(this); break;
		//-------------------------------------------------------------------------------------------------------------------------------
		case AVP_BEHAVIOUR_TYPE_CIVILIAN_ATTACK:			pxBehaviour = new (pxMemory) AVP_Behaviour_Civilian_Attack(this); break;
		//-------------------------------------------------------------------------------------------------------------------------------
		case AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT:				pxBehaviour = new (pxMemory) AVP_Behaviour_Human_Movement(this); break;
		//-------------------------------------------------------------------------------------------------------------------------------
		case AVP_BEHAVIOUR_TYPE_HUMAN_GRAPPLE:				pxBehaviour = new (pxMemory) AVP_Behaviour_Human_Grapple(this); break;
		//-------------------------------------------------------------------------------------------------------------------------------
		default: Asura_Assert(false, "AVP_Brain_Actor_Civilian :: CreateCivilianBeahviour - Unknown Behaviour Type"); break;
		//-------------------------------------------------------------------------------------------------------------------------------
	}

	if (!pxBehaviour)
	{
		s_pxBehaviourMempool->Deallocate(pxMemory);
	}

	return pxBehaviour;
}
#include "Core/Asura_MemoryManagement_DebugOn.h"

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: IsCivilianBehaviour
//----------------------------------------------------------------------------
bool AVP_Brain_Actor_Civilian::IsCivilianBehaviour(const u_int uBehaviourType)
{
	// ADD ALL CIVILIAN BEHAVIOURS HERE!
	switch (uBehaviourType)
	{
		//-----------------------------------------------
		case AVP_BEHAVIOUR_TYPE_CIVILIAN:
		case AVP_BEHAVIOUR_TYPE_CIVILIAN_ALERT:
		case AVP_BEHAVIOUR_TYPE_CIVILIAN_PATROL:
		case AVP_BEHAVIOUR_TYPE_CIVILIAN_PANIC:
		case AVP_BEHAVIOUR_TYPE_CIVILIAN_WANDER:
		case AVP_BEHAVIOUR_TYPE_CIVILIAN_ATTACK:
		case AVP_BEHAVIOUR_TYPE_HUMAN_MOVEMENT:
		case AVP_BEHAVIOUR_TYPE_HUMAN_GRAPPLE:
		{
			return true;
		}
		//-----------------------------------------------
		default:
		{
			return false;
		}
		//-----------------------------------------------
	}
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: CreateDefaultBehaviour
//----------------------------------------------------------------------------
bool AVP_Brain_Actor_Civilian::CreateDefaultBehaviour(void)
{
	return ( CreateActivateAndPushBehaviour(AVP_BEHAVIOUR_TYPE_CIVILIAN) != NULL );
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: ReadFromChunkStream
//----------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iVersion;
	xStream >> iVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( iVersion == CURRENT_CHUNK_VERSION )
	{

	}
	else
	{
		Asura_Assertf( false, ( "AVP_Brain_Actor_Civilian::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: WriteToChunkStream
//----------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << CURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );
}

//----------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: WriteToChunkStream
//----------------------------------------------------------------------------
AVP_ServerActor_Civilian* AVP_Brain_Actor_Civilian::GetCivilianActor() const
{
	return static_cast<AVP_ServerActor_Civilian* >(GetActor());
}

//------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: UpdateKnowledge
//------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::UpdateKnowledge(const float fTimeslice)
{
	if (m_pxKnowledgeSystem)
	{ 
		m_pxKnowledgeSystem->Update(fTimeslice); 
	}
}

//------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: DoesHaveThreat
//------------------------------------------------------------------------
bool AVP_Brain_Actor_Civilian::DoesHaveThreat(void)
{
	AVP_Knowledge_System_Actor* pxKnowledgeSystem = GetKnowledgeSystem();

	if ( ( !pxKnowledgeSystem ) || ( pxKnowledgeSystem->GetBestThreatKnowledge().GetGuid() == ASURA_GUID_UNREGISTERED ) )
	{
		return false;
	}

	return true;
}

//------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: DoesHaveAlert
//------------------------------------------------------------------------
bool AVP_Brain_Actor_Civilian::DoesHaveAlert(void)
{
	AVP_Knowledge_System_Actor* pxKnowledgeSystem = GetKnowledgeSystem();

	if ( ( !pxKnowledgeSystem ) || ( pxKnowledgeSystem->GetBestAlertKnowledge().GetGuid() == ASURA_GUID_UNREGISTERED ) )
	{
		return false;
	}

	return true;
}

//------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: GetCurrentWeaponType
//------------------------------------------------------------------------
u_int AVP_Brain_Actor_Civilian::GetCurrentWeaponType()
{
	// What weapon Do I have?
	AVP_ServerActor_Civilian*	pxOurActor			= GetCivilianActor();
	AVP_Inventory_System*		pxInventorySystem	= pxOurActor->GetInventorySystem();
	AVP_Inventory_Item*			pxCurrentWeapon		= pxInventorySystem ? pxInventorySystem->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	AVP_RangedWeapon_SubType*	pxSubType			= (pxCurrentWeapon && pxCurrentWeapon->GetNumSubTypes() > 0 ) ? ( AVP_RangedWeapon_SubType* )pxCurrentWeapon->GetSubType( 0 ) : NULL;
	
	return ( pxSubType ? pxSubType->GetAIWeaponBehaviourType() : static_cast< u_int >( AVP_WEAPON_BEHAVIOUR_TYPE_NONE ) );
}

//------------------------------------------------------------------------
// AVP_Brain_Actor_Marine :: SetLookAtPosition
//------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::AimAtPosition(const Asura_Vector_3& xPosition)
{
	AVP_ServerActor_Civilian* pxOurActor = GetCivilianActor();
	Asura_Vector_3 xChestPos( Asura_Maths::ZeroVector3 );
	pxOurActor->GetChestPosition( xChestPos );
	Asura_Vector_3 xDirection( xPosition - xChestPos );
	xDirection.Normalise();
	SetAimDirection(xDirection);
}

//------------------------------------------------------------------------
// AVP_Brain_Actor_Marine :: GetThreatPos
//------------------------------------------------------------------------
void AVP_Brain_Actor_Civilian::GetThreatPos( Asura_Vector_3& xPosition )
{
	xPosition = GetKnowledgeSystem()->GetBestThreatKnowledge().m_xPosition;
}

//------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: GetLaregestBehaviourSize
//------------------------------------------------------------------------
u_int AVP_Brain_Actor_Civilian::GetLargestBehaviourSize()
{
	u_int uLargestBehaviourSize = 0;

	AXON_CALC_MAX_BEHAVIOUR_SIZE(uLargestBehaviourSize, AVP_Behaviour_Civilian);
	AXON_CALC_MAX_BEHAVIOUR_SIZE(uLargestBehaviourSize, AVP_Behaviour_Civilian_Alert);
	AXON_CALC_MAX_BEHAVIOUR_SIZE(uLargestBehaviourSize, AVP_Behaviour_Civilian_Patrol);
	AXON_CALC_MAX_BEHAVIOUR_SIZE(uLargestBehaviourSize, AVP_Behaviour_Civilian_Panic);
	AXON_CALC_MAX_BEHAVIOUR_SIZE(uLargestBehaviourSize, AVP_Behaviour_Civilian_Wander);
	AXON_CALC_MAX_BEHAVIOUR_SIZE(uLargestBehaviourSize, AVP_Behaviour_Civilian_Attack);
	AXON_CALC_MAX_BEHAVIOUR_SIZE(uLargestBehaviourSize, AVP_Behaviour_Human_Movement);

	return uLargestBehaviourSize;
}

#ifndef ASURA_NO_DEBUG_OUTPUT
//------------------------------------------------------------------------
// AVP_Brain_Actor_Civilian :: Verify
//------------------------------------------------------------------------
bool AVP_Brain_Actor_Civilian::Verify()
{
	Asura_Assert( m_pxKnowledgeSystem, "AVP_Brain_Actor_Civilian::Verify - no knowledge system" );
	Asura_Assert( GetCivilianActor(), "AVP_Brain_Actor_Civilian::Verify - no civilian actor" );

	return ( PARENT::Verify() && m_pxKnowledgeSystem && m_pxKnowledgeSystem->Verify() && GetCivilianActor() );
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
