//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_FaceHugger.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Core/BrainTypes/AVP_Brain_Actor_FaceHugger.h"

// asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_LOS.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif


// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Attack.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_AttackPrey.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_HideInShadows.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Movement.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_MoveToPrey.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Pet.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_ReturnHome.h"
#include "Behaviour/BehaviourTypes/FaceHugger/AVP_Behaviour_FaceHugger_Wander.h"

// avp
#include "Actor/Vertebrates/AVP_Vertebrate_AIAnimStateMachine.h"
#include "Core/AVP_LOS.h"
#include "Core/BrainTypes/AVP_Brain_Team_Alien.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_FaceHugger.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ServerActor_FaceHugger.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_FaceHugger
//
//////////////////////////////////////////////////////////////////////////////

const float AVP_Brain_Actor_FaceHugger::s_fFrontalAngle = Asura_Maths::DegToRad(60.0f);
const float AVP_Brain_Actor_FaceHugger::s_fFrontalAngleCosine = Asura_Maths::Cosine(AVP_Brain_Actor_FaceHugger::s_fFrontalAngle);
const float AVP_Brain_Actor_FaceHugger::s_fFaceHuggerMinimumComfortDistance = 4.0f;


AVP_Brain_Actor_FaceHugger::AVP_Brain_Actor_FaceHugger( AVP_ServerActor_FaceHugger* pxFaceHugger )
: PARENT( pxFaceHugger )
, m_xSpawnPoint(GetActor()->GetPosition())
, m_uTargetLostCounter(0)
, m_slowUpdateTimeSlice(0.0f)
, m_uTargetGuid(ASURA_GUID_UNREGISTERED)
, m_pxTarget(NULL)
, m_xTargetIgnoreList()
, m_fTargetAquiredTime(-1.0f)
, m_bGotStartWaypointID(false)
, m_bIsFalling(false)
, m_bIsJumping(false)
, m_bJustFinishedFalling(false)
, m_fNotSupportedTime(0.0f)
, m_uMasterGuid(ASURA_GUID_UNREGISTERED)
, m_bIsPet(false)
, m_bPetAggressive(false)
, m_bPetAlwaysAggressive(false)
, m_bLookForDestroyedSquad(false)
, m_bIsLooseAvoidance(false)
, m_uCurrentGPV(ASURA_GUID_UNREGISTERED)
, m_uStartGPV(ASURA_GUID_UNREGISTERED)
, m_bTargetTargeted(false)
, m_bTargetAttacked(false)

{
	SetNearbyIgnoreDeadActors(false);
}

void AVP_Brain_Actor_FaceHugger::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	PARENT::InitialiseFromBlueprint( xBlueprint );

	Asura_Value xValue;

	DestroyAllBehaviours();

	if ( IsActive() )
	{
		bool bCreatedFromBP = false;
		if ( xBlueprint.GetParameterValueByHashID( 0xae7a48c3, xValue ) ) // Asura_GetHashID( "AIStartBehaviour" )
		{
			if ( xValue.GetIntValue() >= AXON_BEHAVIOUR_TYPE_FIRST_PROJECT_SPECIFIC )
			{
				CreateActivateAndPushBehaviour( xValue.GetIntValue() );
				bCreatedFromBP = true;
			}
		}

		if ( !bCreatedFromBP )
		{
			CreateActivateAndPushBehaviour( AVP_BEHAVIOUR_TYPE_FACEHUGGER );
		}
		CreateGS2BehaviourIfNecessary();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0x05fd9bb5, xValue ) ) //"IsPet"
	{
		m_bIsPet = xValue.GetBoolValue();
	}

	EstablishNavData();
}

DEBUGVAR  float g_uSlowUpdatePeriod = 0.1; // 100 milliseconds

void AVP_Brain_Actor_FaceHugger::DoPerFrameUpdate( const float fTimeslice )
{
	m_slowUpdateTimeSlice += fTimeslice;
	if (m_slowUpdateTimeSlice > g_uSlowUpdatePeriod)
	{
		m_slowUpdateTimeSlice = 0;
		SlowUpdate();
	}

	UpdateIgnoreList();

	u_short usLastWPID = GetNavData().m_usCurrentWPID;

	PARENT::DoPerFrameUpdate( fTimeslice );

	if (usLastWPID != GetNavData().m_usCurrentWPID || m_uCurrentGPV == ASURA_GUID_UNREGISTERED)
	{
		UpdateCurrentGPV();
		m_bIsLooseAvoidance = true;

		const AVP_ServerEntity_GamePlayVolume* pxGPV = AVP_ServerEntity_GamePlayVolume::FindEntity(m_uCurrentGPV);
		if (pxGPV)
		{
			switch (pxGPV->GetGamePlayAreaType())
			{
			case AVP_ServerEntity_GamePlayVolume::TIGHT_CORRIDOR:
			case AVP_ServerEntity_GamePlayVolume::SMALL_ROOM:
			case AVP_ServerEntity_GamePlayVolume::NONE_ASSIGNED:
				m_bIsLooseAvoidance = false;
				break;
			case AVP_ServerEntity_GamePlayVolume::LARGE_CORRIDOR:
			case AVP_ServerEntity_GamePlayVolume::LARGE_ROOM:
			case AVP_ServerEntity_GamePlayVolume::OPEN_SPACE:
				m_bIsLooseAvoidance = true;
				break;
			case AVP_ServerEntity_GamePlayVolume::IGNORE_GPV:
			default:
				Asura_Assertf(false, ("Game play volume %d has invalid type", m_uCurrentGPV));
			}
		}
		else
		{
			m_bIsLooseAvoidance = false;
			Asura_Assert(false, "Facehugger outside game play volume");
		}
	}
}

void AVP_Brain_Actor_FaceHugger::Shutdown()
{
	m_bLookForDestroyedSquad = true;
	MarkTargetNotAttacked(true);
	MarkTargetNotTargeted();
	PARENT::Shutdown();
}


void AVP_Brain_Actor_FaceHugger::SlowUpdate()
{

	// update how many updates the target did not see me
	if (!IsTargetVisible())
	{
		++m_uTargetLostCounter;
	}
	else
	{
		m_uTargetLostCounter = 0;
	}

	if (IsPet() && GetMaster() == NULL)
	{
		GetActor()->SetThisAndClientsAsDestroyed();		
	}

}

#include "Core/Asura_MemoryManagement_DebugOff.h"
Axon_Behaviour* AVP_Brain_Actor_FaceHugger::CreateBehaviour( const u_int uBehaviourType, const Asura_Message* pxMessage /*= NULL*/ )
{
    void* pxMemory = s_pxBehaviourMempool->Allocate();
    if ( !pxMemory )
    {
        Asura_Assert( false, "AVP_Brain_Actor_FaceHugger::CreateBehaviour - could not allocate behaviour memory" );
        return NULL;
    }

    Axon_Behaviour* pxBehaviour = NULL;
    switch ( uBehaviourType )
    {
        case AVP_BEHAVIOUR_TYPE_FACEHUGGER:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_FaceHugger( this );
				break;
			}
		case AVP_BEHAVIOUR_TYPE_FACEHUGGER_WANDER:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_FaceHugger_Wander( this );
				break;
			}
		case AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACK:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_FaceHugger_Attack( this );
				break;
			}
		case AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVEMENT:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_FaceHugger_Movement( this );
				break;
			}
		case AVP_BEHAVIOUR_TYPE_FACEHUGGER_MOVETOPREY:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_FaceHugger_MoveToPrey( this );
				break;
			}
		case AVP_BEHAVIOUR_TYPE_FACEHUGGER_ATTACKPREY:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_FaceHugger_AttackPrey( this );
				break;
			}
		case AVP_BEHAVIOUR_TYPE_FACEHUGGER_RETURN_HOME:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_FaceHugger_ReturnHome( this );
				break;
			}
		case AVP_BEHAVIOUR_TYPE_FACEHUGGER_PET:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_FaceHugger_Pet( this );
				break;
			}
		case AVP_BEHAVIOUR_TYPE_FACEHUGGER_GS2_MOVEMENT:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_FaceHugger_GS2_Movement( this );
				break;
			}
		case AVP_BEHAVIOUR_TYPE_FACEHUGGER_HIDE_IN_SHADOWS:
			{
				pxBehaviour = new ( pxMemory ) AVP_Behaviour_FaceHugger_HideInShadows( this );
				break;
			}			
		default:
			{
				break;
			}
    }

    if ( !pxBehaviour )
    {
        // Couldn't create the behaviour - try the parent.
        s_pxBehaviourMempool->Deallocate( pxMemory );
        pxBehaviour = PARENT::CreateBehaviour( uBehaviourType, pxMessage );
    }
    else if ( pxMessage )
    {
        pxBehaviour->DoHandleMessage( pxMessage, true );
    }

    return pxBehaviour;
}
#include "Core/Asura_MemoryManagement_DebugOn.h"

void AVP_Brain_Actor_FaceHugger::EstablishNavData()
{
	if (m_bGotStartWaypointID) // already done this once successfully.
	{
		return;
	}

	AVP_Navigation::NavData& xNavData = GetNavData();

	xNavData.m_uPathfinderFlags = AVP_Navigation::AVP_AI_PATHFINDER_FLAG_ALIEN_HIGH_MOBILITY | ASURA_AI_PATHFINDER_FLAGS_IGNOREZONES;
	SetMovementMode(AVP_Navigation::MM_MEDIUM);
	xNavData.m_fCharacterHeight = 0.5f;
	xNavData.m_fCharacterWidth  = 0.5f;
	xNavData.m_usCurrentZone = ASURA_NAVVERTEX_MAX_ZONE_ID;
 
	AVP_Navigation::TestParams xTestParams;
	xTestParams.m_uPathfinderFlags		= xNavData.m_uPathfinderFlags;
	xTestParams.m_bRejectNonalignedWPs	= true;
	xTestParams.m_xNormal				= -GetActor()->GetCurrentGravityDirection();
	
	m_bGotStartWaypointID = AVP_Navigation::GetNearestWaypointID(GetActor()->GetPosition(), xNavData.m_usCurrentWPID, xTestParams);
}

void AVP_Brain_Actor_FaceHugger::PostLoadInitialise()
{
	RequestRegisterWithTeam();
	EstablishNavData();
	Asura_Assert(m_bGotStartWaypointID, "Initial Waypoint not found"); // now it must have worked!
	PARENT::PostLoadInitialise();
}

bool AVP_Brain_Actor_FaceHugger::RegisterWithTeam()
{
	if ( !PARENT::RegisterWithTeam() )
	{
		return false;
	}

	AVP_ServerEntity_Squad_FaceHugger* pxSquad = GetSquad();
	if ( pxSquad )
	{
		pxSquad->AddMember( GetOwnerGuid() );
		return true;
	}
	else
	{
		return false;
	}

}

bool AVP_Brain_Actor_FaceHugger::UnregisterWithTeam()
{
	if ( !PARENT::UnregisterWithTeam() )
	{
		return false;
	}

	AVP_ServerEntity_Squad_FaceHugger* pxSquad = GetSquad();
	if ( pxSquad )
	{
		pxSquad->RemoveMember( GetOwnerGuid() );
		return true;
	}
	else
	{
		return false;
	}

}


const Asura_Vector_3& AVP_Brain_Actor_FaceHugger::GetSpawnPoint() const
{
	Asura_Assert(m_bGotStartWaypointID, "Initial Waypoint not found"); // now it must be valid!
	return m_xSpawnPoint;
}

void AVP_Brain_Actor_FaceHugger::SetMovementMode(AVP_Navigation::MovementMode uMode)
{
	GetNavData().m_eMovementMode = uMode;
}

void AVP_Brain_Actor_FaceHugger::SetFalling( bool bIsFalling)
{
	Asura_Assert(m_bIsFalling != bIsFalling, "Setting Fall to existing value - mismatch");
	m_bIsFalling = bIsFalling;
}

void AVP_Brain_Actor_FaceHugger::SetJumping( bool bIsJumping )
{
	Asura_Assert(m_bIsJumping != bIsJumping, "Setting Jump to existing value - mismatch");
	m_bIsJumping = bIsJumping;
}


bool AVP_Brain_Actor_FaceHugger::FarFromSpawn() const
{
	return (GetActor()->GetPosition() - GetSpawnPoint()).MagnitudeSquared() > GetBehaviourData().m_fMaxDistanceFromSpawn * GetBehaviourData().m_fMaxDistanceFromSpawn;
}


bool AVP_Brain_Actor_FaceHugger::ShouldAttack() const
{
	if (IsTargetUnderAttack())
	{
		return false;
	}

	// if target stayed too close for too long
	if (IsTooCloseToTarget())
	{
		return false;
	}

	// else attack-ability depends on the orientation of the target to me
	if (IsTargetFacingMe())
	{
		return ShouldAttackFromTheFront();
	}
	else
	{
		return ShouldAttackFromTheBack();
	}
}



#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_Brain_Actor_FaceHugger::DoRenderDebugInfo()
{
	PARENT::DoRenderDebugInfo();

	if (m_uTargetGuid != ASURA_GUID_UNREGISTERED)
	{
		Asura_Font_System::Print( GetDebugOutputPos().x, GetDebugOutputPos().y, 0.5f, 0xffffffff,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING,
			"FaceHugger Target: %d", m_uTargetGuid);
	}
	else
	{
		Asura_Font_System::Print( GetDebugOutputPos().x, GetDebugOutputPos().y, 0.5f, 0xffffffff,
			ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING,			
			"FaceHugger NO TARGET");
	}
	GetDebugOutputPos().x += 10.0f;
	GetDebugOutputPos().y += 10.0f;

}
#endif


Asura_Guid AVP_Brain_Actor_FaceHugger::GetPrimaryThreatGuid()
{
	return ASURA_GUID_UNREGISTERED;
}

Asura_Guid AVP_Brain_Actor_FaceHugger::GetAlertGuid()
{
	return ASURA_GUID_UNREGISTERED;
}

bool AVP_Brain_Actor_FaceHugger::FindTarget(bool bIgnorePGVolumes)
{
	// GBT :: for when Face Huggers are being tested in a template

	Asura_Assert(!IsPet(), "wrong FindTarget - in pet mode");

	if ( !AVP_ServerEntity_Team_Alien::GetInstance() )
	{
		Asura_Assert( false, "AVP_Behaviour_FaceHugger :: Update - No Alien Team Instance... Is this a Template? You should really be testing in a level.");
		MarkTargetNotTargeted();
		m_uTargetGuid = ASURA_GUID_UNREGISTERED;
		return false;
	}

	if (m_uTargetGuid != ASURA_GUID_UNREGISTERED)
	{
		MarkTargetNotTargeted();
		m_uTargetGuid = ASURA_GUID_UNREGISTERED;
	}

	Asura_Vector_3 xCurrentPosition = GetActor()->GetPosition();

	bool bFound = false;
	int iBestPriority = AVP_AI_TargetPriorityData::IGNORE_P;
	float fMinDistanceSquared = Asura_Maths::LargeFloat;

	for ( Asura_Hashed_List_ConstIt< AVP_AlienTeam_Knowledge > xKnowIter( &AVP_ServerEntity_Team_Alien::GetInstance()->GetAlienTeamBrain()->GetKnowledgeSystem()->GetAllKnowledge() );
		!xKnowIter.Done();
		xKnowIter.Next())
	{
		const AVP_AlienTeam_Knowledge* pxKnowledge = xKnowIter.GetCurrent();
		if (!pxKnowledge || pxKnowledge->IsDestroyed())
		{
			continue;
		}

		if (m_xTargetIgnoreList.FindByGuid(pxKnowledge->GetGuid()) )
		{
			continue;
		}

		switch (pxKnowledge->m_iThreatClassification)
		{
		case AVP_ENTITYCLASS_ANDROID:
		case AVP_ENTITYCLASS_MARINE:
		case AVP_ENTITYCLASS_PREDATOR:
		case AVP_ENTITYCLASS_PREDALIEN:
		case AVP_ENTITYCLASS_CIVILIAN:
		case AVP_ENTITYCLASS_HUMAN:
			{
				float fMagnitudeSquared = (pxKnowledge->GetPosition() - xCurrentPosition).MagnitudeSquared();
				int iPriority;

				if(!GetSquad()->CalcTargetingPriority( pxKnowledge->m_iThreatClassification, pxKnowledge->GetGuid(), iPriority ))
				{
					iPriority = AVP_AI_TargetPriorityData::NORMAL;
				}
				
				if ( iPriority != AVP_AI_TargetPriorityData::IGNORE_P &&
					 pxKnowledge->m_uNumTargeters < GetSquad()->GetBehaviourData().m_uMaxTargeters &&
					 !IsIgnored(pxKnowledge->GetGuid()) && 
					 ( (!bIgnorePGVolumes && IsTargetInPGVolumes(pxKnowledge->GetGuid()) ) ||
					   (bIgnorePGVolumes ) ) && 
					 ( iPriority > iBestPriority ||
					   ( iPriority == iBestPriority && fMagnitudeSquared < fMinDistanceSquared ) )
				   )
				{
					bFound = true;
					iBestPriority = iPriority;
					fMinDistanceSquared = fMagnitudeSquared;
					m_uTargetGuid = pxKnowledge->GetGuid();
				}
			}
			break;
		default:
			break;
		}
	}

	if(bFound)
	{
		m_fTargetAquiredTime = Asura_Timers::GetLocalScaledTime();
		Asura_Assert(!m_bTargetTargeted, "Targeting new target without removing from previous target");
		AVP_ServerEntity_Team_Alien::GetInstance()->GetAlienTeamBrain()->GetKnowledgeSystem()->AddTargeter(GetOwnerGuid(), m_uTargetGuid);
		m_bTargetTargeted = true;
		CacheTarget();
		return true;
	}
	else
	{
		m_fTargetAquiredTime = -1.0f;
		m_uTargetGuid = ASURA_GUID_UNREGISTERED;
		CacheTarget();
		return false;
	}
}

DEBUGVAR float dbg_fMaxTargetDistanceFromMaster = 50.0f * 50.0f;

bool AVP_Brain_Actor_FaceHugger::FindTargetPetMode()
{
	// GBT :: for when Face Huggers are being tested in a template

	Asura_Assert(IsPet(), "wrong FindTargetPetMode - not in pet mode");

	if ( !AVP_ServerEntity_Team_Alien::GetInstance() )
	{
		Asura_Assert( false, "AVP_Behaviour_FaceHugger :: Update - No Alien Team Instance... Is this a Template? You should really be testing in a level.");
		MarkTargetNotTargeted();
		m_uTargetGuid = ASURA_GUID_UNREGISTERED;
		return false;
	}

	if (m_uTargetGuid != ASURA_GUID_UNREGISTERED)
	{
		MarkTargetNotTargeted();
		m_uTargetGuid = ASURA_GUID_UNREGISTERED;
	}

	Asura_Vector_3 xCurrentPosition = GetMaster()->GetPosition();

	bool bFound = false;
	int iBestPriority = AVP_AI_TargetPriorityData::IGNORE_P;
	float fMinDistanceSquared = Asura_Maths::LargeFloat;

	for ( Asura_Hashed_List_ConstIt< AVP_AlienTeam_Knowledge > xKnowIter( &AVP_ServerEntity_Team_Alien::GetInstance()->GetAlienTeamBrain()->GetKnowledgeSystem()->GetAllKnowledge() );
		!xKnowIter.Done();
		xKnowIter.Next())
	{
		const AVP_AlienTeam_Knowledge* pxKnowledge = xKnowIter.GetCurrent();
		if (!pxKnowledge || pxKnowledge->IsDestroyed())
		{
			continue;
		}

		if (m_xTargetIgnoreList.FindByGuid(pxKnowledge->GetGuid()) )
		{
			continue;
		}

		switch (pxKnowledge->m_iThreatClassification)
		{
		case AVP_ENTITYCLASS_ANDROID:
		case AVP_ENTITYCLASS_MARINE:
		case AVP_ENTITYCLASS_PREDATOR:
		case AVP_ENTITYCLASS_PREDALIEN:
		case AVP_ENTITYCLASS_CIVILIAN:
		case AVP_ENTITYCLASS_HUMAN:
			{
				float fMagnitudeSquared = (pxKnowledge->GetPosition() - xCurrentPosition).MagnitudeSquared();
				int iPriority;

				if(!GetSquad()->CalcTargetingPriority( pxKnowledge->m_iThreatClassification, pxKnowledge->GetGuid(), iPriority ))
				{
					iPriority = AVP_AI_TargetPriorityData::NORMAL;
				}

				if ( iPriority != AVP_AI_TargetPriorityData::IGNORE_P &&
					pxKnowledge->m_uNumTargeters < GetSquad()->GetBehaviourData().m_uMaxTargeters &&
					!IsIgnored(pxKnowledge->GetGuid()) && 
					fMagnitudeSquared < dbg_fMaxTargetDistanceFromMaster &&
					( iPriority > iBestPriority ||
					( iPriority == iBestPriority && fMagnitudeSquared < fMinDistanceSquared ) )
					)
				{
					bFound = true;
					iBestPriority = iPriority;
					fMinDistanceSquared = fMagnitudeSquared;
					m_uTargetGuid = pxKnowledge->GetGuid();
				}
			}
			break;
		default:
			break;
		}
	}

	if(bFound)
	{
		m_fTargetAquiredTime = Asura_Timers::GetLocalScaledTime();
		Asura_Assert(!m_bTargetTargeted, "Targeting new target without removing from previous target");
		AVP_ServerEntity_Team_Alien::GetInstance()->GetAlienTeamBrain()->GetKnowledgeSystem()->AddTargeter(GetOwnerGuid(), m_uTargetGuid);
		m_bTargetTargeted = true;
		CacheTarget();
		return true;
	}
	else
	{
		m_fTargetAquiredTime = -1.0f;
		m_uTargetGuid = ASURA_GUID_UNREGISTERED;
		CacheTarget();
		return false;
	}
}


bool AVP_Brain_Actor_FaceHugger::IsTargetValid() const
{
	return m_pxTarget != NULL && !m_pxTarget->IsDestroyed();
}


void AVP_Brain_Actor_FaceHugger::IgnoreCurrentTarget(float fTime)
{
	if (m_uTargetGuid != ASURA_GUID_UNREGISTERED)
	{
		Asura_Assert(!IsIgnored(m_uTargetGuid), "Target Already Ignored");
		TargetIgnoreData xIgnoreData;
		xIgnoreData.m_uGuidToIgnore = m_uTargetGuid;
		xIgnoreData.m_fRemoveTime = Asura_Timers::GetLocalScaledTime() + fTime;
		m_xTargetIgnoreList.Push_Back(xIgnoreData);
		MarkTargetNotTargeted();
		MarkTargetNotAttacked(true);
		m_uTargetGuid = ASURA_GUID_UNREGISTERED;
	}
}

AVP_ServerEntity_Squad_FaceHugger* AVP_Brain_Actor_FaceHugger::GetSquad() const
{
	AVP_ServerEntity_Squad_FaceHugger* pxSquad = NULL;

	Asura_Guid uGuid = GetActor()->GetInitialSquadGuid();

	if (IsPet())
	{
		Asura_Assert(uGuid == ASURA_GUID_UNREGISTERED , "Pet FaceHugger belong only to the DefaultPetSquad");
		pxSquad = AVP_ServerEntity_Squad_FaceHugger::GetDefaultPetSquad();
		Asura_Assert(pxSquad , "Default Pet squad not found");
	}
	else
	{

		if ( uGuid == ASURA_GUID_UNREGISTERED )
		{
			pxSquad = AVP_ServerEntity_Squad_FaceHugger::GetDefaultSquad();
			Asura_Assert(pxSquad , "Default squad not found");
		}
		else
		{
			pxSquad = AVP_ServerEntity_Squad_FaceHugger::FindEntity( uGuid, !m_bLookForDestroyedSquad);
			Asura_Assertf(pxSquad, ("Squad %d not found",uGuid) );
			if (!pxSquad)
			{
				pxSquad = AVP_ServerEntity_Squad_FaceHugger::GetDefaultSquad();
				Asura_Assert(pxSquad , "Default squad not found");		
			}
		}
	}

	return pxSquad;
}

bool AVP_Brain_Actor_FaceHugger::IsTargetInPGVolumes( Asura_Guid uGuid ) const
{
	AVP_ServerEntity_PhysicalObject* pxEnity = AVP_ServerEntity_PhysicalObject::FindEntity(uGuid);

	if (!pxEnity || pxEnity->IsDead())
	{
		return false;
	}

	bool bSqadHasGPV = false;

	AVP_ServerEntity_Squad_FaceHugger* pxSquad = GetSquad();
	for (Asura_LinkList_ConstIt<Asura_Guid> xIt(pxSquad->GetGPVolumeIDIter()); !xIt.Done(); xIt.Next())
	{
		bSqadHasGPV = true;
		AVP_ServerEntity_GamePlayVolume* pxVolume = AVP_ServerEntity_GamePlayVolume::FindEntity(*xIt.GetCurrent());
		Asura_Assert(pxVolume, "Facehugger squad Gameplay Volume not found" );
		if (pxVolume)
		{
			if (pxVolume->Contains(pxEnity->GetPosition()))
			{
				return true;
			}
		}
	}

	// for squads that have no GPV assigned to them (typical when the facehugger inside an egg and assigned to the default squd)
	// check the startup GPV
	if (!bSqadHasGPV && m_uStartGPV != ASURA_GUID_UNREGISTERED )
	{
		AVP_ServerEntity_GamePlayVolume* pxVolume = AVP_ServerEntity_GamePlayVolume::FindEntity(m_uStartGPV);
		Asura_Assert(pxVolume, "Facehugger squad Gameplay Volume not found" );
		if (pxVolume)
		{
			if (pxVolume->Contains(pxEnity->GetPosition()))
			{
				return true;
			}
		}
	}

	return false;
}


void AVP_Brain_Actor_FaceHugger::UpdateIgnoreList()
{
	// The ignore list is ordered by remove time - the oldest ones are at the front.
	// Keep popping the front while it too old.
	float fNow = Asura_Timers::GetLocalScaledTime();
	while (m_xTargetIgnoreList.Size() != 0 )
	{
		if(m_xTargetIgnoreList.Front()->Data().m_fRemoveTime < fNow)
		{
					m_xTargetIgnoreList.Remove(m_xTargetIgnoreList.Front());
		}
		else
		{
			return;
		}
	}
}

bool AVP_Brain_Actor_FaceHugger::IsIgnored(Asura_Guid uTargetGuid) const
{
	return ( m_xTargetIgnoreList.FindByGuid( uTargetGuid ) != NULL );
}



DEBUGVAR float g_fJumpDistanceSquared = 4.0f*4.0f;

bool AVP_Brain_Actor_FaceHugger::IsTargetInJumpRange() const
{
	if (!m_pxTarget)
	{
		return false;
	}

	return (m_pxTarget->GetPosition() - GetActor()->GetPosition()).MagnitudeSquared() < g_fJumpDistanceSquared;
}

bool AVP_Brain_Actor_FaceHugger::IsTargetUnderAttack() const
{
	if (!m_pxTarget || m_pxTarget->m_uNumAttackers > 0)
	{
		return true;
	}

	return false;
}

void AVP_Brain_Actor_FaceHugger::UpdateBehaviours( const float fTimeslice )
{
	CacheTarget();
	PARENT::UpdateBehaviours( fTimeslice );
}

void AVP_Brain_Actor_FaceHugger::UpdateCurrentGPV()
{
	const AVP_ServerEntity_GamePlayVolume* pxGPV = AVP_ServerEntity_GamePlayVolume::GetVolumeThatContainsPosition( GetActor()->GetPosition(), m_uCurrentGPV );
	m_uCurrentGPV = pxGPV ? pxGPV->GetGuid() : ASURA_GUID_UNREGISTERED;
	if (m_uStartGPV == ASURA_GUID_UNREGISTERED)
	{
		m_uStartGPV = m_uCurrentGPV;
	}
}


DEBUGVAR float dbg_fSlowRange = 6.0f;
DEBUGVAR float dbg_fMediumRange = 10.0f;
DEBUGVAR float dbg_fFastRange = 14.0f;

float AVP_Brain_Actor_FaceHugger::GetNearbyRange() const
{
	switch (GetActor()->GetAIAnimStateMachine()->GetTargetState())
	{
	case AVP_Vertebrate_AIAnimStateMachine::ACTOR_STATE_IDLE:
		return 0.0f;
	case AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_SLOW:
		return dbg_fSlowRange;
	default:
	case AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_MEDIUM:
		return dbg_fMediumRange;
	case AVP_Vertebrate_AIAnimStateMachine::VERTEBRATE_STATE_MOVE_FAST:
		return dbg_fFastRange;
	}
}

void AVP_Brain_Actor_FaceHugger::CacheTarget()
{
	m_pxTarget = NULL;

	if (m_uTargetGuid == ASURA_GUID_UNREGISTERED)
	{
		return;
	}

	AVP_AlienTeam_Knowledge* pxKnowledge = AVP_ServerEntity_Team_Alien::GetInstance()->GetAlienTeamBrain()->GetKnowledgeSystem()->GetAllKnowledge().FindEntity(m_uTargetGuid, true);

	if (!pxKnowledge)
	{
		return;
	}

	// check that target is still in our zone
	AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity(pxKnowledge->GetGuid());
	if (!pxActor || pxActor->IsDead())
	{
		return;
	}

	Asura_Vector_3 xTargetPosition = pxActor->GetPosition();

	u_short usTargetWP;
	bool bTargetHaveValidWP = AVP_Navigation::GetNearestWaypointID(xTargetPosition, usTargetWP);

	if (!bTargetHaveValidWP)
	{
		return;
	}

	m_pxTarget  = pxKnowledge;
}

bool AVP_Brain_Actor_FaceHugger::IsTargetInPGVolumes() const
{
	if (!m_pxTarget)
	{
		return false;
	}
	return IsTargetInPGVolumes(m_pxTarget->GetGuid());
}


bool AVP_Brain_Actor_FaceHugger::IsTargetFacingMe() const
{
	if (!m_pxTarget)
	{
		return false;
	}

	AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_pxTarget->GetGuid());
	AVP_ServerEntity_Actor* pxMe = GetActor();

	Asura_Assert(pxMe, "My Physical object Not found");
	if (!pxTarget || pxTarget->IsDead() || !pxMe )
	{
		return false;
	}

	Asura_Vector_3 xAim = pxTarget->GetAimDirection();
	Asura_Vector_3 xDirection = pxMe->GetPosition() - pxTarget->GetPosition();
	xDirection.Normalise();

	return xAim * xDirection > s_fFrontalAngleCosine;

}


bool AVP_Brain_Actor_FaceHugger::IsTargetVisible() const
{
	if (!m_pxTarget)
	{
		return false;
	}
	AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_pxTarget->GetGuid());
	AVP_ServerEntity_Actor* pxMe = GetActor();

	Asura_Assert(pxMe, "My Physical object Not found");
	if (!pxTarget || pxTarget->IsDead() || !pxMe )
	{
		return false;
	}

	Asura_Vector_3 xTargetPosition; 
	Asura_Vector_3 xMyPosition; 
	pxTarget->GetHeadPosition(xTargetPosition);
	pxMe->GetHeadPosition(xMyPosition);

	Asura_Vector_3 xTargetDir = (xTargetPosition - xMyPosition); 
	float fMagnitute = xTargetDir.Normalise();

	// calculate the LOS
	Asura_LOS_InputData xLosInput(xMyPosition , xTargetDir, GetActor()->GetGuid(), fMagnitute);
	const Asura_LOS_OutputData* pxLosOutput = AVP_LOS::TestAgainstEnvironmentAndMoveableObjects(&xLosInput);

	if (pxLosOutput->m_fDistanceToIntersection < fMagnitute - 0.1f )
	{
		return false;
	}

	return true;
}

bool AVP_Brain_Actor_FaceHugger::ShouldAttackFromTheFront() const
{
	if (!m_pxTarget)
	{
		return false;
	}

	if ((Asura_Timers::GetLocalScaledTime() - m_pxTarget->m_fLastAttackFinishTime) <= GetSquad()->GetBehaviourData().m_fTimeBetweenAttacks)
	{
		return false;
	}

	return true;
}

//DEBUGVAR float g_fAttackFromBackDelay = 60.0f;

bool AVP_Brain_Actor_FaceHugger::ShouldAttackFromTheBack() const
{
	// never attack from behind.
	return false;

// 	if (!m_pxTarget)
// 	{
// 		return false;
// 	}
// 
// 	if (m_pxTarget->m_uNumAttackers > 0)
// 	{
// 		return false;
// 	}
// 
// 	Asura_Assert(m_fTargetAquiredTime > 0.0f, "Target Aquire time not set");
// 
// 	float fNow = Asura_Timers::GetLocalScaledTime();
// 
// 	if ( (fNow - m_pxTarget->m_fLastAttackFinishTime) <= GetSquad()->GetBehaviourData().m_fTimeBetweenAttacks + g_fAttackFromBackDelay ||
// 		 (fNow - m_fTargetAquiredTime) <= GetSquad()->GetBehaviourData().m_fTimeBetweenAttacks + g_fAttackFromBackDelay)
// 	{
// 		return false;
// 	}
// 
// 	return true;
}

void AVP_Brain_Actor_FaceHugger::AttachToTarget() const
{
	if (m_uTargetGuid == ASURA_GUID_UNREGISTERED || !AVP_ServerActor_Vertebrate::FindEntity(m_uTargetGuid))
	{
		return;
	}

	AVP_ServerEntity_Actor* pxActor = GetActor();
	pxActor->LinkToParent(m_uTargetGuid, 0x002fbc62 /* eyes */, false );
}

bool AVP_Brain_Actor_FaceHugger::IsTooCloseToTarget() const
{
	if (!m_pxTarget)
	{
		return true;
	}

	return (m_pxTarget->GetPosition() - GetActor()->GetPosition()).MagnitudeSquared() < s_fFaceHuggerMinimumComfortDistance*s_fFaceHuggerMinimumComfortDistance;

}

void AVP_Brain_Actor_FaceHugger::MarkTargetAttacked()
{
	if (m_uTargetGuid == ASURA_GUID_UNREGISTERED)
	{
		return ;
	}
	Asura_Assert(!m_bTargetAttacked, "Attacking new target without removing from previous target");
	AVP_ServerEntity_Team_Alien::GetInstance()->GetAlienTeamBrain()->GetKnowledgeSystem()->AddAttacker(GetOwnerGuid(), m_uTargetGuid);
	m_bTargetAttacked = true;
}

void AVP_Brain_Actor_FaceHugger::MarkTargetNotAttacked(bool bAttackAborted)
{
	if (m_uTargetGuid == ASURA_GUID_UNREGISTERED)
	{
		return ;
	}
	if (m_bTargetAttacked)
	{
		AVP_ServerEntity_Team_Alien::GetInstance()->GetAlienTeamBrain()->GetKnowledgeSystem()->RemoveAttacker(GetOwnerGuid(), m_uTargetGuid, bAttackAborted);
		m_bTargetAttacked = false;
	}
}

void AVP_Brain_Actor_FaceHugger::MarkTargetNotTargeted()
{
	if (m_uTargetGuid == ASURA_GUID_UNREGISTERED)
	{
		return ;
	}
	if (m_bTargetTargeted)
	{
		AVP_ServerEntity_Team_Alien::GetInstance()->GetAlienTeamBrain()->GetKnowledgeSystem()->RemoveTargeter(GetOwnerGuid(), m_uTargetGuid);
		m_bTargetTargeted = false;
	}
	m_uTargetGuid = ASURA_GUID_UNREGISTERED;
}


DEBUGVAR float dbg_fPreFallTime = 0.2f;
void AVP_Brain_Actor_FaceHugger::UpdateFalling(float fTimeslice )
{
	m_bJustFinishedFalling = false;

	if (IsJumping())
	{
		m_bIsFalling = false;
		m_bJustFinishedFalling = false;
	}

	bool bIsSupported = GetActor()->GetPhysicsObject()->IsSupported();

	if (!bIsSupported)
	{
		m_fNotSupportedTime += fTimeslice;
		if (m_fNotSupportedTime > dbg_fPreFallTime)
		{
			m_bIsFalling = true;
		}
	}
	else
	{
		m_fNotSupportedTime = 0.0f;
		if (m_bIsFalling)
		{
			m_bIsFalling = false;
			m_bJustFinishedFalling = true;
		}
	}
}

void AVP_Brain_Actor_FaceHugger::ResetSpawnPoint()
{
	m_xSpawnPoint = GetActor()->GetPosition();
}

DEBUGVAR  u_int g_uMaxTargetDidntSeeMeCounter = 3;
bool AVP_Brain_Actor_FaceHugger::WasTargetLost() const
{
	return m_uTargetLostCounter > g_uMaxTargetDidntSeeMeCounter;
}

void AVP_Brain_Actor_FaceHugger::SetMaster(Asura_Guid uMasterGuid)
{
	Asura_Assert(uMasterGuid != ASURA_GUID_UNREGISTERED, "Invalid Master");
	Asura_Assert(m_bIsPet, "Setting master a non pet Facehugger");
	Asura_Assert(m_uMasterGuid == ASURA_GUID_UNREGISTERED, "FaceHugger already has a master");

	m_uMasterGuid = uMasterGuid;
}


void AVP_Brain_Actor_FaceHugger::ChangeAttackMode()
{
	Asura_Assert(IsPet(), "Can only change attack mode to pet Facehugger");
	if (IsPet())
	{
		m_bPetAggressive = !m_bPetAggressive;
	}
}

void AVP_Brain_Actor_FaceHugger::SetAlwaysAggressive(bool bAlwaysAggressive)
{
	Asura_Assert(IsPet(), "Can only change always aggressive mode to pet Facehugger");
	if (IsPet())
	{
		m_bPetAlwaysAggressive = bAlwaysAggressive;
	}
}

DEBUGVAR float dbg_fMasterTanFovX = Asura_Maths::Tangent(Asura_Maths::DegToRad(45.0f));
DEBUGVAR float dbg_fMasterTanFovY = Asura_Maths::Tangent(Asura_Maths::DegToRad(30.0f));
DEBUGVAR float dbg_fMinMasterDistanceSq = 3.0f * 3.0f;
DEBUGVAR float dbg_fMaxMasterDistanceSq = 15.0f * 15.0f;

bool AVP_Brain_Actor_FaceHugger::IsInMasterFoV() const
{
	AVP_ServerEntity_Actor* pxMaster = GetMaster();
	if (!pxMaster)
	{
		return false;
	}

	// build aim orientation
	Asura_Matrix_3x3 xMasterOrientation;
	Asura_Vector_3 xOriZ = pxMaster->GetAimDirection();
	Asura_Vector_3 xDown = Asura_Vector_3(pxMaster->GetCurrentGravityDirection());
	Asura_Vector_3 xOriX = xDown ^ pxMaster->GetAimDirection();
	Asura_Vector_3 xOriY = pxMaster->GetAimDirection() ^ xOriX;

	xMasterOrientation.e11 = xOriX.x;
	xMasterOrientation.e21 = xOriX.y;
	xMasterOrientation.e31 = xOriX.z;
	xMasterOrientation.e12 = xOriY.x;
	xMasterOrientation.e22 = xOriY.y;
	xMasterOrientation.e32 = xOriY.z;
	xMasterOrientation.e13 = xOriZ.x;
	xMasterOrientation.e23 = xOriZ.y;
	xMasterOrientation.e33 = xOriZ.z;
	
	Asura_Vector_3 xMasterPosition = pxMaster->GetPosition();
	Asura_Vector_3 xMyPosition = GetActor()->GetPosition();
	Asura_Vector_3 xRelativePosition = GetActor()->GetPosition() - pxMaster->GetPosition();

	if (xRelativePosition.MagnitudeSquared() > dbg_fMaxMasterDistanceSq )
	{
		return false;
	}

	xRelativePosition = xMasterOrientation.ApplyTranspose(xRelativePosition);

	if ( xRelativePosition.z < 0.0f )
	{
		return false;
	}

	// left and right planes
	const float fZX = xRelativePosition.z * dbg_fMasterTanFovX;
	if ( ( xRelativePosition.x - fZX ) > 0.0f || ( -xRelativePosition.x - fZX ) > 0.0f )
	{
		return false;
	}

	// top and bottom
	const float fZY = xRelativePosition.z * dbg_fMasterTanFovY;
	if ( ( xRelativePosition.x - fZY ) > 0.0f || ( -xRelativePosition.x - fZY ) > 0.0f )
	{
		return false;
	}

	return true;
}

bool AVP_Brain_Actor_FaceHugger::IsTooFarOrToCloseToMaster() const
{
	AVP_ServerEntity_Actor* pxMaster = GetMaster();
	if (!pxMaster)
	{
		return false;
	}

	float xDistanceSq = (GetActor()->GetPosition() - pxMaster->GetPosition()).MagnitudeSquared();

	return (xDistanceSq < dbg_fMinMasterDistanceSq || xDistanceSq > dbg_fMaxMasterDistanceSq);
}


AVP_ServerEntity_Actor* AVP_Brain_Actor_FaceHugger::GetMaster() const
{
	Asura_Assert(m_uMasterGuid != ASURA_GUID_UNREGISTERED, "FaceHugger has no master");

	if (m_uMasterGuid != ASURA_GUID_UNREGISTERED)
	{
		return static_cast<AVP_ServerEntity_Actor*>(AVP_ServerEntity_Actor::FindEntity(m_uMasterGuid));
	}
	else
	{
		return NULL;
	}
}

const AVP_FaceHuggerBehaviourData& AVP_Brain_Actor_FaceHugger::GetBehaviourData() const
{
	return GetSquad()->GetBehaviourData();
}



u_int AVP_Brain_Actor_FaceHugger::GetLargestBehaviourSize()
{
	u_int uLargestBehaviourSize = 0;

	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_FaceHugger );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_FaceHugger_Wander );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_FaceHugger_Attack );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_FaceHugger_Movement );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_FaceHugger_MoveToPrey );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_FaceHugger_AttackPrey );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_FaceHugger_ReturnHome );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_FaceHugger_Pet );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_FaceHugger_GS2_Movement );
	AXON_CALC_MAX_BEHAVIOUR_SIZE( uLargestBehaviourSize, AVP_Behaviour_FaceHugger_HideInShadows	);

	return uLargestBehaviourSize;
}




/* not used currently
bool AVP_Brain_Actor_FaceHugger::CanTargetSeeMe() const
{
	AVP_AlienTeam_Knowledge* pxKnowledge(GetTarget());
	Asura_Assert(pxKnowledge, "Target is Not Valid");
	if (!pxKnowledge)
	{
		return false;
	}
	AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(pxKnowledge->GetGuid());
	AVP_ServerEntity_Actor* pxMe = GetActor();

	Asura_Assert(pxMe, "My Physical object Not found");
	if (!pxTarget || pxTarget->IsDead() || !pxMe )
	{
		return false;
	}

	// For now non player will simply say they cannot see the face hugger for simpler and faster calculation
	if (!pxTarget->IsPlayer())
	{
		return false;
	}

	// simplistic FoV test - check if the fh is inside 45 deg of the player look con
	Asura_Vector_3 xAimDir = pxTarget->GetAimDirection();
	Asura_Vector_3 xTargetPosition; 
	Asura_Vector_3 xMyPosition; 
	pxTarget->GetHeadPosition(xTargetPosition);
	pxMe->GetHeadPosition(xMyPosition);

	Asura_Vector_3 xMyDir = (xMyPosition - xTargetPosition); 
	float fMagnitute = xMyDir.Normalise();
	if (xAimDir * xMyDir < Asura_Maths::Sqrt(2.0f)/2.0f )
	{
		return false;
	}

	// target is in FoV so calculate the LOS
	Asura_LOS_InputData xLosInput(xTargetPosition , xMyDir, pxTarget->GetGuid(), 1.0f + fMagnitute);
	const Asura_LOS_OutputData* pxLosOutput = Asura_LOS::TestAgainstEverything(&xLosInput);

	if (pxLosOutput->m_uHitEntity != GetActor()->GetGuid())
	{
		return false;
	}

	return true;
}
*/


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
