//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_FaceHugger.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BRAIN_ACTOR_FACEHUGGER_H__
#define __AVP_BRAIN_ACTOR_FACEHUGGER_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// axon
#include "Core/BrainTypes/AVP_Brain_Actor.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_ServerActor_FaceHugger;
class AVP_ServerEntity_Squad_FaceHugger;
struct AVP_AlienTeam_Knowledge;
struct AVP_FaceHuggerBehaviourData;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_FaceHugger
//
//////////////////////////////////////////////////////////////////////////////

class AVP_Brain_Actor_FaceHugger : public AVP_Brain_Actor
{
	typedef AVP_Brain_Actor PARENT;

public:
	AVP_Brain_Actor_FaceHugger( AVP_ServerActor_FaceHugger* pxFaceHugger );

	virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );
	virtual void DoPerFrameUpdate( const float fTimeslice );
	virtual void SlowUpdate();
	virtual void Shutdown();

	virtual Axon_Behaviour* CreateBehaviour( const u_int uBehaviourType, const Asura_Message* pxMessage = NULL );

	virtual void EstablishNavData();
	virtual void PostLoadInitialise();

	virtual bool RegisterWithTeam();
	virtual bool UnregisterWithTeam();

	const Asura_Vector_3& GetSpawnPoint() const;

	void SetMovementMode(AVP_Navigation::MovementMode uMode);

	bool CanPlanMove() const { return !m_bIsFalling && !m_bIsJumping; }
	bool IsFalling() const { return m_bIsFalling; }
	void SetFalling( bool bIsFalling); 
	bool IsJumping() const { return m_bIsJumping; }
	void SetJumping( bool bIsJumping );
	bool JustFinishedFalling() const { return m_bJustFinishedFalling; }

	bool ShouldAttack() const;
	bool FarFromSpawn() const;

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void DoRenderDebugInfo();
#endif

	virtual Asura_Guid GetPrimaryThreatGuid();
	virtual Asura_Guid GetAlertGuid();

	bool FindTarget(bool bIgnorePGVolumes);
	bool FindTargetPetMode();
	const AVP_AlienTeam_Knowledge* GetTarget() const { return m_pxTarget; }
	Asura_Guid GetTargetGuid() const { return m_uTargetGuid; }
	bool IsTargetValid() const;
	void IgnoreCurrentTarget(float fTime);
	bool IsTargetInJumpRange() const;
	bool IsTargetUnderAttack() const;
	bool IsTargetInPGVolumes() const;
	bool IsTargetFacingMe() const;
	bool IsTargetVisible() const;
	bool WasTargetLost() const;
	bool ShouldAttackFromTheFront() const;
	bool ShouldAttackFromTheBack() const;
	void AttachToTarget() const;
	bool IsTooCloseToTarget() const;

	void MarkTargetAttacked();
	void MarkTargetNotAttacked(bool bAttackAborted);
	void MarkTargetNotTargeted();
	void UpdateFalling(float fTimeslice );
	void ResetSpawnPoint();

	// Pet mode functions
	void SetMaster(Asura_Guid uMasterGuid);
	void ChangeAttackMode();
	void SetAlwaysAggressive(bool bAlwaysAggressive);
	bool IsPet() const { return m_bIsPet; }
	AVP_ServerEntity_Actor* GetMaster() const; 
	bool IsInMasterFoV() const;
	bool IsTooFarOrToCloseToMaster() const;
	bool IsAggressive() const { return IsAlwaysAggressive() || m_bPetAggressive; }
	bool IsAlwaysAggressive() const { return m_bPetAlwaysAggressive; }

	const AVP_FaceHuggerBehaviourData& GetBehaviourData() const;

	bool IsTightAvoidance() const { return !m_bIsLooseAvoidance; }
	Asura_Guid GetCurrentGPV() const { return m_uCurrentGPV; }

	AVP_ServerEntity_Squad_FaceHugger* GetSquad() const;

	static u_int GetLargestBehaviourSize();
	static const float s_fFrontalAngle;
	static const float s_fFrontalAngleCosine;

	static const float s_fFaceHuggerMinimumComfortDistance;

private:

	virtual void UpdateBehaviours( const float fTimeslice );
	void UpdateCurrentGPV();
	virtual float GetNearbyRange() const;

	void CacheTarget();
	bool IsTargetInPGVolumes( Asura_Guid uGuid ) const;
	void UpdateIgnoreList();
	bool IsIgnored(Asura_Guid uTargetGuid) const;

	Asura_Vector_3 m_xSpawnPoint;
	
	u_int m_uTargetLostCounter;

	float m_slowUpdateTimeSlice;

	Asura_Guid m_uTargetGuid;
	AVP_AlienTeam_Knowledge* m_pxTarget;

	struct TargetIgnoreData
	{
		Asura_Guid m_uGuidToIgnore;
		float m_fRemoveTime;
		Asura_Guid GetGuid() const { return m_uGuidToIgnore; }
	};
	Asura_LinkList< TargetIgnoreData > m_xTargetIgnoreList;
	float m_fTargetAquiredTime;
	bool m_bGotStartWaypointID;

	bool m_bIsFalling; // true while falling
	bool m_bIsJumping; // true from the start of requesting a jump till the animation finished
	bool m_bJustFinishedFalling; // true for one frame right after detecting that a fall finished.
	float m_fNotSupportedTime;
	Asura_Guid m_uMasterGuid;
	bool m_bIsPet;
	bool m_bPetAggressive;
	bool m_bPetAlwaysAggressive;
	bool m_bLookForDestroyedSquad;
	bool m_bIsLooseAvoidance;
	Asura_Guid m_uCurrentGPV;
	Asura_Guid m_uStartGPV;
	bool m_bTargetTargeted;
	bool m_bTargetAttacked;

};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BRAIN_ACTOR_FACEHUGGER_H__
