//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_Predalien.h
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_BRAIN_ACTOR_PREDALIEN_H__
#define __AVP_BRAIN_ACTOR_PREDALIEN_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////


// asura
#include "Core/Asura_HashedList.h"
#include "Core/AVP_Navigation_Types.h"

// axon
#include "Core/BrainTypes/AVP_Brain_Actor.h"

// avp

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Knowledge_System_Actor;
class AVP_ServerActor_Predalien;
class AVP_ClientActor_Predalien;
class AVP_ServerNode_VantagePoint_Predalien;
struct Axon_Knowledge;
class AVP_Brain_Actor_Predalien;
class AVP_Knowledge_System_Predalien;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_Predalien
//
//////////////////////////////////////////////////////////////////////////////

// blueprint configuration
struct AVP_Brain_Actor_Predalien_ConfigurationData
{
	struct MeleeConfig
	{
		float m_fMinTimeBetweenAttacks;
		float m_fMaxTimeBetweenAttacks;
	} m_fMeleeConfig;
};


class AVP_Brain_Actor_Predalien : public AVP_Brain_Actor
{
    typedef AVP_Brain_Actor PARENT;

public:
    AVP_Brain_Actor_Predalien( AVP_ServerActor_Predalien* pxPredalien );

    virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );
    virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	virtual void DoPerFrameUpdate(const float fTimeslice);

    virtual Axon_Behaviour* CreateBehaviour( const u_int uBehaviourType, const Asura_Message* pxMessage = NULL );

	void KnockDown();

    // must implement 
	virtual Asura_Guid GetPrimaryThreatGuid( void );
	virtual Asura_Guid GetAlertGuid( void );

	void SetFastTurnSpeed();
	void SetNormalTurnSpeed();
	void SetNormalAimSpeed();

	AVP_ServerActor_Predalien* GetPredalienActor() const;
	AVP_ClientActor_Predalien* GetPredalienClientActor() const;

	float GetHealthPercentage() const;
	float GetDamageTakenInLast() const;

	const AVP_Brain_Actor_Predalien_ConfigurationData& GetConfigInfo() const { return m_xConfig; }

	const Asura_Vector_3& GetSpawnPoint() const { return m_xSpawnPoint; } 

	bool CanPlanMove() const { return !m_bIsFalling && !m_bIsJumping && !m_bIsMelee; }
	bool CanUpdateMelee() const { return !m_bIsFalling && !m_bIsJumping; }

	bool IsFalling() const { return m_bIsFalling; }
	bool IsJumping() const { return m_bIsJumping; }
	bool IsMelee() const { return m_bIsMelee; }
	void SetJumping( bool bIsJumping );
	bool JustFinishedFalling() const { return m_bJustFinishedFalling; }
	void SetMelee( bool bIsMelee );

	bool ShouldRefreshPath(float fPathLength, float fDeltaDestinationSquared, float fDeltaTargetForwardCos = 1.0f) const; // 1.0f ignore turn  completely

    static u_int GetLargestBehaviourSize ();

protected:
    virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual float GetNearbyRange() const;

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
#endif

private:
	virtual void EstablishNavData();

	void UpdateFalling(float fTimeslice );

private:
    union
    {
        u_int m_uFlags;
        struct 
        {
            u_int m_bUnused : 1;
        };
    };

    enum { CURRENT_CHUNK_VERSION = 1 };

	const Asura_Vector_3 m_xSpawnPoint;

	bool m_bIsFalling; // true while falling
	bool m_bIsJumping; // true from the start of requesting a jump till the animation finished
	bool m_bJustFinishedFalling; // true for one frame right after detecting that a fall finished.
	bool m_bIsMelee; // true from the start of requesting a melee move till it is finished or stopped.

	float m_fNotSupportedTime;

	AVP_Brain_Actor_Predalien_ConfigurationData m_xConfig;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_BRAIN_ACTOR_PREDALIEN_H__
