//-----------------------------------------------------------------------------
//	File		: AVP_Brain_Actor_Sentrygun.h
//	Created		: 12 / 02 / 2008
//	Author		: Gareth Thomas
//	Copyright (C) 2007 Rebellion. All Rights Reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//	BEGIN FILE WRAPPER
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __AVP_BRAIN_ACTOR_SENTRYGUN_H__
#define __AVP_BRAIN_ACTOR_SENTRYGUN_H__	1
//	END FILE WRAPPER
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//	INCLUDES
//-----------------------------------------------------------------------------
#include "Core/BrainTypes/AVP_Brain_Actor.h"

//-----------------------------------------------------------------------------
// FORWARD DECLARATIONS
//-----------------------------------------------------------------------------
class AVP_ServerWeapon_Sentrygun;
class AVP_Behaviour_Sentrygun;
class AVP_Knowledge_System_Sentrygun;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Actor_Sentrygun
//
//////////////////////////////////////////////////////////////////////////////
class AVP_Brain_Actor_Sentrygun : public AVP_Brain_Actor
{
	typedef AVP_Brain_Actor PARENT;

public:

	AVP_Brain_Actor_Sentrygun(AVP_ServerWeapon_Sentrygun* pxSentrygun);

	// Public Member Virtual Functions...
	virtual void					ReadFromChunkStream					(Asura_Chunk_Stream& xStream);
	virtual void					WriteToChunkStream					(Asura_Chunk_Stream& xStream) const;
	virtual void					Initialise							(void);
	virtual void					Shutdown							(void);
	virtual void					InitialiseFromBlueprint				(const Asura_Blueprint& xBlueprint);
	virtual Axon_Behaviour* 		CreateBehaviour						(const u_int uBehaviourID, const Asura_Message* pxMessage = NULL);
	virtual Asura_Guid				GetPrimaryThreatGuid				(void);
	virtual Asura_Guid				GetAlertGuid						(void);

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void					DoRenderDebugInfo					(void);
#endif

	// Public Member Functions...
	AVP_ServerWeapon_Sentrygun*		GetSentrygunActor					(void) const;
	AVP_Knowledge_System_Sentrygun*	GetKnowledgeSystem					(void) const		{return m_pxSentryGunKnowledgeSystem;}
	void							UpdateKnowledge						(const float fTimeslice);

	// Public Statics
	static u_int					GetLargestBehaviourSize				(void);

protected:
	virtual void 					RegisterCallbacks					(void);
	virtual void 					UnregisterCallbacks					(void);
		

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
#endif

	// Protected Member functions...
	bool							IsSentrygunBehaviour				(const u_int uBehaviourType);
	Axon_Behaviour*					CreateSentrygunBehaviour			(const u_int uBehaviourType);
	bool							CreateDefaultBehaviour				(void);
	void							CleanUpBehaviours					(void);

private:

	// Private member Functions
	void							DoKnowledgeUpdate					(const float fTimeslice)		{UpdateKnowledge(fTimeslice);}

	// Note: Parent function is public
	virtual void					EstablishNavData					(void) {}

	Asura_Handle					m_iKnowledgeUpdateHandle;
	AVP_Knowledge_System_Sentrygun*	m_pxSentryGunKnowledgeSystem;

	enum { CURRENT_CHUNK_VERSION = 0 };
};

#endif //__AVP_BRAIN_ACTOR_SENTRYGUN_H__

/// end of file 'AVP_Brain_Actor_Sentrygun.h'
