//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Team_Alien.cpp
//
// Copyright (C) 2007 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Core/BrainTypes/AVP_Brain_Team_Alien.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Scheduler.h"
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
#include "Interface/Asura_CommandConsole.h"
#endif
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// avp
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"

// axon
#include "Core/Axon_System.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"

DVSVAR bool	dvs_bAllowAlienSpawning = true;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Brain_Team_Alien
//
//////////////////////////////////////////////////////////////////////////////

AVP_Brain_Team_Alien::AVP_Brain_Team_Alien( AVP_ServerEntity_Team_Alien* pxAlienTeam )
: PARENT( pxAlienTeam )
, m_iKnowledgeUpdateHandle( ASURA_INVALID_HANDLE )
, m_pxAlienTeamKnowledgeSystem( NULL )
{
#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	if(!Asura_CommandConsole::VarExists(L"AVP.Alien.AllowSpawning"))
	{
		Asura_CommandConsole::AddVar(L"AVP.Alien.AllowSpawning", &dvs_bAllowAlienSpawning);
	}
#endif
}

void AVP_Brain_Team_Alien::Initialise()
{
    PARENT::Initialise();

    m_pxAlienTeamKnowledgeSystem = new AVP_Knowledge_System_AlienTeam( this );
    m_pxAlienTeamKnowledgeSystem->Initialise();
}

void AVP_Brain_Team_Alien::Shutdown()
{
    m_pxAlienTeamKnowledgeSystem->Shutdown();
    delete m_pxAlienTeamKnowledgeSystem;
    m_pxAlienTeamKnowledgeSystem = NULL;

    PARENT::Shutdown();
}

void AVP_Brain_Team_Alien::DoPerFrameUpdate( const float fTimeslice )
{
	PARENT::DoPerFrameUpdate( fTimeslice );

	m_pxAlienTeamKnowledgeSystem->PerFrameUpdate( fTimeslice );
}

void AVP_Brain_Team_Alien::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
    int iVersion;
    xStream >> iVersion;

    PARENT::ReadFromChunkStream( xStream );

    if ( iVersion == CURRENT_CHUNK_VERSION )
	{
		m_pxAlienTeamKnowledgeSystem->ReadFromChunkStream( xStream );
	}
	else if ( iVersion == 4 )
    {
		m_pxAlienTeamKnowledgeSystem->ReadFromChunkStream( xStream );

		float fDummy;
		xStream >> fDummy;
		xStream >> fDummy;
	}
	else if ( iVersion >= 0 )
	{
		// Chuck away all of the old stuff.
	}
	else
    {
        Asura_Assertf( false, ( "AVP_Brain_Team_Alien::ReadFromChunkStream - unhandled version (%d)", iVersion ) );
    }
}

void AVP_Brain_Team_Alien::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
    xStream << CURRENT_CHUNK_VERSION;

    PARENT::WriteToChunkStream( xStream );

    m_pxAlienTeamKnowledgeSystem->WriteToChunkStream( xStream );
}

#if !defined( ASURA_NO_DEBUG_OUTPUT )
void AVP_Brain_Team_Alien::DoRenderDebugInfo()
{
	PARENT::DoRenderDebugInfo();

	Asura_Font_System::Print( GetDebugOutputPos().x, GetDebugOutputPos().y, 0.75f, 0xffffffff,
		ASURA_TEXTFORMAT_LEFTJUSTIFIED, "Alien team data:" );
	GetDebugOutputPos().x += 10.0f;
	GetDebugOutputPos().y += 10.0f;

	GetKnowledgeSystem()->RenderDebugInfo();
}
#endif

void AVP_Brain_Team_Alien::RegisterCallbacks()
{
    PARENT::RegisterCallbacks();

    if ( m_iKnowledgeUpdateHandle == ASURA_INVALID_HANDLE )
    {
        Asura_Scheduler& xKnowledgeUpdateScheduler = Axon_System::GetKnowledgeUpdateScheduler();
        Asura_Callback_MemberFunction_1Arg< void, float, AVP_Brain_Team_Alien > xKnowledgeCallback( this, &AVP_Brain_Team_Alien::DoKnowledgeUpdate );
        m_iKnowledgeUpdateHandle = xKnowledgeUpdateScheduler.RegisterCallback( 0.5f, &xKnowledgeCallback );
    }
}

void AVP_Brain_Team_Alien::UnregisterCallbacks()
{
    PARENT::UnregisterCallbacks();

    if ( m_iKnowledgeUpdateHandle != ASURA_INVALID_HANDLE )
    {
        Axon_System::GetKnowledgeUpdateScheduler().UnregisterCallback( m_iKnowledgeUpdateHandle );
        m_iKnowledgeUpdateHandle = ASURA_INVALID_HANDLE;
    }
}

void AVP_Brain_Team_Alien::UpdateKnowledge( const float fTimeslice )
{
    if ( m_pxAlienTeamKnowledgeSystem )
    { 
        m_pxAlienTeamKnowledgeSystem->Update( fTimeslice ); 
    }
}

#ifndef ASURA_NO_DEBUG_OUTPUT
bool AVP_Brain_Team_Alien::Verify()
{
	Asura_Assert( m_pxAlienTeamKnowledgeSystem, "AVP_Brain_Team_Alien::Verify - no knowledge system" );
	return ( PARENT::Verify() && m_pxAlienTeamKnowledgeSystem && m_pxAlienTeamKnowledgeSystem->Verify() );
}
#endif

AVP_ServerEntity_Team_Alien* AVP_Brain_Team_Alien::GetAlienTeam() const
{
    return static_cast< AVP_ServerEntity_Team_Alien* >( GetTeam() );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
