///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Knowledge_System_AlienQueen.h
// Author : David Wooldridge
// Date   : 19/11/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_KNOWLEDGE_SYSTEM_ALIENQUEEN_H__
#define __AVP_KNOWLEDGE_SYSTEM_ALIENQUEEN_H__ 1

///////////////////////////////////////////////////////////////////////////////
// INCLUDES

#include "Core/BrainTypes/AVP_Brain_Actor.h"
#include "Knowledge/AVP_Knowledge_System.h"

///////////////////////////////////////////////////////////////////////////////
// FORWARD DECLARATIONS

class Asura_Chunk_Stream;

///////////////////////////////////////////////////////////////////////////////
// CLASSES

class AVP_Knowledge_System_AlienQueen : public AVP_Knowledge_System
{
	typedef AVP_Knowledge_System PARENT;

public:
	AVP_Knowledge_System_AlienQueen( AVP_Brain_Actor* pxBrain );
	~AVP_Knowledge_System_AlienQueen();



#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual bool Verify();
#endif

	virtual void ReadFromChunkStream	( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream		( Asura_Chunk_Stream& xStream ) const;
	virtual void Update					( const float fTimeslice );
	virtual void PerFrameUpdate         ( const float fTimeslice );
	virtual void Initialise				( void );

	void UpdateTargetKnowledge			( const float fTimeslice );
	void AddEntityToTargetList			( Asura_Guid uGuid, int iClassification );
	void RemoveEntityFromTargetList		( Asura_Guid uGuid );
	void ClearTargetList				() { m_xTargetList.Clear(); }

	Axon_Knowledge* GetTargetKnowledge(){ return &m_xPriorityTargetKnowledge; }

private:

	bool				AddTarget ( Asura_Guid uGuid, int iClassification );
	bool				RemoveTarget ( Asura_Guid uGuid );
	void				RemoveDeadTargetsFromList ();

	AVP_Brain_Actor*	GetActorBrain()		const { return static_cast<AVP_Brain_Actor*>( GetBrain() ); }

	// Holds data of the currently prioritised target.
	Axon_Knowledge      m_xPriorityTargetKnowledge; 

	// Have the list here not in sences as all entities on it are added from messages via the server and brain. 
	Asura_Hashed_List<Axon_Knowledge> m_xTargetList;

	   enum { CURRENT_CHUNK_VERSION = 0 };
};

///////////////////////////////////////////////////////////////////////////////

#endif