//////////////////////////////////////////////////////////////////////////////
//
// AVP_Sense_Vision.h
//
// Created by NJD - 06 December 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_SENSE_VISION_H__
#define __AVP_SENSE_VISION_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Axon
#include "Knowledge/SenseTypes/Axon_Sense_Vision.h"

// AVP
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Knowledge_System_Actor;
class AVP_ServerActor_Alien;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_Sense_Vision
//
//////////////////////////////////////////////////////////////////////////////

struct KnowledgeSettings
{
	// Blueprinted Values
	float m_fPassiveRange;
	float m_fActiveRange;
	float m_fAggressiveRange;
	float m_fPassiveThreshold;
	float m_fActiveThreshold;
	float m_fAggressiveThreshold;
	float m_fCloakedVisibleRangeSq;
	float m_fKnowledgeLifetime;
	float m_fVisionCutOffDistanceSq;
	float m_fDarknessLightThreshold;
	float m_fBrightnessLightThreshold;
	float m_fDarknessLightScore;
	float m_fBrightnessLightScore;
	float m_fNeutralLightScore;
	float m_fPassiveCloakingThreshold;
	float m_fActiveCloakingThreshold;
	float m_fAggressiveCloakingThreshold;
};

class AVP_Sense_Vision : public Axon_Sense_Vision
{
    typedef Axon_Sense_Vision PARENT;

public:

	enum TypeInfo
	{
		TYPE_INFO_NOT_SET = 0,
		TYPE_INFO_CAN_SEE,
		TYPE_INFO_CANT_SEE,
		TYPE_INFO_LASER,
		TYPE_INFO_CLOAKED_CANT_SEE,
		MAX_TYPE_INFOS,
	};

    AVP_Sense_Vision(AVP_Knowledge_System_Actor* pxAVPKnowledgeSystem);
    virtual ~AVP_Sense_Vision();

	virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );
    virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
    virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	static bool  CullOnFOV(const AVP_ServerEntity_Actor* pxOurActor, const AVP_ServerEntity_Actor* pxThreatActor);
	static bool  IsOtherActorInFieldOfView(const AVP_ServerEntity_Actor* pxOurActor, const AVP_ServerEntity_Actor* pxOtherActor);

    virtual void Update(const float fTimeslice);
	bool LOSCheckOnActor(const AVP_ServerEntity_Actor* pxThreatActor, const bool bFOV = false);

	int		GetTargetPriority( Asura_Guid uThreatGuid );
	bool	IsInDarkness( const Asura_Guid uAlienGuid );
	void	PlayEnteringAggressiveDialogue( void );
	void	ProcessPredatorLaser( void );

	float	GetAlertLevelRange(void);
	bool	ShouldCullActorOnRangeAndFOV(const AVP_ServerEntity_Actor* pxOurActor, const AVP_ServerEntity_Actor* pxOtherActor);
	float	GetScoreForFOVPosition(const AVP_ServerEntity_Actor* pxOurActor, const Asura_Vector_3& xPoint);
	float	GetScoreForRange(const Asura_Vector_3& xOurPos, const Asura_Vector_3& xOtherPos);
	float	GetScoreForSpeed(const AVP_ServerEntity_Actor* pxActor);
	float	GetLightingScore(const Asura_Vector_3& xThreatPos);
	float	GetAlertLevelScoreToBeat(void);
	float	GetCloakingAlertLevelScoreToBeat(void);

	virtual void	AddToVisionList(Axon_Knowledge* pxNewKnowledge);

	float GetAlienPlayerScore()			{return m_fAlienPlayerScore;}
	float GetAlienPlayerScoreToBeat()	{return m_fAlienPlayerScoreToBeat;}

	static float UpdatePlayerAlienThreatStatus(AVP_ServerActor_Alien* pxPlayerAlien);

	float GetPassiveRange()		{ return m_xKnowledgeSettings.m_fPassiveRange; }
	float GetActiveRange()		{ return m_xKnowledgeSettings.m_fActiveRange; }
	float GetAggressiveRange()	{ return m_xKnowledgeSettings.m_fAggressiveRange; }
	float GetRange()			{ return m_fRange; }

	void UpdateKnowlegdeSettings( KnowledgeSettings* pxKnowledgeSettings );

#ifndef ASURA_NO_DEBUG_OUTPUT
	static void RenderKnowledgeDebugInfo(Asura_Guid uGuid);
#endif

	void SetVisionFOV( const float fTanHorHalfFOV, const float fTanVertUpHalfFOV, const float fTanVertDownHalfFOV ) { m_fTanFovX = fTanHorHalfFOV; m_fTanFovYUp = fTanVertUpHalfFOV; m_fTanFovYDown = fTanVertDownHalfFOV; }

protected:
	AVP_Knowledge_System_Actor* GetActorKnowledgeSystem();

	void InitFromSettingsBlueprint(void);
	void DepricateCurrentKnowledge(const float fTimeslice);
	void AddNewKnowledge(void);

	bool ShouldIgnoreCloakedStatus();

private:

	KnowledgeSettings m_xKnowledgeSettings;

	float m_fLightKnowledgeLifetime;

	bool				m_bEnemyLaserActive;
	Asura_Vector_3		m_xLaserPos;
	Asura_Vector_3		m_xLaserDirection;
	float				m_fLaserRange;
	Asura_Guid			m_uLaserOwnerGuid;
	bool				m_bIsInShadow_LastFrame;

#ifndef ASURA_NO_DEBUG_OUTPUT
	// debug rendering variables
	static float s_fSpeedScore;
	static float s_fRangeScore;
	static float s_fFOVScore;
	static float s_fLightScore;
	static float s_fTotalScore;
	static float s_fPassiveThreshold;
	static float s_fActiveThreshold;
	static float s_fAggressiveThreshold;
	static Asura_Guid s_uRenderGuid;
#endif

	// Alien Player Stealth Indicator Values
	float m_fAlienPlayerScore;
	float m_fAlienPlayerScoreToBeat;

private:

	static const u_int uCURRENT_CHUNK_VERSION = 2;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_SENSE_VISION_H__
