//////////////////////////////////////////////////////////////////////////////
//
// AVP_Sense_Vision_Sentrygun.cpp
//
// Created by NJD - 06 December 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Core
#include "Core/Asura.h"

// This files header
#include "Knowledge/SenseTypes/AVP_Sense_Vision_Sentrygun.h"
 
// Asura
#include "Blueprint/Asura_Blueprint_System.h"

// AVP Axon
#include "Knowledge/AVP_Knowledge_System_Sentrygun.h"
#include "Tactical/AVP_Tactical_System.h"

// AVP
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_ServerWeapon_Sentrygun.h"

static const u_int g_uCurrentChunkVersion = 1;

AVP_Sense_Vision_Sentrygun::AVP_Sense_Vision_Sentrygun( AVP_Knowledge_System_Sentrygun* pxAVPKnowledgeSystem )
: PARENT( pxAVPKnowledgeSystem )
{
	m_fRange       = 200.0f;
	m_fTanFovX     = Asura_Maths::Tangent(Asura_Maths::DegToRad(170.0f) * 0.5f);
	m_fTanFovYUp   = Asura_Maths::Tangent(Asura_Maths::DegToRad(130.0f) * 0.25f);
	m_fTanFovYDown = Asura_Maths::Tangent(Asura_Maths::DegToRad(110.0f) * 0.6f);
}

AVP_Sense_Vision_Sentrygun::~AVP_Sense_Vision_Sentrygun() 
{
	DestroyAllKnowledge();
}

void AVP_Sense_Vision_Sentrygun::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int uVersion;
	xStream >> uVersion;

	switch ( uVersion )
	{
		case 0:
		{
			Asura_Assert( false, "AVP_Knowledge_System_Sentrygun never had a chunk version 0" );
		}
		break;

		case 1:
		{
			PARENT::ReadFromChunkStream( xStream );
		}
		break;

		default:
		{
			Asura_Assertf( false, ( "Unhandled chunk version (%d)", uVersion ) );
		}
	}
}

void AVP_Sense_Vision_Sentrygun::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << g_uCurrentChunkVersion;

	PARENT::WriteToChunkStream(xStream);
}

void AVP_Sense_Vision_Sentrygun::Update(const float fTimeslice) 
{
	AVP_ServerEntity_Actor* pxOurServerActor = GetSentrygunKnowledgeSystem()->GetActorBrain()->GetActor();
	Asura_Matrix_3x3 xOri;

	pxOurServerActor->GetWorldSpaceMarkerPoint( 0x0001A7D2 /* muz */, &m_xPosition, &xOri );
	m_xDirection = Asura_Vector_3( xOri.e13, xOri.e23, xOri.e33 );

	//Sentrygun has no memory
	DestroyAllKnowledge();
	m_xVisionList.Clear();

	// lets see what we can see
	for (Asura_Hashed_List_ConstIt<AVP_ServerEntity_Actor> xIt(AVP_ServerEntity_Actor::GetList()); !xIt.Done(); xIt.Next())
	{
		const AVP_ServerEntity_Actor*	pxTargetActor = xIt.GetCurrent();		

		// isActor Valid
		if (!pxTargetActor)
			continue;

		// Ignore ourselves.
		if (pxOurServerActor->GetGuid() == pxTargetActor->GetGuid())
			continue;

		// is the actor dead or inactive
		if (pxTargetActor->IsDead() || !pxTargetActor->IsEntityActive())
			continue;

		AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity(pxTargetActor->GetGuid());

		// For cases of spawned actors make sure we have started to render them before trying to use them for knowledge.
		if (pxClient && pxClient->GetShapeInstance().GetAnimation() && pxClient->GetShapeInstance().GetAnimation()->GetTweenStore())
		{
			if (!pxClient->GetShapeInstance().GetAnimation()->GetTweenStore()->IsPositionDataValid())
			{
				continue;
			}
		}
		else
		{
			continue;
		}

		// Ignore friends
		if ( AVP_ALLEGIANCE_ENEMY != AVP_Tactical_System::GetAllegiance( *pxOurServerActor, *pxTargetActor ) )
		{
			continue;
		}

// 		// can't see cloaked actors ---- Yes we can :)
// 		if (pxTargetActor->IsCloaked(true))
// 		{
// 			continue;
// 		}
		
		const Asura_Vector_3&	xActorPosition	= pxTargetActor->GetPosition();
		Asura_Vector_3			xDirection		= xActorPosition - m_xPosition;
		const float				fDistance		= xDirection.Magnitude();

		Asura_Vector_3			xActorChestPosition;	
		pxTargetActor->GetChestPosition( xActorChestPosition );
		Asura_Vector_3			xDirectionToChest		= xActorChestPosition - m_xPosition;
		const float				fDistanceToChest		= xDirectionToChest.Magnitude();

		Asura_Vector_3			xActorEyePosition;
		pxTargetActor->GetHeadPosition( xActorEyePosition );
		Asura_Vector_3			xDirectionToEyes		= xActorEyePosition - m_xPosition;
		const float				fDistanceToEyes			= xDirectionToEyes.Magnitude();
		

		// If it is outside our range 
		if( ShouldCullOnRange( fDistance, xDirection) )
		{
			continue;
		}

		// if it is outside our fire arch (based on turret pos and dir)
		if( static_cast<AVP_ServerWeapon_Sentrygun*>( GetSentrygunKnowledgeSystem()->GetActorBrain()->GetActor() )->IsOutsideFireArc( xDirection ) )
		{
			continue;
		}

		// if ground AND chest AND eye positions are outside the FOV then skip (based on the barrel pos and dir)
		if( ShouldCullOnFOV( fDistance, xDirection ) && ShouldCullOnFOV( fDistanceToChest, xDirectionToChest ) && ShouldCullOnFOV( fDistanceToEyes, xDirectionToEyes ) /*&& ( ! LOSCheckOnActor( pxTargetActor ) )*/ )
		{
			if( fDistance < 2.0f )
			{
				// if the target is very close to the gun then double check, only continue if we dont have a LOS to the target. 
				// This is because targets very close to the target can actually be rught in front of the gun but have the ground, chest and eyes all out of the FOV. 
				if( ! LOSCheckOnActor( pxTargetActor ) )
				{
					continue;
				}
			}
			else
			{
				continue;
			}
		}

		// as a final check do a los test (incase the target is in the FOV but in cover.)
		if( ! LOSCheckOnActor( pxTargetActor ) )
		{
			continue;
		}

		// Now check to see if an entry already exists.
		Axon_Knowledge* pxExistingKnowledge = m_xVisionList.FindObject( pxTargetActor->GetGuid() );

		if( pxExistingKnowledge )
		{
			pxExistingKnowledge->m_uSourceGuid           = pxTargetActor->GetGuid();
			pxExistingKnowledge->m_uSourceClassification = pxTargetActor->GetClassification();
			pxExistingKnowledge->m_iSenseID              = AXON_SENSE_TYPE_VISION;
			pxExistingKnowledge->m_uTypeInformation      = 0;
			pxExistingKnowledge->m_fAge                  = 0.0f;
			pxExistingKnowledge->m_fThreatValue          = 0.0f;
			pxExistingKnowledge->m_xPosition             = xActorPosition;
			pxExistingKnowledge->m_xDirection            = xDirection;
			pxExistingKnowledge->m_fDistance             = fDistance;
		}
		else
		{
			// Create a new knowledge.
			Axon_Knowledge* pxKnowledge = m_pxKnowledgeSystem->CreateKnowledge();
			Asura_Assert( pxKnowledge, "Unable to allocate knowledge from knowledge pool" );
			if (pxKnowledge)
			{
				pxKnowledge->m_uKnowledgeID          = m_pxKnowledgeSystem->GetNewKnowledgeID();
				pxKnowledge->m_uSourceGuid           = pxTargetActor->GetGuid();
				pxKnowledge->m_uSourceClassification = pxTargetActor->GetClassification();
				pxKnowledge->m_iSenseID              = AXON_SENSE_TYPE_VISION;
				pxKnowledge->m_uTypeInformation      = 0;
				pxKnowledge->m_fAge                  = 0.0f;
				pxKnowledge->m_fThreatValue          = 0.0f;
				pxKnowledge->m_xPosition             = xActorPosition;
				pxKnowledge->m_xDirection            = xDirection;
				pxKnowledge->m_fDistance             = fDistance;

				// Add it to the knowledge list.
				m_xVisionList.AddToList(pxKnowledge);
			}
		}
	}
}

AVP_Knowledge_System_Sentrygun* AVP_Sense_Vision_Sentrygun::GetSentrygunKnowledgeSystem()
{
	return static_cast< AVP_Knowledge_System_Sentrygun* >( m_pxKnowledgeSystem );
}

bool AVP_Sense_Vision_Sentrygun::LOSCheckOnActor( const AVP_ServerEntity_Actor* pxThreatActor )
{	
	const AVP_ServerEntity_Actor* pxOurActor = GetSentrygunKnowledgeSystem()->GetActorBrain()->GetActor();

	// Simple LOS check
	Asura_Assertf(pxOurActor, ("Invalid source actor."));
	Asura_Assertf(pxThreatActor, ("Invalid threat actor."));
	if (pxOurActor && pxThreatActor)
	{
		// Make sure we are all alive and active
		if (!pxOurActor->IsEntityActive() || !pxThreatActor->IsEntityActive() || pxOurActor->IsDead() || pxThreatActor->IsDead())
		{
			return false;
		}

		// Get the eye position
		Asura_Vector_3 xEyePosition = pxOurActor->GetPosition();
		pxOurActor->GetHeadPosition(xEyePosition);

		// Get the Threat chest position
		Asura_Vector_3 xThreatPosition = pxThreatActor->GetPosition();
		pxThreatActor->GetChestPosition(xThreatPosition);

		// Get the direction of the LOS
		Asura_Vector_3 xDirection = xThreatPosition - xEyePosition;
		const float    fDistance  = xDirection.Normalise();

		const Asura_LOS_InputData   xInputData(xEyePosition, xDirection, pxOurActor->GetGuid(), fDistance, true);
		const Asura_LOS_OutputData* pxOutputData = Asura_LOS::TestAgainstEverything(&xInputData);

		// if we didn't hit the environment and hit our Threat we have a sighting.
		if ((!pxOutputData->m_bHitEnvironment) && (pxOutputData->m_uHitEntity == pxThreatActor->GetGuid()))
		{
			return true;
		}

		return false;
	}
	return false;
}
