///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Actor_DamageDirectionIndicator.h
// Author : Chris Wallace
// Date   : 25-Sept-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ACTOR_DAMAGEDIRECTIONINDICATOR_H__
#define __AVP_ACTOR_DAMAGEDIRECTIONINDICATOR_H__

#include "GUIMenu/AVP_GUIMenu_System.h"

#define MAX_DAMAGE_INDICATORS AVP_GUIMenu_System::END_DAMAGE_DIRECTION_INDICATOR - AVP_GUIMenu_System::BEGIN_DAMAGE_DIRECTION_INDICATOR

class AVP_ClientEntity_Actor;
class AVP_EntityDamage_ApplyToEntity;

////////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Actor_DamageDirectionIndicator_System
// Author       : Chris Wallace
// Date         : 28-Nov-2007
//
// Description  : Class to handle the creation/deletion and storing of 
//				  AVP_Actor_DamageDirectionIndicator's
////////////////////////////////////////////////////////////////////////////////////

class AVP_Actor_DamageDirectionIndicator_System
{
public:
	AVP_Actor_DamageDirectionIndicator_System( const AVP_ClientEntity_Actor& xActor );

	void Add( const AVP_EntityDamage_ApplyToEntity& xDamage );
	void Update();
	void Reset();

	static void CreateConsoleVars();

protected:

	// Damage Hit Effect Block
	enum DamageHitEffectType
	{
		DAMAGEHIT_TOPLEFT,
		DAMAGEHIT_TOPCENTRE,
		DAMAGEHIT_TOPRIGHT,
		DAMAGEHIT_CENTRERIGHT,
		DAMAGEHIT_BOTTOMRIGHT,
		DAMAGEHIT_BOTTOMCENTRE,
		DAMAGEHIT_BOTTOMLEFT,
		DAMAGEHIT_CENTRELEFT,
		DAMAGEHIT_MAX
	};

	class DamageHitEffect
	{
	public:
		DamageHitEffect() {	Reset();	}
		void AddConsoleVar( const Asura_UnicodeChar* wszVar1, const Asura_UnicodeChar* wszVar2, const Asura_UnicodeChar* wszVar3 );
		void Update();
		void HitMe();
		void Reset()
		{
			for(int i=0; i<s_iMaxStates;i++)
			{
				m_uCurrentState[i]=0; 
				m_fTimer[i]=0.0f;
			}
		}

		static float s_TimeToFade;
		static u_int s_NumOfDifferentTypes;

	private:
		const static int s_iMaxStates = 3;
		u_int m_uCurrentState[s_iMaxStates];
		float m_fTimer[s_iMaxStates];
	};
	static DamageHitEffect s_xDamageHitEffect[DAMAGEHIT_MAX];
	// End of Damage Hit Effect Block



private:
	const AVP_ClientEntity_Actor& m_xOwnerActor;

	u_int			m_uCurrentIndex;
	float			m_fTimeSinceLastAdd;
	float			m_afLifeTimer[ MAX_DAMAGE_INDICATORS ];
	Asura_Vector_3	m_axDirection[ MAX_DAMAGE_INDICATORS ];

	static bool		s_bShowDamageIndicator[ MAX_DAMAGE_INDICATORS ];

	void AddDamageHitEffect( const Asura_Vector_3& xDirection );	// This function is called only internally from Add()
};

#endif //__AVP_ACTOR_DAMAGEDIRECTIONINDICATOR_H__
