
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Actor_PhysicsObjectHandler.h
// Author : Andy Tate
// Date   : 10-May-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ACTOR_PHYSICSOBJECTHANDLER_H__
#define __AVP_ACTOR_PHYSICSOBJECTHANDLER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Network/Asura_Guid.h"
#include "Physics/Asura_Physics_Object.h"

#include "Physics/AVP_PhysicsObjectHandler.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Collision_Object;
class Asura_PosAndOriInterpolator;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Actor_PhysicsObjectHandler
// Author       : Andy Tate
// Date         : 10-May-2007
//
// Description  : Class to handle the manipulation of the physics-object
//				  used by EITHER client-side or server-side entities.
////////////////////////////////////////////////////////////////////////////////

class AVP_Actor_PhysicsObjectHandler : public AVP_PhysicsObjectHandler
{
public:
	// Public enumerated types...
	enum Type
	{
		FREE_AI,
		PLAYER,
		REMOTE_CLIENT,
		GAMESCENE_AI,
		STATIC_AI, // e.g. Immovable sentry guns

		MAX_TYPES
	};

	enum GroundType
	{
		OFF_GROUND,
		ON_FLOOR,
		ON_CEILING,
		ON_WALL,

		MAX_GROUND_TYPES
	};

	// Public functions...
	AVP_Actor_PhysicsObjectHandler	(const Asura_Guid uOwnerGuid, const Type eType, const Asura_Collision_Object& xCollisionObject, const Asura_Physics_Object::COLLISION_PROXY eCollisionProxy, const u_short usFlagsToIgnore);
	virtual ~AVP_Actor_PhysicsObjectHandler ();

	// Manipulation functions...
	void	UpdateFromInterpolator(const Asura_PosAndOriInterpolator& xInterpolator);
	void	UpdateInterpolator(Asura_PosAndOriInterpolator& xInterpolator) const;

	// Reset functions...

	void			Reset(const Type eType, const Asura_Collision_Object& xCollisionObject, const Asura_Physics_Object::COLLISION_PROXY eCollisionProxy);

	// Ground-type functions...
	static GroundType	CalculateGroundType(const Asura_AVector3& xNormal);
	static GroundType	CalculateGroundType(const Asura_Matrix_3x3& xOrientation);

	static GroundType	CalculateGroundType(const Asura_Vector_3& xNormal)	{ return(CalculateGroundType(Asura_AVector3(xNormal))); }

	GroundType			PerformOnGroundCheck(void) const;

	u_short GetFlagsToIgnore() const { return m_usFlagsToIgnore; }

	static Asura_Physics_Object::COLLISION_PROXY GetActorCollisionProxy(u_short usActorClassification);

protected:
	// Protected variables...
	Asura_Guid		m_uOwnerGuid;					// The GUID of the owner entity.
	u_short			m_usFlagsToIgnore;				// The polygon-flags the physics-object is to ignore.
	mutable float	m_fTimeOfLastInterpolatorData;	// Use when we update the interpolator.

	// Protected functions...


private:
	// Private typedefs...
	typedef AVP_PhysicsObjectHandler PARENT;

	// Private variables...

	// Private functions...
	AVP_Actor_PhysicsObjectHandler  (void);
	AVP_Actor_PhysicsObjectHandler  (const AVP_Actor_PhysicsObjectHandler& xOther);

	AVP_Actor_PhysicsObjectHandler&	operator=	(const AVP_Actor_PhysicsObjectHandler& xOther);

};   // End of class 'AVP_Actor_PhysicsObjectHandler'.


#endif   // ifndef __AVP_ACTOR_PHYSICSOBJECTHANDLER_H__


/// End of file. ///
