////////////////////////////////////////////////////////////////
//
// File:		AVP_Actor_Selector.h
// Author:		Chris Wallace
// Date:		16-March-2007	
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ACTOR_SELECTOR_H__
#define __AVP_ACTOR_SELECTOR_H__

///////////////////////////////////////////////
// #includes
///////////////////////////////////////////////

#include "Network/Asura_Guid.h"
#include "Core/Asura_Hash.h"
#include "GUIMenu/AVP_GUIMenu_System.h"

///////////////////////////////////////////////
// #defines
///////////////////////////////////////////////

class Asura_ControlMap;
class AVP_ClientEntity_Actor;
class AVP_Inventory_Group;

///////////////////////////////////////////////
//
// Class:		AVP_Actor_Selector
//
// Description:	A menu element
//
///////////////////////////////////////////////

class AVP_Actor_Selector
{
public:
	enum Selector_Directions{
		SELECTOR_TOP,
		SELECTOR_TOPRIGHT,
		SELECTOR_RIGHT,
		SELECTOR_BOTTOMRIGHT,
		SELECTOR_BOTTOM,
		SELECTOR_BOTTOMLEFT,
		SELECTOR_LEFT,
		SELECTOR_TOPLEFT,
	};

	AVP_Actor_Selector			( AVP_ClientEntity_Actor& xClient, const u_int uNumSlots, const int* piSlotKey, const Selector_Directions* peSlotDir, const Asura_Hash_ID uInventoryGroupHash, bool* pbShowConsoleVar );
	virtual ~AVP_Actor_Selector ();

	void Initialise( const AVP_ClientEntity_Actor& xClient, const Asura_Hash_ID uGroupHash );
	virtual void Update		( const AVP_ClientEntity_Actor& xClient );

	void SetSlotGuiElements( const AVP_GUIMenu_System::STORED_WIDGETS* peSlotGuiElement ) { m_peSlotGuiElement = peSlotGuiElement; }
	void SetSlotDirections( const Selector_Directions* peSlotDirections ) { m_peSlotDirections = peSlotDirections; }
	void SetEmptySlotImageHash( Asura_Hash_ID uImageHash ) { m_uEmptySlotImageHash = uImageHash; }
	void SetControlMap( Asura_ControlMap* pxControlMap ) { m_pxControlMap = pxControlMap; }

	void SelectItemSlot		( const AVP_ClientEntity_Actor& xClient, u_int uItemSlot );
	void SelectNextItem		( const AVP_ClientEntity_Actor& xClient, bool bWrap );
	void SelectPreviousItem	( const AVP_ClientEntity_Actor& xClient, bool bWrap );

	void Activate( AVP_ClientEntity_Actor& xClient );
	void Deactivate( AVP_ClientEntity_Actor& xClient );

	bool IsActive(void)	const		{ return m_bActive; }
	bool IsItemSelectable( const AVP_ClientEntity_Actor& xClient, int iItemIndex ) const;
	bool IsItemEmpty( const AVP_ClientEntity_Actor& xClient, int iItemIndex ) const;
	bool IsFading(void) const		{ return m_fFadeAmount > 0.0f; }

	bool IsReadyToClose(void) const	{ return m_bReadyToClose; }

	Asura_Hash_ID GetItemHashFromSlot( u_int uIndexFromBlueprints );

protected:
	struct AVP_Actor_SelectorSlot
	{
		AVP_Actor_SelectorSlot();
		void Initialise( const AVP_Inventory_Group* pxGroup, const Asura_Hash_ID uItemHash );

		Asura_Hash_ID	m_uInvItemHash;
		Asura_Hash_ID	m_uSelectedImageHash;
		Asura_Hash_ID	m_uNotSelectedImageHash;
		Asura_Hash_ID	m_uSelectedEmptyImageHash;
		Asura_Hash_ID	m_uNotSelectedEmptyImageHash;
		Asura_Hash_ID	m_uNotSelectableImageHash;

		union
		{
			u_int		m_uFlags;
			struct
			{
				u_int	m_bInUse		: 1,
										: 0;
			};
		};
	};

	u_int GetItemSlotForInvItem( const AVP_Inventory_Group* pxGroup, const Asura_Hash_ID uItemHash );
	bool CanUseSelector( AVP_ClientEntity_Actor& xClient );
	
	AVP_Actor_SelectorSlot*						m_pxSlot;
	const int*									m_piSlotKey;
	const AVP_GUIMenu_System::STORED_WIDGETS*	m_peSlotGuiElement;
	const Selector_Directions*					m_peSlotDirections;

	Asura_ControlMap*							m_pxControlMap;
	bool*										m_pbShowConsoleVar;
	Asura_Vector_2								m_xPosition;
	Asura_Guid									m_uOwnerGuid;
	Asura_Hash_ID								m_uInvGroupHash;
	Asura_Hash_ID								m_uEmptySlotImageHash;

	union
	{
		u_int		m_uFlags;
		struct
		{
			u_int	m_bActive			: 1,
					m_bReadyToClose		: 1,
					m_bSlotSelected		: 1,
					m_uNumSlots			: 4,
					m_uCurrentItem		: 4,
					m_uPrevSelected		: 4,
										: 0;
		};
	};
	
	float	m_fKeepOpenTimer;
	float	m_fFadeAmount;

	float	m_fKeyPressedTime;

private:
	void PlaySound(Asura_Hash_ID uBlueprintParamHash,  const AVP_ClientEntity_Actor& xClient);

	AVP_Actor_Selector();
	AVP_Actor_Selector( const AVP_Actor_Selector& xOther );
	const AVP_Actor_Selector& operator=( const AVP_Actor_Selector& xOther );
};

#endif //__AVP_ACTOR_SELECTOR_H__

/// End of file. ///
