///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Dropship_AnimationHandler.h
// Author : Sharon
// Date   : 6-Oct-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_DROPSHIP_ANIMATIONHANDLER_H__
#define __AVP_DROPSHIP_ANIMATIONHANDLER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/AVP_Actor_AnimationHandler.h"
#include "Animation/Asura_Animation.h"
#include "Entity/PhysicalObject/Actor/Vehicles/Dropship/AVP_Messages_Dropship.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_ClientVehicle_Dropship;
struct AVP_Message_Dropship_Extended_RequestAnimation;
struct AVP_Message_Dropship_RequestAnimation;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Dropship_AnimationHandler
////////////////////////////////////////////////////////////////////////////////
class AVP_Dropship_AnimationHandler : public AVP_Actor_AnimationHandler
{
public:
	// Public functions...
	AVP_Dropship_AnimationHandler(AVP_ClientVehicle_Dropship& xOwner);

	virtual	void Reset (void) { PARENT::Reset(); }

	// Overridden functions...
	virtual	bool MakeRagdoll(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation) { return false; }

    void HandleDropshipAnimationRequest( const AVP_Message_Dropship_RequestAnimation& xAnimRequestData );
    virtual void Update();

	virtual void	UpdateAnimStateData( const AVP_Actor_AnimStateData* pxAnimDataLBSlow, const AVP_Actor_AnimStateData* pxAnimDataLBFast, const AVP_Actor_AnimStateData* pxAnimDataUBSlow, const AVP_Actor_AnimStateData* pxAnimDataUBFast, const bool bLoop, const float fBlendLBSpeed, const float fBlendUBSpeed, const float fOSMoveDirAngle, const float fTweenTime, const bool bSynch ) {}
	virtual void	PauseCurrentAnimState() {}
	virtual void	PlayCurrentAnimState() {}
	// these are only needed by the AI but should work for players too should the need arise
	virtual bool	IsCurrentAnimStateFinished() const { return true; }
	virtual void	SetCurrentAnimStateLooped( const bool bSet ) {}
	virtual const AVP_Actor_AnimStateData* GetCurrentAnimStateData() const { return NULL; }

	void PostCreateInitialise(Asura_Blueprint* pxBluprint);

	bool AreFrontPodsOpened() const;
	bool AreFrontPodsClosed() const;
	bool AreBackPodsOpened() const;
	bool AreBackPodsClosed() const;
	bool AreBackPodsDoorsOpened() const;
	bool AreBackPodsDoorsClosed() const;

protected:
	AVP_ClientVehicle_Dropship&	m_xOwnerDropship;

//    void CreateRequestedAnimation( const Asura_Hash_ID uAnimHashID, const u_int uAnimFlags );

private:
	typedef AVP_Actor_AnimationHandler PARENT;

	AVP_Dropship_AnimationHandler(void);
	AVP_Dropship_AnimationHandler(const AVP_Dropship_AnimationHandler& xOther);

	AVP_Dropship_AnimationHandler& operator=(const AVP_Dropship_AnimationHandler& xOther);

	void OpenFrontPods();
	void CloseFrontPods();
	void OpenBackPods();
	void CloseBackPods();
	void OpenBackPodsDoors();
	void CloseBackPodsDoors();

	void HandleRequestWithTime(const AVP_Message_Dropship_RequestAnimation& xDropshiprequestData);

	enum OpenCloseState
	{
		OCS_CLOSED,
		OCS_OPENED,
		OCS_OPENING,
		OCS_CLOSING
	};

	OpenCloseState m_eFrontPods;
	OpenCloseState m_eBackPods;
	OpenCloseState m_eBackPodsDoors;
	Asura_Hash_ID m_uFrontPodsAnimIDCloseToOpen;
	Asura_Hash_ID m_uBackPodsAnimIDCloseToOpen;
	Asura_Hash_ID m_uBackPodsDoorsAnimIDCloseToOpen;
	Asura_Hash_ID m_uFrontPodsAnimIDOpenToClose;
	Asura_Hash_ID m_uBackPodsAnimIDOpenToClose;
	Asura_Hash_ID m_uBackPodsDoorsAnimIDOpenToClose;

	Asura_Hash_ID m_uFrontPodsControllerID;
	Asura_Hash_ID m_uBackPodsControllerID;
	Asura_Hash_ID m_uBackPodsDoorsControllerID;
	Asura_Hash_ID m_uFrontPodsControllerGroupID;
	Asura_Hash_ID m_uBackPodsControllerGroupID;
	Asura_Hash_ID m_uBackPodsDoorsControllerGroupID;

	// hold all the animations that exist all the time and are controlled by time
	struct TimeDependentAnimDescriptor 
	{
		Asura_Hash_ID m_uAnimID;
		Asura_Hash_ID m_uControllerID;
		float m_fCurrentTime; // time between 0.0 and 1.0
		float m_fDesiredTime; // time between 0.0 and 1.0
	};

	TimeDependentAnimDescriptor m_xTimeDependentAnim[AVP_Message_Dropship_RequestAnimation::REQUEST_ANIM_LAST_ANIM_WITH_TIME];

};   


#endif   // ifndef __AVP_DROPSHIP_ANIMATIONHANDLER_H__


/// End of file. ///
