
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Vertebrate_CameraHandler.cpp
// Author : Chris Wallace
// Date   : 23-Apr-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "AVP_Vertebrate_CameraHandler.h"

// Asura independent header-files...
#include "Core/Asura_Timers.h"

// Game independent header-files...
#include "Actor/Vertebrates/AVP_Vertebrate_HUDHandler.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_CameraHandler::AVP_Vertebrate_CameraHandler
// Author       : Chris Wallace
// Date         : 23-Apr-2007
////////////////////////////////////////////////////////////////////////////////

AVP_Vertebrate_CameraHandler::AVP_Vertebrate_CameraHandler (const Asura_Guid uOwnerGuid, AVP_Actor_CameraProperties& xCameraProperties)
: AVP_Actor_CameraHandler(uOwnerGuid, xCameraProperties)
, m_eState(AVP_ZOOM_NONE)
{
}

/// End of function 'AVP_Predator_CameraHandler::AVP_Predator_CameraHandler'. ///

void AVP_Vertebrate_CameraHandler::UpdateZoomCamera(AVP_ClientActor_Vertebrate& xClient)
{
	float fTimeSlice = Asura_Timers::GetGameTimeSlice();

	switch(m_eState)
	{
		case AVP_ZOOM_IN:
		{
			m_fCameraFieldOfViewFirstPerson -= (fTimeSlice*m_fZoomSpeed);

			if (m_fCameraFieldOfViewFirstPerson <= m_afFovZoom[m_uZoomLevel])
			{
				m_fCameraFieldOfViewFirstPerson = m_afFovZoom[m_uZoomLevel];
				m_eState = AVP_ZOOM_FINISH;
			}
			break;
		}
		case AVP_ZOOM_OUT:
		{
			m_fCameraFieldOfViewFirstPerson += ((fTimeSlice*m_fZoomSpeed) * 2.0f); // zoom out twice as fast

			if (m_fCameraFieldOfViewFirstPerson > m_afFovZoom[m_uZoomLevel])
			{
				m_fCameraFieldOfViewFirstPerson = m_afFovZoom[m_uZoomLevel];
				m_eState = AVP_ZOOM_FINISH;
			}
			break;
		}
		case AVP_ZOOM_FINISH:
		{
			AVP_Vertebrate_HUDHandler*	pxHUDHandler	= static_cast<AVP_Vertebrate_HUDHandler*>(xClient.GetHUDHandler());

			m_fCameraFieldOfViewFirstPerson	= m_afFovZoom[m_uZoomLevel];
			m_eState						= AVP_ZOOM_NONE;

			if(pxHUDHandler != NULL)
			{
				pxHUDHandler->OnEndZooming(m_uZoomLevel);
			}
			break;
		}
		case AVP_ZOOM_NONE:
		{
			break;
		}
	}
	
	return;
}

bool AVP_Vertebrate_CameraHandler::CanZoomIn()
{
	return (m_uZoomLevel < AVP_CAMERA_NUM_ZOOM_LEVELS - 1);
}

bool AVP_Vertebrate_CameraHandler::CanZoomOut()
{
	return (m_uZoomLevel > 0);
}

void AVP_Vertebrate_CameraHandler::BeginZoomIn( AVP_ClientActor_Vertebrate& xClient, bool bWrap)
{
	if(CanZoomIn())
	{
		SetZoomLevel(xClient, m_uZoomLevel+1);
	}
	else if(bWrap)
	{
		SetZoomLevel(xClient, 0);
	}
}

void AVP_Vertebrate_CameraHandler::BeginZoomOut( AVP_ClientActor_Vertebrate& xClient, bool bWrap)
{
	if(CanZoomOut())
	{
		SetZoomLevel(xClient, m_uZoomLevel-1);
	}
	else if(bWrap)
	{
		SetZoomLevel(xClient, AVP_CAMERA_NUM_ZOOM_LEVELS - 1);
	}
}

void AVP_Vertebrate_CameraHandler::SetZoomLevel( AVP_ClientActor_Vertebrate& xClient, u_int uLevel )
{
	if(uLevel != m_uZoomLevel)
	{
		AVP_Vertebrate_HUDHandler*	pxHUDHandler	= static_cast<AVP_Vertebrate_HUDHandler*>(xClient.GetHUDHandler());

		if(pxHUDHandler != NULL)
		{
			if(m_uZoomLevel < uLevel)
			{
				m_eState = AVP_ZOOM_IN;
				pxHUDHandler->OnBeginZoomingIn(uLevel);
			}
			else
			{
				m_eState = AVP_ZOOM_OUT;
				pxHUDHandler->OnBeginZoomingOut(uLevel);
			}
		}

		m_uZoomLevel = uLevel;
	}
}

void AVP_Vertebrate_CameraHandler::SetCameraType(const CameraType eType)
{
	AVP_ClientActor_Vertebrate*	pxClient		= AVP_ClientActor_Vertebrate::FindEntity(m_uOwnerGuid);
	AVP_Vertebrate_HUDHandler*	pxHUDHandler	= (pxClient == NULL) ? NULL : static_cast<AVP_Vertebrate_HUDHandler*>(pxClient->GetHUDHandler());

	if(pxHUDHandler != NULL)
	{
		pxHUDHandler->OnEndZooming(0);
	}

	PARENT::SetCameraType(eType);
}


/// End of file. ///
