
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Vertebrate_CameraHandler.h
// Author : Chris Wallace
// Date   : 23-Apr-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_VERTEBRATE_CAMERAHANDLER_H__
#define __AVP_VERTEBRATE_CAMERAHANDLER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/AVP_Actor_CameraHandler.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_ClientActor_Vertebrate;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Vertebrate_CameraHandler
// Author       : Andy Tate
// Date         : 16-Oct-2006
//
// Description  : Class to the handle the Predator specific camera manipulation.
////////////////////////////////////////////////////////////////////////////////

class AVP_Vertebrate_CameraHandler : public AVP_Actor_CameraHandler
{
public:
	enum ZoomStates {
		AVP_ZOOM_NONE,
		AVP_ZOOM_IN,
		AVP_ZOOM_OUT,
		AVP_ZOOM_FINISH
	};

	AVP_Vertebrate_CameraHandler				(const Asura_Guid uOwnerGuid, AVP_Actor_CameraProperties& xCameraProperties);
//	virtual ~AVP_Vertebrate_CameraHandler ();

	virtual void	SetCameraType(const CameraType eType);

	void			UpdateZoomCamera(AVP_ClientActor_Vertebrate& xClient);

	bool			CanZoomIn(void);
	virtual bool	CanZoomOut(void);
	void			BeginZoomIn( AVP_ClientActor_Vertebrate& xClient, bool bWrap = false );
	void			BeginZoomOut( AVP_ClientActor_Vertebrate& xClient, bool bWrap = false);
	void			SetZoomLevel( AVP_ClientActor_Vertebrate& xClient, u_int uLevel);
	u_int			GetZoomLevel(void) const	{ return(m_uZoomLevel); }
	void			ResetZoomLevel(void)		{ m_uZoomLevel = 0; }

protected:
	
	ZoomStates m_eState;
	
private:

	typedef AVP_Actor_CameraHandler PARENT;

	AVP_Vertebrate_CameraHandler  (const AVP_Vertebrate_CameraHandler& xOther);
	AVP_Vertebrate_CameraHandler&	operator=	(const AVP_Vertebrate_CameraHandler& xOther);

};   // End of class 'AVP_Vertebrate_CameraHandler'.


#endif   // ifndef __AVP_VERTEBRATE_CAMERAHANDLER_H__


/// End of file. ///
