
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Vertebrate_Focus.h
// Author : Anthony Gregson
// Date   : 31/01/08
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_VERTEBRATE_FOCUS_H__
#define __AVP_VERTEBRATE_FOCUS_H__


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////
#include "Network/Asura_Guid.h"

#include "Core/Asura_LinkList.h"
#include "Core/Asura_Maths.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_ClientActor_Vertebrate;
class AVP_ClientEntity_Actor;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class Focus_Target
{
public:
	Asura_Guid m_uGuid;
	float m_fPriority;
	Asura_Vector_2 m_xScreenPos;

	static float CalculatePriority(Asura_Vector_2 xScreenPos, float fRange, float fScreenFraction, Asura_Guid uTargetGuid, Asura_Guid uPreviouGuid)
	{
		float fPosPri; // priority based on screen position
		float fDistPri; // priority based on distance from player
		float fGuidPri; // priority based on target stickiness

		float fPosScale = 1.0f;
		float fGuidScale = 1.0f;
		float fDistScale = 1.0f;

		// stickiness
		{
			fGuidPri = (uPreviouGuid == uTargetGuid)?1.0f:0.75f;
		}
		fGuidPri *= fGuidScale;

		// screen position
		{
			fScreenFraction*=2.0f; // values passed in are for half a screen
			float fXPri = 1.0f-(Asura_Maths::Abs(xScreenPos.x)*2.0f/fScreenFraction);
			float fYPri = 1.0f-(Asura_Maths::Abs(xScreenPos.y)*2.0f/fScreenFraction);
			fPosPri = fXPri * fYPri;
		}
		fPosPri *= fPosScale;

		// distance
		{
			if(fRange < 3.0f)
			{
				fDistPri = 0.0f; // too close
			}
			else
			{
				fDistPri = 1.0f-((fRange-(3.0f)) / (60.0f-(3.0f))); // magic number, 60m max range
			}
		}
		fDistPri *= fDistScale;
		
		return fPosPri * fDistPri * fGuidPri;
	}
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Vertebrate_Focus
// Author       : Anthony Gregson
// Date         : 31/01/08
////////////////////////////////////////////////////////////////////////////////

class AVP_Vertebrate_Focus
{
public:
	// Public functions...
	AVP_Vertebrate_Focus(AVP_ClientActor_Vertebrate& xOwner);
	virtual ~AVP_Vertebrate_Focus();

	virtual void Update();
	virtual void Render() {}

	// Accessor functions...
	bool GetActive() const { return m_bActive; }
	virtual Asura_Guid GetMeleeTarget() const { return ASURA_GUID_UNREGISTERED; }
	
	virtual bool HasValidLockOnTarget() const	{ return m_bTargetIsLockOn && m_uTargetEnemyGuid != ASURA_GUID_UNREGISTERED; }
	Asura_Guid GetLockOnTarget() const	{ return m_bTargetIsLockOn ? m_uTargetEnemyGuid : ASURA_GUID_UNREGISTERED; }
	Asura_Guid GetTargetGuid() const	{ return m_uTargetEnemyGuid; }
	Asura_Guid GetBestTargetGuid() const	{ return m_uBestTargetGuid; }
	void UnSetLockOnTarget() { m_uTargetEnemyGuid = ASURA_GUID_UNREGISTERED; }

	virtual void PostCreateInitialise() {}

	void SetPounceAttackVariationData(u_int uNumVariations, float* pfMaxDistances, float* pfMinDrops);

	const Asura_LinkList<Focus_Target>* GetTargetList() const { return &m_xTargetList; }

protected:
	// Protected functions...
	virtual void Activate();
	virtual void Deactivate();

	bool GetCanPounceToTarget(const AVP_ClientEntity_Actor& xTarget) const;

	// Protected variables...
	AVP_ClientActor_Vertebrate& m_xOwner;

	Asura_Guid m_uTargetEnemyGuid;
	Asura_Guid m_uBestTargetGuid;

	float m_fTimeActive;
	union
	{
		u_int		m_uFlags;
		struct
		{
			u_int	m_bActive			: 1,
					m_bTargetIsLockOn	: 1,
										: 0;
		};
	};

	struct PounceAttackData
	{
		float m_fMaxDistance;
		float m_fMinDrop;
	};

	u_int m_uNumPounceAttackVariations;
	PounceAttackData* m_pxPounceAttacks;

	static float s_fToggleAutomaticTimeout;
	static bool s_bOnlyUseToggle;

	Asura_LinkList<Focus_Target> m_xTargetList;

private:
	// Private functions...

	// Private variables...
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	static int s_iConsoleVarsCount;
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)

};   // End of class 'AVP_Vertebrate_Focus'.

#endif   // ifndef __AVP_VERTEBRATE_FOCUS_H__


/// End of file. ///
