
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Vertebrate_HUDHandler.cpp
// Author : Andy Tate
// Date   : 4-Oct-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/AVP_Vertebrate_HUDHandler.h"

// Asura independent header-files...
#include "Rendering/Asura_Viewport.h"
#include "FSFX/Asura_FSFX_System.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Slider.h"
#include "GUIMenu/Asura_GUIMenu_Root.h"
#include "GUIMenu/Asura_GUIMenu_MessagePump.h"
#include "GUIMenu/Asura_GUIMenu_Element_Path.h"
#include "Sound/Asura_Sound_Mixer.h"
#include "Sound/Asura_Sound_Event_Storage.h"

// Game independent header-files...
#include "Actor/AVP_Actor_PickupHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_Focus.h"
#include "Actor/Vertebrates/AVP_Vertebrate_InputHandler.h"
#include "Animation/AVP_AnimFlags.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Entity/PlayerInfo/AVP_ClientEntity_PlayerInfo.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Entity/PhysicalObject/Pickup/AVP_ClientEntity_Pickup.h"
#include "Entity/Physicalobject/Pickup/AVP_Pickup.h"
#include "GUIMenu/AVP_GUIMenu_System.h"

// hack
#include "GUIMenu/AVP_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Element_Image.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Moves/AVP_Moves.h"

// Axon
#include "Tactical/AVP_Tactical_System.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

#define HUD_SHOWACTIVEGRAPPLEBAR	L"AVP.HUD.ShowActiveGrappleBar"
#define HUD_SHOWACTIVEGRAPPLESLIDER L"AVP.HUD.ShowActiveGrappleMarker"
#define HUD_FLASHACTIVEGRAPPLEBAR	L"AVP.HUD.FlashActiveGrappleBar"
#define HUD_PICKUPINRANGE			L"AVP.HUD.PickupInRange"
#define HUD_FOCUSMODEACTIVE			L"AVP.HUD.FocusModeActive"

#define HUD_WEAPONSELECTOR_DISPCOND_TOP			L"AVP.WeaponSelector.Top"
#define HUD_WEAPONSELECTOR_DISPCOND_TOPRIGHT	L"AVP.WeaponSelector.TopRight"
#define HUD_WEAPONSELECTOR_DISPCOND_RIGHT		L"AVP.WeaponSelector.Right"
#define HUD_WEAPONSELECTOR_DISPCOND_BOTTOMRIGHT L"AVP.WeaponSelector.BottomRight"
#define HUD_WEAPONSELECTOR_DISPCOND_BOTTOM		L"AVP.WeaponSelector.Bottom"
#define HUD_WEAPONSELECTOR_DISPCOND_BOTTOMLEFT	L"AVP.WeaponSelector.BottomLeft"
#define HUD_WEAPONSELECTOR_DISPCOND_LEFT		L"AVP.WeaponSelector.Left"
#define HUD_WEAPONSELECTOR_DISPCOND_TOPLEFT		L"AVP.WeaponSelector.TopLeft"
#define HUD_WEAPONSELECTOR_DISPCOND_CURRENTSLOT L"AVP.WeaponSelector.SelectedWeaponSlot"
#define HUD_WEAPONSELECTOR_DISPCOND_CURRENT		L"AVP.HUD.CurrentWeaponDispCondIndex"

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

bool AVP_Vertebrate_HUDHandler::s_bGrabTargetInRange = false;
bool AVP_Vertebrate_HUDHandler::s_bPounceTargetInRange = false;
bool AVP_Vertebrate_HUDHandler::s_bShowReticle = false;
float AVP_Vertebrate_HUDHandler::s_fFocusModeActive = 0.0f;

static bool s_bHUDPickupInRange = false;
static bool s_bShowActiveGrappleBar = false;
static bool s_bShowActiveGrappleSlider = false;
static u_int s_uFlashActiveGrappleBar = AVP_VERTEBRATE_ACTIVEGRAPPLE_RESULT_UNDECIDED;
static float s_fActiveGrappleSliderPos = 0.0f;
static float s_fActiveGrappleCriticalSuccessStart = 0.0f;
static float s_fActiveGrappleCriticalSuccessWidth = 0.0f;
static float s_fActiveGrappleSuccessStart = 0.0f;
static float s_fActiveGrappleSuccessWidth = 0.0f;

static const float g_fMinReticleSize = 24.0f;

static const AVP_GUIMenu_System::STORED_WIDGETS g_auReticleElements[ 5 ] = 
{
	AVP_GUIMenu_System::RETICLE_CENTER,
	AVP_GUIMenu_System::RETICLE_UP,
	AVP_GUIMenu_System::RETICLE_RIGHT,
	AVP_GUIMenu_System::RETICLE_DOWN,
	AVP_GUIMenu_System::RETICLE_LEFT
};

u_int AVP_Vertebrate_HUDHandler::s_auWeaponSelectorDispCondIndex[8] =
{
	0,0,0,0,0,0,0,0
};

u_int AVP_Vertebrate_HUDHandler::s_uCurrentWeaponDispCondIndex = 0;
u_int AVP_Vertebrate_HUDHandler::s_uCurrentWeaponDispCondSlot = 0;

static const float g_afReticleXOffset[ 5 ] = 
{
	0.0f,
	0.0f,
	1.0f,
	0.0f,
	-1.0f
};

static const float g_afReticleYOffset[ 5 ] = 
{
	0.0f,
	-1.0f,
	0.0f,
	1.0f,
	0.0f
};

// Blue color reticule
static const Asura_RGBA g_xReticuleVoidLook [ 5 ] =
{
	Asura_RGBA(0.32f,0.45f,0.8f,1.0f),
	Asura_RGBA(0.32f,0.45f,0.8f,1.0f),
	Asura_RGBA(0.32f,0.45f,0.8f,1.0f),
	Asura_RGBA(0.32f,0.45f,0.8f,1.0f),
	Asura_RGBA(0.32f,0.45f,0.8f,1.0f)
};

// Green color reticule
static const Asura_RGBA g_xReticuleFriendLook [ 5 ] =
{
	Asura_RGBA(0.6f,0.8f,0.6f,1.0f),
	Asura_RGBA(0.6f,0.8f,0.6f,1.0f),
	Asura_RGBA(0.6f,0.8f,0.6f,1.0f),
	Asura_RGBA(0.6f,0.8f,0.6f,1.0f),
	Asura_RGBA(0.6f,0.8f,0.6f,1.0f)
};

// Red color reticule
static const Asura_RGBA g_xReticuleEnemyLook [ 5 ] =
{
	Asura_RGBA(1.0f,0.0f,0.0f,1.0f),
	Asura_RGBA(1.0f,0.0f,0.0f,1.0f),
	Asura_RGBA(1.0f,0.0f,0.0f,1.0f),
	Asura_RGBA(1.0f,0.0f,0.0f,1.0f),
	Asura_RGBA(1.0f,0.0f,0.0f,1.0f)
};

DEBUGVAR float dbg_fActiveGrappleDefault_CriticalSuccessStart = 0.95f;
DEBUGVAR float dbg_fActiveGrappleDefault_CriticalSuccessEnd = 1.0f;
DEBUGVAR float dbg_fActiveGrappleDefault_SuccessStart = 0.8f;
DEBUGVAR float dbg_fActiveGrappleDefault_SuccessEnd = 1.0f;
DEBUGVAR float dbg_fActiveGrappleDefault_Duration = 1.0f;
DEBUGVAR float dbg_fActiveGrappleDefault_InitialDelay = 0.5f;
DEBUGVAR float dbg_fActiveGrappleDefault_CompletedDelay = 0.5f;

DEBUGVAR float dbg_fActiveGrapple_MultiplayerResultWaitTime = 1.0f;

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_HUDHandler::AVP_Vertebrate_HUDHandler
// Author       : Andy Tate
// Date         : 4-Oct-2007
////////////////////////////////////////////////////////////////////////////////

AVP_Vertebrate_HUDHandler::AVP_Vertebrate_HUDHandler(AVP_ClientEntity_Actor& xClient)
: PARENT(xClient)
, m_xDDI( xClient )
, m_fReticleFadeout(1.0f)
, m_iAdrenalineLoopHandle(ASURA_INVALID_HANDLE)
, m_iBreathingLoopHandle(ASURA_INVALID_HANDLE)
, m_iMeleeJumpLoopSoundHandle(ASURA_INVALID_HANDLE)
, m_iFocusSoundLoopHandle(ASURA_INVALID_HANDLE)
, m_fActiveGrapple_BarSliderTimer(0.0f)
, m_fActiveGrapple_BarSliderPosSendToServer(0.0f)
, m_fActiveGrapple_CriticalSuccess_Start(dbg_fActiveGrappleDefault_CriticalSuccessStart)
, m_fActiveGrapple_CriticalSuccess_End(dbg_fActiveGrappleDefault_CriticalSuccessEnd)
, m_fActiveGrapple_Success_Start(dbg_fActiveGrappleDefault_SuccessStart)
, m_fActiveGrapple_Success_End(dbg_fActiveGrappleDefault_SuccessEnd)
, m_fActiveGrapple_Duration(dbg_fActiveGrappleDefault_Duration)
, m_fActiveGrapple_InitialDelay(dbg_fActiveGrappleDefault_InitialDelay)
, m_fActiveGrapple_CompletedDelay(dbg_fActiveGrappleDefault_CompletedDelay)
, m_fActiveGrapple_MultiplayerResultWaitTimer(0.0f)
, m_uActiveGrapple_StartGrappleSoundEvent(ASURA_HASH_ID_UNSET)
, m_uActiveGrapple_StartMarkerSoundEvent(ASURA_HASH_ID_UNSET)
, m_uActiveGrapple_WinSoundEvent(ASURA_HASH_ID_UNSET)
, m_uActiveGrapple_LoseSoundEvent(ASURA_HASH_ID_UNSET)
, m_uSoundMixerPreset( ASURA_HASH_ID_UNSET )
, m_eStoredActiveGrappleResult(AVP_VERTEBRATE_ACTIVEGRAPPLE_RESULT_UNDECIDED)
, m_uFlags(0)
{
}

AVP_Vertebrate_HUDHandler::~AVP_Vertebrate_HUDHandler()
{
	StopSoundEvent(m_iAdrenalineLoopHandle);
	StopSoundEvent(m_iBreathingLoopHandle);
	StopSoundEvent(m_iMeleeJumpLoopSoundHandle);
	StopSoundEvent(m_iFocusSoundLoopHandle);

	// If we're displaying the grapple bar, clean it up. This only works because we don't have splitscreen
	s_bShowActiveGrappleBar = false;
	s_bShowActiveGrappleSlider = false;
}

DEBUGVAR float dbg_fFocusFadeTime = 0.3f;

/// End of function 'AVP_Vertebrate_HUDHandler::AVP_Vertebrate_HUDHandler'. ///
void AVP_Vertebrate_HUDHandler::Update( const AVP_ClientEntity_Actor& xClient )
{
	PARENT::Update( xClient );
	
	m_xDDI.Update();

	////////////////// RETICLE /////////////////
	const AVP_Inventory_System*	pxInventorySystem	= xClient.GetInventorySystem();
	const AVP_InventoryItem_Weapon*	pxCurrentItem		= (pxInventorySystem == NULL) ? NULL : static_cast< const AVP_InventoryItem_Weapon* >( pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) );

	float fSize;

	if(pxCurrentItem)
	{
		if ( pxCurrentItem->GetHasCrossHair() && !pxCurrentItem->IsEncumberment() )
		{
			m_fReticleFadeout = 1.0f - pxCurrentItem->GetSpreadParametric();
			fSize = Asura_Maths::Max( g_fMinReticleSize, Asura_Viewport::GetWidth() * Asura_Maths::Lerp( pxCurrentItem->GetPrimaryMaxSpread(), pxCurrentItem->GetPrimaryMinSpread(), m_fReticleFadeout ) / Asura_Camera::GetHorizontalFieldOfView() );
		}
		else
		{
			m_fReticleFadeout = 0.0f;
			fSize = 64.0f;
		}
	}
	else
	{
		m_fReticleFadeout = 1.0f;
		fSize = 64.0f;
	}

	float fCenterX = 0.0f;
	float fCenterY = 0.0f;
	GetReticlePosition(fCenterX, fCenterY);

	AVP_ClientEntity_PlayerInfo* pxPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity( AVP_Game_Core::GetPlayerGuid() ); 
	if( pxPlayerInfo )
	{
		const Asura_RGBA* xRetirculeColourArray;
		
		// Get color array
		if ( (xRetirculeColourArray = GetReticuleColorArray(pxPlayerInfo->GetTargetGuid())) == NULL )
		{
			// problem here, put the reticule at default white color
			xRetirculeColourArray = g_xReticuleVoidLook; 
		}

		Asura_GUIMenu_Root* pxReticle = static_cast<Asura_GUIMenu_Root*>(AVP_GUIMenu_System::GetHUDElement( AVP_GUIMenu_System::RETICLE_GROUP ) );
		if(pxReticle)
		{
			Asura_Bounding_Rectangle xBounds(pxReticle->GetScaledWorldBounds());
			Asura_Vector_2 xNewPos;
			xNewPos.x = fCenterX - (xBounds.GetWidth() / 2.0f);
			xNewPos.y = fCenterY - (xBounds.GetHeight() / 2.0f);
			xBounds.Displace(xNewPos - xBounds.GetTopLeft());
			pxReticle->SetScaledWorldBounds( xBounds );
		}

		for ( u_int uItr = 0; uItr < 5; ++uItr )
		{
			Asura_GUIMenu_Element_Image* pxImageItem = static_cast<Asura_GUIMenu_Element_Image*>(AVP_GUIMenu_System::GetHUDElement( g_auReticleElements[ uItr ] ) );
			if( pxImageItem )
			{
				pxImageItem->SetColour(xRetirculeColourArray[uItr].m_fR,xRetirculeColourArray[uItr].m_fG,xRetirculeColourArray[uItr].m_fB, m_fReticleFadeout);
				Asura_Bounding_Rectangle xBounds(pxImageItem->GetScaledWorldBounds());
				Asura_Vector_2 xNewPos;
				xNewPos.x = fCenterX - (xBounds.GetWidth() / 2.0f) + fSize * g_afReticleXOffset[ uItr ];
				xNewPos.y = fCenterY - (xBounds.GetHeight() / 2.0f) + fSize * g_afReticleYOffset[ uItr ];
				xBounds.Displace(xNewPos - xBounds.GetTopLeft());
				pxImageItem->SetScaledWorldBounds( xBounds );
			}
		}
	}
	////////////////// END RETICLE /////////////////

	const AVP_ClientActor_Vertebrate&	xVert					= static_cast< const AVP_ClientActor_Vertebrate& >( xClient );
	const float							fHealthParametric		= xVert.GetHealth() / xVert.GetInitialHealth();
	//const float							fLowHealthParametric	= 1.0f - (fHealthParametric / xVert.GetHealthDangerFraction());

	if(!AVP_Settings::GetGodMode())
	{
		const Asura_Blueprint*	pxBlueprint	= GetPlayerBlueprint();
		if( pxBlueprint )
		{
			Asura_Value	xValueH1, xValueH2, xValueH3;

			if( pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEALTH1_PARAMETRIC, xValueH1) )
			{
				if ( pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEALTH2_PARAMETRIC, xValueH2) )
				{
					if ( pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEALTH3_PARAMETRIC, xValueH3) )
					{
						Asura_Value	xValueS;
						if ( fHealthParametric <= xValueH1.GetFloatValue() && fHealthParametric > xValueH2.GetFloatValue() )
						{
							ResetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH2_FSFX);
							ResetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH3_FSFX);
							if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEALTH1_FSFXSTRENGTH, xValueS))
							{
								const float	fStrength = xValueS.GetFloatValue() * ( fHealthParametric / xValueH1.GetFloatValue() );
								SetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH1_FSFX, fStrength, 0.0f);
							}
							// Play DSP effect
							if ( pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEALTH1_SOUNDMIXERPRESET, xValueS))
							{
								if ( xValueS.GetHashIDValue() != ASURA_HASH_ID_UNSET && xValueS.GetHashIDValue() != m_uSoundMixerPreset )
								{
									if ( m_uSoundMixerPreset != ASURA_HASH_ID_UNSET )
									{
										Asura_Sound_Mixer_System::UnqueuePreset ( m_uSoundMixerPreset, 0.0f );
									}
									Asura_Sound_Mixer_System::QueuePreset   ( xValueS.GetHashIDValue(), 0.0f );
                                    m_uSoundMixerPreset = xValueS.GetHashIDValue(); 
								}
							}
						}
						else if ( fHealthParametric <= xValueH2.GetFloatValue() && fHealthParametric > xValueH3.GetFloatValue() )
						{
							ResetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH1_FSFX);
							ResetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH3_FSFX);
							if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEALTH2_FSFXSTRENGTH, xValueS))
							{
								const float	fStrength = xValueS.GetFloatValue() * ( fHealthParametric / xValueH2.GetFloatValue() );
								SetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH2_FSFX, fStrength, 0.0f);
							}
							// Play DSP effect
							if ( pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEALTH2_SOUNDMIXERPRESET, xValueS))
							{
								if ( xValueS.GetHashIDValue() != ASURA_HASH_ID_UNSET && xValueS.GetHashIDValue() != m_uSoundMixerPreset )
								{
									if ( m_uSoundMixerPreset != ASURA_HASH_ID_UNSET )
									{
										Asura_Sound_Mixer_System::UnqueuePreset ( m_uSoundMixerPreset, 0.0f );
									}
									Asura_Sound_Mixer_System::QueuePreset   ( xValueS.GetHashIDValue(), 0.0f );
									m_uSoundMixerPreset = xValueS.GetHashIDValue(); 
								}
							}
						}
						else if ( fHealthParametric <= xValueH3.GetFloatValue() )
						{
							ResetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH1_FSFX);
							ResetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH2_FSFX);
							if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEALTH3_FSFXSTRENGTH, xValueS))
							{
								const float	fStrength = xValueS.GetFloatValue() * ( fHealthParametric / xValueH3.GetFloatValue() );
								SetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH3_FSFX, fStrength, 0.0f);
							}
							// Play DSP effect
							if ( pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEALTH3_SOUNDMIXERPRESET, xValueS))
							{
								if ( xValueS.GetHashIDValue() != ASURA_HASH_ID_UNSET && xValueS.GetHashIDValue() != m_uSoundMixerPreset )
								{
									if ( m_uSoundMixerPreset != ASURA_HASH_ID_UNSET )
									{
										Asura_Sound_Mixer_System::UnqueuePreset ( m_uSoundMixerPreset, 0.0f );
									}
									Asura_Sound_Mixer_System::QueuePreset   ( xValueS.GetHashIDValue(), 0.0f );
									m_uSoundMixerPreset = xValueS.GetHashIDValue(); 
								}
							}
						}
						else
						{
							ResetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH1_FSFX);
							ResetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH2_FSFX);
							ResetFSFXValue(FSFX_LOWHEALTH, uAVP_BLUEPRINT_PARAM_HEALTH3_FSFX);
							if ( m_uSoundMixerPreset != ASURA_HASH_ID_UNSET  )
							{
								Asura_Sound_Mixer_System::UnqueuePreset ( m_uSoundMixerPreset, 0.0f );
								m_uSoundMixerPreset = ASURA_HASH_ID_UNSET;
							}
						}
					}
				}
			}
		}
	}

	if((m_iBreathingLoopHandle == ASURA_INVALID_HANDLE) && !xVert.IsDead() && AVP_Game_Core::IsLocalPlayer(m_uOwnerGuid))
	{
		PlaySoundEvent(uAVP_BLUEPRINT_PARAM_BREATHING_SOUND, &m_iBreathingLoopHandle);
	}

	UpdateActiveGrapple( xClient );

	UpdateHudFlags( xClient );
}

void AVP_Vertebrate_HUDHandler::SetHealthDSP ( bool bOn ) const
{
	if ( m_uSoundMixerPreset != ASURA_HASH_ID_UNSET )
	{
		if ( bOn)
		{
			Asura_Sound_Mixer_System::QueuePreset   ( m_uSoundMixerPreset, 0.0f );
		}
		else
		{
			Asura_Sound_Mixer_System::UnqueuePreset ( m_uSoundMixerPreset, 0.0f );
		}
	}
}

void AVP_Vertebrate_HUDHandler::UpdateHudFlags( const AVP_ClientEntity_Actor& xClient )
{
	// Update HUD flag which will say, if the pickup is near or not.
	const AVP_Actor_PickupHandler* pxPickupHandler = xClient.GetPickupHandler();

 	if( pxPickupHandler )
	{
		Asura_Guid uPickupGuid = pxPickupHandler->GetNearestPickupGuid();
		s_bHUDPickupInRange = uPickupGuid!=ASURA_GUID_UNREGISTERED ? true:false;
		if( s_bHUDPickupInRange )
		{
			const AVP_ClientEntity_Pickup* pxPickup =  AVP_ClientEntity_Pickup::FindEntity( uPickupGuid );
			if( pxPickup && (pxPickup->GetPickupObject()->GetPickupType() == AVP_Pickup::PICKUP_TYPE_WEAPON) )
			{
				if( !pxPickupHandler->CanCurrentWeaponBeDropped() && !pxPickupHandler->IsSpaceForNewWeapon() )
				{
					s_bHUDPickupInRange = false;
				}
			}
		}
	}
	else
	{
		s_bHUDPickupInRange = false;
	}

	// Update current weapon
	const AVP_Inventory_System* pxSystem = xClient.GetInventorySystem();
	if(pxSystem)
	{
		const AVP_Inventory_Item* pxCurrentItem = pxSystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON);
		if(pxCurrentItem)
		{
			s_uCurrentWeaponDispCondIndex = pxCurrentItem->GetSelectorDisplayConditionIndex();
		}
		else
		{
			s_uCurrentWeaponDispCondIndex = 0;
		}
	}

	// Update the reticle - this is normally on, but can be disabled by interactions and grapples.
	s_bShowReticle = true;
	const Asura_Movement_System* pxMovementSystem = xClient.GetMovementSystem();
	if (pxMovementSystem)
	{
		const AVP_Move* pxMove = static_cast<const AVP_Move*>(pxMovementSystem->GetCurrentMove());
		if (pxMove)
		{
			s_bShowReticle = !pxMove->GetHideReticle();
		}
	}
}

const Asura_RGBA* AVP_Vertebrate_HUDHandler::GetReticuleColorArray(Asura_Guid uTargetGuid)
{
    if(uTargetGuid == ASURA_GUID_UNREGISTERED) return g_xReticuleVoidLook;

	const AVP_ClientEntity_Actor* pxTarget = AVP_ClientEntity_Actor::FindEntity(uTargetGuid);
	
	if(pxTarget)
	{
		 
		if(  AVP_Tactical_System::GetAllegiance(m_xOwner, *pxTarget) == AVP_ALLEGIANCE_FRIEND ) 
		{
			return g_xReticuleFriendLook;
		}
		else
		{
			return g_xReticuleEnemyLook;
		}
	}
	
	return NULL;
}

void AVP_Vertebrate_HUDHandler::OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage) 
{ 
	PARENT::OnApplyDamage( xDamage );
	m_xDDI.Add(xDamage);

//	if ( xDamage.GetAmount() > 0.0f )
//	{
//
//		AVP_ClientEntity_PhysicalObject* pxFrom = AVP_ClientEntity_PhysicalObject::FindEntity(xDamage.GetFromGuid());
//		if(pxFrom)
//		{
//			AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity(m_uOwnerGuid);
//			if(pxClient)
//			{
//				m_xDDI.Add(pxFrom->GetCurrentPosition());
//			}
//		}
//	}
}

void AVP_Vertebrate_HUDHandler::GetReticlePosition(float& fX, float& fY)
{
	fX = Asura_Viewport::GetWidth() / 2.0f;
	fY = Asura_Viewport::GetHeight() / 2.0f;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_HUDHandler::OnBeginJump
// Author       : Andy Tate
// Date         : 1-Jul-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Vertebrate_HUDHandler::OnBeginJump(void)
{
}

/// End of function 'AVP_Vertebrate_HUDHandler::OnBeginJump'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_HUDHandler::OnBeginMeleeJump
// Author       : Andy Tate
// Date         : 30-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Vertebrate_HUDHandler::OnBeginMeleeJump(void)
{
	SetFSFXValue(FSFX_MELEEJUMP, uAVP_BLUEPRINT_PARAM_MELEEJUMP_FSFX, 0.5f, 0.5f);
	PlaySoundEvent(uAVP_BLUEPRINT_PARAM_MELEEJUMP_SOUND_LOOP, &m_iMeleeJumpLoopSoundHandle);
}

/// End of function 'AVP_Actor_HUDHandler::OnBeginMeleeJump'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_HUDHandler::OnEndMeleeJump
// Author       : Andy Tate
// Date         : 30-Jun-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Vertebrate_HUDHandler::OnEndMeleeJump(void)
{
	SetFSFXValue(FSFX_MELEEJUMP, uAVP_BLUEPRINT_PARAM_MELEEJUMP_FSFX, 0.0f, 0.5f);
	StopSoundEvent(m_iMeleeJumpLoopSoundHandle);
}

/// End of function 'AVP_Vertebrate_HUDHandler::OnEndMeleeJump'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_HUDHandler::OnBeginFocus
// Author       : Andy Tate
// Date         : 1-Jul-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Vertebrate_HUDHandler::OnBeginFocus(void)
{
	SetFSFXValue(FSFX_FOCUS, uAVP_BLUEPRINT_PARAM_FOCUS_FSFX, 1.0f, dbg_fFocusFadeTime);
	PlaySoundEvent(uAVP_BLUEPRINT_PARAM_FOCUS_SOUND_BEGIN, NULL);
	PlaySoundEvent(uAVP_BLUEPRINT_PARAM_FOCUS_SOUND_LOOP, &m_iFocusSoundLoopHandle);

	s_fFocusModeActive = 1.0f;
}
/// End of function 'AVP_Vertebrate_HUDHandler::OnBeginFocus'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_HUDHandler::OnEndFocus
// Author       : Andy Tate
// Date         : 1-Jul-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_Vertebrate_HUDHandler::OnEndFocus(void)
{
	SetFSFXValue(FSFX_FOCUS, uAVP_BLUEPRINT_PARAM_FOCUS_FSFX, 0.0f, dbg_fFocusFadeTime);
	PlaySoundEvent(uAVP_BLUEPRINT_PARAM_FOCUS_SOUND_END, NULL);
	StopSoundEvent(m_iFocusSoundLoopHandle);

	s_fFocusModeActive = 0.0f;
}
/// End of function 'AVP_Vertebrate_HUDHandler::OnEndFocus'. ///



///------------------------------------------------------------------------------------------------------------------------------------------------------
///------------------------------------------------------------------------------------------------------------------------------------------------------
///------------------------------------------------------------------------------------------------------------------------------------------------------

void AVP_Vertebrate_HUDHandler::StartActiveGrapple( Asura_Hash_ID uActiveGrappleBlueprint, Asura_Hash_ID uSoundsBlueprint, Asura_Hash_ID uActiveGrappleBlueprintType )	
{
	if (uActiveGrappleBlueprint == ASURA_HASH_ID_UNSET)
	{
		return;
	}

	m_fActiveGrapple_BarSliderTimer				= 0.0f;
	m_fActiveGrapple_BarSliderPosSendToServer	= 0.0f;
	m_fActiveGrapple_MultiplayerResultWaitTimer = 0.0f;
	m_bActiveGrapple_IsWaitingForUserInput		= true;
	m_bActiveGrapple_ResultFinalised			= false;
	m_eStoredActiveGrappleResult				= AVP_VERTEBRATE_ACTIVEGRAPPLE_RESULT_UNDECIDED;
	m_uActiveGrapple_StartGrappleSoundEvent		= ASURA_HASH_ID_UNSET;
	m_uActiveGrapple_StartMarkerSoundEvent		= ASURA_HASH_ID_UNSET;
	m_uActiveGrapple_WinSoundEvent				= ASURA_HASH_ID_UNSET;
	m_uActiveGrapple_LoseSoundEvent				= ASURA_HASH_ID_UNSET;
	s_uFlashActiveGrappleBar					= AVP_VERTEBRATE_ACTIVEGRAPPLE_RESULT_UNDECIDED;
	s_bShowActiveGrappleSlider					= false;

	// Set up some reasonable defaults in case the blueprint doesn't exist.
	m_fActiveGrapple_CriticalSuccess_Start = dbg_fActiveGrappleDefault_CriticalSuccessStart;
	m_fActiveGrapple_CriticalSuccess_End = dbg_fActiveGrappleDefault_CriticalSuccessEnd;
	m_fActiveGrapple_Success_Start = dbg_fActiveGrappleDefault_SuccessStart;
	m_fActiveGrapple_Success_End = dbg_fActiveGrappleDefault_SuccessEnd;
	m_fActiveGrapple_Duration = dbg_fActiveGrappleDefault_Duration;
	m_fActiveGrapple_InitialDelay = dbg_fActiveGrappleDefault_InitialDelay;
	m_fActiveGrapple_CompletedDelay = dbg_fActiveGrappleDefault_CompletedDelay;

	Asura_Value xValue;
	const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uActiveGrappleBlueprintType, uActiveGrappleBlueprint );
	Asura_Assert(pxBlueprint, "Failed to find active grapple blueprint, has the common file been updated?");
	if (pxBlueprint)
	{
		if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_CRITICALSUCCESS_START, xValue))
		{
			m_fActiveGrapple_CriticalSuccess_Start = xValue.GetFloatValue();
		}

		if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_CRITICALSUCCESS_END, xValue))
		{
			m_fActiveGrapple_CriticalSuccess_End = xValue.GetFloatValue();
		}

		if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_SUCCESS_START, xValue))
		{
			m_fActiveGrapple_Success_Start = xValue.GetFloatValue();
		}

		if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_SUCCESS_END, xValue))
		{
			m_fActiveGrapple_Success_End = xValue.GetFloatValue();
		}

		if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_DURATION, xValue))
		{
			m_fActiveGrapple_Duration = xValue.GetFloatValue();
		}

		if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_INITIALDELAY, xValue))
		{
			m_fActiveGrapple_InitialDelay = xValue.GetFloatValue();
		}

		if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_COMPLETEDDELAY, xValue))
		{
			m_fActiveGrapple_CompletedDelay = xValue.GetFloatValue();
		}

		if (uSoundsBlueprint == ASURA_HASH_ID_UNSET)
		{
			// Old-style AG, sounds contained in the AG blueprint
			if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_SOUND_GRAPPLESTART, xValue))
			{
				m_uActiveGrapple_StartGrappleSoundEvent = xValue.GetHashIDValue();
			}

			if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_SOUND_MARKERSTART, xValue))
			{
				m_uActiveGrapple_StartMarkerSoundEvent = xValue.GetHashIDValue();
			}

			if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_SOUND_WIN, xValue))
			{
				m_uActiveGrapple_WinSoundEvent = xValue.GetHashIDValue();
			}

			if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ACTIVEGRAPPLE_SOUND_LOSE, xValue))
			{
				m_uActiveGrapple_LoseSoundEvent = xValue.GetHashIDValue();
			}
		}
	}

	if (uSoundsBlueprint != ASURA_HASH_ID_UNSET)
	{
		// New-style AG, sounds contained in their own blueprint
		pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_GRAPPLEEFFECTS, uSoundsBlueprint );
		if (pxBlueprint)
		{
			if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_GRAPPLESOUNDS_GRAPPLESTART, xValue))
			{
				m_uActiveGrapple_StartGrappleSoundEvent = xValue.GetHashIDValue();
			}

			if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_GRAPPLESOUNDS_MARKERSTART, xValue))
			{
				m_uActiveGrapple_StartMarkerSoundEvent = xValue.GetHashIDValue();
			}

			if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_GRAPPLESOUNDS_WIN, xValue))
			{
				m_uActiveGrapple_WinSoundEvent = xValue.GetHashIDValue();
			}

			if (pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_GRAPPLESOUNDS_LOSE, xValue))
			{
				m_uActiveGrapple_LoseSoundEvent = xValue.GetHashIDValue();
			}
		}
	}

	// Security check. It is possible that designer can swap values by accident, so make sure they are in correct order.
	if( m_fActiveGrapple_CriticalSuccess_Start > m_fActiveGrapple_CriticalSuccess_End )
	{
		float fStore = m_fActiveGrapple_CriticalSuccess_Start;
		m_fActiveGrapple_CriticalSuccess_Start = m_fActiveGrapple_CriticalSuccess_End;
		m_fActiveGrapple_CriticalSuccess_End = fStore;
	}
	if( m_fActiveGrapple_Success_Start > m_fActiveGrapple_Success_End )
	{
		float fStore = m_fActiveGrapple_Success_Start;
		m_fActiveGrapple_Success_Start = m_fActiveGrapple_Success_End;
		m_fActiveGrapple_Success_End = fStore;
	}

	// Initialise menu element positions
	s_fActiveGrappleSliderPos = 0.0f;
	s_fActiveGrappleCriticalSuccessStart = m_fActiveGrapple_CriticalSuccess_Start;
	s_fActiveGrappleCriticalSuccessWidth = m_fActiveGrapple_CriticalSuccess_End - m_fActiveGrapple_CriticalSuccess_Start;
	s_fActiveGrappleSuccessStart = m_fActiveGrapple_Success_Start;
	s_fActiveGrappleSuccessWidth = m_fActiveGrapple_Success_End - m_fActiveGrapple_Success_Start;

	const Asura_Sound_Event* pxStartSoundEvent = Asura_Sound_Event_Storage::FindEvent(m_uActiveGrapple_StartGrappleSoundEvent);
	if (pxStartSoundEvent)
	{
		Asura_Sound_Event_System::PlayEvent(pxStartSoundEvent, NULL);
	}

	SetShowActiveGrappleBar(true);
}

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
void AVP_Vertebrate_HUDHandler::CreateConsoleVars()
{
	////////////////////////////////////////////////////////////////////////////////
	// Static variable declarations...
	////////////////////////////////////////////////////////////////////////////////
	Asura_CommandConsole::AddVar( HUD_PICKUPINRANGE, &s_bHUDPickupInRange, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( HUD_SHOWACTIVEGRAPPLEBAR, &s_bShowActiveGrappleBar, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( HUD_SHOWACTIVEGRAPPLESLIDER, &s_bShowActiveGrappleSlider, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar( HUD_FLASHACTIVEGRAPPLEBAR, &s_uFlashActiveGrappleBar, 0, AVP_VERTEBRATE_ACTIVEGRAPPLE_NUMRESULTS, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::AddVar(L"AVP.HUD.ActiveGrappleMarkerPosition", &s_fActiveGrappleSliderPos, 0.0f, 1.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar(L"AVP.HUD.ActiveGrappleHotSpotStart", &s_fActiveGrappleCriticalSuccessStart, 0.0f, 1.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar(L"AVP.HUD.ActiveGrappleHotSpotWidth", &s_fActiveGrappleCriticalSuccessWidth, 0.0f, 1.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar(L"AVP.HUD.ActiveGrappleWarmSpotStart", &s_fActiveGrappleSuccessStart, 0.0f, 1.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar(L"AVP.HUD.ActiveGrappleWarmSpotWidth", &s_fActiveGrappleSuccessWidth, 0.0f, 1.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);

	Asura_CommandConsole::AddVar( HUD_FOCUSMODEACTIVE, &s_fFocusModeActive, 0.0f, 1.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_FOCUSMODEACTIVE );

	Asura_CommandConsole::AddVar( HUD_WEAPONSELECTOR_DISPCOND_TOP,			&s_auWeaponSelectorDispCondIndex[0], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( HUD_WEAPONSELECTOR_DISPCOND_TOPRIGHT,		&s_auWeaponSelectorDispCondIndex[1], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( HUD_WEAPONSELECTOR_DISPCOND_RIGHT,		&s_auWeaponSelectorDispCondIndex[2], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( HUD_WEAPONSELECTOR_DISPCOND_BOTTOMRIGHT,	&s_auWeaponSelectorDispCondIndex[3], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( HUD_WEAPONSELECTOR_DISPCOND_BOTTOM,		&s_auWeaponSelectorDispCondIndex[4], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( HUD_WEAPONSELECTOR_DISPCOND_BOTTOMLEFT,	&s_auWeaponSelectorDispCondIndex[5], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( HUD_WEAPONSELECTOR_DISPCOND_LEFT,			&s_auWeaponSelectorDispCondIndex[6], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( HUD_WEAPONSELECTOR_DISPCOND_TOPLEFT,		&s_auWeaponSelectorDispCondIndex[7], 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( HUD_WEAPONSELECTOR_DISPCOND_CURRENT,		&s_uCurrentWeaponDispCondIndex,		 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
	Asura_CommandConsole::AddVar( HUD_WEAPONSELECTOR_DISPCOND_CURRENTSLOT,	&s_uCurrentWeaponDispCondSlot,		 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);


	Asura_CommandConsole::AddVar( L"AVP.HUD.GrabTargetInRange", &s_bGrabTargetInRange, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( L"AVP.HUD.GrabTargetInRange" );
	Asura_CommandConsole::AddVar( L"AVP.HUD.PounceTargetInRange", &s_bPounceTargetInRange, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( L"AVP.HUD.PounceTargetInRange" );
	Asura_CommandConsole::AddVar( L"AVP.HUD.ShowCrosshair", &s_bShowReticle, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( L"AVP.HUD.ShowCrosshair" );
}
#endif //#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)

void AVP_Vertebrate_HUDHandler::UpdateActiveGrapple( const AVP_ClientEntity_Actor& xClient )
{
	// Is melee bar visible
	if( s_bShowActiveGrappleBar )
	{
		// End early if the grapple has been aborted.
		bool bGrappleRunning = !xClient.IsRagdoll();
		if (bGrappleRunning)
		{
			// Also find out if the other side is ragdolled.
			const Asura_Movement_System* pxMovementSystem = xClient.GetMovementSystem();
			if (pxMovementSystem)
			{
				const AVP_Move* pxMove = static_cast<const AVP_Move*>(pxMovementSystem->GetCurrentMove());
				if (pxMove)
				{
					const Asura_Guid uMeleeTarget = pxMove->GetForcedMeleeTarget();
					const AVP_ClientEntity_Actor* pxTarget = AVP_ClientEntity_Actor::FindEntity(uMeleeTarget);
					if (!pxTarget || pxTarget->IsRagdoll())
					{
						bGrappleRunning = false;
					}
				}
			}
		}

		if (!bGrappleRunning)
		{
			SetShowActiveGrappleBar(false);
			return;
		}

		// Update time.
		float fTime = Asura_Timers::GetGameTimeSlice();

		if (m_fActiveGrapple_InitialDelay > 0.0f)
		{
			m_fActiveGrapple_InitialDelay -= fTime;
			if (m_fActiveGrapple_InitialDelay <= 0.0f)
			{
				m_fActiveGrapple_InitialDelay = 0.0f;
				s_bShowActiveGrappleSlider = true;
				const Asura_Sound_Event* pxMarkerSoundEvent = Asura_Sound_Event_Storage::FindEvent(m_uActiveGrapple_StartMarkerSoundEvent);
				if (pxMarkerSoundEvent)
				{
					Asura_Sound_Event_System::PlayEvent(pxMarkerSoundEvent, NULL);
				}
			}
		}
		else if (m_bActiveGrapple_ResultFinalised)
		{
			m_fActiveGrapple_CompletedDelay -= fTime;
			if (m_fActiveGrapple_CompletedDelay <= 0.0f)
			{
				m_fActiveGrapple_CompletedDelay = 0.0f;
				SetShowActiveGrappleBar(false);
			}
		}
		else if (!m_bActiveGrapple_IsWaitingForUserInput && (m_fActiveGrapple_BarSliderTimer >= m_fActiveGrapple_Duration))
		{
			// Multiplayer delay, since we haven't yet been finalised by message.
			m_fActiveGrapple_MultiplayerResultWaitTimer += fTime;
			if (m_fActiveGrapple_MultiplayerResultWaitTimer >= dbg_fActiveGrapple_MultiplayerResultWaitTime)
			{
				// Give up waiting and hope what we have is correct.
				FinaliseActiveGrappleResult();
			}
		}
		else
		{
			const AVP_Vertebrate_InputHandler* pxInputHandler = static_cast<const AVP_Vertebrate_InputHandler*>(xClient.GetInputHandler());

			// Make sure that after specified time, ActiveGrappleBar will disappear.
			if (m_fActiveGrapple_BarSliderTimer >= m_fActiveGrapple_Duration)
			{
				if (m_bActiveGrapple_IsWaitingForUserInput)
				{
					m_bActiveGrapple_IsWaitingForUserInput = false;
					m_fActiveGrapple_BarSliderPosSendToServer = 1.0f;
					SendActiveGrappleMessageToServer( xClient, m_fActiveGrapple_BarSliderPosSendToServer );
				}
			}
			else if (m_bActiveGrapple_IsWaitingForUserInput && pxInputHandler && pxInputHandler->GetControlMapValue(AVP_Vertebrate_InputHandler::GRAPPLE_INPUT))
			{
				m_bActiveGrapple_IsWaitingForUserInput = false;
				m_fActiveGrapple_BarSliderPosSendToServer = CalculateActiveGrappleSliderPosFromSliderTimer();

				SendActiveGrappleMessageToServer( xClient, m_fActiveGrapple_BarSliderPosSendToServer );
			}
			else
			{
				m_fActiveGrapple_BarSliderTimer += fTime;

				//Update slider position
				// Make sure that after user presses something, the bar will freeze on the same place, where the user hits the key.
				float fActualBarPos = m_bActiveGrapple_IsWaitingForUserInput ? CalculateActiveGrappleSliderPosFromSliderTimer() : m_fActiveGrapple_BarSliderPosSendToServer;
				fActualBarPos = Asura_Maths::Min(fActualBarPos, 1.0f);

				s_fActiveGrappleSliderPos = fActualBarPos;
			}
		}
	}
}


void AVP_Vertebrate_HUDHandler::SetShowActiveGrappleBar( bool bShow )
{
	s_bShowActiveGrappleBar = bShow;
}


void AVP_Vertebrate_HUDHandler::SendActiveGrappleMessageToServer( const AVP_ClientEntity_Actor& xClient, float fResultValue )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_ACTIVEGRAPPLE_VALUE, sizeof(AVP_Message_Vertebrate_ActiveGrappleValue));

	if (pxMessage)
	{
		AVP_Message_Vertebrate_ActiveGrappleValue* pxData = static_cast <AVP_Message_Vertebrate_ActiveGrappleValue*>(pxMessage->GetData());

		// Make sure that value is in between valid range.
		Asura_Maths::ClampToRange( fResultValue, 0.0f, 1.0f );


		// And now, we have to convert fResultValue which is position between 0.0f to 1.0f and represents position on the bar, to 
		// more meaningful value for server. The new value will be based on m_fActiveGrapple_Success_Start/End and m_fActiveGrapple_CriticalSuccess_Start/End
		// And it will be calculated as this: 

		// Value between two critical success values will be 1.0f
		if( fResultValue>=m_fActiveGrapple_CriticalSuccess_Start && fResultValue<=m_fActiveGrapple_CriticalSuccess_End )
		{
			pxData->SetValue( 1.0f );
		}
		else
		{
			// if value is outside normal success boundaries, it will return 0.0f
			if( fResultValue<=m_fActiveGrapple_Success_Start || fResultValue>=m_fActiveGrapple_Success_End )
			{
				pxData->SetValue( 0.0f );
			}
			else
			{
				float fFinalValue;

				// Value is hit somewhere in normal success boundaries, therefore I need to interpolate value from 0.0f to 1.0f
				// Also there are two boundaries. One is on left side from middle and second one is on right side from middle.
				if( fResultValue>=m_fActiveGrapple_Success_Start && fResultValue<=m_fActiveGrapple_CriticalSuccess_Start )
				{
					fFinalValue = (fResultValue-m_fActiveGrapple_Success_Start) / (m_fActiveGrapple_CriticalSuccess_Start-m_fActiveGrapple_Success_Start);
				}
				else	// I hit the second boundary of normal success.
				{
					fFinalValue = (fResultValue-m_fActiveGrapple_CriticalSuccess_End) / (m_fActiveGrapple_Success_End-m_fActiveGrapple_CriticalSuccess_End);
					fFinalValue = 1.0f - fFinalValue;
				}

				pxData->SetValue( fFinalValue );
			}

		}

		pxMessage->SetFromGuid( xClient.GetGuid() );
		pxMessage->SetToGuid( xClient.GetGuid() );
		Asura_Network::Send( pxMessage );
	}
}

void AVP_Vertebrate_HUDHandler::SetActiveGrappleResult(AVP_VERTEBRATE_HUDHANDLER_ACTIVEGRAPPLE_RESULT eResult)
{
	m_eStoredActiveGrappleResult = eResult;
}

void AVP_Vertebrate_HUDHandler::FinaliseActiveGrappleResult()
{
	if (s_bShowActiveGrappleBar)
	{
		// If we're currently undecided, assume failure.
		if (m_eStoredActiveGrappleResult == AVP_VERTEBRATE_ACTIVEGRAPPLE_RESULT_UNDECIDED)
		{
			m_eStoredActiveGrappleResult = AVP_VERTEBRATE_ACTIVEGRAPPLE_RESULT_FAILURE;
		}

		s_uFlashActiveGrappleBar = m_eStoredActiveGrappleResult;
		m_bActiveGrapple_ResultFinalised = true;

		const Asura_Sound_Event* pxSoundEvent = Asura_Sound_Event_Storage::FindEvent((m_eStoredActiveGrappleResult == AVP_VERTEBRATE_ACTIVEGRAPPLE_RESULT_FAILURE) ? m_uActiveGrapple_LoseSoundEvent : m_uActiveGrapple_WinSoundEvent);
		if (pxSoundEvent)
		{
			Asura_Sound_Event_System::PlayEvent(pxSoundEvent, NULL);
		}
	}
}

///------------------------------------------------------------------------------------------------------------------------------------------------------
///------------------------------------------------------------------------------------------------------------------------------------------------------
///------------------------------------------------------------------------------------------------------------------------------------------------------




/// End of file. ///
