
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Vertebrate_InputHandler.h
// Author : Chris Wallace
// Date   : 17-Apr-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_VERTEBRATE_INPUTHANDLER_H__
#define __AVP_VERTEBRATE_INPUTHANDLER_H__


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/AVP_Actor_InputHandler.h"

#include "Core/Asura_Maths.h"
#include "Input/Asura_ControlMap.h"
#include "Network/Asura_Guid.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Message;
class AVP_ClientActor_Vertebrate;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Vertebrate_InputHandler
// Author       : Chris Wallace
// Date         : 17-Apr-2007
////////////////////////////////////////////////////////////////////////////////

class AVP_Vertebrate_InputHandler : public AVP_Actor_InputHandler
{
public:
	// Public enumerated types...
	enum VertebrateMappings
	{
		// VERTEBRATE SHARED MAPPINGS --------------------------------------------------------
		MIN_VERTEBRATE_MAPPINGS = MAX_ACTOR_MAPPINGS,
		// Attacks...
		ATTACK_LEFT	= MIN_VERTEBRATE_MAPPINGS,
		ATTACK_RIGHT,

		PRIMARY_ATTACK_TAP,
		SECONDARY_ATTACK_TAP,

		PRIMARY_ATTACK_HELD,
		SECONDARY_ATTACK_HELD,

		PRIMARY_ATTACK_RELEASE,	
		SECONDARY_ATTACK_RELEASE,

		// Movement-modifier mappings...
		SLOW,
		FAST,

		// Crouch mappings...
		STANCE_TOGGLE,

		// Grapple mappings...
		GRAPPLE_DODGE,
		GRAPPLE_INPUT,

		// Focus mode
		FOCUS_HELD,
		FOCUS_TOGGLE,

		// Melee Attacks
		MELEE,
		COUNTER,
		BLOCK,

		// Jumping
		JUMP,

		HEALTH_REGAIN, // stim for marines, injection for pred, headbite for alien.
		RELOAD, // Irrelevant to the alien, but the key may match the health regain one, and we want automatic handling of that.

		// Total...
		MAX_VERTEBRATE_MAPPINGS,

		// ALIEN SPECIFIC MAPPINGS -----------------------------------------------------------
		MIN_ALIEN_MAPPINGS = MAX_VERTEBRATE_MAPPINGS,

		TAIL_ATTACK_TAP = MIN_ALIEN_MAPPINGS,
		TAIL_ATTACK_HELD,
		TAIL_ATTACK_RELEASE,
		PET_FACEHUGGER_CONTROL,
		CREATE_DISTRACTION,
		TRANSITION,

		MAX_ALIEN_MAPPINGS,

		// HUMAN SPECIFIC MAPPINGS -----------------------------------------------------------
		MIN_HUMAN_MAPPINGS = MAX_ALIEN_MAPPINGS,

		TORCH_TOGGLE = MIN_HUMAN_MAPPINGS,
		FLARE,
		AIM_DOWN_SIGHTS,

		MAX_HUMAN_MAPPINGS,

		// PREDATOR SPECIFIC MAPPINGS --------------------------------------------------------
		MIN_PREDATOR_MAPPINGS = MAX_HUMAN_MAPPINGS,

		CLOAK = MIN_PREDATOR_MAPPINGS,
		RECON_TOGGLE,

		MAX_PREDATOR_MAPPINGS,

		// Total...
		TOTAL_CONTROL_MAPPINGS = MAX_PREDATOR_MAPPINGS

	};

	// Public functions...
	virtual ~AVP_Vertebrate_InputHandler();
	
	virtual	void Reset(void);
	virtual	Asura_Message*	Update(AVP_ClientEntity_Actor& xClient);

	// Mapping creation functions...
	virtual void	MapAttacks(Asura_ControlMap& xMap, const u_int uKeyPrimary, const u_int uKeySecondary, const u_int uKeyLeft, const u_int uKeyRight);
	virtual void	MapMovementAxes(Asura_ControlMap& xMap, const u_int uAxisForwardsBackwards, const u_int uAxisStrafeLeftRight, const u_int uKeySlow, const u_int uKeyFast, const u_int uKeyStanceToggle);
	virtual void	MapMovementKeys(Asura_ControlMap& xMap, const u_int uKeyForwards, const u_int uKeyBackwards, const u_int uKeyStrafeLeft, const u_int uKeyStrafeRight, const u_int uKeySlow, const u_int uKeyFast, const u_int uKeyStanceToggle);
	virtual void	MapFocusKeys(Asura_ControlMap& xMap, const u_int uKeyFocusHeld, const u_int uKeyFocusToggle);	// Not necessarily used by every species.
	virtual void	MapMelee(Asura_ControlMap& xMap, const u_int uKeyMelee, const u_int uKeyCounter, const u_int uKeyActiveGrapple);
	virtual void	MapJump(Asura_ControlMap& xMap, const u_int uKey);
	virtual void	MapReload(Asura_ControlMap& xMap, const u_int uKey );
	virtual void	MapInteractAndReloadToSameKey(Asura_ControlMap& xMap, const u_int uKeyInteractAndReload );
	virtual void	MapHealth(Asura_ControlMap& xMap, const u_int uKey);

	virtual bool	IsIdle() const;

	virtual void RefreshBlueprintData();

	void GetAirVelocity(Asura_Vector_3& xAirVel) const { xAirVel.SetX(m_xFlatAirVelocity.x); xAirVel.SetY(0.0f); xAirVel.SetZ(m_xFlatAirVelocity.y); }
	void GetAirVelocity(Asura_AVector3& xAirVel) const { xAirVel.SetX(m_xFlatAirVelocity.x); xAirVel.SetY(0.0f); xAirVel.SetZ(m_xFlatAirVelocity.y); }

	u_int GetTileIdOrDescStringForMapping( u_int uMapID, u_int uMapping, u_int uMappingID, bool bTileId, Asura_String* pxDescString = NULL, u_int uComboId = 0 );
	u_int GetTileIdOrDescStringForMapping( const Asura_ControlMapping* pxMapping, bool bTileId, Asura_String* pxDescString = NULL, u_int uComboId = 0 );
	bool IsSpecialCharAComboMapping( u_int uMapping );

	virtual void			SetNewLookDirection( AVP_ClientEntity_Actor& xClient, const Asura_AVector3& xDir, const bool bImmediate );

	virtual void			AddRecoilVelocity( Asura_Euler_3 xRecoilVel ) { m_xCurrentRecoilVelocity += xRecoilVel; }

protected:
	// Protected functions...
	AVP_Vertebrate_InputHandler (const Asura_Guid uOwnerGuid);

	virtual void			UpdateLookInput		( AVP_ClientEntity_Actor& xClient, const float fTiltMaxSpeed, const float fTurnMaxSpeed, const float fInnerAimAssistMultiplier, const float fOuterAimAssistMultiplier, const float fTanLookAssistHalfAngle, const float fLookAssistHorizontalMaxSpeed, const float fLookAssistVerticalMaxSpeed );
	virtual void			UpdateMovementInput	( AVP_ClientEntity_Actor& xClient );
	float					GetLookZoomScale( AVP_ClientEntity_Actor& xClient );

	virtual void UpdateMovement( AVP_ClientEntity_Actor& xClient );

	virtual void	UpdateLockon	(void);
	virtual void	UpdateZoom		( AVP_ClientActor_Vertebrate& xClient );
	virtual void	UpdateMeleeRequests( AVP_ClientActor_Vertebrate& xClient );
	void			UpdateRecoil( AVP_ClientEntity_Actor& xClient );

	// Protected variables...
	Asura_AVector3  m_xDesiredLookDirection;	// used for returning from recoil
	Asura_Euler_3	m_xCurrentRecoilOffset;
	Asura_Euler_3	m_xLastRecoilOffset;
	Asura_Euler_3	m_xCurrentRecoilVelocity;
	Asura_Euler_3	m_xLastRecoilVelocity;
	Asura_Euler_3	m_xLastRecoilForce;

	Asura_Vector_3	m_xSmoothedDeltaVelocity; // used in calculation of look assist

	float	m_fZoomTimer;
	float	m_fZoomHoldTimer;

	float	m_fSprintTime;
	float	m_fSprintTimer;
	float	m_fSprintChargeTime;
	float	m_fSprintChargeTimer;

	// Data for falling state movement.
	Asura_Vector_2 m_xFlatAirVelocity; // Air velocity always relates to down, since you must have a downwards gravity direction when falling.
	float m_fMaxAirSpeed;
	float m_fAirAcceleration;

	bool	m_bZoomed;

	static bool s_bOnlyLeaveBranchByJump;
	
private:
	// Private typedefs...
	typedef AVP_Actor_InputHandler PARENT;

	// Private variables...

	// Private functions...
	AVP_Vertebrate_InputHandler (void);
	AVP_Vertebrate_InputHandler (const AVP_Vertebrate_InputHandler& xOther);
	AVP_Vertebrate_InputHandler&	operator=	(const AVP_Vertebrate_InputHandler& xOther);

	static bool ConvertSpecialCharToMapping(u_int& uMapping);

	bool Platform_GetDescStringForMapping ( Asura_String* pxDescString, const Asura_ControlMapping* pxMapping );
	u_int Platform_GetTileIdForMapping( const Asura_ControlMapping* pxMapping, u_int uComboId = 0 );

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	static int s_iConsoleVarsCount;
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)

};   // End of class 'AVP_Vertebrate_InputHandler'.


#endif   // ifndef __AVP_VERTEBRATE_INPUTHANDLER_H__


/// End of file. ///
