
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_AIAnimStateMachine.h
// Author : Will Plano
// Date   : 22/02/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ALIEN_AIANIMSTATEMACHINE_H__
#define __AVP_ALIEN_AIANIMSTATEMACHINE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/Vertebrates/AVP_Vertebrate_AIAnimStateMachine.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_JumpHandler.h"

class AVP_Alien_AIAnimStateMachine : public AVP_Vertebrate_AIAnimStateMachine
{
public:
	// Public enumerated types...

	enum AlienAnimState
	{
		ALIEN_STATE_BEGIN = VERTEBRATE_STATE_END,
		// standing states
		ALIEN_STATE_STAND_IDLE = ALIEN_STATE_BEGIN,	// 13
		ALIEN_STATE_STAND_MOVE,						// 14
		// transition states
		ALIEN_STATE_IDLE_TO_STAND_IDLE,				// 15
		ALIEN_STATE_STAND_IDLE_TO_IDLE,				// 16
		// movement interest states
		ALIEN_STATE_BURST,							// 17
		ALIEN_STATE_HESITATE,						// 18

		ALIEN_STATE_HIDE_IDLE,						// 19

		ALIEN_STATE_END							// 20

	};

	enum LimbDamage
	{
		LIMBDMG_BEGIN,
		LIMBDMG_NONE = LIMBDMG_BEGIN,
		LIMBDMG_LEFT_LEG,
		LIMBDMG_RIGHT_LEG,
		LIMBDMG_LEFT_ARM,
		LIMBDMG_RIGHT_ARM,
		LIMBDMG_BOTH_LEGS,
		LIMBDMG_END
	};

	enum GroundType
	{
		GROUNDTYPE_BEGIN,
		GROUNDTYPE_ON_FLOOR = GROUNDTYPE_BEGIN,
		GROUNDTYPE_NON_FLOOR,
		GROUNDTYPE_END
	};

	// Public functions...
	AVP_Alien_AIAnimStateMachine( AVP_ServerEntity_Actor& xServerActor ) : PARENT( xServerActor ) {}
	virtual ~AVP_Alien_AIAnimStateMachine() {}

	virtual void Initialise();

	void	RequestStandIdle() { RequestState( ALIEN_STATE_STAND_IDLE ); }
	bool	IsStandIdle() const { return IsStateStandIdle( GetCurrentState() ); }
	bool	IsStateStandIdle( const u_int uState ) const { return ALIEN_STATE_STAND_IDLE == uState; }

	void	RequestHideIdle() { RequestState( ALIEN_STATE_HIDE_IDLE ); }
	bool	IsHideIdle() const { return IsStateHideIdle( GetCurrentState() ); }
	bool	IsStateHideIdle( const u_int uState ) const { return ALIEN_STATE_HIDE_IDLE == uState; }


	void	RequestStandMove() { RequestState( ALIEN_STATE_STAND_MOVE ); }
	bool	IsStandMove() const { return IsStateStandMove( GetCurrentState() ); }
	bool	IsStateStandMove( const u_int uState ) const { return ALIEN_STATE_STAND_MOVE == uState; }

	// Vertebrate functions...
	void    RequestBurst() { RequestState( ALIEN_STATE_BURST ); }
	bool	IsBurst() const { return IsStateBurst( GetCurrentState() ); }
	bool    IsStateBurst( const u_int uState ) const { return ALIEN_STATE_BURST == uState; }

	void    RequestHesitate() { RequestState( ALIEN_STATE_HESITATE ); }
	bool	IsHesitate() const { return IsStateHesitate( GetCurrentState() ); }
	bool    IsStateHesitate( const u_int uState ) const { return ALIEN_STATE_HESITATE == uState; }

	virtual const AVP_Actor_AIAnimStateConnectionData* GetConnectionData( const u_int uFromState, const u_int uToState ) const;

	virtual const char* GetStateDescription( const u_int uState ) const { return GetStateDescriptionStatic( uState ); }
	static const char* GetStateDescriptionStatic( const u_int uState );

	static bool IsDirectionalState( const u_int uState );

	static const char* GetGroundTypeDescription( const u_int uGroundType );
	static const char* GetLimbDamageDescription( const u_int uLimbDamage );

	virtual bool IsLoopedState( const u_int uState ) const { return IsLoopedStateStatic( uState ); }
	static bool IsLoopedStateStatic( const u_int uState );

protected:
	virtual void BeginState		( const u_int uState );
	//virtual void UpdateState	( const u_int uState ) { PARENT::UpdateState( uState ); }
	virtual bool EndState		( const u_int uState );

	virtual bool	IsCurrentStateFinished() const;

	virtual u_int	GetNumStates() const { return ALIEN_STATE_END; }
	virtual const AVP_Actor_AnimStateData* GetStateAnimData( const u_int uState ) const;

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

private:
	typedef AVP_Vertebrate_AIAnimStateMachine PARENT;

	AVP_Alien_AIAnimStateMachine();
	AVP_Alien_AIAnimStateMachine(const AVP_Alien_AIAnimStateMachine& xOther);
	AVP_Alien_AIAnimStateMachine&	operator=(const AVP_Alien_AIAnimStateMachine& xOther);
}; // End of class 'AVP_Alien_AIAnimStateMachine'.

#endif // ifndef __AVP_ALIEN_AIANIMSTATEMACHINE_H__

/// End of file. ///
