
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_HUDHandler.cpp
// Author : Chris Wallace
// Date   : 6-Aug-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "AVP_Alien_HUDHandler.h"

// Asura header-files...
#include "Animation/Asura_Animation.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Core/Asura_Timers.h"
#include "FSFX/Asura_FSFX_System.h"
#include "Interface/Asura_CommandConsole.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Font_System.h"
#include "GUIMenu/Asura_GUIMenu_System.h"

// Game header-files...
#include "Actor/Vertebrates/Alien/AVP_Alien_JumpHandler.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ClientActor_Alien.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Rendering/AVP_Overlay_Render.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

#define HUD_ALIEN_STEALTH_INSHADOW		L"AVP.AlienPlayer.IsInShadow"
#define HUD_ALIEN_STEALTH_VISUAL_01		L"AVP.AlienPlayer.StealthIndicator01"
#define HUD_ALIEN_STEALTH_VISUAL_02		L"AVP.AlienPlayer.StealthIndicator02"
#define HUD_ALIEN_STEALTH_VISUAL_03		L"AVP.AlienPlayer.StealthIndicator03"
#define HUD_ALIEN_STEALTH_VISUAL_04		L"AVP.AlienPlayer.StealthIndicator04"
#define HUD_ALIEN_STEALTH_HEARING_01	L"AVP.AlienPlayer.StealthIndicator05"
#define HUD_ALIEN_FACEHUGGER_SUMMONABLE L"AVP.AlienPlayer.SummoningPointInRange"
#define HUD_ALIEN_FACEHUGGER_PET_STATUS L"AVP.AlienPlayer.SummonedFacehuggerStatus"


////////////////////////////////////////////////////////////////////////////////
// Global variable declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

bool AVP_Alien_HUDHandler::s_bIsInShadow = false;
int AVP_Alien_HUDHandler::s_iStealth01 = 0;
int AVP_Alien_HUDHandler::s_iStealth02 = 0;
int AVP_Alien_HUDHandler::s_iStealth03 = 0;
int AVP_Alien_HUDHandler::s_iStealth04 = 0;
int AVP_Alien_HUDHandler::s_iStealth05 = 0;
bool AVP_Alien_HUDHandler::s_bHeadbiteTargetInRange = false;
bool AVP_Alien_HUDHandler::s_bIsFaceHuggerSummonable = false;
bool AVP_Alien_HUDHandler::s_bSurfaceInRange = false;
bool AVP_Alien_HUDHandler::s_bTailstabTargetInRange = false;
u_int AVP_Alien_HUDHandler::s_uFaceHuggerPetStatus = 0;

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

AVP_Alien_HUDHandler::AVP_Alien_HUDHandler (AVP_ClientEntity_Actor& xClient) 
: PARENT(xClient)
, m_fHealth(100.0f)
, m_fArmour(100.0f)
{
}

AVP_Alien_HUDHandler::~AVP_Alien_HUDHandler () 
{
}

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
void AVP_Alien_HUDHandler::CreateConsoleVars()
{
	Asura_CommandConsole::AddVar( HUD_ALIEN_STEALTH_INSHADOW, &s_bIsInShadow, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_ALIEN_STEALTH_INSHADOW );
	Asura_CommandConsole::AddVar( HUD_ALIEN_STEALTH_VISUAL_01, &s_iStealth01, 0, 1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_ALIEN_STEALTH_VISUAL_01 );
	Asura_CommandConsole::AddVar( HUD_ALIEN_STEALTH_VISUAL_02, &s_iStealth02, 0, 1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_ALIEN_STEALTH_VISUAL_02 );
	Asura_CommandConsole::AddVar( HUD_ALIEN_STEALTH_VISUAL_03, &s_iStealth03, 0, 1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_ALIEN_STEALTH_VISUAL_03 );
	Asura_CommandConsole::AddVar( HUD_ALIEN_STEALTH_VISUAL_04, &s_iStealth04, 0, 1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_ALIEN_STEALTH_VISUAL_04 );
	Asura_CommandConsole::AddVar( HUD_ALIEN_STEALTH_HEARING_01, &s_iStealth05, 0, 1, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_ALIEN_STEALTH_HEARING_01 );
	Asura_CommandConsole::AddVar( HUD_ALIEN_FACEHUGGER_SUMMONABLE, &s_bIsFaceHuggerSummonable, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_ALIEN_FACEHUGGER_SUMMONABLE );
	Asura_CommandConsole::AddVar( HUD_ALIEN_FACEHUGGER_PET_STATUS, &s_uFaceHuggerPetStatus, 0, INT_MAX, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( HUD_ALIEN_FACEHUGGER_PET_STATUS );
	Asura_CommandConsole::AddVar( L"AVP.AlienPlayer.JumpToSurfaceInRange", &s_bSurfaceInRange, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( L"AVP.AlienPlayer.JumpToSurfaceInRange" );
	Asura_CommandConsole::AddVar( L"AVP.AlienPlayer.HeadbiteTargetInRange", &s_bHeadbiteTargetInRange, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( L"AVP.AlienPlayer.HeadbiteTargetInRange" );
	Asura_CommandConsole::AddVar( L"AVP.AlienPlayer.TailstabTargetInRange", &s_bTailstabTargetInRange, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( L"AVP.AlienPlayer.TailstabTargetInRange" );
}
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)	


// hash Id's for stealth indicator
// 0x5C5B7126 // AVP.AlienPlayer.StealthIndicator01
// 0x5C5B7127 // AVP.AlienPlayer.StealthIndicator02
// 0x5C5B7128 // AVP.AlienPlayer.StealthIndicator03
// 0x5C5B7129 // AVP.AlienPlayer.StealthIndicator04
// 0x5C5B712A // AVP.AlienPlayer.StealthIndicator05

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Alien_HUDHandler::Update
// Author       : Chris Wallace
// Date         : 6-Aug-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Alien_HUDHandler::Update ( const AVP_ClientEntity_Actor& xClient )
{
	const AVP_ClientActor_Alien& xAlien	= static_cast< const AVP_ClientActor_Alien& > ( xClient );
	
	//Grab the player values by guid.
	//Update health
	m_fHealth	= xAlien.GetHealth();

	// update the shadow indicator. 
	s_bIsInShadow = xAlien.IsAlienInShadow();

	// Do Alien Stealth Indicator stuff, the actual HUD graphics are only placeholder
	UpdateAlienStealthIndicator(xAlien);
	
	// Do the Alien Jump Indicator
	UpdateAlienJumpIndicator(xAlien);
	
	// Do the Summon FaceHugger indicator
	UpdateFaceHuggerPet(xAlien);

	// Call the base-function...
	PARENT::Update( xClient );
}

/// End of function 'AVP_Alien_HUDHandler::Update'. ///

//------------------------------------------------------------------------------
// Function     : AVP_Alien_HUDHandler::UpdateAlienJumpIndicator
// Author       : Gareth Thomas
// Date         : 26 / 02 / 08
//------------------------------------------------------------------------------
void AVP_Alien_HUDHandler::UpdateAlienJumpIndicator(const AVP_ClientActor_Alien& xAlien)
{
}
/// End of function 'AVP_Alien_HUDHandler::UpdateAlienJumpIndicator'. ///

//------------------------------------------------------------------------------
// Function     : AVP_Alien_HUDHandler::UpdateAlienJumpIndicator
//------------------------------------------------------------------------------
void AVP_Alien_HUDHandler::UpdateFaceHuggerPet(const AVP_ClientActor_Alien& xAlien)
{
	s_bIsFaceHuggerSummonable = xAlien.IsFaceHuggerSummonable();
	s_uFaceHuggerPetStatus = xAlien.GetFaceHuggerPetStatus();
}										


//------------------------------------------------------------------------------
// Function     : AVP_Alien_HUDHandler::UpdateAlienStealthIndicator
// Author       : Gareth Thomas
// Date         : 26 / 02 / 08
//------------------------------------------------------------------------------
void AVP_Alien_HUDHandler::UpdateAlienStealthIndicator(const AVP_ClientActor_Alien& xAlien)
{
	enum 
	{
		STEALTH_INDICATOR_NOT_BEEN_SEEN = 0,
		STEALTH_INDICATOR_NOT_BEEN_SEEN_YET,
		STEALTH_INDICATOR_DANGER_OF_BEING_SEEN,
		STEALTH_INDICATOR_SPOTTED,
	};

	if (AVP_Settings::IsSinglePlayer())
	{
		// Sound Indicator
		if (xAlien.GetHasBeenHeard())
		{
			s_iStealth05 = 1;
		}
		else
		{
			s_iStealth05 = 0;
		}

		// Sight Indicator
		switch (xAlien.GetSpottedState())
		{
			case STEALTH_INDICATOR_NOT_BEEN_SEEN:
			{
				// Turn one on, Turn the rest off
				s_iStealth01 = 1; s_iStealth02 = s_iStealth03 = s_iStealth04 = 0;

			}break;

			case STEALTH_INDICATOR_NOT_BEEN_SEEN_YET:
			{
				// Turn one on, Turn the rest off
				s_iStealth02 = 1; s_iStealth01 = s_iStealth03 = s_iStealth04 = 0;

			}break;

			case STEALTH_INDICATOR_DANGER_OF_BEING_SEEN:
			{
				// Turn one on, Turn the rest off
				s_iStealth03 = 1; s_iStealth01 = s_iStealth02 = s_iStealth04 = 0;

			}break;

			case STEALTH_INDICATOR_SPOTTED:
			{
				// Turn one on, Turn the rest off
				s_iStealth04 = 1; s_iStealth01 = s_iStealth02 = s_iStealth03 = 0; 

			}break;
		}
	}
	else
	{
		// Turn all off
		s_iStealth01 = 0; s_iStealth02 = s_iStealth03 = s_iStealth04 = 0; s_iStealth05 = 0; 
	}
}
/// End of function 'AVP_Alien_HUDHandler::UpdateAlienStealthIndicator'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Alien_HUDHandler::Render
// Author       : Chris Wallace
// Date         : 17-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Alien_HUDHandler::Render( const AVP_ClientEntity_Actor& xClient )
{
	//Render HUD below menu
	RenderText();	
	RenderPanicPrompts();

	PARENT::Render( xClient );
}

/////////////////////////////////////////////////
// Private Render Functions
/////////////////////////////////////////////////

void AVP_Alien_HUDHandler::RenderText( void )
{
 	//////////////////// TEXT ONLY STUFF /////////////
 
 	//Re-set co-ordinates
 	//Ideally this should be in the parent class, but as the parent class is called 
 	//last to keep the display order correct it is here.
 	//Should probably be moved once the HUD is complete.
//  	m_fY = static_cast<float>(m_fHeight) - (m_fIncrement*2.0f);
//  	m_fX = m_fIncrement*1.5f;
//  
//  	Asura_UnicodeChar			wszText[256];
//  
// 	Asura_PrintfToBfr(wszText, L"Health: %u", 256, true, m_uHealth);
// 	Asura_Font_System::Print(m_fLeft+m_fX, m_fTop+m_fY, m_fHUDScaleH*m_fTextScale, m_uTextColour, m_uTextFormat, wszText);
// 
// 	m_fY += m_fIncrement;
  
}

void AVP_Alien_HUDHandler::Reset( void )
{
	PARENT::Reset();
}

void AVP_Alien_HUDHandler::RefreshBlueprintValues(void)
{
	
// TODO, any blueprint values go here


// 		Asura_Value xValue;
// 		Asura_Hash_ID uBlueprintHash;
// 		bool bGotBlueprint;
// 
// 		Asura_Blueprint* pxBlueprint = NULL;
// 		Asura_Hash_ID uTypeHash = Asura_GetHashID("Settings");
// 
// 		uBlueprintHash = Asura_GetHashID("MarineHUD");
// 		bGotBlueprint = Asura_Blueprint_System::GetBlueprint(uTypeHash, uBlueprintHash, pxBlueprint);
// 
// 		if(bGotBlueprint && pxBlueprint)
// 		{
// 			if(pxBlueprint->GetParameterValueByHashID(Asura_GetHashID("HealthBarX"), xValue))
// 			{
// 				m_xHealthBarPos.x = (xValue.GetFloatValue());
// 			}
// 			if(pxBlueprint->GetParameterValueByHashID(Asura_GetHashID("HealthBarY"), xValue))
// 			{
// 				m_xHealthBarPos.y = (xValue.GetFloatValue());
// 			}
// 			if(pxBlueprint->GetParameterValueByHashID(Asura_GetHashID("ArmourBarX"), xValue))
// 			{
// 				m_xArmourBarPos.x = (xValue.GetFloatValue());
// 			}
// 			if(pxBlueprint->GetParameterValueByHashID(Asura_GetHashID("ArmourBarY"), xValue))
// 			{
// 				m_xArmourBarPos.y = (xValue.GetFloatValue());
// 			}
// 		}
	
}


/// End of file. ///
