///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_AlienQueen_Description.h
// Author : David Wooldridge
// Date   : 06/11/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ALIENQUEEN_DESCRIPTION_H__
#define __AVP_ALIENQUEEN_DESCRIPTION_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Hash.h"
#include "Dynamics/Asura_Collision_Object.h"
#include "Physics/Asura_Physics_Ragdoll_Description.h"

#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"

static const u_int uAVP_ALIENQUEEN_NUM_IK_BONES_PER_TAIL = 12;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class AVP_AlienQueen_Description
{
public:
	// Public enumerated types...
	enum BoneID
	{
		BIP_01, 
		PELVIS, 
		SPINE, 
		LEFT_THIGH,
		LEFT_CALF,
		LEFT_FOOT,
		LEFT_TOE0,
		LEFT_TOE01,
		LEFT_TOE02,
		RIGHT_THIGH,
		RIGHT_CALF,
		RIGHT_FOOT,
		RIGHT_TOE0,
		RIGHT_TOE01,
		RIGHT_TOE02,
		SPINE1,
		SPINE2,
		NECK,
		HEAD01,
		HEAD02,
		JAW,
		MOUTH_POINT_BOTTOM_CENTER,
		MOUTH_POINT_BOTTOM_LEFT,
		MOUTH_POINT_BOTTOM_RIGHT,
		MOUTH_POINT_TOP_CENTRE,
		MOUTH_POINT_TOP_LEFT,
		MOUTH_POINT_TOP_RIGHT,
		INNERMOUTH,
		INNERJAWBOTTOM,
		INNERJAWTOP,
		LEFT_CLAVICLE,
		LEFT_UPPERARM,
		LEFT_FORARM,
		LEFT_HAND,
		LEFT_FINGER0,
		LEFT_FINGER01,
		LEFT_FINGER1,
		LEFT_FINGER11,
		LEFT_FINGER12,
		LEFT_FINGER2,
		LEFT_FINGER21,
		LEFT_FINGER22,
		LEFT_FINGER3,
		LEFT_FINGER31,
		LEFT_FINGER32,
		LEFT_FINGER4,
		LEFT_FINGER41,
		LEFT_FORETWIST,
		RIGHT_CLAVICLE,
		RIGHT_UPPERARM,
		RIGHT_FORARM,
		RIGHT_HAND,
		RIGHT_FINGER0,
		RIGHT_FINGER01,
		RIGHT_FINGER1,
		RIGHT_FINGER11,
		RIGHT_FINGER12,
		RIGHT_FINGER2,
		RIGHT_FINGER21,
		RIGHT_FINGER22,
		RIGHT_FINGER3,
		RIGHT_FINGER31,
		RIGHT_FINGER32,
		RIGHT_FINGER4,
		RIGHT_FINGER41,
		RIGHT_FORETWIST,
		SMALLARM_LEFT_UPPER,
		SMALLARM_LEFT_FOREARM,
		SMALLARM_LEFT_HAND,
		SMALLARM_RIGHT_UPPER,
		SMALLARM_RIGHT_FOREARM,
		SMALLARM_RIGHT_HAND,
		SPINE_LEFT_LOWER,
		SPINE_LEFT_MID,
		SPINE_LEFT_UPPER,
		SPINE_RIGHT_LOWER,
		SPINE_RIGHT_MID,
		SPINE_RIGHT_UPPER,
		TAIL,
		TAIL1,
		TAIL2,
		TAIL3,
		TAIL4,
		TAIL5,
		TAIL6,
		TAIL7,
		TAIL8,
		TAIL9,
		TAIL10,
		TAIL11,

		MAX_BONES_IDS
	};

	enum MarkerPointID
	{
		MPID_TONGUE,
		MPID_BODY,
		MPID_TARGET_1,
		MPID_BLOOD,

		NUM_MARKER_POINT_IDS
	};


	// Public functions...
#if defined(ASURA_TOOLS)
	static void	AddFilesToExport(const bool bFirstPerson, const Asura_Hash_ID uBlueprintHash, List<Asura_String>& xNameList);
#endif	// defined(ASURA_TOOLS)

	// Accessor functions...
	static Asura_Hash_ID		GetMarkerPointHashID(const MarkerPointID eID)	{ return(s_auMarkerPointHashIDs[eID]); }
	static int					GetFootBoneID( u_int uFootIndex, bool bFirstPerson ) { Asura_Assert( !bFirstPerson, "No FP facehuggers" ); Asura_Assert( uFootIndex < AVP_ClientEntity_Actor::MAX_NUM_FEET, "Bad foot index" ); return s_aiThirdPersonFootBoneID[ uFootIndex ]; }

	static const int*			GetTailChainArray() { return s_aiTailChainBonesArray; }

protected:
	// Protected variables...
	static const Asura_Hash_ID	s_auMarkerPointHashIDs[NUM_MARKER_POINT_IDS];

	static const int			s_aiThirdPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ];

	static const int 			s_aiTailChainBonesArray[ uAVP_ALIENQUEEN_NUM_IK_BONES_PER_TAIL + 1  ];

	// Protected functions...


private:
	// Private variables...

	// Private functions...
	AVP_AlienQueen_Description(void);
	AVP_AlienQueen_Description(const AVP_AlienQueen_Description& xOther);
	~AVP_AlienQueen_Description();

	AVP_AlienQueen_Description&	operator=(const AVP_AlienQueen_Description& xOther);

};   // End of class 'AVP_FaceHugger_Description'.


#endif