
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_FaceHugger_Description.cpp
// Author : Chris Wallace
// Date   : 24-Jan-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_Description.h"

// Asura independent header-files...

// Asura platform header-files...

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"

// Game platform header-files...

// Tools header-files...
#if defined(ASURA_TOOLS)
#include "Blueprint/Tools_Mode_Blueprint.h"
#include "Particles/Tools_Mode_PFX.h"
#endif	// defined(ASURA_TOOLS)


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

// Marker-point hash-IDs...
const Asura_Hash_ID AVP_FaceHugger_Description::s_auMarkerPointHashIDs[NUM_MARKER_POINT_IDS] =
{
	0xCC41EB04,		// Asura_GetHashID("tongue");	// MPID_TONGUE
	0x002E39A2,		// Asura_GetHashID("body");		// MPID_BODY
	0xE5A85D03,		// Asura_GetHashID("target_1");	// MPID_TARGET_1
	0x0597C5FA,		// Asura_GetHashID("blood");	// MPID_BLOOD
};

const int AVP_FaceHugger_Description::s_aiThirdPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ] = 
{
	FINGER_L4_3,
	FINGER_R4_3,
	FINGER_L2_3,
	FINGER_R2_3
};

const int AVP_FaceHugger_Description::s_aiTailChainBonesArray[ uAVP_FACEHUGGER_NUM_IK_BONES_PER_TAIL + 1 ] = 
{
	TAIL01,	
	TAIL02,	
	TAIL03,	
	TAIL04,	
	TAIL05,	
	TAIL06,	
	TAIL07,	
	TAIL08,	
	TAIL09,	
	TAIL10,	
	-1		// Terminator	
};

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

#if defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_FaceHugger_Description::AddFilesToExport
// Author       : Andy Tate
// Date         : 25-Jul-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_FaceHugger_Description::AddFilesToExport(const bool bFirstPerson, const Asura_Hash_ID uBlueprintHash, List<Asura_String>& xNameList)
{
	// Add all of the animations in the appropriate directory...
	Asura_Assert(!bFirstPerson, "First-person civilian's cannot yet be exported.");
	//Tools_Level_Export::AddAnimDirToExportList(xNameList, "Characters\\ThirdPerson\\Industrial_Worker", true);

	// Export the relevant blueprints
	if (uBlueprintHash != ASURA_HASH_ID_UNSET)
	{
		Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, 0x766161d2, uBlueprintHash, false); //Asura_GetHashID("Actor Initialisation");
	}

	// Also add everything of type "Grapple Alien". This has first and third person anims, so do it regardless of skin type.
	Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uAVP_BLUEPRINT_TYPE_GRAPPLEALIEN, ASURA_HASH_ID_UNSET, true);
}

/// End of function 'AVP_FaceHugger_Description::AddFilesToExport'. ///

#endif	//defined(ASURA_TOOLS)


/// End of file. ///
