
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_Description.h
// Author : Andy Tate
// Date   : 9-Nov-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_HUMAN_DESCRIPTION_H__
#define __AVP_HUMAN_DESCRIPTION_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Hash.h"
#include "Physics/Asura_Physics_Ragdoll_Description.h"

#if defined(ASURA_TOOLS)
#include "Misc/list_tem.hpp"
#endif	// defined(ASURA_TOOLS)

#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Physics/AVP_Collision_Descriptor.h"

static const u_int uAVP_HUMAN_NUM_IK_BONES_PER_LIMB = 3;

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////
class AVP_ServerActor_Android;
class AVP_ServerActor_Civilian;
class AVP_ServerActor_Marine;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Human_Description
// Author       : Andy Tate
// Date         : 9-Nov-2006
//
// Description  : Class used to contain the appropriate descriptions for the
//				  human skins.
////////////////////////////////////////////////////////////////////////////////

class AVP_Human_Description
{
public:
	// Public enumerated types...
	enum BlueprintType
	{ 
		// MUST BE KEPT UP TO DATE WITH s_aszBlueprints 
		// MUST ONLY BE ADDED TO AT THE BOTTOM (saved in chunk)
		AVP_HUMAN_MALE_COLONIAL_MARINE,
		AVP_HUMAN_MALE_COLONIAL_MARINE_VAR_01,
		AVP_HUMAN_MALE_COLONIAL_MARINE_VAR_02,

		AVP_NUM_HUMAN_BLUEPRINT_TYPES
	};

	enum MarkerPointType
	{
		MPT_BLOOD_SPINE,
		MPT_BLOOD_NECK,
		MPT_BLOOD_R_UPPERARM,
		MPT_BLOOD_R_FOREARM,
		MPT_BLOOD_L_UPPERARM,
		MPT_BLOOD_L_FOREARM,
		MPT_BLOOD_R_THIGH,
		MPT_BLOOD_R_CALF,
		MPT_BLOOD_L_THIGH,
		MPT_BLOOD_L_CALF,

		MPT_CAMERA,
		MPT_LEFT_HAND,
		MPT_RIGHT_HAND,
		MPT_TORCH,
		MPT_ARMOUR_FRONT,
		MPT_ARMOUR_BACK,
		MPT_HELMET,
		MPT_GRENADE_TAIL,

		NUM_MARKER_POINT_IDS
	};

	enum ThirdPersonBoneID
	{
		TP_BIP01,				// 0
		TP_BIP01_PELVIS,		// 1
		TP_TURRET,				// 2
		TP_BIP01_SPINE,			// 3
		TP_BIP01_SPINE1,		// 4
		TP_BIP01_SPINE2,		// 5
		TP_BIP01_NECK,			// 6
		TP_BIP01_HEAD,			// 7
		TP_BIP01_L_CLAVICLE,	// 8
		TP_BIP01_L_UPPERARM,	// 9
		TP_BIP01_L_FOREARM,		// 10
		TP_BIP01_L_HAND,		// 11
		TP_BIP01_L_FINGER0,		// 12
		TP_BIP01_L_FINGER01,	// 13
		TP_BIP01_L_FINGER1,		// 14
		TP_BIP01_L_FINGER11,	// 15
		TP_BIP01_L_FINGER2,		// 16
		TP_BIP01_L_FINGER21,	// 17
		TP_BIP01_L_FINGER3,		// 18
		TP_BIP01_L_FINGER31,	// 19
		TP_BIP01_L_FINGER4,		// 20
		TP_BIP01_L_FINGER41,	// 21
		TP_BIP01_L_FORETWIST,	// 22
		TP_BIP01_R_CLAVICLE,	// 23
		TP_BIP01_R_UPPERARM,	// 24
		TP_BIP01_R_FOREARM,		// 25
		TP_BIP01_R_HAND,		// 26
		TP_BIP01_R_FINGER0,		// 27
		TP_BIP01_R_FINGER01,	// 28
		TP_BIP01_R_FINGER1,		// 29
		TP_BIP01_R_FINGER11,	// 30
		TP_BIP01_R_FINGER2,		// 31
		TP_BIP01_R_FINGER21,	// 32
		TP_BIP01_R_FINGER3,		// 33
		TP_BIP01_R_FINGER31,	// 34
		TP_BIP01_R_FINGER4,		// 35
		TP_BIP01_R_FINGER41,	// 36
		TP_BIP01_R_FORETWIST,	// 37
		TP_BIP01_L_THIGH,		// 38
		TP_BIP01_L_CALF,		// 39
		TP_BIP01_L_FOOT,		// 40
		TP_BIP01_L_TOE0,		// 41
		TP_BIP01_R_THIGH,		// 42
		TP_BIP01_R_CALF,		// 43
		TP_BIP01_R_FOOT,		// 44
		TP_BIP01_R_TOE0,		// 45
		TP_L_SHOULDER,			// 46
		TP_PAD_LEFT,			// 47
		TP_PAD_RIGHT,			// 48
		TP_R_SHOULDER,			// 49
		TP_LIGHTPITCH,			// 50
		TP_LIGHTYAW,			// 51
		TP_KNIFE,				// 52
		TP_REARPLATE,			// 53
		TP_SHELLS,				// 54
		TP_FRONT_FLAP,			// 55
		TP_BROWIN_L,			// 56
		TP_BROWIN_R,			// 57
		TP_BROWOUT_L,			// 58
		TP_BROWOUT_R,			// 59
		TP_CHEEK_BL,			// 60
		TP_CHEEK_BR,			// 61
		TP_CHEEK_LEFT,			// 62
		TP_CHEEK_RIGHT,			// 63
		TP_EYE_LEFT,			// 64
		TP_EYE_RIGHT,			// 65
		TP_JAW,					// 66
		TP_MOUTH_B,				// 67
		TP_MOUTH_BL,			// 68
		TP_MOUTH_BR,			// 69
		TP_LOWEREYELID_LEFT,	// 70
		TP_LOWEREYELID_RIGHT,	// 71
		TP_MOUTH_L,				// 72
		TP_MOUTH_R,				// 73
		TP_MOUTH_T,				// 74
		TP_MOUTH_TL,			// 75
		TP_MOUTH_TR,			// 76
		TP_NOSE_L,				// 77
		TP_NOSE_R,				// 78
		TP_UPPEREYELID_LEFT,	// 79
		TP_UPPEREYELID_RIGHT,	// 80

		MAX_THIRD_PERSON_BONES_IDS
	};							

	enum FirstPersonBoneID
	{
		FP_BIP01,					// 0
		FP_BIP01_PELVIS,			// 1	
		FP_BIP01_SPINE,				// 2	
		FP_BIP01_SPINE1,			// 3	
		FP_BIP01_SPINE2,			// 4	
		FP_BIP01_NECK,				// 5	
		FP_BIP01_L_CLAVICLE2,		// 6	
		FP_BIP01_L_UPPERARM2,		// 7	
		FP_BIP01_L_FOREARM,			// 8	
		FP_BIP01_L_HAND,			// 9	
		FP_BIP01_L_FINGER0,			// 10	
		FP_BIP01_L_FINGER01,		// 11	
		FP_BIP01_L_FINGER02,		// 12	
		FP_BIP01_L_FINGER1,			// 13	
		FP_BIP01_L_FINGER11,		// 14	
		FP_BIP01_L_FINGER12,		// 15	
		FP_BIP01_L_FINGER2,			// 16	
		FP_BIP01_L_FINGER21,		// 17	
		FP_BIP01_L_FINGER22,		// 18	
		FP_BIP01_L_FINGER3,			// 19	
		FP_BIP01_L_FINGER31,		// 20	
		FP_BIP01_L_FINGER32,		// 21	
		FP_BIP01_L_FINGER4,			// 22	
		FP_BIP01_L_FINGER41,		// 23	
		FP_BIP01_L_FINGER42,		// 24	
		FP_BIP01_L_FORETWIST,		// 25	
		FP_BIP01_LUPARMTWIST,		// 26	
		FP_SHOLDERFLAP_LEFT,		// 27	
		FP_BIP01_R_CLAVICLE,		// 28	
		FP_BIP01_R_UPPERARM,		// 29	
		FP_BIP01_R_FOREARM	,		// 30	
		FP_BIP01_R_HAND,			// 31	
		FP_BIP01_R_FINGER0,			// 32	
		FP_BIP01_R_FINGER01,		// 33	
		FP_BIP01_R_FINGER02,		// 34	
		FP_BIP01_R_FINGER1,			// 35	
		FP_BIP01_R_FINGER11,		// 36	
		FP_BIP01_R_FINGER12,		// 37	
		FP_BIP01_R_FINGER2,			// 38	
		FP_BIP01_R_FINGER21,		// 39	
		FP_BIP01_R_FINGER22,		// 40	
		FP_BIP01_R_FINGER3,			// 41	
		FP_BIP01_R_FINGER31,		// 42	
		FP_BIP01_R_FINGER32,		// 43	
		FP_BIP01_R_FINGER4,			// 44	
		FP_BIP01_R_FINGER41,		// 45	
		FP_BIP01_R_FINGER42,		// 46	
		FP_BIP01_R_FORETWIST,		// 47	
		FP_BIP01_RUPARMTWIST,		// 48	
		FP_SHOLDERFLAP_RIGHT,		// 49	
		FP_LIGHT,					// 50	
		FP_KNIFE,					// 51	
		FP_SHELLS,					// 52	
		FP_BIP01_L_THIGH,			// 53	
		FP_BIP01_L_CALF,			// 54	
		FP_BIP01_L_FOOT,			// 55	
		FP_BIP01_L_TOE0,			// 56	
		FP_BIP01_R_THIGH,			// 57	
		FP_BIP01_R_CALF,			// 58	
		FP_BIP01_R_FOOT,			// 59	
		FP_BIP01_R_TOE0,			// 60	
		FP_FRONT_FLAP,				// 61	
		FP_CAMERA_BONE,				// 62	

		MAX_FIRST_PERSON_BONES_IDS
	};

#if defined(ASURA_TOOLS)
	static void	AddMarineFilesToExport		(const bool bFirstPerson, const Asura_Hash_ID uBlueprintHash, List<Asura_String>& xNameList);
	static void	AddAndroidFilesToExport		(const bool bFirstPerson, const Asura_Hash_ID uBlueprintHash, List<Asura_String>& xNameList);
	static void	AddCivilianFilesToExport	(const bool bFirstPerson, const Asura_Hash_ID uBlueprintHash, List<Asura_String>& xNameList);
	static void AddDialogueFromBlueprint	(const Asura_Hash_ID uBlueprintHash);
#endif	// defined(ASURA_TOOLS)

	static Asura_Hash_ID	GetMarkerPointHashID(const MarkerPointType eID)	{ return(s_auMarkerPointHashIDs[eID]); }
	static int				GetFootBoneID( u_int uFootIndex, bool bFirstPerson ) { Asura_Assert( uFootIndex < AVP_ClientEntity_Actor::MAX_NUM_FEET, "Bad foot index" ); return bFirstPerson ? s_aiFirstPersonFootBoneID[ uFootIndex ] : s_aiThirdPersonFootBoneID[ uFootIndex ]; }

	static const int* 		GetFirstPersonLeftArmChainBonesArray(void)		{ return(s_aiFirstPersonLeftArmChainBonesArray); }
	static const int* 		GetFirstPersonLeftLegChainBonesArray(void)		{ return(s_aiFirstPersonLeftLegChainBonesArray); }
	static const int* 		GetFirstPersonRightLegChainBonesArray(void)		{ return(s_aiFirstPersonRightLegChainBonesArray); }
	static const int* 		GetFirstPersonLeftArmChainTwistBonesArray(void)	{ return(s_aiFirstPersonLeftArmChainTwistBonesArray); }

	static const int* 		GetThirdPersonLeftArmChainBonesArray(void)		{ return(s_aiThirdPersonLeftArmChainBonesArray); }
	static const int* 		GetThirdPersonLeftLegChainBonesArray(void)		{ return(s_aiThirdPersonLeftLegChainBonesArray); }
	static const int* 		GetThirdPersonRightLegChainBonesArray(void)		{ return(s_aiThirdPersonRightLegChainBonesArray); }
	static const int* 		GetThirdPersonLeftArmChainTwistBonesArray(void)	{ return(s_aiThirdPersonLeftArmChainTwistBonesArray); }

	virtual	bool			CanBeImpaled(const u_int uBoneID) const;

	static u_int			GetElementFromBoneID(int iBoneID);
	static int				GetAnyBoneIDFromElement(u_int uElement); // Since bones->elements is a many->one mapping, this function will return one of the bones associated with the given element, but not necessarily the only one.

protected:
	// Protected variables...
	static const Asura_Hash_ID	s_auMarkerPointHashIDs[NUM_MARKER_POINT_IDS];
	static const int			s_aiFirstPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ];
	static const int			s_aiThirdPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ];

	static const int 			s_aiFirstPersonLeftArmChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1 ];
	static const int 			s_aiFirstPersonLeftLegChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ];
	static const int 			s_aiFirstPersonRightLegChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ];

	static const int 			s_aiThirdPersonLeftArmChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ];
	static const int 			s_aiThirdPersonLeftLegChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ];
	static const int 			s_aiThirdPersonRightLegChainBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ];

	static const int 			s_aiFirstPersonLeftArmChainTwistBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ];
	static const int 			s_aiThirdPersonLeftArmChainTwistBonesArray[ uAVP_HUMAN_NUM_IK_BONES_PER_LIMB + 1  ];

	// Protected functions...


private:
	// Private variables...

	// Private functions...
	AVP_Human_Description(void);
	AVP_Human_Description(const AVP_Human_Description& xOther);
	~AVP_Human_Description();

	AVP_Human_Description&	operator=(const AVP_Human_Description& xOther);

};   // End of class 'AVP_Human_Description'.


#endif   // ifndef __AVP_HUMAN_DESCRIPTION_H__


/// End of file. ///
