
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_HUDHandler.h
// Author : Chris Wallace
// Date   : 8-Jun-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_HUMAN_HUDHANDLER_H__
#define __AVP_HUMAN_HUDHANDLER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Sound/Asura_Sound_Event_System.h"

#include "Actor/AVP_Actor_CameraHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_HUDHandler.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class AVP_ClientActor_Human;
class AVP_Human_MotionTracker;
class AVP_Human_HealthRenderer;

////////////////////////////////////////////////////////////////////////////////
// Variables...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Human_HUDHandler
// Author       : Chris Wallace
// Date         : 8-Jun-2007
//
// Description  : Class to the handle the Human specific HUD manipulation.
////////////////////////////////////////////////////////////////////////////////

class AVP_Human_HUDHandler : public AVP_Vertebrate_HUDHandler
{
public:
	// Public functions...
	explicit AVP_Human_HUDHandler ( AVP_ClientEntity_Actor& xClient );
	
	virtual ~AVP_Human_HUDHandler ();

	// Overridden functions...
	virtual	void	Update	( const AVP_ClientEntity_Actor& xClient );
	virtual void	Render	( const AVP_ClientEntity_Actor& xClient );
	virtual void	Reset	();

	void UpdateHUDCounter(u_int uCounterValue, int& iCounterUnits);
	void UpdateHUDCounter(u_int uCounterValue, int& iCounterUnits, int& iCounterTens);
	void UpdateHUDCounter(u_int uCounterValue, int& iCounterUnits, int& iCounterTens,  int& iCounterHundreds);
	void UpdateHUDCounter(u_int uNumCounters, u_int uCounterValue, u_int u_intMaxValue, int& iCounterUnits, int& iCounterTens, int& iCounterHundreds);
	
	void UpdateMotionTracker( const AVP_ClientActor_Human& xClient );
	void RenderMotionTracker( const AVP_ClientActor_Human& xClient );
	void DisruptMotionTracker();

	static int GetStaticStimAmmo(void)		{ return s_iStimAmmoCounter; }	
	static int GetStaticAmmo(void)			{ return s_iAmmoCounter; }	
	static int GetStaticGrenadeAmmo(void)	{ return s_iGrenadeAmmoCounter; }
	static int GetStaticAmmoMax(void)		{ return s_iAmmoCounterMax; }	
	static int GetStaticGrenadeAmmoMax(void){ return s_iGrenadeAmmoCounterMax; }
	static int GetStaticArmour(void)		{ return s_iArmourCounter; }
	static int GetStaticHeartRate(void)		{ return s_iHeartRateCounter; }
	static bool GetStaticCanMantle()		{ return s_bMantleAvailable; }
	
	static void SetMotionTrackerActive( bool bMotionTrackerActive ){ s_bMotionTrackerActive = bMotionTrackerActive; }
	static void SetTorchActive( bool bTorchActive ){ s_bTorchActive = bTorchActive; }

	virtual void OnBeginSniperScope(void);
	virtual void OnEndSniperScope(void);

	// Create console vars
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	static void CreateConsoleVars();
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)	

protected:
	// Protected variables...
	
	u_int	m_uEnergy;
	float	m_fHealth;
	float	m_fArmour;
	
	static int s_iStimAmmoCounter;
	static int s_iAmmoCounter;
	static int s_iGrenadeAmmoCounter;
	static int s_iAmmoCounterMax;
	static int s_iGrenadeAmmoCounterMax;
	static int s_iArmourCounter;
	static int s_iHeartRateCounter;
	static bool s_bMantleAvailable;

	float	m_fHUDScaleH;
	float	m_fHUDScaleV;

	// Protected functions...

	virtual void GetReticlePosition(float& fX, float &fY);

private:
	// Private typedefs...
	typedef AVP_Vertebrate_HUDHandler PARENT;

	float m_fMenuPhase;
	AVP_Human_MotionTracker* m_pxMotionTracker;

	// Private functions...
	AVP_Human_HUDHandler  (const AVP_Human_HUDHandler& xOther);

	AVP_Human_HUDHandler&	operator=	(const AVP_Human_HUDHandler& xOther);

	void RenderText			(void);
	void RenderPanicPrompts (void) {};
	void RenderFocusTarget(void);
	void RenderTargetName() const;

	void RefreshBlueprintValues(void);

	bool m_bIsAiming;

	static bool s_bGotStaticBlueprintValues;

	static bool s_bHasSecondaryAmmo;

	static bool s_bLastClip;

	static bool s_bLowAmmoInClip;

	static bool s_bShowAimingHUD;

	static bool s_bMotionTrackerActive;

	static bool s_bTorchActive;

};   // End of class 'AVP_Predator_HUDHandler'.


#endif   // ifndef __AVP_PREDATOR_HUDHANDLER_H__

/// End of file. ///
