
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_InputHandler.cpp
// Author : Chris Wallace
// Date   : 5-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Human/AVP_Human_InputHandler.h"

// Asura independent header-files...
#include "Core/Asura_BitPacker.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_CommandConsole.h"
#include "Input/Asura_ControlMapping_Combination.h"
#include "Input/Asura_ControlMapping_Key.h"
#include "Network/Asura_Network.h"
#include "ProjectSettings/Asura_PhysicsConfiguration.h"
#include "Rendering/Asura_Render.h"

// Game independent header-files...
#include "Actor/AVP_Actor_AnimationHandler.h"
#include "Actor/AVP_Actor_CameraHandler.h"
#include "Actor/AVP_Actor_HUDHandler.h"
#include "Actor/AVP_Actor_InteractionHandler.h"
#include "Actor/AVP_Actor_Selector.h"
#include "Actor/Vertebrates/AVP_Vertebrate_CameraHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_JumpHandler.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Messages.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_HumanFlare.h"
#include "Inventory/Items/AVP_InventoryItem_Torch.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_ScopedRifle.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_Messages_Human.h"


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
int	AVP_Human_InputHandler::s_iConsoleVarsCount	= 0;
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)

DEBUGVAR float dbg_fZoomResetTime	= 1.0f;
DEBUGVAR float dbg_fZoomHoldTime	= 0.2f;

DVSVAR float dvs_fVisionNextHoldTime= 0.3f;

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

Asura_Message* AVP_Human_InputHandler::Update( AVP_ClientEntity_Actor& xClient )
{
	if ( !xClient.IsDead() )
	{
		AVP_ClientActor_Human& xHuman = static_cast< AVP_ClientActor_Human& >( xClient );

		AVP_Actor_InteractionHandler* pxInteractionHandler = xClient.GetInteractionHandler();
		if ( ( GetControlMapValue( INTERACT ) > 0.0f ) )
		{
			if ( pxInteractionHandler && pxInteractionHandler->IsInteractionAllowed() )
			{
				if ( pxInteractionHandler->HandleInteraction( AVP_INTERACTION_TYPE_PICKUP_BUDDY, INTERACT, GetControlMapValue( INTERACT ) ) )
				{
					// if we managed an interaction then consume the keypresses
					GetControlMapValue( INTERACT, true, true );
					GetControlMapValue( MELEE, true, true );
					GetControlMapValue( COUNTER, true, true );
					GetControlMapValue( GRAPPLE_INPUT, true, true );
				}
			}
			else
			{
				if ( static_cast< AVP_ClientActor_Human& >( xClient ).IsCarryingBudddy() )
				{
					if ( pxInteractionHandler->HandleInteraction(AVP_INTERACTION_TYPE_PICKUP_BUDDY, INTERACT, GetControlMapValue( INTERACT ) ) )
					{
						// if we managed an interaction then consume the keypresses
						GetControlMapValue( INTERACT, true, true );
						GetControlMapValue( MELEE, true, true );
						GetControlMapValue( COUNTER, true, true );
						GetControlMapValue( GRAPPLE_INPUT, true, true );
					}
				}
			}
		}
		const AVP_InventoryItem_Weapon* pxWeapon = xClient.GetCurrentWeapon();
		AVP_Inventory_System* pxInvSys = xHuman.GetInventorySystem();
		AVP_InventoryItem_HumanFlare* pxFlare = pxInvSys ? static_cast< AVP_InventoryItem_HumanFlare* >( pxInvSys->GetItem( uAVP_INVENTORY_ITEMHASH_FLARE ) ) : NULL;
		Asura_Assert( pxFlare, "Unable to get flare item" );
		if ( pxFlare && (static_cast< AVP_ClientActor_Human& >( xClient ).GetNumActiveFlares() < pxFlare->GetMaxActiveFlares()) && pxFlare->IsIdling( true ) && ( pxWeapon == NULL || !pxWeapon->IsUsing( false ) || pxWeapon->GetCurrentUsageType() != AVP_InventoryItem_Weapon::UT_RELOAD_ALL ) && GetControlMapValue( FLARE, true, true ) > 0.0f )
		{
			pxFlare->RequestBeginUsage( AVP_InventoryItem_HumanFlare::UT_FLARE );
		}

		bool bAllowTorch = true;
		if(pxWeapon)
		{
			if(pxWeapon->GetItemType() == AVP_INVENTORYITEMTYPE_SCOPEDRIFLE)
			{
				if(static_cast<const AVP_InventoryWeapon_ScopedRifle*>(pxWeapon)->IsInSniperMode(false)) bAllowTorch = false; // don't allow this in sniper mode
			}
		}
		// AJK; AVP Functionality for tutorial; check if func allowed
		if (bAllowTorch && AVP_Settings::IsTorchFuncOn())
		{
			AVP_InventoryItem_Torch* pxTorch = pxInvSys ? static_cast< AVP_InventoryItem_Torch* >( pxInvSys->GetItem( uAVP_INVENTORY_ITEMHASH_TORCH ) ) : NULL;
			Asura_Assert( pxTorch, "Unable to get torch item" );
			if ( pxTorch )
			{
				if ( pxTorch->IsIdling( true ) && GetControlMapValue( TORCH_TOGGLE ) > 0.0f )
				{
					pxTorch->RequestBeginUsage( AVP_InventoryItem_Torch::UT_TORCH, AVP_InventoryItem_Torch::TT_MANUAL );
				}
				else if ( pxTorch->IsUsing( true ) && GetControlMapValue( TORCH_TOGGLE ) > 0.0f )
				{
					pxTorch->RequestEndUsage( AVP_InventoryItem_Torch::TT_MANUAL );
				}
			}
		}

		if ( GetControlMapValue( AVP_Human_InputHandler::AIM_DOWN_SIGHTS ) > 0.5f )
		{
			xHuman.ToggleADS();
		}
	}

	return PARENT::Update( xClient );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Human_InputHandler::UpdateMovement
// Author       : Andy Tate
// Date         : 26-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Human_InputHandler::UpdateMovement ( AVP_ClientEntity_Actor& xClient )
{
//	AVP_ClientActor_Human& xHuman = static_cast< AVP_ClientActor_Human& >( xClient );

	// Jumping is now handled in the vertebrate.
	PARENT::UpdateMovement(xClient);
// 
// 	if ( GetControlMapValue( STANCE_TOGGLE, true, true ) > 0.1f )
// 	{
// 		if ( xHuman.IsCrouching() )
// 		{
// 			xHuman.SetCrouching( false, true );
// 		}
// 		else
// 		{
// 			xHuman.SetCrouching( true, true );
// 		}
// 	}
}

/// End of function 'AVP_Human_InputHandler::UpdateMovement'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Human_InputHandler::UpdateVisionModes
// Author       : Andy Tate
// Date         : 13-Feb-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Human_InputHandler::UpdateVisionModes ( AVP_ClientEntity_Actor& xClient )
{
	AVP_ClientActor_Human& xHuman = static_cast< AVP_ClientActor_Human& >( xClient );

	AVP_Inventory_System*		pxInventorySystem	= xHuman.GetInventorySystem();
	AVP_Inventory_Group*		pxVisionGroup		= (pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetGroup(uAVP_INVENTORY_GROUPHASH_VISIONMODE);

	if(GetControlMapValue(VISION_SELECTOR_NEXT, true, true) > 0.0f)
	{
		pxVisionGroup->ChangeToNextItem(true);
	}
	else if(GetControlMapValue(VISION_SELECTOR_PREV, true, true) > 0.0f)
	{
		pxVisionGroup->ChangeToPreviousItem(true);
	}
}

/// End of function 'AVP_Human_InputHandler::UpdateVisionModes'. ///

void AVP_Human_InputHandler::UpdateMeleeRequests( AVP_ClientActor_Vertebrate& xClient )
{
	if ( !static_cast< AVP_ClientActor_Human& >( xClient ).IsCarryingBudddy() )
	{
		PARENT::UpdateMeleeRequests( xClient );
	}
}

void AVP_Human_InputHandler::UpdateWeapons ( AVP_ClientEntity_Actor& xClient )
{
	//Will update only if the client is not carrying a battery
	const AVP_InventoryItem_Weapon* pxWeapon = xClient.GetCurrentWeapon();
	if( !pxWeapon || !pxWeapon->IsEncumberment())
	{
		bool bAllowStart = true;
		if( pxWeapon && ( pxWeapon->GetItemType() == AVP_INVENTORYITEMTYPE_SCOPEDRIFLE ) )
		{
			if(static_cast<const AVP_InventoryWeapon_ScopedRifle*>(pxWeapon)->IsInSniperMode(false)) bAllowStart = false; // don't allow this in sniper mode
		}

		AVP_ClientActor_Human& xHuman = static_cast< AVP_ClientActor_Human& >( xClient );

		AVP_Actor_Selector*		pxWeaponSelector	= xHuman.GetWeaponSelector();

		if(pxWeaponSelector)
		{
			if(bAllowStart)
			{
				if(GetControlMapValue(WEAPON_SELECTOR_NEXT, true, true) > 0.0f)
				{
					pxWeaponSelector->SelectNextItem( xClient, true );
				}
				else if(GetControlMapValue(WEAPON_SELECTOR_PREV, true, true) > 0.0f)
				{
					pxWeaponSelector->SelectPreviousItem( xClient, true );
				}
				else if(GetControlMapValue(WEAPON_SELECTOR_BEGIN, true, true) > 0.0f)
				{
					pxWeaponSelector->Activate( xClient );
				}
			}

			pxWeaponSelector->Update( xClient );

			if(pxWeaponSelector->IsReadyToClose())
			{
				pxWeaponSelector->Deactivate( xClient );
			}
		}
	}
}

DEBUGVAR float dbg_fAutoMantleMinInputMagSquared = 0.81f;
DEBUGVAR float dbg_fAutoMantleMinForwardDP = 0.866f;
DEBUGVAR float dbg_fAutoMantleMaxDistanceSquared = 0.64f;
DEBUGVAR float dbg_fAutoMantleMinFacingDP = 0.707f;

void AVP_Human_InputHandler::UpdateMovementInput(AVP_ClientEntity_Actor& xClient)
{
	PARENT::UpdateMovementInput(xClient);

	// Check for an automatic step-up mantle
	const AVP_ClientActor_Vertebrate& xVert = static_cast<AVP_ClientActor_Vertebrate&>(xClient);
	const AVP_Human_JumpHandler* pxJumpHandler = static_cast<const AVP_Human_JumpHandler*>(xVert.GetJumpHandler());
	if (pxJumpHandler && pxJumpHandler->GetNonJumpRequestMantleAvailable())
	{
		Asura_Vector_2 xMovementInput(GetControlMapValue(STRAFE_LEFT_RIGHT), GetControlMapValue(FORWARDS_BACKWARDS));

		if (xMovementInput.MagnitudeSquared() >= dbg_fAutoMantleMinInputMagSquared)
		{
			xMovementInput.Normalise();
			// Vector2 * forward vector = Vector2.y
			if (xMovementInput.y >= dbg_fAutoMantleMinForwardDP)
			{
				// Make sure we're close enough
				Asura_Vector_3 xOffset;
				pxJumpHandler->GetCurrentMantleTargetPosition(xOffset);
				xOffset -= xVert.GetCurrentPosition();
				xOffset.y = 0.0f;
				const float fMagSquared = xOffset.MagnitudeSquared();
				if (fMagSquared <= dbg_fAutoMantleMaxDistanceSquared)
				{
					// Finally, make sure we're moving roughly towards the attractor.
					const Asura_Matrix_3x3& xOwnerOri = xVert.GetCurrentOrientation();
					const Asura_Vector_3 xFacingDir(xOwnerOri.e13, xOwnerOri.e23, xOwnerOri.e33);
					Asura_Quat xTargetQuat;
					pxJumpHandler->GetCurrentMantleTargetOrientation(xTargetQuat);
					Asura_Matrix_3x3 xTargetMat;
					xTargetQuat.ToMatrix(xTargetMat);
					Asura_Vector_3 xTargetForward(xTargetMat.e13, xTargetMat.e23, xTargetMat.e33);
					if ((xFacingDir * xTargetForward) >= dbg_fAutoMantleMinFacingDP)
					{
						pxJumpHandler->SendMantleMoveRequest();
					}
				}
			}
		}
	}
}

void AVP_Human_InputHandler::UpdateZoom( AVP_ClientActor_Vertebrate& xClient )
{
	AVP_Vertebrate_CameraHandler* pxCameraHandler = static_cast<AVP_Vertebrate_CameraHandler*>(xClient.GetCameraHandler());
	if (pxCameraHandler)
	{
		// Neither zoom update nor activation necessary unless we are in first person camera
		if (pxCameraHandler->GetCameraType() != AVP_Actor_CameraHandler::CT_FIRST_PERSON) return;

		// Update the camera zoom *every* frame
		pxCameraHandler->UpdateZoomCamera( xClient );

		if(pxCameraHandler->GetZoomLevel()!=0)// We can change zoom levels only while already zoomed as a human
		{
			m_fZoomTimer += Asura_Timers::GetGameTimeSlice();
			const float fZoomHold = GetControlMapValue(ZOOM_HOLD);
			// Check for zoom controls

			if( fZoomHold > 0.0f )
			{
				if ( m_bZoomed == false )
				{
					m_fZoomHoldTimer += Asura_Timers::GetGameTimeSlice();
				}
			}
			else
			{
				m_bZoomed = false;
			}

			if ( m_fZoomHoldTimer > 0.0f )
			{
				if ( pxCameraHandler->GetZoomLevel() == 0 || m_fZoomHoldTimer > dbg_fZoomHoldTime )
				{
					pxCameraHandler->BeginZoomIn( xClient, true );
					m_bZoomed = true;
					m_fZoomHoldTimer = 0.0f;
					m_fZoomTimer = 0.0f;
				}
				else if ( fZoomHold == 0.0f )
				{
					if ( m_fZoomTimer > dbg_fZoomResetTime )
					{
						pxCameraHandler->SetZoomLevel( xClient, 0 );
					}
					else
					{
						pxCameraHandler->BeginZoomIn( xClient, true );
					}
					m_fZoomHoldTimer = 0.0f;
					m_fZoomTimer = 0.0f;
				}
			}
			if(GetControlMapValue(ZOOM_IN, true, true))
			{
				pxCameraHandler->BeginZoomIn( xClient );
			}
			else if(GetControlMapValue(ZOOM_OUT, true, true))
			{
				pxCameraHandler->BeginZoomOut( xClient );
			}
		}
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Human_InputHandler::SendPlayerCommandMessageToServer
// Author       : Andy Tate
// Date         : 4-Oct-2006
////////////////////////////////////////////////////////////////////////////////
Asura_Message* AVP_Human_InputHandler::CreatePlayerCommandMessageToServer ( const AVP_ClientEntity_Actor& xClient ) const
{
	bool bShouldCompressMessage = AVP_Game_Core::ShouldCompressPlayerCommandsMessages(); 
	
	if (bShouldCompressMessage)
	{
		// Fill message data
		const AVP_ClientActor_Human& xHuman = static_cast< const AVP_ClientActor_Human& >( xClient );
		AVP_Message_HumanPlayerCommands xHumanPlayerCommands( xHuman ); 
		const u_int uNoCompressionSize = sizeof( AVP_Message_HumanPlayerCommands ); 

		// Compress data
		Asura_BitPacker xPacker( uNoCompressionSize ); 
		xHumanPlayerCommands.Pack( xPacker ); 

		// Send the data
		const u_char* pucSrcData = 0;
		u_int uDataSize = 0;
		xPacker.GetResult( &pucSrcData, &uDataSize );

		Asura_Message*	pxMessage	= Asura_Network::CreateMessage( AVP_MSGID_HUMANPLAYERCOMMANDS_COMPRESSED, uDataSize );

		// Multiple sanity checks
		if (pxMessage && pucSrcData && uDataSize)
		{
			u_char* pucDstData = static_cast<u_char*>( pxMessage->GetData() ); 
			Asura_Mem::Copy( pucDstData, pucSrcData, uDataSize ); 
		}
		else
		{
			Asura_Assert(false, "Failed to create message.");
		}
		
		return pxMessage; 
	}
	else
	{
		Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_HUMANPLAYERCOMMANDS, sizeof(AVP_Message_HumanPlayerCommands));

		if (pxMessage == NULL)
		{
			Asura_Assert(false, "Failed to create message.");
		}
		else
		{
			AVP_Message_HumanPlayerCommands*	pxData = static_cast<AVP_Message_HumanPlayerCommands*>(pxMessage->GetData());

			if (pxData == NULL)
			{
				Asura_Assert(false, "Cannot find data in message.");
			}
			else
			{
				const AVP_ClientActor_Human& xHuman = static_cast< const AVP_ClientActor_Human& >( xClient );
				pxData->FillHuman( xHuman );
			}
		}

		// Return the message created...
		return(pxMessage);
	}
}

/// End of function 'AVP_Human_InputHandler::SendPlayerCommandMessageToServer'. ///


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Human_InputHandler::AVP_Human_InputHandler
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_Human_InputHandler::AVP_Human_InputHandler (const Asura_Guid uOwnerGuid)
: AVP_Vertebrate_InputHandler(uOwnerGuid)
{
	// Initialise the control maps..
	RefreshBlueprintData();
	Platform_InitialiseControlMap();

#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	if (s_iConsoleVarsCount == 0)
	{
		if ( !Asura_CommandConsole::VarExists( L"Controls.NightVision.HoldButtonTime" ) )
		{
			Asura_CommandConsole::AddVar( L"Controls.NightVision.HoldButtonTime", &dvs_fVisionNextHoldTime );
		}
	}
	++s_iConsoleVarsCount;
#endif //#ifdef ASURA_SUPPORT_COMMAND_CONSOLE

}


AVP_Human_InputHandler::~AVP_Human_InputHandler()
{
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	--s_iConsoleVarsCount;
#endif //#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
}

void AVP_Human_InputHandler::MapVisionAndFlare(Asura_ControlMap& xMap, const u_int uKeyVisionNext, const u_int uKeyFlare)
{
	if (uKeyVisionNext == uKeyFlare)
	{
		xMap.AddTapKeyMapping(FLARE, uKeyFlare, dvs_fVisionNextHoldTime);
		xMap.AddHoldKeyMapping(VISION_SELECTOR_NEXT, uKeyVisionNext, dvs_fVisionNextHoldTime, 1.0f, Asura_ControlMapping_HoldKey::NO_RAMPING, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	}
	else
	{
		xMap.AddKeyMapping(FLARE, uKeyFlare, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
		xMap.AddKeyMapping(VISION_SELECTOR_NEXT, uKeyVisionNext, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	}
}

void AVP_Human_InputHandler::MapTorch(Asura_ControlMap& xMap,
										const u_int uKey)
{
	xMap.AddKeyMapping(TORCH_TOGGLE, uKey, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
}


void AVP_Human_InputHandler::MapFlare(Asura_ControlMap& xMap,
									  const u_int uKey)
{
	xMap.AddKeyMapping(FLARE, uKey, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
}

void AVP_Human_InputHandler::MapADS( Asura_ControlMap& xMap, const u_int uKey )
{
	xMap.AddKeyMapping(AIM_DOWN_SIGHTS, uKey, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
}


/// End of file. ///
