//-----------------------------------------------------------------------------
// File   : AVP_Human_MoraleHandler.h
// Author : Gareth Thomas
// Date   : 08 / 02 / 2008
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//-----------------------------------------------------------------------------

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE
#ifndef __AVP_HUMAN_MORALEHANDLER_H__
#define __AVP_HUMAN_MORALEHANDLER_H__

//-----------------------------------------------------------------------------
// Include files...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Forward class declarations...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Class declaration...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Class        : AVP_Human_MoraleHandler
// Author       : Gareth Thomas
// Date         : 08 / 02 / 2008
//
// Description  : This class handles the operation of the Human morale system.
//-----------------------------------------------------------------------------
class AVP_Human_MoraleHandler
{

public:

	enum MoraleType
	{
		LOW_MORALE = 0,
		NORMAL_MORALE,
		HIGH_MORALE,
		MAX_MORALE_TYPES
	};

	enum MoraleEffectors
	{
		// Negative Effectors
		AVP_MORALE_EFFECTOR_HEALTH_DROPPING_BELOW_50_PERCENT = 0,
		AVP_MORALE_EFFECTOR_RELOADING_WHILE_UNDER_THREAT,
		AVP_MORALE_EFFECTOR_HIT_BY_ACID,
		AVP_MORALE_EFFECTOR_SQUAD_MEMBER_KILLED,
		AVP_MORALE_EFFECTOR_LOST_A_LIMB,

		// Positive Effectors
		AVP_MORALE_EFFECTOR_HEALTH_RECOVERED_ABOVE_70_PERCENT,
		AVP_MORALE_EFFECTOR_SQUAD_MEMBER_KILLED_ALIEN,
		AVP_MORALE_EFFECTOR_SQUAD_MEMBER_KILLED_ANDROID,
		AVP_MORALE_EFFECTOR_SQUAD_MEMBER_KILLED_PREDATOR,
		AVP_MORALE_EFFECTOR_KILLED_ALIEN,
		AVP_MORALE_EFFECTOR_KILLED_ANDROID,
		AVP_MORALE_EFFECTOR_KILLED_PREDATOR,

		// Max Effectors - MUST COME LAST!
		MAX_EFFECTORS
	};

	AVP_Human_MoraleHandler						(Asura_Guid uOwnerGuid);
	~AVP_Human_MoraleHandler					(void);

	// Functions
	void		InitaliseFromBlueprint			(void);
	void		Update							(void);
	void		EffectMorale					(u_int eEffectorType);

	// Data Access
	u_int		GetMoraleValue					(void)									{return m_eMoraleType;}
	float		GetCurrentValue					(void)									{return m_fCurrentValue;}
	Asura_Guid	GetOwnerGuid					(void)									{return m_uOwnerGuid;}
	float		GetNormalMoraleLowPoint			(void)									{return m_fNormalMoraleLowPoint;}
	float		GetNormalMoraleHighPoint		(void)									{return m_fNormalMoraleHighPoint;}
	float		GetLowMoraleNormalPoint			(void)									{return m_fLowMoraleNormalPoint;}
	float		GetHighMoraleNormalPoint		(void)									{return m_fHighMoraleNormalPoint;}
	float		GetMaximumMorale				(void)									{return m_fMaxMorale;}
	float		GetMinimumMorale				(void)									{return m_fMinMorale;}

	void		SetDesiredValue					(const float fDesiredMorale)			{m_fDesiredValue = m_fDesiredValue;}
	void		SetNormalMoraleLowPoint			(const float fValue)					{m_fNormalMoraleLowPoint = fValue;}
	void		SetNormalMoraleHighPoint		(const float fValue)					{m_fNormalMoraleHighPoint = fValue;}
	void		SetLowMoraleNormalPoint			(const float fValue)					{m_fLowMoraleNormalPoint = fValue;}
	void		SetHighMoraleNormalPoint		(const float fValue)					{m_fHighMoraleNormalPoint = fValue;}
	void		SetMaximumMorale				(const float fValue)					{m_fMaxMorale = fValue;}
	void		SetMinimumMorale				(const float fValue)					{m_fMinMorale = fValue;}

private:

	// Private Member Functions...
	void	UpdateMoraleType();
	float	CalculateDegradationIfNeeded(const float fTimeslice);
	void	UpdateMorale(const float fTimeslice, const float fDegradationSpeed);

	// Private Member Variables...
	Asura_Guid	m_uOwnerGuid;					// The Guid of the actor who owns this morale handler.
	float		m_fInitialValue;				// The Initial Starting value of the actors morale.
	float		m_fCurrentValue;				// The current value of the morale.
	float		m_fDesiredValue;				// The desired value the morale should move towards until it is at it.
	float		m_fMoraleMoverate;				// The rate at which the.
	float		m_fMinMorale;					// The minimum value which morale can fall to.
	float		m_fMaxMorale;					// The maximum value which morale can rise to.
	float		m_fNormalMoraleLowPoint;		// The point at which normal morale turns into low morale.
	float		m_fNormalMoraleHighPoint;		// The point at which normal morale turns into high morale.
	float		m_fLowMoraleNormalPoint;		// The point at which low morale turns into normal morale.
	float		m_fHighMoraleNormalPoint;		// The point at which high morale turns into normal morale.
	MoraleType	m_eMoraleType;					// Defined enum to check against rather than doing float checks.
	bool		m_bDegrading;					// Flag value used to determine if morale is degrading or not.
	float		m_fDegradationTimer;			// Timer to indicate when to start naturally degrading morale.
	float		m_fTimeBeforeDegradation;		// The time that must pass before a humans morale begins to move towards the initial value.
	float		m_fEffectors[MAX_EFFECTORS];
};

#endif //__AVP_HUMAN_MORALEHANDLER_H__
/// End of file.
