
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_PlayerAnimStateData.cpp
// Author : Will Plano
// Date   : 27/09/2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Human/AVP_Human_PlayerAnimStateData.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_Hash.h"
#include "Dynamics/Asura_Collision_Object.h"

// AVP
#include "Actor/AVP_Actor_AnimStateData.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Settings.h"
#include "Physics/AVP_Collision_Descriptor.h"

#ifdef ASURA_TOOLS
#include "Blueprint/ToolsAVP_Blueprint_AnimSpreadsheetExporter.h"
#endif

// Globals 
AVP_Human_PlayerAnimStateData AVP_Human_PlayerAnimStateData::s_xSingleton; 

AVP_Human_PlayerAnimStateData::AVP_Human_PlayerAnimWeaponStateData::AVP_Human_PlayerAnimWeaponStateData()
{
	for ( u_int uHudItr = 0; uHudItr < 2; ++uHudItr )
	{
		for ( u_int uStateItr = 0; uStateItr < AVP_Human_PlayerAnimStateData::GetSingleton().GetNumStates(); ++uStateItr )
		{
			for ( u_int uSpeedItr = 0; uSpeedItr < AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_END; ++uSpeedItr  )
			{
				m_aaapxAnimStateData[ uHudItr ][ uStateItr ][ uSpeedItr ] = NULL;
			}
		}
	}
}

AVP_Human_PlayerAnimStateData::AVP_Human_PlayerAnimWeaponStateData::~AVP_Human_PlayerAnimWeaponStateData()
{
	for ( u_int uHudItr = 0; uHudItr < 2; ++uHudItr )
	{
		for ( u_int uStateItr = 0; uStateItr < AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_END_NOAIM; ++uStateItr )
		{
			for ( u_int uSpeedItr = 0; uSpeedItr < AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_END; ++uSpeedItr  )
			{
				delete m_aaapxAnimStateData[ uHudItr ][ uStateItr ][ uSpeedItr ];
			}
		}
	}
}

const AVP_Actor_AnimStateData* AVP_Human_PlayerAnimStateData::GetAnimStateData( const Asura_Hash_ID uAnimStateBlueprintType, const Asura_Hash_ID uCurrentWeaponHash, const bool bFirstPerson, const bool bAiming, const bool bCrouching, const bool bJumping, const bool bFalling, AVP_Vertebrate_PlayerAnimStateHandler::MovementSpeed eMoveSpeed )
{
	const AVP_Human_PlayerAnimStateData::AVP_Human_PlayerAnimWeaponStateData* pxWeaponData = GetWeaponDataByHash( uAnimStateBlueprintType, uCurrentWeaponHash );
	if ( pxWeaponData != NULL )
	{
		if ( ( bCrouching || bAiming || bJumping || bFalling ) && eMoveSpeed != AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_IDLE )
		{
			eMoveSpeed = AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_SLOW;
		}

		u_int uState;
		if (bJumping)
		{
			uState = AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_JUMPING;
		}
		else if (bFalling)
		{
			uState = AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_FALLING;
		}
		// 		else if (bAiming)
		// 		{
		// 			uState = bCrouching ? AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_ALTAIM : AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_NORMALAIM;
		// 		}
		else
		{
			uState = bCrouching ? AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_ALTPOSTURE : AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_NORMAL;
		}

		const u_int uFirstPersonState = bFirstPerson ? 0 : 1;
		return pxWeaponData->m_aaapxAnimStateData[ uFirstPersonState ][ uState ][ eMoveSpeed ];
	}
	return NULL;
}

#ifdef ASURA_TOOLS

void AVP_Human_PlayerAnimStateData::AVP_Human_PlayerAnimWeaponStateData::InitialiseFromBlueprint( const Asura_Hash_ID uAnimStateBlueprintType, const Asura_Hash_ID uWeaponBlueprintHash )
{
	const Asura_Blueprint* pxWeaponBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uWeaponBlueprintHash );
	Asura_Assertf( pxWeaponBlueprint, ( "Unable to get weapon blueprint 0x%08x", uWeaponBlueprintHash ) );
	if ( pxWeaponBlueprint )
	{
		for ( u_int uHudItr = 0; uHudItr < 2; ++uHudItr )
		{
			const Asura_Blueprint* pxStateBP = NULL;
			Asura_Value xValue;
			Asura_Hash_ID uStateBPParam = uHudItr == 0 ? uAVP_BLUEPRINT_PARAM_ANIMSTATE_BLUEPRINT_HUD : uAVP_BLUEPRINT_PARAM_ANIMSTATE_BLUEPRINT_TP;
			if ( pxWeaponBlueprint->GetParameterValueByHashID( uStateBPParam, xValue ) )
			{
				Asura_Hash_ID uWeaponBPHash = xValue.GetHashIDValue();
				if ( uWeaponBPHash != ASURA_HASH_ID_UNSET )
				{
					pxStateBP = Asura_Blueprint_System::GetBlueprintByHashID( uAnimStateBlueprintType, uWeaponBPHash );
					Asura_Assertf( pxStateBP, ( "Couldn't get human state BP with hash 0x%08x", uWeaponBPHash ) );
					if ( pxStateBP )
					{
						if ( ToolsAVP_Blueprint_AnimSpreadsheetExporter::IsActive() )
						{
							ToolsAVP_Blueprint_AnimSpreadsheetExporter::CreateFile( ( Asura_String( "Human_" ) + pxStateBP->GetName() + ".csv" ).GetString() );
						}
					}
				}
			}
#ifdef ASURA_ASSERT
			else
			{
				Asura_Assertf( false, ( "Unable to get actor state blueprint parameter 0x%08x from weapon blueprint 0x%08x", uStateBPParam, uWeaponBlueprintHash ) );
			}
#endif

			const u_int uNumStates = GetNumStates();
			for ( u_int uStateItr = 0; uStateItr < AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_END_NOAIM; ++uStateItr )
			{
				const bool bStateValid = ( uStateItr < uNumStates );
				const u_int uNumSpeed = GetNumSpeeds(uStateItr);
				for ( u_int uSpeedItr = 0; uSpeedItr < AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_END; ++uSpeedItr )
				{
					const bool bSpeedValid = ( uSpeedItr < uNumSpeed );
					if ( bStateValid && bSpeedValid && pxStateBP )
					{
						if ( uSpeedItr == AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_IDLE )
						{
							m_aaapxAnimStateData[ uHudItr ][ uStateItr ][ uSpeedItr ] = new AVP_Actor_AnimStateData_NonDirectional();
							m_aaapxAnimStateData[ uHudItr ][ uStateItr ][ uSpeedItr ]->InitialiseFromBlueprint( uAnimStateBlueprintType, *pxStateBP, GetStateDescription( uStateItr, uSpeedItr ).GetString(), true );
						}
						else
						{
							m_aaapxAnimStateData[ uHudItr ][ uStateItr ][ uSpeedItr ] = new AVP_Actor_AnimStateData_Directional();
							m_aaapxAnimStateData[ uHudItr ][ uStateItr ][ uSpeedItr ]->InitialiseFromBlueprint( uAnimStateBlueprintType, *pxStateBP, GetStateDescription( uStateItr, uSpeedItr ).GetString(), true );
						}
						if ( ToolsAVP_Blueprint_AnimSpreadsheetExporter::IsActive() )
						{
							ToolsAVP_Blueprint_AnimSpreadsheetExporter::WriteState( GetStateDescription( uStateItr, uSpeedItr ).GetString(), m_aaapxAnimStateData[ uHudItr ][ uStateItr ][ uSpeedItr ], pxStateBP, uAnimStateBlueprintType );
						}
					}
					else
					{
						m_aaapxAnimStateData[ uHudItr ][ uStateItr ][ uSpeedItr ] = NULL;
					}
				} // end speed loop
			} // end state loop
		} // end hud loop
	} // if weapon blueprint exists
}

#endif

const AVP_Human_PlayerAnimStateData::AVP_Human_PlayerAnimWeaponStateData* AVP_Human_PlayerAnimStateData::GetWeaponDataByHash( const Asura_Hash_ID uAnimStateBlueprintType, const Asura_Hash_ID uHash )
{
	AVP_Human_PlayerAnimWeaponStateData* pxData = m_xHashTree.GetDataByHash( uHash, false );
#ifdef ASURA_TOOLS
	if ( pxData == NULL && uHash != ASURA_HASH_ID_UNSET )
	{
		pxData = m_xHashTree.AddNode( uHash );
		Asura_Assert( pxData != NULL, "Unable to add new anim state data to tree" );
		if ( pxData != NULL )
		{
			pxData->InitialiseFromBlueprint( uAnimStateBlueprintType, uHash );
		}
	}
#endif
	Asura_Assertf( pxData || uHash == ASURA_HASH_ID_UNSET, ( "Unable to get weapon state data for weapon 0x%08x, do characters or common need re-exporting?", uHash ) );
	return pxData;
}

/// End of file. ///
