
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predalien_AnimationHandler.cpp
// Author : Andy Tate
// Date   : 18-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Predalien/AVP_Predalien_AnimationHandler.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationBlend.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "Entity/PhysicalObject/Moves/Asura_Moves.h"
#include "Physics/Asura_Physics_Ragdoll.h"
#include "SpecialFX/pfx/Asura_PFX_Entity_Effect.h"

// Asura platform header-files...

// Game independent header-files...
#include "Actor/Vertebrates/Predalien/AVP_Predalien_Description.h"
#include "Actor/AVP_Actor_InteractionHandler.h"
#include "Animation/AVP_AnimFlags.h"
#include "Animation/AVP_Animation_BodyRig.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_ClientActor_Predalien.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Moves/Vertebrates/Predalien/AVP_Predalien_MovesMelee.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const Asura_AVector3 g_xDownVector( 0.0f, 1.0f, 0.0f );
static const Asura_Vector_3 g_xTailStabPos( 0.4f, -0.8f, 2.8f ); // This is the object space position of the last bone in the tail at the end of the tail stab anim

const Asura_Hash_ID uAVP_PREDALIEN_MOUTH_START_PARAMETER_HASH = Asura_GetHashID("MouthStart");
const Asura_Hash_ID uAVP_PREDALIEN_MOUTH_IDLE_PARAMETER_HASH = Asura_GetHashID("MouthIdle");
const Asura_Hash_ID uAVP_PREDALIEN_MOUTH_FINISH_PARAMETER_HASH = Asura_GetHashID("MouthFinish");

DEBUGVAR float dbg_fRagdollFadePFXTime = 0.5f;

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predalien_AnimationHandler::AVP_Predalien_AnimationHandler
// Author       : Andy Tate
// Date         : 18-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_Predalien_AnimationHandler::AVP_Predalien_AnimationHandler(AVP_ClientActor_Predalien& xOwner)
: PARENT(xOwner)
, m_xOwnerPredalien(xOwner)
, m_pxTailStabBoneModifier(NULL)
, m_xTailStabTargetPos(g_xTailStabPos)
, m_uTailStabEntityTargetGuid(ASURA_GUID_UNREGISTERED)
, m_uTailStabEntityTargetMarkerPoint(ASURA_HASH_ID_UNSET)
, m_uRagdollFadingPFXHash(ASURA_HASH_ID_UNSET)
, m_bTailStabbing(false)
, m_bTailStabUsingWorldspaceTarget(false)
, m_bTailStabUsingEntityTarget(false)
, m_fPFXTimer(dbg_fRagdollFadePFXTime)
{
	const Asura_Blueprint* pxBP = m_xOwnerPredalien.GetBlueprint();
	Asura_Value xValue;
	if ( pxBP && pxBP->GetParameterValueByHashID( 0xB90B66C0, xValue ) ) // AcidDamage.PFX
	{
		m_uRagdollFadingPFXHash	= xValue.GetHashIDValue();
	}
}

/// End of function 'AVP_Predalien_AnimationHandler::AVP_Predalien_AnimationHandler'. ///

AVP_Predalien_AnimationHandler::~AVP_Predalien_AnimationHandler()
{
}

void AVP_Predalien_AnimationHandler::Initialise( void )
{
	PARENT::Initialise();

	if ( m_pxBodyRig )
	{
		m_pxBodyRig->SetMaxAimRightLeft( Asura_Maths::Deg90 );
	}

	InitialiseIKSolver();

	m_pxTailStabBoneModifier = new AVP_BoneDataModifier_TargetRelative( m_pxAnimation, g_xTailStabPos, uAVP_PREDALIEN_NUM_IK_BONES_PER_TAIL, AVP_Predalien_Description::GetTailChainArray(), true );
}

void AVP_Predalien_AnimationHandler::Shutdown( void )
{
	delete m_pxTailStabBoneModifier;
	PARENT::Shutdown();
}

void AVP_Predalien_AnimationHandler::HandleAnimationRequest( const AVP_Message_Actor_RequestAnimation& xAnimRequestData )
{
#if 0
    if ( m_xCurrentAnimSequenceData.IsValid() || m_xPendingAnimSequenceData.IsValid() )
    {
        //Asura_Assert( false, "AVP_Predalien_AnimationHandler::HandleAnimationRequest - shouldn't request an anim whilst playing a gamescene anim template" );
        return;
    }
  
	u_int uAnimFlags = AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE | AAC_FLAG_ENSURE_UNIQUE_HASH | AACA_FLAG_CALCULATE_VELOCITY;
    // Work out if we should add the loop flag.
    if ( xAnimRequestData.m_eLoopStatus == AVP_Message_Actor_RequestAnimation::LOOP_ON )
    {
        uAnimFlags |= AACA_FLAG_LOOPED;
    }
    else if ( xAnimRequestData.m_eLoopStatus == AVP_Message_Actor_RequestAnimation::LOOP_STATUS_UNSET )
    {
        const bool bAnimLoopsByDefault =
            ( xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_IDLE		||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_SNEAK		||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_SNEAK_REST	||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_WALK		||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_JOG		||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_RUN		||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_SPRINT		||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_PIN_LOOP	||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_CHARGE_ATTACK_RUN	||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_CROUCH_ATTACK_REST ||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_STAND_ATTACK_REST  ||
			  xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_CRAWL ||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_WALL_CRAWL ||
              xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_WALL_IDLE );
        uAnimFlags |= bAnimLoopsByDefault ? AACA_FLAG_LOOPED : 0;
    }
    // Work out if we should calculate angular velocity flag	
	if ( xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_SNEAK_TURN_90LEFT  || 
		 xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_SNEAK_TURN_90RIGHT ||
         xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_SNEAK_TURN_180	   ||
		 xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_STAND_ATTACK_TURN_LEFT ||
		 xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_STAND_ATTACK_TURN_RIGHT )
    {
       uAnimFlags |= AACA_FLAG_CALCULATE_ANGULAR_VELOCITY;
    }
    
    // use animations offset during this anim
    if ( xAnimRequestData.m_uAnimation == AVP_Predalien_Description::PREDALIEN_ANIM_TAIL_ATTACK_SPUN )
    {
		uAnimFlags |= AACA_FLAG_CALCULATE_MOVEMENT_OFFSET;
		uAnimFlags |= AACA_FLAG_CALCULATE_VELOCITY;
		uAnimFlags |= AACA_FLAG_CALCULATE_ANGULAR_VELOCITY;
	}

	u_int uHashId = AVP_Predalien_Description::GetAnimationHashID(
		static_cast<AVP_Predalien_Description::AnimationType>(
			xAnimRequestData.m_uAnimation));

	Asura_AnimationBlend* pxAnimBlend = new Asura_AnimationBlend_AnimatedTween( 0.2f, 0.0f, false );
	m_uCurrentRequestedAnimControllerHash = PlayAnimation( uAnimFlags, AVP_AAPC_REQUEST_ANIM, uHashId, pxAnimBlend );
#endif
}

void AVP_Predalien_AnimationHandler::Update()
{	
    PARENT::Update();

	const u_int uAnimFlags = GetAnimation()->GetFlagsForAllAnimsLastFrame();

	u_int u = 0;
	for ( ; u != AVP_Predalien_Description::TAIL; ++u )
	{
		m_axIKLOSChains[ u ].SetAnimVeto( ( uAnimFlags & AVP_ANIMFLAG_NOT_IK_FEET ) != 0 );
	}

	m_xTailIKChain.SetAnimVeto( ( uAnimFlags & AVP_ANIMFLAG_NOT_IK_HANDS_AND_TAIL ) != 0 );

	if ( m_pxTailStabBoneModifier )
	{
		if ( m_bTailStabbing )
		{
			m_xTailIKChain.SetAnimVeto( true );
			m_pxTailStabBoneModifier->TweenIn( DEFAULT_TWEEN_TIME );
			Asura_Vector_3 xOSTargetPos;
			if (CalculateTailStabTargetPosition(xOSTargetPos))
			{
				m_pxTailStabBoneModifier->SetOSTargetPosition(xOSTargetPos);
			}
		}
		else
		{
			m_pxTailStabBoneModifier->TweenOut( 0.5f );
		}
	}
}


const Asura_Animation_IK_Chain_LOS* AVP_Predalien_AnimationHandler::GetLimbIKChainLOS( u_int uLimbItr ) const
{
	if ( uLimbItr < AVP_Predalien_Description::TAIL )
	{
		return &m_axIKLOSChains[ uLimbItr ];
	}
	else
	{
		return NULL;
	}
}

void AVP_Predalien_AnimationHandler::InitialiseIKSolver()
{
	Asura_Assert( m_xIKSolver.GetFirstChain() == NULL, "IK Solver already initialised, clearing it..." );
	if ( m_xIKSolver.GetFirstChain() )
	{
		m_xIKSolver.Clear();
	}

	static const u_short s_ausClassificationExceptionArray[ 5 ] = { AVP_ENTITYCLASS_DISC, AVP_ENTITYCLASS_DROPSHIP_MISSILE, AVP_ENTITYCLASS_GENERICPROJECTILE, AVP_ENTITYCLASS_HUMANFLARE, AVP_ENTITYCLASS_PROJECTILE_ACIDSPIT };

	for ( u_int u = 0; u != AVP_Predalien_Description::NUM_IK_CHAINS; ++u )
	{
		if ( u < AVP_Predalien_Description::TAIL )
		{
			m_axIKLOSChains[ u ].SetBonesArrays( AVP_Predalien_Description::GetThirdPersonChainsArray(u), NULL );
			if ( m_pxAnimation->GetSkin() != NULL )
			{
				m_axIKLOSChains[ u ].SetAxesFromRestPose( m_pxAnimation->GetSkin()->BoneRestPose );
			}
			m_axIKLOSChains[ u ].SetIsRootBoneMover();
			m_axIKLOSChains[ u ].SetLocalLOSDirection( g_xDownVector );
			m_axIKLOSChains[ u ].SetExclusionLists( 4, s_ausClassificationExceptionArray );
			m_axIKLOSChains[ u ].SetTestAgainstEverything( m_uOwnerGuid );
			m_axIKLOSChains[ u ].SetExtendLimit( 0.01f );
			m_axIKLOSChains[ u ].SetDistanceFadeRange( 20.0f, 30.0f );
			m_xIKSolver.AddChain( m_axIKLOSChains[ u ] );
		}
		else if ( u == AVP_Predalien_Description::TAIL )
		{
			m_xTailIKChain.SetBonesArrays( AVP_Predalien_Description::GetTailChainArray(), NULL );
			m_xTailIKChain.SetDoJointConstraints();
			m_xTailIKChain.SetDoCollisionConstraints();
			m_xTailIKChain.SetBoneCollisionRadius( 0.1f );
			m_xTailIKChain.SetDistanceFadeRange( 50.0f, 70.0f );
			m_xIKSolver.AddChain( m_xTailIKChain );
		}
	}

	m_xIKSolver.SetRootRootBoneMaxMovement( 0.3f ); // max 30cm movement.
	EnableIKSolver();
}

void AVP_Predalien_AnimationHandler::EnableAllIKButTail()
{
	for ( u_int u = 0; u != AVP_Predalien_Description::TAIL; ++u )
	{
		m_xIKSolver.AddChain( m_axIKLOSChains[ u ] );
	}
}

void AVP_Predalien_AnimationHandler::DisableAllIKButTail()
{
	for ( u_int u = 0; u != AVP_Predalien_Description::TAIL; ++u )
	{
		m_xIKSolver.RemoveChain( m_axIKLOSChains[ u ] );
	}
}

void AVP_Predalien_AnimationHandler::EnableArmsIK()
{
}

void AVP_Predalien_AnimationHandler::DisableArmsIK()
{
}

bool AVP_Predalien_AnimationHandler::CalculateTailStabTargetPosition(Asura_Vector_3& xOSTargetPos) const
{
	Asura_Assert(m_bTailStabbing, "Shouldn't be trying to calculate a tail stab target position unless we're tail-stabbing.");

	if (m_bTailStabUsingEntityTarget)
	{
		// Entity target, get a worldspace position from them then convert that to object space.
		const AVP_ClientEntity_PhysicalObject* pxTarget = AVP_ClientEntity_PhysicalObject::FindEntity(m_uTailStabEntityTargetGuid);
		if (!pxTarget) { return false; }

		if (!pxTarget->GetMarkerLocationFromHash(m_uTailStabEntityTargetMarkerPoint, &xOSTargetPos, NULL))
		{
			xOSTargetPos = pxTarget->GetCurrentPosition();
		}

		const Asura_Vector_3& xOwnerPos = m_xOwnerActor.GetCurrentPosition();
		Asura_Quat xInverseOwnerOri(m_xOwnerActor.GetCurrentOrientation());
		xInverseOwnerOri.Invert();

		xOSTargetPos = xInverseOwnerOri.RotateVector(xOSTargetPos - xOwnerPos);
		return true;
	}
	else if (m_bTailStabUsingWorldspaceTarget)
	{
		// Worldspace stored target position, convert to object space.
		const Asura_Vector_3& xOwnerPos = m_xOwnerActor.GetCurrentPosition();
		Asura_Quat xInverseOwnerOri(m_xOwnerActor.GetCurrentOrientation());
		xInverseOwnerOri.Invert();

		xOSTargetPos = xInverseOwnerOri.RotateVector(m_xTailStabTargetPos - xOwnerPos);
		return true;
	}
	else
	{
		// Object space stored target position
		xOSTargetPos = m_xTailStabTargetPos;
		return true;
	}
}

void AVP_Predalien_AnimationHandler::UpdateRagdollFading( Asura_Ragdoll_Wrapper* pxRagdollrapper )
{
	PARENT::UpdateRagdollFading( pxRagdollrapper );

	// maybe do some alien acid PFX or something here?
	m_fPFXTimer -= Asura_Timers::GetGameTimeSlice();
	if ( m_fPFXTimer < 0.0f )
	{
		m_fPFXTimer = dbg_fRagdollFadePFXTime;
		int iBoneIdx = Asura_Random::GetInt( 0, m_pxAnimation->GetSkin()->NumberOfBones - 1 );
		Asura_Vector_3 xBonePos;
		Asura_Matrix_3x3 xBoneOri;
		m_pxAnimation->GetBoneLocation( iBoneIdx, &xBonePos, &xBoneOri, true );
		Asura_PFX_Entity_Effect::Create( xBonePos, xBoneOri, m_uRagdollFadingPFXHash, 0, ASURA_PFX_EFFECT_CAMERA_FLAGS );
	}
}

/// End of file. ///
