////////////////////////////////////////////////////////////////
//
// File:		AVP_Predalien_MeleeCounter_System.cpp
// Author:		Anthony Gregson
// Date:		22/02/08
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
////////////////////////////////////////////////////////////////

///////////////////////////////////////////////
// #includes
///////////////////////////////////////////////

#include "Core/Asura.h"

// This files header
#include "Actor/Vertebrates/Predalien/AVP_Predalien_MeleeCounter_System.h"

#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_LOS.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "Physics/Asura_Physics_MovementQuery.h"

#include "Actor/Vertebrates/AVP_Vertebrate_Description.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_MeleeCounter_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Container/AVP_ServerEntity_Container.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_ServerActor_Predalien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/Vertebrates/Predalien/AVP_Predalien_MovesMelee.h"
#include "Moves/Vertebrates/Predator/AVP_Predator_MovesMelee.h"

///////////////////////////////////////////////
// #defines
///////////////////////////////////////////////


DEBUGVAR float dbg_fRandomLeftOrRightClawAngleCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(30.0f));
DEBUGVAR float dbg_fRandomLeftOrRightTailAngleCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(150.0f));

DEBUGVAR float dbg_fLeapAttackDistanceBehindTarget = 0.0f;
DEBUGVAR float dbg_fLeapTimeEstimate = 0.5f;

DEBUGVAR float dbg_fGrappleAngleTang = Asura_Maths::Tangent(Asura_Maths::DegToRad(10.0f));
DEBUGVAR float dbg_fLeapDamageAngleTang = Asura_Maths::Tangent(Asura_Maths::DegToRad(15.0f));

/////////////////////////////////////////////////////////////////////
//
// Class :	AVP_Predalien_MeleeCounter_System
//
//////////////////////////////////////////////////////////////////////


AVP_Predalien_MeleeCounter_System::AVP_Predalien_MeleeCounter_System(AVP_ServerActor_Predalien& xOwner)
: AVP_Vertebrate_MeleeCounter_System(xOwner)
, m_bDoGrapple(false)
, m_bDoSuperJumpAttack(false)
, m_xOwnerStartLeapPosition(Asura_Maths::ZeroVector3)
, m_xTargetStartLeapPosition(Asura_Maths::ZeroVector3)
, m_xTailLastFlatDirection(Asura_Maths::ZeroVector3)
, m_fTailLastLenght(0.0f)
{
}


void AVP_Predalien_MeleeCounter_System::InitialiseFromBlueprint(const Asura_Blueprint& xBlueprint)
{
	PARENT::InitialiseFromBlueprint(xBlueprint);
}


bool AVP_Predalien_MeleeCounter_System::StartMeleeMove(u_int uType)
{

	if (uType == AVP_PREDPREDALIEN_MELEEATTACK_TYPE_LEAP_ATTACK)
	{
		m_uCurrentAttackType = uType;
		m_uNumHitEntities = 0;
		m_bCurrentAttackCountered = false;
		m_bCurrentAttackHasHitEnv = false;
		m_uCurrentGrappleBlueprint = ASURA_HASH_ID_UNSET;
		
		Asura_Vector_3 xTargetDestination;
		if (!CalculateLeapTargetPosition(xTargetDestination))
		{
			return false;
		}

		Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_MOVE_START, sizeof(AVP_Msg_MeleeWithTargetPosition_Move));

		if (pxMessage)
		{
			AVP_Msg_MeleeWithTargetPosition_Move* pxData = static_cast<AVP_Msg_MeleeWithTargetPosition_Move*>(pxMessage->GetData());

			pxData->ClearFlags();

			pxData->SetTargetPosition(xTargetDestination);

			pxData->SetVariation(0);
			pxData->SetPendingAttackType(uType);
			pxData->SetTargetGuid(m_uCurrentTarget);
			pxData->SetMoveID(GetMeleeMoveHash(uType));
			pxData->SetIgnoredByOwner(false);

			pxMessage->SetToGuid(m_xOwner.GetGuid());
			pxMessage->SetFromGuid(m_xOwner.GetGuid());

			Asura_Network::Send(pxMessage);
			return true;
		}
	}

	if (uType == AVP_PREDPREDALIEN_MELEEATTACK_TYPE_JUMP_DOWN_ATTACK)
	{
		m_uCurrentAttackType = uType;
		m_uNumHitEntities = 0;
		m_bCurrentAttackCountered = false;
		m_bCurrentAttackHasHitEnv = false;
		m_uCurrentGrappleBlueprint = ASURA_HASH_ID_UNSET;

		Asura_Vector_3 xTargetDestination;
		const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uCurrentTarget);
		if (!pxTarget)
		{
			return false;
		}

		xTargetDestination = pxTarget->GetPosition();

		Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_MOVE_START, sizeof(AVP_Msg_MeleeWithTargetPosition_Move));

		if (pxMessage)
		{
			AVP_Msg_MeleeWithTargetPosition_Move* pxData = static_cast<AVP_Msg_MeleeWithTargetPosition_Move*>(pxMessage->GetData());

			pxData->ClearFlags();

			pxData->SetTargetPosition(xTargetDestination);

			pxData->SetVariation(0);
			pxData->SetPendingAttackType(uType);
			pxData->SetTargetGuid(m_uCurrentTarget);
			pxData->SetMoveID(GetMeleeMoveHash(uType));
			pxData->SetIgnoredByOwner(false);

			pxMessage->SetToGuid(m_xOwner.GetGuid());
			pxMessage->SetFromGuid(m_xOwner.GetGuid());

			Asura_Network::Send(pxMessage);
			return true;
		}
	}
	
	Asura_Vector_3 xTailPosition;
	CalcluateTailDirection(m_fTailLastLenght, m_xTailLastFlatDirection, xTailPosition);

	return PARENT::StartMeleeMove(uType);
}

DEBUGVAR float dbg_fVerticalEpsilon = 0.5f;
DEBUGVAR float dbg_fHorizontalSuperHitRange = 0.75f;

void AVP_Predalien_MeleeCounter_System::CheckForAndDealMeleeDamage()
{
	if (m_uNumHitEntities >= MAX_HIT_MELEE_ENTITIES) { return; }

	for (u_int u=0; u<m_uNumHitEntities; ++u)
	{
		if (m_auHitEntities[u] == m_uCurrentTarget) { return; }
	}

	switch (m_uCurrentAttackType)
	{
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_JUMP_DOWN_ATTACK:
		{
			const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uCurrentTarget); 
			if (!pxTarget) { return; }
			const Asura_Vector_3& xTarget = pxTarget->GetPosition();
			const Asura_Vector_3& xOwner = m_xOwner.GetPosition();
			float fBBHight = GetTargetBBHight();
			float fVerdicalDelta = xTarget.y - xOwner.y;
			Asura_Vector_3 xHorDetla = xTarget-xOwner;
			xHorDetla.y = 0.0f;
			float fHorizontalDelta = xHorDetla.Normalise();
			// first check for super damage
			m_bDoSuperJumpAttack = false;
			if (fVerdicalDelta < fBBHight + dbg_fVerticalEpsilon && fHorizontalDelta < dbg_fHorizontalSuperHitRange)
			{
				Asura_Vector_3 xHitPosition = xTarget; 
				xHitPosition.y -= fBBHight;
				m_bDoSuperJumpAttack = true;
				Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_ENTITYDAMAGE_REQUEST_VICTIM, sizeof(AVP_Message_EntityDamage_RequestVictim));
				if(pxMessage)
				{
					AVP_Message_EntityDamage_RequestVictim*	pxData = static_cast<AVP_Message_EntityDamage_RequestVictim*>(pxMessage->GetData());
					const AVP_EntityDamage_RequestVictim
						xDamage(GetAttackDamageType(),
								m_xOwner.GetGuid(),
								uMELEE_MARKER_POINT_HASH,
								m_uCurrentTarget,
								Asura_Maths::LargeFloat, // kill 
								xHitPosition,
								xHorDetla, 
								-xHorDetla,
								GetAttackImpulseStrength(),
								-1, false,
								m_xOwner.GetGuid(),
								-1, 
								ASURA_HASH_ID_UNSET, 
								false, 
								false);
					pxData->SetDamage(xDamage);
					pxMessage->SetFromGuid(m_xOwner.GetGuid());
					pxMessage->SetToGuid(m_uCurrentTarget);
					Asura_Network::Send(pxMessage);
				}
				m_auHitEntities[m_uNumHitEntities++] = m_uCurrentTarget;

				return;
			}

			// now check for normal damage
			if (fVerdicalDelta < dbg_fVerticalEpsilon && fHorizontalDelta < m_fDamageRange)
			{
				ApplyMeleeDamage(m_uCurrentTarget);
			}
		}
		break;

	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_LEAP_ATTACK:
		{
			const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uCurrentTarget); 
			if (!pxTarget) { return; }
			const Asura_Vector_3& xTargetPosition = pxTarget->GetPosition();
			Asura_Vector_3 xJumpDirection = m_xTargetStartLeapPosition - m_xOwnerStartLeapPosition;
			xJumpDirection.Normalise();
			float fJumpDistance = (m_xOwnerStartLeapPosition - xTargetPosition).Magnitude();
			Asura_Vector_3 xTargetMovePerpToJump = xJumpDirection * ((xTargetPosition - m_xTargetStartLeapPosition) * xJumpDirection);
			Asura_Vector_3 xTargetMoveOrthoToJump  = xTargetPosition - m_xTargetStartLeapPosition - xTargetMovePerpToJump;
			float fTargetMoveOrthoToJump = xTargetMoveOrthoToJump.Magnitude();
			float fTangent = Asura_Maths::Abs(fTargetMoveOrthoToJump / fJumpDistance);
			if (fTangent < dbg_fGrappleAngleTang )
			{
				Asura_Vector_3 xOwnerPosition = m_xOwner.GetPosition();
				Asura_Vector_3 xToTarget = xTargetPosition - xOwnerPosition;
				float fToTarget = xToTarget.Normalise();
				const Asura_LOS_InputData xInputData( xOwnerPosition, xToTarget, m_xOwner.GetGuid(), fToTarget, false );
				const Asura_LOS_OutputData* pxOutputData = Asura_LOS::TestAgainstEverything( &xInputData );
				if (!pxOutputData->m_bHitEnvironment)
				{
					m_bDoGrapple = true;
					ApplyMeleeDamage(m_uCurrentTarget);
				}
			}
			else if (fTangent < dbg_fLeapDamageAngleTang )
			{
				Asura_Vector_3 xOwnerPosition = m_xOwner.GetPosition();
				Asura_Vector_3 xToTarget = xTargetPosition - xOwnerPosition;
				float fToTarget = xToTarget.Normalise();
				const Asura_LOS_InputData   xInputData( xOwnerPosition, xToTarget, m_xOwner.GetGuid(), fToTarget, false );
				const Asura_LOS_OutputData* pxOutputData = Asura_LOS::TestAgainstEverything( &xInputData );
				if (!pxOutputData->m_bHitEnvironment)
				{
					m_bDoGrapple = false;
					ApplyMeleeDamage(m_uCurrentTarget);
				}			
			}
			else
			{
				m_bDoGrapple = false;
			}
		}
		 break;
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_TAIL_ATTACK:
		{

			const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uCurrentTarget); 
			if (!pxTarget) { return; }
			const Asura_Vector_3& xTargetPosition = pxTarget->GetPosition();
			float fTailCurrentLenth;
			Asura_Vector_3 xTailCurrentFlatDirection;
			Asura_Vector_3 xCurrentTailPosition;
			if (!CalcluateTailDirection(fTailCurrentLenth, xTailCurrentFlatDirection, xCurrentTailPosition))
			{
				return;
			}
			// check if the target is between the current and last tail directions
			Asura_Vector_3 xToTarget = xTargetPosition - m_xOwner.GetPosition();
			xToTarget.y = 0.0f; // ignore hight
			float fBBHight = GetTargetBBHight();

			if (!xToTarget.ApproximatelyEquals(Asura_Maths::ZeroVector3))
			{
				float fToTarget = xToTarget.Normalise();
				if ( (xTailCurrentFlatDirection ^ xToTarget) * (m_xTailLastFlatDirection ^ xToTarget) <= 0.0f  && // not on the same side
					 fToTarget < fTailCurrentLenth && // close enough
					 xCurrentTailPosition.y < xTargetPosition.y && xCurrentTailPosition.y > xTargetPosition.y - fBBHight ) // tail hit body - vertically
				{
					ApplyMeleeDamage(m_uCurrentTarget);
				}
				
			}

			m_fTailLastLenght = fTailCurrentLenth;
			m_xTailLastFlatDirection = xTailCurrentFlatDirection;
		}
		break;
	default:
		PARENT::CheckForAndDealMeleeDamage();
		break;
	}
}

void AVP_Predalien_MeleeCounter_System::ApplyMeleeDamage(Asura_Guid uDamagedGuid)
{

	if (m_uCurrentAttackType == AVP_PREDPREDALIEN_MELEEATTACK_TYPE_LEAP_ATTACK)
	{
		if (!m_bDoGrapple)
		{
			PARENT::ApplyMeleeDamage(uDamagedGuid);
		}
		else
		{
			m_auHitEntities[m_uNumHitEntities++] = uDamagedGuid;
		}
// 		const AVP_ServerEntity_PhysicalObject* pxTarget = AVP_ServerEntity_PhysicalObject::FindEntity(uDamagedGuid);
// 
// 		// decide if grapple or normal damage is going to be applied
// 		if (pxTarget)
// 		{
// 			const Asura_Matrix_3x3& xOwnerOrinet = m_xOwner.GetOrientation();
// 			Asura_Vector_3 xOwnerForward(xOwnerOrinet.e13, xOwnerOrinet.e23, xOwnerOrinet.e33);
// 			xOwnerForward.y = 0.0f;
// 			xOwnerForward.Normalise();
// 			Asura_Vector_3 xToTarget = pxTarget->GetPosition() - m_xOwner.GetPosition();
// 			xToTarget.y = 0.0f;
// 			xToTarget.Normalise();
// 
// 			if (xToTarget * xOwnerForward > dbg_fGrappleAngle) // right in front of us - do a grapple
// 			{
// 				m_bDoGrapple = true;
// 				m_auHitEntities[m_uNumHitEntities++] = uDamagedGuid;
// 			}
// 			else
// 			{
// 				m_bDoGrapple = false;
// 				PARENT::ApplyMeleeDamage(uDamagedGuid);
// 			}
// 		}
		return;
	}

	PARENT::ApplyMeleeDamage(uDamagedGuid);
}


Asura_Hash_ID AVP_Predalien_MeleeCounter_System::GetMeleeMoveHash(u_int uType) const
{
	switch(uType)
	{
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_STAND_ATTACK:
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_WALK_ATTACK:
		{
			const AVP_ServerActor_Vertebrate* pxTarget = AVP_ServerActor_Vertebrate::FindEntity(m_uCurrentTarget);
			if (pxTarget)
			{
				Asura_Vector_3 xToTarget = pxTarget->GetPosition() - m_xOwner.GetPosition();
				xToTarget.y = 0.0f;
				if (xToTarget.ApproximatelyEquals(Asura_Maths::ZeroVector3))
				{
					return AVP_Predalien_Move_Melee_ClawLeft_Start::GetStaticHash();
				}
				bool bRight = true;
				xToTarget.Normalise();
				bool bWalkAttack = uType == AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_WALK_ATTACK;
				const Asura_Matrix_3x3& xOwnerOri = m_xOwner.GetOrientation();
				const Asura_Vector_3 xOwnerForward(xOwnerOri.e13, xOwnerOri.e23, xOwnerOri.e33 );
				const Asura_Vector_3 xOwnerRight(xOwnerOri.e12, xOwnerOri.e22, xOwnerOri.e32 );

				float fDotForward = bWalkAttack ? true :  xOwnerForward * xToTarget;
				float fDotRight = xOwnerRight * xToTarget;

				if (fDotForward > dbg_fRandomLeftOrRightClawAngleCos)
				{
					bRight = Asura_Random::GetFloat() > 0.5f;
				}
				else
				{
					bRight = (fDotRight > 0.0f);
				}

				if (bRight)
				{
					if (bWalkAttack)
					{
						return AVP_Predalien_Move_Melee_WalkClawRight_Start::GetStaticHash();
					}
					else
					{
						return AVP_Predalien_Move_Melee_ClawRight_Start::GetStaticHash();
					}
				}
				else
				{
					if (bWalkAttack)
					{
						return AVP_Predalien_Move_Melee_WalkClawLeft_Start::GetStaticHash();
					}
					else
					{
						return AVP_Predalien_Move_Melee_ClawLeft_Start::GetStaticHash();
					}
				}
			}
			else
			{
				return AVP_Predalien_Move_Melee_ClawLeft_Start::GetStaticHash();			
			}
		}
		break;
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_TAIL_ATTACK:
		{
			const AVP_ServerActor_Vertebrate* pxTarget = AVP_ServerActor_Vertebrate::FindEntity(m_uCurrentTarget);
			if (pxTarget)
			{
				Asura_Vector_3 xToTarget = pxTarget->GetPosition() - m_xOwner.GetPosition();
				xToTarget.y = 0.0f;
				if (xToTarget.ApproximatelyEquals(Asura_Maths::ZeroVector3))
				{
					return AVP_Predalien_Move_Melee_ClawLeft_Start::GetStaticHash();
				}
				bool bRight = true;
				xToTarget.Normalise();
				bool bWalkAttack = uType == AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_WALK_ATTACK;
				const Asura_Matrix_3x3& xOwnerOri = m_xOwner.GetOrientation();
				const Asura_Vector_3 xOwnerForward(xOwnerOri.e13, xOwnerOri.e23, xOwnerOri.e33 );
				const Asura_Vector_3 xOwnerRight(xOwnerOri.e12, xOwnerOri.e22, xOwnerOri.e32 );

				float fDotForward = bWalkAttack ? true :  xOwnerForward * xToTarget;
				float fDotRight = xOwnerRight * xToTarget;
				if (fDotForward < dbg_fRandomLeftOrRightTailAngleCos)
				{
					bRight = Asura_Random::GetFloat() > 0.5f;
				}
				else
				{
					bRight = (fDotRight > 0.0f);
				}

				if (bRight) 
				{
					return AVP_Predalien_Move_Melee_TailRight::GetStaticHash(); 
				}
				else
				{
					return AVP_Predalien_Move_Melee_TailLeft::GetStaticHash();
				}
			}
			else
			{
				return AVP_Predalien_Move_Melee_TailLeft::GetStaticHash();			
			}
		}
		break;
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_LEAP_ATTACK:
		return AVP_Predalien_Move_Melee_LeapAttack_Telegraph::GetStaticHash();
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_JUMP_DOWN_ATTACK:
		return AVP_Predalien_Move_Melee_JumpDownAttack_Start::GetStaticHash();
	default:
		return PARENT::GetMeleeMoveHash(uType);

	}
}


bool AVP_Predalien_MeleeCounter_System::GetCanBlock() const
{
	// Predalien has no weapons, so don't do the parent check.
	return !IsBlockActive();
}

DEBUGVAR float dbg_fCloseCounterAngleCos = Asura_Maths::Cosine(Asura_Maths::DegToRad(30.0f));
DEBUGVAR float dbg_fJumpDownCounterVerMaxDistance = 2.0f;
DEBUGVAR float dbg_fJumpDownCounterHorMaxDistanceSq = 1.0f * 1.0f;

bool AVP_Predalien_MeleeCounter_System::GetCanCounter_PredatorCounter(const AVP_ServerEntity_Actor& xEnemy, const u_int uQueuedCounterType, Asura_Hash_ID& uMoveHash, u_int& uNewAttackType, bool& bMakeEnemyIntoPuppet, Asura_Hash_ID& uGrappleSetBlueprintParam) const
{
	switch (m_uCurrentAttackType)
	{
	default:
		return PARENT::GetCanCounter_PredatorCounter(xEnemy, uQueuedCounterType, uMoveHash, uNewAttackType, bMakeEnemyIntoPuppet, uGrappleSetBlueprintParam);
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_LEAP_ATTACK:
		return false;
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_JUMP_DOWN_ATTACK:
		{
			uNewAttackType = AVP_PREDATOR_MELEEATTACK_TYPE_PREDALIEN_KNOCKDOWN;
			// if the enemy is not almost on top of me couter is not possible
			const AVP_ServerActor_Vertebrate& xVertebrateEnemy = static_cast<const AVP_ServerActor_Vertebrate&>(xEnemy);

			if (!xVertebrateEnemy.GetMeleeSystem() || !xVertebrateEnemy.GetMeleeSystem()->GetHasAttackTarget())
			{
				return PARENT::GetCanCounter_PredatorCounter(xEnemy, uQueuedCounterType, uMoveHash, uNewAttackType, bMakeEnemyIntoPuppet, uGrappleSetBlueprintParam);
			}

			float fBBHight = GetTargetBBHight();

			Asura_Vector_3 xToEnemy = xVertebrateEnemy.GetPosition() - m_xOwner.GetPosition();
			float fVerDistance = Asura_Maths::Abs(xToEnemy.y);
			xToEnemy.y = 0.0f;
			float fHorDistanceSq = xToEnemy.MagnitudeSquared();
			if (fVerDistance > dbg_fJumpDownCounterVerMaxDistance + fBBHight || fHorDistanceSq > dbg_fJumpDownCounterHorMaxDistanceSq)
			{
				return false;
			}
			else
			{
				bMakeEnemyIntoPuppet = true;
				uMoveHash = AVP_Predator_Move_Melee_CounterPredalienJumpDownAttack::GetStaticHash();
				return true;
			}
			break;
		}
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_STAND_ATTACK:
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_WALK_ATTACK:
		{
			uNewAttackType = AVP_PREDATOR_MELEEATTACK_TYPE_PREDALIEN_DONT_KNOCKDOWN;
			const AVP_ServerActor_Vertebrate& xVertebrateEnemy = static_cast<const AVP_ServerActor_Vertebrate&>(xEnemy);

			if (!xVertebrateEnemy.GetMeleeSystem() || !xVertebrateEnemy.GetMeleeSystem()->GetHasAttackTarget())
			{
				return PARENT::GetCanCounter_PredatorCounter(xEnemy, uQueuedCounterType, uMoveHash, uNewAttackType, bMakeEnemyIntoPuppet, uGrappleSetBlueprintParam);
			}

			const Asura_Matrix_3x3& xEnemyOri = xVertebrateEnemy.GetOrientation();
			const Asura_Vector_3 xEnemyForward(xEnemyOri.e13, xEnemyOri.e23, xEnemyOri.e33 );
			Asura_Vector_3 xFromEnemy = m_xOwner.GetPosition() - xVertebrateEnemy.GetPosition();
			xFromEnemy.y = 0.0f;
			if (xFromEnemy.ApproximatelyEquals(Asura_Maths::ZeroVector3))
			{
				return false;
			}
			xFromEnemy.Normalise();
			if (xFromEnemy * xEnemyForward < dbg_fCloseCounterAngleCos)
			{
				return false;
			}	

			switch (GetCurrentCloseMoveType())
			{
			case CLOSE_MOVE_CLAW_LEFT:
				uMoveHash = AVP_Predator_Move_Melee_CounterPredalienLeftClawAttack::GetStaticHash();
				break;
			case CLOSE_MOVE_CLAW_RIGHT:
				uMoveHash = AVP_Predator_Move_Melee_CounterPredalienRightClawAttack::GetStaticHash();
				break;
			default:
			case CLOSE_MOVE_UKNOWN:
				uMoveHash = ASURA_HASH_ID_UNSET;
				return false;
			}

			bMakeEnemyIntoPuppet = true;
			return true;
		}
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_TAIL_ATTACK:
		uNewAttackType = AVP_PREDATOR_MELEEATTACK_TYPE_PREDALIEN_DONT_KNOCKDOWN;
		switch (GetCurrentCloseMoveType())
		{
		case CLOSE_MOVE_TAIL_LEFT:
		case CLOSE_MOVE_TAIL_RIGHT:
			uMoveHash = AVP_Predator_Move_Melee_CounterPredalienTailSwipe::GetStaticHash();
			break;
		default:
		case CLOSE_MOVE_UKNOWN:
			uMoveHash = ASURA_HASH_ID_UNSET;
			return false;
		}

		bMakeEnemyIntoPuppet = false;
		return true;
	}
}

float AVP_Predalien_MeleeCounter_System::GetCurrentMaxCounterDistance() const
{
	switch (m_uCurrentAttackType)
	{
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_JUMP_DOWN_ATTACK:
		return dbg_fJumpDownCounterVerMaxDistance + 2.5f; // estimate of hight above the predator head
	default:
		return PARENT::GetCurrentMaxCounterDistance();
	}
}

float AVP_Predalien_MeleeCounter_System::GetAttackDamage() const
{
	switch (m_uCurrentAttackType)
	{
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_STAND_ATTACK:
		switch (GetCurrentCloseMoveType())
		{
		case CLOSE_MOVE_CLAW_LEFT:
		case CLOSE_MOVE_CLAW_RIGHT:
			return m_fModerateDamageAmount;
		default:
		case CLOSE_MOVE_UKNOWN:
			Asura_Assert(false, "Counter move not found");
			return 0.0f;
		}

    case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_WALK_ATTACK:
		switch (GetCurrentCloseMoveType())
		{
		case CLOSE_MOVE_CLAW_LEFT:
		case CLOSE_MOVE_CLAW_RIGHT:
			return m_fModerateDamageAmount;
		default:
		case CLOSE_MOVE_UKNOWN:
			Asura_Assert(false, "Counter move not found");
			return 0.0f;
		}

    case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_TAIL_ATTACK:
		switch (GetCurrentCloseMoveType())
		{
		case CLOSE_MOVE_TAIL_LEFT:
		case CLOSE_MOVE_TAIL_RIGHT:
			return m_fMajorDamageAmount;
		default:
		case CLOSE_MOVE_UKNOWN:
			Asura_Assert(false, "Counter move not found");
			return 0.0f;
		}

    case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_LEAP_ATTACK:
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_JUMP_DOWN_ATTACK:
		return m_fModerateDamageAmount;
	default:
		return PARENT::GetAttackDamage();
	}
}

Asura_Hash_ID AVP_Predalien_MeleeCounter_System::GetAttackDamageType() const
{
	return uAVP_BLUEPRINT_NAME_DAMAGE_MELEE;
}


bool AVP_Predalien_MeleeCounter_System::IsAttackUnblockable() const
{
	return PARENT::IsAttackUnblockable();
}

bool AVP_Predalien_MeleeCounter_System::GetAttackOnlyHitsCurrentTarget() const
{
	return true; // only going to fight one target anyhow...
}

const Asura_Blueprint* AVP_Predalien_MeleeCounter_System::GetExtraMeleeDataBlueprint() const
{
	return m_xOwner.GetBlueprint();
}


void AVP_Predalien_MeleeCounter_System::SendMeleeAttackHitMessage()
{
	if (m_uCurrentAttackType == AVP_PREDPREDALIEN_MELEEATTACK_TYPE_LEAP_ATTACK)
	{
		if (m_bDoGrapple) 
		{
			m_bDoGrapple = false;

			// Some moves will need to know which enemy was hit in the end, so that data is sent along. This assumes it'll only ever care about the first enemy hit (probably fine).
			Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_MELEE_ATTACK_HIT, sizeof(AVP_Msg_Melee_Attack_Hit));

			if (pxMessage)
			{
				AVP_Msg_Melee_Attack_Hit* pxData = static_cast<AVP_Msg_Melee_Attack_Hit*>(pxMessage->GetData());
				if (pxData)
				{
					pxData->SetHitEntity(m_auHitEntities[0]);
					pxData->SetPendingVariation(m_uCurrentVariation);
					pxData->SetPendingAttackType(AVP_PREDPREDALIEN_MELEEATTACK_TYPE_LEAP_ATTACK_DO_GRAPPLE); // mark that a grapple is required
					pxData->SetDamageType(GetAttackDamageType());
					pxData->SetIsRightHanded(true);
				}

				pxMessage->SetToGuid(m_xOwner.GetGuid());
				pxMessage->SetFromGuid(m_xOwner.GetGuid());

				Asura_Network::Send(pxMessage);
			}

			return;
		}
	}
	else if (m_uCurrentAttackType == AVP_PREDPREDALIEN_MELEEATTACK_TYPE_JUMP_DOWN_ATTACK)
	{
		if (m_bDoSuperJumpAttack)
		{
			m_bDoSuperJumpAttack = false;

			Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_MOVE_START, sizeof(AVP_Msg_MeleeWithTargetPosition_Move));

			if (pxMessage)
			{
				AVP_Msg_Melee_Move* pxData = static_cast<AVP_Msg_Melee_Move*>(pxMessage->GetData());

				pxData->ClearFlags();

				pxData->SetVariation(0);
				pxData->SetPendingAttackType(AVP_PREDPREDALIEN_MELEEATTACK_TYPE_JUMP_DOWN_ATTACK);
				pxData->SetTargetGuid(m_uCurrentTarget);
				pxData->SetMoveID(AVP_Predalien_Move_Melee_JumpDownAttack_SuperHit::GetStaticHash());
				pxData->SetIgnoredByOwner(false);

				pxMessage->SetToGuid(m_xOwner.GetGuid());
				pxMessage->SetFromGuid(m_xOwner.GetGuid());

				Asura_Network::Send(pxMessage);
				return;
			}
		}
	}
	
	PARENT::SendMeleeAttackHitMessage();
}




AVP_Predalien_MeleeCounter_System::CloseMoveType AVP_Predalien_MeleeCounter_System::GetCurrentCloseMoveType() const 
{

	const AVP_ClientEntity_Actor* pxClientOwner = AVP_ClientEntity_Actor::FindEntity(m_xOwner.GetGuid());
	const Asura_Movement_System* pxMovementSystem = pxClientOwner ? pxClientOwner->GetMovementSystem() : NULL;
	const Asura_Move* pxMove = pxMovementSystem ? pxMovementSystem->GetCurrentMove() : NULL;
	if (!pxMove)
	{
		return CLOSE_MOVE_UKNOWN;
	}

	switch (m_uCurrentAttackType)
	{
	case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_STAND_ATTACK:
		if ( pxMove->GetHash() == AVP_Predalien_Move_Melee_ClawRight_Start::GetStaticHash() )
		{
			return CLOSE_MOVE_CLAW_RIGHT;
		}
		else if ( pxMove->GetHash() == AVP_Predalien_Move_Melee_ClawLeft_Start::GetStaticHash() )
		{
			return CLOSE_MOVE_CLAW_LEFT;
		}
		else 
		{
			return CLOSE_MOVE_UKNOWN;
		}

    case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_WALK_ATTACK:
		if ( pxMove->GetHash() == AVP_Predalien_Move_Melee_WalkClawRight_Start::GetStaticHash() )
		{
			return CLOSE_MOVE_CLAW_RIGHT;
		}
		if ( pxMove->GetHash() == AVP_Predalien_Move_Melee_WalkClawLeft_Start::GetStaticHash() )
		{
			return CLOSE_MOVE_CLAW_LEFT;
		}
		else 
		{
			return CLOSE_MOVE_UKNOWN;
		}

    case AVP_PREDPREDALIEN_MELEEATTACK_TYPE_CLOSE_TAIL_ATTACK:
		if (pxMove->GetHash() == AVP_Predalien_Move_Melee_TailRight::GetStaticHash())
		{
			return CLOSE_MOVE_TAIL_RIGHT;
		}
		else if (pxMove->GetHash() == AVP_Predalien_Move_Melee_TailLeft::GetStaticHash())
		{
			return CLOSE_MOVE_TAIL_LEFT;
		}
		else 
		{
			return CLOSE_MOVE_UKNOWN;
		}

	default:
		Asura_Assert(false, "Unexpected move");
		return CLOSE_MOVE_UKNOWN;
	}
}



bool AVP_Predalien_MeleeCounter_System::CalculateLeapTargetPosition(Asura_Vector_3& xDestination)
{	
	const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uCurrentTarget);
	if (!pxTarget)
	{
		return false;
	}

	m_xOwnerStartLeapPosition = m_xOwner.GetPosition();
	m_xTargetStartLeapPosition = pxTarget->GetPosition();

	xDestination = pxTarget->GetPosition();

	Asura_Vector_3 xToTArget = pxTarget->GetPosition() - m_xOwner.GetPosition();
	xToTArget.y = 0.0f;

	// if we too close this can not be done anyhow
	if (xToTArget.ApproximatelyEquals(Asura_Maths::ZeroVector3))
	{
		return false;
	}

	xToTArget.Normalise();

	// add the offset in direction towards us
	xDestination -= xToTArget * dbg_fLeapAttackDistanceBehindTarget;
	// add velocity estimation ignoring hight difference
	Asura_Vector_3 xVelocity = pxTarget->GetVelocity();
	xVelocity.y = 0.0f;
	xDestination += xVelocity * dbg_fLeapTimeEstimate;

	return true;
}

bool AVP_Predalien_MeleeCounter_System::CalcluateTailDirection(float &fTailLenght, Asura_Vector_3& xTailDirection, Asura_Vector_3& xCurrentTailPosition) const
{
	if ( !m_xOwner.GetWorldSpaceMarkerPoint( 0x00363450, &xCurrentTailPosition, NULL ) ) // "tail"
	{
		Asura_Assert(false, "Tail marker point not found");
		return false;
	}
	xTailDirection = xCurrentTailPosition - m_xOwner.GetPosition();
	xTailDirection.y = 0.0f; // ignore any hight difference
	if (xTailDirection.ApproximatelyEquals(Asura_Maths::ZeroVector3))
	{
		fTailLenght = 0.0f;
	}
	else
	{
		fTailLenght = xTailDirection.Normalise();
	}

	return true;
}

float AVP_Predalien_MeleeCounter_System::GetTargetBBHight() const
{
	const AVP_ClientEntity_Actor* pxClientTarget = AVP_ClientEntity_Actor::FindEntity(m_uCurrentTarget); // the target is always the player - his handler is on the client side 
	const AVP_Actor_PhysicsObjectHandler* pxPhyHandler = pxClientTarget ? pxClientTarget->GetPhysicsObjectHandler() : NULL;  
	const Asura_Physics_Object* pxPhyObject =  pxPhyHandler ? pxPhyHandler->GetPhysicsObject() : NULL;
	const Asura_Collision_Object* pxCollObj = pxPhyObject ? pxPhyObject->GetCollisionObject() : NULL;
	if (!pxCollObj) { return 1.5f; } // some estimate
	const Asura_Bounding_Box& xBB = pxCollObj->GetBoundingBox();
	const AVP_ServerEntity_Actor* pxTarget = AVP_ServerEntity_Actor::FindEntity(m_uCurrentTarget); 
	if (!pxTarget) { return 1.5f; } // some estimate
	return Asura_Maths::Abs(xBB.GetMax().y - xBB.GetMin().y );
}

/// End of file. ///
