
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predator_AIAnimStateMachine.h
// Author : Will Plano
// Date   : 27/02/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_PREDATOR_AIANIMSTATEMACHINE_H__
#define __AVP_PREDATOR_AIANIMSTATEMACHINE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/Vertebrates/AVP_Vertebrate_AIAnimStateMachine.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_JumpHandler.h"

class AVP_Predator_AIAnimStateMachine : public AVP_Vertebrate_AIAnimStateMachine
{
public:
	// Public enumerated types...

	enum PredatorAnimState
	{
		PREDATOR_STATE_BEGIN = VERTEBRATE_STATE_END,
		// crouching states
		PREDATOR_STATE_CROUCH_IDLE = PREDATOR_STATE_BEGIN,	// 13
		PREDATOR_STATE_CROUCH_MOVE,							// 14
		// transition states
		PREDATOR_STATE_IDLE_TO_CROUCH_IDLE,					// 15
		PREDATOR_STATE_CROUCH_IDLE_TO_IDLE,					// 16

		PREDATOR_STATE_END									// 17
	};

	// Public functions...
	AVP_Predator_AIAnimStateMachine( AVP_ServerEntity_Actor& xServer ) : PARENT( xServer ) {}
	virtual ~AVP_Predator_AIAnimStateMachine() {}

	virtual void Initialise();

	void	RequestCrouchIdle() { RequestState( PREDATOR_STATE_CROUCH_IDLE ); }
	bool	IsCrouchIdle() const { return IsStateCrouchIdle( GetCurrentState() ); }
	bool	IsStateCrouchIdle( const u_int uState ) const { return PREDATOR_STATE_CROUCH_IDLE == uState; }

	void	RequestCrouchMove() { RequestState( PREDATOR_STATE_CROUCH_MOVE ); }
	bool	IsCrouchMove() const { return IsStateCrouchMove( GetCurrentState() ); }
	bool	IsStateCrouchMove( const u_int uState ) const { return PREDATOR_STATE_CROUCH_MOVE == uState; }

	virtual const AVP_Actor_AIAnimStateConnectionData* GetConnectionData( const u_int uFromState, const u_int uToState ) const;

	virtual const char* GetStateDescription( const u_int uState ) const { return GetStateDescriptionStatic( uState ); }
	static const char* GetStateDescriptionStatic( const u_int uState );

	static bool IsDirectionalState( const u_int uState );

	virtual bool IsLoopedState( const u_int uState ) const { return IsLoopedStateStatic( uState ); }
	static bool IsLoopedStateStatic( const u_int uState );

protected:
	virtual void BeginState		( const u_int uState );
	//virtual void UpdateState	( const u_int uState ) { PARENT::UpdateState( uState ); }
	virtual bool EndState		( const u_int uState );

	virtual u_int GetNumStates() const { return PREDATOR_STATE_END; }
	virtual const AVP_Actor_AnimStateData* GetStateAnimData( const u_int uState ) const;

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

private:
	typedef AVP_Vertebrate_AIAnimStateMachine PARENT;

	AVP_Predator_AIAnimStateMachine();
	AVP_Predator_AIAnimStateMachine(const AVP_Predator_AIAnimStateMachine& xOther);
	AVP_Predator_AIAnimStateMachine&	operator=(const AVP_Predator_AIAnimStateMachine& xOther);
}; // End of class 'AVP_Predator_AIAnimStateMachine'.

#endif // ifndef __AVP_PREDATOR_AIANIMSTATEMACHINE_H__

/// End of file. ///
