
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predator_AnimationHandler.h
// Author : Andy Tate
// Date   : 18-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_PREDATOR_ANIMATIONHANDLER_H__
#define __AVP_PREDATOR_ANIMATIONHANDLER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Animation/IK/Asura_Animation_IK_Chain_LOS.h"

#include "Animation/AVP_Animation_BodyRig.h"
#include "Actor/Vertebrates/AVP_Vertebrate_AnimationHandler.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_Description.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_ClientActor_Predator;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Predator_AnimationHandler
// Author       : Andy Tate
// Date         : 18-Oct-2006
//
// Description  : Class to handle the animations needed by the Predator.
////////////////////////////////////////////////////////////////////////////////

class AVP_Predator_AnimationHandler : public AVP_Vertebrate_AnimationHandler
{
public:

	// Public functions...
	AVP_Predator_AnimationHandler  (AVP_ClientActor_Predator& xOwner);
	~AVP_Predator_AnimationHandler ();

	// Overridden functions...
	virtual void Initialise(void);

	virtual	void	UpdateSkin(void);

	virtual	void	Reset (void) { PARENT::Reset(); }

	virtual void HandleAnimationRequest	(const AVP_Message_Actor_RequestAnimation& xAnimRequestData);
	virtual void Update();

	virtual void CreateRequestedAnimation( const Asura_Hash_ID uAnimHashID, const u_int uAnimFlags );

	const Asura_Animation_IK_Chain_LOS* GetLimbIKChainLOS( u_int uLimbItr ) const;

protected:
	// Protected variables...
	AVP_ClientActor_Predator&		m_xOwnerPredator;
	int								m_iRightHandMarkerPointIndex;	// The index of the Predator's right-hand marker-point.

	Asura_Animation_IK_Chain_LOS	m_xLeftLegIKChain;
	Asura_Animation_IK_Chain_LOS	m_xRightLegIKChain;

	// Targeting functions...
	bool	CalculateMarkerPointToCrosshairData	(const Asura_Hash_ID uSourceMarkerPointHash, const float fInitialSpeed, const Asura_Vector_3& xCrosshairOffset, Asura_Vector_3& xPosition, Asura_Matrix_3x3& xOrientation, Asura_Vector_3& xVelocity) const;

private:
	// Private typedefs...
	typedef AVP_Vertebrate_AnimationHandler PARENT;

	// Private variables...

	// Private functions...
	void	InitialiseIKSolver();

	AVP_Predator_AnimationHandler  (void);
	AVP_Predator_AnimationHandler  (const AVP_Predator_AnimationHandler& xOther);
	AVP_Predator_AnimationHandler&	operator=	(const AVP_Predator_AnimationHandler& xOther);
};   // End of class 'AVP_Predator_AnimationHandler'.


#endif   // ifndef __AVP_PREDATOR_ANIMATIONHANDLER_H__


/// End of file. ///
