
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predator_Description.cpp
// Author : Andy Tate
// Date   : 12-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Predator/AVP_Predator_Description.h"

// Asura independent header-files...
#ifdef ASURA_TOOLS
	#include "Blueprint/Tools_Mode_Blueprint.h"
#endif
// Asura platform header-files...

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Rendering/Asura_Project_Skin_Elements.h"

// Game platform header-files...

// Tools header-files...
#if defined(ASURA_TOOLS)
	#include "Particles/Tools_Mode_PFX.h"
#endif	// defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

// Marker-point hash-IDs...
const Asura_Hash_ID AVP_Predator_Description::s_auMarkerPointHashIDs[NUM_MARKER_POINT_IDS] =
{
	0x57D54335, //Asura_GetHashID("blade_right"),		//BLADE_RIGHT_MARKER_POINT_HASH,
	0xC90417CE, //Asura_GetHashID("blade_left"),		//BLADE_LEFT_MARKER_POINT_HASH,
	0xA282148B, //Asura_GetHashID("righthand"),			//RIGHT_HAND_MARKER_POINT_HASH,
	0xC53E8542, //Asura_GetHashID("plasma"),			//PLASMA_MARKER_POINT_HASH,
	0x061FBECB, //Asura_GetHashID("laser"),				//LASER_MARKER_POINT_HASH,
	0x002FBC62, //Asura_GetHashID("eyes"),				//EYES_MARKER_POINT_HASH,
	0xE5A85D03, //Asura_GetHashID("target_1"),			//TARGET_MARKER_POINT_HASH,
};

const int AVP_Predator_Description::s_aiFirstPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ] = 
{
	FP_BIP01_L_FOOT,
	FP_BIP01_R_FOOT,
	-1,
	-1
};

const int AVP_Predator_Description::s_aiThirdPersonFootBoneID[ AVP_ClientEntity_Actor::MAX_NUM_FEET ] = 
{
	TP_BIP01_L_FOOT,
	TP_BIP01_R_FOOT,
	-1,
	-1
};

const int AVP_Predator_Description::s_aiFirstPersonLeftLegChainBonesArray[ uAVP_PREDATOR_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_L_THIGH,
	FP_BIP01_L_CALF,
	FP_BIP01_L_FOOT,
	-1
};

const int AVP_Predator_Description::s_aiFirstPersonRightLegChainBonesArray[ uAVP_PREDATOR_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	FP_BIP01_R_THIGH,
	FP_BIP01_R_CALF,
	FP_BIP01_R_FOOT,
	-1
};

const int AVP_Predator_Description::s_aiThirdPersonLeftLegChainBonesArray[ uAVP_PREDATOR_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_L_THIGH,
	TP_BIP01_L_CALF,
	TP_BIP01_L_FOOT,
	-1
};

const int AVP_Predator_Description::s_aiThirdPersonRightLegChainBonesArray[ uAVP_PREDATOR_NUM_IK_BONES_PER_LIMB + 1 ] =
{
	TP_BIP01_R_THIGH,
	TP_BIP01_R_CALF,
	TP_BIP01_R_FOOT,
	-1
};

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

u_int AVP_Predator_Description::GetElementFromBoneID(int iBoneID)
{
	switch(iBoneID)
	{
	case TP_BIP01:
	case TP_BIP01_PELVIS:
	case TP_BIP01_SPINE:
	case TP_BIP01_SPINE1:
	case TP_BIP01_SPINE2:
	case TP_BIP01_CLOTH:
	case TP_BIP01_CLOTH02:
	case TP_BIP01_CLOTH_BACK:
	case TP_BIP01_BIP01_PLASMASHOULDER:
	case TP_BIP01_BIP01_PLASMA_GUN_SUPPORT:
	case TP_BIP01_PLASMA_GUN:
		{
			return(AVP_SET_BODY);
		}

	case TP_BIP01_NECK:
	case TP_BIP01_HEAD:
	case TP_BIP01_CHEEK_LEFT:
	case TP_BIP01_CHEEK_RIGHT:
	case TP_BIP01_EYEBROW_LEFT:
	case TP_BIP01_EYEBROW_RIGHT:
	case TP_BIP01_HAIR_BACK:
	case TP_BIP01_HAIR_BACK02:
	case TP_BIP01_HAIR_FRONT_LEFT:
	case TP_BIP01_HAIR_FRONT_LEFT02:
	case TP_BIP01_HAIR_FRONT_RIGHT:
	case TP_BIP01_HAIR_FRONT_RIGHT02:
	case TP_BIP01_JAW:
	case TP_BIP01_MOUTH_BOTTOM_LEFT:
	case TP_BIP01_MOUTH_BOTTOM_LEFT02:
	case TP_BIP01_MOUTH_BOTTOM_RIGHT:
	case TP_BIP01_MOUTH_BOTTOM_RIGHT02:
	case TP_BIP01_MOUTH_TOP_LEFT:
	case TP_BIP01_MOUTH_TOP_LEFT02:
	case TP_BIP01_MOUTH_TOP_RIGHT01:
	case TP_BIP01_MOUTH_TOP_RIGHT02:
	case TP_EYE_LEFT:
	case TP_EYE_RIGHT:
		{
			return(AVP_SET_HEAD);
		}

	case TP_BIP01_L_CLAVICLE:
	case TP_BIP01_L_UPPERARM:
		{
			return(AVP_SET_LEFT_ARM);
		}

	case TP_ARM_PANEL:
	case TP_BIP01_L_FOREARM:
	case TP_BIP01_CLAW_LEFT01:
	case TP_BIP01_CLAW_LEFT02:
	case TP_BIP01_CLAW_LEFT03:
	case TP_BIP01_L_HAND:
	case TP_BIP01_L_FINGER0:
	case TP_BIP01_L_FINGER01:
	case TP_BIP01_L_FINGER1:
	case TP_BIP01_L_FINGER11:
	case TP_BIP01_L_FINGER2:
	case TP_BIP01_L_FINGER21:
	case TP_BIP01_L_FINGER3:
	case TP_BIP01_L_FINGER31:
	case TP_BIP01_L_FINGER4:
	case TP_BIP01_L_FINGER41:
	case TP_LEFT_LOWERFOREARM_TWIST:
	case TP_LEFT_UPPERFOREARM_TWIST:
		{
			return(AVP_SET_LEFT_ELBOW);
		}

	case TP_BIP01_R_CLAVICLE:
	case TP_BIP01_R_UPPERARM:
	case TP_SHOULDER_PINCHER_RIGHT:
		{
			return(AVP_SET_RIGHT_ARM);
		}

	case TP_BIP01_R_FOREARM:
	case TP_BIP01_CLAW_RIGHT01:
	case TP_BIP01_CLAW_RIGHT02:
	case TP_BIP01_CLAW_RIGHT03:
	case TP_BIP01_R_HAND:
	case TP_BIP01_R_FINGER0:
	case TP_BIP01_R_FINGER01:
	case TP_BIP01_R_FINGER1:
	case TP_BIP01_R_FINGER11:
	case TP_BIP01_R_FINGER2:
	case TP_BIP01_R_FINGER21:
	case TP_BIP01_R_FINGER3:
	case TP_BIP01_R_FINGER31:
	case TP_BIP01_R_FINGER4:
	case TP_BIP01_R_FINGER41:
	case TP_RIGHT_LOWERFOREARM_TWIST:
	case TP_RIGHT_UPPERFOREARM_TWIST:
		{
			return(AVP_SET_RIGHT_ELBOW);
		}

	case TP_BIP01_L_THIGH:
	case TP_BIP01_L_CALF:
		{
			return(AVP_SET_LEFT_LEG);
		}

	case TP_BIP01_L_FOOT:
	case TP_BIP01_L_TOE0:
		{
			return(AVP_SET_LEFT_KNEE);
		}

	case TP_BIP01_R_THIGH:
	case TP_BIP01_R_CALF:
		{
			return(AVP_SET_RIGHT_LEG);
		}

	case TP_BIP01_R_FOOT:
	case TP_BIP01_R_TOE0:
		{
			return(AVP_SET_RIGHT_KNEE);
		}

	default:
		{
			return(NUM_AVP_SKIN_ELEMENT_TYPES);
		}
	}	// End of switch.
}

// Since bones->elements is a many->one mapping, this function will return one of the bones associated with the given element, but not necessarily the only one.
int AVP_Predator_Description::GetAnyBoneIDFromElement(u_int uElement)
{
	switch (uElement)
	{
	case AVP_SET_BODY:
		return TP_BIP01_SPINE2;

	case AVP_SET_HEAD:
	case AVP_SET_HEAD_AFTER_HEADBITE:
		return TP_BIP01_HEAD;

	case AVP_SET_LEFT_ARM:
		return TP_BIP01_L_UPPERARM;

	case AVP_SET_LEFT_ELBOW:
		return TP_BIP01_L_FOREARM;

	case AVP_SET_RIGHT_ARM:
		return TP_BIP01_R_UPPERARM;

	case AVP_SET_RIGHT_ELBOW:
		return TP_BIP01_R_FOREARM;

	case AVP_SET_LEFT_LEG:
		return TP_BIP01_L_THIGH;

	case AVP_SET_LEFT_KNEE:
		return TP_BIP01_L_FOOT;

	case AVP_SET_RIGHT_LEG:
		return TP_BIP01_R_THIGH;

	case AVP_SET_RIGHT_KNEE:
		return TP_BIP01_R_FOOT;

	default:
		return -1;
	}
}


#if defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_Description::AddFilesToExport
// Author       : Adam Law
// Date         : 21-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_Description::AddFilesToExport(const bool bFirstPerson,
												const Asura_Hash_ID uBlueprintHash,
												List<Asura_String>& xNameList)
{
	if(uBlueprintHash != ASURA_HASH_ID_UNSET)
	{
		Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uAVP_BLUEPRINT_TYPE_ACTORS, uBlueprintHash, false);
		if ( bFirstPerson )
		{
			Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uAVP_BLUEPRINT_TYPE_PLAYERS, uBlueprintHash, false);
		}
	}

	// Also add everything of type "Grapple Predator". This has first and third person anims, so do it regardless of skin type.
	Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uAVP_BLUEPRINT_TYPE_GRAPPLEPREDATOR, ASURA_HASH_ID_UNSET, true);
}

#endif	// defined(ASURA_TOOLS)


/// End of file. ///
