
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predator_InputHandler.cpp
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Predator/AVP_Predator_InputHandler.h"

// Asura independent header-files...
#include "Attractors/Asura_Attractor_System.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_BitPacker.h"
#include "Network/Asura_Network.h"
#include "Input/Asura_ControlManager.h"
#include "Interface/Asura_CommandConsole.h"

// Game independent header-files...
#include "Actor/AVP_Actor_AnimationHandler.h"
#include "Actor/AVP_Actor_CameraHandler.h"
#include "Actor/AVP_Actor_Selector.h"
#include "Actor/Vertebrates/AVP_Vertebrate_Focus.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_Description.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_InteractionHandler.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_MeleeCounter_System.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_Recon.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ClientActor_Predator.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_Messages_Predator.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Inventory/Items/AVP_InventoryItem_PredatorCloak.h"
#include "Moves/Vertebrates/Predator/AVP_Predator_MovesBranch.h"

// Constants

DVSVAR float dvs_fVisionHoldTime = 0.5f;

#if defined(ASURA_DEBUG_VARIABLE_SYSTEM) && defined(AVP_DEBUG_INPUT)
u_int AVP_Predator_InputHandler::s_uConsoleVarReferenceCount = 0;
#endif

void AVP_Predator_InputHandler::MapCloak( Asura_ControlMap& xMap , const u_int uKeyCloak )
{
	xMap.AddKeyMapping( CLOAK, uKeyCloak, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
}

void AVP_Predator_InputHandler::MapRecon( Asura_ControlMap& xMap , const u_int uKeyRecon )
{
	xMap.AddKeyMapping( RECON_TOGGLE, uKeyRecon, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_InputHandler::UpdateMovement
// Author       : Andy Tate
// Date         : 26-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_InputHandler::UpdateMovement ( AVP_ClientEntity_Actor& xClient )
{
// 	AVP_ClientActor_Predator& xPredator = static_cast< AVP_ClientActor_Predator& >( xClient );

	// Jumping is now handled in the vertebrate.
	PARENT::UpdateMovement(xClient);

// 	if ( GetControlMapValue( STANCE_TOGGLE, true, true ) > 0.1f )
// 	{
// 		if ( xPredator.IsCrouching() )
// 		{
// 			xPredator.SetCrouching( false, true );
// 		}
// 		else
// 		{
// 			xPredator.SetCrouching( true, true );
// 		}
// 	}
}

/// End of function 'AVP_Predator_InputHandler::UpdateMovement'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_InputHandler::UpdateWeapons
// Author       : Andy Tate
// Date         : 26-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_InputHandler::UpdateWeapons ( AVP_ClientEntity_Actor& xClient )
{
	AVP_ClientActor_Predator& xPredator = static_cast< AVP_ClientActor_Predator& >( xClient );

	AVP_Actor_Selector*		pxWeaponSelector	= xPredator.GetWeaponSelector();

	if(GetControlMapValue(WEAPON_SELECTOR_NEXT, true, true) > 0.0f)
	{
		pxWeaponSelector->SelectNextItem( xClient, true );
	}
	else if(GetControlMapValue(WEAPON_SELECTOR_PREV, true, true) > 0.0f)
	{
		pxWeaponSelector->SelectPreviousItem( xClient, true );
	}
	else if(pxWeaponSelector)
	{
		if(GetControlMapValue(WEAPON_SELECTOR_BEGIN, true, true) > 0.0f)
		{
			pxWeaponSelector->Activate( xClient );
		}

		pxWeaponSelector->Update( xClient );

		if(pxWeaponSelector->IsReadyToClose())
		{
			pxWeaponSelector->Deactivate( xClient );
		}
	}
}

/// End of function 'AVP_Predator_InputHandler::UpdateWeapons'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_InputHandler::UpdateVisionModes
// Author       : Chris Wallace
// Date         : 22-Nov-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_InputHandler::UpdateVisionModes ( AVP_ClientEntity_Actor& xClient )
{
	AVP_ClientActor_Predator& xPredator = static_cast<AVP_ClientActor_Predator&>(xClient);

	AVP_Inventory_System*		pxInventorySystem	= xPredator.GetInventorySystem();
	AVP_Inventory_Group*		pxVisionGroup		= (pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetGroup(uAVP_INVENTORY_GROUPHASH_VISIONMODE);
	/*AVP_Actor_Selector*			pxVisionSelector	= xPredator.GetVisionSelector();

	const float fVisionHold( GetControlMapValue( VISION_SELECTOR_HOLD ) );

	if ( fVisionHold > 0.0f )
	{
		if ( m_bVisioned == false )
		{
			m_fVisionHoldTimer += Asura_Timers::GetGameTimeSlice();
		}
	}
	else
	{
		m_bVisioned = false;
	}

	if ( m_fVisionHoldTimer > 0.0f )
	{
		if ( m_fVisionHoldTimer > dvs_fVisionHoldTime && pxVisionSelector != NULL )
		{
			pxVisionSelector->Activate( xClient );
			m_bVisioned = true;
			m_fVisionHoldTimer = 0.0f;
		}
		else if ( fVisionHold == 0.0f )
		{
			pxVisionGroup->ChangeToNextItem(true);
			m_fVisionHoldTimer = 0.0f;
		}
	}

	if ( pxVisionSelector != NULL )
	{
		pxVisionSelector->Update( xClient );

		if(pxVisionSelector->IsReadyToClose())
		{
			pxVisionSelector->Deactivate( xClient );
		}
	}
*/
	//if ( pxVisionSelector == NULL || !pxVisionSelector->IsActive() )
	{
		if(GetControlMapValue(VISION_SELECTOR_NEXT, true, true) > 0.0f)
		{
			pxVisionGroup->ChangeToNextItem(true);
		}
		else if(GetControlMapValue(VISION_SELECTOR_PREV, true, true) > 0.0f)
		{
			pxVisionGroup->ChangeToPreviousItem(true);
		}
	}
}

/// End of function 'AVP_Predator_InputHandler::UpdateVisionModes'. ///

DEBUGVAR float dbg_fPredatorBranchQuickAttachDistanceSquared = 1.0f;

void AVP_Predator_InputHandler::UpdateMovementInput(AVP_ClientEntity_Actor &xClient)
{
	// First, get our input values up to date.
	PARENT::UpdateMovementInput(xClient);

	// Now see if we want to connect to/disconnect from a branch attractor.
	AVP_ClientActor_Predator& xPredator = static_cast<AVP_ClientActor_Predator&>(xClient);


	//const u_int uVertebrateState = xPredator.GetVertebrateStateHandler().GetCurrentState();
	Asura_Vector_3 xVel;
	xPredator.GetCurrentVelocity(xVel);
	const bool bJumping = xPredator.IsJumping() || ( ( xPredator.GetGroundType() == AVP_Actor_PhysicsObjectHandler::OFF_GROUND ) && ( xVel.y < 0.0f ) );

	Asura_Movement_System* pxMovementSystem = xPredator.GetMovementSystem();
	const Asura_Move* pxCurrentMove = pxMovementSystem != NULL ? pxMovementSystem->GetCurrentMove() : NULL;
	if ( pxCurrentMove != NULL && pxCurrentMove->GetHash() == AVP_Move_Predator_DismountBranch::GetStaticHash() )
	{
		m_xMovementInput = static_cast< const AVP_Move_Predator_DismountBranch* >( pxCurrentMove )->GetMovementDirection();
	}
	else if ( !bJumping && pxCurrentMove == NULL )
	{
		Asura_Vector_3 xLosDir( m_xMovementInput );
		if (xLosDir.ApproximatelyEquals(Asura_Maths::ZeroVector3))
		{
			// Fire the LOS anyway to see if we're inside something already, since we may fall onto a branch. Down seems the best option for this, although maybe it should be the gravity direction.
			xLosDir = Asura_Vector_3(0.0f, 1.0f, 0.0f);
		}
		else
		{
			xLosDir = xLosDir * xPredator.GetCurrentOrientation();
			xLosDir.Normalise();
		}

		Asura_Matrix_3x3 xLosOri;
		Asura_Maths::OrientationFromDirection(xLosOri, xLosDir);

		CallbackData xCallbackData(*this, xPredator, xLosDir);

		Asura_Attractor_System::GetInstance()->CollectAttractors(xPredator.GetCurrentPosition(), xLosOri, 0.0f, m_fPredatorBranchLOSDistance, TestBranchAttractors_Callback, &xCallbackData);

		// See if we found a new attractor
		if (xCallbackData.m_uBestAttractor != xPredator.GetConstrainedMovementAttractorID())
		{
			if (xCallbackData.m_uBestAttractor == uASURA_ATTRACTOR_ID_NONE)
			{
				// Just disconnect in this case, moves seem unnecessary if we've lost our current attractor.
			}
			else
			{
				const Asura_Vector_3 xOffset = xCallbackData.m_xBestPosOnAttractor - xPredator.GetCurrentPosition();
				if (xOffset.MagnitudeSquared() > dbg_fPredatorBranchQuickAttachDistanceSquared)
				{
					SendMountBranchMoveMessage(xCallbackData.m_xBestPosOnAttractor, xPredator);
				}
				else
				{
					// Just go straight to it in this case
				}
			}
			xPredator.SetConstrainedMovementAttractorID(xCallbackData.m_uBestAttractor);
			m_fBranchDismountInputTimer = 0.0f;
		}
		else
		{
			// Are we pushing roughly perpendicular to the branch?
			if (!s_bOnlyLeaveBranchByJump && (xPredator.GetConstrainedMovementAttractorID() != uASURA_ATTRACTOR_ID_NONE) && !xLosDir.ApproximatelyEquals(Asura_Vector_3(0.0f, 1.0f, 0.0f)) && (xCallbackData.m_fBestDPResult < m_fPredatorBranchMaxCosAngleForDismount))
			{
				m_fBranchDismountInputTimer += Asura_Timers::GetGameTimeSlice();
				if (m_fBranchDismountInputTimer >= m_fPredatorBranchInputTimeForDismount)
				{
					// Dismount!
					SendDismountBranchMoveMessage(m_xMovementInput, xPredator);
					xPredator.SetConstrainedMovementAttractorID(uASURA_ATTRACTOR_ID_NONE);
				}
			}
			else
			{
				m_fBranchDismountInputTimer = 0.0f;
			}
		}
	}
	else if (bJumping)
	{
		xPredator.SetConstrainedMovementAttractorID(uASURA_ATTRACTOR_ID_NONE);
	}
}

void AVP_Predator_InputHandler::UpdateMeleeRequests( AVP_ClientActor_Vertebrate& xClient )
{
	// if recon mode is active, if we are looking at potential prey to activate recon OR if we have chosen a target position this frame
	bool bMeleeReconClash = false; // Note this only becomes true if recon is active, maybe the name could be more descriptive but it's long enough already.
	if( AVP_Predator_Recon::GetRecon().GetReconIsActive() || AVP_Predator_Recon::GetRecon().GetPotentialPrey() || AVP_Predator_Recon::GetRecon().GetTargetJustSelected() )
	{
		const Asura_ControlMapping* pxReconMapping = GetMapFirstPerson().GetMapping( AVP_Vertebrate_InputHandler::RECON_TOGGLE );
		while( pxReconMapping )
		{
			const Asura_ControlMapping* pxMeleeMapping = GetMapFirstPerson().GetMapping( AVP_Vertebrate_InputHandler::MELEE );
			while( pxMeleeMapping )
			{
				//if any of the keys for Melee are the same as any of the buttons for recon dont allow update
				if( pxReconMapping->GetControlID() == pxMeleeMapping->GetControlID() )
				{
					bMeleeReconClash = true;
				}
				pxMeleeMapping = pxMeleeMapping->GetNextMapping();
			}
			pxReconMapping = pxReconMapping->GetNextMapping();
		}

	}

	const AVP_Inventory_System*	pxInventorySystem = xClient.GetInventorySystem();
	const AVP_InventoryItem_Weapon* pxCurrentWeapon = pxInventorySystem ? static_cast<const AVP_InventoryItem_Weapon*>(pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON)) : NULL;

	// We can only do complex melee when wielding wristblades. Wristblades are the only predator weapon to use the "melee" type.
	if (!pxCurrentWeapon || (pxCurrentWeapon->GetItemType() != AVP_INVENTORYITEMTYPE_WEAPON_MELEE))
	{
		PARENT::UpdateMeleeRequests(xClient);
		return;
	}

	const bool bAttackPressed = GetControlMapValue(AVP_Vertebrate_InputHandler::PRIMARY_ATTACK_TAP) > 0.0f;
	const bool bAttackReleased = GetControlMapValue(AVP_Vertebrate_InputHandler::PRIMARY_ATTACK_RELEASE) > 0.0f;
	const bool bNormalMeleeAttack = !bMeleeReconClash && GetControlMapValue(MELEE) > 0.0f;
	const bool bOffGround = (xClient.GetGroundType() == AVP_Actor_PhysicsObjectHandler::OFF_GROUND) && (xClient.GetConstrainedMovementAttractorID() == uASURA_ATTRACTOR_ID_NONE);
	const AVP_Vertebrate_Focus* pxFocus = xClient.GetFocus();
	const bool bFocusActive = pxFocus && pxFocus->GetActive();
	const Asura_Guid uMeleeTarget = xClient.GetMeleeTarget();
	if ((bAttackPressed || bNormalMeleeAttack) && !bOffGround && bFocusActive && (uMeleeTarget != ASURA_GUID_UNREGISTERED))
	{
		// Send a message to the server
		Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_REQUEST_MELEE, sizeof(AVP_Message_Vertebrate_RequestMelee));

		// Tell the server we want to pounce...
		if(pxMessage != NULL)
		{
			AVP_Message_Vertebrate_RequestMelee* pxData = static_cast<AVP_Message_Vertebrate_RequestMelee*>(pxMessage->GetData());

			// Fill-in the message...
			if (bNormalMeleeAttack)
			{
				pxData->SetType(AVP_MELEEATTACK_TYPE_GRAPPLEPOUNCE);
			}
			else
			{
				pxData->SetType(AVP_PREDATOR_MELEEATTACK_TYPE_FOCUSATTACK);
			}

			pxData->SetTargetGuid(uMeleeTarget);

			// Send the request...
			pxMessage->SetToGuid(m_uOwnerGuid);
			pxMessage->SetFromGuid(m_uOwnerGuid);
			Asura_Network::Send(pxMessage);
		} 
	}
	else if (bAttackPressed || bAttackReleased)
	{
		// Send a message to the server
		Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_REQUEST_MELEE, sizeof(AVP_Message_Vertebrate_RequestMelee));

		// Tell the server we want to interact...
		if(pxMessage != NULL)
		{
			AVP_Message_Vertebrate_RequestMelee* pxData = static_cast<AVP_Message_Vertebrate_RequestMelee*>(pxMessage->GetData());

			// Fill-in the message...
			pxData->SetType(bAttackPressed ? AVP_PREDATOR_MELEEATTACK_TYPE_PRIMARYSLASH : AVP_PREDATOR_MELEEATTACK_TYPE_PRIMARYRELEASE);
			pxData->SetTargetGuid(uMeleeTarget);

			// Send the request...
			pxMessage->SetToGuid(m_uOwnerGuid);
			pxMessage->SetFromGuid(m_uOwnerGuid);
			Asura_Network::Send(pxMessage);
		} 
	}
	else if (!bMeleeReconClash)
	{
		PARENT::UpdateMeleeRequests(xClient);
	}
}


const Asura_Hash_ID uBRANCH_ATTRACTOR_TYPE = 0xADAF25A2; //Asura_GetHashID("Branch");
DEBUGVAR float dbg_fBranchMinDotProductForWalk = 0.707f;

void AVP_Predator_InputHandler::TestBranchAttractor(const Asura_Attractor_Compiled& xAttractor, AVP_Predator_InputHandler::CallbackData& xData)
{
	if (xAttractor.GetType() != uBRANCH_ATTRACTOR_TYPE) { return; }

	Asura_Vector_3 xNormal;
	xAttractor.GetNormal(xNormal);
	// dot product with up, i.e. (0,-1,0), is equal to -y.
	if (-xNormal.y < dbg_fBranchMinDotProductForWalk) { return; }

	const Asura_Vector_3& xPredPos = xData.m_xPredator.GetCurrentPosition();

	if (xAttractor.GetID() == xData.m_xPredator.GetConstrainedMovementAttractorID())
	{
		// This is the attractor we're already on. Just check the dot product of the input against the direction
		if (xData.m_xTestDirection.ApproximatelyEquals(Asura_Vector_3(0.0f, 1.0f, 0.0f)))
		{
			// No input, make this the best choice.
			xData.m_uBestAttractor = xAttractor.GetID();
			xData.m_fBestDPResult = 1.0f;
			xData.m_xBestPosOnAttractor = xPredPos;
		}
		else
		{
			Asura_Vector_3 xAttractorDirection;
			xAttractor.GetAxisDirection(xAttractorDirection);
			xAttractorDirection.y = 0.0f;
			xAttractorDirection.Normalise();
			// Abs means it's irrelevant whether we're pushing along or opposite to the line, which is what we want because we must be somewhere in the middle
			const float fDP = Asura_Maths::Abs(xAttractorDirection * xData.m_xTestDirection);
			// This should take priority on the equality case (if it ever comes up).
			if (fDP >= xData.m_fBestDPResult)
			{
				xData.m_uBestAttractor = xAttractor.GetID();
				xData.m_fBestDPResult = fDP;
				xData.m_xBestPosOnAttractor = xPredPos;
			}
		}
		return;
	}

	const Asura_Vector_3& xNode0Pos = xAttractor.GetNode(0)->GetPosition();
	const Asura_Vector_3 xAttractorVector = xAttractor.GetNode(1)->GetPosition() - xNode0Pos;

	Asura_Vector_3 xPosOnAttractor, xPosOffset;
	Asura_Maths::GetLeastOffsetBetweenLineSegmentAndRay(xPredPos, xData.m_xTestDirection, xNode0Pos, xAttractorVector, xPosOffset, &xPosOnAttractor);

	// The actual offset we care about is the offset between the point on the attractor and the predator position.
	xPosOffset = xPosOnAttractor - xPredPos;

	if (-xPosOffset.y > m_fMaxDistanceBelowBranchForAttach) { return; }
	if (xPosOffset.MagnitudeSquared() > m_fMaxDistanceBetweenPredAndBranchForAttachSquared) { return; }

	xPosOffset.y = 0.0f;

	if (xPosOffset.ApproximatelyEquals(Asura_Maths::ZeroVector3))
	{
		// We're on the line, test against the direction
		if (xData.m_xTestDirection.ApproximatelyEquals(Asura_Vector_3(0.0f, 1.0f, 0.0f)) && (xData.m_xPredator.GetConstrainedMovementAttractorID() == uASURA_ATTRACTOR_ID_NONE))
		{
			// No input and we're not already attached to an attractor (if we are, then that has to take priority), make this the best choice.
			xData.m_uBestAttractor = xAttractor.GetID();
			xData.m_fBestDPResult = 1.0f;
			xData.m_xBestPosOnAttractor = xPredPos;
		}
		else
		{
			Asura_Vector_3 xAttractorDirection;
			xAttractor.GetAxisDirection(xAttractorDirection);
			xAttractorDirection.y = 0.0f;
			xAttractorDirection.Normalise();
			// Abs means it's irrelevant whether we're pushing along or opposite to the line, which is what we want because we're somewhere in the middle
			const float fDP = Asura_Maths::Abs(xAttractorDirection * xData.m_xTestDirection);
			if (fDP > xData.m_fBestDPResult)
			{
				xData.m_uBestAttractor = xAttractor.GetID();
				xData.m_fBestDPResult = fDP;
				xData.m_xBestPosOnAttractor = xPredPos;
			}
		}
	}
	else
	{
		xPosOffset.Normalise();
		// Don't use abs this time, as it matters if we're pushing toward this point.
		const float fDP = xPosOffset * xData.m_xTestDirection;
		if (fDP > xData.m_fBestDPResult)
		{
			xData.m_uBestAttractor = xAttractor.GetID();
			xData.m_fBestDPResult = fDP;
			xData.m_xBestPosOnAttractor = xPosOnAttractor;
		}
	}
}

void AVP_Predator_InputHandler::TestBranchAttractors_Callback(void* pData, const Asura_Attractor_Compiled* const pxAttractor)
{
	if (pData && pxAttractor)
	{
		CallbackData* pxData = static_cast<CallbackData*>(pData);
		pxData->m_xInputHandler.TestBranchAttractor(*pxAttractor, *pxData);
	}
}

void AVP_Predator_InputHandler::SendMountBranchMoveMessage(const Asura_Vector_3& xTarget, AVP_ClientActor_Predator& xOwner) const
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_MOVE_START, sizeof(AVP_Message_Predator_Move_MountBranch_Start));

	if (!pxMessage) { return; }

	AVP_Message_Predator_Move_MountBranch_Start* pxData = static_cast<AVP_Message_Predator_Move_MountBranch_Start*>(pxMessage->GetData());

	// Fill-in the message-body...
	pxData->ClearFlags();
	pxData->SetMoveID(AVP_Move_Predator_MountBranch::GetStaticHash());
	pxData->SetForceAbort(true);
	pxData->SetIgnoredByOwner(false);

	pxData->SetDestinationPosition(xTarget);

	// Set the message source/destination...
	pxMessage->SetFromGuid(m_uOwnerGuid);
	pxMessage->SetToGuid(m_uOwnerGuid);

	// Quick hack - forward the message directly to the client...
	xOwner.HandleMessage(pxMessage);
	// ...and don't bother sending it to the other machines, as they can just update based on position changes as normal.
	Asura_Network::DestroyMessage(pxMessage);
}

void AVP_Predator_InputHandler::SendDismountBranchMoveMessage(const Asura_Vector_3& xDirection, AVP_ClientActor_Predator& xOwner) const
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_MOVE_START, sizeof(AVP_Message_Predator_Move_DismountBranch_Start));

	if (!pxMessage) { return; }

	AVP_Message_Predator_Move_DismountBranch_Start* pxData = static_cast<AVP_Message_Predator_Move_DismountBranch_Start*>(pxMessage->GetData());

	// Fill-in the message-body...
	pxData->ClearFlags();
	pxData->SetMoveID(AVP_Move_Predator_DismountBranch::GetStaticHash());
	pxData->SetForceAbort(true);
	pxData->SetIgnoredByOwner(false);

	pxData->SetDirection(xDirection);

	// Set the message source/destination...
	pxMessage->SetFromGuid(m_uOwnerGuid);
	pxMessage->SetToGuid(m_uOwnerGuid);

	// Quick hack - forward the message directly to the client...
	xOwner.HandleMessage(pxMessage);
	// ...and don't bother sending it to the other machines, as they can just update based on position changes as normal.
	Asura_Network::DestroyMessage(pxMessage);
}

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_InputHandler::AVP_Predator_InputHandler
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_Predator_InputHandler::AVP_Predator_InputHandler (const Asura_Guid uOwnerGuid)
: AVP_Vertebrate_InputHandler(uOwnerGuid)
, m_fBranchDismountInputTimer(0.0f)
, m_fMaxDistanceBetweenPredAndBranchForAttachSquared(2.25f)
, m_fMaxDistanceBelowBranchForAttach(0.25f)
, m_fPredatorBranchLOSDistance(1.0f)
, m_fPredatorBranchInputTimeForDismount(0.25f)
, m_fPredatorBranchMaxCosAngleForDismount(0.4f)
, m_fVisionHoldTimer( 0.0f )
, m_bVisioned( false )
{
	// Initialise the control maps
	RefreshBlueprintData();
	Platform_InitialiseControlMap();
#if defined(ASURA_DEBUG_VARIABLE_SYSTEM) && defined(AVP_DEBUG_INPUT)
	if ( s_uConsoleVarReferenceCount == 0 )
	{
		if ( !Asura_CommandConsole::VarExists( L"Controls.Vision.HoldButtonTime" ) )
		{
			Asura_CommandConsole::AddVar( L"Controls.Vision.HoldButtonTime", &dvs_fVisionHoldTime );
		}
	}
	++s_uConsoleVarReferenceCount;
#endif
}

/// End of function 'AVP_Predator_InputHandler::AVP_Predator_InputHandler'. ///


AVP_Predator_InputHandler::~AVP_Predator_InputHandler()
{
#if defined(ASURA_DEBUG_VARIABLE_SYSTEM) && defined(AVP_DEBUG_INPUT)
	--s_uConsoleVarReferenceCount;
#endif
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_InputHandler::SendPlayerCommandMessageToServer
// Author       : Andy Tate
// Date         : 4-Oct-2006
////////////////////////////////////////////////////////////////////////////////

Asura_Message* AVP_Predator_InputHandler::CreatePlayerCommandMessageToServer ( const AVP_ClientEntity_Actor& xClient ) const
{
	bool bShouldCompressMessage = AVP_Game_Core::ShouldCompressPlayerCommandsMessages(); 
	
	if (bShouldCompressMessage)
	{
		// Fill message data
		const AVP_ClientActor_Predator& xPredator = static_cast< const AVP_ClientActor_Predator& >( xClient );
		AVP_Message_PredatorPlayerCommands xPredatorPlayerCommands( xPredator ); 
		const u_int uNoCompressionSize = sizeof( AVP_Message_PredatorPlayerCommands ); 

		// Compress data
		Asura_BitPacker xPacker( uNoCompressionSize ); 
		xPredatorPlayerCommands.Pack( xPacker ); 

		// Send the data
		const u_char* pucSrcData = 0;
		u_int uDataSize = 0;
		xPacker.GetResult( &pucSrcData, &uDataSize );

		Asura_Message*	pxMessage	= Asura_Network::CreateMessage( AVP_MSGID_PREDATORPLAYERCOMMANDS_COMPRESSED, uDataSize );

		// Multiple sanity checks
		if (pxMessage && pucSrcData && uDataSize)
		{
			u_char* pucDstData = static_cast<u_char*>( pxMessage->GetData() ); 
			Asura_Mem::Copy( pucDstData, pucSrcData, uDataSize ); 
		}
		else
		{
			Asura_Assert(false, "Failed to create message.");
		}
		
		return pxMessage; 
	}
	else
	{
		Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_PREDATORPLAYERCOMMANDS, sizeof(AVP_Message_PredatorPlayerCommands));

		if (pxMessage == NULL)
		{
			Asura_Assert(false, "Failed to create message.");
		}
		else
		{
			AVP_Message_PredatorPlayerCommands*	pxData = static_cast<AVP_Message_PredatorPlayerCommands*>(pxMessage->GetData());

			if (pxData == NULL)
			{
				Asura_Assert(false, "Cannot find data in message.");
			}
			else
			{
				const AVP_ClientActor_Predator&	xPredator = static_cast< const AVP_ClientActor_Predator& >( xClient );
				pxData->FillPredator( xPredator );
			}
		}

		// Return the message created...
		return(pxMessage);
	}
}

/// End of function 'AVP_Predator_InputHandler::SendPlayerCommandMessageToServer'. ///

void AVP_Predator_InputHandler::RefreshBlueprintData(void)
{
	const AVP_ClientActor_Predator*	pxOwner	= AVP_ClientActor_Predator::FindEntity(m_uOwnerGuid);

	PARENT::RefreshBlueprintData();

	if(pxOwner != NULL)
	{
		const Asura_Hash_ID		uBlueprintHash	= pxOwner->GetBlueprintHashID();
		const Asura_Hash_ID		uTypeHash		= uAVP_BLUEPRINT_TYPE_PLAYERS;

		const Asura_Blueprint*	pxBlueprint		= Asura_Blueprint_System::GetBlueprintByHashID(uTypeHash, uBlueprintHash);
		if(pxBlueprint)
		{
			Asura_Value xValue;
			if(pxBlueprint->GetParameterValueByHashID(0x44C21088, xValue))	//Asura_GetHashID("BranchAttachDistanceMax"), xValue))
			{
				m_fMaxDistanceBetweenPredAndBranchForAttachSquared = xValue.GetFloatValue();
				m_fMaxDistanceBetweenPredAndBranchForAttachSquared *= m_fMaxDistanceBetweenPredAndBranchForAttachSquared;
			}
			if(pxBlueprint->GetParameterValueByHashID(0xFDD8F3D1, xValue))	//Asura_GetHashID("BranchAttachBelowLimit"), xValue))
			{
				m_fMaxDistanceBelowBranchForAttach = xValue.GetFloatValue();
			}
			if(pxBlueprint->GetParameterValueByHashID(0xD835567E, xValue))	//Asura_GetHashID("BranchAttachLOSDistance"), xValue))
			{
				m_fPredatorBranchLOSDistance = xValue.GetFloatValue();
			}
			if(pxBlueprint->GetParameterValueByHashID(0x9DC91582, xValue))	//Asura_GetHashID("BranchDetachInputTime"), xValue))
			{
				m_fPredatorBranchInputTimeForDismount = xValue.GetFloatValue();
			}
			if(pxBlueprint->GetParameterValueByHashID(0x24504986, xValue))	//Asura_GetHashID("BranchDetachMinInputAngle"), xValue))
			{
				m_fPredatorBranchMaxCosAngleForDismount = Asura_Maths::Cosine(Asura_Maths::DegToRad(xValue.GetFloatValue()));
			}
		}
	}
}

Asura_Message* AVP_Predator_InputHandler::Update( AVP_ClientEntity_Actor& xClient )
{
	Asura_Message* pxPlayerCommandsMessage = PARENT::Update( xClient );

	const AVP_Predator_InteractionHandler* pxIntHand = static_cast < const AVP_Predator_InteractionHandler* >( xClient.GetInteractionHandler() );
	if ( !xClient.IsDead() && ( pxIntHand == NULL || !pxIntHand->IsInteracting() ) )
	{
		AVP_ClientActor_Predator&		xClientPredator	= static_cast<AVP_ClientActor_Predator&>(xClient);
		//AVP_Actor_DynamicStat&			xDynamicEnergy	= *xClientPredator.GetDynamicEnergy();
		AVP_Inventory_System*			pxInvSys = xClientPredator.GetInventorySystem();
		AVP_InventoryItem_Cloak*		pxCloak = pxInvSys ? static_cast< AVP_InventoryItem_Cloak* >( pxInvSys->GetItem( uAVP_INVENTORY_ITEMHASH_CLOAK ) ) : NULL;
		Asura_Assert( pxCloak, "No cloak found" );
		if ( pxCloak )
		{
			if ( pxCloak->IsIdling( true ) && /*xDynamicEnergy.GetCurrentValue() > 0.0f &&*/ GetControlMapValue( CLOAK, true, true ) > 0.0f )
			{
				pxCloak->RequestBeginUsage( AVP_InventoryItem_Cloak::UT_CLOAK, AVP_InventoryItem_Cloak::TT_MANUAL );
			}
			else if ( pxCloak->IsUsing( true ) && GetControlMapValue( AVP_Predator_InputHandler::CLOAK, true, true ) > 0.0f )
			{
				pxCloak->RequestEndUsage( AVP_InventoryItem_Cloak::TT_MANUAL );
			}
		}
	}

	return pxPlayerCommandsMessage;
}
/// End of file. ///
