
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predator_InputHandler.h
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_PREDATOR_INPUTHANDLER_H__
#define __AVP_PREDATOR_INPUTHANDLER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Attractors/Asura_Attractor_Data.h"

#include "Actor/Vertebrates/AVP_Vertebrate_InputHandler.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////
class Asura_Attractor_Compiled;
class AVP_ClientEntity_Actor;
class AVP_ClientActor_Predator;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Predator_InputHandler
// Author       : Andy Tate
// Date         : 3-Oct-2006
//
// Description  : Class to handle the input for a predator client-entity and
//				  create the appropriate message data for the server-entity.
////////////////////////////////////////////////////////////////////////////////

class AVP_Predator_InputHandler : public AVP_Vertebrate_InputHandler
{
public:

	// Public functions...
	AVP_Predator_InputHandler			(const Asura_Guid uOwnerGuid);
	virtual ~AVP_Predator_InputHandler ();

	virtual void RefreshBlueprintData(void);

	// Mapping functions...
	virtual void	MapCloak( Asura_ControlMap& xMap, const u_int uKeyCloak );
	virtual void	MapRecon( Asura_ControlMap& xMap, const u_int uKeyRecon );

protected:
	// Protected functions...	
	virtual	Asura_Message*	Update( AVP_ClientEntity_Actor& xClient );

	virtual void UpdateMovement( AVP_ClientEntity_Actor& xClient );
	virtual void UpdateWeapons( AVP_ClientEntity_Actor& xClient );
	virtual void UpdateVisionModes( AVP_ClientEntity_Actor& xClient );
	virtual void UpdateMeleeRequests( AVP_ClientActor_Vertebrate& xClient );

	virtual void UpdateMovementInput(AVP_ClientEntity_Actor& xClient);

	virtual	Asura_Message*	CreatePlayerCommandMessageToServer( const AVP_ClientEntity_Actor& xClient ) const;

	float m_fBranchDismountInputTimer;

private:
	struct CallbackData
	{
		AVP_Predator_InputHandler& m_xInputHandler;
		const AVP_ClientActor_Predator& m_xPredator;
		const Asura_Vector_3& m_xTestDirection;
		u_int m_uBestAttractor;
		float m_fBestDPResult;
		Asura_Vector_3 m_xBestPosOnAttractor; // Cached so we don't need to calculate it after returning from the search.

		CallbackData(AVP_Predator_InputHandler& xInputHandler, const AVP_ClientActor_Predator& xPredator, const Asura_Vector_3& xTestDirection)
			: m_xInputHandler(xInputHandler)
			, m_xPredator(xPredator)
			, m_xTestDirection(xTestDirection)
			, m_uBestAttractor(uASURA_ATTRACTOR_ID_NONE)
			, m_fBestDPResult(-1.0f)
			, m_xBestPosOnAttractor(Asura_Maths::ZeroVector3)
		{
		}
	};

	// Private functions...
	AVP_Predator_InputHandler  (const AVP_Predator_InputHandler& xOther);

	AVP_Predator_InputHandler&	operator=	(const AVP_Predator_InputHandler& xOther);

	// Platform specific control-map creation...
	void	Platform_InitialiseControlMap	(void);

	void TestBranchAttractor(const Asura_Attractor_Compiled& xAttractor, CallbackData& xData);
	static void TestBranchAttractors_Callback(void* pData, const Asura_Attractor_Compiled* const pxAttractor);

	void SendMountBranchMoveMessage(const Asura_Vector_3& xTarget, AVP_ClientActor_Predator& xOwner) const;
	void SendDismountBranchMoveMessage(const Asura_Vector_3& xDirection, AVP_ClientActor_Predator& xOwner) const;

	// Private variables...
	float m_fMaxDistanceBetweenPredAndBranchForAttachSquared;
	float m_fMaxDistanceBelowBranchForAttach;
	float m_fPredatorBranchLOSDistance;
	float m_fPredatorBranchInputTimeForDismount;
	float m_fPredatorBranchMaxCosAngleForDismount;

	float m_fVisionHoldTimer;
	bool  m_bVisioned;

#if defined(ASURA_DEBUG_VARIABLE_SYSTEM) && defined (AVP_DEBUG_INPUT)
	static u_int s_uConsoleVarReferenceCount;
#endif

	// Private typedefs...
	typedef AVP_Vertebrate_InputHandler PARENT;

};   // End of class 'AVP_Predator_InputHandler'.


#endif   // ifndef __AVP_PREDATOR_INPUTHANDLER_H__


/// End of file. ///
