
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predator_LockOn.cpp
// Author : Chris Wallace
// Date   : 26-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/Predator/AVP_Predator_LockOn.h"

// Asura independent header-files...
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Timers.h"
#include "GUIMenu/AVP_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Widget_Base.h"
#include "Interface/Asura_CommandConsole.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Colour.h"

// Asura platform header-files...

// Game independent header-files...
#include "Actor/Vertebrates/AVP_Vertebrate_InputHandler.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ClientActor_Predator.h"
// Game platform header-files...

// Tools header-files...

DEBUGVAR float dbg_fOffscreenLimit = 0.7f;

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

AVP_Predator_LockOn AVP_Predator_LockOn::s_xSingleton;
float AVP_Predator_LockOn::s_fRotCounter = 0.0f;

u_int AVP_Predator_LockOn::s_uState = 0;
float AVP_Predator_LockOn::s_fLockOnParametric = 0.0f;

Asura_Vector_3 AVP_Predator_LockOn::s_xTargetPos(Asura_Maths::ZeroVector3);

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_LockOn::Update()
// Author       : Chris Wallace
// Date         : 3-Nov-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_LockOn::Update( void )
{
	const float fTimeSlice = Asura_Timers::GetGameTimeSlice();
	
	s_fRotCounter += fTimeSlice;

	const AVP_ClientActor_Predator* pxPred = AVP_ClientActor_Predator::FindEntity( m_uPlayerGuid );
	if ( pxPred->IsDead() )
	{
		UnLock();
	}

	const AVP_Vertebrate_InputHandler* pxInput = pxPred != NULL ? static_cast<const AVP_Vertebrate_InputHandler*>(pxPred->GetInputHandler()) : NULL;
	if ( m_uLockState != TARGET_UNLOCKED && m_uLockTargetGuid != ASURA_GUID_UNREGISTERED && m_fLockCounter > 0.0f && pxInput && pxInput->GetControlMapValue( AVP_Vertebrate_InputHandler::FOCUS_TOGGLE ) > 0.0f )
	{
		UnLock();
	}

	if(m_uLockState == TARGET_UNLOCKED) // If we do not have a lock target, exit
	{
		m_uLockTargetGuid	= ASURA_GUID_UNREGISTERED;
		SetColourA(0x00);
	}
	else if(m_uLockState == TARGET_UNLOCKING)  //If unlocking, fade out
	{
		// Keep it spinning while unlocking
		if(m_bWasCancelled)
		{
			m_fLockCounter -= fTimeSlice; // count back out again	
			
			m_fScale    = ((-Asura_Maths::Sine(Asura_Maths::Pi/(2.0f*m_fPlasmaLockLength)*m_fLockCounter)+1.0f)*m_fPlasmaLockStartScale)+1.0f;
			m_fDistance = (-Asura_Maths::Sine(Asura_Maths::Pi/(2.0f*m_fPlasmaLockLength)*m_fLockCounter)+1.0f)*m_fPlasmaLockStartDist;
		}

		// If we still have the original target, track it
		SetUpTargetPositions();
		
		if(m_fTimeToDie > 0.0f)
		{
			SetColourA(static_cast<u_char>(static_cast<float>(GetDefaultColourA())*(m_fTimeToDie/TARGET_MAX_UNLOCK_TIME)));

			m_fTimeToDie-=Asura_Timers::GetGameTimeSlice();
		}
		else
		{
			m_uLockState = TARGET_UNLOCKED;
			m_fTimeToDie = TARGET_MAX_UNLOCK_TIME;
		}
	}
	else //we have a target.
	{
		// Get the actor we are locking/locked on to.
		AVP_ClientEntity_Actor *pxTarget = AVP_ClientEntity_Actor::FindEntity(m_uLockTargetGuid);

		//Check the entity still exists (why would the entity not exist?!)
		if(pxTarget != NULL)
		{
			if(pxTarget->IsDead())
			{
				UnLock();
				return;
			}
			else
			{
				//Get TimeSlice and update time counted from when locking on began:
				m_fLockCounter += fTimeSlice;

				SetUpTargetPositions();

				if(m_uLockState == TARGET_LOCKING) //If we are locking on...
				{
					m_fScale    = ((-Asura_Maths::Sine(Asura_Maths::Pi/(2.0f*m_fPlasmaLockLength)*m_fLockCounter)+1.0f)*m_fPlasmaLockStartScale)+1.0f;
					m_fDistance = (-Asura_Maths::Sine(Asura_Maths::Pi/(2.0f*m_fPlasmaLockLength)*m_fLockCounter)+1.0f)*m_fPlasmaLockStartDist;

					if(m_fLockCounter >= m_fPlasmaLockLength) //if we have completed locking on...
					{
						// Set as locked and set the colour as a bright red.
						m_uLockState = TARGET_LOCKED;
						SetColourR(0xFF);
						SetColourG(0x99);
						SetColourB(0x99);

						PlayLockAquiredSound();
					}
				}
				else if(m_uLockState == TARGET_LOCKED) //If locked on
				{
					m_fScale = 1.0f;
					m_fDistance =1.0f;

					// Decrease the colour untill equal to the original/default colour
					if(GetColourR() > GetDefaultColourR())
						m_xLockColourR -= 0x11;
					else
						SetColourR(GetDefaultColourR());

					if(GetColourG() > GetDefaultColourG())
						m_xLockColourG -= 0x11;
					else
						SetColourG(GetDefaultColourG());

					if(GetColourB() > GetDefaultColourB())
						m_xLockColourB -= 0x11;
					else
						SetColourB(GetDefaultColourB());
				}
			}

		
		}
		else
		{
			// if the lock target is null, reset the state
			UnLock();
		}
	}

	UpdateLockLoopedSound();

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	UpdateStaticGUIVars();
#endif
}

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
void AVP_Predator_LockOn::CreateConsoleVars()
{
	Asura_CommandConsole::AddVar( L"AVP.PredatorPlayer.LockOn.State", &s_uState, 0, 4, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( L"AVP.PredatorPlayer.LockOn.State" );	

	Asura_CommandConsole::AddVar( L"AVP.PredatorPlayer.LockOn.Parametric", &s_fLockOnParametric, 0.0f, 1.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN );
	Asura_CommandConsole::MakeVarReadOnly( L"AVP.PredatorPlayer.LockOn.Parametric" );	
}

void AVP_Predator_LockOn::UpdateStaticGUIVars()
{
	s_fLockOnParametric = 1.0f - ((m_fScale-1.0f) / (m_fPlasmaLockStartScale-1.0f)); // -1.0f from both, as scale ends at 1.0f

	switch(GetLockState())
	{
		case TARGET_UNLOCKED:
		{
			s_uState = 0;
			break;
		}
		case TARGET_LOCKING:
		{
			s_uState = 1;
			break;
		}
		case TARGET_LOCKED:
		{
			s_uState = 2;
			break;
		}
		case TARGET_UNLOCKING:
		{
			if(!m_bWasCancelled) s_uState = 3; else s_uState = 4;
		}
	}

	Asura_GUIMenu_Widget_Base* pxLockOnGroup   = static_cast<Asura_GUIMenu_Widget_Base*>(AVP_GUIMenu_System::GetHUDElement(AVP_GUIMenu_System::PREDATOR_LOCKON_GROUP));
	if(pxLockOnGroup)
	{
		Asura_Bounding_Rectangle xCurRect(pxLockOnGroup->GetScaledWorldBounds());
		static float fWidth = xCurRect.GetWidth();
		static float fHeight = xCurRect.GetHeight();
		Asura_Bounding_Rectangle xRect(s_xTargetPos.x - fWidth*0.5f, s_xTargetPos.y - fHeight*0.5f, s_xTargetPos.x + fWidth*0.5f, s_xTargetPos.y + fHeight*0.5f);
		pxLockOnGroup->SetScaledWorldBounds(xRect);
	}
}
#endif

void AVP_Predator_LockOn::SetUpTargetPositions()
{
	AVP_ClientEntity_Actor* pxTarget = AVP_ClientEntity_Actor::FindEntity(m_uLockTargetGuid);

	const float fMinTriangleSize = 30.0;

	static Asura_Matrix_3x3 xMatrix;

	// Get targets world co-ords, its screen co-ords and find out if it is in front of you.
	if(pxTarget)
	{
		if(!pxTarget->GetWorldSpaceMarkerPoint(m_uLockedTargetMarkerPointHash, &s_xTargetPos, &xMatrix))
		{
			s_xTargetPos	= pxTarget->GetCurrentPosition();
			xMatrix			= pxTarget->GetCurrentOrientation();	
		}
		
		Asura_Camera::ProjectWorldCoordsToScreenSpace(s_xTargetPos,false);

		m_fTargetPosZ = s_xTargetPos.GetZ();
		//first, check to see if we have looked away from the target, if we have,
		//discard it and exit now.
		Asura_Vector_2 xLockBounds(s_xTargetPos.GetX()-0.5f,s_xTargetPos.GetY()-0.5f);
		if((Asura_Maths::Abs(xLockBounds.x) > dbg_fOffscreenLimit || Asura_Maths::Abs(xLockBounds.y) > dbg_fOffscreenLimit) || m_fTargetPosZ < 0.0f || m_fTargetPosZ > m_fMaxRange )
		{
			UnLock();
		}

		// Convert targets screen co-ords to pixel values.

		s_xTargetPos.SetX( s_xTargetPos.GetX() * Asura_Viewport::GetWidth() * Asura_Camera::GetViewingArea()->m_fWidth);
		s_xTargetPos.SetY( s_xTargetPos.GetY() * Asura_Viewport::GetHeight() * Asura_Camera::GetViewingArea()->m_fHeight);
	}

	
	Asura_Vector_3 xOrigin(s_xTargetPos.GetX(),s_xTargetPos.GetY(),0.0f);

	//Get Rotation
	m_fRot = s_fRotCounter;

	//get the origin (target screen pos)

	Asura_Euler_3 xEuler(0.0f,0.0f,0.0f);

	Asura_Matrix_3x3 xRotMatrix;

	Asura_Matrix_3x3 xScaleMatrix;

	xScaleMatrix.Zero();
	xScaleMatrix.e11 = xScaleMatrix.e22 = xScaleMatrix.e33 = m_fScale;


	//make a 3d point in space for transformation
	Asura_Vector_3 xPoint;
	xPoint.z = 0;
	int i=0;
	int j=0;

	// Create arrays defining the shape of the polygons to be rendered
	const float plasmapointsx[4] = {-50.0f,-42.0f,42.0f,50.0f};
	const float plasmapointsy[4] = {5.0f,0.0f,0.0f,5.0f};

	const float plasmablurpointsx[4] = {-50.0f,-55.0f,55.0f,50.0f};
	const float plasmablurpointsy[4] = {5.0f,200.0f,200.0f,5.0f};

	//Transformations on pixel co-ordinates.
	for( i=0 ; i<TARGET_NUM_POLYS ; i++ )
	{
		xEuler.z=(m_fRot+((Asura_Maths::Pi)/3.0f)*(i*2));
		xEuler.ToMatrix(xRotMatrix);

		for(j=0 ; j<TARGET_NUM_POINTS ; j++)
		{
			//main reticle
			xPoint.x = plasmapointsx[j];
			xPoint.y = (plasmapointsy[j]+fMinTriangleSize);

			xPoint=xPoint*xScaleMatrix;

			xPoint.y += m_fDistance;

			xPoint=xPoint*xRotMatrix;

			xPoint.x += xOrigin.x;
			xPoint.y += xOrigin.y;

			m_xPoint[i][j].x = xPoint.x;
			m_xPoint[i][j].y = xPoint.y;

		}
	}

	for( i=0 ; i<TARGET_NUM_POLYS ; i++ )
	{
		xEuler.z=(m_fRot+((Asura_Maths::Pi)/3.0f)*(i*2));
		xEuler.ToMatrix(xRotMatrix);

		for(j=0 ; j<TARGET_NUM_POINTS ; j++)
		{
			// 'motion blur'
			xPoint.x = plasmablurpointsx[j];

			float fLength = 0.0f;
		
			fLength = ((m_fLockCounter*plasmablurpointsy[j])/m_fPlasmaLockLength);
			if(fLength >= plasmablurpointsy[j])
			{
				fLength = plasmablurpointsy[j];
			}

			if(j == 1 || j==2)
			{
				xPoint.y = 5.0f+((plasmablurpointsy[j]+fMinTriangleSize)-fLength);
			}
			else
			{
				xPoint.y = (plasmablurpointsy[j]+fMinTriangleSize);
			}
	
			//xPoint.y = xPoint.y/(m_fLockCounter/m_fPlasmaLockLength);

			xPoint=xPoint*xScaleMatrix;

			xPoint.y += m_fDistance;

			xPoint=xPoint*xRotMatrix;

			xPoint.x += xOrigin.x;
			xPoint.y += xOrigin.y;

			m_xBlurPoint[i][j].x = xPoint.x;
			m_xBlurPoint[i][j].y = xPoint.y;

		}
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_LockOn::AVP_Predator_LockOn
// Author       : Chris Wallace
// Date         : 3-Nov-2006
////////////////////////////////////////////////////////////////////////////////

AVP_Predator_LockOn::AVP_Predator_LockOn (const Asura_Guid uGuid)
: m_uPlayerGuid				(uGuid)
, m_uLockTargetGuid			(ASURA_GUID_UNREGISTERED)
, m_fLockCounter			(0.0f)
, m_fPlasmaLockLength		(0.6f)		// default time to lock on
, m_fPlasmaLockStartDist	(500.0f)	// default distance from target
, m_fPlasmaLockStartScale	(5.0f)		// default Scale
, m_fMaxRange				(1000.0f)
, m_fScale					(m_fPlasmaLockStartScale)
, m_fDistance				(500.0f)
, m_fRot					(0.0f)
, m_uLockState				(TARGET_UNLOCKED)
, m_bWasCancelled			(false)
, m_pxLockSound				(NULL)
, m_pxLockCancelSound		(NULL)
, m_pxLockConfirmedSound	(NULL)
, m_pxLockStartSound		(NULL)
, m_pxLockingLoopSound		(NULL)
, m_iSoundHandle			(ASURA_INVALID_HANDLE)
{
	AVP_ClientActor_Predator* pxClient = AVP_ClientActor_Predator::FindEntity(m_uPlayerGuid);
	if(pxClient)
	{
		m_pxLockSound		= pxClient->GetSoundEventFromBlueprints(0x18693CE0); //PredatorPlayerAquireTarget
		m_pxLockCancelSound = pxClient->GetSoundEventFromBlueprints(0xDDC10123); //PredatorPlayerCancelTarget
		m_pxLockConfirmedSound = pxClient->GetSoundEventFromBlueprints(0x908136B9); //PredatorPlayerConfirmTarget
		m_pxLockStartSound = pxClient->GetSoundEventFromBlueprints(0xE541C86F); //PredatorPlayerLockingOnTarget
		m_pxLockingLoopSound= pxClient->GetSoundEventFromBlueprints(0xB21249A0); //PredatorPlayerTrackingTarget
	}

	SetDefaultColourR(0x9A);
	SetDefaultColourG(0x35);
	SetDefaultColourB(0x2D);
	SetDefaultColourA(0x88);

	ResetColour();
	Reset();
}

/// End of function 'AVP_Predator_LockOn::AVP_Predator_LockOn'. ///

void AVP_Predator_LockOn::Shutdown()
{
	m_uPlayerGuid			= ASURA_GUID_UNREGISTERED;
	m_uLockTargetGuid		= ASURA_GUID_UNREGISTERED;
	m_fLockCounter			= 0.0f;
	m_fPlasmaLockLength		= 0.6f;
	m_fPlasmaLockStartDist	= 500.0f;
	m_fPlasmaLockStartScale	= 5.0f;
	m_fMaxRange				= 1000.0f;
	m_fScale				= m_fPlasmaLockStartScale;
	m_fDistance				= 500.0f;
	m_fRot					= 0.0f;
	m_uLockState			= TARGET_UNLOCKED;

	ResetColour();
	Reset();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_LockOn::IsLockedOn
// Author       : Chris Wallace
// Date         : 8-Nov-2006
////////////////////////////////////////////////////////////////////////////////

bool AVP_Predator_LockOn::IsLockedOn (void) const
{
	return(m_uLockState == TARGET_LOCKED);
}

/// End of function 'AVP_Predator_LockOn::IsReaIsLockedOndyToJump'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_LockOn::Render
// Author       : Andy Tate
// Date         : 11-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_LockOn::Render (void) const
{
	//////////////////////////
	// LockOn Rendering     //
	//////////////////////////
/*
	int i=0; 
	int j=0;

	if(m_uLockState != TARGET_UNLOCKED)
	{
		if(m_fTargetPosZ > 0.0f)
		{
			//Set a quad to have no texture and red colour
			Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
			Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
			Asura_Render::Overlay.m_bGouraudShaded = false;
			Asura_Render::Overlay.m_bNonRotated = true;

			Asura_RGBA xRGBA(
				static_cast<float>(GetColourR())/255.0f,
				static_cast<float>(GetColourG())/255.0f,
				static_cast<float>(GetColourB())/255.0f,
				static_cast<float>(GetColourA())/255.0f);

			Asura_Render::Overlay.Colour = xRGBA.ToUIntARGB();

			// Render main reticle (sp?)
			for(i=0;i<TARGET_NUM_POLYS;i++)
			{
				for(j=0;j<TARGET_NUM_POINTS;j++)
				{
					Asura_Render::Overlay.m_bGouraudShaded = false;
					Asura_Render::Overlay.X[j] = m_xPoint[i][j].x;
					Asura_Render::Overlay.Y[j] = m_xPoint[i][j].y;
				}
					Asura_Render::Overlay.Render();
			}
		
			// Render 'motion blur'
			if(m_uLockState == TARGET_LOCKING)
			{
				Asura_Render::Overlay.GouraudColour[0] = xRGBA.ToUIntARGB();
				Asura_Render::Overlay.GouraudColour[3] = xRGBA.ToUIntARGB();

				xRGBA.m_fA = 0.0f;

				Asura_Render::Overlay.GouraudColour[1] = xRGBA.ToUIntARGB();
				Asura_Render::Overlay.GouraudColour[2] = xRGBA.ToUIntARGB();

				for(i=0;i<TARGET_NUM_POLYS;i++)
				{
					for(j=0;j<TARGET_NUM_POINTS;j++)
					{
						Asura_Render::Overlay.m_bGouraudShaded = true;
						Asura_Render::Overlay.X[j] = m_xBlurPoint[i][j].x;
						Asura_Render::Overlay.Y[j] = m_xBlurPoint[i][j].y;
					}

					Asura_Render::Overlay.Render();
				}
			}
		}
	}*/
}

/// End of function 'AVP_Predator_LockOn::DebugRender'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_LockOn::SetLockTarget()
// Author       : Chris Wallace
// Date         : 6-Nov-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_LockOn::SetLockTarget(Asura_Guid uTarget, Asura_Hash_ID uMarkerHash)
{
	if ( uTarget != m_uLockTargetGuid)
	{
		m_uLockTargetGuid = uTarget;
		m_uLockedTargetMarkerPointHash = uMarkerHash;

		if(uTarget != ASURA_GUID_UNREGISTERED)
		{
			m_fLockCounter = 0.0f;
			m_uLockState = TARGET_LOCKING;

			ResetColour();
			Reset();

			PlayLockStartSound();
			PlayLockLoopedSound();
		}
	}		
}

/// End of function 'AVP_Predator_LockOn::SetLockTarget'. ///


void AVP_Predator_LockOn::MoveLockTarget(Asura_Guid uTarget, Asura_Hash_ID uMarkerHash)
{
	if( uTarget != m_uLockTargetGuid )
	{
		m_uLockTargetGuid = uTarget;
		m_uLockedTargetMarkerPointHash = uMarkerHash;
	}
}

void AVP_Predator_LockOn::SetOwnerGuid( const Asura_Guid uGuid )
{
	m_uPlayerGuid = uGuid;
	AVP_ClientActor_Predator* pxClient = AVP_ClientActor_Predator::FindEntity(m_uPlayerGuid);
	if(pxClient)
	{
		m_pxLockSound		= pxClient->GetSoundEventFromBlueprints(0x18693CE0); //PredatorPlayerAquireTarget
		m_pxLockCancelSound = pxClient->GetSoundEventFromBlueprints(0xDDC10123); //PredatorPlayerCancelTarget
		m_pxLockConfirmedSound = pxClient->GetSoundEventFromBlueprints(0x908136B9); //PredatorPlayerConfirmTarget
		m_pxLockStartSound = pxClient->GetSoundEventFromBlueprints(0xE541C86F); //PredatorPlayerLockingOnTarget
		m_pxLockingLoopSound= pxClient->GetSoundEventFromBlueprints(0xB21249A0); //PredatorPlayerTrackingTarget
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_LockOn::ResetColour()
// Author       : Chris Wallace
// Date         : 7-Nov-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_LockOn::ResetColour( void )
{
	SetColourR(GetDefaultColourR());
	SetColourG(GetDefaultColourG());
	SetColourB(GetDefaultColourB());
	SetColourA(GetDefaultColourA());
}
/// End of function 'AVP_Predator_LockOn::ResetColour'. ///

void AVP_Predator_LockOn::Reset( void )
{
	m_fScale	=	m_fPlasmaLockStartScale;
	m_fDistance =	500.0f;
	m_fRot		=	0.0f;
	m_bWasCancelled = false;
	
	Asura_Sound_Event_System::Stop(m_iSoundHandle);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_LockOn::UnLock()
// Author       : Chris Wallace
// Date         : 9-Nov-2006
// Description  : Removes Lock from target
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_LockOn::UnLock( void )
{
	if(m_uLockState	== TARGET_LOCKING || m_uLockState == TARGET_LOCKED)
	{
		m_bWasCancelled		= (m_uLockState==TARGET_LOCKING)?true:false;
		m_uLockState		= TARGET_UNLOCKING;
		m_fTimeToDie		= TARGET_MAX_UNLOCK_TIME;

		PlayLockCancelledSound();
	}
}
/// End of function 'AVP_Predator_LockOn::UnLock'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_LockOn::GetLockTargetPosition()
// Author       : Chris Wallace
// Date         : 20-Nov-2006
// Description  : Get Position locked on to in world co-ordinates
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_LockOn::GetLockTargetPosition(Asura_Vector_3& xOutPut) const
{
	AVP_ClientEntity_Actor *pxTarget = AVP_ClientEntity_Actor::FindEntity(m_uLockTargetGuid);
	if(pxTarget)
	{
		Asura_Matrix_3x3 xMatrix;
		pxTarget->GetWorldSpaceMarkerPoint(m_uLockedTargetMarkerPointHash, &xOutPut, &xMatrix);
	}
	else
	{
		xOutPut.Zero();
	}
}

/// End of function 'AVP_Predator_LockOn::GetLockTargetPosition'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Predator_LockOn::GetLockTargetPosition()
// Author       : Chris Wallace
// Date         : 13-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Predator_LockOn::PlayLockAquiredSound()
{
	if(m_pxLockSound)
	{
		Asura_Sound_Event_System::PlayEvent(m_pxLockSound, NULL);
	}
}

void AVP_Predator_LockOn::PlayLockCancelledSound()
{
	Asura_Sound_Event_System::Stop(m_iSoundHandle);
	if(m_pxLockCancelSound)
	{
		Asura_Sound_Event_System::PlayEvent(m_pxLockCancelSound, NULL);
	}
}

void AVP_Predator_LockOn::PlayLockConfirmedSound()
{
	Asura_Sound_Event_System::Stop(m_iSoundHandle);
	if(m_pxLockConfirmedSound)
	{
		Asura_Sound_Event_System::PlayEvent(m_pxLockConfirmedSound, NULL);
	}
}

void AVP_Predator_LockOn::PlayLockStartSound()
{
	if(m_pxLockStartSound)
	{
		Asura_Sound_Event_System::PlayEvent(m_pxLockStartSound, NULL);
	}
}

void AVP_Predator_LockOn::PlayLockLoopedSound()
{
	if(m_pxLockingLoopSound)
	{
		if (m_iSoundHandle != ASURA_INVALID_HANDLE && !Asura_Sound_Event_System::IsPlaying(m_iSoundHandle))
		{
			Asura_Sound_Event_System::Stop( m_iSoundHandle );
		}
		if ( m_iSoundHandle == ASURA_INVALID_HANDLE)
		{
			Asura_Sound_Event_System::PlayEvent(m_pxLockingLoopSound, &m_iSoundHandle);
		}
	}
}

void AVP_Predator_LockOn::UpdateLockLoopedSound( void )
{
	if(m_pxLockingLoopSound && Asura_Sound_Event_System::IsPlaying(m_iSoundHandle))
	{
		Asura_Sound_Event_System::SetPitchScale(m_iSoundHandle, 1.0f);
		Asura_Sound_Event_System::SetVolumeScale(m_iSoundHandle, 1.0f);
	}
}


/// End of function 'PlayLockSound'

/// End of file. ///
