
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Sentrygun_AIAnimStateData.cpp
// Author : Will Plano
// Date   : 27/09/2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Actor/Weapons/Sentrygun/AVP_Sentrygun_AIAnimStateData.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_Hash.h"
#include "Dynamics/Asura_Collision_Object.h"

// AVP
#include "Actor/AVP_Actor_AnimStateData.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Physics/AVP_Collision_Descriptor.h"

Asura_TemplateHashTree< AVP_Sentrygun_AIAnimStateData::AVP_Sentrygun_AIAnimWeaponStateData > AVP_Sentrygun_AIAnimStateData::s_xHashTree;

AVP_Sentrygun_AIAnimStateData::AVP_Sentrygun_AIAnimWeaponStateData::AVP_Sentrygun_AIAnimWeaponStateData()
{
	for ( u_int uStateItr = 0; uStateItr < AVP_Sentrygun_AIAnimStateMachine::SENTRYGUN_STATE_END; ++uStateItr )
	{
		m_apxAnimStateData[ uStateItr ] = NULL;
	}
}

AVP_Sentrygun_AIAnimStateData::AVP_Sentrygun_AIAnimWeaponStateData::~AVP_Sentrygun_AIAnimWeaponStateData()
{
	for ( u_int uStateItr = 0; uStateItr < AVP_Sentrygun_AIAnimStateMachine::SENTRYGUN_STATE_END; ++uStateItr )
	{
		delete m_apxAnimStateData[ uStateItr ];
	}
}

void AVP_Sentrygun_AIAnimStateData::AVP_Sentrygun_AIAnimWeaponStateData::InitialiseFromBlueprint( const Asura_Hash_ID uAnimStateBlueprintType, const Asura_Hash_ID uWeaponBlueprintHash )
{
	const Asura_Blueprint* pxStateBlueprint = NULL;
	const Asura_Blueprint* pxWeaponBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uWeaponBlueprintHash );
	Asura_Assertf( pxWeaponBlueprint, ( "Unable to get weapon blueprint 0x%08x", uWeaponBlueprintHash ) );
	if ( pxWeaponBlueprint )
	{
		Asura_Value xValue;
		if ( pxWeaponBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ANIMSTATE_BLUEPRINT_TP, xValue ) )
		{
			Asura_Hash_ID uAIAnimStateBPHash = xValue.GetHashIDValue();
			pxStateBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAnimStateBlueprintType, uAIAnimStateBPHash );
			Asura_Assertf( pxStateBlueprint, ( "Unable to get state blueprint 0x%08x", uAIAnimStateBPHash ) );
		}
#ifdef ASURA_ASSERT
		else
		{
			Asura_Assertf( false, ( "Unable to get actor state blueprint parameter from weapon blueprint 0x%08x", uWeaponBlueprintHash ) );
		}
#endif
	}
	
	for ( u_int uStateItr = AVP_Sentrygun_AIAnimStateMachine::ACTOR_STATE_IDLE; uStateItr < AVP_Sentrygun_AIAnimStateMachine::SENTRYGUN_STATE_END; ++uStateItr )
	{
		m_apxAnimStateData[ uStateItr ] = NULL;
		if ( pxStateBlueprint )
		{
			m_apxAnimStateData[ uStateItr ] = new AVP_Actor_AnimStateData_NonDirectional();
			m_apxAnimStateData[ uStateItr ]->InitialiseFromBlueprint( uAnimStateBlueprintType, *pxStateBlueprint, AVP_Sentrygun_AIAnimStateMachine::GetStateDescriptionStatic( uStateItr ), true );
		}
	} // end state loop
}

const AVP_Actor_AnimStateData* AVP_Sentrygun_AIAnimStateData::GetAnimStateData( const Asura_Hash_ID uAnimStateBlueprintType, const Asura_Hash_ID uCurrentWeaponHash, u_int uState )
{
	Asura_Assert( uState < AVP_Sentrygun_AIAnimStateMachine::SENTRYGUN_STATE_END, "Invalid state" );
	if ( uState < AVP_Sentrygun_AIAnimStateMachine::SENTRYGUN_STATE_END )
	{
		const AVP_Sentrygun_AIAnimWeaponStateData* pxWeaponData = GetWeaponDataByHash( uAnimStateBlueprintType, uCurrentWeaponHash );
		Asura_Assert( pxWeaponData != NULL, "No weapon state data" );
		if ( pxWeaponData != NULL )
		{
			return pxWeaponData->m_apxAnimStateData[ uState ];
		}
	}
	return NULL;
}

const AVP_Sentrygun_AIAnimStateData::AVP_Sentrygun_AIAnimWeaponStateData* AVP_Sentrygun_AIAnimStateData::GetWeaponDataByHash( const Asura_Hash_ID uAnimStateBlueprintType, const Asura_Hash_ID uHash )
{
	AVP_Sentrygun_AIAnimWeaponStateData* pxData = s_xHashTree.GetDataByHash( uHash, false );
	if ( pxData == NULL && uHash != ASURA_HASH_ID_UNSET )
	{
		pxData = s_xHashTree.AddNode( uHash );
		Asura_Assert( pxData != NULL, "Unable to add new anim state data to tree" );
		if ( pxData != NULL )
		{
			pxData->InitialiseFromBlueprint( uAnimStateBlueprintType, uHash );
		}
	}
	return pxData;
}

/// End of file. ///
