
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Sentrygun_AIAnimStateMachine.h
// Author : Will Plano
// Date   : 25/06/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SENTRYGUN_AIANIMSTATEMACHINE_H__
#define __AVP_SENTRYGUN_AIANIMSTATEMACHINE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/AVP_Actor_AIAnimStateMachine.h"

class AVP_Sentrygun_AIAnimStateMachine : public AVP_Actor_AIAnimStateMachine
{
public:
	// Public enumerated types...

	enum SentrygunAnimState
	{
		SENTRYGUN_STATE_BEGIN = ACTOR_STATE_END,
		SENTRYGUN_STATE_SWEEPING = SENTRYGUN_STATE_BEGIN,	// 4
		SENTRYGUN_STATE_DISABLED,							// 5
		SENTRYGUN_STATE_DESTROYED,							// 6
		SENTRYGUN_STATE_END									// 7
	};

	// Public functions...
	AVP_Sentrygun_AIAnimStateMachine( AVP_ServerEntity_Actor& xServer ) : PARENT( xServer ) {}
	virtual ~AVP_Sentrygun_AIAnimStateMachine() {}

	virtual void Initialise();

	void	RequestSweeping() { RequestState( SENTRYGUN_STATE_SWEEPING ); }
	bool	IsSweeping() const { return IsStateSweeping( GetCurrentState() ); }
	bool	IsStateSweeping( const u_int uState ) const { return SENTRYGUN_STATE_SWEEPING == uState; }

	void	RequestDisabled() { RequestState( SENTRYGUN_STATE_DISABLED ); }
	bool	IsDisabled() const { return IsStateDisabled( GetCurrentState() ); }
	bool	IsStateDisabled( const u_int uState ) const { return SENTRYGUN_STATE_DISABLED == uState; }

	void	RequestDestroyed() { RequestState( SENTRYGUN_STATE_DESTROYED ); }
	bool	IsDestroyed() const { return IsStateDestroyed( GetCurrentState() ); }
	bool	IsStateDestroyed( const u_int uState ) const { return SENTRYGUN_STATE_DESTROYED == uState; }

	virtual const AVP_Actor_AIAnimStateConnectionData* GetConnectionData( const u_int uFromState, const u_int uToState ) const;

	virtual const char* GetStateDescription( const u_int uState ) const { return GetStateDescriptionStatic( uState ); }
	static const char* GetStateDescriptionStatic( const u_int uState );

	virtual bool IsLoopedState( const u_int uState ) const { return IsLoopedStateStatic( uState ); }
	static bool IsLoopedStateStatic( const u_int uState );

protected:
	//virtual void BeginState	( const u_int uState ) { PARENT::BeginState( uState ); }
	//virtual void UpdateState	( const u_int uState ) { PARENT::UpdateState( uState ); }
	//virtual bool EndState		( const u_int uState ) { return PARENT::EndState( uState ); }

	virtual u_int	GetNumStates() const { return SENTRYGUN_STATE_END; }
	virtual const AVP_Actor_AnimStateData* GetStateAnimData( const u_int uState ) const;

#ifndef ASURA_NO_DEBUG_OUTPUT
	void RenderDebugInfo();
#endif

private:
	typedef AVP_Actor_AIAnimStateMachine PARENT;

	AVP_Sentrygun_AIAnimStateMachine();
	AVP_Sentrygun_AIAnimStateMachine(const AVP_Sentrygun_AIAnimStateMachine& xOther);
	AVP_Sentrygun_AIAnimStateMachine&	operator=(const AVP_Sentrygun_AIAnimStateMachine& xOther);
}; // End of class 'AVP_Sentrygun_AIAnimStateMachine'.

#endif // ifndef __AVP_SENTRYGUN_AIANIMSTATEMACHINE_H__

/// End of file. ///
