
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Sentrygun_AnimationHandler.h
// Author : Chris Wallace
// Date   : 16-Apr-2007
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SENTRYGUN_ANIMATIONHANDLER_H__
#define __AVP_SENTRYGUN_ANIMATIONHANDLER_H__

#include "Animation/Asura_Animation.h"

#include "Actor/AVP_Actor_AnimationHandler.h"

class Asura_AnimationController_Group;
class AVP_ClientWeapon_Sentrygun;


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Sentrygun_AnimationHandler
// Author       : Andy Tate
// Date         : 18-Oct-2006
//
// Description  : Class to handle the animations needed by the Alien.
////////////////////////////////////////////////////////////////////////////////

class AVP_Sentrygun_AnimationHandler : public AVP_Actor_AnimationHandler
{
public:
	AVP_Sentrygun_AnimationHandler(AVP_ClientWeapon_Sentrygun& xOwner);

	virtual void Update();
	virtual void Initialise( void );

	virtual	bool MakeRagdoll( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation ) { return false; }

	virtual void	UpdateAnimStateData( const AVP_Actor_AnimStateData* pxAnimDataLBSlow, const AVP_Actor_AnimStateData* pxAnimDataLBFast, const AVP_Actor_AnimStateData* pxAnimDataUBSlow, const AVP_Actor_AnimStateData* pxAnimDataUBFast, const bool bLoop, const float fBlendLBSpeed, const float fBlendUBSpeed, const float fOSMoveDirAngle, const float fTweenTime, const bool bSynch );
	virtual void	PauseCurrentAnimState();
	virtual void	PlayCurrentAnimState();
	// these are only needed by the AI but should work for players too should the need arise
	virtual bool	IsCurrentAnimStateFinished() const;
	virtual void	SetCurrentAnimStateLooped( const bool bSet );
	
	bool SetDamagedSkin();

	virtual const AVP_Actor_AnimStateData* GetCurrentAnimStateData() const { return m_pxCurrentStateData; }

protected:

private:
	typedef AVP_Actor_AnimationHandler PARENT;

	void SetAimAnimations( Asura_Hash_ID uAimDownUpHash, Asura_Hash_ID uAimRightLeftHash, const float fTweemTime );

	AVP_ClientWeapon_Sentrygun&	m_xOwnerSentryGun;

	Asura_Hash_ID m_uRootAimGroupHash;

	Asura_Hash_ID m_uStateAnim;
	Asura_Hash_ID m_uStateAACAHash;

	Asura_Hash_ID m_uAimRightLeftAnimHash;
	Asura_Hash_ID m_uAimDownUpAnimHash;

	Asura_Hash_ID m_uAimRightLeftAACAHash;
	Asura_Hash_ID m_uAimDownUpAACAHash;

	float m_fMaxAimRightLeft;
	float m_fMaxAimDownUp;

	const AVP_Actor_AnimStateData* m_pxCurrentStateData;

	AVP_Sentrygun_AnimationHandler  (void);
	AVP_Sentrygun_AnimationHandler  (const AVP_Sentrygun_AnimationHandler& xOther);
	AVP_Sentrygun_AnimationHandler&	operator=	(const AVP_Sentrygun_AnimationHandler& xOther);
};   // End of class 'AVP_Sentrygun_AnimationHandler'.

#endif   // ifndef __AVP_SENTRYGUN_ANIMATIONHANDLER_H__

/// End of file. ///
