
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Animation_IK_Chain_Tail.cpp
// Author : Will Plano
// Date   : 04/09/2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_Timers.h"

#include "Animation/AVP_Animation_IK_Chain_Tail.h"

#include "Animation/IK/Asura_Animation_IK_Link.h"
#include "Debugging/Asura_DebugLineRender.h"
#include "Interface/Asura_CommandConsole.h"

DVSVAR bool  dvs_bRenderDebugInfo	= false;
DVSVAR float dvs_fSpringBack		= 0.20f;
DVSVAR float dvs_fSpringShare		= 0.50f;
DVSVAR float dvs_fDefaultAttractive = 0.50f;
DVSVAR float dvs_fDefaultDrag		= 0.65f;
DVSVAR float dvs_fGravity			= 0.01f;

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	u_int AVP_Animation_IK_Chain_Tail::s_uConsoleVarReferenceCount = 0;
#endif

//////////////////////////////////////////////////////////////////////////
// class Asura_Animation_IK_Chain_MarkerPoint
//////////////////////////////////////////////////////////////////////////

AVP_Animation_IK_Chain_Tail::AVP_Animation_IK_Chain_Tail( const u_int uNumBones /* = 0 */ )
: PARENT( uNumBones )
, m_fAttractive( dvs_fDefaultAttractive )
, m_fDrag( dvs_fDefaultDrag )
{
#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	if ( s_uConsoleVarReferenceCount == 0 )
	{
#if !defined(ASURA_NO_DEBUG_OUTPUT)
		if ( !Asura_CommandConsole::VarExists( L"IK.RenderTailDebugInfo" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.RenderTailDebugInfo", &dvs_bRenderDebugInfo );
		}
#endif // ASURA_NO_DEBUG_OUTPUT
		if ( !Asura_CommandConsole::VarExists( L"IK.SpringBack" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.SpringBack", &dvs_fSpringBack, 0.0f );
		}
		if ( !Asura_CommandConsole::VarExists( L"IK.SpringShare" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.SpringShare", &dvs_fSpringShare, 0.0f );
		}
		if ( !Asura_CommandConsole::VarExists( L"IK.Attractive" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.Attractive", &dvs_fDefaultAttractive, 0.0f );
		}
		if ( !Asura_CommandConsole::VarExists( L"IK.Drag" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.Drag", &dvs_fDefaultDrag, 0.0f );
		}
		if ( !Asura_CommandConsole::VarExists( L"IK.Gravity" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.Gravity", &dvs_fGravity, 0.0f );
		}
	}
	++s_uConsoleVarReferenceCount;
#endif // ASURA_SUPPORT_COMMAND_CONSOLE
}

AVP_Animation_IK_Chain_Tail::AVP_Animation_IK_Chain_Tail( const int* piBoneIDs, const int* piTwistBoneIDs /* = NULL */ )
: PARENT( piBoneIDs, piTwistBoneIDs )
, m_fAttractive( dvs_fDefaultAttractive )
, m_fDrag( dvs_fDefaultDrag )
{
#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	if ( s_uConsoleVarReferenceCount == 0 )
	{
#if !defined(ASURA_NO_DEBUG_OUTPUT)
		if ( !Asura_CommandConsole::VarExists( L"IK.RenderTailDebugInfo" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.RenderTailDebugInfo", &dvs_bRenderDebugInfo );
		}
#endif // ASURA_NO_DEBUG_OUTPUT
		if ( !Asura_CommandConsole::VarExists( L"IK.SpringBack" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.SpringBack", &dvs_fSpringBack, 0.0f );
		}
		if ( !Asura_CommandConsole::VarExists( L"IK.SpringShare" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.SpringShare", &dvs_fSpringShare, 0.0f );
		}
		if ( !Asura_CommandConsole::VarExists( L"IK.Attractive" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.Attractive", &dvs_fDefaultAttractive, 0.0f );
		}
		if ( !Asura_CommandConsole::VarExists( L"IK.Drag" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.Drag", &dvs_fDefaultDrag, 0.0f );
		}
		if ( !Asura_CommandConsole::VarExists( L"IK.Gravity" ) )
		{
			Asura_CommandConsole::AddVar( L"IK.Gravity", &dvs_fGravity, 0.0f );
		}
	}
	++s_uConsoleVarReferenceCount;
#endif // ASURA_SUPPORT_COMMAND_CONSOLE
}

AVP_Animation_IK_Chain_Tail::~AVP_Animation_IK_Chain_Tail()
{
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	--s_uConsoleVarReferenceCount;
#endif
}

bool AVP_Animation_IK_Chain_Tail::CalculateTarget()
{
	return true;
}

bool AVP_Animation_IK_Chain_Tail::SolveLoopFunction()
{
	bool bDoneSomething = false;
	for ( Asura_Animation_IK_Link* pxLink = m_pxFirstLink; pxLink; pxLink = pxLink->GetNextLink() )
	{
		// Do a bit of gravity stuff;
		if ( pxLink->GetPrevLink() )
		{
			Asura_AVector3 xGravityEffect( 0.0f, dvs_fGravity, 0.0f );
#ifndef ASURA_NO_DEBUG_OUTPUT
			if ( dvs_bRenderDebugInfo )
			{
				Asura_DebugLineRender::AddLine( pxLink->GetPosition(), pxLink->GetPosition() + xGravityEffect, 0xFFFFFFFF );
			}
#endif
			pxLink->IncPosition( xGravityEffect );
		}

		CollideNodes();

		// Do some springy func.
		if ( pxLink->GetNextLink() && !pxLink->GetQuat().ApproximatelyEquals( pxLink->GetAnimQuat() ) )
		{
			Asura_Quat xPrevOri;
			if ( pxLink->GetPrevLink() )
			{
				xPrevOri = pxLink->GetPrevLink()->GetQuat();
			}
			else
			{
				xPrevOri = m_xChainOri;
			}
			Asura_Quat xInvPrevOri = xPrevOri;
			xInvPrevOri.Invert();
			Asura_Quat xDeltaOri = pxLink->GetQuat();
			xDeltaOri.Invert();
			xDeltaOri = pxLink->GetAnimQuat() * xDeltaOri;
			xDeltaOri = xInvPrevOri * xDeltaOri * xPrevOri;
			xDeltaOri = xDeltaOri.ScaleRotation( dvs_fSpringBack );
			xDeltaOri = xPrevOri * xDeltaOri * xInvPrevOri;
			xDeltaOri.Normalise();
#ifndef ASURA_NO_DEBUG_OUTPUT
			if ( dvs_bRenderDebugInfo )
			{
				Asura_AVector3 xRotVec = xDeltaOri.RotateVector( pxLink->GetNextLink()->GetPosition() - pxLink->GetPosition());
				xRotVec += pxLink->GetPosition();
				Asura_DebugLineRender::AddLine( pxLink->GetNextLink()->GetPosition(), xRotVec, 0xFF0000FF );
			}
#endif
			pxLink->IncQuat( xDeltaOri );
			if ( pxLink->GetPrevLink() )
			{
				xDeltaOri.Invert();
				xDeltaOri = xDeltaOri.ScaleRotation( dvs_fSpringShare );
				xDeltaOri.Normalise();
#ifndef ASURA_NO_DEBUG_OUTPUT
				if ( dvs_bRenderDebugInfo )
				{
					Asura_AVector3 xRotVec = xDeltaOri.RotateVector( pxLink->GetPosition() - pxLink->GetPrevLink()->GetPosition());
					xRotVec += pxLink->GetPrevLink()->GetPosition();
					Asura_DebugLineRender::AddLine( pxLink->GetPosition(), xRotVec, 0xFFFFFF00 );
				}
#endif
				pxLink->GetPrevLink()->IncQuat( xDeltaOri );
			}
			bDoneSomething = true;
		}
	}

	return bDoneSomething;
}

bool AVP_Animation_IK_Chain_Tail::SolveEndFunction()
{
	// dvs_fAttractive is the parameter determining how much the desired position attracts the actual tail, a higher value will try to move to the new position faster.
	// dvs_fDrag is the drag applied to the tail, it is the proportion of the velocity to ignore each frame.
	for ( Asura_Animation_IK_Link* pxLink = ( m_pxFirstLink ? m_pxFirstLink->GetNextLink() : NULL ); pxLink; pxLink = pxLink->GetNextLink() )
	{
		if ( pxLink->GetLastPosition().GetX() < 1.0e+020f )
		{
			pxLink->IncVelocity( ( pxLink->GetPosition() - pxLink->GetLastPosition() ) * m_fAttractive * Asura_Timers::GetFrameRate() );
			pxLink->SetVelocity( pxLink->GetVelocity() * ( 1.0f - Asura_Maths::ClampToRange( m_fDrag, 0.0f, 1.0f ) ) );
			Asura_AVector3 xNewPos = pxLink->GetLastPosition() + pxLink->GetVelocity() * Asura_Timers::GetGameTimeSlice();
#ifndef ASURA_NO_DEBUG_OUTPUT
			if ( dvs_bRenderDebugInfo )
			{
				Asura_DebugLineRender::AddLine( pxLink->GetPosition(), xNewPos, 0xFFFF0000 );
			}
#endif
			pxLink->SetPosition( xNewPos );
		}
		pxLink->SetLastPosition( pxLink->GetPosition() );
		pxLink->SetLastAnimPosition( pxLink->GetAnimPosition() );
		pxLink->SetLastQuat( pxLink->GetQuat() );
		pxLink->SetLastAnimQuat( pxLink->GetAnimQuat() );
	}
	m_pxFirstLink->SetLastPosition( m_pxFirstLink->GetPosition() );
	m_pxFirstLink->SetLastAnimPosition( m_pxFirstLink->GetAnimPosition() );
	m_pxFirstLink->SetLastQuat( m_pxFirstLink->GetQuat() );
	m_pxFirstLink->SetLastAnimQuat( m_pxFirstLink->GetAnimQuat() );

	return true;
}
