//////////////////////////////////////////////////////////////////////////////
//
// AVP_Hierarchy_System.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Animation/AVP_Hierarchy_System.h"

// Asura Platform-independent
#include "Rendering/Asura_SortedRender.h"
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"

#include "SpecialFX/AVP_Platform_CloakEffect.h"
#include "SpecialFX/AVP_GlowOutlines.h"

// AVP Platform-independent
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ClientActor_Predator.h"
#include "SpecialFX/AVP_CloakEffect.h"

bool AVP_Hierarchy_System::s_bRenderingSkin;


/******************************************************************************/
/* AVP_Hierarchy_System Static Class Functions                                */
/******************************************************************************/

void AVP_Hierarchy_System::RenderSkin( const Asura_Animation* const pxAnim,
									   const u_int uCamVisFlags,
									   const u_int uRenderFlags,
									   const Asura_Vector_3& xBSCentre,
									   const float fBSRadius )
{
	if(pxAnim == NULL)
	{
		Asura_Assert(false, "NULL pointer passed to AVP_Hierarchy_System::RenderSkin().");
	}
	else
	{
		const AVP_Animation_ProjectData* pxProjectData	= AVP_Animation_ProjectData::FromAnimation(pxAnim);

		if (!pxProjectData || !pxProjectData->GetCloakEffect())
		{
			const Asura_Animation* pxParent = pxAnim;
			do {
				if ((pxParent = pxParent->GetParent()) != NULL)
				{
					pxProjectData	= AVP_Animation_ProjectData::FromAnimation(pxParent);
				}
			} while ((!pxProjectData || !pxProjectData->GetCloakEffect())  && pxParent);
		}

		if (pxProjectData && pxProjectData->GetPheromoneColour().m_fA > 0.0f && !Asura_SortedRender::IsTransparentPass())
		{
			s_bRenderingSkin = true;
			AVP_GlowOutlines::Render(pxAnim, uCamVisFlags, uRenderFlags, xBSCentre, fBSRadius, pxProjectData->GetPheromoneColour());
		}

		if(pxProjectData && pxProjectData->GetCloakEffect() && pxProjectData->GetCloakEffect()->GetState() != AVP_CloakEffect::UNCLOAKED)
		{
			pxProjectData->GetCloakEffect()->Render(*pxAnim, uCamVisFlags, uRenderFlags, xBSCentre, fBSRadius);
		}
		else
		{
			Asura_Hierarchy_System::Platform_RenderSkin
			(
				pxAnim,
				pxAnim->GetAlpha(),
				pxAnim->GetAnimTexID(),
				uCamVisFlags,
				uRenderFlags,
				xBSCentre,
				fBSRadius
			);
			s_bRenderingSkin = false;
		}
	}
}

void AVP_Hierarchy_System::RenderHierarchy( const Asura_Animation* const pxAnim,
										    const u_int uCamVisFlags,
										    const u_int uRenderFlags,
										    const Asura_Vector_3& xBSCentre,
										    const float fBSRadius )
{
	if(pxAnim == NULL)
	{
		Asura_Assert(false, "NULL pointer passed to AVP_Hierarchy_System::RenderHierarchy().");
	}
	else
	{
#if defined(ASURA_ASSERT)
		const AVP_Animation_ProjectData*	pxProjectData	= AVP_Animation_ProjectData::FromAnimation(pxAnim);

		if((pxProjectData != NULL) && (pxProjectData->GetCloakEffect() != NULL) && (pxProjectData->GetCloakEffect()->GetState() != AVP_CloakEffect::UNCLOAKED))
		{
			Asura_Assert(false, "Cloaking of object-hierarchies not supported!");
		}
#endif	//defined(ASURA_ASSERT)

		Asura_Hierarchy_System::Platform_RenderHierarchy(pxAnim, uCamVisFlags, uRenderFlags, xBSCentre, fBSRadius );
	}
}


/// End of file. ///
