//////////////////////////////////////////////////////////////////////////////
//
// AVP_Hierarchy_System.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#if !defined __AVP_HIERARCHY_SYSTEM_H__
#define __AVP_HIERARCHY_SYSTEM_H__

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

#include <Animation/Asura_Animation.h>

class Asura_Vector_3;

////////////////////////////////////////////////////////////////////////////////
//
// AVP_Hierarchy_System
//
// Wrapper class for AVP hierarchy-related functions
//
////////////////////////////////////////////////////////////////////////////////

class AVP_Hierarchy_System
{
public:

	static void Initialise() { Platform_Initialise(); }
	static void Reset()      { Platform_Reset();      }
	static void Shutdown()   { Platform_Shutdown();   }

	static void RenderSkin		( const Asura_Animation* const pxAnim,
								  const u_int uCamVisFlags,
								  const u_int uRenderFlags,
								  const Asura_Vector_3& xBSCentre,
								  const float fBSRadius );

	static void RenderHierarchy	( const Asura_Animation* const pxAnim,
		const u_int uCamVisFlags,
		const u_int uRenderFlags,
		const Asura_Vector_3& xBSCentre,
		const float fBSRadius );

	static bool s_bRenderingSkin;
protected:

	static void Platform_Initialise();
	static void Platform_Reset();
	static void Platform_Shutdown();
};

class AVP_CloakEffect;

class AVP_Animation_ProjectData
{
	AVP_CloakEffect*	m_pxCloakEffect;
	Asura_RGBA          m_xPheromoneColour;
public:
	AVP_Animation_ProjectData() : m_pxCloakEffect(NULL), m_xPheromoneColour(0) {}

	static AVP_Animation_ProjectData * FromAnimation(Asura_Animation *pxAnimation)
	{
		return static_cast<AVP_Animation_ProjectData *>(pxAnimation ? pxAnimation->GetProjectData() : NULL);
	}
	static const AVP_Animation_ProjectData * FromAnimation(const Asura_Animation *pxAnimation)
	{
		return static_cast<const AVP_Animation_ProjectData *>(pxAnimation ? pxAnimation->GetProjectData() : NULL);
	}

	void SetCloakEffect(AVP_CloakEffect* pxCloakEffect) {m_pxCloakEffect = pxCloakEffect;}
	AVP_CloakEffect* GetCloakEffect(void) {return m_pxCloakEffect;}
	const AVP_CloakEffect* GetCloakEffect(void) const {return m_pxCloakEffect;}

	void SetPheromoneColour(const Asura_RGBA &xPheromoneColour) {m_xPheromoneColour = xPheromoneColour;}
	const Asura_RGBA &GetPheromoneColour(void) const {return m_xPheromoneColour;}
	void DisablePheromoneGlow() {if (m_xPheromoneColour.m_fA >= 0.0f) m_xPheromoneColour.m_fA = -m_xPheromoneColour.m_fA; }
	void EnablePheromoneGlow() {if (m_xPheromoneColour.m_fA < 0.0f) m_xPheromoneColour.m_fA = -m_xPheromoneColour.m_fA; }
};

#endif // !defined __AVP_HIERARCHY_SYSTEM_H__
