
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Chunk_Human_PlayerAnimStateData.cpp
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "ChunkSystem/AVP_Chunk_Human_PlayerAnimStateData.h"

#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

#include "Actor/AVP_Actor_AnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_PlayerAnimStateData.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "ChunkSystem/AVP_Chunk_Loaders.h"

static const u_int g_uChunkVersion = 1;

#if defined (ASURA_TOOLS)

bool AVP_Chunk_Human_PlayerAnimStateData::WriteToFile( Asura_String& xFileName, const Asura_Hash_ID uWeaponHash )
{
	Asura_SaveBlock					xSaveBlock;
	Asura_Chunk_SaveBlock_Stream	xStream( &xSaveBlock );

	// Fill-in the save-block...
	WriteToChunkStream( xStream, uWeaponHash );

	// Terminate the Asura-file...
	xStream.WriteAsuraTerminator();

	// ...before saving the file to disc...
	return xSaveBlock.SaveFile( xFileName.GetString(), false );
}

void AVP_Chunk_Human_PlayerAnimStateData::WriteToChunkStream( Asura_Chunk_Stream& xStream, const Asura_Hash_ID uWeaponHash )
{
	// Begin writing...
	xStream.WriteChunk_Start(AVP_CHUNKID_HUMAN_PLAYERANIMSTATEDATA, g_uChunkVersion, 0);

	// This also initialises the weapon data
	const AVP_Human_PlayerAnimStateData::AVP_Human_PlayerAnimWeaponStateData* pxWeaponData = AVP_Human_PlayerAnimStateData::GetSingleton().GetWeaponDataByHash( uAVP_BLUEPRINT_TYPE_ANIMSTATE_MARINE, uWeaponHash );

	Asura_Assertf( pxWeaponData != NULL, ( "CHUNK ERROR, don't check this export in! Weapon data for weapon 0x%08x not found", uWeaponHash ) );
	xStream << ( pxWeaponData != NULL );
	if ( pxWeaponData )
	{
		xStream << uWeaponHash;

		// chunk out all the array sizes and scream blue murder if they've changed.
		xStream << 2;
		xStream << AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_END_NOAIM;
		xStream << AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_END;

		for ( u_int uHudItr = 0; uHudItr < 2; ++uHudItr )
		{
			for ( u_int uStateItr = 0; uStateItr < AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_END_NOAIM; ++uStateItr )
			{
				for ( u_int uSpeedItr = 0; uSpeedItr < AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_END; ++uSpeedItr )
				{
					const AVP_Actor_AnimStateData* pxStateData = pxWeaponData->m_aaapxAnimStateData[ uHudItr ][ uStateItr ][ uSpeedItr ];
					xStream << ( pxStateData != NULL );
					if ( pxStateData != NULL )
					{
						AVP_Actor_AnimStateData::WriteToChunkStream( xStream, pxStateData );
					}
				}
			}
		}
	}

	AVP_Human_PlayerAnimStateData::GetSingleton().ResetStorage();

	xStream.WriteChunk_End();
}

#endif

void AVP_Chunk_Human_PlayerAnimStateData::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion = static_cast< u_int >( Version );
	Asura_Assertf( uVersion > 0 && uVersion <= g_uChunkVersion, ( "Invalid chunk version (%d)!", uVersion ) );
	if ( uVersion == 1 )
	{
		bool bHasWeaponData;
		xStream >> bHasWeaponData;
		if ( bHasWeaponData )
		{
			Asura_Hash_ID uWeaponHash;
			xStream >> uWeaponHash;

			AVP_Human_PlayerAnimStateData::AVP_Human_PlayerAnimWeaponStateData* pxWeaponData = AVP_Human_PlayerAnimStateData::GetSingleton().AddNewWeaponData( uWeaponHash );
			Asura_Assertf( pxWeaponData != NULL, ( "Unable to add new weapon data for hash ID 0x%08x", uWeaponHash ) );

			u_int uNumHud;
			xStream >> uNumHud;
			Asura_Assert( uNumHud == 2, "Chunk data inconsistent with code, does the common or character data need re-exporting?" );

			u_int uNumStates;
			xStream >> uNumStates;
			Asura_Assert( uNumStates == AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_END_NOAIM, "Chunk data inconsistent with code, does the common or character data need re-exporting?" );

			u_int uNumSpeeds;
			xStream >> uNumSpeeds;
			Asura_Assert( uNumSpeeds == AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_END, "Chunk data inconsistent with code, does the common or character data need re-exporting?" );

			for ( u_int uHudItr = 0; uHudItr < uNumHud; ++uHudItr )
			{
				for ( u_int uStateItr = 0; uStateItr < uNumStates; ++uStateItr )
				{
					for ( u_int uSpeedItr = 0; uSpeedItr < uNumSpeeds; ++uSpeedItr )
					{
						bool bHasStateData;
						xStream >> bHasStateData;
						AVP_Actor_AnimStateData* pxStateData = NULL;
						if ( bHasStateData )
						{
							pxStateData = AVP_Actor_AnimStateData::CreateFromChunkStream( xStream );
						}

						if ( uHudItr < 2 && uStateItr < AVP_Vertebrate_PlayerAnimStateHandler::MOVESTATE_END_NOAIM && uSpeedItr < AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_END && pxWeaponData != NULL )
						{
							pxWeaponData->m_aaapxAnimStateData[ uHudItr ][ uStateItr ][ uSpeedItr ] = pxStateData;
						}
					}
				}
			}
		}
	}
}
