
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Connection.cpp
// Author : Anthony Gregson
// Date   : 21/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Core/AVP_Connection.h"

// Asura independent header-files...
#include "Network/Asura_Network.h"
#include "Network/Online/Asura_Lobby.h"

// Game independent header-files...
#include "Core/States/Connection/AVP_ConnectionState_BeginGameInitialisation.h"
#include "Core/States/Connection/AVP_ConnectionState_EndGameInitialisation.h"
#include "Core/States/Connection/AVP_ConnectionState_WaitForPlayers.h"
#include "Core/States/Connection/AVP_ConnectionState_Loading.h"
#include "Core/States/Connection/AVP_ConnectionState_MapChange.h"
#include "Core/States/Connection/AVP_ConnectionState_Playing.h"
#include "Core/States/Connection/AVP_ConnectionState_Terminated.h"
#include "Core/States/Connection/AVP_ConnectionState_WaitForGuid.h"
#include "Network/AVP_Server.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_Connection::AVP_Connection
// Author	: Anthony Gregson
// Date		: 21/11/07
////////////////////////////////////////////////////////////////////////////////

AVP_Connection::AVP_Connection(void)
: AVP_StateMachine(AVP_Connection::MAX_STATES)
, m_iNetworkDisconnectReason(ASURA_NDR_NOTDISCONNECTED)
{
	RegisterState(STATE_WAITFORPLAYERS, new AVP_ConnectionState_WaitForPlayers(*this));
	RegisterState(STATE_LOADING, new AVP_ConnectionState_Loading(*this));
	RegisterState(STATE_WAITFORGUID, new AVP_ConnectionState_WaitForGuid(*this));
	RegisterState(STATE_BEGINGAMEINITIALISATION, new AVP_ConnectionState_BeginGameInitialisation(*this));
	RegisterState(STATE_ENDGAMEINITIALISATION, new AVP_ConnectionState_EndGameInitialisation(*this));
	RegisterState(STATE_PLAYING, new AVP_ConnectionState_Playing(*this));
	RegisterState(STATE_MAPCHANGE, new AVP_ConnectionState_MapChange(*this));
	RegisterState(STATE_TERMINATED, new AVP_ConnectionState_Terminated(*this));
}

/// End of function 'AVP_Connection::AVP_Connection'. ///


void AVP_Connection::Initialise()
{
	Asura_Assert(m_uCurrentState == STATE_INVALID, "Unexpected initial state for AVP_Connection");

	m_iNetworkDisconnectReason = ASURA_NDR_NOTDISCONNECTED;
#ifdef ASURA_SUPPORT_LOBBY
	if (Asura_Lobby_System::IsInitialised())
	{
		ChangeState(STATE_WAITFORPLAYERS);
	}
	else
#endif
	{
		ChangeState(STATE_LOADING);
	}
}

void AVP_Connection::Shutdown()
{
	Asura_Assert(m_uCurrentState == STATE_TERMINATED, "Unexpected final state for AVP_Connection");

	ChangeState(STATE_INVALID);
	// Force an update so that this state gets processed.
	Update();
}

// ValidateNetworkConnection() returns true if everything is OK.
// Note that this does actually terminate things if there's a problem, so there's no need to terminate things yourself if this returns false.
bool AVP_Connection::ValidateNetworkConnection()
{
	if (!Asura_Network::IsNetworkInitialised())
	{
		Terminate(ASURA_NDR_UNKNOWN);
		return false;
	}
	
	if (Asura_Network::GetReasonForDisconnection() != ASURA_NDR_NOTDISCONNECTED)
	{
		Terminate(Asura_Network::GetReasonForDisconnection());
		return false;
	}

#ifdef ASURA_SUPPORT_LOBBY
 	if (Asura_Lobby_System::IsInitialised() && (Asura_Lobby_System::GetErrorCode() != ASURA_RS_DISCONNECT_NOT_DISCONNECTED))
 	{
 		Terminate(Asura_Lobby_System::GetErrorCode());
 		return false;
 	}
#endif

	return true;
}

void AVP_Connection::Terminate(int iNetworkDisconnectReason)
{
	m_iNetworkDisconnectReason = iNetworkDisconnectReason;
	ChangeState(STATE_TERMINATED);
}

/// End of file. ///
