//////////////////////////////////////////////////////////////////////////////
//
// AVP_Core.h
//
// Copyright (c) 2005 Rebellion. All rights reserved.
//
// Description:
//
//     basic shell to handle switching between Game and Tools
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___AVP_CORE_H___
#define ___AVP_CORE_H___ 1

//////////////////////////////////////////////////////////////////////////
// I N C L U D E   F I L E S /////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
#if defined(ASURA_SUPPORT_FLYCAM)
#include "Camera/Asura_FlyCam.h"
#endif	// defined(ASURA_SUPPORT_FLYCAM)
#include "Core/Asura_Handle.h"
#include "Core/Asura_PlatformBasics.h"	// for Asura_UnicodeChar
#include "Core/Asura_Timers.h"
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
#include "Core/AVP_SmokeTest.h"
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
#include "Rendering/Asura_Render.h"

#include "Core/AVP_StateMachine_Core.h"
#ifdef AVP_USE_DEVELOPMENT_FRONT_END
#include "FrontEnd/AVP_DevelopmentFrontEnd.h" 
#endif //#ifdef AVP_USE_DEVELOPMENT_FRONT_END
#ifdef ASURA_USE_STREAMING
#include "Streaming/Asura_Streaming_Zone.h"
#endif //#ifdef ASURA_USE_STREAMING

#define fAVP_COLLISION_SFX_SPEED_THRESHOLD 1.0f

//////////////////////////////////////////////////////////////////////////
// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////
//////////////////////////////////////////////////////////////////////////
class Asura_Collision_Object;
class Asura_ProtoSaveBlock;
#ifdef ASURA_RESOURCE_ALLOCATOR_USES_MEMORY_BANK
class Asura_MemoryBank;
#endif //#ifdef ASURA_RESOURCE_ALLOCATOR_USES_MEMORY_BANK

class AVP_Player;

//////////////////////////////////////////////////////////////////////////
// C L A S S   D E C L A R A T I O N S ///////////////////////////////////
//////////////////////////////////////////////////////////////////////////
class AVP_Core
{
public:
	// Public enumerated types...
	enum FontType
	{
		HUD_FONT,
		MENU_FONT,
#if defined(ASURA_USE_DEBUG_MESSAGE_TRIGGER)
		DEBUG_MESSAGE_FONT,
#endif	// defined(ASURA_USE_DEBUG_MESSAGE_TRIGGER)
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
		COMMAND_CONSOLE_FONT,
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)

		MAX_FONT_TYPES
	};

	// Core functions...
	static void	EarlyInitialise(void);
	static void	LateInitialise(void);
	static void Reset(void);
	static void	Shutdown(void);

	static bool	MainLoop(void);

#if defined(ASURA_TOOLS)
	// Tools functions...
	static void	SetToolsRequested(bool bRequest)	{ s_bToolsRequested = bRequest; }
	static bool	GetToolsRequested(void)				{ return s_bToolsRequested; }
	static bool	IsInTools(void)						{ return((s_xStateMachine.GetCurrentState() == AVP_StateMachine_Core::STATE_TOOLS) || (s_xStateMachine.GetCurrentState() == AVP_StateMachine_Core::STATE_LOADINGTOOLS)); }

	static bool ToolsQuickSave(void);
	static bool ToolsQuickLoad(void);
#else
	static void SetDesiredScreenSize(const u_int uWidth, const u_int uHeight)	{ s_uDesiredScreenWidth = uWidth; s_uDesiredScreenHeight = uHeight; }
#endif	// defined(ASURA_TOOLS)

	// Network functions...
	static void	InitialiseNetwork(const bool bExternalNetwork);
	static void	ShutdownNetwork(void);

	// We used to shutdown and initialise these effects in the network functions, so I'm separating them out into their own function.
	static void InitialiseEffects();
	static void ShutdownEffects();

	// Load function...
	static bool LoadFile(const char* szFilename, const int iTextureResetGroup = -1);

	// Update functions...
	static void Update(const bool bUpdateScene, AVP_Player* pxPlayer);

	// Rendering functions...
	static void	BeginRender(const bool bRenderEnvironment);
	static void Render(AVP_Player* pxPlayer);
	static void	EndRender(void);
	
	//For use as the callback when taking hi-res screenshots. RAR 03/07/2007
	static void RenderForScreenshot();

	// State change requests. Possibly should be deprecated.
	static void	RequestLevelComplete(void);
	static void	RequestQuit(void);
	static void	RequestRestartAtCheckPoint(void);
	static void	RequestRestartLevel(void);
	static void	RequestGameExit( void );
	static void	RequestDisconnecting(void);
	static void	RequestGameOver(void);

	static u_int GetState(void) { return s_xStateMachine.GetCurrentState(); }

	// state query functions
	static bool IsInGameState(void)					{ return( s_xStateMachine.GetCurrentState() == AVP_StateMachine_Core::STATE_GAME ); }

	// Font accessor functions...
	static bool			HasFont(const FontType eType)		{ return(s_aiFontHandles[eType] != ASURA_INVALID_HANDLE); }
	static Asura_Handle	GetFontHandle(const FontType eType)	{ return(s_aiFontHandles[eType]); }

	static bool IsGamePaused(void)					{ return Asura_Timers::GetGameTimeScale() <= 0.0f; }

	static void SetShouldUpdateSoundSystem( bool bEnable )  { s_bShouldUpdateSoundSystem = bEnable; }

	static bool ShouldRenderHUDToFrontBuffer() { return g_bRenderHUDToFrontBuffer; }

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	static void ClearFileLoadMemoryUseLogfile();
	static int GetAvailableMemory();
#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT
	static bool GetDLCRequiresFilesUpdate() { return s_bDLCRequiresFilesUpdate; }
	static void SetDLCRequiresFilesUpdate(bool bUpdateRequired) { s_bDLCRequiresFilesUpdate = bUpdateRequired; }
#endif //#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT

	// On PS3 this causes us to switch to the frontbuffer so that the UI is at final resolution
	static void						Platform_BeginHUDRender(void);


private:
	// Private static data...
	static const char*						s_aszFontNames[MAX_FONT_TYPES];		// Array of names for the fonts to use.
	static Asura_Handle						s_aiFontHandles[MAX_FONT_TYPES];	// Array of handles for the fonts to use.

	static AVP_StateMachine_Core			s_xStateMachine;					// Static class used to control the AvP state-machine.
	static bool								s_bShouldUpdateSoundSystem;

	static bool 	                        s_bShowDialogue;
	static Asura_UnicodeChar                s_aswzDialogue[256]; 

#if defined(ASURA_TOOLS)
	static bool								s_bToolsRequested;
	static Asura_ProtoSaveBlock*			s_pxToolsQuickSaveBlock;
#else
	static u_int							s_uDesiredScreenWidth;
	static u_int							s_uDesiredScreenHeight;
	static Asura_Render::ScreenBorderType	s_eScreenBorderType;
#endif	// defined(ASURA_TOOLS)

#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT
	static bool s_bDLCRequiresFilesUpdate;
#endif //#ifdef ASURA_SUPPORT_DOWNLOADABLE_CONTENT

	// Private functions...
	static const Asura_Collision_Object* GetPhysicalObjectCollisionObject(int iIndex, Asura_Hash_ID uShapeHash, Asura_Guid uGuid);
	static void RenderDialogueSubtitles(const Asura_UnicodeChar* wszText);
	static void RenderEnvironmentAndEntities(void);

	// Platform-specific functions...
	static const Asura_UnicodeChar*	Platform_GetLevelFileName(void);

	static void						Platform_Initialise(void);
	static void						Platform_Shutdown(void);

	static void						Platform_Update();

	static bool ConsoleCmd_DialogueList( u_int uElementIndex, bool bInterrogate, bool bHelp );


// Create console vars
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	static void CreateConsoleVars();
    static void CreateConsoleCommands();
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)	

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	static u_int Platform_GetAvailableMemory();
#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

	static bool g_bRenderHUDToFrontBuffer;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___AVP_CORE_H___
