//////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_GameStatus.cpp
// Author : Anthony Gregson
// Date   : 04/12/07
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
// The companion to AVP_Multiplayer_GameRules, this is the storage for the variable data
// in a single AVP game. This, the rules, and the PlayerInfo data are used by
// the Playing states to determine whether the game should be finishing or
// not, and potentially for making various other decisions about what needs
// to update (respawn waves being one possible example). Only the server
// should be making any direct decisions on these details, the clients
// updating by message (although that isn't quite true in the initial version,
// as we're storing quit requests here).
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Core/AVP_GameStatus.h"

// Asura
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Guid.h"

// Project
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Messages.h"

// S T A T I C   C L A S S   D A T A /////////////////////////////////////////

bool AVP_GameStatus::s_bGameOver = false;
bool AVP_GameStatus::s_bLevelComplete = false;
bool AVP_GameStatus::s_bQuitApplication = false;
bool AVP_GameStatus::s_bExitGame = false;
bool AVP_GameStatus::s_bLoadCheckpoint = false;
bool AVP_GameStatus::s_bRestoreCheckpoint = false;
bool AVP_GameStatus::s_bRestartLevel = false;
bool AVP_GameStatus::s_bLoadMap = false;

// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////

void AVP_GameStatus::ResetStatusChanges()
{
	s_bGameOver = false;
	s_bLevelComplete = false;
	s_bQuitApplication = false;
	s_bExitGame = false;
	s_bLoadCheckpoint = false;
	s_bRestoreCheckpoint = false;
	s_bRestartLevel = false;
	s_bLoadMap = false; 
}

void AVP_GameStatus::SendGameStatusUpdateMessage()
{
	if (AVP_Settings::IsSinglePlayer() || !AVP_Settings::GetIsServer()) { return; }

	AVP_Msg_GameStatus_FullUpdate xData;
	xData.SetGameOver(s_bGameOver);
	xData.SetLevelComplete(s_bLevelComplete);
	xData.SetRestoreCheckpoint(s_bRestoreCheckpoint);
	xData.SetRestartLevel(s_bRestartLevel);
	xData.SetLoadMap(s_bLoadMap); 

	Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_GAMESTATUS_FULLUPDATE, sizeof(AVP_Msg_GameStatus_FullUpdate), AsuraEntityClass_NotSet, &xData);
	if (pxMessage)
	{
		pxMessage->To = ASURA_GUID_ALL_OTHER_CLIENTMACHINES;
		Asura_Network::Send(pxMessage);
	}
}

bool AVP_GameStatus::HandleMessage(const Asura_Message* pxMessage)
{
	if (!pxMessage) { return false; }

	switch (pxMessage->GetMsgID())
	{
	case AVP_MSGID_GAMESTATUS_FULLUPDATE:
		{
			const AVP_Msg_GameStatus_FullUpdate* pxData = static_cast<const AVP_Msg_GameStatus_FullUpdate*>(pxMessage->GetData());
			if (pxData)
			{
				s_bGameOver = pxData->GetGameOver();
				s_bLevelComplete = pxData->GetLevelComplete();
				s_bRestoreCheckpoint = pxData->GetRestoreCheckpoint();
				s_bRestartLevel = pxData->GetRestartLevel();
				s_bLoadMap = pxData->GetLoadMap(); 
			}
		}
		return true;

	default:
		return false;
	}
}

// end of file
