//////////////////////////////////////////////////////////////////////////////
//
// AVP_Interpolator.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Core/AVP_Interpolator.h"

void AVP_Angle_Interpolator::Interpolate( const Asura_Angle& xFromVal, const Asura_Angle& xToVal, const float fFromTime, const float fToTime, const float fCurrentTime, Asura_Angle& xOutVal ) const
{
	const float fDiff = xToVal - xFromVal;
	float fTo = xToVal;
	if ( fDiff < -Asura_Maths::Pi )
	{
		fTo += 2.0f * Asura_Maths::Pi;
	}
	else if ( fDiff > Asura_Maths::Pi )
	{
		fTo -= 2.0f * Asura_Maths::Pi;
	}
	float fParametric = Asura_Maths::ClampToRange( ( fCurrentTime - fToTime ) / ( fToTime - fFromTime ), 0.0f, 1.0f );
	xOutVal = xFromVal * ( 1.0f - fParametric );
	xOutVal = xOutVal + ( fTo * fParametric );
}

void AVP_Direction_Interpolator::Interpolate( const Asura_AVector3& xFromVal, const Asura_AVector3& xToVal, const float fFromTime, const float fToTime, const float fCurrentTime, Asura_AVector3& xOutVal ) const
{
	// This could be moved into Asura_AVector3::Slerp as it's becoming a common thing to do. (Also used in Hierarchy_Tween::BlendVelocity with a magnitude lerp as well)
	Asura_AVector3 xRet;
	Asura_Quat xRot;
	Asura_AVector3 xAxis;
	xRot.RotationFromAToB( xFromVal, xToVal );
	float fAngle = xRot.ToRotation( xAxis );
	fAngle *= Asura_Maths::ClampToRange( ( fCurrentTime - fToTime ) / ( fToTime - fFromTime ), 0.0f, 1.0f );
	xRot.FromRotation( fAngle, xAxis );
	xOutVal = xRot.RotateVector( xFromVal);
}
