
///////////////////////////////////////////////////////////////////////////////
//
// File		: AVP_Maths.h
// Author	: Paul Humphries
// Date		: 2-Mar-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_MATHS_H__
#define __AVP_MATHS_H__

// Asura files
#include "Core/Asura.h"
#include "Core/Asura_Maths.h"
#include "Network/Asura_Guid.h"

// AVP_Maths class
class AVP_Maths
{
public:
	static bool		ForceMajorAxisPositive(Asura_AVector3_InParam xInput, Asura_AVector3_OutParam xOutput);

	static void		ExtractTurnFrom	(const Asura_Matrix_3x3& xOrientation, Asura_Matrix_3x3& xResult);
	static void		ExtractTiltFrom	(const Asura_Matrix_3x3& xOrientation, Asura_Matrix_3x3& xResult);
	static float	LOSEverything	(const Asura_Vector_3& xStart, const Asura_Vector_3& xNonNormalisedDirection, float fMaximumDistance, Asura_Guid uGuidToIgnore);
	static float	LOSEnvironment	(const Asura_Vector_3& xStart, const Asura_Vector_3& xNonNormalisedDirection, float fMaximumDistance);
	static void		MirrorYZ		(const Asura_Vector_3& xInput, Asura_Vector_3& xOutput);
	static void		MirrorYZ		(const Asura_Euler_3& xInput, Asura_Euler_3& xOutput);

	static void		TurnLinearlyTowards		(Asura_Euler_3& xCurrent, const Asura_Euler_3& xTarget, Asura_Angle fAngularVelocity);
	static void		TurnExponentiallyTowards(Asura_Euler_3& xCurrent, const Asura_Euler_3& xTarget, Asura_Angle fInitialAngularVelocity);

	static void		LinearlyTowards			(float& fCurrent, const float fTarget, const float fSpeed);
	static void		ExponentiallyTowards	(float& fCurrent, const float fTarget, const float fInitialSpeed);

	static void		RayOffsetFromPoint( const Asura_Vector_3& xRayOrigin, const Asura_Vector_3& xRayDirection, const Asura_Vector_3& xPoint, Asura_Vector_3& xOffset );

	static bool		RayIntersectsOBB( const Asura_AVector3& xRayOrigin, const Asura_AVector3& xRayDirection, const Asura_OBB& xOBB, float* pfDistance = NULL, Asura_AVector3* pxHitPosition = NULL, bool* pbInside = NULL, Asura_AVector3* pxNormal = NULL );

	// for angles in radians
	static bool		IsAngleInRange( const float fAngle, float fACWBound, float fCWBound, float* pfLerpParametric = NULL );

	static void		FindClosestPointOnEdge( Asura_AVector3_InParam xPoint, Asura_AVector3_InParam xV0, Asura_AVector3_InParam xV1, Asura_AVector3_OutParam xClosestPointOnEdge );

	static Asura_Vector_3 OffsetDirectionInCone(Asura_Vector_3& xOriginalDirection, Asura_Vector_2 xNewDirection, const float fMaxTheta);

#ifndef ASURA_NO_DEBUG_OUTPUT
	static void		DrawLine		(const Asura_Vector_3& xStart, const Asura_Vector_3& xNonNormalisedDirection, float fLength, u_int uColour = 0xffffffff);
	static void		CreateLine		(const Asura_Vector_3& xStart, const Asura_Vector_3& xNonNormalisedDirection, float fLength, u_int uColour = 0xffffffff, float fDuration = 0.0f, float fFadeDuration = 60.0f);
	static void		CreateArrow		(const Asura_Vector_3& xStart, const Asura_Vector_3& xEnd, u_int uColour = 0xffffffff, float fDuration = 3600.0f, float fFadeDuration = 60.0f);
#endif // ASURA_NO_DEBUG_OUTPUT

};   // End of class 'AVP_Maths'.

#endif   // ifndef __AVP_MATHS_H__


/// End of file. ///

