
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_StateMachine_Playing.cpp
// Author : Anthony Gregson
// Date   : 03/12/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Core/AVP_StateMachine_Playing.h"

// Asura independent header-files...

// Game independent header-files...
#include "Core/AVP_Settings.h"
#include "Core/States/Playing/AVP_PlayingState_GameOver.h"
#include "Core/States/Playing/AVP_PlayingState_InProgress.h"
#include "Core/States/Playing/AVP_PlayingState_Starting.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_StateMachine_Playing::AVP_StateMachine_Playing
// Author	: Anthony Gregson
// Date		: 03/12/07
////////////////////////////////////////////////////////////////////////////////

AVP_StateMachine_Playing::AVP_StateMachine_Playing(void)
: AVP_StateMachine(AVP_StateMachine_Playing::MAX_STATES)
{
	RegisterState(STATE_STARTING, new AVP_PlayingState_Starting(*this));
	RegisterState(STATE_INPROGRESS, new AVP_PlayingState_InProgress(*this));
	RegisterState(STATE_GAMEOVER, new AVP_PlayingState_GameOver(*this));
}

/// End of function 'AVP_StateMachine_Playing::AVP_StateMachine_Playing'. ///


void AVP_StateMachine_Playing::Initialise()
{
	Asura_Assert(m_uCurrentState == STATE_INACTIVE, "Unexpected initial state for AVP_StateMachine_Playing");

	// Only the server makes direct decisions about the playing state, with the exception of when shutting down.
	// The starting state is multiplayer-specific - potentially for some kind of warm-up phase - and isn't properly set up at this time.
	// Potentially, it (or some other state) could be used for some single-player introduction screen too, but I'm not sure if it'll be easier to set up the logic for that elsewhere.
	// JG - 17 October 2008 - To use the STATE_GAMEOVER in clients we need to go through this state. It should be fine as long as the code in this state doesn't make
	// any decissions about its own state in client machines
	ChangeState(STATE_INPROGRESS);
}

void AVP_StateMachine_Playing::Shutdown()
{
	// This can be forced from a variety of playing states, so just get out of the current state.
	ChangeState(STATE_INACTIVE);
	// Force an update so that this state gets processed.
	Update();
}

/// End of file. ///
