
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_surfaceEffects.cpp
// Author : Chris Wallace
// Date   : 2-Aug-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Core/AVP_SurfaceEffects.h"

#include "Animation/Asura_Animation.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Input/AVP_Rumble_System.h"
#include "SpecialFX/Asura_Decal_Manager.h"
#include "SpecialFX/pfx/Asura_PFX_Entity_Effect.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "Sound/Asura_Sound_Event_System.h"

#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/AVP_ClientEntity_PhysicalObject.h"
#include "SpecialFX/pfx/AVP_DamagePFX.h"

////////////////////////////////////////////////////////////////////////////////
// Static variables...
////////////////////////////////////////////////////////////////////////////////

Asura_TemplateHashTree	<AVP_SurfaceEffect_DamageMap>		AVP_SurfaceEffect_System::s_xDamageTableList;
Asura_TemplateHashTree	<AVP_SurfaceEffect_CollisionMap>	AVP_SurfaceEffect_System::s_xCollisionTableList;
Asura_TemplateHashTree	<AVP_SurfaceEffect_EffectTypeMap>	AVP_SurfaceEffect_System::s_xEffectTypeTableList;

////////////////////////////////////////////////////////////////////////////////
// Class functions...
////////////////////////////////////////////////////////////////////////////////

void AVP_SurfaceEffect_System::PlayDamageSurfaceEffect( Asura_Hash_ID uSurfaceHash,const AVP_EntityDamage_ApplyToEntity& xDamage, Asura_Guid uTargetGuid)
{
	AVP_SurfaceEffect_Data xData;

	Asura_Vector_3	xPosition;
	Asura_Vector_3	xNormal;
	xDamage.GetPointOfImpact(xPosition);
	xDamage.GetNormal(xNormal);

	if (GetDamageEffectData(uSurfaceHash, xDamage.GetTypeHash(), xData))
	{
		PlayEffect( xData, xPosition, xNormal, 1.0f, uTargetGuid, xDamage.GetBoneID() );
	}
}


void AVP_SurfaceEffect_System::PlayDamageSurfaceEffect( Asura_Hash_ID uSurfaceHash, Asura_Hash_ID uDamageTypeHash, const Asura_Vector_3& xPosition, const Asura_Vector_3& xNormal, const float fVolumeScale /*= 1.0f */, Asura_Guid uTargetGuid /* = ASURA_GUID_UNREGISTERED*/ )
{
	AVP_SurfaceEffect_Data xData;
	if (GetDamageEffectData(uSurfaceHash, uDamageTypeHash, xData))
	{
		PlayEffect( xData, xPosition, xNormal, fVolumeScale, uTargetGuid );
	}
}

void AVP_SurfaceEffect_System::PlayCollisionSurfaceEffect( Asura_Hash_ID uCollider, Asura_Hash_ID uCollidee, const Asura_Vector_3& xPosition, const Asura_Vector_3& xNormal, const float fVolumeScale /*= 1.0f */, Asura_Guid uTargetGuid /*= ASURA_GUID_UNREGISTERED*/ )
{
	AVP_SurfaceEffect_Data xData;
	if (GetCollisionEffectData(uCollider, uCollidee, xData))
	{
		PlayEffect( xData, xPosition, xNormal, fVolumeScale, uTargetGuid );
	}
}

void AVP_SurfaceEffect_System::PlayEffectTypeSurfaceEffect( Asura_Hash_ID uSurfaceHash, Asura_Hash_ID uEffectTypeHash, const Asura_Vector_3& xPosition, const Asura_Vector_3& xNormal, const float fVolumeScale /*= 1.0f */, Asura_Guid uTargetGuid /*= ASURA_GUID_UNREGISTERED*/ )
{
	AVP_SurfaceEffect_Data xData;
	if (GetEffectTypeEffectData(uSurfaceHash, uEffectTypeHash, xData))
	{
		PlayEffect( xData, xPosition, xNormal, fVolumeScale, uTargetGuid );
	}
}

bool AVP_SurfaceEffect_System::GetDamageEffectData(Asura_Hash_ID uSurfaceHash, Asura_Hash_ID uDamageTypeHash, AVP_SurfaceEffect_Data& xData)
{
	Asura_Hash_ID uEffectHash = GetDamageSurfaceEffect(uSurfaceHash, uDamageTypeHash);
	if (uEffectHash == ASURA_HASH_ID_UNSET) { return false; }

	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_SURFACEEFFECTS, uEffectHash);
	if (!pxBlueprint) { return false; }

	GetEffectData(*pxBlueprint, xData);

	return true;
}

bool AVP_SurfaceEffect_System::GetCollisionEffectData(Asura_Hash_ID uCollider, Asura_Hash_ID uCollidee, AVP_SurfaceEffect_Data& xData)
{
	Asura_Hash_ID uEffectHash = GetCollisionSurfaceEffect(uCollider, uCollidee);
	if (uEffectHash == ASURA_HASH_ID_UNSET) { return false; }

	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_SURFACEEFFECTS, uEffectHash);
	if (!pxBlueprint) { return false; }

	GetEffectData(*pxBlueprint, xData);

	return true;
}

bool AVP_SurfaceEffect_System::GetEffectTypeEffectData(Asura_Hash_ID uSurfaceHash, Asura_Hash_ID uEffectTypeHash, AVP_SurfaceEffect_Data& xData)
{
	Asura_Hash_ID uEffectHash = GetEffectTypeSurfaceEffect(uSurfaceHash, uEffectTypeHash);
	if (uEffectHash == ASURA_HASH_ID_UNSET) { return false; }

	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_SURFACEEFFECTS, uEffectHash);
	if (!pxBlueprint) { return false; }

	GetEffectData(*pxBlueprint, xData);

	return true;
}

void AVP_SurfaceEffect_System::PlayEffect( const AVP_SurfaceEffect_Data& xData, const Asura_Vector_3& xPosition, const Asura_Vector_3& xNormal, const float fVolumeScale /*= 1.0f */, Asura_Guid uTargetGuid /*= ASURA_GUID_UNREGISTERED*/ , int iBoneID)
{
	// DECALS
	// We dont play decals on objects as thay may move and the decal wont!
	if (uTargetGuid == ASURA_GUID_UNREGISTERED)
	{
		if ( xData.m_uDecalHash != ASURA_HASH_ID_UNSET )
		{
			Asura_Decal_Manager::AddDecal( xPosition, xNormal, xData.m_uDecalHash );
		}
	}

	// PFX
	for ( u_int uPFXItr = 0; uPFXItr < xData.m_uNumPFX; ++uPFXItr )
	{
		if ( xData.m_auPFXHash[ uPFXItr ] != ASURA_HASH_ID_UNSET )
		{
			u_int uPFXHandle;

			if (uTargetGuid == ASURA_GUID_UNREGISTERED)
			{
				uPFXHandle = Asura_PFX_Entity_Effect::Create( xPosition, xNormal, xData.m_auPFXHash[ uPFXItr ] );
			} 
			else
			{
				uPFXHandle = CreateDamageEntityPFXEffect(xPosition, xNormal, xData.m_auPFXHash[ uPFXItr ], uTargetGuid, iBoneID);
			}
			// Damage PFX
			if ((Asura_Network::IsServer()) && (uPFXItr == 0) && (xData.m_fDamage !=0.f) ) // PFXDamage only done on first pfx and only on server.
			{
				AVP_DamagePFX::AddDamagePFX(uPFXHandle, xData.m_fDamageDelay,xData.m_fDamageInterval,xData.m_fLifetime, xData.m_fDamage, xData.m_fPFXBBScaling, xData.m_uDamageType);
			}
		}
	}

	// SOUND
	if ( xData.m_uSoundHash != ASURA_HASH_ID_UNSET )
	{
		const Asura_Sound_Event* pxSE = Asura_Sound_Event_Storage::FindEvent( xData.m_uSoundHash );
		if ( pxSE != NULL )
		{
			Asura_Sound_Event_System::PlayEvent( pxSE, NULL, &xPosition, 1.0f , uTargetGuid );
		}
	}

	// VIBRATE
	if ( AVP_Settings::GetCurrentProfile()->GetControls().GetVibrate() )
	{
		if ( uTargetGuid != ASURA_GUID_UNREGISTERED )
		{
			const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( uTargetGuid );
			if ( pxActor && AVP_Game_Core::IsLocalPlayer( uTargetGuid ) )
			{
				if( xData.m_uVibration != ASURA_HASH_ID_UNSET )
				{
					AVP_Rumble_System::AddHUDRumble( xData.m_uVibration );
				}
			}
		}
	}

}

void AVP_SurfaceEffect_System::GetEffectData(const Asura_Blueprint& xBlueprint, AVP_SurfaceEffect_Data& xData)
{
	Asura_Value xValue;
	if(xBlueprint.GetParameterValueByHashID(0x05b097ad, xValue))	// Decal
	{
		xData.m_uDecalHash = xValue.GetHashIDValue();
	}
	else
	{
		xData.m_uDecalHash = ASURA_HASH_ID_UNSET;
	}

	AVP_SurfaceEffect_PFXEnumerator xEnum( xData.m_auPFXHash );
	xData.m_uNumPFX = xBlueprint.EnumerateParameters( "PFX.", 2, xEnum );
	if ( xData.m_uNumPFX >= AVP_MAX_SURFACE_EFFECT_PFX )
	{
		xData.m_uNumPFX = AVP_MAX_SURFACE_EFFECT_PFX;
	}
	
	if(xBlueprint.GetParameterValueByHashID(0x0688c90f, xValue))	// Sound
	{
		xData.m_uSoundHash = xValue.GetHashIDValue();
	}
	else
	{
		xData.m_uSoundHash = ASURA_HASH_ID_UNSET;
	}

	if(xBlueprint.GetParameterValueByHashID(0xFB1EF292, xValue))	// Vibration
	{
		xData.m_uVibration = xValue.GetHashIDValue();
		if ( xData.m_uVibration != ASURA_HASH_ID_UNSET  )
		{
			AVP_Rumble_System::InitFromBlueprint( xData.m_uVibration );
		}
	}

	// Damage PFX values.

	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_PFXDAMAGE_TYPE, xValue))
	{
		xData.m_uDamageType= xValue.GetHashIDValue();
	}	
	else
	{
		Asura_Assert(false, "Warning PFXDamage.Acidic not found in surface effect blueprint");
	}

	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_PFXDAMAGE_DAMAGEINTERVAL, xValue))
	{
		xData.m_fDamageInterval= xValue.GetFloatValue();
	} 
	else
	{
		Asura_Assert(false, "Warning PFXDamage.DamageInterval not found in surface effect  blueprint");
	}

	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_PFXDAMAGE_INITIALDELAY, xValue))
	{
		xData.m_fDamageDelay  = xValue.GetFloatValue();
	} 
	else
	{
		Asura_Assert(false, "Warning PFXDamage.InitialDelay not found in surface effect blueprint");
	}

	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_PFXDAMAGE_LIFETIME, xValue))
	{
		xData.m_fLifetime = xValue.GetFloatValue();
	} 
	else
	{
		Asura_Assert(false, "Warning PFXDamage.Lifetime not found in surface effect blueprint");
	}

	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_PFXDAMAGE_DAMAGE, xValue))
	{
		xData.m_fDamage = xValue.GetFloatValue();
	} 
	else
	{
		Asura_Assert(false, "Warning PFXDamage.Damage not found in surface effect blueprint");
	}

	if(xBlueprint.GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_PFXDAMAGE_SCALE, xValue))
	{
		xData.m_fPFXBBScaling = xValue.GetFloatValue();
	} 
	else
	{
		Asura_Assert(false, "Warning PFXDamage.Scale not found in surface effect blueprint");
	}
	
}

Asura_Hash_ID AVP_SurfaceEffect_System::GetDamageSurfaceEffect(Asura_Hash_ID uSurfaceHash, Asura_Hash_ID uDamageTypeHash)
{
	//Search our linked lists for the required effect
	AVP_SurfaceEffect_DamageMap*	pxDamageMap		= s_xDamageTableList.GetDataByHash(uSurfaceHash);
	if(pxDamageMap)
	{
		AVP_SurfaceEffect_Map*	pxSurfaceEffectMap = pxDamageMap->GetList()->GetDataByHash(uDamageTypeHash);
		if(pxSurfaceEffectMap)
		{
			return(pxSurfaceEffectMap->GetHash());
		}
	}

	return(NULL);
}

Asura_Hash_ID AVP_SurfaceEffect_System::GetCollisionSurfaceEffect(Asura_Hash_ID uCollider, Asura_Hash_ID uCollidee)
{
	//Search our linked lists for the required effect
	AVP_SurfaceEffect_CollisionMap*	pxCollisionMap		= s_xCollisionTableList.GetDataByHash(uCollider);
	if(pxCollisionMap)
	{
		AVP_SurfaceEffect_Map*	pxSurfaceEffectMap = pxCollisionMap->GetList()->GetDataByHash(uCollidee);
		if(pxSurfaceEffectMap)
		{
			return(pxSurfaceEffectMap->GetHash());
		}
	}

	return(NULL);
}

Asura_Hash_ID AVP_SurfaceEffect_System::GetEffectTypeSurfaceEffect(Asura_Hash_ID uSurfaceHash, Asura_Hash_ID uEffectTypeHash)
{
	//Search our linked lists for the required effect
	AVP_SurfaceEffect_EffectTypeMap*	pxEffectTypeMap	= s_xEffectTypeTableList.GetDataByHash(uSurfaceHash);
	if(pxEffectTypeMap)
	{
		AVP_SurfaceEffect_Map*	pxSurfaceEffectMap = pxEffectTypeMap->GetList()->GetDataByHash(uEffectTypeHash);
		if(pxSurfaceEffectMap)
		{
			return(pxSurfaceEffectMap->GetHash());
		}
	}

	return(NULL);
}

AVP_SurfaceEffect_Map* AVP_SurfaceEffect_System::GetDamageMap(Asura_Hash_ID uSurfaceHash, Asura_Hash_ID uDamageHash, bool bAddOnFail /*= false*/)
{
	AVP_SurfaceEffect_DamageMap*	pxDamageMap = NULL;		
	pxDamageMap = s_xDamageTableList.GetDataByHash(uSurfaceHash);
	if(!pxDamageMap && bAddOnFail)
	{
		pxDamageMap		= s_xDamageTableList.AddNode(uSurfaceHash);
	}

	if(pxDamageMap)
	{
		AVP_SurfaceEffect_Map*	pxSurfaceEffect = NULL;
		pxSurfaceEffect = pxDamageMap->GetList()->GetDataByHash(uDamageHash);
		if(!pxSurfaceEffect && bAddOnFail)
		{
			pxSurfaceEffect = pxDamageMap->GetList()->AddNode(uDamageHash);
		}

		if(pxSurfaceEffect)
		{
			return(pxSurfaceEffect);
		}	
	}
	return(NULL);
}


AVP_SurfaceEffect_Map* AVP_SurfaceEffect_System::GetCollisionMap(Asura_Hash_ID uCollider, Asura_Hash_ID uCollidee, bool bAddOnFail /*= false*/)
{
	AVP_SurfaceEffect_CollisionMap*	pxCollisionMap = NULL;		
	pxCollisionMap = s_xCollisionTableList.GetDataByHash(uCollider);
	if(!pxCollisionMap && bAddOnFail)
	{
		pxCollisionMap		= s_xCollisionTableList.AddNode(uCollider);
	}

	if(pxCollisionMap)
	{
		AVP_SurfaceEffect_Map*	pxSurfaceEffect = NULL;
		pxSurfaceEffect = pxCollisionMap->GetList()->GetDataByHash(uCollidee);
		if(!pxSurfaceEffect && bAddOnFail)
		{
			pxSurfaceEffect = pxCollisionMap->GetList()->AddNode(uCollidee);
		}

		if(pxSurfaceEffect)
		{
			return(pxSurfaceEffect);
		}	
	}
	return(NULL);
}


AVP_SurfaceEffect_Map* AVP_SurfaceEffect_System::GetEffectTypeMap(Asura_Hash_ID uSurfaceHash, Asura_Hash_ID uEffectTypeHash, bool bAddOnFail /*= false*/)
{
	AVP_SurfaceEffect_EffectTypeMap*	pxEffectTypeMap = NULL;		
	pxEffectTypeMap = s_xEffectTypeTableList.GetDataByHash(uSurfaceHash);
	if(!pxEffectTypeMap && bAddOnFail)
	{
		pxEffectTypeMap		= s_xEffectTypeTableList.AddNode(uSurfaceHash);
	}

	if(pxEffectTypeMap)
	{
		AVP_SurfaceEffect_Map*	pxSurfaceEffect = NULL;
		pxSurfaceEffect = pxEffectTypeMap->GetList()->GetDataByHash(uEffectTypeHash);
		if(!pxSurfaceEffect && bAddOnFail)
		{
			pxSurfaceEffect = pxEffectTypeMap->GetList()->AddNode(uEffectTypeHash);
		}

		if(pxSurfaceEffect)
		{
			return(pxSurfaceEffect);
		}	
	}
	return(NULL);
}


bool AVP_SurfaceEffect_PFXEnumerator::Enumerate( const u_int uIndex, char const* const szName, const Asura_Value& xValue )
{
	Asura_Assertf( uIndex < AVP_MAX_SURFACE_EFFECT_PFX, ("Too many PFX (%d) in this bleuprint, max is %d", uIndex, AVP_MAX_SURFACE_EFFECT_PFX ) );
	if ( uIndex < AVP_MAX_SURFACE_EFFECT_PFX )
	{
		m_puHash[ uIndex ] = xValue.GetHashIDValue();
		return true;
	}
	return false;
}

u_int AVP_SurfaceEffect_System::CreateDamageEntityPFXEffect( const Asura_Vector_3& xPosition, const Asura_Vector_3& xNormal,
															Asura_Hash_ID uPFXHash, Asura_Guid uTarget, int iBoneID)
{
	AVP_ClientEntity_PhysicalObject *pxPO = AVP_ClientEntity_PhysicalObject::FindEntity(uTarget);
	if (!pxPO) return NULL;
	Asura_Animation* pxAnimation = pxPO->GetShapeInstance().GetAnimation();

	const u_int	uPFXEffectID = Asura_PFX_Entity_Effect::Create(xPosition, xNormal, uPFXHash, 0, ASURA_PFX_EFFECT_CAMERA_FLAGS, uTarget);
	bool bSuccess = uPFXEffectID != ASURA_PFX_INVALID_EFFECT_ID;

	if ( pxAnimation != NULL && iBoneID != -1 )
		// Calculate the bone-relative data...
	{
		Asura_Matrix_3x3	xWorldSpaceOrientationMatrix;
		Asura_Maths::OrientationFromDirection( xWorldSpaceOrientationMatrix, xNormal );

		Asura_Assert( pxAnimation, "No animation passed in" );

		bSuccess = false;
		Asura_Quat			xObjectSpaceOrientation;
		Asura_Vector_3		xObjectSpacePosition;

		Asura_Quat			xInvMyOri( pxAnimation->GetOrientQuat() );
		xInvMyOri.Invert();

		xObjectSpaceOrientation = xInvMyOri * Asura_Quat(xWorldSpaceOrientationMatrix);
		xObjectSpacePosition = xInvMyOri.RotateVector( xPosition - pxAnimation->GetPosition());

		Asura_Vector_3		xBoneRelativePosition;
		Asura_Quat			xBoneRelativeOrientation;
		if(pxAnimation && pxAnimation->GetBoneRelativePositionAndOrientation( xObjectSpacePosition, xObjectSpaceOrientation, iBoneID, xBoneRelativePosition, xBoneRelativeOrientation ) )
		{
			Asura_AnimationAttachment*	pxAttachment	= new Asura_AnimationAttachment_BoneRelativePFX(uPFXEffectID, true);
			if(pxAttachment)
			{
				bSuccess = pxAttachment->LinkAttachment_BoneRelative(pxAnimation, xBoneRelativePosition, xBoneRelativeOrientation, iBoneID);
			}
		}
	}
	return uPFXEffectID;
}


/// End of file. ///



