
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ConnectionState_Terminated.cpp
// Author : Anthony Gregson
// Date   : 23/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Core/States/Connection/AVP_ConnectionState_Terminated.h"

// Asura independent header-files...
#include "Core/Asura_MemoryManagement_Enums.h"
#include "Core/Resources/Asura_Resource_System.h"
#include "Entity/Asura_Entity_System.h"

// Game independent header-files...
#include "Actor/Vertebrates/Alien/AVP_Alien_PlayerAnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_PlayerAnimStateData.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_PlayerAnimStateData.h"
#include "Core/AVP_Connection.h"
#include "Core/AVP_Core.h"
#include "Core/AVP_Game_Core.h"
#include "LoadAndSave/AVP_WeaponData_System.h"
#include "Multiplayer/AVP_Multiplayer_GameCore.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_ConnectionState_Terminated::Exit
// Author	: Anthony Gregson
// Date		: 23/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_ConnectionState_Terminated::Exit(const u_int uNextState)
{
	AVP_Game_Core::Shutdown();
	if (AVP_Settings::IsMultiPlayer())
	{
		AVP_Multiplayer_GameCore::Shutdown(); 
	}

#ifdef ASURA_TOOLS
	// Reset the core if we're not going straight into tools.
	if(AVP_Core::GetToolsRequested())
	{
		// Otherwise just destroy the entities.
		Asura_Timers::SetGameTimeScale(1.0f);
		Asura_Entity_System::Reset();
	}	
	else
#endif
	{
		// Reset all of the systems...
		AVP_Core::Reset();

		AVP_Alien_PlayerAnimStateData::GetSingleton().ResetStorage();
		AVP_Human_PlayerAnimStateData::GetSingleton().ResetStorage();
		AVP_Predator_PlayerAnimStateData::GetSingleton().ResetStorage();

		// Remove all resources associated with the game...
		Asura_Resource_System::DestroyResourceBank(uAVP_RESOURCE_BANK_HASH_LEVEL);
		Asura_Resource_System::DestroyResourceBank(uAVP_RESOURCE_BANK_HASH_PLAYER_PREDATOR);
		Asura_Resource_System::DestroyResourceBank(uAVP_RESOURCE_BANK_HASH_PLAYER_ALIEN);
		Asura_Resource_System::DestroyResourceBank(uAVP_RESOURCE_BANK_HASH_PLAYER_MARINE);
		
		for ( u_int uCharItr = 0; uCharItr < AVP_Game_Core::NUM_CHARACTERTYPES; ++uCharItr )
		{
			AVP_Game_Core::CharacterType eCharType = static_cast < AVP_Game_Core::CharacterType >( uCharItr );
			const u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( eCharType );
			for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons; ++uWeaponItr )
			{
				if ( AVP_WeaponData_System::GetShouldLoadWeapon( uWeaponItr, eCharType ) )
				{
					const char* szWeaponName = AVP_WeaponData_System::GetWeaponNameFromIndex( uWeaponItr, eCharType );
					Asura_Hash_ID uWeaponHash = Asura_GetHashID( szWeaponName );
					Asura_Resource_System::DestroyResourceBank( uWeaponHash );
				}
			}
		}
		AVP_WeaponData_System::ResetShouldBeLoadedData();
	}
	AVP_Core::ShutdownNetwork();
	AVP_Core::ShutdownEffects();

#if defined(ASURA_MEMORYMANAGEMENT_ENABLED) && !defined(ASURA_TOOLS)
	Asura_MemoryManagement_System::Checkpoint(ASURA_MM_STATE_POST_LEVEL_UNLOAD);
#endif
}

/// End of function 'AVP_ConnectionState_Terminated::Exit'. ///


/// End of file. ///
