
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_CoreState_Game.cpp
// Author : Anthony Gregson
// Date   : 21/11/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Core/States/Core/AVP_CoreState_Game.h"

// Asura independent header-files...
#include "Botanicals/Asura_Botanical_Game_Render_Manager.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_UserInput.h"
#include "Core/Resources/Asura_Resource_System.h"
#include "Entity/Asura_Entity_System.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Network/Asura_Network.h"
#if defined(ASURA_SUPPORT_LOBBY)
#include "Network/Online/Asura_Lobby.h"
#endif	//defined(ASURA_SUPPORT_LOBBY)

// Game independent header-files...
#include "Core/AVP_Core.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_GameStatus.h"
#include "Core/AVP_StateMachine_Core.h"
#include "Core/States/Core/AVP_CoreState_LoadingFrontend.h"
#include "Network/AVP_Server.h"

#if defined(ASURA_TOOLS)
#include "ToolsAVP_System.h"
#endif	//defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

AVP_CoreState_Game::AVP_CoreState_Game(AVP_StateMachine_Core& xParentMachine)
: PARENT(xParentMachine)
{
}


////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_CoreState_Game::Enter
// Author	: Anthony Gregson
// Date		: 21/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_CoreState_Game::Enter(const u_int uPreviousState)
{
#if defined(ASURA_TOOLS)
	if(!ToolsAVP_System::IsCancelQuitRequested())
	{
		m_xConnection.Initialise();
	}
#else
	// If we're in multiplayer, in non-debug builds disable asserts until we finish the game...
	#ifndef _DEBUG
	if(AVP_Settings::GetExternalNetworkActive())
	{
		AsuraAssert_SetIgnoreAll(true);
	}
	#endif

	const AVP_Settings::Species	eSpecies	= AVP_Settings::GetPlayerSpecies();
	switch(eSpecies)
	{
		case AVP_Settings::AVP_SPECIES_PREDATOR:
		{
			Asura_Resource_System::CreateResourceBank(uAVP_RESOURCE_BANK_HASH_LOADING, szAVP_RESOURCE_BANK_NAME_LOADING, uAVP_RESOURCE_BANK_TRG_LOADING);
			AVP_Core::LoadFile("GUIMenu\\Predator_Loading." ASURA_GUIMENU_STANDARD_FILE_EXTENSION);
		}
		break;

		case AVP_Settings::AVP_SPECIES_ALIEN:
		{
			Asura_Resource_System::CreateResourceBank(uAVP_RESOURCE_BANK_HASH_LOADING, szAVP_RESOURCE_BANK_NAME_LOADING, uAVP_RESOURCE_BANK_TRG_LOADING);
			AVP_Core::LoadFile("GUIMenu\\Alien_Loading." ASURA_GUIMENU_STANDARD_FILE_EXTENSION);
		}
		break;

		default:
		{
			// We have got this far and no character type has been either selected or found
			// Load in a marine for now and set the type.
			Asura_Assert(false, "Unknown character type.");
			AVP_Settings::SetPlayerSpecies(AVP_Settings::AVP_SPECIES_HUMAN);
		}
		// fall through to human

		case AVP_Settings::AVP_SPECIES_HUMAN:
		{
			Asura_Resource_System::CreateResourceBank(uAVP_RESOURCE_BANK_HASH_LOADING, szAVP_RESOURCE_BANK_NAME_LOADING, uAVP_RESOURCE_BANK_TRG_LOADING);
			AVP_Core::LoadFile("GUIMenu\\Marine_Loading." ASURA_GUIMENU_STANDARD_FILE_EXTENSION);
		}
		break;
	}

	m_xConnection.Initialise();
#endif	//defined(ASURA_TOOLS)
}

////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_CoreState_Game::Update
// Author	: Anthony Gregson
// Date		: 21/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_CoreState_Game::Update()
{
#if defined(ASURA_TOOLS)
	if (ToolsAVP_System::IsCancelQuitRequested())
	{
		GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_TOOLS);
	}
	else
#endif	//defined(ASURA_TOOLS)
	{
		m_xConnection.Update();

		if (m_xConnection.GetCurrentState() == AVP_Connection::STATE_TERMINATED)
		{
			if (AVP_GameStatus::GetQuitApplication())
			{
				GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_SHUTTINGDOWN);
			}
			else
			{
				// Pick the appropriate next state
#if defined(ASURA_SUPPORT_LOBBY)
				if (Asura_Lobby_System::IsInitialised())
				{
					AVP_CoreState_LoadingFrontend::SetStatePostLoad( AVP_CoreState_LoadingFrontend::STATE_POSTLOAD_MPLOBBY );
					GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_GAMEFRONTEND);
				}
				else
#endif	//defined(ASURA_SUPPORT_LOBBY)
				{
#if defined(ASURA_TOOLS)
				GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_TOOLS);
#elif ( defined(AVP_USE_GAME_FRONTEND) && defined AVP_USE_DEVELOPMENT_FRONT_END )
				if( AVP_DevelopmentFrontEnd::IsFrontEndRequested() )
				{
					GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_GAMEFRONTEND);
				}
				else
				{
					GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_DEVELOPMENTFRONTEND);	
				}
#elif defined(AVP_USE_GAME_FRONTEND)
				GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_GAMEFRONTEND);
#elif defined(AVP_USE_DEVELOPMENT_FRONT_END)
				GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_DEVELOPMENTFRONTEND);
#else
				GetParentMachine().ChangeState(AVP_StateMachine_Core::STATE_SHUTTINGDOWN);
#endif
			}
			}
			
			// JG - 08 May 2009 - We need to keep the quit application flag around for parent states to do their stuff, so don't reset it
			bool bGetQuitApplication = AVP_GameStatus::GetQuitApplication();
			
			AVP_GameStatus::ResetStatusChanges();
			
			AVP_GameStatus::SetQuitApplication( bGetQuitApplication ); 
		}
	}
}

/// End of function 'AVP_CoreState_Game::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_CoreState_Game::Render
// Author	: Anthony Gregson
// Date		: 21/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_CoreState_Game::Render() const
{
#if defined(ASURA_TOOLS)
	if(!ToolsAVP_System::IsCancelQuitRequested())
	{
		m_xConnection.Render();
	}
#else
	m_xConnection.Render();
#endif	//defined(ASURA_TOOLS)
}

/// End of function 'AVP_CoreState_Game::Render'. ///



////////////////////////////////////////////////////////////////////////////////
// Function	: AVP_CoreState_Game::Exit
// Author	: Anthony Gregson
// Date		: 21/11/07
////////////////////////////////////////////////////////////////////////////////

void AVP_CoreState_Game::Exit(const u_int uNextState)
{
#if defined(ASURA_TOOLS)
	if(!ToolsAVP_System::IsCancelQuitRequested())
	{
		m_xConnection.Shutdown();
	}
#else
	// Re-enable asserts now that we've finished in multiplayer...
	#ifndef _DEBUG
	if(AVP_Settings::GetExternalNetworkActive())
	{
		AsuraAssert_SetIgnoreAll(false);
	}
	#endif

	m_xConnection.Shutdown();
	Asura_Resource_System::DestroyResourceBank(uAVP_RESOURCE_BANK_HASH_LOADING);
#endif	//defined(ASURA_TOOLS)
}

/// End of function 'AVP_CoreState_Game::Exit'. ///


void AVP_CoreState_Game::RequestQuitApplication()
{
	// Not quite so simple to quit from here. Instead of going straight to shutting down, get our connection to terminate.
	if (m_xConnection.GetCurrentState() != AVP_Connection::STATE_TERMINATED)
	{
		m_xConnection.Terminate((AVP_Settings::IsSinglePlayer() || AVP_Settings::GetIsServer()) ? ASURA_NDR_SERVERQUIT : ASURA_NDR_UNKNOWN);
	}
}

/// End of file. ///
