//////////////////////////////////////////////////////////////////////////////
//
// AVP_Cutscene_Event_SetVisionMode.cpp
//
// (Ali Jakamy - 25.05.2008) Created
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "AVP_Cutscene_Event_SetVisionMode.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Cutscene/Asura_Cutscene.h"
#include "SpecialFX/Asura_FullScreenSFX_System.h"

// AVP
#include "Actor/AVP_Actor_HUDHandler.h"
#include "Core/AVP_Game_Core.h"
#include "Cutscene/AVP_Cutscene_Event.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Network/AVP_Player.h"

// Tools
#ifdef ASURA_TOOLS
	#include "Blueprint/Tools_Mode_Blueprint.h"
	#include "Tools_System.h"
	#include "ToolsAVP_System.h"
#endif // ASURA_TOOLS

AVP_Cutscene_Event_SetVisionMode::AVP_Cutscene_Event_SetVisionMode(Asura_Cutscene* pxCutscene)
: PARENT(AVP_CET_SetVisionMode, pxCutscene)
, m_uVisionModeHash(ASURA_HASH_ID_UNSET)
, m_fDuration(0.0f)
, m_uFlags(0)
, m_uOriginalVisionModeHash(ASURA_HASH_ID_UNSET)
{
}

void AVP_Cutscene_Event_SetVisionMode::SetVisionMode()
{
#ifdef ASURA_TOOLS
	if(Tools_System::IsActive())
	{
		m_uOriginalVisionModeHash = ToolsAVP_System::GetCurrVisionMode();
		ToolsAVP_System::SetCamera(m_uVisionModeHash);
		m_uFlags |= FLAG_VISION_MODE_SET;
	}
	else
#endif // ASURA_TOOLS
	{
		AVP_Player* pxAVPPlayer = AVP_Game_Core::GetPlayer();
		if(pxAVPPlayer)
		{
			AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(pxAVPPlayer->GetPuppetGuid());
			if(pxActor)
			{
				AVP_Actor_HUDHandler* pxHud= pxActor->GetHUDHandler();
				if(pxHud)
				{
					m_uOriginalVisionModeHash = pxHud->GetVisionMode();
					AVP_Game_Core::SetVisionMode(m_uVisionModeHash, m_uOriginalVisionModeHash, ((m_uFlags & FLAG_IMMEDIATE_TRANSITION_AT_START) != 0));
					// Set vision mode
					m_uFlags |= FLAG_VISION_MODE_SET;
				}
				else
				{
					Asura_Assert(false, "Actor has no HUD");
				}
			}
			else
			{
				Asura_Assert(false, "Actor is NULL");
			}
		}
		else
		{
			Asura_Assert(false, "AVP player is NULL");
		}
	}
}

void AVP_Cutscene_Event_SetVisionMode::UnsetVisionMode()
{
#ifdef ASURA_TOOLS
	if(Tools_System::IsActive())
	{
		ToolsAVP_System::SetCamera(m_uOriginalVisionModeHash);
		// Set event as unset
		m_uFlags &= ~FLAG_VISION_MODE_SET; 
	}
	else
#endif // ASURA_TOOLS
	{
		AVP_Player* pxAVPPlayer = AVP_Game_Core::GetPlayer();
		if(pxAVPPlayer)
		{
			AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(pxAVPPlayer->GetPuppetGuid());
			if(pxActor)
			{
				AVP_Actor_HUDHandler* pxHud= pxActor->GetHUDHandler();
				if(pxHud)
				{
					AVP_Game_Core::SetVisionMode(m_uOriginalVisionModeHash, m_uVisionModeHash, ((m_uFlags & FLAG_IMMEDIATE_TRANSITION_AT_END) != 0));
					// Set event as unset
					m_uFlags &= ~FLAG_VISION_MODE_SET; 
				}
				else
				{
					Asura_Assert(false, "Actor has no HUD");
				}
			}
			else
			{
				Asura_Assert(false, "Actor is NULL");
			}
		}
		else
		{
			Asura_Assert(false, "AVP player is NULL");
		}
	}
}

void AVP_Cutscene_Event_SetVisionMode::Finish()
{
    // Vision mode was set and not unset
	if((m_uFlags & FLAG_VISION_MODE_SET) !=0)
	{
		UnsetVisionMode();
	}
}

void AVP_Cutscene_Event_SetVisionMode::UpdateAfterTrigger(Asura_Cutscene* pxCutscene)
{
	// Trigered not unset
	if( (m_uFlags & FLAG_VISION_MODE_SET) !=0 )
	{
		// Time to finish the event and unset vision mode
		if( pxCutscene->Timer > m_fDuration + m_fTriggerTime )
		{
			UnsetVisionMode();
		}
	}
}

void AVP_Cutscene_Event_SetVisionMode::Trigger(Asura_Cutscene* pxCutscene)
{
	if( pxCutscene->Timer < m_fTriggerTime + m_fDuration )
	{
		SetVisionMode();
	}
}

int AVP_Cutscene_Event_SetVisionMode::GetChunkSize()
{
	int iChunkSize = PARENT::GetChunkSize() 
		           + sizeof(int)            // Chunk version
				   + sizeof(Asura_Hash_ID)  // Vision Hash
				   + sizeof(float)          // Duration
	               + sizeof(u_int);         // Flags
	return iChunkSize;
}

void AVP_Cutscene_Event_SetVisionMode::SaveToChunk(char*& pcChunkBuffer)
{
	int iCurrentChunkVersion = CURRENT_CHUNK_VERSION;

	Asura_Chunk_ChunkBuffer_Stream xChunkBufferStream(pcChunkBuffer);

	xChunkBufferStream << iCurrentChunkVersion;

	PARENT::WriteToChunkStream(xChunkBufferStream);

	xChunkBufferStream << m_fDuration;
	xChunkBufferStream << m_uVisionModeHash;
    xChunkBufferStream << m_uFlags;
}

void AVP_Cutscene_Event_SetVisionMode::LoadFromChunk(Asura_Chunk_CutsceneEvent* pxChunk, Asura_Handle iFileHandle)
{
	int iCurrentChunkVersion;

	Asura_Chunk_File_Stream xFstream(iFileHandle);

    xFstream >> iCurrentChunkVersion;

	switch ( iCurrentChunkVersion )
	{
		case 0 :
		{
			PARENT::LoadFromChunk(pxChunk, iFileHandle);

			xFstream >> m_fDuration;
			xFstream >> m_uVisionModeHash;
			xFstream >> m_uFlags;
			break;
		}
		default :
		{
			Asura_Assertf(false, ("Unknown chunk version (%d) in AVP_Cutscene_Event_SetVisionMode", iCurrentChunkVersion));
			break;
		}
	}
}

#ifdef ASURA_TOOLS
void AVP_Cutscene_Event_SetVisionMode::Copy(Asura_Cutscene_Event* pxEvent)
{
	AVP_Cutscene_Event_SetVisionMode* pxNewEvent = static_cast<AVP_Cutscene_Event_SetVisionMode*> (pxEvent);
	PARENT::Copy(pxNewEvent);

	m_uVisionModeHash = pxNewEvent->m_uVisionModeHash;
    m_fDuration       = pxNewEvent->m_fDuration;
	m_uFlags          = pxNewEvent->m_uFlags;
}
#endif // ASURA_TOOLS

#ifdef ASURA_TOOLS
//called during level export to add extra asura files to export list
void AVP_Cutscene_Event_SetVisionMode::AddExtraFilesToList( List<Asura_String>& xNameList ) const
{
	Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, m_uVisionModeHash, false);
}
#endif // ASURA_TOOLS