
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerEntity_CheckPointNode.cpp
// Author : Chris Wallace
// Date   : 28-June-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/CheckPointNode/AVP_ServerEntity_CheckPointNode.h"

// Asura independent header-files...
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Network/Asura_Message.h"

// AVP
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"

#if defined(ASURA_TOOLS)
// Tools header-files...
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif	// defined(ASURA_TOOLS)


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const u_int	g_uCurrentChunkVersion	= 1;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ServerEntity_CheckPointNode>	AVP_ServerEntity_CheckPointNode::s_xList;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_CheckPointNode::AVP_ServerEntity_CheckPointNode
// Author       : Chris Wallace
// Date         : 28-June-2007
////////////////////////////////////////////////////////////////////////////////

AVP_ServerEntity_CheckPointNode::AVP_ServerEntity_CheckPointNode(const Asura_Guid uGuid)
: PARENT(uGuid)
{
	Classification	= AVP_ENTITYCLASS_CHECKPOINTNODE;

	SetNeverUpdateClient();
	s_xList.AddToList(this);
}

/// End of function 'AVP_ServerEntity_CheckPointNode::AVP_ServerEntity_CheckPointNode'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_CheckPointNode::~AVP_ServerEntity_CheckPointNode
// Author       : Chris Wallace
// Date         : 28-June-2007
////////////////////////////////////////////////////////////////////////////////

AVP_ServerEntity_CheckPointNode::~AVP_ServerEntity_CheckPointNode()
{
	s_xList.RemoveFromList(this);
}

/// End of function 'AVP_ServerEntity_CheckPointNode::~AVP_ServerEntity_CheckPointNode'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_CheckPointNode::Create
// Author       : Chris Wallace
// Date         : 28-June-2007
////////////////////////////////////////////////////////////////////////////////

AVP_ServerEntity_CheckPointNode* AVP_ServerEntity_CheckPointNode::Create(const Asura_Guid uGuid)
{
	return(new AVP_ServerEntity_CheckPointNode(uGuid));
}

/// End of function 'Asura_ServerEntity_TeleportDestination::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_CheckPointNode::ReadFromChunkStream
// Author       : Chris Wallace
// Date         : 28-June-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_CheckPointNode::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Read the data...
	xStream	>> uVersion;

	if (uVersion > 0)
	{
		// Version 0 didn't do the parent write.
		PARENT::ReadFromChunkStream(xStream);
	}

	if(uVersion <= g_uCurrentChunkVersion)
	{
		xStream >> m_xPosition;
		xStream >> m_xOrientation;
	}
	else
	{
		Asura_Assert(false, "Unexpected chunk version found for AVP_ServerEntity_CheckPointNode.");
	}

	// This entity doesn't need to update.
	SetNeverUpdates(true);
}

/// End of function 'AVP_ServerEntity_CheckPointNode::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_CheckPointNode::WriteToChunkStream
// Author       : Chris Wallace
// Date         : 28-June-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_CheckPointNode::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// Write the data...
	xStream << g_uCurrentChunkVersion;

	PARENT::WriteToChunkStream(xStream);

	xStream << m_xPosition;
	xStream << m_xOrientation;
}

/// End of function 'AVP_ServerEntity_CheckPointNode::WriteToChunkStream'. ///



#if defined(ASURA_TOOLS)

////////////////////////////////////////////////////////////////////////////////
// Function     : Asura_ServerEntity_TeleportDestination::ApplyChangesForTemplateInstance
// Author       : Andy Tate
// Date         : 7-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_CheckPointNode::ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance)
{
	// Call the base-function...
	PARENT::ApplyChangesForTemplateInstance(pxInstance);

	// Transform the positional-data...
	m_xOrientation	= pxInstance->TransformQuat(m_xOrientation);
	m_xPosition		= pxInstance->TransformPosition(m_xPosition);
}

/// End of function 'AVP_ServerEntity_CheckPointNode::ApplyChangesForTemplateInstance'. ///

#endif	// defined(ASURA_TOOLS)


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_CheckPointNode::HandleMessage
// Author       : Andy Tate
// Date         : 27-Feb-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerEntity_CheckPointNode::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		case ASURA_MSG_TRIGGER:
		{
			// Update the player's checkpoint guid.
			const Asura_Guid uPlayerGuid = AVP_Game_Core::GetPlayerGuid();
			AVP_ServerEntity_PlayerInfo* pxServerPlayer = AVP_ServerEntity_PlayerInfo::FindEntity(uPlayerGuid);
			if ( pxServerPlayer )
			{
				pxServerPlayer->SetCheckPointGuid( GetGuid() );
			}

			Asura_OutputToDebugger( ( "Checkpoint %u triggered.\n", GetGuid() ) );

			return true;
		}

		default:
		{
			return PARENT::HandleMessage( pxMessage );
		}
	}
}

/// End of file. ///
