//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Squad_FaceHugger.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Controller/AVP_ServerEntity_Squad_FaceHugger.h"

// asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_LOS.h"
#include "Core/Asura_Timers.h"

// avp
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ServerActor_FaceHugger.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Entity/Spawners/AVP_ServerEntity_SpawnPoint2.h"

// axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Team_Alien.h"
#include "Knowledge/AVP_Knowledge_System_AlienTeam.h"

//////////////////////////////////////////////////////////////////////////////
//
// AVP_FaceHuggerBehaviourData
//
//////////////////////////////////////////////////////////////////////////////

void AVP_FaceHuggerBehaviourData::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	if ( uVersion == 1002 )
	{
		xStream >> m_iBehaviour;
		xStream >> m_fMaxDistanceFromSpawn;
		xStream >> m_fAvoidanceDistance;
		xStream >> m_uMaxTargeters;
		xStream >> m_fTimeBetweenAttacks;
		xStream >> m_fWanderDistance;	
		xStream >> m_bAttackIfInMarineLight;
		xStream >> m_fTimeInLightBeforeAttack;
		xStream >> m_fMinHideDistance;
		xStream >> m_fChaneToRehideIfCloseFraction;
		xStream >> m_fChaneToRehideIfInLightFraction;
		xStream >> m_fChaneToRehideAfterAttackFraction;
	}
	else if ( uVersion == 1 ) // in this version there was DEFAULT(0) and CUSTOM(1) - both map into attack
	{
		xStream >> m_iBehaviour;
		if ( m_iBehaviour == 1 ) // CUSTOM
		{
			m_iBehaviour = ATTACK;
			xStream >> m_fAvoidanceDistance;
			xStream >> m_uMaxTargeters;
			xStream >> m_fTimeBetweenAttacks;
			xStream >> m_fWanderDistance;
		}
		else
		{
			m_iBehaviour = ATTACK;
			// use the defaults
		}
	}
	else
	{
		Asura_Assertf( false, ( "AVP_FaceHuggerBehaviourData::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void AVP_FaceHuggerBehaviourData::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	xStream << m_iBehaviour;
	xStream << m_fMaxDistanceFromSpawn;
	xStream << m_fAvoidanceDistance;
	xStream << m_uMaxTargeters;
	xStream << m_fTimeBetweenAttacks;
	xStream << m_fWanderDistance;	
	xStream << m_bAttackIfInMarineLight;
	xStream << m_fTimeInLightBeforeAttack;
	xStream << m_fMinHideDistance;
	xStream << m_fChaneToRehideIfCloseFraction;
	xStream << m_fChaneToRehideIfInLightFraction;
	xStream << m_fChaneToRehideAfterAttackFraction;

}

#ifdef ASURA_TOOLS
const char* AVP_FaceHuggerBehaviourData::s_apszBehaviourStrings[ NUM_BEHAVIOURS ] =
{
	"Attack",
	"Hide in shadows"
};

#endif

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Squad_FaceHugger
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List< AVP_ServerEntity_Squad_FaceHugger > AVP_ServerEntity_Squad_FaceHugger::s_xList;
Asura_Mempool* AVP_ServerEntity_Squad_FaceHugger::s_pxMemberDataMempool = NULL;
AVP_ServerEntity_Squad_FaceHugger* AVP_ServerEntity_Squad_FaceHugger::s_pxDefaultSquad = NULL;
AVP_ServerEntity_Squad_FaceHugger* AVP_ServerEntity_Squad_FaceHugger::s_pxDefaultPetSquad = NULL;

AVP_ServerEntity_Squad_FaceHugger::AVP_ServerEntity_Squad_FaceHugger( Asura_Guid uGuid )
: PARENT( uGuid )
, m_xBehaviourData()
{
	Classification = AVP_ENTITYCLASS_FACEHUGGERSQUAD;

	s_xList.AddToList( this );
}

AVP_ServerEntity_Squad_FaceHugger::~AVP_ServerEntity_Squad_FaceHugger()
{
	if ( IsDefaultSquad() )
	{
		s_pxDefaultSquad = NULL;
	}
	else if (IsDefaultPetSquad())
	{
		s_pxDefaultPetSquad = NULL;
	}

	s_xList.RemoveFromList( this );
}

void AVP_ServerEntity_Squad_FaceHugger::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	PARENT::InitialiseFromBlueprint( xBlueprint );	
}

void AVP_ServerEntity_Squad_FaceHugger::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream	>> uVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == 1 || uVersion == 2 )
	{
		for ( u_int u = 0; u < GetNumSquadMembers(); ++u )
		{
			const Asura_Guid uMemberGuid = GetMemberGuid( u );
			MemberData* pxMemberData = static_cast< MemberData* >( GetMemberData( uMemberGuid ) );
			if ( pxMemberData )
			{
				xStream >> pxMemberData->uDummy;
			}
			else
			{
				Asura_Assert( false, "AVP_ServerEntity_Squad_FaceHugger::WriteToChunkStream - couldn't find member data" );
				// Must read something in so that the chunks don't get messed up.
				Asura_Guid uDummy;
				xStream >> uDummy;
				int iDummy;
				xStream >> iDummy;
			}
		}

		m_xBehaviourData.ReadFromChunkStream( xStream );

		u_int uPGVolumesSize = 0;
		xStream >> uPGVolumesSize;

		for (u_int i = 0; i < uPGVolumesSize; ++i)
		{
			Asura_Guid uGuid;
			xStream >> uGuid;
			m_xGPVolumesIDList.Push_Back(uGuid);
		}
	}
	else
	{
		Asura_Assertf( false, ( "AVP_ServerEntity_Squad_FaceHugger::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}

	if ( IsDefaultSquad() )
	{
		Asura_Assert( !s_pxDefaultSquad, "AVP_ServerEntity_Squad_FaceHugger::ReadFromChunkStream - already got a default squad" );
		if (!s_pxDefaultSquad)
		{
			s_pxDefaultSquad = this;
		}
		else
		{
			m_bIsDefaultPetSquad = false;
		}
	}

	if ( IsDefaultPetSquad() )
	{
		if ( uVersion == 1 )
		{
			int iDataSize = sizeof(AVP_Msg_SetAITargetPriorities) + sizeof(AVP_Msg_SetAITargetPriorities::TPData);
			Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_SET_AI_TARGET_PRIORITIES, iDataSize );
			if ( pxMessage )
			{
				AVP_Msg_SetAITargetPriorities* pxData = static_cast< AVP_Msg_SetAITargetPriorities* >( pxMessage->GetData() );
				pxData->Initialise();
				pxData->m_uNumEntries = 1;
				AVP_Msg_SetAITargetPriorities::TPData& xTPData = pxData->GetFirstTPDataPtr()[0];

				xTPData.m_iPriority = AVP_AI_TargetPriorityData::LOW;
				xTPData.m_iSpecificGuidType = AVP_AI_TargetPriorityData::ACTOR;
				xTPData.m_iTargetType = AVP_AI_TargetPriorityData::CIVILIAN;
				xTPData.m_uSpecificGuid = ASURA_GUID_UNREGISTERED;
				HandleMessage(pxMessage);
				Asura_Network::DestroyMessage(pxMessage);
			}
		}

		Asura_Assert( !s_pxDefaultPetSquad, "AVP_ServerEntity_Squad_FaceHugger::ReadFromChunkStream - already got a default pet squad" );
		if (!s_pxDefaultPetSquad)
		{
			s_pxDefaultPetSquad = this;
		}
		else
		{
			m_bIsDefaultPetSquad = false;
		}
	}
}

void AVP_ServerEntity_Squad_FaceHugger::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	// Member data.
	for ( u_int u = 0; u < GetNumSquadMembers(); ++u )
	{
		const Asura_Guid uMemberGuid = GetMemberGuid( u );
		const MemberData* pxMemberData = static_cast< MemberData* >( GetMemberData( uMemberGuid ) );
		if ( pxMemberData )
		{
			xStream << pxMemberData->uDummy;
		}
		else
		{
			Asura_Assert( false, "AVP_ServerEntity_Squad_FaceHugger::WriteToChunkStream - couldn't find member data" );
			// Must write something out so that the chunks don't get messed up.
			xStream << ASURA_GUID_UNREGISTERED;
			xStream << 0;
		}
	}

	m_xBehaviourData.WriteToChunkStream( xStream );

	xStream << m_xGPVolumesIDList.Size();

	Asura_LinkList_ConstIt<Asura_Guid> xIt(m_xGPVolumesIDList);
	while(!xIt.Done())
	{
		xStream << *(xIt.GetCurrent());
		xIt.Next();
	}
}

bool AVP_ServerEntity_Squad_FaceHugger::HandleMessage( const Asura_Message* pxMessage )
{
	if ( pxMessage->GetMsgID() == AVP_MSGID_SET_FACEHUGGER_SQUAD_BEHAVIOUR )
	{
		const AVP_Msg_SetFaceHuggerSquadBehaviour* pxData = static_cast< const AVP_Msg_SetFaceHuggerSquadBehaviour* >( pxMessage->GetData() );
		m_xBehaviourData.m_iBehaviour = pxData->m_iBehaviour;
		m_xBehaviourData.m_fAvoidanceDistance = pxData->m_fAvoidanceDistance;
		m_xBehaviourData.m_uMaxTargeters = pxData->m_uMaxTargeters;
		m_xBehaviourData.m_fTimeBetweenAttacks = pxData->m_fTimeBetweenAttacks;
		m_xBehaviourData.m_fWanderDistance = pxData->m_fWanderDistance;
		m_xBehaviourData.m_bAttackIfInMarineLight = pxData->m_bAttackIfInMarineLight;
		m_xBehaviourData.m_fTimeInLightBeforeAttack = pxData->m_fTimeInLightBeforeAttack;
		m_xBehaviourData.m_fMinHideDistance = pxData->m_fMinHideDistance;
		m_xBehaviourData.m_fChaneToRehideIfCloseFraction = pxData->m_fChaneToRehideIfCloseFraction;
		m_xBehaviourData.m_fChaneToRehideIfInLightFraction = pxData->m_fChaneToRehideIfInLightFraction;
		m_xBehaviourData.m_fChaneToRehideAfterAttackFraction = pxData->m_fChaneToRehideAfterAttackFraction;
		return true;
	}
	else
	{
		return PARENT::HandleMessage( pxMessage );
	}
}

void AVP_ServerEntity_Squad_FaceHugger::InitialiseMempools()
{
	s_pxMemberDataMempool = new Asura_Mempool( sizeof( MemberData ), 128 );
}

void AVP_ServerEntity_Squad_FaceHugger::ShutdownMempools()
{
	Asura_Assert( s_pxMemberDataMempool->GetNumAllocated() == 0, "AVP_ServerEntity_Squad_FaceHugger::ShutdownMempools - there are still member datas allocated" );
	delete s_pxMemberDataMempool;
	s_pxMemberDataMempool = NULL;
}

Asura_LinkList_ConstIt< Asura_Guid > AVP_ServerEntity_Squad_FaceHugger::GetGPVolumeIDIter() const
{
	return Asura_LinkList_ConstIt< Asura_Guid >(m_xGPVolumesIDList);
}

void* AVP_ServerEntity_Squad_FaceHugger::AllocateMemberData( const Asura_Guid uMemberGuid )
{
	MemberData* pxMemberData = static_cast< MemberData* >( s_pxMemberDataMempool->Allocate() );
	if ( pxMemberData )
	{
		pxMemberData->Reset();
	}
	return pxMemberData;
}

void AVP_ServerEntity_Squad_FaceHugger::ReturnMemberData( void* pData )
{
	s_pxMemberDataMempool->Deallocate( pData );
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
