
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientEntity_AcidDamage.cpp
// Author : Andy Tate
// Date   : 28-Feb-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/Damage/AVP_ClientEntity_AcidDamage.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint.h"
#include "Network/Asura_Message.h"
#include "SpecialFX/Asura_Decal_Manager.h"
#include "SpecialFX/PFX/Asura_PFX_Entity_Effect.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"

// Game independent header-files...
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Damage/AVP_AcidDamage_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

AVP_ClientEntity_AcidDamage::AcidDamageList	AVP_ClientEntity_AcidDamage::s_xList;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_AcidDamage::AVP_ClientEntity_AcidDamage
// Author       : Andy Tate
// Date         : 28-Feb-2008
////////////////////////////////////////////////////////////////////////////////

AVP_ClientEntity_AcidDamage::AVP_ClientEntity_AcidDamage(const AVP_Message_AcidDamage_Create& xMessage)
{
	const AVP_ClientEntity_Actor*	pxSource	= AVP_ClientEntity_Actor::FindEntity(xMessage.GetSourceGuid());

	// Initialise the standard entity data...
	Classification	= AVP_ENTITYCLASS_ACID_DAMAGE;
	s_xList.AddToList(this);

	// Get the data from the message...
	xMessage.GetPosition(m_xPosition);
	xMessage.GetSurfaceNormal(m_xSurfaceNormal);

	// Initialise the client-side effects...
	if(pxSource == NULL)
	{
		Asura_Assert(false, "Cannot find the client-entity which created this acid-damage.");
	}
	else
	{
		const Asura_Blueprint*	pxBlueprint	= pxSource->GetBlueprint();

		if(pxBlueprint == NULL)
		{
			Asura_Assert(false, "Failed to find blueprint to use to create acid-damage.");
		}
		else
		{
			Asura_Value	xValue;

			Asura_Hash_ID uPfxBPID;

			if (xMessage.GetIsTrailMode())
			{
				uPfxBPID = 0x5ef1ae6c; //Asura_GetHashID("AcidTrailDamage.PFX"), xValue))
			}
			else
			{
				uPfxBPID = 0xB90B66C0; //Asura_GetHashID("AcidDamage.PFX"), xValue))
			}
			if(pxBlueprint->GetParameterValueByHashID(uPfxBPID, xValue))	//Asura_GetHashID("AcidDamage.PFX"), xValue))
			{
				const Asura_Hash_ID			uPFXHash	= xValue.GetHashIDValue();
				Asura_PFX_Entity_Effect*	pxEffect;

				m_uPFXHandle	= Asura_PFX_Entity_Effect::Create(m_xPosition, m_xSurfaceNormal, uPFXHash, 0, ASURA_PFX_EFFECT_CAMERA_FLAGS);
				pxEffect		= Asura_PFX_System::GetEntityEffect(m_uPFXHandle);

				if(pxEffect != NULL)
				{
					pxEffect->Start();
				}
			}

			if(pxBlueprint->GetParameterValueByHashID(0xA322E5AB, xValue))	//Asura_GetHashID("AcidDamage.Decal"), xValue))
			{
				const Asura_Hash_ID	uDecalHash	= xValue.GetHashIDValue();

				Asura_Decal_Manager::AddDecal(m_xPosition, m_xSurfaceNormal, uDecalHash);
			}
		}
	}
}

/// End of function 'AVP_ClientEntity_AcidDamage::AVP_ClientEntity_AcidDamage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_AcidDamage::~AVP_ClientEntity_AcidDamage
// Author       : Andy Tate
// Date         : 28-Feb-2008
////////////////////////////////////////////////////////////////////////////////

AVP_ClientEntity_AcidDamage::~AVP_ClientEntity_AcidDamage()
{
	Asura_PFX_Entity_Effect*	pxEffect	= Asura_PFX_System::GetEntityEffect(m_uPFXHandle);

	if(pxEffect != NULL)
	{
		pxEffect->Stop(false);
	}

	s_xList.RemoveFromList(this);
}

/// End of function 'AVP_ClientEntity_AcidDamage::~AVP_ClientEntity_AcidDamage'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_AcidDamage::Create
// Author       : Andy Tate
// Date         : 28-Feb-2008
////////////////////////////////////////////////////////////////////////////////

AVP_ClientEntity_AcidDamage* AVP_ClientEntity_AcidDamage::Create(const Asura_Message* pxMessage)
{
	const AVP_Message_AcidDamage_Create*	pxData			= static_cast<const AVP_Message_AcidDamage_Create*>(pxMessage->GetData());
	AVP_ClientEntity_AcidDamage*			pxAcidDamage	= new AVP_ClientEntity_AcidDamage (*pxData);

	return(pxAcidDamage);
}

/// End of function 'AVP_ClientEntity_AcidDamage::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_AcidDamage::Render
// Author       : Andy Tate
// Date         : 28-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientEntity_AcidDamage::Render(void)
{
}

/// End of function 'AVP_ClientEntity_AcidDamage::Render'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_AcidDamage::Update
// Author       : Andy Tate
// Date         : 28-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientEntity_AcidDamage::Update(void)
{
	PARENT::Update();
}

/// End of function 'AVP_ClientEntity_AcidDamage::Update'. ///

/// End of file. ///
