
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_EntityDamage_Apply.h
// Author : Andy Tate
// Date   : 20-Feb-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ENTITYDAMAGE_APPLY_H__
#define __AVP_ENTITYDAMAGE_APPLY_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/Damage/AVP_EntityDamage.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_EntityDamage_ApplyToBotanicals
// Author       : Andy Tate
// Date         : 20-Feb-2008
//
// Description  : Class used to apply damage from an entity to botanicals.
////////////////////////////////////////////////////////////////////////////////

class AVP_EntityDamage_ApplyRadialToBotanicals : public AVP_EntityDamage
{
public:
	// Public functions...
	AVP_EntityDamage_ApplyRadialToBotanicals(void)	{}
	//virtual ~AVP_EntityDamage_ApplyToBotanicals();

	AVP_EntityDamage_ApplyRadialToBotanicals(const Asura_Hash_ID uTypeHash, const Asura_Guid uFromGuid,									   
									   const Asura_Vector_3& xPosition, const float fNearRange, const float fFarRange, const float fAmount, const float fImpulseStrength, const Asura_Guid uParentGuid = ASURA_GUID_UNREGISTERED)
	: PARENT(uTypeHash, uFromGuid, uParentGuid)
	, m_xPosition(xPosition)
	, m_fNearRange(fNearRange)
	, m_fFarRange(fFarRange)
	, m_fAmount(fAmount)
	, m_fImpulseStrength(fImpulseStrength)
	{
	}

	AVP_EntityDamage_ApplyRadialToBotanicals(const AVP_EntityDamage_ApplyRadialToBotanicals& xOther)
	: PARENT(xOther)
	, m_xPosition(xOther.m_xPosition)
	, m_fNearRange(xOther.m_fNearRange)
	, m_fFarRange(xOther.m_fFarRange)
	, m_fAmount(xOther.m_fAmount)
	, m_fImpulseStrength(xOther.m_fImpulseStrength)
	{
	}

	AVP_EntityDamage_ApplyRadialToBotanicals& operator=(const AVP_EntityDamage_ApplyRadialToBotanicals& xOther)
	{
		m_xPosition = xOther.m_xPosition;
		m_fNearRange = xOther.m_fNearRange;
		m_fFarRange = xOther.m_fFarRange;
		m_fAmount = xOther.m_fAmount;
		m_fImpulseStrength = xOther.m_fImpulseStrength;
		PARENT::operator=(xOther);
		return *this;
	}

	const Asura_Vector_3& GetPosition() const { return m_xPosition; }
	float GetNearRange() const	{ return m_fNearRange; }
	float GetFarRange() const	{ return m_fFarRange; }
	float GetAmount() const		{ return m_fAmount; }
	float GetImpulseStrength() const	{ return m_fImpulseStrength; }

protected:
	Asura_Vector_3 m_xPosition;
	float m_fNearRange;
	float m_fFarRange;
	float m_fAmount;
	float m_fImpulseStrength;

private:
	// Private typedefs...
	typedef AVP_EntityDamage PARENT;

};   // End of class 'AVP_EntityDamage_ApplyRadialToBotanicals'.



////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_EntityDamage_ApplyToBotanicals
// Author       : Andy Tate
// Date         : 20-Feb-2008
//
// Description  : Class used to apply damage from an entity to botanicals.
////////////////////////////////////////////////////////////////////////////////

class AVP_EntityDamage_ApplyLinearToBotanicals : public AVP_EntityDamage
{
public:
	// Public functions...
	AVP_EntityDamage_ApplyLinearToBotanicals(void)	{}
	//virtual ~AVP_EntityDamage_ApplyToBotanicals();

	AVP_EntityDamage_ApplyLinearToBotanicals(const Asura_Hash_ID uTypeHash, const Asura_Guid uFromGuid,
		const Asura_Vector_3& xOrigin, const Asura_Vector_3& xDisplacement, const float fAmount, const float fImpulseStrength, const Asura_Guid uParentGuid = ASURA_GUID_UNREGISTERED)
		: PARENT(uTypeHash, uFromGuid, uParentGuid)
		, m_xOrigin(xOrigin)
		, m_xDisplacement(xDisplacement)
		, m_fAmount(fAmount)
		, m_fImpulseStrength(fImpulseStrength)
	{
	}

	AVP_EntityDamage_ApplyLinearToBotanicals(const AVP_EntityDamage_ApplyLinearToBotanicals& xOther)
		: PARENT(xOther)
		, m_xOrigin(xOther.m_xOrigin)
		, m_xDisplacement(xOther.m_xDisplacement)
		, m_fAmount(xOther.m_fAmount)
		, m_fImpulseStrength(xOther.m_fImpulseStrength)
	{
	}

	AVP_EntityDamage_ApplyLinearToBotanicals& operator=(const AVP_EntityDamage_ApplyLinearToBotanicals& xOther)
	{
		m_xOrigin = xOther.m_xOrigin;
		m_xDisplacement = xOther.m_xDisplacement;
		m_fAmount = xOther.m_fAmount;
		m_fImpulseStrength = xOther.m_fImpulseStrength;
		PARENT::operator=(xOther);
		return *this;
	}

	const Asura_Vector_3& GetOrigin() const		{ return m_xOrigin; }
	const Asura_Vector_3& GetDisplacement() const	{ return m_xDisplacement; }
	float GetAmount() const						{ return m_fAmount; }
	float GetImpulseStrength() const			{ return m_fImpulseStrength; }

protected:
	Asura_Vector_3 m_xOrigin;
	Asura_Vector_3 m_xDisplacement;
	float m_fAmount;
	float m_fImpulseStrength;

private:
	// Private typedefs...
	typedef AVP_EntityDamage PARENT;

};   // End of class 'AVP_EntityDamage_ApplyLinearToBotanicals'.


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_EntityDamage_ApplyDirectional
// Author       : Andy Tate
// Date         : 20-Feb-2008
//
// Description  : Abstract class used to apply directional-damage from entities.
////////////////////////////////////////////////////////////////////////////////

class AVP_EntityDamage_ApplyDirectional : public AVP_EntityDamage
{
public:
	// Public functions...

	// Accessor functions...
	void	GetPointOfImpact(Asura_Vector_3& xPointOfImpact) const	{ xPointOfImpact = m_xPointOfImpact; }
	void	GetDirection(Asura_Vector_3& xDirection) const			{ xDirection = m_xDirection; }
	void	GetNormal(Asura_Vector_3& xNormal) const				{ xNormal = m_xNormal; }


protected:
	// Protected variables...
	Asura_Vector_3	m_xPointOfImpact;			// The point at which damage is applied.
	Asura_Vector_3	m_xDirection;				// The direction of damage at the point at which damage is applied.
	Asura_Vector_3	m_xNormal;					// The normal of the surface at the point at which damage is applied.

	// Protected functions...
	AVP_EntityDamage_ApplyDirectional(void)	{}
	//virtual ~AVP_EntityDamage_ApplyDirectional();

	AVP_EntityDamage_ApplyDirectional(const Asura_Hash_ID uTypeHash, const Asura_Guid uFromGuid,
									  const Asura_Vector_3& xPointOfImpact, const Asura_Vector_3& xDirection, const Asura_Vector_3& xNormal, const Asura_Guid uParentGuid = ASURA_GUID_UNREGISTERED)
	: PARENT(uTypeHash, uFromGuid, uParentGuid)
	, m_xPointOfImpact(xPointOfImpact)
	, m_xDirection(xDirection)
	, m_xNormal(xNormal)
	{
	}

	AVP_EntityDamage_ApplyDirectional(const AVP_EntityDamage_ApplyDirectional& xOther)
	: PARENT(xOther)
	, m_xPointOfImpact(xOther.m_xPointOfImpact)
	, m_xDirection(xOther.m_xDirection)
	, m_xNormal(xOther.m_xNormal)
	{
	}

	AVP_EntityDamage_ApplyDirectional& operator=(const AVP_EntityDamage_ApplyDirectional& xOther)
	{
		m_xPointOfImpact	= xOther.m_xPointOfImpact;
		m_xNormal			= xOther.m_xNormal;
		m_xDirection		= xOther.m_xDirection;

		PARENT::operator=(xOther);
		return(*this);
	}

private:
	// Private typedefs...
	typedef AVP_EntityDamage PARENT;

	// Private variables...

	// Private functions...

};   // End of class 'AVP_EntityDamage_ApplyDirectional'.



////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_EntityDamage_ApplyToEntity
// Author       : Andy Tate
// Date         : 20-Feb-2008
//
// Description  : Class used to apply damage from one entity to another.
////////////////////////////////////////////////////////////////////////////////

class AVP_EntityDamage_ApplyToEntity : public AVP_EntityDamage_ApplyDirectional
{
public:
	// Public functions...
	AVP_EntityDamage_ApplyToEntity(void)	{}
	//virtual ~AVP_EntityDamage_ApplyToEntity();

	AVP_EntityDamage_ApplyToEntity(const Asura_Hash_ID uTypeHash, const Asura_Guid uFromGuid,								   
								   const Asura_Vector_3& xPointOfImpact, const Asura_Vector_3& xDirection, const Asura_Vector_3& xNormal,
								   const Asura_Guid uHitGuid, const float fHealth, const float fAmount,
								   const float fImpulseStrength, const int iBoneID = -1,
								   const u_int uNewDestroyedSkinElements = 0, const u_int uGibbedSkinElements = 0,
								   const Asura_Guid uParentGuid = ASURA_GUID_UNREGISTERED,
								   const int iAttachmentID = -1,
								   Asura_Hash_ID uMaterialResponseHashID = ASURA_HASH_ID_UNSET, 
								   bool bStealth = false,
								   bool bRandomiseDirection = false)
	: PARENT(uTypeHash, uFromGuid, xPointOfImpact, xDirection, xNormal, uParentGuid)
	, m_uHitGuid(uHitGuid)
	, m_fHealth(fHealth)
	, m_fAmount( fAmount )
	, m_fImpulseStrength(fImpulseStrength)
	, m_iBoneID(iBoneID)
	, m_uNewDestroyedSkinElementFlags(uNewDestroyedSkinElements)
	, m_uGibbedSkinElementFlags(uGibbedSkinElements)
	, m_iAttachmentID(iAttachmentID)
	, m_uMaterialResponseHashID(uMaterialResponseHashID)
	, m_bStealth(bStealth)
	, m_bRandomiseDirection(bRandomiseDirection)
	{
	}

	AVP_EntityDamage_ApplyToEntity(const AVP_EntityDamage_ApplyToEntity& xOther)
	: PARENT(xOther)
	, m_uHitGuid(xOther.m_uHitGuid)
	, m_fHealth(xOther.m_fHealth)
	, m_fAmount( xOther.m_fAmount )
	, m_fImpulseStrength(xOther.m_fImpulseStrength)
	, m_iBoneID(xOther.m_iBoneID)
	, m_uNewDestroyedSkinElementFlags(xOther.m_uNewDestroyedSkinElementFlags)
	, m_uGibbedSkinElementFlags(xOther.m_uGibbedSkinElementFlags)
	, m_iAttachmentID(xOther.m_iAttachmentID)
	, m_uMaterialResponseHashID(xOther.m_uMaterialResponseHashID)
	, m_bStealth(xOther.m_bStealth)
	, m_bRandomiseDirection(xOther.m_bRandomiseDirection)
	{
	}

	AVP_EntityDamage_ApplyToEntity& operator=(const AVP_EntityDamage_ApplyToEntity& xOther)
	{
		m_uHitGuid						= xOther.m_uHitGuid;
		m_fHealth						= xOther.m_fHealth;
		m_fAmount						= xOther.m_fAmount;
		m_fImpulseStrength				= xOther.m_fImpulseStrength;
		m_iBoneID						= xOther.m_iBoneID;
		m_uNewDestroyedSkinElementFlags	= xOther.m_uNewDestroyedSkinElementFlags;
		m_uGibbedSkinElementFlags		= xOther.m_uGibbedSkinElementFlags;
		m_iAttachmentID					= xOther.m_iAttachmentID;
		m_uMaterialResponseHashID		= xOther.m_uMaterialResponseHashID;
		m_bStealth						= xOther.m_bStealth;
		m_bRandomiseDirection			= xOther.m_bRandomiseDirection;

		PARENT::operator=(xOther);
		return(*this);
	}

	// Accessor functions...
	Asura_Guid	GetHitGuid(void) const								{ return(m_uHitGuid); }
	float		GetHealth(void) const								{ return(m_fHealth); }
	float		GetAmount() const									{ return m_fAmount; }
	float		GetImpulseStrength(void) const						{ return(m_fImpulseStrength); }
	int			GetBoneID(void) const								{ return(m_iBoneID); }
	u_int		GetNewDestroyedSkinElementFlags() const				{ return m_uNewDestroyedSkinElementFlags; }
	u_int		GetGibbedSkinElementFlags() const					{ return m_uGibbedSkinElementFlags; }
	int			GetAttachmentID() const								{ return m_iAttachmentID; }
	Asura_Hash_ID GetMaterialResponseHashID() const					{ return m_uMaterialResponseHashID; }
	bool		GetStealth() const									{ return m_bStealth; }
	bool		GetRandomiseDiretion() const						{ return m_bRandomiseDirection; }
	void		SetToDestroyAndGibBodyPart( u_int uElementType )
	{
		m_uNewDestroyedSkinElementFlags = m_uNewDestroyedSkinElementFlags | (1 << uElementType);
		m_uGibbedSkinElementFlags = m_uGibbedSkinElementFlags | (1 << uElementType);
	}


protected:
	// Protected variables...
	Asura_Guid	m_uHitGuid;							// The GUID of the entity to be damaged.
	float		m_fHealth;							// The health of this entity following the damage.
	float		m_fAmount;
	float		m_fImpulseStrength;					// The amount of impulse to apply to this entity.
	int			m_iBoneID;							// The ID of the bone to be damaged on this entity.
	u_int		m_uNewDestroyedSkinElementFlags;	// Bitfield used to say which skin elements this damage has actually destroyed.
	u_int		m_uGibbedSkinElementFlags;			// Similar bitfield for which elements will be creating gib entities. Should be a subset of m_uNewDestroyedSkinElementFlags.
	int			m_iAttachmentID;					// Hit attachment (used by container system).
	Asura_Hash_ID m_uMaterialResponseHashID;		// Material response (used by container system).
	bool		m_bStealth;							// Damaged entity was unaware (used by the spam box)
	bool		m_bRandomiseDirection;				// Randomise or adjust direction values.
private:
	// Private typedefs...
	typedef AVP_EntityDamage_ApplyDirectional PARENT;

};   // End of class 'AVP_EntityDamage_ApplyToEntity'.



////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_EntityDamage_ApplyToEnvironment
// Author       : Andy Tate
// Date         : 20-Feb-2008
//
// Description  : Class use to apply damage from an entity to the environment.
////////////////////////////////////////////////////////////////////////////////

class AVP_EntityDamage_ApplyToEnvironment : public AVP_EntityDamage_ApplyDirectional
{
public:
	// Public functions...
	AVP_EntityDamage_ApplyToEnvironment(void)	{}
	//virtual ~AVP_EntityDamage_ApplyToEnvironment();

	AVP_EntityDamage_ApplyToEnvironment(const Asura_Hash_ID uTypeHash, const Asura_Guid uFromGuid,
										const Asura_Vector_3& xPointOfImpact, const Asura_Vector_3& xDirection, const Asura_Vector_3& xNormal,
										const Asura_Hash_ID uMaterialResponseHash, const u_short usPolyFlags, const u_int uFaceIndex,
										const Asura_Guid uParentGuid = ASURA_GUID_UNREGISTERED)
	: PARENT(uTypeHash, uFromGuid, xPointOfImpact, xDirection, xNormal, uParentGuid)
	, m_uMaterialResponseHash(uMaterialResponseHash)
	, m_usPolyFlags(usPolyFlags)
	, m_uFaceIndex(uFaceIndex)
	{
	}

	AVP_EntityDamage_ApplyToEnvironment(const AVP_EntityDamage_ApplyToEnvironment& xOther)
	: PARENT(xOther)
	, m_uMaterialResponseHash(xOther.m_uMaterialResponseHash)
	, m_usPolyFlags(xOther.m_usPolyFlags)
	, m_uFaceIndex(xOther.m_uFaceIndex)
	{
	}

	AVP_EntityDamage_ApplyToEnvironment& operator=(const AVP_EntityDamage_ApplyToEnvironment& xOther)
	{
		m_uMaterialResponseHash	= xOther.m_uMaterialResponseHash;
		m_usPolyFlags			= xOther.m_usPolyFlags;
		m_uFaceIndex			= xOther.m_uFaceIndex;

		PARENT::operator=(xOther);
		return(*this);
	}

	// Accessor functions...
	Asura_Hash_ID	GetMaterialResponseHash(void) const	{ return(m_uMaterialResponseHash); }
	u_short			GetPolyFlags(void) const			{ return(m_usPolyFlags); }
	u_int			GetFaceIndex(void) const			{ return(m_uFaceIndex); }


protected:
	// Protected variables...
	Asura_Hash_ID	m_uMaterialResponseHash;	// The hash-ID of the material at the point at which damage is applied.
	u_short			m_usPolyFlags;				// The polygon-flags at the point at which damage is applied.
	u_int			m_uFaceIndex;				// The face-index of the polygon the damage is to be applied to.

	// Protected functions...


private:
	// Private typedefs...
	typedef AVP_EntityDamage_ApplyDirectional PARENT;

	// Private variables...

	// Private functions...

};   // End of class 'AVP_EntityDamage_ApplyToEnvironment'.


#endif   // ifndef __AVP_ENTITYDAMAGE_APPLY_H__


/// End of file. ///
