
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Damage_BotanicalCallback.cpp
// Author : Andy Tate
// Date   : 11-Jan-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////


#include "Core/Asura.h"

#ifdef ASURA_USE_BOTANICALS

// This file's header...
#include "Entity/Damage/AVP_EntityDamage_BotanicalCallback.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance_Header.h"
#include "SpecialFX/pfx/Asura_PFX_Entity_Effect.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_SurfaceEffects.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Network/AVP_Client.h"

static const Asura_Hash_ID s_uDamageParameterHash			= 0x335461a8;	// "BotanicalDamage"
static const Asura_Hash_ID s_uScorchParameterHash			= 0x4d0a6a11;	// "BotanicalScorch"

static const Asura_Hash_ID	s_uBotanicalDamageStateHash		= 0xB02E8B6F;	// Asura_GetHashID("damage");
static const Asura_Hash_ID	s_uBotanicalScorchedStateHash	= 0xE30F0A77;	// Asura_GetHashID("scorched");

bool AVP_EntityDamage_RadialBotanicalCallback::ProcessInstanceHeader (Asura_BGR_Instances_Header *pxInstanceHeader,
																	  const Asura_BGR_Type *pxInstanceType)
{
	return true;
}

bool AVP_EntityDamage_RadialBotanicalCallback::ProcessInstance (Asura_BGR_Instances_Header* pxInstanceHeader,
																const Asura_BGR_Instance* pxInstance,
																const Asura_BGR_Type* pxInstanceType,
																const Asura_AVector3& xInstancePosition,
																Asura_BGR_DynamicInstance* const pxDynamicInstance)
{
	for ( u_int uItr = 0; uItr < 2; ++uItr )
	{
		bool bDoScorch = false;
		bool bDoDamage = false;
		Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_DAMAGE, m_xDamage.GetTypeHash() );
		if ( pxBlueprint )
		{
			Asura_Value xValue;
			if ( pxBlueprint->GetParameterValueByHashID( s_uDamageParameterHash, xValue ) )
			{
				bDoDamage = xValue.GetBoolValue();
			}
			if ( pxBlueprint->GetParameterValueByHashID( s_uScorchParameterHash, xValue ) )
			{
				bDoScorch = xValue.GetBoolValue();
			}
		}
		if ( ( uItr == 0 && bDoDamage ) || ( uItr == 1 && bDoScorch ) )
		{
			Asura_BGR_State_ID	xStateID;		// The state ID for the current botanical.
			xStateID = pxInstanceType->GetStateID( uItr == 0 ? s_uBotanicalDamageStateHash : s_uBotanicalScorchedStateHash );
			if ( xStateID != ASURA_BGR_STATE_ID_INVALID )
			{
				const float		fAmount				= m_xDamage.GetAmount();
				if ( fAmount > 0.0f )
				{
				Asura_Vector_3	xOrigin			= m_xDamage.GetPosition();
				Asura_Vector_3	xDirection		= Asura_Vector_3( xInstancePosition ) - xOrigin;
				const float		fRange			= xDirection.Normalise();
				const float		fCurrentDamage	= static_cast<float>(pxInstanceHeader->GetState(pxDynamicInstance, xStateID));
				const float		fAmount			= m_xDamage.GetAmount();
				Asura_LOS_InputData xLosInput( xOrigin, xDirection, m_xDamage.GetFromGuid(), fRange );

					const Asura_BGR_StateTable&	xStateTable			= pxInstanceType->GetStateTable(xStateID);
					for(u_int u = 0; u < xStateTable.m_xArray_xSections.Size(); u++)
					{
						const Asura_BGR_StateTable_Section&	xSection	= xStateTable.m_xArray_xSections[u];
						for(u_int v = 0; v < xSection.m_xArray_xVersions.Size(); v++)
						{
							const Asura_BGR_StateTable_Version&	xVersion	= xSection.m_xArray_xVersions[v];

							Asura_Matrix_3x3 xOri;
							pxInstance->GetOrientation( xOri );
							float fDistance = fRange;
							Asura_Vector_3 xDamagePosition( xInstancePosition );
							Asura_Vector_3 xNormal( -xDirection );

							if ( xVersion.m_pxCollisionMesh )
							{
								Asura_LOS_OutputData* pxLosOutput = NULL;
								xVersion.m_pxCollisionMesh->LOSTest( Asura_Vector_3( xInstancePosition ), xOri, &xLosInput, pxLosOutput );
								if ( pxLosOutput && pxLosOutput->m_fDistanceToIntersection < fRange )
								{
									fDistance = pxLosOutput->m_fDistanceToIntersection;
									xDamagePosition = xOrigin + xDirection * fDistance;
									xNormal = pxLosOutput->m_xNormal;
								}
							}
							else
							{
								Asura_Bounding_Box xBB = pxInstanceType->GetBoundingBox();
								xBB.Displace( Asura_Vector_3( xInstancePosition ) );
								Asura_Maths::RayIntersectsBoundingBox( xOrigin, xDirection * fRange, xBB, &fDistance, &xDamagePosition, NULL, &xNormal );
							}
							
							if ( fDistance < fRange )
							{
								// surface effect
								const Asura_Hash_ID uSurfaceTypeHash			= xVersion.m_uProjectFlags;
								AVP_SurfaceEffect_System::PlayDamageSurfaceEffect( uSurfaceTypeHash, m_xDamage.GetTypeHash(), xDamagePosition, xNormal );

								// Apply the state damage
								const float					fParametric			= ( fDistance - m_xDamage.GetNearRange() ) / ( m_xDamage.GetFarRange() - m_xDamage.GetNearRange() );
								const float					fDamageToInflict	= fAmount * fParametric;
								const float					fNewDamage			= fCurrentDamage + fDamageToInflict;
								const float					fClampedDamage		= Asura_Maths::ClampToRange(fNewDamage, 0.0f, 100.0f);
								const Asura_BGR_State_Value	xStateValueDamage	= static_cast<Asura_BGR_State_Value>(Asura_Maths::FloatToInt(fClampedDamage));

								pxInstanceHeader->SetState( pxDynamicInstance, xStateID, xStateValueDamage );
							}
						}
					}
				}
			}
		}
	} // damage type loop
	return(true);
}


bool AVP_EntityDamage_LinearBotanicalCallback::ProcessInstanceHeader (Asura_BGR_Instances_Header *pxInstanceHeader,
																	  const Asura_BGR_Type *pxInstanceType)
{
	return true;
}

bool AVP_EntityDamage_LinearBotanicalCallback::ProcessInstance (Asura_BGR_Instances_Header* pxInstanceHeader,
																const Asura_BGR_Instance* pxInstance,
																const Asura_BGR_Type* pxInstanceType,
																const Asura_AVector3& xInstancePosition,
																Asura_BGR_DynamicInstance* const pxDynamicInstance)
{
	for ( u_int uItr = 0; uItr < 2; ++uItr )
	{
		bool bDoScorch = false;
		bool bDoDamage = false;
		Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_DAMAGE, m_xDamage.GetTypeHash() );
		if ( pxBlueprint )
		{
			Asura_Value xValue;
			if ( pxBlueprint->GetParameterValueByHashID( s_uDamageParameterHash, xValue ) )
			{
				bDoDamage = xValue.GetBoolValue();
			}
			if ( pxBlueprint->GetParameterValueByHashID( s_uScorchParameterHash, xValue ) )
			{
				bDoScorch = xValue.GetBoolValue();
			}
		}
		if ( ( uItr == 0 && bDoDamage ) || ( uItr == 1 && bDoScorch ) )
		{
			Asura_BGR_State_ID	xStateID;		// The state ID for the current botanical.
			xStateID = pxInstanceType->GetStateID( uItr == 0 ? s_uBotanicalDamageStateHash : s_uBotanicalScorchedStateHash );
			if ( xStateID != ASURA_BGR_STATE_ID_INVALID )
			{
				const float		fAmount				= m_xDamage.GetAmount();
				if ( fAmount > 0.0f )
				{
					Asura_Vector_3	xOrigin			= m_xDamage.GetOrigin();
					Asura_Vector_3	xDirection		= m_xDamage.GetDisplacement();
					const float		fRange			= xDirection.Normalise();
					const float		fCurrentDamage	= static_cast<float>( pxInstanceHeader->GetState(pxDynamicInstance, xStateID) );
					Asura_LOS_InputData xLosInput( xOrigin, xDirection, m_xDamage.GetFromGuid(), fRange );
					
					const float					fNewDamage			= fCurrentDamage + fAmount;
					const float					fClampedDamage		= Asura_Maths::ClampToRange(fNewDamage, 0.0f, 100.0f);
					const Asura_BGR_State_Value	xStateValueDamage	= static_cast<Asura_BGR_State_Value>(Asura_Maths::FloatToInt(fClampedDamage));
					const Asura_BGR_StateTable&	xStateTable			= pxInstanceType->GetStateTable(xStateID);

					for(u_int u = 0; u < xStateTable.m_xArray_xSections.Size(); u++)
					{
						const Asura_BGR_StateTable_Section&	xSection	= xStateTable.m_xArray_xSections[u];

						for(u_int v = 0; v < xSection.m_xArray_xVersions.Size(); v++)
						{
							const Asura_BGR_StateTable_Version&	xVersion	= xSection.m_xArray_xVersions[v];

							Asura_Matrix_3x3 xOri;
							pxInstance->GetOrientation( xOri );
							float fDistance = fRange;
							Asura_Vector_3 xDamagePosition( xInstancePosition );
							Asura_Vector_3 xNormal( -xDirection );
							if ( xVersion.m_pxCollisionMesh )
							{
								Asura_LOS_OutputData* pxLosOutput = NULL;
								xVersion.m_pxCollisionMesh->LOSTest( Asura_Vector_3( xInstancePosition ), xOri, &xLosInput, pxLosOutput );
								if ( pxLosOutput && pxLosOutput->m_fDistanceToIntersection < fRange )
								{
									fDistance = pxLosOutput->m_fDistanceToIntersection;
									xDamagePosition = xOrigin + xDirection * fDistance;
									xNormal = pxLosOutput->m_xNormal;
								}
							}
							else
							{
								Asura_Bounding_Box xBB = pxInstanceType->GetBoundingBox();
								xBB.Displace( Asura_Vector_3( xInstancePosition ) );
								Asura_Maths::RayIntersectsBoundingBox( xOrigin, xDirection * fRange, xBB, &fDistance, &xDamagePosition, NULL, &xNormal );
							}

							if ( fDistance < fRange )
							{
								// surface effect
								const Asura_Hash_ID uSurfaceTypeHash			= xVersion.m_uProjectFlags;
								AVP_SurfaceEffect_System::PlayDamageSurfaceEffect( uSurfaceTypeHash, m_xDamage.GetTypeHash(), xDamagePosition, xNormal );

								// Apply the state damage
								pxInstanceHeader->SetState( pxDynamicInstance, xStateID, xStateValueDamage );
							}
						}
					}
				}
			}
		}
	} // damage type loop
	return(true);
}

#endif //ifdef ASURA_USE_BOTANICALS

/// End of file. ///
