
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerEntity_AcidDamage.cpp
// Author : Andy Tate
// Date   : 28-Feb-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/Damage/AVP_ServerEntity_AcidDamage.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint.h"
#include "Core/Asura_LOS.h"
#if !defined(ASURA_NO_DEBUG_OUTPUT)
#include "Debugging/Asura_DebugLineRender.h"
#endif	//!defined(ASURA_NO_DEBUG_OUTPUT)
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
#include "Interface/Asura_CommandConsole.h"
#endif	//defined(ASURA_SUPPORT_COMMAND_CONSOLE)
#include "Network/Asura_Network.h"
#include "Volume/Asura_Volume_Cylinder.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_AcidDamage_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

DVSVAR bool		dvs_bAcidDamageDisplayDebugInfo	= false;	//true;	//

DVSVAR float	dvs_fAcidDamageTimeBetweenDamageApplications= 1.0f;
DVSVAR float	dvs_fAcidDamageAmountPerApplication			= 20.0f;

DVSVAR float	dvs_fAcidDamagePoolRadius					= 1.0f;
DVSVAR float	dvs_fAcidDamagePoolHeight					= 0.100f;
DVSVAR float	dvs_fAcidDamagePoolDistanceBelowSurface		= 0.010f;
DVSVAR float	dvs_fAcidDamagePoolDuration					= 5.0f;
DVSVAR float	dvs_fAcidDamagePoolFadeOutDuration			= 0.5f;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

AVP_ServerEntity_AcidDamage::AcidDamageList	AVP_ServerEntity_AcidDamage::s_xList;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_AcidDamage::AVP_ServerEntity_AcidDamage
// Author       : Andy Tate
// Date         : 28-Feb-2008
////////////////////////////////////////////////////////////////////////////////

AVP_ServerEntity_AcidDamage::AVP_ServerEntity_AcidDamage(const AVP_Message_AcidDamage_Create& xMessage)
: PARENT(Asura_Entity_System::GetCreatedByGameGuid())
, m_uSourceGuid(xMessage.GetSourceGuid())
, m_fTimeSinceCreation(0.0f)
, m_fPoolDuration(dvs_fAcidDamagePoolDuration)
, m_fPoolFadeOutDuration(dvs_fAcidDamagePoolFadeOutDuration)
, m_fTimeBetweenDamageApplications(dvs_fAcidDamageTimeBetweenDamageApplications)
, m_fAmountPerApplication(dvs_fAcidDamageAmountPerApplication)
, m_bIsTrailMode(xMessage.GetIsTrailMode())
{
	const AVP_ServerEntity_Actor*	pxSource					= AVP_ServerEntity_Actor::FindEntity(xMessage.GetSourceGuid());

	float							fPoolRadius					= dvs_fAcidDamagePoolRadius;
	float							fPoolHeight					= dvs_fAcidDamagePoolHeight;
	float							fPoolDistanceBelowSurface	= dvs_fAcidDamagePoolDistanceBelowSurface;

	Asura_Matrix_3x3				xOrientation;
	Asura_Vector_3					xVolumePosition;
	Asura_Quat						xQuat;

#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	if(!Asura_CommandConsole::VarExists(L"AVP.AcidDamage.DisplayDebugInfo"))
	{
		Asura_CommandConsole::AddVar(L"AVP.AcidDamage.DisplayDebugInfo", &dvs_bAcidDamageDisplayDebugInfo);

		Asura_CommandConsole::AddVar(L"AVP.AcidDamage.TimeBetweenDamageApplications", &dvs_fAcidDamageTimeBetweenDamageApplications);
		Asura_CommandConsole::AddVar(L"AVP.AcidDamage.AmountPerApplication", &dvs_fAcidDamageAmountPerApplication);

		Asura_CommandConsole::AddVar(L"AVP.AcidDamage.Pool.Radius", &dvs_fAcidDamagePoolRadius);
		Asura_CommandConsole::AddVar(L"AVP.AcidDamage.Pool.Height", &dvs_fAcidDamagePoolHeight);
		Asura_CommandConsole::AddVar(L"AVP.AcidDamage.Pool.DistanceBelowSurface", &dvs_fAcidDamagePoolDistanceBelowSurface);
		Asura_CommandConsole::AddVar(L"AVP.AcidDamage.Pool.Duration", &dvs_fAcidDamagePoolDuration);
		Asura_CommandConsole::AddVar(L"AVP.AcidDamage.Pool.FadeOutDuration", &dvs_fAcidDamagePoolFadeOutDuration);
	}
#endif	//defined(ASURA_SUPPORT_COMMAND_CONSOLE)

	// Initialise the standard entity stuff...
	Classification	= AVP_ENTITYCLASS_ACID_DAMAGE;
	SetNeverUpdateClient();
	s_xList.AddToList(this);

	// Initialise the data from blueprints...
	m_fPoolDuration						= dvs_fAcidDamagePoolDuration;
	m_fPoolFadeOutDuration				= dvs_fAcidDamagePoolFadeOutDuration;
	m_fTimeBetweenDamageApplications	= dvs_fAcidDamageTimeBetweenDamageApplications;
	m_fAmountPerApplication				= dvs_fAcidDamageAmountPerApplication;

	if(pxSource == NULL)
	{
		Asura_Assert(false, "Cannot find the server-entity which created this acid-damage.");
	}
	else
	{
		const Asura_Blueprint*	pxBlueprint	= pxSource->GetBlueprint();

		if(pxBlueprint == NULL)
		{
			Asura_Assert(false, "Failed to find blueprint to use to create acid-damage.");
		}
		else
		{
			Asura_Value	xValue;

			if(pxBlueprint->GetParameterValueByHashID(0x7231BA22, xValue))	//Asura_GetHashID("AcidDamage.Pool.Radius"), xValue))
			{
				fPoolRadius	= xValue.GetFloatValue();
			}

			if(pxBlueprint->GetParameterValueByHashID(0x615BDEB7, xValue))	//Asura_GetHashID("AcidDamage.Pool.Height"), xValue))
			{
				fPoolHeight	= xValue.GetFloatValue();
			}

			if(pxBlueprint->GetParameterValueByHashID(0x56FC0D61, xValue))	//Asura_GetHashID("AcidDamage.Pool.DistanceBelowSurface"), xValue))
			{
				fPoolDistanceBelowSurface	= xValue.GetFloatValue();
			}

			if (m_bIsTrailMode)
			{
				if(pxBlueprint->GetParameterValueByHashID(0xad6c17d0, xValue))	//Asura_GetHashID("AcidTrailDamage.Pool.Duration"), xValue))
				{
					m_fPoolDuration	= xValue.GetFloatValue();
				}
			}
			else
			{
				if(pxBlueprint->GetParameterValueByHashID(0x37CA7724, xValue))	//Asura_GetHashID("AcidDamage.Pool.Duration"), xValue))
				{
					m_fPoolDuration	= xValue.GetFloatValue();
				}
			}

			if (m_bIsTrailMode)
			{
				if(pxBlueprint->GetParameterValueByHashID(0xbbe2cd2a, xValue))	//Asura_GetHashID("AcidTrailDamage.Pool.FadeOutDuration"), xValue))
				{
					m_fPoolFadeOutDuration	= xValue.GetFloatValue();
				}
			}
			else
			{
				if(pxBlueprint->GetParameterValueByHashID(0x114E4756, xValue))	//Asura_GetHashID("AcidDamage.Pool.FadeOutDuration"), xValue))
				{
					m_fPoolFadeOutDuration	= xValue.GetFloatValue();
				}

			}

			if(pxBlueprint->GetParameterValueByHashID(0xC1422DEB, xValue))	//Asura_GetHashID("AcidDamage.TimeBetweenDamageApplications"), xValue))
			{
				m_fTimeBetweenDamageApplications	= xValue.GetFloatValue();
			}

			if(pxBlueprint->GetParameterValueByHashID(0x06187CAD, xValue))	//Asura_GetHashID("AcidDamage.AmountPerApplication"), xValue))
			{
				m_fAmountPerApplication	= xValue.GetFloatValue();
			}
		}
	}

	// Check for the surface...
	xMessage.GetPosition(m_xPosition);
	xMessage.GetSurfaceNormal(m_xSurfaceNormal);
	Asura_Assertf( Asura_Maths::Abs( 1.0f - m_xSurfaceNormal.MagnitudeSquared() ) < 2.0e-4, ( "Surface normal of Acid Damage not normalised it is (%.4f, %.4f, %.4f)", m_xSurfaceNormal.x, m_xSurfaceNormal.y, m_xSurfaceNormal.z )	);

	// Create the appropriate volume...
	Asura_Maths::OrientationFromNormal(xOrientation, m_xSurfaceNormal);
	xQuat.FromMatrix(xOrientation);

	Asura_Assertf( Asura_Maths::Abs( 1.0f - xQuat.MagnitudeSquared() ) < 2.0e-4, (	"Orientation of Acid Damage not normalised.\n" \
																					"Quat is (%.4f, %.4f, %.4f: %.4f), Mag sq. is %.4f\n" \
																					"Normal was (%.4f, %.4f, %.4f)\n" \
																					"Matrix was |%.4f, %.4f, %.4f|\n" \
																					"           |%.4f, %.4f, %.4f|\n" \
																					"           |%.4f, %.4f, %.4f|\n",
																					xQuat.x, xQuat.y, xQuat.z, xQuat.w, xQuat.MagnitudeSquared(),
																					m_xSurfaceNormal.x, m_xSurfaceNormal.y, m_xSurfaceNormal.z,
																					xOrientation.e11, xOrientation.e12, xOrientation.e13,
																					xOrientation.e21, xOrientation.e22, xOrientation.e23,
																					xOrientation.e31, xOrientation.e32, xOrientation.e33));

	xVolumePosition	= m_xPosition - (m_xSurfaceNormal * fPoolDistanceBelowSurface);
	m_pxVolume		= new Asura_Volume_Cylinder (xVolumePosition, xQuat, fPoolRadius, (fPoolHeight + fPoolDistanceBelowSurface));

#if !defined(ASURA_NO_DEBUG_OUTPUT)
	if(dvs_bAcidDamageDisplayDebugInfo)
	{
		Asura_DebugLineRender_LineSet*	pxLineSet	= Asura_DebugLineRender::CreateTimedLineSet(128, m_fPoolDuration, m_fPoolFadeOutDuration);

		m_pxVolume->AddToLineSet(0xFFFFFFFF, *pxLineSet);
	}
#endif	//!defined(ASURA_NO_DEBUG_OUTPUT)
}

/// End of function 'AVP_ServerEntity_AcidDamage::AVP_ServerEntity_AcidDamage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_AcidDamage::~AVP_ServerEntity_AcidDamage
// Author       : Andy Tate
// Date         : 28-Feb-2008
////////////////////////////////////////////////////////////////////////////////

AVP_ServerEntity_AcidDamage::~AVP_ServerEntity_AcidDamage()
{
	delete(m_pxVolume);
	s_xList.RemoveFromList(this);
}

/// End of function 'AVP_ServerEntity_AcidDamage::~AVP_ServerEntity_AcidDamage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_AcidDamage::MakeClientUpdateMessage
// Author       : Andy Tate
// Date         : 29-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_AcidDamage::MakeClientUpdateMessage(void)
{
	Asura_Message*	pxMessage	= Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_AcidDamage_ClientUpdate));

	Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
}

/// End of function 'AVP_ServerEntity_AcidDamage::MakeClientUpdateMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_AcidDamage_VictimData::Update
// Author       : Andy Tate
// Date         : 3-Mar-2008
////////////////////////////////////////////////////////////////////////////////

bool AVP_AcidDamage_VictimData::Update(const AVP_ServerEntity_AcidDamage& xAcidDamage, 
									   const bool bWithinVolume,
									   const float fScale)
{
	const AVP_ServerEntity_PhysicalObject*	pxVictim	= AVP_ServerEntity_PhysicalObject::FindEntity(m_uVictimGuid);
	if(pxVictim == NULL)
	{
		// Victim has probably died and is set to be destroyed.
		return(true);
	}
	else
	{
		if(bWithinVolume)
		{
			const float	fTimeBetweenDamageApplications	= xAcidDamage.GetTimeBetweenDamageApplications();
			bool		bApplyDamage;

			// Check for special first-frame damage...
			if(m_fTimeSinceLastDamage < 0.0f)
			{
				m_fTimeSinceLastDamage	= fTimeBetweenDamageApplications;
				bApplyDamage			= true;
			}
			else
			{
				const float	fTimeStep	= Asura_Timers::GetGameTimeSlice();

				// Update the time since we were last damaged and check whether its time
				// for us to apply acid-damage to this entity again...
				m_fTimeSinceLastDamage	+= fTimeStep;

				bApplyDamage	= (m_fTimeSinceLastDamage >= fTimeBetweenDamageApplications);
			}

			while(bApplyDamage)
			{
				Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_ENTITYDAMAGE_REQUEST_VICTIM, sizeof(AVP_Message_EntityDamage_RequestVictim));

				// Generate the message to request damage for our victim...
				if(pxMessage != NULL)
				{
					AVP_Message_EntityDamage_RequestVictim*	pxData			= static_cast<AVP_Message_EntityDamage_RequestVictim*>(pxMessage->GetData());

					const Asura_Vector_3&					xVictimPosition	= pxVictim->GetPosition();
					const Asura_Guid						uAcidDamageGuid	= ASURA_GUID_UNREGISTERED;
					const float								fAmount			= xAcidDamage.GetAmountPerApplication() * fScale;

					Asura_Vector_3							xAcidDamagePosition;
					Asura_Vector_3							xAcidDamageNormal;
					Asura_Vector_3							xDirection;

					xAcidDamage.GetPosition(xAcidDamagePosition);
					xAcidDamage.GetSurfaceNormal(xAcidDamageNormal);

					xDirection	= xVictimPosition - xAcidDamagePosition;
					xDirection.Normalise();

					if(pxData != NULL)
					{
						const AVP_EntityDamage_RequestVictim	xDamage(uAVP_BLUEPRINT_NAME_DAMAGE_ACID,
																		uAcidDamageGuid,
																		ASURA_HASH_ID_UNSET,
																		m_uVictimGuid,
																		fAmount,
																		xAcidDamagePosition,
																		xDirection,
																		xAcidDamageNormal);

						pxData->SetDamage(xDamage);
					}

					pxMessage->SetFromGuid(uAcidDamageGuid);
					pxMessage->SetToGuid(m_uVictimGuid);
					Asura_Network::Send(pxMessage);
				}

				// Remove this time from our counter...
				m_fTimeSinceLastDamage	-= fTimeBetweenDamageApplications;
				bApplyDamage			 = (m_fTimeSinceLastDamage >= fTimeBetweenDamageApplications);
			}

			return(false);
		}
		else
		{
			// Outside of the volume - remove for now...
			return(true);
		}
	}
}

/// End of function 'AVP_AcidDamage_VictimData::Update'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_AcidDamage::Create
// Author       : Andy Tate
// Date         : 28-Feb-2008
////////////////////////////////////////////////////////////////////////////////

AVP_ServerEntity_AcidDamage* AVP_ServerEntity_AcidDamage::Create(const Asura_Message* pxMessage)
{
	const AVP_Message_AcidDamage_Create*	pxData			= static_cast<const AVP_Message_AcidDamage_Create*>(pxMessage->GetData());
	AVP_ServerEntity_AcidDamage*			pxAcidDamage	= new AVP_ServerEntity_AcidDamage (*pxData);

	// Forward this message to all my client entities
	pxAcidDamage->MakeCreateMessage(ASURA_GUID_UNREGISTERED, false);
	
	return(pxAcidDamage);
}

/// End of function 'AVP_ServerEntity_AcidDamage::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_AcidDamage::HandleMessage
// Author       : Andy Tate
// Date         : 29-Feb-2008
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerEntity_AcidDamage::HandleMessage(const Asura_Message* pxMessage)
{
	const u_int	uMessageID	= pxMessage->GetMsgID();

	switch(uMessageID)
	{
		case ASURA_MSG_CLIENT_REQUESTCREATEMSG:
		{
			MakeCreateMessage(pxMessage->GetFromGuid(), true);
			return(true);
		}

		default:
		{
			return(PARENT::HandleMessage(pxMessage));
		}
	}		// End of switch.
}

/// End of function 'AVP_ServerEntity_AcidDamage::HandleMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_AcidDamage::MakeCreateMessage
// Author       : Andy Tate
// Date         : 29-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_AcidDamage::MakeCreateMessage(Asura_Guid uRequestingClient,
													bool bSendGuaranteed)
{
	Asura_Message*	pxMessage	= Asura_Network::CreateMessage(ASURA_MSG_CLIENTCREATE, sizeof(AVP_Message_AcidDamage_Create), Classification);

	if(pxMessage == NULL)
	{
		Asura_Assert(false, "Couldn't create CreationMessage");
	}
	else
	{
		AVP_Message_AcidDamage_Create*	pxData	= static_cast<AVP_Message_AcidDamage_Create*>(pxMessage->GetData());

		pxData->SetSourceGuid(m_uSourceGuid);
		pxData->SetPosition(m_xPosition);
		pxData->SetSurfaceNormal(m_xSurfaceNormal);

		pxMessage->SetToGuid(ASURA_GUID_MY_CLIENT_ENTITIES);
		pxMessage->SetFromGuid(GetGuid());

		if (bSendGuaranteed)
		{
			pxMessage->UseGuaranteedDelivery();
		}

		Asura_Network::SendToSpecificMachine(pxMessage, uRequestingClient);
	}
}

/// End of function 'AVP_ServerEntity_AcidDamage::MakeCreateMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_AcidDamage::Update
// Author       : Andy Tate
// Date         : 28-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_AcidDamage::Update(void)
{
	const float	fTimeStep	= Asura_Timers::GetGameTimeSlice();

	// Call the base-function...
	PARENT::Update();

	// Update the timer...
	m_fTimeSinceCreation	+= fTimeStep;

	// Do we have a volume to check?
	if(m_pxVolume == NULL)
	{
		// No - might as well destroy ourself then...
		SetThisAndClientsAsDestroyed();
	}
	else
	{
		const float	fOneMinusScale	= (m_fTimeSinceCreation < m_fPoolDuration)
									? 0.0f
									: ((m_fTimeSinceCreation - m_fPoolDuration) / m_fPoolFadeOutDuration);

		if(fOneMinusScale >= 1.0f)
		{
			// We've run out of time...
			SetThisAndClientsAsDestroyed();
		}
		else
		{
			const float											fScale		= 1.0f - fOneMinusScale;
			const Asura_Hashed_List<AVP_ServerEntity_Actor>*	pxActorList	= AVP_ServerEntity_Actor::GetList();

			Asura_Hashed_List_ConstIt<AVP_ServerEntity_Actor>	xActorIterator(pxActorList);

			// Check for actors who've strayed into this volume...
			while(!xActorIterator.Done())
			{
				const AVP_ServerEntity_Actor*	pxActor	= xActorIterator.GetCurrent();

				if(pxActor != NULL)
				{
					const Asura_Vector_3&		xPosition		= pxActor->GetPosition();
					const Asura_Guid			uVictimGuid		= pxActor->GetGuid();
					AVP_AcidDamage_VictimData*	pxVictimData	= m_xVictimTree.GetDataByHash(uVictimGuid);
					const bool					bWithinVolume	= m_pxVolume->Contains(xPosition);

					// Do we already have this victim?
					if(pxVictimData == NULL)
					{
						if(bWithinVolume)
						{
							const AVP_AcidDamage_VictimData	xVictimData(uVictimGuid);

							// No - but they are now within the volume so add them to the list...
							pxVictimData	= m_xVictimTree.AddNode(uVictimGuid, xVictimData);
						}
					}
					else
					{
						if(pxVictimData->Update(*this, bWithinVolume, fScale))
						{
							// We've finished with this victim - remove them...
							m_xVictimTree.DeleteNodeByHash(uVictimGuid);
						}
					}
				}

				// Try the next actor...
				xActorIterator.Next();
			}
		}
	}
}

/// End of function 'AVP_ServerEntity_AcidDamage::Update'. ///


/// End of file. ///
