//
// AVP_ServerEntity_GameMode.h
//
// Created by JG - 04 July 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_SERVERENTITY_GAMEMODE_H__
#define __AVP_SERVERENTITY_GAMEMODE_H__

// Engine
#include "Entity/Triggers/Asura_ServerEntity_Activatable.h"

// Project
#include "Core/AVP_Settings.h"

// Forward declarations
class Asura_Chunk_ChunkBuffer_Stream; 

//----------------------------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////////////////////////
//  CLASS    :  AVP_ServerEntity_GameMode
//  CREATED  :  JG (7.7.2008)
//  DESC     :  Entity container for game modes in maps
//////////////////////////////////////////////////////////////////////////////////////////
//----------------------------------------------------------------------------------------
class AVP_ServerEntity_GameMode: public Asura_ServerEntity_ActivatableWithPosition
{
public:
	// Constructor / destructor
	AVP_ServerEntity_GameMode( Asura_Guid uGuid );
	virtual ~AVP_ServerEntity_GameMode(); 

	// Override of HasClientEntities() - we don't have any.
	virtual bool HasClientEntities() const { return false; }

	static AVP_ServerEntity_GameMode* CreateFromChunkStream(Asura_Guid uGuid, Asura_Chunk_ChunkBuffer_Stream& xStream);

	// Static list functions...
	static AVP_ServerEntity_GameMode*							FindEntity	(const Asura_Guid uGuid){ return(s_xList.FindEntity(uGuid)); }
	static const Asura_Hashed_List<AVP_ServerEntity_GameMode>*	GetList		(void)					{ return(&s_xList); }

	void SendActivateMessage(void);
	virtual bool HandleMessage(const Asura_Message* pxMessage);

	virtual bool IsFilledFromChunkBuffer() const {return false;}
	virtual bool CanBeSavedToChunk() const {return true;}

	virtual void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	AVP_Settings::GameType		GetGameMode( void )	const		{ return m_eGameMode; }

	void						SetPosition( Asura_Vector_3& xPosition )			{ m_xPosition = xPosition; }
	void						SetGameMode( AVP_Settings::GameType eGameMode )		{ m_eGameMode = eGameMode; }

protected:
	AVP_Settings::GameType		m_eGameMode; 

	static const u_int uGAMEMODE_CHUNK_VERSION = 0;

private:
	typedef	Asura_ServerEntity_ActivatableWithPosition		PARENT; 

	// Friend classes...
#ifdef ASURA_TOOLS
	friend class ToolsAVP_Entity_GameMode;
#endif

	// Private variables...

	static Asura_Hashed_List<AVP_ServerEntity_GameMode> s_xList;

};

#endif//__AVP_SERVERENTITY_GAMEMODE_H__