//------------------------------------------------------------------------------
// File		: AVP_ServerNode_GuardZone.cpp
// Created	: 06 / 03 / 08
// Author	: Gareth Thomas
// Copyright (c) 2007 Rebellion. All rights reserved.
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Includes
//------------------------------------------------------------------------------
// Core
#include "Core/Asura.h"

// This file's header.
#include "Entity/Locations/GuardZone/AVP_ServerNode_GuardZone.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

// AVP
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Locations/OrientedNode/AVP_ServerNode_OrientedNode.h"

//------------------------------------------------------------------------------
// Statics and Constants
//------------------------------------------------------------------------------

Asura_Hashed_List<AVP_ServerNode_GuardZone> AVP_ServerNode_GuardZone::s_xList(2);


//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::AVP_ServerNode_GuardZone
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
AVP_ServerNode_GuardZone::AVP_ServerNode_GuardZone(Asura_Guid uGuid)
: PARENT(uGuid)
, m_iPatrolType(0)
, m_uPingPongDirection(0)
{
	Classification = AVP_ENTITYCLASS_GUARDZONE;

	s_xList.AddToList(this);
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::~AVP_ServerNode_GuardZone
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
AVP_ServerNode_GuardZone::~AVP_ServerNode_GuardZone()
{
	s_xList.RemoveFromList(this);
}


//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::Create
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
AVP_ServerNode_GuardZone* AVP_ServerNode_GuardZone::Create(const Asura_Guid uGuid)
{
	return(new AVP_ServerNode_GuardZone(uGuid));
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::CreateFromChunkBuffer
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
void AVP_ServerNode_GuardZone::CreateFromChunkBuffer( Asura_Guid uGUID, const char*& pcChunkBfr )
{
	AVP_ServerNode_GuardZone* pxNode = new AVP_ServerNode_GuardZone( uGUID );
	if( pxNode )
	{
		pxNode->InitialiseFromChunkBuffer( pcChunkBfr );
	}
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::InitialiseFromChunkBuffer
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
void AVP_ServerNode_GuardZone::InitialiseFromChunkBuffer( const char*& pcChunkBfr )
{
	// In order to convert all Asura nodes already in game into our simple nodes
	// peek at the version. If it's less than 1000 than it hasn't been Converted yet, so ignore our chunk
	int iVersion;
	ASURA_CHUNK_PEEK( pcChunkBfr, iVersion, int );
	if ( iVersion < 1000 )
	{
		//Just do Parent
		Asura_Chunk_ChunkBuffer_Stream xStream( pcChunkBfr );
		PARENT::ReadFromChunkStream( xStream );
	}
	else
	{
		//Version
		ASURA_CHUNK_EXTRACT( pcChunkBfr, iVersion, int );

		//Parent
		Asura_Chunk_ChunkBuffer_Stream xStream( pcChunkBfr );
		PARENT::ReadFromChunkStream( xStream );

		//our data
	}
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::WriteToChunkStream
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
void AVP_ServerNode_GuardZone::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//Version
	xStream << CURRENT_CHUNK_VERSION;

	//Parent
	PARENT::WriteToChunkStream( xStream );

	//our data
	xStream << m_iPatrolType;
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::ReadFromChunkStream
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
void AVP_ServerNode_GuardZone::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	//Version
	u_int	uVersion	= CURRENT_CHUNK_VERSION + 1;

	// Read the data...
	xStream	>> uVersion;

	//Parent
	PARENT::ReadFromChunkStream( xStream );

	//our data
	xStream >> m_iPatrolType;
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::GetNearestGuardPoint
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
AVP_ServerNode_OrientedNode* AVP_ServerNode_GuardZone::GetNearestGuardPoint(const Asura_Vector_3 xPosition)
{
	//check we have guardpoints
	if ( m_iNumGuardPoints == 0 )
	{
		Asura_Assert( false, "AVP_ServerNode_GuardZone::GetNearestGuardPoint - GuardZone has no guardpoints" );
		return NULL;
	}

	AVP_ServerNode_OrientedNode*	pxBestPatrolPoint	= NULL;
	float							fBestDistanceSq		= Asura_Maths::LargeFloat;

	for (u_int u = 0; u < (u_int)m_iNumGuardPoints; ++u)
	{
		AVP_ServerNode_OrientedNode* pxPatrolPoint = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[u]);

		if (pxPatrolPoint)
		{
			if (pxPatrolPoint->IsOccupied())
				continue;

			float fDistanceSq = (xPosition - pxPatrolPoint->GetPosition()).MagnitudeSquared();
			
			if (fDistanceSq < fBestDistanceSq)
			{
				fBestDistanceSq		= fDistanceSq;
				pxBestPatrolPoint	= pxPatrolPoint;
			}
		}
	}

	return pxBestPatrolPoint;
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::GetRandomUnoccupiedGuardPoint
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
AVP_ServerNode_OrientedNode* AVP_ServerNode_GuardZone::GetRandomGuardPoint()
{
	if ( m_iNumGuardPoints < 1 )
	{
		Asura_Assert( false, "GuardZone::GetRandomGuardPoint - GuardZone has no Patrol Points" );
		return NULL;
	}

	AVP_ServerNode_OrientedNode*	pxSelectedPoint	= NULL;
	bool							bFound			= false;
	u_int							uCount			= 0;
	const u_int						uCountLimit		= m_iNumGuardPoints * 2;
	
	while ((!bFound) && (uCount < uCountLimit))
	{
		const u_int						uRandomIndex	= Asura_Random::GetIntU(0, m_iNumGuardPoints - 1);
		AVP_ServerNode_OrientedNode*	pxPatrolPoint	= AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[uRandomIndex]);

		// Increase our get out clause
		++uCount;

		if (pxPatrolPoint)
		{
			if (pxPatrolPoint->IsOccupied())
				continue;

			pxSelectedPoint = pxPatrolPoint;
			bFound = true;
		}
	}

	return pxSelectedPoint;
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::GetNextLoopingPatrolOrderPos
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
u_int AVP_ServerNode_GuardZone::GetNextLoopingPatrolOrderPos(u_int uPatrolOrderPos)
{
	u_int uCurrentPos = uPatrolOrderPos;

	// find the highest patrol position
	u_int uHighestPatrolPos = 0;

	for (u_int u = 0; u < (u_int)m_iNumGuardPoints; ++u)
	{
		AVP_ServerNode_OrientedNode* pxNode = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[u]);

		if (pxNode)
		{
			if (pxNode->IsOccupied())
				continue;

			if (pxNode->GetPatrolOrder() > uHighestPatrolPos)
			{
				uHighestPatrolPos = pxNode->GetPatrolOrder();
			}
		}
	}

	// Shift the patrol point on
	if (uCurrentPos >= uHighestPatrolPos)
	{
		// find the lowest
		u_int uLowetPatrolOrderPos = 0xffffffff;

		for (u_int u = 0; u < (u_int)m_iNumGuardPoints; ++u)
		{
			AVP_ServerNode_OrientedNode* pxNode = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[u]);

			if (pxNode)
			{
				if (pxNode->IsOccupied())
					continue;

				if (pxNode->GetPatrolOrder() < uLowetPatrolOrderPos)
				{
					uLowetPatrolOrderPos = pxNode->GetPatrolOrder();
				}
			}
		}

		return uLowetPatrolOrderPos;
	}
	else
	{
		u_int uBestPatrolPos = 0xffffffff;
		
		//get the next
		for (u_int u = 0; u < (u_int)m_iNumGuardPoints; ++u)
		{
			AVP_ServerNode_OrientedNode* pxNode = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[u]);

			if (pxNode)
			{
				if (pxNode->IsOccupied())
					continue;

				if ((pxNode->GetPatrolOrder() > uCurrentPos) && (pxNode->GetPatrolOrder() < uBestPatrolPos))
				{
					uBestPatrolPos = pxNode->GetPatrolOrder();
				}
			}
		}

		return uBestPatrolPos;
	}
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::GetNextLoopingGuardPoint
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
AVP_ServerNode_OrientedNode* AVP_ServerNode_GuardZone::GetNextLoopingGuardPoint(const Asura_Guid uCurrentGPGuid)
{
	//check we have guardpoints
	if (m_iNumGuardPoints < 1)
	{
		Asura_Assert(false, "GuardZone::GetNextLoopingGuardPoint - GuardZone has no guardpoints");
		return NULL;
	}

	AVP_ServerNode_OrientedNode*	pxCurrentPatrolPoint	= AVP_ServerNode_OrientedNode::FindEntity(uCurrentGPGuid);
	AVP_ServerNode_OrientedNode*	pxSelectedPatrolPoint	= NULL;
	u_int							uPatrolOrderPos			= 0;

	// Get the patrol order of our current patrol point
	if (pxCurrentPatrolPoint)
	{
		uPatrolOrderPos = pxCurrentPatrolPoint->GetPatrolOrder();
	}
	else
	{
		Asura_Assert(false, "GuardZone::GetNextLoopingGuardPoint - Couldn't Find Our Current Patrol Point");
		return NULL;
	}

	uPatrolOrderPos = GetNextLoopingPatrolOrderPos(uPatrolOrderPos);

	for (u_int u = 0; u < (u_int)m_iNumGuardPoints; ++u)
	{
		AVP_ServerNode_OrientedNode* pxPatrolPoint = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[u]);

		if (pxPatrolPoint)
		{
			if (pxPatrolPoint->GetPatrolOrder() == uPatrolOrderPos)
			{
				pxSelectedPatrolPoint = pxPatrolPoint;
				break;
			}
		}
	}

	return pxSelectedPatrolPoint;
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::GetNextLoopingPatrolOrderPos
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
u_int AVP_ServerNode_GuardZone::GetNextPingPongPatrolOrderPos(u_int uPatrolOrderPos)
{
	u_int uCurrentPos = uPatrolOrderPos;

	// find the highest patrol position
	u_int uHighestPatrolPos	= 0;
	u_int uLowestPatrolPos	= 0xffffffff;

	for (u_int u = 0; u < (u_int)m_iNumGuardPoints; ++u)
	{
		AVP_ServerNode_OrientedNode* pxNode = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[u]);

		if (pxNode)
		{
			if (pxNode->IsOccupied())
				continue;

			if (pxNode->GetPatrolOrder() > uHighestPatrolPos)
			{
				uHighestPatrolPos = pxNode->GetPatrolOrder();
			}

			if (pxNode->GetPatrolOrder() < uLowestPatrolPos)
			{
				uLowestPatrolPos = pxNode->GetPatrolOrder();
			}
		}
	}

	// Shift the patrol point on
	if (uCurrentPos >= uHighestPatrolPos)
	{
		m_uPingPongDirection = 1;
	}
	else if (uPatrolOrderPos == 0)
	{
		m_uPingPongDirection = 0;
	}

	if (m_uPingPongDirection == 1)
	{
		int iBestPatrolPos = -1;

		//get the next
		for (int i = 0; i < m_iNumGuardPoints; ++i)
		{
			AVP_ServerNode_OrientedNode* pxNode = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[i]);

			if (pxNode)
			{
				if (pxNode->IsOccupied())
					continue;

				if ((pxNode->GetPatrolOrder() < uCurrentPos) && ((int)pxNode->GetPatrolOrder() > iBestPatrolPos))
				{
					iBestPatrolPos = pxNode->GetPatrolOrder();
				}
			}
		}

		// If we didn't find one in order find the first free one
		if (iBestPatrolPos == -1)
		{
			for (int i = 0; i < m_iNumGuardPoints; ++i)
			{
				AVP_ServerNode_OrientedNode* pxNode = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[i]);

				if (pxNode)
				{
					if (pxNode->IsOccupied())
						continue;
					
					iBestPatrolPos = pxNode->GetPatrolOrder();
					break;
				}
			}
		}
		
		// If we still didn't find one just set to the current patrol order
		if (iBestPatrolPos == -1)
		{
			iBestPatrolPos = (int)uPatrolOrderPos;
		}

		return ((u_int)iBestPatrolPos);
	}
	else
	{
		int iBestPatrolPos = m_iNumGuardPoints + 1;

		//get the next
		for (int i = 0; i < m_iNumGuardPoints; ++i)
		{
			AVP_ServerNode_OrientedNode* pxNode = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[i]);

			if (pxNode)
			{
				if (pxNode->IsOccupied())
					continue;

				if ((pxNode->GetPatrolOrder() > uCurrentPos) && ((int)pxNode->GetPatrolOrder() < iBestPatrolPos))
				{
					iBestPatrolPos = pxNode->GetPatrolOrder();
				}
			}
		}

		// If we didn't find one in order find the first free one
		if (iBestPatrolPos == m_iNumGuardPoints + 1)
		{
			for (int i = 0; i < m_iNumGuardPoints; ++i)
			{
				AVP_ServerNode_OrientedNode* pxNode = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[i]);

				if (pxNode)
				{
					if (pxNode->IsOccupied())
						continue;

					iBestPatrolPos = pxNode->GetPatrolOrder();
					break;
				}
			}
		}

		// If we still didn't find one just set to the current patrol order
		if (iBestPatrolPos == m_iNumGuardPoints + 1)
		{
			iBestPatrolPos = (int)uPatrolOrderPos;
		}

		return ((u_int)iBestPatrolPos);
	}
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::GetNextPingPongGuardPoint
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
AVP_ServerNode_OrientedNode* AVP_ServerNode_GuardZone::GetNextPingPongGuardPoint(const Asura_Guid uCurrentGPGuid)
{
	//check we have guardpoints
	if (m_iNumGuardPoints == 0)
	{
		Asura_Assert( false, "GuardZone::GetRandomUnoccupiedGuardPoint - GuardZone has no guardpoints" );
		return NULL;
	}

	AVP_ServerNode_OrientedNode*	pxCurrentPatrolPoint	= AVP_ServerNode_OrientedNode::FindEntity(uCurrentGPGuid);
	AVP_ServerNode_OrientedNode*	pxSelectedPatrolPoint	= NULL;
	u_int							uPatrolOrderPos			= 0;

	// Get the patrol order of our current patrol point
	if (pxCurrentPatrolPoint)
	{
		uPatrolOrderPos = pxCurrentPatrolPoint->GetPatrolOrder();
	}
	else
	{
		Asura_Assert(false, "GuardZone::GetNextLoopingGuardPoint - Couldn't Find Our Current Patrol Point");
		return NULL;
	}

	uPatrolOrderPos = GetNextPingPongPatrolOrderPos(uPatrolOrderPos);

	for (u_int u = 0; u < (u_int)m_iNumGuardPoints; ++u)
	{
		AVP_ServerNode_OrientedNode* pxPatrolPoint = AVP_ServerNode_OrientedNode::FindEntity(m_puGuardPointGuids[u]);

		if (pxPatrolPoint)
		{
			if (pxPatrolPoint->IsOccupied())
				continue;

			if (pxPatrolPoint->GetPatrolOrder() == uPatrolOrderPos)
			{
				pxSelectedPatrolPoint = pxPatrolPoint;
				break;
			}
		}
	}

	return pxSelectedPatrolPoint;
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::GetGuidFromGPPatrolOrder
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
Asura_Guid AVP_ServerNode_GuardZone::GetGuidFromGPPatrolOrder( const u_int uPatrolOrder ) const
{
	AVP_ServerNode_OrientedNode* pxGuardPoint = NULL;
	for ( int i = 0; i < m_iNumGuardPoints; ++i )
	{
		pxGuardPoint = AVP_ServerNode_OrientedNode::FindEntity( m_puGuardPointGuids[ i ] );

		if ( uPatrolOrder == pxGuardPoint->GetPatrolOrder() )
		{
			return m_puGuardPointGuids[ i ];
		}
	}
	return ASURA_GUID_UNREGISTERED;
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::IsBaseOrDerivativeOf
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
#ifdef ASURA_TOOLS
bool AVP_ServerNode_GuardZone::IsBaseOrDerivativeOf( u_short usClassification ) const
{
	if ( usClassification == AVP_ENTITYCLASS_GUARDZONE )
	{
		return true;
	}
	return PARENT::IsBaseOrDerivativeOf(usClassification);
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::ApplyChangesForTemplateInstance
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
void AVP_ServerNode_GuardZone::ApplyChangesForTemplateInstance( class Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance(pxInstance);
}

//------------------------------------------------------------------------------
// Function     : AVP_ServerNode_GuardZone::CanBeIncludedInTemplate
// Author       : Gareth Thomas
// Date         : 06 / 03 / 08
//------------------------------------------------------------------------------
bool AVP_ServerNode_GuardZone::CanBeIncludedInTemplate() const 
{
	return ( Classification == AVP_ENTITYCLASS_GUARDZONE );
}

bool AVP_ServerNode_GuardZone::RemoveGuardPoint( Asura_Guid uGuid )
{
	const AVP_ServerNode_OrientedNode* pxPointToRemove = AVP_ServerNode_OrientedNode::FindEntity( uGuid );
	const int iRemovedPatrolOrder = pxPointToRemove ? pxPointToRemove->GetPatrolOrder() : -1;

	if ( PARENT::RemoveGuardPoint( uGuid ) )
	{
		// Fix the patrol order.
		if ( iRemovedPatrolOrder != -1 )
		{
			for ( int iPoint = 0; iPoint < GetNumGuardPoints(); ++iPoint )
			{
				AVP_ServerNode_OrientedNode* pxPoint = AVP_ServerNode_OrientedNode::FindEntity( GetGuardPointGuid( iPoint ) );
				if ( pxPoint && pxPoint->GetPatrolOrder() > static_cast< u_int >( iRemovedPatrolOrder ) )
				{
					pxPoint->SetPatrolOrder( pxPoint->GetPatrolOrder() - 1 );
				}
			}
		}

		return true;
	}
	else
	{
		return false;
	}
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
//
//////////////////////////////////////////////////////////////////////////////
