//--------------------------------------------------------------------------------------
//	File	:	AVP_ServerEntity_LookAt.cpp
//	Author	:	Gareth Thomas
//	Created	:	18 / 12 / 2007
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//--------------------------------------------------------------------------------------

// Core
#include "Core/Asura.h"

// Header
#include "Entity/Locations/LookAt/AVP_ServerEntity_LookAt.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

// AVP
#include "Entity/AVP_Entity_Classification.h"

// tools
#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//------------------------------------------------------------------------------
// Constants, Defines, Statics and Globals
//------------------------------------------------------------------------------
Asura_Hashed_List<AVP_ServerEntity_LookAt> AVP_ServerEntity_LookAt::s_xList;

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
AVP_ServerEntity_LookAt::AVP_ServerEntity_LookAt(const Asura_Guid uGuid) : PARENT(uGuid)
, m_uLinkedLookAt(ASURA_GUID_UNREGISTERED)
{
	Classification = AVP_ENTITYCLASS_LOOKAT;
		
	SetNeverUpdateClient();
	SetNeverUpdates(true);
	s_xList.AddToList(this);
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
AVP_ServerEntity_LookAt::~AVP_ServerEntity_LookAt(void)
{
	s_xList.RemoveFromList(this);
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_LookAt::PostLoadInitialise(void)
{
	PARENT::PostLoadInitialise();
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_LookAt::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	//Version
	xStream << CURRENT_CHUNK_VERSION;

	//Parent
	PARENT::WriteToChunkStream(xStream);

	xStream << m_uLinkedLookAt;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_LookAt::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//Version
	u_int uVersion;
	xStream	>> uVersion;

	//Parent
	PARENT::ReadFromChunkStream(xStream);

	xStream >> m_uLinkedLookAt;
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
AVP_ServerEntity_LookAt* AVP_ServerEntity_LookAt::Create(const Asura_Guid uGuid)
{
	return(new AVP_ServerEntity_LookAt(uGuid));
}

#ifdef ASURA_TOOLS
void AVP_ServerEntity_LookAt::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance );

	m_uLinkedLookAt = pxInstance->ConvertGuid( m_uLinkedLookAt );
}
#endif
