//--------------------------------------------------------------------------------------
//	File	:	AVP_ServerNode_Objective.cpp
//	Author	:	Gareth Thomas
//	Created	:	23 / 01 / 2008
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//--------------------------------------------------------------------------------------

// Core
#include "Core/Asura.h"

// Header
#include "Entity/Locations/Objective/AVP_ServerNode_Objective.h"

// Asura
#include "ChunkSystem/Asura_Chunk_Stream.h"

// AVP
#include "Entity/AVP_Entity_Classification.h"

//------------------------------------------------------------------------------
// Constants, Defines, Statics and Globals
//------------------------------------------------------------------------------
Asura_Hashed_List<AVP_ServerNode_Objective> AVP_ServerNode_Objective::s_xList;

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
AVP_ServerNode_Objective::AVP_ServerNode_Objective(const Asura_Guid uGuid)
: PARENT(uGuid)
, m_pxVolumeGroup(new Asura_VolumeGroup)
, m_iNumVolumes(0)
, m_uGamePlayVolumeGuid(ASURA_GUID_UNREGISTERED)
, m_bUseSpecificGamePlayVolume(0)
{
	Classification = AVP_ENTITYCLASS_AI_OBJECTIVE;

	SetNeverUpdateClient();
	SetNeverUpdates(true);
	s_xList.AddToList(this);
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
AVP_ServerNode_Objective::~AVP_ServerNode_Objective(void)
{
	if (m_pxVolumeGroup)
	{
		delete m_pxVolumeGroup;
		m_pxVolumeGroup = NULL;
	}

	s_xList.RemoveFromList(this);
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
void AVP_ServerNode_Objective::PostLoadInitialise(void)
{
	PARENT::PostLoadInitialise();
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
void AVP_ServerNode_Objective::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	//Version
	xStream << CURRENT_CHUNK_VERSION;

	xStream << m_uGamePlayVolumeGuid;
	xStream << m_bUseSpecificGamePlayVolume;

	xStream << m_iNumVolumes;
	m_pxVolumeGroup->WriteToChunkStream(xStream);

	//Parent
	PARENT::WriteToChunkStream(xStream);
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
void AVP_ServerNode_Objective::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//Version
	u_int uVersion;
	xStream	>> uVersion;

	if (uVersion == CURRENT_CHUNK_VERSION)
	{
		xStream >> m_uGamePlayVolumeGuid;
		xStream >> m_bUseSpecificGamePlayVolume;
		xStream >> m_iNumVolumes;
		m_pxVolumeGroup->ReadFromChunkStream(xStream);
	}

	//Parent
	PARENT::ReadFromChunkStream(xStream);
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
AVP_ServerNode_Objective* AVP_ServerNode_Objective::Create(const Asura_Guid uGuid)
{
	return(new AVP_ServerNode_Objective(uGuid));
}
