//--------------------------------------------------------------------------------------
//	File	:	AVP_ServerNode_Objective.h
//	Author	:	Gareth Thomas
//	Created	:	23 / 01 / 2008
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
// BEGIN FILE WRAPPER
#ifdef ASURA_USE_PRAGMA_ONCE 
#pragma once
#endif
#ifndef __AVP_SERVERNODE_OBJECTIVE_H__
#define __AVP_SERVERNODE_OBJECTIVE_H__ 1
// END FILE WRAPPER
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
//	INCLUDES
//--------------------------------------------------------------------------------------
#include "Entity/Locations/Asura_ServerEntity_Node.h"
#include "Volume/Asura_Volume.h"
#include "Volume/Asura_VolumeGroup.h"

//--------------------------------------------------------------------------------------
//	Class	:	AVP_ServerNode_Objective
//	Created	:	23 / 01 / 08
//	Author	:	Gareth Thomas
//--------------------------------------------------------------------------------------
class AVP_ServerNode_Objective : public Asura_ServerEntity_Node
{

public:

	// Public Overridden functions...
	virtual void						WriteToChunkStream						( Asura_Chunk_Stream& xStream ) const;
	virtual void						ReadFromChunkStream						( Asura_Chunk_Stream& xStream );
	virtual bool 						CanBeSavedToChunk						( void ) const						{ return ( true ); }
	virtual bool						IsFilledFromChunkBuffer					( void ) const						{ return ( false ); }
	virtual bool						HasClientEntities						( void ) const						{ return ( false ); }
	virtual void						PostLoadInitialise						( void );

	// Public Static Functions...
	static AVP_ServerNode_Objective*								Create		( const Asura_Guid uGuid );
	static AVP_ServerNode_Objective*								FindEntity	( Asura_Guid uGuid )				{ return s_xList.FindEntity(uGuid); }
	static const Asura_Hashed_List<AVP_ServerNode_Objective>*		GetList		( void )							{ return &s_xList; }
    
	// Volume functions
	// Public Volume Functions
	virtual bool		Contains				(const Asura_Vector_3& xPosition) const			{return m_pxVolumeGroup->Contains(xPosition);}
	u_int				GetNumberOfVolumes		(void) const									{return m_pxVolumeGroup->GetNumberOfVolumes();}
	Asura_Quat*			GetVolumeOrient			(u_int uIndex) const							{return m_pxVolumeGroup->GetVolumeOrient(uIndex);}
	Asura_Volume*		GetVolume				(u_int uIndex)									{return m_pxVolumeGroup->GetVolume(uIndex);}
	void				AddVolume				(Asura_Volume* pxVolume)						{m_pxVolumeGroup->AddVolume(pxVolume);}
	void				RemoveVolume			(Asura_Volume* pxVolume)						{m_pxVolumeGroup->RemoveAndDestroyVolume(pxVolume);}
	void				RemoveVolume			(u_int uIndex)									{m_pxVolumeGroup->RemoveAndDestroyVolume(uIndex);}
	void				DisplaceVolumes			(Asura_Vector_3& xOffset)						{m_pxVolumeGroup->DisplaceVolumes(xOffset);}
	Asura_VolumeGroup*	GetVolumes(void) const													{return m_pxVolumeGroup;}
	int					GetNumVolumes(void) const												{return m_iNumVolumes;}
	Asura_Guid			GetGamePlayVolumeGuid(void)												{return m_uGamePlayVolumeGuid;}
	void				SetGamePlayVolumeGuid(Asura_Guid uGuid)									{m_uGamePlayVolumeGuid = uGuid;}
	u_int				GetShouldUseSpecificGPV(void)											{return m_bUseSpecificGamePlayVolume;}
	void				SetShouldUseSpecificGPV(u_int bVal)										{m_bUseSpecificGamePlayVolume = bVal;}

protected:

	Asura_VolumeGroup*			m_pxVolumeGroup; // Container class for Asura_Volumes
	int							m_iNumVolumes;
	Asura_Guid					m_uGamePlayVolumeGuid;
	u_int						m_bUseSpecificGamePlayVolume;

	// Construction / Destruction
	AVP_ServerNode_Objective(const Asura_Guid uGuid);
	virtual ~AVP_ServerNode_Objective(void);

private:

#ifdef ASURA_TOOLS
	// Friend classes
	friend class ToolsAVP_Objective_VolumeEdit_Controller;
	friend class ToolsAVP_Node_Objective;
#endif 

	// Private typedefs
	typedef Asura_ServerEntity_Node PARENT;

	// Chunk Version
	enum { CURRENT_CHUNK_VERSION = 1 };

	// Private statics
	static Asura_Hashed_List < AVP_ServerNode_Objective > s_xList;
};

#endif   // __AVP_SERVERNODE_OBJECTIVE_H__
