//--------------------------------------------------------------------------------------
//	File	:	AVP_ServerNode_Objective_Marine.h
//	Author	:	Gareth Thomas
//	Created	:	23 / 01 / 2008
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
// BEGIN FILE WRAPPER
#ifdef ASURA_USE_PRAGMA_ONCE 
#pragma once
#endif
#ifndef __AVP_SERVERNODE_OBJECTIVE_MARINE_H__
#define __AVP_SERVERNODE_OBJECTIVE_MARINE_H__ 1
// END FILE WRAPPER
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
//	INCLUDES
//--------------------------------------------------------------------------------------
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Locations/Objective/AVP_ServerNode_Objective.h"

//----------------------------------------------------------------------
// Structure :: Objective
// Used to store information about aliens that will be spawned
//----------------------------------------------------------------------
struct Objective
{
	Asura_Guid			uObjectiveGuid;
	u_int				uObjectiveType;
	u_int				uObjectiveState;
	u_int				uObjectiveThreatAction;
	u_int				uObjectiveMovementMode;

	u_int				GetObjectiveType( void )					{ return uObjectiveType; }
	Asura_Guid			GetObjectiveGuid( void )					{ return uObjectiveGuid; }
	u_int				GetObjectiveState( void )					{ return uObjectiveState; }
	u_int				GetObjectiveThreatAction( void )			{ return uObjectiveThreatAction; }
	u_int				GetObjectiveMovementMode( void )			{ return uObjectiveMovementMode; }
	void				SetObjectiveState( u_int uState )			{ uObjectiveState = uState; }
};

//--------------------------------------------------------------------------------------
//	Class	:	AVP_ServerNode_Objective_Marine
//	Created	:	23 / 01 / 08
//	Author	:	Gareth Thomas
//--------------------------------------------------------------------------------------
class AVP_ServerNode_Objective_Marine : public AVP_ServerNode_Objective
{

public:

	// Objective Type - Description of the Objective
// 	enum ObjectiveType
// 	{
// 		MOVE_TO_OBJECTIVE_SLOW_AND_STOP_TO_ENGAGE,
// 		MOVE_TO_OBJECTIVE_MEDIUM_DONT_STOP_TO_ENGAGE,
// 		MOVE_TO_OBJECTIVE_MEDIUM_STOP_TO_ENGAGE,
// 		MOVE_TO_OBJECTIVE_SLOW_AND_SCAN_AREA_STOP_TO_ENGAGE,
// 		MAX_OBJECTIVE_TYPES,
// 	};

	enum ObjectiveMovementModes
	{
		MOVE_SLOW_STOP_TO_ENGAGE,
		MOVE_SLOW_DONT_STOP_TO_ENGAGE,
		MOVE_MEDIUM_STOP_TO_ENGAGE,
		MOVE_MEDIUM_DONT_STOP_TO_ENGAGE,
		ADVANCE_SLOW,
		ADVANCE_MEDIUM,
		RETREAT_SLOW,
		RETREAT_MEDIUM,
		MAX_OBJECTIVE_MOVEMENT_MODES_TYPES,
	};

	// Objective State - This is the current State of the objective
	enum ObjectiveState
	{
		NOT_STARTED,
		MOVING_TO_OBJECTIVE,
		ARRIVED_AT_OBJECTIVE,
		WAITING_FOR_COMPLETION,
		COMPLETE,
		MAX_OBJECTIVE_STATES
	};

	// ObjectiveThreatAction - Action that members should do on acquiring a threat while en route to their objective
	enum ObjectiveThreatAction 
	{
		NO_ACTION,
		STOP_TO_ENGAGE,
		MAX_THREAT_ACTIONS
	};

	// Public Overridden functions...
	virtual void						WriteToChunkStream						( Asura_Chunk_Stream& xStream ) const;
	virtual void						ReadFromChunkStream						( Asura_Chunk_Stream& xStream );
	virtual bool 						CanBeSavedToChunk						( void ) const						{ return ( true ); }
	virtual bool						IsFilledFromChunkBuffer					( void ) const						{ return ( false ); }
	virtual bool						HasClientEntities						( void ) const						{ return ( false ); }
	virtual void						PostLoadInitialise						( void );

	virtual bool						HandleMessage(const Asura_Message* pxMessage);

	void				SendObjectiveReachedMessage(void);
	void				Reset(void);

	void				SetShouldUseSpecificGameScenes(u_int bVal)		{ m_bUseSpecificGameSceneNodes = bVal; }
	u_int				GetShouldUseSpecificGameScenes(void)			{ return m_bUseSpecificGameSceneNodes; }
	Asura_Guid			GetRequiredActionGuid(u_int uIndex)				{ return m_pxRequiredGameScenes[ uIndex ]; }
	Asura_Guid			GetOptionalActionGuid(u_int uIndex)				{ return m_pxOptionalGameScenes[ uIndex ]; }
	u_int				GetNumRequiredActions(void)						{ return m_uNumRequiredGameScenes; }
	u_int				GetNumOptionalActions(void)						{ return m_uNumOptionalGameScenes; }

	void				SetRequiredGuids(Asura_Guid* pxGuids, u_int uNumGuids);
	void				SetOptionalGuids(Asura_Guid* pxGuids, u_int uNumGuids);

	void				SetRunToAction(u_int bVal)									{ m_bRunToAction = bVal; }
	u_int				GetRunToAction(void)										{ return m_bRunToAction;}

	void				SetObjectiveCompleteOnMessageOnly(u_int bVal)				{ m_bObjectiveCompleteOnMessage = bVal; }
	u_int				GetObjectiveCompleteOnMessageOnly(void)						{ return m_bObjectiveCompleteOnMessage; }

	bool				GetHasReceivedCompleteMessage(void)							{ return m_bObjectiveHasReceivedCompleteMessage; }
	void				SetHasReceivedCompleteMessage(bool bVal)					{ m_bObjectiveHasReceivedCompleteMessage = bVal; }
	void				SendCompleteEvent(void);

	// Public Static Functions...
	static AVP_ServerNode_Objective_Marine*								Create		( const Asura_Guid uGuid );
	static AVP_ServerNode_Objective_Marine*								FindEntity	( Asura_Guid uGuid )				{ return s_xList.FindEntity(uGuid); }
	static const Asura_Hashed_List<AVP_ServerNode_Objective_Marine>*	GetList		( void )							{ return &s_xList; }
//	static const char**	GetObjectiveTypeNames( void )								{ return s_aszObjectiveTypeNames; }
	static const char**	GetObjectiveMoveModeNames( void )							{ return s_aszObjectiveMoveModeNames; }

#ifdef ASURA_TOOLS
	virtual void ApplyChangesForTemplateInstance(class Tools_EntityTemplateInstance* pxInstance);
	virtual bool CanBeIncludedInTemplate(void) const {return (Classification == AVP_ENTITYCLASS_AI_MARINE_OBJECTIVE);}
#endif

protected:

	// Construction / Destruction
	AVP_ServerNode_Objective_Marine(const Asura_Guid uGuid);
	virtual ~AVP_ServerNode_Objective_Marine(void);

	Asura_Vector_3				m_xPosition;
	u_int						m_bUseSpecificGameSceneNodes;
	u_int						m_bObjectiveCompleteOnMessage;
	bool						m_bObjectiveHasReceivedCompleteMessage;
	u_int						m_bRunToAction;

	Asura_Guid*					m_pxRequiredGameScenes;
	Asura_Guid*					m_pxOptionalGameScenes;
	u_int						m_uNumRequiredGameScenes;
	u_int						m_uNumOptionalGameScenes;

private:

#ifdef ASURA_TOOLS
	// Friend classes
	friend class ToolsAVP_Node_Objective_Marine;
#endif

//	static const char*	 s_aszObjectiveTypeNames[MAX_OBJECTIVE_TYPES];						// Static array of string of the objectives for tools and debug info
	static const char*	 s_aszObjectiveMoveModeNames[MAX_OBJECTIVE_MOVEMENT_MODES_TYPES];	// Static array of string of the objectives for tools and debug info

	// Private typedefs
	typedef AVP_ServerNode_Objective PARENT;

	// Chunk Version
	enum { CURRENT_CHUNK_VERSION = 4 };

	// Private statics
	static Asura_Hashed_List < AVP_ServerNode_Objective_Marine > s_xList;
};

#endif   // __AVP_SERVERNODE_OBJECTIVE_MARINEH__
