//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_PlayerObjective.cpp
//
// Created by AJK 01 / 10 / 2008
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// Core
#include "Core/Asura.h"

// Header
#include "Entity/ObjectiveSystem/PlayerObjective/AVP_ServerEntity_PlayerObjective.h"

// Asura
#include "Core/Asura_HashedLocalisedText.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Network/Asura_Message.h"

// AVP
#include "Actor/AVP_Actor_ObjectiveHandler.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/ObjectiveSystem/PlayerObjective/AVP_PlayerObjective_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PlayerInfo/AVP_PlayerInfo_Messages.h"

Asura_Hashed_List<AVP_ServerEntity_PlayerObjective> AVP_ServerEntity_PlayerObjective::s_xList(2);
const char*                                        AVP_ServerEntity_PlayerObjective::s_aszTargetTypes[] = { "Own location", "Entity", "Location Marker", "No Marker" };

AVP_ServerEntity_PlayerObjective::AVP_ServerEntity_PlayerObjective(const Asura_Guid uGuid)
: PARENT(uGuid)
, m_xObjectiveText()
, m_uFlags(0)
, m_uTargetGuid(ASURA_GUID_UNREGISTERED)
, m_iTargetType(0)
{
	Classification = AVP_ENTITYCLASS_PLAYER_OBJECTIVE;

	SetNeverUpdateClient();
	SetNeverUpdates(true);

	s_xList.AddToList(this);
}

AVP_ServerEntity_PlayerObjective::~AVP_ServerEntity_PlayerObjective(void)
{
	s_xList.RemoveFromList(this);
}

bool AVP_ServerEntity_PlayerObjective::HandleMessage(const Asura_Message* pxMessage)
{
	switch (pxMessage->GetMsgID())
	{
		case ASURA_MSG_CLIENT_REQUESTCREATEMSG:
		{
			MakeCreateMessage(pxMessage->GetFromGuid());
			return true;
		}	

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_MARKER_LOCATION:
		{
			// Two things : set the marker guid, and set the target type
			const AVP_Message_PlayerObjectiveSetMarker* pxData = static_cast<const AVP_Message_PlayerObjectiveSetMarker*>(pxMessage->GetData());
			if ( pxData )
			{
				m_uTargetGuid = pxData->m_uMarkerGuid;
				m_iTargetType = TARGET_LOCATION_MARKER;
				OnChange();
			}
			return true;
		}

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_TARGET_ENTITY:
		{
			// Two things : set the entity guid, and set the target type
			const AVP_Message_PlayerObjectiveSetEntity* pxData = static_cast<const AVP_Message_PlayerObjectiveSetEntity*>(pxMessage->GetData());
			if ( pxData )
			{
				m_uTargetGuid = pxData->m_uEntityGuid;
				m_iTargetType = TARGET_ENTITY;
				OnChange();
			}
			return true;
		}

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_CURRENT:
		{
			// send message to player info AVP_Message_AddPlayerObjective
			Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_ADD_PLAYER_OBJECTIVE, sizeof(AVP_Message_AddPlayerObjective));
			if( pxMessage )
			{
				AVP_Message_AddPlayerObjective* pxData  = static_cast<AVP_Message_AddPlayerObjective*>(pxMessage->GetData());

				pxData->m_uVersion       = AVP_Message_AddPlayerObjective::uCURRENT_VERSION;
				pxData->m_uObjectiveGuid = GetGuid();
				pxMessage->SetFromGuid( GetGuid() );
				pxMessage->SetToGuid  ( AVP_Game_Core::GetPlayerGuid() );
				Asura_Network::Send(pxMessage);
			}
			return true;
		}

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_COMPLETE:
		{
			// send message to player info AVP_Message_PlayerObjectiveComplete
			Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_SET_PLAYER_OBJECTIVE_COMPLETE, sizeof(AVP_Message_PlayerObjectiveComplete));
			if( pxMessage )
			{
				AVP_Message_PlayerObjectiveComplete* pxData  = static_cast<AVP_Message_PlayerObjectiveComplete*>(pxMessage->GetData());

				pxData->m_uVersion       = AVP_Message_AddPlayerObjective::uCURRENT_VERSION;
				pxData->m_uObjectiveGuid = GetGuid();
				pxMessage->SetFromGuid( GetGuid() );
				pxMessage->SetToGuid  ( AVP_Game_Core::GetPlayerGuid() );
				Asura_Network::Send(pxMessage);
			}
			return true;
		}

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_USE_OWN_LOCATION:
		{
			// change target type
			m_iTargetType = TARGET_OWN_LOCATION;
			OnChange();
			return true;
		}
		
		case AVP_MSGID_PLAYER_OBJECTIVE_SET_NO_MARKER:
		{
			// change target type
			m_iTargetType = TARGET_NOTHING;
			OnChange();
			return true;
		}

		default:
			break;
	}

	return PARENT::HandleMessage(pxMessage);
}

void AVP_ServerEntity_PlayerObjective::OnChange()
{
	// Inform the objective handler
	const AVP_ServerEntity_PlayerInfo* pxPlayer = AVP_ServerEntity_PlayerInfo::FindEntity( AVP_Game_Core::GetPlayerGuid() );
	if ( pxPlayer )
	{
		AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity(pxPlayer->GetPuppetGuid());
		if (pxActor && pxActor->GetObjectiveHandler())
		{
			pxActor->GetObjectiveHandler()->SetObjectiveChanged( true );
		}
	}
}

void AVP_ServerEntity_PlayerObjective::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	//Version
	xStream << CURRENT_CHUNK_VERSION;

	//Parent
	PARENT::WriteToChunkStream(xStream);

	xStream << m_xObjectiveText.GetTextID();
	xStream << m_uFlags;
	xStream << m_uTargetGuid; 
	xStream << m_iTargetType;

	xStream << m_xMarkerList.Size();

	Asura_LinkList_ConstIt< Asura_Guid > xLinkListIter( m_xMarkerList );

	while ( !xLinkListIter.Done() )
	{
		xStream << (*xLinkListIter.GetCurrent());
		xLinkListIter.Next();
	}
}

void AVP_ServerEntity_PlayerObjective::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//Version
	u_int uVersion;
	xStream	>> uVersion;

	//Parent
	PARENT::ReadFromChunkStream(xStream);

	if (uVersion == CURRENT_CHUNK_VERSION)
	{
		m_xObjectiveText.SetTextPageHash(Asura_GetHashID("Menu")) ;

		Asura_Hash_ID uTmpHash;

		// Text hash
		xStream >> uTmpHash;
		m_xObjectiveText.SetTextID(uTmpHash);	

		xStream >> m_uFlags; 
		xStream >> m_uTargetGuid; 
		xStream >> m_iTargetType;

		u_int uNumMarkers = 0;
		xStream >> uNumMarkers;

		Asura_Guid uMarkerGuid;
		for ( u_int u = 0; u < uNumMarkers; ++u )
		{
			m_xMarkerList.Push_Back( ( xStream >> uMarkerGuid, uMarkerGuid ) );
		}
	}
	else if (uVersion == 5)
	{
		m_xObjectiveText.SetTextPageHash(Asura_GetHashID("Menu")) ;

		Asura_Hash_ID uTmpHash;

		// Text hash
		xStream >> uTmpHash;
		m_xObjectiveText.SetTextID(uTmpHash);	

		xStream >> m_uFlags; 

		xStream >> m_uTargetGuid; 
	}
	else if (uVersion == 4)
	{
		m_xObjectiveText.SetTextPageHash(Asura_GetHashID("Menu")) ;

		Asura_Hash_ID uTmpHash;

		// Text hash
		xStream >> uTmpHash;
		m_xObjectiveText.SetTextID(uTmpHash);	

		xStream >> m_uFlags; 
	}
	else if (uVersion == 3)
	{
		// Page hash
		Asura_Hash_ID uTmpHash;
		xStream >> uTmpHash;
		m_xObjectiveText.SetTextPageHash(uTmpHash);

		// Text hash
		xStream >> uTmpHash;
		m_xObjectiveText.SetTextID(uTmpHash);	

		xStream >> m_uFlags; 

		Asura_StringBuffer<64> xTextFileName;     //Menu file name
		xStream >> xTextFileName; 
	}
	else if (uVersion == 1 || uVersion == 2)
	{
		Asura_Vector_3 xTemp;
		xStream >> xTemp;

		// Page hash
		Asura_Hash_ID uTmpHash;
		xStream >> uTmpHash;
		m_xObjectiveText.SetTextPageHash(uTmpHash);

		// Text hash
		xStream >> uTmpHash;
		m_xObjectiveText.SetTextID(uTmpHash);	

		Asura_String xTmpTextFileName;
		xStream >> xTmpTextFileName; 
	}
	else if (uVersion == 0)
	{
		Asura_Vector_3 xTemp;
		xStream >> xTemp;

		// Page hash
		Asura_Hash_ID uTmpHash;
		xStream >> uTmpHash;
		m_xObjectiveText.SetTextPageHash(uTmpHash);

		// Text hash
		xStream >> uTmpHash;
		m_xObjectiveText.SetTextID(uTmpHash);	
	}
}

void AVP_ServerEntity_PlayerObjective::MakeCreateMessage(Asura_Guid uRequestingClient, bool bSendGuaranteed)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTCREATE, sizeof(AVP_Message_PlayerObjective_ClientCreate), GetClassification());
	if(pxMessage)
	{
		AVP_Message_PlayerObjective_ClientCreate* pxData = static_cast<AVP_Message_PlayerObjective_ClientCreate*>(pxMessage->GetData());
		pxData->Fill(this);
		pxMessage->SetToGuid(ASURA_GUID_MY_CLIENT_ENTITIES);
		pxMessage->SetFromGuid(GetGuid());
		if (bSendGuaranteed)
		{
			pxMessage->UseGuaranteedDelivery();
		}
		Asura_Network::SendToSpecificMachine(pxMessage, uRequestingClient);
	}
}