
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_PhysicalObject_Messages.cpp
// Author : Andy Tate
// Date   : 13-Sep-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/AVP_PhysicalObject_Messages.h"
#include "Network/Asura_NetMsg.h"

// Game independent header-files...
#include "Entity/PhysicalObject/AVP_ServerEntity_PhysicalObject.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PhysicalObject_Messages::FillPhysicalObject
// Author       : Andy Tate
// Date         : 13-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_Message_PhysicalObject_ClientUpdate::FillPhysicalObject(const AVP_ServerEntity_PhysicalObject* pxPhysicalObject)
{
	m_uBlueprintHash = pxPhysicalObject->GetBlueprintHashID();
#ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
	m_uNumberOfStoredPositions = pxPhysicalObject->GetNumberOfStoredPositions();
	const Asura_Vector_3* pxSourceStoredPositions = pxPhysicalObject->GetStoredPositions(); 
	const float* pxSourceTimeStamps = pxPhysicalObject->GetTimeStampsForStoredPositions(); 

	if ( pxSourceStoredPositions && pxSourceTimeStamps && m_uNumberOfStoredPositions > 0)
	{
		for ( unsigned int u = 0; u < m_uNumberOfStoredPositions; ++u )
		{
			m_axStoredPositions[ u ] = pxSourceStoredPositions[ u ]; 
			m_afTimeStampsForStoredPositions[ u ] = pxSourceTimeStamps[ u ]; 
		}
	}
	else
	{
		// Could be valid, if it hasn't been updated by physics this frame
		m_uNumberOfStoredPositions = 0; 
	}
#endif // #ifdef ASURA_MAX_NO_OF_STORED_POSITIONS

	FillPhysical(pxPhysicalObject);
}

/// End of function 'AVP_PhysicalObject_Messages::FillPhysicalObject'. ///

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
void AVP_Message_PhysicalObject_ClientUpdate::RegisterFields(Asura_NetMsg* pxNetMsg)
{
	const AVP_Message_PhysicalObject_ClientUpdate xDummy;
	Asura_Message_PhysicalObject_ClientUpdate::RegisterFields(pxNetMsg);

	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uint, m_uBlueprintHash, 1, Asura_Threshold_AnyChange, 0.0f);
#ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_uint,					m_uNumberOfStoredPositions,			1,									Asura_Threshold_AnyChange,	0.0f ); 														
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_CompressedPosition,	m_axStoredPositions,				ASURA_MAX_NO_OF_STORED_POSITIONS,	Asura_Threshold_AnyChange,	0.0f ); 
	Asura_NetMsg_RegisterField(pxNetMsg, xDummy, Asura_FieldType_float,					m_afTimeStampsForStoredPositions,	ASURA_MAX_NO_OF_STORED_POSITIONS,	Asura_Threshold_AnyChange,	0.0f );
#endif // #ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
}
#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM


/// End of file. ///

void AVP_Message_PhysicalObject_ClientCreate::FillCreatePhysicalObject( const AVP_ServerEntity_PhysicalObject* pxPhysicalObject )
{
	m_bIgnoreForSurfaceReorientation = pxPhysicalObject->IsIgnoredForSurfaceReorientation();
	m_uMaterialType = pxPhysicalObject->GetMaterialType();

	FillPhysicalObject(pxPhysicalObject);
}
