
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_PhysicalObject_Messages.h
// Author : Andy Tate
// Date   : 23-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_PHYSICALOBJECT_MESSAGES_H__
#define __AVP_PHYSICALOBJECT_MESSAGES_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_NetMsg;
class AVP_ServerEntity_PhysicalObject;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Message_PhysicalObject_ClientUpdate
// Author       : Andy Tate
// Date         : 23-Feb-2007
//
// Description  : Empty class - needed to allow "ProcessClientUpdate" functions
//				  to match class hierarchy.
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_PhysicalObject_ClientUpdate : public Asura_Message_PhysicalObject_ClientUpdate
{
public:
	void	FillPhysicalObject(const AVP_ServerEntity_PhysicalObject* pxPhysicalObject);

	void			SetBlueprintHash(const Asura_Hash_ID uHash)	{ m_uBlueprintHash = uHash; }
	Asura_Hash_ID	GetBlueprintHash(void) const				{ return(m_uBlueprintHash); }

#ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
	u_int					GetNumberOfStoredPositions()		const { return m_uNumberOfStoredPositions; }
	const Asura_Vector_3*	GetStoredPositions()				const { return &m_axStoredPositions[0]; }
	const float*			GetTimeStampsForStoredPositions()	const { return &m_afTimeStampsForStoredPositions[0]; }
#endif // #ifdef ASURA_MAX_NO_OF_STORED_POSITIONS

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
	// No way to create this class directly, so no need to register this entity type.
	static void RegisterFields(Asura_NetMsg* pxNetMsg);
#ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
	static u_char GetNumFields() { return static_cast<u_char>(1 + 2 * ASURA_MAX_NO_OF_STORED_POSITIONS + 1 + Asura_Message_PhysicalObject_ClientUpdate::GetNumFields()); }
#else
	static u_char GetNumFields() { return static_cast<u_char>(1 + Asura_Message_PhysicalObject_ClientUpdate::GetNumFields()); }
#endif // #ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM

private:
	Asura_Hash_ID	m_uBlueprintHash;
#ifdef ASURA_MAX_NO_OF_STORED_POSITIONS
	u_int			m_uNumberOfStoredPositions; 
	Asura_Vector_3	m_axStoredPositions[ASURA_MAX_NO_OF_STORED_POSITIONS];
	float			m_afTimeStampsForStoredPositions[ASURA_MAX_NO_OF_STORED_POSITIONS];
#endif // #ifdef ASURA_MAX_NO_OF_STORED_POSITIONS

};   // End of class 'AVP_Message_PhysicalObject_ClientUpdate'.

class AVP_Message_PhysicalObject_ClientCreate : public AVP_Message_PhysicalObject_ClientUpdate
{
public:
	void			FillCreatePhysicalObject(const AVP_ServerEntity_PhysicalObject* pxPhysicalObject);

	bool			GetIgnoreForSurfaceReorientation() const { return m_bIgnoreForSurfaceReorientation; }
	Asura_Hash_ID	GetMaterialType() const { return m_uMaterialType; }

private:
	bool			m_bIgnoreForSurfaceReorientation;
	Asura_Hash_ID	m_uMaterialType;
};   // End of class 'AVP_Message_PhysicalObject_ClientCreate'.


class AVP_Message_SetHealth
{
public:

	void Initialise()
	{
		m_uVersion  = 1;
		m_fDirectHealthValue = 10.0f;
		m_fPercentageHealthValue = 100.0f;
		m_bTreatAsDamage = 0;
		m_uSetHealthAsPercentageOfMaxHealth = 0;
		m_bDamageComesFromOtherEntity = false;
		m_uDamageFromGuid = ASURA_GUID_UNREGISTERED;
		m_uDamageType	= 0;
	}

	u_int								m_uVersion;
	float								m_fDirectHealthValue;
	float								m_fPercentageHealthValue;
	u_int								m_bTreatAsDamage;
	u_int								m_uSetHealthAsPercentageOfMaxHealth;
	u_int								m_bDamageComesFromOtherEntity;
	Asura_Guid							m_uDamageFromGuid;
	Asura_Hash_ID						m_uDamageType;
};



#endif   // ifndef __AVP_PHYSICALOBJECT_MESSAGES_H__

/// End of file. ///
