//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Actor.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"

// Engine independent.
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_DebugLineRender.h"
#include "Interface/Asura_CommandConsole.h"
#include "Mesh/Asura_Env_System.h"
#include "Network/Asura_Player.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#include "Physics/Asura_Physics.h"
#include "Physics/Asura_Physics_MovementQuery.h"
#include "Shapes/Asura_Shape_System.h"

// Project independent.
#include "Actor/AVP_Actor_AIAnimStateMachine.h"
#include "Actor/AVP_Actor_AnimationHandler.h"
#include "Actor/AVP_Actor_AttachmentHandler.h"
#include "Actor/AVP_Actor_InputHandler.h"
#include "Actor/AVP_Actor_InteractionHandler.h"
#include "Actor/AVP_Actor_ObjectiveHandler.h"
#include "Actor/AVP_Actor_PhysicsObjectHandler.h"
#include "Actor/AVP_Actor_PickupHandler.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/AVP_server_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Messages.h"
#include "Entity/PhysicalObject/Pickup/AVP_ServerEntity_Pickup.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Ammo.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Weapon.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Health.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerEntity_Projectile.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_AcidSpit.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_GenericProjectile.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_Disc.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_Mine.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_DropshipMissile.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_HumanFlare.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_Flamethrower.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Entity/Spawners/AVP_ServerEntity_SpawnController.h"
#include "Entity/Spawners/AVP_ServerEntity_SpawnPoint2.h"
#include "Inventory/AVP_AmmoStore.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Groups/AVP_InventoryGroup_VisionModes.h"
#include "Inventory/Groups/AVP_InventoryGroup_Weapons.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"
#include "Inventory/Items/Weapons/AVP_RangedWeapon_SubType.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesJump.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesInteraction.h"
#include "Moves/Vertebrates/Predator/AVP_Predator_MovesJump.h"
#include "Multiplayer/AVP_Multiplayer_DamageCounter.h"
#include "Multiplayer/AVP_Multiplayer_GameCore.h"
#include "Multiplayer/AVP_Multiplayer_GameRules.h"
#include "Multiplayer/AVP_Multiplayer_SpawnSystem.h"
#include "Network/AVP_Player.h"
#include "Physics/AVP_Collision_Descriptor.h"
#include "Rendering/Asura_Project_Skin_Elements.h"

// Axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/Axon_Messages.h"
#include "Core/Axon_Project_System.h"
#include "Core/Axon_Sound_Registry.h"
#include "Core/BrainTypes/AVP_Brain_Actor.h"
#include "Knowledge/AVP_Knowledge_System.h"
#include "Tactical/AVP_Tactical_System.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#endif

// Constants...

static const u_int g_uCURRENT_CLIENTCREATE_VERSION = 1;

// Debug variables...
DEBUGVAR float dbg_fClientPosLookaheadConstant = 0.1f;		// fixed offset to time used for client-authoritative position
DEBUGVAR float dbg_fClientPosLookaheadLatencyScale = 1.0f;	// additional offset (multiplier of latency) to time used for client-authoritative position

#ifndef ASURA_NO_DEBUG_OUTPUT
DVSVAR bool dvs_bRenderPosAndOri = false;
#endif

#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
int	AVP_ServerEntity_Actor::s_iConsoleVarsCount	= 0;
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)

static const Asura_Collision_Box g_xDefaultCollBox( -0.3f, 0.3f, 0.0f, -1.8f, -0.3f, 0.3f );
static const Asura_Physics_Object::COLLISION_PROXY g_eDefaultCollProxy = Asura_Physics_Object::COLLISION_PROXY_OBB;

// Static variables...
Asura_Hashed_List<AVP_ServerEntity_Actor> AVP_ServerEntity_Actor::s_xList(2);
// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

AVP_ServerEntity_Actor::AVP_ServerEntity_Actor( const u_short usFlagsToIgnore,
												const Asura_Guid uGUID)
: Asura_Entity_Server(uGUID)
, m_pxAIAnimStateMachine( NULL )
, m_pxPickupHandler(NULL)
, m_fObjectSpaceAngVelocityAngle(0.0f)
, m_xObjectSpaceAngVelocityAxis(0.0f, 1.0f, 0.0f)
, m_pxObjectiveHandler(NULL)
, m_uPickupGuid(ASURA_GUID_UNREGISTERED)
, m_fClientPosAndOriTime(0.0f)
, m_xAnimSequenceData()
, m_uCurrentAnim( 0 )
, m_uLoopData( 0 )
, m_bShouldTween( true )
, m_uGuidOfActorBeingAttackedByUs(ASURA_GUID_UNREGISTERED)
, m_xAimDirection( 0.0f, 0.0f, 1.0f )
, m_xLookAtPosition( 0.0f, 0.0f, 0.0f )
, m_xMovementDirection( 0.0f, 0.0f, 1.0f )
, m_usFlagsToIgnore(usFlagsToIgnore)
, m_usCurrentWPID(ASURA_INVALID_WAYPOINT_ID)
, m_uAnimStateBlueprintType( ASURA_HASH_ID_UNSET )
, m_uPlayerInfoGuid(ASURA_GUID_UNREGISTERED)
, m_uMaxNumHealthBlocks( 5 )
, m_uNumActiveHealthBlocks( 5 )
, m_fHealthPerBlock( 20.0f )
, m_fTimeSinceDamage( 0.0f )
, m_fHealthRefillSpeed( 2.0f )
, m_fTimeBeforeHealthRecharge( 2.0f )
, m_uChunkingFlags( 0 )
, m_uNonChunkingFlags( 0 )
, m_uInitialGamesceneGuid( ASURA_GUID_UNREGISTERED )
, m_uInitialSquadGuid( ASURA_GUID_UNREGISTERED )
, m_fMovementNoiseUpdateTimer( 0.0f )
, m_pxReceivedDamageList( NULL )
, m_fLastDamageTime( 0.0f )
, m_fLastDamageAmount( 0.0f )
, m_fLastDamageInterval( 0.0f )
, m_uKillStreak( 0 )
, m_fAIAnimTweenTime( 0.0f )
, m_fPlayerSpeedParameter( 0.0f )
, m_fPlayerMoveAngle( 0.0f )
, m_uDeadAnimHashID(ASURA_HASH_ID_UNSET)
{
#ifdef ASURA_ASSERT
	m_bInitFromBP = false;
#endif

	m_eGroundType = AVP_Actor_PhysicsObjectHandler::ON_FLOOR;
	Classification = AVP_ENTITYCLASS_ACTOR;

	// Set-up the stats...
	SetDestructible( true );
	SetHealth(100.0f);

	// Physics setup...
	m_pxPhysicsObjectHandler	= new AVP_Actor_PhysicsObjectHandler(GetGuid(), AVP_Actor_PhysicsObjectHandler::FREE_AI, g_xDefaultCollBox, g_eDefaultCollProxy, m_usFlagsToIgnore);
	m_pxDynamicObject			= m_pxPhysicsObjectHandler->GetPhysicsObject();

	m_pxDynamicObject->SetGravityScale(2.0f); // player actors generally feel better with a higher gravity scale
	m_pxDynamicObject->SetActorsDontPushMe(true);
	m_pxDynamicObject->SetActorDontPushActors(true);
	m_pxDynamicObject->SetPolyFlagsToReturnForActorCollision( AVP_PolygonFlag_IgnoreForReorientation );

	// Inventory setup...
	m_pxInventorySystem	= new AVP_Inventory_System(*this);

    SetBlueprintTypeHashID( uAVP_BLUEPRINT_TYPE_ACTORS );

	m_bMarkOutOfPlayOnDeactivate = true;

	s_xList.AddToList(this);

#if defined(ASURA_DEBUG_VARIABLE_SYSTEM)
	if(s_iConsoleVarsCount==0)
	{
#ifndef ASURA_NO_DEBUG_OUTPUT
		if(!Asura_CommandConsole::VarExists(L"AVP.Actor.RenderServerPosAndOri"))
		{
			Asura_CommandConsole::AddVar(L"AVP.Actor.RenderServerPosAndOri", &dvs_bRenderPosAndOri );
		}
#endif
	}
	s_iConsoleVarsCount++;
#endif	// defined(ASURA_SUPPORT_COMMAND_CONSOLE)
}

AVP_ServerEntity_Actor::~AVP_ServerEntity_Actor()
{
#if defined(ASURA_SUPPORT_COMMAND_CONSOLE)
	s_iConsoleVarsCount--;
#endif

	s_xList.RemoveFromList(this);
}

void AVP_ServerEntity_Actor::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int uVersion;

	// Read the chunk-version.
	xStream >> uVersion;

    // Read the parent's data.
    PARENT::ReadFromChunkStream(xStream);

    // Read our data.
    Asura_Guid uPlayerGuid = ASURA_GUID_FIRSTPLAYER;
	u_int      uExtCtrlRequests = 0;
	// initialise for old data
	m_xDeadPos = GetPosition();
	m_xDeadOri = GetOrientation();
	if ( uVersion == 9 )
	{
		// Brain is no longer handled in here, but in Asura_ServerEntity_PhysicalObject.
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkingFlags );
		xStream >> xBitfield;
		xStream >> m_uInitialGamesceneGuid;
		xStream >> m_uInitialSquadGuid;
		xStream >> uPlayerGuid;
		xStream >> m_uDeadAnimHashID;
		xStream >> m_uPickupGuid;
		xStream >> uExtCtrlRequests;

		xStream >> m_xDeadPos;
		xStream >> m_xDeadOri;

		m_pxInventorySystem->ReadFromChunkStream(xStream);
	}
	else if ( uVersion == 8 )
	{
		// Brain is no longer handled in here, but in Asura_ServerEntity_PhysicalObject.
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkingFlags );
		xStream >> xBitfield;
		xStream >> m_uInitialGamesceneGuid;
		xStream >> m_uInitialSquadGuid;
		xStream >> uPlayerGuid;
		xStream >> m_uDeadAnimHashID;
		xStream >> m_uPickupGuid;
		xStream >> uExtCtrlRequests;

		m_pxInventorySystem->ReadFromChunkStream(xStream);
	}
	else if ( uVersion == 7 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkingFlags );
		xStream >> xBitfield;
		xStream >> m_uInitialGamesceneGuid;
		xStream >> m_uInitialSquadGuid;
		xStream >> uPlayerGuid;
		xStream >> m_uDeadAnimHashID;
		xStream >> m_uPickupGuid;

		m_pxInventorySystem->ReadFromChunkStream(xStream);
	}
	else if ( uVersion == 6 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkingFlags );
		xStream >> xBitfield;
		xStream >> m_uInitialGamesceneGuid;
		xStream >> m_uInitialSquadGuid;
		xStream >> uPlayerGuid;
		xStream >> m_uDeadAnimHashID;

		m_pxInventorySystem->ReadFromChunkStream(xStream);
	}
	else if ( uVersion == 5 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkingFlags );
		xStream >> xBitfield;
		xStream >> m_uInitialGamesceneGuid;
		xStream >> m_uInitialSquadGuid;
		xStream >> uPlayerGuid;

		m_pxInventorySystem->ReadFromChunkStream(xStream);
	}
	else if ( uVersion == 4 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkingFlags );
		xStream >> xBitfield;
		Asura_Guid uJunk; // Last checkpoint guid.
		xStream >> uJunk;
		xStream >> m_uInitialGamesceneGuid;
		xStream >> m_uInitialSquadGuid;
		xStream >> uPlayerGuid;

		m_pxInventorySystem->ReadFromChunkStream(xStream);
	}
	else if ( uVersion == 3 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkingFlags );
		xStream >> xBitfield;
		Asura_Guid uJunk; // Last checkpoint guid.
		xStream >> uJunk;
		xStream >> m_uInitialGamesceneGuid;
		xStream >> uPlayerGuid;

		m_pxInventorySystem->ReadFromChunkStream(xStream);
	}
#ifdef ASURA_TOOLS
	else if ( uVersion == 2 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkingFlags );

		xStream >> xBitfield;
		Asura_Guid uJunk; // Last checkpoint guid.
		xStream >> uJunk;
		xStream >> m_uInitialGamesceneGuid;
		xStream >> uPlayerGuid;
	}
    else if ( uVersion == 1 )
	{
	    union V1Flags
	    {
		    u_int m_uFlags;
		    struct 
		    {
			    u_int	m_eGroundType				: 4,
					    m_bIsPlayer					: 1,
                        m_bSentClientUpdateMessage	: 1,
                        m_bCloaked					: 1;
		    };
	    } uFlags;
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( uFlags.m_uFlags );
		xStream >> xBitfield;
        m_eGroundType              = uFlags.m_eGroundType;
        m_bIsPlayer                = uFlags.m_bIsPlayer;
        m_bSentClientUpdateMessage = uFlags.m_bSentClientUpdateMessage;

		Asura_Guid uJunk; // Last checkpoint guid.
		xStream >> uJunk;
        xStream >> m_uInitialGamesceneGuid;
        xStream >> uPlayerGuid;
	}
    else if ( uVersion == 0 )
    {
	    union V0Flags
	    {
		    u_int m_uFlags;
		    struct 
		    {
			    u_int	m_eGroundType						: 4,
					    m_bIsPlayer							: 1,
                        m_bSentClientUpdateMessage			: 1,
                        m_bIsActuallyDead					: 1,
                        m_bCloaked							: 1;
		    };
	    } uFlags;
        Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( uFlags.m_uFlags );
        xStream >> xBitfield;
        m_eGroundType              = uFlags.m_eGroundType;
        m_bIsPlayer                = uFlags.m_bIsPlayer;
        m_bSentClientUpdateMessage = false;

		Asura_Guid uJunk; // Last checkpoint guid.
		xStream >> uJunk;
        xStream >> m_uInitialGamesceneGuid;
    }
#endif
	else
	{
        Asura_Assertf( false, ( "AVP_ServerEntity_Actor::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}


	AVP_Actor_PhysicsObjectHandler::Type uPhyObjHandlerType = AVP_Actor_PhysicsObjectHandler::MAX_TYPES;
	if ( m_bIsPlayer )
	{
		MakePlayer( uPlayerGuid );
		uPhyObjHandlerType = AVP_Actor_PhysicsObjectHandler::PLAYER;
	}
	else if ( m_bDeadBody )
	{
		// Dead bodies don't need to update.
		SetUpdateClientEveryFrame(false);
		ShutdownAndDestroyBrain();
	}
	else
	{
		MakeAI();
		uPhyObjHandlerType = AVP_Actor_PhysicsObjectHandler::FREE_AI;

		if ( uVersion <= 7 )
		{
			// After this version the brain is handled in Asura_ServerEntity_PhysicalObject.
			CreateAndInitialiseBrain();
			Asura_Assert( GetBrain(), "AVP_ServerEntity_Actor::ReadFromChunkStream - we must have a brain, as we have brain data in the chunk" );
			if ( GetBrain() )
			{
				GetBrain()->ReadFromChunkStream( xStream );
			}

			// Some entities in old chunks were active but with their brains set inactive. This is bad.
			SetBrainActive( IsEntityActive() );
		}
	}

	// Reset our physics object handler.
	if ( m_pxPhysicsObjectHandler && uPhyObjHandlerType != AVP_Actor_PhysicsObjectHandler::MAX_TYPES )
	{
		const AVP_Collision_Descriptor* pxCollDesc = GetDefaultCollisionDescriptor();
		if ( pxCollDesc )
		{
			const Asura_Collision_Object* pxCollObj = pxCollDesc->GetCollisionBox();
			Asura_Assert( pxCollObj, "Actor has no default collision object, physics won't be set up" );
			if ( pxCollObj )
			{
				m_pxPhysicsObjectHandler->Reset( static_cast< AVP_Actor_PhysicsObjectHandler::Type >( uPhyObjHandlerType ),
					*pxCollObj, AVP_Actor_PhysicsObjectHandler::GetActorCollisionProxy( GetClassification() ) );
			}
		}

		m_pxPhysicsObjectHandler->SetExtCtrlRequests( uExtCtrlRequests );
	}
}

void AVP_ServerEntity_Actor::WriteToClientCreateStream(Asura_Chunk_Memory_Stream& xStream) const
{
	xStream << g_uCURRENT_CLIENTCREATE_VERSION;

	const AVP_Inventory_System*	pxInventorySystem = GetInventorySystem();

	// No parent call, use the update data...
	AVP_Message_PhysicalObject_ClientCreate xPhysicalObjectClientCreate;
	xPhysicalObjectClientCreate.FillCreatePhysicalObject(this);
	xStream.Write(sizeof(AVP_Message_PhysicalObject_ClientCreate), &xPhysicalObjectClientCreate);

	xStream << GetAimDirection();
	xStream << GetLookAtPosition();
	xStream << GetMovementDirection();

	const Axon_Gamescene_AnimSequenceData& xAnimSD = GetAnimSequenceData();
	xStream.Write(sizeof(Axon_Gamescene_AnimSequenceData), &xAnimSD);
	
	Asura_Assert( GetCurrentAnim() < UCHAR_MAX, "AVP_Message_Actor_ClientUpdate::FillActor - only storing current anim in a u_char" );
	
	xStream << GetCurrentAnim();
	xStream << GetLoopData();
	xStream << ShouldTween();

	xStream << GetAIAnimTweenTime();
	xStream << !GetPlayNextAIStateAnimFromStart();

	xStream << AVP_Actor_AIAnimStateMachine::ACTOR_STATE_IDLE;

	u_int uAIAnimState = 0;
	u_int uAIAnimStateAfterMove = 0;
	float fTimeInAIAnimState = 0.f;
	bool bAIAnimStateLooped = false;

	const AVP_Actor_AIAnimStateMachine* pxStateMachine = GetAIAnimStateMachine();
	if ( pxStateMachine != NULL )
	{
		uAIAnimState	= pxStateMachine->GetCurrentState();
		uAIAnimStateAfterMove = pxStateMachine->GetStateAfterMove();
		fTimeInAIAnimState = pxStateMachine->GetTimeInState();
		bAIAnimStateLooped = pxStateMachine->IsLoopedState( uAIAnimState );
	}
	xStream << uAIAnimState; 
	xStream << uAIAnimStateAfterMove; 
	xStream << fTimeInAIAnimState; 
	xStream << bAIAnimStateLooped; 

	xStream << GetGuidOfActorBeingAttackedByUs();

	xStream << GetPlayerInfoGuid();
	xStream << static_cast<u_char>(GetGroundType());
	xStream << IsPlayer();
	bool    bDeadBody = m_bDeadBody;
	xStream << bDeadBody;
	xStream << m_uDeadAnimHashID;

	xStream << IsMoving();
	xStream << IsMovingSlow();
	xStream << IsMovingFast();
	xStream << IsDestructible();

	xStream << GetPlayerSpeedParameter();
	xStream << GetPlayerMoveAngle();

	pxInventorySystem->WriteToChunkStream(xStream);
}

void AVP_ServerEntity_Actor::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// Write the chunk-version...
	xStream << uCURRENT_CHUNK_VERSION;

	// Write the parent's data...
	PARENT::WriteToChunkStream(xStream);

    // Any data specific to this class goes here...
	Asura_WriteToStream_Bitfield_Wrapper<u_int>	xBitfield(m_uChunkingFlags);
	xBitfield.Add(m_eGroundType, 4);
	xBitfield.Add(m_bIsPlayer,	 1);
	xBitfield.Add(m_bDeadBody,	 1);
	xStream << xBitfield;

    xStream << m_uInitialGamesceneGuid;
	xStream << m_uInitialSquadGuid;
	xStream << m_uPlayerInfoGuid;
	xStream << m_uDeadAnimHashID;
	xStream << m_uPickupGuid;
	xStream << ( m_pxPhysicsObjectHandler ? m_pxPhysicsObjectHandler->GetExtCtrlRequests() : 0 );
	xStream << m_xDeadPos;
	xStream << m_xDeadOri;

	m_pxInventorySystem->WriteToChunkStream(xStream);
}

AVP_Brain_Actor* AVP_ServerEntity_Actor::GetActorBrain() const
{
	return static_cast< AVP_Brain_Actor* >( GetBrain() );
}

void AVP_ServerEntity_Actor::OnEntityActivateMessage()
{
	PARENT::OnEntityActivateMessage();

	if ( GetBrain() )
	{
		Asura_Blueprint* pxBlueprint = GetBlueprint();
		if ( pxBlueprint )
		{
			GetBrain()->InitialiseFromBlueprint( *pxBlueprint );
		}
	}

	MakeClientUpdateMessage();
}

void AVP_ServerEntity_Actor::OnEntityActivate()
{
    PARENT::OnEntityActivate();

	if(m_pxInventorySystem != NULL)
	{
		m_pxInventorySystem->Activate();
	}
	if ( m_bDeadBody )
	{
		TeleportTo( m_xDeadPos, m_xDeadOri, false );
	}
}

void AVP_ServerEntity_Actor::OnEntityDeactivate()
{
	if(m_pxInventorySystem != NULL)
	{
		m_pxInventorySystem->Deactivate();
		
		// WGP - This update is here to try and get the inventory system to process the deactivate request before we deactivate the entity, otherwise the system wont get updated until we re-activate the entity.
		m_pxInventorySystem->Update();
	}

	if ( m_bMarkOutOfPlayOnDeactivate )
	{
		AVP_ServerEntity_SpawnController::ActorOutOfPlay( GetGuid() );
		AVP_ServerEntity_SpawnPoint2::ActorOutOfPlay( GetGuid() );
	}

    PARENT::OnEntityDeactivate();
}

void AVP_ServerEntity_Actor::SendOnDestructionMessages( void )
{
	if ( m_bMarkOutOfPlayOnDeactivate )
	{
		AVP_ServerEntity_SpawnController::ActorOutOfPlay( GetGuid() );
		AVP_ServerEntity_SpawnPoint2::ActorOutOfPlay( GetGuid() );
	}

	PARENT::SendOnDestructionMessages();
}

void AVP_ServerEntity_Actor::SendClientUpdateMessage(Asura_Message* pxMessage)
{
	Send( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES );

	if ( !m_bSentClientUpdateMessage )
	{
		Asura_Message* pxMessage = m_xMessageQueue.PopTailObject();

		// Send any messages we've queued.
		while(pxMessage != NULL)
		{
			Send( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES );
			pxMessage = m_xMessageQueue.PopTailObject();
		}

		m_bSentClientUpdateMessage = true;
	}
}

void AVP_ServerEntity_Actor::MakeClientUpdateMessage( void )
{
	Asura_Message* pxMsg = Asura_Network::CreateMessage( ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_Actor_ClientUpdate), GetClassification() );
	if( !pxMsg ) { return; }

	AVP_Message_Actor_ClientUpdate* pxMsgData = static_cast<AVP_Message_Actor_ClientUpdate*>( pxMsg->GetData() );
	pxMsgData->FillActor( *this );
	m_fAIAnimTweenTime = 0.0f; // clear this down for the next client update message
	m_bAIAnimPlayNextFromStart = false;
	SendClientUpdateMessage(pxMsg);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::MakeCreateMessage
// Author       : Chris Wallace
// Date         : 30/10/08
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_Actor::MakeCreateMessage(Asura_Guid uRequestingClient, bool bSendGuaranteed/* = false*/)
{
	Asura_Chunk_Memory_Stream xStream;

#ifdef ASURA_TOOLS
	xStream.SetSwapEndian(false);
#endif

	WriteToClientCreateStream(xStream);
	
	u_int uSize = xStream.GetPosition();

	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTCREATE, uSize, GetClassification(), xStream.GetStartAddress());
	if(pxMessage)
	{
		pxMessage->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );
		pxMessage->SetFromGuid( GetGuid() );
		if (bSendGuaranteed)
		{
			pxMessage->UseGuaranteedDelivery();
		}
		Asura_Network::SendToSpecificMachine( pxMessage, uRequestingClient );
	}
}

void AVP_ServerEntity_Actor::PostLoadInitialise()
{
    PARENT::PostLoadInitialise();

	if (IsDead() && !m_bDeadBody)   // AJK, don't destroy if it is a dead body
	{
		SetThisAndClientsAsDestroyed();
		return;
	}

	SetAimDirection( Asura_Vector_3( GetOrientation().e13, GetOrientation().e23, GetOrientation().e33 ) );
	SetLookAtPosition( Asura_Vector_3( GetPosition() ) + Asura_Vector_3( GetOrientation().e13, GetOrientation().e23, GetOrientation().e33 ) * 10.0f );

	// Ensure that the default inventory items are active...
	//if(m_pxInventorySystem->IsMaster())
	{
		m_pxInventorySystem->PostLoadInitialise();

		AVP_Inventory_Group*	pxVisionModeGroup	= m_pxInventorySystem->GetGroup(uAVP_INVENTORY_GROUPHASH_VISIONMODE);
		AVP_Inventory_Group*	pxWeaponGroup		= m_pxInventorySystem->GetGroup(uAVP_INVENTORY_GROUPHASH_WEAPON);

		if(pxVisionModeGroup != NULL && pxVisionModeGroup->GetCurrentInventoryItem() == ASURA_HASH_ID_UNSET)
		{
			pxVisionModeGroup->SetCurrentItemToDefaultItem(true);
		}
		if(pxWeaponGroup != NULL && pxWeaponGroup->GetCurrentInventoryItem() == ASURA_HASH_ID_UNSET)
		{
			pxWeaponGroup->SetCurrentItemToDefaultItem(true);
		}
	}

	//Ignore collisions with other actors
	if(	m_bDeadBody )
	{
		if ( m_pxDynamicObject )
		{
			m_pxDynamicObject->SetIgnore(true);
		}
		TeleportTo( m_xDeadPos, m_xDeadOri, false );
	}

    //if ( IsEntityActive() )
    //{
    //    MakeClientUpdateMessage();
    //}
}

void AVP_ServerEntity_Actor::PreDestructionShutdown()
{
	Asura_Assert( m_xMessageQueue.PopHeadObject() == NULL, "AVP_ServerEntity_Actor::PreDestructionShutdown - we still have messages around" );

	// Tidy-up our handlers...
	delete m_pxPhysicsObjectHandler;
	delete m_pxInventorySystem;
	delete m_pxPickupHandler;
	delete m_pxObjectiveHandler;
	delete m_pxAIAnimStateMachine;
	m_pxPhysicsObjectHandler = NULL;
	m_pxInventorySystem = NULL;
	m_pxPickupHandler = NULL;
	m_pxObjectiveHandler = NULL;
	m_pxAIAnimStateMachine = NULL;

	// Now that the physics object handler has been deleted, zero our reference to the physics
	// object it created (otherwise Asura_ServerEntity_PhysicalObject's destructor will try to
	// delete it again soon).
	m_pxDynamicObject = NULL;

	// Delete the received damage list if necessary
	if (m_pxReceivedDamageList)
	{
		// Iterate through the list of damage received
		Asura_Hashed_List_Deleter_It<AVP_Multiplayer_DamageCounter> xReceivedDamageListDeleterIt( m_pxReceivedDamageList ); 
		while(!xReceivedDamageListDeleterIt.Done())
		{
			// Delete the damage counters, clearing the list first
			AVP_Multiplayer_DamageCounter* pxDamageCounter = xReceivedDamageListDeleterIt.GetCurrent(); 
			xReceivedDamageListDeleterIt.DeleteCurrent();
			delete pxDamageCounter;
		}

		// Finally, delete the list itself
		delete m_pxReceivedDamageList;
		m_pxReceivedDamageList = NULL; 
	}

	PARENT::PreDestructionShutdown();
}

void AVP_ServerEntity_Actor::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
    PARENT::InitialiseFromBlueprint( xBlueprint );

	InitialiseInventory();

	if(m_pxInventorySystem && m_pxInventorySystem->GetAmmoStore())
	{
		m_pxInventorySystem->GetAmmoStore()->InitialiseFromBlueprints();
	}

	Asura_Value xValue;
	if(xBlueprint.GetParameterValueByHashID(0x60975A8F, xValue))	//Asura_GetHashID("HealthTimeBeforeRecharge"), xValue))
	{
		m_fTimeBeforeHealthRecharge = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(0xCCC27863, xValue))	//Asura_GetHashID("HealthRechargeRate"), xValue))
	{
		m_fHealthRefillSpeed = xValue.GetFloatValue();
	}
	if(xBlueprint.GetParameterValueByHashID(0x8c764f7a, xValue))	//Asura_GetHashID("Health.NumBlocks"), xValue))
	{
		m_uNumActiveHealthBlocks = m_uMaxNumHealthBlocks = xValue.GetIntValue();
	}
	m_fHealthPerBlock = GetInitialHealth() / m_uMaxNumHealthBlocks;

	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ANIMSTATEBLUEPRINTTYPE, xValue ) )
	{
		m_uAnimStateBlueprintType = xValue.GetHashIDValue();
	}

	if(IsADeadBody())
	{
		// IsDead() return true so AI don't shoot at start dead actors
		SetHealth(0.0f);
	}

#ifdef ASURA_ASSERT
	m_bInitFromBP = true;
#endif
}

void AVP_ServerEntity_Actor::QueueMessage( const Asura_Message* pxMessage )
{
    Asura_Message* pxNewMessage = Asura_Network::DuplicateMessage( pxMessage, false );
    m_xMessageQueue.AddToList( pxNewMessage );
}

bool AVP_ServerEntity_Actor::HandleMessage( const Asura_Message* pxMessage )
{
	switch( pxMessage->GetMsgID() )
	{
		case ASURA_MSG_MOVE_START:
		case AVP_MSGID_MOVESTART_SERVER:
		{
 			if (GetBrain())
 			{
 				GetBrain()->DoHandleMessage(pxMessage);
 			}

			// See if this message will make another entity our puppet.
			const AVP_Msg_Move* pxData = static_cast<const AVP_Msg_Move*>(pxMessage->GetData());
			if (pxData && pxData->GetHasTargetEntity() && pxData->GetTargetEntityIsPuppet())
			{
				Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_GRAPPLE_START);
				if (pxMessage)
				{
					const AVP_Msg_MoveWithTargetEntity* pxMoveMsgData = static_cast<const AVP_Msg_MoveWithTargetEntity*>(pxData);

					pxMessage->SetToGuid(pxMoveMsgData->GetTargetGuid());
					pxMessage->SetFromGuid(GetGuid());

					Asura_Network::Send(pxMessage);
				}
			}

			RegisterMoveStartingSounds(pxMessage);

			// Inform the clients what's going on...
			Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
			return(true);
		}

        case ASURA_MSG_MOVE_ABORT:
        {
			// Inform the clients what's going on...
            Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );

			// Forward the message to the brain if there is one
			if (GetBrain())
			{
				GetBrain()->DoHandleMessage(pxMessage);
			}

			RegisterMoveFinishingSounds(pxMessage);

			return(true);
        }

		case ASURA_MSG_MOVE_FINISHED:
		{
			// Forward the message to the brain if there is one
			if (GetBrain())
			{
				GetBrain()->DoHandleMessage(pxMessage);
			}

			RegisterMoveFinishingSounds(pxMessage);

			// Handle any interaction for marines, predators and aliens
			HandleInteractions();

			// Doesn't need passing to the clients - they will decide when they're finished...
			return(true);
		}

		case ASURA_MSG_MOVECHAINCOMPLETE:
		{
			if ( GetAIAnimStateMachine() != NULL )
			{
				GetAIAnimStateMachine()->HandleMessage( pxMessage );
			}

            if (GetBrain())
            {
                GetBrain()->DoHandleMessage(pxMessage);
            }

			return true;
		}

        case ASURA_MSG_AXON_GS2_ANIM_SEQUENCE_DATA:
        {
            const Axon_Gamescene_AnimSequenceData* pxData = static_cast< const Axon_Gamescene_AnimSequenceData* >( pxMessage->GetData() );
            // Keep a copy of this data so we can send it in the client update message also.
            m_xAnimSequenceData = *pxData;

            if ( m_bSentClientUpdateMessage )
            {
                // If we've already sent a client update message we can forward this message straight on.
                Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
            }

            return true;
        }

		case ASURA_MSG_CLIENT_REQUESTCREATEMSG:
		{
			MakeCreateMessage(pxMessage->GetFromGuid());
			return true;
		}

        case AVP_MSGID_REQUEST_ANIMATION:
        {
            if ( m_bSentClientUpdateMessage )
            {
                // If we've already sent a client update message we can forward this message straight on.
                Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
            }
            else
            {
                // If we've not sent a client update message we need to queue this message.
                QueueMessage( pxMessage );
            }

            return true;
        }

		case AVP_MSGID_CHUNK:
		{
			Asura_Assert(false, "Remove limb message no longer supported. If you want a replacement message added, put in a feature request.");
			return true;
		}

		case AVP_MSGID_ITEM_BEGIN_IDLING:
		case AVP_MSGID_ITEM_BEGIN_USING:
		{
			if(m_pxInventorySystem != NULL)
			{
				m_pxInventorySystem->HandleMessage(pxMessage);
			}

			if (GetBrain() != NULL)
			{
				GetBrain()->DoHandleMessage(pxMessage);
			}

			return(true);
		}

		case AVP_MSGID_ATTACH_WEAPON:
		case AVP_MSGID_DETACH_WEAPON:
		case AVP_MSGID_CREATE_ATTACHMENT:
		case AVP_MSGID_DESTROY_ATTACHMENT:
		case AVP_MSGID_DETACH_ATTACHMENT:
		{
			Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
			return(true);
		}

		case AVP_MSGID_CREATE_GENERICPROJECTILE:
		{
			AVP_ServerProjectile_GenericProjectile::Create( pxMessage );
			return(true);
		}

		case AVP_MSGID_CREATE_PROJECTILE_ACIDSPIT:
		{
			AVP_ServerProjectile_AcidSpit::Create( pxMessage );
			return(true);
		}

		case AVP_MSGID_CREATE_PROJECTILE_FLARE:
		{
			AVP_ServerProjectile_HumanFlare::Create( pxMessage );
			return(true);
		}

		case AVP_MSGID_CREATE_PROJECTILE_DISC:
		{
			AVP_ServerProjectile_Disc::Create( pxMessage );
			return(true);
		}

		case AVP_MSGID_CREATE_PROJECTILE_MINE:
		{
			AVP_ServerProjectile_Mine::Create( pxMessage );
			return(true);
		}

		case AVP_MSGID_CREATE_PROJECTILE_DROPSHIP_MISSILE:
		{
			AVP_ServerProjectile_DropshipMissile::Create( pxMessage );
			return(true);
		}

		case AVP_MSGID_CREATE_PROJECTILE_FLAMETHROWER:
		{
			AVP_ServerProjectile_Flamethrower::Create( pxMessage );
			return(true);
		}

		case AVP_MSGID_SET_PREFERRED_TARGET_TYPE:
		case AVP_MSGID_STOP_AI_OBJECTIVE_ACTIONS:
		case AVP_MSGID_SET_AI_TARGET_PRIORITIES:
		case AVP_MSGID_PHYSICS_REORIENTATION_RESET:
		{
			if ( GetBrain() )
			{
				GetBrain()->DoHandleMessage( pxMessage );
			}
			return true;
		}

		case AVP_MSGID_ENABLE_IK_SOLVER:
		case AVP_MSGID_DISABLE_IK_SOLVER:
		{
			Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
			return true;
		}

		case AVP_MSGID_REQUEST_INTERACTION:
			// TODO - generic interaction handling here.
			return true;

		case AVP_MSGID_REQUEST_PICKUP:
		{
			if(m_pxPickupHandler)
			{
				const AVP_Message_Pickup_RequestCollection* const pxData   = static_cast< const AVP_Message_Pickup_RequestCollection* const>( pxMessage->GetData() );
				if(pxData)
				{
					Asura_Guid uPickupGuid;
					pxData->ExtractInfo(uPickupGuid);
					AVP_ServerEntity_Pickup* const pxPickup = AVP_ServerEntity_Pickup::FindEntity(uPickupGuid);
					if ( pxPickup )
					{
						float fTemp;	// will have the current collection distance, we don't needed it in this case;
						                // We could also test for allegiance, hidden and full stores also here...
						if (AVP_Actor_PickupHandler::CanCollect(*pxPickup, *this, fTemp, true))
						{
							// Send collect message to client and put pickup into inventory
                            m_pxPickupHandler->OnPickup(this, pxPickup);
						}
					}
					else
					{
						Asura_Assert(false, "Where is this pickup you want to collect?");
					}
				}
			}
			else
			{
				Asura_Assert(false, "Shouldn't receive a pickup request since this actor doesn't have a pickup handler!");
			}
			return true;
		}

		case AVP_MSGID_DROP_PICKUP:
		{
			if(m_uPickupGuid != ASURA_GUID_UNREGISTERED)
			{
				// Is a tool or normal weapon
				const AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::FindEntity(m_uPickupGuid);
				if(pxPickup)
				{
					const AVP_Pickup* pxPickupObject = pxPickup->GetPickupObject();
					if(pxPickupObject)
					{
						if(pxPickupObject->GetPickupType() == AVP_Pickup::PICKUP_TYPE_TOOL)
						{
							RestorePreviousWeapon();
						}
						else
						{
							DropPickup();
						}
					}
				}
			}
			else
			{
				Asura_Assert(false, "Shouldn't be getting a drop pickup message!");
			}
			return true;
		}

		case AVP_MSGID_INVENTORY_ITEM_RELOAD:
		{
			return(GetInventorySystem()->HandleMessage(pxMessage));
		}

		default:
		{
			return(PARENT::HandleMessage(pxMessage));
		}
	}
}

void AVP_ServerEntity_Actor::OnPickup(AVP_ServerEntity_Pickup* pxPickup, float fQuantity, float fSecondaryQuantity, bool bSetAsCurrent /* defaults to true */)
{
	if(pxPickup)
	{
		m_uPickupGuid = pxPickup->GetGuid(); //Store Pickup GUID, in case of battery it is needed as the battery is not destroyed

		const AVP_Pickup* pxPickupObject = pxPickup->GetPickupObject();
		if(pxPickupObject)
		{
			switch(pxPickupObject->GetGroupHash())
			{
				case uAVP_INVENTORY_GROUPHASH_AMMO:
				{
					const AVP_Pickup_Ammo* pxAmmoPickup = static_cast<const AVP_Pickup_Ammo*>(pxPickupObject);
					const AVP_Inventory_System* pxInventory = GetInventorySystem();
					if(pxInventory)
					{
						AVP_AmmoStore* pxAmmoStore = pxInventory->GetAmmoStore();
						if(pxAmmoStore)
						{
							pxAmmoStore->AddAmmoQuantity(fQuantity, pxAmmoPickup->GetAmmoType());
						}
					}

					break;
				}
				case uAVP_INVENTORY_GROUPHASH_HEALTH:
				{
					AVP_Inventory_System* pxInventory = GetInventorySystem();
					if(pxInventory)
					{
						AVP_Inventory_Group* pxGroup = pxInventory->GetPermanentlyActiveItemGroup();
						if(pxGroup)
						{
							AVP_Inventory_Item* pxItem = pxGroup->GetItem(uAVP_INVENTORY_ITEMHASH_STIM);
							if(pxItem)
							{
								if(pxItem->GetSubType(0)) 
								{
									pxItem->GetSubType(0)->IncreaseQuantity(1.0f);
								}
							}
						}
					}
					
					break;
				}
				case uAVP_INVENTORY_GROUPHASH_WEAPON:
				{
					const AVP_Pickup_Weapon* pxWeaponPickup = static_cast<const AVP_Pickup_Weapon*>(pxPickupObject);
					const AVP_Inventory_System* pxInventory = GetInventorySystem();
					if(pxInventory)
					{
						AVP_Inventory_Group* pxGroup = pxInventory->GetGroup(uAVP_INVENTORY_GROUPHASH_WEAPON);
						if(pxGroup)
						{
							AVP_Inventory_Item* pxItem = pxGroup->FindItemByBlueprintHash(pxWeaponPickup->GetWeaponType());
							if(pxItem)
							{
								AVP_AmmoStore* pxAmmoStore = GetInventorySystem()->GetAmmoStore();
								if(pxAmmoStore)
								{
									if(pxItem->GetNumSubTypes()>0)
									{
										pxAmmoStore->AddAmmoQuantity(fQuantity, pxItem->GetSubType(0)->GetAmmoType());

										if(pxItem->GetNumSubTypes()>1)
										{
											pxAmmoStore->AddAmmoQuantity(fSecondaryQuantity, pxItem->GetSubType(1)->GetAmmoType());
										}
									}
								}
							}
							else
							{
								if ( !bSetAsCurrent )
								{
									// this doesn't swap the current item
									pxGroup->AddPickup(pxWeaponPickup->GetWeaponType(), fQuantity, fSecondaryQuantity);
								}
								else
								{
									pxGroup->SwapCurrentItemForPickup(pxWeaponPickup->GetWeaponType(), fQuantity, fSecondaryQuantity);
								}								
							}
						}
					}
					break;
				}
				case uAVP_INVENTORY_GROUPHASH_AUDIODIARY:
				{
					// This is not used in multiplayer, no audio diary should be present in multiplayer
					if ( AVP_Settings::IsSinglePlayer() )
					{
						AVP_ServerEntity_PlayerInfo* pxPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( GetPlayerInfoGuid() ); 
						if ( pxPlayerInfo )
						{
							AVP_Player_AudioDiary xAudioDiary;
							xAudioDiary.SetDialogBlueprintHashID( pxPickup->GetDialogueBlueprintHashID() );
							pxPlayerInfo->GetAudioDiariesList().Push_Back( xAudioDiary );
						}
					}
					else
					{
						Asura_Assert(false, "Audio diaries should not be placed in multiplayer levels!");
					}
					break;
				}
				default: break; // What have we picked up?!
			}
		}
	}
}

bool AVP_ServerEntity_Actor::IsRagdoll() const
{
	const AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity(Guid);
	if (pxClient)
	{
		return pxClient->IsRagdoll();
	}

	return false;
}

Asura_Hash_ID AVP_ServerEntity_Actor::GetCurrentGrappleBlueprintID() const
{
	if (!m_pxInventorySystem) { return ASURA_HASH_ID_UNSET; }

	// By default, we'll assume we have a weapon and use a piece of data from that. Other entities (e.g. aliens) can override that behaviour if they need to.
	const AVP_InventoryItem_Weapon* pxCurrentWeapon = static_cast<const AVP_InventoryItem_Weapon*>(m_pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON));
	if (!pxCurrentWeapon) { return ASURA_HASH_ID_UNSET; }

	return pxCurrentWeapon->GetGrappleBlueprintID();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientEntity_Actor::RestorePreviousWeapon
// Author       : AJK
// Date         : 28-11-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_Actor::RestorePreviousWeapon() const
{
	const AVP_Inventory_System* pxInventory = GetInventorySystem();
	if(pxInventory)
	{
		AVP_Inventory_Group* pxGroup = pxInventory->GetGroup(uAVP_INVENTORY_GROUPHASH_WEAPON);
		if(pxGroup)
		{
			if(pxGroup->GetPreviousItemBlueprintHash() != ASURA_HASH_ID_UNSET)
			{
				pxGroup->SwapCurrentItemForPickup(pxGroup->GetPreviousItemBlueprintHash(), pxGroup->GetPreviousItemQuantity(), pxGroup->GetPreviousItemSecondaryQuantity());

				// Send restore to client entities
				Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_RESTORE_PREVIOUS_WEAPON);
				if( pxMessage )
				{
					Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
				}
			}
			else
			{
				Asura_Assert(false, "Previous item must been set!");
			}
		}
	}
}

void AVP_ServerEntity_Actor::SetPhysicsToIgnore(Asura_Guid uGuid)
{	
	Asura_Handle iHandle1 = ASURA_INVALID_HANDLE;
	Asura_Handle iHandle2 = ASURA_INVALID_HANDLE;

	iHandle1 = FindPhysicsObjectHandle(uGuid);
	iHandle2 = FindPhysicsObjectHandle(GetGuid());

	SetPhysicsHandleToIgnore(uGuid, iHandle2);
	SetPhysicsHandleToIgnore(GetGuid(), iHandle1);
}

void AVP_ServerEntity_Actor::SetPhysicsHandleToIgnore(Asura_Guid uGuid, Asura_Handle iHandle)
{
	AVP_ServerEntity_Actor* pxServer = AVP_ServerEntity_Actor::FindEntity(uGuid);

	if(pxServer)
	{
		Asura_Physics_Object* pxPhysics = pxServer->GetPhysicsObject();
		if(pxPhysics)
		{
			pxPhysics->SetHandleToIgnore(iHandle);
		}
	}
	else
	{
		AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity(uGuid);
		if(pxClient)
		{
			Asura_Physics_Object* pxPhysics = pxClient->GetPhysicsObject();
			if(pxPhysics)
			{
				pxPhysics->SetHandleToIgnore(iHandle);
			}
		}
	}
}

Asura_Handle AVP_ServerEntity_Actor::FindPhysicsObjectHandle(Asura_Guid uGuid)
{
	Asura_Handle iHandle = ASURA_INVALID_HANDLE;

	AVP_ServerEntity_Actor* pxServer = AVP_ServerEntity_Actor::FindEntity(uGuid);

	if(pxServer && pxServer->GetPhysicsObjectHandle() != ASURA_INVALID_HANDLE)
	{
		iHandle = pxServer->GetPhysicsObjectHandle();
	}
	else
	{
		AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity(uGuid);
		if(pxClient && pxClient->GetPhysicsObjectHandle() != ASURA_INVALID_HANDLE)
		{
			iHandle = pxClient->GetPhysicsObjectHandle();
		}
	}

	return iHandle;
}

DEBUGVAR float dbg_fActorOutOfWorldKillCoord = 300.0f;

void AVP_ServerEntity_Actor::Update()
{
#if (defined(ASURA_ASSERT ) && ASURA_NETWORK_LOGGING)
	const char* szClassName;
	Asura_Network::GetEntityClassificationDesc( GetClassification(), &szClassName );
	Asura_Assertf( m_bInitFromBP, ( "I've never been initialised from my blueprint, this is going to break big time. It's probably Will's fault! Guid: %d, Classification: %s", GetGuid(), szClassName ) );
#endif

	UpdateMovement();

	if(m_pxInventorySystem != NULL)
	{
		m_pxInventorySystem->Update();
	}

	// Pickups server side Update
	if(m_pxPickupHandler)
	{
	   m_pxPickupHandler->UpdateServer();
	}

	// Update objective handler
	if(m_pxObjectiveHandler)
	{
		m_pxObjectiveHandler->Update();
	}

	// Call the parent function...
    PARENT::Update();

	UpdateHealth();

	if ( m_fLastDamageAmount > 0.0f )
	{
		m_fLastDamageInterval += Asura_Timers::GetGameTimeSlice();
	}

	UpdateMovementSoundsForAI();

	if ( m_pxAIAnimStateMachine != NULL )
	{
		m_pxAIAnimStateMachine->Update();
	}

	// AJG - 19/11/08 - Temporary hack to prevent characters trying to do things after falling out of the world. Assert included so people can keep track of areas and entities that are particularly troublesome.
	if (!IsDead() && IsEntityActive() && (GetPosition().y > dbg_fActorOutOfWorldKillCoord))
	{
		// Assert that this has happened, and kill ourselves.
		Asura_Assertf(false, ("Actor %d (classification %s) appears to have fallen out of the environment. Killing them to prevent further issues.", Guid, Asura_Network::GetEntityClassificationDesc(Classification)));

		Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_ENTITYDAMAGE_REQUEST_VICTIM, sizeof(AVP_Message_EntityDamage_RequestVictim));
		if(pxMessage)
		{
			AVP_Message_EntityDamage_RequestVictim*	pxData = static_cast<AVP_Message_EntityDamage_RequestVictim*>(pxMessage->GetData());
			const AVP_EntityDamage_RequestVictim	xDamage(uAVP_BLUEPRINT_NAME_DAMAGE_VOLUME,
															Guid,
															ASURA_HASH_ID_UNSET,
															Guid,
															1000000.0f,
															GetPosition(),
															Asura_Vector_3(0.0f, 1.0f, 0.0f),
															Asura_Vector_3(0.0f, -1.0f, 0.0f),
															0.0f,
															-1,
															false,
															Guid);
			pxData->SetDamage(xDamage);

			pxMessage->SetFromGuid(Guid);
			pxMessage->SetToGuid(Guid);
			Asura_Network::Send(pxMessage);
		}
	}

#ifndef ASURA_NO_DEBUG_OUTPUT
	if ( dvs_bRenderPosAndOri )
	{
		Asura_DebugLineRender::AddAxis( GetPosition(), GetOrientation(), 0.7f, 0.6f );
	}
#endif
}

void AVP_ServerEntity_Actor::UpdateHealth(void)
{
	float fHealth = GetHealth();

	if(fHealth <= 0.0f) return; // Dead

	float fTimeSlice = Asura_Timers::GetGameTimeSlice();

	m_fTimeSinceDamage += fTimeSlice;

	float fNextHealthBlock = m_uNumActiveHealthBlocks * m_fHealthPerBlock;

	if ( m_fTimeSinceDamage >= m_fTimeBeforeHealthRecharge && fHealth < fNextHealthBlock )
	{
		fHealth+=(fTimeSlice * m_fHealthRefillSpeed);
		if ( fHealth > fNextHealthBlock )
		{
			fHealth = fNextHealthBlock;
		}
	}

	SetHealth(fHealth);
}

void AVP_ServerEntity_Actor::SetNumActiveHealthBlocks()
{
	Asura_Blueprint* pxOurBp = Asura_Blueprint_System::GetBlueprintByHashID( GetBlueprintTypeHashID(), GetBlueprintHashID() );
	float fOriginalHealth = -1.0f;

	if ( pxOurBp )
	{
		Asura_Value xValue;

		if ( pxOurBp->GetParameterValueByHashID( 0xb735025c /*"Health"*/, xValue ) )
		{
			fOriginalHealth = xValue.GetFloatValue();

			m_uNumActiveHealthBlocks = (u_int)( fOriginalHealth / m_fHealthPerBlock );
		}
	}

	float fHealth = GetHealth();
	float fPercentage = fHealth / fOriginalHealth;

	m_uNumActiveHealthBlocks = (u_int)(m_uNumActiveHealthBlocks * fPercentage);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::RegisterMovementSoundsForAI
// Author       : Gareth Thomas
// Date         : 26-Oct-2007
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerEntity_Actor::UpdateMovementSoundsForAI()
{
	float fVelocity = GetVelocity().MagnitudeSquared();
	if (fVelocity > 1.0f)
	{
	m_fMovementNoiseUpdateTimer -= Asura_Timers::GetGameTimeSlice();

	if (m_fMovementNoiseUpdateTimer <= 0.0f)
	{
		m_fMovementNoiseUpdateTimer += 1.0f;
		RegisterMovementSoundsForAI();
		}
	}
	else
	{
		m_fMovementNoiseUpdateTimer = 0.5f;
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::RegisterMovementSoundsForAI
// Author       : Gareth Thomas
// Date         : 18-Jan-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerEntity_Actor::RegisterMovementSoundsForAI()
{
	Axon_System::GetSoundRegistry()->RegisterSound(GetGuid(), GetClassification(), GetPosition(), Axon_Project_System::GetFootstepSoundRangeMed(), AXON_SOUND_TYPE_FOOTSTEP);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::RegisterMoveStartingSounds
// Author       : Anthony Gregson
// Date         : 21/07/08
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerEntity_Actor::RegisterMoveStartingSounds(const Asura_Message* pxMessage)
{
// 	const Asura_Msg_Move* pxData = static_cast<const Asura_Msg_Move*>(pxMessage->GetData());
// 
// 	Asura_Hash_ID uMoveID = pxData->GetMoveID();
// 
// 	if (uMoveID == AVP_Move_Landing::GetStaticHash())
// 	{
// 		Axon_System::GetSoundRegistry()->RegisterSound(GetGuid(), GetClassification(), GetPosition(), 15.0f, AVP_SOUND_TYPE_JUMP_LANDING);
// 	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::RegisterMoveFinishingSounds
// Author       : Gareth Thomas
// Date         : 18-Apr-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerEntity_Actor::RegisterMoveFinishingSounds(const Asura_Message* pxMessage)
{
// 	const Asura_Msg_Move* pxData = static_cast<const Asura_Msg_Move*>(pxMessage->GetData());
// 
// 	Asura_Hash_ID uMoveID = pxData->GetMoveID();
// 
// 	if (
// 		uMoveID == AVP_Move_Predator_JumpTarget::GetStaticHash()			||
// 		uMoveID == AVP_Move_Alien_JumpSurface180::GetStaticHash()			||
// 		uMoveID == AVP_Move_Alien_JumpSurface180::GetStaticHash()			||
// 		uMoveID == AVP_Move_Alien_JumpSurfaceConcave90End::GetStaticHash()	||
// 		uMoveID == AVP_Move_Alien_JumpSurfaceConcave90::GetStaticHash()
// 		)
// 	{
// 		Axon_System::GetSoundRegistry()->RegisterSound(GetGuid(), GetClassification(), GetPosition(), 15.0f, AVP_SOUND_TYPE_JUMP_LANDING);
// 	}

}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::SetToStartPosition
// Author       : Andy Tate
// Date         : 18-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_Actor::SetToStartPosition(const Asura_Hash_ID uStartPointBlueprintParameterHash)
{
	Asura_Vector_3		xPosition;
	Asura_Matrix_3x3	xOrientation;

#if defined(ASURA_TOOLS)
	if(Asura_Environment_System::GetUseCameraAsStartPoint(xPosition, xOrientation))
	{
		Asura_Vector_3 xI;
		Asura_Vector_3 xJ( 0.0f, 1.0f, 0.0f );
		Asura_Vector_3 xK( xOrientation.e13, xOrientation.e23, xOrientation.e33 );

		xI = xJ ^ xK;
		float fTemp = xI.Normalise();
		if ( fTemp == 0.0f )
		{
			xI = Asura_Vector_3( 1.0f, 0.0f, 0.0f );
		}

		xK = xI ^ xJ;
		xK.Normalise();

		xOrientation.e11 = xI.GetX();
		xOrientation.e21 = xI.GetY();
		xOrientation.e31 = xI.GetZ();

		xOrientation.e12 = xJ.GetX();
		xOrientation.e22 = xJ.GetY();
		xOrientation.e32 = xJ.GetZ();

		xOrientation.e13 = xK.GetX();
		xOrientation.e23 = xK.GetY();
		xOrientation.e33 = xK.GetZ();
	}
	else
#endif	// defined(ASURA_TOOLS)
	{
		// Different spawn behaviour for single player / multiplayer 
		if (AVP_Settings::IsSinglePlayer())
		{
			// Search through the list of start points for one appropriate to us.
			Asura_Hashed_List_It<AVP_ServerEntity_StartPoint> xIt(AVP_ServerEntity_StartPoint::GetList());
			AVP_ServerEntity_StartPoint* pxChosenStartPoint = NULL;
			while (!xIt.Done())
			{
				AVP_ServerEntity_StartPoint* pxStartPoint = xIt.GetCurrent();
				xIt.Next();

				if (!pxStartPoint) { continue; }

				const Asura_Hash_ID uBlueprintHashID = pxStartPoint->GetBlueprintHashID();
				const Asura_Hash_ID uBlueprintTypeHashID = pxStartPoint->GetBlueprintTypeHashID();
				Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uBlueprintTypeHashID, uBlueprintHashID);
				if (pxBlueprint)
				{
					Asura_Value xValue;
					if (pxBlueprint->GetParameterValueByHashID(uStartPointBlueprintParameterHash, xValue) && xValue.GetBoolValue())
					{
						// Valid point.
						pxChosenStartPoint = pxStartPoint;
						break;
					}
				}
				else
				{
					// Assume that if no blueprint was chosen, the start point can be used by anyone.
					pxChosenStartPoint = pxStartPoint;
					break;
				}
			}

			if (!pxChosenStartPoint)
			{
				Asura_Assert(false, "No start point for player species in this level. Using first start point.");
				xIt.Restart();
				pxChosenStartPoint = !xIt.Done() ? xIt.GetCurrent() : NULL;
			}

			if (pxChosenStartPoint)
			{
				xPosition = pxChosenStartPoint->GetPosition();
				xOrientation = pxChosenStartPoint->GetOrientation();
				pxChosenStartPoint->SetLastActorSpawnedGuid(ASURA_GUID_UNREGISTERED); 
			}
			else
			{
				Asura_Assert(false, "No start points in level, using origin.");
				xPosition.Zero();
				xOrientation.Identity();
			}
		}
		else
		{
			// The one we will use to get the start position
			AVP_ServerEntity_StartPoint* pxChosenStartPoint = NULL;

			// Select a start point from our candidate list
			switch (AVP_Multiplayer_GameRules::GetRespawnType())
			{
				case AVP_Settings::AVP_RESPAWNTYPE_TEAM:
				{
					// Emulate the single player behaviour, spawning in the first available start point
					// TODO: choose randomly, safer place?
					pxChosenStartPoint = AVP_Multiplayer_SpawnSystem::FindRandomStartPointForTeam( GetPlayerInfoGuid() );
					break; 
				}
				case AVP_Settings::AVP_RESPAWNTYPE_LOCATION:
				case AVP_Settings::AVP_RESPAWNTYPE_HUNTER:
				{
					Asura_Assert( false, "Respawn type not implemented yet, defaulting to random" ); 
				}
				default:
				case AVP_Settings::AVP_RESPAWNTYPE_RANDOM:
				{
					pxChosenStartPoint = AVP_Multiplayer_SpawnSystem::FindRandomStartPointForIndividual( GetPlayerInfoGuid() );  
					break;
				}
			}

			// Use the position / orientation from the chosen start position if we have one
			if (pxChosenStartPoint)
			{
				xPosition = pxChosenStartPoint->GetPosition();
				xOrientation = pxChosenStartPoint->GetOrientation();
				pxChosenStartPoint->SetLastActorSpawnedGuid(GetGuid()); 
			}
			else
			{
				Asura_Assert(false, "No start points in level, using origin.");
				xPosition.Zero();
				xOrientation.Identity();
			}
		}
	}

	// Update the server accordingly...
	SetPosition(xPosition);
	SetOrientation(xOrientation);
}

/// End of function 'AVP_ServerEntity_Actor::SetToStartPosition'. ///


void AVP_ServerEntity_Actor::TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient)
{
	// Is this a multiplayer match?
	if (AVP_Settings::IsMultiPlayer())
	{
		if (IsDamageIgnoredByThisPlayer( xDamage ))
		{
			// abort this damage request
			return; 
		}

		// Update the damage counters
		UpdateDamageCounters( xDamage );
	}

	PARENT::TakeDamageExtended(xDamage, bForceFatal, bSendMessageToClient);

	m_uNumActiveHealthBlocks = Asura_Maths::Min< u_int >( 1 + Asura_Maths::FloatToInt_Chop( GetHealth() / m_fHealthPerBlock ), m_uMaxNumHealthBlocks );
}

void AVP_ServerEntity_Actor::SendDamageMessageToClient(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	Asura_Message*	pxMessage	= Asura_Network::CreateMessage(AVP_MSGID_ENTITYDAMAGE_APPLY_TO_ENTITY, sizeof(AVP_Message_EntityDamage_ApplyToEntity));

	if(pxMessage != NULL)
	{
		AVP_Message_EntityDamage_ApplyToEntity*	pxData		= static_cast<AVP_Message_EntityDamage_ApplyToEntity*>(pxMessage->GetData());

		// Inform the client regardless of whether we can be damaged...
		pxData->SetDamage(xDamage);

		pxMessage->SetFromGuid( GetGuid() );
		pxMessage->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );

		// Dismember damage is handled through its own specific message, so don't forward that to the client.
		if (xDamage.GetTypeHash() != uAVP_BLUEPRINT_NAME_DAMAGE_DISMEMBER)
		{
			Asura_Network::Send( pxMessage, false);
		}

		// Pass damage messages on towards brain if we have one.
		if ( GetBrain() )
		{
			GetBrain()->DoHandleMessage( pxMessage );
		}

		// Destroy damage message
		Asura_Network::DestroyMessage( pxMessage );
	}
}

void AVP_ServerEntity_Actor::MakePlayer(Asura_Guid uPlayerGuid)
{
	if (uPlayerGuid == m_uPlayerInfoGuid) { return; }

	if(m_pxInventorySystem != NULL)
	{
		m_pxInventorySystem->SetAsSlave();
	}

	// Client is now in control...
	delete(m_pxPhysicsObjectHandler);
	m_pxPhysicsObjectHandler	= NULL;
	m_pxDynamicObject			= NULL;

	SetEntityActive( true );

	m_bIsPlayer = true;

	m_uPlayerInfoGuid = uPlayerGuid;

	delete m_pxAIAnimStateMachine;
	m_pxAIAnimStateMachine = NULL;

	// Create objective handler
	if ( !m_pxObjectiveHandler )
	{
		m_pxObjectiveHandler = new AVP_Actor_ObjectiveHandler(*this);
	}

	// Create pickup handler
	if ( !m_pxPickupHandler )
	{
		m_pxPickupHandler    = new AVP_Actor_PickupHandler(GetGuid());
	}
}

void AVP_ServerEntity_Actor::MakeAI()
{
	m_bIsPlayer = false;

	// TODO - is this entirely safe? Can someone respawn after this and end up missing a lot of important data?
	if (IsDead()) { return; }

	if(m_pxPhysicsObjectHandler == NULL)
	{
		const AVP_Collision_Descriptor* pxCollDesc = GetDefaultCollisionDescriptor();
		if ( pxCollDesc )
		{
			const Asura_Collision_Object* pxCollObj = pxCollDesc->GetCollisionBox();
			Asura_Assert( pxCollObj != NULL, "No default collision object for actor, physics won't be set up" );
			if ( pxCollObj != NULL )
			{
				m_pxPhysicsObjectHandler	= new AVP_Actor_PhysicsObjectHandler(GetGuid(), AVP_Actor_PhysicsObjectHandler::FREE_AI, *pxCollObj, AVP_Actor_PhysicsObjectHandler::GetActorCollisionProxy(Classification), m_usFlagsToIgnore);
				m_pxDynamicObject			= m_pxPhysicsObjectHandler->GetPhysicsObject();
				m_pxDynamicObject->SetGravityScale(2.0f); // player actors generally feel better with a higher gravity scale
				m_pxDynamicObject->SetActorsDontPushMe(true);
				m_pxDynamicObject->SetActorDontPushActors(true);
				m_pxDynamicObject->SetPolyFlagsToReturnForActorCollision( AVP_PolygonFlag_IgnoreForReorientation );
			}
		}
	}
	else
	{
		const AVP_Collision_Descriptor* pxCollDesc = GetDefaultCollisionDescriptor();
		if ( pxCollDesc )
		{
			const Asura_Collision_Object* pxCollObj = pxCollDesc->GetCollisionBox();
			Asura_Assert( pxCollObj != NULL, "Actor has no default collision object, physics won't be set up" );
			if ( pxCollObj != NULL )
			{
				m_pxPhysicsObjectHandler->Reset(AVP_Actor_PhysicsObjectHandler::FREE_AI, *pxCollObj, AVP_Actor_PhysicsObjectHandler::GetActorCollisionProxy(Classification));
			}
		}
	}

	// Do we already have a physics-object?
	if(m_pxInventorySystem != NULL)
	{
		m_pxInventorySystem->SetAsMaster();
	}
}

void AVP_ServerEntity_Actor::DisconnectPlayer()
{
	m_uPlayerInfoGuid = ASURA_GUID_UNREGISTERED;
}

void AVP_ServerEntity_Actor::OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	// Simple handling, not really the whole job in a real game.
	if ( GetBrain() )
	{
		SetBrainActive( false );
	}

	if ( m_pxAIAnimStateMachine )
	{
		m_pxAIAnimStateMachine->RequestDead();
	}

	AVP_Inventory_Item* pxItem = GetInventorySystem()->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON );
	if ( pxItem != NULL )
	{ 
		pxItem->RequestDeactivation( true );
	}

	//Axon_System::GetSoundRegistry()->RegisterSound(GetGuid(), GetClassification(), GetPosition(), 10.0f, AVP_SOUND_TYPE_SCREAM); // 10.0f = range

	AVP_ServerEntity_SpawnController::ActorOutOfPlay( GetGuid() );
	AVP_ServerEntity_SpawnPoint2::ActorOutOfPlay( GetGuid() );

	// Score in multiplayer games
	if (AVP_Settings::IsMultiPlayer())
	{
		Asura_Guid uExecutorGuid = xDamage.GetFirerGuid();
		Asura_Guid uVictimGuid = GetGuid(); 
		AVP_Multiplayer_GameCore::OnActorGetsKilled( uExecutorGuid, uVictimGuid ); 
	}

	PARENT::OnDeath( xDamage );

	if ( GetInitialSquadGuid() != ASURA_GUID_UNREGISTERED )
	{
		AVP_ServerEntity_Squad* pxSquadController = AVP_ServerEntity_Squad::FindEntity( GetInitialSquadGuid() );

		if ( pxSquadController )
		{
			pxSquadController->SendMemberDeadEventMessage();
		}
	}

	// Drop a pickup
	DropPickup();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::DropPickup
// Author       : AJK
// Date         : ?
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerEntity_Actor::DropPickup() const
{
	DropPickup(GetInventorySystem()->GetCurrentItemID(uAVP_INVENTORY_GROUPHASH_WEAPON));
}

void AVP_ServerEntity_Actor::DropPickup(Asura_Hash_ID uHash) const
{
	// Get inventory item
	const AVP_Inventory_Item* pxWeapon = GetInventorySystem()->GetItem(uHash);
	if(pxWeapon)
	{
		Asura_Hash_ID uPickupBlueprintHash;
		if( (uPickupBlueprintHash = pxWeapon->GetPickupBlueprintHash()) != ASURA_HASH_ID_UNSET )
		{
			// Get the client
			AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(GetGuid());
			if(pxActor)
			{
				// Get position and orientation of the weapon
				Asura_Vector_3   xAttachPointPosition;
				Asura_Matrix_3x3 xAttachPointOrientation;

				if (pxActor->GetMarkerLocationFromHash(pxWeapon->GetLinkMarkerPoint(),&xAttachPointPosition, &xAttachPointOrientation))
				{
					AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::Create(Asura_Entity_System::GetCreatedByGameGuid(), xAttachPointPosition, xAttachPointOrientation, uPickupBlueprintHash, !IsPlayer());
					if(pxPickup)
					{
						pxPickup->SetQuantity(static_cast<int>((pxWeapon->GetNumSubTypes()>0)?pxWeapon->GetSubType(0)->GetCurrentQuantity():0.0f));
						pxPickup->SetSecondaryQuantity(static_cast<int>((pxWeapon->GetNumSubTypes()>1)?pxWeapon->GetSubType(1)->GetCurrentQuantity():0.0f));
						
						// Give it some motion
						if (pxActor->GetAttachmentHandler() && pxPickup->GetPhysicsObject())
						{
							Asura_Vector_3 xLastDmgImpulse = pxActor->GetAttachmentHandler()->GetLastDamageImpulse();
							// We dont want pickups getting too much impulse, so ill normalise this and random it up a bit.
							xLastDmgImpulse.Normalise();
							xLastDmgImpulse.x += Asura_Random::GetFloat(0.f,1.f) - 0.5f;
							xLastDmgImpulse.y += Asura_Random::GetFloat(0.f,1.f) - 0.5f;
							xLastDmgImpulse.z += Asura_Random::GetFloat(0.f,1.f) - 0.5f;
							xLastDmgImpulse *= Asura_Random::GetFloat(0.1f,1.f);
							pxPickup->GetPhysicsObject()->SetLinImpulse(xLastDmgImpulse);
							// And give it a rotational impulse as well
							Asura_Euler_3 xRotationalImpulse;
							xRotationalImpulse.x = Asura_Random::GetFloat(0.f,0.1f) - 0.05f;
							xRotationalImpulse.y = Asura_Random::GetFloat(0.f,0.1f) - 0.05f;
							xRotationalImpulse.z = Asura_Random::GetFloat(0.f,0.1f) - 0.05f;
							pxPickup->GetPhysicsObject()->ApplyRotationalImpulse(xRotationalImpulse);
						}

					}
				}
				else
				{
					Asura_Assert(false, "Cannot get pickup marker location on client actor!");
				}
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::DropTool (this is like drop pickup, but uses the conserved pickup GUID and no pickup is created
//                                                  just unhidden)
// Author       : AJK
// Date         : 01/12/08
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void AVP_ServerEntity_Actor::DropTool() const
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::FindEntity(m_uPickupGuid);
	if(pxPickup)
	{
		// Get the client
		AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(GetGuid());
		if(pxActor)
		{
			const AVP_InventoryItem_Weapon* pxCurrentWeapon = pxActor->GetCurrentWeapon();
			if(pxCurrentWeapon)
			{
				// Get position and orientation of the weapon
				Asura_Vector_3   xAttachPointPosition;
				Asura_Matrix_3x3 xAttachPointOrientation;
				xAttachPointPosition.Zero();

				if (pxActor->GetMarkerLocationFromHash(pxCurrentWeapon->GetLinkMarkerPoint(),&xAttachPointPosition, &xAttachPointOrientation))
				{
					xAttachPointOrientation.Identity();
					pxPickup->Drop(xAttachPointPosition, xAttachPointOrientation);
					pxPickup->PlayRestAnim();
				}
				else
				{
					Asura_Assert(false, "Cannot get pickup marker location on client actor!");
				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::SetOrientation
// Author       : Andy Tate
// Date         : 18-May-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_Actor::SetOrientation(const Asura_Matrix_3x3& xOrientation)
{
	// Call the base function...
	PARENT::SetOrientation(xOrientation);

	if(m_pxPhysicsObjectHandler != NULL)
	{
		m_pxPhysicsObjectHandler->ResetUsingWorldSpaceOrientation(xOrientation);
	}
}

/// End of function 'AVP_ServerEntity_Actor::SetOrientation'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::TeleportTo
// Author       : Andy Tate
// Date         : 18-May-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerEntity_Actor::TeleportTo(const Asura_Vector_3& xPosition,
										const Asura_Matrix_3x3& xOrientation,
										const bool bInformClients)
{
	// Call the base function...
	PARENT::TeleportTo(xPosition, xOrientation, bInformClients);

	if(m_pxPhysicsObjectHandler != NULL)
	{
		m_pxPhysicsObjectHandler->ResetUsingWorldSpaceOrientation(xOrientation);
	}

	if( GetBrain() && GetBrain()->IsActive() )
	{
		GetActorBrain()->ResetMovementDirection();

		// Also recalculate nav data
		GetActorBrain()->EstablishNavData();
	}
}

/// End of function 'AVP_ServerEntity_Actor::TeleportTo'. ///


Asura_AVector3 AVP_ServerEntity_Actor::GetCurrentGravityDirection() const
{
	if(m_pxPhysicsObjectHandler)
	{
		Asura_AVector3 xDirection;
		m_pxPhysicsObjectHandler->GetCurrentGravityDirection(xDirection);
		return xDirection;
	}
	else
	{
		// See if we can get the physics object on the client.
		const AVP_ClientEntity_Actor* pxClientActor = AVP_ClientEntity_Actor::FindEntity( GetGuid() );
		if ( pxClientActor && pxClientActor->GetPhysicsObjectHandler() )
		{
			Asura_AVector3 xDirection;
			pxClientActor->GetPhysicsObjectHandler()->GetCurrentGravityDirection(xDirection);
			return xDirection;
		}
		else
		{
			return(Asura_Physics_System::GetGravityDirection());
		}
	}
}


bool AVP_ServerEntity_Actor::HasSpaceForAnim( const Asura_Hash_ID uAnimHashID,
											  const float fMovementScale /*= 1.0f*/, 
											  const bool bCollideWithActors /*= false*/,
											  const Asura_Vector_3& xOffset /*= Asura_Maths::ZeroVector3*/,
											  const u_int uNumTestSegments /*= 4*/ ) const
{
	if ( uNumTestSegments == 0 ) { return false; }

	const Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnimByHash( uAnimHashID );
	if ( !pxAnim ) { return false; }

	const Asura_Hierarchy_Skin* pxSkin = NULL;
	u_short usPolyFlagsToIgnore = AsuraPolygonFlag_NoCollisions | AVP_PolygonFlag_IgnoreForPhysicsObjectCollision;
	if ( GetPhysicsObjectHandler() )
	{
		usPolyFlagsToIgnore |= GetPhysicsObjectHandler()->GetFlagsToIgnore();
	}

	const AVP_ClientEntity_Actor* pxClientActor = AVP_ClientEntity_Actor::FindEntity( GetGuid() );
	if ( pxClientActor )
	{
		pxSkin = pxClientActor->GetAnimationHandler()->GetAnimation()->GetSkin();

		if ( pxClientActor->GetPhysicsObjectHandler() )
		{
			usPolyFlagsToIgnore |= pxClientActor->GetPhysicsObjectHandler()->GetFlagsToIgnore();
		}
	}

	const Asura_Vector_3& xOurPosition = GetPosition();
	const Asura_Matrix_3x3& xOurOrientM = GetOrientation();
	const Asura_Quat xOurOrientQ(GetOrientation());

	Asura_OBB xOBB( *GetBB() );

	const float fParametricIncrement = 1.0f / uNumTestSegments;
	float fParametricTimer = fParametricIncrement;
	Asura_Vector_3 xPositionZero = xOurPosition;
	Asura_Quat     xOrientZero   = xOurOrientQ;
	for ( u_int uSegment = 0; uSegment < uNumTestSegments; ++uSegment )
	{
		Asura_Vector_3 xPositionOne;
		Asura_Quat     xOrientOne;
		if ( !pxAnim->GetMovementOffset( pxSkin, fParametricTimer, &xPositionOne, &xOrientOne ) )
		{
			return false;
		}
		xPositionOne = xOurPosition + xPositionOne * fMovementScale * xOurOrientM;
		xOrientOne = xOurOrientQ * xOrientOne;

		Asura_AMatrix3x3 xOrientAMat;
		xOrientZero.ToMatrix( xOrientAMat );
		xOBB.SetOrientation( xOrientAMat );

		const Asura_AVector3 xInitialPosition( xPositionZero + xOffset );
		const Asura_AVector3 xDisplacement( xPositionOne - xPositionZero );
		Asura_Physics_MovementQuery xMovementQuery( xInitialPosition, xDisplacement, xOBB,
			Asura_Physics_MovementQuery::ASURA_PMQCP_OBB, usPolyFlagsToIgnore, true,
			true, true, bCollideWithActors, true );
		Asura_Physics_MovementQuery_Results xResults( xMovementQuery );
		if ( xResults.GetAllowedDisplacement().ApproximatelyEquals( xDisplacement ) )
		{
			xPositionZero = xPositionOne;
			xOrientZero   = xOrientOne;
			fParametricTimer += fParametricIncrement;
		}
		else
		{
			return false;
		}
	}

	return true;
}

bool AVP_ServerEntity_Actor::GetAnimMovementOffset( const Asura_Hash_ID uAnimHashID, const float fParametricTime,
												    Asura_Vector_3* pxPosition, Asura_Quat* pxOrientQ /*= NULL*/ ) const
{
	const Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnimByHash( uAnimHashID );
	if ( !pxAnim ) { return false; }

	const Asura_Hierarchy_Skin* pxSkin = NULL;
	const AVP_ClientEntity_Actor* pxClientActor = AVP_ClientEntity_Actor::FindEntity( GetGuid() );
	if ( pxClientActor )
	{
		pxSkin = pxClientActor->GetAnimationHandler()->GetAnimation()->GetSkin();
	}

	return pxAnim->GetMovementOffset( pxSkin, fParametricTime, pxPosition, pxOrientQ );
}

void AVP_ServerEntity_Actor::GetHeadPosition( Asura_Vector_3& xPosition ) const
{
	static const Asura_Hash_ID uHeadTargetMarkerHashID = 0x002fbc62; //"eyes"

	if ( !GetWorldSpaceMarkerPoint( uHeadTargetMarkerHashID, &xPosition, NULL ) )
	{
#ifdef ASURA_ASSERT
		// Check if the client is able to find marker points
		const AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity(GetGuid());
		const Asura_Animation* pxAnim = pxClient ? pxClient->GetAnimation() : NULL;
		const Asura_Hierarchy_Tween* pxTweenStore = pxAnim ? pxAnim->GetTweenStore() : NULL;
		Asura_Assert(!pxTweenStore || !pxTweenStore->IsPositionDataValid(), "AVP_ServerEntity_Actor::GetHeadPosition - Unable to find 'eyes' marker point for head position - using calculated position by hand");
#endif //#ifdef ASURA_ASSERT
		xPosition = GetPosition();

		const Asura_Matrix_3x3& xServerOrientation = GetOrientation();
		const Asura_Vector_3 xUp( xServerOrientation.e12, xServerOrientation.e22, xServerOrientation.e32 );
		xPosition += xUp * GetHeadPositionYOffset();
	}
}

void AVP_ServerEntity_Actor::GetChestPosition( Asura_Vector_3& xPosition ) const
{
	static const Asura_Hash_ID uChestTargetMarkerHashID = 0xe5a85d03; // Asura_GetHashID( "target_1" )

	if ( !GetWorldSpaceMarkerPoint( uChestTargetMarkerHashID, &xPosition, NULL ) )
	{
#ifdef ASURA_ASSERT
		// Check if the client is able to find marker points
		const AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity(GetGuid());
		const Asura_Animation* pxAnim = pxClient ? pxClient->GetAnimation() : NULL;
		const Asura_Hierarchy_Tween* pxTweenStore = pxAnim ? pxAnim->GetTweenStore() : NULL;
		Asura_Assert(!pxTweenStore || !pxTweenStore->IsPositionDataValid(), "AVP_ServerEntity_Actor::GetChestPosition - Unable to find 'target_1' marker point for chest position - using calculated position by hand");
#endif //#ifdef ASURA_ASSERT
		xPosition = GetPosition();

		const Asura_Matrix_3x3& xServerOrientation = GetOrientation();
		const Asura_Vector_3 xUp( xServerOrientation.e12, xServerOrientation.e22, xServerOrientation.e32 );
		xPosition += xUp * GetChestPositionYOffset();
	}
}

bool AVP_ServerEntity_Actor::GetMuzzlePosition( Asura_Vector_3& xPosition ) const
{

	const AVP_Inventory_System* pxInventorySystem	= GetInventorySystem();

	const AVP_Inventory_Item* pxItem = pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON);
	Asura_Assert(pxItem, "GetMuzzlePosition - weapon not found");
	if (!pxItem) 
	{
		xPosition = GetPosition(); 
		return false;
	}

	if (pxItem->GetItemType() != AVP_INVENTORYITEMTYPE_WEAPON_RANGED &&
		pxItem->GetItemType() != AVP_INVENTORYITEMTYPE_PLASMACASTER &&
		pxItem->GetItemType() != AVP_INVENTORYITEMTYPE_WEAPON_FLAMETHROWER )
	{
		xPosition = GetPosition(); 
		return false;
	}

	const AVP_RangedWeapon_SubType* pxSubType = 
		static_cast<const AVP_RangedWeapon_SubType*>(static_cast<const AVP_InventoryItem_Weapon*>(pxItem)->GetSubType(AVP_InventoryWeapon_RangedWeapon::ST_PRIMARY));
	Asura_Assert(pxItem, "GetMuzzlePosition - No Subtype found");

	const AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity(Guid);
	const AVP_Actor_AnimationHandler* pxAnimHandler = pxClient ? pxClient->GetAnimationHandler() : NULL;
	Asura_Hash_ID uMarkerHash = (pxAnimHandler && pxAnimHandler->IsFirstPerson()) ? pxSubType->GetMuzzleMarkerPointFirstPerson() : pxSubType->GetMuzzleMarkerPointThirdPerson();

	if ( !GetWorldSpaceMarkerPoint( uMarkerHash, &xPosition, NULL ) )
	{
		Asura_Assert(pxClient == NULL, "AVP_ServerEntity_Actor::GetMuzzlePosition - Unable to find marker point for Muzzle Position - using actors orientation");
		xPosition = GetPosition();
		return false;
	}
	return true;
}


void AVP_ServerEntity_Actor::GetHeadOrientation( Asura_Matrix_3x3& xOrientation ) const
{
	static const Asura_Hash_ID uHeadTargetMarkerHashID = 0x002fbc62; //"eyes"

	if ( !GetWorldSpaceMarkerPoint( uHeadTargetMarkerHashID, NULL, &xOrientation) )
	{
		Asura_Assert(AVP_ClientEntity_Actor::FindEntity( GetGuid() ) == NULL, "AVP_ServerEntity_Actor::GetHeadOrientation - Unable to find 'eyes' marker point for head orientation - using server orientation");
		xOrientation = GetOrientation();
	}
}

void AVP_ServerEntity_Actor::GetChestOrientation( Asura_Matrix_3x3& xOrientation ) const
{
	static const Asura_Hash_ID uChestTargetMarkerHashID = 0xe5a85d03; // Asura_GetHashID( "target_1" )

	if ( !GetWorldSpaceMarkerPoint( uChestTargetMarkerHashID, NULL, &xOrientation ) )
	{
		Asura_Assert(AVP_ClientEntity_Actor::FindEntity( GetGuid() ) == NULL, "AVP_ServerEntity_Actor::GetChestOrientation - Unable to find 'target_1' marker point for chest orientation - using server orientation");
		xOrientation = GetOrientation();
	}
}

void AVP_ServerEntity_Actor::GetHeadFowardVector( Asura_Vector_3& xForward ) const
{
	Asura_Matrix_3x3 xOrientation;
	GetHeadOrientation( xOrientation );
	xForward.x = xOrientation.e13;
	xForward.y = xOrientation.e23;
	xForward.z = xOrientation.e33;
}

#ifdef ASURA_TOOLS
void AVP_ServerEntity_Actor::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
    PARENT::ApplyChangesForTemplateInstance( pxInstance );

	m_uInitialGamesceneGuid = pxInstance->ConvertGuid( m_uInitialGamesceneGuid );
	
	m_uInitialSquadGuid = pxInstance->ConvertGuid( m_uInitialSquadGuid );
}
#endif //#ifdef ASURA_TOOLS

void AVP_ServerEntity_Actor::SetCollisionDescriptor( const Asura_Hash_ID uCollDescHash )
{
	const AVP_Collision_Descriptor* pxCollDesc = AVP_Collision_Descriptor::GetDataByHash( uCollDescHash );
	Asura_Assertf( pxCollDesc != NULL, ( "No Collision Descriptor found with hash 0x%08x", uCollDescHash ) );
	if ( pxCollDesc != NULL && m_pxDynamicObject != NULL && m_pxDynamicObject->GetCollisionObject() != pxCollDesc->GetCollisionBox() )
	{
		m_pxDynamicObject->SetCollisionObject( pxCollDesc->GetCollisionBox() );
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::IsCloaked
// Author       : Andy Tate
// Date         : 29-Nov-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerEntity_Actor::IsCloaked(const bool bFullyCloaked) const
{
	const AVP_Inventory_Item*	pxCloakItem	= m_pxInventorySystem->GetItem(uAVP_INVENTORY_ITEMHASH_CLOAK);

	if(pxCloakItem == NULL)
	{
		return(false);
	}
	else
	{
		return(pxCloakItem->IsUsing(bFullyCloaked));
	}
}

/// End of function 'AVP_ServerEntity_Actor::IsCloaked'. ///


u_int AVP_ServerEntity_Actor::GetCurrentWPID()
{
	if ( GetBrain() && GetBrain()->IsActive() )
	{
		m_usCurrentWPID = GetActorBrain()->GetNavData().m_usCurrentWPID;
		return m_usCurrentWPID;
	}
	else
	{
		AVP_Navigation::TestParams xParams;
		xParams.m_bRejectNonalignedWPs = true;
		xParams.m_xNormal = Asura_Vector_3( -GetCurrentGravityDirection() );

		AVP_Navigation::UpdateCurrentWaypointID( GetPosition(), m_usCurrentWPID, xParams );
		return m_usCurrentWPID;
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerEntity_Actor::InitialiseInventory
// Author       : Andy Tate
// Date         : 10-Dec-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerEntity_Actor::InitialiseInventory(void)
{
	const Asura_Hash_ID	uBlueprintTypeHash	= GetBlueprintTypeHashID();
	const Asura_Hash_ID	uBlueprintHash		= GetBlueprintHashID();


	// NB. this function gets called before MakePlayer on reload so the players server side inventory system gets temporarily set as master
	// but since the inventory system is then read in from the checkpoint save and the master flag is included in the chunk data it's all OK :)
	if(!IsPlayer())
	{
		m_pxInventorySystem->SetAsMaster();
	}

	m_pxInventorySystem->Reset();

	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uBlueprintTypeHash, uBlueprintHash);
	if(pxBlueprint)
	{
		AVP_Inventory_Group*	pxWeaponGroup	= new AVP_InventoryGroup_Weapons(*m_pxInventorySystem);

		// Initialise the weapons...
		if(pxWeaponGroup == NULL)
		{
			Asura_Assert(false, "Cannot get inventory weapon group to add to.");
			return(false);
		}
		else
		{
			if ( IsPlayer() )
			{
				AVP_Settings::Species eSpecies = GetSpeciesByClassification();
				AVP_Game_Core::CharacterType eCharType;
				if ( AVP_Settings::IsMultiPlayer() )
				{
					switch( eSpecies )
					{
						case AVP_Settings::AVP_SPECIES_ALIEN:		eCharType = AVP_Game_Core::MULTIPLAYER_ALIEN; break;
						default:	Asura_Assert( false, "Unable to determine species" ); // fall through to human
						case AVP_Settings::AVP_SPECIES_HUMAN:		eCharType = AVP_Game_Core::MULTIPLAYER_MARINE; break;
						case AVP_Settings::AVP_SPECIES_PREDATOR:	eCharType = AVP_Game_Core::MULTIPLAYER_PREDATOR; break;
					}
				}
				else
				{
					switch( eSpecies )
					{
						case AVP_Settings::AVP_SPECIES_ALIEN:		eCharType = AVP_Game_Core::SINGLE_PLAYER_ALIEN; break;
						default:	Asura_Assert( false, "Unable to determine species" ); // fall through to human
						case AVP_Settings::AVP_SPECIES_HUMAN:		eCharType = AVP_Game_Core::SINGLE_PLAYER_MARINE; break;
						case AVP_Settings::AVP_SPECIES_PREDATOR:	eCharType = AVP_Game_Core::SINGLE_PLAYER_PREDATOR; break;
					}
				}

				const char* szLevelName = 
#ifdef ASURA_TOOLS
				Tools_SaveAsura::IsEditingMission() ? Tools_SaveAsura::GetMissionFilenameWithoutExtension() : Tools_SaveAsura::GetMainFilename();
#else
				AVP_Settings::GetLevelName();
#endif

				const Asura_Blueprint* pxLevelBlueprint = AVP_Game_Core::GetLevelBlueprint( szLevelName, eCharType );
				if( pxLevelBlueprint )
				{
					AVP_Inventory_Group_Enumerator	xWeaponEnumerator( *pxWeaponGroup, true );
					pxLevelBlueprint->EnumerateParameters( szAVP_BLUEPRINT_ENUMERATIONNAME_WEAPON, uAVP_BLUEPRINT_ENUMERATIONDIGITS_WEAPON, xWeaponEnumerator );

					u_int uIndex = pxLevelBlueprint->GetParameterAsUInt( 0x33799660, 0 ); // InitialWeapon
					if(uIndex > 0) // if they don't have a default weapon, don't try and set it.
					{
						Asura_Hash_ID uHash = uAVP_INVENTORY_WEAPONHASH_WEAPON00+uIndex;
						pxWeaponGroup->SetDefaultItem(uHash);
						pxWeaponGroup->SetCurrentItemToDefaultItem(true);
					}
				}
			}
			else
			{
				AVP_Inventory_Group_Enumerator	xWeaponEnumerator(*pxWeaponGroup);

				// Enumerate the weapons...
				pxBlueprint->EnumerateParameters(szAVP_BLUEPRINT_ENUMERATIONNAME_WEAPON, uAVP_BLUEPRINT_ENUMERATIONDIGITS_WEAPON, xWeaponEnumerator);

				u_int uIndex = pxBlueprint->GetParameterAsUInt(Asura_GetHashID("Weapon"), 0u);
				if(uIndex > 0) // if they don't have a default weapon, don't try and set it.
				{
					Asura_Hash_ID uHash = uAVP_INVENTORY_WEAPONHASH_WEAPON00+uIndex;
					pxWeaponGroup->SetDefaultItem(uHash);
					pxWeaponGroup->SetCurrentItemToDefaultItem(true);
				}
			}
		}

		return(true);
	}
	else
	{
		Asura_Assert(false, "Failed to find blueprint for this server-side actor.");
		return(false);
	}
}

void AVP_ServerEntity_Actor::ProcessActorPlayerCommands( const AVP_Message_ActorPlayerCommands* pxData, float fMessageSentTime )
{
	AVP_Inventory_Group*	pxWeaponGroup	= (m_pxInventorySystem == NULL) ? NULL : m_pxInventorySystem->GetGroup(uAVP_INVENTORY_GROUPHASH_WEAPON);
	Asura_Matrix_3x3		xOrientation;

	SetAimDirection(pxData->GetAimDirection());
	SetLookAtPosition( pxData->GetLookAtPosition() );

	SetLinVelocity(pxData->GetVelocity());
	SetPosition(pxData->GetPosition());
	pxData->GetOrientation().ToMatrix(xOrientation);
	SetOrientation(xOrientation);

	if(pxWeaponGroup == NULL)
	{
		Asura_Assert(false, "Weapon group not created for this Human.");
	}
	else if(pxWeaponGroup->IsCurrentItemActive())
	{
		pxWeaponGroup->SetCurrentItem(pxData->GetCurrentWeaponHash());
	}

	m_fPlayerSpeedParameter = pxData->GetSpeedParameter();
	m_fPlayerMoveAngle = pxData->GetMoveAngle();

	SetMoving( pxData->IsMoving() );
	SetMovingSlow( pxData->IsMovingSlow() );
	SetMovingFast( pxData->IsMovingFast() );

	// MGH 28/1/2009: Client position/orientation fix copied from BF3.
	// always set client authoritative position.
	m_bClientPosAndOriTimeValid = true;
	
	// By the time this data arrives on the server it is old. In theory we should just add latency * 0.5f
	// to compensate for the time is took to arrive. The numbers are exposed for tweaking and on tweaking it
	// seems that adding the whole latency plus an extra 0.1s to cover the time between network updates
	// gives the smoothest result.
	// latency is in milliseconds hence * 0.001f to get seconds.
	m_fClientPosAndOriTime = fMessageSentTime + dbg_fClientPosLookaheadConstant +
		dbg_fClientPosLookaheadLatencyScale * 0.001f * Asura_Network::GetLatencyForPlayer(GetPlayerInfoGuid());
}

void AVP_ServerEntity_Actor::RegenHealthBlocks( u_int uNumBlocks )
{
	m_uNumActiveHealthBlocks = Asura_Maths::Min( m_uNumActiveHealthBlocks + uNumBlocks, m_uMaxNumHealthBlocks );
	SetHealth( Asura_Maths::Min( GetHealth() + ( uNumBlocks * m_fHealthPerBlock ), GetInitialHealth() ) );
}

bool AVP_ServerEntity_Actor::GetClientAnimVelocityOS( const AVP_ClientEntity_Actor& xClient, Asura_AVector3& xAnimVelocity ) const
{
	const AVP_Actor_AnimationHandler* pxAnimHandler = xClient.GetAnimationHandler();
	if ( !pxAnimHandler ) { return false; }

	const Asura_Animation* pxAnim = pxAnimHandler->GetAnimation();
	if ( !pxAnim ) { return false; }

	Asura_Vector_3 xVAnimVelocity;
	const bool bHasLinearVelocity = pxAnim->GetVelocity( &xVAnimVelocity );
	if ( bHasLinearVelocity )
	{
		Asura_Quat xRotation;
		if ( pxAnim->GetMovementOffset( NULL, &xRotation ) )
		{
			xRotation.Invert();
			xVAnimVelocity = xRotation.RotateVector( xVAnimVelocity);
		}
		xAnimVelocity = xVAnimVelocity;
	}
	else
	{
		xAnimVelocity.Zero();
	}

	return true;
}

bool AVP_ServerEntity_Actor::GetClientAnimAngularVelocityOS( const AVP_ClientEntity_Actor& xClient, Asura_Angle& fAngleOut, Asura_AVector3& xAngVelyAxisOut ) const
{
	const AVP_Actor_AnimationHandler* pxAnimHandler = xClient.GetAnimationHandler();
	if ( !pxAnimHandler ) { return false; }

	const Asura_Animation* pxAnim = pxAnimHandler->GetAnimation();
	if ( !pxAnim ) { return false; }

	Asura_Vector_3 xAngVelocityAxis;
	const bool bHasAngVelocity = pxAnim->GetAngularVelocity( &xAngVelocityAxis, &fAngleOut );
	if ( bHasAngVelocity )
	{
		Asura_Quat xRotation;
		if ( pxAnim->GetMovementOffset( NULL, &xRotation ) )
		{
			xRotation.Invert();
			xAngVelocityAxis = xRotation.RotateVector( xAngVelocityAxis);
		}
		xAngVelyAxisOut = xAngVelocityAxis;
	}
	else
	{
		xAngVelyAxisOut = Asura_AVector3( 0.0f, 1.0f, 0.0f );
		fAngleOut = 0.0f;
	}

	return true;
}

const AVP_Collision_Descriptor* AVP_ServerEntity_Actor::GetDefaultCollisionDescriptor()
{
	const Asura_Blueprint* pxBP = GetBlueprint();
	Asura_Assertf( pxBP, ( "Unable to get blueprint for actor %d, has it been set?", GetGuid() ) );
	if ( pxBP )
	{
		Asura_Value xVal;
		if ( pxBP->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_DEFAULT_POSTURE, xVal ) )
		{
			const AVP_Collision_Descriptor* pxCollDesc = AVP_Collision_Descriptor::GetDataByHash( xVal.GetHashIDValue() );
			return pxCollDesc;
		}
#ifdef ASURA_ASSERT
		else
		{
			Asura_Assert( false, "Unable to get \"DefaultPosture\" parameter from BP" );
		}
#endif
	}
	// else
	return NULL;
}

void AVP_ServerEntity_Actor::OnApplyDamage( const AVP_EntityDamage_ApplyToEntity& xDamage )
{
	PARENT::OnApplyDamage( xDamage );

	m_fTimeSinceDamage    = 0.0f;
	m_fLastDamageTime     = Asura_Timers::GetLocalScaledTime();
	m_fLastDamageInterval = 0.0f;
	m_fLastDamageAmount   = xDamage.GetAmount();
}

const AVP_InventoryItem_Weapon* AVP_ServerEntity_Actor::GetCurrentWeapon() const
{
	const AVP_Inventory_System* pxInventory = GetInventorySystem();
	if(pxInventory)
	{
		AVP_Inventory_Group* pxGroup = pxInventory->GetGroup(uAVP_INVENTORY_GROUPHASH_WEAPON);
		if(pxGroup)
		{
			return static_cast<const AVP_InventoryItem_Weapon*> (pxGroup->GetCurrentInventoryItem());
		}
	}
	return NULL;
}

bool AVP_ServerEntity_Actor::IsDamageIgnoredByThisPlayer( const AVP_EntityDamage_RequestVictim& xDamage ) const
{
	// Is it team based?
	if (AVP_Multiplayer_GameRules::IsTeamBasedGame())
	{
		// Check if friendly fire is off
		if (!AVP_Multiplayer_GameRules::GetFriendlyFireOn())
		{
			// Abort any damage coming from players in the same team
			if (IsPlayer())
			{					
				Asura_Guid uAttackerGuid = xDamage.GetFirerGuid(); 
				AVP_ServerEntity_Actor* pxAttackerActor = AVP_ServerEntity_Actor::FindEntity( uAttackerGuid ); 
				Asura_Guid uAttackerPlayerInfoGuid = pxAttackerActor ? pxAttackerActor->GetPlayerInfoGuid() : ASURA_GUID_UNREGISTERED; 
				AVP_ServerEntity_PlayerInfo* pxAttackerPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( uAttackerPlayerInfoGuid ); 
				AVP_ServerEntity_PlayerInfo* pxVictimPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( GetPlayerInfoGuid() ); 
				// But don't stop self damage!
				if (pxAttackerPlayerInfo && pxVictimPlayerInfo 
					&& pxAttackerPlayerInfo->GetGuid() != pxVictimPlayerInfo->GetGuid())
				{
					// Are this two players in the same team?
					if (pxAttackerPlayerInfo->GetMultiplayerTeam() == pxVictimPlayerInfo->GetMultiplayerTeam())
					{
						// We should ignore this damage 
						return true; 
					}
				}
			}
		}
	}

	// Is it role based?
	if (AVP_Multiplayer_GameRules::IsRoleBasedGame())
	{
		// Abort any damage coming from players playing the same role
		if (IsPlayer())
		{					
			Asura_Guid uAttackerGuid = xDamage.GetFirerGuid(); 
			AVP_ServerEntity_Actor* pxAttackerActor = AVP_ServerEntity_Actor::FindEntity( uAttackerGuid ); 
			Asura_Guid uAttackerPlayerInfoGuid = pxAttackerActor ? pxAttackerActor->GetPlayerInfoGuid() : ASURA_GUID_UNREGISTERED; 
			AVP_ServerEntity_PlayerInfo* pxAttackerPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( uAttackerPlayerInfoGuid ); 
			AVP_ServerEntity_PlayerInfo* pxVictimPlayerInfo = AVP_ServerEntity_PlayerInfo::FindEntity( GetPlayerInfoGuid() ); 
			// But don't stop self damage!
			if (pxAttackerPlayerInfo && pxVictimPlayerInfo 
				&& pxAttackerPlayerInfo->GetGuid() != pxVictimPlayerInfo->GetGuid())
			{
				// Are this two players playing the same role?
				if (pxAttackerPlayerInfo->GetMultiplayerRole() == pxVictimPlayerInfo->GetMultiplayerRole())
				{
					// We should ignore this damage 
					return true; 
				}
			}
		}
	}
	
	return false; 
}

void AVP_ServerEntity_Actor::UpdateDamageCounters( const AVP_EntityDamage_RequestVictim& xDamage )
{
	// Only do this for players
	if (IsPlayer())
	{
		// Check if the damage comes from an actor or it is unregistered
		Asura_Guid uAttackerGuid = xDamage.GetFirerGuid();
		if (uAttackerGuid != ASURA_GUID_UNREGISTERED)
		{
			// The damage is coming from an actor. Check that both actors are players
			AVP_ServerEntity_Actor* pxAttackerActor = AVP_ServerEntity_Actor::FindEntity( uAttackerGuid ); 
			if (pxAttackerActor)
			{
				if (pxAttackerActor->IsPlayer())
				{
					// Create a new damage list if we don't have one
					if (!m_pxReceivedDamageList)
					{
						m_pxReceivedDamageList = new Asura_Hashed_List<AVP_Multiplayer_DamageCounter>; 
					}

					// Find the damage counter for the attacker actor
					AVP_Multiplayer_DamageCounter* pxAttackerDamageCounter = m_pxReceivedDamageList->FindEntity( uAttackerGuid ); 

					// Create one if it doesn't exist
					if (!pxAttackerDamageCounter)
					{
						pxAttackerDamageCounter = new AVP_Multiplayer_DamageCounter( uAttackerGuid ); 
						m_pxReceivedDamageList->AddToList( pxAttackerDamageCounter );
					}

					// Add the new damage to the damage already received from that player
					pxAttackerDamageCounter->SetDamage( pxAttackerDamageCounter->GetDamage() + xDamage.GetAmount() );
				}
			}
			else
			{
				// It's fine, you can receive damage from non-actor entities
			}
		}
		else
		{
			// It's fine, unregistered damage doesn't need to be accounted for
		}
	}
}

void AVP_ServerEntity_Actor::AwardLoadsOfDamage()
{
	const float fLoads = 10000;
	const AVP_EntityDamage_RequestVictim xDamage( ASURA_HASH_ID_UNSET, ASURA_GUID_UNREGISTERED, ASURA_HASH_ID_UNSET, GetGuid(), fLoads, GetPosition(), Asura_Maths::ZeroVector3, Asura_Maths::ZeroVector3, 1.0f );
	TakeDamageExtended( xDamage, false );
}



