//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerVehicle_Dropship.h
// Author : Sharon Galtzur
// Date   : 24-Sep-2008
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef ___AVP_SERVERVEHICLE_DROPSHIP_H___
#define ___AVP_SERVERVEHICLE_DROPSHIP_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_List.h"

// avp
#include "Entity/PhysicalObject/Actor/Vehicles/AVP_ServerActor_Vehicle.h"
#include "Entity/PhysicalObject/Actor/Vehicles/Dropship/AVP_Messages_Dropship.h"
#include "Actor/AVP_Actor_DynamicStat.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimSequenceData.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Collision_Object;
class Axon_Project_Brain;

// C L A S S E S /////////////////////////////////////////////////////////////

class AVP_ServerVehicle_Dropship : public AVP_ServerActor_Vehicle
{
public:
	// Overridden functions...
	virtual	void Update(void);
	virtual Axon_Brain* CreateBrain(void);
	virtual	bool HandleMessage(const Asura_Message* pxMessage);

	virtual bool GetHasCreateMessage() const { return !IsDead() || IsADeadBody(); }
	virtual void MakeCreateMessage( Asura_Guid uRequestingClient, bool bSendGuaranteed = false );

	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );

	Asura_Guid GetStartNode() const { return m_uGS2StartNode; }

	const Asura_Vector_3& GetVelocity() const { return m_pxDynamicObject->GetLinVelocity(); }
	const Asura_Vector_3& GetAcceleration() const { return m_xCurrentAcceleration; }

	// Static list functions...
	static AVP_ServerVehicle_Dropship* FindEntity(const Asura_Guid uGuid) { return(s_xList.FindEntity(uGuid)); }
	static const Asura_Hashed_List<AVP_ServerVehicle_Dropship>*	GetList(void) { return(&s_xList); }

	static void Create( Asura_Guid uGuid ) { new AVP_ServerVehicle_Dropship( uGuid ); }
	static AVP_ServerVehicle_Dropship* CreateAndReturn( Asura_Guid uGuid ) { return new AVP_ServerVehicle_Dropship( uGuid ); }

	void SetNextDestination(const Asura_Vector_3& xDestination);

	void LaserOn(const Asura_Vector_3& xTarget);
	void LaserOff();

	bool GetLaserTarget(Asura_Vector_3& xTarget) const;
	float GetVeloictyFraction() const;

	bool AreFrontPodsOpened() const; 
	bool AreFrontPodsClosed() const; 
	bool AreBackPodsOpened() const; 
	bool AreBackPodsClosed() const; 
	bool AreBackPodsDoorsOpened() const;
	bool AreBackPodsDoorsClosed() const;

	void OpenFrontPods();
	void CloseFrontPods();
	void OpenBackPods();
	void CloseBackPods();
	void OpenBackPodsDoors();
	void CloseBackPodsDoors();

	void MissileLaunched();
	void MissileDestroyed();
	bool IsAnyMissileActive();

protected:
	AVP_ServerVehicle_Dropship(const Asura_Guid uGUID);
	virtual ~AVP_ServerVehicle_Dropship ();
	virtual void UpdateMovement();

	virtual	void MakeClientUpdateMessage(void);

private:

	const Asura_Quat CalculateOrientation();
	const Asura_AVector3 CalculateVelocity() const;
	void PIDUpVelocity(float fDeltaTime);
	void PIDPlaneVelocity(float fDeltaTime);
	void PIDTurnVelocity(float fDeltaTime);

	void UpdateUpVelocity(float fDeltaTime);
	void UpdatePlaneVelocity(float fDeltaTime);
	void UpdateTurnVelocity(float fDeltaTime);
	void SendRequestAnimation(AVP_Message_Dropship_RequestAnimation::RequestType eRequestType, float fAnimTime = 0.0f, bool bIsGuaranteed = false);

	class PIDController
	{
	public:
		PIDController(float fP, float fI, float fD, float fMaxOutput);
		void Update(float fDeltaTime, float fPlantValue, float fDesiredValue);
		float GetValue() { return m_fOutput; }

	private:

		float m_fP;
		float m_fI;
		float m_fD;
		float m_fMaxOutput;

		float m_fOutput;

		float m_fLastPlantValue;
		float m_fLastLastPlantValue;      
	};

	Asura_Guid m_uGS2StartNode;
	Asura_Vector_3 m_xDestination;
	Asura_Vector_3 m_xCurrentVelocity;
	Asura_Vector_3 m_xCurrentVelocityNoise;
	float m_fTimeTillVelocityNoiseChanges;
	Asura_Vector_3 m_xLastVelocity;
	Asura_Vector_3 m_xCurrentHorizontalAcceleration;
	Asura_Vector_3 m_xCurrentAcceleration;
	Asura_Vector_3 m_xCurrentLean;
	float m_fCurrentDropAcceleration;

	Asura_Vector_3 m_xCurrentAim;
	float m_fCurrentTurnVelocity;

	// control system
	PIDController m_xPIDY;
	PIDController m_xPIDX;
	PIDController m_xPIDZ;
	PIDController m_xPIDAim;
	float m_fTimeSinceControlUpdate;

	// Laser
	bool m_bIsLaserOn;
	Asura_Vector_3 m_xLaserTarget;
	Asura_Vector_3 m_xLaserActualTarget;

	// keep track of missiles
	u_int m_uNumberOfMissiles;

	// sound pitch value
	float m_fVelocityFraction;

	// Private typedefs...
	typedef AVP_ServerActor_Vehicle PARENT; 

	// Friend classes...
	friend class ToolsAVP_Entity_Dropship;


	static Asura_Hashed_List<AVP_ServerVehicle_Dropship> s_xList;

	enum { CURRENT_CHUNK_VERSION = 1 };
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___AVP_SERVERVEHICLE_DROPSHIP_H___
