
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_AlienQueen_Messages.h
// Author : Andy McCabe
// Date   : 20-Jan-2009
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_ALIENQUEEN_MESSAGES_H__
#define __AVP_ALIENQUEEN_MESSAGES_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Actor/Vertebrates/AlienQueen/AVP_ServerActor_AlienQueen.h"

#include "Core/AxonAVP_Messages.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_NetMsg;
class Asura_BitPacker;
class Asura_BitUnpacker; 

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Msg_AlienQueen_LaunchAcidSpit_Projectile
// Author       : Andy McCabe
// Date         : 20-Jan-2009
//
// Description  : Msg from client to server to launch an acid spit projectile.
////////////////////////////////////////////////////////////////////////////////

class AVP_Msg_AlienQueen_LaunchAcidSpit_Projectile
{
public:
	void SetTargetGuid(const Asura_Guid uTargetGuid) { m_uTargetGuid = uTargetGuid; }
	Asura_Guid GetTargetGuid() const { return m_uTargetGuid; }
private:
	Asura_Guid m_uTargetGuid;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Msg_Add_Entity_To_Queen_Target_List
// Author       : Callum Hope
// Date         : 7/4/09
//
// Description  : Message to add an entity to the queens target list in her knowledge system. 
//				  This is actually only sent to marines and predators, but this is only for ease of Desiger work in tools. 
//				  Its handled in the queen brain, via the queen server - easy for designers, hard for code. 
////////////////////////////////////////////////////////////////////////////////

class AVP_Msg_Add_Entity_To_Queen_Target_List
{
	public:

		void Initialise( void )
		{
			m_uVersion = 0;
			m_uQueenGuid = ASURA_GUID_UNREGISTERED;
		}

		void			SetQueenGuid(Asura_Guid uGuid)	{ m_uQueenGuid = uGuid; }
		Asura_Guid		GetQueenGuid(void) const		{ return m_uQueenGuid; }

		u_int			m_uVersion;
		Asura_Guid		m_uQueenGuid;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Msg_Remove_Entity_From_Queen_Target_List
// Author       : Callum Hope
// Date         : 7/4/09
//
// Description  : Message to remove an entity from the queens target list in her knowledge system. 
//				  This is actually only sent to marines and predators, but this is only for ease of Desiger work in tools. 
//				  Its handled in the queen brain, via the queen server - easy for designers, hard for code. 
////////////////////////////////////////////////////////////////////////////////

class AVP_Msg_Remove_Entity_From_Queen_Target_List
{
	public:

		void Initialise( void )
		{
			m_uVersion = 0;
			m_uQueenGuid = ASURA_GUID_UNREGISTERED;
		}

		void			SetQueenGuid(Asura_Guid uGuid)	{ m_uQueenGuid = uGuid; }
		Asura_Guid		GetQueenGuid(void) const		{ return m_uQueenGuid; }

		u_int			m_uVersion;
		Asura_Guid		m_uQueenGuid;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Msg_Alien_Queen_Start_Roar_Behaviour
// Author       : Callum Hope
// Date         : 5/5/09
//
// Description  : Message to make the queen stop everything and Roar on the spot 
//				  The target Guid is needed so that we can make sure the queen is facing the right direction before starting roar. 
////////////////////////////////////////////////////////////////////////////////

class AVP_Msg_Alien_Queen_Start_Roar_Behaviour
{
public:

	void Initialise( void )
	{
		m_uVersion = 1;
		m_uTargetGuid = ASURA_GUID_UNREGISTERED;
		m_bRoarAtPlayer = false;
	}

	void			SetTargetGuid( Asura_Guid uGuid )	{ m_uTargetGuid = uGuid; }
	Asura_Guid		GetTargetGuid( void ) const		{ return m_uTargetGuid; }

	void			SetRoarAtPlayer( bool bRoarAtPlayer )	{ m_bRoarAtPlayer = bRoarAtPlayer; }
	bool			GetRoarAtPlayer( void ) const		{ return m_bRoarAtPlayer; }

	u_int			m_uVersion;
	Asura_Guid		m_uTargetGuid;
	bool			m_bRoarAtPlayer;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Msg_Alien_Queen_Start_Roar_Behaviour
// Author       : Callum Hope
// Date         : 5/5/09
//
// Description  : Message to set the behaviour of the queen through tools. 
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_SetAlienQueenBehaviour
{
public:

	void Initialise( void )
	{
		m_uVersion = uCURRENT_VERSION;
		m_iBehaviour = AVP_AlienQueenBehaviourData::WANDER;
	}

	u_int m_uVersion;
	int m_iBehaviour;

	static const u_int uCURRENT_VERSION = 1;
};

#endif   // ifndef __AVP_ALIENQUEEN_MESSAGES_H__

