
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientActor_FaceHugger.cpp
// Author : Andy Tate
// Modified to FaceHugger : Chris Wallace
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ClientActor_FaceHugger.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_Timers.h"
#include "Network/Asura_Message.h"
#include "Physics/Asura_Physics_Object.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Viewport.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "SpecialFX/PFX/Asura_PFX_Entity_Effect.h"

// Game independent header-files...
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AIAnimStateData.h"
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AnimationHandler.h"
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_Description.h"
#include "Animation/AVP_AnimFlags.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/AVP_Movement_System.h"
#include "Moves/Vertebrates/FaceHugger/AVP_FaceHugger_MovesJump.h"
#include "Moves/Vertebrates/FaceHugger/AVP_FaceHugger_MovesMelee.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientActor_FaceHugger
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ClientActor_FaceHugger>	AVP_ClientActor_FaceHugger::s_xList;

AVP_ClientActor_FaceHugger::AVP_ClientActor_FaceHugger(void)
: PARENT(AVP_PolygonFlag_IgnoreForFaceHuggerCollision)
, m_uDroolPFXEffect(ASURA_PFX_INVALID_EFFECT_ID)
, m_uFaceHuggerFlags(0)
{
	Classification = AVP_ENTITYCLASS_ALIENFACEHUGGER;

	SetNumFeet(4);
	
	// Initialise the entity...
	SetTimeLagType(TimeLag_LocalPlayer);

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}

AVP_ClientActor_FaceHugger::~AVP_ClientActor_FaceHugger()
{
	// Don't forget to remove us from the list...
	s_xList.RemoveFromList(this);
}


void AVP_ClientActor_FaceHugger::Create(const Asura_Message* pxMessage)
{
	AVP_ClientActor_FaceHugger*	pxClient = new AVP_ClientActor_FaceHugger;
	pxClient->HandleMessage(pxMessage);
}

void AVP_ClientActor_FaceHugger::PostCreateInitialise()
{
	// If there is no animation handler yet, create it.
	if( !m_pxAnimationHandler && GetBlueprintHashID() != ASURA_HASH_ID_UNSET )
	{
		CreateAnimationHandler();
	}

	PARENT::PostCreateInitialise();
}

bool AVP_ClientActor_FaceHugger::InitialiseMoves( void )
{
	if(PARENT::InitialiseMoves())
	{
		Asura_Movement_System& xMovementSystem = *m_pxMovementSystem;
		Asura_Hash_ID uFirstPersonBlueprintHash, uThirdPersonBlueprintHash;
		GetCurrentMeleeBlueprintIDs(uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Countered(xMovementSystem);
		new AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Kill(xMovementSystem);
		new AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Start(xMovementSystem);
		new AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Countered(xMovementSystem);
		new AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Kill(xMovementSystem);
		new AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Start(xMovementSystem);
		new AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_FaceHugger_TP_Move_Melee_MarineDeath(xMovementSystem);
		new AVP_FaceHugger_TP_Move_Melee_MarineGrapple(xMovementSystem);
		new AVP_FaceHugger_TP_Move_Melee_PredatorGrapple(xMovementSystem);
		
		new AVP_FaceHugger_Move_Melee_JumpAttack_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Move_FaceHugger_Jump(xMovementSystem);

		new AVP_Move_FaceHugger_Falling(xMovementSystem);

		new AVP_FaceHugger_TP_Move_Melee_MarineDeathHug(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		return(true);
	}
	else
	{
		return(false);
	}
}

void AVP_ClientActor_FaceHugger::CreateAnimationHandler()
{
	Asura_Assert( m_pxAnimationHandler == NULL, "Already got an animation handler" );
	m_pxAnimationHandler = new AVP_FaceHugger_AnimationHandler(*this);
	m_pxAnimationHandler->Initialise();
}

Asura_Hash_ID AVP_ClientActor_FaceHugger::GetDeathAnimation(const AVP_EntityDamage_ApplyToEntity& xDamage) const
{
	// Always ragdoll from explosions
	bool bAlwaysRagdoll = false;
	Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_DAMAGE, xDamage.GetTypeHash() );
	if ( pxBP )
	{
		Asura_Value xVal;
		if ( pxBP->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ALWAYSRAGDOLL, xVal ) )
		{
			bAlwaysRagdoll = xVal.GetBoolValue();
		}
	}
	if ( bAlwaysRagdoll ) { return ASURA_HASH_ID_UNSET; }


	const Asura_Blueprint* pxWeaponBlueprint = GetBlueprint();

	Asura_Value xValue;
	if (!pxWeaponBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_THIRDPERSONDEATH, xValue))
	{ 
		return ASURA_HASH_ID_UNSET; 
	}

	return xValue.GetHashIDValue();
}



void AVP_ClientActor_FaceHugger::GetCurrentMeleeBlueprintIDs(Asura_Hash_ID& uFirstPersonID, Asura_Hash_ID& uThirdPersonID) const
{
	uFirstPersonID = uThirdPersonID = ASURA_HASH_ID_UNSET;

	// As the alien, we just get them from our actor blueprint.
	const Asura_Blueprint* pxBlueprint = GetBlueprint();
	if (!pxBlueprint) { return; }

	Asura_Value xValue;
	if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_MELEE_FIRSTPERSON, xValue))
	{
		uFirstPersonID = xValue.GetHashIDValue();
	}

	if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_MELEE_THIRDPERSON, xValue))
	{
		uThirdPersonID = xValue.GetHashIDValue();
	}
}

bool AVP_ClientActor_FaceHugger::HandleMessage( const Asura_Message* pxMessage )
{
	if ( pxMessage->GetMsgID() == ASURA_MSG_CLIENTUPDATE )
	{
		const AVP_Message_Vertebrate_ClientUpdate* pxData = static_cast<const AVP_Message_Vertebrate_ClientUpdate*>(pxMessage->GetData());
		if ( pxData )
		{
			const bool	bIsLocalPlayer	= IsPlayer() && AVP_Game_Core::IsLocalPlayer(GetPlayerInfoGuid());
			ProcessClientUpdate( pxData, pxMessage->GetSentTime(), !bIsLocalPlayer );
		}
		return true;
	}
	else
	{
		return PARENT::HandleMessage( pxMessage );
	}
}

void AVP_ClientActor_FaceHugger::ProcessClientUpdate( const AVP_Message_Vertebrate_ClientUpdate* pxData, const float fServerTime, const bool bProcessPosAndOri /*= true*/ )
{
	PARENT::ProcessClientUpdate( pxData, fServerTime, bProcessPosAndOri );
}

const AVP_Actor_AnimStateData* AVP_ClientActor_FaceHugger::GetStateDataFromMesssageData( const u_int uState, const AVP_Message_Actor_ClientUpdate* pxActorData )
{
	return AVP_FaceHugger_AIAnimStateData::GetAnimStateData( GetAnimStateBlueprintType(), uState );
}

const AVP_Actor_AnimStateData* AVP_ClientActor_FaceHugger::GetStateData( const u_int uState, const u_int uGroundType, const u_int uLimbDamage, const bool bIsAiming, const u_int uCoverType)
{
	return AVP_FaceHugger_AIAnimStateData::GetAnimStateData( GetAnimStateBlueprintType(), uState );
}

/// End of file. ///
