
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerActor_FaceHugger.h
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SERVERACTOR_FACEHUGGER_H__
#define __AVP_SERVERACTOR_FACEHUGGER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

// asura
#include "Core/Asura_HashedList.h"

// axon
#include "Core/BrainTypes/AVP_Brain_Actor_FaceHugger.h"

// Game
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

//class AVP_Message_FaceHuggerPlayerCommands;

class Asura_Collision_Object;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_ServerActor_FaceHugger
// Author       : Andy Tate
// Date         : 3-Oct-2006
//
// Description  : This class handles the operation of the Predator actor, the
//				  Predator can be either player controlled or AI controlled.
////////////////////////////////////////////////////////////////////////////////

class AVP_ServerActor_FaceHugger : public AVP_ServerActor_Vertebrate
{
public:
	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	virtual void PostLoadInitialise();
	virtual void MakeAI();

	virtual	bool HandleMessage(const Asura_Message* pxMessage);
	virtual	void Update();

	void SetMaster(Asura_Guid uMasterGuid);
	void ChangeAttackMode();
	void SetAlwaysAggressive( bool bAlwaysAggressive );

	bool IsHugging() const { return m_uAttachedToGuid != ASURA_GUID_UNREGISTERED; }
	bool IsAggressive() const { return GetFaceHuggerBrain()->IsAggressive(); }

	static void Create(Asura_Guid uGUID) { new AVP_ServerActor_FaceHugger(uGUID); }
	static AVP_ServerActor_FaceHugger* CreateAndReturn(Asura_Guid uGUID) { return new AVP_ServerActor_FaceHugger(uGUID); }

	static	AVP_ServerActor_FaceHugger*								FindEntity(const Asura_Guid uGuid)	{ return(s_xList.FindEntity(uGuid)); }
	static	const Asura_Hashed_List<AVP_ServerActor_FaceHugger>*	GetList(void)						{ return(&s_xList); }

	protected:

	AVP_ServerActor_FaceHugger(const Asura_Guid uGUID);
	virtual ~AVP_ServerActor_FaceHugger();

	virtual Axon_Brain* CreateBrain();

	virtual float GetHeadPositionYOffset() const { return -0.25f; }
	virtual float GetChestPositionYOffset() const { return -0.25f; }

private:

	AVP_Brain_Actor_FaceHugger *GetFaceHuggerBrain() { return static_cast<AVP_Brain_Actor_FaceHugger *>(GetActorBrain()); }
	const AVP_Brain_Actor_FaceHugger *GetFaceHuggerBrain() const { return static_cast<const AVP_Brain_Actor_FaceHugger *>(GetActorBrain()); }

	typedef AVP_ServerActor_Vertebrate PARENT;

#if defined(ASURA_TOOLS)
	friend class ToolsAVP_Entity_FaceHugger;
#endif

	Asura_Guid m_uAttachedToGuid;

	static Asura_Hashed_List<AVP_ServerActor_FaceHugger> s_xList;

	AVP_ServerActor_FaceHugger(void);
	AVP_ServerActor_FaceHugger(const AVP_ServerActor_FaceHugger& xOther);
	AVP_ServerActor_FaceHugger&	operator=(const AVP_ServerActor_FaceHugger& xOther);

	static const u_int uCURRENT_CHUNK_VERSION = 1;
};

#endif   // ifndef __AVP_SERVERACTOR_FACEHUGGER_H__


/// End of file. ///
