
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientActor_Human.cpp
// Author : Andy Tate
// Modified to Human : Chris Wallace
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"

// Asura independent header-files...
#include "Animation/Asura_AnimationBlend.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
#include "Core/Asura_MaterialResponse_System.h"
#endif	// defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
#include "Core/Asura_HashedLocalisedText.h"
#include "Core/Asura_Timers.h"
#include "Entity/Asura_PosAndOriInterpolator.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Project_Skin_Elements.h"
#include "Rendering/Asura_Viewport.h"
#include "SpecialFX/Asura_Decal_Manager.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "SpecialFX/PFX/Asura_PFX_Entity_Effect.h"
#include "Sound/Asura_Dialogue_System.h"
#include "Sound/Asura_Sound.h"

// Game independent header-files...
#include "Actor/AVP_Actor_Selector.h"
#include "Actor/AVP_Actor_PickupHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_AnimationHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_CameraHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_Description.h"
#include "Actor/Vertebrates/Human/AVP_Human_Focus.h"
#include "Actor/Vertebrates/Human/AVP_Human_HUDHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_InputHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_InteractionHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_JumpHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_PlayerAnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_PlayerAnimStateHandler.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Messages.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_Messages_Human.h"
#include "Entity/PhysicalObject/Gib/AVP_ClientEntity_Gib.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Messages.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_HumanFlare.h"
#include "Inventory/Items/AVP_InventoryItem_Stim.h"
#include "Inventory/Items/AVP_InventoryItem_Torch.h"
#include "Inventory/Items/AVP_InventoryItem_VisionMode.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/AVP_Move_Interaction.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesCover.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesGamescene.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesInteraction.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesMantle.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesMelee.h"
#include "Moves/Vertebrates/Human/AVP_Human_Moves_MoveToFloor.h"
#include "Sound/Asura_Dialogue_System.h"
#include "Sound/Asura_Sound_Event_System.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const Asura_Hash_ID g_uNightVisionHash = 0x54046078;	// VM_NightVision

DEBUGVAR float	dbg_fADSLerpSpeed		= 10.0f;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////
Asura_Hashed_List<AVP_ClientActor_Human>	AVP_ClientActor_Human::s_xList;

#if defined(ASURA_USE_DIALOGUE_SYSTEM)
Asura_Handle	AVP_ClientActor_Human::s_iActorDialogueHandle = ASURA_INVALID_HANDLE;
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)

static const int g_aiWeaponSelectorKey[ 3 ] = 
{
	AVP_Actor_InputHandler::WEAPON_SELECTOR_SLOT_UP,
	AVP_Actor_InputHandler::WEAPON_SELECTOR_SLOT_LEFT,
	AVP_Actor_InputHandler::WEAPON_SELECTOR_SLOT_RIGHT,
};

static const AVP_GUIMenu_System::STORED_WIDGETS g_aeWeaponSelectorSlotGuiElement[ 3 ] = 
{
	AVP_GUIMenu_System::WEAPON_SELECTOR_SLOT_UP,
	AVP_GUIMenu_System::WEAPON_SELECTOR_SLOT_LEFT,
	AVP_GUIMenu_System::WEAPON_SELECTOR_SLOT_RIGHT,
};

static const AVP_Actor_Selector::Selector_Directions g_aiWeaponValidSlots[ 3 ] = 
{
	AVP_Actor_Selector::SELECTOR_TOP,
	AVP_Actor_Selector::SELECTOR_LEFT,
	AVP_Actor_Selector::SELECTOR_RIGHT,
};

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

bool AVP_ClientActor_Human::InitialiseInventory(void)
{
	if(PARENT::InitialiseInventory())
	{
		
		AVP_Inventory_Group* pxGroup = m_pxInventorySystem->GetPermanentlyActiveItemGroup();
		AddPermenantlyActiveItems(pxGroup);

		if ( AVP_Game_Core::IsLocalPlayer( GetGuid() ) )
		{
			m_pxWeaponSelector = new AVP_Actor_Selector( *this, 3, g_aiWeaponSelectorKey, g_aiWeaponValidSlots, uAVP_INVENTORY_GROUPHASH_WEAPON, AVP_Actor_HUDHandler::GetShowWeaponSelectorConsoleVarPtr() );
			m_pxWeaponSelector->SetSlotGuiElements( g_aeWeaponSelectorSlotGuiElement );
		}

		return(true);
	}
	else
	{
		return(false);
	}
}

void AVP_ClientActor_Human::AddPermenantlyActiveItems(AVP_Inventory_Group* pxGroup)
{
	const Asura_Blueprint* pxBlueprint = GetBlueprint();
	if(pxBlueprint != NULL)
	{
		Asura_Value	xValue;

		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_TORCH, xValue))
		{
			Asura_Hash_ID uTorchBPHash = xValue.GetHashIDValue();
			AVP_InventoryItem_Torch* pxTorch = new AVP_InventoryItem_Torch(*pxGroup, uTorchBPHash, "Torch");
			if ( pxTorch )
			{
				pxTorch->InitialiseFromBlueprint();
			}
			m_uTorchHash = pxTorch->GetHashInParentGroup();
		}
		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_FLARE, xValue))
		{
			Asura_Hash_ID uFlareBPHash = xValue.GetHashIDValue();
			AVP_InventoryItem_HumanFlare* pxFlare = new AVP_InventoryItem_HumanFlare(*pxGroup, uFlareBPHash, "Flare");
			if ( pxFlare )
			{
				pxFlare->InitialiseFromBlueprint();
			}
		}
		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_STIM, xValue))
		{
			Asura_Hash_ID uStimBPHash = xValue.GetHashIDValue();
			AVP_InventoryItem_Stim* pxStim = new AVP_InventoryItem_Stim(*pxGroup, uStimBPHash, "Stim");
			if ( pxStim )
			{
				pxStim->InitialiseFromBlueprint();
			}
		}
	}
}

bool AVP_ClientActor_Human::InitialiseMoves( void )
{
	if ( PARENT::InitialiseMoves() )
	{
		Asura_Movement_System& xMovementSystem = *m_pxMovementSystem;

		new AVP_Move_Human_MoveToFloor( xMovementSystem );

		new AVP_Move_Human_EnterCover( xMovementSystem );
		new AVP_Move_Human_PeekEmerge( xMovementSystem );
		new AVP_Move_Human_IntoPeekEmerge( xMovementSystem );
		new AVP_Move_Human_StepOutOfCover( xMovementSystem );
		new AVP_Move_Human_MountGamescene( xMovementSystem );
		new AVP_Move_Human_Mantle( xMovementSystem );
		new AVP_Human_Move_Melee_Retaliation_Start( xMovementSystem );

		// Interaction moves
		new AVP_Human_Move_Interaction_GrabHackingTool(xMovementSystem);
		new AVP_Human_Move_Interaction_LetGoHackingTool(xMovementSystem);
		new AVP_Human_Move_Interaction_LetGoBattery(xMovementSystem);
		new AVP_Human_Move_Interaction_GrabBattery(xMovementSystem);
		new AVP_Human_Move_Interaction_PlugBattery(xMovementSystem);
		new AVP_Human_Move_Interaction_UnPlugBattery(xMovementSystem);
		new AVP_Human_Move_Interaction_PlugHackingTool(xMovementSystem);
		new AVP_Human_Move_Interaction_UnPlugHackingTool(xMovementSystem);

		Asura_Move* pxIntMove = xMovementSystem.GetMove( AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() );
		Asura_Assert( pxIntMove, "" );
		if ( pxIntMove )
		{
			pxIntMove->AddNextMove( AVP_Human_Move_Interaction_UnPlugBattery::GetStaticHash() );
			pxIntMove->AddNextMove( AVP_Human_Move_Interaction_UnPlugHackingTool::GetStaticHash() );
			pxIntMove->AddNextMove( AVP_Human_Move_Interaction_PlugHackingTool::GetStaticHash() );
		}

		return true;
	}

	return false;
}


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Human::AVP_ClientActor_Human
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////
AVP_ClientActor_Human::AVP_ClientActor_Human(void)
: AVP_ClientActor_Vertebrate(AVP_PolygonFlag_IgnoreForHumanCollision)
, m_pxWeaponSelector( NULL )
, m_fArmour(100.0f)
, m_uType(0)
, m_uWeapon(0)
, m_uTorchHash( ASURA_HASH_ID_UNSET )
, m_uPickedUpBuddyGuid(ASURA_GUID_UNREGISTERED)
, m_uNumFlaresActive(0)
, m_fTargetADSValue( 0.0f )
, m_fCurrentADSValue( 0.0f )
, m_fStoredNearClip( 0.1f )
, m_uHumanFlags(0)
{
	Classification = AVP_ENTITYCLASS_HUMAN;

	// Initialise the entity...
	SetTimeLagType(TimeLag_LocalPlayer);

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}
/// End of function 'AVP_ClientActor_Human::AVP_ClientActor_Human'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Human::~AVP_ClientActor_Human
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////
AVP_ClientActor_Human::~AVP_ClientActor_Human ()
{
	// Remove whatever we need to...
	delete(m_pxWeaponSelector);

#if defined(ASURA_USE_DIALOGUE_SYSTEM)
	Asura_Dialogue_System::StopByGuid( GetGuid() );
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)

	// Don't forget to remove us from the list...
	s_xList.RemoveFromList(this);
}
/// End of function 'AVP_ClientActor_Human::~AVP_ClientActor_Human'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Human::Create
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////
AVP_ClientActor_Human* AVP_ClientActor_Human::Create(const Asura_Message* pxMessage)
{
	AVP_ClientActor_Human*	pxClient	= new AVP_ClientActor_Human;
		
	// Initialise the entity...
	pxClient->HandleMessage(pxMessage);

	// Return the entity...
	return(pxClient);

}
/// End of function 'AVP_ClientActor_Human::Create'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Human::PostCreateInitalise
// Author       : 
// Date         : 
////////////////////////////////////////////////////////////////////////////////
void AVP_ClientActor_Human::PostCreateInitialise(void)
{
	// If there is no animation handler yet, create it.
	if (!m_pxAnimationHandler && GetBlueprintHashID() != ASURA_HASH_ID_UNSET )
	{
		CreateAnimationHandler();
	}

	PARENT::PostCreateInitialise();

	if ( IsPlayer() )
	{
		if( m_pxPlayerStateHandler == NULL )
		{
			m_pxPlayerStateHandler = new AVP_Human_PlayerAnimStateHandler( *this );
		}
		m_pxPlayerStateHandler->Initialise();
	}
}
/// End of function 'AVP_ClientActor_Human::PostCreateInitalise'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Human::Update
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////
void AVP_ClientActor_Human::Update(void)
{
	// Firstly, ensure that the parent is up-to-date...
	PARENT::Update();

#if defined(ASURA_USE_DIALOGUE_SYSTEM)
	// Tidy-up any outstanding sound-events...
	if ( !Asura_Dialogue_System::IsGuidCurrentlyPlayingDialogue( GetGuid() ) )
	{
		Asura_Dialogue_System::StopByGuid( GetGuid() );
	}
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)

	if ( AVP_Game_Core::IsLocalPlayer( GetGuid() ) )
	{
		UpdateAimDownSights();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Human::HandleMessage
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////
bool AVP_ClientActor_Human::HandleMessage(const Asura_Message* pxMessage)
{
	// What sort of message is it?
	switch(pxMessage->GetMsgID())
	{
		case ASURA_MSG_CLIENTUPDATE:
		{
			const bool	bIsLocalPlayer	= IsPlayer() && AVP_Game_Core::IsLocalPlayer(GetPlayerInfoGuid());

			ProcessClientUpdate(static_cast<const AVP_Message_HumanClientUpdate*>(pxMessage->GetData()), pxMessage->GetSentTime(), !bIsLocalPlayer);
			return(true);
		}

		case AVP_MSGID_PLAY_DIALOGUE_EVENT:
		{
			const AVP_Message_PlayDialogueEvent* pxData = static_cast<const AVP_Message_PlayDialogueEvent*>(pxMessage->GetData());

			if (pxData)
			{
				Asura_Hash_ID uDialogueHash = pxData->GetDialogueHashID();

				PlayDialogue(uDialogueHash);
			}
			
			return true;
		}

		case AVP_MSGID_SET_PICKUPABLE:
		{
			m_bPickUpAble = true;
		}
		return true;

		case AVP_MSGID_SET_NOT_PICKUPABLE:
		{
			m_bPickUpAble = false;
		}
		return true;

		case AVP_MSGID_SET_DROPPABLE:
		{
			m_bDroppable = true;
		}
		return true;

		case AVP_MSGID_SET_NOT_DROPPABLE:
		{
			m_bDroppable = false;
		}
		return true;

		case AVP_MSGID_PICK_TEQUILA_UP:
		{
			SetInvisibleFlag( true );
		}
		return true;

		case AVP_MSGID_PUT_TEQUILA_DOWN:
		{
			SetInvisibleFlag( false );
		}
		return true;

		case AVP_MSGID_REQUEST_INTERACTION:
		{
			const AVP_Message_Actor_RequestInteraction* pxData = static_cast<const AVP_Message_Actor_RequestInteraction*>(pxMessage->GetData());

			switch ( pxData->GetInteractionType() )
			{
				case AVP_INTERACTION_TYPE_PICKUP_BUDDY:
				{
					m_uPickedUpBuddyGuid = pxData->GetActorGuid();
				}
				return true;

				default:
				{
					return PARENT::HandleMessage( pxMessage );
				}
			}
		}

		case AVP_MSGID_DROP_PICKUP:
		{
			m_uPickedUpBuddyGuid = ASURA_GUID_UNREGISTERED;
		}
		return true;

		default:
		{
			return(PARENT::HandleMessage(pxMessage));
		}
	}		// End of switch.
}
/// End of function 'AVP_ClientActor_Human::HandleMessage'. ///

void AVP_ClientActor_Human::ReadFromClientCreateStream(Asura_Chunk_Memory_Stream& xStream, float fServerTime)
{
	u_int uVersion = 0;
	xStream >> uVersion;

	Asura_Assert(uVersion <= AVP_ServerActor_Human::CURRENT_CLIENTCREATE_VERSION, "Bad chunk version in marine client create!");
	
	// Call the base-function...
	PARENT::ReadFromClientCreateStream( xStream, fServerTime );

	u_int uAlertLevel;
	
	xStream >> uAlertLevel;

	Asura_Hash_ID uPageHash;
	Asura_Hash_ID uTextHash;
	xStream >> uPageHash;
	xStream >> uTextHash;
	GetNameText().SetTextPageHash(uPageHash);
	GetNameText().SetTextID(uTextHash);
	
	m_eAlertLevel = static_cast<AVP_ClientActor_Vertebrate::AVP_AlertLevel>(uAlertLevel);
	

	union
	{
		u_int		uServerFlags;
		struct
		{
			u_int	bCrouching		  : 1,
					bAiming		      : 1,
					eCoverType		  : 2,
					bLegsDamaged	  : 1,
					bStartAsInjured	  : 1,
					bPickUpAble		  : 1,
					bDroppable		  : 1,
					bPickedUp		  : 1,
					bHasMotionTracker : 1,
									  : 0;
		} uFlagsStruct; // must be named to compile on PS3
	};

	Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( uServerFlags );
	xStream >> xBitfield;

	if ( !AVP_Game_Core::IsLocalPlayer( GetGuid() ))
	{
		SetCrouching( uFlagsStruct.bCrouching, false );
		SetAiming( uFlagsStruct.bAiming );
	}
	if ( uFlagsStruct.bPickedUp )
	{
		SetInvisibleFlag( true );
	}
	m_bPickUpAble       = uFlagsStruct.bPickUpAble;
	m_bDroppable        = uFlagsStruct.bDroppable;
	m_bHasMotionTracker = uFlagsStruct.bHasMotionTracker;

	xStream >> m_uPickedUpBuddyGuid;

}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Human::ProcessClientUpdate
// Author       : Andy Tate
// Date         : 17-Oct-2006
////////////////////////////////////////////////////////////////////////////////
void AVP_ClientActor_Human::ProcessClientUpdate(const AVP_Message_HumanClientUpdate* pxData, const float fServerTime, const bool bProcessPosAndOri)
{
	// Call the base-function...
	PARENT::ProcessClientUpdate( pxData, fServerTime, bProcessPosAndOri );
	
	if ( !AVP_Game_Core::IsLocalPlayer( GetGuid() ))
	{
		SetCrouching( pxData->IsCrouching(), false );
		SetAiming( pxData->IsAiming() );
	}

	if ( !IsPlayer() )
	{
		if ( !Asura_Network::IsServer() ) // don't do this on local AI client entities as it's already been done
		{
			m_eCoverType = pxData->GetCoverType();
		}
	}

	m_eAlertLevel = (AVP_ClientActor_Vertebrate::AVP_AlertLevel)pxData->GetAlertLevel();
}
/// End of function 'AVP_ClientActor_Human::ProcessClientUpdate'. ///


void AVP_ClientActor_Human::GetCurrentBlockBlueprintHashes(Asura_Hash_ID& uBlueprintTypeHash, Asura_Hash_ID& uFirstPersonBlueprintHash, Asura_Hash_ID& uThirdPersonBlueprintHash) const
{
	uBlueprintTypeHash = uAVP_BLUEPRINT_TYPE_MOVESHUMAN;
	uFirstPersonBlueprintHash = uThirdPersonBlueprintHash = ASURA_HASH_ID_UNSET;

	const AVP_InventoryItem_Weapon* pxCurrentWeapon = m_pxInventorySystem ? static_cast<const AVP_InventoryItem_Weapon*>(m_pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON)) : NULL;
	if (!pxCurrentWeapon) { return; }

	const Asura_Blueprint* pxBlueprint = pxCurrentWeapon->GetBlueprint();
	if (!pxBlueprint) { return; }

	Asura_Value xValue;

	// The human always gets the standard pair of parameters.
	if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_BLOCK_FIRSTPERSON, xValue))
	{
		uFirstPersonBlueprintHash = xValue.GetHashIDValue();
	}

	if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_BLOCK_THIRDPERSON, xValue))
	{
		uThirdPersonBlueprintHash = xValue.GetHashIDValue();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Human::RenderHUD
// Author       : Andy Tate
// Date         : 6-Oct-2006
////////////////////////////////////////////////////////////////////////////////
void AVP_ClientActor_Human::RenderHUD(void) const
{
	// Render the parent...
	PARENT::RenderHUD();
}

void AVP_ClientActor_Human::Render(void)
{
	PARENT::Render();
}
/// End of function 'AVP_ClientActor_Human::RenderHUD'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Human::MakePlayer
// Author       : Chris Wallace
// Date         : 5-Feb-2007
////////////////////////////////////////////////////////////////////////////////
void AVP_ClientActor_Human::MakePlayer(Asura_Guid uPlayerGuid)
{
	if (uPlayerGuid == m_uPlayerInfoGuid) { return; }
	// Check if all local players on this machine is aware of their puppets yet, since MakePlayer can fall over if they aren't.
	if (!AVP_Game_Core::GetCanMakePlayerActors()) { return; }

	const bool bLocalPlayer = AVP_Game_Core::IsLocalPlayer(uPlayerGuid);

	PARENT::MakePlayer(uPlayerGuid);

	if( m_pxPlayerStateHandler == NULL )
	{
		m_pxPlayerStateHandler = new AVP_Human_PlayerAnimStateHandler( *this );
	}
	m_pxPlayerStateHandler->Initialise();

	if ( bLocalPlayer )
	{
		if(!m_pxHUDHandler)
		{
			m_pxHUDHandler	= new AVP_Human_HUDHandler(*this);
// 		if ( m_pxHUDHandler )
// 		{
// 			Asura_Blueprint* pxBlueprint = NULL;
// 			Asura_Blueprint_System::GetBlueprint(GetBlueprintTypeHashID(),GetBlueprintHashID(), pxBlueprint);
// 			if(pxBlueprint)
// 			{
// 				static_cast<AVP_Vertebrate_HUDHandler*>(m_pxHUDHandler)->InitialiseFromBlueprint( *pxBlueprint );
// 			}
// 
// 		}
		}

		if(!m_pxCameraHandler)
		{
			m_pxCameraHandler	= new AVP_Human_CameraHandler(Guid);
			m_pxCameraHandler->Reset();
		}

		if(!m_pxInputHandler)
		{
			m_pxInputHandler	= new AVP_Human_InputHandler(Guid);
			m_pxInputHandler->Reset();
			if (m_pxInputHandler)
			{
				m_pxInputHandler->RefreshBlueprintData();
			}
		}

		if(!m_pxJumpHandler)
		{
			m_pxJumpHandler = new AVP_Human_JumpHandler(*this);
		}

		if (!m_pxFocus)
		{
			m_pxFocus = new AVP_Human_Focus(*this);
		}

		if (!m_pxInteractionHandler)
		{
			m_pxInteractionHandler = new AVP_Human_InteractionHandler(*this);
		}
	}
}

void AVP_ClientActor_Human::MakeAI()
{
	PARENT::MakeAI();
}
/// End of function 'AVP_ClientActor_Human::MakePlayer'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientActor_Human::OnApplyDamage
// Author       : Chris Wallace
// Date         : 24-July-2007
////////////////////////////////////////////////////////////////////////////////
void AVP_ClientActor_Human::OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	PARENT::OnApplyDamage(xDamage);

	const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_DAMAGE, xDamage.GetTypeHash() );
	if ( pxBlueprint )
	{
		Asura_Value xVal;
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_DISRUPT_EQUIPMENT, xVal ) )
		{
			if ( xVal.GetBoolValue() )
			{
				AVP_InventoryItem_Torch* pxTorch = static_cast< AVP_InventoryItem_Torch* >( m_pxInventorySystem->GetItem( m_uTorchHash ) );
				if ( pxTorch )
				{
					pxTorch->Disrupt( *this );
				}

				AVP_InventoryItem_VisionMode* pxVM = static_cast< AVP_InventoryItem_VisionMode* >( m_pxInventorySystem->GetItem( g_uNightVisionHash ) );
				if ( pxVM )
				{
					pxVM->Disrupt();
				}

				if ( m_pxHUDHandler != NULL )
				{
					// AJK; AVP Functionality for tutorial; check if func allowed
					if ( AVP_Settings::IsMotionTrackerFuncOn() )
					{
						static_cast< AVP_Human_HUDHandler* >( m_pxHUDHandler )->DisruptMotionTracker();
					}
				}
			}
		}
	}
}
/// End of function 'AVP_ClientActor_Human::OnApplyDamage'. ///

//------------------------------------------------------------------------------
// AVP_ClientActor_Human :: OnDeath
//------------------------------------------------------------------------------
void AVP_ClientActor_Human::OnDeath( const AVP_EntityDamage_ApplyToEntity& xDamage )
{
	PARENT::OnDeath( xDamage );


	if ( IsPlayer() )
	{
		AVP_InventoryItem_Torch* pxTorch = static_cast< AVP_InventoryItem_Torch* >( m_pxInventorySystem->GetItem( m_uTorchHash ) );

		if ( pxTorch != NULL )
		{
			pxTorch->RequestEndUsage(AVP_InventoryItem_Torch::TT_AUTOMATIC);
		}
	}
}
/// End of function 'AVP_ClientActor_Human::OnDeath'. ///


void AVP_ClientActor_Human::CreateAnimationHandler()
{
	Asura_Assert( m_pxAnimationHandler == NULL, "Already got an animation handler" );
	m_pxAnimationHandler = new AVP_Human_AnimationHandler(*this);
	m_pxAnimationHandler->Initialise();
}

//------------------------------------------------------------------------------
// AVP_ClientActor_Human :: PlayDialogue
//------------------------------------------------------------------------------
void AVP_ClientActor_Human::PlayDialogue(Asura_Hash_ID uDialogueHash)
{
#if defined(ASURA_USE_DIALOGUE_SYSTEM)
	//if ( ( s_iActorDialogueHandle == ASURA_INVALID_HANDLE ) && ( uDialogueHash != ASURA_HASH_ID_UNSET ) )
	{
		const Asura_Dialogue_Event* pxDialogue = Asura_Dialogue_System::FindEvent( uDialogueHash );

		if ( pxDialogue )
		{
			if ( !Asura_Dialogue_System::IsGuidCurrentlyPlayingDialogue( GetGuid() ) )
			{
				Asura_Dialogue_System::PlayEvent( pxDialogue, GetGuid(), NULL );
			}
		}
	}
#endif	//defined(ASURA_USE_DIALOGUE_SYSTEM)
}

void AVP_ClientActor_Human::UpdateAimDownSights()
{
	const AVP_InventoryItem_Weapon* pxWeapon = static_cast<AVP_InventoryItem_Weapon*>(m_pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON));
	if ( pxWeapon != NULL )
	{
		const float fADSFOVDecrease = pxWeapon->GetADSFOVDecrease();
		if ( fADSFOVDecrease > 0.0f )
		{
			if ( m_fCurrentADSValue < m_fTargetADSValue )
			{
				m_fCurrentADSValue += Asura_Maths::Min( m_fTargetADSValue - m_fCurrentADSValue, dbg_fADSLerpSpeed * Asura_Timers::GetGameTimeSlice() );
			}
			else if ( m_fCurrentADSValue > m_fTargetADSValue )
			{
				m_fCurrentADSValue += Asura_Maths::Max( m_fTargetADSValue - m_fCurrentADSValue , -dbg_fADSLerpSpeed * Asura_Timers::GetGameTimeSlice() ); // max as value is negative
			}
			
			AVP_Human_CameraHandler* pxCameraHandler = static_cast< AVP_Human_CameraHandler* >( m_pxCameraHandler );

			if ( m_fCurrentADSValue > 0.0f && !IsAiming() )
			{
				SetAiming( true );
				m_fStoredNearClip = Asura_Camera::GetNearClip();
			}
			else if ( m_fCurrentADSValue == 0.0f && IsAiming() )
			{
				SetAiming( false );
			}

			if ( pxCameraHandler != NULL )
			{
				pxCameraHandler->SetFOVModifier( m_fCurrentADSValue * fADSFOVDecrease, AVP_Human_CameraHandler::FOVMOD_ADS );
				Asura_Camera::SetNearClip( Asura_Maths::Lerp( m_fStoredNearClip, pxWeapon->GetADSNearClipDistance(), m_fCurrentADSValue ) );
			}
		}
	}
}

bool AVP_ClientActor_Human::IsBoneInTorso( const int iBoneID )
{
	return AVP_Human_Description::GetElementFromBoneID( iBoneID ) == AVP_SET_BODY;
}

bool AVP_ClientActor_Human::MakeRagdoll()
{
	bool bSuccess = PARENT::MakeRagdoll();

	if (bSuccess)
	{
		// We also need to clear out our facial animation.
		AVP_Human_AnimationHandler* pxHumanAnimHandler = static_cast<AVP_Human_AnimationHandler*>(m_pxAnimationHandler);
		pxHumanAnimHandler->DisableFacialAnimation();
	}

	return bSuccess;
}

const AVP_Actor_AnimStateData* AVP_ClientActor_Human::GetAnimStateDataForWeapon( const Asura_Hash_ID uWeaponBPHash ) const
{
	if ( IsAIActor() )
	{
		return AVP_Human_AIAnimStateData::GetAnimStateData( GetAnimStateBlueprintType(), uWeaponBPHash, GetAIAnimState(), IsAiming(), GetCoverType() );
	}
	else if ( IsPlayer() && m_pxAnimationHandler && m_pxPlayerStateHandler )
	{
		return AVP_Human_PlayerAnimStateData::GetSingleton().GetAnimStateData( GetAnimStateBlueprintType(), uWeaponBPHash, m_pxAnimationHandler->IsFirstPerson(), IsAiming(), IsCrouching(), IsJumping(), IsFalling(), static_cast< const AVP_Vertebrate_PlayerAnimStateHandler* >( m_pxPlayerStateHandler )->GetBestMoveSpeed() );
	}
	else
	{
		Asura_Assertf( false, ( "Couldn't get state data for entity %d", GetGuid() ) );
		return NULL;
	}
}

void AVP_ClientActor_Human::GetFootPosition( u_int uFootItr, Asura_Vector_3& xFootPos )
{
	if ( m_pxAnimationHandler && m_pxAnimationHandler->GetAnimation() )
	{
		m_pxAnimationHandler->GetAnimation()->GetBoneLocation( AVP_Human_Description::GetFootBoneID( uFootItr, m_pxAnimationHandler->IsFirstPerson() ), &xFootPos, NULL, true );
	}
}

void AVP_ClientActor_Human::SetCrouching( const bool bSet, const bool bCheckIfPossible )
{
	bool bDoIt = true;
	if ( bCheckIfPossible && AVP_Game_Core::IsLocalPlayer( GetGuid() ) )
	{
		const AVP_Inventory_System* pxInvSys = GetInventorySystem();
		if ( pxInvSys )
		{
			const AVP_Inventory_Item* pxItem = pxInvSys->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON );
			if ( pxItem )
			{
				const AVP_Actor_AnimStateData* pxStateData = AVP_Human_PlayerAnimStateData::GetSingleton().GetAnimStateData( GetAnimStateBlueprintType(), pxItem->GetBlueprintHash(), AVP_Game_Core::IsLocalPlayer( GetGuid() ), false, bSet, false, false, AVP_Vertebrate_PlayerAnimStateHandler::MOVESPEED_IDLE );
				const AVP_Collision_Descriptor* pxCollDesc = pxStateData ? AVP_Collision_Descriptor::GetDataByHash( pxStateData->GetCollisionDescriptorHash() ) : NULL;
				if ( pxCollDesc )
				{
					const Asura_Collision_Box* pxCollBox = pxCollDesc->GetCollisionBox();
					if ( pxCollBox )
					{
						bDoIt = CanChangeCollisionObject( *pxCollBox );
					}
				}
			}
		}
	}
	if ( bDoIt )
	{
		m_bCrouching = bSet;
	}
}

void AVP_ClientActor_Human::SetAiming( const bool bSet )
{
	if ( bool(m_bAiming) != bSet )
	{
		m_bAiming = bSet;
		AVP_Animation_BodyRig* pxBodyRig = m_pxAnimationHandler ? m_pxAnimationHandler->GetBodyRig() : NULL;
		if ( pxBodyRig && !IsPlayer() )
		{
			// based on data from http://www.ece.tufts.edu/~vanvo/Visual-project.html and assuming aiming means the human is concentrating on the target
			if ( bSet )
			{
				pxBodyRig->SetBlinkRate( 2.0f, 5.0f );
			}
			else
			{
				pxBodyRig->SetBlinkRate( 15.0f, 30.0f );
			}
			pxBodyRig->Blink();
		}
	}
}

AVP_InventoryItem_Torch* AVP_ClientActor_Human::GetTorchItem(void) const
{
	if (m_pxInventorySystem)
	{
		// The torch is always identified by the same hash, no need to go looking through blueprints.
		AVP_Inventory_Item*	pxItem = m_pxInventorySystem->GetItem( uAVP_INVENTORY_ITEMHASH_TORCH );

		return static_cast<AVP_InventoryItem_Torch*>(pxItem);
	}
	else
	{
		Asura_Assert(false, "Failed to find inventory-system for this human - something's gone badly wrong.");
		return NULL;
	}
}

const AVP_Actor_AnimStateData* AVP_ClientActor_Human::GetStateDataFromMesssageData( const u_int uState, const AVP_Message_Actor_ClientUpdate* pxActorData )
{
	const AVP_Message_HumanClientUpdate* pxData = static_cast< const AVP_Message_HumanClientUpdate* >( pxActorData );
	if ( GetInventorySystem() )
	{
		const AVP_Inventory_Item* pxItem = GetInventorySystem()->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON );
		if ( pxItem )
		{
			return AVP_Human_AIAnimStateData::GetAnimStateData( GetAnimStateBlueprintType(), pxItem->GetBlueprintHash(), uState, pxData->IsAiming(), pxData->GetCoverType() );
		}
	}
	// else
	return NULL;
}

const AVP_Actor_AnimStateData* AVP_ClientActor_Human::GetStateData( const u_int uState, const u_int uGroundType, const u_int uLimbDamage, const bool bIsAiming, const u_int uCoverType )
{
	if ( GetInventorySystem() )
	{
		const AVP_Inventory_Item* pxItem = GetInventorySystem()->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON );
		if ( pxItem )
		{
			return AVP_Human_AIAnimStateData::GetAnimStateData( GetAnimStateBlueprintType(), pxItem->GetBlueprintHash(), uState, bIsAiming, static_cast<AVP_Human_AIAnimStateMachine::CoverType>(uCoverType) );
		}
	}
	// else
	return NULL;
}

/// End of file. ///
