
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerActor_Human.cpp
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_BitPacker.h"
#include "Core/Asura_Timers.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Physics/Asura_Physics.h"

// Game independent header-files...
#include "Actor/AVP_Actor_InputHandler.h"
#include "Actor/AVP_Actor_InteractionHandler.h"
#include "Actor/AVP_Actor_ObjectiveHandler.h"
#include "Actor/AVP_Actor_PickupHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_MeleeCounter_System.h"
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateMachine.h"
#include "Actor/Vertebrates/Human/AVP_Human_AnimationHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_Description.h"
#include "Actor/Vertebrates/Human/AVP_Human_MeleeCounter_System.h"
#include "Actor/Vertebrates/Human/AVP_Human_MoraleHandler.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_Recon.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AVP_ServerNode_GS2Node.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AlienQueen/AVP_AlienQueen_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_Messages_Human.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_Sentrygun_Messages.h"
#include "Entity/PhysicalObject/Interact/InvisiSwitch/AVP_ClientEntity_InteractInvisiSwitch.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/BatteryReceptacle/AVP_ClientEntity_InteractBatteryReceptacle.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/BatteryReceptacle/AVP_ServerEntity_InteractBatteryReceptacle.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/HackingPoint/AVP_ServerEntity_InteractHackingPoint.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Messages.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Tool.h"
#include "Entity/PhysicalObject/Pickup/AVP_ClientEntity_Pickup.h"
#include "Entity/PhysicalObject/Pickup/AVP_ServerEntity_Pickup.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerEntity_Projectile.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_HumanFlare.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_HumanFlare.h"
#include "Inventory/Items/AVP_InventoryItem_Stim.h"
#include "Inventory/Items/AVP_InventoryItem_Torch.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/AVP_Move_Interaction.h"
#include "Moves/AVP_Moves.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesInteraction.h"
#include "Moves/Vertebrates/Human/AVP_Human_Moves_MoveToFloor.h"

// Axon
#include "Core/Axon_Project_System.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Marine.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Civilian.h"
#include "Knowledge/AVP_Knowledge_System.h"
#include "Knowledge/AVP_Knowledge_System_Actor.h"
#include "Knowledge/Axon_Knowledge.h"
#include "Tactical/AVP_Tactical_System.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#endif

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

// Predator movement speeds...

DEBUGVAR float	dbg_fAimingStateChangeTweenTime			= 0.2f;
DEBUGVAR float	dbg_fCrouchingStateChangeTweenTime		= 0.2f;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ServerActor_Human>	AVP_ServerActor_Human::s_xList;

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::Update
// Author       : Chris Wallace
// Date         : 6-Feb-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Human::Update(void)
{
	float fHealthBeforeRecharge = GetHealth();
	float f70PercentOfMaxHealth = GetInitialHealth() * 0.7f;

	// Call the base-function...
	PARENT::Update();

	// If we arent a marine delete the morale handler.
	if ((GetClassification() != AVP_ENTITYCLASS_MARINE) && (m_pxMoraleHandler))
	{
		delete m_pxMoraleHandler;
		m_pxMoraleHandler = NULL;
	}

	if (!IsPlayer())
	{
		if ((fHealthBeforeRecharge < f70PercentOfMaxHealth) && (GetHealth() > f70PercentOfMaxHealth))
		{
			if (m_pxMoraleHandler)
			{
				m_pxMoraleHandler->EffectMorale(AVP_Human_MoraleHandler::AVP_MORALE_EFFECTOR_HEALTH_RECOVERED_ABOVE_70_PERCENT);
			}
		}

		if (m_pxMoraleHandler)
		{
			m_pxMoraleHandler->Update();
		}

		ProcessCurrentObjective();

		if ( !m_bLegsDamaged )
		{
			bool bIsLeftLegDstroyed = !IsSkinElementAvailable(AVP_SET_LEFT_LEG) || !IsSkinElementAvailable(AVP_SET_LEFT_KNEE);
			bool bIsRightLegDstroyed = !IsSkinElementAvailable(AVP_SET_RIGHT_LEG) || !IsSkinElementAvailable(AVP_SET_RIGHT_KNEE);

			if ( bIsLeftLegDstroyed || bIsRightLegDstroyed )
			{
				m_bLegsDamaged = true;
				SetIsAllowedToFlinch( false );
			}
		}
	}
	else
	{
		ProcessFreindlyMelee();
		
		CheckForAlliesinLOF();
	}

	if ( IsCarryingBudddy() )
	{
		AVP_ServerActor_Human* pxBuddy = AVP_ServerActor_Human::FindEntity( m_uPickedUpBuddyGuid );
		if ( pxBuddy )
		{
			const Asura_Matrix_3x3& xOri = GetOrientation();
			pxBuddy->TeleportTo( GetPosition() - Asura_Vector_3( xOri.e11, xOri.e21, xOri.e31 ), xOri, true );
		}
	}
}

void AVP_ServerActor_Human::UpdateMovement()
{
	if ( !m_bPickedUp )
	{
		PARENT::UpdateMovement();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::CheckForAlliesinLOF
// Author       : Gareth Thoams
// Date         : 02 / 03 / 2009
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Human::CheckForAlliesinLOF()
{
	m_fLOSUpdateTimer += Asura_Timers::GetFrameTime() * Asura_Timers::GetGameTimeScale();

	if ( m_fLOSUpdateTimer > 0.5f )
	{
		m_fLOSUpdateTimer = 0.0f;

		Asura_Vector_3		xMuzPos;
		Asura_Matrix_3x3	xMuzOri;

		if ( GetWorldSpaceMarkerPoint( 0x0001a7d2 /* muz */, &xMuzPos, &xMuzOri ) )
		{
			Asura_Vector_3 xMuzFwd( xMuzOri.e13, xMuzOri.e23, xMuzOri.e33 );

			const Asura_LOS_InputData	xInput( xMuzPos, xMuzFwd, GetGuid(), 20.0f, false );
			const Asura_LOS_OutputData*	pxOutput = Asura_LOS::TestAgainstEntities( &xInput );

			if ( pxOutput->m_uHitEntity != 0 )
			{
				for (Asura_Hashed_List_It<AVP_ServerActor_Human> it(AVP_ServerActor_Human::GetList()); !it.Done(); it.Next())
				{
					AVP_ServerActor_Human* pxEntity = it.GetCurrent();

					if ( pxEntity && !pxEntity->IsPlayer() && !pxEntity->IsDead() && 
						( pxEntity->GetClassification() == AVP_ENTITYCLASS_MARINE || pxEntity->GetClassification() == AVP_ENTITYCLASS_CIVILIAN ) )
					{
						if ( pxOutput->m_uHitEntity == pxEntity->GetGuid() )
						{
							Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_CLEAR_PLAYERS_LOF );

							if ( pxMessage )
							{
								pxMessage->SetToGuid( pxEntity->GetGuid() );
								pxMessage->SetFromGuid( pxEntity->GetGuid() );
								pxEntity->HandleMessage( pxMessage );
								Asura_Network::DestroyMessage( pxMessage );
							}
						}
					}
				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::OnApplyDamage
// Author       : Andy Tate
// Date         : 26-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Human::OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	PARENT::OnApplyDamage(xDamage);

//	Asura_Guid uAdditionalInfo = ASURA_GUID_UNREGISTERED;
//
//	// If this is a projectile we are interested in who shot the projectile
//	if (xDamage.GetFromGuid() != ASURA_GUID_UNREGISTERED)
//	{
//		AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( xDamage.GetFromGuid() );
//
//		if (pxActor)
//		{
//			uAdditionalInfo = pxActor->GetGuid();
//		}
//		else
//		{
//			pxActor = AVP_ServerEntity_Actor::FindEntity( xDamage.GetParentGuid() );
//
//			if (pxActor)
//			{
//				uAdditionalInfo = pxActor->GetGuid();
//			}
//		}
//	}
//
//	Axon_System::GetSoundRegistry()->RegisterSound(GetGuid(), GetClassification(), GetPosition(), Axon_Project_System::GetHurtSoundRange(), AVP_SOUND_TYPE_HUMAN_HURT, uAdditionalInfo);
}

/// End of function 'AVP_ServerActor_Human::OnApplyDamage'. ///

//------------------------------------------------------------------------------
// Function     : AVP_ServerActor_Human::PlayDialogue
//------------------------------------------------------------------------------
void AVP_ServerActor_Human::PlayDialogue(DialogueType eDialogType)
{
	if (!IsPlayer())
	{
		Asura_Hash_ID	uDialogueHash = GetDialogueHashIDFromDialoguetype(eDialogType);

		if (uDialogueHash != ASURA_HASH_ID_UNSET)
		{
			Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_PLAY_DIALOGUE_EVENT, sizeof(AVP_Message_PlayDialogueEvent));

			if (pxMessage)
			{
 				AVP_Message_PlayDialogueEvent* pxData = static_cast<AVP_Message_PlayDialogueEvent*>(pxMessage->GetData());

				pxData->SetDialogueHashID(uDialogueHash);
 				pxMessage->SetFromGuid(GetGuid());
				pxMessage->SetToGuid(GetGuid());

				Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
			}
		}
	}

}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::TakeDamageExtended
// Author       : Stuart Perry
// Date         : 25-Jun-2007
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Human::TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient)
{
	if (!IsPlayer())
	{
		// If we have been hit by acid effect our morale
		if (xDamage.GetTypeHash() == uAVP_BLUEPRINT_NAME_DAMAGE_ACID)
		{
			if (m_pxMoraleHandler)
			{
				m_pxMoraleHandler->EffectMorale(AVP_Human_MoraleHandler::AVP_MORALE_EFFECTOR_HIT_BY_ACID);
			}
		}
	}

	float fHealthBeforeDamage	= GetHealth();
	float f50PercentHealth		= GetInitialHealth() * 0.5f;

	PARENT::TakeDamageExtended( xDamage, bForceFatal, bSendMessageToClient );

	if (!IsPlayer())
	{
		// check we still have both our legs, if not kill us
		bool bIsLeftLegDstroyed = !IsSkinElementAvailable(AVP_SET_LEFT_LEG) || !IsSkinElementAvailable(AVP_SET_LEFT_KNEE);
		bool bIsRightLegDstroyed = !IsSkinElementAvailable(AVP_SET_RIGHT_LEG) || !IsSkinElementAvailable(AVP_SET_RIGHT_KNEE);

		if ( bIsLeftLegDstroyed && bIsRightLegDstroyed )
		{
			PARENT::TakeDamageExtended( xDamage, true, bSendMessageToClient );
		}

		// Effect morale if our health goes below 50% of maximum
		if ((fHealthBeforeDamage > f50PercentHealth) && (GetHealth() < f50PercentHealth))
		{
			if (m_pxMoraleHandler)
			{
				m_pxMoraleHandler->EffectMorale(AVP_Human_MoraleHandler::AVP_MORALE_EFFECTOR_HEALTH_DROPPING_BELOW_50_PERCENT);
			}
		}

		if ( GetActorBrain() )
		{
			if ( GetClassification() == AVP_ENTITYCLASS_CIVILIAN )
			{
				(static_cast<AVP_Brain_Actor_Civilian*>(GetActorBrain()))->GetKnowledgeSystem()->SetAlertLevel( AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS );
				(static_cast<AVP_Brain_Actor_Civilian*>(GetActorBrain()))->GetKnowledgeSystem()->ResetAlertTimer();
			}
			else
			{
				(static_cast<AVP_Brain_Actor_Marine*>(GetActorBrain()))->GetKnowledgeSystem()->SetAlertLevel( AVP_Knowledge_System_Actor::AGGRESSIVE_ALERTNESS );
				(static_cast<AVP_Brain_Actor_Marine*>(GetActorBrain()))->GetKnowledgeSystem()->ResetAlertTimer();
			}
		}
	}

	// If this is a projectile we are interested in who shot the projectile
	if (xDamage.GetFromGuid() != ASURA_GUID_UNREGISTERED && xDamage.GetTypeHash() == uAVP_BLUEPRINT_NAME_DAMAGE_MELEE)
	{
		AVP_ServerActor_Human* pxActor = AVP_ServerActor_Human::FindEntity( xDamage.GetFromGuid() );

		if ( pxActor )
		{
			if ( !IsPlayer() && pxActor->IsPlayer() && AVP_Tactical_System::GetAllegiance(*this, *pxActor) == AVP_ALLEGIANCE_FRIEND )
			{
				pxActor->PlayerMeleedFriendly( GetGuid() );
			}
		}
	}

	AVP_ServerEntity_Actor* pxDamageActor = AVP_ServerEntity_Actor::FindEntity( xDamage.GetFromGuid() );

	if ( pxDamageActor )
	{
		m_uLastGuidToDamageUs = pxDamageActor->GetGuid();
	}
	else
	{
		pxDamageActor = AVP_ServerEntity_Actor::FindEntity( xDamage.GetFirerGuid() );

		if ( pxDamageActor )
		{
			m_uLastGuidToDamageUs = pxDamageActor->GetGuid();
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::MakeClientUpdateMessage
// Author       : Chris Wallace
// Date         : 6-Feb-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Human::MakeClientUpdateMessage(void)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_HumanClientUpdate), GetClassification());
	if(pxMessage != NULL)
	{
		AVP_Message_HumanClientUpdate* pxData = static_cast<AVP_Message_HumanClientUpdate*>(pxMessage->GetData());
		pxData->FillHuman(*this);
		SendClientUpdateMessage(pxMessage);
	}
}

/// End of function 'AVP_ServerActor_Predator::MakeClientUpdateMessage'. ///

////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::AVP_ServerActor_Human
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_ServerActor_Human::AVP_ServerActor_Human(const Asura_Guid uGUID)
: Asura_Entity_Server(uGUID)
, PARENT(AVP_PolygonFlag_IgnoreForHumanCollision, uGUID)
, m_uType(0)
, m_bUseTestBehaviour(false)
, m_uGuardZoneGuid(ASURA_GUID_UNREGISTERED)
, m_uActionGamesceneGuid(ASURA_GUID_UNREGISTERED)
, m_bHasObjective( false )
, m_pxCurrentObjective( NULL )
, m_uFormationRole( FORMATION_ROLE_NONE )
, m_uNumTimesMeleedFriendly( 0 )
, m_fFriendlyMeleeTimer( 0.0f )
, m_bFriendlyMeleeTriggered( false )
, m_uLastMeleedFriendlyGuid( ASURA_GUID_UNREGISTERED )
, m_fLastFriendlyMeleeTimer( 0.0f )
, m_fLastFriendlyMeleeTrigger( false )
, m_uHackingToolGuid( ASURA_GUID_UNREGISTERED )
, m_uCurrentObjectiveType( AVP_ServerNode_Objective_Marine::MOVE_SLOW_STOP_TO_ENGAGE )
, m_uCurrentObjectiveThreatAction( AVP_ServerNode_Objective_Marine::STOP_TO_ENGAGE )
, m_uCurrentObjectiveMovementMode( AVP_Navigation::MM_SLOW )
, m_usHeadRippedMessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_uLastGuidToDamageUs( ASURA_GUID_UNREGISTERED )
, m_uGS2MoveSpeed( 0 )
, m_uHackingToolBPHash( ASURA_HASH_ID_UNSET )
, m_bHasHackingTool( false )
, m_fLOSUpdateTimer( 0.0f )
, m_uPickedUpBuddyGuid( ASURA_GUID_UNREGISTERED )
, m_uFlags(0)
, m_pxMoraleHandler( NULL )
, m_uFaceExpression_Passive01( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Passive02( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Passive03( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Passive04( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Active01( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Active02( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Active03( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Active04( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Aggressive01( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Aggressive02( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Aggressive03( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Aggressive04( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Scared01( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Scared02( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Scared03( ASURA_HASH_ID_UNSET )
, m_uFaceExpression_Scared04( ASURA_HASH_ID_UNSET )
{
	// Update the data...
	Classification	= AVP_ENTITYCLASS_HUMAN;

	m_pxMoraleHandler = new AVP_Human_MoraleHandler(GetGuid());

	m_pxMeleeSystem   = new AVP_Human_MeleeCounter_System(*this);

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}

void AVP_ServerActor_Human::PostLoadInitialise(void)
{
	PARENT::PostLoadInitialise();

	if ( !IsPlayer() )
	{
		m_bAiming = true;
	}

	// Create a hacking tool pickup - WGP: Only if we need one!
	if ( m_bHasHackingTool && m_uHackingToolBPHash != ASURA_HASH_ID_UNSET && m_uHackingToolGuid == ASURA_GUID_UNREGISTERED )
	{
		Asura_Vector_3   xAttachPointPosition;
		Asura_Matrix_3x3 xAttachPointOrientation;
		xAttachPointPosition.Zero();
		xAttachPointOrientation.Identity();

		AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::Create( Asura_Entity_System::GetCreatedByGameGuid(), xAttachPointPosition, xAttachPointOrientation, m_uHackingToolBPHash, false, true, true );
		if ( pxPickup )
		{
			m_uHackingToolGuid = pxPickup->GetGuid();
		}
	}

	if ( m_bPickedUp )
	{
		if ( m_pxPhysicsObjectHandler && m_pxPhysicsObjectHandler->GetPhysicsObject() )
		{
			m_pxPhysicsObjectHandler->GetPhysicsObject()->SetIgnore( true );
		}
	}
}

/// End of function 'AVP_ServerActor_Human::AVP_ServerActor_Human'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::~AVP_ServerActor_Human
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_ServerActor_Human::~AVP_ServerActor_Human()
{
	if ( m_pxMoraleHandler )
	{
		delete( m_pxMoraleHandler );
	}

	if ( m_pxCurrentObjective )
	{
		delete ( m_pxCurrentObjective );
	}

	// Don't forget to remove us from the list...
	s_xList.RemoveFromList(this);
}

/// End of function 'AVP_ServerActor_Human::~AVP_ServerActor_Human'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::HandleMessage
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerActor_Human::HandleMessage(const Asura_Message* pxMessage)
{
	// What sort of message is it?
	switch (pxMessage->GetMsgID())
	{
		case AVP_MSGID_ALIENQUEEN_ADD_TO_TARGET_LIST:
		{
			const AVP_Msg_Add_Entity_To_Queen_Target_List *pxData = static_cast<const AVP_Msg_Add_Entity_To_Queen_Target_List*>( pxMessage->GetData() );

			if( pxData->GetQueenGuid() != ASURA_GUID_UNREGISTERED )
			{
				Asura_Network::Forward( pxMessage, pxData->GetQueenGuid(), GetGuid()  );
			}

			return true;
		}
		case AVP_MSGID_ALIENQUEEN_REMOVE_FROM_TARGET_LIST:
		{
			const AVP_Msg_Remove_Entity_From_Queen_Target_List *pxData = static_cast<const AVP_Msg_Remove_Entity_From_Queen_Target_List*>( pxMessage->GetData() );

			if( pxData->GetQueenGuid() != ASURA_GUID_UNREGISTERED )
			{
				Asura_Network::Forward( pxMessage, pxData->GetQueenGuid(), GetGuid()  );
			}

			return true;
		}
		case AVP_MSGID_SET_GS2_MOVE_SPEED:
		{
			const AVP_Message_SetGS2MoveSpeed* pxData = static_cast< const AVP_Message_SetGS2MoveSpeed* >( pxMessage->GetData() );

			if ( pxData )
			{
				m_uGS2MoveSpeed = pxData->m_uMoveSpeed;
			}

			return true;

		}

		case AVP_MSGID_SET_PICKUPABLE:
		{
			m_bPickUpAble = true;
			m_bLegsDamaged = true;
			Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
		}
		return true;

		case AVP_MSGID_SET_NOT_PICKUPABLE:
		{
			m_bPickUpAble = false;
			Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
		}
		return true;

		case AVP_MSGID_SET_DROPPABLE:
		{
			m_bDroppable = true;
			Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
		}
		return true;

		case AVP_MSGID_SET_NOT_DROPPABLE:
		{
			m_bDroppable = false;
			Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
		}
		return true;

		case AVP_MSGID_FORCE_AIM:
		case AVP_MSGID_STOP_FORCE_AIM:
		case AVP_MSGID_MOUNT_LIFT:
		case AVP_MSGID_DISMOUNT_LIFT:
		case AVP_MSGID_PICK_TEQUILA_UP:
		case AVP_MSGID_PUT_TEQUILA_DOWN:
		case AVP_MSGID_CLEAR_PLAYERS_LOF:
		case AVP_MSGID_LOOK_AT_ENTITY:
		case AVP_MSGID_STOP_LOOKING_AT_ENTITY:
		case AVP_MSGID_MELEE_PLAYER_BACK:
		case AVP_MSGID_PERFORM_AI_OBJECTIVE_ACTION:
		case AVP_MSGID_ATTACK_SCRIPTED_TARGET:
		case AVP_MSGID_STOP_ATTACKING_SCRIPTED_TARGET:
		{
			Asura_Assert( GetBrain(), "Server actor handling a message meant for it's brain but it don't not have one." );

			if(GetBrain() && !IsADeadBody())   //AJK; you can melee dead bodies, but they don't have a brain.
			{
				GetBrain()->DoHandleMessage(pxMessage);
			}
			return true;

		}

		case AVP_MSGID_PROMOTE_TO_SQUAD_LEADER:
		{
			if ( GetInitialSquadGuid() != ASURA_GUID_UNREGISTERED )
			{
				AVP_ServerEntity_Squad_Marine* pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity( GetInitialSquadGuid() );

				if ( pxSquad )
				{
					pxSquad->SetSquadLeader( GetGuid() );
				}
			}

			return true;

		}

		case AVP_MSGID_ADD_AI_OBJECTIVE:
		{
			// objectives for individuals
			Asura_Assert( GetClassification() != AVP_ENTITYCLASS_CIVILIAN, " Civilian doesn't handle this message, it can't be sent to one so this is a bad bug if it has ended up here!" );

			if ( GetInitialSquadGuid() != ASURA_GUID_UNREGISTERED )
			{
				AVP_ServerEntity_Squad_Marine* pxSquad = AVP_ServerEntity_Squad_Marine::FindEntity( GetInitialSquadGuid() );

				if ( pxSquad->GetSquadLeaderGuid() == GetGuid() )
				{
					pxSquad->ChooseNewSquadLeader( GetGuid() );
				}
			}

			const AVP_Message_AddObjective* pxMsgData = static_cast< const AVP_Message_AddObjective* >( pxMessage->GetData() );

			if ( pxMsgData )
			{
				if ( pxMsgData->GetObjectiveGuid() == ASURA_GUID_UNREGISTERED )
				{
					Asura_Assertf( false, ( "Trying to add an Goal but you have not set the Goal in the message!" ) );
					return true;
				}

				if ( m_pxCurrentObjective )
				{
					delete m_pxCurrentObjective;
					m_pxCurrentObjective = NULL;
					m_bHasObjective = false;
				}

				// add the objective to the list
				m_pxCurrentObjective = new Objective;

				m_bHasObjective = true;

				m_pxCurrentObjective->uObjectiveGuid	= pxMsgData->GetObjectiveGuid();
				m_pxCurrentObjective->uObjectiveState	= AVP_ServerNode_Objective_Marine::NOT_STARTED;
			}

			return true;
		}

		case AVP_MSGID_CHANGE_HUMANOID_SQUAD_MOVEMENT_MODE:
		{
			const AVP_Message_ChangeHumanoidMovementMode* pxMsgData = static_cast< const AVP_Message_ChangeHumanoidMovementMode*> ( pxMessage->GetData( ) );

			if ( pxMsgData )
			{
				m_uCurrentObjectiveType = pxMsgData->m_iMovementMode;

				switch ( pxMsgData->m_iMovementMode )
				{
					case AVP_ServerNode_Objective_Marine::ADVANCE_SLOW:
					case AVP_ServerNode_Objective_Marine::RETREAT_SLOW:
					case AVP_ServerNode_Objective_Marine::MOVE_SLOW_DONT_STOP_TO_ENGAGE:
					{
						m_uCurrentObjectiveThreatAction = AVP_ServerNode_Objective_Marine::NO_ACTION;
						m_uCurrentObjectiveMovementMode = AVP_Navigation::MM_SLOW;

					}break;
					
					case AVP_ServerNode_Objective_Marine::MOVE_MEDIUM_STOP_TO_ENGAGE:
					{
						m_uCurrentObjectiveThreatAction = AVP_ServerNode_Objective_Marine::STOP_TO_ENGAGE;
						m_uCurrentObjectiveMovementMode = AVP_Navigation::MM_MEDIUM;

					}break;

					case AVP_ServerNode_Objective_Marine::ADVANCE_MEDIUM:
					case AVP_ServerNode_Objective_Marine::RETREAT_MEDIUM:
					case AVP_ServerNode_Objective_Marine::MOVE_MEDIUM_DONT_STOP_TO_ENGAGE:
					{
						m_uCurrentObjectiveThreatAction = AVP_ServerNode_Objective_Marine::NO_ACTION;
						m_uCurrentObjectiveMovementMode = AVP_Navigation::MM_MEDIUM;

					}break;
				
					case AVP_ServerNode_Objective_Marine::MOVE_SLOW_STOP_TO_ENGAGE:
					default:
					{
						m_uCurrentObjectiveThreatAction = AVP_ServerNode_Objective_Marine::STOP_TO_ENGAGE;
						m_uCurrentObjectiveMovementMode = AVP_Navigation::MM_SLOW;

					}break;
				}
			}

			return true;

		}

		case AVP_MSGID_HUMANPLAYERCOMMANDS_COMPRESSED:
		{
			// Get raw compressed data
			const u_char* pucData = static_cast<const u_char*>( pxMessage->GetData() ); 
			u_int uLen = pxMessage->GetDataSize(); 
			
			// Uncompress the data
			if (pucData && uLen)
			{
				Asura_BitUnpacker xUnpacker( pucData, uLen ); 
				AVP_Message_HumanPlayerCommands xHumanPlayerCommands( xUnpacker ); 

				ProcessHumanPlayerCommands( &xHumanPlayerCommands, pxMessage->GetSentTime() ); 
			}

			return true; 
		}

		case AVP_MSGID_HUMANPLAYERCOMMANDS:
		{
			const AVP_Message_HumanPlayerCommands*	pxData	= static_cast<const AVP_Message_HumanPlayerCommands*>(pxMessage->GetData());

			if (pxData != NULL)
			{
				ProcessHumanPlayerCommands( pxData, pxMessage->GetSentTime() );
			}

			return true;
		}

		case AVP_MSGID_REQUEST_INTERACTION:
		{
			const AVP_Message_Actor_RequestInteraction* pxData = static_cast<const AVP_Message_Actor_RequestInteraction*>(pxMessage->GetData());

			switch ( pxData->GetInteractionType() )
			{
				case AVP_INTERACTION_TYPE_PICKUP_BUDDY:
				{
					PickUpBuddy( pxData->GetActorGuid() );

					Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
				}
				return true;

				// The only interaction we can currently do anything with is a headbite, and even then only if we're dead and they're an alien (although, if they're not, why are they requesting a headbite?)
				// TODO - should the early-outs return true, or return parent handling? Probably parent handling, as some generic interactions may be created.
				case AVP_INTERACTION_TYPE_HEADBITE:
				{
					// AJK, added IsADeadBody check
					if (!IsDead() && !IsADeadBody()) { return PARENT::HandleMessage(pxMessage); } 

					const AVP_ServerActor_Alien* pxBiter = AVP_ServerActor_Alien::FindEntity(pxMessage->From);
					if (!pxBiter) { return PARENT::HandleMessage(pxMessage); }

					// OK, we're dead, they're an alien, and at the moment that's all we can check. Tell them to bite our head off.
					// TODO - need to add some way of knowing if the head is still present on the server (currently client-side only).
					Asura_Message* pxNewMessage = Asura_Network::CreateMessage(AVP_MSGID_REQUEST_MELEE, sizeof(AVP_Message_Vertebrate_RequestMelee));
					if (pxNewMessage)
					{
						AVP_Message_Vertebrate_RequestMelee* pxNewData = static_cast<AVP_Message_Vertebrate_RequestMelee*>(pxNewMessage->GetData());
						pxNewData->SetType(AVP_ALIEN_MELEEATTACK_TYPE_HEADBITE);
						pxNewData->SetTargetGuid(Guid);

						Send(pxNewMessage, pxMessage->From);
					}
				}
				return true;

				default:
				{
					return PARENT::HandleMessage( pxMessage );
				}
			}
		}

		case AVP_MSGID_LAUNCH_PROJECTILE:
		{
			const AVP_Message_ActorLaunchProjectile*	pxData	= static_cast<const AVP_Message_ActorLaunchProjectile*>(pxMessage->GetData());

			if(pxData != NULL)
			{
				const Asura_Guid				uProjectileGuid	= pxData->GetGuid();
				AVP_ServerEntity_Projectile*	pxProjectileServer	= NULL;
				float							fDamageMultiplier = 1.0f;

				switch(pxData->GetClassification())
				{
					case AVP_ENTITYCLASS_HUMANFLARE:
					{
						pxProjectileServer	= AVP_ServerProjectile_HumanFlare::FindEntity(uProjectileGuid);
						break;
					}

					default:
					{
						pxProjectileServer	= NULL;
						break;
					}
				}		// End of switch.

				if(pxProjectileServer != NULL)
				{
					const Asura_Vector_3&	xPosition		= pxData->GetPosition();
					const Asura_Vector_3&	xVelocity		= pxData->GetVelocity();
					Asura_Matrix_3x3	xOrientation;
					pxData->GetOrientation().ToMatrix( xOrientation );

					pxProjectileServer->Launch(xPosition, xOrientation, xVelocity, GetGuid(), fDamageMultiplier);
				}

			}

			return true;
		}

        case AVP_MSGID_REQUEST_TORCH_ON:
        case AVP_MSGID_REQUEST_TORCH_OFF:
        {
            if ( GetBrain() )
            {
                GetBrain()->DoHandleMessage( pxMessage );
            }
            return true;
        }

		case AVP_MSGID_DISTRACT_AI:
		{
			const AVP_Msg_DistractAI* pxData = static_cast<const AVP_Msg_DistractAI*>( pxMessage->GetData() );
			if( pxData )
			{
				// RECONTODO, if this is a recon message call funciton to effect the humaniod AI behaviour. 
				if ( GetBrain() )
				{
					if ( GetClassification() != AVP_ENTITYCLASS_CIVILIAN )
					{
						AVP_Brain_Actor_Marine* pxMarineBrain = static_cast<AVP_Brain_Actor_Marine*> ( GetActorBrain() );

						if ( pxMarineBrain )
						{
							pxMarineBrain->GetKnowledgeSystem()->AddReconModeKnowledge();
						}
					}
					else
					{
						GetActorBrain()->ForceLookAtPosition( *AVP_Predator_Recon::GetRecon().GetTargetPosition() );
						Asura_Vector_3 xDirection = *AVP_Predator_Recon::GetRecon().GetTargetPosition() - *AVP_Predator_Recon::GetRecon().GetPreyPosition();
						GetActorBrain()->ForceMovementDirection( xDirection );
					}
				}
			}
			return true;
		}

		case AVP_MSGID_DROP_PICKUP:
		{
			if(GetCurrentWeapon() && GetCurrentWeapon()->GetItemType() == AVP_INVENTORYITEMTYPE_TEQUILA )
			{
				const AVP_ServerActor_Human* pxBuddy = AVP_ServerActor_Human::FindEntity( m_uPickedUpBuddyGuid );
				Asura_Assert( pxBuddy, "Unable to find server entity for buddy to drop" );
				if( pxBuddy && pxBuddy->IsDroppable() )
				{
					DropBuddy();
				}

				Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
				return true;
			}
			else
			{
				return PARENT::HandleMessage(pxMessage);
			}
		}

		default:
		{
			return PARENT::HandleMessage(pxMessage);
		}
	}		// End of switch.
}

/// End of function 'AVP_ServerActor_Human::HandleMessage'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Human::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int	uSkin = ASURA_HASH_ID_UNSET; //stores the skin in old versions
	u_int	uVersion;

	// Read the version...
	xStream >> uVersion;

	if (uVersion > 5)
	{
		// Read the parent data...
		PARENT::ReadFromChunkStream(xStream);
	}
	else
	{
		// The vertebrate didn't have chunk data originally, so direct subclass chunk versions have been bumped to work around it.
		AVP_ServerEntity_Actor::ReadFromChunkStream(xStream);
	}

	if (uVersion <= uCURRENT_CHUNK_VERSION)
	{
		u_int	uWeapon;

		if ( uVersion >= 1 )
		{
			xStream >> m_uType;
		}

		if ( uVersion >= 3)
		{
			xStream >> uSkin;
			xStream >> uWeapon;
		}	

		if (uVersion >= 7)
		{
			xStream >> m_bUseTestBehaviour;
		}

		if (uVersion == 8)
		{
			// Page hash
			Asura_Hash_ID uTmpHash;
			xStream >> uTmpHash;
			m_xNameText.SetTextPageHash(uTmpHash);

			// Text hash
			xStream >> uTmpHash;
			m_xNameText.SetTextID(uTmpHash);	

			Asura_StringBuffer<64>  xTextFileName;     //Menu file name

			xStream >> xTextFileName; 
		}

		if (uVersion >= 9)
		{
			m_xNameText.SetTextPageHash(Asura_GetHashID("Menu")) ;

			Asura_Hash_ID uTmpHash;

			// Text hash
			xStream >> uTmpHash;
			m_xNameText.SetTextID(uTmpHash);	
		}

		if (uVersion >= 10)
		{
			xStream >> m_uHackingToolGuid;
			if ( uVersion <= 12 )
			{
				m_bHasHackingTool = ( m_uHackingToolGuid != ASURA_GUID_UNREGISTERED ) ? true : false;
			}
		}

		if (uVersion >= 11)
		{
			xStream >> m_bHasObjective;

			if ( m_bHasObjective )
			{
				if ( m_pxCurrentObjective )
				{
					delete m_pxCurrentObjective;
				}

				m_pxCurrentObjective = new Objective;
				xStream >> m_pxCurrentObjective->uObjectiveGuid;
				xStream >> m_pxCurrentObjective->uObjectiveState;
			}

			xStream >> m_uCurrentObjectiveType;
			xStream >> m_uCurrentObjectiveThreatAction;
			xStream >> m_uCurrentObjectiveMovementMode;
		}

		if (uVersion >= 12)
		{
			xStream >> m_usHeadRippedMessageBlock;
		}

		if ( uVersion >= 13 )
		{
			xStream >> m_bHasHackingTool;
		}

		if ( uVersion >= 14 )
		{
			xStream >> m_uGS2MoveSpeed;
		}

		if ( uVersion >= 15 )
		{
			Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uFlags );
			xStream >> xBitfield;
		}

		if ( uVersion >= 16 )
		{
			xStream >> m_uPickedUpBuddyGuid;
		}
		else
		{
			m_bPickUpAble	= false;
			m_bDroppable	= false;
			m_bPickedUp		= false;
		}
		if ( uVersion == 17 )
		{
			bool bHasMotionTracker;
			xStream >> bHasMotionTracker;
			m_bHasMotionTracker = bHasMotionTracker;
		}

		Asura_Assert(GetBlueprintHashID()!=ASURA_HASH_ID_UNSET, "No Blueprint found on load!");
	}
	else
	{
		Asura_Assert(false, "Unexpected chunk version found for AVP_ServerActor_Human.");
	}
}

/// End of function 'AVP_ServerActor_Human::ReadFromChunkStream'. ///

void AVP_ServerActor_Human::WriteToClientCreateStream(Asura_Chunk_Memory_Stream& xStream) const
{
	xStream << static_cast<u_int>(CURRENT_CLIENTCREATE_VERSION);

	PARENT::WriteToClientCreateStream(xStream);

	u_int uAlertLevel = 0;

	if (GetClassification() != AVP_ENTITYCLASS_CIVILIAN)
	{
		// Initialise in case we dont get a brain (true for players in multiplayer)
		uAlertLevel = AVP_Knowledge_System_Actor::ACTIVE_ALERTNESS;

		AVP_Brain_Actor_Marine* pxMarineBrain = (AVP_Brain_Actor_Marine*)GetActorBrain();

		if ((pxMarineBrain) && (pxMarineBrain->GetKnowledgeSystem()))
		{
			uAlertLevel = pxMarineBrain->GetKnowledgeSystem()->GetAlertLevel();
		}
	}
	else
	{
		// Initialise in case we dont get a brain (true for players in multiplayer)
		uAlertLevel = AVP_Knowledge_System_Actor::PASSIVE_ALERTNESS;

		AVP_Brain_Actor_Civilian* pxCivilianBrain = (AVP_Brain_Actor_Civilian*)GetActorBrain();

		if ((pxCivilianBrain) && (pxCivilianBrain->GetKnowledgeSystem()))
		{
			uAlertLevel = pxCivilianBrain->GetKnowledgeSystem()->GetAlertLevel();
		}
	}

	xStream << uAlertLevel;
	xStream << GetNameText().GetTextPageHash();
	xStream << GetNameText().GetTextID();

	Asura_WriteToStream_Bitfield_Wrapper<u_int>	xBitfield(m_uFlags);
	xBitfield.Add( m_bCrouching, 1 );
	xBitfield.Add( m_bAiming, 1 );
	xBitfield.Add( m_eCoverType, 2 );
	xBitfield.Add( m_bLegsDamaged, 1 );
	xBitfield.Add( m_bStartAsInjured, 1 );
	xBitfield.Add( m_bPickUpAble, 1 );
	xBitfield.Add( m_bDroppable, 1 );
	xBitfield.Add( m_bPickedUp, 1 );
	xBitfield.Add( m_bHasMotionTracker, 1 );

	xStream << xBitfield;

	xStream << m_uPickedUpBuddyGuid;

}	

void AVP_ServerActor_Human::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	const u_int	uWeapon	= 0;

	// Write the version...
	xStream << uCURRENT_CHUNK_VERSION;

	// Write the parent data...
	PARENT::WriteToChunkStream(xStream);

	xStream << m_uType;
	u_int	uSkin = ASURA_HASH_ID_UNSET;
	xStream << uSkin;
	xStream << uWeapon;

	xStream << m_bUseTestBehaviour;

	xStream << m_xNameText.GetTextID();

	xStream << m_uHackingToolGuid;

	// objectives
	xStream << m_bHasObjective;

	if ( m_bHasObjective )
	{
		xStream << m_pxCurrentObjective->uObjectiveGuid;
		xStream << m_pxCurrentObjective->uObjectiveState;
	}

	xStream << m_uCurrentObjectiveType;
	xStream << m_uCurrentObjectiveThreatAction;
	xStream << m_uCurrentObjectiveMovementMode;

	xStream << m_usHeadRippedMessageBlock;
	
	xStream << m_bHasHackingTool;

	xStream << m_uGS2MoveSpeed;

	// Any data specific to this class goes here...
	Asura_WriteToStream_Bitfield_Wrapper<u_int>	xBitfield(m_uFlags);
	xBitfield.Add( m_bCrouching, 1 );
	xBitfield.Add( m_bAiming, 1 );
	xBitfield.Add( m_eCoverType, 2 );
	xBitfield.Add( m_bLegsDamaged, 1 );
	xBitfield.Add( m_bStartAsInjured, 1 );
	xBitfield.Add( m_bPickUpAble, 1 );
	xBitfield.Add( m_bDroppable, 1 );
	xBitfield.Add( m_bPickedUp, 1 );
	xBitfield.Add( m_bHasMotionTracker, 1 );

	xStream << xBitfield;

	xStream << m_uPickedUpBuddyGuid;
}

void AVP_ServerActor_Human::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
    PARENT::InitialiseFromBlueprint( xBlueprint );

    Asura_Value xValue;

    if (xBlueprint.GetParameterValueByHashID(0x00368F3A, xValue))	//Asura_GetHashID("Type"), xValue))
    {
        m_uType = static_cast<u_int>(xValue.GetIntValue());
    }

	GetFacialAnimationValuesFromBlueprint( xBlueprint );

	if ((GetClassification() == AVP_ENTITYCLASS_MARINE) && (!IsPlayer()))
	{
		// Do the morale handler stuff
		if (m_pxMoraleHandler)
		{
			m_pxMoraleHandler->InitaliseFromBlueprint();
		}
	}
	else
	{
		if (m_pxMoraleHandler)
		{
			delete m_pxMoraleHandler;
			m_pxMoraleHandler = NULL;
		}
	}

	InitialiseDialogueFromBlueprint(GetBlueprintHashID());
}

void AVP_ServerActor_Human::GetFacialAnimationValuesFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	Asura_Value xValue;

	if ( xBlueprint.GetParameterValueByHashID( 0xd44bd101, xValue ) ) //"FaceExpression.Passive.01"
	{
		m_uFaceExpression_Passive01 = xValue.GetHashIDValue();		
	}

	if ( xBlueprint.GetParameterValueByHashID( 0xd44bd102, xValue ) ) //"FaceExpression.Passive.02"
	{
		m_uFaceExpression_Passive02 = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0xd44bd103, xValue ) ) //"FaceExpression.Passive.03"
	{
		m_uFaceExpression_Passive03 = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0xd44bd104, xValue ) ) //"FaceExpression.Passive.04"
	{
		m_uFaceExpression_Passive04 = xValue.GetHashIDValue();
	}

	//---------------------------------------------------------------------------------------------
	if ( xBlueprint.GetParameterValueByHashID( 0x21d31bf0, xValue ) ) //"FaceExpression.Active.01"
	{
		m_uFaceExpression_Active01 = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0x21d31bf1, xValue ) ) //"FaceExpression.Active.02"
	{
		m_uFaceExpression_Active02 = xValue.GetHashIDValue();
	}
	
	if ( xBlueprint.GetParameterValueByHashID( 0x21d31bf2, xValue ) ) //"FaceExpression.Active.03"
	{
		m_uFaceExpression_Active03 = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0x21d31bf3, xValue ) ) //"FaceExpression.Active.04"
	{
		m_uFaceExpression_Active04 = xValue.GetHashIDValue();
	}

	//---------------------------------------------------------------------------------------------
	if ( xBlueprint.GetParameterValueByHashID( 0x2cd45ab2, xValue ) ) //"FaceExpression.Aggressive.01"
	{
		m_uFaceExpression_Aggressive01 = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0x2cd45ab3, xValue ) ) //"FaceExpression.Aggressive.02"
	{
		m_uFaceExpression_Aggressive02 = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0x2cd45ab4, xValue ) ) //"FaceExpression.Aggressive.03"
	{
		m_uFaceExpression_Aggressive03 = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0x2cd45ab5, xValue ) ) //"FaceExpression.Aggressive.04"
	{
		m_uFaceExpression_Aggressive04 = xValue.GetHashIDValue();
	}

	//---------------------------------------------------------------------------------------------
	if ( xBlueprint.GetParameterValueByHashID( 0xaff81dd6, xValue ) ) //"FaceExpression.Scared.01"
	{
		m_uFaceExpression_Scared01 = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0xaff81dd7, xValue ) ) //"FaceExpression.Scared.02"
	{
		m_uFaceExpression_Scared02 = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0xaff81dd8, xValue ) ) //"FaceExpression.Scared.03"
	{
		m_uFaceExpression_Scared03 = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0xaff81dd9, xValue ) ) //"FaceExpression.Scared.04"
	{
		m_uFaceExpression_Scared04 = xValue.GetHashIDValue();
	}
}

Asura_Hash_ID AVP_ServerActor_Human::GetFacialAnimation( ExpressionType eType, u_int iIndex)
{
	switch ( eType )
	{
		case FACE_EXPRESSION_PASSIVE:
		{
			switch ( iIndex )
			{
				case 0:		return m_uFaceExpression_Passive01;
				case 1:		return m_uFaceExpression_Passive02;
				case 2:		return m_uFaceExpression_Passive03;
				case 3:		return m_uFaceExpression_Passive04;
				default:	return ASURA_HASH_ID_UNSET;

			}
		}

		case FACE_EXPRESSION_ACTIVE:
		{
			switch ( iIndex )
			{
			case 0:		return m_uFaceExpression_Active01;
			case 1:		return m_uFaceExpression_Active02;
			case 2:		return m_uFaceExpression_Active03;
			case 3:		return m_uFaceExpression_Active04;
			default:	return ASURA_HASH_ID_UNSET;
			}
		}

		case FACE_EXPRESSION_AGGRESSIVE:
		{
			switch ( iIndex )
			{
			case 0:		return m_uFaceExpression_Aggressive01;
			case 1:		return m_uFaceExpression_Aggressive02;
			case 2:		return m_uFaceExpression_Aggressive03;
			case 3:		return m_uFaceExpression_Aggressive04;
			default:	return ASURA_HASH_ID_UNSET;

			}
		}

		case FACE_EXPRESSION_SCARED:
		{
			switch ( iIndex )
			{
			case 0:		return m_uFaceExpression_Scared01;
			case 1:		return m_uFaceExpression_Scared02;
			case 2:		return m_uFaceExpression_Scared03;
			case 3:		return m_uFaceExpression_Scared04;
			default:	return ASURA_HASH_ID_UNSET;
			}
		}

		default:
		{
			return ASURA_HASH_ID_UNSET;
		}
	}
}

u_int AVP_ServerActor_Human::GetNumFaceAnimsForExpression( ExpressionType uFaceType )
{
	u_int uNumAnims = 0;

	switch ( uFaceType )
	{
		case FACE_EXPRESSION_PASSIVE:
		{
			if ( m_uFaceExpression_Passive01 != ASURA_HASH_ID_UNSET)
			{
				++uNumAnims;

				if ( m_uFaceExpression_Passive02 != ASURA_HASH_ID_UNSET)
				{
					++uNumAnims;

					if ( m_uFaceExpression_Passive03 != ASURA_HASH_ID_UNSET)
					{
						++uNumAnims;

						if ( m_uFaceExpression_Passive04 != ASURA_HASH_ID_UNSET)
						{
							++uNumAnims;
						}
					}
				}
			}
		}break;

		case FACE_EXPRESSION_ACTIVE:
		{
			if ( m_uFaceExpression_Active01 != ASURA_HASH_ID_UNSET)
			{
				++uNumAnims;

				if ( m_uFaceExpression_Active02 != ASURA_HASH_ID_UNSET)
				{
					++uNumAnims;

					if ( m_uFaceExpression_Active03 != ASURA_HASH_ID_UNSET)
					{
						++uNumAnims;

						if ( m_uFaceExpression_Active04 != ASURA_HASH_ID_UNSET)
						{
							++uNumAnims;
						}
					}
				}
			}

		}break;

		case FACE_EXPRESSION_AGGRESSIVE:
		{
			if ( m_uFaceExpression_Aggressive01 != ASURA_HASH_ID_UNSET)
			{
				++uNumAnims;

				if ( m_uFaceExpression_Aggressive02 != ASURA_HASH_ID_UNSET)
				{
					++uNumAnims;

					if ( m_uFaceExpression_Aggressive03 != ASURA_HASH_ID_UNSET)
					{
						++uNumAnims;

						if ( m_uFaceExpression_Aggressive04 != ASURA_HASH_ID_UNSET)
						{
							++uNumAnims;
						}
					}
				}
			}

		}break;

		case FACE_EXPRESSION_SCARED:
		{
			if ( m_uFaceExpression_Scared01 != ASURA_HASH_ID_UNSET)
			{
				++uNumAnims;

				if ( m_uFaceExpression_Scared02 != ASURA_HASH_ID_UNSET)
				{
					++uNumAnims;

					if ( m_uFaceExpression_Scared03 != ASURA_HASH_ID_UNSET)
					{
						++uNumAnims;

						if ( m_uFaceExpression_Scared04 != ASURA_HASH_ID_UNSET)
						{
							++uNumAnims;
						}
					}
				}
			}


		}break;

		default:
		{
			return 0;
		}
	}

	return uNumAnims;
}

bool AVP_ServerActor_Human::InitialiseInventory(void)
{
	if(PARENT::InitialiseInventory())
	{
		AVP_Inventory_Group* pxGroup = m_pxInventorySystem->GetPermanentlyActiveItemGroup();
		AddPermenantlyActiveItems(pxGroup);

		// WGP - 24/02/2009 : because were not a player yet when we get here after a checkpoint load, this code wont run thru, but that's OK cos m_bHasHackingTool will be in the chunk data and we don't need the pickup BP hash as the Pickup will have been chunked as well.
		if ( IsPlayer() )
		{
			AVP_Settings::Species eSpecies = GetSpeciesByClassification();
			AVP_Game_Core::CharacterType eCharType;
			if ( AVP_Settings::IsMultiPlayer() )
			{
				switch( eSpecies )
				{
				case AVP_Settings::AVP_SPECIES_ALIEN:		eCharType = AVP_Game_Core::MULTIPLAYER_ALIEN; break;
				default:	Asura_Assert( false, "Unable to determine species" ); // fall through to human
				case AVP_Settings::AVP_SPECIES_HUMAN:		eCharType = AVP_Game_Core::MULTIPLAYER_MARINE; break;
				case AVP_Settings::AVP_SPECIES_PREDATOR:	eCharType = AVP_Game_Core::MULTIPLAYER_PREDATOR; break;
				}
			}
			else
			{
				switch( eSpecies )
				{
				case AVP_Settings::AVP_SPECIES_ALIEN:		eCharType = AVP_Game_Core::SINGLE_PLAYER_ALIEN; break;
				default:	Asura_Assert( false, "Unable to determine species" ); // fall through to human
				case AVP_Settings::AVP_SPECIES_HUMAN:		eCharType = AVP_Game_Core::SINGLE_PLAYER_MARINE; break;
				case AVP_Settings::AVP_SPECIES_PREDATOR:	eCharType = AVP_Game_Core::SINGLE_PLAYER_PREDATOR; break;
				}
			}

			const char* szLevelName = 
#ifdef ASURA_TOOLS
				Tools_SaveAsura::IsEditingMission() ? Tools_SaveAsura::GetMissionFilenameWithoutExtension() : Tools_SaveAsura::GetMainFilename();
#else
				AVP_Settings::GetLevelName();
#endif

			const Asura_Blueprint* pxLevelBlueprint = AVP_Game_Core::GetLevelBlueprint( szLevelName, eCharType );
			if( pxLevelBlueprint )
			{
				m_uHackingToolBPHash = pxLevelBlueprint->GetParameterAsHash( uAVP_BLUEPRINT_PARAM_HACKINGTOOL, ASURA_HASH_ID_UNSET );
				m_bHasHackingTool = ( m_uHackingToolBPHash != ASURA_HASH_ID_UNSET );
			}
		}

		return(true);
	}
	else
	{
		return(false);
	}
}

void AVP_ServerActor_Human::AddPermenantlyActiveItems(AVP_Inventory_Group* pxGroup)
{
	const Asura_Blueprint* pxBlueprint = GetBlueprint();
	if(pxBlueprint != NULL)
	{
		Asura_Value	xValue;
		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_TORCH, xValue))
		{
			Asura_Hash_ID uTorchBPHash = xValue.GetHashIDValue();
			AVP_InventoryItem_Torch* pxTorch = new AVP_InventoryItem_Torch(*pxGroup, uTorchBPHash, "Torch");
			if ( pxTorch )
			{
				pxTorch->InitialiseFromBlueprint();
			}
		}
		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_FLARE, xValue))
		{
			Asura_Hash_ID uFlareBPHash = xValue.GetHashIDValue();
			AVP_InventoryItem_HumanFlare* pxFlare = new AVP_InventoryItem_HumanFlare(*pxGroup, uFlareBPHash, "Flare");
			if ( pxFlare )
			{
				pxFlare->InitialiseFromBlueprint();
			}
		}
		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_STIM, xValue))
		{
			Asura_Hash_ID uStimBPHash = xValue.GetHashIDValue();
			AVP_InventoryItem_Stim* pxStim = new AVP_InventoryItem_Stim(*pxGroup, uStimBPHash, "Stim");
			if ( pxStim )
			{
				pxStim->InitialiseFromBlueprint();
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::MakePlayer
// Author       : Andy Tate
// Date         : 14-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Human::MakePlayer(Asura_Guid uPlayerGuid)
{
	if (uPlayerGuid == GetPlayerInfoGuid()) { return; }

	if (m_pxMoraleHandler)
	{
		delete(m_pxMoraleHandler);
		m_pxMoraleHandler = NULL;
	}

	PARENT::MakePlayer(uPlayerGuid);
}

void AVP_ServerActor_Human::MakeAI( void )
{
	PARENT::MakeAI();
}


Axon_Brain* AVP_ServerActor_Human::CreateBrain()
{
    return new AVP_Brain_Actor_Marine( this );
}

void AVP_ServerActor_Human::ProcessHumanPlayerCommands( const AVP_Message_HumanPlayerCommands* pxData, float fMessageSentTime )
{
	ProcessVertebratePlayerCommands( pxData, fMessageSentTime );

	SetCrouching( pxData->IsCrouching() );
	SetAiming( pxData->IsAiming() );
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::GetTorchItem
// Author       : Andy Tate
// Date         : 25-Mar-2008
////////////////////////////////////////////////////////////////////////////////

AVP_InventoryItem_Torch* AVP_ServerActor_Human::GetTorchItem(void) const
{
	const Asura_Blueprint*	pxBlueprint	= GetBlueprint();

	if(pxBlueprint == NULL)
	{
		Asura_Assert(false, "Failed to find blueprint for this human.");
		return(NULL);
	}
	else
	{
		if(m_pxInventorySystem == NULL)
		{
			Asura_Assert(false, "Failed to find inventory-system for this human - something's gone badly wrong.");
			return(NULL);
		}
		else
		{
			Asura_Value	xValue;

			if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_TORCH, xValue))
			{
				// This uses the HashID uAVP_INVENTORY_ITEMHASH_TORCH. not the Hash Contained in the paramter Dumb comenting in the code was the cause of this.
				AVP_Inventory_Item*	pxItem				= m_pxInventorySystem->GetItem( uAVP_INVENTORY_ITEMHASH_TORCH );

				return(static_cast<AVP_InventoryItem_Torch*>(pxItem));
			}
			else
			{
				return(NULL);
			}
		}
	}
}

/// End of function 'AVP_ServerActor_Human::GetTorchItem'. ///


const Asura_Blueprint* AVP_ServerActor_Human::GetCurrentFlinchBlueprint() const
{
	// Humans have flinches per weapon
	const AVP_InventoryItem_Weapon* pxCurrentWeapon = m_pxInventorySystem ? static_cast<const AVP_InventoryItem_Weapon*>(m_pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON)) : NULL;
	if (!pxCurrentWeapon) { return NULL; }

	return pxCurrentWeapon->GetBlueprint();
}

Asura_Hash_ID AVP_ServerActor_Human::GetMovesBlueprintType() const
{
	return uAVP_BLUEPRINT_TYPE_MOVESHUMAN;
}

void AVP_ServerActor_Human::SetAiming( bool bSet )
{
	if ( static_cast< bool >( m_bAiming ) != bSet )
	{
		m_bAiming = bSet;
		SetAIStateAnimsNeedToTween( dbg_fAimingStateChangeTweenTime );
		if ( m_pxAIAnimStateMachine )
		{
			m_pxAIAnimStateMachine->SetNeedToTween( dbg_fAimingStateChangeTweenTime );
		}
	}
}

void AVP_ServerActor_Human::SetCrouching( bool bSet )
{
	if ( static_cast< bool >( m_bCrouching ) != bSet )
	{
		m_bCrouching = bSet;
		SetAIStateAnimsNeedToTween( dbg_fCrouchingStateChangeTweenTime );
		SetPlayNextAIStateAnimFromStart();
		if ( m_pxAIAnimStateMachine )
		{
			m_pxAIAnimStateMachine->SetNeedToTween( dbg_fCrouchingStateChangeTweenTime );
			m_pxAIAnimStateMachine->SetPlayNextAnimFromStart();
		}
	}
}

//----------------------------------------------------------------------
// AVP_ServerActor_Human :: RequestAction
//----------------------------------------------------------------------
void AVP_ServerActor_Human::RequestAction( Asura_Guid uIgnoreActionGuid /*= ASURA_GUID_UNREGISTERED*/ )
{
	if ( ( !m_pxCurrentObjective ) || ( m_pxCurrentObjective->GetObjectiveGuid() == ASURA_GUID_UNREGISTERED ) )
	{
		return;
	}

	u_int uNumRequired = 0, uNumOptional = 0;

	// Need to know how many Optional and Required Guids we have, this can change per request also as some gamescenes will be made inactive.
	GetNumOptionalAndRequiredActionGuids( &uNumRequired, &uNumOptional );

	Asura_Guid*	pxRequiredActions = new Asura_Guid[ uNumRequired ];
	Asura_Guid*	pxOptionalActions = new Asura_Guid[ uNumOptional ];

	for ( u_int u = 0; u < uNumRequired; ++u )
	{
		pxRequiredActions[ u ] = ASURA_GUID_UNREGISTERED;
	}
	
	for ( u_int u = 0; u < uNumOptional; ++u )
	{
		pxOptionalActions[ u ] = ASURA_GUID_UNREGISTERED;
	}

	// Fill the arrays with the guids
	GetOptionalAndRequiredGuids( pxRequiredActions, pxOptionalActions );

	// Get a required action Guid
	Asura_Guid uGuidToAssign = GetActionGuidToAssign( pxRequiredActions, uNumRequired, uIgnoreActionGuid, true );

	// Assign a required guid if we can
	if ( uGuidToAssign != ASURA_GUID_UNREGISTERED )
	{
		AssignAction( uGuidToAssign, true, uNumRequired);
	}
	else // If we found no required action to assign then try an optional
	{
		uGuidToAssign = ASURA_GUID_UNREGISTERED;

		// Only try to assign an optional if we have more than zero
		if (uNumOptional > 0)
		{
			Asura_Guid* pxRandomOptionals = new Asura_Guid[uNumOptional];

			// Randomise our optional guids
			bool bFailed = BuildRandomisedOptionalArray( uNumOptional, pxOptionalActions, pxRandomOptionals );

			if (bFailed)
			{
				delete [] pxRandomOptionals;
				delete [] pxRequiredActions;
				delete [] pxOptionalActions;
				return;
			}

			uGuidToAssign = GetActionGuidToAssign( pxRandomOptionals, uNumOptional, uIgnoreActionGuid, false );

			if ( uGuidToAssign != ASURA_GUID_UNREGISTERED )
			{
				AssignAction( uGuidToAssign, false, uNumOptional );
			}

			delete [] pxRandomOptionals;
		}
	}

	delete [] pxRequiredActions;
	delete [] pxOptionalActions;

	if (uGuidToAssign == ASURA_GUID_UNREGISTERED)
	{
		return;
	}
	else
	{
		return;
	}
}

//----------------------------------------------------------------------
// AVP_ServerActor_Human :: GetNumOptionalAndRequiredActionGuids
//----------------------------------------------------------------------
void AVP_ServerActor_Human::GetNumOptionalAndRequiredActionGuids( u_int* uNumRequired, u_int* uNumOptional, bool bConsiderActiveState /*= true*/)
{
	if ( ( !m_pxCurrentObjective ) || ( m_pxCurrentObjective->GetObjectiveGuid() == ASURA_GUID_UNREGISTERED ) )
	{
		return;
	}

	AVP_ServerNode_Objective_Marine* pxObjective = AVP_ServerNode_Objective_Marine::FindEntity( m_pxCurrentObjective->GetObjectiveGuid() );

	if ( !pxObjective )
	{
		return;
	}

	// What method are we doing to find out actions?
	if ( pxObjective->GetShouldUseSpecificGPV() == 1 )
	{
		// We are using a very specific Game Scene
		AVP_ServerEntity_GamePlayVolume* pxVolume = AVP_ServerEntity_GamePlayVolume::FindEntity(pxObjective->GetGamePlayVolumeGuid());

		if (pxVolume)
		{
			for (Asura_Hashed_List_It<AVP_ServerNode_GS2Node> it(AVP_ServerNode_GS2Node::GetList()); !it.Done(); it.Next())
			{
				AVP_ServerNode_GS2Node* pxGS2Node = it.GetCurrent();


				if ( ( pxGS2Node && pxGS2Node->IsEntityActive() && bConsiderActiveState ) || ( pxGS2Node && !bConsiderActiveState ) )
				{
					if (pxVolume->Contains(pxGS2Node->GetPosition()))
					{
						if (pxGS2Node->GetStructureType() == AVP_GS_ST_REQUIRED_MARINE_ACTION)
						{
							++(*uNumRequired);
						}
						else if (pxGS2Node->GetStructureType() == AVP_GS_ST_OPTIONAL_MARINE_ACTION)
						{
							++(*uNumOptional);
						}
					}
				}
			}
		}
	}
	else if ( pxObjective->GetShouldUseSpecificGameScenes() == 1 )
	{
		for ( u_int u = 0; u < pxObjective->GetNumRequiredActions(); ++u )
		{
			AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity( pxObjective->GetRequiredActionGuid( u ) );

			if ( ( pxGS2Node && pxGS2Node->IsEntityActive() && bConsiderActiveState ) || ( pxGS2Node && !bConsiderActiveState ) )
			{
				++(*uNumRequired);
			}
		}

		for ( u_int u = 0; u < pxObjective->GetNumOptionalActions(); ++u )
		{
			AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity( pxObjective->GetOptionalActionGuid( u ) );

			if ( ( pxGS2Node && pxGS2Node->IsEntityActive() && bConsiderActiveState ) || ( pxGS2Node && !bConsiderActiveState ) )
			{
				++(*uNumOptional);
			}
		}
	}
	else
	{
		// We are just using the GPV that the objective is inside
		AVP_ServerEntity_GamePlayVolume* pxVolume = AVP_ServerEntity_GamePlayVolume::GetVolumeThatContainsPosition(pxObjective->GetPosition());

		if (pxVolume)
		{
			for (Asura_Hashed_List_It<AVP_ServerNode_GS2Node> it(AVP_ServerNode_GS2Node::GetList()); !it.Done(); it.Next())
			{
				AVP_ServerNode_GS2Node* pxGS2Node = it.GetCurrent();

				if ( ( pxGS2Node && pxGS2Node->IsEntityActive() && bConsiderActiveState ) || ( pxGS2Node && !bConsiderActiveState ) )
				{
					if (pxVolume->Contains(pxGS2Node->GetPosition()))
					{
						if (pxGS2Node->GetStructureType() == AVP_GS_ST_REQUIRED_MARINE_ACTION)
						{
							++(*uNumRequired);
						}
						else if (pxGS2Node->GetStructureType() == AVP_GS_ST_OPTIONAL_MARINE_ACTION)
						{
							++(*uNumOptional);
						}
					}
				}
			}
		}
	}
}

//----------------------------------------------------------------------
// AVP_ServerActor_Human :: GetOptionalAndRequiredGuids
//----------------------------------------------------------------------
void AVP_ServerActor_Human::GetOptionalAndRequiredGuids( Asura_Guid* pxRequired, Asura_Guid* pxOptional )
{
	if ( ( !m_pxCurrentObjective ) || ( m_pxCurrentObjective->GetObjectiveGuid() == ASURA_GUID_UNREGISTERED ) )
	{
		return;
	}

	AVP_ServerNode_Objective_Marine* pxObjective = AVP_ServerNode_Objective_Marine::FindEntity( m_pxCurrentObjective->GetObjectiveGuid() );

	if ( !pxObjective )
	{
		return;
	}

	// What method are we using to find out actions?
	if ( pxObjective->GetShouldUseSpecificGameScenes() == 0 )
	{
		// We are using a very specific Game Scene
		AVP_ServerEntity_GamePlayVolume* pxVolume = NULL;

		if (pxObjective->GetShouldUseSpecificGPV() == 1)
		{
			pxVolume = AVP_ServerEntity_GamePlayVolume::FindEntity(pxObjective->GetGamePlayVolumeGuid());
		}
		else
		{
			pxVolume = AVP_ServerEntity_GamePlayVolume::GetVolumeThatContainsPosition(pxObjective->GetPosition());
		}

		if ( pxVolume )
		{
			u_int uNumReqProcessed = 0;
			u_int uNumOptProcessed = 0;

			for (Asura_Hashed_List_It<AVP_ServerNode_GS2Node> it(AVP_ServerNode_GS2Node::GetList()); !it.Done(); it.Next())
			{
				AVP_ServerNode_GS2Node* pxGS2Node = it.GetCurrent();

				if (pxGS2Node && pxGS2Node->IsEntityActive())
				{
					if (pxVolume->Contains(pxGS2Node->GetPosition()))
					{
						if (pxGS2Node->GetStructureType() == AVP_GS_ST_REQUIRED_MARINE_ACTION)
						{
							pxRequired[uNumReqProcessed] = pxGS2Node->GetGuid();
							++uNumReqProcessed;
						}
						else if (pxGS2Node->GetStructureType() == AVP_GS_ST_OPTIONAL_MARINE_ACTION)
						{
							pxOptional[uNumOptProcessed] = pxGS2Node->GetGuid();
							++uNumOptProcessed;
						}
					}
				}
			}
		}
	}
	else if ( pxObjective->GetShouldUseSpecificGameScenes() == 1 )
	{
		u_int uNumReqProcessed = 0;
		u_int uNumOptProcessed = 0;

		for (u_int u = 0; u < pxObjective->GetNumRequiredActions(); ++u)
		{
			AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity(pxObjective->GetRequiredActionGuid( u ) );

			if (pxGS2Node && pxGS2Node->IsEntityActive())
			{
				pxRequired[uNumReqProcessed] = pxObjective->GetRequiredActionGuid(u);
				++uNumReqProcessed;
			}
		}

		for (u_int u = 0; u < pxObjective->GetNumOptionalActions(); ++u)
		{
			AVP_ServerNode_GS2Node* pxGS2Node = AVP_ServerNode_GS2Node::FindEntity(pxObjective->GetOptionalActionGuid( u ) );

			if (pxGS2Node && pxGS2Node->IsEntityActive())
			{
				pxOptional[uNumOptProcessed] = pxObjective->GetOptionalActionGuid(u);
				++uNumOptProcessed;
			}
		}
	}
}

//----------------------------------------------------------------------
// AVP_ServerActor_Human :: GetActionGuidToAssign
//----------------------------------------------------------------------
Asura_Guid AVP_ServerActor_Human::GetActionGuidToAssign(Asura_Guid* pxActions, u_int uNumActions, Asura_Guid uIgnoreActionGuid, bool bIsRequiredAction )
{
	Asura_Guid uGuidToAssign = ASURA_GUID_UNREGISTERED;

	for ( u_int u = 0; u < uNumActions; ++u )
	{
		Asura_Guid uGuidToCheck = pxActions[ u ];
		bool bInUse = false;

		if ( uGuidToCheck == ASURA_GUID_UNREGISTERED )
		{
			continue;
		}

		if ( uIgnoreActionGuid == uGuidToCheck )
		{
			bInUse = true;
		}

		if ( !bInUse )
		{
			uGuidToAssign = uGuidToCheck;
			break;
		}
	}

	return uGuidToAssign;
}

//----------------------------------------------------------------------
// AVP_ServerActor_Human :: BuildRandomisedOptionalArray
//----------------------------------------------------------------------
bool AVP_ServerActor_Human::BuildRandomisedOptionalArray( u_int uNumOptional, Asura_Guid* pxOptionalActions, Asura_Guid* pxRandomOptionals )
{
	for ( u_int u = 0; u < uNumOptional; ++u )
	{
		pxRandomOptionals[ u ] = ASURA_GUID_UNREGISTERED;
	}

	u_int uRandomOptionalsProcessed = 0;

	// first things first build a random array of objective optional actions
	bool bFailed = false;
	bool bDone = false;

	if ( uNumOptional > 1 )
	{
		u_int uCount = 0;

		while ( !bDone && !bFailed )
		{
			u_int uActionGuid = pxOptionalActions[ ( Asura_Random::GetIntU( 0, uNumOptional - 1) ) ];

			bool bExists = false;

			for (u_int u = 0; u < uNumOptional; ++u)
			{
				if (pxRandomOptionals[u] == uActionGuid)
				{
					bExists = true;
					break;
				}
			}

			if (!bExists)
			{
				pxRandomOptionals[uRandomOptionalsProcessed] = uActionGuid;
				++uRandomOptionalsProcessed;

				if (uRandomOptionalsProcessed >= uNumOptional)
				{
					bDone = true;
				}
			}
			else
			{
				++uCount;

				if (uCount > (uNumOptional * 4))
				{
					return true;
				}
			}
		}

		return false;
	}
	else
	{
		pxRandomOptionals[0] = pxOptionalActions[0];
		return false;
	}
}

//----------------------------------------------------------------------
// AVP_ServerActor_Human :: AssignAction
//----------------------------------------------------------------------
void AVP_ServerActor_Human::AssignAction(Asura_Guid uGuidToAssign, bool bIsRequiredAction, u_int uNumActions )
{
	if ( ( !m_pxCurrentObjective ) || ( m_pxCurrentObjective->GetObjectiveGuid() == ASURA_GUID_UNREGISTERED ) )
	{
		return;
	}

	AVP_ServerNode_Objective_Marine* pxObjective = AVP_ServerNode_Objective_Marine::FindEntity( m_pxCurrentObjective->GetObjectiveGuid() );

	if ( !pxObjective )
	{
		return;
	}

	SendPerformActionMessage( uGuidToAssign );
}

//----------------------------------------------------------------------
// AVP_ServerActor_Human :: SendPerformActionMessage
//----------------------------------------------------------------------
void AVP_ServerActor_Human::SendPerformActionMessage( Asura_Guid uActionGuid )
{
	AVP_ServerNode_GS2Node* pxNode = AVP_ServerNode_GS2Node::FindEntity( uActionGuid );

	if (pxNode)
	{
		// pxNodeIsValid!
		Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_PERFORM_AI_OBJECTIVE_ACTION, sizeof(AVP_Message_PerformObjectiveAction));

		if (pxMessage)
		{
			AVP_Message_PerformObjectiveAction* pxMsgData = static_cast<AVP_Message_PerformObjectiveAction*>(pxMessage->GetData());

			pxMessage->SetToGuid(GetGuid());
			pxMessage->SetFromGuid(GetGuid());
			pxMsgData->SetActionGuid(pxNode->GetGuid());
			Asura_Network::Send(pxMessage);
		}
	}
}

//----------------------------------------------------------------------
// AVP_ServerActor_Human :: ProcessCurrentObjective
//----------------------------------------------------------------------
void AVP_ServerActor_Human::ProcessCurrentObjective( void )
{
	// Make sure our current Objective is actually valid
	if ( !GetCurrentObjective() )
	{
		return;
	}

	// Get the objective entity
	AVP_ServerNode_Objective_Marine* pxObjective = AVP_ServerNode_Objective_Marine::FindEntity( GetCurrentObjectiveGuid() );

	if ( !pxObjective )
	{
		return;
	}

	if ( pxObjective->GetObjectiveCompleteOnMessageOnly() && pxObjective->GetHasReceivedCompleteMessage() )
	{
		GetCurrentObjective()->SetObjectiveState( AVP_ServerNode_Objective_Marine::COMPLETE );
	}

	// switch on the objective state
	switch ( GetCurrentObjectiveState() )
	{
		// If we haven't started check the members how many members have started moving to objective;
		case AVP_ServerNode_Objective_Marine::NOT_STARTED:
		{

		}break;

		// Check if the squad leader has arrived at the objective
		case AVP_ServerNode_Objective_Marine::MOVING_TO_OBJECTIVE:
		{
			if ( pxObjective->Contains( GetPosition() ) )
			{
				GetCurrentObjective()->SetObjectiveState( AVP_ServerNode_Objective_Marine::ARRIVED_AT_OBJECTIVE );
				pxObjective->SendObjectiveReachedMessage();
			}
		}break;

		// If we have arrived at the objective then members will need actions assigning.
		case AVP_ServerNode_Objective_Marine::ARRIVED_AT_OBJECTIVE:
		{
			u_int uNumRequired = 0, uNumOptional = 0;

			GetNumOptionalAndRequiredActionGuids( &uNumRequired, &uNumOptional, true );

			if ( uNumRequired == 0 && uNumOptional == 0 )
			{
				GetCurrentObjective()->SetObjectiveState( AVP_ServerNode_Objective_Marine::COMPLETE );
			}
			
		}break;

		// waiting for members actions to complete
		case AVP_ServerNode_Objective_Marine::WAITING_FOR_COMPLETION:
		{
			u_int uNumRequired = 0, uNumOptional = 0;

			GetNumOptionalAndRequiredActionGuids( &uNumRequired, &uNumOptional, true );
			
			if ( uNumRequired == 0 && uNumOptional == 0 )
			{
				GetCurrentObjective()->SetObjectiveState( AVP_ServerNode_Objective_Marine::COMPLETE );
			}

		}break;

		case AVP_ServerNode_Objective_Marine::COMPLETE:
		{
			delete m_pxCurrentObjective;
			m_pxCurrentObjective = NULL;
			m_bHasObjective = false;

		}break;
	}
}

//----------------------------------------------------------------------
// AVP_ServerActor_Human :: PlayerMeleedFriendly
//----------------------------------------------------------------------
void AVP_ServerActor_Human::PlayerMeleedFriendly( Asura_Guid uGuid )
{
	m_uNumTimesMeleedFriendly += 1;
	m_bFriendlyMeleeTriggered = true;
	m_fFriendlyMeleeTimer = 2.0f;
	m_uLastMeleedFriendlyGuid = uGuid;
	m_fLastFriendlyMeleeTimer = 20.0f;
	m_fLastFriendlyMeleeTrigger = true;
}

//----------------------------------------------------------------------
// AVP_ServerActor_Human :: PlayerMeleedFriendly
//----------------------------------------------------------------------
void AVP_ServerActor_Human::ProcessFreindlyMelee()
{
	m_fFriendlyMeleeTimer -= Asura_Timers::GetFrameTime() * Asura_Timers::GetGameTimeScale();

	if ( m_fFriendlyMeleeTimer <= 0.0f )
	{
		m_fFriendlyMeleeTimer = 0.0f;
	}

	if ( m_fLastFriendlyMeleeTimer > 0.0f )
	{
		m_fLastFriendlyMeleeTimer -= Asura_Timers::GetFrameTime() * Asura_Timers::GetGameTimeScale();
	}

	if ( m_fLastFriendlyMeleeTimer <= 0.0f )
	{
		m_fLastFriendlyMeleeTimer = 0.0f;
	}

	if ( m_fLastFriendlyMeleeTrigger && m_fLastFriendlyMeleeTimer <= 0.0f )
	{
		m_uNumTimesMeleedFriendly = 0;
		m_bFriendlyMeleeTriggered = false;
		m_fFriendlyMeleeTimer = 0.0f;
		m_uLastMeleedFriendlyGuid = ASURA_GUID_UNREGISTERED;
		m_fLastFriendlyMeleeTimer = 0.0f;
		m_fLastFriendlyMeleeTrigger = false;
	}

	if ( m_bFriendlyMeleeTriggered )
	{
		if ( m_fFriendlyMeleeTimer > 0.0f )
		{
			if ( m_uNumTimesMeleedFriendly == 1 )
			{
				m_bFriendlyMeleeTriggered = false;

				AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity( m_uLastMeleedFriendlyGuid );

				if ( pxHuman )
				{
					pxHuman->PlayDialogue( AVP_DLG_HUMAN_SCARED_REACTION );
				}
			}
			else if ( m_uNumTimesMeleedFriendly > 1 )
			{
				m_bFriendlyMeleeTriggered = false;

				AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity( m_uLastMeleedFriendlyGuid );


				if ( pxHuman )
				{
					Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_MELEE_PLAYER_BACK );
					pxHuman->HandleMessage( pxMessage );
					Asura_Network::DestroyMessage( pxMessage );
				}
			}
		}
	}
}

void AVP_ServerActor_Human::DropBuddy()
{
	AVP_ServerActor_Human* pxBuddyMarine = AVP_ServerActor_Human::FindEntity( m_uPickedUpBuddyGuid );
	Asura_Assert( pxBuddyMarine, "No buddy to drop" );
	if ( pxBuddyMarine )
	{
		pxBuddyMarine->OnDroppedByPlayer();
	}
	m_uPickedUpBuddyGuid = ASURA_GUID_UNREGISTERED;
	
	// This should move to the drop move finish
	// This messages the clients to do the same	
	RestorePreviousWeapon();
}

void AVP_ServerActor_Human::PickUpBuddy( const Asura_Guid uBuddyGuid )
{
	m_uPickedUpBuddyGuid = uBuddyGuid;
	AVP_ServerActor_Human* pxBuddyMarine = AVP_ServerActor_Human::FindEntity( m_uPickedUpBuddyGuid );
	if ( pxBuddyMarine )
	{
		pxBuddyMarine->OnPickedUpByPlayer();
	}

	// This should move to the pick up move finish
	Asura_Hash_ID uTequilaGunHash = AVP_Settings::IsMultiPlayer() ? uAVP_INVENTORY_WEAPONHASH_TEQUILA_MP : uAVP_INVENTORY_WEAPONHASH_TEQUILA_SP;
	AVP_Inventory_Group* pxGroup = GetInventorySystem()->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON );
	if ( pxGroup )
	{
		pxGroup->SwapCurrentItemForPickup( uTequilaGunHash );
	}
	
	AVP_ClientActor_Human* pxClientActor = AVP_ClientActor_Human::FindEntity( GetGuid() );
	AVP_Inventory_Group* pxClientGroup = pxClientActor->GetInventorySystem()->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON );
	if ( pxClientGroup )
	{
		pxClientGroup->SwapCurrentItemForPickup( uTequilaGunHash );
	}
}

void AVP_ServerActor_Human::OnDroppedByPlayer()
{
	m_bPickedUp = false;
	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_PUT_TEQUILA_DOWN );
	HandleMessage( pxMessage );
	pxMessage->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );
	pxMessage->SetFromGuid( GetGuid() );
	Asura_Network::Send( pxMessage );

	if ( m_pxPhysicsObjectHandler && m_pxPhysicsObjectHandler->GetPhysicsObject() )
	{
		m_pxPhysicsObjectHandler->GetPhysicsObject()->SetIgnore( false );
	}
	SendMessagesForEvent( AVP_ENTITY_EVENT_ON_PUT_DOWN_BY_PLAYER );
}

void AVP_ServerActor_Human::OnPickedUpByPlayer()
{
	m_bPickedUp = true;
	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_PICK_TEQUILA_UP );
	HandleMessage( pxMessage );
	pxMessage->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );
	pxMessage->SetFromGuid( GetGuid() );
	Asura_Network::Send( pxMessage );

	if ( m_pxPhysicsObjectHandler && m_pxPhysicsObjectHandler->GetPhysicsObject() )
	{
		m_pxPhysicsObjectHandler->GetPhysicsObject()->SetIgnore( true );
	}
	SendMessagesForEvent( AVP_ENTITY_EVENT_ON_PICKED_UP_BY_PLAYER );
}

void AVP_ServerActor_Human::OnPickup(AVP_ServerEntity_Pickup* pxPickup, float fQuantity, float fSecondaryQuantity, bool bSetAsCurrent /* default to true */)
{
	if(pxPickup)
	{
		const AVP_Pickup* pxPickupObject = pxPickup->GetPickupObject();
		if(pxPickupObject)
		{
			switch(pxPickupObject->GetGroupHash())
			{
				case uAVP_INVENTORY_GROUPHASH_TOOL:
				{
					// Test to see if it is the hacking tool
					if(pxPickupObject->GetBlueprintHash() == AVP_Pickup_Tool::s_uHackingToolHash)
					{
						m_uHackingToolGuid = pxPickup->GetGuid();
						m_bHasHackingTool  = true;
						return;
					}
				}
				default: break;
			}
			PARENT::OnPickup(pxPickup, fQuantity, fSecondaryQuantity, bSetAsCurrent);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::HandleInteractions
// Author       : AJK
// Date         : 27-11-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Human::HandleInteractions()
{
	AVP_ClientActor_Human*		  pxClient          =  AVP_ClientActor_Human::FindEntity(GetGuid());
	if (pxClient)
	{
		AVP_Actor_InteractionHandler* pxInteract = pxClient->GetInteractionHandler();
		if(pxInteract)
		{
			const Asura_Movement_System*  pxMovementSystem	= (pxClient == NULL) ? NULL : pxClient->GetMovementSystem();
			const AVP_Move*				  pxCurrentMove		= static_cast< const AVP_Move* >( (pxMovementSystem == NULL) ? NULL : pxMovementSystem->GetCurrentMove() );

			if(pxCurrentMove)
			{
				// Check if the move is an interaction one for the battery
				if(pxCurrentMove->GetHash() == AVP_Human_Move_Interaction_PlugBattery::GetStaticHash())
				{
					// Attach the battery as pickup
					PlugBattery();
				}
				else if(pxCurrentMove->GetHash() == AVP_Human_Move_Interaction_UnPlugBattery::GetStaticHash())
				{
					// Remove the battery from the receptacle
					RemoveBattery();
				}
				// Check if the move is an interaction one for the battery
				if(pxCurrentMove->GetHash() == AVP_Human_Move_Interaction_PlugHackingTool::GetStaticHash())
				{
					// Attach the hacking tool as pickup
					PlugHackingTool();
				}
				else if(pxCurrentMove->GetHash() == AVP_Human_Move_Interaction_UnPlugHackingTool::GetStaticHash())
				{
					// Remove the hacking from the hacking point
					RemoveHackingTool();
				}
				else if(pxCurrentMove->GetHash() == AVP_Human_Move_Interaction_GrabHackingTool::GetStaticHash())
				{
					// Hide the hacking tool
					HideHackingTool();

					// Show the weapon again
					pxClient->ActivateCurrentWeapon();
				}
				else if(pxCurrentMove->GetHash() == AVP_Human_Move_Interaction_GrabBattery::GetStaticHash())
				{
					SetBatteryAsWeapon();
					
					// show the wepaon battery
					pxClient->ActivateCurrentWeapon();
				}
				else if( pxCurrentMove->GetHash() == AVP_Human_Move_Interaction_LetGoBattery::GetStaticHash() )
				{
					//Restore previous weapon
					RestorePreviousWeapon();
				}
				else if( pxCurrentMove->GetHash() == AVP_Human_Move_Interaction_LetGoHackingTool::GetStaticHash() )
				{
					// Show the weapon again
					pxClient->ActivateCurrentWeapon();
				}
				else if( pxCurrentMove->GetHash() == AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
				{
					// find if the object being interacted with is an invisiswitch
					const AVP_Actor_InteractionHandler*  pxInteractionHandler = pxClient->GetInteractionHandler();
					if (pxInteractionHandler)
					{
						//Get GUID of interaction
						const Asura_Guid uGuid = pxInteractionHandler->GetGuidForInteraction( AVP_INTERACTION_TYPE_DEFAULT );
						if( uGuid != ASURA_GUID_UNREGISTERED )
						{
							AVP_ClientEntity_InteractInvisiSwitch* pxInvisiSwitch = AVP_ClientEntity_InteractInvisiSwitch::FindEntity(uGuid);
							if( pxInvisiSwitch )
							{
								Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_SENTRYGUN_SWITCH_PLAYANIMATION, sizeof( AVP_Message_SentryGun_Switch_PlayAninmation ), GetClassification());
								if(pxMessage)
								{
									AVP_Message_SentryGun_Switch_PlayAninmation* pxData = static_cast< AVP_Message_SentryGun_Switch_PlayAninmation* >( pxMessage->GetData() );
									pxData->SetAnimHash( pxInvisiSwitch->GetAnimHashForSpeciesInteraction( AVP_Settings::AVP_SPECIES_HUMAN ) );

									pxMessage->SetToGuid( pxInvisiSwitch->GetGuid() );
									pxMessage->SetFromGuid( GetGuid() );
									Asura_Network::Send( pxMessage );
								}
							}
						}
					}
				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::PlugBattery
// Author       : AJK
// Date         : 27-11-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Human::PlugBattery()
{
	// Get current inventory item
	AVP_InventoryItem_Weapon* pxCurrentWeapon = static_cast< AVP_InventoryItem_Weapon* >( GetInventorySystem()->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON) );
	if(pxCurrentWeapon)
	{
		Asura_Hash_ID uPickupBlueprintHash;
		if( (uPickupBlueprintHash = pxCurrentWeapon->GetPickupBlueprintHash()) != ASURA_HASH_ID_UNSET )
		{
			// Get the client
			AVP_ClientActor_Human* pxActor = AVP_ClientActor_Human::FindEntity(GetGuid());
			if(pxActor)
			{
				// Hide the weapon battery
				pxActor->DeactivateCurrentWeapon();
	
				const AVP_Actor_InteractionHandler*  pxInteractionHandler = pxActor->GetInteractionHandler();
				if (pxInteractionHandler)
				{
					//Get GUID of interaction
					const Asura_Guid uGuid = pxInteractionHandler->GetGuidForInteraction( AVP_INTERACTION_TYPE_DEFAULT );
					if(uGuid != ASURA_GUID_UNREGISTERED)
					{
						//Get server interaction battery
						AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = AVP_ServerEntity_InteractBatteryReceptacle::FindEntity(uGuid);
						if(pxBatteryReceptacle )
						{
							pxBatteryReceptacle->InsertTool(m_uPickupGuid, GetGuid());

							m_uPickupGuid = ASURA_GUID_UNREGISTERED;
						}
					}
				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::PlugHackingTool
// Author       : AJK
// Date         : 27-11-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Human::PlugHackingTool()
{
	// Get the client, must be human
	if (   GetClassification() == AVP_ENTITYCLASS_CIVILIAN
		|| GetClassification() == AVP_ENTITYCLASS_HUMAN
		|| GetClassification() == AVP_ENTITYCLASS_MARINE )
	{
		AVP_ClientActor_Human* pxClientHuman = AVP_ClientActor_Human::FindEntity(GetGuid());
		if(pxClientHuman)
		{

			const AVP_Actor_InteractionHandler*  pxInteractionHandler = pxClientHuman->GetInteractionHandler();
			if (pxInteractionHandler)
			{
				//Get GUID of interaction
				const Asura_Guid uGuid = pxInteractionHandler->GetGuidForInteraction( AVP_INTERACTION_TYPE_DEFAULT );
				if(uGuid != ASURA_GUID_UNREGISTERED)
				{
					//Get server interaction battery
					AVP_ServerEntity_InteractHackingPoint* pxHackingPoint = AVP_ServerEntity_InteractHackingPoint::FindEntity(uGuid);
					if(pxHackingPoint)
					{

						pxHackingPoint->InsertTool(GetHackingToolGuid(), GetGuid());

						// Set again the hacking tool to unregistred
						SetHackingToolGuid( ASURA_GUID_UNREGISTERED );
						m_bHasHackingTool = false;
					}
				}
			}
		}
	}
	else
	{
		Asura_Assert(false, "Should be a human who use the hacking tool!");
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::HideHackingTool
// Author       : AJK
// Date         : 27-11-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Human::HideHackingTool()
{
	if (m_pxPickupHandler)
	{
		AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::FindEntity(GetHackingToolGuid());
		if(pxPickup)
		{
			if ( pxPickup->GetParentGuid() != ASURA_GUID_UNREGISTERED )
			{
				// Send the message to unlink from the actor
				Asura_Message*	pxMessage = Asura_Network::CreateMessage(ASURA_MSG_UNLINK_FROM_PARENT);

				if(pxMessage)
				{

					pxMessage->SetFromGuid( GetGuid() );
					pxMessage->SetToGuid( pxPickup->GetGuid() );

					Asura_Network::Send( pxMessage );
				}
			}

			// This will update all client entities
			m_pxPickupHandler->OnPickup(this, pxPickup);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::RemoveHackingTool
// Author       : AJK
// Date         : 05-12-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Human::RemoveHackingTool() 
{
	// Get the client
	AVP_ClientActor_Human* pxActor = AVP_ClientActor_Human::FindEntity(GetGuid());
	if(pxActor)
	{
		const AVP_Actor_InteractionHandler*  pxInteractionHandler = pxActor->GetInteractionHandler();
		if (pxInteractionHandler)
		{
			//Get GUID of interaction
			const Asura_Guid uGuid = pxInteractionHandler->GetGuidForInteraction( AVP_INTERACTION_TYPE_DEFAULT );
			if(uGuid != ASURA_GUID_UNREGISTERED)
			{
				//Get server interaction Hacking point
				AVP_ServerEntity_InteractHackingPoint* pxHackingPoint = AVP_ServerEntity_InteractHackingPoint::FindEntity(uGuid);
				if(pxHackingPoint)
				{
					AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::FindEntity(pxHackingPoint->GetToolGuid());
					if(pxPickup)
					{
						if(m_pxPickupHandler)
						{
							// This will update all client entities
							m_pxPickupHandler->OnPickup(this, pxPickup);
						}
					}

					SetHackingToolGuid( pxHackingPoint->GetToolGuid() );

					m_bHasHackingTool = true;

					// remove hacking tool from case
					pxHackingPoint->RemoveTool();

					// Attach it to marine left hand
					pxHackingPoint->AttachHackingToolToLeftHand(GetGuid(), false);

				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::RemoveBattery
// Author       : AJK
// Date         : 27-11-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Human::RemoveBattery() 
{
	// Get current inventory item
	const AVP_Inventory_Item* pxCurrentWeapon = GetInventorySystem()->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON);
	if(pxCurrentWeapon)
	{
		Asura_Hash_ID uPickupBlueprintHash;
		if( (uPickupBlueprintHash = pxCurrentWeapon->GetPickupBlueprintHash()) != ASURA_HASH_ID_UNSET )
		{
			//// Get the client
			AVP_ClientActor_Human* pxActor = AVP_ClientActor_Human::FindEntity(GetGuid());
			if(pxActor)
			{
				const AVP_Actor_InteractionHandler*  pxInteractionHandler = pxActor->GetInteractionHandler();
				if (pxInteractionHandler)
				{
					//Get GUID of interaction
					const Asura_Guid uGuid = pxInteractionHandler->GetGuidForInteraction( AVP_INTERACTION_TYPE_DEFAULT );
					if(uGuid != ASURA_GUID_UNREGISTERED)
					{
						//Get server interaction battery
						AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = AVP_ServerEntity_InteractBatteryReceptacle::FindEntity(uGuid);
						if(pxBatteryReceptacle)
						{
							AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::FindEntity(pxBatteryReceptacle->GetToolGuid());
							if(pxPickup)
							{
								if(m_pxPickupHandler)
								{
									// Set Pickup GUID as the battery
									m_uPickupGuid = pxPickup->GetGuid();

									// To turn off activation of next weapon battery
									pxActor->SetAllowWeaponActivationAnim(false);

									// This will update all client entities
									m_pxPickupHandler->OnPickup(this, pxPickup);

									// remove battery from case
									pxBatteryReceptacle->RemoveTool();

									// Attach it to marine right hand
									pxBatteryReceptacle->AttachBatteryToRightHand(GetGuid());
								}
							}
						}
					}
				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::RemoveBattery
// Author       : AJK
// Date         : 27-11-2008
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Human::SetBatteryAsWeapon() 
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::FindEntity(m_uPickupGuid);
	if(pxPickup)
	{
		if(m_pxPickupHandler)
		{
			pxPickup->SetHidden(true);

			if ( pxPickup->GetParentGuid() != ASURA_GUID_UNREGISTERED )
			{
				// Send the message to unlink from the actor
				Asura_Message*	pxMessage = Asura_Network::CreateMessage(ASURA_MSG_UNLINK_FROM_PARENT);

				if(pxMessage)
				{

					pxMessage->SetFromGuid( GetGuid() );
					pxMessage->SetToGuid( pxPickup->GetGuid() );

					Asura_Network::Send( pxMessage );
				}
			}
		}
	}				
}

float AVP_ServerActor_Human::CalculateAndUpdateLocationSpecificDamage(const AVP_EntityDamage_RequestVictim& xDamage, u_int& uDestroyedSkinElements, u_int& uGibbedSkinElements, bool bForceFatal)
{
	float fAmount = PARENT::CalculateAndUpdateLocationSpecificDamage(xDamage, uDestroyedSkinElements, uGibbedSkinElements, bForceFatal);

//	xDamage.SetImpulseStrength(500.0f);
// 	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_Human_MoveToFloor ) );
// 
// 	if ( pxMessage )
// 	{
// 		// Teleport to right ori
// 		Asura_Matrix_3x3 xOri;
// 		Asura_Vector_3 xDirection( Asura_Maths::ZeroVector3 );
// 		xDamage.GetDirection( xDirection ); xDirection.y = 0.0f;
// 		Asura_Maths::OrientationFromDirection( xOri, -xDirection );
// 		TeleportTo( GetPosition(), xOri, true );
// 
// 		// Send move msg
// 		AVP_Message_Human_MoveToFloor* pxData = static_cast< AVP_Message_Human_MoveToFloor* >( pxMessage->GetData() );
// 		pxData->m_uAnimHash = 0x26884c4b; //"marine_knocked_down"
// 		pxData->SetMoveID( AVP_Move_Human_MoveToFloor::GetStaticHash() );
// 		pxMessage->SetToGuid( GetGuid() );
// 		pxMessage->SetFromGuid( GetGuid() );
// 		Asura_Network::Send( pxMessage );
// 	}

	return fAmount;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Human::OnHeadRipped
// Author       : AJK
// Date         : 08-01-2009
////////////////////////////////////////////////////////////////////////////////
void AVP_ServerActor_Human::OnHeadRipped() const
{
	if ( m_usHeadRippedMessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usHeadRippedMessageBlock, GetGuid() );
	}
}

void AVP_ServerActor_Human::OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	OnHeadRipped();
	PARENT::OnDeath(xDamage);
	
	if (!IsPlayer())
	{
		AVP_InventoryItem_Torch*	pxTorchItem	= GetTorchItem();

		if(pxTorchItem != NULL)
		{
			pxTorchItem->RequestEndUsage(AVP_InventoryItem_Torch::TT_AUTOMATIC);
		}
	}


}

#ifdef ASURA_TOOLS
void AVP_ServerActor_Human::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance(pxInstance);

	m_usHeadRippedMessageBlock = pxInstance->ConvertMessageBlock( m_usHeadRippedMessageBlock );
}

void AVP_ServerActor_Human::GetCustomEventName(int iEventIndex, Asura_String& xEventName)
{
	switch(iEventIndex)
	{
		case MESSAGE_BLOCK_HEAD_RIPPED:		xEventName = "On Head ripped";		return;
	}

	Asura_Assert(FALSE, "Invalid human custom event index");

	xEventName = "";
}

unsigned short* AVP_ServerActor_Human::GetCustomEvent(int iIndex)
{
	switch(iIndex)
	{
		case MESSAGE_BLOCK_HEAD_RIPPED:		return &m_usHeadRippedMessageBlock;
	}

	Asura_Assert(FALSE, "Invalid human custom event index");

	return NULL;
}


#endif // ASURA_TOOLS
