///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerActor_Android.h
// Author : gareth Thomas
// Date   : 17-12-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE
#ifndef __AVP_SERVERACTOR_ANDROID_H__
#define __AVP_SERVERACTOR_ANDROID_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_HashedList.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_ServerActor_Android
// Author       : Andy Tate
// Date         : 3-Oct-2006
//
// Description  : This class handles the operation of the Android actor, the
//				  Android is AI controlled.
////////////////////////////////////////////////////////////////////////////////
class AVP_ServerActor_Android : public AVP_ServerActor_Human
{
#if defined(ASURA_TOOLS)
	// Friend classes...
	friend class ToolsAVP_Entity_Android;
#endif//defined(ASURA_TOOLS)

public:

	enum DialogueHashIDs
	{
		// Passive State Dialogue
		ANDROID_DLG_HUMAN_ALERT_TO_PASSIVE = 0,
		ANDROID_DLG_HUMAN_IDLE_CHATTER,
		ANDROID_DLG_HUMAN_ENTERING_COVER,
		ANDROID_DLG_HUMAN_PASSIVE_TO_ALERT,
		ANDROID_DLG_HUMAN_AGGRESSIVE_TO_ALERT,
		ANDROID_DLG_HUMAN_ENTERING_AGGRESSIVE,
		ANDROID_DLG_HUMAN_ATTACKING,
		ANDROID_DLG_HUMAN_RELOADING,
		ANDROID_DLG_HUMAN_KILLED_THREAT,
		ANDROID_DLG_HUMAN_TEAM_MATE_KILLED,
		ANDROID_DLG_HUMAN_SNEAK_ATTACK_RESPONSE,
		
		ANDROID_DLG_MAX_DIALOGUE_HASH_IDS,
	};

	

	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	virtual void InitialiseDialogueFromBlueprint(Asura_Hash_ID uBlueprintHashID);
	virtual Asura_Hash_ID GetDialogueHashID(u_int uIndex)	{return m_uAndroidDialogueHashIDs[uIndex];}

	virtual void PostLoadInitialise();
	virtual void MakeAI();

	// Static Public functions...
	static void	Create(Asura_Guid uGUID);
	static AVP_ServerActor_Android*	CreateAndReturn(Asura_Guid uGUID);

	// Static list accessors...
	static AVP_ServerActor_Android*							FindEntity(const Asura_Guid uGuid)	{return s_xList.FindEntity(uGuid);}
	static const Asura_Hashed_List<AVP_ServerActor_Android>*	GetList(void)						{return &s_xList;}

	bool HasDialogueBeenInitialised()		{return m_bDialogueHashIdsInitialised;}
	

protected:
	// Overridden event functions...
	virtual void	OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage);

	// protected dialogue functions
	virtual Asura_Hash_ID	GetDialogueHashIDFromDialoguetype	(DialogueType uDialogType);

	void EffectMorale(Asura_Guid uGuidToEffect);

	virtual void OnEntityActivate();
	virtual void OnEntityDeactivate();

	// Protected functions...
	AVP_ServerActor_Android(const Asura_Guid uGUID);
	virtual ~AVP_ServerActor_Android(void);

private:
	enum { CURRENT_CHUNK_VERSION = 1000 };

	// Private typedefs...
	typedef AVP_ServerActor_Human PARENT;

	// Static Private variables...
	static	Asura_Hashed_List<AVP_ServerActor_Android>	s_xList;

	// Private Variables...
	Asura_Hash_ID			m_uAndroidDialogueHashIDs[ANDROID_DLG_MAX_DIALOGUE_HASH_IDS];
	bool					m_bDialogueHashIdsInitialised;

	// Private functions...
	AVP_ServerActor_Android(void);
	AVP_ServerActor_Android(const AVP_ServerActor_Android& xOther);
	AVP_ServerActor_Android& operator=(const AVP_ServerActor_Android& xOther);
};   // End of class 'AVP_ServerActor_Android'.

#endif   // ifndef __AVP_SERVERACTOR_ANDROID_H__

/// End of file. ///
