//-----------------------------------------------------------------------------
// File   : AVP_ServerActor_civilian.h
// Author : Gareth Thomas
// Date   : 12 / 02 / 2008
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//-----------------------------------------------------------------------------

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE
#ifndef __AVP_SERVERACTOR_MARINE_H__
#define __AVP_SERVERACTOR_MARINE_H__

//-----------------------------------------------------------------------------
// Include files...
//-----------------------------------------------------------------------------

#include "Core/Asura_HashedList.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"

//-----------------------------------------------------------------------------
// Forward class declarations...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Class declaration...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Class        : AVP_ServerActor_civilian
// Author       : Gareth Thomas
// Date         : 12 / 02 / 2008
//
// Description  : This class handles the operation of the Marine actor, the
//				  Marine is Player or AI controlled.
//-----------------------------------------------------------------------------
class AVP_ServerActor_Marine : public AVP_ServerActor_Human
{
#if defined(ASURA_TOOLS)
	// Friend classes...
	friend class ToolsAVP_Entity_Marine;
#endif//defined(ASURA_TOOLS)

public:

	enum DialogueHashIDs
	{
		MARINE_DLG_HUMAN_AGGRESSIVE_TO_ALERT = 0,
		MARINE_DLG_HUMAN_ALERT_TO_PASSIVE,
		MARINE_DLG_HUMAN_ALERT_TO_PASSIVE_FROM_AGGRESSIVE,
		MARINE_DLG_HUMAN_ATTACKING,
		MARINE_DLG_HUMAN_ENTERING_AGGRESSIVE,
		MARINE_DLG_HUMAN_ENTERING_COVER,
		MARINE_DLG_HUMAN_FALLING_BACK,
		MARINE_DLG_HUMAN_FLEEING,
		MARINE_DLG_HUMAN_FRIENDLY_FIRE,
		MARINE_DLG_HUMAN_HURRY_UP,
		MARINE_DLG_HUMAN_IDLE_CHATTER,
		MARINE_DLG_HUMAN_KEEP_FORMATION,
		MARINE_DLG_HUMAN_KILLED_THREAT,
		MARINE_DLG_HUMAN_KILLED_THREAT_RESPONSE,
		MARINE_DLG_HUMAN_PASSIVE_TO_ALERT,
		MARINE_DLG_HUMAN_PROVIDING_COVERING_FIRE,
		MARINE_DLG_HUMAN_RELOADING,
		MARINE_DLG_HUMAN_SNEAK_ATTACK_RESPONSE,
		MARINE_DLG_HUMAN_TAUNTING,
		MARINE_DLG_HUMAN_TEAM_MATE_KILLED,

		MARINE_DLG_MAX_DIALOGUE_HASH_IDS,
	};

	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	// Static Public functions...
	static void	Create(Asura_Guid uGUID);
	static AVP_ServerActor_Marine* CreateAndReturn(Asura_Guid uGUID);

	static bool AwardLoadsOfDamageToAllMarines( bool bHelp );

	virtual Asura_Hash_ID	GetDialogueHashID(u_int uIndex)		{return m_uMarineDialogueHashIDs[uIndex];}
	virtual void			InitialiseDialogueFromBlueprint		(Asura_Hash_ID uBlueprintHashID);

	virtual void PostLoadInitialise();
	virtual void MakeAI();

	// Static list accessors...
	static AVP_ServerActor_Marine* FindEntity(const Asura_Guid uGuid)				{return s_xList.FindEntity(uGuid);}
	static const Asura_Hashed_List<AVP_ServerActor_Marine>* GetList(void)			{return &s_xList;}

	bool HasDialogueBeenInitialised()		{return m_bDialogueHashIdsInitialised;}

protected:
	// protected dialogue functions
	virtual Asura_Hash_ID	GetDialogueHashIDFromDialoguetype	(DialogueType uDialogType);
	virtual void	OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage);

	 virtual void	OnEntityActivate();
	 virtual void	OnEntityDeactivate();

	// Protected functions...
	AVP_ServerActor_Marine(const Asura_Guid uGUID);
	virtual ~AVP_ServerActor_Marine(void);

private:
	enum { CURRENT_CHUNK_VERSION = 1000 };

	// Private typedefs...
	typedef AVP_ServerActor_Human PARENT;

	// Static Private variables...
	static	Asura_Hashed_List<AVP_ServerActor_Marine>	s_xList;

	// Private Variables...
	Asura_Hash_ID			m_uMarineDialogueHashIDs[MARINE_DLG_MAX_DIALOGUE_HASH_IDS];
	bool					m_bDialogueHashIdsInitialised;

	// Private functions...
	AVP_ServerActor_Marine(void);
	AVP_ServerActor_Marine(const AVP_ServerActor_Marine& xOther);
	AVP_ServerActor_Marine& operator=(const AVP_ServerActor_Marine& xOther);
};

#endif   // ifndef __AVP_SERVERACTOR_MARINE_H__

/// End of file. ///
