///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientActor_Predalien.cpp
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_ClientActor_Predalien.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
#include "Core/Asura_MaterialResponse_System.h"
#endif	// defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
#include "Core/Asura_Timers.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Viewport.h"
#include "SpecialFX/Asura_Decal_Manager.h"
#include "Animation/Asura_AnimationBlend.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Core/Asura_Timers.h"

// Game independent header-files...
#include "Actor/AVP_Actor_Selector.h"
#include "Actor/AVP_Actor_InteractionHandler.h"
#include "Actor/AVP_Actor_PickupHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_Description.h"
#include "Actor/Vertebrates/AVP_Vertebrate_HUDHandler.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_AIAnimStateData.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_AnimationHandler.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_Description.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_MeleeCounter_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_Messages_Predalien.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/AVP_Move_Interaction.h"
#include "Moves/AVP_Movement_System.h"
#include "Moves/Vertebrates/Predalien/AVP_Predalien_MovesJump.h"
#include "Moves/Vertebrates/Predalien/AVP_Predalien_MovesMelee.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////
//DVSVAR float	dvs_fDownwardMovementMultiplier		= 1.0f; // temp

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ClientActor_Predalien> AVP_ClientActor_Predalien::s_xList;

////////////////////////////////////////////////////////////////////////////////
// Function Declarations...
////////////////////////////////////////////////////////////////////////////////

AVP_ClientActor_Predalien::AVP_ClientActor_Predalien()
: AVP_ClientActor_Vertebrate(AVP_PolygonFlag_IgnoreForPredatorCollision)
, m_uPredalienFlags(0)
{
	Classification = AVP_ENTITYCLASS_PREDALIEN;

	// Initialise the entity...
	SetTimeLagType(TimeLag_LocalPlayer);

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}

AVP_ClientActor_Predalien::~AVP_ClientActor_Predalien ()
{
	// Don't forget to remove us from the list...
	s_xList.RemoveFromList(this);
}

AVP_ClientActor_Predalien* AVP_ClientActor_Predalien::Create(const Asura_Message* pxMessage)
{
	AVP_ClientActor_Predalien*	pxClient	= new AVP_ClientActor_Predalien;

	// Initialise the entity...
	pxClient->HandleMessage(pxMessage);

	// Return the entity...
	return(pxClient);
}
void AVP_ClientActor_Predalien::PostCreateInitialise(void)
{
	//If we do not have an animation handler yet, create it so we can set the skin.
	if ( !m_pxAnimationHandler && GetBlueprintHashID() != ASURA_HASH_ID_UNSET )
	{
		CreateAnimationHandler();
	}

	PARENT::PostCreateInitialise();
}


void AVP_ClientActor_Predalien::Update(void)
{
	// Firstly, ensure that the parent is up-to-date...
	PARENT::Update();
}

bool AVP_ClientActor_Predalien::HandleMessage(const Asura_Message* pxMessage)
{
	// What sort of message is it?
	switch(pxMessage->GetMsgID())
	{
	case ASURA_MSG_CLIENTUPDATE:
		{
			ProcessClientUpdate(static_cast<const AVP_Message_PredalienClientUpdate*>(pxMessage->GetData()), pxMessage->GetSentTime());
			return(true);
		}

	default:
		{
			return(PARENT::HandleMessage(pxMessage));
		}
	}		// End of switch.
}

void AVP_ClientActor_Predalien::CreateAnimationHandler()
{
	Asura_Assert( m_pxAnimationHandler == NULL, "Already got an animation handler" );
	m_pxAnimationHandler = new AVP_Predalien_AnimationHandler(*this);
	m_pxAnimationHandler->Initialise();
}

void AVP_ClientActor_Predalien::GetFootPosition( u_int uFootItr, Asura_Vector_3& xFootPos )
{
	if ( m_pxAnimationHandler )
	{
		m_pxAnimationHandler->GetAnimation()->GetBoneLocation( AVP_Predalien_Description::GetFootBoneID( uFootItr, m_pxAnimationHandler->IsFirstPerson() ), &xFootPos, NULL, true );
	}
}



const AVP_Actor_AnimStateData* AVP_ClientActor_Predalien::GetStateDataFromMesssageData( const u_int uState, const AVP_Message_Actor_ClientUpdate* pxActorData )
{
	return AVP_Predalien_AIAnimStateData::GetAnimStateData( GetAnimStateBlueprintType(), uState );
}

const AVP_Actor_AnimStateData* AVP_ClientActor_Predalien::GetStateData( const u_int uState, const u_int uGroundType, const u_int uLimbDamage, const bool bIsAiming, const u_int uCoverType)
{
	return AVP_Predalien_AIAnimStateData::GetAnimStateData( GetAnimStateBlueprintType(), uState );
}

// must be the same as predator since they using the same waypoint movements.
DEBUGVAR float dbg_fTargetedJumpLowTestsYOffset = -0.5f;
DEBUGVAR float dbg_fTargetedJumpLowTestsXOffset = 0.25f;

bool AVP_ClientActor_Predalien::GetCanUseTargetedJump(const Asura_Vector_3& xTargetPosition, int& iJumpVariation) const
{
	const Asura_Blueprint* pxBlueprint = GetCurrentTargetedJumpBlueprint();
	if (!pxBlueprint) { return false; }

	// To simplify our approximation, just use the y part of the head marker offset. Should still be good enough.
	// Since the jump may change our orientation and look values, attempting to be exact here is pretty much always going to fail.
	Asura_Vector_3 xHeadOffset;
	if (!GetMarkerLocationFromHash(AVP_Vertebrate_Description::GetMarkerPointHashID(AVP_Vertebrate_Description::MPID_EYES), &xHeadOffset, NULL, false)) { return false; }

	const Asura_Vector_3& xMyPos = GetCurrentPosition();

	Asura_Vector_3 xFlatOffset = xTargetPosition - xMyPos;
	const float fHeightOffset = Asura_Maths::Abs(xFlatOffset.y);
	const float fHeightOffsetSign = Asura_Maths::Sign(xFlatOffset.y);
	xFlatOffset.y = 0.0f;
	const float fFlatDistanceSquared = xFlatOffset.MagnitudeSquared();

	if (fFlatDistanceSquared < 0.001f) { return false; }

	Asura_Matrix_3x3 xFlatOri;
	Asura_Maths::OrientationFromDirection(xFlatOri, xFlatOffset); // This normalises the vector internally, so no need to do so in advance.

	Asura_Vector_3 xStarts[3];
	Asura_Vector_3 xTargets[3];
	Asura_Vector_3 xMidpoints[3];

	const Asura_Vector_3 xHeadHeightOffset(0.0f, xHeadOffset.y, 0.0f);
	const Asura_Vector_3 xLowLeftOffset(-dbg_fTargetedJumpLowTestsXOffset, dbg_fTargetedJumpLowTestsYOffset, 0.0f);
	const Asura_Vector_3 xLowRightOffset(dbg_fTargetedJumpLowTestsXOffset, dbg_fTargetedJumpLowTestsYOffset, 0.0f);

	xStarts[0] = xMyPos + xHeadHeightOffset;
	xTargets[0] = xTargetPosition + xHeadHeightOffset;
	xStarts[1] = xMyPos + (xLowLeftOffset * xFlatOri);
	xTargets[1] = xTargetPosition + (xLowLeftOffset * xFlatOri);
	xStarts[2] = xMyPos + (xLowRightOffset * xFlatOri);
	xTargets[2] = xTargetPosition + (xLowRightOffset * xFlatOri);

	for (int i=0; i<3; ++i)
	{
		xMidpoints[i] = (xStarts[i] + xTargets[i]) * 0.5f;
	}

	Asura_Value xValue;
	float fBestMaxFlatDistanceSquared = Asura_Maths::LargeFloat;
	float fBestMinHeightChange = 0.0f; // This will always be positive, regardless of the sign of the actual change.
	float fBestHeadroom = 0.0f;
	int iBestVariation = -1;

	for (int i=0; i<pxBlueprint->GetNumParameters(); ++i)
	{
		if (pxBlueprint->GetParameterValueByIndex(i, xValue))
		{
			const Asura_Blueprint* pxVariationBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_MOVESPREDALIEN, xValue.GetHashIDValue());
			if (!pxVariationBlueprint) { continue; }

			float fMaxFlatDistanceSquared;
			float fMinHeightChange;
			float fHeadroom;

			if (!pxVariationBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_MAXDISTANCE, xValue)) { continue; }
			fMaxFlatDistanceSquared = xValue.GetFloatValue();
			fMaxFlatDistanceSquared *= fMaxFlatDistanceSquared;

			if (!pxVariationBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEIGHTCHANGE, xValue)) { continue; }
			fMinHeightChange = xValue.GetFloatValue();

			if (!pxVariationBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_HEADROOM, xValue)) { continue; }
			fHeadroom = xValue.GetFloatValue();

			// First, make sure its height change is appropriate. 0 height change is always a valid option.
			if ((fMinHeightChange != 0.0f) && (fHeightOffsetSign != Asura_Maths::Sign(fMinHeightChange))) { continue; }
			fMinHeightChange = Asura_Maths::Abs(fMinHeightChange);
			if (fMinHeightChange < fBestMinHeightChange) { continue; }
			if (fMinHeightChange > fHeightOffset) { continue; }

			// Next, make sure it's within valid distance.
			if (fMaxFlatDistanceSquared < fFlatDistanceSquared) { continue; }
			// Only need to compare with the best if our height changes are equal since that's the priority comparison.
			if ((fMinHeightChange == fBestMinHeightChange) && (fMaxFlatDistanceSquared > fBestMaxFlatDistanceSquared)) { continue; }

			// Finally, the headroom check. We want the greatest clearance we think should be safe. We're rapidly approaching needing LOS tests...
			// Headroom is only a deciding factor if the other two values are equal to the previous best.
			if ((fMinHeightChange == fBestMinHeightChange) && (fMaxFlatDistanceSquared == fBestMaxFlatDistanceSquared) && (fHeadroom < fBestHeadroom)) { continue; }

			// We need two LOS tests to make a reasonable guess - from the current head position to an approximation of head position at the midpoint, and from there to the end head position.
			// AJG - 10/09/08 - Now doing this three times to reduce the instances where we jump through geometry.
			bool bFailedLOS = false;
			for (int j=0; j<3; ++j)
			{
				const Asura_Vector_3 xHeadroomMidpoint = xMidpoints[j] + Asura_Vector_3(0.0f, -fHeadroom, 0.0f);

				Asura_Vector_3 xDir = xHeadroomMidpoint - xStarts[j];
				float fDistance = xDir.Normalise();
				Asura_LOS_InputData xInput(xStarts[j], xDir, Guid, fDistance, false, m_usFlagsToIgnore | AsuraPolygonFlag_NoCollisions);
				const Asura_LOS_OutputData* pxOutput = Asura_LOS::TestAgainstEverything(&xInput);
				if (pxOutput->m_fDistanceToIntersection < fDistance)
				{
					bFailedLOS = true;
					break;
				}

				xDir = xTargets[j] - xHeadroomMidpoint;
				fDistance = xDir.Normalise();
				xInput.m_xOrigin = xHeadroomMidpoint;
				xInput.m_xDirection = xDir;
				xInput.m_fInitialMaximumRange = fDistance;
				pxOutput = Asura_LOS::TestAgainstEverything(&xInput);
				if (pxOutput->m_fDistanceToIntersection < fDistance)
				{
					bFailedLOS = true;
					break;
				}
			}

			if (bFailedLOS) { continue; }

			// Finally, this can be considered successful. Store the values.
			fBestMaxFlatDistanceSquared = fMaxFlatDistanceSquared;
			fBestMinHeightChange = fMinHeightChange;
			fBestHeadroom = fHeadroom;
			iBestVariation = i;
		}
	}

	if (iBestVariation >= 0)
	{
		iJumpVariation = iBestVariation;
		return true;
	}
	else
	{
		return false;
	}
}



void AVP_ClientActor_Predalien::GetTargetedJumpAnimationsForVariation(const int iJumpVariation, Asura_Hash_ID& uThirdPersonAnim, bool bRunAfterJump) const
{
	uThirdPersonAnim = ASURA_HASH_ID_UNSET;

	const Asura_Blueprint* pxBlueprint = GetCurrentTargetedJumpBlueprint();
	if (!pxBlueprint) { return; }

	Asura_Value xValue;
	if (!pxBlueprint->GetParameterValueByIndex(iJumpVariation, xValue))
	{
		Asura_Assert(false, "Invalid targeted jump variation.");
		// Try variation 0 as a fallback.
		if (!pxBlueprint->GetParameterValueByIndex(0, xValue)) { return; }
	}

	const Asura_Blueprint* pxVariationBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_MOVESPREDALIEN, xValue.GetHashIDValue());
	if (!pxVariationBlueprint) { return; }

	if (bRunAfterJump)
	{
		if (pxVariationBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_THIRDPERSONANIM_RUN, xValue))
		{
			uThirdPersonAnim = xValue.GetHashIDValue();
		}
	}

	if (uThirdPersonAnim == ASURA_HASH_ID_UNSET)
	{
		if (pxVariationBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_THIRDPERSONANIM, xValue))
		{
			uThirdPersonAnim = xValue.GetHashIDValue();
		}
	}

}

bool AVP_ClientActor_Predalien::InitialiseMoves( void )
{
	if(PARENT::InitialiseMoves())
	{
		Asura_Movement_System& xMovementSystem = *m_pxMovementSystem;
		Asura_Hash_ID uFirstPersonBlueprintHash, uThirdPersonBlueprintHash;
		GetCurrentMeleeBlueprintIDs(uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		// Jump moves
		new AVP_Move_Predalien_JumpTarget(xMovementSystem);
		new AVP_Move_Predalien_PreJumpTarget(xMovementSystem);

		// Melee moves
		new AVP_Predalien_Move_Melee_ClawLeft_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_ClawLeft_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_ClawLeft_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_ClawLeft_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_ClawRight_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_ClawRight_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_ClawRight_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_ClawRight_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Predalien_Move_Melee_TailLeft(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_TailRight(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Predalien_Move_Melee_WalkClawLeft_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_WalkClawLeft_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_WalkClawLeft_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_WalkClawLeft_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_WalkClawRight_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_WalkClawRight_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_WalkClawRight_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_WalkClawRight_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Predalien_Move_Melee_LeapAttack_HitWall(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_PredatorGrapple_Countered(xMovementSystem);
		new AVP_Predalien_Move_Melee_PredatorGrapple_Kill(xMovementSystem);
		new AVP_Predalient_Move_Melee_PredatorGrapple_Start(xMovementSystem);
		new AVP_Predalien_Move_Melee_LeapAttack_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_LeapAttack_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_LeapAttack_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_LeapAttack_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_LeapAttack_Telegraph(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Predalien_Move_Melee_JumpDownAttack_Return_Miss(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_JumpDownAttack_Return_Countered(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_JumpDownAttack_Return_Hit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_JumpDownAttack_MoveDown(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_Melee_JumpDownAttack_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Predalien_Move_Melee_JumpDownAttack_SuperHit(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		new AVP_Predalien_Move_KnockedDown_Finish(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_KnockedDown_Middle(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);
		new AVP_Predalien_Move_KnockedDown_Start(xMovementSystem, uFirstPersonBlueprintHash, uThirdPersonBlueprintHash);

		return(true);
	}
	else
	{
		return(false);
	}
}

static const Asura_Hash_ID g_uTargetedJumpBP = 0xd2ef4584; //"TargetedJumpSet"
const Asura_Blueprint* AVP_ClientActor_Predalien::GetCurrentTargetedJumpBlueprint() const
{
	return  Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_MOVESPREDALIEN, g_uTargetedJumpBP);
}


void AVP_ClientActor_Predalien::ProcessClientUpdate(const AVP_Message_PredalienClientUpdate* pxData, const float fServerTime)
{
	// Call the base-function...
	PARENT::ProcessClientUpdate( pxData, fServerTime, true );

	// Handle the predator...
	m_eAlertLevel = static_cast<AVP_ClientActor_Vertebrate::AVP_AlertLevel>( pxData->GetAlertLevel() );
}


void AVP_ClientActor_Predalien::GetCurrentMeleeBlueprintIDs(Asura_Hash_ID& uFirstPersonID, Asura_Hash_ID& uThirdPersonID) const
{
	uFirstPersonID = uThirdPersonID = ASURA_HASH_ID_UNSET;

	// As the alien, we just get them from our actor blueprint.
	const Asura_Blueprint* pxBlueprint = GetBlueprint();
	if (!pxBlueprint) { return; }

	// TODO - are we better off storing these values?
	Asura_Value xValue;
	// no first person
// 	if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_MELEE_FIRSTPERSON, xValue))
// 	{
// 		uFirstPersonID = xValue.GetHashIDValue();
// 	}

	if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_MELEE_THIRDPERSON, xValue))
	{
		uThirdPersonID = xValue.GetHashIDValue();
	}
}





/// End of file. ///
