
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientActor_Predalien.h
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_CLIENTACTOR_PREDALIEN_H__
#define __AVP_CLIENTACTOR_PREDALIEN_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Actor/Vertebrates/Predalien/AVP_Predalien_Description.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"

#include "Sound/Asura_Sound_Event_System.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_Description.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Blueprint;
class AVP_Actor_Selector;
class AVP_Message_ActorClientPickup;
class AVP_Message_PredalienClientUpdate;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_ClientActor_Predalien
////////////////////////////////////////////////////////////////////////////////

class AVP_ClientActor_Predalien : public AVP_ClientActor_Vertebrate
{
public:
	// Public functions...
	static	AVP_ClientActor_Predalien*	Create(const Asura_Message* pxMessage);

	// Public Virtual Overridden functions...
	virtual	void Update(void);
	virtual	bool HandleMessage(const Asura_Message* pxMessage);
	virtual void PostCreateInitialise(void);

	// Static list accessors...
	static	AVP_ClientActor_Predalien* FindEntity(const Asura_Guid uGuid) { return(s_xList.FindEntity(uGuid)); }
	static	const Asura_Hashed_List<AVP_ClientActor_Predalien>*	GetList(void) { return(&s_xList); }

	virtual void GetFootPosition( u_int uFootItr, Asura_Vector_3& xFootPos );

	bool GetCanUseTargetedJump(const Asura_Vector_3& xTargetPosition, int& iJumpVariation) const;
	void GetTargetedJumpAnimationsForVariation(const int iJumpVariation, Asura_Hash_ID& uThirdPersonAnim, bool bRunAfterJump) const;


protected:

	float m_fFallingSpeedThreshold;

	union
	{
		u_int m_uPredalienFlags;	// Single access point for the Predalien's flags.

		struct
		{
			u_int m_bDummy					: 1,
											: 0;	// 32 spare bits.
		};
	};

	// Protected functions...
	AVP_ClientActor_Predalien();
	virtual ~AVP_ClientActor_Predalien();

	// Function to handle the client messages...
	void ProcessClientUpdate(const AVP_Message_PredalienClientUpdate* pxData, const float fServerTime);
	
	virtual void CreateAnimationHandler();


	// Overridden initialisation functions...
	virtual	bool InitialiseMoves(void);

	virtual void GetCurrentMeleeBlueprintIDs(Asura_Hash_ID& uFirstPersonID, Asura_Hash_ID& uThirdPersonID) const;

private:
	// Private typedefs...
	typedef AVP_ClientActor_Vertebrate PARENT;

	// Private variables...
					
	static	Asura_Hashed_List<AVP_ClientActor_Predalien> s_xList;

	// Private functions...
	virtual const AVP_Actor_AnimStateData* GetStateDataFromMesssageData( const u_int uState, const AVP_Message_Actor_ClientUpdate* pxActorData );
	virtual const AVP_Actor_AnimStateData* GetStateData( const u_int uState, const u_int uGroundType, const u_int uLimbDamage, const bool bIsAiming, const u_int uCoverType );

	const Asura_Blueprint* GetCurrentTargetedJumpBlueprint() const;

	AVP_ClientActor_Predalien(const AVP_ClientActor_Predalien& xOther);
	AVP_ClientActor_Predalien& operator=(const AVP_ClientActor_Predalien& xOther);
};


#endif   // ifndef __AVP_CLIENTACTOR_PREDALIEN_H__

/// End of file. ///
