///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerActor_Predalien.cpp
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_ServerActor_Predalien.h"

// Core
#include "Core/Asura_BitPacker.h"
#include "Core/Asura_Timers.h"
#include "Core/Axon_System.h"
#include "Core/AVP_Settings.h"
#include "Core/AVP_Game_Core.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Physics/Asura_Physics.h"

// Game independent header-files...
#include "Actor/AVP_Actor_InputHandler.h"
#include "Actor/AVP_Actor_ObjectiveHandler.h"
#include "Actor/AVP_Actor_PickupHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_MeleeCounter_System.h"
#include "Actor/Vertebrates/Human/AVP_Human_MoraleHandler.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_AIAnimStateMachine.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_Description.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_MeleeCounter_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Request.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_Messages_Predalien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_ClientActor_Predalien.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerEntity_Projectile.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_Disc.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/AVP_Movement_System.h"
#include "Moves/Vertebrates/Predalien/AVP_Predalien_MovesMelee.h"

// Axon
#include "Core/BrainTypes/AVP_Brain_Actor_Predalien.h"
#include "Knowledge/AVP_Knowledge_System.h"
#include "Knowledge/Axon_Knowledge.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

// Predalien movement speeds...

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ServerActor_Predalien> AVP_ServerActor_Predalien::s_xList;


AVP_ServerActor_Predalien::AVP_ServerActor_Predalien( const Asura_Guid uGUID )
: Asura_Entity_Server(uGUID)
, PARENT(AVP_PolygonFlag_IgnoreForPredatorCollision, uGUID)
, m_uChunkedPredalienFlags( 0 )
, m_uUnchunkedPredalienFlags( 0 )
, m_uMineHitsCount(0)
, m_uGrappleCount(0)
{
	// Update the data...
	Classification	= AVP_ENTITYCLASS_PREDALIEN;

	m_pxMeleeSystem = new AVP_Predalien_MeleeCounter_System(*this);

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}

AVP_ServerActor_Predalien::~AVP_ServerActor_Predalien()
{
	s_xList.RemoveFromList(this);
}

void AVP_ServerActor_Predalien::KnockDown()
{
	if (GetBrain())
	{
		static_cast<AVP_Brain_Actor_Predalien*>(GetBrain())->KnockDown();
	}
}

bool AVP_ServerActor_Predalien::IsKnockedDown() const
{
	const AVP_ClientActor_Predalien* pxClientActor = AVP_ClientActor_Predalien::FindEntity(GetGuid());
	const Asura_Movement_System* pxMovementSystem = pxClientActor ? pxClientActor->GetMovementSystem() : NULL;
	const Asura_Move* pxCurrentMove = pxMovementSystem ? pxMovementSystem->GetCurrentMove() : NULL;

	if (!pxCurrentMove)
	{
		return false;
	}
	else
	{
		return pxCurrentMove->GetHash() == AVP_Predalien_Move_KnockedDown_Middle::GetStaticHash();
	}
}


void AVP_ServerActor_Predalien::SetGrapple()
{
	++m_uGrappleCount;
}

u_int AVP_ServerActor_Predalien::GetGrappleCount() const
{
	return m_uGrappleCount;
}

void AVP_ServerActor_Predalien::MakeClientUpdateMessage(void)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_PredalienClientUpdate), GetClassification());
	if(pxMessage != NULL)
	{
		AVP_Message_PredalienClientUpdate* pxData = static_cast<AVP_Message_PredalienClientUpdate*>(pxMessage->GetData());
		pxData->FillPredalien(*this);
		SendClientUpdateMessage(pxMessage);
	}
}

void AVP_ServerActor_Predalien::OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	PARENT::OnApplyDamage(xDamage);
}

void AVP_ServerActor_Predalien::OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	PARENT::OnDeath(xDamage);
}

void AVP_ServerActor_Predalien::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Read the version...
	xStream >> uVersion;

	PARENT::ReadFromChunkStream(xStream);

	// Reset the variables...
	m_uChunkedPredalienFlags = 0;

	if (uVersion == 1)
	{
		// Now our data...
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkedPredalienFlags );
		xStream >> xBitfield;
	}
	else
	{
		Asura_Assertf(false, ("AVP_ServerActor_Predalien::ReadFromChunkStream() - Unhandled chunk version (%u)", uVersion));
	}
}

void AVP_ServerActor_Predalien::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// Write the version...
	xStream << uCURRENT_CHUNK_VERSION;

	// Write the parent data...
	PARENT::WriteToChunkStream(xStream);

	Asura_WriteToStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkedPredalienFlags );
	xBitfield.Add( m_bUseTestBehaviour, 1);

	xStream << xBitfield;
}


void AVP_ServerActor_Predalien::MakePlayer(Asura_Guid uPlayerGuid)
{
	return; // no palyer of this type.
}

void AVP_ServerActor_Predalien::MakeAI(void)
{
	PARENT::MakeAI();

	if ( m_pxAIAnimStateMachine == NULL )
	{
		m_pxAIAnimStateMachine = new AVP_Predalien_AIAnimStateMachine( *this );
	}
}

void AVP_ServerActor_Predalien::PostLoadInitialise( void )
{
	if ( !IsPlayer() && !IsADeadBody() && !IsDead() )
	{
		if ( m_pxAIAnimStateMachine == NULL )
		{
			m_pxAIAnimStateMachine = new AVP_Predalien_AIAnimStateMachine( *this );
		}
		m_pxAIAnimStateMachine->Initialise();
	}

	PARENT::PostLoadInitialise();

	m_uMineHitsCount = 0;
	m_uGrappleCount = 0;
}

Axon_Brain* AVP_ServerActor_Predalien::CreateBrain()
{
    return new AVP_Brain_Actor_Predalien( this );
}

DEBUGVAR u_int dbg_uMineHitsToKnockDown = 2;

void AVP_ServerActor_Predalien::TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient )
{
	if (xDamage.GetTypeHash() == uAVP_BLUEPRINT_NAME_DAMAGE_MINE && !m_bCountered)
	{
		++m_uMineHitsCount;
		if (m_uMineHitsCount >= dbg_uMineHitsToKnockDown)
		{
			m_uMineHitsCount = 0;
			KnockDown();
		}												  
	}
	else if (xDamage.GetTypeHash() == uAVP_BLUEPRINT_NAME_DAMAGE_GRAPPLE || bForceFatal)
	{
		PARENT::TakeDamageExtended(xDamage, bForceFatal, bSendMessageToClient);
		return;
	}

	// non grapple damage take zero damage...
	Asura_Vector_3 xPointOfImpact;
	Asura_Vector_3 xDirection;
	Asura_Vector_3 xNormal;
	xDamage.GetPointOfImpact(xPointOfImpact);
	xDamage.GetDirection(xDirection);
	xDamage.GetNormal(xNormal);
	AVP_EntityDamage_RequestVictim xNewDamage(xDamage.GetTypeHash(),
		xDamage.GetFromGuid(),
		xDamage.GetFromMarkerPointHash(),
		xDamage.GetEntityToDamage(), 
		0.0f,
		xPointOfImpact, 
		xDirection,
		xNormal,
		xDamage.GetImpulseStrength(),
		xDamage.GetBoneID(),
		xDamage.GetBlocked(),
		xDamage.GetParentGuid(),
		xDamage.GetAttachmentID(),
		xDamage.GetMaterialResponseHashID(),
		xDamage.GetStealth(), 
		xDamage.GetDoNotFlinch());
	PARENT::TakeDamageExtended(xNewDamage, bForceFatal, bSendMessageToClient);
}


bool AVP_ServerActor_Predalien::HandleMessage(const Asura_Message* pxMessage)
{

	switch (pxMessage->GetMsgID())
	{
	case AVP_MSGID_COUNTER_MELEE:
		m_bCountered = true;
		break;
	case ASURA_MSG_MOVECHAINCOMPLETE:
		m_bCountered = false;
	}

	return PARENT::HandleMessage(pxMessage);
	
}



Asura_Hash_ID AVP_ServerActor_Predalien::GetMovesBlueprintType() const
{
	return uAVP_BLUEPRINT_TYPE_MOVESPREDALIEN;
}

/// End of file. ///
