///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Messages_Predator.h
// Author : Andy Tate
// Date   : 4-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_MESSAGES_PREDATOR_H__
#define __AVP_MESSAGES_PREDATOR_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Maths.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"

#include "Actor/Vertebrates/Predator/AVP_Predator_JumpHandler.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_Vertebrate_Messages.h"
//#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_NetMsg;
class Asura_BitPacker; 
class Asura_BitUnpacker; 
class AVP_ClientActor_Predator;
class AVP_ServerActor_Predator;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Message_PredatorPlayerCommands
// Author       : Andy Tate
// Date         : 11-Oct-2006
//
// Description  : Message to handle the passing of player-Predator data from the
//				  client to the server.
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_PredatorPlayerCommands : public AVP_Message_VertebratePlayerCommands
{
public:
	// Contructor
	AVP_Message_PredatorPlayerCommands( const AVP_ClientActor_Predator& xPredator ); 
	AVP_Message_PredatorPlayerCommands( Asura_BitUnpacker& xUnpacker ); 

	void FillPredator( const AVP_ClientActor_Predator& xPred );

	// Accessor functions...
	bool IsCrouching() const			{ return m_bCrouching; }
	bool IsRequestingChargeUp() const	{ return m_bRequestingChargeUp; }
	bool IsLockOn() const				{ return m_bLockOn; }
	bool IsRequestingDistract() const	{ return m_bDistract; }
	
	Asura_Guid	GetTargetGuid() const	{ return m_uTargetGuid; }
	
	// Compression 
	void Pack( Asura_BitPacker& xPacker ) const; 
	void Unpack( Asura_BitUnpacker& xUnpacker ); 

protected:
	AVP_Message_PredatorPlayerCommands() {}

private:
	typedef AVP_Message_VertebratePlayerCommands PARENT;

	struct
	{
		bool 			m_bCrouching;
		bool			m_bRequestingChargeUp;
		bool			m_bLockOn; 
		Asura_Guid		m_uTargetGuid; 
		bool			m_bDistract;
	};

	// Private functions...
	AVP_Message_PredatorPlayerCommands( const AVP_Message_PredatorPlayerCommands& xOther );
	AVP_Message_PredatorPlayerCommands& operator=( const AVP_Message_PredatorPlayerCommands& xOther );

};   // End of class 'AVP_Message_PredatorPlayerCommands'.



////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Message_PredatorClientUpdate
// Author       : Andy Tate
// Date         : 6-Oct-2006
//
// Description  : Message to handle updating the Predators client-entity.
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_PredatorClientUpdate : public AVP_Message_Vertebrate_ClientUpdate
{
public:
	void FillPredator( const AVP_ServerActor_Predator& xPredator );

	// Accessor functions...
	Asura_Guid	GetDiscGuid(void) const					{ return m_uDiscGuid; }
	Asura_Guid	GetHeldProjectileGuid(void) const		{ return m_uHeldProjectileGuid; }
	Asura_Guid	GetTargetGuid(void) const				{ return m_uTargetGuid; }
	bool		IsCrouching() const						{ return m_bCrouching; }
	bool        IsInPounceRange() const					{ return ( m_bInsidePounceRange || m_bLockOn )? true : false; }
	bool		IsLockOn() const						{ return m_bLockOn; }
	u_int		GetAlertLevel() const					{ return m_uAlertLevel; }

#ifdef ASURA_SUPPORT_NETMSG_SYSTEM
	static void RegisterFields(u_short usMsgID, u_short usEntityClassification);
	static void RegisterFields(Asura_NetMsg* pxNetMsg);
	static u_char GetNumFields() { return static_cast<u_char>(6 + AVP_Message_Vertebrate_ClientUpdate::GetNumFields()); }
#endif//#ifdef ASURA_SUPPORT_NETMSG_SYSTEM

protected:
	AVP_Message_PredatorClientUpdate() {}

private:
	// Private variables...
	Asura_Guid	m_uDiscGuid;				// The GUID of this Predator's disc.
	Asura_Guid	m_uHeldProjectileGuid;		// The GUID of the currently held projectile.
	Asura_Guid	m_uTargetGuid; 
	bool		m_bCrouching;
	bool		m_bInsidePounceRange;
	bool		m_bLockOn;
	u_int		m_uAlertLevel;

	// Private functions...
//	AVP_Message_PredatorClientUpdate  (const AVP_Message_PredatorClientUpdate& xOther);

//	AVP_Message_PredatorClientUpdate&	operator=	(const AVP_Message_PredatorClientUpdate& xOther);

};   // End of class 'AVP_Message_PredatorClientUpdate'.

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Message_Predator_Move_JumpStart
// Author       : Anthony Gregson
// Date         : 08/10/07
//
// Description  : Class to send jump-to-target start move data.
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_Predator_Move_JumpStart : public Asura_Msg_Move
{
public:

	void SetJumpData(const Asura_Vector_3& xDestinationPosition, const AVP_ClientActor_Vertebrate::JumpLandType eJumpLandType, int iJumpVariation, float fTurnTime = 0.1f )
	{
		m_xDestinationPosition = xDestinationPosition;
		m_iJumpVariation = iJumpVariation;
		m_fTurnTime = fTurnTime;
		m_eJumpLandType = static_cast< u_char >( eJumpLandType );
	}

	const Asura_Vector_3& GetDestinationPosition() const { return m_xDestinationPosition; }
	int GetJumpVariation() const { return m_iJumpVariation; }
	float GetTurnTime() const { return m_fTurnTime; }
	AVP_ClientActor_Vertebrate::JumpLandType GetJumpLandType() const { return static_cast< AVP_ClientActor_Vertebrate::JumpLandType >( m_eJumpLandType ); }

private:
	// Private variables...
	Asura_Vector_3	m_xDestinationPosition;
	int	m_iJumpVariation;
	float m_fTurnTime;
	u_char m_eJumpLandType;

};   // End of class 'AVP_Message_Alien_Move_JumpSurfaceStart'.

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Message_Predator_Move_MoveAndJumpStart
// Description  : Class to send move-to-position-and-jump-to-target start move data.
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_Predator_Move_WalkAndJumpStart : public AVP_Message_Predator_Move_JumpStart
{
public:


	void SetJumpData(const Asura_Vector_3& xStartPosition, const Asura_Vector_3& xDestinationPosition, const AVP_ClientActor_Vertebrate::JumpLandType eJumpLandType, int iJumpVariation, float fTurnTimeInJump , float fMoveSpeed, float fTurnSpeed )
	{
		AVP_Message_Predator_Move_JumpStart::SetJumpData(xDestinationPosition, eJumpLandType, iJumpVariation, fTurnTimeInJump);
		m_xStartPosition = xStartPosition;
		m_fMoveSpeed = fMoveSpeed;
		m_fTurnSpeed = fTurnSpeed;

	}

	const Asura_Vector_3& GetStartPosition() const { return m_xStartPosition; }
	float GetMoveSpeed() const { return m_fMoveSpeed; }
	float GetTurnSpeed() const { return m_fTurnSpeed; }

private:
	// Private variables...
	Asura_Vector_3	m_xStartPosition;
	float m_fMoveSpeed;
	float m_fTurnSpeed;

};   // End of class 'AVP_Message_Alien_Move_JumpSurfaceStart'.




class AVP_Message_Predator_Move_MountBranch_Start : public Asura_Msg_Move
{
public:
	// Public functions...
	void	SetDestinationPosition(const Asura_Vector_3& xPosition)	{ m_xDestinationPosition = xPosition; }
	void	GetDestinationPosition(Asura_Vector_3& xPosition) const	{ xPosition = m_xDestinationPosition; }

protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...
	Asura_Vector_3	m_xDestinationPosition;

	// Private functions...
	AVP_Message_Predator_Move_MountBranch_Start(void);
	AVP_Message_Predator_Move_MountBranch_Start(const AVP_Message_Predator_Move_MountBranch_Start& xOther);
	~AVP_Message_Predator_Move_MountBranch_Start();

	AVP_Message_Predator_Move_MountBranch_Start& operator=(const AVP_Message_Predator_Move_MountBranch_Start& xOther);

};   // End of class 'AVP_Message_Predator_Move_MountBranch_Start'.


class AVP_Message_Predator_Move_DismountBranch_Start : public Asura_Msg_Move
{
public:
	// Public functions...
	void	SetDirection(const Asura_Vector_3& xDirection)	{ m_xDirection = xDirection; }
	void	GetDirection(Asura_Vector_3& xDirection) const	{ xDirection = m_xDirection; }

protected:
	// Protected variables...

	// Protected functions...

private:
	// Private variables...
	Asura_Vector_3	m_xDirection;
};   // End of class 'AVP_Message_Predator_Move_DismountBranch_Start'.


//////////////////////////////////////////////////////////////////////////////
//
// AVP_Message_SetPredatorBehaviour
//
//////////////////////////////////////////////////////////////////////////////

struct AVP_PredatorBehaviourData
{
	// Don't change the order of these - they are saved out to chunks.
	enum Behaviour
	{
		STALK,
		MELEE,
		NUM_BEHAVIOURS
	};

	int m_iBehaviour;

	AVP_PredatorBehaviourData()
		: m_iBehaviour( STALK )
	{
		SetParamsFromBehaviour();
	}
	void SetParamsFromBehaviour();
	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

#ifdef ASURA_TOOLS
	static const char* s_apszBehaviourStrings[ NUM_BEHAVIOURS ];
#endif

private:
	static const u_int uCURRENT_CHUNK_VERSION = 1;
};

struct AVP_Message_SetPredatorBehaviour
{
	u_int m_uVersion;
	int m_iBehaviour;

	static const u_int uCURRENT_VERSION = 1;
};



#endif   // ifndef __AVP_MESSAGES_PREDATOR_H__


/// End of file. ///
