///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerActor_Predator.cpp
// Author : Andy Tate
// Date   : 3-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"

// Core
#include "Core/Asura_BitPacker.h"
#include "Core/Asura_Timers.h"
#include "Core/Axon_System.h"
#include "Core/AVP_Settings.h"
#include "Core/AVP_Game_Core.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Physics/Asura_Physics.h"

// Game independent header-files...
#include "Actor/AVP_Actor_InputHandler.h"
#include "Actor/AVP_Actor_InteractionHandler.h"
#include "Actor/AVP_Actor_ObjectiveHandler.h"
#include "Actor/AVP_Actor_PickupHandler.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_MeleeCounter_System.h"
#include "Actor/Vertebrates/Human/AVP_Human_MoraleHandler.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_AIAnimStateMachine.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_Description.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_MeleeCounter_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Request.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Entity/Damage/AVP_EntityDamage_Apply.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_Messages_Predator.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ClientActor_Predator.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_Sentrygun_Messages.h"
#include "Entity/PhysicalObject/Interact/InvisiSwitch/AVP_ClientEntity_InteractInvisiSwitch.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerEntity_Projectile.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_Disc.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_PredatorCloak.h"
#include "Inventory/Items/AVP_InventoryItem_Stim.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_Disc.h"
#include "Mesh/AVP_PolygonFlags.h"
#include "Moves/AVP_Move_Interaction.h"
#include "Moves/AVP_Movement_System.h"
#include "Moves/Vertebrates/Predator/AVP_Predator_MovesMelee.h"

// Axon
#include "Core/BrainTypes/AVP_Brain_Actor_Predator.h"
#include "Knowledge/AVP_Knowledge_System.h"
#include "Knowledge/Axon_Knowledge.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

// Predator movement speeds...

DEBUGVAR Asura_Hash_ID dbg_uPlasmaCasterHash = uAVP_INVENTORY_WEAPONHASH_WEAPON01;	// Weapon01
DEBUGVAR Asura_Hash_ID dbg_uWristBladesHash = dbg_uPlasmaCasterHash + 1;				// Weapon02


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ServerActor_Predator>	AVP_ServerActor_Predator::s_xList;


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Predator::AVP_ServerActor_Predator
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_ServerActor_Predator::AVP_ServerActor_Predator( const Asura_Guid uGUID )
: Asura_Entity_Server(uGUID)
, PARENT(AVP_PolygonFlag_IgnoreForPredatorCollision, uGUID)
, m_uDiscGuid( ASURA_GUID_UNREGISTERED )
, m_uHeldProjectileGuid(ASURA_GUID_UNREGISTERED)
, m_uTargetGuid( ASURA_GUID_UNREGISTERED )
, m_bLockOn( false )
, m_fPredatorEnergy( 100.0f )
, m_bDesiredCloakingStatus(false)
, m_bDecloakManually(false)
, m_eDesiredWeaponType(WEAPON_PLASMACASTER)
, m_fPlasmaCasterDesiredCharge(0.0f)
, m_ePlasmacasterUsageState(PLASMACASTER_IDLE)
, m_xMineList()
, m_xBehaviourData()
, m_uChunkedPredatorFlags( 0 )
{
	// Update the data...
	Classification	= AVP_ENTITYCLASS_PREDATOR;

	m_pxMeleeSystem = new AVP_Predator_MeleeCounter_System(*this);

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}

/// End of function 'AVP_ServerActor_Predator::AVP_ServerActor_Predator'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Predator::~AVP_ServerActor_Predator
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

AVP_ServerActor_Predator::~AVP_ServerActor_Predator()
{
	// Don't forget to remove us from the list...
	s_xList.RemoveFromList(this);
}

/// End of function 'AVP_ServerActor_Predator::~AVP_ServerActor_Predator'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Predator::MakeClientUpdateMessage
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Predator::MakeClientUpdateMessage(void)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_PredatorClientUpdate), GetClassification());
	if(pxMessage != NULL)
	{
		AVP_Message_PredatorClientUpdate* pxData = static_cast<AVP_Message_PredatorClientUpdate*>(pxMessage->GetData());
		pxData->FillPredator(*this);
		SendClientUpdateMessage(pxMessage);
	}
}

/// End of function 'AVP_ServerActor_Predator::MakeClientUpdateMessage'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Predator::Update
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Predator::Update(void)
{
	PARENT::Update();

	if(!IsPlayer())
	{
		UpdateCloakingAndWeaponStatus();
	}

	// Horrible test, grab data from client here
	AVP_ClientActor_Predator* pxClient = AVP_ClientActor_Predator::FindEntity(GetGuid());
	if(pxClient)
	{
		m_fPredatorEnergy = pxClient->GetDynamicEnergy()->GetCurrentValue();
	}
}

/// End of function 'AVP_ServerActor_Predator::Update'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Predator::OnApplyDamage
// Author       : Andy Tate
// Date         : 26-Feb-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerActor_Predator::OnApplyDamage(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	PARENT::OnApplyDamage(xDamage);

	if ( xDamage.GetAmount() > 0.0f )
	{
		//Axon_System::GetSoundRegistry()->RegisterSound( GetGuid(), GetClassification(), GetPosition(), 20.0f, AVP_SOUND_TYPE_PRED_ROAR );

// 		AVP_InventoryItem_Cloak* pxCloak = static_cast< AVP_InventoryItem_Cloak* >( GetInventorySystem()->GetItem( uAVP_INVENTORY_ITEMHASH_CLOAK ) );
// 		if ( pxCloak && pxCloak->IsUsing( false ) )
// 		{
// 			pxCloak->RequestEndUsage( AVP_InventoryItem_Cloak::TT_AUTOMATIC );
// 		}
	}
}
/// End of function 'AVP_ServerActor_Predator::OnApplyDamage'. ///

//-----------------------------------------------------------------------------
// Function     : AVP_ServerActor_Predator::OnDeath
// Author       : Gareth Thomas
// Date         : 08 / 02 / 2008
//-----------------------------------------------------------------------------
void AVP_ServerActor_Predator::OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage)
{
	PARENT::OnDeath(xDamage);

	Decloak(false);

	if( IsHoldingPlasmaCaster() && IsPlasmaCasterCharged() )
	{
		// if the predator is now dead, but his plasma caster was charged at the time of death abort the charge. 
		// fix to issue where the charge PRX were still playing after predator death. 
		AbortPlasmaCasterCharge();
	}

	AVP_ServerEntity_Actor* pxKillingActor = AVP_ServerEntity_Actor::FindEntity(xDamage.GetFromGuid());

	if (pxKillingActor)
	{
		if ((pxKillingActor->GetClassification() == AVP_ENTITYCLASS_MARINE) && (!pxKillingActor->IsPlayer()))
		{
			EffectMorale(xDamage.GetFromGuid());
		}
	}
}

//-----------------------------------------------------------------
// AVP_ServerActor_Predator :: EffectMorale
//-----------------------------------------------------------------
void AVP_ServerActor_Predator::EffectMorale(Asura_Guid uGuidToEffect)
{
	AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity(uGuidToEffect);

	if (pxHuman)
	{
		if (pxHuman->GetInitialSquadGuid() != ASURA_GUID_UNREGISTERED)
		{
			AVP_ServerEntity_Squad_Marine* pxMarineSquad = AVP_ServerEntity_Squad_Marine::FindEntity(pxHuman->GetInitialSquadGuid());

			if (pxMarineSquad)
			{
				if (!pxMarineSquad->IsPlayersSquad())
				{
					if (pxHuman->GetMoraleHandler())
					{
						pxHuman->GetMoraleHandler()->EffectMorale(AVP_Human_MoraleHandler::AVP_MORALE_EFFECTOR_KILLED_PREDATOR);
						pxMarineSquad->EffectSquadMorale(AVP_Human_MoraleHandler::AVP_MORALE_EFFECTOR_SQUAD_MEMBER_KILLED_PREDATOR, pxHuman->GetGuid());
					}
				}
			}
		}
		else
		{
			if (!pxHuman->IsPlayer())
			{
				if (pxHuman->GetMoraleHandler())
				{
					pxHuman->GetMoraleHandler()->EffectMorale(AVP_Human_MoraleHandler::AVP_MORALE_EFFECTOR_KILLED_PREDATOR);
				}
			}
		}
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Predator::HandleMessage
// Author       : Andy Tate
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerActor_Predator::HandleMessage(const Asura_Message* pxMessage)
{
	// What sort of message is it?
	switch (pxMessage->GetMsgID())
	{
		case AVP_MSGID_PREDATORPLAYERCOMMANDS_COMPRESSED:
		{
			// Get raw compressed data
			const u_char* pucData = static_cast<const u_char*>( pxMessage->GetData() ); 
			u_int uLen = pxMessage->GetDataSize(); 
			
			// Uncompress the data
			if (pucData && uLen)
			{
				Asura_BitUnpacker xUnpacker( pucData, uLen ); 
				AVP_Message_PredatorPlayerCommands xPredatorPlayerCommands( xUnpacker ); 

				ProcessPredatorPlayerCommands( &xPredatorPlayerCommands, pxMessage->GetSentTime() ); 
			}

			return true; 
		}

		case AVP_MSGID_PREDATORPLAYERCOMMANDS:
		{
			const AVP_Message_PredatorPlayerCommands*	pxData	= static_cast<const AVP_Message_PredatorPlayerCommands*>(pxMessage->GetData());

			if (pxData != NULL)
			{
				ProcessPredatorPlayerCommands( pxData, pxMessage->GetSentTime() );
			}

			return(true);
		}

		case AVP_MSGID_DISC_LASER_TRACKING_OFF:	// FALL THROUGHT!!!
		case AVP_MSGID_DISC_LASER_TRACKING_ON:
		{
			// Change behavior of predator's disc: 
			AVP_InventoryItem_Weapon* pxCurrentWeapon = m_pxInventorySystem ? static_cast<AVP_InventoryItem_Weapon*>(m_pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON)) : NULL;
			if ( pxCurrentWeapon ) 
			{ 
				// Just make sure that it is disc;
				if( pxCurrentWeapon->GetItemType()==AVP_INVENTORYITEMTYPE_PREDATORDISC )
				{
					AVP_InventoryWeapon_Disc* pxDisc = static_cast<AVP_InventoryWeapon_Disc*>(pxCurrentWeapon);

					bool bLaserTrackingModeOn = pxMessage->GetMsgID() == AVP_MSGID_DISC_LASER_TRACKING_ON ? true : false;

					pxDisc->SetLaserTargetMode( bLaserTrackingModeOn );
				}

			}

			// Make sure that this message is also handled in client side. So we can turn on or off the laser for the predator.
			Asura_Network::Forward(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, Guid);
			return true;
		}
		
		case AVP_MSGID_PICKUP_DISC:
			Asura_Network::Forward(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, Guid);
			return true;

        case AVP_MSGID_REQUEST_CLOAK_ON:
		{
			if ( IsPlayer() )
			{
				Asura_Network::Forward(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid());
			}
			else
			{
				Cloak();
			}
			return true;
		}

        case AVP_MSGID_REQUEST_CLOAK_OFF:
		{
			if ( IsPlayer() )
			{
				Asura_Network::Forward(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid());
			}
			else
			{
				Decloak();
			}
		}
		return true;
        
		case AVP_MSGID_REQUEST_INTERACTION:
		{
			const AVP_Message_Actor_RequestInteraction* pxData = static_cast<const AVP_Message_Actor_RequestInteraction*>(pxMessage->GetData());

			// The only interaction we can currently do anything with is a headbite, and even then only if we're dead and they're an alien (although, if they're not, why are they requesting a headbite?)
			// TODO - should the early-outs return true, or return parent handling? Probably parent handling, as some generic interactions may be created.
			if (pxData->GetInteractionType() != AVP_INTERACTION_TYPE_HEADBITE) { return PARENT::HandleMessage(pxMessage); }
			if (!IsDead()) { return PARENT::HandleMessage(pxMessage); }

			const AVP_ServerActor_Alien* pxBiter = AVP_ServerActor_Alien::FindEntity(pxMessage->From);
			if (!pxBiter) { return PARENT::HandleMessage(pxMessage); }

			// OK, we're dead, they're an alien, and at the moment that's all we can check. Tell them to bite our head off.
			// TODO - need to add some way of knowing if the head is still present on the server (currently client-side only).
			Asura_Message* pxNewMessage = Asura_Network::CreateMessage(AVP_MSGID_REQUEST_MELEE, sizeof(AVP_Message_Vertebrate_RequestMelee));
			if (pxNewMessage)
			{
				AVP_Message_Vertebrate_RequestMelee* pxNewData = static_cast<AVP_Message_Vertebrate_RequestMelee*>(pxNewMessage->GetData());
				pxNewData->SetType(AVP_ALIEN_MELEEATTACK_TYPE_HEADBITE);
				pxNewData->SetTargetGuid(Guid);

				Send(pxNewMessage, pxMessage->From);
			}

			return true;
		}

		case AVP_MSGID_SET_PREDATOR_BEHAVIOUR:
		{
			const AVP_Message_SetPredatorBehaviour* pxData = static_cast< const AVP_Message_SetPredatorBehaviour* >( pxMessage->GetData() );
			m_xBehaviourData.m_iBehaviour = pxData->m_iBehaviour;			
			// no other data currently
			return true;
		}

		case AVP_MSGID_REPLENISH_PREDATOR_ENERGY:
		{
			Asura_Network::Forward(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, Guid);
			return true;
		}

		case AVP_MSGID_ATTACK_SCRIPTED_TARGET:
		case AVP_MSGID_STOP_ATTACKING_SCRIPTED_TARGET:
			if (GetActorBrain())
			{
				GetActorBrain()->DoHandleMessage(pxMessage);
				return true;
			}
			else
			{
				return(PARENT::HandleMessage(pxMessage));
			}

        default:
		{
			return(PARENT::HandleMessage(pxMessage));
		}
	}
}

/// End of function 'AVP_ServerActor_Predator::HandleMessage'. ///

void AVP_ServerActor_Predator::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// Read the version...
	xStream >> uVersion;

	// Reset the variables...
	m_uChunkedPredatorFlags	= 0;

	// Read the appropriate data...
	switch(uVersion)
	{
		case 0:
		{
			// The vertebrate didn't have chunk data originally, so direct subclass chunk versions have been bumped to work around it.
			AVP_ServerEntity_Actor::ReadFromChunkStream(xStream);
			break;
		}

		case 1:
		{
			// Read the parent data...
			PARENT::ReadFromChunkStream(xStream);
			break;
		}

		case 2:
		{
			int	iStartsCloaked;

			// Read the parent data...
			PARENT::ReadFromChunkStream(xStream);

			// Now our data...
			xStream >> iStartsCloaked;
			m_bStartsCloaked	= (iStartsCloaked > 0);
			break;
		}

		case 3:
		{
			// Read the parent data...
			PARENT::ReadFromChunkStream(xStream);

			// Now our data...
			Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkedPredatorFlags );
			xStream >> xBitfield;
			break;
		}

		case 4:
		{
			// Read the parent data...
			PARENT::ReadFromChunkStream(xStream);

			// Now our data...
			Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkedPredatorFlags );
			xStream >> xBitfield;
			m_xBehaviourData.ReadFromChunkStream(xStream);
			break;
		}

		case 5:
		{
			// Read the parent data...
			PARENT::ReadFromChunkStream(xStream);

			// Now our data...
			Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkedPredatorFlags );
			xStream >> xBitfield;
			m_xBehaviourData.ReadFromChunkStream(xStream);

			u_int uMineListSize;
			xStream >> uMineListSize;
			
			for(u_int u=0;u<uMineListSize;u++)
			{
				Asura_Guid uMineGuid;
				xStream >> uMineGuid;
				m_xMineList.Push_Back(uMineGuid);
			}

			break;
		}

		case 6:
		{
			// Read the parent data...
			PARENT::ReadFromChunkStream(xStream);

			xStream >> m_fPredatorEnergy;
	
			// Now our data...
			Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkedPredatorFlags );
			xStream >> xBitfield;
			m_xBehaviourData.ReadFromChunkStream(xStream);

			u_int uMineListSize;
			xStream >> uMineListSize;

			for(u_int u=0;u<uMineListSize;u++)
			{
				Asura_Guid uMineGuid;
				xStream >> uMineGuid;
				m_xMineList.Push_Back(uMineGuid);
			}

			break;
		}

		default:
		{
			Asura_Assertf(false, ("AVP_ServerActor_Predator::ReadFromChunkStream() - Unhandled chunk version (%u)", uVersion));
			break;
		}
	}		// End of switch.
}

void AVP_ServerActor_Predator::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// Write the version...
	xStream << uCURRENT_CHUNK_VERSION;

	// Write the parent data...
	PARENT::WriteToChunkStream(xStream);

	xStream << m_fPredatorEnergy;

	Asura_WriteToStream_Bitfield_Wrapper< u_int > xBitfield( m_uChunkedPredatorFlags );

	xBitfield.Add( m_bCrouching, 1);
	xBitfield.Add( m_bStartsCloaked, 1);
	xBitfield.Add( m_bUseTestBehaviour, 1);

	xStream << xBitfield;

	m_xBehaviourData.WriteToChunkStream(xStream);

	xStream << m_xMineList.Size();
	Asura_ListElt<Asura_Guid>* pxCur = GetMineGuidList()->Front();
	while(pxCur)
	{
		xStream << pxCur->Data();
		pxCur = pxCur->Next(); // Get next while exists
	}
}


void AVP_ServerActor_Predator::WriteToClientCreateStream(Asura_Chunk_Memory_Stream& xStream) const
{
	xStream << static_cast<u_int>(CURRENT_CLIENTCREATE_VERSION);

	PARENT::WriteToClientCreateStream(xStream);

	xStream << m_fPredatorEnergy;
}	

void AVP_ServerActor_Predator::Cloak()
{
	if (IsPlayer())
	{
		if (!IsDead())
		{
			if (m_pxInventorySystem)
			{
				AVP_Inventory_Item* pxCloak = m_pxInventorySystem->GetItem(uAVP_INVENTORY_ITEMHASH_CLOAK);
				if (pxCloak && pxCloak->IsIdling(false))
				{
					pxCloak->RequestBeginUsage(AVP_InventoryItem_Cloak::UT_CLOAK, AVP_InventoryItem_Cloak::TT_MANUAL);
				}
			}
		}
	}
	else
	{
		if ( !IsDead() )
		{
			m_bDesiredCloakingStatus = true;
		}
		
		EquipWristBlades(); // no point to cloak if we going to keep that laser showing where we are 
	}
}

void AVP_ServerActor_Predator::Decloak(bool bManually)
{
	if (IsPlayer())
	{
		if (m_pxInventorySystem)
		{
			AVP_Inventory_Item* pxCloak = m_pxInventorySystem->GetItem(uAVP_INVENTORY_ITEMHASH_CLOAK);
			if (pxCloak && pxCloak->IsUsing(false))
			{
				pxCloak->RequestEndUsage(bManually ? AVP_InventoryItem_Cloak::TT_MANUAL : AVP_InventoryItem_Cloak::TT_AUTOMATIC);
			}
		}
	}
	else
	{
		m_bDesiredCloakingStatus = false;
		m_bDecloakManually = bManually;
	}
}

bool AVP_ServerActor_Predator::IsCloaked() const
{
	const AVP_Inventory_System* pxInventorySystem = GetInventorySystem();
	const AVP_Inventory_Item*	pxCloakItem	= (pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetItem(uAVP_INVENTORY_ITEMHASH_CLOAK);

	Asura_Assertf(pxCloakItem, ( "Cloak not found for predator %d", GetGuid() ) );
	if(pxCloakItem)
	{
		return pxCloakItem->IsUsing(true);
	}
	else
	{
		return false;
	}
}


void AVP_ServerActor_Predator::EquipPlasmaCaster()
{
	if (m_eDesiredWeaponType != WEAPON_PLASMACASTER)
	{
		m_eDesiredWeaponType = WEAPON_PLASMACASTER;
	}
}

void AVP_ServerActor_Predator::EquipWristBlades()
{
	if (m_eDesiredWeaponType != WEAPON_WRISTBLADES)
	{
		m_eDesiredWeaponType = WEAPON_WRISTBLADES;
	}
}

bool AVP_ServerActor_Predator::IsHoldingPlasmaCaster() const
{
	const AVP_Inventory_System* pxInventorySystem = GetInventorySystem();
	const AVP_Inventory_Group*  pxWeaponGroup		= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assertf(pxWeaponGroup, ( "Weapon group not found for predator %d", GetGuid() ) );

	return pxWeaponGroup ? pxWeaponGroup->GetCurrentItem() == dbg_uPlasmaCasterHash && pxWeaponGroup->IsCurrentItemActive() : false ;
}

bool AVP_ServerActor_Predator::IsHoldingWristBlades() const
{
	const AVP_Inventory_System* pxInventorySystem = GetInventorySystem();
	const AVP_Inventory_Group*  pxWeaponGroup		= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assertf(pxWeaponGroup, ( "Weapon group not found for predator %d", GetGuid() ) );

	return pxWeaponGroup ? pxWeaponGroup->GetCurrentItem() == dbg_uWristBladesHash && pxWeaponGroup->IsCurrentItemActive() : false;
}


bool AVP_ServerActor_Predator::StartPlasmaCasterCharge(float fDamage)
{
	Decloak(); // this will happened automatically so need to update the desired status - since if the predator wish to fire it also wish to decloak.

	m_fPlasmaCasterDesiredCharge = 0.0f;
	if (m_ePlasmacasterUsageState == PLASMACASTER_CHARGING)
	{
		return true;
	}

	AVP_Inventory_System* pxInventorySystem	= GetInventorySystem();
	AVP_Inventory_Group*  pxWeaponGroup	= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assertf(pxWeaponGroup, ( "Weapon group not found for predator %d", GetGuid() ) );
	if (!pxWeaponGroup) { return true; }

	Asura_Assert(pxWeaponGroup->GetCurrentItem() == dbg_uPlasmaCasterHash, "Trying to charge plasmacaster but not holding plasmacaster");

	AVP_InventoryWeapon_RangedWeapon* pxWeaponItem	= 
		static_cast<AVP_InventoryWeapon_RangedWeapon*>((pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetItem(dbg_uPlasmaCasterHash));
	Asura_Assert(pxWeaponItem, "Weapon not found");
	if (!pxWeaponItem) { return true; }

	if (!pxWeaponItem->IsIdling(true))
	{
		return false;
	}

	if (!pxWeaponItem->RequestBeginUsage(AVP_InventoryWeapon_RangedWeapon::UT_CHARGED_SHOT_PRIMARY))
	{
		return false;
	}

	m_ePlasmacasterUsageState = PLASMACASTER_CHARGING;

	const AVP_RangedWeapon_SubType* pxWeaponSubItem = static_cast<const AVP_RangedWeapon_SubType*>(pxWeaponItem->GetSubType(AVP_InventoryWeapon_RangedWeapon::ST_PRIMARY));
	Asura_Assert(pxWeaponSubItem, "Subitem not found");
	if(!pxWeaponSubItem) { return true; }
	const AVP_GenericProjectileData* pxProjectile = AVP_GenericProjectileData::GetDataByHash(pxWeaponSubItem->GetProjectile());
	Asura_Assert(pxProjectile, "Projectile not found");
	if(!pxProjectile) { return true; }
	// calculate how much charge is needed for the desired damage.
	m_fPlasmaCasterDesiredCharge = fDamage / (pxWeaponSubItem->GetDamageMultiplierPerCharge() * pxProjectile->GetRadialDamageAtNearRange());

	return true;
}

void AVP_ServerActor_Predator::ReleasePlasmaCasterCharge()
{
	Asura_Assert(m_ePlasmacasterUsageState == PLASMACASTER_CHARGING, "Release plasmacaster charge without charging firts");
	m_fPlasmaCasterDesiredCharge = 0.0f;

	AVP_Inventory_System* pxInventorySystem	= GetInventorySystem();
	AVP_Inventory_Group*  pxWeaponGroup	= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assert(pxWeaponGroup, "Weapon group not found");
	if (!pxWeaponGroup) { return; }
	Asura_Assert(pxWeaponGroup->GetCurrentItem() == dbg_uPlasmaCasterHash, "Trying to release plasmacaster but not holding plasmacaster");

	AVP_InventoryWeapon_RangedWeapon* pxWeaponItem	= 
		static_cast<AVP_InventoryWeapon_RangedWeapon*>((pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetItem(dbg_uPlasmaCasterHash));
	Asura_Assert(pxWeaponItem, "Weapon not found");

	if (!pxWeaponItem) { return; }

	if (pxWeaponItem->IsUsing(true))
	{
		pxWeaponItem->RequestEndUsage(AVP_InventoryWeapon_RangedWeapon::TT_SHOOT);
		m_ePlasmacasterUsageState = PLASMACASTER_IDLE;
	}
	else
	{
		m_ePlasmacasterUsageState = PLASMACASTER_REQUIRE_RELEASING;
	}
}

void AVP_ServerActor_Predator::AbortPlasmaCasterCharge()
{
	if (m_ePlasmacasterUsageState != PLASMACASTER_CHARGING)
	{
		return;
	}

	m_fPlasmaCasterDesiredCharge = 0.0f;
	AVP_Inventory_System* pxInventorySystem	= GetInventorySystem();
	AVP_Inventory_Group*  pxWeaponGroup	= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assert(pxWeaponGroup, "Weapon group not found");
	if (!pxWeaponGroup) { return; }
	Asura_Assert(pxWeaponGroup->GetCurrentItem() == dbg_uPlasmaCasterHash, "Trying to abort plasmacaster but not holding plasmacaster");

	AVP_InventoryWeapon_RangedWeapon* pxWeaponItem	= 
		static_cast<AVP_InventoryWeapon_RangedWeapon*>((pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetItem(dbg_uPlasmaCasterHash));
	Asura_Assert(pxWeaponItem, "Weapon not found");

	if (!pxWeaponItem) { return; }


	if (pxWeaponItem->IsUsing(true))
	{
		pxWeaponItem->RequestEndUsage(AVP_InventoryWeapon_RangedWeapon::TT_ABORT_DESTROY_CHARGE);
		m_ePlasmacasterUsageState = PLASMACASTER_IDLE;
	}
	else
	{
		m_ePlasmacasterUsageState = PLASMACASTER_REQUIRE_ABORTING;
	}
}


bool AVP_ServerActor_Predator::IsPlasmaCasterCharged()

{
	AVP_Inventory_System* pxInventorySystem	= GetInventorySystem();
	AVP_Inventory_Group*  pxWeaponGroup	= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assert(pxWeaponGroup, "Weapon group not found");
	if (!pxWeaponGroup) { return false; }
	Asura_Assert(pxWeaponGroup->GetCurrentItem() == dbg_uPlasmaCasterHash, "Trying to check if plasmacaster is charged but not holding plasmacaster");

	AVP_InventoryWeapon_RangedWeapon* pxWeaponItem	= 
		static_cast<AVP_InventoryWeapon_RangedWeapon*>((pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetItem(dbg_uPlasmaCasterHash));
	Asura_Assert(pxWeaponItem, "Weapon not found");
	if (!pxWeaponItem) { return false; }

	return pxWeaponItem->GetCharge() >= m_fPlasmaCasterDesiredCharge;
}


#ifndef ASURA_NO_DEBUG_OUTPUT
void AVP_ServerActor_Predator::GetCloakDebugString(const u_int uMaxChars, char* szText) const
{
	const AVP_Inventory_System* pxInventorySystem = GetInventorySystem();
	const AVP_Inventory_Item*	pxCloakItem	= (pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetItem(uAVP_INVENTORY_ITEMHASH_CLOAK);

	Asura_Assert(pxCloakItem, "Cloak item not found");
	if(pxCloakItem)
	{
		pxCloakItem->CreateDebugString(uMaxChars, szText);
	}
	else
	{
		szText[0] = '\0';
	}
}

void AVP_ServerActor_Predator::GetWeaponDebugString(const u_int uMaxChars, char* szText) const
{
	const AVP_Inventory_System* pxInventorySystem	= GetInventorySystem();
	const AVP_Inventory_Group*  pxWeaponGroup	= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
	Asura_Assert(pxWeaponGroup, "Weapon group not found");

	if ( pxWeaponGroup )
	{
		pxWeaponGroup->GetCurrentInventoryItem()->CreateDebugString(uMaxChars, szText);
	}
	else
	{
		szText[0] = '\0';
	}
}

#endif


void AVP_ServerActor_Predator::MakePlayer(Asura_Guid uPlayerGuid)
{
	if (uPlayerGuid == GetPlayerInfoGuid()) { return; }

	PARENT::MakePlayer(uPlayerGuid);
}

void AVP_ServerActor_Predator::MakeAI(void)
{
	PARENT::MakeAI();

	if ( m_pxAIAnimStateMachine == NULL )
	{
		m_pxAIAnimStateMachine = new AVP_Predator_AIAnimStateMachine( *this );
	}
}

void AVP_ServerActor_Predator::PostLoadInitialise( void )
{
	if ( !IsPlayer() && !IsADeadBody() && !IsDead() )
	{
		if ( m_pxAIAnimStateMachine == NULL )
		{
			m_pxAIAnimStateMachine = new AVP_Predator_AIAnimStateMachine( *this );
		}
		m_pxAIAnimStateMachine->Initialise();
	}

	PARENT::PostLoadInitialise();

	if ( IsDead() )
	{
		return;
	}

	if(m_bStartsCloaked && !IsPlayer() && !IsADeadBody()) // Only AIs should be using this flag to spawn predators cloaked.
	{
		Cloak();
		/*m_bStartsCloaked = false;*/
	}
}

Axon_Brain* AVP_ServerActor_Predator::CreateBrain()
{
    return new AVP_Brain_Actor_Predator( this );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerActor_Predator::InitialiseInventory
// Author       : Andy Tate
// Date         : 28-Nov-2007
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerActor_Predator::InitialiseInventory(void)
{
	if(PARENT::InitialiseInventory())
	{
		AVP_Inventory_Group* pxPermanentlyActiveGroup = m_pxInventorySystem->GetPermanentlyActiveItemGroup();
		AddPermenantlyActiveItems(pxPermanentlyActiveGroup);
	
		return(true);
	}
	else
	{
		return(false);
	}
}

void AVP_ServerActor_Predator::AddPermenantlyActiveItems(AVP_Inventory_Group* pxGroup)
{
	AVP_InventoryItem_Cloak* pxCloak = new AVP_InventoryItem_Cloak(*pxGroup);
	if ( pxCloak )
	{
		pxCloak->InitialiseFromBlueprint();
	}

	const Asura_Blueprint* pxBlueprint = GetBlueprint();
	if(pxBlueprint != NULL)
	{
		Asura_Value	xValue;
		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_STIM, xValue))
		{
			AVP_InventoryItem_Stim* pxStim = new AVP_InventoryItem_Stim(*pxGroup, xValue.GetHashIDValue(), "Stim");
			if ( pxStim )
			{
				pxStim->InitialiseFromBlueprint();
			}
		}
	}
}

void AVP_ServerActor_Predator::ProcessPredatorPlayerCommands( const AVP_Message_PredatorPlayerCommands* pxData, float fMessageSentTime )
{
	ProcessVertebratePlayerCommands( pxData, fMessageSentTime );

	SetCrouching( pxData->IsCrouching() );
	SetLockOn( pxData->IsLockOn() ); 
	SetTargetGuid( pxData->GetTargetGuid() ); 

	if (pxData->IsRequestingChargeUp())
	{
		// If we're not currently handling any attacks, create a request and handle it locally.
		if (m_pxMeleeSystem && m_pxMeleeSystem->GetCanStartNewAttack() && (m_pxMeleeSystem->GetCurrentAttackType() != AVP_PREDATOR_MELEEATTACK_TYPE_PRIMARYSLASH))
		{
			Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_REQUEST_MELEE, sizeof(AVP_Message_Vertebrate_RequestMelee));

			// Tell the server we want to interact...
			if(pxMessage != NULL)
			{
				AVP_Message_Vertebrate_RequestMelee* pxMeleeData = static_cast<AVP_Message_Vertebrate_RequestMelee*>(pxMessage->GetData());

				// Fill-in the message...
				pxMeleeData->SetType(AVP_PREDATOR_MELEEATTACK_TYPE_PRIMARYSLASH);
				pxMeleeData->SetTargetGuid(pxData->GetCurrentMeleeTarget());
			}

			m_pxMeleeSystem->HandleMessage(pxMessage);
			Asura_Network::DestroyMessage(pxMessage);
		}
	}
	else
	{
		// If we're no longer requesting a charge, but we're still in the loop move, request an end.
		const AVP_ClientActor_Predator* pxClient = AVP_ClientActor_Predator::FindEntity(Guid);
		if (pxClient && pxClient->GetMovementSystem())
		{
			const Asura_Move* pxMove = pxClient->GetMovementSystem()->GetCurrentMove();
			if (pxMove && (pxMove->GetHash() == AVP_Predator_Move_MeleeCharge_Loop::GetStaticHash()))
			{
				Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_REQUEST_MELEE, sizeof(AVP_Message_Vertebrate_RequestMelee));

				// Tell the server we want to interact...
				if(pxMessage != NULL)
				{
					AVP_Message_Vertebrate_RequestMelee* pxMeleeData = static_cast<AVP_Message_Vertebrate_RequestMelee*>(pxMessage->GetData());

					// Fill-in the message...
					pxMeleeData->SetType(AVP_PREDATOR_MELEEATTACK_TYPE_PRIMARYCANCEL);
					pxMeleeData->SetTargetGuid(ASURA_GUID_UNREGISTERED);
				}

				m_pxMeleeSystem->HandleMessage(pxMessage);
				Asura_Network::DestroyMessage(pxMessage);
			}
		}
	}

	// now check is we are making a distraction this frame. 
	if ( pxData->IsRequestingDistract() )
	{
		CreateAIReconDistraction();
	}
}

void AVP_ServerActor_Predator::TakeDamageExtended(const AVP_EntityDamage_RequestVictim& xDamage, const bool bForceFatal, const bool bSendMessageToClient )
{
	PARENT::TakeDamageExtended(xDamage, bForceFatal, bSendMessageToClient);

	const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_DAMAGE, xDamage.GetTypeHash());
	bool bDecloack = false;
	if (pxBlueprint)
	{
		Asura_Value xValue;
		if (pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_NAME_DAMAGETYPE_CAUSE_DECLOAK, xValue))
		{
			bDecloack = xValue.GetBoolValue();
		}
		if (bDecloack)
		{
			Decloak(false);
		}
	}
}


const Asura_Blueprint* AVP_ServerActor_Predator::GetCurrentFlinchBlueprint() const
{
	// Predators have flinches per weapon
	const AVP_InventoryItem_Weapon* pxCurrentWeapon = m_pxInventorySystem ? static_cast<const AVP_InventoryItem_Weapon*>(m_pxInventorySystem->GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON)) : NULL;
	if (!pxCurrentWeapon) { return NULL; }

	return pxCurrentWeapon->GetBlueprint();
}

Asura_Hash_ID AVP_ServerActor_Predator::GetMovesBlueprintType() const
{
	return uAVP_BLUEPRINT_TYPE_MOVESPREDATOR;
}


void AVP_ServerActor_Predator::UpdateCloakingAndWeaponStatus()
{
	if (m_bDesiredCloakingStatus != IsCloaked())
	{

		AVP_Inventory_System* pxInventorySystem = GetInventorySystem();
		AVP_Inventory_Item*	pxCloakItem	= pxInventorySystem ? pxInventorySystem->GetItem(uAVP_INVENTORY_ITEMHASH_CLOAK) : NULL ;
		const AVP_ClientActor_Predator* pxClientActor = static_cast<const AVP_ClientActor_Predator*>(AVP_ClientEntity_Actor::FindEntity(GetGuid()));
		const AVP_Inventory_System* pxClientInventorySystem = pxClientActor ? pxClientActor->GetInventorySystem() : NULL;
		const AVP_Inventory_Item*	pxClientCloakItem = pxClientInventorySystem ? pxClientInventorySystem->GetItem(uAVP_INVENTORY_ITEMHASH_CLOAK) : NULL;
		Asura_Assert(pxCloakItem, "Cloak item not found");
		Asura_Assert(pxClientCloakItem, "Client Cloak item not found");
		if(pxCloakItem && pxClientCloakItem)
		{
			if (m_bDesiredCloakingStatus )
			{
				if ( pxCloakItem->IsIdling(true) && pxClientCloakItem->IsIdling(true)) // not yet in transition to cloaked so ask it now 
				{
					pxCloakItem->RequestBeginUsage( AVP_InventoryItem_Cloak::UT_CLOAK, AVP_InventoryItem_Cloak::TT_MANUAL );
				}
			}
			else // !m_bDesiredCloakingStatus
			{
				if ( pxCloakItem->IsUsing(true) && pxClientCloakItem->IsUsing(true)) // not yet in transition to decloaked so ask it now 
				{
					pxCloakItem->RequestEndUsage( m_bDecloakManually ? AVP_InventoryItem_Cloak::TT_MANUAL : AVP_InventoryItem_Cloak::TT_AUTOMATIC );
				}
			}
		}
	}

	bool bNeedUpdate = false;
	switch(m_eDesiredWeaponType)
	{
	case WEAPON_WRISTBLADES:
		bNeedUpdate = !IsHoldingWristBlades();
		break;
	case WEAPON_PLASMACASTER:
		bNeedUpdate = !IsHoldingPlasmaCaster();
		break;
	default:
		bNeedUpdate = true;
		break;
	}

	if (bNeedUpdate)
	{
		AVP_Inventory_System* pxInventorySystem	= GetInventorySystem();
		AVP_Inventory_Group*  pxWeaponGroup	= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
		Asura_Assert(pxWeaponGroup, "Weapon group not found");

		if ( pxWeaponGroup )
		{
			switch (m_eDesiredWeaponType)
			{
			case WEAPON_WRISTBLADES:
				{
					if (pxWeaponGroup->GetCurrentItem() != dbg_uWristBladesHash)
					{
						AVP_InventoryWeapon_RangedWeapon* pxWeaponItem	= 
							static_cast<AVP_InventoryWeapon_RangedWeapon*>((pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetItem(dbg_uPlasmaCasterHash));
						if (pxWeaponItem->IsUsing(true) && 
							pxWeaponItem->RequestEndUsage(AVP_InventoryWeapon_RangedWeapon::TT_ABORT_DESTROY_CHARGE))
						{
						}
						else if (pxWeaponItem->IsIdling(true) &&
	 							pxWeaponGroup->SetCurrentItem( dbg_uWristBladesHash, false ) )
						{
						}
					}
					break;
				}
			case WEAPON_PLASMACASTER:
				if (pxWeaponGroup->GetCurrentItem() == dbg_uPlasmaCasterHash ||
					(pxWeaponGroup->GetCurrentInventoryItem()->IsIdling(true) &&
					pxWeaponGroup->SetCurrentItem( dbg_uPlasmaCasterHash, false ) ) )
				{
				}
				break;
			case WEAPON_UKNOWN:
			default:
				Asura_Assert(false, "Unknown desired weapon");
				break;
			}
		}
	}
	else if (IsHoldingPlasmaCaster())
	{
		if (m_ePlasmacasterUsageState == PLASMACASTER_REQUIRE_ABORTING || m_ePlasmacasterUsageState == PLASMACASTER_REQUIRE_RELEASING )
		{
			AVP_Inventory_System* pxInventorySystem	= GetInventorySystem();
			AVP_Inventory_Group*  pxWeaponGroup	= pxInventorySystem ? pxInventorySystem->GetGroup( uAVP_INVENTORY_GROUPHASH_WEAPON ) : NULL;
			Asura_Assert(pxWeaponGroup, "Weapon group not found");
			if (!pxWeaponGroup) { return; }

			Asura_Assert(pxWeaponGroup->GetCurrentItem() == dbg_uPlasmaCasterHash, "Trying to release plasmacaster but not holding plasmacaster");

			AVP_InventoryWeapon_RangedWeapon* pxWeaponItem	= 
				static_cast<AVP_InventoryWeapon_RangedWeapon*>((pxInventorySystem == NULL) ? NULL : pxInventorySystem->GetItem(dbg_uPlasmaCasterHash));
			Asura_Assert(pxWeaponItem, "Weapon not found");
			if (!pxWeaponItem) { return ; }

			if (pxWeaponItem->IsUsing(true))
			{
				if (m_ePlasmacasterUsageState == PLASMACASTER_REQUIRE_ABORTING)
				{
					pxWeaponItem->RequestEndUsage(AVP_InventoryWeapon_RangedWeapon::TT_SHOOT);
				}
				else
				{
					pxWeaponItem->RequestEndUsage(AVP_InventoryWeapon_RangedWeapon::TT_ABORT_DESTROY_CHARGE);
				}
				m_ePlasmacasterUsageState = PLASMACASTER_IDLE;
			}
		}
	}
}


bool AVP_ServerActor_Predator::AwardLoadsOfDamageToAllPredators( bool bHelp )
{
	if (bHelp) return true;
	for (Asura_Hashed_List_It<AVP_ServerActor_Predator> xIt(GetList()) ; !xIt.Done() ; xIt.Next())
	{
		xIt.GetCurrent()->AwardLoadsOfDamage();
	}
	return true;
}

void AVP_ServerActor_Predator::HandleInteractions()
{
	AVP_ClientActor_Predator* pxClient = AVP_ClientActor_Predator::FindEntity( GetGuid() );
	if( pxClient )
	{
		AVP_Actor_InteractionHandler* pxInteract = pxClient->GetInteractionHandler();
		if( pxInteract )
		{
			const Asura_Movement_System*  pxMovementSystem	= (pxClient == NULL) ? NULL : pxClient->GetMovementSystem();
			const AVP_Move*				  pxCurrentMove		= static_cast< const AVP_Move* >( (pxMovementSystem == NULL) ? NULL : pxMovementSystem->GetCurrentMove() );

			if(pxCurrentMove)
			{
				if( pxCurrentMove->GetHash() == AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
				{
					// find if the object being interacted with is an invisiswitch
					const AVP_Actor_InteractionHandler*  pxInteractionHandler = pxClient->GetInteractionHandler();
					if (pxInteractionHandler)
					{
						//Get GUID of interaction
						const Asura_Guid uGuid = pxInteractionHandler->GetGuidForInteraction( AVP_INTERACTION_TYPE_DEFAULT );
						if( uGuid != ASURA_GUID_UNREGISTERED )
						{
							AVP_ClientEntity_InteractInvisiSwitch* pxInvisiSwitch = AVP_ClientEntity_InteractInvisiSwitch::FindEntity(uGuid);
							if( pxInvisiSwitch )
							{
								Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_SENTRYGUN_SWITCH_PLAYANIMATION, sizeof( AVP_Message_SentryGun_Switch_PlayAninmation ), GetClassification());
								if(pxMessage)
								{
									AVP_Message_SentryGun_Switch_PlayAninmation* pxData = static_cast< AVP_Message_SentryGun_Switch_PlayAninmation* >( pxMessage->GetData() );
									pxData->SetAnimHash( pxInvisiSwitch->GetAnimHashForSpeciesInteraction( AVP_Settings::AVP_SPECIES_PREDATOR ) );

									pxMessage->SetToGuid( pxInvisiSwitch->GetGuid() );
									pxMessage->SetFromGuid( GetGuid() );
									Asura_Network::Send( pxMessage );
								}
							}
						}
					}
				}
			}
		}
	}
}
/// End of file. ///
