//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerWeapon_Sentrygun.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_ServerWeapon_Sentrygun.h"

// Engine independent.
#include "Affectors/Asura_Affector.h"
#include "Affectors/Asura_Affector_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Timers.h"
#include "Entity/Asura_Entity_Event.h"
#include "Mesh/Asura_Env_System.h"
#include "Network/Asura_Network.h"
#include "Network/Asura_Message.h"
#include "Shapes/Asura_Shape_System.h"
#include "Physics/Asura_Physics.h"
#include "Camera/Asura_Camera.h"

// Project independent.
#include "Actor/Weapons/Sentrygun/AVP_Sentrygun_AIAnimStateMachine.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_Sentrygun_Messages.h"
#include "Moves/AVP_Movement_System.h"
#include "Physics/AVP_Collision_Descriptor.h"

// Axon
#include "Core/Axon_Messages.h"
#include "Core/Axon_System.h"
#include "Core/Axon_Sound_Registry.h"

// AVP Axon
#include "Behaviour/BehaviourTypes/AVP_Behaviour_Types.h"
#include "Core/BrainTypes/AVP_Brain_Actor_Sentrygun.h"
#include "Knowledge/AVP_Knowledge_System_Sentrygun.h"

// Constant variables...
static const u_int g_uCurrentVersion = 7;

// Static variables...
Asura_Hashed_List<AVP_ServerWeapon_Sentrygun> AVP_ServerWeapon_Sentrygun::s_xList( 2 );

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

AVP_ServerWeapon_Sentrygun::AVP_ServerWeapon_Sentrygun( const Asura_Guid uGUID )
: Asura_Entity_Server(uGUID)
, PARENT(0, uGUID)
, m_fVisionRange( 100.0f )
, m_fLethalRange( 0.0f )
, m_fSpeed( 0.5f )
, m_fTanHorHalfFOV( Asura_Maths::Tangent( Asura_Maths::DegToRad( 15.0f ) ) )
, m_fTanVertBottomHalfFOV( Asura_Maths::Tangent( Asura_Maths::DegToRad( 15.0f ) ) )
, m_fTanVertTopHalfFOV( Asura_Maths::Tangent( Asura_Maths::DegToRad( 15.0f ) ) )
, m_fTanHalfFireArc( Asura_Maths::Tangent( Asura_Maths::DegToRad( 40.0f ) ) )
, m_bDryFire( false )
, m_uFlags( 0 )
, m_eLifeState( ALIVE )
{
	m_bStartsEnabled = true;

	Classification = AVP_ENTITYCLASS_SENTRYGUN;

	SetBlockWaypointLinksFlag( true );
	SetBlockNavDataGenerationFlag( true );

	// Add to the list...
	s_xList.AddToList( this );
}

AVP_ServerWeapon_Sentrygun::~AVP_ServerWeapon_Sentrygun()
{
	// unblock our blocked waypoint links
	if ( HasBlockedLinks() )
	{
		BlockLinks( false );
	}

	// Remove from the list...
	s_xList.RemoveFromList( this );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerWeapon_Sentrygun::MakeClientUpdateMessage
// Author       : Callum Hope
// Date         : 3-Oct-2006
////////////////////////////////////////////////////////////////////////////////

void AVP_ServerWeapon_Sentrygun::MakeClientUpdateMessage(void)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof( AVP_Message_SentryGun_ClientUpdate ), GetClassification());
	if(pxMessage != NULL)
	{
		AVP_Message_SentryGun_ClientUpdate* pxData = static_cast<AVP_Message_SentryGun_ClientUpdate*>( pxMessage->GetData() );
		pxData->FillSentryGun( *this );
		SendClientUpdateMessage( pxMessage );
	}
}

void AVP_ServerWeapon_Sentrygun::SendSoundMessage( AVP_ClientWeapon_Sentrygun::Sound eSound )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_SENTRYGUN_PLAYSOUND, sizeof( AVP_Message_Sentrygun_PlaySound ), GetClassification());
	if(pxMessage)
	{
		AVP_Message_Sentrygun_PlaySound* pxData = static_cast< AVP_Message_Sentrygun_PlaySound* >( pxMessage->GetData() );
		pxData->SetSound( eSound );

		pxMessage->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );
		pxMessage->SetFromGuid( GetGuid() );
		Asura_Network::Send( pxMessage );
	}
}

void AVP_ServerWeapon_Sentrygun::Update()
{
	// small and quick fix to make sure that the sentry gun never reaches 0 health. 
	if( GetHealth() < 10.0f )
	{
		SetHealth( 1000.0f );
	}

	PARENT::Update();
}

bool AVP_ServerWeapon_Sentrygun::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		////////////////////
		case AVP_MSGID_SENTRYGUN_DESTROY:
		////////////////////
		{
			SendMessagesForEvent( ASURA_ENTITY_EVENT_ONDEATH );
		}	//fall through
		////////////////////
		case AVP_MSGID_SENTRYGUN_ENABLE:
		case AVP_MSGID_SENTRYGUN_DISABLE:
		////////////////////
		{
			// handle message in brain before forwarding on to clients
			if ( GetActorBrain())
			{
				GetActorBrain()->DoHandleMessage( pxMessage );
			}
			
			Asura_Network::Forward( pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
			return true;
		}
		////////////////////
		case AVP_MSGID_SENTRYGUN_DRYFIRE_TOGGLE:
		////////////////////
		{
			const AVP_Message_SentryGun_Toggle_Dry_Fire* pxData = static_cast<const AVP_Message_SentryGun_Toggle_Dry_Fire*>( pxMessage->GetData() );

			if( pxData )
			{
				m_bDryFire = pxData->GetDryFireToggle();
			}

			return true;
		}
		////////////////////
		case AVP_MSGID_SET_SENTRYGUN_SWEEP_SPEED:
		////////////////////
		{
			const AVP_Message_SentryGun_Set_SweepSpeed* pxMsgData = static_cast< const AVP_Message_SentryGun_Set_SweepSpeed*> ( pxMessage->GetData( ) );

			if ( pxMsgData )
			{
				// ranges should be clamped in the tools code where they are set but do another to be sure. 
				m_fSpeed = Asura_Maths::ClampToRange( pxMsgData->m_fSpeed, 0.05f, 1.0f );
			}
			return true;
		}
		////////////////////
		case AVP_MSGID_SET_SENTRYGUN_VISION_RANGE:
		////////////////////
		{
			const AVP_Message_SentryGun_Set_VisionRange* pxMsgData = static_cast< const AVP_Message_SentryGun_Set_VisionRange*> ( pxMessage->GetData( ) );

			if ( pxMsgData )
			{
				// first find the percentage of the full range the lethal range is. 
				float fPercentLethal = m_fLethalRange / m_fVisionRange;

				m_fVisionRange = Asura_Maths::ClampToRange(	pxMsgData->m_fRange, 0.0f, 1000.0f );

				// now adjust the lethal range to make sure it is proportionaly the same as it was before to the total vision range. 
				m_fLethalRange = m_fVisionRange * fPercentLethal;

				if( m_fVisionRange < m_fLethalRange )
				{
					// double check if the vision range is lower than the lethal range lower the lethal range as well. 
					m_fLethalRange = m_fVisionRange;
				}
			}
			return true;
		}
		////////////////////
		case AVP_MSGID_SET_SENTRYGUN_HOR_FOV:
		////////////////////
		{
			const AVP_Message_SentryGun_Set_HorizontalFOV* pxMsgData = static_cast< const AVP_Message_SentryGun_Set_HorizontalFOV*> ( pxMessage->GetData( ) );

			if ( pxMsgData )
			{
				// make sure the value is in the legal range then convert to a half tan value
				float fClampedAngle = Asura_Maths::ClampToRange( pxMsgData->m_fHorFov, 1.0f, 178.0f );
				m_fTanHorHalfFOV = Asura_Maths::Tangent( Asura_Maths::DegToRad( 0.5f * fClampedAngle ) );
				if( m_fTanHorHalfFOV > m_fTanHalfFireArc )
				{
					// if the FOV is set to be larger than the FireArc then increase the Firearc to preserve the most recent request (Horizontal FOV)
					m_fTanHalfFireArc = m_fTanHorHalfFOV;
				}
			}
			return true;
		}
		////////////////////
		case AVP_MSGID_SET_SENTRYGUN_VER_FOV:
		////////////////////
		{
			const AVP_Message_SentryGun_Set_VerticalFOV* pxMsgData = static_cast< const AVP_Message_SentryGun_Set_VerticalFOV*> ( pxMessage->GetData( ) );

			if ( pxMsgData )
			{
				float fClampedAngle = Asura_Maths::ClampToRange( pxMsgData->m_fVerFov, 1.0f, 90.0f );
				m_fTanVertBottomHalfFOV = Asura_Maths::Tangent( Asura_Maths::DegToRad( 0.5f * fClampedAngle ) );
				m_fTanVertTopHalfFOV = Asura_Maths::Tangent( Asura_Maths::DegToRad( 0.5f * fClampedAngle ) );
			}
			return true;
		}
		////////////////////
		case AVP_MSGID_SET_SENTRYGUN_FIRING_ARC:
		////////////////////
		{
			const AVP_Message_SentryGun_Set_FiringArc* pxMsgData = static_cast< const AVP_Message_SentryGun_Set_FiringArc*> ( pxMessage->GetData( ) );

			if ( pxMsgData )
			{
				float fClampedAngle = Asura_Maths::ClampToRange( pxMsgData->m_fArc, 1.0f, 178.0f );
				m_fTanHalfFireArc = Asura_Maths::Tangent( Asura_Maths::DegToRad( 0.5f * fClampedAngle ) );
				if( m_fTanHalfFireArc < m_fTanHorHalfFOV )
				{
					// if the fire arc is now less than the Hor FOV then lover the FOV to always be within it. 
					m_fTanHorHalfFOV = m_fTanHalfFireArc;
				}
			}
			return true;
		}
		////////////////////
		case AVP_MSGID_SET_SENTRYGUN_LETHAL_RANGE:
		////////////////////
		{
			const AVP_Message_SentryGun_Set_LethalRange* pxMsgData = static_cast< const AVP_Message_SentryGun_Set_LethalRange*> ( pxMessage->GetData( ) );

			if ( pxMsgData )
			{
				// get the percentage of the range from 0-100, range it to 0-1 and multiply the total range to find the new lethal range. 
				float fRangePercent = Asura_Maths::ClampToRange( pxMsgData->m_fRange, 0.0f, 100.0f );
				fRangePercent *= 0.01f;
				m_fLethalRange = m_fVisionRange * fRangePercent;
			}
			return true;
		}
		////////////////////
		default:
		////////////////////
		{
			return PARENT::HandleMessage( pxMessage );
		}
	}
}

void AVP_ServerWeapon_Sentrygun::OnDeath( const AVP_EntityDamage_ApplyToEntity& xDamage )
{
	PARENT::OnDeath( xDamage );
	
	SetAsDestroyed();
}

bool AVP_ServerWeapon_Sentrygun::IsEnemyTo( const Asura_Entity& xThem ) const
{
	switch ( xThem.GetClassification() )
	{
		case AVP_ENTITYCLASS_ALIEN:
		case AVP_ENTITYCLASS_ALIENCHESTBURSTER:
		case AVP_ENTITYCLASS_ALIENFACEHUGGER:
		case AVP_ENTITYCLASS_ALIENQUEEN:
		case AVP_ENTITYCLASS_ALIENTEAM:
		case AVP_ENTITYCLASS_PREDALIEN: // is it a predator or alien ?
		{
			return m_bShootsAliens;
		}

		case AVP_ENTITYCLASS_ANDROID:
		{
			return m_bShootsAndroids;
		}

		case AVP_ENTITYCLASS_CIVILIAN:
		{
			return m_bShootsCivilians;
		}

		case AVP_ENTITYCLASS_HUMAN:
		case AVP_ENTITYCLASS_MARINE:
		{
			return m_bShootsMarines;
		}

		case AVP_ENTITYCLASS_PREDATOR:
		{
			return m_bShootsPredators;
		}

		default:
		{
			return false;
		}
	}
}

void AVP_ServerWeapon_Sentrygun::PostLoadInitialise( void )
{
	if ( m_pxAIAnimStateMachine == NULL )
	{
		m_pxAIAnimStateMachine = new AVP_Sentrygun_AIAnimStateMachine( *this );
	}
	m_pxAIAnimStateMachine->Initialise();

	PARENT::PostLoadInitialise();

	if ( GetSentrygunBrain() && GetSentrygunBrain()->GetKnowledgeSystem() )
	{
		GetSentrygunBrain()->GetKnowledgeSystem()->SetVisionFOV( GetVisionRange(), GetTanHorizontalFOVHalfAngle(), GetTanVerticalFOVTopAngle(), GetTanVerticalFOVBottomAngle() );
		GetSentrygunBrain()->GetKnowledgeSystem()->SetVisionFireArc( GetTanFireArcHalfAngle() );
	}

	Asura_Assert( m_pxPhysicsObjectHandler, "No physics object handdler" );
	if ( m_pxPhysicsObjectHandler )
	{
		const AVP_Collision_Descriptor* pxCollDesc = GetDefaultCollisionDescriptor();
		if ( pxCollDesc )
		{
			const Asura_Collision_Object* pxCollObj = pxCollDesc->GetCollisionBox();
			Asura_Assert( pxCollObj != NULL, "Actor has no default collision object, physics won't be set up" );
			if ( pxCollObj != NULL )
			{
				m_pxPhysicsObjectHandler->Reset(AVP_Actor_PhysicsObjectHandler::STATIC_AI, *pxCollObj, AVP_Actor_PhysicsObjectHandler::GetActorCollisionProxy(Classification));
				m_pxDynamicObject = m_pxPhysicsObjectHandler->GetPhysicsObject();
				m_pxDynamicObject->SetActorsDontPushMe(true);
				m_pxDynamicObject->SetActorDontPushActors(true);
				if ( IsEntityActive() )
				{
					m_pxDynamicObject->SetIgnore(false);
				}
			}
		}
	}

	
	if( m_eLifeState == DEAD )
	{
		SendMessagesForEvent( ASURA_ENTITY_EVENT_ONDEATH );
	}
}

void AVP_ServerWeapon_Sentrygun::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int	uVersion;
	// Read the version...
	xStream >> uVersion;

	switch ( uVersion )
	{
		case 0:
		{
			Asura_Assert( false, "sentrygun never had a chunk version 0" );
		}
		break;

		case 1:
		{
			PARENT::ReadFromChunkStream(xStream);
			// uses defaults in constructor for FOV
		}
		break;

		case 2:
		{
			PARENT::ReadFromChunkStream(xStream);
			xStream >> m_fVisionRange;
			xStream >> m_fTanHorHalfFOV;
			xStream >> m_fTanVertBottomHalfFOV;
		}
		break;

		case 3:
		{
			PARENT::ReadFromChunkStream(xStream);
			xStream >> m_fVisionRange;
			xStream >> m_fTanHorHalfFOV;
			xStream >> m_fTanVertBottomHalfFOV;
			Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uFlags );
			xStream >> xBitfield;
		}
		break;

		case 4:	// addition of the fire arch, lethal range and speed
		{
			PARENT::ReadFromChunkStream(xStream);
			xStream >> m_fVisionRange;
			xStream >> m_fLethalRange;
			xStream >> m_fSpeed;
			xStream >> m_fTanHorHalfFOV;
			xStream >> m_fTanVertBottomHalfFOV;
			xStream >> m_fTanHalfFireArc;
			Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uFlags );
			xStream >> xBitfield;
		}
		break;

		case 5:	// addition of the life state
		{
			int iTempLifeState;

			PARENT::ReadFromChunkStream(xStream);
			xStream >> m_fVisionRange;
			xStream >> m_fLethalRange;
			xStream >> m_fSpeed;
			xStream >> m_fTanHorHalfFOV;
			xStream >> m_fTanVertBottomHalfFOV;
			xStream >> m_fTanHalfFireArc;
			xStream >> iTempLifeState;
			Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uFlags );
			xStream >> xBitfield;

			m_eLifeState = static_cast<LifeState>( iTempLifeState );
			m_fTanVertTopHalfFOV = m_fTanVertBottomHalfFOV;
		}
		break;

		case 6:	// addition of seperate angle for top and bottom vertical FOVs
		{
			int iTempLifeState;

			PARENT::ReadFromChunkStream(xStream);
			xStream >> m_fVisionRange;
			xStream >> m_fLethalRange;
			xStream >> m_fSpeed;
			xStream >> m_fTanHorHalfFOV;
			xStream >> m_fTanVertBottomHalfFOV;
			xStream >> m_fTanVertTopHalfFOV;
			xStream >> m_fTanHalfFireArc;
			xStream >> iTempLifeState;
			Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uFlags );
			xStream >> xBitfield;

			m_eLifeState = static_cast<LifeState>( iTempLifeState );
		}
		break;

		case 7:	// addition of Dry Fire Parameter
		{
			int iTempLifeState;

			PARENT::ReadFromChunkStream(xStream);
			xStream >> m_fVisionRange;
			xStream >> m_fLethalRange;
			xStream >> m_fSpeed;
			xStream >> m_fTanHorHalfFOV;
			xStream >> m_fTanVertBottomHalfFOV;
			xStream >> m_fTanVertTopHalfFOV;
			xStream >> m_fTanHalfFireArc;
			xStream >> m_bDryFire;
			xStream >> iTempLifeState;
			Asura_ReadFromStream_Bitfield_Wrapper< u_int > xBitfield( m_uFlags );
			xStream >> xBitfield;

			m_eLifeState = static_cast<LifeState>( iTempLifeState );
		}
		break;

		default:
		{
			Asura_Assertf( false, ( "Invalid chunk version %d for sentrygun", uVersion ) );
		}
		break;
	} // end switch (version)

	if ( IsEntityActive() )
	{
		CalculateAndBlockWaypointLinks();
	}
}

void AVP_ServerWeapon_Sentrygun::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	// Write the version...
	xStream << g_uCurrentVersion;

	// Write the parent data...
	PARENT::WriteToChunkStream(xStream);

	// Write our data
	xStream << m_fVisionRange;
	xStream << m_fLethalRange;
	xStream << m_fSpeed;
	xStream << m_fTanHorHalfFOV;
	xStream << m_fTanVertBottomHalfFOV;
	xStream << m_fTanVertTopHalfFOV;
	xStream << m_fTanHalfFireArc;
	xStream << m_bDryFire;
	xStream << static_cast<int>( m_eLifeState );

	Asura_WriteToStream_Bitfield_Wrapper< u_int > xBitfield( m_uFlags );
	xBitfield.Add( m_bStartsEnabled, 1 );
	xBitfield.Add( m_bShootsAliens, 1 );
	xBitfield.Add( m_bShootsAndroids, 1 );
	xBitfield.Add( m_bShootsCivilians, 1 );
	xBitfield.Add( m_bShootsMarines, 1 );
	xBitfield.Add( m_bShootsPredators, 1 );
	xStream << xBitfield;
}

void AVP_ServerWeapon_Sentrygun::OnEntityActivate( void )
{
	PARENT::OnEntityActivate();

	if ( !HasBlockedLinks() )
	{
		BlockLinks( true );
	}
}

void AVP_ServerWeapon_Sentrygun::OnEntityDeactivate( void )
{
	PARENT::OnEntityDeactivate();

	if ( HasBlockedLinks() )
	{
		BlockLinks( false );
	}
}

void AVP_ServerWeapon_Sentrygun::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	PARENT::InitialiseFromBlueprint( xBlueprint );

	Asura_Value xVal;
	if ( xBlueprint.GetParameterValueByHashID( 0xa8584bc0, xVal ) )	// ShootsAliens
	{
		m_bShootsAliens = xVal.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0x55702418, xVal ) )	// ShootsAndroids
	{
		m_bShootsAndroids = xVal.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0x2261979c, xVal ) )	// ShootsCivilians
	{
		m_bShootsCivilians = xVal.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0xcb37483d, xVal ) )	// ShootsMarines
	{
		m_bShootsMarines = xVal.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0xc45dd6a8, xVal ) )	// ShootsPredators
	{
		m_bShootsPredators = xVal.GetBoolValue();
	}
	if ( m_pxPhysicsObjectHandler && xBlueprint.GetParameterValueByHashID( 0x7d643002, xVal ) && xVal.GetBoolValue() == true )	// NoCollisions
	{
		delete m_pxPhysicsObjectHandler;
	}
}

void AVP_ServerWeapon_Sentrygun::CalculateAndBlockWaypointLinks()
{
	InitialiseWaypointLinkBlockCalculations();
	CalculateWaypointLinkBlocks();
	ShutdownWaypointLinkBlockCalculations();
	
	if( ! HasBlockedLinks() )
	{
		BlockLinks( true );
	}
}

void AVP_ServerWeapon_Sentrygun::MakeAI( void )
{
	if ( m_pxAIAnimStateMachine == NULL )
	{
		m_pxAIAnimStateMachine = new AVP_Sentrygun_AIAnimStateMachine( *this );
	}

	PARENT::MakeAI();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerWeapon_Sentrygun::IsOutsideFireArc
// Scope		: Public
// Author       : Callum Hope
// Date         : 11-Feb-2009
//
// Desc			: Checks if a potential target is outside of the stationalry fire arc of the sentry gun. 
//				  The vector argument is the vector to the target from the gun. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerWeapon_Sentrygun::IsOutsideFireArc( const Asura_Vector_3& xToEntity )
{
	Asura_Vector_3 xPosition;
	Asura_Vector_3 xDirection;
	Asura_Matrix_3x3 xOrientMatrix;

	GetChestPosition( xPosition );
	GetHeadFowardVector( xDirection );
	Asura_Maths::OrientationFromDirection( xOrientMatrix, xDirection );

	const Asura_Vector_3 xLocalSpacePoint = xOrientMatrix.ApplyTranspose( xToEntity );

	// is the point behind us?
	if( xLocalSpacePoint.z < 0.0f )
	{
		return true;
	}

	// is the point outside the left or right planes of the fire arc frustrum
	const float fZX = xLocalSpacePoint.z * m_fTanHalfFireArc;
	if( ( xLocalSpacePoint.x - fZX ) > 0.0f || ( - xLocalSpacePoint.x - fZX ) > 0.0f )
	{
		return true;
	}

// CAH commented out the top and bottom check as the FOV takes care of those (and they are irrelevant as the barrel can look up and down)
/*
	// Is it above the top or bottom planes (plane angles taken from the FOV frustrum angles.)
	if( xLocalSpacePoint.y < 0.0f )
	{
		// top
		const float fZY = xLocalSpacePoint.z * m_fTanVertTopHalfFOV;
		if( - xLocalSpacePoint.y - fZY > 0.0f )
		{
			return true;
		}
	}
	else
	{
		// bottom
		const float fZY = xLocalSpacePoint.z * m_fTanVertBottomHalfFOV;
		if( xLocalSpacePoint.y - fZY > 0.0f )
		{
			return true;
		}
	}
*/
	return false;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerWeapon_Sentrygun::IsWithinLethalRange
// Scope		: Public
// Author       : Callum Hope
// Date         : 16-Feb-2009
//
// Desc			: Checks if a potential target is within the lethal range for this sentry gun. 
//				  The vector argument is the world space position of the target. 
////////////////////////////////////////////////////////////////////////////////

bool AVP_ServerWeapon_Sentrygun::IsWithinLethalRange( const Asura_Vector_3& xTargetPosition ) 
{
	Asura_Vector_3 xSentryHeadPosition;
	Asura_Vector_3 xDifferenceVector;

	GetHeadPosition( xSentryHeadPosition );

	xDifferenceVector = xSentryHeadPosition - xTargetPosition;

	if( xDifferenceVector.MagnitudeSquared() <= ( m_fLethalRange * m_fLethalRange ) )
	{
		return true;
	}
	
	return false;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ServerWeapon_Sentrygun::GetSentryGunAIAnimStateMachine
// Scope		: Public
// Author       : Callum Hope
// Date         : 23-Feb-2009
//
// Desc			: Casts and returns the anim state machine held in AVP_ServerEntity_Actor.h 
//				  to allow Sentry gun specific functionality and Anim requests. 
////////////////////////////////////////////////////////////////////////////////

AVP_Sentrygun_AIAnimStateMachine* AVP_ServerWeapon_Sentrygun::GetSentryGunAIAnimStateMachine() const
{
	return static_cast<AVP_Sentrygun_AIAnimStateMachine*>( m_pxAIAnimStateMachine );
}

Axon_Brain* AVP_ServerWeapon_Sentrygun::CreateBrain()
{
	return new AVP_Brain_Actor_Sentrygun( this );
}

AVP_Brain_Actor_Sentrygun* AVP_ServerWeapon_Sentrygun::GetSentrygunBrain()
{
	return static_cast< AVP_Brain_Actor_Sentrygun* >( GetActorBrain() );
}

const AVP_Brain_Actor_Sentrygun* AVP_ServerWeapon_Sentrygun::GetSentrygunBrain() const
{
	return static_cast< AVP_Brain_Actor_Sentrygun* >( GetActorBrain() );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Message_SentryGun_ClientUpdate::FillSentryGun
// Scope		: Public
// Author       : Callum Hope
// Date         : 19-Feb-2009
//
// Desc			: CLIENT UPDATE MESSAGE FILLER. Needed as our messages cant rely on constructors. 
////////////////////////////////////////////////////////////////////////////////

void AVP_Message_SentryGun_ClientUpdate::FillSentryGun( const AVP_ServerWeapon_Sentrygun &xSentryGun )
{
	FillActor( xSentryGun );

	if( xSentryGun.GetLifeState() == AVP_ServerWeapon_Sentrygun::ALIVE )
	{
		m_bLightOn = true;
	}
	else
	{
		m_bLightOn = false;
	}
}



/// End of file. ///
