//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerWeapon_Sentrygun.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef ___AVP_SERVERVEHICLE_SENTRYGUN_H___
#define ___AVP_SERVERVEHICLE_SENTRYGUN_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_List.h"

// avp
#include "Actor/Weapons/Sentrygun/AVP_Sentrygun_AIAnimStateMachine.h"
#include "Entity/PhysicalObject/Actor/Weapons/AVP_ServerActor_Weapon.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_ClientWeapon_Sentrygun.h"
#include "Actor/AVP_Actor_DynamicStat.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimSequenceData.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Collision_Object;
class Axon_Project_Brain;
class Asura_AnimationAttachment_Projector;
class Asura_AnimationAttachment_ClientContainerInstance;
class AVP_Brain_Actor_Sentrygun;

// C L A S S E S /////////////////////////////////////////////////////////////

class AVP_ServerWeapon_Sentrygun : public AVP_ServerActor_Weapon
{
public:

	enum LifeState
	{
		ALIVE,
		DORMANT,
		DEAD,
		MAX_LIVE_STATES
	};

	// Public functions...
	AVP_ServerWeapon_Sentrygun(const Asura_Guid uGUID);
	virtual ~AVP_ServerWeapon_Sentrygun(void);

	float GetVisionRange() { return m_fVisionRange; }
	float GetLethalRange() { return m_fLethalRange; }
	float GetSpeed() { return m_fSpeed; }
	float GetTanHorizontalFOVHalfAngle() { return m_fTanHorHalfFOV; }
	float GetTanVerticalFOVBottomAngle() { return m_fTanVertBottomHalfFOV; }
	float GetTanVerticalFOVTopAngle() { return m_fTanVertTopHalfFOV; }
	float GetTanFireArcHalfAngle() { return m_fTanHalfFireArc; }

	bool GetWeaponIsDryFiring() { return m_bDryFire; }

	void		SetLifeState( LifeState eLifeState ) { m_eLifeState = eLifeState; }
	LifeState	GetLifeState()const { return m_eLifeState; }

	void SendSoundMessage( AVP_ClientWeapon_Sentrygun::Sound eSound );

	// Chunk processing functions...
	virtual	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	// Overridden functions...
	virtual void MakeAI(void);
	virtual	void Update(void);
	virtual void PostLoadInitialise(void);
	virtual	bool HandleMessage(const Asura_Message* pxMessage);

	virtual void OnDeath(const AVP_EntityDamage_ApplyToEntity& xDamage);

	virtual bool GetHasCreateMessage() const { return !IsDead()|| IsADeadBody(); }

	bool GetStartsEnabled() const { return m_bStartsEnabled; }

	bool IsEnemyTo( const Asura_Entity& xThem ) const;

	bool IsOutsideFireArc( const Asura_Vector_3& xToEntity );

	bool IsWithinLethalRange( const Asura_Vector_3& xTargetPosition );

	AVP_Sentrygun_AIAnimStateMachine* GetSentryGunAIAnimStateMachine() const;
	
	// Static list functions...
	static AVP_ServerWeapon_Sentrygun*							FindEntity(const Asura_Guid uGuid){ return(s_xList.FindEntity(uGuid)); }
	static const Asura_Hashed_List<AVP_ServerWeapon_Sentrygun>*	GetList(void) { return(&s_xList); }

	static void Create( Asura_Guid uGuid ) { new AVP_ServerWeapon_Sentrygun( uGuid ); }
	static AVP_ServerWeapon_Sentrygun* CreateAndReturn( Asura_Guid uGuid ) { return new AVP_ServerWeapon_Sentrygun( uGuid ); }

protected:
	virtual Axon_Brain* CreateBrain();

	AVP_Brain_Actor_Sentrygun* GetSentrygunBrain();
	const AVP_Brain_Actor_Sentrygun* GetSentrygunBrain() const;

	virtual	void OnEntityActivate( void );
	virtual	void OnEntityDeactivate( void );
	virtual void InitialiseFromBlueprint(const Asura_Blueprint& xBlueprint);
	virtual void MakeClientUpdateMessage( void );

	void CalculateAndBlockWaypointLinks();

	float 			m_fVisionRange;
	float			m_fLethalRange;
	float			m_fSpeed;
	float 			m_fTanHorHalfFOV;
	float 			m_fTanVertBottomHalfFOV;
	float 			m_fTanVertTopHalfFOV;
	float			m_fTanHalfFireArc;
	bool			m_bDryFire;

	union
	{
		u_int		m_uFlags;
		struct  
		{
			u_int	m_bStartsEnabled	: 1,
					m_bShootsAliens		: 1,
					m_bShootsAndroids	: 1,
					m_bShootsCivilians	: 1,
					m_bShootsMarines	: 1,
					m_bShootsPredators	: 1,
										: 0;
		};
	};

private:
	// Private typedefs...
	typedef AVP_ServerActor_Weapon PARENT; 

	// Friend classes...
#ifdef ASURA_TOOLS
	friend class ToolsAVP_Entity_Sentrygun;
#endif

	// Private variables...
	LifeState m_eLifeState;

	static Asura_Hashed_List<AVP_ServerWeapon_Sentrygun> s_xList;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___AVP_SERVERVEHICLE_SENTRYGUN_H___
