//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_InteractObject.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////



#include "Core/Asura.h"

#include "Entity/PhysicalObject/Interact/AVP_ClientEntity_InteractObject.h"

#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_OrientedVolume.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#include "SpecialFX/PFX/Asura_PFX_Enums.h"

#include "Actor/AVP_Actor_AnimStateData.h"
#include "Actor/Vertebrates/AVP_Vertebrate_HUDHandler.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/PhysicalObject/AVP_PhysicalObject_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Interact/AVP_Messages_InteractObject.h"
#include "Entity/AVP_Messages.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Moves/AVP_Move_Interaction.h"

Asura_Hashed_List<AVP_ClientEntity_InteractObject> AVP_ClientEntity_InteractObject::s_xList;

AVP_ClientEntity_InteractObject::AVP_ClientEntity_InteractObject()
: m_pxActorVolume( NULL )
, m_pxOnTargetVolume( NULL )
, m_pxOffTargetVolume( NULL )
, m_uOnRestAnimID( ASURA_HASH_ID_UNSET )
, m_uOffRestAnimID( ASURA_HASH_ID_UNSET )
, m_uTurnOnAnimID( ASURA_HASH_ID_UNSET )
, m_uTurnOffAnimID( ASURA_HASH_ID_UNSET )
, m_uFlags( 0 )
, m_uPredatorFocusModePFXHash( ASURA_HASH_ID_UNSET )
, m_uPredatorFocusModePFXMarkerPoint( ASURA_HASH_ID_UNSET )
, m_pxPredatorFocusModePFXAttachment( NULL )
{
	Classification = AVP_ENTITYCLASS_INTERACTOBJECT;

	s_xList.AddToList( this );
}

AVP_ClientEntity_InteractObject::~AVP_ClientEntity_InteractObject() 
{
	delete m_pxActorVolume;
	delete m_pxOnTargetVolume;
	delete m_pxOffTargetVolume;

	s_xList.RemoveFromList( this );
}

// void AVP_ClientEntity_InteractObject::SetupDynamicObject( u_int uPhysicalObjectFlags )
// {
//     ASURA_PHYSICS_OBJECT_TYPE eReqdType = ( uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_STATIC
//                                                 ? ASURA_PHYSICS_OBJECT_STATIC
//                                                 : ASURA_PHYSICS_OBJECT_EXTERNALCONTROL );
// 
// 	if ( m_pxDynamicObject )
// 	{
// 		// Already created. Note that we don't expect the static / non-static
//         // property to change.
//         Asura_Assert( m_pxDynamicObject->GetType() == eReqdType, "" );
// 		return;
// 	}
// 
// 	if ( uPhysicalObjectFlags & ( ASURA_PHYSICALOBJECT_FLAG_NOCOLLISIONS | ASURA_PHYSICALOBJECT_FLAG_ISMERGEDWITHENV ) )
// 	{
// 		return;
// 	}
// 
// 	const Asura_Shape_Instance& xShapeInstance = GetShapeInstance();
// 	const Asura_Shape* pxShape = xShapeInstance.GetShape();
// 	
// 	if ( pxShape && pxShape->HasNoCollisions() )
// 	{
// 		// No collisions, so no need to create a dynamics block.
// 		return;
// 	}
// 
// 	m_pxDynamicObject = Asura_Physics_Object::Create( eReqdType );
//     if ( !m_pxDynamicObject ) { return; }
// 	
// 	m_pxDynamicObject->SetPosition( GetCurrentPosition() );
// 	m_pxDynamicObject->SetOrientation( GetCurrentOrientation() );
// 	m_pxDynamicObject->SetAssociatedEntityGuid( GetGuid() );
// 	
// 	// Find a suitable collision object.
// 	const Asura_Collision_Object* pxCollObj = NULL;
// 	if ( pxShape )
// 	{
// 		pxCollObj = pxShape->GetDynamicsCollisionData();
// 	}
// 	else
// 	{
// 		const Asura_Animation* pxAnim = xShapeInstance.GetAnimation();
//         if ( pxAnim )
//         {
// 		    const Asura_Hierarchy_Skin* pxSkin = pxAnim->GetSkin();
// 		    if ( pxSkin )
// 		    {
// 			    pxCollObj = pxSkin->GetCollisionObject();
// 		    }
//         }
// 	}
// 
//     // Otherwise use some default box.
// 	if ( !pxCollObj )
// 	{
// 		const Asura_Hash_ID	uShapeHash	= GetShapeInstance().GetShapeHashID();
// 
// 		pxCollObj = Asura_Dynamic_Extents::GetCollisionObject( 0, uShapeHash, GetGuid() );
// 	    Asura_Assert( pxCollObj, "No collision objects available?" );
// 	}
// 
// 	m_pxDynamicObject->SetCollisionObject( pxCollObj );
// }
// 

void AVP_ClientEntity_InteractObject::Create( const Asura_Message* pxMsg )
{
	Asura_Assert( pxMsg->GetMsgID() == ASURA_MSG_CLIENTCREATE, "" );
	Asura_Assert( pxMsg->GetEntityClassification() == AVP_ENTITYCLASS_INTERACTOBJECT, "" );

	const AVP_Message_InteractObject_ClientCreate* pxData = static_cast< const AVP_Message_InteractObject_ClientCreate* >( pxMsg->GetData() );

	AVP_ClientEntity_InteractObject* pxNew = new AVP_ClientEntity_InteractObject();
	pxNew->ProcessClientCreate(  pxData, pxMsg->GetSentTime(), true );
}

bool AVP_ClientEntity_InteractObject::HandleMessage( const Asura_Message* pxMsg )
{
	switch ( pxMsg->GetMsgID() )
	{
		case AVP_MSGID_INTERACTION_OBJECT_ENABLE:
		{
			m_bEnabled = true;
			return true;
		}

		case AVP_MSGID_INTERACTION_OBJECT_DISABLE:
		{
			m_bEnabled = false;
			return true;
		}

		case AVP_MSGID_REQUEST_INTERACTION:
		{
			// requesting interaction, do it
			const AVP_Message_Actor_RequestInteraction*	pxData = static_cast< const AVP_Message_Actor_RequestInteraction* >( pxMsg->GetData() );
			Asura_Assert( pxData, "No data" );
			if ( pxData )
			{
				if ( pxData->GetActorGuid() == ASURA_GUID_UNREGISTERED )
				{
					// if we've not got an actor just play the default on/off anim, otherwise it gets played from the move
					Asura_Hash_ID uAnim = pxData->IsOn() ? m_uOffRestAnimID : m_uOnRestAnimID;
					if ( uAnim != ASURA_HASH_ID_UNSET )
					{
						PlayAnimation( uAnim );
					}
				}
				else
				{
					// start the move on our actor entity
					AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( pxData->GetActorGuid() );
					Asura_Assert( pxActor, "No actor" );
					if ( pxActor != NULL )
					{
						Asura_Message* pxMoveMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_InteractionMoveStart ) );
						Asura_Assert( pxMoveMessage , "unable to create message" );
						if ( pxMoveMessage  != NULL )
						{
							AVP_Message_InteractionMoveStart* pxMoveData = static_cast< AVP_Message_InteractionMoveStart* >( pxMoveMessage->GetData() );

							// Fill-in the message-body...
							pxMoveMessage->SetFromGuid( GetGuid() );
							pxMoveMessage->SetToGuid( pxData->GetActorGuid() );

							pxMoveData->ClearFlags();
							pxMoveData->SetMoveID( AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() );
							pxMoveData->SetIgnoredByOwner( false );
							pxMoveData->SetInteractionObject( GetGuid() );
							pxMoveData->SetNextMoveHash( AVP_Move_Interaction::GetStaticHash() );
							pxMoveData->SetOn( !pxData->IsOn() );
							// Forward the message directly to the client actor... we will do this on each client on reciept of the interact message
							pxActor->HandleMessage( pxMoveMessage );
							Asura_Network::DestroyMessage( pxMoveMessage );
						}
					}
				}

				OnInteraction( !pxData->IsOn() );
			}
			return true;
		}

		default:
			return PARENT::HandleMessage(pxMsg);
	}
}

void AVP_ClientEntity_InteractObject::PostCreateInitialise()
{
	PARENT::PostCreateInitialise();

	Asura_Hash_ID uAnim = m_bOn ? m_uOnRestAnimID : m_uOffRestAnimID;
	if ( uAnim != ASURA_HASH_ID_UNSET )
	{
		PlayAnimation( uAnim );
	}
}

bool AVP_ClientEntity_InteractObject::WillInteractWith( u_int uClassification ) const
{
	switch ( uClassification )
	{
		case AVP_ENTITYCLASS_ALIEN:
		{
			return m_bUseByAlien;
		}

		case AVP_ENTITYCLASS_MARINE:
		case AVP_ENTITYCLASS_ANDROID:
		case AVP_ENTITYCLASS_CIVILIAN:
		case AVP_ENTITYCLASS_HUMAN:
		{
			return m_bUseByHuman;
		}

		case AVP_ENTITYCLASS_PREDATOR:
		{
			return m_bUseByPredator;
		}

		default:
		{
			Asura_Assert( false, "Who's this trying to use me? I don't know you, go away! ( Code bug )" );
			return false;
		}
	}
}

bool AVP_ClientEntity_InteractObject::IsActorUsingRightInteractionItem(const AVP_ClientEntity_Actor& xActor) const
{
	return true;			
}

void AVP_ClientEntity_InteractObject::PlayAnimation( const Asura_Hash_ID uAnimHash )
{
	// Play animation
	Asura_Animation* pxAnimation = GetShapeInstance().GetAnimation();
	if ( pxAnimation )
	{
		Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend_AnimatedTween( 0.0f, false );
		Asura_AnimationController_Animation* pxAACA = pxAnimation->CreateControllerAnimation( AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE, 0, uAnimHash, pxBlend );
		if ( pxAACA )
		{
			pxAACA->Play();
		}
		else
		{
			delete pxBlend;
		}
	}
}

const AVP_ClientEntity_InteractObject::AnimData AVP_ClientEntity_InteractObject::GetInteractionData( const AVP_ClientEntity_Actor& xActor, const bool bOn ) const
{
	AnimData xAnimData;
	const Asura_Blueprint* pxMyBp = GetBlueprint();
	if ( pxMyBp )
	{
		Asura_Hash_ID uAnimBlueprintParamHash = ASURA_HASH_ID_UNSET;
	
		AVP_Settings::Species eSpecies = xActor.GetSpeciesByClassification();
		const bool bTP = xActor.GetAnimationHandler() ? xActor.GetAnimationHandler()->IsThirdPerson() : false;
		switch ( eSpecies )
		{
			case AVP_Settings::AVP_SPECIES_ALIEN: { uAnimBlueprintParamHash = bTP ? uAVP_BLUEPRINT_PARAM_INTERACTIONANIMBLUEPRINT_ALIEN_TP : uAVP_BLUEPRINT_PARAM_INTERACTIONANIMBLUEPRINT_ALIEN_HUD; } break;
			case AVP_Settings::AVP_SPECIES_HUMAN: { uAnimBlueprintParamHash = bTP ? uAVP_BLUEPRINT_PARAM_INTERACTIONANIMBLUEPRINT_MARINE_TP : uAVP_BLUEPRINT_PARAM_INTERACTIONANIMBLUEPRINT_MARINE_HUD; } break;
			case AVP_Settings::AVP_SPECIES_PREDATOR: { uAnimBlueprintParamHash = bTP ? uAVP_BLUEPRINT_PARAM_INTERACTIONANIMBLUEPRINT_PREDATOR_TP : uAVP_BLUEPRINT_PARAM_INTERACTIONANIMBLUEPRINT_PREDATOR_HUD; } break;
			default: {} break;
		}

		const Asura_Hash_ID uAnimBlueprintHash = uAnimBlueprintParamHash != ASURA_HASH_ID_UNSET ? pxMyBp->GetParameterAsHash( uAnimBlueprintParamHash, ASURA_HASH_ID_UNSET ) : ASURA_HASH_ID_UNSET;
		Asura_Assertf( uAnimBlueprintHash != ASURA_HASH_ID_UNSET, ( "Actor %d doesn't know how to interact with object %d", xActor.GetGuid(), GetGuid() ) );
		if ( uAnimBlueprintHash != ASURA_HASH_ID_UNSET )
		{
			Asura_Blueprint* pxAnimBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( xActor.GetAnimStateBlueprintType(), uAnimBlueprintHash );
			if ( pxAnimBlueprint )
			{
				const char* szOnOffString = bOn ? ".TurnOn" : ".TurnOff";
				Asura_Value xNewValue;
				if ( pxAnimBlueprint->GetParameterValueByHashID( Asura_ConcatenateHashID( 0x16336986, szOnOffString ), xNewValue ) )	// ActorAnim
				{
					xAnimData.m_uActorAnim = xNewValue.GetHashIDValue();
				}
				if ( pxAnimBlueprint->GetParameterValueByHashID( Asura_ConcatenateHashID( 0x51d5162d, szOnOffString ), xNewValue ) )	// WeaponAnim
				{
					xAnimData.m_uWeaponAnim = xNewValue.GetHashIDValue();
				}
				if ( pxAnimBlueprint->GetParameterValueByHashID( Asura_ConcatenateHashID( 0x19ce83d8, szOnOffString ), xNewValue ) )	// ActorPositionMarker
				{
					xAnimData.m_uActorPositionMarkerPoint = xNewValue.GetHashIDValue();
				}
				xAnimData.m_uObjectAnim = bOn ? m_uTurnOnAnimID : m_uTurnOffAnimID;
			}
		}
	}
	return xAnimData;
}

void AVP_ClientEntity_InteractObject::OnInteraction( const bool bOn )
{
	m_bOn = bOn;
}

void AVP_ClientEntity_InteractObject::ProcessClientCreate( const AVP_Message_InteractObject_ClientCreate* pxData, float fServerTime, bool bProcessPosAndOri )
{
	Asura_Assert( pxData, "Going to crash, report me" );
	PARENT::ProcessClientCreate( pxData, fServerTime, bProcessPosAndOri );
	
	if ( pxData->IsUseActorVolume() )
	{
		m_pxActorVolume = new Asura_OrientedVolume();
		m_pxActorVolume->m_xBox.SetToPoint( pxData->GetActorVolumeMinBound() );
		m_pxActorVolume->m_xBox.Set( pxData->GetActorVolumeMaxBound() );
		m_pxActorVolume->m_xOrient = pxData->GetActorVolumeOrient();
	}

	if ( pxData->IsUseOnTargetVolume() )
	{
		Asura_AMatrix3x3 xOri;
		pxData->GetOnTargetVolumeOrient().ToMatrix( xOri );
		m_pxOnTargetVolume = new Asura_OBB( Asura_AVector3( pxData->GetOnTargetVolumeHalfExtents() ), Asura_AVector3( pxData->GetOnTargetVolumeCenter() ), xOri );
	}
	if ( pxData->IsUseOffTargetVolume() )
	{
		Asura_AMatrix3x3 xOri;
		pxData->GetOffTargetVolumeOrient().ToMatrix( xOri );
		m_pxOffTargetVolume = new Asura_OBB( Asura_AVector3( pxData->GetOffTargetVolumeHalfExtents() ), Asura_AVector3( pxData->GetOffTargetVolumeCenter() ), xOri );
	}

	m_bOn               = pxData->IsOn();
	m_bEnabled			= pxData->IsEnabled();
	m_bUseByAlien		= pxData->IsUseByAlien();
	m_bUseByHuman		= pxData->IsUseByHuman();
	m_bUseByPredator	= pxData->IsUseByPredator();
}

void AVP_ClientEntity_InteractObject::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	PARENT::InitialiseFromBlueprint( xBlueprint );

	// Turn On
	Asura_Value xValue;
	if ( xBlueprint.GetParameterValueByHashID( 0xb23400b0, xValue ) )	//Anim.OnRest
	{
		m_uOnRestAnimID = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0x8601eae6, xValue ) )	//Anim.OffRest
	{
		m_uOffRestAnimID = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0xbb1f0199, xValue ) )	//Anim.TurnOn
	{
		m_uTurnOnAnimID = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0xa8c130f5, xValue ) )	//Anim.TurnOff
	{
		m_uTurnOffAnimID = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0x335ACE73, xValue ) )	//Asura_GetHashID( "PredatorInFocusMode.PFX"); 
	{
		m_uPredatorFocusModePFXHash = xValue.GetHashIDValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0xAD3A81F7, xValue ) )	//Asura_GetHashID( "PredatorInFocusMode.MarkerHash"); 
	{
		m_uPredatorFocusModePFXMarkerPoint = xValue.GetHashIDValue();
	}
}

void AVP_ClientEntity_InteractObject::UpdatePFXs()
{
	Asura_Animation* pxAnimation = GetAnimation();
	if( pxAnimation==NULL )
	{
		// There is no point to continue, if we don't have animation.
		return;
	}

	// Update PFX for predator in focus mode. Each interactive object, can have special PFX effect,when predator is in interactive mode and object is Enabled.
	if( m_uPredatorFocusModePFXHash != ASURA_HASH_ID_UNSET && m_uPredatorFocusModePFXMarkerPoint != ASURA_HASH_ID_UNSET )
	{

		const bool bIsPredator = (AVP_Settings::GetPlayerSpecies() == AVP_Settings::AVP_SPECIES_PREDATOR ) ? true : false;
		const bool bInFocusMode = AVP_Vertebrate_HUDHandler::GetFocusMode()>=0.5f ? true : false;
		const bool bShowPfx = (bInFocusMode && IsEnabled() && bIsPredator) ? true : false;
		
		if( bShowPfx && m_pxPredatorFocusModePFXAttachment==NULL )
		{
			if( pxAnimation->HasMarker(m_uPredatorFocusModePFXMarkerPoint) )
			{
				const u_short usPFXFlags = ASURA_PFX_EFFECT_CAMERA_FLAGS;

				m_pxPredatorFocusModePFXAttachment = new Asura_AnimationAttachment_PFX( m_uPredatorFocusModePFXHash, usPFXFlags);
				if ( m_pxPredatorFocusModePFXAttachment )
				{
					bool bLinkSucceeded = m_pxPredatorFocusModePFXAttachment->LinkAttachment( pxAnimation, m_uPredatorFocusModePFXMarkerPoint, true);
					if( bLinkSucceeded )
					{
						m_pxPredatorFocusModePFXAttachment->SetDeleteWhenFinished(false);
					}
					else
					{
						// Show two different asserts, depending on the fact that Marker Point exist or not
						Asura_Assertf( false, ("Failed to attach PFX to object Guid: %d, when predator is in focus mode. Trying to attach this PFX %x, to this marker point: %x.", GetGuid(), m_uPredatorFocusModePFXHash, m_uPredatorFocusModePFXMarkerPoint) );
						m_pxPredatorFocusModePFXAttachment = NULL;
					}
				}
			}
			else	// !HasMarkerPoint
			{
				Asura_Assertf(  false, ("Failed to attach PFX to object Guid: %d, when the predator is in the focus mode. Trying to attach this PFX %x, to this marker point: %x. Attaching failed because this object doesn't have the marker point. You must add this marker point or change blueprint value 'PhysicalObjects/InteractiveObject/PredatorInFocusMode.MarkerHash' to refer to correct Marker Point.", GetGuid(), m_uPredatorFocusModePFXHash, m_uPredatorFocusModePFXMarkerPoint) );
			}
		}
		else if( bShowPfx==false && m_pxPredatorFocusModePFXAttachment )
		{
			// Destroy PFX.
			if( m_pxPredatorFocusModePFXAttachment )
			{
				if( m_pxPredatorFocusModePFXAttachment->IsLinked() )
				{	
					m_pxPredatorFocusModePFXAttachment->UnlinkAttachment(true);
				}
				else
				{
					delete m_pxPredatorFocusModePFXAttachment;
				}

				m_pxPredatorFocusModePFXAttachment = NULL;
			}
		}
	}
}

// end of file
