//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_InteractObject.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef AVP_CLIENTENTITY_INTERACTOBJECT_H
#define AVP_CLIENTENTITY_INTERACTOBJECT_H

#include "Blueprint/Asura_Blueprint.h"

// Project independent.
#include "Entity/PhysicalObject/AVP_ClientEntity_PhysicalObject.h"

// Forward declarations
class Asura_AnimationAttachment_PFX;
class Asura_OBB;
class Asura_OrientedVolume;
class AVP_ClientEntity_Actor;
class AVP_Message_InteractObject_ClientCreate;

class AVP_ClientEntity_InteractObject : public AVP_ClientEntity_PhysicalObject
{
public:
	enum InteractionType
	{
		INTERACTION_TYPE_DEFAULT,
		INTERACTION_TYPE_PREDATOR_LOCK,
		INTERACTION_TYPE_TOOLRECEPTACLE,
		INTERACTION_TYPE_BATTERYRECEPTACLE,
		INTERACTION_TYPE_HACKINGPOINT,
		INTERACTION_TYPE_RETINALSCANNER,
		INTERACTION_TYPE_SENTRYGUNINVISISWITCH
	};

	struct AnimData
	{
		AnimData() : m_uActorAnim( ASURA_HASH_ID_UNSET ), m_uObjectAnim( ASURA_HASH_ID_UNSET ), m_uWeaponAnim( ASURA_HASH_ID_UNSET ), m_uActorPositionMarkerPoint( ASURA_HASH_ID_UNSET ) {}

		Asura_Hash_ID	m_uActorAnim;
		Asura_Hash_ID	m_uObjectAnim;
		Asura_Hash_ID	m_uWeaponAnim;
		Asura_Hash_ID	m_uActorPositionMarkerPoint;
	};

	static void															Create( const Asura_Message* pxMsg );
	static AVP_ClientEntity_InteractObject*								FindEntity( Asura_Guid uGuid )				{ return s_xList.FindEntity( uGuid ); }
    static const Asura_Hashed_List<AVP_ClientEntity_InteractObject>*	GetList()			{ return &s_xList; }

	virtual bool	HandleMessage(const Asura_Message* pxMessage);
	virtual void	PostCreateInitialise();

	const Asura_OrientedVolume*	GetActorVolume() const { return m_pxActorVolume; }
	const Asura_OBB*			GetTargetVolume() const { return m_bOn ? m_pxOnTargetVolume : m_pxOffTargetVolume; }

	void            SetOn(bool  bOn ) { m_bOn = bOn; }
	bool			IsOn() const { return m_bOn; }
	bool			IsEnabled() const { return m_bEnabled; }
	bool			WillInteractWith( u_int uClassification ) const;
	void			PlayAnimation( const Asura_Hash_ID uAnimHash );
		
	virtual const AnimData	GetInteractionData( const AVP_ClientEntity_Actor& xActor, const bool bOn ) const;
	virtual void			OnInteraction( const bool bOn );
	virtual InteractionType	GetInteractionObjectType() const { return INTERACTION_TYPE_DEFAULT; };
	virtual bool	        IsActorUsingRightInteractionItem(const AVP_ClientEntity_Actor& xActor) const;

	void			UpdatePFXs();

protected:
	AVP_ClientEntity_InteractObject();
	virtual ~AVP_ClientEntity_InteractObject();

	void    ProcessClientCreate( const AVP_Message_InteractObject_ClientCreate* pxData, float fServerTime, bool bProcessPosAndOri );

	virtual void	InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );

	Asura_OrientedVolume*	m_pxActorVolume;
	Asura_OBB*				m_pxOnTargetVolume;
	Asura_OBB*				m_pxOffTargetVolume;

	Asura_Hash_ID			m_uOnRestAnimID;
	Asura_Hash_ID			m_uOffRestAnimID;
	Asura_Hash_ID			m_uTurnOnAnimID;
	Asura_Hash_ID			m_uTurnOffAnimID;

	union
	{
		u_int		m_uFlags;
		struct  
		{
			u_int	m_bOn						: 1,
					m_bEnabled					: 1,
					m_bUseByAlien				: 1,
					m_bUseByHuman				: 1,
					m_bUseByPredator			: 1,
												: 0;
		};
	};

	Asura_Hash_ID			m_uPredatorFocusModePFXHash;
	Asura_Hash_ID			m_uPredatorFocusModePFXMarkerPoint;
	Asura_AnimationAttachment_PFX* m_pxPredatorFocusModePFXAttachment;

private :
	typedef AVP_ClientEntity_PhysicalObject PARENT;

	static Asura_Hashed_List<AVP_ClientEntity_InteractObject> s_xList;
};

#endif // #ifndef AVP_CLIENTENTITY_INTERACTOBJECT_H
