//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_InteractPredatorLock.h
//
// Created by AJK 21/10/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Interact/PredatorLock/AVP_ServerEntity_InteractPredatorLock.h"

// Asura
#include "Entity/Asura_StaticMessage_System.h"

//AVP
#include "Entity/PhysicalObject/Interact/PredatorLock/AVP_Messages_InteractPredatorLock.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

Asura_Hashed_List<AVP_ServerEntity_InteractPredatorLock> AVP_ServerEntity_InteractPredatorLock::s_xList;

AVP_ServerEntity_InteractPredatorLock::AVP_ServerEntity_InteractPredatorLock( Asura_Guid uGuid ) 
: Asura_Entity_Server( uGuid )
, PARENT( uGuid )
, m_usReachPosition1MessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_usReachPosition2MessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_usReachPosition3MessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_usReachPosition4MessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_usLeavePosition1MessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_usLeavePosition2MessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_usLeavePosition3MessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_usLeavePosition4MessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_cLockCurrentPosition(1)
{
	Classification = AVP_ENTITYCLASS_INTERACT_PREDATOR_LOCK;

	s_xList.AddToList(this);
}

AVP_ServerEntity_InteractPredatorLock::~AVP_ServerEntity_InteractPredatorLock()
{
	s_xList.RemoveFromList(this);
}

void AVP_ServerEntity_InteractPredatorLock::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iCurrentChunkVersion;

	xStream >> iCurrentChunkVersion;

	switch ( iCurrentChunkVersion )
	{
		case 0 :
		{
			PARENT::ReadFromChunkStream(xStream);

			xStream >> m_cLockCurrentPosition;
			break;
		}
		case CURRENT_CHUNK_VERSION :
		{
			PARENT::ReadFromChunkStream(xStream);

			xStream >> m_usReachPosition1MessageBlock;
			xStream >> m_usReachPosition2MessageBlock;
			xStream >> m_usReachPosition3MessageBlock;
			xStream >> m_usReachPosition4MessageBlock;

			xStream >> m_usLeavePosition1MessageBlock;
			xStream >> m_usLeavePosition2MessageBlock;
			xStream >> m_usLeavePosition3MessageBlock;
			xStream >> m_usLeavePosition4MessageBlock;

			xStream >> m_cLockCurrentPosition;
			break;
		}
		default :
		{
			Asura_Assertf(false, ("Unknown chunk version (%d) in AVP_ServerEntity_InteractPredatorLock", iCurrentChunkVersion));
			break;
		}
	}
}

void AVP_ServerEntity_InteractPredatorLock::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << CURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_usReachPosition1MessageBlock;
	xStream << m_usReachPosition2MessageBlock;
	xStream << m_usReachPosition3MessageBlock;
	xStream << m_usReachPosition4MessageBlock;

	xStream << m_usLeavePosition1MessageBlock;
	xStream << m_usLeavePosition2MessageBlock;
	xStream << m_usLeavePosition3MessageBlock;
	xStream << m_usLeavePosition4MessageBlock;

	xStream << m_cLockCurrentPosition;
}

void AVP_ServerEntity_InteractPredatorLock::MakeCreateMessage( Asura_Guid uRequestingClient, bool bSendGuaranteed /*= false*/ )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTCREATE, sizeof( AVP_Message_InteractPredatorLock_ClientCreate ), GetClassification());
	if(pxMessage)
	{
		AVP_Message_InteractPredatorLock_ClientCreate* pxData = static_cast< AVP_Message_InteractPredatorLock_ClientCreate* >( pxMessage->GetData() );
		pxData->FillInteractionObject(this);
 
		pxMessage->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );
		pxMessage->SetFromGuid( GetGuid() );
		if (bSendGuaranteed)
		{
			pxMessage->UseGuaranteedDelivery();
		}
		Asura_Network::SendToSpecificMachine( pxMessage, uRequestingClient );
	}
}

#ifdef ASURA_TOOLS

void AVP_ServerEntity_InteractPredatorLock::CreateFromTools(Asura_Guid uGuid)
{
	new AVP_ServerEntity_InteractPredatorLock(uGuid);
}

bool AVP_ServerEntity_InteractPredatorLock::CanBeIncludedInTemplate() const
{
	return (GetClassification() == AVP_ENTITYCLASS_INTERACT_PREDATOR_LOCK);
}

void AVP_ServerEntity_InteractPredatorLock::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance(pxInstance);

	m_usReachPosition1MessageBlock = pxInstance->ConvertMessageBlock( m_usReachPosition1MessageBlock );
	m_usReachPosition2MessageBlock = pxInstance->ConvertMessageBlock( m_usReachPosition2MessageBlock );
	m_usReachPosition3MessageBlock = pxInstance->ConvertMessageBlock( m_usReachPosition3MessageBlock );
	m_usReachPosition4MessageBlock = pxInstance->ConvertMessageBlock( m_usReachPosition4MessageBlock );

	m_usLeavePosition1MessageBlock = pxInstance->ConvertMessageBlock( m_usLeavePosition1MessageBlock );
	m_usLeavePosition2MessageBlock = pxInstance->ConvertMessageBlock( m_usLeavePosition2MessageBlock );
	m_usLeavePosition3MessageBlock = pxInstance->ConvertMessageBlock( m_usLeavePosition3MessageBlock );
	m_usLeavePosition4MessageBlock = pxInstance->ConvertMessageBlock( m_usLeavePosition4MessageBlock );
}
#endif // #ifdef ASURA_TOOLS

#ifdef ASURA_TOOLS
void AVP_ServerEntity_InteractPredatorLock::GetCustomEventName(int iEventIndex, Asura_String& xEventName)
{
	switch(iEventIndex)
	{
		case MESSAGE_BLOCK_REACH_POS1:		xEventName = "On Reach Position 1";		return;
		case MESSAGE_BLOCK_REACH_POS2:		xEventName = "On Reach Position 2";		return;
		case MESSAGE_BLOCK_REACH_POS3:		xEventName = "On Reach Position 3";		return;
		case MESSAGE_BLOCK_REACH_POS4:		xEventName = "On Reach Position 4";		return;

		case MESSAGE_BLOCK_LEAVE_POS1:		xEventName = "On Leave Position 1";		return;
		case MESSAGE_BLOCK_LEAVE_POS2:		xEventName = "On Leave Position 2";		return;
		case MESSAGE_BLOCK_LEAVE_POS3:		xEventName = "On Leave Position 3";		return;
		case MESSAGE_BLOCK_LEAVE_POS4:		xEventName = "On Leave Position 4";		return;
	}

	Asura_Assert(FALSE, "Invalid predator lock custom event index");

	xEventName = "";
}

unsigned short* AVP_ServerEntity_InteractPredatorLock::GetCustomEvent(int iIndex)
{
	switch(iIndex)
	{
		case MESSAGE_BLOCK_REACH_POS1:		return &m_usReachPosition1MessageBlock;
		case MESSAGE_BLOCK_REACH_POS2:		return &m_usReachPosition2MessageBlock;
		case MESSAGE_BLOCK_REACH_POS3:		return &m_usReachPosition3MessageBlock;
		case MESSAGE_BLOCK_REACH_POS4:		return &m_usReachPosition4MessageBlock;

		case MESSAGE_BLOCK_LEAVE_POS1:		return &m_usLeavePosition1MessageBlock;
		case MESSAGE_BLOCK_LEAVE_POS2:		return &m_usLeavePosition2MessageBlock;
		case MESSAGE_BLOCK_LEAVE_POS3:		return &m_usLeavePosition3MessageBlock;
		case MESSAGE_BLOCK_LEAVE_POS4:		return &m_usLeavePosition4MessageBlock;
	}

	Asura_Assert(FALSE, "Invalid predator lock custom event index");

	return NULL;
}
#endif // ASURA_TOOLS

void AVP_ServerEntity_InteractPredatorLock::OnReachPosition1()
{
	if ( m_usReachPosition1MessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usReachPosition1MessageBlock, GetGuid() );
	}
}
void AVP_ServerEntity_InteractPredatorLock::OnReachPosition2()
{
	if ( m_usReachPosition2MessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usReachPosition2MessageBlock, GetGuid() );
	}
}
void AVP_ServerEntity_InteractPredatorLock::OnReachPosition3()
{
	if ( m_usReachPosition3MessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usReachPosition3MessageBlock, GetGuid() );
	}
}
void AVP_ServerEntity_InteractPredatorLock::OnReachPosition4()
{
	if ( m_usReachPosition4MessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usReachPosition4MessageBlock, GetGuid() );
	}
}

void AVP_ServerEntity_InteractPredatorLock::OnLeavePosition1()
{
	if ( m_usLeavePosition1MessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usLeavePosition1MessageBlock, GetGuid() );
	}
}
void AVP_ServerEntity_InteractPredatorLock::OnLeavePosition2()
{
	if ( m_usLeavePosition2MessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usLeavePosition2MessageBlock, GetGuid() );
	}

}
void AVP_ServerEntity_InteractPredatorLock::OnLeavePosition3()
{
	if ( m_usLeavePosition3MessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usLeavePosition3MessageBlock, GetGuid() );
	}
}
void AVP_ServerEntity_InteractPredatorLock::OnLeavePosition4()
{
	if ( m_usLeavePosition4MessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usLeavePosition4MessageBlock, GetGuid() );
	}
}

void AVP_ServerEntity_InteractPredatorLock::PostLoadInitialise()
{
	PARENT::PostLoadInitialise();
	switch(m_cLockCurrentPosition)
	{
	case 1:
		{
			break;
		}
	case 2:
		{
			OnLeavePosition1();
			OnReachPosition2();
			break;
		}
	case 3:
		{
			OnLeavePosition1();
			OnReachPosition2();
			OnLeavePosition2();
			OnReachPosition3();
			break;
		}
	case 4:
		{
			OnLeavePosition1();
			OnReachPosition2();
			OnLeavePosition2();
			OnReachPosition3();
			OnLeavePosition3();
			OnReachPosition4();
			break;
		}
	default:
		{
			Asura_Assert( false , "Invalid lock position!" );
			break;
		}
	}
}

bool AVP_ServerEntity_InteractPredatorLock::HandleMessage(const Asura_Message* pxMsg)
{
	switch ( pxMsg->GetMsgID() )
	{
		case AVP_MSGID_REQUEST_INTERACTION:
		{
			// Actor is requesting interaction, do it
			const AVP_Message_Actor_RequestInteraction*	pxData = static_cast< const AVP_Message_Actor_RequestInteraction* >( pxMsg->GetData() );
			if ( pxData )
			{
				Asura_Assert( pxData->GetInteractionType() == AVP_INTERACTION_TYPE_DEFAULT, "Interaction objects only handle default interactions" );
				if( pxData->GetInteractionMoveType() ==  AVP_Message_Actor_RequestInteraction::INTERACTION_RIGHT)
				{
					SendLeavePositionMessage();
					( m_cLockCurrentPosition < 4 ) ? ++m_cLockCurrentPosition : m_cLockCurrentPosition = 1;
					SendReachPositionMessage();
				}
				else if (pxData->GetInteractionMoveType() ==  AVP_Message_Actor_RequestInteraction::INTERACTION_LEFT)
				{
					SendLeavePositionMessage();
					( m_cLockCurrentPosition > 1 ) ? --m_cLockCurrentPosition : m_cLockCurrentPosition = 4;
					SendReachPositionMessage();
				}
				Asura_Assert( (m_cLockCurrentPosition >= 1 && m_cLockCurrentPosition <= 4), "No data" );

				OnTurnOn();
				// Forward on to the client entities ( ignored by the initiator as the move will have already been started to reduce lag )
				Asura_Network::Forward( pxMsg, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );

				return true;
			}
			else
			{
				Asura_Assert( false, "No data" );
				return false;
			}
		}
		case AVP_MSGID_PREDATOR_LOCK_POSITION_F:
		{
    		m_cLockCurrentPosition = 1;
			Asura_Network::Forward( pxMsg, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
			return true; 
		}
		case AVP_MSGID_PREDATOR_LOCK_POSITION_X:
		{
			OnLeavePosition1();
			OnReachPosition2();
			OnLeavePosition2();
			OnReachPosition3();
			OnLeavePosition3();
			OnReachPosition4();
			m_cLockCurrentPosition = 4;
			Asura_Network::Forward( pxMsg, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
			return true; 
		}
		case AVP_MSGID_PREDATOR_LOCK_POSITION_T:
		{
			OnLeavePosition1();
			OnReachPosition2();
			OnLeavePosition2();
			OnReachPosition3();
			m_cLockCurrentPosition = 3;
			Asura_Network::Forward( pxMsg, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
			return true; 
		}
		case AVP_MSGID_PREDATOR_LOCK_POSITION_O:
		{
			OnLeavePosition1();
			OnReachPosition2();
			m_cLockCurrentPosition = 2;
			Asura_Network::Forward( pxMsg, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
			return true; 
		}

	default:
		return PARENT::HandleMessage(pxMsg);
	}
}

void AVP_ServerEntity_InteractPredatorLock::SendLeavePositionMessage()
{
	switch(m_cLockCurrentPosition)
	{
	case 1:
		{
			OnLeavePosition1();
			break;
		}
	case 2:
		{
			OnLeavePosition2();
			break;
		}
	case 3:
		{
			OnLeavePosition3();
			break;
		}
	case 4:
		{
			OnLeavePosition4();
			break;
		}
	default:
		{
			Asura_Assert( false , "Invalid lock position!" );
			break;
		}
	}
}

void AVP_ServerEntity_InteractPredatorLock::SendReachPositionMessage()
{
	switch(m_cLockCurrentPosition)
	{
	case 1:
		{
			OnReachPosition1();
			break;
		}
	case 2:
		{
			OnReachPosition2();
			break;
		}
	case 3:
		{
			OnReachPosition3();
			break;
		}
	case 4:
		{
			OnReachPosition4();
			break;
		}
	default:
		{
			Asura_Assert( false , "Invalid lock position!" );
			break;
		}
	}
}