//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_InteractToolReceptacle.h
//
// Created by AJK 25/11/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Entity/PhysicalObject/Interact/ToolReceptacle/AVP_ClientEntity_InteractToolReceptacle.h"

//ASURA
#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Network/Asura_Message.h"

// AVP
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Pickup/AVP_ClientEntity_Pickup.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/AVP_Messages_InteractToolReceptacle.h"
#include "Entity/AVP_Messages.h"
#include "Moves/AVP_Move_Interaction.h"
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesInteraction.h"
#include "Moves/Vertebrates/Human/AVP_Human_MovesInteraction.h"

Asura_Hashed_List<AVP_ClientEntity_InteractToolReceptacle> AVP_ClientEntity_InteractToolReceptacle::s_xList;

AVP_ClientEntity_InteractToolReceptacle::AVP_ClientEntity_InteractToolReceptacle()
: m_uPickupUseAnim( ASURA_HASH_ID_UNSET )
, m_uPickupRestAnim( ASURA_HASH_ID_UNSET )
, m_bToolInside(false)
{
	Classification = AVP_ENTITYCLASS_INTERACT_TOOL_RECEPTACLE;

	s_xList.AddToList( this );
}

AVP_ClientEntity_InteractToolReceptacle::~AVP_ClientEntity_InteractToolReceptacle() 
{
	s_xList.RemoveFromList( this );
}

void AVP_ClientEntity_InteractToolReceptacle::ProcessClientCreate( const AVP_Message_InteractToolReceptacle_ClientCreate* pxData, float fServerTime, bool bProcessPosAndOri )
{
	m_bToolInside = pxData->IsToolInside();
	PARENT::ProcessClientCreate(pxData, fServerTime, bProcessPosAndOri);
	// Play rest Anim on pickup
    if ( m_bToolInside ) PlayToolAnim(pxData->GetToolGuid(), m_uPickupRestAnim);
}

bool AVP_ClientEntity_InteractToolReceptacle::HandleMessage( const Asura_Message* pxMsg )
{
	switch ( pxMsg->GetMsgID() )
	{
	case AVP_MSGID_SET_TOOL_IN:
		{
			m_bToolInside = true;
			return true;
		}
	case AVP_MSGID_SET_TOOL_OUT:
		{
			m_bToolInside = false;
			return true;
		}
	case AVP_MSGID_REQUEST_INTERACTION:
		{
			// requesting interaction, do it
			const AVP_Message_Actor_RequestInteraction*	pxData = static_cast< const AVP_Message_Actor_RequestInteraction* >( pxMsg->GetData() );
			Asura_Assert( pxData, "No data" );
			if ( pxData )
			{
				if ( pxData->GetActorGuid() != ASURA_GUID_UNREGISTERED )
				{
					AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( pxData->GetActorGuid() );
					if ( pxActor)
					{
						Asura_Message* pxMoveMessage = Asura_Network::CreateMessage( ASURA_MSG_MOVE_START, sizeof( AVP_Message_InteractionMoveStart ) );
						Asura_Assert( pxMoveMessage , "unable to create message" );
						if ( pxMoveMessage  != NULL )
						{
							// Play Pickup anim
							PlayToolAnim(GetToolGuidFromActor(*pxActor), GetToolAnimRelatedToMove(pxData->GetMoveHash()));

							AVP_Message_InteractionMoveStart* pxMoveData = static_cast< AVP_Message_InteractionMoveStart* >( pxMoveMessage->GetData() );

							// Fill-in the message-body...
							pxMoveMessage->SetFromGuid( GetGuid() );
							pxMoveMessage->SetToGuid(  pxData->GetActorGuid() );

							pxMoveData->ClearFlags();
							if (   pxData->GetMoveHash() == AVP_Human_Move_Interaction_PlugBattery::GetStaticHash() 
								|| pxData->GetMoveHash() == AVP_Alien_Move_Interaction_DestroyBattery::GetStaticHash() )
							{
								pxMoveData->SetMoveID( pxData->GetMoveHash() );
							}
							else
							{
								pxMoveData->SetMoveID( AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() );
								pxMoveData->SetNextMoveHash( pxData->GetMoveHash() );
							}

							pxMoveData->SetOn( !pxData->IsOn() );
							pxMoveData->SetIgnoredByOwner( false );
							pxMoveData->SetInteractionObject( GetGuid() );
							// Forward the message directly to the client... we will do this on each client on reciept of the interact message
							bool bOK = pxActor->HandleMessage( pxMoveMessage );
							Asura_Network::DestroyMessage( pxMoveMessage );
							return bOK;
						}
					}
					// no move message or no actor
					return false;
				}
				// We're probably on the initiating client so we don't want to start the interaction again. or there's no actor guid
				return true;
			}
			// no data
			return false;
		}

	default:
		return PARENT::HandleMessage(pxMsg);
	}
}

void AVP_ClientEntity_InteractToolReceptacle::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	PARENT::InitialiseFromBlueprint( xBlueprint );

	// Pickup Use anim
	Asura_Value xValue;
	if ( xBlueprint.GetParameterValueByHashID( 0x5D50DE1C, xValue ) )	//PickupUseAnim
	{
		m_uPickupUseAnim = xValue.GetHashIDValue();
	}

	if ( xBlueprint.GetParameterValueByHashID( 0x4979E261, xValue ) )	//PickupRestAnim
	{
		m_uPickupRestAnim = xValue.GetHashIDValue();
	}
}

void AVP_ClientEntity_InteractToolReceptacle::PlayToolAnim(const Asura_Guid uPickupGuid, Asura_Hash_ID uAnim)
{
	AVP_ClientEntity_Pickup* pxPickup = AVP_ClientEntity_Pickup::FindEntity(uPickupGuid);
	if(pxPickup)
	{	
		if ( uAnim != ASURA_HASH_ID_UNSET )
		{
			// Play animation
			Asura_Animation* pxAnimation = pxPickup->GetShapeInstance().GetAnimation();
			if ( pxAnimation )
			{
				Asura_AnimationController_Animation* pxAACA = pxAnimation->CreateControllerAnimation( AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE, 0, uAnim, NULL );
				if ( pxAACA )
				{
					pxAACA->Play();
				}
			}
		}
	}
}

Asura_Hash_ID AVP_ClientEntity_InteractToolReceptacle::GetToolAnimRelatedToMove(Asura_Hash_ID uMoveHash)
{
	return m_uPickupRestAnim;
}
// end of file
