//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_InteractHackingPoint.h
//
// Created by AJK 04/12/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Entity/PhysicalObject/Interact/ToolReceptacle/HackingPoint/AVP_ClientEntity_InteractHackingPoint.h"

//ASURA
#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Network/Asura_Message.h"

// AVP
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/HackingPoint/AVP_Messages_InteractHackingPoint.h"
#include "Entity/PhysicalObject/Pickup/AVP_ClientEntity_Pickup.h"
#include "Entity/AVP_Messages.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_ItemTypes.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Moves/AVP_Move_Interaction.h"

Asura_Hashed_List<AVP_ClientEntity_InteractHackingPoint> AVP_ClientEntity_InteractHackingPoint::s_xList;

AVP_ClientEntity_InteractHackingPoint::AVP_ClientEntity_InteractHackingPoint()
: m_uMarineInsertHackingToolHUDAnimID( ASURA_HASH_ID_UNSET )
, m_uMarineRemoveHackingToolHUDAnimID( ASURA_HASH_ID_UNSET )
, m_uMarineInsertHackingToolTPAnimID( ASURA_HASH_ID_UNSET )
, m_uMarineRemoveHackingToolTPAnimID( ASURA_HASH_ID_UNSET )
, m_uMarineLetGoHackingToolHUDAnimID( ASURA_HASH_ID_UNSET )
, m_uMarineLetGoHackingToolTPAnimID( ASURA_HASH_ID_UNSET )
, m_uMarineGrabHackingToolHUDAnimID( ASURA_HASH_ID_UNSET )
, m_uMarineGrabHackingToolTPAnimID( ASURA_HASH_ID_UNSET )
{
	Classification = AVP_ENTITYCLASS_INTERACT_HACKING_POINT;

	s_xList.AddToList( this );
}

AVP_ClientEntity_InteractHackingPoint::~AVP_ClientEntity_InteractHackingPoint() 
{
	s_xList.RemoveFromList( this );
}

void AVP_ClientEntity_InteractHackingPoint::Create( const Asura_Message* pxMsg )
{
	Asura_Assert( pxMsg->GetMsgID() == ASURA_MSG_CLIENTCREATE, "" );
	Asura_Assert( pxMsg->GetEntityClassification() == AVP_ENTITYCLASS_INTERACT_HACKING_POINT, "" );

	const AVP_Message_InteractHackingPoint_ClientCreate* pxData = static_cast< const AVP_Message_InteractHackingPoint_ClientCreate* >( pxMsg->GetData() );

	AVP_ClientEntity_InteractHackingPoint* pxNew = new AVP_ClientEntity_InteractHackingPoint();
	pxNew->ProcessClientCreate(  pxData, pxMsg->GetSentTime(), true );
}

void AVP_ClientEntity_InteractHackingPoint::ProcessClientCreate( const AVP_Message_InteractHackingPoint_ClientCreate* pxData, float fServerTime, bool bProcessPosAndOri )
{
	PARENT::ProcessClientCreate(pxData, fServerTime, bProcessPosAndOri);
}

void AVP_ClientEntity_InteractHackingPoint::GetMarineInsertHackingToolAnimHashes( Asura_Hash_ID& uHUDAnimHash, Asura_Hash_ID& uTPAnimHash ) const
{
	uHUDAnimHash = m_uMarineInsertHackingToolHUDAnimID;
	uTPAnimHash  = m_uMarineInsertHackingToolTPAnimID;
}

void AVP_ClientEntity_InteractHackingPoint::GetMarineRemoveHackingToolAnimHashes( Asura_Hash_ID& uHUDAnimHash, Asura_Hash_ID& uTPAnimHash ) const
{
	uHUDAnimHash = m_uMarineRemoveHackingToolHUDAnimID;
	uTPAnimHash  = m_uMarineRemoveHackingToolTPAnimID;
}

void AVP_ClientEntity_InteractHackingPoint::GetMarineLetGoHackingToolAnimHashes( Asura_Hash_ID& uHUDAnimHash, Asura_Hash_ID& uTPAnimHash ) const
{
	uHUDAnimHash = m_uMarineLetGoHackingToolHUDAnimID;
	uTPAnimHash  = m_uMarineLetGoHackingToolTPAnimID;
}

void AVP_ClientEntity_InteractHackingPoint::GetMarineGrabHackingToolAnimHashes( Asura_Hash_ID& uHUDAnimHash, Asura_Hash_ID& uTPAnimHash ) const
{
	uHUDAnimHash = m_uMarineGrabHackingToolHUDAnimID;
	uTPAnimHash  = m_uMarineGrabHackingToolTPAnimID;
}

void AVP_ClientEntity_InteractHackingPoint::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	PARENT::InitialiseFromBlueprint( xBlueprint );

	Asura_Value xValue;

	if ( xBlueprint.GetParameterValueByHashID( 0x2158D487, xValue ) )	//Asura_GetHashID( "Anim.Interaction.PlugHackingTool.Marine.HUD"); 
	{
		m_uMarineInsertHackingToolHUDAnimID = xValue.GetHashIDValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0x9DFA9CEC, xValue ) )	//Asura_GetHashID( "Anim.Interaction.PlugHackingTool.Marine.TP"); 
	{
		m_uMarineInsertHackingToolTPAnimID = xValue.GetHashIDValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0x0BECD3A0, xValue ) )	//Asura_GetHashID( "Anim.Interaction.UnplugHackingTool.Marine.HUD"); 
	{
		m_uMarineRemoveHackingToolHUDAnimID = xValue.GetHashIDValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0xADCDD6B3, xValue ) )	//Asura_GetHashID( "Anim.Interaction.UnplugHackingTool.Marine.TP"); 
	{
		m_uMarineRemoveHackingToolTPAnimID = xValue.GetHashIDValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0x880068EA, xValue ) )	//Asura_GetHashID( "Anim.Interaction.LetGoHackingTool.Marine.HUD"); 
	{
		m_uMarineLetGoHackingToolHUDAnimID = xValue.GetHashIDValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0x2DAD7029, xValue ) )	//Asura_GetHashID( "Anim.Interaction.LetGoHackingTool.Marine.TP"); 
	{
		m_uMarineLetGoHackingToolTPAnimID = xValue.GetHashIDValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0x06BF8EC5, xValue ) )	//Asura_GetHashID( "Anim.Interaction.GrabHackingTool.Marine.HUD"); 
	{
		m_uMarineGrabHackingToolHUDAnimID = xValue.GetHashIDValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( 0x73D4A2EE, xValue ) )	//Asura_GetHashID( "Anim.Interaction.GrabHackingTool.Marine.TP"); 
	{
		m_uMarineGrabHackingToolTPAnimID = xValue.GetHashIDValue();
	}
}

bool AVP_ClientEntity_InteractHackingPoint::IsActorUsingRightInteractionItem(const AVP_ClientEntity_Actor& xActor) const
{
	if(m_bToolInside) return true;        // HackingTool inside

	// See if human has the hacking tool
	// Must be human (Predator and alien can't use the hacking tool, so it's save to downcast to human (Designer must specify
	const unsigned short uClass = xActor.GetClassification();
	if( uClass == AVP_ENTITYCLASS_HUMAN || uClass == AVP_ENTITYCLASS_MARINE || uClass == AVP_ENTITYCLASS_CIVILIAN )
	{
		const AVP_ServerActor_Human* pxHuman = AVP_ServerActor_Human::FindEntity(xActor.GetGuid());
		if(pxHuman)	
		{
			return ( pxHuman->GetHasHackingTool() );
		}
	}
	return false;
}

Asura_Guid AVP_ClientEntity_InteractHackingPoint::GetToolGuidFromActor(const AVP_ServerEntity_Actor& xActor ) const
{
	// Only human use hacking tool
	// Get the client, must be human
	if (   xActor.GetClassification() == AVP_ENTITYCLASS_CIVILIAN
		|| xActor.GetClassification() == AVP_ENTITYCLASS_HUMAN
		|| xActor.GetClassification() == AVP_ENTITYCLASS_MARINE )
	{
		return (static_cast<const AVP_ServerActor_Human&> (xActor)).GetHackingToolGuid();
	}
	return ASURA_GUID_UNREGISTERED;
};


// end of file
