//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_InteractHackingPoint.h
//
// Created by AJK 24/11/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Interact/ToolReceptacle/HackingPoint/AVP_ServerEntity_InteractHackingPoint.h"

// Asura
#include "Entity/Asura_StaticMessage_System.h"

//AVP
#include "Entity/PhysicalObject/Interact/ToolReceptacle/HackingPoint/AVP_Messages_InteractHackingPoint.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/HackingPoint/AVP_ClientEntity_InteractHackingPoint.h"
#include "Entity/PhysicalObject/Pickup/AVP_ServerEntity_Pickup.h"
#include "Inventory/AVP_Inventory_HashIDs.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

Asura_Hashed_List<AVP_ServerEntity_InteractHackingPoint> AVP_ServerEntity_InteractHackingPoint::s_xList;

AVP_ServerEntity_InteractHackingPoint::AVP_ServerEntity_InteractHackingPoint( Asura_Guid uGuid ) 
: Asura_Entity_Server( uGuid )
, PARENT( uGuid )
, m_usMarineInsertHackingToolMessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_usMarineRemoveHackingToolMessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
{
	Classification = AVP_ENTITYCLASS_INTERACT_HACKING_POINT;

	s_xList.AddToList(this);

	m_uAttachMarker = 0x95A81014; // "HackingPointAttach"
}

AVP_ServerEntity_InteractHackingPoint::~AVP_ServerEntity_InteractHackingPoint()
{
	s_xList.RemoveFromList(this);
}

void AVP_ServerEntity_InteractHackingPoint::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iCurrentChunkVersion;

	xStream >> iCurrentChunkVersion;

	switch ( iCurrentChunkVersion )
	{
	case CURRENT_CHUNK_VERSION :
		{
			PARENT::ReadFromChunkStream(xStream);
			xStream >> m_usMarineInsertHackingToolMessageBlock;
			xStream >> m_usMarineRemoveHackingToolMessageBlock;
			break;
		}
	default :
		{
			Asura_Assertf(false, ("Unknown chunk version (%d) in AVP_ServerEntity_InteractHackingPoint", iCurrentChunkVersion));
			break;
		}
	}
}

void AVP_ServerEntity_InteractHackingPoint::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << CURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_usMarineInsertHackingToolMessageBlock;
	xStream << m_usMarineRemoveHackingToolMessageBlock;

}

void AVP_ServerEntity_InteractHackingPoint::MakeCreateMessage( Asura_Guid uRequestingClient, bool bSendGuaranteed /*= false*/ )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTCREATE, sizeof( AVP_Message_InteractHackingPoint_ClientCreate ), GetClassification());
	if(pxMessage)
	{
		AVP_Message_InteractHackingPoint_ClientCreate* pxData = static_cast< AVP_Message_InteractHackingPoint_ClientCreate* >( pxMessage->GetData() );
		pxData->FillInteractionObject(this);

		pxMessage->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );
		pxMessage->SetFromGuid( GetGuid() );
		if (bSendGuaranteed)
		{
			pxMessage->UseGuaranteedDelivery();
		}
		Asura_Network::SendToSpecificMachine( pxMessage, uRequestingClient );
	}
}

#ifdef ASURA_TOOLS
void AVP_ServerEntity_InteractHackingPoint::CreateFromTools(Asura_Guid uGuid)
{
	new AVP_ServerEntity_InteractHackingPoint(uGuid);
}

bool AVP_ServerEntity_InteractHackingPoint::CanBeIncludedInTemplate() const
{
	return (GetClassification() == AVP_ENTITYCLASS_INTERACT_HACKING_POINT);
}

void AVP_ServerEntity_InteractHackingPoint::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance(pxInstance);

	m_usMarineInsertHackingToolMessageBlock = pxInstance->ConvertMessageBlock( m_usMarineInsertHackingToolMessageBlock );
	m_usMarineRemoveHackingToolMessageBlock = pxInstance->ConvertMessageBlock( m_usMarineRemoveHackingToolMessageBlock );
}
#endif // #ifdef ASURA_TOOLS

bool AVP_ServerEntity_InteractHackingPoint::HandleMessage(const Asura_Message* pxMsg)
{
	switch ( pxMsg->GetMsgID() )
	{
	case AVP_MSGID_REQUEST_INTERACTION:
		{
			// Check for Interaction object shadow setting;
			const AVP_Message_Actor_RequestInteraction*	pxData = static_cast< const AVP_Message_Actor_RequestInteraction* >( pxMsg->GetData() );
			if ( pxData )
			{
				Asura_Assert( pxData->GetInteractionType() == AVP_INTERACTION_TYPE_DEFAULT, "Interaction objects only handle default interactions" );
				if( pxData->GetInteractionMoveType() ==  AVP_Message_Actor_RequestInteraction::INTERACTION_INTIATED)
				{
					// Find the species
					AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( pxData->GetActorGuid() );
					{
						if(pxActor)
						{
							if( !IsToolInside() )
							{
								// Attach hacking tool to marine lefthand
								AttachHackingToolToLeftHand(pxData->GetActorGuid(), true );
								// Insert HackingTool
								SendMarineInsertHackingToolMessage();
							}
							else
							{
								// Remove HackingTool
								SendMarineRemoveHackingToolMessage();
							}
						}
						Asura_Network::Forward( pxMsg, ASURA_GUID_MY_CLIENT_ENTITIES, GetGuid() );
						return true;
					}
				}
			}
			else
			{
				Asura_Assert( false, "No data" );
				return false;
			}
			return false;
		}

	default:
		return PARENT::HandleMessage(pxMsg);
	}
}

void AVP_ServerEntity_InteractHackingPoint::SendMarineInsertHackingToolMessage()
{
	if ( m_usMarineInsertHackingToolMessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usMarineInsertHackingToolMessageBlock, GetGuid() );
	}
}

void AVP_ServerEntity_InteractHackingPoint::SendMarineRemoveHackingToolMessage()
{
	if ( m_usMarineRemoveHackingToolMessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usMarineRemoveHackingToolMessageBlock, GetGuid() );
	}
}

void AVP_ServerEntity_InteractHackingPoint::AttachHackingToolToLeftHand(Asura_Guid uActorGuid, bool bHideTool)
{
	AVP_ServerActor_Human* pxActor = AVP_ServerActor_Human::FindEntity( uActorGuid );
	if(pxActor)
	{
		AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::FindEntity(pxActor->GetHackingToolGuid());
		if(pxPickup)
		{	
			bHideTool ? pxPickup->SetHidden(true) : pxPickup->SetHidden(false);

			//// See if interaction object has shadow, if it has, turn it on of the pickup, else turn it off
			//if ( GetPhysicalObjectFlags() & ASURA_PHYSICALOBJECT_FLAG_NOSHADOW )
			//{
			//	pxPickup->SetPhysicalObjectFlags( pxPickup->GetPhysicalObjectFlags() | ASURA_PHYSICALOBJECT_FLAG_NOSHADOW );
			//}
			//else
			//{
			//	pxPickup->SetPhysicalObjectFlags( pxPickup->GetPhysicalObjectFlags() & (~ASURA_PHYSICALOBJECT_FLAG_NOSHADOW) );
			//}

			Asura_Message*	pxMessage	= Asura_Network::CreateMessage(ASURA_MSG_LINK_TO_PARENT, sizeof(Asura_Message_PhysicalObject_LinkToParent));

			if(pxMessage)
			{
				Asura_Message_PhysicalObject_LinkToParent*	pxData	= static_cast<Asura_Message_PhysicalObject_LinkToParent*>( pxMessage->GetData() );

				// Create and send the message...
				pxData->SetParentGuid( uActorGuid);

				pxData->SetMarkerHash( 0x667604F6 ); // 0x667604F6, Asura_GetHashID("lefthand")

				pxMessage->SetFromGuid( GetGuid() );
				pxMessage->SetToGuid( pxPickup->GetGuid() );

				Asura_Network::Send( pxMessage );
			}
		}
		else
		{
			Asura_Assert(false, "Couldn't find Tool server pickup!");
		}
	}
}

#ifdef ASURA_TOOLS
void AVP_ServerEntity_InteractHackingPoint::GetCustomEventName(int iEventIndex, Asura_String& xEventName)
{
	switch(iEventIndex)
	{
		case MESSAGE_BLOCK_INSERT_HACKING_TOOL:	xEventName = "On marine insert Hacking Tool";		return;
		case MESSAGE_BLOCK_REMOVE_HACKING_TOOL:	xEventName = "On marine remove Hacking Tool";		return;
	}

	Asura_Assert(FALSE, "Invalid Hacking tool object event index");

	xEventName = "";
}

unsigned short* AVP_ServerEntity_InteractHackingPoint::GetCustomEvent(int iIndex)
{
	switch(iIndex)
	{
		case MESSAGE_BLOCK_INSERT_HACKING_TOOL:	return &m_usMarineInsertHackingToolMessageBlock;
		case MESSAGE_BLOCK_REMOVE_HACKING_TOOL:	return &m_usMarineRemoveHackingToolMessageBlock;
	}

	Asura_Assert(FALSE, "Invalid Hacking tool object event index");

	return NULL;
}
#endif // ASURA_TOOLS
