//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_Lift.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Project override of asura lift class
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Lift/AVP_ClientEntity_Lift.h"

// Engine independent.
#ifndef ASURA_NO_NETWORK_LAG_COMPENSATION
#include "Entity/Asura_PosAndOriInterpolator.h"
#endif
#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_Lift.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Message.h"
#include "Physics/Asura_Physics_Object.h"
#include "SpecialFX/Asura_Fragment_System.h"

// Project independent.
#include "Core/AVP_Settings.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// AVP_ClientEntity_Lift
//

Asura_Hashed_List<AVP_ClientEntity_Lift> AVP_ClientEntity_Lift::s_xList( 5 );


AVP_ClientEntity_Lift::AVP_ClientEntity_Lift( void )
{
	Classification = AsuraEntityClass_Lift;

	s_xList.AddToList( this );
	Asura_Entity_Client::AddToUpdateableClientList(this);
}


AVP_ClientEntity_Lift::~AVP_ClientEntity_Lift() 
{
	// Destroy the physics-object (not forgetting to NULL the pointer to prevent
	// parent classes from also attempting to destroy it)...
	Asura_Physics_Object::Destroy(m_pxDynamicObject);
	m_pxDynamicObject	= NULL;

	s_xList.RemoveFromList( this );
}


void AVP_ClientEntity_Lift::SetupDynamicObject( u_int uPhysicalObjectFlags )
{
    if( AVP_Settings::IsSinglePlayer() || Asura_Network::IsServer() )
    {
		// We only deal with the physics client-side for multiplayer
        // client-only machines.
		return;
	}

    ASURA_PHYSICS_OBJECT_TYPE eReqdType = ( uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_STATIC
                                                ? ASURA_PHYSICS_OBJECT_STATIC
                                                : ASURA_PHYSICS_OBJECT_EXTERNALCONTROL );

	if( m_pxDynamicObject )
	{
		// Already created. Note that we don't expect the static / non-static
        // property to change.
        Asura_Assert( m_pxDynamicObject->GetType() == eReqdType, "" );
		return;
	}

	if( uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_ISMERGEDWITHENV )
	{
		return;
	}

	const Asura_Shape_Instance& xShapeInstance = GetShapeInstance();
	const Asura_Shape* pxShape = xShapeInstance.GetShape();
	
	if( pxShape && pxShape->HasNoCollisions() )
	{
		// No collisions, so no need to create a dynamics block.
		return;
	}

	m_pxDynamicObject = Asura_Physics_Object::Create( eReqdType );
    if( !m_pxDynamicObject ) { return; }
	m_pxDynamicObject->SetAssociatedEntityGuid(GetGuid());
	
	m_pxDynamicObject->SetPosition( GetCurrentPosition() );
	m_pxDynamicObject->SetOrientation( GetCurrentOrientation() );
	
	// Find a suitable collision object.
	const Asura_Collision_Object* pxCollObj = NULL;
	if( pxShape )
	{
		pxCollObj = pxShape->GetDynamicsCollisionData();
	}
	else
	{
		const Asura_Animation* pxAnim = xShapeInstance.GetAnimation();
        if( pxAnim )
        {
		    const Asura_Hierarchy_Skin* pxSkin = pxAnim->GetSkin();
		    if( pxSkin )
		    {
			    pxCollObj = pxSkin->GetCollisionObject();
		    }
        }
	}

    // Otherwise use some default box.
	if( !pxCollObj )
	{
		pxCollObj = Asura_Dynamic_Extents::GetCollisionObject( 0, ASURA_HASH_ID_UNSET, GetGuid() );
	    Asura_Assert( pxCollObj, "No collision objects available?" );
	}

	m_pxDynamicObject->SetCollisionObject( pxCollObj );
	m_pxDynamicObject->SetCollisionProxy( Asura_Physics_Object::COLLISION_PROXY_POLYGONAL );
}


void AVP_ClientEntity_Lift::Create( const Asura_Message* pxMsg )
{
	Asura_Assert( pxMsg->GetMsgID() == ASURA_MSG_CLIENTUPDATE, "" );
	Asura_Assert( pxMsg->GetEntityClassification() == AsuraEntityClass_Lift, "" );

	AVP_ClientEntity_Lift* pxLift = new AVP_ClientEntity_Lift();

//	const Asura_MsgData_PhysicsObject_ClientUpdate* pxMsgData = static_cast<const Asura_MsgData_PhysicsObject_ClientUpdate*>( pxMsg->GetData() );
////#ifndef ASURA_NO_NETWORK_LAG_COMPENSATION
//	if( pxMsgData->m_uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_STATIC )
//    {
//	    // (MH - 25.03.2004) these shouldn't ever move, so free their pos interpolator
//	    delete pxLift->m_pxPosAndOriInterpolator;
//	    pxLift->m_pxPosAndOriInterpolator = NULL;
//    }
//#endif

	pxLift->HandleMessage( pxMsg );
	pxLift->InitialiseShapePositionAndOrientation();

	const Asura_Message_PhysicalObject_ClientUpdate* pxMsgData = static_cast<const Asura_Message_PhysicalObject_ClientUpdate*>( pxMsg->GetData() );
    pxLift->SetupDynamicObject( pxMsgData->m_uPhysicalObjectFlags );
}

void AVP_ClientEntity_Lift::Update( void ) 
{
	// Update the collision object
	if (m_pxDynamicObject)
	{
		m_pxDynamicObject->SetPosition( GetCurrentPosition() ); 
		m_pxDynamicObject->SetOrientation( GetCurrentOrientation() ); 
	}
}

bool AVP_ClientEntity_Lift::HandleMessage( const Asura_Message* pxMsg )
{
	return( PARENT_ASURA::HandleMessage(pxMsg) || PARENT::HandleMessage(pxMsg) );
}
