//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_Lift.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Project override of asura lift class
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Lift/AVP_ServerEntity_Lift.h"

// Engine independent.
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Network/Asura_Network.h"
#include "Shapes/Asura_Shapes.h"
#include "Shapes/Asura_Shape_System.h"
#include "SpecialFX/Asura_Fragment_System.h"

// G L O B A L S /////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
static bool g_bNewObjectShouldBeStatic;
#endif

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// AVP_ServerEntity_Lift
//

Asura_Hashed_List<AVP_ServerEntity_Lift> AVP_ServerEntity_Lift::s_xList( 5 );


AVP_ServerEntity_Lift::AVP_ServerEntity_Lift(const Asura_Guid uGUID) 
: Asura_Entity_Server( uGUID )
{
    s_xList.AddToList( this );

//  SetAlwaysUpdateClient( false );
}


AVP_ServerEntity_Lift::~AVP_ServerEntity_Lift( void )
{
    s_xList.RemoveFromList( this );
}


bool AVP_ServerEntity_Lift::HandleMessage( const Asura_Message* pxMsg ) 
{
	return( PARENT_ASURA::HandleMessage(pxMsg) || PARENT::HandleMessage(pxMsg) );
}


void AVP_ServerEntity_Lift::Update( void )
{
    PARENT_ASURA::Update(); 
    PARENT::Update();
}


void AVP_ServerEntity_Lift::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
    xStream << static_cast<u_int>(ChunkData::CURRENT_VERSION);

    PARENT_ASURA::WriteToChunkStream( xStream );
	PARENT::WriteToChunkStream( xStream );

    // Any data specific to this class would go here.
}

void AVP_ServerEntity_Lift::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int uVersion;
	xStream >> uVersion;

	Asura_Assert(uVersion <= ChunkData::CURRENT_VERSION, "Unsupported chunk version");

	PARENT_ASURA::ReadFromChunkStream(xStream);
	PARENT::ReadFromChunkStream(xStream);

	// Any data specific to this class would go here.
}

void AVP_ServerEntity_Lift::InitialiseFromChunkBuffer( const char*& pcChunkBfr )
{
	Asura_Assert(false, "Depracated code - no longer supported.");
}


void AVP_ServerEntity_Lift::MakeClientUpdateMessage( void )
{
    PARENT_ASURA::MakeClientUpdateMessage();
}

#ifdef ASURA_TOOLS

void AVP_ServerEntity_Lift::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
    PARENT_ASURA::ApplyChangesForTemplateInstance( pxInstance );
    PARENT::ApplyChangesForTemplateInstance( pxInstance );
}

bool AVP_ServerEntity_Lift::CanBeIncludedInTemplate( void ) const
{
    return( GetClassification() == AsuraEntityClass_Lift );
}

#endif // #ifdef ASURA_TOOLS

