//////////////////////////////////////////////////////////////////////////////
//
// AVP_Pickup.h
//
// (AJK - 27.08.2008) Created
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef __AVP_PICKUP__
#define __AVP_PICKUP__

#include "Core/AVP_Settings.h"

class AVP_ServerEntity_Pickup;
class AVP_ClientEntity_Pickup;

class AVP_Pickup
{

public :          

	enum PickupType
	{
		PICKUP_TYPE_WEAPON,
		PICKUP_TYPE_HEALTH,
		PICKUP_TYPE_AMMO,
		PICKUP_TYPE_TOOL,
		PICKUP_TYPE_AUDIODIARY,
		PICKUP_TOTAL
	};

	static AVP_Pickup*   Create(Asura_Hash_ID uBlueprintHash);

	virtual void		  InitFromBlueprint();
	virtual Asura_Hash_ID GetGroupHash() const = 0;

	void				  PlayPickupSound(const Asura_Vector_3& xPosition) const;
	void                  SetSkinHash(const Asura_Hash_ID uSkinHash){ m_uSkinHash = uSkinHash;}
	Asura_Hash_ID         GetSkinHash()            const { return m_uSkinHash;            }
	float                 GetCollectDistance()     const { return m_fCollectDistance;     }
	float                 GetCollectDistanceSqrd() const { return m_fCollectDistanceSqrd; }
	bool                  IsAutoPickup()           const { return m_bAutoPickup;          }
	AVP_Settings::Species GetAllegiance()          const { return m_eSpecies;             }
	bool                  GetHasShadow()           const { return m_bHasShadow;			  }
	Asura_Hash_ID		  GetBlueprintHash()       const { return m_uBlueprintHash;		  }
	Asura_Hash_ID		  GetSinglePlayerInventoryItemBlueprintHash() const { return m_uSinglePlayerInventoryItemBlueprintHash;}
	Asura_Hash_ID		  GetMultiplayerInventoryItemBlueprintHash() const { return m_uMultiplayerInventoryItemBlueprintHash;}
	PickupType            GetPickupType()          const { return m_ePickupType;          }
	float				  GetInvertMass()		   const { return m_fInvertMass;          }
	Asura_Hash_ID         GetRestAnim()            const { return m_uRestAnim;            }
	virtual ~AVP_Pickup() {};

protected :

	AVP_Pickup(Asura_Hash_ID);

	PickupType            m_ePickupType;	
	float				  m_fCollectDistance;
	float				  m_fCollectDistanceSqrd;  
	Asura_Hash_ID		  m_uSkinHash;
	Asura_Hash_ID		  m_uPickupSound;
	Asura_Hash_ID		  m_uBlueprintHash;
	Asura_Hash_ID		  m_uSinglePlayerInventoryItemBlueprintHash;
	Asura_Hash_ID		  m_uMultiplayerInventoryItemBlueprintHash;
	bool				  m_bAutoPickup;
	bool				  m_bHasShadow;
	Asura_Hash_ID    	  m_uRestAnim;
	float				  m_fInvertMass;          // (1 / Pickup Mass)
	AVP_Settings::Species m_eSpecies; 			  // The species that can pickup this


};

#endif // #ifndef __AVP_PICKUP__
