//////////////////////////////////////////////////////////////////////////////
//
// File: AVP_ServerEntity_Pickup.cpp
//
//(AJK - 29.08.2008) Created
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Pickup/AVP_ServerEntity_Pickup.h"

// Asura
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Network/Asura_Message.h"

// AVP
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Messages.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Tool.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Weapon.h"
#include "Inventory/AVP_Inventory_HashIDs.h"

// Tools
#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif //ASURA_TOOLS

// Statics definitions
Asura_Hashed_List<AVP_ServerEntity_Pickup> AVP_ServerEntity_Pickup::s_xList(2);
Asura_Hashed_List<AVP_ServerEntity_Pickup> AVP_ServerEntity_Pickup::s_xAutoPickList(2);
Asura_Hashed_List<AVP_ServerEntity_Pickup> AVP_ServerEntity_Pickup::s_xPromptedPickList(2);

// Debug variables
DEBUGVAR float dbg_fDropTime = 20.0f;      // Time after which a dropped pickup disappear

AVP_ServerEntity_Pickup::AVP_ServerEntity_Pickup(Asura_Guid uGuid)
: Asura_Entity_Server(uGuid)
, m_uPickupBlueprintHash(ASURA_HASH_ID_UNSET)
, m_uDialogueBlueprintHash(ASURA_HASH_ID_UNSET)
, m_fRespawnTime(0.0f)
, m_iQuantity(1)
, m_iSecondaryQuantity(1)
, m_uFlags(0)
, m_fRespawnTimer(0.0f)        
, m_fDropTimer(0.0f)
, m_pxPickupObject(NULL)   
, m_usPickedUpMessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_usDroppedMessageBlock( ASURA_STATICMESSAGE_EMPTYBLOCK )
, m_xInitialPosition(Asura_Maths::ZeroVector3)    
, m_xInitialOrientation()
{
	Classification = AVP_ENTITYCLASS_PICKUP;

	if( (m_pxDynamicObject = Asura_Physics_Object::Create( ASURA_PHYSICS_OBJECT_DYNAMIC )) != NULL )
	{
		m_pxDynamicObject->SetOnlyCollideWithEnvironmentAndStaticObjects(true); // Don't collide with dynamic objects
	}
	else
	{
		Asura_Assert(false, ("Could not create Dynamic object in AVP_ServerEntity_Pickup!"));
	}

	s_xList.AddToList(this);
}

AVP_ServerEntity_Pickup::~AVP_ServerEntity_Pickup()
{
	// m_pxPickupObject can be NULL, if the object type wasn't assigned yet; 
	// an assert used to happen if the object is created, then destroyed just after, before choosing pickup type;
	// so i removed the assert, not needed anyway, because, having a NULL pickup object will be asserted elsewhere
	if(m_pxPickupObject)
	{
		if(m_pxPickupObject->IsAutoPickup())
		{
			s_xAutoPickList.RemoveFromList(this);
		}
		else
		{
			s_xPromptedPickList.RemoveFromList(this);
		}
		delete m_pxPickupObject;
	}

	s_xList.RemoveFromList(this);
}

bool AVP_ServerEntity_Pickup::HandleMessage(const Asura_Message* pxMessage) 
{
	switch (pxMessage->GetMsgID())
	{
	case ASURA_MSG_CLIENT_REQUESTCREATEMSG:
		{
			MakeCreateMessage(pxMessage->GetFromGuid());
			return true;
		}	
	case AVP_MSGID_DISABLE_PICKUP:
		{
			SetEnabled(false); 
			return true; 
		}
	case AVP_MSGID_ENABLE_PICKUP:
		{
			SetEnabled(true); 
			return true; 
		}

	default:
		break;
	}

	return PARENT::HandleMessage(pxMessage);
}

void AVP_ServerEntity_Pickup::MakeClientUpdateMessage( void )
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTUPDATE, sizeof(AVP_Message_Pickup_ClientUpdate), GetClassification());
	if (pxMessage)
	{
		AVP_Message_Pickup_ClientUpdate* pxData = static_cast<AVP_Message_Pickup_ClientUpdate*>(pxMessage->GetData());
		pxData->FillPickup(this);
		Send(pxMessage, ASURA_GUID_MY_CLIENT_ENTITIES);
	}
}

void AVP_ServerEntity_Pickup::MakeCreateMessage(Asura_Guid uRequestingClient, bool bSendGuaranteed/* = false*/)
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_CLIENTCREATE, sizeof(AVP_Message_Pickup_ClientCreate), GetClassification());
	if(pxMessage)
	{
		AVP_Message_Pickup_ClientCreate* pxData = static_cast<AVP_Message_Pickup_ClientCreate*>(pxMessage->GetData());
		pxData->FillPickup(this);
		pxMessage->SetToGuid(ASURA_GUID_MY_CLIENT_ENTITIES);
		pxMessage->SetFromGuid(GetGuid());
		if (bSendGuaranteed)
		{
			pxMessage->UseGuaranteedDelivery();
		}
		Asura_Network::SendToSpecificMachine(pxMessage, uRequestingClient);
	}
}

void AVP_ServerEntity_Pickup::CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream)
{
	AVP_ServerEntity_Pickup* pxPickup = new AVP_ServerEntity_Pickup(uGuid);

	if (pxPickup)
	{
		pxPickup->ReadFromChunkStream(xStream);
		// Store original position so the object can re-spawn there
		pxPickup->SetInitialPosition   ( pxPickup->GetPosition() );
		pxPickup->SetInitialOrientation( pxPickup->GetOrientation() );
	}
}

void AVP_ServerEntity_Pickup::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iCurrentChunkVersion;

	xStream >> iCurrentChunkVersion;

	PARENT::ReadFromChunkStream(xStream);

	switch ( iCurrentChunkVersion )
	{
	case CURRENT_CHUNK_VERSION :
		{
			xStream >> m_uPickupBlueprintHash;
			xStream >> m_uDialogueBlueprintHash;
			xStream >> m_fRespawnTime;
			xStream >> m_iQuantity;
			xStream >> m_iSecondaryQuantity;
			xStream >> m_uFlags;
			xStream >> m_fDropTimer;
			xStream >> m_usPickedUpMessageBlock;
			xStream >> m_usDroppedMessageBlock;
			break;
		}
	case 5 :
		{
			xStream >> m_uPickupBlueprintHash;
			xStream >> m_fRespawnTime;
			xStream >> m_iQuantity;
			xStream >> m_iSecondaryQuantity;
			xStream >> m_uFlags;
			xStream >> m_fDropTimer;
			xStream >> m_usPickedUpMessageBlock;
			xStream >> m_usDroppedMessageBlock;
			break;
		}
	case 4 :
		{
			xStream >> m_uPickupBlueprintHash;
			xStream >> m_fRespawnTime;
			xStream >> m_iQuantity;
			xStream >> m_uFlags;
			xStream >> m_fDropTimer;
			xStream >> m_usPickedUpMessageBlock;
			xStream >> m_usDroppedMessageBlock;
			break;
		}
	case 3 :
		{
			xStream >> m_uPickupBlueprintHash;
			xStream >> m_fRespawnTime;
			xStream >> m_iQuantity;
			xStream >> m_uFlags;
			xStream >> m_fDropTimer;
			xStream >> m_usPickedUpMessageBlock;
			break;
		}
	case 2 :
		{
			xStream >> m_uPickupBlueprintHash;
			xStream >> m_fRespawnTime;
			xStream >> m_iQuantity;
			xStream >> m_uFlags;
			xStream >> m_fDropTimer;
			break;
		}
	case 1 :
		{
			xStream >> m_uPickupBlueprintHash;
			xStream >> m_fRespawnTime;
			xStream >> m_iQuantity;
			xStream >> m_uFlags;
			break;
		}
	case 0 :
		{
			xStream >> m_uPickupBlueprintHash;
			xStream >> m_fRespawnTime;
			xStream >> m_iQuantity;
			int iNotUsed;
			xStream >> iNotUsed;    // Was Intelligent amount before, but now this info is in AVP_ServerEntity_PlayerInfo
			xStream >> m_uFlags;
			break;
		}
	default :
		{
			Asura_Assertf(false, ("Unknown chunk version (%d) in AVP_ServerEntity_Pickup", iCurrentChunkVersion));
			break;
		}
	}



	// Create the pickup object
	CreatePickupObject();
}

void AVP_ServerEntity_Pickup::CreatePickupObject()
{
	InitFromBlueprint();
	// The pickup object is changed only in tools;

#ifdef ASURA_TOOLS
	if(m_pxPickupObject)
	{
		if(m_pxPickupObject->IsAutoPickup())
		{
			s_xAutoPickList.RemoveFromList(this);
		}
		else
		{
			s_xPromptedPickList.RemoveFromList(this);
		}
		delete m_pxPickupObject;
	}
#endif //ASURA_TOOLS

	m_pxPickupObject = AVP_Pickup::Create(m_uPickupBlueprintHash);

	if(m_pxPickupObject)
	{
		m_pxPickupObject->InitFromBlueprint();

		if(m_pxPickupObject->IsAutoPickup())
		{
			s_xAutoPickList.AddToList(this);
		}
		else
		{
			s_xPromptedPickList.AddToList(this);
		}

		// Set Skin info to pass on to client entities
		if(m_pxPickupObject->GetSkinHash() != ASURA_HASH_ID_UNSET)
		{
			SetSkinAndAnimID(m_pxPickupObject->GetSkinHash(), ASURA_HASH_ID_UNSET);
		}
		else
		{
			Asura_Assert(false, ("Pickup has no skin!"));
		}

		// Set Mass
		m_pxDynamicObject->SetInvMass( m_pxPickupObject->GetInvertMass() );

		// Enable or disable shadow
		if(!m_pxPickupObject->GetHasShadow())
		{
			SetPhysicalObjectFlags( GetPhysicalObjectFlags() | ASURA_PHYSICALOBJECT_FLAG_NOSHADOW );
		}
	}
	else
	{
		Asura_Assert(false, "Could not create pickup object!");
	}
}

void AVP_ServerEntity_Pickup::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << CURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream(xStream);

	xStream << m_uPickupBlueprintHash;
	xStream << m_uDialogueBlueprintHash;
	xStream << m_fRespawnTime;
	xStream << m_iQuantity;
	xStream << m_iSecondaryQuantity;
	xStream << m_uFlags;
	xStream << m_fDropTimer;
	xStream << m_usPickedUpMessageBlock;
	xStream << m_usDroppedMessageBlock;
}

void AVP_ServerEntity_Pickup::SendPickuedUpMessage() const
{
	if ( m_usPickedUpMessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usPickedUpMessageBlock, GetGuid() );
	}
}

void AVP_ServerEntity_Pickup::SendDroppedMessage() const
{
	if ( m_usDroppedMessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		Asura_StaticMessage_System::SendMessages( m_usDroppedMessageBlock, GetGuid() );
	}
}

#ifdef ASURA_TOOLS
void AVP_ServerEntity_Pickup::CreateFromTools(Asura_Guid uGuid)
{
	new AVP_ServerEntity_Pickup(uGuid);
}

void AVP_ServerEntity_Pickup::Copy(const AVP_ServerEntity_Pickup* const xFrom)
{
	// Loaded chunk objects
	m_uPickupBlueprintHash   = xFrom->m_uPickupBlueprintHash;  
	m_uDialogueBlueprintHash = xFrom->m_uDialogueBlueprintHash;  
	m_fRespawnTime           = xFrom->m_fRespawnTime;  
	m_iQuantity              = xFrom->m_iQuantity;        
	m_iSecondaryQuantity     = xFrom->m_iSecondaryQuantity;  
	m_uFlags                 = xFrom->m_uFlags;
	m_fRespawnTimer          = xFrom->m_fRespawnTimer;           

	// Create the pickup object
	CreatePickupObject();

	// Set the initial position and orientation; to respawn in
	m_xInitialPosition       = xFrom->m_xInitialPosition;      
	m_xInitialOrientation    = xFrom->m_xInitialOrientation;   

	Asura_StaticMessage_System::DeleteMessageBlock(m_usPickedUpMessageBlock);
	m_usPickedUpMessageBlock = Asura_StaticMessage_System::CopyMessageBlock(xFrom->m_usPickedUpMessageBlock,xFrom->GetGuid(),GetGuid());

	Asura_StaticMessage_System::DeleteMessageBlock(m_usDroppedMessageBlock);
	m_usDroppedMessageBlock = Asura_StaticMessage_System::CopyMessageBlock(xFrom->m_usDroppedMessageBlock,xFrom->GetGuid(),GetGuid());

}

bool AVP_ServerEntity_Pickup::CanBeIncludedInTemplate() const
{
	return ( Classification == AVP_ENTITYCLASS_PICKUP );
}

void AVP_ServerEntity_Pickup::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance );

	m_usPickedUpMessageBlock = pxInstance->ConvertMessageBlock( m_usPickedUpMessageBlock );
	m_usDroppedMessageBlock  = pxInstance->ConvertMessageBlock( m_usDroppedMessageBlock  );

	m_xInitialPosition       = pxInstance->TransformPosition( m_xInitialPosition );
	m_xInitialOrientation    = pxInstance->TransformMatrix( m_xInitialOrientation );
}

void AVP_ServerEntity_Pickup::GetCustomEventName(int iEventIndex, Asura_String& xEventName)
{
	switch(iEventIndex)
	{
	case MESSAGE_BLOCK_PICKED_UP: xEventName = "On Picked up"; return;
	case MESSAGE_BLOCK_DROPPED: xEventName = "On Dropped"; return;
	}

	Asura_Assert(false, "Invalid pickup custom event index");

	xEventName = "";
}

unsigned short* AVP_ServerEntity_Pickup::GetCustomEvent(int iIndex)
{
	switch(iIndex)
	{
	case MESSAGE_BLOCK_PICKED_UP: return &m_usPickedUpMessageBlock;
	case MESSAGE_BLOCK_DROPPED: return &m_usDroppedMessageBlock;
	}

	Asura_Assert(FALSE, "Invalid pickup custom event index");

	return NULL;
}
#endif // ASURA_TOOLS


void AVP_ServerEntity_Pickup::Update()
{
	if(IsHidden() && GetCanRespawn())
	{
		//Advance spawn timer
		m_fRespawnTimer += Asura_Timers::GetGameTimeSlice();

		if(m_fRespawnTimer > m_fRespawnTime)
		{
			OnRespawn();
			m_fRespawnTimer = 0.0f;
		}
	}
	else if(IsDroppedByAI())
	{
		m_fDropTimer += Asura_Timers::GetGameTimeSlice();
		if(m_fDropTimer > dbg_fDropTime)
		{
			SetThisAndClientsAsDestroyed();
		}
	}
}

void AVP_ServerEntity_Pickup::OnPickup()
{
	// Send any pickup event static messages
	SendPickuedUpMessage();

	if(m_pxPickupObject && m_pxPickupObject->GetPickupType() == AVP_Pickup::PICKUP_TYPE_TOOL)
	{
		// For now a battery tool need not be destroyed, so it is set to hidden and then it is unhidded when dropped or put into case
		SetHidden(true);

		// Disable it
		SetEnabled(false);


		// Zero the linear velocity; in case the pickup was moving (Explosion blast near pickup ...)  before being picked up 
		if (m_pxDynamicObject) m_pxDynamicObject->ZeroMomentum();

		// deactivate physical object
		if (GetPhysicsObject()) GetPhysicsObject()->SetIgnore(true);
	}
	else
	{
		if(GetCanRespawn())
		{
			// Set as hidden, a hidden message as part of the update message will be sent to client pickup who will then disable rendering
			SetHidden(true);

			// Disable it
			SetEnabled(false);

			// Zero the linear velocity; in case the pickup was moving (Explosion blast near pickup ...)  before being picked up 
			if (m_pxDynamicObject) m_pxDynamicObject->ZeroMomentum();

			// deactivate physical object
			if (GetPhysicsObject()) GetPhysicsObject()->SetIgnore(true);
		}
		else
		{
			SetThisAndClientsAsDestroyed();
		}
	}
}

void AVP_ServerEntity_Pickup::OnRespawn()
{
	// Restore rendering on client
	SetHidden(false);

	SetEnabled(true);

	// Restore original position and orientation
	TeleportTo(m_xInitialPosition, m_xInitialOrientation, true );

	// Restore physics 
	if (GetPhysicsObject()) GetPhysicsObject()->SetIgnore(false);
}

void AVP_ServerEntity_Pickup::InitFromBlueprint()
{
	if(IsFixedQuantity()) return; // Only set these if its a fixed quantity

	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_PICKUPS, m_uPickupBlueprintHash);
	if(pxBlueprint)
	{
		Asura_Value xValue;
		Asura_Hash_ID uInventoryItemParameterHash = AVP_Settings::IsMultiPlayer() ? uAVP_BLUEPRINT_PARAM_PICKUP_MULTIPLAYERINVENTORYITEMBLUEPRINT : uAVP_BLUEPRINT_PARAM_PICKUP_SINGLEPLAYERINVENTORYITEMBLUEPRINT;
		if(pxBlueprint->GetParameterValueByHashID(uInventoryItemParameterHash, xValue))
		{
			Asura_Hash_ID uInventoryItemBlueprintHash = xValue.GetHashIDValue();

			if((pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uInventoryItemBlueprintHash)) != NULL)
			{
				if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_SUBTYPE_PRIMARY, xValue))
				{
					Asura_Blueprint* pxSubtypeBlueprint = NULL;
					if((pxSubtypeBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, xValue.GetHashIDValue())) != NULL)
					{
						if(pxSubtypeBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_INITIALQUANTITY, xValue))
						{
							m_iQuantity = Asura_Maths::FloatToInt(xValue.GetFloatValue());
						}
					}
				}

				if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_SUBTYPE_SECONDARY, xValue))
				{
					Asura_Blueprint* pxSubtypeBlueprint = NULL;
					if((pxSubtypeBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, xValue.GetHashIDValue())) != NULL)
					{
						if(pxSubtypeBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_INITIALQUANTITY, xValue))
						{
							m_iSecondaryQuantity = Asura_Maths::FloatToInt(xValue.GetFloatValue());
						}
					}
				}
			}
		}
	}
}

// This function is called in-game, like when the actor dies
AVP_ServerEntity_Pickup* AVP_ServerEntity_Pickup::Create(const Asura_Guid uGUID, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Hash_ID uPickupHash, const bool bDroppedByAI, const bool bIgnorePhysics /*= false*/, const bool bHidden /*= false*/)
{
	AVP_ServerEntity_Pickup* pxPickup = new AVP_ServerEntity_Pickup( uGUID );
	if( pxPickup )
	{
		pxPickup->m_uPickupBlueprintHash = uPickupHash;

		pxPickup->CreatePickupObject();

		pxPickup->SetPosition(xPosition);
		pxPickup->SetOrientation(xOrientation);

		// React to motion and gravity
		if(pxPickup->m_pxDynamicObject && !bIgnorePhysics)
		{
			pxPickup->m_pxDynamicObject->WakeUp();
		}

		// Created pickup is dropped or not?
		pxPickup->SetDroppedByAI( bDroppedByAI );
		if( bHidden )
		{
			pxPickup->SetHidden(true);
			// Disable it
			pxPickup->SetEnabled(false);
		}
	}

	return pxPickup;
}

void AVP_ServerEntity_Pickup::Drop(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation)
{
	// Send any pickup event static messages
	SendDroppedMessage();

	TeleportTo(xPosition, xOrientation, true );

	SetHidden(false);
	// Enable it
	SetEnabled(true);


	// activate physical object
	if(GetPhysicsObject()) GetPhysicsObject()->SetIgnore(false);
	if(m_pxDynamicObject)   m_pxDynamicObject->WakeUp();
}

void AVP_ServerEntity_Pickup::ChangeSkinToDestroyed(Asura_Hash_ID uAnimID)
{
	if(m_pxPickupObject)
	{
		if(m_pxPickupObject->GetPickupType() == AVP_Pickup::PICKUP_TYPE_TOOL)
		{
			const AVP_Pickup_Tool* pxTool = static_cast<const AVP_Pickup_Tool*> (m_pxPickupObject);
			if(pxTool->GetDestroyedSkin() != ASURA_HASH_ID_UNSET)
			{
				SetSkinAndAnimID(pxTool->GetDestroyedSkin(), uAnimID);
			}
		}
		else
		{
			Asura_Assert(false, "Should not be called for a non tool object!");
		}
	}
}

void AVP_ServerEntity_Pickup::PlayRestAnim() const
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage(AVP_MSGID_PLAY_REST_ANIM_PICKUP);
	if( pxMessage )
	{
		pxMessage->SetFromGuid( GetGuid() );
		pxMessage->SetToGuid( ASURA_GUID_MY_CLIENT_ENTITIES );
		Asura_Network::Send(pxMessage);
	}
}