//////////////////////////////////////////////////////////////////////////////
//
// AVP_SeverEntity_Pickup.h
//
// (AJK - 27.08.2008) Created
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SERVERENTITY_PICKUP__
#define __AVP_SERVERENTITY_PICKUP__

// AVP
#include "Entity/PhysicalObject/AVP_ServerEntity_PhysicalObject.h"

class AVP_Pickup;
class Asura_Chunk_Stream;

class AVP_ServerEntity_Pickup : public AVP_ServerEntity_PhysicalObject
{

	friend class ToolsAVP_Entity_Pickup;

public:

	enum
	{
		FLAG_CAN_RESPAWN       = 0x00000001,
		FLAG_IS_FIXED_QUANTITY = 0x00000002,
		FLAG_IS_HIDDEN         = 0x00000004,
		FLAG_IS_DROPPED        = 0x00000008,  // When the actor is killed, he drops a pickup
		FLAG_IS_DISABLED       = 0x00000010,  // This is from a static message
	};

	static void CreateFromChunkStream( Asura_Guid uGuid, Asura_Chunk_Stream& xStream);
	static AVP_ServerEntity_Pickup* Create(const Asura_Guid uGUID, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Hash_ID uPickupHash, const bool bDroppedByAI, const bool bIgnorePhysics = false, const bool bHidden = false);

	static AVP_ServerEntity_Pickup*	FindEntity(const Asura_Guid uGuid)	{ return(s_xList.FindEntity(uGuid)); }

	static const Asura_Hashed_List<AVP_ServerEntity_Pickup>*  GetList		      (void) { return(&s_xList); }
	static const Asura_Hashed_List<AVP_ServerEntity_Pickup>*  GetAutoPickList     (void) { return(&s_xAutoPickList); }
	static const Asura_Hashed_List<AVP_ServerEntity_Pickup>*  GetPromptedPickList (void) { return(&s_xPromptedPickList); }

	virtual	bool HandleMessage		 (const Asura_Message* pxMessage);
	virtual	void ReadFromChunkStream (Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream	 (Asura_Chunk_Stream& xStream) const;
	virtual bool GetHasCreateMessage() const { return true; }
	virtual void MakeCreateMessage   (Asura_Guid uRequestingClient, bool bSendGuaranteed = false);    
	virtual void Update();

#ifdef ASURA_TOOLS
	static void CreateFromTools(Asura_Guid uGuid);

	void                    Copy(const AVP_ServerEntity_Pickup* const xFrom);
	virtual void            ApplyChangesForTemplateInstance(Tools_EntityTemplateInstance* pxInstance);
	virtual bool            CanBeIncludedInTemplate() const;
	virtual int             GetNumCustomEvents() const { return NUM_MESSAGE_BLOCKS; }
	virtual unsigned short* GetCustomEvent(int iIndex);
	virtual void            GetCustomEventName(int iEventIndex, Asura_String& xEventName);
#endif //ASURA_TOOLS



	void SetInitialPosition   ( const Asura_Vector_3& xPosition      ) { m_xInitialPosition    = xPosition;    }
	void SetInitialOrientation( const Asura_Matrix_3x3& xOrientation ) { m_xInitialOrientation = xOrientation; }


	// Pickup handling
	Asura_Hash_ID           GetPickupBlueprintHash()	  const { return m_uPickupBlueprintHash; }
	const AVP_Pickup*       GetPickupObject()			  const { return m_pxPickupObject; }
	void                    CreatePickupObject();
	int                     GetQuantity()				  const { return m_iQuantity; }
	void                    SetQuantity(int iQuantity)			{ m_iQuantity = iQuantity; }
	int                     GetSecondaryQuantity()		  const { return m_iSecondaryQuantity; }
	void                    SetSecondaryQuantity(int iQuantity)	{ m_iSecondaryQuantity = iQuantity; }

	bool                    GetCanRespawn()          const { return IsFlagged(FLAG_CAN_RESPAWN); }
	bool                    IsHidden()               const { return IsFlagged(FLAG_IS_HIDDEN); }
	bool                    IsFixedQuantity()        const { return IsFlagged(FLAG_IS_FIXED_QUANTITY); }
	bool                    IsDroppedByAI()              const { return IsFlagged(FLAG_IS_DROPPED); }
	bool                    IsEnabled()              const { return !IsFlagged(FLAG_IS_DISABLED); }

	void                    SetCanRespawn    (bool bTrue) { bTrue ? AddFlag(FLAG_CAN_RESPAWN      ) : RemoveFlag(FLAG_CAN_RESPAWN      ); }
	void                    SetHidden        (bool bTrue) { bTrue ? AddFlag(FLAG_IS_HIDDEN        ) : RemoveFlag(FLAG_IS_HIDDEN        ); }
	void                    SetFixedQuantity (bool bTrue) { bTrue ? AddFlag(FLAG_IS_FIXED_QUANTITY) : RemoveFlag(FLAG_IS_FIXED_QUANTITY); }
	void                    SetDroppedByAI   (bool bTrue) { bTrue ? AddFlag(FLAG_IS_DROPPED       ) : RemoveFlag(FLAG_IS_DROPPED       ); }
	void                    SetEnabled       (bool bTrue) { bTrue ? RemoveFlag(FLAG_IS_DISABLED   ) : AddFlag   (FLAG_IS_DISABLED      ); }

	void                    OnPickup();
	void                    Drop(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation);
	void                    SendPickuedUpMessage() const ;
	void                    SendDroppedMessage() const ;
	void                    PlayRestAnim() const;

	//Battery related
	void                    ChangeSkinToDestroyed(Asura_Hash_ID uAnimID);   

	// Audio Dialog pickups
	void                    SetDialogueBlueprintHashID(Asura_Hash_ID uBlueprint) { m_uDialogueBlueprintHash = uBlueprint;}
	Asura_Hash_ID           GetDialogueBlueprintHashID() const                   { return m_uDialogueBlueprintHash;}
protected:
	enum
	{
		MESSAGE_BLOCK_PICKED_UP,
		MESSAGE_BLOCK_DROPPED,
		NUM_MESSAGE_BLOCKS,
	};

	AVP_ServerEntity_Pickup          (const Asura_Guid uGUID);
	virtual ~AVP_ServerEntity_Pickup ();
	virtual void MakeClientUpdateMessage( void );
	void InitFromBlueprint();

private:
	void                    OnRespawn();
	void                    AddFlag    (u_int uFlag) { m_uFlags |= uFlag;               }
	void                    RemoveFlag (u_int uFlag) { m_uFlags &= ~uFlag;              }
	bool                    IsFlagged  (u_int uFlag) const { return ((m_uFlags & uFlag) != 0); }

	static Asura_Hashed_List<AVP_ServerEntity_Pickup> s_xList;
	static Asura_Hashed_List<AVP_ServerEntity_Pickup> s_xAutoPickList;
	static Asura_Hashed_List<AVP_ServerEntity_Pickup> s_xPromptedPickList;

	enum
	{
		CURRENT_CHUNK_VERSION = 6,
	};

	// Loaded variables from tools
	Asura_Hash_ID    m_uPickupBlueprintHash;   // Pickup blueprint hash id   : Load from tools
	Asura_Hash_ID    m_uDialogueBlueprintHash;   // For pickups that uses dialogues
	float			 m_fRespawnTime;           // Saved from tools : Load from tools
	int      	     m_iQuantity;      
	int				 m_iSecondaryQuantity;
	u_int            m_uFlags;                 // CanRespawn, FixedQuantity, Hidden, dropped, enabled;
	float            m_fRespawnTimer;          // this is when the pickup is respawnable and is picked in game
	float            m_fDropTimer;             // When dropped the pickup disappears after some seconds 

	AVP_Pickup*      m_pxPickupObject;         // The actual pickup object has blueprint params

	unsigned short	 m_usPickedUpMessageBlock;
	unsigned short	 m_usDroppedMessageBlock;

	Asura_Vector_3	 m_xInitialPosition;       // Temp to retain initial position, used for respawn only
	Asura_Matrix_3x3 m_xInitialOrientation;    // Temp to retain initial position, used for respawn only

	typedef AVP_ServerEntity_PhysicalObject PARENT;
};

#endif   // ifndef __AVP_SERVERENTITY_PICKUP__
