///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ClientProjectile_HumanFlare.cpp
// Author : Chris Wallace
// Date   : 03-May-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Projectile/AVP_ClientProjectile_HumanFlare.h"

// Asura independent header-files...
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Light/Asura_Photon.h"
#include "Network/Asura_Message.h"
#include "Sound/Asura_Sound_Event_Storage.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_HumanFlare.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fEmbedLerpTime = 0.2f;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List<AVP_ClientProjectile_HumanFlare>	AVP_ClientProjectile_HumanFlare::s_xList;

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_HumanFlare::AVP_ClientProjectile_HumanFlare
// Author       : Ste Butcher
// Date         : 26-March-2007
////////////////////////////////////////////////////////////////////////////////

AVP_ClientProjectile_HumanFlare::AVP_ClientProjectile_HumanFlare( const AVP_Message_ProjectileClientUpdate* pxData )
: PARENT( pxData )
, m_pxFlarePhoton( NULL )
, m_xFlareColour( 0xFFFF4040 )
, m_fLifeTimer( 0.0f )
, m_fDuration( 8.5f )
, m_fFadeOutTime( 2.0f )
, m_fPhotonRange( 8.0f )
, m_fPhotonIntensity( 10.0f )
, m_fDistanceFadeStart( 0.0f )
, m_fDistanceFadeDistance( 0.0f )
, m_fFalloffPower( 1.0f )
, m_fFalloffScale( 1.0f )
, m_fFlicker( 0.1f )
, m_fEmbedParametric( 0.0f )
, m_fPhotonOffset( 0.5f )
, m_bFlareExpired(false)
{
	// Set the classification...
	Classification = AVP_ENTITYCLASS_HUMANFLARE;

	// Don't forget to store us in the list...
	s_xList.AddToList(this);
}

/// End of function 'AVP_ClientProjectile_HumanFlare::AVP_ClientProjectile_HumanFlare'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_HumanFlare::~AVP_ClientProjectile_HumanFlare
// Author       : Ste Butcher
// Date         : 26-March-2007
////////////////////////////////////////////////////////////////////////////////

AVP_ClientProjectile_HumanFlare::~AVP_ClientProjectile_HumanFlare()
{
	Asura_Photon_System::DestroyPhoton(m_pxFlarePhoton);

	s_xList.RemoveFromList(this);
}

/// End of function 'AVP_ClientProjectile_HumanFlare::~AVP_ClientProjectile_HumanFlare'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_HumanFlare::Create
// Author       : Ste Butcher
// Date         : 26-March-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientProjectile_HumanFlare::Create(const Asura_Message *pxMessage)
{
	// Pass the message to our new client...
	const AVP_Message_ProjectileClientUpdate* pxData = static_cast< const AVP_Message_ProjectileClientUpdate* >( pxMessage->GetData() );
	AVP_ClientProjectile_HumanFlare* pxClient = new AVP_ClientProjectile_HumanFlare( pxData );
	if(pxClient)
	{
		pxClient->HandleMessage(pxMessage);

		pxClient->m_uOwnerGuid = pxData->GetOwnerGuid();
		AVP_ClientActor_Human* pxClientHuman = AVP_ClientActor_Human::FindEntity( pxData->GetOwnerGuid() );
		if ( pxClientHuman && pxClientHuman->GetInventorySystem() )
		{
			AVP_InventoryItem_HumanFlare* pxItem = static_cast< AVP_InventoryItem_HumanFlare* >( pxClientHuman->GetInventorySystem()->GetItem( uAVP_INVENTORY_ITEMHASH_FLARE ) );
			if ( pxItem )
			{
				pxItem->SetFlareGuid( pxClient->GetGuid() );
			}
		}
	}
}

/// End of function 'AVP_ClientProjectile_HumanFlare::Create'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_ClientProjectile_HumanFlare::Update
// Author       : Ste Butcher
// Date         : 26-March-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_ClientProjectile_HumanFlare::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	Asura_Value xValue;
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLOUR, xValue ) )
	{
		m_xFlareColour.FromUIntARGB( xValue.GetIntValue() );
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_DURATION, xValue ) )
	{
		m_fDuration = xValue.GetFloatValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_FADEOUTTIME, xValue ) )
	{
		m_fFadeOutTime = xValue.GetFloatValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONRANGE, xValue ) )
	{
		m_fPhotonRange = xValue.GetFloatValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONINTENSITY, xValue ) )
	{
		m_fPhotonIntensity = xValue.GetFloatValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONFADESTART, xValue ) )
	{
		m_fDistanceFadeStart = xValue.GetFloatValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONFADEDISTANCE, xValue ) )
	{
		m_fDistanceFadeDistance = xValue.GetFloatValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONFALLOFFPOWER, xValue ) )
	{
		m_fFalloffPower = xValue.GetFloatValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONFALLOFFSCALE, xValue ) )
	{
		m_fFalloffScale = xValue.GetFloatValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONDISTANCEFADE, xValue ) )
	{
		m_bDoDistanceFade = xValue.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONCORONA, xValue ) )
	{
		m_bDoCorona = xValue.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONENVSHADOWS, xValue ) )
	{
		m_bDoEnvShadows = xValue.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONCASTOWNSHADOWS, xValue ) )
	{
		m_bCastsOwnShadows = xValue.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONCONTRIBUTETOAVG, xValue ) )
	{
		m_bContributeToAvgShadows = xValue.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONSHADOWNOISE, xValue ) )
	{
		m_bDoShadowNoise = xValue.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONSCALEBYBRIGHTNESS, xValue ) )
	{
		m_bScaleByBrightness = xValue.GetBoolValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_FLICKER, xValue ) )
	{
		m_fFlicker = xValue.GetFloatValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SOUNDCREATE, xValue ) )
	{
		m_uBeginSoundEventHash = xValue.GetHashIDValue();
	}
	if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONOFFSET, xValue ) )
	{
		m_fPhotonOffset = xValue.GetFloatValue();
	}

	PARENT::InitialiseFromBlueprint( xBlueprint );
}

void AVP_ClientProjectile_HumanFlare::Update()
{
	// Parent update
	PARENT::Update();

	Asura_Vector_3 xPosition;
	Asura_Vector_3 xPhotonPosition;
	Asura_Matrix_3x3 xOri = GetCurrentOrientation();

	if ( !GetWorldSpaceMarkerPoint( m_pxProjectileData->GetTrailMarkerPoint(), &xPosition, NULL ) )
	{
		xPosition = GetCurrentPosition();
	}
	xPhotonPosition = xPosition;

	if ( m_bIsEmbedded )
	{
		if ( m_fEmbedParametric < 1.0f )
		{
			m_fEmbedParametric += Asura_Timers::GetGameTimeSlice() / dbg_fEmbedLerpTime;
		}
		else
		{
			m_fEmbedParametric = 1.0f;
		}
		Asura_Vector_3 xNormal( xOri.e13, xOri.e23, xOri.e33 );
		xPhotonPosition = xPosition + xNormal * m_fEmbedParametric * m_fPhotonOffset;

		if ( m_iPFXTrailHandle != ASURA_INVALID_HANDLE )
		{
			Asura_PFX_Entity_Effect* pxEffect = static_cast< Asura_PFX_Entity_Effect* >( Asura_PFX_System::GetEffect( m_iPFXTrailHandle ) );
			if ( pxEffect )
			{
				pxEffect->SetOrientation( xOri );
			}
		}
	}

	if ( m_fLifeTimer < m_fDuration )
	{
		if( m_pxFlarePhoton == NULL )
		{
#if defined (ASURA_TARGET_SM3)
			m_pxFlarePhoton = Asura_Photon_System::CreatePhoton();
			m_pxFlarePhoton->SetRange( m_fPhotonRange );
			m_pxFlarePhoton->SetDistanceFade( m_bDoDistanceFade, m_fDistanceFadeStart, m_fDistanceFadeDistance );
			m_pxFlarePhoton->SetFalloffPower( m_fFalloffPower );
			m_pxFlarePhoton->SetFalloffScale( m_fFalloffScale );
			m_pxFlarePhoton->SetCorona( m_bDoCorona );
			const Asura_ShadowParameters xShadowParameters(	( m_bContributeToAvgShadows ? ASURA_SHADOWFLAG_CONTRIBUTES_TO_AVERAGE_SHADOW : 0 )
														  | ( m_bCastsOwnShadows ? ASURA_SHADOWFLAG_CASTS_OWN_SHADOWS : 0 )
														  | ( m_bDoEnvShadows ? ASURA_SHADOWFLAG_CASTS_ENVIRONMENT_SHADOWS : 0 )
														  | ( m_bDoShadowNoise ? ASURA_SHADOWFLAG_USE_NOISE : 0 )
														  | ( m_bScaleByBrightness ? ASURA_SHADOWFLAG_SCALE_STRENGTH_BY_BRIGHTNESS : 0 ) );
			m_pxFlarePhoton->SetShadowParameters( xShadowParameters );
			m_pxFlarePhoton->SetColours( m_xFlareColour.m_fR, m_xFlareColour.m_fG, m_xFlareColour.m_fB );
			m_pxFlarePhoton->SetPosition( xPhotonPosition );
#endif
		}
		else
		{
			m_fLifeTimer += Asura_Timers::GetGameTimeSlice();

			// Move it to the correct position
			m_pxFlarePhoton->SetPosition( xPhotonPosition );

			// Calculate a flicker value
			const float fFlicker = 1.0f - Asura_Random::GetFloat( 0, m_fFlicker );
					
			Asura_RGBA xColour = m_xFlareColour * fFlicker;

			//Calculate an intensity value
			const float fFadeStartTime	= m_fDuration - m_fFadeOutTime;

			const float fIntensity  = Asura_Maths::Lerp( 0.0f, m_fPhotonIntensity, Asura_Maths::ClampToRange( ( m_fDuration - m_fLifeTimer ) / ( m_fDuration - fFadeStartTime ), 0.0f, 1.0f ) );

			xColour	*= fIntensity;

			float fVolume = 0.5f * fFlicker * fIntensity;

			// Apply the new colours
			m_pxFlarePhoton->SetColours( xColour.m_fR, xColour.m_fG, xColour.m_fB );

			if(fIntensity > 0.0f)
			{
				if(m_iSoundHandle != ASURA_INVALID_HANDLE)
				{
					Asura_Sound_Event_System::SetVolumeScale(m_iSoundHandle, fVolume);
				}
			}
			else
			{
				Asura_Sound_Event_System::Stop(m_iSoundHandle);
				Asura_Photon_System::DestroyPhoton(m_pxFlarePhoton);
				m_pxFlarePhoton = NULL;

				AVP_ClientActor_Human* pxHuman = AVP_ClientActor_Human::FindEntity( m_uOwnerGuid );
				if ( pxHuman != NULL && AVP_Game_Core::IsLocalPlayer( pxHuman->GetGuid() ) )
				{
					pxHuman->OnFlareExpire();
					m_bFlareExpired = true;
				}
			}
		}
	}
}


void AVP_ClientProjectile_HumanFlare::UpdateProjectileSound()
{
	if ( m_fLifeTimer < m_fDuration )
	{
		if ( m_pxProjectileData->GetSoundEventHash() != ASURA_HASH_ID_UNSET )
		{
			if (m_iSoundHandle != ASURA_INVALID_HANDLE && !Asura_Sound_Event_System::IsPlaying(m_iSoundHandle))
			{
				Asura_Sound_Event_System::Stop( m_iSoundHandle );
			}
			if (m_iSoundHandle == ASURA_INVALID_HANDLE)
			{
				//create the sound here
				const Asura_Sound_Event* pxSoundEvent = Asura_Sound_Event_Storage::FindEvent( m_pxProjectileData->GetSoundEventHash() );
				Asura_Sound_Event_System::PlayEvent( pxSoundEvent, &m_iSoundHandle, &GetCurrentPosition(), 1.0f, GetGuid() );

				// Falre also plays a start sound
				pxSoundEvent = Asura_Sound_Event_Storage::FindEvent( m_uBeginSoundEventHash ); 
				Asura_Sound_Event_System::PlayEvent( pxSoundEvent, NULL, NULL );
			}
			else
			{
				//If we still have a sound playing update it's position
				Asura_Sound_Event_System::UpdatePosition( m_iSoundHandle, GetCurrentPosition() );
			}
		}
	}
}
/// End of file. ///
