
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_GenericProjectileData.cpp
// Author : Will Plano
// Date   : 07/12/2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Dynamics/Asura_Collision_Object.h"
// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Input/AVP_Rumble_System.h"

Asura_TemplateHashTree< AVP_GenericProjectileData > AVP_GenericProjectileData::s_xHashTree;

const AVP_GenericProjectileData* AVP_GenericProjectileData::GetDataByHash( const Asura_Hash_ID uHash )
{
	AVP_GenericProjectileData* pxData = s_xHashTree.GetDataByHash( uHash, false );
	if ( pxData == NULL )
	{
		pxData = s_xHashTree.AddNode( uHash );
		Asura_Assert( pxData != NULL, "Unable to add new projectile data to tree" );
		if ( pxData != NULL )
		{
			pxData->InitialiseFromBlueprint( uHash );
		}
	}
	return pxData;
}

AVP_GenericProjectileData::AVP_GenericProjectileData()
: m_pxCollisionBox( NULL )
, m_uSkinHash( ASURA_HASH_ID_UNSET )
, m_uContainerTrailHash( ASURA_HASH_ID_UNSET )
, m_uPFXTrail( ASURA_HASH_ID_UNSET )
, m_uPFXExplosion( ASURA_HASH_ID_UNSET )
, m_uMarkerPointTrail( ASURA_HASH_ID_UNSET )
, m_uMarkerPointLink( ASURA_HASH_ID_UNSET )
, m_uRestAnim( ASURA_HASH_ID_UNSET )
, m_uSoundEventHash( ASURA_HASH_ID_UNSET )
, m_uHashID( ASURA_HASH_ID_UNSET )
, m_uImpactDamageTypeHash( ASURA_HASH_ID_UNSET )
, m_uRadialDamageTypeHash( ASURA_HASH_ID_UNSET )
, m_uSecondaryRadialDamageTypeHash( ASURA_HASH_ID_UNSET )
, m_uCameraShakeHash( ASURA_HASH_ID_UNSET )
, m_uVibrationHash( ASURA_HASH_ID_UNSET )
, m_fImpactDamageAtNearRange( 0.0f )
, m_fImpactDamageAtFarRange( 0.0f )
, m_fImpactImpulseAtNearRange( 0.0f )
, m_fImpactImpulseAtFarRange( 0.0f )
, m_fImpactNearRange( 0.0f )
, m_fImpactFarRange( 1000.0f )
, m_fRadialDamageAtNearRange( 0.0f )
, m_fRadialDamageAtFarRange( 0.0f )
, m_fRadialImpulseAtNearRange( 0.0f )
, m_fRadialImpulseAtFarRange( 0.0f )
, m_fRadialNearRange( 0.0f )
, m_fRadialFarRange( 10.0f )
, m_fSecondaryRadialDamageAtNearRange( 0.0f )
, m_fSecondaryRadialDamageAtFarRange( 0.0f )
, m_fSecondaryRadialImpulseAtNearRange( 0.0f )
, m_fSecondaryRadialImpulseAtFarRange( 0.0f )
, m_fSecondaryRadialNearRange( 0.0f )
, m_fSecondaryRadialFarRange( 10.0f )
, m_fVibrationInnerRadius ( 1.0f )
, m_fVibrationOuterRadius ( 1.0f )
, m_fElasticity( 0.2f )
, m_fAngularDrag( 0.99f )
, m_fDrag( 0.001f )
, m_fFriction( 1.0f )
, m_fGravityScale( 1.0f )
, m_fMass( 1.0f )
, m_fMuzzleVelocity( 20.0f )
, m_fTimer( 5.0f )
, m_fCosEmbedAngle( 0.707f )
, m_uNumCorpsesToCollect( 0 )
, m_eCollisionProxy( Asura_Physics_Object::COLLISION_PROXY_OBB )
, m_eImpactType( IMPACT_NONE )
, m_uFlags( 0 )
{
}

AVP_GenericProjectileData::~AVP_GenericProjectileData()
{
	delete m_pxCollisionBox;
}

void AVP_GenericProjectileData::InitialiseFromBlueprint( const Asura_Hash_ID uBlueprintHash )
{
	m_uHashID = uBlueprintHash;
	Asura_Value xValue;
	Asura_Bounding_Box xCollisionBB;
	xCollisionBB.Zero();
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_PROJECTILES, uBlueprintHash );
	if ( pxBlueprint )
	{
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_X_MIN, xValue ) )
		{
			xCollisionBB.MinX = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_X_MAX, xValue ) )
		{
			xCollisionBB.MaxX = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_Y_MIN, xValue ) )
		{
			xCollisionBB.MinY = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_Y_MAX, xValue ) )
		{
			xCollisionBB.MaxY = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_Z_MIN, xValue ) )
		{
			xCollisionBB.MinZ = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISION_OBJ_Z_MAX, xValue ) )
		{
			xCollisionBB.MaxZ = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SKIN, xValue ) )
		{
			m_uSkinHash = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_CONTAINER_TRAIL, xValue ) )
		{
			m_uContainerTrailHash = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PFX_TRAIL, xValue ) )
		{
			m_uPFXTrail = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PFX_EXPLOSION, xValue ) )
		{
			m_uPFXExplosion = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MARKERPOINT_TRAIL, xValue ) )
		{
			m_uMarkerPointTrail = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_IMPACT_DAMAGE_TYPE_BP, xValue ) )
		{
			m_uImpactDamageTypeHash = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_IMPACT_DAMAGE_NEARRANGE, xValue ) )
		{
			m_fImpactDamageAtNearRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_IMPACT_DAMAGE_FARRANGE, xValue ) )
		{
			m_fImpactDamageAtFarRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_IMPACT_IMPULSE_NEARRANGE, xValue ) )
		{
			m_fImpactImpulseAtNearRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_IMPACT_IMPULSE_FARRANGE, xValue ) )
		{
			m_fImpactImpulseAtFarRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_IMPACT_NEARRANGE, xValue ) )
		{
			m_fImpactNearRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_IMPACT_FARRANGE, xValue ) )
		{
			m_fImpactFarRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_RADIAL_DAMAGE_TYPE_BP, xValue ) )
		{
			m_uRadialDamageTypeHash = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_CAMERASHAKE_TYPE_BP, xValue ) )
		{
			m_uCameraShakeHash = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_RADIAL_DAMAGE_NEARRANGE, xValue ) )
		{
			m_fRadialDamageAtNearRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_RADIAL_DAMAGE_FARRANGE, xValue ) )
		{
			m_fRadialDamageAtFarRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_RADIAL_IMPULSE_NEARRANGE, xValue ) )
		{
			m_fRadialImpulseAtNearRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_RADIAL_IMPULSE_FARRANGE, xValue ) )
		{
			m_fRadialImpulseAtFarRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_RADIAL_NEARRANGE, xValue ) )
		{
			m_fRadialNearRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_RADIAL_FARRANGE, xValue ) )
		{
			m_fRadialFarRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SEC_RADIAL_DAMAGE_TYPE_BP, xValue ) )
		{
			m_uSecondaryRadialDamageTypeHash = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SEC_RADIAL_DAMAGE_NEARRANGE, xValue ) )
		{
			m_fSecondaryRadialDamageAtNearRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SEC_RADIAL_DAMAGE_FARRANGE, xValue ) )
		{
			m_fSecondaryRadialDamageAtFarRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SEC_RADIAL_IMPULSE_NEARRANGE, xValue ) )
		{
			m_fSecondaryRadialImpulseAtNearRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SEC_RADIAL_IMPULSE_FARRANGE, xValue ) )
		{
			m_fSecondaryRadialImpulseAtFarRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SEC_RADIAL_NEARRANGE, xValue ) )
		{
			m_fSecondaryRadialNearRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SEC_RADIAL_FARRANGE, xValue ) )
		{
			m_fSecondaryRadialFarRange = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ELASTICITY, xValue ) )
		{
			m_fElasticity =  xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ANGULARDRAG, xValue ) )
		{
			m_fAngularDrag =  xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_DRAG, xValue ) )
		{
			m_fDrag =  xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_FRICTION, xValue ) )
		{
			m_fFriction =  xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_GRAVITYSCALE, xValue ) )
		{
			m_fGravityScale =  xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MASS, xValue ) )
		{
			m_fMass =  xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MUZZLEVELOCITY, xValue ) )
		{
			m_fMuzzleVelocity =  xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_TIMER, xValue ) )
		{
			m_fTimer = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_EMBEDANGLE, xValue ) )
		{
			m_fCosEmbedAngle = Asura_Maths::Cosine( Asura_Maths::DegToRad( xValue.GetFloatValue() ) );
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_NUMCORPSES, xValue ) )
		{
			int iNum = xValue.GetIntValue();
			Asura_Assert( iNum >= 0, "Bad number of corpses to collect" );
			m_uNumCorpsesToCollect = static_cast< u_int > ( iNum );
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_COLLISIONPROXY, xValue ) )
		{
			int iNum = xValue.GetIntValue();
			Asura_Assert( iNum >= 0 && iNum <= Asura_Physics_Object::NUM_COLLISION_PROXIES, "Not a CollisionProxy" );
			m_eCollisionProxy = static_cast< Asura_Physics_Object::COLLISION_PROXY > ( iNum );
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_IMPACTTYPE, xValue ) )
		{
			int iNum = xValue.GetIntValue();
			Asura_Assert( iNum >= 0 && iNum <= NUM_IMPACT_TYPES, "Not an impact type" );
			m_eImpactType = static_cast< ImpactType > ( iNum );
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_DOBALLISTICADJUSTMENT, xValue ) )
		{
			m_bDoBallisticAdjustment = xValue.GetBoolValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_ONLYCOLLIDEEITHENVANDSTATICS, xValue ) )
		{
			m_bOnlyCollideWithEnvironmentAndStaticObjects = xValue.GetBoolValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_DOLOSCOLLISIONS, xValue ) )
		{
			m_bDoLOSCollisions = xValue.GetBoolValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_AVOIDEDBYAI, xValue ) )
		{
			m_bAvoidedByAI = xValue.GetBoolValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_FORWARDPREDICTEDBYAI, xValue ) )
		{
			m_bForwardPredictedByAI = xValue.GetBoolValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MARKERPOINT_LINK, xValue ) )
		{
			m_uMarkerPointLink = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_RESTANIM, xValue ) )
		{
			m_uRestAnim = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SOUND, xValue ) )
		{
			m_uSoundEventHash = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( 0xFB1EF292, xValue ) ) // Vibration
		{
			m_uVibrationHash = xValue.GetHashIDValue();
			if ( m_uVibrationHash != ASURA_HASH_ID_UNSET )
			{
				AVP_Rumble_System::InitFromBlueprint( m_uVibrationHash );
			}
		}

		if ( pxBlueprint->GetParameterValueByHashID( 0xE8F54096, xValue ) )
		{
			m_fVibrationInnerRadius = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( 0x8DEE22FB, xValue ) )
		{
			m_fVibrationOuterRadius = xValue.GetFloatValue();
		}
		
		m_pxCollisionBox = new Asura_Collision_Box( xCollisionBB );
	}
#ifdef ASURA_ASSERT
	else
	{
		Asura_Assert( false, "Unable to get projectiledata BP" );
	}
#endif
}

/// End of file. ///
