
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_GenericProjectileData.h
// Author : Andy Tate
// Date   : 21-Nov-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_GENERICPROJECTILEDATA_H__
#define __AVP_GENERICPROJECTILEDATA_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_HashTree.h"
#include "Core/Asura_Maths.h"
#include "Physics/Asura_Physics_Object.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_PFX;
class Asura_Collision_Box;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_GenericProjectileData
// Author       : Will Plano
// Date         : 07/12/07
//
// Description  : Class uses to store quantity-data for a weapon.
////////////////////////////////////////////////////////////////////////////////

class AVP_GenericProjectileData
{
public:
	enum ImpactType
	{
		IMPACT_NONE,
		IMPACT_EXPLODE,
		IMPACT_EMBED,
		NUM_IMPACT_TYPES
	};

	// Public functions...
	AVP_GenericProjectileData();
	~AVP_GenericProjectileData();
	void InitialiseFromBlueprint( const Asura_Hash_ID uBlueprintHash );

	// Accessor functions...
	const Asura_Collision_Box*	GetCollisionBox() const							{ return m_pxCollisionBox; }
	Asura_Hash_ID				GetSkin() const									{ return m_uSkinHash; }
	Asura_Hash_ID				GetContainerTrailHash() const					{ return m_uContainerTrailHash;}
	Asura_Hash_ID				GetTrailPFX() const								{ return m_uPFXTrail; }
	Asura_Hash_ID				GetExplosionPFX() const							{ return m_uPFXExplosion; }
	Asura_Hash_ID				GetTrailMarkerPoint() const						{ return m_uMarkerPointTrail; }
	Asura_Hash_ID				GetLinkMarkerPoint() const						{ return m_uMarkerPointLink; }
	Asura_Hash_ID				GetRestAnim() const								{ return m_uRestAnim; }
	Asura_Hash_ID				GetSoundEventHash() const						{ return m_uSoundEventHash; }
	Asura_Hash_ID				GetProjectileHash() const						{ return m_uHashID; }
	Asura_Hash_ID				GetCameraShakeHash() const						{ return m_uCameraShakeHash; }
	Asura_Hash_ID				GetVibrationHash() const						{ return m_uVibrationHash; }

	Asura_Hash_ID				GetImpactDamageType() const							{ return m_uImpactDamageTypeHash; }
	float						GetImpactDamageAtRange( const float fRange ) const	{ return Asura_Maths::Lerp( m_fImpactDamageAtNearRange, m_fImpactDamageAtFarRange, Asura_Maths::ClampToRange( ( fRange - m_fImpactNearRange ) / ( m_fImpactFarRange - m_fImpactNearRange ), 0.0f, 1.0f ) ); }
	float						GetImpactDamageAtNearRange() const					{ return m_fImpactDamageAtNearRange; }
	float						GetImpactDamageAtFarRange() const					{ return m_fImpactDamageAtFarRange; }
	float						GetImpactImpulseAtRange( const float fRange ) const	{ return Asura_Maths::Lerp( m_fImpactImpulseAtNearRange, m_fImpactImpulseAtFarRange, Asura_Maths::ClampToRange( ( fRange - m_fImpactNearRange ) / ( m_fImpactFarRange - m_fImpactNearRange ), 0.0f, 1.0f ) ); }
	float						GetImpactImpulseAtNearRange() const					{ return m_fImpactImpulseAtNearRange; }
	float						GetImpactImpulseAtFarRange() const					{ return m_fImpactImpulseAtFarRange; }
	float						GetImpactNearRange() const							{ return m_fImpactNearRange; }
	float						GetImpactFarRange() const							{ return m_fImpactFarRange; }

	Asura_Hash_ID				GetRadialDamageType() const							{ return m_uRadialDamageTypeHash; }
	float						GetRadialDamageAtRange( const float fRange ) const	{ return Asura_Maths::Lerp( m_fRadialDamageAtNearRange, m_fRadialDamageAtFarRange, Asura_Maths::ClampToRange( ( fRange - m_fRadialNearRange ) / ( m_fRadialFarRange - m_fRadialNearRange ), 0.0f, 1.0f ) ); }
	float						GetRadialDamageAtNearRange() const					{ return m_fRadialDamageAtNearRange; }
	float						GetRadialDamageAtFarRange() const					{ return m_fRadialDamageAtFarRange; }
	float						GetRadialImpulseAtRange( const float fRange ) const	{ return Asura_Maths::Lerp( m_fRadialImpulseAtNearRange, m_fRadialImpulseAtFarRange, Asura_Maths::ClampToRange( ( fRange - m_fRadialNearRange ) / ( m_fRadialFarRange - m_fRadialNearRange ), 0.0f, 1.0f ) ); }
	float						GetRadialImpulseAtNearRange() const					{ return m_fRadialImpulseAtNearRange; }
	float						GetRadialImpulseAtFarRange() const					{ return m_fRadialImpulseAtFarRange; }
	float						GetRadialNearRange() const							{ return m_fRadialNearRange; }
	float						GetRadialFarRange() const							{ return m_fRadialFarRange; }

	Asura_Hash_ID				GetSecondaryRadialDamageType() const							{ return m_uSecondaryRadialDamageTypeHash; }
	float						GetSecondaryRadialDamageAtRange( const float fRange ) const		{ return Asura_Maths::Lerp( m_fSecondaryRadialDamageAtNearRange, m_fSecondaryRadialDamageAtFarRange, Asura_Maths::ClampToRange( ( fRange - m_fSecondaryRadialNearRange ) / ( m_fSecondaryRadialFarRange - m_fSecondaryRadialNearRange ), 0.0f, 1.0f ) ); }
	float						GetSecondaryRadialDamageAtNearRange() const						{ return m_fSecondaryRadialDamageAtNearRange; }
	float						GetSecondaryRadialDamageAtFarRange() const						{ return m_fSecondaryRadialDamageAtFarRange; }
	float						GetSecondaryRadialImpulseAtRange( const float fRange ) const	{ return Asura_Maths::Lerp( m_fSecondaryRadialImpulseAtNearRange, m_fSecondaryRadialImpulseAtFarRange, Asura_Maths::ClampToRange( ( fRange - m_fSecondaryRadialNearRange ) / ( m_fSecondaryRadialFarRange - m_fSecondaryRadialNearRange ), 0.0f, 1.0f ) ); }
	float						GetSecondaryRadialImpulseAtNearRange() const					{ return m_fSecondaryRadialImpulseAtNearRange; }
	float						GetSecondaryRadialImpulseAtFarRange() const						{ return m_fSecondaryRadialImpulseAtFarRange; }
	float						GetSecondaryRadialNearRange() const								{ return m_fSecondaryRadialNearRange; }
	float						GetSecondaryRadialFarRange() const								{ return m_fSecondaryRadialFarRange; }

	float						GetVibrationInnerRadius() const								    { return m_fVibrationInnerRadius; }
	float						GetVibrationOuterRadius() const								    { return m_fVibrationOuterRadius; }

	float						GetElasticity() const							{ return m_fElasticity; }
	float						GetAngularDrag() const							{ return m_fAngularDrag; }
	float						GetDrag() const									{ return m_fDrag; }
	float						GetFriction() const								{ return m_fFriction; }
	float						GetGravityScale() const							{ return m_fGravityScale; }
	float						GetMass() const									{ return m_fMass; }
	float						GetMuzzleVelocity()	const						{ return m_fMuzzleVelocity; }
	float						GetTimer() const								{ return m_fTimer; }
	float						GetCosEmbedAngle() const						{ return m_fCosEmbedAngle; }
	u_int						GetNumCorpsesToCollect() const					{ return m_uNumCorpsesToCollect; }
	Asura_Physics_Object::COLLISION_PROXY	GetCollisionProxy() const			{ return m_eCollisionProxy; }
	ImpactType					GetImpactType() const							{ return m_eImpactType; }
	bool						GetDoBallisticAdjustment() const				{ return m_bDoBallisticAdjustment; }
	bool						GetOnlyCollideWithEnvironmentAndStaticObjects() const { return m_bOnlyCollideWithEnvironmentAndStaticObjects; }
	bool						GetDoLOSCollisions() const						{ return m_bDoLOSCollisions; }
	bool						GetAvoidedByAI() const							{ return m_bAvoidedByAI; }
	bool						GetForwardPredictedByAI() const					{ return m_bForwardPredictedByAI; }

	static const AVP_GenericProjectileData* GetDataByHash( const Asura_Hash_ID uHash );
	static void ResetStorage() { s_xHashTree.Reset() ;}

private:
	// Private typedefs...
	Asura_Collision_Box*					m_pxCollisionBox;
	Asura_Hash_ID							m_uSkinHash;
	Asura_Hash_ID							m_uContainerTrailHash;
	Asura_Hash_ID							m_uPFXTrail;
	Asura_Hash_ID							m_uPFXExplosion;
	Asura_Hash_ID							m_uMarkerPointTrail;
	Asura_Hash_ID							m_uMarkerPointLink;
	Asura_Hash_ID							m_uRestAnim;
	Asura_Hash_ID							m_uSoundEventHash;
	Asura_Hash_ID							m_uHashID;
	Asura_Hash_ID							m_uImpactDamageTypeHash;
	Asura_Hash_ID							m_uRadialDamageTypeHash;
	Asura_Hash_ID							m_uSecondaryRadialDamageTypeHash;
	Asura_Hash_ID							m_uCameraShakeHash;
	Asura_Hash_ID                           m_uVibrationHash;

	float									m_fImpactDamageAtNearRange;
	float									m_fImpactDamageAtFarRange;
	float									m_fImpactImpulseAtNearRange;
	float									m_fImpactImpulseAtFarRange;
	float									m_fImpactNearRange;
	float									m_fImpactFarRange;

	float									m_fRadialDamageAtNearRange;
	float									m_fRadialDamageAtFarRange;
	float									m_fRadialImpulseAtNearRange;
	float									m_fRadialImpulseAtFarRange;
	float									m_fRadialNearRange;
	float									m_fRadialFarRange;

	float									m_fSecondaryRadialDamageAtNearRange;
	float									m_fSecondaryRadialDamageAtFarRange;
	float									m_fSecondaryRadialImpulseAtNearRange;
	float									m_fSecondaryRadialImpulseAtFarRange;
	float									m_fSecondaryRadialNearRange;
	float									m_fSecondaryRadialFarRange;

	float                                   m_fVibrationInnerRadius;
	float                                   m_fVibrationOuterRadius;

	float									m_fElasticity;
	float									m_fAngularDrag;
	float									m_fDrag;
	float									m_fFriction;
	float									m_fGravityScale;
	float									m_fMass;
	float									m_fMuzzleVelocity;
	float									m_fTimer;
	float									m_fCosEmbedAngle;
	u_int									m_uNumCorpsesToCollect;                   // Could move all these into the union but then we'd have to check that values were all in range etc.
	Asura_Physics_Object::COLLISION_PROXY	m_eCollisionProxy;
	ImpactType								m_eImpactType;
	union
	{
		u_int								m_uFlags;

		struct
		{
			u_int							m_bDoBallisticAdjustment						: 1,
											m_bOnlyCollideWithEnvironmentAndStaticObjects	: 1,
											m_bDoLOSCollisions								: 1,
										    m_bAvoidedByAI									: 1,
											m_bForwardPredictedByAI							: 1,
																							: 0;
		};
	};

	static Asura_TemplateHashTree< AVP_GenericProjectileData > s_xHashTree;
}; 

#endif   // ifndef __AVP_GENERICPROJECTILEDATA_H__


/// End of file. ///
