
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerProjectile_Disc.h
// Author : Chris Wallace
// Date   : 04-June-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SERVERPROJECTILE_DISC_H__
#define __AVP_SERVERPROJECTILE_DISC_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_GenericProjectile.h"

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_ServerProjectile_Disc
// Author       : Chris Wallace
// Date         : 04-June-2007
//
// Description  : Class to handle the Disc
////////////////////////////////////////////////////////////////////////////////

class AVP_ServerActor_Predator;

class AVP_ServerProjectile_Disc : public AVP_ServerProjectile_GenericProjectile
{
public:
	// Public functions...
	static	AVP_ServerProjectile_Disc*	Create( const Asura_Message* pxMsg );

	// Overridden functions...
	virtual void	Launch	(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xVelocity, const Asura_Guid uGuidToIgnore, float fDamageMultiplier);

	virtual	void	Update			(void);
	virtual	bool	HandleMessage	(const Asura_Message* pxMessage);

	// Overridden chunk-handling functions...
	virtual	void	ReadFromChunkStream	(Asura_Chunk_Stream& xStream);
	virtual	void	WriteToChunkStream	(Asura_Chunk_Stream& xStream) const;
	virtual bool	CanBeSavedToChunk	(void) const	{ return(false); }

	// Legacy chunk handling, to be removed as soon as possible...
	virtual	bool	IsFilledFromChunkBuffer	(void) const	{ return(false); }

	bool			GetLaserTargetMode() const					{ return m_bLaserTargetMode; }
	bool			GetBounceMode() const						{ return m_bBounceMode; }
	u_int			GetDiscState() const						{ return m_uDiscState; }

	void			SetLaserTargetMode( bool bLaserTargetMode )	{ m_bLaserTargetMode = bLaserTargetMode; }
	void			SetBounceMode(bool bBounceMode)				{ m_bBounceMode = bBounceMode;}

	// Static list accessors...
	static	AVP_ServerProjectile_Disc*							FindEntity	(const Asura_Guid uGuid)	{ return(s_xList.FindEntity(uGuid)); }
	static	const Asura_Hashed_List<AVP_ServerProjectile_Disc>*	GetList		(void)						{ return(&s_xList); }

//	void			SetTagetData( const Asura_Guid uGuid, const Asura_Hash_ID uTargetMPHash ) { m_uTargetGuid = uGuid; m_uTargetMPHash = uTargetMPHash; }

protected:
	// Protected variables...
	float				m_fTiltAngle;
	float				m_fReturnTime;
	float				m_fReturnTeleportTime;
	float				m_fReturnMaxBendingAngle;
	float				m_fReturnTimeToReachMaxBendingAngle;
	bool				m_bBounceMode;
	bool				m_bLaserTargetMode;

	float				m_fLaserFollow_MinFlyDistToActivate;		// How many meters disc needs to fly until it will start to follow laser marker point. (if laser is turned on)
	float				m_fLaserFollow_MaxBendingAngle;

	float				m_fSeekEnemyActors_ScanDistance;
	float				m_fSeekEnemyActors_ScanAngle;				// This value actually specifies half of the angle that is needed to check. So if the value is 15 angles, them system will check 15 to left and 15 to right. So in total 30 degree will be tested!
	float				m_fSeekEnemyActors_MinFlyDistToActivate;	// How many meters disc needs to fly until it will start to "seek" for enemies
	float				m_fSeekEnemyActors_MaxBendingAngle;

	float				m_fTraveledDistance;						// Distance the disc traveled from the moment it was released from hand.
	
//	Asura_Guid			m_uTargetGuid;								// OBSOLETE
//	Asura_Hash_ID		m_uTargetMPHash;							// OBSOLETE

	float				m_fTimeSinceLaunch;							// The number of seconds since the disc was launched.
	float				m_fTimeSinceReturn;							// The number of seconds since the disc last hit something.
//	float				m_fPredictedTimeToNextTarget;				// OBSOLETE Calculated based on distance from disc after each collision
	Asura_Vector_3		m_xDiscDirection;							// We can assume that DiscDirection will be always Normalized.

	void FindTargetDirectionAndMaxBendingAngle( const AVP_ServerActor_Predator* pxServerPred, Asura_Vector_3& xTargetDirection, float& fMaxBendingAngle );
	
	// Protected functions...
	AVP_ServerProjectile_Disc          (const Asura_Guid uGuid, const Asura_Message* pxMsg);
	virtual ~AVP_ServerProjectile_Disc ();

	// Overridden entity update functions...
	virtual	void	MakeClientUpdateMessage	(void);

	virtual void	InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );


	// Overridden event functions...
	virtual	void	OnCollision	(const Asura_LOS_InputData& xLOSInputData, const Asura_LOS_OutputData& xLOSOutputData);
	virtual void	OnImpact( const Asura_Vector_3& xImpactPoint, const Asura_Vector_3& xImpactNormal = Asura_Maths::ZeroVector3 );
	virtual	void	OnLaunch(void)	{ PARENT::OnLaunch(); m_fTimeSinceLaunch = 0.0f; }

	bool			SearchForClosestTargetInSearchingCone( Asura_Vector_3& xClosestActorDirVector );
	void			RotateDiscToMakeItLookMoreRealisticaly( const Asura_Vector_3& xDiscDirection, const Asura_Vector_3& xTargetDirection );

private:

	enum
	{
		DISC_IN_FLIGHT,
		DISC_RETURNING_TO_SENDER
	};

	// Private typedefs...
	typedef AVP_ServerProjectile_GenericProjectile PARENT;

	// Private variables...
	u_int		m_uDiscState;
	static		Asura_Hashed_List<AVP_ServerProjectile_Disc>	s_xList;

	// Private functions...
	void							ReturnToSender(void);

	AVP_ServerProjectile_Disc  (void);
	AVP_ServerProjectile_Disc  (const AVP_ServerProjectile_Disc& xOther);

	AVP_ServerProjectile_Disc&	operator=	(const AVP_ServerProjectile_Disc& xOther);

};   // End of class 'AVP_ServerProjectile_Disc'.


#endif   // ifndef __AVP_SERVERPROJECTILE_DISC_H__


/// End of file. ///
