///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerProjectile_HumanFlare.cpp
// Author : Chris Wallace
// Date   : 03-May-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_HumanFlare.h"

#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"

// Asura independent header-files...
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Network/Asura_Network.h"
#include "Core/Asura_Timers.h"

// Asura platform header-files...

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/AVP_server_Messages.h"
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Projectile/AVP_ClientProjectile_HumanFlare.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_HumanFlare.h"

// Game platform header-files...

// Tools header-files...

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const u_int			g_uChunkVersion			= 1;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerProjectile_HumanFlare
//
//////////////////////////////////////////////////////////////////////////////

u_int AVP_ServerProjectile_HumanFlare::s_uRefCount				= 0;
float AVP_ServerProjectile_HumanFlare::s_fFlareAILightRadius	= 10.0f;
float AVP_ServerProjectile_HumanFlare::s_fFlareAILightIntensity	= 7.0f;

Asura_Hashed_List<AVP_ServerProjectile_HumanFlare>	AVP_ServerProjectile_HumanFlare::s_xList;

AVP_ServerProjectile_HumanFlare* AVP_ServerProjectile_HumanFlare::Create( const Asura_Message* pxMsg )
{
	AVP_ServerProjectile_HumanFlare* pxServer = new AVP_ServerProjectile_HumanFlare( Asura_Entity_System::GetCreatedByGameGuid(), pxMsg );

	AVP_ServerActor_Human* pxOwner = AVP_ServerActor_Human::FindEntity( pxMsg->GetFromGuid() );
	if ( pxOwner && pxOwner->GetCurrentGamePlayVolume() )
	{
		pxServer->SetBlueprintHashID( pxOwner->GetCurrentGamePlayVolume()->GetFlareBlueprint() );
	}
	pxServer->InitFromBlueprint();

	const AVP_GenericProjectileData* pxProjectileData = AVP_GenericProjectileData::GetDataByHash( pxServer->GetBlueprintHashID() );
	const AVP_Message_CreateGenericProjectile* pxData = static_cast< const AVP_Message_CreateGenericProjectile* >( pxMsg != NULL ? pxMsg->GetData() : NULL );
	if( pxData != NULL )
	{
		const float				fMuzzleVel = pxProjectileData->GetMuzzleVelocity();
		Asura_Vector_3			xVelocity( 0.0f, 0.0f, fMuzzleVel );
		const Asura_Vector_3&	xPosition = pxData->GetPosition();
		Asura_Matrix_3x3		xOrientation;
		pxData->GetOrientation().ToMatrix( xOrientation );
		xVelocity = xVelocity * xOrientation;
		float fMag = xVelocity.Normalise();
		fMag += xVelocity * pxData->GetActorVelocity();
		if ( pxData->GetLaunch() )
		{
			pxServer->Launch( xPosition, xOrientation, xVelocity * fMag, pxServer->m_uOwnerGuid, pxData->GetDamageScale() ); // Launch last as this creates the client entity
		}
		else
		{
			pxServer->LinkToParent( pxServer->m_uOwnerGuid, pxProjectileData->GetLinkMarkerPoint() );
		}
	}

	return pxServer;
}

AVP_ServerProjectile_HumanFlare::AVP_ServerProjectile_HumanFlare(const Asura_Guid uGuid, const Asura_Message* pxMessage)
: Asura_Entity_Server(uGuid)
, PARENT(uGuid, pxMessage)
, m_bAsleep( true )
, m_bNeedToSendOffMessage(false)
{
	// Update the data...
	Classification = AVP_ENTITYCLASS_HUMANFLARE;

	m_xWPLightingBB.Reset();

	s_xList.AddToList(this);

	//Get the owner and set its flare guid
	AVP_ServerActor_Human* pxServerHuman = AVP_ServerActor_Human::FindEntity(m_uOwnerGuid);
	if ( pxServerHuman && pxServerHuman->GetInventorySystem() )
	{
		AVP_InventoryItem_HumanFlare* pxItem = static_cast< AVP_InventoryItem_HumanFlare* >( pxServerHuman->GetInventorySystem()->GetItem( uAVP_INVENTORY_ITEMHASH_FLARE ) );
		if ( pxItem )
		{
			pxItem->SetFlareGuid( uGuid );
		}
	}
}

AVP_ServerProjectile_HumanFlare::~AVP_ServerProjectile_HumanFlare()
{
#ifdef ASURA_TOOLS
	--s_uRefCount;
#endif
	s_xList.RemoveFromList(this);
}


bool AVP_ServerProjectile_HumanFlare::IsFlareExpired() const
{
	const AVP_ClientProjectile_HumanFlare* pxClient = AVP_ClientProjectile_HumanFlare::FindEntity( GetGuid() );
	return (!pxClient || pxClient->IsFlareExpired());
}

void AVP_ServerProjectile_HumanFlare::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	PARENT::InitialiseFromBlueprint( xBlueprint );
	if ( s_uRefCount == 0 )
	{
		Asura_Value xValue;
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONRANGE, xValue ) )
		{
			s_fFlareAILightRadius = xValue.GetFloatValue();
		}
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PHOTONINTENSITY, xValue ) )
		{
			s_fFlareAILightIntensity = xValue.GetFloatValue();
		}
	}

#ifdef ASURA_TOOLS
	++s_uRefCount;
#else
	s_uRefCount = 1;
#endif
}

void AVP_ServerProjectile_HumanFlare::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int uVersion;

	// Read the version...
	xStream >> uVersion;

	// Read the parent data...
	PARENT::ReadFromChunkStream(xStream);

	if ( uVersion == 1 )
	{
		int iBool;
		xStream >> iBool;
		m_bAsleep = ( iBool != 0 );
	}
	else if ( uVersion == 0 )
	{
	}
	else
	{
		Asura_Assertf( false, ( "AVP_ServerProjectile_HumanFlare::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void AVP_ServerProjectile_HumanFlare::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	// Write the version...
	xStream << g_uChunkVersion;

	// Write the parent data...
	PARENT::WriteToChunkStream(xStream);

	xStream << ( m_bAsleep ? 1 : 0 );
}

void AVP_ServerProjectile_HumanFlare::Launch(const Asura_Vector_3& xPosition,
											 const Asura_Matrix_3x3& xOrientation,
											 const Asura_Vector_3& xVelocity,
											 const Asura_Guid uGuidToIgnore,
											 float fDamageMultiplier)
{
	PARENT::Launch(xPosition, xOrientation, xVelocity, uGuidToIgnore, fDamageMultiplier);
	m_bAsleep = false;
	m_bNeedToSendOffMessage = false;

	if(m_pxDynamicObject)
	{
		m_pxDynamicObject->SetReportEnvironmentCollisionFX(true);
	}
}

void AVP_ServerProjectile_HumanFlare::Update()
{
	PARENT::Update();

	if (IsFlareExpired() )
	{
		if (m_bNeedToSendOffMessage)
		{
			m_bNeedToSendOffMessage = false;
			SendLightOffEvent();
		}
	}
	else if ( !m_bAsleep && (m_pxDynamicObject->IsAsleep() || m_pxDynamicObject->IsIgnore()) )
	{
		m_bAsleep = true;
		m_bNeedToSendOffMessage = true;
		SendLightOnEvent();
	}
	else if (m_bAsleep && (! m_pxDynamicObject->IsAsleep() && !m_pxDynamicObject->IsIgnore()))
	{
		m_bAsleep = false;
		m_bNeedToSendOffMessage = false;
		SendLightOffEvent();
	}

	if ( m_bEmbedded )
	{
		SetHealth( Asura_Maths::Max( GetHealth() - Asura_Timers::GetGameTimeSlice(), 0.0f ) );

		if ( GetHealth() <= 0.0f )
		{
			SetThisAndClientsAsDestroyed();
		}
	}
}

void AVP_ServerProjectile_HumanFlare::SetAsDestroyed()
{
	if (m_bNeedToSendOffMessage)	
	{
		SendLightOffEvent();
	}

	PARENT::SetAsDestroyed();
}

void AVP_ServerProjectile_HumanFlare::OnEmbed( const Asura_Vector_3& xImpactPoint, const Asura_Vector_3& xImpactNormal )
{
	Asura_Matrix_3x3 xOri;

	Asura_Vector_3 xForwards(xImpactNormal);
	Asura_Vector_3 xRight(1.0f, 0.0f, 0.0f);
	Asura_Vector_3 xDown(0.0f, -1.0f, 0.0f);

	const float	fDotProduct = xDown * xForwards;

	// Decide whether which potential side-vector we're furthest away from...
	if ( Asura_Maths::Abs( fDotProduct ) >= 1.0f )
	{
		xDown = xForwards ^ xRight;
		xDown.Normalise();
		xRight = xDown ^ xForwards;
	}
	else
	{
		xRight = xDown ^ xForwards;
		xRight.Normalise();
		xDown = xForwards ^ xRight;
	}

	// Re-create the orientation matrix...
	xOri.e11 = xRight[ 0 ];
	xOri.e21 = xRight[ 1 ];
	xOri.e31 = xRight[ 2 ];
	xOri.e12 = xDown[ 0 ];
	xOri.e22 = xDown[ 1 ];
	xOri.e32 = xDown[ 2 ];
	xOri.e13 = xForwards[ 0 ];
	xOri.e23 = xForwards[ 1 ];
	xOri.e33 = xForwards[ 2 ];

	SetOrientation( xOri );

	PARENT::OnEmbed(xImpactPoint, xImpactNormal);
}

void AVP_ServerProjectile_HumanFlare::SendLightOnEvent()
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_LIGHT_CHANGE_EVENT_CODE, sizeof( AVP_Message_LightChange ) );
	if ( pxMessage )
	{
		AVP_Message_LightChange* pxData = static_cast< AVP_Message_LightChange* >( pxMessage->GetData() );
		if ( pxData )
		{
			pxData->InvalidateOldBB();
			m_xWPLightingBB.SetToPoint( GetPosition() );
			m_xWPLightingBB.Grow( GetFlareAILightRadius() );
			pxData->SetNewBB( m_xWPLightingBB );
			Send( pxMessage, ASURA_GUID_SERVER );
		}
	}
}

void AVP_ServerProjectile_HumanFlare::SendLightOffEvent()
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_LIGHT_CHANGE_EVENT_CODE, sizeof( AVP_Message_LightChange ) );
	if ( pxMessage )
	{
		AVP_Message_LightChange* pxData = static_cast< AVP_Message_LightChange* >( pxMessage->GetData() );
		if ( pxData )
		{
			pxData->SetOldBB( m_xWPLightingBB );
			pxData->InvalidateNewBB();

			Send( pxMessage, ASURA_GUID_SERVER );
		}
	}
}


/// End of file. ///
