///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_ServerProjectile_HumanFlare.h
// Author : Chris Wallace
// Date   : 03-May-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SERVERPROJECTILE_HUMANFLARE_H__
#define __AVP_SERVERPROJECTILE_HUMANFLARE_H__


////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_GenericProjectile.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_ServerProjectile_HumanFlare
// Author       : Chris Wallace
// Date         : 03-May-2007
// Description  : Class to handle a flare
////////////////////////////////////////////////////////////////////////////////

class AVP_ServerProjectile_HumanFlare : public AVP_ServerProjectile_GenericProjectile
{
public:
	static	AVP_ServerProjectile_HumanFlare*		Create(const Asura_Message* pxMessage);

	bool IsFlareExpired() const;

	virtual void	Launch(const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const Asura_Vector_3& xVelocity, const Asura_Guid uGuidToIgnore, float fDamageMultiplier);
	virtual	void	Update();
	virtual void	SetAsDestroyed();

	virtual	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	// Legacy chunk handling, to be removed as soon as possible...
	virtual	bool	IsFilledFromChunkBuffer(void) const	{ return(false); }

	// Static list accessors...
	static AVP_ServerProjectile_HumanFlare* FindEntity(const Asura_Guid uGuid, const bool bIgnoreDestroyedEntities = true)
	{
		return s_xList.FindEntity(uGuid, bIgnoreDestroyedEntities);
	}
	static const Asura_Hashed_List<AVP_ServerProjectile_HumanFlare>* GetList() { return &s_xList; }

	static float GetFlareAILightRadius()	{ return s_fFlareAILightRadius; }
	static float GetFlareAILightIntensity()	{ return s_fFlareAILightIntensity; }
	virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint );

protected:
	AVP_ServerProjectile_HumanFlare(const Asura_Guid uGuid, const Asura_Message* pxMsg);
	virtual ~AVP_ServerProjectile_HumanFlare();

	virtual void	OnEmbed( const Asura_Vector_3& xImpactPoint, const Asura_Vector_3& xImpactNormal );

	void SendLightOnEvent();
	void SendLightOffEvent();


private:
	typedef AVP_ServerProjectile_GenericProjectile PARENT;

	Asura_Bounding_Box m_xWPLightingBB;

	bool m_bAsleep;
	bool m_bNeedToSendOffMessage;

	static float s_fFlareAILightRadius;
	static float s_fFlareAILightIntensity;
	static unsigned int s_uRefCount;

	static Asura_Hashed_List<AVP_ServerProjectile_HumanFlare>		s_xList;

	AVP_ServerProjectile_HumanFlare();
	AVP_ServerProjectile_HumanFlare(const AVP_ServerProjectile_HumanFlare& xOther);

	AVP_ServerProjectile_HumanFlare& operator= (const AVP_ServerProjectile_HumanFlare& xOther);
};   // End of class 'AVP_ServerProjectile_HumanFlare'.

#endif   // ifndef __AVP_SERVERPROJECTILE_HUMANFLARE_H__

/// End of file. ///
