//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_PlayerInfo.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PlayerInfo/AVP_ClientEntity_PlayerInfo.h"
#include "Network/Online/Asura_Lobby.h"

// Engine independent.

// Project...
#include "Actor/AVP_Actor_HUDHandler.h"
#include "Actor/AVP_Actor_PickupHandler.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/ObjectiveSystem/PlayerObjective/AVP_ClientEntity_PlayerObjective.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"
#include "Entity/PlayerInfo/AVP_PlayerInfo_Messages.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Inventory/AVP_Inventory_ItemTypes.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"
#include "Multiplayer/AVP_Multiplayer_GameRules.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////
Asura_Hashed_List<AVP_ClientEntity_PlayerInfo> AVP_ClientEntity_PlayerInfo::s_xList(4);

AVP_ClientEntity_PlayerInfo::AVP_ClientEntity_PlayerInfo(Asura_Guid uGuid)
: Asura_Entity_Client(uGuid)
, m_uPuppetGuid(ASURA_GUID_UNREGISTERED)
, m_uPersecutorGuid(ASURA_GUID_UNREGISTERED)
, m_eMultiplayerTeam( AVP_Settings::AVP_TEAM_NONE )
, m_eMultiplayerRole( AVP_Settings::AVP_ROLE_NORMAL )
, m_xPlayerScore( uGuid )
, m_xQuest()
, m_xAudioDiariesList()
, m_uTargetGuid(ASURA_GUID_UNREGISTERED)
//, m_awcPlayerName()
, m_uLookAtTargetGuid(ASURA_GUID_UNREGISTERED)
, m_uLookAtTargetMP(ASURA_HASH_ID_UNSET)
, m_xLookAtTargetPos(Asura_Maths::ZeroVector3)
, m_bLookingAtTarget(false)
, m_iMissionTimeHours( 0 )
, m_iMissionTimeMinutes( 0 )
, m_iMissionTimeSeconds( 0 )
{
	Classification = AVP_ENTITYCLASS_PLAYERINFO;

	// Use the player info GUID to create a backup unique identifier for each player
	Asura_PrintfToBfr( m_awcPlayerName, L"Player%d", ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE, true, GetGuid() );

	AddToUpdateableClientList(this);

	s_xList.AddToList(this);
}

AVP_ClientEntity_PlayerInfo::~AVP_ClientEntity_PlayerInfo()
{
	RemoveFromUpdateableClientList(this, false);
	s_xList.RemoveFromList(this);
}

bool AVP_ClientEntity_PlayerInfo::HandleMessage(const Asura_Message* pxMessage)
{
	if (!pxMessage) { return false; }

	// Is this something we handle?
	switch (pxMessage->GetMsgID())
	{
		case ASURA_MSG_CLIENTCREATE: // AJK; Create message is the same as the update one + objective Guid
		{
			const AVP_Message_PlayerInfo_ClientCreate* pxData = static_cast<const AVP_Message_PlayerInfo_ClientCreate*>(pxMessage->GetData());
			if (pxData)
			{
				ProcessClientCreate(pxData, pxMessage->GetSentTime()); 
			}
		}
		return true;

		case ASURA_MSG_CLIENTUPDATE:
		{
			const AVP_Message_PlayerInfo_ClientUpdate* pxData = static_cast<const AVP_Message_PlayerInfo_ClientUpdate*>(pxMessage->GetData());
			if (pxData)
			{
				ProcessClientUpdate(pxData, pxMessage->GetSentTime()); 
			}
		}
		return true;

		case AVP_MSGID_SETPUPPETGUID:
		{
			const AVP_Message_SetPuppetGuid* pxData = static_cast<const AVP_Message_SetPuppetGuid*>(pxMessage->GetData());
			if (pxData)
			{
				SetPuppetGuid(pxData->GetPuppetGuid());
			}
		}
		return true;

		case AVP_MSGID_SETMULTIPLAYERTEAM:
		{
			const AVP_Message_SetMultiplayerTeam* pxData = static_cast<const AVP_Message_SetMultiplayerTeam*>(pxMessage->GetData());
			if (pxData)
			{
				SetMultiplayerTeam(static_cast<AVP_Settings::Team>(pxData->GetMultiplayerTeam()));
			}
		}
		return true;

		case ASURA_MSG_DESTRUCT:
		{
			// We need to process the destruct message here, and the server will be forwarding along a destruct to our puppet entity at about the same time.
			PARENT::HandleMessage(pxMessage);
		}
		return true;

		case AVP_MSGID_ADD_PLAYER_OBJECTIVE:
		{
			const AVP_Message_AddPlayerObjective* pxData = static_cast<const AVP_Message_AddPlayerObjective*>(pxMessage->GetData());
			if (pxData)
			{
				// Get client Actor to access the HUD
				AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(m_uPuppetGuid);
				if (pxActor)
				{
					AVP_ClientEntity_PlayerObjective* pxObjective = AVP_ClientEntity_PlayerObjective::FindEntity(pxData->m_uObjectiveGuid);
					if (pxObjective)
					{
						// Test if objective was already added; 
						if (m_xQuest.IsObjectiveInList(pxObjective->GetGuid()))
						{
							Asura_Assert(false, "Objective already added to list!!!");
							return true;
						}
						m_xQuest.AddToObjectivesList(pxObjective->GetGuid());

						// Show in HUD
						AVP_Actor_HUDHandler* pxHudHandler = pxActor->GetHUDHandler();
						if(pxHudHandler)
						{
							pxHudHandler->OnReceiveObjective(pxData->m_uObjectiveGuid);
						}
						else
						{
							Asura_Assert(false, "Player has no HUD?");
						}
					}
					else
					{
						Asura_Assert(false, "Where is the Objective!!!");
					}
				}
			}
			return true;
		}

		case AVP_MSGID_SET_PLAYER_OBJECTIVE_COMPLETE:
		{
			const AVP_Message_AddPlayerObjective* pxData = static_cast<const AVP_Message_AddPlayerObjective*>(pxMessage->GetData());
			if (pxData)
			{
				// Get client Actor to access the HUD
				AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(m_uPuppetGuid);
				if (pxActor)
				{
					// Set objective as completed
					AVP_ClientEntity_PlayerObjective* pxObjective = AVP_ClientEntity_PlayerObjective::FindEntity(pxData->m_uObjectiveGuid);
					if(pxObjective)
					{
						//Make sure the objective is of course already added, if not it is a design error!
						if(m_xQuest.IsObjectiveInList(pxObjective->GetGuid()))
						{
							pxObjective->SetAsCompleted(true);

							// Add to completed list
							m_xQuest.AddToCompletedObjectivesList( pxObjective->GetGuid() );

							AVP_Actor_HUDHandler* pxHudHandler = pxActor->GetHUDHandler();
							if(pxHudHandler)
							{
								pxHudHandler->OnCompleteObjective(pxData->m_uObjectiveGuid);
							}
							else
							{
								Asura_Assert(false, "Player has no HUD?");
							}
						}
						else
						{
							Asura_Assert(false, "Objective completed even if not in the list of objectives!");
						}
					}
					else
					{
						Asura_Assert(false, "Where is the Objective!!!");
						return true;
					}
				}
			}
			return true;
		}

		default:
			break;
	}

	// See if our actor wants to handle the message.
	// TODO - is it better to be selective about which message IDs the actor can handle?
	AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(m_uPuppetGuid);
	if (pxActor && pxActor->HandleMessage(pxMessage))
	{
		return true;
	}

	// Finally, check our parent.
	return PARENT::HandleMessage(pxMessage);
}

void AVP_ClientEntity_PlayerInfo::ProcessClientUpdate(const AVP_Message_PlayerInfo_ClientUpdate* pxData, const float fServerTime)
{
	SetPuppetGuid(pxData->GetPuppetGuid());
	SetPersecutorGuid(pxData->GetPersecutorGuid()); 
	SetMultiplayerTeam(static_cast<AVP_Settings::Team>(pxData->GetMultiplayerTeam()));
	SetMultiplayerRole(static_cast<AVP_Settings::Role>(pxData->GetMultiplayerRole())); 

	// Accessor to the player score
	AVP_Multiplayer_PlayerScore& xPlayerScore( GetPlayerScore() );

	xPlayerScore.SetIndividualScore(pxData->GetIndividualScore()); 
	xPlayerScore.SetTeamOneScore(pxData->GetTeamOneScore()); 
	xPlayerScore.SetTeamTwoScore(pxData->GetTeamTwoScore()); 
	xPlayerScore.SetTeamThreeScore(pxData->GetTeamThreeScore()); 
	xPlayerScore.SetNumberOfKillsAsMarine(pxData->GetNumberOfKillsAsMarine());  
	xPlayerScore.SetNumberOfKillsAsAlien(pxData->GetNumberOfKillsAsAlien());
	xPlayerScore.SetNumberOfKillsAsPredator(pxData->GetNumberOfKillsAsPredator());
	xPlayerScore.SetKillStreak(pxData->GetKillStreak()); 
	xPlayerScore.SetNumberOfKillAssists(pxData->GetNumberOfKillAssists()); 
	xPlayerScore.SetNumberOfTimesKilled(pxData->GetNumberOfTimesKilled()); 
	xPlayerScore.SetNumberOfHeadshots(pxData->GetNumberOfHeadshots());
	xPlayerScore.SetNumberOfTrophies(pxData->GetNumberOfTrophies());
	xPlayerScore.SetNumberOfHeadbites(pxData->GetNumberOfHeadbites());


	AVP_ClientEntity_Actor* pxOurActor = AVP_ClientEntity_Actor::FindEntity( m_uPuppetGuid );
	if ( pxOurActor )
	{
		m_bLookingAtTarget = pxData->GetLookingAtTarget();
		if ( m_bLookingAtTarget )
		{
			m_uLookAtTargetGuid = pxData->GetLookAtTargetGuid();
			m_uLookAtTargetMP = pxData->GetLookAtTargetMP();
			if ( m_uLookAtTargetGuid == ASURA_GUID_UNREGISTERED )
			{
				m_xLookAtTargetPos = pxData->GetLookAtTargetPos();
			}
		}
		else
		{
			if ( !pxOurActor->GetPickupHandler() || !pxOurActor->GetPickupHandler()->GetOverrideAimDirectionUsage() )
			{
				m_uLookAtTargetGuid = ASURA_GUID_UNREGISTERED;
				m_uLookAtTargetMP = ASURA_HASH_ID_UNSET;
				pxOurActor->SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_NOT_USED );
				pxOurActor->SetLookAtPositionUsage( AVP_ClientEntity_Actor::LOOK_AT_POSITION_NOT_USED );
			}
		}
	}

	// Update mission time
	int iMissionTime = pxData->GetMissionTime();

	m_iMissionTimeHours = iMissionTime / 3600;
	iMissionTime -= m_iMissionTimeHours * 3600;

	m_iMissionTimeMinutes = iMissionTime / 60;
	iMissionTime -= m_iMissionTimeMinutes * 60;

	m_iMissionTimeSeconds = iMissionTime;
}

void AVP_ClientEntity_PlayerInfo::MakeActorLookHere( const Asura_Vector_3& xPos )
{
	AVP_ClientEntity_Actor* pxOurActor = AVP_ClientEntity_Actor::FindEntity( m_uPuppetGuid );
	if ( pxOurActor )
	{
		pxOurActor->SetAimDirectionUsage( AVP_ClientEntity_Actor::AIM_DIRECTION_SET_DIRECTLY );
		Asura_Vector_3 xEyePos;
		if ( pxOurActor->GetObjectSpaceMarkerPoint( 0x002fbc62, &xEyePos, NULL ) ) // eyes
		{
			xEyePos = pxOurActor->GetCurrentPosition() + Asura_Vector_3( 0.0f, xEyePos.GetY(), 0.0f );
		}
		else
		{
			xEyePos = pxOurActor->GetCurrentPosition() - Asura_Vector_3( 0.0f, 2.0f, 0.0f );
		}

		Asura_AVector3 xAimDir( xPos - xEyePos );
		xAimDir.Normalise();
		pxOurActor->SetTargetAimDirection( xAimDir );
		pxOurActor->SetLookAtPositionUsage( AVP_ClientEntity_Actor::LOOK_AT_POSITION_SET_DIRECTLY );
		pxOurActor->SetTargetLookAtPosition( Asura_AVector3( xPos ) );
	}
}

void AVP_ClientEntity_PlayerInfo::ProcessClientCreate(const AVP_Message_PlayerInfo_ClientCreate* pxData, const float fServerTime)
{
	ProcessClientUpdate(pxData, fServerTime);

	pxData->Extract(this);
}

AVP_Settings::Species AVP_ClientEntity_PlayerInfo::GetPuppetSpecies() const
{
	AVP_Settings::Species eSpecies = AVP_Settings::AVP_SPECIES_UNKNOWN; 
	const Asura_Guid uPuppetGuid = GetPuppetGuid(); 
	if ( uPuppetGuid != ASURA_GUID_UNREGISTERED )
	{
		AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(uPuppetGuid);
		if (pxActor)
		{
			// Get the player species from the puppet classification
			eSpecies = pxActor->GetSpeciesByClassification();  
		}
	}
	else
	{
		Asura_Assert(false, "Requesting puppet species thorugh the player info but the puppet itself hasn't been set"); 
	}
	return eSpecies; 
}

void AVP_ClientEntity_PlayerInfo::SetPuppetGuid(Asura_Guid uGuid)
{
	if (m_uPuppetGuid == uGuid) { return; }
	if ((uGuid != ASURA_GUID_UNREGISTERED) && !AVP_Game_Core::GetCanMakePlayerActors()) { return; }

	// We no longer attempt to return an actor we're leaving to an AI, as it was breaking in many entertaining ways and we don't actually have game mechanics that need it anyway.
	// We'll assume that if we're leaving an actor, it no longer has much need to do anything anyway.
	if (m_uPuppetGuid != ASURA_GUID_UNREGISTERED)
	{
		// Clean up what elements we can from the actor.
		AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(m_uPuppetGuid);
		if (pxActor)
		{
			Asura_Assert(pxActor->GetPlayerInfoGuid() == Guid, "Attached actor mismatch. Message ordering problem?");
			pxActor->DisconnectPlayer();
		}

		// Unset this immediately.
		m_uPuppetGuid = ASURA_GUID_UNREGISTERED;
	}

	if (uGuid != ASURA_GUID_UNREGISTERED)
	{
		// Set the new actor as a player.
		AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(uGuid);
		if (pxActor && !pxActor->IsRagdoll())
		{
			if (AVP_Game_Core::IsLocalPlayer(GetGuid()))
			{
				// Get the player species from the puppet classification
				AVP_Settings::Species eSpecies = pxActor->GetSpeciesByClassification(); 
				AVP_Settings::SetPlayerSpecies( eSpecies ); 
			}

			pxActor->MakePlayer(GetGuid());

			// Only set the actor guid on the client if the actor exists
			m_uPuppetGuid = uGuid;
		}
	}
}

void AVP_ClientEntity_PlayerInfo::Create(const Asura_Message* pxCreateMessage)
{
	Asura_Assert(pxCreateMessage->GetMsgID() == ASURA_MSG_CLIENTCREATE, "Should only create a client player info from an create message.");

	AVP_ClientEntity_PlayerInfo* pxEntity = new AVP_ClientEntity_PlayerInfo(pxCreateMessage->From);
	if (pxEntity)
	{
		pxEntity->InitialisePlayerName(); 
		pxEntity->HandleMessage(pxCreateMessage);
	}
}

void AVP_ClientEntity_PlayerInfo::SetMultiplayerTeam(AVP_Settings::Team eMultiplayerTeam)
{
	m_eMultiplayerTeam = eMultiplayerTeam;
}

void AVP_ClientEntity_PlayerInfo::InitialisePlayerName()
{
#if defined(ASURA_SUPPORT_LOBBY)
	// Check if the lobby has been initialised
	const Asura_Lobby* pxLobby = Asura_Lobby_System::GetLobby(); 
	if (pxLobby)
	{
		Asura_Lobby_Player* pxPlayer = pxLobby->FindPlayer(GetGuid());
		Asura_Assertf(pxPlayer, ("Couldn't find player %d in lobby to extract name", GetGuid()));
		if (pxPlayer)
		{
			const Asura_UnicodeChar* pwcCurrentPlayerName = pxPlayer->GetName();
			// Use the name from the lobby
			Asura_PrintfToBfr(m_awcPlayerName, pwcCurrentPlayerName, ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE, true ); 
		}
	}
#endif	//defined(ASURA_SUPPORT_LOBBY)

	// TODO: alternatives for player names?
}

const AVP_ClientEntity_Actor* AVP_ClientEntity_PlayerInfo::GetTargetedActor() const
{
	// Get client Actor
	AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(m_uPuppetGuid);
	if(pxActor)
	{
		const AVP_Inventory_Item* pxItem = pxActor->GetInventorySystem()->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON );
		if ( pxItem )
		{
			// Is it a ranged weapon
			AVPInventoryItemType eItemType = pxItem->GetItemType();
			if (   eItemType == AVP_INVENTORYITEMTYPE_WEAPON_RANGED 
				|| eItemType == AVP_INVENTORYITEMTYPE_SMARTGUN 
				|| eItemType == AVP_INVENTORYITEMTYPE_WEAPON_FLAMETHROWER 
				|| eItemType == AVP_INVENTORYITEMTYPE_TASER 
				|| eItemType == AVP_INVENTORYITEMTYPE_SHOTGUN 
				|| eItemType == AVP_INVENTORYITEMTYPE_PLASMACASTER
				|| eItemType == AVP_INVENTORYITEMTYPE_PREDATORDISC
				|| eItemType == AVP_INVENTORYITEMTYPE_PREDATORMINE
				|| eItemType == AVP_INVENTORYITEMTYPE_SCOPEDRIFLE   )
			{
				const AVP_RangedWeapon_SubType*	pxSubType	= static_cast<const AVP_RangedWeapon_SubType*>(pxItem->GetSubType(0));

				if (pxSubType)
				{
					Asura_Vector_3 xAimDirection;
					Asura_Camera::GetForwardVector(xAimDirection);

					Asura_LOS_InputData xInputData(*Asura_Camera::GetPosition(), xAimDirection, m_uPuppetGuid, pxSubType->GetMaxRange());	

					const Asura_LOS_OutputData* pxOutputData = Asura_LOS::TestAgainstEverything(&xInputData);

					if(pxOutputData->m_uHitEntity && ( m_uPuppetGuid != pxOutputData->m_uHitEntity ))
					{
						// An entity is hit, return it
						// NULL if not a human entity
						return ( AVP_ClientEntity_Actor::FindEntity(pxOutputData->m_uHitEntity) );
					}
				}
			}
		}
	}
	return NULL;
}

void AVP_ClientEntity_PlayerInfo::Update()
{
	PARENT::Update();

	UpdateTargeting();

	const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(m_uPuppetGuid);
	if (pxActor && pxActor->IsRagdoll())
	{
		SetPuppetGuid(ASURA_GUID_UNREGISTERED);
	}
}

// AJK : 05/11/08 Player target, this is for the context sensitive reticule, and for displaying some human entities name
void AVP_ClientEntity_PlayerInfo::UpdateTargeting()
{
	// Get client Actor
	const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity(m_uPuppetGuid);
	if(pxActor && (pxActor->GetClassification() == AVP_ENTITYCLASS_MARINE) ) // This concerns marines only for now
	{
		m_uTargetGuid       = ASURA_GUID_UNREGISTERED;
		// Update targeted entities (Cross Hair becomes red)
		const AVP_ClientEntity_Actor* pxTarget = GetTargetedActor();
		if(pxTarget && !pxTarget->IsDead() && !pxTarget->IsADeadBody())
		{
			m_uTargetGuid = pxTarget->GetGuid();
		}
	}
}

void AVP_ClientEntity_PlayerInfo::UpdateForcedLooking()
{
	if ( m_bLookingAtTarget )
	{
		if ( m_uLookAtTargetGuid == ASURA_GUID_UNREGISTERED )
		{
			MakeActorLookHere( m_xLookAtTargetPos );
		}
		else
		{
			Asura_Vector_3 xTargetPos;
			AVP_ClientEntity_PhysicalObject* pxLookAtPhysObj = AVP_ClientEntity_PhysicalObject::FindEntity( m_uLookAtTargetGuid );
			Asura_Assertf( pxLookAtPhysObj, ( "Unable to get entity %d to look at", m_uLookAtTargetGuid ) );
			if ( pxLookAtPhysObj )
			{
				const Asura_Vector_3&	xClientPosition		= pxLookAtPhysObj->GetCurrentPosition();
				Asura_Matrix_3x3		xAnimOrientation	= pxLookAtPhysObj->GetCurrentOrientation();
				u_int					uFlags	= ASURA_HIERARCHY_RENDER_NO_FRUSTUM_TEST | ASURA_HIERARCHY_RENDER_NO_MODULELINK_TEST | ASURA_HIERARCHY_RENDER_DONTRENDER | ASURA_HIERARCHY_RENDER_UPDATEBONESALWAYS;

				AVP_ClientActor_Vertebrate*	pxClientVert	= AVP_ClientActor_Vertebrate::FindEntity( m_uLookAtTargetGuid );
				if (pxClientVert != NULL)
				{
					AVP_Actor_AnimationHandler* pxAnimHandler = pxClientVert->GetAnimationHandler();
					if ( pxAnimHandler != NULL && pxAnimHandler->IsRagdoll() )
					{
						uFlags |= ASURA_HIERARCHY_RENDER_BONES_IN_WORLDSPACE;
					}
				}

				// Ensure that the animation has been updated this frame so our marker points etc. are up to date...
				if ( pxLookAtPhysObj->GetShapeInstance().GetAnimation() )
				{
					pxLookAtPhysObj->GetShapeInstance().GetAnimation()->Update( xClientPosition, xAnimOrientation, pxLookAtPhysObj->GetShapeInstance().GetAnimTimestep(), uFlags );
				}

				if ( m_uLookAtTargetMP != ASURA_HASH_ID_UNSET )
				{
					if ( !pxLookAtPhysObj->GetWorldSpaceMarkerPoint( m_uLookAtTargetMP, &xTargetPos, NULL ) )
					{
						Asura_Assertf( false, ( "Unable to get marker point 0x%08x on entity %d", m_uLookAtTargetMP, m_uLookAtTargetGuid ) );
						xTargetPos = pxLookAtPhysObj->GetCurrentPosition();
					}
				}
				else
				{
					xTargetPos = pxLookAtPhysObj->GetCurrentPosition();
				}
				MakeActorLookHere( xTargetPos );
			}

		}
	}
}

AVP_Player_AudioDiary* AVP_ClientEntity_PlayerInfo::GetAudioDiary( u_int uIndex )
{
	Asura_LinkList_It< AVP_Player_AudioDiary > xLinkListIter( m_xAudioDiariesList );

	u_int uElement = 0;
	while ( !xLinkListIter.Done() )
	{
		if ( uIndex == uElement)
		{
			return (xLinkListIter.GetCurrent());
		}
		++uElement;
		xLinkListIter.Next();
	}
	return NULL;
}
