//////////////////////////////////////////////////////////////////////////////
//
// AVP_ClientEntity_PlayerInfo.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___AVP_CLIENTENTITY_PLAYERINFO_H___
#define ___AVP_CLIENTENTITY_PLAYERINFO_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

// Engine independent.
#include "Entity/Asura_Entity.h"
#include "Network/Online/Asura_Lobby_Types.h"

// Project independent
#include "Core/AVP_Settings.h"
#include "Entity/PlayerInfo/AVP_Player_Quest.h"
#include "Multiplayer/AVP_Multiplayer_PlayerScore.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_Message_PlayerInfo_ClientUpdate; 
class AVP_Message_PlayerInfo_ClientCreate;
class AVP_ClientEntity_Actor;
class AVP_Player_AudioDiary;

// C L A S S E S /////////////////////////////////////////////////////////////

class AVP_ClientEntity_PlayerInfo : public Asura_Entity_Client
{   
	typedef Asura_Entity_Client PARENT;

public:
	AVP_ClientEntity_PlayerInfo(Asura_Guid uGuid);
	virtual ~AVP_ClientEntity_PlayerInfo();

	virtual bool HandleMessage(const Asura_Message* pxMessage);
	virtual void Update();
	void    UpdateTargeting();
	void	UpdateForcedLooking();

	void ProcessClientUpdate(const AVP_Message_PlayerInfo_ClientUpdate* pxData, const float fServerTime); 
	void ProcessClientCreate(const AVP_Message_PlayerInfo_ClientCreate* pxData, const float fServerTime); 

	// This function sets the actor guid here, but doesn't do anything to do with MakePlayer, since that must only be called on the machine the player is on (and so is left to AVP_Player).
	void SetPuppetGuid(Asura_Guid uGuid);
    Asura_Guid GetPuppetGuid() const { return m_uPuppetGuid; }
	
	// Get the player current species, basically the species of the puppet this player is controlling
	AVP_Settings::Species GetPuppetSpecies() const; 

	// Sets the player guid of the player persecuting us
	void SetPersecutorGuid(Asura_Guid uGuid)	{ m_uPersecutorGuid = uGuid; } 
	Asura_Guid GetPersecutorGuid(void) const	{ return m_uPersecutorGuid; }

	// Sets the team for team based multiplayer matches
	void SetMultiplayerTeam(AVP_Settings::Team eMultiplayerTeam);
	AVP_Settings::Team GetMultiplayerTeam(void) const				{ return m_eMultiplayerTeam; }

	// Sets the team for team based multiplayer matches
	void SetMultiplayerRole(AVP_Settings::Role eMultiplayerRole)	{ m_eMultiplayerRole = eMultiplayerRole; }
	AVP_Settings::Role GetMultiplayerRole(void) const				{ return m_eMultiplayerRole; }

	// Player score
	const AVP_Multiplayer_PlayerScore& GetPlayerScore(void) const	{ return m_xPlayerScore; }
	AVP_Multiplayer_PlayerScore& GetPlayerScore(void)				{ return m_xPlayerScore; }

	// Player name
	const Asura_UnicodeChar* GetPlayerName(void) const				{ return m_awcPlayerName; }

	// Get Quest/Audio diaries
	const AVP_Player_Quest& GetQuest() const { return m_xQuest;}
	void                    SetQuest(const AVP_Player_Quest& xQuest) { m_xQuest = xQuest;}
	const Asura_LinkList< AVP_Player_AudioDiary >& GetAudioDiariesList() const { return m_xAudioDiariesList; }
	Asura_LinkList< AVP_Player_AudioDiary >&       GetAudioDiariesList()       { return m_xAudioDiariesList; }
	AVP_Player_AudioDiary*                         GetAudioDiary( u_int uIndex );

	// Target
	Asura_Guid GetTargetGuid()      const { return m_uTargetGuid;       }

	int GetMissionTimeHours( void ) const { return m_iMissionTimeHours; }
	int GetMissionTimeMinutes( void ) const { return m_iMissionTimeMinutes; }
	int GetMissionTimeSeconds( void ) const { return m_iMissionTimeSeconds; }

	static void Create(const Asura_Message* pxCreateMessage);

	static AVP_ClientEntity_PlayerInfo* FindEntity(Asura_Guid uGuid) { return s_xList.FindEntity(uGuid); }
	static const Asura_Hashed_List<AVP_ClientEntity_PlayerInfo>* GetList() { return &s_xList; }

protected:
	void InitialisePlayerName(void); 
	const AVP_ClientEntity_Actor* GetTargetedActor() const;

	void MakeActorLookHere( const Asura_Vector_3& xPos );

	Asura_Guid						m_uPuppetGuid;
	Asura_Guid						m_uPersecutorGuid; 
	AVP_Settings::Team				m_eMultiplayerTeam; 
	AVP_Settings::Role				m_eMultiplayerRole; 
	AVP_Multiplayer_PlayerScore		m_xPlayerScore; 
	AVP_Player_Quest                       m_xQuest;
	Asura_LinkList <AVP_Player_AudioDiary> m_xAudioDiariesList;
	Asura_Guid                      m_uTargetGuid;
	Asura_UnicodeChar				m_awcPlayerName[ASURA_LOBBY_PLAYER_NAME_BUFFER_SIZE];

	Asura_Guid		m_uLookAtTargetGuid;
	Asura_Hash_ID	m_uLookAtTargetMP;
	Asura_Vector_3	m_xLookAtTargetPos;
	bool m_bLookingAtTarget;

private:
	int m_iMissionTimeHours;
	int m_iMissionTimeMinutes;
	int m_iMissionTimeSeconds;

	static Asura_Hashed_List<AVP_ClientEntity_PlayerInfo> s_xList;
};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___AVP_CLIENTENTITY_PLAYERINFO_H___
