//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_SpawnController.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_SERVERENTITY_SPAWNCONTROLLER_H__
#define __AVP_SERVERENTITY_SPAWNCONTROLLER_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Asura
#include "Entity/Asura_Entity.h"

// AVP
#include "Entity/Spawners/AVP_SpawnerInterface.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Tools_EntityTemplateInstance;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_SpawnController
//
//////////////////////////////////////////////////////////////////////////////

class AVP_ServerEntity_SpawnController : public Asura_Entity_Server, public AVP_SpawnerInterface
{
	typedef Asura_Entity_Server PARENT;
	friend class ToolsAVP_Entity_SpawnController;

public:
	AVP_ServerEntity_SpawnController( const Asura_Guid uGuid );
	virtual ~AVP_ServerEntity_SpawnController();

	virtual bool HasClientEntities()	   const { return false; }
	virtual bool CanBeSavedToChunk()	   const { return true; }
	virtual bool IsFilledFromChunkBuffer() const { return false; }
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );

	virtual void Update();
	virtual bool HandleMessage( const Asura_Message* pxMessage );

#ifdef ASURA_TOOLS
	virtual bool CanBeIncludedInTemplate() const { return true; }
	virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );
#endif

	// Spawner interface.
	virtual void 		ScoreSpawners( const Asura_ScoreParams* pxScoreParams, float& fHighestScore, Asura_Guid& uBestGuid ) const;
	virtual bool 		OwnsSpawnerGuid( const Asura_Guid uSpawnerGuid ) const;
	virtual Asura_Guid 	Spawn( const Asura_Guid uSpawnerGuid, const Asura_SpawnParams* pxSpawnParams );

	static void ActorOutOfPlay( const Asura_Guid uGuid, bool bIsDespawning=false );

private:
	void ScoreAndSpawn( const Asura_ScoreParams* pxScoreParams, const Asura_SpawnParams* pxSpawnParams );
	void GenerateScoreParams( const AVP_ScoreParams* pxScoreParamsIn, AVP_ScoreParams& xScoreParamsOut );
	void GenerateSpawnParams( const AVP_ScoreParams& xScoreParamsIn, const AVP_SpawnParams* pxSpawnParamsIn, AVP_SpawnParams& xSpawnParamsOut );
	bool RemoveSpawneeGuidFromStats( const Asura_Guid uGuid, bool bIsDespawning );

	void ReallocateSpawneeArray( const int iSize );

private:
	struct BlueprintAndWeight
	{
		Asura_Hash_ID m_uBPHashID;
		Asura_Hash_ID m_uBPTypeHashID;
		float		  m_fWeight;
	};
	struct SpawnerAndWeight 
	{
		Asura_Guid m_uSpawnerGuid;
		float	   m_fWeight;
	};

	int					m_iMaxInPlay;
	int					m_iMaxSpawnCount;
	float				m_fSpawnPeriodMin;
	float				m_fSpawnPeriodMax;
	Asura_Guid			m_uSpawneeSquadGuid;
	u_int				m_uNumSpawneeTypes;
	BlueprintAndWeight* m_pxSpawneeTypes;
	int					m_iSpawnScheme;
	float				m_fSpawnDistance;
	u_int				m_uNumSpawnPoints;
	SpawnerAndWeight*	m_pxSpawnPoints;
	u_int				m_uNumGPVolumes;
	SpawnerAndWeight*	m_pxGPVolumes;

	float				m_fSpawnTimer;
	u_int				m_uTotalSpawnCount;
	// Number of spawnees in play currently.
	int					m_iSpawneeCount;
	int					m_iSpawneeArraySize;
	Asura_Guid*			m_pxSpawneeGuids;

private:
	static Asura_Hashed_List< AVP_ServerEntity_SpawnController > s_xList;

	static const u_int uCURRENT_CHUNK_VERSION = 0;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_SERVERENTITY_SPAWNCONTROLLER_H__
