//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_SpawnPoint2.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Spawners/AVP_ServerEntity_SpawnPoint2.h"

// asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Core/Asura_Timers.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GS2Node.h"

// avp
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_Controller_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_ServerActor_FaceHugger.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Android/AVP_ServerActor_Android.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Civilian/AVP_ServerActor_Civilian.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Marine/AVP_ServerActor_Marine.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"

#ifdef ASURA_TOOLS
// tools
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_SpawnPoint2
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List< AVP_ServerEntity_SpawnPoint2 > AVP_ServerEntity_SpawnPoint2::s_xList;

AVP_ServerEntity_SpawnPoint2::AVP_ServerEntity_SpawnPoint2( const Asura_Guid uGuid )
: PARENT( uGuid )
, m_uChunkedFlags( 0 )
, m_iMaxInPlay( 1 )
, m_iMaxSpawnCount( -1 )
, m_fSpawnPeriodMin( 1.0f )
, m_fSpawnPeriodMax( 1.0f )
, m_uNumSpawneeTypes( 0 )
, m_pxSpawneeTypes( NULL )
, m_uGSOutNodeGuid( ASURA_GUID_UNREGISTERED )
, m_uGSInNodeGuid( ASURA_GUID_UNREGISTERED )
, m_fSpawnTimer( 0.0f )
, m_uTotalSpawnCount( 0 )
, m_iSpawneeCount( 0 )
, m_iSpawneeArraySize( 0 )
, m_pxSpawneeGuids( NULL )
{
	Classification = AVP_ENTITYCLASS_SPAWNPOINT2;

	m_bUseForDespawning = true;

	SetNeverUpdates( false );
	SetNeverUpdateClient();

	s_xList.AddToList( this );
}

AVP_ServerEntity_SpawnPoint2::~AVP_ServerEntity_SpawnPoint2()
{
	delete [] m_pxSpawneeTypes;
	delete [] m_pxSpawneeGuids;
	m_pxSpawneeTypes = NULL;
	m_pxSpawneeGuids = NULL;
	m_iSpawneeArraySize = 0;

	s_xList.RemoveFromList( this );
}

void AVP_ServerEntity_SpawnPoint2::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	Asura_WriteToStream_Bitfield_Wrapper< u_int > xWrapper( m_uChunkedFlags );
	xWrapper.Add( m_bUseVisibilityTest, 1 );
	xWrapper.Add( m_bUseForDespawning,     1 );
	xStream << xWrapper;

	xStream << m_iMaxInPlay;
	xStream << m_iMaxSpawnCount;
	xStream << m_fSpawnPeriodMin;
	xStream << m_fSpawnPeriodMax;

	xStream << m_uNumSpawneeTypes;
	for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
	{
		xStream << m_pxSpawneeTypes[ u ].m_uBPHashID;
		xStream << m_pxSpawneeTypes[ u ].m_uBPTypeHashID;
		xStream << m_pxSpawneeTypes[ u ].m_fWeight;
	}

	xStream << m_uGSOutNodeGuid;
	xStream << m_uGSInNodeGuid;

	xStream << m_uTotalSpawnCount;
	xStream << m_iSpawneeCount;
	// We don't save the spawnee array size, as we reallocate only as much as we need on load.
	for ( int i = 0; i < m_iSpawneeCount; ++i )
	{
		xStream << m_pxSpawneeGuids[ i ];
	}
}

void AVP_ServerEntity_SpawnPoint2::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream	>> uVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xWrapper( m_uChunkedFlags );
		xStream >> xWrapper;

		xStream >> m_iMaxInPlay;
		xStream >> m_iMaxSpawnCount;
		xStream >> m_fSpawnPeriodMin;
		xStream >> m_fSpawnPeriodMax;

		xStream >> m_uNumSpawneeTypes;
		if ( m_uNumSpawneeTypes > 0 )
		{
			m_pxSpawneeTypes = new BlueprintAndWeight[ m_uNumSpawneeTypes ];
			for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
			{
				xStream >> m_pxSpawneeTypes[ u ].m_uBPHashID;
				xStream >> m_pxSpawneeTypes[ u ].m_uBPTypeHashID;
				xStream >> m_pxSpawneeTypes[ u ].m_fWeight;
			}
		}

		xStream >> m_uGSOutNodeGuid;
		xStream >> m_uGSInNodeGuid;

		xStream >> m_uTotalSpawnCount;
		xStream >> m_iSpawneeCount;
		ReallocateSpawneeArray( m_iSpawneeCount );
		for ( int i = 0; i < m_iSpawneeCount; ++i )
		{
			xStream >> m_pxSpawneeGuids[ i ];
		}
	}
	else if ( uVersion == 1 )
	{
		xStream >> m_iMaxInPlay;
		xStream >> m_iMaxSpawnCount;
		xStream >> m_fSpawnPeriodMin;
		xStream >> m_fSpawnPeriodMax;
		bool bUseVisibilityTest;
		xStream >> bUseVisibilityTest;
		m_bUseVisibilityTest = bUseVisibilityTest;

		xStream >> m_uNumSpawneeTypes;
		if ( m_uNumSpawneeTypes > 0 )
		{
			m_pxSpawneeTypes = new BlueprintAndWeight[ m_uNumSpawneeTypes ];
			for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
			{
				xStream >> m_pxSpawneeTypes[ u ].m_uBPHashID;
				xStream >> m_pxSpawneeTypes[ u ].m_uBPTypeHashID;
				xStream >> m_pxSpawneeTypes[ u ].m_fWeight;
			}
		}

		xStream >> m_uGSOutNodeGuid;
		xStream >> m_uGSInNodeGuid;

		xStream >> m_uTotalSpawnCount;
		xStream >> m_iSpawneeCount;
		ReallocateSpawneeArray( m_iSpawneeCount );
		for ( int i = 0; i < m_iSpawneeCount; ++i )
		{
			xStream >> m_pxSpawneeGuids[ i ];
		}
	}
#ifdef ASURA_TOOLS
	else if ( uVersion == 0 )
	{
		xStream >> m_iMaxInPlay;
		xStream >> m_iMaxSpawnCount;
		xStream >> m_fSpawnPeriodMin;
		xStream >> m_fSpawnPeriodMax;

		xStream >> m_uNumSpawneeTypes;
		if ( m_uNumSpawneeTypes > 0 )
		{
			m_pxSpawneeTypes = new BlueprintAndWeight[ m_uNumSpawneeTypes ];
			for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
			{
				xStream >> m_pxSpawneeTypes[ u ].m_uBPHashID;
				xStream >> m_pxSpawneeTypes[ u ].m_uBPTypeHashID;
				xStream >> m_pxSpawneeTypes[ u ].m_fWeight;
			}
		}

		xStream >> m_uGSOutNodeGuid;
		xStream >> m_uGSInNodeGuid;

		xStream >> m_uTotalSpawnCount;
		xStream >> m_iSpawneeCount;
		ReallocateSpawneeArray( m_iSpawneeCount );
		for ( int i = 0; i < m_iSpawneeCount; ++i )
		{
			xStream >> m_pxSpawneeGuids[ i ];
		}
	}
#endif
	else
	{
		Asura_Assertf( false, ( "AVP_ServerEntity_SpawnPoint2::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

void AVP_ServerEntity_SpawnPoint2::Update()
{
	PARENT::Update();
	m_fSpawnTimer -= Asura_Timers::GetGameTimeSlice();
}

bool AVP_ServerEntity_SpawnPoint2::HandleMessage( const Asura_Message* pxMessage )
{
	if ( pxMessage->GetMsgID() == AVP_MSGID_SPAWN )
	{
		const AVP_Msg_Spawn* pxData = static_cast< const AVP_Msg_Spawn* >( pxMessage->GetData() );

		AVP_SpawnParams xAVPSpawnParams;
		xAVPSpawnParams.m_uBPTypeHashID = pxData->m_uBPTypeHashID;
		xAVPSpawnParams.m_uBPHashID		= pxData->m_uBPHashID;
		xAVPSpawnParams.m_uSquadGuid	= pxData->m_uSquadGuid;

		bool bSupported = false;

		if ( !xAVPSpawnParams.BlueprintSet() )
		{
			bSupported = true;
			PickSpawneeType( xAVPSpawnParams.m_uBPTypeHashID, xAVPSpawnParams.m_uBPHashID );
		}
		else
		{
			for ( u_int u = 0; u < m_uNumSpawneeTypes && !bSupported; ++u )
			{
				if (xAVPSpawnParams.m_uBPTypeHashID == m_pxSpawneeTypes[ u ].m_uBPTypeHashID && 
					xAVPSpawnParams.m_uBPHashID == m_pxSpawneeTypes[ u ].m_uBPHashID)
				{
					bSupported = true;
				}
			}
		}

		if (bSupported)
		{
			Spawn( GetGuid(), &xAVPSpawnParams );
		}
		else
		{
			Asura_Assertf(false, 
				("Spawn point %d: Trying to spawn entity with blueprint %u that is not supported ", GetGuid(), xAVPSpawnParams.m_uBPHashID));
		}
		return true;
	}
	else if ( pxMessage->GetMsgID() == AVP_MSGID_SPAWN_SET_MAX_IN_PLAY )
	{
		const AVP_Msg_Spawn_SetMaxInPlay* pxData = static_cast< const AVP_Msg_Spawn_SetMaxInPlay* >( pxMessage->GetData() );
		m_iMaxInPlay = pxData->m_iMaxInPlay;
		return true;
	}
	else if ( pxMessage->GetMsgID() == AVP_MSGID_SPAWN_SET_MAX_SPAWN_COUNT )
	{
		const AVP_Msg_Spawn_SetMaxSpawnCount* pxData = static_cast< const AVP_Msg_Spawn_SetMaxSpawnCount* >( pxMessage->GetData() );
		m_iMaxSpawnCount = pxData->m_iMaxSpawnCount;
		return true;
	}
	else if ( pxMessage->GetMsgID() == AVP_MSGID_SPAWN_RESET_SPAWN_COUNTER )
	{
		m_uTotalSpawnCount = 0;
		return true;
	}
	else
	{
		return PARENT::HandleMessage( pxMessage );
	}
}

#ifdef ASURA_TOOLS
void AVP_ServerEntity_SpawnPoint2::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance );

	m_uGSOutNodeGuid = pxInstance->ConvertGuid( m_uGSOutNodeGuid );
	m_uGSInNodeGuid  = pxInstance->ConvertGuid( m_uGSInNodeGuid );

	// There shouldn't be any spawnees registered in tools.
}
#endif

void AVP_ServerEntity_SpawnPoint2::ScoreSpawners( const Asura_ScoreParams* pxScoreParams, float& fHighestScore, Asura_Guid& uBestGuid ) const
{
	fHighestScore = -1.0f;
	uBestGuid	  = ASURA_GUID_UNREGISTERED;

	if ( !IsEntityActive() )
	{
		return;
	}
	if ( m_fSpawnTimer > 0.0f )
	{
		return;
	}
	// -1 => 0xffffffff so this test will still work.
	if ( m_uTotalSpawnCount >= static_cast< u_int >( m_iMaxSpawnCount ) )
	{
		return;
	}
	if ( m_iSpawneeCount >= m_iMaxInPlay )
	{
		return;
	}
	if ( !pxScoreParams )
	{
		return;
	}

	const AVP_ScoreParams* pxAVPScoreParams = static_cast< const AVP_ScoreParams* >( pxScoreParams );

	// Check the blueprint matches.
	bool bMatched = false;
	if ( m_uNumSpawneeTypes )
	{
		for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
		{
			if ( pxAVPScoreParams->m_uBPTypeHashID == m_pxSpawneeTypes[ u ].m_uBPTypeHashID &&
				pxAVPScoreParams->m_uBPHashID == m_pxSpawneeTypes[ u ].m_uBPHashID )
			{
				bMatched = true;
				break;
			}
		}

		if ( !bMatched )
		{
			return;
		}
	}

	if (pxAVPScoreParams->m_bTypesMustMach && bMatched == false)
	{
		return;
	}


	// Score ourselves based on the scheme.

	if ( pxAVPScoreParams->m_iSpawnScheme == AVP_SS_RANDOM )
	{
		fHighestScore = 1.0f;
		uBestGuid	  = GetGuid();
	}
	else if ( pxAVPScoreParams->m_iSpawnScheme == AVP_SS_AT_DISTANCE_X )
	{
		Asura_Assert(pxAVPScoreParams->m_fSpawnDistance > 0.0f, "SpawnDistance set to zero");
		if (pxAVPScoreParams->m_fSpawnDistance > 0.0f)
		{
			// Calculate our score based on distance from the position.
			const float fDistance  = ( pxAVPScoreParams->m_xPosition - GetPosition() ).Magnitude();
			// Allow +/- 200% distance.
			const float fTolerance = pxAVPScoreParams->m_fSpawnDistance * 2.0f;

			const float fDistanceFraction = Asura_Maths::ClampToRange( ( fDistance - pxAVPScoreParams->m_fSpawnDistance ) / fTolerance, -1.0f, 1.0f );
			fHighestScore = 1.0f - Asura_Maths::Abs( fDistanceFraction );
			uBestGuid	  = GetGuid();
		}
	}
	else if ( pxAVPScoreParams->m_iSpawnScheme == AVP_SS_CLOSEST )
	{
		const float fDistance  = ( pxAVPScoreParams->m_xPosition - GetPosition() ).Magnitude();
		Asura_Assert(pxAVPScoreParams->m_fSpawnDistance > 0.0f, "SpawnDistance set to zero");
		if (pxAVPScoreParams->m_fSpawnDistance > 0.0f)
		{
			fHighestScore = Asura_Maths::ClampToRange(  1.0f - ( fDistance / pxAVPScoreParams->m_fSpawnDistance), 0.0f, 1.0f );
			uBestGuid = GetGuid();
		}
	}
	else
	{
		Asura_Assert(false, "unknown schema");
	}


	// Check for visibility if necessary.
	if ( fHighestScore > 0.0f && m_bUseVisibilityTest && AVP_ServerEntity_PlayerInfo::IsVisibleByAnyPlayer( GetPosition() ) )
	{
		fHighestScore = 0.0f;
		uBestGuid	  = ASURA_GUID_UNREGISTERED;
	}
}

bool AVP_ServerEntity_SpawnPoint2::OwnsSpawnerGuid( const Asura_Guid uSpawnerGuid ) const
{
	return ( GetGuid() == uSpawnerGuid );
}

Asura_Guid AVP_ServerEntity_SpawnPoint2::Spawn( const Asura_Guid uSpawnerGuid, const Asura_SpawnParams* pxSpawnParams )
{
	if ( !pxSpawnParams )
	{
		return ASURA_GUID_UNREGISTERED;
	}

	const AVP_SpawnParams* pxAVPSpawnParams = static_cast< const AVP_SpawnParams* >( pxSpawnParams );

	AVP_ServerEntity_Actor* pxSpawnedActor = NULL;
	if ( pxAVPSpawnParams->SpawneeGuidSet() )
	{
		// If the spawnee guid is set then this actor should already exist and just need reviving.
		pxSpawnedActor = AVP_ServerEntity_Actor::FindEntity( pxAVPSpawnParams->m_uSpawneeGuid );
	}
	else
	{
		// If the spawnee guid is not set then we need to create this actor from scratch.
		if ( !pxAVPSpawnParams->BlueprintSet() )
		{
			return ASURA_GUID_UNREGISTERED;
		}

		Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( pxAVPSpawnParams->m_uBPTypeHashID, pxAVPSpawnParams->m_uBPHashID );
		if ( !pxBlueprint )
		{
			return ASURA_GUID_UNREGISTERED;
		}

		static const Asura_Hash_ID s_uAlienBPHash      = 0x058996c7; // Asura_GetHashID( "Alien" )
		static const Asura_Hash_ID s_uFaceHuggerBPHash = 0x4633fcb7; // Asura_GetHashID( "FaceHugger" )
		static const Asura_Hash_ID s_uAndroidBPHash    = 0xcca8366f; // Asura_GetHashID( "Android" )
		static const Asura_Hash_ID s_uCivilianBPHash   = 0xadd8d383; // Asura_GetHashID( "Civilian" )
		static const Asura_Hash_ID s_uMarineBPHash     = 0xbf8c8dc2; // Asura_GetHashID( "Marine" )
		static const Asura_Hash_ID s_uPredatorBPHash   = 0xb31282f7; // Asura_GetHashID( "Predator" )

		if ( pxBlueprint->IsChildOf( pxAVPSpawnParams->m_uBPTypeHashID, s_uAlienBPHash, false ) )
		{
			pxSpawnedActor = AVP_ServerActor_Alien::CreateAndReturn( Asura_Entity_System::GetCreatedByGameGuid() );
		}
		else if ( pxBlueprint->IsChildOf( pxAVPSpawnParams->m_uBPTypeHashID, s_uFaceHuggerBPHash, false ) )
		{
			pxSpawnedActor = AVP_ServerActor_FaceHugger::CreateAndReturn( Asura_Entity_System::GetCreatedByGameGuid() );
		}
		else if ( pxBlueprint->IsChildOf( pxAVPSpawnParams->m_uBPTypeHashID, s_uAndroidBPHash, false ) )
		{
			pxSpawnedActor = AVP_ServerActor_Android::CreateAndReturn( Asura_Entity_System::GetCreatedByGameGuid() );
		}
		else if ( pxBlueprint->IsChildOf( pxAVPSpawnParams->m_uBPTypeHashID, s_uCivilianBPHash, false ) )
		{
			pxSpawnedActor = AVP_ServerActor_Civilian::CreateAndReturn( Asura_Entity_System::GetCreatedByGameGuid() );
		}
		else if ( pxBlueprint->IsChildOf( pxAVPSpawnParams->m_uBPTypeHashID, s_uMarineBPHash, false ) )
		{
			pxSpawnedActor = AVP_ServerActor_Marine::CreateAndReturn( Asura_Entity_System::GetCreatedByGameGuid() );
		}
		else if ( pxBlueprint->IsChildOf( pxAVPSpawnParams->m_uBPTypeHashID, s_uPredatorBPHash, false ) )
		{
			pxSpawnedActor = AVP_ServerActor_Predator::CreateAndReturn( Asura_Entity_System::GetCreatedByGameGuid() );
		}

		if ( pxSpawnedActor )
		{
			pxSpawnedActor->SetBlueprintTypeHashID( pxAVPSpawnParams->m_uBPTypeHashID );
			pxSpawnedActor->SetBlueprintHashID( pxAVPSpawnParams->m_uBPHashID );
		}
	}

	// Sort the newly spawned actor out.
	Asura_Guid uSpawneeGuid = ASURA_GUID_UNREGISTERED;
	if ( pxSpawnedActor )
	{
		uSpawneeGuid = pxSpawnedActor->GetGuid();

		pxSpawnedActor->SetInitialGamesceneGuid( m_uGSOutNodeGuid );
		pxSpawnedActor->SetInitialSquadGuid( pxAVPSpawnParams->m_uSquadGuid );
		pxSpawnedActor->SetEntityActive( true );
		if ( pxSpawnedActor->GetPhysicsObjectHandler() )
		{
			pxSpawnedActor->GetPhysicsObjectHandler()->Reset( AVP_Actor_PhysicsObjectHandler::FREE_AI, *pxSpawnedActor->GetCollisionObject(), pxSpawnedActor->GetCollisionProxy() );
		}
		Asura_Matrix_3x3 xOrientM;
		GetOrientation( xOrientM );
		pxSpawnedActor->TeleportTo( GetPosition(), xOrientM, false );
		pxSpawnedActor->MakeAI();
		pxSpawnedActor->CreateAndInitialiseBrain();
		pxSpawnedActor->InitFromBlueprint();
		pxSpawnedActor->PostLoadInitialise();
	}

	// Remember what we've spawned and update our stats if necessary.
	m_fSpawnTimer = Asura_Random::GetFloat( m_fSpawnPeriodMin, m_fSpawnPeriodMax );
	if ( uSpawneeGuid != ASURA_GUID_UNREGISTERED && pxAVPSpawnParams->m_bUpdateStats )
	{
		++m_uTotalSpawnCount;
		ReallocateSpawneeArray( m_iSpawneeCount + 1 );
		m_pxSpawneeGuids[ m_iSpawneeCount++ ] = uSpawneeGuid;

		SendMessagesForEvent( AVP_ENTITY_EVENT_SPAWN_ON_SPAWN );
		if ( m_uTotalSpawnCount == static_cast< u_int >( m_iMaxSpawnCount ) )
		{
			SendMessagesForEvent( AVP_ENTITY_EVENT_SPAWN_ON_SPAWN_COUNT_EXHAUSTED );
		}
	}

	return uSpawneeGuid;
}

const Asura_Vector_3& AVP_ServerEntity_SpawnPoint2::GetGamesceneInPosition() const
{
	Asura_ServerNode_GS2Node* pxNode = Asura_ServerNode_GS2Node::FindEntity( GetGamesceneInNodeGuid() );
	if ( pxNode )
	{
		return pxNode->GetPosition();
	}
	else
	{
		Asura_Assertf( ( GetGamesceneInNodeGuid() == ASURA_GUID_UNREGISTERED ),
			( "AVP_ServerEntity_SpawnPoint2::GetExitPosition - (%d) could not find the GS2 exit node", GetGuid() ) );
		return GetPosition();
	}
}

void AVP_ServerEntity_SpawnPoint2::ActorOutOfPlay( const Asura_Guid uGuid )
{
	// Iterate through all of the spawn points and make sure that this actor is removed from any stats.
	for ( Asura_Hashed_List_It< AVP_ServerEntity_SpawnPoint2 > xSCIter( &s_xList ); !xSCIter.Done(); xSCIter.Next() )
	{
		if ( xSCIter.GetCurrent()->RemoveSpawneeGuidFromStats( uGuid ) )
		{
			break;
		}
	}
}

void AVP_ServerEntity_SpawnPoint2::ScoreAndSpawn( const Asura_ScoreParams* pxScoreParams, const Asura_SpawnParams* pxSpawnParams )
{
	// Generate the score params.
	AVP_ScoreParams xAVPScoreParams;
	GenerateScoreParams( static_cast< const AVP_ScoreParams* >( pxScoreParams ), xAVPScoreParams );

	float	   fHighestScore = 0.0f;
	Asura_Guid uBestGuid     = ASURA_GUID_UNREGISTERED;
	ScoreSpawners( &xAVPScoreParams, fHighestScore, uBestGuid );
	if ( fHighestScore > 0.0f )
	{
		// Generate/override the spawn params.
		AVP_SpawnParams xAVPSpawnParams;
		GenerateSpawnParams( xAVPScoreParams, static_cast< const AVP_SpawnParams* >( pxSpawnParams ), xAVPSpawnParams );

		Spawn( uBestGuid, &xAVPSpawnParams );
	}
}

void AVP_ServerEntity_SpawnPoint2::GenerateScoreParams( const AVP_ScoreParams* pxScoreParamsIn, AVP_ScoreParams& xScoreParamsOut ) const
{
	xScoreParamsOut.m_iSpawnScheme  = AVP_SS_RANDOM;
	xScoreParamsOut.m_uBPTypeHashID = pxScoreParamsIn->m_uBPTypeHashID;
	xScoreParamsOut.m_uBPHashID		= pxScoreParamsIn->m_uBPHashID;
	if ( !pxScoreParamsIn->BlueprintSet() )
	{
		PickSpawneeType( xScoreParamsOut.m_uBPTypeHashID, xScoreParamsOut.m_uBPHashID );
	}
}

void AVP_ServerEntity_SpawnPoint2::GenerateSpawnParams( const AVP_ScoreParams& xScoreParamsIn,
														const AVP_SpawnParams* pxSpawnParamsIn,
														AVP_SpawnParams& xSpawnParamsOut ) const
{
	xSpawnParamsOut.m_uBPTypeHashID = xScoreParamsIn.m_uBPTypeHashID;
	xSpawnParamsOut.m_uBPHashID		= xScoreParamsIn.m_uBPHashID;
	xSpawnParamsOut.m_uSquadGuid	= pxSpawnParamsIn->m_uSquadGuid;
}

void AVP_ServerEntity_SpawnPoint2::PickSpawneeType( Asura_Hash_ID& uBPTypeHashID, Asura_Hash_ID& uBPHashID ) const
{
	if ( m_uNumSpawneeTypes )
	{
		float fTotalWeighting = 0.0f;
		for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
		{
			fTotalWeighting += m_pxSpawneeTypes[ u ].m_fWeight;
		}

		const float fRandomWeight = Asura_Random::GetFloat( 0.0f, fTotalWeighting );
		float fCumulativeWeighting = 0.0f;
		for ( u_int u = 0; u < m_uNumSpawneeTypes; ++u )
		{
			fCumulativeWeighting += m_pxSpawneeTypes[ u ].m_fWeight;
			if ( fRandomWeight <= fCumulativeWeighting )
			{
				uBPTypeHashID = m_pxSpawneeTypes[ u ].m_uBPTypeHashID;
				uBPHashID	  = m_pxSpawneeTypes[ u ].m_uBPHashID;
				break;
			}
		}
	}
	else
	{
		uBPTypeHashID = ASURA_HASH_ID_UNSET;
		uBPHashID	  = ASURA_HASH_ID_UNSET;
	}
}

bool AVP_ServerEntity_SpawnPoint2::RemoveSpawneeGuidFromStats( const Asura_Guid uGuid )
{
	for ( int i = 0; i < m_iSpawneeCount; ++i )
	{
		if ( m_pxSpawneeGuids[ i ] == uGuid )
		{
			m_pxSpawneeGuids[ i ] = m_pxSpawneeGuids[ --m_iSpawneeCount ];
			// We could reallocate the spawnee array here to save memory, but we want to avoid lots of news and deletes.
			SendMessagesForEvent( AVP_ENTITY_EVENT_SPAWN_ON_ANY_SPAWNEE_DEATH );

			if ( m_iSpawneeCount == 0 )
			{
				SendMessagesForEvent( AVP_ENTITY_EVENT_SPAWN_ON_ALL_SPAWNEES_DEAD );
			}
			return true;
		}
	}

	return false;
}

void AVP_ServerEntity_SpawnPoint2::ReallocateSpawneeArray( const int iSize )
{
	if ( iSize == 0 )
	{
		m_iSpawneeArraySize = 0;
		delete [] m_pxSpawneeGuids;
		m_pxSpawneeGuids = NULL;
		m_iSpawneeCount = 0;
	}
	else
	{
		enum { ARRAY_ROUNDUP_SIZE = 4 };

		// Round this up to the nearest ARRAY_ROUNDUP_SIZE.
		const int iRoundedUpSize = ( ( iSize / ARRAY_ROUNDUP_SIZE ) + 1 ) * ARRAY_ROUNDUP_SIZE;
		if ( iRoundedUpSize == m_iSpawneeArraySize )
		{
			return;
		}

		Asura_Guid* pxNewArray = new Asura_Guid[ iRoundedUpSize ];
		// Copy over the old spawnee guids if they exist.
		if ( m_pxSpawneeGuids )
		{
			for ( int i = 0; i < m_iSpawneeCount && i < iRoundedUpSize; ++i )
			{
				pxNewArray[ i ] = m_pxSpawneeGuids[ i ];
			}
		}

		m_iSpawneeArraySize = iRoundedUpSize;
		delete [] m_pxSpawneeGuids;
		m_pxSpawneeGuids = pxNewArray;
		m_iSpawneeCount = Asura_Maths::Min( m_iSpawneeCount, m_iSpawneeArraySize );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
