//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_SpawnPoint2.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AVP_SERVERENTITY_SPAWNPOINT2_H__
#define __AVP_SERVERENTITY_SPAWNPOINT2_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// Asura
#include "Core/Asura_Maths.h"
#include "Entity/Locations/OrientedNode/Asura_ServerNode_OrientedNode.h"

// AVP
#include "Entity/Spawners/AVP_SpawnerInterface.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Tools_EntityTemplateInstance;

//////////////////////////////////////////////////////////////////////////////
//
// AVP_ServerEntity_SpawnPoint2
//
//////////////////////////////////////////////////////////////////////////////

class AVP_ServerEntity_SpawnPoint2 : public Asura_ServerNode_OrientedNode, public AVP_SpawnerInterface
{
	typedef Asura_ServerNode_OrientedNode PARENT;
	friend class ToolsAVP_Entity_SpawnPoint2;

public:
	AVP_ServerEntity_SpawnPoint2( const Asura_Guid uGuid );
	virtual ~AVP_ServerEntity_SpawnPoint2();

	virtual bool HasClientEntities()	   const { return false; }
	virtual bool CanBeSavedToChunk()	   const { return true; }
	virtual bool IsFilledFromChunkBuffer() const { return false; }
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );

	virtual void Update();
	virtual bool HandleMessage( const Asura_Message* pxMessage );

#ifdef ASURA_TOOLS
	virtual bool CanBeIncludedInTemplate() const { return true; }
	virtual void ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance );
#endif

	// Spawner interface.
	virtual void 		ScoreSpawners( const Asura_ScoreParams* pxScoreParams, float& fHighestScore, Asura_Guid& uBestGuid ) const;
	virtual bool 		OwnsSpawnerGuid( const Asura_Guid uSpawnerGuid ) const;
	virtual Asura_Guid 	Spawn( const Asura_Guid uSpawnerGuid, const Asura_SpawnParams* pxSpawnParams );

	Asura_Guid GetGamesceneInNodeGuid()  const { return m_uGSInNodeGuid; }
	Asura_Guid GetGamesceneOutNodeGuid() const { return m_uGSOutNodeGuid; }
	const Asura_Vector_3& GetGamesceneInPosition() const;
	bool GetUseForDespawning() const { return m_bUseForDespawning; }

	static void ActorOutOfPlay( const Asura_Guid uGuid );

	static AVP_ServerEntity_SpawnPoint2* FindEntity( const Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }
	static Asura_Hashed_List< AVP_ServerEntity_SpawnPoint2 >* GetList() { return &s_xList; }

private:
	void ScoreAndSpawn( const Asura_ScoreParams* pxScoreParams, const Asura_SpawnParams* pxSpawnParams );
	void GenerateScoreParams( const AVP_ScoreParams* pxScoreParamsIn, AVP_ScoreParams& xScoreParamsOut ) const;
	void GenerateSpawnParams( const AVP_ScoreParams& xScoreParamsIn, const AVP_SpawnParams* pxSpawnParamsIn, AVP_SpawnParams& xSpawnParamsOut ) const;
	void PickSpawneeType( Asura_Hash_ID& uBPTypeHashID, Asura_Hash_ID& uBPHashID ) const;
	bool RemoveSpawneeGuidFromStats( const Asura_Guid uGuid );

	void ReallocateSpawneeArray( const int iSize );

private:
	struct BlueprintAndWeight
	{
		Asura_Hash_ID m_uBPHashID;
		Asura_Hash_ID m_uBPTypeHashID;
		float		  m_fWeight;
	};

	union
	{
		u_int m_uChunkedFlags;
		struct 
		{
			u_int m_bUseVisibilityTest : 1,
				  m_bUseForDespawning  : 1;
		};
	};

	int					m_iMaxInPlay;
	int					m_iMaxSpawnCount;
	float				m_fSpawnPeriodMin;
	float				m_fSpawnPeriodMax;
	u_int				m_uNumSpawneeTypes;
	BlueprintAndWeight* m_pxSpawneeTypes;

	Asura_Guid			m_uGSOutNodeGuid;
	Asura_Guid			m_uGSInNodeGuid;

	float				m_fSpawnTimer;
	u_int				m_uTotalSpawnCount;
	// Number of spawnees in play currently.
	int					m_iSpawneeCount;
	int					m_iSpawneeArraySize;
	Asura_Guid*			m_pxSpawneeGuids;

private:
	static Asura_Hashed_List< AVP_ServerEntity_SpawnPoint2 > s_xList;

	static const u_int uCURRENT_CHUNK_VERSION = 2;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AVP_SERVERENTITY_SPAWNPOINT2_H__
