//--------------------------------------------------------------------------------------
//	File	:	AVP_ServerEntity_HumanSpawnPoint.cpp
//	Created	:	25 / 02 / 08
//	Author	:	Gaz Thomas
//	Copyright (C) 2006 Rebellion, All Rights Reserved.
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
//	INCLUDES
//--------------------------------------------------------------------------------------

// Always include Core first
#include "Core/Asura.h"

// This Files Header
#include "Entity/Spawners/Human/AVP_ServerEntity_HumanSpawnPoint.h"

// Asura include files
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

// AVP Include files
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_Controller_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Android/AVP_ServerActor_Android.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Civilian/AVP_ServerActor_Civilian.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/Marine/AVP_ServerActor_Marine.h"

#ifdef ASURA_TOOLS
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#endif

//--------------------------------------------------------------------------------------
//	Statics
//--------------------------------------------------------------------------------------
Asura_Hashed_List<AVP_ServerEntity_HumanSpawnPoint> AVP_ServerEntity_HumanSpawnPoint::s_xList;

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_HumanSpawnPoint::AVP_ServerEntity_HumanSpawnPoint
//	Created		:	25 / 02 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerEntity_HumanSpawnPoint::AVP_ServerEntity_HumanSpawnPoint(const Asura_Guid uGuid)
: PARENT( uGuid )
, m_uCivilianGuardZoneGuid(ASURA_GUID_UNREGISTERED)
{
	Classification = AVP_ENTITYCLASS_HUMANSPAWNPOINT;

	SetNeverUpdates( true );
	SetNeverUpdateClient();

	s_xList.AddToList( this );
}
/// End of function 'AVP_ServerEntity_HumanSpawnPoint::AVP_ServerEntity_HumanSpawnPoint'. ///

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_HumanSpawnPoint::Spawn
//	Created		:	25 / 02 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerActor_Human* AVP_ServerEntity_HumanSpawnPoint::Spawn(Asura_Hash_ID uBlueprintTypeHashID, Asura_Hash_ID uBlueprintHashID, u_int uClassification, Asura_Guid uSquadControllerGuid)
{
	if ((uBlueprintTypeHashID == ASURA_HASH_ID_UNSET) || (uBlueprintHashID == ASURA_HASH_ID_UNSET))
	{
		Asura_Assert(false, "No Blueprint ID Given For Spawned Civilian");
		return NULL;
	}

	// Get blueprint
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uBlueprintTypeHashID, uBlueprintHashID);
	if (!pxBlueprint)
	{
		Asura_Assert(false, "Couldn't Find Blueprint For Spawned Civilian");
		return NULL;
	}

	AVP_ServerActor_Human* pxSpawnedHumanoid = CreateSpecifiedHumanoidType(uClassification);

	if (pxSpawnedHumanoid)
	{
		SetupSpawnedHuman(pxSpawnedHumanoid, uBlueprintTypeHashID, uBlueprintHashID);
		
		// If this is an android or a marine and a squad guid was specified send the server message to add it to the squad.
		if ((uSquadControllerGuid != ASURA_GUID_UNREGISTERED) && ((uClassification == AVP_ENTITYCLASS_MARINE) || (uClassification == AVP_ENTITYCLASS_ANDROID)))
		{
			SendAddToSquadMessage(uSquadControllerGuid, pxSpawnedHumanoid->GetGuid());
		}

		if ((uClassification == AVP_ENTITYCLASS_CIVILIAN) && (GetCivilianGuardZoneGuid() != ASURA_GUID_UNREGISTERED))
		{
			AVP_ServerActor_Civilian* pxCivilian = (AVP_ServerActor_Civilian*)pxSpawnedHumanoid;

			if (pxCivilian)
			{
				pxCivilian->SetGuardzoneGuid(GetCivilianGuardZoneGuid());
			}
		}

		return pxSpawnedHumanoid;
	}

	Asura_Assert(false, "AVP_ServerEntity_HumanSpawnPoint :: Spawn - Couldn't Create Spawned Humanoid" );
	return NULL;
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_HumanSpawnPoint::SetupSpawnedHuman
//	Created		:	25 / 02 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_HumanSpawnPoint::SetupSpawnedHuman(AVP_ServerActor_Human* pxSpawnedHumanoid, Asura_Hash_ID uBlueprintTypeHashID, Asura_Hash_ID uBlueprintHashID)
{
	// Transform it into AI
	pxSpawnedHumanoid->SetEntityActive(true);
	pxSpawnedHumanoid->MakeAI();

	// Reset the physics object type
	if (pxSpawnedHumanoid->GetPhysicsObjectHandler())
	{
		pxSpawnedHumanoid->GetPhysicsObjectHandler()->Reset(AVP_Actor_PhysicsObjectHandler::FREE_AI, *pxSpawnedHumanoid->GetCollisionObject(), pxSpawnedHumanoid->GetCollisionProxy());
	}

	// Initalise from the blueprint
	pxSpawnedHumanoid->SetBlueprintTypeHashID(uBlueprintTypeHashID);
	pxSpawnedHumanoid->SetBlueprintHashID(uBlueprintHashID);
	pxSpawnedHumanoid->InitFromBlueprint();

	// If this spawn point has an 'out' gamescene guid, set it as the alien's initial gamescene guid
	if (m_uGameSceneOutGuid != ASURA_GUID_UNREGISTERED)
	{
		pxSpawnedHumanoid->SetInitialGamesceneGuid(m_uGameSceneOutGuid);
	}

	// Don't inform clients as the client hasn't been created yet
	pxSpawnedHumanoid->TeleportTo(GetPosition(), GetOrientation(), false);

	// If we spawn an actor we have to call post load initialise
	pxSpawnedHumanoid->PostLoadInitialise();

}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_HumanSpawnPoint::CreateSpecifiedHumanoidType
//	Created		:	25 / 02 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerActor_Human* AVP_ServerEntity_HumanSpawnPoint::CreateSpecifiedHumanoidType(u_int uClassification)
{
	switch (uClassification)
	{
		case AVP_ENTITYCLASS_ANDROID:
		{
			return (static_cast<AVP_ServerActor_Human*>(AVP_ServerActor_Android::CreateAndReturn(Asura_Entity_System::GetCreatedByGameGuid())));

		}

		case AVP_ENTITYCLASS_CIVILIAN:
		{
			return (static_cast<AVP_ServerActor_Human*>(AVP_ServerActor_Civilian::CreateAndReturn(Asura_Entity_System::GetCreatedByGameGuid())));

		}

		case AVP_ENTITYCLASS_MARINE:
		{
			return (static_cast<AVP_ServerActor_Human*>(AVP_ServerActor_Marine::CreateAndReturn(Asura_Entity_System::GetCreatedByGameGuid())));

		}

		default:
		{
			Asura_Assert(false, "AVP_ServerEntity_HumanSpawnPoint :: Spawn - Invalid Entity Classification To Try And Spawn At A Humanoid Spawn Point");
			return NULL;

		}
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_HumanSpawnPoint::SendAddToSquadMessage
//	Created		:	25 / 02 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_HumanSpawnPoint::SendAddToSquadMessage(Asura_Guid uSquadGuid, Asura_Guid uHumanoidGuid)
{
	Asura_Message *pxMessage = Asura_Network::CreateMessage(AVP_MSGID_ADD_SQUAD_MEMBER, sizeof(AVP_Message_AddSquadMember));

	if (pxMessage)
	{
		AVP_Message_AddSquadMember* pxMsgData = static_cast<AVP_Message_AddSquadMember*>(pxMessage->GetData());

		if (pxMsgData)
		{
			pxMsgData->SetGuidToAdd(uHumanoidGuid);
		}

		pxMessage->SetToGuid(uSquadGuid);
		pxMessage->SetFromGuid(GetGuid());
		Asura_Network::Send(pxMessage);
	}
}

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_HumanSpawnPoint::~AVP_ServerEntity_HumanSpawnPoint
//	Created		:	25 / 02 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerEntity_HumanSpawnPoint::~AVP_ServerEntity_HumanSpawnPoint()
{
	s_xList.RemoveFromList(this);
}
/// End of function 'AVP_ServerEntity_HumanSpawnPoint::~AVP_ServerEntity_HumanSpawnPoint'. ///


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_HumanSpawnPoint::Create
//	Created		:	25 / 02 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
AVP_ServerEntity_HumanSpawnPoint* AVP_ServerEntity_HumanSpawnPoint::Create(const Asura_Guid uGuid)
{
	return(new AVP_ServerEntity_HumanSpawnPoint(uGuid));
}
/// End of function 'AVP_ServerEntity_HumanSpawnPoint::Create'. ///


//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_HumanSpawnPoint::WriteToChunkStream
//	Created		:	25 / 02 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_HumanSpawnPoint::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	//Version
	xStream << CURRENT_CHUNK_VERSION;

	//Parent
	PARENT::WriteToChunkStream(xStream);

	xStream << m_uCivilianGuardZoneGuid;
}
/// End of function 'AVP_ServerEntity_HumanSpawnPoint::WriteToChunkStream'. ///

//--------------------------------------------------------------------------------------
//	Function	:	AVP_ServerEntity_HumanSpawnPoint::ReadFromChunkStream
//	Created		:	25 / 02 / 08
//	Author		:	Gareth Thomas
//--------------------------------------------------------------------------------------
void AVP_ServerEntity_HumanSpawnPoint::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	//Version
	u_int uVersion;
	xStream	>> uVersion;

	//Parent
	PARENT::ReadFromChunkStream(xStream);

	if (uVersion == 0)
	{
		m_uCivilianGuardZoneGuid = ASURA_GUID_UNREGISTERED;
	}
	else if (uVersion == CURRENT_CHUNK_VERSION)
	{
		xStream >> m_uCivilianGuardZoneGuid;
	}
}
/// End of function 'AVP_ServerEntity_HumanSpawnPoint::ReadFromChunkStream'. ///

#ifdef ASURA_TOOLS
bool AVP_ServerEntity_HumanSpawnPoint::CanBeIncludedInTemplate() const
{
	return ( Classification == AVP_ENTITYCLASS_HUMANSPAWNPOINT );
}

void AVP_ServerEntity_HumanSpawnPoint::ApplyChangesForTemplateInstance( Tools_EntityTemplateInstance* pxInstance )
{
	PARENT::ApplyChangesForTemplateInstance( pxInstance );

	// Not sure what to do with the m_uCivilianGuardZoneGuid...
}
#endif
