/**
*
* \file AVP_ServerEntity_VolumeTrigger.cpp
*
* Created by Anthony Gregson - 11/12/07
*
* Copyright (C) 2007 Rebellion, All Rights Reserved.
*
**/

#include "Core/Asura.h"

#include "Entity/Triggers/AVP_ServerEntity_VolumeTrigger.h"

#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

#include "Core/AVP_Game_Core.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ServerActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AlienQueen/AVP_ServerActor_AlienQueen.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"

#ifdef ASURA_TOOLS
const char* AVP_ServerEntity_VolumeTrigger::s_aszAVPVolumeTypes[NUM_AVP_VOLUMETRIGGER_TYPES] = 
{
    "Specific Object",
	"Player Alien",
	"Player Human",
	"Player Predator",
	"AI Alien",
	"AI Human",
	"AI Predator",
	"Any Player",
	"AI AlienQueen",
};
#endif //ASURA_TOOLS


Asura_Hashed_List<AVP_ServerEntity_VolumeTrigger> AVP_ServerEntity_VolumeTrigger::s_xList(3);

AVP_ServerEntity_VolumeTrigger::AVP_ServerEntity_VolumeTrigger(Asura_Guid uGuid)
: Asura_ServerEntity_VolumeTrigger(uGuid)
, m_uTriggerTypeFlags(1<<AVP_VTT_ANY_PLAYER)
{
	s_xList.AddToList(this);
}


AVP_ServerEntity_VolumeTrigger::~AVP_ServerEntity_VolumeTrigger()
{
	s_xList.RemoveFromList(this);
}

bool AVP_ServerEntity_VolumeTrigger::IsVolumeOccupied() const
{
	u_int uTriggerTypeFlagsToTest = m_uTriggerTypeFlags;
	if(uTriggerTypeFlagsToTest & (1<<AVP_VTT_ANY_PLAYER))
	{
		//If the 'any player' flag is set then test each of alien,human and predator instead.
		uTriggerTypeFlagsToTest |= (1<<AVP_VTT_PLAYER_ALIEN)|(1<<AVP_VTT_PLAYER_HUMAN)|(1<<AVP_VTT_PLAYER_PREDATOR);
		uTriggerTypeFlagsToTest &=~ (1<<AVP_VTT_ANY_PLAYER);
	}
	for (int i=0;i<NUM_AVP_VOLUMETRIGGER_TYPES;i++)
	{
		if (uTriggerTypeFlagsToTest & (1 << i))
		{
			switch (i)
			{
                case AVP_VTT_SPECIFICGUID:
                {
                    const AVP_ServerEntity_PhysicalObject* pxPhysicalObject = AVP_ServerEntity_PhysicalObject::FindEntity(m_uSpecificGuid);
                    if(pxPhysicalObject)
                    {
                        return ContainsEntity(*pxPhysicalObject);
                    }
                    else
                    {
						// This might be a player guid. Find out.
						const AVP_ServerEntity_PlayerInfo* pxPlayer = AVP_ServerEntity_PlayerInfo::FindEntity(m_uSpecificGuid);
						if (pxPlayer)
						{
							const Asura_Guid	uPuppetGuid	= pxPlayer->GetPuppetGuid();

							pxPhysicalObject = AVP_ServerEntity_PhysicalObject::FindEntity(uPuppetGuid);
							if(pxPhysicalObject)
							{
								return ContainsEntity(*pxPhysicalObject);
							}
							else
							{
								Asura_Assertf(false, ("AVP_ServerEntity_VolumeTrigger::IsVolumeOccupied() - could not find specific puppet (GUID %d)", uPuppetGuid));
							}
						}
						else
						{
							Asura_Assertf(false, ("AVP_ServerEntity_VolumeTrigger::IsVolumeOccupied() - could not find specific player (GUID %d)", m_uSpecificGuid));
						}
                    }

                    break;
                }

				case AVP_VTT_PLAYER_ALIEN:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerEntity_PlayerInfo> xIt(AVP_ServerEntity_PlayerInfo::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerEntity_PlayerInfo* pxPlayer = xIt.GetCurrent();
						const AVP_ServerActor_Alien* pxPlayerAlien = AVP_ServerActor_Alien::FindEntity(pxPlayer->GetPuppetGuid());

						if(pxPlayerAlien && ContainsEntity(*pxPlayerAlien))
						{
							return true;
						}
					}
					break;
				}

				case AVP_VTT_PLAYER_HUMAN:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerEntity_PlayerInfo> xIt(AVP_ServerEntity_PlayerInfo::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerEntity_PlayerInfo* pxPlayer = xIt.GetCurrent();
						const AVP_ServerActor_Human* pxPlayerHuman = AVP_ServerActor_Human::FindEntity(pxPlayer->GetPuppetGuid());

						if(pxPlayerHuman && ContainsEntity(*pxPlayerHuman))
						{
							return true;
						}
					}
					break;
				}

				case AVP_VTT_PLAYER_PREDATOR:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerEntity_PlayerInfo> xIt(AVP_ServerEntity_PlayerInfo::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerEntity_PlayerInfo* pxPlayer = xIt.GetCurrent();
						const AVP_ServerActor_Predator* pxPlayerPredator = AVP_ServerActor_Predator::FindEntity(pxPlayer->GetPuppetGuid());

						if(pxPlayerPredator && ContainsEntity(*pxPlayerPredator))
						{
							return true;
						}
					}
					break;
				}

				case AVP_VTT_AI_ALIEN:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerActor_Alien> xIt(AVP_ServerActor_Alien::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerActor_Alien* pxAlien = xIt.GetCurrent();
						const bool bIsPlayer = pxAlien->IsPlayer();

						if(!bIsPlayer && ContainsEntity(*pxAlien))
						{
							return true;
						}
					}
					break;
				}

				case AVP_VTT_AI_HUMAN:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerActor_Human> xIt(AVP_ServerActor_Human::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerActor_Human* pxHuman = xIt.GetCurrent();
						const bool bIsPlayer = pxHuman->IsPlayer();

						if(!bIsPlayer && ContainsEntity(*pxHuman))
						{
							return true;
						}
					}
					break;
				}

				case AVP_VTT_AI_PREDATOR:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerActor_Predator> xIt(AVP_ServerActor_Predator::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerActor_Predator* pxPredator = xIt.GetCurrent();
						const bool bIsPlayer = pxPredator->IsPlayer();

						if(!bIsPlayer && ContainsEntity(*pxPredator))
						{
							return true;
						}
					}
					break;
				}

				case AVP_VTT_AI_ALIENQUEEN:
				{
					for(Asura_Hashed_List_ConstIt<AVP_ServerActor_AlienQueen> xIt(AVP_ServerActor_AlienQueen::GetList()); !xIt.Done(); xIt.Next())
					{
						const AVP_ServerActor_AlienQueen* pxAlienQueen = xIt.GetCurrent();
						const bool bIsPlayer = pxAlienQueen->IsPlayer();

						if(!bIsPlayer && ContainsEntity(*pxAlienQueen))
						{
							return true;
						}
					}
					break;
				}

				default:
				{
					Asura_Assert(false, "Unknown triggering type for volume trigger");
					break;
				}
			}		// End of switch.
		}
	}
	return false;
}

void AVP_ServerEntity_VolumeTrigger::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	const int iVersion = iAVP_VOLUME_TRIGGER_CHUNK_VERSION;
	xStream << iVersion;

	PARENT::WriteToChunkStream(xStream);

	xStream << m_uTriggerTypeFlags;
}

void AVP_ServerEntity_VolumeTrigger::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	int iVersion;
	xStream >> iVersion;
	Asura_Assert(iVersion <= iAVP_VOLUME_TRIGGER_CHUNK_VERSION, "Unexpected version for AVP volume trigger");

	PARENT::ReadFromChunkStream(xStream);

	xStream >> m_uTriggerTypeFlags;
}

void AVP_ServerEntity_VolumeTrigger::Create(Asura_Guid uGuid, Asura_Chunk_Stream& xStream)
{
	AVP_ServerEntity_VolumeTrigger* pxEntity = new AVP_ServerEntity_VolumeTrigger(uGuid);
	if (pxEntity)
	{
		pxEntity->ReadFromChunkStream(xStream);
	}
}

#ifdef ASURA_TOOLS
AVP_ServerEntity_VolumeTrigger* AVP_ServerEntity_VolumeTrigger::CreateFromTools(Asura_Guid uGuid)
{
	return new AVP_ServerEntity_VolumeTrigger(uGuid);
}
#endif //#ifdef ASURA_TOOLS

bool AVP_ServerEntity_VolumeTrigger::ContainsEntity(const AVP_ServerEntity_PhysicalObject& xEntity) const
{
	// First make sure we actually want to test the position data. Dead and inactive entities don't count.
	if (!xEntity.IsEntityActive() || (xEntity.IsDead() && (AVP_Settings::IsMultiPlayer() || !AVP_Game_Core::IsLocalPlayer(xEntity.GetGuid()))))
	{
		return false;
	}

	// Now that's out of the way, we can do the usual test.
	return Contains(xEntity.GetPosition());
}
