//////////////////////////////////////////////////////////////////////////////
//
// AVP_DevLobby.h
//
// Description: similar to the dev front end, providing a way to knock up some
//              lobby and matchmaking functionality quickly during development.
//              Made it a separate system for clarity and maintainability.
// 
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef AVP_DEVLOBBY_H___
#define AVP_DEVLOBBY_H___ 1

#if defined(AVP_USE_DEV_LOBBY)

// forward declarations
class Asura_Lobby_SearchCriterion;

// include game lobby for use of shared enums
#include "FrontEnd/AVP_GameLobby.h"

// C L A S S   D E C L A R A T I O N S ///////////////////////////////////////
class AVP_DevLobby_GameSettings
{
public:

	AVP_DevLobby_GameSettings();

	// public data
	u_int m_uMaxPlayers;
	int m_iMapIndex;
	int m_iGametypeIndex;
	int m_iLocalSpecies;
	bool m_bAllowMixedTeams; 
	int m_iSpeciesRatioPredator; 
	int m_iSpeciesRatioHuman;
	int m_iSpeciesRatioAlien;
	int m_iNumberOfTeams;
	int m_iTeamOneSpecies; 
	int m_iTeamTwoSpecies; 
	int m_iTeamThreeSpecies; 
	bool m_bRespawnUponDeath;
	int m_iRespawnType;
	int m_iLives;
	bool m_bFriendlyFire;
	int m_iScoreLimit;
	int m_iTimeLimit; 
};

class AVP_DevLobby_MapData
{
public:
	AVP_DevLobby_MapData(const char* szFilename);

	const char* GetFilename() const { return m_acFilename; }
	const char* GetDisplayName() const { return m_acDisplayName; } // TODO - this won't be good enough for the final game.
	Asura_Hash_ID GetHash() const { return m_uHash; }

private:
	char m_acFilename[256];
	char m_acDisplayName[32]; // TODO - this won't be good enough for the final game.
	Asura_Hash_ID m_uHash;
};

class AVP_DevLobby
{
public:

	static void Initialise();
	static void AddConsoleCmdsAndVars();
	static void Shutdown();

	// Update returns true when we're finished. Then you just check whether
	// quit was requested, and if not assume that the game settings are all
	// made and you proceed into the game.
	static bool Update(); 
	static void Render();

	static bool IsQuitRequested() { return s_bQuitRequested; }

	// Map lists
	static int GetMapIndexByHash(Asura_Hash_ID uHash);
	static Asura_Hash_ID GetMapHashByIndex(int iIndex);
	static const char* GetMapFilename(int iIndex);

	// accessors for project Asura_Matchmaking_System code
	static bool IsInitialised() { return s_bIsInitialised; }
	static void CreateGameForAutomatch();
	static void SetSearchFilters( u_int uSkillThreshold, Asura_Lobby_SearchCriterion* axFilters, u_int& uNumFilters );

private:

	static void Platform_Initialise();
	static void Platform_Shutdown();

	static void UpdateLobbySettings();
	static void InitialiseGameSettings();
	static void UpdateTeamNames();
	
	static void SetLobbyVoiceChannelsSettings();

	static void CreateMapList();
	static void CreateMapList_Callback(const char* szFilename);
	static void ClearMapList();

	// console commands for GUIMenu interface
	static bool RequestQuit( bool bHelp ) { if (!bHelp) { s_bQuitRequested = true; } return true; }
	static bool CreateGame( bool bHelp );
	static bool SearchForGame( bool bHelp );
	static bool BrowseForGame( bool bHelp );
	static bool JoinGame( u_int uGameIndex, bool bInterrogate, bool bHelp );
	static bool ListGame( u_int uGameIndex, bool bInterrogate, bool bHelp );
	static bool StartGame( bool bHelp );
	static bool ChangeSpecies( bool bHelp );
	static bool ChangeTeam( bool bHelp );
	static bool AlterGameType( bool bHelp ); 
	static bool ApplyAdvancedSettings( bool bHelp ); 
	static bool ConsoleCmd_AllowChangingTeam( bool bHelp );
	static bool ConsoleCmd_AllowChangingSpecies( bool bHelp );
	static bool ConsoleCmd_IsRoleBasedGame( bool bHelp );
	static bool PopulateMapSelectorName(u_int uElementIndex, bool bInterrogate, bool bHelp); // For menu items where the user can choose the map.
	static bool PopulateMapSearchName(u_int uElementIndex, bool bInterrogate, bool bHelp); // For menu items where the user can choose the map.
	static bool PopulateMapDisplayName(u_int uElementIndex, bool bInterrogate, bool bHelp); // For menu items where the user is told what map it is.

	static bool PopulateMapName(u_int uElementIndex, bool bIncludeAny, bool bIncludeUnknown);

	// internal checkings
	static bool AllowChangingTeam();
	static bool AllowChangingSpecies();
	static int	CalculateAvailableSpeciesSlotsInTeam( int iSpecies, int iTeam );

	static bool s_bIsInitialised;
	static bool s_bSetupComplete;
	static bool s_bQuitRequested;
	static bool s_bAllowRankedMatch;

	static AVP_DevLobby_GameSettings s_xLobbySettings;
	static int s_aiLobbySearchFilters[AVP_NUM_LOBBY_ATTRIBUTES];

	// hack to restrict available gametypes for milestone builds
	static void CheckGametypeIsAllowed();
	static bool IsGametypeAllowed(int iGametype);
	static bool s_bGametypeChanged;
	static int s_iPrevGametype;
	static bool s_bAutoJoinOnSearchComplete;

	static Asura_LinkList<AVP_DevLobby_MapData*> s_xMapList;
};

#endif	//defined(AVP_USE_DEV_LOBBY)

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef AVP_DEVLOBBY_H___
