////////////////////////////////////////////////////////
// 
// AVP_DevelopmentFrontEnd.h
//
// Development front-end. Supports quick starting of any 
// level or game mode via a text-only interface by-passing 
// the full front-end.
//
/////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __AVP_DEVELOPMENTFRONTEND_H__
#define __AVP_DEVELOPMENTFRONTEND_H__

#include "Core/Asura_PlatformBasics.h"
#include "Core/Asura_String.h"
#include "Core/Asura_List.h"
#include "Core/Asura_Maths.h"

#include "Core/AVP_Settings.h"

#if defined AVP_USE_DEVELOPMENT_FRONT_END
#if !defined(SN_TARGET_PSP_PRX)
#define AVP_DFE_ALLOW_QUIT
#endif // #if defined(AVP_SUBMISSION)

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////
class Asura_ControlMap;

// C O N S T A N T S /////////////////////////////////////////////////////////
static const u_int NUM_AVP_LEVELS = 1;
static const u_int NUM_AVP_MISSIONS = 1;
static const int iAVP_DFE_MAXPATHLENGTH = 256;
#if defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
static const int iAVP_DFE_NUM_WILDCARDS = 1;
#else
static const int iAVP_DFE_NUM_WILDCARDS = 14;
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
static const int iAVP_DFE_NUM_SMOKETESTFLAGS = 8;
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
#endif // defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

// E N U M S /////////////////////////////////////////////////////////////////
enum AVP_DFE_MENUMODE
{
	AVP_DFE_MENUMODE_GAMESETUP,
	AVP_DFE_MENUMODE_STARTING_NETWORK,
	AVP_DFE_MENUMODE_BUILD_FMV_LIST,

	NUM_DFE_MENUMODES
};

enum AVP_DFE_BUTTON
{
	AVP_DFE_BUTTON_UP,
	AVP_DFE_BUTTON_DOWN,
	AVP_DFE_BUTTON_LEFT,
	AVP_DFE_BUTTON_RIGHT,
	AVP_DFE_BUTTON_SELECT,
	AVP_DFE_BUTTON_CANCEL,
	AVP_DFE_BUTTON_QUIT, // well not really a button but near enough

	NUM_AVP_DFE_BUTTON
};

enum AVP_DFE_OPTIONS
{
#if defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND) 
	AVP_DFE_OPTION_GAME_MODE,
	AVP_DFE_OPTION_GAME_TYPE,
	AVP_DFE_OPTION_CHAR_TYPE,
	AVP_DFE_OPTION_LEVEL_SELECTION,
	AVP_DFE_OPTION_NUM_PLAYERS,
	AVP_DFE_OPTION_SERVER_TO_JOIN,
#else
	AVP_DFE_OPTION_GAME_MODE,
	AVP_DFE_OPTION_GAME_TYPE,
	AVP_DFE_OPTION_CHAR_TYPE,
	AVP_DFE_OPTION_SEARCH_WITH_WILDCARD,
	AVP_DFE_OPTION_LEVEL_SELECTION,
	AVP_DFE_OPTION_NUM_PLAYERS,
	AVP_DFE_OPTION_SERVER_TO_JOIN,
	AVP_DFE_OPTION_FMV_TO_PLAY,
	AVP_DFE_OPTION_FMV_AUDIO_TRACK,
	AVP_DFE_OPTION_SMOKETEST_OPTION,
	AVP_DFE_OPTION_FRAMERATES_OPTION,
#endif	// defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

	AVP_DFE_OPTION_ACCEPT,
#if defined(AVP_DFE_ALLOW_QUIT)
	AVP_DFE_OPTION_QUIT,
#endif // #if defined(ALLOW_QUIT)

	NUM_AVP_DFE_OPTIONS
};

enum AVP_DFE_GAMEMODES
{
#if !defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)
	AVP_DFE_GAMEMODE_LOAD_SINGLE_ZONE,
#ifdef ASURA_USE_STREAMING
	AVP_DFE_GAMEMODE_LOAD_STREAMED_ZONE,
	AVP_DFE_GAMEMODE_LOAD_STREAMED_MISSION,
#endif // #ifdef ASURA_USE_STREAMING
	AVP_DFE_GAMEMODE_PREVIEW_FMV,
#endif	// defined(AVP_MULTIPLAYER_ONLY_DEVFRONTEND)

#if defined( AVP_USE_GAME_FRONTEND )
	AVP_DFE_GAMEMODE_GAME_FRONTEND,
#endif	// defined(AVP_USE_GAME_FRONTEND)

#if defined(AVP_USE_DEV_LOBBY)
	AVP_DFE_GAMEMODE_DEV_LOBBY,
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
	AVP_DFE_GAMEMODE_GAME_LOBBY,
#endif	//defined(AVP_USE_GAME_LOBBY)

#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
	AVP_DFE_GAMEMODE_HOST_NETWORK_GAME,
	AVP_DFE_GAMEMODE_JOIN_NETWORK_GAME,
#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	AVP_DFE_GAMEMODE_SMOKE_TEST,
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

	NUM_AVP_DFE_GAMEMODES
};

enum AVP_DFE_ONLINEMODES
{
	AVP_DFE_ONLINEMODE_OFFLINE,
	AVP_DFE_ONLINEMODE_ADHOC,
	AVP_DFE_ONLINEMODE_INFRASTRUCTURE,

	NUM_AVP_DFE_ONLINEMODES
};

class AVP_DevelopmentFrontEnd
{
public:
	static void Initialise();
	static bool Update();
	static void Render();
	static void Shutdown();

#if defined(AVP_USE_GAME_FRONTEND)
	static bool IsFrontEndRequested() { return s_bFrontEndRequested; }
#endif	//defined(AVP_USE_GAME_FRONTEND)
	static bool IsQuitRequested() { return s_bQuitRequested; }

#if defined(AVP_USE_DEV_LOBBY)
	static bool IsDevLobbyRequested() { return s_bDevLobbyRequested; }
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
	static bool IsGameLobbyRequested() { return s_bGameLobbyRequested; }
#endif	//defined(AVP_USE_GAME_LOBBY)

	static int GetRequestedLevel()
	{
		return s_iLevelNumber;
	}

	static bool UsingDevFE()
	{
		return s_bUsingDevFE;
	}

	static int GetGameMode(){return s_iCurrentGameMode;}

private:
	static void GetLevelFiles(Asura_List< Asura_String >& xNames)
	{
		 Platform_GetLevelFiles(xNames);
	}
	
	//Platform_GetLevelFiles - this function should allocate an Asura_String instance 
    //using new for each file found and add the instance to the list which is passed as the parameter. 
	//The calling code will call delete for each instance when it has finished.	
	static void Platform_GetLevelFiles(Asura_List< Asura_String >& xFiles);

	static const char* GetServerDesc(int iServerIndex);

	// page displayed while online mode is starting up
	
	static void Platform_Initialise();
	static void Platform_Shutdown();

	static bool NeedToStartNetworkMode();
	static void RenderStartingNetworkMode();
	static void StartNetworkMode();
	static bool UpdateStartingNetworkMode();
	static void ShutdownNetworkMode();

	static bool Platform_NeedToBuildFMVList();
	static void Platform_RenderBuildFMVList();
	static void Platform_StartBuildFMVList();
	static bool Platform_UpdateBuildFMVList();
	static void Platform_ClearFMVList();
	static void Platform_PreviewFMV();
	static const Asura_UnicodeChar* Platform_GetFMVDescription();

	static void Update_SelectedServer();
	static void Update_KeyControls();
	static void OnAccept();
	static void OnQuit();
	
	static bool MatchWithWildcard(Asura_Char* szFileName, Asura_Char* szWildcard );
	static Asura_Bounding_Rectangle CountRectangle(const float fCurrentPosX, 
												   const float fCurrentPosY, 																	
												   const Asura_Char* szString);
	static void PrintWithDropShadowInRectangle(Asura_Bounding_Rectangle& xRect,
										const u_int uColour,
										const float fSize,
										const u_int uFormat,
										const Asura_Char* szString);

	static void PrintWithDropShadow(const float fXPos,
									const float fYPos,
									const float fSize,
									const u_int uColour,
									const u_int uFormat,
									const Asura_Char* szString);
	static void AlterChosenWildcardLevel(bool bIncrement);
	static void AlterChosenWildcard(bool bIncrement);
	static void AlterGameMode(bool bIncrement);
	static void AlterGameType(bool bIncrement);	
	static void AlterPlayerSpecies(bool bIncrement);
	static void AlterNumPlayers(bool bIncrement);
	static void AlterSkin(bool bIncrement);
	static void AlterFMVToPlay(bool bIncrement);
	static void AlterFMVAudioTrack(bool bIncrement);
	static void AlterServerToJoin(bool bIncrement);
	static void AlterTimeOfDay(bool bIncrement);
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	static void AlterChosenSmokeTestOption( bool bIncrement );
	static void AlterNumberOfFramerates( bool bIncrement );
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	static bool IsOptionSelectable(AVP_DFE_OPTIONS eOption);
	static bool IsLevelSelectable();
	static bool IsNetwork();
	static bool IsServer();
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	static bool MoreLevelsForTheSmokeTest();
	static void PrepareForSmokeTest();
#endif //#ifdef AVP_LOG_FILE_LOAD_MEMORY_US

	
	static u_int GetOptionColour(AVP_DFE_OPTIONS eOption);
	static void OnLeaveMode(int iMode);
	static void OnEnterMode(int iMode);

	static bool						s_bSetupComplete;
	static float					s_fIgnoreInputTimer;
	static AVP_DFE_OPTIONS			s_eCurrentlyHighlitOption;
	static int						s_iCurrentGameMode;
	static AVP_Settings::GameType 	s_eCurrentGameType;
	static AVP_Settings::Species	s_eCurrentSpecies;
	static int						s_iLevelNumber;
	static int						s_iNumberOfPlayers;
	static int						s_iNetworkSessionIndex;
	static int						s_iFMVIndex;
	static int						s_iFMVAudioTrack;
	static int						s_iTimeOfDay;
	static int						s_aaiMinMaxPlayers[NUM_AVP_DFE_GAMEMODES][2];
	static const Asura_Char*		s_aszGameModeNames[NUM_AVP_DFE_GAMEMODES];
	static const bool				s_abGameModesAvailable[NUM_AVP_DFE_GAMEMODES];
//	static const Asura_Char*		s_aszLevelNames[NUM_AVP_LEVELS];
	static const bool				s_abLevelsAvailable[NUM_AVP_LEVELS];
	static const Asura_Char*		s_aszMissionNames[NUM_AVP_MISSIONS];
	static const bool				s_abMissionsAvailable[NUM_AVP_MISSIONS];
	static bool						s_abGameTypeAvailable[AVP_Settings::AVP_NUM_GAMETYPES];
	///my
	static Asura_String*		    s_apxLevelNamesForEachWildcard[iAVP_DFE_NUM_WILDCARDS];
	static Asura_Char			    s_acChosenLevel[iAVP_DFE_MAXPATHLENGTH];
	static Asura_UnicodeChar*	    s_acFileWildcards[iAVP_DFE_NUM_WILDCARDS];
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	static Asura_UnicodeChar*		s_acSmokeTestOptions[iAVP_DFE_NUM_SMOKETESTFLAGS];
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	static int					    s_aiNumFilesWithWildcards[iAVP_DFE_NUM_WILDCARDS];
	static int					    s_aiCurrentFileWithWildcard[iAVP_DFE_NUM_WILDCARDS];
	static int					    s_iCurrentWildcardChosen;
#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	static int						s_iCurrentSmokeTestOptionChosen;
	static int						s_iNumberOfFrameRates;
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	static bool					    s_bLevelNumberInitialised;

	static AVP_DFE_ONLINEMODES		s_eOnlineMode;

	static AVP_DFE_MENUMODE			s_eMenuMode;
#if defined(AVP_USE_GAME_FRONTEND)
	static bool						s_bFrontEndRequested;
#endif	//defined(AVP_USE_GAME_FRONTEND)
	static bool						s_bQuitRequested;

#if defined(AVP_USE_DEV_LOBBY)
	static bool						s_bDevLobbyRequested;
#endif	//defined(AVP_USE_DEV_LOBBY)

#if defined(AVP_USE_GAME_LOBBY)
	static bool						s_bGameLobbyRequested;
#endif	//defined(AVP_USE_GAME_LOBBY)

#ifdef AVP_LOG_FILE_LOAD_MEMORY_USE
	static bool						s_bDoingSmokeTest;
	static bool						s_bTestOnlyOneLevel;
	static bool						s_bSmokeTestsFirstLevelLoaded;
#endif // #ifdef AVP_LOG_FILE_LOAD_MEMORY_USE

	static bool						s_bUsingDevFE;
	static Asura_ControlMap*		s_pxControlMap;
};

#endif //defined AVP_USE_DEVELOPMENT_FRONT_END

#endif//__AVP_DEVELOPMENTFRONTEND_H__
