////////////////////////////////////////////////////////////////////////////////
// AVP_GUIMenu_System.h
//
// Created by GJDH - 15/03/2007
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//
////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __AVP_GUIMENU_SYSTEM_H__
#define __AVP_GUIMENU_SYSTEM_H__

#include "Core/Asura.h"

#if defined(ASURA_USE_GUI_MENU_SYSTEM)

#include "Core/AVP_Settings.h"
#include "GUIMenu/Asura_GUIMenu_Root.h"
#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Animator.h"

/*********/
/* Enums */
/*********/


/***********/
/* Defines */
/***********/

typedef struct _FadingWidgetInfo
{
	Asura_String xLocalName;
	int			 iFadeInRate;
	int			 iFadeInTarget;
	int			 iFadeOutRate;
	int			 iFadeOutTarget;
} FadingWidgetInfo;

/***********/
/* Classes */
/***********/
class AVP_GUIMenu_System
{
public:
	~AVP_GUIMenu_System()			{ Shutdown(); }

	enum GUIMENU_ID
	{
		GUIMENU_ID_LOADING = 0,
		GUIMENU_ID_AWAITINGHOST,
		GUIMENU_ID_HUD,
		GUIMENU_ID_HUD_ELEMENTS,
		GUIMENU_ID_PAUSE,
		GUIMENU_ID_GAMEOVER,
		GUIMENU_ID_MISSIONCOMPLETE,
        GUIMENU_ID_OBJECTIVES,
		GUIMENU_ID_WEAPONSELECTOR,

		GUIMENU_ID_MAX
	};

	enum FADING_WIDGET
	{
		FADING_WIDGET_INTERACTION_PROMPT,

		FADING_WIDGET_MAX
	};

	enum MENU_SPECIES
	{
		MENU_SPECIES_ALIEN,
		MENU_SPECIES_MARINE,
		MENU_SPECIES_PREDATOR,

		NUM_MENU_SPECIES
	};

#if defined(ASURA_SUPPORT_LOBBY)
	enum LOBBY_TYPE
	{
		LT_NONE,
#if defined(AVP_USE_DEV_LOBBY)
		LT_DEVELOPMENT,
#endif	//defined(AVP_USE_DEV_LOBBY)
#if defined(AVP_USE_GAME_LOBBY)
		LT_GAME,
#endif	//defined(AVP_USE_GAME_LOBBY)

		NUM_LOBBY_TYPES
	};
#endif	//defined(ASURA_SUPPORT_LOBBY)

	enum STORED_WIDGETS {
		MOTION_TRACKER,

		MOTION_TRACKER_SCANNER1,
		MOTION_TRACKER_SCANNER2,
		MOTION_TRACKER_SCANNER3,
		MOTION_TRACKER_DOT_CONTAINER,
		MOTION_TRACKER_PULSE1,
		MOTION_TRACKER_PULSE2,
		MOTION_TRACKER_DOT,
		MOTION_TRACKER_DOT_MARINE,
		MOTION_TRACKER_BASE0,
		MOTION_TRACKER_BASE1,
		MOTION_TRACKER_BASE2,
		MOTION_TRACKER_DISTANCE1,
		MOTION_TRACKER_DISTANCE2,
		MOTION_TRACKER_DISTANCE3,

		VISION_SELECTOR_BACKGROUND1,	// keep these in order as actor_selector
		VISION_SELECTOR_BACKGROUND2,	// likes it that way

		VISION_SELECTOR_SLOT_UP,
		VISION_SELECTOR_SLOT_RIGHT,
		VISION_SELECTOR_SLOT_DOWN,
		VISION_SELECTOR_SLOT_LEFT,
		
		WEAPON_SELECTOR_BACKGROUND1,	// keep these in order as actor_selector
		WEAPON_SELECTOR_BACKGROUND2,	// likes it that way

		WEAPON_SELECTOR_SLOT_UP,
		WEAPON_SELECTOR_SLOT_RIGHT,
		WEAPON_SELECTOR_SLOT_DOWN,
		WEAPON_SELECTOR_SLOT_LEFT,

		RETICLE_CENTER,
		RETICLE_UP,
		RETICLE_RIGHT,
		RETICLE_DOWN,
		RETICLE_LEFT,

		RETICLE_GROUP,

		HEALTH_POINTER_EMPTY,
		HEALTH_POINTER_FULL,
		HEALTH_POINTER,

		ENERGY_POINTER_EMPTY,
		ENERGY_POINTER_FULL,
		ENERGY_POINTER,
        
		ACTIVEGRAPPLE_HUDBAR1,
		ACTIVEGRAPPLE_IMAGE1,
		ACTIVEGRAPPLE_IMAGE2,
		ACTIVEGRAPPLE_PATH1,
		
		OBJECTIVE_TEXT,
		OBJECTIVE_ARROW,

		DAMAGE_GP,

		GROUP_COUNTER,

		BEGIN_DAMAGE_DIRECTION_INDICATOR,
		DAMAGE_DIRECTION_INDICATOR_01 = BEGIN_DAMAGE_DIRECTION_INDICATOR,
		DAMAGE_DIRECTION_INDICATOR_02,
		END_DAMAGE_DIRECTION_INDICATOR,
		
		BEGIN_SMARTGUN_TARGETING = END_DAMAGE_DIRECTION_INDICATOR,
		SMARTGUN_TARGET = BEGIN_SMARTGUN_TARGETING,
		SMARTGUN_RETICLE,
		END_SMARTGUN_TARGETING,

		// keep these together
		BEGIN_TICKER = END_SMARTGUN_TARGETING,
		TICKER_01 = BEGIN_TICKER,
		TICKER_02,
		TICKER_03,
		TICKER_04,
		TICKER_05,
		TICKER_06,
		TICKER_07,
		TICKER_08,
		TICKER_09,
		TICKER_10,
		TICKER_11,
		TICKER_12,
		TICKER_13,
		TICKER_14,
		TICKER_15,
		TICKER_16,
		TICKER_17,
		TICKER_18,
		TICKER_19,
		TICKER_20,
		TICKER_21,
		TICKER_22,
		TICKER_23,
		TICKER_24,

		END_TICKER,
		
		BEGIN_MULTIPLAYER = END_TICKER,

		MULTIPLAYER_ENEMY_ARROW = BEGIN_MULTIPLAYER,
		MULTIPLAYER_TEAM_ARROW,
		MULTIPLAYER_PERSECUTOR_ARROW,

		MULTIPLAYER_SCOREBOARD,
		MULTIPLAYER_SCOREBOARD_TITLE,
		MULTIPLAYER_SCOREBOARD_TEAM3_BG,
		MULTIPLAYER_SCOREBOARD_NOTEAM_BG,

		// keep these together and in order
		MULTIPLAYER_LISTBOX_TEAMSCORES,
		MULTIPLAYER_LISTBOX_TEAM0,
		MULTIPLAYER_LISTBOX_TEAM1,
		MULTIPLAYER_LISTBOX_TEAM2,
		MULTIPLAYER_LISTBOX_TEAM3,

		MULTIPLAYER_STATS_MATCH_KILLSMARINE,
		MULTIPLAYER_STATS_MATCH_KILLSMARINE_SP,
		MULTIPLAYER_STATS_MATCH_KILLSPRED,
		MULTIPLAYER_STATS_MATCH_KILLSPRED_SP,
		MULTIPLAYER_STATS_MATCH_KILLSALIEN,
		MULTIPLAYER_STATS_MATCH_KILLSALIEN_SP,
		MULTIPLAYER_STATS_MATCH_KILLSTOTAL,
		MULTIPLAYER_STATS_MATCH_SCORETOTAL,
		MULTIPLAYER_STATS_MATCH_KILLASSISTS,
		MULTIPLAYER_STATS_MATCH_KILLSTREAK,
		MULTIPLAYER_STATS_MATCH_DEATHS,
		MULTIPLAYER_STATS_MATCH_RATIO,
		MULTIPLAYER_STATS_MATCH_XPPOINTS,

		MULTIPLAYER_STATS_CAREER_KILLSMARINE,
		MULTIPLAYER_STATS_CAREER_KILLSMARINE_SP,
		MULTIPLAYER_STATS_CAREER_KILLSPRED,
		MULTIPLAYER_STATS_CAREER_KILLSPRED_SP,
		MULTIPLAYER_STATS_CAREER_KILLSALIEN,
		MULTIPLAYER_STATS_CAREER_KILLSALIEN_SP,
		MULTIPLAYER_STATS_CAREER_KILLSTOTAL,
		MULTIPLAYER_STATS_CAREER_SCORETOTAL,
		MULTIPLAYER_STATS_CAREER_KILLASSISTS,
		MULTIPLAYER_STATS_CAREER_KILLSTREAK,
		MULTIPLAYER_STATS_CAREER_DEATHS,
		MULTIPLAYER_STATS_CAREER_RATIO,
		MULTIPLAYER_STATS_CAREER_XPPOINTS,

		MULTIPLAYER_STATS_WINNINGTEAM,
		MULTIPLAYER_STATS_WINNINGPLAYER,
		MULTIPLAYER_STATS_WINNINGSCORE,

		MULTIPLAYER_RESPAWN_TIMER, 
		MULTIPLAYER_MESSAGE_CONSOLE, 

		MULTIPLAYER_RANK_PLACE,
		MULTIPLAYER_RANK_PLAYERS,

		MULTIPLAYER_TEAMSCORE_TEAM1,
		MULTIPLAYER_TEAMSCORE_TEAM2,
		MULTIPLAYER_TEAMSCORE_TEAM3,

		END_MULTIPLAYER,

		BEGIN_RECON = END_MULTIPLAYER,

		RECON_HEALTH = BEGIN_RECON,
		RECON_CLASS,
		RECON_THREAT,

		END_RECON,

		PREDATOR_LOCKON_GROUP = END_RECON,

		NUM_STORED_WIDGETS 
	};

	static void	Initialise( void );
	static void	Shutdown( void );
	static void	Update( void );
	static void	Render( const bool bClearScreenIfNoRender );
	static void	RenderHUDCounters( void );

	static void	ClearStoredHUDWidgets(void);

	static bool	IsCurrentlyInNonHudOrLoadingMenu( void );
	static void	Activate( GUIMENU_ID eID, AVP_Settings::Species eRequiredSpecies = AVP_Settings::AVP_SPECIES_UNKNOWN );
	static void	Activate( Asura_Hash_ID uHashPageToActivate, bool bPausedInMenu, bool bStealControls );
	static void	Deactivate( void ); // close all open pages.
	static void	Deactivate( GUIMENU_ID eID, AVP_Settings::Species eRequiredSpecies = AVP_Settings::AVP_SPECIES_UNKNOWN );
	static void	Deactivate( Asura_Hash_ID uHashPageToDeactivate );
	
	static Asura_Hash_ID	GetMenuHash( GUIMENU_ID eID = GUIMENU_ID_PAUSE, AVP_Settings::Species eRequiredSpecies = AVP_Settings::AVP_SPECIES_UNKNOWN );
	static float			GetHUDFadeParamentric(void);

	static void	Fade( FADING_WIDGET eWidgetToFade, bool bFadeIn );
	static bool	IsFadingWidgetFullyFadedIn( FADING_WIDGET eWidgetToFade );
	static bool	IsInPauseMenu( void ) {	return(IsCurrentlyInNonHudOrLoadingMenu()); }

	static bool	IsMenuActive( GUIMENU_ID eID );

	static Asura_GUIMenu_Root* GetHUDElement( STORED_WIDGETS eWidget, Asura_Hash_ID uFromMenuPageHash = ASURA_HASH_ID_UNSET, bool bCheckMenuPageIsActive = true, bool bForceLookUp = false );	

#if defined(ASURA_SUPPORT_LOBBY)
	// accessors for various front end loading conditions
	static void			SetLobbyType(const LOBBY_TYPE eLobbyType)	{ s_eLobbyType = eLobbyType; }
	static LOBBY_TYPE	GetLobbyType(void)							{ return(s_eLobbyType); }
#endif	//defined(ASURA_SUPPORT_LOBBY)

	// Command-console functions...
	static bool	SelectCharType( u_int uIn, bool bInterrogate, bool bHelp );
	static bool	SelectDifficulty( u_int uIn, bool bInterrogate, bool bHelp );

private:
	AVP_GUIMenu_System();
	AVP_GUIMenu_System& operator=( const AVP_GUIMenu_System& ref );

	static void		GUIMenuExternalMessageCallback( const Asura_Message_GUIMenu_Notify* pxData );
	
	static void		Platform_Initialise();
	static void		Platform_Update();
	static void		Platform_Shutdown();

	// Command-console functions...
	static bool		ResumeGame( bool bHelp );
	static void		SetGameStats( bool bHelp );
	static bool		ShowDebugVariablesMenu( bool bHelp );
	static bool		Restart( bool bHelp );
	static bool		RestartFromCheckPoint( bool bHelp );
	static bool		Quit( bool bHelp );
	static bool		QuitApplication( bool bHelp );
	static bool		CompleteLevel( bool bHelp );
	static bool		IsSinglePlayer( bool bHelp );
	static bool		ReturnToLevelSelect( bool Help );
	static bool		ExitSaveScreen( int uIn, bool bInterrogate, bool bHelp );
	static void		SetSettingsOnClosure( void );
	static bool		StartSpecificLevel(u_int uLevel, bool bInterrogate, bool bHelp );
	static bool 	StartDevLobby( bool bHelp );
	static bool 	StartGameLobby( bool bHelp );

	static Asura_Hash_ID	GetWidgetOwningPage( FADING_WIDGET eWidgetToFade );

	// this is a polynomial approximation which is valid to within 10^-7 for 1/sqrt(10) < fNum < sqrt(10).
	// I found it taken from "Approximations for digital computers", Princeton Univ. Press (1955)
	static float	ApproxLog2( const float fNum );

	//Pause Menu
	static bool	s_bIsInitialised;
	static bool	s_bShowLocation;
	static bool	s_bDisplayGameStats;
	static bool	s_bOpenDebugVariableMenu;
	static bool	s_bQuitApplication;
	static bool	s_bQuit;
	static bool	s_bRequestRestart;
	static bool	s_bRequestRestartFromCheckpoint;

	//Cheat Menu 
	static bool	s_bFreezeTime;
	static int	s_iCameraMode;

	//Other Menus
	static bool	s_bChooseNextLevel;
	static bool	s_bExitSaveScreen;

	//FrontEnd Choice Toggle
#if defined(ASURA_SUPPORT_LOBBY)
	static LOBBY_TYPE	s_eLobbyType;
#endif	//defined(ASURA_SUPPORT_LOBBY)
	static bool			s_bGeneralTrueReference;

	// Nasty initialisation tables...	
	static Asura_Hash_ID		s_auHashed_GUIMenuNames[NUM_MENU_SPECIES][GUIMENU_ID_MAX];
	static FadingWidgetInfo		s_axFadingWidgets[FADING_WIDGET_MAX];
	static bool					s_abWidgetLookedUp[NUM_STORED_WIDGETS];
	static Asura_GUIMenu_Root*	s_apxStoredHUDWidgets[NUM_STORED_WIDGETS];
	static Asura_Hash_ID		s_auStoredHUDWidgetHashes[NUM_STORED_WIDGETS];

};

#endif //#if defined(ASURA_USE_GUI_MENU_SYSTEM)

#endif // __AVP_GUIMENU_SYSTEM_H__
