// Core
#include "Core/Asura.h"

#ifdef AVP_USE_INPUT_SIMULATOR

// File's header
#include "InputSimulator/AVP_InputSimulator_Behaviour.h"

// AVP
#include "InputSimulator/AVP_InputSimulator_Behaviour_Types.h"

// AVP Input Simulation Behaviours
// Alien
#include "InputSimulator/Behaviours/AVP_InputSimulator_Behaviour_Alien_Hunt.h"
#include "InputSimulator/Behaviours/AVP_InputSimulator_Behaviour_Alien_Idle.h"
#include "InputSimulator/Behaviours/AVP_InputSimulator_Behaviour_Alien_Patrol.h"
// Marine
#include "InputSimulator/Behaviours/AVP_InputSimulator_Behaviour_Marine_Hunt.h"
#include "InputSimulator/Behaviours/AVP_InputSimulator_Behaviour_Marine_Idle.h"
#include "InputSimulator/Behaviours/AVP_InputSimulator_Behaviour_Marine_Patrol.h"
// Predator
#include "InputSimulator/Behaviours/AVP_InputSimulator_Behaviour_Predator_Hunt.h"
#include "InputSimulator/Behaviours/AVP_InputSimulator_Behaviour_Predator_Idle.h"
#include "InputSimulator/Behaviours/AVP_InputSimulator_Behaviour_Predator_Patrol.h"


AVP_InputSimulator_Behaviour::AVP_InputSimulator_Behaviour()
: m_pxChild( NULL )
, m_bIsFinished( false )
, m_uType( AVP_INPUTSIMULATOR_BEHAVIOUR_TYPE_NOT_SET )
{
}

bool AVP_InputSimulator_Behaviour::CreateChildWithType( u_int uType ) 
{
	AVP_InputSimulator_Behaviour* pxLastChild = m_pxChild;
	while( pxLastChild )
	{
		pxLastChild = pxLastChild->m_pxChild;		
	}
	
	//if( !pxLastChild ) { return NULL; }

	AVP_InputSimulator_Behaviour* pxBehaviour = NULL;
	switch( uType )
	{
		case AVP_INPUTSIMULATOR_BEHAVIOUR_TYPE_MARINE_IDLE:
		{
			pxBehaviour = new AVP_InputSimulator_Behaviour_Marine_Idle();
			break;
		}
		case AVP_INPUTSIMULATOR_BEHAVIOUR_TYPE_MARINE_PATROL:
		{
			pxBehaviour = new AVP_InputSimulator_Behaviour_Marine_Patrol();
			break;
		}
		case AVP_INPUTSIMULATOR_BEHAVIOUR_TYPE_MARINE_HUNT:
		{
			pxBehaviour = new AVP_InputSimulator_Behaviour_Marine_Hunt();
			break;
		}
		case AVP_INPUTSIMULATOR_BEHAVIOUR_TYPE_PREDATOR_IDLE:
		{
			pxBehaviour = new AVP_InputSimulator_Behaviour_Predator_Idle();
			break;
		}
		case AVP_INPUTSIMULATOR_BEHAVIOUR_TYPE_PREDATOR_PATROL:
		{
			pxBehaviour = new AVP_InputSimulator_Behaviour_Predator_Patrol();
			break;
		}
		case AVP_INPUTSIMULATOR_BEHAVIOUR_TYPE_PREDATOR_HUNT:
		{
			pxBehaviour = new AVP_InputSimulator_Behaviour_Predator_Hunt();
			break;
		}
		case AVP_INPUTSIMULATOR_BEHAVIOUR_TYPE_ALIEN_PATROL:
		{
			pxBehaviour = new AVP_InputSimulator_Behaviour_Alien_Patrol();
			break;
		}
		case AVP_INPUTSIMULATOR_BEHAVIOUR_TYPE_ALIEN_IDLE:
		{
			pxBehaviour = new AVP_InputSimulator_Behaviour_Alien_Idle();
			break;
		}
		case AVP_INPUTSIMULATOR_BEHAVIOUR_TYPE_ALIEN_HUNT:
		{
			pxBehaviour = new AVP_InputSimulator_Behaviour_Alien_Hunt();
			break;
		}
	}
	
	if( pxBehaviour )
	{
		m_pxChild = pxBehaviour;
		pxBehaviour->SetType( uType );
		return true;
	}
	else
	{
		return false;
	}
}

// Destroys all childrens attached to a node.
void AVP_InputSimulator_Behaviour::DestroyAllChildren()
{
	if( m_pxChild )
	{	
		AVP_InputSimulator_Behaviour* pxCurrentNode = m_pxChild;
		while( pxCurrentNode )
		{
			AVP_InputSimulator_Behaviour* pxTempNode = pxCurrentNode->m_pxChild;
			delete pxCurrentNode;
			pxCurrentNode = pxTempNode;		
		}	
		m_pxChild = NULL;
	}
}

// Returns first child with requested type.
AVP_InputSimulator_Behaviour* AVP_InputSimulator_Behaviour::GetChildWithType( u_int uType )
{
	if( m_pxChild )
	{	
		AVP_InputSimulator_Behaviour* pxCurrentNode = m_pxChild;
		while( pxCurrentNode )
		{
			if( pxCurrentNode->GetType() == uType )
			{
				return pxCurrentNode;
			}
			pxCurrentNode = pxCurrentNode->m_pxChild;		
		}	
	}

	return NULL;
}

// Updates all children.
void AVP_InputSimulator_Behaviour::UpdateBehaviours( u_char* ucCurrentInput, float* pfAxisValue )
{
	if( m_pxChild )
	{	
		AVP_InputSimulator_Behaviour* pxCurrentNode = m_pxChild;
		while( pxCurrentNode )
		{
			pxCurrentNode->Update( ucCurrentInput, pfAxisValue );
			pxCurrentNode = pxCurrentNode->m_pxChild;		
		}	
	}
}

#endif // AVP_USE_INPUT_SIMULATOR
