///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_AmmoStore.h
// Author : Chris Wallace
// Date   : 9-Sept-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_AMMOSTORE_H__
#define __AVP_AMMOSTORE_H__

////////////////////////////////////////////////////////////////////////////////
// Includes ...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_HashedList.h"

////////////////////////////////////////////////////////////////////////////////
// Structs and enums ...
////////////////////////////////////////////////////////////////////////////////

class AVP_InventoryItem_SubType;
class AVP_Pickup_Ammo;
class AVP_Pickup;
class AVP_ServerEntity_Pickup;
class Asura_Chunk_Stream;

class AVP_AmmoType
{
public:
	AVP_AmmoType(Asura_Chunk_Stream& xStream);
	
	AVP_AmmoType():m_uTypeHash(ASURA_HASH_ID_UNSET),m_fQuantity(0.0f),m_fMaxQuantity(0.0f){}
	AVP_AmmoType(Asura_Hash_ID uTypeHash):m_uTypeHash(uTypeHash),m_fQuantity(0.0f),m_fMaxQuantity(0.0f){}
	
	// Chunk processing functions...
	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	Asura_Hash_ID GetGuid(void) const	{ return m_uTypeHash; } // required for use of hashed lists...
	bool	IsDestroyed(void) const		{ return false; }
	bool	IsFull() const              { return (m_fQuantity == m_fMaxQuantity); }

	void	SetTypeHash(Asura_Hash_ID uType){ m_uTypeHash = uType; }

	float	GetAmmoQuantity(void) const		{ return m_fQuantity; }
	void	SetAmmoQuantity(float fAmmo)	{ m_fQuantity = fAmmo; }

	float	GetMaxQuantity(void) const		{ return m_fMaxQuantity; }
	void	SetMaxQuantity(float fAmmo)		{ m_fMaxQuantity = fAmmo; }
	
	void	Reset(void);

private:
	
	enum { CHUNK_VERSION = 0 };

	Asura_Hash_ID m_uTypeHash;
	float m_fQuantity;
	float m_fMaxQuantity;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_AmmoStore
// Author       : Chris Wallace
// Date         : 9-Sept-2008
//
// Description  : Class for the marines ammo storage.
////////////////////////////////////////////////////////////////////////////////

class AVP_AmmoStore
{
public:
	
	AVP_AmmoStore();
	~AVP_AmmoStore();

	// Chunk processing functions...
	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	void InitialiseFromBlueprints(void);
	void ClearAllAmmoTypes(void);
	
	float GetAmmoQuantity(Asura_Hash_ID uType) const;
	void RemoveAmmoQuantity(float fQuantity, Asura_Hash_ID uType); 
	void AddAmmoQuantity(float fQuantity, Asura_Hash_ID uType);
    bool IsFull(Asura_Hash_ID uType) const;

	static void ReloadSubTypeFromAmmoStore(AVP_InventoryItem_SubType* pxSubType, AVP_AmmoStore* pxAmmoStore);
	static float GetIntelligentAmmoQuantity(int iIntelligentAmount, const AVP_Pickup_Ammo& xPickup);
	static float GetIntelligentAmmoQuantity(int iIntelligentAmount, const AVP_Pickup& xPickup);

	Asura_Hashed_List<AVP_AmmoType>* GetList(void) { return &m_xHashedList; }

private:

	enum { CHUNK_VERSION = 0 };

	Asura_Hashed_List<AVP_AmmoType> m_xHashedList;
};

#endif // __AVP_AMMOSTORE_H__
