
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Inventory_System.h
// Author : Andy Tate
// Date   : 25-Oct-2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_INVENTORY_SYSTEM_H__
#define __AVP_INVENTORY_SYSTEM_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_HashTree.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Chunk_Stream;
class Asura_Message;
class AVP_Inventory_Group;
class AVP_Inventory_Item;
class AVP_InventoryItem_Weapon;
class AVP_ClientEntity_Actor;
class AVP_ServerEntity_Actor;
class AVP_AmmoStore;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Inventory_System
// Author       : Andy Tate
// Date         : 25-Oct-2006
//
// Description  : Class to handle the inventory for ANY type of actor.
////////////////////////////////////////////////////////////////////////////////

class AVP_Inventory_System
{
public:
	// Public functions...
	AVP_Inventory_System(AVP_ClientEntity_Actor& xParentClient);
	AVP_Inventory_System(AVP_ServerEntity_Actor& xParentServer);
	~AVP_Inventory_System();

	// Update/render functions...
	void	Activate(void);
	void	Deactivate(void);

	void	Reset(void);

	bool	HandleMessage(const Asura_Message* pxMessage);

	void	Update(void);
	void	Render(void) const;

	// Inventory group functions...
	bool	AddGroup(const Asura_Hash_ID uHash, AVP_Inventory_Group* pxNewGroup);
	bool	DestroyGroup(const Asura_Hash_ID uHash);
	void	DestroyAllGroups(void);

	// Chunk processing functions...
	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	void PostLoadInitialise();

	// Accessor functions...
	Asura_Guid					GetOwnerGuid(void) const	{ return(m_uOwnerGuid); }
	AVP_ClientEntity_Actor*		GetParentClient(void) const	{ return(m_pxParentClient); }
	AVP_ServerEntity_Actor*		GetParentServer(void) const	{ return(m_pxParentServer); }
	AVP_AmmoStore*				GetAmmoStore(void) const	{ return(m_pxAmmoStore); }

	AVP_Inventory_Group*		GetGroup(const Asura_Hash_ID uHash) const	{ AVP_Inventory_Group** ppxGroup = m_xGroupTree.GetDataByHash(uHash); return((ppxGroup == NULL) ? NULL : *ppxGroup); }

	AVP_Inventory_Group*		GetPermanentlyActiveItemGroup(void);
	const AVP_Inventory_Group*	GetPermanentlyActiveItemGroup(void) const;
	void						AddPermenantlyActiveItems(AVP_Inventory_Group* pxGroup);

	static Asura_Hash_ID GetFirstAvailableWeaponSlotHash(AVP_Inventory_System* pxSystem);

	AVP_Inventory_Item*			GetItem(const Asura_Hash_ID uHash) const;

	void	SetAsMaster(void)		{ m_bIsMaster = true; }
	void	SetAsSlave(void)		{ m_bIsMaster = false; }
	bool	IsMaster(void) const	{ return(m_bIsMaster); }
	bool	IsSlave(void) const		{ return(!m_bIsMaster); }

	bool	IsClientSide(void) const	{ return(m_pxParentClient != NULL); }
	bool	IsServerSide(void) const	{ return(m_pxParentServer != NULL); }

	bool	IsLocalPlayer(void) const;

	// Helpers to make it easier for outside systems to get information they care about without needing to know how the system works.
	Asura_Hash_ID				GetCurrentItemID(const Asura_Hash_ID uGroupHash) const;
	AVP_Inventory_Item*			GetCurrentItem(const Asura_Hash_ID uGroupHash);
	const AVP_Inventory_Item*	GetCurrentItem(const Asura_Hash_ID uGroupHash) const;
	u_int						GetNumSelectableItems(const Asura_Hash_ID uGroupHash) const;

protected:
	// Protected variables...
	Asura_Guid					m_uOwnerGuid;		// The GUID of the owner entity.
	AVP_ClientEntity_Actor*		m_pxParentClient;	// A pointer to the parent client-entity (if appropriate).
	AVP_ServerEntity_Actor*		m_pxParentServer;	// A pointer to the parent server-entity (if appropriate).

	AVP_AmmoStore*				m_pxAmmoStore;					

	Asura_TemplateHashTree<AVP_Inventory_Group*>	m_xGroupTree;		// A tree containing the groups we can use.

private:
	union
	{
		u_int		m_uFlags;			// Single access-point for the flags.

		struct
		{
			u_int	m_bIsMaster	: 1,	// Flag to say whether this is a master (or slave) inventory-system.
								: 0;	// 31 spare-bits
		};
	};

	// Private functions...
	AVP_Inventory_System(const AVP_Inventory_System& xOther);

	AVP_Inventory_System&	operator=(const AVP_Inventory_System& xOther);

	static const u_int uCURRENT_CHUNK_VERSION = 2;

};   // End of class 'AVP_Inventory_System'.


#endif   // ifndef __AVP_INVENTORY_SYSTEM_H__


/// End of file. ///
