
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryGroup_Weapons.cpp
// Author : Andy Tate
// Date   : 26-Nov-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Inventory/Groups/AVP_InventoryGroup_Weapons.h"

// Game independent header-files...
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/AVP_Inventory_ItemTypes.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_Disc.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_Mine.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_DropshipMissile.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_Flamethrower.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_MeleeWeapon.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_SmartGun.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_Taser.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_ScopedRifle.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_Shotgun.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_Unarmed.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_PlasmaCaster.h"
#include "Inventory/Items/Weapons/Tools/AVP_InventoryTool_Battery.h"
#include "Inventory/Items/Weapons/Tools/AVP_InventoryTool_Tequila.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

AVP_Inventory_Item* AVP_InventoryGroup_Weapons::CreateItemFromBlueprint(const Asura_Hash_ID uBlueprintHash,
																		const Asura_Hash_ID uNameHash)
{
	AVP_Inventory_Item*	pxNewItem = NULL;

	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uBlueprintHash );
	if ( pxBlueprint )
	{
		Asura_Value xValue;
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_WEAPON_TYPE, xValue ) )
		{
			switch ( xValue.GetIntValue() )
			{
				case AVP_INVENTORYITEMTYPE_WEAPON_FLAMETHROWER:
				{
					pxNewItem = new AVP_InventoryWeapon_Flamethrower( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_WEAPON_RANGED:
				{
					pxNewItem = new AVP_InventoryWeapon_RangedWeapon( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_PREDATORDISC:
				{
					pxNewItem = new AVP_InventoryWeapon_Disc( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_PREDATORMINE:
				{
					pxNewItem = new AVP_InventoryWeapon_Mine( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_WEAPON_MELEE:
				{
					pxNewItem = new AVP_InventoryWeapon_MeleeWeapon( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_SMARTGUN:
				{
					pxNewItem = new AVP_InventoryWeapon_SmartGun( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_TASER:
				{
					pxNewItem = new AVP_InventoryWeapon_Taser( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_SCOPEDRIFLE:
				{
					pxNewItem = new AVP_InventoryWeapon_ScopedRifle( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_SHOTGUN:
				{
					pxNewItem = new AVP_InventoryWeapon_Shotgun( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_PLASMACASTER:
				{
					pxNewItem = new AVP_InventoryWeapon_PlasmaCaster( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_DROPSHIP_MISSILE:
				{
					pxNewItem = new AVP_InventoryWeapon_DropshipMissile( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_UNARMED:
				{
					pxNewItem = new AVP_InventoryWeapon_Unarmed( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_BATTERY:
				{
					pxNewItem = new AVP_InventoryTool_Battery( *this, uNameHash, uBlueprintHash );
					break;
				}

				case AVP_INVENTORYITEMTYPE_TEQUILA:
				{
					pxNewItem = new AVP_InventoryTool_Tequila( *this, uNameHash, uBlueprintHash );
					break;
				}

				default:
				{
					Asura_Assert( false, "Not a valid weapon type" );
					break;
				}
			}
		}
		else
		{
			Asura_Assert( false, "Unable to get weapon type BP parameter" );
		}
	}
	else
	{
		Asura_Assert( false, "Unable to get weapon blueprint" );
	}

	if ( pxNewItem )
	{
		pxNewItem->InitialiseFromBlueprint();
	}
	return pxNewItem;
}

AVP_Inventory_Item*	AVP_InventoryGroup_Weapons::CreateItemFromType( const int iType, const Asura_Hash_ID uBlueprintHash, const Asura_Hash_ID uNameHash )
{
	switch ( iType )
	{
		case AVP_INVENTORYITEMTYPE_WEAPON_FLAMETHROWER:	return new AVP_InventoryWeapon_Flamethrower( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_WEAPON_RANGED:		return new AVP_InventoryWeapon_RangedWeapon( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_PREDATORDISC:		return new AVP_InventoryWeapon_Disc( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_PREDATORMINE:		return new AVP_InventoryWeapon_Mine( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_WEAPON_MELEE:		return new AVP_InventoryWeapon_MeleeWeapon( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_SMARTGUN:			return new AVP_InventoryWeapon_SmartGun( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_TASER:				return new AVP_InventoryWeapon_Taser( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_DROPSHIP_MISSILE:	return new AVP_InventoryWeapon_DropshipMissile( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_UNARMED:				return new AVP_InventoryWeapon_Unarmed( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_BATTERY:				return new AVP_InventoryTool_Battery( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_TEQUILA:				return new AVP_InventoryTool_Tequila( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_SCOPEDRIFLE:			return new AVP_InventoryWeapon_ScopedRifle( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_SHOTGUN:				return new AVP_InventoryWeapon_Shotgun( *this, uNameHash, uBlueprintHash );
		case AVP_INVENTORYITEMTYPE_PLASMACASTER:		return new AVP_InventoryWeapon_PlasmaCaster( *this, uNameHash, uBlueprintHash );
		default:										return PARENT::CreateItemFromType( iType, uNameHash, uBlueprintHash );
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryGroup_Weapons::OnStateChange
// Author       : Andy Tate
// Date         : 26-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryGroup_Weapons::OnStateChange(const u_int uOldState,
											   const u_int uNewState)
{
	// Call the base-function...
	PARENT::OnStateChange(uOldState, uNewState);

	// Do our weapon specific state-change work...
	switch(uNewState)
	{
		case SWAP_PENDING_ITEM_TO_CURRENT_ITEM:  // fall through
		case SWAP_PICKUP_ITEM_WITH_CURRENT_ITEM: 
		{
// 			const bool	bWasInactive	= (uOldState == NO_ITEM_ACTIVE);
// 
// 			if(bWasInactive && (pxClient != NULL))
// 			{
				AVP_ClientEntity_Actor* pxClient = GetParentSystem().GetParentClient();
				const AVP_InventoryItem_Weapon* pxWeapon = static_cast< const AVP_InventoryItem_Weapon* >( GetCurrentInventoryItem() );
				if ( pxWeapon && pxClient )
				{
					pxClient->ChangeToNewWeapon( false, pxWeapon->GetActivationActorAnim(), pxWeapon->IsEncumberment() );
				}
// 			}
			break;
		}

		case ACTIVATING_CURRENT_ITEM:
		default:
		{
			// Nothing to do...
			break;
		}
	}		// End of switch.
}

/// End of function 'AVP_InventoryGroup_Weapons::OnStateChange'. ///



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryGroup_Weapons::Update
// Author       : Andy Tate
// Date         : 26-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryGroup_Weapons::Update(void)
{
	// Call the base-function...
	PARENT::Update();
}

/// End of function 'AVP_InventoryGroup_Weapons::Update'. ///


const AVP_InventoryItem_Weapon* AVP_InventoryGroup_Weapons::GetItemOfType( AVPInventoryItemType eType ) const
{
	Asura_TemplateHashTree_ConstIterator<AVP_Inventory_Item*> xIterator(m_xItemTree);

	while (!xIterator.Done())
	{
		const AVP_Inventory_Item* const* ppxCurrentItem	= xIterator.GetCurrent();
		const AVP_InventoryItem_Weapon* pxCurrentWeapon = ppxCurrentItem ? static_cast<const AVP_InventoryItem_Weapon*>(*ppxCurrentItem) : NULL;

		if (pxCurrentWeapon && (pxCurrentWeapon->GetItemType() == eType))
		{
			return pxCurrentWeapon;
		}

		xIterator.Next();
	}

	return NULL;
}


bool AVP_InventoryGroup_Weapons::GetCanSetNewItem(const bool bInstant) const
{
	if (!bInstant)
	{
		// If doing a normal weapon swap, make sure we're not doing something that would make that difficult.
		const AVP_Inventory_System& xParentSystem = GetParentSystem();

		const AVP_Inventory_Item* pxStim = xParentSystem.GetItem(uAVP_BLUEPRINT_PARAM_STIM);
		if (pxStim && !pxStim->IsIdling(true))
		{
			return false;
		}

		const AVP_Inventory_Item* pxCloak = xParentSystem.GetItem(uAVP_INVENTORY_ITEMHASH_CLOAK);
		if (pxCloak)
		{
			// We need to see if we're in a transition
			const bool bIdling = pxCloak->IsIdling(false);
			const bool bFullyIdling = pxCloak->IsIdling(true);
			const bool bUsing = pxCloak->IsUsing(false);
			const bool bFullyUsing = pxCloak->IsUsing(true);

			if ((bIdling && !bFullyIdling) || (bUsing && !bFullyUsing)) { return false; }
		}
	}

	return PARENT::GetCanSetNewItem(bInstant);
}

AVP_InventoryGroup_Weapons::AVP_InventoryGroup_Weapons(AVP_Inventory_System& xSystem)
: PARENT(xSystem, szAVP_INVENTORY_GROUPNAME_WEAPON, false)
{
	SetAllowSelectionOfEmptyItems(true);
	SetGroupType(WEAPON);
}
/// End of file. ///
