
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryItem_Stim.cpp
// Author : Will Plano
// Date   : 09/05/2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Inventory/Items/AVP_InventoryItem_Stim.h"

// Asura
#include "Animation/Asura_AnimationBlend.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"

//AVP
#include "Actor/AVP_Actor_AnimStateData.h"
#include "Actor/Vertebrates/AVP_Vertebrate_CameraHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_InputHandler.h"
#include "Animation/AVP_AnimFlags.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Inventory/Items/AVP_InventoryItem_DynamicSubType.h"
#include "Inventory/Items/AVP_InventoryItem_PredatorCloak.h"

// Axon
#include "Core/Axon_System.h"
#include "Core/Axon_Sound_Registry.h"

// AVP Axon
#include "Knowledge/AVP_Knowledge_System.h"

AVP_InventoryItem_Stim::AVP_InventoryItem_Stim( AVP_Inventory_Group& xParentGroup, const Asura_Hash_ID uBlueprintHash, const char* szName )
: PARENT(AVP_INVENTORYITEMTYPE_STIM, xParentGroup, uAVP_INVENTORY_ITEMHASH_STIM, uBlueprintHash, szName ) // Always name the item using the hash uAVP_INVENTORY_ITEMHASH_STIM, so we can always find our instance of this.
, m_pxAnimation( NULL )
, m_uSkinHash( ASURA_HASH_ID_UNSET )
, m_uActorAACAHash( ASURA_HASH_ID_UNSET )
, m_uStimRegenBlocks( 1 )
, m_uStimEnergyBlocksLost( 0 )
, m_fStimEnergyCost( 0.0f )
{
	// Get values from the blueprints
	GetStaticDataFromBlueprint();

	// Initialise the valid states...
	SetStateAsValid(BEGIN_USING);
	SetStateAsValid(USING);
	SetStateAsValid(END_USING);
}

AVP_InventoryItem_Stim::~AVP_InventoryItem_Stim()
{
	delete m_pxAnimation;
}

void AVP_InventoryItem_Stim::InitialiseFromBlueprint()
{
	PARENT::InitialiseFromBlueprint();

	const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, m_uBlueprintHash );
	if ( pxBlueprint )
	{
		Asura_Hash_ID uSubTypeHash = ASURA_HASH_ID_UNSET;

		Asura_Value xValue;
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SUBTYPE, xValue ) )
		{
			uSubTypeHash = xValue.GetHashIDValue();
		}

		InitialiseSubTypes( 1 );
		m_ppxSubTypes[ 0 ] = new AVP_InventoryItem_DynamicSubType( AVP_INVENTORYITEMSUBTYPE_DYNAMIC, uSubTypeHash );
		if ( m_ppxSubTypes[ 0 ] )
		{
			m_ppxSubTypes[ 0 ]->InitialiseFromBlueprint();
		}
	}
	else
	{
		Asura_Assertf( false, ( "AVP_InventoryItem_Stim::InitialiseFromBlueprint - cannot find blueprint %u to initialise from.", GetBlueprintHash() ) );
	}
}

void AVP_InventoryItem_Stim::PostLoadInitialise()
{
	PARENT::PostLoadInitialise();

	const AVP_Inventory_System& xParentSystem = GetParentSystem();
	AVP_Actor_DynamicStat* pxDynamicAmmo = NULL;
	if ( xParentSystem.IsMaster() )
	{
		AVP_ClientActor_Vertebrate* pxVert = AVP_ClientActor_Vertebrate::FindEntity( xParentSystem.GetOwnerGuid() );
		if ( pxVert != NULL )
		{
			pxDynamicAmmo = pxVert->GetDynamicEnergy();
		}
	}

	if ( m_ppxSubTypes[ 0 ] && m_ppxSubTypes[ 0 ]->GetType() == AVP_INVENTORYITEMSUBTYPE_DYNAMIC )
	{
		AVP_InventoryItem_DynamicSubType* pxDynamicSubType = static_cast< AVP_InventoryItem_DynamicSubType* >( m_ppxSubTypes[ 0 ] );
		pxDynamicSubType->SetDynamicAmmo( pxDynamicAmmo );
	}
}

void AVP_InventoryItem_Stim::OnStateChange( const u_int uOldState, const u_int uNewState )
{
	AVP_Inventory_System&				xParentSystem	= GetParentSystem();
	AVP_InventoryItem_DynamicSubType*	pxSubType		= static_cast< AVP_InventoryItem_DynamicSubType* >( m_ppxSubTypes[0] );

	// Call the base-function...
	PARENT::OnStateChange(uOldState, uNewState);

	switch ( uOldState )
	{
		case END_USING:
		{
			AVP_ClientEntity_Actor* pxClient = xParentSystem.GetParentClient();
			AVP_Actor_AnimationHandler* pxAnimHandler = pxClient != NULL ? pxClient->GetAnimationHandler() : NULL;
			if ( pxAnimHandler != NULL )
			{
				if ( pxAnimHandler->IsItemAttached( this ) )
				{
					pxAnimHandler->DetachItem( this );
					if ( m_pxAnimation )
					{
						DestroyAnimation();
					}
				}
			}
		}
		break;
		
		default:
		{
			
		}
		break;
	}

	switch(uNewState)
	{
		case BEGIN_USING:
		{
			AVP_InventoryItem_Cloak* pxCloak = static_cast< AVP_InventoryItem_Cloak* >( GetParentSystem().GetItem( uAVP_INVENTORY_ITEMHASH_CLOAK ) );
			if ( pxCloak != NULL && pxCloak->IsUsing( true ) )
			{
				pxCloak->RequestEndUsage( AVP_InventoryItem_Cloak::TT_AUTOMATIC );
			}
		
			if ( xParentSystem.IsClientSide() )
			{
				AVP_ClientActor_Vertebrate* pxVert = AVP_ClientActor_Vertebrate::FindEntity( xParentSystem.GetOwnerGuid() );
				AVP_Vertebrate_CameraHandler* pxCameraHandler = pxVert ? static_cast< AVP_Vertebrate_CameraHandler* >( pxVert->GetCameraHandler() ) : NULL;
				if ( pxCameraHandler )
				{
					pxCameraHandler->SetZoomLevel( *pxVert, 0 );
				}

				AVP_ClientEntity_Actor* pxActor = xParentSystem.GetParentClient();
				AVP_Actor_AnimationHandler* pxAnimHandler = pxActor != NULL ? pxActor->GetAnimationHandler() : NULL;
				u_int uFlags( AAC_FLAG_ACTIVE | AAC_FLAG_DESTROY_WHEN_FINISHED );
				u_int uPriority( AVP_AAPC_GENERAL_ANIM );
				Asura_AnimationBlend* pxBlend( new Asura_AnimationBlend_AnimatedTween( 0.05f, 0.05f, false ) );
				Asura_Hash_ID uStimAnim = ASURA_HASH_ID_UNSET;
				Asura_Hash_ID uActorAnim = ASURA_HASH_ID_UNSET;
				Asura_Hash_ID uWeaponAnim = ASURA_HASH_ID_UNSET;

				if ( pxActor != NULL )
				{
					const AVP_Actor_AnimStateData* pxStateData = pxActor->GetCurrentAnimStateData();
					if ( pxStateData )
					{
						uStimAnim = pxStateData->GetStimItemAnimHash();
						uActorAnim = pxStateData->GetStimActorAnimHash();
						uWeaponAnim = pxStateData->GetStimWeaponAnimHash();
					}	
					
					if ( uWeaponAnim != ASURA_HASH_ID_UNSET )
					{
						AVP_Inventory_System* pxInvSys = pxActor->GetInventorySystem();
						AVP_InventoryItem_Weapon* pxWeapon = pxInvSys ? static_cast< AVP_InventoryItem_Weapon* >( pxInvSys->GetCurrentItem( uAVP_INVENTORY_GROUPHASH_WEAPON ) ) : NULL;
						if ( pxWeapon )
						{
							pxWeapon->PlayWeaponAnim( uWeaponAnim );
						}
					}
				}

				m_uActorAACAHash = pxAnimHandler->PlayAnimation( uFlags, uPriority, uActorAnim, pxBlend );

				if ( pxAnimHandler != NULL )
				{
					pxAnimHandler->AttachItem( this );
				}
				if ( m_pxAnimation != NULL && uStimAnim != NULL )
				{
					u_int uFlags = AAC_FLAG_ACTIVE;
					u_int uPriority = AVP_AAPC_GENERAL_ANIM;
					Asura_AnimationBlend* pxBlend = new Asura_AnimationBlend( 1.0f );
					Asura_AnimationController_Animation* pxAACA = m_pxAnimation->CreateControllerAnimation( uFlags, uPriority, uStimAnim, pxBlend );
					if ( pxAACA == NULL )
					{
						delete pxBlend;
					}
					else
					{
						pxAACA->Play();
					}
				}
	
				if ( m_uActorAACAHash == ASURA_HASH_ID_UNSET )
				{
					Asura_Assert( false, "Stim anim not created" );
				}
			}
		}
		break;

		case USING:
		{
			u_int uStimCount = 1;
			if ( m_fStimEnergyCost != 0.0f )
			{
				AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
				if ( pxActor != NULL )
				{
					uStimCount = ( pxActor->GetMaxHealthBlocks() - Asura_Maths::FloatToInt_Chop( pxActor->GetMaxHealthBlocks() * pxActor->GetHealth() / pxActor->GetInitialHealth() ) / m_uStimRegenBlocks ) * m_uStimRegenBlocks;
					uStimCount = Asura_Maths::Min< u_int >( Asura_Maths::FloatToInt_Chop( pxSubType->GetCurrentQuantity() / m_fStimEnergyCost ), uStimCount );
					if ( xParentSystem.IsMaster() )
					{
						pxSubType->DecreaseQuantity( uStimCount * m_fStimEnergyCost );
						pxSubType->SetMaxQuantity( Asura_Maths::Max( pxActor->GetMinEnergy(), pxSubType->GetMaxQuantity() - ( uStimCount * m_uStimEnergyBlocksLost ) ) );
					}
				}
			}
			else
			{
				//Put this line back when we get pickups
				//pxSubType->DecreaseQuantity( 1.0f );
			}

			if ( xParentSystem.IsServerSide() )
			{
				AVP_ServerEntity_Actor* pxServer = xParentSystem.GetParentServer();

				if ( pxServer && !pxServer->IsDead() )
				{
					pxServer->RegenHealthBlocks( uStimCount * m_uStimRegenBlocks );
				
					//Axon_System::GetSoundRegistry()->RegisterSound( pxServer->GetGuid(), pxServer->GetClassification(), pxServer->GetPosition(), 20.0f, AVP_SOUND_TYPE_PRED_ROAR );
				}
			}
		}
		break;

		default:
		{
			// no nothing
		}
		break;
	} // End of switch.
}

void AVP_InventoryItem_Stim::UpdateClientState( AVP_ClientEntity_Actor& xClient )
{
	const State	eCurrentState	= GetCurrentState();

	// Decide what to do based on our current state...
	switch(eCurrentState)
	{
		case INACTIVE:
		{
			//do nothing
		}
		break;

		case USING:
		{
			if ( GetTimeInState() > 0.0f )
			{
				RequestIdling( false );
			}
		}
		break;

		case IDLING:
		{

		}
		break;

		case BEGIN_USING:
		{
			AVP_Actor_AnimationHandler*					pxAnimationHandler	= xClient.GetAnimationHandler();
			const Asura_Animation*						pxAnimation			= pxAnimationHandler != NULL ? pxAnimationHandler->GetAnimation() : NULL;
			const Asura_AnimationController_Animation*  pxAACA				= pxAnimation != NULL ? pxAnimation->GetControllerAnimationFromHash( m_uActorAACAHash ) : NULL;
			if ( pxAACA != NULL )
			{
				if ( ( pxAACA->GetFlagsForLastFrame() & AVP_ANIMFLAG_ACTION_1 ) != 0 )
				{
					if(GetSubType(0))
					{
						if(m_fStimEnergyCost == 0.0f)
						{
							GetSubType(0)->DecreaseQuantity(1.0f);
						}
						
					}
					RequestNextUsingState();
				}
			}
			else if ( GetTimeInState() > 0.0f )
			{
				//if anim has disappeared then skip on to next state
				if(GetSubType(0))
				{
					if(m_fStimEnergyCost == 0.0f)
					{
						GetSubType(0)->DecreaseQuantity(1.0f);
					}
				}
				RequestNextUsingState();
			}
		}
		break;

		case END_USING:
		{
			AVP_Actor_AnimationHandler*					pxAnimationHandler	= xClient.GetAnimationHandler();
			const Asura_Animation*						pxAnimation			= pxAnimationHandler != NULL ? pxAnimationHandler->GetAnimation() : NULL;
			const Asura_AnimationController_Animation*  pxAACA				= pxAnimation != NULL ? pxAnimation->GetControllerAnimationFromHash( m_uActorAACAHash ) : NULL;
			if ( pxAACA != NULL )
			{
				if ( ( pxAACA->GetFlagsForLastFrame() & AVP_ANIMFLAG_TRIGGER_DETACHMENT ) != 0 )
				{
					if ( pxAnimationHandler->IsItemAttached( this ) )
					{
						pxAnimationHandler->DetachItem( this );
						if ( m_pxAnimation )
						{
							DestroyAnimation();
						}
					}
				}

				if ( pxAACA->IsFinished() )
				{
					RequestNextIdlingState();
				}
			}
			else if ( GetTimeInState() > 0.0f )
			{
				//if anim has disappeared then skip on to next state
				RequestNextIdlingState();
			}
		}
		break;

		default:
		{
			Asura_Assert(false, "Unexpected state found when updating client-state of stim.");
		}
		break;
	}		// End of switch.
}

void AVP_InventoryItem_Stim::UpdateServerState( AVP_ServerEntity_Actor& xServer )
{
	const State	eCurrentState	= GetCurrentState();

	// Decide what to do based on our current state...
	switch(eCurrentState)
	{
		case INACTIVE:
		case IDLING:
		{
			//do nothing
		}
		break;

		case USING:
		{
			if ( GetTimeInState() > 0.0f )
			{
				RequestIdling( false );
			}
		}
		break;

		case BEGIN_USING:
		{
			if ( GetTimeInState() > 0.0f )
			{
				const AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity( xServer.GetGuid() );
				const AVP_Inventory_System* pxInvSys = pxClient != NULL ? pxClient->GetInventorySystem() : NULL;
				const AVP_InventoryItem_Stim* pxClientItem = pxInvSys != NULL ? static_cast< AVP_InventoryItem_Stim* >( pxInvSys->GetItem( GetHashInParentGroup() ) ) : NULL;
				if ( pxClientItem == NULL || pxClientItem->IsUsing( true ) || pxClientItem->GetDestinationState() != USING )
				{
					if(GetSubType(0))
					{
						if(m_fStimEnergyCost == 0.0f)
						{
							GetSubType(0)->DecreaseQuantity(1.0f);
						}
					}
					RequestNextUsingState();
				}
			}
		}
		break;

		case END_USING:
		{
			if ( GetTimeInState() > 0.0f )
			{
				const AVP_ClientEntity_Actor* pxClient = AVP_ClientEntity_Actor::FindEntity( xServer.GetGuid() );
				const AVP_Inventory_System* pxInvSys = pxClient != NULL ? pxClient->GetInventorySystem() : NULL;
				const AVP_InventoryItem_Stim* pxClientItem = pxInvSys != NULL ? static_cast< AVP_InventoryItem_Stim* >( pxInvSys->GetItem( GetHashInParentGroup() ) ) : NULL;
				if ( pxClientItem == NULL || !pxClientItem->IsUsing( false ) || pxClientItem->GetDestinationState() != IDLING )
				{
					RequestNextIdlingState();
				}
			}
		}
		break;

		default:
		{
			Asura_Assert(false, "Unexpected state found when updating server-state of stim.");
		}
		break;
	} // End of switch.
}

bool AVP_InventoryItem_Stim::GetCanUseUsageType(const u_int uUsageType, const u_int uTransitionType) const
{
	// In addition to skipping when in moves, we need to make sure we aren't firing weapons or transitioning the cloak (if a predator).
	const AVP_Inventory_System& xParentSystem = GetParentSystem();

	const AVP_Inventory_Item* pxCurrentWeapon = xParentSystem.GetCurrentItem(uAVP_INVENTORY_GROUPHASH_WEAPON);
	if (pxCurrentWeapon && !pxCurrentWeapon->IsIdling(true))
	{
		return false;
	}

	const AVP_Inventory_Item* pxCloak = xParentSystem.GetItem(uAVP_INVENTORY_ITEMHASH_CLOAK);
	if (pxCloak)
	{
		// We need to see if we're in a transition
		const bool bIdling = pxCloak->IsIdling(false);
		const bool bFullyIdling = pxCloak->IsIdling(true);
		const bool bUsing = pxCloak->IsUsing(false);
		const bool bFullyUsing = pxCloak->IsUsing(true);

		if (bIdling && !bFullyIdling)
		{
			// This is still allowed if the transition type is automatic.
			if (pxCloak->GetCurrentTransitionType() != AVP_InventoryItem_Cloak::TT_AUTOMATIC)
			{
				return false;
			}
		}

		if (bUsing && !bFullyUsing)
		{
			return false;
		}
	}

	return PARENT::GetCanUseUsageType(uUsageType, uTransitionType);
}

Asura_Animation* AVP_InventoryItem_Stim::CreateAnimation()
{
	Asura_Assert( GetParentSystem().IsClientSide(), "Should be client side" );
	if ( m_uSkinHash != ASURA_HASH_ID_UNSET )
	{
		Asura_Assert( m_pxAnimation == NULL, "Already got an animation" );
		m_pxAnimation = new Asura_Animation( m_uSkinHash );
		return m_pxAnimation;
	}
	else
	{
		return NULL;
	}
}

void AVP_InventoryItem_Stim::DestroyAnimation()
{
	Asura_Assert( GetParentSystem().IsClientSide(), "Should be client side" );
	delete m_pxAnimation;
	m_pxAnimation = NULL;
}

Asura_Animation* AVP_InventoryItem_Stim::GetAnimation( bool bCreateIfNecessary /*= false */ )
{
	Asura_Assert( GetParentSystem().IsClientSide(), "Should be client side" );
	if ( m_pxAnimation == NULL && bCreateIfNecessary )
	{
		m_pxAnimation = CreateAnimation();
	}
	return m_pxAnimation;
}

const Asura_Animation* AVP_InventoryItem_Stim::GetAnimation() const
{
	Asura_Assert( GetParentSystem().IsClientSide(), "Should be client side" );
	return m_pxAnimation;
}

bool AVP_InventoryItem_Stim::IsEmpty() const
{
	if ( GetNumSubTypes() > 0 && GetSubType( 0 ) )
	{
		bool bEmpty = GetSubType( 0 )->IsEmpty();
		bEmpty |= GetSubType( 0 )->GetCurrentQuantity() < m_fStimEnergyCost;
		return bEmpty;
	}
	else
	{
		Asura_Assert( false, "No subtype to check for emptyness" );
		return false;
	}
}

void AVP_InventoryItem_Stim::GetStaticDataFromBlueprint()
{
	const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, GetBlueprintHash() );
	if ( pxBlueprint )
	{
		Asura_Value xValue;
		if ( pxBlueprint->GetParameterValueByHashID( 0xe250179f, xValue ) )	// RegenHealthBlocks
		{
			m_uStimRegenBlocks = xValue.GetIntValue();
		}
		if ( pxBlueprint->GetParameterValueByHashID( 0xb4050a32, xValue ) )	// EnergyBlocksLost
		{
			m_uStimEnergyBlocksLost = xValue.GetIntValue();
		}
		if ( pxBlueprint->GetParameterValueByHashID( 0x57595e55, xValue ) )	// EnergyCost
		{
			m_fStimEnergyCost = xValue.GetFloatValue();
		}
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SKIN, xValue ) )
		{
			m_uSkinHash = xValue.GetHashIDValue();
		}
	}
	else
	{
		Asura_Assertf( false, ( "AVP_InventoryItem_Stim::GetStaticDataFromBlueprint - cannot find blueprint %u to initialise from.", GetBlueprintHash() ) );
	}
}
