
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryItem_Torch.cpp
// Author : Andy Tate
// Date   : 24-Jan-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Inventory/Items/AVP_InventoryItem_Torch.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "Projectors/Asura_Projector_System.h"
#include "Sound/Asura_Sound_Event_Storage.h"

// Game independent header-files...
#include "Actor/AVP_Actor_AnimationHandler.h"
#include "Actor/AVP_Actor_HUDHandler.h"
#include "Actor/Vertebrates/Human/AVP_Human_InputHandler.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Entity/AVP_Messages.h"
#include "Entity/AVP_server_Messages.h"
#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "Entity/Container/Asura_ContainerState.h"
#endif //ASURA_USE_CONTAINER_SYSTEM


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

DEBUGVAR float dbg_fDisruptTime		= 1.0f;
DEBUGVAR float dbg_fMinFlickerTime	= 0.2f;
DEBUGVAR float dbg_fMaxFlickerTime	= 0.5f;

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch_ClientData::AVP_InventoryItem_Torch_ClientData
// Author       : Andy Tate
// Date         : 25-Jan-2008
////////////////////////////////////////////////////////////////////////////////

AVP_InventoryItem_Torch_ClientData::AVP_InventoryItem_Torch_ClientData(const Asura_Hash_ID uBlueprintHash)
: m_uBlueprintHash(uBlueprintHash)
, m_pxContainerAttachment(0)
{
	m_xWPLightBB.Reset();
}

/// End of function 'AVP_InventoryItem_Torch_ClientData::AVP_InventoryItem_Torch_ClientData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch_ClientData::~AVP_InventoryItem_Torch_ClientData
// Author       : Andy Tate
// Date         : 25-Jan-2008
////////////////////////////////////////////////////////////////////////////////

AVP_InventoryItem_Torch_ClientData::~AVP_InventoryItem_Torch_ClientData()
{
	DestroyContainer();
}

/// End of function 'AVP_InventoryItem_Torch_ClientData::~AVP_InventoryItem_Torch_ClientData'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch_ClientData::PlayOneShotSound
// Author       : Andy Tate
// Date         : 25-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch_ClientData::PlayOneShotSound(AVP_ClientEntity_Actor& xClient,
														  const Asura_Hash_ID uBlueprintParamHash)
{
	const Asura_Blueprint*	pxBlueprint	= Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, m_uBlueprintHash);

	// Extract the blueprints...
	if(pxBlueprint)
	{
		Asura_Value	xValue;

		if(pxBlueprint->GetParameterValueByHashID(uBlueprintParamHash, xValue))
		{
			const Asura_Hash_ID			uSoundEventHash	= xValue.GetHashIDValue();
			const Asura_Sound_Event*	pxSound			= Asura_Sound_Event_Storage::FindEvent(uSoundEventHash);

			if(pxSound != NULL)
			{
				Asura_Vector_3	xPosition;

				if(xClient.GetWorldSpaceMarkerPoint(AVP_Human_Description::GetMarkerPointHashID(AVP_Human_Description::MPT_TORCH), &xPosition, NULL))
				{
					Asura_Sound_Event_System::PlayEvent(pxSound, NULL, &xPosition, 1.0f, xClient.GetGuid());
				}
			}
		}
	}
}

/// End of function 'AVP_InventoryItem_Torch_ClientData::PlayOneShotSound'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch_ClientData::CreateContainer
// Author       : M.G.Harris
// Date         : 23-Sep-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch_ClientData::CreateContainer(AVP_ClientEntity_Actor& xClient)
{
	AVP_Actor_AnimationHandler*	pxAnimationHandler	= xClient.GetAnimationHandler();
	Asura_Animation*			pxAnimation			= (pxAnimationHandler == NULL) ? NULL : pxAnimationHandler->GetAnimation();

	if(pxAnimation == NULL)
	{
		Asura_Assert(false, "Unable to get animation to attach the torch to.");
	}
	else
	{
		const Asura_Blueprint* pxBlueprint	= Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, m_uBlueprintHash);

		if(pxBlueprint == NULL)
		{
			Asura_Assert(false, "Unable to find the torch item blueprint.");
		}
		else
		{
			const Asura_Hash_ID	uMarkerPointHash	= AVP_Human_Description::GetMarkerPointHashID(AVP_Human_Description::MPT_TORCH);
			Asura_Hash_ID		uContainerHash		= ASURA_HASH_ID_UNSET;

			if(xClient.IsUsingFirstPersonSkin())
			{
				Asura_Value	xValue;

				if(pxBlueprint->GetParameterValueByHashID(0xB8E66DF8, xValue))	//Asura_GetHashID("Torch.Beam.FirstPerson", xValue)))
				{
					uContainerHash	= xValue.GetHashIDValue();
				}
			}
			else
			{
				Asura_Value	xValue;

				if(pxBlueprint->GetParameterValueByHashID(0xFC71FB6F, xValue))	//Asura_GetHashID("Torch.Beam.ThirdPerson", xValue)))
				{
					uContainerHash	= xValue.GetHashIDValue();
				}
			}

			// Do we have a container to use?
			if(uContainerHash == ASURA_HASH_ID_UNSET)
			{
				Asura_Assertf(false, ("Failed to find container to use for the torch (GUID %d).", xClient.GetGuid()));
			}
			else
			{
				// Create the attachment for the container...
				m_pxContainerAttachment = new Asura_AnimationAttachment_ClientContainerInstance(uContainerHash, Asura_Container_InputState::Off());
				m_pxContainerAttachment->LinkAttachment(pxAnimation, uMarkerPointHash, false);	
			}
		}
	}
}



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch_ClientData::DestroyContainer
// Author       : M.G.Harris
// Date         : 23-Sep-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch_ClientData::DestroyContainer()
{
	if (m_pxContainerAttachment != 0)
	{
		if (m_pxContainerAttachment->IsLinked())
		{
			m_pxContainerAttachment->UnlinkAttachment(true);
		}
		else
		{
			delete m_pxContainerAttachment;
		}
		m_pxContainerAttachment = 0;
	}
}



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch::OnStateChange
// Author       : Andy Tate
// Date         : 29-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch::OnStateChange(const u_int uOldState,
											const u_int uNewState)
{
	AVP_Inventory_System&	xParentSystem	= GetParentSystem();
	AVP_ClientActor_Human*	pxClient		= static_cast<AVP_ClientActor_Human*>(xParentSystem.GetParentClient());

	// Call the base-function...
	PARENT::OnStateChange(uOldState, uNewState);

	// Do we need to handle client-side stuff?
	if(pxClient != NULL)
	{
		switch(uOldState)
		{
			case USING:
			{
				const State	eDestinationState	= GetDestinationState();

				if(eDestinationState == IDLING)
				{
					if(m_pxClientData != NULL)
					{
						m_pxClientData->TurnOffTorch(*pxClient);
					}

					AVP_InventoryItem_Torch_ClientData::Destroy(m_pxClientData);
					m_pxClientData	= NULL;
				}
				break;
			}

			case IDLING:
			case INACTIVE: // hmmmm
			{
				const State	eDestinationState	= GetDestinationState();

				if(eDestinationState == USING)
				{
					AVP_InventoryItem_Torch_ClientData::Destroy(m_pxClientData);
					m_pxClientData	= AVP_InventoryItem_Torch_ClientData::Create(m_uBlueprintHash);

					if(m_pxClientData != NULL)
					{
						m_pxClientData->TurnOnTorch(*pxClient);
					}
				}
				break;
			}

			default:
			{
				break;
			}
		}		// End of switch.
	}
}

/// End of function 'AVP_InventoryItem_Torch::OnStateChange'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch::UpdateClientState
// Author       : Andy Tate
// Date         : 28-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch::UpdateClientState(AVP_ClientEntity_Actor& xClient)
{
	const State	eCurrentState	= GetCurrentState();

	// Decide what to do based on our current state...
	switch(eCurrentState)
	{
		case INACTIVE:
		case IDLING:
		{
			break;
		}

		case USING:
		{
			UpdateDisruption( xClient );

// 			m_fLightTimer -= Asura_Timers::GetGameTimeSlice();
// 			if ( m_fLightTimer < 0.0f )
// 			{
// 				m_fLightTimer = 2.0f;
// 				if ( Asura_Network::IsServer() && m_pxClientData )
// 				{
// 					Asura_Message* pxMsg = Asura_Network::CreateMessage( AVP_MSGID_LIGHT_CHANGE_EVENT_CODE, sizeof( AVP_Message_LightChange ), xClient.GetClassification() );
// 					if ( pxMsg )
// 					{
// 						AVP_Message_LightChange* pxData = static_cast< AVP_Message_LightChange* >( pxMsg->GetData() );
// 						if ( pxData )
// 						{
// 							pxData->SetOldBB( m_pxClientData->GetWaypointLightingBB() );
// 							m_pxClientData->UpdateWaypointLightingBB( xClient );
// 							pxData->SetNewBB( m_pxClientData->GetWaypointLightingBB() );
// 
// 							pxMsg->SetToGuid( ASURA_GUID_SERVER );
// 							pxMsg->SetFromGuid( xClient.GetGuid() );
// 							Asura_Network::Send( pxMsg );
// 						}
// 					}
// 				}
// 			}

			break;
		}

		default:
		{
			Asura_Assert(false, "Unexpected state found when updating master-state of weapon.");
			break;
		}
	}		// End of switch.
}

/// End of function 'AVP_InventoryItem_Torch::UpdateClientState'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch::UpdateServerState
// Author       : Andy Tate
// Date         : 28-Nov-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch::UpdateServerState(AVP_ServerEntity_Actor& xServer)
{
	const State	eCurrentState	= GetCurrentState();

	// Decide what to do based on our current state...
	switch(eCurrentState)
	{
		case INACTIVE:
		{
			break;
		}

		case IDLING:
		{
			break;
		}

		case USING:
		{
			break;
		}

		default:
		{
			Asura_Assert(false, "Unexpected state found when updating master-state of weapon.");
			break;
		}
	}		// End of switch.
}

/// End of function 'AVP_InventoryItem_Torch::UpdateServerState'. ///

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch_ClientData::Create
// Author       : Andy Tate
// Date         : 25-Jan-2008
////////////////////////////////////////////////////////////////////////////////

AVP_InventoryItem_Torch_ClientData* AVP_InventoryItem_Torch_ClientData::Create(const Asura_Hash_ID uBlueprintHash)
{
	return(new AVP_InventoryItem_Torch_ClientData (uBlueprintHash));
}

/// End of function 'AVP_InventoryItem_Torch_ClientData::Create'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch_ClientData::Destroy
// Author       : Andy Tate
// Date         : 25-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch_ClientData::Destroy(AVP_InventoryItem_Torch_ClientData* pxClientData)
{
	delete(pxClientData);
}

/// End of function 'AVP_InventoryItem_Torch_ClientData::Destroy'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch_ClientData::IsTorchOff
// Author       : M.G.Harris
// Date         : 23-Sep-2008
////////////////////////////////////////////////////////////////////////////////

bool AVP_InventoryItem_Torch_ClientData::IsTorchOff() const
{
	return !m_pxContainerAttachment || m_pxContainerAttachment->IsOff();
}


void AVP_InventoryItem_Torch_ClientData::UpdateWaypointLightingBB( AVP_ClientEntity_Actor& xClient )
{
	Asura_AVector3 xDirection;
	xClient.GetAimDirection( xDirection );
	m_xWPLightBB.SetToPoint( xClient.GetCurrentPosition() );
	m_xWPLightBB.Set( xClient.GetCurrentPosition() + Asura_Vector_3( xDirection ) * 15.0f );
	m_xWPLightBB.Grow( 5.0f );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch_ClientData::TurnOnTorch
// Author       : Andy Tate
// Date         : 25-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch_ClientData::TurnOnTorch(AVP_ClientEntity_Actor& xClient)
{
	if (!m_pxContainerAttachment)
	{
		CreateContainer(xClient);
	}

	if (m_pxContainerAttachment)
	{
		if (IsTorchOff() && !xClient.IsDead())
		{
			PlayOneShotSound(xClient, 0x851CBF81);	//Asura_GetHashID("Torch.Sound.TurnOff"));
		}

		m_pxContainerAttachment->ChangeState( Asura_Container_InputState::On() );
	}

	if ( xClient.GetHUDHandler() )
	{
		xClient.GetHUDHandler()->SetTorchHDROverride( m_uBlueprintHash );
	}

// 	Asura_Message* pxMsg = Asura_Network::CreateMessage( AVP_MSGID_LIGHT_CHANGE_EVENT_CODE, sizeof( AVP_Message_LightChange ), xClient.GetClassification() );
// 	if ( pxMsg )
// 	{
// 		AVP_Message_LightChange* pxData = static_cast< AVP_Message_LightChange* >( pxMsg->GetData() );
// 		if ( pxData )
// 		{
// 			pxData->InvalidateOldBB();
// 			UpdateWaypointLightingBB( xClient );
// 			pxData->SetNewBB( GetWaypointLightingBB() );
// 
// 			pxMsg->SetToGuid( ASURA_GUID_SERVER );
// 			pxMsg->SetFromGuid( xClient.GetGuid() );
// 			Asura_Network::Send( pxMsg );
// 		}
// 	}
}

/// End of function 'AVP_InventoryItem_Torch_ClientData::TurnOnTorch'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch_ClientData::TurnOffTorch
// Author       : Andy Tate
// Date         : 25-Jan-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch_ClientData::TurnOffTorch(AVP_ClientEntity_Actor& xClient)
{
	if (m_pxContainerAttachment)
	{
		if (!IsTorchOff() && !xClient.IsDead())
		{
			PlayOneShotSound(xClient, 0x1D11718D);	//Asura_GetHashID("Torch.Sound.TurnOn"));
		}

		m_pxContainerAttachment->ChangeState( Asura_Container_InputState::Off() );
	}

	if ( xClient.GetHUDHandler() )
	{
		xClient.GetHUDHandler()->UnSetTorchHDROverride();
	}

// 	Asura_Message* pxMsg = Asura_Network::CreateMessage( AVP_MSGID_LIGHT_CHANGE_EVENT_CODE, sizeof( AVP_Message_LightChange ), xClient.GetClassification() );
// 	if ( pxMsg )
// 	{
// 		AVP_Message_LightChange* pxData = static_cast< AVP_Message_LightChange* >( pxMsg->GetData() );
// 		if ( pxData )
// 		{
// 			pxData->SetOldBB( GetWaypointLightingBB() );
// 			pxData->InvalidateNewBB();
// 
// 			pxMsg->SetToGuid( ASURA_GUID_SERVER );
// 			pxMsg->SetFromGuid( xClient.GetGuid() );
// 			Asura_Network::Send( pxMsg );
// 		}
// 	}
}

/// End of function 'AVP_InventoryItem_Torch_ClientData::TurnOffTorch'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch::AVP_InventoryItem_Torch
// Author       : Andy Tate
// Date         : 24-Jan-2008
////////////////////////////////////////////////////////////////////////////////

AVP_InventoryItem_Torch::AVP_InventoryItem_Torch(AVP_Inventory_Group& xGroup,
												 const Asura_Hash_ID uBlueprintHash,
												 const char* szName)
: PARENT(AVP_INVENTORYITEMTYPE_TORCH, xGroup, uAVP_INVENTORY_ITEMHASH_TORCH, uBlueprintHash, szName)
, m_pxClientData(NULL)
, m_fDisruptTimer( 0.0f )
, m_fFlickerTimer( 0.0f )
, m_fLightTimer( 0.0f )
{
	// Initialise the valid states...
	SetStateAsValid(USING);
}

/// End of function 'AVP_InventoryItem_Torch::AVP_InventoryItem_Torch'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch::~AVP_InventoryItem_Torch
// Author       : Andy Tate
// Date         : 25-Jan-2008
////////////////////////////////////////////////////////////////////////////////

AVP_InventoryItem_Torch::~AVP_InventoryItem_Torch()
{
	AVP_InventoryItem_Torch_ClientData::Destroy(m_pxClientData);
}

void AVP_InventoryItem_Torch::Disrupt( AVP_ClientEntity_Actor& xClient )
{
	m_fDisruptTimer = dbg_fDisruptTime;
	if ( m_pxClientData != NULL )
	{
		m_pxClientData->TurnOffTorch( xClient );
	}
}

void AVP_InventoryItem_Torch::UpdateDisruption( AVP_ClientEntity_Actor& xClient )
{
	// TODO it would be better to define a special "flickering" state for the torch container.
	if ( m_fDisruptTimer > 0.0f )
	{
		const float fTimeSlice = Asura_Timers::GetGameTimeSlice();
		m_fDisruptTimer -= fTimeSlice;
		if ( m_fDisruptTimer <= 0.0f )
		{
			m_fDisruptTimer = 0.0f;
			if ( m_pxClientData != NULL )
			{
				m_pxClientData->TurnOnTorch( xClient );
			}
		}
		else if ( m_fFlickerTimer > 0.0f )
		{
			m_fFlickerTimer -= fTimeSlice;
		}
		else if ( m_pxClientData->IsTorchOff() )
		{
			if ( m_pxClientData != NULL )
			{
				m_pxClientData->TurnOnTorch( xClient );
			}
		}
		else
		{
			if ( m_pxClientData != NULL )
			{
				m_pxClientData->TurnOffTorch( xClient );
			}
			m_fFlickerTimer = Asura_Random::GetFloat( dbg_fMinFlickerTime, dbg_fMaxFlickerTime );
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch::ReadFromChunkStream
// Author       : Chris Wallace
// Date         : 04-Nov-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int uVersion;
	xStream >> uVersion;

	if ( uVersion >= 1 )
	{
		PARENT::ReadFromChunkStream( xStream );
	}

	if ( uVersion == uCURRENT_CHUNK_VERSION || uVersion == 0 )
	{
		bool bInUse;
		xStream >> bInUse;
		if(bInUse)
		{
			if(m_xParentGroup.GetParentSystem().IsMaster())
			{
				RequestBeginUsage(UT_TORCH, TT_AUTOMATIC);
			}
			else
			{
				// Can't request on non-master, set it directly so can be correctly forwarded to master.
				m_uCurrentUsageType = UT_TORCH;
				m_uCurrentTransitionType = TT_AUTOMATIC;
				m_eCurrentState = USING;
			}
		}
	}
	else
	{
		Asura_Assertf( false, ( "AVP_InventoryItem_Torch::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

/// End of function 'AVP_InventoryItem_Torch::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryItem_Torch::WriteToChunkStream
// Author       : Chris Wallace
// Date         : 04-Nov-2008
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryItem_Torch::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << IsUsing(true);
}

/// End of function 'AVP_InventoryItem_Torch::WriteToChunkStream'. ///

/// End of file. ///
