
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryItem_Torch.h
// Author : Andy Tate
// Date   : 24-Jan-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_INVENTORYITEM_TORCH_H__
#define __AVP_INVENTORYITEM_TORCH_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Maths.h"

#include "Inventory/AVP_Inventory_Item.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_AnimationAttachment_Projector;
class Asura_AnimationAttachment_ClientContainerInstance;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_InventoryItem_Torch_ClientData
// Author       : Andy Tate
// Date         : 25-Jan-2008
//
// Description  : Class to contain the client-data needed for the torch
//				  inventory-item, also handles any updating related to this data.
////////////////////////////////////////////////////////////////////////////////

class AVP_InventoryItem_Torch_ClientData
{
public:
	// Public functions...
	void	TurnOnTorch(AVP_ClientEntity_Actor& xClient);
	void	TurnOffTorch(AVP_ClientEntity_Actor& xClient);

	bool	IsTorchOff() const;

	const Asura_Bounding_Box& GetWaypointLightingBB() const { return m_xWPLightBB; }
	void UpdateWaypointLightingBB( AVP_ClientEntity_Actor& xClient );

	// Static creation/destruction functions...
	static AVP_InventoryItem_Torch_ClientData*	Create(const Asura_Hash_ID uBlueprintHash);
	static void									Destroy(AVP_InventoryItem_Torch_ClientData* pxClientData);


protected:
	// Protected variables...
	Asura_Hash_ID										m_uBlueprintHash;
	Asura_AnimationAttachment_ClientContainerInstance*	m_pxContainerAttachment;

	// Protected functions...
	AVP_InventoryItem_Torch_ClientData(const Asura_Hash_ID uBlueprintHash);
	~AVP_InventoryItem_Torch_ClientData();

	void	PlayOneShotSound(AVP_ClientEntity_Actor& xClient, const Asura_Hash_ID uBlueprintParamHash);

	void	CreateContainer(AVP_ClientEntity_Actor& xClient);
	void	DestroyContainer();


private:
	// Private variables...
	Asura_Bounding_Box m_xWPLightBB;

	// Private functions...
	AVP_InventoryItem_Torch_ClientData(void);
	AVP_InventoryItem_Torch_ClientData(const AVP_InventoryItem_Torch_ClientData& xOther);

	AVP_InventoryItem_Torch_ClientData&	operator=(const AVP_InventoryItem_Torch_ClientData& xOther);

};   // End of class 'AVP_InventoryItem_Torch_ClientData'.



////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_InventoryItem_Torch
// Author       : Andy Tate
// Date         : 24-Jan-2008
//
// Description  : Class to handle the use of the torch as an inventory-item.
////////////////////////////////////////////////////////////////////////////////

class AVP_InventoryItem_Torch : public AVP_Inventory_Item
{
public:
	// Public enumerated types...
	enum TorchTransitionTypes
	{
		TT_MANUAL	= NUM_ITEM_TRANSITION_TYPES,
		TT_AUTOMATIC,

		NUM_TORCH_TRANSITION_TYPES
	};

	enum TorchUsageTypes
	{
		UT_TORCH	= NUM_ITEM_USAGE_TYPES,

		NUM_TORCH_USAGE_TYPES
	};

	// Public functions...
	AVP_InventoryItem_Torch(AVP_Inventory_Group& xGroup, const Asura_Hash_ID uBlueprintHash, const char* szName);
	virtual ~AVP_InventoryItem_Torch();

	virtual	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	void Disrupt( AVP_ClientEntity_Actor& xClient );

protected:
	// Protected variables...
	AVP_InventoryItem_Torch_ClientData*	m_pxClientData;		// A pointer to the data used by the client.

	// Protected functions...

	// Overridden state functions...
	virtual void	OnStateChange(const u_int uOldState, const u_int uNewState);

	virtual void	UpdateClientState(AVP_ClientEntity_Actor& xClient);
	virtual void	UpdateServerState(AVP_ServerEntity_Actor& xServer);

private:
	// Private typedefs...
	typedef AVP_Inventory_Item PARENT;

	// Private variables...
	float m_fDisruptTimer;
	float m_fFlickerTimer;
	float m_fLightTimer;

	// Private functions...
	void UpdateDisruption( AVP_ClientEntity_Actor& xClient );

	AVP_InventoryItem_Torch(void);
	AVP_InventoryItem_Torch(const AVP_InventoryItem_Torch& xOther);

	AVP_InventoryItem_Torch&	operator=(const AVP_InventoryItem_Torch& xOther);

	static const u_int uCURRENT_CHUNK_VERSION = 1;
};   // End of class 'AVP_InventoryItem_Torch'.


#endif   // ifndef __AVP_INVENTORYITEM_TORCH_H__


/// End of file. ///
