
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryItem_VisionMode.h
// Author : Andy Tate
// Date   : 28-Nov-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_INVENTORYITEM_VISIONMODE_H__
#define __AVP_INVENTORYITEM_VISIONMODE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/AVP_Entity_Classification.h"
#include "Inventory/AVP_Inventory_Item.h"
#include "Rendering/Asura_Colour.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_InventoryItem_VisionMode
// Author       : Andy Tate
// Date         : 28-Nov-2007
//
// Description  : Class uses to handle ALL vision-modes via the inventory-system.
////////////////////////////////////////////////////////////////////////////////

class AVP_InventoryItem_VisionMode : public AVP_Inventory_Item
{
public:
	AVP_InventoryItem_VisionMode(AVP_Inventory_Group& xParentGroup, const Asura_Hash_ID uBlueprintHash, const Asura_Hash_ID uNameHash );

	bool GetDisablesMotionTracker(void) const	{ return m_bDisablesMotionTracer; }
	bool GetDisablesTorch(void) const			{ return m_bDisablesTorch; }

	float GetVisionModeMultiplier( const AVP_ENTITY_CLASSIFICATION uClass ) const;

	virtual int GetFixedSlot() const { return m_iFixedSlot; }

	const Asura_RGBA &GetGlowColour() const { return m_xGlowColour; }
	const Asura_RGBA &GetLaserColour() const { return m_xLaserColour; }

	void Disrupt();

protected:
	virtual void	OnStateChange(const u_int uOldState, const u_int uNewState);
	virtual void	UpdateClientState(AVP_ClientEntity_Actor& xClient);
	virtual void	UpdateServerState(AVP_ServerEntity_Actor& xServer);


	void UpdateDisruption( AVP_ClientEntity_Actor& xClient );

	float		  m_fFocusLockMultAlien;
	float		  m_fFocusLockMultAndroid;
	float		  m_fFocusLockMultHuman;
	float	      m_fFocusLockMultPredator;
	Asura_Hash_ID m_uDSPChainHash;

private:
	typedef AVP_Inventory_Item PARENT;

	void GetStaticDataFromBlueprint();

	float m_fDisruptTimer;
	float m_fFlickerTimer;

	Asura_RGBA m_xGlowColour;
	Asura_RGBA m_xLaserColour;

	Asura_Hash_ID m_uBustedBlueprintHash;

	union
	{
		u_int m_uFlags;
		struct
		{
			u_int	m_bDisablesMotionTracer : 1,
					m_bDisablesTorch		: 1,
					m_bDisrupted			: 1,
											: 0;
		};
	};

	int  m_iFixedSlot;

	AVP_InventoryItem_VisionMode(void);
	AVP_InventoryItem_VisionMode(const AVP_InventoryItem_VisionMode& xOther);
	AVP_InventoryItem_VisionMode&	operator=(const AVP_InventoryItem_VisionMode& xOther);
};


#endif   // ifndef __AVP_INVENTORYITEM_VISIONMODE_H__


/// End of file. ///
