///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryWeapon_DropshipMissile.cpp
// Author : Chris Wallace
// Date   : 04-Dec-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_DropshipMissile.h"

// Asura independent header-files...

// Game independent header-files...
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"
#include "Inventory/AVP_Inventory_System.h"

AVP_InventoryWeapon_DropshipMissile::AVP_InventoryWeapon_DropshipMissile( AVP_Inventory_Group& xParentGroup, const Asura_Hash_ID uHash, Asura_Hash_ID uBlueprintHash )
: PARENT( xParentGroup, uHash, uBlueprintHash )
{
	m_eItemType = AVP_INVENTORYITEMTYPE_DROPSHIP_MISSILE;
}

void AVP_InventoryWeapon_DropshipMissile::SendCreateProjectileMessage( const AVP_GenericProjectileData* pxProjectileData, const AVP_RangedWeapon_SubType* pxSubType, u_int uUsageType, Asura_Hash_ID uMuzMP, float fRange, float fMaxRange, const Asura_Vector_3& xActorVel, const Asura_Vector_3& xFirePosition, const Asura_Vector_3& xShotDirection, const Asura_Matrix_3x3& xShotOrientation, Asura_Guid* puTargetGuid /*=NULL*/ )
{
	Asura_Guid uOwnerGuid = GetParentSystem().GetOwnerGuid();
	if ( GetParentSystem().IsMaster() )
	{
		Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_CREATE_PROJECTILE_DROPSHIP_MISSILE, sizeof( AVP_Message_CreateGenericProjectile ) );
		if ( pxMessage != NULL )
		{
			AVP_Message_CreateGenericProjectile* pxData = static_cast< AVP_Message_CreateGenericProjectile* >( pxMessage->GetData() );
			
			// Fill-in the message...
			pxData->SetProjectileBlueprintHash( pxProjectileData->GetProjectileHash() );
			pxData->SetPosition( xFirePosition );
			pxData->SetOrientation( Asura_Quat(xShotOrientation) );
			pxData->SetActorVelocity( xActorVel );

			pxData->SetDamageScale( 1.0f );
			pxData->SetLaunch( true );

			if(puTargetGuid)
			{
				pxData->SetTargetGuid(*puTargetGuid);
			}
			else
			{
				pxData->SetTargetGuid(ASURA_GUID_UNREGISTERED);
			}

			pxMessage->SetToGuid( uOwnerGuid );
			pxMessage->SetFromGuid( uOwnerGuid );
			
			Asura_Network::Send( pxMessage );
		}
	}
}

void AVP_InventoryWeapon_DropshipMissile::CalculateFiringDataForSubType( const AVP_ClientEntity_Actor& xActor, Asura_Hash_ID uMuzMP, float fMuzVel, float fMaxRange, Asura_Vector_3& xFirePosition, Asura_Vector_3& xFireDirection, Asura_Matrix_3x3& xMuzzleOri, float& fRange, Asura_Guid* puTargetGuid /*= NULL*/ )
{
	if(!xActor.GetMarkerLocationFromHash( uMuzMP, &xFirePosition, &xMuzzleOri ))
	{
		xFirePosition	= xActor.GetCurrentPosition();
		xMuzzleOri		= xActor.GetCurrentOrientation();
	}

	xFireDirection = Asura_Vector_3( xMuzzleOri.e13, xMuzzleOri.e23, xMuzzleOri.e33 );
	fRange = 1000.0f;
}


/// End of file. ///
