
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryWeapon_Flamethrower.h
// Author : Emil Popovic
// Date   : 11-Sep-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_INVENTORYWEAPON_FLAMETHROWER_H__
#define __AVP_INVENTORYWEAPON_FLAMETHROWER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Animation/Asura_Animation_BoneDataModifier.h"
#include "Entity/Container/Asura_ClientContainerInstance.h"

#include "Entity/PhysicalObject/Projectile/AVP_ClientProjectile_Flamethrower.h"
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_RangedWeapon_SubType;
class AVP_BoneDataModifier_AnimationFlame;
class Asura_AnimationAttachment_ClientContainerInstance;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_InventoryWeapon_Flamethrower
// Author       : Emil Popovic
// Date         : 11-Sept-2008
//
// Description  : Class to handle either client or server-side updating of the
//				  Flamethrower weapon.
////////////////////////////////////////////////////////////////////////////////

class AVP_InventoryWeapon_Flamethrower : public AVP_InventoryWeapon_RangedWeapon
{
public:
	enum FlamethrowerUsageType
	{
		UT_COOL_DOWN = NUM_RANGED_WEAPON_USAGE_TYPES,	// COOl_DOWN is not used for the flamethrower.

		NUM_FLAMETHROWER_USAGE_TYPES
	};

	AVP_InventoryWeapon_Flamethrower( AVP_Inventory_Group& xParentGroup, const Asura_Hash_ID uHash, const Asura_Hash_ID uBlueprintHash );
	virtual ~AVP_InventoryWeapon_Flamethrower();

	virtual int		GetSubTypeIndexFromUsageType( const u_int uUsageType ) const;

protected:
	virtual void	StartUsing(const u_int uUsageType);
	virtual void	StopUsing(const u_int uUsageType);
	virtual bool	IsPlayerIsStillFiring( const AVP_ClientEntity_Actor& xActor, int iSubType ) const;
	virtual void	ProcessPlayerInput( const AVP_ClientEntity_Actor& xActor );
	virtual void	HandleReload();

	virtual void	DoBulletDamage( const AVP_GenericProjectileData* pxProjectileData, Asura_Hash_ID uMuzMP, float fRange, float fMaxRange, const Asura_Vector_3& xShotDirection ) { Asura_Assert( false, "Flamethrower shouldn't be doing bullet damage, is the projectile collision object set?" ); }

	virtual void	SendCreateProjectileMessage( const AVP_GenericProjectileData* pxProjectileData, const AVP_RangedWeapon_SubType* pxSubType, u_int uUsageType, Asura_Hash_ID uMuzMP, float fRange, float fMaxRange, const Asura_Vector_3& xActorVel, const Asura_Vector_3& xFirePosition, const Asura_Vector_3& xShotDirection, const Asura_Matrix_3x3& xShotOrientation, Asura_Guid* puTargetGuid =NULL );

	// It will update graphics of flamethrower. Graphics and visual / sound effects are independent of server side of the game.
	virtual void	UpdateClientState(AVP_ClientEntity_Actor& xClient);
	virtual void	UpdateServerState( AVP_ServerEntity_Actor& xServer );

	virtual void	DestroyAnimation();

private:
	void			GetStaticDataFromBlueprint();
	void			FlameAttachmentOn( const u_int uUsageType );
	void			FlameAttachmentOff();

	void			UpdateAmmoAnim(void);

	Asura_AnimationAttachment_ClientContainerInstance*	m_pxContainerAttachment;
	Asura_Container_InputState							m_xInputStateOn1P;
	Asura_Container_InputState							m_xInputStateOn3P;
	Asura_Animation*									m_pxAnimation;
	AVP_BoneDataModifier_AnimationFlame*				m_pxFlameModifier;
	Asura_Hash_ID										m_uFlameMeshContainerHash;

	float												m_fAmmoParametric;

	u_int												m_uShotCount;

	u_int												m_uAmmoToConsumeBeforeStopFiring;

	typedef AVP_InventoryWeapon_RangedWeapon PARENT;

	AVP_InventoryWeapon_Flamethrower(void);
	AVP_InventoryWeapon_Flamethrower(const AVP_InventoryWeapon_Flamethrower& xOther);

	AVP_InventoryWeapon_Flamethrower&	operator=(const AVP_InventoryWeapon_Flamethrower& xOther);

	static const u_int uCURRENT_CHUNK_VERSION = 0;

	//-------------
	// Internal class that will enumerate all meshes to find correct flame
	class AVP_FindFlameMeshFunctor : public Asura_Container_Functor
	{
		const Asura_ClientContainerInstance_Mesh *m_pxMesh;
	public:
		const Asura_ClientContainerInstance_Mesh *GetMesh() const {return m_pxMesh; }

		AVP_FindFlameMeshFunctor() : m_pxMesh(NULL) {}
		virtual ~AVP_FindFlameMeshFunctor() {}

		virtual void Process(const Asura_ClientContainerInstance* pxEntry)
		{
			if (pxEntry->GetClassification() == AsuraContainerClass_Mesh)
			{
				m_pxMesh = static_cast<const Asura_ClientContainerInstance_Mesh*>(pxEntry);
			}
		}
	};
	//-------------

};   // End of class 'AVP_InventoryWeapon_Flamethrower'.



////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_BoneDataModifier_AnimationFlame
// Author       : Emil Popovic
// Date         : 11-Sept-2008
//
// Description  : Class to handle actual visual of flame. Flame is created from the mesh that has bones in it.
//                So when flame is created, then this mesh is trying to follow the invisible spline line, that is leading
//                the way.
////////////////////////////////////////////////////////////////////////////////



class AVP_BoneDataModifier_AnimationFlame : public Asura_Animation_BoneDataModifier
{
public:
	virtual void UpdateBoneData( const int iFirstBone, const int iLastBone );


	AVP_BoneDataModifier_AnimationFlame(Asura_Animation* pxAnimation, Asura_Animation *pxAttachmentAnimation, Asura_Hash_ID xAttachmentHash);
	virtual ~AVP_BoneDataModifier_AnimationFlame();

	void SetOwnerGuid( Asura_Guid uOwnerGuid )	{	m_uOwnerGuid = uOwnerGuid;	}

protected:

	static bool		FindAllProjectilesFlametrowerFiredOut( Asura_List< AVP_ClientProjectile_Flamethrower > &xList, Asura_Guid uOwnerGuid, const Asura_Vector_3& xCurrentMuzzlePos );

private:

	u_int				m_uFrameNo;
	Asura_Animation*    m_pxAttachmentAnimation;
	Asura_Hash_ID       m_xAttachmentHash;
	Asura_Guid			m_uOwnerGuid;		// Using this value I can identify which projectiles in the air belongs to this weapon. Or better say to owner of this weapon.

	void Debug_PrintBoneInfo();
	static int CompareProjectilesByGuid( const AVP_ClientProjectile_Flamethrower& pxObjectOne, const AVP_ClientProjectile_Flamethrower& pxObjectTwo )	{ return pxObjectTwo.GetGuid() - pxObjectOne.GetGuid(); }
};





#endif   // ifndef __AVP_INVENTORYWEAPON_FLAMETHROWER_H__


/// End of file. ///
