
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryWeapon_PlasmaCaster.cpp
// Author : Chris Wallace
// Date   : 15-Jan-2009
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Inventory/Items/Weapons/AVP_InventoryWeapon_PlasmaCaster.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Entity/PhysicalObject/Moves/Asura_Movement_System.h"
#include "Network/Asura_Message.h"
#include "Network/Asura_Network.h"
#include "SpecialFX/PFX/Asura_PFX_Entity_Effect.h"

// Game independent header-files...
#include "Actor/Vertebrates/AVP_Vertebrate_Focus.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_AnimationHandler.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_InputHandler.h"
#include "Animation/AVP_AnimFlags.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_SurfaceEffects.h"
#include "Entity/AVP_Entity_Classification.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ClientActor_Predator.h"
#include "Entity/PhysicalObject/Projectile/AVP_ClientProjectile_Disc.h"
#include "Entity/PhysicalObject/Projectile/AVP_ServerProjectile_Disc.h"
#include "Entity/PhysicalObject/Projectile/AVP_Projectile_Messages.h"
#include "Entity/PhysicalObject/Projectile/AVP_GenericProjectileData.h"
#include "Inventory/AVP_Inventory_Group.h"
#include "Inventory/AVP_Inventory_HashIDs.h"
#include "Inventory/AVP_Inventory_System.h"
#include "Inventory/Items/Weapons/AVP_RangedWeapon_ClientEffectsHandler.h"
#include "Moves/AVP_Movement_System.h"

#include "Actor/Vertebrates/Predator/AVP_Predator_LockOn.h"

AVP_InventoryWeapon_PlasmaCaster::AVP_InventoryWeapon_PlasmaCaster( AVP_Inventory_Group& xParentGroup, const Asura_Hash_ID uHash, Asura_Hash_ID uBlueprintHash )
: PARENT( xParentGroup, uHash, uBlueprintHash )
, m_uLockingOnPFXBlueprintHash( ASURA_HASH_ID_UNSET )
{
	for(u_int u=0;u<AVP_MAX_LOCKING_ON_PFX;++u)
	{
		m_auLockOnPFXEffectID[u] = ASURA_PFX_INVALID_EFFECT_ID;
	}
	m_eItemType = AVP_INVENTORYITEMTYPE_PLASMACASTER;
}

void AVP_InventoryWeapon_PlasmaCaster::InitialiseFromBlueprint()
{
	PARENT::InitialiseFromBlueprint();

	const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, GetBlueprintHash() );
	if ( pxBlueprint )
	{
		Asura_Value xValue;

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_LOCKON_PFX_BP, xValue ) )
		{
			m_uLockingOnPFXBlueprintHash = xValue.GetHashIDValue();
		}
	}
}

void AVP_InventoryWeapon_PlasmaCaster::ProcessPlayerInput( const AVP_ClientEntity_Actor& xActor )
{
	if(AVP_Predator_LockOn::GetSingleton().IsLockedOn()) // Don't do anything with the PC untill lockon is complete
	{
		StopLockingOnPFXEffect();
		PARENT::ProcessPlayerInput(xActor);
	}
	else
	{
		if(AVP_Predator_LockOn::GetSingleton().GetLockState() == AVP_Predator_LockOn::TARGET_LOCKING)
		{
			const AVP_Actor_InputHandler* pxInputHandler = xActor.GetInputHandler();
			
			// Work out if we should reload...
			const bool bNoPrimaryAmmo = m_ppxSubTypes[ ST_PRIMARY ]->IsEmpty();

			if ( pxInputHandler->GetControlMapValue( AVP_Vertebrate_InputHandler::PRIMARY_ATTACK_HELD ) > 0.0f && ( !bNoPrimaryAmmo ) )
			{
				DoLockingOnPFXEffect();
			}
			else
			{
				StopLockingOnPFXEffect();
			}
		}
		else
		{
			StopLockingOnPFXEffect();
		}
	}
}

bool AVP_InventoryWeapon_PlasmaCaster::IsPlayerIsStillFiring( const AVP_ClientEntity_Actor& xActor, int iSubType ) const
{
	if(AVP_Predator_LockOn::GetSingleton().GetLockState() != AVP_Predator_LockOn::TARGET_UNLOCKED || AVP_Predator_LockOn::GetSingleton().GetLockTargetGuid() != ASURA_GUID_UNREGISTERED) // If we are not in the lockon state, assume we are not firing
	{
		return PARENT::IsPlayerIsStillFiring(xActor, iSubType);
	}
	else
	{
		return false;
	}
}

void AVP_InventoryWeapon_PlasmaCaster::DoLockingOnPFXEffect()
{
	if(!GetParentSystem().IsClientSide()) return;

	Asura_Hash_ID uMarkerPoint = ASURA_HASH_ID_UNSET;
	AVP_Actor_AnimationHandler* pxAnimHandler = GetParentSystem().GetParentClient()->GetAnimationHandler();
	if(pxAnimHandler && m_pxClientEffectsHandler && m_pxClientEffectsHandler->GetClientData())
	{
		uMarkerPoint = (pxAnimHandler && pxAnimHandler->IsFirstPerson()) ? m_pxClientEffectsHandler->GetClientData()->m_axSubTypeData[ ST_PRIMARY ].m_uMuzzleMarkerHashFirstPerson : m_pxClientEffectsHandler->GetClientData()->m_axSubTypeData[ ST_PRIMARY ].m_uMuzzleMarkerHashThirdPerson;
	}
		
	if(uMarkerPoint != ASURA_HASH_ID_UNSET && m_uLockingOnPFXBlueprintHash != ASURA_HASH_ID_UNSET)
	{
		Asura_Vector_3 xPosition(Asura_Maths::ZeroVector3);
		Asura_Matrix_3x3 xOrientation(Asura_Maths::IdentityMatrix3x3);
		Asura_Animation* pxAnim = pxAnimHandler->GetAnimation();

		if(pxAnim && pxAnim->GetMarkerLocationFromHash(uMarkerPoint, &xPosition, &xOrientation, true))
		{
			Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_WEAPONEFFECTS, m_uLockingOnPFXBlueprintHash );
			if(pxBlueprint)
			{
				Asura_Hash_ID auPFXHash[ AVP_MAX_LOCKING_ON_PFX ];

				AVP_SurfaceEffect_PFXEnumerator xEnum( auPFXHash );
				u_int uNumPFX = pxBlueprint->EnumerateParameters( "PFX.", 2, xEnum );
				if ( uNumPFX > AVP_MAX_LOCKING_ON_PFX )
				{
					uNumPFX = AVP_MAX_LOCKING_ON_PFX;
				}

				for(u_int u = 0; u<uNumPFX; ++u)
				{
					if(auPFXHash[u]!=ASURA_HASH_ID_UNSET && m_auLockOnPFXEffectID[u] == ASURA_PFX_INVALID_EFFECT_ID)
					{
						m_auLockOnPFXEffectID[u] = Asura_PFX_Entity_Effect::Create(xPosition, xOrientation, auPFXHash[u], 0, ASURA_PFX_EFFECT_CAMERA_FLAGS, GetParentSystem().GetOwnerGuid());
						if(m_auLockOnPFXEffectID[u] != ASURA_PFX_INVALID_EFFECT_ID)
						{
							Asura_AnimationAttachment* pxAttachment = new Asura_AnimationAttachment_PFX(m_auLockOnPFXEffectID[u], true);

							if (pxAttachment)
							{
								pxAttachment->LinkAttachment(pxAnim, uMarkerPoint);
							}
						}
					}
				}
			}
		}
	}
}

void AVP_InventoryWeapon_PlasmaCaster::StopLockingOnPFXEffect()
{
	for(u_int u=0;u<AVP_MAX_LOCKING_ON_PFX;++u)
	{
		if ( m_auLockOnPFXEffectID[u] != ASURA_PFX_INVALID_EFFECT_ID )
		{
			Asura_PFX_Entity_Effect* pxPFXEffect = Asura_PFX_System::GetEntityEffect( m_auLockOnPFXEffectID[u] );

			if ( pxPFXEffect != NULL )
			{
				pxPFXEffect->Stop( false );
			}

			m_auLockOnPFXEffectID[u] = ASURA_PFX_INVALID_EFFECT_ID;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_InventoryWeapon_PlasmaCaster::Use
// Author       : Chris Wallace
// Date         : 16-Jan-2009
////////////////////////////////////////////////////////////////////////////////

void AVP_InventoryWeapon_PlasmaCaster::Use(void)
{
	const u_int uUsageType	= GetCurrentUsageType();

	if(AVP_Predator_LockOn::GetSingleton().IsLockedOn() || !GetParentSystem().IsLocalPlayer()) // Only allow firing when locked on (or not local player)
	{
		PARENT::Use();
	}

	if ( uUsageType == UT_CHARGED_SHOT_PRIMARY )  // PlasmaCaster uses charged shot, even when not using a charge up time
	{
		if(GetUsesLockOn())
		{
			if(GetParentSystem().IsLocalPlayer())
			{
				AVP_Predator_LockOn::GetSingleton().UnLock(); // After firing cancel the lock
			}
		}
	}	
}

void AVP_InventoryWeapon_PlasmaCaster::RemoveEnergyForChargedShot( const u_int uUsageType )
{
	if(AVP_Predator_LockOn::GetSingleton().IsLockedOn() || !GetParentSystem().IsLocalPlayer()) // Only allow firing when locked on (or not local player)
	{
		PARENT::RemoveEnergyForChargedShot( uUsageType );
	}
}

void AVP_InventoryWeapon_PlasmaCaster::ChargeClient( AVP_RangedWeapon_SubType* pxSubType )
{
	// Suppress normal charge behaviour for clients
}

void AVP_InventoryWeapon_PlasmaCaster::StopUsing( const u_int uUsageType )
{
	if(GetParentSystem().IsLocalPlayer() && AVP_Predator_LockOn::GetSingleton().IsLockedOn() && GetCurrentTransitionType() == TT_SHOOT)
	{
		// Force the charge here instead
		PARENT::ChargeClient( static_cast<AVP_RangedWeapon_SubType*>(GetSubTypeFromUsageType( uUsageType )));
	}
	else
	{
		AVP_Predator_LockOn::GetSingleton().UnLock(); // Cancelled, unlock
	}

	PARENT::StopUsing( uUsageType );
}

/// End of file. ///
