
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryWeapon_RangedWeapon.h
// Author : Andy Tate
// Date   : 13-Nov-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_INVENTORYWEAPON_RANGEDWEAPON_H__
#define __AVP_INVENTORYWEAPON_RANGEDWEAPON_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Inventory/Items/AVP_InventoryItem_Weapon.h"
#include "Inventory/Items/Weapons/AVP_RangedWeapon_SubType.h"
#include "Inventory/Groups/AVP_InventoryGroup_Weapons.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Animation;
class Asura_Physics_Object;
class AVP_Actor_DynamicStat;
class AVP_RangedWeapon_ClientEffectsHandler;
class AVP_GenericProjectileData;
template <class T> class Asura_LinkList;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_InventoryWeapon_RangedWeapon
// Author       : Andy Tate
// Date         : 13-Nov-2007
//
// Description  : Class to handle either client or server-side updating of the
//				  pulse-rifle weapon.
////////////////////////////////////////////////////////////////////////////////

class AVP_InventoryWeapon_RangedWeapon : public AVP_InventoryItem_Weapon
{
public:
	// Public enumerated type...
	enum SubType
	{
		ST_PRIMARY,
		ST_SECONDARY,

		NUM_SUB_TYPES
	};

	enum RangedWeaponTransitionType
	{
		TT_ABORT_RESTORE_CHARGE = NUM_ITEM_TRANSITION_TYPES,
		TT_ABORT_DESTROY_CHARGE,
		TT_ABORT_KEEP_CHARGE,
		TT_SHOOT,

		NUM_CLOAK_TRANSITION_TYPES
	};

	enum RangedWeaponUsageType
	{
		UT_FULL_AUTO_PRIMARY			= NUM_WEAPON_USAGE_TYPES,
		UT_FULL_AUTO_SECONDARY,
		UT_CHARGED_SHOT_PRIMARY,
		UT_CHARGED_SHOT_SECONDARY,
		UT_N_BULLET_BURST_PRIMARY,
		UT_N_BULLET_BURST_SECONDARY,

		NUM_RANGED_WEAPON_USAGE_TYPES
	};

	// Public functions...
	AVP_InventoryWeapon_RangedWeapon( AVP_Inventory_Group& xParentGroup, const Asura_Hash_ID uHash, const Asura_Hash_ID uBlueprintHash );
	virtual ~AVP_InventoryWeapon_RangedWeapon();

	// Overridden functions...
	virtual void	StartUsing(const u_int uUsageType);
	virtual void	Use(void);
	virtual void	StopUsing(const u_int uUsageType);

	virtual void	RemoveEnergyForChargedShot( const u_int uUsageType );

	virtual int		GetSubTypeIndexFromUsageType(const u_int uUsageType) const;

	virtual void InitialiseFromBlueprint();

	virtual void PostLoadInitialise();

	// Chunk processing functions...
	virtual	void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual	void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

// 	// Static initialisation functions...
// 	static	void	InitialisePhysicsObject(Asura_Physics_Object* pxPhysicsObject);
	float GetCharge() const { return m_fChargedPower; }

	virtual Asura_Animation*		CreateAnimation();
	virtual void					DestroyAnimation();
	virtual Asura_Animation*		GetAnimation( bool bCreateIfNecessary = false );
	virtual const Asura_Animation*	GetAnimation() const;

	virtual float	GetSpreadParametric() const { return m_fCurrentSpreadParametric; }
	virtual float	GetPrimaryMinSpread() const;
	virtual float	GetPrimaryMaxSpread() const;
	bool			CanReload() const { return m_bCanReload; }

	virtual void	PlayWeaponAnim( Asura_Hash_ID uHash );
	virtual void	ProcessPlayerInput( const AVP_ClientEntity_Actor& xActor );

	virtual void	PlayActivationWeaponAnim();
	virtual void	PlayDeactivationWeaponAnim();

	bool IsDoingDeadMansClick(const AVP_RangedWeapon_SubType* pxSubtype) const;

	float GetBurstSize(void) const { return m_fMaxBurstBulletsToFire; }

	const AVP_RangedWeapon_ClientEffectsHandler* GetClientEffectsHandler(void) const { return m_pxClientEffectsHandler; }

protected:
	// Protected variables...
	AVP_RangedWeapon_ClientEffectsHandler*	m_pxClientEffectsHandler;
	
	bool	m_bFirstShot;

	float	m_fCurrentSpreadParametric;
	float	m_fTargetSpreadParametric;
	float	m_fSpreadCoolDown;
	float	m_fSpreadCoolDownTimer;
	float	m_fBurstBulletsToFire;
	float	m_fMaxBurstBulletsToFire;

	float	m_fChargedPower; // charge stored in weapon
	float	m_fPrimaryShotTimer;
	float	m_fSecondaryShotTimer;

	bool	m_bReloadHandled; // temporary static until anim flags are correctly added

	bool	m_bCanPlayHotMuzzleFlash;	// This flag is reset at beginning of fire, and then it is set to false, once the effect is played. So it will force effect to be played only once per continuous fire.

	// Protected functions...
	void FireProjectile( AVP_ClientEntity_Actor* pxClient, const Asura_Hash_ID uProjectileHash, const Asura_Hash_ID uMuzzleMPHash, const float fMaxRange );

	// Overridden functions...
	virtual void	OnStateChange(const u_int uOldState, const u_int uNewState);
	virtual void	UpdateClientState(AVP_ClientEntity_Actor& xClient);
	virtual	void	UpdateServerState(AVP_ServerEntity_Actor& xServer);

	virtual void ChargeClient( AVP_RangedWeapon_SubType* pxSubType );
	virtual void ChargeServer( AVP_RangedWeapon_SubType* pxSubType );

	virtual AVP_RangedWeapon_ClientEffectsHandler* CreateClientEffectsHandler(Asura_Hash_ID uBlueprintHash, Asura_Hash_ID uOwnerGuid);

	virtual AVP_InventoryItem_SubType* CreateSubType( const int iSubType, const Asura_Hash_ID uBlueprintHashID );

	virtual bool HandleMessage(const Asura_Message* pxMessage);

	virtual void HandleStartFiringEffects( const u_int uUsageType );
	virtual void HandleFireEffects( const u_int uUsageType, const Asura_Vector_3& xFirePosition, const Asura_Matrix_3x3& xFireOrientation, const float fRange, const bool bFirstShot );
	virtual void HandleStopFiringEffects( const u_int uUsageType, const bool bStopWeaponAnims );

	void HandleStartReloadEffects();
	void HandleStopReloadEffects();
	void HandleChargingEffects( const SubType eSubType, const Asura_Vector_3& xFirePosition, const Asura_Matrix_3x3& xFireOrientation );
	void HandleActivationEffects();
	void HandleDeactivationEffects();

	virtual bool HandleHalfShot(int& iFullBulletsToFire) { return false; }

	void DoReload(void);

	bool IsRoomToFire(Asura_LinkList<Asura_Vector_2>& xList, Asura_Vector_2& xVec, float& fMinRange, int& iErrorCount, bool bIgnoreFirstShot = false);
	
	virtual void HandleReload( void );

	virtual float	CalculateFireAmount( const AVP_RangedWeapon_SubType* pxSubType, u_int uUsageType );

	// NB this removes the ammo as well
	virtual u_int	GetBulletsAvailableToFire( float fAmount, AVP_RangedWeapon_SubType* pxSubType, u_int uUsageType );
	
	virtual void	CalculateFiringDataForSubType( const AVP_ClientEntity_Actor& xActor, Asura_Hash_ID uMuzMP, float fMuzVel, float fMaxRange, Asura_Vector_3& xFirePosition, Asura_Vector_3& xFireDirection, Asura_Matrix_3x3& xMuzzleOri, float& fRange, Asura_Guid* pxTargetGuid = NULL );
	virtual void	DoBulletDamage( const AVP_GenericProjectileData* pxProjectileData, Asura_Hash_ID uMuzMP, float fRange, float fMaxRange, const Asura_Vector_3& xShotDirection );
	virtual void	SendCreateProjectileMessage( const AVP_GenericProjectileData* pxProjectileData, const AVP_RangedWeapon_SubType* pxSubType, u_int uUsageType, Asura_Hash_ID uMuzMP, float fRange, float fMaxRange, const Asura_Vector_3& xActorVel, const Asura_Vector_3& xFirePosition, const Asura_Vector_3& xShotDirection, const Asura_Matrix_3x3& xShotOrientation, Asura_Guid* puTargetGuid =NULL );

	virtual bool	IsPlayerIsStillFiring( const AVP_ClientEntity_Actor& xActor, int iSubType ) const;

	virtual void CheckForAutomaticReload(void);
	virtual bool ShouldAllowReload(void);

	void MoveAmmoToCurrentSubtype(void);
	void MoveAmmoToPrimarySubtype(void);

	void UpdateLaser( const AVP_ClientEntity_Actor& xActor );

	bool IsSubtypeEmpty( int iSubType );	// This function also check if we are sharing ammo between subtypes. If so, we will look into primary subtype, rather that into the current one.

	void DoReloadMuzzleSmoke( int iSubType );

	float GetAmmoFiredSoFar() const	{return m_fAmmoFiredSoFar;}

private:
	void GetStaticDataFromBlueprint();

	typedef AVP_InventoryItem_Weapon PARENT;

	bool	m_bCanReload;
	bool	m_bFirstClick;

	mutable float m_fButtonHeldTime; // Used to work out if the button was released quickly or over prolonged time
	float m_fStoredBulletsToFire;
	float m_fAmmoFiredSoFar;

	void SetServerItemToUseDynamicAmmo( AVP_Actor_DynamicStat* pxDynamicAmmo );
	void SetDynamicAmmo( AVP_Actor_DynamicStat* pxDynamicAmmo );

	AVP_InventoryWeapon_RangedWeapon(void);
	AVP_InventoryWeapon_RangedWeapon(const AVP_InventoryWeapon_RangedWeapon& xOther);

	AVP_InventoryWeapon_RangedWeapon&	operator=(const AVP_InventoryWeapon_RangedWeapon& xOther);

	static const u_int uCURRENT_CHUNK_VERSION = 3;
};

#endif   // ifndef __AVP_INVENTORYWEAPON_RANGEDWEAPON_H__


/// End of file. ///
