
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_InventoryWeapon_ScopedRifle.h
// Author : Chris Wallace
// Date   : 05-Dec-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_INVENTORYWEAPON_SCOPEDRIFLE_H__
#define __AVP_INVENTORYWEAPON_SCOPEDRIFLE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Animation/IK/Asura_Animation_IK_Chain_MarkerPoint.h"
#include "Animation/IK/Asura_Animation_IK_Solver.h"

#include "Inventory/Items/Weapons/AVP_InventoryWeapon_RangedWeapon.h"
#include "Inventory/Items/Weapons/AVP_RangedWeapon_SubType.h"
#include "Inventory/Groups/AVP_InventoryGroup_Weapons.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_InventoryWeapon_Taser
// Author       : Chris Wallace
// Date         : 05-Dec-2008
//
// Description  : Class to handle either client or server-side updating of the
//				  scoped rifle weapon.
////////////////////////////////////////////////////////////////////////////////

class AVP_InventoryWeapon_ScopedRifle : public AVP_InventoryWeapon_RangedWeapon
{
public:
	// Public enumerated type...

	enum SmartGunUsageType
	{
		UT_SNIPER_TOGGLE = NUM_RANGED_WEAPON_USAGE_TYPES,

		NUM_SMARTGUN_USAGE_TYPES
	};

	// Public functions...
	AVP_InventoryWeapon_ScopedRifle( AVP_Inventory_Group& xParentGroup, const Asura_Hash_ID uHash, const Asura_Hash_ID uBlueprintHash );
	
	// Overridden functions...
	virtual int		GetSubTypeIndexFromUsageType( const u_int uUsageType ) const;
	virtual void	InitialiseFromBlueprint();

	void			ProcessPlayerInput( const AVP_ClientEntity_Actor& xActor );
	
	bool			IsInSniperMode(bool bFully) const { return (bFully)?m_bFullSniperMode:m_bIsInSniperMode; }

	static Asura_Hash_ID s_uAimingFSFX;
	// 	// Static initialisation functions...

protected:
	// Protected variables...

	// Protected functions...
	virtual void	StartUsing(const u_int uUsageType);

	virtual void	OnStateChange(const u_int uOldState, const u_int uNewState);
	virtual void	UpdateClientState(AVP_ClientEntity_Actor& xClient);
	//virtual	void	UpdateServerState(AVP_ServerEntity_Actor& xServer);


private:
	// Protected variables...
	bool m_bIsInSniperMode; // Are we aiming? This is the actuall toggled state of the weapon
	bool m_bFullSniperMode;	// Similar to above but updates to match only once the transition ends.

	Asura_Hash_ID m_uIntoScopeAnim;
	Asura_Hash_ID m_uOutofScopeAnim;
	
	// Private typedefs...
	typedef AVP_InventoryWeapon_RangedWeapon PARENT;
	
	// Private functions...
	void StartScope(void);
	void EndScope(void);

};   // End of class 'AVP_InventoryWeapon_ScopedRifle'.


#endif   // ifndef __AVP_INVENTORYWEAPON_SCOPEDRIFLE_H__


/// End of file. ///
