
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_RangedWeapon_SubType.cpp
// Author : Will Plano
// Date   : 07/12/2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Inventory/Items/Weapons/AVP_RangedWeapon_SubType.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

// Game independent header-files...
#include "Actor/AVP_Actor_DynamicStat.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Inventory/AVP_Inventory_ItemTypes.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

AVP_RangedWeapon_SubType::AVP_RangedWeapon_SubType( const int iSubType, const Asura_Hash_ID uBlueprintHashID )
: PARENT( iSubType, uBlueprintHashID )
, m_uBPHashProjectile( ASURA_HASH_ID_UNSET )
, m_uMarkerPointMuzzleFirstPerson( ASURA_HASH_ID_UNSET )
, m_uMarkerPointMuzzleThirdPerson( ASURA_HASH_ID_UNSET )
, m_fMaxChargePerShot( 2.0f )
, m_fMinChargePerShot( 1.0f )
, m_fChargePerSecond( 1.0f )
, m_fDamageMultiplierPerCharge( 1.0f )
, m_fCloakedEnergyCost( 0.0f )
, m_fSpreadHeatUp( 1.0f )
, m_fMinSpread( 0.0f )
, m_fMaxSpread( Asura_Maths::DegToRad( 10.0f ) )
, m_fSpreadCoolDownDelayTime( 0.5f )
, m_fMaxRange( 1000.0f )
, m_fRateOfFire( 1.0f )
, m_fAmmoUsePerShot( 1.0f )
, m_uNumProjectilesPerShot( 1 )
, m_fClickCount( 0.0f )
, m_uMaxClicks( 3 )
, m_uClickHash( ASURA_HASH_ID_UNSET )
, m_xWeaponRecoil( Asura_Maths::ZeroEuler3 )
, m_uAIWeaponBehaviourType( 0 )
, m_uBurstFireMinShots( 1 )
, m_uBurstFireMaxShots( 1 )
, m_fBurstFireAIMinGap( 1.0f )
, m_fBurstFireAIMaxGap( 1.0f )
, m_fMinAIFireTime( 1.0f )
, m_fMaxAIFireTime( 1.0f )
, m_fForceReloadWhenAmmoFallTo(0.0f)
, m_uFlags( 0 )
{
	GetStaticDataFromBlueprint();
}

void AVP_RangedWeapon_SubType::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	PARENT::ReadFromChunkStream( xStream );
}

void AVP_RangedWeapon_SubType::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );
}

void AVP_RangedWeapon_SubType::GetStaticDataFromBlueprint()
{
	const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, GetBlueprintHashID() );
	if ( pxBlueprint )
	{
		Asura_Value xValue;
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PROJECTILETYPE, xValue ) )
		{
			m_uBPHashProjectile = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MARKERPOINT_MUZZLE_HUD, xValue ) )
		{
			m_uMarkerPointMuzzleFirstPerson = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MARKERPOINT_MUZZLE_THIRD, xValue ) )
		{
			m_uMarkerPointMuzzleThirdPerson = xValue.GetHashIDValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MAXCHARGEPERSHOT, xValue ) )
		{
			m_fMaxChargePerShot = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MINCHARGEPERSHOT, xValue ) )
		{
			m_fMinChargePerShot = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_CHARGEPERSECOND, xValue ) )
		{
			m_fChargePerSecond = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_DAMAGEMULTIPLIERPERCHARGE, xValue ) )
		{
			m_fDamageMultiplierPerCharge = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_CLOAKED_ENERGY_COST, xValue ) )
		{
			m_fCloakedEnergyCost = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_PERM_REMOVE_CHARGE, xValue ) )
		{
			m_bPermanentlyLoseCharge = xValue.GetBoolValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_IS_CHARGED_SHOT, xValue ) )
		{
			m_bIsChargedShot = xValue.GetBoolValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_IS_FULL_AUTO, xValue ) )
		{
			m_bIsFullAuto = xValue.GetBoolValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SPREAD_HEAT_UP, xValue ) )
		{
			m_fSpreadHeatUp = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MIN_SPREAD, xValue ) )
		{
			m_fMinSpread = Asura_Maths::DegToRad( xValue.GetFloatValue() );
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MAX_SPREAD, xValue ) )
		{
			m_fMaxSpread = Asura_Maths::DegToRad( xValue.GetFloatValue() );
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_SPREAD_COOLDOWNDELAY, xValue ) )
		{
			m_fSpreadCoolDownDelayTime = xValue.GetFloatValue();
		}

		if(pxBlueprint->GetParameterValueByHashID(0x19565259, xValue))	//Asura_Hash_ID("MaxRange"), xValue))
		{
			m_fMaxRange	= xValue.GetFloatValue();
		}

		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_RATEOFFIRE, xValue))	//Asura_Hash_ID("RateOfFire"), xValue))
		{
			m_fRateOfFire	= xValue.GetFloatValue();
		}
	
		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_AUTORELOAD_ENABLE, xValue))	//"ForceReload.Enable"
		{
			m_bEnableForceReload = xValue.GetBoolValue();
		}

		if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_AUTORELOAD_WHENAMMOFALLTO, xValue))	//"ForceReload.WhenAmmoFallTo"
		{
			m_fForceReloadWhenAmmoFallTo = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( 0x8a72b149, xValue ) ) // Asura_Hash_ID( "AI Weapon Behaviour Type" )
		{
			m_uAIWeaponBehaviourType = ( u_int )xValue.GetIntValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_AMMO_PER_SHOT, xValue ) )
		{
			m_fAmmoUsePerShot = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_NUM_PROJ_PER_SHOT, xValue ) )
		{
			Asura_Assert( xValue.GetIntValue() > 0, "Number of projectiles per shot should be > 0" );
			m_uNumProjectilesPerShot = static_cast< u_int >( xValue.GetIntValue() );
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_NUM_MAX_CLICKS, xValue ) )
		{
			m_uMaxClicks = static_cast< u_int >( xValue.GetIntValue() );
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_CLICK_SOUND_HASH, xValue ) )
		{
			m_uClickHash = static_cast< u_int >( xValue.GetHashIDValue() );
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_WEAPONRECOILHORIZONTAL, xValue ) )
		{
			m_xWeaponRecoil.y = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_WEAPONRECOILVERTICAL, xValue ) )
		{
			m_xWeaponRecoil.x = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MIN_BURST_FIRE_SHOTS, xValue ) )
		{
			m_uBurstFireMinShots = (u_int)xValue.GetIntValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MAX_BURST_FIRE_SHOTS, xValue ) )
		{
			m_uBurstFireMaxShots = (u_int)xValue.GetIntValue();
		}
		
		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MIN_AI_BURST_FIRE_GAP, xValue ) )
		{
			m_fBurstFireAIMinGap = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_MAX_AI_BURST_FIRE_GAP, xValue ) )
		{
			m_fBurstFireAIMaxGap = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_AI_MIN_FIRE_TIME, xValue ) )
		{
			m_fMinAIFireTime = xValue.GetFloatValue();
		}

		if ( pxBlueprint->GetParameterValueByHashID( uAVP_BLUEPRINT_PARAM_AI_MAX_FIRE_TIME, xValue ) )
		{
			m_fMaxAIFireTime = xValue.GetFloatValue();
		}

	}
	else
	{
		Asura_Assertf( false, ( "AVP_RangedWeapon_SubType::GetStaticDataFromBlueprint - cannot find blueprint 0x%x to initialise from.", GetBlueprintHashID() ) );
	}
}


bool AVP_RangedWeapon_SubType::ReadyToAutoReloadNow() const
{
	if( GetEnableForceReloadFlag() && GetCurrentQuantity() <= GetForceReloadWhenAmmoFallTo() )
	{
		return true;
	}

	return false;
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
