
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_RangedWeapon_SubType.h
// Author : Will Plano
// Date   : 11/12/2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_RANGEDWEAPON_SUBTYPE_H__
#define __AVP_RANGEDWEAPON_SUBTYPE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Maths.h"

#include "Inventory/Items/AVP_InventoryItem_DynamicSubType.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_PFX;
class AVP_Actor_DynamicStat;
class AVP_RangedWeapon_SubType_ClientData;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_RangedWeapon_SubType
// Author       : Will Plano
// Date         : 07/12/07
//
// Description  : Class uses to store quantity-data for a weapon.
////////////////////////////////////////////////////////////////////////////////

class AVP_RangedWeapon_SubType : public AVP_InventoryItem_DynamicSubType
{
public:
	AVP_RangedWeapon_SubType( const int iSubType, const Asura_Hash_ID uBlueprintHashID );
	virtual ~AVP_RangedWeapon_SubType() {}

	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	Asura_Hash_ID	GetProjectile() const					{ return m_uBPHashProjectile; }
	Asura_Hash_ID	GetMuzzleMarkerPointFirstPerson() const	{ return m_uMarkerPointMuzzleFirstPerson; }
	Asura_Hash_ID	GetMuzzleMarkerPointThirdPerson() const	{ return m_uMarkerPointMuzzleThirdPerson; }
	float			GetMaxChargePerShot() const				{ return m_fMaxChargePerShot; }
	float			GetMinChargePerShot() const				{ return m_fMinChargePerShot; }
	float			GetChargePerSecond() const				{ return m_fChargePerSecond; }
	float			GetDamageMultiplierPerCharge() const	{ return m_fDamageMultiplierPerCharge; }
	float			GetCloakedEnergyCost() const			{ return m_fCloakedEnergyCost; }

	float			GetSpreadHeatUp() const					{ return m_fSpreadHeatUp; }
	float			GetMinSpread() const					{ return m_fMinSpread; }
	float			GetMaxSpread() const					{ return m_fMaxSpread; }
	float			GetSpreadCoolDownDelayTime() const		{ return m_fSpreadCoolDownDelayTime; }
	float			GetMaxRange() const						{ return m_fMaxRange; }
	float			GetRateOfFire() const					{ return m_fRateOfFire; }

	u_int			GetBurstFireMinShots() const			{ return m_uBurstFireMinShots; }
	u_int			GetBurstFireMaxShots() const			{ return m_uBurstFireMaxShots; }
	float			GetBurstFireAIMinGap() const			{ return m_fBurstFireAIMinGap; }
	float			GetBurstFireAIMaxGap() const			{ return m_fBurstFireAIMaxGap; }

	float			GetMinAIFireTime() const				{ return m_fMinAIFireTime; }
	float			GetMaxAIFireTime() const				{ return m_fMaxAIFireTime; }

	float			GetForceReloadWhenAmmoFallTo() const	{ return m_fForceReloadWhenAmmoFallTo; }

	u_int			GetNumProjectilesPerShot() const		{ return m_uNumProjectilesPerShot; }

	bool			IsFullAuto() const						{ return m_bIsFullAuto; }
	bool			IsChargedShot() const					{ return m_bIsChargedShot; }
	bool			IsPermanentlyLoseCharge() const			{ return m_bPermanentlyLoseCharge; }
	bool			GetEnableForceReloadFlag() const		{ return m_bEnableForceReload; }
	float			GetAmmoUsePerShot()	const				{ return m_fAmmoUsePerShot; }

	u_int			GetAIWeaponBehaviourType() const		{ return m_uAIWeaponBehaviourType; }

	Asura_Hash_ID	GetClickHash(void) const				{ return m_uClickHash; }
	u_int			GetMaxClicks(void) const				{ return m_uMaxClicks; }
	float			GetClickCount(void)	const				{ return m_fClickCount; }
	void			SetClickCount(float fClicks)			{ m_fClickCount = fClicks; }
	
	Asura_Euler_3	GetWeaponRecoil() const					{ return m_xWeaponRecoil; }

	bool			ReadyToAutoReloadNow() const;


protected:
	Asura_Hash_ID			m_uBPHashProjectile;

	Asura_Hash_ID			m_uMarkerPointMuzzleFirstPerson;
	Asura_Hash_ID			m_uMarkerPointMuzzleThirdPerson;
	
	float					m_fMaxChargePerShot;
	float					m_fMinChargePerShot;
	float					m_fChargePerSecond;
	float					m_fDamageMultiplierPerCharge;
	float					m_fCloakedEnergyCost;
	float					m_fSpreadHeatUp;
	float					m_fMinSpread;
	float					m_fMaxSpread;
	float					m_fSpreadCoolDownDelayTime;
	float					m_fMaxRange;
	float					m_fRateOfFire;
	float					m_fAmmoUsePerShot;
	u_int					m_uNumProjectilesPerShot;
	float					m_fClickCount;
	u_int					m_uMaxClicks;
	Asura_Hash_ID			m_uClickHash;
	Asura_Euler_3			m_xWeaponRecoil;

	u_int					m_uAIWeaponBehaviourType;

	u_int					m_uBurstFireMinShots;
	u_int					m_uBurstFireMaxShots;
	float					m_fBurstFireAIMinGap;
	float					m_fBurstFireAIMaxGap;
	float					m_fMinAIFireTime;
	float					m_fMaxAIFireTime;
	float					m_fForceReloadWhenAmmoFallTo;
	union
	{
		u_int	m_uFlags;
		struct
		{
			u_int	m_bIsFullAuto				: 1,
					m_bIsChargedShot			: 1,
					m_bPermanentlyLoseCharge	: 1,
					m_bEnableForceReload		: 1,
												: 0;
		};
	};

private:
	void GetStaticDataFromBlueprint();

	typedef AVP_InventoryItem_DynamicSubType PARENT;

	AVP_RangedWeapon_SubType(void);
	AVP_RangedWeapon_SubType(const AVP_RangedWeapon_SubType& xOther);
	AVP_RangedWeapon_SubType&	operator=(const AVP_RangedWeapon_SubType& xOther);

	static const u_int uCURRENT_CHUNK_VERSION = 0;
};

#endif   // ifndef __AVP_RANGEDWEAPON_SUBTYPE_H__

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
