
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Alien_MovesBranch.cpp
// Author : Anthony Gregson
// Date   : 06/12/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesBranch.h"

// Asura independent header-files...

// Game independent header-files...
#include "Actor/AVP_Actor_PhysicsObjectHandler.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_ClientActor_Alien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/AVP_Messages_Alien.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Move_Alien_MountBranch::AVP_Move_Alien_MountBranch
// Author       : Anthony Gregson
// Date         : 06/12/07
////////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_MountBranch::AVP_Move_Alien_MountBranch(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
{
	// Initialise the move flags...
	m_bDisableMovement	= true;
	m_eAimDirectionUsage = AVP_ClientEntity_Actor::AIM_DIRECTION_LIMIT;
}

/// End of function 'AVP_Move_Alien_MountBranch::AVP_Move_Alien_MountBranch'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Move_Alien_MountBranch::Start
// Author       : Anthony Gregson
// Date         : 06/12/07
////////////////////////////////////////////////////////////////////////////////
DEBUGVAR float dbg_fMountBranchMoveTime = 0.2f;
DEBUGVAR float dbg_fTransitionBranchMoveTime = 0.4f;
DEBUGVAR float dbg_fBranchTransitionMaxCosNormalAngle = 0.7f; // A bit more than 45 degrees
DEBUGVAR float dbg_fMountBranchAimLimitAngle = 0.5f;
DEBUGVAR float dbg_fTransitionBranchAimLimitAngle = 0.95f;

bool AVP_Move_Alien_MountBranch::Start(const Asura_Msg_Move* pxStartMsg)
{
	// Have we been told to start via a message?
	if(pxStartMsg == NULL)
	{
		Asura_Assert(false, "Alien mount branch move can only be triggered by message.");
		return(false);
	}
	else
	{
		AVP_ClientActor_Alien& xAlienOwner = static_cast<AVP_ClientActor_Alien&>(GetAVPParentSystem().GetActorOwner());

		const AVP_Message_Alien_Move_MountBranch_Start* pxMessage = static_cast<const AVP_Message_Alien_Move_MountBranch_Start*>(pxStartMsg);

		Asura_Vector_3 xTargetPosition;
		Asura_Quat xTargetOrientation;

		// Calculate our target position...
		pxMessage->GetDestinationPosition(xTargetPosition);
		pxMessage->GetTargetOrientation(xTargetOrientation);

		float fDuration = dbg_fMountBranchMoveTime;
		m_fAimDirectionLimitAngle = dbg_fMountBranchAimLimitAngle;

		AVP_Actor_PhysicsObjectHandler* pxPhysicsHandler = xAlienOwner.GetPhysicsObjectHandler();

		if (pxPhysicsHandler)
		{
			Asura_Matrix_3x3 xTargetMat;
			xTargetOrientation.ToMatrix(xTargetMat);

			Asura_Vector_3 xTargetNormal(-xTargetMat.e12, -xTargetMat.e22, -xTargetMat.e32);

			Asura_Vector_3 xInitialNormal;
			pxPhysicsHandler->GetCurrentSurfaceNormal(xInitialNormal);

			const float fDP = xInitialNormal * xTargetNormal;
			if (fDP < dbg_fBranchTransitionMaxCosNormalAngle)
			{
				fDuration = dbg_fTransitionBranchMoveTime;
				m_fAimDirectionLimitAngle = dbg_fTransitionBranchAimLimitAngle;
			}

			pxPhysicsHandler->SetForcedTargetSurfaceNormal(xTargetNormal);
		}

		// TODO - vary the duration based on distance to move
		m_xPositionInterpolationData.SetDuration(fDuration);
		m_xPositionInterpolationData.SetFinal(xTargetPosition);

		m_xOrientationInterpolationData.SetDuration(fDuration);
		m_xOrientationInterpolationData.SetFinal(xTargetOrientation);

		InitialisePositionInterpolation(m_xPositionInterpolationData);
		InitialiseOrientationInterpolation(m_xOrientationInterpolationData);

		// Finally, call the base-function...
		return PARENT::Start(pxStartMsg);
	}
}

/// End of function 'AVP_Move_Alien_MountBranch::Start'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Move_Alien_DismountBranch::AVP_Move_Alien_DismountBranch
// Author       : Anthony Gregson
// Date         : 19/11/07
////////////////////////////////////////////////////////////////////////////////

AVP_Move_Alien_DismountBranch::AVP_Move_Alien_DismountBranch(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
, m_xMovementDirection(Asura_Maths::ZeroVector3)
, m_fTimeInMove(0.0f)
{
	m_eAimDirectionUsage = AVP_ClientEntity_Actor::AIM_DIRECTION_LOCK;
}

bool AVP_Move_Alien_DismountBranch::Start(const Asura_Msg_Move* pxStartMsg)
{
	// Have we been told to start via a message?
	if(pxStartMsg == NULL)
	{
		Asura_Assert(false, "Alien dismount branch move can only be triggered by message.");
		return false;
	}
	else
	{
		const AVP_Message_Alien_Move_DismountBranch_Start* pxMessage = static_cast<const AVP_Message_Alien_Move_DismountBranch_Start*>(pxStartMsg);

		// Calculate our target position...
		pxMessage->GetDirection(m_xMovementDirection);

		m_fTimeInMove = 0.0f;

		// Finally, call the base-function...
		return PARENT::Start(pxStartMsg);
	}
}

void AVP_Move_Alien_DismountBranch::UpdateStates(const float fTimeStep)
{
	m_fTimeInMove += Asura_Timers::GetGameTimeSlice();

	PARENT::UpdateStates(fTimeStep);
}

DEBUGVAR float dbg_fDismountBranchMoveTime = 0.5f;

bool AVP_Move_Alien_DismountBranch::HasFinished() const
{
	return m_fTimeInMove >= dbg_fDismountBranchMoveTime;
}


/// End of file. ///
