////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Alien_Move_Interaction_DestroyBattery
// Author       : Ali Jakamy
// Date         : 03/01/08
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Moves/Vertebrates/Alien/AVP_Alien_MovesInteraction.h"

// AVP
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/BatteryReceptacle/AVP_ClientEntity_InteractBatteryReceptacle.h"

////////////////////////////////////////////////////////////////////////////////
// CLASS FUNCTIONS
// AVP_Alien_Move_Interaction_DestroyBattery
////////////////////////////////////////////////////////////////////////////////

AVP_Alien_Move_Interaction_DestroyBattery::AVP_Alien_Move_Interaction_DestroyBattery( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;
}

bool AVP_Alien_Move_Interaction_DestroyBattery::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Have we been told to start via a message?
	if ( pxStartMsg == NULL )
	{
		Asura_Assert( false, "Interaction move can only be triggered by message." );
		return false;
	}
	else
	{
		const AVP_Message_InteractionMoveStart*	pxMessage = static_cast< const AVP_Message_InteractionMoveStart* >( pxStartMsg );
		AVP_ClientEntity_InteractBatteryReceptacle* pxInteractObject = AVP_ClientEntity_InteractBatteryReceptacle::FindEntity( pxMessage->GetInteractionObject() );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", pxMessage->GetInteractionObject() ) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
			Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
			pxInteractObject->GetAlienDestroyBatteryAnimHashes( uFPAnimHash, uTPAnimHash );
			SetAnimHashes( uFPAnimHash, uTPAnimHash );

			const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );
			if ( !pxActor )
			{
				return false;
			}
			else
			{
				const float	fDuration = 0.400f;
				Asura_Vector_3 xTargetPosition;
				Asura_Matrix_3x3 xTargetOrientation;

				if ( uFPAnimHash != ASURA_HASH_ID_UNSET || uTPAnimHash != ASURA_HASH_ID_UNSET )
				{
					AVP_ClientEntity_InteractObject::AnimData xAnimData = pxInteractObject->GetInteractionData( *pxActor, pxMessage->GetOn() );

					if ( !pxInteractObject->GetMarkerLocationFromHash( xAnimData.m_uActorPositionMarkerPoint, &xTargetPosition, &xTargetOrientation ) )
					{
						xTargetPosition = pxActor->GetCurrentPosition();
						xTargetOrientation = pxActor->GetCurrentOrientation();
					}

					// For the time being we'll just move and rotate to our desired position...
					m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET );
					m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET );

					m_xOrientationInterpolationData.SetDuration( fDuration );
					m_xOrientationInterpolationData.SetFinal( xTargetOrientation );
					m_xPositionInterpolationData.SetDuration( fDuration );
					m_xPositionInterpolationData.SetFinal( xTargetPosition );

					InitialiseOrientationInterpolation( m_xOrientationInterpolationData );
					InitialisePositionInterpolation( m_xPositionInterpolationData );
				}

				// Finally, call the base-function...
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}
