///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_FaceHugger_FP_MovesMelee.h
// Author : Anthony Gregson
// Date   : 22/02/08
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_FACEHUGGER_MOVES_MELEE_H__
#define __AVP_FACEHUGGER_MOVES_MELEE_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Moves/Asura_MoveData_Animation.h"
#include "Moves/Vertebrates/AVP_MovesMelee.h"

// Animation Tree
// JumpAttack Start
//  ||  |  |   |    \_____JumpAttack_Return_Hit (should not be used)
//  ||  |  |   \_________JumpAttack_Return_Countered (should not be used)
//  ||  |  \_____________JumpAttack_Return_Miss
//  ||  \________________Human_Death (not implemented yet)
//  |\_________ PredatorGrapple_Start
//  |              |    \______PredatorGrapple_Kill
//  |              \___________PredatorGrapple_Countered
//  \___________ HumanGrapple_Start
//                 |    \_______HumanGrapple_Kill
//                 \________HumanGrapple_Countered (NOT USED)

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_FP_Move_Melee_JumpAttack_Start
////////////////////////////////////////////////////////////////////////////////

class AVP_FaceHugger_Move_Melee_JumpAttack_Start : public AVP_Move_Melee_GenericAttack_Start
{
public:
	AVP_FaceHugger_Move_Melee_JumpAttack_Start(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);

	static Asura_Hash_ID GetStaticHash() { return 0x88e558ce; } //"AVP_FaceHugger_Move_Melee_JumpAttack_Start"
	
private:

	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation m_xPositionInterpolationData;

	typedef AVP_Move_Melee_GenericAttack_Start PARENT;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Hit
////////////////////////////////////////////////////////////////////////////////

class AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Hit : public AVP_Move_Melee_GenericAttack_Return_Hit
{
public:
	AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Hit(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	virtual bool ShouldStart() const;

	static Asura_Hash_ID GetStaticHash() { return 0xbd07c725; } //"AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Hit"

private:
	typedef AVP_Move_Melee_GenericAttack_Return_Hit PARENT;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Countered
////////////////////////////////////////////////////////////////////////////////

class AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Countered : public AVP_Move_Melee_GenericAttack_Return_Countered
{
public:
	AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Countered(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);
	static Asura_Hash_ID GetStaticHash() { return 0xba3b91ad; } //"AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Countered" 

private:
	typedef AVP_Move_Melee_GenericAttack_Return_Countered PARENT;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Miss
////////////////////////////////////////////////////////////////////////////////

class AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Miss : public AVP_Move_Melee_GenericAttack_Return_Miss
{
public:
	AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Miss(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);

	static Asura_Hash_ID GetStaticHash() { return 0xe3f363aa; } //"AVP_FaceHugger_FP_Move_Melee_JumpAttack_Return_Miss"

private:
	typedef AVP_Move_Melee_GenericAttack_Return_Miss PARENT;
};


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Start
////////////////////////////////////////////////////////////////////////////////
class AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Start : public AVP_Move_Melee_GenericPuppeteer_Start
{
public:
	AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Start(Asura_Movement_System& xParentSystem);
	static Asura_Hash_ID GetStaticHash() { return 0xce854f13; } //"AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Start"

	virtual bool ShouldStart() const; 

private:
	typedef AVP_Move_Melee_GenericPuppeteer_Start PARENT;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Kill
////////////////////////////////////////////////////////////////////////////////
class AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Kill : public AVP_Move_Melee_GenericPuppeteer_Success
{
public:
	AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Kill(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xbc53168d; } //"AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Kill"

private:
	typedef AVP_Move_Melee_GenericPuppeteer_Success PARENT;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Countered
////////////////////////////////////////////////////////////////////////////////
class AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Countered : public AVP_Move_Melee_GenericPuppeteer_Countered
{
public:
	AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Countered(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x2dfd9dcc;  } //"AVP_FaceHugger_FP_Move_Melee_PredatorGrapple_Countered"

private:
	typedef AVP_Move_Melee_GenericPuppeteer_Countered PARENT;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Start
////////////////////////////////////////////////////////////////////////////////
class AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Start : public AVP_Move_Melee_GenericPuppeteer_Start
{
public:
	AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Start(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x36f2d428; } //"AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Start"

	virtual bool ShouldStart() const; 

private:
	typedef AVP_Move_Melee_GenericPuppeteer_Start PARENT;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Kill
////////////////////////////////////////////////////////////////////////////////
class AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Kill : public AVP_Move_Melee_GenericPuppeteer_Success
{
public:
	AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Kill(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x12461ad8; } //"AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Kill"

private:
	typedef AVP_Move_Melee_GenericPuppeteer_Success PARENT;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Countered
////////////////////////////////////////////////////////////////////////////////
class AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Countered : public AVP_Move_Melee_GenericPuppeteer_Countered
{
public:
	AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Countered(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xeb489061; } //"AVP_FaceHugger_FP_Move_Melee_MarineGrapple_Countered"

private:
	typedef AVP_Move_Melee_GenericPuppeteer_Countered PARENT;
};

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_TP_Move_Melee_MarineGrapple
////////////////////////////////////////////////////////////////////////////////

class AVP_FaceHugger_TP_Move_Melee_MarineGrapple : public AVP_Move_Melee_GenericPuppeteer_Start
{
public:
	AVP_FaceHugger_TP_Move_Melee_MarineGrapple(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x0b18cf13; } // AVP_FaceHugger_TP_Move_Melee_MarineGrapple
	
	virtual bool ShouldStart() const; 

private:
	typedef AVP_Move_Melee_GenericPuppeteer_Start PARENT;
};   

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_FaceHugger_TP_Move_Melee_MarineDeath
////////////////////////////////////////////////////////////////////////////////

class AVP_FaceHugger_TP_Move_Melee_MarineDeath : public AVP_Move
{
public:
	AVP_FaceHugger_TP_Move_Melee_MarineDeath(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xb5803138; } // AVP_FaceHugger_TP_Move_Melee_MarineDeath

	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

private:
	typedef AVP_Move PARENT;
}; 



////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_TP_Move_Melee_PredatorGrapple
////////////////////////////////////////////////////////////////////////////////

class AVP_FaceHugger_TP_Move_Melee_PredatorGrapple : public AVP_Move_Melee_GenericPuppeteer_Start
{
public:
	AVP_FaceHugger_TP_Move_Melee_PredatorGrapple(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x15202abe; } // AVP_FaceHugger_TP_Move_Melee_PredatorGrapple

	virtual bool ShouldStart() const; 

private:
	typedef AVP_Move_Melee_GenericPuppeteer_Start PARENT;
};


////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_FaceHugger_TP_Move_Melee_MarineDeathHug
////////////////////////////////////////////////////////////////////////////////
class AVP_FaceHugger_TP_Move_Melee_MarineDeathHug : public AVP_Move
{
public:
	AVP_FaceHugger_TP_Move_Melee_MarineDeathHug(Asura_Movement_System& xParentSystem, Asura_Hash_ID uFirstPersonBlueprintHash, Asura_Hash_ID uThirdPersonBlueprintHash);

	static Asura_Hash_ID GetStaticHash() { return 0x70e138c2; } //"AVP_FaceHugger_TP_Move_Melee_MarineDeathHug"

private:
	typedef AVP_Move PARENT;
};

#endif   // ifndef __AVP_FACEHUGGER_MOVES_MELEE_H__
