//----------------------------------//----------------------------------//----------------------------------//----------------------------------
//
// File   : AVP_Human_MovesGamescene.cpp
// Author : Gareth Thomas
// Date   : 01 / 07 / 2008
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//----------------------------------//----------------------------------//----------------------------------//----------------------------------

//----------------------------------//----------------------------------//----------------------------------//----------------------------------
// Include files...
//----------------------------------//----------------------------------//----------------------------------//----------------------------------
// Core
#include "Core/Asura.h"

// This Files Header
#include "Moves/Vertebrates/Human/AVP_Human_MovesGamescene.h"

// Asura
#include "Blueprint/Asura_Blueprint_System.h"

// AVP
#include "Actor/Vertebrates/AVP_Vertebrate_AnimationHandler.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ClientActor_Human.h"
#include "Moves/AVP_Moves.h"
#include "Moves/AVP_MoveData_Animation.h"

//----------------------------------//----------------------------------//----------------------------------//----------------------------------
bool AVP_Move_Human_MountGamescene::Start(const Asura_Msg_Move* pxStartMsg)
{
	if ( pxStartMsg == NULL )
	{
		Asura_Assert( false, "Human enter cover move can only be triggered by message." );
		return(false);
	}

	const AVP_ClientEntity_Actor& xOwner =	GetAVPParentSystem().GetActorOwner();

	const AVP_Message_Human_MountGamescene* pxMessage = static_cast< const AVP_Message_Human_MountGamescene* >( pxStartMsg );

	Asura_Vector_3 xTargetPosition;
	pxMessage->GetDestinationPosition( xTargetPosition );

	Asura_Matrix_3x3 xTargetOrientation;
	pxMessage->GetDestinationOrientation( xTargetOrientation );

	const Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( xOwner.GetAnimStateBlueprintType(), pxMessage->GetAnimBlendTemplateBlueprintHash() );
	Asura_Assertf( pxBP, ( "Unable to get blend template with hash ID 0x%08X specified in move start message.", pxMessage->GetAnimBlendTemplateBlueprintHash() ) );
	if ( pxBP )
	{
		SetAnimBlendTemplateBlueprint( *pxBP );
		if ( m_pxAVPAnimationData && m_bUsingBlendTemplate )
		{
			Asura_AVector3 xMoveDir;
			xOwner.GetMovementDirection( xMoveDir );
			Asura_Vector_3 xMoveDir_3( xMoveDir );
			xMoveDir_3 = xOwner.GetCurrentOrientation().ApplyTranspose( xMoveDir_3 );
			static_cast< AVP_MoveData_BlendedAnimation* >( m_pxAVPAnimationData )->SetBlendAngle( Asura_Maths::ArcTangent( xMoveDir_3.GetX(), xMoveDir_3.GetZ() ) );
		}
	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	// For the time being we'll just move and rotate to our desired position...
	if ( bHasAnimation )
	{
		m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET );
		m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::IGNORE_MOVEMENT_OFFSET );
		m_pxAVPAnimationData->SetAnimPriority( 1000 );
	}

	m_xPositionInterpolationData.SetDuration( fDuration );
	m_xPositionInterpolationData.SetFinal( xTargetPosition );

	m_xOrientationInterpolationData.SetDuration( fDuration );
	m_xOrientationInterpolationData.SetFinal( xTargetOrientation );

	InitialisePositionInterpolation( m_xPositionInterpolationData );
	InitialiseOrientationInterpolation( m_xOrientationInterpolationData );

	// Finally, call the base-function...
	return(PARENT::Start(pxStartMsg));
}

// Asura_Move* AVP_Move_Human_MountGamescene::Update()
// {
// 	const AVP_ClientEntity_Actor& xOwner =	GetAVPParentSystem().GetActorOwner();
// 	if ( m_pxAVPAnimationData && m_bUsingBlendTemplate )
// 	{
// 		Asura_AVector3 xVelocity;
// 		GetParentSystem().GetCurrentMoveVelocity( xVelocity );
// 		xVelocity = Asura_AMatrix3x3( xOwner.GetCurrentOrientation() ).ApplyTranspose( xVelocity );
// 		static_cast< AVP_MoveData_BlendedAnimation* >( m_pxAVPAnimationData )->SetBlendAngle( Asura_Maths::ArcTangent( xVelocity.GetX(), xVelocity.GetZ() ) );
// 	}
// 
// 	return PARENT::Update();
// }
