////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_Human_Move_Interaction_PlugBattery
// Author       : Ali Jakamy
// Date         : 25/11/08
////////////////////////////////////////////////////////////////////////////////

#include "Moves/AVP_Moves.h"

class AVP_Human_Move_Interaction_PlugBattery : public AVP_Move
{
public:
	AVP_Human_Move_Interaction_PlugBattery(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x5B41015A; } // Asura_GetHashID("AVP_Human_Move_Interaction_PlugBattery")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;

private:
	typedef AVP_Move PARENT;
};   

class AVP_Human_Move_Interaction_LetGoBattery : public AVP_Move
{
public:
	AVP_Human_Move_Interaction_LetGoBattery(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x427E4F2F; } // Asura_GetHashID("AVP_Human_Move_Interaction_LetGoBattery")); }
	void       SetInteractionObjectGuid( Asura_Guid uGuid ) { m_uInteractionObject = uGuid; }
	Asura_Guid GetInteractionObjectGuid() const { return m_uInteractionObject; }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;
private:

	Asura_Guid m_uInteractionObject;    // Save the interaction GUID, in case it the moves starts without message

	typedef AVP_Move PARENT;
};   

class AVP_Human_Move_Interaction_UnPlugBattery : public AVP_Move
{
public:
	AVP_Human_Move_Interaction_UnPlugBattery(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xBC834E61; } // Asura_GetHashID("AVP_Human_Move_Interaction_UnPlugBattery")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual bool ShouldStart() const;

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;
private:
	typedef AVP_Move PARENT;
}; 

class AVP_Human_Move_Interaction_GrabBattery : public AVP_Move
{
public:
	AVP_Human_Move_Interaction_GrabBattery(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x3C30D2DC; } // Asura_GetHashID("AVP_Human_Move_Interaction_GrabBattery")); }
	void       SetInteractionObjectGuid( Asura_Guid uGuid ) { m_uInteractionObject = uGuid; }
	Asura_Guid GetInteractionObjectGuid() const { return m_uInteractionObject; }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual Asura_Move* Update();

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;
private:

	Asura_Guid m_uInteractionObject;    // Save the interaction GUID, in case it the moves starts without message

	typedef AVP_Move PARENT;
}; 

class AVP_Human_Move_Interaction_PlugHackingTool : public AVP_Move
{
public:
	AVP_Human_Move_Interaction_PlugHackingTool(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xAFB14F06; } // Asura_GetHashID("AVP_Human_Move_Interaction_PlugHackingTool")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual bool ShouldStart() const;

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;

private:
	typedef AVP_Move PARENT;
};   

class AVP_Human_Move_Interaction_UnPlugHackingTool : public AVP_Move
{
public:
	AVP_Human_Move_Interaction_UnPlugHackingTool(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xDD67C08D; } // Asura_GetHashID("AVP_Human_Move_Interaction_UnPlugHackingTool")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);
	virtual bool ShouldStart() const;

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;
private:
	typedef AVP_Move PARENT;
};

class AVP_Human_Move_Interaction_LetGoHackingTool : public AVP_Move
{
public:
	AVP_Human_Move_Interaction_LetGoHackingTool(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0xF939AA5B; } // Asura_GetHashID("AVP_Human_Move_Interaction_LetGoHackingTool")); }
	void       SetInteractionObjectGuid( Asura_Guid uGuid ) { m_uInteractionObject = uGuid; }
	Asura_Guid GetInteractionObjectGuid() const { return m_uInteractionObject; }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;
private:

	Asura_Guid m_uInteractionObject;    // Save the interaction GUID, in case it the moves starts without message
	
	typedef AVP_Move PARENT;
};

class AVP_Human_Move_Interaction_GrabHackingTool : public AVP_Move
{
public:
	AVP_Human_Move_Interaction_GrabHackingTool(Asura_Movement_System& xParentSystem);

	static Asura_Hash_ID GetStaticHash() { return 0x09788F88; } // Asura_GetHashID("AVP_Human_Move_Interaction_GrabHackingTool")); }
	void       SetInteractionObjectGuid( Asura_Guid uGuid ) { m_uInteractionObject = uGuid; }
	Asura_Guid GetInteractionObjectGuid() const { return m_uInteractionObject; }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

	Asura_MoveData_OrientationInterpolation m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation    m_xPositionInterpolationData;
private:

	Asura_Guid m_uInteractionObject;    // Save the interaction GUID, in case it the moves starts without message

	typedef AVP_Move PARENT;
};