///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Human_MovesMantle.h
// Author : Anthony Gregson
// Date   : 31/07/08
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // #ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_HUMAN_MOVESMANTLE_H__
#define __AVP_HUMAN_MOVESMANTLE_H__

// Includes
#include "Entity/PhysicalObject/Moves/Asura_MoveData_OrientationInterpolation.h"
#include "Entity/PhysicalObject/Moves/Asura_MoveData_PositionInterpolation.h"

#include "Moves/AVP_Movement_System.h"
#include "Moves/AVP_Moves.h"


// Moves
class AVP_Move_Human_Mantle : public AVP_Move
{
public:
	AVP_Move_Human_Mantle(Asura_Movement_System& xParentSystem);

	virtual const char* GetName() const		{ return "AVP_Move_Human_Mantle"; }
	static Asura_Hash_ID GetStaticHash()	{ return 0x09B171EF; }	// Asura_GetHashID("AVP_Move_Human_Mantle")); }

protected:
	virtual bool Start(const Asura_Msg_Move* pxStartMsg);

	Asura_MoveData_OrientationInterpolation	m_xOrientationInterpolationData;
	Asura_MoveData_PositionInterpolation	m_xPositionInterpolationData;

private:
	typedef AVP_Move PARENT;
};


// Associated messages
class AVP_Msg_Move_StartMantle : public AVP_Msg_Move
{
public:
 	const Asura_Vector_3& GetTargetPosition() const					{ return m_xTargetPosition; }
 	void SetTargetPosition(const Asura_Vector_3& xTargetPosition)	{ m_xTargetPosition = xTargetPosition; }

	const Asura_Quat& GetTargetOrientation() const					{ return m_xTargetOrientation; }
 	void SetTargetOrientation(const Asura_Quat& xTargetOrientation)	{ m_xTargetOrientation = xTargetOrientation; }

	Asura_Hash_ID GetFirstPersonAnim() const { return m_uFirstPersonAnim; }
	Asura_Hash_ID GetThirdPersonAnim() const { return m_uThirdPersonAnim; }
	void SetAnimationHashes(Asura_Hash_ID uFirstPersonAnim, Asura_Hash_ID uThirdPersonAnim) { m_uFirstPersonAnim = uFirstPersonAnim; m_uThirdPersonAnim = uThirdPersonAnim; }

	bool GetRequiresJump() const { return (m_usProjectFlags & AVP_MSG_MOVE_STARTMANTLE_FLAG_REQUIRESJUMP) != 0; }
	void SetRequiresJump(bool bRequiresJump);

private:
	Asura_Vector_3 m_xTargetPosition;
	Asura_Quat m_xTargetOrientation;
	Asura_Hash_ID m_uFirstPersonAnim;
	Asura_Hash_ID m_uThirdPersonAnim;

	enum
	{
		AVP_MSG_MOVE_STARTMANTLE_FLAG_REQUIRESJUMP	= 0x1000,
	};
};

#endif // __AVP_HUMAN_MOVESMANTLE_H__
