
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Predalien_MovesJump.cpp
// Author : Anthony Gregson
// Date   : 08/10/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Moves/Vertebrates/Predalien/AVP_Predalien_MovesJump.h"

// Asura independent header-files...

// Game independent header-files...
#include "Actor/AVP_Actor_InputHandler.h"
#include "Actor/AVP_Actor_PhysicsObjectHandler.h"
#include "Actor/Vertebrates/AVP_Vertebrate_AnimationHandler.h"
#include "Animation/AVP_AnimFlags.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_ClientActor_Predalien.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predalien/AVP_Messages_Predalien.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Move_Predalien_PreJumpTarget
////////////////////////////////////////////////////////////////////////////////

AVP_Move_Predalien_PreJumpTarget::AVP_Move_Predalien_PreJumpTarget(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
{
	// Initialise the move flags...
	m_bDisableMovement	= true;
	m_eAimDirectionUsage = AVP_ClientEntity_Actor::AIM_DIRECTION_SET_DIRECTLY;
//	m_fAimDirectionLimitAngle = Asura_Maths::DegToRad( 60.0f );

	m_bPreventsFlinch = true;
	m_bClearQueuedMeleeRequests = true;
	m_bClearQueuedCounterRequest = true;
	m_bSupportEndAnimFlag = true;
	m_bPreventsFocus = true;

	AddNextMove(AVP_Move_Predalien_JumpTarget::GetStaticHash());


}


bool AVP_Move_Predalien_PreJumpTarget::Start(const Asura_Msg_Move* pxStartMsg)
{
	// Have we been told to start via a message?
	if(pxStartMsg == NULL)
	{
		Asura_Assert(false, "Predalien jump-to-target move can only be triggered by message.");
		return(false);
	}
	else
	{

		AVP_Move_Predalien_JumpTarget* pxNextMove = static_cast<AVP_Move_Predalien_JumpTarget*>(GetParentSystem().GetMove(AVP_Move_Predalien_JumpTarget::GetStaticHash()));
		pxNextMove->SetFromMessage(pxStartMsg);

		AVP_ClientActor_Predalien& xPredalienOwner = static_cast<AVP_ClientActor_Predalien&>(GetAVPParentSystem().GetActorOwner());

		const AVP_Message_Predalien_Move_WalkAndJumpStart* pxMessage = static_cast<const AVP_Message_Predalien_Move_WalkAndJumpStart*>(pxStartMsg);

		Asura_Vector_3 xTargetPosition(pxMessage->GetStartPosition());

		// Update the animation to use

		Asura_Vector_3 xFlatOffset = xTargetPosition - xPredalienOwner.GetCurrentPosition();
		xFlatOffset.y = 0.0f;
		const float fWalkDuration = xFlatOffset.Magnitude() / pxMessage->GetMoveSpeed() ;

		m_xPositionInterpolationData.SetDuration(fWalkDuration);
		m_xPositionInterpolationData.SetFinal(xTargetPosition);
		m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);

		Asura_Vector_3 xCurrentHeading(xPredalienOwner.GetCurrentOrientation().e13, xPredalienOwner.GetCurrentOrientation().e23, xPredalienOwner.GetCurrentOrientation().e33);
		Asura_Vector_3 xDesiredHeading(pxMessage->GetDestinationPosition()-pxMessage->GetStartPosition());
		xDesiredHeading.y = 0.0f;
		xDesiredHeading.Normalise();

		Asura_Matrix_3x3 xOri;
		Asura_Quat xTurnQuat;
		xTurnQuat.RotationFromAToB(xCurrentHeading, xDesiredHeading);
		Asura_Vector_3 xTurnAxis;
		float fTurnAngle = xTurnQuat.ToRotation(xTurnAxis);
		float fTurnDuration = fTurnAngle / pxMessage->GetTurnSpeed();

		if (fTurnDuration >= fWalkDuration) // if turn take too much time - turn how much we can
		{
			fTurnAngle = fTurnAngle * fWalkDuration / fTurnDuration;
			xTurnQuat.FromRotation(fTurnAngle, xTurnAxis);
			xDesiredHeading = xTurnQuat.RotateVector(xCurrentHeading);
			fTurnDuration = fWalkDuration;
		}


		Asura_Maths::OrientationFromDirection(xOri, xDesiredHeading);
		m_xOrientationInterpolationData.SetFinal(xOri);
		m_xOrientationInterpolationData.SetDuration(fTurnDuration);

		InitialisePositionInterpolation(m_xPositionInterpolationData);
		InitialiseOrientationInterpolation(m_xOrientationInterpolationData);
		m_xAimDirection = xDesiredHeading;
		m_fAimDirectionAngVel = pxMessage->GetTurnSpeed();
		

		// Finally, call the base-function...
		return(PARENT::Start(pxStartMsg));
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Move_Predalien_JumpTarget::AVP_Move_Predalien_JumpTarget
////////////////////////////////////////////////////////////////////////////////

AVP_Move_Predalien_JumpTarget::AVP_Move_Predalien_JumpTarget(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
{
	// Initialise the move flags...
	m_bDisableMovement	= true;
	m_eAimDirectionUsage = AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;

	m_bPreventsFlinch = true;
	m_bClearQueuedMeleeRequests = true;
	m_bClearQueuedCounterRequest = true;
	m_bSupportEndAnimFlag = true;

	// TODO - setup third person anims and use blueprints.
	SetAnimHashes(0xD6197979, 0xD6197979); //Asura_GetHashID("predator_hud_running_jump_5m");

	if (m_pxAVPAnimationData)
	{
		m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
		m_pxAVPAnimationData->SetTweenData(0.2f, 0.2f, true);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Move_Predalien_JumpTarget::Start
////////////////////////////////////////////////////////////////////////////////
bool AVP_Move_Predalien_JumpTarget::Start(const Asura_Msg_Move* pxStartMsg)
{
	// Have we been told to start via a message?
	if(pxStartMsg)
	{
		SetFromMessage(pxStartMsg);
	}


	InitialisePositionInterpolation(m_xPositionInterpolationData);
	InitialiseOrientationInterpolation(m_xOrientationInterpolationData);
	

	if ( m_pxAVPAnimationData )
	{		
		m_pxAVPAnimationData->SetTweenData( 0.1f, 0.1f, true );
	}

	// Finally, call the base-function...
	return(PARENT::Start(pxStartMsg));

}

void AVP_Move_Predalien_JumpTarget::SetFromMessage(const Asura_Msg_Move* pxStartMsg)
{

	const AVP_Message_Predalien_Move_JumpStart* pxMessage = static_cast<const AVP_Message_Predalien_Move_JumpStart*>(pxStartMsg);
	AVP_ClientActor_Predalien& xPredalienOwner = static_cast<AVP_ClientActor_Predalien&>(GetAVPParentSystem().GetActorOwner());
	Asura_Vector_3 xTargetPosition(pxMessage->GetDestinationPosition());

	// Update the animation to use
	Asura_Hash_ID uThirdPersonAnim = ASURA_HASH_ID_UNSET;
	xPredalienOwner.GetTargetedJumpAnimationsForVariation(pxMessage->GetJumpVariation(), uThirdPersonAnim, pxMessage->RunAfterJump());
	SetAnimHashes(ASURA_HASH_ID_UNSET, uThirdPersonAnim);

	if (m_pxAVPAnimationData)
	{
		// We need to ensure the correct anim hash is set up, as it's about to get used.
		if (xPredalienOwner.IsUsingFirstPersonSkin())
		{
			Asura_Assert(false, "Predalien can not have first person animation");
		}
		else
		{
			m_pxAVPAnimationData->SetAnimHash(m_uThirdPersonAnimHash);
		}
	}

	const bool bHasAnimation = m_pxAVPAnimationData && m_pxAVPAnimationData->HasValidAnimation();
	const float fDuration = (bHasAnimation) ? m_pxAVPAnimationData->GetAnimationDuration() : 0.500f;

	// For the time being we'll just move and rotate to our desired position...
	if(bHasAnimation)
	{
		m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);
		m_xOrientationInterpolationData.SetMovementOffsetOperation(Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET);
	}

	m_xPositionInterpolationData.SetDuration(fDuration);
	m_xPositionInterpolationData.SetFinal(xTargetPosition);
	m_xPositionInterpolationData.SetMovementOffsetOperation(Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET);

	float fTurnDuration = pxMessage->GetTurnTime() < fDuration ? pxMessage->GetTurnTime() : fDuration;

	m_xOrientationInterpolationData.SetDuration(fTurnDuration);
	Asura_Vector_3 xFlatOffset = xTargetPosition - xPredalienOwner.GetCurrentPosition();
	xFlatOffset.y = 0.0f;
	if (xFlatOffset.MagnitudeSquared() > 0.01f)
	{
		xFlatOffset.Normalise();
		Asura_Matrix_3x3 xOri;
		Asura_Maths::OrientationFromDirection(xOri, xFlatOffset);
		m_xOrientationInterpolationData.SetFinal(xOri);
	}
	else
	{
		const Asura_Matrix_3x3& xOri = xPredalienOwner.GetCurrentOrientation();
		m_xOrientationInterpolationData.SetFinal(xOri);
		xFlatOffset.x = xOri.e13;
		xFlatOffset.y = xOri.e23;
		xFlatOffset.z = xOri.e33;
	}

	m_xAimDirection = xFlatOffset;
}

Asura_Move* AVP_Move_Predalien_JumpTarget::Update()
{
	const u_int uAnimFlags = GetAnimFlagsForLastFrame();
	SetAimDirectionUsage(((uAnimFlags & AVP_ANIMFLAG_RESET_LOOK) != 0) ? AVP_ClientEntity_Actor::AIM_DIRECTION_RESET : AVP_ClientEntity_Actor::AIM_DIRECTION_LIMIT);

	return PARENT::Update();
}

/// End of file. ///
