////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Predator_Move_Interaction_PlugKey
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Moves/Vertebrates/Predator/AVP_Predator_MovesInteraction.h"

// Asura
#include "Animation/Asura_AnimationController_Animation.h"
#include "Blueprint/Asura_Blueprint_System.h"

// AVP
#include "Animation/AVP_AnimFlags.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_ServerActor_Predator.h"
#include "Entity/PhysicalObject/Interact/PredatorLock/AVP_ClientEntity_InteractPredatorLock.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/BatteryReceptacle/AVP_ClientEntity_InteractBatteryReceptacle.h"
#include "Entity/PhysicalObject/Interact/ToolReceptacle/RetinalScanner/AVP_ClientEntity_InteractRetinalScanner.h"
#include "Moves/AVP_Move_Interaction.h"

DEBUGVAR float  dbg_fPlugKeyAnimDuration = 0.400f;

AVP_Predator_Move_Interaction_PlugKey::AVP_Predator_Move_Interaction_PlugKey( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;

	AddNextMove(AVP_Predator_Move_Interaction_StandStill::GetStaticHash());
}

bool AVP_Predator_Move_Interaction_PlugKey::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Have we been told to start via a message?
	if ( pxStartMsg == NULL )
	{
		Asura_Assert( false, "Interaction move can only be triggered by message." );
		return false;
	}
	else
	{
		const AVP_Message_InteractionMoveStart*	pxMessage = static_cast< const AVP_Message_InteractionMoveStart* >( pxStartMsg );
		// This is the automatic next move, so we initialise it to use this interaction object as it will not be conveyed to it by message
		static_cast< AVP_Predator_Move_Interaction_StandStill* >(GetParentSystem().GetMove(AVP_Predator_Move_Interaction_StandStill::GetStaticHash()))->SetInteractionObjectGuid( pxMessage->GetInteractionObject() );
		AVP_ClientEntity_InteractPredator_Lock* pxInteractObject = AVP_ClientEntity_InteractPredator_Lock::FindEntity( pxMessage->GetInteractionObject() );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", pxMessage->GetInteractionObject() ) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			u_int uOwnerClass = GetParentSystem().GetOwner().GetClassification();
			Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
			Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
			pxInteractObject->GetPlugKeyAnimHashes( uOwnerClass, uFPAnimHash, uTPAnimHash );
			SetAnimHashes( uFPAnimHash, uTPAnimHash );
			if (m_pxAVPAnimationData)
			{
				m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
				m_pxAVPAnimationData->SetTweenData( 1.0f, 0.0f, true );
			}


			const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );
			if ( !pxActor )
			{
				return false;
			}
			else
			{
				Asura_Vector_3 xTargetPosition;
				Asura_Matrix_3x3 xTargetOrientation;

				if ( uFPAnimHash != ASURA_HASH_ID_UNSET || uTPAnimHash != ASURA_HASH_ID_UNSET )
				{
					const AVP_ClientEntity_InteractObject::AnimData xAnimData = pxInteractObject->GetInteractionData( *pxActor, pxMessage->GetOn() );
					if ( !pxInteractObject->GetMarkerLocationFromHash( xAnimData.m_uActorPositionMarkerPoint, &xTargetPosition, &xTargetOrientation ) )
					{
						xTargetPosition = pxActor->GetCurrentPosition();
						xTargetOrientation = pxActor->GetCurrentOrientation();
					}

					// For the time being we'll just move and rotate to our desired position...
					m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET );
					m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET );

					m_xOrientationInterpolationData.SetDuration( dbg_fPlugKeyAnimDuration );
					m_xOrientationInterpolationData.SetFinal( xTargetOrientation );
					m_xPositionInterpolationData.SetDuration( dbg_fPlugKeyAnimDuration );
					m_xPositionInterpolationData.SetFinal( xTargetPosition );

					InitialiseOrientationInterpolation( m_xOrientationInterpolationData );
					InitialisePositionInterpolation( m_xPositionInterpolationData );
				}

				// Finally, call the base-function...
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Predator_Move_Interaction_UnPlugKey
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

AVP_Predator_Move_Interaction_UnPlugKey::AVP_Predator_Move_Interaction_UnPlugKey( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	SetPositionUsage( POSITION_SET_DIRECTLY );
	m_bHideReticle					= true;
}

bool AVP_Predator_Move_Interaction_UnPlugKey::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Have we been told to start via a message?
	if ( pxStartMsg == NULL )
	{
		Asura_Assert( false, "Interaction move can only be triggered by message." );
		return false;
	}
	else
	{
		const AVP_Message_InteractionMoveStart*	pxMessage = static_cast< const AVP_Message_InteractionMoveStart* >( pxStartMsg );
		// We need to ensure the correct anim hash is set up, as it's about to get used.
		// Set Again the stand still interaction object to Unregistred
		static_cast< AVP_Predator_Move_Interaction_StandStill* >(GetParentSystem().GetMove(AVP_Predator_Move_Interaction_StandStill::GetStaticHash()))->SetInteractionObjectGuid( ASURA_GUID_UNREGISTERED);
		AVP_ClientEntity_InteractPredator_Lock* pxInteractObject = AVP_ClientEntity_InteractPredator_Lock::FindEntity( pxMessage->GetInteractionObject() );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", pxMessage->GetInteractionObject() ) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			u_int uOwnerClass = GetParentSystem().GetOwner().GetClassification();
			Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
			Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
			pxInteractObject->GetUnPlugKeyAnimHashes( uOwnerClass, uFPAnimHash, uTPAnimHash );
			SetAnimHashes( uFPAnimHash, uTPAnimHash );
			if (m_pxAVPAnimationData)
			{
				m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
				m_pxAVPAnimationData->SetTweenData( DEFAULT_TWEEN_TIME, DEFAULT_TWEEN_TIME, true );
			}
			const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );
			if ( !pxActor )
			{
				return false;
			}
			else
			{
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Predator_Move_Interaction_StandStill
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

AVP_Predator_Move_Interaction_StandStill::AVP_Predator_Move_Interaction_StandStill( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
, m_fTweenHack(0.0f)
, m_uInteractionObject(ASURA_GUID_UNREGISTERED)
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;
	SetLoopAtEnd(true);
	SetPositionUsage( POSITION_SET_DIRECTLY );
}

bool AVP_Predator_Move_Interaction_StandStill::Start( const Asura_Msg_Move* pxStartMsg )
{
	//Reset tween hack
	m_fTweenHack = 0.0f;

	AVP_ClientEntity_InteractPredator_Lock* pxInteractObject = NULL;
	// Have we been told to start via a message?
	if ( pxStartMsg )
	{
		const AVP_Message_InteractionMoveStart*	pxMessage = static_cast< const AVP_Message_InteractionMoveStart* >( pxStartMsg );
		m_uInteractionObject = pxMessage->GetInteractionObject();
	}

	pxInteractObject = AVP_ClientEntity_InteractPredator_Lock::FindEntity( m_uInteractionObject );

	Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", m_uInteractionObject ) );
	if ( pxInteractObject == NULL )
	{
		return false;
	}
	else
	{
		u_int uOwnerClass = GetParentSystem().GetOwner().GetClassification();
		Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
		Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
		pxInteractObject->GetStandStillAnimHashes( uOwnerClass, uFPAnimHash, uTPAnimHash );
		SetAnimHashes( uFPAnimHash, uTPAnimHash );
		if (m_pxAVPAnimationData)
		{
			m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
			m_pxAVPAnimationData->SetTweenData( DEFAULT_TWEEN_TIME, 0.0f, true );
		}

		const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
		Asura_Assert( pxActor, "Only actors should be interacting" );
		if ( !pxActor )
		{
			return false;
		}
		else
		{
			return PARENT::Start( pxStartMsg );
		}
	}
}

void AVP_Predator_Move_Interaction_StandStill::AdvanceTweenHack()
{
	m_fTweenHack += Asura_Timers::GetGameTimeSlice();
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Predator_Move_Interaction_TurnLeft
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

AVP_Predator_Move_Interaction_TurnLeft::AVP_Predator_Move_Interaction_TurnLeft( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;
	AddNextMove(AVP_Predator_Move_Interaction_StandStill::GetStaticHash());
	SetPositionUsage( POSITION_SET_DIRECTLY );
}

bool AVP_Predator_Move_Interaction_TurnLeft::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Have we been told to start via a message?
	if ( pxStartMsg == NULL )
	{
		Asura_Assert( false, "Interaction move can only be triggered by message." );
		return false;
	}
	else
	{
		const AVP_Message_InteractionMoveStart*	pxMessage = static_cast< const AVP_Message_InteractionMoveStart* >( pxStartMsg );
		// We need to ensure the correct anim hash is set up, as it's about to get used.
		// This is the automatic next move, so we initialise it to use this interaction object as it will not be conveyed to it by message
		static_cast< AVP_Predator_Move_Interaction_StandStill* >(GetParentSystem().GetMove(AVP_Predator_Move_Interaction_StandStill::GetStaticHash()))->SetInteractionObjectGuid( pxMessage->GetInteractionObject() );
		AVP_ClientEntity_InteractPredator_Lock* pxInteractObject = AVP_ClientEntity_InteractPredator_Lock::FindEntity( pxMessage->GetInteractionObject() );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", pxMessage->GetInteractionObject() ) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			u_int uOwnerClass = GetParentSystem().GetOwner().GetClassification();
			Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
			Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
			pxInteractObject->GetTurnLeftAnimHashes( uOwnerClass, uFPAnimHash, uTPAnimHash );
			SetAnimHashes( uFPAnimHash, uTPAnimHash );
			if (m_pxAVPAnimationData)
			{
				m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
				m_pxAVPAnimationData->SetTweenData( DEFAULT_TWEEN_TIME, 0.0f, true );
			}

			const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );
			if ( !pxActor )
			{
				return false;
			}
			else
			{
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Predator_Move_Interaction_TurnRight
// Author       : Ali Jakamy
// Date         : 23/10/08
////////////////////////////////////////////////////////////////////////////////

AVP_Predator_Move_Interaction_TurnRight::AVP_Predator_Move_Interaction_TurnRight( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;
	AddNextMove(AVP_Predator_Move_Interaction_StandStill::GetStaticHash());
	SetPositionUsage( POSITION_SET_DIRECTLY );
}

bool AVP_Predator_Move_Interaction_TurnRight::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Have we been told to start via a message?
	if ( pxStartMsg == NULL )
	{
		Asura_Assert( false, "Interaction move can only be triggered by message." );
		return false;
	}
	else
	{
		const AVP_Message_InteractionMoveStart*	pxMessage = static_cast< const AVP_Message_InteractionMoveStart* >( pxStartMsg );
		// We need to ensure the correct anim hash is set up, as it's about to get used.
		// This is the automatic next move, so we initialise it to use this interaction object as it will not be conveyed to it by message
		static_cast< AVP_Predator_Move_Interaction_StandStill* >(GetParentSystem().GetMove(AVP_Predator_Move_Interaction_StandStill::GetStaticHash()))->SetInteractionObjectGuid( pxMessage->GetInteractionObject() );
		AVP_ClientEntity_InteractPredator_Lock* pxInteractObject = AVP_ClientEntity_InteractPredator_Lock::FindEntity( pxMessage->GetInteractionObject() );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", pxMessage->GetInteractionObject() ) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			u_int uOwnerClass = GetParentSystem().GetOwner().GetClassification();
			Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
			Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
			pxInteractObject->GetTurnRightAnimHashes( uOwnerClass, uFPAnimHash, uTPAnimHash );
			SetAnimHashes( uFPAnimHash, uTPAnimHash );
			if (m_pxAVPAnimationData)
			{
				m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
				m_pxAVPAnimationData->SetTweenData( DEFAULT_TWEEN_TIME, 0.0f, true );
			}

			const AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );
			if ( !pxActor )
			{
				return false;
			}
			else
			{
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Predator_Move_Interaction_UseBattery
// Author       : Ali Jakamy
// Date         : 01/12/08
////////////////////////////////////////////////////////////////////////////////

AVP_Predator_Move_Interaction_UseBattery::AVP_Predator_Move_Interaction_UseBattery( Asura_Movement_System& xParentSystem )
: PARENT( xParentSystem, GetStaticHash() )
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;

	AddNextMove( AVP_Move_Interaction_EndWeaponBack::GetStaticHash() );
}

bool AVP_Predator_Move_Interaction_UseBattery::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Not via a message?
	if ( pxStartMsg != NULL )
	{
		Asura_Assert( false, "Interaction move can only be triggered by message." );
		return false;
	}
	else
	{
		const Asura_Move* pxMove = GetParentSystem().GetCurrentMove();
		if ( pxMove->GetHash() != AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
		{
			Asura_Assert( false, "Can only be started from AVP_Move_Interaction_Begin" );
			return false;
		}
		const AVP_Move_Interaction_BeginWeaponAway* pxIntBeginMove = static_cast< const AVP_Move_Interaction_BeginWeaponAway* >( pxMove );

		Asura_Guid uInteractionGuid = pxIntBeginMove->GetInteractionObject();
		bool bOn = pxIntBeginMove->GetOn();

		AVP_ClientEntity_InteractBatteryReceptacle* pxInteractObject = AVP_ClientEntity_InteractBatteryReceptacle::FindEntity( uInteractionGuid );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", uInteractionGuid ) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );

			if ( !pxActor )
			{
				return false;
			}
			else
			{
				pxActor->DeactivateCurrentWeapon();

				Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
				Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
				pxInteractObject->GetPredatorUseBatteryAnimHashes( uFPAnimHash, uTPAnimHash );
				SetAnimHashes( uFPAnimHash, uTPAnimHash );

			
				if (m_pxAVPAnimationData)
				{
					m_pxAVPAnimationData->SetTweenData( DEFAULT_TWEEN_TIME, 0.0f, true );
					m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
					m_pxAVPAnimationData->SetControllerGroupHash( pxActor->GetAnimationHandler()->GetWeaponGroupHash() );
					m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_WEAPON_ANIM );
				}


				const float	fDuration = 0.400f;
				Asura_Vector_3 xTargetPosition;
				Asura_Matrix_3x3 xTargetOrientation;

				if ( uFPAnimHash != ASURA_HASH_ID_UNSET || uTPAnimHash != ASURA_HASH_ID_UNSET )
				{
					const AVP_ClientEntity_InteractObject::AnimData xAnimData = pxInteractObject->GetInteractionData( *pxActor, bOn );
					if ( !pxInteractObject->GetMarkerLocationFromHash( xAnimData.m_uActorPositionMarkerPoint, &xTargetPosition, &xTargetOrientation ) )
					{
						xTargetPosition = pxActor->GetCurrentPosition();
						xTargetOrientation = pxActor->GetCurrentOrientation();
					}

					// For the time being we'll just move and rotate to our desired position...
					m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET );
					m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET );

					m_xOrientationInterpolationData.SetDuration( fDuration );
					m_xOrientationInterpolationData.SetFinal( xTargetOrientation );
					m_xPositionInterpolationData.SetDuration( fDuration );
					m_xPositionInterpolationData.SetFinal( xTargetPosition );

					InitialiseOrientationInterpolation( m_xOrientationInterpolationData );
					InitialisePositionInterpolation( m_xPositionInterpolationData );
				}

				// Finally, call the base-function...
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}

void AVP_Predator_Move_Interaction_UseBattery::Abort()
{

	//Asura_AnimationController_Animation* pxAnim = m_pxAVPAnimationData->GetAnimationController();

	// Enable weapon use again
	AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
	if ( pxActor ) pxActor->ActivateCurrentWeapon();

	PARENT::Abort();

	//// Get next move
	//Asura_Move* pxNextMove = GetNextMove();
	//if ( pxNextMove )
	//{
	//	pxNextMove->Start(NULL);
	//	pxNextMove->Abort();
	//}
}

void AVP_Predator_Move_Interaction_UseBattery::Finish()
{
	// Enable weapon use again
	AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
	if ( pxActor ) pxActor->ActivateCurrentWeapon();

	PARENT::Finish();
}

bool AVP_Predator_Move_Interaction_UseBattery::ShouldStart() const
{
	const Asura_Move* pxMove = GetParentSystem().GetCurrentMove();
	if ( pxMove->GetHash() != AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
	{
		Asura_Assert( false, "Can only be started from AVP_Move_Interaction_Begin" );
		return false;
	}
	const AVP_Move_Interaction_BeginWeaponAway* pxIntBeginMove = static_cast< const AVP_Move_Interaction_BeginWeaponAway* >( pxMove );
	return pxIntBeginMove->GetNextMoveHash() == GetStaticHash();
}

////////////////////////////////////////////////////////////////////////////////
// Class		: AVP_Predator_Move_Interaction_UseHead
// Author       : Ali Jakamy
// Date         : 05/01/09
////////////////////////////////////////////////////////////////////////////////

AVP_Predator_Move_Interaction_UseHead::AVP_Predator_Move_Interaction_UseHead(Asura_Movement_System& xParentSystem)
: PARENT(xParentSystem, GetStaticHash())
, m_pxTrophyAttachment(NULL)
{
	// Initialise the move flags...
	m_bClearQueuedCounterRequest	= true;
	m_bClearQueuedMeleeRequests		= true;
	m_bPreventsFlinch				= true;
	m_bDisableMovement				= true;
	m_eAimDirectionUsage			= AVP_ClientEntity_Actor::AIM_DIRECTION_RESET;
	m_bHideReticle					= true;

	m_uTrophyMarkerPoint            = 0xA282148B; // Asura_GetHashID("righthand");

	AddNextMove( AVP_Move_Interaction_EndWeaponBack::GetStaticHash() );

}

AVP_Predator_Move_Interaction_UseHead::~AVP_Predator_Move_Interaction_UseHead()
{
	DetachTrophy();
}

bool AVP_Predator_Move_Interaction_UseHead::Start( const Asura_Msg_Move* pxStartMsg )
{
	// Not via a message?
	if ( pxStartMsg != NULL )
	{
		Asura_Assert( false, "Interaction move cannot be triggered by message." );
		return false;
	}
	else
	{
		const Asura_Move* pxMove = GetParentSystem().GetCurrentMove();
		if ( pxMove->GetHash() != AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
		{
			Asura_Assert( false, "Can only be started from AVP_Move_Interaction_Begin" );
			return false;
		}
		const AVP_Move_Interaction_BeginWeaponAway* pxIntBeginMove = static_cast< const AVP_Move_Interaction_BeginWeaponAway* >( pxMove );

		Asura_Guid uInteractionGuid = pxIntBeginMove->GetInteractionObject();
		bool bOn = pxIntBeginMove->GetOn();

		AVP_ClientEntity_InteractRetinalScanner* pxInteractObject = AVP_ClientEntity_InteractRetinalScanner::FindEntity( uInteractionGuid );
		Asura_Assertf( pxInteractObject, ( "Unable to find client side interaction object %d to interact with", uInteractionGuid) );
		if ( pxInteractObject == NULL )
		{
			return false;
		}
		else
		{
			AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
			Asura_Assert( pxActor, "Only actors should be interacting" );
			if ( !pxActor )
			{
				return false;
			}
			else
			{
				pxActor->DeactivateCurrentWeapon();

				Asura_Hash_ID uFPAnimHash = ASURA_HASH_ID_UNSET;
				Asura_Hash_ID uTPAnimHash = ASURA_HASH_ID_UNSET;
				pxInteractObject->GetPredatorUseHeadAnimHashes( uFPAnimHash, uTPAnimHash );
				SetAnimHashes( uFPAnimHash, uTPAnimHash );
				if (m_pxAVPAnimationData)
				{
					m_pxAVPAnimationData->SetTweenData( DEFAULT_TWEEN_TIME, 0.0f, true );
					m_pxAVPAnimationData->SetLeaveAnimAtEnd(true);
					m_pxAVPAnimationData->SetControllerGroupHash( pxActor->GetAnimationHandler()->GetWeaponGroupHash() );
					m_pxAVPAnimationData->SetAnimPriority( AVP_AAPC_WEAPON_ANIM );
				}

				// Attach the head
				AttachTrophy(*pxInteractObject);

				const float	fDuration = 0.400f;
				Asura_Vector_3 xTargetPosition;
				Asura_Matrix_3x3 xTargetOrientation;

				if ( uFPAnimHash != ASURA_HASH_ID_UNSET || uTPAnimHash != ASURA_HASH_ID_UNSET )
				{
					const AVP_ClientEntity_InteractObject::AnimData xAnimData = pxInteractObject->GetInteractionData( *pxActor, bOn );
					if ( !pxInteractObject->GetMarkerLocationFromHash( xAnimData.m_uActorPositionMarkerPoint, &xTargetPosition, &xTargetOrientation ) )
					{
						xTargetPosition = pxActor->GetCurrentPosition();
						xTargetOrientation = pxActor->GetCurrentOrientation();
					}

					// For the time being we'll just move and rotate to our desired position...
					m_xOrientationInterpolationData.SetMovementOffsetOperation( Asura_MoveData_OrientationInterpolation::STRETCH_MOVEMENT_OFFSET );
					m_xPositionInterpolationData.SetMovementOffsetOperation( Asura_MoveData_PositionInterpolation::STRETCH_MOVEMENT_OFFSET );

					m_xOrientationInterpolationData.SetDuration( fDuration );
					m_xOrientationInterpolationData.SetFinal( xTargetOrientation );
					m_xPositionInterpolationData.SetDuration( fDuration );
					m_xPositionInterpolationData.SetFinal( xTargetPosition );

					InitialiseOrientationInterpolation( m_xOrientationInterpolationData );
					InitialisePositionInterpolation( m_xPositionInterpolationData );
				}

				// Finally, call the base-function...
				return PARENT::Start( pxStartMsg );
			}
		}
	}
}

bool AVP_Predator_Move_Interaction_UseHead::ShouldStart() const
{
	const Asura_Move* pxMove = GetParentSystem().GetCurrentMove();
	if ( pxMove->GetHash() != AVP_Move_Interaction_BeginWeaponAway::GetStaticHash() )
	{
		Asura_Assert( false, "Can only be started from AVP_Move_Interaction_Begin" );
		return false;
	}
	const AVP_Move_Interaction_BeginWeaponAway* pxIntBeginMove = static_cast< const AVP_Move_Interaction_BeginWeaponAway* >( pxMove );
	return pxIntBeginMove->GetNextMoveHash() == GetStaticHash();
}

void AVP_Predator_Move_Interaction_UseHead::AttachTrophy(const AVP_ClientEntity_InteractRetinalScanner& xRetinalScanner)
{
	Asura_Assert(!m_pxTrophyAttachment, "Already have a trophy attached");
	if (m_pxTrophyAttachment) { return; }

	if (m_uTrophyMarkerPoint == ASURA_HASH_ID_UNSET) { return; }

	const Asura_Blueprint* pxActorBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(xRetinalScanner.GetVictimBlueprintType(), xRetinalScanner.GetVictimBlueprintHash());

	if (!pxActorBlueprint) { return; }

	const Asura_Blueprint* pxSkinElementBlueprint = NULL;

	Asura_Value xValue;

	if(pxActorBlueprint->GetParameterValueByHashID(0x0F8AC93F, xValue)) // 0x0F8AC93F GetHash("SkinElementHead")
	{
 		pxSkinElementBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( xRetinalScanner.GetVictimBlueprintType(), xValue.GetHashIDValue());
	}

	if (!pxSkinElementBlueprint) { return; }

	if (!pxSkinElementBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_SKINELEMENT_GIBSKIN, xValue)) { return; }

	const Asura_Hash_ID uSkin = xValue.GetHashIDValue();
	if (uSkin == ASURA_HASH_ID_UNSET) { return; }

	Asura_Animation* pxAnim = new Asura_Animation(uSkin);
	if (!pxAnim) { return; }

	//if (xRetinalScanner.GetToolUseAnim() != ASURA_HASH_ID_UNSET)
	//{
	//	Asura_AnimationController_Animation* pxController = pxAnim->CreateControllerAnimation(xRetinalScanner.GetToolUseAnim(), AAC_FLAG_ACTIVE, 1, xRetinalScanner.GetToolUseAnim(), NULL);
	//	if (pxController)
	//	{
	//		pxController->Play();
	//	}
	//}

	Asura_AnimationController_Animation* pxAACA = pxAnim->CreateControllerAnimation( AAC_FLAG_ACTIVE | AAC_FLAG_ALLOW_DESTROY_FROM_MESSAGE, 0, xRetinalScanner.GetToolUseAnim(), NULL );
	if ( pxAACA )
	{
		pxAACA->Play();
	}

	Asura_AnimationAttachment_Animation* pxAttachment = new Asura_AnimationAttachment_Animation(pxAnim, true);
	if (!pxAttachment)
	{
		delete pxAnim;
		return;
	}

	pxAttachment->SetDeleteAnimOnUnlink(true);

	Asura_Animation* pxOwnerAnim = GetAVPParentSystem().GetActorOwner().GetShapeInstance().GetAnimation();
	if (!pxOwnerAnim) { return; }

	// Don't allow it to delete with the anim, as we need to keep track of it here.
	if (!pxAttachment->LinkAttachment(pxOwnerAnim, m_uTrophyMarkerPoint, false))
	{
		delete pxAttachment;
		return;
	}

	m_pxTrophyAttachment = pxAttachment;

	//// We also want to attach the same pfx we normally do to a gib.
	//AVP_ClientEntity_Gib::PFXAttachData axPFXAttachData[uAVP_VERTEBRATE_GIB_MAX_PFX_ATTACHMENTS];
	//u_int uNumPFXAttachments = 0;
	//AVP_ClientEntity_Gib::FillPFXDetailsFromBlueprint(pxSkinElementBlueprint, axPFXAttachData, uNumPFXAttachments);
	//const u_short usPFXFlags = ASURA_PFX_EFFECT_CAMERA_FLAGS;

	//for (u_int u=0; u<uNumPFXAttachments; ++u)
	//{
	//	const Asura_Hash_ID uPFXHash = axPFXAttachData[u].m_uPFXID;
	//	const Asura_Hash_ID uMarkerHash = axPFXAttachData[u].m_uMarkerID;
	//	if ((uPFXHash != ASURA_HASH_ID_UNSET) && (uMarkerHash != ASURA_HASH_ID_UNSET))
	//	{
	//		Asura_AnimationAttachment* pxAttachment = new Asura_AnimationAttachment_PFX(uPFXHash, usPFXFlags);
	//		if (pxAttachment)
	//		{
	//			pxAttachment->LinkAttachment(pxAnim, uMarkerHash, true);
	//		}
	//	}
	//}
}

void AVP_Predator_Move_Interaction_UseHead::Abort()
{
	DetachTrophy();

	// Enable weapon use again
	AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
	if ( pxActor ) pxActor->ActivateCurrentWeapon();

	PARENT::Abort();
}

void AVP_Predator_Move_Interaction_UseHead::Finish()
{
	DetachTrophy();
 
	// Enable weapon use again
	AVP_ClientEntity_Actor* pxActor = AVP_ClientEntity_Actor::FindEntity( GetParentSystem().GetOwnerGuid() );
	if ( pxActor ) pxActor->ActivateCurrentWeapon();

	PARENT::Finish();
}

void AVP_Predator_Move_Interaction_UseHead::UpdateStates(const float fTimeStep)
{

	const u_int uAnimFlags = GetAnimFlagsForLastFrame();
	if (uAnimFlags & AVP_ANIMFLAG_COMPLETE)
	{
		DetachTrophy();
	}

	PARENT::UpdateStates(fTimeStep);
}

void AVP_Predator_Move_Interaction_UseHead::DetachTrophy()
{
	if (!m_pxTrophyAttachment) { return; }

	if (m_pxTrophyAttachment->IsLinked())
	{
		m_pxTrophyAttachment->UnlinkAttachment();
	}
	delete m_pxTrophyAttachment;
	m_pxTrophyAttachment = NULL;
}