//
// AVP_Multiplayer_ScoreRules.cpp
//
// Created by JG - 30 June 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Multiplayer/AVP_Multiplayer_ScoreRules.h"

// Asura independent headers
#include "Blueprint/Asura_Blueprint_System.h"

// Project independent headers
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Settings.h"

// Static variables
bool						AVP_Multiplayer_ScoreRules::s_bInitialised = false;
AVP_Multiplayer_ScoreRules* AVP_Multiplayer_ScoreRules::s_aapxScoreTable[AVP_Settings::AVP_NUM_SPECIES][AVP_Settings::AVP_NUM_GAMETYPES]; 

////////////////////////////////////////////////////////////
//  Constructor / Destructor
////////////////////////////////////////////////////////////

AVP_Multiplayer_ScoreRules::AVP_Multiplayer_ScoreRules( Asura_Hash_ID uBlueprintType, Asura_Hash_ID uBlueprintName ) 
: m_iRole( AVP_Settings::AVP_ROLE_PREY )
, m_iScoreKillAlien( 0 )
, m_iScoreKillAssist( 0 )
, m_iScoreKillHuman( 0 )
, m_iScoreKillPersecutorBonus( 0 )
, m_iScoreKillPredator( 0 )
, m_iScoreKillTeammate( 0 )
, m_iScoreKillYourself( 0 )
, m_iSpecies( AVP_Settings::AVP_SPECIES_HUMAN )
{
	// Initilise the score from the blueprint
	Initialise( uBlueprintType, uBlueprintName ); 
}

AVP_Multiplayer_ScoreRules::~AVP_Multiplayer_ScoreRules( )
{
	// Nothing to do here
}

////////////////////////////////////////////////////////////
//  Initialisation
////////////////////////////////////////////////////////////
void AVP_Multiplayer_ScoreRules::Initialise( void )
{
	// Only initialise this system once
	if (s_bInitialised) return; 

	// Initialise score tables for different game modes here (no concatenation of hash IDs here as there is not many of them)
	// Species vs. GameType

		// first check we were shutdown correctly
	for ( u_int u = 0; u < AVP_Settings::AVP_NUM_SPECIES; u++ )
	{
		for ( u_int v = 0; v < AVP_Settings::AVP_NUM_GAMETYPES; v++ )
		{
			Asura_Assert(!s_aapxScoreTable[u][v], "memory leak");
		}
	}

	
	////////////////////////////////////////////////////////////
	//  Predator
	////////////////////////////////////////////////////////////
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_PREDATOR][AVP_Settings::AVP_GAMETYPE_DISRUPTION_DOMINATION] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_PREDATOR_DISRUPTION_DOMINATION ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_PREDATOR][AVP_Settings::AVP_GAMETYPE_DEATH_MATCH] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_PREDATOR_DEATH_MATCH ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_PREDATOR][AVP_Settings::AVP_GAMETYPE_HUNT] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_PREDATOR_HUNT ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_PREDATOR][AVP_Settings::AVP_GAMETYPE_SPECIES_TEAM_DEATH_MATCH] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_PREDATOR_SPECIES_TEAM_DEATH_MATCH ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_PREDATOR][AVP_Settings::AVP_GAMETYPE_SURVIVOR] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_PREDATOR_SURVIVOR ); 

	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_PREDATOR][AVP_Settings::AVP_GAMETYPE_MIXED_SPECIES_TEAM_DEATH_MATCH] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_PREDATOR_MIXED_SPECIES_TEAM_DEATH_MATCH ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_PREDATOR][AVP_Settings::AVP_GAMETYPE_DOMINATION] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_PREDATOR_DOMINATION ); 

	////////////////////////////////////////////////////////////
	//  Alien
	////////////////////////////////////////////////////////////
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_ALIEN][AVP_Settings::AVP_GAMETYPE_DISRUPTION_DOMINATION] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_ALIEN_DISRUPTION_DOMINATION ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_ALIEN][AVP_Settings::AVP_GAMETYPE_DEATH_MATCH] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_ALIEN_DEATH_MATCH ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_ALIEN][AVP_Settings::AVP_GAMETYPE_HUNT] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_ALIEN_HUNT ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_ALIEN][AVP_Settings::AVP_GAMETYPE_SPECIES_TEAM_DEATH_MATCH] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_ALIEN_SPECIES_TEAM_DEATH_MATCH ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_ALIEN][AVP_Settings::AVP_GAMETYPE_SURVIVOR] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_ALIEN_SURVIVOR ); 

	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_ALIEN][AVP_Settings::AVP_GAMETYPE_MIXED_SPECIES_TEAM_DEATH_MATCH] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_ALIEN_MIXED_SPECIES_TEAM_DEATH_MATCH ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_ALIEN][AVP_Settings::AVP_GAMETYPE_DOMINATION] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_ALIEN_DOMINATION ); 

	////////////////////////////////////////////////////////////
	//  Human
	////////////////////////////////////////////////////////////
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_HUMAN][AVP_Settings::AVP_GAMETYPE_DISRUPTION_DOMINATION] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_MARINE_DISRUPTION_DOMINATION ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_HUMAN][AVP_Settings::AVP_GAMETYPE_DEATH_MATCH] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_MARINE_DEATH_MATCH ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_HUMAN][AVP_Settings::AVP_GAMETYPE_HUNT] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_MARINE_HUNT ); 

	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_HUMAN][AVP_Settings::AVP_GAMETYPE_SPECIES_TEAM_DEATH_MATCH] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_MARINE_SPECIES_TEAM_DEATH_MATCH ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_HUMAN][AVP_Settings::AVP_GAMETYPE_SURVIVOR] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_MARINE_SURVIVOR ); 

	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_HUMAN][AVP_Settings::AVP_GAMETYPE_MIXED_SPECIES_TEAM_DEATH_MATCH] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_MARINE_MIXED_SPECIES_TEAM_DEATH_MATCH ); 
	
	s_aapxScoreTable[AVP_Settings::AVP_SPECIES_HUMAN][AVP_Settings::AVP_GAMETYPE_DOMINATION] = 
		new AVP_Multiplayer_ScoreRules( uAVP_BLUEPRINT_TYPE_MULTIPLAYER, uAVP_BLUEPRINT_NAME_MARINE_DOMINATION ); 

	// check we remembered them all
	for ( u_int u = 0; u < AVP_Settings::AVP_NUM_SPECIES; u++ )
	{
		for ( u_int v = 0; v < AVP_Settings::AVP_NUM_GAMETYPES; v++ )
		{
			Asura_Assert(s_aapxScoreTable[u][v], "didn't allocate a score table entry");
		}
	}

	// Initialisation completed
	s_bInitialised = true; 
}

void AVP_Multiplayer_ScoreRules::Initialise( Asura_Hash_ID uBlueprintType, Asura_Hash_ID uBlueprintName )
{
	m_uBlueprintType = uBlueprintType; 
	m_uBlueprintName = uBlueprintName;

	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( m_uBlueprintType, m_uBlueprintName );
	if ( pxBlueprint )
	{
		InitialiseFromBlueprint( *pxBlueprint );
	}
	else
	{
		Asura_Assertf( pxBlueprint, ( "Couldn't find blueprint %0X in type %0X", m_uBlueprintName, m_uBlueprintType )); 
	}
}

void AVP_Multiplayer_ScoreRules::Shutdown()
{
	for ( u_int u = 0; u < AVP_Settings::AVP_NUM_SPECIES; u++ )
	{
		for ( u_int v = 0; v < AVP_Settings::AVP_NUM_GAMETYPES; v++ )
		{
			if (s_aapxScoreTable[u][v])
			{
				delete s_aapxScoreTable[u][v];
				s_aapxScoreTable[u][v] = NULL;
			}
		}
	}

	s_bInitialised = false; 
}

void AVP_Multiplayer_ScoreRules::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
	// Role
	{
		Asura_Value xRole; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_ROLE, xRole ))
		{
			SetRole( static_cast<AVP_Settings::Role>(xRole.GetIntValue()) ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_ROLE )); 
		}
	}
	// Score kill alien
	{
		Asura_Value xScoreKillAlien; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SCORE_KILL_ALIEN, xScoreKillAlien ))
		{
			SetScoreKillAlien( xScoreKillAlien.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_SCORE_KILL_ALIEN )); 
		}
	}
	// Score kill assist
	{
		Asura_Value xScoreKillAssist; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SCORE_KILL_ASSIST, xScoreKillAssist ))
		{
			SetScoreKillAssist( xScoreKillAssist.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_SCORE_KILL_ASSIST )); 
		}
	}
	// Score kill human
	{
		Asura_Value xScoreKillHuman; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SCORE_KILL_HUMAN, xScoreKillHuman ))
		{
			SetScoreKillHuman( xScoreKillHuman.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_SCORE_KILL_HUMAN )); 
		}
	}
	// Score kill persecutor bonus
	{
		Asura_Value xScoreKillPersecutorBonus; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SCORE_KILL_PERSECUTOR_BONUS, xScoreKillPersecutorBonus ))
		{
			SetScoreKillPersecutorBonus( xScoreKillPersecutorBonus.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_SCORE_KILL_PERSECUTOR_BONUS )); 
		}
	}
	// Score kill predator
	{
		Asura_Value xScoreKillPredator; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SCORE_KILL_PREDATOR, xScoreKillPredator ))
		{
			SetScoreKillPredator( xScoreKillPredator.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_SCORE_KILL_PREDATOR )); 
		}
	}
	// Score kill teammate
	{
		Asura_Value xScoreKillTeammate; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SCORE_KILL_TEAMMATE, xScoreKillTeammate ))
		{
			SetScoreKillTeammate( xScoreKillTeammate.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_SCORE_KILL_TEAMMATE )); 
		}
	}
	// Score kill yourself
	{
		Asura_Value xScoreKillYourself; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SCORE_KILL_YOURSELF, xScoreKillYourself ))
		{
			SetScoreKillYourself( xScoreKillYourself.GetIntValue() ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_SCORE_KILL_YOURSELF )); 
		}
	}
	// Species
	{
		Asura_Value xSpecies; 
		if ( xBlueprint.GetParameterValueByHashID( uAVP_BLUEPRINT_NAME_SPECIES, xSpecies ))
		{
			SetSpecies( static_cast<AVP_Settings::Species>(xSpecies.GetIntValue()) ); 
		}
		else
		{
			Asura_Assertf( false, ( "Couldn't find blueprint parameter %0X", uAVP_BLUEPRINT_NAME_SPECIES )); 
		}
	}
}

int AVP_Multiplayer_ScoreRules::GetScoreKillSpecies(AVP_Settings::Species eSpecie)
{
	int iScore = 0; 
	switch (eSpecie)
	{
		case AVP_Settings::AVP_SPECIES_ALIEN:
		{
			iScore = GetScoreKillAlien(); 
			break;
		}
		case AVP_Settings::AVP_SPECIES_PREDATOR:
		{
			iScore = GetScoreKillPredator(); 
			break;
		}
		case AVP_Settings::AVP_SPECIES_HUMAN:
		{
			iScore = GetScoreKillHuman(); 
			break;
		}
		default:
		{
			Asura_Assert( false, "Unknown victim species, unable to get score!" ); 
		}
	}

	return iScore; 
}

////////////////////////////////////////////////////////////////////////////////
//  NAME     :  GetScoreTable()
//  CREATED  :  JG (1.8.2008)
//  DESC     :  Score for a species playing a type of game
////////////////////////////////////////////////////////////////////////////////
AVP_Multiplayer_ScoreRules* AVP_Multiplayer_ScoreRules::GetScoreTable(AVP_Settings::Species eSpecies, AVP_Settings::GameType eGametype )
{
	Asura_Assert( s_bInitialised, "Multiplayer score tables haven't been initialised" );

	if ( eSpecies < AVP_Settings::AVP_NUM_SPECIES )
	{
		if ( eGametype < AVP_Settings::AVP_NUM_GAMETYPES )
		{
			return s_aapxScoreTable[eSpecies][eGametype];
		}
		else
		{
			Asura_Assertf( eGametype < AVP_Settings::AVP_NUM_GAMETYPES, ( "Requested score for invalid game type: %d", eGametype )); 
		}
	}
	else
	{
		Asura_Assertf( eSpecies < AVP_Settings::AVP_NUM_SPECIES, ( "Requested score for invalid species: %d", eSpecies )); 
	}
	return NULL; 
}

