//
// AVP_Multiplayer_TeamScore.cpp
//
// Created by JG - 30 June 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Multiplayer/AVP_Multiplayer_TeamScore.h"

// Asura
#include "Network/Asura_Network.h"

// Project 
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Multiplayer/AVP_Multiplayer_GameRules.h"

// Globals
Asura_List<AVP_Multiplayer_TeamScore>			AVP_Multiplayer_TeamScore::s_xTeamScoreList; 

AVP_Multiplayer_TeamScore::AVP_Multiplayer_TeamScore( AVP_Settings::Team eTeam )
: m_eTeam( eTeam )
{
	Reset(); 
}

AVP_Multiplayer_TeamScore::~AVP_Multiplayer_TeamScore()
{
}

void AVP_Multiplayer_TeamScore::Reset()
{
	m_iNumberOfTeamLives = 0; 
	m_iTeamScore = 0; 
}

void AVP_Multiplayer_TeamScore::Create( AVP_Settings::Team eTeam )
{
	// Team score information should only live in the server
	if ( !Asura_Network::IsServer() )
	{
		Asura_Assert( false, "Trying to create team score but this is not the host, invalid" ); 
		return; 
	}

	// Check if the team score for the requested team has been created already
	Asura_Assert( GetList().GetNumEntries() <= AVP_Settings::AVP_NUM_TEAMS, "Invalid number of teams in the team score list" ); 
	AVP_Multiplayer_TeamScore* pxTeamScore = FindTeamScore( eTeam );

	// Is there a team score for this team?
	if ( !pxTeamScore )
	{
		// No, create one
		pxTeamScore = new AVP_Multiplayer_TeamScore( eTeam ); 

		// Set the number of lives available per team if necessary
		if ( AVP_Multiplayer_GameRules::GetTeamLivesOn() )
		{
			pxTeamScore->SetNumberOfTeamLives( AVP_Multiplayer_GameRules::GetLivesAvailablePerTeam() ); 
		}

		// Add to the list
		GetList().AddToList( pxTeamScore ); 
	}
}

void AVP_Multiplayer_TeamScore::DestroyAll()
{
	Asura_List_It<AVP_Multiplayer_TeamScore> xTeamScoreListIt( s_xTeamScoreList ); 
	while ( !xTeamScoreListIt.Done() )
	{
		AVP_Multiplayer_TeamScore* pxTeamScore = xTeamScoreListIt.GetCurrent(); 
		delete pxTeamScore; 
		xTeamScoreListIt.DeleteCurrent();
	}
}

void AVP_Multiplayer_TeamScore::Update()
{
	// Team score information should only be stored in the server
	if ( !Asura_Network::IsServer() )
	{
		return; 
	}

	// Only do this if the game is team based
	if ( AVP_Settings::IsMultiPlayer() && AVP_Multiplayer_GameRules::IsTeamBasedGame() )
	{
		// Team scores
		int iTeamOneScore = 0; 
		int iTeamTwoScore = 0; 
		int iTeamThreeScore = 0; 
		AVP_Multiplayer_TeamScore* pxTeamScore = NULL;

		// Find score for team one
		pxTeamScore = AVP_Multiplayer_TeamScore::FindTeamScore( AVP_Settings::AVP_TEAM_ONE );
		if (pxTeamScore)
		{
			iTeamOneScore = pxTeamScore->GetTeamScore(); 
		}
		
		// Find score for team two
		pxTeamScore = AVP_Multiplayer_TeamScore::FindTeamScore( AVP_Settings::AVP_TEAM_TWO );
		if (pxTeamScore)
		{
			iTeamTwoScore = pxTeamScore->GetTeamScore(); 
		}

		// Find score for team three
		pxTeamScore = AVP_Multiplayer_TeamScore::FindTeamScore( AVP_Settings::AVP_TEAM_THREE );
		if (pxTeamScore)
		{
			iTeamThreeScore = pxTeamScore->GetTeamScore(); 
		}

		// Iterate through all players info and update the score for each team 
		Asura_Hashed_List_It<AVP_ServerEntity_PlayerInfo> xPlayerInfoIt( AVP_ServerEntity_PlayerInfo::GetList() ); 

		while ( !xPlayerInfoIt.Done() )
		{
			// Get the info about current player
			AVP_ServerEntity_PlayerInfo* pxPlayerInfo = xPlayerInfoIt.GetCurrent(); 
			
			// Update this player score for all teams
			AVP_Multiplayer_PlayerScore& xPlayerScore = pxPlayerInfo->GetPlayerScore(); 

			xPlayerScore.SetTeamOneScore( iTeamOneScore ); 
			xPlayerScore.SetTeamTwoScore( iTeamTwoScore ); 
			xPlayerScore.SetTeamThreeScore( iTeamThreeScore ); 

			xPlayerInfoIt.Next(); 
		}
	}
}

AVP_Multiplayer_TeamScore* AVP_Multiplayer_TeamScore::FindTeamScore( AVP_Settings::Team eTeam )
{
	Asura_List_It<AVP_Multiplayer_TeamScore> xTeamScoreListIt( GetList() ); 
	AVP_Multiplayer_TeamScore* pxFoundTeamScore = NULL; 

	// Iterate throught the score list
	while ( !xTeamScoreListIt.Done() )
	{
		AVP_Multiplayer_TeamScore* pxCurrentTeamScore = xTeamScoreListIt.GetCurrent(); 
		// Is this the score for our team?
		if ( pxCurrentTeamScore->GetTeam() == eTeam )
		{
			// Yes, found it
			Asura_Assertf( !pxFoundTeamScore, ( "Duplicated entry for team %d in team score list", eTeam )); 
			pxFoundTeamScore = pxCurrentTeamScore; 
		}
		xTeamScoreListIt.Next(); 
	}

	return pxFoundTeamScore; 
}