//////////////////////////////////////////////////////////////////////////////
//
// AVP_Leaderboards.h
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     platform-independent leaderboards code
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef AVP_LEADERBOARDS_H___
#define AVP_LEADERBOARDS_H___ 1

#ifdef ASURA_SUPPORT_LEADERBOARDS

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////
class AVP_Multiplayer_PlayerScore;

// E N U M S /////////////////////////////////////////////////////////////////
enum AVP_LEADERBOARD
{
	AVP_LEADERBOARD_KILLS,
	AVP_LEADERBOARD_SCORE,
	AVP_LEADERBOARD_MATCHES,
	AVP_LEADERBOARD_MYSTATS,
	AVP_LEADERBOARD_INTERNAL,
	AVP_NUM_LEADERBOARDS
};

enum AVP_KILLS_LEADERBOARD
{
	AVP_TOTAL_KILLS,
	AVP_KILLS_AS_MARINE,
	AVP_KILLS_AS_ALIEN,
	AVP_KILLS_AS_PREDATOR,
	AVP_KILLS_TO_DEATH_RATIO,
	AVP_KILLS_KILL_STREAK,
	AVP_KILLS_TOTAL_DEATHS,
	AVP_NUM_KILLS_STATS
};

enum AVP_SCORE_LEADERBOARD
{
	AVP_SCORE_TOTAL,
	AVP_SCORE_MARINE,
	AVP_SCORE_ALIEN,
	AVP_SCORE_PREDATOR,
	AVP_SCORE_TIME_PLAYED,
	AVP_NUM_SCORE_STATS
};

enum AVP_MATCHES_LEADERBOARD
{
	AVP_MATCHES_TOTAL,
	AVP_MATCHES_AS_MARINE,
	AVP_MATCHES_AS_ALIEN,
	AVP_MATCHES_AS_PREDATOR,
	AVP_MATCHES_WON,
	AVP_MATCHES_LOST,
	AVP_MATCHES_WINNING_STREAK,
	AVP_NUM_MATCHES_STATS
};

enum AVP_MYSTATS_LEADERBOARD
{
	AVP_MYSTATS_TOTAL_RANK,
	AVP_MYSTATS_RANK_MARINE,
	AVP_MYSTATS_RANK_ALIEN,
	AVP_MYSTATS_RANK_PREDATOR,
	AVP_MYSTATS_HEADSHOTS,
	AVP_MYSTATS_HEADBITES,
	AVP_MYSTATS_TROPHIES,
	AVP_NUM_MYSTATS_STATS
};

enum AVP_INTERNAL_LEADERBOARD
{
	AVP_INTERNAL_QUIT_EARLY_COUNT,
	AVP_NUM_INTERNAL_STATS
};

enum AVP_STAT_IDENTIFIER
{
	AVP_STATID_INVALID = 0,
	AVP_STATID_MIN = 1000,
	// start at 1000 to help with detecting if leaderboard column IDs are being mixed up with stat IDs
	AVP_STATID_TOTAL_KILLS = AVP_STATID_MIN,
	AVP_STATID_KILLS_AS_MARINE,
	AVP_STATID_KILLS_AS_ALIEN,
	AVP_STATID_KILLS_AS_PREDATOR,
	AVP_STATID_KILLS_KILL_STREAK,
	AVP_STATID_KILLS_TOTAL_DEATHS,
	AVP_STATID_SCORE_TOTAL,
	AVP_STATID_SCORE_MARINE,
	AVP_STATID_SCORE_ALIEN,
	AVP_STATID_SCORE_PREDATOR,
	AVP_STATID_SCORE_TIME_PLAYED,
	AVP_STATID_MATCHES_TOTAL,
	AVP_STATID_MATCHES_AS_MARINE,
	AVP_STATID_MATCHES_AS_ALIEN,
	AVP_STATID_MATCHES_AS_PREDATOR,
	AVP_STATID_MATCHES_WON,
	AVP_STATID_MATCHES_LOST,
	AVP_STATID_MYSTATS_TOTAL_RANK,
	AVP_STATID_MYSTATS_RANK_MARINE,
	AVP_STATID_MYSTATS_RANK_ALIEN,
	AVP_STATID_MYSTATS_RANK_PREDATOR,
	AVP_STATID_MYSTATS_HEADSHOTS,
	AVP_STATID_MYSTATS_HEADBITES,
	AVP_STATID_MYSTATS_TROPHIES,
	AVP_NUM_ADDITIVE_STATS,
	AVP_STATID_KILLS_TO_DEATH_RATIO = AVP_NUM_ADDITIVE_STATS,
	AVP_STATID_WINNING_STREAK,
	AVP_STATID_QUIT_EARLY_COUNT,
	AVP_STATID_MAX
};

// C L A S S E S /////////////////////////////////////////////////////////////
class AVP_Leaderboards
{
public:

	static void AddConsoleCmdsAndVars();

	// GetAssociatedStatID() returns the AVP_STAT_IDENTIFIER for a given leaderboard
	// and column (where 0 is the ranked stat column). 
	static u_int GetAssociatedStatID( u_int uLeaderboard, u_int uColumn );

	// GetStatResultById() returns results for the match which has just ended. It's up to other systems
	// to keep track of stats spanning multiple matches and to add up the stats for this match with 
	// totals from all previous matches.
	static u_int GetStatResultById( u_int uStatID, const AVP_Multiplayer_PlayerScore& xScore );

	// IsAdditiveStat() returns whether this stat is simply a running total for all matches, or whether
	// some additional calculation is needed (e.g. kill:death ratio).
	static bool IsAdditiveStat( u_int uStatID );
};

#endif // #ifdef ASURA_SUPPORT_LEADERBOARDS

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef AVP_LEADERBOARDS_H___
