///////////////////////////////////////////////////////////////////////////////
//
// File: AVP_Message_Compression.cpp
//
// Created by Martin G.Harris (5/2/2007)
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "Network/AVP_Message_Compression.h"

#include "Debugging/Asura_Assert.h"

// Constants
//static const float fMAX_VALUE_3_BITS = 8.0f;
//static const float fMAX_VALUE_5_BITS = 32.0f;
//static const float fMAX_VALUE_7_BITS = 128.0f;
static const float fMAX_VALUE_8_BITS = 256.0f;
//static const float fMAX_VALUE_9_BITS = 512.0f;
//static const float fMAX_VALUE_14_BITS = 16384.0f;
static const float fMAX_VALUE_16_BITS = 65536.0f;


//////////////////////////
// AVP_Compressed_Angle //
//////////////////////////
void AVP_Compressed_Angle::FromAngle(float fAngle)
{
	fAngle = Asura_Maths::RadiansToZeroMinimumRange(fAngle);
	fAngle *= (fMAX_VALUE_16_BITS / (Asura_Maths::Pi * 2.0f));
	m_usData = static_cast<unsigned short>(Asura_Maths::FloatToInt_Chop(fAngle));
}

void AVP_Compressed_Angle::ToAngle(float &fAngle) const
{
	fAngle = static_cast<float>(m_usData);
	fAngle /= (fMAX_VALUE_16_BITS / (Asura_Maths::Pi * 2.0f));
}

////////////////////////////////
// AVP_Compressed_Angle_Small //
////////////////////////////////
void AVP_Compressed_Angle_Small::FromAngle(float fAngle)
{
	fAngle = Asura_Maths::RadiansToZeroMinimumRange(fAngle);
	fAngle *= (fMAX_VALUE_8_BITS / (Asura_Maths::Pi * 2.0f));
	m_usData = static_cast<unsigned short>(Asura_Maths::FloatToInt_Chop(fAngle));
}

void AVP_Compressed_Angle_Small::ToAngle(float &fAngle) const
{
	fAngle = static_cast<float>(m_usData);
	fAngle /= (fMAX_VALUE_8_BITS / (Asura_Maths::Pi * 2.0f));
}

//////////////////////////
// AVP_Compressed_Float //
//////////////////////////
void AVP_Compressed_Float::FromFloat(float fValue, bool bAssertIfOutOfRange /*= true*/)
{
	const u_int uNumBins = GetNumBins();
	const float fRange = GetMax() - GetMin();

	Asura_Assert(uNumBins>2,  "AVP_Compressed_Float: Too few bins!");
	Asura_Assert(fRange>0.0f, "AVP_Compressed_Float: Range must be positive!");

	Asura_Assert((fValue>=GetMin() && fValue<=GetMax()) || !bAssertIfOutOfRange, "AVP_Compressed_Float: Float out of range!");

	if (fValue<=GetMin())
	{
		m_uData = 0;
	}
	else if (fValue>=GetMax())
	{
		m_uData = uNumBins - 1;
	}
	else
	{
		const float fFraction = (fValue - GetMin())/fRange;
		m_uData = Asura_Maths::FloatToInt_Chop(fFraction*(uNumBins - 2) + 1);
	}
}

void AVP_Compressed_Float::ToFloat(float& fValue) const
{
	const u_int uNumBins = GetNumBins();
	const float fRange = GetMax() - GetMin();

	Asura_Assert(m_uData<uNumBins, "AVP_Compressed_Float: Invalid bin number!");
	
	if (m_uData == 0)
	{
		fValue = GetMin();
	}
	else if (m_uData >= uNumBins - 1)
	{
		fValue = GetMax();
	}
	else
	{
		const float fFraction = (static_cast<float>(m_uData) - 0.5f)/(uNumBins-2);
		fValue = fFraction*fRange + GetMin();
	}
}
