
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Server.h
// Author : Andy Tate
// Date   : 5-Jul-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SERVER_H__
#define __AVP_SERVER_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_LinkList.h"

#include "Network/Asura_Guid.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Message;
class AVP_ServerEntity_PlayerInfo;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class AVP_Server
{
public:
	// Public functions...
	static void	MessageHandler(const Asura_Message* pxMessage);

	static bool	SendTerminationMessage(const int iReason, const Asura_Guid uTo, const Asura_Guid uFrom = ASURA_GUID_SERVER, const bool bGuaranteed = false);
	static bool	SendJoinResponseMessage(const Asura_Guid uPlayer);
	static bool	SendRestartMessage(void);
	static bool	SendRotateMessage(const Asura_Guid uTo = ASURA_GUID_ALL_OTHER_CLIENTMACHINES, const Asura_Guid uFrom = ASURA_GUID_SERVER);
	static bool	SendGameOverMessage(void);

	static void SetCanInitialiseClients(bool bCanInitialise) { s_bCanInitialiseClients = bCanInitialise; }
	static void UpdateClientInitialisation();
	static void ClearClientInitialisationLists();

	// Callback for Asura_Network's client creation functionality
	static void ClientCreateMessagesSent(Asura_Guid uClientGuid);

protected:
	// Protected variables...

	// Protected functions...
	static bool	ProcessJoinRequestMessage(const Asura_Message* pxMessage);
	static bool ProcessJoinReadyMessage(const Asura_Message* pxMessage); 
	static bool	ProcessPlayerConnectedMessage(const Asura_Message* pxMessage);
	static bool	ProcessPlayerDisconnectedMessage(const Asura_Message* pxMessage);
	static bool	ProcessEndOfLevelMessage(const Asura_Message* pxMessage);
	static bool ProcessConvertPlayerToAIMessage(const Asura_Message* pxMessage);
	static bool ProcessSetFormationDirection(const Asura_Message* pxMessage);

	// This currently handles both initial creation and respawning.
	static bool ProcessPlayerActorCreateMessage(const Asura_Message* pxMessage);

private:
	enum ClientAwaitingInitialisationState
	{
		AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEPLAYERINFO,
		AVP_CLIENTAWAITINGINITIALISATION_STATE_CREATEACTOR,
		AVP_CLIENTAWAITINGINITIALISATION_STATE_ADDTOINITIALISATIONLIST,
		AVP_CLIENTAWAITINGINITIALISATION_STATE_CLIENTINITIALISING,
		AVP_CLIENTAWAITINGINITIALISATION_STATE_WAITINGTOCONTINUE,
	};

	struct AwaitingInitialisationData
	{
		Asura_Guid m_uClientGuid;
		ClientAwaitingInitialisationState m_eInitialisationState;
		int m_iSpeciesToCreate;
		Asura_Hash_ID m_uBlueprintHashToCreate;
		int m_iRequestedMultiplayerTeam; 
		int m_iAssignedMultiplayerRole; 
		// TODO - more needs to go here.
	};

	// Private functions...
	static AVP_ServerEntity_PlayerInfo* CreatePlayerInfo(AwaitingInitialisationData* pxInitialisationData);
	static void CreatePlayerActor(AwaitingInitialisationData* pxInitialisationData, AVP_ServerEntity_PlayerInfo* pxPlayerInfo, bool bIsInactive = false);

	// Private variables...
	static Asura_LinkList<AwaitingInitialisationData*> s_xClientsInitialising;
	static float s_fClientInitialisationTimeout; // At this point, everyone who's managed to complete initialisation will be told to start anyway.
	static float s_fAllClientsInitialisedDelay; // A slight delay to allow create messages to reach all clients before allowing them to continue to the game.
	static bool s_bCanInitialiseClients;

};   // End of class 'AVP_Server'.



#endif   // ifndef __AVP_SERVER_H__


/// End of file. ///
