//////////////////////////////////////////////////////////////////////////////
//
// Asura_Project_Merit.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
// Description:
//
//     AVP implementations of Asura_Merit_System functions
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_MERIT_SYSTEM

// Asura includes
#include "Network/Online/Asura_Lobby.h"
#include "Network/Online/Asura_Merit.h"

// AVP includes
#include "Core/AVP_Settings.h"
#include "Entity/PlayerInfo/AVP_ClientEntity_PlayerInfo.h"

// C O N S T A N T S /////////////////////////////////////////////////////////
const int iAVP_MERIT_DISCONNECTION_PENALTY = 10;
//const int iAVP_MERIT_TEAM_WIN_BONUS = 20;

// C L A S S   I M P L E M E N T A T I O N ///////////////////////////////////
bool Asura_Merit_System::Project_GetPlayerData(Asura_Guid uGuid, bool bDisconnectedEarly, u_int &uTeam, int &iScore)
{
	// use the client entity player info (which should exist on the server and contain the same results)
	const AVP_ClientEntity_PlayerInfo* pxPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity(uGuid);
	Asura_Assert(pxPlayerInfo, "failed to get player info to write stats");
	if (!pxPlayerInfo)
	{
		return false;
	}

	const AVP_Multiplayer_PlayerScore& xScore = pxPlayerInfo->GetPlayerScore();

	iScore = xScore.GetIndividualScore();
	if (bDisconnectedEarly)
	{
		iScore -= iAVP_MERIT_DISCONNECTION_PENALTY;
	}

	// get team index. TODO: this might be better got from somewhere else...
	const Asura_Lobby* pxLobby = Asura_Lobby_System::GetLobby();
	Asura_Assert(pxLobby, "no lobby");
	if (pxLobby)
	{
		const Asura_Lobby_Player* pxPlayer = pxLobby->FindPlayer(uGuid);
		Asura_Assert(pxPlayer, "player not found in lobby");
		if (pxPlayer)
		{
			uTeam = pxPlayer->GetTeamIndex();
		}
	}

	return true;
}

bool Asura_Merit_System::Project_GetWinningTeam( u_int& uTeam, int& iBonus )
{
	// TODO - implement this
	return false;
}


#endif // #ifdef ASURA_USE_MERIT_SYSTEM
