//////////////////////////////////////////////////////////////////////////////
//
// Asura_Project_NetTransport_System.cpp
//
// Copyright (c) 2004 Rebellion. All rights reserved.
//
// Description:
//
//     DTR's implementations of project-specific ANTS functions
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header
#include "Network/Asura_NetTransport_System.h"
// Asura headers
#include "Debugging/Asura_Logfile.h"
#include "Network/Asura_NetSocket.h"

// project headers
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Messages.h"


#ifdef ASURA_SUPPORT_EXTERNAL_NETWORK

// change this for every build.
#define AVP_NETWORK_VERSION_ID (0)

u_short Asura_NetTransport_System::Project_GetVersionID( void )
{
	// This should change whenever the network messages become incompatible.
	// new formula uses units for AVP message count modulo 10 and tens for Asura message count modulo 10.
	// This makes the version change automatically when anyone adds a new message.
	return static_cast<u_short>(AVP_NETWORK_VERSION_ID * 100 + ((AVP_MSG_ID_MAX - ASURA_MSG_PROJECT_SPECIFIC) % 10) + ((ASURA_MSG_END_OF_ENGINE_MESSAGES % 10) * 10));
}


u_int Asura_NetTransport_System::Project_GetApplicationGuid( void )
{
	return 0xB025FFF3; // Asura_GetHashID("AliensVsPredator3")
}

void Asura_NetTransport_System::Project_RefreshSessionData( void )
{
	const u_int uNumBytes = sizeof(AVP_NetworkSession);
	if (s_pucSessionData)
	{
		Asura_Assert(s_uSessionDataSize == uNumBytes, "wrong session data size");
	}
	else
	{
		s_pucSessionData = new u_char[uNumBytes];
		s_uSessionDataSize = uNumBytes;
	}

	AVP_NetworkSession* pxSessionData = reinterpret_cast<AVP_NetworkSession*>(s_pucSessionData);

	pxSessionData->Set();
}

u_int Asura_NetTransport_System::Project_GetExtraConnectDataSize( void )
{
	return 0;
}

void Asura_NetTransport_System::Project_FillOutExtraConnectData( void* pData )
{
}

// returns whether connection is allowed and the NDR reason if not.
bool Asura_NetTransport_System::Project_CheckExtraConnectData( const void* pData, u_int& uDisconnectReason )
{
	uDisconnectReason = ASURA_NDR_NOTDISCONNECTED;
	return true;
}

void Asura_NetTransport_System::Project_ProcessConnection( Asura_NetAddress xAddress, u_short usPort, Asura_Guid uGuid, const void* pConnectData )
{
}

void Asura_NetTransport_System::Project_ProcessDisconnection( Asura_Guid uGuid )
{
	// KOR - done when machine is removed from game
	// OR - 04/08/2004 14:21:33 - Send disconnection message
/*	Asura_Message* pxMessage = Asura_Network::CreateMessage(ASURA_MSG_PLAYERDISCONNECTED,sizeof(Asura_MsgData_PlayerDisconnected));

	if (pxMessage)
	{
		Asura_MsgData_PlayerDisconnected* pxData = static_cast<Asura_MsgData_PlayerDisconnected*>(pxMessage->GetData());

		if (pxData)
		{
			pxData->PlayerGuid = uGuid;
			pxData->m_eReason = ASURA_NDR_UNKNOWN;
		}
		pxMessage->SetToGuid(ASURA_GUID_SERVER);
		pxMessage->SetFromGuid(uGuid);
		pxMessage->UseGuaranteedDelivery();
		Asura_Network::Send(pxMessage);
	}*/
}


// (MH - 19.07.2004) no need for this, unless we do XBox
void Asura_NetTransport_System::PlatformProject_UpdateInternetSession(void)
{
}

bool Asura_NetTransport_System::Project_ShouldAddMessageToQueue(u_int uMsgID)
{
	return true;
}

#endif // #ifdef ASURA_SUPPORT_EXTERNAL_NETWORK
