
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Physics_OBBTests.cpp
// Author : Andy Tate
// Date   : 7-Feb-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Physics/AVP_Physics_OBBTests.h"

// Asura independent header-files...
#include "Core/Memory/Asura_Mem.h"
#include "Physics/Asura_Physics_CCD.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

u_int AVP_Physics_OBBTests::ClipPolyToPlane( Asura_AVector3* pxInVertices, u_int uNumberOfVertices, Asura_AVector3* pxOutVertices, Asura_AVector3& xPlaneNormal, float fPlaneConstant )
{
	// KJFL - 10 July 2006 - I've taken this fn direct from the clipper I wrote for the decal system.
	// If it works out ok I'll refactor it to an Asura_Maths fn or somesuch - it may prove too costly to
	// use in this manner.

	u_int uNewVertexCount = 0;

	float fDot[9];

	{
		bool bAllNegative = true;
		for (u_int u=0; u<uNumberOfVertices; u++)
		{
			fDot[u] = pxInVertices[u]*xPlaneNormal + fPlaneConstant;
			if (fDot[u]>=0.0f)
			{
				bAllNegative = false;
			}
		}

		/// Kevin Lea - 14/11/2002 10:49:43 - early exit if all vertices would be clipped
		if (bAllNegative) return 0;
	}

	for (u_int u=0; u<uNumberOfVertices; u++)
	{
		u_int v = (u!=0) ? u-1 : uNumberOfVertices-1; // previous vertex

		if (fDot[u]<0.0f)
		{
			if (fDot[v]>=0.0f)
			{
				// Current vertex is on negative side of plane,
				// but previous vertex is on positive side.
				Asura_AVector3 xEdge = pxInVertices[v] - pxInVertices[u];
				float f = xEdge * xPlaneNormal;

				Asura_Assert( f!=0.0f, "edge parallel to plane");

				float fLambda = fDot[v] / f;

				// Asura_Assertf( fLambda >= -0.0001f && fLambda <= 1.0001f, ("Out of bounds: %f",Asura_Maths::FtoD(fLambda)));
				if (fLambda < 0.0f) fLambda = 0.0f;
				if (fLambda > 1.0f) fLambda = 1.0f;

				pxOutVertices[uNewVertexCount] = pxInVertices[v] - xEdge * fLambda;
				uNewVertexCount++;	
			}
		}
		else
		{
			if (fDot[v]<0.0f)
			{
				// Current vertex is on positive side of plane,
				// but previous vertex is on negative side.
				const Asura_AVector3 xEdge = pxInVertices[u] - pxInVertices[v];
				float f = xEdge * xPlaneNormal;

				Asura_Assert( f!=0.0f, "edge parallel to plane");

				float fLambda = fDot[u] / f;

				// Asura_Assertf( fLambda >= -0.0001f && fLambda <= 1.0001f, ("Out of bounds: %f",Asura_Maths::FtoD(fLambda)));
				if (fLambda < 0.0f) fLambda = 0.0f;
				if (fLambda > 1.0f) fLambda = 1.0f;

				pxOutVertices[uNewVertexCount] = pxInVertices[u] - xEdge * fLambda;
				uNewVertexCount++;	
			}

			/// Kevin Lea - 14/11/2002 11:22:04 - include current vertex
			pxOutVertices[uNewVertexCount] = pxInVertices[u];
			uNewVertexCount++;
		}
	}

	return uNewVertexCount;
}

u_int AVP_Physics_OBBTests::ClipEdgeToPlane( Asura_AVector3* pxInVertices, Asura_AVector3* pxOutVertices, Asura_AVector3& xPlaneNormal, float fPlaneConstant )
{
	// modified version of ClipPolyToPlane to just deal with just edges (2 vert line segments)

	float fDot0 = pxInVertices[0]*xPlaneNormal + fPlaneConstant;
	float fDot1 = pxInVertices[1]*xPlaneNormal + fPlaneConstant;

	bool bVert0Behind = (fDot0 < 0.0f);
	bool bVert1Behind = (fDot1 < 0.0f);

	if ( bVert0Behind && bVert1Behind )
	{
		// both behind plane - edge completely culled
		return 0;
	}
	else if ( bVert0Behind )
	{
		// vertex 0 is on negative side of plane, but previous vertex 1 is on positive side.

		Asura_AVector3 xEdge = pxInVertices[1] - pxInVertices[0];
		float f = xEdge * xPlaneNormal;

		Asura_Assert( f!=0.0f, "edge parallel to plane");

		float fLambda = fDot1 / f;

		// Asura_Assertf( fLambda >= -0.0001f && fLambda <= 1.0001f, ("Out of bounds: %f",Asura_Maths::FtoD(fLambda)));
		if (fLambda < 0.0f) fLambda = 0.0f;
		if (fLambda > 1.0f) fLambda = 1.0f;

		pxOutVertices[0] = pxInVertices[1] - xEdge * fLambda;

		pxOutVertices[1] = pxInVertices[1];

		return 2;

	}
	else if (bVert1Behind )
	{
		Asura_AVector3 xEdge = pxInVertices[0] - pxInVertices[1];
		float f = xEdge * xPlaneNormal;

		Asura_Assert( f!=0.0f, "edge parallel to plane");

		float fLambda = fDot0 / f;

		// Asura_Assertf( fLambda >= -0.0001f && fLambda <= 1.0001f, ("Out of bounds: %f",Asura_Maths::FtoD(fLambda)));
		if (fLambda < 0.0f) fLambda = 0.0f;
		if (fLambda > 1.0f) fLambda = 1.0f;

		pxOutVertices[0] = pxInVertices[0];

		pxOutVertices[1] = pxInVertices[0] - xEdge * fLambda;

		return 2;
	}
	else
	{
		pxOutVertices[0] = pxInVertices[0];
		pxOutVertices[1] = pxInVertices[1];
		return 2;
	}
}

u_int AVP_Physics_OBBTests::ClipPolyToPoly( const Asura_AVector3* pxClipperVertices, const u_int uNumberOfClipperVertices, const Asura_AVector3* pxClippeeVertices, u_int uNumberOfClippeeVertices, Asura_AVector3* pxClippedVertices )
{
	// need at least 3 verts in the clipper poly to make a normal (e.g. we need a tri at least, not an edge)
	if ( uNumberOfClipperVertices < 3 )
	{
		Asura_Assert( false, "Too few verts in clipper poly" );
		return 0;
	}

	// and the clippee needs to be a poly
	if ( uNumberOfClippeeVertices < 3 )
	{
		Asura_Assert( false, "Too few verts in clippee poly" );
		return 0;
	}

	Asura_AVector3 xClipperNormal = (pxClipperVertices[1] - pxClipperVertices[0])^(pxClipperVertices[2] - pxClipperVertices[0]);
	float fMagnitude = xClipperNormal.NormaliseReturningMagnitude();

	if (fMagnitude == 0.0f)
	{
		Asura_Assert( false, "Clipper poly is degenerate" );
		return 0;
	}

	u_int uNumberOfClippedVertices = uNumberOfClippeeVertices;

	// need assert on max clipped verts
	Asura_Mem::Copy( pxClippedVertices, pxClippeeVertices, sizeof( Asura_AVector3 ) * uNumberOfClippeeVertices );

	Asura_AVector3 axBuffer[10];


	u_int uStart = uNumberOfClipperVertices - 1;

	Asura_AVector3 xCentre = pxClipperVertices[0] + pxClipperVertices[1] + pxClipperVertices[2] + pxClipperVertices[3];

	xCentre *=0.25f;

	for ( u_int uEnd = 0; uEnd < uNumberOfClipperVertices; uStart = uEnd, ++uEnd )
	{
		Asura_AVector3 xEdgeStart = pxClipperVertices[ uStart ];
		Asura_AVector3 xEdgeEnd = pxClipperVertices[ uEnd ];

		Asura_AVector3 xPlaneNormal = ( xEdgeEnd - xEdgeStart ) ^ xClipperNormal;


		// probably unused variable when assertions off; may not need to normalise anyway
#if defined(ASURA_ASSERT)
		float fMagnitude =
#endif	// defined(ASURA_ASSERT)
			xPlaneNormal.NormaliseReturningMagnitude();

		Asura_Assert( fMagnitude != 0, "edge parallel to normal!" );

		float fPlaneConstant = -xEdgeStart * xPlaneNormal;


		if ( xCentre*xPlaneNormal + fPlaneConstant < 0.0f )
		{
			xPlaneNormal = -xPlaneNormal;
			fPlaneConstant = -fPlaneConstant;
		}
		if ( xCentre*xPlaneNormal + fPlaneConstant < 0.0f )
		{
			return 0;
		}

		uNumberOfClippedVertices = ClipPolyToPlane( pxClippedVertices, uNumberOfClippedVertices, axBuffer, xPlaneNormal, fPlaneConstant );

		if (uNumberOfClippedVertices > 8)
		{
			return 0;
		}

		// need assert on max clipped verts
		Asura_Mem::Copy( pxClippedVertices, axBuffer, sizeof( Asura_AVector3 ) * uNumberOfClippedVertices );

		if ( uNumberOfClippedVertices == 0 ) return 0;
	}

	return uNumberOfClippedVertices;
}

u_int AVP_Physics_OBBTests::ClipEdgeToPoly( const Asura_AVector3* pxClipperVertices, const u_int uNumberOfClipperVertices, const Asura_AVector3* pxClippeeVertices, Asura_AVector3* pxClippedVertices )
{
	u_int uNumberOfClippeeVertices = 2;

	// need at least 3 verts in the clipper poly to make a normal (e.g. we need a tri at least, not an edge)
	if ( uNumberOfClipperVertices < 3 )
	{
		Asura_Assert( false, "Too few verts in clipper poly" );
		return 0;
	}

	Asura_AVector3 xClipperNormal = (pxClipperVertices[1] - pxClipperVertices[0])^(pxClipperVertices[2] - pxClipperVertices[0]);
	xClipperNormal.Normalise();

	u_int uNumberOfClippedVertices = uNumberOfClippeeVertices;

	// need assert on max clipped verts
	Asura_Mem::Copy( pxClippedVertices, pxClippeeVertices, sizeof( Asura_AVector3 ) * uNumberOfClippeeVertices );

	Asura_AVector3 axBuffer[2];


	u_int uStart = uNumberOfClipperVertices - 1;

	Asura_AVector3 xCentre = pxClipperVertices[0] + pxClipperVertices[1] + pxClipperVertices[2] + pxClipperVertices[3];

	xCentre *=0.25f;

	for ( u_int uEnd = 0; uEnd < uNumberOfClipperVertices; uStart = uEnd, ++uEnd )
	{
		Asura_AVector3 xEdgeStart = pxClipperVertices[ uStart ];
		Asura_AVector3 xEdgeEnd = pxClipperVertices[ uEnd ];

		Asura_AVector3 xPlaneNormal = ( xEdgeEnd - xEdgeStart ) ^ xClipperNormal;


		// probably unused variable when assertions off; may not need to normalise anyway
#if defined(ASURA_ASSERT)
		float fMagnitude =
#endif	// defined(ASURA_ASSERT)
			xPlaneNormal.NormaliseReturningMagnitude();

		Asura_Assert( fMagnitude != 0, "edge parallel to normal!" );

		float fPlaneConstant = -xEdgeStart * xPlaneNormal;


		if ( xCentre*xPlaneNormal + fPlaneConstant < 0.0f )
		{
			xPlaneNormal = -xPlaneNormal;
			fPlaneConstant = -fPlaneConstant;
		}
		if ( xCentre*xPlaneNormal + fPlaneConstant < 0.0f )
		{
			return 0;
		}

		uNumberOfClippedVertices = ClipEdgeToPlane( pxClippedVertices, axBuffer, xPlaneNormal, fPlaneConstant );

		if ( uNumberOfClippedVertices == 0 )
		{
			return 0;
		}

		Asura_Mem::Copy( pxClippedVertices, axBuffer, sizeof( Asura_AVector3 ) * uNumberOfClippedVertices );

	}

	return uNumberOfClippedVertices;
}

void AVP_Physics_OBBTests::ProjectVertexOntoEdge( const Asura_AVector3& xVertex, const Asura_AVector3* pxEdge, Asura_AVector3& xProjectedVertex )
{
	const Asura_AVector3 xAlongEdge = pxEdge[1] - pxEdge[0];
	const Asura_AVector3 xEdgeToVertex = xVertex - pxEdge[0];

	const float fLengthOfEdgeSquared = xAlongEdge.MagnitudeSquared();

	if ( fLengthOfEdgeSquared > 0.0f )
	{
		// find the distance along the edge:
		float fLambda = ( xAlongEdge * xEdgeToVertex ) / fLengthOfEdgeSquared;

		fLambda = Asura_Maths::ClampToRange( fLambda, 0.0f, 1.0f );

		xProjectedVertex = pxEdge[0] + xAlongEdge * fLambda;
	}
	else
	{
		xProjectedVertex = pxEdge[0];
	}
}

void AVP_Physics_OBBTests::ProjectEdgeOntoEdge( const Asura_AVector3* pxEdgeA, const Asura_AVector3* pxEdgeB, Asura_AVector3& xProjectedVertexA, Asura_AVector3& xProjectedVertexB )
{
	const Asura_AVector3 xAlongEdgeA = pxEdgeA[1] - pxEdgeA[0];
	const Asura_AVector3 xAlongEdgeB = pxEdgeB[1] - pxEdgeB[0];

	const Asura_AVector3 xBetweenEdges = xAlongEdgeA ^ xAlongEdgeB;

	const Asura_AVector3 xM = xBetweenEdges ^ xAlongEdgeB;

	float fLambda = ( (pxEdgeB[0] - pxEdgeA[0]) * xM ) / ( xAlongEdgeA * xM );

	fLambda = Asura_Maths::ClampToRange( fLambda, 0.0f, 1.0f );

	xProjectedVertexA = pxEdgeA[0] + xAlongEdgeA * fLambda;

	ProjectVertexOntoEdge( xProjectedVertexA, pxEdgeB, xProjectedVertexB );
}

void AVP_Physics_OBBTests::ProjectVertexOntoFace( const Asura_AVector3& xVertex, const Asura_AVector3* pxFace, Asura_AVector3& xProjectedVertex )
{
	Asura_AVector3 xFaceNormal = (pxFace[1] - pxFace[0])^(pxFace[2] - pxFace[0]);

	// hmm - check maths, can I drop the normalise?
	xFaceNormal.Normalise();

	float fDistanceFromFace = (xVertex - pxFace[0]) * xFaceNormal;

	xProjectedVertex = xVertex - xFaceNormal * fDistanceFromFace;
}

u_int AVP_Physics_OBBTests::ProjectEdgeOntoFace( const Asura_AVector3* pxEdge, const Asura_AVector3* pxFace, Asura_AVector3* pxProjectedVerticesEdge, Asura_AVector3* pxProjectedVerticesFace, u_int uNumberOfVertsInFace )
{
	// take the edge and clip it with the planes made by the edges of the face and the face normal

	u_int uNumberOfPoints = ClipEdgeToPoly( pxFace, uNumberOfVertsInFace, pxEdge, pxProjectedVerticesEdge );

	if (uNumberOfPoints < 2)
	{
		//		Asura_Assert( false, "Edge completely clipped?" );
		return 0;
	}
	Asura_Assert( uNumberOfPoints == 2, "Code Bug: Not an Edge any more" );


	for ( u_int u=0; u<uNumberOfPoints; ++u )
	{
		ProjectVertexOntoFace( pxProjectedVerticesEdge[u], pxFace, pxProjectedVerticesFace[u] );
	}

	return uNumberOfPoints;
}

u_int AVP_Physics_OBBTests::ProjectFaceOntoFace( const Asura_AVector3* pxFaceA, const Asura_AVector3* pxFaceB, Asura_AVector3* pxProjectedVerticesFaceA, Asura_AVector3* pxProjectedVerticesFaceB, u_int uNumberOfVertsInFaceA, u_int uNumberOfVertsInFaceB )
{
	u_int uNumberOfPoints = ClipPolyToPoly( pxFaceA, uNumberOfVertsInFaceA, pxFaceB, uNumberOfVertsInFaceB, pxProjectedVerticesFaceB );

	if (uNumberOfPoints <= 2)
	{
		//		Asura_Assert( false, "Face completely clipped?" );
		return 0;
	}

	for ( u_int u=0; u<uNumberOfPoints; ++u )
	{
		ProjectVertexOntoFace( pxProjectedVerticesFaceB[u], pxFaceA, pxProjectedVerticesFaceA[u] );
	}


	return uNumberOfPoints;
}


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

u_int AVP_Physics_OBBTests::FindContactPoints_OBBTri(const Asura_OBB& xOBB,
													 const Asura_Physics_Tri& xTri,
													 const Asura_AVector3& xAxis,
													 Asura_AVector3* pxContactPoints)
{
	Asura_AVector3 pxContactPointsForB[10];

	// Find the support features for both objects. This can be a face (4 verts), and edge (2 verts) or just a single vertex
	Asura_AVector3 axSupportPointsForA[4];
	Asura_AVector3 axSupportPointsForB[4];

	u_int uNumberOfSupportPointsForA = xOBB.GetSupportFeature( axSupportPointsForA, xAxis );
	u_int uNumberOfSupportPointsForB = xTri.GetSupportFeature( axSupportPointsForB, -xAxis );

	// We have 9 possible combinations. Might be able to minimise tests, but easiest if we do it as switches
	switch( uNumberOfSupportPointsForA )
	{
		default:
		{
			Asura_Assert( false, "Unhandled number of support points" );
			return 0;
		}
		case 1: // vertex
		{
			// all these contacts will just be one vertex, and for A it's the support point
			pxContactPoints[0] = axSupportPointsForA[0];
			return 1;
		}
		case 2: // edge
		{
			switch( uNumberOfSupportPointsForB )
			{
				default:
				{
					Asura_Assert( false, "Unhandled number of support points" );
					return 0;
				}
				case 1: // vertex-edge case
				{
					pxContactPointsForB[0] = axSupportPointsForB[0];
					ProjectVertexOntoEdge( axSupportPointsForB[0], axSupportPointsForA, pxContactPoints[0] );
					return 1;
				}
				case 2: // edge-edge case
				{
					ProjectEdgeOntoEdge( axSupportPointsForA, axSupportPointsForB, pxContactPoints[0], pxContactPointsForB[0] );
					return 1;
				}
				case 3: // edge-face case
				{
					return ProjectEdgeOntoFace( axSupportPointsForA, axSupportPointsForB, pxContactPoints, pxContactPointsForB, 3 );

				}
			}
		}
		case 4:
		{
			switch( uNumberOfSupportPointsForB )
			{
				default:
				{
					Asura_Assert( false, "Unhandled number of support points" );
					return 0;
				}
				case 1: // vertex-face case
				{
					pxContactPointsForB[0] = axSupportPointsForB[0];
					ProjectVertexOntoFace( axSupportPointsForB[0], axSupportPointsForA, pxContactPoints[0] );
					return 1;
				}
				case 2: // edge-face case
				{
					return ProjectEdgeOntoFace( axSupportPointsForB, axSupportPointsForA, pxContactPointsForB, pxContactPoints, 4 );

				}
				case 3: // face-face case
				{
					u_int uVerts =  ProjectFaceOntoFace( axSupportPointsForA, axSupportPointsForB, pxContactPoints, pxContactPointsForB, 4, 3 );

					Asura_AVector3 xFaceNormal = (axSupportPointsForA[1] - axSupportPointsForA[0])^(axSupportPointsForA[2] - axSupportPointsForA[0]);
					xFaceNormal.Normalise();

					for (u_int uV=0; uV<uVerts; ++uV)
					{
						#if defined ASURA_ASSERT
							float f = ( pxContactPoints[uV] - axSupportPointsForA[0] )* xFaceNormal;
						#endif

						Asura_Assert( Asura_Maths::Abs(f) < 0.01f, "Code bug: projection error" );
					}

					return uVerts;
				}
			}
		}
	}
}


/// End of file. ///
