
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_SurfaceReorientationNormalInterpolator.h
// Author : Andy Tate
// Date   : 7-Feb-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __AVP_SURFACEREORIENTATIONNORMALINTERPOLATOR_H__
#define __AVP_SURFACEREORIENTATIONNORMALINTERPOLATOR_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_Maths.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_CalculateSurfaceReorientationNormal;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : AVP_SurfaceReorientationNormalInterpolator
// Author       : Andy Tate
// Date         : 7-Feb-2008
//
// Description  : Abstract class used to interpolate to a desired normal during
//				  surface-reorientation.
////////////////////////////////////////////////////////////////////////////////

class AVP_SurfaceReorientationNormalInterpolator
{
public:
	// Public functions...
	virtual ~AVP_SurfaceReorientationNormalInterpolator()	{ delete(m_pxNext); }

	// Virtual functions...
	virtual void Reset(Asura_AVector3_InParam xNormal)
	{
		if(m_pxNext != NULL)
		{
			m_pxNext->Reset(xNormal);
		}
	}

	// Abstract functions...
	virtual	void	Update(const float fTimeStep, Asura_AVector3_InParam xDesiredNormal, Asura_AVector3_InOutParam xCurrentNormal)	= 0;
	virtual	void	Update(const float fTimeStep, const AVP_CalculateSurfaceReorientationNormal& xNormalCalculation, Asura_AVector3_InOutParam xCurrentNormal)	= 0;

	// Accessor functions...
	void SetNext(AVP_SurfaceReorientationNormalInterpolator* pxNext)
	{
		delete(m_pxNext);
		m_pxNext	= pxNext;
	}

	AVP_SurfaceReorientationNormalInterpolator* GetNext(void) const
	{
		return(m_pxNext);
	}

	void			SetDesiredRotationAxis( Asura_AVector3_InParam xDesiredRotationAxis ) { m_xDesiredRotationAxis = xDesiredRotationAxis; }

protected:
	// Protected variables...
	AVP_SurfaceReorientationNormalInterpolator*	m_pxNext;	// A pointer to the next surface-reorientation interpolator to use.

	Asura_AVector3		m_xDesiredRotationAxis;

	// Protected functions...
	AVP_SurfaceReorientationNormalInterpolator(void) : m_pxNext(NULL), m_xDesiredRotationAxis( 0.0f, 0.0f, 1.0f ) {}

private:
	// Private variables...

	// Private functions...
	AVP_SurfaceReorientationNormalInterpolator(const AVP_SurfaceReorientationNormalInterpolator& xOther);

	AVP_SurfaceReorientationNormalInterpolator&	operator=(const AVP_SurfaceReorientationNormalInterpolator& xOther);

};   // End of class 'AVP_SurfaceReorientationNormalInterpolator'.


#endif   // ifndef __AVP_SURFACEREORIENTATIONNORMALINTERPOLATOR_H__


/// End of file. ///
