//////////////////////////////////////////////////////////////////////////////
//
// AVP_Dialogue_System.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
// Description:
//
//     handles AVP specific code wrt dialogue
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Blueprint/Asura_Blueprint_System.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Entity/Asura_Entity.h"
#include "Sound/Asura_Dialogue.h"
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_Audio_Voice.h"
#include "Sound/Asura_Dialogue_System.h"
#include "Sound/Asura_Sound_Mixer.h"
#ifdef ASURA_TOOLS
#include "Sound/Tools_Sound_System.h"
#endif
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/AVP_Settings.h"
#include "Sound/AVP_Dialogue_System.h"

static float s_fRadioCutoffDistance = 5.0f;

void AVP_Dialogue_System::Initialise(void)
{
	Asura_Dialogue_System::SetProjectVoiceCreateEvent(ProjectVoiceCreateEvent);

	Asura_Blueprint *pxBp = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_PLAYERS, 0x90E64F23); // Asura_Hash_ID("MarinePlayer")
	Asura_Value xValue;

	if (pxBp && pxBp->GetParameterValueByHashID(0x4197F16C, xValue)) // Asura_Hash_ID("RadioMinDistance")
	{
		s_fRadioCutoffDistance  = xValue.GetFloatValue();
	} else {
		s_fRadioCutoffDistance  = 5.0f;
	}
}

void AVP_Dialogue_System::Shutdown(void)
{
	Asura_Dialogue_System::SetProjectVoiceCreateEvent(NULL);
}

Asura_Handle AVP_Dialogue_System::CreateRadioDialogueVoice(const Asura_Dialogue_Line& xLine, Asura_Guid uSpeakerGuid, float fVolumeScale)
{
	Asura_Handle xSoundSystemHandle;
	const Asura_Dialogue_Line_Template* pxTemplate = xLine.GetTemplate();
	Asura_Assert(pxTemplate, "no dialogue template");
	if (!pxTemplate)
	{
		return ASURA_INVALID_HANDLE;
	}

#ifdef ASURA_USE_AUDIO_SYSTEM
	const bool bStreamed = (!pxTemplate || pxTemplate->m_bStreamed);

	Asura_Hash_ID uWave = xLine.GetSoundHash();

#ifdef ASURA_TOOLS
	if(bStreamed)
	{
		if(!Asura_Audio_System::GetStreamInfoTree().GetDataByHash(uWave))
		{
			const char* const szFilename = xLine.GetSoundFilename();

			Asura_Audio_StreamInfo* const pxInfo =
				Asura_Audio_System::AddStreamInfo(szFilename,szFilename,0,false);

			if(pxInfo)
			{
				// Ensure this won't get exported with normal streaming sounds
				pxInfo->m_bDialogue = true;
			}
		}
	}
	else
	{
		Asura_Audio_System::LoadWave(xLine.GetSoundFilename(),false);
	}
#endif
#else
	Asura_Sound_ID eSoundID = (Asura_Sound_ID)xLine.GetSoundID();
#endif

#ifdef ASURA_TOOLS
	if(Tools_Sound_System::IsDontLoadWavs())
	{
#ifdef ASURA_USE_AUDIO_SYSTEM
		uWave = ASURA_HASH_ID_UNSET;
#else
		eSoundID = ASURA_SID_NO_SOUND;
#endif
	}
#endif

#if defined(ASURA_USE_AUDIO_SYSTEM)
	xSoundSystemHandle = bStreamed
		? Asura_Audio_System::CreateStreamingVoice(uWave, true, false,0.0f,  Asura_GetHashID("Radio Effect Chain"))
		: Asura_Audio_System::CreateVoice(uWave,true,false, Asura_GetHashID("Radio Effect Chain"));

#else
	xSoundSystemHandle = Asura_Sound_System::CreateSound(eSoundID);
#endif

	if (xSoundSystemHandle != ASURA_INVALID_HANDLE)
	{
#if defined ASURA_USE_AUDIO_SYSTEM
		Asura_Audio_Voice* const pxVoice = Asura_Audio_System::GetVoice(xSoundSystemHandle);
#endif

		if(pxTemplate)
		{
			// AUDIOTODO: support all these properties
#if defined ASURA_USE_AUDIO_SYSTEM

			if(pxVoice)
			{
				if(pxTemplate->GetMixerChannel())
				{
					pxVoice->SetMixerChannel(pxTemplate->GetMixerChannel()->GetHashID());
				}

//				const bool bApplyReverb = Asura_Sound_Mixer_System::GetMixerUseReverb(pxTemplate->GetMixerChannel());
//				pxVoice->SetUsingReverb(bApplyReverb);

// 				pxVoice->SetUsingOcclusion( bUseOcclusion );
// 
// 				pxVoice->SetUsingDistanceFiltering( bUseDistanceFiltering );

				const bool bIgnoreGameSpeedPitch = Asura_Sound_Mixer_System::IsIgnoreGameSpeedPitch( pxTemplate->GetMixerChannel() );
				pxVoice->SetPitchUsesTimescale( !bIgnoreGameSpeedPitch );
			}

#else

			// JAT 27/02/07 enable/disable game speed pitch scaling based on the flag in the mixer 
			const bool bIgnoreGameSpeedPitch = Asura_Sound_Mixer_System::IsIgnoreGameSpeedPitch( pxTemplate->GetMixerChannel() );
			if ( bIgnoreGameSpeedPitch == true )
			{
				Asura_Sound_System::SetSoundPitchUsesTimeScale( m_iSoundSystemHandle, !bIgnoreGameSpeedPitch );
			}

			// Reverb & distance filtering default to on during Asura_Sound_System::CreateSound()
			const bool bApplyReverb = Asura_Sound_Mixer_System::GetMixerUseReverb( pxTemplate->GetMixerChannel() );
			if ( bApplyReverb == false )
			{
				Asura_Sound_System::SetSoundUsesReverb( m_iSoundSystemHandle, bApplyReverb );
			}

			// Reverb & distance filtering default to on during Asura_Sound_System::CreateSound()
			const bool bDistanceFilter = Asura_Sound_Mixer_System::GetMixerUseDistanceFilter( pxTemplate->GetMixerChannel() );
			if ( bDistanceFilter == false )
			{
				Asura_Sound_System::SetSoundUsesDistanceFilter( m_iSoundSystemHandle, bDistanceFilter );
			}

			const bool bDoppler = Asura_Sound_Mixer_System::GetMixerUseDoppler( pxTemplate->GetMixerChannel() );
			if ( bDoppler == false )
			{
				Asura_Sound_System::SetSoundUsesDoppler( m_iSoundSystemHandle, bDoppler );
			}

			const bool bOcclusion = Asura_Sound_Mixer_System::GetMixerUseOcclusion( pxTemplate->GetMixerChannel() );
			if ( bOcclusion == false )
			{
				Asura_Sound_System::SetSoundUsesOcclusion( m_iSoundSystemHandle, bOcclusion );
			}

#endif // !defined ASURA_USE_AUDIO_SYSTEM
		}
	}
	return xSoundSystemHandle;
}


Asura_Handle AVP_Dialogue_System::ProjectVoiceCreateEvent(const Asura_Dialogue_Line& xLine, Asura_Guid uSpeakerGuid, float fVolumeScale)
{
	if (AVP_Settings::GetPlayerSpecies() == AVP_Settings::AVP_SPECIES_HUMAN)
	{
		const Asura_Entity_Client* pxEntity = FindEntity(uSpeakerGuid);
		if (pxEntity)
		{
			Asura_Vector_3 xPosition;
			if (pxEntity->GetPositionForDialogue(xPosition))
			{
				Asura_Vector_3 xListenerPos;
				Asura_Audio_System::GetListenerPosition(&xListenerPos, NULL);
				if ((xPosition - xListenerPos).MagnitudeSquared() > s_fRadioCutoffDistance  * s_fRadioCutoffDistance )
				{
					return CreateRadioDialogueVoice(xLine, uSpeakerGuid, fVolumeScale);
				}
			}
		}
		else
		{
			// Do something for cutscenes?
// 			const Asura_Cutscene_Actor* pxActor = FindActor(uSpeakerGuid);
// 
// 			if (pxActor)
// 			{
// 			}
		}
	}
	return ASURA_INVALID_HANDLE;
}


const Asura_Entity_Client* AVP_Dialogue_System::FindEntity( Asura_Guid uSpeakerGuid)
{
	//check for a client entity
	const Asura_Entity_Client* pxClient = Asura_Entity_Client::FindEntity(uSpeakerGuid);

	if (!pxClient || !pxClient->GetCanPlayDialogue())
	{
		//Not present or not able to play dialogue (by default, the latter means it's a dead physical object).
		//return NULL in either case so that the dialogue will end up stopping if it already started.
		return NULL;
	}

	return pxClient;
}

const Asura_Cutscene_Actor* AVP_Dialogue_System::FindActor(Asura_Guid uSpeakerGuid)
{
	// JAT 27/09/07 - this will find the actor in any cutscene (active or not)
	const Asura_Cutscene_Actor* pxActor = Asura_Cutscene_System::FindCutsceneActor( uSpeakerGuid );
	return pxActor;
}



#endif // ASURA_USE_DIALOGUE_SYSTEM

