/**
*
* AVP_GlowOutlines.cpp
*
* Created by Andrew Southgate - 30/7/2008
*
* Copyright (C) 2007 Rebellion, All Rights Reserved.
*
**/

/************/
/* Includes */
/************/

// main asura header
#include "Core/Asura.h"

// local header
#include "SpecialFX/AVP_GlowOutlines.h"

// independent Asura
#include "Animation/Asura_Animation.h"
#include "Shapes/Asura_Shapes.h"
//#include "Rendering/Asura_Font_System.h"
#include "Interface/Asura_CommandConsole.h"

#include "Core/AVP_Settings.h"

float												AVP_GlowOutlines::GLOW_RADIUS = 15.0f;
float												AVP_GlowOutlines::GLOW_MAX_DISTANCE = 15.0f;
float												AVP_GlowOutlines::GLOW_MAX_DISTANCE_ALIEN = 15.0f;
float												AVP_GlowOutlines::GLOW_MAX_DISTANCE_PREDATOR = 30.0f;



AVP_GlowOutlines::DepthSortedSkinEntry				AVP_GlowOutlines::s_axSortedSkinentries[MAX_NUM_SORTED_SKINS];
AVP_GlowOutlines::DepthSortedSkinEntry *			AVP_GlowOutlines::s_axDepthSortedSkins[NUM_DEPTH_SORT_SLOTS];
int													AVP_GlowOutlines::s_uCurrentSortedSkin = 0;

DVSVAR bool dvs_bEnablePheromones = true;

void AVP_GlowOutlines::Initialise() 
{ 
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	if (!Asura_CommandConsole::VarExists(L"AVP.Pheromone.Enable"))
	{
		Asura_CommandConsole::AddVar(L"AVP.Pheromone.Enable", &dvs_bEnablePheromones, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
		Asura_CommandConsole::AddVar(L"AVP.Pheromone.Glow.GlowSize", &GLOW_RADIUS, 0.5f, 500.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);	
		Asura_CommandConsole::AddVar(L"AVP.Pheromone.Glow.GlowMaxDistanceAlien", &GLOW_MAX_DISTANCE_ALIEN, 1.0f, 50.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);	
		Asura_CommandConsole::AddVar(L"AVP.Pheromone.Glow.GlowMaxDistancePredator", &GLOW_MAX_DISTANCE_PREDATOR, 1.0f, 50.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);	
	}
#endif
	Platform_Initialise(); 
}
void AVP_GlowOutlines::Shutdown()   
{ 
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	if (Asura_CommandConsole::VarExists(L"AVP.Pheromone.Enable"))
	{
		Asura_CommandConsole::RemoveVar(L"AVP.Pheromone.Enable");
		Asura_CommandConsole::RemoveVar(L"AVP.Pheromone.Glow.GlowSize");	
		Asura_CommandConsole::RemoveVar(L"AVP.Pheromone.Glow.GlowMaxDistanceAlien");	
		Asura_CommandConsole::RemoveVar(L"AVP.Pheromone.Glow.GlowMaxDistancePredator");	
	}
#endif
	Platform_Shutdown(); 
}


void AVP_GlowOutlines::RenderAll( )
{
	Platform_Render(s_axDepthSortedSkins);

	s_uCurrentSortedSkin = 0;
	memset(s_axDepthSortedSkins, 0, sizeof(s_axDepthSortedSkins));

	if (AVP_Settings::GetPlayerSpecies() == AVP_Settings::AVP_SPECIES_ALIEN)
	{
		GLOW_MAX_DISTANCE = GLOW_MAX_DISTANCE_ALIEN;
	} else {
		GLOW_MAX_DISTANCE = GLOW_MAX_DISTANCE_PREDATOR;
	}
}

void AVP_GlowOutlines::Render( const Asura_Animation* const pxAnim,u_int uCamVisFlags, u_int uRenderFlags,
					   const Asura_Vector_3& xBSCentre, float fBSRadius, const Asura_RGBA &xColour)
{
	if (!dvs_bEnablePheromones)
	{
		return;
	}

	if (s_uCurrentSortedSkin == MAX_NUM_SORTED_SKINS)
	{
		Asura_Assert(0, "trying to render more skins with pheromone trails than can be handled, increase MAX_NUM_SORTED_SKINS");
		return;
	}
	if (uRenderFlags & ASURA_HIERARCHY_RENDER_BONES_IN_WORLDSPACE)
	{
		// Dont render when ragdolled as the bones arent right
		return;

	}
	DepthSortedSkinEntry *pxEntry = s_axSortedSkinentries + s_uCurrentSortedSkin++;


	pxEntry->m_xSortedRenderItem.m_eType = ASURA_SORTEDRENDERITEM_TYPE_ANIMATION_SMOOTH_SKIN;
	pxEntry->m_xSortedRenderItem.m_pxAnimation = const_cast<Asura_Animation*>(pxAnim);
	pxEntry->m_xSortedRenderItem.m_uCamVisFlags = uCamVisFlags;
	pxEntry->m_xSortedRenderItem.m_xBSCentre = xBSCentre;
	pxEntry->m_xSortedRenderItem.m_fBSRadius = fBSRadius;
	pxEntry->m_xSortedRenderItem.m_uRenderFlags = uRenderFlags;
	pxEntry->m_xColour = xColour;
	pxEntry->m_xColour.m_fR *= pxEntry->m_xColour.m_fA;
	pxEntry->m_xColour.m_fG *= pxEntry->m_xColour.m_fA;
	pxEntry->m_xColour.m_fB *= pxEntry->m_xColour.m_fA;

	Asura_AMatrix4x4 xBone0Transform;
	Asura_AMatrix4x4 xTransform;
	Asura_Hierarchy_System::s_pxTweenStorage->GetBoneData(0).GetTransform(xBone0Transform);
	xBone0Transform.InvertOrthogonal();
	Asura_Hierarchy_System::s_xBaseTransform.Apply(xBone0Transform, xTransform);

	pxEntry->m_xSortedRenderItem.m_xPosition = xTransform.GetTranslation();
	xTransform.ToMatrix(pxEntry->m_xSortedRenderItem.m_xOrient);

	Asura_Vector_3 xCamPos = *Asura_Camera::GetPosition();
	float fDistanceToCamera = (xCamPos - pxEntry->m_xSortedRenderItem.m_xPosition).Magnitude();

	pxEntry->m_fDistanceToCamera = fDistanceToCamera;
	u_int uSlot = u_int(fDistanceToCamera * NUM_DEPTH_SORT_SLOTS / GLOW_MAX_DISTANCE);

	if (uSlot >= NUM_DEPTH_SORT_SLOTS) uSlot = NUM_DEPTH_SORT_SLOTS - 1;
	
	if (!s_axDepthSortedSkins[uSlot] || s_axDepthSortedSkins[uSlot]->m_fDistanceToCamera < fDistanceToCamera)
	{
		pxEntry->m_pxNext = s_axDepthSortedSkins[uSlot];
		s_axDepthSortedSkins[uSlot] = pxEntry;
	} else {
		DepthSortedSkinEntry *pxSlot =  s_axDepthSortedSkins[uSlot];

		while (pxSlot->m_pxNext && pxSlot->m_pxNext->m_fDistanceToCamera > fDistanceToCamera)
		{
			pxSlot = pxSlot->m_pxNext;
		}
		pxEntry->m_pxNext = pxSlot->m_pxNext;
		pxSlot->m_pxNext = pxEntry;
	}
}
