///////////////////////////////////////////////////////////////////////////////
//
// File: AVP_PerlinNoise.cpp
//
// Created by Andrew Southgate 05/12/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
// Handles AVP procedural perlin noise (to be used platform side to create textures)
//
///////////////////////////////////////////////////////////////////////////////

#include <core/Asura.h>
#include "AVP_PerlinNoise.h"
#include "Core/Asura_Maths.h"

AVP_PerlinNoise::_Grad AVP_PerlinNoise::m_axGradients[16] =
{
	{ 1, 1, 0, 0},   
	{-1, 1, 0, 0},	
	{ 1,-1, 0, 0},	
	{-1,-1, 0, 0},
	{ 1, 0, 1, 0},	 
	{-1, 0, 1, 0},	
	{ 1, 0,-1, 0},	
	{-1, 0,-1, 0},
	{ 0, 1, 1, 0},	 
	{ 0,-1, 1, 0},	
	{ 0, 1,-1, 0},	
	{ 0,-1,-1, 0},
	{ 1, 1, 0, 0},	 
	{ 0,-1, 1, 0},	
	{-1, 1, 0, 0},	
	{ 0,-1,-1, 0}
};
AVP_PerlinNoise::_Grad AVP_PerlinNoise::m_ax4dGradients[32] =
{
	{ 0,-1,-1,-1},
	{ 0,-1,-1, 1},
	{ 0,-1, 1,-1},
	{ 0,-1, 1, 1},
	{ 0, 1,-1,-1},
	{ 0, 1,-1, 1},
	{ 0, 1, 1,-1},
	{ 0, 1, 1, 1},
	{-1,-1, 0,-1},
	{-1, 1, 0,-1},
	{ 1,-1, 0,-1},
	{ 1, 1, 0,-1},
	{-1,-1, 0, 1},
	{-1, 1, 0, 1},
	{ 1,-1, 0, 1},
	{ 1, 1, 0, 1},

	{-1, 0,-1,-1},
	{ 1, 0,-1,-1},
	{-1, 0,-1, 1},
	{ 1, 0,-1, 1},
	{-1, 0, 1,-1},
	{ 1, 0, 1,-1},
	{-1, 0, 1, 1},
	{ 1, 0, 1, 1},
	{ 0,-1,-1, 0},
	{ 0,-1,-1, 0},
	{ 0,-1, 1, 0},
	{ 0,-1, 1, 0},
	{ 0, 1,-1, 0},
	{ 0, 1,-1, 0},
	{ 0, 1, 1, 0},
	{ 0, 1, 1, 0},
};


AVP_PerlinNoise::AVP_PerlinNoise()
{
	for (unsigned int i = 0; i < 256; ++i)
		m_aucPermutation[i] = (unsigned char)i;
	for (unsigned int i = 0; i < 255; ++i)
	{
		unsigned int index = Asura_Random::GetInt(i, 255);
		if (index != i)
		{
			unsigned char tmp = m_aucPermutation[index];
			m_aucPermutation[index] = m_aucPermutation[i];
			m_aucPermutation[i] = tmp;
		}
	}
}
