
///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_DamagePFX.cpp
// Author : Steven Lovesey
// Date   : 30-Apr-2009
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////
// Include files //
///////////////////

#include "Core/Asura.h"

// This file's header.
#include "SpecialFX/pfx/AVP_DamagePFX.h"

// Asura header-files.
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_HashedList.h"
#include "Rendering/Asura_Primitive_Render.h"
#include "SpecialFX/pfx/AVP_DamagePFX.h"
#include "SpecialFX/pfx/Asura_PFX_Effect.h"
#include "SpecialFX/pfx/Asura_PFX_Entity_Effect.h"
#include "SpecialFX/pfx/Asura_PFX_Enums.h"
#include "SpecialFX/pfx/Asura_PFX_System.h"

// Game header-files.
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/Damage/AVP_EntityDamage_Messages.h"
#include "Entity/PhysicalObject/Actor/AVP_ClientEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/AVP_ServerEntity_Actor.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ClientActor_Vertebrate.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AVP_ServerActor_Vertebrate.h"

///////////////////////////
// Constant declarations //
///////////////////////////

////////////////////////////////////////
// Static class variable declarations //
////////////////////////////////////////

Asura_List<AVP_DamagePFX> AVP_DamagePFX::s_xList;
bool AVP_DamagePFX::s_bDebugRender = false;

/////////////////////
// Class functions //
/////////////////////

//////////////////////////////
// Construction/Destruction //
//////////////////////////////

AVP_DamagePFX::AVP_DamagePFX()
: m_uPFXHandle( ASURA_PFX_INVALID_EFFECT_ID )
, m_fDamageDelay (0.f)
, m_fDamageInterval (0.f)
, m_fLifetime (0.f)
, m_fDamage (0.f)
, m_fBBScale(0.f)
, m_fTimeSinceLastDamage (0.f)
{

}

AVP_DamagePFX::~AVP_DamagePFX()
{

}


///////////////////////////////////////
// Public static interface functions //
///////////////////////////////////////

void AVP_DamagePFX::AddDamagePFX( u_int uPFXHandle , float fDamageDelay, float fDamageInterval, float fLifetime, float fDamage, float fBBScale, Asura_Hash_ID uDamageType)
{
	AVP_DamagePFX* pxDPFX = new AVP_DamagePFX();
	Asura_Assert(pxDPFX, "Null new");
	if (!pxDPFX) return;

	pxDPFX->m_uPFXHandle = uPFXHandle;
	pxDPFX->m_fDamageDelay = fDamageDelay;
	pxDPFX->m_fDamageInterval = fDamageInterval;
	pxDPFX->m_fLifetime = fLifetime;
	pxDPFX->m_fDamage = fDamage;
	pxDPFX->m_fBBScale = fBBScale;
	pxDPFX->m_uDamageType = uDamageType;
	s_xList.AddToList(pxDPFX);
}

void AVP_DamagePFX::Update()
{
	Asura_List_It<AVP_DamagePFX> xIt(s_xList);
	while (!xIt.Done())
	{
		AVP_DamagePFX* pxDPFX = xIt.GetCurrent();

		if (pxDPFX->UpdateElement())
		{
			xIt.Next();
		} 
		else
		{
			delete pxDPFX;
			xIt.DeleteCurrent();
		}
	}
}

void AVP_DamagePFX::Shutdown()
{
	for (Asura_List_It<AVP_DamagePFX> xIt(s_xList) ; !xIt.Done() ; xIt.Next())
	{
		AVP_DamagePFX* pxDPFX = xIt.GetCurrent();
		delete pxDPFX;
	}
	s_xList.ClearList();
}

///////////////////////////////
// Private member functions  //
///////////////////////////////


DEBUGVAR float dbg_fMagicDamagePFXScaleValue = 0.2f;

void AVP_DamagePFX::ResizeBoundingBox(Asura_Bounding_Box* xBB)
{
	Asura_Vector_3 xCentre = xBB->CalculateCentre();
	xBB->Displace(-xCentre);
	xBB->Scale(m_fBBScale);
	xBB->Displace(xCentre);
}

bool AVP_DamagePFX::UpdateElement()
{
	Asura_PFX_Entity_Effect* pxEffect = Asura_PFX_System::GetEntityEffect(m_uPFXHandle);
	if (!pxEffect) return false; // Effect no longer around.

	Asura_Bounding_Box xBB = pxEffect->GetBoundingBox();
	ResizeBoundingBox(&xBB); // Shouldnt be necessary, but the BB calculation is too large.


	if (s_bDebugRender)
	{
		Asura_Primitive_Render::UseDefaultRenderParams();
		Asura_Primitive_Render::DrawBoundingBoxWorld(xBB, Asura_Maths::ZeroVector3, Asura_Maths::IdentityMatrix3x3, 0xFFFF0000);
	}

	float fTime = Asura_Timers::GetGameTimeSlice();

	if (m_fDamageDelay > 0.f)
	{
		m_fDamageDelay -= fTime;
	} 

	if (m_fDamageDelay <= 0.f)
	{
		m_fTimeSinceLastDamage += fTime;
		if (m_fTimeSinceLastDamage >= m_fDamageInterval)
		{
			DoCollisionTests(&xBB);
			m_fTimeSinceLastDamage = 0.f;
		}
	}

	m_fLifetime -= fTime;
	if (m_fLifetime <= 0.0f) return false;
	return true;

}


void AVP_DamagePFX::DoCollisionTests(Asura_Bounding_Box* xBB)
{
	Asura_Assert(Asura_Network::IsServer(), "Bad code error, AVP_DamagePFX run on server only");

	for (Asura_Hashed_List_It<AVP_ServerActor_Vertebrate> xIt(AVP_ServerActor_Vertebrate::GetList()) ; !xIt.Done() ; xIt.Next())
	{
		AVP_ServerActor_Vertebrate *pxServerVictim = xIt.GetCurrent();
		const AVP_DamageMultiplier *pxDamageMultiplier = pxServerVictim->FindDamageMultiplier(m_uDamageType);
		if (!pxDamageMultiplier || pxDamageMultiplier->GetMultiplier()!=0.f)
		{
			AVP_ClientActor_Vertebrate *pxVictim = AVP_ClientActor_Vertebrate::FindEntity(pxServerVictim->GetGuid());
			if (!pxVictim) return;

			if (pxVictim->CoarseIntersectsBoundingBox(*xBB))
			{
				if (pxVictim->IntersectsBoundingBox(*xBB))
				{
					const AVP_EntityDamage_RequestVictim xDamage(uAVP_BLUEPRINT_NAME_DAMAGE_ACID, ASURA_GUID_UNREGISTERED, ASURA_HASH_ID_UNSET,
						pxVictim->GetGuid(), m_fDamage, xBB->CalculateCentre(), 
						Asura_Maths::ZeroVector3, Asura_Maths::ZeroVector3);
					AVP_ServerEntity_Actor *pxServerVictim = AVP_ServerEntity_Actor::FindEntity( pxVictim->GetGuid());
					if (pxServerVictim) pxServerVictim->TakeDamageExtended( xDamage, false );
				}
			}
		}
	}
}


/// End of file. ///


