///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_PC_Actor_ControlMapper.cpp
// Author : Will Plano
// Date   : 30-Jul-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
// JL - 01 April 2009 - Modified version of PC file, used in PC-CLI
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

// Asura header...
#include "Core/Asura.h"

// This file's header...
#include "Actor/AVP_PC_Actor_ControlMapper.h"

// Asura platform header-files...
#include "Input/Asura_PlatformInput.h"
//#include "Input/Asura_PC_Input_Joystick.h"

// AVP independent header files...
#include "Actor/AVP_Actor_InputHandler.h"

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Actor_ControlMapper::InitialiseControlsPC
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Actor_ControlMapper::InitialiseControlsPC(AVP_Actor_InputHandler& xInputHandler)
{
	Asura_ControlMap&	xFirst		= xInputHandler.GetMapFirstPerson();
	Asura_ControlMap&	xCutscene	= xInputHandler.GetMapCutscene();

	// first person mappings
	xInputHandler.MapLookMouse				   (xFirst);
	xInputHandler.MapInteract				   (xFirst, 'e', 0.0f );
	xInputHandler.MapMenuToggle				   (xFirst, ASURA_KEYID_ESCAPE);
	xInputHandler.MapObjectivesAndSelectToggle (xFirst, ASURA_KEYID_BACKSPACE, ASURA_KEYID_TAB);
	xInputHandler.MapFreezeTime				   (xFirst, 'p');

	// Cutscene mappings
	xInputHandler.MapCutscene				   (xCutscene, ASURA_KEYID_ENTER, ASURA_KEYID_SPACE, ASURA_KEYID_MOUSEBUTTON1, ASURA_KEYID_MOUSEBUTTON2);
	xInputHandler.MapMenuToggle				   (xCutscene, ASURA_KEYID_ESCAPE);
	xInputHandler.MapObjectivesAndSelectToggle (xCutscene, ASURA_KEYID_BACKSPACE, ASURA_KEYID_TAB);

#if defined(ASURA_USE_TEXTURE_STREAMING)
	xFirst.AddKeyMapping(AVP_Actor_InputHandler::STREAMINGTEXTURES_TOGGLEDELAY, 'k', Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xFirst.AddKeyMapping(AVP_Actor_InputHandler::STREAMINGTEXTURES_UNLOADALL, 'l', Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#endif
}

// JL - 01 April 2009 - Stubbed out for PC-CLI to avoid using anything related to joysticks
void AVP_PC_Actor_ControlMapper::InitialiseControlsPS2(AVP_Actor_InputHandler& xInputHandler) {}
void AVP_PC_Actor_ControlMapper::InitialiseControlsPS3(AVP_Actor_InputHandler& xInputHandler) {}
void AVP_PC_Actor_ControlMapper::InitialiseControlsXBox360(AVP_Actor_InputHandler& xInputHandler) {}

void AVP_PC_Actor_ControlMapper::Initialise(AVP_Actor_InputHandler &xInputHandler)
{
	InitialiseControlsPC( xInputHandler );
}

/// End of file. ///
