//////////////////////////////////////////////////////////////////////////////
//
// AVP_PC_External_Camera.cpp
//
// Created by Anthony Gregson - 09/11/07
// Adapted for PC-CLI by Johan Lerstrom - 01/04/09
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//		PC-specific external camera functions.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Camera/AVP_External_Camera.h"

// Asura headers
#include "Input/Asura_ControlManager.h"
#include "Input/Asura_ControlMap.h"

// Asura PC
#include "Input/Asura_PlatformInput.h"

// Game headers
#include "Core/AVP_Game_Core.h"


// Debugvars
DEBUGVAR float dbg_fTiltRampUpTime = 0.25f;
DEBUGVAR float dbg_fTiltRampDownTime = 0.05f;
DEBUGVAR float dbg_fTiltExponent = 2.0f;
DEBUGVAR float dbg_fMouseVSensitivity = 1.0f; // Bigger is quicker

DEBUGVAR float dbg_fTurnRampUpTime = 0.5f;
DEBUGVAR float dbg_fTurnRampDownTime = 0.10f;
DEBUGVAR float dbg_fMouseHSensitivity = 1.0f; // Bigger is quicker

/////////////////////
// Input functions //
/////////////////////

static void InitialiseControlsPC(Asura_ControlMap& xMap)
{
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_UP_DOWN, ASURA_AXIS_MOUSE_V, -dbg_fMouseVSensitivity);
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_LEFT_RIGHT, ASURA_AXIS_MOUSE_H, dbg_fMouseHSensitivity);

	xMap.AddKeyMapping(AVP_External_Camera::MOVE_FORWARDS_BACKWARDS,	's', Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_FORWARDS_BACKWARDS,	'w', Asura_ControlMapping::ON_PRESS, 1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_LEFT_RIGHT,			'a', Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_LEFT_RIGHT,			'd', Asura_ControlMapping::ON_PRESS, 1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN,				'f', Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN,				'v', Asura_ControlMapping::ON_PRESS, 1.0f);

	xMap.AddKeyMapping(AVP_External_Camera::RESPAWN, ASURA_KEYID_MOUSEBUTTON1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::RESPAWN, ASURA_KEYID_SPACE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_MENU, ASURA_KEYID_ESCAPE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_OBJECTIVES, ASURA_KEYID_BACKSPACE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_DVS, ASURA_KEYID_TAB, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#else
	xMap.AddKeyMapping(AVP_External_Camera::SCOREBOARD, ASURA_KEYID_TAB, Asura_ControlMapping::ON_PRESS );
#endif //#ifdef ASURA_DEBUG_VARIABLE_SYSTEM

	xMap.AddKeyMapping(AVP_External_Camera::NEXT_PLAYER, ASURA_KEYID_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS); 
	xMap.AddKeyMapping(AVP_External_Camera::PREVIOUS_PLAYER, ASURA_KEYID_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_FOLLOW_PLAYER, ASURA_KEYID_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_FIRST_PERSON, ASURA_KEYID_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
}

void AVP_External_Camera::Platform_FillControlMap()
{
	if (!m_pxControlMap) { return; }

	// Always ensure that we've bound the mouse/keyboard controls...
	InitialiseControlsPC(*m_pxControlMap);
}

#ifdef AVP_SECOND_CONTROL_FLY_CAM

void AVP_External_Camera::Platform_FillSecondControlMap()
{
	//MKC - Not using the external camera as fly cam on the PC version so not mapping any controls
}
#endif

/// End of file. ///
