///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_PC_Actor_ControlMapper.cpp
// Author : Will Plano
// Date   : 30-Jul-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

// Asura header...
#include "Core/Asura.h"

// This file's header...
#include "Actor/AVP_PC_Actor_ControlMapper.h"

// Asura platform header-files...
#include "Input/Asura_PlatformInput.h"
#include "Input/Asura_PC_Input_Joystick.h"

// AVP independent header files...
#include "Actor/AVP_Actor_InputHandler.h"



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Actor_ControlMapper::InitialiseControlsPC
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Actor_ControlMapper::InitialiseControlsPC(AVP_Actor_InputHandler& xInputHandler)
{
	Asura_ControlMap&	xFirst		= xInputHandler.GetMapFirstPerson();
	Asura_ControlMap&	xCutscene	= xInputHandler.GetMapCutscene();

	// first person mappings
	xInputHandler.MapLookMouse				   (xFirst);
	xInputHandler.MapInteract				   (xFirst, 'e', 0.0f );
	xInputHandler.MapMenuToggle				   (xFirst, ASURA_KEYID_ESCAPE);
	xInputHandler.MapObjectivesAndSelectToggle (xFirst, ASURA_KEYID_BACKSPACE, ASURA_KEYID_TAB);
	xInputHandler.MapFreezeTime				   (xFirst, 'p');

	// Cutscene mappings
	xInputHandler.MapCutscene				   (xCutscene, ASURA_KEYID_ENTER, ASURA_KEYID_SPACE, ASURA_KEYID_MOUSEBUTTON1, ASURA_KEYID_MOUSEBUTTON2);
	xInputHandler.MapMenuToggle				   (xCutscene, ASURA_KEYID_ESCAPE);
	xInputHandler.MapObjectivesAndSelectToggle (xCutscene, ASURA_KEYID_BACKSPACE, ASURA_KEYID_TAB);

#if defined(ASURA_USE_TEXTURE_STREAMING)
	xFirst.AddKeyMapping(AVP_Actor_InputHandler::STREAMINGTEXTURES_TOGGLEDELAY, 'k', Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xFirst.AddKeyMapping(AVP_Actor_InputHandler::STREAMINGTEXTURES_UNLOADALL, 'l', Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#endif
}

/// End of function 'AVP_PC_Actor_ControlMapper::InitialiseControlsPC'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Actor_ControlMapper::InitialiseControlsPS2
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Actor_ControlMapper::InitialiseControlsPS2(AVP_Actor_InputHandler& xInputHandler)
{
	Asura_ControlMap&	xFirst		= xInputHandler.GetMapFirstPerson();
	Asura_ControlMap&	xCutscene	= xInputHandler.GetMapCutscene();

	// first person mappings
	xInputHandler.MapLookAxes				   (xFirst, ASURA_PC_AXISID_PS2PAD1_RSTICK_H, ASURA_PC_AXISID_PS2PAD1_RSTICK_V);
	xInputHandler.MapInteract				   (xFirst, ASURA_PC_KEYID_PS2PAD1_SQUARE, 1.0f);
	xInputHandler.MapMenuToggle				   (xFirst, ASURA_PC_KEYID_PS2PAD1_START);
	xInputHandler.MapObjectivesAndSelectToggle (xFirst, ASURA_PC_KEYID_PS2PAD1_SELECT, ASURA_PC_KEYID_PS2PAD1_SELECT);

	// Cutscene mappings
	xInputHandler.MapCutscene				   (xCutscene, ASURA_PC_KEYID_PS2PAD1_CROSS);
	xInputHandler.MapMenuToggle			       (xCutscene, ASURA_PC_KEYID_PS2PAD1_START);
	xInputHandler.MapObjectivesAndSelectToggle (xCutscene, ASURA_PC_KEYID_PS2PAD1_SELECT, ASURA_PC_KEYID_PS2PAD1_SELECT);
}

/// End of function 'AVP_PC_Actor_ControlMapper::InitialiseControlsPS2'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Actor_ControlMapper::InitialiseControlsPS3
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Actor_ControlMapper::InitialiseControlsPS3(AVP_Actor_InputHandler& xInputHandler)
{
	Asura_ControlMap&	xFirst		= xInputHandler.GetMapFirstPerson();
	Asura_ControlMap&	xCutscene	= xInputHandler.GetMapCutscene();

	// first person mappings
	xInputHandler.MapLookAxes				   (xFirst, ASURA_PC_AXISID_PS3PAD1_RSTICK_H, ASURA_PC_AXISID_PS3PAD1_RSTICK_V);
	xInputHandler.MapZoom					   (xFirst, ASURA_KEYID_VOID, ASURA_KEYID_VOID, ASURA_PC_KEYID_PS3PAD1_R3);
	xInputHandler.MapInteract			       (xFirst, ASURA_PC_KEYID_PS3PAD1_SQUARE, 1.0f);
	xInputHandler.MapMenuToggle			       (xFirst, ASURA_PC_KEYID_PS3PAD1_START);
	xInputHandler.MapObjectivesAndSelectToggle (xFirst, ASURA_PC_KEYID_PS3PAD1_SELECT, ASURA_PC_KEYID_PS3PAD1_SELECT);

	// Cutscene mappings
	xInputHandler.MapCutscene				   (xCutscene, ASURA_PC_KEYID_PS3PAD1_CROSS);
	xInputHandler.MapMenuToggle				   (xCutscene, ASURA_PC_KEYID_PS3PAD1_START);
	xInputHandler.MapObjectivesAndSelectToggle (xCutscene, ASURA_PC_KEYID_PS3PAD1_SELECT, ASURA_PC_KEYID_PS3PAD1_SELECT);
}

/// End of function 'AVP_PC_Actor_ControlMapper::InitialiseControlsPS3'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Actor_ControlMapper::InitialiseControlsXBox360
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Actor_ControlMapper::InitialiseControlsXBox360(AVP_Actor_InputHandler& xInputHandler)
{
	Asura_ControlMap&	xFirst		= xInputHandler.GetMapFirstPerson();
	Asura_ControlMap&	xCutscene	= xInputHandler.GetMapCutscene();

	// first person mappings
	xInputHandler.MapLookAxes				   (xFirst, ASURA_PC_AXISID_XBOX360PAD1_RSTICK_H, ASURA_PC_AXISID_XBOX360PAD1_RSTICK_V);
	xInputHandler.MapInteract				   (xFirst, ASURA_PC_KEYID_XBOX360PAD1_X, 1.0f);
	xInputHandler.MapMenuToggle				   (xFirst, ASURA_PC_KEYID_XBOX360PAD1_START);
	xInputHandler.MapObjectivesAndSelectToggle (xFirst, ASURA_PC_KEYID_XBOX360PAD1_BACK, ASURA_PC_KEYID_XBOX360PAD1_BACK);

	// Cutscene mappings
	xInputHandler.MapCutscene				   (xCutscene, ASURA_PC_KEYID_XBOX360PAD1_A);
	xInputHandler.MapMenuToggle				   (xCutscene, ASURA_PC_KEYID_XBOX360PAD1_START);
	xInputHandler.MapObjectivesAndSelectToggle (xCutscene, ASURA_PC_KEYID_XBOX360PAD1_BACK, ASURA_PC_KEYID_XBOX360PAD1_BACK);
}

/// End of function 'AVP_PC_Actor_ControlMapper::InitialiseControlsXBox360'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_PC_Actor_ControlMapper::Initialise
// Author       : Andy Tate
// Date         : 27-Sep-2007
////////////////////////////////////////////////////////////////////////////////

void AVP_PC_Actor_ControlMapper::Initialise(AVP_Actor_InputHandler& xInputHandler,
											const Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE eJoystickType)
{
	switch(eJoystickType)
	{
// temp for milestone sM04 Jan 09
#ifndef AVP_SUBMISSION
		case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_PS2_GAMEPAD:
		{
			InitialiseControlsPS2(xInputHandler);
			break;
		}

		case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_PS3_GAMEPAD:
		{
			InitialiseControlsPS3(xInputHandler);
			break;
		}

		#ifdef ASURA_PC_SUPPORT_XINPUT
		case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_XBOX360_XINPUT_GAMEPAD:
		#endif
		case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_XBOX360_GAMEPAD:
		{
			InitialiseControlsXBox360(xInputHandler);
			break;
		}
#endif
		default:
		{
			InitialiseControlsPC(xInputHandler);
			break;
		}
	}		// End of switch.
}

/// End of function 'AVP_PC_Actor_ControlMapper::Initialise'. ///


/// End of file. ///
