///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Platform_Vertebrate_InputHandler.cpp
// Author : Callum Hope
// Date   : 30-Oct-2008
// Desc	  : AVP/PC/Actor/Vertebrates... Platform dependant functionality for the input handler
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// core header
#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/AVP_Vertebrate_InputHandler.h"

// Asura Headers
#include "Input/Asura_ControlMapping_Axis.h"
#include "Input/Asura_ControlMapping_HoldKey.h"
#include "Input/Asura_ControlMapping_Key.h"
#include "Input/Asura_ControlMapping_Combination.h"
#include "Input/Asura_ControlMapping_TapAxis.h"
#include "Input/Asura_ControlMapping_TapKey.h"
#include "Rendering/AVP_Font_System.h"

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_InputHandler::Platform_GetTileIdForMapping (Private)
// Author       : Callum Hope
// Date         : 30-Oct-2008
// Desc			: Finds the Key assigned to a mapping and returns an id to the texture tile that is used to display it in game. 
//
// Arguments	: pxMapping, the mapping that has a key mapped to it. 
//				  uComboId, which of the mappings are we looking at if this is a combo mapping. must be only 0 or 1
//
// Return		: u_int, the id that is found for the key, or 0 in an error state. 
////////////////////////////////////////////////////////////////////////////////
u_int AVP_Vertebrate_InputHandler::Platform_GetTileIdForMapping( const Asura_ControlMapping* pxMapping, u_int uComboId )
{
	int iKeyID;

	if( !pxMapping )
	{
		Asura_Assert( false, "Platform_GetTileIdForMapping\n A Tile ID has been requested but no mapping was sent to find it from!" );
		return 0;
	}

	if( ( uComboId != 0 ) && ( uComboId != 1 ) )
	{
		Asura_Assertf( false, ( "Platform_GetTileIdForMapping\n Combo ID passed in is not valid must be 0 or 1, it is %d", uComboId ) );
		uComboId = 0;
	}

	if( pxMapping->GetInstanceMappingType() == Asura_ControlMapping_Combination::GetClassMappingType() )
	{
		iKeyID = static_cast<const Asura_ControlMapping_Combination*>( pxMapping )->GetControlMapping( uComboId )->GetControlID();
	}
	else
	{
		iKeyID = pxMapping->GetControlID();
	}
	
	if( ( pxMapping->GetInstanceMappingType() != Asura_ControlMapping_Axis::GetClassMappingType() ) && ( pxMapping->GetInstanceMappingType() != Asura_ControlMapping_TapAxis::GetClassMappingType() ) )
	{
		switch( iKeyID )
		{
			////////////////////////////////////////////////////////////////////////////////////
			//  NUMBERS

			case 48:	// '0'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_1;

			case 49:	// '1'
			case ASURA_KEYID_JOYSTICK_BUTTON_1:
			case ASURA_KEYID_PAD1_BUTTON1: 
			case ASURA_KEYID_PAD2_BUTTON1: 
			case ASURA_KEYID_PAD3_BUTTON1: 
			case ASURA_KEYID_PAD4_BUTTON1: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_2;

			case 50:	// '2'
			case ASURA_KEYID_JOYSTICK_BUTTON_2:
			case ASURA_KEYID_PAD1_BUTTON2: 
			case ASURA_KEYID_PAD2_BUTTON2: 
			case ASURA_KEYID_PAD3_BUTTON2: 
			case ASURA_KEYID_PAD4_BUTTON2: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_3;

			case 51:	// '3'
			case ASURA_KEYID_JOYSTICK_BUTTON_3:
			case ASURA_KEYID_PAD1_BUTTON3: 
			case ASURA_KEYID_PAD2_BUTTON3: 
			case ASURA_KEYID_PAD3_BUTTON3: 
			case ASURA_KEYID_PAD4_BUTTON3: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_4;

			case 52:	// '4'
			case ASURA_KEYID_JOYSTICK_BUTTON_4:
			case ASURA_KEYID_PAD1_BUTTON4: 
			case ASURA_KEYID_PAD2_BUTTON4: 
			case ASURA_KEYID_PAD3_BUTTON4: 
			case ASURA_KEYID_PAD4_BUTTON4: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_5;

			case 53:	// '5'
			case ASURA_KEYID_JOYSTICK_BUTTON_5:
			case ASURA_KEYID_PAD1_BUTTON5: 
			case ASURA_KEYID_PAD2_BUTTON5: 
			case ASURA_KEYID_PAD3_BUTTON5: 
			case ASURA_KEYID_PAD4_BUTTON5: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_6;

			case 54:	// '6'
			case ASURA_KEYID_JOYSTICK_BUTTON_6:
			case ASURA_KEYID_PAD1_BUTTON6: 
			case ASURA_KEYID_PAD2_BUTTON6: 
			case ASURA_KEYID_PAD3_BUTTON6: 
			case ASURA_KEYID_PAD4_BUTTON6: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_7;

			case 55:	// '7'
			case ASURA_KEYID_JOYSTICK_BUTTON_7:
			case ASURA_KEYID_PAD1_BUTTON7: 
			case ASURA_KEYID_PAD2_BUTTON7: 
			case ASURA_KEYID_PAD3_BUTTON7: 
			case ASURA_KEYID_PAD4_BUTTON7: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_8;

			case 56:	// '8'
			case ASURA_KEYID_JOYSTICK_BUTTON_8:
			case ASURA_KEYID_PAD1_BUTTON8: 
			case ASURA_KEYID_PAD2_BUTTON8: 
			case ASURA_KEYID_PAD3_BUTTON8: 
			case ASURA_KEYID_PAD4_BUTTON8: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_9;

			case 57:	// '9'
			case ASURA_KEYID_JOYSTICK_BUTTON_9:
			case ASURA_KEYID_PAD1_BUTTON9: 
			case ASURA_KEYID_PAD2_BUTTON9: 
			case ASURA_KEYID_PAD3_BUTTON9: 
			case ASURA_KEYID_PAD4_BUTTON9: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_10;

			case ASURA_KEYID_JOYSTICK_BUTTON_10:
			case ASURA_KEYID_PAD1_BUTTON10: 
			case ASURA_KEYID_PAD2_BUTTON10: 
			case ASURA_KEYID_PAD3_BUTTON10: 
			case ASURA_KEYID_PAD4_BUTTON10: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_11;

			case ASURA_KEYID_JOYSTICK_BUTTON_11:
			case ASURA_KEYID_PAD1_BUTTON11: 
			case ASURA_KEYID_PAD2_BUTTON11: 
			case ASURA_KEYID_PAD3_BUTTON11: 
			case ASURA_KEYID_PAD4_BUTTON11: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_12;

			case ASURA_KEYID_JOYSTICK_BUTTON_12:
			case ASURA_KEYID_PAD1_BUTTON12: 
			case ASURA_KEYID_PAD2_BUTTON12: 
			case ASURA_KEYID_PAD3_BUTTON12: 
			case ASURA_KEYID_PAD4_BUTTON12: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_13;

			case ASURA_KEYID_JOYSTICK_BUTTON_13:
			case ASURA_KEYID_PAD1_BUTTON13: 
			case ASURA_KEYID_PAD2_BUTTON13: 
			case ASURA_KEYID_PAD3_BUTTON13: 
			case ASURA_KEYID_PAD4_BUTTON13: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_14;

			case ASURA_KEYID_JOYSTICK_BUTTON_14:
			case ASURA_KEYID_PAD1_BUTTON14: 
			case ASURA_KEYID_PAD2_BUTTON14: 
			case ASURA_KEYID_PAD3_BUTTON14: 
			case ASURA_KEYID_PAD4_BUTTON14: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_15;

			case ASURA_KEYID_JOYSTICK_BUTTON_15:
			case ASURA_KEYID_PAD1_BUTTON15: 
			case ASURA_KEYID_PAD2_BUTTON15: 
			case ASURA_KEYID_PAD3_BUTTON15: 
			case ASURA_KEYID_PAD4_BUTTON15: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_16;

			case ASURA_KEYID_JOYSTICK_BUTTON_16:
			case ASURA_KEYID_PAD1_BUTTON16: 
			case ASURA_KEYID_PAD2_BUTTON16: 
			case ASURA_KEYID_PAD3_BUTTON16: 
			case ASURA_KEYID_PAD4_BUTTON16: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_17;

			////////////////////////////////////////////////////////////////////////////////////
			//  LETTERS

			case 97:	// 'a'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_18;
			case 98:	// 'b'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_19;
			case 99:	// 'c'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_20;
			case 100:	// 'd'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_21;
			case 101:	// 'e'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_22;
			case 102:	// 'f'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_23;
			case 103:	// 'g'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_24;
			case 104:	// 'h'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_25;
			case 105:	// 'i'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_26;
			case 106:	// 'j'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_27;
			case 107:	// 'k'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_28;
			case 108:	// 'l'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_29;
			case 109:	// 'm'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_30;
			case 110:	// 'n'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_31;
			case 111:	// 'o'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_32;
			case 112:	// 'p'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_33;
			case 113:	// 'q'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_34;
			case 114:	// 'r'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_35;
			case 115:	// 's'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_36;
			case 116:	// 't'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_37;
			case 117:	// 'u'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_38;
			case 118:	// 'v'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_39;
			case 119:	// 'w'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_40;
			case 120:	// 'x'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_41;
			case 121:	// 'y'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_42;
			case 122:	// 'z'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_43;

			////////////////////////////////////////////////////////////////////////////////////
			//  FUNCTIONS

			case ASURA_KEYID_F1:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_44;
			case ASURA_KEYID_F2:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_45;
			case ASURA_KEYID_F3:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_46;
			case ASURA_KEYID_F4:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_47;
			case ASURA_KEYID_F5:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_48;
			case ASURA_KEYID_F6:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_49;
			case ASURA_KEYID_F7:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_50;
			case ASURA_KEYID_F8:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_51;
			case ASURA_KEYID_F9:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_52;
			case ASURA_KEYID_F10:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_53;
			case ASURA_KEYID_F11:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_54;
			case ASURA_KEYID_F12:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_55;
			case ASURA_KEYID_F13:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_56;
			case ASURA_KEYID_F14:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_57;
			case ASURA_KEYID_F15:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_58;
			case ASURA_KEYID_F16:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_59;

			////////////////////////////////////////////////////////////////////////////////////
			//  PUNCTUATION AND SYMBOLS

			case 96: // '`'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_60;
			case 45: // '-'
			case ASURA_KEYID_NUMPADSUB:   
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_61;
			case 61: // '='
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_62;
			case 91: // '['
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_63;
			case 93: // ']'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_64;
			case 59: // ';'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_65;
			case 39: // '''
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_66;
			case 35: // '#'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_67;
			case 44: // ','
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_68;
			case 46: // '.'
			case ASURA_KEYID_NUMPADDEL:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_69;
			case 47: // '/'
			case ASURA_KEYID_NUMPADDIVIDE:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_70;
			case 92: // '\'
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_71;
			case 42: // '*'
			case ASURA_KEYID_NUMPADMULTIPLY:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_72;
			case 43: // '+'
			case ASURA_KEYID_NUMPADADD:   
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_73;

			////////////////////////////////////////////////////////////////////////////////////
			//  FORMATTING (and esc)

			case ASURA_KEYID_BACKSPACE:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_74;
			case ASURA_KEYID_TAB:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_75;
			case ASURA_KEYID_ENTER:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_76;
			case ASURA_KEYID_NUMPADENTER:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_77;
			case ASURA_KEYID_ESCAPE:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_78;
			case ASURA_KEYID_SPACE:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_79;

			////////////////////////////////////////////////////////////////////////////////////
			//  MODE CHANGERS

			case ASURA_KEYID_SHIFT:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_80;
			case ASURA_KEYID_LEFTSHIFT:	
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_81;
			case ASURA_KEYID_RIGHTSHIFT:  
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_82;
			case ASURA_KEYID_CTRL:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_83;
			case ASURA_KEYID_LEFTCTRL:	
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_84;
			case ASURA_KEYID_RIGHTCTRL:   
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_85;
			case ASURA_KEYID_ALT:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_86;
			case ASURA_KEYID_LEFTALT:     
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_87;
			case ASURA_KEYID_RIGHTALT:    
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_88;

			////////////////////////////////////////////////////////////////////////////////////
			//  MICROSOFT SPECIFIC

			case ASURA_KEYID_LWIN:
			case ASURA_KEYID_RWIN:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_89;
			case ASURA_KEYID_APPS:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_90;

			////////////////////////////////////////////////////////////////////////////////////
			//  LOCKS

			case ASURA_KEYID_CAPSLOCK:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_91;
			case ASURA_KEYID_NUMLOCK:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_92;
			case ASURA_KEYID_SCROLLOK:	
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_93;
			case ASURA_KEYID_PAUSE:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_94;

			////////////////////////////////////////////////////////////////////////////////////
			//  DIRECTIONS

			case ASURA_KEYID_UP:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_95;
			case ASURA_KEYID_DOWN:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_96;
			case ASURA_KEYID_LEFT:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_97;
			case ASURA_KEYID_RIGHT:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_98;

			////////////////////////////////////////////////////////////////////////////////////
			//  6 PACK TRAVERSAL

			case ASURA_KEYID_INS:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_99;
			case ASURA_KEYID_DEL:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_100;			
			case ASURA_KEYID_HOME:	
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_101;
			case ASURA_KEYID_END:	
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_102;			
			case ASURA_KEYID_PAGEUP:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_103;
			case ASURA_KEYID_PAGEDOWN:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_104;

			////////////////////////////////////////////////////////////////////////////////////
			//  MOUSE BUTTONS

			case ASURA_KEYID_MOUSEBUTTON1:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_105;
			case ASURA_KEYID_MOUSEBUTTON2:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_106;
			case ASURA_KEYID_MOUSEBUTTON3:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_107;
			case ASURA_KEYID_MOUSEBUTTON4:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_108;
			case ASURA_KEYID_MOUSEBUTTON5:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_109;
			case ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON1:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_110;
			case ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON2:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_111;
			case ASURA_KEYID_MOUSEWHEELUP:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_112;
			case ASURA_KEYID_MOUSEWHEELDOWN:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_113;

			////////////////////////////////////////////////////////////////////////////////////
			//  NUMPAD NUMBERS

			case ASURA_KEYID_NUMPAD0: 
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_114;
			case ASURA_KEYID_NUMPAD1:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_115;
			case ASURA_KEYID_NUMPAD2:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_116;
			case ASURA_KEYID_NUMPAD3:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_117;
			case ASURA_KEYID_NUMPAD4:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_118;
			case ASURA_KEYID_NUMPAD5:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_119;
			case ASURA_KEYID_NUMPAD6:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_120;
			case ASURA_KEYID_NUMPAD7:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_121;
			case ASURA_KEYID_NUMPAD8:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_122;
			case ASURA_KEYID_NUMPAD9:     
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_123;

			////////////////////////////////////////////////////////////////////////////////////
			//  NOT ASSIGNED

			case ASURA_KEYID_VOID:
			default:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_NA;
		}
	}
	else
	{
		// else if this is any kind of axis mapping
		switch( iKeyID )
		{
			case ASURA_AXIS_MOUSE_H:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_MOUSE_H;

			case ASURA_AXIS_MOUSE_V:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_MOUSE_V;

			case ASURA_AXIS_PAD1_LSTICK_H:
			case ASURA_AXIS_PAD2_LSTICK_H:
			case ASURA_AXIS_PAD3_LSTICK_H:
			case ASURA_AXIS_PAD4_LSTICK_H:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_LSTICK_H;

			case ASURA_AXIS_PAD1_LSTICK_V:
			case ASURA_AXIS_PAD2_LSTICK_V:
			case ASURA_AXIS_PAD3_LSTICK_V:
			case ASURA_AXIS_PAD4_LSTICK_V:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_LSTICK_V;
			
			case ASURA_AXIS_PAD1_RSTICK_H:
			case ASURA_AXIS_PAD2_RSTICK_H:
			case ASURA_AXIS_PAD3_RSTICK_H:
			case ASURA_AXIS_PAD4_RSTICK_H:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_RSTICK_H;

			case ASURA_AXIS_PAD1_RSTICK_V:
			case ASURA_AXIS_PAD2_RSTICK_V:
			case ASURA_AXIS_PAD3_RSTICK_V:
			case ASURA_AXIS_PAD4_RSTICK_V:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_RSTICK_V;
			
			case ASURA_AXIS_PAD1_Z:
			case ASURA_AXIS_PAD2_Z:
			case ASURA_AXIS_PAD3_Z:
			case ASURA_AXIS_PAD4_Z:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_Z;

			case ASURA_AXIS_PAD1_ROTATE_X:
			case ASURA_AXIS_PAD2_ROTATE_X:
			case ASURA_AXIS_PAD3_ROTATE_X:
			case ASURA_AXIS_PAD4_ROTATE_X:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_X;

			case ASURA_AXIS_PAD1_ROTATE_Y:
			case ASURA_AXIS_PAD2_ROTATE_Y:
			case ASURA_AXIS_PAD3_ROTATE_Y:
			case ASURA_AXIS_PAD4_ROTATE_Y:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_Y;

			case ASURA_AXIS_PAD1_ROTATE_Z:
			case ASURA_AXIS_PAD2_ROTATE_Z:
			case ASURA_AXIS_PAD3_ROTATE_Z:
			case ASURA_AXIS_PAD4_ROTATE_Z:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_Z;

			case ASURA_AXIS_VOID:
			default:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_NA;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_InputHandler::GetDescStringForMapping (Private)
// Author       : Callum Hope
// Date         : 30-Oct-2008
// Desc			: Finds the Key assigned to a mapping and prints out a string for describe what it is for menus and debugging. 
//
// Arguments	: pxDescString, the AsuraString to be filled with the resulting desc. 
//				  pxMapping, the mapping that has a key mapped to it. 				  
//
// Return		: bool flag to signify if a string was successfully created and entered to Asura_String Argument. 
////////////////////////////////////////////////////////////////////////////////
bool AVP_Vertebrate_InputHandler::Platform_GetDescStringForMapping( Asura_String* pxDescString, const Asura_ControlMapping* pxMapping )
{
	int iStringLength = 64;
	int iKeyID;
	char* psHold = "Hold ";
	char* psTap = "Tap ";

	if( ! pxDescString || ! pxMapping )
	{
		return false;
	}

	// check for any needed prefixes
	if( pxMapping->GetInstanceMappingType() == Asura_ControlMapping_HoldKey::GetClassMappingType() )
	{
		// if its a HOLDKEY mapping
		Asura_PrintfToBfr( pxDescString->GetString(), psHold, iStringLength );
	}
	else if( ( pxMapping->GetInstanceMappingType() == Asura_ControlMapping_TapAxis::GetClassMappingType() ) ||
			 ( pxMapping->GetInstanceMappingType() == Asura_ControlMapping_TapKey::GetClassMappingType() ) )
	{
		// if its a TAPKEY or TAPAXIS mapping
		Asura_PrintfToBfr( pxDescString->GetString(), psTap, iStringLength );
	}
	else if( pxMapping->GetInstanceMappingType() == Asura_ControlMapping_Combination::GetClassMappingType() )
	{
		// if its a COMBINATION mapping
		//double check before casting this maping to unlock functionality of this derioved type
		if( pxMapping->IsComboMapping() )
		{
			// special case, this function must run though twice and return from here.
			Platform_GetDescStringForMapping( pxDescString, static_cast<const Asura_ControlMapping_Combination*>( pxMapping )->GetControlMapping( 0 ) );

			// add on a plus sign for between the two descriptions. 
			Asura_PrintfToBfr( pxDescString->GetString(), "%s + ", iStringLength, true, pxDescString->GetString() );

			Platform_GetDescStringForMapping( pxDescString, static_cast<const Asura_ControlMapping_Combination*>( pxMapping )->GetControlMapping( 1 ) );
			
			// and we are done here. 
			return true;
		}
	}
	else
	{
		// no prefix needed, do nothing
	}

	// now for the long labourious switch case to end all switch cases. 
	iKeyID = pxMapping->GetControlID();

	if( ( pxMapping->GetInstanceMappingType() != Asura_ControlMapping_Axis::GetClassMappingType() ) && ( pxMapping->GetInstanceMappingType() != Asura_ControlMapping_TapAxis::GetClassMappingType() ) )
	{
		switch( iKeyID )
		{
			case ASURA_KEYID_VOID:
				Asura_PrintfToBfr( pxDescString->GetString(), "--", iStringLength, true );
				break;
			
			case ASURA_KEYID_BACKSPACE:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sBACKSPACE", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_TAB:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sTAB", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_ENTER:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sRTN", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_ESCAPE:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sESC", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_SPACE:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sSPACE", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_SHIFT:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sSHIFT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_CTRL:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sCTRL", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_ALT:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sALT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_PAUSE:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAUSE", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_CAPSLOCK:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sCAPS LOCK", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_KEYID_PAGEUP:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPG UP", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_PAGEDOWN:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPG DOWN", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_END:	
				Asura_PrintfToBfr( pxDescString->GetString(), "%sEND", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_HOME:	
				Asura_PrintfToBfr( pxDescString->GetString(), "%sHOME", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_LEFT:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sLEFT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_UP:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sUP", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_RIGHT:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sRIGHT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_DOWN:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sDOWN", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_INS:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sINSERT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_DEL:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sDEL", iStringLength, true, pxDescString->GetString() );
				break;
		
			case ASURA_KEYID_NUMPAD0:     
			case ASURA_KEYID_NUMPAD1:     
			case ASURA_KEYID_NUMPAD2:     
			case ASURA_KEYID_NUMPAD3:     
			case ASURA_KEYID_NUMPAD4:     
			case ASURA_KEYID_NUMPAD5:     
			case ASURA_KEYID_NUMPAD6:     
			case ASURA_KEYID_NUMPAD7:     
			case ASURA_KEYID_NUMPAD8:     
			case ASURA_KEYID_NUMPAD9:     
				Asura_PrintfToBfr( pxDescString->GetString(), "%sNUMPAD %d", iStringLength, true, pxDescString->GetString(), ( iKeyID - ASURA_KEYID_NUMPAD0 ) );
				break;

			case ASURA_KEYID_NUMPADMULTIPLY:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sNUMPAD *", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_NUMPADADD:   
				Asura_PrintfToBfr( pxDescString->GetString(), "%sNUMPAD +", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_NUMPADENTER:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sNUMPAD RTN", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_NUMPADSUB:   
				Asura_PrintfToBfr( pxDescString->GetString(), "%sNUMPAD -", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_NUMPADDEL:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sNUMPAD DEL", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_NUMPADDIVIDE:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sNUMPAD /", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_F1:
			case ASURA_KEYID_F2:
			case ASURA_KEYID_F3:
			case ASURA_KEYID_F4:
			case ASURA_KEYID_F5:
			case ASURA_KEYID_F6:
			case ASURA_KEYID_F7:
			case ASURA_KEYID_F8:								 
			case ASURA_KEYID_F9:
			case ASURA_KEYID_F10:
			case ASURA_KEYID_F11:
			case ASURA_KEYID_F12:
			case ASURA_KEYID_F13:
			case ASURA_KEYID_F14:
			case ASURA_KEYID_F15:
			case ASURA_KEYID_F16:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sF%d", iStringLength, true, pxDescString->GetString(), ( iKeyID + 1 - ASURA_KEYID_F1 ) );
				break;

			case ASURA_KEYID_NUMLOCK:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sNUM LOCK", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_SCROLLOK:	
				Asura_PrintfToBfr( pxDescString->GetString(), "%sSCROLL LOCK", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_LEFTSHIFT:	
				Asura_PrintfToBfr( pxDescString->GetString(), "%sL SHIFT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_RIGHTSHIFT:  
				Asura_PrintfToBfr( pxDescString->GetString(), "%sR SHIFT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_LEFTALT:     
				Asura_PrintfToBfr( pxDescString->GetString(), "%sL ALT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_RIGHTALT:    
				Asura_PrintfToBfr( pxDescString->GetString(), "%sR ALT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_LEFTCTRL:	
				Asura_PrintfToBfr( pxDescString->GetString(), "%sL CTRL", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_RIGHTCTRL:   
				Asura_PrintfToBfr( pxDescString->GetString(), "%sR CTRL", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_LWIN:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sL WIN", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_RWIN:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sR WIN", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_APPS:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sAPP", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_MOUSEBUTTON1:
			case ASURA_KEYID_MOUSEBUTTON2:
			case ASURA_KEYID_MOUSEBUTTON3:
			case ASURA_KEYID_MOUSEBUTTON4:
			case ASURA_KEYID_MOUSEBUTTON5:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sMB%d", iStringLength, true, pxDescString->GetString(), ( iKeyID + 1 - ASURA_KEYID_MOUSEBUTTON1 ) );
				break;

			case ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON1:
			case ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON2:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sD MB%d", iStringLength, true, pxDescString->GetString(), ( iKeyID + 1 - ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON1 ) );
				break;

			case ASURA_KEYID_MOUSEWHEELUP:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sWHEEL UP", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_MOUSEWHEELDOWN:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sWHEEL DOWN", iStringLength, true, pxDescString->GetString() );
				break;
		
			case ASURA_KEYID_JOYSTICK_BUTTON_1:
			case ASURA_KEYID_JOYSTICK_BUTTON_2:
			case ASURA_KEYID_JOYSTICK_BUTTON_3:
			case ASURA_KEYID_JOYSTICK_BUTTON_4:
			case ASURA_KEYID_JOYSTICK_BUTTON_5:
			case ASURA_KEYID_JOYSTICK_BUTTON_6:
			case ASURA_KEYID_JOYSTICK_BUTTON_7:
			case ASURA_KEYID_JOYSTICK_BUTTON_8:
			case ASURA_KEYID_JOYSTICK_BUTTON_9:
			case ASURA_KEYID_JOYSTICK_BUTTON_10:
			case ASURA_KEYID_JOYSTICK_BUTTON_11:
			case ASURA_KEYID_JOYSTICK_BUTTON_12:
			case ASURA_KEYID_JOYSTICK_BUTTON_13:
			case ASURA_KEYID_JOYSTICK_BUTTON_14:
			case ASURA_KEYID_JOYSTICK_BUTTON_15:
			case ASURA_KEYID_JOYSTICK_BUTTON_16:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sJOYSTICK%d", iStringLength, true, pxDescString->GetString(), ( iKeyID + 1 - ASURA_KEYID_JOYSTICK_BUTTON_1 ) );
				break;
			
			case ASURA_KEYID_PAD1_BUTTON1: 
			case ASURA_KEYID_PAD1_BUTTON2: 
			case ASURA_KEYID_PAD1_BUTTON3: 
			case ASURA_KEYID_PAD1_BUTTON4: 
			case ASURA_KEYID_PAD1_BUTTON5: 
			case ASURA_KEYID_PAD1_BUTTON6: 
			case ASURA_KEYID_PAD1_BUTTON7: 
			case ASURA_KEYID_PAD1_BUTTON8: 
			case ASURA_KEYID_PAD1_BUTTON9: 
			case ASURA_KEYID_PAD1_BUTTON10:
			case ASURA_KEYID_PAD1_BUTTON11:
			case ASURA_KEYID_PAD1_BUTTON12:
			case ASURA_KEYID_PAD1_BUTTON13:
			case ASURA_KEYID_PAD1_BUTTON14:
			case ASURA_KEYID_PAD1_BUTTON15:
			case ASURA_KEYID_PAD1_BUTTON16:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD1 BUT%d", iStringLength, true, pxDescString->GetString(), ( iKeyID + 1 - ASURA_KEYID_PAD1_BUTTON1 ) );
				break;

			case ASURA_KEYID_PAD2_BUTTON1: 
			case ASURA_KEYID_PAD2_BUTTON2: 
			case ASURA_KEYID_PAD2_BUTTON3: 
			case ASURA_KEYID_PAD2_BUTTON4: 
			case ASURA_KEYID_PAD2_BUTTON5: 
			case ASURA_KEYID_PAD2_BUTTON6: 
			case ASURA_KEYID_PAD2_BUTTON7: 
			case ASURA_KEYID_PAD2_BUTTON8: 
			case ASURA_KEYID_PAD2_BUTTON9: 
			case ASURA_KEYID_PAD2_BUTTON10:
			case ASURA_KEYID_PAD2_BUTTON11:
			case ASURA_KEYID_PAD2_BUTTON12:
			case ASURA_KEYID_PAD2_BUTTON13:
			case ASURA_KEYID_PAD2_BUTTON14:
			case ASURA_KEYID_PAD2_BUTTON15:
			case ASURA_KEYID_PAD2_BUTTON16:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD2 BUT%d", iStringLength, true, pxDescString->GetString(), ( iKeyID + 1 - ASURA_KEYID_PAD2_BUTTON1 ) );
				break;

			case ASURA_KEYID_PAD3_BUTTON1: 
			case ASURA_KEYID_PAD3_BUTTON2: 
			case ASURA_KEYID_PAD3_BUTTON3: 
			case ASURA_KEYID_PAD3_BUTTON4: 
			case ASURA_KEYID_PAD3_BUTTON5: 
			case ASURA_KEYID_PAD3_BUTTON6: 
			case ASURA_KEYID_PAD3_BUTTON7: 
			case ASURA_KEYID_PAD3_BUTTON8: 
			case ASURA_KEYID_PAD3_BUTTON9: 
			case ASURA_KEYID_PAD3_BUTTON10:
			case ASURA_KEYID_PAD3_BUTTON11:
			case ASURA_KEYID_PAD3_BUTTON12:
			case ASURA_KEYID_PAD3_BUTTON13:
			case ASURA_KEYID_PAD3_BUTTON14:
			case ASURA_KEYID_PAD3_BUTTON15:
			case ASURA_KEYID_PAD3_BUTTON16:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD3 BUT%d", iStringLength, true, pxDescString->GetString(), ( iKeyID + 1 - ASURA_KEYID_PAD3_BUTTON1 ) );
				break;
			
			case ASURA_KEYID_PAD4_BUTTON1: 
			case ASURA_KEYID_PAD4_BUTTON2: 
			case ASURA_KEYID_PAD4_BUTTON3: 
			case ASURA_KEYID_PAD4_BUTTON4: 
			case ASURA_KEYID_PAD4_BUTTON5: 
			case ASURA_KEYID_PAD4_BUTTON6: 
			case ASURA_KEYID_PAD4_BUTTON7: 
			case ASURA_KEYID_PAD4_BUTTON8: 
			case ASURA_KEYID_PAD4_BUTTON9: 
			case ASURA_KEYID_PAD4_BUTTON10:
			case ASURA_KEYID_PAD4_BUTTON11:
			case ASURA_KEYID_PAD4_BUTTON12:
			case ASURA_KEYID_PAD4_BUTTON13:
			case ASURA_KEYID_PAD4_BUTTON14:
			case ASURA_KEYID_PAD4_BUTTON15:
			case ASURA_KEYID_PAD4_BUTTON16:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD4 BUT%d", iStringLength, true, pxDescString->GetString(), ( iKeyID + 1 - ASURA_KEYID_PAD4_BUTTON1 ) );
				break;

			default:
				// if we get here the key will be a letter, number or punctuation, all of which use the standard ASCII values, cast back to chars. 
				Asura_PrintfToBfr( pxDescString->GetString(), "%s%c", 128, true, pxDescString->GetString(), static_cast<char>( iKeyID ) );
				break;
		}
	}
	else
	{
		// else if this is any kind of axis mapping
		switch( iKeyID )
		{
			case ASURA_AXIS_MOUSE_H:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sMOUSE HOR", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_AXIS_MOUSE_V:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sMOUSE VERT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_AXIS_PAD1_LSTICK_H:
			case ASURA_AXIS_PAD2_LSTICK_H:
			case ASURA_AXIS_PAD3_LSTICK_H:
			case ASURA_AXIS_PAD4_LSTICK_H:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD%d LS HOR", iStringLength, true, pxDescString->GetString(), ( ( iKeyID - ( ASURA_AXIS_PAD1_LSTICK_H - 4 ) ) / 4 ) );
				break;

			case ASURA_AXIS_PAD1_LSTICK_V:
			case ASURA_AXIS_PAD2_LSTICK_V:
			case ASURA_AXIS_PAD3_LSTICK_V:
			case ASURA_AXIS_PAD4_LSTICK_V:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD%d LS VERT", iStringLength, true, pxDescString->GetString(), ( ( iKeyID - ( ASURA_AXIS_PAD1_LSTICK_V - 4 ) ) / 4 ) );
				break;
			
			case ASURA_AXIS_PAD1_RSTICK_H:
			case ASURA_AXIS_PAD2_RSTICK_H:
			case ASURA_AXIS_PAD3_RSTICK_H:
			case ASURA_AXIS_PAD4_RSTICK_H:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD%d RS HOR", iStringLength, true, pxDescString->GetString(), ( ( iKeyID - ( ASURA_AXIS_PAD1_RSTICK_H - 4 ) ) / 4 ) );
				break;

			case ASURA_AXIS_PAD1_RSTICK_V:
			case ASURA_AXIS_PAD2_RSTICK_V:
			case ASURA_AXIS_PAD3_RSTICK_V:
			case ASURA_AXIS_PAD4_RSTICK_V:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD%d RSVERT", iStringLength, true, pxDescString->GetString(), ( ( iKeyID - ( ASURA_AXIS_PAD1_RSTICK_V - 4 ) ) / 4 ) );
				break;
			
			case ASURA_AXIS_PAD1_Z:
			case ASURA_AXIS_PAD2_Z:
			case ASURA_AXIS_PAD3_Z:
			case ASURA_AXIS_PAD4_Z:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD%d Z-AXIS", iStringLength, true, pxDescString->GetString(), ( ( iKeyID - ( ASURA_AXIS_PAD1_Z - 4 ) ) / 4 ) );
				break;

			case ASURA_AXIS_PAD1_ROTATE_X:
			case ASURA_AXIS_PAD2_ROTATE_X:
			case ASURA_AXIS_PAD3_ROTATE_X:
			case ASURA_AXIS_PAD4_ROTATE_X:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD%d ROT X-AXIS", iStringLength, true, pxDescString->GetString(), ( ( iKeyID - ( ASURA_AXIS_PAD1_ROTATE_X - 4 ) ) / 4 ) );
				break;

			case ASURA_AXIS_PAD1_ROTATE_Y:
			case ASURA_AXIS_PAD2_ROTATE_Y:
			case ASURA_AXIS_PAD3_ROTATE_Y:
			case ASURA_AXIS_PAD4_ROTATE_Y:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD%d ROT Y-AXIS", iStringLength, true, pxDescString->GetString(), ( ( iKeyID - ( ASURA_AXIS_PAD1_ROTATE_Y - 4 ) ) / 4 ) );
				break;

			case ASURA_AXIS_PAD1_ROTATE_Z:
			case ASURA_AXIS_PAD2_ROTATE_Z:
			case ASURA_AXIS_PAD3_ROTATE_Z:
			case ASURA_AXIS_PAD4_ROTATE_Z:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sPAD%d ROT Z-AXIS", iStringLength, true, pxDescString->GetString(), ( ( iKeyID - ( ASURA_AXIS_PAD1_ROTATE_Z - 4 ) ) / 4 ) );
				break;

			case ASURA_AXIS_VOID:
			default:
				Asura_PrintfToBfr( pxDescString->GetString(), "--", iStringLength, true );
				break;
		}
	}

	return true;
}

/// End of file. ///
