//////////////////////////////////////////////////////////////////////////////
//
// AVP_PC_External_Camera.cpp
//
// Created by Anthony Gregson - 09/11/07
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//		PC-specific external camera functions.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Camera/AVP_External_Camera.h"

// Asura headers
#include "Input/Asura_ControlManager.h"
#include "Input/Asura_ControlMap.h"

// Asura PC
#include "Input/Asura_PlatformInput.h"
#include "Input/Asura_PC_Input_Joystick.h"

// Game headers
#include "Core/AVP_Game_Core.h"


// Debugvars
DEBUGVAR float dbg_fTiltRampUpTime = 0.25f;
DEBUGVAR float dbg_fTiltRampDownTime = 0.05f;
DEBUGVAR float dbg_fTiltExponent = 2.0f;
DEBUGVAR float dbg_fMouseVSensitivity = 1.0f; // Bigger is quicker

DEBUGVAR float dbg_fTurnRampUpTime = 0.5f;
DEBUGVAR float dbg_fTurnRampDownTime = 0.10f;
DEBUGVAR float dbg_fMouseHSensitivity = 1.0f; // Bigger is quicker

/////////////////////
// Input functions //
/////////////////////

static void InitialiseControlsPC(Asura_ControlMap& xMap)
{
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_UP_DOWN, ASURA_AXIS_MOUSE_V, -dbg_fMouseVSensitivity);
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_LEFT_RIGHT, ASURA_AXIS_MOUSE_H, dbg_fMouseHSensitivity);

	xMap.AddKeyMapping(AVP_External_Camera::MOVE_FORWARDS_BACKWARDS,	's', Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_FORWARDS_BACKWARDS,	'w', Asura_ControlMapping::ON_PRESS, 1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_LEFT_RIGHT,			'a', Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_LEFT_RIGHT,			'd', Asura_ControlMapping::ON_PRESS, 1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN,				'f', Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN,				'v', Asura_ControlMapping::ON_PRESS, 1.0f);

	xMap.AddKeyMapping(AVP_External_Camera::RESPAWN, ASURA_KEYID_MOUSEBUTTON1, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::RESPAWN, ASURA_KEYID_SPACE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_MENU, ASURA_KEYID_ESCAPE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_OBJECTIVES, ASURA_KEYID_BACKSPACE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_DVS, ASURA_KEYID_TAB, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#else
	xMap.AddKeyMapping(AVP_External_Camera::SCOREBOARD, ASURA_KEYID_TAB, Asura_ControlMapping::ON_PRESS );
#endif //#ifdef ASURA_DEBUG_VARIABLE_SYSTEM

	xMap.AddKeyMapping(AVP_External_Camera::NEXT_PLAYER, ASURA_KEYID_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS); 
	xMap.AddKeyMapping(AVP_External_Camera::PREVIOUS_PLAYER, ASURA_KEYID_LEFT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_FOLLOW_PLAYER, ASURA_KEYID_DOWN, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_FIRST_PERSON, ASURA_KEYID_UP, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
}

static void InitialiseControlsXBox360(Asura_ControlMap& xMap)
{
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_UP_DOWN, ASURA_PC_AXISID_XBOX360PAD1_RSTICK_V, -1.0f, &dbg_fTiltRampUpTime, &dbg_fTiltRampDownTime, ASURA_CONTROLMAPPING_AXIS_MASK_ALL, &dbg_fTiltExponent);
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_LEFT_RIGHT, ASURA_PC_AXISID_XBOX360PAD1_RSTICK_H, 1.0f, &dbg_fTurnRampUpTime, &dbg_fTurnRampDownTime, ASURA_CONTROLMAPPING_AXIS_MASK_ALL, &dbg_fTiltExponent);

	xMap.AddAxisMapping(AVP_External_Camera::MOVE_FORWARDS_BACKWARDS, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_V);
	xMap.AddAxisMapping(AVP_External_Camera::MOVE_LEFT_RIGHT, ASURA_PC_AXISID_XBOX360PAD1_LSTICK_H);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN, ASURA_PC_KEYID_XBOX360PAD1_R_TRIGGER, Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN, ASURA_PC_KEYID_XBOX360PAD1_L_TRIGGER, Asura_ControlMapping::ON_PRESS, 1.0f);

	xMap.AddKeyMapping(AVP_External_Camera::RESPAWN, ASURA_PC_KEYID_XBOX360PAD1_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_MENU, ASURA_PC_KEYID_XBOX360PAD1_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_OBJECTIVES, ASURA_PC_KEYID_XBOX360PAD1_L_THUMB, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_DVS, ASURA_PC_KEYID_XBOX360PAD1_BACK, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#else
	xMap.AddKeyMapping(AVP_External_Camera::SCOREBOARD, ASURA_PC_KEYID_XBOX360PAD1_BACK, Asura_ControlMapping::ON_PRESS );
#endif //#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
}

static void InitialiseControlsPS2(Asura_ControlMap& xMap)
{
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_UP_DOWN, ASURA_PC_AXISID_PS2PAD1_RSTICK_V, -1.0f, &dbg_fTiltRampUpTime, &dbg_fTiltRampDownTime, ASURA_CONTROLMAPPING_AXIS_MASK_ALL, &dbg_fTiltExponent);
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_LEFT_RIGHT, ASURA_PC_AXISID_PS2PAD1_RSTICK_H, 1.0f, &dbg_fTurnRampUpTime, &dbg_fTurnRampDownTime, ASURA_CONTROLMAPPING_AXIS_MASK_ALL, &dbg_fTiltExponent);

	xMap.AddAxisMapping(AVP_External_Camera::MOVE_FORWARDS_BACKWARDS, ASURA_PC_AXISID_PS2PAD1_LSTICK_V);
	xMap.AddAxisMapping(AVP_External_Camera::MOVE_LEFT_RIGHT, ASURA_PC_AXISID_PS2PAD1_LSTICK_H);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN, ASURA_PC_KEYID_PS2PAD1_R1, Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN, ASURA_PC_KEYID_PS2PAD1_R2, Asura_ControlMapping::ON_PRESS, 1.0f);

	xMap.AddKeyMapping(AVP_External_Camera::RESPAWN, ASURA_PC_KEYID_PS2PAD1_CROSS, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_MENU, ASURA_PC_KEYID_PS2PAD1_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_OBJECTIVES, ASURA_PC_KEYID_PS2PAD1_CIRCLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_DVS, ASURA_PC_KEYID_PS2PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#else
	xMap.AddKeyMapping(AVP_External_Camera::SCOREBOARD, ASURA_PC_KEYID_PS2PAD1_SELECT, Asura_ControlMapping::ON_PRESS );
#endif //#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
}

static void InitialiseControlsPS3(Asura_ControlMap& xMap)
{
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_UP_DOWN, ASURA_PC_AXISID_PS3PAD1_RSTICK_V, -1.0f, &dbg_fTiltRampUpTime, &dbg_fTiltRampDownTime, ASURA_CONTROLMAPPING_AXIS_MASK_ALL, &dbg_fTiltExponent);
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_LEFT_RIGHT, ASURA_PC_AXISID_PS3PAD1_RSTICK_H, 1.0f, &dbg_fTurnRampUpTime, &dbg_fTurnRampDownTime, ASURA_CONTROLMAPPING_AXIS_MASK_ALL, &dbg_fTiltExponent);

	xMap.AddAxisMapping(AVP_External_Camera::MOVE_FORWARDS_BACKWARDS, ASURA_PC_AXISID_PS3PAD1_LSTICK_V);
	xMap.AddAxisMapping(AVP_External_Camera::MOVE_LEFT_RIGHT, ASURA_PC_AXISID_PS3PAD1_LSTICK_H);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN, ASURA_PC_KEYID_PS3PAD1_R1, Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN, ASURA_PC_KEYID_PS3PAD1_R2, Asura_ControlMapping::ON_PRESS, 1.0f);

	xMap.AddKeyMapping(AVP_External_Camera::RESPAWN, ASURA_PC_KEYID_PS3PAD1_CROSS, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_MENU, ASURA_PC_KEYID_PS3PAD1_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_OBJECTIVES, ASURA_PC_KEYID_PS3PAD1_CIRCLE, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#ifdef ASURA_DEBUG_VARIABLE_SYSTEM
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_DVS, ASURA_PC_KEYID_PS3PAD1_SELECT, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
#else
	xMap.AddKeyMapping(AVP_External_Camera::SCOREBOARD, ASURA_PC_KEYID_PS3PAD1_SELECT, Asura_ControlMapping::ON_PRESS );
#endif //#ifdef ASURA_DEBUG_VARIABLE_SYSTEM

}

void AVP_External_Camera::Platform_FillControlMap()
{
	if (!m_pxControlMap) { return; }

	const u_int uNumJoysticks = Asura_PC_Input_Joystick::GetNumJoysticks();

	// Add each connected joystick in-turn...
	for(u_int u = 0; u < uNumJoysticks; u++)
	{
		const Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE	eJoystickType	= Asura_PC_Input_Joystick::GetJoystickType(u);

		switch(eJoystickType)
		{
			case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_PS2_GAMEPAD:
			{
				InitialiseControlsPS2(*m_pxControlMap);
				break;
			}

			case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_PS3_GAMEPAD:
			{
				InitialiseControlsPS3(*m_pxControlMap);
				break;
			}

			#ifdef ASURA_PC_SUPPORT_XINPUT
			case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_XBOX360_XINPUT_GAMEPAD:
			#endif
			case Asura_PC_Input_Joystick::ASURA_PC_JS_TYPE_XBOX360_GAMEPAD:
			{
				InitialiseControlsXBox360(*m_pxControlMap);
				break;
			}

			default:
			{
				InitialiseControlsPC(*m_pxControlMap);
				break;
			}
		}		// End of switch.
	}

	// Always ensure that we've bound the mouse/keyboard controls...
	InitialiseControlsPC(*m_pxControlMap);
}

#ifdef AVP_SECOND_CONTROL_FLY_CAM

void AVP_External_Camera::Platform_FillSecondControlMap()
{
	//MKC - Not using the external camera as fly cam on the PC version so not mapping any controls
}
#endif

/// End of file. ///
