//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_Edges.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined ASURA_USE_FSFX

// Asura Platform-independent
#include "FSFX/Asura_FSFX_System.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_ViewPort.h"
#include "Rendering/Asura_Overlay_Quad.h"


// Asura PC
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"

// AVP Platform-independent
#include "FSFX/AVP_FSFX.h"

// Misc
#include <d3d9.h>

#include "AVP_FSFX_Edges.h"
ASURA_DECLARE_DIRECT3D_EFFECT(AVP_FSFX_Edges);

static Asura_Direct3D_Effect*		g_pxEdgesEffect					= NULL;
static Direct3D_VertexDeclaration*	g_pxEdgesVertexDecl				= NULL;
static D3DXHANDLE					g_pxEdgesInitialImageHandle		= 0;
static D3DXHANDLE					g_pxEdgesInvScreenDimsHandle	= 0;
static D3DXHANDLE					g_pxEdgesTintHandle				= 0;
static D3DXHANDLE					g_pxEdgesParamsHandle			= 0;

static Direct3D_Texture *g_pxRenderTarget;
static Direct3D_Surface *g_pxRenderTargetSurface;


static void OnEffectUnloaded(Asura_Direct3D_Effect* pxEffect)
{
	g_pxEdgesEffect = NULL;
}

static void OnEffectLoaded(Asura_Direct3D_Effect* pxEffect)
{
#define LOADPARAMETER(codeName, shaderName) { codeName = g_pxEdgesEffect->GetD3DEffect()->GetParameterByName(NULL, shaderName); Asura_Assert(codeName, "Could not get shader parameter '" shaderName "'."); }

	g_pxEdgesEffect = Asura_Direct3D_Effects_System::GetEffect("AVP_FSFX_Edges");

	if (!g_pxEdgesEffect)
	{
		return;
	}

#if ASURA_LOAD_EFFECT_FILES == 1
	g_pxEdgesEffect->SetOnLoadCallback(OnEffectLoaded);
	g_pxEdgesEffect->SetOnUnloadCallback(OnEffectUnloaded);
#endif

	// Get the params
	LOADPARAMETER(g_pxEdgesInitialImageHandle, "g_xInitialImageTexture" );
	LOADPARAMETER(g_pxEdgesInvScreenDimsHandle, "g_fInvScreenDims" );
	LOADPARAMETER(g_pxEdgesTintHandle, "g_fTint" );
	LOADPARAMETER(g_pxEdgesParamsHandle, "g_fEdgeDetectionParams" );

#undef LOADPARAMETER
}

static bool PostReset(void*)
{
	Asura_Direct3D::D3DDevice->CreateTexture(Asura_Viewport::GetWidth() /2, Asura_Viewport::GetHeight() / 2, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &g_pxRenderTarget, NULL);
	g_pxRenderTarget->GetSurfaceLevel(0, &g_pxRenderTargetSurface);
	return true;
}
static bool PreReset(void*)
{
	SAFE_RELEASE(g_pxRenderTarget);
	SAFE_RELEASE(g_pxRenderTargetSurface);
	return true;
}



void AVP_FSFX_Component_Edges::Platform_Initialise(  )
{
	const Direct3D_VertexElement axVertDecl[] =
	{
		{ 0, 0, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },	// Position
		{ 0, 8, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },	// UVs
		D3DDECL_END()
	};

	Asura_Direct3D::D3DDevice->CreateVertexDeclaration( axVertDecl, &g_pxEdgesVertexDecl );

	OnEffectLoaded(NULL);

	Asura_Direct3D::AddPreResetCallback(PreReset, NULL);
	Asura_Direct3D::AddPostResetCallback(PostReset, NULL);

	PostReset(NULL);



}
void AVP_FSFX_Component_Edges::Platform_Shutdown(  )
{
	Asura_Direct3D::RemovePostResetCallback(PostReset, NULL);
	Asura_Direct3D::RemovePreResetCallback(PreReset, NULL);

	PreReset(NULL);

	if (g_pxEdgesVertexDecl)
	{
		g_pxEdgesVertexDecl->Release();
		g_pxEdgesVertexDecl = NULL;
	}
}



void AVP_FSFX_Component_Edges::Platform_Render( float fIntensity )
{
	Asura_Direct3D_Effect* const pxEffect = g_pxEdgesEffect;
	if( !pxEffect || !pxEffect->IsD3DEffectValid() ) return;

	SCOPEDPIXEVENT(( 0xFFE0D0C0, L"FSFX: AVP - Edges" ));

	// Setup
	Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();

	pxEffect->SetParameterTexture( g_pxEdgesInitialImageHandle, Asura_Direct3D_BackBufferCapture::GetCaptureTexture() );
	pxEffect->SetParameter( g_pxEdgesInvScreenDimsHandle, D3DXVECTOR4(1.0f / Asura_Viewport::GetWidth(), 1.0f / Asura_Viewport::GetHeight(), 0.0f, 0.0f));
	pxEffect->SetParameter( g_pxEdgesParamsHandle, D3DXVECTOR4(m_fLowThreshold * 0.1f, m_fHighThreshold * 0.1f, m_fEDWidth * 0.1f, 0.0f));

	Asura_Render::AllowFullScreenRendering();

	Direct3D_Surface* pxOldRenderTargetBuffer[5] = 
	{
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
	};

	for (int j = 0; j < 4; ++j)
	{
		if (FAILED(Asura_Direct3D::D3DDevice->GetRenderTarget(j, &pxOldRenderTargetBuffer[j])))
		{
			pxOldRenderTargetBuffer[j] = NULL;
		}

	}
	if (FAILED(Asura_Direct3D::D3DDevice->GetDepthStencilSurface(&pxOldRenderTargetBuffer[4])))
	{
		pxOldRenderTargetBuffer[4] = NULL;
	}

	Asura_Direct3D::D3DDevice->SetRenderTarget(0, g_pxRenderTargetSurface);
	Asura_Direct3D::D3DDevice->SetRenderTarget(1, NULL);
	Asura_Direct3D::D3DDevice->SetRenderTarget(2, NULL);
	Asura_Direct3D::D3DDevice->SetRenderTarget(3, NULL);
	Asura_Direct3D::D3DDevice->SetDepthStencilSurface(NULL);
	Asura_Direct3D::D3DDevice->Clear(0, NULL, D3DCLEAR_TARGET, 0, 0.0f, 0);




	Asura_Render::CurrentStates.RequestAlphaTest( false );
	Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_NONE );
	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_Render::CurrentStates.RequestZBufferEnabled( false );

	Direct3D_VertexDeclaration* const pxVD = g_pxEdgesVertexDecl;
	Asura_Direct3D::D3DDevice->SetVertexDeclaration( pxVD );

	const float fWidth = static_cast< float >( Asura_Viewport::GetWidth() );
	const float fHeight = static_cast< float >( Asura_Viewport::GetHeight() );

	const struct Vertex { float fX, fY, fU, fV; } axVerts[] =
	{
		{ -1.0f,  1.0f, 0.5f / fWidth,            0.5f / fHeight },
		{  1.0f,  1.0f, (fWidth + 0.5f) / fWidth, 0.5f / fHeight },
		{  1.0f, -1.0f, (fWidth + 0.5f) / fWidth, (fHeight + 0.5f) / fHeight },
		{ -1.0f, -1.0f, 0.5f / fWidth,            (fHeight + 0.5f) / fHeight },
	};

	Asura_Direct3D_Effects_System::BeginEffect( *pxEffect, m_uMethod );
	pxEffect->BeginPass(0);

	Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( Vertex ) );

	pxEffect->EndPass();


	Asura_Direct3D::D3DDevice->SetRenderTarget(0, pxOldRenderTargetBuffer[0]);
	Asura_Direct3D::D3DDevice->SetRenderTarget(1, pxOldRenderTargetBuffer[1]);
	Asura_Direct3D::D3DDevice->SetRenderTarget(2, pxOldRenderTargetBuffer[2]);
	Asura_Direct3D::D3DDevice->SetRenderTarget(3, pxOldRenderTargetBuffer[3]);
	Asura_Direct3D::D3DDevice->SetDepthStencilSurface(pxOldRenderTargetBuffer[4]);
	SAFE_RELEASE(pxOldRenderTargetBuffer[0]);
	SAFE_RELEASE(pxOldRenderTargetBuffer[1]);
	SAFE_RELEASE(pxOldRenderTargetBuffer[2]);
	SAFE_RELEASE(pxOldRenderTargetBuffer[3]);
	SAFE_RELEASE(pxOldRenderTargetBuffer[4]);
	pxEffect->SetParameterTexture( g_pxEdgesInitialImageHandle, g_pxRenderTarget );
	fIntensity *= m_fAmount * 0.01f;
	pxEffect->SetParameter( g_pxEdgesTintHandle, D3DXVECTOR4(m_xTintColour.m_fR * fIntensity, m_xTintColour.m_fG * fIntensity, m_xTintColour.m_fB * fIntensity,  m_fBlurWidth * 0.1f));

	Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_GLOWING);


	pxEffect->BeginPass(1);

	Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( Vertex ) );

	pxEffect->EndPass();


	Asura_Direct3D_Effects_System::FlushEffects();

	// Reset this parameter to NULL since the texture is in the default pool (for device resets)
	pxEffect->SetParameterTexture( g_pxEdgesInitialImageHandle, NULL );
	Asura_Direct3D_BackBufferCapture::AdviseNextFrame();

	for( u_int u = 0; u < 16; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_SRGBTEXTURE, FALSE );
	}

}
#endif //defined ASURA_USE_FSFX
