//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_Font_System.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Rendering/AVP_Font_System.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_TiledTextureManagement.h"

#include "Core/AVP_Core.h"

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Buttons::s_auSpecialCharTiles[AVP_FONTSYSTEM_NUM_SPECIAL_CHARS] =
{
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_NA,			0xE800	Asura_GetHashID("pc_NA.tga")	N/A

	0x8C640044,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_1,		0xE801	Asura_GetHashID("pc_0.tga")	0
	0x8C7217C5,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_2,		0xE802	Asura_GetHashID("pc_1.tga")	1
	0x8C802F46,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_3,		0xE803	Asura_GetHashID("pc_2.tga")	2
	0x8C8E46C7,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_4,		0xE804	Asura_GetHashID("pc_3.tga")	3
	0x8C9C5E48,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_5,		0xE805	Asura_GetHashID("pc_4.tga")	4
	0x8CAA75C9,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_6,		0xE806	Asura_GetHashID("pc_5.tga")	5
	0x8CB88D4A,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_7,		0xE807	Asura_GetHashID("pc_6.tga")	6
	0x8CC6A4CB,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_8,		0xE808	Asura_GetHashID("pc_7.tga")	7
	0x8CD4BC4C,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_9,		0xE809	Asura_GetHashID("pc_8.tga")	8
	0x8CE2D3CD,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_10,		0xE80A	Asura_GetHashID("pc_9.tga")	9

	// NB buttons 10 to 16 are for PC controllers and joysticks
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_11,		0xE80B	Asura_GetHashID("pc_NA.tga")	10
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_12,		0xE80C	Asura_GetHashID("pc_NA.tga")	11
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_13,		0xE80D	Asura_GetHashID("pc_NA.tga")	12
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_14,		0xE80E	Asura_GetHashID("pc_NA.tga")	13
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_15,		0xE80F	Asura_GetHashID("pc_NA.tga")	14
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_16,		0xE810	Asura_GetHashID("pc_NA.tga")	15
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_17,		0xE811	Asura_GetHashID("pc_NA.tga")	16

	0x8F167FF5,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_18,		0xE812	Asura_GetHashID("pc_a.tga")	A
	0x8F249776,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_19,		0xE813	Asura_GetHashID("pc_b.tga")	B
	0x8F32AEF7,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_20,		0xE814	Asura_GetHashID("pc_c.tga")	C
	0x8F40C678,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_21,		0xE815	Asura_GetHashID("pc_d.tga")	D
	0x8F4EDDF9,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_22,		0xE816	Asura_GetHashID("pc_e.tga")	E
	0x8F5CF57A,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_23,		0xE817	Asura_GetHashID("pc_f.tga")	F
	0x8F6B0CFB,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_24,		0xE818	Asura_GetHashID("pc_g.tga")	G
	0x8F79247C,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_25,		0xE819	Asura_GetHashID("pc_h.tga")	H
	0x8F873BFD,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_26,		0xE81A	Asura_GetHashID("pc_i.tga")	I
	0x8F95537E,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_27,		0xE81B	Asura_GetHashID("pc_j.tga")	J
	0x8FA36AFF,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_28,		0xE81C	Asura_GetHashID("pc_k.tga")	K
	0x8FB18280,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_29,		0xE81D	Asura_GetHashID("pc_l.tga")	L
	0x8FBF9A01,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_30,		0xE81E	Asura_GetHashID("pc_m.tga")	M
	0x8FCDB182,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_31,		0xE81F	Asura_GetHashID("pc_n.tga")	N
	0x8FDBC903,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_32,		0xE820	Asura_GetHashID("pc_o.tga")	O
	0x8FE9E084,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_33,		0xE821	Asura_GetHashID("pc_p.tga")	P
	0x8FF7F805,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_34,		0xE822	Asura_GetHashID("pc_q.tga")	Q
	0x90060F86,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_35,		0xE823	Asura_GetHashID("pc_r.tga")	R
	0x90142707,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_36,		0xE824	Asura_GetHashID("pc_s.tga")	S
	0x90223E88,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_37,		0xE825	Asura_GetHashID("pc_t.tga")	T
	0x90305609,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_38,		0xE826	Asura_GetHashID("pc_u.tga")	U
	0x903E6D8A,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_39,		0xE827	Asura_GetHashID("pc_v.tga")	V
	0x904C850B,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_40,		0xE828	Asura_GetHashID("pc_w.tga")	W
	0x905A9C8C,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_41,		0xE829	Asura_GetHashID("pc_x.tga")	S
	0x9068B40D,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_42,		0xE82A	Asura_GetHashID("pc_y.tga")	Y
	0x9076CB8E,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_43,		0xE82B	Asura_GetHashID("pc_z.tga")	Z

	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_44,		0xE82C	Asura_GetHashID("pc_NA.tga")	F1
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_45,		0xE82D	Asura_GetHashID("pc_NA.tga")	F2
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_46,		0xE82E	Asura_GetHashID("pc_NA.tga")	F3
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_47,		0xE82F	Asura_GetHashID("pc_NA.tga")	F4
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_48,		0xE830	Asura_GetHashID("pc_NA.tga")	F5
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_49,		0xE831	Asura_GetHashID("pc_NA.tga")	F6
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_50,		0xE832	Asura_GetHashID("pc_NA.tga")	F7
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_51,		0xE833	Asura_GetHashID("pc_NA.tga")	F8
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_52,		0xE834	Asura_GetHashID("pc_NA.tga")	F9
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_53,		0xE835	Asura_GetHashID("pc_NA.tga")	F10
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_54,		0xE836	Asura_GetHashID("pc_NA.tga")	F11
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_55,		0xE837	Asura_GetHashID("pc_NA.tga")	F12
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_56,		0xE838	Asura_GetHashID("pc_NA.tga")	F13
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_57,		0xE839	Asura_GetHashID("pc_NA.tga")	F14
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_58,		0xE83A	Asura_GetHashID("pc_NA.tga")	F15
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_59,		0xE83B	Asura_GetHashID("pc_NA.tga")	F16

	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_60,		0xE83C	Asura_GetHashID("pc_NA.tga")	`
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_61,		0xE83D	Asura_GetHashID("pc_NA.tga")	-
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_62,		0xE83E	Asura_GetHashID("pc_NA.tga")	=
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_63,		0xE83F	Asura_GetHashID("pc_NA.tga")	[
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_64,		0xE840	Asura_GetHashID("pc_NA.tga")	]
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_65,		0xE841	Asura_GetHashID("pc_NA.tga")	;
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_66,		0xE842	Asura_GetHashID("pc_NA.tga")	'
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_67,		0xE843	Asura_GetHashID("pc_NA.tga")	#
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_68,		0xE844	Asura_GetHashID("pc_NA.tga")	,
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_69,		0xE845	Asura_GetHashID("pc_NA.tga")	.
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_70,		0xE846	Asura_GetHashID("pc_NA.tga")	/
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_71,		0xE847	Asura_GetHashID("pc_NA.tga")	'\'
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_72,		0xE848	Asura_GetHashID("pc_NA.tga")	*
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_73,		0xE849	Asura_GetHashID("pc_NA.tga")	+
	
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_74,		0xE84A	Asura_GetHashID("pc_NA.tga")	BACKSPACE
	0x667CA829,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_75,		0xE84B	Asura_GetHashID("pc_tab.tga")	TAB
	0xE386F05C,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_76,		0xE84C	Asura_GetHashID("pc_return.tga")ENTER
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_77,		0xE84D	Asura_GetHashID("pc_NA.tga")	NUMPAD ENTER
	0x6BC48209,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_78,		0xE84E	Asura_GetHashID("pc_esc.tga")	ESC
	0x607E1C1A,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_79,		0xE84F	Asura_GetHashID("pc_space.tga")	SPACE

	0xCE987996,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_80,		0xE850	Asura_GetHashID("pc_shift.tga")	SHIFT
	0xCE987996,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_81,		0xE851	Asura_GetHashID("pc_shift.tga")	LEFT SHIFT
	0xCE987996,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_82,		0xE852	Asura_GetHashID("pc_shift.tga")	RIGHT SHIFT
	0x8ED26317,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_83,		0xE853	Asura_GetHashID("pc_ctrl.tga")	CTRL
	0x8ED26317,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_84,		0xE854	Asura_GetHashID("pc_ctrl.tga")	LEFT CTRL
	0x8ED26317,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_85,		0xE855	Asura_GetHashID("pc_ctrl.tga")	RIGHT CTRL
	0x8D29383D,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_86,		0xE856	Asura_GetHashID("pc_alt.tga")	ALT
	0x8D29383D,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_87,		0xE857	Asura_GetHashID("pc_alt.tga")	LEFT ALT
	0x8D29383D,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_88,		0xE858	Asura_GetHashID("pc_alt.tga")	RIGHT ALT

	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_89,		0xE859	Asura_GetHashID("pc_NA.tga")	WINDOWS
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_90,		0xE85A	Asura_GetHashID("pc_NA.tga")	APPS

	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_91,		0xE85B	Asura_GetHashID("pc_NA.tga")	CAPS LOCK
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_92,		0xE85C	Asura_GetHashID("pc_NA.tga")	NUM LOCK
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_93,		0xE85D	Asura_GetHashID("pc_NA.tga")	SCROLL LOCK
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_94,		0xE85E	Asura_GetHashID("pc_NA.tga")	PAUSE

	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_95,		0xE85F	Asura_GetHashID("pc_NA.tga")	UP
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_96,		0xE860	Asura_GetHashID("pc_NA.tga")	DOWN
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_97,		0xE861	Asura_GetHashID("pc_NA.tga")	LEFT
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_98,		0xE862	Asura_GetHashID("pc_NA.tga")	RIGHT

	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_99,		0xE863	Asura_GetHashID("pc_NA.tga")	INSERT
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_100,	0xE864	Asura_GetHashID("pc_NA.tga")	DELETE
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_101,	0xE865	Asura_GetHashID("pc_NA.tga")	HOME
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_102,	0xE866	Asura_GetHashID("pc_NA.tga")	END
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_103,	0xE867	Asura_GetHashID("pc_NA.tga")	PAGE UP
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_104,	0xE868	Asura_GetHashID("pc_NA.tga")	PAGE DOWN

	0xF3336250,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_105,	0xE869	Asura_GetHashID("pc_mb1.tga")	MOUSE1
	0xF34179D1,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_106,	0xE86A	Asura_GetHashID("pc_mb2.tga")	MOUSE2
	0xF34F9152,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_107,	0xE86B	Asura_GetHashID("pc_mb3.tga")	MOUSE3
	0xF35DA8D3,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_108,	0xE86C	Asura_GetHashID("pc_mb4.tga")	MOUSE4
	0xF36BC054,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_109,	0xE86D	Asura_GetHashID("pc_mb5.tga")	MOUSE5
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_110,	0xE86E	Asura_GetHashID("pc_NA.tga")	MOUSE1 DOUBLE
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_111,	0xE86F	Asura_GetHashID("pc_NA.tga")	MOUSE2 DOUBLE
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_112,	0xE870	Asura_GetHashID("pc_NA.tga")	MOUSE WHEEL UP
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_113,	0xE871	Asura_GetHashID("pc_NA.tga")	MOUSE WHEEL DOWN

	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_114,	0xE872	Asura_GetHashID("pc_NA.tga")	NUMPAD 0
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_115,	0xE873	Asura_GetHashID("pc_NA.tga")	NUMPAD 1
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_116,	0xE874	Asura_GetHashID("pc_NA.tga")	NUMPAD 2
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_117,	0xE875	Asura_GetHashID("pc_NA.tga")	NUMPAD 3
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_118,	0xE876	Asura_GetHashID("pc_NA.tga")	NUMPAD 4
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_119,	0xE877	Asura_GetHashID("pc_NA.tga")	NUMPAD 5
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_120,	0xE878	Asura_GetHashID("pc_NA.tga")	NUMPAD 6
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_121,	0xE879	Asura_GetHashID("pc_NA.tga")	NUMPAD 7
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_122,	0xE87A	Asura_GetHashID("pc_NA.tga")	NUMPAD 8
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_123,	0xE87B	Asura_GetHashID("pc_NA.tga")	NUMPAD 9

	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_LSTICK_H,	0xE87C	Asura_GetHashID("pc_NA.tga")	AXIS LS H
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_LSTICK_V,	0xE87D	Asura_GetHashID("pc_NA.tga")	AXIS LS V
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_RSTICK_H,	0xE87E	Asura_GetHashID("pc_NA.tga")	AXIS RS H
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_RSTICK_V,	0xE87F	Asura_GetHashID("pc_NA.tga")	AXIS RS V
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_Z,		0xE880	Asura_GetHashID("pc_NA.tga")	AXIS Z
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_X,	0xE881	Asura_GetHashID("pc_NA.tga")	AXIS ROT X
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_Y,	0xE882	Asura_GetHashID("pc_NA.tga")	AXIS ROT Y
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_Z,	0xE883	Asura_GetHashID("pc_NA.tga")	AXIS ROT Z
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_MOUSE_H,	0xE884	Asura_GetHashID("pc_NA.tga")	AXIS MOUSE H
	0x6C978D5F,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_MOUSE_V,	0xE885	Asura_GetHashID("pc_NA.tga")	AXIS MOUSE V
};	

void AVP_Buttons::Platform_Initialise()
{
	// if the special char tiles haven't been loaded, attempt to load them here
	if( !Asura_TiledTextureManagement::GetTileInfoByHash( AVP_Buttons::s_auSpecialCharTiles[0] ) )
	{
#if defined(ASURA_ASSERT)
		bool bOK = 
#endif
		AVP_Core::LoadFile( "graphics\\pc_buttons." ASURA_STANDARD_FILE_EXTENSION, ASURA_TEXTURE_LOADED_PERMANENT );
		Asura_Assert(bOK, "failed to load button icons file");
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
