/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Includes must be this way round here
#include "..\..\..\..\..\Asura\PC\Shaders\Shared.fxh"
#include "..\..\..\..\..\Asura\PC\Shaders\MaterialResponse_Shared.fxh"

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float4 g_xColour;
const float g_fTime;

const float4 g_xParams;
// x const float g_fBeamRadius;
// y const float g_fBeamSharpness;
// z const float g_fBeamSpread;
// w const float 1.0f / g_fBeamMaxCameraDistance;

const float2 g_xBeamFadeOff;
// x = start
// y = length



const float2 g_xOneOverScreenSize;


const texture g_xSceneTexture;
const texture g_xDepthTexture;
const texture g_xNoiseTexture;


const float g_fFSPassIntensity;
const sampler g_xSceneTextureSampler  = sampler_state
{
	Texture   = <g_xSceneTexture>;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
	MinFilter = POINT;
	MagFilter = POINT;
	MIPFILTER = POINT;
};
const sampler g_xDepthTextureSampler  = sampler_state
{
	Texture   = <g_xDepthTexture>;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
	MinFilter = POINT;
	MagFilter = POINT;
	MIPFILTER = POINT;
};

const sampler g_xNoiseTextureSampler  = sampler_state
{
	Texture   = <g_xNoiseTexture>;
	AddressU  = WRAP;
	AddressV  = WRAP;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MIPFILTER = LINEAR;
};


/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 m_xPosition     : POSITION;
	float2 m_xPosition2	   : TEXCOORD;
};

struct VertexFS
{
	float4 m_xPosition : POSITION;
	float2 m_xUV       : TEXCOORD;
};


struct OutputVertex
{
	float4 m_xPosition : POSITION;
	float4 m_xWorldPos : TEXCOORD;
	struct {
		float m_fX;
		float m_fY;
		float m_fDepth;
		float m_fLengthFade;
	} uv1 : TEXCOORD1;
};


OutputVertex VShader( const InputVertex xIn )
{
	OutputVertex xOut;
	xOut.m_xWorldPos = xIn.m_xPosition;
	xOut.m_xWorldPos.w = 1.0f;
	 
	xOut.m_xPosition = mul(xOut.m_xWorldPos, g_xViewProjectionMatrix);
	xOut.uv1.m_fLengthFade = (xIn.m_xPosition2.y - g_xBeamFadeOff.x) * g_xBeamFadeOff.y;
	xOut.uv1.m_fY = g_xParams.y + g_xParams.z * xIn.m_xPosition2.y;
	xOut.uv1.m_fX = (xIn.m_xPosition2.x - g_xParams.x);
	xOut.uv1.m_fDepth = xOut.m_xPosition.w;
	
	return xOut;
}
const float s_fContrast = 50.0f;

float3 NoiseSample(float3 fVal)
{
	return fVal;
//	return step(fVal, 0.95f) + fVal * 2.0f - 1.0f;
}


float3 NoiseLookup(float3 xPos)
{
	float3 fNoise;
	fNoise  = NoiseSample(tex2D(g_xNoiseTextureSampler, xPos.xy));
	fNoise *= NoiseSample(tex2D(g_xNoiseTextureSampler, xPos.yz));
	fNoise *= NoiseSample(tex2D(g_xNoiseTextureSampler, xPos.zx));

	return fNoise;
}

float4 PShader( OutputVertex xIn, const float2 xScreenPos : VPOS) : COLOR
{
	float3 p = (xIn.m_xWorldPos - g_xCameraPosition) + (g_xCameraPosition * 0.1f);
	float3 xEyeVec = xIn.m_xWorldPos - g_xCameraPosition;
	float fDistance = length(xEyeVec);
	xEyeVec /= fDistance;
	 
	float fIntens = 1.0f - saturate(xIn.uv1.m_fX / xIn.uv1.m_fY);
	fIntens = pow(saturate(fIntens), 10.0f);

	{
		// Do the soft particle stuff
		const float2 xNormalizedCoords = (xScreenPos + float2(0.5, 0.5)) * g_xOneOverScreenSize;
		float fDepth = abs(tex2D(g_xDepthTextureSampler, xNormalizedCoords).z);
		
	    fIntens *= min(fDepth - xIn.uv1.m_fDepth,1.0f);
	}
	float3 fOffset = float3(sin(g_fTime * 0.1f * 0.3f), cos(g_fTime * 0.13f * 0.3f), cos(g_fTime * 0.17f * 0.3f));
	float fNoise = 6.0f * pow(3.0f * max(NoiseLookup(p * 2.0f + fOffset * 0.8f).g, 0.0f), 2.0f);
	fNoise += 60.0f * max(NoiseLookup(p * 4.0f + fOffset * 0.8f).r, 0.0f) * (1.0f - saturate(fDistance / 2.0f)) * fNoise;
	fNoise = max(fNoise + fIntens * 0.3f, 0.0f);



	float fDistance2 = 1.0f - pow(saturate(fDistance * g_xParams.w ), 3.0f);


	fDistance = min(fDistance * 2, 1.0f);
	float fLengthFade = 1.0f - clamp(xIn.uv1.m_fLengthFade, 0.0f, 1.0f);
	float4 xColour = float4(g_xColour.rgb, g_xColour.a * fNoise * fLengthFade * pow(fIntens, 3.0f) * pow(fDistance, 3.0f) * fDistance2);
	return xColour;
}
	
float4 VShader_FullScreen( const InputVertex xIn ) : POSITION
{
	return xIn.m_xPosition;
}

float4 PShader_FullScreen( const float2 xScreenPos : VPOS ) : COLOR
{
	float4 xInitialColour = tex2D(g_xSceneTextureSampler, (xScreenPos + float2(0.5, 0.5)) * g_xOneOverScreenSize);
	float fL = dot(float3(0.30, 0.59, 0.11), xInitialColour.rgb);
	
 	fL = (1.0f - min(fL, 1.0f));

 	return float4(g_xColour.rgb, g_xColour.a) * fL;
}


/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}

technique
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_FullScreen();
		PixelShader = compile ps_3_0 PShader_FullScreen();
	}
}