/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Includes must be this way round here
#include "..\..\..\..\..\Asura\PC\Shaders\Shared.fxh"
#include "..\..\..\..\..\Asura\PC\Shaders\MaterialResponse_Shared.fxh"

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float g_fTime;
const float2 g_xNoisePower;


const float2 g_xOneOverScreenSize;

const texture g_xDepthTexture;
const texture g_xNoiseTexture;


const sampler g_xDepthTextureSampler  = sampler_state
{
	Texture   = <g_xDepthTexture>;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
	MinFilter = POINT;
	MagFilter = POINT;
	MIPFILTER = POINT;
};

const sampler g_xNoiseTextureSampler  = sampler_state
{
	Texture   = <g_xNoiseTexture>;
	AddressU  = WRAP;
	AddressV  = WRAP;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MIPFILTER = LINEAR;
};


/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 m_xPosition     : POSITION;
	float4 m_xUV		: TEXCOORD;
	float4 m_xColour	   : COLOR;
};



struct OutputVertex
{
	float4 m_xPosition     : POSITION;
	float4 m_xWorldPos		: TEXCOORD;
	float4 m_xUV		: TEXCOORD1;
	float4 m_xColour	   : COLOR;
};


OutputVertex VShader( const InputVertex xIn )
{
	OutputVertex xOut;
	xOut.m_xWorldPos = xIn.m_xPosition;
	xOut.m_xPosition = mul(xOut.m_xWorldPos, g_xViewProjectionMatrix);
	xOut.m_xWorldPos.w = xOut.m_xPosition.z;
	xOut.m_xUV = xIn.m_xUV;

	xOut.m_xColour = xIn.m_xColour;
	
	return xOut;
}


float NoiseSample(float fVal)
{
//	return fVal ;
	return fVal * 2.0f - 1.0f;
//	return step(fVal, 0.95f) + fVal * 2.0f - 1.0f;
}


float NoiseLookup(float3 xPos,const float4 xUV)
{
//	xPos += float3(sin(g_fTime * 0.1) * 1.5f, frac(g_fTime * 0.5f), cos(g_fTime * 0.1f) * 1.5f) + xUV.zzz;
	xPos += float3(sin((xUV.w + xUV.z) * 0.1f) * 1.5f, frac(xUV.w * 0.5f), cos((xUV.w + xUV.z) * 0.1f) * 1.5f);

	float fNoise;
	fNoise = NoiseSample(tex2D(g_xNoiseTextureSampler, xPos.xy).r);
	fNoise += NoiseSample(tex2D(g_xNoiseTextureSampler, xPos.yz).g);
  	fNoise += NoiseSample(tex2D(g_xNoiseTextureSampler, xPos.zx).b);
//	fNoise += NoiseSample(tex2D(g_xNoiseTextureSampler, xUV.xy).b);

	// (AAM - 24.03.2009) The clamp was a saturate, but that breaks the March 2009 HLSL compiler.
	// Change it back once we're beyond that release
	fNoise = 1.0f - pow(clamp(1.0f - abs(fNoise * g_xNoisePower.x), 0.00001f, 1.0f), g_xNoisePower.y);
//	fNoise = pow(abs(fNoise * g_xNoisePower.x), g_xNoisePower.y);

	return fNoise;
}


float4 PShader( OutputVertex xIn, const float2 xScreenPos : VPOS) : COLOR
{
	float3 p = xIn.m_xWorldPos.xyz;
	float3 xEyeVec = p - g_xCameraPosition;
	float fDistance = length(xEyeVec);
	xEyeVec /= fDistance;
	
 	float fIntens = saturate(length(xIn.m_xUV.xy));
 	
 	float fRange = min(fIntens, 0.8f);
 	float fNoise = saturate((NoiseLookup(p, xIn.m_xUV) - fRange) / (1.0f - fRange));
// 	float fNoise = saturate(NoiseLookup(p, xIn.m_xUV * 0.5f));
 
 	fIntens = 1.0f - saturate((fIntens - 0.8f) / 0.2f);

	fDistance =  pow(min(fDistance * 2, 1.0f), 3.0f);

	{
		// Do the soft particle stuff
		const float2 xNormalizedCoords = (xScreenPos + float2(0.5, 0.5)) * g_xOneOverScreenSize;
		float fDepth = abs(tex2D(g_xDepthTextureSampler, xNormalizedCoords).z);
		
	    fIntens *= min(fDepth - xIn.m_xWorldPos.w,1.0f);
	}


	float4 xColour = float4(xIn.m_xColour.rgb, xIn.m_xColour.a * fDistance * fNoise * fIntens);
	return xColour;
}
	
/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

technique
{
	pass
	{
		VertexShader = compile vs_3_0 VShader();
		PixelShader = compile ps_3_0 PShader();
	}
}
