///////////////////////////////////////////////////////////////////////////////
//
// File: AVP_Platform_CloakEffect.cpp
//
// Created by Andrew Southgate 03/122007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
// Handles rendering of the predators cloak
//
///////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"
#include "Core/Asura_TextureManagement.h"
#include "SpecialFX/AVP_CloakEffect.h"
#include "SpecialFX/AVP_Platform_CloakEffect.h"

// Asura Platform-independent
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Debugging/Asura_Assert.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_SortedRender.h"
#include "Rendering/Asura_Viewport.h"
#include "Interface/Asura_CommandConsole.h"

// Asura PC
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_MaterialResponse.h"
#include "Rendering/Asura_Direct3D_SkinRenderer.h"
#include "Rendering/Asura_Direct3D_SmoothSkinRenderer.h"


#include "CloakEffect.h"

ASURA_DECLARE_DIRECT3D_EFFECT(CloakEffect);


static int s_iNoiseTexture;


static bool CreateCaptureTexture(void* = NULL)
{
	s_iNoiseTexture = Asura_Texture_Management::AddTexture("\\SpecialFX\\AVP_PredCloak.dds");


	return true;
}

static bool DestroyCaptureTexture(void* = NULL)
{

	return true;
}

static void CaptureToCapturetexture()
{

	Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();
}




namespace {
	Asura_Direct3D_Effect*		g_pxCloakEffect	= NULL;



	D3DXHANDLE					g_xMaterialsTextureHandle;
	D3DXHANDLE					g_xMaterialsTexturePropertiesHandle;
	D3DXHANDLE					g_xWorldMatrixHandle;
	D3DXHANDLE					g_xWorldViewMatrixHandle;
	D3DXHANDLE					g_xWorldViewProjectionMatrixHandle;
	D3DXHANDLE					g_xObjectSpaceCameraPositionHandle;
	D3DXHANDLE					g_xSceneTextureHandle;
	D3DXHANDLE					g_xNoiseTextureHandle;
	D3DXHANDLE					g_xOneOverScreenDimensionsHandle;
	D3DXHANDLE					g_xDistortionScalesHandle;
	D3DXHANDLE					g_xParamsHandle;
	D3DXHANDLE					g_xOffsetHandle;
	D3DXHANDLE					g_xLightningColourHandle;
	D3DXHANDLE					g_xBoneMatricesHandle;
	D3DXHANDLE					g_xUseDamageMapsHandle;
	D3DXHANDLE					g_xCloakFadeOutHandle;
};


void AVP_Platform_CloakEffect::Initialise()
{
	Asura_Direct3D::AddPreResetCallback( DestroyCaptureTexture, NULL);
	Asura_Direct3D::AddPostResetCallback(CreateCaptureTexture, NULL);

	CaptureEffect();
	CreateCaptureTexture();

}


void AVP_Platform_CloakEffect::OnEffectLoaded(class Asura_Direct3D_Effect* pxEffect)
{
	AVP_Platform_CloakEffect::CaptureEffect();
}
void AVP_Platform_CloakEffect::OnEffectUnloaded(class Asura_Direct3D_Effect* pxEffect)
{
	g_pxCloakEffect = NULL;
}
void AVP_Platform_CloakEffect::CaptureEffect(void)
{
#define LOADPARAMETER(codeName, shaderName) { codeName = g_pxCloakEffect->GetD3DEffect()->GetParameterByName(NULL, shaderName); Asura_Assert(codeName, "Could not get shader parameter '" shaderName "'."); }


	g_pxCloakEffect = Asura_Direct3D_Effects_System::GetEffect( "CloakEffect");

	Asura_Assert( g_pxCloakEffect, "Failed to load predator cloak effect." );

	if (g_pxCloakEffect)
	{	
#if ASURA_LOAD_EFFECT_FILES == 1
		g_pxCloakEffect->SetOnLoadCallback(AVP_Platform_CloakEffect::OnEffectLoaded);
		g_pxCloakEffect->SetOnUnloadCallback(AVP_Platform_CloakEffect::OnEffectUnloaded);
#endif

		LOADPARAMETER(g_xMaterialsTextureHandle, "g_xMaterialsTexture");
		LOADPARAMETER(g_xMaterialsTexturePropertiesHandle, "g_xMaterialsTextureProperties");
		LOADPARAMETER(g_xWorldMatrixHandle, "g_xWorldMatrix");
		LOADPARAMETER(g_xWorldViewMatrixHandle, "g_xWorldViewMatrix");
		LOADPARAMETER(g_xWorldViewProjectionMatrixHandle, "g_xWorldViewProjectionMatrix");
		LOADPARAMETER(g_xObjectSpaceCameraPositionHandle, "g_xObjectSpaceCameraPosition");
		LOADPARAMETER(g_xNoiseTextureHandle, "g_xNoiseTexture");
		LOADPARAMETER(g_xSceneTextureHandle, "g_xSceneTexture");
		LOADPARAMETER(g_xOneOverScreenDimensionsHandle, "g_xOneOverScreenDimensions");
		LOADPARAMETER(g_xDistortionScalesHandle, "g_xDistortionScales");
		LOADPARAMETER(g_xParamsHandle, "g_fParams");
		LOADPARAMETER(g_xOffsetHandle, "g_fOffset");
		LOADPARAMETER(g_xLightningColourHandle, "g_fLightningColour");
		LOADPARAMETER(g_xCloakFadeOutHandle, "g_xCloakFadeOut");

		LOADPARAMETER(g_xBoneMatricesHandle, "g_axBoneMatrices");
		LOADPARAMETER(g_xUseDamageMapsHandle, "g_bUseDamageMaps");

	}
}

void AVP_Platform_CloakEffect::Shutdown()
{
	Asura_Direct3D::RemovePostResetCallback(CreateCaptureTexture, NULL);
	Asura_Direct3D::RemovePreResetCallback(DestroyCaptureTexture, NULL);

	DestroyCaptureTexture();
}
void AVP_Platform_CloakEffect::Reset()
{
}

void AVP_Platform_CloakEffect::RenderSkin( const AVP_CloakEffect *pxCloakEffect, const Asura_Animation& xAnimation,
										const Asura_Hierarchy_Skin& xSkin,
										Asura_AMatrix4x4_InParam xTransform,
										Asura_AMatrix4x4_InParam xInverseTransform,
										const Asura_AMatrix4x4* const pxBoneTransforms)
{
	SCOPEDPIXEVENT(( 0xFF884499, L"Smooth skin '%S' (material response system)", xSkin.Name.GetString() ));

	const Asura_Direct3D_Skin* pxNewSkin = NULL;
	Asura_Direct3D_SmoothSkin* pxOldSkin = NULL;
	
	if( Asura_Direct3D_SkinRenderer::IsEnabled() )
	{
		pxNewSkin = Asura_Direct3D_SkinRenderer::GetSkin( xSkin.m_uNameHashID );
		if( !pxNewSkin ) return;
	}
	else
	{
		pxOldSkin = Asura_Direct3D_SmoothSkinRenderer::GetSmoothSkin( xSkin.m_uNameHashID );
		if( !pxOldSkin ) return;
	}

	if( !xSkin.NumberOfBones ) return;

	// Change technique based on transparency. Opaque strips use the deferred renderer,
	// transparent strips the forward renderer
	const bool bTransparentPass = Asura_SortedRender::IsTransparentPass();


	const bool bFullyCloaked = pxCloakEffect->GetMaxBoneValue() - 1.0f >= (pxCloakEffect->GetLightningScale() - 0.00001f);

	Asura_Direct3D_Effect* const pxEffect = Asura_Direct3D_Effects_System::BeginEffect(
		*g_pxCloakEffect, bTransparentPass ? bFullyCloaked + 1 : 0 );


#ifdef ASURA_USE_DAMAGE_MAPS	
	bool bUseDamageMaps = xAnimation.IsUsingDamageMap();
#endif

	if( !pxEffect || !pxEffect->IsD3DEffectValid() ) return;

	// Bind vertex data
	{
		SCOPEDPIXEVENT(( 0xFF884499, L"Bind vertex data" ));

		if( pxOldSkin )
		{
			Asura_Direct3D::D3DDevice->SetVertexDeclaration(
				Asura_Direct3D_SmoothSkinRenderer::GetVertexDeclaration() );

			pxOldSkin->BindBuffers();
		}
		else
		{
			Asura_Direct3D::D3DDevice->SetVertexDeclaration(
				 Asura_Direct3D_SkinRenderer::GetVertexDeclaration() );

			Asura_Direct3D::D3DDevice->SetStreamSource( 0, pxNewSkin->GetVB(), 0,
				Asura_Direct3D_SkinRenderer::uVERTEX_SIZE );

			Asura_Direct3D::D3DDevice->SetIndices( pxNewSkin->GetIB() );
		}
	}


	// Set main materials data texture
	{
		SCOPEDPIXEVENT(( 0xFF884499, L"Set materials data texture" ));

		pxEffect->SetParameter( g_xMaterialsTextureHandle, Asura_Direct3D_MaterialResponse::GetMaterialsTexture());

		if( g_xMaterialsTexturePropertiesHandle )
		{
			const D3DXVECTOR4 xMaterialProperties( 1.0f / ASURA_MATERIAL_RESPONSE_TEXTURE_WIDTH,
				Asura_Direct3D_MaterialResponse::GetMaterialsTextureScale(), 0.0f, 0.0f );

			pxEffect->SetParameter( g_xMaterialsTexturePropertiesHandle, xMaterialProperties );
		}
	}

	// Set world and world-view-proj matrices
	{
		SCOPEDPIXEVENT(( 0xFF884499, L"Set world-view-projection matrix" ));

		D3DXMATRIX xWorldViewProj;

		xTransform.StoreColsUnaligned(&xWorldViewProj._11);

		pxEffect->SetParameter( g_xWorldMatrixHandle, xWorldViewProj );

		xWorldViewProj *= Asura_Direct3D::GetViewMatrix();

		pxEffect->SetParameter( g_xWorldViewMatrixHandle, xWorldViewProj );

		// View-projection
		xWorldViewProj *= Asura_Direct3D::GetProjectionMatrix();

		pxEffect->SetParameter( g_xWorldViewProjectionMatrixHandle, xWorldViewProj );
	}

	// Transform camera position into object space, since this shader does lighting that way
	{
		SCOPEDPIXEVENT(( 0xFF884499, L"Set object-space camera position" ));

		Asura_AVector4	xObjectSpaceCamPos(*Asura_Camera::GetPosition(), 1.0f);

		xObjectSpaceCamPos = xInverseTransform.Apply(xObjectSpaceCamPos);
		pxEffect->SetParameter( g_xObjectSpaceCameraPositionHandle, &xObjectSpaceCamPos, sizeof( xObjectSpaceCamPos ) );
	}

	const u_int uNumBoneMaps =
		(pxOldSkin ? pxOldSkin->GetNumBoneMaps() : pxNewSkin->GetNumBoneMaps());

	// Set bone matrices
	if (uNumBoneMaps == 0)
	{
		SCOPEDPIXEVENT(( 0xFF884499, L"Set bone matrices" ));

		D3DXMATRIX axBoneMatrices[ ASURA_MAX_NO_OF_BONES_PER_MATERIAL ];
		Asura_Mem::Set( axBoneMatrices, 0, sizeof( axBoneMatrices ) );

		for( int i = 0; i < xSkin.NumberOfBones; ++i )
		{
			pxBoneTransforms[i].StoreColsUnaligned(&axBoneMatrices[i]._11);

#ifdef ASURA_USE_DAMAGE_MAPS	
			// Damage maps assume the matrices are orthogonal, and we can reconstruct
			// 3 elements with a cross-product in the shader. As we're only saving 3
			// elements like this, not a whole vector, we can't split the damage values
			// from the matrix palette. So we poke them into the matrices here...
			if (bUseDamageMaps)
			{
				axBoneMatrices[i]._33 = xAnimation.GetBoneDamageMapValue(i);
			}
#endif
			axBoneMatrices[i]._32 = pxCloakEffect->GetBoneValue(i);
		}

		pxEffect->GetD3DEffect()->SetMatrixArray( g_xBoneMatricesHandle, axBoneMatrices, ASURA_MAX_NO_OF_BONES_PER_MATERIAL );

		pxEffect->SetParameter( g_xUseDamageMapsHandle, bUseDamageMaps );
	}

	// Set render states
	{
		SCOPEDPIXEVENT(( 0xFF884499, L"Set render states" ));

		Asura_Render::CurrentStates.RequestAlphaTest( false );
		Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CW );
		Asura_Render::CurrentStates.RequestDitheringMode( true );
		Asura_Render::CurrentStates.RequestDynamicLights( false );
		Asura_Render::CurrentStates.RequestZBufferCompareMode( ASURA_ZBUFFERCMP_LESSEQUAL );
		Asura_Render::CurrentStates.RequestZBufferEnabled( true );
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled( true );

		// Leave wireframe as it was set elsewhere
	}

	g_pxCloakEffect->SetParameterTexture(g_xNoiseTextureHandle, Asura_Direct3D::GetDirect3DTextureByIndex(s_iNoiseTexture));



	if (!bTransparentPass)
	{					
		Asura_Direct3D_MaterialResponse::SetFogParameters( false, NULL, *pxEffect );
		Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF);
	} else {
		// Set scene texture
		{
			SCOPEDPIXEVENT(( 0xFF884499, L"Set scene texture" ));

			g_pxCloakEffect->SetParameterTexture( g_xSceneTextureHandle, Asura_Direct3D_BackBufferCapture::GetCaptureTexture());

			const float fViewportWidth = static_cast< float >( Asura_Viewport::GetWidth() );
			const float fViewportHeight = static_cast< float >( Asura_Viewport::GetHeight() );

			const D3DXVECTOR4 xOneOverDimensions( 1.0f / fViewportWidth, 1.0f / fViewportHeight, 0.0f, 0.0f );
			g_pxCloakEffect->SetParameter( g_xOneOverScreenDimensionsHandle, xOneOverDimensions );

			const float fDistortionAmount = 128.0f * fViewportWidth / 1280.0f;
			const D3DXVECTOR4 xDistortion( fDistortionAmount, -fDistortionAmount, 0.0f, 0.0f );
			g_pxCloakEffect->SetParameter( g_xDistortionScalesHandle, xDistortion );
		}
		Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL);
	}

	u_int uStartIndex = 0;

	if (Asura_MaterialResponse_System::GetMaterialProjectContextCallback() && bTransparentPass)
	{
		Asura_MaterialResponse_System::GetMaterialProjectContextCallback() (ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID, ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID);
	}

	const u_int uNumStrips =
		(pxOldSkin ? pxOldSkin->GetNumStrips() : pxNewSkin->GetNumStrips());

	for( u_int u = 0; u < uNumStrips; ++u )
	{
		const Asura_Direct3D_SmoothSkin::Strip* pxOldStrip =
			(pxOldSkin ? &pxOldSkin->GetStrip(u) : NULL);

		const Asura_Direct3D_SkinStrip* pxNewStrip =
			(pxNewSkin ? &pxNewSkin->GetStrip(u) : NULL);

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		const int iSkinElement = (pxOldStrip ? pxOldStrip->iElementID : pxNewStrip->m_iElementID);
		bool bVisible = xAnimation.GetSkinElementVisibility(iSkinElement);

		if (bVisible)
#endif
		{
			const Asura_Hash_ID uMaterialHash1 = pxOldStrip
				? pxOldStrip->m_auMaterialResponseHashID[0]
				: pxNewStrip->m_auMaterials[0];

			const Asura_Hash_ID uMaterialHash2 = pxOldStrip
				? pxOldStrip->m_auMaterialResponseHashID[1]
				: pxNewStrip->m_auMaterials[1];

			// Set bone matrices
			if (uNumBoneMaps > 0)
			{
				SCOPEDPIXEVENT(( 0xFF884499, L"Set bone matrices" ));

				D3DXMATRIX axBoneMatrices[ ASURA_MAX_NO_OF_BONES_PER_MATERIAL ];
				Asura_Mem::Set( axBoneMatrices, 0, sizeof( axBoneMatrices ) );

				const Asura_Direct3D_SmoothSkin::BoneMap* pxOldBoneMap =
					(pxOldSkin ? &pxOldSkin->GetBoneMap( pxOldStrip->uBoneMap ) : NULL);

				const Asura_Direct3D_Skin::BoneMap* pxNewBoneMap =
					(pxNewSkin ? &pxNewSkin->GetBoneMap( pxNewStrip->m_uBoneMap ) : NULL);

				const u_int uBoneMapSize =
					(pxOldBoneMap ? pxOldBoneMap->m_uSize : pxNewBoneMap->m_uSize);

				for( u_int i = 0; i < uBoneMapSize; ++i )
				{
					u_int uBone = pxOldBoneMap
						? pxOldBoneMap->m_puEntries[i]
						: pxNewBoneMap->m_puEntries[i];

					pxBoneTransforms[uBone].StoreColsUnaligned(&axBoneMatrices[i]._11);

#ifdef ASURA_USE_DAMAGE_MAPS	
					// Damage maps assume the matrices are orthogonal, and we can reconstruct
					// 3 elements with a cross-product in the shader. As we're only saving 3
					// elements like this, not a whole vector, we can't split the damage values
					// from the matrix palette. So we poke them into the matrices here...
					if (bUseDamageMaps)
					{
						axBoneMatrices[i]._33 = xAnimation.GetBoneDamageMapValue(uBone);
					}
#endif
					axBoneMatrices[i]._32 = pxCloakEffect->GetBoneValue(uBone);
				}

				g_pxCloakEffect->GetD3DEffect()->SetMatrixArray( "g_axBoneMatrices", axBoneMatrices, uBoneMapSize );

				g_pxCloakEffect->GetD3DEffect()->SetBool( "g_bUseDamageMaps", bUseDamageMaps );
			}



			SCOPEDPIXEVENT(( 0xFF884499, L"Strip %u", u ));

			bool bVisible = true;

			const Asura_MaterialResponse* pxMaterial1
				= Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

			if( pxMaterial1 )
			{
				if( pxMaterial1->GetDontRender() )
				{
					bVisible = false;
				}
			}

			if (Asura_MaterialResponse_System::GetMaterialProjectContextCallback() && !bTransparentPass)
			{
				Asura_MaterialResponse_System::GetMaterialProjectContextCallback() (pxMaterial1->GetProjectContext(), ASURA_MATERIAL_RESPONSE_PROJECT_CONTEXT_INVALID);
			}

			if( bVisible )
			{
				// Set material data
				{
					SCOPEDPIXEVENT(( 0xFF884499, L"Set material data" ));

					const Asura_MaterialResponse* pxMaterial2
						= Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash2 );

					Asura_Direct3D_MaterialResponse::SetEffectParameters( pxMaterial1, pxMaterial2,
						&xAnimation, *pxEffect );

					Asura_Direct3D_MaterialResponse::SetSHParameters( pxMaterial1, *pxEffect );

					if (bTransparentPass)
					{					
						Asura_Direct3D_MaterialResponse::SetFogParameters( true, pxMaterial1, *pxEffect );
						Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_NORMAL );
					}

//					Asura_Direct3D_MaterialResponse::SetBlendStates( pxMaterial1 );
				}

				// Tri lists only used in Tools
				D3DPRIMITIVETYPE ePrimType;
				if( pxOldSkin )
				{
					ePrimType = pxOldSkin->GetPrimitiveType();
				}
				else
				{
					ePrimType =
						(pxNewSkin->IsUsingTriLists() ? D3DPT_TRIANGLELIST : D3DPT_TRIANGLESTRIP);
				}

				const u_int uNumIndices =
					(pxOldStrip ? pxOldStrip->uNumIndices : pxNewStrip->m_uNumIndices);

				const u_int uPrimitiveCount = ePrimType == D3DPT_TRIANGLESTRIP
					? uNumIndices - 2
					: uNumIndices / 3;

				if (bTransparentPass)
				{
					// Make sure we only write to the main render target
					Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, 0 );
					Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE2, 0 );
					Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE3, 0 );
				}

				const u_int uNumVertices =
					(pxOldSkin ? pxOldSkin->GetNumVertices() : pxNewSkin->GetNumVertices());

 				pxEffect->RenderAllIndexedPrimitivePasses( ePrimType, 0,
 					uNumVertices, uStartIndex, uPrimitiveCount, NULL );

				Asura_Render::AddToTrianglesRendered( uPrimitiveCount );
			}
		}

		uStartIndex += (pxOldStrip ? pxOldStrip->uNumIndices : pxNewStrip->m_uNumIndices);
	}

}
static bool bPass[4] = {true, true, true, true};


void AVP_CloakEffect::Platform_BeginRender(void) const
{
}

void AVP_CloakEffect::Platform_Initialise(void)
{
	AVP_Platform_CloakEffect::Initialise();
}

void AVP_CloakEffect::Platform_Shutdown(void)
{
	AVP_Platform_CloakEffect::Shutdown();
}

static D3DXVECTOR4 s_afVariations[AVP_CloakEffect::NUM_CLOAK_EFFECT_PERMUTATIONS] =
{
	D3DXVECTOR4( 0.0f, 0.0f, 0.0f, 0.0f),
	D3DXVECTOR4( 0.5f, 0.0f, 0.0f, 0.0f),
	D3DXVECTOR4( 0.0f, 0.5f, 0.0f, 0.0f),
	D3DXVECTOR4( 0.5f, 0.5f, 0.0f, 0.0f),
	D3DXVECTOR4( 0.0f, 0.0f, 0.5f, 0.0f),
	D3DXVECTOR4( 0.5f, 0.0f, 0.5f, 0.0f),
	D3DXVECTOR4( 0.0f, 0.5f, 0.5f, 0.0f),
	D3DXVECTOR4( 0.5f, 0.5f, 0.5f, 0.0f),
};

void AVP_CloakEffect::Platform_RenderSkin(const Asura_Animation& xAnimation,
										  const Asura_Hierarchy_Skin& xSkin) const
{
	if (Asura_SortedRender::IsTransparentPass())
	{
		CaptureToCapturetexture();
	}

	float fTime = Asura_Timers::GetLocalScaledTime() * m_fVariationSpeed;
	D3DXVECTOR4 xOffset = D3DXVECTOR4(Asura_Maths::Sine(fTime), Asura_Maths::Cosine(fTime * 1.53f), Asura_Maths::Cosine(fTime * 1.17f), 1.0f) * m_fVariationScale;
	
	g_pxCloakEffect->SetParameter(g_xOffsetHandle, s_afVariations[m_uSeed] + xOffset);
	g_pxCloakEffect->SetParameter(g_xParamsHandle, D3DXVECTOR4(m_fEffectScale, 1.0f / m_fLightningScale, 1.0f, fTime));
	g_pxCloakEffect->SetParameter(g_xLightningColourHandle, D3DXVECTOR4(m_xLightningColour.m_fR, m_xLightningColour.m_fG, m_xLightningColour.m_fB, m_xLightningColour.m_fA ));
	float fCosAng = Asura_Maths::Cosine(m_fMaxVisibilityAngle);
	g_pxCloakEffect->SetParameter(g_xCloakFadeOutHandle, D3DXVECTOR4(1.0f / m_fMaxVisibilityDistance, fCosAng, fCosAng < 1.0f ? 1.0f / (1.0f - fCosAng) : 0.0f, 0.0f ));

	
	AVP_Platform_CloakEffect::RenderSkin(this, xAnimation, xSkin, Asura_Hierarchy_System::s_xBaseTransform, Asura_Hierarchy_System::s_xInverseBaseTransform, Asura_Hierarchy_System::s_axBoneTransforms);
}

void AVP_CloakEffect::Platform_EndRender(void) const
{
	SCOPEDPIXEVENT(( 0xFF884499, L"Clean up" ));

	g_pxCloakEffect->SetParameterTexture(g_xSceneTextureHandle, NULL );
	g_pxCloakEffect->SetParameterTexture(g_xNoiseTextureHandle, NULL);

	Asura_Render::CurrentStates.RequestZBufferWriteEnabled( true );
	Asura_Direct3D_Effects_System::FlushEffects();

	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );
	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE2, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );
	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE3, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );

	// This is enabled in the effect, so undo that now
	for( u_int u = 0; u < 16; ++u )
	{
		Asura_Direct3D::D3DDevice->SetSamplerState( u, D3DSAMP_SRGBTEXTURE, FALSE );
	}

	const float fDepthBias = 0.0f;
	Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_DEPTHBIAS, *(DWORD*)&fDepthBias );

	Asura_Direct3D_BackBufferCapture::AdviseNextFrame();
}


/// End of file. ///
