/**
*
* AVP_Platform_PheromoneTrail.cpp
*
* Created by Andrew Southgate - 30/7/2008
*
* Copyright (C) 2007 Rebellion, All Rights Reserved.
*
**/

/************/
/* Includes */
/************/

// main asura header
#include "Core/Asura.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_TextureManagement.h"

// local header
#include "SpecialFX/AVP_GlowOutlines.h"

// Asura Platform-independent
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Maths.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Colour.h"
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_MaterialResponse_system.h"
#include "Interface/Asura_CommandConsole.h"

// Asura PC
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_EffectParameterBlock.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_UserOptions.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_RenderTargetManager.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"
#include "Rendering/Asura_Direct3D_SkinRenderer.h"
#include "Rendering/Asura_Direct3D_SmoothSkinRenderer.h"


// AVP PC
#include "Rendering/AVP_Platform_Stencil_Setup.h"



#include "AVP_GlowOutlines.h"
ASURA_DECLARE_DIRECT3D_EFFECT(AVP_GlowOutlines);


static Direct3D_VertexDeclaration*	g_pxVertexDeclFS			= 0;



namespace GlowEffect {
	Asura_Direct3D_Effect*		g_pxEffect	= NULL;

	D3DXHANDLE					g_xWorldViewProjectionMatrixHandle;
	D3DXHANDLE					g_xBoneMatricesHandle;
	D3DXHANDLE					g_xBlurColour;
	D3DXHANDLE					g_xBlurParams;
	D3DXHANDLE					g_xSceneTextureHandle;
	D3DXHANDLE					g_xDepthTextureHandle;
	D3DXHANDLE					g_xOneOverScreenSizeHandle;

	void CaptureEffect(void);


	void Initialise()
	{
		CaptureEffect();
	}

#if ASURA_LOAD_EFFECT_FILES == 1
	void OnEffectLoaded(class Asura_Direct3D_Effect* pxEffect)
	{
		CaptureEffect();
	}
	void OnEffectUnloaded(class Asura_Direct3D_Effect* pxEffect)
	{
		g_pxEffect = NULL;
	}
#endif
	void CaptureEffect(void)
	{
#define LOADPARAMETER(codeName, shaderName) { codeName = g_pxEffect->GetD3DEffect()->GetParameterByName(NULL, shaderName); Asura_Assert(codeName, "Could not get shader parameter '" shaderName "'."); }

		g_pxEffect = Asura_Direct3D_Effects_System::GetEffect( "AVP_GlowOutlines");

		Asura_Assert( g_pxEffect, "Failed to load AVP_GlowOutlines effect." );
		if (g_pxEffect)
		{	
#if ASURA_LOAD_EFFECT_FILES == 1
			g_pxEffect->SetOnLoadCallback(OnEffectLoaded);
			g_pxEffect->SetOnUnloadCallback(OnEffectUnloaded);
#endif
			LOADPARAMETER(g_xWorldViewProjectionMatrixHandle, "g_xWorldViewProjectionMatrix");
			LOADPARAMETER(g_xBoneMatricesHandle, "g_axBoneMatrices");
			LOADPARAMETER(g_xBlurColour, "g_xBlurColour");
			LOADPARAMETER(g_xBlurParams, "g_xBlurParams");
			LOADPARAMETER(g_xDepthTextureHandle, "g_xDepthTexture");
			LOADPARAMETER(g_xSceneTextureHandle, "g_xSceneTexture");
			LOADPARAMETER(g_xOneOverScreenSizeHandle, "g_xOneOverScreenSize");

		}
	}
};


struct AVP_FS_Vertex
{
	float x, y, z, w;
	float x2, y2;
	void SetPosition(const Asura_Vector_3 &xPosition)
	{
		x = xPosition.GetX();
		y = xPosition.GetY();
		z = xPosition.GetZ();
		w = 1.0f;
	}
	void SetPos2(float fx, float fy)
	{
		x2 = fx, y2 = fy;
	}
	void Set(const Asura_Vector_3 &xPosition, float fx, float fy)
	{
		SetPosition(xPosition);
		SetPos2(fx,fy);
	}
};


static Direct3D_Texture *g_pxGlowRenderTarget[3];
static Direct3D_Surface *g_pxGlowRenderTargetSurface[3];

static bool PreReset(void*)
{
	for (int i = 0; i < sizeof(g_pxGlowRenderTargetSurface)/sizeof(g_pxGlowRenderTargetSurface[0]); ++i)
	{
		SAFE_RELEASE(g_pxGlowRenderTargetSurface[i]);
		SAFE_RELEASE(g_pxGlowRenderTarget[i]);
	}
	return true;
}

static bool PostReset(void*)
{

#if defined ASURA_ASSERT
	HRESULT xResult;
#endif

	for (int i = 0; i < sizeof(g_pxGlowRenderTargetSurface)/sizeof(g_pxGlowRenderTargetSurface[0]); ++i)
	{
#if defined ASURA_ASSERT
		xResult =
#endif
		Asura_Direct3D::D3DDevice->CreateTexture(Asura_Viewport::GetWidth() /2, Asura_Viewport::GetHeight() / 2, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A16B16G16R16F, D3DPOOL_DEFAULT, &g_pxGlowRenderTarget[i], NULL);
		Asura_Assertf(g_pxGlowRenderTarget[i], ("Failed to create pheromone glow render texture. Error code 0x%X.", xResult));

#if defined ASURA_ASSERT
		xResult =
#endif
		g_pxGlowRenderTarget[i]->GetSurfaceLevel(0, &g_pxGlowRenderTargetSurface[i]);
		Asura_Assertf(g_pxGlowRenderTargetSurface[i], ("Failed to create pheromone glow render surface. Error code 0x%X.", xResult));
	}
	return true;
}

void AVP_GlowOutlines::Platform_Initialise()
{
	if( NULL == Asura_Direct3D::D3DDevice)
	{
		return;
	}

	Direct3D_VertexElement auVSDFS[] =
	{
		{	0,	0,	D3DDECLTYPE_FLOAT4,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },		
		D3DDECL_END()
	};

	Asura_Direct3D::D3DDevice->CreateVertexDeclaration( auVSDFS, &g_pxVertexDeclFS );

	GlowEffect::Initialise();

	Asura_Direct3D::AddPreResetCallback(PreReset, NULL);
	Asura_Direct3D::AddPostResetCallback(PostReset, NULL);

	PostReset(NULL);

}

void AVP_GlowOutlines::Platform_Shutdown()
{
	SAFE_RELEASE( g_pxVertexDeclFS);

	PreReset(NULL);

	Asura_Direct3D::RemovePostResetCallback(PostReset, NULL);
	Asura_Direct3D::RemovePreResetCallback(PreReset, NULL);
}





#if !defined(ASURA_HIERARCHY_QUAD)
#error AVP requires ASURA_HIERARCHY_QUAD at the moment
#endif


static void FixUpAsura_Hierarchy_SystemForAnimation( Asura_Animation* pxAnim, const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xOri, u_int uRenderFlags )
{
	Asura_Hierarchy_System::s_pxSkin		= pxAnim->GetSkin(0);
	Asura_Hierarchy_System::s_uSkinIndex	= 0;
	if( !Asura_Hierarchy_System::s_pxSkin ) { return; }

	Asura_Hierarchy_System::s_pxTweenStorage = pxAnim->GetTweenStore();
	if(!Asura_Hierarchy_System::s_pxTweenStorage) return;

#ifdef ASURA_TOOLS
	Asura_Hierarchy_System::s_pxMesh = NULL;
#endif

#ifdef ASURA_HIERARCHY_QUAD
	// RJT TODO: Ultimately this transform should be passed in
	Asura_Hierarchy_System::s_xBaseTransform = Asura_AMatrix4x4(xOri, xPos);
	Asura_Hierarchy_System::s_xBaseTransform.InvertOrthogonal(Asura_Hierarchy_System::s_xInverseBaseTransform);
#endif

#ifdef ASURA_TOOLS
	if( !Asura_Hierarchy_System::s_pxSkin->m_pxSectionBoundingBoxData )
	{
		Asura_Hierarchy_System::s_pxSkin->InitSectionBoundingBoxes();
	}
#else
	Asura_Assert( Asura_Hierarchy_System::s_pxSkin->m_pxSectionBoundingBoxData, "" );
#endif

	Asura_Hierarchy_System::BonesToWorldspaceFromTweenStorage();
	pxAnim->UpdateLighting(uRenderFlags);

	if( Asura_Hierarchy_System::s_bBonesInWorldspace )
	{
		Asura_Hierarchy_System::s_xInverseBaseTransform.ApplyToArray( Asura_Hierarchy_System::s_axBoneTransforms, Asura_Hierarchy_System::s_pxSkin->NumberOfBones, Asura_Hierarchy_System::s_axBoneTransforms);
	}

	for( int i = 0; i < Asura_Hierarchy_System::s_pxSkin->NumberOfBones; ++i )
	{
		Asura_Hierarchy_System::s_axBoneTransforms[i].Apply(Asura_Hierarchy_System::s_pxSkin->m_pxBoneRestPoseInverseTransforms[i], Asura_Hierarchy_System::s_axBoneTransforms[i]);
	}
}

static void RenderSkin( const Asura_Animation& xAnimation, const Asura_Hierarchy_Skin& xSkin, Asura_AMatrix4x4_InParam xTransform,
					   Asura_AMatrix4x4_InParam xInverseTransform, const Asura_AMatrix4x4* const pxBoneTransforms)

{
	SCOPEDPIXEVENT(( 0xFF884499, L"Smooth skin '%S' (AVP Pheromone)", xSkin.Name.GetString() ));

	const Asura_Direct3D_Skin* pxNewSkin = NULL;
	Asura_Direct3D_SmoothSkin* pxOldSkin = NULL;
	
	if( Asura_Direct3D_SkinRenderer::IsEnabled() )
	{
		pxNewSkin = Asura_Direct3D_SkinRenderer::GetSkin( xSkin.m_uNameHashID );
	}
	else
	{
		pxOldSkin = Asura_Direct3D_SmoothSkinRenderer::GetSmoothSkin( xSkin.m_uNameHashID );
	}

	if( (pxOldSkin || pxNewSkin) && xSkin.NumberOfBones) 
	{
		Asura_Direct3D_Effect* const pxEffect = Asura_Direct3D_Effects_System::BeginEffect( *GlowEffect::g_pxEffect, 0 );

		if( pxEffect && pxEffect->IsD3DEffectValid() ) 
		{
			{	// Bind vertex data
				SCOPEDPIXEVENT(( 0xFF884499, L"Bind vertex data" ));

				if( pxOldSkin )
				{
					Asura_Direct3D::D3DDevice->SetVertexDeclaration(
						Asura_Direct3D_SmoothSkinRenderer::GetVertexDeclaration() );

					pxOldSkin->BindBuffers();
				}
				else
				{
					Asura_Direct3D::D3DDevice->SetVertexDeclaration(
						 Asura_Direct3D_SkinRenderer::GetVertexDeclaration() );

					Asura_Direct3D::D3DDevice->SetStreamSource( 0, pxNewSkin->GetVB(), 0,
						Asura_Direct3D_SkinRenderer::uVERTEX_SIZE );

					Asura_Direct3D::D3DDevice->SetIndices( pxNewSkin->GetIB() );
				}
			}

			{	// Set world and world-view-proj matrices
				SCOPEDPIXEVENT(( 0xFF884499, L"Set world-view-projection matrix" ));

				D3DXMATRIX xWorldViewProj;
				xTransform.StoreColsUnaligned(&xWorldViewProj._11);

				// View-projection
				xWorldViewProj *= Asura_Direct3D::GetViewProjectionMatrix();
				pxEffect->SetParameter( GlowEffect::g_xWorldViewProjectionMatrixHandle, xWorldViewProj );
			}

			D3DXMATRIX axBoneMatrices[ ASURA_MAX_NO_OF_BONES_PER_MATERIAL ];

			// Set bone matrices

			const u_int uNumBoneMaps =
				(pxOldSkin ? pxOldSkin->GetNumBoneMaps() : pxNewSkin->GetNumBoneMaps());

			if (uNumBoneMaps == 0)
			{
				SCOPEDPIXEVENT(( 0xFF884499, L"Set bone matrices (No Bone Maps)" ));

				for( int i = 0; i < xSkin.NumberOfBones; ++i )
				{
					pxBoneTransforms[i].StoreColsUnaligned(&axBoneMatrices[i]._11);
				}
				Asura_Mem::Set( axBoneMatrices + xSkin.NumberOfBones, 0, (ASURA_MAX_NO_OF_BONES_PER_MATERIAL - xSkin.NumberOfBones) * sizeof(D3DXMATRIX));

				pxEffect->GetD3DEffect()->SetMatrixArray( GlowEffect::g_xBoneMatricesHandle, axBoneMatrices, ASURA_MAX_NO_OF_BONES_PER_MATERIAL );
			} else if (uNumBoneMaps == 1)
			{
				SCOPEDPIXEVENT(( 0xFF884499, L"Set bone matrices (One Bone Maps)" ));

				const Asura_Direct3D_SmoothSkin::BoneMap* pxOldBoneMap =
					(pxOldSkin ? &pxOldSkin->GetBoneMap(0) : NULL);

				const Asura_Direct3D_Skin::BoneMap* pxNewBoneMap =
					(pxNewSkin ? &pxNewSkin->GetBoneMap(0) : NULL);

				const u_int uBoneMapSize =
					(pxOldBoneMap ? pxOldBoneMap->m_uSize : pxNewBoneMap->m_uSize);

				for( u_int i = 0; i < uBoneMapSize; ++i )
				{
					u_int uBone = pxOldBoneMap
						? pxOldBoneMap->m_puEntries[i]
						: pxNewBoneMap->m_puEntries[i];

					pxBoneTransforms[uBone].StoreColsUnaligned(&axBoneMatrices[i]._11);
				}
				Asura_Mem::Set( axBoneMatrices + uBoneMapSize, 0, (ASURA_MAX_NO_OF_BONES_PER_MATERIAL - uBoneMapSize) * sizeof(D3DXMATRIX));

				pxEffect->GetD3DEffect()->SetMatrixArray( GlowEffect::g_xBoneMatricesHandle, axBoneMatrices, uBoneMapSize );

			}

			const u_int uNumStrips =
				(pxOldSkin ? pxOldSkin->GetNumStrips() : pxNewSkin->GetNumStrips());

			for( u_int u = 0, uStartIndex = 0; u < uNumStrips; ++u )
			{
				const Asura_Direct3D_SmoothSkin::Strip* pxOldStrip =
					(pxOldSkin ? &pxOldSkin->GetStrip(u) : NULL);

				const Asura_Direct3D_SkinStrip* pxNewStrip =
					(pxNewSkin ? &pxNewSkin->GetStrip(u) : NULL);

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
				const int iSkinElement =
					(pxOldStrip ? pxOldStrip->iElementID : pxNewStrip->m_iElementID);

				if (xAnimation.GetSkinElementVisibility(iSkinElement))
#endif
				{
					// Set bone matrices
					if (uNumBoneMaps > 1)
					{
						SCOPEDPIXEVENT(( 0xFF884499, L"Set bone matrices (Many Bone Maps)" ));

						const Asura_Direct3D_SmoothSkin::BoneMap* pxOldBoneMap =
							(pxOldSkin ? &pxOldSkin->GetBoneMap( pxOldStrip->uBoneMap ) : NULL);

						const Asura_Direct3D_Skin::BoneMap* pxNewBoneMap =
							(pxNewSkin ? &pxNewSkin->GetBoneMap( pxNewStrip->m_uBoneMap ) : NULL);

						const u_int uBoneMapSize =
							(pxOldBoneMap ? pxOldBoneMap->m_uSize : pxNewBoneMap->m_uSize);

						for( u_int i = 0; i < uBoneMapSize; ++i )
						{
							u_int uBone = pxOldBoneMap
								? pxOldBoneMap->m_puEntries[i]
								: pxNewBoneMap->m_puEntries[i];

							pxBoneTransforms[uBone].StoreColsUnaligned(&axBoneMatrices[i]._11);
						}
						Asura_Mem::Set( axBoneMatrices + uBoneMapSize, 0, (ASURA_MAX_NO_OF_BONES_PER_MATERIAL - uBoneMapSize) * sizeof(D3DXMATRIX));

						pxEffect->GetD3DEffect()->SetMatrixArray( GlowEffect::g_xBoneMatricesHandle, axBoneMatrices, uBoneMapSize );
					}

					SCOPEDPIXEVENT(( 0xFF884499, L"Strip %u", u ));

					const Asura_Hash_ID uMaterialHash1 = pxOldStrip
						? pxOldStrip->m_auMaterialResponseHashID[0]
						: pxNewStrip->m_auMaterials[0];

					const Asura_MaterialResponse* pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

					if( !pxMaterial1 || !pxMaterial1->GetDontRender() )
					{

						// Tri lists only used in Tools
						D3DPRIMITIVETYPE ePrimType;
						if( pxOldSkin )
						{
							ePrimType = pxOldSkin->GetPrimitiveType();
						}
						else
						{
							ePrimType = pxNewSkin->IsUsingTriLists()
								? D3DPT_TRIANGLELIST
								: D3DPT_TRIANGLESTRIP;
						}

						const u_int uNumIndices =
							(pxOldStrip ? pxOldStrip->uNumIndices : pxNewStrip->m_uNumIndices);

						const u_int uPrimitiveCount = ePrimType == D3DPT_TRIANGLESTRIP
							? uNumIndices - 2
							: uNumIndices / 3;

						const u_int uNumVertices = pxOldSkin
							? pxOldSkin->GetNumVertices()
							: pxNewSkin->GetNumVertices();

						pxEffect->RenderAllIndexedPrimitivePasses( ePrimType, 0,
							uNumVertices, uStartIndex, uPrimitiveCount, NULL );

						Asura_Render::AddToTrianglesRendered( uPrimitiveCount );
					}
				}

				uStartIndex += (pxOldStrip ? pxOldStrip->uNumIndices : pxNewStrip->m_uNumIndices);
			}
		}
	}
}



void AVP_GlowOutlines::Platform_Render(  DepthSortedSkinEntry * const * pxBuffer) 
{
	static const AVP_FS_Vertex axVerts[] =
	{
		{ -1.0f,  1.0f, 0.0f,1.0f, 0.0f, 0.0f, },
		{  1.0f,  1.0f, 0.0f,1.0f, 1.0f, 0.0f },
		{  1.0f, -1.0f, 0.0f,1.0f, 1.0f, 1.0f },
		{ -1.0f, -1.0f, 0.0f,1.0f, 0.0f, 1.0f },
	};

	Direct3D_Surface* pxOldRenderTargetBuffer[5] = 
	{
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
	};
	SCOPEDPIXEVENT(( 0xFF884499, L"AVP GlowOutlines"));

	DepthSortedSkinEntry *pxFirst = NULL, *pxPrev = NULL;
	for (int i = NUM_DEPTH_SORT_SLOTS-1; i >= 0; --i)	
	{
		// fix up the list slightly so the next time we iterate its just a flat linked list, we dont need to go through lots of empty buckets twice...
		if (pxBuffer[i])
		{
			if (pxPrev)
			{
				pxPrev->m_pxNext = pxBuffer[i];
			}
			if (!pxFirst)
			{
				pxFirst = pxBuffer[i];

				for (int j = 0; j < 4; ++j)
				{
					if (FAILED(Asura_Direct3D::D3DDevice->GetRenderTarget(j, &pxOldRenderTargetBuffer[j])))
						pxOldRenderTargetBuffer[j] = NULL;

				}
				if (FAILED(Asura_Direct3D::D3DDevice->GetDepthStencilSurface(&pxOldRenderTargetBuffer[4])))
					pxOldRenderTargetBuffer[4] = NULL;

				Asura_Direct3D::D3DDevice->SetRenderTarget(0, g_pxGlowRenderTargetSurface[0]);
				Asura_Direct3D::D3DDevice->SetRenderTarget(1, g_pxGlowRenderTargetSurface[2]);
				Asura_Direct3D::D3DDevice->SetRenderTarget(2, NULL);
				Asura_Direct3D::D3DDevice->SetRenderTarget(3, NULL);
				Asura_Direct3D::D3DDevice->SetDepthStencilSurface(NULL);
				Asura_Direct3D::D3DDevice->Clear(0, NULL, D3DCLEAR_TARGET, 0, 0.0f, 0);


				// Set render states
				{
					SCOPEDPIXEVENT(( 0xFF884499, L"Set render states" ));

					Asura_Render::CurrentStates.RequestAlphaTest( false );
					Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CW );
					Asura_Render::CurrentStates.RequestDitheringMode( true );
					Asura_Render::CurrentStates.RequestDynamicLights( false );
					Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF);
					Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );
					Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_ALPHA );
				}
#if ASURA_D3D_ENABLE_PIX
				D3DPERF_BeginEvent(0xFF884499, L"Render skins to buffers");
#endif
			}
		}
		for (DepthSortedSkinEntry *pxEntry0 = pxBuffer[i]; pxEntry0; pxEntry0 = pxEntry0->m_pxNext)
		{
			pxPrev = pxEntry0;

			Asura_Animation *pxAnim = pxEntry0->m_xSortedRenderItem.m_pxAnimation;

			FixUpAsura_Hierarchy_SystemForAnimation(pxAnim, pxEntry0->m_xSortedRenderItem.m_xPosition, pxEntry0->m_xSortedRenderItem.m_xOrient, pxEntry0->m_xSortedRenderItem.m_uRenderFlags);

			for( int iSkinIndex = 0; iSkinIndex < ASURA_ANIMATION_SKIN_COUNT; ++iSkinIndex )
			{
				const Asura_Hierarchy_Skin *pxSkin = pxAnim->GetSkin(iSkinIndex);

				if (pxSkin)
				{
					for( int i = 0; i < Asura_Camera::GetNumberOfCameras(); ++i )
					{
						if((pxEntry0->m_xSortedRenderItem.m_uCamVisFlags & (1 << i)) != 0)
						{
							if((Asura_Hierarchy_System::ShouldJustRenderForCurCam() && ((u_int)i == Asura_Camera::GetCameraIndex())) || Asura_Camera::IsCameraActive(i))
							{
								Asura_Camera::SetCameraIndex(i);
								Asura_Render::ApplyCameraSettings();
								{
									D3DVIEWPORT9 xViewport;

									xViewport.Width = Asura_Viewport::GetWidth()/2;
									xViewport.Height = Asura_Viewport::GetHeight()/2;
									xViewport.X = 0;
									xViewport.Y = 0;
									xViewport.MinZ = 0.0f;
									xViewport.MaxZ = 1.0f;
									Asura_Direct3D::D3DDevice->SetViewport( &xViewport);
								}

								// Select LOD mesh based on distance to first bone, since object may be animating far away from its location
								Asura_Hierarchy_Skin const* const	pxSkinLOD 
									= pxSkin->GetDetailSkinFromDistance( (Asura_Hierarchy_System::GetWorldspaceBonePos(0) - *Asura_Camera::GetPosition()).Magnitude() );

								if(pxSkinLOD)
								{
									GlowEffect::g_pxEffect->SetParameter( GlowEffect::g_xBlurColour, D3DXVECTOR4(pxEntry0->m_xColour.m_fR, pxEntry0->m_xColour.m_fG, pxEntry0->m_xColour.m_fB, pxEntry0->m_xColour.m_fA));
									RenderSkin(*pxEntry0->m_xSortedRenderItem.m_pxAnimation, *pxSkinLOD, Asura_Hierarchy_System::s_xBaseTransform, Asura_Hierarchy_System::s_xInverseBaseTransform, Asura_Hierarchy_System::s_axBoneTransforms);
								}
								else
								{
									Asura_Assertf(false, ("Couldn't retrieve LOD skin for '%s' at distance %.02f.", pxSkin->GetName(), (Asura_Hierarchy_System::GetWorldspaceBonePos(0) - *Asura_Camera::GetPosition()).Magnitude()));
								}
							}
						}
					}
				}
			}
		}
	}
	
	if (pxFirst)
	{
#if ASURA_D3D_ENABLE_PIX
		D3DPERF_EndEvent();
#endif
		Asura_Direct3D_Effect* pxEffect = Asura_Direct3D_Effects_System::BeginEffect(*GlowEffect::g_pxEffect, 1);

		// Take the colour buffer we have just stored all the skins in and horizontally blur it
		{
			SCOPEDPIXEVENT(( 0xFF884499, L"Horizontal blur" ));
			Asura_Direct3D::D3DDevice->SetRenderTarget(0, g_pxGlowRenderTargetSurface[1]);
			Asura_Direct3D::D3DDevice->SetRenderTarget(1, NULL);
			Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
			Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
			Asura_Direct3D::D3DDevice->SetRenderState( D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );



			Asura_Direct3D::D3DDevice->SetVertexDeclaration(g_pxVertexDeclFS);	


			pxEffect->SetParameter( GlowEffect::g_xBlurParams, D3DXVECTOR4(GLOW_RADIUS / (2.0f * Asura_Viewport::GetWidth()), GLOW_MAX_DISTANCE, 0.0f, 0.0f));
			pxEffect->SetParameter(GlowEffect::g_xOneOverScreenSizeHandle,  D3DXVECTOR4(2.0f / Asura_Viewport::GetWidth(), 2.0f / Asura_Viewport::GetHeight(), 0, 0));
			pxEffect->SetParameterTexture(GlowEffect::g_xSceneTextureHandle, g_pxGlowRenderTarget[0]);
			pxEffect->SetParameterTexture(GlowEffect::g_xDepthTextureHandle, Asura_Direct3D::GetDeferredRenderingTarget()->GetTexture( ASURA_DIRECT3D_DEFFEDRENDERTARGET_NORMAL_INDEX));

			pxEffect->BeginPass(0);
			Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( AVP_FS_Vertex ) );
			pxEffect->EndPass();

		}




		// Now take the horizontally blended texture and vertically blend it and apply it to the screen
		{
			SCOPEDPIXEVENT(( 0xFF884499, L"Vertical blur" ));

			Asura_Direct3D::D3DDevice->SetRenderTarget(0, g_pxGlowRenderTargetSurface[2]);
			pxEffect->SetParameterTexture(GlowEffect::g_xSceneTextureHandle, g_pxGlowRenderTarget[1]);


			pxEffect->SetParameter( GlowEffect::g_xBlurParams, D3DXVECTOR4(GLOW_RADIUS / (2.0f * Asura_Viewport::GetHeight()), GLOW_MAX_DISTANCE, 0.0f, 0.0f));
			pxEffect->BeginPass(1);
			Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( AVP_FS_Vertex ) );
			pxEffect->EndPass();
		}

		// and apply the blurred surface to the screen
		{
			SCOPEDPIXEVENT(( 0xFF884499, L"Draw To screen" ));

			Asura_Direct3D::D3DDevice->SetRenderTarget(0, pxOldRenderTargetBuffer[0]);
			Asura_Direct3D::D3DDevice->SetRenderTarget(1, pxOldRenderTargetBuffer[1]);
			Asura_Direct3D::D3DDevice->SetRenderTarget(2, pxOldRenderTargetBuffer[2]);
			Asura_Direct3D::D3DDevice->SetRenderTarget(3, pxOldRenderTargetBuffer[3]);
			Asura_Direct3D::D3DDevice->SetDepthStencilSurface(pxOldRenderTargetBuffer[4]);
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

			pxEffect->SetParameterTexture(GlowEffect::g_xSceneTextureHandle, g_pxGlowRenderTarget[2]);

			pxEffect->SetParameter(GlowEffect::g_xOneOverScreenSizeHandle,  D3DXVECTOR4(1.0f / Asura_Viewport::GetWidth(), 1.0f / Asura_Viewport::GetHeight(), 0, 0));

			pxEffect->BeginPass(2);
			Asura_Direct3D::D3DDevice->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( AVP_FS_Vertex ) );
			pxEffect->EndPass();
		}
		Asura_Direct3D_Effects_System::FlushEffects();

		{
			pxEffect->SetParameterTexture(GlowEffect::g_xSceneTextureHandle, NULL);
			pxEffect->SetParameterTexture(GlowEffect::g_xDepthTextureHandle, NULL);

			SCOPEDPIXEVENT(( 0xFF884499, L"Restore renderstates" ));
			for (int i = 0; i < 8; i++)
			{
				Asura_Direct3D::D3DDevice->SetSamplerState(i, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
				Asura_Direct3D::D3DDevice->SetSamplerState(i, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
				Asura_Direct3D::D3DDevice->SetSamplerState(i, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
			}
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
			Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
			Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
			Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
			Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_SRCBLENDALPHA, D3DBLEND_ONE);
			Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_DESTBLENDALPHA, D3DBLEND_ZERO);
			Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
		
			Asura_Direct3D::D3DDevice->SetRenderTarget(0, pxOldRenderTargetBuffer[0]);
			Asura_Direct3D::D3DDevice->SetRenderTarget(1, pxOldRenderTargetBuffer[1]);
			Asura_Direct3D::D3DDevice->SetRenderTarget(2, pxOldRenderTargetBuffer[2]);
			Asura_Direct3D::D3DDevice->SetRenderTarget(3, pxOldRenderTargetBuffer[3]);
			Asura_Direct3D::D3DDevice->SetDepthStencilSurface(pxOldRenderTargetBuffer[4]);
			SAFE_RELEASE(pxOldRenderTargetBuffer[0]);
			SAFE_RELEASE(pxOldRenderTargetBuffer[1]);
			SAFE_RELEASE(pxOldRenderTargetBuffer[2]);
			SAFE_RELEASE(pxOldRenderTargetBuffer[3]);
			SAFE_RELEASE(pxOldRenderTargetBuffer[4]);
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

		}
	}

}


