///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Platform_Actor_InputHandler.cpp
// Author : Callum Hope
// Date   : 13-Oct-2008
// Desc	  : AVP/PS3/Actor/... Platform dependant functionality and look up functions for data used by the input handler. 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// core header
#include "Core/Asura.h"

// This file's header...
#include "Actor/AVP_Actor_InputHandler.h"


// Platform dependant functions
void AVP_Actor_InputHandler::Platform_RefreshTurnAndTiltParams( const Asura_Blueprint*	pxBlueprint )
{
	Asura_Value	xValue;

	if( pxBlueprint )
	{
		if(pxBlueprint->GetParameterValueByHashID(0xB263B6AA, xValue))	//Asura_GetHashID("AnalogeStickPS3_TiltMaxSpeed")
		{
			s_fTiltMaxSpeedInDegree = xValue.GetFloatValue();
			s_fTiltMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTiltMaxSpeedInDegree);
		}
		if(pxBlueprint->GetParameterValueByHashID(0x618A7A01, xValue))	//Asura_GetHashID("AnalogeStickPS3_TiltRampUpTime")
		{
			s_fTiltRampUpTime = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x13621248, xValue))	//Asura_GetHashID("AnalogeStickPS3_TiltRampDownTime")
		{
			s_fTiltRampDownTime = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x262997F6, xValue))	//Asura_GetHashID("AnalogeStickPS3_TiltExponent")
		{
			s_fTiltExponent = xValue.GetFloatValue();
		}

		if(pxBlueprint->GetParameterValueByHashID(0x260FEDBD, xValue))	//Asura_GetHashID("AnalogeStickPS3_TurnDefaultMaxSpeed")
		{
			s_fTurnDefaultMaxSpeedInDegree = xValue.GetFloatValue();
			s_fTurnDefaultMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTurnDefaultMaxSpeedInDegree);
		}
		if(pxBlueprint->GetParameterValueByHashID(0xD36D6ADA, xValue))	//Asura_GetHashID("AnalogeStickPS3_TurnTurboMaxSpeed")
		{
			s_fTurnTurboMaxSpeedInDegree = xValue.GetFloatValue();
			s_fTurnTurboMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTurnTurboMaxSpeedInDegree);
		}
		if(pxBlueprint->GetParameterValueByHashID(0xD649B36C, xValue))	//Asura_GetHashID("AnalogeStickPS3_TurnTurboDelay")
		{
			s_fTurnTurboDelay = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x66F7E231, xValue))	//Asura_GetHashID("AnalogeStickPS3_TurnTurboRampUpTime")
		{
			s_fTurnTurboRampUpTime = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x407C77C1, xValue))	//Asura_GetHashID("AnalogeStickPS3_TurnRampUpTime")
		{
			s_fTurnRampUpTime = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0xFDCBA008, xValue))	//Asura_GetHashID("AnalogeStickPS3_TurnRampDownTime")
		{
			s_fTurnRampDownTime = xValue.GetFloatValue();
		}
		if(pxBlueprint->GetParameterValueByHashID(0x610005B6, xValue))	//Asura_GetHashID("AnalogeStickPS3_TurnExponent")
		{
			s_fTurnExponent = xValue.GetFloatValue();
		}
	}
	else
	{
		s_fTiltMaxSpeedInDegree = 90.0f;
		s_fTiltMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTiltMaxSpeedInDegree);
		s_fTiltRampUpTime = 0.25f;
		s_fTiltRampDownTime = 0.05f;
		s_fTiltExponent = 2.0f;

		s_fTurnDefaultMaxSpeedInDegree = 180.0f;
		s_fTurnDefaultMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTurnDefaultMaxSpeedInDegree);
		s_fTurnTurboMaxSpeedInDegree = 180.0f;
		s_fTurnTurboMaxSpeedInRadian = Asura_Maths::DegToRad(s_fTurnTurboMaxSpeedInDegree);
		s_fTurnTurboDelay = 1.0f;
		s_fTurnTurboRampUpTime = 0.5f;
		s_fTurnRampUpTime = 0.5f;
		s_fTurnRampDownTime = 0.10f;
		s_fTurnExponent = 2.0f;
	}
}

/// End of file. ///
