//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_Edges.cpp
//
// Copyright (c) 2009 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#if defined ASURA_USE_FSFX

// Asura Platform-independent
#include "FSFX/Asura_FSFX_System.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_ViewPort.h"
#include "Rendering/Asura_Overlay_Quad.h"

// Asura PS3
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_Graphics.h"

#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_GCM_VertexBuffer.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"

// AVP Platform-independent
#include "FSFX/AVP_FSFX.h"

#include "Rendering/ShadersDefinition/vp/FSFX_Noise_VP.h"
#include "Rendering/ShadersDefinition/vd/FSFX_Noise_VD.h"
#include "Rendering/ShadersDefinition/fp/FSFX_Edges_Pass1_FP.h"
#include "Rendering/ShadersDefinition/fp/FSFX_Edges_Pass2_FP.h"

static AVP_PS3_FSFX_Edges_Pass1_PS*			g_pxEdgesPass1FP	= NULL;
static AVP_PS3_FSFX_Edges_Pass2_PS*			g_pxEdgesPass2FP	= NULL;
static Asura_PS3_GCM_VertexShader*			g_pxVertexShader	= NULL;
static Asura_PS3_GCM_VertexBuffer			g_xVertexBuffer;

static CellGcmSurface					g_xRenderTarget;
static CellGcmTexture					g_xRenderTargetCellTexture;
static Asura_PS3_Texture				g_xRenderTargetTexture;


#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
static bool ReloadShaders(bool bHelp)
{
	if ( bHelp )
	{
		Asura_CommandConsole::Printf("recompile & load FSFX Edges shader");
		return true;
	}

	g_pxEdgesPass1FP->ReloadFromDisc();
	g_pxEdgesPass2FP->ReloadFromDisc();
	return true;
}
#endif


void AVP_FSFX_Component_Edges::Platform_Initialise(  )
{
	g_pxEdgesPass1FP = new AVP_PS3_FSFX_Edges_Pass1_PS();
	g_pxEdgesPass2FP = new AVP_PS3_FSFX_Edges_Pass2_PS();
	g_xVertexBuffer.Initialise( NOISE_VERTEX_SIZE, 4, CELL_GCM_LOCATION_MAIN );

	const float fWidth = static_cast< float >( Asura_Viewport::GetWidth()/2 );
	const float fHeight = static_cast< float >( Asura_Viewport::GetHeight()/2 );
	const float fVerticalOffset = 0.0f;
	const float fHorizontalOffset = 0.0f;



//  	Asura_CommandConsole::AddVar(L"AVP.FSFX.Edges.Offset.Vertical", &fVerticalOffset, -5.0f, 5.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
//  	Asura_CommandConsole::AddVar(L"AVP.FSFX.Edges.Offset.Horizontal", &fHorizontalOffset, -5.0f, 5.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
//  	Asura_CommandConsole::AddVar(L"AVP.FSFX.Edges.Width", &fWidth, 0.0f, 2000.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
//  	Asura_CommandConsole::AddVar(L"AVP.FSFX.Edges.Height", &fHeight, 0.0f, 2000.0f, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);

	float* pf = ( float* )g_xVertexBuffer.LockForWriting();
	pf[  0 ] = -1; pf[  1 ] =  1; pf[  2 ] =             fHorizontalOffset / fWidth; pf[  3 ] = (fHeight + fVerticalOffset) / fHeight; 
	pf[  4 ] = -1; pf[  5 ] = -1; pf[  6 ] =             fHorizontalOffset / fWidth; pf[  7 ] =             fVerticalOffset / fHeight; 
	pf[  8 ] =  1; pf[  9 ] = -1; pf[ 10 ] =  (fWidth + fHorizontalOffset) / fWidth; pf[ 11 ] =             fVerticalOffset / fHeight; 
	pf[ 12 ] =  1; pf[ 13 ] =  1; pf[ 14 ] =  (fWidth + fHorizontalOffset) / fWidth; pf[ 15 ] = (fHeight + fVerticalOffset) / fHeight; 

	g_xVertexBuffer.Unlock();



	g_pxVertexShader = new Asura_PS3_VP_Noise();

#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
	Asura_CommandConsole::AddCmd(L"Render.ReloadShaders.FSFX.Edges", ReloadShaders, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
#endif

	g_xRenderTarget = *Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->GetSurface();
	g_xRenderTarget.height >>= 1;
	g_xRenderTarget.width >>= 1;
	g_xRenderTarget.colorTarget = CELL_GCM_SURFACE_TARGET_0;

	g_xRenderTargetCellTexture.mipmap		= 1;
	g_xRenderTargetCellTexture.dimension	= CELL_GCM_TEXTURE_DIMENSION_2;
	g_xRenderTargetCellTexture.cubemap		= CELL_GCM_FALSE;
	g_xRenderTargetCellTexture.remap		= CELL_GCM_REMAP_MODE( CELL_GCM_TEXTURE_REMAP_ORDER_XYXY, CELL_GCM_TEXTURE_REMAP_FROM_A,CELL_GCM_TEXTURE_REMAP_FROM_R, CELL_GCM_TEXTURE_REMAP_FROM_G, CELL_GCM_TEXTURE_REMAP_FROM_B, CELL_GCM_TEXTURE_REMAP_REMAP, CELL_GCM_TEXTURE_REMAP_REMAP, CELL_GCM_TEXTURE_REMAP_REMAP, CELL_GCM_TEXTURE_REMAP_REMAP );
	g_xRenderTargetCellTexture.width		= g_xRenderTarget.width;
	g_xRenderTargetCellTexture.height		= g_xRenderTarget.height;
	g_xRenderTargetCellTexture.depth		= 1;
	g_xRenderTargetCellTexture.location		= g_xRenderTarget.colorLocation[0];
	g_xRenderTargetCellTexture._padding		= 0;
	g_xRenderTargetCellTexture.offset		= g_xRenderTarget.colorOffset[0];
	g_xRenderTargetCellTexture.pitch		= g_xRenderTarget.colorPitch[0];
	g_xRenderTargetCellTexture.format		= CELL_GCM_TEXTURE_LN | CELL_GCM_TEXTURE_UN | CELL_GCM_TEXTURE_W16_Z16_Y16_X16_FLOAT;

	g_xRenderTargetTexture =  Asura_PS3_Texture(&g_xRenderTargetCellTexture);
}

void AVP_FSFX_Component_Edges::Platform_Shutdown(  )
{
	delete g_pxEdgesPass1FP;
	g_pxEdgesPass1FP = NULL;
	delete g_pxEdgesPass2FP;
	g_pxEdgesPass2FP = NULL;

	g_xVertexBuffer.Destroy();
}


void AVP_FSFX_Component_Edges::Platform_Render( float fIntensity )
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "FSFX: AVP - Edges" );
	Asura_PS3_DeferredRendering::ResolveBackBuffer();

	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Pass1" );
	Asura_PS3_RenderingStates::SetSurface( &g_xRenderTarget );
	Asura_PS3_RenderingStates::SetViewport( g_xRenderTarget.width, g_xRenderTarget.height );

	g_pxEdgesPass1FP->Begin();
	g_pxEdgesPass1FP->SetTexture( AVP_PS3_FSFX_Edges_Pass1_PS::FP_SceneTextureSampler, Asura_PS3_DeferredRendering::GetResolvedTexture(), true );
	float afParams[4] = {m_fLowThreshold * 0.001f, m_fHighThreshold * 0.001f, m_fEDWidth * 0.1f, 0.0f};
	g_pxEdgesPass1FP->SetFloats( AVP_PS3_FSFX_Edges_Pass1_PS::FP_EdgeDetectionParams, 4, afParams);
	float afParams2[4] = {1.0f / Asura_Viewport::GetWidth(), 1.0f / Asura_Viewport::GetHeight(), 0.0f, 0.0f};
	g_pxEdgesPass1FP->SetFloats( AVP_PS3_FSFX_Edges_Pass1_PS::FP_InvScreenDims, 4, afParams2);

	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_NONE );
	Asura_PS3_RenderingStates::RequestAlphaTest( false );

	// Taken these settings out of the effect, as they were screwing up the cached states
	Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
	Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);

	g_pxVertexShader->Bind();
	g_pxEdgesPass1FP->Bind();
	g_xVertexBuffer.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
	g_xVertexBuffer.RenderQuads( 4 );
	g_pxEdgesPass1FP->End();
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__MISC, "Pass2" );


	Asura_PS3_DeferredRendering::BindAsSingleRT();
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_GLOWING);

	g_pxEdgesPass2FP->Begin();
	g_pxEdgesPass2FP->SetTexture( AVP_PS3_FSFX_Edges_Pass2_PS::FP_SceneTextureSampler, &g_xRenderTargetTexture, true );
	float afParams3[4] = {m_xTintColour.m_fR * fIntensity, m_xTintColour.m_fG * fIntensity, m_xTintColour.m_fB * fIntensity,  m_fBlurWidth * 0.1f};
	g_pxEdgesPass2FP->SetFloats( AVP_PS3_FSFX_Edges_Pass2_PS::FP_Tint, 4, afParams3);
	float afParams4[4] = {1.0f / Asura_Viewport::GetWidth(), 1.0f / Asura_Viewport::GetHeight(), 0.0f, 0.0f};
	g_pxEdgesPass2FP->SetFloats( AVP_PS3_FSFX_Edges_Pass2_PS::FP_InvScreenDims, 4, afParams4);

	g_pxVertexShader->Bind();
	g_pxEdgesPass2FP->Bind();
	g_xVertexBuffer.Bind( g_axVertexDeclarationN, VDN_ELEMENTS );
	g_xVertexBuffer.RenderQuads( 4 );
	g_pxEdgesPass2FP->End();


	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__MISC );
}
#endif //defined ASURA_USE_FSFX
