//////////////////////////////////////////////////////////////////////////////
//
// AVP_PS3_Stats.cpp
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
// Description:
//
//     Project-specific PS3 code to fill out stats
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_SUPPORT_LEADERBOARDS

// Asura includes
#include "Network/Online/Asura_Leaderboards.h"

// Asura PS3
#include "Network/Online/Asura_PS3_Stats.h"

// AVP independent
#include "Core/AVP_Settings.h"
#include "Entity/PlayerInfo/AVP_ClientEntity_PlayerInfo.h"
#include "Network/AVP_Leaderboards.h"

// C O N S T A N T   D A T A /////////////////////////////////////////////////
// match up to data set up in Server Management Tool on PS3 DevNet
static const u_int auAVP_PS3_LEADERBOARD_BOARDID[AVP_NUM_LEADERBOARDS] = 
{
	1, 								// AVP_LEADERBOARD_KILLS,
	2, 								// AVP_LEADERBOARD_SCORE,
	3, 								// AVP_LEADERBOARD_MATCHES,
	4, 								// AVP_LEADERBOARD_MYSTATS,
	uASURA_PS3_INVALID_BOARD_ID,	// AVP_LEADERBOARD_INTERNAL,
};

// number of entries in above array which are not uASURA_PS3_INVALID_BOARD_ID
static const u_int uAVP_PS3_LEADERBOARD_NUM_VALID_BOARDIDS = 4;

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////
u_int Asura_PS3_Stats::Project_GetBoardID( u_int uLeaderboardIndex )
{
	Asura_Assert(uLeaderboardIndex < AVP_NUM_LEADERBOARDS, "invalid leaderboard");
	return auAVP_PS3_LEADERBOARD_BOARDID[uLeaderboardIndex];
}

u_int Asura_PS3_Stats::Project_GetNumValidBoardIDs()
{
	return uAVP_PS3_LEADERBOARD_NUM_VALID_BOARDIDS;
}

// get project code to fill out the stats for the logged-in player
void Asura_PS3_Stats::Project_GetMyStats( bool bDisconnectedEarly, u_int uGuid, 
											u_int uLeaderboardIndex, SceNpScoreValue& xRankingStat, 
											Asura_PS3_Stats_ExtraColumns& xColumns )
{
	Asura_Assert(uLeaderboardIndex < AVP_NUM_LEADERBOARDS, "invalid leaderboard");

	// use the client entity player info (which should exist on the server and contain the same results)
	const AVP_ClientEntity_PlayerInfo* pxPlayerInfo = AVP_ClientEntity_PlayerInfo::FindEntity(uGuid);
	Asura_Assert(pxPlayerInfo || bDisconnectedEarly, "failed to get player info to write stats");

	AVP_Multiplayer_PlayerScore xNullScore( pxPlayerInfo ? pxPlayerInfo->GetGuid() : ASURA_GUID_UNREGISTERED );
	const AVP_Multiplayer_PlayerScore& xScore = pxPlayerInfo ? pxPlayerInfo->GetPlayerScore() : xNullScore;

	// fetch old value from storage and add it to the values for this match
	Asura_Assert(!Asura_PS3_StatsPreGameReader::HasErrorOccurred(), 
		"shouldn't attempt to write stats if error occurred in pre-game stat retrieval");
	const SceNpScoreValue* pxOldRankingStat = Asura_PS3_StatsPreGameReader::GetRankingStat(uLeaderboardIndex);
	const SceNpScoreValue xOldRankingStat = pxOldRankingStat ? (*pxOldRankingStat) : 0;
	const u_int uRankingStatID = AVP_Leaderboards::GetAssociatedStatID(uLeaderboardIndex, 0);
	Asura_Assert(uRankingStatID != AVP_STATID_INVALID, "ranking stat should be available");
	xRankingStat = AVP_Leaderboards::GetStatResultById(uRankingStatID, xScore) + xOldRankingStat;

	const Asura_PS3_Stats_ExtraColumns* pxOldColumns = Asura_PS3_StatsPreGameReader::GetExtraColumnDataArray(uLeaderboardIndex);
	const u_int uNumCols = Asura_Leaderboard_System::Project_GetNumLeaderboardColumns(uLeaderboardIndex);
	Asura_Assert(uNumCols <= uASURA_PS3_MAX_STAT_COLS + 1, "too many columns in leaderboard");
	for (u_int uCol = 1; uCol < uNumCols; ++uCol)
	{
		const u_int uStatID = AVP_Leaderboards::GetAssociatedStatID(uLeaderboardIndex, uCol);
		const bool bIsAdditiveStat = AVP_Leaderboards::IsAdditiveStat(uStatID);
		if (bIsAdditiveStat)
		{
			// add to previous score gathered from TUS.
			const u_int uOldStat = pxOldColumns ? pxOldColumns->m_xColumns[uCol-1].m_uData : 0;
			xColumns.m_xColumns[uCol-1].m_uData = AVP_Leaderboards::GetStatResultById(uStatID, xScore) + uOldStat;
		}
		else
		{
			// it's a computed stat
			switch (uStatID)
			{
				case AVP_STATID_KILLS_TO_DEATH_RATIO:
				{
					const u_int uTotalKills = xRankingStat;
					const u_int uDeathsCol = AVP_KILLS_TOTAL_DEATHS;
					const u_int uOldDeaths = pxOldColumns ? pxOldColumns->m_xColumns[uDeathsCol-1].m_uData : 0;
					const u_int uTotalDeaths = uOldDeaths + xScore.GetNumberOfTimesKilled();
					// set ratio as float - use 1.0f if never died.
					xColumns.m_xColumns[uCol-1].m_fData = uTotalDeaths ? 
						static_cast<float>(uTotalKills) / static_cast<float>(uTotalDeaths) : 1.0f;
					break;
				}
				case AVP_STATID_WINNING_STREAK:
				{
					// TODO
					xColumns.m_xColumns[uCol-1].m_uData = 0;
					break;
				}
				default:
				{
					Asura_Assert(false, "unexpected computed stat");
					xColumns.m_xColumns[uCol-1].m_uData = 0;
					break;
				}
			}
		}
	}
}

#endif // #ifdef ASURA_SUPPORT_LEADERBOARDS
