/**
*
* AVP_Platform_GlowOutlines.cpp
*
* Created by Andrew Southgate - 30/7/2008
*
* Copyright (C) 2008 Rebellion, All Rights Reserved.
*
**/

/************/
/* Includes */
/************/

// main asura header
#include "Core/Asura.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_TextureManagement.h"

// local header
#include "SpecialFX/AVP_GlowOutlines.h"

// Asura Platform-independent
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Maths.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Colour.h"
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_MaterialResponse_system.h"
#include "Interface/Asura_CommandConsole.h"

//Asura Platform
#include "Rendering/Shaders/Asura_PS3_GCM_Shader.h"
#include "Rendering/Asura_PS3_GCM_VertexBuffer.h"
#include "Rendering/Asura_PS3_Graphics.h"
#include "Rendering/Asura_PS3_FrameBuffer.h"
#include "Rendering/Asura_PS3_GCM_IndexBuffer.h"
#include "Rendering/Asura_PS3_MaterialResponse_System.h"
#include "Rendering/Asura_PS3_RenderingOptions.h"
#include "Rendering/Asura_PS3_TextureManagement.h"
#include "Rendering/Asura_PS3_RenderingStates.h"
#include "Rendering/Asura_PS3_RSXProfiling.h"
#include "Rendering/Asura_PS3_CharacterRenderer.h"
#include "Rendering/Asura_PS3_Spu_Skinning.h"
#include "Rendering/Asura_PS3_QuadOverlayRenderer.h"
#include "Rendering/Asura_PS3_DeferredRendering.h"

// AVP Platform
#include "Rendering/AVP_Platform_Stencil_Setup.h"

// Shader definition
#include "Rendering/ShadersDefinition/AVP_PS3_ProjectShaders.h"


static float dbg_fGlowMaxDist = 15.0f;

#include "Rendering/ShadersDefinition/VP/GlowOutlinesFS_VP.h"
#include "Rendering/ShadersDefinition/VP/GlowOutlinesSkin_VP.h"

#include "Rendering/ShadersDefinition/VD/GlowOutlinesSkin_VD.h"
#include "Rendering/ShadersDefinition/VD/GlowOutlinesFS_VD.h"

#include "Rendering/ShadersDefinition/FP/GlowOutlinesFS_Horizontal_FP.h"
#include "Rendering/ShadersDefinition/FP/GlowOutlinesFS_Vertical_FP.h"
#include "Rendering/ShadersDefinition/FP/GlowOutlinesFS_Final_FP.h"
#include "Rendering/ShadersDefinition/FP/GlowOutlinesSkin_FP.h"

namespace GlowEffect {

	Asura_PS3_GCM_VertexShader*				g_pxSkinVertexShader = NULL;
	Asura_PS3_GCM_VertexShader*				g_pxSkinVertexShader_NoSPU = NULL;
	Asura_PS3_GCM_VertexShader*				g_pxFSVertexShader = NULL;

	AVP_PS3_GlowOutlinesFS_Horizontal_PS *	g_pxGlowOutlinesFS_Horizontal_PS;
	AVP_PS3_GlowOutlinesFS_Vertical_PS *	g_pxGlowOutlinesFS_Vertical_PS;
	AVP_PS3_GlowOutlinesFS_Final_PS *		g_pxGlowOutlinesFS_Final_PS;
	AVP_PS3_GlowOutlinesSkin_PS *			g_pxGlowOutlinesSkin_PS;

	Asura_PS3_GCM_VertexBuffer				g_xVertexBufferFS;

	CellGcmSurface							g_xGlowRenderTarget[3];
	CellGcmTexture							g_xGlowRenderTargetCellTexture[3];
	Asura_PS3_Texture						g_xGlowRenderTargetTexture[3];

	Asura_PS3_SPU_SkinningConstants		g_xShaderConstants;

#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
	bool ReloadShaders( bool bHelp );
#endif


	void Initialise()
	{
		g_pxSkinVertexShader = 	new AVP_PS3_VP_GlowOutlinesSkin();
		g_pxSkinVertexShader_NoSPU = 	new AVP_PS3_VP_GlowOutlinesSkin_No_SPU();
		g_pxFSVertexShader = 	new AVP_PS3_VP_GlowOutlinesFS();

		g_pxSkinVertexShader->InitVertexBufferDeclaration( g_axVertexDeclarationGlowOutlinesSkin, VD_GLOWOUTLINESSKIN_ELEMENTS );
		g_pxSkinVertexShader_NoSPU->InitVertexBufferDeclaration( g_axVertexDeclarationGlowOutlinesSkin_No_SPU, VD_GLOWOUTLINESSKIN_NO_SPU_ELEMENTS );
		g_pxFSVertexShader->InitVertexBufferDeclaration( g_axVertexDeclarationGlowOutlinesFS, VD_GLOWOUTLINESFS_ELEMENTS );


		g_pxGlowOutlinesFS_Horizontal_PS = new AVP_PS3_GlowOutlinesFS_Horizontal_PS();
		g_pxGlowOutlinesFS_Vertical_PS = new AVP_PS3_GlowOutlinesFS_Vertical_PS();
		g_pxGlowOutlinesFS_Final_PS = new AVP_PS3_GlowOutlinesFS_Final_PS();
		g_pxGlowOutlinesSkin_PS = new AVP_PS3_GlowOutlinesSkin_PS();


		g_xGlowRenderTarget[0] = *Asura_PS3_DeferredRendering::GetDeferredFrameBuffer()->GetSurface();
		g_xGlowRenderTarget[0].height >>= 1;
		g_xGlowRenderTarget[0].width >>= 1;
		g_xGlowRenderTarget[1] = g_xGlowRenderTarget[0];
		g_xGlowRenderTarget[2] = g_xGlowRenderTarget[1];
		g_xGlowRenderTarget[1].colorOffset[0] =  g_xGlowRenderTarget[0].colorOffset[2];
		g_xGlowRenderTarget[2].colorOffset[0] =  g_xGlowRenderTarget[0].colorOffset[1];
		
		g_xGlowRenderTarget[0].colorTarget = CELL_GCM_SURFACE_TARGET_MRT1;
		g_xGlowRenderTarget[1].colorTarget = CELL_GCM_SURFACE_TARGET_0;
		g_xGlowRenderTarget[2].colorTarget = CELL_GCM_SURFACE_TARGET_0;

		g_xGlowRenderTargetCellTexture[0].mipmap	= 1;
		g_xGlowRenderTargetCellTexture[0].dimension	= CELL_GCM_TEXTURE_DIMENSION_2;
		g_xGlowRenderTargetCellTexture[0].cubemap	= CELL_GCM_FALSE;
		g_xGlowRenderTargetCellTexture[0].remap		= CELL_GCM_REMAP_MODE( CELL_GCM_TEXTURE_REMAP_ORDER_XYXY, CELL_GCM_TEXTURE_REMAP_FROM_A,CELL_GCM_TEXTURE_REMAP_FROM_R, CELL_GCM_TEXTURE_REMAP_FROM_G, CELL_GCM_TEXTURE_REMAP_FROM_B, CELL_GCM_TEXTURE_REMAP_REMAP, CELL_GCM_TEXTURE_REMAP_REMAP, CELL_GCM_TEXTURE_REMAP_REMAP, CELL_GCM_TEXTURE_REMAP_REMAP );
		g_xGlowRenderTargetCellTexture[0].width		= g_xGlowRenderTarget[0].width;
		g_xGlowRenderTargetCellTexture[0].height	= g_xGlowRenderTarget[0].height;
		g_xGlowRenderTargetCellTexture[0].depth		= 1;
		g_xGlowRenderTargetCellTexture[0].location	= g_xGlowRenderTarget[0].colorLocation[0];
		g_xGlowRenderTargetCellTexture[0]._padding	= 0;
		g_xGlowRenderTargetCellTexture[0].offset	= g_xGlowRenderTarget[0].colorOffset[0];
		g_xGlowRenderTargetCellTexture[0].pitch		= g_xGlowRenderTarget[0].colorPitch[0];
		g_xGlowRenderTargetCellTexture[0].format	= CELL_GCM_TEXTURE_LN | CELL_GCM_TEXTURE_UN | CELL_GCM_TEXTURE_W16_Z16_Y16_X16_FLOAT;

		g_xGlowRenderTargetCellTexture[1] = g_xGlowRenderTargetCellTexture[0];
		g_xGlowRenderTargetCellTexture[2] = g_xGlowRenderTargetCellTexture[0];

		g_xGlowRenderTargetCellTexture[0].offset	= g_xGlowRenderTarget[0].colorOffset[0];
		g_xGlowRenderTargetCellTexture[1].offset	= g_xGlowRenderTarget[1].colorOffset[0];
		g_xGlowRenderTargetCellTexture[2].offset	= g_xGlowRenderTarget[2].colorOffset[0];

		g_xGlowRenderTargetTexture[0] =  Asura_PS3_Texture(&g_xGlowRenderTargetCellTexture[0]);
		g_xGlowRenderTargetTexture[1] =  Asura_PS3_Texture(&g_xGlowRenderTargetCellTexture[1]);
		g_xGlowRenderTargetTexture[2] =  Asura_PS3_Texture(&g_xGlowRenderTargetCellTexture[2]);

		g_xVertexBufferFS.Initialise( GLOWOUTLINESFS_VERTEX_SIZE, 4, CELL_GCM_LOCATION_MAIN );

		float* pfVertexData = ( float* )g_xVertexBufferFS.LockForWriting();
		pfVertexData[ 0 ] = -1; pfVertexData[ 1 ] =  1; pfVertexData[ 2 ] = -0; pfVertexData[ 3 ] =  1;//Top left
		pfVertexData[ 4 ] =  1; pfVertexData[ 5 ] =  1; pfVertexData[ 6 ] = -0; pfVertexData[ 7 ] =  1; //Top right
		pfVertexData[ 8 ] =  1; pfVertexData[ 9 ] = -1; pfVertexData[10 ] = -0; pfVertexData[11 ] =  1; //Bottom right
		pfVertexData[12 ] = -1; pfVertexData[13 ] = -1; pfVertexData[14 ] = -0; pfVertexData[15 ] =  1; //Bottom left
		g_xVertexBufferFS.Unlock();

#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
		Asura_CommandConsole::AddCmd(L"Render.ReloadShaders.GlowOutlines", ReloadShaders, Asura_CommandConsole::REMOVE_AT_SHUTDOWN);
#endif

	}

	void Shutdown()
	{
		delete g_pxSkinVertexShader; g_pxSkinVertexShader = NULL;
		delete g_pxSkinVertexShader_NoSPU; g_pxSkinVertexShader_NoSPU = NULL;
		delete g_pxFSVertexShader; g_pxFSVertexShader = NULL;

		delete g_pxGlowOutlinesFS_Horizontal_PS; g_pxGlowOutlinesFS_Horizontal_PS = NULL;
		delete g_pxGlowOutlinesFS_Vertical_PS; g_pxGlowOutlinesFS_Vertical_PS = NULL;
		delete g_pxGlowOutlinesFS_Final_PS; g_pxGlowOutlinesFS_Final_PS = NULL;
		delete g_pxGlowOutlinesSkin_PS; g_pxGlowOutlinesSkin_PS = NULL;

		g_xVertexBufferFS.Destroy();

#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
		Asura_CommandConsole::RemoveCmd(L"Render.ReloadShaders.GlowOutlines");
#endif

	}
#if defined(ASURA_PS3_ALLOW_SHADER_RELOAD)
	bool ReloadShaders( bool bHelp )
	{
		if ( bHelp )
		{
			Asura_CommandConsole::Printf("recompile & load pheromone trail shader");
			return true;
		}
		g_pxGlowOutlinesFS_Horizontal_PS->ReloadFromDisc();
		g_pxGlowOutlinesFS_Vertical_PS->ReloadFromDisc();
		g_pxGlowOutlinesFS_Final_PS->ReloadFromDisc();
		g_pxGlowOutlinesSkin_PS->ReloadFromDisc();



		return true;
	}
#endif

};


struct AVP_GlowOutlines_Vertex
{
	float x, y, z;
	u_int colour;
	float u, v;
	float u2, v2;
	void SetPosition(const Asura_Vector_3 &xPosition)
	{
		x = xPosition.GetX();
		y = xPosition.GetY();
		z = xPosition.GetZ();
	}
	void SetUV(const Asura_Vector_2 &xUV)
	{
		u = xUV.x;
		v = xUV.y;
	}
	void SetUV2(const Asura_Vector_2 &xUV)
	{
		u2 = xUV.x;
		v2 = xUV.y;
	}

	void SetColour(u_int uColour)
	{
		colour = uColour;
	}
	void Set(const Asura_Vector_3 &xPosition, const Asura_Vector_2 &xUV, const Asura_Vector_2 &xUV2, u_int uColour)
	{
		SetPosition(xPosition);
		SetColour(uColour);
		SetUV(xUV);
		SetUV2(xUV2);
	}
};


void AVP_GlowOutlines::Platform_Initialise()
{
	GlowEffect::Initialise();
}

void AVP_GlowOutlines::Platform_Shutdown()
{
	GlowEffect::Shutdown();
}

static void FixUpAsura_Hierarchy_SystemForAnimation( Asura_Animation* pxAnim, const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xOri, u_int uRenderFlags )
{
	Asura_Hierarchy_System::s_pxSkin		= pxAnim->GetSkin(0);
	Asura_Hierarchy_System::s_uSkinIndex	= 0;
	if( !Asura_Hierarchy_System::s_pxSkin ) { return; }

	Asura_Hierarchy_System::s_pxTweenStorage = pxAnim->GetTweenStore();
	if(!Asura_Hierarchy_System::s_pxTweenStorage) return;

#ifdef ASURA_HIERARCHY_QUAD
	// RJT TODO: Ultimately this transform should be passed in
	Asura_Hierarchy_System::s_xBaseTransform = Asura_AMatrix4x4(xOri, xPos);
	Asura_Hierarchy_System::s_xBaseTransform.InvertOrthogonal(Asura_Hierarchy_System::s_xInverseBaseTransform);
#endif

	Asura_Assert( Asura_Hierarchy_System::s_pxSkin->m_pxSectionBoundingBoxData, "" );

	Asura_Hierarchy_System::BonesToWorldspaceFromTweenStorage();
	pxAnim->UpdateLighting(uRenderFlags);

	if( Asura_Hierarchy_System::s_bBonesInWorldspace )
	{
		Asura_Hierarchy_System::s_xInverseBaseTransform.ApplyToArray( Asura_Hierarchy_System::s_axBoneTransforms, Asura_Hierarchy_System::s_pxSkin->NumberOfBones, Asura_Hierarchy_System::s_axBoneTransforms);
	}

	for( int i = 0; i < Asura_Hierarchy_System::s_pxSkin->NumberOfBones; ++i )
	{
		Asura_Hierarchy_System::s_axBoneTransforms[i].Apply(Asura_Hierarchy_System::s_pxSkin->m_pxBoneRestPoseInverseTransforms[i], Asura_Hierarchy_System::s_axBoneTransforms[i]);
	}
}



static Asura_PS3_GCM_IndexBuffer* SetupSPU(	Asura_PS3_CharacterRenderer_Skin* pxCharacterStrips,
															  const Asura_Animation& xAnimation,
															  Asura_AMatrix4x4_InParam xTransform,
															  Asura_AMatrix4x4_InParam xInverseTransform,
															  const Asura_AMatrix4x4* const pxBoneTransforms,
															  u_int uNumBones,
															  u_int uSkinIdx)
{
	Asura_Platform_Animation_RenderCache* pxRenderCache	= xAnimation.GetRenderCache();

	// Decide if we're going to use the vertex shader or SPU for rendering
	bool bSpuVS = Asura_PS3_SPU_Skinning::IsSpuable(pxCharacterStrips);

	// fill vertex program constants
	if ( bSpuVS )
	{
		Asura_PS3_CharacterRenderer::UseSpuSkinning( pxCharacterStrips, xAnimation, SPU_SKINNING_POS_AND_CULL,
														xTransform, pxBoneTransforms, uNumBones, uSkinIdx );

		GlowEffect::g_pxSkinVertexShader->Bind();
		const Asura_AMatrix4x4& xWorldViewProjection = Asura_PS3_Graphics::GetWorldViewProjectionMatrix( &Asura_Hierarchy_System::s_xBaseTransform );
		GlowEffect::g_pxSkinVertexShader->SetUniformParameter( SK_PhermoneGlowSkin_WorldViewProj, xWorldViewProjection.GetElements() );
	}
	else
	{
		GlowEffect::g_pxSkinVertexShader_NoSPU->Bind();
		const Asura_AMatrix4x4& xWorldViewProjection = Asura_PS3_Graphics::GetWorldViewProjectionMatrix( &Asura_Hierarchy_System::s_xBaseTransform );
		GlowEffect::g_pxSkinVertexShader_NoSPU->SetUniformParameter( SK_PhermoneGlowSkin_No_SPU_WorldViewProj, xWorldViewProjection.GetElements() );

		// Set bone matrices
		// Pack bone matrices as 4x3 for vertex shader
		const Asura_AMatrix4x4 *pxRenderBones = pxBoneTransforms;

		static float axData[150*3*4];
		Asura_Assertf( uNumBones <= 150, ("PS3 only supports up to 220 bones, this skin has %d", uNumBones) );
		for ( u_int i=0; i < uNumBones; i++ )
		{
			// store transposed, so only need 3 float4's not 4 in shader
			pxRenderBones[i].StoreRowsUnaligned(&axData[i*3*4]); // this stores 4x4 to our 3x4 space, hopefully it won't break anything
		}

		//g_pxInUseVS->SetUniformParameter( SK_BoneTransforms, axData, xSkin.NumberOfBones*3 );
		//avoid the huge copy by just setting it directly

		cellGcmSetVertexProgramParameterBlockUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), GlowEffect::g_pxSkinVertexShader_NoSPU->GetRegisterIndex(SK_PhermoneGlowSkin_No_SPU_BoneTransforms), uNumBones*3, axData );
		
		//invalidate the cached vertex parameters, since we just set them directly
		GlowEffect::g_pxSkinVertexShader_NoSPU->InvalidateRegisterCache();
	}

	Asura_PS3_GCM_IndexBuffer *pIndexBuffer = NULL;

	if ( bSpuVS )
	{
		Asura_PS3_GCM_VertexBuffer *pProcessedVerts = NULL;

		u_int uNumVerts = pxCharacterStrips->GetVertexBuffer()->GetNumVerts();

		pIndexBuffer = pxRenderCache->ObtainIndexBuffer(ASURA_PS3_RENDERCACHE_BUFFER_VIEWCULLED,uSkinIdx);
		pProcessedVerts = pxRenderCache->ObtainVertexBuffer( ASURA_PS3_RENDERCACHE_BUFFER_WORLDPOS, uSkinIdx );

		//use the verts processed by SPU
		pProcessedVerts->Bind( g_axVertexDeclarationGlowOutlinesSkin, VD_GLOWOUTLINESSKIN_ELEMENTS);


	}
	else
	{
		//failed to setup SPU processing, do it using a vertex shader
		pxCharacterStrips->GetVertexBuffer()->Bind( g_axVertexDeclarationGlowOutlinesSkin_No_SPU, VD_GLOWOUTLINESSKIN_NO_SPU_ELEMENTS );

		pIndexBuffer = pxCharacterStrips->GetIndexBuffer();
	}

	return pIndexBuffer;
}



static void RenderSkin( const Asura_Animation& xAnimation, const Asura_Hierarchy_Skin& xSkin, Asura_AMatrix4x4_InParam xTransform,
					   Asura_AMatrix4x4_InParam xInverseTransform, const Asura_AMatrix4x4* const pxBoneTransforms, u_int uSkinIdx)

{
	Asura_PS3_CharacterRenderer_Skin* const pxCharacterStrips = Asura_PS3_CharacterRenderer::GetCharacter( xSkin.m_uNameHashID );
	if( !pxCharacterStrips ) return;

	// Decide if we're going to use the vertex shader or SPU for rendering
	bool bSpuVS = Asura_PS3_SPU_Skinning::IsSpuable(pxCharacterStrips);



	const Asura_AMatrix4x4& xWorldViewProjection = Asura_PS3_Graphics::GetWorldViewProjectionMatrix( &Asura_Hierarchy_System::s_xBaseTransform );

	Asura_PS3_GCM_IndexBuffer *pxIndexBuffer = SetupSPU(pxCharacterStrips, xAnimation, xTransform, xInverseTransform, pxBoneTransforms, xSkin.NumberOfBones, uSkinIdx);

	for( u_int u = 0; u < pxCharacterStrips->GetNumStrips(); ++u )
	{
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		if (xAnimation.GetSkinElementVisibility(pxCharacterStrips->GetStrip(u).m_iElementID))
#endif
		{
			const Asura_Hash_ID uHash = pxCharacterStrips->GetStrip(u).m_auMaterialResponseHashIDs[0];
			const Asura_MaterialResponse* const pxMaterial = Asura_MaterialResponse_System::GetMaterialByHashID( uHash );

			if( pxMaterial && !pxMaterial->GetDontRender())
			{
				pxIndexBuffer->RenderTriangles( pxCharacterStrips->GetStrip(u).m_uNumIndices, pxCharacterStrips->GetStrip( u ).m_uStartIndex );

				Asura_PS3_Graphics::Flush();
			}
		}
	}

	if (bSpuVS)
	{
		Asura_PS3_SPU_Skinning::NotifyBufferUsed();
	}

}

void SetupInitialRenderState()
{
	Asura_PS3_RenderingStates::SetSurface( &GlowEffect::g_xGlowRenderTarget[0] );
	Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0123_ALL );

	Asura_PS3_FragmentShaderCopy *pxFP = Asura_PS3_GCM_ShadersLibrary::GetPermanentFP( ASURA_SHADER_HASH_GLOBAL, ASURA_GLOBAL_SHADER_BLANK_MRT );

	static const float afCols[16] = { 0,0.0f,0,0, 0,0,0,0, 0,0,0,0, 0,Asura_Camera::GetFarClip(),0,0 };
	pxFP->SetArray( 0, 4, 4, afCols ); //BW this is nasty hard coded index of 0, perfectly safe but really it should do this through a wrapper
	pxFP->Bind();


	Asura_Overlay_Quad xQuad;
	xQuad.Colour = 0xFFCC00FF;

	xQuad.m_bNonRotated = true;
	xQuad.m_bGouraudShaded=false;
	xQuad.TextureIndex = ASURA_TEXTURE_NONE;
	xQuad.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;

	xQuad.SetRectXY( 0.0f, 0.0f, Asura_Viewport::GetWidth(), Asura_Viewport::GetHeight() );
	Asura_PS3_QuadOverlayRenderer::RenderEx ( &xQuad );

	Asura_PS3_RenderingStates::RequestAlphaTest( false );
	Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
	Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF);
	Asura_PS3_RenderingStates::SetColorMaskMrtState(ASURA_PS3_COLOR_MASK_MRT0_ALL | (ASURA_PS3_COLOR_MASK_MRT0_ALPHA << 4) );
	Asura_PS3_RenderingStates::RequestZBufferEnabled( false);
	GlowEffect::g_pxGlowOutlinesSkin_PS->Begin();
	GlowEffect::g_pxGlowOutlinesSkin_PS->Bind();
}



void AVP_GlowOutlines::Platform_Render( DepthSortedSkinEntry * const *pxBuffer)
{
	ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__PROJECTSPECIFIC_2, " AVP_GlowOutlines::Platform_RenderGlows" );

 	DepthSortedSkinEntry *pxFirst = NULL, *pxPrev = NULL;
	for (int i = NUM_DEPTH_SORT_SLOTS-1; i >= 0; --i)	
	{
		// fix up the list slightly so the next time we iterate its just a flat linked list, we dont need to go through lots of empty buckets twice...
		if (pxBuffer[i])
		{
			if (pxPrev)
				pxPrev->m_pxNext = pxBuffer[i];

			if (!pxFirst)
			{
				pxFirst = pxBuffer[i];

				// Set render states
				SetupInitialRenderState();
				ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__PROJECTSPECIFIC_2, "Render Skins" );
			}
		}
		for (DepthSortedSkinEntry *pxEntry0 = pxBuffer[i]; pxEntry0; pxEntry0 = pxEntry0->m_pxNext)
		{
			pxPrev = pxEntry0;

			Asura_Animation *pxAnim = pxEntry0->m_xSortedRenderItem.m_pxAnimation;

			//TODO
			FixUpAsura_Hierarchy_SystemForAnimation(pxAnim, pxEntry0->m_xSortedRenderItem.m_xPosition, pxEntry0->m_xSortedRenderItem.m_xOrient, pxEntry0->m_xSortedRenderItem.m_uRenderFlags);

			for( int iSkinIndex = 0; iSkinIndex < ASURA_ANIMATION_SKIN_COUNT; ++iSkinIndex )
			{
				const Asura_Hierarchy_Skin *pxSkin = pxAnim->GetSkin(iSkinIndex);

				if (pxSkin)
				{
					for( int i = 0; i < Asura_Camera::GetNumberOfCameras(); ++i )
					{
						if((pxEntry0->m_xSortedRenderItem.m_uCamVisFlags & (1 << i)) != 0)
						{
							if((Asura_Hierarchy_System::ShouldJustRenderForCurCam() && ((u_int)i == Asura_Camera::GetCameraIndex())) || Asura_Camera::IsCameraActive(i))
							{
								Asura_Camera::SetCameraIndex(i);
								Asura_Render::ApplyCameraSettings();

								// Select LOD mesh based on distance to first bone, since object may be animating far away from its location
								Asura_Hierarchy_Skin const* const	pxSkinLOD 
									= pxSkin->GetDetailSkinFromDistance( (Asura_Hierarchy_System::GetWorldspaceBonePos(0) - *Asura_Camera::GetPosition()).Magnitude() );

								if(pxSkinLOD)
								{
									//TODO
									float afBuf[4] = {pxEntry0->m_xColour.m_fR, pxEntry0->m_xColour.m_fG, pxEntry0->m_xColour.m_fB, dbg_fGlowMaxDist};
									GlowEffect::g_pxSkinVertexShader->SetUniformParameter( SK_PhermoneGlowSkin_BlurColour, afBuf);
									GlowEffect::g_pxSkinVertexShader_NoSPU->SetUniformParameter( SK_PhermoneGlowSkin_No_SPU_BlurColour, afBuf);
									RenderSkin(*pxEntry0->m_xSortedRenderItem.m_pxAnimation, *pxSkinLOD, Asura_Hierarchy_System::s_xBaseTransform, Asura_Hierarchy_System::s_xInverseBaseTransform, Asura_Hierarchy_System::s_axBoneTransforms, iSkinIndex);
								}
								else
								{
									Asura_Assertf(false, ("Couldn't retrieve LOD skin for '%s' at distance %.02f.", pxSkin->GetName(), (Asura_Hierarchy_System::GetWorldspaceBonePos(0) - *Asura_Camera::GetPosition()).Magnitude()));
								}
							}
						}
					}
				}
			}
		}
	}

 
  	if (pxFirst)
  	{
  		GlowEffect::g_pxGlowOutlinesSkin_PS->End();
  		ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__PROJECTSPECIFIC_2);

		ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__PROJECTSPECIFIC_2, "Horizontal blur" );
		// Take the colour buffer we have just stored all the skins in and horizontally blur it
		{
			Asura_PS3_RenderingStates::SetSurface( &GlowEffect::g_xGlowRenderTarget[1] );
			Asura_PS3_RenderingStates::SetColorMaskMrtState( ASURA_PS3_MRT_MRT0_ALL );

			// Pixel shader will output fog colour with fog factor as alpha
			Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
			Asura_PS3_RenderingStates::RequestCullMode( ASURA_CULLMODE_CW );
			Asura_PS3_RenderingStates::RequestAlphaTest( false );

			// Taken these settings out of the effect, as they were screwing up the cached states
			Asura_PS3_RenderingStates::RequestZBufferEnabled(false);
			Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);
			GlowEffect::g_pxFSVertexShader->Bind();

			GlowEffect::g_pxGlowOutlinesFS_Horizontal_PS->Begin();
			float afParams[4] = {0.0f, 0.0f, 0.0f, 0.0f};
			afParams[0] = GLOW_RADIUS;
			GlowEffect::g_pxGlowOutlinesFS_Horizontal_PS->SetFloats(AVP_PS3_GlowOutlinesFS_Horizontal_PS::FP_BlurWidth, 1, afParams);
			afParams[1] = GLOW_MAX_DISTANCE;
			GlowEffect::g_pxGlowOutlinesFS_Horizontal_PS->SetFloats(AVP_PS3_GlowOutlinesFS_Horizontal_PS::FP_BlurDistance, 1, afParams);


			GlowEffect::g_pxGlowOutlinesFS_Horizontal_PS->SetTexture(AVP_PS3_GlowOutlinesFS_Horizontal_PS::FP_SceneTexture, GlowEffect::g_xGlowRenderTargetTexture + 0, false);

			GlowEffect::g_pxGlowOutlinesFS_Horizontal_PS->Bind();
			GlowEffect::g_xVertexBufferFS.Bind( g_axVertexDeclarationGlowOutlinesFS, VD_GLOWOUTLINESFS_ELEMENTS );
			GlowEffect::g_xVertexBufferFS.RenderQuads(4);
			GlowEffect::g_pxGlowOutlinesFS_Horizontal_PS->End();

		}
		ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__PROJECTSPECIFIC_2);


 		ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__PROJECTSPECIFIC_2, "Vertical blur" );
  		// Take the colour buffer we have just stored all the skins in and horizontally blur it
  		{
			Asura_PS3_RenderingStates::SetSurface( &GlowEffect::g_xGlowRenderTarget[2] );

			// Pixel shader will output fog colour with fog factor as alpha
			Asura_PS3_RenderingStates::SetAlphaBlending(true);
			Asura_PS3_RenderingStates::InvalidateTranslucencyMode();
			cellGcmSetBlendFuncUnsafeInline( Asura_PS3_Graphics::GetGcmCurrentContext(), CELL_GCM_DST_ALPHA, CELL_GCM_ZERO, CELL_GCM_ZERO, CELL_GCM_ONE );


			// Taken these settings out of the effect, as they were screwing up the cached states
			GlowEffect::g_pxFSVertexShader->Bind();

			GlowEffect::g_pxGlowOutlinesFS_Vertical_PS->Begin();
			float afParams[4] = {0.0f, 0.0f, 0.0f, 0.0f};
			afParams[0] = GLOW_RADIUS;
			GlowEffect::g_pxGlowOutlinesFS_Vertical_PS->SetFloats(AVP_PS3_GlowOutlinesFS_Vertical_PS::FP_BlurWidth, 1, afParams);
			afParams[1] = GLOW_MAX_DISTANCE;
			GlowEffect::g_pxGlowOutlinesFS_Vertical_PS->SetFloats(AVP_PS3_GlowOutlinesFS_Vertical_PS::FP_BlurDistance, 1, afParams);


			GlowEffect::g_pxGlowOutlinesFS_Vertical_PS->SetTexture(AVP_PS3_GlowOutlinesFS_Vertical_PS::FP_SceneTexture, GlowEffect::g_xGlowRenderTargetTexture + 1, false);

			GlowEffect::g_pxGlowOutlinesFS_Vertical_PS->Bind();
			GlowEffect::g_xVertexBufferFS.Bind( g_axVertexDeclarationGlowOutlinesFS, VD_GLOWOUTLINESFS_ELEMENTS );
			GlowEffect::g_xVertexBufferFS.RenderQuads(4);
			GlowEffect::g_pxGlowOutlinesFS_Vertical_PS->End();

		}
  		ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__PROJECTSPECIFIC_2);

		ASURA_PS3_RSX_PROFILE__BEGIN( ASURA_PS3_RSX_PROFILE__PROJECTSPECIFIC_2, "Apply to screen" );
		// Take the colour buffer we have just stored all the skins in and horizontally blur it
		{
			Asura_PS3_DeferredRendering::Bind();

			// Pixel shader will output fog colour with fog factor as alpha
			Asura_PS3_RenderingStates::RequestTranslucencyMode( ASURA_TRANSLUCENCY_GLOWING );

			GlowEffect::g_pxFSVertexShader->Bind();

			GlowEffect::g_pxGlowOutlinesFS_Final_PS->Begin();

			GlowEffect::g_pxGlowOutlinesFS_Final_PS->SetTexture(AVP_PS3_GlowOutlinesFS_Final_PS::FP_SceneTexture, GlowEffect::g_xGlowRenderTargetTexture + 2, false);

			GlowEffect::g_pxGlowOutlinesFS_Final_PS->Bind();
			GlowEffect::g_xVertexBufferFS.Bind( g_axVertexDeclarationGlowOutlinesFS, VD_GLOWOUTLINESFS_ELEMENTS );
			GlowEffect::g_xVertexBufferFS.RenderQuads(4);
			GlowEffect::g_pxGlowOutlinesFS_Final_PS->End();

		}
		ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__PROJECTSPECIFIC_2);

		{
			Asura_PS3_RenderingStates::SetAlphaBlending(false);
			// Restore renderstates
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
			Asura_PS3_RenderingStates::SetAlphaBlending(false);
			Asura_PS3_RenderingStates::InvalidateTranslucencyMode();

			Asura_PS3_RenderingStates::SetColorMaskMrtState(ASURA_PS3_COLOR_MASK_MRT0_ALL);

			Asura_PS3_RenderingStates::RequestZBufferWriteEnabled(false);
			Asura_PS3_RenderingStates::RequestZBufferEnabled( true);
		}

	}
	ASURA_PS3_RSX_PROFILE__END( ASURA_PS3_RSX_PROFILE__PROJECTSPECIFIC_2);

}


