///////////////////////////////////////////////////////////////////////////////
//
// File   : AVP_Platform_Vertebrate_InputHandler.cpp
// Author : Callum Hope
// Date   : 30-Oct-2008
// Desc	  : AVP/PC/Actor/Vertebrates... Platform dependant functionality for the input handler
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

// core header
#include "Core/Asura.h"

// This file's header...
#include "Actor/Vertebrates/AVP_Vertebrate_InputHandler.h"

// Asura Headers
#include "Input/Asura_ControlMapping_Axis.h"
#include "Input/Asura_ControlMapping_HoldKey.h"
#include "Input/Asura_ControlMapping_Key.h"
#include "Input/Asura_ControlMapping_Combination.h"
#include "Input/Asura_ControlMapping_TapAxis.h"
#include "Input/Asura_ControlMapping_TapKey.h"
#include "Input/Asura_PlatformInput.h"
#include "Rendering/AVP_Font_System.h"

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_InputHandler::Platform_GetTileIdForMapping (Private)
// Author       : Callum Hope
// Date         : 30-Oct-2008
// Desc			: Finds the Key assigned to a mapping and returns an id to the texture tile that is used to display it in game. 
//
// Arguments	: pxMapping, the mapping that has a key mapped to it. 	
//				  uComboId, which of the mappings are we looking at if this is a combo mapping. must be only 0 or 1
//
// Return		: u_int, the id that is found for the key, or 0 in an error state. 
////////////////////////////////////////////////////////////////////////////////
u_int AVP_Vertebrate_InputHandler::Platform_GetTileIdForMapping( const Asura_ControlMapping* pxMapping, u_int uComboId )
{
	int iKeyID;

	if( !pxMapping )
	{
		Asura_Assert( false, "Platform_GetTileIdForMapping\n A Tile ID has been requested but no mapping was sent to find it from!" );
		return 0;
	}

	if( ( uComboId != 0 ) && ( uComboId != 1 ) )
	{
		Asura_Assertf( false, ( "Platform_GetTileIdForMapping\n Combo ID passed in is not valid must be 0 or 1, it is %d", uComboId ) );
		uComboId = 0;
	}

	if( pxMapping->GetInstanceMappingType() == Asura_ControlMapping_Combination::GetClassMappingType() )
	{
		iKeyID = static_cast<const Asura_ControlMapping_Combination*>( pxMapping )->GetControlMapping( uComboId )->GetControlID();
	}
	else
	{
		iKeyID = pxMapping->GetControlID();
	}
	
	if( ( pxMapping->GetInstanceMappingType() != Asura_ControlMapping_Axis::GetClassMappingType() ) && ( pxMapping->GetInstanceMappingType() != Asura_ControlMapping_TapAxis::GetClassMappingType() ) )
	{
		switch( iKeyID )
		{
			////////////////////////////////////////////////////////////////////////////////////
			//  BUTTONS

			case ASURA_XBOX360_KEYID_PAD1_A:
			case ASURA_XBOX360_KEYID_PAD2_A:
			case ASURA_XBOX360_KEYID_PAD3_A:
			case ASURA_XBOX360_KEYID_PAD4_A:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_1;

			case ASURA_XBOX360_KEYID_PAD1_B:
			case ASURA_XBOX360_KEYID_PAD2_B:
			case ASURA_XBOX360_KEYID_PAD3_B:
			case ASURA_XBOX360_KEYID_PAD4_B:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_2;

			case ASURA_XBOX360_KEYID_PAD1_X:
			case ASURA_XBOX360_KEYID_PAD2_X:
			case ASURA_XBOX360_KEYID_PAD3_X:
			case ASURA_XBOX360_KEYID_PAD4_X:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_3;

			case ASURA_XBOX360_KEYID_PAD1_Y:
			case ASURA_XBOX360_KEYID_PAD2_Y:
			case ASURA_XBOX360_KEYID_PAD3_Y:
			case ASURA_XBOX360_KEYID_PAD4_Y:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_4;

			case ASURA_XBOX360_KEYID_PAD1_L_BUMPER:
			case ASURA_XBOX360_KEYID_PAD2_L_BUMPER:
			case ASURA_XBOX360_KEYID_PAD3_L_BUMPER:
			case ASURA_XBOX360_KEYID_PAD4_L_BUMPER:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_5;

			case ASURA_XBOX360_KEYID_PAD1_R_BUMPER:
			case ASURA_XBOX360_KEYID_PAD2_R_BUMPER:
			case ASURA_XBOX360_KEYID_PAD3_R_BUMPER:
			case ASURA_XBOX360_KEYID_PAD4_R_BUMPER:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_6;

			case ASURA_XBOX360_KEYID_PAD1_L_TRIGGER:
			case ASURA_XBOX360_KEYID_PAD2_L_TRIGGER:
			case ASURA_XBOX360_KEYID_PAD3_L_TRIGGER:
			case ASURA_XBOX360_KEYID_PAD4_L_TRIGGER:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_7;

			case ASURA_XBOX360_KEYID_PAD1_R_TRIGGER:
			case ASURA_XBOX360_KEYID_PAD2_R_TRIGGER:
			case ASURA_XBOX360_KEYID_PAD3_R_TRIGGER:
			case ASURA_XBOX360_KEYID_PAD4_R_TRIGGER:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_8;

			case ASURA_XBOX360_KEYID_PAD1_L_THUMB:
			case ASURA_XBOX360_KEYID_PAD2_L_THUMB:
			case ASURA_XBOX360_KEYID_PAD3_L_THUMB:
			case ASURA_XBOX360_KEYID_PAD4_L_THUMB:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_9;

			case ASURA_XBOX360_KEYID_PAD1_R_THUMB:
			case ASURA_XBOX360_KEYID_PAD2_R_THUMB:
			case ASURA_XBOX360_KEYID_PAD3_R_THUMB:
			case ASURA_XBOX360_KEYID_PAD4_R_THUMB:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_10;

			case ASURA_XBOX360_KEYID_PAD1_START:
			case ASURA_XBOX360_KEYID_PAD2_START:
			case ASURA_XBOX360_KEYID_PAD3_START:
			case ASURA_XBOX360_KEYID_PAD4_START:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_11;

			case ASURA_XBOX360_KEYID_PAD1_BACK:
			case ASURA_XBOX360_KEYID_PAD2_BACK:
			case ASURA_XBOX360_KEYID_PAD3_BACK:
			case ASURA_XBOX360_KEYID_PAD4_BACK:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_12;

			case ASURA_XBOX360_KEYID_PAD1_UP:
			case ASURA_XBOX360_KEYID_PAD2_UP:
			case ASURA_XBOX360_KEYID_PAD3_UP:
			case ASURA_XBOX360_KEYID_PAD4_UP:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_13;

			case ASURA_XBOX360_KEYID_PAD1_DOWN:
			case ASURA_XBOX360_KEYID_PAD2_DOWN:
			case ASURA_XBOX360_KEYID_PAD3_DOWN:
			case ASURA_XBOX360_KEYID_PAD4_DOWN:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_14;

			case ASURA_XBOX360_KEYID_PAD1_LEFT:
			case ASURA_XBOX360_KEYID_PAD2_LEFT:
			case ASURA_XBOX360_KEYID_PAD3_LEFT:
			case ASURA_XBOX360_KEYID_PAD4_LEFT:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_15;

			case ASURA_XBOX360_KEYID_PAD1_RIGHT:
			case ASURA_XBOX360_KEYID_PAD2_RIGHT:
			case ASURA_XBOX360_KEYID_PAD3_RIGHT:
			case ASURA_XBOX360_KEYID_PAD4_RIGHT:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_16;

			////////////////////////////////////////////////////////////////////////////////////
			//  NOT ASSIGNED OR NOT USED

			case ASURA_KEYID_VOID:
			default:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_NA;
		}
	}
	else
	{
		// else if this is any kind of axis mapping
		switch( iKeyID )
		{
			////////////////////////////////////////////////////////////////////////////////////
			//  AXIS ON CONTROLLER

			case ASURA_XBOX360_AXISID_PAD1_LSTICK_H:
			case ASURA_XBOX360_AXISID_PAD2_LSTICK_H:
			case ASURA_XBOX360_AXISID_PAD3_LSTICK_H:
			case ASURA_XBOX360_AXISID_PAD4_LSTICK_H:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_LSTICK_H;

			case ASURA_XBOX360_AXISID_PAD1_LSTICK_V:
			case ASURA_XBOX360_AXISID_PAD2_LSTICK_V:
			case ASURA_XBOX360_AXISID_PAD3_LSTICK_V:
			case ASURA_XBOX360_AXISID_PAD4_LSTICK_V:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_LSTICK_V;
			
			case ASURA_XBOX360_AXISID_PAD1_RSTICK_H:
			case ASURA_XBOX360_AXISID_PAD2_RSTICK_H:
			case ASURA_XBOX360_AXISID_PAD3_RSTICK_H:
			case ASURA_XBOX360_AXISID_PAD4_RSTICK_H:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_RSTICK_H;

			case ASURA_XBOX360_AXISID_PAD1_RSTICK_V:
			case ASURA_XBOX360_AXISID_PAD2_RSTICK_V:
			case ASURA_XBOX360_AXISID_PAD3_RSTICK_V:
			case ASURA_XBOX360_AXISID_PAD4_RSTICK_V:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_AXIS_RSTICK_V;

			////////////////////////////////////////////////////////////////////////////////////
			//  NOT ASSIGNED OR NOT USED
			case ASURA_AXIS_VOID:
			default:
				return AVP_Buttons::AVP_FONTSYSTEM_SPECIALCHAR_NA;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Vertebrate_InputHandler::GetDescStringForMapping
// Author       : Callum Hope
// Date         : 30-Oct-2008
// Desc			: Finds the Key assigned to a mapping and prints out a string for describe what it is for menus and debugging. 
//
// Arguments	: pxDescString, the AsuraString to be filled with the resulting desc. 
//				  pxMapping, the mapping that has a key mapped to it. 				  
//
// Return		: bool flag to signify if a string was successfully created and entered to Asura_String Argument. 
////////////////////////////////////////////////////////////////////////////////
bool AVP_Vertebrate_InputHandler::Platform_GetDescStringForMapping( Asura_String* pxDescString, const Asura_ControlMapping* pxMapping )
{
	int iStringLength = 64;
	int iKeyID;
	char* psHold = "Hold ";
	char* psTap = "Tap ";

	if( ! pxDescString || ! pxMapping )
	{
		return false;
	}

	// check for any needed prefixes
	if( pxMapping->GetInstanceMappingType() == Asura_ControlMapping_HoldKey::GetClassMappingType() )
	{
		// if its a HOLDKEY mapping
		Asura_PrintfToBfr( pxDescString->GetString(), psHold, iStringLength );
	}
	else if( ( pxMapping->GetInstanceMappingType() == Asura_ControlMapping_TapAxis::GetClassMappingType() ) ||
			 ( pxMapping->GetInstanceMappingType() == Asura_ControlMapping_TapKey::GetClassMappingType() ) )
	{
		// if its a TAPKEY or TAPAXIS mapping
		Asura_PrintfToBfr( pxDescString->GetString(), psTap, iStringLength );
	}
	else if( pxMapping->GetInstanceMappingType() == Asura_ControlMapping_Combination::GetClassMappingType() )
	{
		// if its a COMBINATION mapping
		//double check before casting this maping to unlock functionality of this derioved type
		if( pxMapping->IsComboMapping() )
		{
			// special case, this function must run though twice and return from here.
			Platform_GetDescStringForMapping( pxDescString, static_cast<const Asura_ControlMapping_Combination*>( pxMapping )->GetControlMapping( 0 ) );

			// add on a plus sign for between the two descriptions. 
			Asura_PrintfToBfr( pxDescString->GetString(), "%s + ", iStringLength, true, pxDescString->GetString() );

			Platform_GetDescStringForMapping( pxDescString, static_cast<const Asura_ControlMapping_Combination*>( pxMapping )->GetControlMapping( 1 ) );
			
			// and we are done here. 
			return true;
		}
	}
	else
	{
		// no prefix needed, do nothing
	}

	// now for the long labourious switch case to end all switch cases. 
	iKeyID = pxMapping->GetControlID();

	if( ( pxMapping->GetInstanceMappingType() != Asura_ControlMapping_Axis::GetClassMappingType() ) && ( pxMapping->GetInstanceMappingType() != Asura_ControlMapping_TapAxis::GetClassMappingType() ) )
	{
		switch( iKeyID )
		{
			case ASURA_XBOX360_KEYID_PAD1_A: 
			case ASURA_XBOX360_KEYID_PAD2_A: 
			case ASURA_XBOX360_KEYID_PAD3_A: 
			case ASURA_XBOX360_KEYID_PAD4_A: 
				Asura_PrintfToBfr( pxDescString->GetString(), "%sA", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_B: 
			case ASURA_XBOX360_KEYID_PAD2_B: 
			case ASURA_XBOX360_KEYID_PAD3_B: 
			case ASURA_XBOX360_KEYID_PAD4_B:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sB", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_X: 
			case ASURA_XBOX360_KEYID_PAD2_X: 
			case ASURA_XBOX360_KEYID_PAD3_X: 
			case ASURA_XBOX360_KEYID_PAD4_X: 
				Asura_PrintfToBfr( pxDescString->GetString(), "%sX", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_Y: 
			case ASURA_XBOX360_KEYID_PAD2_Y: 
			case ASURA_XBOX360_KEYID_PAD3_Y: 
			case ASURA_XBOX360_KEYID_PAD4_Y: 
				Asura_PrintfToBfr( pxDescString->GetString(), "%sY", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_L_BUMPER: 
			case ASURA_XBOX360_KEYID_PAD2_L_BUMPER: 
			case ASURA_XBOX360_KEYID_PAD3_L_BUMPER: 
			case ASURA_XBOX360_KEYID_PAD4_L_BUMPER:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sLB", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_R_BUMPER: 
			case ASURA_XBOX360_KEYID_PAD2_R_BUMPER: 
			case ASURA_XBOX360_KEYID_PAD3_R_BUMPER: 
			case ASURA_XBOX360_KEYID_PAD4_R_BUMPER:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sRB", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_L_TRIGGER: 
			case ASURA_XBOX360_KEYID_PAD2_L_TRIGGER: 
			case ASURA_XBOX360_KEYID_PAD3_L_TRIGGER: 
			case ASURA_XBOX360_KEYID_PAD4_L_TRIGGER:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sLT", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_R_TRIGGER: 
			case ASURA_XBOX360_KEYID_PAD2_R_TRIGGER: 
			case ASURA_XBOX360_KEYID_PAD3_R_TRIGGER: 
			case ASURA_XBOX360_KEYID_PAD4_R_TRIGGER:
				Asura_PrintfToBfr( pxDescString->GetString(), "%RT", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_L_THUMB: 
			case ASURA_XBOX360_KEYID_PAD2_L_THUMB: 
			case ASURA_XBOX360_KEYID_PAD3_L_THUMB: 
			case ASURA_XBOX360_KEYID_PAD4_L_THUMB:
				Asura_PrintfToBfr( pxDescString->GetString(), "%LS", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_R_THUMB:
			case ASURA_XBOX360_KEYID_PAD2_R_THUMB:
			case ASURA_XBOX360_KEYID_PAD3_R_THUMB:
			case ASURA_XBOX360_KEYID_PAD4_R_THUMB:
				Asura_PrintfToBfr( pxDescString->GetString(), "%RS", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_START:
			case ASURA_XBOX360_KEYID_PAD2_START:
			case ASURA_XBOX360_KEYID_PAD3_START:
			case ASURA_XBOX360_KEYID_PAD4_START:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sSTART", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_BACK:
			case ASURA_XBOX360_KEYID_PAD2_BACK:
			case ASURA_XBOX360_KEYID_PAD3_BACK:
			case ASURA_XBOX360_KEYID_PAD4_BACK:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sBACK", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_UP:
			case ASURA_XBOX360_KEYID_PAD2_UP:
			case ASURA_XBOX360_KEYID_PAD3_UP:
			case ASURA_XBOX360_KEYID_PAD4_UP:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sDIR UP", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_DOWN:
			case ASURA_XBOX360_KEYID_PAD2_DOWN:
			case ASURA_XBOX360_KEYID_PAD3_DOWN:
			case ASURA_XBOX360_KEYID_PAD4_DOWN:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sDIR DOWN", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_LEFT:
			case ASURA_XBOX360_KEYID_PAD2_LEFT:
			case ASURA_XBOX360_KEYID_PAD3_LEFT:
			case ASURA_XBOX360_KEYID_PAD4_LEFT:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sDIR LEFT", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_KEYID_PAD1_RIGHT:
			case ASURA_XBOX360_KEYID_PAD2_RIGHT:
			case ASURA_XBOX360_KEYID_PAD3_RIGHT:
			case ASURA_XBOX360_KEYID_PAD4_RIGHT:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sDIR RIGHT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_KEYID_VOID:
			default:
				Asura_PrintfToBfr( pxDescString->GetString(), "--", iStringLength, true );
				break;
		}
	}
	else
	{
		switch( iKeyID )
		{
			case ASURA_XBOX360_AXISID_PAD1_LSTICK_H:
			case ASURA_XBOX360_AXISID_PAD2_LSTICK_H:
			case ASURA_XBOX360_AXISID_PAD3_LSTICK_H:
			case ASURA_XBOX360_AXISID_PAD4_LSTICK_H:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sLS HOR", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_AXISID_PAD1_LSTICK_V:
			case ASURA_XBOX360_AXISID_PAD2_LSTICK_V:
			case ASURA_XBOX360_AXISID_PAD3_LSTICK_V:
			case ASURA_XBOX360_AXISID_PAD4_LSTICK_V:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sLS VERT", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_AXISID_PAD1_RSTICK_H:
			case ASURA_XBOX360_AXISID_PAD2_RSTICK_H:
			case ASURA_XBOX360_AXISID_PAD3_RSTICK_H:
			case ASURA_XBOX360_AXISID_PAD4_RSTICK_H:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sRS HOR", iStringLength, true, pxDescString->GetString() );
				break;
			
			case ASURA_XBOX360_AXISID_PAD1_RSTICK_V:
			case ASURA_XBOX360_AXISID_PAD2_RSTICK_V:
			case ASURA_XBOX360_AXISID_PAD3_RSTICK_V:
			case ASURA_XBOX360_AXISID_PAD4_RSTICK_V:
				Asura_PrintfToBfr( pxDescString->GetString(), "%sRS VERT", iStringLength, true, pxDescString->GetString() );
				break;

			case ASURA_AXIS_VOID:
			default:
				Asura_PrintfToBfr( pxDescString->GetString(), "--", iStringLength, true );
				break;
		}
	}

	return false;
}

/// End of file. ///
