//////////////////////////////////////////////////////////////////////////////
//
// AVP_Xbox360_External_Camera.cpp
//
// Created by Andrew McDonald - 09/11/07
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//		360-specific external camera functions.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Camera/AVP_External_Camera.h"

// Asura headers
#include "Input/Asura_ControlManager.h"
#include "Input/Asura_ControlMap.h"

// Asura Xbox 360
#include "Input/Asura_PlatformInput.h"

// Game headers
#include "Core/AVP_Game_Core.h"


// Debugvars
DEBUGVAR float dbg_fTiltRampUpTime = 0.25f;
DEBUGVAR float dbg_fTiltRampDownTime = 0.05f;
DEBUGVAR float dbg_fTiltExponent = 2.0f;

DEBUGVAR float dbg_fTurnRampUpTime = 0.5f;
DEBUGVAR float dbg_fTurnRampDownTime = 0.10f;

/////////////////////
// Input functions //
/////////////////////

void AVP_External_Camera::Platform_FillControlMap()
{
	if (!m_pxControlMap) { return; }

	// Just do pad 1

	Asura_ControlMap& xMap = *m_pxControlMap;

	xMap.AddAxisMapping(AVP_External_Camera::LOOK_UP_DOWN, ASURA_XBOX360_AXISID_PAD1_RSTICK_V, -1.0f, &dbg_fTiltRampUpTime, &dbg_fTiltRampDownTime, ASURA_CONTROLMAPPING_AXIS_MASK_ALL, &dbg_fTiltExponent);
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_LEFT_RIGHT, ASURA_XBOX360_AXISID_PAD1_RSTICK_H, 1.0f, &dbg_fTurnRampUpTime, &dbg_fTurnRampDownTime, ASURA_CONTROLMAPPING_AXIS_MASK_ALL, &dbg_fTiltExponent);

	xMap.AddAxisMapping(AVP_External_Camera::MOVE_FORWARDS_BACKWARDS, ASURA_XBOX360_AXISID_PAD1_LSTICK_V);
	xMap.AddAxisMapping(AVP_External_Camera::MOVE_LEFT_RIGHT, ASURA_XBOX360_AXISID_PAD1_LSTICK_H);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN, ASURA_XBOX360_KEYID_PAD1_R_TRIGGER, Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN, ASURA_XBOX360_KEYID_PAD1_L_TRIGGER, Asura_ControlMapping::ON_PRESS, 1.0f);

	xMap.AddKeyMapping(AVP_External_Camera::RESPAWN, ASURA_XBOX360_KEYID_PAD1_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_MENU, ASURA_XBOX360_KEYID_PAD1_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::SCOREBOARD, ASURA_XBOX360_KEYID_PAD1_BACK, Asura_ControlMapping::ON_PRESS);
}

#ifdef AVP_SECOND_CONTROL_FLY_CAM
void AVP_External_Camera::Platform_FillSecondControlMap()
{
	if (!m_pxControlMap) { return; }

	// Setting the controls for the second controler when we are using it as a flycam

	Asura_ControlMap& xMap = *m_pxControlMap;

	xMap.AddAxisMapping(AVP_External_Camera::LOOK_UP_DOWN, ASURA_XBOX360_AXISID_PAD2_RSTICK_V, -1.0f, &dbg_fTiltRampUpTime, &dbg_fTiltRampDownTime, ASURA_CONTROLMAPPING_AXIS_MASK_ALL, &dbg_fTiltExponent);
	xMap.AddAxisMapping(AVP_External_Camera::LOOK_LEFT_RIGHT, ASURA_XBOX360_AXISID_PAD2_RSTICK_H, 1.0f, &dbg_fTurnRampUpTime, &dbg_fTurnRampDownTime, ASURA_CONTROLMAPPING_AXIS_MASK_ALL, &dbg_fTiltExponent);

	xMap.AddAxisMapping(AVP_External_Camera::MOVE_FORWARDS_BACKWARDS, ASURA_XBOX360_AXISID_PAD2_LSTICK_V);
	xMap.AddAxisMapping(AVP_External_Camera::MOVE_LEFT_RIGHT, ASURA_XBOX360_AXISID_PAD2_LSTICK_H);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN, ASURA_XBOX360_KEYID_PAD2_R_TRIGGER, Asura_ControlMapping::ON_PRESS, -1.0f);
	xMap.AddKeyMapping(AVP_External_Camera::MOVE_UP_DOWN, ASURA_XBOX360_KEYID_PAD2_L_TRIGGER, Asura_ControlMapping::ON_PRESS, 1.0f);

	xMap.AddKeyMapping(AVP_External_Camera::SCREENSHOT, ASURA_XBOX360_KEYID_PAD2_START, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_FREEZE, ASURA_XBOX360_KEYID_PAD2_A, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_FLY_CAM, ASURA_XBOX360_KEYID_PAD2_B, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_RENDER_ALL, ASURA_XBOX360_KEYID_PAD2_X, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_TIME, ASURA_XBOX360_KEYID_PAD2_L_BUMPER, Asura_ControlMapping::ON_DEBOUNCED_PRESS, -0.1f);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_TIME, ASURA_XBOX360_KEYID_PAD2_R_BUMPER, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 0.1f);
	xMap.AddKeyMapping(AVP_External_Camera::TOGGLE_WIREFRAME, ASURA_XBOX360_KEYID_PAD2_Y, Asura_ControlMapping::ON_DEBOUNCED_PRESS);
}
#endif

/// End of file. ///
