//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_Profile.cpp
//
// (AJK - 25.07.2008) Created
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#include <xonline.h>

// header
#include "Core/Profile/AVP_Profile.h"

// Asura
#include "ChunkSystem/Asura_SaveBlock.h"


////////////////////////////////////////////////////////////////////////////////
// Constants...
////////////////////////////////////////////////////////////////////////////////

static const DWORD g_adwProfileSettings[] =
{
	XPROFILE_OPTION_CONTROLLER_VIBRATION,
	XPROFILE_GAMER_CONTROL_SENSITIVITY,
	XPROFILE_GAMER_ACTION_AUTO_AIM,
	XPROFILE_GAMER_ACTION_AUTO_CENTER,
	XPROFILE_GAMER_YAXIS_INVERSION,
	XPROFILE_GAMER_ACTION_MOVEMENT_CONTROL
};

static const DWORD	g_dwNumProfileSettings	= sizeof(g_adwProfileSettings) / sizeof(g_adwProfileSettings[0]);


////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Profile::Platform_SetToDefaults
// Author       : Andy Tate
// Date         : 24-Apr-2009
////////////////////////////////////////////////////////////////////////////////

void AVP_Profile::Platform_SetToDefaults(void)
{
#if 0
	DWORD	dwSignInStatus;

	do 
	{
		XUSER_SIGNIN_INFO	xSigninInfo;

		dwSignInStatus	= XUserGetSigninInfo(0, 0, &xSigninInfo);

		if(dwSignInStatus == ERROR_SUCCESS)
		{
			// We're already logged-in - find our user-name...
			m_xName	= xSigninInfo.szUserName;
		}
		else
		{
			dwSignInStatus	= XShowSigninUI(1, XSSUI_FLAGS_LOCALSIGNINONLY);
		}

	} while(dwSignInStatus != ERROR_SUCCESS);
#endif

	DWORD		dwBufferSize	= 0;
	const DWORD	dwSizeStatus	= XUserReadProfileSettings(0, 0, g_dwNumProfileSettings, g_adwProfileSettings, &dwBufferSize, NULL, NULL);

	// Did we succeed in finding out how large our data is?
	if(dwSizeStatus == ERROR_INSUFFICIENT_BUFFER)
	{
		XUSER_READ_PROFILE_SETTING_RESULT*	pxProfileSettingResults	= reinterpret_cast<XUSER_READ_PROFILE_SETTING_RESULT*>(new u_char[dwBufferSize]);
		const DWORD							dwSettingsStatus		= XUserReadProfileSettings(0, 0, g_dwNumProfileSettings, g_adwProfileSettings, &dwBufferSize, pxProfileSettingResults, NULL);

		// Did we succeed in fetching the appropriate profile data?
		if(dwSettingsStatus == ERROR_SUCCESS)
		{
			// Check each profile-setting in-turn...
			for(u_int u = 0; u < pxProfileSettingResults->dwSettingsLen; u++)
			{
				const XUSER_PROFILE_SETTING&	xProfileSetting	= pxProfileSettingResults->pSettings[u];
				const DWORD						dwSettingID		= xProfileSetting.dwSettingId;

				// What type of setting was this?
				switch(dwSettingID)
				{
					case XPROFILE_OPTION_CONTROLLER_VIBRATION:
					{
						const bool	bVibration	= (xProfileSetting.data.nData == XPROFILE_CONTROLLER_VIBRATION_ON);

						m_xControls.SetVibrate(bVibration);
						break;
					}

					case XPROFILE_GAMER_CONTROL_SENSITIVITY:
					{
						switch(xProfileSetting.data.nData)
						{
							case XPROFILE_CONTROL_SENSITIVITY_LOW:
							{
								m_xControls.SetSensitivityX(0.0f);
								m_xControls.SetSensitivityY(0.0f);
								break;
							}

							case XPROFILE_CONTROL_SENSITIVITY_MEDIUM:
							{
								m_xControls.SetSensitivityX(0.5f);
								m_xControls.SetSensitivityY(0.5f);
								break;
							}

							case XPROFILE_CONTROL_SENSITIVITY_HIGH:
							{
								m_xControls.SetSensitivityX(1.0f);
								m_xControls.SetSensitivityY(1.0f);
								break;
							}

							default:
							{
								Asura_Assert(false, "Unexpected control-sensitivity found.");
								break;
							}
						}		// End of switch.
						break;
					}

					case XPROFILE_GAMER_YAXIS_INVERSION:
					{
						const bool	bInvertY	= (xProfileSetting.data.nData == XPROFILE_YAXIS_INVERSION_ON);

						m_xControls.SetInvertY(bInvertY);
						break;
					}

					case XPROFILE_GAMER_ACTION_AUTO_AIM:
					{
						const bool	bAutoCentre	= (xProfileSetting.data.nData == XPROFILE_ACTION_AUTO_AIM_ON);

						m_xControls.SetAimAssist(bAutoCentre);
						break;
					}

					case XPROFILE_GAMER_ACTION_AUTO_CENTER:
					{
						const bool	bAutoCentre	= (xProfileSetting.data.nData == XPROFILE_ACTION_AUTO_CENTER_ON);

						m_xControls.SetAutoCenter(bAutoCentre);
						break;
					}

					case XPROFILE_GAMER_ACTION_MOVEMENT_CONTROL:
					{
#if 1
						// ART - 27/04/2009 - Need to allow this to choose between left and right-handed control schemes
						// which will be done via default control-map selection, there's currently no code for this so
						// currently no implementation...
#else
						const bool	bLeftThumbMovement	= (xProfileSetting.data.nData == XPROFILE_ACTION_MOVEMENT_CONTROL_L_THUMBSTICK);

						if(bLeftThumbMovement)
						{
							m_xControls.SetControlScheme(LEFT_STICK_DEFAULT_MOVEMENT);
						}
						else
						{
							m_xControls.SetControlScheme(RIGHT_STICK_DEFAULT_MOVEMENT);
						}
#endif
						break;
					}

					default:
					{
						Asura_Assert(false, "Unexpected profile setting found.");
						break;
					}
				}		// End of switch.
			}
		}
		else if (dwSettingsStatus == ERROR_FUNCTION_FAILED)
		{
			Asura_Assert(false, "Failed to get profile data - is anyone signed in?");
		}

		// Finally, tidy-up our dynamically allocated data...
		delete[](pxProfileSettingResults);
	}
}

/// End of function 'AVP_Profile::Platform_SetToDefaults'. ///


bool AVP_Profile::Platform_FillFilename(Asura_StringBuffer<128>& xFilename)
{
	// Add here filename + path

	// CAH Pseudo code
	/*find the 360 name of the player signed in
	if no one sigend in or fail make it "default"
	add this to a string. 
	add the name of the save we want for avp. 
	bolt on file extension*/

    return false;
}

bool AVP_Profile::Platform_SaveFile(Asura_SaveBlock& xSaveBlock)
{
	// Add here save file routine
	return false;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : AVP_Profile::Platform_Initialise
// Scope		: Private
// Author       : Callum Hope
// Date         : 5-Jan-2009
//
// Returns		: Bool, true if all was executed successfully, an assert should fire if returned false. 
// Desc			: Function to hold all of the platform dependant profile initialisaton code. 
//				  For the 360 this involves calling
////////////////////////////////////////////////////////////////////////////////

bool AVP_Profile::Platform_Initialise()
{
	// CAH pseudo code. 
	//call platform fill file name above to get the name of the save. 
	//call SetBaseFilename in Asura_xbox360_Profile_system with the filename as the argument. 
	//
	//Call BegineProfileFlow. - this checks the player is signed in and if not asks them to choose a profile. 

	return true;
}

