//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_DevelopmentFrontEnd.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "FrontEnd/AVP_DevelopmentFrontEnd.h"

// Asura independent
#include "Input/Asura_ControlMap.h"

// Asura Xbox 360
#include "Input/Asura_PlatformInput.h"

#if defined AVP_USE_DEVELOPMENT_FRONT_END

/******************************************************************************/
/* AVP_DevelopmentFrontEnd Static Class Data                              */
/******************************************************************************/

void AVP_DevelopmentFrontEnd::Platform_Initialise()
{
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_UP,     ASURA_XBOX360_KEYID_PAD1_UP,    Asura_ControlMapping::ON_DEBOUNCED_PRESS );
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_DOWN,   ASURA_XBOX360_KEYID_PAD1_DOWN,  Asura_ControlMapping::ON_DEBOUNCED_PRESS );
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_LEFT,   ASURA_XBOX360_KEYID_PAD1_LEFT,  Asura_ControlMapping::ON_DEBOUNCED_PRESS );
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_RIGHT,  ASURA_XBOX360_KEYID_PAD1_RIGHT, Asura_ControlMapping::ON_DEBOUNCED_PRESS );
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_SELECT, ASURA_XBOX360_KEYID_PAD1_A,     Asura_ControlMapping::ON_DEBOUNCED_PRESS );
	s_pxControlMap->AddKeyMapping( AVP_DFE_BUTTON_CANCEL, ASURA_XBOX360_KEYID_PAD1_B,     Asura_ControlMapping::ON_DEBOUNCED_PRESS );

	s_pxControlMap->AddTapAxisMapping( AVP_DFE_BUTTON_LEFT, ASURA_XBOX360_AXISID_PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f);
	s_pxControlMap->AddTapAxisMapping( AVP_DFE_BUTTON_RIGHT, ASURA_XBOX360_AXISID_PAD1_LSTICK_H, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f);
	s_pxControlMap->AddTapAxisMapping( AVP_DFE_BUTTON_UP, ASURA_XBOX360_AXISID_PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, 0.5f);
	s_pxControlMap->AddTapAxisMapping( AVP_DFE_BUTTON_DOWN, ASURA_XBOX360_AXISID_PAD1_LSTICK_V, 1, Asura_ControlMapping::ON_DEBOUNCED_PRESS, 1.0f, 0.1f, -0.5f);
}

void AVP_DevelopmentFrontEnd::Platform_Shutdown()
{
}

bool AVP_DevelopmentFrontEnd::Platform_NeedToBuildFMVList()
{
	return false;
}

void AVP_DevelopmentFrontEnd::Platform_RenderBuildFMVList()
{
}

void AVP_DevelopmentFrontEnd::Platform_StartBuildFMVList()
{
}

bool AVP_DevelopmentFrontEnd::Platform_UpdateBuildFMVList()
{
	return true;
}

void AVP_DevelopmentFrontEnd::Platform_ClearFMVList()
{
}

void AVP_DevelopmentFrontEnd::Platform_PreviewFMV()
{
}

const Asura_UnicodeChar* AVP_DevelopmentFrontEnd::Platform_GetFMVDescription()
{
	return L"Only works on PSP.";
}


int LevelSorterFunc( const Asura_String & xObjectOne, const Asura_String & xObjectTwo )
{
	return xObjectTwo.Compare(xObjectOne, true);

}
static void GetFilesInFolder(const Asura_String &xFolderName, Asura_List< Asura_String >& xFiles)
{
	Asura_String xSubDirs = xFolderName + "*";

	WIN32_FIND_DATA xData;
	HANDLE pHandle;
	if( (pHandle = FindFirstFile( xSubDirs.GetString(), &xData )) != INVALID_HANDLE_VALUE )
	{
		do
		{
			if (xData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
				GetFilesInFolder(xFolderName + xData.cFileName + "\\", xFiles);
			}
		} while( FindNextFile( pHandle, &xData ) );

		FindClose( pHandle );
	}

	Asura_String xLevelFiles = xFolderName + "\\*.xbox360";
	if( (pHandle = FindFirstFile( xLevelFiles.GetString(), &xData )) != INVALID_HANDLE_VALUE )
	{
		do
		{
			Asura_String* pxNewString = new Asura_String( xFolderName.GetString() + sizeof("Game:\\") - 1);
			*pxNewString += xData.cFileName;

			if( pxNewString )
			{
				xFiles.InsertInList( pxNewString, LevelSorterFunc );
			}
		} while( FindNextFile( pHandle, &xData ) );

		FindClose( pHandle );
	}
}

void AVP_DevelopmentFrontEnd::Platform_GetLevelFiles( Asura_List< Asura_String >& xFiles )
{
	GetFilesInFolder("Game:\\Envs\\", xFiles);
}

#endif // defined AVP_USE_DEVELOPMENT_FRONT_END