//////////////////////////////////////////////////////////////////////////////
//
// AVP_Platform_Font_System.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Rendering/AVP_Font_System.h"

#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_TiledTextureManagement.h"

#include "Core/AVP_Core.h"

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hash_ID AVP_Buttons::s_auSpecialCharTiles[AVP_FONTSYSTEM_NUM_SPECIAL_CHARS] =
{
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_NA,			0xE800	Asura_GetHashID("xbox360_NA.tga")	N/A

	0xD6D987BC,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_1,		0xE801	Asura_GetHashID("xbox360_a.tga")		A	
	0xD6E79F3D,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_2,		0xE802	Asura_GetHashID("xbox360_b.tga")		B
	0xD81DA453,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_3,		0xE803	Asura_GetHashID("xbox360_x.tga")		X
	0xD82BBBD4,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_4,		0xE804	Asura_GetHashID("xbox360_y.tga")		Y
	0x19D9E4BB,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_5,		0xE805	Asura_GetHashID("xbox360_lb.tga")		LB
	0x2416F875,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_6,		0xE806	Asura_GetHashID("xbox360_rb.tga")		RB
	0x1AD78BCD,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_7,		0xE807	Asura_GetHashID("xbox360_lt.tga")		LT
	0x25149F87,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_8,		0xE808	Asura_GetHashID("xbox360_rt.tga")		RT
	0x1AC9744C,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_9,		0xE809	Asura_GetHashID("xbox360_ls.tga")		LS
	0x25068806,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_10,		0xE80A	Asura_GetHashID("xbox360_rs.tga")		RS
	0xFA89E844,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_11,		0xE80B	Asura_GetHashID("xbox_360_start.tga")	START
	0xC455EF05,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_12,		0xE80C	Asura_GetHashID("xbox_360_back.tga")	BACK
	0xF4935B1F,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_13,		0xE80D	Asura_GetHashID("xbox_dpad_up.tga")		UP
	0x7E727CA6,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_14,		0xE80E	Asura_GetHashID("xbox_dpad_down.tga")	DOWN
	0x8FCFBD8B,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_15,		0xE80F	Asura_GetHashID("xbox_dpad_left.tga")	LEFT
	0xA76E2258,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_16,		0xE810	Asura_GetHashID("xbox_dpad_right.tga")	RIGHT

	// CAH	All of these N/A entries are buttons that the PC has but the 360 doesnt,
	// the enum to define all of the buttons if platform independant (and should stay that way for general ease) so they just have to entered and not used. 
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_17,		0xE811	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_18,		0xE812	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_19,		0xE813	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_20,		0xE814	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_21,		0xE815	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_22,		0xE816	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_23,		0xE817	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_24,		0xE818	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_25,		0xE819	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_26,		0xE81A	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_27,		0xE81B	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_28,		0xE81C	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_29,		0xE81D	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_30,		0xE81E	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_31,		0xE81F	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_32,		0xE820	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_33,		0xE821	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_34,		0xE822	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_35,		0xE823	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_36,		0xE824	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_37,		0xE825	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_38,		0xE826	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_39,		0xE827	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_40,		0xE828	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_41,		0xE829	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_42,		0xE82A	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_43,		0xE82B	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_44,		0xE82C	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_45,		0xE82D	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_46,		0xE82E	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_47,		0xE82F	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_48,		0xE830	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_49,		0xE831	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_50,		0xE832	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_51,		0xE833	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_52,		0xE834	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_53,		0xE835	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_54,		0xE836	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_55,		0xE837	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_56,		0xE838	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_57,		0xE839	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_58,		0xE83A	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_59,		0xE83B	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_60,		0xE83C	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_61,		0xE83D	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_62,		0xE83E	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_63,		0xE83F	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_64,		0xE840	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_65,		0xE841	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_66,		0xE842	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_67,		0xE843	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_68,		0xE844	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_69,		0xE845	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_70,		0xE846	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_71,		0xE847	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_72,		0xE848	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_73,		0xE849	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_74,		0xE84A	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_75,		0xE84B	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_76,		0xE84C	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_77,		0xE84D	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_78,		0xE84E	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_79,		0xE84F	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_80,		0xE850	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_81,		0xE851	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_82,		0xE852	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_83,		0xE853	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_84,		0xE854	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_85,		0xE855	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_86,		0xE856	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_87,		0xE857	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_88,		0xE858	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_89,		0xE859	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_90,		0xE85A	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_91,		0xE85B	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_92,		0xE85C	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_93,		0xE85D	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_94,		0xE85E	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_95,		0xE85F	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_96,		0xE860	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_97,		0xE861	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_98,		0xE862	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_99,		0xE863	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_100,	0xE864	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_101,	0xE865	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_102,	0xE866	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_103,	0xE867	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_104,	0xE868	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_105,	0xE869	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_106,	0xE86A	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_107,	0xE86B	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_108,	0xE86C	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_109,	0xE86D	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_110,	0xE86E	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_111,	0xE86F	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_112,	0xE870	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_113,	0xE871	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_114,	0xE872	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_115,	0xE873	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_116,	0xE874	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_117,	0xE875	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_118,	0xE876	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_119,	0xE877	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_120,	0xE878	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_121,	0xE879	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_122,	0xE87A	Asura_GetHashID("xbox360_NA.tga")	N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_BUTTON_123,	0xE87B	Asura_GetHashID("xbox360_NA.tga")	N/A

	0xB967A9C1,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_LSTICK_H,	0xE87C	Asura_GetHashID("xbox360_analogue_left.tga")	AXIS LS H
	0xB967A9C1,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_LSTICK_V,	0xE87D	Asura_GetHashID("xbox360_analogue_left.tga")	AXIS LS V
	0xB0D3BCE2,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_RSTICK_H,	0xE87E	Asura_GetHashID("xbox360_analogue_right.tga")	AXIS RS H
	0xB0D3BCE2,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_RSTICK_V,	0xE87F	Asura_GetHashID("xbox360_analogue_right.tga")	AXIS RS V
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_Z,		0xE880	Asura_GetHashID("xbox360_NA.tga") N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_X,	0xE881	Asura_GetHashID("xbox360_NA.tga") N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_Y,	0xE882	Asura_GetHashID("xbox360_NA.tga") N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_ROTATE_Z,	0xE883	Asura_GetHashID("xbox360_NA.tga") N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_MOUSE_H,	0xE884	Asura_GetHashID("xbox360_NA.tga") N/A
	0x1D357E78,	//AVP_FONTSYSTEM_SPECIALCHAR_AXIS_MOUSE_V,	0xE885	Asura_GetHashID("xbox360_NA.tga") N/A

};

void AVP_Buttons::Platform_Initialise()
{
	// if the special char tiles haven't been loaded, attempt to load them here
	if( !Asura_TiledTextureManagement::GetTileInfoByHash( AVP_Buttons::s_auSpecialCharTiles[0] ) )
	{
#ifdef ASURA_ASSERT
		bool bOK = 
#endif
		AVP_Core::LoadFile( "graphics\\xbox360_buttons." ASURA_STANDARD_FILE_EXTENSION, ASURA_TEXTURE_LOADED_PERMANENT );
		Asura_Assert(bOK, "failed to load button icons file");
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
