/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Includes must be this way round here
#include "..\..\..\..\..\Asura\Xbox360\Shaders\Shared.fxh"
#include "..\..\..\..\..\Asura\Xbox360\Shaders\MaterialResponse_Shared.fxh"

/******************************************************************************/
/* Parameters                                                                 */
/******************************************************************************/

const float4x4 g_xWorldViewProjectionMatrix;
const float4x4 g_xWorldViewMatrix;
const float4x4 g_xWorldMatrix;
const float4x3 g_axBoneMatrices[80];

const float3 g_xObjectSpaceCameraPosition;

const bool g_bUseDamageMaps;

static const float2 g_xDistortionScales = { 128, -128.0f };

const float4 g_fOffset;

const float3 g_xCloakFadeOut = float3(1.0f / 30.0f, 0.707f, 1.0f / (1.0f - 0.707f));
// x = Maximum Distance
// y = cos(Maximum angle)
// z = 1.0f / (1.0f - cos(Maximum angle))

const float2 g_fParams = float2(3.61f, 0.06f);
// x = Scale
// y = LightningScale
const float4 g_fLightningColour;
// w = Threshold

const float g_fTransitionValue;



const texture2D g_xSceneTexture;
const sampler2D g_xSceneSampler = sampler_state
{
	Texture   = <g_xSceneTexture>;
	AddressU  = CLAMP;
	AddressV  = CLAMP;
};

const texture2D g_xNoiseTexture;
const sampler2D g_xNoiseTextureSampler = sampler_state
{
	Texture   = <g_xNoiseTexture>;
	AddressU  = WRAP;
	AddressV  = WRAP;
};


/******************************************************************************/
/* Types                                                                      */
/******************************************************************************/

struct InputVertex
{
	float4 m_xPosition			: POSITION;
	float2 m_xUV				: TEXCOORD0;
	float2 m_xUV2				: TEXCOORD1;
	float  m_fDualTextureLerp   : TEXCOORD2;

	int4   m_aiBoneIndices		: BLENDINDICES;
	float4 m_afBoneWeights		: BLENDWEIGHT;

	float3 m_xNormal			: NORMAL;
	float3 m_xTangent			: TANGENT;
	float3 m_xBinormal			: BINORMAL;
};

struct OutputVertex_Deferred
{
	float4 m_xPosition              : POSITION;

	float3 m_xNormal                : NORMAL;
	float3 m_xTangent               : TANGENT;
	float3 m_xBinormal              : BINORMAL;

	float4 m_xUVs					: TEXCOORD0;
	float2 m_xBlends				: TEXCOORD1;

	float4 m_xCameraDirDistance     : TEXCOORD2; // Direction to camera, and distance from view plane in w
	float3 m_xTangentSpaceCameraDir : TEXCOORD3;
	
	float3 m_xObjectSpacePosition   : TEXCOORD4;
	
};

struct OutputVertex_Forward
{
	float4 m_xPosition              : POSITION;

	float3 m_xNormal                : NORMAL;


	float3 m_xTangent               : TANGENT;
	float3 m_xBinormal              : BINORMAL;

	float4 m_xUVs					: TEXCOORD0;
	float2 m_xBlends				: TEXCOORD1;

	float3 m_xTangentSpaceCameraDir : TEXCOORD2;

	float m_fDistortionIntensity    : TEXCOORD3; 
	float3 m_xObjectSpacePosition   : TEXCOORD4;
};


struct OutputVertex_Forward2
{
	float4 m_xPosition              : POSITION;

	float3 m_xNormal                : NORMAL;


	float3 m_xTangent               : TANGENT;
	float3 m_xBinormal              : BINORMAL;

	float4 m_xUVs					: TEXCOORD0;
	float2 m_xBlends				: TEXCOORD1;

	float3 m_xTangentSpaceCameraDir : TEXCOORD2;

	float m_fDistortionIntensity    : TEXCOORD3; 
};


/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

float4x3 GetBoneMatrix_Constants( const float fIndex )
{
	return g_axBoneMatrices[ fIndex ];
}

float4x3 GetBoneMatrix_VertexStream( const float fIndex )
{
	float4 xBoneMat0, xBoneMat1, xBoneMat2;

	// Bones stored in semantics POSITION1-3. Ignore the last row, it's always (0 0 0 1)

	asm
	{
		vfetch xBoneMat0, fIndex, position1, UseTextureCache=true
		vfetch xBoneMat1, fIndex, position2, UseTextureCache=true
		vfetch xBoneMat2, fIndex, position3, UseTextureCache=true
	};

	// Stored as 3x4 in the vertex stream
	return transpose( float3x4( xBoneMat0, xBoneMat1, xBoneMat2 ) );
}

OutputVertex_Deferred VShader_Deferred( const InputVertex xIn, uniform bool bNewRenderer )
{
	// Position and UVs
	float4   xObjectSpacePosition = float4(0,0,0,1); 
	float4x3 axBoneMatrix;
	float    fAlbedoBlend = 0;
	float	 fVertexOffset = 0.0f;
	
	OutputVertex_Deferred xOut;
	
	if ( g_bUseDamageMaps )
	{
		// the code section below and comment from Dave Reed

		// Decrementing loop counter doesn't seem to work, and
		// so when we finish we want the most heavily weighted bone - BoneWeights[0] - in 
		// axBoneMatrix, hence the '3-i' indexing
		[loop]
		for(int i=0; i<4; i++)
		{
			if( bNewRenderer )
			{
				axBoneMatrix = GetBoneMatrix_VertexStream( xIn.m_aiBoneIndices[3-i] );
			}
			else
			{
				axBoneMatrix = GetBoneMatrix_Constants( xIn.m_aiBoneIndices[3-i] );
			}

			// Get the damagemap blend value
			fAlbedoBlend += (axBoneMatrix[2][2]*xIn.m_afBoneWeights[3-i]);
			// Get the offset value
			fVertexOffset += (axBoneMatrix[2][1]*xIn.m_afBoneWeights[3-i]);

			// Recompute the matrix elements that we've stored per-bone-effects values in
			axBoneMatrix[2] = cross (axBoneMatrix[0], axBoneMatrix[1]);
			xObjectSpacePosition.xyz += mul( xIn.m_xPosition, axBoneMatrix) * xIn.m_afBoneWeights[3-i];
		}

		// Tangent, binormal & normal (just transformed to object space to match SH)
		xOut.m_xNormal	 = mul( xIn.m_xNormal,	axBoneMatrix );
		xOut.m_xTangent  = mul( xIn.m_xTangent,	axBoneMatrix );
		xOut.m_xBinormal = mul( xIn.m_xBinormal,	axBoneMatrix );
		
		// Damage map values are reversed with respect to normal mat resp lookups
		fAlbedoBlend = 1.0f - fAlbedoBlend;
	}
	else
	{
		fAlbedoBlend = xIn.m_fDualTextureLerp;
		
		// Decrementing loop counter doesn't seem to work, and
		// so when we finish we want the most heavily weighted bone - BoneWeights[0] - in 
		// axBoneMatrix, hence the '3-i' indexing
		[loop]
		for(int i=0; i<4; i++)
		{
			if( bNewRenderer )
			{
				axBoneMatrix = GetBoneMatrix_VertexStream( xIn.m_aiBoneIndices[3-i] );
			}
			else
			{
				axBoneMatrix = GetBoneMatrix_Constants( xIn.m_aiBoneIndices[3-i] );
			}

			// Get the offset value
			fVertexOffset += (axBoneMatrix[2][1]*xIn.m_afBoneWeights[3-i]);

			// Recompute the matrix elements that we've stored per-bone-effects values in
			axBoneMatrix[2] = cross (axBoneMatrix[0], axBoneMatrix[1]);
			xObjectSpacePosition.xyz += mul( xIn.m_xPosition, axBoneMatrix) * xIn.m_afBoneWeights[3-i];
		}

		// Tangent, binormal & normal (just transformed to object space to match SH)
		xOut.m_xNormal	 = mul( xIn.m_xNormal,	axBoneMatrix );
		xOut.m_xTangent  = mul( xIn.m_xTangent,	axBoneMatrix );
		xOut.m_xBinormal = mul( xIn.m_xBinormal,	axBoneMatrix );
	}
 
	xOut.m_xPosition = mul( xObjectSpacePosition, g_xWorldViewProjectionMatrix );;
	
	// damage maps use the first set of UV's twice	
	if (g_bUseDamageMaps)
	{
		xOut.m_xUVs = float4(xIn.m_xUV, xIn.m_xUV);
	} else {
		xOut.m_xUVs = float4(xIn.m_xUV, xIn.m_xUV2);
	}
	xOut.m_xBlends = float2(fAlbedoBlend, fVertexOffset);
	xOut.m_xObjectSpacePosition.xyz = xIn.m_xPosition;

	// Direction to camera, and world-space distance to vertex from viewer

	const float4 xWorldSpacePosition = mul( xObjectSpacePosition, g_xWorldMatrix );


	xOut.m_xCameraDirDistance.xyz = g_xCameraPosition.xyz - xWorldSpacePosition.xyz;
	xOut.m_xCameraDirDistance.w = xOut.m_xPosition.w;

	const float3x3 xTangentToWorldMatrix = { xOut.m_xTangent, xOut.m_xBinormal, xOut.m_xNormal };

	// Note implicit transpose in the mul, by swapping operand order
	xOut.m_xTangentSpaceCameraDir = mul( xTangentToWorldMatrix, xOut.m_xCameraDirDistance.xyz );

	// We're doing lighting in world-space now
	xOut.m_xNormal	 = mul( xOut.m_xNormal,	  g_xWorldMatrix );
	xOut.m_xTangent	 = mul( xOut.m_xTangent,  g_xWorldMatrix );
	xOut.m_xBinormal = mul( xOut.m_xBinormal, g_xWorldMatrix );

	return xOut;
}

OutputVertex_Forward VShader_Forward( const InputVertex xIn, uniform bool bNewRenderer )
{
	// Position and UVs
	float4   xObjectSpacePosition = float4(0,0,0,1); 
	float4x3 axBoneMatrix;
	float    fAlbedoBlend = 0;
	float	 fVertexOffset = 0.0f;
	
	OutputVertex_Forward xOut;
	
	if ( g_bUseDamageMaps )
	{
		// the code section below and comment from Dave Reed

		// Decrementing loop counter doesn't seem to work, and
		// so when we finish we want the most heavily weighted bone - BoneWeights[0] - in 
		// axBoneMatrix, hence the '3-i' indexing
		[loop]
		for(int i=0; i<4; i++)
		{
			if( bNewRenderer )
			{
				axBoneMatrix = GetBoneMatrix_VertexStream( xIn.m_aiBoneIndices[3-i] );
			}
			else
			{
				axBoneMatrix = GetBoneMatrix_Constants( xIn.m_aiBoneIndices[3-i] );
			}

			// Get the damagemap blend value
			fAlbedoBlend += (axBoneMatrix[2][2]*xIn.m_afBoneWeights[3-i]);
			// Get the offset value
			fVertexOffset += (axBoneMatrix[2][1]*xIn.m_afBoneWeights[3-i]);

			// Recompute the matrix elements that we've stored per-bone-effects values in
			axBoneMatrix[2] = cross (axBoneMatrix[0], axBoneMatrix[1]);
			xObjectSpacePosition.xyz += mul( xIn.m_xPosition, axBoneMatrix) * xIn.m_afBoneWeights[3-i];
		}

		// Tangent, binormal & normal (just transformed to object space to match SH)
		xOut.m_xNormal	= mul( xIn.m_xNormal,	axBoneMatrix );
		xOut.m_xTangent = mul( xIn.m_xTangent,	axBoneMatrix );
		xOut.m_xBinormal= mul( xIn.m_xBinormal,	axBoneMatrix );
		
		// Damage map values are reversed with respect to normal mat resp lookups
		fAlbedoBlend = 1.0f - fAlbedoBlend;
	}
	else
	{
		fAlbedoBlend = xIn.m_fDualTextureLerp;
			
		// Decrementing loop counter doesn't seem to work, and
		// so when we finish we want the most heavily weighted bone - BoneWeights[0] - in 
		// axBoneMatrix, hence the '3-i' indexing
		[loop]
		for(int i=0; i<4; i++)
		{
			if( bNewRenderer )
			{
				axBoneMatrix = GetBoneMatrix_VertexStream( xIn.m_aiBoneIndices[3-i] );
			}
			else
			{
				axBoneMatrix = GetBoneMatrix_Constants( xIn.m_aiBoneIndices[3-i] );
			}

			// Get the offset value
			fVertexOffset += (axBoneMatrix[2][1]*xIn.m_afBoneWeights[3-i]);

			// Recompute the matrix elements that we've stored per-bone-effects values in
			axBoneMatrix[2] = cross (axBoneMatrix[0], axBoneMatrix[1]);
			xObjectSpacePosition.xyz += mul( xIn.m_xPosition, axBoneMatrix) * xIn.m_afBoneWeights[3-i];
		}

		// Tangent, binormal & normal (just transformed to object space to match SH)
		xOut.m_xNormal	= mul( xIn.m_xNormal,	axBoneMatrix );
		xOut.m_xTangent = mul( xIn.m_xTangent,	axBoneMatrix );
		xOut.m_xBinormal= mul( xIn.m_xBinormal,	axBoneMatrix );
	}
 
	xOut.m_xPosition = mul( xObjectSpacePosition, g_xWorldViewProjectionMatrix );;
	
	// kick the ligthning effect slightly closer to the camera as Z-Bias is just
	// not working out for us due to having a different apparent effect at different
	// distance
	xOut.m_xPosition.z -= 0.001f / xOut.m_xPosition.w;

	// damage maps use the first set of UV's twice	
	if (g_bUseDamageMaps)
	{
		xOut.m_xUVs = float4(xIn.m_xUV, xIn.m_xUV);
	} else {
		xOut.m_xUVs = float4(xIn.m_xUV, xIn.m_xUV2);
	}
	xOut.m_xBlends = float2(fAlbedoBlend, fVertexOffset);
	xOut.m_xObjectSpacePosition.xyz = xIn.m_xPosition;

	// Direction to camera, and world-space distance to vertex from viewer

	const float4 xWorldSpacePosition = mul( xObjectSpacePosition, g_xWorldMatrix );

	float3 xCameraDirDistance = g_xObjectSpaceCameraPosition.xyz - xObjectSpacePosition.xyz;

	const float3x3 xTangentToObjectMatrix = float3x3( xOut.m_xTangent,
													  xOut.m_xBinormal,
													  xOut.m_xNormal );

	// Note implicit transpose in the mul, by swapping operand order
	xOut.m_xTangentSpaceCameraDir = mul( xTangentToObjectMatrix, xCameraDirDistance.xyz );


	xCameraDirDistance = normalize(mul(-xCameraDirDistance,  g_xWorldViewMatrix ));
	xOut.m_fDistortionIntensity = saturate((xCameraDirDistance.z - g_xCloakFadeOut.y) * g_xCloakFadeOut.z)  * 0.5f;
	xOut.m_fDistortionIntensity *= (1.0f - saturate(xOut.m_xPosition.w * g_xCloakFadeOut.x)) / xOut.m_xPosition.w;


	// We're doing lighting in world-space now
	xOut.m_xNormal	 = mul( xOut.m_xNormal,	  g_xWorldViewMatrix );
	xOut.m_xTangent	 = mul( xOut.m_xTangent,  g_xWorldViewMatrix );
	xOut.m_xBinormal = mul( xOut.m_xBinormal, g_xWorldViewMatrix );


	return xOut;
}


half3 GetDistortionValue(float3 xNormal, half2 xScreenPos : VPOS, float fDistortionIntensity )
{
	const half3 xViewSpaceNormal = xNormal;
  
	half hDistortionAmount = length( xViewSpaceNormal.xy );
	hDistortionAmount = fmod( hDistortionAmount, 0.2h ) * 5.0h;

	const half2 xDistortion = xViewSpaceNormal.xy * hDistortionAmount;
	xScreenPos += xDistortion * g_xDistortionScales * fDistortionIntensity;

	half4 xSceneSample;
	asm
	{
		tfetch2D xSceneSample, xScreenPos, g_xSceneSampler, MinFilter=point, MagFilter=point, MipFilter=point, UnnormalizedTextureCoords=true, OffsetX=0.5, OffsetY=0.5
	};
	return xSceneSample.rgb;

}

float NoiseLookup(float3 xPos)
{
	float fNoise = TexFetch2D_LinearLinearLinear(g_xNoiseTextureSampler, xPos.xy).r
		+ TexFetch2D_LinearLinearLinear(g_xNoiseTextureSampler, xPos.yz).g
		+ TexFetch2D_LinearLinearLinear(g_xNoiseTextureSampler, xPos.zx).b;

	return fNoise * 0.333f * 2.0f - 1.0f;
}

float CalculateNoise(float3 p, float fOffset)
{
	return NoiseLookup(p * g_fParams.x + g_fOffset) + fOffset;
}

[sampreg( 0, 17 )]
DeferredOutputPixel PShader_Deferred( const OutputVertex_Deferred xIn, uniform bool bUseResponse )
{
	const half3 xInterpolatedTangent  = normalize( xIn.m_xTangent );
	const half3 xInterpolatedBinormal = normalize( xIn.m_xBinormal );
	const half3 xInterpolatedNormal   = normalize( xIn.m_xNormal );

	const half3x3 xTangentToWorldMatrix
		= { xInterpolatedTangent, xInterpolatedBinormal, xInterpolatedNormal };

	const half3 xViewDir = normalize( xIn.m_xCameraDirDistance.xyz );
	const half3 xTangentSpaceViewDir = normalize( xIn.m_xTangentSpaceCameraDir );

	const half hDualTextureLerp = xIn.m_xBlends.x;

	half4 xUVPair = xIn.m_xUVs;
	xUVPair = MaterialResponse_AnimateUVs( xUVPair );
	ApplyParallaxMapping( xTangentSpaceViewDir, hDualTextureLerp, xUVPair );

	const half4 xWorldSpaceNormal = NormalLookup( xUVPair, hDualTextureLerp, xTangentToWorldMatrix, xTangentToWorldMatrix );

	half3 xAlbedo;
	half hSpecular;
	float fMaterialsTextureY;
	half4 xColour;

	if( bUseResponse )
	{
		const half4 xSHLookupDir = half4( xInterpolatedNormal, 1.0h );
		const float3 xLightColour
			= SHLookup( xSHLookupDir, g_xRedSH, g_xGreenSH, g_xBlueSH ) * 2.0f;

		const float fLightStrength = GetLuminance( xLightColour );

		xColour = MaterialResponse( xUVPair, hDualTextureLerp, xWorldSpaceNormal, xViewDir,
			xInterpolatedNormal, xLightColour, xLightColour, fLightStrength, true, xAlbedo,
			hSpecular, fMaterialsTextureY );

		xColour.rgb += SelfIlluminationLookup( xUVPair, hDualTextureLerp, xAlbedo );
	}
	else
	{
		fMaterialsTextureY = g_xMaterialProperties.x;

		// For this base geometry pass, we'll just use the SH lighting + self illumination

		const float3 xAlbedoTimes2 = AlbedoLookup( xUVPair,
												   hDualTextureLerp,
												   xWorldSpaceNormal,
												   xViewDir,
												   xAlbedo );

		xColour.rgb =
			SHLookup( xWorldSpaceNormal, g_xRedSH, g_xGreenSH, g_xBlueSH ) * xAlbedoTimes2
			+ SelfIlluminationLookup( xUVPair, hDualTextureLerp, xAlbedo );

		// Transparent env strips aren't sorted - but we need to use the transparency maps if present
		// to allow alpha decals
		xColour.a = TransparencyLookup( xUVPair, hDualTextureLerp );

		hSpecular = SpecularLookup( xUVPair, hDualTextureLerp );
	}

	// Do 1-bit alpha testing like this, since it seems to be done for deferred objects this way
	clip( xColour.a - 0.05h );

	DeferredOutputPixel xOut = Deferred_FillOutputPixel(
										xColour,
										xWorldSpaceNormal.xyz,
										xIn.m_xCameraDirDistance.w,
										xAlbedo,
										hSpecular,
										fMaterialsTextureY
										);
	
	float fNoise = CalculateNoise(xIn.m_xObjectSpacePosition, g_fTransitionValue);
	clip(-fNoise);
																
	return( xOut);
}

float4 SampleTextureBias( const sampler2D xSampler, const float2 xUV )
{
	float4 xResult;
	asm
	{
		tfetch2D xResult, xUV, xSampler, MinFilter=linear, MagFilter=linear, MipFilter=linear, AnisoFilter=disabled, LODBias=0.5f
	};

	return xResult;
}


half4 NormalLookupBias( const float4 xUVPair, const half hLerpValue, const half3x3 xTangentToWorldMatrix, const half3x3 xTangentToWorldMatrix2 )
{
	if( g_bUseNormalMap )
	{
		half4 xNormal;
		xNormal.xy = SampleTextureBias( g_xNormalMapSampler, xUVPair.xy ).rg;
		xNormal.xy = xNormal.xy * 2.0h - 1.0h;

		if( g_bUseNormalDetailMap )
		{
			const half2 xDetailUV = xUVPair.xy * g_xDetailMapParameters.z;
			half2 xDetailNormal = SampleTextureBias( g_xNormalDetailMapSampler, xDetailUV ).rg;
			xDetailNormal = xDetailNormal * 2.0h - 1.0h;
			xNormal.xy += xDetailNormal * g_xDetailMapParameters.w;
		}
		// Our normal maps are output upside-down. TODO: do this in a preprocessing step
		xNormal.y = -xNormal.y;
		xNormal.z = sqrt( 1.0h - dot( xNormal.xy, xNormal.xy ) );
		xNormal.xyz = mul( xNormal.xyz, xTangentToWorldMatrix );

		if( g_bDualNormalMap )
		{
			half3 xNormal2 = SampleTextureBias( g_xNormalMap2Sampler, xUVPair.zw ).rgg;
			xNormal2 = xNormal2 * 2.0h - 1.0h;

			if( g_bDualNormalDetailMap )
			{
				const half2 xDetailUV = xUVPair.zw * g_xDetailMap2Parameters.z;
				half2 xDetailNormal = SampleTextureBias( g_xNormalDetailMap2Sampler, xDetailUV ).rg;
				xDetailNormal = xDetailNormal * 2.0h - 1.0h;
				xNormal2.xy += xDetailNormal * g_xDetailMap2Parameters.w;

			}
			xNormal2.y = -xNormal2.y;
			xNormal2.z = sqrt( 1.0h - dot( xNormal2.xy, xNormal2.xy ) );
			xNormal2.xyz = mul( xNormal2.xyz, xTangentToWorldMatrix2 );

			xNormal.xyz = lerp( xNormal2.xyz, xNormal.xyz, hLerpValue );
		}

		xNormal.xyz = normalize(xNormal.xyz);
		xNormal.w = 1.0h;

		return xNormal;
	}
	else
	{
		return half4( xTangentToWorldMatrix[2], 1.0h );
	}
}


half4 PShader_Forward( const OutputVertex_Forward xIn, half2 xScreenPos : VPOS) : COLOR
{
	xScreenPos += g_xVPOSOffsetVector;

	const half3 xInterpolatedTangent  = normalize( xIn.m_xTangent );
	const half3 xInterpolatedBinormal = normalize( xIn.m_xBinormal );
	const half3 xInterpolatedNormal   = normalize( xIn.m_xNormal );

	const half3x3 xTangentToWorldMatrix
		= { xInterpolatedTangent, xInterpolatedBinormal, xInterpolatedNormal };

	const half3 xTangentSpaceViewDir = normalize( xIn.m_xTangentSpaceCameraDir );

	const half hDualTextureLerp = xIn.m_xBlends.x;

	half4 xUVPair = xIn.m_xUVs;
	xUVPair = MaterialResponse_AnimateUVs( xUVPair );
	ApplyParallaxMapping( xTangentSpaceViewDir, hDualTextureLerp, xUVPair );

	const half4 xWorldSpaceNormal = NormalLookupBias( xUVPair, hDualTextureLerp, xTangentToWorldMatrix, xTangentToWorldMatrix );


	half4 hScene;
	float fDistortionIntensity = saturate(xIn.m_fDistortionIntensity);
	half3 hDistortion = GetDistortionValue(xWorldSpaceNormal.xyz, xScreenPos, fDistortionIntensity);
	
	float fNoise = CalculateNoise(xIn.m_xObjectSpacePosition.xyz, g_fTransitionValue);
	half4 h4Distortion = float4(hDistortion, sign(fNoise) * 0.5f + 0.5f);
	half4 h4Lightning = float4(g_fLightningColour.rgb, 1.0f);
	float fLightningAmount = pow(1.0f - min(abs(fNoise) * g_fParams.y, 1.0f), 5);

	hScene = lerp(h4Distortion, h4Lightning, fLightningAmount * (1.0f - pow(1.0f - fDistortionIntensity, 3.0f)));
	hScene.a *= g_fLightningColour.w;

	return hScene;
}

half4 PShader_Forward2( const OutputVertex_Forward2 xIn, half2 xScreenPos : VPOS) : COLOR
{
	xScreenPos += g_xVPOSOffsetVector;

	const half3 xInterpolatedTangent  = normalize( xIn.m_xTangent );
	const half3 xInterpolatedBinormal = normalize( xIn.m_xBinormal );
	const half3 xInterpolatedNormal   = normalize( xIn.m_xNormal );

	const half3x3 xTangentToWorldMatrix
		= { xInterpolatedTangent, xInterpolatedBinormal, xInterpolatedNormal };

	const half3 xTangentSpaceViewDir = normalize( xIn.m_xTangentSpaceCameraDir );

	const half hDualTextureLerp = xIn.m_xBlends.x;

	half4 xUVPair = xIn.m_xUVs;
	xUVPair = MaterialResponse_AnimateUVs( xUVPair );
	ApplyParallaxMapping( xTangentSpaceViewDir, hDualTextureLerp, xUVPair );

	const half4 xWorldSpaceNormal = NormalLookupBias( xUVPair, hDualTextureLerp, xTangentToWorldMatrix, xTangentToWorldMatrix );


	half4 hScene;
	float fDistortionIntensity = saturate(xIn.m_fDistortionIntensity);
	half3 hDistortion = GetDistortionValue(xWorldSpaceNormal.xyz, xScreenPos, fDistortionIntensity);
	
	hScene = float4(hDistortion, g_fLightningColour.w);

	return hScene;
}


/******************************************************************************/
/* Techniques                                                                 */
/******************************************************************************/

// Perform material response using the SH
technique Deferred_MR
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_Deferred( false );
		PixelShader = compile ps_3_0 PShader_Deferred( true );
	}
}

// Forward renderer, for transparent strips
technique Forward
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_Forward( false );
		PixelShader = compile ps_3_0 PShader_Forward();
	}
}

// Forward renderer, Assume fully transparent (no transition lightning)
technique Forward_Cloaked
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_Forward( false );
		PixelShader = compile ps_3_0 PShader_Forward2();
	}
}

// These will go when the new skin renderer is fully switched on, we'll always pull the bones from
// the secondary vertex stream

// Perform material response using the SH
technique Deferred_MR_NewRenderer
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_Deferred( true );
		PixelShader = compile ps_3_0 PShader_Deferred( true );
	}
}

// Forward renderer, for transparent strips
technique Forward_NewRenderer
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_Forward( true );
		PixelShader = compile ps_3_0 PShader_Forward();
	}
}

// Forward renderer, Assume fully transparent (no transition lightning)
technique Forward_Cloaked_NewRenderer
{
	pass
	{
		VertexShader = compile vs_3_0 VShader_Forward( true );
		PixelShader = compile ps_3_0 PShader_Forward2();
	}
}
