/**
*
* AVP_Platform_GlowOutlines.cpp
*
* Created by Andrew Southgate - 30/7/2008
*
* Copyright (C) 2008 Rebellion, All Rights Reserved.
*
**/

/************/
/* Includes */
/************/

// main asura header
#include "Core/Asura.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_TextureManagement.h"

// local header
#include "SpecialFX/AVP_GlowOutlines.h"

// Asura Platform-independent
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Maths.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Colour.h"
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_MaterialResponse_system.h"
#include "Interface/Asura_CommandConsole.h"

//Asura Platform
#include "Rendering/Asura_Xbox360_Graphics.h"
#include "Rendering/Asura_Xbox360_CharacterRenderer.h"
#include "Rendering/Asura_Xbox360_SkinRenderer.h"

// AVP Platform
#include "Rendering/AVP_Platform_Stencil_Setup.h"

// Shader definition
#include "AVP_GlowOutlines.h"

#include <xgraphics.h>


static Direct3D_VertexDeclaration*	g_pxVertexDeclFS			= 0;


namespace GlowEffect {
	Asura_Xbox360_Effect*					g_pxEffect	= NULL;

	Asura_Xbox360_Effect_System::Handle		g_xWorldViewProjectionMatrixHandle;
	Asura_Xbox360_Effect_System::Handle		g_xBoneMatricesHandle;
	Asura_Xbox360_Effect_System::Handle		g_xBlurColour;
	Asura_Xbox360_Effect_System::Handle		g_xBlurParams;
	Asura_Xbox360_Effect_System::Handle		g_xSceneTextureHandle;
	Asura_Xbox360_Effect_System::Handle		g_xDepthTextureHandle;
	
	void CaptureEffect(void);


	void Initialise()
	{
		CaptureEffect();
	}

	void CaptureEffect(void)
	{
#define LOADPARAMETER(codeName, shaderName) { codeName = g_pxEffect->GetParameterHandle(shaderName); Asura_Assert(codeName, "Could not get shader parameter '" shaderName "'."); }
		g_pxEffect = Asura_Xbox360_Effect_System::CreateEffectFromMemory(g_auAVP_GlowOutlines);

		Asura_Assert( g_pxEffect, "Failed to load Glow Outlines effect." );
		if (g_pxEffect)
		{	
			LOADPARAMETER(g_xWorldViewProjectionMatrixHandle, "g_xWorldViewProjectionMatrix");
			LOADPARAMETER(g_xBoneMatricesHandle, "g_axBoneMatrices");
			LOADPARAMETER(g_xBlurColour, "g_xBlurColour");
			LOADPARAMETER(g_xBlurParams, "g_xBlurParams");
			LOADPARAMETER(g_xSceneTextureHandle, "g_xSceneTextureSampler");
			LOADPARAMETER(g_xDepthTextureHandle, "g_xDepthTextureSampler");
		}
	}
};

struct AVP_FS_Vertex
{
	float x, y, z, w;
	float x2, y2;
	void SetPosition(const Asura_Vector_3 &xPosition)
	{
		x = xPosition.GetX();
		y = xPosition.GetY();
		z = xPosition.GetZ();
		w = 1.0f;
	}
	void SetPos2(float fx, float fy)
	{
		x2 = fx, y2 = fy;
	}
	void Set(const Asura_Vector_3 &xPosition, float fx, float fy)
	{
		SetPosition(xPosition);
		SetPos2(fx,fy);
	}
};



static Direct3D_Texture *g_pxGlowRenderTargetTexture0;
static Direct3D_Texture g_xGlowRenderTargetTexture1;
static Direct3D_Surface *g_pxGlowRenderTarget[2];

static bool PreReset(void*)
{
	SAFE_RELEASE(g_pxGlowRenderTargetTexture0);

	SAFE_RELEASE(g_pxGlowRenderTarget[0]);
	SAFE_RELEASE(g_pxGlowRenderTarget[1]);
	return true;
}

static bool PostReset(void*)
{
	const D3DFORMAT eCOLOUR_TILE_FORMAT   = D3DFMT_A2B10G10R10F_EDRAM;
	const D3DFORMAT eDEPTH_BUFFER_FORMAT       = D3DFMT_D24S8;

	// Full-screen final render target
	const u_int uFINAL_RT_SIZE = XGSurfaceSize( Asura_Xbox360_Graphics::GetScreenWidth(), Asura_Xbox360_Graphics::GetScreenHeight(),
		eCOLOUR_TILE_FORMAT, D3DMULTISAMPLE_NONE );
 	const u_int uFINAL_DEPTH_BUFFER_SIZE = XGSurfaceSize( Asura_Xbox360_Graphics::GetScreenWidth(), Asura_Xbox360_Graphics::GetScreenHeight(),
 		eDEPTH_BUFFER_FORMAT, D3DMULTISAMPLE_NONE );

	D3DSURFACE_PARAMETERS xSurfaceParams;
	xSurfaceParams.Base = uFINAL_RT_SIZE + uFINAL_DEPTH_BUFFER_SIZE;
	xSurfaceParams.HierarchicalZBase = 0;
	xSurfaceParams.ColorExpBias = 0;
	xSurfaceParams.HiZFunc = D3DHIZFUNC_DEFAULT;

	const u_int BUFFER_SIZE = XGSurfaceSize( Asura_Viewport::GetWidth() /2, Asura_Viewport::GetHeight() / 2,
		D3DFMT_A16B16G16R16F, D3DMULTISAMPLE_NONE );

#if defined ASURA_ASSERT
	HRESULT xResult;
#endif

#if defined ASURA_ASSERT
	xResult =
#endif
	Asura_Xbox360_Graphics::GetDevice()->CreateTexture(Asura_Viewport::GetWidth() /2, Asura_Viewport::GetHeight() / 2, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A16B16G16R16F, D3DPOOL_DEFAULT, &g_pxGlowRenderTargetTexture0, NULL);
	Asura_Assertf(g_pxGlowRenderTargetTexture0, ("Failed to create pheromone glow render texture. Error code 0x%X.", xResult));

	// this can share the same memory as the other one
	XGSetTextureHeader( Asura_Viewport::GetWidth() /2, Asura_Viewport::GetHeight() / 2, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8,
		D3DPOOL_DEFAULT, 0, 0, 0, &g_xGlowRenderTargetTexture1, NULL, NULL );
	g_xGlowRenderTargetTexture1.Format.BaseAddress = g_pxGlowRenderTargetTexture0->Format.BaseAddress;



	Asura_Xbox360_Graphics::GetDevice()->CreateRenderTarget( Asura_Viewport::GetWidth() /2, Asura_Viewport::GetHeight() / 2, D3DFMT_A16B16G16R16F, D3DMULTISAMPLE_NONE, 0, FALSE, &g_pxGlowRenderTarget[0], &xSurfaceParams );
	xSurfaceParams.Base += BUFFER_SIZE;
	Asura_Xbox360_Graphics::GetDevice()->CreateRenderTarget( Asura_Viewport::GetWidth() /2, Asura_Viewport::GetHeight() / 2, D3DFMT_A8R8G8B8, D3DMULTISAMPLE_NONE, 0, FALSE, &g_pxGlowRenderTarget[1], &xSurfaceParams );

	return true;
}

void AVP_GlowOutlines::Platform_Initialise()
{
	Direct3D_VertexElement auVSDFS[] =
	{
		{	0,	0,	D3DDECLTYPE_FLOAT4,		D3DDECLMETHOD_DEFAULT,	D3DDECLUSAGE_POSITION,	0 },		
		D3DDECL_END()
	};

	Asura_Xbox360_Graphics::GetDevice()->CreateVertexDeclaration( auVSDFS, &g_pxVertexDeclFS );

	GlowEffect::Initialise();


	PostReset(NULL);

}

void AVP_GlowOutlines::Platform_Shutdown()
{
	SAFE_RELEASE( g_pxVertexDeclFS);


	if (GlowEffect::g_pxEffect)
	{
		Asura_Xbox360_Effect_System::Destroy( GlowEffect::g_pxEffect ); 
		GlowEffect::g_pxEffect = NULL;
	}
	PreReset(NULL);
}




#if !defined(ASURA_HIERARCHY_QUAD)
#error AVP requires ASURA_HIERARCHY_QUAD at the moment
#endif


static void FixUpAsura_Hierarchy_SystemForAnimation( Asura_Animation* pxAnim, const Asura_Vector_3& xPos, const Asura_Matrix_3x3& xOri, u_int uRenderFlags )
{
	Asura_Hierarchy_System::s_pxSkin		= pxAnim->GetSkin(0);
	Asura_Hierarchy_System::s_uSkinIndex	= 0;
	if( !Asura_Hierarchy_System::s_pxSkin ) { return; }

	Asura_Hierarchy_System::s_pxTweenStorage = pxAnim->GetTweenStore();
	if(!Asura_Hierarchy_System::s_pxTweenStorage) return;

#ifdef ASURA_HIERARCHY_QUAD
	// RJT TODO: Ultimately this transform should be passed in
	Asura_Hierarchy_System::s_xBaseTransform = Asura_AMatrix4x4(xOri, xPos);
	Asura_Hierarchy_System::s_xBaseTransform.InvertOrthogonal(Asura_Hierarchy_System::s_xInverseBaseTransform);
#endif

	Asura_Assert( Asura_Hierarchy_System::s_pxSkin->m_pxSectionBoundingBoxData, "" );

	Asura_Hierarchy_System::BonesToWorldspaceFromTweenStorage();
	pxAnim->UpdateLighting(uRenderFlags);

	if( Asura_Hierarchy_System::s_bBonesInWorldspace )
	{
		Asura_Hierarchy_System::s_xInverseBaseTransform.ApplyToArray( Asura_Hierarchy_System::s_axBoneTransforms, Asura_Hierarchy_System::s_pxSkin->NumberOfBones, Asura_Hierarchy_System::s_axBoneTransforms);
	}

	for( int i = 0; i < Asura_Hierarchy_System::s_pxSkin->NumberOfBones; ++i )
	{
		Asura_Hierarchy_System::s_axBoneTransforms[i].Apply(Asura_Hierarchy_System::s_pxSkin->m_pxBoneRestPoseInverseTransforms[i], Asura_Hierarchy_System::s_axBoneTransforms[i]);
	}
}

static void RenderSkin( const Asura_Animation& xAnimation, const Asura_Hierarchy_Skin& xSkin, Asura_AMatrix4x4_InParam xTransform,
					   Asura_AMatrix4x4_InParam xInverseTransform, const Asura_AMatrix4x4* const pxBoneTransforms)

{
	SCOPEDPIXEVENT(( 0xFF884499, "Smooth skin '%s' (AVP Glow Outlines)", xSkin.Name.GetString() ));

	const Asura_Xbox360_Skin* pxNewSkin = NULL;
	Asura_Xbox360_CharacterRenderer_Skin* pxOldSkin = NULL;
	
	if( Asura_Xbox360_SkinRenderer::IsEnabled() )
	{
		pxNewSkin = Asura_Xbox360_SkinRenderer::GetSkin( xSkin.m_uNameHashID );
	}
	else
	{
		pxOldSkin = Asura_Xbox360_CharacterRenderer::GetCharacter( xSkin.m_uNameHashID );
	}

	if( (pxOldSkin || pxNewSkin) && xSkin.NumberOfBones) 
	{
		Asura_Xbox360_Effect* pxEffect = GlowEffect::g_pxEffect;

		const u_int uTechnique = (Asura_Xbox360_SkinRenderer::IsEnabled() ? 2 : 0);
		pxEffect->Begin( uTechnique );

		if( pxEffect) 
		{
			{	// Bind vertex data
				SCOPEDPIXEVENT(( 0xFF884499, "Bind vertex data" ));

				if( pxOldSkin )
				{
					Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration(
						Asura_Xbox360_CharacterRenderer::GetVertexDeclaration() );

					pxOldSkin->GetIB().Bind();
					pxOldSkin->GetVB().Bind();
				}
				else
				{
					Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration(
						 Asura_Xbox360_SkinRenderer::GetVertexDeclaration() );

					pxNewSkin->GetIB().Bind();
					pxNewSkin->GetVB().Bind();

					Asura_Xbox360_SkinRenderer::CopyBones( pxBoneTransforms, xSkin.NumberOfBones, xAnimation );
				}
			}

			{	// Set world and world-view-proj matrices
				SCOPEDPIXEVENT(( 0xFF884499, "Set world-view-projection matrix" ));

				D3DXMATRIX xWorldViewProj;
				xTransform.StoreColsUnaligned(&xWorldViewProj._11);

				// View-projection
				xWorldViewProj *= Asura_Xbox360_Graphics::GetViewProjectionMatrix();
				pxEffect->SetMatrix( GlowEffect::g_xWorldViewProjectionMatrixHandle, xWorldViewProj );
			}

			D3DXMATRIX axBoneMatrices[ ASURA_MAX_NO_OF_BONES_PER_MATERIAL ];

			// Set bone matrices

			const u_int uNumBoneMaps = (pxOldSkin ? pxOldSkin->GetNumBoneMaps() : 0);

			if (uNumBoneMaps == 0 && !Asura_Xbox360_SkinRenderer::IsEnabled())
			{
				SCOPEDPIXEVENT(( 0xFF884499, "Set bone matrices (No Bone Maps)" ));

				for( int i = 0; i < xSkin.NumberOfBones; ++i )
				{
					pxBoneTransforms[i].StoreColsUnaligned(&axBoneMatrices[i]._11);
				}
				pxEffect->GetInterface()->SetMatrixArrayF4x4( GlowEffect::g_xBoneMatricesHandle, axBoneMatrices[0], xSkin.NumberOfBones );
			} 
			else if (uNumBoneMaps == 1 && !Asura_Xbox360_SkinRenderer::IsEnabled())
			{
				SCOPEDPIXEVENT(( 0xFF884499, "Set bone matrices (One Bone Maps)" ));

				const Asura_Xbox360_CharacterRenderer_Skin::BoneMap &xBoneMap = pxOldSkin->GetBoneMap(0);

				for( u_int i = 0; i < xBoneMap.m_uSize; ++i )
				{
					pxBoneTransforms[xBoneMap.m_puEntries[i]].StoreColsUnaligned(&axBoneMatrices[i]._11);
				}
				Asura_Mem::Set( axBoneMatrices + xBoneMap.m_uSize, 0, (ASURA_MAX_NO_OF_BONES_PER_MATERIAL - xBoneMap.m_uSize) * sizeof(D3DXMATRIX));

				pxEffect->GetInterface()->SetMatrixArrayF4x4( GlowEffect::g_xBoneMatricesHandle, axBoneMatrices[0], xBoneMap.m_uSize );
			}

			const u_int uNumStrips =
				(pxOldSkin ? pxOldSkin->GetNumStrips() : pxNewSkin->GetNumStrips());

			for( u_int u = 0, uStartIndex = 0; u < uNumStrips; ++u )
			{
				const Asura_Xbox360_CharacterRenderer_Strip* pxOldStrip =
					(pxOldSkin ? &pxOldSkin->GetStrip(u) : NULL);

				const Asura_Xbox360_SkinStrip* pxNewStrip =
					(pxNewSkin ? &pxNewSkin->GetStrip(u) : NULL);

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
				const int iSkinElement =
					(pxOldStrip ? pxOldStrip->m_iElementID : pxNewStrip->m_iElementID);

				if (xAnimation.GetSkinElementVisibility(iSkinElement))
#endif
				{
					// Set bone matrices
					if (uNumBoneMaps > 1 && !Asura_Xbox360_SkinRenderer::IsEnabled())
					{
						SCOPEDPIXEVENT(( 0xFF884499, "Set bone matrices (Many Bone Maps)" ));

						const Asura_Xbox360_CharacterRenderer_Skin::BoneMap &xBoneMap = pxOldSkin->GetBoneMap(pxOldStrip->m_uBoneMap);

						for( u_int i = 0; i < xBoneMap.m_uSize; ++i )
						{
							pxBoneTransforms[xBoneMap.m_puEntries[i]].StoreColsUnaligned(&axBoneMatrices[i]._11);
						}

						pxEffect->GetInterface()->SetMatrixArrayF4x4( GlowEffect::g_xBoneMatricesHandle, axBoneMatrices[0], xBoneMap.m_uSize );
					}

					SCOPEDPIXEVENT(( 0xFF884499, "Strip %u", u ));

					const Asura_Hash_ID uMaterialHash1 = pxOldStrip
						? pxOldStrip->m_auMaterialResponseHashIDs[0]
						: pxNewStrip->m_auMaterials[0];

					const Asura_MaterialResponse* pxMaterial1 = Asura_MaterialResponse_System::GetMaterialByHashID( uMaterialHash1 );

					if( !pxMaterial1 || !pxMaterial1->GetDontRender() )
					{
						pxEffect->Commit();

						const u_int uNumIndices =
							(pxOldStrip ? pxOldStrip->m_uNumIndices : pxNewStrip->m_uNumIndices);

						Asura_Xbox360_Graphics::GetDevice()->DrawIndexedVertices( D3DPT_TRIANGLESTRIP, 0, uStartIndex, uNumIndices );

					}
				}

				uStartIndex += (pxOldStrip ? pxOldStrip->m_uNumIndices : pxNewStrip->m_uNumIndices);
			}
			pxEffect->End();
		}

		const u_int uNumTriangles =
			(pxOldSkin ? pxOldSkin->GetNumTriangles() : pxNewSkin->GetNumTriangles());

		Asura_Render::AddToTrianglesRendered( uNumTriangles );
	}
}



void AVP_GlowOutlines::Platform_Render(  DepthSortedSkinEntry * const * pxBuffer) 
{
	static const AVP_FS_Vertex axVerts[] =
	{
		{ -1.0f,  1.0f, 0.0f,1.0f, 0.0f, 0.0f, },
		{  1.0f,  1.0f, 0.0f,1.0f, 1.0f, 0.0f },
		{  1.0f, -1.0f, 0.0f,1.0f, 1.0f, 1.0f },
		{ -1.0f, -1.0f, 0.0f,1.0f, 0.0f, 1.0f },
	};

	Direct3D_Surface* pxOldRenderTargetBuffer[5] = 
	{
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
	};
	SCOPEDPIXEVENT(( 0xFF884499, "AVP Glow Outlines"));

	DepthSortedSkinEntry *pxFirst = NULL, *pxPrev = NULL;
	for (int i = NUM_DEPTH_SORT_SLOTS-1; i >= 0; --i)	
	{
		// fix up the list slightly so the next time we iterate its just a flat linked list, we dont need to go through lots of empty buckets twice...
		if (pxBuffer[i])
		{
			if (pxPrev)
			{
				pxPrev->m_pxNext = pxBuffer[i];
			}
			if (!pxFirst)
			{
				pxFirst = pxBuffer[i];

				Asura_Xbox360_Graphics::GetDevice()->GetRenderTarget(0, &pxOldRenderTargetBuffer[0]);
				Asura_Xbox360_Graphics::GetDevice()->GetRenderTarget(1, &pxOldRenderTargetBuffer[1]);
				Asura_Xbox360_Graphics::GetDevice()->GetRenderTarget(2, &pxOldRenderTargetBuffer[2]);
				Asura_Xbox360_Graphics::GetDevice()->GetRenderTarget(3, &pxOldRenderTargetBuffer[3]);
				Asura_Xbox360_Graphics::GetDevice()->GetDepthStencilSurface(&pxOldRenderTargetBuffer[4]);

				Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(0, g_pxGlowRenderTarget[0]);
				Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(1, g_pxGlowRenderTarget[1]);
				Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(2, NULL);
				Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(3, NULL);
				Asura_Xbox360_Graphics::GetDevice()->SetDepthStencilSurface(NULL);
				Asura_Xbox360_Graphics::GetDevice()->Clear(0, NULL, D3DCLEAR_TARGET, 0, 0.0f, 0);


				// Set render states
				{
					SCOPEDPIXEVENT(( 0xFF884499, "Set render states" ));

					Asura_Render::CurrentStates.RequestAlphaTest( false );
					Asura_Render::CurrentStates.RequestCullMode( ASURA_CULLMODE_CW );
					Asura_Render::CurrentStates.RequestDitheringMode( true );
					Asura_Render::CurrentStates.RequestDynamicLights( false );
					Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF);
					Asura_Xbox360_Graphics::GetDevice()->SetRenderState( D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );
					Asura_Xbox360_Graphics::GetDevice()->SetRenderState( D3DRS_COLORWRITEENABLE1, D3DCOLORWRITEENABLE_ALPHA );
				}
#if ASURA_D3D_ENABLE_PIX
				D3DPERF_BeginEvent(0xFF884499, "Render skins to buffers");
#endif
			}
		}
		for (DepthSortedSkinEntry *pxEntry0 = pxBuffer[i]; pxEntry0; pxEntry0 = pxEntry0->m_pxNext)
		{
			pxPrev = pxEntry0;

			Asura_Animation *pxAnim = pxEntry0->m_xSortedRenderItem.m_pxAnimation;

			FixUpAsura_Hierarchy_SystemForAnimation(pxAnim, pxEntry0->m_xSortedRenderItem.m_xPosition, pxEntry0->m_xSortedRenderItem.m_xOrient, pxEntry0->m_xSortedRenderItem.m_uRenderFlags);

			for( int iSkinIndex = 0; iSkinIndex < ASURA_ANIMATION_SKIN_COUNT; ++iSkinIndex )
			{
				const Asura_Hierarchy_Skin *pxSkin = pxAnim->GetSkin(iSkinIndex);

				if (pxSkin)
				{
					for( int i = 0; i < Asura_Camera::GetNumberOfCameras(); ++i )
					{
						if((pxEntry0->m_xSortedRenderItem.m_uCamVisFlags & (1 << i)) != 0)
						{
							if((Asura_Hierarchy_System::ShouldJustRenderForCurCam() && ((u_int)i == Asura_Camera::GetCameraIndex())) || Asura_Camera::IsCameraActive(i))
							{
								Asura_Camera::SetCameraIndex(i);
								Asura_Render::ApplyCameraSettings();

								// Select LOD mesh based on distance to first bone, since object may be animating far away from its location
								Asura_Hierarchy_Skin const* const	pxSkinLOD 
									= pxSkin->GetDetailSkinFromDistance( (Asura_Hierarchy_System::GetWorldspaceBonePos(0) - *Asura_Camera::GetPosition()).Magnitude() );

								if(pxSkinLOD)
								{
									GlowEffect::g_pxEffect->SetVector( GlowEffect::g_xBlurColour, &pxEntry0->m_xColour.m_fR);
									RenderSkin(*pxEntry0->m_xSortedRenderItem.m_pxAnimation, *pxSkinLOD, Asura_Hierarchy_System::s_xBaseTransform, Asura_Hierarchy_System::s_xInverseBaseTransform, Asura_Hierarchy_System::s_axBoneTransforms);
								}
								else
								{
									Asura_Assertf(false, ("Couldn't retrieve LOD skin for '%s' at distance %.02f.", pxSkin->GetName(), (Asura_Hierarchy_System::GetWorldspaceBonePos(0) - *Asura_Camera::GetPosition()).Magnitude()));
								}
							}
						}
					}
				}
			}
		}
	}

	if (pxFirst)
	{
		Asura_Xbox360_Graphics::GetDevice()->Resolve(D3DRESOLVE_RENDERTARGET0|D3DRESOLVE_CLEARRENDERTARGET, NULL, g_pxGlowRenderTargetTexture0, NULL, 0, 0, NULL, 0.0f, 0, NULL);

#if ASURA_D3D_ENABLE_PIX
		D3DPERF_EndEvent();
#endif
		Asura_Xbox360_Effect* pxEffect = GlowEffect::g_pxEffect;

		// Take the colour buffer we have just stored all the skins in and horizontally blur it
		{
			SCOPEDPIXEVENT(( 0xFF884499, "Horizontal blur" ));
			Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(1, NULL);
			Asura_Render::CurrentStates.RequestTranslucencyMode( ASURA_TRANSLUCENCY_OFF );
			Asura_Render::CurrentStates.RequestCullMode(ASURA_CULLMODE_NONE);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState( D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA );



			Asura_Xbox360_Graphics::GetDevice()->SetVertexDeclaration(g_pxVertexDeclFS);	

			float afParams[4];
			afParams[0] = GLOW_RADIUS;
			afParams[1] = GLOW_MAX_DISTANCE;
			pxEffect->SetVector( GlowEffect::g_xBlurParams, afParams);
			pxEffect->SetTexture(GlowEffect::g_xSceneTextureHandle, g_pxGlowRenderTargetTexture0);
			pxEffect->SetTexture(GlowEffect::g_xDepthTextureHandle, Asura_Xbox360_Graphics::GetDepthMtlTexture());

			pxEffect->Begin(1, 0);
			pxEffect->Commit();
			Asura_Xbox360_Graphics::GetDevice()->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( AVP_FS_Vertex ) );
			pxEffect->End();

		}

		Asura_Xbox360_Graphics::GetDevice()->Resolve(D3DRESOLVE_RENDERTARGET0|D3DRESOLVE_CLEARRENDERTARGET, NULL, g_pxGlowRenderTargetTexture0, NULL, 0, 0, NULL, 0.0f, 0, NULL);



		// Now take the horizontally blended texture and vertically blend it and apply it to the screen
		{
			SCOPEDPIXEVENT(( 0xFF884499, "Vertical blur" ));
			Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(0, g_pxGlowRenderTarget[1]);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState( D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN | D3DCOLORWRITEENABLE_BLUE );

			float afParams[4];
			afParams[0] = GLOW_RADIUS;
			afParams[1] = GLOW_MAX_DISTANCE;
			pxEffect->SetVector( GlowEffect::g_xBlurParams, afParams);


			pxEffect->Begin(1, 1);
			pxEffect->Commit();
			Asura_Xbox360_Graphics::GetDevice()->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( AVP_FS_Vertex ) );
			pxEffect->End();
		}
		Asura_Xbox360_Graphics::GetDevice()->Resolve(D3DRESOLVE_RENDERTARGET0|D3DRESOLVE_CLEARRENDERTARGET, NULL, &g_xGlowRenderTargetTexture1, NULL, 0, 0, NULL, 0.0f, 0, NULL);

		// and apply the blurred surface to the screen
		{
			SCOPEDPIXEVENT(( 0xFF884499, "Draw To screen" ));

			Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(0, pxOldRenderTargetBuffer[0]);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(1, pxOldRenderTargetBuffer[1]);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(2, pxOldRenderTargetBuffer[2]);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(3, pxOldRenderTargetBuffer[3]);
			Asura_Xbox360_Graphics::GetDevice()->SetDepthStencilSurface(pxOldRenderTargetBuffer[4]);
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

			pxEffect->SetTexture(GlowEffect::g_xSceneTextureHandle, &g_xGlowRenderTargetTexture1);

			pxEffect->Begin(1, 2);
			pxEffect->Commit();
			Asura_Xbox360_Graphics::GetDevice()->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, axVerts, sizeof( AVP_FS_Vertex ) );
			pxEffect->End();
		}


		{
			pxEffect->SetTexture(GlowEffect::g_xSceneTextureHandle, NULL);
			pxEffect->SetTexture(GlowEffect::g_xDepthTextureHandle, NULL);

			SCOPEDPIXEVENT(( 0xFF884499, "Restore renderstates" ));
			for (int i = 0; i < 8; i++)
			{
				Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(i, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
				Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(i, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
				Asura_Xbox360_Graphics::GetDevice()->SetSamplerState(i, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
			}
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_SRCBLENDALPHA, D3DBLEND_ONE);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_DESTBLENDALPHA, D3DBLEND_ZERO);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderState( D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_ALL );
		
			Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(0, pxOldRenderTargetBuffer[0]);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(1, pxOldRenderTargetBuffer[1]);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(2, pxOldRenderTargetBuffer[2]);
			Asura_Xbox360_Graphics::GetDevice()->SetRenderTarget(3, pxOldRenderTargetBuffer[3]);
			Asura_Xbox360_Graphics::GetDevice()->SetDepthStencilSurface(pxOldRenderTargetBuffer[4]);
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);

		}
	}

}


