
/////////////////////////////////////////////////////////////
// ToolsAVP_Blueprint_AnimSpreadsheetExporter.h
// Will Plano - 19/08/2008
// Copyright (C) 2008 Rebellion, All Rights Reserved.
/////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Blueprint/ToolsAVP_Blueprint_AnimSpreadsheetExporter.h"

#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"

#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Waypointing/Tools_Mode_Waypointing.h"

#include "Actor/AVP_Actor_AnimStateData.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_AIAnimStateData.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_PlayerAnimStateData.h"
#include "Actor/Vertebrates/AlienQueen/AVP_AlienQueen_AIAnimStateData.h"
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_AIAnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_AIAnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_PlayerAnimStateData.h"
#include "Actor/Vertebrates/Predalien/AVP_Predalien_AIAnimStateData.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_AIAnimStateData.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_PlayerAnimStateData.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"

static bool s_bActive = false;
static FILE* s_pxFile = NULL;

static const int s_iNumParts = 5 + s_uNumHumanAIWeapons + s_uNumHumanMultiPlayerWeapons + s_uNumPredatorAIWeapons + s_uNumPredatorMultiPlayerWeapons;

void ToolsAVP_Blueprint_AnimSpreadsheetExporter::Run()
{
	s_bActive = true;

	int iCurrentPart = 0;
	
	AVP_Predalien_AIAnimStateData::Initialise( uAVP_BLUEPRINT_TYPE_ANIMSTATE_PREDALIEN );
	AVP_Predalien_AIAnimStateData::ResetStorage();
	
	if ( Tools_Mode_Waypointing::UpdateProgress( ++iCurrentPart, s_iNumParts, "Anim Spreadsheets Exporting..." ) )
	{
		return;
	}

	AVP_Alien_AIAnimStateData::Initialise( uAVP_BLUEPRINT_TYPE_ANIMSTATE_ALIEN );
	AVP_Alien_AIAnimStateData::ResetStorage();

	if ( Tools_Mode_Waypointing::UpdateProgress( ++iCurrentPart, s_iNumParts, "Anim Spreadsheets Exporting..." ) )
	{
		return;
	}

	AVP_Alien_PlayerAnimStateData::GetSingleton().Initialise( uAVP_BLUEPRINT_TYPE_ANIMSTATE_ALIEN, true );
	AVP_Alien_PlayerAnimStateData::GetSingleton().ResetStorage();

	if ( Tools_Mode_Waypointing::UpdateProgress( ++iCurrentPart, s_iNumParts, "Anim Spreadsheets Exporting..." ) )
	{
		return;
	}

	AVP_AlienQueen_AIAnimStateData::Initialise( uAVP_BLUEPRINT_TYPE_ANIMSTATE_ALIENQUEEN );
	AVP_AlienQueen_AIAnimStateData::ResetStorage();

	if ( Tools_Mode_Waypointing::UpdateProgress( ++iCurrentPart, s_iNumParts, "Anim Spreadsheets Exporting..." ) )
	{
		return;
	}

	AVP_FaceHugger_AIAnimStateData::Initialise( uAVP_BLUEPRINT_TYPE_ANIMSTATE_FACEHUGGER );
	AVP_FaceHugger_AIAnimStateData::ResetStorage();

	if ( Tools_Mode_Waypointing::UpdateProgress( ++iCurrentPart, s_iNumParts, "Anim Spreadsheets Exporting..." ) )
	{
		return;
	}

	for ( u_int uHumanWeaponItr = 0; uHumanWeaponItr != s_uNumHumanAIWeapons; ++uHumanWeaponItr )
	{
		AVP_Human_AIAnimStateData::InitialiseWeaponData( uAVP_BLUEPRINT_TYPE_ANIMSTATE_MARINE, s_auHumanAIWeaponHash[ uHumanWeaponItr ] );

		if ( Tools_Mode_Waypointing::UpdateProgress( ++iCurrentPart, s_iNumParts, "Anim Spreadsheets Exporting..." ) )
		{
			return;
		}
	}
	AVP_Human_AIAnimStateData::ResetStorage();

	for ( u_int uHumanWeaponItr = 0; uHumanWeaponItr != s_uNumHumanMultiPlayerWeapons; ++uHumanWeaponItr )
	{
		AVP_Human_PlayerAnimStateData::GetSingleton().InitialiseWeaponData( uAVP_BLUEPRINT_TYPE_ANIMSTATE_MARINE, s_auHumanMultiPlayerWeaponHash[ uHumanWeaponItr ] );

		if ( Tools_Mode_Waypointing::UpdateProgress( ++iCurrentPart, s_iNumParts, "Anim Spreadsheets Exporting..." ) )
		{
			return;
		}
	}
	AVP_Human_PlayerAnimStateData::GetSingleton().ResetStorage();

	for ( u_int uPredatorWeaponItr = 0; uPredatorWeaponItr != s_uNumPredatorAIWeapons; ++uPredatorWeaponItr )
	{
		AVP_Predator_AIAnimStateData::InitialiseWeaponData( uAVP_BLUEPRINT_TYPE_ANIMSTATE_PREDATOR, s_auPredatorAIWeaponHash[ uPredatorWeaponItr ] );

		if ( Tools_Mode_Waypointing::UpdateProgress( ++iCurrentPart, s_iNumParts, "Anim Spreadsheets Exporting..." ) )
		{
			return;
		}
	}
	AVP_Predator_AIAnimStateData::ResetStorage();

	for ( u_int uPredatorWeaponItr = 0; uPredatorWeaponItr != s_uNumPredatorMultiPlayerWeapons; ++uPredatorWeaponItr )
	{
		AVP_Predator_PlayerAnimStateData::GetSingleton().InitialiseWeaponData( uAVP_BLUEPRINT_TYPE_ANIMSTATE_PREDATOR, s_auPredatorMultiPlayerWeaponHash[ uPredatorWeaponItr ] );

		if ( Tools_Mode_Waypointing::UpdateProgress( ++iCurrentPart, s_iNumParts, "Anim Spreadsheets Exporting..." ) )
		{
			return;
		}
	}
	AVP_Predator_PlayerAnimStateData::GetSingleton().ResetStorage();

	CloseFile();

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_INFO, "Files saved to \"Logfiles\\AnimState\\\"" );
	s_bActive = false;
}

bool ToolsAVP_Blueprint_AnimSpreadsheetExporter::IsActive()
{
	return s_bActive;
}

void ToolsAVP_Blueprint_AnimSpreadsheetExporter::CreateFile( const char* szFileName)
{
	if ( s_pxFile != NULL )
	{
		CloseFile();
	}

	Asura_String xFullPath = "Logfiles\\AnimState\\";
	Tools_FileSystem::EnsureDirectoryExists( xFullPath.GetString() );
	xFullPath += szFileName;
	if ( Tools_SaveAsura::CanSaveFile( xFullPath.GetString(), true ) )
	{
		s_pxFile = fopen( xFullPath.GetString(), "w" );
		Asura_Assertf( s_pxFile != NULL, ( "fopen failed! with error code %d", errno ) );
		WriteCell( "State" );
		WriteCell( "Primary" );
		WriteCell( "Secondary" );
		WriteCell( "AimDownToUp" );
		WriteCell( "AimRightToLeft" );
		WriteCell( "TurnLeft45" );
		WriteCell( "TurnLeft90" );
		WriteCell( "TurnRight45" );
		WriteCell( "TurnRight90" );
		WriteCell( "Posture" );
		WriteCell( "PrimaryActorShootBegin" );
		WriteCell( "PrimaryWeaponShootBegin" );
		WriteCell( "SecondaryActorShootBegin" );
		WriteCell( "SecondaryWeaponShootBegin" );
		WriteCell( "PrimaryActorShootLoop" );
		WriteCell( "PrimaryWeaponShootLoop" );
		WriteCell( "SecondaryActorShootLoop" );
		WriteCell( "SecondaryWeaponShootLoop" );
		WriteCell( "PrimaryActorShootEnd" );
		WriteCell( "PrimaryWeaponShootEnd" );
		WriteCell( "SecondaryActorShootEnd" );
		WriteCell( "SecondaryWeaponShootEnd" );
		WriteCell( "WeaponInActorAnim" );
		WriteCell( "WeaponInWeaponAnim" );
		WriteCell( "WeaponOutActorAnim" );
		WriteCell( "WeaponOutWeaponAnim" );
		WriteCell( "WeaponRestWeaponAnim" );
		WriteCell( "ReloadActor" );
		WriteCell( "ReloadWeapon" );
		WriteCell( "StimActorAnim" );
		WriteCell( "StimItemAnim" );
		WriteCell( "StimWeaponAnim" );
		WriteCell( "FlareActorAnim" );
		WriteCell( "CloakActorAnim" );
		WriteCell( "CloakWeaponAnim" );
		WriteCell( "FidgetBlueprint", true );
	}
}

void ToolsAVP_Blueprint_AnimSpreadsheetExporter::WriteState( const char* szStateName, const AVP_Actor_AnimStateData* pxAnimStateData, const Asura_Blueprint* pxStateBP, const Asura_Hash_ID uStateBPType )
{
	Asura_String xCurrentStateName = szStateName;
	const AVP_Actor_AnimStateData* pxCurrentStateData = pxAnimStateData;
	u_int uRegionItr = 0;
	u_int uAnimItr = 0;
	while ( pxCurrentStateData )
	{
		if ( pxAnimStateData->GetNumBlendRegions() > 0 && pxAnimStateData->GetBlendRegion( 0 ) && pxAnimStateData->GetBlendRegion( 0 )->GetNumAnims() > 0 )
		{
			pxCurrentStateData = pxAnimStateData->GetBlendRegion( uRegionItr ) != NULL ? pxAnimStateData->GetBlendRegion( uRegionItr )->GetAnimData( uAnimItr ) : NULL;
			char szRegionNumBuffer[3];
			Asura_PrintfToBfr( szRegionNumBuffer, "%02d", 3, true, uRegionItr + 1 );
			char szAnimNumBuffer[3];
			Asura_PrintfToBfr( szAnimNumBuffer, "%02d", 3, true, uAnimItr + 1 );
			xCurrentStateName = Asura_String( szStateName ) + ".Region." + szRegionNumBuffer + ".Anim." + szAnimNumBuffer;
			
			if ( pxAnimStateData->GetBlendRegion( uRegionItr ) == NULL || ( ++uAnimItr == pxAnimStateData->GetBlendRegion( uRegionItr )->GetNumAnims() ) )
			{
				uAnimItr = 0;
				++uRegionItr;
			}
		}
		
		WriteCell( xCurrentStateName.GetString() );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetSecondaryAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetAimDownUpAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetAimRightLeftAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetTurn45LeftAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetTurn90LeftAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetTurn45RightAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetTurn90RightAnimHash() ) );
		WriteCell( GetBPNameFromHash( uAVP_BLUEPRINT_TYPE_ACTOR_STATE, pxCurrentStateData->GetCollisionDescriptorHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetPrimaryActorShootBeginHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetPrimaryWeaponShootBeginHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetSecondaryActorShootBeginHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetSecondaryWeaponShootBeginHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetPrimaryActorShootLoopHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetPrimaryWeaponShootLoopHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetSecondaryActorShootLoopHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetSecondaryWeaponShootLoopHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetPrimaryActorShootEndHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetPrimaryWeaponShootEndHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetSecondaryActorShootEndHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetSecondaryWeaponShootEndHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetWeaponInActorAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetWeaponInWeaponAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetWeaponOutActorAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetWeaponOutWeaponAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetWeaponRestWeaponAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetReloadActorHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetReloadWeaponHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetStimActorAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetStimItemAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetStimWeaponAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetFlareActorAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetCloakActorAnimHash() ) );
		WriteCell( GetAnimNameFromHash( pxCurrentStateData->GetCloakWeaponAnimHash() ) );
		WriteCell( GetBPNameFromHash( uStateBPType, pxCurrentStateData->GetFidgetBlueprintHash() ) );
		if ( pxAnimStateData->GetAnimHash() != ASURA_HASH_ID_UNSET || ( uRegionItr == pxAnimStateData->GetNumBlendRegions() ) )
		{
			pxCurrentStateData = NULL;
		}
	}
}

void ToolsAVP_Blueprint_AnimSpreadsheetExporter::CloseFile()
{
	Asura_Assert( s_pxFile != NULL, "Can't close file, none open!" );
	if ( s_pxFile != NULL )
	{
		if ( fclose( s_pxFile ) != 0 )
		{
			Asura_Assertf( false, ( "fclose failed! with error code %d", errno ) );
		}
		s_pxFile = NULL;
	}
}

void ToolsAVP_Blueprint_AnimSpreadsheetExporter::WriteCell(	const char* szCellText, const bool bLastInRow /*= false*/ )
{
	Asura_Assert( s_pxFile != NULL, "No file open!" );
	if ( s_pxFile != NULL )
	{
		fputs( szCellText, s_pxFile );
		if ( bLastInRow )
		{
			fputc( '\n', s_pxFile );
		}
		else
		{
			fputc( ',',	s_pxFile );
		}
	}
}

const char* ToolsAVP_Blueprint_AnimSpreadsheetExporter::GetAnimNameFromHash( const Asura_Hash_ID uAnimHash )
{
	if ( uAnimHash == ASURA_HASH_ID_UNSET )
	{
		return "<None>";
	}
	const Asura_Hierarchy_Anim* pxAnim = Tools_Animation_Files::GetAnimFromID( uAnimHash );
	if ( pxAnim != NULL )
	{
		return pxAnim->GetName();
	}
	else
	{
		return "<Error>";
	}
}

const char* ToolsAVP_Blueprint_AnimSpreadsheetExporter::GetBPNameFromHash( const Asura_Hash_ID uBPTypeHash, const Asura_Hash_ID uBPHash )
{
	if ( uBPHash == ASURA_HASH_ID_UNSET )
	{
		return "<None>";
	}
	const Asura_Blueprint* pxBP = Asura_Blueprint_System::GetBlueprintByHashID( uBPTypeHash, uBPHash );
	Asura_Assert( pxBP != NULL, "No Blueprint Found" );
	if ( pxBP != NULL )
	{
		return pxBP->GetName().GetString();
	}
	else
	{
		return "<Error>";
	}
}
