//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Cutscene_Event_CloakActor.cpp
//
// Created by Ali Jakamy 25/06/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "ToolsAVP_Cutscene_Event_CloakActor.h"

// Asura
#include "Core/Asura_String.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

// AVP
#include "Cutscene/AVP_Cutscene_Event_CloakActor.h"

// Tools
#include "Cutscene/Tools_Cutscene_Dialog.h"
#include "Cutscene/Tools_Mode_Cutscene.h"


ToolsAVP_Cutscene_Event_CloakActor::ToolsAVP_Cutscene_Event_CloakActor(Asura_Cutscene_Event* pxEvent, Tools_Cutscene_Object* pxCutsceneObject, int iEventID)
: PARENT(pxEvent,pxCutsceneObject,iEventID)
, m_fDurationBackup(0.0f)
, m_fCloakingDurationBackup(0.0f)
, m_fUncloakingDurationBackup(0.0f)
, m_fLightningScaleBackup(0.0f)
{
}


void ToolsAVP_Cutscene_Event_CloakActor::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	AVP_Cutscene_Event_CloakActor* pxCloakEvent = static_cast<AVP_Cutscene_Event_CloakActor*> (Event);

	char acText[200];

	//add actor selection control
	if(pxCloakEvent->m_xTargetActorName.GetLength())
		sprintf(acText,"Actor : %s",pxCloakEvent->m_xTargetActorName.GetString());
	else
		strcpy(acText,"Actor : *None*");

	Asura_Dialog* pxDialog = new Asura_Function_Dialog(acText, xOffset, ChooseActor);
	pxDialog->SetHelpText("Select actor you want cloaked.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Create duration time dialog
	pxDialog = new Tools_CutsceneTime_Dialog("Cloak event duration", xOffset, &pxCloakEvent->m_fDuration, 0.000f, 1000.0f, 0.1f, &Changed);
	pxDialog->SetHelpText("Time over which to set cloaking");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Create uncloaked to cloaked time dialog
	pxDialog = new Tools_CutsceneTime_Dialog("Cloaking duration", xOffset, &pxCloakEvent->m_fCloakingDuration, 0.000f, 1000.0f, 0.1f, &Changed);
	pxDialog->SetHelpText("Time it takes to go from uncloaked to cloaked");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Create cloaked to uncloaked time dialog
	pxDialog = new Tools_CutsceneTime_Dialog("Uncloaking duration", xOffset, &pxCloakEvent->m_fUncloakingDuration, 0.000f, 1000.0f, 0.1f, &Changed);
	pxDialog->SetHelpText("Time it takes to go from cloaked to uncloaked");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Create lightning dialog
	pxDialog = new Tools_CutsceneTime_Dialog("Lightning scale", xOffset, &pxCloakEvent->m_fLightningScale, 0.0001f, 1.0f, 0.01f, &Changed);
	pxDialog->SetHelpText("Set the scale of lightning during the Enter/Exit cloaking phase");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}


Asura_String ToolsAVP_Cutscene_Event_CloakActor::GetDescription()
{
	return "Cloak Actor";
}

void ToolsAVP_Cutscene_Event_CloakActor::UpdateBackupVariables()
{
	PARENT::UpdateBackupVariables();

	AVP_Cutscene_Event_CloakActor* pxEvent = static_cast<AVP_Cutscene_Event_CloakActor*> (Event);

	m_fDurationBackup		    = pxEvent->m_fDuration;
	m_fCloakingDurationBackup   = pxEvent->m_fCloakingDuration;
	m_fUncloakingDurationBackup = pxEvent->m_fUncloakingDuration;
	m_fLightningScaleBackup     = pxEvent->m_fLightningScale;
}

void ToolsAVP_Cutscene_Event_CloakActor::AskForRequiredData()
{
	//select target
	ChooseActor();
}
