//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Cutscene_Event_SetVisionMode.cpp
//
// Created by Ali Jakamy 25/06/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "ToolsAVP_Cutscene_Event_SetVisionMode.h"

// Asura
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Core/Asura_String.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

// AVP
#include "Cutscene/AVP_Cutscene_Event_SetVisionMode.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"

// Tools
#include "Cutscene/Tools_Cutscene_Dialog.h"
#include "Cutscene/Tools_Mode_Cutscene.h"

//
// CLASS ToolsAVP_Cutscene_Event_SetVisionMode
//
ToolsAVP_Cutscene_Event_SetVisionMode::ToolsAVP_Cutscene_Event_SetVisionMode(Asura_Cutscene_Event* pxEvent, Tools_Cutscene_Object* pxCutsceneObject, int iEventID)
: PARENT(pxEvent,pxCutsceneObject,iEventID)
, m_uVisionModeHashBackup(ASURA_HASH_ID_UNSET)
, m_fDurationBackup(0.0f)
, m_uFlagsBackup(0)
{
}


void ToolsAVP_Cutscene_Event_SetVisionMode::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	// Get AVP cutscene event
	AVP_Cutscene_Event_SetVisionMode* pxVisionMode = static_cast<AVP_Cutscene_Event_SetVisionMode*> (Event);

	// Create drop down dialog for vision modes
	Asura_Dialog* pxDialog = new ToolsAVP_SetVisionMode_Event_DropDownList_Dialog(xOffset, NUM_VISION_MODES_DISPLAYED, &pxVisionMode->m_uVisionModeHash, &Changed, "Vision Mode");
	pxDialog->SetHelpText("Set the vision mode during this cutscene.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += ToolsAVP_SetVisionMode_Event_DropDownList_Dialog::DefaultSpacing();

	// Create duration time dialog
	pxDialog = new Tools_CutsceneTime_Dialog("Duration",xOffset,&pxVisionMode->m_fDuration, 0.0f, 1000.0f, 0.1f, &Changed);
	pxDialog->SetHelpText("Time over which to set the vision mode.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Create tickbox for immediate transition of vision mode at start of event
	pxDialog = new Asura_FlagTickbox_Dialog( "Immediate start transition", xOffset, &pxVisionMode->m_uFlags, pxVisionMode->FLAG_IMMEDIATE_TRANSITION_AT_START, &Changed);
	pxDialog->SetHelpText("Set the transition from normal vision mode to choosen vision mode as immediate or not.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Create tickbox for immediate transition of vision mode at end of event
	pxDialog = new Asura_FlagTickbox_Dialog( "Immediate end transition", xOffset, &pxVisionMode->m_uFlags, pxVisionMode->FLAG_IMMEDIATE_TRANSITION_AT_END, &Changed);
	pxDialog->SetHelpText("Set the transition from choosen vision to normal vision mode as immediate or not.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();
}

void ToolsAVP_Cutscene_Event_SetVisionMode::UpdateBackupVariables()
{
	PARENT::UpdateBackupVariables();

	AVP_Cutscene_Event_SetVisionMode* pxEvent = static_cast<AVP_Cutscene_Event_SetVisionMode*> (Event);

	m_uVisionModeHashBackup = pxEvent->m_uVisionModeHash;
	m_fDurationBackup       = pxEvent->m_fDuration;
	m_uFlagsBackup          = pxEvent->m_uFlags;
}

//
// CLASS ToolsAVP_SetVisionMode_Event_DropDownList_Dialog
//
ToolsAVP_SetVisionMode_Event_DropDownList_Dialog::ToolsAVP_SetVisionMode_Event_DropDownList_Dialog( const Asura_Vector_2& xOffset, const int iNumberOfLines, Asura_Hash_ID* puSelected, bool* pbChangedFlag, const char* szName)
: PARENT(xOffset)
{
	Initialise(xOffset, iNumberOfLines, puSelected, pbChangedFlag, szName);
}


u_int ToolsAVP_SetVisionMode_Event_DropDownList_Dialog::GetNumberOfHashValues() const
{
	return Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uAVP_BLUEPRINT_NAME_INVENTORYITEM_VISIONMODE, false);
}

const char* ToolsAVP_SetVisionMode_Event_DropDownList_Dialog::GetNameByIndex(int iIndex) const
{
	Asura_Blueprint* pxBluePrint = Asura_Blueprint_System::GetChildBlueprint(uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uAVP_BLUEPRINT_NAME_INVENTORYITEM_VISIONMODE, false, iIndex);
	if(pxBluePrint)
	{
		return (pxBluePrint->GetName().GetString());
	}
	Asura_Assert(false, "Index return no blueprint.");
	return NULL;
}