//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_Squad_FaceHugger.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Controller/ToolsAVP_Entity_Squad_FaceHugger.h"

// asura
#include "Interface/Asura_Table_Dialog.h"

// avp
#include "Entity/AVP_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_FaceHugger.h"

// tools
#include "Entity/GamePlayVolume/ToolsAVP_Entity_GamePlayVolume.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"

// tools avp
#include "ToolsAVP_ChooseObjects.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_Squad_FaceHugger
//
//////////////////////////////////////////////////////////////////////////////

ToolsAVP_Entity_Squad_FaceHugger* ToolsAVP_Entity_Squad_FaceHugger::s_pxCurrentActive = NULL;

ToolsAVP_Entity_Squad_FaceHugger::ToolsAVP_Entity_Squad_FaceHugger( Asura_Entity* pxEntity )
: PARENT( pxEntity )
, m_bRefreshOverlay( false )
, m_xArray_pszGPVolumeNames()
, m_iPGVolumeSelectedIndex(0)
{
	Type = TOT_Entity_FaceHuggerSquad;
	SetMesh( "ToolsShape_FaceHuggerSquad" );
}

ToolsAVP_Entity_Squad_FaceHugger::~ToolsAVP_Entity_Squad_FaceHugger()
{
	DestroyGPVolumeList();
}

bool ToolsAVP_Entity_Squad_FaceHugger::CanBeDestroyed()
{
	return (!GetFaceHuggerSquad()->IsDefaultSquad() && !GetFaceHuggerSquad()->IsDefaultPetSquad());
}

void ToolsAVP_Entity_Squad_FaceHugger::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	AVP_ServerEntity_Squad_FaceHugger *pxSquadFacehugger = GetFaceHuggerSquad();
	PARENT::AddOverlayControls( pxOverlay, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	AVP_ServerEntity_Squad_FaceHugger* pxSquad = GetFaceHuggerSquad();

	Asura_Dialog* pxDialogue = NULL;

	static float s_fMin = 0.0f;
	static float s_fMax = 1.0f;

	if (!pxSquadFacehugger->IsDefaultPetSquad())
	{
		pxDialogue = new Asura_Dialog( "Behaviour:", xOffset );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0f;

		Asura_DropDown_List_Dialog* pxDDLDialogue = new Asura_DropDown_List_Dialog(
			xOffset,
			AVP_FaceHuggerBehaviourData::s_apszBehaviourStrings,
			&pxSquad->m_xBehaviourData.m_iBehaviour,
			AVP_FaceHuggerBehaviourData::NUM_BEHAVIOURS,
			AVP_FaceHuggerBehaviourData::NUM_BEHAVIOURS,
			NULL,
			&Changed,
			"" );
		pxDDLDialogue->SetOnSelectCallback( BehaviourDataChanged, reinterpret_cast< int >( this ) );
		pxOverlay->AddDialog( pxDDLDialogue );
		xOffset.x -= 7.0f;
		xOffset.y += Asura_Dialog::LargeSpacing();
	}
	else
	{
		pxSquad->m_xBehaviourData.m_iBehaviour = AVP_FaceHuggerBehaviourData::ATTACK;
	}

	if (pxSquad->m_xBehaviourData.m_iBehaviour == AVP_FaceHuggerBehaviourData::ATTACK || 
		pxSquad->m_xBehaviourData.m_iBehaviour == AVP_FaceHuggerBehaviourData::HIDE_IN_SHADOWS )
	{
		pxDialogue = new Asura_Numerical_Dialog( "Avoidance Distance", xOffset, &pxSquad->m_xBehaviourData.m_fAvoidanceDistance , 5.0f, 20.0f, .5f, "%f", &Changed );
		pxDialogue->SetEnabled( true );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Max Targeters", xOffset, &pxSquad->m_xBehaviourData.m_uMaxTargeters, 1, 10, 1, "%d", &Changed );
		pxDialogue->SetEnabled( true );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Time Between Attacks", xOffset, &pxSquad->m_xBehaviourData.m_fTimeBetweenAttacks, 1.0f, 60.0f, .5f, "%f", &Changed );
		pxDialogue->SetEnabled( true );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Wander Distance", xOffset, &pxSquad->m_xBehaviourData.m_fWanderDistance, 4.0f, 10.0f, .5f, "%f", &Changed );
		pxDialogue->SetEnabled( true );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		if (!pxSquadFacehugger->IsDefaultPetSquad())
		{
			pxDialogue = new Asura_Dialog( "Gameplay Volumes:", xOffset );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.y += Asura_Dialog::DefaultSpacing();

			if ( pxSquadFacehugger->m_xGPVolumesIDList.Size() > 0)
			{
				pxDialogue = CreateGPVolumeList( xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += 3 + 0.5f;
			}

			pxDialogue = new Asura_Function_Dialog( "Add Gameplay Volume", xOffset, SelectGPVolume );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.x += 18.0f;

			pxDialogue = new Asura_Function_Dialog( "Remove Selected", xOffset, RemoveSelectedGPVolume);
			pxOverlay->AddDialog( pxDialogue );
			xOffset.x -= 18.0f;
			xOffset.y += Asura_Dialog::LargeSpacing();

		}
	}		

	if (pxSquad->m_xBehaviourData.m_iBehaviour == AVP_FaceHuggerBehaviourData::HIDE_IN_SHADOWS )
	{
		pxDialogue = new Asura_Tickbox_Dialog("Attack if in marine light", xOffset, &pxSquad->m_xBehaviourData.m_bAttackIfInMarineLight, &Changed);
		pxDialogue->SetEnabled( true );
		static_cast<Asura_Tickbox_Dialog *>(pxDialogue)->SetClickedFunction( BehaviourDataChanged, reinterpret_cast< int >( this ) );
		pxOverlay->AddDialog( pxDialogue );				
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Time in light before attack", xOffset, &pxSquad->m_xBehaviourData.m_fTimeInLightBeforeAttack , 0.5f, 10.0f, 0.1f, "%.2f", &Changed );
		pxDialogue->SetEnabled( pxSquad->m_xBehaviourData.m_bAttackIfInMarineLight );
		pxOverlay->AddDialog( pxDialogue );				
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Numerical_Dialog( "Min distance from target to hide at", xOffset, &pxSquad->m_xBehaviourData.m_fMinHideDistance , 5.0f, 10.0f, 0.5f, "%.2f", &Changed );
		pxDialogue->SetEnabled( true );
		pxOverlay->AddDialog( pxDialogue );				
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialogue = new Asura_Dialog( "Too Close :", xOffset );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0f;
		pxDialogue = new Asura_Dialog( "Attack", xOffset );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 5.0f;

		pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxSquad->m_xBehaviourData.m_fChaneToRehideIfCloseFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f", &Changed );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 15.0f;

		pxDialogue = new Asura_Dialog( "Hide", xOffset );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 27.0f;
		xOffset.y += Asura_Dialog::LargeSpacing();

		pxDialogue = new Asura_Dialog( "In Light :", xOffset );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0f;
		pxDialogue = new Asura_Dialog( "Attack", xOffset );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 5.0f;

		pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxSquad->m_xBehaviourData.m_fChaneToRehideIfInLightFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f", &Changed );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 15.0f;

		pxDialogue = new Asura_Dialog( "Hide", xOffset );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 27.0f;
		xOffset.y += Asura_Dialog::LargeSpacing();	

		pxDialogue = new Asura_Dialog( "After Attack :", xOffset );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 7.0f;
		pxDialogue = new Asura_Dialog( "Attack", xOffset );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 5.0f;

		pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxSquad->m_xBehaviourData.m_fChaneToRehideAfterAttackFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f", &Changed );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x += 15.0f;

		pxDialogue = new Asura_Dialog( "Hide", xOffset );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.x -= 27.0f;
		xOffset.y += Asura_Dialog::LargeSpacing();


	}
}

void ToolsAVP_Entity_Squad_FaceHugger::OnSelection()
{
	Validate();

	m_bRefreshOverlay = false;
	PARENT::OnSelection();
	Asura_Assert(s_pxCurrentActive == NULL, "Selection without Deslection");
	s_pxCurrentActive = this;

}

void ToolsAVP_Entity_Squad_FaceHugger::OnDeselection()
{
	PARENT::OnDeselection();
	s_pxCurrentActive = NULL;
}

void ToolsAVP_Entity_Squad_FaceHugger::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( m_bRefreshOverlay )
	{
		DeleteOverlay();
		CreateOverlay();
		m_bRefreshOverlay = false;
	}
}

bool ToolsAVP_Entity_Squad_FaceHugger::HandlesThisMessage( int iMessage )
{
	if ( !GetFaceHuggerSquad()->IsDefaultPetSquad() && iMessage == AVP_MSGID_SET_FACEHUGGER_SQUAD_BEHAVIOUR )
	{
		return true;
	}
	else
	{
		return PARENT::HandlesThisMessage( iMessage );
	}
}

void ToolsAVP_Entity_Squad_FaceHugger::CreateDefault()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Default FaceHugger Squad", "Default FaceHugger Squad", uGuid );
	if ( !pxToolsData ) { return; }

	AVP_ServerEntity_Squad_FaceHugger* pxSquad = new AVP_ServerEntity_Squad_FaceHugger( uGuid );
	if ( !pxSquad ) { return; }
	pxSquad->ToolsData = pxToolsData;
	pxSquad->SetIsDefaultSquad( true );
	AVP_ServerEntity_Squad_FaceHugger::s_pxDefaultSquad = pxSquad;

	ToolsAVP_Entity_Squad_FaceHugger* pxToolsObject = new ToolsAVP_Entity_Squad_FaceHugger( pxSquad );
	if ( !pxToolsObject ) { return; }
	Tools_Object_System::SelectObject( pxToolsObject );

	pxToolsObject->Position		= *Asura_Camera::GetPosition();
	pxToolsObject->Orientation  = Asura_Maths::IdentityMatrix3x3;
	pxToolsObject->m_uGuid		= uGuid;

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new default FaceHugger squad." );
	Tools_SaveAsura::SetEntityModified();
}

void ToolsAVP_Entity_Squad_FaceHugger::CreateDefaultPet()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Default FaceHugger Pet Squad", "Default FaceHugger Pet Squad", uGuid );
	if ( !pxToolsData ) { return; }

	AVP_ServerEntity_Squad_FaceHugger* pxSquad = new AVP_ServerEntity_Squad_FaceHugger( uGuid );
	if ( !pxSquad ) { return; }
	pxSquad->ToolsData = pxToolsData;
	pxSquad->SetIsDefaultPetSquad( true );

	int iDataSize = sizeof(AVP_Msg_SetAITargetPriorities) + sizeof(AVP_Msg_SetAITargetPriorities::TPData);
	Asura_Message* pxMessage = Asura_Network::CreateMessage( AVP_MSGID_SET_AI_TARGET_PRIORITIES, iDataSize );
	if ( pxMessage )
	{
		AVP_Msg_SetAITargetPriorities* pxData = static_cast< AVP_Msg_SetAITargetPriorities* >( pxMessage->GetData() );
		pxData->Initialise();
		pxData->m_uNumEntries = 1;
		AVP_Msg_SetAITargetPriorities::TPData& xTPData = pxData->GetFirstTPDataPtr()[0];

		xTPData.m_iPriority = AVP_AI_TargetPriorityData::LOW;
		xTPData.m_iSpecificGuidType = AVP_AI_TargetPriorityData::ACTOR;
		xTPData.m_iTargetType = AVP_AI_TargetPriorityData::CIVILIAN;
		xTPData.m_uSpecificGuid = ASURA_GUID_UNREGISTERED;
		pxSquad->HandleMessage(pxMessage);
		Asura_Network::DestroyMessage(pxMessage);
	}

	AVP_ServerEntity_Squad_FaceHugger::s_pxDefaultPetSquad = pxSquad;

	ToolsAVP_Entity_Squad_FaceHugger* pxToolsObject = new ToolsAVP_Entity_Squad_FaceHugger( pxSquad );
	if ( !pxToolsObject ) { return; }
	Tools_Object_System::SelectObject( pxToolsObject );

	pxToolsObject->Position		= *Asura_Camera::GetPosition();
	pxToolsObject->Orientation  = Asura_Maths::IdentityMatrix3x3;
	pxToolsObject->m_uGuid		= uGuid;

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new default FaceHugger Pet squad." );
	Tools_SaveAsura::SetEntityModified();
}


Tools_Object* ToolsAVP_Entity_Squad_FaceHugger::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Entity_Squad_FaceHugger( pxEntity );
}


void ToolsAVP_Entity_Squad_FaceHugger::Create()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Face-Hugger Squad", "Face-Hugger Squad", uGuid );
	if ( !pxToolsData ) { return; }

	AVP_ServerEntity_Squad_FaceHugger* pxSquad = new AVP_ServerEntity_Squad_FaceHugger( uGuid );
	if ( !pxSquad ) { return; }
	pxSquad->ToolsData = pxToolsData;

	ToolsAVP_Entity_Squad_FaceHugger* pxToolsObject = new ToolsAVP_Entity_Squad_FaceHugger( pxSquad );
	if ( !pxToolsObject ) { return; }
	Tools_Object_System::SelectObject( pxToolsObject );

	pxToolsObject->Position		= *Asura_Camera::GetPosition();
	pxToolsObject->Orientation  = Asura_Maths::IdentityMatrix3x3;
	pxToolsObject->m_uGuid		= uGuid;

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Face-Hugger squad." );
	Tools_SaveAsura::SetEntityModified();
}


void ToolsAVP_Entity_Squad_FaceHugger::SelectGPVolume()
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_GamePlayVolume, AddGPVolume );
	Tools_Object_System::ChooseObject( "Select A Gameplay Volume", pxChooseObject );
}

void ToolsAVP_Entity_Squad_FaceHugger::RemoveSelectedGPVolume()
{
	Asura_Assert(s_pxCurrentActive, "Current Facehugger Squad not set");
	if (s_pxCurrentActive)
	{
		AVP_ServerEntity_Squad_FaceHugger* pxSquadFacehugger = s_pxCurrentActive->GetFaceHuggerSquad();
		Asura_LinkList_It<Asura_Guid> xIt(pxSquadFacehugger->m_xGPVolumesIDList);
		int iIndex = s_pxCurrentActive->m_iPGVolumeSelectedIndex;
		while ( !xIt.Done() && iIndex-- > 0 )
		{
			xIt.Next();
		}

		Asura_Assert(!xIt.Done(), "Volume to remove not found");
		if (!xIt.Done())
		{
			xIt.DeleteCurrent();
			Tools_SaveAsura::SetEntityModified();
		}

	}

	s_pxCurrentActive->RequestRefreshOverlay();

}

void ToolsAVP_Entity_Squad_FaceHugger::AddGPVolume(Asura_Guid uGuid)
{
	Asura_Assert(s_pxCurrentActive, "Current Facehugger Squad not set");
	if (s_pxCurrentActive && !s_pxCurrentActive->GetFaceHuggerSquad()->m_xGPVolumesIDList.Find(uGuid) && uGuid != ASURA_GUID_UNREGISTERED)
	{
		s_pxCurrentActive->GetFaceHuggerSquad()->m_xGPVolumesIDList.Push_Back(uGuid);
		s_pxCurrentActive->RequestRefreshOverlay();
		Tools_SaveAsura::SetEntityModified();
	}
}

AVP_ServerEntity_Squad_FaceHugger* ToolsAVP_Entity_Squad_FaceHugger::GetFaceHuggerSquad() const
{
	return static_cast< AVP_ServerEntity_Squad_FaceHugger* >( Entity );
}


Asura_List_Dialog* ToolsAVP_Entity_Squad_FaceHugger::CreateGPVolumeList( const Asura_Vector_2& xOffset )
{

	DestroyGPVolumeList();

	AVP_ServerEntity_Squad_FaceHugger* pxSquadFacehugger = GetFaceHuggerSquad();

	m_xArray_pszGPVolumeNames.SetSize(pxSquadFacehugger->m_xGPVolumesIDList.Size());

	int index = 0;
	for(Asura_LinkList_It<Asura_Guid> xIt(&pxSquadFacehugger->m_xGPVolumesIDList); !xIt.Done(); xIt.Next())
	{
		Asura_Guid uGuid = *xIt.GetCurrent();
		ToolsAVP_Entity_GamePlayVolume *pxGPVolume = ToolsAVP_Entity_GamePlayVolume::FindEntity(uGuid);
		Asura_Assert(pxGPVolume, "GamePlayVolume not found");
		if (pxGPVolume)
		{
			m_xArray_pszGPVolumeNames[index++] = Asura_StringUtil::Duplicate(pxGPVolume->GetName());
		}
	}

	if (index == 0)
	{
		m_xArray_pszGPVolumeNames.SetSize(1);
		m_xArray_pszGPVolumeNames[index++] = Asura_StringUtil::Duplicate(" ");
	}

	char ** apszGPVolumeNames = &m_xArray_pszGPVolumeNames[0];

	return new Asura_List_Dialog(xOffset, apszGPVolumeNames, &m_iPGVolumeSelectedIndex, 3, m_xArray_pszGPVolumeNames.GetSize(),  NULL);
}

void ToolsAVP_Entity_Squad_FaceHugger::DestroyGPVolumeList()
{

	for (u_int index = 0; index < m_xArray_pszGPVolumeNames.GetSize(); ++index )
	{
		delete [] m_xArray_pszGPVolumeNames[index];
	}
	m_xArray_pszGPVolumeNames.SetSize(0);

}


void ToolsAVP_Entity_Squad_FaceHugger::BehaviourDataChanged( int iThis )
{
	ToolsAVP_Entity_Squad_FaceHugger* pxThis = reinterpret_cast< ToolsAVP_Entity_Squad_FaceHugger* >( iThis );
	pxThis->SetEntityModified();
	pxThis->RequestRefreshOverlay();
}

void ToolsAVP_Entity_Squad_FaceHugger::Validate()
{
	AVP_ServerEntity_Squad_FaceHugger* pxSquadFacehugger = GetFaceHuggerSquad();

	for(Asura_LinkList_It<Asura_Guid> xIt(&pxSquadFacehugger->m_xGPVolumesIDList); !xIt.Done(); xIt.Next())
	{
		Asura_Guid uGuid = *xIt.GetCurrent();
		ToolsAVP_Entity_GamePlayVolume *pxGPVolume = ToolsAVP_Entity_GamePlayVolume::FindEntity(uGuid);
		if (!pxGPVolume)
		{
			xIt.DeleteCurrent();
			Tools_MessageLog::AddMessage( "Face-Hugger Squad", "Missing Dependent GamePlayVolume", new Tools_MessageLogAction_GotoEntity( GetGuid() ), "Could not retrieve GamePlayVolume %u", uGuid );
		}
	}


}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
