//------------------------------------------------------------------------------
// File   : ToolsAVP_Entity_GamePlayVolume.cpp
// Author : Gareth Thomas
// Date   : 22 / 10 / 07
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Include files...
//------------------------------------------------------------------------------
// Always include Asura Core first
#include "Core/Asura.h"

// This Files Header
#include "Entity/GamePlayVolume/ToolsAVP_Entity_GamePlayVolume.h"

// The Game Entities Include Files
#include "Entity/GamePlayVolume/AVP_ServerEntity_GamePlayVolume.h"

// Asura include files
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Render.h"
#include "Volume/Asura_Volume.h"
#include "Volume/Asura_Volume_Cuboid.h"
#include "Volume/Asura_Volume_Sphere.h"
#include "Volume/Asura_Volume_Cylinder.h"
#include "Volume/Asura_VolumeGroup.h"

// AVP Include files
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Spawners/Alien/AVP_ServerEntity_AlienSpawnPoint.h"

// Tools Include Files
#include "Blueprint/Tools_Mode_Blueprint.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"
#include "Objects/Axon/Tools_Object_Axon_ChooseObjects.h"

//------------------------------------------------------------------------------
// Constants, Defines, Statics and Globals
//------------------------------------------------------------------------------
Asura_Hashed_List<ToolsAVP_Entity_GamePlayVolume>	ToolsAVP_Entity_GamePlayVolume::s_xList;

ToolsAVP_Entity_GamePlayVolume*						ToolsAVP_Entity_GamePlayVolume::s_pxGPvolume						= NULL;
bool												ToolsAVP_Entity_GamePlayVolume::s_bMoveVolumeWithObject				= true;
bool												ToolsAVP_Entity_GamePlayVolume::s_bAdd								= true;
bool												ToolsAVP_Entity_GamePlayVolume::s_bShowVolumes						= false;
bool												ToolsAVP_Entity_GamePlayVolume::s_bShowSelectedVolume				= false;
bool												ToolsAVP_Entity_GamePlayVolume::s_bEntitesOutsideAreHidden			= false;
bool												ToolsAVP_Entity_GamePlayVolume::s_bHideEntitiesOutside				= false;
bool												ToolsAVP_Entity_GamePlayVolume::s_bIncludeLinkedVolumes				= false;
bool												ToolsAVP_Entity_GamePlayVolume::s_bLinkedEntitesOutsideAreHidden	= false;
bool												ToolsAVP_Entity_GamePlayVolume::s_bVolumesEdited					= false;
u_int												ToolsAVP_Entity_GamePlayVolume::s_uCurrentTab						= ToolsAVP_Entity_GamePlayVolume::TAB_GENERAL;

const char* ToolsAVP_Entity_GamePlayVolume::s_aszTabNames[ TAB_END_ENUM ] =
{
	"General",
	"Entity Control"
};

const char* ToolsAVP_Entity_GamePlayVolume::s_aszFlagNames[AVP_ServerEntity_GamePlayVolume::FLAG_END_ENUM] =
{
	"All Types Listed Below",
	"Triggers - (Volume & Multi Damage Volume)",
	"Lights",
	"Physics Objects",
	"Predators",
	"Aliens",
	"All Human Types",
	"Lifts",
	"PFX",
	"Sounds",
	"Face Huggers",
	"Sentry guns",
	"All Actor Types",
	"Civilians",
	"Androids",
	"Marines",
	"Containers",
};


//------------------------------------------------------------------------------
// Function Implementations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: Constructor
//------------------------------------------------------------------------------
ToolsAVP_Entity_GamePlayVolume::ToolsAVP_Entity_GamePlayVolume(Asura_Entity* pxEntity) : PARENT(pxEntity)
, m_bAreaChanged(false)
, m_bLinkChanged(false)
, m_pxTabsDialog(NULL)
, m_iBitFlags(0)
{
	Type	= TOT_Entity_GamePlayVolume;
	Mesh	= Asura_Mesh_System::GetMesh("ToolsShape_GamePlayVolume");

	m_pxVolumeController	= 0;

	for (int i = 0; i < MAX_LINKED_GAMEPLAYVOLUMES; i += 1)
	{
		m_puLinkedGamePlayVolumes[i] = ASURA_GUID_UNREGISTERED;
	}

	AVP_ServerEntity_GamePlayVolume* pxAVPEntity = (AVP_ServerEntity_GamePlayVolume*)pxEntity;

	if (pxAVPEntity)
	{
		Position = pxAVPEntity->GetPosition();
		m_xPreviousPosition = Position;

		m_uNumLinkedGamePlayVolumes = pxAVPEntity->GetNumLinkedGamePlayVolumes();

		for (int i = 0; i < MAX_LINKED_GAMEPLAYVOLUMES; i += 1)
		{
			m_puLinkedGamePlayVolumes[i] = pxAVPEntity->GetLinkedGamePlayVolumeGuid(i);
		}

		m_iGamePlayAreaType = (int)pxAVPEntity->GetGamePlayAreaType();

		m_iBitFlags = pxAVPEntity->GetBitFlags();
	}

	m_pxAlienSpawnerGuids = new List<Asura_Guid>;

	s_xList.AddToList(this);

}
/// End of function "ToolsAVP_Entity_GamePlayVolume.cpp::".

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: Deconstructor
//------------------------------------------------------------------------------
ToolsAVP_Entity_GamePlayVolume::~ToolsAVP_Entity_GamePlayVolume(void)
{
	if (m_pxAlienSpawnerGuids)
	{
		delete (m_pxAlienSpawnerGuids);
	}

	if (m_pxVolumeController)
	{
		delete (m_pxVolumeController);
		m_pxVolumeController = NULL;
	}

	s_xList.RemoveFromList(this);
}
/// End of function "ToolsAVP_Entity_GamePlayVolume.cpp::".

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: HandlesThisMessage
//------------------------------------------------------------------------------
bool ToolsAVP_Entity_GamePlayVolume::HandlesThisMessage(int iMessageID)
{
	switch (iMessageID)
	{
		case AVP_MSGID_ACTIVATE_SELECTED_ENTITIES:
		case AVP_MSGID_DEACTIVATE_SELECTED_ENTITIES:
		{
			return true;
		}
	}

	return PARENT::HandlesThisMessage(iMessageID);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: Verify
//-----------------------------------------------------------------------------
bool ToolsAVP_Entity_GamePlayVolume::Verify( const bool bAutomatic )
{
	bool bOk = true;

	AVP_ServerEntity_GamePlayVolume* pxGamePlayVolume = AVP_ServerEntity_GamePlayVolume::FindEntity( GetGuid() );

	if ( pxGamePlayVolume )
	{
		if ( pxGamePlayVolume->GetNumberOfVolumes() == 0)
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Game Play Volume", new Tools_MessageLogAction_GotoEntity( GetGuid() ), "Game Play Volume %u has NO VOLUME", GetGuid() );
		}
	}

	return bOk;
}

void ToolsAVP_Entity_GamePlayVolume::AddFilesToExportList( List<Asura_String>& xNameList ) const
{
	PARENT::AddFilesToExportList(xNameList);
	
	AVP_ServerEntity_GamePlayVolume* pxGamePlayVolume = AVP_ServerEntity_GamePlayVolume::FindEntity( GetGuid() );
	if ( pxGamePlayVolume && pxGamePlayVolume->GetGamePlayAreaType() < AVP_ServerEntity_GamePlayVolume::MAX_AREA_TYPES )
	{
		const char* pchAreaTypeName = AVP_ServerEntity_GamePlayVolume::GetAreaTypeNames()[ pxGamePlayVolume->GetGamePlayAreaType() ];
		const Asura_Hash_ID	uBlueprintHash	= Asura_GetHashID( pchAreaTypeName );
		if( uBlueprintHash != ASURA_HASH_ID_UNSET )
		{
			Tools_Mode_Blueprint::AddBlueprintFilesToExportList( xNameList, uAVP_BLUEPRINT_TYPE_GAMEPLAYVOLUMES, uBlueprintHash, false );

#ifdef ASURA_ASSERT
			const Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_GAMEPLAYVOLUMES, uBlueprintHash);
			Asura_Assertf( pxBlueprint, ("Failed To find blueprint in GamePlayVolume (guid:%d) with name: %s", GetGuid(), pchAreaTypeName) );
#endif
		}
	}
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: Project_Create
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::Project_Create(void)
{
	ToolsAVP_Entity_GamePlayVolume::Create();
}
/// End of function "ToolsAVP_Entity_GamePlayVolume.cpp::".

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: Create
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::Create(void)
{
	const Asura_Guid	uGuid	= Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = Tools_Entity_Object::CreateToolsData("Game Play Volume", "Game Play Volume");

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_GamePlayVolume::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_GamePlayVolume* pxServerObject = new AVP_ServerEntity_GamePlayVolume(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_GamePlayVolume::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_GamePlayVolume*	pxToolsObject	= new ToolsAVP_Entity_GamePlayVolume(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_GamePlayVolume::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created New Game Play Volume");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}
/// End of function "ToolsAVP_Entity_GamePlayVolume.cpp::".

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: Create
//------------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_GamePlayVolume::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Entity_GamePlayVolume(pxEntity);
}
/// End of function "ToolsAVP_Entity_GamePlayVolume.cpp::".

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: OnSelection
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::OnSelection(void)
{
	s_pxGPvolume = this;

	PARENT::OnSelection();

	//UnhideEverything();
	//HideOutsideEntities(s_bEntitesOutsideAreHidden);
	
// 	if (s_bHideEntitiesOutside && s_bIncludeLinkedVolumes)
// 	{
// 		HideLinkedOutsideEntities();
// 	}
// 	else
// 	{
// 		ShowLinkedOutSideEntities();
// 	}

}
/// End of function "ToolsAVP_Entity_GamePlayVolume.cpp::".

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: OnDeselection
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::OnDeselection(void)
{
	PARENT::OnDeselection();

	s_pxGPvolume = NULL;
}
/// End of function "ToolsAVP_Entity_GamePlayVolume.cpp::".

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: UpdatePositionInAsuraData
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	AVP_ServerEntity_GamePlayVolume* pxAsuraEntity = static_cast<AVP_ServerEntity_GamePlayVolume*>(Entity);

	if (s_bMoveVolumeWithObject)
	{
		Asura_Vector_3 xOffset = Position - m_xPreviousPosition;

		for (int i = 0; i < pxAsuraEntity->m_iNumVolumes; i += 1)
		{
			pxAsuraEntity->DisplaceVolumes(xOffset);
		}

		if (m_pxVolumeController)
		{
			m_pxVolumeController->MoveVolumeObjects(xOffset);
		}
	}

	//	Update the position if it has changed
	if	(pxAsuraEntity->m_xPosition != Position)
	{
		pxAsuraEntity->SetPosition(Position);

		Changed	= true;
	}

	m_xPreviousPosition = Position;

	PARENT::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}
/// End of function "ToolsAVP_Entity_GamePlayVolume.cpp::".

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: Update
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::Update(void)
{
	for (Asura_Hashed_List_It<AVP_ServerEntity_AlienSpawnPoint> it(AVP_ServerEntity_AlienSpawnPoint::GetList()); !it.Done(); it.Next())
	{
		AVP_ServerEntity_AlienSpawnPoint* pxSpawner = it.GetCurrent();

		if (pxSpawner)
		{
			AVP_ServerEntity_GamePlayVolume* pxGPVolume = (AVP_ServerEntity_GamePlayVolume*)Entity;

			bool bAdd = false;

			if (pxGPVolume->Contains(pxSpawner->GetPosition()))
			{
				bAdd = true;
			}
			else
			{
				bAdd = false;
			}

			if (bAdd)
			{
				// If it doesn't contain our spawner then add it
				if (!GetAlienSpawnerList()->contains(pxSpawner->GetGuid()))
				{
					GetAlienSpawnerList()->add_entry(pxSpawner->GetGuid());
				}
			}
			else
			{
				// If it does contain our spawner then delete it
				if (GetAlienSpawnerList()->contains(pxSpawner->GetGuid()))
				{
					GetAlienSpawnerList()->delete_entry(pxSpawner->GetGuid());
				}
			}

			// This is wrong here! TODO: put to the right place (AN 18/12/07)
//			Tools_SaveAsura::SetEntityModified(); 
		}
	}

	bool bRemoved = false;

	// Clean up any links that have been destroyed!
	for (u_int i = 0; i < MAX_LINKED_GAMEPLAYVOLUMES; i += 1)
	{
		if (m_puLinkedGamePlayVolumes[i] != ASURA_GUID_UNREGISTERED)
		{
			Tools_Entity_Object* pxGPVolume = ToolsAVP_Entity_GamePlayVolume::FindEntity(m_puLinkedGamePlayVolumes[i]);

			if (!pxGPVolume || bRemoved)
			{
				bRemoved = true;
				m_bLinkChanged = true;

				if (i >= (MAX_LINKED_GAMEPLAYVOLUMES - 1))
				{
					m_puLinkedGamePlayVolumes[i] = ASURA_GUID_UNREGISTERED;
				}
				else
				{
					m_puLinkedGamePlayVolumes[i] = m_puLinkedGamePlayVolumes[i + 1];
				}
			}
		}
	}

	u_int uCount = 0;

	for (u_int i = 0; i < MAX_LINKED_GAMEPLAYVOLUMES; i += 1)
	{
		if (m_puLinkedGamePlayVolumes[i] != ASURA_GUID_UNREGISTERED)
		{
			uCount += 1;
		}
	}

	m_uNumLinkedGamePlayVolumes = uCount;

	// As other GP volumes can change our links update the game side entities.
	if (m_bLinkChanged)
	{
		AVP_ServerEntity_GamePlayVolume* pxAVPEntity = (AVP_ServerEntity_GamePlayVolume*)Entity;

		if (pxAVPEntity)
		{
			pxAVPEntity->SetNumLinkedGamePlayVolumes(m_uNumLinkedGamePlayVolumes);

			for (u_int i = 0; i < MAX_LINKED_GAMEPLAYVOLUMES; i += 1)
			{
				pxAVPEntity->SetLinkedGamePlayVolumeGuid(i, m_puLinkedGamePlayVolumes[i]);
			}
		}

		m_bLinkChanged = false;

		if (this == s_pxGPvolume)
		{
			DeleteOverlay();
			CreateOverlay();
		}

		Tools_SaveAsura::SetEntityModified();
	}

// 	if (this == s_pxGPvolume)
// 	{
// 		ProcessShowHide();
// 	}
}
//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: UpdateWhileSelected
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::UpdateWhileSelected(void)
{
	if (m_bAreaChanged)
	{
		AVP_ServerEntity_GamePlayVolume* pxAVPEntity = (AVP_ServerEntity_GamePlayVolume*)Entity;

		if (pxAVPEntity)
		{
			pxAVPEntity->SetGamePlayAreaType((u_int)m_iGamePlayAreaType);
		}

		DeleteOverlay();
		CreateOverlay();

		Tools_SaveAsura::SetEntityModified();

		m_bAreaChanged = false;
	}

	if (Changed)
	{
		AVP_ServerEntity_GamePlayVolume* pxAVPEntity = (AVP_ServerEntity_GamePlayVolume*)Entity;

		if (pxAVPEntity)
		{
			pxAVPEntity->SetBitFlags(m_iBitFlags);
			Changed = false;
		}

		Tools_SaveAsura::SetEntityModified();
	}

	if (s_bVolumesEdited)
	{
		s_bVolumesEdited = false;
		Tools_SaveAsura::SetEntityModified();
	}
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: UnhideEverything
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::ProcessShowHide(void)
{
// 	if (s_bHideEntitiesOutside && !s_bEntitesOutsideAreHidden)
// 	{
// 		UnhideEverything();
// 		HideOutsideEntities(true);
// 	}
// 
// 	if (!s_bHideEntitiesOutside && s_bEntitesOutsideAreHidden)
// 	{
// 		UnhideEverything();
// 		HideOutsideEntities(false);
// 	}
// 
// 	if ((s_bIncludeLinkedVolumes) && (s_bHideEntitiesOutside))
// 	{
// 		if (!s_bLinkedEntitesOutsideAreHidden)
// 		{
// 			// Hide linked inside entities
// 			HideLinkedOutsideEntities();
// 		}
// 	}
// 	else
// 	{
// 		if (!s_bHideEntitiesOutside)
// 		{
// 			UnhideEverything();
// 			s_bLinkedEntitesOutsideAreHidden = false;
// 
// 		}
// 		else
// 		{
// 			ShowLinkedOutSideEntities();
// 		}
// 	}
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: UnhideEverything
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::UnhideEverything(void)
{
// 	for (Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList()); !xIt.Done(); xIt.Next())
// 	{
// 		Tools_Entity_Object* pxObject = xIt.GetCurrent();
// 
// 		if (pxObject)
// 		{
// 			if (pxObject->Type == TOT_Entity_GamePlayVolume)
// 			{
// 				continue;
// 			}
// 
// 			pxObject->SetHiddenFlag(ObjectHideFlag_User, false);
// 		}
// 	}
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: HideOutsideEntities
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::HideOutsideEntities(bool bVal)
{
// 	for (Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList()); !xIt.Done(); xIt.Next())
// 	{
// 		Tools_Entity_Object* pxObject = xIt.GetCurrent();
// 
// 		if (pxObject)
// 		{
// 			if (pxObject->Type == TOT_Entity_GamePlayVolume)
// 			{
// 				continue;
// 			}
// 
// 			AVP_ServerEntity_GamePlayVolume* pxGamePlayVol = (AVP_ServerEntity_GamePlayVolume*)Entity;
// 
// 			if (pxGamePlayVol)
// 			{
// 				if (!pxGamePlayVol->Contains(pxObject->Position))
// 				{
// 					pxObject->SetHiddenFlag(ObjectHideFlag_User, bVal);
// 				}
// 			}
// 		}
// 	}
// 
// 	s_bEntitesOutsideAreHidden = bVal;

}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: 
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::HideLinkedOutsideEntities(void)
{
// 	for (Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList()); !xIt.Done(); xIt.Next())
// 	{
// 		Tools_Entity_Object* pxObject = xIt.GetCurrent();
// 
// 		if (pxObject)
// 		{
// 			if (pxObject->Type == TOT_Entity_GamePlayVolume)
// 			{
// 				continue;
// 			}
// 
// 			AVP_ServerEntity_GamePlayVolume* pxGamePlayVol = (AVP_ServerEntity_GamePlayVolume*)Entity;
// 
// 			if (pxGamePlayVol)
// 			{
// 				bool bOutside = true;
// 
// 				if (pxGamePlayVol->Contains(pxObject->Position))
// 				{
// 					bOutside = false;
// 				}
// 
// 				if (bOutside)
// 				{
// 					for (u_int i = 0; i < m_uNumLinkedGamePlayVolumes; i += 1)
// 					{
// 						AVP_ServerEntity_GamePlayVolume* pxLinkedGPVol = AVP_ServerEntity_GamePlayVolume::FindEntity(GetLinkedGamePlayVolumeGuid(i));
// 
// 						if (pxLinkedGPVol)
// 						{
// 							if (pxLinkedGPVol->Contains(pxObject->Position))
// 							{
// 								bOutside = false;
// 								break;
// 							}
// 						}
// 					}
// 				}
// 
// 				if (bOutside)
// 				{
// 					pxObject->SetHiddenFlag(ObjectHideFlag_User, true);
// 				}
// 				else
// 				{
// 					pxObject->SetHiddenFlag(ObjectHideFlag_User, false);
// 				}
// 			}
// 		}
// 	}
// 
// 	s_bLinkedEntitesOutsideAreHidden = true;
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: ShowLinkedOutSideEntities
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::ShowLinkedOutSideEntities(void)
{
// 	for (Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList()); !xIt.Done(); xIt.Next())
// 	{
// 		Tools_Entity_Object* pxObject = xIt.GetCurrent();
// 
// 		if (pxObject)
// 		{
// 			if (pxObject->Type == TOT_Entity_GamePlayVolume)
// 			{
// 				continue;
// 			}
// 
// 			AVP_ServerEntity_GamePlayVolume* pxGamePlayVol = (AVP_ServerEntity_GamePlayVolume*)Entity;
// 
// 			if (pxGamePlayVol)
// 			{
// 				if (!pxGamePlayVol->Contains(pxObject->Position))
// 				{
// 					pxObject->SetHiddenFlag(ObjectHideFlag_User, true);
// 				}
// 				else
// 				{
// 					pxObject->SetHiddenFlag(ObjectHideFlag_User, false);
// 				}
// 			}
// 		}
// 	}
// 
// 	s_bLinkedEntitesOutsideAreHidden = false;

}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: Render
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::Render()
{
	PARENT::Render();

	if ((s_bShowVolumes) || ((s_bShowSelectedVolume) && (this == s_pxGPvolume)))
	{
		AVP_ServerEntity_GamePlayVolume* pxAVPEntity = (AVP_ServerEntity_GamePlayVolume*)Entity;

		if (pxAVPEntity)
		{
			for (int i = 0; i < pxAVPEntity->m_iNumVolumes; ++i)
			{
				switch (pxAVPEntity->GetVolume(i)->GetClassification())
				{
					case ASURA_VOLUME_CLASSIFICATION_CUBOID:
					{
						Asura_Volume_Cuboid*	pxCubeVolume	= static_cast<Asura_Volume_Cuboid*>(pxAVPEntity->GetVolume(i));
						Asura_Bounding_Box		xRelativeBox	= *pxCubeVolume->GetVolumeBox();
						Asura_Vector_3			xCentre			= xRelativeBox.CalculateCentre();

						xRelativeBox.Displace(-xCentre);

						Asura_Matrix_3x3 xMat;
						pxCubeVolume->GetVolumeOrient()->ToMatrix(xMat);

						u_int uEdgeColour, uBodyColour;

						switch ( m_iGamePlayAreaType )
						{
							case AVP_ServerEntity_GamePlayVolume::TIGHT_CORRIDOR:				uEdgeColour = 0xffff8888; uBodyColour = 0x66ff5555; break;
							case AVP_ServerEntity_GamePlayVolume::LARGE_CORRIDOR:				uEdgeColour = 0xff55ff55; uBodyColour = 0x6655ff55; break;
							case AVP_ServerEntity_GamePlayVolume::SMALL_ROOM:					uEdgeColour = 0xff8888ff; uBodyColour = 0x665555ff; break;
							case AVP_ServerEntity_GamePlayVolume::LARGE_ROOM:					uEdgeColour = 0xffffff00; uBodyColour = 0x66ffff55; break;
							case AVP_ServerEntity_GamePlayVolume::OPEN_SPACE:					uEdgeColour = 0xffff00ff; uBodyColour = 0x66ff55ff; break;
							case AVP_ServerEntity_GamePlayVolume::IGNORE_GPV:					uEdgeColour = 0xff000000; uBodyColour = 0x66111111; break;
							case AVP_ServerEntity_GamePlayVolume::NONE_ASSIGNED:	default:	uEdgeColour = 0xff00ffff; uBodyColour = 0x6655ffff; break;
						}

						Tools_Render::RenderCube(xRelativeBox, xCentre, xMat, uEdgeColour, true);
						Tools_Render::RenderCube(xRelativeBox, xCentre, xMat, uBodyColour, false);

					}break;
					case ASURA_VOLUME_CLASSIFICATION_SPHERE:
					{
						Asura_Volume_Sphere* pxSphereVolume = static_cast<Asura_Volume_Sphere*>(pxAVPEntity->GetVolume(i));

						Asura_Matrix_3x3 xMat;
						pxSphereVolume->GetVolumeOrient()->ToMatrix(xMat);

						float			fRadius = *pxSphereVolume->GetRadiusPtr();
						Asura_Vector_3 xExtents = Asura_Vector_3(fRadius, fRadius, fRadius);

						u_int uEdgeColour, uBodyColour;

						switch ( m_iGamePlayAreaType )
						{
							case AVP_ServerEntity_GamePlayVolume::TIGHT_CORRIDOR:				uEdgeColour = 0xffff8888; uBodyColour = 0xffff5555; break;
							case AVP_ServerEntity_GamePlayVolume::LARGE_CORRIDOR:				uEdgeColour = 0xff55ff55; uBodyColour = 0xff55ff55; break;
							case AVP_ServerEntity_GamePlayVolume::SMALL_ROOM:					uEdgeColour = 0xff8888ff; uBodyColour = 0xff5555ff; break;
							case AVP_ServerEntity_GamePlayVolume::LARGE_ROOM:					uEdgeColour = 0xffffff00; uBodyColour = 0xffffff55; break;
							case AVP_ServerEntity_GamePlayVolume::OPEN_SPACE:					uEdgeColour = 0xffff00ff; uBodyColour = 0xffff55ff; break;
							case AVP_ServerEntity_GamePlayVolume::IGNORE_GPV:					uEdgeColour = 0xff222222; uBodyColour = 0x66333333; break;
							case AVP_ServerEntity_GamePlayVolume::NONE_ASSIGNED:	default:	uEdgeColour = 0xff00ffff; uBodyColour = 0xff55ffff; break;
						}

						Tools_Render::RenderSpheroid(*pxSphereVolume->GetPositionPtr(), xMat, xExtents, uEdgeColour, true);
						Tools_Render::RenderSpheroid(*pxSphereVolume->GetPositionPtr(), xMat, xExtents, uBodyColour, false);

					}break;
					case ASURA_VOLUME_CLASSIFICATION_CYLINDER:
					{
						Asura_Volume_Cylinder* pxCylinderVolume = static_cast<Asura_Volume_Cylinder*>(pxAVPEntity->GetVolume(i));

						Asura_Matrix_3x3 xMat;
						pxCylinderVolume->GetVolumeOrient()->ToMatrix(xMat);

						Asura_Vector_3 xUp = Asura_Vector_3(xMat.e12, xMat.e22, xMat.e32);
						Asura_Vector_3 xStart = *pxCylinderVolume->GetPositionPtr();
						Asura_Vector_3 xEnd = *pxCylinderVolume->GetPositionPtr() + (-xUp * *pxCylinderVolume->GetHeightPtr());

						u_int uEdgeColour, uBodyColour;

						switch ( m_iGamePlayAreaType )
						{
							case AVP_ServerEntity_GamePlayVolume::TIGHT_CORRIDOR:				uEdgeColour = 0xffff8888; uBodyColour = 0x66ff5555; break;
							case AVP_ServerEntity_GamePlayVolume::LARGE_CORRIDOR:				uEdgeColour = 0xff55ff55; uBodyColour = 0x6655ff55; break;
							case AVP_ServerEntity_GamePlayVolume::SMALL_ROOM:					uEdgeColour = 0xff8888ff; uBodyColour = 0x665555ff; break;
							case AVP_ServerEntity_GamePlayVolume::LARGE_ROOM:					uEdgeColour = 0xffffff00; uBodyColour = 0x66ffff55; break;
							case AVP_ServerEntity_GamePlayVolume::OPEN_SPACE:					uEdgeColour = 0xffff00ff; uBodyColour = 0x66ff55ff; break;
							case AVP_ServerEntity_GamePlayVolume::IGNORE_GPV:					uEdgeColour = 0xff222222; uBodyColour = 0x66333333; break;
							case AVP_ServerEntity_GamePlayVolume::NONE_ASSIGNED:	default:	uEdgeColour = 0xff00ffff; uBodyColour = 0x6655ffff; break;
						}

						Tools_Render::RenderCylinder(xStart, xEnd, *pxCylinderVolume->GetRadiusPtr(), uEdgeColour, true);
						Tools_Render::RenderCylinder(xStart, xEnd, *pxCylinderVolume->GetRadiusPtr(), uBodyColour, false);

					}break;
					default:
					{
						Asura_Assert(0,"ToolsAVP_Entity_GamePlayVolume :: Render - Unkown Volume Classification");

					}break;
				}
			}
		}
	}

	if (this == s_pxGPvolume)
	{
		for (u_int i = 0; i < m_uNumLinkedGamePlayVolumes; i += 1)
		{
			if (m_puLinkedGamePlayVolumes[i] != ASURA_GUID_UNREGISTERED)
			{
				Tools_Entity_Object* pxGPVolume = ToolsAVP_Entity_GamePlayVolume::FindEntity(m_puLinkedGamePlayVolumes[i]);

				if (pxGPVolume)
				{
					Asura_Vector_3 xStart = pxGPVolume->Position;
					xStart.y -= 0.1f;
					Asura_Vector_3 xEnd = Position;
					xEnd.y -= 0.1f;

					Asura_Render::RenderLine(xStart, xEnd, 0xffffff00);
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: AddOverlayControls
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	m_pxTabsDialog = new Asura_Tabs_Dialog(xOffset, s_aszTabNames, sizeof(s_aszTabNames) / sizeof(s_aszTabNames[0]));
	pxOverlay->AddDialog(m_pxTabsDialog);

	m_pxTabsDialog->SelectTab(TAB_GENERAL);
	xOffset.y = m_pxTabsDialog->m_xMaxCoord.y + 0.5f;
	float fTabsEndY = xOffset.y;

	AddGeneralTabControls(m_pxTabsDialog, xOffset);
	fTabsEndY = Asura_Maths::Max(fTabsEndY, xOffset.y);

	m_pxTabsDialog->SelectTab(TAB_ENTITY_CONTROL);
	xOffset.y = m_pxTabsDialog->m_xMaxCoord.y + 0.5f;

	AddEntityControlTabControls(m_pxTabsDialog, xOffset);
	fTabsEndY = Asura_Maths::Max(fTabsEndY, xOffset.y);

	xOffset.y = fTabsEndY + Asura_Dialog::SmallSpacing();
	AddUnderTabsControls(pxOverlay, xOffset);

	m_pxTabsDialog->SetSelectedTabPointer(&s_uCurrentTab);
} 
/// End of function "ToolsAVP_Entity_GamePlayVolume.cpp::".

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: AddGeneralTabControls
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::AddGeneralTabControls(Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;
	pxDialog = new Asura_Function_Dialog("Edit Volumes", xOffset, EditVolume, &s_bVolumesEdited);
	pxDialog->SetHelpText("Edits The Volume(s).");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Selected Volume", xOffset, &s_bShowSelectedVolume, &Changed);
	pxDialog->SetHelpText("Select To Show All The Volumes Of The Selected GamePlayVolume.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show All Volumes", xOffset, &s_bShowVolumes, &Changed);
	pxDialog->SetHelpText("Select To Show All The Volumes Of All GamePlayVolume In The Level.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Move Volume With Object", xOffset, &s_bMoveVolumeWithObject);
	pxDialog->SetHelpText("The Volumes Attached To This Entity Should Be Moved When The Trigger's Tools Object Is Moved.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Linked Volumes
	pxDialog = new Asura_StringDisplay_Dialog("Select Game Play Area Type:", xOffset);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ((m_iGamePlayAreaType < 0) || (m_iGamePlayAreaType >= AVP_ServerEntity_GamePlayVolume::MAX_AREA_TYPES))
	{
		m_iGamePlayAreaType = 0;
	}

	pxDialog = new Asura_DropDown_List_Dialog(xOffset, AVP_ServerEntity_GamePlayVolume::GetAreaTypeNames(), &m_iGamePlayAreaType, AVP_ServerEntity_GamePlayVolume::MAX_AREA_TYPES, AVP_ServerEntity_GamePlayVolume::MAX_AREA_TYPES, NULL, &m_bAreaChanged);
	pxDialog->SetHelpText("Select The Type Of Area That Best Describes This Volume");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Linked Volumes
	char szText[256] = "";
	Asura_PrintfToBfr(szText, "Linked Game Play Volumes: ( %u / %u )", 256, true, m_uNumLinkedGamePlayVolumes, MAX_LINKED_GAMEPLAYVOLUMES);
	pxDialog = new Asura_StringDisplay_Dialog(szText, xOffset);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add Link Game Play Volume", xOffset, AddGamePlayVolumeLink, NULL, NULL);
	pxDialog->SetHelpText("Add A Game Play Volume Link.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove Link Game Play Volume", xOffset, RemoveGamePlayVolumeLink, NULL, NULL);
	pxDialog->SetHelpText("Remove A Game Play Volume Link.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}


TOOLS_DECLARE_SHARED_PARAM(bool,	0x304548C6,	ActivationFlags_AllTypes );
TOOLS_DECLARE_SHARED_PARAM(bool,	0x1D82EF09,	ActivationFlags_Triggers );
TOOLS_DECLARE_SHARED_PARAM(bool,	0xF2ADF6EB,	ActivationFlags_Lights );
TOOLS_DECLARE_SHARED_PARAM(bool,	0xE03B6BBB,	ActivationFlags_PhysicsObjects );
TOOLS_DECLARE_SHARED_PARAM(bool,	0x673BEF7C,	ActivationFlags_Predators );
TOOLS_DECLARE_SHARED_PARAM(bool,	0xE013CB5A,	ActivationFlags_Aliens );
TOOLS_DECLARE_SHARED_PARAM(bool,	0xC3AFDE39,	ActivationFlags_AllHumans );
TOOLS_DECLARE_SHARED_PARAM(bool,	0xACFD583A,	ActivationFlags_Lifts );
TOOLS_DECLARE_SHARED_PARAM(bool,	0x6DAA8B34,	ActivationFlags_PFX );
TOOLS_DECLARE_SHARED_PARAM(bool,	0xFEFAE212,	ActivationFlags_Sounds );
TOOLS_DECLARE_SHARED_PARAM(bool,	0xF0BC628E,	ActivationFlags_FaceHuggers );
TOOLS_DECLARE_SHARED_PARAM(bool,	0x17B15960,	ActivationFlags_SentryGuns );
TOOLS_DECLARE_SHARED_PARAM(bool,	0xB6C3AE71,	ActivationFlags_AllActors );
TOOLS_DECLARE_SHARED_PARAM(bool,	0xC6C7B9C2,	ActivationFlags_Civilians );
TOOLS_DECLARE_SHARED_PARAM(bool,	0x8C4A2532,	ActivationFlags_Androids );
TOOLS_DECLARE_SHARED_PARAM(bool,	0x8AEBBBE3,	ActivationFlags_Marines );
TOOLS_DECLARE_SHARED_PARAM(bool,	0x7FE5B5E0,	ActivationFlags_Containers );

// g_xParam_ActivationFlags_AllTypes
// g_xParam_ActivationFlags_Triggers
// g_xParam_ActivationFlags_Lights
// g_xParam_ActivationFlags_PhysicsObjects
// g_xParam_ActivationFlags_Predators
// g_xParam_ActivationFlags_Aliens
// g_xParam_ActivationFlags_AllHumans
// g_xParam_ActivationFlags_Lifts
// g_xParam_ActivationFlags_PFX
// g_xParam_ActivationFlags_Sounds
// g_xParam_ActivationFlags_FaceHuggers
// g_xParam_ActivationFlags_SentryGuns
// g_xParam_ActivationFlags_AllActors
// g_xParam_ActivationFlags_Civilians
// g_xParam_ActivationFlags_Androids
// g_xParam_ActivationFlags_Marines
// g_xParam_ActivationFlags_Containers

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: AddEntityControlTabControls
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::AddEntityControlTabControls(Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	pxDialog = new Asura_StringDisplay_Dialog("Select some/all of the following items to have", xOffset);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::SmallSpacing();
	pxDialog = new Asura_StringDisplay_Dialog("them activate/deactivate on receiving an", xOffset);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::SmallSpacing();
	pxDialog = new Asura_StringDisplay_Dialog("'activate / deactivate selected entities' Message", xOffset);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

// 	for (int i = 0; i < AVP_ServerEntity_GamePlayVolume::FLAG_END_ENUM; i++)
// 	{
// 		pxDialog = new Asura_FlagTickbox_Dialog(s_aszFlagNames[i], xOffset, &m_iBitFlags, (1 << i), &Changed);
// 		pxTabsDialog->AddDialog(pxDialog);
// 		xOffset.y += Asura_Dialog::DefaultSpacing();
// 	}

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[0], xOffset, &g_xParam_ActivationFlags_AllTypes);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[1], xOffset, &g_xParam_ActivationFlags_Triggers);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[2], xOffset, &g_xParam_ActivationFlags_Lights);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[3], xOffset, &g_xParam_ActivationFlags_PhysicsObjects);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[4], xOffset, &g_xParam_ActivationFlags_Predators);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[5], xOffset, &g_xParam_ActivationFlags_Aliens);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[6], xOffset, &g_xParam_ActivationFlags_AllHumans);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[7], xOffset, &g_xParam_ActivationFlags_Lifts);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[8], xOffset, &g_xParam_ActivationFlags_PFX);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[9], xOffset, &g_xParam_ActivationFlags_Sounds);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[10], xOffset, &g_xParam_ActivationFlags_FaceHuggers);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[11], xOffset, &g_xParam_ActivationFlags_SentryGuns);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[12], xOffset, &g_xParam_ActivationFlags_AllActors);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[13], xOffset, &g_xParam_ActivationFlags_Civilians);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[14], xOffset, &g_xParam_ActivationFlags_Androids);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[15], xOffset, &g_xParam_ActivationFlags_Marines);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog(s_aszFlagNames[16], xOffset, &g_xParam_ActivationFlags_Containers);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}
// 
// g_uHash_ActivationFlags_AllTypes
// g_uHash_ActivationFlags_Triggers
// g_uHash_ActivationFlags_Lights
// g_uHash_ActivationFlags_PhysicsObjects
// g_uHash_ActivationFlags_Predators
// g_uHash_ActivationFlags_Aliens
// g_uHash_ActivationFlags_AllHumans
// g_uHash_ActivationFlags_Lifts
// g_uHash_ActivationFlags_PFX
// g_uHash_ActivationFlags_Sounds
// g_uHash_ActivationFlags_FaceHuggers
// g_uHash_ActivationFlags_SentryGuns
// g_uHash_ActivationFlags_AllActors
// g_uHash_ActivationFlags_Civilians
// g_uHash_ActivationFlags_Androids
// g_uHash_ActivationFlags_Marines
// g_uHash_ActivationFlags_Containers

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: GetParameter
//------------------------------------------------------------------------------
bool ToolsAVP_Entity_GamePlayVolume::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	AVP_ServerEntity_GamePlayVolume* pxAVPEntity = (AVP_ServerEntity_GamePlayVolume*)Entity;

	if (!pxAVPEntity)
	{
		return false;
	}

	switch(uParamID)
	{
		case g_uHash_ActivationFlags_AllTypes:
		{
			pxParam->SetBool( pxAVPEntity->IncludeAllTypes() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_Triggers:
		{
			pxParam->SetBool( pxAVPEntity->IncludeTriggers() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_Lights:
		{
			pxParam->SetBool( pxAVPEntity->IncludeLights() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_PhysicsObjects:
		{
			pxParam->SetBool( pxAVPEntity->IncludePhysicsObjects() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_Predators:
		{
			pxParam->SetBool( pxAVPEntity->IncludePredators() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_Aliens:
		{
			pxParam->SetBool( pxAVPEntity->IncludeAliens() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_AllHumans:
		{
			pxParam->SetBool( pxAVPEntity->IncludeHumans() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_Lifts:
		{
			pxParam->SetBool( pxAVPEntity->IncludeLifts() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_PFX:
		{
			pxParam->SetBool( pxAVPEntity->IncludePFX() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_Sounds:
		{
			pxParam->SetBool( pxAVPEntity->IncludeSounds() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_FaceHuggers:
		{
			pxParam->SetBool( pxAVPEntity->IncludeFaceHuggers() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_SentryGuns:
		{
			pxParam->SetBool( pxAVPEntity->IncludeSentryguns() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_AllActors:
		{
			pxParam->SetBool( pxAVPEntity->IncludeAllActors() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_Civilians:
		{
			pxParam->SetBool( pxAVPEntity->IncludeCivilains() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_Androids:
		{
			pxParam->SetBool( pxAVPEntity->IncludeAndroids() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_Marines:
		{
			pxParam->SetBool( pxAVPEntity->IncludeMarines() != 0 );
		}
		break;
		case g_uHash_ActivationFlags_Containers:
		{
			pxParam->SetBool( pxAVPEntity->IncludeContainers() != 0 );
		}
		break;
		default:
		{
			return PARENT::GetParameter(uParamID, pxParam);
		}break;
	}
	return true;
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: SetParameter
//------------------------------------------------------------------------------
bool ToolsAVP_Entity_GamePlayVolume::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	AVP_ServerEntity_GamePlayVolume* pxAVPEntity = (AVP_ServerEntity_GamePlayVolume*)Entity;

	if (!pxAVPEntity)
	{
		return false;
	}

	switch(uParamID)
	{
		case g_uHash_ActivationFlags_AllTypes:
		{
			pxAVPEntity->SetIncludeAllTypes( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_Triggers:
		{
			pxAVPEntity->SetIncludeTriggers( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_Lights:
		{
			pxAVPEntity->SetIncludeLights( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_PhysicsObjects:
		{
			pxAVPEntity->SetIncludePhysicsObjects( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_Predators:
		{
			pxAVPEntity->SetIncludePredators( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_Aliens:
		{
			pxAVPEntity->SetIncludeAliens( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_AllHumans:
		{
			pxAVPEntity->SetIncludeHumans( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_Lifts:
		{
			pxAVPEntity->SetIncludeLifts( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_PFX:
		{
			pxAVPEntity->SetIncludePFX( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_Sounds:
		{
			pxAVPEntity->SetIncludeSounds( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_FaceHuggers:
		{
			pxAVPEntity->SetIncludeFaceHuggers( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_SentryGuns:
		{
			pxAVPEntity->SetIncludeSentryguns( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_AllActors:
		{
			pxAVPEntity->SetIncludeAllActors( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_Civilians:
		{
			pxAVPEntity->SetIncludeCivilains( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_Androids:
		{
			pxAVPEntity->SetIncludeAndroids( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_Marines:
		{
			pxAVPEntity->SetIncludeMarines( pxParam->GetBool() );
		}
		break;
		case g_uHash_ActivationFlags_Containers:
		{
			pxAVPEntity->SetIncludeContainers( pxParam->GetBool() );
		}
		break;
		default :
		{
			return PARENT::SetParameter(uParamID, pxParam);
		}break;
	}

	SetEntityModified();
	return true;
}


//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: AddUnderTabsControls
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::AddUnderTabsControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
// 	Asura_Dialog* pxDialog;
// 
// 	pxDialog = new Asura_StringDisplay_Dialog("Hiding / Unhiding :", xOffset);
// 	pxOverlay->AddDialog(pxDialog);
// 	xOffset.y += Asura_Dialog::DefaultSpacing();
// 
// 	pxDialog = new Asura_Tickbox_Dialog("Hide All Entities Outside This Volume", xOffset, &s_bHideEntitiesOutside);
// 	pxDialog->SetHelpText("Any Entities Outside This Volume Will Be Hidden");
// 	pxOverlay->AddDialog(pxDialog);
// 	xOffset.y += Asura_Dialog::DefaultSpacing();
// 
// 	pxDialog = new Asura_Tickbox_Dialog("Include Linked Volumes When Hiding", xOffset, &s_bIncludeLinkedVolumes);
// 	pxDialog->SetHelpText("Hiding / Unhiding Entities Volume And Linked Volumes");
// 	pxOverlay->AddDialog(pxDialog);
// 	xOffset.y += Asura_Dialog::DefaultSpacing();
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: EditVolume
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::EditVolume()
{
	Asura_Assert(ThisEntity, "ToolsAVP_Entity_GamePlayVolume::EditVolume - Entity Doesn't exist for editing");

	ToolsAVP_Entity_GamePlayVolume* pxToolsEntity = s_pxGPvolume;

	//create controller for editing volumes if it hasn't been done yet
	if (!pxToolsEntity->GetVolumeController())
	{
		pxToolsEntity->m_pxVolumeController = new ToolsAVP_GamePlayVolume_VolumeEdit_Controller(pxToolsEntity);
	}

	//and get it to create the volume objects
	pxToolsEntity->GetVolumeController()->StartVolumeEditing();

	s_bVolumesEdited = true;
}

/// End of file "ToolsAVP_Entity_GamePlayVolume.cpp".


//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: AddGamePlayVolumeLink
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::AddGamePlayVolumeLink(int iIndex)
{
	const Asura_Guid								uDestinationGuid	= ASURA_GUID_UNREGISTERED;
	Tools_Object*									pxToolsObject		= Tools_Object_System::GetObjectFromGuid(uDestinationGuid);
	ToolsAVP_ChooseObject_GamePlayVolume*			pxChooseObject		= new ToolsAVP_ChooseObject_GamePlayVolume(iIndex, s_pxGPvolume);
	char											szPrompt[256];

	// Make this true as we want to add
	s_bAdd = true;

	Asura_PrintfToBfr(szPrompt, "Select the GamePlayVolume to Link to:", sizeof(szPrompt), true);

	Tools_Object_System::ChooseObject(szPrompt, pxChooseObject, pxToolsObject);
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: RemoveGamePlayVolumeLink
//------------------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::RemoveGamePlayVolumeLink(int iIndex)
{
	const Asura_Guid								uDestinationGuid	= ASURA_GUID_UNREGISTERED;
	Tools_Object*									pxToolsObject		= Tools_Object_System::GetObjectFromGuid(uDestinationGuid);
	ToolsAVP_ChooseObject_GamePlayVolume*			pxChooseObject		= new ToolsAVP_ChooseObject_GamePlayVolume(iIndex, s_pxGPvolume);
	char											szPrompt[256];

	// Make this false as we want to remove
	s_bAdd = false;

	Asura_PrintfToBfr(szPrompt, "Select the GamePlayVolume to Link to:", sizeof(szPrompt), true);

	Tools_Object_System::ChooseObject(szPrompt, pxChooseObject, pxToolsObject);

}

//----------------------------------------------------------------------
// ToolsAVP_ChooseObject_GamePlayVolume :: CanBeSelected
//----------------------------------------------------------------------
bool ToolsAVP_ChooseObject_GamePlayVolume::CanBeSelected(Tools_Object* pxObject)
{
	// don't allow non game play volumes to be selected
	if (pxObject->Type != TOT_Entity_GamePlayVolume)
		return false;

	// Get the pointer to the current GamePlayVolume
	ToolsAVP_Entity_GamePlayVolume* pxServer = ToolsAVP_Entity_GamePlayVolume::s_pxGPvolume;

	// don't allow ourselves to be selectable
	if (pxObject->GetGuid() == pxServer->GetGuid())
		return false;

	// Adding of gameplay volumes
	if (ToolsAVP_Entity_GamePlayVolume::s_bAdd)
	{
		// If the other game play volume is at the max links then don't allow to add
		ToolsAVP_Entity_GamePlayVolume* pxOtherGPVolume = static_cast<ToolsAVP_Entity_GamePlayVolume*>(pxObject);
		if (pxOtherGPVolume)
		{
			if (pxOtherGPVolume->GetNumLinkedGamePlayVolumes() == ToolsAVP_Entity_GamePlayVolume::MAX_LINKED_GAMEPLAYVOLUMES)
			{
				return false;
			}
		}

		// if we are at the maximum allowed to be linked don't allow any more
		if (pxServer->GetNumLinkedGamePlayVolumes() >= ToolsAVP_Entity_GamePlayVolume::MAX_LINKED_GAMEPLAYVOLUMES)
			return false;

		// Don't allow a GamePlayvolume to be selectable if its already linked
		for (int i = 0; i < ToolsAVP_Entity_GamePlayVolume::MAX_LINKED_GAMEPLAYVOLUMES; i += 1)
		{
			if (pxServer->GetLinkedGamePlayVolumeGuid(i) == pxObject->GetGuid())
			{
				return false;
			}
		}

		return true;
	}
	// Removal of GamePlayVolumes
	else
	{
		// If we are at ZERO number of linked don't allow any more to be removed
		if (pxServer->GetNumLinkedGamePlayVolumes() == 0)
			return false;

		// Don't allow a GamePlayvolume to be selectable if its already linked
		for (int i = 0; i < ToolsAVP_Entity_GamePlayVolume::MAX_LINKED_GAMEPLAYVOLUMES; i += 1)
		{
			if (pxServer->GetLinkedGamePlayVolumeGuid(i) == pxObject->GetGuid())
			{
				return true;
			}
		}

		return false;
	}
}

//----------------------------------------------------------------------
// ToolsAVP_ChooseObject_SquadMember :: ObjectSelected
//----------------------------------------------------------------------
void ToolsAVP_ChooseObject_GamePlayVolume::ObjectSelected(Tools_Object* pxObject)
{
	if (ToolsAVP_Entity_GamePlayVolume::s_bAdd)
	{
		m_pxGamePlayVolume->ActuallyAddGamePlayVolumeLink(m_iIndex, pxObject);
	}
	else
	{
		m_pxGamePlayVolume->ActuallyRemoveGamePlayVolumeLink(m_iIndex, pxObject);
	}
}

//----------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: ActuallyAddGamePlayVolumeLink
//----------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::ActuallyAddGamePlayVolumeLink(const int iIndex, Tools_Object* pxObject)
{
	Asura_Guid							uMemberGuid			= ASURA_GUID_UNREGISTERED;
	ToolsAVP_Entity_GamePlayVolume*		pxServer			= ToolsAVP_Entity_GamePlayVolume::s_pxGPvolume;
	u_int								uNumGamePlayVolumes	= pxServer->GetNumLinkedGamePlayVolumes();

	if (!pxObject)
	{
		return;
	}

	if (pxObject->GetGuid() != ASURA_GUID_UNREGISTERED)
	{
		uMemberGuid = pxObject->GetGuid();
	}

	if (uNumGamePlayVolumes < MAX_LINKED_GAMEPLAYVOLUMES)
	{
		if (uMemberGuid != ASURA_GUID_UNREGISTERED)
		{
			pxServer->m_puLinkedGamePlayVolumes[uNumGamePlayVolumes] = uMemberGuid;
		
			pxServer->m_uNumLinkedGamePlayVolumes += 1;

			pxServer->m_bLinkChanged = true;

			// We need to add the link to the other GP volume also!
			ToolsAVP_Entity_GamePlayVolume* pxOtherGPVolume = static_cast<ToolsAVP_Entity_GamePlayVolume*>(pxObject);

			if (pxOtherGPVolume)
			{
				AddLinkToGPVolume(pxOtherGPVolume, pxServer->GetGuid());
			}
		}
	}
}

//----------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: AddLinkToGPVolume
//----------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::AddLinkToGPVolume(ToolsAVP_Entity_GamePlayVolume* pxGPToAddTo, Asura_Guid uGuidToAdd)
{
	// if either aren't valid return
	if ((!pxGPToAddTo) || (uGuidToAdd == ASURA_GUID_UNREGISTERED))
	{
		return;
	}

	if (pxGPToAddTo->GetNumLinkedGamePlayVolumes() > MAX_LINKED_GAMEPLAYVOLUMES)
	{
		return;
	}

	pxGPToAddTo->m_puLinkedGamePlayVolumes[pxGPToAddTo->GetNumLinkedGamePlayVolumes()] = uGuidToAdd;
	pxGPToAddTo->m_uNumLinkedGamePlayVolumes += 1;
	pxGPToAddTo->m_bLinkChanged = true;
}

//----------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: ActuallyAddGamePlayVolumeLink
//----------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::ActuallyRemoveGamePlayVolumeLink(const int iIndex, Tools_Object* pxObject)
{
	Asura_Guid							uMemberGuid			= ASURA_GUID_UNREGISTERED;
	ToolsAVP_Entity_GamePlayVolume*		pxServer			= ToolsAVP_Entity_GamePlayVolume::s_pxGPvolume;
	u_int								uNumGamePlayVolumes	= pxServer->GetNumLinkedGamePlayVolumes();

	if (!pxObject)
	{
		return;
	}

	if (pxObject->GetGuid() != ASURA_GUID_UNREGISTERED)
	{
		uMemberGuid = pxObject->GetGuid();
	}

	if (uNumGamePlayVolumes > 0)
	{
		bool bFound = false;

		if (uMemberGuid != ASURA_GUID_UNREGISTERED)
		{
			for (u_int i = 0; i < MAX_LINKED_GAMEPLAYVOLUMES; i += 1)
			{
				if ((pxServer->m_puLinkedGamePlayVolumes[i] == uMemberGuid) || (bFound))
				{
					bFound = true;
					
					if (i >= (MAX_LINKED_GAMEPLAYVOLUMES - 1))
					{
						pxServer->m_puLinkedGamePlayVolumes[i] = ASURA_GUID_UNREGISTERED;
					}
					else
					{
						pxServer->m_puLinkedGamePlayVolumes[i] = pxServer->m_puLinkedGamePlayVolumes[i + 1];
					}
				}
			}

			if (bFound)
			{
				pxServer->m_uNumLinkedGamePlayVolumes -= 1;
				pxServer->m_bLinkChanged = true;
			}

			// We need to add the link to the other GP volume also!
			ToolsAVP_Entity_GamePlayVolume* pxOtherGPVolume = static_cast<ToolsAVP_Entity_GamePlayVolume*>(pxObject);

			if (pxOtherGPVolume)
			{
				RemoveLinkToGPVolume(pxOtherGPVolume, pxServer->GetGuid());
			}
		}
	}
}

//----------------------------------------------------------------------
// ToolsAVP_Entity_GamePlayVolume :: ActuallyAddGamePlayVolumeLink
//----------------------------------------------------------------------
void ToolsAVP_Entity_GamePlayVolume::RemoveLinkToGPVolume(ToolsAVP_Entity_GamePlayVolume* pxGPToRemoveFrom, Asura_Guid uGuidToRemove)
{
	// if either aren't valid return
	if ((!pxGPToRemoveFrom) || (uGuidToRemove == ASURA_GUID_UNREGISTERED))
	{
		return;
	}

	if (pxGPToRemoveFrom->GetNumLinkedGamePlayVolumes() == 0)
	{
		return;
	}

	bool bFound = false;

	for (u_int i = 0; i < MAX_LINKED_GAMEPLAYVOLUMES; i += 1)
	{
		if ((pxGPToRemoveFrom->m_puLinkedGamePlayVolumes[i] == uGuidToRemove) || (bFound))
		{
			bFound = true;

			if (i >= (MAX_LINKED_GAMEPLAYVOLUMES - 1))
			{
				pxGPToRemoveFrom->m_puLinkedGamePlayVolumes[i] = ASURA_GUID_UNREGISTERED;
			}
			else
			{
				pxGPToRemoveFrom->m_puLinkedGamePlayVolumes[i] = pxGPToRemoveFrom->m_puLinkedGamePlayVolumes[i + 1];
			}
		}
	}

	if (bFound)
	{
		pxGPToRemoveFrom->m_uNumLinkedGamePlayVolumes -= 1;
		pxGPToRemoveFrom->m_bLinkChanged = true;
	}
}

//---------------------------------------------------
// Tools_AdvancedVolumeTrigger_VolumeEdit_Controller
//---------------------------------------------------
ToolsAVP_GamePlayVolume_VolumeEdit_Controller::ToolsAVP_GamePlayVolume_VolumeEdit_Controller(ToolsAVP_Entity_GamePlayVolume* pxToolsEntity)
:PARENT(&pxToolsEntity->Changed)
{
	m_pxToolsEntity = pxToolsEntity;
	m_pxAVPEntity = (AVP_ServerEntity_GamePlayVolume*) pxToolsEntity->Entity;
}

void ToolsAVP_GamePlayVolume_VolumeEdit_Controller::StartVolumeEditing()
{
	PARENT::StartVolumeEditing();
	
	if (GetNumberOfVolumes() == 0)
	{
		//create a volume if none exists yet
		CreateVolumeCuboid();
	}
}

void ToolsAVP_GamePlayVolume_VolumeEdit_Controller::EndVolumeEditing()
{
	PARENT::EndVolumeEditing();

	//select the owner
	Tools_Object_System::SelectObject(m_pxToolsEntity);
}


void ToolsAVP_GamePlayVolume_VolumeEdit_Controller::CreateVolumeCuboidMain()
{
	Asura_Volume_Cuboid* pxVolume = new Asura_Volume_Cuboid();
	pxVolume->SetPosition(m_pxToolsEntity->Position);
	m_pxAVPEntity->AddVolume(pxVolume);
	++m_pxAVPEntity->m_iNumVolumes;
}

void ToolsAVP_GamePlayVolume_VolumeEdit_Controller::CreateVolumeSphereMain()
{
	Asura_Volume_Sphere* pxVolume = new Asura_Volume_Sphere();
	pxVolume->SetPosition(m_pxToolsEntity->Position);
	m_pxAVPEntity->AddVolume(pxVolume);
	++m_pxAVPEntity->m_iNumVolumes;
}

void ToolsAVP_GamePlayVolume_VolumeEdit_Controller::CreateVolumeCylinderMain()
{
	Asura_Volume_Cylinder* pxVolume = new Asura_Volume_Cylinder();
	pxVolume->SetPosition(m_pxToolsEntity->Position);
	m_pxAVPEntity->AddVolume(pxVolume);
	++m_pxAVPEntity->m_iNumVolumes;
}

void ToolsAVP_GamePlayVolume_VolumeEdit_Controller::DeleteVolumeMain(int iIndex)
{
	m_pxAVPEntity->RemoveVolume(static_cast<u_int>(iIndex));
	--m_pxAVPEntity->m_iNumVolumes;
}