//-----------------------------------------------------------------------------
// File			: ToolsAVP_Node_FormationPoint.cpp
// Author		: Gareth Thomas
// Created		: 18 / 12 / 2007
// Copyright (c) 2007 Rebellion. All rights reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------

// Core
#include "Core/Asura.h"

// Header
#include "Entity/Locations/FormationPoint/ToolsAVP_Node_FormationPoint.h"

// Asura
#include "Camera/Asura_Camera.h"
#include "Mesh/Asura_Mesh_System.h"

// AVP
#include "Entity/Locations/FormationPoint/AVP_ServerNode_FormationPoint.h"

// Tools
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"

//------------------------------------------------------------------------------
// Constants, Defines, Statics and Globals
//------------------------------------------------------------------------------
Asura_Hashed_List<ToolsAVP_Node_FormationPoint>				ToolsAVP_Node_FormationPoint::s_xList;
ToolsAVP_Node_FormationPoint*								ToolsAVP_Node_FormationPoint::s_pxSelectedFormationPointNode = NULL;

//-----------------------------------------------------------------------------
// Function Implementations
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// ToolsAVP_Node_FormationPoint :: Constructor
//-----------------------------------------------------------------------------
ToolsAVP_Node_FormationPoint::ToolsAVP_Node_FormationPoint(Asura_Entity* pxEntity)
: PARENT(pxEntity)
{
	Type = TOT_Entity_FormationPoint;
	SetMesh("ToolsShape_MarineFormationPoint");

	s_xList.AddToList(this);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Node_FormationPoint :: Deconstructor
//-----------------------------------------------------------------------------
ToolsAVP_Node_FormationPoint::~ToolsAVP_Node_FormationPoint(void)
{
	s_xList.RemoveFromList(this);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Node_FormationPoint :: Create
//-----------------------------------------------------------------------------
void ToolsAVP_Node_FormationPoint::Create(void)
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("FormationPoint", "FormationPoint", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Node_FormationPoint::Create() - no tools data");
	}
	else
	{
		AVP_ServerNode_FormationPoint* pxServerObject = new AVP_ServerNode_FormationPoint(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Node_FormationPoint::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Node_FormationPoint* pxToolsObject = new ToolsAVP_Node_FormationPoint(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Node_FormationPoint::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_INFO, "Created New Formation Point");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}

//-----------------------------------------------------------------------------
// ToolsAVP_Node_FormationPoint :: Create
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Node_FormationPoint::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Node_FormationPoint(pxEntity);
}

//-----------------------------------------------------------------------------
// ToolsAVP_Node_FormationPoint :: CreateAndReturn
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Node_FormationPoint::CreateAndReturn(void)
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("FormationPoint", "FormationPoint", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Node_FormationPoint::Create() - no tools data");
	}
	else
	{
		AVP_ServerNode_FormationPoint* pxServerObject = new AVP_ServerNode_FormationPoint(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Node_FormationPoint::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Node_FormationPoint* pxToolsObject = new ToolsAVP_Node_FormationPoint(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Node_FormationPoint::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);

				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_INFO, "Created New Formation Point");
				Tools_SaveAsura::SetEntityModified();
				return pxToolsObject;
			}
		}
	}

	return NULL;
}

//-----------------------------------------------------------------------------
// ToolsAVP_Node_FormationPoint :: Project_Create
//-----------------------------------------------------------------------------
void ToolsAVP_Node_FormationPoint::Project_Create(void)
{
	ToolsAVP_Node_FormationPoint::Create();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Node_FormationPoint :: CreateObjectOfThisType
//-----------------------------------------------------------------------------
Tools_Object* ToolsAVP_Node_FormationPoint::CreateObjectOfThisType(void) const
{
	return CreateAndReturn();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Node_FormationPoint :: OnSelection
//-----------------------------------------------------------------------------
void ToolsAVP_Node_FormationPoint::OnSelection(void)
{
	s_pxSelectedFormationPointNode = this;
	PARENT::OnSelection();
}

//-----------------------------------------------------------------------------
// ToolsAVP_Node_FormationPoint :: OnDeselection
//-----------------------------------------------------------------------------
void ToolsAVP_Node_FormationPoint::OnDeselection(void)
{
	PARENT::OnDeselection();
	s_pxSelectedFormationPointNode = NULL;
}

bool ToolsAVP_Node_FormationPoint::IsBaseOrDerivativeOf( int iToolsObjectType ) const
{
	if ( iToolsObjectType == TOT_Entity_FormationPoint )
	{
		return true;
	}
	return PARENT::IsBaseOrDerivativeOf( iToolsObjectType );
}

