//-----------------------------------------------------------------------------
// File			: ToolsAVP_Node_Objective_Marine.h
// Author		: Gareth Thomas
// Created		: 18 / 12 / 2007
// Copyright (c) 2007 Rebellion. All rights reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Begin File Wrapper
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __TOOLSAVP_NODE_OBJECTIVE_MARINE_H___
#define __TOOLSAVP_NODE_OBJECTIVE_MARINE_H___ 1
// End File Wrapper
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------

// Tools
#include "Entity/Locations/Objective/ToolsAVP_Node_Objective.h"
#include "ToolsAVP_ChooseObjects.h"

#include "Entity/Locations/Objective/Marine/AVP_ServerEntity_Objective_Marine.h"

class ToolsAVP_ChooseGS2Object;

//-----------------------------------------------------------------------------
// Class		: ToolsAVP_Node_Objective_Marine
// Author		: Gareth Thomas
// Created		: 23 / 01 / 08
// Description	: 
//-----------------------------------------------------------------------------
class ToolsAVP_Node_Objective_Marine : public ToolsAVP_Node_Objective
{
public:

	enum ActionLinkingType
	{
		NO_TYPE,
		SPECIFIC_GAMEPLAYVOLUME,
		SPECIFIC_GAMESCENES,
		MAX_TYPES,
	};

	// Public Construction / Deconstruction functions...
	ToolsAVP_Node_Objective_Marine						( Asura_Entity* pxEntity );
	virtual ~ToolsAVP_Node_Objective_Marine				( void );

	// Public Static functions...
	static void				Create						( void );
	static Tools_Object*	Create						( Asura_Entity* pxEntity);
	static Tools_Object*	CreateAndReturn				( void );
	static void				Project_Create				( void );

	// Public Overridden Virtual Functions...
	virtual bool			IsBaseOrDerivativeOf		( int iToolsObjectType ) const;
	virtual bool			CanBeCopied					( void ) const							{ return ( true ); }
	virtual Tools_Object*	CreateObjectOfThisType		( void ) const;
	virtual const char*		GetTypeDescription			( void )								{ return ( "AVP AI Goal" ); }
	virtual void			OnSelection					( void );
	virtual void			OnDeselection				( void );
	virtual void			AddOverlayControls			( class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset );
	virtual void			UpdateWhileSelected			( void );
	virtual void			Update						( void );
	virtual void			Render						( void );
	virtual void			UpdatePositionInAsuraData	( bool bUpdateLinkedObjects );
	virtual bool			HandlesThisEvent			( int iEventID ) const;
	virtual bool			HandlesThisMessage          ( int iMessageID );
	virtual bool			Verify						( const bool bAutomatic );

	// Static Public Functions
	static const Asura_Hashed_List < ToolsAVP_Node_Objective_Marine >* GetList	( void )		{ return &s_xList; }
	static ToolsAVP_Node_Objective_Marine* GetSelectedObjective					( void )		{ return s_pxSelectedMarineObjective; }
	static const char**	GetActionLinkTypeNames									( void )		{ return s_aszActionLinktypeNames; }

protected:

	Asura_LinkList<Asura_Guid>										m_xRequiredList;
	Asura_LinkList<Asura_Guid>										m_xOptionalList;

private:

	friend  ToolsAVP_ChooseGS2Object;
	typedef ToolsAVP_Node_Objective PARENT;

	// Static Private Functions

	static void SelectRequiredActionToAdd				( void );
	static void SelectRequiredActionToRemove			( void );
	static void SetRequiredAction						( Asura_Guid uGuid );
	static void	RemoveRequiredAction					( Asura_Guid uGuid );
	static void SelectOptionalActionToAdd				( void );
	static void SelectOptionalActionToRemove			( void );
	static void SetOptionalAction						( Asura_Guid uGuid );
	static void	RemoveOptionalAction					( Asura_Guid uGuid );

	// Members
	Asura_Vector_3													m_xPreviousPosition;
	int																m_iActionLinkingType;
	bool															m_bObjectiveCompleteOnMessage;
	bool															m_bRunToAction;

	// Statics
	static Asura_Hashed_List<ToolsAVP_Node_Objective_Marine>		s_xList;
	static ToolsAVP_Node_Objective_Marine*							s_pxSelectedMarineObjective;
	static bool														s_bActionLinkTypeChanged;
	static bool														s_bRequiredGameSceneMethodChanged;
	static bool														s_bOptionalGameSceneMethodChanged;
	static const char*												s_aszActionLinktypeNames[MAX_TYPES];
};

//----------------------------------------------------------------------------------
// Class        : ToolsAVP_ChooseGS2Object
// Author       : Gareth Thomas
// Date         : 06 / 06 / 08
// Description  : For Choosing Game Scene Objects
//----------------------------------------------------------------------------------
class ToolsAVP_ChooseGS2Object : public ToolsAVP_ChooseObject_ByType
{
public:

	typedef ToolsAVP_ChooseObject_ByType PARENT;

	ToolsAVP_ChooseGS2Object( const int iType, const int iStructureType, bool bAdd, Asura_Guid* puGuidToSet, bool* pbChanged = NULL )
		: PARENT( iType, puGuidToSet, pbChanged)
		, m_iStructureType(iStructureType)
		, m_bAdd(bAdd)
	{
	}

	ToolsAVP_ChooseGS2Object( const int iType, const int iStructureType, bool bAdd, void( *pfnOnChosen )( Asura_Guid ), bool* pbChanged = NULL )
		: PARENT( iType, pfnOnChosen, pbChanged)
		, m_iStructureType(iStructureType)
		, m_bAdd(bAdd)
	{
	}

	virtual bool CanBeSelected( Tools_Object* pxObject );

	int		m_iStructureType;
	bool	m_bAdd;
};

#endif // __TOOLSAVP_NODE_OBJECTIVE_MARINE_H___