//------------------------------------------------------------------------------------
// File			: ToolsAVP_Entity_ShootAt.h
// Author		: Gareth Thomas
// Created		: 18 / 12 / 2007
// Copyright (c) 2007 Rebellion. All rights reserved.
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
// Begin File Wrapper
#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif
#ifndef __TOOLSAVP_ENTITY_SHOOTAT_H___
#define __TOOLSAVP_ENTITY_SHOOTAT_H___ 1
// End File Wrapper
//------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------
// Includes
//------------------------------------------------------------------------------------
#include "Entity/Locations/Tools_Entity_Node.h"

//------------------------------------------------------------------------------------
// Class		: ToolsAVP_Entity_ShootAt
// Author		: Gareth Thomas
// Created		: 16 / 09 / 2007
// Description	: Class to be used as a shoot at marker for marines and any 
// other entity types that need.
//------------------------------------------------------------------------------------
class AVP_ServerEntity_ShootAt;

class ToolsAVP_Entity_ShootAt : public Tools_Entity_Node
{

public:

	// Public Construction / Deconstruction functions...
	ToolsAVP_Entity_ShootAt								(Asura_Entity* pxEntity);
	virtual ~ToolsAVP_Entity_ShootAt					(void);

	// Public Static functions...
	static void				Create						(void);
	static Tools_Object*	Create						(Asura_Entity* pxEntity);
	static Tools_Object*	CreateAndReturn				(void);
	static void				Project_Create				(void);

	// Public Overridden Virtual Functions...
	virtual void			AddOverlayControls			(class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset);
	virtual bool			CanBeCopied					(void) const						{return true;}
	virtual void			Copy						(const Tools_Object* pxObject);
	virtual Tools_Object*	CreateObjectOfThisType		(void) const;
	virtual const char*		GetTypeDescription			(void)								{return ("ShootAtNode");}
	virtual void			OnSelection					(void);
	virtual void			OnDeselection				(void);
	virtual void			Render						(void);
	virtual void			UpdateWhileSelected			(void);
	virtual bool			IsBaseOrDerivativeOf( int iToolsObjectType ) const;

	static ToolsAVP_Entity_ShootAt*							FindEntity(Asura_Guid uGuid)	{return s_xList.FindEntity(uGuid);}
	static const Asura_Hashed_List<ToolsAVP_Entity_ShootAt>* GetList(void)					{return &s_xList;}

protected:

	AVP_ServerEntity_ShootAt* GetGameShootAtNode() const;

	static void SelectShootAtEntity(void);
	static void ClearShootAtEntity(void);

private:

	typedef Tools_Entity_Node PARENT;

	bool												m_bShootAtPlayer;
	bool												m_bShootAtLinkedEntity;
	Asura_Guid											m_uLinkedShootAtEntity;
	bool												m_bShootAtPlayerChanged;
	bool												m_bShootAtLinkedEntityChanged;

	static Asura_Hashed_List<ToolsAVP_Entity_ShootAt>	s_xList;
	static ToolsAVP_Entity_ShootAt*						s_pxSelectedShootAtNode;
};

//------------------------------------------------------------------------------------
// Class		: Tools_ChooseObject_AnyActor
// Author		: Gareth Thomas
// Created		: 16 / 09 / 2007
// Description	: Tools Choose Object class which allows selection of any actor type.
//------------------------------------------------------------------------------------
class ToolsAVP_ChooseObject_AnyActor : public Tools_ChooseObject
{
public:

	ToolsAVP_ChooseObject_AnyActor( Asura_Guid* puGuidToSet, bool* bChanged = NULL )
	: m_puGuidToSet( puGuidToSet ), m_pbChanged( bChanged )
	{

	}

	virtual bool CanBeSelected	( Tools_Object* pxObject );
	virtual void ObjectSelected	( Tools_Object* pxObject );
	virtual void OnFinish		( void )					{ delete this; }

private:

	Asura_Guid* m_puGuidToSet;
	bool*       m_pbChanged;
};

#endif // __TOOLSAVP_ENTITY_SHOOTAT_H___