//////////////////////////////////////////////////////////////////////////////
//
// FILE : ToolsAVP_Entity_AlienQueen.cpp
//
// Inherits from Tools_Entity_PhysicalObject
//
// AUTHORS : Andy McCabe
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/ToolsAVP_Entity_AlienQueen.h"

// asura
#include "Mesh/Asura_Mesh_System.h"

// avp
#include "Actor/Vertebrates/AlienQueen/AVP_AlienQueen_Description.h"
#include "Entity/AVP_Messages.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AlienQueen/AVP_ServerActor_AlienQueen.h"
#include "Camera/Asura_Camera.h"

// tools avp
#include "ToolsAVP_ChooseObjects.h"
#include "Entity/Tools_Entity_Data.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"
#include "Objects/Tools_Object_System.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_AlienQueen
//
//////////////////////////////////////////////////////////////////////////////

ToolsAVP_Entity_AlienQueen* ToolsAVP_Entity_AlienQueen::s_pxThisTool = NULL;

ToolsAVP_Entity_AlienQueen::ToolsAVP_Entity_AlienQueen(AVP_ServerActor_AlienQueen* pxActor)
: Tools_Entity_PhysicalObject(pxActor)
, PARENT( pxActor )
, m_pxAlienQueen(pxActor)
{
	Asura_String mesh_name = "ToolsShape_AlienQueen";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);

	Type	= TOT_Entity_AlienQueen;
}

void ToolsAVP_Entity_AlienQueen::RestoreMesh()
{
	// Restore original mesh
	SetMesh(Asura_Mesh_System::GetMesh("ToolsShape_AlienQueen"));
}

void ToolsAVP_Entity_AlienQueen::Project_Create()
{
    ToolsAVP_Entity_AlienQueen::Create( true );
}

Tools_Object* ToolsAVP_Entity_AlienQueen::Create( Asura_Entity* pxEntity )
{
    return new ToolsAVP_Entity_AlienQueen( AVP_ServerActor_AlienQueen::FindEntity(pxEntity->GetGuid()) );
}

Tools_Object* ToolsAVP_Entity_AlienQueen::Create( const bool bFullInit )
{
	// Get the user to pick a blueprint to initialise from.
	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;
	Asura_Hash_ID uBlueprintHashID     = 0x5907A4C3;
	if ( bFullInit )
	{
		BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );
		if ( !xBPOverlay.DoModal() ) { return NULL; }
	}

	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("AlienQueen", "AlienQueen", uGuid);
	if ( !pxToolsData ) { return NULL; }

	AVP_ServerActor_AlienQueen* pxServerObject = new AVP_ServerActor_AlienQueen(uGuid);
	if ( !pxServerObject )
	{
		delete pxToolsData;
		return NULL;
	}

	// Set the tools data before we create the tools object - otherwise it will have already been created!
	pxServerObject->ToolsData = pxToolsData;
	if ( bFullInit )
	{
		pxServerObject->SetPosition( pxToolsData->Position );
		pxServerObject->SetBlueprintTypeHashID( uBlueprintTypeHashID );
		pxServerObject->SetBlueprintHashID( uBlueprintHashID );
		pxServerObject->MakeAI();
		pxServerObject->CreateAndInitialiseBrain();
		pxServerObject->InitFromBlueprint();
	}

	ToolsAVP_Entity_AlienQueen* pxToolsObject = new ToolsAVP_Entity_AlienQueen( pxServerObject );
	if ( !pxToolsObject )
	{
		delete pxToolsObject;
		delete pxServerObject;
		return NULL;
	}
	Tools_Object_System::SelectObject(pxToolsObject);

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new AlienQueen." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsObject;
}

void ToolsAVP_Entity_AlienQueen::OnSelection()
{
	s_pxThisTool = this;
	PARENT::OnSelection();
}

void ToolsAVP_Entity_AlienQueen::OnDeselection()
{
	PARENT::OnDeselection();
	s_pxThisTool = NULL;
}

void ToolsAVP_Entity_AlienQueen::AddFilesToExportList(List<Asura_String>& nameList) const
{
	// Call the base-function...
	PARENT::AddFilesToExportList(nameList);

	// Export the AlienQueen data...
	AVP_AlienQueen_Description::AddFilesToExport(false, m_uBlueprintHashID, nameList);
}

bool ToolsAVP_Entity_AlienQueen::HandlesThisMessage( int iMessageID )
{
	switch( iMessageID )
	{
		////////////////////
		case AVP_MSGID_LOOK_AT_ENTITY:
		case AVP_MSGID_STOP_LOOKING_AT_ENTITY:
		case AVP_MSGID_ALIENQUEEN_FORCE_ROAR:
		case AVP_MSGID_ALIENQUEEN_SET_BEHAVIOUR:
		////////////////////
		{
			return( true );
		}
		////////////////////
		default:
		////////////////////
		{
			return( PARENT::HandlesThisMessage( iMessageID ) );
		}
	}	
}

/// End of file. ///
