//////////////////////////////////////////////////////////////////////////////
// 
// FILE : ToolsAVP_Entity_Human.h
//
// AUTHOR : Martin G. Harris - Modified By Chris Wallace
//
// Copyright 2002 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLSAVP_ENTITY_HUMAN_H_
#define _TOOLSAVP_ENTITY_HUMAN_H_

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Entity/PhysicalObject/Actor/ToolsAVP_Entity_Actor.h"
//#include "Objects/Tools_Object_System.h"
//#include "Tools_Mode.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_ServerActor_Human;

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_Human
//
//////////////////////////////////////////////////////////////////////////////

class ToolsAVP_Entity_Human : public ToolsAVP_Entity_Actor
{
	typedef ToolsAVP_Entity_Actor PARENT;

public:
	static void				SelectGuardZoneAddLink		(int iIndex);
	static void				SelectGuardZoneRemoveLink	(int iIndex);
	void					SetDestination						(const int iIndex, Tools_Object* pxObject);
	virtual void			Render						(void);
	static ToolsAVP_Entity_Human* GetSelectedHumanPointer(void)		{ return s_pxSelectedHuman; }
	static bool				GetAddRemoveLink			(void) 		{ return s_bAddRemoveLink; }
	Asura_Guid				GetLinkedGuardZoneGuid		(void) 		{ return m_uLinkedGuardZoneGuid; }
	static ToolsAVP_Entity_Human* FindEntity			(Asura_Guid uGuid)		{return s_xList.FindEntity(uGuid);}
	virtual void			Copy(const Tools_Object* pxObject);
	virtual bool			GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const;
	virtual bool			SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam );
	virtual bool			OwnsThisMessageBlock( unsigned short usMessageBlockID );
	virtual bool			GetEventDescForMessageBlock( unsigned short usMessageBlockID, Asura_String& xOutputDesc );
	virtual	void			Destroy();

protected:
	ToolsAVP_Entity_Human(AVP_ServerActor_Human* pxActor);
	virtual ~ToolsAVP_Entity_Human();

    virtual const char*		GetTypeDescription() { return "Human"; }
	virtual void			AddFilesToExportList(List<Asura_String>& nameList) const;
	virtual void			OnSelection();
	virtual void			OnDeselection();
	virtual bool			HandlesThisMessage( int iMessageID );
	virtual bool			HandlesThisEvent( int iEventID ) const;

	virtual void			AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset );
	virtual void			UpdateWhileSelected();
	virtual void			Update();
	virtual void			PrintInfo(float& x, float& y);
	virtual void            RestoreMesh(); 

    virtual Asura_Hash_ID   GetDefaultBlueprintHashID() const { return 0x0284E09A; } // Asura_GetHashID( "Marine Variant 01" )

	void					ChooseText();
	static  void			ChooseText(int iThis){((ToolsAVP_Entity_Human*)iThis)->ChooseText();}
	void					UpdateText();

	Asura_Overlay*			m_pxOverlay;

private:
    typedef					ToolsAVP_Entity_Actor PARENT;
	
	AVP_ServerActor_Human*	m_pxHuman; // cached pointer to entity
	bool					m_bChanged;
	Asura_Guid				m_uLinkedGuardZoneGuid;
	
	static bool				s_bAddRemoveLink;
	static ToolsAVP_Entity_Human*	s_pxSelectedHuman;
	// Static pointer to 'this' updated during construction for use in updating blueprints
	static ToolsAVP_Entity_Human*	s_pxStaticThisPointer; 

	static Asura_Hashed_List<ToolsAVP_Entity_Human> s_xList;

	static void SelectSquad(void);
	static void ClearSelectedSquad(void);
	static void SetSelectedSquad(Asura_Guid uGuid);
	static void	EditOnHeadRippedMessages();
};

////////////////////////////////////////////////////////////////////////////////
// Class        : ToolsAVP_ChooseObject_Human
// Author       : Stuart Perry
// Date         : 22-Mar-2007
//
// Description  : Class to handle the choosing of entities link for the Human
////////////////////////////////////////////////////////////////////////////////

class ToolsAVP_ChooseObject_Human : public Tools_ChooseObject
{
public:

	// Public functions...
	ToolsAVP_ChooseObject_Human (const int iIndex, ToolsAVP_Entity_Human* pxHuman)	
		:	m_iIndex(iIndex), 
		m_pxHuman(pxHuman)	{}

	// Overridden functions...
	bool	CanBeSelected	(Tools_Object* pxObject);
	void	ObjectSelected	(Tools_Object* pxObject);

protected:
	// Protected variables...

	// Protected functions...

private:
	// Private variables...
	int											m_iIndex;					// The index of the human we're using.
	ToolsAVP_Entity_Human*						m_pxHuman;		// A pointer to the Tools Human-entity which we're choosing our object for.

	// Private functions...
	ToolsAVP_ChooseObject_Human		(void);
	ToolsAVP_ChooseObject_Human		(const ToolsAVP_ChooseObject_Human& xOther);

	ToolsAVP_ChooseObject_Human&	operator=	(const ToolsAVP_ChooseObject_Human& xOther);

};   // End of class 'ToolsAVP_ChooseObject_Human'.

#endif //_TOOLSAVP_ENTITY_HUMAN_H_
