//
// ToolsAVP_Entity_CommandPost.cpp
//
// Created by RDM - 21 November 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

/************/
/* Includes */
/************/

// main asura header
#include "Core/Asura.h"

// This file's header.
#include "ToolsAVP_Entity_CommandPost.h"

// asura
#include "Camera/Asura_Camera.h"
#include "Core/Asura_HashedLocalisedText.h"
#include "Core/Asura_UserInput.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Shapes/Asura_Shape_System.h"

// tools
#include "Load&Save/Tools_SaveAsura.h"
#include "Entity/Tools_Entity_Object.h"
#include "GUIMenu/Tools_GUIMenu_Element_Text.h"
#include "Objects/Tools_Object_System.h"
#include "Shapes/Tools_Shapes.h"
//#include "ToolsAVP_Entity_CommandPost_Node.h"
#include "Misc/Tools_LOS.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Movement/Tools_Movement.h"
#include "Particles/Tools_Mode_PFX.h"

// AVP
#include "Entity/AVP_Entity_EventIDs.h"
#include "Core/AVP_Settings.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Locations/GuardZone/AVP_ServerNode_GuardZone.h"
#include "Entity/PhysicalObject/CommandPost/AVP_ClientEntity_CommandPost.h"

// tools AVP
#include "ToolsAVP_System.h"
#include "ToolsAVP_ChooseObjects.h"
#include "Load&Save/ToolsAVP_Level_Export.h"
#include "Objects/ToolsAVP_Object_Type.h"

/********/
/* Data */
/********/

// static members
Asura_Hashed_List<ToolsAVP_Entity_CommandPost> ToolsAVP_Entity_CommandPost::s_xList(5);
//ToolsAVP_ChooseObject* ToolsAVP_Entity_CommandPost::s_pxObjectChooser = NULL;
ToolsAVP_Entity_CommandPost* ToolsAVP_Entity_CommandPost::s_pxThis = NULL;
u_int ToolsAVP_Entity_CommandPost::s_uGuardZoneLink = 0;

/*************/
/* Functions */
/*************/

ToolsAVP_Entity_CommandPost::ToolsAVP_Entity_CommandPost( AVP_ServerEntity_CommandPost* pxCmdPost )
: ToolsAVP_Entity_PhysicsObject( pxCmdPost )
, Tools_Entity_PhysicalObject( pxCmdPost )
, m_bCaptureDisabled(false)
, m_bCaptureByBotDisabled(false)
, m_bDefaultSpawnPoint(false)
, m_bUsesOrientation(false)
, m_pxOverlay(NULL)
{
	Type = TOT_Entity_CommandPost;

	if( pxCmdPost )
	{
		m_bCaptureDisabled	    = pxCmdPost->IsCaptureDisabled();
		m_bCaptureByBotDisabled = pxCmdPost->IsCaptureByBotDisabled();
		m_bDefaultSpawnPoint    = pxCmdPost->IsDefaultSpawnPoint();
		m_bUsesOrientation		= pxCmdPost->IsUsingOrientation();

		for ( u_int i = 0; i < AVP_COMMANDPOST_MAX_NUM_LINKED_GUARDZONES; ++i )
		{
			m_auLinkedGuardZones[i] = pxCmdPost->m_auLinkedGuardZoneGuids[i];
			m_apxGuardZoneDialogueText[i] = NULL;
		}

		pxCmdPost->SetSkinAndAnimID( Asura_GetHashID("CapturePoint_predator"), ASURA_HASH_ID_UNSET );
		SetSkinAndAnim( pxCmdPost->GetSkinID(), ASURA_HASH_ID_UNSET ); 
	}

	s_xList.AddToList(this);
}

ToolsAVP_Entity_CommandPost::~ToolsAVP_Entity_CommandPost()
{
	//if (s_pxObjectChooser)
	//{
	//	delete s_pxObjectChooser;
	//	s_pxObjectChooser = NULL;
	//}

	s_xList.RemoveFromList(this);
}

void ToolsAVP_Entity_CommandPost::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( Asura_UserInput::DebouncedKeyPressed( 'n' ) )      // 'n' to add a command node
	{
		// Create and link child node to selected command post
		ToolsAVP_Entity_CommandPost::CreateNewChildNode();
	}

	AVP_ServerEntity_CommandPost* pxCmdPost = AVP_ServerEntity_CommandPost::FindEntity( GetGuid() );
	if( pxCmdPost )
	{
		pxCmdPost->SetCaptureDisabled( m_bCaptureDisabled );
		pxCmdPost->SetCaptureByBotDisabled( m_bCaptureByBotDisabled );
		pxCmdPost->SetDefaultSpawnPoint( m_bDefaultSpawnPoint );
		pxCmdPost->SetUsesOrientation( m_bUsesOrientation );
		for ( u_int i = 0; i < AVP_COMMANDPOST_MAX_NUM_LINKED_GUARDZONES; ++i )
		{
			pxCmdPost->m_auLinkedGuardZoneGuids[i] = m_auLinkedGuardZones[i];
		}
	}
}

void ToolsAVP_Entity_CommandPost::OnSelection()
{
	PARENT::OnSelection();
	s_pxThis = this;
	ValidateChildNodes();
	ValidateGuardZoneLinks();
}

void ToolsAVP_Entity_CommandPost::OnDeselection()
{
	PARENT::OnDeselection();
	s_pxThis = NULL;
	ValidateChildNodes();
}

void ToolsAVP_Entity_CommandPost::ValidateChildNodes()
{
	AVP_ServerEntity_CommandPost* pxCommandPost = AVP_ServerEntity_CommandPost::FindEntity( GetGuid() );
	Asura_Assert(pxCommandPost,"Couldnt find server entity");
	if (pxCommandPost)
	{
		Asura_Guid uInvalidNode = pxCommandPost->m_xChildNodes.ValidateNodes();
		while (uInvalidNode != ASURA_GUID_UNREGISTERED)
		{
			// Warn user
			char szMessage[100];
			Asura_PrintfToBfr(szMessage, "An Invalid Child node with Guid %d was found for this entity (%s) and has been deleted", 100, true, uInvalidNode, GetName());
			Message_Overlay(Asura_String(szMessage)).DoModal();

			// Remove invalid node.
			bool bResult = pxCommandPost->m_xChildNodes.RemoveNode( uInvalidNode );
			Asura_Assert(bResult,"Couldn't remove invalid node");

			uInvalidNode = pxCommandPost->m_xChildNodes.ValidateNodes();
		}
	}
}

void ToolsAVP_Entity_CommandPost::Render()
{
	PARENT::Render();
	
	if ( IsSelected() )
	{
		AVP_ServerEntity_CommandPost* pxCommandPost = AVP_ServerEntity_CommandPost::FindEntity( GetGuid() );

		if ( pxCommandPost )
		{

			Asura_Vector_3 xOffGround = Asura_Vector_3( 0.0f, -0.2f, 0.0f );
			Asura_Vector_3 xStartPos = pxCommandPost->GetPosition();

			if ( !pxCommandPost->m_xChildNodes.IsEmpty() )
			{
				for (int i = 0; i < pxCommandPost->m_xChildNodes.s_uMAX_NODES; i++)
				{
					Asura_Guid uChildGuid = pxCommandPost->m_xChildNodes.GetNode(i).m_uGuid;
					if (uChildGuid != ASURA_GUID_UNREGISTERED)
					{
						// TODO: do we need command post nodes?
						//AVP_ServerEntity_CommandPost_Node* pxNode = AVP_ServerEntity_CommandPost_Node::FindEntity(uChildGuid);
						//Asura_Assert(pxNode,"Missing child node");
						//if (pxNode) Asura_Render::RenderLine(pxCommandPost->GetPosition()+Asura_Vector_3(0.0f,-0.1f,0.0f),pxNode->GetPosition(),0xffff0000);
					}
				}
			}

			if ( pxCommandPost->m_bUsesOrientation )
			{
				xStartPos.y -= 0.1f;

				Asura_Vector_3 xEndPos = xStartPos + ( Asura_Vector_3(0.0f,0.0f,2.0f) * pxCommandPost->GetSpawnOrientation() );

				Asura_Render::RenderLine( xStartPos, xEndPos, 0xff00ff00 );
			}

			// Search through all GuardZone Links render lines to them
			AVP_ServerNode_GuardZone* pxGuardZone = NULL;
			u_int uColour = 0xff0000ff;
			for (u_int i = 0; i < AVP_COMMANDPOST_MAX_NUM_LINKED_GUARDZONES; ++i)
			{

				if ( m_auLinkedGuardZones[i] != ASURA_GUID_UNREGISTERED )
				{

					pxGuardZone = AVP_ServerNode_GuardZone::FindEntity( m_auLinkedGuardZones[i] );
					if ( pxGuardZone )
					{

						switch ( i )
						{
						case 0: { uColour = 0xff000080; } break;
						case 1: { uColour = 0xff0147FA; } break;
						case 2: { uColour = 0xff1E90FF; } break;
						case 3: { uColour = 0xff87CEFF; } break;
						}

						Asura_Render::RenderLine( xStartPos + xOffGround, pxGuardZone->GetPosition() + xOffGround, uColour );
					}
				}
			}
		}
	}
}

void ToolsAVP_Entity_CommandPost::AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxNewOverlay, xOffset );

	// Controls for the command post object
	AVP_ServerEntity_CommandPost* pxCommandPost = AVP_ServerEntity_CommandPost::FindEntity( GetGuid() );
    Asura_Assert( pxCommandPost, "Failed to find server entity" );
	if (!pxCommandPost) return;

	Asura_Dialog* pxNewDialog;

	pxNewDialog = new Asura_Function_Dialog("Text",xOffset,ChooseText,(int)this);
	pxNewDialog->SetHelpText("Select the text to use.");
	pxNewDialog->m_iControlID = Asura_GetHashID("SelectText");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxNewDialog = new Asura_StringDisplay_Dialog( "Initial Team: ", xOffset );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_DropDown_List_Dialog( xOffset, AVP_Settings::GetTeamNames(), reinterpret_cast<int*>(&pxCommandPost->m_eInitialTeam), 
		AVP_Settings::AVP_NUM_TEAMS, AVP_Settings::AVP_NUM_TEAMS, NULL, &Changed, "" );
	pxNewDialog->SetHelpText("Set the team that starts with this command post");
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += (2 *Asura_Dialog::DefaultSpacing());

	pxNewDialog = new Asura_Numerical_Dialog( "Capture Time", xOffset, &pxCommandPost->m_fCaptureTime, 1.0f, fAVP_COMMAND_POST_MAX_CAPTURE_TIME, 1.0f, "%.0f", &Changed );
	pxNewDialog->SetHelpText("Set the time in seconds it takes to capture or neutralise this command post");
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxNewDialog = new Asura_Tickbox_Dialog( "Cannot Be Captured", xOffset, &m_bCaptureDisabled, &Changed );
	pxNewDialog->SetHelpText("Sets whether this command post is captureable (by anyone)");
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Tickbox_Dialog( "Cannot Be Captured By Bots", xOffset, &m_bCaptureByBotDisabled, &Changed );
	pxNewDialog->SetHelpText("Sets whether this command post is captureable (by bots)");
	pxNewDialog->SetIsEnabledFunction(CaptureByBotsButtonIsEnabled);
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Tickbox_Dialog( "Default Spawn Point", xOffset, &m_bDefaultSpawnPoint, &Changed );
	pxNewDialog->SetHelpText("Sets whether this command post is used as the default spawn point for the current owning team (players only).");
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Tickbox_Dialog( "Uses Orientation", xOffset, &m_bUsesOrientation, &Changed );
	pxNewDialog->SetHelpText("When selected bots will be spawned with the orientation of the command post (marked by the green line)");
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.y += (2 * Asura_Dialog::DefaultSpacing());

	pxNewDialog = new Asura_Divider_Dialog( xOffset );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_StringDisplay_Dialog( "Child Nodes:", xOffset );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Update text
	m_pxOverlay = pxNewOverlay; 
	UpdateText();

	// TODO: do we need child nodes?
	//Add the child nodes
	//xOffset.x += 1;
	//bool bAddedNode = false;
	//for (int i = 0; i < pxCommandPost->m_xChildNodes.s_uMAX_NODES; i++)
	//{
	//	Asura_Guid uChildGuid = pxCommandPost->m_xChildNodes.GetNode(i).m_uGuid;
	//	if (uChildGuid != ASURA_GUID_UNREGISTERED)
	//	{
	//		ToolsAVP_Entity_CommandPost_Node* pxNode = ToolsAVP_Entity_CommandPost_Node::FindEntity(uChildGuid);
	//		if (pxNode)
	//		{
	//			pxNewDialog = new Asura_Numerical_Dialog( "", xOffset, &pxCommandPost->m_xChildNodes.GetNodeProbability(i), 0, 100, 1, "%d", &Changed);
	//			pxNewDialog->SetHelpText("Relative chance for this node to be chosen.");
	//			pxNewOverlay->AddDialog(pxNewDialog);
	//			xOffset.x += 10.0f;

	//			pxNewDialog = new Asura_Function_Dialog( "", xOffset, GotoChildNode, uChildGuid );
	//			pxNewDialog->SetHelpText("Click to goto this child node");
	//			pxNewOverlay->AddDialog( pxNewDialog );

	//			xOffset.x += 2;
	//			pxNewDialog = new Asura_Function_Dialog( pxNode->GetName(), xOffset, DeleteChildNode, uChildGuid );
	//			pxNewDialog->SetHelpText("Click to remove this node from the child list");
	//			pxNewOverlay->AddDialog( pxNewDialog );

	//			xOffset.x -= 12;
	//			xOffset.y += Asura_Dialog::DefaultSpacing();

	//			bAddedNode = true;
	//		}
	//	}
	//}
//	pxNewDialog = new Asura_StringDisplay_Dialog( (bAddedNode) ? "[Click 1st to Goto, 2nd to Remove]" : "[None - Click below to add one]", xOffset );
//	pxNewOverlay->AddDialog(pxNewDialog);
//	xOffset.y += ( 2 *Asura_Dialog::DefaultSpacing() );
//	
//	pxNewDialog = new Asura_Function_Dialog( "Attach existing node", xOffset, AddNewChildNode );
//	pxNewDialog->SetHelpText("Adds an existing child node for bots to spawn at");
//	pxNewOverlay->AddDialog(pxNewDialog);
//	xOffset.y += Asura_Dialog::DefaultSpacing();
////	xOffset.x -= 1;
//
//	pxNewDialog = new Asura_Function_Dialog( "Add new node (N)", xOffset, CreateNewChildNode );
//	pxNewDialog->SetHelpText("Creates a new child node at cursor position and attaches it for bots to spawn at");
//	pxNewOverlay->AddDialog(pxNewDialog);
//	xOffset.y += Asura_Dialog::LargeSpacing();
//	xOffset.x -= 1;

	///////////////////////////// GUARD ZONE STUFF ////////////////////////////////


	#ifdef TEST_COMMAND_POST_SPAWNER
	xOffset.x += 1;
	pxNewDialog = new Asura_Function_Dialog( "Test Spawner", xOffset, TestSpawner );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x -= 1;
	#endif //__TEST_SPAWNER__
}

void ToolsAVP_Entity_CommandPost::ChooseText()
{
	Asura_Hash_ID uMenuPage = Asura_GetHashID("Menu") ;

	AVP_ServerEntity_CommandPost* pxAVPEntity = AVP_ServerEntity_CommandPost::FindEntity(Entity->GetGuid());

	if(pxAVPEntity)
	{
		Asura_Hash_ID uTextID = pxAVPEntity->GetNameHashID();
		if(!Tools_Mode_GUIMenu::ChooseText(uMenuPage, uTextID)) { return; }

		if(uTextID != pxAVPEntity->GetNameHashID())
		{
			pxAVPEntity->SetNameHashID(uTextID);
			UpdateText();
			Tools_SaveAsura::SetEntityModified(pxAVPEntity);
		}	
	}
}

void ToolsAVP_Entity_CommandPost::UpdateText()
{
	Asura_Hash_ID uMenuPage = Asura_GetHashID("Menu") ;

	AVP_ServerEntity_CommandPost* pxAVPEntity = AVP_ServerEntity_CommandPost::FindEntity(Entity->GetGuid());

	if(pxAVPEntity)
	{
		char szText[100];
		const Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::GetPage(uMenuPage);

		const char* szTextID = 0;

		if(pxPage && pxPage->HasTextIDStrings())
		{
			szTextID = pxPage->GetTextIDString(pxAVPEntity->GetNameHashID());
		}

		if(szTextID)
		{
			Asura_PrintfToBfr(szText,"Name : %s",sizeof(szText),true,szTextID);	
		}
		else
		{
			strcpy(szText,"Name : *Unknown*");
		}

		m_pxOverlay->UpdateDialogLabel(Asura_GetHashID("SelectText"),szText);
	}
}


void ToolsAVP_Entity_CommandPost::AddFilesToExportList( List<Asura_String>& xNameList ) const
{
	PARENT::AddFilesToExportList( xNameList );

	// Note: capture point object is a skin and will be added by parent exporting
	
	// Add PFX & icons skins
	for( u_int uTeam=0; uTeam<AVP_Settings::AVP_NUM_TEAMS; ++uTeam )
	{
		Tools_Mode_PFX::AddTemplateToExportListByHash( &xNameList, AVP_ClientEntity_CommandPost::s_auPulsePFX[uTeam], false );
		Tools_Mode_PFX::AddTemplateToExportListByHash( &xNameList, AVP_ClientEntity_CommandPost::s_auBeamPFXCaptured[uTeam], false );
		Tools_Mode_PFX::AddTemplateToExportListByHash( &xNameList, AVP_ClientEntity_CommandPost::s_auBeamPFXNeutral[uTeam], false );
		Tools_Level_Export::AddSkinToExportList( AVP_ClientEntity_CommandPost::s_auIconSkinIDs[uTeam] );
	}

	Tools_Level_Export::AddAnimToExportList( AVP_ClientEntity_CommandPost::s_uIconAnimID ); 
	Tools_Level_Export::AddAnimToExportList( AVP_ClientEntity_CommandPost::s_uOpenAnimID ); 

	// add sounds
	Tools_Level_Export::AddSoundEventToExportList( Asura_GetHashID("command_post_change"), "command_post_change" );
	Tools_Level_Export::AddSoundEventToExportList( Asura_GetHashID("command_post_hum"), "command_post_hum" );
}


bool ToolsAVP_Entity_CommandPost::HandlesThisEvent( int iEventID ) const
{
	switch( iEventID )
	{
		case AVP_ENTITY_EVENT_CAPTURE_POINT_ONCAPTUREDBYTEAM1 : return true;
		case AVP_ENTITY_EVENT_CAPTURE_POINT_ONCAPTUREDBYTEAM2 : return true;
		case AVP_ENTITY_EVENT_CAPTURE_POINT_ONCAPTUREDBYTEAM3 : return true;
		case AVP_ENTITY_EVENT_CAPTURE_POINT_ONCONTESTED : return true;
		case AVP_ENTITY_EVENT_CAPTURE_POINT_ONNEUTRALIZED: return true; 
		default : break;
	}

	return PARENT::HandlesThisEvent( iEventID );
}

bool ToolsAVP_Entity_CommandPost::HandlesThisMessage( int messageID )
{
	switch (messageID)
	{
		case AVP_MSGID_CHANGE_OWNERSHIP_TO_TEAM1:
		case AVP_MSGID_CHANGE_OWNERSHIP_TO_TEAM2:
		case AVP_MSGID_CHANGE_OWNERSHIP_TO_TEAM3:
		case AVP_MSGID_CAPTUREPOINT_LOCK:
		case AVP_MSGID_CAPTUREPOINT_UNLOCK:
		{
			return true;	
			break;
		}
		default :	break;
	}
	
	return PARENT::HandlesThisMessage( messageID );
}

void ToolsAVP_Entity_CommandPost::PrintInfo( float& x, float& y )
{
	PARENT::PrintInfo(x,y);

	AVP_ServerEntity_CommandPost* pxCmdPst = AVP_ServerEntity_CommandPost::FindEntity( GetGuid() );
	if ( pxCmdPst )
	{
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Capture Time: %2.f", pxCmdPst->m_fCaptureTime );
		
		char szWho[100];
		Asura_PrintfToBfr(szWho, "%s", 100, true, (pxCmdPst->m_bCaptureByBotDisabled) ? "Non-Bots" : "Anyone" );
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Capturable by: %s", (pxCmdPst->m_bCaptureDisabled) ? "No-one" : szWho );

		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Uses Orientation: %s", (pxCmdPst->IsUsingOrientation()) ? "True" : "False" );

		char szChild[100];
		Asura_PrintfToBfr(szChild, "%s", 100, true, (pxCmdPst->GetChildNodes().HasValidProbabilities()) ? "Valid Probabilities" : "Invalid Probabilities" );
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Child Nodes: %s", (pxCmdPst->GetChildNodes().IsEmpty()) ? "None" : szChild );
	}
}

void ToolsAVP_Entity_CommandPost::Create()
{
	const char* const szLongName = "Command Post";
	const char* const szShortName = "CmdPost";

	// Get a new GUID
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();
	
	// Create the server object and set its data.
	Asura_Entity* pxEntity = AVP_ServerEntity_CommandPost::Create( uGuid );
	AVP_ServerEntity_CommandPost* pxServerObject = AVP_ServerEntity_CommandPost::FindEntity( pxEntity->GetGuid() );
	if( !pxServerObject ) return;

	// Create the physics object 
	pxServerObject->SetupPhysicsObject( ASURA_PHYSICS_OBJECT_STATIC, false );

	// Get the camera position
	const Asura_Vector_3 xPosition = *Asura_Camera::GetPosition();

	pxServerObject->SetPosition(xPosition);

	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName(szLongName, szShortName, uGuid);
	pxServerObject->ToolsData = pxToolsData;

	// Create the tools object
	Tools_Object* pxToolsObject = new ToolsAVP_Entity_CommandPost(pxServerObject);
	Tools_Object_System::SelectObject(pxToolsObject);

	Tools_SaveAsura::SetEntityModified();
}

bool ToolsAVP_Entity_CommandPost::CaptureByBotsButtonIsEnabled()
{
	const Asura_Guid				uGuid			= Tools_Object_System::SelectedObject->GetGuid();
	AVP_ServerEntity_CommandPost*	pxServerEntity	= AVP_ServerEntity_CommandPost::FindEntity(uGuid);
    
	if (pxServerEntity) return !pxServerEntity->m_bCaptureDisabled;
	return true;
}

void ToolsAVP_Entity_CommandPost::AddNewChildNode()
{
	//if (s_pxObjectChooser)
	//{
	//	delete s_pxObjectChooser;
	//	s_pxObjectChooser = NULL;
	//}

	//AVP_ServerEntity_CommandPost* pxCommandPost = AVP_ServerEntity_CommandPost::FindEntity( s_pxThis->GetGuid() );
	//Asura_Assert(pxCommandPost,"Couldnt Find our commandpost server entity");
	//if (pxCommandPost)
	//{
	//	if (pxCommandPost->m_xChildNodes.IsFull())
	//	{
	//		//List is full... warn user
	//		char szMessage[100];
	//		Asura_PrintfToBfr(szMessage, "The maximum limit of %d children has been reached. Please increase s_uMAX_NODES.", 100, true, pxCommandPost->m_xChildNodes.s_uMAX_NODES);
	//		Message_Overlay(Asura_String(szMessage)).DoModal();
	//	}
	//	else
	//	{
	//		s_pxObjectChooser = new ToolsAVP_ChooseObject( TOT_COMMAND_POST_NODE, OnSelectNewChildNode );
	//		Tools_Object_System::ChooseObject( "Select CommandPost Node", s_pxObjectChooser );
	//	}

	//}

	//s_pxThis->SetEntityModified();
}

void ToolsAVP_Entity_CommandPost::CreateNewChildNode()
{
	//// Save currently selected object so we can restore it after we're done
	//Tools_Object* pxOldSelection = Tools_Object_System::SelectedObject;

	//AVP_ServerEntity_CommandPost* pxCommandPost = AVP_ServerEntity_CommandPost::FindEntity( s_pxThis->GetGuid() );
	//Asura_Assert(pxCommandPost,"Couldnt Find our commandpost server entity");
	//if (pxCommandPost)
	//{
	//	if (pxCommandPost->m_xChildNodes.IsFull())
	//	{
	//		//List is full... warn user
	//		char szMessage[100];
	//		Asura_PrintfToBfr(szMessage, "The maximum limit of %d children has been reached. Please increase s_uMAX_NODES.", 100, true, pxCommandPost->m_xChildNodes.s_uMAX_NODES);
	//		Message_Overlay(Asura_String(szMessage)).DoModal();
	//	}
	//	else
	//	{
	//		// Create new command post node, then link to... this is kind of round-the-houses... 
	//		ToolsAVP_Entity_CommandPost_Node::Create();
	//		Tools_Object* pxNewSelection = Tools_Object_System::SelectedObject;
	//		ToolsAVP_Entity_CommandPost_Node* pxNewNode = ToolsAVP_Entity_CommandPost_Node::FindEntity( pxNewSelection->GetGuid() );

	//		// If Cursor position is valid, create it there, otherwise use the camera position.
	//		Asura_Vector_3 xCommandNodePosition = Asura_Maths::ZeroVector3;
	//		int iDummy1, iDummy2;
	//		if ( Tools_LOS::FindPolyAndPositionUnderCursor( iDummy1, iDummy2, xCommandNodePosition ) )
	//		{
	//			pxNewNode->Position = xCommandNodePosition;
	//			pxNewNode->UpdatePositionInAsuraData( false );
	//		}

	//		bool bResult = pxCommandPost->m_xChildNodes.AddNode( pxNewNode->Entity->GetGuid(), 1 );
	//		Asura_Assert(bResult, "Couldn't add child node");
	//	}
	//}

	//// Restore previously selected object - likely to be the command post object we called this function from...
	//Tools_Object_System::SelectObject(pxOldSelection);
	//pxOldSelection->SetEntityModified();
}

// RWM - 2007-10-09 - When we're copying CommandPost_Node entities, we don't need to go through the object choosing steps above and can just link directly
void ToolsAVP_Entity_CommandPost::AddNewChildNode( Asura_Guid uGuid, u_int uProbability )
{
	//AVP_ServerEntity_CommandPost* pxCommandPost = AVP_ServerEntity_CommandPost::FindEntity(GetGuid());
	//Asura_Assert(pxCommandPost,"Couldnt Find our commandpost server entity");
	//if (pxCommandPost)
	//{
	//	if ( pxCommandPost->m_xChildNodes.HasNode( uGuid ) )
	//	{
	//		// Already exists... Warn User
	//		char szMessage[100];
	//		Asura_PrintfToBfr(szMessage, "A Child node with Guid %d already exists. It will not be re-added.", 100, true, uGuid);
	//		Message_Overlay(Asura_String(szMessage)).DoModal();
	//	}
	//	else
	//	{
	//		// Add it
	//		bool bResult = pxCommandPost->m_xChildNodes.AddNode( uGuid, uProbability );
	//		Asura_Assert(bResult, "Couldn't add child node");
	//	}
	//}

	//SetEntityModified();
}


void ToolsAVP_Entity_CommandPost::DeleteChildNode( int iGuid )
{
	Asura_Assert(s_pxThis,"no pointer to self");
	if (!s_pxThis) return;

	AVP_ServerEntity_CommandPost* pxCommandPost = AVP_ServerEntity_CommandPost::FindEntity( s_pxThis->GetGuid() );
	Asura_Assert(pxCommandPost,"no pointer to server entity");
	if (pxCommandPost)
	{
		bool bResult = pxCommandPost->m_xChildNodes.RemoveNode( iGuid );
		Asura_Assert(bResult,"Couldnt remove child node");

		if (bResult)
		{
			// s_pxList will be cleared on deselection
			ToolsAVP_Entity_CommandPost* pxThis = s_pxThis;
			Tools_Object_System::SelectObject( NULL );
			Tools_Object_System::SelectObject( pxThis );
			pxThis->SetEntityModified();
		}
	}
}

void ToolsAVP_Entity_CommandPost::GotoChildNode( int iGuid )
{
	//ToolsAVP_Entity_CommandPost_Node* pxChild = ToolsAVP_Entity_CommandPost_Node::FindEntity(iGuid);
	//Asura_Assert(pxChild,"Couldnt find the child node");
	//Tools_Object_System::SelectObject(pxChild);
}


void ToolsAVP_Entity_CommandPost::OnSelectNewChildNode( Tools_Object* pxObject )
{
	if (s_pxThis && pxObject)
	{
		// s_pxList will be cleared on deselection
		ToolsAVP_Entity_CommandPost* pxThis = s_pxThis;

		// Add us to the command post's child list
		AVP_ServerEntity_CommandPost* pxCommandPost = AVP_ServerEntity_CommandPost::FindEntity(pxThis->GetGuid());
		Asura_Assert(pxCommandPost,"Couldnt Find our commandpost server entity");
		if (pxCommandPost)
		{
			if ( pxCommandPost->m_xChildNodes.HasNode(pxObject->GetGuid()) )
			{
				// Already exists... Warn User
				char szMessage[100];
				Asura_PrintfToBfr(szMessage, "A Child node with Guid %d already exists. It will not be re-added.", 100, true, pxObject->GetGuid());
				Message_Overlay(Asura_String(szMessage)).DoModal();
			}
			else
			{
				// Add it
				bool bResult = pxCommandPost->m_xChildNodes.AddNode(pxObject->GetGuid(), 1);
				Asura_Assert(bResult, "Couldn't add child node");

				// Reselect us so we can repopulate the child section of the overlay (lazy but tidy!)
				Tools_Object_System::SelectObject(NULL);
				Tools_Object_System::SelectObject(pxThis);
				pxThis->SetEntityModified();
			}
		}
	}
}

////////////// ADD LINK
void ToolsAVP_Entity_CommandPost::GuardZone0AddLink()
{
	s_uGuardZoneLink = 0;
	SelectGuardZoneAddLink();
}
void ToolsAVP_Entity_CommandPost::GuardZone1AddLink()
{
	s_uGuardZoneLink = 1;
	SelectGuardZoneAddLink();
}
void ToolsAVP_Entity_CommandPost::GuardZone2AddLink()
{
	s_uGuardZoneLink = 2;
	SelectGuardZoneAddLink();
}
void ToolsAVP_Entity_CommandPost::GuardZone3AddLink()
{
	s_uGuardZoneLink = 3;
	SelectGuardZoneAddLink();
}
/////////////////// SWAP POSITION
void ToolsAVP_Entity_CommandPost::GuardZoneSwap0()
{
	s_uGuardZoneLink = 0;
	SelectGuardZoneMoveLink();
}
void ToolsAVP_Entity_CommandPost::GuardZoneSwap1()
{
	s_uGuardZoneLink = 1;
	SelectGuardZoneMoveLink();
}
void ToolsAVP_Entity_CommandPost::GuardZoneSwap2()
{
	s_uGuardZoneLink = 2;
	SelectGuardZoneMoveLink();
}
/////////////////// GOTO
void ToolsAVP_Entity_CommandPost::GuardZone0GoTo()
{
	s_uGuardZoneLink = 0;
	MoveCameraToGuardZone();
}
void ToolsAVP_Entity_CommandPost::GuardZone1GoTo()
{
	s_uGuardZoneLink = 1;
	MoveCameraToGuardZone();
}
void ToolsAVP_Entity_CommandPost::GuardZone2GoTo()
{
	s_uGuardZoneLink = 2;
	MoveCameraToGuardZone();
}
void ToolsAVP_Entity_CommandPost::GuardZone3GoTo()
{
	s_uGuardZoneLink = 3;
	MoveCameraToGuardZone();
}

void ToolsAVP_Entity_CommandPost::SelectGuardZoneMoveLink()
{

	ToolsAVP_Entity_CommandPost* pxToolsCommandPost = ToolsAVP_Entity_CommandPost::FindEntity( s_pxThis->GetGuid() );
	Asura_Assert(pxToolsCommandPost,"Couldn't find our command post entity");
	if (pxToolsCommandPost)
	{
	
		Asura_Guid uTempHold = ASURA_GUID_UNREGISTERED;

		s_pxThis->SetEntityModified();
		uTempHold = pxToolsCommandPost->m_auLinkedGuardZones[s_uGuardZoneLink];
		pxToolsCommandPost->m_auLinkedGuardZones[s_uGuardZoneLink] = pxToolsCommandPost->m_auLinkedGuardZones[s_uGuardZoneLink+1];
		pxToolsCommandPost->m_auLinkedGuardZones[s_uGuardZoneLink+1] = uTempHold;
		pxToolsCommandPost->UpdateGuardZoneDialogueText(s_uGuardZoneLink);
		pxToolsCommandPost->UpdateGuardZoneDialogueText(s_uGuardZoneLink+1);
	}
}

void ToolsAVP_Entity_CommandPost::SelectGuardZoneAddLink()
{
	//if (s_pxObjectChooser)
	//{
	//	delete s_pxObjectChooser;
	//	s_pxObjectChooser = NULL;
	//}

	//AVP_ServerEntity_CommandPost* pxCommandPost = AVP_ServerEntity_CommandPost::FindEntity( s_pxThis->GetGuid() );
	//Asura_Assert(pxCommandPost,"Couldnt Find our commandpost server entity");
	//if (pxCommandPost)
	//{
	//	s_pxObjectChooser = new ToolsAVP_ChooseObject( TOT_Entity_GuardZone, OnSelectGuardZone );
	//	Tools_Object_System::ChooseObject( "Select CommandPost Node", s_pxObjectChooser );
	//}

	//s_pxThis->SetEntityModified();
}

void ToolsAVP_Entity_CommandPost::OnSelectGuardZone( Tools_Object* pxObject )
{

	if ( s_pxThis )
	{

		// s_pxList will be cleared on deselection
		ToolsAVP_Entity_CommandPost* pxThis = s_pxThis;

		// Add us to the command post's child list
		AVP_ServerEntity_CommandPost* pxCommandPost = AVP_ServerEntity_CommandPost::FindEntity(pxThis->GetGuid());
		ToolsAVP_Entity_CommandPost*  pxToolsCommandPost = ToolsAVP_Entity_CommandPost::FindEntity(pxThis->GetGuid());
		Asura_Assert((pxCommandPost && pxToolsCommandPost),"Couldnt Find our commandpost server entity");
		if (pxCommandPost || pxToolsCommandPost)
		{

			// If we did not click on anything then we remove the existing link
			if (!pxObject)
			{
				pxToolsCommandPost->AddGuardZoneLink(ASURA_GUID_UNREGISTERED, s_uGuardZoneLink);
				pxToolsCommandPost->UpdateGuardZoneDialogueText(s_uGuardZoneLink);
			}
			else
			{

				// Make sure we aren't already connected to this GuardZone
				if ( !pxToolsCommandPost->AreWeLinkedToThisGuardZone(pxObject->GetGuid()) )
				{
					// Add it
					pxToolsCommandPost->AddGuardZoneLink(pxObject->GetGuid(), s_uGuardZoneLink);
					pxToolsCommandPost->UpdateGuardZoneDialogueText(s_uGuardZoneLink);
					// Reselect us so we can repopulate the child section of the overlay (lazy but tidy!)
					Tools_Object_System::SelectObject(NULL);
					Tools_Object_System::SelectObject(pxThis);
					pxThis->SetEntityModified();
				}
				else
				{
					Asura_Assert( false, "We are already linked to this GuardZone" );
				}
			}
		}
	}
}

void ToolsAVP_Entity_CommandPost::AddGuardZoneLink(Asura_Guid uGuid, u_int iIndex)
{
	m_auLinkedGuardZones[iIndex] = uGuid;
}

bool ToolsAVP_Entity_CommandPost::AreWeLinkedToThisGuardZone(Asura_Guid uGuid)
{

	for (u_int i = 0; i < AVP_COMMANDPOST_MAX_NUM_LINKED_GUARDZONES; ++i)
	{

		if ( m_auLinkedGuardZones[i] == uGuid )
		{

			return true;
		}
	}
	return false;
}

void ToolsAVP_Entity_CommandPost::ValidateGuardZoneLinks()
{

	AVP_ServerNode_GuardZone *pxGuardZone = NULL;

	// Search through all Links and make sure they link to a valid GuardZone
	for (u_int i = 0; i < AVP_COMMANDPOST_MAX_NUM_LINKED_GUARDZONES; ++i)
	{

		if ( m_auLinkedGuardZones[i] != ASURA_GUID_UNREGISTERED )
		{

			pxGuardZone = AVP_ServerNode_GuardZone::FindEntity( m_auLinkedGuardZones[i] );
			if ( !pxGuardZone )
			{
				Asura_Assertf( false, ("This CommandPost( %u ) Had an invalid GuardZone( %u ) link. It has been removed", GetGuid(), m_auLinkedGuardZones[i]) );
				m_auLinkedGuardZones[i] = ASURA_GUID_UNREGISTERED;
			}
		}
	}
}

void ToolsAVP_Entity_CommandPost::MoveCameraToGuardZone()
{
	// Move the camera to the GuardZone link, move it back slightly so you can see the GuardZone in view
	ToolsAVP_Entity_CommandPost* pxToolsCommandPost = ToolsAVP_Entity_CommandPost::FindEntity( s_pxThis->GetGuid() );
	Asura_Assert(pxToolsCommandPost,"Couldn't find our command post entity");
	if (pxToolsCommandPost)
	{
		AVP_ServerNode_GuardZone *pxGuardZone = AVP_ServerNode_GuardZone::FindEntity( pxToolsCommandPost->m_auLinkedGuardZones[s_uGuardZoneLink] );
		Asura_Assert(pxToolsCommandPost,"Couldn't find our GuardZone entity for this link");
		if ( pxGuardZone )
		{
			Asura_Vector_3 xGuardZonePos = pxGuardZone->GetPosition();
			const Asura_Matrix_3x3 *xCameraOrientation = Asura_Camera::GetOrientation();
			Asura_Vector_3 xForward = Asura_Vector_3(xCameraOrientation->e13,xCameraOrientation->e23,xCameraOrientation->e33);
			xGuardZonePos -= (xForward*5);
			Tools_Movement::TeleportCameraToPosition(&xGuardZonePos,Asura_Camera::GetOrientation());
		}
	}
}

void ToolsAVP_Entity_CommandPost::UpdateGuardZoneDialogueText(const int iIndex)
{
	//Asura_Dialog*						pxDialogue			= m_apxGuardZoneDialogueText[iIndex];
	//Asura_Overlay*						pxOverlay			= pxDialogue->Overlay;

	//char							szText[256];

	//if ( m_auLinkedGuardZones[iIndex] == ASURA_GUID_UNREGISTERED )
	//{
	//	Asura_PrintfToBfr(szText, "%u: *None", 256, true, iIndex);
	//}
	//else
	//{
	//	Asura_PrintfToBfr(szText, "%u: %u", 256, true, iIndex, m_auLinkedGuardZones[iIndex]);
	//}
	//
	//pxDialogue->UpdateLabel(szText);
	//pxOverlay->CalculateOverlaySize();
}

void ToolsAVP_Entity_CommandPost::Copy(const Tools_Object* pxObject)
{
	PARENT::Copy(pxObject);
	
	const ToolsAVP_Entity_CommandPost* pxFromObject = s_xList.FindEntity( pxObject->GetGuid() );

	if (!pxFromObject)
	{
		Asura_Assert(0, "Failed to find command post node entity to copy from");
		return;
	}

	const AVP_ServerEntity_CommandPost* pxFromEntity = (const AVP_ServerEntity_CommandPost*) AVP_ServerEntity_CommandPost::FindEntity( pxFromObject->Entity->GetGuid() );
	AVP_ServerEntity_CommandPost* pxToEntity = AVP_ServerEntity_CommandPost::FindEntity( Entity->GetGuid() );

	if (!pxFromEntity || !pxToEntity )
	{
		Asura_Assert( 0, "Failed to find command post node entity for copy" );
		return;
	}

	// Tools object data
	m_bCaptureDisabled = pxFromObject->m_bCaptureDisabled;
	m_bCaptureByBotDisabled = pxFromObject->m_bCaptureByBotDisabled;
	m_bDefaultSpawnPoint = pxFromObject->m_bDefaultSpawnPoint;
	m_bUsesOrientation = pxFromObject->m_bUsesOrientation;

	// ServerEntity object data
	pxToEntity->m_eInitialTeam = pxFromEntity->m_eInitialTeam;
	pxToEntity->m_fCaptureTime = pxFromEntity->m_fCaptureTime;
	pxToEntity->m_bCaptureDisabled = pxFromEntity->m_bCaptureDisabled;
	pxToEntity->m_bCaptureByBotDisabled = pxFromEntity->m_bCaptureByBotDisabled;
	pxToEntity->m_bMobileCommandPost = pxFromEntity->m_bMobileCommandPost;
	pxToEntity->m_bDefaultSpawnPoint = pxFromEntity->m_bDefaultSpawnPoint;
	pxToEntity->m_bUsesOrientation = pxFromEntity->m_bUsesOrientation;

	// Now need to copy command node array...
	const AVP_CommandPost_Node_Array& xFromArray = pxFromEntity->GetChildNodes();
	for (u_int i = 0; i < AVP_CommandPost_Node_Array::s_uMAX_NODES; i++)
	{
		const AVP_CommandPost_Node_ArrayElement& xElem = xFromArray.GetNode(i);
		if (xElem.m_uGuid != ASURA_GUID_UNREGISTERED)
		{
			AddNewChildNode( xElem.m_uGuid, xElem.m_uProbability );
		}
	}

	SetEntityModified();
}

void ToolsAVP_Entity_CommandPost::Project_Create(void)
{
	ToolsAVP_Entity_CommandPost::Create();
}

Tools_Object* ToolsAVP_Entity_CommandPost::CreateObjectOfThisType() const
{
	Tools_Object* pxOldSelection = Tools_Object_System::SelectedObject;
	Create();
	Tools_Object* pxNewSelection = Tools_Object_System::SelectedObject;

	return (pxOldSelection!=pxNewSelection) ? pxNewSelection : 0;
}

#ifdef TEST_COMMAND_POST_SPAWNER
void ToolsAVP_Entity_CommandPost::TestSpawner()
{
	if (s_pxThis)
	{
		AVP_ServerEntity_CommandPost* pxCmdPst = AVP_ServerEntity_CommandPost::FindEntity( s_pxThis->GetGuid() );
		if (pxCmdPst)
		{
			for (int i = 0; i < AVP_CommandPost_Node_Array::s_uMAX_NODES; i++ )
			{
				pxCmdPst->m_uCount[i] = 0;
			}

			for (int i = 0; i < 10000; i++)
			{				
				pxCmdPst->GetSpawnPosition(true);
			}
			
			Asura_String xResults("Test Results\n------------\n\n");
			xResults += "      prob      count\n";


			for (int i = 0; i < AVP_CommandPost_Node_Array::s_uMAX_NODES; i++)
			{
				char szText[100];
				Asura_PrintfToBfr(szText, "      %d      %d\n", 100, true, pxCmdPst->m_xChildNodes.GetNodeProbability(i), pxCmdPst->m_uCount[i] );
				xResults += Asura_String(szText);
			}

			xResults += "\n\nDone.";
			Tools_Object_System::SelectObject(NULL);
			Message_Overlay(xResults).DoModal();
		}
	}
}
#endif //__TEST_COMMAND_POST_SPAWNER__

/***************/
/* End of File */
/***************/
