//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_InteractPredatorLock.cpp
//
// Created by AJK 21/10/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// Includes
#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Interact/PredatorLock/ToolsAVP_Entity_InteractPredatorLock.h"

// Asura
#include "Entity/Asura_StaticMessage_System.h"

// AVP
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Interact/PredatorLock/AVP_ServerEntity_InteractPredatorLock.h"

// Tools Asura
#include "Entity/Tools_MessageEditor.h"
#include "Load&Save/Tools_SaveAsura.h"

// Class statics
Asura_Hashed_List<ToolsAVP_Entity_InteractPredatorLock>	ToolsAVP_Entity_InteractPredatorLock::s_xList;

ToolsAVP_Entity_InteractPredatorLock::ToolsAVP_Entity_InteractPredatorLock( AVP_ServerEntity_InteractPredatorLock* pxInteractObject )
: PARENT( pxInteractObject )
, Tools_Entity_PhysicalObject( pxInteractObject )
{
	Type = TOT_Entity_InteractPredatorLock;
	s_xList.AddToList(this);
}

ToolsAVP_Entity_InteractPredatorLock::~ToolsAVP_Entity_InteractPredatorLock()
{
	s_xList.RemoveFromList(this);
}

void ToolsAVP_Entity_InteractPredatorLock::Create()
{
	CreationHelper( "Predator Lock", "PredatorLock", 
		AVP_ServerEntity_InteractPredatorLock::CreateFromTools, Create);
}

Tools_Object* ToolsAVP_Entity_InteractPredatorLock::Create( Asura_Entity* pxEntity )
{
	ToolsAVP_Entity_InteractPredatorLock* pxObject = new ToolsAVP_Entity_InteractPredatorLock( AVP_ServerEntity_InteractPredatorLock::FindEntity( pxEntity->GetGuid() ) );
	pxObject->UpdateToolsData();
	return pxObject;
}

Tools_Object* ToolsAVP_Entity_InteractPredatorLock::CreateObjectOfThisType() const
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("Predator Lock", "PredatorLock", uGuid);

	if(pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_InteractPredatorLock::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_InteractPredatorLock* pxServerObject = new AVP_ServerEntity_InteractPredatorLock(uGuid);
		pxServerObject->ToolsData = pxToolsData;
		ToolsAVP_Entity_InteractPredatorLock *pxToolsObject = new ToolsAVP_Entity_InteractPredatorLock(pxServerObject);
		Tools_Object_System::SelectObject(pxToolsObject);

		pxToolsObject->Position		= *Asura_Camera::GetPosition();
		pxToolsObject->Orientation.Identity();

		Tools_SaveAsura::SetEntityModified();
		return pxToolsObject;
	}
	return NULL;
}

void ToolsAVP_Entity_InteractPredatorLock::AddActionsControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	Asura_Char szBuffer[32];

	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (m_pxInteractObject);

	Asura_PrintfToBfr( szBuffer,"On Reach Position 1 (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxPredatorLock->m_usReachPosition1MessageBlock ) );
	Asura_Dialog* pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnReachPosition1Messages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On Leave Position 1 (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxPredatorLock->m_usLeavePosition1MessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnLeavePosition1Messages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On Reach Position 2 (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxPredatorLock->m_usReachPosition2MessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnReachPosition2Messages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On Leave Position 2 (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxPredatorLock->m_usLeavePosition2MessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnLeavePosition2Messages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On Reach Position 3 (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxPredatorLock->m_usReachPosition3MessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnReachPosition3Messages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On Leave Position 3 (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxPredatorLock->m_usLeavePosition3MessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnLeavePosition3Messages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On Reach Position 4 (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxPredatorLock->m_usReachPosition4MessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnReachPosition4Messages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On Leave Position 4 (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxPredatorLock->m_usLeavePosition4MessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnLeavePosition4Messages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

bool ToolsAVP_Entity_InteractPredatorLock::GetEventDescForMessageBlock( unsigned short usMessageBlockID, Asura_String& xOutputDesc )
{
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (m_pxInteractObject);

	if ( m_pxInteractObject ) 
	{
		if ( pxPredatorLock->m_usReachPosition1MessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Reach Position 1";
			return true;
		}
		else if ( pxPredatorLock->m_usReachPosition2MessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Reach Position 2";
			return true;
		}
		else if ( pxPredatorLock->m_usReachPosition3MessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Reach Position 3";
			return true;
		}
		else if ( pxPredatorLock->m_usReachPosition4MessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Reach Position 4";
			return true;
		}
		else if ( pxPredatorLock->m_usLeavePosition1MessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Leave Position 1";
			return true;
		}
		else if ( pxPredatorLock->m_usLeavePosition2MessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Leave Position 2";
			return true;
		}
		else if ( pxPredatorLock->m_usLeavePosition3MessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Leave Position 3";
			return true;
		}
		else if ( pxPredatorLock->m_usLeavePosition4MessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Leave Position 4";
			return true;
		}
		return PARENT::GetEventDescForMessageBlock( usMessageBlockID, xOutputDesc );
	}
	else
	{
		Asura_Assert(false, "No Interaction object!");
		return false;
	}
}

void ToolsAVP_Entity_InteractPredatorLock::EditOnReachPosition1Messages()
{
	Asura_Assert( s_pxThis, "No current tools predator tools object!, will crash!" );
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxPredatorLock, "No predator lock; will crash!" );
	EditMessageBlock( "On Reach Position 1", &pxPredatorLock->m_usReachPosition1MessageBlock );
}

void ToolsAVP_Entity_InteractPredatorLock::EditOnLeavePosition1Messages()
{
	Asura_Assert( s_pxThis, "No current tools predator tools object!, will crash!" );
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxPredatorLock, "No predator lock; will crash!" );
	EditMessageBlock( "On Leave Position 1", &pxPredatorLock->m_usLeavePosition1MessageBlock );
}

void ToolsAVP_Entity_InteractPredatorLock::EditOnReachPosition2Messages()
{
	Asura_Assert( s_pxThis, "No current tools predator tools object!, will crash!" );
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxPredatorLock, "No predator lock; will crash!" );
	EditMessageBlock( "On Reach Position 1", &pxPredatorLock->m_usReachPosition2MessageBlock );
}
void ToolsAVP_Entity_InteractPredatorLock::EditOnLeavePosition2Messages()
{
	Asura_Assert( s_pxThis, "No current tools predator tools object!, will crash!" );
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxPredatorLock, "No predator lock; will crash!" );
	EditMessageBlock( "On Leave Position 1", &pxPredatorLock->m_usLeavePosition2MessageBlock );
}

void ToolsAVP_Entity_InteractPredatorLock::EditOnReachPosition3Messages()
{
	Asura_Assert( s_pxThis, "No current tools predator tools object!, will crash!" );
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxPredatorLock, "No predator lock; will crash!" );
	EditMessageBlock( "On Reach Position 1", &pxPredatorLock->m_usReachPosition3MessageBlock );
}
void ToolsAVP_Entity_InteractPredatorLock::EditOnLeavePosition3Messages()
{
	Asura_Assert( s_pxThis, "No current tools predator tools object!, will crash!" );
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxPredatorLock, "No predator lock; will crash!" );
	EditMessageBlock( "On Leave Position 1", &pxPredatorLock->m_usLeavePosition3MessageBlock );
}

void ToolsAVP_Entity_InteractPredatorLock::EditOnReachPosition4Messages()
{
	Asura_Assert( s_pxThis, "No current tools predator tools object!, will crash!" );
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxPredatorLock, "No predator lock; will crash!" );
	EditMessageBlock( "On Reach Position 1", &pxPredatorLock->m_usReachPosition4MessageBlock );
}

void ToolsAVP_Entity_InteractPredatorLock::EditOnLeavePosition4Messages()
{
	Asura_Assert( s_pxThis, "No current tools predator tools object!, will crash!" );
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxPredatorLock, "No predator lock; will crash!" );
	EditMessageBlock( "On Leave Position 1", &pxPredatorLock->m_usLeavePosition4MessageBlock );
}

void ToolsAVP_Entity_InteractPredatorLock::EditMessageBlock(const char* szText, u_short* pusMessageBlock)
{
	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(pusMessageBlock);
	#else
	Tools_Object_System::EditMessageBlock(szText, pusMessageBlock);
	#endif
}

bool ToolsAVP_Entity_InteractPredatorLock::OwnsThisMessageBlock( unsigned short usMessageBlockID )
{
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (m_pxInteractObject);
	if (     pxPredatorLock 
		&& (   pxPredatorLock->m_usReachPosition1MessageBlock == usMessageBlockID 
		    || pxPredatorLock->m_usLeavePosition1MessageBlock == usMessageBlockID
			|| pxPredatorLock->m_usReachPosition2MessageBlock == usMessageBlockID
			|| pxPredatorLock->m_usLeavePosition2MessageBlock == usMessageBlockID
			|| pxPredatorLock->m_usReachPosition3MessageBlock == usMessageBlockID
			|| pxPredatorLock->m_usLeavePosition3MessageBlock == usMessageBlockID
			|| pxPredatorLock->m_usReachPosition4MessageBlock == usMessageBlockID
			|| pxPredatorLock->m_usLeavePosition4MessageBlock == usMessageBlockID
		   ) 
	   )
	{
		return true;
	}

	return PARENT::OwnsThisMessageBlock( usMessageBlockID );
}

void ToolsAVP_Entity_InteractPredatorLock::Destroy()
{
	AVP_ServerEntity_InteractPredatorLock* pxPredatorLock = static_cast<AVP_ServerEntity_InteractPredatorLock*> (m_pxInteractObject);

	//delete all the static messages
	Asura_StaticMessage_System::DeleteMessageBlock(pxPredatorLock->m_usReachPosition1MessageBlock);
	pxPredatorLock->m_usReachPosition1MessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxPredatorLock->m_usReachPosition2MessageBlock);
	pxPredatorLock->m_usReachPosition2MessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxPredatorLock->m_usReachPosition3MessageBlock);
	pxPredatorLock->m_usReachPosition3MessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxPredatorLock->m_usReachPosition4MessageBlock);
	pxPredatorLock->m_usReachPosition4MessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxPredatorLock->m_usLeavePosition1MessageBlock);
	pxPredatorLock->m_usLeavePosition1MessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxPredatorLock->m_usLeavePosition2MessageBlock);
	pxPredatorLock->m_usLeavePosition2MessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxPredatorLock->m_usLeavePosition3MessageBlock);
	pxPredatorLock->m_usLeavePosition3MessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxPredatorLock->m_usLeavePosition4MessageBlock);
	pxPredatorLock->m_usLeavePosition4MessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	PARENT::Destroy();
}

bool ToolsAVP_Entity_InteractPredatorLock::HandlesThisMessage(int iMessageID)
{
	switch(iMessageID)
	{
	case AVP_MSGID_PREDATOR_LOCK_POSITION_F:
	case AVP_MSGID_PREDATOR_LOCK_POSITION_T:
	case AVP_MSGID_PREDATOR_LOCK_POSITION_X:
	case AVP_MSGID_PREDATOR_LOCK_POSITION_O:
		{
			// This entity can receive change ownership messages
			return true;
		}
	default:
		{
			return PARENT::HandlesThisMessage(iMessageID); 
		}
	}
}