//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_InteractBatteryReceptacle.cpp
//
// Created by AJK 24/11/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// Includes
#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Interact/ToolReceptacle/BatteryReceptacle/ToolsAVP_Entity_InteractBatteryReceptacle.h"

// Asura
#include "Entity/Asura_StaticMessage_System.h"

// AVP
#include "Entity/PhysicalObject/Interact/ToolReceptacle/BatteryReceptacle/AVP_ServerEntity_InteractBatteryReceptacle.h"
#include "Entity/PhysicalObject/Pickup/AVP_ServerEntity_Pickup.h"

// Tools Asura
#include "Entity/Tools_MessageEditor.h"
#include "Load&Save/Tools_SaveAsura.h"

// Class statics
Asura_Hashed_List<ToolsAVP_Entity_InteractBatteryReceptacle>	ToolsAVP_Entity_InteractBatteryReceptacle::s_xList;

ToolsAVP_Entity_InteractBatteryReceptacle::ToolsAVP_Entity_InteractBatteryReceptacle( AVP_ServerEntity_InteractBatteryReceptacle* pxInteractObject )
: PARENT( pxInteractObject )
, Tools_Entity_PhysicalObject( pxInteractObject )
{
	Type = TOT_Entity_InteractBatteryReceptacle;
	s_xList.AddToList(this);
}

ToolsAVP_Entity_InteractBatteryReceptacle::~ToolsAVP_Entity_InteractBatteryReceptacle()
{
	s_xList.RemoveFromList(this);
}

void ToolsAVP_Entity_InteractBatteryReceptacle::Create()
{
	CreationHelper( "Battery Receptacle", "BatteryReceptacle", 
		AVP_ServerEntity_InteractBatteryReceptacle::CreateFromTools, Create);
}

Tools_Object* ToolsAVP_Entity_InteractBatteryReceptacle::Create( Asura_Entity* pxEntity )
{
	ToolsAVP_Entity_InteractBatteryReceptacle* pxObject = new ToolsAVP_Entity_InteractBatteryReceptacle( AVP_ServerEntity_InteractBatteryReceptacle::FindEntity( pxEntity->GetGuid() ) );
	pxObject->UpdateToolsData();
	return pxObject;
}

Tools_Object* ToolsAVP_Entity_InteractBatteryReceptacle::CreateObjectOfThisType() const
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("Battery Receptacle", "BatteryReceptacle", uGuid);

	if(pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_InteractBatteryReceptacle::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_InteractBatteryReceptacle* pxServerObject = new AVP_ServerEntity_InteractBatteryReceptacle(uGuid);
		pxServerObject->ToolsData = pxToolsData;
		ToolsAVP_Entity_InteractBatteryReceptacle *pxToolsObject = new ToolsAVP_Entity_InteractBatteryReceptacle(pxServerObject);
		Tools_Object_System::SelectObject(pxToolsObject);

		pxToolsObject->Position		= *Asura_Camera::GetPosition();
		pxToolsObject->Orientation.Identity();

		Tools_SaveAsura::SetEntityModified();
		return pxToolsObject;
	}
	return NULL;
}

void ToolsAVP_Entity_InteractBatteryReceptacle::AddActionsControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddActionsControls(pxOverlay, xOffset);

	Asura_Char szBuffer[32];

	AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = static_cast<AVP_ServerEntity_InteractBatteryReceptacle*> (m_pxInteractObject);

	Asura_PrintfToBfr( szBuffer,"On marine insert battery(%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxBatteryReceptacle->m_usMarineInsertBatteryMessageBlock ) );
	Asura_Dialog* pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnMarineInsertBatteryMessages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On marine remove battery (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxBatteryReceptacle->m_usMarineRemoveBatteryMessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnMarineRemoveBatteryMessages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On predator use battery(%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxBatteryReceptacle->m_usPredatorUseBatteryMessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnPredatorUseBatteryMessages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr( szBuffer,"On alien destroy battery (%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxBatteryReceptacle->m_usAlienDestroyBatteryMessageBlock ) );
	pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnAlienDestroyBatteryMessages );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

bool ToolsAVP_Entity_InteractBatteryReceptacle::GetEventDescForMessageBlock( unsigned short usMessageBlockID, Asura_String& xOutputDesc )
{
	AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = static_cast<AVP_ServerEntity_InteractBatteryReceptacle*> (m_pxInteractObject);

	if ( m_pxInteractObject ) 
	{
		if ( pxBatteryReceptacle->m_usMarineInsertBatteryMessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Marine Insert Battery";
			return true;
		}
		else if ( pxBatteryReceptacle->m_usMarineRemoveBatteryMessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Marine Remove Battery";
			return true;
		}
		else if ( pxBatteryReceptacle->m_usAlienDestroyBatteryMessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Alien Destroy Battery";
			return true;
		}
		else if ( pxBatteryReceptacle->m_usPredatorUseBatteryMessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Predator Use Battery";
			return true;
		}
		return PARENT::GetEventDescForMessageBlock(usMessageBlockID, xOutputDesc);
	}
	else
	{
		Asura_Assert(false, "No Interaction object!");
		return false;
	}
}

void ToolsAVP_Entity_InteractBatteryReceptacle::EditOnMarineInsertBatteryMessages()
{
	Asura_Assert( s_pxThis, "No current tools battery receptacle object!, will crash!" );
	AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = static_cast<AVP_ServerEntity_InteractBatteryReceptacle*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxBatteryReceptacle, "No battery receptacle; will crash!" );
	
	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(&pxBatteryReceptacle->m_usMarineInsertBatteryMessageBlock);
	#else
	Tools_Object_System::EditMessageBlock( "On Do Action", &pxBatteryReceptacle->m_usMarineInsertBatteryMessageBlock );
	#endif
}

void ToolsAVP_Entity_InteractBatteryReceptacle::EditOnMarineRemoveBatteryMessages()
{
	Asura_Assert( s_pxThis, "No current tools battery receptacle object!, will crash!" );
	AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = static_cast<AVP_ServerEntity_InteractBatteryReceptacle*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxBatteryReceptacle, "No battery receptacle; will crash!" );

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(&pxBatteryReceptacle->m_usMarineRemoveBatteryMessageBlock);
	#else
	Tools_Object_System::EditMessageBlock( "On Undo Action", &pxBatteryReceptacle->m_usMarineRemoveBatteryMessageBlock );
	#endif
}

void ToolsAVP_Entity_InteractBatteryReceptacle::EditOnAlienDestroyBatteryMessages()
{
	Asura_Assert( s_pxThis, "No current tools battery receptacle object!, will crash!" );
	AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = static_cast<AVP_ServerEntity_InteractBatteryReceptacle*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxBatteryReceptacle, "No battery receptacle; will crash!" );
	
	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(&pxBatteryReceptacle->m_usAlienDestroyBatteryMessageBlock);
	#else
	Tools_Object_System::EditMessageBlock( "On Do Action", &pxBatteryReceptacle->m_usAlienDestroyBatteryMessageBlock );
	#endif
}

void ToolsAVP_Entity_InteractBatteryReceptacle::EditOnPredatorUseBatteryMessages()
{
	Asura_Assert( s_pxThis, "No current tools battery receptacle object!, will crash!" );
	AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = static_cast<AVP_ServerEntity_InteractBatteryReceptacle*> (s_pxThis->GetInteractObject());
	Asura_Assert( pxBatteryReceptacle, "No battery receptacle; will crash!" );

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(&pxBatteryReceptacle->m_usPredatorUseBatteryMessageBlock);
	#else
	Tools_Object_System::EditMessageBlock( "On Do Action", &pxBatteryReceptacle->m_usPredatorUseBatteryMessageBlock );
	#endif
}

bool ToolsAVP_Entity_InteractBatteryReceptacle::OwnsThisMessageBlock( unsigned short usMessageBlockID )
{
	AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = static_cast<AVP_ServerEntity_InteractBatteryReceptacle*> (m_pxInteractObject);
	if (    pxBatteryReceptacle 
		&& (   pxBatteryReceptacle->m_usMarineInsertBatteryMessageBlock == usMessageBlockID 
			|| pxBatteryReceptacle->m_usMarineRemoveBatteryMessageBlock == usMessageBlockID
			|| pxBatteryReceptacle->m_usPredatorUseBatteryMessageBlock  == usMessageBlockID
			|| pxBatteryReceptacle->m_usAlienDestroyBatteryMessageBlock == usMessageBlockID
		   ) 
	   )
	{
		return true;
	}

	return PARENT::OwnsThisMessageBlock( usMessageBlockID );
}

void ToolsAVP_Entity_InteractBatteryReceptacle::Destroy()
{
	AVP_ServerEntity_InteractBatteryReceptacle* pxBatteryReceptacle = static_cast<AVP_ServerEntity_InteractBatteryReceptacle*> (m_pxInteractObject);

	//delete all the static messages
	Asura_StaticMessage_System::DeleteMessageBlock(pxBatteryReceptacle->m_usMarineInsertBatteryMessageBlock);
	pxBatteryReceptacle->m_usMarineInsertBatteryMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxBatteryReceptacle->m_usMarineRemoveBatteryMessageBlock);
	pxBatteryReceptacle->m_usMarineRemoveBatteryMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxBatteryReceptacle->m_usPredatorUseBatteryMessageBlock);
	pxBatteryReceptacle->m_usPredatorUseBatteryMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	Asura_StaticMessage_System::DeleteMessageBlock(pxBatteryReceptacle->m_usAlienDestroyBatteryMessageBlock);
	pxBatteryReceptacle->m_usAlienDestroyBatteryMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

	PARENT::Destroy();
}