//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_InteractToolReceptacle.cpp
//
// Created by AJK 24/11/2008
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// Includes
#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Interact/ToolReceptacle/ToolsAVP_Entity_InteractToolReceptacle.h"

// AVP
#include "Entity/PhysicalObject/Interact/ToolReceptacle/AVP_ServerEntity_InteractToolReceptacle.h"
#include "Entity/PhysicalObject/Pickup/AVP_ServerEntity_Pickup.h"

// Tools Asura
#include "Load&Save/Tools_SaveAsura.h"

// Tools AVP
#include "ToolsAVP_ChooseObjects.h"

// Class statics
ToolsAVP_Entity_InteractToolReceptacle*						ToolsAVP_Entity_InteractToolReceptacle::s_pxSelectedReceptacle = NULL;	

ToolsAVP_Entity_InteractToolReceptacle::ToolsAVP_Entity_InteractToolReceptacle( AVP_ServerEntity_InteractToolReceptacle* pxInteractObject )
: PARENT( pxInteractObject )
, Tools_Entity_PhysicalObject( pxInteractObject )
, m_pxOverlay(NULL)
{
	Type = TOT_Entity_InteractToolReceptacle;
}

ToolsAVP_Entity_InteractToolReceptacle::~ToolsAVP_Entity_InteractToolReceptacle()
{
}

void ToolsAVP_Entity_InteractToolReceptacle::AddActionsControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	if( AllowToolInside())
	{
		AVP_ServerEntity_InteractToolReceptacle* pxToolReceptacle = static_cast<AVP_ServerEntity_InteractToolReceptacle*> (m_pxInteractObject);

		Asura_Dialog* pxDialog = new Asura_FlagTickbox_Dialog("Tool inside?", xOffset, &pxToolReceptacle->m_uFlags, AVP_ServerEntity_InteractToolReceptacle::FLAG_TOOL_INSIDE, &Changed);
		pxDialog->SetHelpText("Set if the Tool is already inside for use by the predator and alien players )");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("Select Tool", xOffset, SelectTool, &Changed);
		pxDialog->SetHelpText("Enables the selection of a squad for this humanoid.");
		pxDialog->SetIsEnabledFunction(IsToolInside);
		pxOverlay->AddDialog(pxDialog);
		pxDialog->m_iControlID = Asura_GetHashID("SelectTool");
		xOffset.y += Asura_Dialog::DefaultSpacing();

		m_pxOverlay = pxOverlay;

		UpdateToolInsideGuid();
	}
}

void ToolsAVP_Entity_InteractToolReceptacle::SelectTool(void)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_Pickup, SetSelectedTool);
	Tools_Object_System::ChooseObject("Select The Squad To Join", pxChooseObject);
}

bool ToolsAVP_Entity_InteractToolReceptacle::IsToolInside()
{
	AVP_ServerEntity_InteractToolReceptacle* pxEntity = AVP_ServerEntity_InteractToolReceptacle::GetList()->FindObject(ThisEntity->GetGuid());

	return(pxEntity->IsToolInside());
}

void ToolsAVP_Entity_InteractToolReceptacle::SetSelectedTool(Asura_Guid uGuid)
{
	if (s_pxSelectedReceptacle != NULL)
	{
		AVP_ServerEntity_InteractToolReceptacle* pxEntity = AVP_ServerEntity_InteractToolReceptacle::FindEntity(s_pxSelectedReceptacle->GetGuid());

		if (pxEntity)
		{
			if(uGuid != ASURA_GUID_UNREGISTERED)
			{
				pxEntity->SetToolGuid(uGuid);
			}
			else
			{
				pxEntity->SetToolInside(false);
			}
			s_pxSelectedReceptacle->UpdateToolInsideGuid();
		}
	}
}

void ToolsAVP_Entity_InteractToolReceptacle::OnSelection()
{
	s_pxSelectedReceptacle = this;

	PARENT::OnSelection();
}

void ToolsAVP_Entity_InteractToolReceptacle::Update()
{
	AVP_ServerEntity_InteractToolReceptacle* pxToolReceptacle = static_cast<AVP_ServerEntity_InteractToolReceptacle*> (m_pxInteractObject);

	if (pxToolReceptacle)
	{
		if (pxToolReceptacle->GetToolGuid() != ASURA_GUID_UNREGISTERED)
		{
			AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::FindEntity(pxToolReceptacle->GetToolGuid());

			if (!pxPickup)
			{
				pxToolReceptacle->SetToolGuid(ASURA_GUID_UNREGISTERED);
			}
		}
	}
}

void ToolsAVP_Entity_InteractToolReceptacle::UpdateToolInsideGuid()
{
	AVP_ServerEntity_InteractToolReceptacle* pxToolReceptacle = static_cast<AVP_ServerEntity_InteractToolReceptacle*> (m_pxInteractObject);

	if(pxToolReceptacle)
	{
		char szText[100];

		if ( pxToolReceptacle->IsToolInside())
		{
			Asura_PrintfToBfr(szText,"Tool GUID : %d",sizeof(szText),true,pxToolReceptacle->GetToolGuid());	
		}
		else
		{
			strcpy(szText,"Select Tool");
		}
		m_pxOverlay->UpdateDialogLabel(Asura_GetHashID("SelectTool"),szText);
	}
}