//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_InteractObject.h
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef TOOLSAVP_ENTITY_INTERACTOBJECT_H
#define TOOLSAVP_ENTITY_INTERACTOBJECT_H

#include "Objects/Tools_VolumeEdit_Object.h"

#include "Core/Asura_OrientedVolume.h"

#include "Entity/PhysicalObject/ToolsAVP_Entity_PhysicalObject.h"
#include "Objects/ToolsAVP_Object_Type.h"

// Forward declarations
class Tools_Parameter;
class AVP_ServerEntity_InteractObject;

class ToolsAVP_Entity_InteractObject : public ToolsAVP_Entity_PhysicalObject
{
public:
	static void				Create();
	static Tools_Object*	Create(Asura_Entity* pxEntity);
	static ToolsAVP_Entity_InteractObject*	FindEntity(Asura_Guid uGuid)		{ return s_xList.FindEntity(uGuid); }

	AVP_ServerEntity_InteractObject* GetInteractObject() const { return m_pxInteractObject; }

	virtual bool			CanBeCopied() const { return ( Type == TOT_Entity_InteractiveObject ); }
	virtual void			Copy( const Tools_Object* pxObject );
	virtual Tools_Object*	CreateObjectOfThisType() const;

	virtual void			Destroy();

	virtual const char*		GetTypeDescription(){return "Interaction Object";}

	virtual void			AddOverlayControls(Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset);
	virtual void			AddActionsControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset );

	virtual bool			HandlesThisMessage( int iMessageID );
	virtual bool			OwnsThisMessageBlock( unsigned short usMessageBlockID );
	virtual bool			GetEventDescForMessageBlock( unsigned short usMessageBlockID, Asura_String& xOutputDesc );

	virtual void			OnSelection();
	virtual void			OnDeselection();

	virtual void			UpdateToolsData();

	virtual void			UpdateWhileSelected();
	virtual void            UpdateAnimationWhileSelected();
	virtual void			UpdatePositionInAsuraData( bool bUpdateLinkedObjects );

protected:
	ToolsAVP_Entity_InteractObject(AVP_ServerEntity_InteractObject* pxInteractObject);
	virtual ~ToolsAVP_Entity_InteractObject();

	AVP_ServerEntity_InteractObject*	m_pxInteractObject;
	float								m_fAlphaTimer;

	ToolsAVP_Entity_InteractObject	(void);
	ToolsAVP_Entity_InteractObject	(const ToolsAVP_Entity_InteractObject& xOther);
	ToolsAVP_Entity_InteractObject&	operator=	(const ToolsAVP_Entity_InteractObject& xOther);

	Asura_Vector_3															m_xPreviousPositionForVolumes;
	Asura_Quat																m_xPreviousOrientationForVolumes;

	class ToolsAVP_Entity_InteractObject_ActorVolumeEdit_Controller*		m_pxActorVolumeController;
	class ToolsAVP_Entity_InteractObject_OnTargetVolumeEdit_Controller*		m_pxOnTargetVolumeController;
	class ToolsAVP_Entity_InteractObject_OffTargetVolumeEdit_Controller*	m_pxOffTargetVolumeController;

	static ToolsAVP_Entity_InteractObject*									s_pxThis;

	static void	EditOnTurnOnMessages();
	static void	EditOnTurnOffMessages();
	static void EditActorVolume();
	static void EditOnTargetVolume();
	static void EditOffTargetVolume();
	friend class ToolsAVP_Entity_InteractObject_ActorVolumeEdit_Controller;
	friend class ToolsAVP_Entity_InteractObject_OnTargetVolumeEdit_Controller;
	friend class ToolsAVP_Entity_InteractObject_OffTargetVolumeEdit_Controller;

private :

	typedef ToolsAVP_Entity_PhysicalObject PARENT;
	static Asura_Hashed_List<ToolsAVP_Entity_InteractObject>				s_xList;

};

//For dealing with editing the volumes attached to the trigger
class ToolsAVP_Entity_InteractObject_ActorVolumeEdit_Controller : public Tools_VolumeEdit_Controller
{
public :
	ToolsAVP_Entity_InteractObject_ActorVolumeEdit_Controller( ToolsAVP_Entity_InteractObject* pxToolsEntity );
	virtual void StartVolumeEditing();
	virtual void EndVolumeEditing();
	Asura_Vector_3 CalculateCentre() const { return m_xOV.CalculateCentre(); }
	void RotateVolume( const Asura_Quat& xRot ) { m_xOV.m_xOrient = xRot * m_xOV.m_xOrient; }

protected :
	virtual bool CanCreateVolume() { return false; }
	virtual bool CanDeleteVolume() { return false; }
	virtual int GetNumberOfVolumes() { return 1; }
	virtual Asura_Bounding_Box* GetVolumeBox( int iIndex ) { return &m_xOV.m_xBox; }
	virtual Asura_Quat* GetVolumeOrient( int iIndex ) { return &m_xOV.m_xOrient; }
	virtual Asura_Vector_3 GetDefaultPosition()	{ return m_pxToolsEntity->Position; }
	virtual const char* GetName() { return m_pxToolsEntity->GetName(); }
	virtual void CreateVolumeMain() { Asura_Assert( false, "Shouldn't be here" ); }
	virtual void DeleteVolumeMain( int iIndex ) { Asura_Assert( false, "Shouldn't be here" ); }

private :
	typedef Tools_VolumeEdit_Controller PARENT;
	ToolsAVP_Entity_InteractObject*	m_pxToolsEntity;
	Asura_OrientedVolume m_xOV;
};


//For dealing with editing the volumes attached to the trigger
class ToolsAVP_Entity_InteractObject_OnTargetVolumeEdit_Controller : public Tools_VolumeEdit_Controller
{
public :
	ToolsAVP_Entity_InteractObject_OnTargetVolumeEdit_Controller ( ToolsAVP_Entity_InteractObject* pxToolsEntity );
	virtual void StartVolumeEditing();
	virtual void EndVolumeEditing();
	Asura_Vector_3 CalculateCentre() const { return m_xOV.CalculateCentre(); }
	void RotateVolume( const Asura_Quat& xRot ) { m_xOV.m_xOrient = xRot * m_xOV.m_xOrient; }

protected :
	virtual bool CanCreateVolume() { return false; }
	virtual bool CanDeleteVolume() { return false; }
	virtual int GetNumberOfVolumes() { return 1; }
	virtual Asura_Bounding_Box* GetVolumeBox( int iIndex ) { return &m_xOV.m_xBox; }
	virtual Asura_Quat* GetVolumeOrient( int iIndex ) { return &m_xOV.m_xOrient; }
	virtual Asura_Vector_3 GetDefaultPosition()	{ return m_pxToolsEntity->Position; }
	virtual const char* GetName() { return m_pxToolsEntity->GetName(); }
	virtual void CreateVolumeMain() { Asura_Assert( false, "Shouldn't be here" ); }
	virtual void DeleteVolumeMain( int iIndex ) { Asura_Assert( false, "Shouldn't be here" ); }
private :
	typedef Tools_VolumeEdit_Controller PARENT;
	ToolsAVP_Entity_InteractObject*	m_pxToolsEntity;
	Asura_OrientedVolume m_xOV;
};

class ToolsAVP_Entity_InteractObject_OffTargetVolumeEdit_Controller : public Tools_VolumeEdit_Controller
{
public :
	ToolsAVP_Entity_InteractObject_OffTargetVolumeEdit_Controller ( ToolsAVP_Entity_InteractObject* pxToolsEntity );
	virtual void StartVolumeEditing();
	virtual void EndVolumeEditing();
	Asura_Vector_3 CalculateCentre() const { return m_xOV.CalculateCentre(); }
	void RotateVolume( const Asura_Quat& xRot ) { m_xOV.m_xOrient = xRot * m_xOV.m_xOrient; }

protected :
	virtual bool CanCreateVolume() { return false; }
	virtual bool CanDeleteVolume() { return false; }
	virtual int GetNumberOfVolumes() { return 1; }
	virtual Asura_Bounding_Box* GetVolumeBox( int iIndex ) { return &m_xOV.m_xBox; }
	virtual Asura_Quat* GetVolumeOrient( int iIndex ) { return &m_xOV.m_xOrient; }
	virtual Asura_Vector_3 GetDefaultPosition()	{ return m_pxToolsEntity->Position; }
	virtual const char* GetName() { return m_pxToolsEntity->GetName(); }
	virtual void CreateVolumeMain() { Asura_Assert( false, "Shouldn't be here" ); }
	virtual void DeleteVolumeMain( int iIndex ) { Asura_Assert( false, "Shouldn't be here" ); }
private :
	typedef Tools_VolumeEdit_Controller PARENT;
	ToolsAVP_Entity_InteractObject*	m_pxToolsEntity;
	Asura_OrientedVolume m_xOV;
};
#endif // #ifndef TOOLSAVP_ENTITY_INTERACTOBJECT_H
