//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_Lift.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Project override of asura lift class
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/PhysicalObject/Lift/ToolsAVP_Entity_Lift.h"

// Project independent.
#include "Entity/PhysicalObject/Lift/AVP_ServerEntity_Lift.h"

// G L O B A L S /////////////////////////////////////////////////////////////

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// ToolsAVP_Entity_Lift
//

TOOLS_DECLARE_SHARED_PARAM( bool,			0x690e8c33, Lift_IgnoreForSurfaceReorientation )	// Asura_GetHashID("Lift_IgnoreForSurfaceReorientation")

Asura_Hashed_List<ToolsAVP_Entity_Lift> ToolsAVP_Entity_Lift::s_xList( 5 );


void ToolsAVP_Entity_Lift::Create()
{
    CreationHelper( PARENT_ASURA::s_szLongName,
                    PARENT_ASURA::s_szShortName, 
                    AVP_ServerEntity_Lift::CreateFromTools, 
                    Create );
}


Tools_Object* ToolsAVP_Entity_Lift::Create( Asura_Entity* pxEntity )
{
    return new ToolsAVP_Entity_Lift( AVP_ServerEntity_Lift::FindEntity(pxEntity->GetGuid()) );
}


ToolsAVP_Entity_Lift::ToolsAVP_Entity_Lift( AVP_ServerEntity_Lift* pxAVPLiftObj )
    : PARENT_ASURA( pxAVPLiftObj )
	, PARENT(pxAVPLiftObj)
	, Tools_Entity_PhysicalObject( pxAVPLiftObj )
{
    s_xList.AddToList( this );

    m_pxAVPLiftObj = pxAVPLiftObj;
}


ToolsAVP_Entity_Lift::~ToolsAVP_Entity_Lift( void )
{
    s_xList.RemoveFromList( this );
}


void ToolsAVP_Entity_Lift::Edit( void ) 
{ 
    PARENT_ASURA::Edit(); 
}

void ToolsAVP_Entity_Lift::AddOverlayControls(Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset)
{
	PARENT_ASURA::AddOverlayControls(pxNewOverlay, xOffset);

	Asura_SharedTickbox_Dialog* pxTickDialog = new Asura_SharedTickbox_Dialog("Ignore For Surface Reorientation", xOffset, &g_xParam_Lift_IgnoreForSurfaceReorientation );
	pxTickDialog->SetHelpText( "This object should be ignored for reorientation (alien movement)" );
	pxNewOverlay->AddDialog( pxTickDialog );

	// EP-19.05.2009. I added this function here, to avoid more duplication of code. This class has 2 parents, however 
	// we can't call AddOverlayControls on both of them. Because we end up with duplicated controls. Therefore we are calling
	// only one. But functionality from second one is missing. So instead of duplicating it here, I simply separated function
	// that is needed for this lift too.
	xOffset.y += Asura_Dialog::DefaultSpacing();
	PARENT::AddOverlayControls_CollisionFXMaterialBP( pxNewOverlay, xOffset, false );
}

bool ToolsAVP_Entity_Lift::GetParameter(Asura_Hash_ID uParamID, Tools_Parameter* pxParam) const
{
	switch(uParamID)
	{
		case g_uHash_Lift_IgnoreForSurfaceReorientation:
		{
			pxParam->SetBool( m_pxPhysicalObject->IsIgnoredForSurfaceReorientation() );
		}
		break;

		default:
			return( PARENT_ASURA::GetParameter(uParamID, pxParam) || PARENT::GetParameter(uParamID, pxParam) );
	}

	return true;
}

bool ToolsAVP_Entity_Lift::SetParameter(Asura_Hash_ID uParamID, const Tools_Parameter* pxParam)
{
	switch(uParamID)
	{
		case g_uHash_Lift_IgnoreForSurfaceReorientation:
		{
			m_pxPhysicalObject->SetIgnoreForSurfaceReorientation( pxParam->GetBool() );
			SetEntityModified();
			return true;
		}
		break;

		default:
		{
			bool bAsuraOK	= PARENT_ASURA::SetParameter(uParamID, pxParam);
			bool bParentOK	= PARENT::SetParameter(uParamID, pxParam);
			return( bAsuraOK || bParentOK );
		}
	}
}


const char* ToolsAVP_Entity_Lift::GetTypeDescription( void ) 
{ 
    return PARENT_ASURA::GetTypeDescription(); 
}


bool ToolsAVP_Entity_Lift::HandlesThisMessage( int iMessageID )
{
    return( PARENT_ASURA::HandlesThisMessage(iMessageID) || PARENT::HandlesThisMessage(iMessageID) );
}


void ToolsAVP_Entity_Lift::PrintInfo( float& fX, float& fY )
{
    PARENT_ASURA::PrintInfo( fX, fY );
    PARENT::PrintInfo( fX, fY );
}

void ToolsAVP_Entity_Lift::UpdateWhileSelected( void )
{
    PARENT::UpdateWhileSelected();
    PARENT_ASURA::UpdateWhileSelected();
}

void ToolsAVP_Entity_Lift::AddFilesToExportList(List<Asura_String>& xNameList) const
{
	PARENT::AddFilesToExportList(xNameList);
	PARENT_ASURA::AddFilesToExportList(xNameList);
}


/// End of file. ///
