//////////////////////////////////////////////////////////////////////////////
//
// FILE : ToolsAVP_Entity_DamageableObject.cpp
//
// AUTHORS : Paul Humphries
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/PhysicalObject/PhysicsObject/DamageableObject/ToolsAVP_Entity_DamageableObject.h"

// asura
#include "Core/Asura_TextureManagement.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Shapes/Tools_Shapes.h"

// avp
#include "Entity/PhysicalObject/PhysicsObject/DamageableObject/AVP_ServerEntity_DamageableObject.h"

// tools
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "Entity/Tools_Entity_PhysicsObject.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_ObjectSystem_Param.h"

// avp tools
#include "Objects/ToolsAVP_Object_Type.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_DamageableObject
//
//////////////////////////////////////////////////////////////////////////////

TOOLS_DECLARE_SHARED_PARAM( float, uTOOLS_SHAREDPARAMHASH_DAMAGEABLEOBJECT_THRESHOLDSKIN, DamageableObject_ThresholdSkin )
TOOLS_DECLARE_SHARED_PARAM( float, uTOOLS_SHAREDPARAMHASH_DAMAGEABLEOBJECT_THRESHOLDDYNAMIC, DamageableObject_ThresholdDynamic )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_DAMAGEABLEOBJECT_BEGINSSTATIC, DamageableObject_BeginsStatic )

ToolsAVP_Entity_DamageableObject* ToolsAVP_Entity_DamageableObject::s_pxCurrentlySelectedObject = NULL;

ToolsAVP_Entity_DamageableObject::ToolsAVP_Entity_DamageableObject(AVP_ServerEntity_DamageableObject* pxDamageableObject)
: PARENT(pxDamageableObject)
, Tools_Entity_PhysicalObject(pxDamageableObject)
, m_pxDamageableObject(pxDamageableObject)
, m_bBeginsStatic(pxDamageableObject->GetBeginsStatic())
, m_fDamageThresholdSkin(pxDamageableObject->GetDamageThresholdSkin())
, m_fDamageThresholdDynamic(pxDamageableObject->GetDamageThresholdDynamic())
, m_bChanged(false)
{
	Type = TOT_Entity_DamageableObject;
	m_pxPhysicalObject->SetSkinAndAnimID(pxDamageableObject->GetSkinID(), pxDamageableObject->GetAnimID());

	for (u_int i = 0; i < 6; i++)
	{
		m_auDamagedSkinFileHashID[i] = pxDamageableObject->GetDamagedSkinFile(i);
		m_auDamagedSkinHashID[i] = pxDamageableObject->GetDamagedSkin(i);
		m_auDamagedAnimFileHashID[i] = pxDamageableObject->GetDamagedAnimFile(i);
		m_auDamagedAnimHashID[i] = pxDamageableObject->GetDamagedAnim(i);
	}
}

Tools_Object* ToolsAVP_Entity_DamageableObject::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Entity_DamageableObject( AVP_ServerEntity_DamageableObject::FindEntity(pxEntity->GetGuid()) );

}

void ToolsAVP_Entity_DamageableObject::Create()
{
	CreationHelper( "Damageable Object",
		"Damageable Object", 
		AVP_ServerEntity_DamageableObject::CreateFromTools,
		Create );
}

void ToolsAVP_Entity_DamageableObject::AddFilesToExportList(List<Asura_String>& xNameList) const
{
	PARENT::AddFilesToExportList(xNameList);

	for (int i = 0; i < 6; i++)
	{
		if (m_auDamagedSkinFileHashID[i] != ASURA_HASH_ID_UNSET)
		{
			const char* szFileName = Asura_HashedFilename_System::GetFilename(m_auDamagedSkinFileHashID[i], false);
			if (szFileName)
			{
				Tools_Level_Export::AddFileToExportList(xNameList, Asura_String(szFileName), true);
			}
			else
			{
				Asura_Assertf(false, ("Failed to find damaged skin with hash ID 0x%X, used by damageable object %u ('%s').",
							  m_auDamagedSkinFileHashID[i], m_uGuid, Entity->ToolsData->Name.GetString()));
			}
		}
		if (m_auDamagedAnimFileHashID[i] != ASURA_HASH_ID_UNSET)
		{
			const char* szFileName = Asura_HashedFilename_System::GetFilename(m_auDamagedAnimFileHashID[i], false);
			if (szFileName)
			{
				Tools_Level_Export::AddFileToExportList(xNameList, Asura_String(szFileName), false);
			}
			else
			{
				Asura_Assertf(false, ("Failed to find damaged anim with hash ID 0x%X, used by damageable object %u ('%s').",
							  m_auDamagedAnimFileHashID[i], m_uGuid, Entity->ToolsData->Name.GetString()));
			}
		}
	}
}

void ToolsAVP_Entity_DamageableObject::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );

	// blueprints - code below ifdef'd out; values now specified in blueprints
#if 0
	Asura_Dialog* pxDialog;

	m_pxTabsDialog->SetSelectedTabPointer(NULL);
	m_pxTabsDialog->AddTab("Damaged Skins");
	pxOverlay->SetCurrentParentDialog(m_pxTabsDialog);
	xOffset = m_xTabsOffset;

	pxDialog = new Asura_StringDisplay_Dialog("Select damaged skins & anims to use:", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	char *s[] = {"+X", "+Y", "+Z", "-X", "-Y", "-Z"};

	for (int iIndex = 0; iIndex < 6; iIndex++)
	{
		pxDialog = new Asura_Function_Dialog(s[iIndex], xOffset, SelectDamagedSkinAndAnim_Dialog, iIndex, &Changed);
		pxOverlay->AddDialog(pxDialog);
		if (iIndex == 2)
		{
			xOffset.x += Asura_Dialog::LargeSpacing() * 3.0f;
			pxDialog = new Asura_Function_Dialog("+(ALL)", xOffset, SelectDamagedSkinAndAnimGroup_Dialog, 0, &Changed);
			pxOverlay->AddDialog(pxDialog);
			xOffset.x -= Asura_Dialog::LargeSpacing() * 9.0f;
			xOffset.y += Asura_Dialog::DefaultSpacing();
		}
		else if (iIndex == 5)
		{
			xOffset.x += Asura_Dialog::LargeSpacing() * 3.0f;
			pxDialog = new Asura_Function_Dialog("-(ALL)", xOffset, SelectDamagedSkinAndAnimGroup_Dialog, 3, &Changed);
			pxOverlay->AddDialog(pxDialog);
			xOffset.x -= Asura_Dialog::LargeSpacing() * 9.0f;
			xOffset.y += Asura_Dialog::LargeSpacing();
		}
		else
		{
			xOffset.x += Asura_Dialog::LargeSpacing() * 3.0f;
		}
	}

	pxDialog = new Asura_Numerical_Dialog("Damage Threshold (Skin)", xOffset, &g_xParam_DamageableObject_ThresholdSkin, 0.0f, 1000.00f, 5.0f, " %.2f ");
	pxDialog->SetHelpText("The minimum amount of damage required for the object to change to one of the damaged skins.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Numerical_Dialog("Damage Threshold (Dynamic)", xOffset, &g_xParam_DamageableObject_ThresholdDynamic, 0.0f, 1000.00f, 5.0f, " %.2f ");
	pxDialog->SetHelpText("The minimum amount of damage required for the object to become dynamic.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Begins Static", xOffset, &g_xParam_DamageableObject_BeginsStatic);
	pxDialog->SetHelpText("Whether this damageable object begins as a static object");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += Asura_Dialog::DefaultSpacing();
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxOverlay->SetCurrentParentDialog( NULL );
	m_pxTabsDialog->SelectTab(0);
#endif
}

void ToolsAVP_Entity_DamageableObject::SelectDamagedSkinAndAnim_Dialog(int iIndex)
{
	Asura_Assert(s_pxCurrentlySelectedObject, "ToolsAVP_Entity_DamageableObject::SelectSkinAndAnim_Dialog() called with no object selected");
	if (s_pxCurrentlySelectedObject)
	{
		s_pxCurrentlySelectedObject->SelectDamagedSkinAndAnim(iIndex);
	}
}

void ToolsAVP_Entity_DamageableObject::SelectDamagedSkinAndAnimGroup_Dialog(int iIndex)
{
	Asura_Assert(s_pxCurrentlySelectedObject, "ToolsAVP_Entity_DamageableObject::SelectSkinAndAnimGroup_Dialog() called with no object selected");
	if (s_pxCurrentlySelectedObject)
	{
		s_pxCurrentlySelectedObject->SelectDamagedSkinAndAnimGroup(iIndex);
	}
}

void ToolsAVP_Entity_DamageableObject::OnSelection()
{
	PARENT::OnSelection();
	s_pxCurrentlySelectedObject = this;
}

void ToolsAVP_Entity_DamageableObject::OnDeselection()
{
	PARENT::OnDeselection();
	s_pxCurrentlySelectedObject = NULL;
}

void ToolsAVP_Entity_DamageableObject::SelectDamagedSkinAndAnimGroup(int iIndex)
{
	Asura_Assert(IsSelected(), "SelectDamagedSkinAndAnimGroup() called on non-selected object");
	if (!IsSelected()) return;

	Asura_Assert(m_pxPhysicalObject, "Tools_Entity_PhysicalObject has no m_pxPhysicalObject");

	Asura_String xSkinName;
	Asura_String xSkinFilename;
	Asura_String xAnimName;
	Asura_String xAnimFilename;

	if (Tools_Animation_Files::ChooseSkin(&xSkinName, &xSkinFilename))
	{
		for (int i = iIndex; i < iIndex + 3; i++)
		{
			m_auDamagedSkinFileHashID[i]   = Asura_GetHashID(xSkinFilename.GetString());
			m_auDamagedSkinHashID[i]       = Asura_GetHashID(xSkinName.GetString());
		}
	}
	else
	{
		return;
	}

	if (Tools_Animation_Files::ChooseAnimation(xSkinName.GetString(), NULL, &xAnimName, &xAnimFilename,0,true))
	{
		for (int i = iIndex; i < iIndex + 3; i++)
		{
			m_auDamagedAnimFileHashID[i]   = Asura_GetHashID(xAnimFilename.GetString());
			m_auDamagedAnimHashID[i]       = Asura_GetHashID(xAnimName.GetString());
		}
	}
	else
	{
		for (int i = iIndex; i < iIndex + 3; i++)
		{
			m_auDamagedAnimFileHashID[iIndex]	= ASURA_HASH_ID_UNSET;
			m_auDamagedAnimHashID[iIndex]		= ASURA_HASH_ID_UNSET;
		}
	}

	//UnmergePolygonsFromEnvironment();
	//Asura_Dynamics_System::RegenerateStaticObjectAABBTree();

	if (m_pxDamageableObject)
	{
		for (int i = iIndex; i < iIndex + 3; i++)
		{
			m_pxDamageableObject->SetDamagedSkinFile(i, m_auDamagedSkinFileHashID[i]);
			m_pxDamageableObject->SetDamagedSkin(i, m_auDamagedSkinHashID[i]);
			m_pxDamageableObject->SetDamagedAnimFile(i, m_auDamagedAnimFileHashID[i]);
			m_pxDamageableObject->SetDamagedAnim(i, m_auDamagedAnimHashID[i]);
		}
	}
}

void ToolsAVP_Entity_DamageableObject::SelectDamagedSkinAndAnim(int iIndex)
{
	Asura_Assert(IsSelected(), "SelectDamagedSkinAndAnim() called on non-selected object");
	if (!IsSelected()) return;

	Asura_Assert(m_pxPhysicalObject, "Tools_Entity_PhysicalObject has no m_pxPhysicalObject");

	Asura_String xSkinName;
	Asura_String xSkinFilename;
	Asura_String xAnimName;
	Asura_String xAnimFilename;

	if (Tools_Animation_Files::ChooseSkin(&xSkinName, &xSkinFilename))
	{
		m_auDamagedSkinFileHashID[iIndex]   = Asura_GetHashID(xSkinFilename.GetString());
		m_auDamagedSkinHashID[iIndex]       = Asura_GetHashID(xSkinName.GetString());
	}
	else
	{
		return;
	}

	if (Tools_Animation_Files::ChooseAnimation(xSkinName.GetString(), NULL, &xAnimName, &xAnimFilename,0,true))
	{
		m_auDamagedAnimFileHashID[iIndex]   = Asura_GetHashID(xAnimFilename.GetString());
		m_auDamagedAnimHashID[iIndex]       = Asura_GetHashID(xAnimName.GetString());
	}
	else
	{
		m_auDamagedAnimFileHashID[iIndex]	= ASURA_HASH_ID_UNSET;
		m_auDamagedAnimHashID[iIndex]		= ASURA_HASH_ID_UNSET;
	}

	//UnmergePolygonsFromEnvironment();

	//Asura_Dynamics_System::RegenerateStaticObjectAABBTree();

	if (m_pxDamageableObject)
	{
		m_pxDamageableObject->SetDamagedSkinFile(iIndex, m_auDamagedSkinFileHashID[iIndex]);
		m_pxDamageableObject->SetDamagedSkin(iIndex, m_auDamagedSkinHashID[iIndex]);
		m_pxDamageableObject->SetDamagedAnimFile(iIndex, m_auDamagedAnimFileHashID[iIndex]);
		m_pxDamageableObject->SetDamagedAnim(iIndex, m_auDamagedAnimHashID[iIndex]);
	}
}

void ToolsAVP_Entity_DamageableObject::UpdateWhileSelected()
{
	// PH - update values before updating the parent, otherwise changed is already set back to false
	if (Changed)
	{
		for (int i = 0; i < 6; i++)
		{
			m_pxDamageableObject->SetDamagedSkin(i, m_auDamagedSkinHashID[i]);
			m_pxDamageableObject->SetDamagedAnim(i, m_auDamagedAnimHashID[i]);
			m_pxDamageableObject->SetDamagedSkinFile(i, m_auDamagedSkinFileHashID[i]);
			m_pxDamageableObject->SetDamagedAnimFile(i, m_auDamagedAnimFileHashID[i]);
		}
		m_pxDamageableObject->SetDamageThresholdSkin(m_fDamageThresholdSkin);
		m_pxDamageableObject->SetDamageThresholdDynamic(m_fDamageThresholdDynamic);
	}
	PARENT::UpdateWhileSelected();
}

Tools_Object* ToolsAVP_Entity_DamageableObject::CreateObject()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data *pxToolsData = CreateToolsDataWithDefaultName("Damageable Object", "Damageable Object", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_DamageableObject::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_DamageableObject *pxServerObject = new AVP_ServerEntity_DamageableObject(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_DamageableObject::Create() - no server object" );
		}
		else
		{
			ToolsAVP_Entity_DamageableObject *pxToolsObject = new ToolsAVP_Entity_DamageableObject(pxServerObject);

			pxServerObject->ToolsData = pxToolsData;

			// Create the game specific tools object and select it
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_DamageableObject::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->Orientation.Identity();		//	= *Asura_Camera::GetOrientation();

				//Tools_Timed_Message_System::AddMessage(0xffffff, "Created new DamageableObject");
				Tools_SaveAsura::SetEntityModified();
			}
			return pxToolsObject;
		}
	}
	return NULL;
}

bool ToolsAVP_Entity_DamageableObject::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{
	switch(uParamID)
	{
	case g_uHash_DamageableObject_ThresholdSkin:
		pxParam->SetFloat(m_fDamageThresholdSkin);
		break;

	case g_uHash_DamageableObject_ThresholdDynamic:
		pxParam->SetFloat(m_fDamageThresholdDynamic);
		break;

	case g_uHash_DamageableObject_BeginsStatic:
		pxParam->SetBool(m_bBeginsStatic);
		break;

	default:
		return PARENT::GetParameter(uParamID, pxParam);
	}
	return true;
}

bool ToolsAVP_Entity_DamageableObject::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	switch(uParamID)
	{
	case g_uHash_DamageableObject_ThresholdSkin:
		m_fDamageThresholdSkin = pxParam->GetFloat();
		m_pxDamageableObject->SetDamageThresholdSkin(m_fDamageThresholdSkin);
		break;

	case g_uHash_DamageableObject_ThresholdDynamic:
		m_fDamageThresholdDynamic = pxParam->GetFloat();
		m_pxDamageableObject->SetDamageThresholdDynamic(m_fDamageThresholdDynamic);
		break;

	case g_uHash_DamageableObject_BeginsStatic:
		m_bBeginsStatic = pxParam->GetBool();
		m_pxDamageableObject->SetBeginsStatic(m_bBeginsStatic);
		break;

	default:
		return PARENT::SetParameter(uParamID, pxParam);
	}

	SetEntityModified();

	return true;
}
