/////////////////////////////////////////////////////////////////////////////////////////////////////////
///
/// Module:					ToolsAVP_Entity_Pickup.cpp
///
/// Created:				AJK 26/08/2008
///
/// Description:			Tools object for Pickups
///
/////////////////////////////////////////////////////////////////////////////////////////////////////////


#include "Core/Asura.h"

// Header
#include "Entity/PhysicalObject/Pickup/ToolsAVP_Entity_Pickup.h"

// Asura
#include "Entity/Asura_StaticMessage_System.h"

// AVP
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup.h"
#include "Entity/PhysicalObject/Pickup/AVP_Pickup_Tool.h"

// Tools
#include "Blueprint/Tools_Mode_Blueprint.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_SaveAsura.h"

// Tools AVP
#include "Entity/PhysicalObject/Pickup/AVP_ServerEntity_Pickup.h"
#include "Entity/Tools_MessageEditor.h"
#include "Load&Save/ToolsAVP_Level_Export.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"

// Statics def
ToolsAVP_Entity_Pickup* ToolsAVP_Entity_Pickup::s_pxCurrentPickupToolObject = NULL;

ToolsAVP_Entity_Pickup::ToolsAVP_Entity_Pickup(AVP_ServerEntity_Pickup* pxEntity)
: Tools_Entity_PhysicalObject( pxEntity )
, PARENT( pxEntity )
{
	Type = TOT_Entity_Pickup;
	SetToolsPickupSkin();
}

ToolsAVP_Entity_Pickup::~ToolsAVP_Entity_Pickup()
{
}

void ToolsAVP_Entity_Pickup::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{

	// get selected pickup current selection of dropdown list according to our type
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(ThisEntity->GetGuid());

	if(pxPickup)
	{
		Tools_Entity_Object::AddOverlayControls(pxOverlay, xOffset);
		// Create drop down dialog for pickup types
		Asura_Dialog* pxDialog = new ToolsAVP_Entity_Pickup_DropDownList_Dialog(xOffset, NUM_PICKUPS_DISPLAYED, &pxPickup->m_uPickupBlueprintHash, &Changed, "Pickup type");
		pxDialog->SetHelpText("Choose pickup type");
		static_cast<ToolsAVP_Entity_Pickup_DropDownList_Dialog*> (pxDialog)->SetOnSelectCallback(OnSelectPickupType);
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += ToolsAVP_Entity_Pickup_DropDownList_Dialog::DefaultSpacing();

		// Check out if it is an audio dairy;
		if ( pxPickup->GetPickupObject() && pxPickup->GetPickupObject()->GetPickupType() == AVP_Pickup::PICKUP_TYPE_AUDIODIARY )
		{
			char szName[100];
			GetDialogueBlueprintName(szName, 100);
			pxDialog = new Asura_Function_Dialog( szName, xOffset, ChooseBlueprint, &Changed );
			pxOverlay->AddDialog( pxDialog );
			pxDialog->m_iControlID = Asura_GetHashID("ChooseBlueprint");
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Function_Dialog( "Edit Current Blueprint", xOffset, EditBlueprint, &Changed );
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::DefaultSpacing();
		}

		Asura_Char szBuffer[32];

		Asura_PrintfToBfr( szBuffer,"On picked up(%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxPickup->m_usPickedUpMessageBlock ) );
		pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnPickedUpMessages );
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		Asura_PrintfToBfr( szBuffer,"On dropped(%d)", 32, true, Asura_StaticMessage_System::GetNumberOfMessagesInBlock( pxPickup->m_usDroppedMessageBlock ) );
		pxDialog = new Asura_Function_Dialog( szBuffer, xOffset, EditOnDroppedMessages );
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Create tickbox for Can Respwan?
		pxDialog = new Asura_FlagTickbox_Dialog("Can Respawn?", xOffset, &pxPickup->m_uFlags, AVP_ServerEntity_Pickup::FLAG_CAN_RESPAWN, &Changed);
		pxDialog->SetHelpText("Set if the pickup can respawn (mainly for Multiplayer)");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Create can respawn time dialog
		pxDialog = new Asura_Numerical_Dialog("Respawn time",xOffset, &pxPickup->m_fRespawnTime, 0.0f, 360.0f, 0.1f, "%.2f", &Changed);
		pxDialog->SetHelpText("Time to respwan.");
		pxDialog->SetIsEnabledFunction(CanRespawn);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Create tickbox for fixed quantity?
		pxDialog = new Asura_FlagTickbox_Dialog("Fixed Quantity?", xOffset, &pxPickup->m_uFlags, AVP_ServerEntity_Pickup::FLAG_IS_FIXED_QUANTITY, &Changed);
		pxDialog->SetHelpText("Set if the pickup quantity (health and ammo) are fixed or intelligently based");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Old hard-coded values
		int iMinQuantity = 1;
		int iMaxQuantity = 999;
		int iMinSecondaryQuantity = 1;
		int iMaxSecondaryQuantity = 999;

		if(IsWeapon())
		{
			iMinQuantity			= 0;
			iMinSecondaryQuantity	= 0;

			iMaxQuantity			= Asura_Maths::FloatToInt(GetMaxQuantityFromBlueprints(0));
			iMaxSecondaryQuantity	= Asura_Maths::FloatToInt(GetMaxQuantityFromBlueprints(1));
		}
	
		// Create numerical dialog for this quantity
		pxDialog = new Asura_Numerical_Dialog("Quantity (For weapons, 1st subtype)", xOffset, &pxPickup->m_iQuantity, iMinQuantity, iMaxQuantity, 1, "%d", &Changed);
		pxDialog->SetHelpText("Set the quantity of Ammo or Health supplied by this pickup");
		pxDialog->SetIsEnabledFunction(IsFixedAmount);
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Create numerical dialog for this quantity
		pxDialog = new Asura_Numerical_Dialog("Quantity (For weapons, 2st subtype)", xOffset, &pxPickup->m_iSecondaryQuantity, iMinSecondaryQuantity, iMaxSecondaryQuantity, 1, "%d", &Changed);
		pxDialog->SetHelpText("Set the quantity of the secondary subtype in weapon pickups");
		pxDialog->SetIsEnabledFunction(IsWeaponWithSecondaryFire);
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void ToolsAVP_Entity_Pickup::ChooseBlueprint()
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(s_pxCurrentPickupToolObject->GetGuid());
	if ( pxPickup )
	{	
		Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_DIALOGUE;
		Asura_Hash_ID uBlueprintHashID     = pxPickup->GetDialogueBlueprintHashID();

		BlueprintSelect_Overlay xBPOverlay( "Choose A Dialogue Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );
		if ( !xBPOverlay.DoModal() ) { return; }

		pxPickup->SetDialogueBlueprintHashID( uBlueprintHashID );

		s_pxCurrentPickupToolObject->SetEntityModified();
		s_pxCurrentPickupToolObject->RefreshOverlay();
	}
}

void ToolsAVP_Entity_Pickup::GetDialogueBlueprintName(char* pcName, u_int uSize) const
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(ThisEntity->GetGuid());
	strcpy(pcName,"Dialogue Blueprint: ");
	if ( pxPickup && pxPickup->GetDialogueBlueprintHashID() != ASURA_HASH_ID_UNSET)
	{
		
		Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_DIALOGUE,  pxPickup->GetDialogueBlueprintHashID());
		if ( pxBlueprint )
		{
			Asura_PrintfToBfr(pcName,"Dialogue Blueprint: %s",uSize,true,pxBlueprint->GetName());	
		}
	}
}

void ToolsAVP_Entity_Pickup::EditBlueprint()
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(s_pxCurrentPickupToolObject->GetGuid());
	if ( pxPickup )
	{
		const Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_DIALOGUE;
		const Asura_Hash_ID uBlueprintHashID     = pxPickup->GetDialogueBlueprintHashID();
		if ( uBlueprintHashID == ASURA_HASH_ID_UNSET )
		{
			Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_WARNING, "Blueprint details have not been set." );
			return;
		}

		if ( !Tools_Mode_Blueprint::CreateEditOverlay( uBlueprintTypeHashID, uBlueprintHashID ) )
		{
			Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Could not open the blueprint edit overlay." );
		}
	}
}

void ToolsAVP_Entity_Pickup::EditOnPickedUpMessages()
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(s_pxCurrentPickupToolObject->GetGuid());
	if ( pxPickup )
	{
		#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
		Tools_MessageEditor::StartMessageEditingMode(&pxPickup->m_usPickedUpMessageBlock);
		#else
		Tools_Object_System::EditMessageBlock( "On Picked up", &pxPickup->m_usPickedUpMessageBlock );
		#endif
	}
}

void ToolsAVP_Entity_Pickup::EditOnDroppedMessages()
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(s_pxCurrentPickupToolObject->GetGuid());
	if ( pxPickup )
	{
		#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
		Tools_MessageEditor::StartMessageEditingMode(&pxPickup->m_usDroppedMessageBlock);
		#else
		Tools_Object_System::EditMessageBlock( "On Dropped", &pxPickup->m_usDroppedMessageBlock );
		#endif
	}
}

bool ToolsAVP_Entity_Pickup::OwnsThisMessageBlock( unsigned short usMessageBlockID )
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(GetGuid());	
	if ( pxPickup && ( pxPickup->m_usPickedUpMessageBlock == usMessageBlockID || pxPickup->m_usDroppedMessageBlock == usMessageBlockID ) )
	{
		return true;
	}

	return PARENT::OwnsThisMessageBlock( usMessageBlockID );
}

bool ToolsAVP_Entity_Pickup::GetEventDescForMessageBlock( unsigned short usMessageBlockID, Asura_String& xOutputDesc )
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(GetGuid());

	if ( pxPickup ) 
	{
		if ( pxPickup->m_usPickedUpMessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Picked up";
			return true;
		}
		else if ( pxPickup->m_usDroppedMessageBlock == usMessageBlockID )
		{
			xOutputDesc = "On Dropped";
			return true;
		}
		return PARENT::GetEventDescForMessageBlock(usMessageBlockID, xOutputDesc);
	}
	else
	{
		Asura_Assert(false, "No pickup object!");
		return false;
	}
}

void ToolsAVP_Entity_Pickup::Destroy()
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(GetGuid());

	//delete all the static messages
	if ( pxPickup )
	{
		Asura_StaticMessage_System::DeleteMessageBlock(pxPickup->m_usPickedUpMessageBlock);
		pxPickup->m_usPickedUpMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;

		Asura_StaticMessage_System::DeleteMessageBlock(pxPickup->m_usDroppedMessageBlock);
		pxPickup->m_usDroppedMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
	}

	PARENT::Destroy();
}

float ToolsAVP_Entity_Pickup::GetMaxQuantityFromBlueprints(int iSubTypeIndex)
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(ThisEntity->GetGuid());

	if(pxPickup)
	{
		Asura_Hash_ID uPickupBlueprintHash = pxPickup->GetPickupBlueprintHash();
		Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_PICKUPS, uPickupBlueprintHash);
		if(pxBlueprint)
		{
			Asura_Value xValue;
			if(pxBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_PICKUP_SINGLEPLAYERINVENTORYITEMBLUEPRINT, xValue))
			{
				Asura_Hash_ID uInventoryItemHashID = xValue.GetHashIDValue();
				Asura_Blueprint* pxInventoryBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uInventoryItemHashID);
				if(pxInventoryBlueprint)
				{
					Asura_Hash_ID uSubtypeParamHash = (iSubTypeIndex>0)?uAVP_BLUEPRINT_PARAM_SUBTYPE_SECONDARY:uAVP_BLUEPRINT_PARAM_SUBTYPE_PRIMARY;
					if(pxInventoryBlueprint->GetParameterValueByHashID(uSubtypeParamHash, xValue))
					{
						Asura_Hash_ID uSubTypeBlueprintHashID = xValue.GetHashIDValue();
						Asura_Blueprint* pxSubTypeBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uSubTypeBlueprintHashID);
						if(pxSubTypeBlueprint)
						{
							if(pxSubTypeBlueprint->GetParameterValueByHashID(uAVP_BLUEPRINT_PARAM_MAXQUANTITY, xValue))
							{
								return xValue.GetFloatValue();
							}
						}
					}
				}
			}
		}
	}

	return(0);
}

bool ToolsAVP_Entity_Pickup::IsFixedAmount()
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(ThisEntity->GetGuid());

	return(pxPickup->IsFixedQuantity());
}

bool ToolsAVP_Entity_Pickup::IsWeapon()
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(ThisEntity->GetGuid());

	return(pxPickup->IsFixedQuantity() && pxPickup->GetPickupObject()->GetPickupType() == AVP_Pickup::PICKUP_TYPE_WEAPON);
}

bool ToolsAVP_Entity_Pickup::IsWeaponWithSecondaryFire()
{
	return(IsWeapon() && GetMaxQuantityFromBlueprints(1)!=0);
}

bool ToolsAVP_Entity_Pickup::CanRespawn()
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(ThisEntity->GetGuid());

	return(pxPickup->GetCanRespawn());
}

bool ToolsAVP_Entity_Pickup::IsIntelligentAmount()
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(ThisEntity->GetGuid());

	return(!pxPickup->IsFixedQuantity());
}

void ToolsAVP_Entity_Pickup::OnSelectPickupType()
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(ThisEntity->GetGuid());
	pxPickup->CreatePickupObject();
	// Update the tools object skin
    s_pxCurrentPickupToolObject->SetToolsPickupSkin();

	s_pxCurrentPickupToolObject->RefreshOverlay();
}

void ToolsAVP_Entity_Pickup::OnSelection()
{
	s_pxCurrentPickupToolObject = this;
	PARENT::OnSelection();
}


void ToolsAVP_Entity_Pickup::SetToolsPickupSkin()
{
	// get selected pickup skin
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(GetGuid());
	if(pxPickup)
	{
		if(pxPickup->GetPickupObject())
		{
			if(pxPickup->GetPickupObject()->GetSkinHash() != ASURA_HASH_ID_UNSET)
			{
				SetSkinAndAnim( pxPickup->GetPickupObject()->GetSkinHash(), ASURA_HASH_ID_UNSET);
				if(!pxPickup->GetPickupObject()->GetHasShadow())
				{
					SetPhysicalObjectFlags( GetPhysicalObjectFlags() | ASURA_PHYSICALOBJECT_FLAG_NOSHADOW );
				}
			}
			else
			{
				Asura_Assert(false, "Pickup skin is not set in blueprint!");
			}
		}
		//else
		//{
		//	// Either a new pikcup object or one that has blueprint problems
 	    //}
	}
	else
	{
		Asura_Assert(false, "Can't happen!");
	}
}

void ToolsAVP_Entity_Pickup::AddFilesToExportList(List<Asura_String>& xNameList) const
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::GetList()->FindObject(GetGuid());
	if(pxPickup)
	{
		if(pxPickup->GetPickupObject())
		{
			Tools_Level_Export::AddSkinToExportList(Asura_Hash_ID(pxPickup->GetPickupObject()->GetSkinHash()));

			if(pxPickup->GetPickupObject()->GetPickupType() == AVP_Pickup::PICKUP_TYPE_TOOL)
			{
				const AVP_Pickup_Tool* pxTool = static_cast<const AVP_Pickup_Tool*> (pxPickup->GetPickupObject());
				Tools_Level_Export::AddSkinToExportList(pxTool->GetDestroyedSkin());
			}
		}
		else
		{
			Asura_Assert(false, "Cannot add files to export list (pikcup object doesn't exist)");
		}
	}

	PARENT::AddFilesToExportList(xNameList);
}

void ToolsAVP_Entity_Pickup::Create()
{
	CreationHelper( "Pickup", "Pickup", AVP_ServerEntity_Pickup::CreateFromTools, ToolsAVP_Entity_Pickup::Create);
}

Tools_Object* ToolsAVP_Entity_Pickup::Create(Asura_Entity* pxEntity)
{
	AVP_ServerEntity_Pickup* pxPickup = AVP_ServerEntity_Pickup::FindEntity(pxEntity->GetGuid());
	return new ToolsAVP_Entity_Pickup( pxPickup );
}

void ToolsAVP_Entity_Pickup::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Object::Copy(pxObject);

	ToolsAVP_Entity_Pickup *pxFromObject  = static_cast<ToolsAVP_Entity_Pickup *>( ToolsAVP_Entity_Pickup::FindEntity(pxObject->GetGuid()) );

	AVP_ServerEntity_Pickup* pxEntity     = static_cast<AVP_ServerEntity_Pickup*>( AVP_ServerEntity_Pickup::FindEntity(GetGuid()) );
	AVP_ServerEntity_Pickup* pxFromEntity = static_cast<AVP_ServerEntity_Pickup*>( AVP_ServerEntity_Pickup::FindEntity(pxObject->GetGuid()) );

	if( pxEntity )
	{
		pxEntity->Copy(pxFromEntity);
		SetToolsPickupSkin();
	}

	if (pxFromObject)
	{	
		Orientation = *Asura_Camera::GetOrientation();	
		Position    = *Asura_Camera::GetPosition();	
	}
}

Tools_Object* ToolsAVP_Entity_Pickup::CreateObjectOfThisType() const
{
	return CreateMain();	
}

ToolsAVP_Entity_Pickup* ToolsAVP_Entity_Pickup::CreateMain()
{
	const Asura_Guid uGUID = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data*			pxToolsData = NULL;
	AVP_ServerEntity_Pickup*	pxEntity    = NULL;
	ToolsAVP_Entity_Pickup*		pxToolsObj  = NULL;

	pxToolsData = CreateToolsDataWithDefaultName( "Pickup", "Pickup", uGUID );
	if( !pxToolsData ) 
	{ 
		Asura_Assert(false, "Couldn't not create Tools_Entity_Data for COPY!");
		return NULL;
	}
	pxEntity = new AVP_ServerEntity_Pickup( uGUID);
	if( !pxEntity )
	{
		Asura_Assert(false, "Couldn't not create AVP_ServerEntity_Pickup for COPY!");
		return NULL;
	}
	pxEntity->ToolsData = pxToolsData;

	pxToolsObj = new ToolsAVP_Entity_Pickup( pxEntity );
	if( !pxToolsObj ) 
	{ 
		Asura_Assert(false, "Couldn't not create ToolsAVP_Entity_Pickup for COPY!");
		return NULL;
	}

	Tools_SaveAsura::SetEntityModified();
	return pxToolsObj;
}

bool ToolsAVP_Entity_Pickup::HandlesThisMessage(int iMessageID)
{
	switch(iMessageID)
	{
		case AVP_MSGID_ENABLE_PICKUP:
		case AVP_MSGID_DISABLE_PICKUP:
		{
			// This entity can receive change ownership messages
			return true;
		}
		default:
		{
			return PARENT::HandlesThisMessage(iMessageID); 
		}
	}
}

//
// CLASS ToolsAVP_Entity_Pickup_DropDownList_Dialog
//
ToolsAVP_Entity_Pickup_DropDownList_Dialog::ToolsAVP_Entity_Pickup_DropDownList_Dialog( const Asura_Vector_2& xOffset, const int iNumberOfLines, Asura_Hash_ID* puSelected, bool* pbChangedFlag, const char* szName)
: PARENT(xOffset)
{
	Initialise(xOffset, iNumberOfLines, puSelected, pbChangedFlag, szName);
}


u_int ToolsAVP_Entity_Pickup_DropDownList_Dialog::GetNumberOfHashValues() const
{
	// Return WEAPON + AMMO + HEALTH + TOOL PICKUPS
	return ( Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_AMMO,       true) +
		     Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_WEAPON,     true) +
			 Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_TOOL,       true) +
			 Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_HEALTH,     true) +
			 Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_AUDIODIARY, true) );
}

const char* ToolsAVP_Entity_Pickup_DropDownList_Dialog::GetNameByIndex(int iIndex) const
{
	int uFirstCount, uSecondCount, uThridCount, uFourthCount;
	Asura_Blueprint* pxBluePrint = NULL;

	if (( iIndex ) < (uFirstCount = Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_AMMO, true)))
	{
		pxBluePrint = Asura_Blueprint_System::GetChildBlueprint(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_AMMO, true, iIndex);
	}
	else if ( ( iIndex ) <  (uSecondCount = Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_WEAPON, true ) + uFirstCount ))
	{
		pxBluePrint = Asura_Blueprint_System::GetChildBlueprint(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_WEAPON, true, iIndex - uFirstCount);
	}
	else if ( ( iIndex ) <  (uThridCount = Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_TOOL, true ) + uSecondCount ))
	{
		pxBluePrint = Asura_Blueprint_System::GetChildBlueprint(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_TOOL, true, iIndex - uSecondCount);
	}
	else if ( ( iIndex ) <  (uFourthCount = Asura_Blueprint_System::GetNumChildBlueprints(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_HEALTH, true ) + uThridCount ))
	{
		pxBluePrint = Asura_Blueprint_System::GetChildBlueprint(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_HEALTH, true, iIndex - uThridCount);
	}
	else
	{
		pxBluePrint = Asura_Blueprint_System::GetChildBlueprint(uAVP_BLUEPRINT_TYPE_PICKUPS, uAVP_BLUEPRINT_NAME_PICKUP_AUDIODIARY, true, iIndex - uFourthCount );
	}

	if(pxBluePrint)
	{
		return (pxBluePrint->GetName().GetString());
	}
	Asura_Assert(false, "Index return no blueprint.");
	return NULL;
}