//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_PhysicalObject.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef TOOLSAVP_ENTITY_PHYSICALOBJECT_H___
#define TOOLSAVP_ENTITY_PHYSICALOBJECT_H___ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Entity/Tools_Entity_PhysicalObject.h"
#include "Interface/Asura_Dialog.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class AVP_ServerEntity_PhysicalObject;

// C O N S T A N T S

// C L A S S E S /////////////////////////////////////////////////////////////

class ToolsAVP_Entity_PhysicalObject : virtual public Tools_Entity_PhysicalObject
{
	friend class ToolsAVP_Entity_ShootAt;

public:
	virtual void	AddFilesToExportList(List<Asura_String>& xNameList) const;

	virtual bool	CanBeCopied			(void) const			{ return true; }
	virtual bool	UseStreamCopy		(void) const			{ return false; }

	virtual bool GetParameter(Asura_Hash_ID uParamID, class Tools_Parameter* pxParam) const;
	virtual bool SetParameter(Asura_Hash_ID uParamID, const class Tools_Parameter* pxParam);

	virtual void UpdateToolsData() {}

	// Provided for the blueprint tools to create a new physical object (or derived type).
	void SetBlueprintHashID( const Asura_Hash_ID uBlueprintHashID );
	void RefreshOverlay();

	static ToolsAVP_Entity_PhysicalObject* FindEntity( const Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }

protected:
    ToolsAVP_Entity_PhysicalObject( AVP_ServerEntity_PhysicalObject* pxPhysicalObject );
    virtual ~ToolsAVP_Entity_PhysicalObject();

	virtual void OnSelection();
	virtual void OnDeselection();
	virtual void UpdateWhileSelected();
	virtual void AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset );
	virtual bool HandlesThisEvent( int iEventID ) const;
	virtual bool HandlesThisMessage( int iMessageID );
	virtual void AddEventMessageBlock( int iEventID );
	virtual int RemoveEventMessageBlock( int iEventID );


    virtual Asura_Hash_ID GetDefaultBlueprintTypeHashID() const;
    virtual Asura_Hash_ID GetDefaultBlueprintHashID() const;

	void AddOverlayControls_CollisionFXMaterialBP( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, const bool bGotBlueprint );

    AVP_ServerEntity_PhysicalObject*	m_pxPhysicalObject;
	Asura_Hash_ID					    m_uBlueprintHashID;

private:
    static void ChooseBlueprint( int iThis );
	static void ChooseMaterialBlueprint( int iThis );
	static void RefreshBlueprintProperties();
    static void SelectAllWithBlueprint( int iThis );
	static void EditBlueprint( int iThis );
    bool m_bChanged;

	static ToolsAVP_Entity_PhysicalObject* s_pxThisToolsPhyObj;
    static Asura_Hashed_List< ToolsAVP_Entity_PhysicalObject > s_xList;

    typedef Tools_Entity_PhysicalObject PARENT;
};

class ToolsAVP_Dialog_SelectBlueprint : public Asura_Hash_DropDownList_Dialog
{
public:
	ToolsAVP_Dialog_SelectBlueprint(Asura_Vector_2& xOffset, Asura_Dialog_SharedValue<Asura_Hash_ID>* pxSharedHashValue, Asura_Hash_ID uBlueprintTypeHash, Asura_Hash_ID uBlueprintDefaultHash)
		: Asura_Hash_DropDownList_Dialog(xOffset)
	{
		m_uBlueprintTypeHash = uBlueprintTypeHash;
		m_uBlueprintDefaultHash = uBlueprintDefaultHash;

		Initialise(xOffset, 10, pxSharedHashValue, &m_bChanged, "");
	}

	virtual u_int GetNumberOfHashValues() const
	{
		return Asura_Blueprint_System::GetNumChildBlueprints(m_uBlueprintTypeHash, m_uBlueprintDefaultHash, false);
	}

	virtual const char* GetNameByIndex(int iIndex) const
	{
		Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetChildBlueprint(m_uBlueprintTypeHash, m_uBlueprintDefaultHash, false, iIndex);
		if (pxBlueprint)
		{
			return pxBlueprint->GetName().GetString();
		}
		return "NONE";
	}

private:
	Asura_Hash_ID m_uBlueprintTypeHash;
	Asura_Hash_ID m_uBlueprintDefaultHash;
	bool m_bChanged;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // #ifndef TOOLSAVP_ENTITY_PHYSICALOBJECT_H___
