//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_PlayerProxy.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/PlayerProxy/ToolsAVP_Entity_PlayerProxy.h"

// asura
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

// avp
#include "Entity/AVP_Messages.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/PlayerProxy/AVP_ServerEntity_PlayerProxy.h"

// tools
#include "Misc/Tools_Timed_Messages.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/ToolsAVP_Object_Type.h"


//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_PlayerProxy
//
//////////////////////////////////////////////////////////////////////////////

Asura_Hashed_List< ToolsAVP_Entity_PlayerProxy > ToolsAVP_Entity_PlayerProxy::s_xList;

ToolsAVP_Entity_PlayerProxy::ToolsAVP_Entity_PlayerProxy( Asura_Entity* pxEntity )
: PARENT( pxEntity )
{
	Type = TOT_Entity_PlayerProxy;
	s_xList.AddToList( this );
}

ToolsAVP_Entity_PlayerProxy::~ToolsAVP_Entity_PlayerProxy()
{
	s_xList.RemoveFromList( this );
}

AVP_ServerEntity_PlayerProxy* ToolsAVP_Entity_PlayerProxy::GetPlayerProxy() const
{
	return static_cast< AVP_ServerEntity_PlayerProxy* >( Entity );
}

bool ToolsAVP_Entity_PlayerProxy::HandlesThisEvent( int iEventID ) const
{
	switch( iEventID )
	{
	case AVP_ENTITY_EVENT_PLAYER_LOW_AMMO_CURRENT_WEAPON : return true;
	case AVP_ENTITY_EVENT_PLAYER_LOW_AMMO_BOTH_WEAPONS : return true;
	case AVP_ENTITY_EVENT_PLAYER_NO_AMMO_PRIMARY_WEAPON : return true;
	case AVP_ENTITY_EVENT_PLAYER_CLOAKED : return true;
	case AVP_ENTITY_EVENT_PLAYER_DECLOAKED : return true;
	case AVP_ENTITY_EVENT_PLAYER_ON_DEATH :  return true;
	default : break;
	}

	return PARENT::HandlesThisEvent( iEventID );
}

bool ToolsAVP_Entity_PlayerProxy::HandlesThisMessage( int iMessageID )
{
	return PARENT::HandlesThisMessage( iMessageID );
}


void ToolsAVP_Entity_PlayerProxy::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	AVP_ServerEntity_PlayerProxy* pxPlayerProxy = GetPlayerProxy();

	Asura_Dialog* pxDialogue = new Asura_Numerical_Dialog( "Player Number", xOffset, &pxPlayerProxy->m_uPlayerNumber, 1, 32, 1, "%d", &Changed );
	pxDialogue->SetEnabled( true );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

}

void ToolsAVP_Entity_PlayerProxy::OnSelection()
{
	PARENT::OnSelection();
	m_bRefreshOverlay = false;

}

void ToolsAVP_Entity_PlayerProxy::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( m_bRefreshOverlay )
	{
		DeleteOverlay();
		CreateOverlay();
		m_bRefreshOverlay = false;
	}
}


void ToolsAVP_Entity_PlayerProxy::Create()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Player Proxy", "Player Proxy", uGuid );
	if ( !pxToolsData ) { return; }

	AVP_ServerEntity_PlayerProxy* pxPlayerProxy = AVP_ServerEntity_PlayerProxy::Create( uGuid );
	if ( !pxPlayerProxy ) { return; }
	pxPlayerProxy->ToolsData = pxToolsData;

	ToolsAVP_Entity_PlayerProxy* pxToolsObject = ToolsAVP_Entity_PlayerProxy::Create( pxPlayerProxy );
	if ( !pxToolsObject ) { return; }
	Tools_Object_System::SelectObject( pxToolsObject );

	pxToolsObject->Position		= *Asura_Camera::GetPosition();
	pxToolsObject->Orientation  = Asura_Maths::IdentityMatrix3x3;
	pxToolsObject->m_uGuid		= uGuid;

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Player Proxy." );
	Tools_SaveAsura::SetEntityModified();
}

ToolsAVP_Entity_PlayerProxy* ToolsAVP_Entity_PlayerProxy::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Entity_PlayerProxy( pxEntity );
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
