//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_SpawnController.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Spawners/ToolsAVP_Entity_SpawnController.h"

// asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Core/Asura_String.h"
#include "Core/Memory/Asura_Mem.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Table_Dialog.h"

// axon

// avp
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Actor/Vertebrates/FaceHugger/AVP_FaceHugger_Description.h"
#include "Actor/Vertebrates/Human/AVP_Human_Description.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_Description.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/Spawners/AVP_ServerEntity_SpawnController.h"

// tools
#include "ToolsAVP_ChooseObjects.h"
#include "Entity/Controller/ToolsAVP_Entity_Squad.h"
#include "Entity/GamePlayVolume/ToolsAVP_Entity_GamePlayVolume.h"
#include "Entity/Spawners/ToolsAVP_Entity_SpawnPoint2.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/ToolsAVP_Object_Type.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_TableCell_NumericalDialog
//
//////////////////////////////////////////////////////////////////////////////

class ToolsAVP_TableCell_NumericalDialog : public Asura_TableCell_Dialog
{
	typedef Asura_TableCell_Dialog PARENT;

public:
	ToolsAVP_TableCell_NumericalDialog( Asura_Table_Dialog& xTableDialogue, const Asura_Vector_2& xOffset,
										const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn,
										float* pfDialogVar, bool* pbChanged )
	: PARENT( xTableDialogue, xOffset, xMinCellSize, uRow, uColumn )
	{
		Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
		Asura_Dialog* pxDialog = new Asura_Numerical_Dialog( "", xLocalOffset, pfDialogVar, 0.0f, 10.0f, 0.05f, "%0.2f", pbChanged );
		AddDialogToPanel( pxDialog );
		xLocalOffset.x += pxDialog->m_xMaxCoord.x;
	}

	virtual int	Compare( const Asura_TableCell_Dialog& xOther ) const
	{
		return 0;
	}
};

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_SpawnController
//
//////////////////////////////////////////////////////////////////////////////

TOOLS_DECLARE_SHARED_PARAM( int,   uTOOLSAVP_SHAREDPARAMHASH_SPAWN_MAXINPLAY,      Spawn_MaxInPlay );
TOOLS_DECLARE_SHARED_PARAM( int,   uTOOLSAVP_SHAREDPARAMHASH_SPAWN_MAXSPAWNCOUNT,  Spawn_MaxSpawnCount );
TOOLS_DECLARE_SHARED_PARAM( float, uTOOLSAVP_SHAREDPARAMHASH_SPAWN_SPAWNPERIODMIN, Spawn_SpawnPeriodMin );
TOOLS_DECLARE_SHARED_PARAM( float, uTOOLSAVP_SHAREDPARAMHASH_SPAWN_SPAWNPERIODMAX, Spawn_SpawnPeriodMax );

static const char* s_aszSpawnSchemeStrings[ AVP_SS_NUM_SPAWN_SCHEMES ] =
{
	"Random",
	"At Distance From Player"
};

Asura_Hash_ID ToolsAVP_Entity_SpawnController::s_uLastSpawneeTypeBPHashID = ASURA_HASH_ID_UNSET;

ToolsAVP_Entity_SpawnController::ToolsAVP_Entity_SpawnController( Asura_Entity* pxEntity )
: PARENT( pxEntity )
, m_bRefreshOverlay( false )
, m_bMaxInPlayChanged( false )
, m_xSquadFunctionDialogLabel()
, m_uSpawneeTypeListSize( 0 )
, m_ppszSpawneeTypeList( NULL )
, m_pxSpawneeTypeTableDialog( NULL )
, m_uSpawnPointListSize( 0 )
, m_ppszSpawnPointList( NULL )
, m_pxSpawnPointTableDialog( NULL )
, m_uGPVolumeListSize( 0 )
, m_ppszGPVolumeList( NULL )
, m_pxGPVolumeTableDialog( NULL )
{
	Type = TOT_Entity_SpawnController;
	SetMesh( "ToolsShape_SpawnController" );
}

ToolsAVP_Entity_SpawnController::~ToolsAVP_Entity_SpawnController()
{
	// These shouldn't still be around, but just in case.
	DestroySpawneeTypeList();
	DestroySpawnPointList();
	DestroyGPVolumeList();
}

void ToolsAVP_Entity_SpawnController::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();


	Asura_Dialog* pxDialogue = NULL;
	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	pxDialogue = new Asura_Numerical_Dialog( "Max In Play", xOffset, &g_xParam_Spawn_MaxInPlay, 1, 9999, 1, "%d" );
	pxDialogue->SetHelpText( "The max number of entities spawned from this controller allowed in play at a time." );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialogue = new Asura_Numerical_Dialog( "Max To Spawn (-1 for infinite)", xOffset, &g_xParam_Spawn_MaxSpawnCount, -1, 9999, 1, "%d" );
	pxDialogue->SetHelpText( "The max number of entities spawned from this controller in total." );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialogue = new Asura_Numerical_Dialog( "Time Between Spawns", xOffset, &g_xParam_Spawn_SpawnPeriodMin, 0.0f, 999.0f, 0.1f, "%.1f" );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x += 23.0f;

	pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &g_xParam_Spawn_SpawnPeriodMax, 0.0f, 999.0f, 0.1f, "%.1f" );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x -= 23.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();


	BuildSquadFunctionDialogLabel();
	pxDialogue = new Asura_Function_Dialog( m_xSquadFunctionDialogLabel.GetString(), xOffset, SetSpawneeSquad, reinterpret_cast< int >( this ) );
	pxDialogue->SetHelpText( "The squad the spawned entities will join. If not set they will join the default squad." );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialogue = new Asura_Function_Dialog( "Reset", xOffset, ResetSpawneeSquad, reinterpret_cast< int >( this ) );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialogue = new Asura_Dialog( "Enemies To Spawn:", xOffset );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( pxSpawnController->m_uNumSpawneeTypes )
	{
		m_pxSpawneeTypeTableDialog = CreateSpawneeTypeTable( xOffset );
		pxOverlay->AddDialog( m_pxSpawneeTypeTableDialog );
		xOffset.y += m_pxSpawneeTypeTableDialog->GetTableSize().y + 0.5f;
	}

	pxDialogue = new Asura_Function_Dialog( "Add Enemy", xOffset, AddSpawneeType, reinterpret_cast< int >( this ) );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x += 18.0f;

	pxDialogue = new Asura_Function_Dialog( "Remove Selected", xOffset, RemoveSelectedSpawneeType, reinterpret_cast< int >( this ) );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x -= 18.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialogue = new Asura_Dialog( "Spawn Scheme:", xOffset );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x += 10.0f;

	Asura_DropDown_List_Dialog* pxDDLDialogue = new Asura_DropDown_List_Dialog( xOffset, s_aszSpawnSchemeStrings, &pxSpawnController->m_iSpawnScheme,
		AVP_SS_NUM_SPAWN_SCHEMES, AVP_SS_NUM_SPAWN_SCHEMES, NULL, &Changed );
	pxDDLDialogue->SetOnSelectCallback( OnSpawnSchemeChanged, reinterpret_cast< int >( this ) );
	pxOverlay->AddDialog( pxDDLDialogue );
	xOffset.x -= 10.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( pxSpawnController->m_iSpawnScheme == AVP_SS_AT_DISTANCE_X )
	{
		pxDialogue = new Asura_Numerical_Dialog( "Distance", xOffset, &pxSpawnController->m_fSpawnDistance, 0.0f, 999.0f, 1.0f, "%.1f", &Changed );
		pxDialogue->SetHelpText( "The distance from the players' centroid to spawn at." );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::LargeSpacing();
	}
	else
	{
		xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();
	}


	pxDialogue = new Asura_Dialog( "Spawn Points:", xOffset );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( pxSpawnController->m_uNumSpawnPoints )
	{
		m_pxSpawnPointTableDialog = CreateSpawnPointTable( xOffset );
		pxOverlay->AddDialog( m_pxSpawnPointTableDialog );
		xOffset.y += m_pxSpawnPointTableDialog->GetTableSize().y + 0.5f;
	}

	pxDialogue = new Asura_Function_Dialog( "Add Spawn Points", xOffset, AddSpawnPoints, reinterpret_cast< int >( this ) );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x += 18.0f;

	pxDialogue = new Asura_Function_Dialog( "Remove Selected", xOffset, RemoveSelectedSpawnPoint, reinterpret_cast< int >( this ) );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x -= 18.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialogue = new Asura_Dialog( "Gameplay Volumes:", xOffset );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( pxSpawnController->m_uNumGPVolumes )
	{
		m_pxGPVolumeTableDialog = CreateGPVolumeTable( xOffset );
		pxOverlay->AddDialog( m_pxGPVolumeTableDialog );
		xOffset.y += m_pxGPVolumeTableDialog->GetTableSize().y + 0.5f;
	}

	pxDialogue = new Asura_Function_Dialog( "Add Gameplay Volume", xOffset, AddGPVolume, reinterpret_cast< int >( this ) );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x += 18.0f;

	pxDialogue = new Asura_Function_Dialog( "Remove Selected", xOffset, RemoveSelectedGPVolume, reinterpret_cast< int >( this ) );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x -= 18.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();
}

void ToolsAVP_Entity_SpawnController::Update()
{
	PARENT::Update();

	// If any of our data has changed then handle it.
	if ( m_bMaxInPlayChanged )
	{
		AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();
		delete [] pxSpawnController->m_pxSpawneeGuids;
		pxSpawnController->m_pxSpawneeGuids = new Asura_Guid[ pxSpawnController->m_iMaxInPlay ];
		m_bMaxInPlayChanged = false;
		Changed = true;
	}
}

void ToolsAVP_Entity_SpawnController::OnSelection()
{
	PARENT::OnSelection();
	m_bRefreshOverlay = false;
}

void ToolsAVP_Entity_SpawnController::OnDeselection()
{
	DestroySpawneeTypeList();
	DestroySpawnPointList();
	DestroyGPVolumeList();
	PARENT::OnDeselection();
}

void ToolsAVP_Entity_SpawnController::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( m_bRefreshOverlay )
	{
		RefreshOverlay();
		m_bRefreshOverlay = false;
	}
}

bool ToolsAVP_Entity_SpawnController::Verify( const bool bAutomatic )
{
	if ( !PARENT::Verify( bAutomatic ) )
	{
		return false;
	}

	bool bOk = true;

	// Make sure everything referenced in the game object actually exists.
	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	// Squad.
	if ( pxSpawnController->m_uSpawneeSquadGuid != ASURA_GUID_UNREGISTERED )
	{
		const ToolsAVP_Entity_Squad* pxSquad = ToolsAVP_Entity_Squad::FindEntity( pxSpawnController->m_uSpawneeSquadGuid );
		if ( !pxSquad )
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Objects", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"Spawn Controller '%s': could not find spawnee squad %u.", GetName(), pxSpawnController->m_uSpawneeSquadGuid );
			bOk = false;
		}
	}

	// Spawnee types.
	for ( u_int u = 0; u < pxSpawnController->m_uNumSpawneeTypes; ++u )
	{
		const AVP_ServerEntity_SpawnController::BlueprintAndWeight& xBPAndWeight = pxSpawnController->m_pxSpawneeTypes[ u ];

		Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( xBPAndWeight.m_uBPTypeHashID, xBPAndWeight.m_uBPHashID );
		if ( !pxBlueprint )
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Objects", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"Spawn Controller '%s': could not find spawnee blueprint with type hash 0x%08x, bp hash 0x%08x.",
				GetName(), xBPAndWeight.m_uBPTypeHashID, xBPAndWeight.m_uBPHashID );
			bOk = false;
		}
	}

	// Spawn points.
	for ( u_int u = 0; u < pxSpawnController->m_uNumSpawnPoints; ++u )
	{
		const AVP_ServerEntity_SpawnController::SpawnerAndWeight& xSpawnerAndWeight = pxSpawnController->m_pxSpawnPoints[ u ];

		const ToolsAVP_Entity_SpawnPoint2* pxSpawnPointTools = ToolsAVP_Entity_SpawnPoint2::FindEntity( xSpawnerAndWeight.m_uSpawnerGuid );
		if ( !pxSpawnPointTools )
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Objects", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"Spawn Controller '%s': could not find spawn point %u.", GetName(), xSpawnerAndWeight.m_uSpawnerGuid );
			bOk = false;
		}
	}

	// GP volumes.
	for ( u_int u = 0; u < pxSpawnController->m_uNumGPVolumes; ++u )
	{
		const AVP_ServerEntity_SpawnController::SpawnerAndWeight& xSpawnerAndWeight = pxSpawnController->m_pxGPVolumes[ u ];

		const ToolsAVP_Entity_GamePlayVolume* pxGPVolumeTools = ToolsAVP_Entity_GamePlayVolume::FindEntity( xSpawnerAndWeight.m_uSpawnerGuid );
		if ( !pxGPVolumeTools )
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Objects", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"Spawn Controller '%s': could not find game play volume %u.", GetName(), xSpawnerAndWeight.m_uSpawnerGuid );
			bOk = false;
		}
	}

	return bOk;
}

bool ToolsAVP_Entity_SpawnController::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch ( uParamID )
	{
		case g_uHash_Spawn_MaxInPlay:      pxParam->SetInt( GetSpawnController()->m_iMaxInPlay );        return true;
        case g_uHash_Spawn_MaxSpawnCount:  pxParam->SetInt( GetSpawnController()->m_iMaxSpawnCount );    return true;
		case g_uHash_Spawn_SpawnPeriodMin: pxParam->SetFloat( GetSpawnController()->m_fSpawnPeriodMin ); return true;
		case g_uHash_Spawn_SpawnPeriodMax: pxParam->SetFloat( GetSpawnController()->m_fSpawnPeriodMax ); return true;

		default: return PARENT::GetParameter( uParamID, pxParam );
	}
}

bool ToolsAVP_Entity_SpawnController::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	switch ( uParamID )
	{
		case g_uHash_Spawn_MaxInPlay:
		{
			GetSpawnController()->m_iMaxInPlay = pxParam->GetInt();
			m_bMaxInPlayChanged = true;
			return true;
		}

		case g_uHash_Spawn_MaxSpawnCount:
		{
			GetSpawnController()->m_iMaxSpawnCount = pxParam->GetInt();
			SetEntityModified();
			return true;
		}

		case g_uHash_Spawn_SpawnPeriodMin:
		{
			GetSpawnController()->m_fSpawnPeriodMin = pxParam->GetFloat();
			SetEntityModified();
			return true;
		}

		case g_uHash_Spawn_SpawnPeriodMax:
		{
			GetSpawnController()->m_fSpawnPeriodMax = pxParam->GetFloat();
			SetEntityModified();
			return true;
		}

		default: return PARENT::SetParameter( uParamID, pxParam );
	}
}

void ToolsAVP_Entity_SpawnController::Copy( const Tools_Object* pxObject )
{
	// Copy parent data.
	PARENT::Copy( pxObject );

	// Copy our data.
	const ToolsAVP_Entity_SpawnController* pxFromObject = static_cast< const ToolsAVP_Entity_SpawnController* >( pxObject );

	// Copy the engine entity's data across.
	AVP_ServerEntity_SpawnController* pxToEntity   = static_cast< AVP_ServerEntity_SpawnController* >( Entity );
	AVP_ServerEntity_SpawnController* pxFromEntity = static_cast< AVP_ServerEntity_SpawnController* >( pxFromObject->Entity );
	pxToEntity->m_iMaxInPlay		= pxFromEntity->m_iMaxInPlay;
	pxToEntity->m_iMaxSpawnCount	= pxFromEntity->m_iMaxSpawnCount;
	pxToEntity->m_fSpawnPeriodMin	= pxFromEntity->m_fSpawnPeriodMin;
	pxToEntity->m_fSpawnPeriodMax	= pxFromEntity->m_fSpawnPeriodMax;
	pxToEntity->m_uSpawneeSquadGuid = pxFromEntity->m_uSpawneeSquadGuid;
	pxToEntity->m_uNumSpawneeTypes	= pxFromEntity->m_uNumSpawneeTypes;
	delete [] pxToEntity->m_pxSpawneeTypes;
	pxToEntity->m_pxSpawneeTypes = NULL;
	if ( pxFromEntity->m_uNumSpawneeTypes )
	{
		pxToEntity->m_pxSpawneeTypes = new AVP_ServerEntity_SpawnController::BlueprintAndWeight[ pxFromEntity->m_uNumSpawneeTypes ];
		for ( u_int u = 0; u < pxFromEntity->m_uNumSpawneeTypes; ++u )
		{
			pxToEntity->m_pxSpawneeTypes[ u ] = pxFromEntity->m_pxSpawneeTypes[ u ];
		}
	}

	pxToEntity->m_iSpawnScheme		= pxFromEntity->m_iSpawnScheme;
	pxToEntity->m_fSpawnDistance	= pxFromEntity->m_fSpawnDistance;
	pxToEntity->m_uNumSpawnPoints	= pxFromEntity->m_uNumSpawnPoints;
	delete [] pxToEntity->m_pxSpawnPoints;
	pxToEntity->m_pxSpawnPoints = NULL;
	if ( pxFromEntity->m_uNumSpawnPoints )
	{
		pxToEntity->m_pxSpawnPoints = new AVP_ServerEntity_SpawnController::SpawnerAndWeight[ pxFromEntity->m_uNumSpawnPoints ];
		for ( u_int u = 0; u < pxFromEntity->m_uNumSpawnPoints; ++u )
		{
			pxToEntity->m_pxSpawnPoints[ u ] = pxFromEntity->m_pxSpawnPoints[ u ];
		}
	}

	pxToEntity->m_uNumGPVolumes	= pxFromEntity->m_uNumGPVolumes;
	delete [] pxToEntity->m_pxGPVolumes;
	pxToEntity->m_pxGPVolumes = NULL;
	if ( pxFromEntity->m_uNumGPVolumes )
	{
		pxToEntity->m_pxGPVolumes = new AVP_ServerEntity_SpawnController::SpawnerAndWeight[ pxFromEntity->m_uNumGPVolumes ];
		for ( u_int u = 0; u < pxFromEntity->m_uNumGPVolumes; ++u )
		{
			pxToEntity->m_pxGPVolumes[ u ] = pxFromEntity->m_pxGPVolumes[ u ];
		}
	}

	pxToEntity->m_fSpawnTimer		= pxFromEntity->m_fSpawnTimer;
	pxToEntity->m_uTotalSpawnCount  = pxFromEntity->m_uTotalSpawnCount;
	pxToEntity->m_iSpawneeCount		= pxFromEntity->m_iSpawneeCount;
	delete [] pxToEntity->m_pxSpawneeGuids;
	pxToEntity->m_pxSpawneeGuids = NULL;
	if ( pxFromEntity->m_iSpawneeCount )
	{
		pxToEntity->m_pxSpawneeGuids = new Asura_Guid[ pxFromEntity->m_iSpawneeCount ];
		for ( int i = 0; i < pxFromEntity->m_iSpawneeCount; ++i )
		{
			pxToEntity->m_pxSpawneeGuids[ i ] = pxFromEntity->m_pxSpawneeGuids[ i ];
		}
	}

	m_bRefreshOverlay = true;
}

Tools_Object* ToolsAVP_Entity_SpawnController::CreateObjectOfThisType() const
{
	return CreateAndReturn();
}

bool ToolsAVP_Entity_SpawnController::HandlesThisEvent( int iEventID ) const
{
	if ( iEventID == AVP_ENTITY_EVENT_SPAWN_ON_SPAWN ||
		iEventID == AVP_ENTITY_EVENT_SPAWN_ON_SPAWN_COUNT_EXHAUSTED || 
		iEventID == AVP_ENTITY_EVENT_SPAWN_ON_ANY_SPAWNEE_DEATH ||
		iEventID == AVP_ENTITY_EVENT_SPAWN_ON_ALL_SPAWNEES_DEAD ||
		iEventID == AVP_ENTITY_EVENT_DESPAWN_ON_SPAWN )
	{
		return true;
	}
	else
	{
		return PARENT::HandlesThisEvent( iEventID );
	}
}

bool ToolsAVP_Entity_SpawnController::HandlesThisMessage( int iMessage )
{
	if ( iMessage == AVP_MSGID_SPAWN_SET_MAX_IN_PLAY ||
		iMessage == AVP_MSGID_SPAWN_SET_MAX_SPAWN_COUNT ||
		iMessage == AVP_MSGID_SPAWN_RESET_SPAWN_COUNTER )
	{
		return true;
	}
	else
	{
		return PARENT::HandlesThisMessage( iMessage );
	}
}

void ToolsAVP_Entity_SpawnController::AddFilesToExportList( List< Asura_String >& xNameList ) const
{
	PARENT::AddFilesToExportList( xNameList );

	static const Asura_Hash_ID s_uAlienBPHash      = 0x058996c7; // Asura_GetHashID( "Alien" )
	static const Asura_Hash_ID s_uFaceHuggerBPHash = 0x4633fcb7; // Asura_GetHashID( "FaceHugger" )
	static const Asura_Hash_ID s_uAndroidBPHash    = 0xcca8366f; // Asura_GetHashID( "Android" )
	static const Asura_Hash_ID s_uCivilianBPHash   = 0xadd8d383; // Asura_GetHashID( "Civilian" )
	static const Asura_Hash_ID s_uMarineBPHash     = 0xbf8c8dc2; // Asura_GetHashID( "Marine" )
	static const Asura_Hash_ID s_uPredatorBPHash   = 0xb31282f7; // Asura_GetHashID( "Predator" )

	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();
	for ( u_int u = 0; u < pxSpawnController->m_uNumSpawneeTypes; ++u )
	{
		const Asura_Hash_ID uBPTypeHashID = pxSpawnController->m_pxSpawneeTypes[ u ].m_uBPTypeHashID;
		const Asura_Hash_ID uBPHashID	  = pxSpawnController->m_pxSpawneeTypes[ u ].m_uBPHashID;

		Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uBPTypeHashID, uBPHashID );
		if ( !pxBlueprint )
		{
			continue;
		}

		if ( pxBlueprint->IsChildOf( uBPTypeHashID, s_uAlienBPHash, false ) )
		{
			AVP_Alien_Description::AddFilesToExport( false, uBPHashID, xNameList );
		}
		else if ( pxBlueprint->IsChildOf( uBPTypeHashID, s_uFaceHuggerBPHash, false ) )
		{
			AVP_FaceHugger_Description::AddFilesToExport( false, uBPHashID, xNameList );
		}
		else if ( pxBlueprint->IsChildOf( uBPTypeHashID, s_uAndroidBPHash, false ) )
		{
			AVP_Human_Description::AddAndroidFilesToExport( false, uBPHashID, xNameList );
		}
		else if ( pxBlueprint->IsChildOf( uBPTypeHashID, s_uCivilianBPHash, false ) )
		{
			AVP_Human_Description::AddCivilianFilesToExport( false, uBPHashID, xNameList );
		}
		else if ( pxBlueprint->IsChildOf( uBPTypeHashID, s_uMarineBPHash, false ) )
		{
			AVP_Human_Description::AddMarineFilesToExport( false, uBPHashID, xNameList );
		}
		else if ( pxBlueprint->IsChildOf( uBPTypeHashID, s_uPredatorBPHash, false ) )
		{
			AVP_Predator_Description::AddFilesToExport( false, uBPHashID, xNameList );
		}
	}
}

void ToolsAVP_Entity_SpawnController::Create()
{
	CreateAndReturn();
}

ToolsAVP_Entity_SpawnController* ToolsAVP_Entity_SpawnController::CreateAndReturn()
{
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Spawn Controller", "Spawn Controller", uGuid );
	if ( !pxToolsData ) { return NULL; }

	AVP_ServerEntity_SpawnController* pxSpawnController = new AVP_ServerEntity_SpawnController( uGuid );
	if ( !pxSpawnController ) { return NULL; }
	pxSpawnController->ToolsData = pxToolsData;

	ToolsAVP_Entity_SpawnController* pxToolsObject = new ToolsAVP_Entity_SpawnController( pxSpawnController );
	if ( !pxToolsObject ) { return NULL; }
	Tools_Object_System::SelectObject( pxToolsObject );

	pxToolsObject->Position		= *Asura_Camera::GetPosition();
	pxToolsObject->Orientation  = Asura_Maths::IdentityMatrix3x3;
	pxToolsObject->m_uGuid		= uGuid;

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new spawn controller." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsObject;
}

Tools_Object* ToolsAVP_Entity_SpawnController::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Entity_SpawnController( pxEntity );
}

AVP_ServerEntity_SpawnController* ToolsAVP_Entity_SpawnController::GetSpawnController() const
{
	return static_cast< AVP_ServerEntity_SpawnController* >( Entity );
}

void ToolsAVP_Entity_SpawnController::RefreshOverlay()
{
	DeleteOverlay();
	CreateOverlay();
}

void ToolsAVP_Entity_SpawnController::AddNewSpawneeType( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID )
{
	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	// Check it's not already there.
	for ( u_int u = 0; u < pxSpawnController->m_uNumSpawneeTypes; ++u )
	{
		if ( pxSpawnController->m_pxSpawneeTypes[ u ].m_uBPTypeHashID == uBlueprintTypeHashID &&
			pxSpawnController->m_pxSpawneeTypes[ u ].m_uBPHashID == uBlueprintHashID )
		{
			return;
		}
	}

	const u_int uOldCount = pxSpawnController->m_uNumSpawneeTypes;
	AVP_ServerEntity_SpawnController::BlueprintAndWeight* pxOldArray = pxSpawnController->m_pxSpawneeTypes;
	const u_int uNewCount = uOldCount + 1;
	AVP_ServerEntity_SpawnController::BlueprintAndWeight* pxNewArray = new AVP_ServerEntity_SpawnController::BlueprintAndWeight[ uNewCount ];

	// Copy across.
	Asura_Mem::Copy( pxNewArray, pxOldArray, uOldCount * sizeof( AVP_ServerEntity_SpawnController::BlueprintAndWeight ) );

	// Add new.
	pxNewArray[ uOldCount ].m_uBPTypeHashID = uBlueprintTypeHashID;
    pxNewArray[ uOldCount ].m_uBPHashID     = uBlueprintHashID;
    pxNewArray[ uOldCount ].m_fWeight	    = 1.0f;

	// Write back.
	pxSpawnController->m_uNumSpawneeTypes = uNewCount;
	pxSpawnController->m_pxSpawneeTypes   = pxNewArray;

	// Ditch old.
	delete [] pxOldArray;
}

void ToolsAVP_Entity_SpawnController::RemoveSpawneeType( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID )
{
	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	const u_int uOldCount = pxSpawnController->m_uNumSpawneeTypes;
	AVP_ServerEntity_SpawnController::BlueprintAndWeight* pxOldArray = pxSpawnController->m_pxSpawneeTypes;
	const u_int uNewCount = uOldCount - 1;

	if ( uNewCount == 0 )
	{
		// Remove all.
		delete [] pxSpawnController->m_pxSpawneeTypes;
		pxSpawnController->m_pxSpawneeTypes   = NULL;
		pxSpawnController->m_uNumSpawneeTypes = 0;
	}
	else
	{
		AVP_ServerEntity_SpawnController::BlueprintAndWeight* pxNewArray = new AVP_ServerEntity_SpawnController::BlueprintAndWeight[ uNewCount ];

		// Copy across.
		for ( u_int uReadIndex = 0, uWriteIndex = 0; uReadIndex < uOldCount; ++uReadIndex )
		{
			if ( pxOldArray[ uReadIndex ].m_uBPTypeHashID != uBlueprintTypeHashID ||
				pxOldArray[ uReadIndex ].m_uBPHashID != uBlueprintHashID )
			{
				pxNewArray[ uWriteIndex++ ] = pxOldArray[ uReadIndex ];
			}
		}

		// Write back.
		pxSpawnController->m_uNumSpawneeTypes = uNewCount;
		pxSpawnController->m_pxSpawneeTypes   = pxNewArray;

		// Ditch old.
		delete [] pxOldArray;
	}
}

void ToolsAVP_Entity_SpawnController::AddNewSpawnPoint( const Asura_Guid uSpawnPointGuid )
{
	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	// Check it's not already there.
	for ( u_int u = 0; u < pxSpawnController->m_uNumSpawnPoints; ++u )
	{
		if ( pxSpawnController->m_pxSpawnPoints[ u ].m_uSpawnerGuid == uSpawnPointGuid )
		{
			return;
		}
	}

	const u_int uOldCount = pxSpawnController->m_uNumSpawnPoints;
	AVP_ServerEntity_SpawnController::SpawnerAndWeight* pxOldArray = pxSpawnController->m_pxSpawnPoints;
	const u_int uNewCount = uOldCount + 1;
	AVP_ServerEntity_SpawnController::SpawnerAndWeight* pxNewArray = new AVP_ServerEntity_SpawnController::SpawnerAndWeight[ uNewCount ];

	// Copy across.
	Asura_Mem::Copy( pxNewArray, pxOldArray, uOldCount * sizeof( AVP_ServerEntity_SpawnController::SpawnerAndWeight ) );

	// Add new.
	pxNewArray[ uOldCount ].m_uSpawnerGuid = uSpawnPointGuid;
	pxNewArray[ uOldCount ].m_fWeight	   = 1.0f;

	// Write back.
	pxSpawnController->m_uNumSpawnPoints = uNewCount;
	pxSpawnController->m_pxSpawnPoints   = pxNewArray;

	// Ditch old.
	delete [] pxOldArray;
}

void ToolsAVP_Entity_SpawnController::RemoveSpawnPoint( const Asura_Guid uSpawnPointGuid )
{
	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	const u_int uOldCount = pxSpawnController->m_uNumSpawnPoints;
	AVP_ServerEntity_SpawnController::SpawnerAndWeight* pxOldArray = pxSpawnController->m_pxSpawnPoints;
	const u_int uNewCount = uOldCount - 1;

	if ( uNewCount == 0 )
	{
		// Remove all.
		delete [] pxSpawnController->m_pxSpawnPoints;
		pxSpawnController->m_pxSpawnPoints   = NULL;
		pxSpawnController->m_uNumSpawnPoints = 0;
	}
	else
	{
		AVP_ServerEntity_SpawnController::SpawnerAndWeight* pxNewArray = new AVP_ServerEntity_SpawnController::SpawnerAndWeight[ uNewCount ];

		// Copy across.
		for ( u_int uReadIndex = 0, uWriteIndex = 0; uReadIndex < uOldCount; ++uReadIndex )
		{
			if ( pxOldArray[ uReadIndex ].m_uSpawnerGuid != uSpawnPointGuid )
			{
				pxNewArray[ uWriteIndex++ ] = pxOldArray[ uReadIndex ];
			}
		}

		// Write back.
		pxSpawnController->m_uNumSpawnPoints = uNewCount;
		pxSpawnController->m_pxSpawnPoints   = pxNewArray;

		// Ditch old.
		delete [] pxOldArray;
	}
}

void ToolsAVP_Entity_SpawnController::AddNewGPVolume( const Asura_Guid uGPVolumeGuid )
{
	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	// Check it's not already there.
	for ( u_int u = 0; u < pxSpawnController->m_uNumGPVolumes; ++u )
	{
		if ( pxSpawnController->m_pxGPVolumes[ u ].m_uSpawnerGuid == uGPVolumeGuid )
		{
			return;
		}
	}

	const u_int uOldCount = pxSpawnController->m_uNumGPVolumes;
	AVP_ServerEntity_SpawnController::SpawnerAndWeight* pxOldArray = pxSpawnController->m_pxGPVolumes;
	const u_int uNewCount = uOldCount + 1;
	AVP_ServerEntity_SpawnController::SpawnerAndWeight* pxNewArray = new AVP_ServerEntity_SpawnController::SpawnerAndWeight[ uNewCount ];

	// Copy across.
	Asura_Mem::Copy( pxNewArray, pxOldArray, uOldCount * sizeof( AVP_ServerEntity_SpawnController::SpawnerAndWeight ) );

	// Add new.
	pxNewArray[ uOldCount ].m_uSpawnerGuid = uGPVolumeGuid;
	pxNewArray[ uOldCount ].m_fWeight	   = 1.0f;

	// Write back.
	pxSpawnController->m_uNumGPVolumes = uNewCount;
    pxSpawnController->m_pxGPVolumes   = pxNewArray;

	// Ditch old.
	delete [] pxOldArray;
}

void ToolsAVP_Entity_SpawnController::RemoveGPVolume( const Asura_Guid uGPVolumeGuid )
{
	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	const u_int uOldCount = pxSpawnController->m_uNumGPVolumes;
	AVP_ServerEntity_SpawnController::SpawnerAndWeight* pxOldArray = pxSpawnController->m_pxGPVolumes;
	const u_int uNewCount = uOldCount - 1;

	if ( uNewCount == 0 )
	{
		// Remove all.
		delete [] pxSpawnController->m_pxGPVolumes;
		pxSpawnController->m_pxGPVolumes   = NULL;
		pxSpawnController->m_uNumGPVolumes = 0;
	}
	else
	{
		AVP_ServerEntity_SpawnController::SpawnerAndWeight* pxNewArray = new AVP_ServerEntity_SpawnController::SpawnerAndWeight[ uNewCount ];

		// Copy across.
		for ( u_int uReadIndex = 0, uWriteIndex = 0; uReadIndex < uOldCount; ++uReadIndex )
		{
			if ( pxOldArray[ uReadIndex ].m_uSpawnerGuid != uGPVolumeGuid )
			{
				pxNewArray[ uWriteIndex++ ] = pxOldArray[ uReadIndex ];
			}
		}

		// Write back.
		pxSpawnController->m_uNumGPVolumes = uNewCount;
		pxSpawnController->m_pxGPVolumes   = pxNewArray;

		// Ditch old.
		delete [] pxOldArray;
	}
}

void ToolsAVP_Entity_SpawnController::BuildSquadFunctionDialogLabel()
{
	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	m_xSquadFunctionDialogLabel = "Squad To Spawn In: ";

	if ( pxSpawnController->m_uSpawneeSquadGuid == ASURA_GUID_UNREGISTERED )
	{
		m_xSquadFunctionDialogLabel += "unset";
	}
	else
	{
		ToolsAVP_Entity_Squad* pxSquad = ToolsAVP_Entity_Squad::FindEntity( pxSpawnController->m_uSpawneeSquadGuid );
		if ( pxSquad )
		{
			m_xSquadFunctionDialogLabel += pxSquad->GetName();
		}
		else
		{
			m_xSquadFunctionDialogLabel += "ERROR - CANNOT FIND SQUAD";
		}
	}
}

void ToolsAVP_Entity_SpawnController::SetSpawneeSquad( int iThis )
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_AlienSquad, SetSpawneeSquad, iThis );
	pxChooseObject->AddType( TOT_Entity_MarineSquad );
	pxChooseObject->AddType( TOT_Entity_FaceHuggerSquad );
	Tools_Object_System::ChooseObject( "Select A Squad", pxChooseObject );
}

void ToolsAVP_Entity_SpawnController::SetSpawneeSquad( int iThis, const Asura_Guid uSpawneeSquadGuid )
{
	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );
	AVP_ServerEntity_SpawnController* pxSpawnController = pxThis->GetSpawnController();
	pxSpawnController->m_uSpawneeSquadGuid = uSpawneeSquadGuid;
	pxThis->SetEntityModified();
	pxThis->RequestRefreshOverlay();
}

void ToolsAVP_Entity_SpawnController::ResetSpawneeSquad( int iThis )
{
	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );

	pxThis->GetSpawnController()->m_uSpawneeSquadGuid = ASURA_GUID_UNREGISTERED;
	pxThis->Changed;
	pxThis->RequestRefreshOverlay();
}

Asura_Table_Dialog* ToolsAVP_Entity_SpawnController::CreateSpawneeTypeTable( const Asura_Vector_2& xOffset )
{
	CreateSpawneeTypeList();

	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	Asura_Table_Dialog* pxTableDialogue = new Asura_Table_Dialog( xOffset, 40.0f, 10.0f, true );
	Asura_Vector_2 xCellSpacing = pxTableDialogue->GetCellSpacing();
	xCellSpacing.y = 0.25f;
	pxTableDialogue->SetCellSpacing( xCellSpacing );
	pxTableDialogue->EnableRowSelection( true );

	const Asura_Vector_2 xCellSize( 5.0f, 1.0f );
	pxTableDialogue->CreateTable( xCellSize, pxSpawnController->m_uNumSpawneeTypes, 1, CreateSpawneeTypeCell, this, m_ppszSpawneeTypeList );

	return pxTableDialogue;
}

void ToolsAVP_Entity_SpawnController::CreateSpawneeTypeList()
{
	DestroySpawneeTypeList();

	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();
	m_uSpawneeTypeListSize = pxSpawnController->m_uNumSpawneeTypes;
    m_ppszSpawneeTypeList  = new char*[ m_uSpawneeTypeListSize ];

	for ( u_int u = 0; u < pxSpawnController->m_uNumSpawneeTypes; ++u )
	{
		const AVP_ServerEntity_SpawnController::BlueprintAndWeight& xBPAndWeight = pxSpawnController->m_pxSpawneeTypes[ u ];

		Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( xBPAndWeight.m_uBPTypeHashID, xBPAndWeight.m_uBPHashID );
		if ( pxBlueprint )
		{
			const u_int uNameLengthIncNull = strlen( pxBlueprint->GetName().GetString() ) + 1;
			m_ppszSpawneeTypeList[ u ] = new char[ uNameLengthIncNull ];
			Asura_PrintfToBfr( m_ppszSpawneeTypeList[ u ], "%s", uNameLengthIncNull, true, pxBlueprint->GetName().GetString() );
		}
		else
		{
			static const char* s_pszErrorString = "ERROR - CANNOT FIND BLUEPRINT";
			const u_int uNameLengthIncNull = strlen( s_pszErrorString ) + 1;
			m_ppszSpawneeTypeList[ u ] = new char[ uNameLengthIncNull ];
			Asura_PrintfToBfr( m_ppszSpawneeTypeList[ u ], "%s", uNameLengthIncNull, true, s_pszErrorString );
		}
	}
}

void ToolsAVP_Entity_SpawnController::DestroySpawneeTypeList()
{
	if ( m_ppszSpawneeTypeList )
	{
		for ( u_int u = 0; u < m_uSpawneeTypeListSize; ++u )
		{
			delete [] m_ppszSpawneeTypeList[ u ];
			m_ppszSpawneeTypeList[ u ] = NULL;
		}
		delete [] m_ppszSpawneeTypeList;
        m_ppszSpawneeTypeList  = NULL;
		m_uSpawneeTypeListSize = 0;
	}
}

Asura_TableCell_Dialog* ToolsAVP_Entity_SpawnController::CreateSpawneeTypeCell( Asura_Table_Dialog& xTableDialog,
																			    const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																			    const u_int uRow, const u_int uColumn,
																				void* const pxUserData )
{
	ToolsAVP_Entity_SpawnController*						pxThis				= static_cast< ToolsAVP_Entity_SpawnController* >( pxUserData );
	AVP_ServerEntity_SpawnController*						pxSpawnController	= pxThis->GetSpawnController();
	AVP_ServerEntity_SpawnController::BlueprintAndWeight&	xBPAndWeight		= pxSpawnController->m_pxSpawneeTypes[ uRow ];

	return new ToolsAVP_TableCell_NumericalDialog( xTableDialog, xOffset, xMinCellSize, uRow, uColumn, &xBPAndWeight.m_fWeight, &pxThis->Changed );
}

void ToolsAVP_Entity_SpawnController::AddSpawneeType( int iThis )
{
	const Asura_Hash_ID uActorBPHashID = Asura_GetHashID( "Actor" );
	Asura_Hash_ID uBlueprintHashID     = ( s_uLastSpawneeTypeBPHashID == ASURA_HASH_ID_UNSET ) ? uActorBPHashID : s_uLastSpawneeTypeBPHashID;
	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;

	BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID, 1, &uBlueprintTypeHashID, 1, &uActorBPHashID );
	if ( !xBPOverlay.DoModal() ) { return; }

	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );
	pxThis->AddNewSpawneeType( uBlueprintTypeHashID, uBlueprintHashID );
	pxThis->SetEntityModified();
	pxThis->RequestRefreshOverlay();

	s_uLastSpawneeTypeBPHashID = uBlueprintHashID;
}

void ToolsAVP_Entity_SpawnController::RemoveSelectedSpawneeType( int iThis )
{
	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );

	const int iSelectedRow = pxThis->m_pxSpawneeTypeTableDialog ? pxThis->m_pxSpawneeTypeTableDialog->GetCurrentlySelectedRow() : -1;
	if ( iSelectedRow != -1 )
	{
		AVP_ServerEntity_SpawnController* pxSpawnController = pxThis->GetSpawnController();
		AVP_ServerEntity_SpawnController::BlueprintAndWeight& xBPAndWeight = pxSpawnController->m_pxSpawneeTypes[ iSelectedRow ];
		pxThis->RemoveSpawneeType( xBPAndWeight.m_uBPTypeHashID, xBPAndWeight.m_uBPHashID );
		pxThis->SetEntityModified();
		pxThis->RequestRefreshOverlay();
	}
}

void ToolsAVP_Entity_SpawnController::OnSpawnSchemeChanged( int iThis )
{
	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );
	pxThis->RequestRefreshOverlay();
}

Asura_Table_Dialog* ToolsAVP_Entity_SpawnController::CreateSpawnPointTable( const Asura_Vector_2& xOffset )
{
	CreateSpawnPointList();

	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	Asura_Table_Dialog* pxTableDialogue = new Asura_Table_Dialog( xOffset, 40.0f, 10.0f, true );
	Asura_Vector_2 xCellSpacing = pxTableDialogue->GetCellSpacing();
	xCellSpacing.y = 0.25f;
	pxTableDialogue->SetCellSpacing( xCellSpacing );
	pxTableDialogue->EnableRowSelection( true );

	const Asura_Vector_2 xCellSize( 5.0f, 1.0f );
	pxTableDialogue->CreateTable( xCellSize, pxSpawnController->m_uNumSpawnPoints, 1, CreateSpawnPointCell, this, m_ppszSpawnPointList );

	return pxTableDialogue;
}

void ToolsAVP_Entity_SpawnController::CreateSpawnPointList()
{
	DestroySpawnPointList();

	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();
	m_uSpawnPointListSize = pxSpawnController->m_uNumSpawnPoints;
    m_ppszSpawnPointList  = new char*[ m_uSpawnPointListSize ];

	for ( u_int u = 0; u < pxSpawnController->m_uNumSpawnPoints; ++u )
	{
		const AVP_ServerEntity_SpawnController::SpawnerAndWeight& xSpawnerAndWeight = pxSpawnController->m_pxSpawnPoints[ u ];

		ToolsAVP_Entity_SpawnPoint2* pxSpawnPointTools = ToolsAVP_Entity_SpawnPoint2::FindEntity( xSpawnerAndWeight.m_uSpawnerGuid );
		if ( pxSpawnPointTools )
		{
			const u_int uNameLengthIncNull = strlen( pxSpawnPointTools->GetName() ) + 1;
			m_ppszSpawnPointList[ u ] = new char[ uNameLengthIncNull ];
			Asura_PrintfToBfr( m_ppszSpawnPointList[ u ], "%s", uNameLengthIncNull, true, pxSpawnPointTools->GetName() );
		}
		else
		{
			static const char* s_pszErrorString = "ERROR - CANNOT FIND SPAWN POINT";
			const u_int uNameLengthIncNull = strlen( s_pszErrorString ) + 1;
			m_ppszSpawnPointList[ u ] = new char[ uNameLengthIncNull ];
			Asura_PrintfToBfr( m_ppszSpawnPointList[ u ], "%s", uNameLengthIncNull, true, s_pszErrorString );
		}
	}
}

void ToolsAVP_Entity_SpawnController::DestroySpawnPointList()
{
	if ( m_ppszSpawnPointList )
	{
		for ( u_int u = 0; u < m_uSpawnPointListSize; ++u )
		{
			delete [] m_ppszSpawnPointList[ u ];
			m_ppszSpawnPointList[ u ] = NULL;
		}
		delete [] m_ppszSpawnPointList;
        m_ppszSpawnPointList  = NULL;
		m_uSpawnPointListSize = 0;
	}
}

Asura_TableCell_Dialog* ToolsAVP_Entity_SpawnController::CreateSpawnPointCell( Asura_Table_Dialog& xTableDialog, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, void* const pxUserData )
{
	ToolsAVP_Entity_SpawnController*					pxThis				= static_cast< ToolsAVP_Entity_SpawnController* >( pxUserData );
	AVP_ServerEntity_SpawnController*					pxSpawnController	= pxThis->GetSpawnController();
	AVP_ServerEntity_SpawnController::SpawnerAndWeight&	xSpawnerAndWeight	= pxSpawnController->m_pxSpawnPoints[ uRow ];

	return new ToolsAVP_TableCell_NumericalDialog( xTableDialog, xOffset, xMinCellSize, uRow, uColumn, &xSpawnerAndWeight.m_fWeight, &pxThis->Changed );
}

void ToolsAVP_Entity_SpawnController::AddSpawnPoints( int iThis )
{
	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );

	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_SpawnPoint2, AddNewSpawnPoint, reinterpret_cast< int >( pxThis ) );
	pxChooseObject->SetAllowMultipleSelection( true );
	Tools_Object_System::ChooseObject( "Select Spawn Points", pxChooseObject );
}

void ToolsAVP_Entity_SpawnController::AddNewSpawnPoint( int iThis, Asura_Guid uSpawnPointGuid )
{
	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );
	pxThis->AddNewSpawnPoint( uSpawnPointGuid );
	pxThis->SetEntityModified();
	pxThis->RequestRefreshOverlay();
}

void ToolsAVP_Entity_SpawnController::RemoveSelectedSpawnPoint( int iThis )
{
	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );

	const int iSelectedRow = pxThis->m_pxSpawnPointTableDialog ? pxThis->m_pxSpawnPointTableDialog->GetCurrentlySelectedRow() : -1;
	if ( iSelectedRow != -1 )
	{
		AVP_ServerEntity_SpawnController* pxSpawnController = pxThis->GetSpawnController();
		AVP_ServerEntity_SpawnController::SpawnerAndWeight& xSpawnerAndWeight = pxSpawnController->m_pxSpawnPoints[ iSelectedRow ];
		pxThis->RemoveSpawnPoint( xSpawnerAndWeight.m_uSpawnerGuid );
		pxThis->RequestRefreshOverlay();
		pxThis->Changed = true;
	}
}

Asura_Table_Dialog* ToolsAVP_Entity_SpawnController::CreateGPVolumeTable( const Asura_Vector_2& xOffset )
{
	CreateGPVolumeList();

	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();

	Asura_Table_Dialog* pxTableDialogue = new Asura_Table_Dialog( xOffset, 40.0f, 10.0f, true );
	Asura_Vector_2 xCellSpacing = pxTableDialogue->GetCellSpacing();
	xCellSpacing.y = 0.25f;
	pxTableDialogue->SetCellSpacing( xCellSpacing );
	pxTableDialogue->EnableRowSelection( true );

	const Asura_Vector_2 xCellSize( 5.0f, 1.0f );
	pxTableDialogue->CreateTable( xCellSize, pxSpawnController->m_uNumGPVolumes, 1, CreateGPVolumeCell, this, m_ppszGPVolumeList );

	return pxTableDialogue;
}

void ToolsAVP_Entity_SpawnController::CreateGPVolumeList()
{
	DestroyGPVolumeList();

	AVP_ServerEntity_SpawnController* pxSpawnController = GetSpawnController();
	m_uGPVolumeListSize = pxSpawnController->m_uNumGPVolumes;
    m_ppszGPVolumeList  = new char*[ m_uGPVolumeListSize ];

	for ( u_int u = 0; u < pxSpawnController->m_uNumGPVolumes; ++u )
	{
		const AVP_ServerEntity_SpawnController::SpawnerAndWeight& xSpawnerAndWeight = pxSpawnController->m_pxGPVolumes[ u ];

		ToolsAVP_Entity_GamePlayVolume* pxGPVolumeTools = ToolsAVP_Entity_GamePlayVolume::FindEntity( xSpawnerAndWeight.m_uSpawnerGuid );
		if ( pxGPVolumeTools )
		{
			const u_int uNameLengthIncNull = strlen( pxGPVolumeTools->GetName() ) + 1;
			m_ppszGPVolumeList[ u ] = new char[ uNameLengthIncNull ];
			Asura_PrintfToBfr( m_ppszGPVolumeList[ u ], "%s", uNameLengthIncNull, true, pxGPVolumeTools->GetName() );
		}
		else
		{
			static const char* s_pszErrorString = "ERROR - CANNOT FIND GPV";
			const u_int uNameLengthIncNull = strlen( s_pszErrorString ) + 1;
			m_ppszGPVolumeList[ u ] = new char[ uNameLengthIncNull ];
			Asura_PrintfToBfr( m_ppszGPVolumeList[ u ], "%s", uNameLengthIncNull, true, s_pszErrorString );
		}
	}
}

void ToolsAVP_Entity_SpawnController::DestroyGPVolumeList()
{
	if ( m_ppszGPVolumeList )
	{
		for ( u_int u = 0; u < m_uGPVolumeListSize; ++u )
		{
			delete [] m_ppszGPVolumeList[ u ];
			m_ppszGPVolumeList[ u ] = NULL;
		}
		delete [] m_ppszGPVolumeList;
        m_ppszGPVolumeList  = NULL;
		m_uGPVolumeListSize = 0;
	}
}

Asura_TableCell_Dialog* ToolsAVP_Entity_SpawnController::CreateGPVolumeCell( Asura_Table_Dialog& xTableDialog, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, void* const pxUserData )
{
	ToolsAVP_Entity_SpawnController*					pxThis				= static_cast< ToolsAVP_Entity_SpawnController* >( pxUserData );
	AVP_ServerEntity_SpawnController*					pxSpawnController	= pxThis->GetSpawnController();
	AVP_ServerEntity_SpawnController::SpawnerAndWeight&	xSpawnerAndWeight	= pxSpawnController->m_pxGPVolumes[ uRow ];

	return new ToolsAVP_TableCell_NumericalDialog( xTableDialog, xOffset, xMinCellSize, uRow, uColumn, &xSpawnerAndWeight.m_fWeight, &pxThis->Changed );
}

void ToolsAVP_Entity_SpawnController::AddGPVolume( int iThis )
{
	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );

	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_GamePlayVolume, AddNewGPVolume, reinterpret_cast< int >( pxThis ) );
	pxChooseObject->SetAllowMultipleSelection( true );
	Tools_Object_System::ChooseObject( "Select A Gameplay Volume", pxChooseObject );
}

void ToolsAVP_Entity_SpawnController::AddNewGPVolume( int iThis, Asura_Guid uGPVolumeGuid )
{
	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );
	pxThis->AddNewGPVolume( uGPVolumeGuid );
	pxThis->SetEntityModified();
	pxThis->RequestRefreshOverlay();
}

void ToolsAVP_Entity_SpawnController::RemoveSelectedGPVolume( int iThis )
{
	ToolsAVP_Entity_SpawnController* pxThis = reinterpret_cast< ToolsAVP_Entity_SpawnController* >( iThis );

	const int iSelectedRow = pxThis->m_pxGPVolumeTableDialog ? pxThis->m_pxGPVolumeTableDialog->GetCurrentlySelectedRow() : -1;
	if ( iSelectedRow != -1 )
	{
		AVP_ServerEntity_SpawnController* pxSpawnController = pxThis->GetSpawnController();
		AVP_ServerEntity_SpawnController::SpawnerAndWeight& xSpawnerAndWeight = pxSpawnController->m_pxGPVolumes[ iSelectedRow ];
		pxThis->RemoveGPVolume( xSpawnerAndWeight.m_uSpawnerGuid );
		pxThis->RequestRefreshOverlay();
		pxThis->Changed = true;
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
