//------------------------------------------------------------------------------
// File   : ToolsAVP_Entity_SpawnPoint.cpp
// Author : Gareth Thomas
// Date   : 22 / 10 / 07
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Include files...
//------------------------------------------------------------------------------
// Always include Asura Core first
#include "Core/Asura.h"
// This Files Header
#include "Entity/Spawners/ToolsAVP_Entity_SpawnPoint.h"
// The Game Entities Include Files
#include "Entity/Spawners/AVP_ServerEntity_SpawnPoint.h"
// Asura include files
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Network/Asura_Message.h"
#include "Entity/Locations/Asura_ServerEntity_Node.h"
#include "Rendering/Asura_Render.h"

// Axon Include Files
// AVP Include files
// AVP Axon Include Files
// Tools Include Files
#include "Entity/Tools_Entity_Object.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Axon/Tools_Object_Axon_ChooseObjects.h"

//------------------------------------------------------------------------------
// Constants, Defines, Statics and Globals
//------------------------------------------------------------------------------
ToolsAVP_Entity_SpawnPoint*	ToolsAVP_Entity_SpawnPoint::s_pxSpawnPoint;

Asura_Dialog* ToolsAVP_Entity_SpawnPoint::s_pxGameSceneOutNameDialog = NULL;
Asura_Dialog* ToolsAVP_Entity_SpawnPoint::s_pxGameSceneInNameDialog = NULL;

Asura_Guid ToolsAVP_Entity_SpawnPoint::s_uOldGameSceneOutGuid = ASURA_GUID_UNREGISTERED;
Asura_Guid ToolsAVP_Entity_SpawnPoint::s_uOldGameSceneInGuid = ASURA_GUID_UNREGISTERED;

// Substitute text for gamescene names
const char* ToolsAVP_Entity_SpawnPoint::s_pszUnset = "UNSET";
const char* ToolsAVP_Entity_SpawnPoint::s_pszNotFound = "NOT FOUND";

//------------------------------------------------------------------------------
// Function Implementations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// ToolsAVP_Entity_SpawnPoint :: Constructor
//------------------------------------------------------------------------------
ToolsAVP_Entity_SpawnPoint::ToolsAVP_Entity_SpawnPoint(Asura_Entity* pxEntity) : PARENT(pxEntity)
, m_uGameSceneOut(ASURA_GUID_UNREGISTERED)
, m_uGameSceneIn(ASURA_GUID_UNREGISTERED)
, m_bUseGameSceneIn(true)
{
	Type = TOT_Entity_SpawnPoint;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_SpawnPoint");

	AVP_ServerEntity_SpawnPoint* pxAVPEntity = static_cast< AVP_ServerEntity_SpawnPoint* >( pxEntity );
	Position		= pxAVPEntity->GetPosition();
	m_uGameSceneOut	= pxAVPEntity->m_uGameSceneOutGuid;
	m_uGameSceneIn	= pxAVPEntity->m_uGameSceneInGuid;
}

ToolsAVP_Entity_SpawnPoint::~ToolsAVP_Entity_SpawnPoint(void)
{
}

void ToolsAVP_Entity_SpawnPoint::Project_Create(void)
{
	ToolsAVP_Entity_SpawnPoint::Create();
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_SpawnPoint :: Create
//------------------------------------------------------------------------------
void ToolsAVP_Entity_SpawnPoint::Create(void)
{
	const Asura_Guid	uGuid	= Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Spawn Point", "Spawn Point", uGuid);

	if (pxToolsData == NULL)
	{
		Asura_Assert(false, "ToolsAVP_Entity_SpawnPoint::Create() - no tools data");
	}
	else
	{
		AVP_ServerEntity_SpawnPoint* pxServerObject = new AVP_ServerEntity_SpawnPoint(uGuid);

		if (pxServerObject == NULL)
		{
			Asura_Assert(false, "ToolsAVP_Entity_SpawnPoint::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			ToolsAVP_Entity_SpawnPoint*	pxToolsObject	= new ToolsAVP_Entity_SpawnPoint(pxServerObject);

			// Create the game specific tools object and select it.
			if (pxToolsObject == NULL)
			{
				Asura_Assert(false, "ToolsAVP_Entity_SpawnPoint::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				Asura_Matrix_3x3 xMat;

				xMat.Identity();

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->Orientation  = xMat;
				pxToolsObject->m_uGuid		= uGuid;

				pxServerObject->SetPosition(pxToolsObject->Position);
				pxServerObject->SetOrientation(pxToolsObject->Orientation);

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created New Spawn Point");
				Tools_SaveAsura::SetEntityModified();
			}
		}
	}
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_SpawnPoint :: Create
//------------------------------------------------------------------------------
Tools_Object* ToolsAVP_Entity_SpawnPoint::Create(Asura_Entity* pxEntity)
{
	return new ToolsAVP_Entity_SpawnPoint(pxEntity);
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_SpawnPoint :: OnSelection
//------------------------------------------------------------------------------
void ToolsAVP_Entity_SpawnPoint::OnSelection(void)
{
	s_pxSpawnPoint = this;
	PARENT::OnSelection();
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_SpawnPoint :: OnDeselection
//------------------------------------------------------------------------------
void ToolsAVP_Entity_SpawnPoint::OnDeselection(void)
{
	PARENT::OnDeselection();
	s_pxSpawnPoint = NULL;
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_SpawnPoint :: UpdatePositionInAsuraData
//------------------------------------------------------------------------------
void ToolsAVP_Entity_SpawnPoint::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	AVP_ServerEntity_SpawnPoint* pxAsuraEntity = static_cast<AVP_ServerEntity_SpawnPoint*>(Entity);

	//	Update the position if it has changed
	if	(pxAsuraEntity->m_xPosition != Position)
	{
		pxAsuraEntity->SetPosition(Position);
		Changed	= true;
	}

	if (pxAsuraEntity->GetOrientation() != Orientation)
	{
		pxAsuraEntity->SetOrientation(Orientation);
		Changed	= true;
	}

	PARENT::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_SpawnPoint :: UpdateWhileSelected
//------------------------------------------------------------------------------
void ToolsAVP_Entity_SpawnPoint::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	// Keep gamescene node name text on overlay up-to-date
	if (m_uGameSceneOut != s_uOldGameSceneOutGuid )
	{
		UpdateGameSceneName( s_pxGameSceneOutNameDialog, s_pxSpawnPoint->m_uGameSceneOut );
		s_uOldGameSceneOutGuid = m_uGameSceneOut;
	}

	// Keep gamescene node name text on overlay up-to-date
	if (m_uGameSceneIn != s_uOldGameSceneInGuid )
	{
		UpdateGameSceneName( s_pxGameSceneInNameDialog, s_pxSpawnPoint->m_uGameSceneIn );
		s_uOldGameSceneInGuid = m_uGameSceneIn;
	}
}

void ToolsAVP_Entity_SpawnPoint::Update()
{
	AVP_ServerEntity_SpawnPoint* pxAVPEntity = (AVP_ServerEntity_SpawnPoint*)Entity;

	if (pxAVPEntity)
	{
		// See if gamescene nodes still exist
		if (m_uGameSceneOut != ASURA_GUID_UNREGISTERED)
		{
			//Asura_ServerNode_GamesceneNode* pxNode = Asura_ServerNode_GamesceneNode::FindEntity(m_uGameSceneOut);

			//if (!pxNode)
			//{
			//	m_uGameSceneOut = ASURA_GUID_UNREGISTERED;
			//}
		}

		if (m_uGameSceneIn != ASURA_GUID_UNREGISTERED)
		{
			//Asura_ServerNode_GamesceneNode* pxNode = Asura_ServerNode_GamesceneNode::FindEntity(m_uGameSceneIn);

			//if (!pxNode)
			//{
			//	m_uGameSceneIn = ASURA_GUID_UNREGISTERED;
			//}
		}

		// Update gamescene node guid in entity
		if (pxAVPEntity->m_uGameSceneOutGuid != m_uGameSceneOut)
		{
			pxAVPEntity->m_uGameSceneOutGuid = m_uGameSceneOut;
			Changed = true;
		}

		if (pxAVPEntity->m_uGameSceneInGuid != m_uGameSceneIn)
		{
			pxAVPEntity->m_uGameSceneInGuid = m_uGameSceneIn;
			Changed = true;
		}
	}
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_SpawnPoint :: Render
//------------------------------------------------------------------------------
void ToolsAVP_Entity_SpawnPoint::Render()
{
	PARENT::Render();

	if (m_uGameSceneOut != ASURA_GUID_UNREGISTERED)
	{
		Asura_ServerEntity_Node* pxNode = Asura_ServerEntity_Node::FindEntity(m_uGameSceneOut);

		if (pxNode)
		{
			Asura_Render::RenderLine(pxNode->GetPosition(), Position, 0xff0000ff);
		}
	}

	if (m_uGameSceneIn != ASURA_GUID_UNREGISTERED)
	{
		Asura_ServerEntity_Node* pxNode = Asura_ServerEntity_Node::FindEntity(m_uGameSceneIn);

		if (pxNode)
		{
			Asura_Render::RenderLine(pxNode->GetPosition(), Position, 0xffff0000 );
		}
	}
}

//------------------------------------------------------------------------------
// ToolsAVP_Entity_SpawnPoint :: AddOverlayControls
//------------------------------------------------------------------------------
void ToolsAVP_Entity_SpawnPoint::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	PARENT::AddOverlayControls(pxOverlay, xOffset);

	// Reset gamescene node guid variables
	s_uOldGameSceneOutGuid = ASURA_GUID_UNREGISTERED;
	s_uOldGameSceneInGuid = ASURA_GUID_UNREGISTERED;

	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_StringDisplay_Dialog("GameScene Out Node:", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( s_pszUnset, xOffset, SelectGamesceneOut, &Changed);
	pxDialog->SetHelpText( "The node the actor will start a gamescene animation at once spawned." );
	s_pxGameSceneOutNameDialog = pxDialog;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Clear", xOffset, ClearGamesceneOut, &Changed);
	pxDialog->SetHelpText( "Clears the gamescene out value." );
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	if (m_bUseGameSceneIn)
	{
		pxDialog = new Asura_StringDisplay_Dialog("GameScene In Node:", xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog( s_pszUnset, xOffset, SelectGamesceneIn, &Changed);
		pxDialog->SetHelpText( "The node the actor will use when attempting to enter this spawn point." );
		s_pxGameSceneInNameDialog = pxDialog;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog( "Clear", xOffset, ClearGamesceneIn, &Changed);
		pxDialog->SetHelpText( "Clears the gamescene in value." );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::LargeSpacing();
	}
} 

void ToolsAVP_Entity_SpawnPoint::SelectGamesceneOut(void)
{
	Tools_ChooseObject_GS2Node* pxChooseObject = new Tools_ChooseObject_GS2Node( &s_pxSpawnPoint->m_uGameSceneOut, &s_pxSpawnPoint->Changed );
	if ( pxChooseObject )
	{
		Tools_Object_System::ChooseObject( "Pick the gamescene node the spawned actor will use after spawning.", pxChooseObject );
	}
}

void ToolsAVP_Entity_SpawnPoint::ClearGamesceneOut(void)
{
	s_pxSpawnPoint->m_uGameSceneOut = ASURA_GUID_UNREGISTERED;
}

void ToolsAVP_Entity_SpawnPoint::SelectGamesceneIn(void)
{
	Tools_ChooseObject_GS2Node* pxChooseObject = new Tools_ChooseObject_GS2Node( &s_pxSpawnPoint->m_uGameSceneIn, &s_pxSpawnPoint->Changed );
	if ( pxChooseObject )
	{
		Tools_Object_System::ChooseObject( "Pick the gamescene node the spawned actor will use when entering the spawn point.", pxChooseObject );
	}
}

void ToolsAVP_Entity_SpawnPoint::ClearGamesceneIn(void)
{
	s_pxSpawnPoint->m_uGameSceneIn = ASURA_GUID_UNREGISTERED;
}

void ToolsAVP_Entity_SpawnPoint::SetSelectedSpawner(ToolsAVP_Entity_SpawnPoint* pxSpawnPoint)
{
	s_pxSpawnPoint = pxSpawnPoint;
}

void ToolsAVP_Entity_SpawnPoint::ClearSelectedSpawner()
{
	s_pxSpawnPoint = NULL;
	s_pxGameSceneOutNameDialog = NULL;
	s_pxGameSceneInNameDialog = NULL;
}

void ToolsAVP_Entity_SpawnPoint::UpdateGameSceneName(Asura_Dialog* pxDialog, Asura_Guid uGameSceneGuid)
{
	if( pxDialog )
	{
		if( uGameSceneGuid != ASURA_GUID_UNREGISTERED )
		{
			Tools_Entity_Object* pxGameSceneToolsNode = Tools_Entity_Object::FindEntity( uGameSceneGuid );

			// Just a safety check, shouldn't ever be NULL (AN 04/02/08)
			if( pxGameSceneToolsNode )
			{
				pxDialog->UpdateLabel( pxGameSceneToolsNode->GetName() );
			}
			else
			{
				pxDialog->UpdateLabel( s_pszNotFound );
			}
		}
		else
		{
			pxDialog->UpdateLabel( s_pszUnset );
		}
	}
}
