//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_SpawnPoint2.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLSAVP_ENTITY_SPAWNPOINT2_H__
#define __TOOLSAVP_ENTITY_SPAWNPOINT2_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// tools
#include "Entity/Locations/OrientedNode/Tools_EntityNode_OrientedNode.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Table_Dialog;
class Asura_TableCell_Dialog;
class AVP_ServerEntity_SpawnPoint2;

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_SpawnPoint2
//
//////////////////////////////////////////////////////////////////////////////

const Asura_Hash_ID uTOOLSAVP_SHAREDPARAMHASH_SPAWN_MAXINPLAY      = 0xc6289879; // Asura_GetHashID( "Spawn_MaxInPlay" )
const Asura_Hash_ID uTOOLSAVP_SHAREDPARAMHASH_SPAWN_MAXSPAWNCOUNT  = 0xcf0d1d14; // Asura_GetHashID( "Spawn_MaxSpawnCount" )
const Asura_Hash_ID uTOOLSAVP_SHAREDPARAMHASH_SPAWN_SPAWNPERIODMIN = 0x517c491a; // Asura_GetHashID( "Spawn_SpawnPeriodMin" )
const Asura_Hash_ID uTOOLSAVP_SHAREDPARAMHASH_SPAWN_SPAWNPERIODMAX = 0x517c482c; // Asura_GetHashID( "Spawn_SpawnPeriodMax" )

class ToolsAVP_Entity_SpawnPoint2 : public Tools_EntityNode_OrientedNode
{
	typedef Tools_EntityNode_OrientedNode PARENT;

public:
	ToolsAVP_Entity_SpawnPoint2( Asura_Entity* pxEntity );
	virtual ~ToolsAVP_Entity_SpawnPoint2();

	virtual const char*	GetTypeDescription() { return "Spawn Point"; }
	virtual void		AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset );
	virtual void		Update();
	virtual void		OnSelection();
	virtual void		OnDeselection();
	virtual void		UpdateWhileSelected();
	virtual bool		Verify( const bool bAutomatic );

	virtual bool GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const;
	virtual bool SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam );

	virtual bool CanBeCopied() const { return true; }
	virtual void Copy( const Tools_Object* pxObject );
	virtual Tools_Object* CreateObjectOfThisType() const;

	virtual bool HandlesThisEvent( int iEventID ) const;
	virtual bool HandlesThisMessage( int iMessage );
	virtual void AddFilesToExportList( List< Asura_String >& xNameList ) const;

	static void	Create();
	static ToolsAVP_Entity_SpawnPoint2* CreateAndReturn();
	static Tools_Object* Create( Asura_Entity* pxEntity );

	static ToolsAVP_Entity_SpawnPoint2* FindEntity( const Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }

private:
	AVP_ServerEntity_SpawnPoint2* GetSpawnPoint() const;
	void VerifyGameObjectData();
	void RequestRefreshOverlay() { m_bRefreshOverlay = true; }
	void RefreshOverlay();
	void AddNewSpawneeType( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID );
	void RemoveSpawneeType( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID );

	Asura_Table_Dialog* CreateSpawneeTypeTable( const Asura_Vector_2& xOffset );
	void CreateSpawneeTypeList();
	void DestroySpawneeTypeList();
	static Asura_TableCell_Dialog* CreateSpawneeTypeCell( Asura_Table_Dialog& xTableDialog,
														  const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
														  const u_int uRow, const u_int uColumn,
														  void* const pxUserData );
	static void AddSpawneeType( int iThis );
	static void RemoveSelectedSpawneeType( int iThis );

	void BuildGSOutNodeString();
	static void ChooseGSOutNode( int iThis );
	static void ResetGSOutNode( int iThis );

	void BuildGSInNodeString();
	static void ChooseGSInNode( int iThis );
	static void ResetGSInNode( int iThis );


	ToolsAVP_Entity_SpawnPoint2();
	ToolsAVP_Entity_SpawnPoint2( const ToolsAVP_Entity_SpawnPoint2& xOther );
	ToolsAVP_Entity_SpawnPoint2& operator= ( const ToolsAVP_Entity_SpawnPoint2& xOther );

private:
	bool m_bRefreshOverlay;

	u_int				m_uSpawneeTypeListSize;
	char**				m_ppszSpawneeTypeList;
	Asura_Table_Dialog* m_pxSpawneeTypeTableDialog;
	Asura_Hash_ID		m_uNewBlueprintTypeHashID;
	Asura_Hash_ID		m_uNewBlueprintHashID;

	Asura_String		m_xGSOutNodeString;
	Asura_Guid			m_uNewGSOutNodeGuid;
	Asura_String		m_xGSInNodeString;
	Asura_Guid			m_uNewGSInNodeGuid;

	static Asura_Hash_ID s_uLastSpawneeTypeBPHashID;
	static Asura_Hashed_List< ToolsAVP_Entity_SpawnPoint2 > s_xList;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLSAVP_ENTITY_SPAWNPOINT2_H__
