///////////////////////////////////////////////////////////////////////////////
//
// File   : ToolsAVP_Entity_StartPoint.cpp
// Author : Anthony Gregson
// Date   : 09/10/07
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "ToolsAVP_Entity_StartPoint.h"

// Asura independent header-files...
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Entity/Asura_Entity.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"

// Tools header-files...
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_ObjectSystem_Param.h"

// AVP includes
#include "Entity/AVP_Messages.h"

// AVP tools includes

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const Asura_Hash_ID uSTART_POINT_PARENT_HASH = 0xA328FE6E;	//Asura_GetHashID("StartPoint");

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

TOOLS_DECLARE_SHARED_PARAM( Asura_Hash_ID,	uTOOLS_SHAREDPARAMHASH_STARTPOINT_BLUEPRINT,		StartPoint_Blueprint )
TOOLS_DECLARE_SHARED_PARAM( Asura_String,   uTOOLS_SHAREDPARAMHASH_STARTPOINT_BLUEPRINT_FUNC,   StartPoint_Blueprint_Func )

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

ToolsAVP_Entity_StartPoint::ToolsAVP_Entity_StartPoint(Asura_Entity* pxEntity) : Tools_Entity_StartPoint(pxEntity)
{
	SetToolsSkinOverrideFromBlueprint( m_pxServerEntity->GetBlueprintTypeHashID(), m_pxServerEntity->GetBlueprintHashID() );
}

bool ToolsAVP_Entity_StartPoint::HandlesThisMessage(int iMessageID)
{
	switch(iMessageID)
	{
		case AVP_MSGID_CHANGE_OWNERSHIP_TO_TEAM1:
		case AVP_MSGID_CHANGE_OWNERSHIP_TO_TEAM2:
		case AVP_MSGID_CHANGE_OWNERSHIP_TO_TEAM3:
		{
			// This entity can receive change ownership messages
			return true;
		}
		default:
		{
			return PARENT::HandlesThisMessage(iMessageID); 
		}
	}
}

void ToolsAVP_Entity_StartPoint::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );

	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	Asura_Dialog* pxDialog;
	pxDialog = new Asura_Function_Dialog( "Blueprint: ", &g_xParam_StartPoint_Blueprint_Func, xOffset, ChooseBlueprint, reinterpret_cast< int >( this ), &Changed );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

bool ToolsAVP_Entity_StartPoint::GetParameter(Asura_Hash_ID uParamID, Tools_Parameter* pxParam) const
{
	switch(uParamID)
	{
		case g_uHash_StartPoint_Blueprint:
			pxParam->SetHash(m_pxServerEntity->GetBlueprintHashID());
			break;

		case g_uHash_StartPoint_Blueprint_Func:
		{
			if ( m_pxServerEntity->GetBlueprintHashID() == ASURA_HASH_ID_UNSET )
			{
				pxParam->SetString( "None" );
			}
			else
			{
				Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( m_pxServerEntity->GetBlueprintTypeHashID(), m_pxServerEntity->GetBlueprintHashID() );
				if ( pxBlueprint )
				{
					pxParam->SetString( pxBlueprint->GetName() );
				}
				else
				{
					pxParam->SetString( "ERROR" );
				}
			}

			break;
		}

		default:
			return PARENT::GetParameter(uParamID,pxParam);
	}

	return true;
}

bool ToolsAVP_Entity_StartPoint::SetParameter(Asura_Hash_ID uParamID, const Tools_Parameter* pxParam)
{
	switch(uParamID)
	{
		case g_uHash_StartPoint_Blueprint:
			m_pxServerEntity->SetBlueprintHashID(pxParam->GetHash());
			break;

		default:
			return PARENT::SetParameter(uParamID,pxParam);
	}

	SetEntityModified();

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Entity_StartPoint::ChooseBlueprint( int iThis )
{
	ToolsAVP_Entity_StartPoint* pxThis = reinterpret_cast< ToolsAVP_Entity_StartPoint* >( iThis );
	if ( !pxThis ) { return; }

	Asura_Hash_ID uBlueprintTypeHashID = pxThis->m_pxServerEntity->GetBlueprintTypeHashID();
	Asura_Hash_ID uBlueprintHashID     = pxThis->m_pxServerEntity->GetBlueprintHashID();
	BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID, 0, NULL, 1, &uSTART_POINT_PARENT_HASH );
	if ( !xBPOverlay.DoModal() ) { return; }

	// It's our job now to iterate through all of the selected tools objects (of the correct type) and set this as the blueprint hash ID.
	for ( LIF< Tools_Object* > xObjectIter( &Tools_Object_System::SelectedObjectList ); !xObjectIter.done(); xObjectIter.next() )
	{
		Tools_Entity_Object* pxObject = Tools_Entity_Object::FindEntity(xObjectIter()->GetGuid());
		if ( !pxObject ) { continue; }
		if (pxObject->Type != TOT_Entity_StartPoint) { continue; }

		pxObject->m_pxServerEntity->SetBlueprintHashID( uBlueprintHashID );
		pxObject->m_pxServerEntity->SetBlueprintTypeHashID( uBlueprintTypeHashID );
	}

	pxThis->SetToolsSkinOverrideFromBlueprint( uBlueprintTypeHashID, uBlueprintHashID );
}


void ToolsAVP_Entity_StartPoint::SetToolsSkinOverrideFromBlueprint( Asura_Hash_ID uBlueprintTypeHashID, Asura_Hash_ID uBlueprintHashID )
{
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uBlueprintTypeHashID, uBlueprintHashID );
	if ( pxBlueprint )
	{
		Asura_Hash_ID uSkinOverride = pxBlueprint->GetParameterAsHash( 0x628E5722, ASURA_HASH_ID_UNSET ); // "ToolsSkin.Override"
		if( uSkinOverride!=ASURA_HASH_ID_UNSET )
		{
			SetSkinAndAnim( uSkinOverride, ASURA_HASH_ID_UNSET );
		}
		else
		{
			// If we don't have any 
			SetMesh( "ToolsShape_CutsceneActor" );
		}
	}
}