///////////////////////////////////////////////////////////////////////////////
//
// File   : ToolsAVP_Message_Editing.h
// Author : Chris Wallace
// Date   : 26-Jan-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLSAVP_MESSAGE_EDITING_H__
#define __TOOLSAVP_MESSAGE_EDITING_H__

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Entity/Tools_Message_Editing.h"
#include "ToolsAVP_ChooseObjects.h"

////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class AVP_Message_ChunkActor;
class AVP_Message_AddObjective;
class AVP_Message_AddPlayerObjective;
class AVP_Message_PlayerObjectiveComplete;
class AVP_Message_AddPlayerPickup;
class AVP_Message_AddSquadMember;
class AVP_Message_RemoveSquadMember;
class AVP_Message_MergeSquads;
class AVP_Message_SplitSquad;
class AVP_Message_RequestPatrol;
class AVP_Message_ChangeMarineSquadSettings;
class AVP_Message_AttackScriptedTarget;
class AVP_Message_DropshipAttackMode;
struct AVP_Msg_SetAITargetPriorities;
struct AVP_Msg_SetAlienSquadBehaviour;
struct AVP_Message_SetPredatorBehaviour;
class AVP_Message_ChangeHumanoidMovementMode;
class AVP_Message_LookAtEntity;
class AVP_Message_SetGS2MoveSpeed;
class AVP_Message_SetHealth;
struct AVP_Msg_SetFaceHuggerSquadBehaviour;
class AVP_Message_PlayerObjectiveSetMarker;
class AVP_Message_PlayerObjectiveSetEntity;
class AVP_Message_ChangeHumanoidKnowledgeSettings;
class AVP_Message_SentryGun_Set_SweepSpeed;
class AVP_Message_SentryGun_Set_VisionRange;
class AVP_Message_SentryGun_Set_HorizontalFOV;
class AVP_Message_SentryGun_Set_VerticalFOV;
class AVP_Message_SentryGun_Set_FiringArc;
class AVP_Message_SentryGun_Set_LethalRange;
class AVP_Message_SentryGun_Toggle_Dry_Fire;
class AVP_Message_Actor_Attachment;
class AVP_Msg_Add_Entity_To_Queen_Target_List;
class AVP_Msg_Remove_Entity_From_Queen_Target_List;
class AVP_Msg_Alien_Queen_Start_Roar_Behaviour;
struct AVP_Msg_CameraShake;
class AVP_Msg_Dismember;
class Tools_PositionEdit_Object;
class AVP_Message_SetAlienQueenBehaviour;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : ToolsAVP_Message_Editing
// Author       : Chris Wallace
// Date         : 26-Jan-2007
////////////////////////////////////////////////////////////////////////////////

class ToolsAVP_Message_Editing : public Tools_Message_Editing
{
	typedef Tools_Message_Editing PARENT;

public:
	ToolsAVP_Message_Editing(void);

	static AVP_Message_SplitSquad* GetSplitSquadMessage(void) {return s_pxSplitSquadMessage;}

	static Asura_Guid GetSplitSquadSentToGuid(void)	{return s_uSplitSquadSentToGuid;}

protected:

	virtual bool GetMessageDescriptionMain( int iMessageID, const Asura_StaticMessage* pxMessage, Asura_String& xOutputDesc );

private:
	virtual	bool	GuidHandlesThisMessage(Asura_Guid uGuid, int iMessageID) const;
	virtual void	InitialiseMessageData(struct Asura_StaticMessage* pxMessage);
	virtual void	AddOverlayControls(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	virtual void	UpdateMain();

	virtual const EventIDAndDescription* GetProjectEventList();
	virtual const char*					 GetEventDescriptionMain(int iEventID);

	void UpdateChangeHumanoidSquadSettingsMessage( void );
	void UpdateShootAtMessage( void );
	void UpdateLookAtMessage( void );
	void UpdateHuggeEntityMessage();
	void UpdateDropshipAttackModeMessage( void );
	void UpdateObjectiveMoveModeMessage( void );
	void UpdateQueenMessages( void );

	void UpdateAddObjectiveMessage();
	void UpdateAddPlayerObjectiveMessage();
	void UpdatePlayerObjectiveCompleteMessage();
	void UpdateSetMarkerLocationMessage();
	void UpdateSetTargetEntityMessage();
	void UpdateSetPlayerPickupMessage();
	void UpdateSetPlayerIntelligentAmountMessage();

	static  void	SelectObjective(void);
	static  void	SelectPlayerObjective(void);
	static  void	SelectCompletedPlayerObjective(void);
	static  void	SelectMarkerLocation(void);
	static  void	SelectTargetEntity(void);
	static  void	SelectPlayerPickup(void);
	static	void	SelectMarineToAdd(void);
	static	void	SelectMarineToRemove(void);
	static	void	SelectMarineSquadToMergeFrom(void);
	static	void	SelectMarineSquadToMergeInto(void);
	static	void	SelectSquadToSplit(void);
	static	void	SelectSquadToAddTo(void);
	static	void	SelectSquadMemberToMove(int iArrayIndex);
	static	void	ClearSplitSquadMember(int iArrayIndex);
	static	void	SelectPatrolGuardZone(void);
	static	void	ClearPatrolGuardZone(void);

	static void		SelectShootAt(void);
	static void		ClearShootAt(void);
	static void		SelectLookAt(void);
	static void		ClearLookAt(void);

	static void		SelectDamager(void);
	static void		ClearDamager(void);
	static void		SetSelectedDamager( Asura_Guid uGuid );

	static void		SelectQueenToAddTo( void );
	static void		SetSelectedQueen_ADD( Asura_Guid uGuid );
	static void		ClearSelectedQueen_ADD( void );

	static void		SelectQueenToRemoveFrom( void );
	static void		SetSelectedQueen_REMOVE( Asura_Guid uGuid );
	static void		ClearSelectedQueen_REMOVE( void );

	static void		SelectTargetForQueenToRoarAt( void );
	static void		SetSelectedTargetOfQueensRoar( Asura_Guid uGuid );
	static void		ClearSelectedTargetForQueensRoar( void );

	static void		SetAlienQueenBehaviour( int iThis );

	static void		SelectSpawneeAlienSquad(int iDataPtr);
	static void		ClearSpawneeAlienSquad(int iDataPtr);
	static void		SetSelectedSpawneeAlienSquad( int iDataPtr, Asura_Guid uGuid );

	static void		SelectHuggedEntity();
	static void		ClearHuggedEntity();


	// Message target display functions
	static const char*	AVP_GetNameFromGuid(Asura_Guid guid);
	static void			AVP_EditDestionationGuid(ListSelection_Overlay& xOverlay);

	static void SetAddObjectiveMessageObjectiveGuid(Asura_Guid uGuid);
	static void SetAddPlayerObjectiveMessageObjectiveGuid(Asura_Guid uGuid);
	static void SetPlayerObjectiveCompleteMessageObjectiveGuid(Asura_Guid uGuid);
	static void SetPlayerObjectiveMessageMarkerGuid(Asura_Guid uGuid);
	static void SetPlayerObjectiveMessageTargetEntityGuid(Asura_Guid uGuid);
	static void SetPlayerPickupMessagePickupGuid(Asura_Guid uGuid);
	static void SetFromSquadGuid_MergeSquadMessage(Asura_Guid uGuid);
	static void SetIntoSquadGuid_MergeSquadMessage(Asura_Guid uGuid);
	static void SetSquadToSplitGuid_SplitSquadMessage(Asura_Guid uGuid);
	static void SetSquadToAddToGuid_SplitSquadMessage(Asura_Guid uGuid);
	static void	SetMemberToMove_SplitSquadMessage(Asura_Guid uGuid);
	static void SetGuardZone_RequestPatrolMessage(Asura_Guid uGuid);
	static void SetSelectedShootAt(Asura_Guid uGuid);
	static void SetSelectedLookAt( Asura_Guid uGuid );

	static void SetSpawnBlueprint( int iData );
	static void ResetSpawnBlueprint( int iData );

	static void UpdateSquadTextDialog(Asura_Guid uSquadGuid);
	static void UpdateSquadMemberTextDialog(Asura_Guid uGuid);
	static bool IsFlagUnchecked(int iFlag) { return ( s_puTokenFlags && ( ( (*s_puTokenFlags) & ( 1 << iFlag ) ) == 0 ) ); }
	static bool IsFlagChecked(int iFlag) { return !IsFlagUnchecked( iFlag ); }

	static void SetAlienSquadBehaviour( int iThis );
	static void SetFacehuggerSquadBehaviour( int iThis );

	static void SetAITargetPriority( int iData );
	static void SetAITargetType( int iData );
	static void ChooseAITargetEntity( int iData );
	static void ChosenAITargetEntity( int iData );
	static void ChooseShakeObject(void);
	static void ChooseShakeBlueprint(int pShake);
	static void OnSelectShakeObject( Tools_Object* pxObject );
	static void SetObjectNameString( Asura_String &xNameString, const Asura_Guid uGuid );


	static void	SelectRestrictedCombatZoneGPV( void );
	static void	ClearRestrictedCombatZoneGPV( void );
	static void SetSelectedRestrictedCombatZoneGPV( Asura_Guid uGuid );

	static void SetModifiedAndUpdateMessageOverlay( int iThis );

	static void SePredatorBehaviour( int iThis );

	static void ConditionedSetModified( int iThis );

	static void ChooseKnowledgeSettingsBlueprint( void );
	static void SetAlienBehviourPrototype( int iThis );

	void AVP_SetModifiedAndUpdateMessageOverlay();

	static void MsgDismember_SetDamagePosition();
	static void MsgDismember_SetRagdollDirection();
	static void MsgDismember_SetGibDirection();


private:

	static AVP_Message_ChunkActor*							s_pxChunkActorMessage;
	static AVP_Message_AddObjective*						s_pxAddObjectiveMessage;
	static AVP_Message_AddPlayerObjective*                  s_pxAddPlayerObjectiveMessage;
	static AVP_Message_PlayerObjectiveComplete*             s_pxPlayerObjectiveCompleteMessage;
	static AVP_Message_PlayerObjectiveSetMarker*            s_pxSetMarkerLocationMessage;
	static AVP_Message_PlayerObjectiveSetEntity*            s_pxSetTargetEntityMessage;
	static AVP_Message_AddPlayerPickup*			            s_pxSetPlayerPickupMessage;
	static AVP_Message_DropshipAttackMode*					s_pxDropshipAttackModeMessage;
	static AVP_Message_AddSquadMember*						s_pxAddSquadMemberMessage;
	static AVP_Message_RemoveSquadMember*					s_pxRemoveSquadMemberMessage;
	static AVP_Message_MergeSquads*							s_pxMergeSquadMessage;
	static AVP_Message_SplitSquad*							s_pxSplitSquadMessage;
	static AVP_Message_RequestPatrol*						s_pxRequestPatrolMessage;
	static AVP_Msg_SetAITargetPriorities*					s_pxSetAITargetPrioritiesData;
	static AVP_Message_ChangeMarineSquadSettings*			s_pxChangeHumanoidSquadSettingsMessage;
	static AVP_Msg_SetAlienSquadBehaviour*					s_pxSetAlienSquadBehaviourMsg;
	static AVP_Message_AttackScriptedTarget*				s_pxAttackScriptedTargetMessage;
	static AVP_Message_SetPredatorBehaviour*				s_pxSetPredatorBehaviourMessage;
	static AVP_Message_ChangeHumanoidMovementMode*			s_pxChangeHumanoidMoveModeMessage;
	static AVP_Message_LookAtEntity*						s_pxLookAtEntityMessage;
	static AVP_Message_SetGS2MoveSpeed*						s_pxSetGS2MoveSpeed;
	static AVP_Message_SetHealth*							s_pxSetHealthMessage;
	static AVP_Msg_SetFaceHuggerSquadBehaviour*				s_pxSetFaceHuggerSquadBehaviourMsg;
	static AVP_Message_ChangeHumanoidKnowledgeSettings*		s_pxSetHumanoidKnowledgeSettingsMsg;
	static AVP_Message_SentryGun_Set_SweepSpeed*			s_pxSetSentryGunSweepSpeed;
	static AVP_Message_SentryGun_Set_VisionRange*			s_pxSetSentryGunVisionRange;
	static AVP_Message_SentryGun_Set_HorizontalFOV*			s_pxSetSentryGunHorizontalFOV;
	static AVP_Message_SentryGun_Set_VerticalFOV*			s_pxSetSentryGunVerticalFOV;
	static AVP_Message_SentryGun_Set_FiringArc*				s_pxSetSentryGunFiringArc;
	static AVP_Message_SentryGun_Set_LethalRange*			s_pxSetSentryGunLethalRange;
	static AVP_Message_SentryGun_Toggle_Dry_Fire*			s_pxSetSentryGunDryFireToggle;
	static AVP_Message_Actor_Attachment*					s_pxActorAttachmentMessage;
	static AVP_Msg_Add_Entity_To_Queen_Target_List*			s_pxAddEntityToAlienQueenTargetList;
	static AVP_Msg_Remove_Entity_From_Queen_Target_List*	s_pxRemoveEntityFromAlienQueenTargetList;
	static AVP_Msg_Alien_Queen_Start_Roar_Behaviour*		s_pxForceAlienQueenRoar;
	static AVP_Message_SetAlienQueenBehaviour*				s_pxSetAlienQueenBehaviourMessage;
	static AVP_Msg_CameraShake*								s_pxCameraShakeMessage;
	static AVP_Msg_Dismember*								s_pxDismemberMessage;

	// Message-dependent static variables & functions
	static Asura_Dialog*	s_pxSquadTextDialog;
	static Asura_Dialog*	s_pxHumanoidMemberTextDialog;
	static u_int*			s_puTokenFlags;
	static u_int			s_uSquadMemberIndex;
	static Asura_Guid		s_uSplitSquadSentToGuid;
	static bool				s_bPatrolTypeChanged;
	static bool				s_bPatrolFullSquad;
	static bool				s_bShootAtChanged;
	static bool				s_bLookAtChanged;

	static bool				s_bPlayersSquadTypeChanged;
	static bool				s_bPlayersAILedSquadTypeChanged;
	static bool				s_bSquadCombatZoneChanged;	
	static bool				s_bIsPlayersSquad;
	static bool				s_bIsAILedPlayersSquad;
	static bool				s_bIsRestrictedToCombatZone;
	static bool				s_bIgnoreAlertState;
	static bool				s_bIgnoreAlertStateChanged;
	static bool				s_bSquadCombatStateChanged;
	static bool				s_bDropshipModeChanged;
	static bool				s_bObjectiveMoveModeChanged;
	static Asura_Guid		s_uGuidToIgnoreForSelectionInMessage;
	static bool				s_bLookAtPlayer;
	static bool				s_bGS2MoveModeChanged;
	static bool				s_bAlienSquadBehaviourChanged;
	static bool				s_bHealthSettingTypeChanged;
	static bool				s_bTakeAsDamageFromEntity;
	static bool				s_bTreatAsDamage;

	static bool				s_bLookAtMarkerStringChanged;
	static Asura_String		s_xLookAtMarkerString;

	static Asura_String		s_xObjectNameString;

	static int				s_iDamageTypeValue;
	static int				s_iHumanoidTypeValue;

	static bool				s_bAddObjectiveChanged;
	static bool				s_bAddPlayerObjectiveChanged;
	static bool				s_bPlayerObjectiveCompleteChanged;
	static bool				s_bSetMarkerLocationChanged;
	static bool				s_bSetTargetEntityChanged;
	static bool				s_bSetPlayerPickupChanged;
	static bool				s_bIntelligentAmountChanged;
	static bool				s_bHumanoidKnowledgeSettingTypeChanged;
	static bool				s_bQueenTargetListChanged;
	static Asura_Guid*		s_puEntityGuid;
	static bool				s_bHuggedEntityChanged;
};

class ToolsAVP_SplitSquad_ChooseObject : public ToolsAVP_ChooseObject_ByType
{
	typedef ToolsAVP_ChooseObject_ByType PARENT;

public:
	ToolsAVP_SplitSquad_ChooseObject( const int iType, void( *pfnOnChosen )( Asura_Guid ), Asura_Guid uSquadGuid, bool* pbChanged = NULL )
	: PARENT(iType, pfnOnChosen, pbChanged)
	, m_uSquadGuid(uSquadGuid)
	{
	}

	virtual bool CanBeSelected(Tools_Object* pxObject);

private:
	Asura_Guid m_uSquadGuid;
};

class ToolsAVP_SelectDamager_ChooseObject : public ToolsAVP_ChooseObject_ByType
{
	typedef ToolsAVP_ChooseObject_ByType PARENT;

public:
	ToolsAVP_SelectDamager_ChooseObject( void( *pfnOnChosen )( Asura_Guid ), bool* pbChanged = NULL, const int iType = 0  )
		: PARENT(iType, pfnOnChosen, pbChanged)
	{
	}

	virtual bool CanBeSelected(Tools_Object* pxObject);

private:
	Asura_Guid m_uSquadGuid;
};

#endif
