//
// ToolsAVP_Entity_InteractionTrigger.cpp
//
// Created by Chris Wallace
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

// core
#include "Core/Asura.h"
#include "Entity/Triggers/ToolsAVP_Entity_InteractionTrigger.h"

// asura
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

#include "Entity/Triggers/AVP_ServerEntity_InteractionTrigger.h"
#include "Entity/Triggers/AVP_ClientEntity_InteractionTrigger.h"

#include "Objects/Tools_Object_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"

//////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------------------**
** ToolsAVP_Entity_InteractionTrigger **
**-------------------------------------------*/

ToolsAVP_Entity_InteractionTrigger* ToolsAVP_Entity_InteractionTrigger::s_pxThis = 0;

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

ToolsAVP_Entity_InteractionTrigger::ToolsAVP_Entity_InteractionTrigger(Asura_Entity* pxEntity)
: Tools_Entity_AdvancedVolumeTrigger(pxEntity)
, m_pxChooseObject( 0 )
{
}

ToolsAVP_Entity_InteractionTrigger::~ToolsAVP_Entity_InteractionTrigger()
{
	if ( m_pxChooseObject )
	{
		Tools_Object_System::ChooseObject_Finish();
		Asura_Assert( !m_pxChooseObject, "" );
	}
}

void ToolsAVP_Entity_InteractionTrigger::Project_Create()
{
	ToolsAVP_Entity_InteractionTrigger::Create();
}

void ToolsAVP_Entity_InteractionTrigger::Create()
{
	Asura_Assert( false, "These entities are deprecated in favour of Interaction Objects and are likely to be removed soon, if you've got a problem with this then report this assert as a code bug on mantis." ); // WGP

	//get a name , and allocate the tools data
	Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();
	Tools_Entity_Data* pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Interaction Trigger","Interaction Trigger",uGuid);
	
	if(!pxToolsData) return;

	//create the asura data
	AVP_ServerEntity_InteractionTrigger* pxAsuraEntity = new AVP_ServerEntity_InteractionTrigger(uGuid);
	pxAsuraEntity->ToolsData = pxToolsData;

	//create the tools object
	Tools_Object* pxObject = new ToolsAVP_Entity_InteractionTrigger(pxAsuraEntity);
	Tools_Object_System::SelectObject(pxObject);

	Tools_SaveAsura::SetEntityModified();
}

void ToolsAVP_Entity_InteractionTrigger::OnSelection()
{
    Tools_Entity_AdvancedVolumeTrigger::OnSelection();
    s_pxThis = this;
}

void ToolsAVP_Entity_InteractionTrigger::OnDeselection()
{
    Tools_Entity_AdvancedVolumeTrigger::OnDeselection();
    s_pxThis = 0;
}

Tools_Object* ToolsAVP_Entity_InteractionTrigger::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Entity_InteractionTrigger( pxEntity );
}

void ToolsAVP_Entity_InteractionTrigger::AddOverlayControls(class Asura_Overlay* pxNewOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_AdvancedVolumeTrigger::AddOverlayControls(pxNewOverlay, xOffset);

	Asura_Assert(ThisEntity,"");
	Tools_Entity_AdvancedVolumeTrigger* pxToolsEntity = static_cast<Tools_Entity_AdvancedVolumeTrigger*>(ThisEntity);
	AVP_ServerEntity_InteractionTrigger* pxAsuraEntity = static_cast<AVP_ServerEntity_InteractionTrigger*>(pxToolsEntity->Entity);
	Asura_Assert(AVP_ServerEntity_InteractionTrigger::NUM_AVP_INTERACTIONTRIGGEREDBY_TYPES<=32, "Error, maximum number of interaction trigger lists is 32");

    Asura_Dialog* pxNewDialog;
	for (int i=0;i<AVP_ServerEntity_InteractionTrigger::NUM_AVP_INTERACTIONTRIGGEREDBY_TYPES;i++)
	{
		pxNewDialog = new Asura_FlagTickbox_Dialog(AVP_ServerEntity_InteractionTrigger::s_aszAVPVolumeTypes[i],xOffset,&pxAsuraEntity->m_iListFlags, (1 << i),&Changed);
		pxNewOverlay->AddDialog(pxNewDialog);
		xOffset.y += 1.5f;
	}


}

void ToolsAVP_Entity_InteractionTrigger::SetSpecificGuid()
{
}

void ToolsAVP_Entity_InteractionTrigger::GoToObjectWithGuid()
{
}

void ToolsAVP_Entity_InteractionTrigger::SetSpecificGuid( Tools_Object* pxObject )
{
}

