//
// ToolsAVP_Entity_WaypointBlockerVolume.cpp
//
// Created by Anthony Gregson - 11/12/07
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

// core
#include "Core/Asura.h"

// header
#include "Entity/Triggers/ToolsAVP_Entity_WaypointBlockerVolume.h"

// asura
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

// avp
#include "Entity/Triggers/AVP_ServerEntity_WaypointBlockerVolume.h"

// tools
#include "Entity/Tools_Entity_Data.h"
#include "Objects/Tools_Object_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"

// avp tools
#include "Objects/ToolsAVP_Object_Type.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Entity_WaypointBlockerVolume
//
//////////////////////////////////////////////////////////////////////////////

ToolsAVP_Entity_WaypointBlockerVolume::ToolsAVP_Entity_WaypointBlockerVolume( Asura_Entity* pxEntity )
: TOOLS_PARENT(pxEntity)
, m_bChanged(false)
, m_bNewHuman(false)
, m_bNewAlien(false)
, m_bNewPredator(false)
{
	Type = TOT_Entity_AVPWaypointBlockerVolume;
}

void ToolsAVP_Entity_WaypointBlockerVolume::OnSelection()
{
	AVP_ServerEntity_WaypointBlockerVolume* pxServerEntity = static_cast<AVP_ServerEntity_WaypointBlockerVolume*>(m_pxServerEntity);

	m_bNewHuman = pxServerEntity->m_bBlockHuman;
	m_bNewAlien = pxServerEntity->m_bBlockAlien;
	m_bNewPredator = pxServerEntity->m_bBlockPredator;

	TOOLS_PARENT::OnSelection();
}

void ToolsAVP_Entity_WaypointBlockerVolume::OnDeselection()
{
	TOOLS_PARENT::OnDeselection();
}

void ToolsAVP_Entity_WaypointBlockerVolume::AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	TOOLS_PARENT::AddOverlayControls( pxNewOverlay, xOffset );

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Dialog( "Block species:", xOffset);
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Tickbox_Dialog( "Human", xOffset, &m_bNewHuman );
	static_cast<Asura_Tickbox_Dialog*>(pxDialog)->SetClickedFunction(UpdateClick, reinterpret_cast<int>(this));
	pxDialog->SetHelpText( "This Waypoint blocker will block humans and creatures that use the human routing." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Alien", xOffset, &m_bNewAlien );
	static_cast<Asura_Tickbox_Dialog*>(pxDialog)->SetClickedFunction(UpdateClick, reinterpret_cast<int>(this));
	pxDialog->SetHelpText( "This Waypoint blocker will block aliens, facehuggers and other creatures that use alien routing." );
	pxNewOverlay->AddDialog( pxDialog );

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog( "Predator", xOffset, &m_bNewPredator );
	static_cast<Asura_Tickbox_Dialog*>(pxDialog)->SetClickedFunction(UpdateClick, reinterpret_cast<int>(this));
	pxDialog->SetHelpText( "This Waypoint blocker will block predator, predalien and other creatures that use predator routing." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

ToolsAVP_Entity_WaypointBlockerVolume* ToolsAVP_Entity_WaypointBlockerVolume::Create( Asura_Entity* pxEntity )
{
	return new ToolsAVP_Entity_WaypointBlockerVolume( pxEntity );
}

void ToolsAVP_Entity_WaypointBlockerVolume::Create()
{
	ToolsAVP_Entity_WaypointBlockerVolume* pxToolsObj = CreateMain();

	if ( pxToolsObj )
	{    
		Tools_Object_System::SelectObject( pxToolsObj );
	}
}

void ToolsAVP_Entity_WaypointBlockerVolume::UpdateClick(int iThis)
{
	ToolsAVP_Entity_WaypointBlockerVolume* pxThis = reinterpret_cast<ToolsAVP_Entity_WaypointBlockerVolume*>(iThis);

	AVP_ServerEntity_WaypointBlockerVolume* pxServerEntity = static_cast<AVP_ServerEntity_WaypointBlockerVolume*>(pxThis->m_pxServerEntity);

	bool bIsBlocking = pxServerEntity->m_bBlockedLinks;

	if (bIsBlocking)
	{
		pxServerEntity->BlockLinks(false);
	}

	pxServerEntity->m_bBlockHuman = pxThis->m_bNewHuman;
	pxServerEntity->m_bBlockAlien = pxThis->m_bNewAlien;
	pxServerEntity->m_bBlockPredator = pxThis->m_bNewPredator;

	if (bIsBlocking)
	{
		pxServerEntity->BlockLinks(true);
	}

	pxThis->SetEntityModified();
}

ToolsAVP_Entity_WaypointBlockerVolume* ToolsAVP_Entity_WaypointBlockerVolume::CreateMain()
{
	const Asura_Guid uGUID = Asura_Entity_System::GetCreatedByToolsGuid();

	Tools_Entity_Data*                      pxToolsData = NULL;
	AVP_ServerEntity_WaypointBlockerVolume* pxEntity    = NULL;
	ToolsAVP_Entity_WaypointBlockerVolume*  pxToolsObj  = NULL;

	pxToolsData = CreateToolsDataWithDefaultName( "AVP Waypoint Blocker Volume", "AVP Waypoint Blocker Volume", uGUID );
	if( !pxToolsData )
	{ 
		return NULL;
	}

	pxEntity = new AVP_ServerEntity_WaypointBlockerVolume( uGUID );
	pxEntity->ToolsData = pxToolsData;
	if( !pxEntity )
	{
		delete pxToolsData;
		return NULL;
	}

	pxToolsObj = new ToolsAVP_Entity_WaypointBlockerVolume( pxEntity );
	if( !pxToolsObj )
	{
		delete pxToolsData;
		pxEntity->PreDestructionShutdown();
		delete pxEntity;
		return NULL;
	}

	Tools_SaveAsura::SetEntityModified();

	return pxToolsObj;
}

bool ToolsAVP_Entity_WaypointBlockerVolume::Verify( const bool bAutomatic )
{
	bool bGood = TOOLS_PARENT::Verify( bAutomatic );

	AVP_ServerEntity_WaypointBlockerVolume* pxServerEntity = static_cast<AVP_ServerEntity_WaypointBlockerVolume*>(m_pxServerEntity);
	if (!(pxServerEntity->m_bBlockHuman || pxServerEntity->m_bBlockAlien || pxServerEntity->m_bBlockPredator ))
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Waypoint Blocker Volume (AVP)", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
			"Waypoint blocker volume %d has no species selected.", GetGuid() );
		bGood = false;
	}

	return bGood;
}
