//////////////////////////////////////////////////////////////////////////////
// 
// FILE : ToolsAVP_Entity_Vent.h
//
// AUTHORS: Will Plano
//
// Copyright 2008 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef _TOOLSAVP_ENTITY_VENT_H_
#define _TOOLSAVP_ENTITY_VENT_H_

#include "Network/Asura_Guid.h"
#include "Entity/Tools_Entity_Object.h"
#include "Objects/ToolsAVP_Object_Type.h"

class AVP_ServerEntity_Vent;
class Asura_ServerNode_GS2Node;

class ToolsAVP_Entity_Vent : public Tools_Entity_Object
{
public:
	static void				Create();
	static Tools_Object*	Create( Asura_Entity* pxEntity );

	virtual bool ObjectIsInLOS(const Asura_Vector_3& xLOSOrigin, const Asura_Vector_3& xLOSDirection, float& fNearest);
	virtual void UpdatePositionInAsuraData(bool bUpdateLinkedObjects);

	virtual bool			CanBeCopied() const { return ( Type == TOT_Entity_Vent ); }
	virtual void			Copy( const Tools_Object* pxObject );
	virtual Tools_Object*	CreateObjectOfThisType() const;

	virtual void OnSelection();
	virtual void OnReselection();
	virtual void OnDeselection();

	virtual bool GetParameter(Asura_Hash_ID uParamID, class Tools_Parameter* pxParam) const;
	virtual bool SetParameter(Asura_Hash_ID uParamID, const class Tools_Parameter* pxParam);

	virtual void Update();
	virtual void UpdateWhileSelected();
	virtual void AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset );
	virtual const char* GetTypeDescription() { return "Vent"; }
	virtual void AddFilesToExportList( List<Asura_String>& xNameList ) const;

	virtual void Render();

	static ToolsAVP_Entity_Vent* FindEntity( const Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }
	static const Asura_Hashed_List<ToolsAVP_Entity_Vent>* GetList(void)	{ return &s_xList; }

protected:
	ToolsAVP_Entity_Vent(AVP_ServerEntity_Vent* pxEntity);
	virtual ~ToolsAVP_Entity_Vent();

	virtual bool Verify( const bool bAutomatic );

private:
	typedef Tools_Entity_Object PARENT;

	void UpdateVentNodeList();
	void UpdateVentNodeListStrings();

	void AddVentNode( const Asura_Guid uNode );
	void RemoveVentNode( const Asura_Guid uNode );

	// widget callback functions
	static void AddExitPoint();
	static void RemoveExitPoint();
	static void EditExitPoint();
	static void SetUndamagedSkin();
	static void SetDamagedSkin();
	static void SetDamageAnim();
	static void AddVentNode();
	static void RemoveVentNode();
	static void EditVentNode();
	static void FocusVentNode();
	static void VerifyVent();

	AVP_ServerEntity_Vent* m_pxVentEntity; // cached pointer to entity

	u_int m_uSelectedExitPoint;
	Asura_Mesh* m_pxExitPointMesh;
	u_int m_uPositionEditObjectID;
	int m_iLOSHitExitPoint; // Stored since LOSTest isn't const and it allows us to do some useful things after selection.

	static u_int					s_uObjectCount;
	static char**					s_pszVentNode;
	static int						s_iSelectedVentNode;
	static u_int					s_uNumVentNodes;
	static bool						s_bRefreshOverlay;
	static bool						s_bNewVentNode;
	static Asura_Guid				s_uNewVentNode;
	static ToolsAVP_Entity_Vent*	s_pxSelectedVent;

	static bool s_bMoveExitPointsWithEntity;

	static Asura_Hashed_List<ToolsAVP_Entity_Vent> s_xList;

	ToolsAVP_Entity_Vent	(void);
	ToolsAVP_Entity_Vent	(const ToolsAVP_Entity_Vent& xOther);
	ToolsAVP_Entity_Vent&	operator=	(const ToolsAVP_Entity_Vent& xOther);
};

#endif //_TOOLSAVP_ENTITY_VENT_H_
