//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Level_Export.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Load&Save/ToolsAVP_Level_Export.h"

#include "Core/Asura_LocalisedText.h"
#include "Core/Asura_MaterialResponse_System.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"

// Game independent files...
#include "Actor/Vertebrates/Predator/AVP_Predator_Description.h"
#include "Actor/Vertebrates/Predator/AVP_Predator_PlayerAnimStateData.h"
#include "Actor/Vertebrates/Human/AVP_Human_Description.h"
#include "Actor/Vertebrates/Human/AVP_Human_PlayerAnimStateData.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_Description.h"
#include "Actor/Vertebrates/Alien/AVP_Alien_PlayerAnimStateData.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "ChunkSystem/AVP_Chunk_Alien_PlayerAnimStateData.h"
#include "ChunkSystem/AVP_Chunk_Human_PlayerAnimStateData.h"
#include "ChunkSystem/AVP_Chunk_Predator_PlayerAnimStateData.h"
#include "ChunkSystem/AVP_Chunk_SurfaceOverlay.h"
#include "ChunkSystem/AVP_Chunk_WeaponData.h"
#include "Core/AVP_Game_Core.h"
#include "Core/AVP_Settings.h"
#include "Core/AVP_SurfaceEffects.h"
#include "Core/AVP_Waypoint_Manager.h"
#include "LoadAndSave/AVP_WeaponData_System.h"
#include "Network/AVP_Client.h"

// Tools files...
#include "Tools_Overlays.h"
#include "Blueprint/Tools_Mode_Blueprint.h"
#include "Load&Save/Tools_BatchProcessing.h"
#include "Particles/Tools_Mode_PFX.h"

// ToolsAVP files...
#include "Load&Save/ToolsAVP_SaveAsura.h"
#include "Waypointing/ToolsAVP_Mode_Waypointing.h"
#include "ToolsAVP_Overlays.h"

static bool s_bSPCharOn = true;
static bool s_bMPCharOn = true;

static bool s_bSPWeaponOn = true;
static bool s_bMPWeaponOn = true;

// Static variables
ToolsAVP_Level_Export::ExportData ToolsAVP_Level_Export::s_axExportData[AVP_Game_Core::NUM_CHARACTERTYPES]	=
{
	{"Alien",		uAVP_BLUEPRINT_NAME_ALIENSINGLEPLAYER,		AVP_Settings::AVP_SPECIES_ALIEN,	true, true},	//SINGLE_PLAYER_ALIEN
	{"Marine",		uAVP_BLUEPRINT_NAME_MARINESINGLEPLAYER,		AVP_Settings::AVP_SPECIES_HUMAN,	true, true},	//SINGLE_PLAYER_MARINE
	{"Predator",	uAVP_BLUEPRINT_NAME_PREDATORSINGLEPLAYER,	AVP_Settings::AVP_SPECIES_PREDATOR,	true, true},	//SINGLE_PLAYER_PREDATOR

	{"Alien",		uAVP_BLUEPRINT_NAME_ALIENMULTIPLAYER,		AVP_Settings::AVP_SPECIES_ALIEN,	true, true},	//MULTIPLAYER_ALIEN
	{"Marine",		uAVP_BLUEPRINT_NAME_MARINEMULTIPLAYER,		AVP_Settings::AVP_SPECIES_HUMAN,	true, true},	//MULTIPLAYER_MARINE
	{"Predator",	uAVP_BLUEPRINT_NAME_PREDATORMULTIPLAYER,	AVP_Settings::AVP_SPECIES_PREDATOR,	true, true},	//MULTIPLAYER_PREDATOR
};

u_int			ToolsAVP_Level_Export::s_uNumMaterialResponseOverridesForExport	= 0;
Asura_Hash_ID*	ToolsAVP_Level_Export::s_pxMaterialResponseOverridesForExport	= NULL;

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

//
// ToolsAVP_Level_Export
//

ToolsAVP_Level_Export::ToolsAVP_Level_Export( void )
{
}

void ToolsAVP_Level_Export::AddInGameFilesToList( List<Asura_String>* pxList, const Asura_String& xEnvFilename )
{
    // General stuff - files required by entities, etc.
	Tools_Level_Export::AddInGameFilesToList( pxList, xEnvFilename );

	if(IsDoingFakeLevelExport())
	{
		AVP_WeaponData_System::InitialiseFromBlueprints();

		Asura_String xLevelName;

		// Set the current level name to that loaded into tools
		AVP_Settings::SetLevelName(xEnvFilename.GetString());

		xEnvFilename.SplitFilename(NULL,&xLevelName,NULL);
		AVP_Settings::RefreshBlueprintData(xLevelName.GetString());
		AVP_Settings::Species eSpecies	= AVP_Settings::GetPlayerSpecies();
		switch(eSpecies)
		{
			case AVP_Settings::AVP_SPECIES_PREDATOR:
			{
				AVP_Predator_Description::AddFilesToExport(true, uAVP_BLUEPRINT_NAME_PREDATORSINGLEPLAYER, *pxList);
				
				const u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( AVP_Game_Core::SINGLE_PLAYER_PREDATOR );
				for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons; ++uWeaponItr )
				{
					const char* szWeaponName = AVP_WeaponData_System::GetWeaponNameFromIndex( uWeaponItr, AVP_Game_Core::SINGLE_PLAYER_PREDATOR );
					const Asura_Hash_ID uWeaponHash = Asura_GetHashID( szWeaponName );
					Tools_Mode_Blueprint::AddBlueprintFilesToExportList( m_xAsuraFileList, uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uWeaponHash, false );
					AVP_Predator_PlayerAnimStateData::GetSingleton().InitialiseWeaponData( uAVP_BLUEPRINT_TYPE_ANIMSTATE_PREDATOR, uWeaponHash );
				}

				break;
			}

			case AVP_Settings::AVP_SPECIES_ALIEN:
			{
				AVP_Alien_Description::AddFilesToExport(true, uAVP_BLUEPRINT_NAME_ALIENSINGLEPLAYER, *pxList);

				const u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( AVP_Game_Core::SINGLE_PLAYER_ALIEN );
				for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons; ++uWeaponItr )
				{
					const char* szWeaponName = AVP_WeaponData_System::GetWeaponNameFromIndex( uWeaponItr, AVP_Game_Core::SINGLE_PLAYER_ALIEN );
					const Asura_Hash_ID uWeaponHash = Asura_GetHashID( szWeaponName );
					Tools_Mode_Blueprint::AddBlueprintFilesToExportList( m_xAsuraFileList, uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uWeaponHash, false );
					AVP_Alien_PlayerAnimStateData::GetSingleton().Initialise( uAVP_BLUEPRINT_TYPE_ANIMSTATE_ALIEN, false );
				}

				break;
			}

			case AVP_Settings::AVP_SPECIES_HUMAN:
			{
				AVP_Human_Description::AddMarineFilesToExport(true, uAVP_BLUEPRINT_NAME_MARINESINGLEPLAYER, *pxList);

				const u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( AVP_Game_Core::SINGLE_PLAYER_MARINE );
				for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons; ++uWeaponItr )
				{
					const char* szWeaponName = AVP_WeaponData_System::GetWeaponNameFromIndex( uWeaponItr, AVP_Game_Core::SINGLE_PLAYER_MARINE );
					const Asura_Hash_ID uWeaponHash = Asura_GetHashID( szWeaponName );
	
					Asura_Blueprint* pxWeaponBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uWeaponHash );
					if ( pxWeaponBP )
					{
						Tools_Mode_Blueprint::AddBlueprintFilesToExport( m_xAsuraFileList, pxWeaponBP, false );
						AVP_Human_PlayerAnimStateData::GetSingleton().InitialiseWeaponData( uAVP_BLUEPRINT_TYPE_ANIMSTATE_MARINE, uWeaponHash );
					}
				}

				break;
			}

			default:
			{
				// No character type set yet, load in the marine as a default
				Asura_Assert(false, "Unexepected character for export - defaulting to Marine.");
				AVP_Human_Description::AddMarineFilesToExport(true, uAVP_BLUEPRINT_NAME_MARINESINGLEPLAYER, *pxList);
				eSpecies	= AVP_Settings::AVP_SPECIES_HUMAN;
				break;
			}
		}		// End of switch.

		// Add all the pickups to the tools fake level export list
		Tools_Mode_Blueprint::AddBlueprintFilesToExportList( m_xAsuraFileList, uAVP_BLUEPRINT_TYPE_PICKUPS );

		AVP_Settings::SetPlayerSpecies(eSpecies);
		AVP_Game_Core::SetRespawnSpecies(eSpecies);
	}
}

bool ToolsAVP_Level_Export::IsPlatformAllowed( EXPORT_TYPE ePlatform ) const
{
	switch( ePlatform )
	{
		case Export_PC:
		case Export_PS3:
		case Export_Xbox360:
		{
			return true;
		}

		default:
		{
			return false;
		}
	}
}

void ToolsAVP_Level_Export::AddCharacterExportOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	AddSharedExportControls(pxOverlay, xOffset, AddCharacterSelectionControls, ExportAVPCharacters);
}

void ToolsAVP_Level_Export::AddWeaponExportOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	AddSharedExportControls(pxOverlay, xOffset, AddWeaponSelectionControls, ExportAVPWeapons);
}

void ToolsAVP_Level_Export::AddSharedExportControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, void (*pfAddSelectionControls)( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset ), void(*pfExportFunction)() )
{
	Asura_Tabs_Dialog*	pxTabsDialog	= new Asura_Tabs_Dialog(xOffset, NULL, 0);
	Asura_Dialog*		pxDialog;

	//Set up the Tabs...
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y	+= Asura_Dialog::LargeSpacing();

	pxOverlay->SetCurrentParentDialog(pxTabsDialog);

	float	fTabsStartY	= xOffset.y;
	float	fTabsEndY	= xOffset.y;

	//Main tab
	pxTabsDialog->AddTab("Main");
	{
		(*pfAddSelectionControls)( pxOverlay, xOffset );

		// Add the platform buttons...
		pxDialog = new Asura_Dialog( "Select Platforms", xOffset );
		pxOverlay->AddDialog(pxDialog);
		xOffset.y	+= Asura_Dialog::DefaultSpacing();

		AddPlatformExportTickboxes(pxOverlay, xOffset);
	}

	fTabsEndY	= Asura_Maths::Max(fTabsEndY, xOffset.y);

	//Advanced Tab... contains all the other options
	pxTabsDialog->AddTab("Advanced");
	{
		xOffset.y = fTabsStartY;

		AddAdvanceOptionsOverlayControls(pxOverlay,xOffset);
	}
	fTabsEndY	= Asura_Maths::Max(fTabsEndY, xOffset.y);

	//Language tab
	pxTabsDialog->AddTab("Language");
	{
		xOffset.y = fTabsStartY;

		AddLanguageOverlayControls(pxOverlay,xOffset);
	}
	fTabsEndY	= Asura_Maths::Max(fTabsEndY, xOffset.y);

	// End of tabs...
	xOffset.y	 = fTabsEndY;
	pxOverlay->SetCurrentParentDialog(NULL);
	pxTabsDialog->SelectTab(0);

	// Add common dialogues...
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
	pxDialog	 = new Asura_Function_Dialog("Export",xOffset,pfExportFunction);
	pxDialog->SetIsEnabledFunction(IsEnabled_ExportLevel);
	pxDialog->SetHelpText("Run the export.");
	pxOverlay->AddDialog(pxDialog);

	pxOverlay->m_uID = TOOLS_OVERLAY_CHAREXPORT;
}

void ToolsAVP_Level_Export::AddCharacterSelectionControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	// Generate the single-player tick-boxes...
	Asura_Dialog* pxDialog = new Asura_Function_Dialog ("All Single-player Characters", xOffset, ToggleSPCharacters);
	pxDialog->SetHelpText("Toggles selection of all single player characters.");
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
	pxOverlay->AddDialog(pxDialog);

	for(u_int u = AVP_Game_Core::FIRST_SINGLE_PLAYER_CHARACTER; u <= AVP_Game_Core::LAST_SINGLE_PLAYER_CHARACTER; u++)
	{
		ExportData&	xExportData	= s_axExportData[u];

		pxDialog	 = new Asura_Tickbox_Dialog( xExportData.m_szName, xOffset, &xExportData.m_bExport );
		xOffset.y	+= Asura_Dialog::DefaultSpacing();
		pxDialog->SetHelpText("Exports this single-player character.");
		pxOverlay->AddDialog(pxDialog);
	}

	// Add spacing...
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
	pxDialog	 = new Asura_Divider_Dialog (xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	// Generate the single-player tick-boxes...
	pxDialog	 = new Asura_Function_Dialog ("All Multiplayer Characters", xOffset, ToggleMPCharacters);
	pxDialog->SetHelpText("Toggles selection of all multiplayer characters.");
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
	pxOverlay->AddDialog(pxDialog);

	for(u_int u = AVP_Game_Core::FIRST_MULTIPLAYER_CHARACTER; u <= AVP_Game_Core::LAST_MULTIPLAYER_CHARACTER; u++)
	{
		ExportData&	xExportData	= s_axExportData[u];

		pxDialog	 = new Asura_Tickbox_Dialog( xExportData.m_szName, xOffset, &xExportData.m_bExport );
		xOffset.y	+= Asura_Dialog::DefaultSpacing();
		pxDialog->SetHelpText("Exports this multiplayer character.");
		pxOverlay->AddDialog(pxDialog);
	}
	// Add spacing...
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
	pxDialog	 = new Asura_Divider_Dialog (xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
}

void ToolsAVP_Level_Export::AddWeaponSelectionControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	// Generate the single-player tick-boxes...
	AVP_WeaponData_System::InitialiseFromBlueprints();
	AVP_WeaponData_System::ResetShouldBeLoadedData( true );

	Asura_Dialog* pxDialog = new Asura_Function_Dialog ("All Single-player Weapons", xOffset, ToggleSPWeapons);
	pxDialog->SetHelpText("Toggles selection of all single player weapons.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
	float fStartYOffset = xOffset.y;
	float fNextYOffset = xOffset.y;

	for(u_int uCharItr = AVP_Game_Core::FIRST_SINGLE_PLAYER_CHARACTER; uCharItr <= AVP_Game_Core::LAST_SINGLE_PLAYER_CHARACTER; uCharItr++)
	{
		ExportData&	xExportData	= s_axExportData[uCharItr];

		xOffset.x = 1.0f + ( uCharItr * 20.0f );
		xOffset.y = fStartYOffset;
		Asura_String xDialogString( "All SP_" );
		xDialogString += xExportData.m_szName;
		pxDialog = new Asura_Function_Dialog ( xDialogString.GetString(), xOffset, ToggleWeaponsForCharacter, static_cast< int >( uCharItr ) );
		pxDialog->SetHelpText("Toggles selection of all single player weapons for this species.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y	+= Asura_Dialog::DefaultSpacing();

		AVP_Game_Core::CharacterType eChar = static_cast< AVP_Game_Core::CharacterType >( uCharItr );
		u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( eChar );
		for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons; ++uWeaponItr )
		{
			const char* szWeaponName = AVP_WeaponData_System::GetWeaponNameFromIndex( uWeaponItr, eChar );
			pxDialog	 = new Asura_Tickbox_Dialog( szWeaponName, xOffset, AVP_WeaponData_System::GetShouldLoadWeaponBoolPtr( uWeaponItr, eChar ) );
			xOffset.y	+= Asura_Dialog::DefaultSpacing();
			pxDialog->SetHelpText("Exports this weapon.");
			pxOverlay->AddDialog(pxDialog);
		}
		if ( xOffset.y > fNextYOffset )
		{
			fNextYOffset = xOffset.y;
		}
	}
	xOffset.x = 1.0f;
	xOffset.y = fNextYOffset;

	// Add spacing...
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
	pxDialog	 = new Asura_Divider_Dialog (xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	// Generate the single-player tick-boxes...
	pxDialog	 = new Asura_Function_Dialog ("All Multiplayer Weapons", xOffset, ToggleMPWeapons);
	pxDialog->SetHelpText("Toggles selection of all multiplayer weapons.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
	fStartYOffset = xOffset.y;

	for(u_int uCharItr = AVP_Game_Core::FIRST_MULTIPLAYER_CHARACTER; uCharItr <= AVP_Game_Core::LAST_MULTIPLAYER_CHARACTER; uCharItr++)
	{
		ExportData&	xExportData	= s_axExportData[uCharItr];

		xOffset.x = 1.0f + ( ( uCharItr - AVP_Game_Core::FIRST_MULTIPLAYER_CHARACTER ) * 20.0f );
		xOffset.y = fStartYOffset;
		Asura_String xDialogString( "All MP_" );
		xDialogString += xExportData.m_szName;
		pxDialog = new Asura_Function_Dialog ( xDialogString.GetString(), xOffset, ToggleWeaponsForCharacter, static_cast< int >( uCharItr ) );
		pxDialog->SetHelpText("Toggles selection of all multiplayer weapons for this species.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y	+= Asura_Dialog::DefaultSpacing();

		AVP_Game_Core::CharacterType eChar = static_cast< AVP_Game_Core::CharacterType >( uCharItr );
		u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( eChar );
		for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons; ++uWeaponItr )
		{
			const char* szWeaponName = AVP_WeaponData_System::GetWeaponNameFromIndex( uWeaponItr, eChar );
			pxDialog	 = new Asura_Tickbox_Dialog( szWeaponName, xOffset, AVP_WeaponData_System::GetShouldLoadWeaponBoolPtr( uWeaponItr, eChar ) );
			xOffset.y	+= Asura_Dialog::DefaultSpacing();
			pxDialog->SetHelpText("Exports this weapon.");
			pxOverlay->AddDialog(pxDialog);
		}

		if ( xOffset.y > fNextYOffset )
		{
			fNextYOffset = xOffset.y;
		}
	}
	xOffset.x = 1.0f;
	xOffset.y = fNextYOffset;

	// Add spacing...
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
	pxDialog	 = new Asura_Divider_Dialog (xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y	+= Asura_Dialog::DefaultSpacing();
}

void ToolsAVP_Level_Export::ExportAVPCharacters()
{
	int iLastSelectedPlatform = -1;
	for(int i=0;i<ExportType_NumPlatforms;i++)
	{
		s_eExportType = (EXPORT_TYPE)i;
		if(This->IsPlatformAllowed(s_eExportType) && This->IsPlatformSetToExport(s_eExportType))
		{
			iLastSelectedPlatform = i;
		}
	}

	for(int i=0;i<ExportType_NumPlatforms;i++)
	{
		s_eExportType = (EXPORT_TYPE)i;
		if(This->IsPlatformAllowed(s_eExportType) && This->IsPlatformSetToExport(s_eExportType))
		{
			char szHeading[100];
			Asura_PrintfToBfr(szHeading,"%s Character Export",sizeof(szHeading),true,GetPlatformName(i));
			s_szMainHeading = szHeading;

			AddMessage(MT_INFORMATION, "Starting %s Export...", GetPlatformName(i) );

			Asura_Assert(This,"");

			ToolsAVP_Level_Export* pxAVPThis = static_cast<ToolsAVP_Level_Export*>(This);
			
			pxAVPThis->StartExport();
			{
				for(u_int u = 0; u < AVP_Game_Core::NUM_CHARACTERTYPES; u++)
				{
					const ExportData&	xExportData	= s_axExportData[u];

					if(xExportData.m_bExport)
					{
						const AVP_Game_Core::CharacterType eType	= static_cast<AVP_Game_Core::CharacterType>(u);

						pxAVPThis->ExportCharacterFiles(eType);
					}
				}
			}
			pxAVPThis->EndExport( iLastSelectedPlatform == i );
		}
	}
	//Reset type to PC at the end to be on the safe side.
	s_eExportType = Export_PC;
}


void ToolsAVP_Level_Export::ExportAVPWeapons()
{
	int iLastSelectedPlatform = -1;
	for(int i=0;i<ExportType_NumPlatforms;i++)
	{
		s_eExportType = (EXPORT_TYPE)i;
		if(This->IsPlatformAllowed(s_eExportType) && This->IsPlatformSetToExport(s_eExportType))
		{
			iLastSelectedPlatform = i;
		}
	}

	for(int i=0;i<ExportType_NumPlatforms;i++)
	{
		s_eExportType = (EXPORT_TYPE)i;
		if(This->IsPlatformAllowed(s_eExportType) && This->IsPlatformSetToExport(s_eExportType))
		{
			char szHeading[100];
			Asura_PrintfToBfr(szHeading,"%s Weapon Export",sizeof(szHeading),true,GetPlatformName(i));
			s_szMainHeading = szHeading;

			AddMessage(MT_INFORMATION, "Starting %s Export...", GetPlatformName(i) );

			Asura_Assert(This,"");

			ToolsAVP_Level_Export* pxAVPThis = static_cast<ToolsAVP_Level_Export*>(This);

			pxAVPThis->StartExport();
			{
				for(u_int uCharItr = 0; uCharItr < AVP_Game_Core::NUM_CHARACTERTYPES; uCharItr++)
				{
					const AVP_Game_Core::CharacterType eCharType	= static_cast<AVP_Game_Core::CharacterType>(uCharItr);

					u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( eCharType );
					for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons && !IsAbortFlagSet(); ++uWeaponItr )
					{
						if ( AVP_WeaponData_System::GetShouldLoadWeapon( uWeaponItr, eCharType ) )
						{
							pxAVPThis->ExportWeaponFiles( uWeaponItr, eCharType );
						}
					}
				}
			}
			pxAVPThis->EndExport( iLastSelectedPlatform == i );
		}
	}
	//Reset type to PC at the end to be on the safe side.
	s_eExportType = Export_PC;
}

void ToolsAVP_Level_Export::ExportAllCharacterFiles(void)
{
	StartExport();
	{
		for ( u_int uCharItr = 0; uCharItr < AVP_Game_Core::NUM_CHARACTERTYPES; ++uCharItr )
		{
			const AVP_Game_Core::CharacterType eCharType = static_cast< AVP_Game_Core::CharacterType >( uCharItr );
			ExportCharacterFiles( eCharType );

			if ( Tools_BatchProcessing::IsBatch() )
			{
				AVP_WeaponData_System::InitialiseFromBlueprints();
				u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( eCharType );
				for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons && !IsAbortFlagSet(); ++uWeaponItr )
				{
					ExportWeaponFiles( uWeaponItr, eCharType );
				}
			}
		}
	}
	EndExport();
}

void ToolsAVP_Level_Export::ExportCharacterFiles(const AVP_Game_Core::CharacterType eType)
{
	if( !IsAbortFlagSet() )
	{
		StartFileExport();
		{
			const ExportData&			xExportData		= s_axExportData[eType];
			const char*					szName			= xExportData.m_szName;
			const AVP_Settings::Species	eSpecies		= xExportData.m_eSpecies;
			const Asura_Hash_ID			uBlueprintHash	= s_axExportData[eType].m_uBlueprintHash;

			Asura_String				xFilename("Characters\\");

			if((eType >= AVP_Game_Core::FIRST_SINGLE_PLAYER_CHARACTER) && (eType <= AVP_Game_Core::LAST_SINGLE_PLAYER_CHARACTER))
			{
				xFilename	+= "SP_";
				xFilename	+= szName;
				OpenExportLog(xFilename.GetString());
				switch(eSpecies)
				{
					case AVP_Settings::AVP_SPECIES_ALIEN:
					{
						AVP_Alien_Description::AddFilesToExport(true, uBlueprintHash, m_xAsuraFileList);

						Asura_String xFilename = GetTempDirectory();
						xFilename += "\\PlayerAlien_AnimTable.asr";

						// Complete the temporary filename generation...
						if ( ShouldExportBigEndian() )
						{
							xFilename	+= ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
						}

						// Save the file...
						Asura_String xPath;
						xFilename.SplitFilename( &xPath, NULL, NULL );
						CreateDirectory( xPath.GetString(), 0 );
						if ( AVP_Chunk_Alien_PlayerAnimStateData::WriteToFile( xFilename, false ) )
						{
							AddFileToExportList( xFilename, false );
						}
						else
						{
							AddMessage( MT_ERROR, "Failed to create \"%s\"", xFilename.GetString() );
						}
						break;
					}

					case AVP_Settings::AVP_SPECIES_HUMAN:
					{
						AVP_Human_Description::AddMarineFilesToExport(true, uBlueprintHash, m_xAsuraFileList);
						break;
					}

					case AVP_Settings::AVP_SPECIES_PREDATOR:
					{
						AVP_Predator_Description::AddFilesToExport(true, uBlueprintHash, m_xAsuraFileList);
						break;
					}

					default:
					{
						Asura_Assert(false, "Unexepected single-player character-type for export.");
						return;
					}
				}		// End of switch.

			}
			else if((eType >= AVP_Game_Core::FIRST_MULTIPLAYER_CHARACTER) && (eType <= AVP_Game_Core::LAST_MULTIPLAYER_CHARACTER))
			{
				xFilename	+= "MP_";
				xFilename	+= szName;
				OpenExportLog(xFilename.GetString());
				switch(eSpecies)
				{
					case AVP_Settings::AVP_SPECIES_ALIEN:
					{
						AVP_Alien_Description::AddFilesToExport(true, uBlueprintHash, m_xAsuraFileList);
						AVP_Alien_Description::AddFilesToExport(false, uBlueprintHash, m_xAsuraFileList);
						
						Asura_String xFilename = GetTempDirectory();
						xFilename += "\\PlayerAlien_AnimTable.asr";

						// Complete the temporary filename generation...
						if ( ShouldExportBigEndian() )
						{
							xFilename	+= ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
						}

						// Save the file...
						if ( AVP_Chunk_Alien_PlayerAnimStateData::WriteToFile( xFilename, true ) )
						{
							AddFileToExportList( xFilename, false );
						}
						else
						{
							AddMessage( MT_ERROR, "Failed to create \"%s\"", xFilename.GetString() );
						}
						break;
					}

					case AVP_Settings::AVP_SPECIES_HUMAN:
					{
						AVP_Human_Description::AddMarineFilesToExport(true, uBlueprintHash, m_xAsuraFileList);
						AVP_Human_Description::AddMarineFilesToExport(false, uBlueprintHash, m_xAsuraFileList);
						break;
					}

					case AVP_Settings::AVP_SPECIES_PREDATOR:
					{
						AVP_Predator_Description::AddFilesToExport(true, uBlueprintHash, m_xAsuraFileList);
						AVP_Predator_Description::AddFilesToExport(false, uBlueprintHash, m_xAsuraFileList);
						break;
					}

					default:
					{
						Asura_Assert(false, "Unexepected multiplayer character-type for export.");
						return;
					}
				}		// End of switch.
			}
			else
			{
				Asura_Assert(false, "Unexpected character-type to export.");
				return;
			}

			AddMessage(MT_INFORMATION, "Exporting \"%s\"...", xFilename.GetString());

			// Perform the export...
			ExportCharacterFile(xFilename.GetString());
			CloseExportLog();
		}
		EndFileExport();
	}
}

void ToolsAVP_Level_Export::ExportWeaponFiles( const u_int uWeaponIndex, const AVP_Game_Core::CharacterType eType )
{
	if( !IsAbortFlagSet() )
	{
		StartFileExport();
		{
			Asura_String xWeaponFilename("Characters\\");
			const char* szWeaponName = AVP_WeaponData_System::GetWeaponNameFromIndex( uWeaponIndex, eType );
			const Asura_Hash_ID uWeaponHash = Asura_GetHashID( szWeaponName );
			xWeaponFilename += szWeaponName;
			OpenExportLog( xWeaponFilename.GetString() );

			// Add the weapon blueprint data (inc anims) to the export list.
			Asura_Blueprint* pxWeaponBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_INVENTORY_ITEMS, uWeaponHash );
			if ( pxWeaponBP != NULL )
			{
				Tools_Mode_Blueprint::AddBlueprintFilesToExport( m_xAsuraFileList, pxWeaponBP, false );

				Asura_String xFilename = GetTempDirectory();
				xFilename += "\\";
				xFilename += xWeaponFilename;
				xFilename += "_AnimTable.asr";

				// Complete the temporary filename generation...
				if ( ShouldExportBigEndian() )
				{
					xFilename	+= ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
				}

				// Save the file...
				Asura_String xPath;
				xFilename.SplitFilename( &xPath, NULL, NULL );
				CreateDirectory( xPath.GetString(), 0 );

				switch ( eType )
				{
					case AVP_Game_Core::SINGLE_PLAYER_ALIEN:
					case AVP_Game_Core::MULTIPLAYER_ALIEN:
					{
						if ( AVP_Chunk_Alien_PlayerAnimStateData::WriteToFile( xFilename, eType == AVP_Game_Core::MULTIPLAYER_ALIEN ) )
						{
							AddFileToExportList( xFilename, false );
						}
						else
						{
							AddMessage( MT_ERROR, "Failed to create \"%s\"", xFilename.GetString() );
						}
					}
					break;

					case AVP_Game_Core::SINGLE_PLAYER_MARINE:
					case AVP_Game_Core::MULTIPLAYER_MARINE:
					{
						if ( AVP_Chunk_Human_PlayerAnimStateData::WriteToFile( xFilename, uWeaponHash ) )
						{
							AddFileToExportList( xFilename, false );
						}
						else
						{
							AddMessage( MT_ERROR, "Failed to create \"%s\"", xFilename.GetString() );
						}
					}
					break;

					case AVP_Game_Core::SINGLE_PLAYER_PREDATOR:
					case AVP_Game_Core::MULTIPLAYER_PREDATOR:
					{
						if ( AVP_Chunk_Predator_PlayerAnimStateData::WriteToFile( xFilename, uWeaponHash ) )
						{
							AddFileToExportList( xFilename, false );
						}
						else
						{
							AddMessage( MT_ERROR, "Failed to create \"%s\"", xFilename.GetString() );
						}
					}
					break;
				}
			}
			else
			{
				// not a weapon but might be a pickup
				pxWeaponBP = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_PICKUPS, uWeaponHash );
				if ( pxWeaponBP != NULL )
				{
					Tools_Mode_Blueprint::AddBlueprintFilesToExport( m_xAsuraFileList, pxWeaponBP, false );
				}
			}

			AddMessage( MT_INFORMATION, "Exporting \"%s\"...", xWeaponFilename.GetString() );
			ExportCharacterFile( xWeaponFilename.GetString() );
			CloseExportLog();
		}
		EndFileExport();
	}
}

void ToolsAVP_Level_Export::StartLevelExport()
{
	PARENT::StartLevelExport();

	if ( !m_bFakeLevelExport )
	{
		if ( !AVP_Waypoint_Manager::ExtraWaypointingDataValid() )
		{
			ToolsAVP_Mode_Waypointing::GenerateExtraWaypointingData();
			// The extra waypointing data will have marked the waypointing and possibly the entities as modified.
			// If we're batch exporting we must forget about this, as we may have other levels to do after this one.
			if ( Tools_BatchProcessing::IsBatch() )
			{
				Tools_SaveAsura::SetWaypointingModified( false );
				Tools_SaveAsura::SetEntityModified( false );
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Level_Export::StartFileExport
// Author       : Andy Tate
// Date         : 17-Oct-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Level_Export::StartFileExport(void)
{
	// Call the base-function...
	PARENT::StartFileExport();

	// Allocate the data...
	s_uNumMaterialResponseOverridesForExport	= Asura_MaterialResponse_System::GetNumOverrideMaps();
	s_pxMaterialResponseOverridesForExport		= Asura_MaterialResponse_System::CreateOverrideMapHashArray();

	SetMaterialResponseOverridesForExport(s_uNumMaterialResponseOverridesForExport, s_pxMaterialResponseOverridesForExport);
}

/// End of function 'ToolsAVP_Level_Export::StartFileExport'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Level_Export::EndFileExport
// Author       : Andy Tate
// Date         : 17-Oct-2007
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Level_Export::EndFileExport(void)
{
	// Deallocate the data...
	Asura_MaterialResponse_System::DestroyOverrideMapHashArray(s_pxMaterialResponseOverridesForExport);
	ResetMaterialResponseOverridesForExport();

	s_uNumMaterialResponseOverridesForExport	= 0;
	s_pxMaterialResponseOverridesForExport		= NULL;

	// Call the base-function...
	PARENT::EndFileExport();
}

/// End of function 'ToolsAVP_Level_Export::EndFileExport'. ///



void ToolsAVP_Level_Export::AddFilesForCharacterExport(List<Asura_String>& xFileList,const char* szDirectory)
{
 	//The files are added else where so empty function here
}

void ToolsAVP_Level_Export::AddCommonExportFilesToList(List<Asura_String>& xNameList)
{
	PARENT::AddCommonExportFilesToList(xNameList);

	AddSurfaceEffectsToExportList(xNameList);

	int iTex;
	iTex = Asura_Texture_Management::GetIndexFromHashID(Asura_GetHashID_IgnoreExtension("\\SpecialFX\\AVP_Laser.dds"));
	if (iTex!=ASURA_TEXTURE_NONE)
	{	
		Tools_Level_Export::AddTextureToGUIMenuList(iTex);
	}
	iTex = Asura_Texture_Management::GetIndexFromHashID(Asura_GetHashID_IgnoreExtension("\\SpecialFX\\AVP_PredCloak.dds"));
	if (iTex!=ASURA_TEXTURE_NONE)
	{	
		Tools_Level_Export::AddTextureToGUIMenuList(iTex);
	}
	iTex = Asura_Texture_Management::GetIndexFromHashID(Asura_GetHashID_IgnoreExtension("\\SpecialFX\\AVP_Pheromone.dds"));
	if (iTex!=ASURA_TEXTURE_NONE)
	{	
		Tools_Level_Export::AddTextureToGUIMenuList(iTex);
	}

	AddWeaponDataToExport( xNameList );
}


void ToolsAVP_Level_Export::GetExportedCharacterDir(Asura_String& xOutputDir, bool bCreate)
{
	char acFilename[MAX_PATH];

	const char* szPlatformName = GetPlatformName(s_eExportType);
	{
		Asura_PrintfToBfr(acFilename, "..\\%s_Release\\Characters", MAX_PATH, true, szPlatformName);

		xOutputDir = acFilename;

		if (bCreate)
		{
			CreateDirectory(xOutputDir.GetString(),0);
		}
	}
}

bool ToolsAVP_Level_Export::ShouldExportAnimationUncompressed(const Asura_Hierarchy_Anim& xAnimation) const                                                                                                                                                                                                                            
{
	// Don't compress first-person character animations, you can notice the lack of data...
	return((s_eExportType == Export_PC) || s_bIsCharacterExport || xAnimation.m_bDontCompressKeyframes);
}

bool ToolsAVP_Level_Export::ShouldExportAnimationWithoutRemovingKeyframes(const Asura_Hierarchy_Anim& xAnimation) const
{
	// PC doesn't need the animations to be compressed - lets keep it at the highest quality...
	return((s_eExportType == Export_PC) || xAnimation.m_bDontRemoveKeyframes);
}

void ToolsAVP_Level_Export::AddSurfaceEffectsToExportList(List<Asura_String>& xNameList)
{
	Asura_String xFilename = GetTempDirectory();
	xFilename += "\\SurfaceEffects.asr";

	// Complete the temporary filename generation...
	if(ShouldExportBigEndian())
	{
		xFilename	+= ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}

	// Save the file...
	if(AVP_Chunk_SurfaceOverlay::Save(xFilename.GetString()))
	{
		AddFileToExportList(xFilename, false);
		// Now export the actual surface effects (from blueprints)
		const Asura_Hash_ID	uBlueprintHash	= Asura_GetHashID("SurfaceEffect");
		if(uBlueprintHash != ASURA_HASH_ID_UNSET)
		{
			const Asura_Hash_ID	uTypeHash	= Asura_GetHashID("SurfaceEffects");
			const Asura_Hash_ID	uParentHash	= uBlueprintHash;

			Tools_Mode_Blueprint::AddBlueprintFilesToExportList(xNameList, uTypeHash, uParentHash);
		}
	}
	else
	{
		AddMessage(MT_ERROR,"Failed to create \"%s\"", xFilename.GetString());
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Level_Export::IsLanguageAllowed
// Author       : Andy Tate
// Date         : 4-Aug-2008
////////////////////////////////////////////////////////////////////////////////

bool ToolsAVP_Level_Export::IsLanguageAllowed(int iLanguage) const
{
	switch(iLanguage)
	{
		case ASURA_LT_L_ENGLISH:
		//case ASURA_LT_L_FRENCH:
		//case ASURA_LT_L_GERMAN:
		//case ASURA_LT_L_ITALIAN:
		//case ASURA_LT_L_SPANISH:
		//case ASURA_LT_L_RUSSIAN:
		//case ASURA_LT_L_AMERICAN:
		//case ASURA_LT_L_JAPANESE:
		//case ASURA_LT_L_DUTCH:
		//case ASURA_LT_L_PORTUGUESE:
		//case ASURA_LT_L_DANISH:
		//case ASURA_LT_L_FINNISH:
		//case ASURA_LT_L_NORWEGIAN:
		{
			return(true);
		}

		default:
		{
			return(false);
		}
	}		// End of switch.
}

void ToolsAVP_Level_Export::CreateExportWeaponOverlay()
{
	Asura_Assert(This,"");

	// Don't allow character export and level export dialogs to co-exist.
	if(Asura_Overlay_System::DoesOverlayExist(TOOLSAVP_OVERLAY_WEAPONEXPORT)) { return; }
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_LEVELEXPORT)) { return; }
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_CHAREXPORT)) { return; }

	Asura_Overlay*	pxOverlay	= new Asura_Overlay("Export Character Files", false);
	Asura_Vector_2	xOffset(1.0f,3.0f);

	// Add overlay controls
	static_cast< ToolsAVP_Level_Export* >( This )->AddWeaponExportOverlayControls(pxOverlay, xOffset);
	pxOverlay->m_uID = TOOLSAVP_OVERLAY_WEAPONEXPORT;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
}

void ToolsAVP_Level_Export::ToggleSPCharacters()
{
	s_bSPCharOn = !s_bSPCharOn;
	for(u_int uCharItr = AVP_Game_Core::FIRST_SINGLE_PLAYER_CHARACTER; uCharItr <= AVP_Game_Core::LAST_SINGLE_PLAYER_CHARACTER; uCharItr++)
	{
		ExportData&	xExportData	= s_axExportData[ uCharItr ];
		xExportData.m_bExport = s_bSPCharOn;
	}
}

void ToolsAVP_Level_Export::ToggleMPCharacters()
{
	s_bMPCharOn = !s_bMPCharOn;
	for(u_int uCharItr = AVP_Game_Core::FIRST_MULTIPLAYER_CHARACTER; uCharItr <= AVP_Game_Core::LAST_MULTIPLAYER_CHARACTER; uCharItr++)
	{
		ExportData&	xExportData	= s_axExportData[ uCharItr ];
		xExportData.m_bExport = s_bMPCharOn;
	}
}

void ToolsAVP_Level_Export::ToggleSPWeapons()
{
	s_bSPWeaponOn = !s_bSPWeaponOn;
	for(u_int uCharItr = AVP_Game_Core::FIRST_SINGLE_PLAYER_CHARACTER; uCharItr <= AVP_Game_Core::LAST_SINGLE_PLAYER_CHARACTER; uCharItr++)
	{
		ExportData&	xExportData	= s_axExportData[ uCharItr ];
		xExportData.m_bExportWeaponsOn = s_bSPWeaponOn;
	
		AVP_Game_Core::CharacterType eChar = static_cast< AVP_Game_Core::CharacterType >( uCharItr );
		u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( eChar );
		for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons; ++uWeaponItr )
		{
			AVP_WeaponData_System::SetShouldLoadWeapon( uWeaponItr, eChar, s_bSPWeaponOn );
		}
	}
}

void ToolsAVP_Level_Export::ToggleMPWeapons()
{
	s_bMPWeaponOn = !s_bMPWeaponOn;
	for(u_int uCharItr = AVP_Game_Core::FIRST_MULTIPLAYER_CHARACTER; uCharItr <= AVP_Game_Core::LAST_MULTIPLAYER_CHARACTER; uCharItr++)
	{
		ExportData&	xExportData	= s_axExportData[uCharItr];
		xExportData.m_bExportWeaponsOn = s_bMPWeaponOn;

		AVP_Game_Core::CharacterType eChar = static_cast< AVP_Game_Core::CharacterType >( uCharItr );
		u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( eChar );
		for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons; ++uWeaponItr )
		{
			AVP_WeaponData_System::SetShouldLoadWeapon( uWeaponItr, eChar, s_bMPWeaponOn );
		}
	}
}

void ToolsAVP_Level_Export::ToggleWeaponsForCharacter( int iChar )
{
	Asura_Assert( iChar >= 0 && iChar < AVP_Game_Core::NUM_CHARACTERTYPES, "" );
	if ( iChar < 0 || iChar >= AVP_Game_Core::NUM_CHARACTERTYPES )
	{
		 return;
	}

	ExportData&	xExportData	= s_axExportData[ iChar ];
	xExportData.m_bExportWeaponsOn = !xExportData.m_bExportWeaponsOn;

	AVP_Game_Core::CharacterType eChar = static_cast< AVP_Game_Core::CharacterType >( iChar );
	u_int uNumWeapons = AVP_WeaponData_System::GetNumWeaponsForCharacterType( eChar );
	for ( u_int uWeaponItr = 0; uWeaponItr < uNumWeapons; ++uWeaponItr )
	{
		AVP_WeaponData_System::SetShouldLoadWeapon( uWeaponItr, eChar, xExportData.m_bExportWeaponsOn );
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Level_Export::GenerateExportedLevelFilename
// Author       : Andy Tate
// Date         : 2-Oct-2008
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Level_Export::GenerateExportedLevelFilename( Asura_String xEnvironmentFilename,
														  const char* szExtension,
														  Asura_String& xOutputFilename)
{
	const Asura_String	xMainFilename(Tools_SaveAsura::GetMainFilename());

	const char*			szPlatformName	= GetPlatformName(s_eExportType);

	Asura_String		xMissionName;

	// Begin creating the output filename...
	xOutputFilename	 = "..\\";
	xOutputFilename	+= szPlatformName;
	xOutputFilename	+= "_Release\\envs\\";

	// Decide what we're to call our output file...
	xEnvironmentFilename.SplitFilename(NULL, &xMissionName, NULL);

	// Check for file prefixes...
	if(xMissionName.StartsWith("Demo_") || xMissionName.StartsWith("demo_") )
	{
		xOutputFilename	+= "Demos\\";
	}
	else if(xMissionName.StartsWith("FE_") || xMissionName.StartsWith("fe_") )
	{
		xOutputFilename	+= "Frontend\\";
	}
	else if(xMissionName.StartsWith("MP_") || xMissionName.StartsWith("mp_") )
	{
		xOutputFilename	+= "MP\\";
	}
	else if(xMissionName.StartsWith("Test_") || xMissionName.StartsWith("test_") )
	{
		xOutputFilename	+= "Tests\\";
	}
	else
	{
		xEnvironmentFilename.MakeLowercase();

		// Check for particular environments...
		if(xEnvironmentFilename.Contains("\\colony") || xEnvironmentFilename.Contains("/colony"))
		{
			xOutputFilename	+= "Colony\\";
		}
		else if(xEnvironmentFilename.Contains("\\jungle") || xEnvironmentFilename.Contains("/jungle") )
		{
			xOutputFilename	+= "Jungle\\";
		}
		else if(xEnvironmentFilename.Contains("\\lab") ||xEnvironmentFilename.Contains("/lab") )
		{
			xOutputFilename	+= "Lab\\";
		}
		else if(xEnvironmentFilename.Contains("\\pyramid") || xEnvironmentFilename.Contains("/pyramid") )
		{
			xOutputFilename	+= "Pyramid\\";
		}
		else if(xEnvironmentFilename.Contains("\\refinery") || xEnvironmentFilename.Contains("/refinery") )
		{
			xOutputFilename	+= "Refinery\\";
		}
		else if( xEnvironmentFilename.Contains("\\ruins") || xEnvironmentFilename.Contains("/ruins") )
		{
			xOutputFilename	+= "Ruins\\";
		}
		else if( xEnvironmentFilename.Contains("\\tests") || xEnvironmentFilename.Contains("/tests") )
		{
			xOutputFilename	+= "Tests\\";
		}
		else
		{
			// Don't recognise the path - so simply put it in the "Envs" directory...
		}
	}

	// Complete the output filename...
	xOutputFilename	+= xMissionName;
	xOutputFilename	+= szExtension;
}

/// End of function 'ToolsAVP_Level_Export::GenerateExportedLevelFilename'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Level_Export::GetExportedLevelFilename
// Author       : Andy Tate
// Date         : 2-Oct-2008
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Level_Export::GetExportedLevelFilename(const Asura_String& xEnvironmentFilename, Asura_String& xOutputFilename)
{
	const char*	szPlatformExt	= GetPlatformExtension();

	GenerateExportedLevelFilename(xEnvironmentFilename, szPlatformExt, xOutputFilename);
}

/// End of function 'ToolsAVP_Level_Export::GetExportedLevelFilename'. ///



#if defined(ASURA_USE_TEXTURE_STREAMING)

////////////////////////////////////////////////////////////////////////////////
// Function     : ToolsAVP_Level_Export::GetExportedLevelTextureStreamingFilename
// Author       : Andy Tate
// Date         : 2-Oct-2008
////////////////////////////////////////////////////////////////////////////////

void ToolsAVP_Level_Export::GetExportedLevelTextureStreamingFilename(const Asura_String& xEnvironmentFilename, Asura_String& xOutputFilename)
{
	const char*		szPlatformExt	= GetPlatformExtension();

	Asura_String	xExtension(szPlatformExt);

	xExtension	+= "_textures";
	GenerateExportedLevelFilename(xEnvironmentFilename, xExtension.GetString(), xOutputFilename);
}

/// End of function 'ToolsAVP_Level_Export::GetExportedLevelTextureStreamingFilename'. ///


#endif	//defined(ASURA_USE_TEXTURE_STREAMING)

void ToolsAVP_Level_Export::AddWeaponDataToExport( List<Asura_String>& xNameList )
{
	Asura_String xFilename = GetTempDirectory();
	xFilename += "\\WeaponData.asr";

	// Complete the temporary filename generation...
	if ( ShouldExportBigEndian() )
	{
		xFilename	+= ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}

	// Save the file...
	if ( AVP_Chunk_WeaponData::WriteToFile( xFilename ) )
	{
		AddFileToExportList( xFilename, false );
	}
	else
	{
		AddMessage( MT_ERROR, "Failed to create \"%s\"", xFilename.GetString() );
	}
}

/// End of file. ///
