//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Mode_MeshEdit.cpp
//
// Copyright (C) 2009 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "MeshEdit/ToolsAVP_Mode_MeshEdit.h"

// asura
#include "Camera/Asura_Camera.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Module.h"

// avp
#include "Core/AVP_Plane.h"

// tools
#include "Misc/Tools_Adjacency.h"
#include "Misc/Tools_MessageLog.h"
#include "Misc/Tools_Render.h"

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_MLA_BadPolyAngle
//
//////////////////////////////////////////////////////////////////////////////

class ToolsAVP_MLA_BadPolyAngle : public Tools_MessageLogAction
{
public:
	ToolsAVP_MLA_BadPolyAngle( const u_int uIndex ) : m_uIndex( uIndex ) {}

	virtual void Execute()
	{
		// Go to the bad poly.
		const Asura_ListElt< ToolsAVP_Mode_MeshEdit::BadPolyPair >* pxBPPElt = ToolsAVP_Mode_MeshEdit::GetBPPElt( m_uIndex );
		if ( !pxBPPElt ) { return; }

		const Asura_CollisionMesh* pxBaseMesh = Asura_Module_System::GetCollisionMeshForModule( pxBPPElt->Data().m_xBase.ModuleIndex );
		if ( !pxBaseMesh ) { return; }

		const Asura_Vector_3& xModuleCentre = Asura_Module_System::GetPositionOfModule( pxBPPElt->Data().m_xBase.ModuleIndex );

		Asura_Vector_3 axVertices[ 4 ];
		pxBaseMesh->GetVertices( axVertices, pxBPPElt->Data().m_xBase.PolyIndex );
		Asura_Vector_3 xAveragePos = axVertices[ 0 ];
		xAveragePos += axVertices[ 1 ];
		xAveragePos += axVertices[ 2 ];
		if ( pxBaseMesh->IsQuad( pxBPPElt->Data().m_xBase.PolyIndex ) )
		{
			xAveragePos += axVertices[ 3 ];
			xAveragePos *= 0.25f;
			xAveragePos += xModuleCentre;
		}
		else
		{
			xAveragePos *= 0.333333f;
			xAveragePos += xModuleCentre;
		}

		// If we're a long way away move the camera, otherwise just reorientate it.
		if ( ( *Asura_Camera::GetPosition() - xAveragePos ).MagnitudeSquared() > 400.0f )
		{
			const Asura_Matrix_3x3* xCameraOrientation = Asura_Camera::GetOrientation();

			Asura_Vector_3 xCamPos = xAveragePos;
			xCamPos.x -= xCameraOrientation->e13 * 20.0f;
			xCamPos.y -= xCameraOrientation->e23 * 20.0f;
			xCamPos.z -= xCameraOrientation->e33 * 20.0f;

			Asura_Camera::SetPosition( xCamPos );
		}
		else
		{
			Asura_Vector_3 xToPoly = xAveragePos - *Asura_Camera::GetPosition();
			xToPoly.Normalise();

			Asura_Matrix_3x3 xCamOrientM;
			Asura_Maths::OrientationFromDirection( xCamOrientM, xToPoly );

			Asura_Camera::SetOrientation( xCamOrientM );
		}

		// Render an outline around it.
		ToolsAVP_Mode_MeshEdit::RenderBPP( m_uIndex );
	}

private:
	u_int m_uIndex;
};

//////////////////////////////////////////////////////////////////////////////
//
// ToolsAVP_Mode_MeshEdit
//
//////////////////////////////////////////////////////////////////////////////

bool  ToolsAVP_Mode_MeshEdit::s_bMinAngleChanged = false;
bool  ToolsAVP_Mode_MeshEdit::s_bMaxAngleChanged = false;
float ToolsAVP_Mode_MeshEdit::s_fMinAngle = 0.0f;
float ToolsAVP_Mode_MeshEdit::s_fMaxAngle = 89.0f * Asura_Maths::DegToRadConstant;
Asura_LinkList< ToolsAVP_Mode_MeshEdit::BadPolyPair > ToolsAVP_Mode_MeshEdit::s_xBadPolyList;
u_int ToolsAVP_Mode_MeshEdit::s_uRenderBadPolyPairIndex = 0xffffffff;

void ToolsAVP_Mode_MeshEdit::Update()
{
	PARENT::Update();

	if ( s_bMinAngleChanged )
	{
		s_bMinAngleChanged = false;
		s_fMaxAngle = Asura_Maths::Max( s_fMinAngle, s_fMaxAngle );
	}
	else if ( s_bMaxAngleChanged )
	{
		s_bMaxAngleChanged = false;
		s_fMinAngle = Asura_Maths::Min( s_fMinAngle, s_fMaxAngle );
	}
}

void ToolsAVP_Mode_MeshEdit::RenderWorld_PostToneMap()
{
	PARENT::RenderWorld_PostToneMap();

	if ( s_uRenderBadPolyPairIndex != 0xffffffff )
	{
		const Asura_ListElt< BadPolyPair >* pxBPPElt = GetBPPElt( s_uRenderBadPolyPairIndex );
		if ( pxBPPElt )
		{
			const int iBaseModuleIndex = pxBPPElt->Data().m_xBase.ModuleIndex;
			const int iBasePolyIndex   = pxBPPElt->Data().m_xBase.PolyIndex;
			const int iAdjModuleIndex = pxBPPElt->Data().m_xAdj.ModuleIndex;
			const int iAdjPolyIndex   = pxBPPElt->Data().m_xAdj.PolyIndex;

			const Asura_CollisionMesh* pxCollMeshBase = Asura_Module_System::GetCollisionMeshForModule( iBaseModuleIndex );
			if ( pxCollMeshBase )
			{
				const Asura_Vector_3& xModuleCentre = Asura_Module_System::GetPositionOfModule( iBaseModuleIndex );

				Asura_Vector_3 axVertices[ 4 ];
				pxCollMeshBase->GetVertices( axVertices, iBasePolyIndex );
				axVertices[ 0 ] += xModuleCentre;
				axVertices[ 1 ] += xModuleCentre;
				axVertices[ 2 ] += xModuleCentre;

				if ( pxCollMeshBase->IsQuad( iBasePolyIndex ) )
				{
					axVertices[ 3 ] += xModuleCentre;

					Tools_Render::RenderCylinder( axVertices[ 0 ], axVertices[ 1 ], 0.1f, 0xffff0000, false );
					Tools_Render::RenderCylinder( axVertices[ 1 ], axVertices[ 2 ], 0.1f, 0xffff0000, false );
					Tools_Render::RenderCylinder( axVertices[ 2 ], axVertices[ 3 ], 0.1f, 0xffff0000, false );
					Tools_Render::RenderCylinder( axVertices[ 3 ], axVertices[ 0 ], 0.1f, 0xffff0000, false );
				}
				else
				{
					Tools_Render::RenderCylinder( axVertices[ 0 ], axVertices[ 1 ], 0.1f, 0xffff0000, false );
					Tools_Render::RenderCylinder( axVertices[ 1 ], axVertices[ 2 ], 0.1f, 0xffff0000, false );
					Tools_Render::RenderCylinder( axVertices[ 2 ], axVertices[ 0 ], 0.1f, 0xffff0000, false );
				}
			}
			const Asura_CollisionMesh* pxCollMeshAdj = Asura_Module_System::GetCollisionMeshForModule( iAdjModuleIndex );
			if ( pxCollMeshAdj )
			{
				const Asura_Vector_3& xModuleCentre = Asura_Module_System::GetPositionOfModule( iAdjModuleIndex );

				Asura_Vector_3 axVertices[ 4 ];
				pxCollMeshAdj->GetVertices( axVertices, iAdjPolyIndex );
				axVertices[ 0 ] += xModuleCentre;
				axVertices[ 1 ] += xModuleCentre;
				axVertices[ 2 ] += xModuleCentre;

				if ( pxCollMeshAdj->IsQuad( iAdjPolyIndex ) )
				{
					axVertices[ 3 ] += xModuleCentre;

					Tools_Render::RenderCylinder( axVertices[ 0 ], axVertices[ 1 ], 0.1f, 0xffff0000, false );
					Tools_Render::RenderCylinder( axVertices[ 1 ], axVertices[ 2 ], 0.1f, 0xffff0000, false );
					Tools_Render::RenderCylinder( axVertices[ 2 ], axVertices[ 3 ], 0.1f, 0xffff0000, false );
					Tools_Render::RenderCylinder( axVertices[ 3 ], axVertices[ 0 ], 0.1f, 0xffff0000, false );
				}
				else
				{
					Tools_Render::RenderCylinder( axVertices[ 0 ], axVertices[ 1 ], 0.1f, 0xffff0000, false );
					Tools_Render::RenderCylinder( axVertices[ 1 ], axVertices[ 2 ], 0.1f, 0xffff0000, false );
					Tools_Render::RenderCylinder( axVertices[ 2 ], axVertices[ 0 ], 0.1f, 0xffff0000, false );
				}
			}
		}
		else
		{
			s_uRenderBadPolyPairIndex = 0xffffffff;
		}
	}
}

void ToolsAVP_Mode_MeshEdit::ShutdownMode()
{
	PARENT::ShutdownMode();

	s_xBadPolyList.Clear();
	s_uRenderBadPolyPairIndex = 0xffffffff;
	Tools_MessageLog::DeleteMessages( Tools_MessageLog::VERIFICATION, "Bad Poly Angle" );
}

void ToolsAVP_Mode_MeshEdit::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();


	Asura_Dialog* pxDialogue = new Asura_Divider_Dialog( xOffset );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();


	// Angle testing stuff.
	pxDialogue = new Asura_AngleNumerical_Dialog( "Min/Max Poly Angle", xOffset, &s_fMinAngle, 0.0f, 180.0f, 1.0f, "%0.1f", &s_bMinAngleChanged );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x += 20.5f;

	pxDialogue = new Asura_AngleNumerical_Dialog( "", xOffset, &s_fMaxAngle, 0.0f, 180.0f, 1.0f, "%0.1f", &s_bMaxAngleChanged );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x -= 20.5f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialogue = new Asura_Function_Dialog( "Check Selected Collision Poly Angles", xOffset, CheckPolyAngles );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void ToolsAVP_Mode_MeshEdit::CheckPolyAngles()
{
	const float fMinDot = Asura_Maths::Cosine( s_fMaxAngle );
	const float fMaxDot = Asura_Maths::Cosine( s_fMinAngle );

	s_xBadPolyList.Clear();
	s_uRenderBadPolyPairIndex = 0xffffffff;
	Tools_MessageLog::DeleteMessages( Tools_MessageLog::MESH_EDIT, "Bad Poly Angle" );

	Tools_Polygon_Selection::StartIteration();
	PolygonID xBasePolygonID;
	while ( Tools_Polygon_Selection::GetNextSelected( &xBasePolygonID ) )
	{
		const Asura_CollisionMesh* pxBaseMesh = Asura_Module_System::GetCollisionMeshForModule( xBasePolygonID.ModuleIndex );
		if ( pxBaseMesh == NULL ) { continue; }

		Asura_Vector_3 xBaseNormal;
		pxBaseMesh->GetNormal( xBasePolygonID.PolyIndex, xBaseNormal );
		Asura_Vector_3 axBaseVertices[ 4 ];
		pxBaseMesh->GetVertices( axBaseVertices, xBasePolygonID.PolyIndex );
		const bool bBaseIsQuad = pxBaseMesh->IsQuad( xBasePolygonID.PolyIndex );

		const Tools_Polygon_Adjacency* pxPolyAdj = Tools_Adjacency::GetPolygonAdjacencies( xBasePolygonID.ModuleIndex, xBasePolygonID.PolyIndex );
		for ( const Tools_Adjacent_Polygon* pxAdj = pxPolyAdj->GetFirstAdjacency(); pxAdj; pxAdj = pxAdj->GetNext() )
		{
			const int iAdjModuleIndex = pxAdj->GetModuleIndex();
			const int iAdjPolyIndex   = pxAdj->GetPolygonIndex();

			// Avoid duplicates.
			if ( xBasePolygonID.PolyIndex > iAdjPolyIndex ) { continue; }

			PolygonID xAdjPolygonID;
			xAdjPolygonID.ModuleIndex = iAdjModuleIndex;
			xAdjPolygonID.PolyIndex   = iAdjPolyIndex;
			if ( !Tools_Polygon_Selection::IsSelected( xAdjPolygonID ) ) { continue; }

			const Asura_CollisionMesh* pxAdjMesh = Asura_Module_System::GetCollisionMeshForModule( iAdjModuleIndex );
			if ( pxAdjMesh == NULL ) { continue; }

			Asura_Vector_3 xAdjPolyNormal;
			pxAdjMesh->GetNormal( iAdjPolyIndex, xAdjPolyNormal );
			// Because we are interested in the internal angle between the two polys, not the external angle, we must flip the normal.
			xAdjPolyNormal *= -1.0f;
			Asura_Vector_3 axAdjVertices[ 4 ];
			pxAdjMesh->GetVertices( axAdjVertices, iAdjPolyIndex );
			const bool bAdjIsQuad = pxAdjMesh->IsQuad( iAdjPolyIndex );

			// We must check that this poly is fully adjacent, that is there are at least 2 co-incident verts.
			int iNonCoincidentAdjPolyVertIndex;
			if ( !PolyIsFullyAdjacent( axBaseVertices, bBaseIsQuad, axAdjVertices, bAdjIsQuad, iNonCoincidentAdjPolyVertIndex ) ) { continue; }

			// Check that the polys meet at an angle < 180.
			const AVP_Plane xBasePlane( Asura_AVector3( axBaseVertices[ 0 ].x, axBaseVertices[ 0 ].y, axBaseVertices[ 0 ].z ), Asura_AVector3( xBaseNormal ) );
	        const float fTargetDistance = xBasePlane.CalculateClosestDistance( axAdjVertices[ iNonCoincidentAdjPolyVertIndex ] );
			if ( fTargetDistance < 0.0f ) { continue; }

			// Check that the angle is within bounds.
			const float fDot = xBaseNormal * xAdjPolyNormal;
			if ( fMinDot <= fDot && fDot <= fMaxDot )
			{
				BadPolyPair xBPP;
				xBPP.m_xBase.ModuleIndex = xBasePolygonID.ModuleIndex;
				xBPP.m_xBase.PolyIndex   = xBasePolygonID.PolyIndex;
				xBPP.m_xAdj.ModuleIndex = iAdjModuleIndex;
				xBPP.m_xAdj.PolyIndex   = iAdjPolyIndex;
				s_xBadPolyList.PushBack( xBPP );
			}
		}
	}

	if ( s_xBadPolyList.Size() )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::MESH_EDIT, "Bad Poly Angle", "Found %u bad poly angle(s):", s_xBadPolyList.Size() );
		u_int uIndex = 0;
		for ( Asura_LinkList_ConstIt< BadPolyPair > xBPPIter( s_xBadPolyList ); !xBPPIter.Done(); xBPPIter.Next() )
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::MESH_EDIT, "Bad Poly Angle", new ToolsAVP_MLA_BadPolyAngle( uIndex ),
				"%u - module: %d, poly: %d", uIndex, xBPPIter.GetCurrent()->m_xBase.ModuleIndex, xBPPIter.GetCurrent()->m_xBase.PolyIndex );
			++uIndex;
		}
	}
	else
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::MESH_EDIT, "Bad Poly Angle", "No bad poly angles found." );
	}
	Tools_MessageLog::ShowOverlay( Tools_MessageLog::MESH_EDIT, "Bad Poly Angle" );
}

bool ToolsAVP_Mode_MeshEdit::PolyIsFullyAdjacent( const Asura_Vector_3 axBaseVerts[ 4 ], const bool bBaseIsQuad,
												  const Asura_Vector_3 axAdjVerts[ 4 ], const bool bAjdIsQuad,
												  int& iNonCoincidentAdjPolyVertIdx )
{
	int iNumCoincidentVerts = 0;
	int abAdjVertIndexCoincident[ 4 ] = { false, false, false, false };

	for ( int i = 0; i < ( bBaseIsQuad ? 4 : 3 ); ++i )
	{
		for ( int j = 0; j < ( bAjdIsQuad ? 4 : 3 ); ++j )
		{
			if ( axBaseVerts[ i ].ApproximatelyEquals( axAdjVerts[ j ] ) )
			{
				abAdjVertIndexCoincident[ j ] = true;
				++iNumCoincidentVerts;
				if ( iNumCoincidentVerts == 2 )
				{
					// Find a vert on the adjacent poly that is not coincident.
					for ( int k = 0; k < ( bAjdIsQuad ? 4 : 3 ); ++k )
					{
						if ( !abAdjVertIndexCoincident[ k ] )
						{
							iNonCoincidentAdjPolyVertIdx = k;
							return true;
						}
					}

					return false;
				}
			}
		}
	}

	return false;
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
