//////////////////////////////////////////////////////////////////////////////
//
// Tools_Project_Texturing_MaterialResponse.cpp
//
// Copyright (c) 2007 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#include "Texturing/Tools_Texturing_MaterialResponse.h"

#include "Material/ToolsAVP_SurfaceOverlay.h"
#include "FSFX/AVP_FSFX.h"
#include "Materials/Tools_MaterialResponse_System.h"

#include "Blueprint/AVP_Blueprint_HashIDs.h"

static bool IsAlienChecked(void *pData)
{
	Tools_MaterialResponse* pxMaterialResponse = (Tools_MaterialResponse*)pData;

	return (pxMaterialResponse->GetProjectContext() & AVP_MATERIAL_RESPONSE_PROJECT_FLAG_ALIEN) != 0;
}
static void SetAlienChecked(bool bValue, void *pData)
{
	Tools_MaterialResponse* pxMaterialResponse = (Tools_MaterialResponse*)pData;

	if (bValue)
	{
		pxMaterialResponse->SetProjectContext(pxMaterialResponse->GetProjectContext() | AVP_MATERIAL_RESPONSE_PROJECT_FLAG_ALIEN);
		pxMaterialResponse->SetModified(true);
	}
	else 
	{
		pxMaterialResponse->SetProjectContext(pxMaterialResponse->GetProjectContext() & ~AVP_MATERIAL_RESPONSE_PROJECT_FLAG_ALIEN);
		pxMaterialResponse->SetModified(true);
	}
}

static bool IsHotChecked(void *pData)
{
	Tools_MaterialResponse* pxMaterialResponse = (Tools_MaterialResponse*)pData;

	return (pxMaterialResponse->GetProjectContext() & AVP_MATERIAL_RESPONSE_PROJECT_FLAG_HOT) != 0;
}
static void SetHotChecked(bool bValue, void *pData)
{
	Tools_MaterialResponse* pxMaterialResponse = (Tools_MaterialResponse*)pData;

	if (bValue)
	{
		pxMaterialResponse->SetProjectContext(pxMaterialResponse->GetProjectContext() | AVP_MATERIAL_RESPONSE_PROJECT_FLAG_HOT);
		pxMaterialResponse->SetModified(true);
	}
	else 
	{
		pxMaterialResponse->SetProjectContext(pxMaterialResponse->GetProjectContext() & ~AVP_MATERIAL_RESPONSE_PROJECT_FLAG_HOT);
		pxMaterialResponse->SetModified(true);
	}
}

static bool IsTechChecked(void *pData)
{
	Tools_MaterialResponse* pxMaterialResponse = (Tools_MaterialResponse*)pData;

	return (pxMaterialResponse->GetProjectContext() & AVP_MATERIAL_RESPONSE_PROJECT_FLAG_TECH) != 0;
}
static void SetTechChecked(bool bValue, void *pData)
{
	Tools_MaterialResponse* pxMaterialResponse = (Tools_MaterialResponse*)pData;

	if (bValue)
	{
		pxMaterialResponse->SetProjectContext(pxMaterialResponse->GetProjectContext() | AVP_MATERIAL_RESPONSE_PROJECT_FLAG_TECH);
		pxMaterialResponse->SetModified(true);
	}
	else 
	{
		pxMaterialResponse->SetProjectContext(pxMaterialResponse->GetProjectContext() & ~AVP_MATERIAL_RESPONSE_PROJECT_FLAG_TECH);
		pxMaterialResponse->SetModified(true);
	}
}

static bool IsHologramChecked(void *pData)
{
	Tools_MaterialResponse* pxMaterialResponse = (Tools_MaterialResponse*)pData;

	return (pxMaterialResponse->GetProjectContext() & AVP_MATERIAL_RESPONSE_PROJECT_FLAG_HOLOGRAM) != 0;
}
static void SetHologramChecked(bool bValue, void *pData)
{
	Tools_MaterialResponse* pxMaterialResponse = (Tools_MaterialResponse*)pData;

	if (bValue)
	{
		pxMaterialResponse->SetProjectContext(pxMaterialResponse->GetProjectContext() | AVP_MATERIAL_RESPONSE_PROJECT_FLAG_HOLOGRAM);
		pxMaterialResponse->SetModified(true);
	}
	else 
	{
		pxMaterialResponse->SetProjectContext(pxMaterialResponse->GetProjectContext() & ~AVP_MATERIAL_RESPONSE_PROJECT_FLAG_HOLOGRAM);
		pxMaterialResponse->SetModified(true);
	}
}



void ToolsAVP_MaterialResponse_AddProjectUI(Tools_MaterialResponse* const pxMaterialResponse, Asura_Tabs_Dialog *pxParentDialog, Asura_Vector_2 xOffset)
{
	Asura_Dialog *pxDialog;

	pxDialog = new Asura_Dialog( "Vision Mode Flags", xOffset);
	pxParentDialog->AddDialog( pxDialog );
	xOffset.y += Asura_FlagTickbox_Dialog::DefaultSpacing();

	pxDialog = new Asura_FunctionTickbox_Dialog( "Hot", xOffset, pxMaterialResponse, IsHotChecked, SetHotChecked );
	pxDialog->SetHelpText( "If ticked, the material will be treated as Hot for the vision modes." );
	pxParentDialog->AddDialog( pxDialog );
	xOffset.y += Asura_FlagTickbox_Dialog::DefaultSpacing();

	pxDialog = new Asura_FunctionTickbox_Dialog( "Alien", xOffset, pxMaterialResponse, IsAlienChecked, SetAlienChecked );
	pxDialog->SetHelpText( "If ticked, the material will be treated as an alien for the vision modes." );
	pxParentDialog->AddDialog( pxDialog );
	xOffset.y += Asura_FlagTickbox_Dialog::DefaultSpacing();

	pxDialog = new Asura_FunctionTickbox_Dialog( "Tech", xOffset, pxMaterialResponse, IsTechChecked, SetTechChecked );
	pxDialog->SetHelpText( "If ticked, the material will be treated as Tech for the vision modes." );
	pxParentDialog->AddDialog( pxDialog );
	xOffset.y += Asura_FlagTickbox_Dialog::DefaultSpacing();

	pxDialog = new Asura_FunctionTickbox_Dialog( "Hologram", xOffset, pxMaterialResponse, IsHologramChecked, SetHologramChecked );
	pxDialog->SetHelpText( "If ticked, the material will be treated as Hologram for the vision modes." );
	pxParentDialog->AddDialog( pxDialog );
	xOffset.y += Asura_FlagTickbox_Dialog::DefaultSpacing();

}



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Texturing_MaterialResponse::Project_CreateSurfaceTypeDialog
// Author       : Andy Tate
// Date         : 6-Aug-2007
////////////////////////////////////////////////////////////////////////////////

Asura_Hash_DropDownList_Dialog* Tools_Texturing_MaterialResponse::Project_CreateSurfaceTypeDialog(Asura_Vector_2& xOffset,
																								  const int iNumberOfLines,
																								  Asura_Hash_ID* puSelected,
																								  bool* pbChangedFlag,
																								  const char* szName)
{
	return(new ToolsAVP_MaterialSurfaceType_DropDownListDialog (xOffset, iNumberOfLines, puSelected, pbChangedFlag, szName));
}

/// End of function 'Tools_Texturing_MaterialResponse::Project_CreateSurfaceTypeDialog'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Texturing_MaterialResponse::Project_ValidateSurfaceTypeHash
// Author       : Andy Tate
// Date         : 6-Aug-2007
////////////////////////////////////////////////////////////////////////////////

bool Tools_Texturing_MaterialResponse::Project_ValidateSurfaceTypeHash(Asura_Hash_ID& uHash)
{
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_SURFACEEFFECTS, uAVP_BLUEPRINT_NAME_SURFACETYPE );
	if(pxBlueprint == NULL)
	{
		uHash	= ASURA_HASH_ID_UNSET;
		return(false);
	}
	else
	{
		Asura_Parameter* pxParam = pxBlueprint->GetParameterByHashID(uHash);

		return(pxParam!=NULL);
	}
}

/// End of function 'Tools_Texturing_MaterialResponse::Project_ValidateSurfaceTypeHash'. ///


/// End of file. ///
