//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_AnimTemplates.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// This file's header.
#include "Axon_Gamescene_AnimTemplates.h"

//Asura
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

//Axon
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"
#include "AnimTemplates/Axon_GS2_AnimPackage.h"

#ifdef ASURA_TOOLS
#include "ChunkSystem/Asura_Chunk_Loaders.h"

#include "AnimTemplates/Tools_Mode_AnimTemplates.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Misc/Tools_MessageLog.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TOOLS
const char* s_pszAnimTemplateTypeStrings[ NUM_AXON_ANIM_TEMPLATE_TYPES ] =
{
	"Static",
	"Looping",
	"Rigid",
	"Fidget"
};
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_AnimTemplate_Entry
//
//////////////////////////////////////////////////////////////////////////////

Axon_Gamescene_AnimTemplate_Entry::Axon_Gamescene_AnimTemplate_Entry()
: m_uAnimTemplateFlags( 0 )
, m_fTweenTime( DEFAULT_TWEEN_TIME )
, m_pxNextEntry( NULL )
, m_uAnimHashIDOrUniqueID( ASURA_HASH_ID_UNSET )
, m_uAnimFilenameHashID( ASURA_HASH_ID_UNSET )
{
	m_bTween = true;
}

void Axon_Gamescene_AnimTemplate_Entry::ReadFromChunkStream( Asura_Chunk_Stream& xStream, const Axon_Gamescene_AnimTemplate& xParentTemplate )
{
	// Version
	int iVersion;
	xStream >> iVersion;

    // Parent, if we had one.

	// Us.
	if ( iVersion == CURRENT_CHUNK_VERSION )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uAnimTemplateFlags );
		xStream >> xFlagsBitfield;

		xStream >> m_fTweenTime;

		xStream >> m_uAnimHashIDOrUniqueID;
		xStream >> m_uAnimFilenameHashID;
	}
	else if ( iVersion == 1 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uAnimTemplateFlags );
		xStream >> xFlagsBitfield;

		xStream >> m_fTweenTime;

		xStream >> m_uAnimHashIDOrUniqueID;
		xStream >> m_uAnimFilenameHashID;
		Asura_Hash_ID uDummy;
		xStream >> uDummy;
		xStream >> uDummy;

		m_bContainsAnim = true;
	}
	else if ( iVersion == 0 )
	{
		Asura_ReadFromStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uAnimTemplateFlags );
		xStream >> xFlagsBitfield;

		xStream >> m_uAnimHashIDOrUniqueID;
		xStream >> m_uAnimFilenameHashID;
		Asura_Hash_ID uDummy;
		xStream >> uDummy;
		xStream >> uDummy;

		m_bContainsAnim = true;
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Gamescene_AnimTemplate_Entry::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

#ifdef ASURA_TOOLS
void Axon_Gamescene_AnimTemplate_Entry::WriteToChunkStream(	Asura_Chunk_Stream& xStream ) const
{
	//Version
	xStream << static_cast< int >( CURRENT_CHUNK_VERSION );

	//Parent, if we had one

	//Us.
	Asura_WriteToStream_Bitfield_Wrapper< u_int > xFlagsBitfield( m_uAnimTemplateFlags );
	xFlagsBitfield.Add( m_bLoop,              1 );
	xFlagsBitfield.Add( m_bTween,             1 );
	xFlagsBitfield.Add( m_bBackwards,         1 );
	xFlagsBitfield.Add( m_bLoopTimeInSeconds, 1 );
	xFlagsBitfield.Add( m_xDeathType,         1 );
	xFlagsBitfield.Add( m_bMirror,            1 );
	xFlagsBitfield.Add( m_bContainsAnim,	  1 );
	xStream << xFlagsBitfield;

	xStream << m_fTweenTime;

	xStream << m_uAnimHashIDOrUniqueID;
	xStream << m_uAnimFilenameHashID;
}

void Axon_Gamescene_AnimTemplate_Entry::Copy( const Axon_Gamescene_AnimTemplate_Entry* pxTemplateEntry )
{
	m_uAnimTemplateFlags       = pxTemplateEntry->m_uAnimTemplateFlags;
	m_fTweenTime               = pxTemplateEntry->m_fTweenTime;
	m_uAnimHashIDOrUniqueID    = pxTemplateEntry->m_uAnimHashIDOrUniqueID;
	m_uAnimFilenameHashID      = pxTemplateEntry->m_uAnimFilenameHashID;
}

void Axon_Gamescene_AnimTemplate_Entry::LoadAnims() const
{
	if ( ContainsAnim() )
	{
		if ( GetAnimHashID() != ASURA_HASH_ID_UNSET && !Asura_Animation_System::GetAnimByHash( GetAnimHashID() ) )
		{
			const char* const szFilename = Asura_HashedFilename_System::GetFilename( GetAnimFilenameHashID() );
			if ( szFilename )
			{
				LoadAsuraFile( szFilename );
			}
		}
	}
	else
	{
		const Axon_GS2_AnimPackage* pxAnimPackage = Axon_Gamescene_AnimTemplate_System::GetAnimPackageByID( GetAnimPackageUniqueID() );
		if ( pxAnimPackage )
		{
			pxAnimPackage->LoadAnims();
		}
	}
}

void Axon_Gamescene_AnimTemplate_Entry::AddFilesToExportList( List< Asura_String >& xNameList ) const
{
	if ( ContainsAnim() )
	{
		if ( GetAnimFilenameHashID() != ASURA_HASH_ID_UNSET )
		{
			Asura_String xFilename( Asura_HashedFilename_System::GetFilename( GetAnimFilenameHashID() ) );
			if ( !xFilename.IsBlank() )
			{
				Tools_Level_Export::AddFileToExportList( xNameList, xFilename, false );
			}
		}
	}
	else
	{
		const Axon_GS2_AnimPackage* pxAnimPackage = Axon_Gamescene_AnimTemplate_System::GetAnimPackageByID( GetAnimPackageUniqueID() );
		if ( pxAnimPackage )
		{
			pxAnimPackage->AddFilesToExportList( xNameList );
		}
	}
}

bool Axon_Gamescene_AnimTemplate_Entry::Verify( const Axon_Gamescene_AnimTemplate& xParentTemplate ) const
{
	bool bOk = true;

	if ( ContainsAnim() )
	{
		if ( GetAnimHashID() != ASURA_HASH_ID_UNSET && GetAnimFilenameHashID() != ASURA_HASH_ID_UNSET )
		{
			const Asura_Hierarchy_Anim* pxAnim = Tools_Animation_Files::GetAnimFromID( GetAnimHashID(), GetAnimFilenameHashID() );
			if ( !pxAnim )
			{
				Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Anim Templates",
					new Tools_MessageLogAction_OpenAnimTemplate( xParentTemplate.GetUniqueID() ),
					"Could not find an animation for anim template '%s'. Has it been deleted?", xParentTemplate.GetName() );
				bOk = false;
			}
		}
		else
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Anim Templates",
				new Tools_MessageLogAction_OpenAnimTemplate( xParentTemplate.GetUniqueID() ),
				"Anim template '%s' has an entry without an animation.", xParentTemplate.GetName() );
			bOk = false;
		}
	}
	else
	{
		const Axon_GS2_AnimPackage* pxAnimPackage = Axon_Gamescene_AnimTemplate_System::GetAnimPackageByID( GetAnimPackageUniqueID() );
		if ( pxAnimPackage )
		{
			bOk &= pxAnimPackage->Verify();
		}
		else
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Anim Templates",
				new Tools_MessageLogAction_OpenAnimTemplate( xParentTemplate.GetUniqueID() ),
				"Could not find an anim package for anim template '%s'. Has it been deleted?", xParentTemplate.GetName() );
			bOk = false;
		}
	}

	return bOk;
}

bool Axon_Gamescene_AnimTemplate_Entry::WillFinish() const
{
	if ( m_bLoop )
	{
		// If m_fUserSetDuration is zero we will loop forever.
		return ( GetUserSetDuration() != 0.0f );
	}
	else
	{
		return true;
	}
}

void Axon_Gamescene_AnimTemplate_Entry::SetAnim( Asura_Hash_ID uAnimHashID, Asura_Hash_ID uAnimFilenameHashID )
{
	m_bContainsAnim = true;
	m_uAnimHashIDOrUniqueID = uAnimHashID;
	m_uAnimFilenameHashID = uAnimFilenameHashID;

	LoadAnims();
	//Tools_Animation_Files::GetAnimFromID( m_uAnimHashIDOrUniqueID, m_uAnimFilenameHashID );
}

void Axon_Gamescene_AnimTemplate_Entry::SetAnimPackage( u_int uUniqueID )
{
	m_bContainsAnim = false;
	m_uAnimHashIDOrUniqueID = uUniqueID;
	m_uAnimFilenameHashID = ASURA_HASH_ID_UNSET;

	LoadAnims();
}
#endif // ASURA_TOOLS

float Axon_Gamescene_AnimTemplate_Entry::GetAnimationDuration( const void* pMappingData /*= NULL*/ ) const
{
	const Asura_Hierarchy_Anim* pxHierAnim = Asura_Animation_System::GetAnimByHash( CalcAnimHashIDToPlay( pMappingData ) );
	return pxHierAnim ? pxHierAnim->Duration : 0.0f;
}

Asura_Hash_ID Axon_Gamescene_AnimTemplate_Entry::CalcAnimHashIDToPlay( const void* pMappingData /*= NULL*/, Asura_Hash_ID* puAnimFilenameHashID /*= NULL*/ ) const
{
	if ( ContainsAnim() )
	{
		if ( puAnimFilenameHashID )
		{
			*puAnimFilenameHashID = GetAnimFilenameHashID();
		}
		return GetAnimHashID();
	}
	else
	{
		Axon_GS2_AnimPackage* pxAnimPackage = Axon_Gamescene_AnimTemplate_System::GetAnimPackageByID( GetAnimPackageUniqueID() );
		return pxAnimPackage ? pxAnimPackage->GetAnim( pMappingData, puAnimFilenameHashID ) : ASURA_HASH_ID_UNSET;
	}
}

Asura_Hash_ID Axon_Gamescene_AnimTemplate_Entry::GetAnimHashID() const
{
	Asura_Assert( m_bContainsAnim, "Axon_Gamescene_AnimTemplate_Entry::GetAnimHashID - we don't contain an anim" );
	return m_uAnimHashIDOrUniqueID;
}

u_int Axon_Gamescene_AnimTemplate_Entry::GetAnimPackageUniqueID() const
{
	Asura_Assert( !m_bContainsAnim, "Axon_Gamescene_AnimTemplate_Entry::GetAnimHashID - we don't contain an anim mapping" );
	return m_uAnimHashIDOrUniqueID;
}

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_StaticAnimTemplate_Entry
//
//////////////////////////////////////////////////////////////////////////////

Axon_Gamescene_StaticAnimTemplate_Entry::Axon_Gamescene_StaticAnimTemplate_Entry()
: PARENT()
, m_fUserSetDuration( 0.0f )
, m_uFidgetBehaviourID( UINT_MAX )
{
}

void Axon_Gamescene_StaticAnimTemplate_Entry::ReadFromChunkStream( Asura_Chunk_Stream& xStream, const Axon_Gamescene_AnimTemplate& xParentTemplate )
{
	// Version
	int iVersion;
	xStream >> iVersion;

    // Parent
    PARENT::ReadFromChunkStream( xStream, xParentTemplate );

	if ( iVersion == CURRENT_CHUNK_VERSION )
	{
		// Us.
		xStream >> m_fUserSetDuration;
		xStream >> m_uFidgetBehaviourID;
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Gamescene_StaticAnimTemplate_Entry::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

u_int Axon_Gamescene_StaticAnimTemplate_Entry::GetFidgetBehaviourID() const 
{
	return ( m_fUserSetDuration == 0.0f ) ? m_uFidgetBehaviourID : uAXON_ANIMATION_SEQUENCE_ID_NONE;
}

#ifdef ASURA_TOOLS
void Axon_Gamescene_StaticAnimTemplate_Entry::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//version
	xStream << static_cast< int >( CURRENT_CHUNK_VERSION );

	//parent
	PARENT::WriteToChunkStream(	xStream );

	//Us.
	xStream << m_fUserSetDuration;
	xStream << m_uFidgetBehaviourID;
}

void Axon_Gamescene_StaticAnimTemplate_Entry::Copy( const Axon_Gamescene_AnimTemplate_Entry* pxTemplateEntry )
{
	PARENT::Copy( pxTemplateEntry );

	const Axon_Gamescene_StaticAnimTemplate_Entry* pxStatic = static_cast< const Axon_Gamescene_StaticAnimTemplate_Entry* >( pxTemplateEntry );
	m_fUserSetDuration   = pxStatic->m_fUserSetDuration;
	m_uFidgetBehaviourID = pxStatic->m_uFidgetBehaviourID;
}

void Axon_Gamescene_StaticAnimTemplate_Entry::LoadAnims() const
{
	PARENT::LoadAnims();

	const Axon_Gamescene_AnimTemplate* const pxFidget = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( GetFidgetBehaviourID() );
	if ( pxFidget )
	{
		pxFidget->LoadAnims();
	}
}

bool Axon_Gamescene_StaticAnimTemplate_Entry::Verify( const Axon_Gamescene_AnimTemplate& xParentTemplate ) const
{
	bool bOk = PARENT::Verify( xParentTemplate );

	// Check our fidget anim template exists, if we have one.
	if ( m_uFidgetBehaviourID != uAXON_ANIMATION_SEQUENCE_ID_NONE )
	{
		const Axon_Gamescene_AnimTemplate* pxFidgetTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( m_uFidgetBehaviourID );
		if ( !pxFidgetTemplate )
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Anim Templates",
				new Tools_MessageLogAction_OpenAnimTemplate( xParentTemplate.GetUniqueID() ),
				"Anim template '%s' has an entry that cannot find its fidget anim template.", xParentTemplate.GetName() );
			bOk = false;
		}
	}

	return bOk;
}
#endif	// ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_LoopingAnimTemplate_Entry
//
//////////////////////////////////////////////////////////////////////////////

Axon_Gamescene_LoopingAnimTemplate_Entry::Axon_Gamescene_LoopingAnimTemplate_Entry()
: PARENT()
, m_fUserSetDuration( 0.0f )
{
}

void Axon_Gamescene_LoopingAnimTemplate_Entry::ReadFromChunkStream( Asura_Chunk_Stream& xStream, const Axon_Gamescene_AnimTemplate& xParentTemplate )
{
	// Version
	int iVersion;
	xStream >> iVersion;

    // Parent
    PARENT::ReadFromChunkStream( xStream, xParentTemplate );

	if ( iVersion == CURRENT_CHUNK_VERSION )
	{
		// Us.
		xStream >> m_fUserSetDuration;
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Gamescene_LoopingAnimTemplate_Entry::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

#ifdef ASURA_TOOLS
void Axon_Gamescene_LoopingAnimTemplate_Entry::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//version
	xStream << static_cast< int >( CURRENT_CHUNK_VERSION );

	//parent
	PARENT::WriteToChunkStream( xStream );

	//us
	xStream << m_fUserSetDuration;
}

void Axon_Gamescene_LoopingAnimTemplate_Entry::Copy( const Axon_Gamescene_LoopingAnimTemplate_Entry* pxTemplateEntry )
{
	PARENT::Copy( pxTemplateEntry );

	const Axon_Gamescene_LoopingAnimTemplate_Entry* pxLooping = static_cast< const Axon_Gamescene_LoopingAnimTemplate_Entry* >( pxTemplateEntry );
	m_fUserSetDuration = pxLooping->m_fUserSetDuration;
}
#endif // ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_RigidAnimTemplate_Entry
//
//////////////////////////////////////////////////////////////////////////////

void Axon_Gamescene_RigidAnimTemplate_Entry::ReadFromChunkStream( Asura_Chunk_Stream& xStream, const Axon_Gamescene_AnimTemplate& xParentTemplate )
{
	// Version.
	int iVersion;
	xStream >> iVersion;

    // Parent.
    PARENT::ReadFromChunkStream( xStream, xParentTemplate );

	// Us.
	if ( iVersion == CURRENT_CHUNK_VERSION )
	{
	}
	else if ( iVersion == 0 )
	{
		Asura_Matrix_3x3 xOrientationOffset;
		Asura_Vector_3   xPositionOffset;
		xStream >> xOrientationOffset;
		xStream >> xPositionOffset;
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Gamescene_RigidAnimTemplate_Entry::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

#ifdef ASURA_TOOLS
void Axon_Gamescene_RigidAnimTemplate_Entry::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	// Version.
	xStream << static_cast< int >( CURRENT_CHUNK_VERSION );

	// Parent.
	PARENT::WriteToChunkStream( xStream );

	// Us.
}
#endif // ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_FidgetAnimTemplate_Entry
//
//////////////////////////////////////////////////////////////////////////////

Axon_Gamescene_FidgetAnimTemplate_Entry::Axon_Gamescene_FidgetAnimTemplate_Entry()
: PARENT()
, m_fFidgetWeight( 1.0f )
{
	m_bTween = true;
}

void Axon_Gamescene_FidgetAnimTemplate_Entry::ReadFromChunkStream( Asura_Chunk_Stream& xStream, const Axon_Gamescene_AnimTemplate& xParentTemplate )
{
	// Version.
	int iVersion;
	xStream >> iVersion;

    // Parent.
    PARENT::ReadFromChunkStream( xStream, xParentTemplate );

	// Us.
	if ( iVersion == CURRENT_CHUNK_VERSION )
	{
		xStream >> m_fFidgetWeight;
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Gamescene_FidgetAnimTemplate_Entry::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

#ifdef ASURA_TOOLS
void Axon_Gamescene_FidgetAnimTemplate_Entry::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//version
	xStream << static_cast< int >( CURRENT_CHUNK_VERSION );

	//parent
	PARENT::WriteToChunkStream( xStream );

	//us
	xStream << m_fFidgetWeight;
}

void Axon_Gamescene_FidgetAnimTemplate_Entry::Copy( const Axon_Gamescene_AnimTemplate_Entry* pxTemplateEntry )
{
	PARENT::Copy( pxTemplateEntry );

	const Axon_Gamescene_FidgetAnimTemplate_Entry *pxFidget = static_cast< const Axon_Gamescene_FidgetAnimTemplate_Entry* >( pxTemplateEntry );
	m_fFidgetWeight = pxFidget->m_fFidgetWeight;
}
#endif // ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_AnimTemplate
//
//////////////////////////////////////////////////////////////////////////////

Axon_Gamescene_AnimTemplate::Axon_Gamescene_AnimTemplate( const u_int uType )
: m_uTemplateType( uType )
, m_uUniqueID( UINT_MAX )
, m_pxFirstEntry( NULL )
#ifdef ASURA_TOOLS
, m_xPath()
, m_xTemplateName()
, m_bLocalOnlyAndWritable( false )
#endif
{
}

Axon_Gamescene_AnimTemplate::~Axon_Gamescene_AnimTemplate()
{
	Empty();
}

void Axon_Gamescene_AnimTemplate::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	// Version.
	int iVersion;
	xStream >> iVersion;

    // Parent, if we had one.

	// Our data.
	if ( iVersion == CURRENT_VERSION )
	{
        // We don't read the type in, as this is saved before us by the chunk creation stuff.
        // We will automatically have been created as the correct type.

        xStream >> m_uUniqueID;
		#ifdef ASURA_TOOLS
		xStream >> m_xTemplateName;
		#else
		Asura_String xDummy;
		xStream >> xDummy;
		#endif

		// Create a linked list of templates.
		int iNumberOfEntries;
		xStream >> iNumberOfEntries;
		if ( iNumberOfEntries )
		{
			Axon_Gamescene_AnimTemplate_Entry** ppxFirstEntry = &m_pxFirstEntry;
			while ( iNumberOfEntries-- )
			{
				switch ( m_uTemplateType )
				{
				    case AXON_ATT_STATIC:
					{
						*ppxFirstEntry = new Axon_Gamescene_StaticAnimTemplate_Entry();
						break;
					}
				    case AXON_ATT_LOOPINGMOVEMENT:
					{
						*ppxFirstEntry = new Axon_Gamescene_LoopingAnimTemplate_Entry();
						break;
					}
				    case AXON_ATT_RIGIDMOVEMENT:
					{
						*ppxFirstEntry = new Axon_Gamescene_RigidAnimTemplate_Entry();
						break;
					}
				    case AXON_ATT_FIDGETBEHAVIOUR:
					{
						*ppxFirstEntry = new Axon_Gamescene_FidgetAnimTemplate_Entry();
						break;
					}

				    default:
					{
						Asura_Assert(false, "Axon_Gamescene_AnimTemplate::ReadFromChunkStream - error, unknown animation template type");
						break;
					}
				}

                (*ppxFirstEntry)->ReadFromChunkStream( xStream, *this );

                ppxFirstEntry = &(*ppxFirstEntry)->m_pxNextEntry;
			}
		}
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Gamescene_AnimTemplate::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

void Axon_Gamescene_AnimTemplate::Empty()
{
	while (m_pxFirstEntry)
	{
		Axon_Gamescene_AnimTemplate_Entry* pxEntryToDelete = m_pxFirstEntry;
		m_pxFirstEntry = m_pxFirstEntry->GetNextEntry();
		delete pxEntryToDelete;
	}
}

u_int Axon_Gamescene_AnimTemplate::GetNumEntries() const
{
	u_int uCount = 0;
	Axon_Gamescene_AnimTemplate_Entry* pxCurrent = m_pxFirstEntry;
	while ( pxCurrent )
	{
		++uCount;
		pxCurrent = pxCurrent->GetNextEntry();
	}
	return uCount;
}

Axon_Gamescene_AnimTemplate_Entry* Axon_Gamescene_AnimTemplate::GetCurrentEntry( float fTime, const void* pMappingData, float& fTimeRemainder ) const
{
	Axon_Gamescene_AnimTemplate_Entry* pxCurrent = static_cast< Axon_Gamescene_AnimTemplate_Entry* >( m_pxFirstEntry );
	while ( pxCurrent )
	{
		if ( pxCurrent->m_pxNextEntry && pxCurrent->m_bTween )
		{
			fTime -= pxCurrent->m_fTweenTime;
			if ( fTime <= 0.0f )
			{
				fTimeRemainder = 0.0f;
				return pxCurrent;
			}
		}

		if ( pxCurrent->m_bLoop )
		{
			Asura_Assert( m_uTemplateType == AXON_ATT_STATIC || m_uTemplateType == AXON_ATT_LOOPINGMOVEMENT,
				"Axon_Gamescene_AnimTemplate::GetCurrentEntry - template should either be static or looping" );

			const float fTemplateDuration = pxCurrent->GetUserSetDuration();
			const Asura_Hierarchy_Anim* pxHierAnim = Asura_Animation_System::GetAnimByHash( pxCurrent->CalcAnimHashIDToPlay( pMappingData ) );
			const float fAnimDuration = pxHierAnim ? pxHierAnim->Duration : 0.0f;

			bool bWillFinish = false;
			if ( fTemplateDuration == 0.0f )
			{
				// Will loop forever.
				bWillFinish = true;
			}
			else if ( pxCurrent->m_bLoopTimeInSeconds )
			{
				if ( fTime <= fTemplateDuration )
				{
					bWillFinish = true;
				}
				else
				{
					fTime -= fTemplateDuration;
				}
			}
			else
			{
				if ( fTime <= fTemplateDuration * fAnimDuration )
				{
					bWillFinish = true;
				}
				else
				{
					fTime -= fTemplateDuration * fAnimDuration;
				}
			}

			if ( bWillFinish )
			{
				if ( fAnimDuration > 0.0f )
				{
					while ( fTime >= fAnimDuration ) { fTime -= fAnimDuration; }
					fTimeRemainder = fTime;
				}
				else
				{
					fTimeRemainder = 0.0f;
				}
				return pxCurrent;
			}
		}
		else
		{
			Asura_Hierarchy_Anim* pxHierAnim = Asura_Animation_System::GetAnimByHash( pxCurrent->CalcAnimHashIDToPlay( pMappingData ) );
			const float fAnimDuration = pxHierAnim ? pxHierAnim->Duration : 0.0f;

			if ( fTime <= fAnimDuration )
			{
				if ( fAnimDuration > 0.0f )
				{
					while ( fTime >= fAnimDuration ) { fTime -= fAnimDuration; }
					fTimeRemainder = fTime;
				}
				else
				{
					fTimeRemainder = 0.0f;
				}
				return pxCurrent;
			}
			else
			{
				fTime -= fAnimDuration;
			}
		}
		pxCurrent = pxCurrent->GetNextEntry();
	}
	return NULL;
}

float Axon_Gamescene_AnimTemplate::GetDuration( const void* pMappingData ) const
{
	float fTotalDuration = 0.0f;

	Axon_Gamescene_AnimTemplate_Entry* pxCurrent = static_cast< Axon_Gamescene_AnimTemplate_Entry* >( m_pxFirstEntry );
	while ( pxCurrent )
	{
		if ( pxCurrent->m_bLoop )
		{
			Asura_Assert(m_uTemplateType == AXON_ATT_STATIC || m_uTemplateType == AXON_ATT_LOOPINGMOVEMENT, "Sanity check: Template should either be static or looping");

			const float fDuration = pxCurrent->GetUserSetDuration();
			if ( fDuration == 0.0f )
			{
				// Will loop forever.
				return 0.0f;
			}
			else if ( pxCurrent->m_bLoopTimeInSeconds )
			{
				fTotalDuration += fDuration;
			}
			else
			{
				Asura_Hierarchy_Anim* pxHierAnim = Asura_Animation_System::GetAnimByHash( pxCurrent->CalcAnimHashIDToPlay( pMappingData ) );
				if ( pxHierAnim )
				{
					fTotalDuration += fDuration * pxHierAnim->Duration;
				}
			}
		}
		else
		{
			Asura_Hierarchy_Anim* pxHierAnim = Asura_Animation_System::GetAnimByHash( pxCurrent->CalcAnimHashIDToPlay( pMappingData ) );
			if ( pxHierAnim )
			{
				fTotalDuration += pxHierAnim->Duration;
			}
		}
		pxCurrent = pxCurrent->GetNextEntry();
	}

	return fTotalDuration;
}

void Axon_Gamescene_AnimTemplate::GetMovementOffset( float fTime, const void* pMappingData, Asura_Vector_3& xPosOffset, Asura_Matrix_3x3& xOriOffset ) const
{
	xPosOffset.Zero();
	xOriOffset.Identity();
}

void Axon_Gamescene_AnimTemplate::GetMovementVelocity( float fTime, const void* pMappingData, Asura_Vector_3& xVelocity ) const
{
    xVelocity.Zero();

    Axon_Gamescene_AnimTemplate_Entry* pxCurrentEntry = static_cast< Axon_Gamescene_AnimTemplate_Entry* >( m_pxFirstEntry );
    Asura_Hierarchy_Anim*              pxCurrentAnim  = pxCurrentEntry ?
														Asura_Animation_System::GetAnimByHash( pxCurrentEntry->CalcAnimHashIDToPlay( pMappingData ) ) :
														NULL;

    while ( pxCurrentAnim )
    {
        // Chip away at the time until it falls inside the current animation.

        // If the entry loops and is of the correct type then we need to incorporate the looping into the time.
        if ( pxCurrentEntry->m_bLoop && ( m_uTemplateType == AXON_ATT_STATIC || m_uTemplateType == AXON_ATT_LOOPINGMOVEMENT ) )
        {
            float fDuration = pxCurrentEntry->GetUserSetDuration();
            if ( fDuration == 0.0f )
            {
                // We should loop forever. Put the duration just above the time left so the following calculation operates correctly.
                while ( fDuration < fTime )
                {
                    fDuration += pxCurrentAnim->Duration;
                }
            }
            else if ( !pxCurrentEntry->m_bLoopTimeInSeconds )
            {
                // The loop 'time' is in number of loops - resolve this to number of seconds.
                const int iNumLoops = Asura_Maths::FloatToInt_Near( fDuration );
                fDuration = iNumLoops * pxCurrentAnim->Duration;
            }

            if ( fTime <= fDuration )
            {
                while ( fTime > pxCurrentAnim->Duration )
                {
                    fTime -= pxCurrentAnim->Duration;
                }
                break;
            }
            else
            {
                fTime -= fDuration;
            }
        }
        else
        {
            // This entry does not loop.
            if ( fTime <= pxCurrentAnim->Duration )
            {
                break;
            }
            else
            {
                fTime -= pxCurrentAnim->Duration;
            }
        }

        // Move on to the next entry and anim.
        pxCurrentEntry = pxCurrentEntry->GetNextEntry();
        pxCurrentAnim  = pxCurrentEntry ? Asura_Animation_System::GetAnimByHash( pxCurrentEntry->CalcAnimHashIDToPlay( pMappingData ) ) : NULL;
#ifdef ASURA_TOOLS
		if ( !pxCurrentAnim && pxCurrentEntry )
		{
			Asura_Hash_ID uAnimFilenameHashID = ASURA_HASH_ID_UNSET;
			Asura_Hash_ID uAnimHashID = pxCurrentEntry->CalcAnimHashIDToPlay( pMappingData, &uAnimFilenameHashID );
			pxCurrentAnim = Tools_Animation_Files::GetAnimFromID( uAnimHashID, uAnimFilenameHashID );
		}
#endif    
	}

    if ( fTime >= 0.0f && pxCurrentEntry && pxCurrentAnim )
    {
        const float fProportion = pxCurrentEntry->m_bBackwards ?
            Asura_Maths::ClampToRange( ( 1.0f - fTime ) / pxCurrentAnim->Duration, 0.0f, 1.0f ) :
            Asura_Maths::ClampToRange( fTime / pxCurrentAnim->Duration, 0.0f, 1.0f );
#ifdef ASURA_ASSERT
        const bool bGotVelocity =
#endif
            pxCurrentAnim->GetVelocity( NULL, fProportion, pxCurrentAnim->Duration, &xVelocity );
#ifdef ASURA_ASSERT
        Asura_Assert( bGotVelocity, "Axon_Gamescene_AnimTemplate::GetMovementVelocity - could not get velocity from animation (not exported with movement offsets?)" );
#endif
    }
}

#ifdef ASURA_TOOLS
Axon_Gamescene_AnimTemplate::Axon_Gamescene_AnimTemplate( const Asura_String& xName, const u_int uType )
: m_uTemplateType( uType )
, m_uUniqueID( CalculateUniqueID( xName ) )
, m_pxFirstEntry( NULL )
, m_xPath()
, m_xTemplateName( xName )
, m_bLocalOnlyAndWritable( false )
{
}

bool Axon_Gamescene_AnimTemplate::WillFinish() const
{
	Axon_Gamescene_AnimTemplate_Entry* pxCurrent = m_pxFirstEntry;
	while ( pxCurrent )
	{
		if ( !pxCurrent->WillFinish() )
		{
			return false;
		}
		pxCurrent = pxCurrent->GetNextEntry();
	}
	return true;
}

void Axon_Gamescene_AnimTemplate::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//Version
	xStream << static_cast< int >( CURRENT_VERSION );

	//Parent, if we had one

	// We don't save the type out, as this is saved before us by the chunk creation stuff.
	xStream << m_uUniqueID;
	xStream << m_xTemplateName;

    // First count the number of entries and write this out.
	int iNumberOfEntries = 0;
    Axon_Gamescene_AnimTemplate_Entry* pxEntry = m_pxFirstEntry;
    while ( pxEntry )
    {
        ++iNumberOfEntries;
        pxEntry = pxEntry->GetNextEntry();
    }
	xStream << iNumberOfEntries;

    // Now write the entries out.
    pxEntry = m_pxFirstEntry;
    while ( pxEntry )
    {
        pxEntry->WriteToChunkStream( xStream );
        pxEntry = pxEntry->GetNextEntry();
    }
}

void Axon_Gamescene_AnimTemplate::Copy( const Axon_Gamescene_AnimTemplate* pxTemplate )
{
	Empty();
	int iNumberOfEntries = 0;
    {
    Axon_Gamescene_AnimTemplate_Entry* pxEntry = pxTemplate->m_pxFirstEntry;
	while ( pxEntry )
	{
		++iNumberOfEntries;
        pxEntry = pxEntry->GetNextEntry();
	}
    }

	Axon_Gamescene_AnimTemplate_Entry** ppxEntry      = &m_pxFirstEntry;
	Axon_Gamescene_AnimTemplate_Entry*  pxEntryToCopy = pxTemplate->m_pxFirstEntry;
	for ( int i = 0; i < iNumberOfEntries; i++ )
	{
		switch ( m_uTemplateType )
		{
		    case AXON_ATT_STATIC:
			{
				*ppxEntry = new Axon_Gamescene_StaticAnimTemplate_Entry();
				break;
			}
		    case AXON_ATT_LOOPINGMOVEMENT:
			{
				*ppxEntry = new Axon_Gamescene_LoopingAnimTemplate_Entry();
				break;
			}
		    case AXON_ATT_RIGIDMOVEMENT:
			{
				*ppxEntry = new Axon_Gamescene_RigidAnimTemplate_Entry();
				break;
			}
		    case AXON_ATT_FIDGETBEHAVIOUR:
			{
				*ppxEntry = new Axon_Gamescene_FidgetAnimTemplate_Entry();
				break;
			}
		    default:
			{
				Asura_Assert(false, "Error, unknown animation template type");
				break;
			}
		}

		Asura_Assert( pxEntryToCopy, "Error, Unhandled NULL pointer" );
		(*ppxEntry)->Copy( pxEntryToCopy );
		pxEntryToCopy = pxEntryToCopy->GetNextEntry();
        ppxEntry = &((*ppxEntry)->m_pxNextEntry);
	}
}

void Axon_Gamescene_AnimTemplate::AddFilesToExportList( List< Asura_String >& xNameList ) const
{
	const Axon_Gamescene_AnimTemplate_Entry* pxEntry = m_pxFirstEntry;
	while ( pxEntry )
	{
		pxEntry->AddFilesToExportList( xNameList );

		if ( m_uTemplateType == AXON_ATT_STATIC )
		{
			const Axon_Gamescene_StaticAnimTemplate_Entry* const pxStatic = static_cast<const Axon_Gamescene_StaticAnimTemplate_Entry*>( pxEntry );
			const Axon_Gamescene_AnimTemplate* const pxFidget = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( pxStatic->GetFidgetBehaviourID() );
			if ( pxFidget )
			{
				pxFidget->AddFilesToExportList( xNameList );
			}
		}

		pxEntry = pxEntry->GetNextEntry();
	}
}

bool Axon_Gamescene_AnimTemplate::Verify() const
{
	bool bOk = true;
	const Axon_Gamescene_AnimTemplate_Entry* pxEntry = m_pxFirstEntry;
	while ( pxEntry )
	{
		bOk &= pxEntry->Verify( *this );
		pxEntry = pxEntry->GetNextEntry();
	}
	return bOk;
}

void Axon_Gamescene_AnimTemplate::LoadAnims() const
{
	const Axon_Gamescene_AnimTemplate_Entry* pxEntry = m_pxFirstEntry;
	while ( pxEntry )
	{
		pxEntry->LoadAnims();
		pxEntry = pxEntry->GetNextEntry();
	}
}

u_int Axon_Gamescene_AnimTemplate::CalculateUniqueID( const Asura_String& xName )
{
	u_int uUniqueID = Asura_GetHashID( xName.GetString() );
	// Make sure it really is unique.
	while ( Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( uUniqueID ) ) { ++uUniqueID; }
	return uUniqueID;
}
#endif // ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_StaticAnimTemplate
//
//////////////////////////////////////////////////////////////////////////////

Axon_Gamescene_StaticAnimTemplate::Axon_Gamescene_StaticAnimTemplate()
: PARENT( AXON_ATT_STATIC )
{
}

Axon_Gamescene_StaticAnimTemplate::~Axon_Gamescene_StaticAnimTemplate()
{
}

void Axon_Gamescene_StaticAnimTemplate::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	//Version
	int iVersion;
	xStream >> iVersion;

	//Parent
	PARENT::ReadFromChunkStream( xStream );

	if ( iVersion == CURRENT_VERSION )
	{
		//Us
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Gamescene_StaticAnimTemplate::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

#ifdef ASURA_TOOLS
Axon_Gamescene_StaticAnimTemplate::Axon_Gamescene_StaticAnimTemplate( const Asura_String& xName )
: PARENT( xName, AXON_ATT_STATIC )
{
}

void Axon_Gamescene_StaticAnimTemplate::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//Version
	xStream << static_cast< int >( CURRENT_VERSION );

	//Parent
	PARENT::WriteToChunkStream( xStream );

	//Us
}
#endif	//ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_LoopingAnimTemplate
//
//////////////////////////////////////////////////////////////////////////////

Axon_Gamescene_LoopingAnimTemplate::Axon_Gamescene_LoopingAnimTemplate()
: PARENT( AXON_ATT_LOOPINGMOVEMENT )
{
}

Axon_Gamescene_LoopingAnimTemplate::~Axon_Gamescene_LoopingAnimTemplate()
{
}

void Axon_Gamescene_LoopingAnimTemplate::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	//Version
	int iVersion;
	xStream >> iVersion;

    //Parent
    PARENT::ReadFromChunkStream( xStream );

    if ( iVersion == CURRENT_VERSION )
	{
		//Us
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Gamescene_LoopingAnimTemplate::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

#ifdef ASURA_TOOLS
Axon_Gamescene_LoopingAnimTemplate::Axon_Gamescene_LoopingAnimTemplate( const Asura_String& xName )
: PARENT( xName, AXON_ATT_LOOPINGMOVEMENT )
{
}

void Axon_Gamescene_LoopingAnimTemplate::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//Version
	xStream << static_cast< int >( CURRENT_VERSION );

	//Parent
	PARENT::WriteToChunkStream( xStream );

	//Us
}

bool Axon_Gamescene_LoopingAnimTemplate::Verify() const
{
	bool bOk = PARENT::Verify();

	// Check we won't finish.
	if ( WillFinish() )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Anim Templates",
			new Tools_MessageLogAction_OpenAnimTemplate( GetUniqueID() ),
			"Anim template '%s' is a looping template but will not loop forever.", GetName() );
		bOk = false;
	}

	return bOk;
}
#endif // ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_RigidAnimTemplate
//
//////////////////////////////////////////////////////////////////////////////

Axon_Gamescene_RigidAnimTemplate::Axon_Gamescene_RigidAnimTemplate()
: PARENT( AXON_ATT_RIGIDMOVEMENT )
{
}

Axon_Gamescene_RigidAnimTemplate::~Axon_Gamescene_RigidAnimTemplate()
{
}

void Axon_Gamescene_RigidAnimTemplate::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	//Version
	int iVersion;
	xStream >> iVersion;

    //Parent
    PARENT::ReadFromChunkStream( xStream );

    if ( iVersion == CURRENT_VERSION )
	{
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Gamescene_RigidAnimTemplate::ReadFromChunkStream - unhandled chunk version (%d)", iVersion ) );
	}
}

void Axon_Gamescene_RigidAnimTemplate::GetMovementOffset( float fTime, const void* pScoringData, Asura_Vector_3& xPosOffset, Asura_Matrix_3x3& xOriOffset ) const
{
	xPosOffset.Zero();
	xOriOffset.Identity();

	Axon_Gamescene_AnimTemplate_Entry* pxCurrent = static_cast< Axon_Gamescene_AnimTemplate_Entry* >( m_pxFirstEntry );
	while ( pxCurrent )
	{
		//if ( pxCurrent->m_bTween )
		//{
		//	fTime -= DEFAULT_TWEEN_TIME;
		//}
		if ( fTime <= 0.0f )
		{
			return;
		}

		Asura_Hierarchy_Anim* pxHierAnim = Asura_Animation_System::GetAnimByHash( pxCurrent->CalcAnimHashIDToPlay( pScoringData ) );
		if ( pxHierAnim )
		{
			float fProportion = Asura_Maths::ClampToRange( fTime / pxHierAnim->Duration, 0.0f, 1.0f );

			// Add on the movement.
			Asura_Vector_3 xMovementPos;
			Asura_Quat xMovementQuat;
			//Altered to add extra parameter to GetMovementOffset(). It may be desirable to pass in a valid skin so
			//that scaling can be done as needed. RAR 23/11/2006
			if ( pxHierAnim->GetMovementOffset(NULL, fProportion, &xMovementPos, &xMovementQuat ) )
			{
				Asura_Matrix_3x3 xMovementMatrix;
				xMovementQuat.ToMatrix( xMovementMatrix );
				xPosOffset += xMovementPos * xOriOffset;
				xOriOffset = xOriOffset * xMovementMatrix;
			}
			fTime -= pxHierAnim->Duration;
			if ( fTime < 0.0f )
			{
				return;
			}
		}
		pxCurrent = pxCurrent->GetNextEntry();
	}
}

#ifdef ASURA_TOOLS
Axon_Gamescene_RigidAnimTemplate::Axon_Gamescene_RigidAnimTemplate( const Asura_String& xName )
: PARENT( xName, AXON_ATT_RIGIDMOVEMENT )
{
}

void Axon_Gamescene_RigidAnimTemplate::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	//Version
	xStream << static_cast< int >( CURRENT_VERSION );

	//Parent
	PARENT::WriteToChunkStream( xStream );

	//Us
}

bool Axon_Gamescene_RigidAnimTemplate::Verify() const
{
	bool bOk = PARENT::Verify();

	// Check we will finish.
	if ( !WillFinish() )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Anim Templates",
			new Tools_MessageLogAction_OpenAnimTemplate( GetUniqueID() ),
			"Anim template '%s' is a rigid template but will loop forever.", GetName() );
		bOk = false;
	}

	return bOk;
}
#endif //ASURA_TOOLS

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Gamescene_FidgetAnimTemplate
//
//////////////////////////////////////////////////////////////////////////////

Axon_Gamescene_FidgetAnimTemplate::Axon_Gamescene_FidgetAnimTemplate()
: PARENT( AXON_ATT_FIDGETBEHAVIOUR )
, m_fMaxFidgetDelay( 30.0f )
, m_fMinFidgetDelay( 10.0f )
, m_fTotalWeighting( 0.0f )
{
}

Axon_Gamescene_FidgetAnimTemplate::~Axon_Gamescene_FidgetAnimTemplate()
{
}

void Axon_Gamescene_FidgetAnimTemplate::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	// Version.
	u_int uVersion;
	xStream >> uVersion;

    // Parent.
    PARENT::ReadFromChunkStream( xStream );

	// Us.
	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_fMaxFidgetDelay;
		xStream >> m_fMinFidgetDelay;
		xStream >> m_fTotalWeighting;
	}
	else if ( uVersion == 0 )
	{
		xStream >> m_fMaxFidgetDelay;
		xStream >> m_fMinFidgetDelay;
		xStream >> m_fTotalWeighting;

		// Inexplicably this was previously only calculated in tools builds when a fidget was chosen,
		// meaning that if the template was never previewed in tools the weighting would be invalid.
		CalculateTotalWeighting();
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Gamescene_FidgetAnimTemplate::ReadFromChunkStream - unhandled chunk version (%u)", uVersion ) );
	}
}

u_int Axon_Gamescene_FidgetAnimTemplate::GetNumValidEntries() const
{
	u_int uCount = 0;
	Axon_Gamescene_FidgetAnimTemplate_Entry* pxEntry = static_cast< Axon_Gamescene_FidgetAnimTemplate_Entry* >( m_pxFirstEntry );
	while ( pxEntry )
	{
		if ( pxEntry->GetFidgetWeight() > 0.0f )
		{
			++uCount;
		}
		pxEntry = static_cast< Axon_Gamescene_FidgetAnimTemplate_Entry* >( pxEntry->GetNextEntry() );
	}
	return uCount;
}

Axon_Gamescene_FidgetAnimTemplate_Entry* Axon_Gamescene_FidgetAnimTemplate::ChooseFidget()
{
	// Randomly chooses a fidget based on weighting.
	if ( m_fTotalWeighting == 0.0f )
	{
		return NULL;
	}

	float fRandom = Asura_Random::GetFloat( 0.0f, 1.0f ) * m_fTotalWeighting;
	Axon_Gamescene_FidgetAnimTemplate_Entry* pxEntry = static_cast< Axon_Gamescene_FidgetAnimTemplate_Entry* >( m_pxFirstEntry );
	while ( pxEntry )
	{
		fRandom -= pxEntry->GetFidgetWeight();
		if ( fRandom < 0.0f )
		{
			return pxEntry;
		}
		if ( pxEntry->GetNextEntry() )
		{
			pxEntry = static_cast< Axon_Gamescene_FidgetAnimTemplate_Entry* >( pxEntry->GetNextEntry() );
		}
		else
		{
			return pxEntry;
		}
	}

	return NULL;
}

#ifdef ASURA_TOOLS
Axon_Gamescene_FidgetAnimTemplate::Axon_Gamescene_FidgetAnimTemplate( const Asura_String& xName )
: PARENT( xName, AXON_ATT_FIDGETBEHAVIOUR )
, m_fMaxFidgetDelay( 30.0f )
, m_fMinFidgetDelay( 10.0f )
, m_fTotalWeighting( 0.0f )
{
}

void Axon_Gamescene_FidgetAnimTemplate::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	// Version.
	xStream << uCURRENT_CHUNK_VERSION;

	// Parent.
	PARENT::WriteToChunkStream( xStream );

	// Us.
	xStream << m_fMaxFidgetDelay;
	xStream << m_fMinFidgetDelay;
	xStream << m_fTotalWeighting;
}

bool Axon_Gamescene_FidgetAnimTemplate::Verify() const
{
	bool bOk = PARENT::Verify();

	if ( m_fTotalWeighting == 0.0f )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Anim Templates",
			new Tools_MessageLogAction_OpenAnimTemplate( GetUniqueID() ),
			"Fidget anim template '%s' has invalid fidget weightings. These must add up to greater than zero.", GetName() );
		bOk = false;
	}

	if ( m_fMinFidgetDelay > m_fMaxFidgetDelay )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Anim Templates",
			new Tools_MessageLogAction_OpenAnimTemplate( GetUniqueID() ),
			"Fidget anim template '%s' has corrupt fidget delay values.", GetName() );
		bOk = false;
	}

	return bOk;
}

void Axon_Gamescene_FidgetAnimTemplate::Copy( const Axon_Gamescene_AnimTemplate* pxTemplate )
{
	PARENT::Copy( pxTemplate );

	const Axon_Gamescene_FidgetAnimTemplate* pxFidget = static_cast< const Axon_Gamescene_FidgetAnimTemplate* >( pxTemplate );
	m_fMinFidgetDelay = pxFidget->m_fMinFidgetDelay;
	m_fMaxFidgetDelay = pxFidget->m_fMaxFidgetDelay;
	m_fTotalWeighting = pxFidget->m_fTotalWeighting;
}
#endif	//ASURA_TOOLS

void Axon_Gamescene_FidgetAnimTemplate::CalculateTotalWeighting()
{
	m_fTotalWeighting = 0.0f;
	Axon_Gamescene_FidgetAnimTemplate_Entry* pxEntry = static_cast< Axon_Gamescene_FidgetAnimTemplate_Entry* >( m_pxFirstEntry );
	while ( pxEntry )
	{
		m_fTotalWeighting += pxEntry->GetFidgetWeight();
		pxEntry = static_cast< Axon_Gamescene_FidgetAnimTemplate_Entry* >( pxEntry->GetNextEntry() );
	}
}

//////////////////////////////////////////////////////////////////////////////
//
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
