//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_GS2Actor.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Actor.h"

// asura
#include "AnimTemplates/Axon_Gamescene_AnimSequenceData.h"
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GS2Node.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AttractorNode/Asura_ServerNode_GS2Attractor.h"
#include "Entity/Locations/Spline/GamesceneSpline/Asura_ServerNode_GS2Spline.h"
#include "Entity/PhysicalObject/Asura_ClientEntity_PhysicalObject.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Network/Asura_Message.h"
#include "Physics/Asura_Physics_MovementQuery.h"
#include "Physics/Asura_Physics_RigidBody.h"

// axon
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Interface.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"
#include "Behaviour/BehaviourTypes/Axon_Gamescene_AStar.h"
#include "Core/Axon_Brain.h"
#include "Core/Axon_Messages.h"

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_GS2Actor
//
//////////////////////////////////////////////////////////////////////////////

Axon_Behaviour_GS2Actor::Axon_Behaviour_GS2Actor( Axon_Brain* pxBrain )
: PARENT( AXON_BEHAVIOUR_TYPE_GS2_ACTOR, pxBrain )
, m_pxOwnerPhyObj( Asura_ServerEntity_PhysicalObject::FindEntity( GetBrain()->GetOwnerGuid() ) )
, m_iState( UNSET )
, m_bLeaveNode( false )
, m_fRemainingTimeslice( 0.0f )
, m_uInitialNodeGuid( ASURA_GUID_UNREGISTERED )
, m_uCurrentEntityGuid( ASURA_GUID_UNREGISTERED )
, m_uCurrentAnimSet( uAXON_ANIMATION_SEQUENCE_ID_NONE )
, m_fAnimSequenceStartTime( 0.0f )
, m_pxCurrentAnimationTemplate( NULL )
, m_iPendingAnimSetCmd( NO_CMD_PENDING )
, m_uPendingAnimSet( uAXON_ANIMATION_SEQUENCE_ID_NONE )
, m_xLinksToTake()
, m_bSkipNextNode( false )
, m_bSkipAllNodes( false )
, m_xNodesToSkip()
, m_xRoute()
{
	Asura_Assertf( m_pxOwnerPhyObj, ( "Axon_Behaviour_GS2Actor::Axon_Behaviour_GS2Actor - cannot find owner physical object. Owner Guid = %u", GetBrain()->GetOwnerGuid() ) );
}

void Axon_Behaviour_GS2Actor::SetOwnerPhysicsToExtControl()
{
	if ( m_pxOwnerPhyObj )
	{
		m_pxOwnerPhyObj->ChangeType( ASURA_PHYSICS_OBJECT_EXTERNALCONTROL );
	}
}

void Axon_Behaviour_GS2Actor::ResetOwnerPhysicsFromExtControl()
{
	if ( m_pxOwnerPhyObj )
	{
		m_pxOwnerPhyObj->ChangeType( ASURA_PHYSICS_OBJECT_ACTOR );
	}
}

void Axon_Behaviour_GS2Actor::EnableOwnerPhysicsCollisions( const bool bEnable )
{
	if ( m_pxOwnerPhyObj )
	{
		// Safer to use this than simply setting the physics to ignore, as we may have physics of actor type as well as external control.
		m_pxOwnerPhyObj->SetOnlyCollideWithEnvironmentAndStaticObjects( !bEnable );
	}
}

void Axon_Behaviour_GS2Actor::GetOwnerPosition( Asura_Vector_3& xPosition ) const
{
	if ( m_pxOwnerPhyObj )
	{
		xPosition = m_pxOwnerPhyObj->GetPosition();
	}
}

bool Axon_Behaviour_GS2Actor::SetOwnerPosition( const Asura_Vector_3& xPosition, bool bShouldIgnoreCollisions /*=false*/ )
{
	if ( !m_pxOwnerPhyObj ) { return false; }

	// If we're not moving anywhere then we can ignore collisions.
	if ( xPosition.ApproximatelyEquals( m_pxOwnerPhyObj->GetPosition() ) )
	{
		bShouldIgnoreCollisions = true;
	}

	bool bShouldSetPosition = true;
	if ( !bShouldIgnoreCollisions && m_pxOwnerPhyObj->GetBB() )
	{
		const Asura_AVector3   xCurrentPosition( m_pxOwnerPhyObj->GetPosition() );
		const Asura_AVector3   xTargetPosition( xPosition );
		const Asura_AVector3   xDisplacement( xTargetPosition - xCurrentPosition );
		const Asura_AMatrix3x3 xOurOrientation( m_pxOwnerPhyObj->GetOrientation() );
		const Asura_OBB        xOurOBB( *m_pxOwnerPhyObj->GetBB(), xOurOrientation );

		Asura_Physics_MovementQuery xMovementQuery( xCurrentPosition, xDisplacement, xOurOBB,
			Asura_Physics_MovementQuery::ASURA_PMQCP_OBB, 0xffff,
			true, true, true, true, false );
		xMovementQuery.SetHandleToIgnore( m_pxOwnerPhyObj->GetPhysicsObjectHandle() );
		Asura_Physics_MovementQuery_Results xResults( xMovementQuery );
		bShouldSetPosition = xResults.GetAllowedDisplacement().ApproximatelyEquals( xDisplacement );
	}

	if ( bShouldSetPosition )
	{
		m_pxOwnerPhyObj->SetPosition( xPosition );
		return true;
	}
	else
	{
		return false;
	}
}

void Axon_Behaviour_GS2Actor::GetOwnerOrientation( Asura_Matrix_3x3& xOrientation ) const
{
	if ( m_pxOwnerPhyObj )
	{
		xOrientation = m_pxOwnerPhyObj->GetOrientation();
	}
}

void Axon_Behaviour_GS2Actor::SetOwnerOrientation( const Asura_Matrix_3x3& xOrientation )
{
	if ( m_pxOwnerPhyObj )
	{
		m_pxOwnerPhyObj->SetOrientation( xOrientation );
	}
}

void Axon_Behaviour_GS2Actor::TeleportOwnerToPosOri( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation )
{
	if ( m_pxOwnerPhyObj )
	{
		m_pxOwnerPhyObj->TeleportTo( xPosition, xOrientation, true );
	}
}

bool Axon_Behaviour_GS2Actor::GetOwnerAnimationVelocity( Asura_Vector_3& xVelocity ) const
{
	const Asura_ClientEntity_PhysicalObject* const pxClientOwnerPhyObj = Asura_ClientEntity_PhysicalObject::FindEntity( GetBrain()->GetOwnerGuid() );
	Asura_Assert( pxClientOwnerPhyObj, "Axon_Behaviour_GS2Actor::GetOwnerAnimationVelocity - cannot find owner physical object" );
	Asura_Assert( !pxClientOwnerPhyObj || pxClientOwnerPhyObj->GetShapeInstance().GetAnimation(),
		"Axon_Behaviour_GS2Actor::GetOwnerAnimationVelocity - cannot find owner animation" );
	if ( pxClientOwnerPhyObj && pxClientOwnerPhyObj->GetShapeInstance().GetAnimation() )
	{
		return pxClientOwnerPhyObj->GetShapeInstance().GetAnimation()->GetVelocity( &xVelocity );
	}
	else
	{
		return false;
	}
}

bool Axon_Behaviour_GS2Actor::GetOwnerAnimationAngularVelocity( Asura_Vector_3& xVelocity, Asura_Angle& fAngle ) const
{
	const Asura_ClientEntity_PhysicalObject* const pxClientOwnerPhyObj = Asura_ClientEntity_PhysicalObject::FindEntity( GetBrain()->GetOwnerGuid() );
	Asura_Assert( pxClientOwnerPhyObj, "Axon_Behaviour_GS2Actor::GetOwnerAnimationAngularVelocity - cannot find owner physical object" );
	Asura_Assert( !pxClientOwnerPhyObj || pxClientOwnerPhyObj->GetShapeInstance().GetAnimation(),
		"Axon_Behaviour_GS2Actor::GetOwnerAnimationAngularVelocity - cannot find owner animation" );
	if ( pxClientOwnerPhyObj && pxClientOwnerPhyObj->GetShapeInstance().GetAnimation() )
	{
		return pxClientOwnerPhyObj->GetShapeInstance().GetAnimation()->GetAngularVelocity( &xVelocity, &fAngle );
	}
	else
	{
		return false;
	}
}

bool Axon_Behaviour_GS2Actor::GetOwnerAnimationMovementOffset( Asura_Vector_3& xPosition, Asura_Quat& xQOrient ) const
{
	const Asura_ClientEntity_PhysicalObject* const pxClientOwnerPhyObj = Asura_ClientEntity_PhysicalObject::FindEntity( GetBrain()->GetOwnerGuid() );
	Asura_Assert( pxClientOwnerPhyObj, "Axon_Behaviour_GS2Actor::GetOwnerAnimationMovementOffset - cannot find owner physical object" );
	Asura_Assert( !pxClientOwnerPhyObj || pxClientOwnerPhyObj->GetShapeInstance().GetAnimation(),
		"Axon_Behaviour_GS2Actor::GetOwnerAnimationMovementOffset - cannot find owner animation" );
	if ( pxClientOwnerPhyObj && pxClientOwnerPhyObj->GetShapeInstance().GetAnimation() )
	{
		return pxClientOwnerPhyObj->GetShapeInstance().GetAnimation()->GetMovementOffset( &xPosition, &xQOrient );
	}
	else
	{
		return false;
	}
}

float Axon_Behaviour_GS2Actor::GetAnimSetDuration( const u_int uAnimSet )
{
	Axon_Gamescene_AnimTemplate* pxAnimationTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( uAnimSet );
	if ( pxAnimationTemplate )
	{
		return pxAnimationTemplate->GetDuration( GetAnimMappingData() );
	}
	else
	{
		return 0.0f;
	}
}

void Axon_Behaviour_GS2Actor::PlayAnimSet( const u_int uAnimSet )
{
	m_iPendingAnimSetCmd = PLAY_ANIM_SET;
	m_uPendingAnimSet    = uAnimSet;
}

bool Axon_Behaviour_GS2Actor::AnimSetIsFinished( const u_int uAnimSet )
{
	if ( m_uCurrentAnimSet == uAnimSet )
	{
		if ( m_pxCurrentAnimationTemplate )
		{
			const float fAnimTemplateTime = Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() - m_fAnimSequenceStartTime;
			float fRemainder = 0.0f;
			const Axon_Gamescene_AnimTemplate_Entry* const pxCurrentEntry =
				m_pxCurrentAnimationTemplate->GetCurrentEntry( fAnimTemplateTime, GetAnimMappingData(), fRemainder );
			return ( pxCurrentEntry == NULL );
		}
		else
		{
			return true;
		}
	}
	else if ( m_iPendingAnimSetCmd == PLAY_ANIM_SET	&& m_uPendingAnimSet == uAnimSet )
	{
		return false;
	}
	else
	{
		// If we get here we're expecting a particular set to be playing but it's not and we've no record of it, which is an error.
		Asura_Assert( false, "Axon_Behaviour_GS2Actor::AnimSetIsFinished - unexpected call to this function" );
		return true;
	}
}

void Axon_Behaviour_GS2Actor::StopAnimSet()
{
	m_iPendingAnimSetCmd = STOP_ANIM_SET;
}

void Axon_Behaviour_GS2Actor::AppendRoute( const Asura_LinkList< Asura_Guid >& xRoute )
{
	for ( Asura_LinkList_ConstIt< Asura_Guid > xRouteIt( xRoute ); !xRouteIt.Done(); xRouteIt.Next() )
	{
		m_xRoute.Push_Back( *xRouteIt.GetCurrent() );
	}
}

bool Axon_Behaviour_GS2Actor::FollowSurfaceTest( const Asura_Vector_3& xOrigin, const Asura_Vector_3& xDirection, float& fDistanceToIntersection )
{
	static const float s_fLOSDistance   = 10.0f;
	static const u_int s_uFlagsToIgnore = AsuraPolygonFlag_IgnoreLOS;

	Asura_Assert( ( Asura_Maths::Abs( xDirection.Magnitude() - 1.0f ) < 0.001f ),
		"Axon_Behaviour_GS2Actor::FollowSurfaceTest - direction must be normalised" );

	Asura_LOS_InputData xLOSInput( xOrigin - xDirection, xDirection, ASURA_GUID_UNREGISTERED, s_fLOSDistance, false, s_uFlagsToIgnore );
	const Asura_LOS_OutputData* pxLOSOutput = Asura_LOS::TestAgainstEnvironment( &xLOSInput );
	if ( pxLOSOutput->m_bHitEnvironment )
	{
		fDistanceToIntersection = pxLOSOutput->m_fDistanceToIntersection - 1.0f;
		return true;
	}
	else
	{
		return false;
	}
}

bool Axon_Behaviour_GS2Actor::HandleMessage( const Asura_Message* pxMessage )
{
	switch ( pxMessage->GetMsgID() )
	{
		//case ASURA_MSG_AXON_GS2_ANIM_CHANGE:
		// Our child behaviour should handle this message. They can best decide what to do.

		case ASURA_MSG_AXON_GS2_ATTRACTOR_CANCEL:
		{
			if ( m_iState == MOVING_TO_ATTRACTOR )
			{
				SwitchToState( FINISHED );
				ActionPendingAnimStateCommand();
			}

			return true;
		}

		case ASURA_MSG_AXON_GS2_ATTRACTOR_PULSE:
		{
			const Axon_Msg_GS2_Attractor_Node_Pulse* const pxPulseMessage =
				static_cast< const Axon_Msg_GS2_Attractor_Node_Pulse* >( pxMessage->GetData() );

			if ( m_iState == UNSET )
			{
				SwitchToState( MOVING_TO_ATTRACTOR, pxPulseMessage->m_uAttractorGuid );
				ActionPendingAnimStateCommand();
			}

			return true;
		}

		case ASURA_MSG_AXON_GS2_DONT_STOP_AT_NODE:
		{
			const Axon_Msg_GS2_Dont_Stop_At_Node* pxDSANMsg = static_cast< const Axon_Msg_GS2_Dont_Stop_At_Node* >( pxMessage->GetData() );

			switch ( pxDSANMsg->m_iNodeType )
			{
				case Axon_Msg_GS2_Dont_Stop_At_Node::NEXT_NODE:
				{
					m_bSkipNextNode = true;
					break;
				}

				case Axon_Msg_GS2_Dont_Stop_At_Node::ALL_NODES:
				{
					m_bSkipAllNodes = true;
					break;
				}

				case Axon_Msg_GS2_Dont_Stop_At_Node::SPECIFIC_NODE:
				{
					// Add the specific node to the list of nodes to skip.
					if ( pxDSANMsg->m_uNodeGuid != ASURA_GUID_UNREGISTERED )
					{
						m_xNodesToSkip.Push_Front( pxDSANMsg->m_uNodeGuid );
					}
					break;
				}

				case Axon_Msg_GS2_Dont_Stop_At_Node::CLEAR_SETTINGS:
				{
					m_bSkipNextNode = false;
					m_bSkipAllNodes = false;
					m_xNodesToSkip.Clear();
					break;
				}

				default:
					Asura_Assert( false, "Axon_Behaviour_GS2Actor::HandleMessage - bad node type" );
					break;
			}

			return true;
		}

		case ASURA_MSG_AXON_GS2_EXIT_GAMESCENE:
		{
			SwitchToState( FINISHED );
			ActionPendingAnimStateCommand();
			return true;
		}

		case ASURA_MSG_AXON_GS2_LEAVE_NODE:
		{
			m_bLeaveNode = ( m_iState == AT_NODE );
			return true;
		}

		case ASURA_MSG_AXON_GS2_PATHFIND_TO_NODE:
		{
			const Axon_Msg_GS2_Pathfind_To_Node* pxPTNMgs = static_cast< const Axon_Msg_GS2_Pathfind_To_Node* >( pxMessage->GetData() );
			Axon_Gamescene_AStar_System::GetRouteTo( pxPTNMgs->m_uDestNodeGuid, GetCurrentEntityGuid(), m_xRoute );
			return true;
		}

		case ASURA_MSG_AXON_GS2_TAKE_LINK:
		{
			const Axon_Msg_GS2_Take_Link* pxTLMsg = static_cast< const Axon_Msg_GS2_Take_Link* >( pxMessage->GetData() );
			m_xLinksToTake.Push_Front( pxTLMsg->m_uLinkGuid );
			return true;
		}

		case ASURA_MSG_AXON_GS2_TELEPORT_TO_NODE:
		{
			const Axon_Msg_GS2_Teleport_To_Node* pxTTNMessage = static_cast< const Axon_Msg_GS2_Teleport_To_Node* >( pxMessage->GetData() );
			Asura_ServerNode_GS2Node* pxTargetNode = Asura_ServerNode_GS2Node::FindEntity( pxTTNMessage->m_uNodeGuid );
			if ( pxTargetNode )
			{
				Asura_Matrix_3x3 xOrientMatrix;
				pxTargetNode->GetOrientation( xOrientMatrix );

				TeleportOwnerToPosOri( pxTargetNode->GetPosition(), xOrientMatrix );
				SwitchToState( AT_NODE, pxTTNMessage->m_uNodeGuid );
				ActionPendingAnimStateCommand();
			}
			else
			{
				Asura_Assertf( false, ( "Axon_Behaviour_GS2Actor::HandleMessage - could not find node %u to teleport to.", pxTTNMessage->m_uNodeGuid ) );
			}

			return true;
		}

		default:
		{
			return PARENT::HandleMessage( pxMessage );
		}
	}
}

void Axon_Behaviour_GS2Actor::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_iState;
		xStream >> m_bLeaveNode;
		xStream >> m_uInitialNodeGuid;
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		m_uInitialNodeGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( m_uInitialNodeGuid );
		#endif
		xStream >> m_uCurrentEntityGuid;
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		m_uCurrentEntityGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( m_uCurrentEntityGuid );
		#endif
		xStream >> m_uCurrentAnimSet;
		float fAnimSequenceDuration;
		xStream >> fAnimSequenceDuration;
		m_fAnimSequenceStartTime = Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() - fAnimSequenceDuration;
		xStream >> m_iPendingAnimSetCmd;
		xStream >> m_uPendingAnimSet;

		u_int uNumLinksToTake;
		xStream >> uNumLinksToTake;
		for ( u_int u = 0; u < uNumLinksToTake; ++u )
		{
			Asura_Guid uLinkToTake;
			xStream >> uLinkToTake;
			m_xLinksToTake.Push_Back( uLinkToTake );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xLinksToTake ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		xStream >> m_bSkipNextNode;
		xStream >> m_bSkipAllNodes;

		u_int uNumNodesToSkip;
		xStream >> uNumNodesToSkip;
		for ( u_int u = 0; u < uNumNodesToSkip; ++u )
		{
			Asura_Guid uNodeToSkip;
			xStream >> uNodeToSkip;
			m_xNodesToSkip.Push_Back( uNodeToSkip );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xNodesToSkip ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		u_int uRouteLength;
		xStream >> uRouteLength;
		for ( u_int u = 0; u < uRouteLength; ++u )
		{
			Asura_Guid uLinkGuid;
			xStream >> uLinkGuid;
			m_xRoute.Push_Back( uLinkGuid );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xRoute ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		u_int uCurrentChildType;
		xStream >> uCurrentChildType;
		if ( uCurrentChildType != AXON_BEHAVIOUR_TYPE_UNSET )
		{
			Axon_Behaviour_GS2Interface* pxGSBehaviour = GetInterface( CreateChildWithType( uCurrentChildType, NULL, false ) );
			Asura_Assertf( pxGSBehaviour,
				( "Axon_Behaviour_GS2Actor::ReadFromChunkStream - could not create previously saved child (owner guid: %u)", GetBrain()->GetOwnerGuid() ) );
			if ( pxGSBehaviour )
			{
				pxGSBehaviour->SetGSActorBehaviour( this );
				pxGSBehaviour->ReadFromChunkStream( xStream );
			}
		}
	}
	else if ( uVersion == 4 )
	{
		xStream >> m_iState;
		xStream >> m_bLeaveNode;
		xStream >> m_uCurrentEntityGuid;
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		m_uCurrentEntityGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( m_uCurrentEntityGuid );
		#endif
		xStream >> m_uCurrentAnimSet;
		float fAnimSequenceDuration;
		xStream >> fAnimSequenceDuration;
		m_fAnimSequenceStartTime = Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() - fAnimSequenceDuration;
		xStream >> m_iPendingAnimSetCmd;
		xStream >> m_uPendingAnimSet;

		u_int uNumLinksToTake;
		xStream >> uNumLinksToTake;
		for ( u_int u = 0; u < uNumLinksToTake; ++u )
		{
			Asura_Guid uLinkToTake;
			xStream >> uLinkToTake;
			m_xLinksToTake.Push_Back( uLinkToTake );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xLinksToTake ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		xStream >> m_bSkipNextNode;
		xStream >> m_bSkipAllNodes;

		u_int uNumNodesToSkip;
		xStream >> uNumNodesToSkip;
		for ( u_int u = 0; u < uNumNodesToSkip; ++u )
		{
			Asura_Guid uNodeToSkip;
			xStream >> uNodeToSkip;
			m_xNodesToSkip.Push_Back( uNodeToSkip );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xNodesToSkip ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		u_int uRouteLength;
		xStream >> uRouteLength;
		for ( u_int u = 0; u < uRouteLength; ++u )
		{
			Asura_Guid uLinkGuid;
			xStream >> uLinkGuid;
			m_xRoute.Push_Back( uLinkGuid );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xRoute ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		u_int uCurrentChildType;
		xStream >> uCurrentChildType;
		if ( uCurrentChildType != AXON_BEHAVIOUR_TYPE_UNSET )
		{
			Axon_Behaviour_GS2Interface* pxGSBehaviour = GetInterface( CreateChildWithType( uCurrentChildType, NULL, false ) );
			Asura_Assertf( pxGSBehaviour,
				( "Axon_Behaviour_GS2Actor::ReadFromChunkStream - could not create previously saved child (owner guid: %u)", GetBrain()->GetOwnerGuid() ) );
			if ( pxGSBehaviour )
			{
				pxGSBehaviour->SetGSActorBehaviour( this );
				pxGSBehaviour->ReadFromChunkStream( xStream );
			}
		}
	}
	else if ( uVersion == 3 )
	{
		xStream >> m_iState;
		xStream >> m_bLeaveNode;
		xStream >> m_uCurrentEntityGuid;
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		m_uCurrentEntityGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( m_uCurrentEntityGuid );
		#endif
		xStream >> m_uCurrentAnimSet;
		float fAnimSequenceDuration;
		xStream >> fAnimSequenceDuration;
		m_fAnimSequenceStartTime = Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() - fAnimSequenceDuration;

		u_int uNumLinksToTake;
		xStream >> uNumLinksToTake;
		for ( u_int u = 0; u < uNumLinksToTake; ++u )
		{
			Asura_Guid uLinkToTake;
			xStream >> uLinkToTake;
			m_xLinksToTake.Push_Back( uLinkToTake );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xLinksToTake ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		xStream >> m_bSkipNextNode;
		xStream >> m_bSkipAllNodes;

		u_int uNumNodesToSkip;
		xStream >> uNumNodesToSkip;
		for ( u_int u = 0; u < uNumNodesToSkip; ++u )
		{
			Asura_Guid uNodeToSkip;
			xStream >> uNodeToSkip;
			m_xNodesToSkip.Push_Back( uNodeToSkip );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xNodesToSkip ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		u_int uRouteLength;
		xStream >> uRouteLength;
		for ( u_int u = 0; u < uRouteLength; ++u )
		{
			Asura_Guid uLinkGuid;
			xStream >> uLinkGuid;
			m_xRoute.Push_Back( uLinkGuid );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xRoute ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		u_int uCurrentChildType;
		xStream >> uCurrentChildType;
		if ( uCurrentChildType != AXON_BEHAVIOUR_TYPE_UNSET )
		{
			Axon_Behaviour_GS2Interface* pxGSBehaviour = GetInterface( CreateChildWithType( uCurrentChildType, NULL, false ) );
			Asura_Assertf( pxGSBehaviour,
				( "Axon_Behaviour_GS2Actor::ReadFromChunkStream - could not create previously saved child (owner guid: %u)", GetBrain()->GetOwnerGuid() ) );
			if ( pxGSBehaviour )
			{
				pxGSBehaviour->SetGSActorBehaviour( this );
				pxGSBehaviour->ReadFromChunkStream( xStream );
			}
		}
	}
	#ifdef ASURA_TOOLS
	else if ( uVersion == 2 )
	{
		xStream >> m_iState;
		xStream >> m_uCurrentEntityGuid;
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		m_uCurrentEntityGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( m_uCurrentEntityGuid );
		#endif
		xStream >> m_uCurrentAnimSet;
		float fAnimSequenceDuration;
		xStream >> fAnimSequenceDuration;
		m_fAnimSequenceStartTime = Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() - fAnimSequenceDuration;

		u_int uNumLinksToTake;
		xStream >> uNumLinksToTake;
		for ( u_int u = 0; u < uNumLinksToTake; ++u )
		{
			Asura_Guid uLinkToTake;
			xStream >> uLinkToTake;
			m_xLinksToTake.Push_Back( uLinkToTake );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xLinksToTake ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		xStream >> m_bSkipNextNode;
		xStream >> m_bSkipAllNodes;

		u_int uNumNodesToSkip;
		xStream >> uNumNodesToSkip;
		for ( u_int u = 0; u < uNumNodesToSkip; ++u )
		{
			Asura_Guid uNodeToSkip;
			xStream >> uNodeToSkip;
			m_xNodesToSkip.Push_Back( uNodeToSkip );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xNodesToSkip ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		u_int uRouteLength;
		xStream >> uRouteLength;
		for ( u_int u = 0; u < uRouteLength; ++u )
		{
			Asura_Guid uLinkGuid;
			xStream >> uLinkGuid;
			m_xRoute.Push_Back( uLinkGuid );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xRoute ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		u_int uCurrentChildType;
		xStream >> uCurrentChildType;
		if ( uCurrentChildType != AXON_BEHAVIOUR_TYPE_UNSET )
		{
			Axon_Behaviour_GS2Interface* pxGSBehaviour = GetInterface( CreateChildWithType( uCurrentChildType, NULL, false ) );
			Asura_Assertf( pxGSBehaviour,
				( "Axon_Behaviour_GS2Actor::ReadFromChunkStream - could not create previously saved child (owner guid: %u)", GetBrain()->GetOwnerGuid() ) );
			if ( pxGSBehaviour )
			{
				pxGSBehaviour->SetGSActorBehaviour( this );
				pxGSBehaviour->ReadFromChunkStream( xStream );
			}
		}
	}
	else if ( uVersion == 1 )
	{
		xStream >> m_iState;
		xStream >> m_uCurrentEntityGuid;
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		m_uCurrentEntityGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( m_uCurrentEntityGuid );
		#endif
		xStream >> m_uCurrentAnimSet;
		float fAnimSequenceDuration;
		xStream >> fAnimSequenceDuration;
		m_fAnimSequenceStartTime = Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() - fAnimSequenceDuration;

		u_int uNumLinksToTake;
		xStream >> uNumLinksToTake;
		for ( u_int u = 0; u < uNumLinksToTake; ++u )
		{
			Asura_Guid uLinkToTake;
			xStream >> uLinkToTake;
			m_xLinksToTake.Push_Back( uLinkToTake );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xLinksToTake ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif
		u_int uNumNodesNotToStopAt;
		xStream >> uNumNodesNotToStopAt;
		for ( u_int u = 0; u < uNumNodesNotToStopAt; ++u )
		{
			Asura_Guid uNodeNotToStopAt;
			xStream >> uNodeNotToStopAt;
			m_xNodesToSkip.Push_Back( uNodeNotToStopAt );
		}
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		for ( Asura_LinkList_It< Asura_Guid > xIt( m_xNodesToSkip ); !xIt.Done(); xIt.Next() )
		{
			Asura_Guid* pxGuid = xIt.GetCurrent();
			*pxGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( *pxGuid );
		}
		#endif

		u_int uCurrentChildType;
		xStream >> uCurrentChildType;
		if ( uCurrentChildType != AXON_BEHAVIOUR_TYPE_UNSET )
		{
			Axon_Behaviour_GS2Interface* pxGSBehaviour = GetInterface( CreateChildWithType( uCurrentChildType, NULL, false ) );
			Asura_Assertf( pxGSBehaviour,
				( "Axon_Behaviour_GS2Actor::ReadFromChunkStream - could not create previously saved child (owner guid: %u)", GetBrain()->GetOwnerGuid() ) );
			if ( pxGSBehaviour )
			{
				pxGSBehaviour->SetGSActorBehaviour( this );
				pxGSBehaviour->ReadFromChunkStream( xStream );
			}
		}
	}
	else if ( uVersion == 0 )
	{
		xStream >> m_iState;
		xStream >> m_uCurrentEntityGuid;
		#if defined(ASURA_USE_STREAMING) && !defined(ASURA_TOOLS)
		m_uCurrentEntityGuid = Asura_Streaming_System::MakeCurrentMasterZoneGuid( m_uCurrentEntityGuid );
		#endif
		xStream >> m_uCurrentAnimSet;
		float fAnimSequenceDuration;
		xStream >> fAnimSequenceDuration;
		m_fAnimSequenceStartTime = Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() - fAnimSequenceDuration;

		u_int uCurrentChildType;
		xStream >> uCurrentChildType;
		if ( uCurrentChildType != AXON_BEHAVIOUR_TYPE_UNSET )
		{
			Axon_Behaviour_GS2Interface* pxGSBehaviour = GetInterface( CreateChildWithType( uCurrentChildType, NULL, false ) );
			Asura_Assertf( pxGSBehaviour,
				( "Axon_Behaviour_GS2Actor::ReadFromChunkStream - could not create previously saved child (owner guid: %u)", GetBrain()->GetOwnerGuid() ) );
			if ( pxGSBehaviour )
			{
				pxGSBehaviour->SetGSActorBehaviour( this );
				pxGSBehaviour->ReadFromChunkStream( xStream );
			}
		}
	}
	#endif
	else
	{
		Asura_Assertf( false, ( "Axon_Behaviour_GS2Actor::ReadFromChunkStream - unhandled chunk version (%d)", uVersion ) );
	}
}

void Axon_Behaviour_GS2Actor::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_iState;
	xStream << m_bLeaveNode;
	xStream << m_uInitialNodeGuid;
	xStream << m_uCurrentEntityGuid;
	xStream << m_uCurrentAnimSet;
	const float fAnimSequenceDuration = Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence() - m_fAnimSequenceStartTime;
	xStream << fAnimSequenceDuration;
	xStream << m_iPendingAnimSetCmd;
	xStream << m_uPendingAnimSet;

	const u_int uNumLinksToTake = m_xLinksToTake.Size();
	xStream << uNumLinksToTake;
	for ( Asura_LinkList_ConstIt< Asura_Guid > xIt( m_xLinksToTake ); !xIt.Done(); xIt.Next() )
	{
		xStream << *xIt.GetCurrent();
	}

	xStream << m_bSkipNextNode;
	xStream << m_bSkipAllNodes;

	const u_int uNumNodesToSkip = m_xNodesToSkip.Size();
	xStream << uNumNodesToSkip;
	for ( Asura_LinkList_ConstIt< Asura_Guid > xIt( m_xNodesToSkip ); !xIt.Done(); xIt.Next() )
	{
		xStream << *xIt.GetCurrent();
	}

	const u_int uRouteLength = m_xRoute.Size();
	xStream << uRouteLength;
	for ( Asura_LinkList_ConstIt< Asura_Guid > xIt( m_xRoute ); !xIt.Done(); xIt.Next() )
	{
		xStream << *xIt.GetCurrent();
	}

	if ( GetChild() )
	{
		xStream << GetChild()->GetType();
		GetChild()->WriteToChunkStream( xStream );
	}
	else
	{
		xStream << AXON_BEHAVIOUR_TYPE_UNSET;
	}
}

bool Axon_Behaviour_GS2Actor::Activate()
{
	const bool bParentOk = PARENT::Activate();

	SetOwnerPhysicsToExtControl();

	return bParentOk;
}

void Axon_Behaviour_GS2Actor::Deactivate()
{
	SwitchToState( FINISHED );

	ResetOwnerPhysicsFromExtControl();
	EnableOwnerPhysicsCollisions( true );

	m_uCurrentEntityGuid = ASURA_GUID_UNREGISTERED;
}

void Axon_Behaviour_GS2Actor::PostLoadInitialise()
{
	PARENT::PostLoadInitialise();

	// If we have an initial node to start on then teleport there.
	if ( m_uInitialNodeGuid != ASURA_GUID_UNREGISTERED )
	{
		// Teleport ourselves to our initial node.
		Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_AXON_GS2_TELEPORT_TO_NODE, sizeof( Axon_Msg_GS2_Teleport_To_Node ) );
		if ( pxMessage )
		{
			Axon_Msg_GS2_Teleport_To_Node* pxData = static_cast< Axon_Msg_GS2_Teleport_To_Node* >( pxMessage->GetData() );
			pxData->m_uNodeGuid = m_uInitialNodeGuid;
			DoHandleMessage( pxMessage );
			Asura_Network::DestroyMessage( pxMessage );
		}
		else
		{
			SetInError();
		}

		// Wipe our initial node, so that we don't do this on subsequent checkpoint loads.
		m_uInitialNodeGuid = ASURA_GUID_UNREGISTERED;
	}
	else
	{
		Axon_Gamescene_AnimTemplate* pxAnimationTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( m_uCurrentAnimSet );
		if ( pxAnimationTemplate )
		{
			RestartAnimTemplate( pxAnimationTemplate );
		}
	}
}

void Axon_Behaviour_GS2Actor::PerFrameUpdate( const float fTimeslice )
{
	PARENT::Update( fTimeslice );

	if ( IsInError() || fTimeslice == 0.0f ) { return; }

	// If we switch state we must make sure we update again, in case we need to switch state again immediately.
	// We need to keep our own timeslice internally for sharing between links across nodes, etc.
	int iOldState = UNSET;
	do
	{
		iOldState = m_iState;

		switch ( m_iState )
		{
			case MOVING_TO_ATTRACTOR:	UpdateStateMovingToAttractor();	break;
			case AT_NODE:				UpdateStateAtNode();			break;
			case ON_LINK:				UpdateStateOnLink();			break;

			case FINISHED:
			case UNSET:
			case NUM_STATES:
			default:
			{
				break;
			}
		}

		if ( iOldState != m_iState && GetChild() )
		{
			GetChild()->DoPerFrameUpdate( m_fRemainingTimeslice );
			m_fRemainingTimeslice = 0.0f;
		}
	} while ( iOldState != m_iState );

	ActionPendingAnimStateCommand();
}

Axon_Behaviour_GS2Actor::LinkChoiceResult Axon_Behaviour_GS2Actor::ChooseLinkToTake( const Asura_Guid uSourceNodeGuid, Asura_Guid& uLinkGuid )
{
	uLinkGuid = ASURA_GUID_UNREGISTERED;

	Asura_ServerNode_GS2Node* pxSourceNode = Asura_ServerNode_GS2Node::FindEntity( uSourceNodeGuid );
	if ( !pxSourceNode ) { return LCR_ERROR; }
	if ( pxSourceNode->GetNumberLinks() == 0 ) { return LCR_NO_LINKS_EXIST; }

	// Priorities are:
	//		- route find
	//		- take specific link
	//		- link weights
	//		- random active link

	if ( m_xRoute.Size() > 0 )
	{
		// Take the next link in our route if we can.
		const Asura_Guid uRouteLink = m_xRoute.Front()->Data();
		for ( u_int u = 0; u < pxSourceNode->GetNumberLinks(); ++u )
		{
			const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxSourceNode->GetLink( u ) );
			if ( pxLink->GetGuid() == uRouteLink && pxLink->IsEntityActive() )
			{
				uLinkGuid = uRouteLink;
				IncrementAlongRoute();
				return LCR_LINK_CHOSEN;
			}
		}

		// If we have a route but cannot find the correct links we should discard the route and carry on as normal.
		m_xRoute.Clear();
	}

	if ( m_xLinksToTake.Size() > 0 )
	{
		// Search for any specific link to take.
		for ( Asura_LinkList_ConstIt< Asura_Guid > xIt( m_xLinksToTake ); !xIt.Done(); xIt.Next() )
		{
			const Asura_Guid uLinkToTake = *xIt.GetCurrent();

			for ( u_int u = 0; u < pxSourceNode->GetNumberLinks(); ++u )
			{
				const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxSourceNode->GetLink( u ) );
				if ( pxLink->GetGuid() == uLinkToTake && pxLink->IsEntityActive() )
				{
					uLinkGuid = uLinkToTake;
					return LCR_LINK_CHOSEN;
				}
			}
		}
	}

	// Calculate this for the following algorithms.
	u_int uNumActiveLinks = 0;
	for ( u_int u = 0; u < pxSourceNode->GetNumberLinks(); ++u )
	{
		const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxSourceNode->GetLink( u ) );
		if ( pxLink && pxLink->IsEntityActive() )
		{
			++uNumActiveLinks;
		}
	}
	if ( uNumActiveLinks == 0 ) { return LCR_NO_LINKS_ACTIVE; }

	if ( pxSourceNode->ShouldUseLinkWeights() )
	{
		// Use link weights to choose.
		float fTotalWeighting = 0.0f;
		for ( u_int u = 0; u < pxSourceNode->GetNumberLinks(); ++u )
		{
			const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxSourceNode->GetLink( u ) );
			if ( pxLink && pxLink->IsEntityActive() )
			{
				fTotalWeighting += pxSourceNode->GetLinkWeight( u );
			}
		}

		if ( fTotalWeighting > 0.0f )
		{
			const float fRandomWeight = Asura_Random::GetFloat( 0.0f, fTotalWeighting );
			float fCumulativeWeighting = 0.0f;
			for ( u_int u = 0; u < pxSourceNode->GetNumberLinks(); ++u )
			{
				const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxSourceNode->GetLink( u ) );
				if ( pxLink && pxLink->IsEntityActive() )
				{
					fCumulativeWeighting += pxSourceNode->GetLinkWeight( u );
					if ( fRandomWeight <= fCumulativeWeighting )
					{
						uLinkGuid = pxLink->GetGuid();
						return LCR_LINK_CHOSEN;
					}
				}
			}
		}
	}

	// Pick a random active link.
	const u_int uRandomLink = Asura_Random::GetIntU( 0, uNumActiveLinks - 1 );
	uNumActiveLinks = 0;
	for ( u_int u = 0; u < pxSourceNode->GetNumberLinks(); ++u )
	{
		const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( pxSourceNode->GetLink( u ) );
		if ( pxLink && pxLink->IsEntityActive() )
		{
			if ( uNumActiveLinks == uRandomLink )
			{
				uLinkGuid = pxLink->GetGuid();
				return LCR_LINK_CHOSEN;
			}
			++uNumActiveLinks;
		}
	}

	// Shouldn't ever get down here.
	return LCR_ERROR;
}

Axon_Behaviour* Axon_Behaviour_GS2Actor::CreateBehaviours( const Axon_BehaviourParams* pxBehaviourParams )
{
	if ( !pxBehaviourParams ) { return NULL; }

	// Create this behaviour.
	Axon_Behaviour_GS2Interface* pxGSBehaviour = GetInterface( GetBrain()->CreateBehaviour( pxBehaviourParams->GetBehaviourType() ) );
	if ( pxGSBehaviour )
	{
		pxGSBehaviour->SetGSActorBehaviour( this );
		pxGSBehaviour->DoHandleParams( pxBehaviourParams );
	}

	// Create all the child behaviours.
	pxBehaviourParams = pxBehaviourParams->GetChild();
	while ( pxBehaviourParams )
	{
		Axon_Behaviour* pxChildBehaviour = CreateBehaviours( pxBehaviourParams );
		if ( pxChildBehaviour )
		{
			pxGSBehaviour->LinkChild( pxChildBehaviour );
		}

		pxBehaviourParams = pxBehaviourParams->GetSibling();
	}

	return pxGSBehaviour;
}

void Axon_Behaviour_GS2Actor::UpdateStateMovingToAttractor()
{
	if ( !GetChild() || GetChild()->IsInError() )
	{
		// Nothing to do but exit gamescene.
		SetInError();
		SwitchToState( FINISHED );
	}
	else if ( GetChild()->IsFinished() )
	{
		Axon_Behaviour_GS2Interface* pxInterface = GetInterface( GetChild() );
		if ( pxInterface->ShouldExitGamescene() )
		{
			SwitchToState( FINISHED );
		}
		else
		{
			// We're on our way to our attractor. When we arrive we want to switch to the same attractor.
			m_fRemainingTimeslice = pxInterface->GetRemainingTimeslice();
			SwitchToState( AT_NODE, GetCurrentEntityGuid() );
		}
	}
}

void Axon_Behaviour_GS2Actor::UpdateStateAtNode()
{
	if ( GetChild() )
	{
		if ( GetChild()->IsInError() )
		{
			// Nothing to do but exit gamescene.
			SetInError();
			SwitchToState( FINISHED );
		}
		else if ( GetChild()->IsFinished() || m_bLeaveNode )
		{
			// Determine what we should do next.
			Axon_Behaviour_GS2Interface* pxInterface = GetInterface( GetChild() );
			if ( pxInterface->ShouldExitGamescene() )
			{
				SwitchToState( FINISHED );
			}
			else
			{
				m_fRemainingTimeslice = pxInterface->GetRemainingTimeslice();
				if ( pxInterface->HasNextGuidControl() )
				{
					const Asura_Guid uNextLink = pxInterface->GetNextGamesceneEntityGuid();
					if ( uNextLink != ASURA_GUID_UNREGISTERED )
					{
						SwitchToState( ON_LINK, uNextLink );
					}
					else
					{
						// Couldn't get a next link, so we'll have to wait here.
					}
				}
				else
				{
					Asura_Guid uNextLink = ASURA_GUID_UNREGISTERED;
					const LinkChoiceResult eResult = ChooseLinkToTake( GetCurrentEntityGuid(), uNextLink );
					switch ( eResult )
					{
						case LCR_LINK_CHOSEN:
							SwitchToState( ON_LINK, uNextLink );
							break;

						case LCR_NO_LINKS_ACTIVE:
							// Couldn't get a next link, so we'll have to wait here.
							break;

						case LCR_ERROR:
						default:
							SetInError();
							// fallthrough
						case LCR_NO_LINKS_EXIST:
							SwitchToState( FINISHED );
							break;
					}
				}
			}
		}
	}
	else // if ( !GetChild() )
	{
		// No behaviour set on a node causes us to skip over the node.
		m_fRemainingTimeslice = 0.0f;
		Asura_Guid uNextLink = ASURA_GUID_UNREGISTERED;
		const LinkChoiceResult eResult = ChooseLinkToTake( GetCurrentEntityGuid(), uNextLink );
		switch ( eResult )
		{
			case LCR_LINK_CHOSEN:
				SwitchToState( ON_LINK, uNextLink );
				break;

			case LCR_NO_LINKS_ACTIVE:
				// Couldn't get a next link, so we'll have to wait here.
				break;

			case LCR_ERROR:
			default:
				SetInError();
				// fallthrough
			case LCR_NO_LINKS_EXIST:
				SwitchToState( FINISHED );
				break;
		}
	}
}

void Axon_Behaviour_GS2Actor::UpdateStateOnLink()
{
	const Asura_ServerNode_GS2Spline* pxLink = Asura_ServerNode_GS2Spline::FindEntity( GetCurrentEntityGuid() );

	if ( !GetChild() || GetChild()->IsInError() || !pxLink )
	{
		// Nothing to do but exit gamescene.
		SetInError();
		SwitchToState( FINISHED );
	}
	else if ( GetChild()->IsFinished() )
	{
		// Determine what we should do next.
		Axon_Behaviour_GS2Interface* pxInterface = GetInterface( GetChild() );
		if ( pxInterface->ShouldExitGamescene() )
		{
			SwitchToState( FINISHED );
		}
		else
		{
			// Can't switch to anything but our target node.
			m_fRemainingTimeslice = pxInterface->GetRemainingTimeslice();
			SwitchToState( AT_NODE, pxLink->GetTargetGuid() );
		}
	}
}

void Axon_Behaviour_GS2Actor::SwitchToState( const int iNextState, const Asura_Guid uNextEntityGuid /*= ASURA_GUID_UNREGISTERED*/ )
{
	ExitState( m_iState );
	m_iState = iNextState;
	EnterState( m_iState, uNextEntityGuid );
}

void Axon_Behaviour_GS2Actor::EnterState( const int iState, const Asura_Guid uNextEntityGuid )
{
	switch ( iState )
	{
		case MOVING_TO_ATTRACTOR:
		{
			bool bOk = false;

			Asura_ServerNode_GS2Attractor* pxTargetAttractor = Asura_ServerNode_GS2Attractor::FindEntity( uNextEntityGuid );
			m_uCurrentEntityGuid = uNextEntityGuid;
			if ( pxTargetAttractor )
			{
				pxTargetAttractor->SetAttractedGuid( GetBrain()->GetOwnerGuid() );
				pxTargetAttractor->IncPendingCount();

				// Traverse the parameter tree and create the necessary behaviours.
				Axon_Behaviour* pxNewChild = CreateBehaviours( pxTargetAttractor->GetEnRouteBehaviourParams() );
				if ( pxNewChild )
				{
					if ( pxNewChild->DoActivate() )
					{
						LinkChild( pxNewChild );
						bOk = true;
					}
				}
			}

			if ( !bOk )
			{
				Asura_Assertf( false,
					( "Axon_Behaviour_GS2Actor::EnterState - could not enter MOVING_TO_ATTRACTOR state (%u)", GetBrain()->GetOwnerGuid() ) );
				SetInError();
				SwitchToState( FINISHED );
			}

			break;
		}

		case AT_NODE:
		{
			Asura_ServerNode_GS2Node* pxTargetNode = Asura_ServerNode_GS2Node::FindEntity( uNextEntityGuid );
			m_uCurrentEntityGuid = uNextEntityGuid;
			if ( pxTargetNode )
			{
				pxTargetNode->EnterNode( GetBrain()->GetOwnerGuid() );

				if ( !m_bSkipNextNode && !m_bSkipAllNodes && !m_xNodesToSkip.Find( m_uCurrentEntityGuid ) )
				{
					// Traverse the parameter tree and create the necessary behaviours.
					// Note that it is valid here not to have any behaviours - it just means we skip this node.
					Axon_Behaviour* pxNewChild = CreateBehaviours( pxTargetNode->GetBehaviourParams() );
					if ( pxNewChild )
					{
						if ( pxNewChild->DoActivate() )
						{
							LinkChild( pxNewChild );
						}
					}
				}
			}
			else
			{
				Asura_Assertf( false,
					( "Axon_Behaviour_GS2Actor::EnterState - could not enter AT_NODE state (%u)", GetBrain()->GetOwnerGuid() ) );
				SetInError();
				SwitchToState( FINISHED );
			}

			break;
		}

		case ON_LINK:
		{
			bool bOk = false;

			Asura_ServerNode_GS2Spline* pxTargetSpline = Asura_ServerNode_GS2Spline::FindEntity( uNextEntityGuid );
			Asura_ServerNode_GS2Node*   pxTargetNode = pxTargetSpline ? Asura_ServerNode_GS2Node::FindEntity( pxTargetSpline->GetTargetGuid() ) : NULL;
			m_uCurrentEntityGuid = uNextEntityGuid;
			if ( pxTargetSpline && pxTargetNode )
			{
				pxTargetSpline->EnterSpline( GetBrain()->GetOwnerGuid() );
				pxTargetNode->IncPendingCount();

				// Traverse the parameter tree and create the necessary behaviours.
				Axon_Behaviour* pxNewChild = CreateBehaviours( pxTargetSpline->GetBehaviourParams() );
				if ( pxNewChild )
				{
					if ( pxNewChild->DoActivate() )
					{
						LinkChild( pxNewChild );
						bOk = true;
					}
				}
			}

			if ( !bOk )
			{
				Asura_Assertf( false,
					( "Axon_Behaviour_GS2Actor::EnterState - could not enter ON_LINK state (%u)", GetBrain()->GetOwnerGuid() ) );
				SetInError();
				SwitchToState( FINISHED );
			}

			break;
		}

		case FINISHED:
		{
			m_uCurrentEntityGuid = ASURA_GUID_UNREGISTERED;
			ActionPendingAnimStateCommand();
			SetFinished();
			break;
		}

		case UNSET:
		case NUM_STATES:
		default:
		{
			break;
		}
	}
}

void Axon_Behaviour_GS2Actor::ExitState( const int iState )
{
	switch ( iState )
	{
		case MOVING_TO_ATTRACTOR:
		{
			Asura_ServerNode_GS2Attractor* pxCurrentAttractor = Asura_ServerNode_GS2Attractor::FindEntity( m_uCurrentEntityGuid );
			if ( pxCurrentAttractor )
			{
				pxCurrentAttractor->StopAttractingMe( GetBrain()->GetOwnerGuid() );
				pxCurrentAttractor->DecPendingCount();
			}

			DestroyAllChildren( true );
			break;
		}

		case AT_NODE:
		{
			m_bLeaveNode = false;
			m_bSkipNextNode = false;

			Asura_ServerNode_GS2Node* pxCurrentNode = Asura_ServerNode_GS2Node::FindEntity( m_uCurrentEntityGuid );
			if ( pxCurrentNode )
			{
				pxCurrentNode->ExitNode( GetBrain()->GetOwnerGuid() );
			}

			DestroyAllChildren( true );
			break;
		}

		case ON_LINK:
		{
			Asura_ServerNode_GS2Spline* pxCurrentSpline = Asura_ServerNode_GS2Spline::FindEntity( m_uCurrentEntityGuid );
			if ( pxCurrentSpline )
			{
				pxCurrentSpline->ExitSpline( GetBrain()->GetOwnerGuid() );

				Asura_ServerNode_GS2Node* pxTargetNode = Asura_ServerNode_GS2Node::FindEntity( pxCurrentSpline->GetTargetGuid() );
				if ( pxTargetNode )
				{
					pxTargetNode->DecPendingCount();
				}
			}

			DestroyAllChildren( true );
			break;
		}

		case FINISHED:
		{
			SetFinished( false );
			break;
		}

		case UNSET:
		case NUM_STATES:
		default:
		{
			break;
		}
	}
}

Axon_Behaviour_GS2Interface* Axon_Behaviour_GS2Actor::GetInterface( Axon_Behaviour* pxBehaviour )
{
	// Potentially unsafe downcast. All behaviours used with gamescene must inherit from Axon_Behaviour_GS2Interface.
	return static_cast< Axon_Behaviour_GS2Interface* >( pxBehaviour );
}

void Axon_Behaviour_GS2Actor::ActionPendingAnimStateCommand()
{
	switch ( m_iPendingAnimSetCmd )
	{
		case PLAY_ANIM_SET:
		{
			Axon_Gamescene_AnimTemplate* pxAnimationTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( m_uPendingAnimSet );
			if ( pxAnimationTemplate )
			{
				StartAnimTemplate( pxAnimationTemplate );
				m_uCurrentAnimSet = m_uPendingAnimSet;
			}
			else
			{
				StopAnimTemplate();
				m_uCurrentAnimSet = uAXON_ANIMATION_SEQUENCE_ID_NONE;
			}

			m_iPendingAnimSetCmd = NO_CMD_PENDING;
			break;
		}

		case STOP_ANIM_SET:
		{
			StopAnimTemplate();
			m_uCurrentAnimSet = uAXON_ANIMATION_SEQUENCE_ID_NONE;

			m_iPendingAnimSetCmd = NO_CMD_PENDING;
			break;
		}

		case NO_CMD_PENDING:
		default:
			break;
	}
}

void Axon_Behaviour_GS2Actor::StopAnimTemplate()
{
	Axon_Gamescene_AnimSequenceData xSequenceData;
	xSequenceData.m_uAnimTemplateSequenceID = uAXON_ANIMATION_SEQUENCE_ID_NONE;
	SendAnimationSequenceData( xSequenceData );

	m_fAnimSequenceStartTime     = 0.0f;
	m_pxCurrentAnimationTemplate = NULL;
}

void Axon_Behaviour_GS2Actor::StartAnimTemplate( Axon_Gamescene_AnimTemplate* pxAnimationTemplate )
{
	m_fAnimSequenceStartTime	 = Axon_Gamescene_AnimTemplate_System::GetTimeForAnimSequence();
	m_pxCurrentAnimationTemplate = pxAnimationTemplate;

	Axon_Gamescene_AnimSequenceData xSequenceData;
	xSequenceData.m_fStartTimer				= m_fAnimSequenceStartTime;
	xSequenceData.m_uAnimTemplateSequenceID = m_pxCurrentAnimationTemplate->m_uUniqueID;
	GetOwnerPosition( xSequenceData.m_xStartPosition );
	Asura_Matrix_3x3 xOrientM;
	GetOwnerOrientation( xOrientM );
	xSequenceData.m_xStartOrientQ.FromMatrix( xOrientM );
	SendAnimationSequenceData( xSequenceData );
}

void Axon_Behaviour_GS2Actor::RestartAnimTemplate( Axon_Gamescene_AnimTemplate* pxAnimationTemplate )
{
	// We're using the anim sequence start time that we've already got.
	m_pxCurrentAnimationTemplate = pxAnimationTemplate;

	Axon_Gamescene_AnimSequenceData xSequenceData;
	xSequenceData.m_fStartTimer				= m_fAnimSequenceStartTime;
	xSequenceData.m_uAnimTemplateSequenceID = m_pxCurrentAnimationTemplate->m_uUniqueID;
	GetOwnerPosition( xSequenceData.m_xStartPosition );
	Asura_Matrix_3x3 xOrientM;
	GetOwnerOrientation( xOrientM );
	xSequenceData.m_xStartOrientQ.FromMatrix( xOrientM );
	SendAnimationSequenceData( xSequenceData );
}

void Axon_Behaviour_GS2Actor::SendAnimationSequenceData( const Axon_Gamescene_AnimSequenceData& xSequenceData ) const
{
	Asura_Message* pxMessage = Asura_Network::CreateMessage( ASURA_MSG_AXON_GS2_ANIM_SEQUENCE_DATA,
		sizeof( Axon_Gamescene_AnimSequenceData ), 0, &xSequenceData );
	if ( pxMessage )
	{
		pxMessage->SetFromGuid( GetBrain()->GetOwnerGuid() );
		pxMessage->SetToGuid( GetBrain()->GetOwnerGuid() );
		Asura_Network::Send( pxMessage );
	}
}

#ifndef ASURA_NO_DEBUG_OUTPUT
const char* Axon_Behaviour_GS2Actor::s_pszStateStrings[ NUM_STATES ] =
{
	"UNSET",
	"MOVING_TO_ATTRACTOR",
	"AT_NODE",
	"ON_LINK",
	"FINISHED"
};

const char* Axon_Behaviour_GS2Actor::GetStateString() const
{
	return s_pszStateStrings[ m_iState ];
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
