#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_GS2Spline.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AXON_BEHAVIOUR_GS2SPLINE_H__
#define __AXON_BEHAVIOUR_GS2SPLINE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_Maths.h"

// axon
#include "Behaviour/BehaviourTypes/Axon_Behaviour_GS2Interface.h"

//////////////////////////////////////////////////////////////////////////////
//
// Axon_GS2_SplineTraverseMode
//
//////////////////////////////////////////////////////////////////////////////

enum Axon_GS2_SplineTraverseMode
{
	AXON_GS2_STM_ANIM_MOVEMENT,
	AXON_GS2_STM_SPLINE_PATH_ANIM_SPEED,
	AXON_GS2_STM_SPLINE_PATH_SPLINE_SPEED,
	AXON_GS2_STM_FREE_MOVEMENT,

	AXON_GS2_STM_NUM_CORE_TRAVERSE_MODES,

	AXON_GS2_STM_FIRST_PROJECT_SPECIFIC = 1000
};

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_GS2Spline
//
//////////////////////////////////////////////////////////////////////////////

class Axon_Behaviour_GS2Spline : public Axon_Behaviour_GS2Interface
{
	typedef Axon_Behaviour_GS2Interface PARENT;

public:
	Axon_Behaviour_GS2Spline( Axon_Brain* pxBrain );
	Axon_Behaviour_GS2Spline( const u_int uBehaviourType, Axon_Brain* pxBrain );

protected:
	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	virtual bool HandleMessage( const Asura_Message* pxMessage );
	virtual void HandleParams( const Axon_BehaviourParams* pxParams );
	virtual bool Activate();
	virtual void Deactivate();
	virtual void PerFrameUpdate( const float fTimeslice );

private:
	enum State
	{
		UNSET,
		FINISHED,
		AM_MOVING,
		SPAS_MOVING,
		SPSS_MOVING,
		FM_MOVING
	};
	void SwitchToState( int iState );
	void ExitState( int iState );
	void EnterState( int iState );
	float UpdateAnimMovement( const float fTimeslice );
	float UpdateSPASMovement( const float fTimeslice );
	float UpdateSPSSMovement( const float fTimeslice );
	void UpdateFreeMovement();

private:
	int   m_iState;
	float m_fTimeInState;
	u_int m_uAnimSet;
	int   m_iTraverseMode;
	float m_fArrivalRange;
	float m_fRemainingTimeslice;
	u_int m_uSplineEventIndex;
	u_int m_uAnimQueryFailCount;

	// Anim movement variables.
	float		   m_fAnimSetDuration;
	Asura_Vector_3 m_xAnimStartPos;
	Asura_Quat     m_xAnimStartQOri;
	Asura_Vector_3 m_xAnimCurrPosOffset;
	Asura_Quat     m_xAnimCurrQOriOffset;

	// Spline movement variables.
	float m_fDistanceAlongSpline;
	float m_fTimeAlongSpline;
	float m_fPrevFollowSurfaceDistance;

	// Free movement variables.
	Axon_Behaviour* m_pxGetToBehaviour;

	static const u_int uCURRENT_CHUNK_VERSION = 1;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AXON_BEHAVIOUR_GS2SPLINE_H__
#endif // ASURA_USE_AXON
