//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_Movement.cpp
//
// Copyright (C) 2009 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_NEW_NAV_STUFF

// header
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Movement.h"

// asura
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

// axon
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"
#include "Core/Axon_Brain.h"

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_Movement
//
//////////////////////////////////////////////////////////////////////////////

Axon_Behaviour_Movement::Axon_Behaviour_Movement( Axon_Brain* pxBrain )
: PARENT( AXON_BEHAVIOUR_TYPE_MOVEMENT, pxBrain )
, m_pxOwnerPhyObj( Asura_ServerEntity_PhysicalObject::FindEntity( GetBrain()->GetOwnerGuid() ) )
, m_xPath()
{
	Asura_Assert( m_pxOwnerPhyObj, "Axon_Behaviour_Movement::Axon_Behaviour_Movement - could not find owner physical object" );
}

bool Axon_Behaviour_Movement::FindPathTo( const Asura_Vector_3& xDestPos )
{
	Asura_Navigation::FromToPathFindFunctor xFromToFunctor;
	xFromToFunctor.SetTestParams( GetTestParams() );
	xFromToFunctor.SetShouldRefinePath( true );
	xFromToFunctor.SetFromToPos( GetOwnerPosition(), xDestPos, GetTestParams() );
	const Asura_Navigation::Path::Status ePathStatus = m_xPath.FindPath( xFromToFunctor );
	OnPathChanged();
	return ( ePathStatus == Asura_Navigation::Path::STATUS_OK );
}

bool Axon_Behaviour_Movement::FindPathAwayFrom( const Asura_Vector_3& xAvoidPos, const float fAvoidDistance )
{
	Asura_Navigation::AwayFromPathFindFunctor xAwayFromFunctor;
	xAwayFromFunctor.SetTestParams( GetTestParams() );
	xAwayFromFunctor.SetShouldRefinePath( true );
	xAwayFromFunctor.SetFromPos( GetOwnerPosition(), GetTestParams() );
	xAwayFromFunctor.SetAvoidPosAndDistance( xAvoidPos, fAvoidDistance );
	const Asura_Navigation::Path::Status ePathStatus = m_xPath.FindPath( xAwayFromFunctor );
	OnPathChanged();
	return ( ePathStatus == Asura_Navigation::Path::STATUS_OK );
}

bool Axon_Behaviour_Movement::FindPath( Asura_Navigation::AStarFunctor& xFunctor )
{
	const Asura_Navigation::Path::Status ePathStatus = m_xPath.FindPath( xFunctor );
	OnPathChanged();
	return ( ePathStatus == Asura_Navigation::Path::STATUS_OK );
}

void Axon_Behaviour_Movement::LookInDirection( const Asura_Vector_3& xDirection )
{
	Asura_Matrix_3x3 xOrientM;
	Asura_Maths::OrientationFromDirection( xOrientM, xDirection );
	SetOwnerOrientation( xOrientM );
}

void Axon_Behaviour_Movement::LookAtPosition( const Asura_Vector_3& xPosition )
{
	const Asura_Vector_3 xDirection = xPosition - GetOwnerPosition();
	// Direction gets normalised in Asura_Maths::OrientationFromDirection anyway.
	LookInDirection( xDirection );
}

void Axon_Behaviour_Movement::TeleportOwnerTo( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation )
{
	m_pxOwnerPhyObj->TeleportTo( xPosition, xOrientation, true );
}

void Axon_Behaviour_Movement::SetOwnerPosition( const Asura_Vector_3& xPosition )
{
	m_pxOwnerPhyObj->SetPosition( xPosition );
}

const Asura_Vector_3& Axon_Behaviour_Movement::GetOwnerPosition() const
{
	return m_pxOwnerPhyObj->GetPosition();
}

void Axon_Behaviour_Movement::SetOwnerOrientation( const Asura_Matrix_3x3& xOrientation )
{
	m_pxOwnerPhyObj->SetOrientation( xOrientation );
}

const Asura_Matrix_3x3& Axon_Behaviour_Movement::GetOwnerOrientation() const
{
	return m_pxOwnerPhyObj->GetOrientation();
}

void Axon_Behaviour_Movement::Deactivate()
{
	StopPathTraversal();
	PARENT::Deactivate();
}

void Axon_Behaviour_Movement::OnPathChanged()
{
	SetInError( false );
	SetFinished( false );
}

bool Axon_Behaviour_Movement::CalculateMovementVector( Asura_Vector_3& xVector, bool& bAtDestination )
{
	const Asura_Vector_3& xOwnerPos = GetOwnerPosition();

	Asura_Vector_3 xPathPosition;
	if ( m_xPath.UpdateNearestPositionOnPath( xOwnerPos, 0.5f, xPathPosition, bAtDestination, GetAPParams() ) )
	{
		xVector = xPathPosition - xOwnerPos;
		return true;
	}
	else
	{
		return false;
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // #ifdef ASURA_USE_NEW_NAV_STUFF
