//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_UseGuardZone.cpp
//
// Copyright (C) 2009 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_NEW_NAV_STUFF

// header
#include "Behaviour/BehaviourTypes/Axon_Behaviour_UseGuardZone.h"

// asura
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Locations/GuardZone/Asura_ServerNode_GuardZone.h"
#include "Entity/Locations/OrientedNode/Asura_ServerNode_OrientedNode.h"
#ifndef ASURA_NO_DEBUG_OUTPUT
#include "Rendering/Asura_Font_System.h"
#endif

// axon
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Movement.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"
#include "Core/Axon_Brain.h"

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_UseGuardZone
//
//////////////////////////////////////////////////////////////////////////////

Axon_Behaviour_UseGuardZone::Axon_Behaviour_UseGuardZone( Axon_Brain* pxBrain )
: PARENT( AXON_BEHAVIOUR_TYPE_USE_GUARD_ZONE, pxBrain )
, m_uGuardZoneGuid( ASURA_GUID_UNREGISTERED )
, m_uGuardPointGuid( ASURA_GUID_UNREGISTERED )
, m_iState( WAITING_FOR_GUARD_ZONE )
, m_fWaitTimer( 0.0f )
{
}

void Axon_Behaviour_UseGuardZone::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
	u_int uVersion;
	xStream >> uVersion;

	PARENT::ReadFromChunkStream( xStream );

	if ( uVersion == uCURRENT_CHUNK_VERSION )
	{
		xStream >> m_iState;
		xStream >> m_uGuardZoneGuid;
		xStream >> m_uGuardPointGuid;
	}
	else
	{
		Asura_Assertf( false, ( "Axon_Behaviour_UseGuardZone::ReadFromChunkStream - unhanded chunk version (%u)", uVersion ) );
	}
}

void Axon_Behaviour_UseGuardZone::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << uCURRENT_CHUNK_VERSION;

	PARENT::WriteToChunkStream( xStream );

	xStream << m_iState;
	xStream << m_uGuardZoneGuid;
	xStream << m_uGuardPointGuid;
}

void Axon_Behaviour_UseGuardZone::Deactivate()
{
	SwitchToState( WAITING_FOR_GUARD_ZONE );
	PARENT::Deactivate();
}

void Axon_Behaviour_UseGuardZone::PostLoadInitialise()
{
	PARENT::PostLoadInitialise();

	if ( m_iState == MOVING_TO_GUARD_POINT )
	{
		Asura_ServerNode_OrientedNode* pxGP = static_cast< Asura_ServerNode_OrientedNode* >( Asura_ServerNode_OrientedNode::FindEntity( m_uGuardPointGuid ) );
		if ( pxGP )
		{
			SwitchToState( MOVING_TO_GUARD_POINT, pxGP );
		}
		else
		{
			SwitchToState( WAITING_FOR_GUARD_POINT );
		}
	}
}

void Axon_Behaviour_UseGuardZone::Update( const float fTimeslice )
{
	PARENT::Update( fTimeslice );

	m_fWaitTimer += fTimeslice;

	switch ( m_iState )
	{
		case WAITING_FOR_GUARD_POINT:
		{
			if ( m_fWaitTimer > 1.0f )
			{
				m_fWaitTimer -= 1.0f;
				const Asura_ServerNode_GuardZone* pxGZ =
					static_cast< Asura_ServerNode_GuardZone* >( Asura_ServerNode_GuardZone::FindEntity( GetGuardZoneGuid() ) );
				if ( pxGZ )
				{
					Asura_ServerNode_OrientedNode* pxGP = pxGZ->GetRandomUnoccupiedGuardPoint();
					if ( pxGP )
					{
						SwitchToState( MOVING_TO_GUARD_POINT, pxGP );
					}
				}
				else
				{
					SwitchToState( WAITING_FOR_GUARD_ZONE );
				}
			}

			break;
		}

		case WAITING_AT_GUARD_POINT:
		{
			// Could add a proper guard point and have weights/guard times/etc.
			if ( m_fWaitTimer > 5.0f )
			{
				const Asura_ServerNode_GuardZone* pxGZ = static_cast< const Asura_ServerNode_GuardZone* >( Asura_ServerNode_GuardZone::FindEntity( GetGuardZoneGuid() ) );
				if ( pxGZ )
				{
					Asura_ServerNode_OrientedNode* pxGP = pxGZ->GetRandomUnoccupiedGuardPoint();
					if ( pxGP )
					{
						SwitchToState( MOVING_TO_GUARD_POINT, pxGP );
					}
					else
					{
						m_fWaitTimer -= 1.0f;
					}
				}
				else
				{
					SwitchToState( WAITING_FOR_GUARD_ZONE );
				}
			}

			break;
		}

		case MOVING_TO_GUARD_POINT:
		{
			Axon_Behaviour_Movement* pxMovement = static_cast< Axon_Behaviour_Movement* >( GetChildWithType( this, AXON_BEHAVIOUR_TYPE_MOVEMENT ) );
			if ( pxMovement )
			{
				if ( pxMovement->GetPathTraversalState() == Axon_Behaviour_Movement::PTS_AT_DESTINATION )
				{
					SwitchToState( WAITING_AT_GUARD_POINT );
				}
				else if ( pxMovement->GetPathTraversalState() == Axon_Behaviour_Movement::PTS_IN_ERROR )
				{
					SwitchToState( WAITING_FOR_GUARD_POINT );
				}
			}
			else
			{
				SwitchToState( WAITING_FOR_GUARD_POINT );
			}

			break;
		}

		default:
			m_uGuardZoneGuid = ASURA_GUID_UNREGISTERED;
			// fallthrough
		case WAITING_FOR_GUARD_ZONE:
			break;
	}
}

void Axon_Behaviour_UseGuardZone::SetGuardZoneGuid( const Asura_Guid uGuardZoneGuid )
{
	m_uGuardZoneGuid = uGuardZoneGuid;
	OnNewGuardZone();
}

void Axon_Behaviour_UseGuardZone::OnNewGuardZone()
{
	const Asura_ServerNode_GuardZone* pxGZ = static_cast< const Asura_ServerNode_GuardZone* >( Asura_ServerNode_GuardZone::FindEntity( GetGuardZoneGuid() ) );
	if ( pxGZ )
	{
		Asura_ServerNode_OrientedNode* pxGP = pxGZ->GetRandomUnoccupiedGuardPoint();
		if ( pxGP )
		{
			SwitchToState( MOVING_TO_GUARD_POINT, pxGP );
		}
		else
		{
			SwitchToState( WAITING_FOR_GUARD_POINT );
		}
	}
	else
	{
		Asura_Assert( false, "Axon_Behaviour_UseGuardZone::OnNewGuardZone - could not find guard zone" );
		SwitchToState( WAITING_FOR_GUARD_ZONE );
	}
}

void Axon_Behaviour_UseGuardZone::SwitchToState( int iNextState, Asura_ServerNode_OrientedNode* pxGP /*= NULL*/ )
{
	// Exit old state.
	switch ( m_iState )
	{
		case WAITING_FOR_GUARD_POINT:
		{
			break;
		}

		case WAITING_AT_GUARD_POINT:
		{
			Asura_ServerNode_OrientedNode* pxOldGuardPoint = static_cast< Asura_ServerNode_OrientedNode* >( Asura_ServerNode_OrientedNode::FindEntity( m_uGuardPointGuid ) );
			if ( pxOldGuardPoint )
			{
				pxOldGuardPoint->SetIsOccupied( false );
			}
			m_uGuardPointGuid = ASURA_GUID_UNREGISTERED;
			break;
		}

		case MOVING_TO_GUARD_POINT:
		{
			if ( iNextState != WAITING_AT_GUARD_POINT )
			{
				// Something's gone wrong.
				Asura_ServerNode_OrientedNode* pxOldGuardPoint = static_cast< Asura_ServerNode_OrientedNode* >( Asura_ServerNode_OrientedNode::FindEntity( m_uGuardPointGuid ) );
				if ( pxOldGuardPoint )
				{
					pxOldGuardPoint->SetIsOccupied( false );
				}
				m_uGuardPointGuid = ASURA_GUID_UNREGISTERED;
			}
			StopMoving();

			break;
		}

		default:
		case WAITING_FOR_GUARD_ZONE:
			break;
	}

	// Enter new state.
	m_iState = iNextState;
	m_fWaitTimer = 0.0f;
	switch ( m_iState )
	{
		case WAITING_FOR_GUARD_POINT:
		{
			break;
		}

		case WAITING_AT_GUARD_POINT:
		{
			break;
		}

		case MOVING_TO_GUARD_POINT:
		{
			Asura_Assert( pxGP, "Axon_Behaviour_UseGuardZone::SwitchToState - must supply a point when switching to MOVING_TO_GUARD_POINT" );
			m_uGuardPointGuid = pxGP->GetGuid();
			pxGP->SetIsOccupied( true );
			if ( !StartMoving( pxGP ) )
			{
				SwitchToState( WAITING_FOR_GUARD_POINT );
			}
			break;
		}

		default:
		case WAITING_FOR_GUARD_ZONE:
			m_uGuardZoneGuid = ASURA_GUID_UNREGISTERED;
			break;
	}
}

bool Axon_Behaviour_UseGuardZone::StartMoving( const Asura_ServerNode_OrientedNode* pxGP )
{
	Axon_Behaviour_Movement* pxMovement = static_cast< Axon_Behaviour_Movement* >( CreateChildWithType( AXON_BEHAVIOUR_TYPE_MOVEMENT ) );
	if ( pxMovement )
	{
		if ( pxMovement->FindPathTo( pxGP->GetPosition() ) && pxMovement->StartPathTraversal() )
		{
			return true;
		}
	}

	return false;
}

void Axon_Behaviour_UseGuardZone::StopMoving()
{
	DestroyChildWithType( AXON_BEHAVIOUR_TYPE_MOVEMENT );
}

#ifndef ASURA_NO_DEBUG_OUTPUT
static const char* s_aszStateStrings[] =
{
	// N.B. Keep in updated with behaviour states.
	"UseGuardZone - WAITING_FOR_GUARD_ZONE",
	"UseGuardZone - WAITING_FOR_GUARD_POINT",
	"UseGuardZone - WAITING_AT_GUARD_POINT",
	"UseGuardZone - MOVING_TO_GUARD_POINT",
};

void Axon_Behaviour_UseGuardZone::RenderDebugInfo()
{
	PARENT::RenderDebugInfo();

	if ( GetBrain()->DebugTextPosOnScreen() )
	{
		Asura_Font_System::Print( GetBrain()->GetDebugTextPos().x, GetBrain()->GetDebugTextPos().y, 0.5f, 0xffffffff, 
			ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING, s_aszStateStrings[m_iState] );
		GetBrain()->IncrementDebugTextPos();
	}
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // #ifdef ASURA_USE_NEW_NAV_STUFF
