#ifdef ASURA_USE_NEW_NAV_STUFF

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_Wander.h
//
// Copyright (C) 2009 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AXON_BEHAVIOUR_WANDER_H__
#define __AXON_BEHAVIOUR_WANDER_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_Maths.h"
#include "Network/Asura_Guid.h"

// axon
#include "AI/Asura_Nav_Types.h"
#include "Behaviour/Axon_Behaviour.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Axon_Behaviour_Movement;
class Axon_Brain;
class Asura_ServerEntity_PhysicalObject;

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Behaviour_Wander
//
//////////////////////////////////////////////////////////////////////////////

class Axon_Behaviour_Wander : public Axon_Behaviour
{
	typedef Axon_Behaviour PARENT;

public:
	Axon_Behaviour_Wander( Axon_Brain* pxBrain );

	virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream );
	virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;

	void SetCentralWanderPos( const Asura_Vector_3 &xCentralPos );
	void SetWanderRange( const float fRange );

	const Asura_Vector_3& GetCentralWanderPos() const { return m_xCentralWanderPosition; }
	float GetWanderRange() const { return m_fWanderRange; }
	const Asura_ServerEntity_PhysicalObject* GetOwnerPhyObj() const { return m_pxOwnerPhyObj; }

protected:
	enum State
	{
		WAITING_AT_POSITION,
		MOVING_TO_POSITION,
		MOVING_INTO_RANGE,
	};

	virtual void Deactivate();
	virtual void PostLoadInitialise();
	virtual void PerFrameUpdate( const float fTimeslice );
	virtual void OnWanderParamChange();
	virtual void SetWaitAtPositionTimer();
	virtual void SwitchToState( int iNextState );
	virtual bool StartWanderMove();
	virtual bool StartMoveIntoRange();

	float GetWaitAtPositionTimer() const { return m_fWaitAtPositionTimer; }
	bool GetIsInRange() const { return m_bInRange; }

#ifndef ASURA_NO_DEBUG_OUTPUT
	virtual void RenderDebugInfo();
#endif

private:
	bool UpdateOwnerPhyObjPtr();
	void UpdateIsInRange();

private:
	Asura_Vector_3 m_xCentralWanderPosition;
	float m_fWanderRange;

	int        m_iState;
	float      m_fTimer;
	float	   m_fWaitAtPositionTimer;

	union 
	{
		u_int m_uWanderFlags;
		struct 
		{
			u_int	m_bInRange		: 1,
					_UNUSED			: 31;
		};
	};

	Asura_ServerEntity_PhysicalObject* m_pxOwnerPhyObj;

	static const u_int uCURRENT_CHUNK_VERSION = 0;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AXON_BEHAVIOUR_WANDER_H__
#endif // #ifdef ASURA_USE_NEW_NAV_STUFF
