#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Axon_Sense.h
//
// Created by NJD - 29 August 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __AXON_SENSE_H__
#define __AXON_SENSE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_Maths.h"

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Sense
//
// NOTE: Axon_Sense contains a position and direction member, as the vast
// majority of senses will require this information. However, it is up to the
// project specific sense to keep these members up to date before using any
// sense's ShouldCull... methods.
//
//////////////////////////////////////////////////////////////////////////////

class Asura_Blueprint;
class Asura_Chunk_Stream;
class Asura_Message;

template< typename KnowledgeType >
class Axon_Knowledge_System;

template < class Knowledge_Type >
class Axon_Sense
{
public:
    Axon_Sense( Axon_Knowledge_System< Knowledge_Type >* pxKnowledgeSystem )
    : m_pxKnowledgeSystem( pxKnowledgeSystem )
    , m_xPosition( Asura_Maths::ZeroVector3 )
    , m_xDirection( Asura_Maths::ZeroVector3 )
    {}
    virtual ~Axon_Sense() {}
    virtual void InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint ) {}
    virtual void ReadFromChunkStream( Asura_Chunk_Stream& xStream )      = 0;
    virtual void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const = 0;
    virtual void Update( const float fTimeslice )                        = 0;
    virtual bool HandleMessage( const Asura_Message* pxMessage ) { return false; }
#ifndef ASURA_NO_DEBUG_OUTPUT
    virtual void RenderDebugInfo() {}
#endif

    void SetPosition( const Asura_Vector_3& xPosition )   { m_xPosition = xPosition; }
    void SetDirection( const Asura_Vector_3& xDirection ) { m_xDirection = xDirection; }

protected:
    Axon_Knowledge_System< Knowledge_Type >* GetKnowledgeSystem() const { return m_pxKnowledgeSystem; }

protected:
    Axon_Knowledge_System< Knowledge_Type >* m_pxKnowledgeSystem;
    Asura_Vector_3                           m_xPosition;
    Asura_Vector_3                           m_xDirection;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__AXON_SENSE_H__
#endif
