//////////////////////////////////////////////////////////////////////////////
//
// Axon_Knowledge_System_Standard.cpp
//
// Created by NJD - 29 August 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Knowledge/KnowledgeSystemTypes/Axon_Knowledge_System_Standard.h"

// asura
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Network/Asura_Message.h"

// axon
#include "Knowledge/SenseTypes/Axon_Sense_Hearing.h"
#include "Knowledge/SenseTypes/Axon_Sense_Touch.h"
#include "Knowledge/SenseTypes/Axon_Sense_Vision.h"

//////////////////////////////////////////////////////////////////////////////
//
// Axon_Knowledge_System_Standard
//
//////////////////////////////////////////////////////////////////////////////

Axon_Knowledge_System_Standard::Axon_Knowledge_System_Standard()
: m_pxVision( NULL )
, m_pxHearing( NULL )
, m_pxTouch( NULL )
{
}

Axon_Knowledge_System_Standard::~Axon_Knowledge_System_Standard()
{
}

void Axon_Knowledge_System_Standard::InitialiseFromBlueprint( const Asura_Blueprint& xBlueprint )
{
    if ( m_pxVision )  { m_pxVision->InitialiseFromBlueprint( xBlueprint );  }
    if ( m_pxHearing ) { m_pxHearing->InitialiseFromBlueprint( xBlueprint ); }
    if ( m_pxTouch )   { m_pxTouch->InitialiseFromBlueprint( xBlueprint );   }
}

void Axon_Knowledge_System_Standard::ReadFromChunkStream( Asura_Chunk_Stream& xStream )
{
    int iChunkVersion;
    xStream >> iChunkVersion;

    // We would do our parent chunk now, if it existed.

    if ( iChunkVersion == CURRENT_CHUNK_VERSION )
    {
        if ( m_pxVision )  { m_pxVision->ReadFromChunkStream( xStream );  }
        if ( m_pxHearing ) { m_pxHearing->ReadFromChunkStream( xStream ); }
        if ( m_pxTouch )   { m_pxTouch->ReadFromChunkStream( xStream );   }
    }
    else if ( iChunkVersion == 0 )
    {
        if ( m_pxVision )  { m_pxVision->Axon_Sense_Vision::ReadFromChunkStream( xStream );  }
        if ( m_pxHearing ) { m_pxHearing->Axon_Sense_Hearing::ReadFromChunkStream( xStream ); }
        if ( m_pxTouch )   { m_pxTouch->Axon_Sense_Touch::ReadFromChunkStream( xStream );   }
    }
    else
    {
        Asura_Assertf( false, ( "Axon_Knowledge_System_Standard::ReadFromChunkStream - bad chunk version (%d)", iChunkVersion ) );
    }
}

void Axon_Knowledge_System_Standard::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
    // Our version.
    xStream << CURRENT_CHUNK_VERSION;

    // We would do our parent chunk now, if it existed.

    // Our data.
    if ( m_pxVision )  { m_pxVision->WriteToChunkStream( xStream );  }
    if ( m_pxHearing ) { m_pxHearing->WriteToChunkStream( xStream ); }
    if ( m_pxTouch )   { m_pxTouch->WriteToChunkStream( xStream );   }
}

void Axon_Knowledge_System_Standard::Initialise()
{
    m_pxVision  = CreateVisionSense();
    m_pxHearing = CreateHearingSense();
    m_pxTouch   = CreateTouchSense();
}

void Axon_Knowledge_System_Standard::Update( const float fTimeslice )
{
    if ( m_pxVision )  { m_pxVision->Update( fTimeslice );  }
    if ( m_pxHearing ) { m_pxHearing->Update( fTimeslice ); }
    if ( m_pxTouch )   { m_pxTouch->Update( fTimeslice );   }
}

void Axon_Knowledge_System_Standard::Shutdown()
{
    DestroyVisionSense( m_pxVision );
    DestroyHearingSense( m_pxHearing );
    DestroyTouchSense( m_pxTouch );
}

bool Axon_Knowledge_System_Standard::HandleMessage( const Asura_Message* pxMessage )
{
    bool bHandled = false;
	if ( m_pxVision )  { bHandled |= m_pxVision->HandleMessage( pxMessage );  }
	if ( m_pxHearing ) { bHandled |= m_pxHearing->HandleMessage( pxMessage ); }
	if ( m_pxTouch )   { bHandled |= m_pxTouch->HandleMessage( pxMessage );   }
    return bHandled;
}

#ifndef ASURA_NO_DEBUG_OUTPUT
void Axon_Knowledge_System_Standard::RenderDebugInfo()
{
    if ( m_pxVision )  { m_pxVision->RenderDebugInfo();  }
    if ( m_pxHearing ) { m_pxHearing->RenderDebugInfo(); }
    if ( m_pxTouch )   { m_pxTouch->RenderDebugInfo();   }
}
#endif

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

Axon_Sense_Vision* Axon_Knowledge_System_Standard::CreateVisionSense()
{
    return new Axon_Sense_Vision( this );
}

void Axon_Knowledge_System_Standard::DestroyVisionSense( Axon_Sense_Vision*& pxSense )
{
    delete pxSense;
    pxSense = NULL;
}

Axon_Sense_Hearing* Axon_Knowledge_System_Standard::CreateHearingSense()
{
    return new Axon_Sense_Hearing( this );
}

void Axon_Knowledge_System_Standard::DestroyHearingSense( Axon_Sense_Hearing*& pxSense )
{
    delete pxSense;
    pxSense = NULL;
}

Axon_Sense_Touch* Axon_Knowledge_System_Standard::CreateTouchSense()
{
    return new Axon_Sense_Touch( this );
}

void Axon_Knowledge_System_Standard::DestroyTouchSense( Axon_Sense_Touch*& pxSense )
{
    delete pxSense;
    pxSense = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif
