// DSP_AddNoise_SPU.cpp
// Thomas Young - 23.03.09
// Copyright (C) 2009 Rebellion, All Rights Reserved.

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Asura_DSP_SPU.h"
#include "Sound/Asura_PS3_DSP_Params.h"

#include <stdlib.h>
#include <math.h>
#include <spu_mfcio_gcc.h>

// G L O B A L   V A R I A B L E S ///////////////////////////////////////////

static const unsigned int uNOISE_RANDOM_SEED	= 1531641981;

// E N U M E R A T I O N S ///////////////////////////////////////////////////

enum SPU_GET_PUT
{
	SPU_GET = 0,
	SPU_PUT = 1
};

// F U N C T I O N   P R O T O T Y P E S  ///////////////////////////////////

void DMATransfer			( void *pSPUAddr, void* pRAMAddr, unsigned int uSize, SPU_GET_PUT eGetOrPut, MS_DSP_INFO* pxDSP_Info );
void WaitforDMACompletion	( );

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

void DSPInit ( char *pcParams, void *pWorkArea, MS_DSP_INFO *pxDSP_Info )
{
	Asura_PS3_DSP_Params::AddNoise* pxParams = reinterpret_cast< Asura_PS3_DSP_Params::AddNoise* >( pcParams ) ;

	if( pxParams->m_uScratchMemSize < Asura_PS3_DSP_Params::AddNoise::uSCRATCH_MEMORY_REQ )
	{
		return;
	}

	float* pfStatic = reinterpret_cast< float* >( pWorkArea );

	// Make some pink-ish noise
	srand( uNOISE_RANDOM_SEED );
	const float fRecip = 1.0f / 512.0f;
	for( float fFreq = 2.0f; fFreq < 512.0f; fFreq *= 1.5f )
	{
		float fPhase = static_cast<float>( rand() % 10000 ) / 10000.0f * 6.2831f;
		float fContribution = 1.0f - ( 1.0f / 512.0f * fFreq );
		for( unsigned int uSample = 0; uSample < Asura_PS3_DSP_Params::AddNoise::uNOISE_SAMPLES; ++ uSample )
		{
			pfStatic[ uSample ] += sinf( fFreq * fRecip * static_cast<float>( uSample ) * 6.28318f + fPhase ) * fContribution;
			if( pfStatic[ uSample ] > 1.0f )
			{
				pfStatic[ uSample ] = 1.0f;
			}
			if( pfStatic[ uSample ] < -1.0f )
			{
				pfStatic[ uSample ] = -1.0f;
			}
		}
	}

	// Now copy this data into main memory so we can get at it each frame
	DMATransfer( pfStatic, pxParams->m_pScratchMem, Asura_PS3_DSP_Params::AddNoise::uSCRATCH_MEMORY_REQ, SPU_PUT, pxDSP_Info );
}


int DSPApply ( void* pInput, void* pOutput, char *pcParams, void *pWorkArea, MS_DSP_INFO *pxDSP_Info )
{
	(void) pxDSP_Info;	// Prevent compiler warnings

	Asura_PS3_DSP_Params::AddNoise* pxParams = reinterpret_cast< Asura_PS3_DSP_Params::AddNoise* >( pcParams ) ;
	
	if( pxParams->m_uScratchMemSize < Asura_PS3_DSP_Params::AddNoise::uSCRATCH_MEMORY_REQ )
	{
		return 0;
	}

	DMATransfer( pWorkArea, pxParams->m_pScratchMem, Asura_PS3_DSP_Params::AddNoise::uSCRATCH_MEMORY_REQ, SPU_GET, pxDSP_Info );
		
	float* pfStatic = reinterpret_cast< float* >( pWorkArea );
	float* pfInput = reinterpret_cast< float* >( pInput );
	float* pfOutput = reinterpret_cast< float* >( pOutput );

	// Apply the noise
	unsigned int uStatic = 0;
	for( int iSample = 0; iSample < 512; ++iSample )
	{
		// Move the noise towards 1 to 'reduce' it when multiplying by the original signal
		pfOutput[ iSample ] = pfInput[ iSample ] * ( 1.0f - ( 1.0f - pfStatic[ iSample ] ) * pxParams->m_fNoiseAmount );

		++uStatic;
		if( uStatic == Asura_PS3_DSP_Params::AddNoise::uNOISE_SAMPLES )
		{
			uStatic = 0;
		}
	}

	return 0;
}


void DSPPlugInSetup( DSPPlugin* pxPluginDetails )
{
	pxPluginDetails->DSPInit	= (int(*)(void*))( DSPInit );
	pxPluginDetails->DSPApply	= (int(*)(void*))( DSPApply);
	pxPluginDetails->Domain	= MS_DSP_TIMEBASE;
}


void DMATransfer( void *pSPUAddr, void* pRAMAddr, unsigned int uSize, SPU_GET_PUT eGetOrPut, MS_DSP_INFO* pxDSP_Info )
{
	if( eGetOrPut == SPU_GET )
	{
		spu_mfcdma32 ( pSPUAddr, reinterpret_cast<unsigned int>(pRAMAddr), uSize, 0, MFC_GET_CMD ); 
		pxDSP_Info->bandwidthIn += uSize;
		WaitforDMACompletion();
	}
	else
	{
		spu_mfcdma32( pSPUAddr, reinterpret_cast<unsigned int>(pRAMAddr), uSize, 0, MFC_PUT_CMD );
		pxDSP_Info->bandwidthOut += uSize;
	}
}

void WaitforDMACompletion ()
{
	spu_writech( MFC_WrTagUpdate, 0x0 );
	while ( spu_readchcnt( MFC_WrTagUpdate ) !=1 ) {};
	spu_readch(MFC_RdTagStat);
	spu_writech(MFC_WrTagMask, ( 0xffff));
	spu_writech(MFC_WrTagUpdate, 0x2);
	spu_readch(MFC_RdTagStat);
}
