#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Tools_AnimTemplates_Play.h
//
// Created by NJD - 07 November 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_ANIMTEMPLATES_PLAY_H__
#define __TOOLS_ANIMTEMPLATES_PLAY_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Animation;
class Asura_Photon;
class Axon_Gamescene_AnimTemplate;
class Axon_Gamescene_FidgetAnimTemplate;
class Axon_Gamescene_FidgetAnimTemplate_Entry;

//////////////////////////////////////////////////////////////////////////////
//
// Tools_AnimTemplates_Play
//
//////////////////////////////////////////////////////////////////////////////

class Tools_AnimTemplates_Preview
{
public:
	Tools_AnimTemplates_Preview();
	~Tools_AnimTemplates_Preview() {Shutdown();}

	void Initialise();
	void Shutdown();
	void Render(Asura_Vector_3 xMovementOffset, Asura_Matrix_3x3 xOrientation, float fTime);
	void Render();
	void Play( const Asura_String& xSkinName, Asura_Hash_ID uPrevAnimHash, Asura_Hash_ID uPostAnimHash, void* pScoringData,
			   Axon_Gamescene_AnimTemplate* pxTemplate, const Asura_Vector_3& xStartPosition, const Asura_Matrix_3x3& xStartOrientation );
	void Stop();
	void NextAnimation();
	bool IsAnimationFinished();
	void PlayFidget();

	u_int GetAnimationTemplateType();

	void SetLoop(bool bLoop) { m_bLoop = bLoop; }
	bool IsPlaying() { return m_bPlaying; }

	class Asura_Animation* GetHierarchy() {return m_pxHierarchy;}

private:
	Axon_Gamescene_AnimTemplate*		m_pxTemplate;
	Axon_Gamescene_FidgetAnimTemplate*	m_pxFidgetTemplate;
	Axon_Gamescene_FidgetAnimTemplate_Entry* m_pxPrevChosenFidget;
	Asura_Animation*					m_pxHierarchy;
	Asura_Hash_ID                       m_uPrevAnimHash;
	Asura_Hash_ID                       m_uPostAnimHash;
	void*								m_pScoringData;
	Asura_Vector_3                      m_xPositionAtStartOfAnim;
	Asura_Matrix_3x3                    m_xOrientationAtStartOfAnim;
	Asura_Vector_3                      m_xPositionAtStartOfPlay;
	Asura_Matrix_3x3                    m_xOrientationAtStartOfPlay;
	bool                                m_bPlaying;
	bool								m_bLoop;
	int                                 m_iCurrentAnim;
	float                               m_fDurationRemaining;
	float                               m_fTimeToNextFidget;
	float								m_fTimeElapsed;
	// We need to remember the hash of our last animation so we can get the controller.
	Asura_Hash_ID                       m_uCurrentAnimControllerHash;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_AnimTemplates_Play
// Used by Tools_AnimTemplates to preview templates.
//
//////////////////////////////////////////////////////////////////////////////

class Tools_AnimTemplates_Play
{
public:
	// As we're previewing a single template at a time we always loop.
	static void Initialise() { m_xPreview.Initialise(); m_xPreview.SetLoop( true ); }
    static void Shutdown() { m_xPreview.Shutdown(); }
    static void Render() { m_xPreview.Render(); }
    static void Play( const Asura_String& xSkinName, Asura_Hash_ID uPrevAnimHash, Asura_Hash_ID uPostAnimHash, void* pScoringData,
					  Axon_Gamescene_AnimTemplate* pxTemplate, const Asura_Vector_3& xStartPosition, const Asura_Matrix_3x3& xStartOrientation )
	{
		m_xPreview.Play(xSkinName, uPrevAnimHash, uPostAnimHash, pScoringData, pxTemplate, xStartPosition, xStartOrientation);
	}
    static void Stop() { m_xPreview.Stop(); }

private:
    static Tools_AnimTemplates_Preview m_xPreview;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_ANIMTEMPLATES_PLAY_H__
#endif // ASURA_USE_AXON
