//
// Tools_Animation_IK_Template_System.h
//
// Created by Richard Rice - 14/05/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_ANIMATION_IK_TEMPLATE_SYSTEM_H__
#define __TOOLS_ANIMATION_IK_TEMPLATE_SYSTEM_H__
#ifdef ASURA_USE_ANIMATION_IK_SOLVER

#include "Interface/Asura_Dialog.h"

class Asura_StringTree;
class Asura_Animation_IK_Template;
class Asura_Animation_IK_Template_Chain;
class Asura_Hierarchy_Skin;
class Asura_Overlay;

class Tools_Animation_IK_Template_System
{
public :
	static void Initialise();
	static void Shutdown();
	static void Update();

	static void CreateOverlay();
	static void ToggleOverlay();
	static bool IsOverlayOpen();
	static void CloseOverlay();

	//flag the IK template file as modified
	static void SetModified(bool bWarnAlways = false);
	static bool IsModified() {return s_bModified;}

	static const char* GetIKTemplateFilename() {return "Misc\\IKTemplates.asr";}
	static void SaveFile();

	//adds all the templates to the file
	static bool AddToExport(HANDLE pHandle);

	static void EditTemplate(Asura_Hash_ID uTemplateID, Asura_Hash_ID uChainID);

private :
	static void TransferModeData(int iMode);

	//main overlay functions
	static void CreateTemplate();
	static void DeleteTemplate();
	static void EditTemplate();
	static bool IsAnyTemplateSelected();
	static void RenameTemplate();

	static void UpdateTemplateNameList();
	static void DeleteTemplateNameList();
	
	//edit template overlay functions
	static void CreateEditTemplateOverlay(Asura_Animation_IK_Template* pxTemplate);
	static void CloseEditTemplateOverlay();

	//re-makes the edit template overlay if it is currently open
	static void RefreshEditTemplateOverlay();

	static void AddOverlayControls_Chain(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, Asura_Animation_IK_Template_Chain* pxChain);
	static void AddOverlayControls_Chain_LOS(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, Asura_Animation_IK_Template_Chain* pxChain);
	static void AddOverlayControls_Chain_MarkerPoint(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, Asura_Animation_IK_Template_Chain* pxChain);

	static void CreateChain();
	static void DeleteChain();
	static bool IsAnyChainSelected();

	static void UpdateChainNameList();
	static void DeleteChainNameList();

	static void UpdateBoneNameList(Asura_Animation_IK_Template_Chain* pxChain);
	static void DeleteBoneNameList();

	static bool IsEnabled_PosSmoothing();
	static bool IsEnabled_OriSmoothing();

	static void SetTwistBone();
	static void UpdateTwistBoneControl();

	static void SetMarkerPoint();
	static void UpdateMarkerPointControl(Asura_Dialog* pxDialog = 0);

	static void SelectFirstAndLastBone();

	static void RenameChain();

	static void SelectSkin();
	static void UpdateTemplateFromSkin(Asura_Animation_IK_Template* pxTemplate);
	static void Verify();

	static int GetBoneIndex(const Asura_Hierarchy_Skin* pxSkin, const char* szName);



	static const char* GetChainTypeName(int iType);
	static const char* GetChainTypeHelpText(int iType);

	static int SelectBone(const Asura_Hierarchy_Skin* pxSkin, const char* szTitle, int iInitialSelection = -1, bool bAllowNone = false);
	static const char* SelectMarkerPoint(const Asura_Hierarchy_Skin* pxSkin, const char* szTitle, const char* szInitialSelection = 0);


	//main overlay vars
	static Asura_Vector_2 s_xMainOverlayPosition;
	static u_int s_uMainOverlayMinimised;

	//for list control for selecting a template
	static Asura_StringTree s_xTemplateNameTree;
	static u_int s_uTemplateNameListSize;
	static const char** s_pszTemplateNameList;
	static int s_iSelectedTemplateIndex;//index into displayed list rather than into the template system
	static Asura_List_Dialog* s_pxTemplateListDialog;


	//edit template overlay vars
	static Asura_Vector_2 s_xEditOverlayPosition;
	static Asura_Animation_IK_Template* s_pxEditedTemplate;
	static Asura_Hierarchy_Skin* s_pxEditedTemplateSkin;

	//IK chains in current template
	static u_int s_uChainNameListSize;
	static const char** s_pszChainNameList;
	static int s_iSelectedChainIndex;
	static bool s_bSelectedChainChanged;

	//bones in current chain
	static u_int s_uBoneNameListSize;
	static char** s_pszBoneNameList;
	static int s_iSelectedBoneIndex;
	static Asura_List_Dialog* s_pxBoneListDialog;
	static bool s_bSelectedBoneChanged;

	static bool s_bTemplateChanged;

	static bool s_bModified;//whether the file needs saving
	
};

//drop down list for selecting an IK template
class Tools_PickIKTemplate_Dialog : public Asura_Hash_DropDownList_Dialog
{
public :
	Tools_PickIKTemplate_Dialog(Asura_Vector_2& xOffset,int iNumberOfLines,Asura_Hash_ID* puIKTemplate,bool* pbChangedFlag,const char* szName = NULL)
	:Asura_Hash_DropDownList_Dialog(xOffset)
	{
		SetOnlyUpdateSelectionIndexOnClick(true);
		Initialise(xOffset, iNumberOfLines, puIKTemplate, pbChangedFlag, szName);
	}

	static void InvalidateLists(){s_iListValidID++;}
	
protected :
	virtual u_int GetNumberOfHashValues() const;
	
	virtual const char* GetNameByIndex(int iIndex) const;
	virtual int GetListValidID() const {return s_iListValidID;}

	static int s_iListValidID;

};





#endif //ASURA_USE_ANIMATION_IK_SOLVER
#endif //__TOOLS_ANIMATION_IK_TEMPLATE__SYSTEM_H__
