////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//
// Tools_Mode_AssetManager.h
//
// Created by Dan Clarke - 16/03/2008
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_MODE_ASSETMANAGER_H__
#define __TOOLS_MODE_ASSETMANAGER_H__

#ifdef ASURA_USE_ASSET_MANAGER

#define STATIC_VOID_FN_VOIDSTAR(functionName)\
	static void _ ## functionName (void* pxVoid){s_pxThis->functionName(pxVoid);}\
	void functionName(void* pxVoid);

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Tools_Mode.h"
#include "Animation/Asura_Animation.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct AssetInformationStruct
{
	Asura_String xASRFileName;
	Asura_String xTextureFileName;
	Asura_String xThumbnailFileName;
	int iPolyCount;
	Asura_Vector_3 xBoundingBoxSize;
	Asura_NameList xNameList;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Tools_Mode_AssetManager : public Tools_Mode
{
	public:
		Tools_Mode_AssetManager();
		
		~Tools_Mode_AssetManager()
		{
			s_pxThis = 0;
		}

		virtual void InitialiseMode(void);
		virtual void ShutdownMode(void);

		virtual void Update(void);
		void RenderUI(void);
		void EarlyRender();

		virtual void QuickSave(void);
		virtual void PreSave();

 		bool IsModified();

		virtual void RenderWorld_PreToneMap();
		virtual void RenderWorld_PostToneMap();

	protected:
		const Asura_Vector_3& GetHierarchyModeOrigin();

		Asura_Handle xMenuHandle;

	private:
		STATIC_VOID_FN(LoadAsset);
		STATIC_VOID_FN(AddToDatabase);
		STATIC_VOID_FN(CloseAssetInfoOverlay);
		STATIC_VOID_FN(SetToDefaultBackgroundColour);

		STATIC_VOID_FN_VOIDSTAR(ChangeToStockBGColour1);
		STATIC_VOID_FN_VOIDSTAR(ChangeToStockBGColour2);
		STATIC_VOID_FN_VOIDSTAR(ChangeToStockBGColour3);
		STATIC_VOID_FN_VOIDSTAR(ChangeToStockBGColour4);
		STATIC_VOID_FN(ChangeToCustomBGColour);

		void TransferModeData(int mode);
		void CreateAssetInfoOverlay();
		void HandleKeyboard(void);
		void AddTextureToList(Asura_Mesh_TextureInfo* pxTexInfo);
		void DisableOldLights();
		void RestoreOldLights();

		STATIC_VOID_FN(ToggleMainOverlay)
		STATIC_VOID_FN(RequestScreenGrab)
		STATIC_BOOL_FN(AssetManagerOverlayIsOn)
		void CreateMainOverlay(void);

		static u_int g_auStockBGColours[4];

		Asura_Vector_2 xAssetManagerModePosition;
		bool bAssetManagerOpen;

		Asura_Overlay* m_pxAssetInfoOverlay;
		static Asura_Vector_2 s_xAssetInfoOverlayPosition;

		Asura_Animation* m_pxAnimation;

		AssetInformationStruct m_xAssetInfo;

		enum { NUM_TEMP_LIGHTS = 3 };
		int m_aiTempLightToolsIDs[NUM_TEMP_LIGHTS];

		Asura_String m_xNotesString;
		Asura_String m_xArtistString;
		Asura_String m_xTagsString;

		bool m_bWasSkyboxVisible;
		u_int m_uBackgroundColour;
		bool m_bRequestingScreengrab;
		List<struct AMLightInfoStruct*> m_xOldLightColourList;

		class Asura_DropDown_List_Dialog* m_pxCategoryDropdown;
		static const char* s_paszCategoryNames[];
		u_int m_uCategorySelectionIndex;

		List<class Asura_Photon*> m_xPhotonsActiveAtStart;

		static Tools_Mode_AssetManager* s_pxThis; // Pointer to the mode for use by the static members
		Asura_CameraData::HDRSettings m_xOldHDRSettings;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_ASSET_MANAGER
#endif // __TOOLS_MODE_ASSETMANAGER_H__

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
