//
// Tools_Attractor_System.cpp
//
// Created by JGP 10/05/06
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_USE_ATTRACTORS

// This file's header
#include "Tools_Attractor_System.h"

// Asura headers
#include "Attractors/Asura_Attractor_System.h"
#include "Attractors/Asura_Attractor_Compiled.h"
#include "Core/Memory/Asura_Mem.h"
#include "Entity/Attractors/Asura_ServerEntity_AttractorController.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

#include "Tools_Attractor_Node.h"
#include "Tools_Attractor.h"
#include "Tools_Object_Attractor_Node.h"
#include "EntityTemplate/Tools_EntityTemplate_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_DynamicArray.h"
#include "Objects/Tools_Object_System.h"


// Static singleton access
Tools_Attractor_System* Tools_Attractor_System::s_pxInstance;



///////////////////////////////////////////////////////////////////////////////////////
// Singleton
///////////////////////////////////////////////////////////////////////////////////////


Tools_Attractor_System* Tools_Attractor_System::Initialise()
{
	s_pxInstance = new Tools_Attractor_System();
	return s_pxInstance;
}


void Tools_Attractor_System::Shutdown()
{
	delete s_pxInstance;
	s_pxInstance = NULL;
}


void Tools_Attractor_System::Reset()
{
	GetInstance()->ClearLists();
}


///////////////////////////////////////////////////////////////////////////////////////
// Construct / Destruct
///////////////////////////////////////////////////////////////////////////////////////


Tools_Attractor_System::Tools_Attractor_System()
:m_xAttractorList(5, true)
{
	m_xCommonIDHandler.Initialise(1);
	m_xMissionIDHandler.Initialise(1|TOOLS_ATTRACTORID_MISSIONFILE);
}


Tools_Attractor_System::~Tools_Attractor_System()
{
	ClearLists();
}


///////////////////////////////////////////////////////////////////////////////////////
// Types
///////////////////////////////////////////////////////////////////////////////////////

void Tools_Attractor_System::RegisterType( const char* const szTypeName,u_int uColour)
{
	RegisterType(Tools_Attractor_Type(szTypeName,uColour));
}

void Tools_Attractor_System::RegisterType( const Tools_Attractor_Type& xNewType )
{
	Asura_Hash_ID uHash = Asura_GetHashID(xNewType.m_xName.GetString());
	
	Tools_Attractor_Type* pxType = m_xTypeTree.AddNode(uHash);	
	if(!pxType)
	{
		Asura_Assertf(0,("Failed to register environment hook type %s. (Another already exists with the same hash value)",xNewType.m_xName.GetString()));
		return;
	}

	*pxType = xNewType;
}

void Tools_Attractor_System::UnregisterAllTypes()
{
    m_xTypeTree.Reset();
}

void Tools_Attractor_System::UpdateAttractorsFromType()
{
	for(Asura_Hashed_List_It<Tools_Attractor> xIt(&m_xAttractorList);!xIt.Done();xIt.Next())
	{
		xIt.GetCurrent()->UpdateFromType();
	}
}


///////////////////////////////////////////////////////////////////////////////////////
// Lists
///////////////////////////////////////////////////////////////////////////////////////


void Tools_Attractor_System::ClearLists()
{
	for ( Tools_Attractor_Node_List* pxItNodes = m_xNodeList.Front(); pxItNodes; pxItNodes = pxItNodes->Next() )
	{
		delete pxItNodes->Data();
	}
	m_xNodeList.Clear();

	for(Asura_Hashed_List_It<Tools_Attractor> xIt(&m_xAttractorList);!xIt.Done();xIt.Next())
	{
		delete xIt.GetCurrent();
	}
	m_xAttractorList.Clear();
}


Tools_Attractor_Node_List* Tools_Attractor_System::GetNodeList()
{
	return m_xNodeList.Front();
}




///////////////////////////////////////////////////////////////////////////////////////
// Add / Delete attractors and nodes
///////////////////////////////////////////////////////////////////////////////////////


void Tools_Attractor_System::AddAttractor( Tools_Attractor* const pxAttractor )
{
	Asura_Assert(pxAttractor->GetID(),"Environment hook needs to have been assigned an ID first");
	m_xAttractorList.AddToList( pxAttractor );
}


void Tools_Attractor_System::AddAttractorNode( Tools_Attractor_Node* const pxNode )
{
	m_xNodeList.Push_Back( pxNode );
}


void Tools_Attractor_System::DeleteAttractorNode( Tools_Attractor_Node* const pxNode )
{

	// store attractor
	Tools_Attractor* pxAttractor = pxNode->GetAttractor();

	// remove from list
	Tools_Attractor_Node_List* pxListEntry = m_xNodeList.Find( pxNode );
	m_xNodeList.Remove( pxListEntry );
	
	// and delete the template
	delete pxNode;

	// if attractor is now empty then delete it

	if ( pxAttractor->GetFirstNode() == NULL )
	{
		DeleteAttractor(pxAttractor);
	}

}

void Tools_Attractor_System::DeleteAttractor(Tools_Attractor* pxAttractor)
{
	Asura_Assert(!pxAttractor->GetFirstNode(),"Expecting environment hook to have had all nodes deleted already");

	// remove from list
	m_xAttractorList.RemoveFromList(pxAttractor);

	// remove any links to this attractor
	for(Asura_Hashed_List_It<Tools_Attractor> xIt(&m_xAttractorList);!xIt.Done();xIt.Next())
	{
		xIt.GetCurrent()->RemoveLink(pxAttractor->GetID());
	}
		
	// and delete the template
	delete pxAttractor;
}

void Tools_Attractor_System::DeleteAttractorByID(u_int uID,bool bRecycleIDs)
{
	Tools_Attractor* pxAttractor = GetAttractorByID(uID);
	if(!pxAttractor) return;

	pxAttractor->SetModified();

	//delete the nodes
	Tools_Attractor_Node* pxNode = pxAttractor->GetFirstNode();
	while(pxNode)
	{
		Tools_Attractor_Node* pxNext = pxNode->GetNext();
		//delete the tools object if it exists
		Tools_Object* pxObject = pxNode->GetToolsNode();
		if(pxObject) Tools_Object_System::DeleteObject(pxObject);

		// remove node from list
		Tools_Attractor_Node_List* pxListEntry = m_xNodeList.Find( pxNode );
		m_xNodeList.Remove( pxListEntry );

		if(bRecycleIDs) RecycleAttractorID(pxNode->GetID());

		//delete and move on to next
		delete pxNode;
		pxNode = pxNext;
	}

	//delete the attractor itself
	m_xAttractorList.RemoveFromList(pxAttractor);

	for(Asura_Hashed_List_It<Tools_Attractor> xIt(&m_xAttractorList);!xIt.Done();xIt.Next())
	{
		xIt.GetCurrent()->RemoveLink(pxAttractor->GetID());
	}

	if(bRecycleIDs) RecycleAttractorID(pxAttractor->GetID());
	delete pxAttractor;
}


///////////////////////////////////////////////////////////////////////////////////////
// Output tree
///////////////////////////////////////////////////////////////////////////////////////



void Tools_Attractor_System::CompileAttractors()
{
	UpdateAttractorsFromType();
	SyncAttractorsWithControllerEntities();

	//determine the total number of compiled attractors and links
	u_int uNumFixedAttractors = 0;
	u_int uNumRelativeAttractors = 0;
	u_int uNumLinks = 0;

	for(Asura_Hashed_List_It<Tools_Attractor> xIt(&m_xAttractorList);!xIt.Done();xIt.Next())
	{
		Tools_Attractor* const pxAttractor = xIt.GetCurrent();
		u_int uNumCompiled = pxAttractor->CountCompiledAttractors();
		if(!uNumCompiled) continue;

		if(pxAttractor->IsParentRelative())
		{
			uNumRelativeAttractors += uNumCompiled;	
		}
		else
		{
			uNumFixedAttractors += uNumCompiled;
		}

		//determine the number of links required by this attractor
		for(u_int i=0;i<pxAttractor->GetNumLinks();i++)
		{
			u_int uLinkID = pxAttractor->GetLink(i);

			const Tools_Attractor* pxLinkedAttractor = m_xAttractorList.FindObject(uLinkID);
			if(pxLinkedAttractor)
			{
				uNumLinks += pxLinkedAttractor->CountCompiledAttractors();
			}
		}
	}

	//Allocate storage for the attractors and links
	Asura_Attractor_Compiled* pxCompiledAttractors = 0;
	Asura_Attractor_EntityRelative* pxRelativeAttractors = 0;
	u_int* puLinks = 0;

	Asura_ResourcePushType(ASURA_RESOURCE_ATTRACTORS);
	
	Asura_ResourceAllocateArray(pxCompiledAttractors,uNumFixedAttractors);
	Asura_ResourceAllocateArray(pxRelativeAttractors,uNumRelativeAttractors);
	Asura_ResourceAllocateArray(puLinks,uNumLinks);
    	
	Asura_ResourcePopType();

	//Go through the attractor list again and fill everything in
	u_int uFixedIndex = 0;
	u_int uRelativeIndex = 0;
	u_int uLinkIndex = 0;

	
	for(Asura_Hashed_List_It<Tools_Attractor> xIt(&m_xAttractorList);!xIt.Done();xIt.Next())
	{
		const Tools_Attractor* const pxAttractor = xIt.GetCurrent();
		if(!pxAttractor->HasAtLeastTwoNodes()) continue;

		//Find the links. The same link pointer will be used for all attractors made from this one
		u_int* puAttractorLinks = &puLinks[uLinkIndex];
		u_int uNumLinks = 0;

		for(u_int i=0;i<pxAttractor->GetNumLinks();i++)
		{
			u_int uLinkID = pxAttractor->GetLink(i);

			const Tools_Attractor* pxLinkedAttractor = m_xAttractorList.FindObject(uLinkID);
			if(pxLinkedAttractor)
			{
				if(!pxLinkedAttractor->HasAtLeastTwoNodes()) continue;
				
				//get the IDs of the nodes in the linked attractor
				const Tools_Attractor_Node* pxLinkedNode = pxLinkedAttractor->GetFirstNode();
				while(pxLinkedNode)
				{
					if(!pxLinkedNode->GetNext() && !pxLinkedAttractor->IsClosed()) break;

					puLinks[uLinkIndex++] = pxLinkedNode->GetID();
					uNumLinks++;
					pxLinkedNode = pxLinkedNode->GetNext();
				}
			}
		}
		if(!uNumLinks) puAttractorLinks = 0;

		//create the compiled attractors from this tools attractor
		
		
		for(const Tools_Attractor_Node* pxNode = pxAttractor->GetFirstNode(); pxNode; pxNode = pxNode->GetNext())
		{
			const Tools_Attractor_Node* pxNextNode = pxNode->GetNext();
			if(!pxNextNode)
			{
				if(pxAttractor->IsClosed())
				{
					//next node is the first node
					pxNextNode = pxAttractor->GetFirstNode();
				}
				else
				{
					break;
				}
			}

			Asura_Attractor_Compiled* pxCompiled = 0;
			Asura_Attractor_EntityRelative* pxRelative = 0;

			if(pxAttractor->IsParentRelative())
			{
				pxCompiled = pxRelative = &pxRelativeAttractors[uRelativeIndex++];
			}
			else
			{
				pxCompiled = &pxCompiledAttractors[uFixedIndex++];
			}
			

			//Take the attractor ID from the first node , rather than the tools attractor , so
			//that each attractor generated gets a different ID.
			//RAR 06/10/2006
			pxCompiled->SetID(pxNode->GetID());

			pxCompiled->SetType( pxAttractor->GetType() );
			pxCompiled->SetRadius( pxAttractor->GetRadius() );
			pxCompiled->SetFlags( pxAttractor->GetFlags());

			pxCompiled->GetNode( 0 )->SetPosition( pxNode->GetPosition() );
			pxCompiled->GetNode( 1 )->SetPosition( pxNextNode->GetPosition() );

			pxCompiled->SetLinks(uNumLinks,puAttractorLinks);

			Asura_Vector_3 xNormal;
			Asura_Vector_3 xDir(pxNextNode->GetPosition() - pxNode->GetPosition());
			xDir.Normalise();
			pxAttractor->GetNormalFromDirection(xNormal, xDir);

			if(pxRelative)
			{
				// Rotate normal into object-relative
				const Asura_ServerEntity_PhysicalObject* pxEntity = Asura_ServerEntity_PhysicalObject::FindEntity(pxAttractor->GetParentGuid());
				if (pxEntity)
				{
					xNormal = pxEntity->GetOrientation().ApplyTranspose(xNormal);
				}
				pxCompiled->SetNormal(xNormal);

				pxRelative->m_uParentGuid = pxAttractor->GetParentGuid();				
				pxRelative->m_axRelativePosition[0] = pxNode->GetRelativePosition();
				pxRelative->m_axRelativePosition[1] = pxNextNode->GetRelativePosition();
			}
			else
			{
				pxCompiled->SetNormal(xNormal);
			}
		}
	}

	Asura_Assert(uLinkIndex == uNumLinks,"Error compiling env hooks");
	Asura_Assert(uFixedIndex == uNumFixedAttractors,"Error compiling env hooks");
	Asura_Assert(uRelativeIndex == uNumRelativeAttractors,"Error compiling env hooks");

	// copy data to attractor system
	Asura_Attractor_Handler* const pxAttractorHandler = &Asura_Attractor_System::GetInstance()->GetCurrentHandler();
	pxAttractorHandler->Reset();
	pxAttractorHandler->m_iNumCompiledAttractors = uNumFixedAttractors;


	pxAttractorHandler->m_uNumLinks = uNumLinks;
	pxAttractorHandler->m_puLinks = puLinks;

	pxAttractorHandler->m_iNumRelativeAttractors = uNumRelativeAttractors;
	pxAttractorHandler->m_pxRelativeAttractors = pxRelativeAttractors;

	// now make tree

	if(uNumFixedAttractors)
	{
		Asura_LBAABB_Tree* const pxTree = &pxAttractorHandler->m_xAttractorTree;
		Asura_Bounding_Box* pxLeafBBs = new Asura_Bounding_Box[ uNumFixedAttractors ];
		Asura_Vector_3* pxLeafCentres = new Asura_Vector_3[ uNumFixedAttractors ];

		for ( u_int i = 0; i < uNumFixedAttractors; i++ )
		{
			const Asura_Vector_3 xPos0 = pxCompiledAttractors[ i ].GetNode( 0 )->GetPosition();
			const Asura_Vector_3 xPos1 = pxCompiledAttractors[ i ].GetNode( 1 )->GetPosition();
			const float fRadius = pxCompiledAttractors[ i ].GetRadius();

			pxLeafCentres[ i ] = ( xPos0 + xPos1 ) * 0.5f;
			pxLeafBBs[ i ].Reset();
			pxLeafBBs[ i ].Set( xPos0 );
			pxLeafBBs[ i ].Set( xPos1 );
			if(fRadius)
			{
				pxLeafBBs[ i ].Grow(fRadius);
			}
		}	

		u_int* puAttractorTreeMapping = Tools_LBAABB_Tree::CreateRunTimeTree( *pxTree, uNumFixedAttractors, pxLeafBBs, pxLeafCentres );

		//reorder the compiled attractors to match the tree
		Asura_ResourcePushType(ASURA_RESOURCE_ATTRACTORS);
		{
			//allocate reordered attractor array
			Asura_Attractor_Compiled* pxReorderedAttractorStore = 0;
			Asura_ResourceAllocateArray(pxReorderedAttractorStore,uNumFixedAttractors);

			//copy across
			for(u_int i=0;i<uNumFixedAttractors;i++)
			{
				const Asura_Attractor_Compiled* pxSource = &pxCompiledAttractors[puAttractorTreeMapping[i]];
				Asura_Attractor_Compiled* pxDest = &pxReorderedAttractorStore[i];
				*pxDest = *pxSource;
			}

			pxAttractorHandler->m_pxCompiledAttractors = pxReorderedAttractorStore;

			//delete the old array 
			Asura_ResourceDeallocateArray(pxCompiledAttractors);
			pxCompiledAttractors = 0;

		}
		Asura_ResourcePopType();
	
		delete [] puAttractorTreeMapping;
		delete [] pxLeafCentres;
		delete [] pxLeafBBs;

	}


	//add attractors to hashed list
	for(u_int i=0;i<uNumFixedAttractors;i++)
	{
		pxAttractorHandler->m_xAttractorList.AddToList(&pxAttractorHandler->m_pxCompiledAttractors[i]);
	}

	for(u_int i=0;i<uNumRelativeAttractors;i++)
	{
		pxAttractorHandler->m_xAttractorList.AddToList(&pxAttractorHandler->m_pxRelativeAttractors[i]);
	}

}


Tools_Attractor* Tools_Attractor_System::GetAttractorByID(u_int uID)
{
	return m_xAttractorList.FindObject(uID);
}

void Tools_Attractor_System::RecycleAttractorID(u_int uID)
{
	if(uID == 0) return;

	if(uID & TOOLS_ATTRACTORID_MISSIONFILE)
		m_xMissionIDHandler.RecycleID(uID);
	else
		m_xCommonIDHandler.RecycleID(uID);
}

u_int Tools_Attractor_System::GenerateID(bool bMissionFile)
{
	return bMissionFile ? m_xMissionIDHandler.GenerateID() : m_xCommonIDHandler.GenerateID();
}

u_int Tools_Attractor_System::GetNextID(bool bMissionFile) const
{
	return bMissionFile ? m_xMissionIDHandler.GetNextID() : m_xCommonIDHandler.GetNextID();
}

void Tools_Attractor_System::SetNextID(u_int uID)
{
	if(uID & TOOLS_ATTRACTORID_MISSIONFILE)
		m_xMissionIDHandler.SetNextID(uID);
	else
		m_xCommonIDHandler.SetNextID(uID);

}

void Tools_Attractor_System::SyncAttractorsWithControllerEntities()
{

	//search for any attractor controller entities that correspond to attractors that no longer exist
	for(Asura_Hashed_List_It<Asura_ServerEntity_AttractorController> xIt(Asura_ServerEntity_AttractorController::GetList()); !xIt.Done(); xIt.Next())
	{
		Asura_ServerEntity_AttractorController* pxEntity = xIt.GetCurrent();
	
		//any matching attractor?
		u_int uID = pxEntity->GetToolsAttractorID();
		Tools_Attractor* pxAttractor = GetAttractorByID(uID);

		if(pxAttractor)
		{
			if(pxAttractor->GetControllerGuid() != pxEntity->GetGuid())
			{
				//Attractor exists ,but isn't assigned to the same controller.
				//Reassign if the attractor isn't supposed to be connected to a different entity
				if(pxAttractor->GetControllerGuid() == ASURA_GUID_UNREGISTERED)
				{
					pxAttractor->SetControllerGuid(pxEntity->GetGuid());
					Tools_SaveAsura::SetAttractorsModified();
				}
				else
				{
					Tools_SaveAsura::SetEntityModified(pxEntity);
					pxEntity->SetAsDestroyed();
				}
			}
		}
		else
		{
			Tools_SaveAsura::SetEntityModified(pxEntity);
			pxEntity->SetAsDestroyed();
		}
	}

	//search the attractors for any that refer to controller entities
	for(Asura_Hashed_List_It<Tools_Attractor> xIt(&m_xAttractorList); !xIt.Done(); xIt.Next())
	{
		Tools_Attractor* pxAttractor = xIt.GetCurrent();
		Asura_Guid uGuid = pxAttractor->GetControllerGuid();
		if(uGuid != ASURA_GUID_UNREGISTERED)
		{
			Asura_ServerEntity_AttractorController* pxEntity = Asura_ServerEntity_AttractorController::FindEntity(uGuid);

			if(pxEntity)
			{
				//update the list of compiled attractor IDs in the entity
				pxEntity->m_uNumCompiledAttractorIDs = pxAttractor->CountCompiledAttractors();

				delete [] pxEntity->m_puCompiledAttractorIDs;
				pxEntity->m_puCompiledAttractorIDs = pxEntity->m_uNumCompiledAttractorIDs ? new u_int[pxEntity->m_uNumCompiledAttractorIDs] : 0;

				const Tools_Attractor_Node* pxNode = pxAttractor->GetFirstNode();
				for(u_int i=0;i<pxEntity->m_uNumCompiledAttractorIDs;i++)
				{
					Asura_Assert(pxNode,"Error updating list of compiled attractor IDs");
					pxEntity->m_puCompiledAttractorIDs[i] = pxNode->GetID();

					pxNode = pxNode->GetNext();
				}
			}
			else
			{
				pxAttractor->SetControllerGuid(ASURA_GUID_UNREGISTERED);
				pxAttractor->SetModified();
			}
		}
	}
}


///////////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_Attractor_IDHandler **
**---------------------------*/

void Tools_Attractor_IDHandler::Initialise(u_int uFirstID)
{
	m_uNextID = uFirstID;
}

u_int Tools_Attractor_IDHandler::GenerateID()
{
	if(m_xSpareIDs.size())
	{
		u_int uID = m_xSpareIDs.first_entry();
		m_xSpareIDs.delete_first_entry();

		return uID;
	}
	else
	{
		return m_uNextID++;
	}
}

void Tools_Attractor_IDHandler::RecycleID(u_int uID)
{
	if(uID == 0) return;
	if(!m_xSpareIDs.contains(uID))
	{
		m_xSpareIDs.add_entry(uID);
	}
	else
	{
		Asura_Assert(0,"ID already in list");
	}
}



#endif // ASURA_USE_ATTRACTORS
