//
// Tools_Attractor_System.h
//
// Created by JGP 10/05/06
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_ATTRACTOR_SYSTEM_H__
#define __TOOLS_ATTRACTOR_SYSTEM_H__


#include "Core/Asura_LinkList.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_HashedList.h"
#include "Misc/list_tem.hpp"
#include "Core/Asura_String.h"


typedef Asura_ListElt<class Tools_Attractor_Node*> Tools_Attractor_Node_List;
class Tools_Attractor_Type;


/*
	Tools_Attractor_System

	Storage class for tools 
	attractors and attractor nodes.
*/

//Value ORed into ID for attractors that go to the mission file
#define TOOLS_ATTRACTORID_MISSIONFILE 0x80000000

//deals with doleing out new attractor IDs
class Tools_Attractor_IDHandler
{
public :
	void Initialise(u_int uFirstID);
	u_int GenerateID();
	void RecycleID(u_int uID);

	u_int GetNextID() const {return m_uNextID;}
	void SetNextID(u_int uID) {m_uNextID = uID;}


private :
	u_int m_uNextID;
	List<u_int> m_xSpareIDs;
};

class Tools_Attractor_System
{

public:

	static Tools_Attractor_System* Initialise();
	static void Shutdown();
	static void Reset();

	static Tools_Attractor_System* GetInstance() { Asura_Assert( s_pxInstance != NULL, "Tools_Attractor_System not created" ); return s_pxInstance; }

	// construct / destruct

	Tools_Attractor_System();
	~Tools_Attractor_System();

	// register types
	void RegisterType( const char* const szTypeName,u_int uColour = 0xffffffff );
	void RegisterType( const Tools_Attractor_Type& xType );
    void UnregisterAllTypes();

	int GetNumAttractorTypes() const {return m_xTypeTree.GetNumberOfNodes();}
	const Tools_Attractor_Type* GetAttractorTypeByIndex(u_int uIndex) {return m_xTypeTree.GetDataByIndex(uIndex);}
	const Tools_Attractor_Type* GetAttractorTypeByHash(Asura_Hash_ID uHash) {return m_xTypeTree.GetDataByHash(uHash);}

	//updates non-editable properties from the attractor type
	void UpdateAttractorsFromType();

	// lists

	Tools_Attractor_Node_List* GetNodeList();
	const Asura_Hashed_List<class Tools_Attractor>* GetAttractorList() const {return &m_xAttractorList;}

	// add / remove

	void AddAttractor( class Tools_Attractor* const pxAttractor );
	void AddAttractorNode( class Tools_Attractor_Node* const pxNode );
	void DeleteAttractorNode( class Tools_Attractor_Node* const pxNode );
	void DeleteAttractor( class Tools_Attractor* pxAttractor );

	//Used by template system to delete attractors , including nodes and tools objects
	void DeleteAttractorByID(u_int uID,bool bRecycleIDs);

	Tools_Attractor* GetAttractorByID(u_int uID);

	//Used by the template system to indicate which IDs can safely be reused , since they couldn't have been referred to externally
	void RecycleAttractorID(u_int uID);


	// output

	void CompileAttractors();

	u_int GenerateID(bool bMissionFile);

	u_int GetNextID(bool bMissionFile) const;
	void SetNextID(u_int uID);

	//Updates attractor controller entities from the attractors
	void SyncAttractorsWithControllerEntities();

private:

	// functions

	void ClearLists();

	// static singleton

	static Tools_Attractor_System* s_pxInstance;

	// vars

	Asura_LinkList<class Tools_Attractor_Node*> m_xNodeList;
	Asura_Hashed_List<class Tools_Attractor> m_xAttractorList;


	Asura_TemplateHashTree<Tools_Attractor_Type> m_xTypeTree;

	Tools_Attractor_IDHandler m_xCommonIDHandler;
	Tools_Attractor_IDHandler m_xMissionIDHandler;


};

class Tools_Attractor_Type
{
public :
	Tools_Attractor_Type(){}

	Tools_Attractor_Type(const char* szName, u_int uColour, float fDefaultRadius = 1.0f, bool bMustBeDefaultRadius = false,
						 bool bMustBeHorizontal = false, bool bMustBeVertical = false)
	:m_xName(szName)
	,m_uColour(uColour)
	,m_fDefaultRadius(fDefaultRadius)
	,m_bMustBeDefaultRadius(bMustBeDefaultRadius)
	,m_bMustBeHorizontal(bMustBeHorizontal)
	,m_bMustBeVertical(bMustBeVertical)
	{}


	Asura_String m_xName;
	u_int m_uColour;//colour for the tools object

	float m_fDefaultRadius; //default radius for new attractors of this type
	bool m_bMustBeDefaultRadius; //All attractors of this type must use the default radius

	bool m_bMustBeHorizontal;
	bool m_bMustBeVertical; 

};


#endif // __TOOLS_ATTRACTOR_SYSTEM_H__
