//////////////////////////////////////////////////////////////////////////////
//
// Tools_Mode_Blueprint.cpp
//
// Created by NJD - 07 September 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Blueprint/Tools_Mode_Blueprint.h"

// asura
#include "Animation/Asura_AnimTable_System.h"
#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_Chunk_Blueprints.h"
#include "Core/Asura_LocalisedText.h"
#include "Core/Asura_Maths.h" 
#include "Core/Asura_NameList.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Table_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Tree_Dialog.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#ifdef ASURA_USE_DECAL_MANAGER
#include "SpecialFX/Asura_Decal_Manager.h"
#endif

#include "shellapi.h"	// For function ShellExecute()

// tools
#include "Tools_Overlays.h"
#include "Tools_System.h"
#include "Blueprint/Tools_Blueprint_Edit_Object.h"
#include "Blueprint/Tools_Blueprint_Export.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Load&Save/Tools_LoadedFileList.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_ResourceExportLog.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_ProgressIndicator.h"
#include "Misc/Tools_Timed_Messages.h"
#include "PC/FileSystem/Tools_FileSystem.h"

#ifdef _DEBUG
#define OUTPUT_BLUEPRINT_DEBUG_INFO 1
#endif

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Mode_Blueprint
//
//////////////////////////////////////////////////////////////////////////////

#define szBLUEPRINTS_DIRECTORY "Blueprints\\"
#define	szBLUEPRINTS_EXTENSION ".bpt"

const char*           Tools_Mode_Blueprint::s_aszTabNames[ NUM_TAB_NAMES ] =
{
    "Editing",
    " Tools "
};

Asura_Vector_2        Tools_Mode_Blueprint::s_xOverlayPosition        = Asura_Vector_2( 3.0f, 3.0f );
Asura_Vector_2        Tools_Mode_Blueprint::s_xTreeDialogueDimensions = Asura_Vector_2( 30.0f, 30.0f );
bool                  Tools_Mode_Blueprint::s_bOverlayWasOnTop        = false;
Asura_TreeDialog_Tree Tools_Mode_Blueprint::s_xTree;
bool                  Tools_Mode_Blueprint::s_bFirstUpdate                   = true;
bool                  Tools_Mode_Blueprint::s_bRefreshMainOverlay            = false;
bool                  Tools_Mode_Blueprint::s_bRefreshDisplayOverlay         = false;
bool                  Tools_Mode_Blueprint::s_bRefreshEditObjectOverlays     = false;
bool				  Tools_Mode_Blueprint::s_bRefreshDisplayAndEditOverlays = false;

Asura_Vector_2 Tools_Mode_Blueprint::s_xDisplayOverlayPosition = Asura_Vector_2( 8.0f, 8.0f );
bool           Tools_Mode_Blueprint::s_bDisplayOverlayIsOpen   = false;

int                                       Tools_Mode_Blueprint::s_iNumTypes                     = 0;
Tools_Mode_Blueprint::ModifiedTypeHashID* Tools_Mode_Blueprint::s_pxPerTypeData                 = NULL;
Tools_Mode_Blueprint*                     Tools_Mode_Blueprint::s_pxThisBlueprintMode           = NULL;
bool                                      Tools_Mode_Blueprint::s_bOverlayIsOpen                = false;
bool                                      Tools_Mode_Blueprint::s_bOverlayWasOpenOnModeShutdown = false;
Asura_LinkList< Tools_BEO_Display_Data >  Tools_Mode_Blueprint::s_xEditObjectDisplayData;

bool                                      Tools_Mode_Blueprint::s_bAllowRecursiveExportsUsingParameters = false;

bool                                      Tools_Mode_Blueprint::s_bShowFullStrings   = false;
bool					                  Tools_Mode_Blueprint::s_bShowEditButtons   = true;
bool                                      Tools_Mode_Blueprint::s_bShowFilterButtons = true;
int                                       Tools_Mode_Blueprint::s_iFilterChoice    = 0;
Asura_String                              Tools_Mode_Blueprint::s_xFilterGroupName;
bool                                      Tools_Mode_Blueprint::s_abColumnVisible[ Tools_Blueprint_Edit_Object::CV_NUM_COLUMN_VALUES ] =
{
    true, false, true, true, true, true, true
};
int					                      Tools_Mode_Blueprint::s_iCurrentNumColumns = 0;
const char*                               Tools_Mode_Blueprint::s_aszCurrentColumnHeadingStrings[ Tools_Blueprint_Edit_Object::CV_NUM_COLUMN_VALUES ] =
{
    "", "", "", "", "", "", ""
};
Asura_Vector_2							  Tools_Mode_Blueprint::s_xMaxTableSize( 100.0f, 21.0f );

const char* Tools_Mode_Blueprint::s_pszLogCategory = "Blueprint";
const char* Tools_Mode_Blueprint::s_pszLogSubCategory[ END_LOG_SUBCATEGORIES ] =
{
    "Debug",
	"Search",
    "Verification",
    "Errors"
};

Asura_String							  Tools_Mode_Blueprint::s_xLastSkinAnimPath    = NULL;
Asura_String							  Tools_Mode_Blueprint::s_xLastTexturePath	   = NULL;
Asura_String							  Tools_Mode_Blueprint::s_xLastTypeChoice	   = NULL;
Asura_Hash_ID							  Tools_Mode_Blueprint::s_uLastBPTypeChoice	   = ASURA_HASH_ID_UNSET;
Asura_Hash_ID							  Tools_Mode_Blueprint::s_uLastBPChoice		   = ASURA_HASH_ID_UNSET;

const Asura_Hash_ID*					  Tools_Mode_Blueprint::s_puTemplateFilterList = NULL;
Tools_Blueprint_PasteBuffer               Tools_Mode_Blueprint::s_xPasteBuffer;

Tools_Mode_Blueprint::StaticStorageReleaser Tools_Mode_Blueprint::s_xStaticStorageReleaser;

void ( *Tools_Mode_Blueprint::s_pfnBlueprintChanged )( void ) = NULL;

void Tools_Mode_Blueprint::LoadBlueprints()
{
	Tools_FileSystem::LoadAsuraFiles( szBLUEPRINTS_DIRECTORY "*" szBLUEPRINTS_EXTENSION, TOOLS_FILEFIND_RECURSIVE | TOOLS_FILEFIND_IGNOREBACKUPDIR );
}

void Tools_Mode_Blueprint::Initialise()
{
    s_xLastSkinAnimPath = "";

    if ( !s_pxThisBlueprintMode )
    {
        s_pxThisBlueprintMode = Project_CreateBlueprintMode();
        Asura_Assert( s_pxThisBlueprintMode, "Tools_Mode_Blueprint::Initialise - could not create blueprint mode" );
        if ( s_pxThisBlueprintMode )
        {
            s_pxThisBlueprintMode->InitialiseMode();
        }
    }
}

void Tools_Mode_Blueprint::Update()
{
    if ( s_pxThisBlueprintMode )
    {
        s_pxThisBlueprintMode->UpdateMode();
    }
}

void Tools_Mode_Blueprint::Render()
{
    if ( s_pxThisBlueprintMode )
    {
        s_pxThisBlueprintMode->RenderUI();
    }
}

void Tools_Mode_Blueprint::Shutdown()
{
    if ( s_pxThisBlueprintMode )
    {
        s_pxThisBlueprintMode->ShutdownMode();
        delete s_pxThisBlueprintMode;
        s_pxThisBlueprintMode = NULL;
    }
}

void Tools_Mode_Blueprint::ToggleOverlay()
{
    if ( !s_pxThisBlueprintMode ) { return; }

    if ( OverlayIsOpen() )
    {
		RememberEditObjects();
		DestroyEditObjects();

        s_pxThisBlueprintMode->DestroyOverlay();
        if ( s_pxThisBlueprintMode->s_bDisplayOverlayIsOpen )
        {
            s_pxThisBlueprintMode->DestroyDisplayOptionsOverlay();
        }
    }
    else
    {
        s_pxThisBlueprintMode->CreateOverlay();

		s_pxThisBlueprintMode->RecreateStoredEditObjects();
    }
}

void Tools_Mode_Blueprint::RefreshOverlay()
{
	if ( OverlayIsOpen() )
	{
		s_pxThisBlueprintMode->DestroyOverlay();
		s_pxThisBlueprintMode->CreateOverlay();
	}
}

void Tools_Mode_Blueprint::SetSelectedBlueprint( const Asura_Blueprint* pxBlueprint )
{
	Asura_TreeDialog_Tree* pxBPTree = s_pxThisBlueprintMode ? s_pxThisBlueprintMode->GetTree() : NULL;
	if( pxBPTree )
	{
		Asura_TreeDialog_Item* pxBPItem = pxBPTree->FindByName( pxBlueprint->GetName().GetString() );
		if( pxBPItem )
		{
			pxBPTree->SetSelectedItem( pxBPItem );
		}
	}
}

bool Tools_Mode_Blueprint::CreateEditOverlay( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID, const Asura_Hash_ID uParamHashID /*= ASURA_HASH_ID_UNSET*/ )
{
	// Check that this overlay isn't already open.
	for ( Asura_List_It< Tools_Blueprint_Edit_Object > xBEOIter( Tools_Blueprint_Edit_Object::GetList() ); !xBEOIter.Done(); xBEOIter.Next() )
	{
		Tools_Blueprint_Edit_Object* pxBEO = xBEOIter.GetCurrent();
		if ( pxBEO && pxBEO->GetBlueprintTypeHashID() == uBlueprintTypeHashID && pxBEO->GetBlueprintHashID() == uBlueprintHashID )
		{
			// Bring this overlay to the top (by destroying and recreating it).
			Tools_BEO_Display_Data xDisplayData;
			pxBEO->RecordOverlayState( xDisplayData );
			pxBEO->DestroyOverlay();
			pxBEO->RestoreOverlayState( xDisplayData );
			if ( uParamHashID != ASURA_HASH_ID_UNSET )
			{
				pxBEO->SelectParameterRow( uParamHashID );
			}
			return true;
		}
	}

	// Doesn't exist, so create it.
    if ( s_pxThisBlueprintMode )
    {
        if ( Asura_Blueprint_System::GetBlueprintByHashID( uBlueprintTypeHashID, uBlueprintHashID ) )
        {
            Tools_Blueprint_Edit_Object* pxBEO = s_pxThisBlueprintMode->CreateBlueprintEditObject();
            pxBEO->SetBlueprintTypeHashID( uBlueprintTypeHashID );
            pxBEO->SetBlueprintHashID( uBlueprintHashID );
            pxBEO->CreateOverlay();
			if ( uParamHashID != ASURA_HASH_ID_UNSET )
			{
				pxBEO->SelectParameterRow( uParamHashID );
			}
            return true;
        }
    }

    return false;
}

Tools_Blueprint_Edit_Object* Tools_Mode_Blueprint::GetEditObject( const Asura_Hash_ID uBlueprintTypeHashID, const Asura_Hash_ID uBlueprintHashID )
{
	// Gets the edit object for a particular blueprint, assuming it has already been opened
	for ( Asura_List_It< Tools_Blueprint_Edit_Object > xBEOIter( Tools_Blueprint_Edit_Object::GetList() ); !xBEOIter.Done(); xBEOIter.Next() )
	{
		Tools_Blueprint_Edit_Object* pxBEO = xBEOIter.GetCurrent();
		if ( pxBEO && pxBEO->GetBlueprintTypeHashID() == uBlueprintTypeHashID && pxBEO->GetBlueprintHashID() == uBlueprintHashID )
		{
			return pxBEO;
		}
	}

	return NULL;
}


void Tools_Mode_Blueprint::SetBlueprintChangedCallback( void( *pfnBlueprintChanged )( void ) )
{
	Asura_Assert( ( s_pfnBlueprintChanged == NULL ) || ( s_pfnBlueprintChanged == pfnBlueprintChanged ), "Overriding the current BlueprintChanged callback" );
	s_pfnBlueprintChanged = pfnBlueprintChanged;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

void Tools_Mode_Blueprint::InitialiseMode()
{
    TransferModeData( TOOLSMODEDATAFILE_LOAD );

    // Re-establish the filter group choice, in case our filter group name no longer exists in the blueprint data.
    SetFilterGroupChoiceFromFilterName();

	CreatePerTypeData();

    // If the overlay was previously open then recreate it again, along with any blueprint edit overlays that were around.
    if ( s_bOverlayWasOpenOnModeShutdown )
    {
        CreateOverlay();
		RecreateStoredEditObjects();
    }
}

void Tools_Mode_Blueprint::ShutdownMode()
{
    s_bOverlayWasOpenOnModeShutdown = OverlayIsOpen();
    DestroyOverlay();
    DestroyDisplayOptionsOverlay();
    if ( Tools_System::ShouldQuitToDesktop() )
    {
        DestroyBlueprintTree();
    }
    else
    {
		RememberEditObjects();
    }
	DestroyEditObjects();
	Tools_Blueprint_Edit_Object::DestroyBufferStorage();

	// These were released by the static cleanup stuff before, but the order is not guaranteed, and it was happening after the
	// value mempool (which will shortly be going) was released, and so crashing.
	ReleasePasteBuffer();

    TransferModeData( TOOLSMODEDATAFILE_SAVE );
}

void Tools_Mode_Blueprint::UpdateMode()
{
    const bool bOverlayIsOnTop = ( Asura_Overlay_System::GetTopOverlayID() == reinterpret_cast< u_int >( this ) );
    if ( bOverlayIsOnTop != s_bOverlayWasOnTop || s_bFirstUpdate )
    {
        SetTreeTypeNodeColours();
    }
    s_bOverlayWasOnTop = bOverlayIsOnTop;

    if ( s_bRefreshMainOverlay )
    {
        s_bRefreshMainOverlay = false;
        DestroyOverlay();
        CreateOverlay();
    }

    if ( s_bRefreshDisplayOverlay )
    {
        s_bRefreshDisplayOverlay = false;
        if ( s_bDisplayOverlayIsOpen )
        {
            DestroyDisplayOptionsOverlay();
            CreateDisplayOptionsOverlay();
        }
    }

    if ( s_bRefreshEditObjectOverlays )
    {
        s_bRefreshEditObjectOverlays = false;
        RefreshBlueprintEditObjectOverlays();
    }

	if ( s_bRefreshDisplayAndEditOverlays )
	{
		s_bRefreshDisplayAndEditOverlays = false;
		if ( s_bDisplayOverlayIsOpen )
		{
			DestroyDisplayOptionsOverlay();
			CreateDisplayOptionsOverlay();
		}
		RefreshBlueprintEditObjectOverlays();
	}

    // Update all the blueprint edit objects.
    for ( Asura_List_It< Tools_Blueprint_Edit_Object > xIt( Tools_Blueprint_Edit_Object::GetList() ); !xIt.Done(); xIt.Next() )
    {
        xIt.GetCurrent()->Update();
    }

    s_bFirstUpdate = false;
}

void Tools_Mode_Blueprint::RenderUI()
{
    // Render all the blueprint edit objects.
    for ( Asura_List_It< Tools_Blueprint_Edit_Object > xIt( Tools_Blueprint_Edit_Object::GetList() ); !xIt.Done(); xIt.Next() )
    {
        xIt.GetCurrent()->Render();
    }
}

void Tools_Mode_Blueprint::SetTypeAsModified( const Asura_Hash_ID uTypeHashID )
{
    if ( !s_pxPerTypeData ) { return; }

    for ( int i = 0; i < s_iNumTypes; ++i )
    {
        if ( s_pxPerTypeData[ i ].m_uTypeHashID == uTypeHashID )
        {
            s_pxPerTypeData[ i ].m_bModified = true;
            Tools_SaveAsura::SetBlueprintsModified();
        }
    }

    if ( s_pxThisBlueprintMode )
    {
        s_pxThisBlueprintMode->SetTreeTypeNodeColours();
    }
}

void Tools_Mode_Blueprint::SaveAndCloseOverlay()
{
    if ( s_pxThisBlueprintMode )
    {
        SaveAll( true );
		ToggleOverlay();
    }
}

void Tools_Mode_Blueprint::SaveAll( bool bOutputAllNotifications /*= true*/ )
{
    // If the mode is null, then we've never used it so nothing can have changed.
    if ( !s_pxThisBlueprintMode ) { return; }

    // Save all blueprint types that have been modified.
    int iNumModified = 0;
    for ( int i = 0; i < s_iNumTypes; ++i )
    {
        if ( !s_pxPerTypeData[ i ].m_bModified )
        {
            continue;
        }
        ++iNumModified;

        const Asura_Hash_ID uTypeHashID = s_pxPerTypeData[ i ].m_uTypeHashID;
		const Asura_String* pxTypeName  = Asura_Blueprint_System::GetTypeName( uTypeHashID );
		if ( !pxTypeName )
		{
			continue;
		}

		const Asura_String* pxTypeFileName = Asura_Blueprint_System::GetTypeFileName( uTypeHashID );
		Asura_String xTypeFileName( pxTypeFileName ? *pxTypeFileName : "" );
		if ( !xTypeFileName.IsBlank() )
		{
			if ( Tools_SaveAsura::CanSaveFile( pxTypeFileName->GetString(), true ) )
			{
				if ( Asura_Chunk_Blueprints::SaveBlueprints( pxTypeFileName->GetString(), uTypeHashID ) )
				{
					Tools_LoadedFileList::UpdateModifiedTime( pxTypeFileName->GetString() );
					s_pxPerTypeData[ i ].m_bModified = false;
					if ( bOutputAllNotifications )
					{
						Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_INFO,
							"Saved %s to file %s.", pxTypeName->GetString(), pxTypeFileName->GetString() );
					}
				}
				else
				{
					Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR,
						"Failed to save %s to file %s - is there a directory missing?", pxTypeName->GetString(), pxTypeFileName->GetString() );
				}
			}
		}
		else
		{
			Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR,
				"Could not save %s - please set a filename for this type.", pxTypeName->GetString() );
		}
    }

    if ( iNumModified == 0 && bOutputAllNotifications )
    {
        Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_INFO, "No blueprints have been modified - nothing to save." );
    }

    bool bAnyStillModified = false;
    for ( int i = 0; i < s_iNumTypes && !bAnyStillModified; ++i )
    {
        if ( s_pxPerTypeData[ i ].m_bModified )
        {
            bAnyStillModified = true;
        }
    }
    if ( !bAnyStillModified )
    {
        Tools_SaveAsura::SetBlueprintsModified( false );
    }
    s_pxThisBlueprintMode->SetTreeTypeNodeColours();
}

void Tools_Mode_Blueprint::ReleaseStaticStorage()
{
    delete [] s_pxPerTypeData;
    s_pxPerTypeData = NULL;
    s_iNumTypes     = 0;
}

void Tools_Mode_Blueprint::CloseDisplayOverlay()
{
    if ( s_pxThisBlueprintMode && s_bDisplayOverlayIsOpen )
    {
        s_pxThisBlueprintMode->DestroyDisplayOptionsOverlay();
    }
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

void Tools_Mode_Blueprint::CreateOverlay()
{
    Asura_Overlay* pxNewOverlay = new Asura_Overlay( "Blueprints", false, &s_xOverlayPosition );
    if ( !pxNewOverlay ) { return; }

    Asura_Vector_2 xOffset = Asura_Vector_2( 1.0f, 3.0f );
    pxNewOverlay->m_uID = reinterpret_cast< u_int >( this );

    CreateBlueprintTreeAndPerTypeData();
    SetTreeTypeNodeColours();
    AddOverlayItems( *pxNewOverlay, xOffset );
    pxNewOverlay->SetContextMenuCallback( ContextMenuCallback, NULL );

    Asura_Overlay_System::SubmitNewOverlay( pxNewOverlay, false );
    s_bOverlayIsOpen = true;
}

void Tools_Mode_Blueprint::AddOverlayItems( Asura_Overlay& xNewOverlay, Asura_Vector_2& xOffset )
{
    Asura_Dialog* pxNewDialogue = NULL;

    Asura_Tree_Dialog* pxNewTreeDialogue = new Asura_Tree_Dialog( xOffset, &s_xTree, s_xTreeDialogueDimensions.x, s_xTreeDialogueDimensions.y );
    pxNewTreeDialogue->SetDoubleClickedFunction( TreeDoubleClicked );
	pxNewTreeDialogue->SetRenameCallback( Asura_Callback_Void_2Arg<void, Asura_TreeDialog_Item*,const char*> (RenameTreeItem));
	pxNewTreeDialogue->SetDragDropCallback( Asura_Callback_Void_1Arg<void, Asura_TreeDialog_Item*> (DragAndDrop));
	s_xTree.EnableMultipleSelection(true);
    xNewOverlay.AddDialog( pxNewTreeDialogue );
    xOffset.y += s_xTreeDialogueDimensions.y + 1.0f;

	//add control for resizing the tree
	pxNewDialogue = new Asura_Resize_Dialog(pxNewTreeDialogue, &s_xTreeDialogueDimensions);
	xNewOverlay.AddDialog( pxNewDialogue);

    Asura_Tabs_Dialog* pxTabsDialogue = new Asura_Tabs_Dialog( xOffset, s_aszTabNames, sizeof( s_aszTabNames ) / sizeof( s_aszTabNames[ 0 ] ) );
    xNewOverlay.AddDialog( pxTabsDialogue );

    pxTabsDialogue->SelectTab( TAB_EDITING );
    xOffset.y = pxTabsDialogue->m_xMaxCoord.y + 0.5f;
    float fTabsEndY = xOffset.y;

	pxNewDialogue = new Asura_Function_Dialog( "Create New Blueprint", xOffset, CreateNewBlueprint, NULL, TreeSelectionValid );
	pxNewDialogue->SetHelpText( "Creates a new blueprint as a child of the currently selected type or blueprint." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.x += 20.0f;

	pxNewDialogue = new Asura_Function_Dialog( "Create New Blueprint Type", xOffset, CreateNewType );
    pxNewDialogue->SetHelpText( "Creates a new blueprint type." );
    pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.x -= 20.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxNewDialogue = new Asura_Dialog( "Selected Type:", xOffset );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialogue = new Asura_Function_Dialog( "Rename", xOffset, RenameSelectedType, NULL, TreeSelectionIsType );
	pxNewDialogue->SetHelpText( "Renames the currently selected blueprint type." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.x += 10.0f;

	pxNewDialogue = new Asura_Function_Dialog( "Set File Name", xOffset, SetSelectedTypeFileName, NULL, TreeSelectionIsType );
	pxNewDialogue->SetHelpText( "Sets the file name associated with the currently selected blueprint type." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.x -= 10.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxNewDialogue = new Asura_Dialog( "Selected Blueprint:", xOffset );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialogue = new Asura_Function_Dialog( "Edit", xOffset, EditSelectedBlueprint, NULL, TreeSelectionIsBlueprint );
    pxNewDialogue->SetHelpText( "Edits the currently selected blueprint." );
    pxTabsDialogue->AddDialog( pxNewDialogue );
    xOffset.x += 10.0f;

	pxNewDialogue = new Asura_Function_Dialog( "Rename", xOffset, RenameSelectedBlueprint, NULL, TreeSelectionIsBlueprint );
	pxNewDialogue->SetHelpText( "Renames the currently selected blueprint." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.x += 10.0f;

	pxNewDialogue = new Asura_Function_Dialog( "Delete", xOffset, DeleteSelected, 0, NULL, TreeSelectionIsBlueprint );
	pxNewDialogue->SetHelpText( "Deletes the currently selected blueprint and all blueprints below." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.x -= 20.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialogue = new Asura_Function_Dialog( "Cut", xOffset, CutSelectedBlueprint, NULL, TreeSelectionIsBlueprint );
	pxNewDialogue->SetHelpText( "Copies the selected blueprint to a temporary buffer." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.x += 10.0f;

	pxNewDialogue = new Asura_Function_Dialog( "Copy", xOffset, CopySelected, NULL, TreeSelectionIsBlueprint );
	pxNewDialogue->SetHelpText( "Copies the selected blueprint to a temporary buffer." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.x += 10.0f;

	pxNewDialogue = new Asura_Function_Dialog( "Paste", xOffset, PasteBlueprintOrType, NULL, CanPasteBlueprint );
	pxNewDialogue->SetHelpText( "Pastes previously copied blueprint(s) below the currently selected blueprint or type." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.x -= 20.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();
    fTabsEndY = Asura_Maths::Max( fTabsEndY, xOffset.y );

    pxTabsDialogue->SelectTab( TAB_TOOLS );
    xOffset.y = pxTabsDialogue->m_xMaxCoord.y + 0.5f;

	pxNewDialogue = new Asura_Function_Dialog( "Search Blueprints", xOffset, SearchBlueprints );
	pxNewDialogue->SetHelpText( "Finds a blueprint from a given hash ID." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialogue = new Asura_Function_Dialog( "Verify Blueprints", xOffset, VerifyBlueprintInformation );
    pxNewDialogue->SetHelpText( "Verifies all of the blueprint information to ensure it is correct." );
    pxTabsDialogue->AddDialog( pxNewDialogue );
    xOffset.y += Asura_Dialog::LargeSpacing();

	pxNewDialogue = new Asura_Function_Dialog( "Display Options", xOffset, SetDisplayOptions );
	pxNewDialogue->SetHelpText( "Set the default look for the blueprint overlays." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxNewDialogue = new Asura_Function_Dialog( "Export All Blueprints to XML (raw data)", xOffset, ExportAllBlueprintsRaw );
    pxNewDialogue->SetHelpText( "Fast, doesn't translate HashIDs to anim/skin/texture names" );
    pxTabsDialogue->AddDialog( pxNewDialogue );
    xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialogue = new Asura_Function_Dialog( "Export All Blueprints to XML (pretty version)", xOffset, ExportAllBlueprints );
	pxNewDialogue->SetHelpText( "Slow, translates HashIDs into anim/skin/texture names" );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialogue = new Asura_Function_Dialog( "Export Blueprint Names to Text", xOffset, ExportBlueprintNames );
	pxNewDialogue->SetHelpText( "Will export all names of all blueprints to text file." );
	pxTabsDialogue->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::LargeSpacing();

    fTabsEndY = Asura_Maths::Max( fTabsEndY, xOffset.y );

    pxTabsDialogue->SelectTab( TAB_EDITING );

    xOffset.y = fTabsEndY;
    pxNewDialogue = new Asura_Function_Dialog( "Save & Close", xOffset, SaveAndCloseOverlay );
    xNewOverlay.AddDialog( pxNewDialogue );
    xOffset.y += Asura_Dialog::DefaultSpacing();

	xNewOverlay.MakeDialogsRelativeToResizableDialog();
}

void Tools_Mode_Blueprint::DestroyOverlay()
{
    Asura_Overlay_System::CloseOverlays( reinterpret_cast< u_int >( this ) );
    s_bOverlayIsOpen = false;
}

void Tools_Mode_Blueprint::CreateDisplayOptionsOverlay()
{
    // Create and display an overlay with our display options.
    Asura_Overlay* pxNewOverlay = new Asura_Overlay( "Blueprint Display Options", false, &s_xDisplayOverlayPosition );
    if ( !pxNewOverlay ) { return; }

    Asura_Vector_2 xOffset = Asura_Vector_2( 1.0f, 3.0f );
    pxNewOverlay->m_uID = reinterpret_cast< u_int >( this + 1 );

    Asura_Dialog* pxNewDialogue = new Asura_Dialog( "-- Main Overlay ------------------", xOffset );
    pxNewOverlay->AddDialog( pxNewDialogue );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialogue = new Asura_Dialog( "Tree Dimensions:", xOffset );
    pxNewOverlay->AddDialog( pxNewDialogue );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialogue = new Asura_Numerical_Dialog( "height", xOffset, &s_xTreeDialogueDimensions.y, 10.0f, 300.0f, 0.5f, "%0.1f", &s_bRefreshMainOverlay );
    pxNewOverlay->AddDialog( pxNewDialogue );
    xOffset.x += 15.0f;

    pxNewDialogue = new Asura_Numerical_Dialog( "width", xOffset, &s_xTreeDialogueDimensions.x, 10.0f, 300.0f, 0.5f, "%0.1f", &s_bRefreshMainOverlay );
    pxNewOverlay->AddDialog( pxNewDialogue );
    xOffset.x -= 15.0f;
    xOffset.y += Asura_Dialog::LargeSpacing();


    pxNewDialogue = new Asura_Dialog( "-- Default Blueprint Edit Overlay -------------", xOffset );
    pxNewOverlay->AddDialog( pxNewDialogue );
    xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialogue = new Asura_Dialog( "Max Table Dimensions:", xOffset );
	pxNewOverlay->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialogue = new Asura_Numerical_Dialog( "height", xOffset, &s_xMaxTableSize.y, 10.0f, 300.0f, 0.5f, "%0.1f", &s_bRefreshDisplayAndEditOverlays );
	pxNewOverlay->AddDialog( pxNewDialogue );
	xOffset.x += 15.0f;

	pxNewDialogue = new Asura_Numerical_Dialog( "width", xOffset, &s_xMaxTableSize.x, 10.0f, 300.0f, 0.5f, "%0.1f", &s_bRefreshDisplayAndEditOverlays );
	pxNewOverlay->AddDialog( pxNewDialogue );
	xOffset.x -= 15.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxNewDialogue = new Asura_Tickbox_Dialog( "Show Full Strings", xOffset, &s_bShowFullStrings );
	pxNewDialogue->SetHelpText( "Expands/collapses strings." );
	pxNewOverlay->AddDialog( pxNewDialogue );
	xOffset.x += 15.0f;

	pxNewDialogue = new Asura_Tickbox_Dialog( "Show Edit Buttons", xOffset, &s_bShowEditButtons );
	pxNewDialogue->SetHelpText( "Shows/hides the overlay edit buttons." );
	pxNewOverlay->AddDialog( pxNewDialogue );
	xOffset.x += 15.0f;

	pxNewDialogue = new Asura_Tickbox_Dialog( "Show Filter Buttons", xOffset, &s_bShowFilterButtons );
	pxNewDialogue->SetHelpText( "Shows/hides the overlay filter buttons." );
	pxNewOverlay->AddDialog( pxNewDialogue );
	xOffset.x -= 15.0f * 2;
	xOffset.y += Asura_Dialog::LargeSpacing();

    Tools_Blueprint_Edit_Object::ColumnValue aeCurrentColumnValues[ Tools_Blueprint_Edit_Object::CV_NUM_COLUMN_VALUES ];
    Tools_Blueprint_Edit_Object::CalculateCurrentColumnData( s_abColumnVisible, s_iCurrentNumColumns, aeCurrentColumnValues, s_aszCurrentColumnHeadingStrings );

    const Asura_Vector_2 xTableOffset( xOffset.x, xOffset.y );
    static const Asura_Vector_2	s_xMinimumCellSize( 6.0f, 1.2f );
    Asura_Table_Dialog* pxTableDialog = new Asura_Table_Dialog( xTableOffset, s_xMaxTableSize.x, s_xMaxTableSize.y, false );
    pxNewOverlay->AddDialog( pxTableDialog );
    pxTableDialog->CreateTable( s_xMinimumCellSize, 0, s_iCurrentNumColumns, NULL, this, NULL, s_aszCurrentColumnHeadingStrings );
    xOffset.y = xTableOffset.y + pxTableDialog->GetTableSize().y + Asura_Dialog::SmallSpacing();


	pxNewDialogue = new Asura_Dialog( "Filter:", xOffset );
	pxNewOverlay->AddDialog( pxNewDialogue );
	xOffset.x += 4.5f;

	Asura_DropDown_List_Dialog* pxDDLDialogue = new Asura_DropDown_List_Dialog( xOffset, Asura_Parameter::GetGroupNameList(), &s_iFilterChoice,
		Asura_Parameter::GetNumParameterGroups(), Asura_Parameter::GetNumParameterGroups(), NULL, NULL, "" );
	pxDDLDialogue->SetOnSelectCallback( SetFilterGroupNameFromFilterChoice );
	pxNewOverlay->AddDialog( pxDDLDialogue );
	xOffset.x = pxDDLDialogue->m_xMaxCoord[0] + 3.0f;

	pxNewDialogue = new Asura_Function_Dialog( "Add Parameter Group Name", xOffset, AddParameterGroupName );
	pxNewOverlay->AddDialog( pxNewDialogue );
	xOffset.x  = 1.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxNewDialogue = new Asura_Dialog( "Columns:", xOffset );
	pxNewOverlay->AddDialog( pxNewDialogue );
	xOffset.x += 6.0f;
	for ( u_int uColumn = 0; uColumn < Tools_Blueprint_Edit_Object::CV_NUM_COLUMN_VALUES; ++uColumn )
	{
		pxNewDialogue = new Asura_Tickbox_Dialog( "", xOffset, &s_abColumnVisible[ uColumn ], &s_bRefreshDisplayOverlay );
		pxNewDialogue->SetHelpText(Tools_Blueprint_Edit_Object::s_aszAllColumnHeadingStrings[uColumn]);
		pxNewDialogue->SetEnabled((uColumn != Tools_Blueprint_Edit_Object::CV_PARAMETER_NAME) && (uColumn != Tools_Blueprint_Edit_Object::CV_FINAL_VALUE));
		pxNewOverlay->AddDialog( pxNewDialogue );
		xOffset.x += 3.0f;
	}

	xOffset.x  = 1.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

    pxNewDialogue = new Asura_Function_Dialog( "Close", xOffset, CloseDisplayOverlay );
    pxNewOverlay->AddDialog( pxNewDialogue );
    xOffset.y += Asura_Dialog::DefaultSpacing();

    Asura_Overlay_System::SubmitNewOverlay( pxNewOverlay, false );
    s_bDisplayOverlayIsOpen = true;
}

void Tools_Mode_Blueprint::DestroyDisplayOptionsOverlay()
{
    Asura_Overlay_System::CloseOverlays( reinterpret_cast< u_int >( this + 1 ) );
    s_bDisplayOverlayIsOpen = false;
}

void Tools_Mode_Blueprint::ContextMenuCallback( Tools_ContextMenu& xMenu, const Tools_ContextMenu_Context& xContext )
{
    if ( s_pxThisBlueprintMode )
    {
        s_pxThisBlueprintMode->AddContextMenuItems( xMenu, xContext );
    }
}

void Tools_Mode_Blueprint::AddContextMenuItems( Tools_ContextMenu& xMenu, const Tools_ContextMenu_Context& xContext )
{

    Tools_ContextMenu_Section* pxSection = xMenu.GetSection( TOOLS_CONTEXTMENU_SECTION_SELECTION );

    Tools_ContextMenu_Item* pxItem = NULL;

    if ( TreeSelectionIsBlueprint() )
	{

		pxItem = new Tools_ContextMenu_Item_Function( "Edit Selected Blueprint", EditSelectedBlueprint );
		pxItem->SetHelpText( "Edits the currently selected blueprint." );
		pxSection->AddItem( pxItem );

		pxItem = new Tools_ContextMenu_Item_Function( "Create New Blueprint", CreateNewBlueprint );
		pxItem->SetHelpText( "Creates a new blueprint as a child of the currently selected blueprint." );
		pxSection->AddItem( pxItem );

		pxItem = new Tools_ContextMenu_Item_Function( "Delete Selected Blueprint", DeleteSelected, 0 );
		pxItem->SetHelpText( "Deletes the currently selected blueprint and all blueprints below." );
		pxSection->AddItem( pxItem );

		pxItem = new Tools_ContextMenu_Item_Function( "Rename Selected Blueprint", RenameSelectedBlueprint );
		pxItem->SetHelpText( "Renames the currently selected blueprint." );
		pxSection->AddItem( pxItem );

	}
	else if ( TreeSelectionIsType() )
	{
	
		pxItem = new Tools_ContextMenu_Item_Function( "Create New Blueprint", CreateNewBlueprint );
		pxItem->SetHelpText( "Creates a new blueprint as a child of the currently selected type." );
		pxSection->AddItem( pxItem );

		pxItem = new Tools_ContextMenu_Item_Function( "Rename Selected Type", RenameSelectedType );
		pxItem->SetHelpText( "Renames the currently selected blueprint type." );
		pxSection->AddItem( pxItem );

		pxItem = new Tools_ContextMenu_Item_Function( "Set Selected Type File Name", SetSelectedTypeFileName );
		pxItem->SetHelpText( "Sets the file name associated with the currently selected blueprint type." );
		pxSection->AddItem( pxItem );


	}

	pxSection->AddSeparator();

	pxItem = new Tools_ContextMenu_Item_Function("Copy", CopySelected);
	pxItem->SetHelpText("Copies the selected blueprint(s) to the paste buffer");
	pxItem->SetEnabled(TreeSelectionIsBlueprint());
	pxSection->AddItem( pxItem );

	pxItem = new Tools_ContextMenu_Item_Function("Paste", PasteBlueprintOrType);
	pxItem->SetHelpText("Pastes the blueprints in the paste buffer as children of the selection");
	pxItem->SetEnabled(s_xPasteBuffer.IsValid());
	pxSection->AddItem( pxItem );

}

void Tools_Mode_Blueprint::CreatePerTypeData()
{
	ModifiedTypeHashID* pxOldPerTypeData = s_pxPerTypeData;
	int                 iOldNumTypes     = s_iNumTypes;
	s_pxPerTypeData = NULL;
	s_iNumTypes     = 0;

	s_iNumTypes = Asura_Blueprint_System::GetNumTypes();
	if ( s_iNumTypes )
	{
		s_pxPerTypeData = new ModifiedTypeHashID[ s_iNumTypes ];

		int iWriteIndex = 0;
		for( Asura_Blueprint_Type_ConstIterator xIterator; !xIterator.Done(); xIterator.Next(), iWriteIndex++ )
		{
			s_pxPerTypeData[ iWriteIndex ].m_bModified   = false;
			s_pxPerTypeData[ iWriteIndex ].m_uTypeHashID = xIterator.GetCurrentHash();
		}

		// Keep the modified fields up to date.
		for ( int iNewTypeIndex = 0; iNewTypeIndex < s_iNumTypes; ++iNewTypeIndex )
		{
			// Search for this type hash ID in the old list and determine if it was modified.
			bool bWasModified = false;
			for ( int iOldTypeIndex = 0; iOldTypeIndex < iOldNumTypes; ++iOldTypeIndex )
			{
				if ( pxOldPerTypeData[ iOldTypeIndex ].m_uTypeHashID == s_pxPerTypeData[ iNewTypeIndex ].m_uTypeHashID )
				{
					bWasModified = pxOldPerTypeData[ iOldTypeIndex ].m_bModified;
					break;
				}
			}

			s_pxPerTypeData[ iNewTypeIndex ].m_bModified = bWasModified;
		}
	}

	delete [] pxOldPerTypeData;
}

void Tools_Mode_Blueprint::CreateBlueprintTreeAndPerTypeData()
{
    // Build a tree listing all of the blueprint types and the blueprints they contain.
    ModifiedTypeHashID* pxOldPerTypeData = s_pxPerTypeData;
    int                 iOldNumTypes     = s_iNumTypes;
    s_pxPerTypeData = NULL;
    s_iNumTypes     = 0;

	PopulateBlueprintTree( &s_xTree, &s_iNumTypes );
	
    // Create the per type data.
    if ( s_iNumTypes )
    {
        s_pxPerTypeData = new ModifiedTypeHashID[ s_iNumTypes ];

        int iWriteIndex = 0;
		for ( Asura_Blueprint_Type_ConstIterator xIterator; !xIterator.Done(); xIterator.Next(), iWriteIndex++ )
        {
            s_pxPerTypeData[ iWriteIndex ].m_bModified   = false;
            s_pxPerTypeData[ iWriteIndex ].m_uTypeHashID = xIterator.GetCurrentHash();
        }

        // Keep the modified fields up to date.
        for ( int iNewTypeIndex = 0; iNewTypeIndex < s_iNumTypes; ++iNewTypeIndex )
        {
            // Search for this type hash ID in the old list and determine if it was modified.
            bool bWasModified = false;
            for ( int iOldTypeIndex = 0; iOldTypeIndex < iOldNumTypes; ++iOldTypeIndex )
            {
                if ( pxOldPerTypeData[ iOldTypeIndex ].m_uTypeHashID == s_pxPerTypeData[ iNewTypeIndex ].m_uTypeHashID )
                {
                    bWasModified = pxOldPerTypeData[ iOldTypeIndex ].m_bModified;
                    break;
                }
            }

            s_pxPerTypeData[ iNewTypeIndex ].m_bModified = bWasModified;
        }
    }

    delete [] pxOldPerTypeData;
}

static int CompareTreeItemsByName( const Asura_TreeDialog_Item& xOne, const Asura_TreeDialog_Item& xTwo )
{
    return stricmp( xOne.GetName(), xTwo.GetName() );
}

void Tools_Mode_Blueprint::PopulateBlueprintTree( Asura_TreeDialog_Tree* pxTree, int* piNumTypes/*= NULL*/, int iNumValidTypes/*= 0*/, const Asura_Hash_ID* puValidTypesArray/* = NULL*/, int iNumValidParents/* = 0*/, const Asura_Hash_ID* puValidParentsArray/*= NULL*/ )
{
	for ( Asura_Blueprint_Type_ConstIterator xIterator; !xIterator.Done(); xIterator.Next() )
	{
		const Asura_Hash_ID		   uTypeHashID		= xIterator.GetCurrentHash();
		const Asura_BlueprintType* pxBlueprintType	= xIterator.GetCurrent();

		if (iNumValidTypes && puValidTypesArray)
		{
			bool bFound = false;
			for (int i=0; i<iNumValidTypes; ++i)
			{
				if (uTypeHashID == puValidTypesArray[i])
				{
					bFound = true;
					break;
				}
			}

			if (!bFound)
			{
				// Not in our list of valid types, don't bother adding it
				continue;
			}
		}

		if (iNumValidParents && puValidParentsArray)
		{
			// Make sure there is at least one of the valid parents in this type.
			bool bFound = false;
			for (int i=0; i<iNumValidParents; ++i)
			{
				if (Asura_Blueprint_System::GetBlueprintByHashID(uTypeHashID, puValidParentsArray[i]))
				{
					bFound = true;
					break;
				}
			}

			if (!bFound)
			{
				// None of the valid parents are in this type, don't bother adding it
				continue;
			}
		}

		// If the type does not exist in the tree then add it as a root.
		Asura_TreeDialog_Item* pxTypeItem = pxTree->GetRoot()->FindChildByName( pxBlueprintType->m_xTypeName.GetString(), true );
		if ( !pxTypeItem )
		{
            pxTypeItem = pxTree->InsertItem( CompareTreeItemsByName, pxBlueprintType->m_xTypeName.GetString() );
		}

		// Add the type tree nodes, if they do not already exist.
		for ( Asura_Blueprint_ConstIterator xBPIter( uTypeHashID ); !xBPIter.Done(); xBPIter.Next() )
		{
			const Asura_Blueprint* pxBlueprint = xBPIter.GetCurrent();
			if (iNumValidParents && puValidParentsArray)
			{
				bool bFound = false;
				for (int i=0; i<iNumValidParents; ++i)
				{
					if ((pxBlueprint->GetHashID() == puValidParentsArray[i]) || pxBlueprint->IsChildOf(uTypeHashID, puValidParentsArray[i], false))
					{
						bFound = true;
						break;
					}
				}

				if (!bFound)
				{
					// Doesn't have a valid parent, don't bother adding it.
					continue;
				}
			}

			if ( !FindByNameFromChildren( pxTree, pxBlueprint->GetName().GetString(), pxTypeItem ) )
			{
				InsertTreeNode( pxTree, pxTypeItem, pxBlueprint );
			}
		}

		if( piNumTypes )
		{
			*piNumTypes = *piNumTypes + 1;
		}
	}	
}

Asura_TreeDialog_Item* Tools_Mode_Blueprint::FindByNameFromChildren( Asura_TreeDialog_Tree* pxTree, const char* pszName, Asura_TreeDialog_Item* pxParentItem )
{
    Asura_TreeDialog_Item* pxChild = pxParentItem->GetChild();
    while ( pxChild )
    {
        Asura_TreeDialog_Item* pxSearchResult = pxTree->FindByName( pszName, pxChild );
        if ( pxSearchResult )
        {
            return pxSearchResult;
        }

        pxChild = pxChild->GetSibling();
    }

    return NULL;
}

void Tools_Mode_Blueprint::InsertTreeNode( Asura_TreeDialog_Tree* pxTree, Asura_TreeDialog_Item* pxTypeItem, const Asura_Blueprint* pxBlueprint )
{
    const Asura_Hash_ID uTypeHashID = Asura_GetHashID( pxTypeItem->GetName() );

    // First off, check to see if this blueprint has already been added to the tree.
    // We can have no name duplication in a type, so if we find a node with this name underneath the type item this blueprint has already been added.
    if ( FindByNameFromChildren( pxTree, pxBlueprint->GetName().GetString(), pxTypeItem ) )
    {
        return;
    }

    if ( pxBlueprint->GetParentHashID() == ASURA_HASH_ID_UNSET )
    {
        // This blueprint is at the top of a hiearchy, so add it underneath the type item.
        pxTree->InsertItem( CompareTreeItemsByName, pxTypeItem, pxBlueprint->GetName().GetString(), static_cast< int >( uTypeHashID ) );
    }
    else
    {
        // See if this blueprint's parent has already been added.
        Asura_Blueprint* pxParentBlueprint = pxBlueprint->GetParent();
        if ( !pxParentBlueprint )
        {
            Asura_Assert( false, "Tools_Mode_Blueprint::AddTreeNode - could not find parent blueprint" );
            return;
        }

        Asura_TreeDialog_Item* pxParentItem = FindByNameFromChildren( pxTree, pxParentBlueprint->GetName().GetString(), pxTypeItem );
        if ( !pxParentItem )
        {
            // If the parent's not there we need to sort that out first.
            InsertTreeNode( pxTree, pxTypeItem, pxParentBlueprint );
            pxParentItem = FindByNameFromChildren( pxTree, pxParentBlueprint->GetName().GetString(), pxTypeItem );
        }

        Asura_Assert( pxParentItem, "Tools_Mode_Blueprint::AddTreeNode - this should be added by now" );
        if ( pxParentItem )
        {
            // We can now add this.
            pxTree->InsertItem( CompareTreeItemsByName, pxParentItem, pxBlueprint->GetName().GetString(), static_cast< int >( uTypeHashID ) );
        }
    }
}

void Tools_Mode_Blueprint::DestroyBlueprintTree()
{
    s_xTree.DeleteAll();
}

void Tools_Mode_Blueprint::SetTreeTypeNodeColours()
{
    const bool bOverlayIsOnTop = ( Asura_Overlay_System::GetTopOverlayID() == reinterpret_cast< u_int >( this ) );

    // Make sure that the type tree nodes have the correct colour.
    int iReadIndex = 0;
	for ( Asura_Blueprint_Type_ConstIterator xIterator; !xIterator.Done(); xIterator.Next(), iReadIndex++ )
    {
		const Asura_BlueprintType*	pxBlueprintType	= xIterator.GetCurrent();

        // If the type does not exist in the tree then add it as a root.
        Asura_TreeDialog_Item* pxTypeItem = s_xTree.FindByName( pxBlueprintType->m_xTypeName.GetString() );
        if ( pxTypeItem )
        {
            if ( s_pxPerTypeData[ iReadIndex ].m_bModified )
            {
                pxTypeItem->SetColourOverride( bOverlayIsOnTop ? 0xffff0000 : 0xffB03434 );
            }
            else
            {
                pxTypeItem->SetColourOverride( 0 );
            }
        }
    }    
}

void Tools_Mode_Blueprint::AddParameterGroupName()
{
    GetString_Overlay xGetStringOverlay( "Enter the new parameter group name:", "" );
    bool bOk = false;
    do 
    {
        if ( !xGetStringOverlay.DoModal() )
        {
            return;
        }

        // Check the name is not already being used.
        bOk = ( Asura_Parameter::GetGroupListIndex( Asura_String( xGetStringOverlay.GetString() ) ) == -1 );
        // Check the name is not our special UI 'no group' name.
        bOk &= ( stricmp( xGetStringOverlay.GetString(), "*None*" ) != 0 );
        if ( !bOk )
        {
            Message_Overlay xMessageOverlay( "Please choose a name that is not already being used." );
            xMessageOverlay.DoModal();
            continue;
        }
    } while ( !bOk );

    Asura_Parameter::AddGroupName( xGetStringOverlay.GetString() );

    // We'll now need to rebuild all of the blueprint edit object overlays, as the parameter group name list will be invalid.
    s_bRefreshEditObjectOverlays = true;
    // We'll also have to rebuild the display options overlay, if it's open.
    s_bRefreshDisplayOverlay = true;
}

Tools_Blueprint_Edit_Object* Tools_Mode_Blueprint::CreateBlueprintEditObject()
{
    return new Tools_Blueprint_Edit_Object;
}

void Tools_Mode_Blueprint::RecreateStoredEditObjects()
{
	while ( s_xEditObjectDisplayData.Front() != NULL )
	{
		Asura_ListElt< Tools_BEO_Display_Data >* pxElt = s_xEditObjectDisplayData.Front();
		const Tools_BEO_Display_Data xDisplayData = pxElt->Data();
		s_xEditObjectDisplayData.Remove( pxElt );

		Tools_Blueprint_Edit_Object* pxBEO = CreateBlueprintEditObject();
		pxBEO->RestoreOverlayState( xDisplayData );
	}
}

int CompareEditObjectOverlayIndices( const Tools_BEO_Display_Data& xOne, const Tools_BEO_Display_Data& xTwo )
{
	// Overlay index is always positive, so we can use the quick and dirty test.
	return ( xOne.m_iOverlayIndex - xTwo.m_iOverlayIndex );
}

void Tools_Mode_Blueprint::RememberEditObjects()
{
	s_xEditObjectDisplayData.Clear();

	for ( Asura_List_It< Tools_Blueprint_Edit_Object > xBEOIter( Tools_Blueprint_Edit_Object::GetList() ); !xBEOIter.Done(); xBEOIter.Next() )
	{
		Tools_Blueprint_Edit_Object* pxBEO = xBEOIter.GetCurrent();
		Tools_BEO_Display_Data xDisplayData;
		pxBEO->RecordOverlayState( xDisplayData );
		s_xEditObjectDisplayData.Insert( xDisplayData, CompareEditObjectOverlayIndices );
	}
}

void Tools_Mode_Blueprint::DestroyEditObjects()
{
    Asura_List_It< Tools_Blueprint_Edit_Object > xIt( Tools_Blueprint_Edit_Object::GetList() );
    while ( !xIt.Done() )
    {
        // The destructor for the edit object takes itself off the list.
        delete xIt.GetCurrent();
        xIt.Restart();
    }
}

void Tools_Mode_Blueprint::TransferModeData( const int iMode )
{
    Tools_ModeDataFile xFile( "BlueprintMode.dat", iMode );
    TransferToolsModeData( xFile );
}

void Tools_Mode_Blueprint::TransferToolsModeData( Tools_ModeDataFile& xFile )
{
    int		iVersion = CURRENT_SAVE_VERSION;
	int		iSortChoice;
	bool	bSortReverse;

    xFile.Transfer( iVersion );

	if ( iVersion == CURRENT_SAVE_VERSION )
	{
		xFile.Transfer( s_xOverlayPosition );
		xFile.Transfer( s_xTreeDialogueDimensions );
		xFile.Transfer( Tools_Blueprint_Edit_Object::s_xBlueprintOverlayPosition );
		xFile.Transfer( s_bShowFullStrings );
		xFile.Transfer( s_bShowEditButtons );
		xFile.Transfer( s_bShowFilterButtons );
		xFile.Transfer( s_xFilterGroupName );
		for ( int iColumnValue = 0; iColumnValue < Tools_Blueprint_Edit_Object::CV_NUM_COLUMN_VALUES; ++iColumnValue )
		{
			xFile.Transfer( s_abColumnVisible[ iColumnValue ] );
		}
		xFile.Transfer( s_xMaxTableSize );
		xFile.Transfer( s_xLastSkinAnimPath );
		xFile.Transfer( s_xLastTexturePath );
		xFile.Transfer( s_xLastTypeChoice );
	}
	else if ( iVersion == 7 )
	{
		xFile.Transfer( s_xOverlayPosition );
		xFile.Transfer( s_xTreeDialogueDimensions );
		xFile.Transfer( Tools_Blueprint_Edit_Object::s_xBlueprintOverlayPosition );
		xFile.Transfer( s_bShowFullStrings );
		xFile.Transfer( s_bShowEditButtons );
		xFile.Transfer( s_bShowFilterButtons );
		xFile.Transfer( s_xFilterGroupName );
		for ( int iColumnValue = 0; iColumnValue < Tools_Blueprint_Edit_Object::CV_NUM_COLUMN_VALUES; ++iColumnValue )
		{
			xFile.Transfer( s_abColumnVisible[ iColumnValue ] );
		}
		xFile.Transfer( s_xMaxTableSize );
		xFile.Transfer( s_xLastSkinAnimPath );
		xFile.Transfer( s_xLastTexturePath );
		int iDummy;
		xFile.Transfer( iDummy );
	}
	else if ( iVersion == 6 )
	{
		xFile.Transfer( s_xOverlayPosition );
		xFile.Transfer( s_xTreeDialogueDimensions );
		xFile.Transfer( Tools_Blueprint_Edit_Object::s_xBlueprintOverlayPosition );
		xFile.Transfer( s_bShowFullStrings );
		xFile.Transfer( s_bShowEditButtons );
		xFile.Transfer( s_bShowFilterButtons );
		xFile.Transfer( s_xFilterGroupName );
		for ( int iColumnValue = 0; iColumnValue < Tools_Blueprint_Edit_Object::CV_NUM_COLUMN_VALUES; ++iColumnValue )
		{
			xFile.Transfer( s_abColumnVisible[ iColumnValue ] );
		}
		xFile.Transfer( s_xMaxTableSize );
		xFile.Transfer( s_xLastSkinAnimPath );
		xFile.Transfer( s_xLastTexturePath );
	}
	else if ( iVersion == 5 )
	{
		xFile.Transfer( s_xOverlayPosition );
		xFile.Transfer( s_xTreeDialogueDimensions );
		xFile.Transfer( Tools_Blueprint_Edit_Object::s_xBlueprintOverlayPosition );
		xFile.Transfer( s_bShowFullStrings );
		xFile.Transfer( s_bShowEditButtons );
		xFile.Transfer( s_bShowFilterButtons );
		xFile.Transfer( s_xFilterGroupName );
		for ( int iColumnValue = 0; iColumnValue < Tools_Blueprint_Edit_Object::CV_NUM_COLUMN_VALUES; ++iColumnValue )
		{
			xFile.Transfer( s_abColumnVisible[ iColumnValue ] );
		}
		xFile.Transfer( s_xMaxTableSize );
	}
	else if ( iVersion == 4 )
    {
        xFile.Transfer( s_xOverlayPosition );
        xFile.Transfer( s_xTreeDialogueDimensions );
        xFile.Transfer( Tools_Blueprint_Edit_Object::s_xBlueprintOverlayPosition );
        xFile.Transfer( s_bShowFullStrings );
        xFile.Transfer( s_bShowEditButtons );
		xFile.Transfer( iSortChoice );
		xFile.Transfer( bSortReverse );
        xFile.Transfer( s_xFilterGroupName );
		for ( int iColumnValue = 0; iColumnValue < 6; ++iColumnValue )
		{
			xFile.Transfer( s_abColumnVisible[ iColumnValue + 1 ] );
		}
		xFile.Transfer( s_xMaxTableSize );
    }
    else if ( iVersion == 3 )
    {
        xFile.Transfer( s_xOverlayPosition );
        xFile.Transfer( s_xTreeDialogueDimensions );
        xFile.Transfer( Tools_Blueprint_Edit_Object::s_xBlueprintOverlayPosition );
        xFile.Transfer( s_bShowFullStrings );
        xFile.Transfer( s_bShowEditButtons );
		xFile.Transfer( iSortChoice );
		xFile.Transfer( bSortReverse );
        for ( int iColumnValue = 0; iColumnValue < 6; ++iColumnValue )
        {
            xFile.Transfer( s_abColumnVisible[ iColumnValue + 1 ] );
        }
    }
    else if ( iVersion == 2 )
    {
        xFile.Transfer( s_xOverlayPosition );
        xFile.Transfer( s_xTreeDialogueDimensions );
        xFile.Transfer( Tools_Blueprint_Edit_Object::s_xBlueprintOverlayPosition );
        xFile.Transfer( s_bShowFullStrings );
		xFile.Transfer( iSortChoice );
		xFile.Transfer( bSortReverse );
    }
    else if ( iVersion == 1 )
    {
        xFile.Transfer( s_xOverlayPosition );
        xFile.Transfer( s_xTreeDialogueDimensions );
        xFile.Transfer( Tools_Blueprint_Edit_Object::s_xBlueprintOverlayPosition );
        Asura_Vector_2 xDummyVector;
        xFile.Transfer( xDummyVector );
        xFile.Transfer( s_bShowFullStrings );
		xFile.Transfer( iSortChoice );
		xFile.Transfer( bSortReverse );
    }
    else if ( iVersion == 0 )
    {
        xFile.Transfer( s_xOverlayPosition );
        xFile.Transfer( Tools_Blueprint_Edit_Object::s_xBlueprintOverlayPosition );
        Asura_Vector_2 xDummyVector;
        xFile.Transfer( xDummyVector );
        xFile.Transfer( s_bShowFullStrings );
		xFile.Transfer( iSortChoice );
		xFile.Transfer( bSortReverse );
    }
	else
	{
		Asura_Assert(false, "Unexpected version for blueprint options file.");
	}

	s_abColumnVisible[Tools_Blueprint_Edit_Object::CV_PARAMETER_NAME]	= true;
	s_abColumnVisible[Tools_Blueprint_Edit_Object::CV_FINAL_VALUE]		= true;
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

bool Tools_Mode_Blueprint::TreeSelectionValid()
{
    Asura_TreeDialog_Item* pxCurrentItem = s_pxThisBlueprintMode->s_xTree.GetSelectedItem();
    return ( pxCurrentItem != NULL );
}

bool Tools_Mode_Blueprint::TreeSelectionIsType()
{
    const Asura_TreeDialog_Item* pxCurrentItem = s_pxThisBlueprintMode->s_xTree.GetSelectedItem();
    return ( pxCurrentItem && pxCurrentItem->GetUserData() == 0 );
}

bool Tools_Mode_Blueprint::TreeSelectionIsBlueprint()
{
    const Asura_TreeDialog_Item* pxCurrentItem = s_pxThisBlueprintMode->s_xTree.GetSelectedItem();
    return ( pxCurrentItem && pxCurrentItem->GetUserData() != 0 );
}

void Tools_Mode_Blueprint::EditSelectedBlueprint()
{
	const List<Asura_TreeDialog_Item*>& xSelectionList = s_pxThisBlueprintMode->s_xTree.GetSelection();

	if(xSelectionList.size() > 4)
	{
		//Opening large number of edit overlays at once is probably a bad plan , so confirm that the user wants to do it.
		bool bOkay = ShowYesNoOverlay("%d blueprints have been selected. Open overlays for editing all of them?",xSelectionList.size());
		if(!bOkay) return;
	}
   
	for(CLIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelectionList); !xSelectionLIF.done(); xSelectionLIF.next())
	{
    	Asura_TreeDialog_Item* pxCurrentItem = xSelectionLIF();
    	if ( !pxCurrentItem )
    	{
    	    return;
    	}
    	else if ( pxCurrentItem->GetUserData() )
    	{
    	    // If the current item has user data then it is a blueprint, the user data being the blueprint type hash.
    	    const Asura_Hash_ID uTypeHashID      = pxCurrentItem->GetUserData();
    	    const Asura_Hash_ID uBlueprintHashID = Asura_GetHashID( pxCurrentItem->GetName() );

			CreateEditOverlay( uTypeHashID, uBlueprintHashID );
    	}
    	else
    	{
    	    // If the current item does not have user data then it is a blueprint type.
    	}
	}
}

void Tools_Mode_Blueprint::RenameSelectedType()
{
	Asura_TreeDialog_Item* pxCurrentItem = s_pxThisBlueprintMode->s_xTree.GetSelectedItem();
	if ( pxCurrentItem && !pxCurrentItem->GetUserData() )
	{
		Asura_String xTitleString( "Enter The New Name For Type '" );
		xTitleString += pxCurrentItem->GetName();
		xTitleString += "':";

		Asura_String xNewTypeName( pxCurrentItem->GetName() );
		if ( GetUniqueBlueprintTypeName( xTitleString, xNewTypeName ) )
		{
			RenameBlueprintType(pxCurrentItem, xNewTypeName.GetString());
		}
	}
}

void Tools_Mode_Blueprint::RenameTreeItem(Asura_TreeDialog_Item* pxTreeItem, const char* szName)
{
	//rename blueprint or type depending on the user data
	if(pxTreeItem)
	{
		if(pxTreeItem->GetUserData())
		{
			RenameBlueprint(pxTreeItem, szName);
		}
		else
		{
			RenameBlueprintType(pxTreeItem, szName);
		}
	}
}

void Tools_Mode_Blueprint::RenameBlueprintType(Asura_TreeDialog_Item* pxTreeItem, const char* szName)
{
	if ( pxTreeItem && !pxTreeItem->GetUserData() )
	{
		if ( !stricmp(pxTreeItem->GetName(), szName) )
		{
			// Handle this like a cancel - no further work to do.
			return;
		}

		if ( IsValidBlueprintTypeName(szName ) )
		{
			if ( Asura_Blueprint_System::RenameBlueprintType( pxTreeItem->GetName(), szName ) )
			{
				Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_INFO,
					"Renamed type '%s' to '%s'.", pxTreeItem->GetName(), szName );

				Asura_Hash_ID uOldTypeHashID = Asura_GetHashID( pxTreeItem->GetName() );
				Asura_Hash_ID uNewTypeHashID = Asura_GetHashID( szName );
				
				// Update the modified status and hash ID of the per-type data.
				if ( s_pxPerTypeData )
				{
					for ( int i = 0; i < s_iNumTypes; ++i )
					{
						if ( s_pxPerTypeData[ i ].m_uTypeHashID == uOldTypeHashID )
						{
                            s_pxPerTypeData[ i ].m_bModified   = true;
							s_pxPerTypeData[ i ].m_uTypeHashID = uNewTypeHashID;
							break;
						}
					}
				}
				
				//Set the new type ID in the user data for all the children
				for(Asura_TreeDialog_Item* pxChild = pxTreeItem->GetChild(); pxChild; pxChild = pxChild->GetSibling())
				{
					UpdateUserDataRecursive(pxChild, uOldTypeHashID, uNewTypeHashID);
				}

				// Update the label of the tree dialog.
				pxTreeItem->SetName( szName );

				// Re-create the blueprint overlay to be sure.
				s_pxThisBlueprintMode->DestroyOverlay();
				s_pxThisBlueprintMode->CreateOverlay();
				s_pxThisBlueprintMode->RequestRefreshBlueprintEditObjectOverlays();
			}
			else
			{
				Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Could not rename type." );
			}
		}
	}
}

void Tools_Mode_Blueprint::UpdateUserDataRecursive(Asura_TreeDialog_Item* pxItem, Asura_Hash_ID uOldTypeID, Asura_Hash_ID uNewTypeID)
{
	//update this
	if(pxItem->GetUserData() == (int)uOldTypeID)
	{
		pxItem->SetUserData(uNewTypeID);
	}
	else
	{
		Asura_Assert(0,"Was expecting all the child items to be using the old blueprint type ID");
	}
	//and recurse down
	for(Asura_TreeDialog_Item* pxChild = pxItem->GetChild(); pxChild; pxChild = pxChild->GetSibling())
	{
		UpdateUserDataRecursive(pxChild, uOldTypeID, uNewTypeID);
	}
}

void Tools_Mode_Blueprint::SetSelectedTypeFileName()
{
	Asura_TreeDialog_Item* pxCurrentItem = s_pxThisBlueprintMode->s_xTree.GetSelectedItem();
	if ( pxCurrentItem && !pxCurrentItem->GetUserData() )
	{
		Asura_String xTitleString;
		xTitleString = Asura_String( "Enter The File Name For Type '" ) + pxCurrentItem->GetName() + Asura_String( "':" );
		xTitleString += "\n";
		xTitleString += "(normally something like Blueprints\\TypeName.bpt)";

        const Asura_Hash_ID uTypeHashID    = Asura_GetHashID( pxCurrentItem->GetName() );
		const Asura_String*	pxTypeFileName = Asura_Blueprint_System::GetTypeFileName( uTypeHashID );
		Asura_String xTypeFileName( pxTypeFileName ? *pxTypeFileName : "" );
		if ( GetUniqueBlueprintTypeFileName( xTitleString, xTypeFileName ) )
		{
			// Add the extension on if people have forgotten it.
			Asura_String xPathName;
			Asura_String xCleanTypeFileName;
			xTypeFileName.TidyFilename();
			xTypeFileName.SplitFilename( &xPathName, &xCleanTypeFileName, NULL );
			if ( xPathName.IsBlank() )
			{
				xTypeFileName = "Blueprints\\" + xCleanTypeFileName + szBLUEPRINTS_EXTENSION;
				Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_INFO,
					"You did not specify a path. The path has been set to Blueprints\\." );
			}
			else
			{
				xTypeFileName = xPathName + "\\" + xCleanTypeFileName + szBLUEPRINTS_EXTENSION;
			}

			if ( Asura_Blueprint_System::SetBlueprintTypeFileName( uTypeHashID, xTypeFileName ) )
			{
				Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_INFO,
					"Set file name of type '%s' to '%s'.", pxCurrentItem->GetName(), xTypeFileName );
				SetTypeAsModified( uTypeHashID );
			}
			else
			{
				Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Could not set type file name." );
			}
		}
	}
}

void Tools_Mode_Blueprint::RenameSelectedBlueprint()
{
    Asura_TreeDialog_Item* pxCurrentItem = s_pxThisBlueprintMode->s_xTree.GetSelectedItem();
    if ( pxCurrentItem && pxCurrentItem->GetUserData() )
    {
        // If the current item has user data then it is a blueprint, the user data being the blueprint type hash.
        const Asura_Hash_ID uTypeHashID      = pxCurrentItem->GetUserData();

        // Get a new name for the blueprint.
        Asura_String xTitleString( "Enter the new name for blueprint '" );
        xTitleString += pxCurrentItem->GetName();
		xTitleString += "':";
        Asura_String xNameString( pxCurrentItem->GetName() );
        if ( !GetUniqueBlueprintName( xTitleString, uTypeHashID, xNameString ) )
        {
            return;
        }

		RenameBlueprint(pxCurrentItem, xNameString.GetString());

    }
}

void Tools_Mode_Blueprint::RenameBlueprint(Asura_TreeDialog_Item* pxTreeItem, const char* szName)
{
	if(!szName || !*szName) return;

    if ( pxTreeItem && pxTreeItem->GetUserData() )
    {
        // If the current item has user data then it is a blueprint, the user data being the blueprint type hash.
        const Asura_Hash_ID uTypeHashID      = pxTreeItem->GetUserData();
        const Asura_Hash_ID uBlueprintHashID = Asura_GetHashID( pxTreeItem->GetName() );

        // Check through the list of edit objects to see if this is already being edited.
        Tools_Blueprint_Edit_Object* pxEditObject = NULL;
        for ( Asura_List_It< Tools_Blueprint_Edit_Object > xIt( Tools_Blueprint_Edit_Object::GetList() ); !xIt.Done(); xIt.Next() )
        {
            if ( xIt.GetCurrent() &&
                xIt.GetCurrent()->GetBlueprintTypeHashID() == uTypeHashID &&
                xIt.GetCurrent()->GetBlueprintHashID() == uBlueprintHashID )
            {
                // Already editing this.
                pxEditObject = xIt.GetCurrent();
                break;
            }
        }

       	if ( !IsValidBlueprintName( pxTreeItem->GetName(), uTypeHashID, szName ) )
       	{
       	    return;
       	}


        // Update the blueprint data and refresh the overlays if necessary.
        Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uTypeHashID, uBlueprintHashID );
		Asura_Assert( pxBlueprint, "Tools_Mode_Blueprint::RenameSelectedBlueprint - could not find the blueprint." );
		Asura_BlueprintType* pxType = Asura_Blueprint_System::GetType( uTypeHashID );
		Asura_Assert( pxType, "Tools_Mode_Blueprint::RenameSelectedBlueprint - could not find the blueprint type" );
        if ( pxBlueprint && pxType )
        {
            const Asura_Hash_ID uNewBlueprintHashID = Asura_GetHashID( szName );

			// Update all the child blueprints' parent hash IDs.
			int iNumChildBlueprints = Asura_Blueprint_System::GetNumChildBlueprints( uTypeHashID, uBlueprintHashID, true );
			while ( --iNumChildBlueprints >= 0 )
			{
				Asura_Blueprint* pxChildBlueprint = Asura_Blueprint_System::GetChildBlueprint( uTypeHashID, uBlueprintHashID, true, iNumChildBlueprints );
				if ( pxChildBlueprint )
				{
					pxChildBlueprint->SetParentHashID( uNewBlueprintHashID );
				}
			}

            // Update this blueprint's hash ID and name, in both the blueprint and the tree storage.
            pxBlueprint->SetHashID( uNewBlueprintHashID );
			pxBlueprint->SetName( szName );
			if ( !pxType->m_xBlueprintHashTree.ReplaceHash( uBlueprintHashID, uNewBlueprintHashID ) )
			{
				Asura_Assert( false, "Tools_Mode_Blueprint::RenameSelectedBlueprint - failed to replace hash in hash tree" );
			}

			// Relink all of the blueprints in this type again.
            Asura_Blueprint_System::EstablishAllLinkage( uTypeHashID );

            // Update the UI data.
            pxTreeItem->SetName( szName );

            if ( pxEditObject )
            {
                pxEditObject->SetBlueprintHashID( uNewBlueprintHashID );
            }

			s_pxThisBlueprintMode->RequestRefreshBlueprintEditObjectOverlays();
            SetTypeAsModified( uTypeHashID );
			BlueprintChanged();
		}
    }
}


void Tools_Mode_Blueprint::CreateNewType()
{
    // Get a new type name, and make sure it doesn't conflict with an already existing one.
    const Asura_String xTitleString( "Enter the name of the new blueprint type:" );
    Asura_String       xNameString;
    if ( !GetUniqueBlueprintTypeName( xTitleString, xNameString ) )
    {
        return;
    }

#ifdef OUTPUT_BLUEPRINT_DEBUG_INFO
    Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_DEBUG ], NULL, "Creating new type: %s", xNameString.GetString() );
#endif

    Asura_Blueprint_System::AddBlueprintType( xNameString );

    s_pxThisBlueprintMode->DestroyOverlay();
    s_pxThisBlueprintMode->CreateOverlay();

    SetTypeAsModified( Asura_GetHashID( xNameString.GetString() ) );
    Tools_SaveAsura::SetBlueprintsModified();
}

void Tools_Mode_Blueprint::CreateNewBlueprint()
{
    Asura_TreeDialog_Item* pxCurrentItem = s_pxThisBlueprintMode->s_xTree.GetSelectedItem();
    if ( !pxCurrentItem ) { return; }

    Asura_Hash_ID uTypeHashID            = ASURA_HASH_ID_UNSET;
    Asura_Hash_ID uParentBlueprintHashID = ASURA_HASH_ID_UNSET;
    if ( pxCurrentItem->GetUserData() )
    {
        // If the current item has user data then it is a blueprint, the user data being the blueprint type hash.
        uTypeHashID            = pxCurrentItem->GetUserData();
        uParentBlueprintHashID = Asura_GetHashID( pxCurrentItem->GetName() );
    }
    else
    {
        // If the current item does not have user data then it is a blueprint type.
        uTypeHashID = Asura_GetHashID( pxCurrentItem->GetName() );
    }

    const Asura_String xTitleString( "Enter the name of the new blueprint:" );
    Asura_String       xNameString;
    if ( !GetUniqueBlueprintName( xTitleString, uTypeHashID, xNameString ) )
    {
        return;
    }

#ifdef OUTPUT_BLUEPRINT_DEBUG_INFO
    if ( pxCurrentItem->GetUserData() )
    {
        Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_DEBUG ], NULL, "Creating new blueprint: %s, child of blueprint %s",
            xNameString.GetString(), pxCurrentItem->GetName() );
    }
    else
    {
        Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_DEBUG ], NULL, "Creating new blueprint: %s, child of type %s",
            xNameString.GetString(), pxCurrentItem->GetName() );
    }
#endif
	// Expand the selected item so that the new blueprint will be visible in the tree. RAR 13/02/2007
	pxCurrentItem->Expand( true );

    Asura_Blueprint_System::AddBlueprint( uTypeHashID, xNameString, uParentBlueprintHashID );

    s_pxThisBlueprintMode->DestroyOverlay();
    s_pxThisBlueprintMode->CreateOverlay();

    SetTypeAsModified( uTypeHashID );
    Tools_SaveAsura::SetBlueprintsModified();
	BlueprintChanged();
}

void Tools_Mode_Blueprint::DeleteSelected( int iSilent )
{
	List<Asura_TreeDialog_Item*> xSelectionList;
	s_xTree.GetSelectionRemovingChildren(xSelectionList);
	if(!xSelectionList.size()) return;

	bool bDeleteEverything = true;
	
	int iNumChildren = 0;
	//Remove any blueprint types from the selection , and determine the number of children , for showing the appropriate
	//confirmation message
	for(LIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelectionList); !xSelectionLIF.done();)
	{
		Asura_TreeDialog_Item* pxSelectedItem = xSelectionLIF();
		
	    if ( pxSelectedItem->GetUserData() )
	    {
	        // If the current item has user data then it is a blueprint, the user data being the blueprint type hash.
	        const Asura_Hash_ID uBlueprintTypeHashID = static_cast< Asura_Hash_ID >( pxSelectedItem->GetUserData() );
	        const Asura_Hash_ID uBlueprintHashID     = Asura_GetHashID( pxSelectedItem->GetName() );

			iNumChildren += Asura_Blueprint_System::GetNumChildBlueprints( uBlueprintTypeHashID, uBlueprintHashID, false );

			xSelectionLIF.next();
		}
		else
		{
			xSelectionLIF.delete_current();
		}
	}
	if(!xSelectionList.size()) return;

	if(!iSilent)
	{
		if(xSelectionList.size() == 1)
		{
			Asura_TreeDialog_Item* pxSelectedItem = xSelectionList.first_entry();
	        
	        if ( iNumChildren == 0 )
	        {
	            //single selection with no children
	            Asura_String xQuestion( "Are you sure you want to delete blueprint '" );
				xQuestion += pxSelectedItem->GetName();
				xQuestion += "'?";
	            YesNo_Overlay xYesNoOverlay( xQuestion );
	            if ( !xYesNoOverlay.DoModal() )
	            {
	                return;
	            }
	            bDeleteEverything = true;
	        }
	        else if ( iNumChildren == 1 )
	        {
	            //single selection with one child
	            Asura_String xQuestion( "Blueprint '" );
				xQuestion += pxSelectedItem->GetName();
				xQuestion += "' has a child! Do you want to delete this also?\nIf not deleted it will be moved up the hierarchy.";
	            YesNoCancel_Overlay xYesNoCancelOverlay( xQuestion );
	            if ( !xYesNoCancelOverlay.DoModal() )
	            {
	                return;
	            }
	            bDeleteEverything = xYesNoCancelOverlay.GetResult();
	        }
	        else
	        {
	            //single selection with multiple children
				Asura_String xQuestion( "Blueprint '" );
				xQuestion += pxSelectedItem->GetName();
				xQuestion += "' has ";
				Asura_String xNumber;
				Asura_StringUtil::ToString( iNumChildren, xNumber );
				xQuestion += xNumber;
				xQuestion += " children! Do you want to delete these also?\nIf not deleted they will be moved up the hierarchy.";
	            YesNoCancel_Overlay xYesNoCancelOverlay( xQuestion );
	            if ( !xYesNoCancelOverlay.DoModal() )
	            {
	                return;
	            }
	            bDeleteEverything = xYesNoCancelOverlay.GetResult();
	        }
		}
		else
		{
	        if ( iNumChildren == 0 )
	        {
	            //multiple selection with no children
				if(!ShowYesNoOverlay("Are you sure you want to delete the %d selected blueprints?", xSelectionList.size()))
				{
					return;
				}

	            bDeleteEverything = true;
	        }
	        else
	        {
	            //multiple selection with some children
	            char acMessage[200];
				Asura_PrintfToBfr(acMessage, "The %d selected blueprints have %d children! Do you want to delete these also?\nIf not deleted they will be moved up the hierarchy.", sizeof(acMessage), true,
									xSelectionList.size(),
									iNumChildren);

	            YesNoCancel_Overlay xYesNoCancelOverlay( acMessage );
	            if ( !xYesNoCancelOverlay.DoModal() )
	            {
	                return;
	            }
	            bDeleteEverything = xYesNoCancelOverlay.GetResult();
	        }
		}
	}

	bool bNeedToRebuildTree = false;

	for(LIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelectionList); !xSelectionLIF.done(); xSelectionLIF.next())
	{
		Asura_TreeDialog_Item* pxSelectedItem = xSelectionLIF();

	    if ( pxSelectedItem->GetUserData() )
	    {
	        // If the current item has user data then it is a blueprint, the user data being the blueprint type hash.
	        // Determine how many children this blueprint has, and what the user wants to do with them.
	        const Asura_Hash_ID uBlueprintTypeHashID = static_cast< Asura_Hash_ID >( pxSelectedItem->GetUserData() );
	        const Asura_Hash_ID uBlueprintHashID     = Asura_GetHashID( pxSelectedItem->GetName() );

#ifdef OUTPUT_BLUEPRINT_DEBUG_INFO
			const int iNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( uBlueprintTypeHashID, uBlueprintHashID, false );
#endif

	        if ( bDeleteEverything )
	        {
	            // Delete this blueprint and its children.
				for ( Asura_Blueprint_ConstIterator xBPIter( uBlueprintTypeHashID ); !xBPIter.Done(); xBPIter.Next() )
				{
					const Asura_Blueprint* pxBlueprint = xBPIter.GetCurrent();
					if ( pxBlueprint->IsChildOf( uBlueprintTypeHashID, uBlueprintHashID, false ) )
					{
						Asura_Blueprint_System::MarkBlueprintForDeletion( uBlueprintTypeHashID, pxBlueprint->GetName() );
					}
				}
	            Asura_Blueprint_System::MarkBlueprintForDeletion( uBlueprintTypeHashID, pxSelectedItem->GetName() );
	            Asura_Blueprint_System::DeleteAllMarkedBlueprints( uBlueprintTypeHashID );

#ifdef OUTPUT_BLUEPRINT_DEBUG_INFO
	            if ( iNumChildren == 0 )
	            {
	                Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_DEBUG ], NULL,
	                    "Deleting blueprint '%s'", pxSelectedItem->GetName() );
	            }
	            else
	            {
	                Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_DEBUG ], NULL,
	                    "Deleting blueprint '%s' and all children", pxSelectedItem->GetName() );
	            }
#endif

	            s_pxThisBlueprintMode->s_xTree.DeleteItem( pxSelectedItem );
	        }
	        else
	        {
	            // Delete just this blueprint, and set its children to have this blueprint's parent as their parent.
	            // If the blueprint has any parameters then the user will have to resolve these first.
	            Asura_Blueprint* pxToDelete = Asura_Blueprint_System::GetBlueprintByHashID( uBlueprintTypeHashID, uBlueprintHashID );
	            if ( !pxToDelete )
	            {
	                continue;
	            }

	            bool bOkayToDelete = true;
	            for ( int i = 0; i < pxToDelete->GetNumParameters(); ++i )
	            {
	                if ( pxToDelete->GetParameterByIndex( i )->GetOverrideType() == ABOT_NONE )
	                {
						Asura_String xTitle( "Blueprint '" );
						xTitle += pxToDelete->GetName();
						xTitle += "' has parameters which must be deleted first.";
	                    Message_Overlay xResolveError( xTitle );
	                    xResolveError.DoModal();
	                    bOkayToDelete = false;
						break;
	                }
	            }
				if(!bOkayToDelete) continue;

				for ( Asura_Blueprint_Iterator xBPIter( uBlueprintTypeHashID ); !xBPIter.Done(); xBPIter.Next() )
				{
					Asura_Blueprint* pxBlueprint = xBPIter.GetCurrent();
					if ( pxBlueprint->IsChildOf( uBlueprintTypeHashID, uBlueprintHashID, false ) )
					{
						pxBlueprint->SetParentHashID( pxToDelete->GetParentHashID() );
					}
				}
	            Asura_Blueprint_System::MarkBlueprintForDeletion( uBlueprintTypeHashID, pxSelectedItem->GetName() );
	            Asura_Blueprint_System::DeleteAllMarkedBlueprints( uBlueprintTypeHashID );
	            Asura_Blueprint_System::EstablishAllLinkage( uBlueprintTypeHashID );

#ifdef OUTPUT_BLUEPRINT_DEBUG_INFO
	            Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_DEBUG ], NULL,
	                "Deleting blueprint %s but not its children", pxSelectedItem->GetName() );
#endif

	            // No delete from the tree that does not delete the children as well. Seems the best way to do this for now is just to make the tree up again.
				bNeedToRebuildTree = true;
	        }

	        SetTypeAsModified( uBlueprintTypeHashID );
	    }
#ifdef _DEBUG
	    else
	    {
	        // The selected item is a blueprint type. We're not allowing deletion of types any more. (The file can simply be deleted instead.)
	    }
#endif // _DEBUG
	}

	if(bNeedToRebuildTree)
	{
		s_pxThisBlueprintMode->s_xTree.DeleteAll();
	}

	s_pxThisBlueprintMode->DestroyOverlay();
    s_pxThisBlueprintMode->CreateOverlay();
	s_pxThisBlueprintMode->RequestRefreshBlueprintEditObjectOverlays();

	Tools_SaveAsura::SetBlueprintsModified();
	BlueprintChanged();
}

void Tools_Mode_Blueprint::TreeDoubleClicked()
{
    if ( TreeSelectionIsBlueprint() )
    {
        // Only automatically edit this blueprint if it has no children, otherwise we conflict with the automatic expansion of tree nodes.
        Asura_TreeDialog_Item* pxCurrentItem = s_pxThisBlueprintMode->s_xTree.GetSelectedItem();
        if ( pxCurrentItem && pxCurrentItem->GetUserData() )
        {
            // If the current item has user data then it is a blueprint, the user data being the blueprint type hash.
            const Asura_Hash_ID uTypeHashID      = pxCurrentItem->GetUserData();
            const Asura_Hash_ID uBlueprintHashID = Asura_GetHashID( pxCurrentItem->GetName() );

            if ( Asura_Blueprint_System::GetNumChildBlueprints( uTypeHashID, uBlueprintHashID, true ) == 0 )
            {
                EditSelectedBlueprint();
            }
        }
    }
}

void Tools_Mode_Blueprint::VerifyBlueprintInformation()
{
    Tools_MessageLog::DeleteMessages( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ] );
    Tools_MessageLog::DeleteMessages( s_pszLogCategory, s_pszLogSubCategory[ LOG_DEBUG ] );

	s_pxThisBlueprintMode->DoVerificationTests();

    Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Finished verifying blueprint information." );
    Tools_MessageLog::ShowOverlay( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ] );
    RequestRefreshBlueprintEditObjectOverlays();
}

void Tools_Mode_Blueprint::DoVerificationTests()
{
	Tools_ProgressIndicator xProgressIndicator( "Verifying blueprints" );

#ifdef ASURA_USE_DECAL_MANAGER
	TestForOrphanedDecals( xProgressIndicator );
#endif
#ifdef ASURA_USE_FSFX
	TestForOrphanedFSFX( xProgressIndicator );
#endif
	TestForOphanedPFX( xProgressIndicator );
	TestForOrphanedRagdollDescriptions( xProgressIndicator );
	TestForOphanedSkinsAndAnims( xProgressIndicator );
	TestForOphanedSoundEvents( xProgressIndicator );
	TestForOrphanedTextures( xProgressIndicator );
	TestForOrphanedTextureTiles( xProgressIndicator );
	TestForOrphanedBlueprints( xProgressIndicator );
	TestForOrphanedBlueprintTypes( xProgressIndicator );

	TestForAndConvertOrphanedOverrides( xProgressIndicator );

	// A bug introduced some overrides with group names that differ from their parent parameter. These need fixing up.
	CorrectBadGroupNames( xProgressIndicator );

	//// Check that a parameter only exists once in each blueprint.
	//TestForAndDeleteDuplicateParameters();

	//// Check all overrides have a correct override type (!ABOT_NONE).
	//TestForAndConvertDoubleParameters();

	//// Check all overrides have the same value type and template hash ID as the root parameter.
	//TestForAndConvertBadOverrides();

	//OutputBlueprintDataStats();

	xProgressIndicator.Finish();
}

void Tools_Mode_Blueprint::ExportAllBlueprints()
{
	Tools_Blueprint_XMLExporter xBptExporter( false, "logfiles/blueprints", "<?xml-stylesheet type='text/xsl' href='../../misc/blueprint_styles.xsl'?>" );
	xBptExporter.StartXMLExport();
		
	Message_Overlay xMessageOverlay("Blueprints exported to logfiles/blueprints!");
	xMessageOverlay.DoModal();
}

void Tools_Mode_Blueprint::ExportBlueprintNames()
{
	Asura_String xFileName = "logfiles\\blueprints\\AllBlueprintsExported.txt";
	Tools_FileSystem::EnsureDirectoryExists(xFileName.GetString());
		
	if( s_xTree.ExportToTXTFile( xFileName.GetString() ) )
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_SUCCESS, "Exported to file %s", xFileName.GetString() );

		ShellExecute( NULL, "edit", xFileName.GetString(),  NULL, NULL, SW_SHOWDEFAULT );
	}
	else
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "Failed to save %s", xFileName.GetString() );
	}
}

void Tools_Mode_Blueprint::ExportAllBlueprintsRaw()
{
	Tools_Blueprint_XMLExporter xBptExporter( true, "logfiles/blueprints", "<?xml-stylesheet type='text/xsl' href='../../misc/blueprint_styles.xsl'?>" );
	xBptExporter.StartXMLExport();
		
	Message_Overlay xMessageOverlay("Blueprints exported to logfiles/blueprints!");
	xMessageOverlay.DoModal();
}

class Tools_Modal_Overlay_SearchOptions : public Tools_Modal_Overlay
{
public:
	Tools_Modal_Overlay_SearchOptions( const char* pszTitleString )
	: Tools_Modal_Overlay( pszTitleString )
	, m_bReturnValue( false )
	, m_xSearchString()
	, m_bCheckBlueprintNames( true )
	, m_bCheckParamNames( true )
	, m_bCheckParamValues( false )
	, m_bMatchCase( false )
	, m_bMatchWholeWord( false )
	, m_bIncludeParamOverrides( true )
	{
	}

	virtual bool DoModal()
	{
		ShowOverlay();
		return m_bReturnValue;
	}

	bool Matches( const Asura_String& xString );

	const Asura_String& GetSearchString() const { return m_xSearchString; }
    bool ShouldCheckBlueprintNames()      const { return m_bCheckBlueprintNames; }
    bool ShouldCheckParamNames()	      const { return m_bCheckParamNames; }
    bool ShouldCheckParamValues()	      const { return m_bCheckParamValues; }
    bool ShouldMatchCase()	              const { return m_bMatchCase; }
    bool ShouldMatchWholeWord()	          const { return m_bMatchWholeWord; }
	bool ShouldIncludeParamOverrides()    const { return m_bIncludeParamOverrides; }

protected:
	virtual void InitialiseOverlay( Asura_Overlay* pxOverlay );

private:
	static void OkayPressed( int iUserData )
	{
		Tools_Modal_Overlay_SearchOptions* pxThis = reinterpret_cast< Tools_Modal_Overlay_SearchOptions* >( iUserData );
		pxThis->m_bReturnValue = true;
	}
	static void CancelPressed( int iUserData )
	{
		Tools_Modal_Overlay_SearchOptions* pxThis = reinterpret_cast< Tools_Modal_Overlay_SearchOptions* >( iUserData );
		pxThis->m_bReturnValue = false;
	}

private:
    bool		 m_bReturnValue;
	Asura_String m_xSearchString;
	bool		 m_bCheckBlueprintNames;
	bool		 m_bCheckParamNames;
	bool		 m_bCheckParamValues;
	bool		 m_bMatchCase;
	bool		 m_bMatchWholeWord;
	bool		 m_bIncludeParamOverrides;
};

bool Tools_Modal_Overlay_SearchOptions::Matches( const Asura_String& xString )
{
	if ( xString.Compare( m_xSearchString, !m_bMatchCase ) == 0 )
	{
		return true;
	}
	if ( !m_bMatchWholeWord )
	{
		if ( m_bMatchCase )
		{
			return xString.Contains( m_xSearchString.GetString() );
		}
		else
		{
			Asura_String xLowercaseString = xString;
			xLowercaseString.MakeLowercase();
			Asura_String xLowercaseSearchString = m_xSearchString;
			xLowercaseSearchString.MakeLowercase();
			return xLowercaseString.Contains( xLowercaseSearchString.GetString() );
		}
	}
	return false;
}

void Tools_Modal_Overlay_SearchOptions::InitialiseOverlay( Asura_Overlay* pxOverlay )
{
    m_bReturnValue			   = false;
	m_xSearchString.MakeBlank();
    m_bCheckBlueprintNames     = true;
    m_bCheckParamNames         = true;
    m_bCheckParamValues        = false;
    m_bMatchCase	           = false;
    m_bMatchWholeWord	       = false;
	m_bIncludeParamOverrides   = true;

	Asura_Vector_2 xOffset( 1.0f, 3.0f );
	Asura_Dialog* pxDialogue = new Asura_Dialog( "Enter a string or a hash ID to search for:", xOffset );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_String_Dialog* pxStringDialogue = new Asura_String_Dialog( "", xOffset, &m_xSearchString, 64 );
	pxStringDialogue->SetDefaultFocus( true );
	pxOverlay->AddDialog( pxStringDialogue );
	xOffset.y += Asura_Dialog::LargeSpacing();
	
	pxDialogue = new Asura_Tickbox_Dialog( "Search Blueprint Names", xOffset, &m_bCheckBlueprintNames );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialogue = new Asura_Tickbox_Dialog( "Search Parameter Names", xOffset, &m_bCheckParamNames );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialogue = new Asura_Tickbox_Dialog( "Search Parameter Values", xOffset, &m_bCheckParamValues );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialogue = new Asura_Tickbox_Dialog( "Match Case", xOffset, &m_bMatchCase );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialogue = new Asura_Tickbox_Dialog( "Match Whole Word", xOffset, &m_bMatchWholeWord );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialogue = new Asura_Tickbox_Dialog( "Include Parameter Overrides", xOffset, &m_bIncludeParamOverrides );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialogue = new Asura_Function_Dialog( "Okay", xOffset, OkayPressed, reinterpret_cast< int >( this ), &pxOverlay->m_bShouldClose );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x += 7.0f;

	pxDialogue = new Asura_Function_Dialog( "Cancel", xOffset, CancelPressed, reinterpret_cast< int >( this ), &pxOverlay->m_bShouldClose );
	pxOverlay->AddDialog( pxDialogue );
}

void Tools_Mode_Blueprint::SearchBlueprints()
{
	Tools_Modal_Overlay_SearchOptions xSearchOptions( "Search Blueprints" );
	if ( !xSearchOptions.DoModal() ) { return; }

	Asura_String xSearchString = xSearchOptions.GetSearchString();
	if ( xSearchString.IsBlank() ) { return; }

	if ( !xSearchOptions.ShouldCheckBlueprintNames() && !xSearchOptions.ShouldCheckParamNames() && !xSearchOptions.ShouldCheckParamValues() ) { return; }

	// Determine whether what we got was a hash or a name.
	Asura_Hash_ID uSearchHashID = ASURA_HASH_ID_UNSET;
    const bool	  bUseHashID    = Asura_StringUtil::GetIntValue( xSearchString.GetString(), ( int* )&uSearchHashID );

	Asura_String xLogHeaderString( "Searching for " );
	if ( bUseHashID )
	{
		xLogHeaderString += "hash ID '";
		xLogHeaderString += xSearchString;
		xLogHeaderString += "'";
	}
	else
	{
		xLogHeaderString += "string '";
		xLogHeaderString += xSearchString;
		xLogHeaderString += "'";
	}
	xLogHeaderString += " in";
	if ( xSearchOptions.ShouldCheckBlueprintNames() )
	{
		xLogHeaderString += " - blueprint names";
	}
	if ( xSearchOptions.ShouldCheckParamNames() )
	{
		xLogHeaderString += " - parameter names";
	}
	if ( xSearchOptions.ShouldCheckParamValues() )
	{
		xLogHeaderString += " - parameter values";
	}
	xLogHeaderString += ":";

	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_SEARCH ], xLogHeaderString.GetString() );

	if ( xSearchOptions.ShouldCheckBlueprintNames() )
	{
		for ( Asura_Blueprint_ConstIterator xBPIter; !xBPIter.Done(); xBPIter.Next() )
		{
			const Asura_Blueprint* pxBlueprint = xBPIter.GetCurrent();
			const bool bFound = bUseHashID ? ( pxBlueprint->GetHashID() == uSearchHashID ) : xSearchOptions.Matches( pxBlueprint->GetName() );
			if ( bFound )
			{
				const Asura_Hash_ID uTypeHashID = xBPIter.GetCurrentBlueprintTypeHash();
				Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_SEARCH ],
					new Tools_MessageLogAction_Blueprint( uTypeHashID, pxBlueprint->GetHashID() ),
					"Found blueprint '%s' in type '%s'.", pxBlueprint->GetName().GetString(), *Asura_Blueprint_System::GetTypeName( uTypeHashID ) );
			}
		}
	}

	if ( xSearchOptions.ShouldCheckParamNames() )
	{
		for ( Asura_Blueprint_Parameter_ConstIterator xBPParamIter; !xBPParamIter.Done(); xBPParamIter.Next() )
		{
			const Asura_Parameter* pxParam = xBPParamIter.GetCurrent();
			if ( !xSearchOptions.ShouldIncludeParamOverrides() && pxParam->GetOverrideType() != ABOT_NONE )
			{
				continue;
			}

			const bool bFound = bUseHashID ? ( pxParam->GetHashID() == uSearchHashID ) : xSearchOptions.Matches( pxParam->GetName() );
			if ( bFound )
			{
				const Asura_Hash_ID uTypeHashID = xBPParamIter.GetCurrentBlueprintTypeHash();
				const Asura_Hash_ID uBPHashID   = xBPParamIter.GetCurrentBlueprintHash();
				const Asura_Blueprint* pxBP     = xBPParamIter.GetCurrentBlueprint();

				Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_SEARCH ],
					new Tools_MessageLogAction_Blueprint( uTypeHashID, uBPHashID ),
					"Found parameter '%s' in blueprint '%s', type '%s'.",
					pxParam->GetName().GetString(), pxBP->GetName().GetString(), *Asura_Blueprint_System::GetTypeName( uTypeHashID ) );
			}
		}
	}

	if ( xSearchOptions.ShouldCheckParamValues() )
	{
		for ( Asura_Blueprint_Parameter_ConstIterator xBPParamIter; !xBPParamIter.Done(); xBPParamIter.Next() )
		{
			const Asura_Parameter* pxParam = xBPParamIter.GetCurrent();
			if ( !xSearchOptions.ShouldIncludeParamOverrides() && pxParam->GetOverrideType() != ABOT_NONE )
			{
				continue;
			}
			if ( pxParam->GetValue().GetValueType() != Asura_Value::VT_HASH )
			{
				continue;
			}

			const bool bFound = bUseHashID ?
				( pxParam->GetValue().GetHashIDValue() == uSearchHashID ) :
			xSearchOptions.Matches( pxParam->GetValue().GetHashStringValue() );
			if ( bFound )
			{
				const Asura_Hash_ID uTypeHashID = xBPParamIter.GetCurrentBlueprintTypeHash();
				const Asura_Hash_ID uBPHashID   = xBPParamIter.GetCurrentBlueprintHash();
				const Asura_Blueprint* pxBP     = xBPParamIter.GetCurrentBlueprint();

				Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_SEARCH ],
					new Tools_MessageLogAction_Blueprint( uTypeHashID, uBPHashID ),
					"Found value '%s' in parameter '%s', blueprint '%s', type '%s'.",
					pxParam->GetValue().GetHashStringValue(), pxParam->GetName().GetString(),
					pxBP->GetName().GetString(), *Asura_Blueprint_System::GetTypeName( uTypeHashID ) );
			}
		}
	}

	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_SEARCH	], "Search complete." );
	Tools_MessageLog::ShowOverlay( s_pszLogCategory, s_pszLogSubCategory[ LOG_SEARCH ] );
}

void Tools_Mode_Blueprint::RefreshBlueprintEditObjectOverlays()
{
    // First, remember which overlay is on top.
    const u_int uTopOverlayID = Asura_Overlay_System::GetTopOverlayID();

    // Then refresh the overlays from the first to the last.
    Asura_List_ReverseIt< Tools_Blueprint_Edit_Object > xIt( Tools_Blueprint_Edit_Object::GetList() );
    while ( !xIt.Done() )
    {
        Tools_Blueprint_Edit_Object* pxEditObject = xIt.GetCurrent();
        if ( pxEditObject )
        {
            if ( Asura_Blueprint_System::GetBlueprintByHashID( pxEditObject->GetBlueprintTypeHashID(), pxEditObject->GetBlueprintHashID() ) )
            {
				Tools_BEO_Display_Data xDisplayData;
                pxEditObject->RecordOverlayState( xDisplayData );
                pxEditObject->DestroyOverlay();
                pxEditObject->RestoreOverlayState( xDisplayData );
                xIt.Next();
            }
            else
            {
                delete pxEditObject;
                xIt.Restart();
            }
        }
    }

    // Then put the correct overlay back on top.
    Asura_Overlay_System::BringOverlayToTop( uTopOverlayID );
}

bool Tools_Mode_Blueprint::IsValidBlueprintName(const char* szOriginalName, const Asura_Hash_ID uTypeHashID, const char* szNewName)
{
	if(!szNewName) return false;

	if ( strchr(szNewName,'.') )
	{
		ShowMessageOverlay( "Blueprint names should not contain dot '.' characters" );
		return false;
	}

	const Asura_Hash_ID uNewNameHashID = Asura_GetHashID( szNewName );
	if ( uNewNameHashID == ASURA_HASH_ID_UNSET )
	{
		ShowMessageOverlay( "I think you might have forgotten something..." );
		return false;
	}

	if ( szOriginalName && !stricmp(szOriginalName, szNewName) )
	{
		//name not changed
		return true;
	}

	if ( Asura_Blueprint_System::GetBlueprintByHashID( uTypeHashID, uNewNameHashID ) )
	{
		ShowMessageOverlay( "This name is already in use. Please choose another." );
		return false;
	}

	return true;
}

bool Tools_Mode_Blueprint::GetUniqueBlueprintName( const Asura_String& xOverlayTitle, const Asura_Hash_ID uTypeHashID, Asura_String& xName )
{
    GetString_Overlay  xGetStringOverlay( xOverlayTitle, xName.GetString(), 256 );
	const Asura_String xOriginalName( xName );
    bool bNameOK = true;
    do 
    {
        if ( !xGetStringOverlay.DoModal() ) { return false; }

        bNameOK = IsValidBlueprintName(xOriginalName.GetString(), uTypeHashID, xGetStringOverlay.GetString());

    } while ( !bNameOK );

    xName = xGetStringOverlay.GetString();
    return true;
}

bool Tools_Mode_Blueprint::IsValidBlueprintTypeName( const char* szNewName)
{
	if(!szNewName) return false;

	if ( strchr(szNewName,'.') )
	{
		ShowMessageOverlay( "Blueprint type names should not contain dot '.' characters" );
		return false;
	}

	const Asura_Hash_ID uNewNameHashID = Asura_GetHashID( szNewName );
	if ( uNewNameHashID == ASURA_HASH_ID_UNSET )
	{
		ShowMessageOverlay( "I think you might have forgotten something..." );
		return false;
	}

	for ( Asura_Blueprint_Type_ConstIterator xIterator; !xIterator.Done(); xIterator.Next() )
	{
	    if ( xIterator.GetCurrentHash() == uNewNameHashID )
	    {
	        ShowMessageOverlay( "This name is already in use. Please choose another." );
			return false;
	    }
	}

	return true;
}
bool Tools_Mode_Blueprint::GetUniqueBlueprintTypeName( const Asura_String& xOverlayTitle, Asura_String& xName )
{
    GetString_Overlay  xGetStringOverlay( xOverlayTitle, xName.GetString(), 256 );
	const Asura_String xOriginalName( xName );
    bool bNameOK = true;
    do 
    {
        if ( !xGetStringOverlay.DoModal() ) { return false; }
        bNameOK = IsValidBlueprintTypeName(xGetStringOverlay.GetString());

    } while ( !bNameOK );

    xName = xGetStringOverlay.GetString();
    return true;
}

bool Tools_Mode_Blueprint::GetUniqueBlueprintTypeFileName( const Asura_String& xOverlayTitle, Asura_String& xName )
{
	// ASURA_MAX_FILENAME_LENGTH is hidden!
	GetString_Overlay  xGetStringOverlay( xOverlayTitle, xName.GetString(), 256 );
	const Asura_String xOriginalName( xName );
	bool bNameOK = true;
	do 
	{
		bNameOK = true;
		if ( !xGetStringOverlay.DoModal() ) { return false; }

		const Asura_String xChosenFileName( xGetStringOverlay.GetString() );
		if ( xChosenFileName.IsBlank() )
		{
			Asura_String xMessage( "I think you might have forgotten something..." );
			Message_Overlay xMessageOverlay( xMessage );
			xMessageOverlay.DoModal();
			bNameOK = false;
			continue;
		}

		if ( xOriginalName == xGetStringOverlay.GetString() )
		{
			// Handle this like a cancel - no further work to do.
			return false;
		}

		for ( Asura_Blueprint_Type_ConstIterator xIterator; !xIterator.Done(); xIterator.Next() )
		{
			const Asura_BlueprintType* pxBlueprintType = xIterator.GetCurrent();
			if ( pxBlueprintType->m_xFileName == xChosenFileName )
			{
				Asura_String xMessage( "This name is already in use. Please choose another." );
				Message_Overlay xMessageOverlay( xMessage );
				xMessageOverlay.DoModal();
				bNameOK = false;
				break;
			}
		}
	} while ( !bNameOK );

	xName = xGetStringOverlay.GetString();
	return true;
}

void Tools_Mode_Blueprint::ReportOrphanedFromBlueprint( const char* szTemplatedTypeName, const char* szValueString,
														const Asura_Blueprint* pxBlueprint,
														const Asura_Hash_ID uBlueprintTypeHashID,
														const Asura_Hash_ID uParamHashID /*= ASURA_HASH_ID_UNSET*/ )
{
	Tools_MessageLogAction*	pxAction = new Tools_MessageLogAction_Blueprint( uBlueprintTypeHashID, pxBlueprint->GetHashID(), uParamHashID );
	Tools_MessageLog::AddMessage( s_pszLogCategory,
								  s_pszLogSubCategory[ LOG_VERIFICATION ],
								  pxAction,
								  "Blueprint '%s'.'%s' references %s '%s' that doesn't exist.",
								  Asura_Blueprint_System::GetTypeName( uBlueprintTypeHashID )->GetString(),
								  pxBlueprint->GetName().GetString(),
								  szTemplatedTypeName,
								  szValueString );
}

void Tools_Mode_Blueprint::TestForOphanedFiles(const char* szType, const char* szFilenamePrefix, const char* szFilenameSuffix, Tools_ProgressIndicator& xProgressIndicator)
{
	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Testing for orphaned %s files.", szType );
	xProgressIndicator.AddMessage( "Testing for orphaned %s files.", szType );

	// Work out how many blueprints there are.
	u_int uNumBPs = 0;
	for ( Asura_Blueprint_Type_ConstIterator xTypeIter; !xTypeIter.Done(); xTypeIter.Next() )
	{
		uNumBPs += Asura_Blueprint_System::GetNumBlueprintsInType( xTypeIter.GetCurrentHash() );
	}

	// Check all of the parameters.
	const Asura_Hash_ID						uParameterTemplateHash = Asura_GetHashID( szType );
	Asura_Blueprint_Parameter_ConstIterator	xIterator( ASURA_HASH_ID_UNSET, ASURA_HASH_ID_UNSET );
	Asura_Hash_ID uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
	u_int uBPIndex = 0;
	while ( !xIterator.Done() )
	{
		if ( uCurrentBPHash != xIterator.GetCurrentBlueprintHash() )
		{
			++uBPIndex;
			uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
		}
		xProgressIndicator.Update( uBPIndex, uNumBPs );

		const Asura_Parameter* pxParameter = xIterator.GetCurrent();
		if ( pxParameter && pxParameter->GetTemplate() == uParameterTemplateHash )
		{
			const Asura_Value& xValue = pxParameter->GetValue();
			if ( xValue.GetHashIDValue() != ASURA_HASH_ID_UNSET )
			{
				// See if we can find the file referenced.
				Asura_String xFilename;
				if ( szFilenamePrefix != NULL )
				{
					xFilename += szFilenamePrefix;
				}

				xFilename += pxParameter->GetValue().GetHashStringValue();

				if ( szFilenameSuffix != NULL )
				{
					xFilename += szFilenameSuffix;
				}

				if ( !Tools_FileSystem::FileExists_CheckNetwork( xFilename.GetString() ) )
				{
					ReportOrphanedFromBlueprint( szType, xFilename.GetString(),
						xIterator.GetCurrentBlueprint(), xIterator.GetCurrentBlueprintTypeHash(), pxParameter->GetHashID() );
				}
			}
		}

		xIterator.Next();
	}
}

#ifdef ASURA_USE_DECAL_MANAGER
void Tools_Mode_Blueprint::TestForOrphanedDecals( Tools_ProgressIndicator& xProgressIndicator )
{
	// Say what we're doing...
	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Testing for orphaned decals." );
	xProgressIndicator.AddMessage( "Testing for orphaned decals." );

	// Work out how many blueprints there are.
	u_int uNumBPs = 0;
	for ( Asura_Blueprint_Type_ConstIterator xTypeIter; !xTypeIter.Done(); xTypeIter.Next() )
	{
		uNumBPs += Asura_Blueprint_System::GetNumBlueprintsInType( xTypeIter.GetCurrentHash() );
	}

	// Check all of the parameters...
	const char*								szTemplateString = "Decal";
	Asura_Hash_ID							uParameterTemplateHash = Asura_GetHashID( szTemplateString );
	Asura_Blueprint_Parameter_ConstIterator	xIterator(ASURA_HASH_ID_UNSET, ASURA_HASH_ID_UNSET);
	Asura_Hash_ID uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
	u_int uBPIndex = 0;
	while(!xIterator.Done())
	{
		if ( uCurrentBPHash != xIterator.GetCurrentBlueprintHash() )
		{
			++uBPIndex;
			uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
		}
		xProgressIndicator.Update( uBPIndex, uNumBPs );

		const Asura_Parameter*	pxParameter	= xIterator.GetCurrent();

		if(pxParameter && pxParameter->GetTemplate() == uParameterTemplateHash)
		{
			const Asura_Value&	xValue	= pxParameter->GetValue();
			const Asura_Hash_ID	uHash	= xValue.GetHashIDValue();

			if(uHash != ASURA_HASH_ID_UNSET)
			{
				const Asura_DecalManager_DecalSet*	pxDecalSet	= Asura_Decal_Manager::GetDecalSet(uHash);

				if(pxDecalSet == NULL)
				{
					ReportOrphanedFromBlueprint( szTemplateString, xValue.GetHashStringValue(),
						xIterator.GetCurrentBlueprint(), xIterator.GetCurrentBlueprintTypeHash(), pxParameter->GetHashID() );
				}
			}
		}
		xIterator.Next();
	}
}
#endif

#ifdef ASURA_USE_FSFX
void Tools_Mode_Blueprint::TestForOrphanedFSFX( Tools_ProgressIndicator& xProgressIndicator )
{
	TestForOphanedFiles("FSFX", "FSFX\\", ".fsx", xProgressIndicator);
}
#endif

void Tools_Mode_Blueprint::TestForOphanedPFX( Tools_ProgressIndicator& xProgressIndicator )
{
	TestForOphanedFiles("PFX", "PFXTemps\\", ".pfx", xProgressIndicator);
}

void Tools_Mode_Blueprint::TestForOrphanedRagdollDescriptions( Tools_ProgressIndicator& xProgressIndicator )
{
	TestForOphanedFiles("Ragdoll Description", "Ragdolls", ".ragdoll", xProgressIndicator);
}

void Tools_Mode_Blueprint::TestForOphanedSkinsAndAnims( Tools_ProgressIndicator& xProgressIndicator )
{
	TestForOphanedFiles("Skin/Anim", NULL, NULL, xProgressIndicator);
}

void Tools_Mode_Blueprint::TestForOphanedSoundEvents( Tools_ProgressIndicator& xProgressIndicator )
{
	// Say what we're doing...
	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Testing for orphaned sound events." );
	xProgressIndicator.AddMessage( "Testing for orphaned sound events." );

	// Work out how many blueprints there are.
	u_int uNumBPs = 0;
	for ( Asura_Blueprint_Type_ConstIterator xTypeIter; !xTypeIter.Done(); xTypeIter.Next() )
	{
		uNumBPs += Asura_Blueprint_System::GetNumBlueprintsInType( xTypeIter.GetCurrentHash() );
	}

	// Check all of the parameters...
	const char*								szTemplateString = "Sound Event";
	Asura_Hash_ID							uParameterTemplateHash = Asura_GetHashID( szTemplateString );
	Asura_Blueprint_Parameter_ConstIterator	xIterator(ASURA_HASH_ID_UNSET, ASURA_HASH_ID_UNSET);
	Asura_Hash_ID uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
	u_int uBPIndex = 0;
	while(!xIterator.Done())
	{
		if ( uCurrentBPHash != xIterator.GetCurrentBlueprintHash() )
		{
			++uBPIndex;
			uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
		}
		xProgressIndicator.Update( uBPIndex, uNumBPs );

		const Asura_Parameter*	pxParameter	= xIterator.GetCurrent();

		if(pxParameter && pxParameter->GetTemplate() == uParameterTemplateHash)
		{
			const Asura_Value&	xValue	= pxParameter->GetValue();
			const Asura_Hash_ID	uHash	= xValue.GetHashIDValue();

			if(uHash != ASURA_HASH_ID_UNSET)
			{
				const Asura_Sound_Event*	pxEvent	= Asura_Sound_Event_Storage::GetEventByHash(uHash);

				if(pxEvent == NULL)
				{
					ReportOrphanedFromBlueprint( szTemplateString, xValue.GetHashStringValue(),
						xIterator.GetCurrentBlueprint(), xIterator.GetCurrentBlueprintTypeHash(), pxParameter->GetHashID() );
				}
			}
		}
		xIterator.Next();
	}
}

void Tools_Mode_Blueprint::TestForOrphanedTextures( Tools_ProgressIndicator& xProgressIndicator )
{
	TestForOphanedFiles("Texture", "Graphics\\", NULL, xProgressIndicator);
}

void Tools_Mode_Blueprint::TestForOrphanedTextureTiles( Tools_ProgressIndicator& xProgressIndicator )
{
	// Say what we're doing...
	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Testing for orphaned texture tiles." );
	xProgressIndicator.AddMessage( "Testing for orphaned texture tiles." );

	// Work out how many blueprints there are.
	u_int uNumBPs = 0;
	for ( Asura_Blueprint_Type_ConstIterator xTypeIter; !xTypeIter.Done(); xTypeIter.Next() )
	{
		uNumBPs += Asura_Blueprint_System::GetNumBlueprintsInType( xTypeIter.GetCurrentHash() );
	}

	// Check all of the parameters...
	const char*								szTemplateString = "Texture Tile";
	Asura_Hash_ID							uParameterTemplateHash = Asura_GetHashID( szTemplateString );
	Asura_Blueprint_Parameter_ConstIterator	xIterator(ASURA_HASH_ID_UNSET, ASURA_HASH_ID_UNSET);
	Asura_Hash_ID uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
	u_int uBPIndex = 0;
	while(!xIterator.Done())
	{
		if ( uCurrentBPHash != xIterator.GetCurrentBlueprintHash() )
		{
			++uBPIndex;
			uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
		}
		xProgressIndicator.Update( uBPIndex, uNumBPs );

		const Asura_Parameter*	pxParameter	= xIterator.GetCurrent();

		if(pxParameter && pxParameter->GetTemplate() == uParameterTemplateHash)
		{
			const Asura_Value&	xValue	= pxParameter->GetValue();
			const Asura_Hash_ID	uHash	= xValue.GetHashIDValue();

			if(uHash != ASURA_HASH_ID_UNSET)
			{
				const Asura_Handle	iTileHandle	= Asura_TiledTextureManagement::GetHandleOfTile(uHash);

				if(iTileHandle == ASURA_INVALID_HANDLE)
				{
					ReportOrphanedFromBlueprint( szTemplateString, xValue.GetHashStringValue(),
						xIterator.GetCurrentBlueprint(), xIterator.GetCurrentBlueprintTypeHash(), pxParameter->GetHashID() );
				}
			}
		}
		xIterator.Next();
	}
}

void Tools_Mode_Blueprint::TestForOrphanedBlueprints( Tools_ProgressIndicator& xProgressIndicator )
{
	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Testing for orphaned blueprint references." );
	xProgressIndicator.AddMessage( "Testing for orphaned blueprint references." );

	// Work out how many blueprints there are.
	u_int uNumBPs = 0;
	for ( Asura_Blueprint_Type_ConstIterator xTypeIter; !xTypeIter.Done(); xTypeIter.Next() )
	{
		uNumBPs += Asura_Blueprint_System::GetNumBlueprintsInType( xTypeIter.GetCurrentHash() );
	}

	Asura_Hash_ID uParameterTemplateHash = Asura_GetHashID( "Blueprint" );
	Asura_Hash_ID uParameterTemplateHash2 = Asura_GetHashID( "Blueprint and children" );
	Asura_Blueprint_Parameter_ConstIterator	xIterator( ASURA_HASH_ID_UNSET, ASURA_HASH_ID_UNSET );
	Asura_Hash_ID uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
	u_int uBPIndex = 0;
	while ( !xIterator.Done() )
	{
		if ( uCurrentBPHash != xIterator.GetCurrentBlueprintHash() )
		{
			++uBPIndex;
			uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
		}
		xProgressIndicator.Update( uBPIndex, uNumBPs );

		const Asura_Parameter* pxParameter = xIterator.GetCurrent();
		if ( !pxParameter )
		{
		}
		else if ( pxParameter->GetTemplate() == uParameterTemplateHash || pxParameter->GetTemplate() == uParameterTemplateHash2 )
		{
			const Asura_Value& xValue = pxParameter->GetValue();

			Asura_String xTypeName = xValue.GetHashStringValue();
			while ( xTypeName.StripExtension() ) {}

			const Asura_Hash_ID uBPHashID   = xValue.GetHashIDValue();
			const Asura_Hash_ID uTypeHashID = xTypeName.GetHash();

			if ( uBPHashID != ASURA_HASH_ID_UNSET && uTypeHashID != ASURA_HASH_ID_UNSET )
			{
				if ( !Asura_Blueprint_System::GetBlueprintByHashID( uTypeHashID, uBPHashID ) )
				{
					if ( pxParameter->GetTemplate() == uParameterTemplateHash )
					{
						ReportOrphanedFromBlueprint( "Blueprint", xValue.GetHashStringValue(),
							xIterator.GetCurrentBlueprint(), xIterator.GetCurrentBlueprintTypeHash(), pxParameter->GetHashID() );
					}
					else
					{
						ReportOrphanedFromBlueprint( "Blueprint and children", xValue.GetHashStringValue(),
							xIterator.GetCurrentBlueprint(), xIterator.GetCurrentBlueprintTypeHash(), pxParameter->GetHashID() );
					}
				}
			}
		}

		xIterator.Next();
	}
}

void Tools_Mode_Blueprint::TestForOrphanedBlueprintTypes( Tools_ProgressIndicator& xProgressIndicator )
{
	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Testing for orphaned blueprint type references." );
	xProgressIndicator.AddMessage( "Testing for orphaned blueprint type references." );

	// Work out how many blueprints there are.
	u_int uNumBPs = 0;
	for ( Asura_Blueprint_Type_ConstIterator xTypeIter; !xTypeIter.Done(); xTypeIter.Next() )
	{
		uNumBPs += Asura_Blueprint_System::GetNumBlueprintsInType( xTypeIter.GetCurrentHash() );
	}

	Asura_Hash_ID uParameterTemplateHash = Asura_GetHashID( "Blueprint Type" );
	Asura_Blueprint_Parameter_ConstIterator	xIterator( ASURA_HASH_ID_UNSET, ASURA_HASH_ID_UNSET );
	Asura_Hash_ID uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
	u_int uBPIndex = 0;
	while ( !xIterator.Done() )
	{
		if ( uCurrentBPHash != xIterator.GetCurrentBlueprintHash() )
		{
			++uBPIndex;
			uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
		}
		xProgressIndicator.Update( uBPIndex, uNumBPs );

		const Asura_Parameter* pxParameter = xIterator.GetCurrent();
		if ( !pxParameter )
		{
		}
		else if ( pxParameter->GetTemplate() == uParameterTemplateHash )
		{
			const Asura_Value& xValue = pxParameter->GetValue();

			const Asura_Hash_ID uTypeHashID   = xValue.GetHashIDValue();

			if ( uTypeHashID != ASURA_HASH_ID_UNSET )
			{
				if ( !Asura_Blueprint_System::GetType( uTypeHashID ) )
				{
					ReportOrphanedFromBlueprint( "Blueprint Type", xValue.GetHashStringValue(),
						xIterator.GetCurrentBlueprint(), xIterator.GetCurrentBlueprintTypeHash(), pxParameter->GetHashID() );
				}
			}
		}

		xIterator.Next();
	}
}

void Tools_Mode_Blueprint::TestForOrphanedLocalisedText( Tools_ProgressIndicator& xProgressIndicator )
{
	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Testing for orphaned localised text." );
	xProgressIndicator.AddMessage( "Testing for orphaned localised text." );

	// Work out how many blueprints there are.
	u_int uNumBPs = 0;
	for ( Asura_Blueprint_Type_ConstIterator xTypeIter; !xTypeIter.Done(); xTypeIter.Next() )
	{
		uNumBPs += Asura_Blueprint_System::GetNumBlueprintsInType( xTypeIter.GetCurrentHash() );
	}

	Asura_Hash_ID uParameterTemplateHash = Asura_GetHashID( "Localised Text" );
	Asura_Blueprint_Parameter_ConstIterator	xIterator( ASURA_HASH_ID_UNSET, ASURA_HASH_ID_UNSET );
	Asura_Hash_ID uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
	u_int uBPIndex = 0;
	while ( !xIterator.Done() )
	{
		if ( uCurrentBPHash != xIterator.GetCurrentBlueprintHash() )
		{
			++uBPIndex;
			uCurrentBPHash = xIterator.GetCurrentBlueprintHash();
		}
		xProgressIndicator.Update( uBPIndex, uNumBPs );

		const Asura_Parameter* pxParameter = xIterator.GetCurrent();
		if ( !pxParameter )
		{
		}
		else if ( pxParameter->GetTemplate() == uParameterTemplateHash )
		{
			const Asura_Value& xValue = pxParameter->GetValue();

			const Asura_UnicodeChar* pwszTextString = Asura_LocalisedText::GetString( xValue.GetIntValue() );
			if ( !pwszTextString )
			{
				ReportOrphanedFromBlueprint( "Localised Text", xValue.GetHashStringValue(),
					xIterator.GetCurrentBlueprint(), xIterator.GetCurrentBlueprintTypeHash(), pxParameter->GetHashID() );
			}
		}

		xIterator.Next();
	}
}

void Tools_Mode_Blueprint::TestForAndConvertOrphanedOverrides( Tools_ProgressIndicator& xProgressIndicator )
{
    Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Testing for orphaned overrides (overrides without a parent parameter)." );
	xProgressIndicator.AddMessage( "Testing for orphaned blueprint references." );

	// Work out how many blueprints there are.
	u_int uNumBPs = 0;
	for ( Asura_Blueprint_Type_ConstIterator xTypeIter; !xTypeIter.Done(); xTypeIter.Next() )
	{
		uNumBPs += Asura_Blueprint_System::GetNumBlueprintsInType( xTypeIter.GetCurrentHash() );
	}

    // Starting from each root blueprint in a type, work down the hierarchy.
    bool bFoundOrphans = false;
	Asura_Blueprint_Iterator xBPIter;
	Asura_Hash_ID uCurrentBPHash = xBPIter.GetCurrent()->GetHashID();
	u_int uBPIndex = 0;
	while ( !xBPIter.Done() )
	{
		Asura_Blueprint* pxBP = xBPIter.GetCurrent();

		if ( uCurrentBPHash != pxBP->GetHashID() )
		{
			++uBPIndex;
			uCurrentBPHash = pxBP->GetHashID();
		}
		xProgressIndicator.Update( uBPIndex, uNumBPs );

		// If this is a root blueprint then work from here down through the hierarchy.
		if ( pxBP->GetParentHashID() == ASURA_HASH_ID_UNSET )
		{
			TestForAndConvertOrphanedOverrides( xBPIter.GetCurrentBlueprintTypeHash(), pxBP, bFoundOrphans );
		}

		xBPIter.Next();
	}

    if ( bFoundOrphans )
    {
        Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ],
                                      "All orphaned overrides have been converted to parameters. Please resolve them." );
    }
}

void Tools_Mode_Blueprint::TestForAndConvertOrphanedOverrides( const Asura_Hash_ID uTypeHashID, Asura_Blueprint* pxBlueprint, bool& bFoundOrphans )
{
    // Test our blueprint for orphaned overrides. If any are found convert them into parameters and output a warning.
    for ( int i = 0; i < pxBlueprint->GetNumParameters(); ++i )
    {
        Asura_Parameter* pxParameter = pxBlueprint->GetParameterByIndex( i );
        if ( !pxParameter ) { continue; }

        if ( pxParameter->GetOverrideType() != ABOT_NONE )
        {
            Asura_Value xFinalValue;
            if ( !pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xFinalValue ) )
            {
                // If we cannot resolve the value it means that the original parameter doesn't exist for this override.
                bFoundOrphans = true;

                // Output an error.
                Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ],
					new Tools_MessageLogAction_Blueprint( uTypeHashID, pxBlueprint->GetHashID() ),
                    "Found an orphaned override '%s' in blueprint '%s'.",
                    pxParameter->GetName().GetString(), pxBlueprint->GetName().GetString() );

                // Convert this override to a proper parameter.
                pxParameter->SetOverrideType( ABOT_NONE );
                SetTypeAsModified( uTypeHashID );
            }
        }
    }

    // Now go on and test our blueprint's children.
    const int iNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( uTypeHashID, pxBlueprint->GetHashID(), true );
    for ( int iChildIndex = 0; iChildIndex < iNumChildren; ++iChildIndex )
    {
        Asura_Blueprint* pxChild = Asura_Blueprint_System::GetChildBlueprint( uTypeHashID, pxBlueprint->GetHashID(), true, iChildIndex );
        if ( pxChild )
        {
            TestForAndConvertOrphanedOverrides( uTypeHashID, pxChild, bFoundOrphans );
        }
    }
}

void Tools_Mode_Blueprint::CorrectBadGroupNames( Tools_ProgressIndicator& xProgressIndicator )
{
	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Correcting bad group names." );
	xProgressIndicator.AddMessage( "Correcting bad group names." );

	// Work out how many blueprints there are.
	u_int uNumBPs = 0;
	for ( Asura_Blueprint_Type_ConstIterator xTypeIter; !xTypeIter.Done(); xTypeIter.Next() )
	{
		uNumBPs += Asura_Blueprint_System::GetNumBlueprintsInType( xTypeIter.GetCurrentHash() );
	}

	// For each type, make sure that every override has the same group name as its base parameter.
	Asura_Blueprint_Parameter_Iterator xBPParamIter;
	Asura_Hash_ID uCurrentBPHash = xBPParamIter.GetCurrentBlueprintHash();
	u_int uBPIndex = 0;
	while ( !xBPParamIter.Done() )
	{
		if ( uCurrentBPHash != xBPParamIter.GetCurrentBlueprintHash() )
		{
			++uBPIndex;
			uCurrentBPHash = xBPParamIter.GetCurrentBlueprintHash();
		}
		xProgressIndicator.Update( uBPIndex, uNumBPs );

		const Asura_Parameter* pxParam = xBPParamIter.GetCurrent();
		if ( pxParam->GetOverrideType() == ABOT_NONE )
		{
			// Ensure all matching parameters from this blueprint down the hieararchy have the same group name.
			Asura_Blueprint* pxBP = xBPParamIter.GetCurrentBlueprint();
			CorrectBadGroupNames( xBPParamIter.GetCurrentBlueprintTypeHash(), pxParam->GetHashID(), pxParam->GetGroupName(), pxBP );
		}

		xBPParamIter.Next();
	}
}

void Tools_Mode_Blueprint::CorrectBadGroupNames( const Asura_Hash_ID uTypeHashID,
												 const Asura_Hash_ID uParamHashID,
												 const Asura_String& xGroupName,
												 Asura_Blueprint* pxBlueprint )
{
	// Do this blueprint.
	for ( int i = 0; i < pxBlueprint->GetNumParameters(); ++i )
	{
		Asura_Parameter* pxParameter = pxBlueprint->GetParameterByIndex( i );
		if ( !pxParameter || pxParameter->GetHashID() != uParamHashID ) { continue; }

		if ( pxParameter->GetGroupName().Compare( xGroupName ) != 0 )
		{
			pxParameter->SetGroupName( xGroupName );
		}
	}

	// Do all the children.
	const int iNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( uTypeHashID, pxBlueprint->GetHashID(), true );
	for ( int iChildIndex = 0; iChildIndex < iNumChildren; ++iChildIndex )
	{
		Asura_Blueprint* pxChild = Asura_Blueprint_System::GetChildBlueprint( uTypeHashID, pxBlueprint->GetHashID(), true, iChildIndex );
		if ( pxChild )
		{
			CorrectBadGroupNames( uTypeHashID, uParamHashID, xGroupName, pxChild );
		}
	}
}

void Tools_Mode_Blueprint::OutputBlueprintDataStats()
{
	// Count the number of blueprint types, blueprints, and parameters/overrides,
	// and the average number of blueprints per type and parameters/overrides per blueprint.

	const int iNumTypes = Asura_Blueprint_System::GetNumTypes();
	if ( iNumTypes == 0 ) { return; }

	int iNumBPs = 0;
	for ( Asura_Blueprint_Type_Iterator xIter; !xIter.Done(); xIter.Next() )
	{
		const int iNumNodes = xIter.GetCurrent()->m_xBlueprintHashTree.GetNumberOfNodes();
		iNumBPs += iNumNodes;
	}
	const float fBPsPerTypeMean = iNumBPs / static_cast< float >( iNumTypes );

	float fBPsPerTypeStdDev = 0.0f;
	for ( Asura_Blueprint_Type_Iterator xIter; !xIter.Done(); xIter.Next() )
	{
		const float fDev = xIter.GetCurrent()->m_xBlueprintHashTree.GetNumberOfNodes() - fBPsPerTypeMean;
		fBPsPerTypeStdDev += fDev * fDev;
	}
	fBPsPerTypeStdDev = Asura_Maths::Sqrt( fBPsPerTypeStdDev / static_cast< float >( iNumTypes ) );

	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ], "Types: %i", iNumTypes );
	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ],
		"BPs: %i (%f/type average, %f stddev)", iNumBPs, fBPsPerTypeMean, fBPsPerTypeStdDev );

	int iNumParams = 0;
	for ( Asura_Blueprint_ConstIterator xIter; !xIter.Done(); xIter.Next() )
	{
		const int iNum = xIter.GetCurrent()->GetNumParameters();
		iNumParams += iNum;
	}
	const float fParamsPerBPMean = iNumParams / static_cast< float >( iNumBPs );

	float fParamsPerBPStdDev = 0.0f;
	for ( Asura_Blueprint_ConstIterator xIter; !xIter.Done(); xIter.Next() )
	{
		const float fDev = xIter.GetCurrent()->GetNumParameters() - fParamsPerBPMean;
		fParamsPerBPStdDev += fDev * fDev;
	}
	fParamsPerBPStdDev = Asura_Maths::Sqrt( fParamsPerBPStdDev / static_cast< float >( iNumBPs ) );

	// Non-empty stuff.
	int iNumNEBPs = 0;
	int iMinNEParams = INT_MAX;
	int iMaxNEParams = -1;
	for ( Asura_Blueprint_ConstIterator xIter; !xIter.Done(); xIter.Next() )
	{
		const int iNum = xIter.GetCurrent()->GetNumParameters();
		if ( iNum > 0 )
		{
			++iNumNEBPs;
			if ( iNum < iMinNEParams ) { iMinNEParams = iNum; }
			if ( iNum > iMaxNEParams ) { iMaxNEParams = iNum; }
		}
	}
	const float fParamsPerNEBPMean = iNumParams / static_cast< float >( iNumNEBPs );

	float fParamsPerNEBPStdDev = 0.0f;
	for ( Asura_Blueprint_ConstIterator xIter; !xIter.Done(); xIter.Next() )
	{
		if ( xIter.GetCurrent()->GetNumParameters() > 0 )
		{
			const float fDev = xIter.GetCurrent()->GetNumParameters() - fParamsPerNEBPMean;
			fParamsPerNEBPStdDev += fDev * fDev;
		}
	}
	fParamsPerNEBPStdDev = Asura_Maths::Sqrt( fParamsPerNEBPStdDev / static_cast< float >( iNumNEBPs ) );

	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ],
		"Params: %i (%f/BP average %f stddev) (%f/NEBP average %f stddev)",
		iNumParams, fParamsPerBPMean, fParamsPerBPStdDev, fParamsPerNEBPMean, fParamsPerNEBPStdDev );
	Tools_MessageLog::AddMessage( s_pszLogCategory, s_pszLogSubCategory[ LOG_VERIFICATION ],
		"Params/NEBP: %i min, %i max", iMinNEParams, iMaxNEParams );
}

void Tools_Mode_Blueprint::SetDisplayOptions()
{
    if ( !s_pxThisBlueprintMode ) { return; }
        
    if ( s_pxThisBlueprintMode->s_bDisplayOverlayIsOpen )
    {
        Asura_Overlay_System::BringOverlayToTop( reinterpret_cast< u_int >( s_pxThisBlueprintMode + 1 ) );
    }
	else
    {
        s_pxThisBlueprintMode->CreateDisplayOptionsOverlay();
    }
}

void Tools_Mode_Blueprint::SetFilterGroupNameFromFilterChoice()
{
    Asura_Parameter::GetGroupName( s_iFilterChoice, s_xFilterGroupName );
}

void Tools_Mode_Blueprint::SetFilterGroupChoiceFromFilterName()
{
    s_iFilterChoice = Asura_Parameter::GetGroupListIndex( s_xFilterGroupName );
    if ( s_iFilterChoice == -1 )
    {
        s_iFilterChoice = 0;
        Asura_Parameter::GetGroupName( s_iFilterChoice, s_xFilterGroupName );
    }
}

void Tools_Mode_Blueprint::AddBlueprintFilesToExportList( List<Asura_String>& xNameList, Asura_Hash_ID uTypeHashID, Asura_Hash_ID uParentHashID, const bool bExportChildBlueprints )
{
	if(bExportChildBlueprints)
	{
		for ( Asura_Blueprint_Iterator xBPIter( uTypeHashID ); !xBPIter.Done(); xBPIter.Next() )
		{
			Asura_Blueprint* pxBlueprint = xBPIter.GetCurrent();

			bool bDoExport = false;

			// make sure this blueprint has the parent we're exporting for
			if ( uParentHashID == ASURA_HASH_ID_UNSET )
			{
				bDoExport = true;
			}
			else if ( pxBlueprint->GetHashID() == uParentHashID || pxBlueprint->IsChildOf( uTypeHashID, uParentHashID, false ) )
			{
				bDoExport = true;
			}

			// correct parent, add to export
			if(bDoExport)
			{
				s_pxThisBlueprintMode->AddBlueprintFilesToExportList(xNameList, pxBlueprint, true);
			}
		}	
	}
	else
	{
		Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID(uTypeHashID, uParentHashID);

		// Not recursive - just export the parent blueprint and nothing else...
		if(pxBlueprint)
		{
			AddBlueprintFilesToExport(xNameList, pxBlueprint, false);
		}
#ifdef ASURA_ASSERT
		else
		{
			const Asura_String* pxTypeName = Asura_Blueprint_System::GetTypeName( uTypeHashID );
			if ( pxTypeName )
			{
				Asura_Assertf( false, ( "Cannot find blueprint 0x%08x in type '%s' to export.", uParentHashID, pxTypeName->GetString() ) );
			}
			else
			{
				Asura_Assertf( false, ( "Cannot find blueprint 0x%08x in type 0x%08x to export.", uParentHashID, uTypeHashID ) );
			}
		}
#endif
	}
}

void Tools_Mode_Blueprint::AddBlueprintFilesToExport( List<Asura_String>& xNameList, Asura_Blueprint* pxBlueprint, const bool bExportChildBlueprints )
{
	if(s_pxThisBlueprintMode)
	{
		s_pxThisBlueprintMode->AddBlueprintFilesToExportList(xNameList, pxBlueprint, bExportChildBlueprints);
	}
}

void Tools_Mode_Blueprint::AddBlueprintFilesToExportList( List<Asura_String>& xNameList, Asura_Blueprint* pxBlueprint, const bool bExportChildBlueprints )
{	
	if( !Tools_Level_Export::ShouldExportBlueprint_Static(pxBlueprint, bExportChildBlueprints) ) return;

	Asura_NameList xExportedParameterList(false);

	extern u_int g_uBlueprintLogfileIndent;
	if( g_uBlueprintLogfileIndent == 0 )
	{
		Asura_SimpleLogfileOutput(( Tools_Level_Export::s_szBlueprintLogPath, "Exporting Blueprint '%s'\n", pxBlueprint->GetName().GetString() ));
	}	

	pxBlueprint->SetExported(bExportChildBlueprints);

	TOOLS_RESOURCEEXPORTLOG("Blueprint",pxBlueprint->GetName().GetString(),false);


	// export any templated-type parameters or special parameter types
	while( pxBlueprint )
	{
		for( int iParam=0; iParam<pxBlueprint->GetNumParameters(); ++iParam )
		{
			Asura_Parameter* pxParameter = pxBlueprint->GetParameterByIndex( iParam );
			if( !pxParameter ) continue; 

			// don't add parameter again if we've got an override from a previous child
			if( xExportedParameterList.GetIndexOfName( pxParameter->GetName() ) != -1 ) continue; 

			// add the parameter to the list of already exported params
			xExportedParameterList.AddName( pxParameter->GetName() );

			if( pxParameter->GetTemplate() != ASURA_HASH_ID_UNSET )
			{
				// check this is a valid template to export for
				if( s_puTemplateFilterList )
				{
					u_int u=0;
					while( s_puTemplateFilterList[u] != ASURA_HASH_ID_UNSET )
					{
						if( s_puTemplateFilterList[u] == pxParameter->GetTemplate() ) break;						
						u++;
					}

					// template not found in valid list, skip this parameter
					if( s_puTemplateFilterList[u] == ASURA_HASH_ID_UNSET ) continue;
				}

				Tools_Blueprint_Edit_Object::ExportTemplatedType( xNameList, pxParameter, pxBlueprint );
			}	

#if !defined(ASURA_DISABLE_ANIM_TABLE_CODE)
			// if this is an anim-table, export it
			// SJL 13.2.08 this block is redundant, at least for BF3, animtables now come with their own TVT but for old or other projects
			// but for old or other projects ill leave this in.
			if( (pxParameter->GetHashID() == Asura_GetHashID("Animtable")) && !Tools_Level_Export::IsGUIMenuExport()  )
			{
				Asura_Value xValue;
				if( pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) )
				{
					// make sure it's been loaded
					Asura_Assertf( Asura_AnimTable_System::GetAnimTable( xValue.GetHashIDValue() ), ("Trying to export animtable '%s' for blueprint '%s' but it's not been loaded. Please let Rich know.",xValue.GetHashStringValue(),pxBlueprint->GetName()) );
					Tools_Level_Export::AddAnimTableToExportList( xValue.GetHashIDValue() );
				}				
			}

			if (pxParameter->GetTemplate() == Tools_Blueprint_Edit_Object::TVT_ANIMTABLE)
			{
				Asura_Value xValue;
				if( pxBlueprint->GetParameterValueByHashID( pxParameter->GetHashID(), xValue ) )
				{
					// make sure it's been loaded
					Asura_Assertf( Asura_AnimTable_System::GetAnimTable( xValue.GetHashIDValue() ), ("Trying to export animtable '%s' for blueprint '%s' but it's not been loaded. Please let Rich know.",xValue.GetHashStringValue(),pxBlueprint->GetName()) );
					Tools_Level_Export::AddAnimTableToExportList( xValue.GetHashIDValue() );
				}				

			}
#endif	// !defined(ASURA_DISABLE_ANIM_TABLE_CODE)
		}

		pxBlueprint = pxBlueprint->GetParent();
	}

	if( g_uBlueprintLogfileIndent == 0 )
	{
		Asura_SimpleLogfileOutput(( Tools_Level_Export::s_szBlueprintLogPath, "\n\n" ));
	}	
}

bool Tools_Mode_Blueprint::CanPasteBlueprint() 
{
	return s_xPasteBuffer.IsValid();
}

void Tools_Mode_Blueprint::CopySelected()
{
	CopySelected(s_xPasteBuffer);
}

void Tools_Mode_Blueprint::CopySelected(Tools_Blueprint_PasteBuffer& xPasteBuffer)
{
	List<Asura_TreeDialog_Item*> xSelectionList;
	s_xTree.GetSelectionRemovingChildren(xSelectionList);
	if(!xSelectionList.size()) return;

	// clear the blueprint buffer
	xPasteBuffer.Reset();

	//add the selection to the paste buffer
	for(LIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelectionList); !xSelectionLIF.done(); xSelectionLIF.next())
	{
		xPasteBuffer.Add(xSelectionLIF());
	}

	int iNumEntry = xPasteBuffer.GetNumEntries();
	int iTotalBP = xPasteBuffer.GetTotalNumberOfBlueprints();

	if(iNumEntry)
	{
		if(iNumEntry == 1)
		{
			const Tools_Blueprint_PasteBuffer_Entry* pxEntry = xPasteBuffer.GetEntry(0);

			Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT,
					"Blueprint '%s' copied (copied %d children).", pxEntry->GetBlueprint(0)->GetName(), iTotalBP-1 );
		}
		else
		{
			Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT,
					"Copied %d blueprints (with %d children).", iNumEntry, iTotalBP-iNumEntry );
		}
	}
	
}

void Tools_Mode_Blueprint::CutSelectedBlueprint()
{
	CopySelected();
	DeleteSelected( 1 );
}

void Tools_Mode_Blueprint::PasteBlueprintOrType()
{
	PasteBlueprintOrType(s_xPasteBuffer, s_xTree.GetSelectedItem());
}


void Tools_Mode_Blueprint::PasteBlueprintOrType(const Tools_Blueprint_PasteBuffer& xPasteBuffer, const Asura_TreeDialog_Item* pxParentItem)
{
	Asura_Assert(pxParentItem, "");
	const bool bParentIsType = (pxParentItem->GetUserData() == 0);
	const Asura_Hash_ID uTargetTypeHashID = bParentIsType ? Asura_GetHashID( pxParentItem->GetName() ) :  pxParentItem->GetUserData();
	const Asura_Hash_ID uNewParentHashID  = bParentIsType ? ASURA_HASH_ID_UNSET : Asura_GetHashID( pxParentItem->GetName() );

	if(!xPasteBuffer.IsValid()) return;

	Tools_Blueprint_Edit_Object::ClearParameterPasteConflictList();	
	
	// paste blueprints into hierarchy
	for(int iEntry=0 ; iEntry<xPasteBuffer.GetNumEntries(); iEntry++)
	{
		const Tools_Blueprint_PasteBuffer_Entry* pxPasteBufferEntry = xPasteBuffer.GetEntry(iEntry);
	

		// take a copy of the copy buffer for working
		u_int uNumCopiedBlueprints = pxPasteBufferEntry->GetNumBlueprints();
		Asura_Blueprint* pxCopiedBlueprintsWorking = new Asura_Blueprint[uNumCopiedBlueprints];
		for( u_int u=0; u<uNumCopiedBlueprints; ++u )
		{
			pxCopiedBlueprintsWorking[u] = *pxPasteBufferEntry->GetBlueprint(u);
		}

		// change the parent hash ID of the root blueprint to the paste target (either the blueprint or the current type)
		pxCopiedBlueprintsWorking[0].SetParentHashID( uNewParentHashID );
		
		// firstly, we need to make sure there aren't any name clashes in the new destination type
		bool bFailed = false;
		for( u_int u=0; u<uNumCopiedBlueprints; ++u )
		{
			// If a blueprint with this name already exists in the target type: we'll need to rename the one in 
			// the working copy buffer before pasting. Keep trying to rename by appending '_##' to the end until 
			// we find a valid one.
			Asura_Hash_ID uOldHashID = pxCopiedBlueprintsWorking[u].GetHashID();
			Asura_Hash_ID uNewHashID = pxCopiedBlueprintsWorking[u].GetHashID();
			
			u_int uAttempts = 0;
			while( Asura_Blueprint_System::GetBlueprintByHashID( uTargetTypeHashID, pxCopiedBlueprintsWorking[u].GetHashID() ) && (uAttempts < 100) )
			{
				// use the original copy buffer to get previous name/hash, not our working copy				
				Asura_String xOldName = pxPasteBufferEntry->GetBlueprint(u)->GetName();
				const char* szEnd = &(xOldName[xOldName.GetLength() - 3]);
				if( szEnd[0] == '_'
				 && isdigit(szEnd[1])
				 && isdigit(szEnd[2]) )
				{
					xOldName.Truncate( xOldName.GetLength() - 3 );
				}				
				
				char szNewName[256];
				Asura_PrintfToBfr( szNewName, "%s_%.2d", 256, true, xOldName.GetString(), uAttempts );

				pxCopiedBlueprintsWorking[u].SetName( szNewName );
				pxCopiedBlueprintsWorking[u].SetHashID( Asura_GetHashID(szNewName) );				
				uNewHashID = pxCopiedBlueprintsWorking[u].GetHashID();
				uAttempts++;
			}

			if( uAttempts >= 100 )
			{
				Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR,
					"Could not copy blueprint: failed to resolve blueprint name clash." );
				bFailed = true;
				break;
			}

			// armed with our newly-renamed blueprint, we need to re-map the hierarchy below
			if( uNewHashID != uOldHashID )
			{
				for( u_int v=1; v<uNumCopiedBlueprints; ++v ) // skip the first entry as that's the blueprint we're pasting so we don't want to consider it's parent
				{
					if( pxCopiedBlueprintsWorking[v].GetParentHashID() == uOldHashID )
					{
						pxCopiedBlueprintsWorking[v].SetParentHashID( uNewHashID );
					}					
				}
			}
		}
		if(bFailed)
		{
			delete [] pxCopiedBlueprintsWorking;
			continue;	
		}

		// Okay, armed with the cleaned-up copy buffer we now run through, adding blueprints in turn to 
		// the Asura_Blueprint_System. This doesn't need to be done in order as we can call EstablishLinkage later
		for( u_int u=0; u<uNumCopiedBlueprints; ++u )
		{
			Asura_Blueprint_System::AddBlueprint( uTargetTypeHashID, pxCopiedBlueprintsWorking[u].GetName(), pxCopiedBlueprintsWorking[u].GetParentHashID(), false );
		}
		Asura_Blueprint_System::EstablishAllLinkage( uTargetTypeHashID );

		// Now we need to copy in the blueprint parameters from the copy buffer. To do this we're going to use 
		// the already-existing copy/paste parameter functionality in the Tools_Blueprint_Edit_Object to save
		// us re-doing a load of work and also because it deals with a lot of the nastier conflict issues already
		// This requires the blueprints in the copy buffer to be sorted in parent/child order

		for( u_int u=0; u<uNumCopiedBlueprints; ++u )
		{
			Asura_Blueprint* pxTargetBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uTargetTypeHashID, pxCopiedBlueprintsWorking[u].GetHashID() );
			if( pxTargetBlueprint )
			{
				Tools_Blueprint_Edit_Object::CopyParameters( &(pxCopiedBlueprintsWorking[u]), pxCopiedBlueprintsWorking[u].GetNumParameters(),
															 pxPasteBufferEntry->GetType() );
				Tools_Blueprint_Edit_Object::PasteOverParameters( pxTargetBlueprint, uTargetTypeHashID );
			}
		}

		// clean up working buffer
		delete [] pxCopiedBlueprintsWorking;
	}

	// report any conflicts
	Tools_Blueprint_Edit_Object::ReportParameterPasteConflicts( "The following parameter(s) could not be pasted, as they already exist as different types in this blueprint hierarchy:" );
	
	SetTypeAsModified( uTargetTypeHashID );

	// re-create the blueprint overlays so we can see any changes 
	s_pxThisBlueprintMode->DestroyOverlay();
	s_pxThisBlueprintMode->CreateOverlay();
	s_pxThisBlueprintMode->RequestRefreshBlueprintEditObjectOverlays();
	BlueprintChanged();
}

void Tools_Mode_Blueprint::ReleasePasteBuffer()
{
	s_xPasteBuffer.Reset();
}

void Tools_Mode_Blueprint::DragAndDrop(Asura_TreeDialog_Item* pxDropTarget)
{
	Asura_Assert(pxDropTarget, "");

	List<Asura_TreeDialog_Item*> xSelectionList;
	s_xTree.GetSelectionRemovingChildren(xSelectionList);
	if(!xSelectionList.size()) return;

	//make sure the target isn't a child of any part of the selection
	for(LIF<Asura_TreeDialog_Item*> xSelectionLIF(&xSelectionList); !xSelectionLIF.done(); xSelectionLIF.next())
	{
		if(pxDropTarget->IsDescendentOf(xSelectionLIF()))
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Target of drag operation is part of the selection (or a child of a selected folder)");
			return;
		}
	}

	//Perform the operation by using the existing functions for copying, deleting and pasting the selection
	//Use a temporary paste buffer to avoid overwriting the user's one
	Tools_Blueprint_PasteBuffer xPasteBuffer;
	CopySelected(xPasteBuffer);
	if(xPasteBuffer.IsValid())
	{
		DeleteSelected(1);
		PasteBlueprintOrType(xPasteBuffer, pxDropTarget);
	}
}

bool Tools_Mode_Blueprint::IsSkinFilename_Static(const char* szLowerCaseFilename)
{
	Asura_Assert(s_pxThisBlueprintMode,"Blueprint mode not allocated");
	return s_pxThisBlueprintMode ? s_pxThisBlueprintMode->IsSkinFilename(szLowerCaseFilename) : false;
}

bool Tools_Mode_Blueprint::IsSkinFilename(const char* szLowerCaseFilename)
{
	if(strstr(szLowerCaseFilename,"anims\\")) return false;
	if(strstr(szLowerCaseFilename,"animations\\")) return false;
	return true;
}

//////////////////////////////////////////////////////////////////////////////
//
// Tools_MessageLogAction_Blueprint
//
//////////////////////////////////////////////////////////////////////////////

Tools_MessageLogAction_Blueprint::Tools_MessageLogAction_Blueprint( Asura_Hash_ID uTypeID, Asura_Hash_ID uBPID, Asura_Hash_ID uParamID/*=ASURA_HASH_ID_UNSET*/ )
: m_uTypeID(uTypeID)
, m_uBPID(uBPID)
, m_uParamID(uParamID)
{
}
	
void Tools_MessageLogAction_Blueprint::Execute()
{
	PARENT::Execute();
	Tools_Mode_Blueprint::CreateEditOverlay( m_uTypeID, m_uBPID, m_uParamID );
}

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Blueprint_PasteBuffer
//
//////////////////////////////////////////////////////////////////////////////

bool Tools_Blueprint_PasteBuffer::Add(const Asura_TreeDialog_Item* pxItem)
{
	//make sure this isn't a blueprint type
	if(pxItem->GetUserData() == 0) return false;

	//and make sure the blueprint is available
	const Asura_Hash_ID uTypeHashID      = pxItem->GetUserData();
	const Asura_Hash_ID uBlueprintHashID = Asura_GetHashID( pxItem->GetName() );

	Asura_Blueprint* pxBlueprintToCopy = Asura_Blueprint_System::GetBlueprintByHashID( uTypeHashID, uBlueprintHashID );
	if(!pxBlueprintToCopy) return false;
	

	//allocate an entry for this
	Tools_Blueprint_PasteBuffer_Entry* pxEntry = new Tools_Blueprint_PasteBuffer_Entry;
	m_xList.add_entry(pxEntry);
	pxEntry->m_uType = uTypeHashID;

	int iNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( uTypeHashID, uBlueprintHashID, false );
	
	// allocate blueprint storage
	Asura_Blueprint* pxCopiedBlueprints = pxEntry->m_pxBlueprints= new Asura_Blueprint[iNumChildren + 1]; // all the children plus ourselves
	
	int iBPsCopied = 0;			

	// copy ourselves
	pxCopiedBlueprints[iBPsCopied] = *pxBlueprintToCopy;
	iBPsCopied++;
	
	// and the children
	if( iNumChildren > 0 )
	{	
		for( int i=0; i<iNumChildren; ++i )
		{					
			pxBlueprintToCopy = Asura_Blueprint_System::GetChildBlueprint( uTypeHashID, uBlueprintHashID, false, i );
			pxCopiedBlueprints[iBPsCopied] = *pxBlueprintToCopy;
			iBPsCopied++;						
		}

		// we need to sort the array in parent->child order to make the pasting work properly		
		// The sorting algorithm works by starting with the first blueprint (which is always the root) and swapping 
		// any subsequent entries for its direct children.
		int iFirstUnsorted = 0;
		for( int i=0; i<iBPsCopied; ++i  )
		{
			iFirstUnsorted = Asura_Maths::Max( iFirstUnsorted, i+1 );
			if( iFirstUnsorted >= iBPsCopied ) break; // no more unsorted bps left

			Asura_Hash_ID uCurrentParentID = pxCopiedBlueprints[i].GetHashID();
			for( int j=iFirstUnsorted; j<iBPsCopied; ++j )
			{					
				if( pxCopiedBlueprints[j].GetParentHashID() != uCurrentParentID )
				{
					// swap with the first direct parent we find 
					for( int k=j; k<iBPsCopied; ++k )
					{
						if( pxCopiedBlueprints[k].GetParentHashID() == uCurrentParentID )
						{
							Asura_Blueprint xTemp = pxCopiedBlueprints[j];
							pxCopiedBlueprints[j] = pxCopiedBlueprints[k];
							pxCopiedBlueprints[k] = xTemp;

							iFirstUnsorted = j+1;
							break;
						}
					}
				}						
				else
				{
					iFirstUnsorted = j+1;
				}
			}
		}
	}
	pxEntry->m_iNumBlueprints = iBPsCopied;
	
	return true;	
}

void Tools_Blueprint_PasteBuffer::Reset()
{
	while(m_xList.size())
	{
		delete m_xList.first_entry();
		m_xList.delete_first_entry();
	}
}

int Tools_Blueprint_PasteBuffer::GetTotalNumberOfBlueprints() const
{
	int iTotal = 0;
	for(CLIF<Tools_Blueprint_PasteBuffer_Entry*> xEntryLIF(&m_xList); !xEntryLIF.done(); xEntryLIF.next())
	{
		iTotal += xEntryLIF()->m_iNumBlueprints;
	}
	return iTotal;
}

////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------**
** Tools_Blueprint_PasteBuffer_Entry **
**-----------------------------------*/

Tools_Blueprint_PasteBuffer_Entry::Tools_Blueprint_PasteBuffer_Entry()
{
	m_iNumBlueprints = 0;
	m_pxBlueprints = 0;
	m_uType = 0;
}

Tools_Blueprint_PasteBuffer_Entry::~Tools_Blueprint_PasteBuffer_Entry()
{
	delete [] m_pxBlueprints;
}


//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#undef szBLUEPRINTS_DIRECTORY
#undef szBLUEPRINTS_EXTENSION



