//
// Tools_Mode_BotanicalCreation.h
//
// Created by Dave Reed - 04/01/2006
// 
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#if defined( ASURA_USE_BOTANICALS) && defined( ASURA_TOOLS)

#ifndef __TOOLS_MODE_BOTANICALCREATION_H__
#define __TOOLS_MODE_BOTANICALCREATION_H__

#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_LinkList.h"
#include "Network/Asura_Guid.h"
#include "Botanicals/Asura_Botanicals_ShapeManager.h"
#include "Tools_Mode.h"

class Tools_BotanicalShape;

class Tools_Mode_BotanicalCreation : public Tools_Mode
{
public:
	Tools_Mode_BotanicalCreation()
	{
		Initialise();
	}

	virtual ~Tools_Mode_BotanicalCreation()
	{
		Shutdown();
	}	

	virtual void PreShutdown();
	virtual void PreSave();
	virtual void PostSave();
	virtual void QuickSave(void);

	virtual bool IsModified() {return true;};

	static  bool Save(void);

	void Initialise(void);
	void Update(void);
	void RenderUI(void);
	void Shutdown(void);


	static float					GetCurrentLODValue() 		{return s_fLODSliderValue;};
	static Asura_BotanicalShape*	GetCurrentBotanicalShape()	{return s_pxCurrentBotanicalShape;}
	static Asura_BotanicalShape*	CreateBotanicalShape( const char *szFileName, bool bForceReload=false, bool bFirstTime=false);
	static float					CalculateLOD(Asura_Vector_3 *pxPos, float fInvMaxDrawDistance = 0.01f);
	static bool						IsLightingEnabled(void);
	static void						CreateBillboardFromQuad(Tools_Botanicals_Billboard *pxBB,Asura_Mesh *pxMesh,int iPoly);
	static Asura_Vector_3           GetOrigin();	
	static void						SetRenderStateToShowAllElements();
	static const char*				GetCurrentDefinitionFilename(void);

private:

	enum DialogIDs
	{
		DialogID_LODSlider		
	};

	Asura_Vector_3			m_xPreModeCameraPosition;
	Asura_Matrix_3x3		m_xPreModeCameraOrientation;

	static void				CreatePreviewControlsOverlay(void);
	static void				CreateMainOverlay(void);
	static void				CloseMainOverlay(void);
	static void				CreateLODsOverlay(void);
	static void				CloseLODsOverlay(void);		
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	static void				CreateBasicInteractiveOverlay( void);
	static void				CloseBasicInteractiveOverlay( void);
#endif
#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	static void				CreateStateTableOverlay( void);
	static void				CloseStateTableOverlay( void);
#endif
	static void				CreateFromShapeFile(void);
	static void				LoadBotanicalShape(void);
	static bool				SaveChanges(void);
	static void				ReloadShape(void);	

	// The shape we're currently working on, if we've got one open
	static Asura_BotanicalShape* s_pxCurrentBotanicalShape;

	// Loads of statics for the overlays..
	// Preview Overlay (bottom of screen, with slider for LOD)
	static Asura_Vector_2	s_xPreviewOverlayPosition;
	static float			s_fLODSliderValue;
	static float			s_fLODSliderMin;
	static float			s_fLODSliderMax;	
	static bool				s_bShowChecks[3];
	static bool				s_bShowChecksChanged[3];
	static bool				s_bManuallySetLOD;

	// Main overlay ('Botanical Creation' menu)
	static bool				s_bMainOverlayOpen;
	static Asura_Vector_2	s_xMainOverlayPosition;
	static bool				s_bUseLocalCamera;
	static bool				s_bCameraModeChanged;	

public:
	static bool				s_bLightingEnabled;
};

#endif //__TOOLS_MODE_BOTANICALCREATION_H__
#endif //ASURA_USE_BOTANICALS


