//
// Tools_Botanicals_Styles.cpp
//
// Created by Dave Reed - 17/11/2005
// 
// Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_USE_BOTANICALS

#include "Core/Asura_UserInput.h"

#include "Tools_Overlays.h"

#include "Botanicals/Asura_Botanicals_TemplateManager.h"
#include "Botanicals/Tools_Botanicals_Styles.h"

#include "Interface/Asura_Interface.h"
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"

#include "Misc/Tools_Modal_Overlay.h"


int						   Tools_Botanicals_StyleManager::s_iNumStyles=0;
Tools_Botanicals_Style*	   Tools_Botanicals_StyleManager::s_pxStyles=NULL;
int						   Tools_Botanicals_StyleManager::s_iStyleBeingEdited=-1;

bool					   Tools_Botanicals_StyleManager::s_bEditStyleOverlayVisible=false;
Asura_Vector_2			   Tools_Botanicals_StyleManager::s_xEditStyleOverlayPosition=Asura_Vector_2(20,5);
Asura_String			   Tools_Botanicals_StyleManager::s_xStyleNameString="";
char					   Tools_Botanicals_StyleManager::s_aszTemplateNames[TOOLS_BOTANICALS_ENTRIES_PER_STYLE][32];

void Tools_Botanicals_StyleManager::Reset()
{
	if (s_pxStyles!=NULL)
	{	
		delete [] s_pxStyles;
		s_pxStyles=0;
	}

	s_iNumStyles=0;
}


void	Tools_Botanicals_StyleManager::Shutdown()
{
	Reset();
}

Tools_Botanicals_Style* Tools_Botanicals_StyleManager::AddStyle()
{
	// Realloc the buffer
	Tools_Botanicals_Style* pxNewStyles=new Tools_Botanicals_Style[s_iNumStyles+1];

	// Copy in all the existing styles
	for(int i=0;i<s_iNumStyles;i++)
	{
		pxNewStyles[i]=s_pxStyles[i];
	}

	// Set up the new one
	Tools_Botanicals_Style* pxNewStyle=&pxNewStyles[s_iNumStyles];
	s_iNumStyles++;
	pxNewStyle->xStyleName="(Unnamed Style)";

	for(int i=0;i<TOOLS_BOTANICALS_ENTRIES_PER_STYLE;i++)
	{
		Tools_Botanicals_StyleEntry* pxEntry=&pxNewStyle->axEntries[i];
		pxEntry->iTemplateIdx=-1;
		pxEntry->iTemplateUID=-1;
		pxEntry->fWeight=1;
	}

	// Swap the buffers over
	if (s_pxStyles)
	{
		delete [] s_pxStyles;
	}

	s_pxStyles=pxNewStyles;

	return pxNewStyle;
}


void Tools_Botanicals_StyleManager::RemoveStyle(int iIdx)
{
	Tools_Botanicals_Style* pxNewStyles=NULL;

	if (s_iNumStyles>1)
	{
		// Realloc the buffer
		pxNewStyles=new Tools_Botanicals_Style[s_iNumStyles-1];

		
		// Copy in all the remaining styles
		for(int i=0;i<s_iNumStyles-1;i++)
		{
			if (i<iIdx) 
				pxNewStyles[i]=s_pxStyles[i];
			else
				pxNewStyles[i]=s_pxStyles[i+1];
		}

		s_iNumStyles--;
	}
	else
	{
		// No styles left		
		s_iNumStyles=0;
	}

	// Swap the buffers over
	if (s_pxStyles)
	{
		delete [] s_pxStyles;
	}

	s_pxStyles=pxNewStyles;	
}


int	Tools_Botanicals_StyleManager::GetNumStyles()
{
	return s_iNumStyles;
}


Tools_Botanicals_Style *Tools_Botanicals_StyleManager::GetStyle(int iIdx)
{
	return &s_pxStyles[iIdx];
}


char *Tools_Botanicals_StyleManager::GetStyleName(int iIndex)
{
	if (iIndex>=0 && iIndex<s_iNumStyles)
	{
		return s_pxStyles[iIndex].xStyleName.GetString();
	}
	else
	{
		return "<Invalid Style Idx>";
	}	
}


void Tools_Botanicals_StyleManager::RemapStyles()
{
	// Go through each style, getting style indexes from their UIDs
	for(int i=0;i<s_iNumStyles;i++)
	{
		Tools_Botanicals_Style *pxStyle=&s_pxStyles[i];
	
		for(int j=0;j<TOOLS_BOTANICALS_ENTRIES_PER_STYLE;j++)
		{
			Tools_Botanicals_StyleEntry* pxEntry=&pxStyle->axEntries[j];			
			pxEntry->iTemplateIdx=Asura_Botanicals_TemplateManager::GetTemplateIndexFromUID(pxEntry->iTemplateUID);			
		}
	}
}


void Tools_Botanicals_StyleManager::OpenEditStyleOverlay(int iStyleIdx)
{
	Tools_Botanicals_Style* pxStyle=&s_pxStyles[iStyleIdx];

	s_iStyleBeingEdited=iStyleIdx;	

	if (s_bEditStyleOverlayVisible)
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_BOTANICALS_STYLE);
	}

	s_bEditStyleOverlayVisible=true;


	Asura_Overlay* pxOverlay;	
	pxOverlay = new Asura_Overlay("Edit Style",false,&s_xEditStyleOverlayPosition);
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	Asura_Dialog*  pxDialog;

	// Name
	s_xStyleNameString=pxStyle->xStyleName;
	pxDialog= new Asura_String_Dialog("Name",xOffset,&s_xStyleNameString,31);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=2.0f*Asura_Dialog::DefaultSpacing();
	
	// Headers
	pxDialog = new Asura_Dialog("Template",xOffset);	
	pxOverlay->AddDialog(pxDialog);

	xOffset.x+=25;
	pxDialog = new Asura_Dialog("Weighting",xOffset);			
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	xOffset.x-=25;


	// Template/weight list
	for(int i=0; i<TOOLS_BOTANICALS_ENTRIES_PER_STYLE; i++)
	{
		// Template select button
		int iTemplate=pxStyle->axEntries[i].iTemplateIdx;						

		if (iTemplate<0)
		{
			pxDialog = new Asura_Function_Dialog("<None>",xOffset,SelectTemplate,i);
		}
		else
		{
			Asura_Botanicals_Template *pxTemplate=Asura_Botanicals_TemplateManager::GetTemplate(iTemplate);
			strncpy(s_aszTemplateNames[i],pxTemplate->xTemplateName.GetString(),32);
			pxDialog = new Asura_Function_Dialog(s_aszTemplateNames[i],xOffset,SelectTemplate,i);
		}

		pxOverlay->AddDialog(pxDialog);
	
		// Weight
		xOffset.x+=25.0f;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&pxStyle->axEntries[i].fWeight,0.0f,5.0f,0.1f,"%.01f");
		pxOverlay->AddDialog(pxDialog);

		xOffset.x=1.0f;
		xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();
	}

	xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();
	

	// Close
	pxDialog = new Asura_Function_Dialog("Close",xOffset,CloseEditStyleOverlay);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxOverlay->m_uID = TOOLS_OVERLAY_BOTANICALS_STYLE;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}


void Tools_Botanicals_StyleManager::CloseEditStyleOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_BOTANICALS_STYLE);
	s_bEditStyleOverlayVisible=false;
}


bool Tools_Botanicals_StyleManager::UpdateEditing(void)
{
	if (s_bEditStyleOverlayVisible)
	{
		// Update the name string
		Tools_Botanicals_Style* pxStyle=&s_pxStyles[s_iStyleBeingEdited];		
		pxStyle->xStyleName = s_xStyleNameString.GetString();
		return true;
	}

	return false;
}


void Tools_Botanicals_StyleManager::SelectTemplate(int i)
{
	Tools_Botanicals_Style* pxStyle=&s_pxStyles[s_iStyleBeingEdited];		

	// Pop up the modal style selector
	ListSelection_Overlay xOverlay("Select Botanical Template: ");

	xOverlay.AddItem("<None>",0);
	for (int i=0;i<Asura_Botanicals_TemplateManager::GetNumTemplates();i++)	
	{
		xOverlay.AddItem(Asura_Botanicals_TemplateManager::GetTemplateName(i), 1+i);	
	}

	if (!xOverlay.DoModal()) return;
	
	int iIdx=xOverlay.GetSelectionData()-1;

	// Store the style index and UID
	if (iIdx<0)
	{
		pxStyle->axEntries[i].iTemplateIdx=-1;
		pxStyle->axEntries[i].iTemplateUID=-1;
	}
	else
	{
		Asura_Botanicals_Template* pxTemplate=Asura_Botanicals_TemplateManager::GetTemplate(iIdx);
		int iUID=pxTemplate->iUID;
		pxStyle->axEntries[i].iTemplateIdx=iIdx;
		pxStyle->axEntries[i].iTemplateUID=iUID;
	}

	// Refresh the overlay
	CloseEditStyleOverlay();
	OpenEditStyleOverlay(s_iStyleBeingEdited);
}



int Tools_Botanicals_StyleManager::GetRandomTemplate(int iStyle)
{
	Tools_Botanicals_Style* pxStyle=&s_pxStyles[iStyle];		

	float fMax=0;
	int   iSelected=-1;

	for(int i=0;i<TOOLS_BOTANICALS_ENTRIES_PER_STYLE;i++)
	{
		if (pxStyle->axEntries[i].iTemplateIdx>=0)
		{
			fMax+=pxStyle->axEntries[i].fWeight;
		}
	}

	float fRandom = Asura_Random::GetFloat(0,fMax);
	float fMin=0;
	fMax=0;

	for(int i=0;i<TOOLS_BOTANICALS_ENTRIES_PER_STYLE;i++)
	{
		if (pxStyle->axEntries[i].iTemplateIdx>=0)
		{
			fMax+=pxStyle->axEntries[i].fWeight;

			if (fRandom>=fMin && fRandom<fMax)
			{
				iSelected=pxStyle->axEntries[i].iTemplateIdx;
			}
		}

		fMin=fMax;
	}

	return iSelected;
}

#endif // ASURA_USE_BOTANICALS
