//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE        :  Tools_Botanicals_Templates.cpp
//
//  Created by Dave Reed - 04/12/05
//
//  Copyright (C) 2004 Rebellion (Derby), All Rights Reserved.
//
//  Vegetation template editing
//
//////////////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_BOTANICALS

#include "Core/Asura_UserInput.h"

#include "Tools_Mode_Botanicals.h"
#include "Tools_Botanicals_Object.h"
#include "Tools_Overlays.h"
#include "PC/FileSystem/Tools_FileSystem.h"

#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"
#include "Botanicals/Asura_Botanicals_TemplateManager.h"
#include "Botanicals/Asura_Botanicals_InstanceManager.h"

#include "Botanicals/Tools_Botanicals_Styles.h"

#include "Shapes/Tools_Shapes.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"

#include "Interface/Asura_Interface.h"
#include "Interface/Asura_CommandConsole.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"



Asura_Vector_2 Tools_Mode_Botanicals::s_xEditTemplatesOverlayPosition = Asura_Vector_2(25,10);
Asura_Vector_2 Tools_Mode_Botanicals::s_xTemplateOverlayPosition	  = Asura_Vector_2(50,20);
char		 **Tools_Mode_Botanicals::s_pszTemplateNames			  = NULL;
int			   Tools_Mode_Botanicals::s_iNumTemplatesInList           = 0;
int			   Tools_Mode_Botanicals::s_iTemplateListOffset			  = 0;
int            Tools_Mode_Botanicals::s_iSelectedTemplateInList		  = 0;
int			   Tools_Mode_Botanicals::s_iTemplateBeingEdited		  = 0;

char		 **Tools_Mode_Botanicals::s_pszStyleNames				  = NULL;
int			   Tools_Mode_Botanicals::s_iNumStylesInList		      = 0;
int			   Tools_Mode_Botanicals::s_iStyleListOffset			  = 0;
int            Tools_Mode_Botanicals::s_iSelectedStyleInList		  = 0;
Asura_Dialog  *Tools_Mode_Botanicals::s_pxTemplateCountDialog		  = 0;
bool           Tools_Mode_Botanicals::s_bSelectedTemplateChanged	  = false;

static Asura_List_Dialog*    g_pxTemplateList = NULL;
static Asura_List_Dialog*    g_pxStyleList    = NULL;
static Asura_String			 g_xTemplateNameString=Asura_String("string");



void Tools_Mode_Botanicals::CreateEditTemplatesOverlay(void)
{
	// If it's already open, don't create another
	if(s_bEditTemplatesOverlayOpen) return;
	s_bEditTemplatesOverlayOpen=true;

	Asura_Overlay* pxOverlay;	
	pxOverlay = new Asura_Overlay("Edit Botanical Templates/Styles",false,&s_xEditTemplatesOverlayPosition);

	Asura_Vector_2 vOffset = Asura_Vector_2(1.0f,3.0f);
	Asura_Dialog*  pxDialog;

	// Rebuild list of pointers to template names, as the list dialog
	// needs them in an array
	int nNumTemplateItems=RebuildTemplateList();
	int nNumStyleItems=RebuildStyleList();

	// 'Templates' title
	pxDialog=new Asura_Dialog("Templates",vOffset);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	// Template List
	Asura_List_Dialog* pxList = new Asura_List_Dialog(vOffset,s_pszTemplateNames,&s_iTemplateListOffset,&s_iSelectedTemplateInList,8,nNumTemplateItems,NULL);
	pxList->SetExpandWidthToFillOverlay(false);
	pxList->SetMinWidth(16.0f);
	pxDialog = pxList;	
	g_pxTemplateList=pxList;

	pxOverlay->AddDialog(pxDialog);
	vOffset.y += 8.0f+(2.0f*Asura_Dialog::DefaultSpacing());


	// Number in use
	s_pxTemplateCountDialog = new Asura_Dialog("# in use",vOffset);
	pxOverlay->AddDialog(s_pxTemplateCountDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	UpdateTemplateInstanceCount();
	

	// Create
	pxDialog = new Asura_Function_Dialog("Create Template",vOffset,CreateTemplate);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	// Edit
	pxDialog = new Asura_Function_Dialog("Edit Template",vOffset,EditTemplate);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	// Delete  (needs to remap all instances!)
	pxDialog = new Asura_Function_Dialog("Delete Template",vOffset,DeleteTemplate);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	// Toggle hidden flag
	pxDialog = new Asura_Function_Dialog("Hide/Unhide Template",vOffset,ToggleTemplateHidden);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Invert Hidden",vOffset,InvertTemplatesHidden);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Function_Dialog("Unhide All Templates",vOffset,UnhideAllTemplates);
	pxOverlay->AddDialog(pxDialog);
//	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();


	// Close
	vOffset.x=20.0f;
	pxDialog = new Asura_Function_Dialog("Close",vOffset,ToggleEditTemplatesOverlay);
	pxOverlay->AddDialog(pxDialog);
	


	// 'Styles' title
	vOffset = Asura_Vector_2(20.0f,3.0f);
	pxDialog=new Asura_Dialog("Styles",vOffset);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	// Style List	
	pxList = new Asura_List_Dialog(vOffset,s_pszStyleNames,&s_iStyleListOffset,&s_iSelectedStyleInList,8,nNumStyleItems,NULL);
//	pxList->SetBorder(true,18.0f);
	pxDialog = pxList;	
	g_pxStyleList=pxList;
	pxOverlay->AddDialog(pxDialog);
	vOffset.y += 9.5f+(2.0f*Asura_Dialog::DefaultSpacing());
	

	// Create Style 
	pxDialog = new Asura_Function_Dialog("Create Style",vOffset,CreateStyle);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	// Edit Style 
	pxDialog = new Asura_Function_Dialog("Edit Style",vOffset,EditStyle);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	// Delete Style 
	pxDialog = new Asura_Function_Dialog("Delete Style",vOffset,DeleteStyle);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();


	pxOverlay->m_uID = TOOLS_OVERLAY_BOTANICALS_EDITTEMPLATES;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}



void Tools_Mode_Botanicals::UpdateTemplateInstanceCount(void)
{
	static char s_szText[32];

	// Count the instances that exist as tool objects, and those
	// in the instancemanager
	int iTemplate = s_iSelectedTemplateInList+s_iTemplateListOffset;
	int iCount = Asura_Botanicals_InstanceManager::GetNumInstancesOfTemplate(iTemplate);
	int iTotal = Asura_Botanicals_InstanceManager::GetNumInstances();

	// Find all botanicals tool_objects
	for(LIF<Tools_Object*> olif(&Tools_Object_System::ObjectList);!olif.done();olif.next())
	{
		if (olif()->Type == TOT_Botanicals)
		{	
			Tools_Botanicals_Object *pxToolsObject = (Tools_Botanicals_Object *)olif();

			if (pxToolsObject->GetTemplateIdx()==iTemplate)
			{
				iCount++;
			}			

			iTotal++;
		}
	}


	Asura_PrintfToBfr(s_szText,"Instances: %d / %d",32,true,iCount, iTotal);
	s_pxTemplateCountDialog->UpdateLabel(s_szText);	
}


void Tools_Mode_Botanicals::FreeTemplateNameStrings(void)
{	
	if (s_pszTemplateNames!=NULL)
	{
		for(int n=0;n<s_iNumTemplatesInList;n++)
		{
			delete[] s_pszTemplateNames[n];
			s_pszTemplateNames[n] = NULL;
		}

		delete [] s_pszTemplateNames;
		s_pszTemplateNames = 0;
	}
}


int Tools_Mode_Botanicals::RebuildTemplateList()
{
	// This has become a bit of a hacky mess, as the UI want's a list of 
	// char* strings, but everything else prefers Asura_Strings.....
	int iNumItems=Asura_Botanicals_TemplateManager::GetNumTemplates();

	FreeTemplateNameStrings();	

	int iCount=iNumItems;
	if (iCount<1) iCount=1;	 // zero-length allocs == bad

	s_iNumTemplatesInList=iNumItems;
	s_pszTemplateNames=new char*[iCount];	

	for(int n=0;n<iNumItems;n++)
	{
		s_pszTemplateNames[n]=new char[64];		
		

		if (Asura_Botanicals_TemplateManager::GetTemplateVisible(n))
		{		
			Asura_PrintfToBfr(s_pszTemplateNames[n],"%s",64,true,Asura_Botanicals_TemplateManager::GetTemplateName(n));
		}
		else
		{		
			Asura_PrintfToBfr(s_pszTemplateNames[n],"(Hidden) %s",64,true,Asura_Botanicals_TemplateManager::GetTemplateName(n));
		}		
	}

	return iNumItems;
}



void Tools_Mode_Botanicals::FreeStyleNameStrings(void)
{	
	if (s_pszStyleNames!=NULL)
	{
		for(int n=0;n<s_iNumStylesInList;n++)
		{
			delete[] s_pszStyleNames[n];
			s_pszStyleNames[n] = 0;
		}

		delete [] s_pszStyleNames;
		s_pszStyleNames = 0;
	}
}

int Tools_Mode_Botanicals::RebuildStyleList()
{
	// Same again, this time for the style list
	int iNumItems=Tools_Botanicals_StyleManager::GetNumStyles();
	
	FreeStyleNameStrings();
	
	int iCount=iNumItems;
	if (iCount<1) iCount=1;	 // zero-length allocs == bad

	s_pszStyleNames=new char*[iCount];
	s_iNumStylesInList=iNumItems;

	for(int n=0;n<iNumItems;n++)
	{
		s_pszStyleNames[n]=new char[32];		
		Asura_PrintfToBfr(s_pszStyleNames[n],"%s",32,true,Tools_Botanicals_StyleManager::GetStyleName(n));
	}

	return iNumItems;
}




void Tools_Mode_Botanicals::CreateTemplateOverlay(void)
{
	// If it's already open, don't create another
	if(s_bTemplateOverlayOpen) return;
	s_bTemplateOverlayOpen=true;	

	Asura_Overlay* pxOverlay;	

	ASURA_BOTANICALS_TEMPLATE_TYPE eType = Asura_Botanicals_TemplateManager::GetTemplate(s_iTemplateBeingEdited)->eType;
	
	pxOverlay = new Asura_Overlay("Botanical Template",false,&s_xTemplateOverlayPosition);
	
	Asura_Vector_2 vOffset = Asura_Vector_2(1.0f,3.0f);
	Asura_Dialog *pxDialog;
	
	Asura_Botanicals_Template *pxTemplate=Asura_Botanicals_TemplateManager::GetTemplate(s_iTemplateBeingEdited);

	// Name
	g_xTemplateNameString=pxTemplate->xTemplateName;
	pxDialog= new Asura_String_Dialog("Name",vOffset,&g_xTemplateNameString,31);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	// Placement params - Random rotation/scale control
	pxDialog = new Asura_Dialog("Placement",vOffset);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Random Y rotation - Min",vOffset,&pxTemplate->fMinYRotation,-360,360,1,"%.2f");
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Random Y rotation - Max",vOffset,&pxTemplate->fMaxYRotation,-360,360,1,"%.2f");
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Random scale factor - Min",vOffset,&pxTemplate->fMinScale,0.5f,2.0f,0.01f,"%.2f");
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Random scale factor - Max",vOffset,&pxTemplate->fMaxScale,0.5f,2.0f,0.01f,"%.2f");
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	// Overlap test control
	pxDialog = new Asura_Dialog("Overlap Test",vOffset);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Collision sphere radius",vOffset,&pxTemplate->fCollisionRadius,0,50,0.1f,"%.2f");
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Collision sphere height",vOffset,&pxTemplate->fCollisionHeight,0,50,0.1f,"%.2f");
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Dialog("Merging",vOffset);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Merge Render Mesh",vOffset,&pxTemplate->bMergeRenderMeshWithEnv);
	pxDialog->SetHelpText("This will cause the render mesh of any instances of this template to be merged in to the environment mesh");
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Merge Collision Mesh",vOffset,&pxTemplate->bMergeCollisionMeshWithEnv);
	pxDialog->SetHelpText("This will cause the collision mesh of any instances of this template to be merged in to the environment mesh");
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Permanently Merge Mesh",vOffset,&pxTemplate->bPermanentlyMergeMesh);
	pxDialog->SetHelpText("This will cause any instances of this template to be permanently merged into the environment once 'Commit permanently merged templates' is used.\nAny such instances will be deleted once the merging has been done.");
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();



	// Items specific to Shapes
	if (eType==ASURA_BOTANICALS_TEMPLATE_SHAPE)
	{
		pxDialog = new Asura_Dialog("Shape",vOffset);
		pxOverlay->AddDialog(pxDialog);
		vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

		char szText[256];
		Asura_PrintfToBfr( szText, "Select Shape [%s]", 256, true, pxTemplate->xShapeName.GetString() );

		pxDialog = new Asura_Function_Dialog(szText,vOffset,ChangeShape);
		pxOverlay->AddDialog(pxDialog);		
	}

	// Items specific to BotanicalShapes
	if (eType==ASURA_BOTANICALS_TEMPLATE_BOTANICALSHAPE)
	{
		pxDialog = new Asura_Dialog("BotanicalShape",vOffset);
		pxOverlay->AddDialog(pxDialog);
		vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

		char szText[256];
		Asura_PrintfToBfr( szText, "Select BotanicalShape [%s]", 256, true,
						   pxTemplate->xShapeName.IsBlank() ? "None" : pxTemplate->xShapeName.GetString() );

		pxDialog = new Asura_Function_Dialog(szText,vOffset,ChangeBotanicalShape);
		pxOverlay->AddDialog(pxDialog);		
		vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog( "Remove BotanicalShape",vOffset,RemoveBotanicalShape);
		pxOverlay->AddDialog(pxDialog);		
		vOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
		
		Asura_PrintfToBfr( szText, "Select Selection Shape (Optional) [%s]", 256, true,
						   pxTemplate->xSelectionShapeName.IsBlank() ? "None" : pxTemplate->xSelectionShapeName.GetString() );

		pxDialog = new Asura_Function_Dialog(szText,vOffset,ChangeSelectionShape);
		pxOverlay->AddDialog(pxDialog);	
		vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();
			
		pxDialog = new Asura_Function_Dialog( "Remove Selection Shape",vOffset,RemoveSelectionShape);
		pxOverlay->AddDialog(pxDialog);		
	}


	vOffset.y+=2.0f*Asura_Dialog::DefaultSpacing();

	// Close	
	pxDialog = new Asura_Function_Dialog("Close",vOffset,ToggleTemplateOverlay);
	pxOverlay->AddDialog(pxDialog);
	vOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	pxOverlay->m_uID = TOOLS_OVERLAY_BOTANICALS_TEMPLATE;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
	
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	Asura_Direct3D_Botanicals_Game_Render::UpdateTypes();
#endif
}



void Tools_Mode_Botanicals::ChangeShape(void)
{
	Asura_Botanicals_Template *pTemplate = Asura_Botanicals_TemplateManager::GetTemplate(s_iTemplateBeingEdited);

	pTemplate->uShapeID = Tools_ShapeTable::SelectShape(pTemplate->uShapeID);

	// Also update the name
	if (pTemplate->uShapeID !=ASURA_HASH_ID_UNSET)
	{			
		pTemplate->xShapeName=Asura_HashedFilename_System::GetFilename(pTemplate->uShapeID,false);		
	}
	else
	{
		pTemplate->xShapeName="";
	}

	Asura_Botanicals_TemplateManager::LoadResources();
	
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	Asura_Direct3D_Botanicals_Game_Render::UpdateTypes();
	Asura_Botanicals_InstanceManager::ConnectToolObjectsToBGR();
#endif

	// Recreate the template properties overlay to update for the name change
	if( s_bTemplateOverlayOpen )
	{
		ToggleTemplateOverlay();
		ToggleTemplateOverlay();
	}
}


void Tools_Mode_Botanicals::RemoveSelectionShape(void)
{
	Asura_Botanicals_Template *pTemplate = Asura_Botanicals_TemplateManager::GetTemplate(s_iTemplateBeingEdited);

	pTemplate->uSelectionShapeID = ASURA_HASH_ID_UNSET;
	pTemplate->xSelectionShapeName = "";

	Asura_Botanicals_TemplateManager::LoadResources();
	
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	Asura_Direct3D_Botanicals_Game_Render::UpdateTypes();
	Asura_Botanicals_InstanceManager::ConnectToolObjectsToBGR();
#endif

	// Recreate the template properties overlay to update for the name change
	if( s_bTemplateOverlayOpen )
	{
		ToggleTemplateOverlay();
		ToggleTemplateOverlay();
	}
}

void Tools_Mode_Botanicals::ChangeSelectionShape(void)
{
	Asura_Botanicals_Template *pTemplate = Asura_Botanicals_TemplateManager::GetTemplate(s_iTemplateBeingEdited);

	pTemplate->uSelectionShapeID = Tools_ShapeTable::SelectShape(pTemplate->uSelectionShapeID);

	// Also update the name
	if (pTemplate->uSelectionShapeID !=ASURA_HASH_ID_UNSET)
	{			
		pTemplate->xSelectionShapeName=Asura_HashedFilename_System::GetFilename(pTemplate->uSelectionShapeID,false);		
	}
	else
	{
		pTemplate->xSelectionShapeName="";
	}

	Asura_Botanicals_TemplateManager::LoadResources();
	
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	Asura_Direct3D_Botanicals_Game_Render::UpdateTypes();
	Asura_Botanicals_InstanceManager::ConnectToolObjectsToBGR();
#endif

	// Recreate the template properties overlay to update for the name change
	if( s_bTemplateOverlayOpen )
	{
		ToggleTemplateOverlay();
		ToggleTemplateOverlay();
	}
}


void Tools_Mode_Botanicals::RemoveBotanicalShape(void)
{
	Asura_Botanicals_Template *pTemplate = Asura_Botanicals_TemplateManager::GetTemplate(s_iTemplateBeingEdited);
	
	if( NULL == pTemplate)
	{
		return;
	}
	
	pTemplate->xShapeName="";

	// And load it
	Asura_Botanicals_TemplateManager::LoadResources();
	
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	Asura_Direct3D_Botanicals_Game_Render::UpdateTypes();
	Asura_Botanicals_InstanceManager::ConnectToolObjectsToBGR();
#endif

	// Recreate the template properties overlay to update for the name change
	if( s_bTemplateOverlayOpen )
	{
		ToggleTemplateOverlay();
		ToggleTemplateOverlay();
	}
}

void Tools_Mode_Botanicals::ChangeBotanicalShape(void)
{
	Asura_Botanicals_Template *pTemplate = Asura_Botanicals_TemplateManager::GetTemplate(s_iTemplateBeingEdited);
	
	// Select a *.btd with a file requester
	Tools_FileSystem_DialogInfo xInfo;
	xInfo.DefaultExtension = "";
	xInfo.DefaultFilename  = pTemplate->xShapeName.GetString(); 
	xInfo.DefaultPath = "Objects\\";
	xInfo.FileFilter = "BotanicalShape Files\0*.btd\0"; 
	xInfo.WindowTitle = "Load Shape...";
	xInfo.AllowMultipleSelect = false;

	if (!xInfo.DefaultFilename) xInfo.DefaultFilename = "";

	// Get the name, and strip the extension
	Asura_String name = Tools_FileSystem::GetLoadFilename(xInfo);

	if(name.IsNull())
	{
		// Cancelled
		return;
	}

	pTemplate->xShapeName=name;

	// And load it
	Asura_Botanicals_TemplateManager::LoadResources();
	
#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
	Asura_Direct3D_Botanicals_Game_Render::UpdateTypes();
	Asura_Botanicals_InstanceManager::ConnectToolObjectsToBGR();
#endif

	// Recreate the template properties overlay to update for the name change
	if( s_bTemplateOverlayOpen )
	{
		ToggleTemplateOverlay();
		ToggleTemplateOverlay();
	}
}



void Tools_Mode_Botanicals::UpdateTemplateEditing()
{		
	if (s_bTemplateOverlayOpen)
	{
		Asura_Botanicals_TemplateManager::SetTemplateName(
			Asura_Botanicals_TemplateManager::GetTemplate(s_iTemplateBeingEdited),g_xTemplateNameString.GetString());

		if (s_bEditTemplatesOverlayOpen)
		{		
			int nCount=RebuildTemplateList();
			g_pxTemplateList->ReplaceStrings(nCount,s_pszTemplateNames);
		}			
	}


	// Process style editing mode
	if (Tools_Botanicals_StyleManager::UpdateEditing())
	{
		// If something's changed, we need to update the list
		if (s_bEditTemplatesOverlayOpen)
		{		
			int nCount=RebuildStyleList();
			g_pxStyleList->ReplaceStrings(nCount,s_pszStyleNames);
		}
	}

	if (s_bEditTemplatesOverlayOpen)
	{
		int nCount=RebuildStyleList();
		g_pxStyleList->ReplaceStrings(nCount,s_pszStyleNames);

		UpdateTemplateInstanceCount();		
	}

}


void Tools_Mode_Botanicals::EditTemplate(void)
{
	Tools_Mode_Botanicals::SetModifiedFlag(true);

	// If the overlay is already open, close it so we can recreate it
	if(s_bTemplateOverlayOpen) 	
		ToggleTemplateOverlay();
	
	// No templates - nothing to edit
	if (Asura_Botanicals_TemplateManager::GetNumTemplates()<1)
	{
		Tools_Timed_Message_System::AddMessage(0xff8000, "No Template Selected");
		return;
	}

	// Store the index of the template we're working on
	s_iTemplateBeingEdited=s_iSelectedTemplateInList+s_iTemplateListOffset;

	// Create it
	ToggleTemplateOverlay();
}


void Tools_Mode_Botanicals::ToggleTemplateHidden(void)
{
	// No templates - nothing to edit
	if (Asura_Botanicals_TemplateManager::GetNumTemplates()<1)
	{
		Tools_Timed_Message_System::AddMessage(0xff8000, "No Template Selected");
		return;
	}

	// Store the index of the template we're working on
	int iTemplateIdx=s_iSelectedTemplateInList+s_iTemplateListOffset;

	Asura_Botanicals_TemplateManager::ToggleTemplateVisible(iTemplateIdx);

	// Recreate the list overlay, to refresh the list	
	ToggleEditTemplatesOverlay();
	ToggleEditTemplatesOverlay();
}


void Tools_Mode_Botanicals::InvertTemplatesHidden(void)
{
	for(int i=0; i<Asura_Botanicals_TemplateManager::GetNumTemplates();i++)
	{
		Asura_Botanicals_TemplateManager::ToggleTemplateVisible(i);
	}	

	// Recreate the list overlay, to refresh the list	
	ToggleEditTemplatesOverlay();
	ToggleEditTemplatesOverlay();
}


void Tools_Mode_Botanicals::UnhideAllTemplates(void)
{
	for(int i=0; i<Asura_Botanicals_TemplateManager::GetNumTemplates();i++)
	{
		if (!Asura_Botanicals_TemplateManager::GetTemplateVisible(i))
		{
			Asura_Botanicals_TemplateManager::ToggleTemplateVisible(i);
		}		
	}	

	// Recreate the list overlay, to refresh the list	
	ToggleEditTemplatesOverlay();
	ToggleEditTemplatesOverlay();
}

void Tools_Mode_Botanicals::CreateTemplate(void)
{
	Tools_Mode_Botanicals::SetModifiedFlag(true);

	// Create the template
	// Select it (update the details dialog)
	// If the details window isn't visible, show it
	int nIdx=0;
 

	ListSelection_Overlay xOverlay("Select Template Type:");
	xOverlay.AddItem("BotanicalShape (with LOD)",0);	
	xOverlay.AddItem("Shape (no LOD)",1);		
	if (!xOverlay.DoModal()) return;
	int iType=xOverlay.GetSelectionData();	
	
	nIdx=Asura_Botanicals_TemplateManager::CreateTemplate((ASURA_BOTANICALS_TEMPLATE_TYPE)iType);

	// Recreate the list overlay, to refresh the list with the
	// newly created item
	ToggleEditTemplatesOverlay();
	ToggleEditTemplatesOverlay();

	// Select it
	s_iSelectedTemplateInList=nIdx;
	s_iTemplateListOffset=0;

	// Open the edit window
	EditTemplate();
}




void Tools_Mode_Botanicals::DeleteTemplate(void)
{
	Tools_Mode_Botanicals::SetModifiedFlag(true);

	// This will be a bit nasty... need to remap all instances in the instance manager, and remove any
	// instances of this type
	//
	// Template deletion checklist
	// - Remap all instances
	// - Remap all styles
	// - Clear paint selection
	// - Hide 'edit template' if it's >= the one we're deleting


	if (Asura_Botanicals_TemplateManager::GetNumTemplates()==0) 
	{
		return;
	}

	// Ask for confirmation first...
	YesNo_Overlay xDlg("Delete this template? All instances of it will be deleted!");
	if (!xDlg.DoModal())
	{
		return;
	}


	int nIdxToDelete=s_iSelectedTemplateInList+s_iTemplateListOffset;
	
	if (s_bTemplateOverlayOpen)
	{
		if (s_iTemplateBeingEdited>=nIdxToDelete)
		{		
			ToggleTemplateOverlay();
		}
	}
	
	
	ToggleEditTemplatesOverlay();

	// Deselect whatever we're painting with
	UpdateSelectedPaintStyle(-1,-1);
	
	Asura_Botanicals_InstanceManager::ConvertFromToolObjects();
	
	Asura_Botanicals_TemplateManager::RemoveTemplate(nIdxToDelete);
	
	Asura_Botanicals_InstanceManager::ConvertToToolObjects();

	Tools_Botanicals_StyleManager::RemapStyles();

	ToggleEditTemplatesOverlay();
	
}


void Tools_Mode_Botanicals::CreateStyle(void)
{
	Tools_Mode_Botanicals::SetModifiedFlag(true);
	Tools_Botanicals_StyleManager::AddStyle();
}

void Tools_Mode_Botanicals::DeleteStyle(void)
{
	Tools_Mode_Botanicals::SetModifiedFlag(true);
	// Make sure there's something to delete
	if (Tools_Botanicals_StyleManager::GetNumStyles()==0) 
	{
		return;
	}

	// Ask for confirmation first...
	YesNo_Overlay xDlg("Delete this style?\n\nAre you sure?");
	if (!xDlg.DoModal())
	{
		return;
	}

	// If we're editing it, close the editor
	Tools_Botanicals_StyleManager::CloseEditStyleOverlay();
	
	// Deselect whatever we're painting with
	UpdateSelectedPaintStyle(-1,-1);		

	// Remove it
	Tools_Botanicals_StyleManager::RemoveStyle(s_iSelectedStyleInList+s_iStyleListOffset);
}




void Tools_Mode_Botanicals::EditStyle(void)
{
	Tools_Mode_Botanicals::SetModifiedFlag(true);
	if (Tools_Botanicals_StyleManager::GetNumStyles()==0) 
	{
		return;
	}

	Tools_Botanicals_StyleManager::OpenEditStyleOverlay(s_iSelectedStyleInList+s_iStyleListOffset);
}

#endif // ASURA_USE_BOTANICALS
