#include "Core/Asura.h"

#if defined( ASURA_USE_BOTANICALS) && defined( ASURA_TOOLS)

// main header
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"

// other libraries
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance_Lighting.h"
#include "Botanicals/Asura_Botanical_Game_Render_Instance_Transform.h"
#include "Botanicals/Asura_Botanicals_InstanceManager.h"
#include "Botanicals/Asura_Botanicals_TemplateManager.h"
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	#include "Botanicals/Asura_Botanical_Game_Render_Type_BasicInteractive.h"
#endif

#include "Core/Asura_LinkList.h"
#include "Core/Asura_MaterialResponse_System.h"

#include "Debugging/Asura_Debug_Timers.h"

#include "FileAccess/Asura_FileAccess.h"

#include "Mesh/Asura_Mesh.h"
#include "MeshProcessing/Tools_TangentSpace.h"

#include "Rendering/Asura_Module.h"

// internal dependencies
#include "Light/Asura_GI_Solution.h"
#include "Light/Asura_Lighting_Solution.h"

// tools
#include "Misc/Tools_MessageLog.h"

//*******************************************************************
// Local Classes
//*******************************************************************

struct Asura_Direct3D_BGR_MDS_ToolShapeType
{
public:
	Tools_BotanicalShape* m_pxToolShape;
	Asura_BGR_TypeID m_xTypeID;
	
	bool operator==( const Asura_Direct3D_BGR_MDS_ToolShapeType& xRHS) const;
};

struct Asura_Direct3D_BGR_MDS_InstanceType
{
public:
	Asura_Botanicals_Instance* m_pxInstance;
	Tools_BotanicalShape* m_pxToolShape;
	Asura_BGR_TypeID m_xTypeID;
};

struct Asura_Direct3D_BGR_MDS_BillboardDetails
{
public:
	float m_fWidth;
	float m_fHeight;
//	float m_afTextcoordMin[ 2];
//	float m_afTextcoordMax[ 2];
	Asura_AVector3 m_xPositionCenter;
	Asura_AVector3 m_axPositions[4];
	float m_afTextcoord[ 4][ 2];
	uint32 m_auColour[4];
};

//*******************************************************************
// Function Definitions
//*******************************************************************

#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
bool Asura_Direct3D_BGR_Material::SetMaterialResponseHashID( const Asura_Hash_ID xMaterialResponseHashID)
{
	this->m_xMaterialResponseHashID = xMaterialResponseHashID;
	
	return( true);
}
#endif

#if !defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
bool Asura_Direct3D_BGR_Material::SetMaterial( const int iMaterialID)
{
	this->m_iDiffuseTexture0 = Asura_Material_System::GetTextureID( iMaterialID, 0);
	this->m_bDiffuseDualPassTexture = Asura_Material_System::IsMaterialDualTexture( iMaterialID);
	this->m_iDiffuseTexture1 = (true == this->m_bDiffuseDualPassTexture) ? Asura_Material_System::GetTextureID( iMaterialID, 1) : ASURA_TEXTURE_NONE;

	this->m_iSpecularTexture0 = Asura_Material_System::GetSpecularMapID( iMaterialID, 0);
	this->m_bSpecularDualPassTexture = Asura_Material_System::IsMaterialDualSpecularMap( iMaterialID);
	this->m_iSpecularTexture1 = (true == this->m_bSpecularDualPassTexture) ? Asura_Material_System::GetSpecularMapID( iMaterialID, 1) : ASURA_TEXTURE_NONE;

	this->m_iNormalTexture0 = Asura_Material_System::GetNormalMapID( iMaterialID, 0);
	this->m_bHasNormalMaps = (this->m_iNormalTexture0 != ASURA_TEXTURE_NONE);
	this->m_iNormalTexture1 = (true == this->m_bHasNormalMaps) ? Asura_Material_System::GetNormalMapID( iMaterialID, 1) : ASURA_TEXTURE_NONE;
	this->m_bNormalDualPassTexture = (true == this->m_bHasNormalMaps) ? Asura_Material_System::IsMaterialDualNormalMap( iMaterialID) : false;

	this->m_bParallaxMap = Asura_Material_System::IsMaterialParallaxMapping( iMaterialID);
	this->m_bSelfIlluminating = Asura_Material_System::IsMaterialSelfIlluminating( iMaterialID);
	this->m_bSpecular = Asura_Material_System::IsMaterialSpecularSharp( iMaterialID) || Asura_Material_System::IsMaterialSpecularSoft( iMaterialID);
	
	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
static void Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_CalculateTangentSpaceVectors(const class Asura_Mesh* pxMesh, u_int uPoly, Asura_Vector_3& xTangent, Asura_Vector_3& xBinormal)
{
	Asura_Vector_3	axPositions [3];
	Asura_Vector_2	axTextureCoords [3];

	// Initialise the vertex-data...

	for (u_int u = 0; u < 3; u++)
	{
		u_int	uVertexID	= pxMesh->PolygonDataPtr [uPoly].VertexID [u];


		// Fill in the data for this vertex...

		axPositions [u]			= pxMesh->VertexDataPtr [uVertexID].Position;
		axTextureCoords [u].x	= pxMesh->TextureDataPtr [uPoly].U [0][u];
		axTextureCoords [u].y	= 1.0f - pxMesh->TextureDataPtr [uPoly].V [0][u];
	}

	// Calculate the tangent AND binormal...

	Tools_TangentSpace::CalculateTangentAndBinormal	(axPositions, axTextureCoords, xTangent, xBinormal);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
static void Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_CalculateTangentSpaceVectors( const float afCornerScale[ 4][ 2], const float afTextcoordScale[ 4][ 2], Asura_Vector_3& xTangent, Asura_Vector_3& xBinormal)
{
	Asura_Vector_3	axPositions [3];
	Asura_Vector_2	axTextureCoords [3];

	// Initialise the vertex-data...

	for (u_int u = 0; u < 3; u++)
	{
		axPositions[u].x = afCornerScale[ u][ 0];
		axPositions[u].y = afCornerScale[ u][ 1];
		axPositions[u].z = 0;
		axTextureCoords [u].x = afTextcoordScale[ u][ 0];
		axTextureCoords [u].y = afTextcoordScale[ u][ 1];
	}

	// Calculate the tangent AND binormal...

	Tools_TangentSpace::CalculateTangentAndBinormal	(axPositions, axTextureCoords, xTangent, xBinormal);
}
#endif

u_int Asura_Direct3D_BGR_Manager::GetModuleIndexFromPosition( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation)
{
	int iModuleIndex = Asura_Botanicals_InstanceManager::GetModuleIndexFromPosition( xPosition, xOrientation);

	if( iModuleIndex < 0)
	{
		iModuleIndex = Asura_Module_System::GetNumberOfModules();
	}
	else
	{
#if defined( ASURA_ASSERT)
		u_int uModuleCount = Asura_Module_System::GetNumberOfModules();
		Asura_Assert( static_cast< u_int>( iModuleIndex) < uModuleCount, "Module is out of range");
#endif
	}
	
	return( static_cast< u_int>( iModuleIndex) );
}

bool Asura_Direct3D_BGR_MDS_ToolShapeType::operator==( const Asura_Direct3D_BGR_MDS_ToolShapeType& xRHS) const
{
	return( this->m_pxToolShape == xRHS.m_pxToolShape);
}

void Asura_BGR_Instance::SetInstanceData( const Asura_BGR_InstanceLighting& xInstanceLighting)
{
	#if !defined( ASURA_DIRECT3D_BGR_INSTANCE_ENABLE_COMPRESSED_DATA)
		this->m_afSphericalR[0] = xInstanceLighting.m_afSphericalR[ 0];
		this->m_afSphericalR[1] = xInstanceLighting.m_afSphericalR[ 1];
		this->m_afSphericalR[2] = xInstanceLighting.m_afSphericalR[ 2];
		this->m_afSphericalR[3] = xInstanceLighting.m_afSphericalR[ 3];
		
		this->m_afSphericalG[0] = xInstanceLighting.m_afSphericalG[ 0];
		this->m_afSphericalG[1] = xInstanceLighting.m_afSphericalG[ 1];
		this->m_afSphericalG[2] = xInstanceLighting.m_afSphericalG[ 2];
		this->m_afSphericalG[3] = xInstanceLighting.m_afSphericalG[ 3];
		
		this->m_afSphericalB[0] = xInstanceLighting.m_afSphericalB[ 0];
		this->m_afSphericalB[1] = xInstanceLighting.m_afSphericalB[ 1];
		this->m_afSphericalB[2] = xInstanceLighting.m_afSphericalB[ 2];
		this->m_afSphericalB[3] = xInstanceLighting.m_afSphericalB[ 3];
	#else

		float fMin = Asura_Maths::LargeFloat, fMax = -Asura_Maths::LargeFloat;

		for( u_int u = 0; u < 4; ++u )
		{
			fMin = Asura_Maths::Min( fMin, xInstanceLighting.m_afSphericalR[u] );
			fMin = Asura_Maths::Min( fMin, xInstanceLighting.m_afSphericalG[u] );
			fMin = Asura_Maths::Min( fMin, xInstanceLighting.m_afSphericalB[u] );

			fMax = Asura_Maths::Max( fMax, xInstanceLighting.m_afSphericalR[u] );
			fMax = Asura_Maths::Max( fMax, xInstanceLighting.m_afSphericalG[u] );
			fMax = Asura_Maths::Max( fMax, xInstanceLighting.m_afSphericalB[u] );
		}

		const float fRange = fMax - fMin;
		const float fRangeRcp = (fRange > 0.0f ? 255.0f / fRange : 0.0f);

		for( u_int u = 0; u < 4; ++u )
		{
			float fResult = (xInstanceLighting.m_afSphericalR[u] - fMin) * fRangeRcp;
			m_auSphericalR[u] = static_cast< uint8 >( Asura_Maths::FloatToInt( fResult ) );

			fResult = (xInstanceLighting.m_afSphericalG[u] - fMin) * fRangeRcp;
			m_auSphericalG[u] = static_cast< uint8 >( Asura_Maths::FloatToInt( fResult ) );

			fResult = (xInstanceLighting.m_afSphericalB[u] - fMin) * fRangeRcp;
			m_auSphericalB[u] = static_cast< uint8 >( Asura_Maths::FloatToInt( fResult ) );
		}

		m_fDecompSHScale = fRange / 255.0f;
		m_fDecompSHBias = fMin;

	#endif
}

void Asura_BGR_Instance::SetInstanceData( const Asura_BGR_InstanceTransform& xInstanceTransform)
{
#if !defined( ASURA_DIRECT3D_BGR_INSTANCE_ENABLE_COMPRESSED_DATA)
	this->m_xPosition = xInstanceTransform.m_xPosition;
#else
	this->m_afPosition[0] = xInstanceTransform.m_xPosition.GetX();
	this->m_afPosition[1] = xInstanceTransform.m_xPosition.GetY();
	this->m_afPosition[2] = xInstanceTransform.m_xPosition.GetZ();
#endif
	
	{
		#if defined( ASURA_ASSERT )
			const float fMatrixMax3x3Value = ASURA_DIRECT3D_BGR_INSTANCE_SCALE_MAX;
			float fScaleMax = 0;
			
			fScaleMax = Asura_Maths::Max( fScaleMax, Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._11));
			fScaleMax = Asura_Maths::Max( fScaleMax, Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._12));
			fScaleMax = Asura_Maths::Max( fScaleMax, Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._13));
			
			fScaleMax = Asura_Maths::Max( fScaleMax, Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._21));
			fScaleMax = Asura_Maths::Max( fScaleMax, Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._22));
			fScaleMax = Asura_Maths::Max( fScaleMax, Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._23));
			
			fScaleMax = Asura_Maths::Max( fScaleMax, Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._31));
			fScaleMax = Asura_Maths::Max( fScaleMax, Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._32));
			fScaleMax = Asura_Maths::Max( fScaleMax, Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._33));
		#endif

		// DLH - 11 July 2006 11:17:02 - StuartM wanted this assert so the problem assert is classable as human readable.
		Asura_Assertf( fScaleMax <= fMatrixMax3x3Value, ("The botanical at poition X %f Y %f Z %f is scaled geater than %f times. Reduce to less than %f times scale.", xInstanceTransform.m_xPosition.GetX(), xInstanceTransform.m_xPosition.GetY(), xInstanceTransform.m_xPosition.GetZ(), fMatrixMax3x3Value, fMatrixMax3x3Value));
		
		Asura_Assertf( Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._11) <= fMatrixMax3x3Value, ("Rotation/Scale Value %f is too large. Max Value of %f. Instance at X %f Y %f Z %f", xInstanceTransform.m_xTransformMatrix._11, fMatrixMax3x3Value, xInstanceTransform.m_xPosition.GetX(), xInstanceTransform.m_xPosition.GetY(), xInstanceTransform.m_xPosition.GetZ()));
		Asura_Assertf( Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._21) <= fMatrixMax3x3Value, ("Rotation/Scale Value %f is too large. Max Value of %f. Instance at X %f Y %f Z %f", xInstanceTransform.m_xTransformMatrix._21, fMatrixMax3x3Value, xInstanceTransform.m_xPosition.GetX(), xInstanceTransform.m_xPosition.GetY(), xInstanceTransform.m_xPosition.GetZ()));
		Asura_Assertf( Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._31) <= fMatrixMax3x3Value, ("Rotation/Scale Value %f is too large. Max Value of %f. Instance at X %f Y %f Z %f", xInstanceTransform.m_xTransformMatrix._31, fMatrixMax3x3Value, xInstanceTransform.m_xPosition.GetX(), xInstanceTransform.m_xPosition.GetY(), xInstanceTransform.m_xPosition.GetZ()));
		Asura_Assertf( Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._12) <= fMatrixMax3x3Value, ("Rotation/Scale Value %f is too large. Max Value of %f. Instance at X %f Y %f Z %f", xInstanceTransform.m_xTransformMatrix._12, fMatrixMax3x3Value, xInstanceTransform.m_xPosition.GetX(), xInstanceTransform.m_xPosition.GetY(), xInstanceTransform.m_xPosition.GetZ()));
		Asura_Assertf( Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._22) <= fMatrixMax3x3Value, ("Rotation/Scale Value %f is too large. Max Value of %f. Instance at X %f Y %f Z %f", xInstanceTransform.m_xTransformMatrix._22, fMatrixMax3x3Value, xInstanceTransform.m_xPosition.GetX(), xInstanceTransform.m_xPosition.GetY(), xInstanceTransform.m_xPosition.GetZ()));
		Asura_Assertf( Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._32) <= fMatrixMax3x3Value, ("Rotation/Scale Value %f is too large. Max Value of %f. Instance at X %f Y %f Z %f", xInstanceTransform.m_xTransformMatrix._32, fMatrixMax3x3Value, xInstanceTransform.m_xPosition.GetX(), xInstanceTransform.m_xPosition.GetY(), xInstanceTransform.m_xPosition.GetZ()));
		Asura_Assertf( Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._13) <= fMatrixMax3x3Value, ("Rotation/Scale Value %f is too large. Max Value of %f. Instance at X %f Y %f Z %f", xInstanceTransform.m_xTransformMatrix._13, fMatrixMax3x3Value, xInstanceTransform.m_xPosition.GetX(), xInstanceTransform.m_xPosition.GetY(), xInstanceTransform.m_xPosition.GetZ()));
		Asura_Assertf( Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._23) <= fMatrixMax3x3Value, ("Rotation/Scale Value %f is too large. Max Value of %f. Instance at X %f Y %f Z %f", xInstanceTransform.m_xTransformMatrix._23, fMatrixMax3x3Value, xInstanceTransform.m_xPosition.GetX(), xInstanceTransform.m_xPosition.GetY(), xInstanceTransform.m_xPosition.GetZ()));
		Asura_Assertf( Asura_Maths::Abs( xInstanceTransform.m_xTransformMatrix._33) <= fMatrixMax3x3Value, ("Rotation/Scale Value %f is too large. Max Value of %f. Instance at X %f Y %f Z %f", xInstanceTransform.m_xTransformMatrix._33, fMatrixMax3x3Value, xInstanceTransform.m_xPosition.GetX(), xInstanceTransform.m_xPosition.GetY(), xInstanceTransform.m_xPosition.GetZ()));
		
		#if !defined( ASURA_DIRECT3D_BGR_INSTANCE_ENABLE_COMPRESSED_DATA)
			this->m_xTransformMatrix = xInstanceTransform.m_xTransformMatrix;
		#else
			this->m_aiTransformMatrix3x3[0][0] = static_cast< int8>( xInstanceTransform.m_xTransformMatrix._11 * ASURA_DIRECT3D_BGR_INSTANCE_SCALE_COMPRESS);
			this->m_aiTransformMatrix3x3[0][1] = static_cast< int8>( xInstanceTransform.m_xTransformMatrix._21 * ASURA_DIRECT3D_BGR_INSTANCE_SCALE_COMPRESS);
			this->m_aiTransformMatrix3x3[0][2] = static_cast< int8>( xInstanceTransform.m_xTransformMatrix._31 * ASURA_DIRECT3D_BGR_INSTANCE_SCALE_COMPRESS);
			
			this->m_aiTransformMatrix3x3[1][0] = static_cast< int8>( xInstanceTransform.m_xTransformMatrix._12 * ASURA_DIRECT3D_BGR_INSTANCE_SCALE_COMPRESS);
			this->m_aiTransformMatrix3x3[1][1] = static_cast< int8>( xInstanceTransform.m_xTransformMatrix._22 * ASURA_DIRECT3D_BGR_INSTANCE_SCALE_COMPRESS);
			this->m_aiTransformMatrix3x3[1][2] = static_cast< int8>( xInstanceTransform.m_xTransformMatrix._32 * ASURA_DIRECT3D_BGR_INSTANCE_SCALE_COMPRESS);
			
			this->m_aiTransformMatrix3x3[2][0] = static_cast< int8>( xInstanceTransform.m_xTransformMatrix._13 * ASURA_DIRECT3D_BGR_INSTANCE_SCALE_COMPRESS);
			this->m_aiTransformMatrix3x3[2][1] = static_cast< int8>( xInstanceTransform.m_xTransformMatrix._23 * ASURA_DIRECT3D_BGR_INSTANCE_SCALE_COMPRESS);
			this->m_aiTransformMatrix3x3[2][2] = static_cast< int8>( xInstanceTransform.m_xTransformMatrix._33 * ASURA_DIRECT3D_BGR_INSTANCE_SCALE_COMPRESS);
		#endif
	}
}
	
void Asura_BGR_Instance::SetInstanceData( const Asura_BGR_InstanceTransform& xInstanceTransform, const Asura_BGR_InstanceLighting& xInstanceLighting)
{
	this->SetInstanceData( xInstanceTransform);
	this->SetInstanceData( xInstanceLighting);
}
	
void Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_ConvertLOD( const Tools_BotanicalElementLOD* const pxElement, Asura_Direct3D_BGR_Type_GeometryLODRange& xLodRange)
{
	xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::HIGH_0] = pxElement->m_fLod0;
	xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::HIGH_1] = pxElement->m_fLod0 + pxElement->m_fFadeLength0;
	xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_1] = pxElement->m_fLod1 - pxElement->m_fFadeLength1;
	xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_0] = pxElement->m_fLod1;
	
	// DLH - 24 March 2006 16:04:35 - Invert values so 0 is closest to the camera
	xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::HIGH_0] = 1.0f - xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::HIGH_0];
	xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::HIGH_1] = 1.0f - xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::HIGH_1];
	xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_1]  = 1.0f - xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_1]; 
	xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_0]  = 1.0f - xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_0]; 

	Asura_Assert( xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_0] <= xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_1], "LOD Ranges out of range");
	Asura_Assert( xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_1] <= xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::HIGH_1], "LOD Ranges out of range");
	Asura_Assert( xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::HIGH_1] <= xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::HIGH_0], "LOD Ranges out of range");
	
	xLodRange.CalcScales();
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
bool Asura_Direct3D_BGR_Type_VertexGeometry3DStorage::operator == ( const Asura_Direct3D_BGR_Type_VertexGeometry3DStorage& xRHS) const
{
	if( this->m_xPosition != xRHS.m_xPosition)
	{
		return( false);
	}
	if( this->m_xNormal != xRHS.m_xNormal)
	{
		return( false);
	}
	if( this->m_uColour != xRHS.m_uColour)
	{
		return( false);
	}
	if( this->m_afTextcoord0[ 0] != xRHS.m_afTextcoord0[ 0])
	{
		return( false);
	}
	if( this->m_afTextcoord0[ 1] != xRHS.m_afTextcoord0[ 1])
	{
		return( false);
	}
	return( true);
}
#endif

template< class Tvertex> 
static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_OptimiseVertexCount( Asura_Botanical_Array< Tvertex>& xArrayVertices, Asura_Botanical_Array< uint32>& xArrayIndices)
{
	const uint32 uIndexInvalid = static_cast< uint32>( -1);
	Asura_Botanical_Array< uint32> xArrayIndexRemap;
	
	const uint32 uVertexOriginalCount = xArrayVertices.Size();
	// DLH - 28 March 2006 12:46:51 - Create and setup the remap table
	
	xArrayIndexRemap.SetSize( uVertexOriginalCount);
	for( uint32 uRemapIndexIndex = 0; uRemapIndexIndex < uVertexOriginalCount; uRemapIndexIndex += 1)
	{
		xArrayIndexRemap[ uRemapIndexIndex] = uIndexInvalid;
	}
	
	// DLH - 28 March 2006 12:52:57 - Find the Duplicate vertices and mark which vertex they should be moved to
	for( uint32 uVertexOriginalIndex = 0; uVertexOriginalIndex < uVertexOriginalCount; uVertexOriginalIndex += 1)
	{
		const uint32 uVertexOriginalRemap = xArrayIndexRemap[ uVertexOriginalIndex];
		if( uIndexInvalid != uVertexOriginalRemap)
		{
			continue;
		}
		
		const Tvertex& tVertexOriginal = xArrayVertices[ uVertexOriginalIndex];
		
		for( uint32 uVertexCheckIndex = uVertexOriginalIndex + 1; uVertexCheckIndex < uVertexOriginalCount; uVertexCheckIndex += 1)
		{
			uint32& uVertexCheckRemap = xArrayIndexRemap[ uVertexCheckIndex];
			if( uIndexInvalid != uVertexCheckRemap)
			{
				continue;
			}
			
			const Tvertex& tVertexCheck = xArrayVertices[ uVertexCheckIndex];
			if( tVertexCheck != tVertexOriginal)
			{
				continue;
			}
			
			uVertexCheckRemap = uVertexOriginalIndex;
			
		}
	}
	
	Asura_Botanical_Array< uint32> xArrayVertexRemap;
	
	xArrayVertexRemap.SetSize( uVertexOriginalCount);
	
	uint32 uVertexRemapNextIndex = 0;
	// DLH - 28 March 2006 14:59:08 - Calc reordering of vertices
	for( uint32 uVertexOriginalIndex = 0; uVertexOriginalIndex < uVertexOriginalCount; uVertexOriginalIndex += 1)
	{
		uint32& uVertexOriginalRemap = xArrayIndexRemap[ uVertexOriginalIndex];
		if( uIndexInvalid != uVertexOriginalRemap)
		{
			// DLH - 28 March 2006 15:04:43 - Change the duplicate vertex to the remapped value of original vertex
			const uint32 uVertexRemappedRemapIndex = xArrayIndexRemap[ uVertexOriginalRemap];
			Asura_Assert( uIndexInvalid != uVertexRemappedRemapIndex, "Oops. Cannot remap to a not allocated vertex");
			uVertexOriginalRemap = uVertexRemappedRemapIndex;
			//uVertexOriginalRemap = uIndexInvalid;
			continue;
		}
		
		// DLH - 28 March 2006 15:06:17 - Find a new free vertex
	#if 0
		uint32 uVertexCheckIndex = 0;
		for( ; uVertexCheckIndex < uVertexOriginalRemap; uVertexCheckIndex += 1)
		{
			const uint32 uVertexCheckRemap = xArrayIndexRemap[ uVertexCheckIndex];
			if( uIndexInvalid == uVertexCheckRemap)
			{
				break;
			}
		}
	#endif
		
		uVertexOriginalRemap = uVertexRemapNextIndex;
		xArrayVertexRemap[ uVertexRemapNextIndex] = uVertexOriginalIndex;
		uVertexRemapNextIndex += 1;
	}
	
	// DLH - 28 March 2006 15:00:37 - Reorder Vertices
	const uint32 uVertexNewCount = uVertexRemapNextIndex;
	Asura_Assert( uVertexNewCount <= uVertexOriginalCount, "HOw on earth can the new index array be bigger!");
	for( uint32 uVertexNewIndex = 0; uVertexNewIndex < uVertexNewCount; uVertexNewIndex += 1)
	{
		uint32 uVertexOldIndex = xArrayVertexRemap[ uVertexNewIndex];
		Asura_Assert( uIndexInvalid != uVertexOldIndex, "Index is invalid");
		const Tvertex& tVertexOriginal = xArrayVertices[ uVertexOldIndex];
		
		xArrayVertices[ uVertexNewIndex] = tVertexOriginal;
	}
	
	// DLH - 28 March 2006 15:22:20 - Remap Indices
	const uint32 uIndexCount = xArrayIndices.Size();
	for( uint32 uIndexIndex = 0; uIndexIndex < uIndexCount; uIndexIndex += 1)
	{
		uint32& uIndex = xArrayIndices[ uIndexIndex];
		
		Asura_Assert( uIndex < uVertexOriginalCount, "Old Index Out of Range");
		const uint32 uIndexRemapped = xArrayIndexRemap[ uIndex];
		
		Asura_Assert( uIndexRemapped < uVertexNewCount, "New Index Out of Range");
		Asura_Assert( uIndexInvalid != uIndexRemapped, "New Index is invalid");
		uIndex = uIndexRemapped;
	}
	
	
	xArrayVertices.Shrink( uVertexNewCount);
	
	return( true);
}

template< class Tvertex> 
static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_OptimiseVertexOrder( Asura_Botanical_Array< Tvertex>& xArrayVertices, Asura_Botanical_Array< uint32>& xArrayIndices)
{
// DLH - 28 March 2006 16:38:55 - Commented Out as it is not needed at present
#if 0
	const uint32 uIndexInvalid = static_cast< uint32>( -1);
	Asura_Botanical_Array< uint32> xArrayIndexRemap;
	
	const uint32 uVertexCount = xArrayVertices.Size();
	// DLH - 28 March 2006 12:46:51 - Create and setup the remap table
	
	xArrayIndexRemap.SetSize( uVertexCount);
	for( uint32 uRemapIndexIndex = 0; uRemapIndexIndex < uVertexCount; uRemapIndexIndex += 1)
	{
		xArrayIndexRemap[ uRemapIndexIndex] = uIndexInvalid;
	}
	
	const uint32 uIndexCount = xArrayIndices.Size();
	uint32 uNextVertexIndex = 0;
	for( uint32 uRemapIndexIndex = 0; uRemapIndexIndex < uIndexCount; uRemapIndexIndex += 1)
	{
		const uint32 uVertexIndex = xArrayIndices[ uRemapIndexIndex];
		uint32& uRemapIndex = xArrayIndexRemap[ uVertexIndex];
		
		if( uIndexInvalid != uRemapIndex)
		{
			continue;
		}
		
		uRemapIndex = uNextVertexIndex;
		uNextVertexIndex += 1;
	}
	
	Asura_Assert( uVertexCount == uNextVertexIndex, "We seem to have the incorect number of indices!");
#endif
	
#if 0
	// DLH - 28 March 2006 15:00:37 - Reorder Vertices
	const uint32 uVertexNewCount = uVertexRemapNextIndex;
	Asura_Assert( uVertexNewCount <= uVertexOriginalCount, "HOw on earth can the new index array be bigger!");
	for( uint32 uVertexNewIndex = 0; uVertexNewIndex < uVertexNewCount; uVertexNewIndex += 1)
	{
		uint32 uVertexOldIndex = xArrayVertexRemap[ uVertexNewIndex];
		Asura_Assert( uIndexInvalid != uVertexOldIndex, "Index is invalid");
		const Tvertex& tVertexOriginal = xArrayVertices[ uVertexOldIndex];
		
		xArrayVertices[ uVertexNewIndex] = tVertexOriginal;
	}
	
	// DLH - 28 March 2006 15:22:20 - Remap Indices
	const uint32 uIndexCount = xArrayIndices.Size();
	for( uint32 uIndexIndex = 0; uIndexIndex < uIndexCount; uIndexIndex += 1)
	{
		uint32& uIndex = xArrayIndices[ uIndexIndex];
		
		Asura_Assert( uIndex < uVertexOriginalCount, "Old Index Out of Range");
		const uint32 uIndexRemapped = xArrayIndexRemap[ uIndex];
		
		Asura_Assert( uIndexRemapped < uVertexNewCount, "New Index Out of Range");
		Asura_Assert( uIndexInvalid != uIndexRemapped, "New Index is invalid");
		uIndex = uIndexRemapped;
	}
#endif
	
	return( true);
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
Asura_Bounding_Box Asura_Direct3D_BGR_Type_Geometry3D::GetBoundingBox() const
{
	Asura_Bounding_Box xBoundingBox;
	
	xBoundingBox.Reset();
	
	const u_int uVertexCount = this->m_xArrayVertices.Size();
	for( u_int uVertexIndex = 0; uVertexIndex < uVertexCount; uVertexIndex += 1)
	{
		const Asura_Direct3D_BGR_Type_VertexGeometry3DStorage& xVertex = this->m_xArrayVertices[ uVertexIndex];
		
		xBoundingBox.Set( Asura_Vector_3( xVertex.m_xPosition));
	}
	
	return( xBoundingBox);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
Asura_Bounding_Box Asura_Direct3D_BGR_Type_GeometryLeaves::GetBoundingBox() const
{
	Asura_Bounding_Box xBoundingBox;
	
	xBoundingBox.Reset();
	
	const u_int uVertexCount = this->m_xArrayVertices.Size();
	for( u_int uVertexIndex = 0; uVertexIndex < uVertexCount; uVertexIndex += 1)
	{
		const Asura_Direct3D_BGR_Type_VertexGeometryLeafStorage& xVertex = this->m_xArrayVertices[ uVertexIndex];
		
		// DLH - 11 April 2006 16:13:55 - Multiply the width and hight by a value if the rotation was 45 degress to stick the corners out 
		const float fSize = 1.5f * Asura_Maths::Max( xVertex.m_afCorner[0], xVertex.m_afCorner[1]);
		
		xBoundingBox.Set( Asura_Vector_3( xVertex.m_xPosition - Asura_AVector3( fSize, fSize, fSize)));
		xBoundingBox.Set( Asura_Vector_3( xVertex.m_xPosition + Asura_AVector3( fSize, fSize, fSize)));
	}
	
	return( xBoundingBox);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
Asura_Bounding_Box Asura_Direct3D_BGR_Type_GeometryBillboard::GetBoundingBox() const
{
	Asura_Bounding_Box xBoundingBox;
	
	xBoundingBox.Reset();
	
	const u_int uBillboardCount = this->m_xArrayBillboards.Size();
	for( u_int uBillboardIndex = 0; uBillboardIndex < uBillboardCount; uBillboardIndex += 1)
	{
		const Asura_Direct3D_BGR_Type_VertexGeometryImposterStorage& xImposter = this->m_xArrayBillboards[ uBillboardIndex];
		const float fSizeX = 0.5f * xImposter.m_fWidth;
		const float fSizeY = xImposter.m_fHeight;
		
		Asura_Bounding_Box xImposterBoundingBox;
		
		xImposterBoundingBox.Reset();
		
		xImposterBoundingBox.Set( Asura_Vector_3( -fSizeX, -fSizeY, -fSizeX));
		xImposterBoundingBox.Set( Asura_Vector_3( +fSizeX, 0, +fSizeX));
		
		xBoundingBox.Set( xImposterBoundingBox);
	}
	
	return( xBoundingBox);
}
#endif

//-------------------------------------------------------------------
// Function:    Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_GetBillboardFromPolygon
//-------------------------------------------------------------------
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS) || defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_GetBillboardFromPolygon( const Asura_Mesh& xMesh, const Asura_Mesh_Polygon& xPolygonDataPtr, const Asura_Mesh_TextureInfo& xTextureDataPtr, Asura_Direct3D_BGR_MDS_BillboardDetails& xBillboardData)
{
	const Asura_AVector3 xDirectionWorldWidth( 1.0f, 0.0f, 0.0f);
	const Asura_AVector3 xDirectionWorldHeight( 0.0f, 1.0f, 0.0f);

	#if defined( ASURA_ASSERT )
		const uint32 uElementVertexCount = xMesh.NumberOfVertices;
	#endif
	
	xBillboardData.m_afTextcoord[ 0][ 0] = 0.0f;
	xBillboardData.m_afTextcoord[ 0][ 1] = 0.0f;
	xBillboardData.m_afTextcoord[ 1][ 0] = 0.0f;
	xBillboardData.m_afTextcoord[ 1][ 1] = 1.0f;
	xBillboardData.m_afTextcoord[ 2][ 0] = 1.0f;
	xBillboardData.m_afTextcoord[ 2][ 1] = 0.0f;
	xBillboardData.m_afTextcoord[ 3][ 0] = 1.0f;
	xBillboardData.m_afTextcoord[ 3][ 1] = 1.0f;
	
	//Asura_AVector3 axPositions[ 4];
	float afTextcoord0[4][2] = { 0};
	//uint32 auColour[ 4] = { 0};
	
	for( uint32 uPolygonVertex = 0; uPolygonVertex < 4; uPolygonVertex += 1)
	{
		uint32 uVertexIndex = xPolygonDataPtr.VertexID[ uPolygonVertex];
		Asura_Assert( uVertexIndex < uElementVertexCount, "Element Vertex Index Out of Range");
	
		xBillboardData.m_axPositions[ uPolygonVertex] = xMesh.VertexDataPtr[ uVertexIndex].Position;

		afTextcoord0[ uPolygonVertex][ 0] = xTextureDataPtr.U[0][uPolygonVertex];
		afTextcoord0[ uPolygonVertex][ 1] = xTextureDataPtr.V[0][uPolygonVertex];

		xBillboardData.m_auColour[ uPolygonVertex] = 0xFFFFFFFF;
	}
	
	xBillboardData.m_xPositionCenter = (xBillboardData.m_axPositions[0] + xBillboardData.m_axPositions[1] + xBillboardData.m_axPositions[2] + xBillboardData.m_axPositions[3]) / 4;
	
	const uint32 uIndexBiggestInvalid = static_cast< uint32>( -1);
	
	// DLH - 30 March 2006 16:04:21 - Find the Natutal Directions (NDs)
	const Asura_AVector3 axNaturalDirections[3] =
	{
		xBillboardData.m_axPositions[0] - xBillboardData.m_axPositions[1],
		xBillboardData.m_axPositions[0] - xBillboardData.m_axPositions[2],
		xBillboardData.m_axPositions[0] - xBillboardData.m_axPositions[3]
	};
	// DLH - 30 March 2006 16:03:50 - Find the angles between the NDs
	const float afDotNDs[ 3] = 
	{
		axNaturalDirections[0] * axNaturalDirections[1],
		axNaturalDirections[0] * axNaturalDirections[2],
		axNaturalDirections[2] * axNaturalDirections[1]
	};
	
	const float afAbsDotNDs[ 3] = 
	{
		Asura_Maths::Abs( afDotNDs[0]),
		Asura_Maths::Abs( afDotNDs[1]),
		Asura_Maths::Abs( afDotNDs[2])
	};
	
	// DLH - 30 March 2006 16:46:45 - Find the outer two NDs
	uint32 uUseNDIndex0 = uIndexBiggestInvalid;
	uint32 uUseNDIndex1 = uIndexBiggestInvalid;
	uint32 uUseNDIndex2 = uIndexBiggestInvalid;
	
	if( afAbsDotNDs[ 0] < afAbsDotNDs[ 1])
	{
		if( afAbsDotNDs[ 0] < afAbsDotNDs[ 2])
		{
			uUseNDIndex0 = 0;
			uUseNDIndex1 = 1;
			uUseNDIndex2 = 2;
		}
		else
		{
			if( afAbsDotNDs[ 1] < afAbsDotNDs[ 2])
			{
				uUseNDIndex0 = 0;
				uUseNDIndex1 = 2;
				uUseNDIndex2 = 1;
			}
			else
			{
				uUseNDIndex0 = 2;
				uUseNDIndex1 = 1;
				uUseNDIndex2 = 0;
			}
		}
	}
	else
	{
		if( afAbsDotNDs[ 1] < afAbsDotNDs[ 2])
		{
			uUseNDIndex0 = 0;
			uUseNDIndex1 = 2;
			uUseNDIndex2 = 1;
		}
		else
		{
			if( afAbsDotNDs[ 0] < afAbsDotNDs[ 2])
			{
				uUseNDIndex0 = 0;
				uUseNDIndex1 = 1;
				uUseNDIndex2 = 2;
			}
			else
			{
				uUseNDIndex0 = 2;
				uUseNDIndex1 = 1;
				uUseNDIndex2 = 0;
			}
		}
	}
	Asura_Assert( uIndexBiggestInvalid != uUseNDIndex0, "Outter NDs not found!");
	Asura_Assert( uIndexBiggestInvalid != uUseNDIndex1, "Outter NDs not found!");
	
	// DLH - 30 March 2006 16:40:23 - Find one that is nice and close to the Height Direction
	const float afDotUp[ 2] = 
	{
		xDirectionWorldHeight * axNaturalDirections[ uUseNDIndex0],
		xDirectionWorldHeight * axNaturalDirections[ uUseNDIndex1],
	};
	const float afAbsDotUp[ 2] = 
	{
		Asura_Maths::Abs( afDotUp[0]),
		Asura_Maths::Abs( afDotUp[1]),
	};
	
	uint32 uIndexBiggestDotUp = uIndexBiggestInvalid;
	uint32 uIndexBiggestDotSide = uIndexBiggestInvalid;
	const uint32 uIndexBiggestDotDiagonal = uUseNDIndex2;
	
	if( afAbsDotUp[ 0] < afAbsDotUp[ 1])
	{
		uIndexBiggestDotUp = uUseNDIndex1;
		uIndexBiggestDotSide = uUseNDIndex0;
	}
	else
	{
		uIndexBiggestDotUp = uUseNDIndex0;
		uIndexBiggestDotSide = uUseNDIndex1;
	}
	
	Asura_Assert( uIndexBiggestInvalid != uIndexBiggestDotUp, "Biggest Up Vector not found!");
	Asura_Assert( uIndexBiggestInvalid != uIndexBiggestDotSide, "Biggest Side Vector not found!");
	
	// DLH - 30 March 2006 16:26:02 - Does the Direction need Inverting
	//const float fScaleDirectionUp = 0.0f < afDotUp[ uIndexBiggestDotUp] ? 1.0f : -1.0f;
	Asura_AVector3 xDirectionHeight( axNaturalDirections[ uIndexBiggestDotUp]);
	xDirectionHeight.Normalise();
	
	// DLH - 30 March 2006 16:26:02 - Does the Direction need Inverting
	//const float fScaleDirectionSide = 0.0f < afDotUp[ uIndexBiggestDotSide] ? 1.0f : -1.0f;
	Asura_AVector3 xDirectionWidth( axNaturalDirections[ uIndexBiggestDotSide]);
	xDirectionWidth.Normalise();
	
	const float afHeights[ 3] = 
	{
		axNaturalDirections[ 0] * xDirectionHeight,
		axNaturalDirections[ 1] * xDirectionHeight,
		axNaturalDirections[ 2] * xDirectionHeight
	};
	const float afWidths[ 3] = 
	{
		axNaturalDirections[ 0] * xDirectionWidth,
		axNaturalDirections[ 1] * xDirectionWidth,
		axNaturalDirections[ 2] * xDirectionWidth
	};
	
	xBillboardData.m_fWidth = 0.5f * Asura_Maths::Max( afWidths[0], Asura_Maths::Max( afWidths[1], afWidths[2]));
	xBillboardData.m_fHeight = 0.5f * Asura_Maths::Max( afHeights[0], Asura_Maths::Max( afHeights[1], afHeights[2]));

	// (CJW, 06.03.2008) - Fix for billboards appearing up-side-down:
	uint32 uMinXMinYCornerIndex		= uIndexBiggestInvalid;		// index for the texture coordinate to be mapped to the bottom left corner of the quad
	uint32 uMinXMaxYCornerIndex		= uIndexBiggestInvalid;		// index for the texture coordinate to be mapped to the top left corner of the quad
	uint32 uMaxXMinYCornerIndex		= uIndexBiggestInvalid;		// index for the texture coordinate to be mapped to the bottom right corner of the quad
	uint32 uMaxXMaxYCornerIndex		= uIndexBiggestInvalid;		// index for the texture coordinate to be mapped to the top right corner of the quad

	if( axNaturalDirections[uIndexBiggestDotSide].GetX() < 0.0f )
	{
		uMinXMinYCornerIndex		= uIndexBiggestDotSide + 1;
		uMinXMaxYCornerIndex		= uIndexBiggestDotDiagonal + 1;
		uMaxXMinYCornerIndex		= 0;
		uMaxXMaxYCornerIndex		= uIndexBiggestDotUp + 1;
	}
	else  
	{
		uMinXMinYCornerIndex		= uIndexBiggestDotUp + 1;
		uMinXMaxYCornerIndex		= 0;
		uMaxXMinYCornerIndex		= uIndexBiggestDotDiagonal + 1;
		uMaxXMaxYCornerIndex		= uIndexBiggestDotSide + 1;
	}


	xBillboardData.m_afTextcoord[0][0] = afTextcoord0[uMinXMaxYCornerIndex][0];
	xBillboardData.m_afTextcoord[0][1] = afTextcoord0[uMinXMaxYCornerIndex][1];
	xBillboardData.m_afTextcoord[1][0] = afTextcoord0[uMinXMinYCornerIndex][0];
	xBillboardData.m_afTextcoord[1][1] = afTextcoord0[uMinXMinYCornerIndex][1];
	xBillboardData.m_afTextcoord[2][0] = afTextcoord0[uMaxXMaxYCornerIndex][0];
	xBillboardData.m_afTextcoord[2][1] = afTextcoord0[uMaxXMaxYCornerIndex][1];
	xBillboardData.m_afTextcoord[3][0] = afTextcoord0[uMaxXMinYCornerIndex][0];
	xBillboardData.m_afTextcoord[3][1] = afTextcoord0[uMaxXMinYCornerIndex][1];
	
	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_CopyMesh(  const Asura_Direct3D_BGR_Type& xBGRType, const Tools_BotanicalElementLOD& xElement, Asura_Direct3D_BGR_Type_GeometryBillboard& xGeometry)
{
	const u_int uImposterCount = xElement.m_xImposterList.Size();
	
	xGeometry.m_xArrayBillboards.SetSize( uImposterCount);
	
	u_int uImposterIndex = 0;
	for( Asura_ListElt< Tools_Botanicals_Imposter*>* pxItImposter = xElement.m_xImposterList.Front(); NULL != pxItImposter; pxItImposter = pxItImposter->Next(), uImposterIndex += 1)
	{
		const Tools_Botanicals_Imposter* pxElementImposter = pxItImposter->Data();
		//const Asura_Mesh* pxElementMesh = pxElementImposter->m_pxMesh;
		
		Asura_Direct3D_BGR_Type_VertexGeometryImposterStorage& xBGRImposter = xGeometry.m_xArrayBillboards[ uImposterIndex];
		
		if( false == pxElementImposter->m_bValid)
		{
			continue;
		}
	
		xBGRImposter.m_fWidth = pxElementImposter->m_xBillboard.fWidth;
		xBGRImposter.m_fHeight = pxElementImposter->m_xBillboard.fHeight;
		
		xBGRImposter.m_afTextcoord0[0] = pxElementImposter->m_xBillboard.fU[0];
		xBGRImposter.m_afTextcoord0[1] = pxElementImposter->m_xBillboard.fV[0];
		
		xBGRImposter.m_afTextcoord1[0] = pxElementImposter->m_xBillboard.fU[1];
		xBGRImposter.m_afTextcoord1[1] = pxElementImposter->m_xBillboard.fV[1];
		
		xBGRImposter.m_xDirection = pxElementImposter->m_xBillboard.xNormal;
		
		const float fFieldOfViewAngleOutter = Asura_Maths::DegToRad( 0.5f * pxElementImposter->m_fAngleRange);
		const float fFieldOfViewAngleInner = fFieldOfViewAngleOutter - Asura_Maths::DegToRad( pxElementImposter->m_fFadeRange);
		
		xBGRImposter.m_fFieldOfViewAngleInner = fFieldOfViewAngleInner;
		xBGRImposter.m_fFieldOfViewAngleOutter = fFieldOfViewAngleOutter;
		xBGRImposter.m_fFieldOfViewDotOutter = Asura_Maths::Cosine( fFieldOfViewAngleOutter);
		
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
		xBGRImposter.m_uMaterialResponseHashID = pxElementImposter->m_uMaterialResponseHashID;
#else
		xBGRImposter.m_iTextureDiffuse = pxElementImposter->m_iTextureIndexID;
		xBGRImposter.m_iTextureBump = ASURA_TEXTURE_INVALID;
#endif
	}
	
	if( uImposterIndex < xGeometry.m_xArrayBillboards.Size()) 
	{
		xGeometry.m_xArrayBillboards.Shrink( uImposterIndex);
	}
	return( true);
}
#endif

//-------------------------------------------------------------------
// Function:    Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_CopyMesh
//
// Description: 
//
// Parameters:  xElement        - 
//              xGeometry       - 
//
// Returns:     
//-------------------------------------------------------------------
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_CopyMesh(  const Asura_Direct3D_BGR_Type& xBGRType, const Tools_BotanicalElementLOD& xElement, Asura_Direct3D_BGR_Type_GeometryLeaves& xGeometry)
{
	Asura_Mesh* pxElementMesh = xElement.m_pxMesh;
	
	if( NULL == pxElementMesh)
	{
		return( true);
	}
	
	// DLH - 22 March 2006 17:14:06 - Check the incomming mesh for bad data
	Asura_Assert( NULL != pxElementMesh, "Mesh Pointer is NULL");
	
	Asura_Assert( true == pxElementMesh->VertexDataValid, "Vertex Data not Valid");
	Asura_Assert( true == pxElementMesh->TextureDataValid, "Texture Data not Valid");
	Asura_Assert( NULL != pxElementMesh->PolygonDataPtr, "Polygon Data Pointer is NULL");
	Asura_Assert( NULL != pxElementMesh->VertexDataPtr, "Position Vertex Data Pointer is NULL");
	Asura_Assert( NULL != pxElementMesh->TextureDataPtr, "Texture Vertex Data Pointer is NULL");
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	Tools_BGR_Type_BoneWeightCalc xBoneWeightData;
	const Asura_BGR_Type_BasicInteractive* const pxTypeBasicInteractive = xBGRType.GetBasicInteractive();
	const Asura_Mesh_PaintedColours* pxPaintedColourData = NULL;
	if( NULL != pxTypeBasicInteractive)
	{
		if( true == pxElementMesh->m_bPaintedColourDataValid && true == pxTypeBasicInteractive->m_bUsePaintedColoursAsWeights)
		{
			pxPaintedColourData = pxElementMesh->m_pxPaintedColourDataPtr;
		}
		if( NULL == pxPaintedColourData)
		{
			xBoneWeightData.SetupBoneWeight( *pxTypeBasicInteractive);
		}
	}
#endif

//	const Asura_AVector3 xDirectionWorldWidth( 1.0f, 0.0f, 0.0f);
//	const Asura_AVector3 xDirectionWorldHeight( 0.0f, 1.0f, 0.0f);

	// DLH - 22 March 2006 17:13:49 - Make the Mesh initale Mesh
	xGeometry.m_xArrayVertices.SetSize( 4 * pxElementMesh->NumberOfPolygons);
	xGeometry.m_xArrayIndices.SetSize( 6 * pxElementMesh->NumberOfPolygons);
	
	const uint32 uElementPolygonCount = pxElementMesh->NumberOfPolygons;
//	const uint32 uElementVertexCount = pxElementMesh->NumberOfVertices;
	 
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
	Asura_Hash_ID xMaterialResponseHashID = ASURA_HASH_ID_UNSET;
#endif
	int32 iMaterialID = ASURA_TEXTURE_INVALID;
	uint32 uGeometryVertexCount = 0;
	uint32 uGeometryIndexCount = 0;
	for( uint32 uElementPolygonIndex = 0; uElementPolygonIndex < uElementPolygonCount; uElementPolygonIndex += 1)
	{
		const Asura_Mesh_Polygon& xPolygonDataPtr = pxElementMesh->PolygonDataPtr[ uElementPolygonIndex];
		const Asura_Mesh_TextureInfo& xTextureDataPtr = pxElementMesh->TextureDataPtr[ uElementPolygonIndex];
//		const Asura_Mesh_Normals& xPolygonNormalDataPtr = pxElementMesh->NormalsDataPtr[ uElementPolygonIndex];
		const bool bIsQuad = xPolygonDataPtr.IsQuad();
		
		if( false == bIsQuad)
		{
			continue;
		}
		
#if 0 && defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
		Asura_Vector_3 xElementTangent;
		Asura_Vector_3 xElementBinormal;

		Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_CalculateTangentSpaceVectors( pxElementMesh, uElementPolygonIndex, xElementTangent, xElementBinormal);
#endif

#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
		Asura_Assert( ASURA_HASH_ID_UNSET == xMaterialResponseHashID || xMaterialResponseHashID == xTextureDataPtr.m_auMaterialResponseHashID[0], "There seems to be more than one material on this mesh");
		xMaterialResponseHashID = xTextureDataPtr.m_auMaterialResponseHashID[0];
#endif
		Asura_Assert( ASURA_TEXTURE_INVALID == iMaterialID || iMaterialID == xTextureDataPtr.m_iMaterialID, "There seems to be more than one material on this mesh");
		iMaterialID = xTextureDataPtr.m_iMaterialID;
			
		Asura_Direct3D_BGR_MDS_BillboardDetails xBillboardData;
		
		Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_GetBillboardFromPolygon( *pxElementMesh, xPolygonDataPtr, xTextureDataPtr, xBillboardData);

		xGeometry.m_xArrayIndices[ uGeometryIndexCount + 0] = uGeometryVertexCount + 2;
		xGeometry.m_xArrayIndices[ uGeometryIndexCount + 1] = uGeometryVertexCount + 1;
		xGeometry.m_xArrayIndices[ uGeometryIndexCount + 2] = uGeometryVertexCount + 0;
		xGeometry.m_xArrayIndices[ uGeometryIndexCount + 3] = uGeometryVertexCount + 1;
		xGeometry.m_xArrayIndices[ uGeometryIndexCount + 4] = uGeometryVertexCount + 2;
		xGeometry.m_xArrayIndices[ uGeometryIndexCount + 5] = uGeometryVertexCount + 3;
		
		uGeometryIndexCount += 6;
			
		const float afCornerScale[ 4][ 2] =
		{
			{ -1, -1 },
			{ -1, +1 },
			{ +1, -1 },
			{ +1, +1 }
		};
		
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
		Asura_Vector_3 xElementTangent;
		Asura_Vector_3 xElementBinormal;

		Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_CalculateTangentSpaceVectors( afCornerScale, xBillboardData.m_afTextcoord, xElementTangent, xElementBinormal);
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
		float fBoneWeight0 = 0.0f;
		float fBoneWeight1 = 0.0f;
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
		if( NULL != pxPaintedColourData)
		{
			//uint32 uVertexIndex = xPolygonDataPtr.VertexID[ uElementPolygonIndex];
			//Asura_Assert( uVertexIndex < uElementVertexCount, "Element Vertex Index Out of Range");
			
			{
				const Asura_Mesh_PaintedColours &xPaintedPolygonColour = pxPaintedColourData[ uElementPolygonIndex];
				
				const u_int uColourRGBA = xPaintedPolygonColour.Colour[ 0];
				fBoneWeight0 = (0xFF & (uColourRGBA >> 16)) / float( 0xFF);
				fBoneWeight1 = (0xFF & (uColourRGBA >> 8)) / float( 0xFF);
			}
		}
		else
		{
			if( NULL != pxTypeBasicInteractive)
			{
				fBoneWeight0 = xBoneWeightData.CalculateBoneWeight( xBillboardData.m_xPositionCenter);
			}
		}
#endif
#endif
		

		for( uint32 uPolygonVertex = 0; uPolygonVertex < 4; uPolygonVertex += 1, uGeometryVertexCount += 1)
		{
			Asura_Direct3D_BGR_Type_VertexGeometryLeafStorage& xGeometryVertex = xGeometry.m_xArrayVertices[uGeometryVertexCount];

			xGeometryVertex.m_xPosition = xBillboardData.m_xPositionCenter;
			// Get billboard extents:
			xGeometryVertex.m_afCorner[0] = xBillboardData.m_fWidth * afCornerScale[uPolygonVertex][0];
			xGeometryVertex.m_afCorner[1] = xBillboardData.m_fHeight * afCornerScale[uPolygonVertex][1];
			// Get texture info:
			xGeometryVertex.m_afTextcoord0[0] = xBillboardData.m_afTextcoord[uPolygonVertex][0];
			xGeometryVertex.m_afTextcoord0[1] = xBillboardData.m_afTextcoord[uPolygonVertex][1];
			// Get colour info:
			xGeometryVertex.m_uColour = xBillboardData.m_auColour[uPolygonVertex];
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
			{
//				const Asura_Vector_3& xElementNormal = xPolygonNormalDataPtr.VertexNormal[ uPolygonVertex];
				
//				xGeometryVertex.m_xNormal = xElementNormal;
				xGeometryVertex.m_xNormal = Asura_AVector3( 0, 0, 1);
			}
			xGeometryVertex.m_xTangent = xElementTangent;
			xGeometryVertex.m_xBinormal = xElementBinormal;
			xGeometryVertex.m_fBoneWeight0 = fBoneWeight0;
			xGeometryVertex.m_fBoneWeight1 = fBoneWeight1;
			
			xGeometryVertex.m_afTextcoord1[ 0] = xBillboardData.m_afTextcoord[ uPolygonVertex][0];
			xGeometryVertex.m_afTextcoord1[ 1] = xBillboardData.m_afTextcoord[ uPolygonVertex][1];
			
#endif
		}
	}
	
	Asura_Assertf( xGeometry.m_xArrayIndices.Size() == uGeometryIndexCount, ("Difference in the number of indices in the element and the geometry. This might be coursed by non-quad polygons in mesh '%s'", pxElementMesh->Name.GetString()));
	Asura_Assertf( xGeometry.m_xArrayVertices.Size() == uGeometryVertexCount, ("Difference in the number of vertices in the element and the geometry. This might be coursed by non-quad polygons in mesh '%s'", pxElementMesh->Name.GetString()));
	
	xGeometry.m_xArrayVertices.Shrink( uGeometryVertexCount);
	xGeometry.m_xArrayIndices.Shrink( uGeometryIndexCount);
	
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
	xGeometry.xMaterial.SetMaterialResponseHashID( xMaterialResponseHashID);
#else
	xGeometry.xMaterial.SetMaterial( iMaterialID);
#endif
#if 0
	{
		Asura_Assert( iMaterialID != ASURA_TEXTURE_INVALID, "There is not material on this mesh");
		const int iTextureID = Asura_Material_System::GetTextureID( iMaterialID);
		xGeometry.m_iTextureDiffuse = iTextureID;
	}
#endif
	
//	Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_OptimiseVertexCount( xGeometry.m_xArrayVertices, xGeometry.m_xArrayIndices);
//	Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_OptimiseVertexOrder( xGeometry.m_xArrayVertices, xGeometry.m_xArrayIndices);

	return( true);
}
#endif

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_CopyMesh( const Asura_Direct3D_BGR_Type& xBGRType, const Tools_BotanicalElementLOD& xElement, Asura_Direct3D_BGR_Type_Geometry3D& xGeometry)
{
	Asura_Mesh* pxElementMesh = xElement.m_pxMesh;
	
	if( NULL == pxElementMesh)
	{
		xGeometry.m_xArrayIndices.Clear();
		xGeometry.m_xArrayVertices.Clear();
		return( true);
	}
	
	// DLH - 22 March 2006 17:14:06 - Check the incomming mesh for bad data
	Asura_Assert( NULL != pxElementMesh, "Mesh Pointer is NULL");
	
	Asura_Assert( true == pxElementMesh->VertexDataValid, "Vertex Data not Valid");
	Asura_Assert( true == pxElementMesh->m_bVertexNormalsValid, "Normal Vertex Data not Valid");
	Asura_Assert( true == pxElementMesh->TextureDataValid, "Texture Data not Valid");
	Asura_Assert( NULL != pxElementMesh->PolygonDataPtr, "Polygon Data Pointer is NULL");
	Asura_Assert( NULL != pxElementMesh->VertexDataPtr, "Position Vertex Data Pointer is NULL");
	Asura_Assert( NULL != pxElementMesh->NormalsDataPtr, "Normal Vertex Data Pointer is NULL");
	Asura_Assert( NULL != pxElementMesh->TextureDataPtr, "Texture Vertex Data Pointer is NULL");
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	Tools_BGR_Type_BoneWeightCalc xBoneWeightData;
	const Asura_BGR_Type_BasicInteractive* const pxTypeBasicInteractive = xBGRType.GetBasicInteractive();
	const Asura_Mesh_PaintedColours* pxPaintedColourData = NULL;
	if( NULL != pxTypeBasicInteractive)
	{
		if( true == pxElementMesh->m_bPaintedColourDataValid && true == pxTypeBasicInteractive->m_bUsePaintedColoursAsWeights)
		{
			pxPaintedColourData = pxElementMesh->m_pxPaintedColourDataPtr;
		}
		if( NULL == pxPaintedColourData)
		{
			xBoneWeightData.SetupBoneWeight( *pxTypeBasicInteractive);
		}
	}
#endif
		
	// DLH - 22 March 2006 17:13:49 - Make the Mesh initale Mesh
	xGeometry.m_xArrayVertices.SetSize( 3 * pxElementMesh->NumberOfTriangles);
	xGeometry.m_xArrayIndices.SetSize( 3 * pxElementMesh->NumberOfTriangles);
	
	const uint32 uElementPolygonCount = pxElementMesh->NumberOfPolygons;

	#if defined( ASURA_ASSERT )
		const uint32 uElementVertexCount = pxElementMesh->NumberOfVertices;
	#endif

#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
	Asura_Hash_ID xMaterialResponseHashID = ASURA_HASH_ID_UNSET;
#endif
	int32 iMaterialID = ASURA_TEXTURE_INVALID;
	uint32 uGeometryTriangleCount = 0;
	uint32 uGeometryVertexCount = 0;
	for( uint32 uElementPolygonIndex = 0; uElementPolygonIndex < uElementPolygonCount; uElementPolygonIndex += 1)
	{
		const Asura_Mesh_Polygon& xPolygonDataPtr = pxElementMesh->PolygonDataPtr[ uElementPolygonIndex];
		const Asura_Mesh_Normals& xPolygonNormalDataPtr = pxElementMesh->NormalsDataPtr[ uElementPolygonIndex];
		const Asura_Mesh_TextureInfo& xTextureDataPtr = pxElementMesh->TextureDataPtr[ uElementPolygonIndex];
		const uint32 uPolygonTriangleCount = (true == xPolygonDataPtr.IsQuad()) ? 2 : 1;
		
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
		Asura_Vector_3 xElementTangent;
		Asura_Vector_3 xElementBinormal;

		Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_CalculateTangentSpaceVectors( pxElementMesh, uElementPolygonIndex, xElementTangent, xElementBinormal);
#endif

		for( uint32 uPolygonTriangleIndex = 0; uPolygonTriangleIndex < uPolygonTriangleCount; uPolygonTriangleIndex += 1, uGeometryTriangleCount += 1)		
		{
			const uint32 uPolygonVertexBaseOffset = 2 * uPolygonTriangleIndex;
			for( uint32 uPolygonVertexOffset = 0; uPolygonVertexOffset < 3; uPolygonVertexOffset += 1, uGeometryVertexCount += 1)
			{
				Asura_Direct3D_BGR_Type_VertexGeometry3DStorage& xGeometryVertex = xGeometry.m_xArrayVertices[ uGeometryVertexCount];
				
				const uint32 uPolygonVertex = (uPolygonVertexOffset + uPolygonVertexBaseOffset) % 4;
			
				{
					uint32 uVertexIndex = xPolygonDataPtr.VertexID[ uPolygonVertex];
					Asura_Assert( uVertexIndex < uElementVertexCount, "Element Vertex Index Out of Range");
					
					{
						const Asura_Vector_3& xElementVertexPosition = pxElementMesh->VertexDataPtr[ uVertexIndex].Position;
						
						xGeometryVertex.m_xPosition = xElementVertexPosition;
					}
				}

				{
					const Asura_Vector_3& xElementNormal = xPolygonNormalDataPtr.VertexNormal[ uPolygonVertex];
					
					xGeometryVertex.m_xNormal = xElementNormal;
				}
				
#if defined( ASURA_ASSERT)
				if( ASURA_TEXTURE_INVALID != iMaterialID && iMaterialID != xTextureDataPtr.m_iMaterialID)
				{
					Asura_Material* pxMaterial0 = Asura_Material_System::GetMaterial( iMaterialID);
					Asura_Material* pxMaterial1 = Asura_Material_System::GetMaterial( xTextureDataPtr.m_iMaterialID);
					
					Asura_Assertf( false, ("There seems to be more than one material on mesh '%s'\nBoth materials '%s' and '%s' seem to be in the mesh.", pxElementMesh->Name.GetString(), pxMaterial0->m_xMaterialName.GetString(), pxMaterial1->m_xMaterialName.GetString()) );
				}
#endif

#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
				xMaterialResponseHashID = xTextureDataPtr.m_auMaterialResponseHashID[0];
#endif
				iMaterialID = xTextureDataPtr.m_iMaterialID;
									
				xGeometryVertex.m_afTextcoord0[ 0] = xTextureDataPtr.U[0][uPolygonVertex];
				xGeometryVertex.m_afTextcoord0[ 1] = xTextureDataPtr.V[0][uPolygonVertex];
				xGeometryVertex.m_uColour = 0xFFFFFFFF;
				
#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30) && 1 == ASURA_DIRECT3D_BGR_SUPPORT_SM30
				xGeometryVertex.m_afTextcoord1[ 0] = xTextureDataPtr.U[1][uPolygonVertex];
				xGeometryVertex.m_afTextcoord1[ 1] = xTextureDataPtr.V[1][uPolygonVertex];
				
				xGeometryVertex.m_xTangent = xElementTangent;
				xGeometryVertex.m_xBinormal = xElementBinormal;
				
				float fBoneWeight0 = 0.0f;
				float fBoneWeight1 = 0.0f;
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
				if( NULL != pxPaintedColourData)
				{
					//uint32 uVertexIndex = xPolygonDataPtr.VertexID[ uPolygonVertex];
					//Asura_Assert( uVertexIndex < uElementVertexCount, "Element Vertex Index Out of Range");
					
					{
						const Asura_Mesh_PaintedColours &xPaintedPolygonColour = pxPaintedColourData[ uElementPolygonIndex];
						
						const u_int uColourRGBA = xPaintedPolygonColour.Colour[ uPolygonVertex];
						fBoneWeight0 = (0xFF & (uColourRGBA >> 16)) / float( 0xFF);
						fBoneWeight1 = (0xFF & (uColourRGBA >> 8)) / float( 0xFF);
					}
				}
				else
				{
					if( NULL != pxTypeBasicInteractive)
					{
						fBoneWeight0 = xBoneWeightData.CalculateBoneWeight( xGeometryVertex.m_xPosition);
					}
				}
#endif
				xGeometryVertex.m_fBoneWeight0 = fBoneWeight0;
				xGeometryVertex.m_fBoneWeight1 = fBoneWeight1;
#endif				
				
				xGeometry.m_xArrayIndices[ uGeometryVertexCount] = uGeometryVertexCount;
				
			}
		}
	}
	
	Asura_Assert( static_cast< uint32>( pxElementMesh->NumberOfTriangles) == uGeometryTriangleCount, "Difference in the number of triangles in the element and the geometry");
	Asura_Assert( xGeometry.m_xArrayVertices.Size() == uGeometryVertexCount, "Difference in the number of vertices in the element and the geometry");
	
#if defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM )
	xGeometry.xMaterial.SetMaterialResponseHashID( xMaterialResponseHashID);
#else
	xGeometry.xMaterial.SetMaterial( iMaterialID);
#endif

#if 0
	{
		Asura_Assert( iMaterialID != ASURA_TEXTURE_INVALID, "There is not material on this mesh");
		const int iTextureID = Asura_Material_System::GetTextureID( iMaterialID);
		xGeometry.m_iTextureDiffuse = iTextureID;
	}
#endif
	
	Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_OptimiseVertexCount( xGeometry.m_xArrayVertices, xGeometry.m_xArrayIndices);
	Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_OptimiseVertexOrder( xGeometry.m_xArrayVertices, xGeometry.m_xArrayIndices);

	return( true);
}
#endif

template< class Tgeometry>
static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_Geometry( Asura_Direct3D_BGR_Type& xBGRType, Asura_LinkList< Tools_BotanicalElementLOD*>& xListElements, Asura_Botanical_Array< Tgeometry>& xArrayGeometry)
{
	xArrayGeometry.Clear();
	xArrayGeometry.SetSize( xListElements.Size());
	
	Asura_Bounding_Box xBoundingBox;
	
	xBoundingBox.Reset();
	
	uint32 uGeometryIndex = 0;
	for( Asura_ListElt< Tools_BotanicalElementLOD*>* pItElement = xListElements.Front(); NULL != pItElement; pItElement = pItElement->Next(), uGeometryIndex += 1)
	{
		const Tools_BotanicalElementLOD* pxElement = pItElement->Data();
		Tgeometry& tGeometry = xArrayGeometry[ uGeometryIndex];
		
		Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_ConvertLOD( pxElement, tGeometry.m_xLodRange);
		Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_CopyMesh( xBGRType, *pxElement, tGeometry);
		
		xBoundingBox.Set( tGeometry.GetBoundingBox());
	}
	
	{
		Asura_Bounding_Box xBGRTypeBoundingBox = xBGRType.GetBoundingBox();
		xBGRTypeBoundingBox.Set( xBoundingBox);
		xBGRType.SetBoundingBox( xBGRTypeBoundingBox);
	}
	
	return( true);
}

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_Geometry( Asura_Direct3D_BGR_Type& xBGRType, Asura_LinkList< Tools_BotanicalElementLOD*>& xListElements, Asura_Botanical_Array< Asura_Direct3D_BGR_Type_GeometryBillboard>& xArrayGeometry)
{
	u_int uImposterCount = 0;
	for( Asura_ListElt< Tools_BotanicalElementLOD*>* pItElement = xListElements.Front(); NULL != pItElement; pItElement = pItElement->Next())
	{
		const Tools_BotanicalElementLOD* pxElement = pItElement->Data();
		
		uImposterCount += pxElement->m_xImposterList.Size();
	}
	
	xArrayGeometry.Clear();
	xArrayGeometry.SetSize( xListElements.Size());

	Asura_Bounding_Box xBoundingBox;
	
	xBoundingBox.Reset();
	
	uint32 uGeometryIndex = 0;
	for( Asura_ListElt< Tools_BotanicalElementLOD*>* pItElement = xListElements.Front(); NULL != pItElement; pItElement = pItElement->Next(), uGeometryIndex += 1)
	{
		const Tools_BotanicalElementLOD* pxElement = pItElement->Data();
		Asura_Direct3D_BGR_Type_GeometryBillboard& xGeometry = xArrayGeometry[ uGeometryIndex];
		
		Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_ConvertLOD( pxElement, xGeometry.m_xLodRange);
		
		xBGRType.m_fLodLimitDeltaImposterNear = Asura_Maths::Min( xBGRType.m_fLodLimitDeltaImposterNear, xGeometry.m_xLodRange.m_afDelta[ Asura_Direct3D_BGR_Type_GeometryLODRange::LOW_0]);
		
		Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_CopyMesh( xBGRType, *pxElement, xGeometry);
		
		xBoundingBox.Set( xGeometry.GetBoundingBox());
		
		xBGRType.m_bHasImposters = true;
	}
	
	{
		Asura_Bounding_Box xBGRTypeBoundingBox = xBGRType.GetBoundingBox();
		
		xBGRTypeBoundingBox.Set( xBoundingBox);
		xBGRType.SetBoundingBox( xBGRTypeBoundingBox);
	}
	
	
	return( true);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_SectionVersion( const Tools_BotanicalShape& xBotanicalToolShape, Asura_Direct3D_BGR_Type& xBGRType, const Asura_BGR_TypeID& xTypeID)
{
	// DLH - Create Lists of unique SectionID and VersionID
	xBGRType.m_xHashTree_xSectionID.Reset();
	xBGRType.m_xHashTree_xVersionID.Reset();

	{
		Asura_String xStringBlank( "");
		xBGRType.m_xHashTree_xSectionID.AddNode( xStringBlank.GetHash(), 0);
		xBGRType.m_xHashTree_xVersionID.AddNode( xStringBlank.GetHash(), 0);
	}

	const u_int uSectionCount = xBGRType.m_xArray_xSections.Size();
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Asura_Direct3D_BGR_Section& xBGRSection = xBGRType.m_xArray_xSections[ uSectionIndex];
		
		{
			Asura_Hash_ID xSectionHashID = xBGRSection.m_xName.GetHash();
			if( -1 == xBGRType.m_xHashTree_xSectionID.GetIndexOfHash( xSectionHashID))
			{
				xBGRType.m_xHashTree_xSectionID.AddNode( xSectionHashID, static_cast< Asura_BGR_Section_ID>( xBGRType.m_xHashTree_xSectionID.GetNumberOfNodes()));
			}
		}
		
		const u_int uVersionCount = xBGRSection.m_xArray_xVersions.Size();
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Asura_Direct3D_BGR_Section_Version& xBGRSectionVersion = xBGRSection.m_xArray_xVersions[ uVersionIndex]; 
			
			{
				Asura_Hash_ID xSectionVersionHashID = xBGRSectionVersion.m_xName.GetHash();
				if( -1 == xBGRType.m_xHashTree_xVersionID.GetIndexOfHash( xSectionVersionHashID))
				{
					xBGRType.m_xHashTree_xVersionID.AddNode( xSectionVersionHashID, static_cast< Asura_BGR_Version_ID>( xBGRType.m_xHashTree_xVersionID.GetNumberOfNodes()));
				}
			}
		}
	}
	
	const u_int uVersionIDCount = xBGRType.m_xHashTree_xVersionID.GetNumberOfNodes();
	
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Asura_Direct3D_BGR_Section& xBGRSection = xBGRType.m_xArray_xSections[ uSectionIndex];
		const u_int uVersionCount = xBGRSection.m_xArray_xVersions.Size();

		xBGRSection.m_xArray_uVersionIDToVersionIndex.SetSize( uVersionIDCount);
		xBGRSection.m_xArray_uVersionIndexToVersionID.SetSize( uVersionCount);

		for( u_int uVersionIDIndex = 0 ; uVersionIDIndex < uVersionIDCount; uVersionIDIndex += 1)		
		{
			xBGRSection.m_xArray_uVersionIDToVersionIndex[ uVersionIDIndex] = ASURA_BGR_VERSION_ID_OFF;
		}
		
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Asura_Direct3D_BGR_Section_Version& xBGRSectionVersion = xBGRSection.m_xArray_xVersions[ uVersionIndex]; 
			Asura_Hash_ID xSectionVersionHashID = xBGRSectionVersion.m_xName.GetHash();
			
			Asura_BGR_Version_ID* pxVersionID = xBGRType.m_xHashTree_xVersionID.GetDataByHash( xSectionVersionHashID);
			Asura_Assert( NULL != pxVersionID, "");
			if( NULL == pxVersionID)
			{
				continue;
			}
			
			Asura_BGR_Version_ID xVersionID = *pxVersionID;
			xBGRSection.m_xArray_uVersionIndexToVersionID[ uVersionIndex] = xVersionID;
			xBGRSection.m_xArray_uVersionIDToVersionIndex[ xVersionID] = uVersionIndex;
		}
		
#if defined( ASURA_ASSERT)	
		if( 0 < uVersionIDCount)
		{
			Asura_Assertf( ASURA_BGR_VERSION_ID_OFF != xBGRSection.m_xArray_uVersionIDToVersionIndex[ 0], ("There is no default Version mesh for Botanical Type '%s' Section '%s'.\nThis will produce undefined behaviour.", xBGRType.m_xName.GetString(), xBGRSection.m_xName.GetString()));
		}
#endif
	}
	
	return( true);
}
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_StateTables( const Tools_BotanicalShape& xBotanicalToolShape, Asura_Direct3D_BGR_Type& xBGRType, const Asura_BGR_TypeID& xTypeID)
{
	if( false == xBGRType.CreateStates( xBotanicalToolShape))
	{
		return( false);
	}
	
	if( false == xBGRType.CreateStateTables( xBotanicalToolShape))
	{
		return( false);
	}
	
	return( true);
}
#endif

static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type( const Tools_BotanicalShape& xBotanicalToolShape, Asura_Direct3D_BGR_Type& xBGRType, const Asura_BGR_TypeID& xTypeID)
{
	xBGRType.m_pxBotanicalToolShape = &xBotanicalToolShape;
	xBGRType.SetType( xTypeID);
	xBGRType.m_uInstanceMaxCount = 0;
	xBGRType.m_bHasImposters = false;
	
	xBGRType.SetBoundingRadius( -1e6);
	xBGRType.m_fLodLimitNear = 0;
//DeadCode CSB 25/04/2006 	xBGRType.m_fLodLimitFar = 100;
	xBGRType.m_fLodLimitFar = 1.0f / xBotanicalToolShape.m_fInvMaxDrawDistance;
	xBGRType.m_fLodLimitDelta = xBGRType.m_fLodLimitFar - xBGRType.m_fLodLimitNear;
	
	xBGRType.m_fLodLimitDeltaImposterNear = 1.0f;

	if( xBotanicalToolShape.m_fMaxDrawDistanceFOV <= 0.0f )
	{
		Tools_MessageLog::AddMessage( "Botanicals",
									  "Invalid Data",
									  "Botanical shape %s: authored field of view is bad (%f)",
									  xBotanicalToolShape.GetName(),
									  xBotanicalToolShape.m_fMaxDrawDistanceFOV );

		Tools_MessageLog::ShowOverlay();
	}

	if( xBotanicalToolShape.m_uMaxDrawDistanceRes <= 0 )
	{
		Tools_MessageLog::AddMessage( "Botanicals",
									  "Invalid Data",
									  "Botanical shape %s: authored res is bad (%u)",
									  xBotanicalToolShape.GetName(),
									  xBotanicalToolShape.m_uMaxDrawDistanceRes );

		Tools_MessageLog::ShowOverlay();
	}

	{	
		const float fAuthorDistance = xBGRType.m_fLodLimitFar;
		
		const float fAuthorFOV = 0.5f * xBotanicalToolShape.m_fMaxDrawDistanceFOV;
		const float fAuthorTanFOV = Asura_Maths::Tangent( fAuthorFOV);
		const float fAuthorWidth = fAuthorDistance * fAuthorTanFOV;
		
		xBGRType.m_fLodAuthoredFOVWidth = fAuthorWidth;
	}
	
	xBGRType.m_fLodAuthoredRes = static_cast< float>( xBotanicalToolShape.m_uMaxDrawDistanceRes);

	{	
		Asura_Bounding_Box xBoundingBox;
		xBoundingBox.Reset();
		xBGRType.SetBoundingBox( xBoundingBox);
	}
	
	xBGRType.m_xName = xBotanicalToolShape.GetName();
	
	xBGRType.m_xName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( xBotanicalToolShape.GetName());
	
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
	if( true == xBotanicalToolShape.m_bIsInteractive)
	{
		Asura_BGR_Type_BasicInteractive* pxBasicInteractive = xBGRType.CreateBasicInteractive();
		
		Asura_BGR_Type_BasicInteractive& xBasicInteractive = *pxBasicInteractive;
		
		
		xBasicInteractive.Setup( xBotanicalToolShape);
	#if 0
		xBasicInteractive.m_xBonePosition0 = xBotanicalToolShape.m_xInteractiveBone0;
		xBasicInteractive.m_xBonePosition1 = xBotanicalToolShape.m_xInteractiveBone1;
		xBasicInteractive.m_fMass = xBotanicalToolShape.m_fMass;
		xBasicInteractive.m_fDrag = xBotanicalToolShape.m_fDrag;
		xBasicInteractive.m_fSpringStrength = xBotanicalToolShape.m_fSpringStrength;
		xBasicInteractive.m_fSpringLength = xBotanicalToolShape.m_fSpringLength;
		xBasicInteractive.m_fBoneWeightGamma = xBotanicalToolShape.m_fBoneWeightGamma;
		//xBasicInteractive.m_fBoneLength = (xBasicInteractive.m_xBonePosition1 - xBasicInteractive.m_xBonePosition0).Magnitude();
		xBasicInteractive.m_fBoneLength = xBasicInteractive.m_xBonePosition1.Magnitude();
	#endif
	}
	else
	{
		// DLH - 20 November 2006 10:56:05 - This needs to be cleared as loading a new shape the editor does not clear the data structures
		xBGRType.ClearBasicInteractive();
	}
#endif

	const u_int uSectionCount = xBotanicalToolShape.m_xHashTree_xSections.GetNumberOfNodes();

	xBGRType.m_xArray_xSections.Clear();
	xBGRType.m_xArray_xSections.SetSize( uSectionCount);
	for( u_int uSectionIndex = 0; uSectionIndex < uSectionCount; uSectionIndex += 1)
	{
		Asura_Direct3D_BGR_Section& xBGRSection = xBGRType.m_xArray_xSections[ uSectionIndex];
		Tools_BotanicalSection* pxToolSection = xBotanicalToolShape.m_xHashTree_xSections.GetDataByIndex( uSectionIndex);
		
		xBGRSection.m_xName = pxToolSection->m_xName;
		
		const u_int uVersionCount = pxToolSection->m_xHashTree_xVersions.GetNumberOfNodes();
		
		xBGRSection.m_xArray_xVersions.SetSize( uVersionCount);
		for( u_int uVersionIndex = 0; uVersionIndex < uVersionCount; uVersionIndex += 1)
		{
			Asura_Direct3D_BGR_Section_Version& xBGRSectionVersion = xBGRSection.m_xArray_xVersions[ uVersionIndex]; 
			Tools_BotanicalVersion* pxToolVersion = pxToolSection->m_xHashTree_xVersions.GetDataByIndex( uVersionIndex);
			
			xBGRSectionVersion.m_xName = pxToolVersion->m_xName;
			
			Asura_LinkList< Tools_BotanicalElementLOD*> xListImposters;
			Asura_LinkList< Tools_BotanicalElementLOD*> xListBranches;
			Asura_LinkList< Tools_BotanicalElementLOD*> xListFronds;
			Asura_LinkList< Tools_BotanicalElementLOD*> xListLeaves;
			
			for( Asura_ListElt< Tools_BotanicalElement*>* pxItElement = pxToolVersion->m_xList_pxElements.Front(); NULL != pxItElement; pxItElement = pxItElement->Next())
			{
				Tools_BotanicalElement* pxBotanicalInstance = pxItElement->Data();
				
				for( Asura_ListElt< Tools_BotanicalElementLOD*>* pItBotanicalLOD = pxBotanicalInstance->m_xLODList.Front();	NULL != pItBotanicalLOD; pItBotanicalLOD = pItBotanicalLOD->Next())
				{
					Tools_BotanicalElementLOD* pxBotanicalLOD = pItBotanicalLOD->Data();
					
					switch( pxBotanicalInstance->m_eType)
					{
						case TOOLS_BOTANICALS_ELEMENTTYPE_IMPOSTER:
						{
							xListImposters.Push_Front( pxBotanicalLOD);
							break;
						}
						case TOOLS_BOTANICALS_ELEMENTTYPE_1SIDED_MESH:
						{
							xListBranches.Push_Front( pxBotanicalLOD);
							break;
						}
						case TOOLS_BOTANICALS_ELEMENTTYPE_2SIDED_MESH:
						{
							xListFronds.Push_Front( pxBotanicalLOD);
							break;
						}
						case TOOLS_BOTANICALS_ELEMENTTYPE_BILLBOARDS:
						{
							xListLeaves.Push_Front( pxBotanicalLOD);
							break;
						}
						default:
						{
							Asura_Assert( 0, "Unknown Botanical Element Type");
							break;
						}
					}
				}
			}

			Asura_Direct3D_BGR_Geometry& xGeometry = xBGRSectionVersion.m_xGeometry;
		
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_GEOMETRY3D)
			Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_Geometry( xBGRType, xListBranches, xGeometry.m_xArrayBranchs);
			Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_Geometry( xBGRType, xListFronds, xGeometry.m_xArrayFronds);
	#endif
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_LEAVES)
			Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_Geometry( xBGRType, xListLeaves, xGeometry.m_xArrayLeaves);
	#endif
	#if defined( ASURA_DIRECT3D_BGR_SUPPORT_IMPOSTERS)
			Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_Geometry( xBGRType, xListImposters, xGeometry.m_xArrayBillboards);
	#endif
		}
		
#if !defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) || 1 != ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
		Asura_Assertf( xBGRSection.m_xArray_xVersions.Size() <= 1, ("There are too many versions on this. Only 1 supported but %d present.", xBGRSection.m_xArray_xVersions.Size()));
#endif
	}


#if !defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) || 1 != ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	Asura_Assertf( xBGRType.m_xArray_xSections.Size() <= 1, ("There are too many sections on this. Only 1 supported but %d present.", xBGRType.m_xArray_xSections.Size()));
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_SectionVersion( xBotanicalToolShape, xBGRType, xTypeID);
#endif

#if defined( ASURA_BOTANICAL_SUPPORT_STATES) && 1 == ASURA_BOTANICAL_SUPPORT_STATES
	Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type_StateTables( xBotanicalToolShape, xBGRType, xTypeID);
#endif

	xBGRType.CalcBoundingZeroRadius();	
	
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
	xBGRType.GetDynamicDataLayout().SetupDynamicData( xBGRType);
#endif
		
	return( true);
}

void Asura_BGR_InstanceTransform::Set( const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const float fScale)
{
	this->m_xPosition = xPosition;

	{
		D3DXMATRIX xTransform;
		
		xTransform._11 = xOrientation.e11;
		xTransform._12 = xOrientation.e21;
		xTransform._13 = xOrientation.e31;
		xTransform._14 = 0.0f;
		
		xTransform._21 = xOrientation.e12;
		xTransform._22 = xOrientation.e22;
		xTransform._23 = xOrientation.e32;
		xTransform._24 = 0.0f;
		
		xTransform._31 = xOrientation.e13;
		xTransform._32 = xOrientation.e23;
		xTransform._33 = xOrientation.e33;
		xTransform._34 = 0.0f;
		
		xTransform._41 = 0.0f;
		xTransform._42 = 0.0f;
		xTransform._43 = 0.0f;
		xTransform._44 = 1.0f;

		{
			D3DXMATRIX xScale;
			
			D3DXMatrixScaling( &xScale, fScale, fScale, fScale);
			D3DXMatrixMultiply( &xTransform, &xTransform, &xScale);

			xTransform._41 = xPosition.GetX();
			xTransform._42 = xPosition.GetY();
			xTransform._43 = xPosition.GetZ();

			this->m_xTransformMatrix = xTransform;
		}
	}

	this->m_fScale = fScale;
}

Asura_Matrix_3x3 Asura_BGR_InstanceTransform::GetOrientation() const
{
	Asura_Matrix_3x3 xOrientation;
	D3DXMATRIX xTransform;
	
	const float fInvScale = 1.0f / this->m_fScale;
	
	D3DXMATRIX xScale;
	
	D3DXMatrixScaling( &xScale, fInvScale, fInvScale, fInvScale);
	D3DXMatrixMultiply( &xTransform, &xScale, &this->m_xTransformMatrix);

	xOrientation.e11 = xTransform._11;
	xOrientation.e21 = xTransform._12;
	xOrientation.e31 = xTransform._13;
				   	
	xOrientation.e12 = xTransform._21;
	xOrientation.e22 = xTransform._22;
	xOrientation.e32 = xTransform._23;
				   	
	xOrientation.e13 = xTransform._31;
	xOrientation.e23 = xTransform._32;
	xOrientation.e33 = xTransform._33;
	
	return( xOrientation);
}

static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_InstanceHeader( Asura_Direct3D_BGR_Manager& xBGRManager, const Asura_Direct3D_BGR_Type* pxBGRType, Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader, const uint32 uInstanceIndex, const Asura_Botanicals_Instance* pxBotanicalInstance)
{
	Asura_BGR_InstanceTransform xInstanceTransform;
	
	xInstanceTransform.Set( pxBotanicalInstance->xPosition, pxBotanicalInstance->xOrientation, pxBotanicalInstance->fScale);
	
	

	Asura_BGR_InstanceLighting xInstanceLighting;

	{
		Asura_SphericalHarmonics_Order2_RGB xSH;
		Asura_GI_Solution::GetStaticIrradiance( xSH, Asura_Vector_3(xInstanceTransform.m_xPosition) );

		xInstanceLighting.m_afSphericalR[0] = xSH.m_afRedCoefficients[1] * pxBotanicalInstance->fBrightness;
		xInstanceLighting.m_afSphericalR[1] = xSH.m_afRedCoefficients[2] * pxBotanicalInstance->fBrightness;
		xInstanceLighting.m_afSphericalR[2] = xSH.m_afRedCoefficients[3] * pxBotanicalInstance->fBrightness;
		xInstanceLighting.m_afSphericalR[3] = xSH.m_afRedCoefficients[0] * pxBotanicalInstance->fBrightness;

		xInstanceLighting.m_afSphericalG[0] = xSH.m_afGreenCoefficients[1] * pxBotanicalInstance->fBrightness;
		xInstanceLighting.m_afSphericalG[1] = xSH.m_afGreenCoefficients[2] * pxBotanicalInstance->fBrightness;
		xInstanceLighting.m_afSphericalG[2] = xSH.m_afGreenCoefficients[3] * pxBotanicalInstance->fBrightness;
		xInstanceLighting.m_afSphericalG[3] = xSH.m_afGreenCoefficients[0] * pxBotanicalInstance->fBrightness;

		xInstanceLighting.m_afSphericalB[0] = xSH.m_afBlueCoefficients[1] * pxBotanicalInstance->fBrightness;
		xInstanceLighting.m_afSphericalB[1] = xSH.m_afBlueCoefficients[2] * pxBotanicalInstance->fBrightness;
		xInstanceLighting.m_afSphericalB[2] = xSH.m_afBlueCoefficients[3] * pxBotanicalInstance->fBrightness;
		xInstanceLighting.m_afSphericalB[3] = xSH.m_afBlueCoefficients[0] * pxBotanicalInstance->fBrightness;
	}

	Asura_Direct3D_BGR_Instance* pxInstance = pxInstanceHeader->GetDirect3DInstance( uInstanceIndex);
	
	pxInstance->SetActive( true);
	pxInstance->SetInstanceData( xInstanceTransform, xInstanceLighting);
	pxInstance->SetInstanceBrightness(pxBotanicalInstance->fBrightness);
	
	pxInstanceHeader->ExtendBoundingBox( pxBGRType, pxBotanicalInstance->xPosition, pxBotanicalInstance->xOrientation, pxBotanicalInstance->fScale);
	
	return( true);
}

void Asura_Direct3D_BGR_Instances_Header::ExtendBoundingBox( const Asura_Direct3D_BGR_Type* pxBGRType, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation, const float fScale)
{
	Asura_Bounding_Box xInstanceBoundingBox;

	xInstanceBoundingBox.FromOBB( pxBGRType->GetBoundingBox(), xOrientation);
	xInstanceBoundingBox.Scale( Asura_Vector_3( fScale, fScale, fScale));
	xInstanceBoundingBox.Displace( xPosition);
	
	this->m_xBoundingBox.Set( xInstanceBoundingBox);
}

static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_CollectInstances_Optimised( const Asura_LinkList<Asura_Botanicals_Instance*>& xListBotanicalToolInstances, Asura_LinkList< Asura_Direct3D_BGR_MDS_ToolShapeType>& xListBotanicalToolShape, Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType>& xListBotanicalMakeInstances)
{
	for( Asura_ListElt< Asura_Botanicals_Instance*>* pxItBotanicalInstances = xListBotanicalToolInstances.Front(); NULL != pxItBotanicalInstances; pxItBotanicalInstances = pxItBotanicalInstances->Next())
	{
		Asura_Botanicals_Instance* pxBotanicalInstance = pxItBotanicalInstances->Data();
		if( NULL == pxBotanicalInstance)
		{
			continue;
		}

		Asura_Botanicals_Template* pxBotanicalTemplate = Asura_Botanicals_TemplateManager::GetTemplate( pxBotanicalInstance->iTemplateIdx);
		if( NULL == pxBotanicalTemplate)
		{
			continue;
		}
		
		if( true == pxBotanicalTemplate->bMergeRenderMeshWithEnv)
		{
			continue;
		}
		
		if (pxBotanicalTemplate->eType != ASURA_BOTANICALS_TEMPLATE_BOTANICALSHAPE)
		{
			continue;
		}
		
		Asura_BotanicalShape* pxBotanicalShape = pxBotanicalTemplate->pxBotanicalShape;
		if ( NULL == pxBotanicalShape)
		{
			continue;		
		}
		
		Tools_BotanicalShape* pxBotanicalToolShape = pxBotanicalShape->GetToolsShape();
		if( NULL == pxBotanicalToolShape)
		{
			continue;
		}
		
		Asura_BGR_TypeID xTypeID;
		
		{
			Asura_Direct3D_BGR_MDS_ToolShapeType xToolShapeTypeFind;
			xToolShapeTypeFind.m_pxToolShape = pxBotanicalToolShape;
			Asura_ListElt< Asura_Direct3D_BGR_MDS_ToolShapeType>* xItBotanicalToolShape = xListBotanicalToolShape.Find( xToolShapeTypeFind);
			if( NULL == xItBotanicalToolShape)
			{
				xTypeID = Asura_BGR_TypeID( xListBotanicalToolShape.Size() + 1);
				
				Asura_Direct3D_BGR_MDS_ToolShapeType xToolShapeType;
				
				xToolShapeType.m_pxToolShape = pxBotanicalToolShape;
				xToolShapeType.m_xTypeID = xTypeID;
				
				xListBotanicalToolShape.Push_Front( xToolShapeType);
			}
			else
			{
				const Asura_Direct3D_BGR_MDS_ToolShapeType& xToolShapeType = xItBotanicalToolShape->Data();
				
				xTypeID = xToolShapeType.m_xTypeID;
			}
		}
		
		Asura_Direct3D_BGR_MDS_InstanceType xInstanceShape;
		
		xInstanceShape.m_pxInstance = pxBotanicalInstance;
		xInstanceShape.m_pxToolShape = pxBotanicalToolShape;
		xInstanceShape.m_xTypeID = xTypeID;
		
		xListBotanicalMakeInstances.Push_Front( xInstanceShape);
	}
	
	return( true);
}

#if 1
static bool Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_CollectInstances_Editable( const Asura_LinkList<Asura_Botanicals_Instance*>& xListBotanicalToolInstances, Asura_LinkList< Asura_Direct3D_BGR_MDS_ToolShapeType>& xListBotanicalToolShape, Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType>& xListBotanicalMakeInstances)
{
	// DLH - 10 October 2006 15:20:17 - Add ALL the Botanical Tools Shapes to the Shape List
	{
		const u_int uTemplateCount = Asura_Botanicals_TemplateManager::GetNumTemplates();
		for( u_int uTemplateIndex = 0; uTemplateIndex < uTemplateCount; uTemplateIndex += 1)
		{
			Asura_Botanicals_Template* pxBotanicalTemplate = Asura_Botanicals_TemplateManager::GetTemplate( uTemplateIndex);
			if( NULL == pxBotanicalTemplate)
			{
				continue;
			}
			
			if (pxBotanicalTemplate->eType != ASURA_BOTANICALS_TEMPLATE_BOTANICALSHAPE)
			{
				continue;
			}
			
			Tools_BotanicalShape* pxBotanicalToolShape = NULL;
			{
				Asura_BotanicalShape* pxBotanicalShape = pxBotanicalTemplate->pxBotanicalShape;
				if ( NULL != pxBotanicalShape)
				{
					pxBotanicalToolShape = pxBotanicalShape->GetToolsShape();
				}
			}

			
			{
				const Asura_BGR_TypeID xTypeID = Asura_BGR_TypeID( pxBotanicalTemplate->iUID);
				Asura_Assert( xTypeID != Asura_BGR_TypeID::INVALID_ID, "Invalid TypeID");
				Asura_Direct3D_BGR_MDS_ToolShapeType xToolShapeType;
				
				xToolShapeType.m_pxToolShape = pxBotanicalToolShape;
				xToolShapeType.m_xTypeID = xTypeID;
				
				xListBotanicalToolShape.Push_Front( xToolShapeType);
			}
		}
	}
	
	for( Asura_ListElt< Asura_Botanicals_Instance*>* pxItBotanicalInstances = xListBotanicalToolInstances.Front(); NULL != pxItBotanicalInstances; pxItBotanicalInstances = pxItBotanicalInstances->Next())
	{
		Asura_Botanicals_Instance* pxBotanicalInstance = pxItBotanicalInstances->Data();
		if( NULL == pxBotanicalInstance)
		{
			continue;
		}

		Asura_Botanicals_Template* pxBotanicalTemplate = Asura_Botanicals_TemplateManager::GetTemplate( pxBotanicalInstance->iTemplateIdx);
		if( NULL == pxBotanicalTemplate)
		{
			continue;
		}
		
		if( true == pxBotanicalTemplate->bMergeRenderMeshWithEnv)
		{
			continue;
		}
		
		if (pxBotanicalTemplate->eType != ASURA_BOTANICALS_TEMPLATE_BOTANICALSHAPE)
		{
			continue;
		}
		
		Tools_BotanicalShape* pxBotanicalToolShape = NULL;
		{
			Asura_BotanicalShape* pxBotanicalShape = pxBotanicalTemplate->pxBotanicalShape;
			if ( NULL != pxBotanicalShape)
			{
				pxBotanicalToolShape = pxBotanicalShape->GetToolsShape();
			}
		}
		
		Asura_BGR_TypeID xTypeID( pxBotanicalTemplate->iUID);
		Asura_Assert( xTypeID != Asura_BGR_TypeID::INVALID_ID, "Invalid TypeID");

#if 0		
		{
			Asura_Direct3D_BGR_MDS_ToolShapeType xToolShapeTypeFind;
			xToolShapeTypeFind.m_pxToolShape = pxBotanicalToolShape;
			Asura_ListElt< Asura_Direct3D_BGR_MDS_ToolShapeType>* xItBotanicalToolShape = xListBotanicalToolShape.Find( xToolShapeTypeFind);
			Asura_Assertf( NULL != xItBotanicalToolShape, ("Cannot find a Tools Shape for a botanical Instance! '%s'", pxBotanicalToolShape->GetName()));
			if( NULL == xItBotanicalToolShape)
			{
			}
			else
			{
				const Asura_Direct3D_BGR_MDS_ToolShapeType& xToolShapeType = xItBotanicalToolShape->Data();
				
				xTypeID = xToolShapeType.m_xTypeID;
			}
		}
#endif
		
		Asura_Direct3D_BGR_MDS_InstanceType xInstanceShape;
		
		xInstanceShape.m_pxInstance = pxBotanicalInstance;
		xInstanceShape.m_pxToolShape = pxBotanicalToolShape;
		xInstanceShape.m_xTypeID = xTypeID;
		
		xListBotanicalMakeInstances.Push_Front( xInstanceShape);
	}
	
	return( true);
}
#endif

struct Asura_Direct3D_Botanicals_Game_Render_UpdateTypes_NewShapes
{
	Asura_BGR_TypeID m_xTypeID;
	Tools_BotanicalShape* m_pxToolsBortanicalShape;
};

#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
bool Asura_Direct3D_Botanicals_Game_Render::UpdateTypesInteractive( const Tools_BotanicalShape* pxTypeBotanicalToolShapeFind)
{
	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();
	
	// DLH - 17 October 2006 16:40:54 - Update Changed Types
	{
		const u_int uTypeCount = xBGRManager.m_xArray_xTypes.Size();
		for( u_int uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
		{
			Asura_Direct3D_BGR_Type& xBGRType = xBGRManager.m_xArray_xTypes[ uTypeIndex];
			const Tools_BotanicalShape* pxTypeBotanicalToolShape = xBGRType.m_pxBotanicalToolShape;
			if( pxTypeBotanicalToolShape != pxTypeBotanicalToolShapeFind)
			{
				continue;
			}
			
			Asura_BGR_TypeID xTypeID = xBGRType.GetType();
			if( xTypeID == Asura_BGR_TypeID::INVALID_ID)
			{
				continue;
			}

#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
			Asura_Assertf( false == pxTypeBotanicalToolShapeFind->m_bIsInteractive || NULL != xBGRType.GetBasicInteractive(), ("This is not an allowed State"));
			Asura_Assertf( true == pxTypeBotanicalToolShapeFind->m_bIsInteractive || NULL == xBGRType.GetBasicInteractive(), ("This is not an allowed State"));
			
			if( true == pxTypeBotanicalToolShapeFind->m_bIsInteractive)
			{
				Asura_BGR_Type_BasicInteractive* pxBasicInteractive = xBGRType.CreateBasicInteractive();
				
				pxBasicInteractive->Setup( *pxTypeBotanicalToolShapeFind);
			}
#endif	// defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
		}
	}

	return( true);
}
#endif

#if defined( ASURA_BOTANICALS_BGR_IN_EDIT)
bool Asura_Direct3D_Botanicals_Game_Render::UpdateTypes()
{
	bool bDirect3DDataInvalid = false;
	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();
	
	// DLH - 17 October 2006 16:40:54 - Update Changed Types
	{
		const u_int uTypeCount = xBGRManager.m_xArray_xTypes.Size();
		for( u_int uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
		{
			Asura_Direct3D_BGR_Type& xBGRType = xBGRManager.m_xArray_xTypes[ uTypeIndex];
			const Tools_BotanicalShape* pxTypeBotanicalToolShape = xBGRType.m_pxBotanicalToolShape;
			
			Asura_BGR_TypeID xTypeID = xBGRType.GetType();
			if( xTypeID == Asura_BGR_TypeID::INVALID_ID)
			{
				continue;
			}
			
			const u_int uTypeID = xTypeID.GetID();
			const u_int uTemplateIndex = Asura_Botanicals_TemplateManager::GetTemplateIndexFromUID( uTypeID);
			if(uTemplateIndex == (u_int)-1)
			{
				//Tools version of the template has been deleted , so skip this to avoid assert.
				//I'm not entirely sure how this works , and its possible that the asura type should be removed as well.
				//I suspect it will be cleared up properly when the level next gets loaded.
				//RAR 22/01/2008
				continue;
			}
			Asura_Botanicals_Template* pxBotanicalTemplate = Asura_Botanicals_TemplateManager::GetTemplate( uTemplateIndex);
			if( NULL == pxBotanicalTemplate)
			{
				continue;
			}
			
			if (pxBotanicalTemplate->eType != ASURA_BOTANICALS_TEMPLATE_BOTANICALSHAPE)
			{
				continue;
			}
			
			Asura_BotanicalShape* pxBotanicalShape = pxBotanicalTemplate->pxBotanicalShape;
			if ( NULL == pxBotanicalShape)
			{
				if( false == xBGRType.m_xName.IsBlank())
				{
					xBGRManager.ClearDirect3DData();
					
					xBGRType.ClearGeometry();
					//xBGRManager.DeleteTypeID( xTypeID);
				
					bDirect3DDataInvalid = true;
				}
				continue;		
			}
			
			Tools_BotanicalShape* pxBotanicalToolShape = pxBotanicalShape->GetToolsShape();
			if( NULL == pxBotanicalToolShape)
			{
				continue;
			}
			
			if( pxBotanicalToolShape == pxTypeBotanicalToolShape)
			{
				continue;
			}

			xBGRManager.ClearDirect3DData();
			
			Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type( *pxBotanicalToolShape, xBGRType, xTypeID);
			
			xBGRManager.InstanceHeaderTypeChanged( xBGRType);
			
			bDirect3DDataInvalid = true;
		}
	}
	
	{
		// DLH - 17 October 2006 16:41:09 - Find New Types
		Asura_LinkList< Asura_Direct3D_Botanicals_Game_Render_UpdateTypes_NewShapes> xListBotanicalToolShape;
		
		{
			const u_int uTemplateCount = Asura_Botanicals_TemplateManager::GetNumTemplates();
			for( u_int uTemplateIndex = 0; uTemplateIndex < uTemplateCount; uTemplateIndex += 1)
			{
				Asura_Botanicals_Template* pxBotanicalTemplate = Asura_Botanicals_TemplateManager::GetTemplate( uTemplateIndex);
				if( NULL == pxBotanicalTemplate)
				{
					continue;
				}
				
				Tools_BotanicalShape* pxBotanicalToolShape = NULL;
				if (pxBotanicalTemplate->eType == ASURA_BOTANICALS_TEMPLATE_BOTANICALSHAPE)
				{
					Asura_BotanicalShape* pxBotanicalShape = pxBotanicalTemplate->pxBotanicalShape;
					if ( NULL != pxBotanicalShape)
					{
						pxBotanicalToolShape = pxBotanicalShape->GetToolsShape();
					}
				}
				
				Asura_BGR_TypeID xTypeID( pxBotanicalTemplate->iUID);
				Asura_Direct3D_BGR_Type* pxType = xBGRManager.GetDirect3DType( xTypeID);
				if( NULL != pxType)
				{
					continue;
				}
				
				Asura_Direct3D_Botanicals_Game_Render_UpdateTypes_NewShapes xNewShape;
				
				xNewShape.m_xTypeID = xTypeID;
				xNewShape.m_pxToolsBortanicalShape = pxBotanicalToolShape;
				
				xListBotanicalToolShape.Push_Front( xNewShape);
			}
		}
		
		uint32 uBotanicalToolShapeListSize = xListBotanicalToolShape.Size();
		if( 0 != uBotanicalToolShapeListSize)
		{
			xBGRManager.ClearDirect3DData();
			
			uint32 uBotanicalToolShapeIndex = xBGRManager.m_xArray_xTypes.Size();

			xBGRManager.m_xArray_xTypes.Exspand( uBotanicalToolShapeIndex + uBotanicalToolShapeListSize);
			
			for( Asura_ListElt< Asura_Direct3D_Botanicals_Game_Render_UpdateTypes_NewShapes>* pxItBotanicalToolShape = xListBotanicalToolShape.Front(); NULL != pxItBotanicalToolShape; pxItBotanicalToolShape = pxItBotanicalToolShape->Next(), uBotanicalToolShapeIndex += 1)
			{
				const Asura_Direct3D_Botanicals_Game_Render_UpdateTypes_NewShapes& xNewShape = pxItBotanicalToolShape->Data();
				
				const Asura_BGR_TypeID xTypeID = xNewShape.m_xTypeID;
				Asura_Direct3D_BGR_Type& xBGRType = xBGRManager.m_xArray_xTypes[ uBotanicalToolShapeIndex];
				
				Tools_BotanicalShape* pxBotanicalToolShape = xNewShape.m_pxToolsBortanicalShape;
				if( NULL != pxBotanicalToolShape)
				{
					Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type( *pxBotanicalToolShape, xBGRType, xTypeID);
					Asura_Assert( xBGRType.GetType() == xTypeID, "TypeID mismatch");
				}
				else
				{
					xBGRType.Clear();
				}
				
				// DLH - 18 October 2006 15:38:32 - Make Sure DirectX code knows to setup buffers for as many instances as possible in Edit mode
				xBGRType.m_uInstanceMaxCount = static_cast< uint16>( ~0L);
				xBGRType.SetType( xTypeID);
				
				xBGRManager.InstanceHeaderTypeChanged( xBGRType);
				
			}		
			
			xBGRManager.AddMissingTypeToModuleInstanceHeaderArrays();
			
			bDirect3DDataInvalid = true;
			xBGRManager.SetupTypeCache();
		}
	}
	
	if( true == bDirect3DDataInvalid)
	{
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
		xBGRManager.SetupSectionCache();
#endif
		xBGRManager.CreateDirect3DData();
	}

	return( true);
}
#endif

bool Asura_Direct3D_Botanicals_Game_Render::MakeDataSet( const Asura_LinkList<Asura_Botanicals_Instance*>& xListBotanicalToolInstances, const bool bEditableData)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_Botanicals_Game_Render::MakeDataSet")
	
	ClearDataSet();

	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();
	
	Asura_LinkList< Asura_Direct3D_BGR_MDS_ToolShapeType> xListBotanicalToolShape;
	Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType> xListBotanicalMakeInstances;

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_Botanicals_Game_Render::MakeDataSet CollectData");
		
	#if 1
		if( true == bEditableData)
		{
			Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_CollectInstances_Editable( xListBotanicalToolInstances, xListBotanicalToolShape, xListBotanicalMakeInstances);
		}
		else
	#endif
		{
			Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_CollectInstances_Optimised( xListBotanicalToolInstances, xListBotanicalToolShape, xListBotanicalMakeInstances);
		}
	}
	
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_Botanicals_Game_Render::MakeDataSet MakeMeshes");
		
		const uint32 uBotanicalToolShapeCount = xListBotanicalToolShape.Size();
		
		if( false == bEditableData && 0 == uBotanicalToolShapeCount)
		{
			return( true);
		}
		
		if( 0 != uBotanicalToolShapeCount)
		{
			// DLH - 12 October 2006 10:38:05 - Reserve Some Extra Space so the Editor Mode can add more Types later with out reallocation
			if( true == bEditableData)
			{
				xBGRManager.m_xArray_xTypes.Reserve( 2*uBotanicalToolShapeCount);
			}
			xBGRManager.m_xArray_xTypes.Exspand( uBotanicalToolShapeCount);
			
			{
				uint32 uBotanicalToolShapeIndex = 0;
				for( Asura_ListElt< Asura_Direct3D_BGR_MDS_ToolShapeType>* pxItBotanicalToolShape = xListBotanicalToolShape.Front(); NULL != pxItBotanicalToolShape; pxItBotanicalToolShape = pxItBotanicalToolShape->Next(), uBotanicalToolShapeIndex += 1)
				{
					Asura_Direct3D_BGR_MDS_ToolShapeType& xBotanicalShapeType = pxItBotanicalToolShape->Data();
					Asura_BGR_TypeID& xTypeID = xBotanicalShapeType.m_xTypeID;
					
					Asura_Direct3D_BGR_Type& xBGRType = xBGRManager.m_xArray_xTypes[ uBotanicalToolShapeIndex];
					
					Tools_BotanicalShape* pxBotanicalToolShape = xBotanicalShapeType.m_pxToolShape;
					if( NULL != pxBotanicalToolShape)
					{
						Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_Type( *pxBotanicalToolShape, xBGRType, xTypeID);
					}
					else
					{
						xBGRType.Clear();
						xBGRType.SetType( xTypeID);
					}
					
					if( true == bEditableData)
					{
						// DLH - 18 October 2006 15:38:32 - Make Sure DirectX code knows to setup buffers for as many instances as possible in Edit mode
						xBGRType.m_uInstanceMaxCount = static_cast< uint16>( ~0L);
					}
				}		
			}
		}
	}
	
#if 1
	// DLH - 11 April 2006 16:56:03 - Calc the module/type lists
	{
		// DLH - 12 April 2006 16:49:31 - Isolate all the instances into module and type
		const uint32 uTypeIDCeiling = xBGRManager.GetTypeIDCeiling();
		Asura_Assert( true == bEditableData || Asura_BGR_TypeID::INVALID_ID != uTypeIDCeiling, "No Type defined");
		
		Asura_Botanical_Array< Asura_Botanical_Array< Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType*> > > xArrayBotanicalModules_xArrayTypes_xList_pxInstances;
		
//		if( Asura_BGR_TypeID::INVALID_ID != uTypeIDCeiling)
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_Botanicals_Game_Render::MakeDataSet ModuleType");
		
			const uint32 uTypeIDCount =  (Asura_BGR_TypeID::INVALID_ID != uTypeIDCeiling) ? (1 + uTypeIDCeiling) : 0;
		
			const u_int uModuleCount = Asura_Module_System::GetNumberOfModules();
//			const u_int uModuleOutside = uModuleCount;
			
			xArrayBotanicalModules_xArrayTypes_xList_pxInstances.SetSize( 1 + uModuleCount);
			xBGRManager.m_xArray_xModule.SetSize( 1 + uModuleCount);

			for( Asura_ListElt< Asura_Direct3D_BGR_MDS_InstanceType>* pxItBotanicalInstances = xListBotanicalMakeInstances.Front(); NULL != pxItBotanicalInstances; pxItBotanicalInstances = pxItBotanicalInstances->Next())
			{
				Asura_Direct3D_BGR_MDS_InstanceType& xInstanceShape = pxItBotanicalInstances->Data();
				Asura_Botanicals_Instance* pxBotanicalInstance = xInstanceShape.m_pxInstance;
				
				int iModuleIndex = pxBotanicalInstance->iModuleIdx;
				if( iModuleIndex < 0 || uModuleCount <= static_cast< u_int>( iModuleIndex))
				{
					//ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_Botanicals_Game_Render::MakeDataSet FindPosition");
					
					iModuleIndex = xBGRManager.GetModuleIndexFromPosition( pxBotanicalInstance->xPosition, pxBotanicalInstance->xOrientation);
					pxBotanicalInstance->iModuleIdx = iModuleIndex;
				}
				
				Asura_Botanical_Array< Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType*> >& xArrayTypes_xList_pxInstances = xArrayBotanicalModules_xArrayTypes_xList_pxInstances[ iModuleIndex];
				
				if( 0 == xArrayTypes_xList_pxInstances.Size())
				{
					xArrayTypes_xList_pxInstances.SetSize( uTypeIDCount);
				}
				
				const uint32 uTypeID = xInstanceShape.m_xTypeID.GetID();
				Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType*>& xList_pxInstances = xArrayTypes_xList_pxInstances[ uTypeID];
				
				xList_pxInstances.Push_Front( &xInstanceShape);
			}
		}
		
		const u_int uModuleArraySize = xArrayBotanicalModules_xArrayTypes_xList_pxInstances.Size();
		
		// DLH - 12 April 2006 16:49:59 - Count the needed number of Instance Headers
		{
			uint32 uInstanceHeaderCount = 0;
			
			if( true == bEditableData)
			{
				// DLH - 12 October 2006 10:50:59 - If in edit mode allocate one header per type for each instance
				const uint32 uTypeIDCeiling = xBGRManager.GetTypeIDCeiling();

				if( Asura_BGR_TypeID::INVALID_ID != uTypeIDCeiling)
				{
					uInstanceHeaderCount = uModuleArraySize * (1 + uTypeIDCeiling);
				}
			}
			else
			{
				for( u_int uModuleIndex = 0; uModuleIndex < uModuleArraySize; uModuleIndex += 1)
				{
					Asura_Botanical_Array< Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType*> >& xArrayTypes_xList_pxInstances = xArrayBotanicalModules_xArrayTypes_xList_pxInstances[ uModuleIndex];
					
					const uint32 uTypeCount = xArrayTypes_xList_pxInstances.Size();
					
					for( uint32 uTypeIndex = 0; uTypeIndex < uTypeCount; uTypeIndex += 1)
					{
						Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType*>& xList_pxInstances = xArrayTypes_xList_pxInstances[ uTypeIndex];
						
						const u_int uInstanceCount = xList_pxInstances.Size();
						
						if( 0 == uInstanceCount)
						{
							continue;
						}
						
						uInstanceHeaderCount += 1;
					}
				}
			}
			
			if( true == bEditableData)
			{
				xBGRManager.m_xArray_pxInstanceHeaders.Reserve( 2 * uInstanceHeaderCount);
			}
			if( 0 < uInstanceHeaderCount)
			{
				xBGRManager.m_xArray_pxInstanceHeaders.Exspand( uInstanceHeaderCount);
			}
			
			for( u_int uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
			{
				xBGRManager.m_xArray_pxInstanceHeaders[ uInstanceHeaderIndex] = NULL;
			}
		}
		
		// DLH - 12 April 2006 16:50:22 - Create the headers
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_Botanicals_Game_Render::MakeDataSet MakeHeaders");
		
			const uint32 uTypeIDCeiling = xBGRManager.GetTypeIDCeiling();
			uint32 uInstanceHeaderIndex = 0;
			
			for( u_int uModuleIndex = 0; uModuleIndex < uModuleArraySize; uModuleIndex += 1)
			{
				Asura_Botanical_Array< Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType*> >& xArrayTypes_xList_pxInstances = xArrayBotanicalModules_xArrayTypes_xList_pxInstances[ uModuleIndex];
				Asura_Direct3D_BGR_Module& xModule = xBGRManager.m_xArray_xModule[ uModuleIndex];
				
				if( true == bEditableData && Asura_BGR_TypeID::INVALID_ID != uTypeIDCeiling)
				{
					uInstanceHeaderIndex = (1 + uTypeIDCeiling) * uModuleIndex;
				}
				
				xModule.m_uInstanceHeaderStartIndex = uInstanceHeaderIndex;
				xModule.m_uInstanceHeadeCount = 0;
				
				if( Asura_BGR_TypeID::INVALID_ID == uTypeIDCeiling)
				{
					continue;
				}
				
				const uint32 uInstanceTypeArraySize = xArrayTypes_xList_pxInstances.Size();
				const uint32 uInstanceTypeCount = (true == bEditableData) ? (1+uTypeIDCeiling) : uInstanceTypeArraySize;
				for( uint32 uTypeIndex = 0; uTypeIndex < uInstanceTypeCount; uTypeIndex += 1)
				{
					u_int uInstanceCount = 0;
						
					if( false == bEditableData || uTypeIndex < uInstanceTypeArraySize)
					{
						Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType*>& xList_pxInstances = xArrayTypes_xList_pxInstances[ uTypeIndex];
						
						uInstanceCount = xList_pxInstances.Size();
						
						if( 0 == uInstanceCount && false == bEditableData)
						{
							continue;
						}
					}
					
					Asura_Assert( true == bEditableData || 0 < uInstanceCount, "There must be instances to in this header");
					
					Asura_Direct3D_BGR_Instances_Header*& pxInstanceHeader = xBGRManager.m_xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
					uInstanceHeaderIndex += 1;
					Asura_Assert( NULL == pxInstanceHeader, "Instance Header Slot is already occupied");
					pxInstanceHeader = xBGRManager.CreateInstanceHeader( uInstanceCount);
					pxInstanceHeader->SetType( Asura_BGR_TypeID::INVALID_ID);
					
					if( true == bEditableData)
					{
						pxInstanceHeader->SetType( uTypeIndex);
					}
					
					pxInstanceHeader->m_uImposterVBOffset = 0;
					pxInstanceHeader->m_uImposterVBStart = 0;
					pxInstanceHeader->m_uImposterVBSize = 0;
					pxInstanceHeader->m_uImposterIBStart = 0;
					pxInstanceHeader->m_uImposterIBSize = 0;
					
					Asura_Bounding_Box xInstanceHeaderBoundingBox;
					
					xInstanceHeaderBoundingBox.Reset();
					pxInstanceHeader->SetBoundingBox( xInstanceHeaderBoundingBox);
					
					xModule.m_uInstanceHeadeCount += 1;
					
					Asura_Direct3D_BGR_Type* pxBGRType = NULL;
					if( true == bEditableData)
					{
						pxBGRType = xBGRManager.GetDirect3DType( pxInstanceHeader->GetType());
					}
					
					if( 0 != uInstanceCount )
					{
						Asura_LinkList< Asura_Direct3D_BGR_MDS_InstanceType*>& xList_pxInstances = xArrayTypes_xList_pxInstances[ uTypeIndex];
						
						uint32 uInstanceIndex = 0;
						for( Asura_ListElt< Asura_Direct3D_BGR_MDS_InstanceType*>* pxItBotanicalInstances = xList_pxInstances.Front(); NULL != pxItBotanicalInstances; pxItBotanicalInstances = pxItBotanicalInstances->Next(), uInstanceIndex += 1)
						{
							Asura_Assert( uInstanceIndex < pxInstanceHeader->GetInstanceCount(), "Not enought Space in InstanceHeader to store instance");
							
							const Asura_Direct3D_BGR_MDS_InstanceType* pxInstanceShape = pxItBotanicalInstances->Data();
							Asura_Assert( Asura_BGR_TypeID::INVALID_ID != pxInstanceShape->m_xTypeID.GetID(), "Bad ID");
							
							const Asura_Botanicals_Instance* pxBotanicalInstance = pxInstanceShape->m_pxInstance;
							if( NULL == pxBGRType)
							{
								pxBGRType = xBGRManager.GetDirect3DType( pxInstanceShape->m_xTypeID);
								Asura_Assert( pxInstanceHeader->GetType() == Asura_BGR_TypeID::INVALID_ID, "Instance Header has already been given an ID");
								pxInstanceHeader->SetType( pxInstanceShape->m_xTypeID);
							}
							
							Asura_Assert( NULL != pxBGRType, "BGR Type Seems to be missing");
							Asura_Assert( pxInstanceHeader->GetType() == pxInstanceShape->m_xTypeID, "TypeID mismatch");
							
							Asura_Direct3D_Botanicals_Game_Render_MakeDataSet_InstanceHeader( xBGRManager, pxBGRType, pxInstanceHeader, uInstanceIndex, pxBotanicalInstance);
						}
					}
				}
			}
			
			Asura_Assert( uInstanceHeaderIndex <= xBGRManager.m_xArray_pxInstanceHeaders.Size(), "Mismatch in the InstanceHeader Calculation");
			Asura_Assert( true == bEditableData || uInstanceHeaderIndex == xBGRManager.m_xArray_pxInstanceHeaders.Size(), "Mismatch in the InstanceHeader Calculation");
		}
	}
#endif

	
	// DLH - 23 March 2006 12:28:45 - Scan New instances and create accumulated values 
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_Botanicals_Game_Render::MakeDataSet SummeryData");
		
		const uint32 uInstanceHeaderCount = xBGRManager.m_xArray_pxInstanceHeaders.Size();
		for( uint32 uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
		{
			Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = xBGRManager.m_xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
			
			if( NULL == pxInstanceHeader)
			{
				continue;
			}

			const Asura_BGR_TypeID xTypeID = pxInstanceHeader->GetType();
			if( xTypeID == Asura_BGR_TypeID::INVALID_ID)
			{
				continue;
			}
			
			Asura_Direct3D_BGR_Type* pxBGRType = xBGRManager.GetDirect3DType( xTypeID);
			Asura_Assert( true == bEditableData || NULL != pxBGRType, "BGR Type Seems to be missing");
			if( NULL == pxBGRType)
			{
				continue;
			}
			
			const uint32 uInstanceCount = pxInstanceHeader->GetInstanceCount();
			
			pxBGRType->m_uInstanceMaxCount += uInstanceCount;
			
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
			pxInstanceHeader->SetupDynamicData( *pxBGRType);
			pxInstanceHeader->AllocateDynamicData( uInstanceCount);
#endif					
					
		}
	}
	
#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
	xBGRManager.SetupSectionCache();
#endif
	xBGRManager.SetupTypeCache();
	
	xBGRManager.CreateDirect3DData();
	
	return( true);
}

void Asura_BGR_Instance::Relight()
{
	Asura_AVector3 xInstancePosition;
	this->GetInstancePosition( xInstancePosition);	

	Asura_BGR_InstanceLighting xInstanceLighting;

	{
		Asura_SphericalHarmonics_Order2_RGB xSH;
		Asura_GI_Solution::GetStaticIrradiance( xSH, Asura_Vector_3( xInstancePosition) );
		
		xInstanceLighting.m_afSphericalR[0] = xSH.m_afRedCoefficients[1] * m_fBrightness;
		xInstanceLighting.m_afSphericalR[1] = xSH.m_afRedCoefficients[2] * m_fBrightness;
		xInstanceLighting.m_afSphericalR[2] = xSH.m_afRedCoefficients[3] * m_fBrightness;
		xInstanceLighting.m_afSphericalR[3] = xSH.m_afRedCoefficients[0] * m_fBrightness;

		xInstanceLighting.m_afSphericalG[0] = xSH.m_afGreenCoefficients[1] * m_fBrightness;
		xInstanceLighting.m_afSphericalG[1] = xSH.m_afGreenCoefficients[2] * m_fBrightness;
		xInstanceLighting.m_afSphericalG[2] = xSH.m_afGreenCoefficients[3] * m_fBrightness;
		xInstanceLighting.m_afSphericalG[3] = xSH.m_afGreenCoefficients[0] * m_fBrightness;

		xInstanceLighting.m_afSphericalB[0] = xSH.m_afBlueCoefficients[1] * m_fBrightness;
		xInstanceLighting.m_afSphericalB[1] = xSH.m_afBlueCoefficients[2] * m_fBrightness;
		xInstanceLighting.m_afSphericalB[2] = xSH.m_afBlueCoefficients[3] * m_fBrightness;
		xInstanceLighting.m_afSphericalB[3] = xSH.m_afBlueCoefficients[0] * m_fBrightness;
	}

	this->SetInstanceData( xInstanceLighting);
}

bool Asura_Direct3D_Botanicals_Game_Render::RelightEverything()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_Direct3D_Botanicals_Game_Render::RelightEverything")
	
	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();

	const uint32 uInstanceHeaderCount = xBGRManager.m_xArray_pxInstanceHeaders.Size();
	for( uint32 uInstanceHeaderIndex = 0; uInstanceHeaderIndex < uInstanceHeaderCount; uInstanceHeaderIndex += 1)
	{
		Asura_Direct3D_BGR_Instances_Header* pxInstanceHeader = xBGRManager.m_xArray_pxInstanceHeaders[ uInstanceHeaderIndex];
		if( NULL == pxInstanceHeader)
		{
			continue;
		}

		const uint32 uInstanceCount = pxInstanceHeader->GetInstanceCount();
		for( uint32 uInstanceIndex = 0; uInstanceIndex < uInstanceCount; uInstanceIndex += 1)			
		{
			Asura_Direct3D_BGR_Instance* pxInstance = pxInstanceHeader->GetDirect3DInstance( uInstanceIndex);
			
			pxInstance->Relight();
		}					
	}		
	
	xBGRManager.ClearDirect3DData();
	xBGRManager.CreateDirect3DData();
	
	return( true);
}

bool Asura_Direct3D_Botanicals_Game_Render::DeleteModule( const u_int uDeleteModuleIndex)
{
	Asura_Direct3D_BGR_Manager& xBGRManager = GetDirect3DBGRManager();
	
	const u_int uBGRModuleCount = xBGRManager.m_xArray_xModule.Size();
	const u_int uModuleCount = Asura_Module_System::GetNumberOfModules();
	Asura_Assert( 0 != uBGRModuleCount, "There are no modules in the BGR data!");
	Asura_Assertf( uModuleCount <= (uBGRModuleCount - 1), ("There is a miss match in the number of BGR modules (%d) and module system (%d) ", uBGRModuleCount, uModuleCount));
	Asura_Assertf( uDeleteModuleIndex < uBGRModuleCount, ("Module to delete (%d) has invalid index. BGR Module Count %d.", uDeleteModuleIndex, uBGRModuleCount));
	
	// DLH - 11 May 2006 11:40:17 - Move the Delete Module Data to the end of the Module List
	// DLH - 11 May 2006 11:40:17 - Move the Module Data at the the of the module list into the freed up deleted module
	
	Asura_Direct3D_BGR_Module& xBGRModuleDelete = xBGRManager.m_xArray_xModule[ uDeleteModuleIndex];
	Asura_Direct3D_BGR_Module& xBGRModuleLast = xBGRManager.m_xArray_xModule[ uModuleCount];
	
	Asura_Direct3D_BGR_Module xBGRModuleTemp = xBGRModuleDelete;
	
	xBGRModuleDelete = xBGRModuleLast;
	xBGRModuleLast = xBGRModuleTemp;

	return( true);
}

#endif
