//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE        :  Tools_Carpet_Mesh.cpp
//
//  Created by Dave Reed - 22/11/05
//
//  Copyright (C) 2005 Rebellion (Derby), All Rights Reserved.
//
//  'Carpet' grass editing/painting
//
//////////////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_CARPET

#include "Core/Asura_LinkList.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_TextureFlags.h"
#include "Core/Asura_Timers.h"

#include "Carpet/Asura_Carpet_StyleManager.h"
#include "Carpet/Asura_Carpet_MeshManager.h"
#include "Carpet/Asura_Direct3D_Carpet_Game_Render.h"
#include "Carpet/Asura_Direct3D_Carpet_Render.h"
#include "Carpet/Tools_Mode_Carpet.h"
#include "Carpet/Tools_Carpet_Mesh.h"

#include "Debugging/Asura_Debug_Timers.h"

#include "Lighting/Tools_PhotonMapping_System.h"
#include "Lighting/Tools_EnvironmentalLightTransport.h"

#include "Mesh/Asura_Mesh.h"

#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Direct3d.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Module.h"

#include "Lighting/Tools_Environment_LOS.h"
#include "Tools_System.h"


struct Tools_Carpet_PolyInAdjustSphere
{
	uint32 uModuleIdx;
	uint32 uPolyIdx;
};


Asura_LinkList<Asura_Carpet_Triangle>*				Tools_Carpet_Mesh::s_pxModuleLists=NULL;
int													Tools_Carpet_Mesh::s_iNumLists=0;
int													Tools_Carpet_Mesh::s_iTriangleCount=0;
float												Tools_Carpet_Mesh::s_fTimer=0;

uint32												g_uCurrentModule;
Asura_LinkList<Tools_Carpet_PolyInAdjustSphere>		g_xAdjustPolyList;




void Tools_Carpet_Mesh::Init()
{
	// Clear anything that's already allocated
	Reset();

	// Create the empty lists (one per module)
	int iNumberOfModules=Asura_Module_System::GetNumberOfModules();

	s_pxModuleLists = new Asura_LinkList<Asura_Carpet_Triangle>[iNumberOfModules];
	s_iNumLists		= iNumberOfModules;
	s_iTriangleCount=0;
}



void Tools_Carpet_Mesh::Reset()
{
	// Clear the mesh, free all memory
	if (s_pxModuleLists)
	{
		// Clear the lists
		for(int i=0;i<s_iNumLists;i++)
		{
			s_pxModuleLists[i].Clear();
		}

		delete [] s_pxModuleLists;
		s_pxModuleLists=NULL;
		s_iNumLists=0;
	}
}



void Tools_Carpet_Mesh::Render(bool bShowPolys, int iSelectedStyle)
{
	SCOPEDPIXEVENT(( 0, L"Carpet Tools Render" ));

	{
		SCOPEDPIXEVENT(( 0, L"Setup" ));

		Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_GENERICVERTEX);
		Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
		Asura_Render::CurrentStates.RequestTranslucencyMode(ASURA_TRANSLUCENCY_NORMAL);
		Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
		Asura_Render::CurrentStates.RequestTextureIndex(0,ASURA_TEXTURE_NONE);					
		Asura_Render::CurrentStates.RequestAlphaTest( false );
		Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);	
		
		Asura_Direct3D::D3DDevice->SetSamplerState	(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
		Asura_Direct3D::D3DDevice->SetSamplerState	(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
		Asura_Direct3D::D3DDevice->SetSamplerState	(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	}

	// Render the mesh as triangles
	if (bShowPolys)
	{
		SCOPEDPIXEVENT(( 0, L"Show carpet coverage" ));

		for(int i=0;i<s_iNumLists;i++)
		{		
			// Is this module visible?				
			if (Asura_Module_System::IsModuleInFrustum(i))
			{
				for(Asura_ListElt<Asura_Carpet_Triangle>* pxElt=s_pxModuleLists[i].Front(); pxElt; pxElt=pxElt->Next())
				{
					SCOPEDPIXEVENT(( 0, L"Carpet triangle" ));

					Asura_Carpet_Triangle *pxTri=&pxElt->Data();


					D3DGenericVertex axVerts[4];
					for (int j=0;j<3;j++)
					{
						axVerts[j].U = 0;
						axVerts[j].V = 0;
						axVerts[j].X = pxTri->xVerts[j].xPos.x;
						axVerts[j].Y = pxTri->xVerts[j].xPos.y-0.1f;
						axVerts[j].Z = pxTri->xVerts[j].xPos.z;

						if (pxTri->iStyle==iSelectedStyle)
							axVerts[j].Diffuse = 0xffff40 + (((int)(pxTri->xVerts[j].fDensity*255.0f))<<24);
						else
							axVerts[j].Diffuse = 0x40ff40 + (((int)(pxTri->xVerts[j].fDensity*255.0f))<<24);						
					}

					axVerts[3]=axVerts[0];


					D3DXMATRIX xMatWorld;
					xMatWorld._12 = xMatWorld._13 = xMatWorld._14 = xMatWorld._21 = xMatWorld._23 = xMatWorld._24 = 0.0f;
					xMatWorld._31 = xMatWorld._32 = xMatWorld._34 = xMatWorld._41 = xMatWorld._42 = xMatWorld._43 = 0.0f;
					xMatWorld._11 = xMatWorld._22 = xMatWorld._33 = 1.0f;
					xMatWorld._44 = 1.0f;
					xMatWorld._41 = 0.0f;
					xMatWorld._42 = 0.0f;
					xMatWorld._43 = 0.0f;

					float fZBias=-0.00002f;
					Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_DEPTHBIAS, *((DWORD *)&fZBias) );

					Asura_Direct3D::D3DDevice->SetTransform( D3DTS_WORLD,      &xMatWorld );

					Asura_Direct3D::D3DDevice->DrawPrimitiveUP(D3DPT_LINESTRIP,
															3,(void*)axVerts,sizeof(D3DGenericVertex));

					
					for(int j=0;j<3;j++)
					{
						if (pxTri->iStyle==iSelectedStyle)
							axVerts[j].Diffuse = (((axVerts[j].Diffuse>>24)>>1)<<24)+0xA0A040;
						else
							axVerts[j].Diffuse = (((axVerts[j].Diffuse>>24)>>1)<<24)+0x40A040;
					
				//		axVerts[j].Diffuse=pxTri->xVerts[n].uColour|0xff000000;
					}


					Asura_Direct3D::D3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLELIST,
															1,(void*)axVerts,sizeof(D3DGenericVertex));

					fZBias=0;
					Asura_Direct3D::D3DDevice->SetRenderState(D3DRS_DEPTHBIAS, *((DWORD *)&fZBias) );
	
				}
			}
		}
	}


	// Renderstates are now set up in here
	Asura_Direct3D_Carpet_Render::BeginRender();

	for(int i=0;i<s_iNumLists;i++)
	{		
		// Is this module visible?				
		if (Asura_Module_System::IsModuleInFrustum(i))
		{
			Asura_Carpet_Mesh_Data_Frame xDataFrame( Asura_Timers::GetLocalUnscaledTime());
			for(Asura_ListElt<Asura_Carpet_Triangle>* pxElt=s_pxModuleLists[i].Front(); pxElt; pxElt=pxElt->Next())
			{				
				Asura_Carpet_Triangle* pxTri=&pxElt->Data();
				RenderCarpetTriangle( xDataFrame, pxTri);
			}
		}
	}
	Asura_Direct3D_Carpet_Render::EndRender();

	Asura_Render::CurrentStates.RequestAlphaTest( false );
}

void Tools_Carpet_Mesh::RenderCarpetTriangle( const Asura_Carpet_Mesh_Data_Frame& xDataFrame, Asura_Carpet_Triangle* pxTri)
{
	Asura_Carpet_Style* pxStyle=Asura_Carpet_StyleManager::GetStyle(pxTri->iStyle);
		
	// Is the style valid?
	if (!pxStyle) return;	


	// Create offset table, for grass waving
	// This should only be done once per frame, not per triangle!
	s_fTimer = xDataFrame.m_fTime;
	
	// Reject triangle if it's too far away

	// Use camera pos/fwd to get plane eq, and use this to get distance to centre 
	// of triangle
	float fDist = (pxTri->xCentre * xDataFrame.m_xCameraForwards) + xDataFrame.m_fCameraPlaneDistance;

	// Too far away, or behind the camera?
	if ((fDist-pxTri->fRadius)>pxStyle->fLodFadeFar || (fDist < -pxTri->fRadius))
	{
		return;
	}

	// Outside the frustum?
	if ( false == Asura_Camera::IsSphereInFrustum( pxTri->xCentre, pxTri->fRadius))
	{
		return;
	}

	Asura_Direct3D_Carpet_Render::SetTextureID(pxStyle->iTextureID);
	

	// Determine how many billboards to render
	float fDensity=(pxTri->xVerts[0].fDensity+
		            pxTri->xVerts[1].fDensity+
		            pxTri->xVerts[2].fDensity)*0.3333f;

	fDensity*=pxStyle->fDensity;

	int iCount=(int)(pxTri->fArea*fDensity);

	// Reseed the random number generator with any value that's constant
	// for this triangle. I've just used the pointer to the triangle struct...
	srand(pxTri->uSeed);

	for(int i=0; i<iCount; i++)
	{
		int iIdx = pxStyle->aiSequence[i % ASURA_CARPET_BILLBOARD_SEQUENCE_LENGTH];
		Asura_Carpet_Billboard *pxBB=&pxStyle->axBillboards[iIdx];

		// Pick a random point within the triangle (barycentric coords)
		float fA=((rand()&255)/255.0f);
		float fB=((rand()&255)/255.0f);

		if (fA+fB>1.0f)
		{
			fA=1.0f-fA;
			fB=1.0f-fB;
		}

		float fC=1.0f-(fA+fB);

		Asura_Vector_3 xPos = (pxTri->xVerts[0].xPos*fA)+(pxTri->xVerts[1].xPos*fB)+(pxTri->xVerts[2].xPos*fC);		

		// Distance from camera
		float fDist = (xPos * xDataFrame.m_xCameraForwards) + xDataFrame.m_fCameraPlaneDistance;

		if (pxStyle->fLodFadeFar <= fDist)
		{	
			continue;
		}
		
		Asura_Vector_3 xOfs = xPos - xDataFrame.m_xCameraPosition;
		Asura_Vector_3 xPos2;

#if 0
		float fL=xOfs.Magnitude();
		if (fL<2.0f)
		{			
			xOfs.Normalise();
			xOfs = xOfs*2.0f;
			xPos2= xDataFrame.m_xCameraPosition + xOfs;

			float fLerp=1.0f*(2.0f-fL);
			if (fLerp>1.0f) fLerp=1.0f;
			
			xPos=(xPos2*fLerp) + (xPos*(1.0f-fLerp));
		}
#endif

		{				
			// Add some motion
			Asura_Vector_3 xWave=Asura_Vector_3(0,0,0);
			int iIdx = (i+pxTri->uSeed) % ASURA_CARPET_MESH_DATA_FRAME_SIZE_WAVEX;
			xWave.x = xDataFrame.m_afWaveX[ iIdx ] * pxStyle->fMovementScale;			
			xWave.z = xDataFrame.m_afWaveZ[ iIdx ] * pxStyle->fMovementScale;

			int aiRed[3], aiGreen[3], aiBlue[3];

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30)
			for(u_int u=0; u<3; u++)
			{
				aiRed[u]=Asura_Maths::FloatToInt(pxTri->xVerts[u].afColour[0]*255.0f);
				aiRed[u]=Asura_Maths::ClampToRange(aiRed[u],0,255);

				aiGreen[u]=Asura_Maths::FloatToInt(pxTri->xVerts[u].afColour[1]*255.0f);
				aiGreen[u]=Asura_Maths::ClampToRange(aiGreen[u],0,255);

				aiBlue[u]=Asura_Maths::FloatToInt(pxTri->xVerts[u].afColour[2]*255.0f);
				aiBlue[u]=Asura_Maths::ClampToRange(aiBlue[u],0,255);
			}
#else
			for(u_int u=0; u<3; u++)
			{
				aiRed[u]=(pxTri->xVerts[u].uColour>>16)&0xff;
				aiGreen[u]=(pxTri->xVerts[u].uColour>>8)&0xff;
				aiBlue[u]=(pxTri->xVerts[u].uColour>>0)&0xff;
			}
#endif

			// Get the colour... fixed point... 
			int iA=(int)(fA*256.0f);
			int iB=(int)(fB*256.0f);
			int iC=(int)(fC*256.0f);


			int iRed=((aiRed[0]*iA)>>8)+((aiRed[1]*iB)>>8)+((aiRed[2]*iC)>>8);
			int iGreen=((aiGreen[0]*iA)>>8)+((aiGreen[1]*iB)>>8)+((aiGreen[2]*iC)>>8);
			int iBlue=((aiBlue[0]*iA)>>8)+((aiBlue[1]*iB)>>8)+((aiBlue[2]*iC)>>8);

			uint32 uCol= ((iRed<<16)+(iGreen<<8)+iBlue);					



			// And the alpha - from the LOD fade
			float fLod=1.0f-(fDist-pxStyle->fLodFadeNear)/(pxStyle->fLodFadeFar-pxStyle->fLodFadeNear);

			if (1 < fLod)
			{
				uCol=uCol|0x80000000;
			}
			else
			{
				if (fLod < 0)
				{
					uCol=uCol|0xFF000000;
				}
				else
				{	
					int iAlpha=(int)((fLod) * 0x80);					
					uCol=uCol|(iAlpha<<24);
				}
			}

			// Calc the scale for this 
			float fScale = (pxTri->xVerts[0].fDensity*fA)+(pxTri->xVerts[1].fDensity*fB)+(pxTri->xVerts[2].fDensity*fC);
			
			// We only scale then down to fMinScale, not 0
			fScale = ((1.0f-fScale)*pxBB->fMinScale)+fScale;

			// Create the verts
			float afUp[4];
			float afRight[4];

			afRight[3]=afRight[0]=-0.5f*fScale*pxBB->fWidth;
			afRight[1]=afRight[2]= 0.5f*fScale*pxBB->fWidth;
			afUp[0]=afUp[1]=fScale*pxBB->fHeight;
			afUp[2]=afUp[3]=0;


			// DLH - 12 January 2006 17:00:46 - get a point to the next four verts to use. If the Cache has run out of space then the cache will be drawn.
			Asura_Direct3D_Carpet_Render_Vertex* axVerts = Asura_Direct3D_Carpet_Render::GetDrawQuad();
			
			// Position and colour
			for(int i=0;i<4;i++)
			{						
				const Asura_Vector_3& xBillboardUp = xDataFrame.m_xBillboardUp;
				const Asura_Vector_3& xBillboardRight = xDataFrame.m_xBillboardRight;
				
				axVerts[i].X = xPos[0] + (xBillboardUp[0] * afUp[i])+(xBillboardRight[0] * afRight[i]);
				axVerts[i].Y = xPos[1] + (xBillboardUp[1] * afUp[i])+(xBillboardRight[1] * afRight[i]);
				axVerts[i].Z = xPos[2] + (xBillboardUp[2] * afUp[i])+(xBillboardRight[2] * afRight[i]);		

				if (i<2)
				{
					axVerts[i].X += fScale*pxBB->fHeight*xWave.x;
					axVerts[i].Z += fScale*pxBB->fHeight*xWave.z;
				}

				axVerts[i].Diffuse=uCol; 
			}

			// UVs - Just copy them from the billboard definition
			axVerts[0].U=axVerts[3].U=pxBB->fU[0];
			axVerts[1].U=axVerts[2].U=pxBB->fU[1];
			axVerts[0].V=axVerts[1].V=pxBB->fV[0];
			axVerts[2].V=axVerts[3].V=pxBB->fV[1];

			// Draw it. As we're drawing single billboards, we can draw it as a fan
			//Asura_Direct3D::D3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN,2,(void*)axVerts,sizeof(D3DGenericVertex));
		}
	}
}




void Tools_Carpet_Mesh::AddPolyInAdjustSphereCallback(uint32 uPolygonIndex)
{
	Tools_Carpet_PolyInAdjustSphere xPoly;
	xPoly.uModuleIdx = g_uCurrentModule;
	xPoly.uPolyIdx = uPolygonIndex;
	g_xAdjustPolyList.Push_Back(xPoly);
}

void Tools_Carpet_Mesh::BeginAdjustVertex(Asura_Vector_3& xPos, float fRadius)
{
	Asura_Bounding_Box xBB;
	xBB.Reset();
	xBB.Set(xPos + Asura_Vector_3(fRadius,fRadius,fRadius));
	xBB.Set(xPos - Asura_Vector_3(fRadius,fRadius,fRadius));
	Tools_Environment_LOS::TestWithAABB(xBB, &g_uCurrentModule, AddPolyInAdjustSphereCallback);				
}


void Tools_Carpet_Mesh::EndAdjustVertex(void)
{
	g_xAdjustPolyList.Clear();
}


void Tools_Carpet_Mesh::AdjustVertex(Asura_Vector_3& xPos, int iStyle, float fDensity, bool bAbsolute, bool bRestrictToSelection, float fMaxSlopeAngle, bool bDeleteTris /* = false */, bool bUseTextureFlags /* = true */, bool bUsePolyFlags /* = true */ )
{
	// For each nearby module
	//   For each polygon
	//     If it includes this vertex
	//		{
	//			Does the poly exist in our carpet mesh - Search by poly ID
	//			- if not, add it (and store it's poly ID)
	//			- Update it (abs/rel updates)
	//			- If empty, remove it
	//		}	

	for (Asura_ListElt<Tools_Carpet_PolyInAdjustSphere>* pxElt=g_xAdjustPolyList.Front(); pxElt; pxElt=pxElt->Next())
	{
		int i = pxElt->Data().uModuleIdx;
		int j = pxElt->Data().uPolyIdx;

		Asura_Vector_3 xModulePosition = Asura_Module_System::GetPositionOfModule(i);
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);

		Asura_Mesh_Polygon* p=&pxMesh->PolygonDataPtr[j];

		// Is our vert present?
		bool bVertFound=false;
		int  iVertIndex=-1;

		for(int k=0;k<p->NumberOfVertices();k++)
		{
			const Asura_Vector_3 xVertPos = xModulePosition+pxMesh->VertexDataPtr[p->VertexID[k]].Position;
			if ( xPos.ApproximatelyEquals( xVertPos, 0.01f ) )
			{
				bVertFound=true;
				iVertIndex=k;
			}
		}

		// Reject polygon if the slope is too steep, or if it's not selected and 
		// we're in 'restrict to selection' mode
		if (bVertFound)
		{
			Asura_Vector_3 xNormal=pxMesh->NormalsDataPtr[j].FaceNormal;
		
			float fAngle = ( 90.0f - fMaxSlopeAngle ) * (3.142f/180.0f);

			if (fabs(xNormal.y) < fabs(sin(fAngle)))
			{
				bVertFound=false;
			}
		}
		
		if (bVertFound)
		{
			Asura_Vector_3* avTri[3];					

			// Arse. Got to support quads :(				
			for(int iPass=0;iPass<(p->NumberOfVertices()-2);iPass++)
			{
				int iTriVertIndex=-1;

				if (iPass==0)
				{ 
					avTri[0]=&pxMesh->VertexDataPtr[p->VertexID[0]].Position;
					avTri[1]=&pxMesh->VertexDataPtr[p->VertexID[1]].Position;
					avTri[2]=&pxMesh->VertexDataPtr[p->VertexID[2]].Position;
				
					
					if (iVertIndex<3) iTriVertIndex=iVertIndex;
				}
				else
				{
					// 2nd tri in a quad
					avTri[0]=&pxMesh->VertexDataPtr[p->VertexID[0]].Position;
					avTri[1]=&pxMesh->VertexDataPtr[p->VertexID[2]].Position;
					avTri[2]=&pxMesh->VertexDataPtr[p->VertexID[3]].Position;

					if (iVertIndex==0) iTriVertIndex=0;
					if (iVertIndex>1) iTriVertIndex=iVertIndex-1;
				}

				if (iTriVertIndex>=0)
				{												
					bool bSkipThisTri=false;
						
					Asura_Carpet_Triangle* pxTri = NULL;

					// Do back-face based rejection here


					// Do flag-based rejection
					if( bUseTextureFlags )
					{
						int iMtl=pxMesh->TextureDataPtr[j].m_iMaterialID;							
						int iTexture=Asura_Material_System::GetTextureID(iMtl);
						uint32 uFlags= Asura_Texture_Management::GetTextureFlags(iTexture);

						if (!(uFlags&AsuraTextureFlag_AcceptCarpet))
						{
							bSkipThisTri=true;
						}
					}

					if( bUsePolyFlags )
					{
						// PAZ - reject if "DontRender" (unless we are deleting)
						if( !bDeleteTris )
						{
							const int iPoloygonFlags = pxMesh->GetPolygonFlags( j );
							if( iPoloygonFlags & AsuraPolygonFlag_DontRender )
							{
								bSkipThisTri = true;
							}
						}
					}

					if (!bSkipThisTri)
					{							
						// Does it exist already?
						if (LocateOrAddTriangle(i,iStyle,(Asura_Vector_3**)&avTri[0],&pxTri,bDeleteTris))
						{
							if (bAbsolute)
							{
								// Absolute setting
								pxTri->xVerts[iTriVertIndex].fDensity=fDensity;
							}
							else
							{
								// Relative setting
								pxTri->xVerts[iTriVertIndex].fDensity+=fDensity;

								if (pxTri->xVerts[iTriVertIndex].fDensity<0)
									pxTri->xVerts[iTriVertIndex].fDensity=0;

								if (pxTri->xVerts[iTriVertIndex].fDensity>1)
									pxTri->xVerts[iTriVertIndex].fDensity=1;
							}

							// If there's nothing left, delete the triangle from the mesh
							if (pxTri->xVerts[0].fDensity<=0 && 
								pxTri->xVerts[1].fDensity<=0 &&
								pxTri->xVerts[2].fDensity<=0 ||
								bDeleteTris)
							{
								RemoveTriangle(i,pxTri);
							}								
						}
					}
				}
			}
		}
	}
}



void Tools_Carpet_Mesh::RemoveTriangle(int iModule, Asura_Carpet_Triangle* pxTri)
{
	Asura_ListElt<Asura_Carpet_Triangle>* pxEltToRemove=NULL;

	for(Asura_ListElt<Asura_Carpet_Triangle>* pxElt=s_pxModuleLists[iModule].Front(); pxElt; pxElt=pxElt->Next())
	{				
		if (&pxElt->Data()==pxTri)
		{
			pxEltToRemove=pxElt;
		}		
	}

	if (pxEltToRemove!=NULL)
	{	
		s_pxModuleLists[iModule].Remove(pxEltToRemove);
		s_iTriangleCount--;
	}

}



bool Tools_Carpet_Mesh::LocateOrAddTriangle(int iModule,int iStyle,Asura_Vector_3** ppxVerts,Asura_Carpet_Triangle** ppxTri, bool bDontAdd)
{
	*ppxTri=NULL;

	Asura_Vector_3 xModulePosition = Asura_Module_System::GetPositionOfModule(iModule);

	int iNumberOfModules=Asura_Module_System::GetNumberOfModules();

	if (iModule<0 || iModule>=iNumberOfModules)
	{	
		Asura_Assert (false, "Invalid module!");
		return false;
	}

	if (s_pxModuleLists==NULL)
	{
		Asura_Assert (false, "s_pxModuleLists not allocated!");
		return false;
	}

	// Iterate through our list, see if we can find it
	for(Asura_ListElt<Asura_Carpet_Triangle>* pxElt=s_pxModuleLists[iModule].Front(); pxElt; pxElt=pxElt->Next())
	{
		Asura_Carpet_Triangle* pxTri=&pxElt->Data();

		if (iStyle==pxTri->iStyle || iStyle==-1)
		{
			const Asura_Vector_3 xVert0Pos = *ppxVerts[0] + xModulePosition;
			const Asura_Vector_3 xVert1Pos = *ppxVerts[1] + xModulePosition;
			const Asura_Vector_3 xVert2Pos = *ppxVerts[2] + xModulePosition;
			if (pxTri->xVerts[0].xPos.ApproximatelyEquals( xVert0Pos, 0.01f ) &&
				pxTri->xVerts[1].xPos.ApproximatelyEquals( xVert1Pos, 0.01f ) &&
				pxTri->xVerts[2].xPos.ApproximatelyEquals( xVert2Pos, 0.01f ) )
			{
				*ppxTri=pxTri;
			}
		}
	}
	

	// Not found - add a new one
	if (*ppxTri==NULL)
	{
		if (bDontAdd)
		{
			return false;
		}

	    Asura_Carpet_Triangle xTri;

		for(int i=0;i<3;i++)
		{
			xTri.xVerts[i].xPos=*ppxVerts[i]+xModulePosition ;
			xTri.xVerts[i].fDensity=0.0f;
		}					

		// Store the style index and UID
		Asura_Carpet_Style* pxStyle=Asura_Carpet_StyleManager::GetStyle(iStyle);

		xTri.iStyle=iStyle;

		if (pxStyle)
			xTri.iStyleUID=pxStyle->iUID;
		else
			xTri.iStyleUID=-1;

		// Get some vertex colours for it
		CaptureLightingForOneTri(&xTri);

		// Calculate it's area
		xTri.fArea = Asura_Maths::CalculateTriArea(xTri.xVerts[0].xPos,xTri.xVerts[1].xPos,xTri.xVerts[2].xPos);

		// Calculate a centre and radius for distance/frustum-based culling
		xTri.xCentre = (xTri.xVerts[0].xPos+xTri.xVerts[1].xPos+xTri.xVerts[2].xPos)*(1.0f/3.0f);
		xTri.fRadius = 0;



		xTri.uSeed=GetTickCount();

		for(int i=0;i<3;i++)
		{
			Asura_Vector_3 xOfs=xTri.xVerts[i].xPos-xTri.xCentre;
			float fDist=xOfs.Magnitude();

			if (fDist>xTri.fRadius)
			{
				xTri.fRadius=fDist;
			}
		}

		s_pxModuleLists[iModule].Push_Back(xTri);
		*ppxTri=&s_pxModuleLists[iModule].Back()->Data();
		s_iTriangleCount++;
	}

	return true;
}




void Tools_Carpet_Mesh::CaptureLightingForOneTri(Asura_Carpet_Triangle* pxTri)
{
	for(int j=0;j<3;j++)
	{	
		int				iModule;
		int				iPoly;
		int				iVtx;
		Asura_Vector_3  xNormal;

		Asura_Carpet_Vertex& xVertex = pxTri->xVerts[j];

#if defined( ASURA_DIRECT3D_BGR_SUPPORT_SM30)
		xVertex.afColour[0] = 1.0f;
		xVertex.afColour[1] = 1.0f;
		xVertex.afColour[2] = 1.0f;
		xVertex.afColour[3] = 0.0f;
#else
		xVertex.uColour=0x00ffff;
#endif

		u_int uPaintedColour;

		if (FindScenePoly(pxTri,j,&iModule,&iPoly,&iVtx,&xNormal,&uPaintedColour))
		{	
			xVertex.m_uModule = iModule;
			xVertex.m_uPoly = iPoly;
			xVertex.m_uVtx = iVtx;
			xVertex.m_xNormal = xNormal;

#ifndef ASURA_TARGET_SM3			
			const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule( xVertex.m_uModule );
			Asura_Assert( uPaintedColour == pxMesh->m_pxPaintedColourDataPtr[iPoly].Colour[iVtx], "" );
			uPaintedColour = pxMesh->m_pxPaintedColourDataPtr[iPoly].Colour[iVtx];
			xVertex.uColour=Tools_EnvironmentalLightTransport::GetDiffuseForVertex( xVertex.xPos,xNormal,iPoly,iVtx,iModule, uPaintedColour);
#else
			Asura_SphericalHarmonics_Order2_RGB xDiffuseSH;
			Asura_RGBA xDiffuse;

			const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule( xVertex.m_uModule );
			int iVertID = pxMesh->PolygonDataPtr[iPoly].VertexID[iVtx];

			Tools_EnvironmentalLightTransport::GetDiffuseForVertex( xDiffuseSH, iVertID, iModule );

			Tools_EnvironmentalLightTransport::UpdateLightingFromVertexPainting( xDiffuseSH, pxMesh, iPoly, iVtx );

			Asura_RGBA xColour;
			xDiffuseSH.EvaluateInDirection( xNormal, xColour );

			xVertex.afColour[0] = xColour.m_fR;
			xVertex.afColour[1] = xColour.m_fG;
			xVertex.afColour[2] = xColour.m_fB;
			xVertex.afColour[3] = 1.0f;
#endif

		}				
	}
}



void Tools_Carpet_Mesh::CaptureLighting()
{
	for(int i=0;i<s_iNumLists;i++)
	{		
		for(Asura_ListElt<Asura_Carpet_Triangle>* pxElt=s_pxModuleLists[i].Front(); pxElt; pxElt=pxElt->Next())
		{
			Asura_Carpet_Triangle* pxTri=&pxElt->Data();

			CaptureLightingForOneTri(pxTri);			
		}
	}	
}

// From a carpet triangle and vtx index, locate it within the level mesh
// This is needed when updating the lighting, and when determining if a carpet triangle is still valid (on reload)

bool PolysMatch(const Asura_Carpet_Triangle* pxTri, int iVtx, Asura_Mesh* pxMesh, Asura_Vector_3& xModulePosition, Asura_Mesh_Polygon* pxPoly, int &iTargetVtx)
{
	bool bFound[3]={false,false,false};
	for(int j=0;j<pxPoly->NumberOfVertices();j++)
	{
		for(int k=0;k<3;k++)
		{
			const Asura_Vector_3 xVertPos = xModulePosition+pxMesh->VertexDataPtr[pxPoly->VertexID[j]].Position;
			if ( xVertPos.ApproximatelyEquals( pxTri->xVerts[k].xPos, 0.01f ) )
			{
				bFound[k]=true;
				if (k==iVtx)
				{
					iTargetVtx=j;
				}
			}						
		}
	}

	return (bFound[0] && bFound[1] && bFound[2]);
}




void Tools_Carpet_Mesh::BeginFindScenePoly()
{	
	// In these modes, the LOS system is already initialised, so don't mess with it....
	if (Tools_System::GetCurrentMode()!=TOOLS_MODE_PHOTONMAPPING &&
		Tools_System::GetCurrentMode()!=TOOLS_MODE_CARPET)
	{
		// make sure data is up-to-date
		Tools_Environment_LOS::Reset();		
	}
}

void Tools_Carpet_Mesh::EndFindScenePoly()
{
	
}

bool Tools_Carpet_Mesh::FindScenePoly( const Asura_Carpet_Triangle* pxTri, int iVtx, 
									   int* const piModule, int* const piPoly, int* const piVtx,
									   Asura_Vector_3* const pxNormal, u_int* 
									   const puPaintedColour )
{
//	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Carpet_Mesh::FindScenePoly");
	
	// For each module the vertex may be in
	//   For each polygon
	//     Check all our tri verts are present (might be a quad), and record 
	//     the index of the one we're looking for

	Asura_Vector_3 xPos=pxTri->xVerts[iVtx].xPos;



	{
		// First, use the Tools_LOS		
		Asura_Vector_3 xA =  pxTri->xVerts[0].xPos-pxTri->xVerts[1].xPos;
		Asura_Vector_3 xB =  pxTri->xVerts[2].xPos-pxTri->xVerts[1].xPos;
		Asura_Vector_3 xC = (pxTri->xVerts[0].xPos+pxTri->xVerts[1].xPos+pxTri->xVerts[2].xPos)/3.0f;
		Asura_Vector_3 xN =  xA^xB;
		xN.Normalise();
		
		Tools_Environment_LOS_Data xLOSData(xC - (xN * 0.05f), xN);
		xLOSData.SetReturnOnFirstHit(true);

		if (Tools_Environment_LOS::TestWithRay(xLOSData))		
		{
			const u_int uModule = xLOSData.GetModuleIndex();
			const u_int uPoly = xLOSData.GetPolyIndex();
			
			
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(uModule);
			Asura_Mesh_Polygon* pxPoly=&pxMesh->PolygonDataPtr[uPoly];				
			int  iTargetVtx=-1;

			Asura_Vector_3 xModulePos = Asura_Module_System::GetPositionOfModule(uModule);
			if  (PolysMatch(pxTri, iVtx, pxMesh, xModulePos, pxPoly, iTargetVtx))
			{
				Asura_Assert( NULL != pxMesh, "Mesh not setup");
				Asura_Assert( NULL != pxMesh->NormalsDataPtr, "Mesh Normals not setup");
				
				// Found it!
				*piVtx=iTargetVtx; 
				*piModule=uModule;
				*piPoly=uPoly;
				*pxNormal = pxMesh->NormalsDataPtr[uPoly].VertexNormal[iTargetVtx];
				*puPaintedColour = pxMesh->m_pxPaintedColourDataPtr[uPoly].Colour[iTargetVtx];
				return true;
			}
		}
	}


	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		Asura_Vector_3 xCentre=Asura_Module_System::GetCentreOfModule(i);
		float		   fRadius=Asura_Module_System::GetRadiusOfModule(i);
		Asura_Vector_3 xModulePosition = Asura_Module_System::GetPositionOfModule(i);
		fRadius+=0.5f;
		Asura_Vector_3 xOfs=xPos-xCentre;


		// Is the vertex in the module's bounding sphere
		if (xOfs.MagnitudeSquared()<= (fRadius*fRadius))
		{
			// Find all the polys containing this vertex
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
			
			// DLH - 16 May 2006 16:19:57 - There is no normals on this mesh so ignore it
			if( NULL == pxMesh->NormalsDataPtr)
			{
				continue;
			}

			int iNumPolys=pxMesh->NumberOfPolygons;

			for(int iN=0;iN<iNumPolys;iN++)
			{
				
				int  iTargetVtx=-1;
				Asura_Mesh_Polygon* pxPoly=&pxMesh->PolygonDataPtr[iN];				

				if  (PolysMatch(pxTri, iVtx, pxMesh, xModulePosition, pxPoly, iTargetVtx))
				{
					Asura_Assert( NULL != pxMesh, "Mesh not setup");
					Asura_Assert( NULL != pxMesh->NormalsDataPtr, "Mesh Normals not setup");
					Asura_Assertf( iN < pxMesh->NumberOfPolygons, ("Polygon Index (%d) is out of range (%d)", iN, pxMesh->NumberOfPolygons));
					// Found it!
					*piVtx=iTargetVtx; 
					*piModule=i;
					*piPoly=iN;
					*pxNormal = pxMesh->NormalsDataPtr[iN].VertexNormal[iTargetVtx];
					*puPaintedColour = pxMesh->m_pxPaintedColourDataPtr[iN].Colour[iTargetVtx];
					return true;
				}
			}
		}
	}

	return false;
}







int			 					   		Tools_Carpet_Mesh::s_iListIndex=0;
Asura_ListElt<Asura_Carpet_Triangle>* 	Tools_Carpet_Mesh::s_pxElt=0;



Asura_Carpet_Triangle* Tools_Carpet_Mesh::GetFirstTriangle(void)
{
	s_iListIndex=-1;
	s_pxElt=0;
	return GetNextTriangle();
}


Asura_Carpet_Triangle* Tools_Carpet_Mesh::GetNextTriangle(void)
{
	if (!s_pxElt)
	{
		while (!s_pxElt)
		{		
			s_iListIndex++;
			if (s_iListIndex==s_iNumLists)
			{
				return false;
			}	

			s_pxElt=s_pxModuleLists[s_iListIndex].Front();
		}

		Asura_Carpet_Triangle* pxTri=&s_pxElt->Data();

		if (s_pxElt->Next()==NULL)
		{
			// Need to move to the next list next time
			s_pxElt=NULL;
		}

		return pxTri;
	}
	else
	{
		s_pxElt=s_pxElt->Next();
		Asura_Carpet_Triangle* pxTri=&s_pxElt->Data();

		if (s_pxElt->Next()==NULL)
		{
			// Need to move to the next list next time
			s_pxElt=NULL;
		}

		return pxTri;
	}
}




bool Tools_Carpet_Mesh::AddTriangle(Asura_Carpet_Triangle* pxTri)
{
	// Copy it
	Asura_Carpet_Triangle xTri=*pxTri;

	int iModule,iPoly,iVtx;
	Asura_Vector_3 xNormal;

	// Create the module lists if they haven't been created yet
	if (s_iNumLists==0)
	{
		Init();
	}

	// Determine which module it's in	
	u_int uPaintedColour;
	if (FindScenePoly(&xTri,0,&iModule,&iPoly,&iVtx,&xNormal,&uPaintedColour))
	{
		s_pxModuleLists[iModule].Push_Back(xTri);
		s_iTriangleCount++;
		return true;
	}

	return false;
}



void Tools_Carpet_Mesh::CleanMesh()
{
	Asura_LinkList<Asura_ListElt<Asura_Carpet_Triangle>*> xDeletionList;

	int iModule,iPoly,iVtx;
	Asura_Vector_3 xNormal;
	u_int uPaintedColour;

	for(int i=0;i<s_iNumLists;i++)
	{		
		for(Asura_ListElt<Asura_Carpet_Triangle>* pxElt=s_pxModuleLists[i].Front(); pxElt; pxElt=pxElt->Next())
		{
			Asura_Carpet_Triangle* pxTri=&pxElt->Data();

			if (!FindScenePoly(pxTri,0,&iModule,&iPoly,&iVtx,&xNormal,&uPaintedColour))
			{
				xDeletionList.Push_Back(pxElt);				
			}
		}

		// Delete anything that needs deleting
		for(Asura_ListElt<Asura_ListElt<Asura_Carpet_Triangle>*> *pxElt=xDeletionList.Front(); pxElt; pxElt=pxElt->Next())
		{				
			s_pxModuleLists[i].Remove(pxElt->Data());
			s_iTriangleCount--;
		}

		xDeletionList.Clear();
	}	
};



void	Tools_Carpet_Mesh::Delete(Asura_Vector_3 xPos, float fRadius)
{
	Asura_LinkList<Asura_ListElt<Asura_Carpet_Triangle>*> xDeletionList;

//	int iModule,iPoly,iVtx;
	Asura_Vector_3 xNormal;

	for(int i=0;i<s_iNumLists;i++)
	{		
		for(Asura_ListElt<Asura_Carpet_Triangle>* pxElt=s_pxModuleLists[i].Front(); pxElt; pxElt=pxElt->Next())
		{
			Asura_Carpet_Triangle* pxTri=&pxElt->Data();
			bool bDel=false;

			for(int i=0;i<3;i++)
			{			
				Asura_Vector_3 xOfs = pxTri->xVerts[i].xPos - xPos;
				
				if (xOfs.MagnitudeSquared()<(fRadius*fRadius))
				{
					bDel=true;
				}
			}

			if (bDel)
			{
				xDeletionList.Push_Back(pxElt);			
			}
		}

		// Delete anything that needs deleting
		for(Asura_ListElt<Asura_ListElt<Asura_Carpet_Triangle>*> *pxElt=xDeletionList.Front(); pxElt; pxElt=pxElt->Next())
		{				
			s_pxModuleLists[i].Remove(pxElt->Data());
			s_iTriangleCount--;
		}

		xDeletionList.Clear();
	}
}

void Tools_Carpet_Mesh::ClearMesh( void)
{
	// PAZ - done as a list so it can easily be extended, if required
	Asura_LinkList< Asura_ListElt< Asura_Carpet_Triangle >* > xDeletionList;

	for( int i = 0; i < s_iNumLists; i++ )
	{		
		for( Asura_ListElt< Asura_Carpet_Triangle >* pxElt = s_pxModuleLists[ i ].Front(); pxElt; pxElt = pxElt->Next() )
		{
			xDeletionList.Push_Back( pxElt );			
		}

		// Delete anything that needs deleting
		for( Asura_ListElt< Asura_ListElt < Asura_Carpet_Triangle >* >* pxElt = xDeletionList.Front(); pxElt; pxElt = pxElt->Next() )
		{				
			s_pxModuleLists[ i ].Remove( pxElt->Data() );
			s_iTriangleCount--;
		}

		xDeletionList.Clear();
	}
}

void Tools_Carpet_Mesh::ClearMeshIfDontRender( void )
{
	Asura_LinkList< Asura_ListElt< Asura_Carpet_Triangle >* > xDeletionList;

	for( int i = 0; i < s_iNumLists; i++ )
	{		
		for( Asura_ListElt< Asura_Carpet_Triangle >* pxElt = s_pxModuleLists[ i ].Front(); pxElt; pxElt = pxElt->Next() )
		{
			Asura_Carpet_Triangle* pxTri = &pxElt->Data();

			for(int j = 0; j < 3; j++)
			{			
				int iModule;
				int iPoly;
				int iVtx;
				Asura_Vector_3 xNormal;
				u_int uPaintedColour;

				if( FindScenePoly( pxTri, j,
							&iModule, &iPoly, &iVtx, &xNormal, &uPaintedColour ) )
				{
					const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule( iModule );
					if( pxMesh != NULL )
					{
						const int iPoloygonFlags = pxMesh->GetPolygonFlags( iPoly );
						if( iPoloygonFlags & AsuraPolygonFlag_DontRender )
						{
							xDeletionList.Push_Back( pxElt );			
							break;
						}
					}
				}
			}
		}

		// Delete anything that needs deleting
		for( Asura_ListElt< Asura_ListElt < Asura_Carpet_Triangle >* >* pxElt = xDeletionList.Front(); pxElt; pxElt = pxElt->Next() )
		{				
			s_pxModuleLists[ i ].Remove( pxElt->Data() );
			s_iTriangleCount--;
		}
		
		xDeletionList.Clear();
	}
}


#endif //ASURA_USE_CARPET
