//*******************************************************************
//
// File: Tools_Carpet_Statistics.cpp         
//
// Created: 26 Jan 2006
//
// Description: 
//
//*******************************************************************

// system libraries
#include "Core/Asura.h"

#if defined( ASURA_USE_CARPET) && defined( ASURA_TOOLS)


// main header
#include "Carpet/Tools_Carpet_Statistics.h"
#include "Carpet/Asura_Direct3D_Carpet_Game_Render.h"

// other libraries
#include "Core/Asura_Timers.h"
#include "Camera/Asura_Camera.h"

#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Viewport.h"

// internal dependencies
#include "Misc/Tools_CyclicBuffer.h"

//*******************************************************************
// Constants
//*******************************************************************


//*******************************************************************
// Local Classes
//*******************************************************************

//*******************************************************************
// Static Data
//*******************************************************************

float Tools_Carpet_Statistics::s_fTime = 0;
float Tools_Carpet_Statistics::s_fTimeRTree = 0;
uint32 Tools_Carpet_Statistics::s_uDIPCount = 0;
uint32 Tools_Carpet_Statistics::s_uDIPIndexCount = 0;
uint32 Tools_Carpet_Statistics::s_uDIPVertexCount = 0;
uint32 Tools_Carpet_Statistics::s_uDIPPrimitiveCount = 0;
uint32 Tools_Carpet_Statistics::s_uCarpetTriangleCount = 0;
uint32 Tools_Carpet_Statistics::s_uCarpetInstanceCount = 0;
bool Tools_Carpet_Statistics::s_bShowCulling = false;

static uint32 s_uTreeNode = 0;

static Tools_CyclicBuffer< float> g_CarpetTimeAverage( 128);
static Tools_CyclicBuffer< float> g_CarpetTimeRTreeAverage( 128);

//*******************************************************************
// Function Definitions
//*******************************************************************

bool Tools_Carpet_Statistics::Reset()
{
	s_fTime = 0;
	s_fTimeRTree = 0;
	s_uDIPCount = 0;
	s_uDIPIndexCount = 0;
	s_uDIPVertexCount = 0;
	s_uDIPPrimitiveCount = 0;
	s_uCarpetTriangleCount = 0;
	s_uCarpetInstanceCount = 0;
	return( true);
}

bool Tools_Carpet_Statistics::Render()
{
	const bool bCGRHasDataSet = Asura_Direct3D_Carpet_Game_Render::HasDataSet();
	const float fWindowWidth = 400;
	const float fWindowX = Asura_Viewport::GetWidth() - fWindowWidth;
	float printx;
	float printy;
	int colour = 0xff50ff90;

	Asura_Render::CurrentStates.RequestAlphaTest( false);
	
//	{
		//determine print position
		float width = (float)Asura_Viewport::GetWidth();
		float height = (float)Asura_Viewport::GetHeight();
		Asura_Text_Output::ConvertPixelsToPrintCoords(width,height);
		printx = width-35;
		printy = 2;
//	}

	float fCursorInfoHeightInChars = 12;
	
	if( true == bCGRHasDataSet)
	{
		fCursorInfoHeightInChars += 2 + Asura_Direct3D_Carpet_Game_Render::GetMeshCount();
	}
	
	const float fLineHeight = 14;
	//draw a translucent polygon as a background
	Asura_Render::Overlay.SetRectXY( fWindowX, 1*fLineHeight, fWindowWidth, fCursorInfoHeightInChars * fLineHeight);
	Asura_Render::Overlay.SetRectUV( 0, 0, 1, 1);
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Colour=0x4f000000;
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	Asura_Text_Output::Print(printx,printy++,colour,0,"Carpet Statistics:");
	Asura_Text_Output::Print(printx,printy++,colour,0,"CGR: %s", (true == bCGRHasDataSet ? "Active" : "Deactive"));
	
	{
		const float fTime = 1000.0f * s_fTime;
		g_CarpetTimeAverage.ValueAdd( fTime);
		const float fTimeAverage = g_CarpetTimeAverage.GetAverage();
		Asura_Text_Output::Print(printx,printy++,colour,0,"Time %f ms (%f) ms", fTime, fTimeAverage);
	}
	
	{
		const float fTime = 1000.0f * s_fTimeRTree;
		g_CarpetTimeRTreeAverage.ValueAdd( fTime);
		const float fTimeAverage = g_CarpetTimeRTreeAverage.GetAverage();
		Asura_Text_Output::Print(printx,printy++,colour,0,"RTree %f ms (%f) ms", fTime, fTimeAverage);
	}
	
	Asura_Text_Output::Print(printx,printy++,colour,0,"DIP C %d", s_uDIPCount);
	Asura_Text_Output::Print(printx,printy++,colour,0,"DIP P %d", s_uDIPPrimitiveCount);
	Asura_Text_Output::Print(printx,printy++,colour,0,"DIP I %d", s_uDIPIndexCount);
	Asura_Text_Output::Print(printx,printy++,colour,0,"DIP V %d", s_uDIPVertexCount);
	Asura_Text_Output::Print(printx,printy++,colour,0,"C P %d", s_uCarpetTriangleCount);
	Asura_Text_Output::Print(printx,printy++,colour,0,"C I %d", s_uCarpetInstanceCount);
	
	printy++;
	
	if( true == bCGRHasDataSet && true == s_bShowCulling)
	{
#if 1
		const float fPrintYDebugStart = printy;
		{

			Tools_Carpet_Statistics_RTreeView xView;
			
			//xView.m_xBoundingBox.SetToPoint( Asura_Vector_3( -100, -100, -100));
			//xView.m_xBoundingBox.Set( Asura_Vector_3( 100, 100, 100));

			xView.m_fViewStartX = fWindowX;		
			xView.m_fViewStartY = (2 + fCursorInfoHeightInChars) * fLineHeight;		
			xView.m_fViewSizeX = fWindowWidth / 2;
			xView.m_fViewSizeY = fWindowWidth / 2;
			
			const uint32 uiCGRMeshCount = Asura_Direct3D_Carpet_Game_Render::GetMeshCount();
			for( uint32 uiCGRMeshIndex = 0; uiCGRMeshIndex < uiCGRMeshCount; uiCGRMeshIndex += 1)
			{
				const Asura_Direct3D_CGR_Mesh& xCGRMesh = Asura_Direct3D_Carpet_Game_Render::GetMesh( uiCGRMeshIndex);
				const Asura_Direct3D_CGR_RTreeNode* pxCGRRTree = xCGRMesh.m_pxRTree;
				
				if( NULL == pxCGRRTree)
				{
					continue;
				}
				
				const bool bIsVisible = Asura_Camera::IsAABBInFrustum( xCGRMesh.m_xBoundingBox);
				if( false == bIsVisible)
				{
					continue;
				}
				
				Asura_Render::Overlay.SetRectXY( xView.m_fViewStartX, xView.m_fViewStartY, xView.m_fViewSizeX, xView.m_fViewSizeY);
				Asura_Render::Overlay.SetRectUV( 0, 0, 1, 1);
				Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
				Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
				Asura_Render::Overlay.Colour=0x4f000000;
				Asura_Render::Overlay.Render();
				Asura_Render::FlushDisplay();
				
				xView.m_xBoundingBox = xCGRMesh.m_xBoundingBox;
				xView.m_xCamera.GetCurrentCamera( xCGRMesh.m_fLodFadeFar);
				
				s_uTreeNode = 0;
				RenderRTree( xView, pxCGRRTree);
				
				Asura_Render::FlushDisplay();
				
				xView.m_fViewStartY += xView.m_fViewSizeY + 5;
			}
		}
		
		printy = fPrintYDebugStart;
		
#endif
		{
			const uint32 uMeshCount = Asura_Direct3D_Carpet_Game_Render::GetMeshCount();
			
			for( uint32 uMeshIndex = 0; uMeshIndex < uMeshCount; uMeshIndex += 1)
			{
				const Asura_Direct3D_CGR_Mesh& xCGRMesh = Asura_Direct3D_Carpet_Game_Render::GetMesh( uMeshIndex);
				
				const bool bIsVisible = Asura_Camera::IsAABBInFrustum( xCGRMesh.m_xBoundingBox);
				if( false == bIsVisible)
				{
					continue;
				}
				
#if 0
				{
					const float fStartY = printy*fLineHeight;
					Asura_Render::Overlay.SetRectXY( fWindowX, fStartY, fWindowWidth, 1);
					Asura_Render::Overlay.SetRectUV( 0, 0, 1, 1);
					Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
					Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
					Asura_Render::Overlay.Colour=0xFFFF0000;
					Asura_Render::Overlay.Render();
					Asura_Render::FlushDisplay();
					
					const Asura_Direct3D_CGR_RangeManager& xRangeManager = xCGRMesh.m_xVBRangeManager;
					const float fRange = static_cast< float>( xRangeManager.m_uRangeMaxCount);
				
					for (Asura_ListElt< Asura_Direct3D_CGR_PolygonBatch*>* pxIteratorBatch = xRangeManager.m_xListBatches.Front(); pxIteratorBatch; pxIteratorBatch = pxIteratorBatch->Next())
					{
						Asura_Direct3D_CGR_PolygonBatch* pxBatchSearch = pxIteratorBatch->Data();		
						
						const float fDeltaStart = pxBatchSearch->m_uVertexBufferOffset / fRange;
						const float fDeltaStop = pxBatchSearch->m_uVertexBufferSize / fRange;
						const float fStartX = fWindowX + (fDeltaStart * fWindowWidth);
						const float fWidthX = (fDeltaStop * fWindowWidth);
						
						const uint32 uColour = (pxBatchSearch->m_uTimeStampLastUsed == Asura_Direct3D_Carpet_Game_Render::s_uTimeStampLast) ? 0x00FF00 : 0x0000FF;
						
						Asura_Render::Overlay.SetRectXY( fStartX, fStartY, fWidthX, 1);
						Asura_Render::Overlay.SetRectUV( 0, 0, 1, 1);
						Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
						Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
						Asura_Render::Overlay.Colour = 0xFF000000 | uColour;
						Asura_Render::Overlay.Render();
						Asura_Render::FlushDisplay();
					}
					
				}
#endif
				
				Asura_Text_Output::Print(printx,printy++,colour,0,"%d: B %d S 0x%04X VB 0x%04X", uMeshIndex, xCGRMesh.m_xVBRangeManager.m_uBlockUsedCount, xCGRMesh.m_xVBRangeManager.m_uRangeUsedCount, xCGRMesh.m_uBatchVertexBufferSize);
				
				
			}
		}
		
#if 0
		{
			const uint32 uMeshCount = Asura_Direct3D_Carpet_Game_Render::GetMeshCount();
			
			printx = 1;
			
			for( uint32 uMeshIndex = 0; uMeshIndex < uMeshCount; uMeshIndex += 1)
			{
				const Asura_Direct3D_CGR_Mesh& xCGRMesh = Asura_Direct3D_Carpet_Game_Render::GetMesh( uMeshIndex);
				
				const bool bIsVisible = Asura_Camera::IsAABBInFrustum( xCGRMesh.m_xBoundingBox);
				if( false == bIsVisible)
				{
					continue;
				}
				
				printy = 1;
		
				Asura_Text_Output::Print(10,printy++,colour,0,"%d: B %d S 0x%04X VB 0x%04X", uMeshIndex, xCGRMesh.m_xVBRangeManager.m_uBlockUsedCount, xCGRMesh.m_xVBRangeManager.m_uRangeUsedCount, xCGRMesh.m_uBatchVertexBufferSize);
				const uint32 uBatchCount = xCGRMesh.m_uPolygonBatchCount;
				for( uint32 uBatchIndex = 0; uBatchIndex < uBatchCount; uBatchIndex += 1)
				{
					const Asura_Direct3D_CGR_PolygonBatch& xCGRBatch = xCGRMesh.m_pxPolygonBatches[ uBatchIndex];
					
					if( 0 == xCGRBatch.m_uVertexBufferSize)
					{
						continue;
					}
				
					Asura_Text_Output::Print(printx,printy++,colour,0," %d:%03d 0x%08X %2d 0x%04X 0x%04X", uMeshIndex, uBatchIndex, xCGRBatch.m_uTimeStampLastUsed, xCGRBatch.m_uDataSize, xCGRBatch.m_uVertexBufferOffset, xCGRBatch.m_uVertexBufferSize);
				}
				
				printx += 24;
			}
		}
#endif
		
	}
    
	return( true);
}

#if 1
bool Tools_Carpet_Statistics::RenderRTree( const Tools_Carpet_Statistics_RTreeView& xView, const Asura_Direct3D_CGR_RTreeNode* pxRTreeNode, const uint32 uiTreeDepth)
{
//	const float fModTimeLength = 10.0f;
//	const float fRealTime = Asura_Timers::GetLocalUnscaledTime();
//	const float fModTime = fmod( fRealTime, fModTimeLength);
//	const float fTimeDelta = fModTime / fModTimeLength;
//	const uint32 uiTimeDepth = uint32( 100 * fTimeDelta);
	
	const Asura_Vector_3 xViewBBMax = xView.m_xBoundingBox.GetMax();
	const Asura_Vector_3 xViewBBMin = xView.m_xBoundingBox.GetMin();
	const Asura_Vector_3 xViewBBDelta = xViewBBMax - xViewBBMin;
	
//	const uint32 uiColourBase = uiTimeDepth == uiTreeDepth ? (0x80FF0000):(0x10000000);
	
	for( ; NULL != pxRTreeNode; pxRTreeNode = pxRTreeNode->m_pxSibling)
	{
		s_uTreeNode += 1;
		
		const Asura_Direct3D_Carpet_Game_Render::CullState eIsAABBInFrustum = Asura_Direct3D_Carpet_Game_Render::GetCullStateOfAABBInFrustum( xView.m_xCamera, pxRTreeNode->m_xBoundingBox);
		if( uint32(-1) != pxRTreeNode->m_uData)
		//if( Asura_Direct3D_Carpet_Game_Render::CULL_OUTSIDE != eIsAABBInFrustum && uint32(-1) != pxRTreeNode->m_uData)
		//const bool bIsAABBInFrustum = Asura_Camera::IsAABBInFrustum( pxRTreeNode->m_xBoundingBox);
		//if( true == bIsAABBInFrustum && uint32(-1) != pxRTreeNode->m_uData)
		{
			//const uint32 uiColourVisible = ( (uint32(-1) != pxRTreeNode->m_uData) && (true == bIsAABBInFrustum) ) ? 0x0000FF00 : 0;
			uint32 uiColourVisible = 0x00000000;
			uiColourVisible |= ( Asura_Direct3D_Carpet_Game_Render::CULL_INSIDE == eIsAABBInFrustum ) ? 0x000000FF : 0x0000000;
			uiColourVisible |= ( Asura_Direct3D_Carpet_Game_Render::CULL_TOUCHING == eIsAABBInFrustum ) ? 0x0000FF00 : 0x0000000;
			uiColourVisible |= ( Asura_Direct3D_Carpet_Game_Render::CULL_OUTSIDE == eIsAABBInFrustum ) ? 0x00FF0000 : 0x00000000;
			//const uint32 uiColourBase = uiTimeDepth == s_uTreeNode ? (0x80FF0000):(0x80000000);
			const uint32 uiColourBase = (0x80000000);
			
	//		const Asura_Vector_3 xExtents = pxRTreeNode->m_xBoundingBox.CalculateExtents();
	//		const Asura_Vector_3 xCenter = pxRTreeNode->m_xBoundingBox.CalculateCentre();
	//		const Asura_Vector_3 xMin = xCenter - xExtents;
			const Asura_Vector_3 xMin = pxRTreeNode->m_xBoundingBox.GetMin();
			const Asura_Vector_3 xMax = pxRTreeNode->m_xBoundingBox.GetMax();
			const Asura_Vector_3 xExtents = xMax - xMin;
			//const Asura_Vector_3 xMax = xCenter + xExtents;
			const float fStartX = (xMin[0] - xViewBBMin.GetX()) / (xViewBBDelta.GetX());
			const float fStartY = (xMin[2] - xViewBBMin.GetZ()) / (xViewBBDelta.GetZ());
			const float fScaleX = (xExtents[0]) / xViewBBDelta.GetX();
			const float fScaleY = (xExtents[2]) / xViewBBDelta.GetZ();
			const float fWindowStartX = xView.m_fViewStartX + (fStartX * xView.m_fViewSizeX);
			const float fWindowStartY = xView.m_fViewStartY + (fStartY * xView.m_fViewSizeY);
			const float fWindowSizeX = (fScaleX * xView.m_fViewSizeX);
			const float fWindowSizeY = (fScaleY * xView.m_fViewSizeY);
			
			Asura_Render::Overlay.SetRectXY( fWindowStartX, fWindowStartY, fWindowSizeX, fWindowSizeY);
			Asura_Render::Overlay.SetRectUV( 0, 0, 1, 1);
			Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_NORMAL;
			Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
			Asura_Render::Overlay.Colour = uiColourBase | uiColourVisible | (rand() % 0x100);
			Asura_Render::Overlay.Render();
		}

		const Asura_Direct3D_CGR_RTreeNode* pxRTreeChildNode = pxRTreeNode->m_pxChild;
		if( NULL != pxRTreeChildNode)
		{
			RenderRTree( xView, pxRTreeChildNode, 1 + uiTreeDepth);
		}
	}
	return( true);
}
#endif

//*******************************************************************
//
// (c) 2006 Rebellion (Derby)
//
// Warning: This computer program is protected by copyright law and 
// international treaties. Unauthorized reproduction or distribution
// of this program, or any portion of it, may result in severe civil and
// criminal penalties, and will be prosecuted to the maximum extent 
// possible under the law.
//
//*******************************************************************

#endif //defined( ASURA_USE_CARPET) && defined( ASURA_TOOLS)
