//
// Asura_Chunk_Skybox.cpp
//
// Created by Kevin Lea - 02/10/2001 16:57:35
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "ChunkSystem/Tools_Chunk_Skybox.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_Chunk_Macros.h"
#include "ChunkSystem/Asura_Chunk_WeatherSystem.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_Chunk_Skybox.h"
#include "FileAccess/Asura_FileAccess.h"
#include "SpecialFX/Asura_Skybox.h"

#include "Lighting/Tools_LightSets.h"

bool Tools_Chunk_SkyBox::Process(Asura_Handle iFileHandle)
{
	Asura_Assert(Version<=ASURA_CHUNK_SKYBOX_VERSION,"Unrecognised pxChunk version");
	
	Asura_Skybox::SetColours(m_fRed, m_fGreen, m_fBlue);

	// Kevin Lea - 02/10/2001 18:04:49 - check for old version
	// this code can be removed when we are sure no Asura files
	// using this version number exist anymore.
	if (Version == 1) 
	{
		// Kevin Lea - 02/10/2001 18:05:26 - set the new data fields to 'safe' values
		Asura_Skybox::SetOrientationAroundYAxis( 0.0f );
		for(int i=0; i<Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES; i++)
		{
			Asura_Skybox::SetTexture( "", i );
		}
		return true;
	}

	float fAngle;
 	Asura_FileAccess::ReadData(iFileHandle,&fAngle,sizeof(float));

	Asura_Skybox::SetOrientationAroundYAxis(fAngle);

	int iValidValues;
	switch (Version)
	{
	case 2:
		iValidValues = 6;
		break;
	case 3:
	case 4:
		iValidValues = 7;
		break;
	case 5:
	case 6:
	case 7:
	case 8:
	case 9:
	case 10:
	case 11:
	case 12:
	case 13:
		iValidValues = 8;
		break;
	default:
		Asura_Assert(false, "Unexpected skybox version");
		iValidValues = 0;
		break;
	}
	
	for (int i=0; i<iValidValues; i++)
	{
		// (AW - 24.09.2004) - I have reordered the skybox faces for chunk version 6 to reduce texture
		// uploads when using the upside-down texture options.
		if (Version < 6 &&
			i == Asura_Skybox::ASURA_SKYBOX_BK)
		{
			Asura_Skybox::SetTexture(Asura_FileAccess::ReadString(iFileHandle), Asura_Skybox::ASURA_SKYBOX_LF);
		}
		else if (Version < 6 &&
				i == Asura_Skybox::ASURA_SKYBOX_LF)
		{
			Asura_Skybox::SetTexture(Asura_FileAccess::ReadString(iFileHandle), Asura_Skybox::ASURA_SKYBOX_BK);
		}
		else
		{
			Asura_Skybox::SetTexture(Asura_FileAccess::ReadString(iFileHandle),i);
		}
	}
	for (int i=iValidValues; i<Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES; i++)
	{
		Asura_Skybox::SetTexture( "", i);
	}

	if (Version >= 6)
	{
		u_int uFlags;
		Asura_FileAccess::ReadData(iFileHandle, &uFlags, sizeof(u_int));
		Asura_Skybox::SetDrawClouds((uFlags & ASURA_CHUNK_SKYBOX_DRAW_CLOUD_FLAG) != 0);
		Asura_Skybox::SetBackTextureIsFrontUpsideDown((uFlags & ASURA_CHUNK_SKYBOX_BACK_IS_FRONT_UPSIDE_DOWN) != 0);
		Asura_Skybox::SetRightTextureIsLeftUpsideDown((uFlags & ASURA_CHUNK_SKYBOX_RIGHT_IS_LEFT_UPSIDE_DOWN) != 0);
		Asura_Skybox::SetMirrorBelowHorizon((uFlags & ASURA_CHUNK_SKYBOX_MIRROR_BELOW_HORIZON) != 0);
	}
	else if(Version >= 4)
	{
		int iTemp;
		Asura_FileAccess::ReadData(iFileHandle,&iTemp,sizeof(int));
		Asura_Skybox::SetDrawClouds(iTemp ? true : false);
	}

	#ifndef ASURA_TOOLS
	//blank out cloud filenames , if they aren't used. RAR 01/04/2004
	if(!Asura_Skybox::GetDrawClouds())
	{
		Asura_Skybox::SetTexture("", Asura_Skybox::ASURA_SKYBOX_FRACTALSKY);
		Asura_Skybox::SetTexture("", Asura_Skybox::ASURA_SKYBOX_FRACTALSKY_2);
	}
	#endif

	// Skybox object list (obsolete)
	if(Version >= 7 && Version<12)
	{
		u_int uNumObjects = 0;
		Asura_FileAccess::ReadData(iFileHandle, &uNumObjects, sizeof(u_int));
		while(uNumObjects--)
		{
			Asura_FileAccess::SkipData(iFileHandle, sizeof(Asura_Vector_3));
			Asura_FileAccess::SkipData(iFileHandle, sizeof(Asura_Matrix_3x3));

			Asura_FileAccess::SkipData(iFileHandle, sizeof(Asura_Hash_ID));

			Asura_Hash_ID uAnimID;
			Asura_FileAccess::ReadData(iFileHandle, &uAnimID, sizeof(Asura_Hash_ID));

			if(uAnimID != ASURA_HASH_ID_UNSET)
			{
				Asura_FileAccess::SkipData(iFileHandle, sizeof(int));
			}

			if(Version >= 9)
			{
				Asura_FileAccess::SkipData(iFileHandle, sizeof(u_int));
			}

			if(Version >= 11)
			{
				Asura_FileAccess::SkipData(iFileHandle, sizeof(u_int));
			}
		}
	
	}

	//skybox object colours

	if(Version >= 8)
	{
		float fObjectR,fObjectG,fObjectB;
		Asura_FileAccess::ReadData(iFileHandle,&fObjectR,sizeof(float));
		Asura_FileAccess::ReadData(iFileHandle,&fObjectG,sizeof(float));
		Asura_FileAccess::ReadData(iFileHandle,&fObjectB,sizeof(float));
		Asura_Skybox::SetObjectColours(fObjectR,fObjectG,fObjectB);
	}
	else
	{
		Asura_Skybox::SetObjectColours(255.0f,255.0f,255.0f);
	}

	//skybox name
	if(Version>=10)
	{
		#ifdef ASURA_TOOLS
		const char* szName = Asura_FileAccess::ReadString(iFileHandle);
		Asura_Skybox::SetSkyboxName(szName);
		#else
		//just ignore the name outside of tools
		Asura_FileAccess::ReadString(iFileHandle);
		#endif
	}

#if defined( ASURA_TARGET_SM3)
	if( Version >= 13 )
	{
		float af[ 6 ];
		bool ab[ 4 ];

		Asura_FileAccess::ReadData( iFileHandle, af, sizeof( float ) * 3 );
		Asura_Skybox::SetSkyboxColour( af );
		Asura_FileAccess::ReadData( iFileHandle, af, sizeof( float ) * 6 );
		Asura_FileAccess::ReadData( iFileHandle, ab, sizeof( bool ) * 4 );
		Asura_Skybox::SetHaze( &af[ 0 ], &af[ 3 ], ab[ 0 ], ab[ 1 ], ab[ 2 ], ab[ 3 ] );
		Asura_FileAccess::ReadData( iFileHandle, af, sizeof( float ) * 6 );
		Asura_FileAccess::ReadData( iFileHandle, ab, sizeof( bool ) * 2 );
		Asura_Skybox::SetSun( &af[ 0 ], &af[ 3 ], ab[ 0 ], ab[ 1 ] );
		Asura_FileAccess::ReadData( iFileHandle, af, sizeof( float ) * 2 );
		Asura_Skybox::SetHazeStrength( af[ 0 ], af[ 1 ] );
		Asura_FileAccess::ReadData( iFileHandle, af, sizeof( float ) * 2 );
		Asura_Skybox::SetSunSize( af[ 0 ], af[ 1 ] );
		Asura_FileAccess::ReadData( iFileHandle, &af, sizeof( float ) * 4 );
		Asura_Skybox::SetSunDir( &af[ 0 ], &af[ 2 ] );
		Asura_FileAccess::ReadData( iFileHandle, ab, sizeof( bool ) );
		Asura_Skybox::SetUseNewSkyboxShader( ab[ 0 ] );
	}
	else
	{
		Asura_Skybox::SetUseNewSkyboxShader( false );
	}
#else
	if( Version >= 13 )
	{
		float af[ 6 ];
		bool ab[ 4 ];

		Asura_FileAccess::ReadData( iFileHandle, af, sizeof( float ) * 3 );
		Asura_FileAccess::ReadData( iFileHandle, af, sizeof( float ) * 6 );
		Asura_FileAccess::ReadData( iFileHandle, ab, sizeof( bool ) * 4 );
		Asura_FileAccess::ReadData( iFileHandle, af, sizeof( float ) * 6 );
		Asura_FileAccess::ReadData( iFileHandle, ab, sizeof( bool ) * 2 );
		Asura_FileAccess::ReadData( iFileHandle, af, sizeof( float ) * 2 );
		Asura_FileAccess::ReadData( iFileHandle, af, sizeof( float ) * 2 );
		Asura_FileAccess::ReadData( iFileHandle, &af, sizeof( float ) * 4 );
		Asura_FileAccess::ReadData( iFileHandle, ab, sizeof( bool ) );
	}
#endif
	return true;
}

void Tools_Chunk_SkyBox::Create(Asura_Chunk_Stream& xStream)
{
	float	fRotationY;
	float	fRed;
	float	fGreen;
	float	fBlue;

	if( Tools_Level_Export::IsExport() )
	{
#if defined( ASURA_TARGET_SM3)
		xStream.WriteChunk_Start( ASURA_CHUNK_SKYBOX_SHADER, ASURA_CHUNK_SKYBOX_VERSION, 0 );
#else
		xStream.WriteChunk_Start( ASURA_CHUNK_SKYBOX, ASURA_CHUNK_SKYBOX_VERSION, 0 );
#endif
	}
	else
	{
		xStream.WriteChunk_Start( TOOLS_CHUNK_SKYBOX, ASURA_CHUNK_SKYBOX_VERSION, 0 );
	}
	Asura_Skybox::GetColours(fRed,fGreen,fBlue);

	xStream << fRed;
	xStream << fGreen;
	xStream << fBlue;

	Asura_Skybox::GetOrientationAroundYAxis(fRotationY);

	xStream << fRotationY;

	for(int i = 0; i < Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES; i++)
	{
		Asura_String xName = Asura_Skybox::GetTexture(i);
		if (xName.IsNull())
		{
			xName.MakeBlank();
		}
		xStream << xName;
	}

	u_int uFlags = 0;
	if (Asura_Skybox::GetDrawClouds())
	{
		uFlags |= ASURA_CHUNK_SKYBOX_DRAW_CLOUD_FLAG;
	}
	if (Asura_Skybox::GetBackTextureIsFrontUpsideDown())
	{
		uFlags |= ASURA_CHUNK_SKYBOX_BACK_IS_FRONT_UPSIDE_DOWN;
	}
	if (Asura_Skybox::GetRightTextureIsLeftUpsideDown())
	{
		uFlags |= ASURA_CHUNK_SKYBOX_RIGHT_IS_LEFT_UPSIDE_DOWN;
	}
	if(Asura_Skybox::GetMirrorBelowHorizon())
	{
		uFlags |= ASURA_CHUNK_SKYBOX_MIRROR_BELOW_HORIZON;
	}

	xStream << uFlags;

	//skybox object colours
	float fObjectR,fObjectG,fObjectB;
	Asura_Skybox::GetObjectColours(fObjectR,fObjectG,fObjectB);
	xStream << fObjectR << fObjectG << fObjectB;

	//skybox name
	xStream << Asura_String(Asura_Skybox::GetSkyboxName());

#if defined( ASURA_TARGET_SM3)
	{ //Version 13
		float af[ 6 ];
		bool ab[ 4 ];

		Asura_Skybox::GetSkyboxColour( af, Tools_Level_Export::IsExport() );
		xStream << af[ 0 ] << af[ 1 ] << af[ 2 ];

		Asura_Skybox::GetHaze( &af[ 0 ], &af[ 3 ], ab[ 0 ], ab[ 1 ], ab[ 2 ], ab[ 3 ], Tools_Level_Export::IsExport() );
		xStream << af[ 0 ] << af[ 1 ] << af[ 2 ] << af[ 3 ] << af[ 4 ] << af[ 5 ];
		xStream << ab[ 0 ] << ab[ 1 ] << ab[ 2 ] << ab[ 3 ];

		Asura_Skybox::GetSun( &af[ 0 ], &af[ 3 ], ab[ 0 ], ab[ 1 ], Tools_Level_Export::IsExport() );
		xStream << af[ 0 ] << af[ 1 ] << af[ 2 ] << af[ 3 ] << af[ 4 ] << af[ 5 ];
		xStream << ab[ 0 ] << ab[ 1 ];

		Asura_Skybox::GetHazeStrength( &af[ 0 ], &af[ 1 ], Tools_Level_Export::IsExport() );
		xStream << af[ 0 ] << af[ 1 ];

		Asura_Skybox::GetSunSize( &af[ 0 ], &af[ 1 ], Tools_Level_Export::IsExport() );
		xStream << af[ 0 ] << af[ 1 ];

		if( Tools_Level_Export::IsExport() )
		{
			Asura_Skybox::GetSunDir( &af[ 0 ], &af[ 3 ], true );
			xStream << af[ 0 ] << af[ 1 ] << af[ 2 ] << af[ 3 ] << af[ 4 ] << af[ 5 ];
		}
		else
		{
			Asura_Skybox::GetSunDir( &af[ 0 ], &af[ 2 ] );
			xStream << af[ 0 ] << af[ 1 ] << af[ 2 ] << af[ 3 ];
		}

		ab[ 0 ] = Asura_Skybox::GetUseNewSkyboxShader();
		xStream << ab[ 0 ];
	}
#else
	if( !Tools_Level_Export::IsExport() )
	{
		float af[ 6 ] = { 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f };
		bool ab[ 4 ] = { true, true, true, true };

		//Asura_Skybox::GetSkyboxColour( af, Tools_Level_Export::IsExport() );
		xStream << af[ 0 ] << af[ 1 ] << af[ 2 ];

		//Asura_Skybox::GetHaze( &af[ 0 ], &af[ 3 ], ab[ 0 ], ab[ 1 ], ab[ 2 ], ab[ 3 ], Tools_Level_Export::IsExport() );
		xStream << af[ 0 ] << af[ 1 ] << af[ 2 ] << af[ 3 ] << af[ 4 ] << af[ 5 ];
		xStream << ab[ 0 ] << ab[ 1 ] << ab[ 2 ] << ab[ 3 ];

		//Asura_Skybox::GetSun( &af[ 0 ], &af[ 3 ], ab[ 0 ], ab[ 1 ], Tools_Level_Export::IsExport() );
		xStream << af[ 0 ] << af[ 1 ] << af[ 2 ] << af[ 3 ] << af[ 4 ] << af[ 5 ];
		xStream << ab[ 0 ] << ab[ 1 ];

		//Asura_Skybox::GetHazeStrength( &af[ 0 ], &af[ 1 ], Tools_Level_Export::IsExport() );
		xStream << af[ 0 ] << af[ 1 ];

		//Asura_Skybox::GetSunSize( &af[ 0 ], &af[ 1 ], Tools_Level_Export::IsExport() );
		xStream << af[ 0 ] << af[ 1 ];

		//Asura_Skybox::GetSunDir( &af[ 0 ], &af[ 2 ], true );
		xStream << af[ 0 ] << af[ 1 ] << af[ 2 ] << af[ 3 ];

		ab[ 0 ] = false;//Asura_Skybox::GetUseNewSkyboxShader();
		xStream << ab[ 0 ];
	}
#endif
	// Finish
	xStream.WriteChunk_End();
}
