//
// Tools_Cutscene_Attachment.cpp
//
// Created by Richard Rice - 21/10/02
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Cutscene/Tools_Cutscene_Attachment.h"
#include "Debugging/Asura_Assert.h"


#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_Attachment.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"

#include "Tools_Overlays.h"
#include "Cutscene/Tools_Mode_Cutscene.h"
#include "Cutscene/Tools_Cutscene_Object.h"
#include "Cutscene/Tools_CutsceneActor_Object.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"
#include "Particles/Tools_PFX_Dialog.h"
#include "Shapes/Tools_Shapes.h"
#include "Sound/Tools_PickSoundEvent_Dialog.h"
#include "SpecialFX/Pfx/Asura_PFX_System.h"
#include "Texturing/Tools_PickTexture.h"


///////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_Cutscene_Attachment **
**---------------------------*/

Asura_Vector_2 Tools_Cutscene_Attachment::s_xOverlayPosition(10,10);
Tools_Cutscene_Attachment* Tools_Cutscene_Attachment::s_pxThis = 0;


Tools_Cutscene_Attachment* Tools_Cutscene_Attachment::AllocateToolsAttachment(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor)
{
    // First check if project code wants to handle this type of attachment.
    {
        Tools_Cutscene_Attachment* pxToolsAttachment = Project_AllocateToolsAttachment( pxAsuraAttachment, pxToolsActor );
        if( pxToolsAttachment ) { return pxToolsAttachment; }
    }

	switch(pxAsuraAttachment->GetType())
	{
		case AsuraCutsceneAttachment_Hierarchy :
			return new Tools_Cutscene_Attachment_Hierarchy(pxAsuraAttachment,pxToolsActor);
			break;

		case AsuraCutsceneAttachment_Shape :
			return new Tools_Cutscene_Attachment_Shape(pxAsuraAttachment,pxToolsActor);
			break;

		case AsuraCutsceneAttachment_Actor :
			return new Tools_Cutscene_Attachment_Actor(pxAsuraAttachment,pxToolsActor);
			break;

		case AsuraCutsceneAttachment_Entity :
			return new Tools_Cutscene_Attachment_Entity(pxAsuraAttachment,pxToolsActor);
			break;

		case AsuraCutsceneAttachment_Corona :
			return new Tools_Cutscene_Attachment_Corona(pxAsuraAttachment,pxToolsActor);
			break;

		case AsuraCutsceneAttachment_PFX :
			return new Tools_Cutscene_Attachment_PFX(pxAsuraAttachment,pxToolsActor);
			break;

		case AsuraCutsceneAttachment_SoundEvent :
			return new Tools_Cutscene_Attachment_SoundEvent(pxAsuraAttachment,pxToolsActor);
			break;

		default :
			Asura_Assert(0,"Unknown attachment type");
	}

	return 0;
}

Tools_Cutscene_Attachment::Tools_Cutscene_Attachment(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor)
{
	Asura_Assert(pxAsuraAttachment,"");
	Asura_Assert(pxToolsActor,"");

	m_pxAsuraAttachment = pxAsuraAttachment;
	m_pxToolsActor = pxToolsActor;
	m_bChanged = false;


	if(!m_pxAsuraAttachment->m_uID)
	{
		m_pxAsuraAttachment->m_uID = m_pxToolsActor->GetNextAttachmentID();		
	}

	m_pxToolsActor->m_xAttachmentList.add_entry(this);

	//fix for duplicate ID bug... check to see if this ID is used by a different attachment , if so , reset
	for(LIF<Tools_Cutscene_Attachment*> xAttachmentLIF(&m_pxToolsActor->m_xAttachmentList);!xAttachmentLIF.done();xAttachmentLIF.next())
	{
		if(xAttachmentLIF()->GetID() == GetID())
		{
			if(this != xAttachmentLIF())
			{
				m_pxAsuraAttachment->m_uID = m_pxToolsActor->GetNextAttachmentID();		
			}
			break;
		}
	}


	UpdateToolsName();
}

Tools_Cutscene_Attachment::~Tools_Cutscene_Attachment()
{
	if(s_pxThis == this)
	{
		OnDeselection();
	}
	
	m_pxToolsActor->m_xAttachmentList.delete_entry(this);
	m_pxToolsActor->UpdateAttachmentSelection();

}

void Tools_Cutscene_Attachment::Destroy()
{
	Tools_Mode_Cutscene::RefreshPlayback();
	Tools_SaveAsura::SetCutscenesModified();

	m_pxAsuraAttachment->PreDestructionCleanup();

	//remove from the asura actor , and delete
	m_pxToolsActor->RemoveAttachmentFromActor(m_pxAsuraAttachment);

	delete m_pxAsuraAttachment;
	delete this;
}

void Tools_Cutscene_Attachment::OnSelection()
{
	s_pxThis = this;
	CreateOverlay();
}

void Tools_Cutscene_Attachment::OnDeselection()
{
	DeleteOverlay();
	s_pxThis = 0;
}

void Tools_Cutscene_Attachment::CreateOverlay()
{
	char szText[200];

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Attachment",false,&s_xOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxDialog = new Asura_Numerical_Dialog("Attachment To Edit",xOffset,&m_pxToolsActor->m_iSelectedAttachmentIndex,0,m_pxToolsActor->m_xAttachmentList.size()-1,1,"%d",&m_pxToolsActor->m_bSelectedAttachmentIndexChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//Name selection dialog
	if(m_pxAsuraAttachment->m_xName.IsBlank())
	{
		strcpy(szText,"Name : *Not Set*");
	}
	else
	{
		sprintf(szText,"Name : %s",m_pxAsuraAttachment->m_xName.GetString());
	}
	pxDialog = new Asura_Function_Dialog(szText,xOffset,_EditName);
	pxDialog->SetHelpText("Edits name of attachment. Used for identification by cutscene events.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	//Marker point selection dialog
	if(m_pxAsuraAttachment->m_xMarkerName.IsBlank())
	{
		strcpy(szText,"Marker : *Not Set*");
	}
	else
	{
		sprintf(szText,"Marker : %s",m_pxAsuraAttachment->m_xMarkerName.GetString());
	}

	pxDialog = new Asura_Function_Dialog(szText,xOffset,_SelectMarkerPoint);
	pxDialog->SetHelpText("Edits marker point that attachment should be linked to.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Initially Active",xOffset,(int*)(&m_pxAsuraAttachment->m_uFlags),AsuraCutsceneAttachmentFlag_InitiallyActive,&m_bChanged);
	pxDialog->SetHelpText("Attachment is active (visible) at start of cutscene.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	//add the attachment specific controls
	AddOverlayControls(pxOverlay,xOffset);
	
	xOffset.y+=1;
	
	pxDialog = new Asura_Function_Dialog("Delete",xOffset,DestroyAttachment);
	pxDialog->SetHelpText("Deletes this attachment");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxOverlay->m_uID = TOOLS_OVERLAY_CUTSCENE_ATTACHMENT;

	pxOverlay->m_xOrigin = s_xOverlayPosition;
 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);

	UpdateOverlay();
}

void Tools_Cutscene_Attachment::DeleteOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_CUTSCENE_ATTACHMENT);
}

void Tools_Cutscene_Attachment::UpdateCutsceneEdit()
{
	if(m_bChanged)
	{
		Tools_SaveAsura::SetCutscenesModified();
		Tools_Mode_Cutscene::RefreshPlayback();
		m_bChanged = false;

	}
}

void Tools_Cutscene_Attachment::DestroyAttachment()
{
	Asura_Assert(s_pxThis,"");
	s_pxThis->Destroy();
}

void Tools_Cutscene_Attachment::SelectMarkerPoint()
{
	//get the actor's skin
	Asura_Assert(m_pxToolsActor->Actor->IsHierarchyActor(),"");
	Asura_ClientCutscene_Actor_Hierarchy* pxHierarchyActor = (Asura_ClientCutscene_Actor_Hierarchy*)m_pxToolsActor->Actor;
	
	//make sure the cutscene files have bee loaded
	m_pxAsuraAttachment->GetCutscene()->LoadFiles();	
	
	const char* pszSkinName = pxHierarchyActor->GetSkinName();
	if(!pszSkinName)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"A skin needs to be selected for the actor first.");
		return;
	}

	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(pszSkinName);
	if(!pxSkin)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find skin. Unable to give list of marker points.");
		return;
	}

	if(!pxSkin->NumberOfMarkerPoints)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"The skin doesn't have any marker points");
		return;
	}

	//add all the marker points to a selection overlay
	ListSelection_Overlay xOverlay("Select marker point for attachment");
	for(int i=0;i<pxSkin->NumberOfMarkerPoints;i++)
	{
		xOverlay.AddItem(pxSkin->MarkerPoint[i].GetName().GetString());
	}

	///////////////////////////////////
	for ( int i=0; i<pxHierarchyActor->m_iNumberOfAttachments; i++ )
	{
		if (pxHierarchyActor->m_ppxAttachments[i]->GetType() == AsuraCutsceneAttachment_Hierarchy )
		{
			const Asura_Cutscene_Attachment_Hierarchy* pxAttachment = static_cast<const Asura_Cutscene_Attachment_Hierarchy*>(pxHierarchyActor->m_ppxAttachments[i]);
			if ( pxAttachment && pxAttachment->GetSkinName(0) )
			{
				Asura_Hierarchy_Skin* pxAttachmentSkin = Asura_Animation_System::GetSkin(pxAttachment->GetSkinName(0));
				if ( pxAttachmentSkin )
				{
					for(int i=0;i<pxAttachmentSkin->NumberOfMarkerPoints;i++)
					{
						xOverlay.AddItem(pxAttachmentSkin->MarkerPoint[i].GetName().GetString());
					}
				}
			}
		}
	}
	///////////////////////////////////

	if(!xOverlay.DoModal()) return;

	m_pxAsuraAttachment->m_xMarkerName = xOverlay.GetSelectionString();

	//update the overlays
	if(s_pxThis == this)
	{
		DeleteOverlay();
		CreateOverlay();
	}
	Tools_Mode_Cutscene::RefreshPlayback();
	Tools_SaveAsura::SetCutscenesModified();

}

void Tools_Cutscene_Attachment::EditName()
{
	GetString_Overlay xOverlay("Attachment Name :",m_pxAsuraAttachment->m_xName.GetString());
	if(!xOverlay.DoModal()) return;

	m_pxAsuraAttachment->m_xName = xOverlay.GetString();
	
	//update the overlays
	if(s_pxThis == this)
	{
		DeleteOverlay();
		CreateOverlay();
	}
	Tools_SaveAsura::SetCutscenesModified();

	UpdateToolsName();
}

void Tools_Cutscene_Attachment::UpdateToolsName()
{
	if(m_pxAsuraAttachment->m_xName.IsBlank())
	{
		m_xToolsName = "Unnamed";
	}
	else
	{
		m_xToolsName = m_pxAsuraAttachment->m_xName;
	}
}

///////////////////////////////////////////////////////////////////////////////
/*-------------------------------------**
** Tools_Cutscene_Attachment_Hierarchy **
**-------------------------------------*/

Tools_Cutscene_Attachment_Hierarchy::Tools_Cutscene_Attachment_Hierarchy(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor)
:Tools_Cutscene_Attachment(pxAsuraAttachment,pxToolsActor)
{
}

void Tools_Cutscene_Attachment_Hierarchy::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	char szText[200];

	for(int i=0; i<ASURA_ANIMATION_SKIN_COUNT; i++)
	{
		const char* pszSkinName = GetSkinName(i);
		if(pszSkinName)
		{
       	    #if ASURA_ANIMATION_SKIN_COUNT > 1
			sprintf(szText,"Skin %d : %s",i,pszSkinName);
			#else
			sprintf(szText,"Skin : %s",pszSkinName);
			#endif
		}
		else
		{
       	    #if ASURA_ANIMATION_SKIN_COUNT > 1
			sprintf(szText,"Skin %d : *Not Set*",i);
			#else
			strcpy(szText,"Skin : *Not Set*");
			#endif
		}

		pxDialog = new Asura_Function_Dialog(szText,xOffset,_ChooseSkin, i);
		pxDialog->SetIsEnabledFunction(_IsEnabled_ChooseSkin, i);
		pxDialog->SetHelpText("Set skin used by this attachment.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
	
	const char* pszAnimName = GetAnimName();
	if(pszAnimName)
	{
		sprintf(szText,"Anim : %s",pszAnimName);
	}
	else
	{
		strcpy(szText,"Anim : *Not Set*");
	}

	pxDialog = new Asura_Function_Dialog(szText,xOffset,_ChooseAnim);
	pxDialog->SetHelpText("Set animation used by this attachment.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}

const char* Tools_Cutscene_Attachment_Hierarchy::GetSkinName(int iSkinIndex)
{
	Asura_Cutscene_Attachment_Hierarchy* pxAsuraAttachment = (Asura_Cutscene_Attachment_Hierarchy*) m_pxAsuraAttachment;
	if(pxAsuraAttachment->m_aiSkinID[iSkinIndex] == -1) return 0;

	Asura_Cutscene* pxCutscene = pxAsuraAttachment->GetCutscene();
	Asura_String& xSkinName = pxCutscene->SkinNames.GetName(pxAsuraAttachment->m_aiSkinID[iSkinIndex]);

	return xSkinName.IsBlank() ? 0 : xSkinName.GetString();
}

const char* Tools_Cutscene_Attachment_Hierarchy::GetAnimName()
{
	Asura_Cutscene_Attachment_Hierarchy* pxAsuraAttachment = (Asura_Cutscene_Attachment_Hierarchy*) m_pxAsuraAttachment;
	if(pxAsuraAttachment->m_iAnimID == -1) return 0;

	Asura_Cutscene* pxCutscene = pxAsuraAttachment->GetCutscene();
	Asura_String& xAnimName = pxCutscene->AnimationNames.GetName(pxAsuraAttachment->m_iAnimID);

	return xAnimName.IsBlank() ? 0 : xAnimName.GetString();
}
/*
void Tools_Cutscene_Attachment_Hierarchy::ChoosePFX()
{
	Asura_Cutscene_Attachment_Hierarchy* pxAsuraAttachment = (Asura_Cutscene_Attachment_Hierarchy*) m_pxAsuraAttachment;
	Asura_Cutscene* pxCutscene = pxAsuraAttachment->GetCutscene();

	bool bGotAMarker = false;

	if ( pxAsuraAttachment->m_iSkinID == -1 )
		return;

	Asura_String& xSkinName = pxCutscene->SkinNames.GetName(pxAsuraAttachment->m_iSkinID);
	if ( !xSkinName.IsNull() && !xSkinName.IsBlank() )
	{
		Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::GetSkinFromID(Asura_GetHashID(xSkinName.GetString()));
		if (pxSkin)
		{
			if ( pxSkin->NumberOfMarkerPoints == 0 )
				return;

			//add all the marker points to a selection overlay
			ListSelection_Overlay xOverlay("Select marker point for attachment");
			for(int i=0;i<pxSkin->NumberOfMarkerPoints;i++)
			{
				xOverlay.AddItem(pxSkin->MarkerPoint[i].GetName().GetString());
			}

			if(!xOverlay.DoModal()) return;

			pxAsuraAttachment->m_uPFXMarkerHash = Asura_GetHashID(xOverlay.GetSelectionString());

			bGotAMarker = true;
		}
	}

	if ( bGotAMarker )
	{
		ListSelection_Overlay xOverlay("Select PFX");
		u_int uNumTemplates = Asura_PFX_System::GetNumberOfEffectTemps();

		xOverlay.AddItem("None");
		//get all the template names
		for(u_int i=0;i<uNumTemplates;i++)
		{
			xOverlay.AddItem(Asura_PFX_System::GetEffectTempNameByID(i)->GetString());
		}

		if(!xOverlay.DoModal()) return;
		pxAsuraAttachment->m_uPFXHash = Asura_GetHashID(xOverlay.GetSelectionString());
	}
}
*/

bool Tools_Cutscene_Attachment_Hierarchy::IsEnabled_ChooseSkin(int iSkinIndex)
{
	if(iSkinIndex == 0) return true;
	//only allow secondary skins to be edited if a main skin has been selected , or if they
	//already have a skin(so as not to intefere with removing them)
	Asura_Cutscene_Attachment_Hierarchy* pxAsuraAttachment = (Asura_Cutscene_Attachment_Hierarchy*) m_pxAsuraAttachment;

	return (pxAsuraAttachment->m_aiSkinID[0] != -1) || (pxAsuraAttachment->m_aiSkinID[iSkinIndex] != -1);

}

void Tools_Cutscene_Attachment_Hierarchy::ChooseSkin(int iSkinIndex)
{
	Asura_Cutscene_Attachment_Hierarchy* pxAsuraAttachment = (Asura_Cutscene_Attachment_Hierarchy*) m_pxAsuraAttachment;
	Asura_Cutscene* pxCutscene = pxAsuraAttachment->GetCutscene();


	Asura_String xSkinName ,xSkinFilename;

	//get the user to select a skin
	if(!Tools_Animation_Files::ChooseSkin(&xSkinName,&xSkinFilename))
	{
		return;
	}

	//get an entry in the name lists , if we don't already have one
	if(pxAsuraAttachment->m_aiSkinID[iSkinIndex]==-1)
	{
		Asura_String xTemp = "";
		pxAsuraAttachment->m_aiSkinID[iSkinIndex] = pxCutscene->SkinNames.AddName(xTemp);

		#if defined ASURA_ASSERT
			int iFilenameIndex =
		#endif
			pxCutscene->SkinFileNames.AddName(xTemp);

		Asura_Assert(iFilenameIndex == pxAsuraAttachment->m_aiSkinID[iSkinIndex],"");
	}
    Asura_String & xSkinNameRef = pxCutscene->SkinNames.GetName(pxAsuraAttachment->m_aiSkinID[iSkinIndex]);
    Asura_String & xSkinFileNameRef = pxCutscene->SkinFileNames.GetName(pxAsuraAttachment->m_aiSkinID[iSkinIndex]);
    xSkinNameRef = xSkinName;
	xSkinFileNameRef = xSkinFilename;

	//update the overlays
	if(s_pxThis == this)
	{
		DeleteOverlay();
		CreateOverlay();
	}
	Tools_Mode_Cutscene::RefreshPlayback();
	Tools_SaveAsura::SetCutscenesModified();
}

void Tools_Cutscene_Attachment_Hierarchy::SetSkin( const Asura_Hash_ID uSkinHash, int iSkinIndex/*=0*/ )
{
	Asura_Cutscene_Attachment_Hierarchy* pxAsuraAttachment = (Asura_Cutscene_Attachment_Hierarchy*) m_pxAsuraAttachment;
	Asura_Cutscene* pxCutscene = pxAsuraAttachment->GetCutscene();

	const char* pszFullFileName = Asura_HashedFilename_System::GetFullFilename(uSkinHash);
	Asura_String xSkinName, xSkinFileName;
	if(pszFullFileName)
	{
		xSkinFileName = Asura_String(pszFullFileName);
		xSkinFileName.SplitFilename(NULL, &xSkinName, NULL);
	}


	//get an entry in the name lists , if we don't already have one
	if(pxAsuraAttachment->m_aiSkinID[iSkinIndex]==-1)
	{
		Asura_String xTemp = "";
		pxAsuraAttachment->m_aiSkinID[iSkinIndex] = pxCutscene->SkinNames.AddName(xTemp);

#if defined ASURA_ASSERT
		int iFilenameIndex =
#endif
			pxCutscene->SkinFileNames.AddName(xTemp);

		Asura_Assert(iFilenameIndex == pxAsuraAttachment->m_aiSkinID[iSkinIndex],"");
	}
	Asura_String & xSkinNameRef = pxCutscene->SkinNames.GetName(pxAsuraAttachment->m_aiSkinID[iSkinIndex]);
	Asura_String & xSkinFileNameRef = pxCutscene->SkinFileNames.GetName(pxAsuraAttachment->m_aiSkinID[iSkinIndex]);
	xSkinNameRef = xSkinName;
	xSkinFileNameRef = xSkinFileName;


	Asura_String xAnimName,xAnimFilename;
	//get an entry in the name lists , if we don't already have one
	if(pxAsuraAttachment->m_iAnimID==-1)
	{
		Asura_String xTemp = "";
		pxAsuraAttachment->m_iAnimID = pxCutscene->AnimationNames.AddName(xTemp);

#if defined ASURA_ASSERT
		int iFilenameIndex =
#endif
			pxCutscene->AnimationFileNames.AddName(xTemp);

		Asura_Assert(iFilenameIndex == pxAsuraAttachment->m_iAnimID,"");
	}
	Asura_String & xAnimNameRef = pxCutscene->AnimationNames.GetName(pxAsuraAttachment->m_iAnimID);
	Asura_String & xAnimFileNameRef = pxCutscene->AnimationFileNames.GetName(pxAsuraAttachment->m_iAnimID);

	Tools_Animation_Files::GetStandardRestAnim(xAnimNameRef, xAnimFileNameRef);

	//xAnimNameRef = "Standard_Rest_Anim";
	//xAnimFileNameRef = "Objects\\Standard_Rest_Anim.ASR";

	//update the overlays
	if(s_pxThis == this)
	{
		DeleteOverlay();
		CreateOverlay();
	}
	Tools_Mode_Cutscene::RefreshPlayback();
	Tools_SaveAsura::SetCutscenesModified();
}

void Tools_Cutscene_Attachment_Hierarchy::ChooseAnim()
{
	Asura_Cutscene_Attachment_Hierarchy* pxAsuraAttachment = (Asura_Cutscene_Attachment_Hierarchy*) m_pxAsuraAttachment;
	Asura_Cutscene* pxCutscene = pxAsuraAttachment->GetCutscene();

	const char* pszSkinName = GetSkinName(0);
	if(!pszSkinName)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Select a skin for the attachment first");
	}


	Asura_String xAnimName,xAnimFilename;

	//get the user to select a skin
	if(!Tools_Animation_Files::ChooseAnimation(pszSkinName,&m_xCurrentDir,&xAnimName,&xAnimFilename))
	{
		return;
	}

	//get an entry in the name lists , if we don't already have one
	if(pxAsuraAttachment->m_iAnimID==-1)
	{
		Asura_String xTemp = "";
		pxAsuraAttachment->m_iAnimID = pxCutscene->AnimationNames.AddName(xTemp);

		#if defined ASURA_ASSERT
			int iFilenameIndex =
		#endif
			pxCutscene->AnimationFileNames.AddName(xTemp);

		Asura_Assert(iFilenameIndex == pxAsuraAttachment->m_iAnimID,"");
	}
    Asura_String & xAnimNameRef = pxCutscene->AnimationNames.GetName(pxAsuraAttachment->m_iAnimID);
    Asura_String & xAnimFileNameRef = pxCutscene->AnimationFileNames.GetName(pxAsuraAttachment->m_iAnimID);
    xAnimNameRef = xAnimName;
	xAnimFileNameRef = xAnimFilename;

	//update the overlays
	if(s_pxThis == this)	
	{
		DeleteOverlay();
		CreateOverlay();
	}

	Tools_Mode_Cutscene::RefreshPlayback();
	Tools_SaveAsura::SetCutscenesModified();
}

///////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_Cutscene_Attachment_Shape **
**---------------------------------*/

Tools_Cutscene_Attachment_Shape::Tools_Cutscene_Attachment_Shape(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor)
:Tools_Cutscene_Attachment(pxAsuraAttachment,pxToolsActor)
{
}

void Tools_Cutscene_Attachment_Shape::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Attachment_Shape* pxAsuraAttachment = (Asura_Cutscene_Attachment_Shape*) m_pxAsuraAttachment;

	Asura_Dialog* pxDialog;

	//add the shape selection option
	char szText[200];

	const char* pszShapeName = 0;
	Asura_Hash_ID uShapeID = pxAsuraAttachment->m_xShapeInstance.GetShapeHashID();
	

	
	if(uShapeID)
	{
		pszShapeName = Tools_ShapeTable::LookupObjectMeshName(uShapeID);
		if(pszShapeName)
		{
			sprintf(szText,"Shape : %s",pszShapeName);
		}
		else
		{
			strcpy(szText,"Shape : *Unknown*");
		}
	}
	else
	{
		strcpy(szText,"Shape : *Not Set*");
	}

	pxDialog = new Asura_Function_Dialog(szText,xOffset,_ChooseShape);
	pxDialog->SetHelpText("Set shape used by this attachment.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}

void Tools_Cutscene_Attachment_Shape::Destroy()
{
	Tools_SaveAsura::SetShapesModified();
	Tools_Cutscene_Attachment::Destroy();
}

void Tools_Cutscene_Attachment_Shape::ChooseShape()
{
	Asura_Cutscene_Attachment_Shape* pxAsuraAttachment = (Asura_Cutscene_Attachment_Shape*) m_pxAsuraAttachment;

	//ask for the new shape
	Asura_Hash_ID uShape = Tools_ShapeTable::SelectShape();
	if(uShape != ASURA_HASH_ID_UNSET)
	{
		pxAsuraAttachment->m_xShapeInstance.SetShape(uShape);		

		//update the overlay
		if(s_pxThis == this)	
		{
			DeleteOverlay();
			CreateOverlay();
		}
		Tools_SaveAsura::SetCutscenesModified();
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_Cutscene_Attachment_Actor **
**---------------------------------*/

Tools_Cutscene_Attachment_Actor::Tools_Cutscene_Attachment_Actor(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor)
:Tools_Cutscene_Attachment(pxAsuraAttachment,pxToolsActor)
{
}

void Tools_Cutscene_Attachment_Actor::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Attachment_Actor* pxAsuraAttachment = (Asura_Cutscene_Attachment_Actor*) m_pxAsuraAttachment;

	Asura_Dialog* pxDialog;

	//add the actor selection option
	char szText[200];

	if(pxAsuraAttachment->m_xAttachedActorName.IsBlank())
	{
		strcpy(szText,"Actor : *Not Set*");
	}
	else
	{
		sprintf(szText,"Actor : %s",pxAsuraAttachment->m_xAttachedActorName.GetString());
	}

	pxDialog = new Asura_Function_Dialog(szText,xOffset,_SelectActor);
	pxDialog->SetHelpText("Set cutscene actor that this is moved by this attachment.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
}


void Tools_Cutscene_Attachment_Actor::SelectActor()
{
	Asura_Cutscene_Attachment_Actor* pxAsuraAttachment = (Asura_Cutscene_Attachment_Actor*) m_pxAsuraAttachment;
	
	
	Tools_Cutscene_Object* pxCutscene = Tools_Mode_Cutscene::CurrentCutscene;
	
	//add all the hierarchy actors to a selection overlay
	ListSelection_Overlay xOverlay("Select Actor:");

	for(LIF<Tools_CutsceneActor_Object*> xActorLIF(&pxCutscene->ActorObjects);!xActorLIF.done();xActorLIF.next())
	{
		//don't want to be able to choose the actor that this attachment belongs to
		if(xActorLIF() == m_pxToolsActor) continue;

		xOverlay.AddItem(xActorLIF()->Actor->ActorName.GetString());
	}

	if(!xOverlay.GetNumItems())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"There aren't any other actors");
		return;
	}

	if(!xOverlay.DoModal()) return;

	pxAsuraAttachment->m_xAttachedActorName = xOverlay.GetSelectionString();

	//attaching an actor to another may require a change to the ordering
	pxCutscene->SortActors();
	

	//update the overlay
	if(s_pxThis == this)	
	{
		DeleteOverlay();
		CreateOverlay();
	}
	Tools_Mode_Cutscene::RefreshPlayback();
	Tools_SaveAsura::SetCutscenesModified();
	
}


////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------**
** Tools_Cutscene_Attachment_Entity **
**------------------------------------*/

Tools_Cutscene_Attachment_Entity::Tools_Cutscene_Attachment_Entity(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor)
:Tools_Cutscene_Attachment(pxAsuraAttachment,pxToolsActor)
{
	m_pxChooseObject = 0;
}

Tools_Cutscene_Attachment_Entity::~Tools_Cutscene_Attachment_Entity()
{
	if(m_pxChooseObject)
	{
		Tools_Object_System::ChooseObject_Finish();
		Asura_Assert(!m_pxChooseObject,"");
	}
}

void Tools_Cutscene_Attachment_Entity::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Attachment_Entity* pxAsuraAttachment = (Asura_Cutscene_Attachment_Entity*) m_pxAsuraAttachment;

	Asura_Dialog* pxDialog;

	//add the particle selection option
	char szText[200];

	Tools_Object * pxEntityObject = Tools_Object_System::GetObjectFromGuid( pxAsuraAttachment->m_uEntityGUID );

	if( pxAsuraAttachment->m_uEntityGUID == ASURA_GUID_UNREGISTERED )
	{
		strcpy(szText,"Entity : *Not Set*");
	}
	else if (pxEntityObject)
	{
		sprintf(szText,"Entity : %s",pxEntityObject->GetName());
	}
	else
	{
		strcpy(szText,"Entity : *Not Found*");
	}
	

	pxDialog = new Asura_Function_Dialog(szText,xOffset,_SelectEntity);
	pxDialog->SetHelpText("Choose entity (particle generator, sound or advanced light) to attach.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
}


void Tools_Cutscene_Attachment_Entity::SelectEntity()
{
	//start the object selection process
	m_pxChooseObject = new Tools_ChooseEntityForAttachment(this);
	Tools_Object_System::ChooseObject("Select particle generator , sound or advanced light",m_pxChooseObject);
}

void Tools_Cutscene_Attachment_Entity::SetEntity(Tools_Object* pxObject)
{
	Asura_Cutscene_Attachment_Entity* pxAsuraAttachment = (Asura_Cutscene_Attachment_Entity*) m_pxAsuraAttachment;
	if(!pxObject) return;
	
	if(pxObject->GetGuid()==ASURA_GUID_UNREGISTERED)
	{
		pxObject->AcquireGuid();
	}
	
	pxAsuraAttachment->m_uEntityGUID = pxObject->GetGuid();
	

	//update the overlay
	if(s_pxThis == this)	
	{
		DeleteOverlay();
		CreateOverlay();
	}
	Tools_Mode_Cutscene::RefreshPlayback();
	Tools_SaveAsura::SetCutscenesModified();
}


////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_ChooseEntityForAttachment **
**---------------------------------*/
PROJECT_CAN_SELECT_ENTITY_FOR_ATTACHMENT_FN Tools_ChooseEntityForAttachment::s_pfnProjectCanBeSelected = 0;
bool Tools_ChooseEntityForAttachment::CanBeSelected(Tools_Object* pxObject)
{
	Asura_Assert(pxObject,"");
	if (s_pfnProjectCanBeSelected &&
		s_pfnProjectCanBeSelected(pxObject))
	{
		return true;
	}

	else
	{
		switch (pxObject->Type)
		{
			case TOT_Phonon:
			case TOT_Entity_ParticleGenerator:
			case TOT_Entity_ExtendedParticleGenerator:
			case TOT_Entity_PFX_Effect:

#ifdef ASURA_TARGET_SM3
			case TOT_Entity_DynamicLight :
#else // #ifdef ASURA_TARGET_SM3
			case TOT_Entity_AdvancedLight:
#endif // #ifdef ASURA_TARGET_SM3

			{
				return	(true);
			}

			default:
			{
				return	(false);
			}
		}		// End of switch.
	}
}

void Tools_ChooseEntityForAttachment::ObjectSelected(Tools_Object* pxObject)
{
	if(Tools_SecondaryEntityFile_System::IsAllowableTarget(0,true,pxObject))
	{
		m_pxAttachment->SetEntity(pxObject);
	}
	
}

////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_Cutscene_Attachment_Corona **
**----------------------------------*/

Tools_Cutscene_Attachment_Corona::Tools_Cutscene_Attachment_Corona(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor)
:Tools_Cutscene_Attachment(pxAsuraAttachment,pxToolsActor)
{
}

Tools_Cutscene_Attachment_Corona::~Tools_Cutscene_Attachment_Corona()
{
}

void Tools_Cutscene_Attachment_Corona::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Attachment_Corona* pxAsuraAttachment = (Asura_Cutscene_Attachment_Corona*) m_pxAsuraAttachment;

	Asura_Dialog* pxDialog;
     

	pxDialog = new Asura_Function_Dialog("",xOffset,_SelectTexture);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->m_iControlID = DialogID_SelectTexture;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog  = new Asura_ColourNumerical_Dialog("Red",xOffset,&(pxAsuraAttachment->m_fRed),"%d", &m_bChanged);
	pxDialog->SetHelpText("Red component (0-255).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    pxDialog = new Asura_ColourNumerical_Dialog("Green",xOffset,&(pxAsuraAttachment->m_fGreen),"%d", &m_bChanged);
	pxDialog->SetHelpText("Green component (0-255).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    pxDialog = new Asura_ColourNumerical_Dialog("Blue",xOffset,&(pxAsuraAttachment->m_fBlue),"%d", &m_bChanged);
	pxDialog->SetHelpText("Blue component (0-255).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Radius", xOffset, &pxAsuraAttachment->m_fSize, 0.0f, 100.0f, 0.01f, "%.2f", &m_bChanged);
	pxDialog->SetHelpText("Radius in metres.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Use Orient",xOffset,(int*)&pxAsuraAttachment->m_uCoronaFlags,AsuraCoronaAttachmentFlag_UseOrientation,&m_bChanged);
	pxDialog->SetHelpText("Causes the corona to fade out if not viewed head on.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	
}

void Tools_Cutscene_Attachment_Corona::UpdateOverlay()
{
	Tools_Cutscene_Attachment::UpdateOverlay();

	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_CUTSCENE_ATTACHMENT);
	if(!pxOverlay) return;

	Asura_Cutscene_Attachment_Corona* pxAsuraAttachment = (Asura_Cutscene_Attachment_Corona*) m_pxAsuraAttachment;
	char szText[100];

	//update the texture select control
	if(pxAsuraAttachment->m_iTextureID == ASURA_TEXTURE_NONE)
	{
		strcpy(szText,"Texture : <Not Set>");
	}
	else
	{
		const Asura_String* pxName = Asura_Texture_Management::GetTextureName(pxAsuraAttachment->m_iTextureID);
		if(pxName)
		{
			//get the name without path/extension
			Asura_String xFilename;
			pxName->SplitFilename(0,&xFilename,0);

			Asura_PrintfToBfr(szText,"Texture : %s",100,true,xFilename.GetString());
		}
		else
		{
			strcpy(szText,"Texture : <Unknown>");
		}

	}
	pxOverlay->UpdateDialogLabel(DialogID_SelectTexture,szText);

	
}


void Tools_Cutscene_Attachment_Corona::SelectTexture()
{
	Asura_Cutscene_Attachment_Corona* pxAsuraAttachment = (Asura_Cutscene_Attachment_Corona*) m_pxAsuraAttachment;

	int iSelected = Tools_PickTexture::PickTexture("Select Corona Texture",pxAsuraAttachment->m_iTextureID,PICKTEXTURE_ALLTEXTURES);

	if(iSelected != ASURA_TEXTURE_NONE && iSelected != pxAsuraAttachment->m_iTextureID)
	{
		pxAsuraAttachment->m_iTextureID = iSelected;
		m_bChanged = true;
		UpdateOverlay();
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------**
** Tools_Cutscene_Attachment_PFX **
**-------------------------------*/

Tools_Cutscene_Attachment_PFX::Tools_Cutscene_Attachment_PFX(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor)
:Tools_Cutscene_Attachment(pxAsuraAttachment,pxToolsActor)
{
}

Tools_Cutscene_Attachment_PFX::~Tools_Cutscene_Attachment_PFX()
{
}

void Tools_Cutscene_Attachment_PFX::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Attachment_PFX* pxAsuraAttachment = (Asura_Cutscene_Attachment_PFX*) m_pxAsuraAttachment;

	Asura_Dialog* pxDialog;

	pxDialog = new Tools_PFXSelect_Dialog(Tools_PFXSelect_Dialog::MODE_EFFECTS, xOffset,&pxAsuraAttachment->m_uEffectTemplateHash,&m_bChanged);
	pxDialog->SetHelpText("The PFX effect to use for this attachment");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	     
	
}

////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_Cutscene_Attachment_SoundEvent **
**--------------------------------------*/

Tools_Cutscene_Attachment_SoundEvent::Tools_Cutscene_Attachment_SoundEvent(Asura_Cutscene_Attachment* pxAsuraAttachment,Tools_CutsceneActor_Object* pxToolsActor)
:Tools_Cutscene_Attachment(pxAsuraAttachment,pxToolsActor)
{
}

Tools_Cutscene_Attachment_SoundEvent::~Tools_Cutscene_Attachment_SoundEvent()
{
}

void Tools_Cutscene_Attachment_SoundEvent::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Attachment_SoundEvent* pxAsuraAttachment = (Asura_Cutscene_Attachment_SoundEvent*) m_pxAsuraAttachment;

	Asura_Dialog* pxDialog;

	Asura_Dialog* pxSoundEventDialog = pxDialog = new Tools_PickSoundEvent_Dialog("Event : ",xOffset,&pxAsuraAttachment->m_uSoundEvent,&m_bChanged);
	pxDialog->SetHelpText("Select the sound event to use.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//add control for editing the selected sound event
	pxDialog = new Tools_EditSoundEvent_Dialog("Edit",xOffset,&pxAsuraAttachment->m_uSoundEvent);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxSoundEventDialog,Asura_Vector_2(3.0f,0.0f),false,true);

	     
	
}




/// End of file. ///
