#include "Core/Asura.h"

#include "Cutscene/Tools_Cutscene_Timeline.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_String.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_Event.h"
#include "Cutscene/Asura_Cutscene_Actor.h"
#include "Core/Asura_TextureManagement.h"
#include "Debugging/Asura_CPUProfile_Renderer.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Interface.h"
#include "Core/Asura_UserInput.h"
#include "Rendering/Asura_Text_Output.h"
#include "Core/Asura_Windows.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_Overlay_System.h"

#include "Cutscene/Tools_Mode_Cutscene.h"
#include "Cutscene/Tools_CutsceneActor_Object.h"
#include "Cutscene/Tools_Cutscene_Object.h"
#include "Cutscene/Tools_Cutscene_Track.h"
#include "Cutscene/Tools_Cutscene_Event.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"

#include "Debugging/Asura_Assert.h"

#include <math.h>

///////////////////////////////////////////////////////////////////////////////
/*-------------------------**
** Tools_Cutscene_Timeline **
**-------------------------*/
Tools_Cutscene_Object* Tools_Cutscene_Timeline::CutsceneObject = 0;
Tools_CutsceneActor_Object* Tools_Cutscene_Timeline::ActorObject = 0;
bool Tools_Cutscene_Timeline::s_bCurrentActorChanged = false;
List<Tools_Timeline_Actor*> Tools_Cutscene_Timeline::ActorList;
float Tools_Cutscene_Timeline::CutsceneLength = 0;
List<Tools_Timeline_Data*> Tools_Cutscene_Timeline::SelectedList;
bool Tools_Cutscene_Timeline::Valid = false;
float Tools_Cutscene_Timeline::MinVisibleTime = 0;
float Tools_Cutscene_Timeline::MaxVisibleTime = 20.0f;
List<int> Tools_Cutscene_Timeline::s_xFullScreenHiddenEventTypes;
List<int> Tools_Cutscene_Timeline::s_xTimelineBarVisibleEventTypes;
int Tools_Cutscene_Timeline::s_iPrimarySelectedID = 0;
bool Tools_Cutscene_Timeline::s_bUseSnap = true;
float Tools_Cutscene_Timeline::s_fFullScreenActorYOffset = 0;
float Tools_Cutscene_Timeline::s_fTimelineBarActorYOffset = 0;

static float PixelsPerSecond = 50;
static float FirstTimeShown = 0;
static const float TimelineBarsStartX = 100;
static float ActorStartY = 14;
//static float StartY = 0.0f;
static float g_fTimelineBarTopY = 0;
static float g_fTimelineBarBottomY = 0.0f;//Y pos to use for bottom when rendering single timeline for single actor
static float g_fTimelineBarHeight = 100.0f;
static float g_fTimelineBarWidth = 0.0f;
static Tools_Timeline_Data* MouseOverItem = 0;
static u_int g_uMouseOverItemFlags = 0;
static float g_fTimelineDataMaxY = 0.0f;


static Asura_Bounding_Rectangle g_xFullRenderRect;
static Asura_Bounding_Rectangle g_xNameRenderRect;
static Asura_Bounding_Rectangle g_xBarRenderRect;
static Asura_Bounding_Rectangle g_xLowerTextRenderRect;
static Asura_Vector_2 g_xBarOffset;

static bool FullTimelineScreen = false;
							
enum
{
	DragMode_None = 0,
	DragMode_Undetermined,
	DragMode_ReplaceSelection,
	DragMode_AddToSelection,
	DragMode_Deselect,
	DragMode_Move,
	DragMode_Scroll,
};

static int DragMode = DragMode_None;
static Asura_Vector_2 DragStart;
static Asura_Vector_2 DragCurrent;
static int g_iDragMouseButton;
static bool g_bDraggedItemTimeInitialised;
static float g_fDraggedItemStartTime;
static float g_fDraggedItemEndTime;
static float g_fDragInitialFirstTimeShown;
static float g_fDragInitialActorYOffset;
static bool TimeToGo = false;
static bool THasBeenDebounced = false;//prevent the T used to enter this screen from being picked up for adjusting time
static bool MovingItems = false;


void Tools_Cutscene_Timeline::ShowTimeline(Tools_Cutscene_Object* cutsceneObject)
{
	Asura_Assert(cutsceneObject,"");
	CutsceneObject = cutsceneObject;
	FullTimelineScreen = true;
	GenerateTimelineData();

	Tools_Mode_Cutscene::RefreshPlayback();
	
	DragMode = DragMode_None;
	TimeToGo = false;
	THasBeenDebounced = false;//prevent the T used to enter this screen from being picked up for adjusting time
	
	//prevents profile system from complaining as a result of profiling multiple things at once
	ASURA_CPUPROFILE_SCOPED_PAUSE

	//now for our main loop
	do
	{
		Asura_Windows::CheckForWindowsMessages();
		Asura_UserInput::Update();
		Asura_Cursor::OncePerFrameUpdate();
		Asura_Timers::HandlePrimaryTimer();

		Update();

		Tools_Help::Update();

		
		//do the rendering
		Asura_Render::BeginFrame();

		Render();

		Tools_Help::Render();


		Asura_Cursor::SetType(ASURA_CURSOR_CROSSHAIR);
		Asura_Cursor::Render();
		Asura_Render::EndFrame();
	
	}
	while(!TimeToGo);
	
	DeleteTimelineData();

	FullTimelineScreen = false;

}

void Tools_Cutscene_Timeline::Update(int focusFlags)
{
	if(!Valid)
	{
		GenerateTimelineData();
	}

	CalculateRenderRectangles();
	
	//Force all undo data logged while dragging to be included in same batch
	if(DragMode == DragMode_Move)
	{
		CutsceneUndo->KeepUndoBatchOpenThisFrame();
	}
	
	//mark start of another frame for undo if part of the full timeline screen
	//(if being called by bar at bottom of screen  , this gets dealt with in the main
	//mode update). RAR 30/06/2005
	if (FullTimelineScreen)
	{
		CutsceneUndo->MarkUndoStartIfEndOfBatch();
	}


	//get the current mouse position
	int mx,my;
	Asura_Cursor::GetScreenSpaceCoords(mx,my);
	
	MouseOverItem = GetItemAtCoord(mx,my,&g_uMouseOverItemFlags);
	
	DragCurrent.x = (float)mx;
	DragCurrent.y = (float)my;
	if(DragMode != DragMode_Scroll)
	{
		DragCurrent.x = Asura_Maths::ClampToRange(DragCurrent.x, g_xBarRenderRect.MinX, g_xBarRenderRect.MaxX);
		DragCurrent.y = Asura_Maths::ClampToRange(DragCurrent.y, g_xBarRenderRect.MinY, g_xBarRenderRect.MaxY);
	}

	//handle input

	if(focusFlags & OverlayFocusType_Mouse)
	{
		if(DragMode == DragMode_None)
		{
			if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
			{
				//start drag with LMB
				DragStart = DragCurrent;
				g_iDragMouseButton = ASURA_KEYID_MOUSEBUTTON1;
				g_bDraggedItemTimeInitialised = false;
				
				if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
				{
					//hold shift for deselection
					DragMode = DragMode_Deselect;
				}
				else if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
				{
					//hold control to add to current selection
					DragMode = DragMode_AddToSelection;
				}
				else if(MouseOverItem)
				{
					if((MouseOverItem->GetSelectionFlags() & g_uMouseOverItemFlags) != g_uMouseOverItemFlags)
					{
						//clicking on an item that wasn't already selected , clear everything else
						while(SelectedList.size())
						{
							SelectItem(SelectedList.first_entry(),TOOLS_TIMELINE_SELECTED_ALL,false);
						}
					}
					
					//make sure the item the mouse is over is the first in the list
					SelectItem(MouseOverItem, g_uMouseOverItemFlags,true);
					//mode depends on amount of mouse movement
					DragMode = DragMode_Undetermined;
				}
				else
				{
					DragMode = DragMode_ReplaceSelection;
				}
			}
			else if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON3))
			{
				//start scrolling the view via dragging
				DragStart = DragCurrent;
				g_iDragMouseButton = ASURA_KEYID_MOUSEBUTTON3;
				DragMode = DragMode_Scroll;

				g_fDragInitialFirstTimeShown = FirstTimeShown;
				g_fDragInitialActorYOffset = FullTimelineScreen ? s_fFullScreenActorYOffset : s_fTimelineBarActorYOffset;

			}
		}
		else
		{
			//Continue/stop dragging depending on whether the appropriate mouse button is still held
			if(Asura_UserInput::KeyPressed(g_iDragMouseButton))
			{
				if(DragMode == DragMode_Undetermined)
				{
					if((DragCurrent-DragStart).Magnitude()>3)
					{
						//mouse has moved a fair way... assume the user wants to move the item , instead
						//of just selecting it
						DragMode = DragMode_Move;
					}
				}
				if(DragMode == DragMode_Scroll)
				{
					Asura_Vector_2 xDragOffset = DragCurrent-DragStart;

					FirstTimeShown = g_fDragInitialFirstTimeShown - xDragOffset.x / PixelsPerSecond;
					if(FirstTimeShown < 0.0f) FirstTimeShown = 0.0f;

					float& fActorYOffset = FullTimelineScreen ? s_fFullScreenActorYOffset : s_fTimelineBarActorYOffset;
					fActorYOffset = g_fDragInitialActorYOffset - xDragOffset.y;
					if(fActorYOffset<0.0f) fActorYOffset=0.0f;
				}
			}
			else
			{
				if(DragMode == DragMode_Undetermined)
				{
					DragMode = DragMode_ReplaceSelection;
				}
			
				if(DragMode == DragMode_AddToSelection || DragMode == DragMode_ReplaceSelection || DragMode == DragMode_Deselect)
				{

					if(DragMode == DragMode_ReplaceSelection)
					{
						//clear existing selection before selecting the new items
						while(SelectedList.size())
						{
							SelectItem(SelectedList.first_entry(), TOOLS_TIMELINE_SELECTED_ALL,false);
						}
					}

					if((DragCurrent-DragStart).Magnitude()<=2)
					{
						//single selection/deslection
						if(MouseOverItem)
						{
							SelectItem(MouseOverItem, g_uMouseOverItemFlags,DragMode != DragMode_Deselect);
						}
					}
					else
					{
						//select everything in the rectangle
						for(LIF<Tools_Timeline_Actor*> alif(&ActorList);!alif.done();alif.next())
						{
							alif()->SelectInRect(DragStart,DragCurrent,DragMode != DragMode_Deselect);
						}

					}
				}
				DragMode = DragMode_None;
			}
		}

		
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON2))
		{
			Tools_ContextMenu_System::SetRightClickUsed();
			
			if(MouseOverItem)
			{
				TimeToGo = MouseOverItem->Edit();
			}
			else
			{
				//is the mouse over the name of an actor?
				Tools_Timeline_Actor* actor = GetActorAtCoord(mx,my);
				if(actor) actor->Edit();
			}
		}
	}
	MovingItems = false;
	if(focusFlags & OverlayFocusType_Keyboard)
	{
		if(Asura_UserInput::KeyPressed('t') || DragMode == DragMode_Move)
		{
			if(THasBeenDebounced)
			{
				MovingItems = true;
				
				UpdateDragMovement();
			}
		}
		else
		{
			THasBeenDebounced = true;
		}
		if(Asura_UserInput::KeyPressed('l'))
		{
			//L : deselect all
			while(SelectedList.size())
			{
				SelectItem(SelectedList.first_entry(), TOOLS_TIMELINE_SELECTED_ALL, false);
			}
		}
		if(Asura_UserInput::DebouncedKeyPressed('r'))
		{
			//R : Run cutscene to time
			float time = ((mx-TimelineBarsStartX)/PixelsPerSecond)+FirstTimeShown;
			if(MouseOverItem)
			{
				time = MouseOverItem->Time;
			}
			if(time<0) time = 0;
			Tools_Mode_Cutscene::s_pxThis->AdvanceCutsceneToTime(time);
			if(FullTimelineScreen)
			{
				TimeToGo = true;
			}

		}
#if 0
		if(Asura_UserInput::DebouncedKeyPressed('p'))
		{
			//P : Set parent event
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
			{
				SetParentEvent(0);
			}
			else
			{
				if(MouseOverItem && MouseOverItem->GetEvent())
				{
					SetParentEvent(MouseOverItem->GetEvent());
				}
			}
		}
#endif
		if(Asura_UserInput::DebouncedKeyPressed('a'))
		{
			//determine which actors to hide
			EditHiddenActors();
		}
		if(Asura_UserInput::DebouncedKeyPressed('e'))
		{
			//determine which events to hide
			EditHiddenEvents();
		}
		if(Asura_UserInput::DebouncedKeyPressed('s'))
		{
			s_bUseSnap = !s_bUseSnap;
		}

        // (MH - 21.06.2005) added undo support here
		if((Asura_UserInput::DebouncedKeyPressed('z') && Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
			|| Asura_UserInput::DebouncedKeyPressed(','))// 'ctrl+z' or '<'for  undo
		{
			DeleteTimelineData();
			CutsceneUndo->Undo();
			GenerateTimelineData();
		}
		if((Asura_UserInput::DebouncedKeyPressed('y') && Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
			|| Asura_UserInput::DebouncedKeyPressed('.'))// 'ctrl+y' or '>'for  redo
		{
			DeleteTimelineData();
			CutsceneUndo->Redo();
			GenerateTimelineData();
		}
		

		HandleScrollAndZoom();
	}
	
	if(!Tools_Help::IsDisplayingHelp())
	{
		
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
		{
			TimeToGo = true;
		}
	
	}
}

void Tools_Cutscene_Timeline::Render()
{
	if(!Valid)
	{
		GenerateTimelineData();
	}

	if(s_bCurrentActorChanged)
	{
		//Set the scrollbar to make the current actor visible
		if(ActorObject)
		{
			Tools_Timeline_Actor* pxTimelineActor = 0;
			for(LIF<Tools_Timeline_Actor*> xTimelineActorLIF(&ActorList); !xTimelineActorLIF.done(); xTimelineActorLIF.next())
			{
				if(xTimelineActorLIF()->ActorObject == ActorObject)
				{
					pxTimelineActor = xTimelineActorLIF();
					break;
				}
			}

			if(pxTimelineActor)
			{
				s_fTimelineBarActorYOffset = pxTimelineActor->Y;
			}
		}
		s_bCurrentActorChanged = false;
	}


	float width = (float)Asura_Viewport::GetWidth();
	float height = (float)Asura_Viewport::GetHeight();
	CalculateRenderRectangles();

	if(g_xFullRenderRect.GetHeight()==0.0f) return;


	//first draw a big black poly over everything that has taken place so far...
	if(FullTimelineScreen)
	{
		Asura_Render::Overlay.SetRectXY(0,0,width,height);
	}
	else
	{
		width =  g_fTimelineBarWidth;
		Asura_Render::Overlay.SetRectXY(0,g_fTimelineBarTopY,width,g_fTimelineBarHeight);
	}

	Asura_Render::Overlay.SetRectUV(0,0,1,1);
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Colour=0xff000000;
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	//render the names
	Asura_Render::Tools_SetClippingRectangle(g_xNameRenderRect);
	for(LIF<Tools_Timeline_Actor*> alif(&ActorList);!alif.done();alif.next())
	{
		alif()->RenderHeadings();
	}
	Asura_Render::FlushDisplay();


	Asura_Render::Tools_SetClippingRectangle(g_xBarRenderRect);

	//render the timeline data
	for(LIF<Tools_Timeline_Actor*> alif(&ActorList);!alif.done();alif.next())
	{
		alif()->Render();
	}

	Asura_Render::FlushDisplay();
	Asura_Render::AllowFullScreenRendering();



	float print_x,print_y;
	
	Asura_Render::Tools_SetClippingRectangle(g_xLowerTextRenderRect);
	{
		print_x = 0;
		print_y = g_xLowerTextRenderRect.MinY;
	
		Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
		const Tools_Timeline_Data* pxItemToDisplay = MovingItems ? GetPrimarySelection() : MouseOverItem;

		if(pxItemToDisplay)
		{
			//show a description of the item the mouse is over
			const char* pszFormat;
			float fTime;
									
			if(Tools_Mode_Cutscene::GetTimerMode() == CUTSCENETIMER_SECONDS)
			{
				fTime = pxItemToDisplay->GetStartTime();
				pszFormat = "Time : %.3f sec";
			}
			else
			{
				fTime = Tools_Mode_Cutscene::ConvertSecondsToDisplayedTime(fTime = pxItemToDisplay->GetStartTime());
				pszFormat = "Time : %.0f";
			}
			Asura_Text_Output::Print(print_x,print_y,0xffffffff,0,pszFormat,fTime);
			pxItemToDisplay->PrintDescription(print_x,print_y+1);
			
		}
		if(FullTimelineScreen)
		{
			Asura_Text_Output::Print(print_x,print_y+2,0xffffffff,0,"(S)nap : %s",s_bUseSnap ? "On" : "Off");
			Asura_Text_Output::Print(print_x,print_y+3,0xffffffff,0,"Ctrl+H Help     Escape : Exit");
		}
	}
	Asura_Render::FlushDisplay();

	
	//draw the times in seconds across the top
	{
		//determine what multiples of time to display
		int multiplesToShow = 1;
		float fPixelsPerValue = PixelsPerSecond;
		if(Tools_Mode_Cutscene::GetTimerMode() == CUTSCENETIMER_25FPS)
		{
			fPixelsPerValue /= 25.0f;
		}
		else if(Tools_Mode_Cutscene::GetTimerMode() == CUTSCENETIMER_30FPS)
		{
			fPixelsPerValue /= 30.0f;
		}
		if(fPixelsPerValue<50) multiplesToShow = (int)(50/fPixelsPerValue);
		
		int time = (int)(Tools_Mode_Cutscene::ConvertSecondsToDisplayedTime(FirstTimeShown)+0.999f);
		if(time%multiplesToShow)
		{
			time+=multiplesToShow;
			time -= (time%multiplesToShow);
		}

		Asura_Render::Tools_SetClippingRectangle(g_xFullRenderRect);
		
		do
		{
			float fTimeSeconds = Tools_Mode_Cutscene::ConvertDisplayedTimeToSeconds((float)time);
			
			print_x =(float)(int) (((fTimeSeconds - FirstTimeShown)*PixelsPerSecond)+TimelineBarsStartX);
			//if(FullTimelineScreen)
			//{
			//	print_y = StartY;
			//}
			//else
			//{
			//	print_y = g_fTimelineBarBottomY-14.0f;
			//}
			print_y = g_xFullRenderRect.MinY;

			if(print_x+50>width) break;

			
			Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
			Asura_Text_Output::Print(print_x,print_y,0xff00ff7f,0,"%d",time);
	
			time += multiplesToShow;

						
		}while(1);

		Asura_Render::FlushDisplay();
	}
	//draw a vertical line for the current cutscene time
	{
		float fCurrentTime = Tools_Mode_Cutscene::s_pxThis->CutscenePlayback_Timer;

		float fX = (fCurrentTime-FirstTimeShown)*PixelsPerSecond+TimelineBarsStartX;
		if(fX >= g_xBarRenderRect.MinX && fX <= g_xBarRenderRect.MaxX)
		{
			Asura_Vector_2 xLineStart,xLineEnd;
			xLineStart.x = fX;
			xLineStart.y = g_xBarRenderRect.MinY;
			xLineEnd.x = fX;
			xLineEnd.y = g_xBarRenderRect.MaxY;

			Asura_Render::Render2DLine(xLineStart, xLineEnd, 0xff7f0000);
		}
	}

	Asura_Render::AllowFullScreenRendering();


	if(DragMode == DragMode_ReplaceSelection || DragMode == DragMode_AddToSelection || DragMode == DragMode_Deselect)
	{
		//draw lines for the selection rectangle
		Asura_Vector_2 start,end;
		end = DragStart;

		start = end;
		end.x = DragCurrent.x;
		Asura_Render::Render2DLine(start,end,0xffffffff);

		start = end;
		end.y = DragCurrent.y;
		Asura_Render::Render2DLine(start,end,0xffffffff);

		start = end;
		end.x = DragStart.x;
		Asura_Render::Render2DLine(start,end,0xffffffff);

		start = end;
		end.y = DragStart.y;
		Asura_Render::Render2DLine(start,end,0xffffffff);
	}

	if(MovingItems && SelectedList.size())
	{
	 	//show the time of the first selected item (to show where it has been moved to)
	 	float x,y;
	 	Asura_Cursor::GetScreenCoords(x,y);
		const Tools_Timeline_Data* pxPrimarySelection = SelectedList.first_entry();

		float fStartTime = pxPrimarySelection->GetStartTime();
		float fEndTime = pxPrimarySelection->GetEndTime();

		if(Tools_Mode_Cutscene::GetTimerMode() == CUTSCENETIMER_SECONDS)
		{
			Asura_Text_Output::Print(x+1,y-1,0xff00ff00,0,"Start %.3f sec",fStartTime);
			if(fEndTime != fStartTime)
			{
				Asura_Text_Output::Print(x+1,y,0xff00ff00,0,"End   %.3f sec",fEndTime);
			}
		}
		else
		{
			fStartTime = Tools_Mode_Cutscene::ConvertSecondsToDisplayedTime(fStartTime);
			fEndTime = Tools_Mode_Cutscene::ConvertSecondsToDisplayedTime(fEndTime);
			Asura_Text_Output::Print(x+1,y-1,0xff00ff00,0,"Start %.0f",fStartTime);
			if(fEndTime != fStartTime)
			{
				Asura_Text_Output::Print(x+1,y,0xff00ff00,0,"End   %.0f",fEndTime);
			}
		}
	}

	// (MH - 21.06.2005) added render of timed messages
	Tools_Timed_Message_System::Render();
}

void Tools_Cutscene_Timeline::SetActor(Tools_CutsceneActor_Object* actorObject)
{
	if(actorObject!=ActorObject)
	{
		ActorObject = actorObject;
		s_bCurrentActorChanged = true;
	}
}

void Tools_Cutscene_Timeline::SetCutscene(Tools_Cutscene_Object* cutsceneObject)
{
	if(cutsceneObject!=CutsceneObject)
	{
		CutsceneObject = cutsceneObject;
		ActorObject = 0;
		s_iPrimarySelectedID = 0;
		Invalidate();
		DeleteTimelineData();
	}
}

void Tools_Cutscene_Timeline::FocusLost()
{
	DragMode = DragMode_None;
	MouseOverItem = 0;
	MovingItems = false;
}

void Tools_Cutscene_Timeline::HandleScrollAndZoom()
{
	//deal with scrolling time line , if help not being displayed (share the same keys)
	if(!Tools_Help::IsDisplayingHelp())
	{
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_LEFT))
		{
			float fScrollAmount = 300.0f*Asura_Timers::GetFrameTime()/PixelsPerSecond;
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) fScrollAmount*=10.0f;

			FirstTimeShown-= fScrollAmount;
			if(FirstTimeShown<0)FirstTimeShown = 0;
			//CalculateScreenCoords();
			
		}
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_RIGHT))
		{
			float fScrollAmount = 300.0f*Asura_Timers::GetFrameTime()/PixelsPerSecond;
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) fScrollAmount*=10.0f;

			FirstTimeShown+=fScrollAmount;
			//CalculateScreenCoords();
		}
		
		
		float& fActorYOffset = FullTimelineScreen ? s_fFullScreenActorYOffset : s_fTimelineBarActorYOffset;
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_UP))
		{
			float fScrollAmount = 120.0f*Asura_Timers::GetFrameTime();
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) fScrollAmount*=10.0f;

			fActorYOffset -= fScrollAmount;
			if(fActorYOffset<0)fActorYOffset=0;
		}
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_DOWN))
		{
			float fScrollAmount = 120.0f*Asura_Timers::GetFrameTime();
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) fScrollAmount*=10.0f;

			fActorYOffset += fScrollAmount;
			
		}
		
		
		//check for zooming in/out
		int zoomDirection = 0;
		if(Asura_UserInput::DebouncedKeyPressed('z'))
		{
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT)) 
				zoomDirection = -1;
			else 
				zoomDirection = 1;
		}
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEWHEELUP))
		{
			zoomDirection = 1;
		}
		else if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEWHEELDOWN))
		{
			zoomDirection = -1;
		}
		
		
		if(zoomDirection)
		{
			int mx,my;
			Asura_Cursor::GetScreenSpaceCoords(mx,my);
			
			//get the time under the mouse
			float time = ((mx-TimelineBarsStartX)/PixelsPerSecond)+FirstTimeShown;
			
			//zoom in/out
			if(zoomDirection<0)
			{
				PixelsPerSecond/=2;
				if(PixelsPerSecond< 50/16.0f) PixelsPerSecond = 50/16.0f;
			}
			else
			{
				PixelsPerSecond*=2;
				if(PixelsPerSecond> 800) PixelsPerSecond = 800;
			}
			if(mx>TimelineBarsStartX)
			{
				//Alter the first time shown so that the time under the cursor is unchanged from before
				float fNewTimeUnderCursor = ((mx-TimelineBarsStartX)/PixelsPerSecond)+FirstTimeShown;
				FirstTimeShown += time - fNewTimeUnderCursor;
				if(FirstTimeShown<0) FirstTimeShown = 0;
			}

			CalculateScreenCoords();
		}
	}
}


void Tools_Cutscene_Timeline::UpdateForOverlay(float fBottomY,float fWidth,int focusFlags)
{
	if(FullTimelineScreen) return;//If already doing full timeline screen , skip this
	
	if(!CutsceneObject) return;
	FullTimelineScreen = false;

	g_fTimelineBarBottomY = fBottomY;
	g_fTimelineBarTopY = fBottomY - g_fTimelineBarHeight;
	g_fTimelineBarWidth = fWidth;

	Update(focusFlags);

	float width = (float)Asura_Viewport::GetWidth()-GetTimelineBarMinX();
	//update minimum/maximum times visible
	MinVisibleTime = FirstTimeShown;
	MaxVisibleTime = FirstTimeShown+(width/PixelsPerSecond);

}

void Tools_Cutscene_Timeline::RenderForOverlay(float fBottomY,float fWidth)
{
	if(FullTimelineScreen) return;//If already doing full timeline screen , skip this
	
	if(!CutsceneObject) return;
	FullTimelineScreen = false;
	g_fTimelineBarBottomY = fBottomY;
	g_fTimelineBarTopY = fBottomY - g_fTimelineBarHeight;
	g_fTimelineBarWidth = fWidth;

	Render();

	float width = (float)Asura_Viewport::GetWidth()-GetTimelineBarMinX();
	//update minimum/maximum times visible
	MinVisibleTime = FirstTimeShown;
	MaxVisibleTime = FirstTimeShown+(width/PixelsPerSecond);


}

void Tools_Cutscene_Timeline::GenerateTimelineData()
{
	DeleteTimelineData();
//	TimelineBarsStartX = FullTimelineScreen ? 100.0f : 0; 
//	StartY = startY;


	//generate timeline actors for each actor in the cutscene
	for(LIF<Tools_CutsceneActor_Object*> alif(&CutsceneObject->ActorObjects);!alif.done();alif.next())
	{
		u_int uSortKey = alif()->Actor->m_uToolsSortOrder;
		bool bAdded = false;
		//Sort based on the tools sort order
		for(LIB<Tools_Timeline_Actor*> xTimelineActorLIB(&ActorList); !xTimelineActorLIB.done(); xTimelineActorLIB.next())
		{
			if(uSortKey >= xTimelineActorLIB()->ActorObject->Actor->m_uToolsSortOrder)
			{
				ActorList.add_entry_after(new Tools_Timeline_Actor(alif()) ,xTimelineActorLIB());
				bAdded = true;
				break;

			}
		}
		if(!bAdded)
		{
			ActorList.add_entry_start(new Tools_Timeline_Actor(alif()));
		}
	}
	//and 1 for things that don't affect actors
	ActorList.add_entry(new Tools_Timeline_Actor(0));


	CutsceneLength = CutsceneObject->Cutscene->CutsceneEndTime;
	
	//generate timeline data for each position
	for(LIF<Tools_CutsceneActor_Object*> alif(&CutsceneObject->ActorObjects);!alif.done();alif.next())
	{
		
		for(LIF<Tools_Cutscene_Track*> tlif(&alif()->Tracks);!tlif.done();tlif.next())
		{
			for(LIF<Tools_CutsceneTrackPoint_Object*> plif(&tlif()->m_xPointObjects);!plif.done();plif.next())
			{
				if( !plif()->AddToTimeline() ) continue;

				Tools_Timeline_Data* data = new Tools_Timeline_Data_Position(plif());
				AddTimelineData(alif(),data,TTC_Position);
				

				CutsceneLength = max(data->Time,CutsceneLength);
			}
		}
	
		//generate an initial anim timeline data thingy if needed
		//if(FullTimelineScreen)
//		{
//			if(alif()->IsHierarchyActor())
//			{
//				Tools_Timeline_Data* data = new Tools_Timeline_Data_Initial_Anim(alif());
//				AddTimelineData(alif(),data,TTC_Animation);
//			}
//		}
		
	}

	
	//and for events
	//if(FullTimelineScreen)
	{
		for(LIF<Tools_Cutscene_Event*> elif(&CutsceneObject->EventList);!elif.done();elif.next())
		{
			//if(!FullTimelineScreen)
			//{
			//	//skip events that don't apply to the current actor
			//	if(!ActorObject) continue;
			//	if(ActorObject->Actor != elif()->Event->m_pxTargetActor) continue;
			//}
			
			Tools_Timeline_Data* data = new Tools_Timeline_Data_Event(elif());
			switch(elif()->Event->m_iType)
			{
				case ACET_AnimationChange :
					AddTimelineData(elif()->Event->m_pxTargetActor,data,TTC_Animation);
					break;
				case ACET_SecondaryAnimation :
					AddTimelineData(elif()->Event->m_pxTargetActor,data,TTC_AnimationSecondary);
					break;
				default :
					AddTimelineData(elif()->Event->m_pxTargetActor,data,elif()->Event->m_iType);
					break;
			}
			CutsceneLength = max(data->Time+data->Duration,CutsceneLength);
		}

		//sort out durations for animation events
		for(LIF<Tools_Timeline_Actor*> alif(&ActorList);!alif.done();alif.next())
		{
			Tools_Timeline_Category* pxCat = alif()->GetCategory(TTC_Animation);
			if(!pxCat || !pxCat->m_xTimelineData.size()) continue;
			

			LIF<Tools_Timeline_Data*> tlif(&pxCat->m_xTimelineData);
			Tools_Timeline_Data* previous = tlif();
			for(tlif.next();!tlif.done();tlif.next())
			{
				float timeDiff = tlif()->Time - previous->Time;
				if(previous->Duration>0)
				{
					if(previous->Duration > timeDiff)
					{
						previous->Duration = min(previous->Duration,timeDiff);
						previous->m_bCanSnapEnd = false;
					}
				}
				else
				{
					//no duration set , so just lasts until next anim change
					previous->Duration = timeDiff;
						previous->m_bCanSnapEnd = false;

				}
				previous = tlif();
			}
			//do the last one
			float timeDiff = CutsceneLength - previous->Time;
			if(previous->Duration>0)
			{
				if(previous->Duration > timeDiff)
				{
					previous->Duration = min(previous->Duration,timeDiff);
					previous->m_bCanSnapEnd = false;
				}
			}
			else
			{
				//no duration set , so just lasts until next anim change
				previous->Duration = timeDiff;
				previous->m_bCanSnapEnd = false;

			}
		}
	}

	CalculateScreenCoords();
	Valid = true;


	//if there was a primary selection , put it back at the start of the selection list
	if(s_iPrimarySelectedID)
	{
		Tools_Timeline_Data* pxItem = GetItemByID(s_iPrimarySelectedID);
		if(pxItem && pxItem->GetSelectionFlags())
		{
			SelectItem(pxItem,pxItem->GetSelectionFlags(),true);
		}
	}

}

void Tools_Cutscene_Timeline::DeleteTimelineData()
{
	while(ActorList.size())
	{
		delete ActorList.first_entry();
		ActorList.delete_first_entry();
	}

	while(SelectedList.size())
	{
		SelectedList.delete_first_entry();
	}

	Valid = false;
}


void Tools_Cutscene_Timeline::AddTimelineData(Tools_CutsceneActor_Object* actor,Tools_Timeline_Data * data,int iCategory)
{
	//locate the timeline actor
	for(LIF<Tools_Timeline_Actor*> alif(&ActorList);!alif.done();alif.next())
	{
		if(alif()->ActorObject == actor)
		{
			alif()->AddTimelineData(data,iCategory);
			return;
		}
	}
	Asura_Assert(0,"Failed to find actor in list");
}

void Tools_Cutscene_Timeline::AddTimelineData(Asura_Cutscene_Actor* actor,Tools_Timeline_Data * data,int iCategory)
{
	//locate the timeline actor
	for(LIF<Tools_Timeline_Actor*> alif(&ActorList);!alif.done();alif.next())
	{
		if(alif()->ActorObject)
		{
			if(alif()->ActorObject->Actor == actor)
			{
				alif()->AddTimelineData(data,iCategory);
				return;
			}
		}
		else if(!actor)
		{
			alif()->AddTimelineData(data,iCategory);
			return;
		}
	}
	Asura_Assert(0,"Failed to find actor in list");
}

void Tools_Cutscene_Timeline::CalculateScreenCoords()
{
	if(!ActorList.size()) return;
	
	
	float y = 0.0f;
	for(LIF<Tools_Timeline_Actor*> alif(&ActorList);!alif.done();alif.next())
	{
		alif()->DetermineScreenCoords(y);
	}
	g_fTimelineDataMaxY = y;
}


Tools_Timeline_Data* Tools_Cutscene_Timeline::GetItemAtCoord(int mx,int my, u_int* puSelectionFlags,bool ignoreSelected)
{
	if(puSelectionFlags) *puSelectionFlags = 0;

	Asura_Vector_2 xMousePos(float(mx),(float)my);
	if(!g_xBarRenderRect.Contains(xMousePos))
	{
		return 0;
	}

	Tools_Timeline_Data* retval = 0;
	for(LIF<Tools_Timeline_Actor*> alif(&ActorList);!alif.done();alif.next())
	{
		retval = alif()->GetItemAtCoord(mx,my,puSelectionFlags,ignoreSelected);
		if(retval) break;
	}

	return retval;
}

Tools_Timeline_Actor* Tools_Cutscene_Timeline::GetActorAtCoord(int mx,int my)
{
	Asura_Vector_2 xMousePos(float(mx),(float)my);
	if(!g_xNameRenderRect.Contains(xMousePos))
	{
		return 0;
	}

	//check each actior to see if the name text is under the mouse
	for(LIF<Tools_Timeline_Actor*> alif(&ActorList);!alif.done();alif.next())
	{
		if(alif()->PointIsInName(mx,my)) return alif();
	}
	return 0;

}

Tools_Timeline_Data* Tools_Cutscene_Timeline::GetItemByID(int iID)
{
	if(!iID) return 0;
	for(LIF<Tools_Timeline_Actor*> alif(&ActorList);!alif.done();alif.next())
	{
		Tools_Timeline_Data* pxItem = alif()->GetItemByID(iID);
		if(pxItem) return pxItem;
	}
	return 0;
}

void Tools_Cutscene_Timeline::SelectItem(Tools_Timeline_Data* pxItem, u_int uSelectionFlags, bool bSelect)
{
	if(bSelect && pxItem->Selectable() && !pxItem->IsHidden())
	{
		//add/move this pxItem to the start of the list 
		if(pxItem->IsSelected())
		{
			SelectedList.delete_entry(pxItem);
		}
		if(uSelectionFlags & TOOLS_TIMELINE_SELECTED_START) pxItem->SelectStart(true);
		if(uSelectionFlags & TOOLS_TIMELINE_SELECTED_END) pxItem->SelectEnd(true);
		SelectedList.add_entry_start(pxItem);
	}
	else
	{
		if(pxItem->IsSelected())
		{
			if(uSelectionFlags & TOOLS_TIMELINE_SELECTED_START) pxItem->SelectStart(false);
			if(uSelectionFlags & TOOLS_TIMELINE_SELECTED_END) pxItem->SelectEnd(false);
			if(!pxItem->IsSelected())
			{
				SelectedList.delete_entry(pxItem);
			}
		}
	}

	if(SelectedList.size())
	{
		s_iPrimarySelectedID = SelectedList.first_entry()->GetID();
	}
	else
	{
		s_iPrimarySelectedID = 0;
	}
}


void Tools_Cutscene_Timeline::MoveSelected(float fDeltaTime)
{
	if(!SelectedList.size()) return;

	if(fDeltaTime<0.0f)
	{
		//get the lowest time , to make sure it won't become negative
		float fLowestTime = SelectedList.first_entry()->Time;
		for(LIF<Tools_Timeline_Data*> slif(&SelectedList);!slif.done();slif.next())
		{
			Tools_Timeline_Data* pxItem = slif();
			if(pxItem->IsStartSelected())
			{
				if(pxItem->GetStartTime()<fLowestTime) fLowestTime = pxItem->GetStartTime();
			}
			if(pxItem->IsEndSelected())
			{
				if(pxItem->GetEndTime()<fLowestTime) fLowestTime = pxItem->GetEndTime();
			}

		}
		fDeltaTime = max(fDeltaTime,-fLowestTime);
	}
	//Ignore small moves to prevent numerical innaccuracies from causing continual updates
	//while the move isn't moving.
	if(Asura_Maths::Abs(fDeltaTime) < 0.01f) return;
	
   	List<Tools_Cutscene_Event*> alteredEventList;
   	
   	for(LIF<Tools_Timeline_Data*> slif(&SelectedList);!slif.done();slif.next())
	{
		Tools_Timeline_Data* pxItem = slif();

		if(pxItem->IsStartSelected() && pxItem->IsEndSelected())
		{
			pxItem->SetTime(pxItem->Time + fDeltaTime);
		}
		else if(pxItem->IsStartSelected())
		{
			float fNewTime = pxItem->Time + fDeltaTime;
			float fNewDuration = Asura_Maths::Max(0.0f, pxItem->Duration-fDeltaTime);

			pxItem->SetTime(fNewTime);
			pxItem->SetDuration(fNewDuration);
		}
		else if(pxItem->IsEndSelected())
		{
			float fNewDuration = Asura_Maths::Max(0.0f, pxItem->Duration+fDeltaTime);
			pxItem->SetDuration(fNewDuration);
		}
		
		if(slif()->GetEvent())
		{
			alteredEventList.add_entry(slif()->GetEvent());
		}
	}

	if(alteredEventList.size())
	{
		//update times for any children of selected events
		CutsceneObject->UpdateEventsAfterTimeChange(alteredEventList);
	}

	CutsceneObject->SortEvents();
	CutsceneObject->UpdateParentEvents();

	//after doing this we need to resort the items , and adjust animation durations
	//go for the sledgehammer approach of recalculating the whole lot
	Invalidate();

	if(!FullTimelineScreen)
	{
		//If not buried in the fTimeline screen then the cutscene playback will need to be refreshed
		//to show the changes.
		Tools_Mode_Cutscene::RefreshPlayback();
	}
}


void Tools_Cutscene_Timeline::SetParentEvent(Tools_Cutscene_Event* parentEvent)
{
	List<Tools_Cutscene_Event*> disallowedList;
	//get a list of parents/grandparents etc of the MouseOverItem... none of those can
	//be made a child of it
	Tools_Cutscene_Event* event = parentEvent;
	while(event)
	{
		disallowedList.add_entry(event);
		event = GetToolsCutsceneEvent(event->ParentEventID);
	}

	//adjust the parents for all selected events
	for(LIF<Tools_Timeline_Data*> slif(&SelectedList);!slif.done();slif.next())
	{
		event = slif()->GetEvent();
		if(!event) continue;
		if(disallowedList.contains(event))continue;

		event->ParentEventID = parentEvent ? parentEvent->EventID : -1;
		if(CutsceneUndo) CutsceneUndo->Log_Event_Changed(event);
	}

	CutsceneObject->UpdateParentEvents();
}


void Tools_Cutscene_Timeline::EditHiddenActors()
{
	if(!CutsceneObject || !CutsceneObject->ActorObjects.size()) return;
	TimelineHideActor_Overlay xOverlay(CutsceneObject);	
	xOverlay.DoModal();

	DeselectHiddenItems();

	CalculateScreenCoords();
		
}

void Tools_Cutscene_Timeline::EditHiddenEvents()
{
	if(!CutsceneObject) return;
	TimelineHideByType_Overlay xOverlay(CutsceneObject);	
	xOverlay.DoModal();

	DeselectHiddenItems();

	CalculateScreenCoords();
}

bool Tools_Cutscene_Timeline::IsEventTypeHidden(int iType)
{
	#if 1
	return s_xFullScreenHiddenEventTypes.contains(iType);	
	#else
	if(FullTimelineScreen)
	{
		return s_xFullScreenHiddenEventTypes.contains(iType);	
	}
	else
	{
		static bool s_bFirstTime = true;
		if(s_bFirstTime)
		{
			//visible event list should start off with the positions by default
			SetEventTypeHidden(TTC_Position,false);
			s_bFirstTime = false;
		} 
		return !s_xTimelineBarVisibleEventTypes.contains(iType);
	}
	#endif
}

void Tools_Cutscene_Timeline::SetEventTypeHidden(int iType,bool bHide)
{
	#if 1

	if(bHide)
	{
		if(!s_xFullScreenHiddenEventTypes.contains(iType))
		{
			s_xFullScreenHiddenEventTypes.add_entry(iType);
		}
	}
	else
	{
		if(s_xFullScreenHiddenEventTypes.contains(iType))
		{
			s_xFullScreenHiddenEventTypes.delete_entry(iType);
		}
	}
	#else
	//Get the list to use depending on whether this is for the full screen.
	List<int>& xList = FullTimelineScreen ? s_xFullScreenHiddenEventTypes : s_xTimelineBarVisibleEventTypes;
	//The meaning of the list is reversed depending on whether this is the fullscreen.
	bool bAddToList = FullTimelineScreen ? bHide : !bHide;


	if(bAddToList)
	{
		if(!xList.contains(iType))
		{
			xList.add_entry(iType);
		}
	}
	else
	{
		if(xList.contains(iType))
		{
			xList.delete_entry(iType);
		}
	}
	#endif
}

void Tools_Cutscene_Timeline::DeselectHiddenItems()
{
	for(LIF<Tools_Timeline_Data*> slif(&SelectedList);!slif.done();)
	{
		Tools_Timeline_Data* pxItem = slif();
		slif.next();
		if(pxItem->IsHidden())
		{
			SelectItem(pxItem,TOOLS_TIMELINE_SELECTED_ALL,false);
		}
	}
}


void Tools_Cutscene_Timeline::SetTimelineBarHeight(float fHeight)
{
	g_fTimelineBarHeight = fHeight;
	g_fTimelineBarTopY = g_fTimelineBarBottomY - fHeight;
}

float Tools_Cutscene_Timeline::GetTimelineBarHeight()
{
	return g_fTimelineBarHeight;
}

float Tools_Cutscene_Timeline::GetTimelineBarMinX()
{
	return TimelineBarsStartX;
}

void Tools_Cutscene_Timeline::CalculateRenderRectangles()
{
	float fWidth = (float)Asura_Viewport::GetWidth();
	float fHeight = (float)Asura_Viewport::GetHeight();

	//calculate the full timeline bounding rectangle
	g_xFullRenderRect.MinX = 0.0f;
	g_xFullRenderRect.MaxX = fWidth;
	if(FullTimelineScreen)
	{
		g_xFullRenderRect.MinY = 0.0f;
		g_xFullRenderRect.MaxY = fHeight;
	}
	else
	{
		g_xFullRenderRect.MinY = g_fTimelineBarTopY;
		g_xFullRenderRect.MaxY = g_fTimelineBarBottomY;
		g_xFullRenderRect.MaxX = g_fTimelineBarWidth;
	}

	//Lower few lines of this is used for the text at the bottom of the screen
	g_xLowerTextRenderRect = g_xFullRenderRect;
	//2 lines for timeline bar , and 4 lines for full screen
	g_xLowerTextRenderRect.MinY = g_xLowerTextRenderRect.MaxY - 2.0f*ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	if(FullTimelineScreen) g_xLowerTextRenderRect.MinY -= 2.0f*ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	//make sure it doesn't use up the top line of the screen , and that the min doesn't go above the max
	g_xLowerTextRenderRect.MinY = Asura_Maths::Max(g_xLowerTextRenderRect.MinY, g_xFullRenderRect.MinY+ASURA_TEXT_OUTPUT_FONT_HEIGHT);
	g_xLowerTextRenderRect.MinY = Asura_Maths::Min(g_xLowerTextRenderRect.MinY, g_xLowerTextRenderRect.MaxY);

	//Get the subset used by the actor names and bars
	g_xNameRenderRect = g_xFullRenderRect;
	g_xNameRenderRect.MinY += ActorStartY;
	g_xNameRenderRect.MaxY = g_xLowerTextRenderRect.MinY;
	g_xNameRenderRect.MinY = Asura_Maths::Min(g_xNameRenderRect.MinY,g_xNameRenderRect.MaxY);

	//and the subset of that just used by the bars
	g_xBarRenderRect = g_xNameRenderRect;
	g_xBarRenderRect.MinX += TimelineBarsStartX;

	const float& fActorYOffset = FullTimelineScreen ? s_fFullScreenActorYOffset : s_fTimelineBarActorYOffset;

	//get the offset to add onto all the bar positions
	g_xBarOffset.x = g_xBarRenderRect.MinX - (FirstTimeShown*PixelsPerSecond) ;
	g_xBarOffset.y = g_xBarRenderRect.MinY - fActorYOffset;
}

bool Tools_Cutscene_Timeline::GetSnappedTime(float fTime, float& fOutputTime)
{
	float fBestTime = -1.0f;
	float fBestDiff = Asura_Maths::LargeFloat;
		
	//test against the events in each actor
	for(LIF<Tools_Timeline_Actor*> xActorLIF(&ActorList);!xActorLIF.done();xActorLIF.next())
	{
		float fNearTime = xActorLIF()->GetNearestSnapTime(fTime);

		if(fNearTime >= 0.0f)
		{
			float fDiff = Asura_Maths::Abs(fTime-fNearTime);
			if(fDiff < fBestDiff)
			{
				fBestTime = fNearTime;
				fBestDiff = fDiff;
			}
		}
	}
	//and against the current cutscene time
	float fCurrentTime = Tools_Mode_Cutscene::s_pxThis->CutscenePlayback_Timer;
	float fDiff = Asura_Maths::Abs(fCurrentTime - fTime);

	if(fDiff < fBestDiff)
	{
		fBestTime = fCurrentTime;
		fBestDiff = fDiff;
	}

	if(fBestTime < 0.0f)
	{
		fOutputTime = fTime;
		return false;
	}

	//accept the best time if the distance in pixels is low enough
	float fDist = fBestDiff * PixelsPerSecond;

	if(fDist < 10.0f)
	{
		fOutputTime = fBestTime;
		return true;
	}
	else
	{
		fOutputTime = fTime;
		return false;
	}
}

void Tools_Cutscene_Timeline::UpdateDragMovement()
{
	Tools_Timeline_Data* pxPrimarySelection = GetPrimarySelection();
	if(!pxPrimarySelection) return;

	if(!g_bDraggedItemTimeInitialised)
	{
		//First update in the drag.
		//Store the start/end time of the primary selection , so it can
		//be used as the starting point for each frame's update
		g_fDraggedItemStartTime = pxPrimarySelection->GetStartTime();
		g_fDraggedItemEndTime = pxPrimarySelection->GetEndTime();
		g_bDraggedItemTimeInitialised = true;
	}


	//Get the time difference from the start of the drag
	float fDeltaTime = (DragCurrent.x-DragStart.x)/PixelsPerSecond;
	if(Tools_Mode_Cutscene::GetTimerMode()!=CUTSCENETIMER_SECONDS)
	{
		//if showing times in frames , make sure the time used is an integer number of frames.
		fDeltaTime = Tools_Mode_Cutscene::ConvertDisplayedTimeToSeconds(Tools_Mode_Cutscene::ConvertSecondsToDisplayedTime(fDeltaTime));
	}

	//Get the target start and end time for the selected item
	bool bUseStart = pxPrimarySelection->IsStartSelected() ? true : false;
	float fTargetStartTime = g_fDraggedItemStartTime + fDeltaTime;
	float fTargetEndTime = g_fDraggedItemEndTime + fDeltaTime;
	float fBestDiff = Asura_Maths::LargeFloat;
	
	//if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
	if(s_bUseSnap)
	{
		//If moving the start , see if the start has anything to snap to
		if(pxPrimarySelection->IsStartSelected())
		{
			float fUnsnappedStartTime = g_fDraggedItemStartTime + fDeltaTime;
			float fSnappedStartTime;

			if(GetSnappedTime(fUnsnappedStartTime, fSnappedStartTime))
			{
				float fDiff = Asura_Maths::Abs(fSnappedStartTime-fUnsnappedStartTime);
				if(fDiff < fBestDiff)
				{
					fBestDiff = fDiff;
					fTargetStartTime = fSnappedStartTime;
					bUseStart = true;

				}
			}
		}

		//And if moving the end , then see if that has anything to snap to
		if(pxPrimarySelection->IsEndSelected() && pxPrimarySelection->CanSnapEnd())
		{
			float fUnsnappedEndTime = g_fDraggedItemEndTime + fDeltaTime;
			float fSnappedEndTime;

			if(GetSnappedTime(fUnsnappedEndTime, fSnappedEndTime))
			{
				float fDiff = Asura_Maths::Abs(fSnappedEndTime-fUnsnappedEndTime);
				if(fDiff < fBestDiff)
				{
					fBestDiff = fDiff;
					fTargetEndTime = fSnappedEndTime;
					bUseStart = false;

				}
			}
		}
	}

	float fCurrentDelta;
	//Work out the delta from the current positions to get the start/end to the desired point
	if(bUseStart)
	{
		fCurrentDelta = fTargetStartTime - pxPrimarySelection->GetStartTime();
	}
	else
	{
		fCurrentDelta = fTargetEndTime - pxPrimarySelection->GetEndTime();
	}
	//move everything 
	MoveSelected(fCurrentDelta);

}

float Tools_Cutscene_Timeline::GetMaxTimelineDataY()
{
	return g_fTimelineDataMaxY;
}

///////////////////////////////////////////////////////////////////////////////
/*----------------------**
** Tools_Timeline_Actor **
**----------------------*/

Tools_Timeline_Actor::Tools_Timeline_Actor(Tools_CutsceneActor_Object* actorObject)
{
	ActorObject = actorObject;
}

Tools_Timeline_Actor::~Tools_Timeline_Actor()
{
	while(m_xCategoryList.size())
	{
		delete m_xCategoryList.first_entry();
		m_xCategoryList.delete_first_entry();
	}

}

void Tools_Timeline_Actor::AddTimelineData(Tools_Timeline_Data * pxData,int iCategory)
{
	//find/create the category
	Tools_Timeline_Category* pxCat = GetCategory(iCategory,true);
	Asura_Assert(pxCat,"");

	//and add to it
	pxCat->AddTimelineData(pxData);
	

}

void Tools_Timeline_Actor::DetermineScreenCoords(float& y)
{
	if(IsHidden()) return;

	Y = y;
	y+=16;
	for(LIF<Tools_Timeline_Category*> xCatLIF(&m_xCategoryList);!xCatLIF.done();xCatLIF.next())
	{
		if(xCatLIF()->IsHidden()) continue;//ignore hidden event types
		
		List<Tools_Timeline_Data*>& xTimelineData = xCatLIF()->m_xTimelineData;
		if(!xTimelineData.size()) continue;
		

		float highestXOnLine[MAX_LINES_FOR_TIMELINE_CAT] = {0,};
		int numLinesUsed = 1;
		for(LIF<Tools_Timeline_Data*> tlif(&xTimelineData);!tlif.done();tlif.next())
		{
			tlif()->DetermineScreenCoords(y,numLinesUsed,highestXOnLine);
		}
		y+=16*numLinesUsed;
	}
	y+=4;
}
/*
void Tools_Timeline_Actor::AdjustScreenCoords(const Asura_Vector_2& xOffset)
{
	Y += xOffset.y;
	for(LIF<Tools_Timeline_Category*> xCatLIF(&m_xCategoryList);!xCatLIF.done();xCatLIF.next())
	{
		List<Tools_Timeline_Data*>& xTimelineData = xCatLIF()->m_xTimelineData;
		if(!xTimelineData.size()) continue;
		
		for(LIF<Tools_Timeline_Data*> tlif(&xTimelineData);!tlif.done();tlif.next())
		{
			tlif()->AdjustScreenCoords(xOffset);
		}
	}
}
*/

void Tools_Timeline_Actor::RenderHeadings()
{
	if(IsHidden()) return;

	//Draw a (relatively) bright horizontal line to separate the actors
	Asura_Vector_2 xLineStart,xLineEnd;
	xLineStart.x = g_xNameRenderRect.MinX;
	xLineStart.y = Y+g_xBarOffset.y-2.0f;
	xLineEnd.x = g_xNameRenderRect.MaxX;
	xLineEnd.y = Y+g_xBarOffset.y-2.0f;
	if(xLineStart.y >= g_xBarRenderRect.MinY && xLineEnd.y <= g_xBarRenderRect.MaxY)
	{
		Asura_Render::Render2DLine(xLineStart,xLineEnd,0x7fffffff);
	}


	//show the actor name :
	float print_x = 0;
	float print_y = Y + g_xBarOffset.y;
  	Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
	Asura_Text_Output::Print(print_x,print_y,0xff00ff00,0,"%s :",ActorObject ? ActorObject->GetName() : "Actorless");

	for(LIF<Tools_Timeline_Category*> xCatLIF(&m_xCategoryList);!xCatLIF.done();xCatLIF.next())
	{
		if(xCatLIF()->IsHidden()) continue;//ignore hidden event types
	
		List<Tools_Timeline_Data*>& xTimelineData = xCatLIF()->m_xTimelineData;
		if(!xTimelineData.size()) continue;

		//draw a dim line to separate the events
		xLineStart.y = xTimelineData.first_entry()->Top-2.0f + g_xBarOffset.y;
		xLineEnd.y = xLineStart.y;
		if(xLineStart.y >= g_xBarRenderRect.MinY && xLineEnd.y <= g_xBarRenderRect.MaxY)
		{
			Asura_Render::Render2DLine(xLineStart,xLineEnd,0x1fffffff);
		}
		
		
		float print_y = xTimelineData.first_entry()->Top-1 + g_xBarOffset.y;
	 	Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
		xCatLIF()->RenderHeading(print_x,print_y);
	}
}

void Tools_Timeline_Actor::Render()
{
	if(IsHidden()) return;

	for(LIF<Tools_Timeline_Category*> xCatLIF(&m_xCategoryList);!xCatLIF.done();xCatLIF.next())
	{
		if(xCatLIF()->IsHidden()) continue;//ignore hidden event types
	
		List<Tools_Timeline_Data*>& xTimelineData = xCatLIF()->m_xTimelineData;
		if(!xTimelineData.size()) continue;

		//draw all the timeline rectangles
		for(LIF<Tools_Timeline_Data*> tlif(&xTimelineData);!tlif.done();tlif.next())
		{
			tlif()->Render();
		}
	}
}

Tools_Timeline_Data* Tools_Timeline_Actor::GetItemAtCoord(int mx,int my, u_int* puSelectionFlags,bool ignoreSelected)
{
	if(IsHidden()) return 0;

	float fOffsetMouseX = mx - g_xBarOffset.x;
	float fOffsetMouseY = my - g_xBarOffset.y;

	for(LIF<Tools_Timeline_Category*> xCatLIF(&m_xCategoryList);!xCatLIF.done();xCatLIF.next())
	{
		if(xCatLIF()->IsHidden()) continue;//ignore hidden event types
		
		List<Tools_Timeline_Data*>& xTimelineData = xCatLIF()->m_xTimelineData;
		
		for(LIB<Tools_Timeline_Data*> tlib(&xTimelineData);!tlib.done();tlib.next())
		{
			const Tools_Timeline_Data* pxItem = tlib();
			
			if(ignoreSelected && pxItem->IsSelected()) continue;

			if(fOffsetMouseX>=pxItem->Left && fOffsetMouseX<pxItem->Right && fOffsetMouseY>=pxItem->Top && fOffsetMouseY< pxItem->Bottom)
			{
				if(puSelectionFlags)
				{
					//see if this close to the start or end
					float fOffset = fOffsetMouseX - pxItem->Left;
					float fWidth = pxItem->Right-pxItem->Left;

					float fStartEndSize = Asura_Maths::Min(fWidth*0.25f, 8.0f);

					if(fOffset < fStartEndSize)
					{
						*puSelectionFlags = TOOLS_TIMELINE_SELECTED_START;
					}
					else if(fOffset > fWidth-fStartEndSize)
					{
						*puSelectionFlags = TOOLS_TIMELINE_SELECTED_END;
					}
					else
					{
						*puSelectionFlags = TOOLS_TIMELINE_SELECTED_START|TOOLS_TIMELINE_SELECTED_END;
					}
				}
				
				return tlib();
			}
		}
	}
	return 0;
}

Tools_Timeline_Data* Tools_Timeline_Actor::GetItemByID(int iID)
{

	for(LIF<Tools_Timeline_Category*> xCatLIF(&m_xCategoryList);!xCatLIF.done();xCatLIF.next())
	{
		
		List<Tools_Timeline_Data*>& xTimelineData = xCatLIF()->m_xTimelineData;
		
		for(LIF<Tools_Timeline_Data*> tlif(&xTimelineData);!tlif.done();tlif.next())
		{
			if(tlif()->GetID() == iID) return tlif();			
		}
	}
	return 0;
}

bool Tools_Timeline_Actor::PointIsInName(int _mx,int _my)
{
	if(IsHidden()) return false;

	if(ActorObject)
	{
		float name_x = 0;
		float name_y = Y;
		float mx = (float) _mx;
		float my = (float) _my;
	  	Asura_Text_Output::ConvertPixelsToPrintCoords(name_x,name_y);
	  	Asura_Text_Output::ConvertPixelsToPrintCoords(mx,my);

		if(my<name_y || my>=name_y+1) return false;
		if(mx>= strlen(ActorObject->GetName())) return false;
		return true;
	}
	return false;
}


void Tools_Timeline_Actor::SelectInRect(Asura_Vector_2 p1,Asura_Vector_2 p2,bool select)
{
	if(IsHidden()) return;

	//make sure the second point is the bottom-right of the rect
	if(p1.x>p2.x)
	{
		float temp = p1.x;
		p1.x = p2.x;
		p2.x = temp;
	}
	if(p1.y>p2.y)
	{
		float temp = p1.y;
		p1.y = p2.y;
		p2.y = temp;
	}

	for(LIF<Tools_Timeline_Category*> xCatLIF(&m_xCategoryList);!xCatLIF.done();xCatLIF.next())
	{
		if(xCatLIF()->IsHidden()) continue;//ignore hidden event types
		
		List<Tools_Timeline_Data*>& xTimelineData = xCatLIF()->m_xTimelineData;
		for(LIB<Tools_Timeline_Data*> tlib(&xTimelineData);!tlib.done();tlib.next())
		{
			Asura_Bounding_Rectangle xDataRect;
			xDataRect.MinX = tlib()->Left + g_xBarOffset.x;
			xDataRect.MaxX = tlib()->Right + g_xBarOffset.x;
			xDataRect.MinY = tlib()->Top + g_xBarOffset.y;
			xDataRect.MaxY = tlib()->Bottom + g_xBarOffset.y;
			
			if((p1.y<=xDataRect.MinY && p2.y>=xDataRect.MaxY))
			{
				if(tlib()->CanEditDuration())
				{
					if(p1.x<=xDataRect.MinX && p2.x>=xDataRect.MinX)
					{
						Tools_Cutscene_Timeline::SelectItem(tlib(),TOOLS_TIMELINE_SELECTED_START,select);
					}
					if(p1.x<=xDataRect.MaxX && p2.x>=xDataRect.MaxX)
					{
						Tools_Cutscene_Timeline::SelectItem(tlib(),TOOLS_TIMELINE_SELECTED_END,select);
					}
					
				}
				else
				{
					if(p1.x<=xDataRect.MinX && p2.x>=xDataRect.MinX)
					{
						Tools_Cutscene_Timeline::SelectItem(tlib(),TOOLS_TIMELINE_SELECTED_START,select);
					}
				}
				//else
				//{
				//	//ignore items that are only partially displayed
				//	if(!g_xBarRenderRect.Contains(xDataRect.GetTopLeft())) continue;
				//	if(!g_xBarRenderRect.Contains(xDataRect.GetBottomRight())) continue;
				//
				//	if(p1.x<=xDataRect.MinX && p2.x>=xDataRect.MaxX)
				// 	{
				//		Tools_Cutscene_Timeline::SelectItem(tlib(),TOOLS_TIMELINE_SELECTED_ALL,select);
				//	}
				//}
				
			}
		}
	}
	
}

void Tools_Timeline_Actor::Edit()
{
	if(ActorObject)
	{
		ActorObject->Rename();
	}
}

Tools_Timeline_Category* Tools_Timeline_Actor::GetCategory(int iCategory,bool bCreate /*= false*/)
{
	//search for an existing matching category
	LIF<Tools_Timeline_Category*> xCatLIF(&m_xCategoryList);
	for(;!xCatLIF.done();xCatLIF.next())
	{
		if(xCatLIF()->m_iCategory == iCategory)
		{
			//found it
			return xCatLIF();
		}
		if(xCatLIF()->m_iCategory > iCategory)
		{
			//the categories are sorted , so since we have gone past it , its not there
			break;
		}
	}
	//create if needed
	if(!bCreate) return 0;

	Tools_Timeline_Category* pxCat = new Tools_Timeline_Category(this,iCategory);
	if(xCatLIF.done())
	{
		m_xCategoryList.add_entry(pxCat);//add to the end
	}
	else
	{
		m_xCategoryList.add_entry_before(pxCat,xCatLIF());
	}

	return pxCat;
}

bool Tools_Timeline_Actor::IsHidden()
{
	return ActorObject ? ActorObject->m_bHiddenInTimeline : false;
}

float Tools_Timeline_Actor::GetNearestSnapTime(float fTime)
{
	//ignore hidden actors
	if(IsHidden()) return -1.0f;

	float fBestTime = -1.0f;
	float fBestDiff = Asura_Maths::LargeFloat;

	for(LIF<Tools_Timeline_Category*> xCatLIF(&m_xCategoryList);!xCatLIF.done();xCatLIF.next())
	{
		if(xCatLIF()->IsHidden()) continue;//ignore hidden event types
		
		
		for(LIF<Tools_Timeline_Data*> xDataLIF(&xCatLIF()->m_xTimelineData);!xDataLIF.done();xDataLIF.next())
		{
			const Tools_Timeline_Data* pxItem = xDataLIF();
			
			//test against the start if the start isn't selected
			if(!pxItem->IsStartSelected())
			{
				float fTestTime = pxItem->GetStartTime();
				float fDiff = Asura_Maths::Abs(fTestTime-fTime);

				if(fDiff<fBestDiff)
				{
					fBestDiff = fDiff;
					fBestTime = fTestTime;
				}
			}
			//and the end if the end isn't selected
			if(!pxItem->IsEndSelected())
			{
				float fTestTime = pxItem->GetEndTime();
				float fDiff = Asura_Maths::Abs(fTestTime-fTime);

				if(fDiff<fBestDiff)
				{
					fBestDiff = fDiff;
					fBestTime = fTestTime;
				}
			}
		}
	}
	return fBestTime;	
}

///////////////////////////////////////////////////////////////////////////////
/*-------------------------**
** Tools_Timeline_Category **
**-------------------------*/
Tools_Timeline_Category::Tools_Timeline_Category(Tools_Timeline_Actor* pxTimelineActor,int iCategory)
{
	m_pxTimelineActor = pxTimelineActor;
	m_iCategory = iCategory;
}

Tools_Timeline_Category::~Tools_Timeline_Category()
{
	while(m_xTimelineData.size())
	{
		delete m_xTimelineData.first_entry();
		m_xTimelineData.delete_first_entry();
	}
}

void Tools_Timeline_Category::AddTimelineData(Tools_Timeline_Data * pxData)
{
	LIF<Tools_Timeline_Data*> tlif(&m_xTimelineData);
	for(;!tlif.done();tlif.next())
	{
		if(pxData->Time<tlif()->Time)
		{
			m_xTimelineData.add_entry_before(pxData,tlif());

			if(m_iCategory == TTC_Position)
			{
				//some earlier points in the track have a later time than this one
		   		pxData->IsOkay = false;
			}
			return;
		}
		if(m_iCategory == TTC_Position)
		{
			//if the time on this point is less than the departure time of an earlier point  , then
			//makr it as being not okay
			if(pxData->Time<tlif()->Time+tlif()->Duration)
			{
		   		pxData->IsOkay = false;
			}
		}

	}
	m_xTimelineData.add_entry(pxData);	

	pxData->m_pxTimelineCategory = this;

}

void Tools_Timeline_Category::RenderHeading(float fPrintX,float fPrintY)
{
	char szHeading[7] = "";

	switch(m_iCategory)
	{
		case TTC_Position :
			strcpy(szHeading,"Pos");
			break;
		case TTC_Animation :
			strcpy(szHeading,"Anim");
			break;
		case TTC_AnimationSecondary :
			break;//no heading done for this

		case TTC_Misc :
			strcpy(szHeading,"Misc");
			break;

		default :
			//should be an event specific category
			if(m_xTimelineData.size())
			{
				Tools_Cutscene_Event* pxEvent = m_xTimelineData.first_entry()->GetEvent();
				if(pxEvent)
				{
					const char* pszDesc = pxEvent->GetAbbreviatedTypeDescription();
					if(pszDesc)
					{
						Asura_PrintfToBfr(szHeading,"%s",7,true,pszDesc);
					}
				}
			}
			break;

	}

	if(!szHeading[0]) return;

	Asura_Text_Output::Print(fPrintX,fPrintY,0xff0000ff,0,"%s :",szHeading);
}

bool Tools_Timeline_Category::IsHidden()
{
	//if(!FullTimelineScreen) return false;
	if(m_pxTimelineActor->IsHidden()) return true;
	return Tools_Cutscene_Timeline::IsEventTypeHidden(m_iCategory);
}


///////////////////////////////////////////////////////////////////////////////
/*---------------------**
** Tools_Timeline_Data **
**---------------------*/
Tools_Timeline_Data::Tools_Timeline_Data()
{
	Time = 0;
	Duration = 0;
	m_bCanSnapEnd = true;
	IsOkay = true;
	m_pxTimelineCategory = 0;
}

Tools_Timeline_Data::~Tools_Timeline_Data()
{
	if(this == MouseOverItem) MouseOverItem = NULL;
}

void Tools_Timeline_Data::DetermineScreenCoords(float y,int& numLinesUsed,float highestXOnLine[MAX_LINES_FOR_TIMELINE_CAT])
{
	//float screenWidth = (float)Asura_Viewport::GetWidth();
	//float screenHeight = (float)Asura_Viewport::GetHeight();


	Left = Time * PixelsPerSecond;

	if(Duration)
	{
		Right = (Duration*PixelsPerSecond) + Left;
	}
	else
	{
		Right = Left+14;
	}


	{
		//try to place this item so as to avoid overlaps with previous ones , by moving down lines
		//as needed.
		int line;
		int bestLine=0;
		float lowestOverlap = 100000;
//		int maxLines = FullTimelineScreen ? MAX_LINES_FOR_TIMELINE_CAT : 1;
		int maxLines = MAX_LINES_FOR_TIMELINE_CAT;
		for(line = 0;line<maxLines;line++)
		{
			float overlap = highestXOnLine[line]-Left;
			if(overlap<=2)
			{
				bestLine = line;
				break;
			}
			if(overlap<lowestOverlap)
			{
				lowestOverlap = overlap;
				bestLine = line;
			}
		}
		highestXOnLine[bestLine] = Right;
		y+=16*bestLine;
		if(bestLine>=numLinesUsed)numLinesUsed = bestLine+1;
	}
	
	Top = y+1;
	Bottom = y+15;
	
}

/*
void Tools_Timeline_Data::AdjustScreenCoords(const Asura_Vector_2& xOffset)
{
	Left += xOffset.x;
	Right += xOffset.x;
	Top += xOffset.y;
	Bottom += xOffset.y;
}
*/

void Tools_Timeline_Data::Render()
{
	if(Left>=Right) return;

	Asura_Bounding_Rectangle xRect;
	xRect.MinX = Left + g_xBarOffset.x;
	xRect.MaxX = Right + g_xBarOffset.x;
	xRect.MinY = Top + g_xBarOffset.y;
	xRect.MaxY = Bottom + g_xBarOffset.y;

	if(!xRect.Intersects(g_xBarRenderRect))
	{
		return;
	}


	bool isParent = false;
	bool isChild = false;
	if(MouseOverItem && MouseOverItem->GetEvent() && GetEvent())
	{
		if(GetEvent()->ParentEventID == MouseOverItem->GetEvent()->EventID)
		{
			isChild = true;
		}
		else if(MouseOverItem->GetEvent()->ParentEventID == GetEvent()->EventID)
		{
			isParent = true;
		}
	}

	u_int uSelectColour,uNonSelectColour;
	if(this == Tools_Cutscene_Timeline::GetPrimarySelection())
		uSelectColour=0xffff0000;
	else
		uSelectColour=0xffff9090;

  	if(Selectable())
		uNonSelectColour=0xffffffff;
	else
		uNonSelectColour=0xff000000;
	
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	
	float fMidPoint = (float)Asura_Maths::FloatToInt((xRect.MaxX+xRect.MinX)/2.0f);
	//render the first half of the item background
	Asura_Render::Overlay.SetRectXY(xRect.MinX, xRect.MinY, fMidPoint-xRect.MinX, xRect.GetHeight());
	Asura_Render::Overlay.Colour = (GetSelectionFlags() & TOOLS_TIMELINE_SELECTED_START) ? uSelectColour : uNonSelectColour;
	Asura_Render::Overlay.Render();

	//render the second half of the item background
	Asura_Render::Overlay.SetRectXY(fMidPoint, xRect.MinY, xRect.MaxX-fMidPoint, xRect.GetHeight());
	Asura_Render::Overlay.Colour = (GetSelectionFlags() & TOOLS_TIMELINE_SELECTED_END) ? uSelectColour : uNonSelectColour;
	Asura_Render::Overlay.Render();
	
	
	if(IsSelected())
	{
		if(this == Tools_Cutscene_Timeline::SelectedList.first_entry())
			Asura_Render::Overlay.Colour=0xffff0000;
		else
			Asura_Render::Overlay.Colour=0xffff9090;
	}
  	else if(Selectable())
	{
		Asura_Render::Overlay.Colour=0xffffffff;
	}
	else
	{
		Asura_Render::Overlay.Colour=0xff000000;
	}
	if(xRect.GetWidth() > 4)
	{
		Asura_Render::Overlay.SetRectXY(xRect.MinX+2, xRect.MinY+2, xRect.GetWidth()-4, xRect.GetHeight()-4);
		if(this == MouseOverItem)
		{
			Asura_Render::Overlay.Colour=0xff00ff00;
		}
		else if(isParent)
		{
			Asura_Render::Overlay.Colour=0xffffff00;
		}
		else if(isChild)
		{
			Asura_Render::Overlay.Colour=0xff00ffff;
		}
		else if(!IsOkay)
		{
			Asura_Render::Overlay.Colour = 0xff7f0000;
		}
		else
		{
			Asura_Render::Overlay.Colour = GetColour();
		}
		Asura_Render::Overlay.Render();
	}

}

void Tools_Timeline_Data::SelectStart(bool bSelect)
{
	u_int uSelectionFlags = GetSelectionFlags();
	u_int uFlagsToChange = TOOLS_TIMELINE_SELECTED_START;
	//If the duration can't be edited then both start and end are selected/deselected at the same time
	if(!CanEditDuration()) uFlagsToChange |= TOOLS_TIMELINE_SELECTED_END;

	if(bSelect)
	{
		uSelectionFlags |= uFlagsToChange;
	}				
	else
	{
		uSelectionFlags &= ~uFlagsToChange;
	}

	SetSelectionFlags(uSelectionFlags);
}

void Tools_Timeline_Data::SelectEnd(bool bSelect)
{
	u_int uSelectionFlags = GetSelectionFlags();
	u_int uFlagsToChange = TOOLS_TIMELINE_SELECTED_END;
	//If the duration can't be edited then both start and end are selected/deselected at the same time
	if(!CanEditDuration()) uFlagsToChange |= TOOLS_TIMELINE_SELECTED_START;

	if(bSelect)
	{
		uSelectionFlags |= uFlagsToChange;
	}				
	else
	{
		uSelectionFlags &= ~uFlagsToChange;
	}

	SetSelectionFlags(uSelectionFlags);
}



///////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Tools_Timeline_Data_Position **
**------------------------------*/

Tools_Timeline_Data_Position::Tools_Timeline_Data_Position(Tools_CutsceneTrackPoint_Object* pointObject)
{
	PointObject = pointObject;
	Time = PointObject->Time;
	Duration = PointObject->Duration;
	if(IsSelected())
	{
		Tools_Cutscene_Timeline::SelectedList.add_entry(this);
	}
}

int Tools_Timeline_Data_Position::GetColour()
{
	return 0xff0000ff;
}

void Tools_Timeline_Data_Position::PrintDescription(float x,float y) const
{
	Asura_Text_Output::Print(x,y,0xffffffff,0,"%s",PointObject->GetName());
}

void Tools_Timeline_Data_Position::SetTime(float time)
{
	Time = time;
	PointObject->SetTime(time);
	PointObject->Track->m_bAsuraTrackInvalid = true;

	Tools_SaveAsura::SetCutscenesModified();
}

void Tools_Timeline_Data_Position::SetDuration(float fDuration)
{
	Duration = fDuration;
	PointObject->SetDuration(fDuration);
	PointObject->Track->m_bAsuraTrackInvalid = true;

	Tools_SaveAsura::SetCutscenesModified();
}


void Tools_Timeline_Data_Position::SetSelectionFlags(u_int uSelectionFlags)
{
	PointObject->m_uTimelineSelectionFlags = uSelectionFlags;
}

u_int Tools_Timeline_Data_Position::GetSelectionFlags() const
{
	return PointObject->m_uTimelineSelectionFlags;
}

bool Tools_Timeline_Data_Position::Edit()
{
	Tools_Object_System::GotoAndSelect(PointObject)	;
	return true;
}

///////////////////////////////////////////////////////////////////////////////
/*--------------------------**
** Tools_Timeline_Data_Event**
**--------------------------*/
Tools_Timeline_Data_Event::Tools_Timeline_Data_Event(Tools_Cutscene_Event* event)
{
	Event = event;
	Event->FillInTimelineData(this);

	if(GetSelectionFlags())
	{
		if (Selectable())
		{
			Tools_Cutscene_Timeline::SelectedList.add_entry(this);
		}
		else
		{
			SetSelectionFlags(0);
		}
	}
}

Tools_Timeline_Data_Event::~Tools_Timeline_Data_Event()
{
	//Vaguely possible that the event may have been deleted by this point ,so not using IsSelected()
	//to determine whether this will be in the list.
	if(Tools_Cutscene_Timeline::SelectedList.contains(this))
	{
		Tools_Cutscene_Timeline::SelectedList.delete_entry(this);
	}
}

int Tools_Timeline_Data_Event::GetColour()
{
	return Event->GetTimelineColour();
}

void Tools_Timeline_Data_Event::PrintDescription(float x,float y) const
{
	Event->PrintDescription(x,y);
}

void Tools_Timeline_Data_Event::SetTime(float time)
{
	Time = time;
	Event->SetNewTriggerTime(time);

	if(CutsceneUndo) CutsceneUndo->Log_Event_Changed(Event);

}

bool Tools_Timeline_Data_Event::CanEditDuration() const
{
	return Event->CanEditDuration();
}

void Tools_Timeline_Data_Event::SetDuration(float fDuration)
{
	Duration = fDuration;
	Event->SetTotalDuration(fDuration);

	if(CutsceneUndo) CutsceneUndo->Log_Event_Changed(Event);

}


void Tools_Timeline_Data_Event::SetSelectionFlags(u_int uSelectionFlags)
{
	Event->m_uTimelineSelectionFlags = uSelectionFlags;
}

u_int Tools_Timeline_Data_Event::GetSelectionFlags() const
{
	return Event->m_uTimelineSelectionFlags;
}

bool Tools_Timeline_Data_Event::Selectable()
{
	return Event->IsEditable();
}

bool Tools_Timeline_Data_Event::Edit()
{
	Event->CutsceneObject->SelectEvent(Event);
	return true;
}

///////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_Timeline_Data_Initial_Anim **
**----------------------------------*/
Tools_Timeline_Data_Initial_Anim::Tools_Timeline_Data_Initial_Anim(Tools_CutsceneActor_Object* actorObject)
{
	ActorObject = actorObject;
	Asura_ClientCutscene_Actor_Hierarchy* pHierarchyActor = (Asura_ClientCutscene_Actor_Hierarchy*)actorObject->Actor;

	if(!(pHierarchyActor->m_iAnimationFlags & AsuraAnimationFlag_Loop))
	{
		if(pHierarchyActor->AnimID!=-1)
		{
			//get the length of the animation
        	Asura_String xAnimationName = Tools_Mode_Cutscene::CurrentCutscene->Cutscene->AnimationNames.GetName(pHierarchyActor->AnimID);
			Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnim(xAnimationName.GetString());
			if(pxAnim)
			{
				Duration = pxAnim->Duration;
				Duration += pHierarchyActor->m_fAnimationDelay;
			}
		}
	}

}

int Tools_Timeline_Data_Initial_Anim::GetColour()
{
	return 0xff707000;
}

void Tools_Timeline_Data_Initial_Anim::PrintDescription(float x,float y) const
{
	Asura_Assert(ActorObject->IsHierarchyActor(),"");
	Asura_Cutscene_Actor_Hierarchy* hierarchyActor = (Asura_Cutscene_Actor_Hierarchy*) ActorObject->Actor;

    Asura_String xNullString = "";
    Asura_String& InitialAnimName = (hierarchyActor->AnimID == -1) ? xNullString : Tools_Mode_Cutscene::CurrentCutscene->Cutscene->AnimationNames.GetName(hierarchyActor->AnimID);
    if (InitialAnimName.GetLength())
    {
		//Asura_Text_Output::Print(x,y,0xffffffff,0,"Initial Animation : %s",hierarchyActor->InitialAnimationName.GetString());
        Asura_Text_Output::Print(x,y,0xffffffff,0,"Initial Animation : %s",InitialAnimName.GetString());
	}
	else
	{
		Asura_Text_Output::Print(x,y,0xffffffff,0,"Initial Animation : *Not Set*");
	}
	
}

bool Tools_Timeline_Data_Initial_Anim::Edit()
{
	Tools_Object_System::GotoAndSelect(ActorObject);
	return true;
}


///////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** TimelineHideActor_Overlay **
**---------------------------*/


TimelineHideActor_Overlay::TimelineHideActor_Overlay(Tools_Cutscene_Object* pxCutsceneObject)
:ListSelection_Overlay("Edit Visible Actors")
{
	m_pxCutsceneObject = pxCutsceneObject;
	
	
	for(LIF<Tools_CutsceneActor_Object*> xActorLIF(&m_pxCutsceneObject->ActorObjects);!xActorLIF.done();xActorLIF.next())
	{
		AddItem(xActorLIF()->GetName(),(int)xActorLIF());
		SetItemTicked((int)xActorLIF(),!xActorLIF()->m_bHiddenInTimeline);
	}
}


bool TimelineHideActor_Overlay::DoModal()
{
	if(ListSelection_Overlay::DoModal())
	{
		//apply the changes
		for(int i=0;i<GetNumItems();i++)
		{
			Tools_CutsceneActor_Object* pxActor = (Tools_CutsceneActor_Object*)GetItemData(i);
			pxActor->m_bHiddenInTimeline = !GetItemTicked(i);
		}
		return true;
	}
	return false;
}




//////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** TimelineHideByType_Overlay **
**----------------------------*/

TimelineHideByType_Overlay::TimelineHideByType_Overlay(Tools_Cutscene_Object* pxCutsceneObject)
:ListSelection_Overlay("Edit Event Filter")
{
	m_pxCutsceneObject = pxCutsceneObject;
	
	//add entries for the special categories
	AddItem("Position",TTC_Position);
	SetItemTicked(TTC_Position,!Tools_Cutscene_Timeline::IsEventTypeHidden(TTC_Position));

	AddItem("Animation",TTC_Animation);
	SetItemTicked(TTC_Animation,!Tools_Cutscene_Timeline::IsEventTypeHidden(TTC_Animation));

	AddItem("Secondary Anim",TTC_AnimationSecondary);
	SetItemTicked(TTC_AnimationSecondary,!Tools_Cutscene_Timeline::IsEventTypeHidden(TTC_AnimationSecondary));
	
	//add all the type names to the overlay
	for(CLIF<Tools_Cutscene_Event*> elif(&m_pxCutsceneObject->EventList);!elif.done();elif.next())
	{
		int iType = elif()->Event->m_iType;

		//ignore anim change events
		if(iType == ACET_AnimationChange || iType == ACET_SecondaryAnimation) continue;
		
		//add this event's type to the type list
		AddItem(elif()->GetTypeDescription(),iType,false,NULL,true);
		SetItemTicked(iType,!Tools_Cutscene_Timeline::IsEventTypeHidden(iType));
	}
}


bool TimelineHideByType_Overlay::DoModal()
{
	if(ListSelection_Overlay::DoModal())
	{
		//update the hidden state for all these events
		for(int i=0;i<GetNumItems();i++)
		{
			Tools_Cutscene_Timeline::SetEventTypeHidden(GetItemData(i),!GetItemTicked(i));
	
		}
		return true;
	}
	return false;
}
