#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_CUTSCENE_TRACK_H_
#define _TOOLS_CUTSCENE_TRACK_H_

#include "Objects/Tools_Object.h"
#include "Misc/list_tem.hpp"
#include "Misc/Tools_Modal_Overlay.h"
#include "Tools_Undo.h"

class Asura_Cutscene_Track;
class Tools_CutsceneActor_Object;
class Tools_CutsceneTrackPoint_Object;

//////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------**
** Tools_Cutscene_Track **
**----------------------*/
class Tools_Cutscene_Track
{
	public :
		Tools_Cutscene_Track(Asura_Cutscene_Track* track,Tools_CutsceneActor_Object*  actorObject,int iTrackID);
		virtual ~Tools_Cutscene_Track();

		void UpdatePointIndeces();//sets the index number for each track point
		virtual bool UpdateAsuraTrack();
		virtual Tools_CutsceneTrackPoint_Object* AddPoint(int index,bool bUsingAddBefore,const Asura_Vector_3* pxPos = 0, int iPointID = 0);
		virtual void Destroy();
		virtual void Rename(const char* name = NULL);

		virtual void SetTimeForPoints(int firstIndex,int lastIndex,float finalTime);

		virtual void RenderSpline();

		int GetTrackIndex();

		//re-sorts point list by time
		void SortPointList();

	
		bool m_bAsuraTrackInvalid;
		Asura_Cutscene_Track* m_pxTrack;

		bool m_bBeingDestroyed;
        bool m_bIsActorPositionTrack;// (MH - 05.11.2002) if true, track keeps actor's position track array up-to-date.
		bool m_bSelected;//set to true if a point in the track is selected.

		// (AAM - 21.10.2004) If false, track never sets "cutscenes modified" flag (defaults to true)
		bool m_bSetCutscenesModified;

		int m_iTrackID;//used for undo
		static int s_iNextTrackID;

		unsigned m_uTrackFlagsBackup;

		Tools_CutsceneActor_Object* m_pxActorObject;
		List<Tools_CutsceneTrackPoint_Object*> m_xPointObjects;

	private :
		//render data for spine
		int m_iNumRenderVertices;
		u_short* m_pusRenderIndeces;
		struct Tools_D3DTrackVertex* m_pxRenderVertices;
		u_int m_uRenderFrameStamp;

};

struct Tools_D3DTrackVertex
{
	Asura_Vector_3 xPos;
	u_int uColour;
};

//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_CutsceneTrackPoint_Object **
**---------------------------------*/

class Tools_CutsceneTrackPoint_Object : public Tools_Object
{
	public :
		Tools_CutsceneTrackPoint_Object(const Asura_Vector_3& xPosition,float fTime,int iPointFlags,Tools_Cutscene_Track* pxTrack, int iPointID = 0);
		~Tools_CutsceneTrackPoint_Object();

		//virtual function overides
		virtual void Render();
		void UpdatePositionInAsuraData(bool updateLinkedObjects /*=true*/);
		virtual const char* GetName();
		const char* GetTypeDescription(){return "Track Point";}
		void Destroy();
		void OnSelection();
		void OnDeselection();
		void UpdateWhileSelected();
		void AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext);
		void AlignWithTrack();
		virtual bool AddToTimeline() { return true; }
		virtual void SetTime( float fTime ) { Time = fTime; }
		virtual void SetDuration( float fDuration ) { Duration = fDuration; }

		static void ToggleOverlay(void);
		static bool OverlayIsOn(void);

		static bool TrackHasSmoothSpeed();//for an IsEnabled function
		
		virtual void CreateOverlays();
		virtual void DeleteOverlays();

		virtual void CreatePanelOverlay();
		virtual void CreateMainOverlay();
		
		static void AddPointAfter();
		static void AddPointBefore();
		static void DuplicatePointAfter();
		static void DuplicatePointBefore();
		static void AdjustTimes();
		static void RenameTrack();

		void UpdateBackupVariables();
		bool BackupVariablesAreDifferent();
		const char* GetNameOfChangedVariable() const;//gets the name of one of the variables that differ from the backups
		void UpdateOverlayControls();

		bool IsFirstPoint(){return this == Track->m_xPointObjects.first_entry();}
		bool IsLastPoint(){return this == Track->m_xPointObjects.last_entry();}


		void ListObjectsForTrack(List<Tools_Object*>& xObjectList);
		static void ContextMenu_SelectTrack(void* pObject, u_int uMode);
		static void ContextMenu_SelectTrackAfterPoint(void* pObject, u_int uMode);
		static void ContextMenu_SelectTrackBeforePoint(void* pObject, u_int uMode);

	protected :
		virtual bool GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const;
		virtual bool SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam);

	public :
		
		int Index;
		Tools_Cutscene_Track* Track;
		float Time;
		float Duration;
		bool SectionStraight;
		bool m_bStationaryFlag;
		u_int m_uTimelineSelectionFlags;

		int m_iSelectedIndex;
		bool m_bSelectedIndexChanged;

		//Unique ID used for identifying the point within undo.
		//Only stays fixed while editing the current cutscene.
		int m_iPointID;
		static int s_iNextPointID;

		// (AAM - 21.10.2004) If false, track never sets "cutscenes modified" flag (defaults to true)
		bool m_bSetCutscenesModified;

		//copied of some of the variables , so the undo system can find out what they were changed from
		float TimeBackup;
		float DurationBackup;
		bool SectionStraightBackup;
		bool m_bStationaryFlagBackup;
		Asura_Vector_3 PositionBackup;
		Asura_Matrix_3x3 OrientationBackup;

		//overlay
		static Asura_Vector_2 OverlayPosition;
		static bool OverlayOpen;
		static Tools_CutsceneTrackPoint_Object* ThisPoint;//(a 'this' substitute for functions called by the overlay)

		enum DialogID
		{
			DialogID_OrientationFromPoints,
			DialogID_RollFromPoints,
			DialogID_PitchAlongTrack,
		};

};

//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------**
** Track_Times_Overlay **
**---------------------*/

class Track_Times_Overlay : public Tools_Modal_Overlay
{
	public :
		Track_Times_Overlay(Tools_Cutscene_Track* track,int index);
		void DoModal(){ShowOverlay();}

		void InitialiseOverlay(class Asura_Overlay* Overlay);

		void Update();

		static void GetSpeedFromAnimation();
		static void OkaySelected();
		static void CancelSelected();


		Tools_Cutscene_Track* Track;
		Asura_Cutscene_Track* AsuraTrack;

		float Time;
		float TimeSince;
		float Speed;
		float IndexTo;
		float IndexFrom;

		bool TimeChanged;
		bool TimeSinceChanged;
		bool SpeedChanged;
		bool IndexToChanged;
		bool IndexFromChanged;

		static Track_Times_Overlay* This;
};

//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------**
** Cutscene Undo **
**---------------*/

class CutsceneUndoData_TrackPoint_Created : public RootUndoData
{
public :
	CutsceneUndoData_TrackPoint_Created(Tools_CutsceneTrackPoint_Object* pxPointObject);
	virtual RootUndoData* ProcessUndoData();
	
	virtual int GetPriority() const;


	Asura_String m_xActorName;
	int m_iTrackID;
	int m_iPointID;

};

class CutsceneUndoData_TrackPoint_Destroyed : public RootUndoData
{
public :	
	CutsceneUndoData_TrackPoint_Destroyed(Tools_CutsceneTrackPoint_Object* pxPointObject);
	virtual RootUndoData* ProcessUndoData();
	virtual int GetPriority() const;
	
	Asura_String m_xActorName;
	int m_iTrackID;
	int m_iPointID;
	int m_iPointIndex;

	Asura_Vector_3 m_xPosition;
	Asura_Matrix_3x3 m_xOrientation;
	float m_fTime;
	float m_fDuration;
	bool m_bSectionStraight;
	bool m_bStationaryFlag;

};

class CutsceneUndoData_TrackPoint_Changed : public RootUndoData
{
public :
	CutsceneUndoData_TrackPoint_Changed(Tools_CutsceneTrackPoint_Object* pxPointObject);
	virtual RootUndoData* ProcessUndoData();

	virtual int GetPriority() const;


	Asura_String m_xActorName;
	int m_iTrackID;
	int m_iPointID;
	int m_iPointIndex;

	Asura_Vector_3 m_xPosition;
	Asura_Matrix_3x3 m_xOrientation;
	float m_fTime;
	float m_fDuration;
	bool m_bSectionStraight;
	bool m_bStationaryFlag;

};

class CutsceneUndoData_Track_Created : public RootUndoData
{
public :
	CutsceneUndoData_Track_Created(Tools_Cutscene_Track* track);
	virtual RootUndoData* ProcessUndoData();

	virtual int GetPriority() const;


	Asura_String ActorName;
	int TrackID;
};

class CutsceneUndoData_Track_Destroyed : public RootUndoData
{
public :	
	CutsceneUndoData_Track_Destroyed(Tools_Cutscene_Track* track);
	virtual RootUndoData* ProcessUndoData();

	virtual int GetPriority() const;

	
	Asura_String ActorName;
	int TrackID;

	Asura_String TrackName;
	unsigned m_uTrackFlags;
};

class CutsceneUndoData_Track_Rename : public RootUndoData
{
public :	
	CutsceneUndoData_Track_Rename(class Tools_Cutscene_Track* track);
	virtual RootUndoData* ProcessUndoData();

	virtual int GetPriority() const;


	Asura_String ActorName;
	int TrackID;

	Asura_String TrackName;
};

class CutsceneUndoData_Track_Changed : public RootUndoData
{
public :	
	CutsceneUndoData_Track_Changed(class Tools_Cutscene_Track* pxTrack);
	virtual RootUndoData* ProcessUndoData();
	
	virtual int GetPriority() const;


	Asura_String m_xActorName;
	int m_iTrackID;

	unsigned m_uTrackFlags;
};





#endif //_TOOLS_CUTSCENE_TRACK_H_
