

//////////////////////////////////////////////////////////////////////////////////////////
//
//  FILE : Tools_Mode_Decals.h
//
//  Created by Dave Reed - 07/08/07
//
//  Copyright (C) 2007 Rebellion (Derby), All Rights Reserved.
//
//  Decal mode - for creating and editing static decals
//
//////////////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM

#include "Core/Asura_LOS.h"
#include "Core/Asura_UserInput.h"
#include "Debugging/Asura_Logfile.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Lighting/Tools_Environment_LOS.h"
#include "Misc/Tools_LOS.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Mesh/Asura_Mesh.h"
#include "Objects/Tools_Object_System.h"
#include "Rendering/Asura_Module.h"
#include "Tools_System.h"
#include "Tools_Overlays.h"
#include "Tools_Mode_Decals.h"
#include "Decals/Tools_StaticDecal_System.h"
#include "Decals/Tools_Overlay_Decals.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Render.h"
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Text_Output.h"

bool				Tools_Mode_Decals::s_bOverlayOpen			       = false;
Asura_Vector_2		Tools_Mode_Decals::s_xOverlayPosition		       = Asura_Vector_2(1.0f, 15.0f);
Asura_Vector_2		Tools_Mode_Decals::s_xDecalOverlayPosition	       = Asura_Vector_2(1.0f, 30.0f);
Tools_Mode_Decals*	Tools_Mode_Decals::s_pxThis					       = NULL;
bool				Tools_Mode_Decals::s_bExitingMode			       = false;
bool				Tools_Mode_Decals::s_bQuickPlacementEnabled        = false;
int					Tools_Mode_Decals::s_iSelectedDecalIndex	       = 0;
uint32				Tools_Mode_Decals::s_uSelectedDecalSetHash         = 0;
int					Tools_Mode_Decals::s_iNextDecalIndex		       = 0;		// When we're using quick placement with random decal indices
float				Tools_Mode_Decals::s_fNextDecalAngle		       = 0;		// When we're using quick placement with random decal orientation
bool                Tools_Mode_Decals::s_bShowModuleBBForSelectedDecal = false;


static Asura_Overlay*			 	   g_pxOverlay				= NULL;
static Tools_StaticDecal_Object*	   g_pxThisObj				= NULL;



void Tools_Mode_Decals::Initialise(void)
{	
	// store this pointer for static access
	s_pxThis = this;

	Tools_Timed_Message_System::AddMessage(0x00ff00, "Switched to Decals Mode");

	// Create the overlays
	CreateOverlay();	
	
	// Create tools objects
	Tools_StaticDecal_System::CreateToolsObjects();

	Tools_Object_System::EnableAllControls(true);
}


void Tools_Mode_Decals::Shutdown(void)
{
	//delete all the objects
	s_bExitingMode = true;
	Tools_Object_System::DeleteAllModeSpecific();
	s_bExitingMode = false;

	// Close our overlays
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DECAL_PLACEMENT);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_STATICDECAL);
	s_bOverlayOpen=false;

	Tools_Object_System::EnableAllControls(false);
}




void Tools_Mode_Decals::ToggleOverlay(void)
{
	if (!s_bOverlayOpen)
	{
		CreateOverlay();	
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DECAL_PLACEMENT);
		s_bOverlayOpen=false;
	}
}
	



void Tools_Mode_Decals::CreateOverlay(void)
{
	// If it's already open, don't create another
	if(s_bOverlayOpen) return;
	s_bOverlayOpen=true;

	Asura_Overlay* pxNewOverlay;
	pxNewOverlay = new Asura_Overlay("Decals",false,&s_xOverlayPosition);
	g_pxOverlay=pxNewOverlay;

	Asura_Dialog*  pxNewDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	
	Asura_DecalManager_DecalSet *pxSet = Asura_Decal_Manager::GetDecalSet(s_uSelectedDecalSetHash);
	

	pxNewDialog = new Asura_Tickbox_Dialog("Quick Placement (P)", xOffset, &s_bQuickPlacementEnabled);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.x+=18;	

	pxNewDialog = new Asura_Function_Dialog("Select Decal Set",xOffset, _SelectCurrentDecalSet );
	pxNewOverlay->AddDialog(pxNewDialog);	
	xOffset.x-=18;	
	
	if (pxSet)
	{	
		xOffset.y+=Asura_Dialog::DefaultSpacing();		

		int iMax = pxSet->m_iNumDecals;
		if (s_iSelectedDecalIndex>iMax) s_iSelectedDecalIndex=iMax;
		pxNewDialog = new Asura_Numerical_Dialog("Decal Index", xOffset, &s_iSelectedDecalIndex,-1,iMax,1,"%d");
		pxNewDialog->SetHelpText("Set to -1 to get a random decal index");
		pxNewOverlay->AddDialog(pxNewDialog);			
		xOffset.x+=18;	

		static char szSetName[80];
		Asura_PrintfToBfr(szSetName, "Set: %s", 80, true, pxSet->m_xSetName);
		pxNewDialog = new Asura_Dialog(szSetName,xOffset);			
		pxNewOverlay->AddDialog(pxNewDialog);		
		xOffset.x-=18;	
	}

	xOffset.y+=Asura_Dialog::LargeSpacing();	

	pxNewDialog = new Asura_Function_Dialog("Create Single Decal (C)",xOffset, _CreateDecal );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog("Recalc All Decals (needed if modules have changed)",xOffset,_UpdateDecals );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Function_Dialog("Decal Editor",xOffset, _EditDecals );
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Dialog("Decals:",xOffset);	
	pxNewDialog->m_iControlID = DialogID_NumDecals;
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.x+=9;

	pxNewDialog = new Asura_Dialog("Buffers:",xOffset);	
	pxNewDialog->m_iControlID = DialogID_NumBuffers;
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.x+=9;

	pxNewDialog = new Asura_Dialog("Tris:",xOffset);	
	pxNewDialog->m_iControlID = DialogID_NumPolys;
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.x-=18;

	xOffset.y += Asura_Dialog::LargeSpacing();

    pxNewDialog = new Asura_Tickbox_Dialog( "Show module BB for selected decal", xOffset, &s_bShowModuleBBForSelectedDecal );
    pxNewDialog->SetHelpText( "Each decal is assigned a module that it 'belongs' to and that is used for visibility checks. If this option is selected then the assigned module's BB will be rendered for the selected decal." );
    pxNewOverlay->AddDialog( pxNewDialog );
    xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewOverlay->m_uID = TOOLS_OVERLAY_DECAL_PLACEMENT;
	Asura_Overlay_System::SubmitNewOverlay(pxNewOverlay,false);		
}



void Tools_Mode_Decals::RenderWorld_PostToneMap(void)
{
	Tools_StaticDecal_System::RenderWireframe();

	if (s_bQuickPlacementEnabled && !Asura_Interface::UsingMenusOrOverlays())
	{		
		int iPoly,iModule;
		Asura_Vector_3 xPosition;
		if(Tools_LOS::FindPolyAndPositionUnderCursor(iPoly,iModule,xPosition))
		{		
			Asura_Vector_3 xNormal=Asura_Vector_3(0,-1,0);

			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModule);

			//get the normal for the polygon
			if(pxMesh && pxMesh->AreNormalsValid())
			{		
				xNormal = pxMesh->NormalsDataPtr[iPoly].FaceNormal;

				Asura_Vector_3 xForward, xRight;
				Asura_Camera::GetForwardVector(xForward);
				xRight = xForward ^ xNormal;
				xRight.Normalise();
				xForward = xNormal ^ xRight;
				xForward.Normalise();
				xNormal =- xNormal;

				// Create an orientation from the normal and camera-forward vector				
				Asura_Matrix_3x3 xOrient;
				xOrient.e11 = xRight.x;		xOrient.e21 = xRight.y;		xOrient.e31 = xRight.z;	
				xOrient.e12 = xNormal.x;	xOrient.e22 = xNormal.y;	xOrient.e32 = xNormal.z;	
				xOrient.e13 = xForward.x;	xOrient.e23 = xForward.y;	xOrient.e33 = xForward.z;

				Asura_DecalManager_DecalSet * pxSet = Asura_Decal_Manager::GetDecalSet(s_uSelectedDecalSetHash);

				if (pxSet && pxSet->m_iNumDecals)
				{
					if (s_iNextDecalIndex>=pxSet->m_iNumDecals)
					{
						s_iNextDecalIndex = Asura_Random::GetInt(0,pxSet->m_iNumDecals-1);
					}

					if (pxSet->m_bRandomOrientation)
					{					
						Asura_Matrix_3x3 xRot;
						Asura_Euler_3 xAngles = Asura_Euler_3(0,s_fNextDecalAngle,0);
						xAngles.ToMatrix(xRot);	
						xOrient = xOrient * xRot;
					}
				
					// Render the decal box
					Asura_DecalManager_Decal *pxDecal = &pxSet->m_pxDecals[s_iNextDecalIndex];
					Asura_Vector_3 xDimensions = Asura_Vector_3(pxDecal->m_fWidth, 0.5f*(pxDecal->m_fWidth+pxDecal->m_fHeight), pxDecal->m_fHeight);

					Asura_Bounding_Box xBB;
					xBB.Reset();
					xBB.Set(xDimensions);
					xBB.Set(-xDimensions);
					Asura_Render::RenderBB(xBB,xPosition, xOrient);				
				
					if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
					{
						s_fNextDecalAngle = Asura_Random::GetFloat(0,3.142f*2.0f);
						s_iNextDecalIndex = Asura_Random::GetInt(0,pxSet->m_iNumDecals-1);

						CreateDecalObject(xPosition, xOrient, false, false);
					}
				}

				// Render the normal, too
				Asura_Render::RenderLine( xPosition, xPosition+xNormal, 0xff00ff00);
			}
		}
	}
}



void Tools_Mode_Decals::Update(void)
{	
	if (s_bQuickPlacementEnabled)
	{
		Tools_Object_System::EnableAllControls(false);
	}
	else
	{
		Tools_Object_System::EnableAllControls(true);
	}
	

	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT) && 
			!Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) &&
			!Asura_UserInput::KeyPressed(ASURA_KEYID_ALT))
		{
			// Handle keyboard shortcuts			
			if(Asura_UserInput::DebouncedKeyPressed('p') || Asura_UserInput::DebouncedKeyPressed('P'))
			{
				s_bQuickPlacementEnabled =! s_bQuickPlacementEnabled;
			}

			if(Asura_UserInput::DebouncedKeyPressed('c') || Asura_UserInput::DebouncedKeyPressed('C'))
			{
				CreateDecal();
			}
		}
	}

	Tools_StaticDecal_System::UpdateBuffers();

	if (g_pxOverlay)
	{
		char szStats[80];
		sprintf(szStats, "Decals: %d", Tools_StaticDecal_System::GetNumDecals());
		g_pxOverlay->UpdateDialogLabel(DialogID_NumDecals, szStats);

		sprintf(szStats, "Buffers: %d", Tools_StaticDecal_System::GetNumDecalBuffers());
		g_pxOverlay->UpdateDialogLabel(DialogID_NumBuffers, szStats);

		sprintf(szStats, "Tris: %d", Tools_StaticDecal_System::GetNumDecalPolys());
		g_pxOverlay->UpdateDialogLabel(DialogID_NumPolys, szStats);
	}		
}



bool Tools_Mode_Decals::IsModified()
{
	return Tools_SaveAsura::IsStaticDecalsModified();
}

void Tools_Mode_Decals::SetModified(bool bHasChanged)
{
	Tools_SaveAsura::SetStaticDecalsModified(bHasChanged);
}

void Tools_Mode_Decals::QuickSave(void)
{		
	// Save just this mode
	PreSave();
	Tools_SaveAsura::SaveStaticDecals();
	PostSave();
}


void Tools_Mode_Decals::PreSave()
{		
}


void Tools_Mode_Decals::PostSave()
{		
}



void Tools_Mode_Decals::CreateDecal(void)
{
	Asura_Matrix_3x3 xOrient = Asura_Maths::IdentityMatrix3x3;
	CreateDecalObject(*Asura_Camera::GetPosition(), xOrient, true, true);
}


void Tools_Mode_Decals::CreateDecalObject(Asura_Vector_3 xPos, Asura_Matrix_3x3 xOrient, bool bSelect, bool bHandleRandomOrient)
{
	Asura_DecalManager_DecalSet * pxSet = Asura_Decal_Manager::GetDecalSet(s_uSelectedDecalSetHash);
	
	if (!pxSet)
	{
		return;
	}

	if(!pxSet->m_iNumDecals)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000, "The decal set doesn't have any decal types defined");
		return;
	}


	if (pxSet->m_bRandomOrientation && bHandleRandomOrient)
	{
		Asura_Matrix_3x3 xRot;
		Asura_Euler_3 xAngles = Asura_Euler_3(0,Asura_Random::GetFloat(0,3.142f*2.0f),0);
		xAngles.ToMatrix(xRot);	
		xOrient = xOrient * xRot;
	}

	Tools_StaticDecal_Object *pxObj = new Tools_StaticDecal_Object(xPos, xOrient);


	if (pxObj)
	{			
		if (s_iSelectedDecalIndex<0)
		{								
			if (pxSet)
			{				
				pxObj->m_iDecalIndex = Asura_Random::GetInt(0, pxSet->m_iNumDecals-1);
			}
			else
			{
				pxObj->m_iDecalIndex = 0;
			}
		}
		else

		{
			pxObj->m_iDecalIndex = s_iSelectedDecalIndex;
		}

		pxObj->m_uDecalSetHash = s_uSelectedDecalSetHash;

		if (bSelect)
		{		
			// Select it
			Tools_Object_System::DeselectAllObjects();
			Tools_Object_System::SelectObject(pxObj, false);	
		}
		else
		{
			Tools_StaticDecal_System::UpdateDecal(pxObj);
		}
	}

	// [11/11/2008 AS] Apparently quick placement wasnt setting decals to be modified.
	s_pxThis->SetModified(true);

}

void Tools_Mode_Decals::UpdateDecals(void)
{
	Tools_StaticDecal_System::UpdateBuffers(true);
	Tools_Timed_Message_System::AddMessage(0x00ff00, "Static decal recalc complete!");
}

void Tools_Mode_Decals::EditDecals(void)
{
	Tools_Overlay_Decals::ToggleOverlay();
}



void Tools_Mode_Decals::CreateDecalOverlay(Tools_StaticDecal_Object* pxObj)
{
	static char szTemplateName[256]="";	

	Asura_Overlay* pxNewOverlay;
	pxNewOverlay = new Asura_Overlay("Static Decal",false,&s_xDecalOverlayPosition);

	Asura_Dialog*  pxNewDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);




	// DecalSet name + Decal index
	Asura_DecalManager_DecalSet *pxSet = Asura_Decal_Manager::GetDecalSet(g_pxThisObj->m_uDecalSetHash);
	
	if (pxSet)
	{
		static char szDecalSet[128];
		Asura_PrintfToBfr(szDecalSet,"Decal Set: '%s'",128,true,pxSet->m_xSetName.GetString());
		pxNewDialog = new Asura_Dialog(szDecalSet, xOffset);
		pxNewOverlay->AddDialog(pxNewDialog);	
		xOffset.y+=Asura_Dialog::DefaultSpacing();				
	}
	else
	{
		pxNewDialog = new Asura_Dialog("Decal Set: *None*", xOffset);
		pxNewOverlay->AddDialog(pxNewDialog);	
		xOffset.y+=Asura_Dialog::DefaultSpacing();		
	}

	// Change Template
	pxNewDialog = new Asura_Function_Dialog("Select Decal Set",xOffset, SelectDecalSet);
	pxNewOverlay->AddDialog(pxNewDialog);	
	xOffset.y+=Asura_Dialog::LargeSpacing();

	if (pxSet)
	{	
		int iMax = pxSet->m_iNumDecals - 1;
		if (g_pxThisObj->m_iDecalIndex>iMax)
		{
			g_pxThisObj->m_iDecalIndex = iMax;
			g_pxThisObj->m_bModified = true;
		}	
		pxNewDialog = new Asura_Numerical_Dialog("Decal Index", xOffset, &g_pxThisObj->m_iDecalIndex, 0, iMax, 1, "%d", &g_pxThisObj->m_bModified);
		pxNewOverlay->AddDialog(pxNewDialog);	
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	// Scale	
	pxNewDialog = new Asura_Numerical_Dialog("Scale", xOffset, &g_pxThisObj->m_fScale, 0.1f, 10.0f, 0.1f, "%.1f", &g_pxThisObj->m_bModified);
	pxNewOverlay->AddDialog(pxNewDialog);	
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	// Opacity
	pxNewDialog = new Asura_Numerical_Dialog("Opacity", xOffset, &g_pxThisObj->m_fAlpha, 0.01f, 1.0f, 0.01f, "%.2f", &g_pxThisObj->m_bModified);
	pxNewOverlay->AddDialog(pxNewDialog);	
	xOffset.y+=1.5f*Asura_Dialog::DefaultSpacing();
	
	// Close
	pxNewDialog = new Asura_Function_Dialog("Close",xOffset,CloseInstanceOverlay);
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y+=1.0f*Asura_Dialog::DefaultSpacing();

	// Create it
	pxNewOverlay->m_uID = TOOLS_OVERLAY_STATICDECAL;
	Asura_Overlay_System::SubmitNewOverlay(pxNewOverlay,false);	

}



void Tools_Mode_Decals::CloseInstanceOverlay(void)
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_STATICDECAL);
}



void Tools_Mode_Decals::SelectDecalSet(void)
{
	if (g_pxThisObj)
	{
		if (Asura_Decal_Manager::GetNumDecalSets()==0)
		{
			Tools_Timed_Message_System::AddMessage(0x00ff00, "No Decal Sets Exist!");
			return;
		}

		ListSelection_Overlay xOverlay("Select Decal Set:");		

		xOverlay.AddItem("<None>",0);
		for (int i=0;i<Asura_Decal_Manager::GetNumDecalSets();i++)	
		{
			xOverlay.AddItem(Asura_Decal_Manager::GetDecalSetByIndex(i)->m_xSetName.GetString(), 1+i);	
		}

		if (!xOverlay.DoModal())
		{
			return;
		}

		int iIndex = xOverlay.GetSelectionData()-1;
		if (iIndex<0)
		{
			g_pxThisObj->m_uDecalSetHash  = 0;
			g_pxThisObj->m_bModified = true;
		}
		else
		{
			Asura_DecalManager_DecalSet* pxSet = Asura_Decal_Manager::GetDecalSetByIndex(iIndex);
			g_pxThisObj->m_uDecalSetHash = pxSet->m_uSetNameHash;
			g_pxThisObj->m_bModified = true;
		}

		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_STATICDECAL);
		CreateDecalOverlay(g_pxThisObj);
	}
}




void Tools_Mode_Decals::SelectCurrentDecalSet(void)
{
	if (Asura_Decal_Manager::GetNumDecalSets()==0)
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00, "No Decal Sets Exist!");
		return;
	}

	ListSelection_Overlay xOverlay("Select Decal Set:");		

	xOverlay.AddItem("<None>",0);
	for (int i=0;i<Asura_Decal_Manager::GetNumDecalSets();i++)	
	{
		xOverlay.AddItem(Asura_Decal_Manager::GetDecalSetByIndex(i)->m_xSetName.GetString(), 1+i);	
	}

	if (!xOverlay.DoModal())
	{
		return;
	}

	int iIndex = xOverlay.GetSelectionData()-1;
	if (iIndex<0)
	{
		s_uSelectedDecalSetHash = 0;
	}
	else
	{
		Asura_DecalManager_DecalSet* pxSet = Asura_Decal_Manager::GetDecalSetByIndex(iIndex);
		s_uSelectedDecalSetHash = pxSet->m_uSetNameHash;				

		if(!pxSet->m_iNumDecals)
		{
			ShowMessageOverlay("The selected decal set hasn't got any types defined , so it won't be possible to create any instances of it.");
		}
	}	


	// Close+Open the overlay to refresh the params
	ToggleOverlay();
	ToggleOverlay();	
}



Tools_StaticDecal_Object::Tools_StaticDecal_Object(const Asura_Vector_3 &xPosition, const Asura_Matrix_3x3 &xOrient, bool bUpdateDecal) : Tools_Object(TOT_StaticDecal)
{
	Position	= xPosition;
	Orientation = xOrient;

	Asura_String mesh_name = "ToolsShape_CutsceneTrackPoint";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);


	m_uDecalSetHash		= 0;
	m_iDecalIndex		= 0;
	m_fScale			= 1.0f;
	m_fAlpha			= 1.0f;
	m_pxDecal			= 0;

	m_bModified			= false;

	if (bUpdateDecal)
	{	
		// This will add it to the staticdecal system, and mark the affected
		// vertex/index buffers as dirty
		Tools_StaticDecal_System::UpdateDecal(this);
	}
}


void Tools_StaticDecal_Object::UpdateWhileSelected()
{
	Tools_StaticDecal_System::SetSelectedDecal(this);	
}


void Tools_StaticDecal_Object::Update()
{
	if (m_pxDecal)
	{	
		if (Position != m_pxDecal->m_xPosition ||
			Orientation != m_pxDecal->m_xOrient)
		{
			m_bModified = true;
		}
	}

	if (m_bModified)
	{
		Tools_SaveAsura::SetStaticDecalsModified(true);
		Tools_StaticDecal_System::UpdateDecal(this);
		m_bModified=false;

		if (IsSelected())
		{		
			Tools_StaticDecal_System::SetSelectedDecal(this);	
		}
	}		
}


void Tools_StaticDecal_Object::Render()
{
	Asura_DecalManager_DecalSet *pxSet = Asura_Decal_Manager::GetDecalSet(m_uDecalSetHash);

	if (IsSelected() && pxSet && m_iDecalIndex>=0 && m_iDecalIndex<pxSet->m_iNumDecals)
	{
		Asura_DecalManager_Decal *pxDecal = &pxSet->m_pxDecals[m_iDecalIndex];
		
		Asura_Bounding_Box xBB;
		xBB.Reset();

		float fScale = m_fScale;
		float fDepth = 0.5f*(pxDecal->m_fWidth+pxDecal->m_fHeight);

		xBB.Set(Asura_Vector_3(fScale*pxDecal->m_fWidth,   fScale*fDepth,  fScale*pxDecal->m_fHeight));
		xBB.Set(Asura_Vector_3(fScale*-pxDecal->m_fWidth,  fScale*-fDepth, fScale*-pxDecal->m_fHeight));

		Asura_Render::RenderBB(xBB, Position, Orientation, 0xffffff00);
	}
	else
	{
		if (Tools_Object_System::AreControlsEnabled())
		{		
			Tools_Object::Render();
		}
	}
	
	if (IsSelected() && m_pxDecal)		
	{
        const int iModuleIdx = m_pxDecal->m_iModuleIndex;
		if( iModuleIdx >= 0 )
		{		
			const Asura_Mesh* pxModuleMesh = Asura_Module_System::GetMeshForModule( iModuleIdx );
			if (pxModuleMesh)
			{			
				Asura_Text_Output::Print( 1, 15, 0xff00ff00, 0, "Decal in module '%s'", pxModuleMesh->Name.GetString());
			}

            if( Tools_Mode_Decals::s_bShowModuleBBForSelectedDecal )
            {
                Tools_Render::RenderCube( Asura_Module_System::GetBoundingBoxOfModule(iModuleIdx),
                                          Asura_Module_System::GetPositionOfModule(iModuleIdx),
                                          Asura_Maths::IdentityMatrix3x3,
                                          0x7F2222FF,
                                          false );
            }
		}			
	}
}


void Tools_StaticDecal_Object::Destroy()
{
	if (!Tools_Mode_Decals::s_bExitingMode)
	{	
		// If we're leaving the mode, we aren't deleting the decal - we're destroying
		// all tools objects - so don't remove it from the staticdecal manager
		Tools_StaticDecal_System::RemoveDecal(this);
	}

	Tools_Object::Destroy();
}



void Tools_StaticDecal_Object::OnDeselection()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_STATICDECAL);
	g_pxThisObj = NULL;
	Tools_StaticDecal_System::SetSelectedDecal(NULL);
	Tools_Object::OnDeselection();
}



void Tools_StaticDecal_Object::OnSelection()
{	
	g_pxThisObj = this;	
	m_bModified = true;
	Tools_StaticDecal_System::SetSelectedDecal(this);
	Tools_Object::OnSelection();	
	Tools_Mode_Decals::CreateDecalOverlay(g_pxThisObj);
}



const char* Tools_StaticDecal_Object::GetName()
{
	Asura_DecalManager_DecalSet* pxSet = Asura_Decal_Manager::GetDecalSet(m_uDecalSetHash);

	if (pxSet)
	{
		return pxSet->m_xSetName.GetString();
	}
	else
	{
		return "<Decal Set: *None*>";
	}
}


void Tools_StaticDecal_Object::Copy(const Tools_Object* pxObject)
{
	const Tools_StaticDecal_Object* pxFrom = (const Tools_StaticDecal_Object*) pxObject;
	
	m_fScale		= pxFrom->m_fScale;
	m_fAlpha		= pxFrom->m_fAlpha;
	m_iDecalIndex	= pxFrom->m_iDecalIndex;
	m_uDecalSetHash = pxFrom->m_uDecalSetHash;	

	Tools_StaticDecal_System::UpdateDecal(this);

	Tools_SaveAsura::SetStaticDecalsModified();
	m_bModified = false;

}

Tools_Object* Tools_StaticDecal_Object::CreateObjectOfThisType() const
{
	return new Tools_StaticDecal_Object(*Asura_Camera::GetPosition(), Asura_Maths::IdentityMatrix3x3, false);
}



#endif // ASURA_USE_STATIC_DECAL_SYSTEM