///////////////////////////////////////////////////////////////////////////////
//
// File: Tools_Container_Attachment.h
//
// Created by Martin G.Harris (3/7/2008)
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_CONTAINER_ATTACHMENT_H__
#define __TOOLS_CONTAINER_ATTACHMENT_H__

#ifdef ASURA_USE_CONTAINER_SYSTEM

// Includes
#include "Entity/Container/Tools_Container.h"

////////////////////////////////
// Tools_Container_Attachment //
////////////////////////////////
class Tools_Container_Attachment : public Tools_Container
{
	typedef Tools_Container PARENT;

public:
	Tools_Container_Attachment();

	virtual void AddOverlayControls(Asura_Overlay& xNewOverlay, Asura_Vector_2& xOffset);
	
	virtual	void ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	virtual	void WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	virtual void CopyDataToAsura();
	virtual void Verify(bool bRecurse) const;

	// Get a short name to be used in the instance overlay (use marker name).
	virtual const char* GetInstanceName() const { return m_bAttachToPhysicsObject ? "physics" : m_xMarkerName.GetString(); }

	Asura_Hash_ID GetChildContainerHash() const { return m_uChildContainerHash; }
	Tools_Container* GetChildContainer() const;

	// Calling code should check for circular references, and set modified.
	void SetChildContainer(Asura_Hash_ID uChildHash) { m_uChildContainerHash = uChildHash; }

	// Add a child with the specified hash. Doesn't check that the child exists.
	// Nor does it check for circular references (calling code should prevent this).
	virtual void AddChild(Asura_Hash_ID uChildHash) { SetChildContainer(uChildHash); }

	// Add a child chosen from a modal overlay.
	virtual void AddChildFromList() { SelectChild(); }

	// Can we add a new child? Yes, if it isn't already set.
	virtual bool CanAddChild() const { return m_uChildContainerHash == ASURA_HASH_ID_UNSET; }

	// Remove specified child. Return true if succeeded.
	virtual bool RemoveChild(Asura_Hash_ID uChildHash);

	// Set up the child iterator.
	virtual void FillChildIterator(Tools_Container_ChildIterator& xIterator) const;

	// Is this a marker attachment?
	virtual bool IsMarkerAttachment() const
	{
		return !m_bAttachToPhysicsObject && !m_xMarkerName.IsBlank();
	}

	// Is this a physics attachment?
	virtual bool IsPhysicsAttachment() const { return m_bAttachToPhysicsObject; }

	const Asura_String& GetMarkerName() const { return m_xMarkerName; }
	Asura_Hash_ID GetMarkerHash() const { return m_xMarkerName.GetHash(); }

	// Set parameters from preview mode.
	void SetPosOriFromMode(const Asura_Vector_3& xPosition, const Asura_Quat& xQuat);

	virtual u_int GetNumInputs() const;

protected:
	void RefreshDialogue();

	// Callbacks
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis) {((Tools_Container_Attachment*)iThis)->__FnName();}\
	void __FnName();

	OVERLAY_VOID_CALLBACK(SelectChild);
	OVERLAY_VOID_CALLBACK(EditChild);
	OVERLAY_VOID_CALLBACK(ResetPosition);
	OVERLAY_VOID_CALLBACK(ResetOrientation);
	OVERLAY_VOID_CALLBACK(SelectMarker);
	OVERLAY_VOID_CALLBACK(AttachToPhysicsClicked);
	
	#undef OVERLAY_VOID_CALLBACK


protected:
	Asura_Hash_ID m_uChildContainerHash;

	Asura_String m_xMarkerName;		// Name of marker point to attach to or blank if not using markers.
	
	Asura_String m_xChildDialogue;	// String displayed on dialogue.
	Asura_String m_xMarkerDialogue;	// String displayed on dialogue.

	bool m_bAttachToPhysicsObject;	// Attach to the parent collection's physics object (if there is one).

	// Offsets
	Asura_Vector_3 m_xAttachPosition;
	Asura_Quat m_xAttachQuat;
	Asura_Euler_3 m_xAttachEuler; // Equivalent to quat version, used for user input.
};


#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#endif //__TOOLS_CONTAINER_ATTACHMENT_H__

