///////////////////////////////////////////////////////////////////////////////
//
// File: Tools_Entity_ConsoleVar.h
//
// Created by Martin G.Harris (20/6/2007)
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_ENTITY_CONSOLE_VAR_H__
#define __TOOLS_ENTITY_CONSOLE_VAR_H__

// Need command console for the console variables.
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE

// Includes
#include "Entity/Tools_Entity_Object.h"
#include "Objects/Tools_Object_System.h"

#include "Core/Asura_String.h"

// Forward declarations

/////////////////////////////
// Tools_Entity_ConsoleVar //
/////////////////////////////
class Tools_Entity_ConsoleVar : public Tools_Entity_Object
{
	typedef Tools_Entity_Object PARENT;

public:
	Tools_Entity_ConsoleVar(Asura_Entity* pxEntity);
	virtual ~Tools_Entity_ConsoleVar();

	virtual	const char*	GetTypeDescription() { return "Console Variable"; }

	virtual const char* GetMyNamePrefix() const { return Tools_Entity_ConsoleVar::GetNamePrefix(); }

	virtual void AddOverlayControls(Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset);
	virtual bool HandlesThisMessage(int iMessageID);
	virtual void PrintInfo(float& fX, float& fY);

	virtual void Copy(const Tools_Object* pxObject);
	virtual bool CanBeCopied() const { return Type == TOT_Entity_ConsoleVar; }
	virtual Tools_Object* CreateObjectOfThisType() const;

	virtual void Update();

// Static members
	static void	Create();
	static void ChangeVarName();
	static void SelectEntityToTrack();

	static void AddMessageOverlayControls_Change(struct Asura_StaticMessage* pxMessage, class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset);

	static const char* GetNamePrefix() { return "CVar_"; }

protected:
	void CopyVarNameFromClient();
	void CopyTypeFromClient();

private:
	// Private functions...
	Tools_Entity_ConsoleVar(void);
	Tools_Entity_ConsoleVar(const Tools_Entity_ConsoleVar& xOther);

	Tools_Entity_ConsoleVar& operator= (const Tools_Entity_ConsoleVar& xOther);

	// Data
	Asura_String m_xVarName;
	int m_iTypeFromList;
	bool m_bTypeChangedFlag;
};


//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_ConsoleVar_ChooseEntityToTrack
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Entity_ConsoleVar_ChooseEntityToTrack : public Tools_ChooseObject
{
public :
	Tools_Entity_ConsoleVar_ChooseEntityToTrack( Asura_Guid* puGuidToSet )
		: m_puGuidToSet( puGuidToSet )
	{}

	virtual bool CanBeSelected( Tools_Object* pxObject );

	virtual void ObjectSelected( Tools_Object* pxObject )
	{
		*m_puGuidToSet = pxObject ? pxObject->GetGuid() : ASURA_GUID_UNREGISTERED;
	}

	virtual void OnFinish() { delete this; }

private:
	Asura_Guid* m_puGuidToSet;
};

#endif //ASURA_SUPPORT_COMMAND_CONSOLE

#endif //__TOOLS_ENTITY_CONSOLE_VAR_H__

