#ifdef ASURA_USE_AXON

//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_CoverpointNode.h
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_ENTITYNODE_COVERPOINTNODE_H__
#define __TOOLS_ENTITYNODE_COVERPOINTNODE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Entity/Locations/OrientedNode/CoverpointNode/Asura_ServerNode_Coverpoint.h"

// tools
#include "Entity/Locations/OrientedNode/Tools_EntityNode_OrientedNode.h"
#include "Objects/Tools_Object_System.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

template < class T > class Asura_List;
class Asura_Physics_RigidBodyInstance;

// S H A R E D _ P A R A M S /////////////////////////////////////////////////

const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_COVERPOINT_LOWCOVER = 0x731FDE5C; //Asura_GetHashID("Asura_Coverpoint_LowCover")
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_COVERPOINT_MINRANGE = 0x8F0CEFA4; //Asura_GetHashID("Asura_Coverpoint_MinRange")
const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_COVERPOINT_MAXRANGE = 0xF8EB8BD2; //Asura_GetHashID("Asura_Coverpoint_MaxRange")

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_CoverpointEmergePoint
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Object_CoverpointEmergePoint : public Tools_Object
{
    typedef Tools_Object PARENT;

public:
    Tools_Object_CoverpointEmergePoint();
    Tools_Object_CoverpointEmergePoint( const Tools_Object_CoverpointEmergePoint& xRhs );
    virtual ~Tools_Object_CoverpointEmergePoint() {}

    virtual const char* GetTypeDescription() { return "Emerge Point"; }
    virtual const char* GetName()            { return "Emerge Point"; }
    virtual void UpdatePositionInAsuraData( bool bUpdateLinkedObjects );
    virtual void Destroy();
	virtual bool IsHidden();

    void SetOwnerCoverpoint( class Tools_EntityNode_CoverpointNode* pxMyCoverpoint );
    void SetPositionPointer( Asura_Vector_3* pxPosition )      { m_pxPosition = pxPosition; Position = *m_pxPosition; }
    void SetIndex( u_int uIndex )                              { m_uIndex = uIndex; }
    void SetNext( Tools_Object_CoverpointEmergePoint* pxNext ) { m_pxNext = pxNext; }

    u_int                               GetIndex() const { return m_uIndex; }
    Tools_Object_CoverpointEmergePoint* GetNext()  const { return m_pxNext; }

protected:
    class Tools_EntityNode_CoverpointNode* m_pxMyCoverpoint;
    // Pointer to the emerge point position data in the game coverpoint entity.
    Asura_Vector_3*                        m_pxPosition;
    u_int                                  m_uIndex;
    Tools_Object_CoverpointEmergePoint*    m_pxNext;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_CoverpointNode
//
//////////////////////////////////////////////////////////////////////////////

class Tools_EntityNode_CoverpointNode : public Tools_EntityNode_OrientedNode
{
    typedef Tools_EntityNode_OrientedNode PARENT;

public:
    Tools_EntityNode_CoverpointNode( Asura_Entity* pxEntity );
    virtual ~Tools_EntityNode_CoverpointNode();

    virtual void AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset );
    virtual const char* GetTypeDescription() { return "Coverpoint Node"; }
    virtual bool IsBaseOrDerivativeOf( int iToolsObjectType ) const;

	virtual const char* GetMyNamePrefix() const { return Tools_EntityNode_CoverpointNode::GetNamePrefix(); }

	virtual void OnSelection();
    virtual void OnDeselection();
    virtual void Render();

    virtual void UpdatePositionInAsuraData( bool bUpdateLinkedObjects );
    virtual void Update();
    virtual void UpdateWhileSelected();

	virtual bool GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const;
	virtual bool SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam );

    virtual bool          CanBeCopied() const { return Type == TOT_Entity_CoverpointNode; }
    virtual void          Copy( const Tools_Object* pxObject );
    virtual Tools_Object* CreateObjectOfThisType() const;

	void SetRequiresValidation() { m_bRequiresValidation = true; }
	bool RequiresValidation() const { return m_bRequiresValidation; }
    void DestroyEmergePoint( const Tools_Object_CoverpointEmergePoint& xEmergePoint );

	// Create a new object in tools.
    static void Create();

	static void CheckCoverpointsForValidity();

	static const char* GetNamePrefix() { return "CP_"; }

protected:
	void DeleteAllEmergePoints();

	void SetIsLowCover( bool bSet )         { m_bIsLowCover = bSet; }
	void SetCoverAngle( float fCoverAngle ) { m_fCoverAngle = fCoverAngle; }
	void SetChanged( bool bSet )            { m_bChanged = bSet; }
	void SetMinCoverRange( float fRange )   { m_fMinCoverRange = fRange; }
	void SetMaxCoverRange( float fRange )   { m_fMaxCoverRange = fRange; }

	void ThisAddEmergePoint( const Asura_Vector_3* pxPosition = NULL );

private:
    static bool CanAddEmergePoint();
    static void AddEmergePoint();

	static void RememberPhysicsObjectsStatus();
	static void SetPhysicsObjectsAsIgnored();
	static void RestorePhysicsObjectsStatus();
	static void ForgetPhysicsObjectsStatus();

    

    static Tools_EntityNode_CoverpointNode* CreateAndReturn();

protected:
	Tools_Object_CoverpointEmergePoint* m_pxEmergePoints;

private:
	bool			 m_bRequiresValidation;
    bool             m_bChanged;
    bool             m_bIsLowCover;
    float            m_fCoverAngle;
    float            m_fMinCoverRange;
    float            m_fMaxCoverRange;
    Asura_Vector_3   m_xPrevPosition;
    Asura_Matrix_3x3 m_xPrevOrientation;
    Asura_ServerNode_Coverpoint* m_pxGameEntity;

	static Asura_List< Asura_Physics_RigidBodyInstance > s_xRigidBodyIgnoreList;
	static Asura_List< Asura_Physics_RigidBodyInstance > s_xRigidBodyNonIgnoreList;

    static Tools_EntityNode_CoverpointNode* s_pxThisToolsCoverpointNode;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_ENTITYNODE_COVERPOINTNODE_H__
#endif // ASURA_USE_AXON
