//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_GS2Node.cpp
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

#ifdef ASURA_USE_AXON

// header
#include "Entity/Locations/OrientedNode/GamesceneNode/Tools_EntityNode_GS2Node.h"

// asura
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Debugging/Asura_Logfile.h"
#include "Entity/Asura_Entity_Event.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GS2Node.h"
#include "Entity/Locations/Spline/GamesceneSpline/Asura_ServerNode_GS2Spline.h"
#include "Interface/Asura_Table_Dialog.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Text_Output.h"

// axon
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"
#include "Behaviour/Axon_BehaviourParams.h"
#include "Behaviour/BehaviourTypes/Axon_Behaviour_Types.h"
#include "Core/Axon_Messages.h"

// tools
#include "Entity/Locations/Spline/GamesceneSpline/Tools_EntityNode_GS2Spline.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_ObjectSystem_Param.h"
#include "Objects/Axon/Tools_Object_Axon_Interface.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_GS2Node
//
//////////////////////////////////////////////////////////////////////////////

TOOLS_DECLARE_SHARED_PARAM( bool,         0x44bbb101, GS2Node_TestForCollisions );		// Asura_GetHashID( "GS2Node_TestForCollisions" )
TOOLS_DECLARE_SHARED_PARAM( bool,         0x9a85fe6a, GS2Node_IgnoreCollisions );		// Asura_GetHashID( "GS2Node_IgnoreCollisions" )
TOOLS_DECLARE_SHARED_PARAM( bool,         0x7f4957a5, GS2Node_FollowSurface );			// Asura_GetHashID( "GS2Node_FollowSurface" )
TOOLS_DECLARE_SHARED_PARAM( bool,         0x4eb19931, GS2Node_UseLinkWeights );			// Asura_GetHashID( "GS2Node_UseLinkWeights" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, 0x43d19a87, GS2Node_StructureTypeString );	// Asura_GetHashID( "GS2Node_StructureTypeString" )
TOOLS_DECLARE_SHARED_PARAM( int,          0x34066139, GS2Node_StructureTypeInt );		// Asura_GetHashID( "GS2Node_StructureTypeInt" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, 0x65c92cad, GS2Node_BehaviourSetString );		// Asura_GetHashID( "GS2Node_BehaviourSetString" )
TOOLS_DECLARE_SHARED_PARAM( int,          0x052670d3, GS2Node_BehaviourSetInt );		// Asura_GetHashID( "GS2Node_BehaviourSetInt" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, uTOOLS_SHAREDPARAMHASH_ENTITYNODE_SETTHISTODEFAULTS, EntityNode_SetThisToDefaults );

const char*	Tools_EntityNode_GS2Node::s_aszBehaviourStringList[ MAX_NUM_BEHAVIOUR_STRINGS ];
int			Tools_EntityNode_GS2Node::s_iNumListEntries = 0;

Asura_Hashed_List< Tools_EntityNode_GS2Node > Tools_EntityNode_GS2Node::s_xList;
const char* Tools_EntityNode_GS2Node::s_aszTabNames[ TAB_END_ENUM ] =
{
	"Properties",
	"Behaviour"
};
u_int Tools_EntityNode_GS2Node::s_uCurrentTab = Tools_EntityNode_GS2Node::TAB_PROPERTIES;

Asura_Vector_2 Tools_EntityNode_GS2Node::s_xDefaultsOverlayPosition( 10.0f, 10.0f );
bool Tools_EntityNode_GS2Node::s_bRefreshDefaultsOverlay = false;
int Tools_EntityNode_GS2Node::s_iDefaultStructureType = AXON_GS2_ST_NONE;
bool Tools_EntityNode_GS2Node::s_bDefaultSmoothOverNode = false;
bool Tools_EntityNode_GS2Node::s_bDefaultFollowSurface = false;
bool Tools_EntityNode_GS2Node::s_bDefaultTestForCollisions = true;
bool Tools_EntityNode_GS2Node::s_bDefaultIgnoreCollisions = false;
int Tools_EntityNode_GS2Node::s_iDefaultBehaviourSet = AXON_BEHAVIOUR_TYPE_GS2_NODE;

Tools_EntityNode_GS2Node::Tools_EntityNode_GS2Node( Asura_Entity* pxEntity )
: PARENT( pxEntity )
, m_pxTabsDialogue( NULL )
, m_uOwnerGuid( ASURA_GUID_UNREGISTERED )
, m_bPreviewing( false )
, m_bPreviewIsOccupied( false )
, m_uPreviewPendingCount( 0 )
, m_uLinkNameListSize( 0 )
, m_ppszLinkNameList( NULL )
, m_pxWeightTableDialogue( NULL )
{
	Type = TOT_Entity_GS2Node;

	if ( GetGameGS2Node()->GetBehaviourParams() )
	{
		GetGameGS2Node()->GetBehaviourParams()->DoSetToolsNode( this );
	}

	SetMesh( "ToolsShape_GSNode" );

	s_xList.AddToList( this );
}

Tools_EntityNode_GS2Node::~Tools_EntityNode_GS2Node()
{
	DestroyLinkNameList();

	if ( m_bPreviewing )
	{
		StopPreview();
	}

	if ( GetGameGS2Node() && GetGameGS2Node()->GetBehaviourParams() )
	{
		GetGameGS2Node()->GetBehaviourParams()->DoSetToolsNode( NULL );
	}

	s_xList.RemoveFromList( this );
}

bool Tools_EntityNode_GS2Node::IsBaseOrDerivativeOf( int iToolsObjectType ) const
{
	if ( iToolsObjectType == TOT_Entity_GS2Node )
	{
		return true;
	}
	else
	{
		return PARENT::IsBaseOrDerivativeOf( iToolsObjectType );
	}
}

void Tools_EntityNode_GS2Node::AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxNewOverlay, xOffset );

	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	m_pxTabsDialogue = new Asura_Tabs_Dialog( xOffset, s_aszTabNames, sizeof( s_aszTabNames ) / sizeof( s_aszTabNames[ 0 ] ) );
	pxNewOverlay->AddDialog( m_pxTabsDialogue );

	m_pxTabsDialogue->SelectTab( TAB_PROPERTIES );
	xOffset.y = m_pxTabsDialogue->m_xMaxCoord.y + 0.5f;
	float fTabsEndY = xOffset.y;

	AddPropertiesTabControls( m_pxTabsDialogue, xOffset );
	fTabsEndY = Asura_Maths::Max( fTabsEndY, xOffset.y );

	m_pxTabsDialogue->SelectTab( TAB_BEHAVIOUR );
	xOffset.y = m_pxTabsDialogue->m_xMaxCoord.y + 0.5f;
	AddBehaviourTabControls( m_pxTabsDialogue, xOffset );
	fTabsEndY = Asura_Maths::Max( fTabsEndY, xOffset.y );

	xOffset.y = fTabsEndY + Asura_Dialog::DefaultSpacing();
	AddUnderTabsControls( pxNewOverlay, xOffset );

	m_pxTabsDialogue->SetSelectedTabPointer( &s_uCurrentTab );
}

bool Tools_EntityNode_GS2Node::IsSelectable()
{
	return ( ( Tools_System::GetCurrentMode() == TOOLS_MODE_OBJECT ) || ( Tools_System::GetCurrentMode() == TOOLS_MODE_CUTSCENE ) ) && PARENT::IsSelectable();
}

bool Tools_EntityNode_GS2Node::HandlesThisMessage( int iMessageID )
{
	if ( iMessageID == ASURA_MSG_AXON_GS2_DONT_STOP_AT_NODE ||
		iMessageID == ASURA_MSG_AXON_GS2_TAKE_LINK ||
		iMessageID == ASURA_MSG_AXON_GS2_LEAVE_NODE ||
		iMessageID == ASURA_MSG_AXON_GS2_EXIT_GAMESCENE ||
		iMessageID == ASURA_MSG_AXON_GS2_SET_LINK_WEIGHT ||
		iMessageID == ASURA_MSG_AXON_GS2_ANIM_CHANGE ||
		iMessageID == ASURA_MSG_AXON_GS2_PATHFIND_TO_NODE )
	{
		return true;
	}
	else if ( GetGameGS2Node()->GetBehaviourParams() && GetGameGS2Node()->GetBehaviourParams()->DoHandlesThisMessage( iMessageID ) )
	{
		return true;
	}
	else
	{
		return PARENT::HandlesThisMessage( iMessageID );
	}
}

bool Tools_EntityNode_GS2Node::HandlesThisEvent( int iEventID ) const
{
	if ( iEventID == ASURA_ENTITY_EVENT_ONARRIVEDAT || iEventID == ASURA_ENTITY_EVENT_ONDEPARTEDFROM )
	{
		return true;
	}

	return PARENT::HandlesThisEvent( iEventID );
}

bool Tools_EntityNode_GS2Node::OwnsThisMessageBlock( unsigned short usMessageBlockID )
{
	if ( GetGameGS2Node()->GetBehaviourParams() && GetGameGS2Node()->GetBehaviourParams()->DoOwnsThisMessageBlock( usMessageBlockID ) )
	{
		return true;
	}

	return PARENT::OwnsThisMessageBlock( usMessageBlockID );
}

void Tools_EntityNode_GS2Node::Render()
{
	PARENT::Render();

	const Asura_ServerNode_GS2Node* pxGameGS2Node = GetGameGS2Node();
	if ( pxGameGS2Node->GetBehaviourParams() )
	{
		pxGameGS2Node->GetBehaviourParams()->DoRender();
	}

	if ( IsGroupSelected() && pxGameGS2Node->ShouldUseLinkWeights() )
	{
		// Render the link weights.
		for ( u_int u = 0; u < pxGameGS2Node->GetNumberLinks(); ++u )
		{
			const Tools_EntityNode_GS2Spline* pxToolsGS2Spline = Tools_EntityNode_GS2Spline::FindEntity( pxGameGS2Node->GetLink( u ) );
			if ( !pxToolsGS2Spline ) { continue; }
			const Asura_ServerNode_GS2Spline* pxGameGS2Spline = pxToolsGS2Spline->GetGameGS2Spline();
			if ( !pxGameGS2Spline ) { continue; }

			Asura_Vector_3 xRenderPosition;
			pxToolsGS2Spline->GetSplineRenderPosition( xRenderPosition, pxGameGS2Spline->GetSourceGuid(), pxGameGS2Spline->GetTargetGuid() );
			xRenderPosition.y -= 0.5f;

			if ( !Asura_Camera::ProjectWorldCoordsToScreenSpace( xRenderPosition, true ) ) { continue; }
			Asura_Font_System::SetFont( 0 );
			Asura_Font_System::Print( xRenderPosition.x, xRenderPosition.y,
				0.75f, 0xffffffff,
				ASURA_TEXTFORMAT_CENTREJUSTIFIED | ASURA_TEXTFORMAT_FORMATSTRING,
				"%0.2f", pxGameGS2Node->GetLinkWeight( u ) );
		}
	}
}

void Tools_EntityNode_GS2Node::Update()
{
	PARENT::Update();

	UpdatePreview();

	if ( GetGameGS2Node()->GetBehaviourParams() )
	{
		GetGameGS2Node()->GetBehaviourParams()->DoUpdate();
	}

	if ( s_bRefreshDefaultsOverlay )
	{
		RefreshDefaultsOverlay();
		s_bRefreshDefaultsOverlay = false;
	}

	//## 2009/05/05 This can be removed once all levels have been re-saved.
	if ( GetGameGS2Node()->GetOccupierList().Size() )
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_INFO, "Cleared node %u occupier list.", GetGuid() );
		GetGameGS2Node()->m_xOccupierGuids.Clear();
		SetEntityModified();
	}
}

void Tools_EntityNode_GS2Node::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( GetGameGS2Node()->GetBehaviourParams() )
	{
		GetGameGS2Node()->GetBehaviourParams()->DoUpdateWhileSelected();
	}
}

void Tools_EntityNode_GS2Node::RefreshOverlay()
{
	const u_int uCurrentTab = m_pxTabsDialogue ? m_pxTabsDialogue->GetSelected() : 0xffffffff;
	DeleteOverlay();
	CreateOverlay();
	if ( uCurrentTab < 0xffffffff )
	{
		m_pxTabsDialogue->SelectTab( static_cast< int >( uCurrentTab ) );
	}
}

void Tools_EntityNode_GS2Node::UpdatePositionInAsuraData( bool bUpdateLinkedObjects )
{
	PARENT::UpdatePositionInAsuraData( bUpdateLinkedObjects );

	GetGameGS2Node()->UpdateFollowSurface();
}

bool Tools_EntityNode_GS2Node::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	if ( GetGameGS2Node()->GetBehaviourParams() && GetGameGS2Node()->GetBehaviourParams()->DoGetParameter( uParamID, pxParam ) )
	{
		return true;
	}

	switch ( uParamID )
	{
		case g_uHash_GS2Node_TestForCollisions:
			pxParam->SetBool( GetGameGS2Node()->ShouldTestForCollisions() );
			return true;

		case g_uHash_GS2Node_IgnoreCollisions:
			pxParam->SetBool( GetGameGS2Node()->ShouldIgnoreCollisions() );
			return true;

		case g_uHash_GS2Node_FollowSurface:
			pxParam->SetBool( GetGameGS2Node()->ShouldFollowSurface( true ) );
			return true;

		case g_uHash_GS2Node_UseLinkWeights:
			pxParam->SetBool( GetGameGS2Node()->ShouldUseLinkWeights() );
			return true;

		case g_uHash_GS2Node_StructureTypeString:
		{
			ListSelection_Overlay xStructureList( "" );
			Tools_Object_Axon_Interface::GetInstance()->BuildGS2StructureList( xStructureList, this );
			pxParam->SetString( xStructureList.GetItemNameByData( GetGameGS2Node()->GetStructureType() ) );
			return true;
		}

		case g_uHash_GS2Node_BehaviourSetString:
		{
			ListSelection_Overlay xBehaviourList( "" );
			Tools_Object_Axon_Interface::GetInstance()->BuildGS2NodeBehaviourSetList( xBehaviourList, this );
			pxParam->SetString( xBehaviourList.GetItemNameByData( GetGameGS2Node()->GetBehaviourSet() ) );
			return true;
		}

		case g_uHash_GS2Node_StructureTypeInt:
		case g_uHash_GS2Node_BehaviourSetInt:
			// This is only ever set, not got, because of the way shared function dialogues work.
			return true;

		default:
			return PARENT::GetParameter( uParamID, pxParam );
	}
}

bool Tools_EntityNode_GS2Node::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	if ( GetGameGS2Node()->GetBehaviourParams() && GetGameGS2Node()->GetBehaviourParams()->DoSetParameter( uParamID, pxParam ) )
	{
		return true;
	}

	switch ( uParamID )
	{
		case g_uHash_GS2Node_TestForCollisions:
		{
			GetGameGS2Node()->SetShouldTestForCollisions( pxParam->GetBool() );
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Node_IgnoreCollisions:
		{
			GetGameGS2Node()->SetShouldIgnoreCollisions( pxParam->GetBool() );
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Node_FollowSurface:
		{
			GetGameGS2Node()->SetShouldFollowSurface( pxParam->GetBool() );
			GetGameGS2Node()->UpdateFollowSurface();
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Node_UseLinkWeights:
		{
			GetGameGS2Node()->SetShouldUseLinkWeights( pxParam->GetBool() );
			if ( IsSelected() )
			{
				RequestRefreshOverlay();
			}
			SetEntityModified();
			return true;
		}

		case g_uHash_GS2Node_StructureTypeString:
		case g_uHash_GS2Node_BehaviourSetString:
			// This is only ever got, not set, because of the way shared function dialogues work.
			return true;

		case g_uHash_GS2Node_StructureTypeInt:
		{
			GetGameGS2Node()->SetStructureType( pxParam->GetInt() );
			if ( IsSelected() )
			{
				RequestRefreshOverlay();
			}
			return true;
		}

		case g_uHash_GS2Node_BehaviourSetInt:
		{
			GetGameGS2Node()->SetBehaviourSet( pxParam->GetInt() );
			if ( GetGameGS2Node()->GetBehaviourParams() )
			{
				GetGameGS2Node()->GetBehaviourParams()->DoSetToolsNode( this );
			}
			if ( IsSelected() )
			{
				RequestRefreshOverlay();
			}
			SetEntityModified();
			return true;
		}

		case g_uHash_EntityNode_SetThisToDefaults:
		{
			GetGameGS2Node()->SetStructureType( s_iDefaultStructureType );
			GetGameGS2Node()->SetIsSmoothOverNode( s_bDefaultSmoothOverNode );
			GetGameGS2Node()->SetShouldFollowSurface( s_bDefaultFollowSurface );
			GetGameGS2Node()->SetShouldTestForCollisions( s_bDefaultTestForCollisions );
			GetGameGS2Node()->SetShouldIgnoreCollisions( s_bDefaultIgnoreCollisions );
			if ( GetGameGS2Node()->GetBehaviourSet() != s_iDefaultBehaviourSet )
			{
				GetGameGS2Node()->SetBehaviourSet( s_iDefaultBehaviourSet );
				if ( GetGameGS2Node()->GetBehaviourParams() )
				{
					GetGameGS2Node()->GetBehaviourParams()->DoSetToolsNode( this );
				}
			}
			if ( IsSelected() )
			{
				RequestRefreshOverlay();
			}
			SetEntityModified();
			return true;
		}

		default:
			return PARENT::SetParameter( uParamID, pxParam );
	}
}

void Tools_EntityNode_GS2Node::Copy( const Tools_Object* pxObject )
{
	// Copy parent data.
	PARENT::Copy( pxObject );

	// Copy our data.
	const Tools_EntityNode_GS2Node* pxFromObject = static_cast< const Tools_EntityNode_GS2Node* >( pxObject );

	// Copy the engine entity's data across.
    Asura_ServerNode_GS2Node* pxToEntity   = static_cast< Asura_ServerNode_GS2Node* >( Entity );
	Asura_ServerNode_GS2Node* pxFromEntity = static_cast< Asura_ServerNode_GS2Node* >( pxFromObject->Entity );

	pxToEntity->m_uFlags = pxFromEntity->m_uFlags;
	// Not copying the current actor guid as this is only set in game.
    pxToEntity->m_iStructureType    = pxFromEntity->m_iStructureType;
    pxToEntity->m_iBehaviourSet     = pxFromEntity->m_iBehaviourSet;
	delete pxToEntity->m_pxBehaviourParams;
	pxToEntity->m_pxBehaviourParams = pxFromEntity->m_pxBehaviourParams ? pxFromEntity->m_pxBehaviourParams->DoClone() : NULL;
	if ( pxToEntity->m_pxBehaviourParams )
	{
		pxToEntity->m_pxBehaviourParams->DoSetToolsNode( this );
	}
}

Tools_Object* Tools_EntityNode_GS2Node::CreateObjectOfThisType() const
{
	return CreateAndReturn();
}

void Tools_EntityNode_GS2Node::Create()
{
	CreateAndReturn();
}

Tools_EntityNode_GS2Node* Tools_EntityNode_GS2Node::CreateAndReturn()
{
	// Get a new GUID
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "GS2 Node", "GS2 Node", uGuid, GetNamePrefix() );
	if ( !pxToolsData ) { return NULL; }

	// Create the game specfic server object and set its data.
	Asura_ServerNode_GS2Node* pxGameNode = new Asura_ServerNode_GS2Node( uGuid );
	if ( !pxGameNode ) { return NULL; }

	pxGameNode->ToolsData = pxToolsData;

	// Create the game specific tools object and select it.
	Tools_EntityNode_GS2Node* pxToolsNode = new Tools_EntityNode_GS2Node( pxGameNode );
	if ( !pxToolsNode ) { return NULL; }

	// Set everything to the default values.
	pxToolsNode->SetThisToDefaults();

	Tools_Object_System::SelectObject( pxToolsNode );

	pxToolsNode->Position = *Asura_Camera::GetPosition();
	// Set the orient to the camera heading but vertical.
	Asura_Euler_3 xHeading = *Asura_Camera::GetOriEuler();
	xHeading.x = xHeading.z = 0.0f;
	xHeading.ToMatrix( pxToolsNode->Orientation );

	pxGameNode->SetPosition( pxToolsNode->Position );
	pxGameNode->SetOrientation( pxToolsNode->Orientation );

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new GS2 node." );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsNode;
}

bool Tools_EntityNode_GS2Node::PromptForOwnerSkinName()
{
	Asura_String xSkinFilename;
	Tools_Animation_Files::ChooseSkin( &m_xOwnerSkinName, &xSkinFilename );
	return ( !m_xOwnerSkinName.IsBlank() );
}

void Tools_EntityNode_GS2Node::AddFilesToExportList( List< Asura_String >& xNameList ) const
{
	PARENT::AddFilesToExportList( xNameList );

	if ( GetGameGS2Node()->GetBehaviourParams() )
	{
		GetGameGS2Node()->GetBehaviourParams()->DoAddFilesToExportList( xNameList );
	}
}

void Tools_EntityNode_GS2Node::AddFilesToExportListFromMessage( u_short usMessageID, void* pData, List< Asura_String >& xNameList ) const
{
	PARENT::AddFilesToExportListFromMessage( usMessageID, pData, xNameList );

	if ( GetGameGS2Node()->GetBehaviourParams() )
	{
		GetGameGS2Node()->GetBehaviourParams()->DoAddFilesToExportListFromMessage( usMessageID, pData, xNameList );
	}
}

bool Tools_EntityNode_GS2Node::Verify( const bool bAutomatic )
{
	bool bOk = PARENT::Verify( bAutomatic );

	// Check 'smooth over node' is used correctly.
	if ( GetGameNode()->IsSmoothOverNode() )
	{
		// Check we only have one outgoing link.
		if ( GetGameNode()->GetNumberLinks() != 1 )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"GS2 node %u has 'smooth over node' ticked. This option can only be used with exactly one outgoing link.", GetGuid() );
		}
		// Check we only have one incoming link.
		u_int uIncomingLinkCount = 0;
		for ( Asura_Hashed_List_ConstIt< Asura_ServerNode_GS2Spline > xSplineIter( Asura_ServerNode_GS2Spline::GetList() ); !xSplineIter.Done(); xSplineIter.Next() )
		{
			const Asura_ServerNode_GS2Spline* pxSpline = xSplineIter.GetCurrent();
			if ( pxSpline->GetTargetGuid() == GetGuid() )
			{
				++uIncomingLinkCount;
			}
			if ( uIncomingLinkCount > 1 )
			{
				break;
			}
		}
		if ( uIncomingLinkCount != 1 )
		{
			bOk = false;
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Gamescene", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"GS2 node %u has 'smooth over node' ticked. This option can only be used with exactly one incoming link.", GetGuid() );
		}
	}

	if ( GetGameGS2Node()->GetBehaviourParams() )
	{
		bOk &= GetGameGS2Node()->GetBehaviourParams()->DoVerify( bAutomatic );
	}

	return bOk;
}

void Tools_EntityNode_GS2Node::PrintInfo( float& fX, float& fY )
{
	PARENT::PrintInfo( fX, fY );

	if ( m_uOwnerGuid == ASURA_GUID_UNREGISTERED )
	{
		Asura_Text_Output::Print( fX, fY++, 0xffffffff, 0, "Owner Guid : *unset*" );
	}
	else
	{
		Asura_Text_Output::Print( fX, fY++, 0xffffffff, 0, "Owner Guid : %u", m_uOwnerGuid );
	}
	Asura_Text_Output::Print( fX, fY++, 0xffffffff, 0, "Owner Skin : %s", m_xOwnerSkinName.IsBlank() ? "*unset*" : m_xOwnerSkinName.GetString() );
}

void Tools_EntityNode_GS2Node::UpdateBehaviourTree()
{
	if ( GetGameGS2Node()->UpdateBehaviourTree() )
	{
		SetEntityModified();
	}
}

void Tools_EntityNode_GS2Node::AddPropertiesTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	if ( Type == TOT_Entity_GS2Node )
	{
		AddStructureTypeDialogue( pxTabsDialogue, xOffset );
		AddSmoothOverNodeOverlayControls( pxTabsDialogue, xOffset );
		AddFollowSurfaceDialogue( pxTabsDialogue, xOffset );
		AddCollisionsDialogues( pxTabsDialogue, xOffset );
		xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

		AddLinkWeightsDialogue( pxTabsDialogue, xOffset );
		xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();
	}
}

void Tools_EntityNode_GS2Node::AddBehaviourTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	ListSelection_Overlay xBehaviourList( "" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2NodeBehaviourSetList( xBehaviourList, this );

	Asura_Function_Dialog* pxFDialog = new Asura_Function_Dialog( "Behaviour: ", &g_xParam_GS2Node_BehaviourSetString, xOffset,
		PickBehaviourSet, reinterpret_cast< int >( this ) );
	pxFDialog->SetHelpText( "Selects the behaviour for the actor to use on this node." );
	pxTabsDialogue->AddDialog( pxFDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	if ( GetGameGS2Node()->GetBehaviourParams() )
	{
		GetGameGS2Node()->GetBehaviourParams()->DoAddOverlayControls( pxTabsDialogue, xOffset );
	}
}

void Tools_EntityNode_GS2Node::AddUnderTabsControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	if ( Type == TOT_Entity_GS2Node )
	{
		AddLinkControls( pxNewOverlay, xOffset );
		xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

		AddMainDefaultsOverlayControls( pxNewOverlay, xOffset );
		Tools_Object_Axon_Interface::AddPreviewControls( pxNewOverlay, xOffset );
	}
}

void Tools_EntityNode_GS2Node::SetDefaultsToThis()
{
    s_iDefaultStructureType     = GetGameGS2Node()->GetStructureType();
    s_bDefaultSmoothOverNode    = GetGameGS2Node()->IsSmoothOverNode();
    s_bDefaultFollowSurface     = GetGameGS2Node()->ShouldFollowSurface( true );
	s_bDefaultTestForCollisions = GetGameGS2Node()->ShouldTestForCollisions();
	s_bDefaultIgnoreCollisions  = GetGameGS2Node()->ShouldIgnoreCollisions();
    s_iDefaultBehaviourSet      = GetGameGS2Node()->GetBehaviourSet();
	s_bRefreshDefaultsOverlay = true;
}

void Tools_EntityNode_GS2Node::AddDefaultsOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	AddPropertiesDefaultControls( pxNewOverlay, xOffset );
	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	AddBehaviourDefaultControls( pxNewOverlay, xOffset );
}

void Tools_EntityNode_GS2Node::AddPropertiesDefaultControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	ListSelection_Overlay xStructureList( "" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2StructureList( xStructureList );
	Asura_String xStructureTypeString( "Structure Type: " );
	xStructureTypeString += xStructureList.GetItemNameByData( s_iDefaultStructureType );

	Asura_Dialog* pxDialog = new Asura_Function_Dialog( xStructureTypeString.GetString(), xOffset, PickDefaultStructureType, reinterpret_cast< int >( this ) );
	pxDialog->SetHelpText( "Sets the structure type for this gamescene structure." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Smooth Over Node", xOffset, &s_bDefaultSmoothOverNode );
	pxDialog->SetHelpText( "Smooths the path over this node. Only applicable if node has one incoming link in and one outgoing link and actor doesn't stop" );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Follow Surface", xOffset, &s_bDefaultFollowSurface );
	pxDialog->SetHelpText( "Whether the actor should use LOS tests when moving to stick closely to the surface." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Collide With Movable Objects", xOffset, &s_bDefaultTestForCollisions );
	pxDialog->SetHelpText( "If ticked the entity will be solid and will push movable objects out of the way. (It will always go through immovable barriers.)\n"
		"If unticked the entity will be ghostlike and pass through everything." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Ignore Blocked Path", xOffset, &s_bDefaultIgnoreCollisions );
	pxDialog->SetHelpText( "Whether the actor should ignore movable objects in its way or stop when blocked." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_EntityNode_GS2Node::AddBehaviourDefaultControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	ListSelection_Overlay xBehaviourList( "" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2NodeBehaviourSetList( xBehaviourList );
	Asura_String xBehaviourSetString = "Behaviour: ";
	xBehaviourSetString += xBehaviourList.GetItemNameByData( s_iDefaultBehaviourSet );

	Asura_Dialog* pxDialog = new Asura_Function_Dialog( xBehaviourSetString.GetString(), xOffset, PickDefaultBehaviourSet, reinterpret_cast< int >( this ) );
	pxDialog->SetHelpText( "Selects the behaviour for the actor to use on this node." );
	pxNewOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_EntityNode_GS2Node::AddCollisionsDialogues( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxTDialogue = new Asura_SharedTickbox_Dialog( "Collide With Movable Objects", xOffset, &g_xParam_GS2Node_TestForCollisions );
	pxTDialogue->SetHelpText( "If ticked the entity will be solid and will push movable objects out of the way. (It will always go through immovable barriers.)\n"
		"If unticked the entity will be ghostlike and pass through everything." );
	pxTabsDialogue->AddDialog( pxTDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxTDialogue = new Asura_SharedTickbox_Dialog( "Ignore Blocked Path", xOffset, &g_xParam_GS2Node_IgnoreCollisions );
	pxTDialogue->SetHelpText( "Whether the actor should ignore movable objects in its way or stop when blocked." );
	pxTabsDialogue->AddDialog( pxTDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_EntityNode_GS2Node::AddFollowSurfaceDialogue( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog = new Asura_SharedTickbox_Dialog( "Follow Surface", xOffset, &g_xParam_GS2Node_FollowSurface );
	pxNewDialog->SetHelpText( "Whether the actor should use LOS tests when moving to stick closely to the surface." );
	pxTabsDialogue->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_EntityNode_GS2Node::AddStructureTypeDialogue( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	Asura_Function_Dialog* pxFDialog = new Asura_Function_Dialog( "Node Type: ", &g_xParam_GS2Node_StructureTypeString, xOffset,
		PickStructureType, reinterpret_cast< int >( this ) );
	pxFDialog->SetHelpText( "Sets the node type for this GS2 node." );
	pxTabsDialogue->AddDialog( pxFDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_EntityNode_GS2Node::AddLinkWeightsDialogue( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset )
{
	const Asura_ServerNode_GS2Node* pxGameGS2Node = GetGameGS2Node();

	Asura_Dialog* pxDialogue = new Asura_SharedTickbox_Dialog( "Use Link Weights", xOffset, &g_xParam_GS2Node_UseLinkWeights );
	pxDialogue->SetHelpText( "Whether or not to use the link weights when deciding which link to take next." );
	pxDialogue->SetEnabled( pxGameGS2Node->GetNumberLinks() > 1 );
	pxTabsDialogue->AddDialog( pxDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( pxGameGS2Node->ShouldUseLinkWeights() )
	{
		// Create the weight table dialogue.
		CreateLinkNameList();

		m_pxWeightTableDialogue = new Asura_Table_Dialog( xOffset, 40.0f, 7.5f, true );
		Asura_Vector_2 xCellSpacing = m_pxWeightTableDialogue->GetCellSpacing();
		xCellSpacing.y = 0.25f;
		m_pxWeightTableDialogue->SetCellSpacing( xCellSpacing );

		const Asura_Vector_2 xCellSize( 5.0f, 1.0f );
		m_pxWeightTableDialogue->CreateTable( xCellSize, pxGameGS2Node->GetNumberLinks(), 1, CreateWeightTableCell, this, m_ppszLinkNameList );
		pxTabsDialogue->AddDialog( m_pxWeightTableDialogue );
		xOffset.y += m_pxWeightTableDialogue->GetTableSize().y + 0.5f;
	}
}

void Tools_EntityNode_GS2Node::AddLinkControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	Tools_Object_Axon_Interface* pxInterface = Tools_Object_Axon_Interface::GetInstance();
	if ( pxInterface )
	{
		Asura_Dialog* pxNewDialog = new Asura_Function_Dialog( "Create Linked Node", xOffset,
			CreateLinkedNode, NULL,
			Tools_Object_Axon_Interface::CanCreateGS2LinkedNode );
		pxNewDialog->SetHelpText( "Create a new gamescene link and target node from this node." );
		pxNewOverlay->AddDialog( pxNewDialog );
		xOffset.x += 15.0f;

		pxNewDialog = new Asura_Function_Dialog( "Create Link", xOffset, CreateLink, NULL,
			Tools_Object_Axon_Interface::CanCreateGS2LinkBetweenNodes );
		pxNewDialog->SetHelpText( "Create a new gamescene link between two nodes." );
		pxNewOverlay->AddDialog( pxNewDialog );
		xOffset.x -= 15.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void Tools_EntityNode_GS2Node::UpdatePreview()
{
	if ( m_bPreviewing )
	{
		bool bPreviewRunning = true;

		Axon_BehaviourParams* pxBehaviourParams = GetGameGS2Node()->GetBehaviourParams();
		if ( pxBehaviourParams )
		{
			bPreviewRunning = pxBehaviourParams->DoUpdatePreview();
		}
		else
		{
			bPreviewRunning = false;
		}

		// Propagate the preview if necessary.
		// Note that we should propagate before stopping ourselves so that the count of items previewing in Tools_Object_Axon_Interface
		// doesn't reach zero before the whole structure has previewed.
		if ( !bPreviewRunning )
		{
			if ( Tools_Object_Axon_Interface::ShouldPropagatePreview() )
			{
				for ( u_int uLink = 0; uLink < GetGameGS2Node()->GetNumberLinks(); ++uLink )
				{
					Tools_EntityNode_GS2Spline* pxToolsSpline = Tools_EntityNode_GS2Spline::FindEntity( GetGameGS2Node()->GetLink( uLink ) );
					if ( pxToolsSpline )
					{
						pxToolsSpline->StartPreview();
					}
				}
			}

			// Stop ourselves.
			StopPreview();
		}
	}
}

Asura_ServerNode_GS2Node* Tools_EntityNode_GS2Node::GetGameGS2Node() const
{
	return static_cast< Asura_ServerNode_GS2Node* >( Entity );
}

void Tools_EntityNode_GS2Node::CreateLinkNameList()
{
	DestroyLinkNameList();

	const Asura_ServerNode_GS2Node* pxGameGS2Node = GetGameGS2Node();
	m_uLinkNameListSize = pxGameGS2Node->GetNumberLinks();
	m_ppszLinkNameList = new char*[ m_uLinkNameListSize ];

	for ( u_int u = 0; u < m_uLinkNameListSize; ++u )
	{
		Tools_EntityNode_GS2Spline* pxToolsGS2Spline = Tools_EntityNode_GS2Spline::FindEntity( pxGameGS2Node->GetLink( u ) );
		if ( pxToolsGS2Spline )
		{
			const u_int uNameLengthIncNull = strlen( pxToolsGS2Spline->GetName() ) + 1;
			m_ppszLinkNameList[ u ] = new char[ uNameLengthIncNull ];
			Asura_PrintfToBfr( m_ppszLinkNameList[ u ], "%s", uNameLengthIncNull, true, pxToolsGS2Spline->GetName() );
		}
		else
		{
			static const char* s_pszErrorString = "ERROR - CANNOT FIND GS2 SPLINE";
			const u_int uNameLengthIncNull = strlen( s_pszErrorString ) + 1;
			m_ppszLinkNameList[ u ] = new char[ uNameLengthIncNull ];
			Asura_PrintfToBfr( m_ppszLinkNameList[ u ], "%s", uNameLengthIncNull, true, s_pszErrorString );
		}
	}
}

void Tools_EntityNode_GS2Node::DestroyLinkNameList()
{
	if ( m_ppszLinkNameList )
	{
		for ( u_int u = 0; u < m_uLinkNameListSize; ++u )
		{
			delete [] m_ppszLinkNameList[ u ];
			m_ppszLinkNameList[ u ] = NULL;
		}
		delete [] m_ppszLinkNameList;
		m_ppszLinkNameList  = NULL;
		m_uLinkNameListSize = 0;
	}
}

class Tools_TableCell_GS2LinkWeight : public Asura_TableCell_Dialog
{
	typedef Asura_TableCell_Dialog PARENT;

public:
	Tools_TableCell_GS2LinkWeight( Asura_Table_Dialog& xTableDialogue, const Asura_Vector_2& xOffset,
								   const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn,
								   float* pfDialogVar, bool* pbChanged )
	: PARENT( xTableDialogue, xOffset, xMinCellSize, uRow, uColumn )
	{
		Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
		Asura_Dialog* pxDialog = new Asura_Numerical_Dialog( "", xLocalOffset, pfDialogVar, 0.0f, 10.0f, 0.05f, "%0.2f", pbChanged );
		AddDialogToPanel( pxDialog );
		xLocalOffset.x += pxDialog->m_xMaxCoord.x;
	}

	virtual int	Compare( const Asura_TableCell_Dialog& xOther ) const { return 0; }
};

Asura_TableCell_Dialog* Tools_EntityNode_GS2Node::CreateWeightTableCell( Asura_Table_Dialog& xTableDialog,
																		 const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																		 const u_int uRow, const u_int uColumn, void* const pxUserData )
{
	Tools_EntityNode_GS2Node* pxThis		  = static_cast< Tools_EntityNode_GS2Node* >( pxUserData );
	Asura_ServerNode_GS2Node* pxGameGS2Node   = pxThis->GetGameGS2Node();

	return new Tools_TableCell_GS2LinkWeight( xTableDialog, xOffset, xMinCellSize,
											  uRow, uColumn, pxGameGS2Node->GetLinkWeightPointer( uRow ), &pxThis->Changed );
}

void Tools_EntityNode_GS2Node::PickStructureType( int iThis )
{
	Tools_EntityNode_GS2Node* pxThis = reinterpret_cast< Tools_EntityNode_GS2Node* >( iThis );

	ListSelection_Overlay xStructureList( "Pick Structure Type:" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2StructureList( xStructureList, pxThis );
	xStructureList.SetInitialSelection( pxThis->GetGameGS2Node()->GetStructureType() );
	if ( xStructureList.DoModal() )
	{
		// Setting this will cause SetParameter to be called with this parameter hash on all selected objects.
		g_xParam_GS2Node_StructureTypeInt.SetValue( xStructureList.GetSelectionData() );
	}
}

void Tools_EntityNode_GS2Node::PickBehaviourSet( int iThis )
{
	Tools_EntityNode_GS2Node* pxThis = reinterpret_cast< Tools_EntityNode_GS2Node* >( iThis );

	ListSelection_Overlay xBehaviourList( "Pick Behaviour To Use:" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2NodeBehaviourSetList( xBehaviourList, pxThis );
	if ( !xBehaviourList.DoModal() )
	{
		return;
	}

	// Warn if the behaviour is about to be reset.
	const int iNewBehaviourSet = xBehaviourList.GetSelectionData();
	if ( pxThis->IsSelected() && Tools_Object_System::SelectedObjectList.size() == 1 &&
		iNewBehaviourSet == pxThis->GetGameGS2Node()->GetBehaviourSet() && iNewBehaviourSet != AXON_BEHAVIOUR_TYPE_UNSET )
	{
		YesNo_Overlay xYNOverlay( "This will reset the current behaviour. Are you sure?" );
		if ( !xYNOverlay.DoModal() )
		{
			return;
		}
	}

	// Setting this will cause SetParameter to be called with this parameter hash on all selected objects.
	g_xParam_GS2Node_BehaviourSetInt.SetValue( iNewBehaviourSet );
}

void Tools_EntityNode_GS2Node::PickDefaultStructureType( int iThis )
{
	ListSelection_Overlay xStructureList( "Pick Structure Type:" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2StructureList( xStructureList );
	xStructureList.SetInitialSelection( s_iDefaultStructureType );
	if ( xStructureList.DoModal() )
	{
		s_iDefaultStructureType = xStructureList.GetSelectionData();
		s_bRefreshDefaultsOverlay = true;
	}
}

void Tools_EntityNode_GS2Node::PickDefaultBehaviourSet( int iThis )
{
	ListSelection_Overlay xBehaviourList( "Pick Behaviour To Use:" );
	Tools_Object_Axon_Interface::GetInstance()->BuildGS2NodeBehaviourSetList( xBehaviourList );
	xBehaviourList.SetInitialSelection( s_iDefaultBehaviourSet );
	if ( xBehaviourList.DoModal() )
	{
		s_iDefaultBehaviourSet = xBehaviourList.GetSelectionData();
		s_bRefreshDefaultsOverlay = true;
	}
}

void Tools_EntityNode_GS2Node::StartPreview()
{
	if ( !m_bPreviewing )
	{
		m_bPreviewing = true;
		if ( GetGameGS2Node()->GetBehaviourParams() )
		{
			GetGameGS2Node()->GetBehaviourParams()->DoStartPreview();
		}
		Tools_Object_Axon_Interface::IncrementNumPreviewing();
	}
}

void Tools_EntityNode_GS2Node::StopPreview()
{
	if ( m_bPreviewing )
	{
		m_bPreviewing = false;
		if ( GetGameGS2Node()->GetBehaviourParams() )
		{
			GetGameGS2Node()->GetBehaviourParams()->DoStopPreview();
		}

		// This must come last otherwise the looping doesn't work correctly.
		Tools_Object_Axon_Interface::DecrementNumPreviewing();
	}
}

void Tools_EntityNode_GS2Node::TransferToolsModeData( Tools_ModeDataFile& xFile )
{
	u_int uVersion = uCURRENT_TOOLS_DATA_VERSION;
	xFile.Transfer( uVersion );

	if ( uVersion == uCURRENT_TOOLS_DATA_VERSION )
	{
		xFile.Transfer( s_xDefaultsOverlayPosition );
		xFile.Transfer( s_iDefaultStructureType );
		xFile.Transfer( s_bDefaultSmoothOverNode );
		xFile.Transfer( s_bDefaultFollowSurface );
		xFile.Transfer( s_bDefaultTestForCollisions );
		xFile.Transfer( s_bDefaultIgnoreCollisions );
		xFile.Transfer( s_iDefaultBehaviourSet );
	}
	else if ( uVersion == 0 )
	{
		xFile.Transfer( s_xDefaultsOverlayPosition );
		xFile.Transfer( s_iDefaultStructureType );
		xFile.Transfer( s_bDefaultSmoothOverNode );
		xFile.Transfer( s_bDefaultFollowSurface );
		xFile.Transfer( s_bDefaultTestForCollisions );
		xFile.Transfer( s_iDefaultBehaviourSet );
	}

	//bool bHaveBehaviourParams = ( m_pxBehaviourParams != NULL );
	//xFile.Transfer( bHaveBehaviourParams );
	//if ( bHaveBehaviourParams )
	//{
	//	if ( xFile.GetMode() == TOOLSMODEDATAFILE_LOAD )
	//	{
	//		u_int uChunkSize;
	//		xFile.Transfer( uChunkSize );
	//		Asura_Chunk_Memory_Stream xChunkStream( uChunkSize );
	//		u_char ucWorking;
	//		for ( u_int u = 0; u < uChunkSize; ++u )
	//		{
	//			xFile.Transfer( ucWorking );
	//			xChunkStream << ucWorking;
	//		}

	//		Axon_BehaviourParams::CreateTreeFromChunkStream( xChunkStream, m_pxBehaviourParams );

	//		Axon_BehaviourParams* pxBehaviourParams = NULL;
	//		if ( Axon_BehaviourParams::ConvertTree( m_iBehaviourSet, m_pxBehaviourParams, pxBehaviourParams ) )
	//		{
	//			delete m_pxBehaviourParams;
	//			m_pxBehaviourParams = pxBehaviourParams;
	//		}
	//	}
	//	else
	//	{
	//		Asura_Chunk_Memory_Stream xChunkStream;
	//		Axon_BehaviourParams::WriteTreeToChunkStream( m_pxBehaviourParams, xChunkStream );

	//		u_int uChunkSize = xChunkStream.GetPosition();
	//		xFile.Transfer( uChunkSize );
	//		u_char ucWorking;
	//		for ( u_int u = 0; u < uChunkSize; ++u )
	//		{
	//			xChunkStream >> ucWorking;
	//			xFile.Transfer( ucWorking );
	//		}
	//	}
	//}
}

void Tools_EntityNode_GS2Node::CreateLinkedNode()
{
	if ( Tools_Object_Axon_Interface::GetInstance() )
	{
		Tools_Object_Axon_Interface::GetInstance()->CreateGS2LinkedNode();
	}
}

void Tools_EntityNode_GS2Node::CreateLink()
{
	if ( Tools_Object_Axon_Interface::GetInstance() )
	{
		Tools_Object_Axon_Interface::GetInstance()->CreateGS2LinkBetweenNodes();
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // ASURA_USE_AXON
 