//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_LiftNode.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Node class for lift splines
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___TOOLS_ENTITYNODE_LIFTNODE_H___
#define ___TOOLS_ENTITYNODE_LIFTNODE_H___ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Entity/Locations/OrientedNode/LiftNode/Asura_ServerNode_LiftNode.h"

// tools
#include "Entity/Locations/OrientedNode/Tools_EntityNode_OrientedNode.h"
#include "Objects/Tools_Object_System.h"

// F O R W A R D   D E C L A R T I O N S /////////////////////////////////////

struct Asura_StaticMessage;

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Lift_SelectLiftNode
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Lift_SelectLiftNode : public Tools_ChooseObject
{
public:
	Tools_Lift_SelectLiftNode( Asura_Guid* puGuidToSet ) : m_uIgnoreGuid( ASURA_GUID_UNREGISTERED ), m_puGuidToSet( puGuidToSet ) {}
	Tools_Lift_SelectLiftNode( Asura_Guid uIgnoreGuid, Asura_Guid* puGuidToSet ) : m_uIgnoreGuid( uIgnoreGuid ), m_puGuidToSet( puGuidToSet ) {}
	virtual bool CanBeSelected( Tools_Object* pxObject );
	virtual void ObjectSelected( Tools_Object* pxObject ) { pxObject?(*m_puGuidToSet = pxObject->GetGuid()):ASURA_GUID_UNREGISTERED; }
	virtual void OnFinish() { delete this; }

private:
	Asura_Guid  m_uIgnoreGuid;
	Asura_Guid* m_puGuidToSet;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Lift_SelectSplineEnd
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Lift_SelectSplineEnd : public Tools_ChooseObject
{
public:
	Tools_Lift_SelectSplineEnd( Asura_Guid* puGuidToSet ) : m_puGuidToSet( puGuidToSet ) {}
	virtual bool CanBeSelected( Tools_Object* pxObject );
	virtual void ObjectSelected( Tools_Object* pxObject ) { pxObject?(*m_puGuidToSet = pxObject->GetGuid()):ASURA_GUID_UNREGISTERED; }
	virtual void OnFinish() { delete this; }

private:
	Asura_Guid* m_puGuidToSet;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Lift_SelectGotoNode
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Lift_SelectGotoNode : public Tools_ChooseObject
{
public:
	Tools_Lift_SelectGotoNode( Asura_Guid* puGuidToSet ) : m_puGuidToSet( puGuidToSet ) {}
	virtual bool CanBeSelected( Tools_Object* pxObject );
	virtual void ObjectSelected( Tools_Object* pxObject ) { pxObject?(*m_puGuidToSet = pxObject->GetGuid()):ASURA_GUID_UNREGISTERED; }
	virtual void OnFinish() { delete this; }

private:
	Asura_Guid* m_puGuidToSet;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_LiftNode
//
//////////////////////////////////////////////////////////////////////////////

class Tools_EntityNode_LiftNode : public Tools_EntityNode_OrientedNode
{
    typedef Tools_EntityNode_OrientedNode PARENT;

public:
    Tools_EntityNode_LiftNode(Asura_Entity* pxEntity);

	static void AddMessageOverlayControls(Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
	static void SelectSplineEnd();
	//Set functions
    void SetOverlayModified( bool bModified ) { m_bChanged = bModified; }

    //Virtual Functions Overides
	virtual bool HandlesThisEvent(int iEventID) const;
	virtual bool HandlesThisMessage(int iMessageID);
    virtual void AddOverlayControls(Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset);
    virtual const char* GetTypeDescription()   { return "Lift Node"; }
    virtual bool IsBaseOrDerivativeOf( int iToolsObjectType ) const;

	virtual const char* GetMyNamePrefix() const { return Tools_EntityNode_LiftNode::GetNamePrefix(); }

	virtual void OnSelection();
    virtual void OnDeselection();
	virtual bool Verify( const bool bAutomatic );

    virtual void UpdateWhileSelected();

    virtual bool          CanBeCopied() const { return Type == TOT_Entity_LiftNode; }
    virtual void          Copy( const Tools_Object* pxObject );
    virtual Tools_Object* CreateObjectOfThisType() const;

	virtual bool GetParameter( Asura_Hash_ID uParamID, class Tools_Parameter* pxParam ) const;
	virtual bool SetParameter( Asura_Hash_ID uParamID, const class Tools_Parameter* pxParam );

    static void Create();
    static Tools_EntityNode_LiftNode* CreateAndReturn();

	static const char* GetNamePrefix() { return "LiftN_"; }

private:
	bool IsSelected() const { return (s_pxThisToolsLiftNode == this); }

    bool	   m_bChanged;
	bool	   m_bStopHere;
	bool	   m_bBlockWhenOccupied;
	bool	   m_bBlockWhenUnoccupied;
	Asura_Guid m_uSelectedSplineEndGuid;

    static Tools_EntityNode_LiftNode* s_pxThisToolsLiftNode;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif // #ifndef ___TOOLS_ENTITYNODE_LIFTNODE_H___
