#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_GS2Spline.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_ENTITYNODE_GS2SPLINE_H__
#define __TOOLS_ENTITYNODE_GS2SPLINE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// asura
#include "Core/Asura_HashedList.h"

// tools
#include "Entity/Locations/Spline/Tools_EntityNode_Spline.h"
#include "Objects/Tools_Object_System.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Overlay;
class Asura_ServerNode_GS2Spline;
template< typename T > class Asura_Graph_Dialog;
class Asura_Tabs_Dialog;
class Asura_Vector_2;
template< typename T > class List;
class Tools_Parameter;

//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_GS2Spline
//
//////////////////////////////////////////////////////////////////////////////

class Tools_EntityNode_GS2Spline : public Tools_EntityNode_Spline
{
	typedef Tools_EntityNode_Spline PARENT;

public:
	Tools_EntityNode_GS2Spline( Asura_Entity* pxEntity );
	virtual ~Tools_EntityNode_GS2Spline();

	const char* GetTypeDescription() { return "GS2 Spline"; }

	virtual const char* GetMyNamePrefix() const { return Tools_EntityNode_GS2Spline::GetNamePrefix(); }

	virtual bool IsBaseOrDerivativeOf( int iToolsObjectType ) const;
	virtual bool IsSelectable();
	virtual void OnSelection();
	virtual void OnDeselection();
	virtual void Update();
	virtual void UpdateWhileSelected();
	virtual void Render();
	virtual void AddOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset );
	virtual bool GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const;
	virtual bool SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam );
	virtual void AddFilesToExportList( List< Asura_String >& xNameList ) const;
	virtual void AddFilesToExportListFromMessage( u_short usMessageID, void* pData, List< Asura_String >& xNameList ) const;
	virtual void RefreshOverlay();
	virtual bool HandlesThisMessage( int iMessageID );
	virtual bool OwnsThisMessageBlock( unsigned short uMessageBlockID );
	virtual void UpdatePositionInAsuraData( bool bUpdateLinkedObjects );
	virtual bool Verify( const bool bAutomatic );
	virtual void PrintInfo( float& fX, float& fY );
	virtual void Copy( const Tools_Object* pxObject );

	virtual void OnSplinePointMoved( const int iIndex );

	virtual void CreateAllToolsPathObjects( const int iSelectIndexAfter = -1, const bool bShowEndPoints = true );

	void UpdateBehaviourTree();

	Asura_Guid			GetOwnerGuid() const { return m_uOwnerGuid; }
	void				SetOwnerGuid( Asura_Guid uOwnerGuid ) { m_uOwnerGuid = uOwnerGuid; }
	const Asura_String& GetOwnerSkinName() const { return m_xOwnerSkinName; }
	void				SetOwnerSkinName( const Asura_String& xSkinName ) { m_xOwnerSkinName = xSkinName; }
	bool				PromptForOwnerSkinName();

	void SetSourcePosOriFromTarget( const Asura_Vector_3& xPositionOffset, const Asura_Quat& xQOrientOffset );
	void SetTargetPosOriFromSource( const Asura_Vector_3& xPositionOffset, const Asura_Quat& xQOrientOffset );

	bool IsUsingDistanceVsTimeGraph() const { return m_bDistanceVsTimeGraph; }

	virtual void StartPreview();
	virtual void StopPreview();
	bool PreviewIsOccupied() const { return m_bPreviewIsOccupied; }
	void PreviewSetIsOccupied( const bool bOccupied ) { m_bPreviewIsOccupied = bOccupied; }

	// Turns on/off the path editing controls, for use with spline behaviours that don't require these.
	void SetAllowPathEditing( const bool bAllow )  { m_bAllowPathEditing = bAllow; }
	// Turns on/off the graph editing controls, for use with spline behaviours that don't require these.
	void SetAllowGraphEditing( const bool bAllow ) { m_bAllowGraphEditing = bAllow; }

	Asura_ServerNode_GS2Spline* GetGameGS2Spline() const;

	static void TransferToolsModeData( Tools_ModeDataFile& xFile );

	static Tools_EntityNode_GS2Spline* Create();

	static Asura_Hashed_List< Tools_EntityNode_GS2Spline >* GetList() { return &s_xList; }
	static Tools_EntityNode_GS2Spline* FindEntity( Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }

	static const char* GetNamePrefix() { return "GS2S_"; }

protected:
	virtual void AddPropertiesTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset );
	virtual void AddPathTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset );
	virtual void AddBehaviourTabControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset );
	virtual void AddUnderTabsControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset );

	virtual Asura_Vector_2* GetDefaultsOverlayPosition() { return &s_xDefaultsOverlayPosition; }
	virtual const char*     GetDefaultsOverlayTitle()    { return "GS2 Spline Defaults"; }
	virtual u_int           GetDefaultsOverlayID()       { return 0x006ec667; } // Asura_GetHashID( "GS2 Spline Defaults" )
	virtual void SetDefaultsToThis();
	virtual void AddDefaultsOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset );

	virtual Tools_ChooseObject* CreateSourceNodeChooseObject( Asura_Guid* pxGuid ) const;
	virtual Tools_ChooseObject* CreateTargetNodeChooseObject( Asura_Guid* pxGuid ) const;

	void AddCollisionsDialogues( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset );
	void AddSplinePathEditControls( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset );
	void AddFollowSurfaceDialogue( Asura_Tabs_Dialog* pxTabsDialogue, Asura_Vector_2& xOffset );
	void AddGraphControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset );
	void UpdateSpeedGraph();
	void UpdateGraphGridLines();

	void UpdatePreview();

	static void PickBehaviourSet();
	static void AutoSetStartSpeed();
	static bool CanAutoSetStartSpeed();
	static void AutoSetEndSpeed();
	static bool CanAutoSetEndSpeed();
	static void CopyStartSpeedToEnd();
	static void CopyEndSpeedToStart();

private:
	static void SetToDefaultOrientations();
	static void PickDefaultBehaviourSet( int iThis );

private:
	bool m_bChanged;
	bool m_bChangedSpeedGraph;
	bool m_bPreviewing;
	bool m_bPreviewIsOccupied;
	Asura_Guid m_uOwnerGuid;
	Asura_String m_xOwnerSkinName;
	Asura_Tabs_Dialog* m_pxTabsDialogue;
	Asura_Graph_Dialog< float >* m_pxGraphDialogue;

	bool m_bOverrideDuration;
	bool m_bDistanceVsTimeGraph;
	float m_fDurationOverride;
	float m_fLength;

	bool m_bAllowPathEditing;
	bool m_bAllowGraphEditing;

	static Tools_EntityNode_GS2Spline* s_pxThisToolsSpline;

	static Asura_Hashed_List< Tools_EntityNode_GS2Spline > s_xList;
	enum Tabs { TAB_PROPERTIES, TAB_PATH, TAB_BEHAVIOUR, TAB_END_ENUM };
	static const char* s_aszTabNames[ TAB_END_ENUM ];
	static u_int	   s_uCurrentTab;

	static const u_int uCURRENT_TOOLS_DATA_VERSION = 1;
	static Asura_Vector_2 s_xDefaultsOverlayPosition;
	static bool s_bRefreshDefaultsOverlay;
	static bool s_bDefaultFollowSurface;
	static bool s_bDefaultTestForCollisions;
	static bool s_bDefaultIgnoreCollisions;
	static int s_iDefaultBehaviourSet;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_ENTITYNODE_GS2SPLINE_H__
#endif // ASURA_USE_AXON
