//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_Spline.h
//
// Created by NJD - 13 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_ENTITYNODE_SPLINE_H__
#define __TOOLS_ENTITYNODE_SPLINE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura_List.h"

// tools
#include "Entity/Locations/Tools_Entity_Node.h"

#include "Entity/Locations/Spline/Asura_ServerNode_Spline.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_ServerNode_Spline;
class Asura_Tabs_Dialog;
class Asura_Vector_2;
class Tools_EntityNode_Spline;
class Tools_Entity_TemplateInstance_Object;
class Tools_Object_NodeSplinePoint;

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_SplineEventPoint
//
//////////////////////////////////////////////////////////////////////////////

class Tools_Object_SplineEventPoint : public Tools_Object
{
	typedef Tools_Object PARENT;

public:
	Tools_Object_SplineEventPoint( Asura_ServerNode_Spline* pxParentGameSpline, Tools_EntityNode_Spline* pxParentToolsSpline, u_int uIndex );

	virtual void UpdatePositionFromAsuraData();
	virtual Asura_Guid  GetGuid() const;
	virtual const char* GetName()			 { return m_xName.GetString(); }
	virtual const char* GetTypeDescription() { return "Spline Event"; }
	static const char* GetNamePrefix()		 { return "SE_"; }
	virtual const char* GetMyNamePrefix() const { return Tools_Object_SplineEventPoint::GetNamePrefix(); }
	virtual void Destroy();
	virtual void OnSelection();
	virtual void OnDeselection();
	virtual void Update();
	virtual void UpdateWhileSelected();
	virtual void AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset );
	virtual bool OwnsThisMessageBlock( unsigned short usMessageBlockID );
	virtual bool CanBeMoved() { return false; }
	virtual bool CanBeCopied() { return false; }
	virtual bool ShouldCreateObjectOverlay() { return true; }
	// Prevent warning message caused by selecting these along with the entities when exporting to a template.
	virtual bool SuppressCantExportToTemplateWarning() const { return true; }
	virtual bool SupportsMessageEditor() { return true; }

	virtual Asura_Entity_Server* GetServerEntity() { return m_pxParentGameSpline; }

	void UpdateRenderMeshScaling();

private:
	static void Rename( int iThis );
	static void EditMessages( int iThis );
	static void StartSnapToCursor( int iThis );
	static void EditLinkProperties( int iThis );

private:
	Asura_ServerNode_Spline* m_pxParentGameSpline;
	Tools_EntityNode_Spline* m_pxParentToolsSpline;
	const u_int				 m_uIndex;
	// Note that we keep a reference to our distance, message block and name in the Asura data, so it gets updated automatically.
	float&					 m_fDistance;
	u_short&				 m_usMessageBlock;
	u_int&                   m_uFlags;
	Asura_String&			 m_xName;
	float					 m_fSplineLength;
	bool					 m_bSnappingToCursor;
	bool					 m_bChanged;
};

//////////////////////////////////////////////////////////////////////////////
//
// Tools_EntityNode_Spline
//
//////////////////////////////////////////////////////////////////////////////

class Tools_EntityNode_Spline : public Tools_Entity_Node
{
    typedef Tools_Entity_Node PARENT;

public:
	Tools_EntityNode_Spline( Asura_Entity* pxEntity );

	const char* GetTypeDescription(){return "Spline";}
    virtual bool IsBaseOrDerivativeOf( int iToolsObjectType ) const;

	virtual void OnSelection();
	virtual void UpdateWhileSelected();
	virtual void Update();
	virtual void OnDeselection();
	virtual bool CanBeMoved();
	virtual void Destroy();
	virtual bool ShouldBeRendered();
	virtual void Render();
	virtual bool ObjectIsInLOS(const Asura_Vector_3& xAlpha, const Asura_Vector_3& xBeta, float& fLowest);
    virtual void AddChildObjectsToTemplate( Tools_Entity_TemplateInstance_Object* pxTemplateInstance );
	virtual void AddContextMenuItems( class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext );
	virtual bool Verify( const bool bAutomatic );

	virtual void OnSplinePointMoved( const int iIndex );

	void RemoveAllToolsPathObjects();
	void AddAllToolsPathObjectsToList( List< Tools_Object* >& xList );
	virtual void CreateAllToolsPathObjects( const int iSelectIndexAfter = -1, const bool bShowEndPoints = true );
	virtual void GetBoundingBoxForModLinkTest(Asura_Bounding_Box& xOutputBB) const;

	Asura_ServerNode_Spline* GetGameSpline() const;
	const Asura_Vector_3& GetSourceNodePosition() const { return m_xSourceNodePosition; }
	const Asura_Vector_3& GetTargetNodePosition() const { return m_xTargetNodePosition; }

	void RemoveFromEventList( Tools_Object_SplineEventPoint* pxPoint );
	void RequestRecreateSplineEventObjects() { m_bRecreateSplineEventObjects = true; }

	void SetShouldRenderPath( const bool bRender );

	virtual const char* GetMyNamePrefix() { return Tools_EntityNode_Spline::GetNamePrefix(); }

	static void Create();

	virtual bool CanBeCopied() const { return true; }
	// Splines cannot be pasted easily with the current system, as the start and end nodes cannot be pasted simultaneously and matched up.
	virtual bool CanBePasted() const { return false; }
	virtual bool CanBePastedOverAnother() const { return true; }

	const List<Tools_Object_NodeSplinePoint*>& GetSplinePointObjects() const {return m_xListAttachedPoints;}

	static float GetMeshScaleFactor( const Asura_Vector_3& xPosition );
	static void GetSplineRenderPosition( Asura_Vector_3& xPosition, const Asura_Guid uSource, const Asura_Guid uTarget );

	static const char* GetNamePrefix() { return "Spl_"; }

protected:
	template< typename T >
	void AddSplineSourceTargetControls( T* pxDialogueContainer, Asura_Vector_2& xOffset );
	template< typename T >
	void AddSplinePathEditControls( T* pxDialogueContainer, Asura_Vector_2& xOffset );
	template< typename T >
	void AddSplineOverlayControls( T* pxDialogueContainer, Asura_Vector_2& xOffset );
	template< typename T >
	void AddSplineEventOverlayControls( T* pxDialogueContainer, Asura_Vector_2& xOffset );

	void RenderPath(unsigned uColour);

	// Node manipulation functions...
	virtual void UpdateSplinePoint(const int iIndex, const Asura_Vector_3& xPosition, const Asura_Matrix_3x3& xOrientation);
	void UpdateToolsSpline(const Asura_Guid uSourceNode, const Asura_Guid uTargetNode);

	// Override these and return a choose object that chooses only the entities you consider to be valid start or end nodes.
	virtual Tools_ChooseObject* CreateSourceNodeChooseObject( Asura_Guid* pxGuid ) const { return NULL; }
	virtual Tools_ChooseObject* CreateTargetNodeChooseObject( Asura_Guid* pxGuid ) const { return NULL; }

	void UpdateRenderMeshScaling();
	void SetPointsToDefaultOrientations();

protected:
	// Rendering data.
	Asura_Vector_3	m_xSourceNodePosition;
	Asura_Vector_3	m_xTargetNodePosition;
	bool			m_bSourceNodeIsSmoothed;
	bool			m_bTargetNodeIsSmoothed;
	List< Tools_Object_NodeSplinePoint* > m_xListAttachedPoints; // Note that this list is currently NOT ordered.
	Asura_List< Tools_Object_SplineEventPoint > m_xEventPoints;

private:
	// Link editing.
	static void GetNewSourceNode( int iThis );
	static void GetNewTargetNode( int iThis );
	// Path editing.
	static void EditNodeLinkPathFromStart();
	static void EditNodeLinkPathFromEnd();

	void EstablishNewSourceNode();
	void EstablishNewTargetNode();

	static void AddSplineEvent( int iThis );
	void CreateAllSplineEventObjects( const u_int* puIndexToSelect = NULL );
	void DeleteAllSplineEventObjects();
	void DestroyAllSplineEventObjects();

	static void SelectSourceNode( int iThis );
	static void SelectTargetNode( int iThis );

private:
	Asura_Guid m_uNewSourceNode;
	Asura_Guid m_uNewTargetNode;
	bool m_bRecreateSplineEventObjects;
	bool m_bRenderSplinePath;

	// static data.
	static Tools_EntityNode_Spline* s_pxThisToolsSpline;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

template< typename T >
void Tools_EntityNode_Spline::AddSplineSourceTargetControls( T* pxDialogueContainer, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog;

	//Link to different node.
	pxNewDialog = new Asura_Dialog( "Set New:", xOffset );
	pxDialogueContainer->AddDialog(pxNewDialog);
	xOffset.x += 7.0f;

	pxNewDialog = new Asura_Function_Dialog( "Start Node", xOffset, GetNewSourceNode, reinterpret_cast< int >( this ) );
	pxNewDialog->SetHelpText( "Set a new node for this link to start from." );
	pxDialogueContainer->AddDialog(pxNewDialog);
	xOffset.x += 10.0f;

	//Link from different node.
	pxNewDialog = new Asura_Function_Dialog( "End Node", xOffset, GetNewTargetNode, reinterpret_cast< int >( this ) );
	pxNewDialog->SetHelpText( "Set a new node for this link to end at." );
	pxDialogueContainer->AddDialog(pxNewDialog);
	xOffset.x -= 17.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

template< typename T >
void Tools_EntityNode_Spline::AddSplinePathEditControls( T* pxDialogueContainer, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog;

	// Edit path dialog
	pxNewDialog = new Asura_Dialog( "Edit Path From:", xOffset );
	pxDialogueContainer->AddDialog( pxNewDialog );
	xOffset.x += 10.0f;

	pxNewDialog = new Asura_Function_Dialog( "Start", xOffset, EditNodeLinkPathFromStart );
	pxNewDialog->SetHelpText( "Edit the path from the start. This selects the first path point." );
	pxDialogueContainer->AddDialog( pxNewDialog );
	xOffset.x += 7.0f;

	// Edit path dialog
	pxNewDialog = new Asura_Function_Dialog( "End", xOffset, EditNodeLinkPathFromEnd );
	pxNewDialog->SetHelpText( "Edit the path from the end. This selects the last path point." );
	pxDialogueContainer->AddDialog( pxNewDialog );
	xOffset.x -= 17.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

template< typename T >
void Tools_EntityNode_Spline::AddSplineOverlayControls( T* pxDialogueContainer, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog;

	//Link to different node.
	pxNewDialog = new Asura_Dialog( "Set New:", xOffset );
	pxDialogueContainer->AddDialog(pxNewDialog);
	xOffset.x += 10.0f;

	pxNewDialog = new Asura_Function_Dialog( "Start Node", xOffset, GetNewSourceNode, reinterpret_cast< int >( this ) );
	pxNewDialog->SetHelpText( "Set a new node for this link to start from." );
	pxDialogueContainer->AddDialog(pxNewDialog);
	xOffset.x += 10.0f;

	//Link from different node.
	pxNewDialog = new Asura_Function_Dialog( "End Node", xOffset, GetNewTargetNode, reinterpret_cast< int >( this ) );
	pxNewDialog->SetHelpText( "Set a new node for this link to end at." );
	pxDialogueContainer->AddDialog(pxNewDialog);
	xOffset.x -= 20.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Edit path dialog
	pxNewDialog = new Asura_Dialog( "Edit Path From:", xOffset );
	pxDialogueContainer->AddDialog(pxNewDialog);
	xOffset.x += 10.0f;

	pxNewDialog = new Asura_Function_Dialog( "Start", xOffset, EditNodeLinkPathFromStart );
	pxNewDialog->SetHelpText("Edit the path from the start. This selects the first path point.");
	pxDialogueContainer->AddDialog(pxNewDialog);
	xOffset.x += 10.0f;

	// Edit path dialog
	pxNewDialog = new Asura_Function_Dialog( "End", xOffset, EditNodeLinkPathFromEnd );
	pxNewDialog->SetHelpText("Edit the path from the end. This selects the last path point.");
	pxDialogueContainer->AddDialog(pxNewDialog);
	xOffset.x -= 20.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

template< typename T >
void Tools_EntityNode_Spline::AddSplineEventOverlayControls( T* pxDialogueContainer, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog;

	pxNewDialog = new Asura_Function_Dialog( "Add Spline Event", xOffset, AddSplineEvent, reinterpret_cast< int >( this ) );
	pxNewDialog->SetHelpText( "Add a new message-sending event to this spline." );
	pxDialogueContainer->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_ENTITYNODE_SPLINE_H__
