//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_SplinePoint.cpp
//
// Created by NJD - 16 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Locations/Spline/Tools_Object_SplinePoint.h"

// asura
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Spline.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"

// tools
#include "Entity/Locations/Spline/Tools_EntityNode_Spline.h"
#include "Objects/Tools_Object_System.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_SplinePoint
//
//////////////////////////////////////////////////////////////////////////////


Tools_Object_SplinePoint* Tools_Object_SplinePoint::s_pxThisSplinePoint = NULL;

Tools_Object_SplinePoint::Tools_Object_SplinePoint(Asura_Spline<Asura_Vector_3>* pxParentSpline,
												   Tools_Object* pxParentObject,
												   int iIndex,
												   bool bIsEndPoint)
: PARENT( TOT_Spline_Point )
, m_pxPositionInAsura(NULL)
, m_iIndex(iIndex)
, m_bIsEndPoint(bIsEndPoint)
, m_bStraight(false)
, m_bChanged(false)
, m_szName(NULL)
, m_pxParentSpline(pxParentSpline)
, m_pxParentObject(pxParentObject)
{
    m_pxPositionInAsura = &pxParentSpline->GetPointPosition(m_iIndex);
    Position = *m_pxPositionInAsura;
    Orientation.Identity();
	m_bStraight = (pxParentSpline->GetSectionFlags(m_iIndex - 1) & Asura_Spline_Point_Flag_Straight) != 0;

    SetMesh( "ToolsShape_SplineNode" );

    char szText[255];
    const int iNumChars = Asura_PrintfToBfr(szText, "%s Point %d", 32, true, pxParentObject->GetName(), iIndex );
    m_szName = new char[ iNumChars + 1 ];
    strcpy(m_szName, szText);
}

Tools_Object_SplinePoint::~Tools_Object_SplinePoint()
{
    delete [] m_szName;
}

bool Tools_Object_SplinePoint::CanBeDestroyed()
{
	return ( m_bIsEndPoint ? false : PARENT::CanBeDestroyed() );
}

void Tools_Object_SplinePoint::UpdatePositionInAsuraData(bool /*updateLinkedObjects*/)
{
    PARENT::UpdatePositionInAsuraData(true);
    m_pxParentSpline->SetPointPosition( m_iIndex, Position );
}

void Tools_Object_SplinePoint::UpdatePositionFromAsuraData()
{
    PARENT::UpdatePositionFromAsuraData();
    Position = *m_pxPositionInAsura;
}

void Tools_Object_SplinePoint::DestroyToolsObjectOnly()
{
    PARENT::Destroy();
}

void Tools_Object_SplinePoint::Destroy()
{
    m_pxParentSpline->RemovePoint(m_iIndex);
    // Don't call PARENT::Destroy() since we expect child classes to do the cleanup.
}

void Tools_Object_SplinePoint::OnDeselection()
{
    s_pxThisSplinePoint = NULL;
    PARENT::OnDeselection();
}

void Tools_Object_SplinePoint::OnSelection()
{
	PARENT::OnSelection();
    s_pxThisSplinePoint = this;
}

void Tools_Object_SplinePoint::UpdateWhileSelected()
{
	if(m_bChanged)
	{
		if(m_bStraight)
		{
			m_pxParentSpline->AddSectionFlag(m_iIndex - 1, Asura_Spline_Point_Flag_Straight);
		}
		else
		{
			m_pxParentSpline->RemoveSectionFlag(m_iIndex - 1, Asura_Spline_Point_Flag_Straight);
		}
		SetEntityModified();
		m_bChanged = false;
	}
}

void Tools_Object_SplinePoint::AddOverlayControls(class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

    Asura_Dialog* pxNewDialog = NULL;
    pxNewDialog = new Asura_Function_Dialog("Add After", xOffset, AddAfter, NULL, CanAddAfter);
    pxNewDialog->SetHelpText("Add a new control point after the currently selected one");
    pxOverlay->AddDialog(pxNewDialog);
    xOffset.y += Asura_Dialog::DefaultSpacing();

    pxNewDialog = new Asura_Function_Dialog("Add Before", xOffset, AddBefore, NULL, CanAddBefore);
    pxNewDialog->SetHelpText("Add a new control point before the currently selected one");
    pxOverlay->AddDialog(pxNewDialog);
    xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_Tickbox_Dialog("Straight", xOffset, &m_bStraight, &m_bChanged);
	pxNewDialog->SetHelpText("Flags the following section of spline as straight");
	pxOverlay->AddDialog(pxNewDialog);
    xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Object_SplinePoint::AddAfter()
{
    const Asura_Vector_3 xPosition = *Asura_Camera::GetPosition();
    s_pxThisSplinePoint->CreateNew(xPosition, s_pxThisSplinePoint->m_iIndex);
}

void Tools_Object_SplinePoint::AddBefore()
{
    const Asura_Vector_3 xPosition = *Asura_Camera::GetPosition();
    Asura_Assert(s_pxThisSplinePoint->m_iIndex > 1, "Sanity check");
    s_pxThisSplinePoint->CreateNew(xPosition, s_pxThisSplinePoint->m_iIndex - 1);
}

bool Tools_Object_SplinePoint::CanAddAfter()
{
    if ( !s_pxThisSplinePoint )
    {
        return false;
    }
    if ( s_pxThisSplinePoint->m_bIsEndPoint && s_pxThisSplinePoint->m_iIndex != 1 )
    {
        return false;
    }
    return true;
}

bool Tools_Object_SplinePoint::CanAddBefore()
{
    if ( !s_pxThisSplinePoint )
    {
        return false;
    }
    if ( s_pxThisSplinePoint->m_bIsEndPoint && s_pxThisSplinePoint->m_iIndex == 1 )
    {
        return false;
    }
    return true;
}

void Tools_Object_SplinePoint::CreateNew(const Asura_Vector_3& xPosition, const int iIndexAfter)
{
    m_pxParentSpline->AddPoint(iIndexAfter, xPosition);
}

bool Tools_Object_SplinePoint::IsHidden()
{
	//Hide if the parent node is hidden
	if(m_pxParentObject && m_pxParentObject->IsHidden()) return true;

	return PARENT::IsHidden();	
}

void Tools_Object_SplinePoint::SetPosition( const Asura_Vector_3& xPosition )
{
	Position = xPosition;
	UpdatePositionInAsuraData( true );
}

void Tools_Object_SplinePoint::SetOrientation( const Asura_Matrix_3x3& xOrientM )
{
	Orientation = xOrientM;
	UpdatePositionInAsuraData( true );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
