//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_Node.cpp
//
// Created by NJD - 11 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/Locations/Tools_Entity_Node.h"

// asura
#include "Camera/Asura_Camera.h"
#include "Core/Asura_UserInput.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Locations/Asura_ServerEntity_Node.h"
#include "Entity/Locations/Spline/Asura_ServerNode_Spline.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Colour.h"

//tools
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Locations/Spline/Tools_EntityNode_Spline.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Interface/Tools_ColourPicker.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_ObjectSystem_Param.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_Node
//
//////////////////////////////////////////////////////////////////////////////

TOOLS_DECLARE_SHARED_PARAM( bool,  0xd5ea579e, EntityNode_SmoothOverNode ); // Asura_GetHashID( "EntityNode_SmoothOverNode" )
TOOLS_DECLARE_SHARED_PARAM( float, 0xedeaffeb, EntityNode_ColourRed );      // Asura_GetHashID( "EntityNode_ColourRed" )
TOOLS_DECLARE_SHARED_PARAM( float, 0x1e95a7dd, EntityNode_ColourGreen );    // Asura_GetHashID( "EntityNode_ColourGreen" )
TOOLS_DECLARE_SHARED_PARAM( float, 0xcf6dd440, EntityNode_ColourBlue );     // Asura_GetHashID( "EntityNode_ColourBlue" )
TOOLS_DECLARE_SHARED_PARAM( Asura_String, uTOOLS_SHAREDPARAMHASH_ENTITYNODE_SETTHISTODEFAULTS, EntityNode_SetThisToDefaults );

Asura_Hashed_List< Tools_Entity_Node > Tools_Entity_Node::s_xList;

Tools_Entity_Node::Tools_Entity_Node( Asura_Entity* pxEntity )
: PARENT( pxEntity )
, m_bShouldRefreshOverlay( false )
, m_uTraverseHelper( 0 )
, m_pxPreviousMesh( NULL )
{
	Type = TOT_Entity_Node;

	Position = GetGameNode()->GetPosition();

    SetColour( GetGameNode()->GetToolsObjectColour() );

	s_xList.AddToList( this );
}

Tools_Entity_Node::~Tools_Entity_Node() 
{
	s_xList.RemoveFromList( this );
}

void Tools_Entity_Node::Destroy()
{
	// Destroy any splines that were linked to this node
	while ( GetGameNode()->GetNumberLinks() > 0 )
	{
		bool bDeleted = false;

		Asura_ServerNode_Spline* pxGameSpline = static_cast< Asura_ServerNode_Spline* >( Asura_ServerEntity_Node::GetList()->FindEntity( GetGameNode()->GetLink( 0 ) ) );
		if ( pxGameSpline && pxGameSpline->IsBaseOrDerivativeOf( AsuraEntityClass_Spline ) )
		{
			if ( pxGameSpline->GetSourceGuid() == GetGuid() )
			{
				Asura_ServerEntity_Node* pxSourceNode = Asura_ServerEntity_Node::GetList()->FindEntity( pxGameSpline->GetSourceGuid() );
				if ( pxSourceNode )
				{
					pxGameSpline->ToolsData->ToolsObject->Destroy();
					bDeleted = true;
				}
			}
		}

		if ( !bDeleted )
		{
			GetGameNode()->RemoveSpline( GetGameNode()->GetLink( 0 ) );
		}
	}

	// Destroy any splines that link to this node
	for ( Asura_Hashed_List_It< Asura_ServerEntity_Node > xIt( Asura_ServerEntity_Node::GetList() ); !xIt.Done(); xIt.Next() )
	{
		Asura_ServerEntity_Node* pxNode = xIt.GetCurrent();
		if ( !pxNode || !pxNode->IsBaseOrDerivativeOf( AsuraEntityClass_Spline ) )
		{
			continue;
		}
		
		Asura_ServerNode_Spline* pxSpline = static_cast<Asura_ServerNode_Spline*>( pxNode );
		if ( pxSpline->GetTargetGuid() == GetGuid() )
		{
			if ( pxSpline->ToolsData->ToolsObject )
			{
				pxSpline->ToolsData->ToolsObject->Destroy();
			}
		}
	}

	PARENT::Destroy();
}

bool Tools_Entity_Node::IsBaseOrDerivativeOf( int iToolsObjectType ) const
{
	if ( iToolsObjectType == TOT_Entity_Node )
	{
		return true;
	}
	else
	{
		return PARENT::IsBaseOrDerivativeOf( iToolsObjectType );
	}
}

void Tools_Entity_Node::AddContextMenuItems( Tools_ContextMenu& xMenu, const Tools_ContextMenu_Context& xContext )
{
	PARENT::AddContextMenuItems( xMenu, xContext );

	Tools_ContextMenu_Section* pxSection = xMenu.GetSection( TOOLS_CONTEXTMENU_SECTION_SELECTION );
	Tools_ContextMenu_Item* pxItem = NULL;

	const bool bOutgoingLink = ( GetGameNode()->GetNumberLinks() > 0 );
	if ( bOutgoingLink )
	{
		pxItem = new Tools_ContextMenu_Item_Function( "Select link out", SelectOutgoingLink, reinterpret_cast< int >( this ) );
		pxSection->AddItem( pxItem );
	}

	bool bIncomingLink = false;
	for ( Asura_Hashed_List_It< Tools_Entity_Node > xNodeIter( Tools_Entity_Node::GetList() ); !xNodeIter.Done(); xNodeIter.Next() )
	{
		const Tools_Entity_Node* pxNode = xNodeIter.GetCurrent();
		if ( pxNode->IsBaseOrDerivativeOf( TOT_Entity_Spline ) )
		{
			const Tools_EntityNode_Spline* pxToolsSpline = static_cast< const Tools_EntityNode_Spline* >( pxNode );
			if ( pxToolsSpline->GetGameSpline()->GetTargetGuid() == GetGuid() )
			{
				bIncomingLink = true;
				break;
			}
		}
	}
	if ( bIncomingLink )
	{
		pxItem = new Tools_ContextMenu_Item_Function( "Select link in", SelectIncomingLink, reinterpret_cast< int >( this ) );
		pxSection->AddItem( pxItem );
	}

	if ( bOutgoingLink || bIncomingLink )
	{
		AddSelectStructureContextControls( xMenu, xContext );
	}

	pxSection = xMenu.GetSection( TOOLS_CONTEXTMENU_SECTION_OBJECT );

	pxItem = new Tools_ContextMenu_Item_Function( "Colour", SetObjectColour, reinterpret_cast< int >( this ) );
	pxSection->AddItem( pxItem );

	pxItem = new Tools_ContextMenu_Item_Function( "Colour selected", SetAllObjectsColour );
	// Only allow this option if objects that are selected can be coloured.
	pxItem->SetEnabled( false );
	for(CLIF<Tools_Object*> olif(&Tools_Object_System::SelectedObjectList);!olif.done();olif.next())
	{
		if ( Tools_Entity_Node::FindEntity( olif()->GetGuid() ) )
		{
			pxItem->SetEnabled( true );
			break;
		}
	}
	pxSection->AddItem( pxItem );
}

void Tools_Entity_Node::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	PARENT::UpdatePositionInAsuraData( bUpdateLinkedObjects );

	GetGameNode()->SetPosition(Position);

	SetEntityModified();
}

void Tools_Entity_Node::RefreshOverlay()
{
	DeleteOverlay();
	CreateOverlay();
}

bool Tools_Entity_Node::Verify( const bool bAutomatic )
{
	bool bOK = true;
	for ( u_int u = 0; u < GetGameNode()->m_uNumberOfSplines;   )
	{
		bool bValidSpline = false;

		Asura_ServerNode_Spline* pxGameSpline = static_cast< Asura_ServerNode_Spline* >( Asura_ServerEntity_Node::FindEntity( GetGameNode()->m_puSplines[ u ] ) );
		if ( pxGameSpline )
		{
			if ( pxGameSpline->GetSourceGuid() == GetGuid() )
			{
				bValidSpline = true;
			}
			else
			{
				Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Objects", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
					"Node %d contains a reference to spline %d that doesn't start at that node.  This is being removed.", GetGuid(), GetGameNode()->m_puSplines[ u ] );
			}
		}
		else
		{
			Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Objects", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
				"Node %d contains a reference to spline %d that doesn't exist.  This is being removed.", GetGuid(), GetGameNode()->m_puSplines[ u ] );
		}

		if ( bValidSpline )
		{
			++u;
		}
		else
		{
			bOK = false;
			GetGameNode()->RemoveSpline( GetGameNode()->m_puSplines[ u ] );
			SetEntityModified();
		}
	}

	return PARENT::Verify( bAutomatic ) && bOK;
}

void Tools_Entity_Node::UpdateWhileSelected()
{
	PARENT::UpdateWhileSelected();

	if ( m_bShouldRefreshOverlay )
	{
		RefreshOverlay();
		m_bShouldRefreshOverlay = false;
	}

	if ( m_pxPreviousMesh == NULL )
	{
		if ( Asura_UserInput::KeyPressed( ASURA_KEYID_SPACE ) )
		{
			m_pxPreviousMesh = Mesh;
			SetMesh( "ToolsShape_CutsceneTrackPoint" );
		}
	}
	else
	{
		if ( !Asura_UserInput::KeyPressed( ASURA_KEYID_SPACE ) )
		{
			SetMesh( m_pxPreviousMesh );
			m_pxPreviousMesh = NULL;
		}
	}
}

bool Tools_Entity_Node::HandlesThisMessage( int iMessageID )
{
	if ( iMessageID == ASURA_MSG_ACTIVATE || iMessageID == ASURA_MSG_DEACTIVATE )
	{
		return true;
	}
	else
	{
		return PARENT::HandlesThisMessage( iMessageID );
	}
}

void Tools_Entity_Node::Copy( const Tools_Object* pxObject )
{
	// Copy parent data.
	PARENT::Copy( pxObject );

	// Copy our data.
	const Tools_Entity_Node* pxFromNodeObject = static_cast< const Tools_Entity_Node* >( pxObject );

	// Copy the engine entity's data across.
	const Asura_ServerEntity_Node* pxFromEntity = static_cast< const Asura_ServerEntity_Node* >( pxFromNodeObject->Entity );
	Asura_ServerEntity_Node*       pxToEntity   = static_cast< Asura_ServerEntity_Node* >( Entity );

	// Forget about changing the spline data, just copy the node stuff.
	// Also, we don't copy the position, as the copy is either to a new node, or over an existing node.
	pxToEntity->m_bIsOccupied       = pxFromEntity->m_bIsOccupied;
	pxToEntity->m_usSplittingPlane  = pxFromEntity->m_usSplittingPlane;
	pxToEntity->m_bIsSmoothOverNode = pxFromEntity->m_bIsSmoothOverNode;
	pxToEntity->m_bIsValid          = pxFromEntity->m_bIsValid;
}

Tools_Object* Tools_Entity_Node::CreateObjectOfThisType() const
{
	return CreateAndReturn();
}

void Tools_Entity_Node::Create()
{
	CreateAndReturn();
}

Tools_Entity_Node* Tools_Entity_Node::CreateAndReturn()
{
	// Get a new GUID
	const Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName( "Node", "Node", uGuid, GetNamePrefix() );
	if ( !pxToolsData ) { return NULL; }

	// Create the game specfic server object and set its data.
	Asura_ServerEntity_Node* pxGameNode = new Asura_ServerEntity_Node( uGuid );
	if ( !pxGameNode ) { return NULL; }

	pxGameNode->ToolsData = pxToolsData;

	// Create the game specific tools object and select it.
	Tools_Entity_Node* pxToolsNode = new Tools_Entity_Node( pxGameNode );
	if ( !pxToolsNode ) { return NULL; }

	Tools_Object_System::SelectObject( pxToolsNode );

	pxToolsNode->Position    = *Asura_Camera::GetPosition();
	pxToolsNode->Orientation = *Asura_Camera::GetOrientation();

	pxGameNode->SetPosition( pxToolsNode->Position );

	Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_DEFAULT, "Created new Node" );
	Tools_SaveAsura::SetEntityModified();

	return pxToolsNode;
}

bool Tools_Entity_Node::GetParameter( Asura_Hash_ID uParamID, Tools_Parameter* pxParam ) const
{
	switch ( uParamID )
	{
		case g_uHash_EntityNode_SmoothOverNode:
			pxParam->SetBool( GetGameNode()->IsSmoothOverNode() );
			return true;

		case g_uHash_EntityNode_ColourRed:
		{
			Asura_RGBA xRGBA( GetGameNode()->GetToolsObjectColour() );
			pxParam->SetFloat( xRGBA.m_fR );
			return true;
		}

		case g_uHash_EntityNode_ColourGreen:
		{
			Asura_RGBA xRGBA( GetGameNode()->GetToolsObjectColour() );
			pxParam->SetFloat( xRGBA.m_fG );
			return true;
		}

		case g_uHash_EntityNode_ColourBlue:
		{
			Asura_RGBA xRGBA( GetGameNode()->GetToolsObjectColour() );
			pxParam->SetFloat( xRGBA.m_fB );
			return true;
		}

		case g_uHash_EntityNode_SetThisToDefaults:
		{
			// No need to supply a string for this.
			pxParam->SetString( " " );
			return true;
		}

		default:
			return PARENT::GetParameter( uParamID, pxParam );
	}
}

bool Tools_Entity_Node::SetParameter( Asura_Hash_ID uParamID, const Tools_Parameter* pxParam )
{
	switch ( uParamID )
	{
		case g_uHash_EntityNode_SmoothOverNode:
		{
			GetGameNode()->SetIsSmoothOverNode( pxParam->GetBool() );
			UpdateAdjacentSplines();
			SetEntityModified();
			return true;
		}

		case g_uHash_EntityNode_ColourRed:
		{
			Asura_RGBA xRGBA( GetGameNode()->GetToolsObjectColour() );
			xRGBA.m_fR = pxParam->GetFloat();
			const u_int uColour = xRGBA.ToUIntARGB();
			GetGameNode()->SetToolsObjectColour( uColour );
			SetColour( uColour );
			SetEntityModified();
			return true;
		}

		case g_uHash_EntityNode_ColourGreen:
		{
			Asura_RGBA xRGBA( GetGameNode()->GetToolsObjectColour() );
			xRGBA.m_fG = pxParam->GetFloat();
			const u_int uColour = xRGBA.ToUIntARGB();
			GetGameNode()->SetToolsObjectColour( uColour );
			SetColour( uColour );
			SetEntityModified();
			return true;
		}

		case g_uHash_EntityNode_ColourBlue:
		{
			Asura_RGBA xRGBA( GetGameNode()->GetToolsObjectColour() );
			xRGBA.m_fB = pxParam->GetFloat();
			const u_int uColour = xRGBA.ToUIntARGB();
			GetGameNode()->SetToolsObjectColour( uColour );
			SetColour( uColour );
			SetEntityModified();
			return true;
		}

		case g_uHash_EntityNode_SetThisToDefaults:
		{
			// Handle this case here in case any derived type doesn't have defaults.
			return true;
		}

		default:
			return PARENT::SetParameter( uParamID, pxParam );
	}
}

Asura_ServerEntity_Node* Tools_Entity_Node::GetGameNode() const
{
	return static_cast< Asura_ServerEntity_Node* >( Entity );
}

void Tools_Entity_Node::AddMainDefaultsOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog = new Asura_Dialog( "Defaults:", xOffset );
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.x += 6.0f;

	pxNewDialog = new Asura_Function_Dialog( "Set This To", &g_xParam_EntityNode_SetThisToDefaults, xOffset, StaticSetThisToDefaults );
	pxNewDialog->SetHelpText( "Sets the properties of this node to the defaults." );
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.x += 10.0f;

	pxNewDialog = new Asura_Function_Dialog( "Set To This", xOffset, StaticSetDefaultsToThis, reinterpret_cast< int >( this ) );
	pxNewDialog->SetHelpText( "Sets the default properties to this node's." );
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.x += 10.0f;

	pxNewDialog = new Asura_Function_Dialog( "Edit", xOffset, EditDefaults, reinterpret_cast< int >( this ) );
	pxNewDialog->SetHelpText( "Edits/shows the default properties for this node." );
	pxNewOverlay->AddDialog( pxNewDialog );
	xOffset.x -= 26.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_Node::AddMainDefaultsOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog = new Asura_Dialog( "Defaults:", xOffset );
	pxTabsDialog->AddDialog( pxNewDialog );
	xOffset.x += 6.0f;
		
	pxNewDialog = new Asura_Function_Dialog( "Set This To", &g_xParam_EntityNode_SetThisToDefaults, xOffset, StaticSetThisToDefaults );
	pxNewDialog->SetHelpText( "Sets the properties of this node to the defaults." );
	pxTabsDialog->AddDialog( pxNewDialog );
	xOffset.x += 10.0f;

	pxNewDialog = new Asura_Function_Dialog( "Set To This", xOffset, StaticSetDefaultsToThis, reinterpret_cast< int >( this ) );
	pxNewDialog->SetHelpText( "Sets the default properties to this node's." );
	pxTabsDialog->AddDialog( pxNewDialog );
	xOffset.x += 10.0f;

	pxNewDialog = new Asura_Function_Dialog( "Edit", xOffset, EditDefaults, reinterpret_cast< int >( this ) );
	pxNewDialog->SetHelpText( "Edits/shows the default properties for this node." );
	pxTabsDialog->AddDialog( pxNewDialog );
	xOffset.x -= 26.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_Node::AddSelectStructureContextControls( Tools_ContextMenu& xMenu, const Tools_ContextMenu_Context& xContext )
{
	Tools_ContextMenu_Section* pxSection = xMenu.GetSection( TOOLS_CONTEXTMENU_SECTION_SELECTION );
	Tools_ContextMenu_Item* pxItem = new Tools_ContextMenu_Item_Function( "Select whole structure", SelectStructure, reinterpret_cast< int >( this ) );
	pxSection->AddItem( pxItem );
}

void Tools_Entity_Node::AddSmoothOverNodeOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog = new Asura_SharedTickbox_Dialog("Smooth Over Node", xOffset, &g_xParam_EntityNode_SmoothOverNode);
	pxNewDialog->SetHelpText("Smooths the path over this node. Only applicable if node has one incoming link in and one outgoing link and actor doesn't stop");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_Node::AddSmoothOverNodeOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialog = new Asura_SharedTickbox_Dialog("Smooth Over Node", xOffset, &g_xParam_EntityNode_SmoothOverNode);
	pxNewDialog->SetHelpText("Smooths the path over this node. Only applicable if node has one incoming link in and one outgoing link and actor doesn't stop");
	pxTabsDialog->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_Node::UpdateAdjacentSplines()
{
	for ( Asura_Hashed_List_It< Asura_ServerEntity_Node > xGameNodeIter( Asura_ServerEntity_Node::GetList() ); !xGameNodeIter.Done(); xGameNodeIter.Next() )
	{
		Asura_ServerEntity_Node* pxGameNode = xGameNodeIter.GetCurrent();
		if ( !pxGameNode->IsBaseOrDerivativeOf( AsuraEntityClass_Spline ) ) { continue; }

		Asura_ServerNode_Spline* pxGameSpline = static_cast< Asura_ServerNode_Spline* >( pxGameNode );
		if ( pxGameSpline->GetSourceGuid() == GetGuid() || pxGameSpline->GetTargetGuid() == GetGuid() )
		{
			pxGameSpline->OnChanged();
		}
	}
}

void Tools_Entity_Node::SelectOutgoingLink( int iThis )
{
	Tools_Entity_Node* pxToolsNode = reinterpret_cast< Tools_Entity_Node* >( iThis );
	const Asura_Guid uLinkGuid = pxToolsNode->GetGameNode()->GetLink( 0 );
	Tools_Object_System::SelectObject( Tools_Object_System::GetObjectFromGuid( uLinkGuid ) );
}

void Tools_Entity_Node::SelectIncomingLink( int iThis )
{
	Tools_Entity_Node* pxToolsNode = reinterpret_cast< Tools_Entity_Node* >( iThis );

	Asura_Guid uIncomingLink = ASURA_GUID_UNREGISTERED;
	for ( Asura_Hashed_List_It< Tools_Entity_Node > xNodeIter( Tools_Entity_Node::GetList() ); !xNodeIter.Done(); xNodeIter.Next() )
	{
		const Tools_Entity_Node* pxNode = xNodeIter.GetCurrent();
		if ( pxNode->IsBaseOrDerivativeOf( TOT_Entity_Spline ) )
		{
			const Tools_EntityNode_Spline* pxToolsSpline = static_cast< const Tools_EntityNode_Spline* >( pxNode );
			if ( pxToolsSpline->GetGameSpline()->GetTargetGuid() == pxToolsNode->GetGuid() )
			{
				uIncomingLink = pxToolsSpline->GetGameSpline()->GetGuid();
				break;
			}
		}
	}
	Tools_Object_System::SelectObject( Tools_Object_System::GetObjectFromGuid( uIncomingLink ) );
}

void Tools_Entity_Node::SelectStructure( int iThis )
{
	// Add the whole of this structure to the selection list.
	List< Tools_Object* > xObjectSelectionList;

	Tools_Entity_Node* pxStartNodeTools = reinterpret_cast< Tools_Entity_Node* >( iThis );
	Asura_ServerEntity_Node::TraverseStructure( pxStartNodeTools->GetGuid(), AddNodeToSelectionList, reinterpret_cast< void* >( &xObjectSelectionList ) );

	Tools_Object_System::SelectObjects( xObjectSelectionList, false );
}

void Tools_Entity_Node::AddNodeToSelectionList( Asura_ServerEntity_Node* pxNode, void* pUserData )
{
	List< Tools_Object* >* pxObjectSelectionList = reinterpret_cast< List< Tools_Object* >* >( pUserData );

	Tools_Entity_Node* pxNodeTools = Tools_Entity_Node::FindEntity( pxNode->GetGuid() );
	if ( !pxNodeTools ) { return; }

	if ( pxNodeTools->IsBaseOrDerivativeOf( TOT_Entity_Spline ) )
	{
		Tools_EntityNode_Spline* pxSplineNodeTools = static_cast< Tools_EntityNode_Spline* >( pxNodeTools );
		pxSplineNodeTools->AddAllToolsPathObjectsToList( *pxObjectSelectionList );
		pxObjectSelectionList->add_entry( pxSplineNodeTools );
	}
	else
	{
		pxObjectSelectionList->add_entry( pxNodeTools );
	}
}

void Tools_Entity_Node::SetObjectColour( int iThis )
{
	Tools_Entity_Node* pxThis = reinterpret_cast< Tools_Entity_Node* >( iThis );
	bool bChanged;
	Asura_RGBA xColour( pxThis->GetGameNode()->GetToolsObjectColour() );
	Tools_ColourPicker xColourPicker( "Pick Colour", &xColour.m_fR, &xColour.m_fG, &xColour.m_fB, 1.0f, &bChanged );
	if ( xColourPicker.DoModal() )
	{
		const u_int uColour = xColour.ToUIntARGB();
		pxThis->GetGameNode()->SetToolsObjectColour( uColour );
		pxThis->SetColour( uColour );
		pxThis->SetEntityModified();
	}
}

void Tools_Entity_Node::SetAllObjectsColour()
{
	bool bChanged;
	Tools_ColourPicker xColourPicker( "Pick Colour",
		&g_xParam_EntityNode_ColourRed, &g_xParam_EntityNode_ColourGreen, &g_xParam_EntityNode_ColourBlue, 1.0f, &bChanged );
	xColourPicker.DoModal();
}

void Tools_Entity_Node::SetThisToDefaults()
{
	Tools_ParameterDerived< Asura_String > xDummyParam( "dummy value" );
	SetParameter( g_uHash_EntityNode_SetThisToDefaults, &xDummyParam );
}

void Tools_Entity_Node::StaticSetThisToDefaults()
{
	// Setting this will cause SetParameter to be called with this parameter hash on all selected objects.
	g_xParam_EntityNode_SetThisToDefaults.SetValue( "dummy value" );
}

void Tools_Entity_Node::StaticSetDefaultsToThis( int iThis )
{
	Tools_Entity_Node* pxThis = reinterpret_cast< Tools_Entity_Node* >( iThis );
	pxThis->SetDefaultsToThis();
	if ( pxThis->DefaultsOverlayIsOpen() )
	{
		pxThis->CloseDefaultsOverlay();
		pxThis->OpenDefaultsOverlay();
	}
}

void Tools_Entity_Node::EditDefaults( int iThis )
{
	Tools_Entity_Node* pxThis = reinterpret_cast< Tools_Entity_Node* >( iThis );
	if ( pxThis->DefaultsOverlayIsOpen() )
	{
		pxThis->BringDefaultsOverlayToTop();
	}
	else
	{
		pxThis->OpenDefaultsOverlay();
	}
}

bool Tools_Entity_Node::DefaultsOverlayIsOpen()
{
	return Asura_Overlay_System::DoesOverlayExist( GetDefaultsOverlayID() );
}

void Tools_Entity_Node::OpenDefaultsOverlay()
{
	Asura_Vector_2* pxDefaultOverlayPosition = GetDefaultsOverlayPosition();
	if ( pxDefaultOverlayPosition )
	{
		Asura_Overlay* pxOverlay = new Asura_Overlay( GetDefaultsOverlayTitle(), false, pxDefaultOverlayPosition );
		Asura_Vector_2 xOffset = Asura_Vector_2( 1.0f, 3.0f );
		AddDefaultsOverlayControls( pxOverlay, xOffset );

		pxOverlay->m_uID = GetDefaultsOverlayID();
		Asura_Overlay_System::SubmitNewOverlay( pxOverlay, true );
	}
	else
	{
		Message_Overlay xOverlay( "There are no defaults for this object." );
		xOverlay.DoModal();
	}
}

void Tools_Entity_Node::BringDefaultsOverlayToTop()
{
	Asura_Overlay_System::BringOverlayToTop( GetDefaultsOverlayID() );
}

void Tools_Entity_Node::RefreshDefaultsOverlay()
{
	if ( DefaultsOverlayIsOpen() )
	{
		CloseDefaultsOverlay();
		OpenDefaultsOverlay();
	}
}

void Tools_Entity_Node::CloseDefaultsOverlay()
{
	Asura_Overlay_System::CloseOverlays( GetDefaultsOverlayID() );
}

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////
