//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_Node.h
//
// Created by NJD - 11 October 2006
//
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_ENTITY_NODE_H__
#define __TOOLS_ENTITY_NODE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// tools
#include "Entity/Tools_Entity_Object.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_ServerEntity_Node;
class Tools_ContextMenu;
class Tools_ContextMenu_Context;
class Tools_Entity_Node;
class Tools_ModeDataFile;

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_Node
//
//////////////////////////////////////////////////////////////////////////////

const Asura_Hash_ID uTOOLS_SHAREDPARAMHASH_ENTITYNODE_SETTHISTODEFAULTS = 0x250d4273; // Asura_GetHashID( "EntityNode_SetThisToDefaults" )

class Tools_Entity_Node : public Tools_Entity_Object
{
    typedef Tools_Entity_Object PARENT;

public:
    Tools_Entity_Node( Asura_Entity* pxEntity );
    virtual ~Tools_Entity_Node();

	virtual void Destroy();
    virtual bool HandlesThisMessage( int iMessageID );

    const char* GetTypeDescription() { return "Node"; }
    virtual bool IsBaseOrDerivativeOf( int iToolsObjectType ) const;

	virtual const char* GetMyNamePrefix() const { return Tools_Entity_Node::GetNamePrefix(); }

	virtual void AddContextMenuItems( Tools_ContextMenu& xMenu, const Tools_ContextMenu_Context& xContext );
    virtual void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);
    virtual void UpdateWhileSelected();
	virtual void RefreshOverlay();
	virtual bool Verify( const bool bAutomatic );

    virtual bool          CanBeCopied() const { return Type == TOT_Entity_Node; }
    virtual void          Copy( const Tools_Object* pxObject );
    virtual Tools_Object* CreateObjectOfThisType() const;

	virtual bool GetParameter(Asura_Hash_ID uParamID, Tools_Parameter* pxParam) const;
	virtual bool SetParameter(Asura_Hash_ID uParamID, const Tools_Parameter* pxParam);

	// Used for refreshing the overlay on the next update, if we're in a situation where we can't refresh the overlay immediately.
	void RequestRefreshOverlay() { m_bShouldRefreshOverlay = true; }

	void UpdateAdjacentSplines();

	Asura_ServerEntity_Node* GetGameNode() const;

    static void Create();
	static Tools_Entity_Node* FindEntity( const Asura_Guid uGuid ) { return s_xList.FindEntity( uGuid ); }
	static Asura_Hashed_List< Tools_Entity_Node >* GetList() { return &s_xList; }

	static const char* GetNamePrefix() { return "Node_"; }

protected:
	// Adds the main 'edit defaults' etc controls to an overlay.
	void AddMainDefaultsOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset );
	void AddMainDefaultsOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset );
	//## remove me once the old gamescene stuff is gone
	virtual void SetPropertiesToUserDefaults() {}

	void AddSelectStructureContextControls( Tools_ContextMenu& xMenu, const Tools_ContextMenu_Context& xContext );

	void OpenDefaultsOverlay();
	void BringDefaultsOverlayToTop();
	bool DefaultsOverlayIsOpen();
	void CloseDefaultsOverlay();
	void RefreshDefaultsOverlay();

	// If you return NULL for this you'll effectively disable the defaults stuff.
	virtual Asura_Vector_2* GetDefaultsOverlayPosition() { return NULL; }
	virtual const char*     GetDefaultsOverlayTitle()    { return ""; }
	virtual u_int           GetDefaultsOverlayID()       { return 0; }
	void SetThisToDefaults();
	virtual void SetDefaultsToThis() {}
	virtual void AddDefaultsOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset ) {}

	void AddSmoothOverNodeOverlayControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset );
	void AddSmoothOverNodeOverlayControls( Asura_Tabs_Dialog* pxTabsDialog, Asura_Vector_2& xOffset );

private:
	static Tools_Entity_Node* CreateAndReturn();
	static void SelectOutgoingLink( int iThis );
	static void SelectIncomingLink( int iThis );
	static void SelectStructure( int iThis );
	static void AddNodeToSelectionList( Asura_ServerEntity_Node* pxNode, void* pUserData );
	static void SetObjectColour( int iThis );
	static void SetAllObjectsColour();

	static void StaticSetThisToDefaults();
	static void StaticSetDefaultsToThis( int iThis );
	static void EditDefaults( int iThis );

private:
	bool        m_bShouldRefreshOverlay;
	u_int		m_uTraverseHelper;
    // Used for storing the previous mesh when the user presses SPACE.
    Asura_Mesh* m_pxPreviousMesh;

	static Asura_Hashed_List< Tools_Entity_Node > s_xList;
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_ENTITY_NODE_H__
