//
// Tools_Event_DialogueTrigger.cpp
//
// Created by Richard Rice - 07/11/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Entity/Sound/Tools_Entity_DialogueTrigger.h"

#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Entity/Triggers/Asura_ServerEntity_DialogueTrigger.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Asura_Entity_Event.h"
#include "Entity/Sound/Asura_ServerEntity_DialogueController.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Text_Output.h"
#include "Sound/Asura_Dialogue.h"
#include "Sound/Asura_Dialogue_System.h"

#include "Entity/Sound/Tools_Entity_DialogueController.h"
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_MessageEditor.h"
#include "Entity/Tools_Message_Editing.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/Tools_Object_System.h"
#include "Sound/Tools_PickDialogueEvent_Overlay.h"
#include "Sound/Tools_Dialogue_System.h"

//////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_ChooseObject_CharacterMap **
**---------------------------------*/

class Tools_ChooseObject_CharacterMap : public Tools_ChooseObject
{
public :
	Tools_ChooseObject_CharacterMap(Tools_Entity_DialogueTrigger* pxDialogueEntity)
	{
		m_pxDialogueEntity = pxDialogueEntity;
	}

	virtual bool CanBeSelected(Tools_Object* pxObject);
	virtual void ObjectSelected(Tools_Object* pxObject);
	virtual void OnFinish(){delete this;}

	Tools_Entity_DialogueTrigger* m_pxDialogueEntity;
};

bool Tools_ChooseObject_CharacterMap::CanBeSelected(Tools_Object* pxObject)
{
	//allow anything that has a guid for now
	return pxObject->GetGuid() != ASURA_GUID_UNREGISTERED;
}

void Tools_ChooseObject_CharacterMap::ObjectSelected(Tools_Object* pxObject)
{
	Asura_Guid uGuid = pxObject ? pxObject->GetGuid() : ASURA_GUID_UNREGISTERED;
	m_pxDialogueEntity->EditCharacterMap_GuidSelected(uGuid);
}


//////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Tools_Entity_DialogueTrigger **
**------------------------------*/

List<Asura_Dialogue_CharacterMapping> Tools_Entity_DialogueTrigger::s_xRecentCharacterMappings;

Tools_Entity_DialogueTrigger::Tools_Entity_DialogueTrigger(Asura_Entity* pxEntity)
:Tools_Entity_Activatable_Object(pxEntity)
{
	Type = TOT_Entity_DialogueTrigger;
	SetMesh("ToolsShape_Tannoy");

	m_iSelectedCharacter = 0;
	m_iNumCharacterMapStrings = 0;
	m_pszCharacterMapStrings = 0;

	//update character mapping , in case the event has been changed.
	UpdateCharacterMappings();
}

Tools_Entity_DialogueTrigger::~Tools_Entity_DialogueTrigger()
{
	DeleteCharacterMapStrings();
}


void Tools_Entity_DialogueTrigger::Create()
{
	Tools_Object* pxObject = Create(false,0);
	Tools_Object_System::SelectObject(pxObject);
}

Tools_Entity_DialogueTrigger* Tools_Entity_DialogueTrigger::Create(bool bGenerateName,Asura_Hash_ID uEventHash)
{
	if(!bGenerateName && !uEventHash)
	{
		//Ask which dialogue event to use
		uEventHash = Tools_PickDialogueEvent_Overlay::PickEvent("Select dialogue for trigger");
		if(!uEventHash) return 0;
	}

	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Dialogue Trigger","Dialogue Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		Asura_String xDefaultName;
		GenerateName(uEventHash,xDefaultName);

		pxToolsData = Tools_Entity_Object::CreateToolsData("Dialogue Trigger",xDefaultName.GetString(),GetNamePrefix());
	}
	if(!pxToolsData) return 0 ;


	//create the asura data
	Asura_ServerEntity_DialogueTrigger* pxAsuraEntity = new Asura_ServerEntity_DialogueTrigger();
	pxAsuraEntity->SetEventHashID(uEventHash);
	pxAsuraEntity->ToolsData = pxToolsData;

	//create the tools object
	Tools_Entity_DialogueTrigger* pxObject = new Tools_Entity_DialogueTrigger(pxAsuraEntity);

	Tools_SaveAsura::SetEntityModified();

	return pxObject;
}


Tools_Object* Tools_Entity_DialogueTrigger::CreateObjectOfThisType() const
{
	return Create(true,0);	
}

void Tools_Entity_DialogueTrigger::Destroy()
{
	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (Asura_ServerEntity_DialogueTrigger*)(Entity);

	Asura_StaticMessage_System::DeleteMessageBlock(pxDialogueEntity->m_usMessageBlock);
	pxDialogueEntity->m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
	
	Tools_Entity_Activatable_Object::Destroy();
}

void Tools_Entity_DialogueTrigger::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Activatable_Object::Copy(pxObject);

	const Tools_Entity_DialogueTrigger* pxFromObject = (const Tools_Entity_DialogueTrigger*) pxObject;

	Asura_ServerEntity_DialogueTrigger* pxFromEntity = (Asura_ServerEntity_DialogueTrigger*) pxFromObject->Entity;
	Asura_ServerEntity_DialogueTrigger* pxToEntity = (Asura_ServerEntity_DialogueTrigger*) Entity;

	pxToEntity->m_uEventHash = pxFromEntity->m_uEventHash;
	pxToEntity->m_pxEvent = pxFromEntity->m_pxEvent;
	
	pxToEntity->m_pxMappings->m_xMappings.Clear();
	pxToEntity->m_pxMappings->m_xMappings.SetSize(pxFromEntity->m_pxMappings->m_xMappings.Size());
	for(u_int i=0;i<pxFromEntity->m_pxMappings->m_xMappings.Size();i++)
	{
		pxToEntity->m_pxMappings->m_xMappings[i] = pxFromEntity->m_pxMappings->m_xMappings[i];
	}

	//copy the message block
	Asura_StaticMessage_System::DeleteMessageBlock(pxToEntity->m_usMessageBlock);
	pxToEntity->m_usMessageBlock = Asura_StaticMessage_System::CopyMessageBlock(pxFromEntity->m_usMessageBlock,pxFromObject->GetGuid(),GetGuid());

	
}

void Tools_Entity_DialogueTrigger::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Activatable_Object::AddOverlayControls(pxOverlay,xOffset);

	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (Asura_ServerEntity_DialogueTrigger*)(Entity);

	Asura_Dialog* pxDialog;
	char szText[200];

	pxDialog = new Asura_Function_Dialog("Dialogue",xOffset,ChooseDialogue,(int)this);
	pxDialog->SetHelpText("Select the dialogue event to trigger");
	pxDialog->m_iControlID = Asura_GetHashID("Dialogue");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr(szText,"Messages (%d)",sizeof(szText),true,Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxDialogueEntity->m_usMessageBlock));
	pxDialog = new Asura_Function_Dialog(szText,xOffset,EditMessages,(int)this);
	pxDialog->SetHelpText("A set of messages that may be triggered by the dialogue event.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//character mapping list
	pxDialog = new Asura_Dialog("Character Map :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if(!m_pszCharacterMapStrings) UpdateCharacterMapStrings();

	Asura_List_Dialog* pxListDialog;
	pxDialog = pxListDialog = new Asura_List_Dialog(xOffset,m_pszCharacterMapStrings,&m_iSelectedCharacter,6,m_iNumCharacterMapStrings,NULL);
	pxListDialog->SetDoubleClickFunction(EditCharacterMap,(int)this);
	pxDialog->SetHelpText("Map of characters in dialogue to guids/entities in level");
	pxDialog->m_iControlID = Asura_GetHashID("Character Map");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;

	pxDialog = new Asura_Function_Dialog("Edit Character Map",xOffset,EditCharacterMap,(int)this);
	pxDialog->SetHelpText("Edit mapping for selected entry in list");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Clear Character Map",xOffset,ClearCharacterMap,(int)this);
	pxDialog->SetHelpText("Clear mapping for selected entry in list");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Convert to dialogue controller",xOffset,ConvertToController,(int)this);
	pxDialog->SetHelpText("Convert this dialogue trigger to a dialogue controller entity");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Entity_DialogueTrigger::ChooseDialogue()
{
	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (Asura_ServerEntity_DialogueTrigger*)(Entity);

	Asura_Hash_ID uEventHash = Tools_PickDialogueEvent_Overlay::PickEvent("Select dialogue for trigger",pxDialogueEntity->m_uEventHash);
	if(uEventHash && uEventHash != pxDialogueEntity->m_uEventHash)
	{
		//update the entity with the new event
		pxDialogueEntity->m_uEventHash = uEventHash;
		pxDialogueEntity->m_pxEvent = Asura_Dialogue_System::FindEvent(uEventHash);

		UpdateCharacterMappings();

		UpdateName(pxDialogueEntity->m_uEventHash);

		UpdateOverlay();
		SetEntityModified();
	}
}

void Tools_Entity_DialogueTrigger::UpdateOverlayControls(Asura_Overlay* pxOverlay)
{
	Tools_Entity_Activatable_Object::UpdateOverlayControls(pxOverlay);


	//update the text on the choose dialogue control
	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (Asura_ServerEntity_DialogueTrigger*) Entity;
	char szText[100];
	if(pxDialogueEntity->m_pxEvent)
	{
		Asura_PrintfToBfr(szText,"Dialogue : %s",sizeof(szText),true,pxDialogueEntity->m_pxEvent->GetName());
	}
	else
	{
		Asura_PrintfToBfr(szText,"Dialogue : 0x%x",sizeof(szText),true,pxDialogueEntity->m_uEventHash);
	}

	pxOverlay->UpdateDialogLabel(Asura_GetHashID("Dialogue"),szText);

}

void Tools_Entity_DialogueTrigger::UpdateCharacterMappings()
{
	m_uCharacterMappingValidity = Tools_Dialogue_System::GetCharacterMappingValidity();
	
	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (Asura_ServerEntity_DialogueTrigger*) Entity;

	if(!pxDialogueEntity->m_pxEvent) return;

	//get a list of all characters used by the event
	List<Asura_Hash_ID> xCharacterList;

	List<const Asura_Dialogue_Event*> xEventQueue;
	xEventQueue.add_entry(pxDialogueEntity->m_pxEvent);

	for(LIF<const Asura_Dialogue_Event*> xEventLIF(&xEventQueue);!xEventLIF.done();xEventLIF.next())
	{
		const Asura_Dialogue_Event* pxEvent = xEventLIF();
		for(u_int i=0;i<pxEvent->GetNumPlayables();i++)
		{
			const Asura_Dialogue_Playable* pxPlayable = pxEvent->GetPlayable(i);
			if(pxPlayable->IsLine())
			{
				const Asura_Dialogue_Line* pxChildLine = (const Asura_Dialogue_Line*) pxPlayable;
				if(!xCharacterList.contains(pxChildLine->GetCharacter())) xCharacterList.add_entry(pxChildLine->GetCharacter());

			}
			else
			{
				const Asura_Dialogue_Event* pxChildEvent = (const Asura_Dialogue_Event*) pxPlayable;
				if(!xEventQueue.contains(pxChildEvent)) xEventQueue.add_entry(pxChildEvent);
			}
		}
	}

	//build a new character mapping list containing these characters
	Asura_Dialogue_CharacterMappingList* pxMappingList = new Asura_Dialogue_CharacterMappingList;
	pxMappingList->m_xMappings.SetSize(xCharacterList.size());

	Asura_Dialogue_CharacterMappingList* pxOldList = pxDialogueEntity->m_pxMappings;

	for(u_int i=0;i<pxMappingList->m_xMappings.Size();i++)
	{
		Asura_Hash_ID uCharacter = xCharacterList[i];
		pxMappingList->m_xMappings[i].m_uCharacterHashID = uCharacter;

		//see if there was a mapping for this character in the old list
		Asura_Guid uGuid = ASURA_GUID_UNREGISTERED;
		bool bNewEntry = true;
		for(u_int j=0;j<pxOldList->m_xMappings.Size();j++)
		{
			if(pxOldList->m_xMappings[j].m_uCharacterHashID == uCharacter)
			{
				uGuid = pxOldList->m_xMappings[j].m_uCharacterGuid;
				bNewEntry = false;
				break;
			}
		}

		if(bNewEntry)
		{
			uGuid = GetDefaultCharacterMapping(uCharacter);
		}
		pxMappingList->m_xMappings[i].m_uCharacterGuid = uGuid;
	}

	//replace the old list
	delete pxOldList;
	pxDialogueEntity->m_pxMappings = pxMappingList;

	if(IsSelected())
	{
		//Update the list on the overlay
		UpdateCharacterMapStrings();
	}
	else
	{
		//Old character map strings (if any) will be invalid , but don't need them yet
		DeleteCharacterMapStrings();
	}
}

void Tools_Entity_DialogueTrigger::AddToRecentCharacterMappings(const Asura_Dialogue_CharacterMapping& xMapping)
{
	//remove existing entry for this hash (if any)
	for(LIF<Asura_Dialogue_CharacterMapping> xMappingLIF(&s_xRecentCharacterMappings); !xMappingLIF.done(); xMappingLIF.next())
	{
		if(xMappingLIF().m_uCharacterHashID == xMapping.m_uCharacterHashID)
		{
			xMappingLIF.delete_current();
			break;
			
		}
	}
	//and add a new entry
	s_xRecentCharacterMappings.add_entry_start(xMapping);
}

Asura_Guid Tools_Entity_DialogueTrigger::GetDefaultCharacterMapping(Asura_Hash_ID uCharacterHash)
{
	//check for recently done mappings first
	for(LIF<Asura_Dialogue_CharacterMapping> xMappingLIF(&s_xRecentCharacterMappings); !xMappingLIF.done(); xMappingLIF.next())
	{
		if(xMappingLIF().m_uCharacterHashID == uCharacterHash)
		{
			return xMappingLIF().m_uCharacterGuid;
		}
	}

	//failing that , check the mappings of existing triggers in the level
	for(Asura_Hashed_List_It<Asura_ServerEntity_DialogueTrigger> xIt(Asura_ServerEntity_DialogueTrigger::GetList());!xIt.Done();xIt.Next())
	{
		Asura_ServerEntity_DialogueTrigger* pxEntity = xIt.GetCurrent();
		if(pxEntity->IsDestroyed()) continue;

		const Asura_Dialogue_CharacterMappingList* pxList = pxEntity->m_pxMappings;

		if(pxList->m_xMappings.Size())
		{
			for(u_int i=0;i<pxList->m_xMappings.Size();i++)
			{
				const Asura_Dialogue_CharacterMapping& xMapping = pxList->m_xMappings[i];
				if(xMapping.m_uCharacterHashID == uCharacterHash && xMapping.m_uCharacterGuid != ASURA_GUID_UNREGISTERED)
				{
					return xMapping.m_uCharacterGuid;
				}
			}
		}
	}

	return ASURA_GUID_UNREGISTERED;
}


void Tools_Entity_DialogueTrigger::UpdateCharacterMapStrings()
{
	DeleteCharacterMapStrings();

	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (Asura_ServerEntity_DialogueTrigger*) Entity;
	const Asura_Dialogue_CharacterMappingList* pxList = pxDialogueEntity->m_pxMappings;

	if(pxList->m_xMappings.Size())
	{
		m_iNumCharacterMapStrings = pxList->m_xMappings.Size();
		m_pszCharacterMapStrings = new char*[pxList->m_xMappings.Size()];

		for(u_int i=0;i<pxList->m_xMappings.Size();i++)
		{
			const Asura_Dialogue_CharacterMapping& xMapping = pxList->m_xMappings[i];
		
			//get the character name
			const char* szCharacter = Tools_Dialogue_System::GetCharacterNameByHash(xMapping.m_uCharacterHashID);

			char szCharacterBuffer[100];
			if(!szCharacter)
			{
				Asura_PrintfToBfr(szCharacterBuffer,"Character 0x%x",sizeof(szCharacterBuffer),true,xMapping.m_uCharacterHashID);
				szCharacter = szCharacterBuffer;
			}

			//get the name of the target object
			char szEntity[100];
			Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(xMapping.m_uCharacterGuid);
			if(pxObject)
			{
				Asura_PrintfToBfr(szEntity,"%s",sizeof(szEntity),true,pxObject->GetName());
			}
			else if(xMapping.m_uCharacterGuid == ASURA_GUID_UNREGISTERED)
			{
				strcpy(szEntity,"*Not Set*");
			}
			else
			{
				Asura_PrintfToBfr(szEntity,"Guid %d",sizeof(szEntity),true,xMapping.m_uCharacterGuid);
			}

			//build the full string for the list
			char szString[100];
			Asura_PrintfToBfr(szString,"%s - %s",sizeof(szString),true,szCharacter,szEntity);

			m_pszCharacterMapStrings[i] = new char[strlen(szString)+1];
			strcpy(m_pszCharacterMapStrings[i],szString);
		}
	}
	else
	{
		m_iNumCharacterMapStrings = 1;
		m_pszCharacterMapStrings = new char*[1];
		m_pszCharacterMapStrings[0] = new char[10];
		strcpy(m_pszCharacterMapStrings[0],"*None*");
	}
	
	if(m_iSelectedCharacter >= m_iNumCharacterMapStrings) m_iSelectedCharacter = m_iNumCharacterMapStrings-1;

	//update the overlay if needed
	if(IsSelected())
	{
		Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(GetOverlayID());
		if(pxOverlay)
		{
			Asura_List_Dialog* pxDialog = (Asura_List_Dialog*)pxOverlay->GetDialog(Asura_GetHashID("Character Map"));
			if(pxDialog)
			{
				pxDialog->ReplaceStrings(m_iNumCharacterMapStrings,m_pszCharacterMapStrings);
			}
		}
	}
}

void Tools_Entity_DialogueTrigger::DeleteCharacterMapStrings()
{
	for(int i=0;i<m_iNumCharacterMapStrings;i++)
	{
		delete [] m_pszCharacterMapStrings[i];
	}
	delete [] m_pszCharacterMapStrings;
	m_pszCharacterMapStrings = 0;

	m_iNumCharacterMapStrings = 0;

}

void Tools_Entity_DialogueTrigger::EditCharacterMap()
{
	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (Asura_ServerEntity_DialogueTrigger*) Entity;
	const Asura_Dialogue_CharacterMappingList* pxList = pxDialogueEntity->m_pxMappings;

	//get the selected mapping
	if(m_iSelectedCharacter >= (int)pxList->m_xMappings.Size()) return;
	const Asura_Dialogue_CharacterMapping& xMapping = pxList->m_xMappings[m_iSelectedCharacter];

	//sort out the prompt
	char szPrompt[200];
	const char* szCharacter = Tools_Dialogue_System::GetCharacterNameByHash(xMapping.m_uCharacterHashID);
	Asura_PrintfToBfr(szPrompt,"Select the entity to use for %s",sizeof(szPrompt),true,szCharacter ? szCharacter : "this character");

	//get the initial selection
	Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(xMapping.m_uCharacterGuid);

	
	//initialise and start the object choosing process
	Tools_ChooseObject_CharacterMap* pxChooseObject = new Tools_ChooseObject_CharacterMap(this);
	Tools_Object_System::ChooseObject(szPrompt,pxChooseObject,pxObject);
}

void Tools_Entity_DialogueTrigger::EditCharacterMap_GuidSelected(Asura_Guid uGuid)
{
	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (Asura_ServerEntity_DialogueTrigger*) Entity;
	Asura_Dialogue_CharacterMappingList* pxList = pxDialogueEntity->m_pxMappings;

	//update the character mapping in the entity
	if(m_iSelectedCharacter >= (int)pxList->m_xMappings.Size()) return;

	if(pxList->m_xMappings[m_iSelectedCharacter].m_uCharacterGuid != uGuid)
	{
		pxList->m_xMappings[m_iSelectedCharacter].m_uCharacterGuid = uGuid;
		SetEntityModified();
		UpdateCharacterMapStrings();

		AddToRecentCharacterMappings(pxList->m_xMappings[m_iSelectedCharacter]);

	}
}

void Tools_Entity_DialogueTrigger::ClearCharacterMap()
{	
	EditCharacterMap_GuidSelected(ASURA_GUID_UNREGISTERED);
}

void Tools_Entity_DialogueTrigger::EditMessages()
{
	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (Asura_ServerEntity_DialogueTrigger*) Entity;
	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(&pxDialogueEntity->m_usMessageBlock);
	#else
	Tools_Object_System::EditMessageBlock("Messages for event",&pxDialogueEntity->m_usMessageBlock);
	#endif
}

bool Tools_Entity_DialogueTrigger::HandlesThisEvent(int iEventID) const
{
	if(iEventID == ASURA_ENTITY_EVENT_ONFINISHED) return true;
	return Tools_Entity_Activatable_Object::HandlesThisEvent(iEventID);
}

void Tools_Entity_DialogueTrigger::AddFilesToExportList(List<Asura_String>& nameList) const
{
	const Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (const Asura_ServerEntity_DialogueTrigger*) Entity;

	Tools_Level_Export::AddDialogueEventToExportList(pxDialogueEntity->m_uEventHash);
}

void Tools_Entity_DialogueTrigger::UpdateName(Asura_Hash_ID uEventHash)
{
	Asura_String& xName = Entity->ToolsData->Name;

	//rename if this instance appears to be using a default name
	if(!strncmp(xName.GetString(),"Dialogue ",9) || !strncmp(xName.GetString(),"DLG ",4))
	{
		GenerateName(uEventHash,xName);
	}
}

void Tools_Entity_DialogueTrigger::GenerateName(Asura_Hash_ID uEventHash,Asura_String& xOutput)
{

 	const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(uEventHash);
	const char* szBaseName = pxEvent ? pxEvent->GetName() : "Unknown";

	int iBufferSize = strlen(szBaseName) + 50;//overkill really
	char* szBuffer = new char[iBufferSize];

	//search for an available name
	for(int i=0;;i++)
	{
		if(i)
			Asura_PrintfToBfr(szBuffer,"DLG %s_%d",iBufferSize,true,szBaseName,i);
		else
			Asura_PrintfToBfr(szBuffer,"DLG %s",iBufferSize,true,szBaseName);

		if(!Tools_Object_System::IsNameInUse(szBuffer)) break;
	}

	xOutput = szBuffer;
	delete [] szBuffer;
	return;
}

void Tools_Entity_DialogueTrigger::UpdateWhileSelected()
{
	Tools_Entity_Activatable_Object::UpdateWhileSelected();

	if(m_uCharacterMappingValidity != Tools_Dialogue_System::GetCharacterMappingValidity())
	{
		UpdateCharacterMappings();
	}
}

void Tools_Entity_DialogueTrigger::PrintInfo(float& fX,float& fY)
{
	Tools_Entity_Activatable_Object::PrintInfo(fX,fY);

	//Display some information about the targeted event
	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = (Asura_ServerEntity_DialogueTrigger*) Entity;
	const Asura_Dialogue_Event* pxEvent = pxDialogueEntity->m_pxEvent;
	if(!pxEvent) return;

	
	const char* szPlaybackType = 0;
	switch(pxEvent->GetPlaybackType())
	{
		case ASURA_DPT_ALLSEQUENTIAL :
			szPlaybackType = "All Sequential";
			break;
		case ASURA_DPT_ONEBYONE :
			szPlaybackType = "One By One";
			break;
		case ASURA_DPT_RANDOM :
			szPlaybackType = "Random";
			break;
		case ASURA_DPT_CDPLAYERRANDOM :
			szPlaybackType = "CD Player Random";
			break;
		default :
			Asura_Assert(0,"Unexpected dialogue event playback type");
			szPlaybackType = "Unknown";
			break;

	}

	Asura_Text_Output::Print(fX,fY++,0xffffffff,0,"Play Type : %s",szPlaybackType);
	Asura_Text_Output::Print(fX,fY++,0xffffffff,0,"Play Prob : %d%%",pxEvent->GetPlayProbability());
	Asura_Text_Output::Print(fX,fY++,0xffffffff,0,"Replay Delay : %.3f",pxEvent->GetReplayDelayTime());
	Asura_Text_Output::Print(fX,fY++,0xffffffff,0,"Num Lines/Events : %d",pxEvent->GetNumPlayables());
}

// Returns true if the message block belongs to this entity
bool Tools_Entity_DialogueTrigger::OwnsThisMessageBlock(unsigned short usMessageBlockID)
{
	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = static_cast<Asura_ServerEntity_DialogueTrigger*>(Entity);
	if (pxDialogueEntity && pxDialogueEntity->m_usMessageBlock == usMessageBlockID)
	{
		return true;
	}
	return Tools_Entity_Activatable_Object::OwnsThisMessageBlock(usMessageBlockID);
}

// Gets description of what causes message block to be sent, returns false if not implemented, or message isn't found.
bool Tools_Entity_DialogueTrigger::GetEventDescForMessageBlock(unsigned short usMessageBlockID,Asura_String& xOutputDesc)
{
	Asura_ServerEntity_DialogueTrigger* pxDialogueEntity = static_cast<Asura_ServerEntity_DialogueTrigger*>(Entity);
	if (pxDialogueEntity && pxDialogueEntity->m_usMessageBlock == usMessageBlockID)
	{
		xOutputDesc = "On Dialogue Trigger";
		return true;
	}
	return Tools_Entity_Activatable_Object::GetEventDescForMessageBlock(usMessageBlockID, xOutputDesc);
}


void Tools_Entity_DialogueTrigger::ConvertToController()
{
	if( !ShowYesNoOverlay( "Are you sure you want to convert this dialogue trigger to a dialogue controller?") )
	{
		return;
	}
	
	Tools_Entity_DialogueController* pxController = Tools_Entity_DialogueController::Create( true );

	if( !pxController )
	{
		return;
	}

	Asura_ServerEntity_DialogueController*	pxControllerEntity	= static_cast<Asura_ServerEntity_DialogueController*>( pxController->Entity );
	Asura_ServerEntity_DialogueTrigger*		pxTriggerEntity		= static_cast<Asura_ServerEntity_DialogueTrigger*>( Entity );

	// Set the event hash and copy messages
	pxControllerEntity->SetEventHashID( pxTriggerEntity->m_uEventHash, true );

	// Copy messages

	pxControllerEntity->CopyMessagesForEvent( pxTriggerEntity, ASURA_ENTITY_EVENT_ONFINISHED );

	// If anything targets this enitity with messages, move them over
	for( Tools_StaticMessageIterator xIterator(true);!xIterator.Done();xIterator.Next() )
	{
		Asura_StaticMessage* pxMessage = xIterator.GetCurrent();
		if( pxMessage->m_uSendTo == GetGuid() )
		{
			//Redirect to the new controller
			pxMessage->m_uSendTo = pxControllerEntity->GetGuid();
		}
	}

	// Copy mappings
	pxControllerEntity->m_xMappings.Clear();
	pxControllerEntity->m_xMappings.SetSize( pxTriggerEntity->m_pxMappings->m_xMappings.Size() );
	for( u_int uMapping = 0; uMapping < pxTriggerEntity->m_pxMappings->m_xMappings.Size(); uMapping++ )
	{
		pxControllerEntity->m_xMappings[uMapping] = pxTriggerEntity->m_pxMappings->m_xMappings[uMapping];
	}

	// Copy some properties

	pxControllerEntity->ToolsData->Position			= Position;
	pxControllerEntity->ToolsData->Position.x		+= 0.75f;
	pxControllerEntity->ToolsData->m_xOrientation	= Orientation;

	Tools_Entity_DialogueController* pxToolsController = static_cast<Tools_Entity_DialogueController*>( Tools_Entity_DialogueController::FindEntity( pxController->GetGuid() ) );
	if( pxToolsController )
	{
		pxToolsController->Position		= pxControllerEntity->ToolsData->Position;
		pxToolsController->Orientation	= pxControllerEntity->ToolsData->m_xOrientation;
		pxToolsController->SetEntityModified();
	}

	Tools_ParameterDerived<bool> xParam;
	GetParameter( g_uHash_Entity_StartsActive, &xParam );
	pxToolsController->SetParameter( g_uHash_Entity_StartsActive, &xParam );

	// Pick a name for this entity
	pxControllerEntity->ToolsData->Name	= pxTriggerEntity->ToolsData->Name;

	// Change the DLG to DLC (if present)
	Asura_Char* pcString = pxControllerEntity->ToolsData->Name.GetString();
	if(	strncmp(pcString, "DLG ", 4 ) == 0 )
	{
		pcString[2]	= 'C';
	}
	else
	{
		pxControllerEntity->ToolsData->Name += "(DLC)";
	}

	// Delete the original entity (optional)
	if( pxTriggerEntity->m_usMessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK )
	{
		ShowMessageOverlay( "The original dialogue trigger entity has not been deleted as it contains message blocks which could not be moved to the dialogue controller");
	}
	else
	{
		Tools_Object_System::SelectObject( this );
		Tools_Object_System::DestroySelected();
	}

	Tools_SaveAsura::SetEntityModified();

	Tools_Object_System::SelectObject( pxController );
}

#endif //ASURA_USE_DIALOGUE_SYSTEM
