//////////////////////////////////////////////////////////////////////////////////////////
//
// FILE : Asura_ApplyForce_Messages.cpp
//
// AUTHORS : Emil Popovic - 22.04.2009
//
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////


//////////////
// INCLUDES //
//////////////

// core
#include "Core/Asura.h"

// This file's header.
#include "Entity/Tools_ApplyForce_Messages.h"

// Engine independent.
#include "Independent/Entity/Asura_Entity.h"

#include "Blueprint/Asura_Blueprint.h"
#include "Blueprint/Asura_Blueprint_System.h"
#include "Blueprint/Asura_Blueprint_HashIDs.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/PhysicalObject/Asura_ApplyForce_Messages.h"
#include "Network/Asura_Guid.h"

// Tools independent.
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Message_Editing.h"
#include "Objects/Tools_Object.h"
#include "Objects/Tools_Object_System.h"
#include "Misc/Tools_Prompted_Selection.h"



// Static functions
Asura_Msg_ApplyForce_Linear* Tools_ApplyForce_Messages::s_pxApplyLinearForceMessage=NULL;
Asura_Msg_ApplyForce_Radial* Tools_ApplyForce_Messages::s_pxApplyRadialForceMessage=NULL;
Asura_String				 Tools_ApplyForce_Messages::s_xObjectNameString;



void Tools_ApplyForce_Messages::SetObjectNameString( Asura_String &xNameString, const Asura_Guid uGuid )
{
	if ( uGuid == ASURA_GUID_UNREGISTERED )
	{
		xNameString = "*Unset*";
	}
	else
	{
		Asura_Entity_Server* pxPhyObj = Asura_Entity_Server::FindEntity( uGuid );
		if ( !pxPhyObj )
		{
			// Check if we can get a tools object from it.
			Tools_Object *pxToolsObject = Tools_Object_System::GetObjectFromGuid( uGuid );
			if ( pxToolsObject )
			{
				xNameString = pxToolsObject->GetName();
			}
			else
			{
				xNameString = "*Not Found*";
			}
		}
		else if ( pxPhyObj->ToolsData )
		{
			xNameString = pxPhyObj->ToolsData->Name;
		}
		else
		{
			static char s_szNumberString[ 16 ];
			sprintf( s_szNumberString, "%u", uGuid );
			xNameString = s_szNumberString;
		}
	}
}

void Tools_ApplyForce_Messages::OnSelect_ChooseLinearForcePosObject(Tools_Object* pxObject)
{
	if(s_pxApplyLinearForceMessage)
	{		
		s_pxApplyLinearForceMessage->m_uSourcePosObjectGuid = pxObject->GetGuid();
		SetObjectNameString( s_xObjectNameString, pxObject->GetGuid() );
	}
}

void Tools_ApplyForce_Messages::OnSelect_ChooseRadialForcePosObject(Tools_Object* pxObject)
{
	if(s_pxApplyRadialForceMessage)
	{		
		s_pxApplyRadialForceMessage->m_uSourcePosObjectGuid = pxObject->GetGuid();
		SetObjectNameString( s_xObjectNameString, pxObject->GetGuid() );
	}
}

void Tools_ApplyForce_Messages::ChooseLinearForcePosObject(void)
{
	Tools_Prompted_Selection::ChooseObject( "Select Object which will be source of the force", Tools_ApplyForce_Messages::OnSelect_ChooseLinearForcePosObject );
}

void Tools_ApplyForce_Messages::ChooseRadialForcePosObject(void)
{
	Tools_Prompted_Selection::ChooseObject( "Select Object which will be source of the force", Tools_ApplyForce_Messages::OnSelect_ChooseRadialForcePosObject );
}

void Tools_ApplyForce_Messages::ChooseBlueprintForApplyForceMsg(int pExtraInfoBPHash)
{
	Asura_Hash_ID uBPHash = 0x05D18AEB; //"Force"
	Asura_Hash_ID uBPType = uASURA_BLUEPRINT_TYPE_FORCES;
	BlueprintSelect_Overlay xBPSelOvrly( "Select a Force", &uBPHash, &uBPType, 1, &uBPType, 1, &uBPHash );

	if (xBPSelOvrly.DoModal() && uBPType == uASURA_BLUEPRINT_TYPE_FORCES)
	{
		int* puExtraInfoBPHash= reinterpret_cast<int*>(pExtraInfoBPHash);
		*puExtraInfoBPHash = uBPHash;

		Tools_Message_Editing::UpdateMessageOverlay();
	}
	else
	{
		// This should be impossible but might as well check
		Asura_Assert(uBPType == uASURA_BLUEPRINT_TYPE_FORCES, "You must select a Force Blueprint");
	}
}


void Tools_ApplyForce_Messages::ApplyLinearForce_SetCentreOfMassOffset()
{
	if( s_pxApplyLinearForceMessage )
	{
		Tools_PositionEdit_Object* pxPosObject = new Tools_PositionEdit_Object(&s_pxApplyLinearForceMessage->m_xOffsetFromCentreOfMass, NULL, "Offset from centre of mass", "Edit Offset from centre of mass.", NULL );
		if (pxPosObject)
		{
			// We need to remember our currently selected object, because we are going to select PositionEdit object.
			// However, upon destruction of PositionEdit object, we need to return focus to this object.
			Tools_Object* pxPreviousObject = Tools_Object_System::GetMainSelectionForCurrentSelectionMode();

			// Now set new object to have a focus. It is important so we can move this object using F and V keys.
			Tools_Object_System::SelectObject( pxPosObject );

			// Create "Chooser" object. This object is special, because it will set tools to special mode. Where you are
			// selecting new object.
			Tools_ChooseMoveObject* pxChooseObject = new Tools_ChooseMoveObject( pxPosObject, pxPreviousObject );
			Tools_Object_System::ChooseObject("Please choose offset from center of mass. Use 'f' and 'v' keys to do this", pxChooseObject);
		}
	}
}


void Tools_ApplyForce_Messages::AddOverlayControls_LinearForce(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Msg_ApplyForce_Linear* pxData = static_cast<Asura_Msg_ApplyForce_Linear*>( pxMessage->m_pData );

	s_pxApplyLinearForceMessage = pxData;

	Asura_Dialog* pxDialog = 0;

	pxDialog = new Asura_Function_Dialog( "Choose Source Position of Force:", xOffset, ChooseLinearForcePosObject );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	SetObjectNameString( s_xObjectNameString, pxData->m_uSourcePosObjectGuid );
	pxDialog = new Asura_Dynamic_Dialog( &s_xObjectNameString, xOffset );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y += Asura_Dialog::DefaultSpacing();

	// 

//	pxDialog = new Asura_Function_Dialog("Set Centre of mass offset", xOffset, ApplyLinearForce_SetCentreOfMassOffset );
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Centre of mass offset X", xOffset, &s_pxApplyLinearForceMessage->m_xOffsetFromCentreOfMass.x, -1000000.0f, 1000000.0f, 0.1f, "%0.01f" );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Centre of mass offset Y", xOffset, &s_pxApplyLinearForceMessage->m_xOffsetFromCentreOfMass.y, -1000000.0f, 1000000.0f, 0.1f, "%0.01f" );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Centre of mass offset Z", xOffset, &s_pxApplyLinearForceMessage->m_xOffsetFromCentreOfMass.z, -1000000.0f, 1000000.0f, 0.1f, "%0.01f" );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y += Asura_Dialog::DefaultSpacing();

	//---------------------------

	// Won't work on 64bit
	pxDialog = new Asura_Function_Dialog( "Extra Info Blueprint:", xOffset, ChooseBlueprintForApplyForceMsg, reinterpret_cast<int>(&(pxData->m_uExtraInfoBPHash)) );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Find our shake name
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uASURA_BLUEPRINT_TYPE_FORCES, pxData->m_uExtraInfoBPHash);
	if (pxBlueprint)
	{
		pxDialog = new Asura_StringDisplay_Dialog( pxBlueprint->GetName().GetString(), xOffset );	
	}
	else if (pxData->m_uExtraInfoBPHash== ASURA_HASH_ID_UNSET)
	{
		pxDialog = new Asura_StringDisplay_Dialog( "*None*", xOffset );	
	}
	else
	{
		pxDialog = new Asura_StringDisplay_Dialog( "*Not Found*", xOffset );	
	}
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();	
}


void Tools_ApplyForce_Messages::AddOverlayControls_RadialForce(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Msg_ApplyForce_Radial* pxData = static_cast<Asura_Msg_ApplyForce_Radial*>( pxMessage->m_pData );

	s_pxApplyRadialForceMessage = pxData;

	Asura_Dialog* pxDialog = 0;

	pxDialog = new Asura_Function_Dialog( "Choose Source Position of Force:", xOffset, ChooseRadialForcePosObject );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	SetObjectNameString( s_xObjectNameString, pxData->m_uSourcePosObjectGuid );
	pxDialog = new Asura_Dynamic_Dialog( &s_xObjectNameString, xOffset );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y += Asura_Dialog::DefaultSpacing();


	//--------------------- Blueprint --------------

	// Won't work on 64bit
	pxDialog = new Asura_Function_Dialog( "Extra Info Blueprint:", xOffset, ChooseBlueprintForApplyForceMsg, reinterpret_cast<int>(&(pxData->m_uExtraInfoBPHash)) );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Find our shake name
	Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uASURA_BLUEPRINT_TYPE_FORCES, pxData->m_uExtraInfoBPHash);
	if (pxBlueprint)
	{
		pxDialog = new Asura_StringDisplay_Dialog( pxBlueprint->GetName().GetString(), xOffset );	
	}
	else if (pxData->m_uExtraInfoBPHash== ASURA_HASH_ID_UNSET)
	{
		pxDialog = new Asura_StringDisplay_Dialog( "*None*", xOffset );	
	}
	else
	{
		pxDialog = new Asura_StringDisplay_Dialog( "*Not Found*", xOffset );	
	}
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();	
}
