//
// Tools_Entity_Activatable_Object.cpp
//
// Created by Richard Rice - 18/09/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Entity/Tools_Entity_Activatable_Object.h"

#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_Entity.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Triggers/Asura_ClientEntity_Activatable.h"
#include "Entity/Triggers/Asura_ServerEntity_Activatable.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Rendering/Asura_Text_Output.h"

#include "Entity/Tools_Entity_Simulation.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/Tools_ObjectSystem_Param.h"


//TOOLS_DECLARE_SHARED_PARAM(bool,	0x7A880899,	Activatable_Active)
//Now uses the same ID for Active as Tools_Entity_Object
Tools_ObjectSystem_SharedValue< bool > g_xParam_Activatable_Active( g_uHash_Entity_StartsActive);


///////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_Entity_Activatable_Object **
**---------------------------------*/

Tools_Entity_Activatable_Object::Tools_Entity_Activatable_Object(Asura_Entity* pxEntity)
:Tools_Entity_Object(pxEntity)
{
	Asura_ServerEntity_Activatable* pxAsuraEntity = (Asura_ServerEntity_Activatable*) Entity;
	m_bActive = pxAsuraEntity->m_bActive!=0;
}

void Tools_Entity_Activatable_Object::UpdateWhileSelected()
{
	Asura_ServerEntity_Activatable* pxAsuraEntity = (Asura_ServerEntity_Activatable*) Entity;

	if(Changed)
	{
		//update the asura data
		pxAsuraEntity->SetInternalActiveFlag(m_bActive);
	}

	Tools_Entity_Object::UpdateWhileSelected();
}



void Tools_Entity_Activatable_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Object::AddOverlayControls(pxOverlay,xOffset);
	Asura_Dialog* pxDialog;

	xOffset.y += 0.5f;

	pxDialog = new Asura_SharedTickbox_Dialog("Active",xOffset,&g_xParam_Activatable_Active);
	pxDialog->SetHelpText("This is active at the start of the level.\nThe exact behaviour of inactive objects varies by type of entity.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += 1.5f;
}




bool Tools_Entity_Activatable_Object::HandlesThisMessage(int messageID)
{
	if(messageID == ASURA_MSG_DESTRUCT ||
	   messageID == ASURA_MSG_ACTIVATE ||
	   messageID == ASURA_MSG_DEACTIVATE ||
	   messageID == ASURA_MSG_TRIGGER ||
	   messageID == ASURA_MSG_UNTRIGGER)
	{
		return true;
	}
	return false;

}

void Tools_Entity_Activatable_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Object::Copy(pxObject);

	const Tools_Entity_Activatable_Object* pxFromObject = (const Tools_Entity_Activatable_Object*) pxObject;

	Asura_ServerEntity_Activatable* pxFromEntity = (Asura_ServerEntity_Activatable*) pxFromObject->Entity;
	Asura_ServerEntity_Activatable* pxToEntity = (Asura_ServerEntity_Activatable*) Entity;

	pxToEntity->SetInternalActiveFlag(pxFromEntity->m_bActive);
	m_bActive = pxFromObject->m_bActive;

}

void Tools_Entity_Activatable_Object::PrintInfo(float& x,float& y)
{
	Tools_Entity_Object::PrintInfo(x,y);

	if(Tools_Entity_Simulation::IsActive())
	{
		Asura_ServerEntity_Activatable* pxAsuraEntity = (Asura_ServerEntity_Activatable*) Entity;
	
		Asura_Text_Output::Print(x,y++,0xffffff00,0,"Active : %s",pxAsuraEntity->m_bActive ? "Yes" : "No");
	}
}

void Tools_Entity_Activatable_Object::UpdateClient()
{
	if(!m_pxClientEntity) return;

	Tools_Entity_Object::UpdateClient();

	Asura_ServerEntity_Activatable* pxServerEntity = (Asura_ServerEntity_Activatable*) Entity;
	Asura_ClientEntity_Activatable* pxClientEntity = (Asura_ClientEntity_Activatable*) m_pxClientEntity;

	pxClientEntity->m_bActive = pxServerEntity->m_bActive;
	
}

bool Tools_Entity_Activatable_Object::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{

	switch(uParamID)
	{
		case g_uHash_Entity_StartsActive :
			pxParam->SetBool(m_bActive);
			break;

		default :
			return PARENT::GetParameter(uParamID, pxParam);

	}

	return true;

}

bool Tools_Entity_Activatable_Object::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{

	Asura_ServerEntity_Activatable* pxAsuraEntity = (Asura_ServerEntity_Activatable*) Entity;
	switch(uParamID)
	{
		case g_uHash_Entity_StartsActive :
			m_bActive = pxParam->GetBool();
			pxAsuraEntity->SetInternalActiveFlag(m_bActive);
			break;


		default :
			return PARENT::SetParameter(uParamID, pxParam);
	}
	
	SetEntityModified();

	return true;

}


///////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------**
** Tools_Entity_ActivatableWithPosition_Object **
**---------------------------------------------*/

Tools_Entity_ActivatableWithPosition_Object::Tools_Entity_ActivatableWithPosition_Object(Asura_Entity* _pxEntity)
:Tools_Entity_Activatable_Object(_pxEntity)
{
	Asura_ServerEntity_ActivatableWithPosition*	pxServerEntity	= (Asura_ServerEntity_ActivatableWithPosition*) _pxEntity;

	//get the position and orientation
	Position	= pxServerEntity->m_xPosition;
	Orientation	= pxServerEntity->m_xOrientation;
}

void Tools_Entity_ActivatableWithPosition_Object::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	Tools_Entity_Activatable_Object::UpdatePositionInAsuraData(bUpdateLinkedObjects);

	Asura_ServerEntity_ActivatableWithPosition* pxEntity = (Asura_ServerEntity_ActivatableWithPosition*) Entity;

	//update position in entity
	pxEntity->m_xPosition = Position;
	pxEntity->m_xOrientation = Orientation;

	UpdateClient();
	SetEntityModified();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Entity_ActivatableWithPosition_Object::UpdateClient
// Author       : Andy Tate
// Date         : 1-Sep-2005
//
// Description  : 
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Tools_Entity_ActivatableWithPosition_Object::UpdateClient (void)
{
	// Do we have a corresponding client-entity?
	if (NULL != m_pxClientEntity)
	{
		Asura_ClientEntity_ActivatableWithPosition*			pxClientEntity	= (Asura_ClientEntity_ActivatableWithPosition*) m_pxClientEntity;
		Asura_ServerEntity_ActivatableWithPosition const*	pxServerEntity	= (Asura_ServerEntity_ActivatableWithPosition*) Entity;

		// Update the base-class...
		Tools_Entity_Activatable_Object::UpdateClient	();

		// Update position in the entities...
		pxClientEntity->SetPositionAndOrientation	(pxServerEntity->m_xPosition, pxServerEntity->m_xOrientation);
	}
}

/// End of function 'Tools_Entity_ActivatableWithPosition_Object::UpdateClient'. ///

