//
// Tools_Entity_AdvancedVolumeTrigger.cpp
//
// Created by Andrew Weinkove - 30/01/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "Tools_Entity_AdvancedVolumeTrigger.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Entity/Asura_Entity.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Triggers/Asura_ClientEntity_AdvancedVolumeTrigger.h"
#include "Entity/Triggers/Asura_ServerEntity_AdvancedVolumeTrigger.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Triggers/Asura_ServerEntity_CountedTrigger.h"
#include "Mesh/Asura_Mesh_System.h"

#include "Objects/Tools_VolumeEdit_Object.h"
#include "Objects/Tools_Object_System.h"
#include "Load&Save/Tools_SaveAsura.h"

//////////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------**
** Tools_Entity_AdvancedVolumeTrigger **
**-----------------------------------*/

bool Tools_Entity_AdvancedVolumeTrigger::s_bMoveVolumeWithObject = true;
bool Tools_Entity_AdvancedVolumeTrigger::s_bShowAllVolumes = false;

Tools_Entity_AdvancedVolumeTrigger::Tools_Entity_AdvancedVolumeTrigger(Asura_Entity* pxEntity)
:Tools_Entity_Trigger_Object(pxEntity)
{
/// DWE - 24/03/2003 10:37:39 - I've removed this assert because I have over-ridden the class with the 
///		hostage rescue zones and they have a different classification
//	Asura_Assert(pxEntity->GetClassification()==AsuraEntityClass_AdvancedVolumeTrigger,"");
	Type = TOT_Entity_AdvancedVolumeTrigger;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_V");
	m_xPreviousPosition = Position;
	m_pxVolumeController = 0;
}

Tools_Entity_AdvancedVolumeTrigger::~Tools_Entity_AdvancedVolumeTrigger()
{
	delete m_pxVolumeController;
}

void Tools_Entity_AdvancedVolumeTrigger::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Trigger_Object::AddOverlayControls(pxOverlay,xOffset);

	Asura_ServerEntity_AdvancedVolumeTrigger* pxAsuraEntity = (Asura_ServerEntity_AdvancedVolumeTrigger*) Entity;

	Asura_Dialog* pxDialog;
	pxDialog = new Asura_Function_Dialog("Edit Volume",xOffset,EditVolume);
	pxDialog->SetHelpText("Edits the volume for this trigger.");
	pxOverlay->AddDialog(pxDialog);


	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
 	pxDialog = new Asura_Tickbox_Dialog( "Show All", xOffset, &s_bShowAllVolumes);
	pxDialog->SetHelpText("Shows volumes for all volume triggers.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.x =  1.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();


 	pxDialog = new Asura_Tickbox_Dialog( "Move Volume With Object", xOffset, &s_bMoveVolumeWithObject);
	pxDialog->SetHelpText("The volume attached to this trigger should be moved when the trigger's tools object is moved.\nThis is a global setting for all trigger volumes.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

    pxDialog = new Asura_FlagTickbox_Dialog( "Trigger Once", xOffset, &pxAsuraEntity->m_uFlags,ASURAVOLUMETRIGGERFLAG_TRIGGERONCE,&Changed);
	pxDialog->SetHelpText("This trigger will deactivate itself once it becomes untriggered.\n Therefore it will only be triggerable once , unless it is re-activated.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    pxDialog = new Asura_FlagTickbox_Dialog( "Trigger On Not Occupied", xOffset, &pxAsuraEntity->m_uFlags,ASURAVOLUMETRIGGERFLAG_TRIGGERONNOTOCCUPIED,&Changed);
	pxDialog->SetHelpText("The volume trigger becomes triggered if it isn't occupied by anyone.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();


}

void Tools_Entity_AdvancedVolumeTrigger::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	//shift the bounding box by the amount that the object has moved
	Asura_ServerEntity_AdvancedVolumeTrigger* pxAsuraEntity = (Asura_ServerEntity_AdvancedVolumeTrigger*) Entity;
	if(s_bMoveVolumeWithObject)
	{
		for(int i=0;i<pxAsuraEntity->m_iNumberOfVolumes;i++)
		{
			pxAsuraEntity->m_pxVolumes[i].m_xBox.Displace(Position-m_xPreviousPosition);
		}
		if(m_pxVolumeController)
		{
			m_pxVolumeController->MoveVolumeObjects(Position-m_xPreviousPosition);
		}
	}

	m_xPreviousPosition = Position;

	Tools_Entity_Trigger_Object::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}

void Tools_Entity_AdvancedVolumeTrigger::Update()
{
	Tools_Entity_Trigger_Object::Update();

	ShowVolume(s_bShowAllVolumes);
}

void Tools_Entity_AdvancedVolumeTrigger::EditVolume()
{
	Asura_Assert(ThisEntity,"");

	Tools_Entity_AdvancedVolumeTrigger* pxToolsEntity = (Tools_Entity_AdvancedVolumeTrigger*) ThisEntity;
	//create controller for editing volumes if it hasn't been done yet
	if(!pxToolsEntity->m_pxVolumeController)
	{
		pxToolsEntity->m_pxVolumeController = new Tools_AdvancedVolumeTrigger_VolumeEdit_Controller(pxToolsEntity);
	}

	//and get it to create the volume objects
	pxToolsEntity->m_pxVolumeController->StartVolumeEditing();
}


void Tools_Entity_AdvancedVolumeTrigger::ShowVolume(bool bShow)
{
	if(!m_pxVolumeController && !bShow) return;

	//create controller for editing volumes if it hasn't been done yet
	if(!m_pxVolumeController)
	{
		m_pxVolumeController = new Tools_AdvancedVolumeTrigger_VolumeEdit_Controller(this);
	}

	//hide/show volume objects
	m_pxVolumeController->ShowVolumes(bShow || m_pxVolumeController->IsSelected());
	
}

Tools_Object* Tools_Entity_AdvancedVolumeTrigger::CreateObjectOfThisType() const
{
	Tools_Object* pxOldSelection = Tools_Object_System::SelectedObject;
	Project_Create();
	Tools_Object* pxNewSelection = Tools_Object_System::SelectedObject;

	return (pxOldSelection!=pxNewSelection) ? pxNewSelection : 0;
}

void Tools_Entity_AdvancedVolumeTrigger::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Trigger_Object::Copy(pxObject);
	
	const Tools_Entity_AdvancedVolumeTrigger* pxFromObject = (const Tools_Entity_AdvancedVolumeTrigger*) pxObject;

	Asura_ServerEntity_AdvancedVolumeTrigger* pxFromEntity = (Asura_ServerEntity_AdvancedVolumeTrigger*) pxFromObject->Entity;
	Asura_ServerEntity_AdvancedVolumeTrigger* pxToEntity = (Asura_ServerEntity_AdvancedVolumeTrigger*) Entity;

	//delete the volume controller if it exists ,since we will be replacing the volumes
	delete m_pxVolumeController;
	m_pxVolumeController = 0;
	
	//copy everything between the entities
	pxToEntity->m_uFlags = pxFromEntity->m_uFlags;
	pxToEntity->m_iListFlags = pxFromEntity->m_iListFlags;
	pxToEntity->m_uSpecificGuid = pxFromEntity->m_uSpecificGuid;
	pxToEntity->m_iNumberOfVolumes = pxFromEntity->m_iNumberOfVolumes;

	delete [] pxToEntity->m_pxVolumes;
	if(pxToEntity->m_iNumberOfVolumes)
	{
		pxToEntity->m_pxVolumes = new Asura_OrientedVolume[pxToEntity->m_iNumberOfVolumes];
		for(int i=0;i<pxToEntity->m_iNumberOfVolumes;i++)
		{
			pxToEntity->m_pxVolumes[i] = pxFromEntity->m_pxVolumes[i];

			if(s_bMoveVolumeWithObject)
			{
				//move the volumes to take into account the different object positions
				pxToEntity->m_pxVolumes[i].m_xBox.Displace(Position-pxObject->Position);
			}
		}
	}
	else
	{
		pxToEntity->m_pxVolumes = 0;
	}
}

void Tools_Entity_AdvancedVolumeTrigger::UpdateClient()
{
	if(!m_pxClientEntity) return;

	Tools_Entity_Trigger_Object::UpdateClient();

	Asura_ServerEntity_AdvancedVolumeTrigger* pxServerEntity = (Asura_ServerEntity_AdvancedVolumeTrigger*) Entity;
	Asura_ClientEntity_AdvancedVolumeTrigger* pxClientEntity = (Asura_ClientEntity_AdvancedVolumeTrigger*) m_pxClientEntity;

	pxClientEntity->m_iListFlags = pxServerEntity->m_iListFlags;
	pxClientEntity->m_uSpecificGuid = pxServerEntity->m_uSpecificGuid;
	pxClientEntity->m_uFlags = pxServerEntity->m_uFlags;

	delete [] pxClientEntity->m_pxVolumes;

	pxClientEntity->m_iNumberOfVolumes = pxServerEntity->m_iNumberOfVolumes;
	pxClientEntity->m_pxVolumes = pxClientEntity->m_iNumberOfVolumes ? new  Asura_OrientedVolume[pxClientEntity->m_iNumberOfVolumes] : 0;

	for(int i =0;i<pxClientEntity->m_iNumberOfVolumes;i++)
	{
		pxClientEntity->m_pxVolumes[i] = pxServerEntity->m_pxVolumes[i];
	}
	
}


//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------------------------**
** Tools_AdvancedVolumeTrigger_VolumeEdit_Controller **
**---------------------------------------------------*/

Tools_AdvancedVolumeTrigger_VolumeEdit_Controller::Tools_AdvancedVolumeTrigger_VolumeEdit_Controller(Tools_Entity_AdvancedVolumeTrigger* pxToolsEntity)
:Tools_VolumeEdit_Controller(&pxToolsEntity->Changed)
{
	m_pxToolsEntity = pxToolsEntity;
	m_pxAsuraEntity = (Asura_ServerEntity_AdvancedVolumeTrigger*) pxToolsEntity->Entity;

	if(m_pxAsuraEntity->m_iNumberOfVolumes == 1)
	{
		//if there is only one volume , and it has zero size , then give it a default position/size
		if(m_pxAsuraEntity->m_pxVolumes[0].m_xBox.GetVolume() == 0.0f)
		{
			Asura_Bounding_Box& xBox = m_pxAsuraEntity->m_pxVolumes[0].m_xBox;
			xBox.MinX = m_pxToolsEntity->Position.x-1.0f;
			xBox.MaxX = m_pxToolsEntity->Position.x+1.0f;
			xBox.MinY = m_pxToolsEntity->Position.y-4.0f;
			xBox.MaxY = m_pxToolsEntity->Position.y+1.0f;
			xBox.MinZ = m_pxToolsEntity->Position.z-4.0f;
			xBox.MaxZ = m_pxToolsEntity->Position.z+4.0f;
		}
	}
}

void Tools_AdvancedVolumeTrigger_VolumeEdit_Controller::StartVolumeEditing()
{

	Tools_VolumeEdit_Controller::StartVolumeEditing();

	if(GetNumberOfVolumes() == 0)
	{
		//create a volume if none exists yet
		CreateVolume();
	}
}

void Tools_AdvancedVolumeTrigger_VolumeEdit_Controller::EndVolumeEditing()
{
	Tools_VolumeEdit_Controller::EndVolumeEditing();
	//select the owner
	Tools_Object_System::SelectObject(m_pxToolsEntity);
}


void Tools_AdvancedVolumeTrigger_VolumeEdit_Controller::CreateVolumeMain()
{
	//allocate a new array of volumes , and copy the old ones across
	Asura_OrientedVolume* pxVolumes = new Asura_OrientedVolume[m_pxAsuraEntity->m_iNumberOfVolumes+1];

	for(int i=0;i<m_pxAsuraEntity->m_iNumberOfVolumes;i++)
	{
		pxVolumes[i] = m_pxAsuraEntity->m_pxVolumes[i];
	}

	//initialise the new volume
	Asura_Bounding_Box& xBox = pxVolumes[m_pxAsuraEntity->m_iNumberOfVolumes].m_xBox;
	xBox.MinX = m_pxToolsEntity->Position.x-1.0f;
	xBox.MaxX = m_pxToolsEntity->Position.x+1.0f;
	xBox.MinY = m_pxToolsEntity->Position.y-4.0f;
	xBox.MaxY = m_pxToolsEntity->Position.y+1.0f;
	xBox.MinZ = m_pxToolsEntity->Position.z-4.0f;
	xBox.MaxZ = m_pxToolsEntity->Position.z+4.0f;
	pxVolumes[m_pxAsuraEntity->m_iNumberOfVolumes].m_xOrient = Asura_Maths::IdentityQuat;

	//replace the old array
	delete [] m_pxAsuraEntity->m_pxVolumes;
	m_pxAsuraEntity->m_pxVolumes = pxVolumes;
	m_pxAsuraEntity->m_iNumberOfVolumes++;

}

void Tools_AdvancedVolumeTrigger_VolumeEdit_Controller::DeleteVolumeMain(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<m_pxAsuraEntity->m_iNumberOfVolumes,"");

	//shuffle down any volumes after the one being deleted
	for(int i=iIndex;i<m_pxAsuraEntity->m_iNumberOfVolumes-1;i++)
	{
		m_pxAsuraEntity->m_pxVolumes[i] = m_pxAsuraEntity->m_pxVolumes[i+1];
	}
	m_pxAsuraEntity->m_iNumberOfVolumes--;
}

