//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_Lift.cpp
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// Description:
//
//     Lift Entity - External control physics object moving along a spline
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header.
#include "Entity/Tools_Entity_Lift.h"

// Engine independent.
#include "Animation/Asura_Hierarchy.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_Entity_Event.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Locations/OrientedNode/LiftNode/Tools_EntityNode_LiftNode.h"
#include "Entity/Locations/Spline/Asura_ServerNode_Spline.h"
#include "Entity/Locations/Spline/Tools_Object_SplinePoint.h"
#include "Entity/Locations/Spline/LiftSpline/Asura_ServerNode_LiftSpline.h"
#include "Entity/Locations/Spline/LiftSpline/Tools_EntityNode_LiftSpline.h"
#include "Entity/Locations/Spline/LiftSpline/Tools_Object_LiftSplinePoint.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_Lift.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Network/Asura_Network.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Shapes/Asura_Shapes.h"
#include "SpecialFX/Asura_Skybox.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "Sound/Asura_Sound_Event_System.h"

// Tools.
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Locations/Tools_Entity_Node.h"
#ifndef ASURA_TARGET_SM3
#include "Entity/Tools_Entity_DestructibleLight.h"
#endif // #ifndef ASURA_TARGET_SM3
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_ObjectSystem_Param.h"
#include "Shapes/Tools_Shapes.h"
#include "Sound/Tools_PickSoundEvent_Dialog.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_ANIM_LOOPING,			Lift_Looping )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_ANIM_REVERSED,		Lift_Reversed )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_USE_ANIM_MOVEMENT,	Lift_UseAnimMovement )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_NOSHADOW,				Lift_NoShadow )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_NOCOLLISIONS,			Lift_NoCollisions )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_IGNOREMODLINKING,		Lift_IgnoreModuleLinking )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_HIDDEN,				Lift_Hidden )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_IGNOREFORLOS,			Lift_IgnoreForLOS )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_FULLBRIGHT,			Lift_FullBright )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_POLYGONAL,			Lift_Polygonal )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_RUNNING,				Lift_Running )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_REVERSE,				Lift_Reverse )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_AUTOREVERSE,			Lift_AutoReverse )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_COLLIDEENV,			Lift_CollideEnv )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_COLLIDESTATIC,		Lift_CollideStatic )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_COLLIDEDYNAMIC,		Lift_CollideDynamic )
TOOLS_DECLARE_SHARED_PARAM( bool, uTOOLS_SHAREDPARAMHASH_LIFT_PAUSEONCOLLIDE,		Lift_PauseOnCollide )

// S T A T I C   V A R I A B L E S ///////////////////////////////////////////

static Asura_Shape_Instance	g_xShapeInstance;

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////

bool Tools_Lift_SelectSpline::CanBeSelected( Tools_Object* pxObject )
{
	if (Asura_ServerEntity_Node::FindEntity(pxObject->GetGuid()))
	{
		Tools_Entity_Node* pxNode = static_cast<Tools_Entity_Node*>(pxObject);
		return pxNode->IsBaseOrDerivativeOf(TOT_Entity_LiftSpline);
	}
	return false;
}

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_Lift
//
//////////////////////////////////////////////////////////////////////////////

const char* const Tools_Entity_Lift::s_szLongName  = "Lift";
const char* const Tools_Entity_Lift::s_szShortName = "Lift";
Tools_Entity_Lift* Tools_Entity_Lift::s_pxCurrentlySelectedObject = NULL;
Asura_Hashed_List<Tools_Entity_Lift> Tools_Entity_Lift::s_xList( 5 );

Tools_Entity_Lift::Tools_Entity_Lift(Asura_ServerEntity_Lift* pxLift)
: PARENT(pxLift) // ignored
, m_iAnimFlags(AsuraAnimationFlag_Loop)
, m_iPreviewSoundHandle(ASURA_INVALID_HANDLE)
{
	Type = TOT_Entity_Lift;

	if (!m_pxPhysicalObject) return; // no PhysicalObject

	m_pxLift = pxLift;

	m_iAnimFlags = m_pxPhysicalObject->GetAnimFlags();
	
    s_xList.AddToList(this);

	m_bPreviewing = false;
	m_uPreviewSpline = ASURA_GUID_UNREGISTERED;
	m_bPreviewReverse = false;
	m_fPreviewProgress = 0.0f;
	m_bChangedSoundEvent = false;

	m_bSnapLiftToSplineNode	= false;
	m_bSnapLiftToSplineNodeChanged = false;
	m_iCurrentSplineNode	= 0;
	m_iPreviousSplineNode	= -1;

	m_bShowStartPosition = true;
	m_bShowStartPositionChanged = false;
	m_iStartLiftNode = ( m_pxLift->GetProgress() > 0.999999f ) ? 1 : 0;
	m_bStartLiftNodeChanged = false;
}

Tools_Entity_Lift::~Tools_Entity_Lift()
{
	m_xCreationList.Clear();
	s_xList.RemoveFromList(this);
	if( m_iPreviewSoundHandle != ASURA_INVALID_HANDLE )
	{
		Asura_Sound_Event_System::Stop(m_iPreviewSoundHandle);
	}
}

void Tools_Entity_Lift::AddFlagsMessageOverlayControls(Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_Lift_SetFlags* pxLiftMessage = static_cast< Asura_Message_Lift_SetFlags* >( pxMessage->m_pData );

	Asura_Dialog* pxDialog = 0;

	pxDialog = new Asura_FlagTickbox_Dialog("Set AutoReverse On", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<0);
	pxDialog->SetHelpText("turn autoreverse on for this lift");
	pxDialog->SetRadioButtonGroup(20, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_FlagTickbox_Dialog("Set AutoReverse Off", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<1);
	pxDialog->SetHelpText("turn autoreverse off for this lift");
	pxDialog->SetRadioButtonGroup(20, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing() * 1.5f;

	pxDialog = new Asura_FlagTickbox_Dialog("Set Pause On Blocked On", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<2);
	pxDialog->SetHelpText("turn Pause On Blocked on for this lift");
	pxDialog->SetRadioButtonGroup(21, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_FlagTickbox_Dialog("Set Pause On Blocked Off", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<3);
	pxDialog->SetHelpText("turn Pause On Blocked off for this lift");
	pxDialog->SetRadioButtonGroup(21, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing() * 1.5f;

	pxDialog = new Asura_FlagTickbox_Dialog("Set Blocked by environment On", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<4);
	pxDialog->SetHelpText("turn Blocked by environment on for this lift");
	pxDialog->SetRadioButtonGroup(22, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_FlagTickbox_Dialog("Set Blocked by environment Off", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<5);
	pxDialog->SetHelpText("turn Blocked by environment off for this lift");
	pxDialog->SetRadioButtonGroup(22, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing() * 1.5f;

	pxDialog = new Asura_FlagTickbox_Dialog("Set Blocked by statics On", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<6);
	pxDialog->SetHelpText("turn Blocked by statics on for this lift");
	pxDialog->SetRadioButtonGroup(23, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_FlagTickbox_Dialog("Set Blocked by statics Off", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<7);
	pxDialog->SetHelpText("turn Blocked by statics off for this lift");
	pxDialog->SetRadioButtonGroup(23, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing() * 1.5f;

	pxDialog = new Asura_FlagTickbox_Dialog("Set Blocked by dynamics On", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<8);
	pxDialog->SetHelpText("turn Blocked by dynamics on for this lift");
	pxDialog->SetRadioButtonGroup(24, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_FlagTickbox_Dialog("Set Blocked by dynamics Off", xOffset, pxLiftMessage->GetFlagsPtr(), 1<<9);
	pxDialog->SetHelpText("turn Blocked by dynamics off for this lift");
	pxDialog->SetRadioButtonGroup(24, false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing() * 1.5f;
}

void Tools_Entity_Lift::OnSelection()
{
    PARENT::OnSelection();
    s_pxCurrentlySelectedObject = this;
}

void Tools_Entity_Lift::OnDeselection()
{
	if ( m_bPreviewing )
	{
		PreviewEnd();
	}
    s_pxCurrentlySelectedObject = NULL;
	PARENT::OnDeselection();
}

void Tools_Entity_Lift::Update()
{
	PARENT::Update();

	// We now specify that all lifts must start from a node initially.
	// This will help us fix up any lifts that were created before we specified this.
	if ( !m_pxLift->IsAtNode() )
	{
		Asura_ServerNode_LiftSpline* pxLiftSpline = Asura_ServerNode_LiftSpline::FindEntity( m_pxLift->GetSplineGuid() );
		if ( pxLiftSpline )
		{
			m_pxLift->SetInitialNode( m_iStartLiftNode );

			Asura_Vector_3 xPosition;
			Asura_Quat	   xOrientation;
			pxLiftSpline->GetPositionFromProportion( m_pxLift->GetProgress(), xPosition, &xOrientation );

			Position = xPosition;
			xOrientation.ToMatrix( Orientation );
			UpdatePositionInAsuraData( true );
			Changed = true;
		}
	}

	// If anything is deleted outside of our control then update our data.
	Asura_ListElt< Asura_Guid >* pxBack = m_xCreationList.Back();
	while ( pxBack && !Asura_ServerNode_LiftNode::FindEntity( pxBack->Data() ) )
	{
		m_xCreationList.Remove( pxBack );
		pxBack = m_xCreationList.Back();
	}
	if ( ( m_pxLift->GetSplineGuid() != ASURA_GUID_UNREGISTERED ) && !Asura_ServerNode_LiftSpline::FindEntity( m_pxLift->GetSplineGuid() ) )
	{
		m_pxLift->SetInitialSplineGuid( ASURA_GUID_UNREGISTERED );
		Changed = true;
	}
}

void Tools_Entity_Lift::UpdateWhileSelected()
{
	if(Changed)
	{
		if (m_pxPhysicalObject)
		{
			m_pxPhysicalObject->SetAnimFlags(m_iAnimFlags);
		}
 	}

	//check that all sound events that should be looping, are
	if ( m_bChangedSoundEvent )
	{
		const Asura_Sound_Event* pxSoundEvent = Asura_Sound_Event_Storage::GetEventByHash( m_pxLift->GetStopSoundEventHash() );
		if( pxSoundEvent && pxSoundEvent->IsEndlessPlayback() )
		{
			//force reselect - shouldn't loop
			Message_Overlay xMessageOverlay("Lift stopping sound shouldn't loop - please select a non-looping sound");
			xMessageOverlay.DoModal();
			*m_pxLift->GetStopSoundEventHashPointer() = ASURA_HASH_ID_UNSET;
		}
		else
		{
			m_bChangedSoundEvent = false;
		}
		
		Changed = true;
	}

	PARENT::UpdateWhileSelected();  

	if ( m_bSnapLiftToSplineNodeChanged )
	{
		m_bSnapLiftToSplineNodeChanged = false;
		if ( m_bSnapLiftToSplineNode )
		{
			m_bShowStartPosition = false;
		}
	}
	if ( m_bShowStartPositionChanged )
	{
		m_bShowStartPositionChanged = false;
		if ( m_bShowStartPosition )
		{
			m_bSnapLiftToSplineNode = false;
		}
	}

	if ( m_bSnapLiftToSplineNode )
	{
		const Asura_Guid uSplineGuid = m_pxLift->GetSplineGuid();
		if ( m_iCurrentSplineNode == m_iPreviousSplineNode )
		{
			Tools_EntityNode_LiftSpline* pxToolsLiftSpline = static_cast<Tools_EntityNode_LiftSpline*>(Tools_EntityNode_LiftSpline::FindEntity(uSplineGuid));
			if ( pxToolsLiftSpline )
			{
				// Force the current spline-node to be placed wherever the lift is...
				pxToolsLiftSpline->UpdateSplinePoint(m_iCurrentSplineNode + 1, Position, Orientation);
			}
		}
		else
		{
			Asura_ServerNode_LiftSpline* pxLiftSpline = Asura_ServerNode_LiftSpline::FindEntity( uSplineGuid );
			if ( pxLiftSpline )
			{
				const int iNumSplineNodes = pxLiftSpline->GetNumberOfPoints() - 2;

				// Clamp the maximum index to the number of nodes in the spline (minus 2 since the 
				// start and end nodes are duplicated)...
				if(m_iCurrentSplineNode >= iNumSplineNodes)
				{
					m_iCurrentSplineNode = iNumSplineNodes - 1;
				}

				m_iPreviousSplineNode = m_iCurrentSplineNode;

				{
					const Asura_Vector_3&	xPosition		= pxLiftSpline->GetPointPosition(m_iCurrentSplineNode + 1);
					const Asura_Quat&		xOrientation	= pxLiftSpline->GetLinkOrientation(m_iCurrentSplineNode + 1);

					// Force the lift to be placed wherever the current spline-node is...
					Position	= xPosition;
					xOrientation.ToMatrix(Orientation);
				}
			}
		}
	}
	else
	{
		// Not snapping - reset the previous spline-node...
		m_iPreviousSplineNode = -1;
	}

	if ( m_bStartLiftNodeChanged )
	{
		m_bStartLiftNodeChanged = false;

		Asura_ServerNode_LiftSpline* pxLiftSpline = Asura_ServerNode_LiftSpline::FindEntity( m_pxLift->GetSplineGuid() );
		if ( pxLiftSpline )
		{
			m_pxLift->SetInitialNode( m_iStartLiftNode );

			Asura_Vector_3 xPosition;
			Asura_Quat	   xOrientation;
			pxLiftSpline->GetPositionFromProportion( m_pxLift->GetProgress(), xPosition, &xOrientation );

			Position = xPosition;
			xOrientation.ToMatrix( Orientation );
			UpdatePositionInAsuraData( true );
		}

		Changed = true;
	}
}
    
void Tools_Entity_Lift::Render()
{
	if (IsSelected())
	{
		const Asura_ServerEntity_Node* pxSpline = Asura_ServerEntity_Node::FindEntity(m_pxLift->GetSplineGuid());
		if (pxSpline)
		{
			Asura_Render::RenderLine(Position,pxSpline->GetPosition(),0xFF7FFF7F);
		}
	}
	if (m_bPreviewing)
	{
		Asura_Vector_3 xPos(Position);
		Asura_Matrix_3x3 xOri(Orientation);
		Asura_ServerNode_LiftNode* pxArrivedAtNode = NULL;
		bool bFinished = Asura_ServerEntity_Lift::UpdateLift( m_uPreviewSpline, m_fPreviewProgress, m_bPreviewReverse,
			m_pxLift->GetAutoReverse(), xPos, xOri, ASURA_GUID_UNREGISTERED, pxArrivedAtNode, false );
		if(m_iPreviewSoundHandle != ASURA_INVALID_HANDLE)
		{
			Asura_Sound_Event_System::UpdatePosition(m_iPreviewSoundHandle, xPos);
		}
		if (GetAnimation())
		{
			u_int uRenderFlags = 0;
			if(!ShouldRenderShadow())
			{
				uRenderFlags |= ASURA_HIERARCHY_RENDER_NO_SHADOW;
			}
			if(!UseModuleLinking())
			{
				uRenderFlags |= ASURA_HIERARCHY_RENDER_NO_MODULELINK_TEST;
			}
			GetAnimation()->Render(xPos, xOri, uRenderFlags);
		}
		else
		{
			const Asura_Hash_ID	uShapeHashID	= GetShapeOrSkinFileID();

			g_xShapeInstance.SetShape(uShapeHashID);
			g_xShapeInstance.SetNewPosAndOri(xPos, xOri);
			g_xShapeInstance.Render();
		}
		if (bFinished)
		{
			PreviewEnd();
		}
	}
	PARENT::Render();
}

void Tools_Entity_Lift::AddOverlayControls(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

    Asura_Dialog* pxDialog;

	xOffset.x += Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();

	//Set up Tabs
//	static const char* s_aszTabs[] = {"Shape & Anim","Merge","Misc"};

	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset,0,0);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxOverlay->SetCurrentParentDialog(pxTabsDialog);

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	//Shape and anim tab
	pxTabsDialog->AddTab("Shape & Anim");
    
    pxDialog = new Asura_Function_Dialog("Select Shape", xOffset, SelectShape_Dialog);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Select Skin and Animation", xOffset, SelectSkinAndAnim_Dialog);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Select Animation", xOffset, SelectAnim_Dialog);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_StringDisplay_Dialog("Animation flags:", xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

    AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_Looping );
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_Reversed );
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_UseAnimMovement );
	xOffset.y+=Asura_Dialog::LargeSpacing();

	if (Type == TOT_Entity_Lift)
	{
		pxDialog = new Asura_Function_Dialog("Select all with this shape", xOffset, SelectAllOfType);
		pxDialog->SetHelpText("Selects all lifts that use the same shape/skin as this one.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;


	//Misc stuff tab
	pxTabsDialog->AddTab("Misc");
	xOffset.y = fTabsStartY;

    AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_NoShadow );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_NoCollisions );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_Hidden );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_IgnoreForLOS );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_IgnoreModuleLinking );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_FullBright );
	xOffset.y += Asura_Dialog::DefaultSpacing();

    AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_Polygonal );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	// Flags tab...
	pxTabsDialog->AddTab("Flags");
	xOffset.y = fTabsStartY;

	AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_Running );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_Reverse );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_AutoReverse );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_CollideEnv );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_CollideStatic );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_CollideDynamic );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	AddSharedParamDialogToOverlay( pxOverlay, xOffset, g_uHash_Lift_PauseOnCollide );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	// Spline tab...
	pxTabsDialog->AddTab("Spline");
	xOffset.y = fTabsStartY;

	pxDialog = new Asura_Function_Dialog( "Select Existing Spline", xOffset, SelectSpline, NULL, IsPreviewInactive );
	pxDialog->SetHelpText( "Select an existing spline for the lift to start on." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Reset Spline", xOffset, ResetSpline, NULL, IsPreviewInactiveAndLiftSplinePresent );
	pxDialog->SetHelpText( "Deselect the spline the lift starts on." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Add New Lift-Node At Current Position", xOffset, AddLiftNodeAtCurrentLiftPosition, NULL, CanAddLiftNode );
	pxDialog->SetHelpText( "Adds a new lift-node and spline to this lift." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Snap Lift To Spline-Node", xOffset, &m_bSnapLiftToSplineNode, &m_bSnapLiftToSplineNodeChanged );
	pxDialog->SetHelpText( "Forces the lift to be snapped to the specified node on the current spline and allows the lift (and hence the node) to be moved and rotated." );
	pxDialog->SetIsEnabledFunction( IsPreviewInactiveAndLiftSplinePresent );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_ServerNode_LiftSpline* pxLiftSpline = Asura_ServerNode_LiftSpline::FindEntity( m_pxLift->GetSplineGuid() );
	const int iNumSplineNodes = pxLiftSpline ? ( pxLiftSpline->GetNumberOfPoints() - 2 ) : 2;
	pxDialog = new Asura_Numerical_Dialog("Snap Spline-Node", xOffset, &m_iCurrentSplineNode, 0, iNumSplineNodes - 1, 1, "%d");
	pxDialog->SetHelpText( "The current node on the spline to snap the lift to." );
	pxDialog->SetIsEnabledFunction( CanSetSnapSplineNode );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Tickbox_Dialog( "Show Lift At Start Position", xOffset, &m_bShowStartPosition, &m_bShowStartPositionChanged );
	pxDialog->SetHelpText( "Renders the lift at its start position." );
	pxDialog->SetIsEnabledFunction( IsPreviewInactiveAndLiftSplinePresent );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Start Lift-Node", xOffset, &m_iStartLiftNode, 0, 1, 1, "%d", &m_bStartLiftNodeChanged );
	pxDialog->SetHelpText( "Lift-node the lift should start on." );
	pxDialog->SetIsEnabledFunction( CanSetStartPosition );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Function_Dialog( "Start Preview", xOffset, Preview, NULL, IsPreviewInactiveAndLiftSplinePresent );
	pxDialog->SetHelpText( "previews lift movement" );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Stop Preview", xOffset, PreviewEnd, NULL, IsPreviewActiveAndLiftSplinePresent );
	pxDialog->SetHelpText( "stops current preview" );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//Sound tab...

	pxTabsDialog->AddTab("Sounds");
	xOffset.y = fTabsStartY;

	Asura_Dialog* pxSoundEventDialog = new Tools_PickSoundEvent_Dialog("Start sound event: ",xOffset,m_pxLift->GetStartSoundEventHashPointer(), &m_bChangedSoundEvent);
	pxSoundEventDialog->SetHelpText("The sound event to play when the lift starts.");
	pxOverlay->AddDialog(pxSoundEventDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxSoundEventDialog = new Tools_PickSoundEvent_Dialog("Stop sound event: ",xOffset,m_pxLift->GetStopSoundEventHashPointer(), &m_bChangedSoundEvent);
	pxSoundEventDialog->SetHelpText("The sound event to play when the lift stops.");
	pxOverlay->AddDialog(pxSoundEventDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;


	//end of tabs
	xOffset.y = fTabsEndY + 1.0f;
	pxOverlay->SetCurrentParentDialog(0);
	pxTabsDialog->SelectTab(0);

	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);


	// Remaining dialogues...
	pxDialog = new Asura_Function_Dialog("Selected Lift Control Points use this shape", xOffset, ControlPointsUseThisShape);
	pxDialog->SetHelpText("Selected lift control points mimic the shape of this lift, so they can be placed the match the lift.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Reset Lift Control Points to default shape", xOffset, ControlPointsUseDefaultShape);
	pxDialog->SetHelpText("resets all the control points back to their default shape.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
}

void Tools_Entity_Lift::AddFilesToExportList(List<Asura_String>& xNameList) const
{
	if (m_pxLift )
	{
		Tools_Level_Export::AddSoundEventToExportList(m_pxLift->GetStartSoundEventHash());
		Tools_Level_Export::AddSoundEventToExportList(m_pxLift->GetStopSoundEventHash());
	}

	PARENT::AddFilesToExportList(xNameList);
}

void Tools_Entity_Lift::SelectSpline()
{
	if (!s_pxCurrentlySelectedObject)
    {
        return;
    }
    Tools_Lift_SelectSpline* pxChooseObject = new Tools_Lift_SelectSpline( s_pxCurrentlySelectedObject->m_pxLift->GetSplineGuidPtr() );
    Tools_Object_System::ChooseObject( "Pick the spline this lift will start on", pxChooseObject );
}

void Tools_Entity_Lift::AddLiftNodeAtCurrentLiftPosition()
{
	s_pxCurrentlySelectedObject->AddLiftNodeAtCurrentPosition();
}

void Tools_Entity_Lift::AddLiftNodeAtCurrentPosition()
{
	Tools_SecondaryEntityFile_System::PreCreateObject();

	if ( m_pxLift->GetSplineGuid() == ASURA_GUID_UNREGISTERED )
	{
		if ( m_xCreationList.Back() )
		{
			// No spline, but a node - create a second node, then make a spline from our 2 nodes.
			const Asura_Guid uStartGuid = m_xCreationList.Back()->Data();
			Tools_Entity_Lift::CreateLiftNodeAtCurrentPosition();
			Tools_EntityNode_LiftSpline* pxLiftSpline = Tools_EntityNode_LiftSpline::CreateAndReturn(uStartGuid, m_xCreationList.Back()->Data());
			if ( pxLiftSpline )
			{
				m_pxLift->SetInitialSplineGuid( pxLiftSpline->GetGuid() );
			}
		}
		else
		{
			// No spline or node - create a first node.
			Tools_Entity_Lift::CreateLiftNodeAtCurrentPosition();
		}
	}
	else if ( m_xCreationList.Back() )
	{
		// Create a second spline and another node.
		const Asura_Guid uStartGuid = m_xCreationList.Back()->Data();
		Tools_Entity_Lift::CreateLiftNodeAtCurrentPosition();
		Tools_EntityNode_LiftSpline::CreateAndReturn(uStartGuid, m_xCreationList.Back()->Data());
	}

	// Ensure that our lift is selected again...
	Tools_Object_System::SelectObject(this);

	Tools_SecondaryEntityFile_System::PostCreateObject();
}

void Tools_Entity_Lift::ResetSpline()
{
	if (!s_pxCurrentlySelectedObject)
    {
        return;
    }
    s_pxCurrentlySelectedObject->m_pxLift->SetInitialSplineGuid(ASURA_GUID_UNREGISTERED);
}

void Tools_Entity_Lift::Preview()
{
	if (!s_pxCurrentlySelectedObject)
    {
        return;
    }
	s_pxCurrentlySelectedObject->m_bPreviewing = true;
	s_pxCurrentlySelectedObject->m_bPreviewReverse = s_pxCurrentlySelectedObject->m_pxLift->GetReversing();
	s_pxCurrentlySelectedObject->m_uPreviewSpline = s_pxCurrentlySelectedObject->m_pxLift->GetSplineGuid();
	s_pxCurrentlySelectedObject->m_fPreviewProgress = s_pxCurrentlySelectedObject->m_pxLift->GetProgress();
	s_pxCurrentlySelectedObject->PlayLiftSound(s_pxCurrentlySelectedObject->m_pxLift->GetStartSoundEventHash());
}

void Tools_Entity_Lift::PreviewEnd()
{
	if (!s_pxCurrentlySelectedObject)
    {
        return;
    }
	s_pxCurrentlySelectedObject->m_bPreviewing = false;
	s_pxCurrentlySelectedObject->PlayLiftSound(s_pxCurrentlySelectedObject->m_pxLift->GetStopSoundEventHash());
}

void Tools_Entity_Lift::PlayLiftSound(Asura_Hash_ID uSoundEventHash)
{
	if( m_iPreviewSoundHandle != ASURA_INVALID_HANDLE )
	{
		Asura_Sound_Event_System::Stop( m_iPreviewSoundHandle );
		m_iPreviewSoundHandle = ASURA_INVALID_HANDLE;
	}
	const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent( uSoundEventHash );
	if( pxEvent )
	{
		Asura_Vector_3 xPos(Position);
		Asura_Matrix_3x3 xOri(Orientation);
		Asura_ServerNode_LiftNode* pxArrivedAtNode = NULL;
		Asura_ServerEntity_Lift::UpdateLift( m_uPreviewSpline, m_fPreviewProgress, m_bPreviewReverse,
			m_pxLift->GetAutoReverse(), xPos, xOri, ASURA_GUID_UNREGISTERED, pxArrivedAtNode, false );
		if( pxEvent->IsEndlessPlayback() )
		{
			Asura_Sound_Event_System::PlayEvent( pxEvent, &m_iPreviewSoundHandle, &xPos, 1.0f );
		}
		else
		{
			Asura_Sound_Event_System::PlayEvent( pxEvent, NULL, &xPos, 1.0f );
		}
	}
}

void Tools_Entity_Lift::SelectShape_Dialog()
{
    Asura_Assert(s_pxCurrentlySelectedObject, "Tools_Entity_Lift::SelectShape_Dialog() called with no object selected");
	if (s_pxCurrentlySelectedObject)
	{
        s_pxCurrentlySelectedObject->SelectShape();
	}
}


//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Tools_Entity_Lift::SelectSkinAndAnim_Dialog().
// PARAMETERS   None.
// RETURNS      None.
// DESCRIPTION  Static member function called when this option is selected in 
//              the tools overlay,
// AUTHOR       Mike Healey - 17.10.2002.
//
//////////////////////////////////////////////////////////////////////////////
void Tools_Entity_Lift::SelectSkinAndAnim_Dialog()
{
    Asura_Assert(s_pxCurrentlySelectedObject, "Tools_Entity_Lift::SelectSkinAndAnim_Dialog() called with no object selected");
	if (s_pxCurrentlySelectedObject)
	{
        s_pxCurrentlySelectedObject->SelectSkinAndAnim();
	}
}
// End function Tools_Entity_Lift::SelectSkinAndAnim_Dialog().


//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Tools_Entity_Lift::SelectAnim_Dialog().
// PARAMETERS   None.
// RETURNS      None.
// DESCRIPTION  Static member function called when this option is selected in 
//              the tools overlay,
// AUTHOR       Mike Healey - 18.10.2002.
//
//////////////////////////////////////////////////////////////////////////////
void Tools_Entity_Lift::SelectAnim_Dialog()
{
    Asura_Assert(s_pxCurrentlySelectedObject, "Tools_Entity_Lift::SelectAnim_Dialog() called with no object selected");
	if (s_pxCurrentlySelectedObject)
	{
        s_pxCurrentlySelectedObject->SelectAnim();
	}
}
// End function Tools_Entity_Lift::SelectAnim_Dialog().


void Tools_Entity_Lift::SelectShape()
{
	if (!IsSelected()) return;

	Asura_Hash_ID uShapeHashID = Tools_ShapeTable::SelectShape();

	if( ASURA_HASH_ID_UNSET != uShapeHashID )
	{
		Changed = true;

		if (m_pxPhysicalObject)
		{
			// Update server object's shape ID.
			m_pxPhysicalObject->SetShapeFileID( uShapeHashID );

			// Update the tools shape also
			SetToolsShape( uShapeHashID );

			//Shape has changed , so if merged to environment , we need to unmerge now
			UnmergePolygonsFromEnvironment();


		}

        // (MH - 28.11.2002) attempt to get the textures in for this object
        Asura_Texture_Management::LoadTextures();
    }
}


//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Tools_Entity_Lift::SelectSkinAndAnim().
// PARAMETERS   None.
// RETURNS      None.
// DESCRIPTION  Displays a skin selection dialog, and if that gets a skin, 
//              displays an anim selection dialog. If both are valid, updates
//              the tools object and server entity with the new values.
// AUTHOR       Mike Healey - 17.10.2002.
//
//////////////////////////////////////////////////////////////////////////////
void Tools_Entity_Lift::SelectSkinAndAnim()
{
    Asura_Assert(IsSelected(), "SelectSkinAndAnim() called on non-selected object");
    if (!IsSelected()) return;

    Asura_Assert(m_pxPhysicalObject, "Tools_Entity_Lift has no m_pxPhysicalObject");

    Asura_String xSkinName;
    Asura_String xSkinFilename;
    Asura_String xAnimName;
    Asura_String xAnimFilename;

    if (!Tools_Animation_Files::ChooseSkin(&xSkinName, &xSkinFilename))
    {
        return;
    }

    Tools_Animation_Files::ChooseAnimation(xSkinName.GetString(), NULL, &xAnimName, &xAnimFilename,0,true);


    Asura_Hash_ID uSkinFileHashID   = Asura_GetHashID(xSkinFilename.GetString());
    Asura_Hash_ID uSkinHashID       = Asura_GetHashID(xSkinName.GetString());
    Asura_Hash_ID uAnimFileHashID   = Asura_GetHashID(xAnimFilename.GetString());
    Asura_Hash_ID uAnimHashID       = Asura_GetHashID(xAnimName.GetString());
    SetToolsSkinAndAnim(uSkinFileHashID, uSkinHashID, uAnimHashID, uAnimFileHashID);
    m_pxPhysicalObject->SetSkinAndAnimID(uSkinHashID, uAnimHashID);
    m_pxPhysicalObject->SetSkinAndAnimFileID(uSkinFileHashID, uAnimFileHashID);

	//Shape has changed , so if merged to environment , we need to unmerge now
	UnmergePolygonsFromEnvironment();


    
    // (MH - 28.11.2002) attempt to get the textures in for this skin
    Asura_Texture_Management::LoadTextures();
}
// End function Tools_Entity_Lift::SelectSkinAndAnim().


//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Tools_Entity_Lift::SelectAnim().
// PARAMETERS   None.
// RETURNS      None.
// DESCRIPTION  Displays an anim selection dialog. If valid, updates
//              the tools object and server entity with the new value.
// AUTHOR       Mike Healey - 18.10.2002.
//
//////////////////////////////////////////////////////////////////////////////
void Tools_Entity_Lift::SelectAnim()
{
    Asura_Assert(IsSelected(), "SelectAnim() called on non-selected object");
    if (!IsSelected()) return;

    Asura_Assert(m_pxPhysicalObject, "Tools_Entity_Lift has no m_pxPhysicalObject");
    if (!m_pxPhysicalObject) return;

    if (!m_pxPhysicalObject->HasSkinOrHierarchy())
    {
        Tools_Timed_Message_System::AddMessage(0xffffff, "object is not animating");
        return;
    }

    const Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkinByHash(m_pxPhysicalObject->GetSkinID());
    Asura_Assert(pxSkin, "skin hash ID failed to find skin");
    if (!pxSkin) return;

    Asura_String xAnimName;
    Asura_String xAnimFilename;
	Asura_Hierarchy_Anim* pxInitialAnim = 0;
	if(m_pxPhysicalObject->GetAnimID())
	{
		pxInitialAnim = Asura_Animation_System::GetAnimByHash(m_pxPhysicalObject->GetAnimID());
	}


    if(Tools_Animation_Files::ChooseAnimation(pxSkin->Name.GetString(), NULL, &xAnimName, &xAnimFilename,pxInitialAnim,true) == pxInitialAnim)
	{
		//If user cancels and we get back the original animation ,then the anim name and filename won't have been set , 
		//so return here instead. RAR 27/06/2006
		return;
	}

    Asura_Hash_ID uSkinFileHashID   = m_pxPhysicalObject->GetSkinFileID();
    Asura_Hash_ID uSkinHashID       = m_pxPhysicalObject->GetSkinID();
    Asura_Hash_ID uAnimFileHashID   = Asura_GetHashID(xAnimFilename.GetString());
    Asura_Hash_ID uAnimHashID       = Asura_GetHashID(xAnimName.GetString());
    SetToolsSkinAndAnim(uSkinFileHashID, uSkinHashID, uAnimHashID, uAnimFileHashID);
    m_pxPhysicalObject->SetSkinAndAnimID(uSkinHashID, uAnimHashID);
    m_pxPhysicalObject->SetSkinAndAnimFileID(uSkinFileHashID, uAnimFileHashID);

	Changed = true;
}
// End function Tools_Entity_Lift::SelectAnim().


bool Tools_Entity_Lift::IsNoneAnimatingObject()
{
	if (Tools_Object_System::SelectedObject)
	{
		if (s_pxCurrentlySelectedObject)
		{
			return (!s_pxCurrentlySelectedObject->m_pxPhysicalObject->HasSkinOrHierarchy());
		}
	}
	return false;
}

bool Tools_Entity_Lift::IsLiftSplinePresent()
{
	return ( s_pxCurrentlySelectedObject && ( s_pxCurrentlySelectedObject->m_pxLift->GetSplineGuid() != ASURA_GUID_UNREGISTERED ) );
}

bool Tools_Entity_Lift::IsLiftSplineMissing()
{
	return !IsLiftSplinePresent();
}

bool Tools_Entity_Lift::IsPreviewActiveAndLiftSplinePresent()
{
	return ( !IsPreviewInactive() && IsLiftSplinePresent() );
}

bool Tools_Entity_Lift::IsPreviewInactive()
{
	return ( s_pxCurrentlySelectedObject && !s_pxCurrentlySelectedObject->m_bPreviewing );
}

bool Tools_Entity_Lift::IsPreviewInactiveAndLiftSplinePresent()
{
	return ( IsPreviewInactive() && IsLiftSplinePresent() );
}

bool Tools_Entity_Lift::AlreadyCreatedNode()
{
	return ( s_pxCurrentlySelectedObject && s_pxCurrentlySelectedObject->m_xCreationList.Back() );
}

bool Tools_Entity_Lift::CanAddLiftNode()
{
	return ( IsPreviewInactive() && ( !IsLiftSplinePresent() || AlreadyCreatedNode() ) );
}

bool Tools_Entity_Lift::CanSetSnapSplineNode()
{
	return ( IsPreviewInactiveAndLiftSplinePresent() && s_pxCurrentlySelectedObject && s_pxCurrentlySelectedObject->m_bSnapLiftToSplineNode );
}

bool Tools_Entity_Lift::CanSetStartPosition()
{
	return ( IsPreviewInactiveAndLiftSplinePresent() && s_pxCurrentlySelectedObject && s_pxCurrentlySelectedObject->m_bShowStartPosition );
}

void Tools_Entity_Lift::SelectAllOfType()
{
	if (s_pxCurrentlySelectedObject)
	{
		Asura_Hash_ID uShapeHashID = s_pxCurrentlySelectedObject->m_pxLift->GetShapeFileID();
		Asura_Hash_ID uSkinHashID = s_pxCurrentlySelectedObject->m_pxLift->GetSkinID();

		List<Tools_Object*> xListObjects;
		for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
		{
			Asura_Entity_Server* pxEntity = it.GetCurrent();
			if (pxEntity->GetClassification() == AsuraEntityClass_Lift)
			{
				Asura_Guid uGuid = pxEntity->GetGuid();
				Asura_ServerEntity_PhysicalObject* pxPhysicalObject = Asura_ServerEntity_PhysicalObject::FindEntity(uGuid);
				if (!pxPhysicalObject)
				{
					continue;
				}
				Asura_Hash_ID uShapeHashID2 = pxPhysicalObject->GetShapeFileID();
				Asura_Hash_ID uSkinHashID2 = pxPhysicalObject->GetSkinID();
				if (uShapeHashID2 == uShapeHashID && uSkinHashID2 == uSkinHashID)
				{
					Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(uGuid);
					xListObjects.add_entry(pxObject);
				}
			}
		}
		Tools_Object_System::SelectObjects(xListObjects, true);
	}
}

void Tools_Entity_Lift::ControlPointsUseThisShape()
{
	if (s_pxCurrentlySelectedObject)
	{
		for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::SelectedObjectList);!xObjectLIF.done();xObjectLIF.next())
		{
			if (xObjectLIF()->Type == TOT_Entity_LiftSpline_Point
				|| xObjectLIF()->Type == TOT_Entity_LiftNode)
			{
				if (Tools_Object_System::SelectedObject->GetAnimation())
				{
					xObjectLIF()->SetSkinAndAnim(s_pxCurrentlySelectedObject->GetAnimation()->GetSkin(), 0);
				}
				else
				{
					xObjectLIF()->SetMesh(s_pxCurrentlySelectedObject->Mesh);
				}
			}
		}
	}
}

void Tools_Entity_Lift::ControlPointsUseDefaultShape()
{
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if (xObjectLIF()->Type == TOT_Entity_LiftSpline_Point
			|| xObjectLIF()->Type == TOT_Entity_LiftNode)
		{
			xObjectLIF()->SetMesh("ToolsShape_CutsceneTrackPoint");
		}
	}
}

void Tools_Entity_Lift::CreateLiftNodeAtCurrentPosition()
{
	Tools_SecondaryEntityFile_System::PreCreateObject();

	Tools_Object*				pxOldToolsObject	= Tools_Object_System::SelectedObject;

	Tools_Object*				pxToolsObject		= Tools_EntityNode_LiftNode::CreateAndReturn();
	const Asura_Guid			uGuid				= pxToolsObject->GetGuid();
	Tools_EntityNode_LiftNode*	pxToolsLiftNode		= static_cast<Tools_EntityNode_LiftNode*>(Tools_EntityNode_LiftNode::FindEntity(uGuid));

	// Update the new lift-node's position...
	pxToolsLiftNode->Position		= pxOldToolsObject->Position;
	pxToolsLiftNode->Orientation	= pxOldToolsObject->Orientation;
	pxToolsLiftNode->UpdatePositionInAsuraData(true);

	// Ensure that our lift is selected again...
	Tools_Object_System::SelectObject(pxOldToolsObject);

	// Finally, store the GUID...
	s_pxCurrentlySelectedObject->m_xCreationList.Push_Back( uGuid );

	Tools_SecondaryEntityFile_System::PostCreateObject();
}

void Tools_Entity_Lift::Copy(const Tools_Object* pxObject)
{
	const Tools_Entity_Lift* pxFromObject = s_xList.FindEntity(pxObject->GetGuid());
	if(!pxFromObject) return;

	//get the entities to copy from and to
	const Asura_ServerEntity_Lift* pxFromEntity = pxFromObject->m_pxLift;
	Asura_ServerEntity_Lift* pxToEntity = m_pxLift;

	//copy everything between the entities
	m_iAnimFlags = pxFromEntity->GetAnimFlags();
	pxToEntity->SetAnimFlags(m_iAnimFlags);

	pxToEntity->SetRunning(pxFromEntity->IsRunning());
	pxToEntity->SetReversing(pxFromEntity->GetReversing());
	pxToEntity->SetAutoReverse(pxFromEntity->GetAutoReverse());
	pxToEntity->SetProgress(pxFromEntity->GetProgress());
	pxToEntity->SetCollideEnv(pxFromEntity->GetCollideEnv());
	pxToEntity->SetCollideStatic(pxFromEntity->GetCollideStatic());
	pxToEntity->SetCollideDynamic(pxFromEntity->GetCollideDynamic());
	pxToEntity->SetPauseOnCollide(pxFromEntity->GetPauseOnCollide());
	pxToEntity->SetInitialSplineGuid(pxFromEntity->GetSplineGuid());

	//Parent Copy() not called since it should have been dealt with by the project class
}


Tools_Object* Tools_Entity_Lift::CreateObjectOfThisType() const
{
	Tools_Object* pxOldSelection = Tools_Object_System::SelectedObject;
	Tools_Entity_Lift::Project_Create();
	Tools_Object* pxNewSelection = Tools_Object_System::SelectedObject;

	return (pxOldSelection!=pxNewSelection) ? pxNewSelection : 0;
}

bool Tools_Entity_Lift::UseModuleLinking()
{
	return !(m_uPhysicalObjectFlags & (ASURA_PHYSICALOBJECT_FLAG_IGNOREMODULELINKING));
}

bool Tools_Entity_Lift::HandlesThisMessage(int iMessageID)
{
	if(iMessageID == ASURA_MSG_HIDE
		|| iMessageID == ASURA_MSG_UNHIDE
		|| iMessageID == ASURA_MSG_COLLISIONSON
		|| iMessageID == ASURA_MSG_COLLISIONSOFF
		|| iMessageID == ASURA_MSG_ACTIVATE
		|| iMessageID == ASURA_MSG_DEACTIVATE
		|| iMessageID == ASURA_MSG_LIFT_START
		|| iMessageID == ASURA_MSG_LIFT_STOP
		|| iMessageID == ASURA_MSG_LIFT_REVERSE
		|| iMessageID == ASURA_MSG_LIFT_GOTO_NODE
		|| iMessageID == ASURA_MSG_LIFT_SETFLAGS) return true;

	//Calling parent to avoid forcing projects to define this function for derived classes
	//(since it doesn't really matter if the parent version gets called twice)
	return PARENT::HandlesThisMessage(iMessageID);
}

bool Tools_Entity_Lift::HandlesThisEvent(int iEventID) const
{
	if (iEventID == ASURA_ENTITY_EVENT_ONARRIVEDAT
		|| iEventID == ASURA_ENTITY_EVENT_ONDEPARTEDFROM
		|| iEventID == ASURA_ENTITY_EVENT_ONSTARTED
		|| iEventID == ASURA_ENTITY_EVENT_ONSTOPPED
		|| iEventID == ASURA_ENTITY_EVENT_ONREVERSED)
	{
		return true;
	}
	else if( (ASURA_ENTITY_EVENT_ONHEALTHPERCENT_0 <= iEventID) && (iEventID <= ASURA_ENTITY_EVENT_ONHEALTHPERCENT_99) ) 
	{
		// NY - 17 July 2008 - these are probably spurious for most project's lifts. project function can always override this anyway
		return false;
	}
	return PARENT::HandlesThisEvent(iEventID);
}

bool Tools_Entity_Lift::ShouldHideDuringCutscene()
{
	//Want to show physics objects , unless they haven't had a shape/skin assigned to them
	return !GetShapeOrSkinFileID();

}

void Tools_Entity_Lift::AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext)
{
	PARENT::AddContextMenuItems(xMenu, xContext);

	
	Tools_ContextMenu_Item* pxItem;

	Tools_ContextMenu_Section* pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_SELECTION);
	pxSection->AddSeparator();

	//Add option for selecting/deselecting the lift and its spline objects
	u_int uMode;
	const char* szText;
	if(xContext.m_bShiftHeld)
	{
		szText = "Deselect lift & spline points";
		uMode = 0;
	}
	else if(xContext.m_bControlHeld)
	{
		szText = "Select lift & spline points";
		uMode = 1;
	}
	else
	{
		szText = "Select lift & spline points";
		uMode = 2;
	}
	
	pxItem = new Tools_ContextMenu_Item_Function(szText, ContextMenu_SelectLiftAndSpline, this, uMode);
	pxItem->SetID("Select lift & spline points");
	pxItem->SetHelpText("Selects/deselects the lift and its spline points");
	pxSection->AddItem(pxItem);
}

void Tools_Entity_Lift::ContextMenu_SelectLiftAndSpline(void* pObject, u_int uMode)
{
	Tools_Entity_Lift* pxLiftObject = (Tools_Entity_Lift*)pObject;

	List<Tools_Object*> xSelectionList;
	
	//get the list of spline point objects , and add them to the selection list
	Asura_Guid uSplineGuid = pxLiftObject->m_pxLift->GetSplineGuid();
	Tools_EntityNode_LiftSpline* pxToolsLiftSpline = static_cast<Tools_EntityNode_LiftSpline*>(Tools_EntityNode_LiftSpline::FindEntity(uSplineGuid));
	if(pxToolsLiftSpline)
	{
		const List<Tools_Object_NodeSplinePoint*>& xPointList = pxToolsLiftSpline->GetSplinePointObjects();
		for(CLIF<Tools_Object_NodeSplinePoint*> xPointLIF(&xPointList); !xPointLIF.done(); xPointLIF.next())
		{
			xSelectionList.add_entry(xPointLIF());
		}
	}
	
	//Get the asura spline, so we can get the start/end nodes , and select those as well
	Asura_ServerNode_LiftSpline* pxLiftSpline = Asura_ServerNode_LiftSpline::FindEntity(uSplineGuid);
	if(pxLiftSpline)
	{
		Tools_Object* pxSourceNode = Tools_Object_System::GetObjectFromGuid(pxLiftSpline->GetSourceGuid());
		if(pxSourceNode)
		{
			xSelectionList.add_entry(pxSourceNode);
		}

		Tools_Object* pxTargetNode = Tools_Object_System::GetObjectFromGuid(pxLiftSpline->GetTargetGuid());
		if(pxTargetNode)
		{
			xSelectionList.add_entry(pxTargetNode);
		}

	}

	//add the lift object itself
	xSelectionList.add_entry(pxLiftObject);

	//And (de)select them
	if(uMode == 0)
	{
		Tools_Object_System::DeselectObjects(xSelectionList);
	}
	else
	{
		Tools_Object_System::SelectObjects(xSelectionList, uMode == 1);
	}
}



bool Tools_Entity_Lift::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{
	switch(uParamID)
	{
		case g_uHash_Lift_Looping:
		{
			pxParam->SetBool((m_iAnimFlags & AsuraAnimationFlag_Loop)!=0);
			break;
		}
		case g_uHash_Lift_Reversed:
		{
			pxParam->SetBool((m_iAnimFlags & AsuraAnimationFlag_Reversed)!=0);
			break;
		}
		case g_uHash_Lift_UseAnimMovement:
		{
			pxParam->SetBool((m_uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_USEANIMMOVEMENT)!=0);
			break;
		}
		case g_uHash_Lift_NoShadow:
		{
			pxParam->SetBool((m_uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_NOSHADOW)!=0);
			break;
		}
		case g_uHash_Lift_NoCollisions:
		{
			pxParam->SetBool((m_uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_NOCOLLISIONS)!=0);
			break;
		}
		case g_uHash_Lift_Hidden:
		{
			pxParam->SetBool((m_uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_INVISIBLE)!=0);
			break;
		}
		case g_uHash_Lift_IgnoreForLOS:
		{
			pxParam->SetBool((m_uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_IGNOREFORLOS)!=0);
			break;
		}
		case g_uHash_Lift_IgnoreModuleLinking:
		{
			pxParam->SetBool((m_uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_IGNOREMODULELINKING)!=0);
			break;
		}
		case g_uHash_Lift_FullBright:
		{
			pxParam->SetBool((m_uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_FULLBRIGHT)!=0);
			break;
		}
		case g_uHash_Lift_Polygonal:
		{
			pxParam->SetBool(m_pxLift->GetCollisionProxy() == Asura_Physics_Object::COLLISION_PROXY_POLYGONAL);
			break;
		}
		case g_uHash_Lift_Running:
		{
			pxParam->SetBool(m_pxLift->IsRunning());
			break;
		}
		case g_uHash_Lift_Reverse:
		{
			pxParam->SetBool(m_pxLift->GetReversing());
			break;
		}
		case g_uHash_Lift_AutoReverse:
		{
			pxParam->SetBool(m_pxLift->GetAutoReverse());
			break;
		}
		case g_uHash_Lift_CollideEnv:
		{
			pxParam->SetBool(m_pxLift->GetCollideEnv());
			break;
		}
		case g_uHash_Lift_CollideStatic:
		{
			pxParam->SetBool(m_pxLift->GetCollideStatic());
			break;
		}
		case g_uHash_Lift_CollideDynamic:
		{
			pxParam->SetBool(m_pxLift->GetCollideDynamic());
			break;
		}
		case g_uHash_Lift_PauseOnCollide:
		{
			pxParam->SetBool(m_pxLift->GetPauseOnCollide());
			break;
		}
		default:
		{
			return false;
		}
	}
	return true;
}

bool Tools_Entity_Lift::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	switch(uParamID)
	{
		case g_uHash_Lift_Looping:
		{
			if(pxParam->GetBool())
			{
				m_iAnimFlags |= AsuraAnimationFlag_Loop;
			}
			else
			{
				m_iAnimFlags &=~ AsuraAnimationFlag_Loop;
			}
			m_pxPhysicalObject->SetAnimFlags(m_iAnimFlags);
			break;
		}
		case g_uHash_Lift_Reversed:
		{
			if(pxParam->GetBool())
			{
				m_iAnimFlags |= AsuraAnimationFlag_Reversed;
			}
			else
			{
				m_iAnimFlags &=~ AsuraAnimationFlag_Reversed;
			}
			m_pxPhysicalObject->SetAnimFlags(m_iAnimFlags);
			break;
		}
		case g_uHash_Lift_UseAnimMovement:
		{
			if(pxParam->GetBool())
			{
				m_uPhysicalObjectFlags |= ASURA_PHYSICALOBJECT_FLAG_USEANIMMOVEMENT;
			}
			else
			{
				m_uPhysicalObjectFlags &=~ ASURA_PHYSICALOBJECT_FLAG_USEANIMMOVEMENT;
			}
			m_pxPhysicalObject->SetPhysicalObjectFlags(m_uPhysicalObjectFlags);
			SetUseAnimMovement((m_uPhysicalObjectFlags & ASURA_PHYSICALOBJECT_FLAG_USEANIMMOVEMENT)!=0);
			break;
		}
		case g_uHash_Lift_NoShadow:
		{
			if(pxParam->GetBool())
			{
				m_uPhysicalObjectFlags |= ASURA_PHYSICALOBJECT_FLAG_NOSHADOW;
			}
			else
			{
				m_uPhysicalObjectFlags &=~ ASURA_PHYSICALOBJECT_FLAG_NOSHADOW;
			}
			m_pxPhysicalObject->SetPhysicalObjectFlags(m_uPhysicalObjectFlags);
			break;
		}
		case g_uHash_Lift_NoCollisions:
		{
			if(pxParam->GetBool())
			{
				m_uPhysicalObjectFlags |= ASURA_PHYSICALOBJECT_FLAG_NOCOLLISIONS;
			}
			else
			{
				m_uPhysicalObjectFlags &=~ ASURA_PHYSICALOBJECT_FLAG_NOCOLLISIONS;
			}
			if((m_uPhysicalObjectFlags^m_pxPhysicalObject->GetPhysicalObjectFlags()) & ASURA_PHYSICALOBJECT_FLAG_NOCOLLISIONS)
			{
				//no collisions flag has changed ,so polygons will need to be remerged
				UnmergePolygonsFromEnvironment();
			}
			m_pxPhysicalObject->SetPhysicalObjectFlags(m_uPhysicalObjectFlags);
			break;
		}
		case g_uHash_Lift_Hidden:
		{
			if(pxParam->GetBool())
			{
				m_uPhysicalObjectFlags |= ASURA_PHYSICALOBJECT_FLAG_INVISIBLE;
			}
			else
			{
				m_uPhysicalObjectFlags &=~ ASURA_PHYSICALOBJECT_FLAG_INVISIBLE;
			}
			if((m_uPhysicalObjectFlags^m_pxPhysicalObject->GetPhysicalObjectFlags()) & ASURA_PHYSICALOBJECT_FLAG_INVISIBLE)
			{
				//invis flag has changed ,so polygons will need to be remerged
				UnmergePolygonsFromEnvironment();
			}
			m_pxPhysicalObject->SetPhysicalObjectFlags(m_uPhysicalObjectFlags);
			break;
		}
		case g_uHash_Lift_IgnoreForLOS:
		{
			if(pxParam->GetBool())
			{
				m_uPhysicalObjectFlags |= ASURA_PHYSICALOBJECT_FLAG_IGNOREFORLOS;
			}
			else
			{
				m_uPhysicalObjectFlags &=~ ASURA_PHYSICALOBJECT_FLAG_IGNOREFORLOS;
			}
			if((m_uPhysicalObjectFlags^m_pxPhysicalObject->GetPhysicalObjectFlags()) & ASURA_PHYSICALOBJECT_FLAG_IGNOREFORLOS)
			{
				//ignore los flag has changed ,so polygons will need to be remerged
				UnmergePolygonsFromEnvironment();
			}
			m_pxPhysicalObject->SetPhysicalObjectFlags(m_uPhysicalObjectFlags);
			break;
		}
		case g_uHash_Lift_IgnoreModuleLinking:
		{
			if(pxParam->GetBool())
			{
				m_uPhysicalObjectFlags |= ASURA_PHYSICALOBJECT_FLAG_IGNOREMODULELINKING;
			}
			else
			{
				m_uPhysicalObjectFlags &=~ ASURA_PHYSICALOBJECT_FLAG_IGNOREMODULELINKING;
			}
			m_pxPhysicalObject->SetPhysicalObjectFlags(m_uPhysicalObjectFlags);
			break;
		}
		case g_uHash_Lift_FullBright:
		{
			if(pxParam->GetBool())
			{
				m_uPhysicalObjectFlags |= ASURA_PHYSICALOBJECT_FLAG_FULLBRIGHT;
			}
			else
			{
				m_uPhysicalObjectFlags &=~ ASURA_PHYSICALOBJECT_FLAG_FULLBRIGHT;
			}
			m_pxPhysicalObject->SetPhysicalObjectFlags(m_uPhysicalObjectFlags);

			// refresh skybox object list
			Asura_Skybox::InvalidateSkyboxObjects();
			break;
		}
		case g_uHash_Lift_Polygonal:
		{
			if(pxParam->GetBool())
			{
				m_pxLift->SetCollisionProxy(Asura_Physics_Object::COLLISION_PROXY_POLYGONAL);
			}
			else
			{
				m_pxLift->SetCollisionProxy(Asura_Physics_Object::COLLISION_PROXY_OBB);
			}

			break;
		}
		case g_uHash_Lift_Running:
		{
			m_pxLift->SetRunning(pxParam->GetBool());
			break;
		}
		case g_uHash_Lift_Reverse:
		{
			m_pxLift->SetReversing(pxParam->GetBool());
			break;
		}
		case g_uHash_Lift_AutoReverse:
		{
			m_pxLift->SetAutoReverse(pxParam->GetBool());
			break;
		}
		case g_uHash_Lift_CollideEnv:
		{
			m_pxLift->SetCollideEnv(pxParam->GetBool());
			break;
		}
		case g_uHash_Lift_CollideStatic:
		{
			m_pxLift->SetCollideStatic(pxParam->GetBool());
			break;
		}
		case g_uHash_Lift_CollideDynamic:
		{
			m_pxLift->SetCollideDynamic(pxParam->GetBool());
			break;
		}
		case g_uHash_Lift_PauseOnCollide:
		{
			m_pxLift->SetPauseOnCollide(pxParam->GetBool());
			break;
		}
		default:
		{
			return false;
		}
	}

	SetEntityModified();

	return true;
}


bool Tools_Entity_Lift::AddSharedParamDialogToOverlay( Asura_Overlay* pxOverlay, const Asura_Vector_2& xOffset, Asura_Hash_ID uParamID )
{
	Asura_Dialog* pxDialog = NULL;

	switch( uParamID )
	{
		case g_uHash_Lift_Looping:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Looping", xOffset, &g_xParam_Lift_Looping );
			break;
		}
		case g_uHash_Lift_Reversed:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Reversed", xOffset, &g_xParam_Lift_Reversed );
			break;
		}
		case g_uHash_Lift_UseAnimMovement:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Use Anim Movement", xOffset, &g_xParam_Lift_UseAnimMovement );
			pxDialog->SetHelpText("Causes the object to make use of the 'compass' movement from the animation for rendering and client side LOS tests.\nThis won't cause the object to move for collision purposes.");
			break;
		}
		case g_uHash_Lift_NoShadow:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "No shadow", xOffset, &g_xParam_Lift_NoShadow );
			pxDialog->SetHelpText( "Stops this object from casting a shadow." );
			break;
		}
		case g_uHash_Lift_NoCollisions:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "No collisions", xOffset, &g_xParam_Lift_NoCollisions );
			pxDialog->SetHelpText( "Stops anything from colliding with this object.\nThis flag will be copied to the polygons if merged into the environment." );
			break;
		}
		case g_uHash_Lift_IgnoreModuleLinking:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Ignore module linking", xOffset, &g_xParam_Lift_IgnoreModuleLinking );
			pxDialog->SetHelpText( "Forces shape to be rendered regardless of whether it is in a visible module." );
			break;
		}
		case g_uHash_Lift_Hidden:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Hidden", xOffset, &g_xParam_Lift_Hidden );
			pxDialog->SetHelpText( "Stops object from being rendered or hit by LOS tests.\nIf merged into the environment then the Don't Render and Ignore LOS flags will be applied." );
			break;
		}
		case g_uHash_Lift_IgnoreForLOS:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Ignore For LOS", xOffset, &g_xParam_Lift_IgnoreForLOS );
			pxDialog->SetHelpText( "Entity is ignored for all LOS tests.\nThis flag will be copied to the polygons if merged into the environment." );
			break;
		}
		case g_uHash_Lift_FullBright:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Full bright", xOffset, &g_xParam_Lift_FullBright );
			pxDialog->SetHelpText( "Object ignores lighting (both static and dynamic) and is drawn at maximum normal brightness instead." );
			break;
		}
		case g_uHash_Lift_Polygonal:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Polygonal Collision", xOffset, &g_xParam_Lift_Polygonal );
			pxDialog->SetHelpText( "Lift has mesh based collision to objects standing on it. This can be expensive, use only as needed. The lift will only use it's bounding box for obstruction testing." );
			break;
		}
		case g_uHash_Lift_Running:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Is Moving", xOffset, &g_xParam_Lift_Running );
			pxDialog->SetHelpText( "Whether the lift should start moving immediately." );
			break;
		}
		case g_uHash_Lift_Reverse:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Reversing", xOffset, &g_xParam_Lift_Reverse );
			pxDialog->SetHelpText( "Whether the lift should move backwards along the spline initially." );
			break;
		}
		case g_uHash_Lift_AutoReverse:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Auto Reverse", xOffset, &g_xParam_Lift_AutoReverse );
			pxDialog->SetHelpText( "Whether the lift should automatically reverse it's route when it hits a dead end." );
			break;
		}
		case g_uHash_Lift_CollideEnv:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Blocked by environment", xOffset, &g_xParam_Lift_CollideEnv );
			pxDialog->SetHelpText( "Whether the lift should be blocked by the enviroment." );
			break;
		}
		case g_uHash_Lift_CollideStatic:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Blocked by statics", xOffset, &g_xParam_Lift_CollideStatic );
			pxDialog->SetHelpText( "Whether the lift should be blocked by static objects." );
			break;
		}
		case g_uHash_Lift_CollideDynamic:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Blocked by Dynamics", xOffset, &g_xParam_Lift_CollideDynamic );
			pxDialog->SetHelpText( "Whether the lift should be blocked by actors, other lifts, etc." );
			break;
		}
		case g_uHash_Lift_PauseOnCollide:
		{
			pxDialog = new Asura_SharedTickbox_Dialog( "Pause when blocked", xOffset, &g_xParam_Lift_PauseOnCollide );
			pxDialog->SetHelpText( "Whether the lift should pause when blocked, rather than treat as a dead end." );
			break;
		}
		default:
		{
			break;
		}
	}

	if( pxDialog )
	{
		pxOverlay->AddDialog( pxDialog );
		return true;
	}
	else
	{
		return false;
	}
}

void Tools_Entity_Lift::StartChoiceChanged()
{
	if ( s_pxCurrentlySelectedObject )
	{
		s_pxCurrentlySelectedObject->DeleteOverlay();
		s_pxCurrentlySelectedObject->CreateOverlay();
	}
}
