//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_LightShaft.cpp
//
// Copyright (c) 2006 Rebellion (Derby). All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"

#ifdef ASURA_TARGET_SM3


// This file's header.
#include "Entity/Tools_Entity_LightShaft.h"

// Engine independent.
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"

// Tools.
#include "Entity/Tools_Entity_Data.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/Tools_Object_System.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Texturing/Tools_PickTexture.h"
#include "Rendering/Asura_Render.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Interface/Tools_ColourPicker.h"
#include "Interface/Asura_TexturePicker_Dialog.h"

// Project independent.
#include "Entity/LightShaft/Asura_ClientEntity_LightShaft.h"
#include "Entity/LightShaft/Asura_ServerEntity_LightShaft.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"

// C L A S S   I M P L E M E N T A T I O N S /////////////////////////////////


Tools_Entity_LightShaft*     Tools_Entity_LightShaft::s_pxSelected = NULL;
bool						Tools_Entity_LightShaft::s_bModified = false;


static Asura_Dialog	*g_apxTextureNames[3] = {NULL,NULL,NULL};

Tools_Entity_LightShaft::Tools_Entity_LightShaft( Asura_ServerEntity_LightShaft* pxEntity )
    : PARENT( pxEntity )
{
	Type				= TOT_Entity_LightShaft;
	m_pxServerEntity	= pxEntity;
    m_xParams			= pxEntity->GetParams();
    m_bParamsModified	= true;	
	s_bModified			= false;

	Position			= m_xParams.m_xPosition;
	Orientation			= m_xParams.m_xOrientation;

	Asura_String mesh_name = "ToolsShape_Light";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);

	// Add the textures now. Can't rely on AddFilesToExportList to load the textures, as it only
	// gets called on quit to game. Which my SS2 build was always doing when entering tools.
	Asura_Texture_Management::AddTexture(m_xParams.m_axTextureNames[0]);
	Asura_Texture_Management::AddTexture(m_xParams.m_axTextureNames[1]);
	Asura_Texture_Management::AddTexture(m_xParams.m_xCoronaTextureName);
	Asura_Texture_Management::LoadTextures();

	pxEntity->SetParams(m_xParams);
}

Tools_Entity_LightShaft::~Tools_Entity_LightShaft( void )
{

}



void Tools_Entity_LightShaft::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls(pxOverlay, xOffset);

    Asura_Dialog* pxDialog;
	Tools_ColourPicker_Dialog* pxColourPickerDialog;

    // Sort out spacing between our gadgets and default object stuff.
    if( pxOverlay->m_pxDialogList )
    {
        xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();
    }

	// Create the tabs
	static const char* aszTabNames[] = { "Beam", "Corona" };

	Asura_Tabs_Dialog *pxTabs = NULL;
	pxDialog = pxTabs = new Asura_Tabs_Dialog(xOffset, aszTabNames, 2, 70);
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();
	float fTabTop = xOffset.y;

	// ======================================================================================================
	// 'BEAM' tab
	// ======================================================================================================
	pxTabs->SelectTab(0);

	// Near Radius    Far Radius
	// Near Plane     Far Plane
	// Near Colour    Far Colour

	// Texture 1      Texture 2
	// Repeats		  Repeats
	// U scroll       U scroll

	for(int i=0; i<2; i++)
	{
		pxDialog = new Asura_Dialog(i==0?"Near":"Far",xOffset);
		pxTabs->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Distance", xOffset, &s_pxSelected->m_xParams.m_afDistances[i] , 0.0f, 2500.0f, 0.25f, "%0.1f", &s_bModified);
		pxTabs->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Radius", xOffset, &s_pxSelected->m_xParams.m_afRadii[i] , 0.0f, 250.0f, 0.1f, "%0.2f", &s_bModified);
		pxTabs->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		
		Asura_Vector_2 xOfs2 = xOffset + Asura_Vector_2(4,0);
		pxColourPickerDialog = new Tools_ColourPicker_Dialog("Colour", xOffset,
			Asura_Vector_2(3.0f, 1.2f), &s_pxSelected->m_xParams.m_axColours[i].m_fR,
			&s_pxSelected->m_xParams.m_axColours[i].m_fG,
			&s_pxSelected->m_xParams.m_axColours[i].m_fB, 1.0f, &s_bModified);

		pxColourPickerDialog->EnableHDR(true);
		pxTabs->AddDialog( pxColourPickerDialog );

		pxDialog = new Asura_Dialog("Colour",xOfs2);
		pxTabs->AddDialog( pxDialog );

		xOffset.y += Asura_Dialog::DefaultSpacing();
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_TexturePicker_Dialog( (i==0)?"Texture A":"Texture B", xOffset, NULL, &s_pxSelected->m_xParams.m_axTextureIDs[i], &s_pxSelected->m_xParams.m_axTextureNames[i], 60, 60, &s_bModified );
		pxTabs->AddDialog( pxDialog );				
		xOffset.y += 5*Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("U Repeats", xOffset, &s_pxSelected->m_xParams.m_afTextureUScales[i] , 0.0f, 100.0f, 1.0f, "%0.4f", &s_bModified);
		pxTabs->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("U Scroll", xOffset, &s_pxSelected->m_xParams.m_afTextureScrollRates[i] , -25.0f, 25.0f, 0.01f, "%0.4f", &s_bModified);
		pxTabs->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		if (i==0)
		{
			xOffset.x+=21;
			xOffset.y -= 12*Asura_Dialog::DefaultSpacing();
		}		
	}

	xOffset.x-=21;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	float fTabBottom = xOffset.y;

	// ======================================================================================================
	// 'CORONA' tab
	// ======================================================================================================
	pxTabs->SelectTab(1);
	xOffset.y = fTabTop;

	pxDialog = new Asura_TexturePicker_Dialog( "Corona Texture", xOffset, NULL, &s_pxSelected->m_xParams.m_xCoronaTextureID, &s_pxSelected->m_xParams.m_xCoronaTextureName, 60, 60, &s_bModified );
	pxTabs->AddDialog( pxDialog );				
	

	xOffset.y += 5*Asura_Dialog::DefaultSpacing();


	Asura_Vector_2 xOfs2 = xOffset + Asura_Vector_2(4,0);
	pxDialog = new Tools_ColourPicker_Dialog("Colour",	xOffset,Asura_Vector_2(3.0f,1.2f),
		&s_pxSelected->m_xParams.m_xCoronaColour.m_fR,
		&s_pxSelected->m_xParams.m_xCoronaColour.m_fG,
		&s_pxSelected->m_xParams.m_xCoronaColour.m_fB,1.0f,&s_bModified);
	pxTabs->AddDialog( pxDialog );		

	pxDialog = new Asura_Dialog("Colour",xOfs2);
	pxTabs->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();
	

	pxDialog = new Asura_Numerical_Dialog("Size", xOffset, &s_pxSelected->m_xParams.m_fCoronaSize , 0.0f, 100.0f, 0.1f, "%0.2f", &s_bModified);
	pxTabs->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Depth", xOffset, &s_pxSelected->m_xParams.m_fCoronaDepth , 0.0f, 100.0f, 0.1f, "%0.2f", &s_bModified);
	pxTabs->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Angle Range", xOffset, &s_pxSelected->m_xParams.m_fCoronaAngleRange , 0.0f, 1.0f, 0.1f, "%0.4f", &s_bModified);
	pxTabs->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();



	pxTabs->SelectTab(0);
	xOffset.y = fTabBottom;

	pxDialog = new Asura_Numerical_Dialog("Close Fade: Start Dist", xOffset,&s_pxSelected->m_xParams.m_fFadeCloseStart, 0.0f, 2500.0f, 1.0f, "%0.1f", &s_bModified);
	pxDialog->SetHelpText("For fading out (and disabling) the light shaft based on distance from the camera");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Close Fade: Range", xOffset, &s_pxSelected->m_xParams.m_fFadeCloseDist, 0.0f, 2500.0f, 1.0f, "%0.1f", &s_bModified);
	pxDialog->SetHelpText("For fading out (and disabling) the light shaft light/projector based on distance from the camera");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();	


	// Add the distance fade here, too
	pxDialog = new Asura_Numerical_Dialog("Distance Fade: Start Dist", xOffset,&s_pxSelected->m_xParams.m_fFadeoutStart , 0.0f, 2500.0f, 1.0f, "%0.1f", &s_bModified);
	pxDialog->SetHelpText("For fading out (and disabling) the light shaft based on distance from the camera");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Numerical_Dialog("Distance Fade: Range", xOffset, &s_pxSelected->m_xParams.m_fFadeoutDist , 0.0f, 2500.0f, 1.0f, "%0.1f", &s_bModified);
	pxDialog->SetHelpText("For fading out (and disabling) the light shaft light/projector based on distance from the camera");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();


}

bool Tools_Entity_LightShaft::CanBeCopied( void ) const
{
    return (Type == TOT_Entity_LightShaft);
}

void Tools_Entity_LightShaft::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Object::Copy(pxObject);

	const Tools_Entity_LightShaft* pxFromObject = (const Tools_Entity_LightShaft*) pxObject;

	//copy between the tools objects
	m_xParams = pxFromObject->m_xParams;

	// and update the server entity
	Asura_ServerEntity_LightShaft* pxEntity = static_cast<Asura_ServerEntity_LightShaft*>( Asura_Entity_Server::FindEntity(GetGuid()) );
	if( !pxEntity ) { return; }
	pxEntity->SetParams( m_xParams );

	Orientation = m_xParams.m_xOrientation;	

}

void Tools_Entity_LightShaft::Create( void )
{
    Tools_Entity_LightShaft* pxToolsObj = CreateMain();
	if(pxToolsObj)
	{    
    	Tools_Object_System::SelectObject( pxToolsObj );
	}
}

Tools_Entity_LightShaft* Tools_Entity_LightShaft::CreateMain()
{
    const Asura_Guid uGUID = Asura_Entity_System::GetCreatedByToolsGuid();

    Tools_Entity_Data*                 pxToolsData = NULL;
    Asura_ServerEntity_LightShaft*      pxEntity    = NULL;
    Tools_Entity_LightShaft*			   pxToolsObj  = NULL;

    pxToolsData = CreateToolsDataWithDefaultName( "TextureLightShaft", "LightShaft", uGUID, GetNamePrefix() );
    if( !pxToolsData ) { goto CreateFailed; }

	pxEntity = Asura_ServerEntity_LightShaft::CreateFromTools( uGUID,pxToolsData->Position);
    pxEntity->ToolsData = pxToolsData;
    if( !pxEntity ) { goto CreateFailed; }

    pxToolsObj = new Tools_Entity_LightShaft( pxEntity );
    if( !pxToolsObj ) { goto CreateFailed; }

    Tools_SaveAsura::SetEntityModified();

    return pxToolsObj;

CreateFailed:

    if( !pxEntity ) { delete pxToolsObj; }
	if ( pxEntity )
	{
		pxEntity->PreDestructionShutdown();
	}
    delete pxEntity;
    delete pxToolsData;
	return 0;
}

void Tools_Entity_LightShaft::Create( Asura_Entity* pxEntity )
{
    Asura_Assert( AsuraEntityClass_LightShaft == pxEntity->GetClassification(), "" );
    new Tools_Entity_LightShaft( static_cast<Asura_ServerEntity_LightShaft*>(pxEntity) );
}

void Tools_Entity_LightShaft::OnDeselection( void )
{	
	Verify(true);
	PARENT::OnDeselection();
}

void Tools_Entity_LightShaft::OnSelection( void )
{
	s_pxSelected = this;

	PARENT::OnSelection();

	if(m_pxServerEntity)
	{				
		// Fill in the params
	}	
}

void Tools_Entity_LightShaft::Render( void )
{
	// Has the tools object moved?
	// Had to add this extra case to make it work with templates, as the updatewhileselected isn't called.
	Asura_ServerEntity_LightShaft* pxEntity = static_cast<Asura_ServerEntity_LightShaft*>( Asura_Entity_Server::FindEntity(GetGuid()) );

	// Has the tools object moved?
	if (pxEntity)
	{	
		if( m_xParams.m_xPosition    != Position  ||
			m_xParams.m_xOrientation != Orientation)
		{
			m_xParams.m_xPosition    = Position;
			m_xParams.m_xOrientation = Orientation;        

			// Update entity with new parameters
			pxEntity->SetParams( m_xParams );  		  
		}
	}

    PARENT::Render();

	if (m_pxServerEntity)
	{
		Asura_ClientEntity_LightShaft *pxClient = (Asura_ClientEntity_LightShaft *)Asura_ClientEntity_LightShaft::FindEntity(m_pxServerEntity->GetGuid());

		if (!pxClient)
		{
			// There's no client entity yet - create one	
			pxClient = Asura_ClientEntity_LightShaft::CreateFromTools(m_pxServerEntity->GetGuid(), m_pxServerEntity->GetParams());
		}
	}

}

const char* Tools_Entity_LightShaft::GetTypeDescription( void )
{
	return "Volumetric(ish) Light Shaft";
}



void Tools_Entity_LightShaft::UpdateWhileSelected( void )
{
    PARENT::UpdateWhileSelected();

    // Find corresponding entity.
	Asura_ServerEntity_LightShaft* pxEntity = static_cast<Asura_ServerEntity_LightShaft*>( Asura_Entity_Server::FindEntity(GetGuid()) );
	
    if( !pxEntity ) { return; }

    // Has the tools object moved?
    if( m_xParams.m_xPosition    != Tools_Object_System::SelectedObject->Position  ||
		m_xParams.m_xOrientation != Tools_Object_System::SelectedObject->Orientation)
    {
        m_xParams.m_xPosition    = Tools_Object_System::SelectedObject->Position;
		m_xParams.m_xOrientation = Tools_Object_System::SelectedObject->Orientation;        
    }
    
    // Update entity with new parameters
    pxEntity->SetParams( m_xParams );  	

	if (s_bModified)
	{
		SetEntityModified();
		s_bModified = false;
	}
}

bool Tools_Entity_LightShaft::Verify( const bool bAutomatic )
{
	if ( m_xParams.m_axTextureIDs[0] == 0 && m_xParams.m_axTextureIDs[1] == 0  )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Lightshafts", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
			"Lightshaft %d has no textures assigned to the beam, this will render incorrectly on consoles", GetGuid() );
		return false;
	} 
	else if (m_xParams.m_axTextureIDs[0] == 0)
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Lightshafts", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
			"Lightshaft %d has no textures assigned to the beam in slot A, this will render incorrectly on consoles", GetGuid() );
		return false;
	}
	else if (m_xParams.m_axTextureIDs[1] == 0)
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::VERIFICATION, "Lightshafts", new Tools_MessageLogAction_GotoEntity( GetGuid() ),
			"Lightshaft %d has no textures assigned to the beam in slot B, this will render incorrectly on consoles", GetGuid() );
		return false;
	}
	return true;
}


void Tools_Entity_LightShaft::SetEntityModified()
{
	PARENT::SetEntityModified();

	Asura_ServerEntity_LightShaft* pxEntity = static_cast<Asura_ServerEntity_LightShaft*>( Asura_Entity_Server::FindEntity(GetGuid()) );
	
	if (pxEntity)
	{
		pxEntity->SetParams( m_xParams );
	}
}



Tools_Object* Tools_Entity_LightShaft::CreateObjectOfThisType() const
{
	return CreateMain();	
}



bool Tools_Entity_LightShaft::HandlesThisMessage(int iMessageID)
{
/*
	switch (iMessageID)
	{
	case SS2_MSG_CHANGE_WEATHER:
	case SS2_MSG_TRIGGER_LIGHTNING:
		return true;
	}
*/
	if (PARENT::HandlesThisMessage(iMessageID))
	{
		return true;
	}

	return false;
}






#endif //#ifdef ASURA_TARGET_SM3
