//
// Tools_Entity_MusicTrigger_Object.cpp
//
// Created by Richard Rice - 18/03/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

#include "Entity/Tools_Entity_MusicTrigger_Object.h"

#include "Entity/Triggers/Asura_ServerEntity_MusicTrigger.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Sound/Asura_Project_Music.h"
#include "Sound/Asura_Music.h"

#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"




Tools_Entity_MusicTrigger_Object::Tools_Entity_MusicTrigger_Object(Asura_Entity* pxEntity)
:Tools_Entity_Activatable_Object(pxEntity)
{
	Type = TOT_Entity_MusicTrigger;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_Note");
}


void Tools_Entity_MusicTrigger_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Activatable_Object::AddOverlayControls(pxOverlay,xOffset);
	Asura_Dialog* pxDialog;

	Asura_ServerEntity_MusicTrigger* pxAsuraEntity = (Asura_ServerEntity_MusicTrigger*) Entity;

	//get text for music selection control
	char szText[100];
	if(pxAsuraEntity->m_iMusicTrack == ASURA_MUSIC_TRACK_NONE)
	{
		strcpy(szText,"Track : <Stop Music>");
	}
	else
	{
		const char* pszTrackName = Asura_Music_System::GetTrackName(pxAsuraEntity->m_iMusicTrack);
		Asura_PrintfToBfr(szText,"Track : %s",100,true,pszTrackName ? pszTrackName : "<Unknown>");
	}
	

	pxDialog = new Asura_Function_Dialog(szText,xOffset,SelectMusicTrack,(int)this);
	pxDialog->SetHelpText("Choose the track to play.\nAlso used for stopping music.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += 1.5f;

	pxDialog = new Asura_Numerical_Dialog("Volume",xOffset,&pxAsuraEntity->m_fVolume,0,1.5f,0.01f,"%.2f",&Changed);
	pxDialog->SetHelpText("Volume at which music should be played.\nNote that 1.33 is effectively maximum , since all volumes are scaled by 0.75 and capped at 1.0 at the end.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += 1.5f;


	pxDialog = new Asura_Numerical_Dialog("Fade Time",xOffset,&pxAsuraEntity->m_fFadeTime,0,99.0f,0.1f,"%.2f",&Changed);
	pxDialog->SetHelpText("Time over which this track should be faded in (or the previous track faded out , if this is a Stop Music trigger).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += 1.5f;

	pxDialog = new Asura_FlagTickbox_Dialog("Looping", xOffset, &pxAsuraEntity->m_uFlags, ASURA_MUSICTRIGGER_FLAG_LOOPING, &Changed);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += 1.5f;
}

void Tools_Entity_MusicTrigger_Object::Create()
{
	Tools_Object* pxObject = Create(false);
	Tools_Object_System::SelectObject(pxObject);
}

Tools_Entity_MusicTrigger_Object* Tools_Entity_MusicTrigger_Object::Create(bool bGenerateName)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Music Trigger","Music Trigger",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("Music Trigger",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0 ;

	//create the asura data
	Asura_ServerEntity_MusicTrigger* pxAsuraEntity = new Asura_ServerEntity_MusicTrigger();
	pxAsuraEntity->ToolsData = pxToolsData;

	//create the tools pxObject
	Tools_Entity_MusicTrigger_Object* pxObject = new Tools_Entity_MusicTrigger_Object(pxAsuraEntity);

	Tools_SaveAsura::SetEntityModified();

	return pxObject;
}

void Tools_Entity_MusicTrigger_Object::SelectMusicTrack()
{
	Asura_ServerEntity_MusicTrigger* pxAsuraEntity = (Asura_ServerEntity_MusicTrigger*) Entity;

									
	ListSelection_Overlay xOverlay("Select Music Track");
	//add all the track names to the overlay
	for(int i=ASURA_MUSIC_TRACK_NONE+1;i<ASURA_NUM_MUSIC_TRACKS;i++)
	{
		const char* pszName = Asura_Music_System::GetTrackName(i);
		if(pszName && strlen(pszName))
		{
			xOverlay.AddItem(pszName,i);
		}

	}

	//add option for stopping music
	xOverlay.AddItem("<Stop Music>",ASURA_MUSIC_TRACK_NONE);


	//show selection overlay
	if(!xOverlay.DoModal()) return;

	pxAsuraEntity->m_iMusicTrack = xOverlay.GetSelectionData();
	SetEntityModified();

	//update the music trigger's overlay
	DeleteOverlay();
	CreateOverlay();
	
}

Tools_Object* Tools_Entity_MusicTrigger_Object::CreateObjectOfThisType() const
{
	return Create(true);	
}

void Tools_Entity_MusicTrigger_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Activatable_Object::Copy(pxObject);

	const Tools_Entity_MusicTrigger_Object* pxFromObject = (const Tools_Entity_MusicTrigger_Object*) pxObject;

	Asura_ServerEntity_MusicTrigger* pxFromEntity = (Asura_ServerEntity_MusicTrigger*) pxFromObject->Entity;
	Asura_ServerEntity_MusicTrigger* pxToEntity = (Asura_ServerEntity_MusicTrigger*) Entity;

	pxToEntity->m_iMusicTrack = pxFromEntity->m_iMusicTrack;
	pxToEntity->m_fFadeTime = pxFromEntity->m_fFadeTime;
	pxToEntity->m_uFlags = pxFromEntity->m_uFlags;
}

#endif // #ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

//eof
