//
// Tools_Entity_Object.cpp
//
// Created by Richard Rice - 01/05/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

// core
#include "Core/Asura.h"

// header
#include "Entity/Tools_Entity_Object.h"

// asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Camera/Asura_Camera.h"
#include "Core/Asura_Time_System.h"
#include "Core/Asura_Windows.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Entity/Asura_Entity.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Asura_Entity_Event.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Font_System.h"
#include "Rendering/Asura_Viewport.h"
#include "SpecialFX/Asura_Skybox.h"

// tools
#include "Tools_Overlays.h"
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Entity/Intangible/Tools_Entity_ConsoleVar.h"
#endif //ASURA_SUPPORT_COMMAND_CONSOLE
#include "Entity/EffectVolume/Tools_Entity_EffectVolume.h"
#include "Entity/Tools_Entity_AdvancedVolumeTrigger.h"
#if defined(ASURA_USE_AFFECTORS)
#include "Entity/Tools_Entity_Affector.h"
#include "Entity/Tools_Entity_Affector_Wind.h"
#endif	// defined(ASURA_USE_AFFECTORS)
#include "Entity/Tools_Entity_ClockTrigger_Object.h"
#include "Entity/Tools_Entity_CountedTrigger_Object.h"
#include "Entity/Tools_Entity_CutsceneController_Object.h"
#include "Entity/Tools_Entity_Data.h"
#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
#include "Entity/Tools_Entity_DynamicMusicTrigger_Object.h"
#endif
#include "Entity/Tools_Entity_ForceField_Conveyor.h"

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
#include "Entity/Tools_Entity_FMVController.h"
#include "Entity/Tools_Entity_FMVSpeaker.h"
#endif

#include "Entity/Tools_Entity_FMVTrigger_Object.h"
#include "Entity/Tools_Entity_LogicTrigger_Object.h"
#include "Entity/Tools_Entity_MetaMusicTrigger_Object.h"
#include "Entity/Tools_Entity_MusicTrigger_Object.h"
#include "Entity/Tools_Entity_Objective.h"
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "Entity/Tools_Entity_ProxyTrigger_Object.h"
#include "Entity/Tools_Entity_Simulation.h"
#include "Entity/Tools_Entity_SplitterBlock_Object.h"
#include "Entity/Tools_Entity_StartPoint.h"
#include "Entity/Tools_Entity_TimelineTrigger_Object.h"
#include "Entity/Tools_Entity_Trigger_Object.h"
#include "Entity/Tools_Message_Editing.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
#include "Entity/Tools_MessageEditor.h"
#endif
#if !defined(ASURA_DONT_USE_TELEPORTERS)
#include "Entity/Tools_Entity_Teleporter.h"
#include "Entity/Tools_Entity_TeleportDestination.h"
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

#include "Entity/Tools_Entity_PFX_Effect.h"
#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Entity/Sound/Tools_Entity_DialogueTrigger.h"
#include "Entity/Sound/Tools_Entity_DialogueController.h"
#endif
#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "Entity/Container/Tools_Entity_Container.h"
#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#include "Entity/Locations/Tools_Entity_Node.h"
#include "Entity/Locations/OrientedNode/Tools_EntityNode_OrientedNode.h"
#include "Entity/Locations/Spline/Tools_EntityNode_Spline.h"
#include "Entity/Locations/OrientedNode/LiftNode/Tools_EntityNode_LiftNode.h"
#include "Entity/Locations/Spline/LiftSpline/Tools_EntityNode_LiftSpline.h"
#ifdef ASURA_USE_AXON
#include "Entity/Locations/GuardZone/Tools_EntityNode_GuardZone.h"
#include "Entity/Locations/OrientedNode/CoverpointNode/Tools_EntityNode_CoverpointNode.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Tools_EntityNode_GS2Node.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/AtttactorNode/Tools_EntityNode_GS2Attractor.h"
#include "Entity/Locations/Spline/GamesceneSpline/Tools_EntityNode_GS2Spline.h"
#endif // ASURA_USE_AXON

#include "Entity/Tools_Entity_ClientVolumeTrigger_Object.h"
#include "Entity/Tools_Entity_EnvTextureAnimControl.h"

#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER
#include "Entity/Tools_Entity_DebugWaypointTrigger_Object.h"
#endif // #ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER

#include "EntityTemplate/Tools_Entity_Template_Object.h"
#include "Lighting/Tools_LightSets.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_SpecialGuid_Object.h"
#include "Objects/Tools_StartPosition_Object.h"
#include "Sound/Tools_Mode_Sound.h"
#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Sound/Tools_DialogueSpeaker_Object.h"
#endif
#include "Sound/Tools_StreamingBackgroundSound_Object.h"
#ifdef ASURA_TARGET_SM3
#include "Entity/Tools_Entity_Projector.h"
#include "Entity/Tools_Entity_DynamicLight.h"
#include "Entity/Tools_Entity_LightShaft.h"
#endif
#include "Entity/Tools_Entity_WaypointBlockerVolume.h"
#include "Mesh/Tools_PlatformMesh_System.h"
#include "Misc/Tools_Render_Options.h"

//////////////////////////////////////////////////////////////////////////////
//
// Tools_BaseEntity_Object
//
//////////////////////////////////////////////////////////////////////////////

/// static data
Tools_BaseEntity_Object* Tools_BaseEntity_Object::ThisEntity = 0;
Asura_Vector_2 Tools_BaseEntity_Object::OverlayPosition(10,10);
Asura_Vector_2 Tools_BaseEntity_Object::s_xPlatformOverlayPosition(35,10);
bool Tools_BaseEntity_Object::s_bShowPlatformOverlay = false;
Asura_Hashed_List<Tools_BaseEntity_Object> Tools_BaseEntity_Object::s_xList;

void (*Tools_Entity_Object::s_pfnProjectCreateAllObjects)() = NULL;
Asura_Hashed_List<Tools_Entity_Object> Tools_Entity_Object::s_xList;

//hash ID of first platform tickbox
static const Asura_Hash_ID g_uHash_EntityObject_Platform0 = 0x73613F00;
static const Asura_Hash_ID g_uHash_EntityObject_LightSet0 = 0xBD2D1016;//uses same value as Asura_Hash_ID g_uHash_LightSource_LightSet0 for the lights

// We need the 'starts active' shared parameter to be accessible from other classes.
// The hash value is declared in the header.
//TOOLS_DECLARE_SHARED_PARAM(bool, 0xabf43573, Entity_StartsActive)
Tools_ObjectSystem_SharedValue< bool > g_xParam_Entity_StartsActive( g_uHash_Entity_StartsActive, "Active" );

Tools_BaseEntity_Object::Tools_BaseEntity_Object(Asura_Entity* entity) 
: Entity( entity )
{
	m_uGuid = Entity->GetGuid();

	if(!Entity->ToolsData)
	{
		//generate default tools data
		Entity->ToolsData = new Tools_Entity_Data;
		Entity->ToolsData->Position.Zero();
		char name[50];
		sprintf(name,"Entity %d",Entity->GetGuid());
		Entity->ToolsData->Name = name;
		Entity->ToolsData->m_bEntityModified = true;
	}

	m_pxServerEntity = Asura_Entity_Server::FindEntity(Entity->GetGuid());
	Asura_Assert(m_pxServerEntity,"");

	m_pxClientEntity = Asura_Entity_Client::FindEntity(Entity->GetGuid());

	Position = Entity->ToolsData->Position;
	Orientation = Entity->ToolsData->m_xOrientation;
	
	Asura_String mesh_name = "ToolsShape_CutsceneEvent";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);

	Changed = false;

	RemoveEmptyMessageBlocks();

    s_xList.AddToList(this);
}

Tools_BaseEntity_Object::~Tools_BaseEntity_Object()
{
	if(Entity && Entity->ToolsData)
	{
		Entity->ToolsData->ToolsObject = 0;
	}	
    s_xList.RemoveFromList(this);

	RemoveEmptyMessageBlocks();

	Tools_SecondaryEntityFile_System::RemoveEntityFromTree(m_uGuid);
}



/////////////////////////////////////////////////////
// CreateToolsData                                 //
// Prompt for name and then initialise tools data. //
/////////////////////////////////////////////////////
Tools_Entity_Data* Tools_BaseEntity_Object::CreateToolsData(const char* szTypeName, const char* szInitialName /*= 0*/, const char* szNamePrefix /*= 0*/)
{
	Asura_String xName(szInitialName);
	if (!Tools_Object_System::AskForName(xName,szTypeName,0,false,szNamePrefix))
	{
		return 0;
	}

	//allocate , and fill in the tools data
	Tools_Entity_Data* pxToolsData = new Tools_Entity_Data;
	pxToolsData->Name = xName;
	pxToolsData->Position = *Asura_Camera::GetPosition();
	pxToolsData->m_bEntityModified = true;

	UpdateToolsEntityTimeStamp( pxToolsData, true );

	return pxToolsData;
}

///////////////////////////////////////////////////////////////////////////////
// CreateToolsDataWithDefaultName                                            //
// Create default name from szShortName and uGuid, if it isn't unique prompt //
// for a new one, then initialise tools data.                                //
///////////////////////////////////////////////////////////////////////////////
Tools_Entity_Data* Tools_BaseEntity_Object::CreateToolsDataWithDefaultName(const char* szTypeName, const char* szShortName, Asura_Guid uGuid, const char* szNamePrefix /*= 0*/)
{	
	// Create a default name for the tools object.
	if (uGuid >= ASURA_GUID_TOOLSCREATEDENTITY_START)
	{
		uGuid -= ASURA_GUID_TOOLSCREATEDENTITY_START;
	}

	char szNameBuffer[128];
	Asura_PrintfToBfr( szNameBuffer, "%s%s", 128, true, szNamePrefix ? szNamePrefix : "", szShortName );
	char* pcPostFixPos = &szNameBuffer[strlen(szNameBuffer)];

	for(u_int u= uGuid; ; u++)
	{
		sprintf(pcPostFixPos, "%u", u);

		// Check that it isn't already being used...
		bool bOkay = true;
		for(LIF<Tools_Object*> olif(&Tools_Object_System::ObjectList); !olif.done(); olif.next())
		{
			if ( !stricmp(szNameBuffer, olif()->GetName()) )
			{
				bOkay = false; // Name already in use...
				break;
			}
		}
		if(bOkay) break;
	}

	// Allocate, and fill in the tools data.
	Tools_Entity_Data* pxToolsData = new Tools_Entity_Data;
	pxToolsData->Name = szNameBuffer;
	pxToolsData->Position = *Asura_Camera::GetPosition();
	pxToolsData->m_bEntityModified = true;

	UpdateToolsEntityTimeStamp( pxToolsData, true );

	return pxToolsData;
}
///////////////////////////////////////////////////////////////////////////////
// CreateToolsDataWithDuplicateName                                            //
// Create a default name from szShortName and uGuid, removing the guid in    //
// szShortName and replacing it with the new one. If the resulting name is   //
// not unique prompt for a new one. The tools data is also initialised.      //
///////////////////////////////////////////////////////////////////////////////
Tools_Entity_Data* Tools_BaseEntity_Object::CreateToolsDataWithDuplicateName(const char* szTypeName, const char* szShortName, Asura_Guid uGuid)
{
	// Create a default name for the tools object.
	if (uGuid >= ASURA_GUID_TOOLSCREATEDENTITY_START)
	{
		uGuid -= ASURA_GUID_TOOLSCREATEDENTITY_START;
	}

	char szNameBuffer[128];
	//sprintf(szNameBuffer,"%s %u", szShortName, uGuid);

	sprintf(szNameBuffer,"%s", szShortName);
	char* pcPostFixPos;
	//determine the point at which the numerical postfix should be added
	if (strlen(szNameBuffer) > 0)
	{
		char *szPos = szNameBuffer + strlen(szNameBuffer) - 1;
		while (szPos > szNameBuffer && (*szPos == ' ' || (*szPos >= '0' && *szPos <= '9'))) szPos--;

		pcPostFixPos = szPos+1;
		*pcPostFixPos++ = ' ';
		*pcPostFixPos = 0;
	}
	else
	{
		pcPostFixPos = szNameBuffer;
	}

	//search for a useable name , starting with one based on the guid
	for(u_int u = uGuid; ; u++)
	{
		sprintf(pcPostFixPos, "%u", u);
				
		// Check that it isn't already being used...
		bool bOkay = true;
		for(LIF<Tools_Object*> olif(&Tools_Object_System::ObjectList); !olif.done(); olif.next())
		{
			if ( !stricmp(szNameBuffer, olif()->GetName()) )
			{
				bOkay = false; // Name already in use...
				break;
			}
		}
		if(bOkay) break;
	}

	// Allocate, and fill in the tools data.
	Tools_Entity_Data* pxToolsData = new Tools_Entity_Data;
	pxToolsData->Name = szNameBuffer;
	pxToolsData->Position = *Asura_Camera::GetPosition();
	pxToolsData->m_bEntityModified = true;

	UpdateToolsEntityTimeStamp( pxToolsData, true );

	return pxToolsData;
}

Tools_Entity_Data* Tools_BaseEntity_Object::CreateToolsDataWithThisName(const char* szName)
{
    Asura_String xStringName( szName );

	// Check that this name isn't already being used...
	bool bOkay = true;
	for(LIF<Tools_Object*> olif(&Tools_Object_System::ObjectList); !olif.done(); olif.next())
	{
		if ( !stricmp(xStringName.GetString(), olif()->GetName()) )
		{
			bOkay = false; // Name already in use...
			Tools_Timed_Message_System::AddMessage(0xff0000,"That name is already in use");
			break;
		}
	}

    if ( !bOkay )
    {
        return NULL;
    }

	//allocate , and fill in the tools data
	Tools_Entity_Data* pxToolsData = new Tools_Entity_Data;
	pxToolsData->Name = xStringName;
	pxToolsData->Position = *Asura_Camera::GetPosition();
	pxToolsData->m_bEntityModified = true;

	UpdateToolsEntityTimeStamp( pxToolsData, true );

    return pxToolsData;
}

void Tools_BaseEntity_Object::UpdateToolsEntityTimeStamp( Tools_Entity_Data* const pxToolsData, bool bCreated /*= false*/ )
{
	if( !pxToolsData )
	{
		Asura_Assert( 0, "Can't update Tools Entity data timestamp, supplied pointer is NULL" );
		return;
	}

	Asura_Assertf( ( !bCreated ) || pxToolsData->m_xCreationData.IsBlank(), ( "Overwriting previous Tools Entity creation data: %s", pxToolsData->m_xCreationData.GetString() ) );

	Asura_Time xTime = Asura_Time_System::GetCurrentDateAndTime();
	Asura_Char szDateTime[128];
	Asura_PrintfToBfr( szDateTime, "%02d/%02d/%04d %02d:%02d", 128, true, xTime.m_ucDayOfMonth+1, xTime.m_ucMonth+1, xTime.m_usYear, xTime.m_ucHour, xTime.m_ucMinute );

	Asura_Char szUserName[128];
	if( !Asura_Windows::GetUserNameFromPC( szUserName, 128 ) )
	{
		szUserName[0] = '\0';
	}

	pxToolsData->m_xLastModified = szDateTime;
	pxToolsData->m_xLastModified += " by ";
	pxToolsData->m_xLastModified += szUserName;

	if( bCreated )
	{
		pxToolsData->m_xCreationData = pxToolsData->m_xLastModified;
	}
}

void Tools_BaseEntity_Object::Destroy()
{
	Asura_Assert(!Tools_Entity_Simulation::IsActive(),"");

	SetEntityFileModified();

	Tools_Entity_Simulation::Disable();//to supress its fiddling with SetAsDestroyed()

	//Clear out any entries in the message block system
	for(int i=0;i<m_pxServerEntity->m_iNumberOfEvents;i++)
	{
		Asura_EventAndMessage* pxEvent = &m_pxServerEntity->m_pxEvents[i];

		Asura_StaticMessage_System::DeleteMessageBlock(pxEvent->m_usMessageBlockID);
		pxEvent->m_usEventID = ASURA_STATICMESSAGE_EMPTYBLOCK;
	}

	//kill the client entity if it exists
	Asura_Entity_Client* pxClientEntity = Asura_Entity_Client::FindEntity(Entity->GetGuid());
	if(pxClientEntity)
	{
		pxClientEntity->SetAsDestroyed();
	}
	//and the server entity
	Entity->SetAsDestroyed();
	
	Tools_Entity_Simulation::Enable();

	Tools_Object::Destroy();
}

void Tools_BaseEntity_Object::UpdatePositionInAsuraData(bool /*updateLinkedObjects*/)
{
	Asura_Assert(Entity && Entity->ToolsData,"");
	if(Position != Entity->ToolsData->Position || Orientation != Entity->ToolsData->m_xOrientation)
	{
		SetEntityModified();
	}
	Entity->ToolsData->Position = Position;
	Entity->ToolsData->m_xOrientation = Orientation;
	Entity->ToolsData->m_bEntityModified = true;
}


const char* Tools_BaseEntity_Object::GetName()
{
	Asura_Assert(Entity && Entity->ToolsData,"");
	return Entity->ToolsData->Name.GetString();
}


void Tools_BaseEntity_Object::OnSelection()
{
	ThisEntity = this;
	if(!IsMemberOfTemplateInstance())
	{
		CreateOverlay();
		if(s_bShowPlatformOverlay)
		{
			CreatePlatformOverlay();
		}
	}

	Tools_Object_System::SetCommentText(Entity->ToolsData->m_xComment.GetString(),this);
}

void Tools_BaseEntity_Object::OnDeselection()
{
	Tools_Object_System::ClearCommentText(this);

	DeleteOverlay();
	DeletePlatformOverlay();
	ThisEntity = 0;
}

void Tools_BaseEntity_Object::UpdateWhileSelected()
{
	if(Changed)
	{
		if(m_pxClientEntity) UpdateClient();
		
		SetEntityModified();

		Changed = false;	
	}
	Tools_Object::UpdateWhileSelected();

	//detect if platform overlay was closed by hand
	s_bShowPlatformOverlay = Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_ENTITYPLATFORMS);
}


void Tools_BaseEntity_Object::CreateOverlay()
{
	PARENT::CreateObjectOverlay();

	if(Tools_Entity_Simulation::IsActive())
	{
		CreateSimulationOverlay();
	}

	UpdateOverlay();
}

void Tools_BaseEntity_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;
	if(CanBeRenamed())
	{
		pxDialog = new Asura_Function_Dialog("Rename",xOffset,Rename);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		// Rename all selected entities
		if( Tools_Object_System::SelectedObjectList.size() > 1 )
		{
			pxDialog = new Asura_Function_Dialog("Rename All",xOffset,RenameAll);
			pxDialog->SetHelpText("Rename all selected entities");
			static_cast<Asura_Function_Dialog*>(pxDialog)->SetUseSharedDialogColour(true);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y+=Asura_Dialog::DefaultSpacing();
		}
	}

	pxDialog = new Asura_Function_Dialog("Edit Comment",xOffset,EditComment);
	pxDialog->SetHelpText("Edit the tools comment text for this entity.");
  	pxOverlay->AddDialog(pxDialog);


	xOffset.x += 12.0f;
	pxDialog = new Asura_Tickbox_Dialog("Show",xOffset,&Tools_Object_System::s_bShowComment);
	pxDialog->SetHelpText("Displays the comment (if any) in the corner of the screen");
  	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;

  	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Export Settings",xOffset,EditPlatformsClicked);
	pxDialog->SetHelpText("Creates an overlay for editing the platforms and light sets that the entities should be exported to.");
  	pxOverlay->AddDialog(pxDialog);
  	xOffset.y+=Asura_Dialog::DefaultSpacing();

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	u_int uNumMessages = Tools_MessageEditor::GetNumberOfMessagesInEntity(this);

	Asura_Char szBuffer[64];
	Asura_PrintfToBfr(szBuffer, "Edit Messages (%u) [m]", 64, true, uNumMessages);

	pxDialog = new Asura_Function_Dialog(szBuffer, xOffset, EditMessages, NULL);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	#else

	//add event message block controls
	char szText[100];

	for(int i=0;i<m_pxServerEntity->m_iNumberOfEvents;i++)
	{
		const Asura_EventAndMessage* pxEvent = &m_pxServerEntity->m_pxEvents[i];

		const char* szHelpText = Tools_Message_Editing::GetEventHelpText(pxEvent->m_usEventID);
		if(!szHelpText || !*szHelpText) szHelpText = "Edit messages that should be sent when this event happens.";

		Asura_PrintfToBfr(szText,"%s (%d)",100,true,Tools_Message_Editing::GetEventDescription(pxEvent->m_usEventID),Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxEvent->m_usMessageBlockID));
		pxDialog = new Asura_Function_Dialog(szText,xOffset,EditMessages,i);
		pxDialog->SetHelpText(szHelpText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	if(CanAddAnyMoreMessageBlocks())
	{
		//add option for adding more message blocks
		pxDialog = new Asura_Function_Dialog("Add Message Block",xOffset,AddMessageBlock);
		pxDialog->SetHelpText("Add a new event (such as destruction) on which messages should be sent.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
	#endif
	
	//add "starts active" tickbox
	if (!DealsInternallyWithActivate())
	{
		pxDialog = new Asura_SharedTickbox_Dialog("Starts Active", xOffset, &g_xParam_Entity_StartsActive);
		pxDialog->SetHelpText("Entities which don't start active must be sent an activate message before they will do anything or appear in-game. (If you have 2 tickboxes like this one, use the other one and ask the programmers to remove this one).");
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	//set things up so that right clicking on the overlay will be equivalent to right clicking on the object
	//RAR 06/11/2007
	pxOverlay->SetContextMenuCallback(Tools_Object_System::AddContextMenuItemsFromOverlay, this);
}

void Tools_BaseEntity_Object::CreateSimulationOverlay()
{
#if 0
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Entity Test",false,&OverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	
	//add controls according to the type of entity
	AddSimulationOverlayControls(pxOverlay,xOffset);

	pxOverlay->ID = TOOLS_OVERLAY_SELECTEDOBJECT;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
#endif
}

void Tools_BaseEntity_Object::DeleteOverlay()
{
	PARENT::DeleteObjectOverlay();
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT_SIMUL);
}

void Tools_BaseEntity_Object::EditPlatformsClicked()
{
	s_bShowPlatformOverlay = true;
	CreatePlatformOverlay();
}

void Tools_BaseEntity_Object::CreatePlatformOverlay()
{
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_ENTITYPLATFORMS)) return;

	Asura_Overlay* pxOverlay = new Asura_Overlay("Export Settings",false,&s_xPlatformOverlayPosition);
	pxOverlay->m_uID = TOOLS_OVERLAY_ENTITYPLATFORMS;	
	
	Asura_Vector_2 xOffset(1.0f,3.0f);
	if( ThisEntity )
	{
		ThisEntity->AddPlatformOverlayControls( pxOverlay, xOffset );

		Asura_Tabs_Dialog* pxTabsDialog = static_cast<Asura_Tabs_Dialog*> (pxOverlay->GetDialog( (int)uTOOLS_DIALOGID_ENTITY_PLATFORM_TAB ));
		if( pxTabsDialog )
		{
			static u_int s_uSelectedTab = 0;
			pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);
		}
	}	
	
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
}

void Tools_BaseEntity_Object::DeletePlatformOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ENTITYPLATFORMS);
}


void Tools_BaseEntity_Object::AddPlatformOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxDialog;

	static Tools_ObjectSystem_SharedValue<bool> s_axParam_EntityObject_Platform[Tools_Level_Export::ExportType_NumPlatforms];
	for(int i=0;i<Tools_Level_Export::ExportType_NumPlatforms;i++)
	{
		//initialise the hash IDs
		s_axParam_EntityObject_Platform[i].SetParamID(g_uHash_EntityObject_Platform0+i);
	}

	static Tools_ObjectSystem_SharedValue<bool> s_axParam_EntityObject_LightSet[TOOLS_MAX_LIGHT_SETS];
	for(int i=0;i<TOOLS_MAX_LIGHT_SETS;i++)
	{
		//initialise the hash IDs
		s_axParam_EntityObject_LightSet[i].SetParamID(g_uHash_EntityObject_LightSet0+i);
	}

	//Tab control
	static char* s_aszTabs[2] = {"Platforms","Light Sets"};
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset,s_aszTabs,2);
	pxTabsDialog->SetControlID( (int)uTOOLS_DIALOGID_ENTITY_PLATFORM_TAB );
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	pxOverlay->SetCurrentParentDialog(pxTabsDialog);	

	//Platform tab
	pxTabsDialog->SelectTab(0);
	xOffset.y = fTabsStartY;

	const char* pszHelpText = "Indicates whether the selected entities should be included in level exports for this platform.";
	for(int i=0;i<Tools_Level_Export::ExportType_NumPlatforms;i++)
	{
		const char* pszPlatform = Tools_Level_Export::GetPlatformName(i);
		pxDialog = new Asura_SharedTickbox_Dialog(pszPlatform,xOffset,&s_axParam_EntityObject_Platform[i]);		
		pxDialog->SetHelpText(pszHelpText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	//End of platform tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//Light set tab
	pxTabsDialog->SelectTab(1);
	xOffset.y = fTabsStartY;

	pszHelpText = "Indicates whether the selected entities should be included in level exports for this light set.";
	for(u_int i=0;i<TOOLS_MAX_LIGHT_SETS;i++)
	{
		const char* szSetName = Tools_LightSets::GetLightSetNameByIndex(i);
		if(!szSetName) continue;

		pxDialog = new Asura_SharedTickbox_Dialog(szSetName,xOffset,&s_axParam_EntityObject_LightSet[i]);
		pxDialog->SetHelpText(pszHelpText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	//End of light set tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//End of tabs
	pxOverlay->SetCurrentParentDialog(0);
	xOffset.y = fTabsEndY + 1.0f;
}



void Tools_BaseEntity_Object::UpdateOverlay()
{
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(GetOverlayID());
	if(pxOverlay)
	{
		UpdateOverlayControls(pxOverlay);
	}
	pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_SELECTEDOBJECT_SIMUL);
	if(pxOverlay)
	{
		UpdateSimulationOverlayControls(pxOverlay);
	}
}


void Tools_BaseEntity_Object::Rename()
{
	Asura_Assert(ThisEntity,"");

	if(Tools_Object_System::AskForName(ThisEntity->Entity->ToolsData->Name,"Rename entity :",ThisEntity,0,ThisEntity->GetMyNamePrefix()))
	{
		Tools_SecondaryEntityFile_System::UpdateEntityInTree(ThisEntity, false);
		ThisEntity->SetEntityModified();
	}
}

void Tools_BaseEntity_Object::RenameAll()
{
	Asura_String xNewName;

	GetString_Overlay xOverlay( "Rename selected entities (Prefix + Name) :", xNewName.GetString() );
	if( !xOverlay.DoModal() ) return;

	xNewName = xOverlay.GetString();

	// Iterate over all selected Tools objects
	for( LIF<Tools_Object*> xObjectLIF( &Tools_Object_System::SelectedObjectList ); !xObjectLIF.done(); xObjectLIF.next() )
	{
		if( !xObjectLIF()->CanBeRenamed() ) continue;

		Tools_BaseEntity_Object* pxObject = static_cast<Tools_BaseEntity_Object*>( xObjectLIF() );
		if( !pxObject ) continue;

		// Prepend new name with object type prefix...
		Asura_String xObjectName = xNewName;
		xObjectName.Prepend( pxObject->GetMyNamePrefix() );

		int iNameIdx = 0;

		while( true )
		{
			Asura_String xFinalName = xObjectName;

			if( iNameIdx > 0 )
			{
				// Append index if needed
				Asura_Char szNameIdx[32];
				Asura_PrintfToBfr( szNameIdx, " %d", 32, true, iNameIdx );
				xFinalName += szNameIdx;
			}

			iNameIdx++;

			if( !Tools_Object_System::IsNameInUse( xFinalName, pxObject ) )
			{
				// Rename object, continue with next in list
				pxObject->Rename( xFinalName.GetString() );
				break;
			}

			// For extra safety:
			if( iNameIdx < 0 )
			{
				Asura_Assert( 0, "Unlikely error: too many objects with the same name. Some objects were renamed, others were not." );
				return;
			}
		}
	}
}

bool Tools_BaseEntity_Object::Rename(const char* szName)
{
	if(!szName || !*szName) return false;

	if(Tools_Object_System::IsNameInUse(szName, this))
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"That name is already in use");
		return false;
	}

	Entity->ToolsData->Name = szName;
	Tools_SecondaryEntityFile_System::UpdateEntityInTree(this, false);
	SetEntityModified();
	
	return true;
}

void Tools_BaseEntity_Object::EditComment()
{
	Asura_Assert(ThisEntity,"");

	GetString_Overlay xOverlay("Comment Text :",ThisEntity->Entity->ToolsData->m_xComment.GetString(),1000);
	if(xOverlay.DoModal())
	{
		ThisEntity->Entity->ToolsData->m_xComment = xOverlay.GetString();
		ThisEntity->SetEntityFileModified();

		Tools_Object_System::SetCommentText(ThisEntity->Entity->ToolsData->m_xComment.GetString(),ThisEntity);
	}
}	



void Tools_BaseEntity_Object::RenderMessageBlockLinks(int messageBlock)
{

	int numMessages = Asura_StaticMessage_System::GetNumberOfMessagesInBlock(messageBlock);
	if(!numMessages) return;

	for(int i=0;i<numMessages;i++)
	{
		Asura_StaticMessage* message = Asura_StaticMessage_System::GetMessagePointer(messageBlock,i);
		//find the destination object
		Tools_Object* object = Tools_Object_System::GetObjectFromGuid(message->m_uSendTo,true);
		if(!object) continue;

		Asura_Render::RenderLine(Position,object->Position,0xffff0000);
	}

}

bool Tools_BaseEntity_Object::HandlesThisMessage( int iMessage )
{
	switch (iMessage)
	{
		case ASURA_MSG_ACTIVATE:
		case ASURA_MSG_DEACTIVATE:
		case ASURA_MSG_DESTRUCT:
		{
			return true;
		}
		default:
		{
			return PARENT::HandlesThisMessage( iMessage );
		}
	}
}

bool Tools_BaseEntity_Object::HandlesThisEvent(int iEventID) const
{
	switch(iEventID)
	{
		case ASURA_ENTITY_EVENT_ONACTIVATION:
		case ASURA_ENTITY_EVENT_ONDEACTIVATION:
		{
			// if parent didn't say we support these, then only support them if 
			// we don't deal with activate internally. (As it's the base class
			// activation support that causes messages to be sent).
			return !DealsInternallyWithActivate();
		}
		default:
		{
			return false;
		}
	}
}

void Tools_BaseEntity_Object::SetEntityActive(bool bActive)
{
	Entity->m_bEntityActive = bActive;
	SetEntityModified();
}

void Tools_BaseEntity_Object::SetEntityModified()
{
	SetEntityFileModified();
	if(Entity->ToolsData) Entity->ToolsData->m_bEntityModified = true;
	UpdateToolsEntityTimeStamp( Entity->ToolsData );
}

void Tools_BaseEntity_Object::SetEntityFileModified()
{
	if(Entity && Entity->ToolsData)
	{
		Tools_SecondaryEntityFile_System::SetFileModified(Entity->ToolsData->m_xSecondaryEntityFilename.GetString());
	}
	else
	{
		Tools_SaveAsura::SetEntityModified();
	}
}

bool Tools_BaseEntity_Object::CanSaveFile() const
{
	if(Entity && Entity->ToolsData)
	{
		return Tools_SecondaryEntityFile_System::CanSaveFile(Entity->ToolsData->m_xSecondaryEntityFilename.GetString());
	}
	else
	{
		return Tools_SaveAsura::CanSaveEntity();
	}
}


bool Tools_BaseEntity_Object::IsMemberOfTemplateInstance() const
{
	return Entity->ToolsData->IsMemberOfTemplate();
}

bool Tools_BaseEntity_Object::IsSelectable()
{
	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	return ((IsMemberOfTemplateInstance() && !Tools_MessageEditor::IsInMessageEditingMode()) && !Tools_Object::GetAllowChildObjectSelection()) ? false : Tools_Object::IsSelectable();
	#else
	return (IsMemberOfTemplateInstance() && !Tools_Object::GetAllowChildObjectSelection())? false : Tools_Object::IsSelectable();
	#endif
}


void Tools_BaseEntity_Object::PrintInfo(float& x,float& y)
{
	Tools_Object::PrintInfo(x,y);

	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Created : %s",Entity->ToolsData->m_xCreationData.GetString());
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Modified : %s",Entity->ToolsData->m_xLastModified.GetString());

	if(Entity->ToolsData->m_xSecondaryEntityFilename.IsBlank())
	{
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Entity File : %s",Tools_SecondaryEntityFile_System::GetMainFileLabel());
	}
	else
	{
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Entity File : %s",Entity->ToolsData->m_xSecondaryEntityFilename.GetString());
	}

	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Folder : %s",Entity->ToolsData->m_xEntityBrowserPath.IsBlank() ? "" : Entity->ToolsData->m_xEntityBrowserPath.GetString());


	if(Tools_Entity_Simulation::IsActive())
	{
		Asura_Text_Output::Print(x,y++,0xffffff00,0,"Destroyed : %s",Entity->IsDestroyed() ? "Yes" : "No");
	}
}


void Tools_BaseEntity_Object::Copy(const Tools_Object* pxObject)
{
	const Tools_BaseEntity_Object*	pxFromObject	= static_cast<const Tools_BaseEntity_Object*>(pxObject);

	const Asura_Entity_Server* pxFromEntity	= static_cast<const Asura_Entity_Server*>(pxFromObject->Entity);
	Asura_Entity_Server* pxToEntity = m_pxServerEntity;

	//Delete any old message blocks
	for(int i=0;i<pxToEntity->m_iNumberOfEvents;i++)
	{
		Asura_EventAndMessage* pxEvent = &pxToEntity->m_pxEvents[i];

		Asura_StaticMessage_System::DeleteMessageBlock(pxEvent->m_usMessageBlockID);
		pxEvent->m_usEventID = ASURA_STATICMESSAGE_EMPTYBLOCK;
	}
	pxToEntity->m_iNumberOfEvents = 0;
	delete [] pxToEntity->m_pxEvents;
	pxToEntity->m_pxEvents = 0;

	//And copy some any new ones
	pxToEntity->m_iNumberOfEvents = pxFromEntity->m_iNumberOfEvents;
	if(pxToEntity->m_iNumberOfEvents)
	{
		pxToEntity->m_pxEvents = new Asura_EventAndMessage[pxToEntity->m_iNumberOfEvents];

		for(int i=0;i<pxToEntity->m_iNumberOfEvents;i++)
		{
			const Asura_EventAndMessage* pxFromEvent = &pxFromEntity->m_pxEvents[i];
			Asura_EventAndMessage* pxToEvent = &pxToEntity->m_pxEvents[i];

			pxToEvent->m_usEventID = pxFromEvent->m_usEventID;
			pxToEvent->m_usMessageBlockID = Asura_StaticMessage_System::CopyMessageBlock(pxFromEvent->m_usMessageBlockID,pxFromObject->GetGuid(),GetGuid());
		}
	}

	pxToEntity->m_bEntityActive = pxFromEntity->m_bEntityActive;

	//copy platform setting in the tools data
	Entity->ToolsData->m_uPlatforms = pxFromObject->Entity->ToolsData->m_uPlatforms;
	Entity->ToolsData->m_uLightSets = pxFromObject->Entity->ToolsData->m_uLightSets;

	//If this was a newly created object  ,then attempt to generate a name from the object being copied. RAR 16/06/2006
	if(Tools_Object_System::GetCopyType() == TOOLS_COPYOBJECT_PASTENEW || Tools_Object_System::GetCopyType() == TOOLS_COPYOBJECT_DUPLICATE)
	{
		GenerateCopiedName(pxObject);
	}
}


void Tools_BaseEntity_Object::GenerateCopiedName(const Tools_Object* pxFromObject)
{
	//Casting away constness for GetName() since it is in practice const , but too much of a pain
	//to correct it since it is virtual. RAR 16/06/2006
	Asura_String xFromName = ((Tools_Object*)pxFromObject)->GetName();
	if(xFromName.IsBlank()) return;

	//If the name of the object being copied contained a guid , then it is most likely just a default name , so
	//there is no point in trying to generate another name from it
	Asura_Guid uGuid = pxFromObject->GetGuid();
	char szGuidString[20];
	Asura_PrintfToBfr(szGuidString,"%d",sizeof(szGuidString),true,uGuid);

	if(xFromName.Contains(szGuidString))
	{
		return;
	}

	//check also subtracting the 100000 for the start of tools guids
	Asura_PrintfToBfr(szGuidString,"%d",sizeof(szGuidString),true,uGuid-ASURA_GUID_TOOLSCREATEDENTITY_START);

	if(xFromName.Contains(szGuidString))
	{
		return;
	}


	

	//If there is a numerical postfix on the name , cut it off , so it can be incremented when making the new name
	char szPrefix[200];
	Asura_PrintfToBfr(szPrefix,"%s",sizeof(szPrefix),true,xFromName.GetString());

	char* pcEnd = &szPrefix[strlen(szPrefix)-1];

	while(pcEnd>&szPrefix[0])
	{
		if(*pcEnd == ' ' || (*pcEnd >= '0' && *pcEnd <= '9'))
		{
			pcEnd--;
		}
		else
		{
			break;
		}
	}
	pcEnd++;

	//Get the number (if any) from the end
	int iPostfixNumber = 0;

	sscanf(pcEnd,"%d",&iPostfixNumber);
	*pcEnd = 0;

	char szNewName[300];
	do
	{
		Asura_PrintfToBfr(szNewName,"%s %d",sizeof(szNewName),true,szPrefix,++iPostfixNumber);
	}
	while(Tools_Object_System::IsNameInUse(szNewName));

	Entity->ToolsData->Name = szNewName;

}


bool Tools_BaseEntity_Object::IsHidden()
{
	if(Tools_Object::IsHidden()) return true;

	//Check for visibility by platform and light set type
	if(Tools_Object_System::IsHideByPlatformEnabled())
	{
		if(!(Entity->ToolsData->m_uPlatforms & Tools_Object_System::GetVisiblePlatforms())) return true;
	}
	if(Tools_Object_System::IsHideByLightSetEnabled())
	{
		if(!(Entity->ToolsData->m_uLightSets & Tools_Object_System::GetVisibleLightSets())) return true;
	}
	if(Tools_LightSets::ShouldHideObjectsInOtherSets())
	{
		if(!Tools_LightSets::IsMemberOfCurrentLightSets(Entity->ToolsData->m_uLightSets)) return true;
	}

	//If we've only created some of the entity objects (so not in a mode where all entities are shown) and don't have the flag
	//for showing objects in other modes, then hide this entitiy
	//RAR 02/04/2008
	if(!Tools_Entity_Object::HasCreatedAllObjects())
	{
		if(!Tools_Object_System::ShouldShowPhysicalObjectsInOtherModes() || Tools_Render_Options::IsEnvironmentHidden())
		{
			return true;
		}
	}
	
	return false;
}


bool Tools_BaseEntity_Object::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{
	if(uParamID == g_uHash_Entity_StartsActive)
	{
		pxParam->SetBool(Entity->m_bEntityActive);
		return true;
	}
	if(uParamID>=g_uHash_EntityObject_Platform0 && uParamID<g_uHash_EntityObject_Platform0+Tools_Level_Export::ExportType_NumPlatforms)
	{
		//matches id of one of the platform tickboxes
		u_int uPlatform = uParamID - g_uHash_EntityObject_Platform0;
		pxParam->SetBool(Entity->ToolsData->ShouldExportToPlatform((int)uPlatform));

		return true;
	}
	if(uParamID>=g_uHash_EntityObject_LightSet0 && uParamID<g_uHash_EntityObject_LightSet0+TOOLS_MAX_LIGHT_SETS)
	{
		//matches id of one of the light set tickboxes
		u_int uLightSet = uParamID - g_uHash_EntityObject_LightSet0;
		pxParam->SetBool(Entity->ToolsData->IsMemberOfLightSets(1<<uLightSet));

		return true;
	}

	return false;
}

bool Tools_BaseEntity_Object::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	if(uParamID==g_uHash_Entity_StartsActive)
	{
		Entity->m_bEntityActive = pxParam->GetBool();

		SetEntityModified();
		return true;
	}
	if(uParamID>=g_uHash_EntityObject_Platform0 && uParamID<g_uHash_EntityObject_Platform0+Tools_Level_Export::ExportType_NumPlatforms)
	{
		//matches id of one of the platform tickboxes
		u_int uPlatform = uParamID - g_uHash_EntityObject_Platform0;
		Entity->ToolsData->SetExportToPlatform((int)uPlatform,pxParam->GetBool());

		SetEntityModified();
		return true;
	}
	if(uParamID>=g_uHash_EntityObject_LightSet0 && uParamID<g_uHash_EntityObject_LightSet0+TOOLS_MAX_LIGHT_SETS)
	{
		//matches id of one of the light set tickboxes
		u_int uLightSet = uParamID - g_uHash_EntityObject_LightSet0;
		u_int uNewLightSets = Entity->ToolsData->m_uLightSets;
		// RWM - technically, ToolsData->SetExportForLightSet does this, but unifying the calls to setting lightsets.
		if(pxParam->GetBool())
			uNewLightSets |= (1<<uLightSet);
		else
			uNewLightSets &=~ (1<<uLightSet);

		SetLightSets(uNewLightSets);
		SetEntityModified();
		return true;
	}

	return false;
}



void Tools_MessageLogAction_GotoEntity::Execute()
{
	//find the object
	Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(m_uGuid);
	if(pxObject)
	{
		Tools_Object_System::GotoAndSelect(pxObject);
	}
	else
	{
		Tools_Timed_Message_System::AddUnloggedMessage(0xff0000,"Failed to find a tools object for the entity");
	}
}

void Tools_BaseEntity_Object::EditMessages(int iIndex)
{
	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode();
	#else
	Asura_Assert(ThisEntity,"");
	Asura_Entity_Server* pxEntity = ThisEntity->m_pxServerEntity;

	Asura_Assert(iIndex>=0 && iIndex<pxEntity->m_iNumberOfEvents,"");

	Asura_EventAndMessage* pxEvent = &pxEntity->m_pxEvents[iIndex];
	Tools_Object_System::EditMessageBlock(Tools_Message_Editing::GetEventDescription(pxEvent->m_usEventID),&pxEvent->m_usMessageBlockID);
	#endif
}

bool Tools_BaseEntity_Object::OwnsThisMessageBlock(unsigned short usBlockID)
{
	
	for(int i=0;i<m_pxServerEntity->m_iNumberOfEvents;i++)
	{
		if(m_pxServerEntity->m_pxEvents[i].m_usMessageBlockID == usBlockID)
		{
			return true;
		}
	}
	return false;
}

bool Tools_BaseEntity_Object::GetEventDescForMessageBlock(unsigned short usBlockID,Asura_String& xOutputDesc)
{
	
	for(int i=0;i<m_pxServerEntity->m_iNumberOfEvents;i++)
	{
		if(m_pxServerEntity->m_pxEvents[i].m_usMessageBlockID == usBlockID)
		{
			xOutputDesc = Tools_Message_Editing::GetEventDescription(m_pxServerEntity->m_pxEvents[i].m_usEventID);
			return true;
		}
	}
	return false;
}

Asura_String Tools_BaseEntity_Object::GetBlueprintTypeName() const
{
    if ( !Entity ) { return ""; }

    const Asura_String* pxTypeName = Asura_Blueprint_System::GetTypeName( Entity->GetBlueprintTypeHashID() );
    return ( pxTypeName ? *pxTypeName : "" );
}

Asura_String Tools_BaseEntity_Object::GetBlueprintName() const
{
    if ( !Entity ) { return ""; }
    if ( Entity->GetBlueprintTypeHashID() == ASURA_HASH_ID_UNSET || Entity->GetBlueprintHashID() == ASURA_HASH_ID_UNSET ) { return ""; }

    Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( Entity->GetBlueprintTypeHashID(), Entity->GetBlueprintHashID() );
    if ( pxBlueprint )
    {
        return pxBlueprint->GetName();
    }

    return "";
}

bool Tools_BaseEntity_Object::CanAddAnyMoreMessageBlocks() const
{
	//see if there is any event that this object can handle ,that hasn't been added yet
	const EventIDAndDescription* pxEventList = Tools_Message_Editing::GetEventList();
	if(!pxEventList) return false;

	const Asura_Entity_Server* pxEntity = m_pxServerEntity;
	const Asura_EventAndMessage* pxExistingEvents = pxEntity->GetEvents();
	const int iNumExistingEvents = pxEntity->GetNumEvents();

	for( const EventIDAndDescription* pxEvent = pxEventList; pxEvent->m_pszDescription; ++pxEvent )
	{
		if(!HandlesThisEvent(pxEvent->m_iEventID)) continue;

		//have we a message block for this event?
		bool bEventExists = false;
		for(int i=0;i<iNumExistingEvents;i++)
		{
			if(pxExistingEvents[i].m_usEventID == pxEvent->m_iEventID)
			{
				bEventExists = true;
				break;
			}
		}
		if(!bEventExists) return true;
	}

	// See if any of the 'on health %' ones can be added.
	for( u_int u = 0; u < 100; ++u )
	{
		int iEventID = ASURA_ENTITY_EVENT_ONHEALTHPERCENT_0 + u;

		if( !HandlesThisEvent(iEventID) ) { continue; }

		bool bEventExists = false;
		for( int i = 0; i < iNumExistingEvents; ++i )
		{
			if( pxExistingEvents[i].m_usEventID == iEventID )
			{
				bEventExists = true;
				break;
			}
		}
		if( bEventExists ) { continue; }

		// Ok, this one can be added.
		return true;
	}

	
	//failed to find any events that could be added
	return false;
}

void Tools_BaseEntity_Object::AddMessageBlock()
{
	Asura_Assert(ThisEntity,"");
	Asura_Entity_Server* pxEntity = ThisEntity->m_pxServerEntity;

	//get a list of events that this object can handle , but hasn't got a message block for yet
	ListSelection_Overlay xOverlay("Add messages for which event?");
	
	const Asura_EventAndMessage* pxExistingEvents = pxEntity->m_pxEvents;
	const int iNumExistingEvents = pxEntity->m_iNumberOfEvents;

	const EventIDAndDescription* pxEventList = Tools_Message_Editing::GetEventList();
	if(pxEventList)
	{
		for(const EventIDAndDescription* pxEvent = pxEventList;pxEvent->m_pszDescription;pxEvent++)
		{
			if(!ThisEntity->HandlesThisEvent(pxEvent->m_iEventID)) continue;

			//have we a message block for this event?
			bool bEventExists = false;
			for(int i=0;i<iNumExistingEvents;i++)
			{
				if(pxExistingEvents[i].m_usEventID == pxEvent->m_iEventID)
				{
					bEventExists = true;
					break;
				}
			}
			if(!bEventExists)
			{
				xOverlay.AddItem(pxEvent->m_pszDescription,pxEvent->m_iEventID,false,pxEvent->m_szHelpText);
			}
		}
	}

	// if there are any health percent events we can still handle, add the 'On X% health' item
	if(ThisEntity->HandlesThisEvent(ASURA_ENTITY_EVENT_ONHEALTHPERCENT_0))
	{
		for( u_int u = 0; u < 100; ++u )
		{
			int iEventID = ASURA_ENTITY_EVENT_ONHEALTHPERCENT_0 + u;

			// We assume that if we've got in this function then this object
			// supports all of these events (ie. no objects that support 'on 50%'
			// but not 'on 51%').
			Asura_Assert( ThisEntity->HandlesThisEvent(iEventID), "" );

			// But still don't let the user add the block twice.
			bool bEventExists = false;
			for( int i = 0; i < iNumExistingEvents; ++i )
			{
				if( pxExistingEvents[i].m_usEventID == iEventID )
				{
					bEventExists = true;
					break;
				}
			}
			if( bEventExists ) { continue; }

			xOverlay.AddItem( "On X% Health", iEventID );
			break;
		}
	}

	if(!xOverlay.GetNumItems())
	{
		//shouldn't be getting to this function if no events can be added
		Asura_Assert(0,"No more message blocks can be added to this object");
		return;
	}

	if(!xOverlay.DoModal()) return;	

	ThisEntity->Changed = true; // Make sure we prompt for saving when we exit the mode.

	int iEventID = xOverlay.GetSelectionData();

	// If the 'On X% health' event was selected, spawn a second overlay to set the health
	if( (iEventID >= ASURA_ENTITY_EVENT_ONHEALTHPERCENT_0) && (iEventID <= ASURA_ENTITY_EVENT_ONHEALTHPERCENT_99) )
	{
		bool bValid = false;
		while( !bValid )
		{
			GetNumber_Overlay xHealthOverlay("Select Health Percent",50.0f,0.0f,99.0f,"%2.0f");
			if( !xHealthOverlay.DoModal() ) return;

			const int iHealth = Asura_Maths::FloatToInt_Chop( xHealthOverlay.GetNumber() );
			iEventID = ASURA_ENTITY_EVENT_ONHEALTHPERCENT_0 + iHealth;

			// Still don't let the user add the block twice.
			bValid = true;
			for( int i = 0; i < iNumExistingEvents; ++i )
			{
				if( pxExistingEvents[i].m_usEventID == iEventID )
				{
					bValid = false;

					char szTemp[256];
					Asura_PrintfToBfr( szTemp, "An 'On %d%% Health' event already exists for this entity.", 256, true, iHealth );
					Message_Overlay xMsgOverlay( szTemp );
					xMsgOverlay.DoModal();				
				}
			}
		}
	}

	ThisEntity->AddEventMessageBlock( iEventID );

	//Edit the message block for the new event
	EditMessages(pxEntity->m_iNumberOfEvents-1);
}


void Tools_BaseEntity_Object::AddEventMessageBlock( int iEventID )
{
	Asura_Assert(ThisEntity,"");
	Asura_Entity_Server* pxEntity = ThisEntity->m_pxServerEntity;

	Asura_EventAndMessage* pxOldEvents = pxEntity->m_pxEvents;
	pxEntity->m_pxEvents = new Asura_EventAndMessage[pxEntity->m_iNumberOfEvents+1];
	for(int i=0;i<pxEntity->m_iNumberOfEvents;i++)
	{
		pxEntity->m_pxEvents[i] = pxOldEvents[i];
	}
	delete [] pxOldEvents;

	//initialse new event
	pxEntity->m_pxEvents[pxEntity->m_iNumberOfEvents].m_usEventID = (unsigned short) iEventID;
	pxEntity->m_pxEvents[pxEntity->m_iNumberOfEvents].m_usMessageBlockID = ASURA_STATICMESSAGE_EMPTYBLOCK;
	pxEntity->m_iNumberOfEvents++;
};


int Tools_BaseEntity_Object::RemoveEventMessageBlock(int iEventID)
{
	Asura_Assert(ThisEntity,"");
	Asura_Entity_Server* pxEntity = ThisEntity->m_pxServerEntity;

	Asura_Assert(pxEntity->m_iNumberOfEvents > 0, "No events exist");

	if(pxEntity->m_iNumberOfEvents <= 0)
	{
		return -1;
	}

	// Find this event in our list

	int iEventIndex = -1;

	for(int i = 0; i < pxEntity->m_iNumberOfEvents; i++)
	{
		if(pxEntity->m_pxEvents[i].m_usEventID == iEventID)
		{
			iEventIndex = i;
			break;
		}
	}

	Asura_Assert(iEventIndex != -1, "Could not find event id in entity's event list");

	if(iEventIndex == -1)
	{
		return -1;
	}

	// Create a new list with one less element and copy the data

	Asura_EventAndMessage* pxOldEvents = pxEntity->m_pxEvents;

	if(pxEntity->m_iNumberOfEvents == 1)
	{
		pxEntity->m_pxEvents = NULL;
	}
	else
	{
		pxEntity->m_pxEvents = new Asura_EventAndMessage[pxEntity->m_iNumberOfEvents - 1];
	}

	int iCounter = 0;

	for(int i = 0; i < pxEntity->m_iNumberOfEvents; i++)
	{
		if(i != iEventIndex)
		{
			pxEntity->m_pxEvents[iCounter] = pxOldEvents[i];
			iCounter++;
		}
	}

	// Delete the old array

	delete [] pxOldEvents;

	pxEntity->m_iNumberOfEvents--;

	return iEventIndex;
}


void Tools_BaseEntity_Object::RemoveEmptyMessageBlocks()
{
	if(IsSelected())
	{
		return;//shouldn't be doing this while the object is selected
	}	

	if(Entity)
	{
		//get rid of any unused message blocks
		int iNumberOfEvents = 0;
		for(int i=0;i<m_pxServerEntity->m_iNumberOfEvents;i++)
		{
			if(m_pxServerEntity->m_pxEvents[i].m_usMessageBlockID != ASURA_STATICMESSAGE_EMPTYBLOCK)
			{
				if(i!=iNumberOfEvents)
				{
					m_pxServerEntity->m_pxEvents[iNumberOfEvents] = m_pxServerEntity->m_pxEvents[i];
				}
				iNumberOfEvents++;
			}
		}
		m_pxServerEntity->m_iNumberOfEvents = iNumberOfEvents;
	}
}

const char* Tools_BaseEntity_Object::GetEntityFileName() const
{
	if(Entity->ToolsData->m_xSecondaryEntityFilename.IsBlank())
	{
		return Tools_SecondaryEntityFile_System::GetMainFileLabel();
	}
	else
	{
		return Entity->ToolsData->m_xSecondaryEntityFilename.GetString();
	}
	
}

void Tools_BaseEntity_Object::SetLightSets(u_int uLightSets)
{

	if(Entity->ToolsData->m_uLightSets != uLightSets)
	{
		Entity->ToolsData->m_uLightSets = uLightSets;
		SetEntityModified();
	}
}

u_int Tools_BaseEntity_Object::GetLightSets() const
{
	if( Entity && Entity->ToolsData )
	{
		return Entity->ToolsData->m_uLightSets;
	}
	
	return 0;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------**
** Tools_EntityObject **
**--------------------*/

Tools_Entity_Object::CreatedEntityObjects Tools_Entity_Object::s_eCreatedEntityObjects = Tools_Entity_Object::TOOLS_CREATEDENTITYOBJECTS_NONE;

Tools_Entity_Object::Tools_Entity_Object(Asura_Entity* entity)
: PARENT(entity)
{
	if (Entity && Entity->ToolsData)
	{
		Entity->ToolsData->ToolsObject = this;
	}

	s_xList.AddToList(this);
}

Tools_Entity_Object::~Tools_Entity_Object()
{
	s_xList.RemoveFromList(this);
}

void Tools_Entity_Object::CreateAllObjects()
{
	s_eCreatedEntityObjects = TOOLS_CREATEDENTITYOBJECTS_ALL;

	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_BaseEntity_Object::CreateAllObjects");

	// Piggyback here to create all Phonons.
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Mode_Sound::CreatePhononToolsObjects" );
		Tools_Mode_Sound::CreatePhononToolsObjects();
	}
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_StreamingBackgroundSound_Object::CreateAllObjects" );
		Tools_StreamingBackgroundSound_Object::CreateAllObjects();
	}

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "creating objects" );
		for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
		{
			Asura_Entity_Server* entity = it.GetCurrent();
			//ignore entities marked for destruction
			if(entity->IsDestroyed()) continue;
			//and those with non-tools guids
			if(entity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || entity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

			CreateObject(entity);
		}
	}

	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Entity_CutsceneController_Object::UpdateCutsceneControllersFromCutscenes" );
		Tools_Entity_CutsceneController_Object::UpdateCutsceneControllersFromCutscenes();
	}
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_SecondaryEntityFile_System::UpdateFromSecondaryFiles" );
		Tools_SecondaryEntityFile_System::UpdateFromSecondaryFiles();
	}
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Entity_TemplateInstance_Object::UpdateAllTemplates" );
		Tools_Entity_TemplateInstance_Object::UpdateAllTemplates();
	}
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Entity_PhysicalObject::CheckIsMergedWithEnvironmentFlags" );
		Tools_Entity_PhysicalObject::CheckIsMergedWithEnvironmentFlags();
	}

	//Creating the tools objects may result in missing skins being loaded , and so
	//collision meshes now being available.
	//Reset all the dynamics objects to make sure they are up to date. RAR 07/04/2005
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "Asura_ServerEntity_PhysicalObject::ResetAllCollisionObjects" );
		Asura_ServerEntity_PhysicalObject::ResetAllCollisionObjects();
	}

	//Force all the entities to update their client entities so that any changes
	//will be copied across when quitting to game
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "updating clients" );
		for(Asura_Hashed_List_It<Tools_Entity_Object> xIt(&s_xList);!xIt.Done();xIt.Next())
		{
			xIt.GetCurrent()->UpdateClient();
		}
	}
	

	//Create objects to represent some of the non-tools guids
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "create objects to represent some of the non-tools guids" );
		new Tools_SpecialGuid_Object(ASURA_GUID_FIRSTPLAYER,"Player 1","ToolsShape_CutsceneActor");
		new Tools_SpecialGuid_Object(ASURA_GUID_FIRSTPLAYER+1,"Player 2","ToolsShape_CutsceneActor");
		new Tools_SpecialGuid_Object(ASURA_GUID_FIRSTPLAYER+2,"Player 3","ToolsShape_CutsceneActor");
		new Tools_SpecialGuid_Object(ASURA_GUID_FIRSTPLAYER+3,"Player 4","ToolsShape_CutsceneActor");
		new Tools_SpecialGuid_Object(ASURA_GUID_SERVER,"Game Server","ToolsShape_CutsceneCameraTarget");
		new Tools_SpecialGuid_Object(ASURA_GUID_ALL_CLIENTMACHINES,"All Clients","ToolsShape_CutsceneCameraTarget");
	}

	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "new Tools_DialogueSpeaker_Object" );
		new Tools_DialogueSpeaker_Object();
	}
	#endif

	if( s_pfnProjectCreateAllObjects )
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER( "s_pfnProjectCreateAllObjects" );
		s_pfnProjectCreateAllObjects();
	}
}

void Tools_Entity_Object::CreatePhysicalObjects()
{
	if(s_eCreatedEntityObjects != TOOLS_CREATEDENTITYOBJECTS_NONE) return;
	s_eCreatedEntityObjects = TOOLS_CREATEDENTITYOBJECTS_PHYSICAL;
	
	for(Asura_Hashed_List_It<Asura_ServerEntity_PhysicalObject> xIt(Asura_ServerEntity_PhysicalObject::GetList());!xIt.Done();xIt.Next())
	{
		Asura_Entity_Server* pxEntity = xIt.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

		CreateObject(pxEntity);
	}

	
}


void Tools_Entity_Object::CreateObject(Asura_Entity* pxEntity)
{
	Asura_Assert(!pxEntity->IsDestroyed(),"");

	//Skip entities that already have tools objects
	if(pxEntity->ToolsData && pxEntity->ToolsData->ToolsObject) return;
	
	
	//create a tools object for this pxEntity
	if(!Project_CreateToolsEntityObject(pxEntity))
	{
		switch(pxEntity->GetClassification())
		{
			case AsuraEntityClass_VolumeTriggerRedundant :
				Asura_Assert(false, "The old volume trigger class is now redundant");
				break;

			case AsuraEntityClass_TimeTrigger:
				new Tools_Entity_TimeTrigger_Object(pxEntity);
				break;

			case AsuraEntityClass_TimelineTrigger:
				new Tools_Entity_TimelineTrigger_Object(pxEntity);
				break;


			case AsuraEntityClass_CutsceneController:
				new Tools_Entity_CutsceneController_Object(pxEntity);
				break;

			case AsuraEntityClass_SplitterBlock:
				new Tools_Entity_SplitterBlock_Object(pxEntity);
				break;

			case AsuraEntityClass_CountedTrigger:
				new Tools_Entity_CountedTrigger_Object(pxEntity);
				break;

#ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
			case AsuraEntityClass_MusicTrigger:
				new Tools_Entity_MusicTrigger_Object(pxEntity);
				break;
#endif

#ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
			case AsuraEntityClass_MetaMusicTrigger:
				new Tools_Entity_MetaMusicTrigger_Object(pxEntity);
				break;
#endif

			case AsuraEntityClass_FMVTrigger:
				new Tools_Entity_FMVTrigger_Object(pxEntity);
				break;

       	    case AsuraEntityClass_SoundController:
       	        // phonon object should exist for this, maybe assert?
       	        break;

			case AsuraEntityClass_Template:
				Tools_Entity_Template_Object::CreateToolsObject(pxEntity);
				break;

			case AsuraEntityClass_LookAtTrigger:
				new Tools_Entity_LookAtTrigger_Object(pxEntity);
				break;

            case AsuraEntityClass_ClockTrigger:
                new Tools_Entity_ClockTrigger_Object(pxEntity);
                break;

			case AsuraEntityClass_PFX_Effect:
				new Tools_Entity_PFX_Effect(pxEntity);
				break;

			case AsuraEntityClass_LogicTrigger :
				new Tools_Entity_LogicTrigger_Object(pxEntity);
				break;

#ifdef ASURA_USE_DIALOGUE_SYSTEM
			case AsuraEntityClass_DialogueTrigger :
				new Tools_Entity_DialogueTrigger(pxEntity);
				break;

			case AsuraEntityClass_DialogueController :
				new Tools_Entity_DialogueController(pxEntity);
				break;
#endif 
			case AsuraEntityClass_ClientVolumeTrigger :
				new Tools_Entity_ClientVolumeTrigger_Object(pxEntity);
				break;

			case AsuraEntityClass_StartPoint:
				new Tools_Entity_StartPoint(pxEntity);
				break;

			case AsuraEntityClass_EnvTextureAnimControl :
				new Tools_Entity_EnvTextureAnimControl(pxEntity);
				break;

#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER
			case AsuraEntityClass_DebugMessageTrigger:
				new Tools_Entity_DebugMessageTrigger_Object(pxEntity);
				break;
			case AsuraEntityClass_DebugWaypointTrigger:
				new Tools_Entity_DebugWaypointTrigger_Object( pxEntity );
				break;
#endif

			case AsuraEntityClass_ProxyTrigger :
				new Tools_Entity_ProxyTrigger_Object(pxEntity);
				break;

            case AsuraEntityClass_Node:
                new Tools_Entity_Node(pxEntity);
                break;

            case AsuraEntityClass_OrientedNode:
                new Tools_EntityNode_OrientedNode(pxEntity);
                break;

            case AsuraEntityClass_Spline:
                new Tools_EntityNode_Spline(pxEntity);
                break;

			case AsuraEntityClass_LiftNode:
                new Tools_EntityNode_LiftNode(pxEntity);
                break;

			case AsuraEntityClass_LiftSpline:
                new Tools_EntityNode_LiftSpline(pxEntity);
                break;
#ifdef ASURA_USE_AXON
            case AsuraEntityClass_GamesceneNode:
                break;

            case AsuraEntityClass_CoverpointNode:
                new Tools_EntityNode_CoverpointNode(pxEntity);
                break;

            case AsuraEntityClass_GuardZone:
                new Tools_EntityNode_GuardZone(pxEntity);
                break;

            case AsuraEntityClass_AttractorNode:
                break;

            case AsuraEntityClass_GamesceneSpline:
                break;

            case AsuraEntityClass_GamesceneBase:
                Asura_Assert( false, "Shouldn't be creating a gamescene base like this." );
                break;

			case AsuraEntityClass_GS2Node:
				new Tools_EntityNode_GS2Node( pxEntity );
				break;

			case AsuraEntityClass_GS2Spline:
				new Tools_EntityNode_GS2Spline( pxEntity );
				break;

			case AsuraEntityClass_GS2Attractor:
				new Tools_EntityNode_GS2Attractor( pxEntity );
				break;
#endif

#if defined(ASURA_USE_AFFECTORS)
			case AsuraEntityClass_Affector:
				new Tools_Entity_Affector(pxEntity);
				break;

			case AsuraEntityClass_Affector_Wind:
				new Tools_Entity_Affector_Wind(pxEntity);
				break;
#endif	// defined(ASURA_USE_AFFECTORS)

#ifdef ASURA_TARGET_SM3
			case AsuraEntityClass_Projector:
				Tools_Entity_Projector::Create(pxEntity);
				break;

			case AsuraEntityClass_DynamicLight:
				Tools_Entity_DynamicLight::Create(pxEntity);
				break;

			case AsuraEntityClass_LightShaft:
				Tools_Entity_LightShaft::Create(pxEntity);
				break;
#endif

#if !defined(ASURA_DONT_USE_TELEPORTERS)
			case AsuraEntityClass_Teleporter:
				new Tools_Entity_Teleporter(pxEntity);
				break;

			case AsuraEntityClass_TeleportDestination:
				new Tools_Entity_TeleportDestination(pxEntity);
				break;
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

			case AsuraEntityClass_ForceField_Conveyor:
				new Tools_Entity_ForceField_Conveyor(pxEntity);
				break;

#ifdef ASURA_USE_ATTRACTORS
			case AsuraEntityClass_AttractorController :
				//Attractor controllers use the same tools objects as the attractor nodes
				//(Which get created elsewhere).
				break;
#endif

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
			case AsuraEntityClass_ConsoleVar:
				new Tools_Entity_ConsoleVar(pxEntity);
				break;
#endif //ASURA_SUPPORT_COMMAND_CONSOLE

			case AsuraEntityClass_StreamingBackgroundSoundController :
				//Streaming sound controllers use the same tools objects as the streaming sound objects
				//(Which get created elsewhere).
				break;

			case AsuraEntityClass_WaypointBlockerVolume:
				new Tools_Entity_WaypointBlockerVolume(pxEntity);
				break;

			case AsuraEntityClass_Objective:
				new Tools_Entity_Objective(pxEntity);
				break;

			case AsuraEntityClass_EffectVolume:
				new Tools_Entity_EffectVolume(pxEntity);
				break;

#ifdef ASURA_USE_CONTAINER_SYSTEM
			case AsuraEntityClass_Container:
				Tools_Entity_Container::Create(pxEntity);
				break;
#endif //#ifdef ASURA_USE_CONTAINER_SYSTEM

#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
			case AsuraEntityClass_DynamicMusicTrigger:
			new Tools_Entity_DynamicMusicTrigger_Object(pxEntity);
			break;
#endif

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
			case AsuraEntityClass_FMVController:
			{
				new Tools_Entity_FMVController(pxEntity);
				break;
			}

			case AsuraEntityClass_FMVSpeaker:
			{
				new Tools_Entity_FMVSpeaker(pxEntity);
				break;
			}
#endif

			default :
				Asura_Assertf(0,("Unrecognised pxEntity type : %d",pxEntity->GetClassification()));
				break;
		}
	}

	Tools_Entity_Object* pxToolsObject = Tools_Entity_Object::FindEntity( pxEntity->GetGuid() );
	if( pxToolsObject 
	 && pxToolsObject->DealsInternallyWithActivate() 
	 && !pxEntity->m_bEntityActive )
	{
		Asura_Assertf( false, ("Entity '%s' (Guid '%d') deals internally with active flag but m_bEntityActive is false. m_bEntityActive will be reset to true.", pxToolsObject->GetName(), pxToolsObject->GetGuid() ) );
		pxEntity->m_bEntityActive = true;
	}
}

void Tools_Entity_Object::DeleteUnwantedEntitiesForQuitToGame()
{
	for(Asura_Classified_List_It<Asura_Entity_Server> xIt(Asura_Entity_Server::GetList());!xIt.Done();xIt.Next())
	{
		Asura_Entity_Server* pxEntity = xIt.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;

		if(!pxEntity->ShouldIncludeInLevelExport())
		{
			//kill the client entity if it exists
			Asura_Entity_Client* pxClientEntity = Asura_Entity_Client::FindEntity(pxEntity->GetGuid());
			if(pxClientEntity)
			{
				pxClientEntity->SetAsDestroyed();
			}
			//and the server entity
			pxEntity->SetAsDestroyed();
		}
	}

	Asura_Entity_System::PruneEntityLists();

}
