//////////////////////////////////////////////////////////////////////////////
//
// Tools_Entity_Projector.h
//
// Copyright (c) 2007 Rebellion (Derby). All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___TOOLS_ENTITY_PROJECTOR_H___
#define ___TOOLS_ENTITY_PROJECTOR_H___ 1

#ifdef ASURA_TARGET_SM3

// I N C L U D E S ///////////////////////////////////////////////////////////

// Tools.
#include "Entity/Tools_Entity_Object.h"

// Project independent.
#include "Entity/Projector/Asura_ServerEntity_Projector.h"
#include "Entity/Projector/Asura_ClientEntity_Projector.h"

// C L A S S E S /////////////////////////////////////////////////////////////

class Tools_Entity_Projector : public Tools_Entity_Object
{

    public:

        // C L A S S   M E T H O D S /////////////////////////////////////////

        static void Create( void );
        static void Create( Asura_Entity* pxEntity );
        static Tools_Entity_Projector* CreateMain( void );

        // M E T H O D S /////////////////////////////////////////////////////

        // Query methods.
        virtual const char* GetTypeDescription( void );
		virtual bool  HandlesThisMessage(int iMessageID);

		static const char* GetNamePrefix() { return "Proj_"; }
		virtual const char* GetMyNamePrefix() const { return Tools_Entity_Projector::GetNamePrefix(); }

        // Behaviour / appearance methods.
	    virtual void AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffsetWithinOverlay );
        virtual void OnDeselection( void );
        virtual void OnSelection( void );
	    virtual void Render( void );
        virtual void UpdateWhileSelected( void );

		virtual void Copy(const Tools_Object* pxObject);
        virtual bool CanBeCopied( void ) const;
		virtual Tools_Object* CreateObjectOfThisType() const;

		virtual void AddFilesToExportList(List<Asura_String>& nameList) const;
		
		static void PickTexture_Falloff(void);
		static void PickTexture_Projected(void);
		static void PickTexture_Detail(void);
		static void ClearTexture_Falloff(void);
		static void ClearTexture_Projected(void);
		static void ClearTexture_Detail(void);

		static void UpdateTextureNames(void);

    protected:

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////
        
        virtual ~Tools_Entity_Projector( void );

    private:

		static void CreateAttachDlg(void);
		static void UpdateAttachDlg();
		static void CreateSelectMasterDlg(void);
		static void SetAttachOffset(void);
		static void SetAttachMarker();
		static void SetMasterObject(void);
		static void UnattachLight(void);
		static void GetAttachMarkerName(Asura_String& xName);

		enum DialogID
		{
			DialogID_SelectMarker
		};

        // T Y P E S /////////////////////////////////////////////////////////

        typedef Tools_Entity_Object PARENT;

        // C O N S T R U C T O R S   /   D E S T R U C T O R S ///////////////
        
        Tools_Entity_Projector( Asura_ServerEntity_Projector* pxEntity );

        // M E T H O D S /////////////////////////////////////////////////////


        // S T A T E /////////////////////////////////////////////////////////        
		Asura_ServerEntity_Projector::ProjectorParams   m_xParams;
		Asura_ServerEntity_Projector		  *m_pxServerEntity;
        bool								   m_bParamsModified;			
		
        // Display.
        static bool m_bShowVolume;
		static bool	s_bDynamicLightFlagChanged;
		static bool	s_bFakeLightFlagChanged;
		
		static bool s_bModified;
		static bool s_bResetCurrentProjectorPtrOnDeselect;

public:
		static Tools_Entity_Projector *s_pxSelected;
};



// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif //ASURA_TARGET_SM3

#endif // #ifndef ___TOOLS_ENTITY_PROJECTOR_H___
