//
// Tools_Entity_SplitterBlock_Object.cpp
//
// Created by Richard Rice - 18/09/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Debugging/Asura_Assert.h"

#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

//#include "Asura_Entity.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Entity/Triggers/Asura_ServerEntity_SplitterBlock.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Text_Output.h"

#include "Objects/Tools_Object_System.h"
#include "Load&Save/Tools_SaveAsura.h"

#include "Entity/Tools_Entity_SplitterBlock_Object.h"
#include "Entity/Tools_Entity_Simulation.h"
#include "Entity/Tools_Message_Editing.h"

#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
#include "Entity/Tools_MessageEditor.h"
#endif

Tools_Entity_SplitterBlock_Object::Tools_Entity_SplitterBlock_Object(Asura_Entity* pxEntity)
:Tools_Entity_Activatable_Object(pxEntity)
{
	Asura_Assert(pxEntity->GetClassification()==AsuraEntityClass_SplitterBlock,"");
	Type = TOT_Entity_SplitterBlock;
	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_Splitter");

	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) Entity;
	m_pbActive = new bool[pxAsuraEntity->m_uNumberOfOutputs];
	for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfOutputs;i++)
	{
		m_pbActive[i] = pxAsuraEntity->m_pxOutputs[i].m_bActive ? true : false;
	}
	m_bToggledOutputDelay = false;
	m_uSplitterFlags = pxAsuraEntity->m_uSplitterFlags;
}

Tools_Entity_SplitterBlock_Object::~Tools_Entity_SplitterBlock_Object()
{
	delete [] m_pbActive;
}

void Tools_Entity_SplitterBlock_Object::AddOverlayControls(class Asura_Overlay* pxNewOverlay,class Asura_Vector_2& xOffset)
{
	Tools_Entity_Activatable_Object::AddOverlayControls(pxNewOverlay,xOffset);
	Asura_Dialog* pxNewDialog;

	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) Entity;

	pxNewDialog = new Asura_FlagTickbox_Dialog("Random",xOffset,(int*)&m_uSplitterFlags,Asura_SplitterBlockFlag_Random,&Changed);
	pxNewDialog->SetHelpText("If on , then the message from a single active output (chosen at random) are sent when this is triggered.\nOtherwise the messages for all active outputs are sent.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_FlagTickbox_Dialog("Cycle Outputs",xOffset,(int*)&m_uSplitterFlags,Asura_SplitterBlockFlag_CycleOutputs,&Changed);
	pxNewDialog->SetHelpText("Causes one output to be used each time the splitter block is triggered. The outputs are used in order , skipping any that are inactive.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxNewDialog = new Asura_FlagTickbox_Dialog("Deactivate Outputs When Used",xOffset,(int*)&m_uSplitterFlags,Asura_SplitterBlockFlag_DeactivateOutputWhenUsed,&Changed);
 	pxNewDialog->SetHelpText("Each output is deactivated as it is used.\nUseful for random triggers so that each event happens once.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_FlagTickbox_Dialog("Use Random Delay", xOffset, (int*)&m_uSplitterFlags, Asura_SplitterBlockFlag_UseRandomDelay, &m_bToggledOutputDelay);
	pxNewDialog->SetHelpText("If on then the message is delayed by a random amount of time before being sent.\nUseful to desynchronise behaviour when triggering AI.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialog = new Asura_FlagTickbox_Dialog("Send Random Delay Immediately", xOffset, (int*)&m_uSplitterFlags, Asura_SplitterBlockFlag_SendRandomDelayImmediately,&Changed);
	pxNewDialog->SetIsEnabledFunction(IsRandomDelay);
	pxNewDialog->SetHelpText("This will cause any random delay messages to be sent immediately into the message system , but they won't be recieved until the delay is over.\nThe main difference here is that the messages are no longer handled by the splitter block at this point ,so the output can be triggered a second time if desired before the previous messages have finished.\nIt will also mean that outputs that deactivate upon use will be deactivated immediately , rather than when the delay has finished.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	if (pxAsuraEntity->m_uSplitterFlags & Asura_SplitterBlockFlag_UseRandomDelay)
	{
		pxNewDialog = new Asura_Numerical_Dialog("Min Delay", xOffset, &pxAsuraEntity->m_fOutputMinDelayTime, 0.0f, 9999.9f, 0.1f, "%.1f", &Changed);
		pxNewDialog->SetHelpText("Set the minimum delay time for use with random delays");
		pxNewOverlay->AddDialog(pxNewDialog);
		xOffset.x += 20.0f;

		pxNewDialog = new Asura_Numerical_Dialog("Max Delay", xOffset, &pxAsuraEntity->m_fOutputMaxDelayTime, 0.0f, 9999.9f, 0.1f, "%.1f", &Changed);
		pxNewDialog->SetHelpText("Set the maximum delay time for use with random delays");
		pxNewOverlay->AddDialog(pxNewDialog);
		xOffset.x -= 20.0f;
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	xOffset.y += 1.0f;

	pxNewDialog = new Asura_Function_Dialog("Reorder Outputs",xOffset,ReorderOutputs);
	pxNewDialog->SetIsEnabledFunction(CanReorderOutputs);
	pxNewDialog->SetHelpText("Displays an overlay for reordering the outputs in this splitter block.\nThis will also update any static messages that target this.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxNewDialog = new Asura_Function_Dialog("Add Output",xOffset,AddOutput);
 	pxNewDialog->SetHelpText("Add a new output to this trigger.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.x += 14.0f;

	pxNewDialog = new Asura_Function_Dialog("Delete Output",xOffset,DeleteOutput);
 	pxNewDialog->SetHelpText("Deletes the last output on this trigger.");
	pxNewOverlay->AddDialog(pxNewDialog);
	xOffset.x -= 14.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();


	//add some headings for the checkboxes/number controls
	float fCol0 = 1.0f;//index
	float fCol1 = fCol0+2.0f;//active
	float fCol2 = fCol1+3.0f;//weight
	float fCol3 = fCol2+10.0f;//messages
	float fCol4 = fCol3+5.0f;//name

	xOffset.x = fCol1;
	pxNewDialog = new Asura_Dialog("Active",xOffset);
	pxNewOverlay->AddDialog(pxNewDialog);

	xOffset.x = fCol2+2.0f;
	pxNewDialog = new Asura_Dialog("Weight",xOffset);
	pxNewOverlay->AddDialog(pxNewDialog);

	xOffset.x = fCol3;
	pxNewDialog = new Asura_Dialog("Msgs",xOffset);
	pxNewOverlay->AddDialog(pxNewDialog);

	xOffset.x = fCol4;
	pxNewDialog = new Asura_Dialog("Name",xOffset);
	pxNewOverlay->AddDialog(pxNewDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();


	char szText[100];
	for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfOutputs;i++)
	{
		Asura_SplitterBlockOutput* pxOutput = &pxAsuraEntity->m_pxOutputs[i];

		//add row heading
		sprintf(szText,"%d:",i);
		xOffset.x = fCol0;
		pxNewDialog = new Asura_Dialog(szText,xOffset);
		pxNewOverlay->AddDialog(pxNewDialog);

		xOffset.x = fCol1;
		pxNewDialog = new Asura_Tickbox_Dialog("",xOffset,&m_pbActive[i],&Changed);
	 	pxNewDialog->SetHelpText("This output starts active. An inactive output can't be triggered until activated by having an activate message sent to it.");
		pxNewOverlay->AddDialog(pxNewDialog);
		
		xOffset.x = fCol2;
		pxNewDialog = new Asura_Numerical_Dialog("",xOffset,(int*)&pxOutput->m_uWeight,1,1000,1,"%d",&Changed);
	 	pxNewDialog->SetHelpText("Relative chance for this output to be chosen if this is a random trigger.");
		pxNewOverlay->AddDialog(pxNewDialog);

		xOffset.x = fCol3;
		sprintf(szText,"%d",Asura_StaticMessage_System::GetNumberOfMessagesInBlock(pxOutput->m_usMessageBlock));
		pxNewDialog = new Asura_Function_Dialog(szText,xOffset,EditMessages,(int)i);
	 	pxNewDialog->SetHelpText("Edit the messages to be sent when this output is used.");
		pxNewOverlay->AddDialog(pxNewDialog);

		xOffset.x = fCol4;
		if(pxOutput->m_xName.IsBlank())
		{
			strcpy(szText,"*Not Set*");
		}
		else
		{
			strcpy(szText,pxOutput->m_xName.GetString());
		}
		pxNewDialog = new Asura_Function_Dialog(szText,xOffset,EditOutputName,(int)i);
	 	pxNewDialog->SetHelpText("Edits the (optional) name for this output.");
		pxNewOverlay->AddDialog(pxNewDialog);
		
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	
}

void Tools_Entity_SplitterBlock_Object::Create()
{
	Tools_Object* pxObject = Create(false);
	Tools_Object_System::SelectObject(pxObject);
}

Tools_Entity_SplitterBlock_Object* Tools_Entity_SplitterBlock_Object::Create(bool bGenerateName)
{
	//get a name , and the position data
	Asura_Guid uGuid = Asura_Entity_System::GetNextToolsCreatedGuid();
	Tools_Entity_Data* pxToolsData = 0;
	if(bGenerateName)
	{
		pxToolsData = Tools_Entity_Object::CreateToolsDataWithDefaultName("Splitter Block","Splitter Block",uGuid,GetNamePrefix());
	}
	else
	{
		pxToolsData = Tools_Entity_Object::CreateToolsData("Splitter Block",0,GetNamePrefix());
	}
	if(!pxToolsData) return 0 ;

	//create the asura data
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = new Asura_ServerEntity_SplitterBlock();
	pxAsuraEntity->ToolsData = pxToolsData;

	//create the tools pxObject
	Tools_Entity_SplitterBlock_Object* pxObject = new Tools_Entity_SplitterBlock_Object(pxAsuraEntity);

	Tools_SaveAsura::SetEntityModified();

	return pxObject;
}



void Tools_Entity_SplitterBlock_Object::Destroy()
{
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) Entity;
	//clear out any message blocks associated with this
	for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfOutputs;i++)
	{
		Asura_StaticMessage_System::DeleteMessageBlock(pxAsuraEntity->m_pxOutputs[i].m_usMessageBlock);
		pxAsuraEntity->m_pxOutputs[i].m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
	}

	Tools_Entity_Activatable_Object::Destroy();
}

void Tools_Entity_SplitterBlock_Object::UpdateWhileSelected()
{
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) Entity;

	if(m_bToggledOutputDelay || Changed)
	{
		//only one of random and cycle outputs allowed
		if((m_uSplitterFlags & Asura_SplitterBlockFlag_Random) &&
			(m_uSplitterFlags & Asura_SplitterBlockFlag_CycleOutputs))
		{
			if(pxAsuraEntity->m_uSplitterFlags & Asura_SplitterBlockFlag_Random)
			{
				//cycle outputs was just clicked
				m_uSplitterFlags &=~ Asura_SplitterBlockFlag_Random;
			}
			else
			{
				//random was just clicked
				m_uSplitterFlags &=~ Asura_SplitterBlockFlag_CycleOutputs;
			}
		}
		pxAsuraEntity->m_uSplitterFlags = m_uSplitterFlags;
	}
	
	if (m_bToggledOutputDelay || pxAsuraEntity->NeedsTimerArray() != pxAsuraEntity->HasTimerArray())
	{
		UpdateOutputTimerArray();
		m_bToggledOutputDelay = false;
		Changed = true;
		DeleteOverlay();
		CreateOverlay();
	}
	if(Changed)
	{
		//update the asura data
		for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfOutputs;i++)
		{
	   		pxAsuraEntity->m_pxOutputs[i].m_bActive = m_pbActive[i];
		}

		#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
		Tools_MessageEditor::UpdateMessageEditorTable();
		#endif
	}

	Tools_Entity_Activatable_Object::UpdateWhileSelected();
}

bool Tools_Entity_SplitterBlock_Object::HandlesThisMessage(int iMessageID)
{
	if(iMessageID == ASURA_MSG_SPLITTER_ACTIVATEOUTPUT ||
	   iMessageID == ASURA_MSG_SPLITTER_DEACTIVATEOUTPUT ||
	   iMessageID == ASURA_MSG_SPLITTER_TRIGGEROUTPUT)
	{
		return true;
	}
	return Tools_Entity_Activatable_Object::HandlesThisMessage(iMessageID);
}

void Tools_Entity_SplitterBlock_Object::AddOutputBlock()
{
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) this->Entity;

	//allocate an enlarged array of outputs , and copy over the old values
	Asura_SplitterBlockOutput* pxOutputs = new 	Asura_SplitterBlockOutput[pxAsuraEntity->m_uNumberOfOutputs+1];
	for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfOutputs;i++)
	{
		pxOutputs[i] = pxAsuraEntity->m_pxOutputs[i];
	}
	pxOutputs[pxAsuraEntity->m_uNumberOfOutputs].SetDefaultValues();

	delete [] pxAsuraEntity->m_pxOutputs;
	pxAsuraEntity->m_pxOutputs = pxOutputs;
	pxAsuraEntity->m_uNumberOfOutputs++;

	//update active output array
	delete [] m_pbActive;
	m_pbActive = new bool[pxAsuraEntity->m_uNumberOfOutputs];
	for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfOutputs;i++)
	{
		m_pbActive[i] = pxAsuraEntity->m_pxOutputs[i].m_bActive ? true : false;
	}

	UpdateOutputTimerArray();
}

void Tools_Entity_SplitterBlock_Object::AddOutput()
{
	Asura_Assert(ThisEntity,"");
	Tools_Entity_SplitterBlock_Object* pxToolsEntity = static_cast<Tools_Entity_SplitterBlock_Object*> (ThisEntity);

	pxToolsEntity->AddOutputBlock();

	//update the overlay
	ThisEntity->DeleteOverlay();
	ThisEntity->CreateOverlay();

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::UpdateMessageEditorTable();
	#endif
}

void Tools_Entity_SplitterBlock_Object::DeleteOutputBlock(int iIndex)
{
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) Entity;

	u_int uNumOutputs = pxAsuraEntity->m_uNumberOfOutputs;

	Asura_SplitterBlockOutput* pxOutput = &pxAsuraEntity->m_pxOutputs[iIndex];
	//delete the message block
	if(pxOutput->m_usMessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK)
	{
		Asura_StaticMessage_System::DeleteMessageBlock(pxOutput->m_usMessageBlock);
		pxOutput->m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
	}

	//shuffle the other outputs down
	for(u_int i=iIndex; i<uNumOutputs-1; i++)
	{
		pxAsuraEntity->m_pxOutputs[i] = pxAsuraEntity->m_pxOutputs[i+1];
	}

	pxAsuraEntity->m_uNumberOfOutputs--;
	if(pxAsuraEntity->m_uNumberOfOutputs == 0)
	{
		delete [] pxAsuraEntity->m_pxOutputs;
		pxAsuraEntity->m_pxOutputs = 0;
	}

	int iNumUnfixedMessages = 0;

	//adjust any splitter block messages that target this entity
	for(Tools_StaticMessageIterator xMessageIt;!xMessageIt.Done();xMessageIt.Next())
	{
		Asura_StaticMessage* pxMessage = xMessageIt.GetCurrent();
		//message targets this?
		if(pxMessage->m_uSendTo != pxAsuraEntity->GetGuid()) continue;
		//is it a splitter block output affecting message?
		if(pxMessage->m_usMsgID == ASURA_MSG_SPLITTER_ACTIVATEOUTPUT ||
			pxMessage->m_usMsgID == ASURA_MSG_SPLITTER_DEACTIVATEOUTPUT ||
			pxMessage->m_usMsgID == ASURA_MSG_SPLITTER_TRIGGEROUTPUT)
		{
			Asura_Message_SplitterBlock_Output* pxData = (Asura_Message_SplitterBlock_Output*) pxMessage->m_pData;

			//update the targeted output , and set as modified
			if(pxData->m_uOutputIndex<uNumOutputs && pxData->m_uOutputIndex> (u_int)iIndex)
			{
				pxData->m_uOutputIndex--;
				xMessageIt.SetModified();
			}
			else if(pxData->m_uOutputIndex == (u_int)iIndex)
			{
				//message deals with the deleted output , so attempt to delete the message
				if(!xMessageIt.DeleteCurrent())
				{
					//can't delete the message ,so the best that can be done is probably to leave as an invalid index
					pxData->m_uOutputIndex = 10000;
					iNumUnfixedMessages++;
				}
			}
		}
	}
	
	UpdateOutputTimerArray();	

	if(iNumUnfixedMessages)
	{
		ShowMessageOverlay("Failed to remove %d activate/deativate output message(s) that were referring to the deleted output.\nThese have been changed to refer to output 10000 to prevent them from doing anything",iNumUnfixedMessages);
	}

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::UpdateMessageEditorTable();
	#endif
}

void Tools_Entity_SplitterBlock_Object::DeleteOutput()
{
	Asura_Assert(ThisEntity,"");
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) ThisEntity->Entity;

	if(!pxAsuraEntity->m_uNumberOfOutputs) return;

	//ask for the output to delete
	ListSelection_Overlay xOverlay("Delete which output?");
	xOverlay.SetSortList(false);

	u_int uNumOutputs = pxAsuraEntity->m_uNumberOfOutputs;

	for(u_int i=0; i<uNumOutputs; i++)
	{
		Asura_SplitterBlockOutput* pxOutput = &pxAsuraEntity->m_pxOutputs[i];
	
		char szName[100];
		Asura_PrintfToBfr(szName,"%d : %s",100,true,i,pxOutput->m_xName.IsBlank() ? "*Unnamed*" : pxOutput->m_xName.GetString());

		xOverlay.AddItem(szName, i);
	}
	if(!xOverlay.DoModal()) return;

	
	int iIndex = xOverlay.GetSelectionData();

	Tools_Entity_SplitterBlock_Object* pxToolsEntity = static_cast<Tools_Entity_SplitterBlock_Object*> (ThisEntity);
	pxToolsEntity->DeleteOutputBlock(iIndex);

	ThisEntity->DeleteOverlay();
	ThisEntity->CreateOverlay();
}

void Tools_Entity_SplitterBlock_Object::UpdateOutputTimerArray()
{
	Asura_Assert(ThisEntity,"");
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) ThisEntity->Entity;

	pxAsuraEntity->UpdateOutputTimerArray();	
}

void Tools_Entity_SplitterBlock_Object::EditMessages(int iIndex)
{
	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::StartMessageEditingMode(ThisEntity->m_pxServerEntity->GetCustomEvent(iIndex));
	#else

	Asura_Assert(ThisEntity,"");
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) ThisEntity->Entity;
	Asura_Assert(iIndex>=0 && iIndex<(int)pxAsuraEntity->m_uNumberOfOutputs,"");

	char szText[100];
	sprintf(szText,"Output %d",iIndex);

	Tools_Object_System::EditMessageBlock(szText,&pxAsuraEntity->m_pxOutputs[iIndex].m_usMessageBlock);

	#endif
}

void Tools_Entity_SplitterBlock_Object::EditOutputName(int iIndex)
{
	Asura_Assert(ThisEntity,"");
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) ThisEntity->Entity;
	Asura_Assert(iIndex>=0 && iIndex<(int)pxAsuraEntity->m_uNumberOfOutputs,"");

	GetString_Overlay xOverlay("Edit output name",pxAsuraEntity->m_pxOutputs[iIndex].m_xName.GetString());

	if(xOverlay.DoModal())
	{
		//get the new name
		pxAsuraEntity->m_pxOutputs[iIndex].m_xName = xOverlay.GetString();

		//and update the overlay
	   	ThisEntity->DeleteOverlay();
		ThisEntity->CreateOverlay();

		#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
		Tools_MessageEditor::UpdateMessageEditorTable();
		#endif
	}
}

bool Tools_Entity_SplitterBlock_Object::IsRandomDelay()
{
	if(!ThisEntity) return false;
	Tools_Entity_SplitterBlock_Object* pxAsuraEntity = (Tools_Entity_SplitterBlock_Object*) ThisEntity;

	return (pxAsuraEntity->m_uSplitterFlags & Asura_SplitterBlockFlag_UseRandomDelay) ? true : false;
}


bool Tools_Entity_SplitterBlock_Object::CanReorderOutputs()
{
	if(!ThisEntity) return false;
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) ThisEntity->Entity;

	if(pxAsuraEntity->m_uNumberOfOutputs<2) return false;

	return true;
}

void Tools_Entity_SplitterBlock_Object::ReorderOutputs()
{
	if(!CanReorderOutputs())return;
	
	Asura_Assert(ThisEntity,"");
	Tools_Entity_SplitterBlock_Object* pxObject = (Tools_Entity_SplitterBlock_Object*) ThisEntity;
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) ThisEntity->Entity;

	int iNumOutputs = (int)pxAsuraEntity->m_uNumberOfOutputs;
	
	//build a list of output names for the reordering overlay
	const char** pszNameList = new const char*[iNumOutputs];
	for(int i=0;i<iNumOutputs;i++)
	{
		Asura_SplitterBlockOutput* pxOutput = &pxAsuraEntity->m_pxOutputs[i];
	
		char acName[100];
		Asura_PrintfToBfr(acName,"%d : %s",100,true,i,pxOutput->m_xName.IsBlank() ? "*Unnamed*" : pxOutput->m_xName.GetString());

		char* szName = new char[strlen(acName)+1];
		strcpy(szName, acName);
		pszNameList[i] = szName;
	}

	//show the reordering overlay
	ListOrdering_Overlay xOverlay("Reorder outputs",iNumOutputs,pszNameList);
	if(xOverlay.DoModal())
	{
		//check the name list to determine the new ordering
		int *piOrder = new int[iNumOutputs];

		for(int i=0;i<iNumOutputs;i++)
		{
			int iIndex;
			sscanf(pszNameList[i],"%d",&iIndex);

			piOrder[iIndex] = i;
		}

		//reorder outputs in tools object
		bool* pbActive = new bool[iNumOutputs];
		for(int i=0;i<iNumOutputs;i++)
		{
			pbActive[piOrder[i]] = pxObject->m_pbActive[i];
		}
		delete [] pxObject->m_pbActive;
		pxObject->m_pbActive = pbActive;

		//reorder outputs in asura entity
		Asura_SplitterBlockOutput* pxOutputs = new Asura_SplitterBlockOutput[iNumOutputs];
		float* pfOutputTimers = pxAsuraEntity->m_pfOutputTimers ? new float[iNumOutputs] : 0;
		
		for(int i=0;i<iNumOutputs;i++)
		{
			pxOutputs[piOrder[i]] = pxAsuraEntity->m_pxOutputs[i];
			if(pfOutputTimers) pfOutputTimers[piOrder[i]] = pxAsuraEntity->m_pfOutputTimers[i];
		}

		delete [] pxAsuraEntity->m_pxOutputs;
		pxAsuraEntity->m_pxOutputs = pxOutputs;

		delete [] pxAsuraEntity->m_pfOutputTimers;
		pxAsuraEntity->m_pfOutputTimers = pfOutputTimers;

		//adjust any splitter block messages that target this entity
		for(Tools_StaticMessageIterator xMessageIt;!xMessageIt.Done();xMessageIt.Next())
		{
			Asura_StaticMessage* pxMessage = xMessageIt.GetCurrent();
			//message targets this?
			if(pxMessage->m_uSendTo != pxAsuraEntity->GetGuid()) continue;
			//is it a splitter block output affecting message?
			if(pxMessage->m_usMsgID == ASURA_MSG_SPLITTER_ACTIVATEOUTPUT ||
				pxMessage->m_usMsgID == ASURA_MSG_SPLITTER_DEACTIVATEOUTPUT ||
				pxMessage->m_usMsgID == ASURA_MSG_SPLITTER_TRIGGEROUTPUT)
			{
				Asura_Message_SplitterBlock_Output* pxData = (Asura_Message_SplitterBlock_Output*) pxMessage->m_pData;

				//update the targeted output , and set as modified
				if(pxData->m_uOutputIndex<(u_int)iNumOutputs)
				{
					pxData->m_uOutputIndex = piOrder[pxData->m_uOutputIndex];
					xMessageIt.SetModified();
				}
			}
		}

		delete [] piOrder;

		#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
		Tools_MessageEditor::UpdateMessageEditorTable();
		#endif

		//force overlay to be updated
		ThisEntity->DeleteOverlay();
		ThisEntity->CreateOverlay();

		ThisEntity->SetEntityModified();
		
	}

	for(int i=0;i<iNumOutputs;i++)
	{
		delete [] pszNameList[i];
	}
	delete [] pszNameList;
	
}


void Tools_Entity_SplitterBlock_Object::PrintInfo(float& x,float& y)
{
	Tools_Entity_Activatable_Object::PrintInfo(x,y);
	if(Tools_Entity_Simulation::IsActive())
	{
		Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) Entity;

		//display state of outputs
		for(u_int i=0;i<pxAsuraEntity->m_uNumberOfOutputs;i++)
		{
			char szText[100];
			sprintf(szText,"Output %d : %s",i,pxAsuraEntity->m_pxOutputs[i].m_bActive ? "Active" : "Inactive");
			if(pxAsuraEntity->m_pfOutputTimers && pxAsuraEntity->m_pfOutputTimers[i] > 0.0f)
			{
				char szNum[20];
				sprintf(szNum,"  %.3f",pxAsuraEntity->m_pfOutputTimers[i]);
				strcat(szText,szNum);
			}

			Asura_Text_Output::Print(x,y++,0xffffff00,0,"%s",szText);
		}
	}
}

bool Tools_Entity_SplitterBlock_Object::OwnsThisMessageBlock(unsigned short usBlockID)
{
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) Entity;
	for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfOutputs;i++)
	{
		if(pxAsuraEntity->m_pxOutputs[i].m_usMessageBlock == usBlockID) return true;
	}

	return Tools_Entity_Activatable_Object::OwnsThisMessageBlock(usBlockID);
}

bool Tools_Entity_SplitterBlock_Object::GetEventDescForMessageBlock(unsigned short usBlockID,Asura_String& xOutputDesc)
{
	Asura_ServerEntity_SplitterBlock* pxAsuraEntity = (Asura_ServerEntity_SplitterBlock*) Entity;
	for(unsigned i=0;i<pxAsuraEntity->m_uNumberOfOutputs;i++)
	{
		Asura_SplitterBlockOutput* pxOutput = &pxAsuraEntity->m_pxOutputs[i];
		
		if(pxOutput->m_usMessageBlock == usBlockID)
		{
			char szTemp[100];
			if(pxOutput->m_xName.IsBlank())
				Asura_PrintfToBfr(szTemp,"Trigger Output %d",sizeof(szTemp),true,i);
			else
				Asura_PrintfToBfr(szTemp,"Trigger Output '%s'",sizeof(szTemp),true,pxOutput->m_xName.GetString());
			xOutputDesc = szTemp;
			return true;
		}
	}
	return false;
}


Tools_Object* Tools_Entity_SplitterBlock_Object::CreateObjectOfThisType() const
{
	return Create(true);	
}

void Tools_Entity_SplitterBlock_Object::Copy(const Tools_Object* pxObject)
{
	Tools_Entity_Activatable_Object::Copy(pxObject);

	const Tools_Entity_SplitterBlock_Object* pxFromObject = (const Tools_Entity_SplitterBlock_Object*) pxObject;

	Asura_ServerEntity_SplitterBlock* pxFromEntity = (Asura_ServerEntity_SplitterBlock*) pxFromObject->Entity;
	Asura_ServerEntity_SplitterBlock* pxToEntity = (Asura_ServerEntity_SplitterBlock*) Entity;

	//destroy the old outputs
	for(u_int i=0;i<pxToEntity->m_uNumberOfOutputs;i++)
	{
		Asura_StaticMessage_System::DeleteMessageBlock(pxToEntity->m_pxOutputs[i].m_usMessageBlock);
	}
	delete [] pxToEntity->m_pxOutputs;
	pxToEntity->m_pxOutputs = 0;
	delete [] m_pbActive;
	m_pbActive = 0;
	
	
	//copy between the entities
	pxToEntity->m_uSplitterFlags = pxFromEntity->m_uSplitterFlags;
	pxToEntity->m_uNumberOfOutputs = pxFromEntity->m_uNumberOfOutputs;
	pxToEntity->m_fOutputMaxDelayTime = pxFromEntity->m_fOutputMaxDelayTime;
	pxToEntity->m_fOutputMinDelayTime = pxFromEntity->m_fOutputMinDelayTime;

	if(pxToEntity->m_uNumberOfOutputs)
	{
		pxToEntity->m_pxOutputs = new Asura_SplitterBlockOutput[pxToEntity->m_uNumberOfOutputs];
		for(u_int i=0;i<pxToEntity->m_uNumberOfOutputs;i++)
		{
			pxToEntity->m_pxOutputs[i].m_usFlags = pxFromEntity->m_pxOutputs[i].m_usFlags;
			pxToEntity->m_pxOutputs[i].m_uWeight = pxFromEntity->m_pxOutputs[i].m_uWeight;
			pxToEntity->m_pxOutputs[i].m_xName = pxFromEntity->m_pxOutputs[i].m_xName;
			pxToEntity->m_pxOutputs[i].m_usMessageBlock = Asura_StaticMessage_System::CopyMessageBlock(pxFromEntity->m_pxOutputs[i].m_usMessageBlock,pxFromObject->GetGuid(),GetGuid());
		}
	}

	//copy between the tools objects
	m_uSplitterFlags = pxToEntity->m_uSplitterFlags;
	m_pbActive = new bool[pxToEntity->m_uNumberOfOutputs];
	for(u_int i=0;i<pxToEntity->m_uNumberOfOutputs;i++)
	{
		m_pbActive[i] = pxFromObject->m_pbActive[i];
	}
	pxToEntity->UpdateOutputTimerArray();

}
