
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Entity_TeleportDestination.cpp
// Author : Andy Tate
// Date   : 2-Mar-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Entity/Tools_Entity_TeleportDestination.h"

// Asura independent header-files...
#include "Camera/Asura_Camera.h"
#include "Entity/Spawners/Asura_ServerEntity_TeleportDestination.h"
#include "Mesh/Asura_Mesh_System.h"

// Tools header-files...
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Entity_TeleportDestination::Tools_Entity_TeleportDestination
// Author       : Andy Tate
// Date         : 2-Mar-2007
////////////////////////////////////////////////////////////////////////////////

Tools_Entity_TeleportDestination::Tools_Entity_TeleportDestination(Asura_Entity* pxEntity)
: PARENT(pxEntity)
{
	Type	= TOT_Entity_TeleportDestination;
	Mesh	= Asura_Mesh_System::GetMesh("ToolsShape_TeleportExit");
}

/// End of function 'Tools_Entity_TeleportDestination::Tools_Entity_TeleportDestination'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Entity_TeleportDestination::Create
// Author       : Andy Tate
// Date         : 2-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Entity_TeleportDestination::Create(void)
{
	CreateMain();
}

/// End of function 'Tools_Entity_TeleportDestination::Create'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Entity_TeleportDestination::CreateMain
// Author       : Richard Rice
// Date         : 9-Feb-2009
////////////////////////////////////////////////////////////////////////////////

Tools_Object* Tools_Entity_TeleportDestination::CreateMain(void)
{
	const Asura_Guid	uGuid	= Asura_Entity_System::GetCreatedByToolsGuid();

	// Set up tools data and auto-create a unique name (prompt for new name if there is a clash).
	Tools_Entity_Data* pxToolsData = CreateToolsDataWithDefaultName("Teleport Destination", "Teleport Destination", uGuid, GetNamePrefix());

	if(pxToolsData == NULL)
	{
		Asura_Assert(false, "Tools_Entity_TeleportDestination::Create() - no tools data");
	}
	else
	{
		Asura_ServerEntity_TeleportDestination*	pxServerObject	= new Asura_ServerEntity_TeleportDestination(uGuid);

		if(pxServerObject == NULL)
		{
			Asura_Assert(false, "Tools_Entity_TeleportDestination::Create() - no server object" );
		}
		else
		{
			pxServerObject->ToolsData = pxToolsData;

			Tools_Entity_TeleportDestination*	pxToolsObject	= new Tools_Entity_TeleportDestination(pxServerObject);

			// Create the game specific tools object and select it.
			if(pxToolsObject == NULL)
			{
				Asura_Assert(false, "Tools_Entity_TeleportDestination::Create() - no tools object");
			}
			else
			{
				Tools_Object_System::SelectObject(pxToolsObject);

				pxToolsObject->Position		= *Asura_Camera::GetPosition();
				pxToolsObject->Orientation	= *Asura_Camera::GetOrientation();

				pxServerObject->TeleportTo(pxToolsObject->Position, Asura_Quat(pxToolsObject->Orientation));

				Tools_Timed_Message_System::AddMessage(0xffffff, "Created new Teleport Destination");
				Tools_SaveAsura::SetEntityModified();
			}
			return pxToolsObject;
		}
	}
	return 0;
}

/// End of function 'Tools_Entity_TeleportDestination::CreateMain'. ///




////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Entity_TeleportDestination::UpdatePositionInAsuraData
// Author       : Andy Tate
// Date         : 2-Mar-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Entity_TeleportDestination::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	Asura_ServerEntity_TeleportDestination*	pxAsuraEntity	= static_cast<Asura_ServerEntity_TeleportDestination*>(Entity);
	Asura_Quat								xOrientation(Orientation);

	if((pxAsuraEntity->m_xPosition != Position) || (pxAsuraEntity->m_xOrientation != xOrientation))
	{
		pxAsuraEntity->m_xOrientation	= xOrientation;
		pxAsuraEntity->m_xPosition		= Position;

		Changed							= true;
	}

	PARENT::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}

/// End of function 'Tools_Entity_TeleportDestination::UpdatePositionInAsuraData'. ///

Tools_Object* Tools_Entity_TeleportDestination::CreateObjectOfThisType(void) const
{
	return CreateMain();	
}


/// End of file. ///
