
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Entity_Teleporter.h
// Author : Andy Tate
// Date   : 28-Feb-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_ENTITY_TELEPORTER_H__
#define __TOOLS_ENTITY_TELEPORTER_H__

#if !defined(ASURA_DONT_USE_TELEPORTERS)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Teleport/Asura_Teleport_System.h"

#include "Entity/Tools_Entity_Object.h"
#include "Objects/Tools_Object_System.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////
// Typedefs...
////////////////////////////////////////////////////////////////////////////////
typedef bool ProjectCanBeSelectedFunction(const Tools_Object* pxObject);


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////
struct Asura_StaticMessage;
////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_Entity_Teleporter
// Author       : Andy Tate
// Date         : 28-Feb-2007
//
// Description  : Tools class used to allow manipulation of teleporter entities.
////////////////////////////////////////////////////////////////////////////////

class Tools_Entity_Teleporter : public Tools_Entity_Object
{
public:
	// Public functions...
	Tools_Entity_Teleporter          (Asura_Entity* pxEntity);
//	virtual ~Tools_Entity_Teleporter ();

	void	SetDestination	(const int iIndex, Tools_Object* pxObject);

	static void	AddMessageOverlayControls_TeleportEntity	(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
	static void	UpdateTeleportEntityText					(const Asura_StaticMessage& xMessage, Asura_String& xString);

	// Overridden functions...
	virtual	const char*	GetTypeDescription	(void)	{ return("Teleporter"); }

	virtual const char* GetMyNamePrefix		() const { return Tools_Entity_Teleporter::GetNamePrefix(); }

	virtual bool		DealsInternallyWithActivate() const { return true; }
	virtual	void		AddOverlayControls	(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset);
	virtual	void		UpdateWhileSelected	(void);
	virtual	bool		HandlesThisMessage	(int iMessageID);
	virtual	void		PrintInfo			(float& fX, float& fY);
	virtual void		Render				(void);

	virtual void		OnSelection			(void);
	virtual void		OnDeselection		(void);

	virtual bool SupportsMessageEditor() { return false; }

	// Overridden copy functions...
	virtual	bool			CanBeCopied				(void) const	{ return(true); }
	virtual	void			Copy					(const Tools_Object* pxObject);
	virtual	Tools_Object*	CreateObjectOfThisType	(void) const;

	virtual bool GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const;
	virtual bool SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam);

	// Static functions...
	static void		Create	(void);

	static const char*	GetNamePrefix		()		{ return "Tele_"; }

protected:
	
    static void SetSelectedStaticMessage( Asura_StaticMessage* pxMsg );
    static void SetTeleportEntityDialog( Asura_Dialog* pxDialog );

	static bool	IsEnabled_PerformFade		(void);

	static void	SelectTeleportDestination	(int iIndex);
	static void SelectGuidToTeleport		(void);

	void		UpdateDialogueText			(const int iIndex);


private:
	//Copies changes from the tools object to the server entity
	void		UpdateAsuraEntity();

	// Private typedefs...
	typedef Tools_Entity_Object PARENT;

	// Private variables...
	bool							m_bActive;
	bool							m_bPerformFade;
	float							m_fRed;
	float							m_fGreen;
	float							m_fBlue;

	Asura_Dialog*					m_apxDestinationDialogues[ASURA_TELEPORTER_ENTITY_MAX_DESTINATIONS];

	static Asura_StaticMessage*		s_pxSelectedStaticMessage;
	static Tools_Entity_Teleporter*	s_pxSelectedTeleporter;

	// Private functions...
	Tools_Entity_Teleporter  (void);
	Tools_Entity_Teleporter  (const Tools_Entity_Teleporter& xOther);

	Tools_Entity_Teleporter&	operator=	(const Tools_Entity_Teleporter& xOther);
};   // End of class 'Tools_Entity_Teleporter'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_ChooseObject_TeleporterDestination
// Author       : Andy Tate
// Date         : 1-Mar-2007
//
// Description  : Class to handle the choosing of a destination entity for the
//				  teleporter class.
////////////////////////////////////////////////////////////////////////////////

class Tools_ChooseObject_TeleporterDestination : public Tools_ChooseObject
{
public:
	// Public functions...
	Tools_ChooseObject_TeleporterDestination          (const int iIndex, Tools_Entity_Teleporter* pxTeleporter)	: m_iIndex(iIndex), m_pxTeleporter(pxTeleporter)	{}
//	virtual ~Tools_ChooseObject_TeleporterDestination ();

	// Overridden functions...
//	virtual	void	OnStart			(void);
	virtual	bool	CanBeSelected	(Tools_Object* pxObject);
	virtual	void	ObjectSelected	(Tools_Object* pxObject);
	virtual	void	OnFinish		(void) {delete this;}


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...
	int							m_iIndex;			// The index of the teleporter destination we're using.
	Tools_Entity_Teleporter*	m_pxTeleporter;		// A pointer to the Tools teleporter-entity which we're choosing our object for.

	// Private functions...
	Tools_ChooseObject_TeleporterDestination  (void);
	Tools_ChooseObject_TeleporterDestination  (const Tools_ChooseObject_TeleporterDestination& xOther);

	Tools_ChooseObject_TeleporterDestination&	operator=	(const Tools_ChooseObject_TeleporterDestination& xOther);

};   // End of class 'Tools_ChooseObject_TeleporterDestination'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_ChooseObject_EntityToTeleport
// Author       : Andy Tate
// Date         : 2-Mar-2007
//
// Description  : Class to handle the selection of the entity to be teleported.
////////////////////////////////////////////////////////////////////////////////

class Tools_ChooseObject_EntityToTeleport : public Tools_ChooseObject
{
public:
	// Public functions...
	Tools_ChooseObject_EntityToTeleport          (Asura_StaticMessage* pxSelectedStaticMessage)	: m_pxStaticMessage(pxSelectedStaticMessage)	{}
//	virtual ~Tools_ChooseObject_EntityToTeleport ();

	// Overridden functions...
//	virtual	void	OnStart			(void);
	virtual	bool	CanBeSelected	(Tools_Object* pxObject);
	virtual	void	ObjectSelected	(Tools_Object* pxObject);
	virtual	void	OnFinish		(void) {delete this;}

	static void SetProjectCanBeSelectedFunction(ProjectCanBeSelectedFunction* const pfnProjectCanBeSelected) { s_pfnProjectCanBeSelected = pfnProjectCanBeSelected; }

protected:
	// Protected variables...

	// Protected functions...


private:
	// Private variables...
	Asura_StaticMessage*	m_pxStaticMessage;

	// Private functions...
	Tools_ChooseObject_EntityToTeleport  (void);
	Tools_ChooseObject_EntityToTeleport  (const Tools_ChooseObject_EntityToTeleport& xOther);

	Tools_ChooseObject_EntityToTeleport&	operator=	(const Tools_ChooseObject_EntityToTeleport& xOther);

	static ProjectCanBeSelectedFunction* s_pfnProjectCanBeSelected;

};   // End of class 'Tools_ChooseObject_EntityToTeleport'.


#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

#endif   // ifndef __TOOLS_ENTITY_TELEPORTER_H__


/// End of file. ///
