//
// Tools_Entity_Trigger_Object.h
//
// Created by Richard Rice - 01/05/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_ENTITY_TRIGGER_OBJECT_H_
#define _TOOLS_ENTITY_TRIGGER_OBJECT_H_

#include "Tools_Entity_Activatable_Object.h"

//////////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------**
** Tools_Entity_Trigger_Object **
**-----------------------------*/
class Tools_Entity_Trigger_Object : public Tools_Entity_Activatable_Object
{
	public :
		Tools_Entity_Trigger_Object(Asura_Entity* entity);
		
		//virtual functions
		//NB: Because Asura triggers don't in fact handle this message, but it nevertheless
		//exists in the messageblock, I have added this function as a temporary measure (in
		//tools only), to avoid confusing artists with an option to edit a message which
		//in fact does nothing.
		virtual bool UsesUnTriggerMessage() { return false; }
		virtual bool UsesTriggerMessage() { return true; }
		virtual bool UsesActivationMessage(){return true;}
		virtual bool UsesDeactivationMessage(){return true;}
		virtual bool UsesDestructionMessage(){return true;}

		#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
		virtual const char* GetShortName(){return "TRI";}
		#endif

		//virtual function overides
		const char* GetTypeDescription(){return "Trigger";}
		void Destroy();

		virtual const char* GetMyNamePrefix() const { return Tools_Entity_Trigger_Object::GetNamePrefix(); }

		void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);

		virtual bool OwnsThisMessageBlock(unsigned short uMessageBlockID);
		virtual bool GetEventDescForMessageBlock(unsigned short usMessageBlockID,Asura_String& xOutputDesc);

		virtual void Copy(const Tools_Object* pxObject);

		virtual void UpdateClient();

		//Returns true for triggers that send messages on the client. (Most just exist on server)
		virtual bool IsClientSideTrigger(){return false;}


		static void EditMessages_OnActivation();
		static void EditMessages_OnDeactivation();
		static void EditMessages_OnBeingTriggered();
		static void EditMessages_OnDestruction();
		static void EditMessages_OnUnTrigger();

		static const char* GetNamePrefix() { return "Tri_"; }
};

//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_Entity_TimeTrigger_Object **
**---------------------------------*/
class Tools_Entity_TimeTrigger_Object : public Tools_Entity_Trigger_Object
{
	public :
		Tools_Entity_TimeTrigger_Object(Asura_Entity* entity);
		static void Project_Create(void);
		static Tools_Entity_TimeTrigger_Object* Create(bool bGenerateName);
		
		//virtual function overides
		const char* GetTypeDescription(){return "Time Trigger";}
		bool HandlesThisMessage(int iMessageID);
		
		virtual const char* GetMyNamePrefix() const { return Tools_Entity_TimeTrigger_Object::GetNamePrefix(); }

		void AddOverlayControls(class Asura_Overlay* newOverlay,class Asura_Vector_2& offset);

		virtual void UpdateWhileSelected(void);	
		virtual void PrintInfo(float& x,float& y);

		virtual void Copy(const Tools_Object* pxObject);
		virtual bool CanBeCopied() const {return Type == TOT_Entity_TimeTrigger;}
		virtual Tools_Object* CreateObjectOfThisType() const;

		static void AddOverlayControls_SetTimer(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
		static void AddOverlayControls_SetTriggerTime(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
		static void AddOverlayControls_AddToTriggerTime(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
		static void AddOverlayControls_AddTime(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);

		static const char* GetNamePrefix() { return "TT_"; }
};



//////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------**
** Tools_Entity_LookAtTrigger_Object **
**------------------------------------*/
class Tools_Entity_LookAtTrigger_Object : public Tools_Entity_Trigger_Object
{
public :
	Tools_Entity_LookAtTrigger_Object(Asura_Entity* pxEntity);
	static void Create();
	static Tools_Entity_LookAtTrigger_Object* Create(bool bGenerateName);
	static bool ShowCone() { return s_bShowCone; } 
	static bool WireframeCone() { return s_bWireframeCone; } 
	
	//virtual function overides
	const char* GetTypeDescription(){return "Look-at Trigger";}

	virtual const char* GetMyNamePrefix() const { return Tools_Entity_LookAtTrigger_Object::GetNamePrefix(); }

	void AddOverlayControls(class Asura_Overlay* pxNewOverlay,class Asura_Vector_2& xOffset);
	
	float GetAngle() const { return m_fInwardConeMaxAngle; }

	virtual void UpdateWhileSelected(void);	

	virtual void Copy(const Tools_Object* pxObject);
	virtual bool CanBeCopied() const {return Type == TOT_Entity_LookAtTrigger;}
	virtual Tools_Object* CreateObjectOfThisType() const;

	virtual bool UsesUnTriggerMessage() { return true; }

	static const char* GetNamePrefix() { return "LAT_"; }

protected:
	virtual void PrintInfo(float& x, float& y);
	virtual void UpdateClient( void );

private:
	bool IsEnabled_SpecifiedPlayer();

	float m_fMinAngle;
	float m_fMaxAngle;
	float m_fInwardConeMaxAngle;
	int m_iListOffset;
	int m_iListSelection;
	class Asura_Dialog* m_pxMinAngleDialog;
	class Asura_Dialog* m_pxMaxAngleDialog;
	class Asura_Dialog* m_pxEdgeDeadzoneDialog;
	class Asura_Dialog* m_pxCentreDeadzoneDialog;
	class Asura_Dialog* m_pxSpecifyPlayerDialog;

	bool m_bCameraPassesTests;

	static const char* s_aszNameList[4];
	static bool	s_bShowCone; // (AMM 7/11/08) Allow user to view a cone representing look-at angle
	static bool	s_bWireframeCone; // (AMM 7/11/08) Allow user to view look-at cone as wireframe

};

#ifdef ASURA_USE_DEBUG_MESSAGE_TRIGGER
//////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------------**
** Tools_Entity_DebugMessageTrigger_Object **
**------------------------------------------*/
class Tools_Entity_DebugMessageTrigger_Object : public Tools_Entity_Trigger_Object
{
public :
	Tools_Entity_DebugMessageTrigger_Object(Asura_Entity* pxEntity);
	static void Create();
	static Tools_Entity_DebugMessageTrigger_Object* Create(bool bGenerateName);

	//virtual function overides
	const char* GetTypeDescription(){return "DebugMessage_Trigger";}
	bool HandlesThisMessage(int iMessageID);

	virtual const char* GetMyNamePrefix() const { return Tools_Entity_DebugMessageTrigger_Object::GetNamePrefix(); }

	void AddOverlayControls(class Asura_Overlay* pxNewOverlay,class Asura_Vector_2& xOffset);

	virtual void UpdateWhileSelected(void);	
	virtual void PrintInfo(float& fX,float& fY);

	virtual void Copy(const Tools_Object* pxObject);
	virtual bool CanBeCopied() const {return Type == TOT_Entity_DebugMessageTrigger;}
	virtual Tools_Object* CreateObjectOfThisType() const;

	static const char* GetNamePrefix() { return "DMT_"; }

};
#endif //ASURA_USE_DEBUG_MESSAGE_TRIGGER

#endif //_TOOLS_ENTITY_TRIGGER_OBJECT_H_
