//////////////////////////////////////////////////////////////////////
// Tools_Entity_VolumeTrigger.h										//
// SJL 6/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_ENTITY_VOLUMETRIGGER_H_
#define _TOOLS_ENTITY_VOLUMETRIGGER_H_

#include "Entity/Triggers/Asura_ServerEntity_VolumeTrigger.h"
#include "Tools_Entity_Trigger_Object.h"
#include "Objects/Tools_VolumeBaseEdit_Object.h"
#include "Objects/Tools_VolumeGroupEdit_Controller.h"

class Tools_Entity_VolumeTrigger : public Tools_Entity_Trigger_Object
{
	public :

		Tools_Entity_VolumeTrigger(Asura_Entity* pxEntity);
		~Tools_Entity_VolumeTrigger();

		virtual bool UsesUnTriggerMessage() { return true; }
		//static void Project_Create();
	
		#ifndef TOOLS_USE_OLD_ENTITY_MANAGER
		virtual const char* GetShortName(){return "VT";}
		#endif

		//virtual function overrides
		virtual const char* GetTypeDescription(){return "Volume Trigger";}
		virtual void AddOverlayControls(class Asura_Overlay* pxNewOverlay,class Asura_Vector_2& xOffset);
		virtual void Copy(const Tools_Object*);
		virtual bool CanBeCopied() const {return Type == TOT_Entity_VolumeTrigger;}
		virtual Tools_Object* CreateObjectOfThisType() const;
		virtual void UpdateClient();

		virtual const char* GetMyNamePrefix() const { return Tools_Entity_VolumeTrigger::GetNamePrefix(); }

		void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);
		void Update();

		void SetParentObjectGuid (const Asura_Guid uGuid);

		static const char* GetNamePrefix() { return "VT_"; }

	protected:

		static void EditVolume();
		void ShowVolume(bool bShow);

		virtual void UpdateOverlayControls (class Asura_Overlay* pxOverlay);

		static void LinkToParent ();

		virtual bool AllowClientVolumes() const { return true; }

		static bool s_bMoveVolumeWithObject;
		static bool s_bShowAllVolumes;
		static bool s_bShowSelectedVolume;

	private :

		Asura_Vector_3 m_xPreviousPosition;
		Asura_Matrix_3x3 m_xPreviousOrientation;
		Tools_VolumeTrigger_VolumeEdit_Controller* m_pxVolumeController;
		static Asura_Function_Dialog* s_pxLinkToParentDialogButton;
};

class Tools_VolumeTrigger_VolumeEdit_Controller : public Tools_VolumeGroupEdit_Controller
{
public :

	Tools_VolumeTrigger_VolumeEdit_Controller(Tools_Entity_VolumeTrigger* pxToolsEntity);

	virtual void StartVolumeEditing();
	virtual void EndVolumeEditing();

protected :

	virtual bool CanCreateVolume()	{ return true; }
	virtual bool CanDeleteVolume()  { return GetNumberOfVolumes()>1; }

	virtual Asura_Volume* GetVolume(u_int uIndex) { return m_pxAsuraEntity->GetVolume(uIndex); }

	//Gets the current number of volumes
	virtual int GetNumberOfVolumes() { return m_pxAsuraEntity->GetNumberOfVolumes(); }

	//Gets a pointer to the orientation for a given volume. Returns 0 if there is none
	virtual Asura_Quat* GetVolumeOrient(int iIndex)	{ return m_pxAsuraEntity->GetVolumeOrient(static_cast<u_int>(iIndex)); }

	//Gets a default position for volume objects that have yet to gain a proper volume
	virtual Asura_Vector_3 GetDefaultPosition()	{ return m_pxToolsEntity->Position; }

	//Gets the name of the owner , to use for part of the volume names
	virtual const char* GetName() { return m_pxToolsEntity->GetName(); }

	//Creates the new volume in the asura data
	virtual void CreateVolumeCuboidMain();
	//Creates the new volume in the asura data
	virtual void CreateVolumeSphereMain();
	//Creates the new volume in the asura data
	virtual void CreateVolumeCylinderMain();

	//Deletes the volume from the asura data
	virtual void DeleteVolumeMain(int iIndex);

private :

	Tools_Entity_VolumeTrigger*	m_pxToolsEntity;
	Asura_ServerEntity_VolumeTrigger* m_pxAsuraEntity;

};



#endif //_TOOLS_ENTITY_VOLUMETRIGGER_H_
