////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//
// Tools_MessageEditor.h
//
// Created by Dan Clarke - 28/10/2008
// 
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_MESSAGEEDITOR_H_
#define _TOOLS_MESSAGEEDITOR_H_

#ifdef ASURA_TOOLS

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Table_Dialog.h"
#include "Objects/Tools_Object.h"
#include "Core/Asura_list.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Tools_Entity_Object.h"
#include "Entity/Tools_Message_Editing.h"
#include "Entity/Tools_ReceivedMessages.h"
#include "Interface/Asura_MessageTable_Dialog.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct Tools_MessageEditor_ClipboardItem
{
	Asura_StaticMessage xMessage;
	unsigned short usSourceBlockId;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Tools_MessageEditor_Clipboard
{
	public:
		Tools_MessageEditor_Clipboard()
		{
			m_pxMessages = NULL;
			m_uNumMessagesInClipboard = 0;
			m_uAllocatedArraySize = 0;
			m_eMode = MESSAGEEDITOR_CLIPBOARDMODE_COPY;
			m_eType = MESSAGEEDITOR_CLIPBOARDTYPE_OUTGOING;
		}

		~Tools_MessageEditor_Clipboard()
		{
			Clear();
		}

		enum ModeEnum
		{
			MESSAGEEDITOR_CLIPBOARDMODE_COPY,
			MESSAGEEDITOR_CLIPBOARDMODE_CUT,
		};

		enum TypeEnum
		{
			MESSAGEEDITOR_CLIPBOARDTYPE_OUTGOING,
			MESSAGEEDITOR_CLIPBOARDTYPE_INCOMING,
		};

		void Clear();
		void Add(Asura_StaticMessage* pxMessage, u_short usSourceBlockId);
		void Paste();
		u_int Count() { return m_uNumMessagesInClipboard; }
		Tools_MessageEditor_ClipboardItem* GetArray() { return m_pxMessages; }

		ModeEnum GetMode() const { return m_eMode; }
		void SetMode(ModeEnum eMode) { m_eMode = eMode; }

		TypeEnum GetType() const { return m_eType; }
		void SetType(TypeEnum eType) { m_eType = eType; }

	private:
		Tools_MessageEditor_ClipboardItem* m_pxMessages;
		u_int m_uNumMessagesInClipboard;
		u_int m_uAllocatedArraySize;
		ModeEnum m_eMode;
		TypeEnum m_eType;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Tools_MessageEditor_ReceivedMessageData
{
public :
	//message block that this comes from
	Tools_StaticMessageID m_xStaticMessageID;

	int m_iMessageID;

	//sender of message (if known)
	Asura_Guid m_uSender;
	Asura_String m_xSenderName;

	//description of message type
	Asura_String m_xMessageDesc;
	//condition for sending message (if known)
	Asura_String m_xMessageEvent;

	Asura_StaticMessage* m_pxStaticMessage;

	u_short m_uMessageBlockId;
	
	bool m_bDisabled;//disabled flag from message
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Tools_MessageEditor
{
	friend class Tools_MessageEditor_TableCell;
	friend class Tools_ChooseMessageTargetObject;
	friend class Tools_ChooseAddMessageTargets;
	friend class Tools_MessageEditor_Resize_Dialog;

	public:

		enum TabNamesEnum
		{
			TAB_OUTGOING_MESSAGES,
			TAB_INCOMING_MESSAGES,

			NUM_TABS,
		};

		static void ToggleOverlay();
		static void CreateOverlay();
		static void CloseOverlay();
		static void CloseMessageAttributeOverlay();
		static void Shutdown();
		static void Update();
		static void ProcessKeyboardShortcuts();
		static bool IsOpen() { return s_bIsOverlayVisible; }

		//Opens the message editor if needed , and attempts to select the object that owns the message block , 
		//and select the message
		static void OpenMessageEditorAtMessage(u_short usMessageID, int iMessageIndex);
		
		static void StartMessageEditingMode(u_short* pusMessageBlockToSelect = NULL);
		static void StopMessageEditingMode();
		static bool IsInMessageEditingMode() { return s_bInMessageEditingMode; }

		static void UpdateMessageEditorTable() { s_bForceRecreateTable = true; }

		static void HandleDeletedObject(Tools_Object* pxObject);

		static Tools_Object* GetReferenceObject() { return s_pxReferenceObject; }

		static void UpdateObjectOverlay();

		static u_int GetSelectedTabIndex() { return s_uSelectedTab; }

		static void CopyToClipboard();
		static void CutToClipboard();
		static void PasteFromClipboard();
		static void SendToClipboard(bool bCut);

		static Asura_StaticMessage* GetSelectedStaticMessage();

		static void CreateMessageAttributesOverlay(Asura_StaticMessage* pxMessage);

		static u_int GetNumberOfMessagesInEntity(Tools_BaseEntity_Object* pxObject);

		static void ResetOverlayToOpenPointer() { s_pxMessageOverlayToOpen = NULL; }

		//Flags the entity or cutscene file as modified depending on where the message block belongs
		static void SetModified(const u_short* pusBlockID);

		//Flags the appropriate file(s) as modified for a static message
		static void SetModified(const Asura_StaticMessage* pxMessage);

		//If a message is being edited by this system , then it flags the appproprate file as modified.
		static void SetEditedMessageModified();
		
		//Determines whether the block should belong to the entity or the cutscene file
		static Asura_StaticMessageSetIndex GetMessageSetFromBlockId(const u_short* pusBlockID);

		static Asura_Vector_2 s_xOverlayPosition;
		static Asura_Vector_2 s_xMessageAttributeOverlayPosition;

		static Asura_Vector_2 s_xDialogSize;

	private:
		enum ColumnNamesEnum
		{
			COLUMN_NAME_DELAY,
			COLUMN_NAME_MESSAGE,
			COLUMN_NAME_TARGET,
			COLUMN_NAME_PROBABILITY,
			COLUMN_NAME_DISABLE,

			NUM_COLUMN_NAMES,
		};

		static void SelectTab(TabNamesEnum eTab);

		static void CreateOutgoingMessagesTabPage(Asura_Vector_2 xPosition);
		static void CreateIncomingMessagesTabPage(Asura_Vector_2 xPosition);

		static void CreateOutgoingMessagesTable();
		static void CreateIncomingMessagesTable();

		static void SelectedRowChangeCallback(bool bSelected, class Asura_MessageTableRow* pxRow);

		static class Asura_MessageTableRow_Outgoing* CreateOutgoingMessageRow(u_short* pusMessageBlockId, int iMessageIndex);
		static void CreateOutgoingEventRow(int iEventId, const Asura_String xEventName, u_short* pusMessageBlockId, int iNumMessagesInBlock);

		static void RecreateTable();

		static void SetHideButtons(bool bFlag);

		static void SortMessages(unsigned short usMessageBlock);

		static void AddMessageCallback();
		static Asura_StaticMessage* AddMessage(u_short usMessageId, Asura_Guid xTargetGuid);
		static u_short* GetBlockIdFromRowSelection(bool bCreateBlockIfNeeded);
		static u_short* GetCachedBlockIdFromRowSelection();

		static void DeleteMessage();
		static void DeleteTriggerObjectMessage();
		static void DeleteNormalMessage();

		static bool IsCustomEventRow(Asura_MessageTableRow* pxRow, int& iCustomEventIndex, bool bCheckEventRowsOnly = true);
		static void CollapseAllRows();
		static void ExpandAllRows();
		static void ChangeMessageTarget();
		static void CreateOutputBlockCallback();
		static void RenameCallback();
		static u_short* CreateOutputBlock(const char* pszMessageBlockName = NULL);
		static void GotoTarget();
		static void GotoSource();
		static void SelectTarget();
		static void SelectSource();
		static void EditMessageCallback();

		static int RequestEventIdFromUser(Asura_Entity_Server* pxEntity, Tools_Entity_Object* pxToolsEntity);

		static bool GetAvailableMessage(Asura_Guid uGuid, Asura_List<MessageIDAndDescription>& xMessageList);
		static const char** GenerateIncomingMessageList(Tools_Object* pxObject, Asura_List<Tools_MessageEditor_ReceivedMessageData>& xIncomingMessageList);

		static void AddEventRow(int iEventId, const Asura_String& xEventName, unsigned short* pusMessageBlockId);



		//Resets a message block to ASURA_STATICMESSAGE_EMPTYBLOCK if it contains no messages
		static void FixMessageBlockIfEmpty(u_short* pusBlockID);

		static void SetReferenceObject(Tools_Object* pxObject);


		static Asura_Overlay* s_pxOverlay;
		static Asura_Tabs_Dialog* s_pxTabDialog;
		static bool s_bIsOverlayVisible;
		static bool s_bForceRecreateTable;
		static class Asura_MessageTable_Dialog* s_paxMessageTables[NUM_TABS];
		static Tools_Object* s_pxReferenceObject;
		static u_int s_uSelectedTab;
		static bool s_bInMessageEditingMode;
		static Asura_StaticMessage* s_pxMessageOverlayToOpen;
		static u_short* s_pusMessageBlockToSelect;
		static bool s_bOriginalHideTemplateTriggers;
		static bool s_bBanDelayReordering;

		static Asura_StaticMessage* s_pxMessageEditedByAttributeOverlay;//pointer to currently edited message
		static Asura_StaticMessage s_xMessageBackup;//backup copy of edited message , used for detecting any changes


		static const char* s_aszTabNames[NUM_TABS];
		static const char* s_aszColumnNames[NUM_COLUMN_NAMES];
		static Tools_Entity_Object* s_pxReferenceToolsObject;
		static Asura_Entity_Server* s_pxReferenceEntity;

		static Asura_Function_Dialog* s_pxAddMessageButton;
		static Asura_Function_Dialog* s_pxRetargetMessageButton;
		static Asura_Function_Dialog* s_pxGotoTargetButton;
		static Asura_Function_Dialog* s_pxSelectTargetButton;
		static Asura_Function_Dialog* s_pxCreateBlockButton;
		static Asura_Function_Dialog* s_pxRenameButton;
		static Asura_Function_Dialog* s_pxDeleteMessageButton;
		static Asura_Function_Dialog* s_pxCopyMessageButton;
		static Asura_Function_Dialog* s_pxPasteMessageButton;
		static Asura_Function_Dialog* s_pxCollapseAllButton;
		static Asura_Function_Dialog* s_pxExpandAllButton;

		static Tools_MessageEditor_Clipboard xClipboard;

		static u_short* s_pusCachedBlockIdFromRowSelection;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Tools_MessageEditor_Resize_Dialog : public Asura_Resize_Dialog
{
	public:
		Tools_MessageEditor_Resize_Dialog(Asura_Dialog* pxDialogToResize, Asura_Vector_2* pxUserSetSize) : Asura_Resize_Dialog(pxDialogToResize, pxUserSetSize)
		{
		}

		virtual void UpdateResize(u_int uResizeFlags, const Asura_Vector_2& xDelta)
		{
			Tools_MessageEditor::s_paxMessageTables[Tools_MessageEditor::s_pxTabDialog->GetSelected()]->SetDirty();

			Asura_Resize_Dialog::UpdateResize(uResizeFlags, xDelta);
		}
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif // ASURA_TOOLS
#endif // _TOOLS_MESSAGEEDITOR_H_

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

