//
// Tools_Mode_FSFX.cpp
//
// Created by JGP 31/01/06
// 
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#include "Core/Asura.h"

#ifdef ASURA_USE_FSFX

#include "Tools_Mode_FSFX.h"

#include "Tools_FSFX_System.h"
#include "Tools_Mode_FSFX_Template.h"
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"
#include "FSFX/Asura_FSFX_Component.h"

#include "Core/Asura_NameTree.h"
#include "Core/Asura_TextureManagement.h"
#include "Tools_Overlays.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Render_Options.h"
#include "Load&Save/Tools_ResourceExportLog.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_FSFX.h"
#include "Debugging/Asura_Logfile.h"
#include "Misc/Tools_IniFile.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Tools_Message_Editing.h"


///////////////////////////////////////////////////////////////////////////////////////
// Static "this" pointer
///////////////////////////////////////////////////////////////////////////////////////

Asura_StringTree	Tools_Mode_FSFX::s_xTemplateNameTree;
u_int				Tools_Mode_FSFX::s_uTemplateNameListSize;
const char**		Tools_Mode_FSFX::s_pszTemplateNameList;
int					Tools_Mode_FSFX::s_iSelectedTemplateIndex = 0;
Asura_List_Dialog*	Tools_Mode_FSFX::s_pxTemplateListDialog = NULL;

Tools_Mode_FSFX*	Tools_Mode_FSFX::s_pxInstance = NULL;
Asura_Vector_2		Tools_Mode_FSFX::s_xOverlayPosition( 3.0f, 20.0f );

Tools_Mode_FSFX::ProjectBuildReferencedExportTemplates Tools_Mode_FSFX::s_pfnProjectBuildReferencedExportTemplatesCallback = NULL;

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
Asura_HashTree* Tools_Mode_FSFX::s_pxFSFXToIgnoreList = NULL;
#endif 

///////////////////////////////////////////////////////////////////////////////////////
// Tools_Mode virtual functions
///////////////////////////////////////////////////////////////////////////////////////


/*
	Initialise the mode after construction. Virtual function.
*/
void Tools_Mode_FSFX::InitialiseMode()
{

	m_bDisplayInEditor = true;
	m_fDisplayIntensity = 1.0f;

	s_iSelectedTemplateIndex = 0;	// Just for sure, always select first template when we are opening this mode

	m_pxTemplateEditor = NULL;

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
	s_pxFSFXToIgnoreList = 0;
#endif
	// overlay initialise

	InitialiseOverlay();

	//Prevent render options overlay from changing the FSFX being shown
	Tools_Render_Options::SetFSFXControlledByMode(true);

}


/*
	Shut down the mode before deletion. Virtual function.
*/
void Tools_Mode_FSFX::ShutdownMode()
{
	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_FSFX_MAIN );
	s_pxTemplateListDialog = 0;
	
	// shut template editor
	CloseEditTemplateOverlay();

	// free memory being used by names
	DeleteTemplateNameList();

	// dont override
	Tools_FSFX_System::SetOverrideFSFX( NULL );
	Tools_Render_Options::SetFSFXControlledByMode(false);
}


/*
	Frame update function for editor.
	Will set override FSFX on the FSFX
	system if the flag is set
*/
void Tools_Mode_FSFX::Update(void)
{
	if ( m_bDisplayInEditor )
	{
		// if we have a sub window then try that
		Tools_FSFX_System::SetOverrideFSFX( ( m_pxTemplateEditor ) ? m_pxTemplateEditor->GetTemplate() : GetSelectedTemplate(), m_fDisplayIntensity );
	}
	else
	{
		Tools_FSFX_System::SetOverrideFSFX( NULL );
	}
}


/*
	Blank render function
*/
void Tools_Mode_FSFX::RenderUI(void)
{
}


void Tools_Mode_FSFX::PreSave()
{
}


void Tools_Mode_FSFX::PostSave()
{
}


void Tools_Mode_FSFX::QuickSave(void)
{
}


/*
	Currently always return false, as
	individual templates deal with their
	own save and load
*/
bool Tools_Mode_FSFX::IsModified() 
{ 
	return false; 
}


///////////////////////////////////////////////////////////////////////////////////////
// Overlay
///////////////////////////////////////////////////////////////////////////////////////


/*
	Create the overlay and populate it with
	the list of templates and buttons for
	add / delete / rename / edit / copy.
*/
void Tools_Mode_FSFX::InitialiseOverlay()
{
	// if it exists already, shut it down
	if ( Asura_Overlay_System::DoesOverlayExist( TOOLS_OVERLAY_FSFX_MAIN ) )
	{
		Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_FSFX_MAIN );
	}

	// create an overlay
	Asura_Overlay* pxOverlay = new Asura_Overlay( "FSFX", false, &s_xOverlayPosition );
	pxOverlay->m_uID = TOOLS_OVERLAY_FSFX_MAIN;
	Asura_Assert( pxOverlay != NULL, "Cant create FSFX overlay" );

	Asura_Vector_2 xOffset( 1.0f, 3.0f );
	Asura_Dialog* pxDialog;
	Asura_Tickbox_Dialog* pxTickBox;

	// create box of templates

	s_pxTemplateListDialog = 0;
	UpdateTemplateNameList();
	s_pxTemplateListDialog = new Asura_List_Dialog( xOffset, s_pszTemplateNameList, &s_iSelectedTemplateIndex, 8, s_uTemplateNameListSize, NULL );
	s_pxTemplateListDialog->ShowScrollBar( true );
	s_pxTemplateListDialog->SetDoubleClickFunction( _OnEditTemplate, (int)this );
	pxOverlay->AddDialog( s_pxTemplateListDialog );
	xOffset.y = s_pxTemplateListDialog->GetMaxCoords().y;
	xOffset.y += Asura_Dialog::SmallSpacing();

	// create dialogs for create, edit, copy, delete

	pxDialog = new Asura_Function_Dialog( "Edit Template", xOffset, _OnEditTemplate, (int)this );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Function_Dialog( "Create Template", xOffset, _OnCreateTemplate, (int)this );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Copy Template", xOffset, _OnCopyTemplate, (int)this );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Rename Template", xOffset, _OnRenameTemplate, (int)this );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Delete Template", xOffset, _OnDeleteTemplate, (int)this );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y += Asura_Dialog::LargeSpacing();

	pxTickBox = new Asura_Tickbox_Dialog( "Display in Editor", xOffset, &m_bDisplayInEditor );
	pxOverlay->AddDialog( pxTickBox );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Test Intensity", xOffset, &m_fDisplayIntensity, 0.0f, 1.0f, 0.05f, "%0.2f" );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// submit the overlay
	Asura_Overlay_System::SubmitNewOverlay( pxOverlay, false );	
}


///////////////////////////////////////////////////////////////////////////////////////
// FSFX callbacks
///////////////////////////////////////////////////////////////////////////////////////


/*
	Called on creation of a new template.
	Asks for a name, checks it doesn't already
	exist and then creates a new one, saving
	it to disk
*/
void Tools_Mode_FSFX::OnCreateTemplate()
{

	// get a pre-approved new name
	Asura_String xName = GetBlankTemplateName();
	
	GetString_Overlay xOverlay( "Enter name for new template", xName.GetString() );
	if( !xOverlay.DoModal() ) 
	{
		return;
	}

	Asura_String xEditedName = xOverlay.GetString();

	// check this is a unique name
	if ( Asura_FSFX_System::FindTemplate( xEditedName.GetString() ) != NULL )
	{
		Message_Overlay xMessage( "That name already exists." );
		xMessage.DoModal();
		return;
	}

	// create new template called xNewName

	Asura_FSFX_Template* pxTemplate = new Asura_FSFX_Template;
	pxTemplate->SetName( xEditedName.GetString() );
	Tools_FSFX_System::AddTemplate( pxTemplate );

	// save the template

	bool bSaved = SaveTemplateFile( pxTemplate );
	if ( !bSaved )
	{
		Asura_Assert( 0, "Can't save new template file. JGP error" );
		Asura_FSFX_System::DeleteTemplate( pxTemplate->GetNameHash() );
	}

	// need to rebuild interface

	UpdateTemplateNameList();
}


/*
	Called on delete a template.
	Checks user is sure and then deletes
	the template and deletes it from disk
*/
void Tools_Mode_FSFX::OnDeleteTemplate()
{

	// get current

	Asura_FSFX_Template* pxCurrentTemplate = GetSelectedTemplate();

	// do nothing if none selected

	if ( pxCurrentTemplate == NULL )
	{
		return;
	}

	// don't delete one we are editing

	if ( ( m_pxTemplateEditor ) && ( pxCurrentTemplate == m_pxTemplateEditor->GetOldTemplate() ) )
	{
		Message_Overlay xMessage( "Cant delete that template, it is being edited." );
		xMessage.DoModal();
		return;
	}

	// check we defo want to

	YesNo_Overlay xYesNo( "Are you sure you want to delete the selected template?" );
	bool bOkay = xYesNo.DoModal();

	if ( !bOkay )
	{
		return;
	}

	// delete the file

	if ( !DeleteTemplateFile( pxCurrentTemplate ) )
	{
		Message_Overlay xMessage( "Cant delete template file. It might be read only" );
		xMessage.DoModal();
		return;
	}

	// delete the current one

	Asura_FSFX_System::DeleteTemplate( pxCurrentTemplate->GetNameHash() );

	// need to rebuild interface

	UpdateTemplateNameList();
}


/*
	Called on rename template.
	Queries for a new name, checks that
	new name is unique, renames the template
	and renames it on disk by deleting the old
	one and saving the new one
*/
void Tools_Mode_FSFX::OnRenameTemplate()
{
	// get current

	Asura_FSFX_Template* pxCurrentTemplate = GetSelectedTemplate();

	// do nothing if none selected

	if ( pxCurrentTemplate == NULL )
	{
		return;
	}

	// get a current new name
	Asura_String xName = pxCurrentTemplate->GetName();
	
	GetString_Overlay xOverlay( "Edit name of template", xName.GetString() );
	if( !xOverlay.DoModal() ) 
	{
		return;
	}

	Asura_String xEditedName = xOverlay.GetString();

	// check for no change

	if ( strcmp( xEditedName.GetString(), xName.GetString() ) == 0 )
	{
		return;
	}

	// check this is a unique name
	if ( Asura_FSFX_System::FindTemplate( xEditedName.GetString() ) != NULL )
	{
		Message_Overlay xMessage( "That name already exists." );
		xMessage.DoModal();
		return;
	}

	// rename the file

	if ( !RenameTemplateFile( pxCurrentTemplate, xEditedName.GetString() ) )
	{
		Message_Overlay xMessage( "Rename failed. Template might be read only" );
		xMessage.DoModal();
		return;
	}

	// need to rebuild interface

	UpdateTemplateNameList();

	// if the edit template overlay is active we need to 
	// update it to the current name...

	if ( ( m_pxTemplateEditor ) && ( m_pxTemplateEditor->GetOldTemplate() == pxCurrentTemplate ) )
	{
		m_pxTemplateEditor->InvalidateOverlay();
	}

}


/*
	Called on copy of template.
	Queries for a new name, checks it's 
	not already used and creates a copy with
	the new name, saving it to disk
*/
void Tools_Mode_FSFX::OnCopyTemplate()
{
	// get current

	Asura_FSFX_Template* pxCurrentTemplate = GetSelectedTemplate();

	// do nothing if none selected

	if ( pxCurrentTemplate == NULL )
	{
		return;
	}

	// get a current new name
	Asura_String xName = GetCopyTemplateName( pxCurrentTemplate->GetName() );
	
	GetString_Overlay xOverlay( "Enter name for copied template", xName.GetString() );
	if( !xOverlay.DoModal() ) 
	{
		return;
	}

	Asura_String xEditedName = xOverlay.GetString();

	// check this is a unique name
	if ( Asura_FSFX_System::FindTemplate( xEditedName.GetString() ) != NULL )
	{
		Message_Overlay xMessage( "That name already exists." );
		xMessage.DoModal();
		return;
	}

	// create new template called xNewName

	Asura_FSFX_Template* pxTemplate = pxCurrentTemplate->CreateCopy();
	pxTemplate->SetName( xEditedName.GetString() );
	Tools_FSFX_System::AddTemplate( pxTemplate );

	// save the template

	bool bSaved = SaveTemplateFile( pxTemplate );
	if ( !bSaved )
	{
		Asura_Assert( 0, "Can't save new template file. Discarding." );
		Asura_FSFX_System::DeleteTemplate( pxTemplate->GetNameHash() );
	}

	// need to rebuild interface

	UpdateTemplateNameList();
}


/*
	Called on edit template.
	Edits the currently selected template.
	Checks to see if it is writable, otherwise
	you can view the template but not save changes,
	so the template editor is constructed slightly
	differently.
*/
void Tools_Mode_FSFX::OnEditTemplate()
{
	// get current

	Asura_FSFX_Template* pxCurrentTemplate = GetSelectedTemplate();

	// do nothing if none selected

	if ( pxCurrentTemplate == NULL )
	{
		return;
	}

	// don't edit the currently editing one

	if ( m_pxTemplateEditor )
	{
		Message_Overlay xMessage( "Edit window already open. Close before editing a new template." );
		xMessage.DoModal();
		return;
	}

	bool bReadOnly = false;
	if ( !IsTemplateReadOnly( pxCurrentTemplate ) )
	{
		Message_Overlay xMessage( "Template is read only. Will not save changes." );
		xMessage.DoModal();
		bReadOnly = true;
	}

	// if the edit template is open we need to close it
	
	CloseEditTemplateOverlay();

	// then open it with the correct stuff

	m_pxTemplateEditor = new Tools_Mode_FSFX_Template( pxCurrentTemplate, bReadOnly );

}


///////////////////////////////////////////////////////////////////////////////////////
// Public access
///////////////////////////////////////////////////////////////////////////////////////


/*
	Close the template editing overlay. Called
	here so we can set the pointer to the template
	editor to NULL, informing the rest of the class
	that it isn't open.
*/
void Tools_Mode_FSFX::CloseEditTemplateOverlay()
{
	if ( m_pxTemplateEditor )
	{
		delete m_pxTemplateEditor;
		m_pxTemplateEditor = NULL;

		UpdateTemplateNameList();
	}
}


///////////////////////////////////////////////////////////////////////////////////////
// Private functions
///////////////////////////////////////////////////////////////////////////////////////


/*
	Get currently selected template. Since templates are held in a linked
	list this isn't particularly quick...
*/
Asura_FSFX_Template* const Tools_Mode_FSFX::GetSelectedTemplate()
{
	if ( ((u_int)s_iSelectedTemplateIndex) >= s_uTemplateNameListSize )
	{
		return NULL;
	}

	Asura_Hash_ID uNameHash = Asura_GetHashID( s_pszTemplateNameList[s_iSelectedTemplateIndex] );

	return Asura_FSFX_System::FindTemplate( uNameHash );
}


/*
	Generate lists of templates. Frees memory first.
*/


void Tools_Mode_FSFX::DeleteTemplateNameList()
{
	s_xTemplateNameTree.Reset();
	delete [] s_pszTemplateNameList;
	s_pszTemplateNameList = 0;
	s_uTemplateNameListSize = 0;
}

void Tools_Mode_FSFX::UpdateTemplateNameList()
{
	DeleteTemplateNameList();

	//get all the template names
	for(u_int i=0; i<Asura_FSFX_System::GetNumTemplates(); i++)
	{
		const Asura_FSFX_Template* pxTemplate = Asura_FSFX_System::GetTemplateByIndex(i);

		int iJunkIndex;
		s_xTemplateNameTree.FindNodeIndexByName(pxTemplate->GetName(), true, iJunkIndex);
	}

	//get a sorted list of the names
	if(s_xTemplateNameTree.GetNumberOfNodes())
	{
		s_uTemplateNameListSize = s_xTemplateNameTree.GetNumberOfNodes();
		s_pszTemplateNameList = new const char*[s_uTemplateNameListSize];

		s_xTemplateNameTree.MakeSortedNameList(s_pszTemplateNameList);

	}
	else
	{
		//no templates , so just add a dummy entry
		s_uTemplateNameListSize = 1;
		s_pszTemplateNameList = new const char*[1];
		s_pszTemplateNameList[0] = "*None*";
	}

	if( (u_int)s_iSelectedTemplateIndex >= s_uTemplateNameListSize)
	{
		s_iSelectedTemplateIndex = s_uTemplateNameListSize-1;
	}

	if(s_pxTemplateListDialog)
	{
		s_pxTemplateListDialog->ReplaceStrings(s_uTemplateNameListSize, s_pszTemplateNameList);
		s_pxTemplateListDialog->ShowScrollBar( true );
	}
}


/*
	Generate a new name of the form "FSFX Template N" where
	N is an integer starting at 1 which makes the name unique
*/
const Asura_String Tools_Mode_FSFX::GetBlankTemplateName()
{
	char szName[ 256 ];
	int iIndex = 1;

	while ( 1 )
	{
		// generate a name
		Asura_PrintfToBfr( szName, "FSFX_Template_%d", 256, true, iIndex );

		// see if this name is available
		if ( Asura_FSFX_System::FindTemplate( szName ) == NULL )
		{
			return szName;
		}

		// try next one
		iIndex++;
	}
}


/*
	Given a template name make a copy of ... name
*/
const Asura_String Tools_Mode_FSFX::GetCopyTemplateName( const char* const szExistingName )
{
	char szName[ 256 ];
	int iIndex = 1;

	while ( 1 )
	{
		// generate a name
		if ( iIndex == 1 )
		{
			Asura_PrintfToBfr( szName, "Copy_of_%s", 256, true, szExistingName );	
		}
		else
		{
			Asura_PrintfToBfr( szName, "Copy_%d_of_%s", 64, true, iIndex, szExistingName );	
		}

		// see if this name is available
		if ( Asura_FSFX_System::FindTemplate( szName ) == NULL )
		{
			return szName;
		}

		// try next one
		iIndex++;
	}
}


///////////////////////////////////////////////////////////////////////////////////////
// Save and load
///////////////////////////////////////////////////////////////////////////////////////


/*
	Save the given template to disk
*/
const bool Tools_Mode_FSFX::SaveTemplateFile( Asura_FSFX_Template* const pxTemplate )
{
	Asura_String xFn = Tools_FSFX_System::MakeFilenameForTemplate( pxTemplate );
	Tools_FileSystem::EnsureDirectoryExists( Tools_FSFX_System::GetTemplateDirectory() );
	
	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	Asura_Assert( pxSaveBlock, "Unable to create save block?" );
	Asura_Chunk_SaveBlock_Stream xStream( pxSaveBlock );
	Asura_Chunk_FSFX::Create( pxTemplate, xStream );
	pxSaveBlock->AddAsuraTerminator();				
	bool bSaved = pxSaveBlock->SaveFile( xFn.GetString(), false );
	
	delete pxSaveBlock;
	
	if(bSaved)
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved %s.", xFn.GetString());
	}

	return bSaved;
}


/*
	Rename the given template on disk
*/
const bool Tools_Mode_FSFX::RenameTemplateFile( Asura_FSFX_Template* const pxTemplate, const char* const szNewTemplateName )
{
	// first we try deleting the template file. If we can delete it
	// we can probably re-create it, otherwise we will be in trouble
	Asura_String xFn = Tools_FSFX_System::MakeFilenameForTemplate( pxTemplate );
	if ( !DeleteFile( xFn.GetString() ) )
	{
		return false;
	}

	// set new name
	Asura_String xOldName = pxTemplate->GetName();

	// We are renaming object, so we have also fix Hash_id in tree list of that object.
	Asura_FSFX_System::RelocateTemplate( pxTemplate->GetNameHash() , Asura_GetHashID( szNewTemplateName ) );

	pxTemplate->SetName( szNewTemplateName );
	
	// if we can't now save it with the 
	// new name then try setting the old
	// one back and re-saving it
	if ( !SaveTemplateFile( pxTemplate ) )
	{
		pxTemplate->SetName( xOldName.GetString() );
		
		if ( !SaveTemplateFile( pxTemplate ) )
		{
			// otherwise we're lost
			Asura_Assert( 0, "Deleted old filename, cant write new filename and now cant write old filename again, so we're in trouble here. JGP error" );
		}

		return false;
	}

	return true;
}


/*
	Delete the given template on disk
*/
const bool Tools_Mode_FSFX::DeleteTemplateFile( Asura_FSFX_Template* const pxTemplate )
{
	Asura_String xFn = Tools_FSFX_System::MakeFilenameForTemplate( pxTemplate );
	if ( !DeleteFile( xFn.GetString() ) )
	{
		return false;
	}
	return true;
}


/*
	Queries if the template file is read only
*/
const bool Tools_Mode_FSFX::IsTemplateReadOnly( Asura_FSFX_Template* const pxTemplate )
{
	Asura_String xFn = Tools_FSFX_System::MakeFilenameForTemplate( pxTemplate );
	return Tools_SaveAsura::CanSaveFile( xFn.GetString(), false );
}


///////////////////////////////////////////////////////////////////////////////////////
// Export
///////////////////////////////////////////////////////////////////////////////////////


void Tools_Mode_FSFX::ExportTemplates( List<Asura_String>* pxList )
{

#ifndef ASURA_USING_ALL_LEVEL_EXPORT
	// clear the runtime system first
	// DF 19/04/07 - moved the reset to the StartFileExport function
	//Asura_FSFX_System::Reset();

	// now export and also add to the runtime system all the
	// correct templates


	// DF 19/04/07 - Don't use the AlwaysExportFSFX.ini file now
	Tools_Mode_FSFX::AddAlwaysExportTemplates( pxList, true );
#endif // ASURA_USING_ALL_LEVEL_EXPORT

	Tools_Mode_FSFX::AddReferencedExportTemplates( pxList, true );

	// Only export textures on non fake level export
	if ( Tools_Level_Export::IsDoingFakeLevelExport() == false )
	{
		// Export the textures used by the FSFX
		Tools_Mode_FSFX::ExportTextures();
	}
}

// DF 19/04/07 - 
// Function used to pass in a list of fsfx templates that should be exported
void Tools_Mode_FSFX::ExportTemplateList(List<Asura_String>* pxList, const List<const Asura_FSFX_Template* const>& xTemplateList )
{
	// DF 19/04/07 - moved the reset to the StartFileExport function
	//Asura_FSFX_System::Reset();


	bool bAddToSystem = Tools_Level_Export::IsDoingFakeLevelExport() ? true : false;
	for( CLIF<const Asura_FSFX_Template* const> ilif( &xTemplateList ); !ilif.done(); ilif.next() )
	{
		Tools_Mode_FSFX::ExportTemplate( ilif(), pxList, bAddToSystem );
	
	}
	ExportTexturesFromTemplateList(xTemplateList);
}

/*
	Go through all the messages finding ones that are SET_FSFX
	and adding the template they reference to the exported file list
*/
void Tools_Mode_FSFX::AddReferencedExportTemplates( List<Asura_String>* pxList, const bool bAddToSystem )
{
	List<const Asura_FSFX_Template* const> xReferencedTemplateList;
	BuildReferencedExportTemplates( &xReferencedTemplateList );

	for( CLIF<const Asura_FSFX_Template* const> ilif( &xReferencedTemplateList ); !ilif.done(); ilif.next() )
	{
		ExportTemplate( ilif(), pxList, bAddToSystem );
	}
}


/*
	Go through all the messages finding ones that are SET_FSFX
	and adding the template they reference to the list
*/
void Tools_Mode_FSFX::BuildReferencedExportTemplates( List<const Asura_FSFX_Template* const>* pxTemplateList )
{
	// Get the project specific exported templates e.g. from project specific tools_objects which contain FSFX data
	if ( s_pfnProjectBuildReferencedExportTemplatesCallback )
	{
		s_pfnProjectBuildReferencedExportTemplatesCallback( pxTemplateList );
	}


	for(Tools_StaticMessageIterator xIterator;!xIterator.Done();xIterator.Next())
	{
		const Asura_StaticMessage* pxStaticMessage = xIterator.GetCurrent();

		Asura_Hash_ID xNameHash = ASURA_HASH_ID_UNSET;

		if ( pxStaticMessage->m_usMsgID == ASURA_MSG_SET_FSFX )
		{
			xNameHash = *( static_cast<Asura_Hash_ID*>( pxStaticMessage->m_pData ) );
		}
		else if ( pxStaticMessage->m_usMsgID == ASURA_MSG_SET_FSFX_NEW )
		{
			const Asura_FSFX_Message* const pxFSFXMessage = static_cast<const Asura_FSFX_Message*>( pxStaticMessage->m_pData );
			xNameHash = pxFSFXMessage->m_xID;
		}

		if ( xNameHash != ASURA_HASH_ID_UNSET )
		{
			// DF 19/04/07 - if FSFX Hash is in the ignore list, ignore it from the export
#ifdef ASURA_USING_ALL_LEVEL_EXPORT
			int iJunk;
			if ( s_pxFSFXToIgnoreList->FindNodeIndexByHash(xNameHash, false, iJunk) )
			{
				continue;
			}
#endif //ASURA_USING_ALL_LEVEL_EXPORT

			const Asura_FSFX_Template* const pxTemplate = Asura_FSFX_System::FindTemplate( xNameHash );
			if ( ( pxTemplate ) && ( !pxTemplateList->contains( pxTemplate ) ) )
			{
				pxTemplateList->add_entry( pxTemplate );
			}
		}
	}
}


/* 	
	Load the AlwaysExportFSFX.ini file and exports
	to the list all the templates named within it
*/
void Tools_Mode_FSFX::AddAlwaysExportTemplates( List<Asura_String>* pxList, const bool bAddToSystem )
{
	List<const Asura_FSFX_Template* const> xGlobalTemplateList;
	BuildAlwaysExportTemplates( &xGlobalTemplateList );

	for( CLIF<const Asura_FSFX_Template* const> ilif( &xGlobalTemplateList ); !ilif.done(); ilif.next() )
	{
		ExportTemplate( ilif(), pxList, bAddToSystem );
	}
}


/*
	Load the AlwaysExportFSFX.ini file and generates
	the list of all the templates named within it
*/
void Tools_Mode_FSFX::BuildAlwaysExportTemplates( List<const Asura_FSFX_Template* const>* pxTemplateList )
{
#ifdef ASURA_TURN_OFF_ALWAYSEXPORTINIFILES_SUPPORT
	// DF 19/04/07 - This function was used for exporting the AlwaysExportFSFX.ini file. 
	// We won't be needing this anymore, should now be reading this in from the Common_Export.txt file
	return;
#else
	const char* const szIniName = "ToolsIniFiles\\AlwaysExportFSFX.ini";
	
	Tools_IniFile xIni( szIniName );
	if( !xIni.LoadIniFile() )
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_WARNING, "Failed to load %s", szIniName );
		return;
	}
	
	Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "Loaded %s", szIniName );

	while( xIni.GetNextIniTextLine() )
	{
		Asura_String xTemplateName;
		if( !xIni.GetString( xTemplateName ) ) 
		{
			continue;
		}
		const Asura_FSFX_Template* const pxTemplate = Asura_FSFX_System::FindTemplate( xTemplateName.GetString() );
		
		if ( ( pxTemplate ) && ( !pxTemplateList->contains( pxTemplate ) ) )
		{
			pxTemplateList->add_entry( pxTemplate );
		}
	}
#endif //ASURA_USING_ALL_LEVEL_EXPORT
}


/*
	Add the specified template to the list, forming
	a proper filename first.
*/
void Tools_Mode_FSFX::ExportTemplate( const Asura_FSFX_Template* const pxTemplate, List<Asura_String>* pxList, const bool bAddToSystem )
{
	const Asura_String xFileName = Tools_FSFX_System::MakeFilenameForTemplate( pxTemplate );

	static bool bReEntrant = false;
	if (!bReEntrant)
	{
		bReEntrant = true;
		for(Asura_FSFX_Template_ResourceSet_ConstMapIterator xMapIter(Asura_FSFX_System::GetResourceSet()); !xMapIter.Done(); xMapIter.Next())
		{
			Asura_Hash_ID uHash = pxTemplate->GetNameHash();
			Asura_Hash_ID uNewHash = xMapIter.GetCurrent()->GetOverride(uHash);

			if ( uNewHash != uHash)
			{
				Asura_FSFX_Template *pxTemplate = Asura_FSFX_System::FindTemplate(uNewHash);
				if (pxTemplate)
				{
					ExportTemplate(pxTemplate, pxList, bAddToSystem);
					ExportTexturesForTemplate(pxTemplate);
				}
			}
		}
		bReEntrant = false;
	}


	if( Tools_Level_Export::IsDoingFakeLevelExport() )
	{
		// (AAM - 04.10.2006) Just add the file to the list if quitting to game
		TOOLS_RESOURCEEXPORTLOG("FSFX Template", xFileName.GetString(), true);
		Tools_Level_Export::AddFileToExportList( *pxList, xFileName, false );
	}
	else
	{
		if( ShouldExportTemplateForPlatform( pxTemplate, Tools_Level_Export::s_eExportType ) )
		{		
			// Otherwise, create a copy in LevelExportTemp
			Tools_FileSystem::EnsureDirectoryExists( Tools_Level_Export::GetTempDirectory() );

			Asura_String xExportName( Tools_Level_Export::GetTempDirectory() );
			xExportName += "\\";
			xExportName += pxTemplate->GetName();
			xExportName += Tools_FSFX_System::GetTemplateExtension();

			if( Tools_Level_Export::ShouldExportBigEndian() )
			{
				xExportName += ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
			}

			// The chunk will automatically be created with the correct endianness

			Asura_SaveBlock xSaveBlock;
			Asura_Chunk_SaveBlock_Stream xStream( &xSaveBlock );
			Asura_Chunk_FSFX::Create( pxTemplate, xStream );
			xSaveBlock.AddAsuraTerminator();

			if( xSaveBlock.SaveFile( xExportName.GetString(), false ) )
			{
				TOOLS_RESOURCEEXPORTLOG("FSFX Template", xExportName.GetString(), true);
				Tools_Level_Export::AddFileToExportList( *pxList, xExportName, false );
			}
			else
			{
				Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Failed to save FSFX template to %s.", xExportName.GetString() );
			}
		}
	}

	if ( bAddToSystem )
	{
		const Asura_Hash_ID			uHash				= pxTemplate->GetNameHash();
		const Asura_FSFX_Template*	pxExistingTemplate	= Asura_FSFX_System::FindTemplate(uHash);

		if(pxExistingTemplate == NULL)
		{
			// make a copy of the template to add to the system
			Asura_FSFX_Template* const pxAddTemplate = pxTemplate->CreateCopy();
			Asura_FSFX_System::AddTemplate( pxAddTemplate );
		}
	}

}


bool Tools_Mode_FSFX::ShouldExportTemplateForPlatform( const Asura_FSFX_Template* const pxTemplate, Tools_Level_Export::EXPORT_TYPE eExportPlatform )
{
	Asura_FSFX_Component::Platform eFSFXPlatform = Asura_FSFX_Component::FSFX_PLATFORM_PC;

	// typically, the platform enums are completely different from the export types/tools platform definitions
	switch( eExportPlatform )
	{
		case Tools_Level_Export::Export_PC		: eFSFXPlatform = Asura_FSFX_Component::FSFX_PLATFORM_PC; break;
		case Tools_Level_Export::Export_PS2		: eFSFXPlatform = Asura_FSFX_Component::FSFX_PLATFORM_PS2; break;	
		case Tools_Level_Export::Export_PSP		: eFSFXPlatform = Asura_FSFX_Component::FSFX_PLATFORM_PSP; break;
		case Tools_Level_Export::Export_PS3		: eFSFXPlatform = Asura_FSFX_Component::FSFX_PLATFORM_PS3; break;
		case Tools_Level_Export::Export_Xbox360 : eFSFXPlatform = Asura_FSFX_Component::FSFX_PLATFORM_XB360; break;
		case Tools_Level_Export::Export_Wii		: eFSFXPlatform = Asura_FSFX_Component::FSFX_PLATFORM_WII; break;
		default : Asura_Assert( false, "Unsupported FSFX platform!" ); return false;
	}

	// run through the individual components to see whether there are any valid for this platform
	Asura_FSFX_ComponentList*	pxComponentList	= pxTemplate->GetComponentList();
	// We need to export FSFX with no components - the FSFX still does exist
	// and may be relied upon elsewhere...
	bool						bExport			= (pxComponentList == NULL);
	while( pxComponentList )
	{
		Asura_FSFX_Component* pxComponent = pxComponentList->Data();
		if( pxComponent )
		{
			bExport |= pxComponent->IsPlatformEnabled( eFSFXPlatform );
		}	
		
		pxComponentList = pxComponentList->Next();
	}

	return bExport;
}


/*
	Export the textures used by the FSFX templates
*/
void Tools_Mode_FSFX::ExportTextures( void )
{
	List<const Asura_FSFX_Template* const> xTemplateList;

	// Make the list of used templates
	BuildAlwaysExportTemplates( &xTemplateList );
	BuildReferencedExportTemplates( &xTemplateList );

	Tools_Mode_FSFX::ExportTexturesFromTemplateList( xTemplateList );

// DF 19/04/07 -  Moved this bit of code into the above function so it can be used in other places

/*	for( CLIF<const Asura_FSFX_Template* const> ilif( &xTemplateList ); !ilif.done(); ilif.next() )
	{
		const Asura_FSFX_Template* const pxFSFXTemplate = ilif();
		for ( const Asura_FSFX_ComponentList* pxItFSFXComponent = pxFSFXTemplate->GetComponentList(); NULL != pxItFSFXComponent; pxItFSFXComponent = pxItFSFXComponent->Next() )
		{
			const Asura_FSFX_Component* const pxFSFXComponent = pxItFSFXComponent->Data();

			const u_int uTextureCount = pxFSFXComponent->GetTextureCount();
			for( u_int uTextureIndex = 0; uTextureIndex < uTextureCount; uTextureIndex += 1)
			{
				const int iTextureID = pxFSFXComponent->GetTextureIndex( uTextureIndex );
				if ( iTextureID == ASURA_TEXTURE_NONE )
				{
					// Ignore untextured things
					Asura_Assertf( false, ( "No texture set on FSFX template '%s' component type '%s'\n", 
								   pxFSFXTemplate->GetName(),
								   pxFSFXComponent->GetTypeDesc() ) );
					continue;

				}
				if ( iTextureID < 0 )
				{
					// Ignore invalid texture ID's
					Asura_Assertf( false, ( "Invalid texture ID on FSFX template '%s' component type '%s'\n", 
								   pxFSFXTemplate->GetName(),
								   pxFSFXComponent->GetTypeDesc() ) );
					continue;

				}

				// Add this texture to the exported texture list
				// Enable alpha by default and don't premultiply alpha because PS2 uses alpha channel and not the RGB channels
				const u_int iDefaultTextureFalgs = AsuraTextureFlag_8BitAlpha | AsuraTextureFlag_DontPremultiplyAlphaInPaletteGeneration;
				Asura_Texture_Management::AddTextureFlags( iTextureID, iDefaultTextureFalgs );
				Tools_Level_Export::AddTextureToExportList( iTextureID );
			}
		}
	}
*/
}

void Tools_Mode_FSFX::ExportTexturesForTemplate( const Asura_FSFX_Template* pxFSFXTemplate )
{
	if( !ShouldExportTemplateForPlatform( pxFSFXTemplate, Tools_Level_Export::s_eExportType ) ) return;

	for ( const Asura_FSFX_ComponentList* pxItFSFXComponent = pxFSFXTemplate->GetComponentList(); NULL != pxItFSFXComponent; pxItFSFXComponent = pxItFSFXComponent->Next() )
	{
		const Asura_FSFX_Component* const pxFSFXComponent = pxItFSFXComponent->Data();			

		for( u_int uTextureIndex = 0, uTextureCount = pxFSFXComponent->GetTextureCount(); uTextureIndex < uTextureCount; uTextureIndex += 1)
		{
			const int iTextureID = pxFSFXComponent->GetTextureIndex( uTextureIndex );
			if ( iTextureID == ASURA_TEXTURE_NONE )
			{
				// Ignore untextured things
				Asura_Assertf( false, ( "No texture set on FSFX template '%s' component type '%s'\n", 
					pxFSFXTemplate->GetName(),
					pxFSFXComponent->GetTypeDesc() ) );
			} 
			else if ( iTextureID < 0 )
			{
				// Ignore invalid texture ID's
				Asura_Assertf( false, ( "Invalid texture ID on FSFX template '%s' component type '%s'\n", 
					pxFSFXTemplate->GetName(),
					pxFSFXComponent->GetTypeDesc() ) );
			}
			else
			{
				// Add this texture to the exported texture list
				// Enable alpha by default and don't premultiply alpha because PS2 uses alpha channel and not the RGB channels
				const u_int iDefaultTextureFalgs = AsuraTextureFlag_8BitAlpha | AsuraTextureFlag_DontPremultiplyAlphaInPaletteGeneration;
				Asura_Texture_Management::AddTextureFlags( iTextureID, iDefaultTextureFalgs );
				Tools_Level_Export::AddTextureToExportList( iTextureID );
			}
		}
	}
}

// DF 19/04/07 - Exports the textures from the fsfx template list passed in
void Tools_Mode_FSFX::ExportTexturesFromTemplateList( const List<const Asura_FSFX_Template* const>& xTemplateList )
{
	for( CLIF<const Asura_FSFX_Template* const> ilif( &xTemplateList ); !ilif.done(); ilif.next() )
	{
		ExportTexturesForTemplate(ilif());
	}
}

///////////////////////////////////////////////////////////////////////////////////////
// Overlay for message
///////////////////////////////////////////////////////////////////////////////////////


/*
	When creating a message which sets the current FSFX, this
	little overlay creates a window with a template button 
	for selecting a template. Static function as can be called
	out of this tools mode...
*/
void Tools_Mode_FSFX::AddMessageOverlayControls( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
 	const Asura_Hash_ID xNameHash = *( static_cast<const Asura_Hash_ID*>( pxMessage->m_pData ) );
 	Asura_FSFX_Template* const pxTemplate = Asura_FSFX_System::FindTemplate( xNameHash );
 
 	char szText[ 256 ];
 	Asura_PrintfToBfr( szText, "Template: %s", 256, false, ( pxTemplate ) ? pxTemplate->GetName() : "<none>" );
 
	Asura_Dialog *pxDialog = new Asura_Function_Dialog( szText, xOffset, SelectFSFX, (int)( pxMessage->m_pData ) );
 	pxDialog->SetHelpText("Select which FSFX to trigger");
 	pxOverlay->AddDialog( pxDialog );
 	xOffset.y += Asura_Dialog::DefaultSpacing();
}


/*
	Newer version with value and time
*/
void Tools_Mode_FSFX::AddMessageOverlayControlsNew( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
 	Asura_FSFX_Message* const pxFSFXMessage = static_cast<Asura_FSFX_Message*>( pxMessage->m_pData );
	
	const Asura_Hash_ID xNameHash = pxFSFXMessage->m_xID;

 	Asura_FSFX_Template* const pxTemplate = Asura_FSFX_System::FindTemplate( xNameHash );
 
 	char szText[ 256 ];
 	Asura_PrintfToBfr( szText, "Template: %s", 256, false, ( pxTemplate ) ? pxTemplate->GetName() : "<none>" );
 
	Asura_Dialog* pxDialog = new Asura_Function_Dialog( szText, xOffset, SelectFSFX, (int)( &( pxFSFXMessage->m_xID ) ) );
 	pxDialog->SetHelpText("Select which FSFX to trigger");
 	pxOverlay->AddDialog( pxDialog );
 	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Value", xOffset, &( pxFSFXMessage->m_fValue ), 0.0f, 1.0f, 0.1f, "%0.2f" );
 	pxDialog->SetHelpText("Value to set. Value of 1.0 is full intensity. Value of 0.0 is off.");
 	pxOverlay->AddDialog( pxDialog );
 	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Time", xOffset, &( pxFSFXMessage->m_fTime ), 0.0f, 10.0f, 0.1f, "%0.1f" );
 	pxDialog->SetHelpText("Time to set over. 0 is instant.");
 	pxOverlay->AddDialog( pxDialog );
 	xOffset.y += Asura_Dialog::DefaultSpacing();

}


/*
	Brings up a selection dialog of all available
	templates and waits for one to be selected. Sets
	it in the given message (which is passed as an int).
	This assumes that the message is of the correct type
	so the data can be set to a hash of the template name.
	Static function as called from a button callback in the
	message editing system.
*/
void Tools_Mode_FSFX::SelectFSFX( int iMessageAsInt )
{

	Asura_Hash_ID* pxNameHash = reinterpret_cast<Asura_Hash_ID*>( iMessageAsInt );

	ListSelection_Overlay xOverlay( "Select FSFX Template" );

	xOverlay.AddItem( "<none>", static_cast<int>( ASURA_HASH_ID_UNSET ) );

	// Scan all templates and add their names to list.
	for( u_int uIndex = 0; uIndex<Asura_FSFX_System::GetNumTemplates(); uIndex++ )
	{
		const Asura_FSFX_Template* pxTemplate = Asura_FSFX_System::GetTemplateByIndex(uIndex);

		xOverlay.AddItem( pxTemplate->GetName(), static_cast<int>( pxTemplate->GetNameHash() ) );
	}

	bool bDone = xOverlay.DoModal();
	if ( !bDone )
	{
		return;
	}

	*pxNameHash = static_cast<Asura_Hash_ID>( xOverlay.GetSelectionData() );

	Tools_Message_Editing::UpdateMessageOverlay();
}


#ifdef ASURA_USING_ALL_LEVEL_EXPORT
// DF 19/04/07 - Add fsfx to ignore list to not be exported in the level export
bool Tools_Mode_FSFX::AddFSFXToIgnoreList(const Asura_Hash_ID uFSFXToIgnore)
{
	int iJunk;
	return s_pxFSFXToIgnoreList->FindNodeIndexByHash(uFSFXToIgnore, true, iJunk);
}

void Tools_Mode_FSFX::EndFileExport() 
{
	delete s_pxFSFXToIgnoreList;
	s_pxFSFXToIgnoreList = 0;
}
#endif //ASURA_USING_ALL_LEVEL_EXPORT

void Tools_Mode_FSFX::StartExport(void)
{
	Asura_FSFX_System::Reset();

	#if ASURA_USING_ALL_LEVEL_EXPORT
	s_pxFSFXToIgnoreList = new Asura_HashTree;
	#endif //ASURA_USING_ALL_LEVEL_EXPORT
}






#endif //ASURA_USE_FSFX
