//
// Tools_Mode_FSFX.h
//
// Created by JGP 31/01/06
// 
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifdef ASURA_USE_FSFX

#ifndef __TOOLS_MODE_FSFX_H__
#define __TOOLS_MODE_FSFX_H__

#include "Tools_Mode.h"
#include "Core/Asura_String.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_List.h"
#include "Tools_Mode_FSFX_Helpers.h"
#include "Tools_FSFX_System.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Misc/list_tem.hpp"

class Asura_FSFX_Template;
class Tools_Mode_FSFX_Template;
struct Asura_StaticMessage;
class Asura_Overlay;
class Asura_NameTree;
class Asura_List_Dialog;


class Tools_Mode_FSFX : public Tools_Mode
{
public:	
	Tools_Mode_FSFX() { s_pxInstance = this; }
	virtual ~Tools_Mode_FSFX() { s_pxInstance = NULL; }

	// Tools_Mode functions

	virtual void InitialiseMode();
	virtual void ShutdownMode();

	virtual void Update(void);
	virtual void RenderUI(void);
	
	virtual void PreSave();
	virtual void PostSave();
	virtual void QuickSave(void);

	virtual bool IsModified();

	// public access

	void CloseEditTemplateOverlay();
	const bool SaveTemplateFile( Asura_FSFX_Template* const pxTemplate );

	// static get

	static Tools_Mode_FSFX* GetInstance() { return s_pxInstance; }

	// export

	static void ExportTemplates( List<Asura_String>* pxList );
	static void ExportTemplateList(List<Asura_String>* pxList, const List<const Asura_FSFX_Template* const>& xTemplateList );
	static bool ShouldExportTemplateForPlatform( const Asura_FSFX_Template* const pxTemplate, Tools_Level_Export::EXPORT_TYPE eExportPlatform );

	typedef void(*ProjectBuildReferencedExportTemplates)(List<const Asura_FSFX_Template* const>* pxTemplateList);
	static void SetProjectBuildReferencedExportTemplatesCallback( ProjectBuildReferencedExportTemplates pfnCallback ) { s_pfnProjectBuildReferencedExportTemplatesCallback = pfnCallback; }

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
	static bool AddFSFXToIgnoreList(const Asura_Hash_ID uFSFXHashToIgnore);
	static void EndFileExport();
#endif
	static void StartExport(void);
	
	

	// overlay for message

	static void AddMessageOverlayControls( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset );
	static void AddMessageOverlayControlsNew( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset );

private:

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
	static Asura_HashTree* s_pxFSFXToIgnoreList;
#endif //ASURA_USING_ALL_LEVEL_EXPORT

	// functions

	void DeleteTemplateNameList();
	void UpdateTemplateNameList();
	
	void InitialiseOverlay();

	const Asura_String GetBlankTemplateName();
	const Asura_String GetCopyTemplateName( const char* const szExistingName );
	Asura_FSFX_Template* const GetSelectedTemplate();

	// save load

	const bool RenameTemplateFile( Asura_FSFX_Template* const pxTemplate, const char* const szNewTemplateName );
	const bool IsTemplateReadOnly( Asura_FSFX_Template* const pxTemplate );
	const bool DeleteTemplateFile( Asura_FSFX_Template* const pxTemplate );

	// for message editing

	static void SelectFSFX( int iMessageAsInt );

	// callbacks

	STATIC_VOID_USING_INT_THIS( Tools_Mode_FSFX, OnCreateTemplate );
	STATIC_VOID_USING_INT_THIS( Tools_Mode_FSFX, OnDeleteTemplate );
	STATIC_VOID_USING_INT_THIS( Tools_Mode_FSFX, OnRenameTemplate );
	STATIC_VOID_USING_INT_THIS( Tools_Mode_FSFX, OnCopyTemplate );
	STATIC_VOID_USING_INT_THIS( Tools_Mode_FSFX, OnEditTemplate );

	// for export

	static void AddReferencedExportTemplates( List<Asura_String>* pxList, const bool bAddToSystem = false );
	static void AddAlwaysExportTemplates( List<Asura_String>* pxList, const bool bAddToSystem = false );
	static void BuildAlwaysExportTemplates( List<const Asura_FSFX_Template* const>* pxTemplateList );
	static void BuildReferencedExportTemplates( List<const Asura_FSFX_Template* const>* pxTemplateList );
	static void ExportTemplate( const Asura_FSFX_Template* const pxTemplate, List<Asura_String>* pxList, const bool bAddToSystem );
	static void ExportTextures( void );
	static void ExportTexturesFromTemplateList( const List<const Asura_FSFX_Template* const>& xTemplateList );
	static void ExportTexturesForTemplate( const Asura_FSFX_Template* pxFSFXTemplate );


	// static this pointer

	static Tools_Mode_FSFX* s_pxInstance;

	// static data

	static Asura_Vector_2 s_xOverlayPosition;

	// variables

	bool m_bDisplayInEditor;
	float m_fDisplayIntensity;

	Tools_Mode_FSFX_Template* m_pxTemplateEditor;

	//for list control for selecting a template
	static Asura_StringTree		s_xTemplateNameTree;
	static u_int				s_uTemplateNameListSize;
	static const char**			s_pszTemplateNameList;
	static int					s_iSelectedTemplateIndex;		//index into displayed list rather than into the template system
	static Asura_List_Dialog*	s_pxTemplateListDialog;


	static ProjectBuildReferencedExportTemplates s_pfnProjectBuildReferencedExportTemplatesCallback;
};


#endif //__TOOLS_MODE_FSFX_H__

#endif // ASURA_USE_FSFX
