//
// Tools_Dialog_FacialAnimation_PickAnimation.cpp
//
// Created by Richard Rice 18/11/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "FacialAnimation/Dialog/Tools_Dialog_FacialAnimation_PickAnimation.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Tree_Dialog.h"

#include "FacialAnimation/Tools_FacialAnimation_System.h"
#include "Misc/Tools_Modal_Overlay.h"

Tools_Dialog_FacialAnimation_PickAnimation::Tools_Dialog_FacialAnimation_PickAnimation(const char* szPrefix,const Asura_Vector_2& xOffset,Asura_Hash_ID* puAnimID,bool* pbModified /*= 0*/)
:Asura_Function_Dialog(szPrefix,xOffset,OnClicked,0)
{
	m_xPrefix = szPrefix;
	m_puAnimID = puAnimID;
	m_uAnimIDBackup = *puAnimID;
	m_pbModified = pbModified;

	//The callback user data is set here , rather than in the Asura_Function_Dialog constructor ,to avoid a warning
	SetCallback(OnClicked,(int)this);

	UpdateText();
}

void Tools_Dialog_FacialAnimation_PickAnimation::PerFrameUpdate()
{
	Asura_Function_Dialog::PerFrameUpdate();

	//update the label if the anim ID has changed externally
	if(m_uAnimIDBackup != *m_puAnimID)
	{
		m_uAnimIDBackup = *m_puAnimID;
		UpdateText();
	}
}

void Tools_Dialog_FacialAnimation_PickAnimation::UpdateText()
{
	char szText[100];
	if(*m_puAnimID == 0)
	{
		Asura_PrintfToBfr(szText,"%s*None*",sizeof(szText),true,m_xPrefix.GetString());
	}
	else
	{
		const Tools_FacialAnimation_FaceAnimation* pxAnim = Tools_FacialAnimation_System::FindOrLoadAnimation(*m_puAnimID);

		if(pxAnim)
		{
			Asura_String xName;
			pxAnim->GetName().SplitFilename(0,&xName,0);
			Asura_PrintfToBfr(szText,"%s%s",sizeof(szText),true,m_xPrefix.GetString(),xName.GetString());
		}
		else
		{
			Asura_PrintfToBfr(szText,"%s*Unknown* (0x%x)",sizeof(szText),true,m_xPrefix.GetString(),*m_puAnimID);
		}
	}
	if(m_pxOverlay)
	{
		m_pxOverlay->UpdateDialogLabel(this,szText);
	}
	else
	{
		UpdateLabel(szText);
	}
}




void Tools_Dialog_FacialAnimation_PickAnimation::OnClicked()
{
	const char*				szDirectory		= Tools_FacialAnimation_System::GetAnimationDirectoryName();
	const char*				szFileExtension	= Tools_FacialAnimation_System::GetAnimationFileExtension();

	FileSelect_Overlay		xSelectAnim( "Select animation", szDirectory, szFileExtension );

	if( m_puAnimID )
	{
		xSelectAnim.SelectFile( *m_puAnimID, true );
	}

	if( xSelectAnim.DoModal() )
	{
		Asura_String xFileNameWithPath;
		xSelectAnim.GetSelectedFileWithPath( m_uAnimIDBackup, xFileNameWithPath );
		if(m_puAnimID)		*m_puAnimID	= m_uAnimIDBackup;
		if(m_pbModified)	*m_pbModified = true;
		UpdateText();
	}
}


#endif //defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
