//
// Tools_Dialog_FacialAnimation_PickFace.cpp
//
// Created by Richard Rice 17/11/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "FacialAnimation/Dialog/Tools_Dialog_FacialAnimation_PickFace.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Tree_Dialog.h"

#include "FacialAnimation/Tools_FacialAnimation_System.h"
#include "Misc/Tools_Modal_Overlay.h"

Tools_Dialog_FacialAnimation_PickFace::Tools_Dialog_FacialAnimation_PickFace(const char* szPrefix,const Asura_Vector_2& xOffset,Asura_Hash_ID* puFaceID,bool* pbModified /*= 0*/)
:Asura_Function_Dialog(szPrefix,xOffset,OnClicked,0)
{
	m_xPrefix = szPrefix;
	m_puFaceID = puFaceID;
	m_uFaceIDBackup = *puFaceID;
	m_pbModified = pbModified;

	//The callback user data is set here , rather than in the Asura_Function_Dialog constructor ,to avoid a warning
	SetCallback(OnClicked,(int)this);

	UpdateText();
}

void Tools_Dialog_FacialAnimation_PickFace::PerFrameUpdate()
{
	Asura_Function_Dialog::PerFrameUpdate();

	//update the label if the face ID has changed externally
	if(m_uFaceIDBackup != *m_puFaceID)
	{
		m_uFaceIDBackup = *m_puFaceID;
		UpdateText();
	}
}

void Tools_Dialog_FacialAnimation_PickFace::UpdateText()
{
	char szText[100];
	if(*m_puFaceID == 0)
	{
		Asura_PrintfToBfr(szText,"%s*None*",sizeof(szText),true,m_xPrefix.GetString());
	}
	else
	{
		const Tools_FacialAnimation_Face* pxFace = Tools_FacialAnimation_System::FindOrLoadFace(*m_puFaceID);

		if(pxFace)
		{
			Asura_PrintfToBfr(szText,"%s%s",sizeof(szText),true,m_xPrefix.GetString(),pxFace->GetName().GetString());
		}
		else
		{
			Asura_PrintfToBfr(szText,"%s*Unknown* (0x%x)",sizeof(szText),true,m_xPrefix.GetString(),*m_puFaceID);
		}
	}
	if(m_pxOverlay)
	{
		m_pxOverlay->UpdateDialogLabel(this,szText);
	}
	else
	{
		UpdateLabel(szText);
	}
}




void Tools_Dialog_FacialAnimation_PickFace::OnClicked()
{
	const char*				szDirectory		= Tools_FacialAnimation_System::GetFaceDirectoryName();
	const char*				szFileExtension	= Tools_FacialAnimation_System::GetFaceFileExtension();


	FileSelect_Overlay		xSelectFace( "Select face", szDirectory, szFileExtension );

	if( m_puFaceID )
	{
		xSelectFace.SelectFile( *m_puFaceID, false );
	}

	if( xSelectFace.DoModal() )
	{
		Asura_String xSelectedItemName;
		xSelectFace.GetSelectedFile( m_uFaceIDBackup, xSelectedItemName );

		//update the hash ID
		if(m_puFaceID)	 *m_puFaceID   = m_uFaceIDBackup;
		if(m_pbModified) *m_pbModified = true;
		UpdateText();
	}
}


#endif //defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
