
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FacialAnimation_LipsyncConversion_Annosoft.h
// Author : Andy Tate
// Date   : 5-Aug-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_FACIALANIMATION_LIPSYNCCONVERSION_ANNOSOFT_H__
#define __TOOLS_FACIALANIMATION_LIPSYNCCONVERSION_ANNOSOFT_H__

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Annosoft/lib/libarticulator.h"
#include "Annosoft/lib/liblipsync.h"

#include "FacialAnimation/Tools_FacialAnimation_LipsyncConversion.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Tools_FacialAnimation_FaceAnimation;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_FacialAnimation_LipsyncConversion_Annosoft
// Author       : Andy Tate
// Date         : 5-Aug-2008
//
// Description  : Class used to interface to the Annosoft SDK for converting
//				  .WAV files into facial-animation data for Asura.
////////////////////////////////////////////////////////////////////////////////

class Tools_FacialAnimation_LipsyncConversion_Annosoft : public Tools_FacialAnimation_LipsyncConversion
{
public:
	// Public functions...
	Tools_FacialAnimation_LipsyncConversion_Annosoft(const char* szUserName, const char* szCompanyName, const char* szLicenseKey);
	virtual ~Tools_FacialAnimation_LipsyncConversion_Annosoft();

	// Overridden conversion functions...
	virtual bool									BeginConversion(void);
	virtual Tools_FacialAnimation_FaceAnimation*	ConvertTextless(const char* szFaceTypeName, const char* szBaseFacePoseName, const char* szWAVFilename);
	virtual bool									EndConversion(void);

	// Overridden accessor functions...
	virtual bool		IsConversionInProgress(void) const	{ return(m_pxHmm != NULL); }

	virtual const u_int	GetNumPhonemes(void) const						{ return(s_uNumPhonemes); }
	virtual char const*	GetPhonemeName(const u_int uIndex) const		{ return(s_aszPhonemes[uIndex][0]); }
	virtual char const*	GetPhonemeComments(const u_int uIndex) const	{ return(s_aszPhonemes[uIndex][1]); }


protected:
	// Protected variables...

	// Protected functions...
	bool									AnnosoftCommandSuccessful(const char* szOperationText, const serror xError);

	void									InitialiseAnimationFromMarkerList(ISyncResultsCollection& xMarkerList, Tools_FacialAnimation_FaceAnimation& xAnimation);
	void									InitialiseAnimationFromArticulation(IPhnMixtureArticulator& xArticulator, Tools_FacialAnimation_FaceAnimation& xAnimation);
	void									InitialiseAnimationFromArticulationEnvelopes(IPhnMixtureArticulator& xArticulator, Tools_FacialAnimation_FaceAnimation& xAnimation);

	Tools_FacialAnimation_FaceAnimation*	CreateAnimation(const char* szFaceTypeName, const char* szBaseFacePoseName, const char* szWAVFilename, ISyncResultsCollection& xMarkerList);


private:
	// Private typedefs...
	typedef Tools_FacialAnimation_LipsyncConversion PARENT;

	// Private variables...
	const char*		m_szHmmFilename;
	const char*		m_szTrigrammerFilename;
	CAccousticHMM*	m_pxHmm;

	// Static data...
	static char const* const	s_aszPhonemes[][2];
	static const u_int			s_uNumPhonemes;

	// Private functions...
	Tools_FacialAnimation_LipsyncConversion_Annosoft(const Tools_FacialAnimation_LipsyncConversion_Annosoft& xOther);

	Tools_FacialAnimation_LipsyncConversion_Annosoft&	operator=(const Tools_FacialAnimation_LipsyncConversion_Annosoft& xOther);

};   // End of class 'Tools_FacialAnimation_LipsyncConversion_Annosoft'.

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#endif   // ifndef __TOOLS_FACIALANIMATION_LIPSYNCCONVERSION_ANNOSOFT_H__


/// End of file. ///
