
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType.cpp
// Author : Andy Tate
// Date   : 3-Jul-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Overlays/Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType.h"

// Asura independent header-files...
#include "FacialAnimation/Asura_FacialAnimation_MuscleGroup.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"

// Tools header-files...
#include "FacialAnimation/Tools_FacialAnimation_Face_MuscleGroup.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

int	Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::s_iControlType	= Asura_FacialAnimation_MuscleGroup::CT_SINGLE;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType(void)
: PARENT("Select Control-Type")
, m_bAcceptControlType(false)
{
}

/// End of function 'Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::InitialiseOverlay
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::InitialiseOverlay(Asura_Overlay* pxOverlay)
{
	const char**		pszControlTypeNames	= Asura_FacialAnimation_MuscleGroup::Tools_GetControlTypeNames();
	const u_int			uMaxControlTypes	= Asura_FacialAnimation_MuscleGroup::MAX_CONTROL_TYPES;

	Asura_Vector_2		xOffset(1.0f, 3.0f);

	Asura_List_Dialog*	pxNewListDialogue;
	Asura_Dialog*		pxNewDialogue;

	// Create the list of control-types...
	pxNewListDialogue	= new Asura_List_Dialog (xOffset, pszControlTypeNames, &s_iControlType, uMaxControlTypes, uMaxControlTypes, NULL);
	pxNewListDialogue->SetDoubleClickFunction(OnOkaySelected, reinterpret_cast<int>(this));
	pxOverlay->AddDialog(pxNewListDialogue);

	// Create the "Okay" and "Cancel" buttons...
	xOffset[1]		= pxNewListDialogue->m_xMaxCoord[1] + 1.0f;
	pxNewDialogue	= new Asura_Function_Dialog ("Okay", xOffset, OnOkaySelected, reinterpret_cast<int>(this));
	pxOverlay->AddDialog(pxNewDialogue);

	xOffset[0]		= pxNewDialogue->m_xMaxCoord[0] + 5.0f;
	pxNewDialogue	= new Asura_Function_Dialog ("Cancel", xOffset, OnCancelSelected, reinterpret_cast<int>(this));
	pxOverlay->AddDialog(pxNewDialogue);
}

/// End of function 'Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::InitialiseOverlay'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::OnOkaySelected
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::OnOkaySelected(int iUserData)
{
	Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType*	pxThis	= reinterpret_cast<Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType*>(iUserData);

	pxThis->m_bAcceptControlType	= true;
	pxThis->CloseOverlay();
}

/// End of function 'Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::OnOkaySelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::OnCancelSelected
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::OnCancelSelected(int iUserData)
{
	Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType*	pxThis	= reinterpret_cast<Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType*>(iUserData);

	pxThis->m_bAcceptControlType	= false;
	pxThis->CloseOverlay();
}

/// End of function 'Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::OnCancelSelected'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::DoModal
// Author       : Andy Tate
// Date         : 23-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::DoModal(void)
{
	ShowOverlay();
	return(m_bAcceptControlType);
}

/// End of function 'Tools_ModalOverlay_FacialAnimation_MuscleGroupSelectControlType::DoModal'. ///

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
