
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Overlay_FacialAnimation.h
// Author : Andy Tate
// Date   : 8-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_OVERLAY_FACIALANIMATION_H__
#define __TOOLS_OVERLAY_FACIALANIMATION_H__

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Tree_Dialog.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_Overlay_FacialAnimation
// Author       : Andy Tate
// Date         : 8-May-2008
//
// Description  : Class to handle the displaying of the facial-animation overlay
//				  in Tools.
////////////////////////////////////////////////////////////////////////////////

class Tools_Overlay_FacialAnimation : public Asura_Overlay
{
public:
	// Public functions...
	static void	CreateOverlay(void);
	static void	CloseOverlay(void);

	static void ToggleOverlay(void)		{ if(OverlayExists()) CloseOverlay(); else CreateOverlay(); }
	static bool	OverlayExists(void)		{ return(s_pxThis != NULL); }


protected:
	// Protected variables...
	Asura_TreeDialog_FileTree	m_xTypeTree;		// Tree of face-types.
	Asura_TreeDialog_FileTree	m_xFaceTree;		// Tree of faces.
	Asura_TreeDialog_FileTree	m_xAnimationTree;	// Tree of face-animations.
	Asura_TreeDialog_FileTree	m_xPoseTree;		// Tree of face-poses used for expressions to create face animations from .WAV files.

	// Protected functions...
	Tools_Overlay_FacialAnimation(void);
	virtual ~Tools_Overlay_FacialAnimation();

	void	AddOverlayItems(Asura_Vector_2& xOffset);

	// Face-type functions...
	static void	CreateNewType(const char* szDirectory);

	static void OnCreateNewTypeSelected(void);
	static void OnCreateNewTypeInCurrentDirectorySelected(void);
	static void OnEditCurrentTypeSelected(void);

	static bool	IsTypeTreeItemADirectory(void);
	static bool	IsTypeTreeItemAFile(void);

	// Face functions...
	static void	CreateNewFace(const char* szDirectory);

	static void OnCreateNewFaceSelected(void);
	static void OnCreateNewFaceInCurrentDirectorySelected(void);
	static void OnEditCurrentFaceSelected(void);

	static bool	IsFaceTreeItemADirectory(void);
	static bool	IsFaceTreeItemAFile(void);

	// Face-animation functions...
	static void OnCreateFaceAnimationSelected(void);


private:
	// Private typedefs...
	typedef Asura_Overlay PARENT;

	// Private variables...
	static Tools_Overlay_FacialAnimation*	s_pxThis;	// We only ever create a single instance of this overlay.

	// Private functions...
	Tools_Overlay_FacialAnimation(const Tools_Overlay_FacialAnimation& xOther);

	Tools_Overlay_FacialAnimation&	operator=(const Tools_Overlay_FacialAnimation& xOther);

};   // End of class 'Tools_Overlay_FacialAnimation'.

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#endif   // ifndef __TOOLS_OVERLAY_FACIALANIMATION_H__


/// End of file. ///
