
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FacialAnimation_FaceAnimation_FacePose.cpp
// Author : Andy Tate
// Date   : 5-Aug-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Tools_FacialAnimation_FaceAnimation_FacePose.h"

// Asura independent header-files...
#include "ChunkSystem/Asura_Chunk_Stream.h"

// Tools header-files...


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		ART		Created.
////////////////////////////////////////////////////////////////////////////////
const u_int	Tools_FacialAnimation_FaceAnimation_FacePose::s_uChunkVersion	= 1;


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceAnimation_FacePose::Tools_FacialAnimation_FaceAnimation_FacePose
// Author       : Andy Tate
// Date         : 5-Aug-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceAnimation_FacePose::Tools_FacialAnimation_FaceAnimation_FacePose(void)
{
}

/// End of function 'Tools_FacialAnimation_FaceAnimation_FacePose::Tools_FacialAnimation_FaceAnimation_FacePose'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceAnimation_FacePose::Tools_FacialAnimation_FaceAnimation_FacePose
// Author       : Andy Tate
// Date         : 6-Aug-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceAnimation_FacePose::Tools_FacialAnimation_FaceAnimation_FacePose(const Asura_Hash_ID uHash,
																						   Asura_Chunk_Stream& xStream)
{
	ReadFromChunkStream(xStream);
}

/// End of function 'Tools_FacialAnimation_FaceAnimation_FacePose::Tools_FacialAnimation_FaceAnimation_FacePose'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceAnimation_FacePose::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 6-Aug-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FaceAnimation_FacePose::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;
	u_int	uNumKeyFrames;

	// Remove any existing data...
	m_xKeyFrameContainer.Clear();

	// Firstly, read the version, then the data itself...
	xStream >> uVersion;

	xStream >> m_xFacePoseName;
	xStream >> uNumKeyFrames;

	for(u_int u = 0; u < uNumKeyFrames; u++)
	{
		float	fTime;
		float	fWeight;

		xStream >> fTime;
		xStream >> fWeight;

		{
			Tools_FacialAnimation_FaceAnimation_FacePose_KeyFrame	xNewKeyFrame(fTime, fWeight);

			m_xKeyFrameContainer.Push_Back(xNewKeyFrame);
		}
	}
}

/// End of function 'Tools_FacialAnimation_FaceAnimation_FacePose::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceAnimation_FacePose::WriteToChunkStream
// Author       : Andy Tate
// Date         : 6-Aug-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FaceAnimation_FacePose::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	const u_int	uNumKeyFrames	= m_xKeyFrameContainer.Size();

	xStream << s_uChunkVersion;

	xStream << m_xFacePoseName;
	xStream << uNumKeyFrames;

	for(KeyFrameConstIterator xIterator(m_xKeyFrameContainer); !xIterator.Done(); xIterator.Next())
	{
		const Tools_FacialAnimation_FaceAnimation_FacePose_KeyFrame*	pxKeyFrame	= xIterator.GetCurrent();
		const float														fTime		= pxKeyFrame->GetTime();
		const float														fWeight		= pxKeyFrame->GetWeight();

		xStream << fTime;
		xStream << fWeight;
	}
}

/// End of function 'Tools_FacialAnimation_FaceAnimation_FacePose::WriteToChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceAnimation_FacePose::AddKeyFrame
// Author       : Andy Tate
// Date         : 5-Aug-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FaceAnimation_FacePose::AddKeyFrame(const u_int uMS,
															   const float fWeight)
{
	const Tools_FacialAnimation_FaceAnimation_FacePose_KeyFrame	xNewKeyFrame(uMS, fWeight);

	// See whether this key-frame needs adding somewhere within the list...
	for(KeyFrameItem* pxItem = m_xKeyFrameContainer.Back(); (pxItem != NULL); pxItem = pxItem->Prev())
	{
		if(xNewKeyFrame > pxItem->Data())
		{
			m_xKeyFrameContainer.Insert_After(pxItem, xNewKeyFrame);
			return;
		}
	}

	// Doesn't need inserting - add to the start of the list...
	m_xKeyFrameContainer.Push_Front(xNewKeyFrame);
}

/// End of function 'Tools_FacialAnimation_FaceAnimation_FacePose::AddKeyFrame'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceAnimation_FacePose::AddWindow
// Author       : Andy Tate
// Date         : 7-Aug-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FaceAnimation_FacePose::AddWindow(const u_int uStartMS,
															 const u_int uEndMS,
															 const float fWeight)
{
	////////////////////////////////////////////////////////////////////////////////
	// ARTTODO: For now just add 2 key-frames - this is not a good solution,
	// since there could be other weights of higher/lower value around and 
	// we need to ensure that we always store the strongest possible weight.
	// Lets get something running first, then we can make alterations...
	////////////////////////////////////////////////////////////////////////////////
	AddKeyFrame(uStartMS, fWeight);
	AddKeyFrame(uEndMS, fWeight);
}

/// End of function 'Tools_FacialAnimation_FaceAnimation_FacePose::AddWindow'. ///

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
