
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FacialAnimation_FaceType.cpp
// Author : Andy Tate
// Date   : 13-Jun-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

// This file's header...
#include "FacialAnimation/Tools_FacialAnimation_FaceType.h"

// Asura independent header-files...
#include "ChunkSystem/Asura_Chunk_Stream.h"

// Tools header-files...
#include "FacialAnimation/Tools_FacialAnimation_FacePose.h"
#include "FacialAnimation/Tools_FacialAnimation_LipsyncConversion.h"
#include "FacialAnimation/Tools_FacialAnimation_System.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Version	Author	Comment
// ~~~~~~~	~~~~~~	~~~~~~~
// 1		ART		Created.
// 2		ART		Muscle-groups are now their own class instead of just names.
// 3		ART		Phoneme to face-pose mappings are now stored.
////////////////////////////////////////////////////////////////////////////////
const u_int	Tools_FacialAnimation_FaceType::s_uChunkVersion	= 3;


////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::Tools_FacialAnimation_FaceType
// Author       : Andy Tate
// Date         : 13-Jun-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceType::Tools_FacialAnimation_FaceType(void)
{
	const Tools_FacialAnimation_LipsyncConversion*	pxLipsyncConversion	= Tools_FacialAnimation_System::GetLipsyncConversion();

	if(pxLipsyncConversion != NULL)
	{
		const u_int	uNumPhonemes	= pxLipsyncConversion->GetNumPhonemes();

		for(u_int u = 0; u < uNumPhonemes; u++)
		{
			const char*	szPhoneme	= pxLipsyncConversion->GetPhonemeName(u);

			CreatePhonemeName(szPhoneme);
		}
	}
}

/// End of function 'Tools_FacialAnimation_FaceType::Tools_FacialAnimation_FaceType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::Tools_FacialAnimation_FaceType
// Author       : Andy Tate
// Date         : 13-Jun-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceType::Tools_FacialAnimation_FaceType(const Asura_Hash_ID uHash, Asura_Chunk_Stream& xStream)
{
	ReadFromChunkStream(xStream);
}

/// End of function 'Tools_FacialAnimation_FaceType::Tools_FacialAnimation_FaceType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::~Tools_FacialAnimation_FaceType
// Author       : Andy Tate
// Date         : 13-Jun-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceType::~Tools_FacialAnimation_FaceType()
{
	Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup	= m_xMuscleGroupContainer.PopHeadObject();
	Asura_String*								pxPhonemeName	= m_xPhonemeNameContainer.PopHeadObject();

	while(pxMuscleGroup != NULL)
	{
		delete(pxMuscleGroup);
		pxMuscleGroup	= m_xMuscleGroupContainer.PopHeadObject();
	}

	while(pxPhonemeName != NULL)
	{
		delete(pxPhonemeName);
		pxPhonemeName	= m_xPhonemeNameContainer.PopHeadObject();
	}
}

/// End of function 'Tools_FacialAnimation_FaceType::~Tools_FacialAnimation_FaceType'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::ReadFromChunkStream
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FaceType::ReadFromChunkStream(Asura_Chunk_Stream& xStream)
{
	u_int	uVersion;

	// First, read (and verify) the chunk-version.
	xStream >> uVersion;

	switch(uVersion)
	{
		case 1:
		{
			u_int	uNumPhonemeNames;
			u_int	uNumMuscleGroups;

			// Read the common data...
			xStream >> m_xName;
			xStream >> uNumPhonemeNames;
			xStream >> uNumMuscleGroups;

			// Read the phoneme names AND add them to the data...
			for(u_int u = 0; u < uNumPhonemeNames; u++)
			{
				Asura_String*	pxName	= new Asura_String;

				xStream >> *pxName;
				m_xPhonemeNameContainer.PushTail(pxName);
			}

			// Read the muscle-group names AND add them to the data...
			for(u_int u = 0; u < uNumMuscleGroups; u++)
			{
				Asura_String								xName;
				Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup;
				
				xStream >> xName;
				pxMuscleGroup	= new Tools_FacialAnimation_FaceType_MuscleGroup (xName.GetString(), Asura_FacialAnimation_MuscleGroup::CT_SINGLE);
				m_xMuscleGroupContainer.PushTail(pxMuscleGroup);
			}
			break;
		}

		case 2:
		{
			u_int	uNumPhonemeNames;
			u_int	uNumMuscleGroups;

			// Read the common data...
			xStream >> m_xName;
			xStream >> uNumPhonemeNames;
			xStream >> uNumMuscleGroups;

			// Read the phoneme names AND add them to the data...
			for(u_int u = 0; u < uNumPhonemeNames; u++)
			{
				Asura_String*	pxName	= new Asura_String;

				xStream >> *pxName;
				m_xPhonemeNameContainer.PushTail(pxName);
			}

			// Read the muscle-group names AND add them to the data...
			for(u_int u = 0; u < uNumMuscleGroups; u++)
			{
				Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup	= new Tools_FacialAnimation_FaceType_MuscleGroup (xStream);
				
				m_xMuscleGroupContainer.PushTail(pxMuscleGroup);
			}
			break;
		}

		case 3:
		{
			u_int	uNumPhonemeNames;
			u_int	uNumMuscleGroups;
			u_int	uNumMappings;

			// Read the common data...
			xStream >> m_xName;
			xStream >> uNumPhonemeNames;
			xStream >> uNumMuscleGroups;
			xStream >> uNumMappings;

			// Read the phoneme names AND add them to the data...
			for(u_int u = 0; u < uNumPhonemeNames; u++)
			{
				Asura_String*	pxName	= new Asura_String;

				xStream >> *pxName;
				m_xPhonemeNameContainer.PushTail(pxName);
			}

			// Read the muscle-group names AND add them to the data...
			for(u_int u = 0; u < uNumMuscleGroups; u++)
			{
				Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup	= new Tools_FacialAnimation_FaceType_MuscleGroup (xStream);
				
				m_xMuscleGroupContainer.PushTail(pxMuscleGroup);
			}

			// Read the mappings AND add them to the data...
			for(u_int u = 0; u < uNumMappings; u++)
			{
				Asura_String*	pxName;
				Asura_Hash_ID	uHash;

				xStream >> uHash;
				pxName	= m_xMappingContainer.AddNode(uHash);
				xStream >> *pxName;
			}
			break;
		}

		default:
		{
			Asura_Assertf(false, ("Unexpected chunk-version %d", uVersion));
			break;
		}
	}		// End of switch.
}

/// End of function 'Tools_FacialAnimation_FaceType::ReadFromChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::WriteToChunkStream
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FaceType::WriteToChunkStream(Asura_Chunk_Stream& xStream) const
{
	const u_int								uNumPhonemeNames	= m_xPhonemeNameContainer.GetNumEntries();
	const u_int								uNumMuscleGroups	= m_xMuscleGroupContainer.GetNumEntries();
	const u_int								uNumMappings		= m_xMappingContainer.GetNumberOfNodes();

	PhonemeNameConstIterator				xPhonemeNameIterator(m_xPhonemeNameContainer);
	MuscleGroupConstIterator				xMuscleGroupIterator(m_xMuscleGroupContainer);
	PhonemeToFacePoseMappingConstIterator	xMappingIterator(m_xMappingContainer);

	// First write the chunk-version...
	xStream << s_uChunkVersion;

	xStream << m_xName;
	xStream << uNumPhonemeNames;
	xStream << uNumMuscleGroups;
	xStream << uNumMappings;

	// Write the phoneme names...
	for(u_int u = 0; u < uNumPhonemeNames; u++, xPhonemeNameIterator.Next())
	{
		const Asura_String*	pxName	= xPhonemeNameIterator.GetCurrent();

		xStream << *pxName;
	}

	// Write the muscle-group names...
	for(u_int u = 0; u < uNumMuscleGroups; u++, xMuscleGroupIterator.Next())
	{
		const Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup	= xMuscleGroupIterator.GetCurrent();

		pxMuscleGroup->WriteToChunkStream(xStream);
	}

	// Write the mappings between phonemes and face-poses...
	for(u_int u = 0; u < uNumMappings; u++, xMappingIterator.Next())
	{
		const Asura_Hash_ID	uHash	= xMappingIterator.GetCurrentHash();
		const Asura_String*	pxName	= xMappingIterator.GetCurrent();

		xStream << uHash;
		xStream << *pxName;
	}
}

/// End of function 'Tools_FacialAnimation_FaceType::WriteToChunkStream'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::CreateMuscleGroup
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceType_MuscleGroup* Tools_FacialAnimation_FaceType::CreateMuscleGroup(const char* szName,
																							  const Asura_FacialAnimation_MuscleGroup::ControlType eControlType)
{
	Tools_FacialAnimation_FaceType_MuscleGroup*	pxNewMuscleGroup	= new Tools_FacialAnimation_FaceType_MuscleGroup(szName, eControlType);

	m_xMuscleGroupContainer.PushTail(pxNewMuscleGroup);
	return(pxNewMuscleGroup);
}

/// End of function 'Tools_FacialAnimation_FaceType::CreateMuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::DestroyMuscleGroup
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_FacialAnimation_FaceType::DestroyMuscleGroup(const char* szName)
{
	Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup	= FindMuscleGroup(szName);

	if(pxMuscleGroup == NULL)
	{
		Asura_Assertf(false, ("Failed to find muscle-group \"%s\" to destroy.", szName));
		return(false);
	}
	else
	{
		m_xMuscleGroupContainer.RemoveFromList(pxMuscleGroup);
		delete(pxMuscleGroup);
		return(true);
	}
}

/// End of function 'Tools_FacialAnimation_FaceType::DestroyMuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::SwapNames
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FaceType::SwapMuscleGroups(const Tools_FacialAnimation_FaceType_MuscleGroup* pxMuscleGroup1,
													  const Tools_FacialAnimation_FaceType_MuscleGroup* pxMuscleGroup2)
{
	const u_int										uNumMuscleGroups	= m_xMuscleGroupContainer.GetNumEntries();
	Tools_FacialAnimation_FaceType_MuscleGroup**	apxMuscleGroups		= new Tools_FacialAnimation_FaceType_MuscleGroup* [uNumMuscleGroups];

	int												iIndex1				= -1;
	int												iIndex2				= -1;

	// Empty our list of names AND find the indices to be swapped...
	for(u_int u = 0; u < uNumMuscleGroups; u++)
	{
		apxMuscleGroups[u]	= m_xMuscleGroupContainer.PopHeadObject();

		if(apxMuscleGroups[u] == pxMuscleGroup1)
		{
			iIndex1	= u;
		}
		else if(apxMuscleGroups[u] == pxMuscleGroup2)
		{
			iIndex2	= u;
		}
	}

	// Swap our 2 muscle-groups over...
	if(iIndex1 < 0)
	{
		Asura_Assert(false, "Failed to find first muscle-group to be swapped.");
	}
	else if(iIndex2 < 0)
	{
		Asura_Assert(false, "Failed to find second muscle-group to be swapped.");
	}
	else
	{
		Tools_FacialAnimation_FaceType_MuscleGroup*	pxTempMuscleGroup	= apxMuscleGroups[iIndex1];

		apxMuscleGroups[iIndex1]	= apxMuscleGroups[iIndex2];
		apxMuscleGroups[iIndex2]	= pxTempMuscleGroup;

		// Re-populate or muscle-groups...
		for(u_int u = 0; u < uNumMuscleGroups; u++)
		{
			m_xMuscleGroupContainer.PushTail(apxMuscleGroups[u]);
		}
	}

	// Finally, free-up the allocated data...
	delete[](apxMuscleGroups);
}

/// End of function 'Tools_FacialAnimation_FaceType::SwapNames'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::FindMuscleGroup
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

Tools_FacialAnimation_FaceType_MuscleGroup* Tools_FacialAnimation_FaceType::FindMuscleGroup(const char* szName)
{
	for(MuscleGroupIterator xIterator(m_xMuscleGroupContainer); !xIterator.Done(); xIterator.Next())
	{
		Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup	= xIterator.GetCurrent();

		if(pxMuscleGroup != NULL)
		{
			const Asura_String&	xMuscleGroupName	= pxMuscleGroup->GetName();

			if(xMuscleGroupName.Compare(szName, true) == 0)
			{
				return(pxMuscleGroup);
			}
		}
	}

	return(NULL);
}

/// End of function 'Tools_FacialAnimation_FaceType::FindMuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::FindMuscleGroup
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

const Tools_FacialAnimation_FaceType_MuscleGroup* Tools_FacialAnimation_FaceType::FindMuscleGroup(const char* szName) const
{
	for(MuscleGroupConstIterator xIterator(m_xMuscleGroupContainer); !xIterator.Done(); xIterator.Next())
	{
		const Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup	= xIterator.GetCurrent();

		if(pxMuscleGroup != NULL)
		{
			const Asura_String&	xMuscleGroupName	= pxMuscleGroup->GetName();

			if(xMuscleGroupName.Compare(szName, true) == 0)
			{
				return(pxMuscleGroup);
			}
		}
	}

	return(NULL);
}

/// End of function 'Tools_FacialAnimation_FaceType::FindMuscleGroup'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::FindMuscleGroupIndex
// Author       : Andy Tate
// Date         : 28-Aug-2008
////////////////////////////////////////////////////////////////////////////////

int Tools_FacialAnimation_FaceType::FindMuscleGroupIndex(const Asura_Hash_ID uHash) const
{
	int	iIndex	= 0;

	for(MuscleGroupConstIterator xIterator(m_xMuscleGroupContainer); !xIterator.Done(); xIterator.Next(), iIndex++)
	{
		const Tools_FacialAnimation_FaceType_MuscleGroup*	pxMuscleGroup	= xIterator.GetCurrent();

		if(pxMuscleGroup != NULL)
		{
			const Asura_String&	xMuscleGroupName	= pxMuscleGroup->GetName();
			const Asura_Hash_ID	uMuscleGroupHash	= xMuscleGroupName.GetHash();

			if(uMuscleGroupHash == uHash)
			{
				return(iIndex);
			}
		}
	}

	return(-1);
}

/// End of function 'Tools_FacialAnimation_FaceType::FindMuscleGroupIndex'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::CreatePhonemeName
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_String* Tools_FacialAnimation_FaceType::CreatePhonemeName(const char* szName)
{
	Asura_String*	pxNewName	= new Asura_String(szName);

	m_xPhonemeNameContainer.PushTail(pxNewName);
	return(pxNewName);
}

/// End of function 'Tools_FacialAnimation_FaceType::CreatePhonemeName'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::DestroyPhonemeName
// Author       : Andy Tate
// Date         : 9-May-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_FacialAnimation_FaceType::DestroyPhonemeName(const char* szName)
{
	Asura_String*	pxName	= FindPhonemeName(szName);

	if(pxName == NULL)
	{
		Asura_Assertf(false, ("Failed to find phoneme-name \"%s\" to destroy.", szName));
		return(false);
	}
	else
	{
		m_xPhonemeNameContainer.RemoveFromList(pxName);
		delete(pxName);
		return(true);
	}
}

/// End of function 'Tools_FacialAnimation_FaceType::DestroyPhonemeName'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::SwapNames
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

void Tools_FacialAnimation_FaceType::SwapPhonemeNames(const Asura_String* pxName1,
													  const Asura_String* pxName2)
{
	const u_int		uNumNames	= m_xPhonemeNameContainer.GetNumEntries();
	Asura_String**	apxNames	= new Asura_String* [uNumNames];

	int				iIndex1		= -1;
	int				iIndex2		= -1;

	// Empty our list of names AND find the indices to be swapped...
	for(u_int u = 0; u < uNumNames; u++)
	{
		apxNames[u]	= m_xPhonemeNameContainer.PopHeadObject();

		if(apxNames[u] == pxName1)
		{
			iIndex1	= u;
		}
		else if(apxNames[u] == pxName2)
		{
			iIndex2	= u;
		}
	}

	// Swap our 2 muscle-groups over...
	if(iIndex1 < 0)
	{
		Asura_Assert(false, "Failed to find first name to be swapped.");
	}
	else if(iIndex2 < 0)
	{
		Asura_Assert(false, "Failed to find second name to be swapped.");
	}
	else
	{
		Asura_String*	pxTempName	= apxNames[iIndex1];

		apxNames[iIndex1]	= apxNames[iIndex2];
		apxNames[iIndex2]	= pxTempName;

		// Re-populate or muscle-groups...
		for(u_int u = 0; u < uNumNames; u++)
		{
			m_xPhonemeNameContainer.PushTail(apxNames[u]);
		}
	}

	// Finally, free-up the allocated data...
	delete[](apxNames);
}

/// End of function 'Tools_FacialAnimation_FaceType::SwapNames'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::FindPhonemeName
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

Asura_String* Tools_FacialAnimation_FaceType::FindPhonemeName(const char* szName)
{
	for(PhonemeNameIterator xIterator(m_xPhonemeNameContainer); !xIterator.Done(); xIterator.Next())
	{
		Asura_String*	pxName	= xIterator.GetCurrent();

		if(pxName != NULL)
		{
			if(pxName->Compare(szName, true) == 0)
			{
				return(pxName);
			}
		}
	}

	return(NULL);
}

/// End of function 'Tools_FacialAnimation_FaceType::FindPhonemeName'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::FindPhonemeName
// Author       : Andy Tate
// Date         : 27-May-2008
////////////////////////////////////////////////////////////////////////////////

const Asura_String* Tools_FacialAnimation_FaceType::FindPhonemeName(const char* szName) const
{
	for(PhonemeNameConstIterator xIterator(m_xPhonemeNameContainer); !xIterator.Done(); xIterator.Next())
	{
		const Asura_String*	pxName	= xIterator.GetCurrent();

		if(pxName != NULL)
		{
			if(pxName->Compare(szName, true) == 0)
			{
				return(pxName);
			}
		}
	}

	return(NULL);
}

/// End of function 'Tools_FacialAnimation_FaceType::FindPhonemeName'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::CreatePhonemeToFacePoseMapping
// Author       : Andy Tate
// Date         : 8-Jul-2008
////////////////////////////////////////////////////////////////////////////////

Asura_String* Tools_FacialAnimation_FaceType::CreatePhonemeToFacePoseMapping(const char* szPhonemeName,
																			 const Tools_FacialAnimation_FacePose& xPose)
{
	const Asura_Hash_ID	uPhonemeHash	= Asura_GetHashID(szPhonemeName);
	Asura_String*		pxFacePoseName	= m_xMappingContainer.GetDataByHash(uPhonemeHash);

	// If we already have this phoneme-mapping, delete it...
	if(pxFacePoseName != NULL)
	{
		m_xMappingContainer.DeleteNodeByHash(uPhonemeHash);
	}

	// Create the mapping...
	pxFacePoseName	= m_xMappingContainer.AddNode(uPhonemeHash);

	if(pxFacePoseName == NULL)
	{
		Asura_Assertf(false, ("Failed to create a face-pose mapping for phoneme \"%s\".", szPhonemeName));
		return(NULL);
	}
	else
	{
		*pxFacePoseName	= xPose.GetName();
		return(pxFacePoseName);
	}
}

/// End of function 'Tools_FacialAnimation_FaceType::CreatePhonemeToFacePoseMapping'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_FacialAnimation_FaceType::DestroyPhonemeToFacePoseMapping
// Author       : Andy Tate
// Date         : 8-Jul-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_FacialAnimation_FaceType::DestroyPhonemeToFacePoseMapping(const char* szPhonemeName)
{
	const Asura_Hash_ID	uPhonemeHash	= Asura_GetHashID(szPhonemeName);

	return(m_xMappingContainer.DeleteNodeByHash(uPhonemeHash));
}

/// End of function 'Tools_FacialAnimation_FaceType::DestroyPhonemeToFacePoseMapping'. ///

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


/// End of file. ///
