
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_FacialAnimation_MuscleGroupAnimation.h
// Author : Richard Rice
// Date   : 23-Sep-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_FACIALANIMATION_MUSCLEGROUPANIMATION_H__
#define __TOOLS_FACIALANIMATION_MUSCLEGROUPANIMATION_H__

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura_LinkList.h"
#include "FacialAnimation/Asura_FacialAnimation_MuscleGroupAnimation.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_Chunk_Stream;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_FacialAnimation_MuscleGroupAnimation
// Author       : Richard Rice
// Date         : 23-Sep-2008
//
// Description  : Class used to hold the animation data for a particular 
//				  muscle group (during facial-animation).
//                Essentially the same as the asura version , except it stores the 
//				  muscle group name as well				
////////////////////////////////////////////////////////////////////////////////

class Tools_FacialAnimation_MuscleGroupAnimation
{
public:
	// Public typedefs...
	typedef Asura_LinkList<Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame> KeyFrameContainer;
	typedef Asura_LinkList_It<Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame> KeyFrameIterator;
	typedef Asura_LinkList_ConstIt<Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame> KeyFrameConstIterator;

	// Public functions...
	Tools_FacialAnimation_MuscleGroupAnimation(void);
	Tools_FacialAnimation_MuscleGroupAnimation(const Asura_Hash_ID uHash, Asura_Chunk_Stream& xStream);
	//~Tools_FacialAnimation_MuscleGroupAnimation();

	// Chunk-stream functions...
	void	ReadFromChunkStream(Asura_Chunk_Stream& xStream);
	void	WriteToChunkStream(Asura_Chunk_Stream& xStream) const;

	// Manipulation functions...
	void	AddKeyFrame( float fTime, float fWeight, float fBalance);
	void	MergeSameTimeKeyFrames();
	void	CombineBaseFacePose(const float fBaseFacePoseWeight, const float fBaseFacePoseBalance);

	//Strips out and excess keyframes that can be generated by interpolating the frames either side
	void	RemoveRedundantKeyFrames();

	// Accessor functions...
	u_int	GetNumKeyFrames(void) const	{ return(m_xKeyFrameContainer.Size()); }

	const char* GetMuscleGroupName() const {return m_xMuscleGroupName.GetString();}
	void SetMuscleGroupName(const char* szName) {m_xMuscleGroupName = szName;}

	Asura_Hash_ID GetHashID() const {return m_xMuscleGroupName.GetHash();}

	KeyFrameContainer&			GetKeyFrameContainer(void)			{ return(m_xKeyFrameContainer); }
	const KeyFrameContainer&	GetKeyFrameContainer(void) const	{ return(m_xKeyFrameContainer); }

	float GetDuration() const;



protected:
	// Protected variables...

	// Protected functions...


private:
	// Private typedefs...
	typedef Asura_ListElt<Asura_FacialAnimation_MuscleGroupAnimation_KeyFrame> KeyFrameItem;

	// Private variables...
	Asura_String		m_xMuscleGroupName;		// The name of the muscle group being animated
	KeyFrameContainer	m_xKeyFrameContainer;	// Container class for the key-frames we're going to use.

	static const u_int	s_uChunkVersion;		// Current version used for reading/writing data.

	// Private functions...
	Tools_FacialAnimation_MuscleGroupAnimation(const Tools_FacialAnimation_MuscleGroupAnimation& xOther);

	Tools_FacialAnimation_MuscleGroupAnimation&	operator=(const Tools_FacialAnimation_MuscleGroupAnimation& xOther);

};   // End of class 'Tools_FacialAnimation_MuscleGroupAnimation'.

#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#endif   // ifndef __TOOLS_FACIALANIMATION_MUSCLEGROUPANIMATION_H__


/// End of file. ///
