#include "Core/Asura.h"

#include "Flags/Tools_Mode_Flags.h"

#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_SortAndSearch.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_Cursor.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_CollMeshGenData.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Surface.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Text_Output.h"


#include "Tools_Overlays.h"
#include "Tools_System.h"
#include "Flags/Tools_CollBox_Object.h"
#include "Lighting/Tools_GlobalIllumination_EntitySamples.h"
#include "Lighting/Tools_EnvironmentalLightTransport.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "MeshEdit/Tools_Mode_MeshEdit.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Adjacency.h"
#include "Misc/Tools_Render_Options.h"
#include "Misc/Tools_Render.h"
#include "Objects/Tools_Object_System.h"
#include "Rendering/Tools_BatchRenderPolygonOutline.h"
#include "Rendering/Tools_RenderModules.h"
#include "Rendering/Tools_RenderCollisionMesh.h"
#include "Texturing/Tools_Texturing_Find.h"

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
#include "Rendering/Asura_Project_Skin_Elements.h"
#endif


///////////////////////////////////////////////////////////////////////////////////////
/*--------------**
** Help Strings **
**--------------*/
static const char* FlagsModeHelpStrings [] =
{
	"F :           Apply the current flag",
	"Shift+F :     Remove the current flag",
	"Ctrl+Z :      Undo",
	"Ctrl+Y :      Redo",
	"< :           Undo",
	"> :           Redo",
	NULL
};
///////////////////////////////////////////////////////////////////////////////////////

#define NumberOfPolygonFlags 18 
static int g_aiPolygonFlagValues[NumberOfPolygonFlags] =
{
	AsuraPolygonFlag_FullBright,
	AsuraPolygonFlag_Darken,
	AsuraPolygonFlag_IgnoreAmbientLight,
	AsuraPolygonFlag_IgnoreNormalForLighting,
	AsuraPolygonFlag_SplayNormals,
	AsuraPolygonFlag_DontBlockLight,
	AsuraPolygonFlag_UseForEntityGISamples,
	AsuraPolygonFlag_Floor,
	AsuraPolygonFlag_IgnoreWhenWaypointing,
    AsuraPolygonFlag_DenseWaypoints,
	AsuraPolygonFlag_DontExport,
	AsuraPolygonFlag_DontRender,
	AsuraPolygonFlag_DoubleSided,
	AsuraPolygonFlag_IgnoreLOS,
	AsuraPolygonFlag_IgnoreForIK,
	AsuraPolygonFlag_Water,
	AsuraPolygonFlag_NoCollisions,
    AsuraPolygonFlag_Physics_WalkableSurface,
};


static char* g_aszPolygonFlagNames[NumberOfPolygonFlags] =
{
	"Full Bright",
	"Darken",
	"Ignore Ambient Light",
	"Ignore Normal For Lighting",
	"Splay Normals",
	"Don't Block Light",
	"Use For Entity GI Samples",
	"Use For Waypointing",
	"Ignore When Waypointing",
    "Create Dense Waypoints",
	"Don't Export",
	"Don't Render",
	"Doubled-Sided",
	"Ignored By Line-Of-Sight (Not IK)",
	"Ignored By IK L-O-S",
	"Water",
	"No Collisions",
    "Physics Walkable Surface",
};

static const char* g_aszPolygonFlagHelp[NumberOfPolygonFlags] =
{
	"Prelighting for this polygon is set to maximum normal brightness",
	"Prelighting is halved for this polygon",
	"Ambient light value isn't used for prelighting",
	"Light influence on this polygon isn't affected by the facing of the polygon relative to the light.\nA constant multiplier of 0.5 is used instead regardless of where the light comes from.\n(Prelighting only).",
	"Changes the vertex normal calculation so that they point more away from the centre of the polygon.",
	"Polygon doesn't block LOS tests when doing volumetric lighting.",
    "The vertices of the polygon are used to make samples of the GI to later light entities.",
	"Polygon is considered to be 'floor' for calculating waypoints",
	"Polygon is ignored when checking to see how far waypoints can extend before hitting something.",
    "Waypoints on these polys will obey the dense min and max waypoint radii settings.",
	"Slightly obsolete flag that is used to mark polygons that only exist for affecting volumetric prelighting.\n(Can be duplicated by combinations of the other flags).",
	"Polygon is invisible.",
	"Polygon is visible from both sides.",
	"Polygon doesn't block line of sight tests.",
	"Polygon doesn't block line of sight tests in IK system. (This flag is independent of the above Ignored By LOS flag). If there's no render mesh below the poly then IK will turn itself off.",
	"Polygons marked with this flag are used for defining the area covered by water.",
	"Polygon isn't for collisions.",
    "Indicates that actors can walk or step over this polygon.",
};

//no non project specific vertex flags yet
#define NumberOfVertexFlags 0

static int g_auVertexFlagValues[/*NumberOfVertexFlags*/] =
{
	0,
};

static const char* g_aszVertexFlagNames[/*NumberOfVertexFlags*/] =
{
	0,
};

static const char* g_aszVertexFlagHelp[/*NumberOfVertexFlags*/] = 
{
	0,
};

///////////////////////////////////////////////////////////////////////////////////////

//static member variables
Tools_Mode_Flags* 				Tools_Mode_Flags::s_pxThis = 0;

Asura_Vector_2					Tools_Mode_Flags::FlagsOverlayPosition = Asura_Vector_2(3.0f,8.0f);
bool							Tools_Mode_Flags::FlagsOverlayOpen = true;

Asura_Vector_2					Tools_Mode_Flags::s_xElementsOverlayPos= Asura_Vector_2(60.0f,3.0f);

int								Tools_Mode_Flags::s_iPolyFlagSelectionIndex;
int								Tools_Mode_Flags::s_iSelectedPolyFlags;

int								Tools_Mode_Flags::s_iVertFlagSelectionIndex;
u_int							Tools_Mode_Flags::s_uSelectedVertFlags;

int								Tools_Mode_Flags::s_iProjectPolygonDataSelectionIndex;
u_int							Tools_Mode_Flags::s_uProjectPolygonDataValue;

bool							Tools_Mode_Flags::s_bHighlightFlag = true;
bool							Tools_Mode_Flags::s_bAllowMultipleFlagSelection = false;
bool							Tools_Mode_Flags::s_bHighlightOnlyIfHaveAllFlags = false;
bool                            Tools_Mode_Flags::s_bSelectPhysicsWalkableWithFloorFlag = true;
bool							Tools_Mode_Flags::s_bShowHiddenPolysWithFlag = true;

Asura_Handle					Tools_Mode_Flags::MenuHandle;
FlagsModeUndo				 	Tools_Mode_Flags::FlagsUndo;

int								Tools_Mode_Flags::s_iNumberOfPolygonFlags;
char**							Tools_Mode_Flags::s_pszPolygonFlagNames;
char**							Tools_Mode_Flags::s_pszPolygonFlagHelp;
int*							Tools_Mode_Flags::s_piPolygonFlagValues;

int								Tools_Mode_Flags::s_iNumberOfVertexFlags;
char**							Tools_Mode_Flags::s_pszVertexFlagNames;
char**							Tools_Mode_Flags::s_pszVertexFlagHelp;
u_int*							Tools_Mode_Flags::s_puVertexFlagValues;

int								Tools_Mode_Flags::s_iNumberOfProjectPolygonDataValues;
char**							Tools_Mode_Flags::s_pszProjectPolygonDataNames;
char**							Tools_Mode_Flags::s_pszProjectPolygonDataHelp;

u_int							Tools_Mode_Flags::s_uSelectedTab = 0;
Tools_Mode_Flags::SubMode		Tools_Mode_Flags::s_eSubMode = Tools_Mode_Flags::SUBMODE_POLYGON_FLAGS;
Tools_Mode_Flags::SubMode		Tools_Mode_Flags::s_aeTabToSubMode[];

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
int								Tools_Mode_Flags::s_iElementSelectionIndex = 0;
#endif


Tools_Mode_Flags::Tools_Mode_Flags()
{
	Asura_Assert(!s_pxThis,"");
	s_pxThis = this;
}

Tools_Mode_Flags::~Tools_Mode_Flags()
{
	Asura_Assert(s_pxThis,"");
	s_pxThis = 0;
}


void Tools_Mode_Flags::InitialiseMode()
{
	//Initalise polygon selection
	Tools_Polygon_Selection::Activate();
	Tools_Vertex_Selection::Activate();

	//create the menubar
	MenuHandle = Asura_Menubar_System::CreateMenuSection();
	Asura_Menubar_Title* title;

	title = Asura_Menubar_System::AddTitle(MenuHandle,"Overlays");
	title->AddItem("Flags",Tools_Mode_Flags::ToggleFlagsOverlay,Tools_Mode_Flags::FlagsOverlayIsOn);
	title->AddItem("Selection",Tools_Polygon_Selection::ToggleSelectionOverlay,Tools_Polygon_Selection::SelectionOverlayIsOn);
	title->AddItem("Vert Selection",Tools_Vertex_Selection::ToggleSelectionOverlay,Tools_Vertex_Selection::SelectionOverlayIsOn);
	title->AddItem("Find",Tools_Texturing_Find::ToggleFindOverlay,Tools_Texturing_Find::FindOverlayIsOn);

	//rest of intialisation is common with other modes doing flags
	InitialiseExMode(0);
}

void Tools_Mode_Flags::InitialiseExMode(int iDefaultFlagValue)
{
	FlagsUndo.Initialise();

	//load values from .dat file
	TransferModeData(TOOLSMODEDATAFILE_LOAD);

	BuildPolygonFlagArrays();
	BuildVertexFlagArrays();
	BuildProjectPolygonDataArrays();

	if(iDefaultFlagValue)
	{
		//find the index of the default flag to select
		for(s_iPolyFlagSelectionIndex=0;s_iPolyFlagSelectionIndex<s_iNumberOfPolygonFlags-1;s_iPolyFlagSelectionIndex++)
		{
			if(s_piPolygonFlagValues[s_iPolyFlagSelectionIndex] == iDefaultFlagValue) break;
		}	
	}
	if(s_iPolyFlagSelectionIndex<0 || s_iPolyFlagSelectionIndex>=s_iNumberOfPolygonFlags)
	{
		s_iPolyFlagSelectionIndex = 0;
	}
	s_iSelectedPolyFlags = s_piPolygonFlagValues[s_iPolyFlagSelectionIndex];

	if ( s_iSelectedPolyFlags == AsuraPolygonFlag_Floor && s_bSelectPhysicsWalkableWithFloorFlag )
	{
		// Walkable surface is selected by default with the floor flag.
		s_iSelectedPolyFlags |= AsuraPolygonFlag_Physics_WalkableSurface;
	}


	if(s_iNumberOfVertexFlags)
	{
		if(s_iVertFlagSelectionIndex<0 || s_iVertFlagSelectionIndex>=s_iNumberOfVertexFlags)
		{
			s_iVertFlagSelectionIndex = 0;
		}
		s_uSelectedVertFlags = s_puVertexFlagValues[s_iVertFlagSelectionIndex];
	}

	s_eSubMode = SUBMODE_POLYGON_FLAGS;

	//create overlays
	CreateFlagsOverlay();

	//add help
	Tools_Help::AddHelpTopic("Flags Mode",FlagsModeHelpStrings);

}


void Tools_Mode_Flags::InitialiseForGI()
{
	//Initalise polygon selection
	FlagsUndo.Initialise();


	BuildPolygonFlagArrays();


	//reinitiaise stuff
	for(s_iPolyFlagSelectionIndex=0;s_iPolyFlagSelectionIndex<s_iNumberOfPolygonFlags-1;s_iPolyFlagSelectionIndex++)
	{
		if(s_piPolygonFlagValues[s_iPolyFlagSelectionIndex] == AsuraPolygonFlag_UseForEntityGISamples) break;
	}	
	Asura_Assert(s_piPolygonFlagValues[s_iPolyFlagSelectionIndex] == AsuraPolygonFlag_UseForEntityGISamples, "Initialised GI Sample flag incorrectly");
	s_iSelectedPolyFlags = AsuraPolygonFlag_UseForEntityGISamples;
	s_eSubMode = SUBMODE_POLYGON_FLAGS;
}

void Tools_Mode_Flags::ShutdownMode()
{
	Tools_Polygon_Selection::Deactivate();
	Tools_Vertex_Selection::Deactivate();

	//shutdown the menu
	if(MenuHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Menubar_System::DestroyMenuSection(MenuHandle);
		MenuHandle = ASURA_INVALID_HANDLE;
	}

	// (SB - 13/06/08) Reset the render mode when we leave flagging mode
	// Leaving it on confused people and made them think they were still in flags mode an export
	Tools_RenderCollisionMesh_System::SetRenderMode( TOOLS_COLLRENDER_NONE );

	ShutdownExMode();
}

void Tools_Mode_Flags::ShutdownExMode()
{
	Tools_Help::RemoveHelpTopic("Flags Mode");
	FlagsUndo.Shutdown();

	//save values to .dat file
	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	//reactivate the DontRender flag if it was disabled
	Tools_Render_Options::SetAlwaysVisiblePolygonFlags(0);

	Tools_CollBox_Object::DeleteAll();

	//close overlays
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_FLAGS);

	DeleteProjectPolygonDataArrays();
	DeletePolygonFlagArrays();
	DeleteVertexFlagArrays();
}

void Tools_Mode_Flags::ShutdownForGI()
{
	FlagsUndo.Shutdown();

	DeletePolygonFlagArrays();

}

void Tools_Mode_Flags::TransferModeData(int mode)
{
	//open the file
	Tools_ModeDataFile file("FlagsMode.dat",mode);

	//transfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	file.Transfer(FlagsOverlayPosition);
	file.Transfer(FlagsOverlayOpen);
}

void Tools_Mode_Flags::Update(void)
{
	SubMode eOldSubMode = s_eSubMode;
	
	DetermineCurrentSubMode();
	if(s_eSubMode == SUBMODE_PROJECTDATA)
	{
		EnsureProjectPolygonDataIsInRange();
	}

	//update visibility of selection overlays
	bool bVertexSelection = (s_eSubMode == SUBMODE_VERTEX_FLAGS);
 	if(bVertexSelection)
	{
		Tools_Vertex_Selection::CreateSelectionOverlay();
		Tools_Polygon_Selection::CloseSelectionOverlay();
	}
	else
	{
		Tools_Vertex_Selection::CloseSelectionOverlay();
		Tools_Polygon_Selection::CreateSelectionOverlay();
	}

	if(eOldSubMode != s_eSubMode)
	{
		//Bring the flags overlay back to the top , since it may have been displaced by the creation of
		//one of the selection overlays
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_FLAGS);
	}

	UpdateExMode( true );
}

void Tools_Mode_Flags::UpdateExMode( const bool bUpdateSelection,
									 const int iPolygonSelectionUpdateMode /*= PolygonSelectionUpdate_Standard*/,
									 const int iVertexSelectionUpdateMode /*= VertexSelectionUpdate_Standard*/ )
{
	//Flag the undo to start a new undo batch
	FlagsUndo.MarkUndoStart();

	bool bVertexSelection = (s_eSubMode == SUBMODE_VERTEX_FLAGS);

	//Tell render options to always draw polygons with the current flags
	if(Tools_RenderCollisionMesh_System::IsRenderingCollMeshes() || !s_bShowHiddenPolysWithFlag)
	{
		Tools_Render_Options::SetAlwaysVisiblePolygonFlags(0);
	}
	else
	{
		Tools_Render_Options::SetAlwaysVisiblePolygonFlags(s_iSelectedPolyFlags);
	}

	if(Tools_Object_System::SelectedObject)
	{
		if(bUpdateSelection)
		{
			if(bVertexSelection)
			{
				Tools_Vertex_Selection::Update(VertexSelectionUpdate_NoSelection);
			}
			else
			{
				Tools_Polygon_Selection::Update(PolygonSelectionUpdate_NoSelection);
			}
		}
	}
	else
	{
		Tools_CollBox_Object::DeleteAll();
		
		if(bUpdateSelection)
		{
			if(bVertexSelection)
			{
				Tools_Vertex_Selection::Update( iVertexSelectionUpdateMode );
			}
			else
			{
				Tools_Polygon_Selection::Update( iPolygonSelectionUpdateMode );
			}
		}
	
		//act upon keyboard input
		Tools_Mode_Flags::HandleKeyboard();
	}
}

void Tools_Mode_Flags::UpdateForGI()
{
	//Flag the undo to start a new undo batch
	FlagsUndo.MarkUndoStart();

	//act upon keyboard input
	Tools_Mode_Flags::HandleKeyboard();
}

void Tools_Mode_Flags::RenderWorld_PostToneMap(void)
{

	if(s_eSubMode == SUBMODE_POLYGON_FLAGS)
	{
		if(s_bHighlightFlag) HighlightPolyFlags();
		Tools_Polygon_Selection::Render();
	}
	else if(s_eSubMode == SUBMODE_VERTEX_FLAGS)
	{
		if(s_bHighlightFlag) HighlightVertFlags();
		Tools_Vertex_Selection::Render();
	}
	else if(s_eSubMode == SUBMODE_PROJECTDATA)
	{
		if(s_bHighlightFlag) HighlightProjectPolygonData();
		Tools_Polygon_Selection::Render();
	}
	else if(s_eSubMode == SUBMODE_ELEMENTS)
	{
		Tools_Polygon_Selection::Render();
	}
	else
	{
		Asura_Assert(0,"Unexpected submode");
	}

}

void Tools_Mode_Flags::RenderUI(void)
{
	float fPrintX = 1.0f;
	float fPrintY = 3.0f;
	
	if(s_eSubMode == SUBMODE_POLYGON_FLAGS)
	{
		RenderPolyFlagText(fPrintX,fPrintY);
		Tools_Polygon_Selection::RenderUI();
	}
	else if(s_eSubMode == SUBMODE_VERTEX_FLAGS)
	{
		RenderVertFlagText(fPrintX,fPrintY);
	}
	else if(s_eSubMode == SUBMODE_PROJECTDATA)
	{
		RenderProjectPolygonDataText(fPrintX,fPrintY);
		Tools_Polygon_Selection::RenderUI();
	}
	else if(s_eSubMode == SUBMODE_ELEMENTS)
	{
		Tools_Polygon_Selection::RenderUI();
	}
	else
	{
		Asura_Assert(0,"Unexpected submode");
	}

}

void Tools_Mode_Flags::RenderPolyFlagText(float& fPrintX,float& fPrintY)
{
	//show the flags currently on the polygon under the cursor
	PolygonID polygon;
	if(Tools_Polygon_Selection::GetHighlitPolygon(polygon))
	{
		Asura_Text_Output::Print(fPrintX,fPrintY++,0xffffff00,0,"Current Flags :");
		++fPrintX;

		Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(polygon.ModuleIndex);
		int flags = 0;
		if(mesh->PolygonFlagDataValid) flags = mesh->PolygonFlagDataPtr[polygon.PolyIndex].Flags;

		if(flags)
		{
			for(int i=0;i<s_iNumberOfPolygonFlags;i++)
			{
				if(flags & s_piPolygonFlagValues[i])
				{
					Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,0,"%s",s_pszPolygonFlagNames[i]);
				}
			}
			//step needs to be detected seperately , since it isn't a flag that gets applied directly
			if(flags & AsuraPolygonFlag_Step)
			{
				Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,0,"Step");
			}

		}
		else
		{
			Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,0,"None");
		}

        --fPrintX;
	}
}

void Tools_Mode_Flags::RenderVertFlagText(float& fPrintX,float& fPrintY)
{
	//show the flags currently on the polygon under the cursor
	ModuleVertexPair xVertex;
	if(Tools_Vertex_Selection::GetHighlitVertex(xVertex))
	{
		Asura_Text_Output::Print(fPrintX,fPrintY++,0xffffff00,0,"Current Flags :");
		fPrintX++;

		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(xVertex.iModuleIndex);
		u_int uFlags = 0;
		uFlags = pxMesh->GetVertexFlags(xVertex.iVertexIndex);

		if(uFlags)
		{
			for(int i=0;i<s_iNumberOfVertexFlags;i++)
			{
				if(uFlags & s_puVertexFlagValues[i])
				{
					Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,0,"%s",s_pszVertexFlagNames[i]);
				}
			}
		}
		else
		{
			Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,0,"None");
		}
	
	}
}

void Tools_Mode_Flags::RenderProjectPolygonDataText(float& fPrintX,float& fPrintY)
{
	if(!s_pxThis) return;

	//show the values for the polygon under the cursor
	PolygonID xPolygon;
	if(Tools_Polygon_Selection::GetHighlitPolygon(xPolygon))
	{
		Asura_Text_Output::Print(fPrintX,fPrintY++,0xffffff00,0,"Current Project Data :");
		fPrintX++;

		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(xPolygon.ModuleIndex);

		u_int uData = pxMesh->GetProjectPolygonData(xPolygon.PolyIndex);

		for(int i=0;i<s_iNumberOfProjectPolygonDataValues;i++)
		{
			Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,0,"%s : %d",s_pszProjectPolygonDataNames[i],s_pxThis->GetValueFromProjectPolygonData(i,uData));
		}
	}
}


bool Tools_Mode_Flags::IsModified()
{
	return Tools_SaveAsura::IsMainFileModified();
}


void Tools_Mode_Flags::HandleKeyboard()
{
	if(Asura_Overlay_System::OverlaysHaveKeyboardFocus()) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
 	{
		if(Asura_UserInput::DebouncedKeyPressed('z'))// 'ctrl+z' for  undo
		{
			Undo();
		}
		if(Asura_UserInput::DebouncedKeyPressed('y'))// 'ctrl+y' for  redo
		{
			Redo();
		}
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
		if(Asura_UserInput::DebouncedKeyPressed('f'))// 'Shift + F' to remove a flag
		{
			RemoveFlag();
		}

	}
	else
	{
		if(Asura_UserInput::DebouncedKeyPressed('f'))// 'F' to apply a flag
		{
			ApplyFlag();
		}
		if(Asura_UserInput::DebouncedKeyPressed(','))// 'ctrl+z' or '<' for  undo
		{
			Undo();
		}
		if(Asura_UserInput::DebouncedKeyPressed('.'))// 'ctrl+y' or '>' for  redo
		{
			Redo();
		}

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		if(Asura_UserInput::DebouncedKeyPressed('e'))
		{
			if (s_eSubMode == SUBMODE_ELEMENTS)
			{			
				ApplyElement();
			}
		}
#endif
	}
	// (SB - 12/12/06) Give the project a chance to intercept keyboard events
	if (s_pxThis != NULL)
	{
		s_pxThis->ProjectHandleKeyboard();
	}
}


void Tools_Mode_Flags::ToggleFlagsOverlay(void)
{
	FlagsOverlayOpen = !FlagsOverlayOpen;

	if (FlagsOverlayOpen)
	{
		CreateFlagsOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_FLAGS);
	}
}
bool Tools_Mode_Flags::FlagsOverlayIsOn(void)
{
	return FlagsOverlayOpen;
}

void Tools_Mode_Flags::CreateFlagsOverlay(void)
{
	if(!FlagsOverlayOpen) return;

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Flags",false,&FlagsOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
			

	//determine the maximum number of lines to use for the flag lists
	float fMaxPrintY = Asura_Viewport::GetHeight() / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const int iMaxLines = Asura_Maths::Max(4,Asura_Maths::FloatToInt_Chop((fMaxPrintY - 35.0f)/1.2f));

	//Set up Tabs

	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset,0,0);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxOverlay->SetCurrentParentDialog(pxTabsDialog);
	int iTabIndex = 0;

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;
	
	//Polygon flag tab
	pxTabsDialog->AddTab("Poly Flags");
	s_aeTabToSubMode[iTabIndex++] = SUBMODE_POLYGON_FLAGS;
	
	//add the flag list
	int iLines = Asura_Maths::Min(s_iNumberOfPolygonFlags,iMaxLines);
	
	pxDialog = new Asura_List_Dialog(xOffset,s_pszPolygonFlagNames,&s_iPolyFlagSelectionIndex,iLines,s_iNumberOfPolygonFlags,IsPolyFlagSelected,TogglePolyFlagSelection,0);
	((Asura_List_Dialog*)pxDialog)->SetListHelpStrings(s_pszPolygonFlagHelp);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y=pxDialog->m_xMaxCoord.y;
	xOffset.y+=1.0f;

	// (SB - 12/12/06) Allow project specific items to be added to the flags mode tabs
	if (s_pxThis != NULL)
	{
		s_pxThis->ProjectAddItemsToFlagsPane(pxOverlay, xOffset);
	}

	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	if(Tools_System::IsInFlagsMode())
	{
		//Vertex flag tab
		if(s_iNumberOfVertexFlags)
		{
			pxTabsDialog->AddTab("Vert Flags");
			s_aeTabToSubMode[iTabIndex++] = SUBMODE_VERTEX_FLAGS;
			xOffset.y = fTabsStartY;
	
			int iLines = Asura_Maths::Min(s_iNumberOfVertexFlags,iMaxLines);

			// (SB - 12/12/06) Allow project specific items to be added to the flags mode tabs
			if (s_pxThis != NULL)
			{
				s_pxThis->ProjectAddItemsToVertexFlagsPaneEarly(pxOverlay, xOffset);
			}

			pxDialog = new Asura_List_Dialog(xOffset,s_pszVertexFlagNames,&s_iVertFlagSelectionIndex,iLines,s_iNumberOfVertexFlags,IsVertFlagSelected,ToggleVertFlagSelection,0);
			((Asura_List_Dialog*)pxDialog)->SetListHelpStrings(s_pszVertexFlagHelp);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y=pxDialog->m_xMaxCoord.y;
			xOffset.y+=1.0f;

			// (SB - 12/12/06) Allow project specific items to be added to the flags mode tabs
			if (s_pxThis != NULL)
			{
				s_pxThis->ProjectAddItemsToVertexFlagsPane(pxOverlay, xOffset);
			}

			if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;
		}

		//Project polygon data tab
		if(s_iNumberOfProjectPolygonDataValues)
		{
			pxTabsDialog->AddTab("Poly Data");
			s_aeTabToSubMode[iTabIndex++] = SUBMODE_PROJECTDATA;

			xOffset.y = fTabsStartY;
	
			int iLines = Asura_Maths::Min(s_iNumberOfProjectPolygonDataValues,iMaxLines);

			pxDialog = new Asura_List_Dialog(xOffset,s_pszProjectPolygonDataNames,&s_iProjectPolygonDataSelectionIndex,iLines,s_iNumberOfProjectPolygonDataValues,NULL);
			((Asura_List_Dialog*)pxDialog)->SetListHelpStrings(s_pszProjectPolygonDataHelp);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y=pxDialog->m_xMaxCoord.y;
			xOffset.y+=1.0f;

			// (SB - 12/12/06) Allow project specific items to be added to the flags mode tabs
			if (s_pxThis != NULL)
			{
				s_pxThis->ProjectAddItemsToPolyFlagsPaneEarly(pxOverlay, xOffset, s_iProjectPolygonDataSelectionIndex);
			}

			pxDialog = new Asura_Numerical_Dialog("Value",xOffset,&s_uProjectPolygonDataValue,0,0x7fffffff,1,"%d");
			pxDialog->SetHelpText("The value that will be set for this piece of data when using 'Apply Flag'.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			// (SB - 12/12/06) Allow project specific items to be added to the flags mode tabs
			if (s_pxThis != NULL)
			{
				s_pxThis->ProjectAddItemsToPolyFlagsPane(pxOverlay, xOffset, s_iProjectPolygonDataSelectionIndex);
			}

			if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;
		}

#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		// Add the elements tab
		pxTabsDialog->AddTab("Elements");
		s_aeTabToSubMode[iTabIndex++] = SUBMODE_ELEMENTS;

		xOffset.y = fTabsStartY;
		pxDialog = new Asura_List_Dialog(xOffset,Asura_Project_Skin_Elements::GetElementTypeNames(),
										&s_iElementSelectionIndex,14,Asura_Project_Skin_Elements::GetNumElementTypes(),NULL,NULL,0);

		pxOverlay->AddDialog(pxDialog);
		xOffset.y = pxDialog->m_xMaxCoord.y;
		xOffset.y += 1.0f;

		pxDialog = new Asura_Function_Dialog("Assign To Element (E)",xOffset,Tools_Mode_Flags::ApplyElement);
		pxDialog->SetHelpText("Assign selected polygons/vertices to the selected element");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();		
		
		if( xOffset.y > fTabsEndY ) 
		{
			fTabsEndY = xOffset.y;
		}
#endif
	}

	//end of tabs
	xOffset.y = fTabsEndY + Asura_Dialog::DefaultSpacing();
	pxOverlay->SetCurrentParentDialog(0);
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);

	Asura_Assert(iTabIndex <= sizeof(s_aeTabToSubMode)/sizeof(SubMode),"Array not large enough");


	pxDialog = new Asura_Tickbox_Dialog("Highlight Flag",xOffset,&s_bHighlightFlag);
	pxDialog->SetHelpText("Highlights all polygons/vertices that are using the selected flag.");
    pxDialog->SetEnabled(!Tools_System::IsInWaypointingMode());
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+= Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Multiple Flag Selection",xOffset,&s_bAllowMultipleFlagSelection);
	pxDialog->SetHelpText("Allows multiple flags from the above list to be selected at a time.\nIf not ticked , then the tickboxes will act as radio buttons.");
	pxDialog->SetIsEnabledFunction(IsInFlagSubMode);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Highlight Only If All Flags",xOffset,&s_bHighlightOnlyIfHaveAllFlags);
	pxDialog->SetHelpText("Only highlights polygons that have all the selected flags.\nIf not ticked then polygons with just some of the flags will be shown in a different colour.");
	pxDialog->SetIsEnabledFunction(IsInFlagSubMode);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Hidden Polys With Flag",xOffset,&s_bShowHiddenPolysWithFlag);
	pxDialog->SetHelpText("Causes polygons with the current flags to be shown regardless of the don't render flag.");
	pxDialog->SetIsEnabledFunction(IsInFlagSubMode);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();


	
	pxDialog = new Asura_Function_Dialog("Apply Flag(F)",xOffset,Tools_Mode_Flags::ApplyFlag);
	pxDialog->SetHelpText("Add the selected flag(s) to all selected polygons/vertices.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove Flag (Shift+F)",xOffset,Tools_Mode_Flags::RemoveFlag);
	pxDialog->SetHelpText("Remove the selected flag(s) from all selected polygons.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Remove All Flags",xOffset,Tools_Mode_Flags::ClearFlags);
	pxDialog->SetHelpText("Removes all flags from all selected polygons/vertices.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	//coll mesh rendering
	Tools_RenderCollisionMesh_System::AddRenderModeOverlayControls(pxOverlay,xOffset);


	pxDialog = new Asura_Function_Dialog("Undo (Ctrl+Z)",xOffset,Tools_Mode_Flags::Undo);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Function_Dialog("Redo (Ctrl+Y)",xOffset,Tools_Mode_Flags::Redo);
	pxOverlay->AddDialog(pxDialog);

	
	
	pxOverlay->m_uID = TOOLS_OVERLAY_FLAGS;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}


void Tools_Mode_Flags::ApplyFlag()
{
	if(s_eSubMode == SUBMODE_POLYGON_FLAGS)
	{
		ApplyPolyFlag();
	}
	else if(s_eSubMode == SUBMODE_VERTEX_FLAGS)
	{
		ApplyVertFlag();
	}
	else if(s_eSubMode == SUBMODE_PROJECTDATA)
	{
		ApplyProjectPolygonData();
	}
	else
	{
		Asura_Assert(0,"Unexepected submode");
	}
}

void Tools_Mode_Flags::ApplyPolyFlag()
{
	if(!Tools_Polygon_Selection::NumSelectedModules)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No polygons have been selected");
		return;
	}

	SetPolyFlag(s_iSelectedPolyFlags,true);

	int iIndex;
	for(iIndex = 0;iIndex<s_iNumberOfPolygonFlags;iIndex++)
	{
		if(s_piPolygonFlagValues[iIndex] == s_iSelectedPolyFlags) break;
	}
	
	if(iIndex<s_iNumberOfPolygonFlags)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Added '%s'",s_pszPolygonFlagNames[iIndex]);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Added selected flags");
	}
}

void Tools_Mode_Flags::ApplyVertFlag()
{
	if(!Tools_Vertex_Selection::s_iNumberOfModules)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No vertices have been selected");
		return;
	}

	SetVertFlag(s_uSelectedVertFlags,true);

	int iIndex;
	for(iIndex = 0;iIndex<s_iNumberOfVertexFlags;iIndex++)
	{
		if(s_puVertexFlagValues[iIndex] == s_uSelectedVertFlags) break;
	}
	
	if(iIndex<s_iNumberOfVertexFlags)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Added '%s'",s_pszVertexFlagNames[iIndex]);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Added selected flags");
	}
}

void Tools_Mode_Flags::ApplyProjectPolygonData()
{
	if(!Tools_Polygon_Selection::NumSelectedModules)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No polygons have been selected");
		return;
	}

	EnsureProjectPolygonDataIsInRange();
	SetProjectPolygonData(s_iProjectPolygonDataSelectionIndex,s_uProjectPolygonDataValue);
	
	Tools_Timed_Message_System::AddMessage(0xffffff,"Set '%s' to %d",s_pszProjectPolygonDataNames[s_iProjectPolygonDataSelectionIndex],s_uProjectPolygonDataValue);
	
}


void Tools_Mode_Flags::RemoveFlag()
{
	if(s_eSubMode == SUBMODE_POLYGON_FLAGS)
	{
		RemovePolyFlag();
	}
	else if(s_eSubMode == SUBMODE_VERTEX_FLAGS)
	{
		RemoveVertFlag();
	}
	else if(s_eSubMode == SUBMODE_PROJECTDATA)
	{
		RemoveProjectPolygonData();
	}
	else
	{
		Asura_Assert(0,"Unexepected submode");
	}
}

void Tools_Mode_Flags::RemovePolyFlag()
{
	if(!Tools_Polygon_Selection::NumSelectedModules)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No polygons have been selected");
		return;
	}

	SetPolyFlag(s_iSelectedPolyFlags,false);

	int iIndex;
	for(iIndex = 0;iIndex<s_iNumberOfPolygonFlags;iIndex++)
	{
		if(s_piPolygonFlagValues[iIndex] == s_iSelectedPolyFlags) break;
	}
	
	if(iIndex<s_iNumberOfPolygonFlags)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Removed '%s'",s_pszPolygonFlagNames[iIndex]);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Removed selected flags");
	}
}

void Tools_Mode_Flags::RemoveVertFlag()
{
	if(!Tools_Vertex_Selection::s_iNumSelectedModules)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No vertices have been selected");
		return;
	}

	SetVertFlag(s_uSelectedVertFlags,false);

	int iIndex;
	for(iIndex = 0;iIndex<s_iNumberOfVertexFlags;iIndex++)
	{
		if(s_puVertexFlagValues[iIndex] == s_uSelectedVertFlags) break;
	}
	
	if(iIndex<s_iNumberOfVertexFlags)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Removed '%s'",s_pszVertexFlagNames[iIndex]);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Removed selected flags");
	}
}

void Tools_Mode_Flags::RemoveProjectPolygonData()
{
	if(!Tools_Polygon_Selection::NumSelectedModules)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No polygons have been selected");
		return;
	}

	SetProjectPolygonData(s_iProjectPolygonDataSelectionIndex,0);
	
	Tools_Timed_Message_System::AddMessage(0xffffff,"Reset '%s' to 0",s_pszProjectPolygonDataNames[s_iProjectPolygonDataSelectionIndex]);
}



void Tools_Mode_Flags::ClearFlags()
{
	if(s_eSubMode == SUBMODE_POLYGON_FLAGS)
	{
		ClearPolyFlags();
	}
	else if(s_eSubMode == SUBMODE_VERTEX_FLAGS)
	{
		ClearVertFlags();
	}
	else if(s_eSubMode == SUBMODE_PROJECTDATA)
	{
		ClearProjectPolygonData();
	}
	else
	{
		Asura_Assert(0,"Unexepected submode");
	}
}

void Tools_Mode_Flags::ClearPolyFlags()
{
	if(!Tools_Polygon_Selection::NumSelectedModules)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No polygons have been selected");
		return;
	}

	SetPolyFlag(0xffffffff,false);

	Tools_Timed_Message_System::AddMessage(0xffffff,"Removed all flags from selection");
}

void Tools_Mode_Flags::ClearVertFlags()
{
	if(!Tools_Vertex_Selection::s_iNumSelectedModules)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No vertices have been selected");
		return;
	}

	SetVertFlag(0xffffffff,false);

	Tools_Timed_Message_System::AddMessage(0xffffff,"Removed all flags from selection");
}

void Tools_Mode_Flags::ClearProjectPolygonData()
{
	if(!Tools_Polygon_Selection::NumSelectedModules)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No polygons have been selected");
		return;
	}

	SetProjectPolygonData(0xffffffff,0);

	Tools_Timed_Message_System::AddMessage(0xffffff,"Reset all project data to zero");
}



static void RegenerateStepFlagsForAdjacentPolys( int iModuleIdx, int iPolyIdx )
{
    // We need to flag the tops of steps with the step flag as well as the
    // vertical polys. We'll do this by taking all these vertical step polys
    // and then adding the step flag to all floor polys inbetween them.

    // Get this poly's mesh.
    const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule( iModuleIdx );
    if( !pxMesh->PolygonFlagDataValid ) { return; }

    // Skip non-step polys.
    if( !(pxMesh->PolygonFlagDataPtr[iPolyIdx].Flags & AsuraPolygonFlag_Step) ) { return; }

    // Find adjacencies for this poly.
    const Tools_Polygon_Adjacency* pxAdjList = Tools_Adjacency::GetPolygonAdjacencies( iModuleIdx, iPolyIdx );
    
    // Iterate through them and look for polys with the floor flag
    // but no step flag.
    for( const Tools_Adjacent_Polygon* pxAdj = pxAdjList->GetFirstAdjacency(); pxAdj; pxAdj = pxAdj->GetNext() )
    {

        // Only interested in polys sharing an edge at least.
        if( pxAdj->GetNumSharedVertices() < 2 ) { continue; }

	   	// RAR 02/05/2006 Ignore adjacencies between streaming zones for now
	   	if (pxAdj->IsBetweenZones()) continue;
	
		const int iAdjModuleIndex = pxAdj->GetModuleIndex();
		const int iAdjPolygonIndex = pxAdj->GetPolygonIndex();
        
        
        Asura_Mesh* pxOtherMesh = Asura_Module_System::GetMeshForModule( iAdjModuleIndex );
        if( !pxOtherMesh->PolygonFlagDataValid ) { continue; }

        // Only interested in adjacent polys that have the floor
        // flag set and that we haven't already added the step flag
        // to.
        {
            const int iFlags = pxOtherMesh->PolygonFlagDataPtr[iAdjPolygonIndex].Flags;
            if( !(iFlags & AsuraPolygonFlag_Floor) || (iFlags & AsuraPolygonFlag_Step) )
            {
                continue;
            }
        }

        // Okay, now we need to see if this other poly has *another*
        // adjacent step poly. Basically we only want the flag the
        // tops of proper steps, ie. in the following
        // 
        //     A _________
        //                |_
        //                  |________  
        //                            B
        //
        // the polys level with A and B shouldn't be flagged but the
        // other middle horizontal polys should be.
        bool bAddStepFlag = false;
        const Tools_Polygon_Adjacency* pxAdjList2 = Tools_Adjacency::GetPolygonAdjacencies( iAdjModuleIndex, iAdjPolygonIndex );
	    for( const Tools_Adjacent_Polygon* pxAdj2 = pxAdjList2->GetFirstAdjacency(); pxAdj2; pxAdj2 = pxAdj2->GetNext() )
        {
		   	// RAR 02/05/2006 Ignore adjacencies between streaming zones for now
		   	if (pxAdj2->IsBetweenZones()) continue;

			const int iAdjModuleIndex2 = pxAdj->GetModuleIndex();
			const int iAdjPolygonIndex2 = pxAdj->GetPolygonIndex();
            
            // Don't count the original poly.
            if( iAdjModuleIndex2 == iModuleIdx && iAdjPolygonIndex2 == iPolyIdx )
            {
                continue;
            }

            // Check enough vertices are shared.
            if( pxAdj->GetNumSharedVertices() < 2 ) { continue; }

            Asura_Mesh* pxOtherMesh2 = Asura_Module_System::GetMeshForModule( iAdjModuleIndex2 );
            if( !pxOtherMesh2->PolygonFlagDataValid ) { continue; }

            // Check this other poly is a step one. Note that we
            // require a true step one, as opposed to some other
            // floor poly we've added the step flag to.
            {
                const int iFlags = pxOtherMesh2->PolygonFlagDataPtr[iAdjPolygonIndex2].Flags;
                if( !(iFlags & AsuraPolygonFlag_Step) || (iFlags & AsuraPolygonFlag_Floor) )
                {
                    continue;
                }
            }

            // Ok, seems the poly adjacent to the original is between
            // two step polys and so needs flagging itself.
            bAddStepFlag = true;
            break;
        }

        // So, does this poly need the flag?
        if( bAddStepFlag )
        {
            pxOtherMesh->PolygonFlagDataPtr[iAdjPolygonIndex].Flags |= AsuraPolygonFlag_Step;
			//changes to polygon flags may result in the polygon being hidden/unhidden
			Tools_Render_Options::UpdatePolygonVisibility(iAdjModuleIndex,iAdjPolygonIndex);

			Tools_Mode_MeshEdit::InvalidateModuleCollisionMesh(iAdjModuleIndex);
        }
    }
    // End loop through adjacent polys.
}

void Tools_Mode_Flags::SetPolyFlag( int iFlag, bool bAdd )
{
	//log for undo
	PrepareUndo_PolyFlag();

	PolygonID polygon;

	Tools_Polygon_Selection::StartIteration();
	while(Tools_Polygon_Selection::GetNextSelected(&polygon))
	{
		Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(polygon.ModuleIndex);
	  	if(!mesh->PolygonFlagDataValid) mesh->AllocatePolygonFlags();
		if(!mesh->m_bFaceNormalsValid) mesh->CreateFaceNormals();

		int iFlagToSet = iFlag;
			
		if( AsuraPolygonFlag_Floor & iFlag )
		{
            if( bAdd )
            {
			    // Only one out of floor/step should be set on this pass. The
                // tops of steps will get the step flag set in the second pass
                // below.
//			    mesh->PolygonFlagDataPtr[polygon.PolyIndex].Flags &=~ AsuraPolygonFlag_Floor|AsuraPolygonFlag_Step;
			    if(mesh->NormalsDataPtr[polygon.PolyIndex].FaceNormal.y>TOOLS_MAX_FLOOR_NORMAL)
			    {
				    //set step flag instead if normal is high enough
				    iFlagToSet |= AsuraPolygonFlag_Step;
			    }
            }
            else
            {
                // If we're removing the floor flag then we need to remove
                // step flags at the same time.
                iFlagToSet |= AsuraPolygonFlag_Step;
            }
		}
		if( AsuraPolygonFlag_DoubleSided & iFlag)
		{
			Asura_Mesh_System::PlatformResetRenderMesh(mesh);
			Tools_RenderModule_System::UpdateModuleMesh(mesh->Index);
		}
		if( AsuraPolygonFlag_UseForEntityGISamples & iFlag)
		{
			Tools_GlobalIllumination_EntitySamples::Reset();
		}
		if( AsuraPolygonFlag_IgnoreNormalForLighting & iFlag)
		{
			Tools_EnvironmentalLightTransport::Invalidate();
		}
		if( bAdd )
		{
			mesh->PolygonFlagDataPtr[polygon.PolyIndex].Flags |= iFlagToSet;
		}
		else
		{
			mesh->PolygonFlagDataPtr[polygon.PolyIndex].Flags &=~ iFlagToSet;
		}
		
		Tools_Render_Options::UpdatePolygonVisibility(polygon.ModuleIndex,polygon.PolyIndex);
			
		if(iFlag & AsuraPolygonFlag_SplayNormals)
		{
			mesh->InvalidateNormals();			
		}

		if( iFlag & (AsuraPolygonFlag_FullBright|AsuraPolygonFlag_IgnoreNormalForLighting|AsuraPolygonFlag_IgnoreAmbientLight|AsuraPolygonFlag_Darken|AsuraPolygonFlag_SplayNormals))
		{
			Tools_GlobalIllumination_EntitySamples::LightingMayHaveChanged();
			mesh->PrelightingDataValid = false;
		}

		Tools_Mode_MeshEdit::InvalidateModuleCollisionMesh(polygon.ModuleIndex);

	}

    // If we're applying floor flag then we need to do a second pass to add
    // step flags to floor polys on the tops of steps.
    if( AsuraPolygonFlag_Floor & iFlag )
    {
 	    Tools_Polygon_Selection::StartIteration();
	    while( Tools_Polygon_Selection::GetNextSelected(&polygon) )
	    {
            RegenerateStepFlagsForAdjacentPolys( polygon.ModuleIndex, polygon.PolyIndex );
        }
    }
}

void Tools_Mode_Flags::SetVertFlag( u_int uFlag, bool bAdd )
{
	//log for undo
	PrepareUndo_VertFlag();

	ModuleVertexPair xVertex;

	Tools_Vertex_Selection::StartIteration();
	while(Tools_Vertex_Selection::GetNextSelected(&xVertex))
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(xVertex.iModuleIndex);

		u_int uVertFlags = pxMesh->GetVertexFlags(xVertex.iVertexIndex);
		if(bAdd)
		{
			uVertFlags |= uFlag;
		}
		else
		{
			uVertFlags &=~ uFlag;
		}

		pxMesh->SetVertexFlags(xVertex.iVertexIndex,uVertFlags);
	}
}

void Tools_Mode_Flags::SetProjectPolygonData( int iIndex, u_int uValue )
{
	Asura_Assert(s_pxThis,"");

	//log for undo
	PrepareUndo_ProjectPolygonData();

	PolygonID polygon;

	Tools_Polygon_Selection::StartIteration();
	while(Tools_Polygon_Selection::GetNextSelected(&polygon))
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(polygon.ModuleIndex);

		u_int uData = pxMesh->GetProjectPolygonData(polygon.PolyIndex);

		if(iIndex == 0xffffffff)
		{
			//sets all data at once
			uData = uValue;
		}
		else
		{
			uData = s_pxThis->SetValueInProjectPolygonData(iIndex,uData,uValue);
		}

		pxMesh->SetProjectPolygonData(polygon.PolyIndex,uData);
	}
}



void Tools_Mode_Flags::RegenerateStepFlags( void )
{
    // Prepare undo.
    PrepareUndo_PolyFlag();

    // First pass: remove step flags from floor polys. 
    const int iNumModules = Asura_Module_System::GetNumberOfModules();
    for( int iModule = 0; iModule < iNumModules; ++iModule )
    {
        Asura_Mesh* pxModuleMesh = Asura_Module_System::GetMeshForModule( iModule );
        if( !pxModuleMesh->PolygonFlagDataValid ) { continue; }

        const int iNumPolysInModule = pxModuleMesh->NumberOfPolygons;
        for( int iPoly = 0; iPoly < iNumPolysInModule; ++iPoly )
        {
            int iPolyFlags = pxModuleMesh->PolygonFlagDataPtr[iPoly].Flags;
            if( iPolyFlags & AsuraPolygonFlag_Floor )
            {
                pxModuleMesh->PolygonFlagDataPtr[iPoly].Flags = (iPolyFlags & ~AsuraPolygonFlag_Step);
				Tools_Render_Options::UpdatePolygonVisibility(iModule,iPoly);
            }
        }
		Tools_Mode_MeshEdit::InvalidateModuleCollisionMesh(iModule);

    }

    // Second pass: add required step flags.
    for( int iModule = 0; iModule < iNumModules; ++iModule )
    {
        Asura_Mesh* pxModuleMesh = Asura_Module_System::GetMeshForModule( iModule );

        const int iNumPolysInModule = pxModuleMesh->NumberOfPolygons;
        for( int iPoly = 0; iPoly < iNumPolysInModule; ++iPoly )
        {
            RegenerateStepFlagsForAdjacentPolys( iModule, iPoly );
        }
    }

    // Report that the process is complete.
	Tools_Timed_Message_System::AddMessage( 0xffffff, "Step flags regenerated." );

    // Flag that these changes will need to be saved.
    Tools_SaveAsura::SetMainFileModified();
}

void Tools_Mode_Flags::HighlightPolyFlags()
{
	static const u_int uGreen		 = 0xff00ff00;
	static const u_int uBlueishGreen = 0xff00df9f;
	static const u_int uYellow       = 0xffffff00;
	static const u_int uPurple		 = 0xffff00ff;

	int iFlagToCheck  = s_iSelectedPolyFlags;
	int iFlagToCheck2 = 0;
	int iFlagToCheck3 = 0;

	if ( iFlagToCheck & AsuraPolygonFlag_Floor )
	{
		iFlagToCheck2 |= AsuraPolygonFlag_DenseWaypoints;
		iFlagToCheck3 |= AsuraPolygonFlag_IgnoreWhenWaypointing;
	}
	if ( iFlagToCheck & AsuraPolygonFlag_DenseWaypoints )
	{
		iFlagToCheck2 |= AsuraPolygonFlag_Floor;
		iFlagToCheck3 |= AsuraPolygonFlag_IgnoreWhenWaypointing;
	}
	if ( iFlagToCheck & AsuraPolygonFlag_IgnoreWhenWaypointing )
	{
		iFlagToCheck3 |= AsuraPolygonFlag_Floor | AsuraPolygonFlag_DenseWaypoints;
	}

	// start batched renderer
	Tools_BatchRenderPolygonOutline::BeginBatch();

	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		// are there any other reasons to reject the module as a whole?

		// module hidden
		if (!Asura_Module_System::IsModuleVisible( i )) continue;

		// module too far from camera
		if(!Tools_Render_Options::IsModuleWithinViewDistance( i )) continue;

		Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(i);
		Asura_Assert(mesh,"Failed to get the mesh for the module");
		if(!mesh->PolygonFlagDataValid) continue;

		for(int j=0;j<mesh->NumberOfPolygons;j++)
		{
            const int iFlags = mesh->PolygonFlagDataPtr[j].Flags;

			if ( iFlags & iFlagToCheck )
			{
				if ( ( iFlags & iFlagToCheck ) == iFlagToCheck )
				{
					Tools_BatchRenderPolygonOutline::AddOutlineToBatch( j, i, uGreen );
				}
				else if ( !s_bHighlightOnlyIfHaveAllFlags )
				{
					// Only has some of the selected flags.
					Tools_BatchRenderPolygonOutline::AddOutlineToBatch( j, i, uBlueishGreen );
				}
			}
			else if ( iFlags & iFlagToCheck2 )
			{
				Tools_BatchRenderPolygonOutline::AddOutlineToBatch( j, i, uYellow );
			}
			else if ( iFlags & iFlagToCheck3 )
			{
				Tools_BatchRenderPolygonOutline::AddOutlineToBatch( j, i, uPurple );
			}
		}
	}
	
	// end batched renderer
	Tools_BatchRenderPolygonOutline::EndBatch();	
}

void Tools_Mode_Flags::HighlightVertFlags()
{
	u_int uFlagToCheck = s_uSelectedVertFlags;
	
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		// are there any other reasons to reject the module as a whole?

		// module hidden
		if (!Asura_Module_System::IsModuleVisible( i )) continue;

		// module too far from camera
		if(!Tools_Render_Options::IsModuleWithinViewDistance( i )) continue;

		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
		Asura_Assert(pxMesh,"Failed to get the mesh for the module");
		if(!pxMesh->m_pxVertexFlags) continue;

		for(int j=0;j<pxMesh->NumberOfVertices;j++)
		{
            const u_int uFlags = pxMesh->GetVertexFlags(j);

            if( uFlags & uFlagToCheck )
			{
				if((uFlags & uFlagToCheck) == uFlagToCheck)
				{
					Tools_Render::RenderVertexOutline(i,j,0xff00ff00);//green
				}
				else if(!s_bHighlightOnlyIfHaveAllFlags)
				{
					//only has some of the selected flags
					Tools_Render::RenderVertexOutline(i,j,0xff00df9f);
				}
			}
		}
	}
}

void Tools_Mode_Flags::HighlightProjectPolygonData()
{
	if(!s_pxThis) return;

	// start batched renderer
	Tools_BatchRenderPolygonOutline::BeginBatch();

	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		// are there any other reasons to reject the module as a whole?

		// module hidden
		if (!Asura_Module_System::IsModuleVisible( i )) continue;

		// module too far from camera
		if(!Tools_Render_Options::IsModuleWithinViewDistance( i )) continue;

		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
		Asura_Assert(pxMesh,"Failed to get the mesh for the module");


		for(int j=0;j<pxMesh->NumberOfPolygons;j++)
		{
			u_int uData = pxMesh->GetProjectPolygonData(j);
			u_int uValue = s_pxThis->GetValueFromProjectPolygonData(s_iProjectPolygonDataSelectionIndex,uData);


            if(uValue == s_uProjectPolygonDataValue)
			{
				Tools_BatchRenderPolygonOutline::AddOutlineToBatch(j,i,0xff00ff00);//green
			}
		}
	}

	// end batched renderer
	Tools_BatchRenderPolygonOutline::EndBatch();	

}



void Tools_Mode_Flags::Undo()
{
	if(FlagsUndo.Undo())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Undo");
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		UpdateElementVisibilities();
#endif
	}
}
void Tools_Mode_Flags::Redo()
{
	if(FlagsUndo.Redo())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Redo");
#ifdef ASURA_SUPPORT_SKIN_ELEMENTS
		UpdateElementVisibilities();
#endif
	}
}


void Tools_Mode_Flags::PrepareUndo_PolyFlag()
{
	PolygonID polygon;
	Tools_Polygon_Selection::StartIteration();

	int iPrevModuleIndex = -1;
	while(Tools_Polygon_Selection::GetNextSelected(&polygon))
	{
		FlagsUndo.LogPolyFlagsChanged(polygon);

		if(iPrevModuleIndex != polygon.ModuleIndex)
		{
			Tools_SaveAsura::SetModuleModified(polygon.ModuleIndex);
			iPrevModuleIndex = polygon.ModuleIndex;
		}
	}
}

void Tools_Mode_Flags::PrepareUndo_VertFlag()
{
	ModuleVertexPair xVertex;
	Tools_Vertex_Selection::StartIteration();

	int iPrevModuleIndex = -1;
	while(Tools_Vertex_Selection::GetNextSelected(&xVertex))
	{
		FlagsUndo.LogVertFlagsChanged(xVertex);
		if(iPrevModuleIndex != xVertex.iModuleIndex)
		{
			Tools_SaveAsura::SetModuleModified(xVertex.iModuleIndex);
			iPrevModuleIndex = xVertex.iModuleIndex;
		}
	}
}

void Tools_Mode_Flags::PrepareUndo_ProjectPolygonData()
{
	PolygonID polygon;
	Tools_Polygon_Selection::StartIteration();

	int iPrevModuleIndex = -1;
	while(Tools_Polygon_Selection::GetNextSelected(&polygon))
	{
		FlagsUndo.LogProjectPolygonDataChanged(polygon);
		if(iPrevModuleIndex != polygon.ModuleIndex)
		{
			Tools_SaveAsura::SetModuleModified(polygon.ModuleIndex);
			iPrevModuleIndex = polygon.ModuleIndex;
		}
	}
}



void Tools_Mode_Flags::CollisionMesh_ReplaceByBox()
{
	//Find each module with polygons selected (most likely just one)  , and flag
	//those polygons as being replaced by a bounding cube
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		if(!Tools_Polygon_Selection::CountSelectedInModule(i)) continue;

		List<int> xPolyList;
		Tools_Polygon_Selection::ListSelectedInModule(i,xPolyList);

		CollisionMesh_ReplaceByBox(i,xPolyList);

	}
}

void Tools_Mode_Flags::CollisionMesh_ReplaceByBox(int iModule,const List<int>& xPolyList)
{
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModule);

	//allocate collision mesh generation data if needed
	if(!pxMesh->m_pxCollMeshGenData)
	{
		pxMesh->m_pxCollMeshGenData = new Asura_Mesh_CollMeshGenData();
	}

	Asura_Mesh_CollMeshGenData* pxCollGenData = pxMesh->m_pxCollMeshGenData;
	//initialise polygon data
	if(!pxCollGenData->m_pxPolygons)
	{
		pxCollGenData->InitialisePolygonArray(pxMesh);
	}

	//add a new box for these polygons
	Asura_Mesh_CollBoxInfo* pxBox = pxCollGenData->CreateBox();
	u_int uID = pxBox->m_uGroupID;

	for(CLIF<int> xPolyLIF(&xPolyList);!xPolyLIF.done();xPolyLIF.next())
	{
		pxCollGenData->m_pxPolygons[xPolyLIF()].m_bReplaceByBox = true;
		pxCollGenData->m_pxPolygons[xPolyLIF()].m_usGroupID = (u_short) pxBox->m_uGroupID;

	}

	pxCollGenData->RemoveUnusedGroups();

	//invalidate collision mesh
	Tools_Mode_MeshEdit::InvalidateModuleCollisionMesh(iModule);

	//edit box
	Tools_CollBox_Object* pxObject = new Tools_CollBox_Object(iModule,uID);
	Tools_Object_System::SelectObject(pxObject);

    Tools_SaveAsura::SetMainFileModified();

}

void Tools_Mode_Flags::CollisionMesh_ResetPolygons()
{
	//Find all polygons that are selected , and remove the flag for replacing
	//them with a bounding box

	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		if(!Tools_Polygon_Selection::CountSelectedInModule(i)) continue;

		List<int> xPolyList;
		Tools_Polygon_Selection::ListSelectedInModule(i,xPolyList);

		CollisionMesh_ResetPolygons(i,xPolyList);

	}
}

void Tools_Mode_Flags::CollisionMesh_ResetPolygons(int iModule,const List<int>& xPolyList)
{
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModule);

	if(!pxMesh->m_pxCollMeshGenData) return;

	Asura_Mesh_CollMeshGenData* pxCollGenData = pxMesh->m_pxCollMeshGenData;
	if(!pxCollGenData->m_pxPolygons) return;

	
	//reset the selected polygons
	for(CLIF<int> xPolyLIF(&xPolyList);!xPolyLIF.done();xPolyLIF.next())
	{
		pxCollGenData->m_pxPolygons[xPolyLIF()].m_usFlags = 0;
		pxCollGenData->m_pxPolygons[xPolyLIF()].m_usGroupID = 0;

	}

	pxCollGenData->RemoveUnusedGroups();

	//invalidate collision mesh
	Tools_Mode_MeshEdit::InvalidateModuleCollisionMesh(iModule);
    Tools_SaveAsura::SetMainFileModified();

}

struct Tools_PolygonFlagNameAndValue
{
	char* m_szName;
	int m_iValue;
	char* m_szHelp;
};

static int SortFlags(const void* pFlag1, const void* pFlag2)
{
	const Tools_PolygonFlagNameAndValue* pxFlag1 = (const Tools_PolygonFlagNameAndValue*) pFlag1;
	const Tools_PolygonFlagNameAndValue* pxFlag2 = (const Tools_PolygonFlagNameAndValue*) pFlag2;

	return stricmp(pxFlag1->m_szName, pxFlag2->m_szName);
}

void Tools_Mode_Flags::BuildPolygonFlagArrays()
{
	int iCommonFlags = GetNumberOfPolygonFlags();
	int iProjectFlags = GetNumberOfProjectPolygonFlags();

	s_iNumberOfPolygonFlags = iCommonFlags + iProjectFlags;

	//allocate temporary array for getting the flag names etc
	Tools_PolygonFlagNameAndValue* pxFlags = new Tools_PolygonFlagNameAndValue[s_iNumberOfPolygonFlags];


	int iIndex=0;
	
	for(int iLoop=0;iLoop<2;iLoop++)
	{
		for(int i=0;i<(iLoop ? iProjectFlags : iCommonFlags );i++,iIndex++)
		{
			pxFlags[iIndex].m_iValue = iLoop ? GetProjectPolygonFlagValue(i) : GetPolygonFlagValue(i);

			const char* pszFlagName = iLoop ? GetProjectPolygonFlagName(i) : GetPolygonFlagName(i);
			if(pszFlagName)
			{
				pxFlags[iIndex].m_szName = new char[strlen(pszFlagName)+1];
				strcpy(pxFlags[iIndex].m_szName,pszFlagName);
			}
			else
			{
				pxFlags[iIndex].m_szName = new char[20];
				strcpy(pxFlags[iIndex].m_szName,"*Unknown Flag*");
			}

			const char* pszFlagHelp = iLoop ? GetProjectPolygonFlagHelpText(i) : GetPolygonFlagHelpText(i);
			if(pszFlagHelp)
			{
				pxFlags[iIndex].m_szHelp = new char[strlen(pszFlagHelp)+1];
				strcpy(pxFlags[iIndex].m_szHelp,pszFlagHelp);
			}
			else
			{
				pxFlags[iIndex].m_szHelp = 0;
			}
		}
	}

	//sort the flags
	Asura_SortAndSearch::Quicksort(pxFlags, s_iNumberOfPolygonFlags, SortFlags);

	//allocate the global arrays for the flags and copy the sorted values
	s_pszPolygonFlagNames = new char*[s_iNumberOfPolygonFlags];
	s_pszPolygonFlagHelp = new char*[s_iNumberOfPolygonFlags];
	s_piPolygonFlagValues = new int[s_iNumberOfPolygonFlags];

	for(int i=0; i<s_iNumberOfPolygonFlags; i++)
	{
		s_pszPolygonFlagNames[i] = pxFlags[i].m_szName;
		s_pszPolygonFlagHelp[i] = pxFlags[i].m_szHelp;
		s_piPolygonFlagValues[i] = pxFlags[i].m_iValue;
	
	}

	delete [] pxFlags;
}

void Tools_Mode_Flags::DeletePolygonFlagArrays()
{
	for(int i=0;i<s_iNumberOfPolygonFlags;i++)
	{
		delete [] s_pszPolygonFlagNames[i];
		delete [] s_pszPolygonFlagHelp[i];
	}
	delete [] s_pszPolygonFlagNames;
	delete [] s_pszPolygonFlagHelp;
	delete [] s_piPolygonFlagValues;

	s_pszPolygonFlagNames = 0;
	s_pszPolygonFlagHelp = 0;
	s_piPolygonFlagValues = 0;
	s_iNumberOfPolygonFlags = 0;
}

void Tools_Mode_Flags::BuildVertexFlagArrays()
{
	int iCommonFlags = GetNumberOfVertexFlags();
	int iProjectFlags = GetNumberOfProjectVertexFlags();

	s_iNumberOfVertexFlags = iCommonFlags + iProjectFlags;

	//allocate arrays for flags
	s_pszVertexFlagNames = new char*[s_iNumberOfVertexFlags];
	s_pszVertexFlagHelp = new char*[s_iNumberOfVertexFlags];
	s_puVertexFlagValues = new u_int[s_iNumberOfVertexFlags];

	int iIndex=0;
	
	for(int iLoop=0;iLoop<2;iLoop++)
	{
		for(int i=0;i<(iLoop ? iProjectFlags : iCommonFlags );i++,iIndex++)
		{
			s_puVertexFlagValues[iIndex] = iLoop ? GetProjectVertexFlagValue(i) : GetVertexFlagValue(i);

			const char* pszFlagName = iLoop ? GetProjectVertexFlagName(i) : GetVertexFlagName(i);
			if(pszFlagName)
			{
				s_pszVertexFlagNames[iIndex] = new char[strlen(pszFlagName)+1];
				strcpy(s_pszVertexFlagNames[iIndex],pszFlagName);
			}
			else
			{
				s_pszVertexFlagNames[iIndex] = new char[20];
				strcpy(s_pszVertexFlagNames[iIndex],"*Unknown Flag*");
			}

			const char* pszFlagHelp = iLoop ? GetProjectVertexFlagHelpText(i) : GetVertexFlagHelpText(i);
			if(pszFlagHelp)
			{
				s_pszVertexFlagHelp[iIndex] = new char[strlen(pszFlagHelp)+1];
				strcpy(s_pszVertexFlagHelp[iIndex],pszFlagHelp);
			}
			else
			{
				s_pszVertexFlagHelp[iIndex] = 0;
			}
		}
	}
}

void Tools_Mode_Flags::DeleteVertexFlagArrays()
{
	for(int i=0;i<s_iNumberOfVertexFlags;i++)
	{
		delete [] s_pszVertexFlagNames[i];
		delete [] s_pszVertexFlagHelp[i];
	}
	delete [] s_pszVertexFlagNames;
	delete [] s_pszVertexFlagHelp;
	delete [] s_puVertexFlagValues;

	s_pszVertexFlagNames = 0;
	s_pszVertexFlagHelp = 0;
	s_puVertexFlagValues = 0;
	s_iNumberOfVertexFlags = 0;
}

void Tools_Mode_Flags::BuildProjectPolygonDataArrays()
{
	if(!s_pxThis) return;
	
	s_iNumberOfProjectPolygonDataValues = s_pxThis->GetNumberOfProjectPolygonDataValues();

	//allocate arrays for flags
	s_pszProjectPolygonDataNames = new char*[s_iNumberOfProjectPolygonDataValues];
	s_pszProjectPolygonDataHelp = new char*[s_iNumberOfProjectPolygonDataValues];

	for(int i=0;i<s_iNumberOfProjectPolygonDataValues;i++)
	{

		const char* pszValueName = s_pxThis->GetProjectPolygonDataName(i);
		if(pszValueName)
		{
			s_pszProjectPolygonDataNames[i] = new char[strlen(pszValueName)+1];
			strcpy(s_pszProjectPolygonDataNames[i],pszValueName);
		}
		else
		{
			s_pszProjectPolygonDataNames[i] = new char[20];
			strcpy(s_pszProjectPolygonDataNames[i],"*Unknown Flag*");
		}

		const char* pszValueHelp = s_pxThis->GetProjectPolygonDataHelpText(i);
		if(pszValueHelp)
		{
			s_pszProjectPolygonDataHelp[i] = new char[strlen(pszValueHelp)+1];
			strcpy(s_pszProjectPolygonDataHelp[i],pszValueHelp);
		}
		else
		{
			s_pszProjectPolygonDataHelp[i] = 0;
		}
	}
}

void Tools_Mode_Flags::DeleteProjectPolygonDataArrays()
{
	for(int i=0;i<s_iNumberOfProjectPolygonDataValues;i++)
	{
		delete [] s_pszProjectPolygonDataNames[i];
		delete [] s_pszProjectPolygonDataHelp[i];
	}
	delete [] s_pszProjectPolygonDataNames;
	delete [] s_pszProjectPolygonDataHelp;

	s_pszProjectPolygonDataNames = 0;
	s_pszProjectPolygonDataHelp = 0;
	s_iNumberOfProjectPolygonDataValues = 0;
}



bool Tools_Mode_Flags::IsPolyFlagSelected(int iIndex,int iUnused)
{
	Asura_Assert(iIndex<s_iNumberOfPolygonFlags,"Unexpected flag index");
	return (s_piPolygonFlagValues[iIndex] & s_iSelectedPolyFlags) ? true : false;
}

void Tools_Mode_Flags::TogglePolyFlagSelection(int iIndex,int iUnused)
{
    Asura_Assert( iIndex < s_iNumberOfPolygonFlags, "Unexpected flag index" );
    const int iFlagValue = s_piPolygonFlagValues[ iIndex ];

	if(iFlagValue == AsuraPolygonFlag_DontRender || iFlagValue == AsuraPolygonFlag_IgnoreLOS )
	{
		u_int uLinkedFlags = AsuraPolygonFlag_DontRender|AsuraPolygonFlag_IgnoreLOS;
		
		//by default select 'ignore LOS' at same time as 'dont render'
		if(s_bAllowMultipleFlagSelection)
		{
			if(iFlagValue == AsuraPolygonFlag_DontRender && !(s_iSelectedPolyFlags & iFlagValue))
			{
				//clicked don't render when it was currently off , so apply both flags
				s_iSelectedPolyFlags |= (uLinkedFlags);
			}
			else
			{
				//just toggle the selected flag
				s_iSelectedPolyFlags ^= iFlagValue;
			}
		}
		else
		{
			if(s_iSelectedPolyFlags & uLinkedFlags)
			{
				//already have one of the linked flags selected ,so toggle the flag clicked on.
				s_iSelectedPolyFlags ^= iFlagValue;
				if(!s_iSelectedPolyFlags) s_iSelectedPolyFlags = iFlagValue;
			}
			else if(iFlagValue == AsuraPolygonFlag_DontRender)
			{
				s_iSelectedPolyFlags = uLinkedFlags;
			}
			else
			{
				s_iSelectedPolyFlags = iFlagValue;
			}
		}
	}
	else if ( ( iFlagValue == AsuraPolygonFlag_Floor || iFlagValue == AsuraPolygonFlag_Physics_WalkableSurface ) && s_bSelectPhysicsWalkableWithFloorFlag )
	{
		//also default to selecting 'walkable surface' at the same time as 'floor' (AKA use for waypointing)
		u_int uLinkedFlags = AsuraPolygonFlag_Floor|AsuraPolygonFlag_Physics_WalkableSurface;
		
		if(s_bAllowMultipleFlagSelection)
		{
			if(iFlagValue == AsuraPolygonFlag_Floor && !(s_iSelectedPolyFlags & iFlagValue))
			{
				//clicked don't render when it was currently off , so apply both flags
				s_iSelectedPolyFlags |= (uLinkedFlags);
			}
			else
			{
				//just toggle the selected flag
				s_iSelectedPolyFlags ^= iFlagValue;
			}
		}
		else
		{
			if(s_iSelectedPolyFlags & uLinkedFlags)
			{
				//already have one of the linked flags selected ,so toggle the flag clicked on.
				s_iSelectedPolyFlags ^= iFlagValue;
				if(!s_iSelectedPolyFlags) s_iSelectedPolyFlags = iFlagValue;
			}
			else if(iFlagValue == AsuraPolygonFlag_Floor)
			{
				s_iSelectedPolyFlags = uLinkedFlags;
			}
			else
			{
				s_iSelectedPolyFlags = iFlagValue;
			}
		}

	}
	else
	{
		if(s_bAllowMultipleFlagSelection)
		{
			s_iSelectedPolyFlags ^= iFlagValue;
		}
		else
		{
			//only one flag can be selected at a time.. so we select
			//the given flag rather than toggling it
			s_iSelectedPolyFlags = iFlagValue;
		}
	}
}

bool Tools_Mode_Flags::IsVertFlagSelected(int iIndex,int iUnused)
{
	Asura_Assert(iIndex<s_iNumberOfVertexFlags,"Unexpected flag index");
	return (s_puVertexFlagValues[iIndex] & s_uSelectedVertFlags) ? true : false;
}

void Tools_Mode_Flags::ToggleVertFlagSelection(int iIndex,int iUnused)
{
    Asura_Assert( iIndex < s_iNumberOfVertexFlags, "Unexpected flag index" );
    const u_int uFlagValue = s_puVertexFlagValues[ iIndex ];

	if(s_bAllowMultipleFlagSelection)
	{
		s_uSelectedVertFlags ^= uFlagValue;
	}
	else
	{
		//only one flag can be selected at a time.. so we select
		//the given flag rather than toggling it
		//(unless multiple flags are already selected , in which case allow existing ones to be removed)
		if(s_uSelectedVertFlags & uFlagValue)
		{
			if(s_uSelectedVertFlags != uFlagValue)
			{
				s_uSelectedVertFlags &=~ uFlagValue;
			}
		}
		else
		{
			s_uSelectedVertFlags = uFlagValue;
		}
	}
}


void Tools_Mode_Flags::DetermineCurrentSubMode()
{
	if(Tools_System::IsInFlagsMode())
	{
		s_eSubMode = s_aeTabToSubMode[s_uSelectedTab];
	}
}

void Tools_Mode_Flags::EnsureProjectPolygonDataIsInRange()
{
	if(!s_pxThis) return;

	u_int uMax = s_pxThis->GetProjectPolygonDataMax(s_iProjectPolygonDataSelectionIndex);
	if(s_uProjectPolygonDataValue > uMax) s_uProjectPolygonDataValue = uMax;

}

bool Tools_Mode_Flags::IsInFlagSubMode()
{
	return s_eSubMode == SUBMODE_POLYGON_FLAGS || s_eSubMode == SUBMODE_VERTEX_FLAGS;
}

/////////////////////////////////////////////////////////////////////////////////////////

int Tools_Mode_Flags::GetNumberOfPolygonFlags()
{
	return NumberOfPolygonFlags;
}

int Tools_Mode_Flags::GetPolygonFlagValue(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<NumberOfPolygonFlags,"");
	return g_aiPolygonFlagValues[iIndex];
}

const char* Tools_Mode_Flags::GetPolygonFlagName(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<NumberOfPolygonFlags,"");
	return g_aszPolygonFlagNames[iIndex];
}

const char* Tools_Mode_Flags::GetPolygonFlagHelpText(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<NumberOfPolygonFlags,"");
	return g_aszPolygonFlagHelp[iIndex];
}

int Tools_Mode_Flags::GetNumberOfProjectPolygonFlags()
{
	return Tools_System::This->GetNumberOfProjectPolygonFlags();
}

int Tools_Mode_Flags::GetProjectPolygonFlagValue(int iIndex)
{
	return Tools_System::This->GetProjectPolygonFlagValue(iIndex);
}

const char* Tools_Mode_Flags::GetProjectPolygonFlagName(int iIndex)
{
	return Tools_System::This->GetProjectPolygonFlagName(iIndex);
}

const char* Tools_Mode_Flags::GetProjectPolygonFlagHelpText(int iIndex)
{
	return Tools_System::This->GetProjectPolygonFlagHelpText(iIndex);
}

/////////////////////////////////////////////////////////////////////////////////////////

int Tools_Mode_Flags::GetNumberOfVertexFlags()
{
	return NumberOfVertexFlags;
}

u_int Tools_Mode_Flags::GetVertexFlagValue(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<NumberOfVertexFlags,"");
	return g_auVertexFlagValues[iIndex];
}

const char* Tools_Mode_Flags::GetVertexFlagName(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<NumberOfVertexFlags,"");
	return g_aszVertexFlagNames[iIndex];
}

const char* Tools_Mode_Flags::GetVertexFlagHelpText(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<NumberOfVertexFlags,"");
	return g_aszVertexFlagHelp[iIndex];
}

int Tools_Mode_Flags::GetNumberOfProjectVertexFlags()
{
	return Tools_System::This->GetNumberOfProjectVertexFlags();
}

u_int Tools_Mode_Flags::GetProjectVertexFlagValue(int iIndex)
{
	return Tools_System::This->GetProjectVertexFlagValue(iIndex);
}

const char* Tools_Mode_Flags::GetProjectVertexFlagName(int iIndex)
{
	return Tools_System::This->GetProjectVertexFlagName(iIndex);
}

const char* Tools_Mode_Flags::GetProjectVertexFlagHelpText(int iIndex)
{
	return Tools_System::This->GetProjectVertexFlagHelpText(iIndex);
}




/////////////////////////////////////////////////////////////////////////////////////////
/*---------------**
** FlagsModeUndo **
**---------------*/


void FlagsModeUndo::ProcessUndoData(void* data)
{
	FlagsUndoData* pxUndoData = (FlagsUndoData*) data;
	
	switch(pxUndoData->m_eUndoType)
	{
		case FlagsUndoData::UndoType_PolyFlagsChanged :
		{
			int iModuleIndex = pxUndoData->m_iModuleIndex;
			int iPolyIndex = pxUndoData->m_iPolygonIndex;
			//get the mesh
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModuleIndex);
			Tools_Mode_MeshEdit::InvalidateModuleCollisionMesh(iModuleIndex);

			//allocate polygon flags if needed
			if(!pxMesh->PolygonFlagDataValid)
			{
				if(!pxUndoData->m_iFlags) break;
				pxMesh->AllocatePolygonFlags();
			}
			
			//store current flags and update from undo data
			int iFlags = pxMesh->PolygonFlagDataPtr[iPolyIndex].Flags;
			pxMesh->PolygonFlagDataPtr[iPolyIndex].Flags = pxUndoData->m_iFlags;

			//convert the undo data
			pxUndoData->m_iFlags = iFlags;

			int iChangedFlags = iFlags^pxMesh->PolygonFlagDataPtr[iPolyIndex].Flags;
			if(iChangedFlags & AsuraPolygonFlag_SplayNormals)
			{
				pxMesh->InvalidateNormals();
			}
			//did the FullBright flag change?
			if(iChangedFlags&(AsuraPolygonFlag_FullBright|AsuraPolygonFlag_IgnoreNormalForLighting|AsuraPolygonFlag_IgnoreAmbientLight|AsuraPolygonFlag_Darken|AsuraPolygonFlag_SplayNormals))
			{
				//yep.... invalidate prelighting
				Tools_GlobalIllumination_EntitySamples::LightingMayHaveChanged();
				pxMesh->PrelightingDataValid = false;
			}
			Tools_Render_Options::UpdatePolygonVisibility(iModuleIndex,iPolyIndex);

			if(iChangedFlags & AsuraPolygonFlag_DoubleSided)
			{
				Asura_Mesh_System::PlatformResetRenderMesh(pxMesh);
				Tools_RenderModule_System::UpdateModuleMesh(pxMesh->Index);
			}

			Tools_SaveAsura::SetMeshModified(pxMesh);

			break;
		}

		case FlagsUndoData::UndoType_VertFlagsChanged :
		{
			int iModuleIndex = pxUndoData->m_iModuleIndex;
			int iVertIndex = pxUndoData->m_iVertexIndex;
			//get the mesh
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModuleIndex);

			
			//store current flags and update from undo data
			u_int uFlags = pxMesh->GetVertexFlags(iVertIndex);
			pxMesh->SetVertexFlags(iVertIndex,pxUndoData->m_uFlags);

			//convert the undo data
			pxUndoData->m_uFlags = uFlags;

			Tools_SaveAsura::SetMeshModified(pxMesh);

			break;
		}

		case FlagsUndoData::UndoType_ProjectPolygonDataChanged :
		{
			int iModuleIndex = pxUndoData->m_iModuleIndex;
			int iPolyIndex = pxUndoData->m_iPolygonIndex;
			//get the mesh
			Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModuleIndex);

			
			//store current flags and update from undo data
			u_int uData = pxMesh->GetProjectPolygonData(iPolyIndex);
			pxMesh->SetProjectPolygonData(iPolyIndex,pxUndoData->m_uData);

			//convert the undo data
			pxUndoData->m_uData = uData;

			Tools_SaveAsura::SetMeshModified(pxMesh);

			break;
		}


		default :
			Asura_Assert(0,"Unexpected undo type for flag editing");
			break;
	}
}

void FlagsModeUndo::LogPolyFlagsChanged(const PolygonID& polygon)
{
	//allocate undo data
	FlagsUndoData* pxUndo = (FlagsUndoData*) AllocateNextUndoData();
	if(!pxUndo) return;

	//get the mesh data
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(polygon.ModuleIndex);

	
	//fill in the undo data	
	pxUndo->m_eUndoType = FlagsUndoData::UndoType_PolyFlagsChanged;
	pxUndo->m_iModuleIndex = polygon.ModuleIndex;
	pxUndo->m_iPolygonIndex = polygon.PolyIndex;
	if(pxMesh->PolygonFlagDataValid)
		pxUndo->m_iFlags  = pxMesh->PolygonFlagDataPtr[polygon.PolyIndex].Flags;
	else
		pxUndo->m_iFlags = 0;
}

void FlagsModeUndo::LogVertFlagsChanged(const ModuleVertexPair& xVertex)
{
	//allocate undo data
	FlagsUndoData* pxUndo = (FlagsUndoData*) AllocateNextUndoData();
	if(!pxUndo) return;

	//get the mesh data
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(xVertex.iModuleIndex);

	
	//fill in the undo data	
	pxUndo->m_eUndoType = FlagsUndoData::UndoType_VertFlagsChanged;

	pxUndo->m_iModuleIndex = xVertex.iModuleIndex;
	pxUndo->m_iVertexIndex = xVertex.iVertexIndex;
	pxUndo->m_uFlags = pxMesh->GetVertexFlags(xVertex.iVertexIndex);
}



void FlagsModeUndo::LogProjectPolygonDataChanged(const PolygonID& polygon)
{
	//allocate undo data
	FlagsUndoData* pxUndo = (FlagsUndoData*) AllocateNextUndoData();
	if(!pxUndo) return;

	//get the mesh data
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(polygon.ModuleIndex);

	
	//fill in the undo data	
	pxUndo->m_eUndoType = FlagsUndoData::UndoType_ProjectPolygonDataChanged;
	pxUndo->m_iModuleIndex = polygon.ModuleIndex;
	pxUndo->m_iPolygonIndex = polygon.PolyIndex;
	pxUndo->m_uData  = pxMesh->GetProjectPolygonData(polygon.PolyIndex);
}






//test
#if 0
int Tools_Mode_Flags::GetNumberOfProjectPolygonDataValues() const
{
	return 2;
}

const char* Tools_Mode_Flags::GetProjectPolygonDataName(int iIndex) const
{
	return (iIndex==0) ? "Test1" : "Test2";
}

const char* Tools_Mode_Flags::GetProjectPolygonDataHelpText(int iIndex) const
{
	return (iIndex==0) ? "Help1" : "Help2";
}

u_int Tools_Mode_Flags::GetProjectPolygonDataMax(int iIndex) const
{
	return (iIndex==0) ? 255 : 65535;
}

u_int Tools_Mode_Flags::GetValueFromProjectPolygonData(int iIndex, u_int uData) const
{
	if(iIndex == 0)
	{
		return uData & 0xff;
	}
	else
	{
		return (uData >> 8) & 0xffff;
	}
}

u_int Tools_Mode_Flags::SetValueInProjectPolygonData(int iIndex,u_int uData,u_int uValue) const
{
	if(iIndex == 0)
	{
		uData &=~ 0xff;
		uData |= uValue;
	}
	else
	{
		uData &=~ 0xffff00;
		uData |= uValue << 8;
	}
	return uData;
}
#endif


#ifdef ASURA_SUPPORT_SKIN_ELEMENTS

void Tools_Mode_Flags::ApplyElement(void)
{
	PrepareUndo_PolyFlag();

	PolygonID polygon;

	Tools_Polygon_Selection::StartIteration();
	while(Tools_Polygon_Selection::GetNextSelected(&polygon))
	{
		// Don't 
		Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(polygon.ModuleIndex);
		if(!mesh->PolygonFlagDataValid) mesh->AllocatePolygonFlags();
		if(!mesh->m_bFaceNormalsValid) mesh->CreateFaceNormals();

		if (!Tools_Render_Options::IsHidden(mesh,polygon.PolyIndex))
		{		
			Asura_Project_Skin_Elements::PackElementIDIntoPolyFlags((u_int *)(&mesh->PolygonFlagDataPtr[polygon.PolyIndex].Flags), s_iElementSelectionIndex);
		}
	}

	UpdateElementVisibilities();
}


void Tools_Mode_Flags::ToggleElementVisibility(int iElement, int iUnused)
{
	Asura_Project_Skin_Elements::ToggleElementVisibility(iElement,0);
	UpdateElementVisibilities();
}


void Tools_Mode_Flags::UpdateElementVisibilities(void)
{
	// Only do this if we're in a suitable mode!
	// (Don't complain if we're in anim/hierarchy mode, as they use the
	//  actual skin renderer with overridden visibilities)
	
	if ( Tools_System::GetCurrentMode() == TOOLS_MODE_FLAGS     ||
		 Tools_System::GetCurrentMode() == TOOLS_MODE_TEXTURING ||
		 Tools_System::GetCurrentMode() == TOOLS_MODE_MESHEDIT  ||
  	     Tools_System::GetCurrentMode() == TOOLS_MODE_ANIMATION ||
		 Tools_System::GetCurrentMode() == TOOLS_MODE_HIERARCHY)
	{
		// Valid mode
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff8888,"Element Visibilities not supported in this mode!");
	}

	// Even if we're in a mode where updates won't be shown, we can do this anyway, 
	// to avoid getting out-of-sync with the overlay
	for (int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		Asura_Mesh* pxMesh =  Asura_Module_System::GetMeshForModule(i);		

		if (pxMesh->PolygonFlagDataValid)
		{		
			for (int j=0;j<pxMesh->NumberOfPolygons;j++)
			{
				uint32 uElement = Asura_Project_Skin_Elements::UnpackElementIDFromPolyFlags(pxMesh->PolygonFlagDataPtr[j].Flags);

				if (Asura_Project_Skin_Elements::IsElementVisible(uElement,0))
				{
					Tools_Render_Options::RemoveRenderFlagPolygon(i,j,ToolsRenderFlag_PolygonSelectionHide);
				}
				else
				{
					Tools_Render_Options::AddRenderFlagPolygon(i,j,ToolsRenderFlag_PolygonSelectionHide);
				}
			}						
		}
	}	
	
}


// This overlay is used to show/hide skin elements in most tools modes
void Tools_Mode_Flags::ToggleElementsOverlay()
{
	if(!Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_SKINELEMENTS))
	{
		Asura_Overlay* pxOverlay;
		pxOverlay = new Asura_Overlay("Show/Hide Elements",false,&s_xElementsOverlayPos);

		Asura_Dialog* pxDialog;
		Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
		
		pxDialog = new Asura_List_Dialog(xOffset,Asura_Project_Skin_Elements::GetElementTypeNames(),
										&s_iElementSelectionIndex,14,Asura_Project_Skin_Elements::GetNumElementTypes(),
										Asura_Project_Skin_Elements::IsElementVisible,
										ToggleElementVisibility,0);

		pxOverlay->AddDialog(pxDialog);

		xOffset.y=pxDialog->m_xMaxCoord.y;
		xOffset.y+=1.0f;

		pxDialog = new Asura_Function_Dialog("Show All Elements",xOffset,Tools_Mode_Flags::ChangeVisibleElementSelection,0);
		pxDialog->SetHelpText("Show all skin elements");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();		

		pxDialog = new Asura_Function_Dialog("Hide All Elements",xOffset,Tools_Mode_Flags::ChangeVisibleElementSelection,1);
		pxDialog->SetHelpText("Hide all skin elements");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();		

		pxDialog = new Asura_Function_Dialog("Restore Defaults",xOffset,Tools_Mode_Flags::ChangeVisibleElementSelection,2);
		pxDialog->SetHelpText("Show the skin in it's default (undamaged) state");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();		

		pxOverlay->m_uID = TOOLS_OVERLAY_SKINELEMENTS;
		Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKINELEMENTS);
	}		
}


void Tools_Mode_Flags::ChangeVisibleElementSelection(int iSelectionType)
{
	for(int i=0; i<Asura_Project_Skin_Elements::GetNumElementTypes();i++)
	{	
		// Show all
		if (iSelectionType==0 && Asura_Project_Skin_Elements::IsElementVisible(i,0)==false)
		{
			Asura_Project_Skin_Elements::ToggleElementVisibility(i,0);
		}

		// Hide all
		if (iSelectionType==1 && Asura_Project_Skin_Elements::IsElementVisible(i,0)==true)
		{
			Asura_Project_Skin_Elements::ToggleElementVisibility(i,0);
		}

		// Show defaults
		if (iSelectionType==2 && Asura_Project_Skin_Elements::IsElementVisible(i,0)!=Asura_Project_Skin_Elements::GetDefaultElementVisibility(i))
		{
			Asura_Project_Skin_Elements::ToggleElementVisibility(i,0);
		}
	}

	UpdateElementVisibilities();
}

#endif // ASURA_SUPPORT_SKIN_ELEMENTS


void Tools_Mode_Flags::HandleMeshChange(int iModuleIndex)
{
	//Invalidate any undos
	ClearUndo();
}

void Tools_Mode_Flags::ClearUndo()
{
	FlagsUndo.FlushUndoBuffer();
}