//
// Tools_GUIMenu_Element_Animation.cpp
//
// Created by RDM - 16 April 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

/************/
/* Includes */
/************/

// main asura header
#include "Core/Asura.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

// This file's header.
#include "GUIMenu/Tools_GUIMenu_Element_Animation.h"

// asura
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_AnimationController_Animation.h"
#include "ChunkSystem/Asura_Chunk_FileInfo.h"
#include "GUIMenu/Asura_GUIMenu_Element_Animation.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Text_Output.h"

// tools
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "PC/FileSystem/Tools_FileSystem.h"

/********/
/* Data */
/********/

// constants
static const u_int uASURA_GUIMENU_ELEMENT_ANIMATION_DIALOGID_SKINPICKER = 0x6B976833; //Asura_GetHashID("skin picker display")

// globals
static Asura_String g_xSkinName;
static Asura_String g_xAnimName;

/*************/
/* Functions */
/*************/

Tools_GUIMenu_Element_Animation::Tools_GUIMenu_Element_Animation( Asura_GUIMenu_Element_Animation* pxAsuraAnim, Tools_GUIMenu_Root* pxToolsParent )
: Tools_GUIMenu_Element_Base( pxAsuraAnim, pxToolsParent )
, m_pxAsuraAnim( pxAsuraAnim )
{
	// ensure all necessary files are loaded
	m_pxAsuraAnim->LoadSkinAndAnimFiles();
}

Tools_GUIMenu_Element_Animation::~Tools_GUIMenu_Element_Animation()
{
}

void Tools_GUIMenu_Element_Animation::Update()
{
	PARENT::Update();

	Asura_Animation* pxAnim = m_pxAsuraAnim->GetAnim();
	if( !pxAnim && (m_pxAsuraAnim->m_uSkinID != ASURA_HASH_ID_UNSET) )
	{
		m_pxAsuraAnim->SetupAnimation();
	}

	if( pxAnim && (pxAnim->GetSkinHashID() != m_pxAsuraAnim->m_uSkinID) )
	{		
		pxAnim->SetSkin( m_pxAsuraAnim->m_uSkinID );
	}	

	m_pxAsuraAnim->PlayAnim( m_pxAsuraAnim->m_uAnimID, m_pxAsuraAnim->m_uAnimFlags );

	if (m_bChanged)
	{
		m_pxAsuraAnim->m_xRenderOrient.y = Asura_Maths::RadiansToZeroCentredRange(m_pxAsuraAnim->m_xRenderOrient.y);
		m_pxAsuraAnim->m_xCameraOrient.y = Asura_Maths::RadiansToZeroCentredRange(m_pxAsuraAnim->m_xCameraOrient.y);
	}
}

void Tools_GUIMenu_Element_Animation::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	PARENT::AddOverlayControls( pxOverlay, xOffset );
	
	Asura_Dialog* pxDialog = NULL;
	pxDialog = new Asura_Divider_Dialog( xOffset );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	AddSkinPickerDialog( "Skin", pxOverlay, xOffset, &m_pxAsuraAnim->m_uSkinID, g_xSkinName, "Select the skin to display." );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	AddSkinPickerDialog( "Anim", pxOverlay, xOffset, &m_pxAsuraAnim->m_uAnimID, g_xAnimName, "Select the animation to play." );
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Dialog( "Camera Offset:", xOffset );
	pxDialog->SetHelpText( "Set the camera offset used when rendering the skin. Pull the camera back (decrease the Z value) to make the object appear smaller." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x += 2.0f;

 	pxDialog = new Asura_Numerical_Dialog( "x", xOffset, &(m_pxAsuraAnim->m_xCameraOffset.x), -999.9f, 999.9f, 0.01f, "%2.2f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Numerical_Dialog( "y", xOffset, &(m_pxAsuraAnim->m_xCameraOffset.y), -999.9f, 999.9f, 0.01f, "%2.2f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Numerical_Dialog( "z", xOffset, &(m_pxAsuraAnim->m_xCameraOffset.z), -999.9f, 0, 0.01f, "%2.2f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.x -= 2.0f;

	pxDialog = new Asura_Dialog( "Initial Orientation:", xOffset );
	pxDialog->SetHelpText( "Set the initial orientation used for rendering." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x += 2.0f;

	pxDialog = new Asura_AngleNumerical_Dialog( "x", xOffset, &m_pxAsuraAnim->m_xRenderOrient.x, -180.0f, 180.0f, 1.0f, "%2.1f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_AngleNumerical_Dialog( "y", xOffset, &m_pxAsuraAnim->m_xRenderOrient.y, -360.0f, 360.0f, 1.0f, "%2.1f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_AngleNumerical_Dialog( "z", xOffset, &m_pxAsuraAnim->m_xRenderOrient.z, -180.0f, 180.0f, 1.0f, "%2.1f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.x -= 2.0f;

	pxDialog = new Asura_Dialog( "Skin Offset:", xOffset );
	pxDialog->SetHelpText( "Set the position used when rendering the skin." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x += 2.0f;

 	pxDialog = new Asura_Numerical_Dialog( "x", xOffset, &(m_pxAsuraAnim->m_xSkinOffset.x), -999.9f, 999.9f, 0.01f, "%2.2f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Numerical_Dialog( "y", xOffset, &(m_pxAsuraAnim->m_xSkinOffset.y), -999.9f, 999.9f, 0.01f, "%2.2f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Numerical_Dialog( "z", xOffset, &(m_pxAsuraAnim->m_xSkinOffset.z), -999.9f, 999.9f, 0.01f, "%2.2f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.x -= 2.0f;

	pxDialog = new Asura_Dialog( "Camera Orientation:", xOffset );
	pxDialog->SetHelpText( "Set the camera orientation used for rendering." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x += 2.0f;

	pxDialog = new Asura_AngleNumerical_Dialog( "x", xOffset, &m_pxAsuraAnim->m_xCameraOrient.x, -180.0f, 180.0f, 1.0f, "%2.1f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_AngleNumerical_Dialog( "y", xOffset, &m_pxAsuraAnim->m_xCameraOrient.y, -360.0f, 360.0f, 1.0f, "%2.1f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_AngleNumerical_Dialog( "z", xOffset, &m_pxAsuraAnim->m_xCameraOrient.z, -180.0f, 180.0f, 1.0f, "%2.1f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.x -= 2.0f;

	pxDialog = new Asura_Dialog( "Rotation Speed:", xOffset );
	pxDialog->SetHelpText( "Set the speed of rotation for the skin." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x += 2.0f;

	pxDialog = new Asura_AngleNumerical_Dialog( "x", xOffset, &m_pxAsuraAnim->m_xRotationSpeed.x, -180.0f, 180.0f, 1.0f, "%2.1f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_AngleNumerical_Dialog( "y", xOffset, &m_pxAsuraAnim->m_xRotationSpeed.y, -180.0f, 180.0f, 1.0f, "%2.1f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_AngleNumerical_Dialog( "z", xOffset, &m_pxAsuraAnim->m_xRotationSpeed.z, -180.0f, 180.0f, 1.0f, "%2.1f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.x -= 2.0f;

	pxDialog = new Asura_AngleNumerical_Dialog( "Field of View", xOffset, &m_pxAsuraAnim->m_fFOVAngle, 0.0f, 360.0f, 1.0f, "%2.1f", &m_bChanged );
	pxOverlay->AddDialog( pxDialog );

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Tickbox_Dialog( "Vertical", xOffset, &m_pxAsuraAnim->m_bUseVerticalFOV, &m_bChanged );
	pxDialog->SetHelpText("Whether the specified field of view is horizontal or vertical");
	pxOverlay->AddDialog( pxDialog );

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Tickbox_Dialog( "Stretch", xOffset, &m_pxAsuraAnim->m_bStretch, &m_bChanged );
	pxDialog->SetHelpText("Whether to stretch the animation with the box or maintain screen aspect ratio");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	xOffset.x = 1.0f;

	pxDialog = new Asura_FlagTickbox_Dialog( "Looped", xOffset, &m_pxAsuraAnim->m_uAnimFlags, AACA_FLAG_LOOPED, &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Reversed", xOffset, &m_pxAsuraAnim->m_uAnimFlags, AACA_FLAG_REVERSED, &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog( "Mirrored", xOffset, &m_pxAsuraAnim->m_uAnimFlags, AACA_FLAG_MIRRORED, &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Export Skin & Anim", xOffset, &m_pxAsuraAnim->m_bExportSkinAndAnim, &m_bChanged );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_GUIMenu_Element_Animation::AddSkinPickerDialog( const char* szName, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, Asura_Hash_ID* puOutputValue, Asura_String& xDisplayString, const char* szHelpText/*=NULL*/, bool* pbChanged/*=NULL*/ )
{
	// find the current name from the hash ID	
	if( *puOutputValue == ASURA_HASH_ID_UNSET )
	{
		xDisplayString = "<UNSET>";
	}
	else
	{
		const char* szFilename = Asura_HashedFilename_System::GetFilename( *puOutputValue, false );
		xDisplayString = szFilename ? szFilename : "<UNKNOWN>";
	}

	Asura_Dialog* pxDialog = new Asura_Function_Dialog( szName, xOffset, ChooseSkin, (int)puOutputValue, pbChanged );
	pxDialog->SetHelpText( szHelpText );
	pxOverlay->AddDialog( pxDialog );
	
	const float fXOffset = pxDialog->GetMaxCoords().x + 2.0f;
	xOffset.x += fXOffset;
	
	pxDialog = new Asura_Dynamic_Dialog( &xDisplayString, xOffset );
	pxDialog->m_iControlID = uASURA_GUIMENU_ELEMENT_ANIMATION_DIALOGID_SKINPICKER + (int)puOutputValue;
	pxOverlay->AddDialog( pxDialog );

	xOffset.x -= fXOffset;
}

void Tools_GUIMenu_Element_Animation::ChooseSkin( int iData )
{
	Tools_FileSystem_DialogInfo	xFileSelector;

	xFileSelector.DefaultExtension		= ".ASR";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= "Characters";
	xFileSelector.FileFilter			= "Asura Files\0*.ASR;\0";
	xFileSelector.WindowTitle			= "Select Skin/Anim File";
	xFileSelector.AllowMultipleSelect	= false;

	Asura_Hash_ID uSelectedHashID = ASURA_HASH_ID_UNSET;
	Asura_String xSelectedString = "<UNSET>";

	Asura_String xFilename = Tools_FileSystem::GetLoadFilename( xFileSelector );
	if( xFilename.IsNull() )
	{
		// pop up a window asking if they want it to be added as 'unset'
		YesNo_Overlay xYesNo( "No file selected. Do you wish to set the value to 'UNSET' (ASURA_HASHID_UNSET)." );
		if( !xYesNo.DoModal() ) return;
	}
	else
	{
		// check this is actually a skin/anim asura file
		Tools_AsuraFileContents xFileContents;
		if( !xFileContents.LoadAsuraFile( xFilename.GetString() ) )	
		{
			Asura_Assertf( false, ("Failed to get file contents for file '%s'", xFilename.GetString()) );
			return;
		}

		if( (xFileContents.GetFileType() != AsuraFile_Skin) && (xFileContents.GetFileType() != AsuraFile_Anim) )
		{
			Asura_Assertf( false, ("File '%s' is not a skin or anim.", xFilename.GetString()) );
			return;
		}

		// split the path to get the hash ID
		Asura_String xPath;
		Asura_String xName;
		Asura_String xExt;

		xFilename.SplitFilename( &xPath, &xName, &xExt );
		uSelectedHashID = Asura_GetHashID( xName.GetString() );
		xSelectedString = xName;

		// make sure it's been loaded
		bool bLoaded = (xFileContents.GetFileType() == AsuraFile_Skin) 
					 ? (Asura_Animation_System::GetSkinByHash( uSelectedHashID ) != NULL) 
					 : (Asura_Animation_System::GetAnimByHash( uSelectedHashID ) != NULL) ;

		if( !bLoaded )
		{
			LoadAsuraFile( xFilename.GetString() );
		}
	}

	Asura_Hash_ID* puOutputHash = reinterpret_cast<Asura_Hash_ID*> (iData);
	if( puOutputHash )
	{
		*puOutputHash = uSelectedHashID;
	}

 	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetCurrentOverlay();
	if( pxOverlay )
	{
		Asura_Dynamic_Dialog* pxDialog = static_cast<Asura_Dynamic_Dialog*> (pxOverlay->GetDialog( uASURA_GUIMENU_ELEMENT_ANIMATION_DIALOGID_SKINPICKER + iData ));
		if( pxDialog )
		{
			pxDialog->SetNameValue( xSelectedString );
		}
	}
}



/***************/
/* End of File */
/***************/
#endif // ASURA_USE_GUI_MENU_SYSTEM
