//
// Tools_GUIMenu_Element_Text.cpp
//
// Created by Richard Rice - 05/12/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "GUIMenu/Tools_GUIMenu_Element_Text.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

// Asura
#include "Core/Asura_HashedLocalisedText.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "GUIMenu/Asura_GUIMenu_Element_Text.h"
#include "GUIMenu/Asura_GUIMenu_Modifier.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Rendering/Asura_Font_System.h"

// Tools
#include "GUIMenu/Tools_Mode_GUIMenu.h"
#include "Misc/Tools_Modal_Overlay.h"

static const Asura_Char* g_apcTeletypeDirNames[] =
{
	"Top Left",  // ASURA_GUIMENU_TELETYPE_TOP_LEFT
	"Top Right",  // ASURA_GUIMENU_TELETYPE_TOP_RIGHT
	"Bottom Left",  // ASURA_GUIMENU_TELETYPE_BOTTOM_LEFT
	"Bottom Right",  // ASURA_GUIMENU_TELETYPE_BOTTOM_RIGHT
	"Sweep Left", //ASURA_GUIMENU_TELETYPE_SWEEP_LEFT
	"Sweep Right", //ASURA_GUIMENU_TELETYPE_SWEEP_RIGHT
};

static const u_int g_uNumTeletypeDirs = Asura_Font_Special_Effects::ASURA_GUIMENU_TELETYPE_DIR_MAX;

// Tools effects class.
// Creates an overlay tab for Text and Status Text elements.
Tools_GUIMenu_Font_Special_Effects::Tools_GUIMenu_Font_Special_Effects()
{
}

void Tools_GUIMenu_Font_Special_Effects::Initialise()
{
	m_bSpecialEffects = false;
	m_fDelayTime = 0.0f;
	m_bTeleType = false;
	m_uCharsPerSecond = 100;
	m_fAlphaFadeTime = 0.0f;
	m_iTeletypeDirIndex = 0;
	m_bDecrypt = false;
	m_fDecryptTime = 0.0f;
	m_fDecryptChangeCharTime = 0.0f;
	m_bCursor = false;
	m_fCursorFlashTime = 0.0f;
	m_fCursorStopTime = 0.0f;

	m_pxPlayEffects = NULL;
}

void Tools_GUIMenu_Font_Special_Effects::Copy(const Tools_GUIMenu_Font_Special_Effects* pxSource)
{
	m_bSpecialEffects = pxSource->m_bSpecialEffects;

	m_fDelayTime =  pxSource->m_fDelayTime;

	m_bTeleType = pxSource->m_bTeleType;
	m_uCharsPerSecond = pxSource->m_uCharsPerSecond;
	m_fAlphaFadeTime = pxSource->m_fAlphaFadeTime;
	m_iTeletypeDirIndex = pxSource->m_iTeletypeDirIndex;

	m_bDecrypt =  pxSource->m_bDecrypt;
	m_fDecryptTime = pxSource->m_fDecryptTime;
	m_fDecryptChangeCharTime = pxSource->m_fDecryptChangeCharTime;

	m_bCursor =  pxSource->m_bCursor;
	m_fCursorFlashTime =  pxSource->m_fCursorFlashTime;
	m_fCursorStopTime = pxSource->m_fCursorStopTime;

	m_pxPlayEffects = NULL;
}

// Copy the data from the Asura class into the tools overlay
void Tools_GUIMenu_Font_Special_Effects::CopyFromAsura(Asura_Font_Special_Effects* pxEffects)
{
	m_pxPlayEffects = pxEffects;

	if ( pxEffects != NULL )
	{
		m_bSpecialEffects = true;

		m_fDelayTime = pxEffects->GetDelayTime();

		m_bTeleType = pxEffects->GetTeletype();
		m_uCharsPerSecond = pxEffects->GetCharsPerSecond();
		m_fAlphaFadeTime = pxEffects->GetAlphaFadeTime();

		m_iTeletypeDirIndex = pxEffects->GetTeletypeDir();

		m_bDecrypt = pxEffects->GetDecrypt();
		m_fDecryptTime = pxEffects->GetDecryptTime();
		m_fDecryptChangeCharTime = pxEffects->GetDecryptChangeCharTime();

		m_bCursor = pxEffects->GetCursor();
		m_fCursorFlashTime = pxEffects->GetCursorFlashTime();
		m_fCursorStopTime = pxEffects->GetCursorStopTime();
	}
	else
	{
		m_bSpecialEffects = false;
		m_fDelayTime = 0.0f;
		m_bTeleType = false;
		m_uCharsPerSecond = 100;
		m_fAlphaFadeTime = 0.0f;
		m_iTeletypeDirIndex = 0;
		m_bDecrypt = false;
		m_fDecryptTime = 0.0f;
		m_fDecryptChangeCharTime = 0.0f;
		m_bCursor = false;
		m_fCursorFlashTime = 0.0f;
		m_fCursorStopTime = 0.0f;
	}
}

// Copy the data from the tools overlay to to asura class
void Tools_GUIMenu_Font_Special_Effects::CopyToAsura(Asura_Font_Special_Effects* pxEffects)
{
	m_pxPlayEffects = pxEffects;

	if ( pxEffects != NULL )
	{
		pxEffects->SetDelayTime(m_fDelayTime);

		pxEffects->SetTeletype(m_bTeleType);
		pxEffects->SetCharsPerSecond(m_uCharsPerSecond);
		pxEffects->SetAlphaFadeTime(m_fAlphaFadeTime);
		pxEffects->SetTeletypeDir(static_cast<Asura_Font_Special_Effects::TeletypeDir>(m_iTeletypeDirIndex));

		pxEffects->SetDecrypt(m_bDecrypt);
		pxEffects->SetDecryptTime(m_fDecryptTime);
		pxEffects->SetDecryptChangeCharTime(m_fDecryptChangeCharTime);

		pxEffects->SetCursor(m_bCursor);
		pxEffects->SetCursorFlashTime(m_fCursorFlashTime);
		pxEffects->SetCursorStopTime(m_fCursorStopTime);
	}
	else
	{
		m_bSpecialEffects = false;
		m_fDelayTime = 0.0f;
		m_bTeleType = false;
		m_uCharsPerSecond = 100;
		m_fAlphaFadeTime = 0.0f;
		m_iTeletypeDirIndex = 0;
		m_bDecrypt = false;
		m_fDecryptTime = 0.0f;
		m_fDecryptChangeCharTime = 0.0f;
		m_bCursor = false;
		m_fCursorFlashTime = 0.0f;
		m_fCursorStopTime = 0.0f;
	}
}

// Add an effects tab to the overlay
void Tools_GUIMenu_Font_Special_Effects::AddEffectsOverlayControls(Asura_Overlay* pxOverlay, Asura_Tabs_Dialog* pxTabs, Asura_Vector_2& xOffset, bool* pbChanged)
{
	Asura_Dialog* pxDialog;

	pxTabs->AddTab("Text Effects");

	pxDialog = new Asura_Tickbox_Dialog("Special Effects", xOffset, &m_bSpecialEffects, pbChanged);
	pxDialog->SetHelpText("Enables special effects for text.");
	pxDialog->SetIsEnabledFunction(CanEnableSpecialEffects,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Delay Time", xOffset, &m_fDelayTime, 0.000f, 100.0f, 0.01f, "%0.3f", pbChanged);
	pxDialog->SetIsEnabledFunction(CanEditSpecialEffects,(int)this);
	pxDialog->SetHelpText("How long before the effect starts");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Teletype", xOffset, &m_bTeleType, pbChanged);
	pxDialog->SetIsEnabledFunction(CanEditSpecialEffects,(int)this);
	pxDialog->SetHelpText("Enables teletype settings");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Characters Per Second", xOffset, &m_uCharsPerSecond, 1, 10000, 1, "%d", pbChanged);
	pxDialog->SetIsEnabledFunction(CanEditTeletypeSettings,(int)this);
	pxDialog->SetHelpText("Number of characters revealed per second.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_DropDown_List_Dialog( xOffset,
		&g_apcTeletypeDirNames[0],
		&m_iTeletypeDirIndex,
		g_uNumTeletypeDirs, 
		g_uNumTeletypeDirs,
		NULL, pbChanged, "Direction Mode");
	pxDialog->SetHelpText("Sets the direction mode for the effect.");
	pxDialog->SetIsEnabledFunction(CanEditTeletypeSettings,(int)this);
	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Alpha Fade Time", xOffset, &m_fAlphaFadeTime, 0.000f, 10.0f, 0.01f, "%0.3f", pbChanged);
	pxDialog->SetIsEnabledFunction(CanEditTeletypeSettings,(int)this);
	pxDialog->SetHelpText("Time it takes to fade in each character. Setting to 0.0 will stop any fading.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Decrypt", xOffset, &m_bDecrypt, pbChanged);
	pxDialog->SetIsEnabledFunction(CanEditSpecialEffects,(int)this);
	pxDialog->SetHelpText("Enables decryption settings");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Decryption Time", xOffset, &m_fDecryptTime, 0.000f, 100.0f, 0.01f, "%0.3f", pbChanged);
	pxDialog->SetIsEnabledFunction(CanEditDecryptSettings,(int)this);
	pxDialog->SetHelpText("Time it takes to decrypt the string");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Decryption Character Change Time", xOffset, &m_fDecryptChangeCharTime, 0.000f, 100.0f, 0.01f, "%0.3f", pbChanged);
	pxDialog->SetIsEnabledFunction(CanEditDecryptSettings,(int)this);
	pxDialog->SetHelpText("Time to render an encrypted character before changing to another on.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Cursor", xOffset, &m_bCursor, pbChanged);
	pxDialog->SetIsEnabledFunction(CanEditSpecialEffects,(int)this);
	pxDialog->SetHelpText("Enables rendering of cursor");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Cursor Flash Time", xOffset, &m_fCursorFlashTime, 0.000f, 100.0f, 0.01f, "%0.3f", pbChanged);
	pxDialog->SetIsEnabledFunction(CanEditCursorSettings,(int)this);
	pxDialog->SetHelpText("Flash rate for the cursor");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Cursor Wait Time", xOffset, &m_fCursorStopTime, 0.000f, 100.0f, 0.01f, "%0.3f", pbChanged);
	pxDialog->SetIsEnabledFunction(CanEditCursorSettings,(int)this);
	pxDialog->SetHelpText("Cursor will stop drawing once it has been displayed from this length of time at the end of the text.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Play Effect", xOffset, PlayEffect, (int)this);
	pxDialog->SetIsEnabledFunction( CanPlayEffect,(int)this );
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
}

bool Tools_GUIMenu_Font_Special_Effects::CanEnableSpecialEffects()
{
	return !m_bDisableEditing;
}

bool Tools_GUIMenu_Font_Special_Effects::CanEditSpecialEffects()
{
	return m_bSpecialEffects && !m_bDisableEditing;
}

bool Tools_GUIMenu_Font_Special_Effects::CanEditTeletypeSettings()
{
	return m_bTeleType && !m_bDisableEditing;
}

bool Tools_GUIMenu_Font_Special_Effects::CanEditDecryptSettings()
{
	return m_bDecrypt && !m_bDisableEditing;
}

bool Tools_GUIMenu_Font_Special_Effects::CanEditCursorSettings()
{
	return m_bCursor && !m_bDisableEditing;
}

bool Tools_GUIMenu_Font_Special_Effects::CanPlayEffect()
{
	return m_pxPlayEffects != NULL;
}

void Tools_GUIMenu_Font_Special_Effects::PlayEffect()
{
	if ( m_bSpecialEffects )
	{
		m_pxPlayEffects->Start();
	}
}

// --------------------------------------------------------------------------------------

struct TextElementFormatingRenderFlag
{
	const Asura_Char* m_szName;
	u_int m_uValue;
};

static TextElementFormatingRenderFlag g_axHorzFormatFlags[] =
{
	{"None", 0},
	{"Left - Justify", ASURA_GUIMENU_TEXT_JUSTIFY_LEFT},
	{"Centre - Justify", ASURA_GUIMENU_TEXT_JUSTIFY_CENTRE},
	{"Right - Justify", ASURA_GUIMENU_TEXT_JUSTIFY_RIGHT},
};

static const Asura_Char* g_apcHorzFormatFlagNames[] =
{
	g_axHorzFormatFlags[0].m_szName,
	g_axHorzFormatFlags[1].m_szName,
	g_axHorzFormatFlags[2].m_szName,
	g_axHorzFormatFlags[3].m_szName
};

static const u_int g_uNumHorzRenderFlags = sizeof(g_axHorzFormatFlags) / sizeof(TextElementFormatingRenderFlag);

static TextElementFormatingRenderFlag g_axVertFormatFlags[] =
{
	{"None", 0},
	{"Centre - Justify", ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_CENTRE},
	{"Bottom - Justify", ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_BOTTOM},
};

static const Asura_Char* g_apcVertFormatFlagNames[] =
{
	g_axVertFormatFlags[0].m_szName,
	g_axVertFormatFlags[1].m_szName,
	g_axVertFormatFlags[2].m_szName
};

static const u_int g_uNumVertRenderFlags = sizeof(g_axVertFormatFlags) / sizeof(TextElementFormatingRenderFlag);

static TextElementFormatingRenderFlag g_axShadowFlags[] =
{
	{"None", 0},
	{"Shadow - Light", ASURA_GUIMENU_TEXT_DROPSHADOW_LIGHT},
	{"Shadow - Hard", ASURA_GUIMENU_TEXT_DROPSHADOW}
};

static const Asura_Char* g_apcShadowFlagNames[] =
{
	g_axShadowFlags[0].m_szName,
	g_axShadowFlags[1].m_szName,
	g_axShadowFlags[2].m_szName
};

static const u_int g_uNumShadowFlags = sizeof(g_axShadowFlags) / sizeof(TextElementFormatingRenderFlag);

// --------------------------------------------------------------------------------------


int Tools_GUIMenu_Element_Text::s_iMostRecentSelectedFont = 0;

Tools_GUIMenu_Element_Text::Tools_GUIMenu_Element_Text(Asura_GUIMenu_Element_Text* pxAsuraText,Tools_GUIMenu_Root* pxToolsParent)
:Tools_GUIMenu_Element_Base(pxAsuraText,pxToolsParent)
{
	u_int u;

	m_pxAsuraText = pxAsuraText;

	for( u=0; u<ASURA_GUIMENU_TEXT_ELEMENT_MAX_CUSTOM_STRING_SIZE; u++)
	{
		m_wszString[u] = 0x0;
	}

	if( m_pxAsuraText->GetText())
	{
		wcsncpy( &m_wszString[0], m_pxAsuraText->GetText(), ASURA_GUIMENU_TEXT_ELEMENT_MAX_CUSTOM_STRING_SIZE);
	}

	m_fOffsetX = m_pxAsuraText->GetOffsetX();
	m_fOffsetY = m_pxAsuraText->GetOffsetY();
	m_fScrollRate = m_pxAsuraText->GetScrollRate();

	// MRD 15/06/07: Line Spacing setting
	m_fLineSpacing = m_pxAsuraText->GetTextLineSpacing();
	
	m_iCurrentHorzJustifyIndex		= 0;
	m_bHorzJustifyChanged			= false;
	m_iCurrentVertJustifyIndex		= 0;
	m_bVertJustifyChanged			= false;
	m_iCurrentShadowIndex			= 0;
	m_bShadowChanged				= false;

	m_uHorzRenderFlag				= 0;
	m_uVertRenderFlag				= 0;
	m_uShadowFlag					= 0;

	m_bWordWrap = m_pxAsuraText->GetTextRenderFlag(ASURA_GUIMENU_TEXT_WORDWRAP);
	m_bAutoScale = m_pxAsuraText->GetTextRenderFlag(ASURA_GUIMENU_TEXT_SCALE_ON_PARENT_RESIZE);

	m_xToolsSpecialEffects.Initialise();
}

void Tools_GUIMenu_Element_Text::InitialiseNewItem()
{
	PARENT::InitialiseNewItem();

	//set an initial font
	const Asura_Font* pxFont = Asura_Font_System::GetFont(s_iMostRecentSelectedFont);
	if(pxFont)
	{
		m_pxAsuraText->SetFontByName(pxFont->m_szFileName);
	}
	
}

void Tools_GUIMenu_Element_Text::Update()
{	
	PARENT::Update();

	if( m_bHorzJustifyChanged)
	{
		if( m_iCurrentHorzJustifyIndex < g_uNumHorzRenderFlags)
		{
			m_uHorzRenderFlag = g_axHorzFormatFlags[m_iCurrentHorzJustifyIndex].m_uValue;
		
			m_pxAsuraText->SetTextRenderFlag(ASURA_GUIMENU_TEXT_JUSTIFY_LEFT|ASURA_GUIMENU_TEXT_JUSTIFY_CENTRE|ASURA_GUIMENU_TEXT_JUSTIFY_RIGHT,false);
			m_pxAsuraText->SetTextRenderFlag( m_uHorzRenderFlag, true);
		}

		SetModified();
		m_bChanged = true;
		m_bHorzJustifyChanged = false;
	}

	if( m_bVertJustifyChanged)
	{
		if( m_iCurrentVertJustifyIndex < g_uNumVertRenderFlags)
		{
			m_uVertRenderFlag = g_axVertFormatFlags[m_iCurrentVertJustifyIndex].m_uValue;
			m_pxAsuraText->SetTextRenderFlag(ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_CENTRE|ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_BOTTOM,false);
			m_pxAsuraText->SetTextRenderFlag( m_uVertRenderFlag, true);
		}

		SetModified();
		m_bChanged = true;

		m_bVertJustifyChanged = false;
	}

	if( m_bShadowChanged)
	{
		if( m_iCurrentShadowIndex < g_uNumShadowFlags)
		{
			m_uShadowFlag = g_axShadowFlags[m_iCurrentShadowIndex].m_uValue;
			m_pxAsuraText->SetTextRenderFlag(ASURA_GUIMENU_TEXT_DROPSHADOW | ASURA_GUIMENU_TEXT_DROPSHADOW_LIGHT, false);
			m_pxAsuraText->SetTextRenderFlag( m_uShadowFlag, true);
		}

		SetModified();
		m_bChanged = true;

		m_bShadowChanged = false;
	}

	if ( m_pxAsuraText->GetStyleObjectHash() != ASURA_HASH_ID_UNSET )
	{
		m_xToolsSpecialEffects.DisableEditing(true);
	}
	else
	{
		m_xToolsSpecialEffects.DisableEditing(false);
	}
}


void Tools_GUIMenu_Element_Text::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	int i;

	PARENT::AddOverlayControls(pxOverlay,xOffset);

	Asura_Dialog* pxDialog;

	xOffset.y += Asura_Dialog::SmallSpacing();

	//don't really want to be setting text like this. But just to get something up there...
//	pxDialog = new Asura_String_Dialog("Text",xOffset,&m_xString,30,&m_bChanged);
//	((Asura_String_Dialog*)pxDialog)->SetMaxStringLength(200);
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.y += Asura_Dialog::DefaultSpacing();

	//temporary text selection control
	pxDialog = new Asura_Function_Dialog("Text Page",xOffset,ChooseTextPage,(int)this);
	//pxDialog->SetIsEnabledFunction(CanEditTextPage,(int)this);
	pxDialog->SetHelpText("Select the text page to use.");
	pxDialog->m_iControlID = Asura_GetHashID("SelectTextPage");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Text",xOffset,ChooseText,(int)this);
	pxDialog->SetIsEnabledFunction(CanEditTextID,(int)this);
	pxDialog->SetHelpText("Select the text to use.");
	pxDialog->m_iControlID = Asura_GetHashID("SelectText");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Function_Dialog("Font",xOffset,ChooseFont,(int)this);
	pxDialog->SetIsEnabledFunction(CanEditFontName,(int)this);
	pxDialog->SetHelpText("Select the font for this text item");
	pxDialog->m_iControlID = Asura_GetHashID("SelectFont");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Font Scale", xOffset, m_pxAsuraText->GetFontScalePointer(), 0.0f, 9999.0f, 0.001f, "%3.3f", &m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEditFontScale,(int)this);
	pxDialog->SetHelpText("Set the font scale for this text item");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Numerical_Dialog("X Offset", xOffset, &m_fOffsetX, -999.0f, 999.0f, 1.0f, "%.0f", &m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEditXOffset,(int)this);
	pxDialog->SetHelpText("Offset to position at which the text gets drawn.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Numerical_Dialog("Y Offset", xOffset, &m_fOffsetY, -999.0f, 999.0f, 1.0f, "%.0f", &m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEditYOffset,(int)this);
	pxDialog->SetHelpText("Offset to position at which the text gets drawn.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Scroll Rate", xOffset, &m_fScrollRate, 0.0f, 999.0f, 0.1f, "%.1f", &m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEditXOffset,(int)this);
	pxDialog->SetHelpText("Scroll rate in pixels/second to use if the text is too long to fit in the box.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// MRD 15/06/07: Allow adjustment of line spacing for elements
	pxDialog = new Asura_Numerical_Dialog("Line Spacing", xOffset, &m_fLineSpacing, 0.0f, 2.0f, 0.05f, "%.05f", &m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEditLineSpacing,(int)this);
	pxDialog->SetHelpText("Offset of spacing between horizontal lines (0.0 will overlap completely, 1.0 will be the text height)");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Word Wrap", xOffset, &m_bWordWrap, &m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEditRenderFlags,(int)this);
	pxDialog->SetHelpText("Enables word wrapping if the string is too wide to fit.\nIgnored if using x offset , or scrolling");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Scale on resize", xOffset, &m_bAutoScale, &m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEditRenderFlags,(int)this);
	pxDialog->SetHelpText("Enables font to rescale when element is scaled");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();



	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_uHorzRenderFlag = 0;

	if( m_pxAsuraText->GetTextRenderFlag( ASURA_GUIMENU_TEXT_JUSTIFY_LEFT))
	{
		m_uHorzRenderFlag = ASURA_GUIMENU_TEXT_JUSTIFY_LEFT;
	}
	else if( m_pxAsuraText->GetTextRenderFlag( ASURA_GUIMENU_TEXT_JUSTIFY_CENTRE))
	{
		m_uHorzRenderFlag = ASURA_GUIMENU_TEXT_JUSTIFY_CENTRE;
	}
	else if( m_pxAsuraText->GetTextRenderFlag( ASURA_GUIMENU_TEXT_JUSTIFY_RIGHT))
	{
		m_uHorzRenderFlag = ASURA_GUIMENU_TEXT_JUSTIFY_RIGHT;
	}

	for( i=0; i<g_uNumHorzRenderFlags; i++)
	{
		if( g_axHorzFormatFlags[i].m_uValue == m_uHorzRenderFlag)
		{
			m_iCurrentHorzJustifyIndex = i;
			break;
		}
	}

	pxDialog = new Asura_DropDown_List_Dialog( xOffset,
												&g_apcHorzFormatFlagNames[0],
												&m_iCurrentHorzJustifyIndex,
												g_uNumHorzRenderFlags, 
												g_uNumHorzRenderFlags,
												NULL, &m_bHorzJustifyChanged, "Text Justify - Horz");

	pxDialog->SetIsEnabledFunction(CanEditRenderFlags,(int)this);
	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	m_uVertRenderFlag = 0;

	if( m_pxAsuraText->GetTextRenderFlag( ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_BOTTOM))
	{
		m_uVertRenderFlag = ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_BOTTOM;
	}
	else if( m_pxAsuraText->GetTextRenderFlag( ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_CENTRE))
	{
		m_uVertRenderFlag = ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_CENTRE;
	}

	for( i=0; i<g_uNumVertRenderFlags; i++)
	{
		if( g_axVertFormatFlags[i].m_uValue == m_uVertRenderFlag)
		{
			m_iCurrentVertJustifyIndex = i;
			break;
		}
	}


	pxDialog = new Asura_DropDown_List_Dialog( xOffset,
												&g_apcVertFormatFlagNames[0],
												&m_iCurrentVertJustifyIndex,
												g_uNumVertRenderFlags, 
												g_uNumVertRenderFlags,
												NULL,
												&m_bVertJustifyChanged, "Text Justify - Vert");

	pxDialog->SetIsEnabledFunction(CanEditRenderFlags,(int)this);
	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	m_uShadowFlag = 0;

	if( m_pxAsuraText->GetTextRenderFlag( ASURA_GUIMENU_TEXT_DROPSHADOW_LIGHT))
	{
		m_uShadowFlag = ASURA_GUIMENU_TEXT_DROPSHADOW_LIGHT;
	}
	else if( m_pxAsuraText->GetTextRenderFlag( ASURA_GUIMENU_TEXT_DROPSHADOW))
	{
		m_uShadowFlag = ASURA_GUIMENU_TEXT_DROPSHADOW;
	}

	m_iCurrentShadowIndex = 0;

	for( i=0; i<g_uNumShadowFlags; i++)
	{
		if( g_axShadowFlags[i].m_uValue == m_uShadowFlag)
		{
			m_iCurrentShadowIndex = i;
			break;
		}
	}

	pxDialog = new Asura_DropDown_List_Dialog( xOffset,
												&g_apcShadowFlagNames[0],
												&m_iCurrentShadowIndex,
												g_uNumShadowFlags, 
												g_uNumShadowFlags,
												NULL, &m_bShadowChanged, "Drop Shadow");

	pxDialog->SetIsEnabledFunction(CanEditRenderFlags,(int)this);
	pxOverlay->AddDialog( pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
}

void Tools_GUIMenu_Element_Text::AddExtraOverlayControls(Asura_Overlay* pxOverlay, Asura_Tabs_Dialog* pxTabs, Asura_Vector_2& xOffset)
{
	m_xToolsSpecialEffects.CopyFromAsura(m_pxAsuraText->GetSpecialEffectsSettings());
	m_xToolsSpecialEffects.AddEffectsOverlayControls(pxOverlay,pxTabs,xOffset,&m_bChanged);
}

void Tools_GUIMenu_Element_Text::UpdateOverlayControls(Asura_Overlay* pxOverlay)
{
	PARENT::UpdateOverlayControls(pxOverlay);

	//update font control
	char szText[100];
	Asura_PrintfToBfr(szText,"Font : %s",sizeof(szText),true,m_pxAsuraText->GetFontName());

	pxOverlay->UpdateDialogLabel(Asura_GetHashID("SelectFont"),szText);

	//update modifier font control
	if(pxOverlay->GetDialog(Asura_GetHashID("SelectModifierFont")))
	{
		Asura_PrintfToBfr(szText,"Font : %s",sizeof(szText),true,m_xModifierData_String);
		pxOverlay->UpdateDialogLabel(Asura_GetHashID("SelectModifierFont"),szText);
	}

	// Update text page
	const Asura_String* pxTextPageName = Asura_GUIMenu_System::GetMenuTextFilenameFromHash(m_pxAsuraText->GetTextPageHash());
	Asura_Assert(pxTextPageName, "Could not find text page name");
	Asura_PrintfToBfr(szText,"Text Page : %s",sizeof(szText),true, pxTextPageName ? pxTextPageName->GetString() : "*Unknown*");	
	pxOverlay->UpdateDialogLabel(Asura_GetHashID("SelectTextPage"),szText);

	//update text control
	Asura_Hash_ID uTextID = m_pxAsuraText->GetTextID();
	if(uTextID)
	{
		const Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::GetPage(m_pxAsuraText->GetTextPageHash());
		
		const char* szTextID = 0;

		if(pxPage && pxPage->HasTextIDStrings())
		{
			szTextID = pxPage->GetTextIDString(uTextID);
		}

		if(szTextID)
		{
			Asura_PrintfToBfr(szText,"Text : %s",sizeof(szText),true,szTextID);	
		}
		else
		{
			strcpy(szText,"Text : *Unknown*");
		}

	}
	else
	{
		strcpy(szText,"Text : *None*");
	}	
	pxOverlay->UpdateDialogLabel(Asura_GetHashID("SelectText"),szText);

	//update text control for modifiers
	if(pxOverlay->GetDialog(Asura_GetHashID("SelectModifierText")))
	{
		uTextID = m_uModifierData;
		if(uTextID)
		{
			const Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::GetPage(m_pxAsuraText->GetTextPageHash());
			
			const char* szTextID = 0;

			if(pxPage && pxPage->HasTextIDStrings())
			{
				szTextID = pxPage->GetTextIDString(uTextID);
			}

			if(szTextID)
			{
				Asura_PrintfToBfr(szText,"Text : %s",sizeof(szText),true,szTextID);	
			}
			else
			{
				strcpy(szText,"Text : *Unknown*");
			}

		}
		else
		{
			strcpy(szText,"Text : *None*");
		}	
		pxOverlay->UpdateDialogLabel(Asura_GetHashID("SelectModifierText"),szText);
	}
}

void Tools_GUIMenu_Element_Text::ChooseFont()
{
	Asura_String xCurrentFont = m_pxAsuraText->GetFontName();

	ListSelection_Overlay xOverlay("Select Font");	

	//get a list of all loaded fonts
	for(int i=0;i<ASURA_MAX_FONTS;i++)	
	{
		const Asura_Font* pxFont = Asura_Font_System::GetFont(i);
		if(!pxFont) continue;

		bool bInitialSelection = !stricmp(xCurrentFont.GetString(),pxFont->m_szFileName);
		xOverlay.AddItem(pxFont->m_szFileName,i,bInitialSelection);
	}

	if(!xOverlay.GetNumItems())
	{
		//no fonts... means there won't be a tools font either , so the user is unlikely to have got this far.
		Asura_Assert(0,"Failed to find any loaded fonts");
		return;
	}

	if(!xOverlay.DoModal()) return;

	//update the asura data
	m_pxAsuraText->SetFontByName(xOverlay.GetSelectionString());
	s_iMostRecentSelectedFont = xOverlay.GetSelectionData();

	//and the overlay
	UpdateOverlay();

	SetModified();
}

void Tools_GUIMenu_Element_Text::ChooseModifierFont()
{
	ListSelection_Overlay xOverlay("Select Font");	

	//get a list of all loaded fonts
	for(int i=0;i<ASURA_MAX_FONTS;i++)	
	{
		const Asura_Font* pxFont = Asura_Font_System::GetFont(i);
		if(!pxFont) continue;

		bool bInitialSelection = !stricmp(m_xModifierData_String.GetString(),pxFont->m_szFileName);
		xOverlay.AddItem(pxFont->m_szFileName,i,bInitialSelection);
	}

	if(!xOverlay.GetNumItems())
	{
		//no fonts... means there won't be a tools font either , so the user is unlikely to have got this far.
		Asura_Assert(0,"Failed to find any loaded fonts");
		return;
	}

	if(!xOverlay.DoModal()) return;

	m_xModifierData_String = xOverlay.GetSelectionString();
	m_bModifierChanged = true;

	UpdateOverlay();
}

void Tools_GUIMenu_Element_Text::SetFontOnSelection()
{
	ListSelection_Overlay xOverlay("Apply which font to the selection?");	

	//get a list of all loaded fonts
	for(int i=0;i<ASURA_MAX_FONTS;i++)	
	{
		const Asura_Font* pxFont = Asura_Font_System::GetFont(i);
		if(!pxFont) continue;

		xOverlay.AddItem(pxFont->m_szFileName,i,i==s_iMostRecentSelectedFont);
	}

	if(!xOverlay.GetNumItems())
	{
		//no fonts... means there won't be a tools font either , so the user is unlikely to have got this far.
		Asura_Assert(0,"Failed to find any loaded fonts");
		return;
	}

	if(!xOverlay.DoModal()) return;

	const char* szFontName = xOverlay.GetSelectionString();
	s_iMostRecentSelectedFont = xOverlay.GetSelectionData();

	Tools_GUIMenu_Root* pxSelection = Tools_Mode_GUIMenu::GetSelectedItem();
	if(pxSelection)
	{
		pxSelection->SetFontRecursive(szFontName);
	}
}

void Tools_GUIMenu_Element_Text::SetFont(const char* szFontName)
{
	PARENT::SetFont(szFontName);

	if(CanEditFontName())
	{
		m_pxAsuraText->SetFontByName(szFontName);
		SetModified();

		if(IsSelected()) UpdateOverlay();
	}
}


void Tools_GUIMenu_Element_Text::Copy(const Tools_GUIMenu_Root* pxSource)
{
	PARENT::Copy(pxSource);

	const Tools_GUIMenu_Element_Text* pxSourceText = (const Tools_GUIMenu_Element_Text*) pxSource;

	//copy tools data
	wcsncpy( &m_wszString[0], &pxSourceText->m_wszString[0], ASURA_GUIMENU_TEXT_ELEMENT_MAX_CUSTOM_STRING_SIZE);
	m_fOffsetX = pxSourceText->m_fOffsetX;
	m_fOffsetY = pxSourceText->m_fOffsetY;
	m_fScrollRate = pxSourceText->m_fScrollRate;

	// MRD 15/06/07: Line spacing setting
	m_fLineSpacing = pxSourceText->m_fLineSpacing;

	m_bWordWrap = pxSourceText->m_bWordWrap;
	m_bAutoScale = pxSourceText->m_bAutoScale;

	m_xToolsSpecialEffects.Copy(&pxSourceText->m_xToolsSpecialEffects);

	// Once tools settings have been copied make sure Asura object is correctly updated
	m_xToolsSpecialEffects.CopyFromAsura(m_pxAsuraText->GetSpecialEffectsSettings());
}


void Tools_GUIMenu_Element_Text::CopyToAsura()
{
	PARENT::CopyToAsura();

	//Setting the string like this should probably be removed now there is the ability to use localised text
	//RAR 09/02/2007
	if( wcslen( m_wszString) > 0)
	{
		m_pxAsuraText->SetText( &m_wszString[0]);
	}

	m_pxAsuraText->SetOffsetX(m_fOffsetX);
	m_pxAsuraText->SetOffsetY(m_fOffsetY);
	m_pxAsuraText->SetScrollRate(m_fScrollRate);

	// MRD 15/06/07: Line spacing setter call
	m_pxAsuraText->SetTextLineSpacing(m_fLineSpacing);

	m_pxAsuraText->SetTextRenderFlag(ASURA_GUIMENU_TEXT_WORDWRAP,m_bWordWrap);
	m_pxAsuraText->SetTextRenderFlag(ASURA_GUIMENU_TEXT_SCALE_ON_PARENT_RESIZE, m_bAutoScale);

	m_pxAsuraText->SetEnableSpecialEffects(m_xToolsSpecialEffects.GetEnabledFlag());
	m_xToolsSpecialEffects.CopyToAsura(m_pxAsuraText->GetSpecialEffectsSettings());
}

void Tools_GUIMenu_Element_Text::CopyFromAsura()
{
	PARENT::CopyFromAsura();

	m_fOffsetX = m_pxAsuraText->GetOffsetX();
	m_fOffsetY = m_pxAsuraText->GetOffsetY();
	m_fScrollRate= m_pxAsuraText->GetScrollRate();

	// MRD 15/06/07: Line Spacing setting
	m_fLineSpacing = m_pxAsuraText->GetTextLineSpacing();

	m_bWordWrap = m_pxAsuraText->GetTextRenderFlag(ASURA_GUIMENU_TEXT_WORDWRAP);
	m_bAutoScale = m_pxAsuraText->GetTextRenderFlag(ASURA_GUIMENU_TEXT_SCALE_ON_PARENT_RESIZE);

	m_xToolsSpecialEffects.CopyFromAsura(m_pxAsuraText->GetSpecialEffectsSettings());
}

void Tools_GUIMenu_Element_Text::ChooseText()
{
	Asura_Hash_ID uTextID = m_pxAsuraText->GetTextID();
	if(!Tools_Mode_GUIMenu::ChooseText(m_pxAsuraText->GetTextPageHash(), uTextID)) { return; }

	m_pxAsuraText->SetTextID(uTextID);

	UpdateOverlay();
	SetModified();
}

void Tools_GUIMenu_Element_Text::ChooseTextPage()
{
	Asura_Hash_ID uTextPage = m_pxAsuraText->GetTextPageHash();
	if(!Tools_Mode_GUIMenu::ChooseTextPage(uTextPage)) { return; }

	if(uTextPage != m_pxAsuraText->GetTextPageHash())
	{
		m_pxAsuraText->SetTextPageHash(uTextPage);
		// Text ID will not make any sense for this page.
		m_pxAsuraText->SetTextID(ASURA_HASH_ID_UNSET);
	}

	UpdateOverlay();
	SetModified();
}

void Tools_GUIMenu_Element_Text::ChooseModifierText()
{
	Asura_Hash_ID uTextID = m_uModifierData;
	if(!Tools_Mode_GUIMenu::ChooseText(m_pxAsuraText->GetTextPageHash(), uTextID)) return;

	m_uModifierData = uTextID;

	UpdateOverlay();
	m_bModifierChanged = true;
}

const char* Tools_GUIMenu_Element_Text::GetLocalDataFlagName(u_int uFlagValue) const
{
	switch(uFlagValue)
	{
		case Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_FONT_SCALE :
			return "Font Scale";

		case Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT :
			return "Text";

		case Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_FONT_NAME :
			return "Font Name";

		case Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT_ID :
			return "Text ID";

		case Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_X :
			return "X Offset/Scroll Rate";

		case Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_Y:
			return "Y Offset";

		case Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_RENDER_FLAGS :
			return "Render Flags";

		case Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_LINE_SPACING :
			return "Line Spacing";

		default :
			return PARENT::GetLocalDataFlagName(uFlagValue);

	}
}

// returns whether this object supports e.g. position, or UVs etc.
bool Tools_GUIMenu_Element_Text::IsModifierTypeSupported( ASURA_GUIMENU_MODIFY_TYPE eType ) const
{
	switch (eType)
	{
		case ASURA_GUIMENU_MODIFY_TEXT_STRING:
		case ASURA_GUIMENU_MODIFY_TEXT_SCROLL_RATE:
		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_X:
		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_Y:
		case ASURA_GUIMENU_MODIFY_TEXT_RENDER_FLAGS:
		case ASURA_GUIMENU_MODIFY_TEXT_ID:
		case ASURA_GUIMENU_MODIFY_FONT_SCALE:
		case ASURA_GUIMENU_MODIFY_FONT_NAME:
			return true;
		default:
			return PARENT::IsModifierTypeSupported(eType);
	}
}


bool Tools_GUIMenu_Element_Text::CanEditFontScale()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_FONT_SCALE);
}

bool Tools_GUIMenu_Element_Text::CanEditFontName()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_FONT_NAME);
}

bool Tools_GUIMenu_Element_Text::CanEditTextID()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT_ID);
}

bool Tools_GUIMenu_Element_Text::CanEditXOffset()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_X);
}

bool Tools_GUIMenu_Element_Text::CanEditYOffset()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_OFFSET_Y);
}

bool Tools_GUIMenu_Element_Text::CanEditRenderFlags()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_RENDER_FLAGS);
}

bool Tools_GUIMenu_Element_Text::CanEditLineSpacing()
{
	return ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_LINE_SPACING);
}

void Tools_GUIMenu_Element_Text::AddModifierOverlayControls(Asura_GUIMenu_Modifier* pxModifier, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxDialog;

	switch(pxModifier->GetModifyType())
	{
		case ASURA_GUIMENU_MODIFY_TEXT_STRING:
		{
			//get the existing text and convert from unicode
			char szText[200] = "";
			Asura_PrintfToBfr(szText,"%S",sizeof(szText),true,pxModifier->GetUnicodeCharStringData());
			m_xModifierData_String = szText;

			pxDialog = new Asura_String_Dialog("Text",xOffset,&m_xModifierData_String,30,&m_bModifierChanged);
			pxDialog->SetHelpText("The text to use when the modifier is active");
			((Asura_String_Dialog*)pxDialog)->SetMaxStringLength(200);
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			
		}
		break;

		case ASURA_GUIMENU_MODIFY_TEXT_SCROLL_RATE:
		{
			m_fModifierData = pxModifier->GetFloatData();

			pxDialog = new Asura_Numerical_Dialog("Scroll Rate", xOffset, &m_fModifierData, 0.0f, 999.0f, 0.1f, "%.1f", &m_bModifierChanged);
			pxDialog->SetHelpText("The text scroll rate to use when the modifier is active");
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
		
		}
		break;

		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_X:
		{
			m_fModifierData = pxModifier->GetFloatData();

			pxDialog = new Asura_Numerical_Dialog("Text X Offset", xOffset, &m_fModifierData, -9999.0f, 9999.0f, 1.0f, "%.0f", &m_bModifierChanged);
			pxDialog->SetHelpText("The text offset to use when the modifier is active");
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
		
		}
		break;

		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_Y:
		{
			m_fModifierData = pxModifier->GetFloatData();

			pxDialog = new Asura_Numerical_Dialog("Text Y Offset", xOffset, &m_fModifierData, -9999.0f, 9999.0f, 1.0f, "%.0f", &m_bModifierChanged);
			pxDialog->SetHelpText("The text offset to use when the modifier is active");
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
		}
		break;

		case ASURA_GUIMENU_MODIFY_TEXT_RENDER_FLAGS:
		{
			m_uModifierData = pxModifier->GetUIntData();

			int iHorizontalGroupID = Asura_GetHashID("ModifierHorizontalJustify");
			int iVerticalGroupID = Asura_GetHashID("ModifierVerticalJustify");

			pxDialog = new Asura_Dialog("Horizontal Justification :",xOffset);
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			
			pxDialog = new Asura_FlagTickbox_Dialog("Left",xOffset,&m_uModifierData, ASURA_GUIMENU_TEXT_JUSTIFY_LEFT, &m_bModifierChanged);
			pxDialog->SetRadioButtonGroup(iHorizontalGroupID,false);
			AddModifierDialog(pxDialog);
			xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

			pxDialog = new Asura_FlagTickbox_Dialog("Centre",xOffset,&m_uModifierData, ASURA_GUIMENU_TEXT_JUSTIFY_CENTRE, &m_bModifierChanged);
			pxDialog->SetRadioButtonGroup(iHorizontalGroupID,false);
			AddModifierDialog(pxDialog);
			xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

			pxDialog = new Asura_FlagTickbox_Dialog("Right",xOffset,&m_uModifierData, ASURA_GUIMENU_TEXT_JUSTIFY_RIGHT, &m_bModifierChanged);
			pxDialog->SetRadioButtonGroup(iHorizontalGroupID,false);
			AddModifierDialog(pxDialog);
			xOffset.x = 1.0f;
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Dialog("Vertical Justification :",xOffset);
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_FlagTickbox_Dialog("Centre",xOffset,&m_uModifierData, ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_CENTRE, &m_bModifierChanged);
			pxDialog->SetRadioButtonGroup(iVerticalGroupID,false);
			AddModifierDialog(pxDialog);
			xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

			pxDialog = new Asura_FlagTickbox_Dialog("Right",xOffset,&m_uModifierData, ASURA_GUIMENU_TEXT_JUSTIFY_VERTICAL_BOTTOM, &m_bModifierChanged);
			pxDialog->SetRadioButtonGroup(iVerticalGroupID,false);
			AddModifierDialog(pxDialog);
			xOffset.x = 1.0f;
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_FlagTickbox_Dialog("Word Wrap",xOffset,&m_uModifierData, ASURA_GUIMENU_TEXT_WORDWRAP, &m_bModifierChanged);
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();


		}
		break;

		case ASURA_GUIMENU_MODIFY_FONT_SCALE:
		{
			m_fModifierData = pxModifier->GetFloatData();

			pxDialog = new Asura_Numerical_Dialog("Font Scale", xOffset, &m_fModifierData, 0.0f, 9999.0f, 0.001f, "%3.3f", &m_bModifierChanged);
			pxDialog->SetHelpText("The font scale to use when the modifier is active");
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
		}
		break;

		case ASURA_GUIMENU_MODIFY_TEXT_ID:
		{
			m_uModifierData = pxModifier->GetUIntData();

			pxDialog = new Asura_Function_Dialog("Text",xOffset,ChooseModifierText,(int)this);
			pxDialog->SetHelpText("Select the text to use when the modifier is active.");
			pxDialog->m_iControlID = Asura_GetHashID("SelectModifierText");
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			
		}
		break;

		case ASURA_GUIMENU_MODIFY_FONT_NAME:
		{
			m_xModifierData_String = pxModifier->GetCharStringData();

			pxDialog = new Asura_Function_Dialog("Font",xOffset,ChooseModifierFont,(int)this);
			pxDialog->SetIsEnabledFunction(CanEditFontName,(int)this);
			pxDialog->SetHelpText("Select the font to use when the modifier is active");
			pxDialog->m_iControlID = Asura_GetHashID("SelectModifierFont");
			AddModifierDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
		}
		break;

		default :
			PARENT::AddModifierOverlayControls(pxModifier,xOffset);
			break;
	}
}

void Tools_GUIMenu_Element_Text::InitialiseModifier(Asura_GUIMenu_Modifier* pxModifier)
{
	switch(pxModifier->GetModifyType())
	{
		case ASURA_GUIMENU_MODIFY_TEXT_STRING:
			pxModifier->SetData(L"");
			break;

		case ASURA_GUIMENU_MODIFY_TEXT_SCROLL_RATE:
			pxModifier->SetData(m_pxAsuraText->GetScrollRate());
			break;

		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_X:
			pxModifier->SetData(m_pxAsuraText->GetOffsetX());
			break;

		case ASURA_GUIMENU_MODIFY_TEXT_OFFSET_Y:
			pxModifier->SetData(m_pxAsuraText->GetOffsetY());
			break;

		case ASURA_GUIMENU_MODIFY_TEXT_RENDER_FLAGS:
			pxModifier->SetData(m_pxAsuraText->GetTextRenderFlags());
			break;

		case ASURA_GUIMENU_MODIFY_FONT_SCALE:
			pxModifier->SetData(m_pxAsuraText->GetFontScale());
			break;

		case ASURA_GUIMENU_MODIFY_TEXT_ID:
			pxModifier->SetData((u_int)m_pxAsuraText->GetTextID());
			break;

		case ASURA_GUIMENU_MODIFY_FONT_NAME:
			pxModifier->SetData(m_pxAsuraText->GetFontName());
			break;

		default :
			PARENT::InitialiseModifier(pxModifier);
			break;
	}
}

// static function
bool Tools_GUIMenu_Element_Text::GetTextToPreserve(Asura_Hash_ID& uTextID, Asura_Hash_ID& uTextPage) const
{
	//if the text ID is editable by this item , and has been set , then return it
	if(ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT_ID))
	{
		const Asura_Hash_ID uOurTextID = m_pxAsuraText->GetTextID();
		const Asura_Hash_ID uOurTextPage = m_pxAsuraText->GetTextPageHash();
		if(uOurTextID && uOurTextPage)
		{
			uTextID = uOurTextID;
			uTextPage = uOurTextPage;
			return true;
		}
	}

	return PARENT::GetTextToPreserve(uTextID, uTextPage);
}

void Tools_GUIMenu_Element_Text::ApplyPreservedText(Asura_Hash_ID uTextID, Asura_Hash_ID uTextPage)
{
	//Apply the text , if it is editiable for this menu item
	if(IsStyleInstance() || IsGeneratedByStyle())
	{
		if(ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT_ID))
		{
			m_pxAsuraText->SetTextID(uTextID);
			m_pxAsuraText->SetTextPageHash(uTextPage);
		}
	}

	PARENT::ApplyPreservedText(uTextID, uTextPage);
}


#endif //ASURA_USE_GUI_MENU_SYSTEM
