//
// Tools_GUIMenu_ImportText.cpp
//
// Created by Tom Perry - 16/09/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "GUIMenu/Tools_GUIMenu_ImportText.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
// Independent
#include "Core/Asura_HashedLocalisedText.h"
#include "GUIMenu/Asura_GUIMenu_Element_Text.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Viewport.h"

// PC
#include "PC/FileSystem/Tools_FileSystem.h"

// Tools
#include "GUIMenu/Tools_Mode_GUIMenu.h"
#include "GUIMenu/Tools_GUIMenu_Element_Text.h"
#include "GUIMenu/Tools_GUIMenu_Widget_Group.h"
#include "Load&Save/Tools_AsuraFileContents.h"

////////////////////////////////////////////////////////////////////////////////////////////
// Tools_GUIMenu_ImportText
////////////////////////////////////////////////////////////////////////////////////////////

// Globals
static const char* g_szLineNone = "*None*";

// Static declarations
Asura_Vector_2							Tools_GUIMenu_ImportText::s_xPosChoosen;
Asura_Hash_ID							Tools_GUIMenu_ImportText::s_uTextPage;
Tools_GUIMenu_Root*						Tools_GUIMenu_ImportText::s_pxToolsParentItem;
// The two overlays
static	Tools_GUIMenu_ImportText_Overlay		g_xImportTextOverlay("Import Text");
static	Tools_GUIMenu_ImportAllText_Overlay		g_xImportAllTextOverlay("Import All Text");


// Class implementation
Tools_GUIMenu_ImportText::Tools_GUIMenu_ImportText(const Asura_String& xTitle)
: Tools_Modal_Overlay( xTitle )
{
}

Tools_GUIMenu_ImportText::~Tools_GUIMenu_ImportText()
{
	DeleteStyleList();
	delete [] m_aiStyleChoice;
}

void Tools_GUIMenu_ImportText::SelectFileToImportText(Tools_GUIMenu_Root* pxToolsParentItem, bool bImportAll, Asura_Vector_2 xPos)
{
	// Save the position and the parent
	s_xPosChoosen = xPos;
	s_pxToolsParentItem = pxToolsParentItem;
	if(!pxToolsParentItem) return;
	
	// Get the user to choose a file to grab the text from
	s_uTextPage = Tools_Mode_GUIMenu::BrowseForTextPage();

	if (!s_uTextPage) return;

	// Pop up the desired overlay
	if (bImportAll)
	{
		g_xImportAllTextOverlay.ShowOverlay();
	}
	else
	{
		g_xImportTextOverlay.ShowOverlay();
	}
}

Tools_GUIMenu_Root* Tools_GUIMenu_ImportText::InsertText(Asura_Vector_2 &xPos, int iCurLevel)
{
	// The rectangle will get overruled when the tool class is created so it doesn't matter the value of Rect at the moment
	Asura_Bounding_Rectangle xRect(0.0f,0.0f,20.0f,20.0f);
	// Create the text
	Asura_GUIMenu_Root* pxAsuraItem = Asura_GUIMenu_System::Create(Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT,xRect);
	if(!pxAsuraItem) return 0;

	Asura_GUIMenu_Root* pxAsuraParentItem = s_pxToolsParentItem->GetAsuraData();
	// Generate a name
	// First of we want to try using the Text ID as the name
	// If that fails we'll generate a unique one
	const Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::GetPage(s_uTextPage);
	if (!pxPage) return 0;

	const char* szID = (pxPage->GetTextIDString(m_uHashID));
	Asura_String xName(szID);
	if (Tools_Mode_GUIMenu::IsNameInUse(s_pxToolsParentItem,szID))
	{
		// The ID is already in use as a name, generate a unique one
		xName = Tools_Mode_GUIMenu::GenerateUniqueName(s_pxToolsParentItem,"Text");
	}

	pxAsuraItem->SetName(xName);
	
	//attempt to add it to the menu
	if(!pxAsuraParentItem->AddToThisContainer(pxAsuraItem))
	{
		delete pxAsuraItem;
		ShowMessageOverlay("Error adding text element");
		return 0;
	}
	pxAsuraItem->GenerateHashID();

	//create the tools class
	Tools_GUIMenu_Root* pxToolsItem = Tools_Mode_GUIMenu::AllocateToolsMenuItem(pxAsuraItem,s_pxToolsParentItem);
	// Temporally turn off clamping to parent so that we aren't restricted to the menu page when the object
	// is being created
	pxToolsItem->SetIsClampedToParent(false);
	pxToolsItem->InitialiseNewItem();
	pxToolsItem->SetIsClampedToParent(true);

	// now lets set the style, then we can set the other properties that aren't covered by it
	if (m_aiStyleChoice[iCurLevel] != 0)
	{
		// A style was choosen (s_iStyleChoice = 0 when *none* is choosen)
		// Lets apply the style
		pxToolsItem->ApplyStyle(m_ppxStyleItems[m_aiStyleChoice[iCurLevel]]->GetAsuraData()->GetID(),false);
	}

	// now we have the style, we want to change the text, size and position of the text element if that property
	// is editable under the current style
	// First, the text:
	if (pxToolsItem->ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Text::ASURA_GUIMENU_ELEMENT_TEXT_LOCAL_FLAG_TEXT_ID))
	{
		// Make sure it isn't the "none" option, as that isn't really an ID
		if (m_uHashID != Asura_GetHashID(g_szLineNone))
		{
			static_cast<Asura_GUIMenu_Element_Text*>(pxAsuraItem)->SetTextID(m_uHashID);
		}
		static_cast<Asura_GUIMenu_Element_Text*>(pxAsuraItem)->SetTextPageHash(s_uTextPage);
	}

	// Set the size if we can
	bool bCanResizeWidth = pxToolsItem->ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Base::ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_WIDTH);
	bool bCanResizeHeight = pxToolsItem->ShouldStyleUseLocalDataFor(Asura_GUIMenu_Element_Base::ASURA_GUIMENU_ELEMENT_BASE_LOCAL_FLAG_HEIGHT);;
	if ( bCanResizeWidth || bCanResizeHeight )
	{
		if (static_cast<Asura_GUIMenu_Element_Text*>(pxAsuraItem)->GetTextID() != 0)
		{
			// Load in the correct font for this text
			Asura_Font_System::SetFont(Asura_Font_System::GetFontHandle(static_cast<Asura_GUIMenu_Element_Text*>(pxAsuraItem)->GetFontNameHash()));
			// Get the Font scale
			float fFontScale = static_cast<Asura_GUIMenu_Element_Text*>(pxAsuraItem)->GetFontScale();
			// Get the text
			const Asura_UnicodeChar* wszText = static_cast<Asura_GUIMenu_Element_Text*>(pxAsuraItem)->GetTextUsingTextID();
			// Now we want resize to the text's width and height
			if (bCanResizeWidth)
			{
				xRect.MaxX = Asura_Font_System::GetWidthOfString(wszText,fFontScale);
			}
			if (bCanResizeHeight)
			{
				xRect.MaxY = Asura_Font_System::GetHeightOfText(wszText,NULL,1.0f,fFontScale,NULL);
			}
			// Set the bounds
			pxAsuraItem->SetUnscaledLocalBounds(xRect);
		}
	}

	//set the position if we are allowed to change that
	if (pxToolsItem->ShouldStyleUseLocalDataFor(Asura_GUIMenu_Root::ASURA_GUIMENU_ROOT_LOCAL_FLAG_POSITION))
	{
		pxAsuraItem->SetUnscaledLocalPos(xPos - s_pxToolsParentItem->GetAsuraData()->GetScaledWorldPos() - s_pxToolsParentItem->GetAsuraData()->GetChildPosOffset());
	}

	// Update the Y position
	xPos.y += xRect.GetHeight() + m_fYOffset;
	// and the X position
	//xPos.x += m_fXOffset;

	//flag as modified
	pxToolsItem->SetModified();

	return pxToolsItem;
}

Tools_GUIMenu_Root* Tools_GUIMenu_ImportText::InsertGroup(Asura_Vector_2 &xPos, int iCurLevel, bool bUseGroup /*true*/)
{
	// Lets get our own copy of the position, for use later when wraping the group around
	Asura_Vector_2 xOriPos = xPos;

	// Keep a list of everything in this group
	List<Tools_GUIMenu_Root*> xGroupList;

	// Keep a running count of min and max values we need to cover the objects in the group
	float fMaxX = 0.0f;
	float fMinX = Asura_Maths::LargeFloat;

	// This bool tracks whether or not we have found a text element of this level before
	// If we haven't then we want to insert the text. But if we have we want
	// the close the current group and start a new one.
	bool bFound = false;

	while (m_uAmountOfTextToInsert > m_uNumberOfTextsInserted)
	{
		// Lets get the heading level of text
		int iTextLevel = GetTextLevel(m_uHashID);
		if (iTextLevel > m_iLevelCap) { iTextLevel = m_iLevelCap; }

		if (iTextLevel == iCurLevel)
		{
			if (!bFound || iCurLevel == 0)
			{
				// The text is at the same level we are at. Create it and add it to the group list
				Tools_GUIMenu_Root* pxToolObject = InsertText(xPos,iTextLevel);
				m_uNumberOfTextsInserted++;
				if (pxToolObject)	
				{
					xGroupList.add_entry(pxToolObject);

					float fX = pxToolObject->GetAsuraData()->GetUnscaledLocalBounds().MaxX;
					if (fX > fMaxX) { fMaxX = fX; }
					fX = pxToolObject->GetAsuraData()->GetUnscaledLocalBounds().MinX;
					if (fX < fMinX) { fMinX = fX; }
				}

				bFound = true;

				if ( m_uHashID != m_xSelectionList.last_entry())
				{
					m_uHashID = m_xSelectionList.next_entry(m_uHashID);
				}
				else
				{
					break;
				}
			}
			else
			{
				// Groups over, exit the while loop
				break;
			}
		}
		else if (iTextLevel > iCurLevel)
		{
			// Groups over, exit the while loop
			break;
		}
		else
		{
			// The test is of a lower level, so create a group with all the text in it and add it to the group list.
			// Apply the XOffset
			xPos.x += m_fXOffset;
			Tools_GUIMenu_Root* pxToolObject = InsertGroup(xPos,iTextLevel);
			// And put it back
			xPos.x = xOriPos.x;
			if (pxToolObject)	
			{
				xGroupList.add_entry(pxToolObject);
				float fX = pxToolObject->GetAsuraData()->GetUnscaledLocalBounds().MaxX;
				if (fX > fMaxX) { fMaxX = fX; }
				fX = pxToolObject->GetAsuraData()->GetUnscaledLocalBounds().MinX;
				if (fX < fMinX) { fMinX = fX; }
			}
		}
	}

	// We now want to wrap the group around
	if (!bUseGroup) return 0;
	// We only want to do this if there is more then one item in the group, otherwise there is no point
	int iSize = xGroupList.size();
	if (iSize == 0)
	{
		Asura_Assert(false, "Nothing in the group, this shouldn't happen!");
		return 0;
	}
	if (iSize == 1)
	{
		// Only one object, so return that pointer
		return xGroupList.first_entry();
	}
	else
	{
		// Else we have a group of objects
		// The rectangle will get overruled when the tool class is created so it doesn't matter the value of Rect at the moment
		Asura_Bounding_Rectangle xRect(0.0f,0.0f,20.0f,20.0f);
		Asura_GUIMenu_Root* pxAsuraItem = Asura_GUIMenu_System::Create(Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_GROUP,xRect);
		if(!pxAsuraItem) return 0;

		Asura_GUIMenu_Root* pxAsuraParentItem = s_pxToolsParentItem->GetAsuraData();
		// Generate a name
		Asura_String xName = Tools_Mode_GUIMenu::GenerateUniqueName(s_pxToolsParentItem,"Group");

		pxAsuraItem->SetName(xName);
		
		//attempt to add it to the menu
		if(!pxAsuraParentItem->AddToThisContainer(pxAsuraItem))
		{
			delete pxAsuraItem;
			ShowMessageOverlay("Error adding group widget");
			return 0;
		}
		pxAsuraItem->GenerateHashID();

		//create the tools class
		Tools_GUIMenu_Root* pxToolsItem = Tools_Mode_GUIMenu::AllocateToolsMenuItem(pxAsuraItem,s_pxToolsParentItem);
		pxToolsItem->SetIsClampedToParent(false);
		pxToolsItem->InitialiseNewItem();

		Asura_Vector_2 xStartPos(fMinX,xOriPos.y);
		pxAsuraItem->SetUnscaledLocalPos(xStartPos);
		
		// Now we need to add the items to the group
		Tools_GUIMenu_Root* pxAddedItem = xGroupList.first_entry();

		while (true)
		{
			// If it's not already (the groups wont be) we want this object to be bound to the parent
			pxAddedItem->SetIsClampedToParent(true);
			pxAddedItem->ChangeParent(pxToolsItem);
			// If this element is a group we may want to link it
			if (pxAddedItem->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_GROUP)
			{
				LinkGroups(pxAddedItem);
			}
			if (pxAddedItem != xGroupList.last_entry())
			{
				pxAddedItem = xGroupList.next_entry(pxAddedItem);
			}
			else
			{
				break;
			}
		}

		// Now we want to link the groups in this group together
		LinkGroups(pxToolsItem);

		// Fix random bug where the first text element's position is thrown
		pxAddedItem = xGroupList.first_entry();
		pxAddedItem->GetAsuraData()->SetUnscaledLocalPos(xOriPos - pxToolsItem->GetAsuraData()->GetScaledWorldPos() - pxToolsItem->GetAsuraData()->GetChildPosOffset());

		// Correct the groups bounding box
		Asura_Bounding_Rectangle xBoundRect = pxAsuraItem->GetUnscaledLocalBounds();
		xBoundRect.MaxX = fMaxX+10;
		xBoundRect.MinX = fMinX;
		xBoundRect.MaxY = xPos.y;
		pxAsuraItem->SetUnscaledLocalBounds(xBoundRect);

		//flag as modified
		pxToolsItem->SetModified();
		
		// return the group
		return pxToolsItem;
	}
}

void Tools_GUIMenu_ImportText::LinkGroups(Tools_GUIMenu_Root* pxGroup)
{
	const List<Tools_GUIMenu_Root*>* xList = pxGroup->GetChildList();

	Tools_GUIMenu_Widget_Group* pxLinkedGroup = NULL;

	Tools_GUIMenu_Root* pxItem = xList->first_entry();

	while (true)
	{
		if (pxItem->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_GROUP)
		{
			LinkGroups(pxItem);
			if (pxLinkedGroup)
			{
				static_cast<Tools_GUIMenu_Widget_Group*>(pxItem)->LinkGroup(pxLinkedGroup);
			}
			else
			{
				pxLinkedGroup = NULL;
			}
			pxLinkedGroup = static_cast<Tools_GUIMenu_Widget_Group*>(pxItem);
		}
		if (pxItem != xList->last_entry())
		{
			pxItem = xList->next_entry(pxItem);
		}
		else
		{
			return;
		}
	}
}

//			= Normal Text (0)
// +		= Bottom Heading (1)
// ++		= Medium Heading (2)
// +++		= Top Heading (3)
// etc etc
int Tools_GUIMenu_ImportText::GetTextLevel(Asura_Hash_ID uHashID)
{
	// We want to see how many '+'s are at the beginning of the ID
	const Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::GetPage(s_uTextPage);
	if (!pxPage) return 0;

	// Make sure it isn't the "none" option, as that isn't really an ID
	if (uHashID == Asura_GetHashID(g_szLineNone)) return 0;
	
	const char* szString = pxPage->GetTextIDString(uHashID);

	if (!szString) return 0;

	int iTypeCount = 0;
	while( szString[ 0 ] == '+' )
	{
		iTypeCount++;
		szString++;
	}
	return iTypeCount;
}

void Tools_GUIMenu_ImportText::UpdateStyleList()
{
	// Delete the old list
	DeleteStyleList();

	//determine type to filter by
	int iType = Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT;

	//Determine the number of matching styles
	// Starts at 1 for the 'no style' option
	m_iNumStyles = 1;
	for(CLIF<Tools_GUIMenu_Root*> xLIF(Tools_GUIMenu_Root::GetList());!xLIF.done();xLIF.next())
	{
		if(!xLIF()->IsStyle()) continue;
		if(iType && xLIF()->GetType() != iType) continue;

		m_iNumStyles++;
	}

	if(m_iNumStyles)
	{
		m_ppxStyleItems = new Tools_GUIMenu_Root*[m_iNumStyles];
		//find the items again to fill in the style list

		int iIndex = 1;
		for(CLIF<Tools_GUIMenu_Root*> xLIF(Tools_GUIMenu_Root::GetList());!xLIF.done();xLIF.next())
		{
			if(!xLIF()->IsStyle()) continue;
			if(iType && xLIF()->GetType() != iType) continue;

			m_ppxStyleItems[iIndex++] = xLIF();
		}
		Asura_Assert(iIndex == m_iNumStyles,"");

		//sort the items alphabetically
		qsort(m_ppxStyleItems+1,m_iNumStyles-1,sizeof(Tools_GUIMenu_Root*),SortStyleCallback);

		//and get a list of names for the overlay
		m_pszStyleNames = new char*[m_iNumStyles];
		m_pszStyleNames[0] = "*No Style*";
		for(int i=1;i<m_iNumStyles;i++)
		{
			const char* szName = m_ppxStyleItems[i]->GetName();
			m_pszStyleNames[i] = new char[strlen(szName)+1];
			strcpy(m_pszStyleNames[i],szName);
		}	
	}
}

int Tools_GUIMenu_ImportText::SortStyleCallback(const void* pItem1,const void* pItem2)
{
	const Tools_GUIMenu_Root* pxItem1 = *(const Tools_GUIMenu_Root**)pItem1;
	const Tools_GUIMenu_Root* pxItem2 = *(const Tools_GUIMenu_Root**)pItem2;

	return stricmp(pxItem1->GetName(),pxItem2->GetName());
}	

void Tools_GUIMenu_ImportText::DeleteStyleList()
{
	for(int i=1;i<m_iNumStyles;i++)
	{
		delete [] m_pszStyleNames[i];
	}
	delete [] m_pszStyleNames;
	m_pszStyleNames = 0;

	delete [] m_ppxStyleItems;
	m_ppxStyleItems = 0;

	m_iNumStyles = 0;
}

////////////////////////////////////////////////////////////////////////////////////////////
//Tools_GUIMenu_ImportText_Overlay
////////////////////////////////////////////////////////////////////////////////////////////

//search types
enum
{
	TEXT_SEARCH_BEGINS,
	TEXT_SEARCH_CONTAINS,
};

static const char* g_aszSearchOptions[] =
{
	"Begins With",
	"Contains",
};

Tools_GUIMenu_ImportText_Overlay::Tools_GUIMenu_ImportText_Overlay( const Asura_String& xTitle )
: Tools_GUIMenu_ImportText(xTitle)
{
	m_iLevelCap = 0;
	m_aiStyleChoice = new int[m_iLevelCap+1];
	m_aiStyleChoice[0] = 0;
	m_bFilterChanged = false;
	m_bLineChanged = false;
	m_bSortList = false;
	m_bPrevSortList = true;
	m_uLineListSize = 0;
	m_apszLineList = NULL;
	m_pxLineStringTree = new Asura_StringTree;
	m_iSearchOption = 0;
	m_bGroupTextTogether = false;
	m_xSearchText = NULL;
}

Tools_GUIMenu_ImportText_Overlay::~Tools_GUIMenu_ImportText_Overlay()
{
	DeleteLineList();
	delete m_pxLineStringTree;
}

void Tools_GUIMenu_ImportText_Overlay::InitialiseOverlay(Asura_Overlay* pxOverlay )
{
	m_pxLineListDialog = 0;

	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	// Delete the old line list
	DeleteLineList();

	//build temporary list , which will be corrected on the first frame
	m_apszLineList = new const char*[1];
	m_apszLineList[0] = g_szLineNone;
	m_uLineListSize = 1;
	m_bFilterChanged = true;

	//determine the maximum number of lines to use for the list
	float fMaxPrintY = Asura_Viewport::GetHeight() / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const int iMaxLines = Asura_Maths::FloatToInt_Chop((fMaxPrintY - 41.0f)/1.2f);

	//add drop down lists for filters
	Asura_DropDown_List_Dialog* pxDropListDialog;

	//Search/Search Type
	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset,g_aszSearchOptions,&m_iSearchOption,iMaxLines,2,NULL,&m_bFilterChanged,"");
	pxDropListDialog->SetHelpText("Determines how the search text should be used for filtering the list.");
	pxOverlay->AddDialog(pxDropListDialog);


	xOffset.x = pxDropListDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_String_Dialog("",xOffset,&m_xSearchText,30,&m_bFilterChanged);
	((Asura_String_Dialog*)pxDialog)->SetUpdateImmediatelyWhileTyping(true);
	pxDialog->SetHelpText("Filters the line list so that it only shows those IDs and text that begin with/contain this string");;
	xOffset.x = 1.0f;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	//list of lines
	m_bLineChanged = false;
	pxDialog = m_pxLineListDialog = new Asura_List_Dialog(xOffset,m_apszLineList,&m_iLineIndex,iMaxLines,m_uLineListSize,NULL,&m_bLineChanged);
	m_pxLineListDialog->EnableMultipleSelection(true);
	m_pxLineListDialog->SetTooltipCallback(ShowTooltipForLine);
	pxOverlay->	AddDialog(pxDialog);
	xOffset.y += iMaxLines*1.2f + 1.0f;

	// Do we want the list to be sorted?
	pxDialog = new Asura_Tickbox_Dialog("Sort List Alphabetically",xOffset,&m_bSortList);
	pxDialog->SetHelpText("Tick this if you want the list of IDs to be sorted alphabetically");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Style
	UpdateStyleList();
	pxDialog = new Asura_StringDisplay_Dialog("Style", xOffset);
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = 11.0f;
	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset, m_pszStyleNames,&m_aiStyleChoice[0],iMaxLines,m_iNumStyles,NULL);
	pxDropListDialog->SetHelpText("Choose a style for the imported text");
	pxOverlay->AddDialog(pxDropListDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Group
	xOffset.x = 1.0f;
	pxDialog = new Asura_Tickbox_Dialog("Group",xOffset,&m_bGroupTextTogether);
	pxDialog->SetHelpText("Tick this if you want all the text objects to be grouped together on creation");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::SmallSpacing();

	//Offset controls
	pxDialog = new Asura_Numerical_Dialog("X Offset",xOffset,&m_fXOffset,-9999.0f,9999.0f,0.2f,"%.1f");
	pxDialog->SetHelpText("The X Offset between each text item");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Y Offset",xOffset,&m_fYOffset,0.0f,9999.0f,0.2f,"%.1f");
	pxDialog->SetHelpText("The X Offset between each text item");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

    pxDialog = new Asura_Function_Dialog( "Ok", xOffset, OkaySelected,(int)this, &pxOverlay->m_bShouldClose );
    pxOverlay->AddDialog( pxDialog );
    xOffset.x += 10.0f;

    pxDialog = new Asura_Function_Dialog( "Cancel", xOffset, CancelSelected, &pxOverlay->m_bShouldClose);
    pxOverlay->AddDialog( pxDialog );

	BuildLineList();
}

void Tools_GUIMenu_ImportText_Overlay::ShowTooltipForLine(int iIndex, const char* szLineName)
{
	if(!szLineName || !*szLineName) return;
	if(!strcmp(szLineName,g_szLineNone)) return;

	const Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::GetPage(s_uTextPage);
	if (!pxPage) return;

	Asura_Hash_ID uLineHash = Asura_GetHashID(szLineName);
	const Asura_UnicodeChar* wszText = pxPage->GetString(uLineHash);
	if(!wszText || !*wszText) return;

	char szHelpText[1024];
	Asura_PrintfToBfr(szHelpText,"%S",sizeof(szHelpText),true,wszText);

	Asura_Overlay_System::SetHelpText(szHelpText);
}

void Tools_GUIMenu_ImportText_Overlay::Update()
{
	if(m_bFilterChanged)
	{
		//rebuild the line list
		BuildLineList();
		m_bFilterChanged = false;
	}
	if(m_bSortList == m_bPrevSortList)
	{
		// Rebuild the line list
		BuildLineList();
		m_bPrevSortList = !m_bSortList;
	}

	if(m_bLineChanged)
	{
		//get the hash of the new sample
		m_uMostRecentSelectedLine = m_uSelectedLine = GetLineHash(m_iLineIndex);
		m_bLineChanged = false;

		// get hashes of all in list
		while(m_xSelectionList.size()) m_xSelectionList.delete_first_entry();

		int iNumSelected = m_pxLineListDialog->GetNumSelected();
		for(int i=0;i<iNumSelected;i++)
		{
			int iIndex = m_pxLineListDialog->GetSelection(i);
			Asura_Hash_ID uHash = GetLineHash(iIndex);
			if(uHash) m_xSelectionList.add_entry(uHash);
		}
	}
}

void Tools_GUIMenu_ImportText_Overlay::BuildLineList()
{
	const Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::GetPage(s_uTextPage);
	if(!pxPage || !pxPage->HasTextIDStrings())
	{
		ShowMessageOverlay("No menu strings available");
		return;
	}

	// Delete the old line list
	DeleteLineList();

	char szWildcardSearchString[100];
	if(!m_xSearchText.IsBlank() && m_iSearchOption == TEXT_SEARCH_CONTAINS)
	{
		//prepare search string for wildcard search
		Asura_PrintfToBfr(szWildcardSearchString,"*%s*",sizeof(szWildcardSearchString),true,m_xSearchText.GetString());
	}

	//get a list of all samples that meet the current filter requirements
	int iJunkIndex;
	for(int i=0;i<pxPage->GetNumberOfStrings();i++)
	{
		if(!m_xSearchText.IsBlank())
		{
			if(!DoesLineSatisfySearchFilter(i,szWildcardSearchString))
			{
				continue;
			}
		}
		const char* wszString = pxPage->GetTextIDStringByIndex(i);

		Asura_Assert(wszString,"");

		m_pxLineStringTree->FindNodeIndexByName(wszString,true,iJunkIndex);
	}

	//get array of sample names , and fill in the special samples that aren't in the tree (i.e. the *None* option)
	m_uLineListSize = m_pxLineStringTree->GetNumberOfNodes() + 1;
	m_apszLineList = new const char*[m_uLineListSize];
	m_apszLineList[0] = g_szLineNone;
	// We will make the name list, either sorted or unsorted depending on what the user selected
	if (m_bSortList)
	{
		m_pxLineStringTree->MakeSortedNameList(m_apszLineList+1);
	}
	else
	{
		m_pxLineStringTree->MakeNameList(m_apszLineList+1);
	}

	//Update the current selected index to be as close as possible to the previous selection
	if(m_uMostRecentSelectedLine == 0)
	{
		m_iLineIndex = 0;
	}
	else
	{
		const char* pxLine = pxPage->GetTextIDString(m_uMostRecentSelectedLine);
		m_iLineIndex = 0;
		if(pxLine)
		{
			for(u_int i=0;i<m_uLineListSize;i++)
			{
				if(stricmp(pxLine,m_apszLineList[i])>=0)
				{
					m_iLineIndex = i;
				}
				else
				{
					break;
				}
			}
		}
	}

	//get the hash ID of the line selected
	m_uSelectedLine = GetLineHash(m_iLineIndex);

	//multiple selection list will be junk after list change
	while(m_xSelectionList.size()) m_xSelectionList.delete_first_entry();

	//update the list dialog if it has been created yet
	if(m_pxLineListDialog)
	{
		m_pxLineListDialog->ReplaceStrings(m_uLineListSize,m_apszLineList);

		if(m_uLineListSize > 0)
		{
			//select the item in the list
			m_pxLineListDialog->SetSelection(m_iLineIndex);

			// force the list to be rebuilt on the first update
			m_bLineChanged = true;
		}
	}
}

bool Tools_GUIMenu_ImportText_Overlay::DoesLineSatisfySearchFilter(int iIndex,const char* szWildcardSearchString)
{
	// Now lets search the actual text itself
	const Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::GetPage(s_uTextPage);
	if (!pxPage) return false;

	const char* szText = pxPage->GetTextIDStringByIndex(iIndex);

	//check against the name first
	if(m_iSearchOption == TEXT_SEARCH_BEGINS)
	{
		if(!strnicmp(m_xSearchText.GetString(),szText,m_xSearchText.GetLength()))
		{
			return true;
		}
	}
	else
	{
		if(Asura_StringUtil::CompareAgainstWildcardString(szText,szWildcardSearchString,true))
		{
			return true;
		}
	}

	char szCharText[128];
	Asura_PrintfToBfr(szCharText,"%S",sizeof(szCharText),true,pxPage->GetStringByIndex(iIndex));

	//check against the name first
	if(m_iSearchOption == TEXT_SEARCH_BEGINS)
	{
		if(!strnicmp(m_xSearchText.GetString(),szCharText,m_xSearchText.GetLength()))
		{
			return true;
		}
	}
	else
	{
		if(Asura_StringUtil::CompareAgainstWildcardString(szCharText,szWildcardSearchString,true))
		{
			return true;
		}
	}

	return false;
}

bool Tools_GUIMenu_ImportText_Overlay::WantsGroup()
{
	return m_bGroupTextTogether;
}

void Tools_GUIMenu_ImportText_Overlay::DeleteLineList()
{
	delete [] m_apszLineList;
	m_apszLineList = 0;
	m_uLineListSize = 0;
	m_pxLineStringTree->Reset();
}

Asura_Hash_ID Tools_GUIMenu_ImportText_Overlay::GetLineHash(int iIndex) const
{
	/*if(!stricmp(m_apszLineList[iIndex],g_szLineNone))
	{
		return 0;
	}*/
	return Asura_GetHashID(m_apszLineList[iIndex]);
}

void Tools_GUIMenu_ImportText_Overlay::OkaySelected()
{
	m_uAmountOfTextToInsert = m_xSelectionList.size();
	if (m_uAmountOfTextToInsert == 0) return;

	m_uNumberOfTextsInserted = 0;

	m_uHashID = m_xSelectionList.first_entry();
	InsertGroup(s_xPosChoosen,0,m_bGroupTextTogether);
	// remove all the stuff from the selection
	while(m_xSelectionList.size()) m_xSelectionList.delete_first_entry();
}

////////////////////////////////////////////////////////////////////////////////////////////
//Tools_GUIMenu_ImportAllText_Overlay
////////////////////////////////////////////////////////////////////////////////////////////

Tools_GUIMenu_ImportAllText_Overlay::Tools_GUIMenu_ImportAllText_Overlay( const Asura_String& xTitle )
:Tools_GUIMenu_ImportText(xTitle)
{
	m_iLevelCap = 3;
	m_aiStyleChoice = new int[m_iLevelCap+1];
	for (int i = 0; i <= m_iLevelCap; i++)
	{
		m_aiStyleChoice[i] = 0;
	}
}

Tools_GUIMenu_ImportAllText_Overlay::~Tools_GUIMenu_ImportAllText_Overlay()
{
}

void Tools_GUIMenu_ImportAllText_Overlay::InitialiseOverlay(Asura_Overlay* pxOverlay )
{
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	//determine the maximum number of lines to use for the list
	float fMaxPrintY = Asura_Viewport::GetHeight() / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const int iMaxLines = Asura_Maths::FloatToInt_Chop((fMaxPrintY - 41.0f)/1.2f);

	//add drop down lists for styles
	Asura_DropDown_List_Dialog* pxDropListDialog;

	// Styles
	UpdateStyleList();
	
	pxDialog = new Asura_StringDisplay_Dialog("Top Heading \n Style", xOffset);
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = 11.0f;
	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset, m_pszStyleNames,&m_aiStyleChoice[3],iMaxLines,m_iNumStyles,NULL);
	pxDropListDialog->SetHelpText("Choose a style for the top level headings (ID's start with +++)");
	pxOverlay->AddDialog(pxDropListDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.y += Asura_Dialog::LargeSpacing();

	xOffset.x = 1.0f;
	pxDialog = new Asura_StringDisplay_Dialog("Med. Heading \n Style", xOffset);
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = 11.0f;
	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset, m_pszStyleNames,&m_aiStyleChoice[2],iMaxLines,m_iNumStyles,NULL);
	pxDropListDialog->SetHelpText("Choose a style for the medium level headings (ID's start with ++)");
	pxOverlay->AddDialog(pxDropListDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.y += Asura_Dialog::LargeSpacing();

	xOffset.x = 1.0f;
	pxDialog = new Asura_StringDisplay_Dialog("Low Heading \n Style", xOffset);
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = 11.0f;
	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset, m_pszStyleNames,&m_aiStyleChoice[1],iMaxLines,m_iNumStyles,NULL);
	pxDropListDialog->SetHelpText("Choose a style for the low level headings (ID's start with +)");
	pxOverlay->AddDialog(pxDropListDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.y += Asura_Dialog::LargeSpacing();

	xOffset.x = 1.0f;
	pxDialog = new Asura_StringDisplay_Dialog("Default Style", xOffset);
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = 11.0f;
	pxDropListDialog = new Asura_DropDown_List_Dialog(xOffset, m_pszStyleNames,&m_aiStyleChoice[0],iMaxLines,m_iNumStyles,NULL);
	pxDropListDialog->SetHelpText("Choose a style for the default text (ID's dont start with any +'s)");
	pxOverlay->AddDialog(pxDropListDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.y += Asura_Dialog::LargeSpacing();

	//Offset controls
	xOffset.x = 1.0f;
	pxDialog = new Asura_Numerical_Dialog("X Offset",xOffset,&m_fXOffset,-9999.0f,9999.0f,0.2f,"%.1f");
	pxDialog->SetHelpText("The X Offset between each text item");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Y Offset",xOffset,&m_fYOffset,0.0f,9999.0f,0.2f,"%.1f");
	pxDialog->SetHelpText("The X Offset between each text item");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

    pxDialog = new Asura_Function_Dialog( "Ok", xOffset, OkaySelected,(int)this, &pxOverlay->m_bShouldClose );
    pxOverlay->AddDialog( pxDialog );
    xOffset.x += 10.0f;

    pxDialog = new Asura_Function_Dialog( "Cancel", xOffset, CancelSelected, &pxOverlay->m_bShouldClose);
    pxOverlay->AddDialog( pxDialog );
}


void Tools_GUIMenu_ImportAllText_Overlay::OkaySelected()
{
	// We want to get all the text from the page
	const Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::GetPage(s_uTextPage);
	if(!pxPage || !pxPage->HasTextIDStrings())
	{
		ShowMessageOverlay("No menu strings available");
		return;
	}

	//go through all the lines and add them to the selection list
	for(int i=0;i<pxPage->GetNumberOfStrings();i++)
	{
		Asura_Hash_ID xHash = Asura_GetHashID(pxPage->GetTextIDStringByIndex(i));
		m_xSelectionList.add_entry(xHash);
	}

	m_uAmountOfTextToInsert = m_xSelectionList.size();
	if (m_uAmountOfTextToInsert == 0) return;

	m_uNumberOfTextsInserted = 0;

	m_uHashID = m_xSelectionList.first_entry();

	InsertGroup(s_xPosChoosen,m_iLevelCap+1);

	// remove all the stuff from the selection
	while(m_xSelectionList.size()) m_xSelectionList.delete_first_entry();
}
#endif //ASURA_USE_GUI_MENU_SYSTEM
