//
// Tools_GUIMenu_Root.h
//
// Created by Richard Rice - 30/11/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif


#ifndef _TOOLS_GUIMENU_ROOT_H_
#define _TOOLS_GUIMENU_ROOT_H_
#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "Tools_Mode_GUIMenu.h"
#include "GUIMenu/Asura_GUIMenu_Element_Base.h"
#include "Rendering/Asura_Colour.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/list_tem.hpp"

class Asura_Overlay;
class Asura_Dialog;
class Asura_Tabs_Dialog;
class Asura_Subwindow_Dialog;
class Asura_GUIMenu_Root;
class Asura_GUIMenu_Modifier;
class Asura_GUIMenu_Animator_Sequence;
class Tools_GUIMenu_Widget_Base;
class Tools_GUIMenu_PickItem;

//class for dealing with the tools side stuff for editing a Asura_GUIMenu_Root
class Tools_GUIMenu_Root
{
	friend class Tools_GUIMenu_PickItem_AddToGroup;

public :
	Tools_GUIMenu_Root(Asura_GUIMenu_Root* pxAsuraRoot,Tools_GUIMenu_Root* pxToolsParent,const char* szName = 0);
	virtual ~Tools_GUIMenu_Root();

	//Sets some defaults for a newly created menu item
	virtual void InitialiseNewItem();
	//Gets the default initial size for a new item
	virtual void GetInitialSize(float& fWidth,float& fHeight){fWidth = 20.0f;fHeight = 20.0f;}

	//deletes both the asura and tools data.
	virtual void Destroy();

	//Gets confirmation as to whether a menu item should be deleted.
	virtual bool ConfirmDeletion();

	//Checks this item and any children for one under the specified position
	//If pxPickItem is passed ,then it used for filtering the return value
	//If bPreferWidget is true then it will favour selecting widgets over elements
	Tools_GUIMenu_Root* GetItemFromPosition(const Asura_Vector_2& xPos,Tools_GUIMenu_PickItem* pxPickItem = 0,bool bPreferWidget = true);

	//per frame update
	virtual void Update();
	void UpdateRecursive();

	//Currently renders elements.. not sure I want to do it this way however.
	virtual void Render(){};
	void RenderRecursive();

	//render extra UI for the selected item
	virtual void RenderUI(){};


	//draws bounding rectangle
	void RenderOutline(u_int uColour) const;
	void RenderOutlineRecursive(u_int uColour) const;

	//draws sizing handles around bounding rect
	void RenderEditHandles() const;

	//get the bounding rectangle in screen coords
	Asura_Bounding_Rectangle GetScreenRectangle() const;

	virtual bool CanBeRenamed() const {return true;}
	virtual bool CanBeResized() const {return false;}
	virtual bool CanEditWidth() const {return CanBeResized();}
	virtual bool CanEditHeight() const {return CanBeResized();}
	virtual bool CanBeMoved() const {return false;}

	virtual bool CanEditDisplayWidth() const	{	return CanEditWidth();	}
	virtual bool CanEditDisplayHeight() const	{	return CanEditHeight();	}

	//determines what type of drag to do from the menu coord mouse position
	u_int GetExtentsToDrag(const Asura_Vector_2& xPos) const;
	//takes backup of rectangle before doing a drag operation
	void StorePreDragState();
	void GetPreDragState(Asura_Bounding_Rectangle& xRect);
	//per frame update of drag
	void UpdateDrag(const Asura_Vector_2& xOffset,u_int uDragFlags);

	//slightly convoluted name to distinguish from the virtual SetRectangle function
	//Updates the rectangle , and moves/resizes the children according to the flags
	void MoveOrResizeRect(const Asura_Bounding_Rectangle& xOldRect, const Asura_Bounding_Rectangle& xNewRect, u_int uDragFlags, bool bClampToParent, bool bUsePreDragRect);

	void ResizePostChildChanges();

	// are we currently renaming?
	static bool IsRenaming() { return s_bIsRenaming; }

	//gets list of all members of this class
	static const List<Tools_GUIMenu_Root*>* GetList(){return &s_xList;}

	//gets a list of the children
	const List<Tools_GUIMenu_Root*>* GetChildList() const {return &m_xChildren;}

	//functions just here so that I can get menu page transferral to work
	void AddChild(Tools_GUIMenu_Root* pxItem);
	void RemoveChild(Tools_GUIMenu_Root* pxItem);

	//gets the tools parent of this item
	Tools_GUIMenu_Root* GetParent() {return m_pxToolsParent;}
	const Tools_GUIMenu_Root* GetParent() const {return m_pxToolsParent;}

	// Returns true if this element is clamped to it's parent
	bool ClampToParent() const;

	void SetIsClampedToParent(bool bClamp);

	//Returns true for the root of the hierarchy (which has the files as children)
	bool IsRoot() const {return m_pxToolsParent == 0;}

	//returns true if this doesn't correspond to a menu page/widget/element
	bool IsFakeMenuItem() const {return m_pxAsuraRoot==0;}

	//returns true if this is the specified item , or if it a child , grandchild etc
	bool IsDescendentOf(const Tools_GUIMenu_Root* pxItem) const
	{
		if(this == pxItem) return true;
		return m_pxToolsParent ? m_pxToolsParent->IsDescendentOf(pxItem) : false;
	}

	const char* GetName() const {return m_xName.GetString();}

	//Returns the name of the menu item type (Checkbox etc)
	//This currently returns 0 for fake menu items.
	virtual const char* GetTypeName() const;

	//Gets a name to use for the treeview
	void GetNameForTree(char* szOutput,int iBufferLength) const;

	virtual void OnSelection();
	virtual void OnDeselection();

	virtual void NotifyParentOnDeselection( void);

	Asura_GUIMenu_Root* GetAsuraData() {return m_pxAsuraRoot;}
	const Asura_GUIMenu_Root* GetAsuraData() const {return m_pxAsuraRoot;}

	//recalculates extents if they depend on the children extents (for widgets).
	virtual void CalculateExtents();

	//returns true if currently using the key/button for own UI purposes (to prevent mouse being used for selection etc)
	virtual bool IsUsingKey(int iKey){return false;}

	bool IsSelected() const {return m_bSelected;}

	//Gets the top most item that is still a proper menu item , which ought to be the menu page
	Tools_GUIMenu_Root* GetMenuPage() {return m_pxToolsParent->IsFakeMenuItem() ? this : m_pxToolsParent->GetMenuPage();}

	//Finds a widget with the given ID
	Tools_GUIMenu_Widget_Base* FindWidget(Asura_Hash_ID uID);

	//returns true if this is a widget
	virtual bool IsWidget() const {return false;}
	virtual bool IsElement() const {return false;}
	virtual bool IsMenu() const {return false;}
	virtual bool IsFile() const {return false;}

	//returns true if this can take the menu item type as a child
	virtual bool IsValidChildType(int iItemType) const {return false;}

	//gets the item type from the asura menu item
	int GetType() const;

	void Copy(const Tools_GUIMenu_Root* pxSource,u_int uCopyFlags);

	virtual void Copy(const Tools_GUIMenu_Root* pxSource);

	//Deals with the special case of copying data for listbox entries where the population functions are stored
	//in the listbox , rather than in the element that they are associated with
	virtual void CopyDataForThisStoredInParent(const Tools_GUIMenu_Root* pxSource, Tools_GUIMenu_Root* pxDest) {};

	//Clamps to parent rectangle , preserving dimensions if possible.
	void ClampToParentRect();

	//flags the file containing this menu as modified
	virtual void SetModified();

	//updates referencse in response to an ID changing
	void UpdateReferencesRecursive(Asura_Hash_ID uOldID,Asura_Hash_ID uNewID,bool bPostCopy);

	//updates display conditions in response to an ID changing
	void UpdateWidgetReferencesRecursive( Asura_Hash_ID uOldID, Asura_Hash_ID uNewID, const char* const szNewID );
	void UpdateWidgetReferences( Asura_Hash_ID uOldID, Asura_Hash_ID uNewID, const char* const szNewID );


	//Does a recursive search and tries to fix up any problems with menu items not
	//having appropriate references to required children.
	//(Eg text boxes need to know which text element to use)
	void CheckRequiredReferencesRecursive();
	virtual void CheckRequiredReferences(){};

	//Searches for the specified item in the hash tree
	Tools_GUIMenu_Root* FindByID(Asura_Hash_ID uID);

	//Resets asura data after previewing menus (such as the position of path elements)
	void PostMenuPreviewCleanupRecursive();

	//Returns true if this is a menu item that is flagged as being a style
	bool IsStyle() const;
	bool IsStyleContainer() const;

	void SetGeneratedByStyle(bool bSet);
	bool IsGeneratedByStyle() const;

	void SetStyleInstance(bool bSet);
	bool IsStyleInstance() const;

	//returns true if this is a member of a master copy of a style
	bool IsStyleMember() const;


	//Recursively checks items updating them from their styles.
	//Limited to uStyleID if it is set to a non-zero value
	void UpdateStyleInstances(Asura_Hash_ID uStyleID = 0);

	//Marks this item as inheriting from the style item
	void SetStyleItem(const Tools_GUIMenu_Root* pxStyleItem);

	//Attempts to apply the specified to this item
	void ApplyStyle(Asura_Hash_ID uStyleID,bool bKeepChildren);

	//calls the equivalent function in the asura data
	Asura_Hash_ID GetStyleObjectHash() const;

	//Unflags this as being from a style
	void DivorceFromStyle(bool bFirst);

	//returns true if this item isn't inherited , or if the flag is set for using the local data
//	bool ShouldUseLocalDataFor(u_int uFlag) const;
	bool ShouldStyleUseLocalDataFor(u_int uFlag) const;

	//Returns true if this (or any children) is a master copy of a style.
	bool ContainsStyles() const;

	//Used to register all the console commands/variables used with the preview overlay
	void AddToPreviewOverlayRecursive() const;
	virtual void AddToPreviewOverlay() const;
	void AddToPreviewOverlay( Asura_GUIMenu_ConditionVar* const pxBegin,Asura_GUIMenu_ConditionVar* const pxEnd ) const;

	//Sorts the children in this and the tree view if this is the root menu item.
	void SortFiles();

	//checks for errors , adding them to the message log
	bool VerifyRecursive();
	virtual bool Verify();


	//Gets the hash ID of the asura menu item. Returns 0 for the fake items
	Asura_Hash_ID GetID() const;

	//Adds the named animator sequence
	void AddAnimator(const char* szSequenceName);
	//Checks to see if the animator sequence is allowed for this menu item.
	bool IsAnimatorAllowed(const Asura_GUIMenu_Animator_Sequence* pxSeq) const;

	//Updates menu items in response to changes to the animators
	void HandleRenameAnimator(const char* szOldName,const char* szNewName);
	void HandleDeleteAnimator(const char* szOldName);

	// adds any extra overlay controls for when this is a container
	virtual void AddParentOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, Tools_GUIMenu_Root* pxChild ) {};

	virtual void SetRectangle(const Asura_Bounding_Rectangle& xRect);

	//gets list of possible menu statse
	static u_int GetNumMenuItemStates();
	static const char* GetMenuItemStateName(u_int uIndex);
	static u_int GetMenuItemStateValue(u_int uIndex);

	//Moves item to give it a different parent
	virtual void ChangeParent(Tools_GUIMenu_Root* pxParent);

	// Calls RecalcualteID(), setting isRenaming to true(false) before(after) call
	void RecalculateIDAfterRename();

	//Sets the rectangles for everthing again , forcing the local positions to be recalculated
	void SetRectangleRecursive();

	//Hides/unhides the tools menu item
	void SetHidden(bool bHide) {m_bHidden = bHide;}
	void SetHiddenRecursive(bool bHide);

	//Checks for the tools item being hidden for any reason
	bool IsHidden() const;

	const bool FindAndReplaceConsoleVarName( const Asura_UnicodeChar* wszName,const Asura_UnicodeChar* wszReplacementName );

	float	GetPixelX( void)	{	return m_fX;	}
	float	GetPixelY( void)	{	return m_fY;	}

	virtual bool NeedsPosOptions() const {return false;}

	// sets anchoring to top-left corner - NDB 260308
	virtual void ResetAnchoring();

	void SetScalingFlags( const u_int uFlags );
	void SetPosMode( const u_int uAxis,const Asura_GUIMenu_Root::Asura_GUIMenu_Anchoring_Flag ePosMode );
	
	// copy this objects scaling flags to all of its children
	void ApplyScalingFlagsToChildren();

	// copy this objects anchoring flags to all of its children
	void ApplyAnchoringFlagsToChildren();

	// public static methods

	static void ApplyScalingFlagsToChildrenCallback();
	static void ApplyAnchoringFlagsToChildrenCallback();

	void SetTreeItem(Asura_TreeDialog_Item* pxTreeItem) {m_pxTreeItem = pxTreeItem;}
	Asura_TreeDialog_Item* GetTreeItem() {return m_pxTreeItem;}

	virtual Asura_Bounding_Rectangle GetRectangle() const;

	//Adds a comment indicating that the current user created this item
	void AddCreationComment();


	bool HasAnyAnimators()
	{
		if( m_pxAsuraRoot->GetAnimatorCount() == 0 ) return false;
		return true;
	}

	bool HasAnyModifiers()
	{
		if( m_pxAsuraRoot->GetModifierCount() == 0 ) return false;
		return true;
	}

	// Get the index of the current tab selected in the dialog box.
	static u_int GetSelectedDialogTab();

	// Is the Animator tab selected.
	static bool IsAnimatorTabSelected();

	// Is the Modifier tab selected.
	static bool IsModifierTabSelected();

	virtual void DeleteCurrentModifier();
	virtual void DeleteCurrentAnimator();

	//Returns true if the menu item uses the given texture
	virtual bool UsesTexture(int iTexture) const;

	//Sets the font for any menu items that it applied to
	void SetFontRecursive(const char* szFont);
	virtual void SetFont(const char* szFont){};

	//Reset the scale world bounds as this can be changed when previewing a gui menu
	//Tools requires scaled bounds and unscaled bound to remain the same during editing.
	void ResizeScaledWorldBoundsRecursive();

protected :
	void RenderRectangle(const Asura_Bounding_Rectangle& xRect, u_int uColour) const;
	void UpdateAfterParentDrag(const Asura_Bounding_Rectangle& xOldParentRect,const Asura_Bounding_Rectangle& xNewParentRect,u_int uDragFlags,bool bUsePreDragRect);
	void ClampToParentRect(Asura_Bounding_Rectangle& xRect, u_int uDragFlags);
	void UpdateRectAndRepositionChildren(const Asura_Bounding_Rectangle& xNewRect, bool bUsePreDragRect);


	//gets bitfield mask for the flags to show on the overlay.
	virtual u_int GetFlagsToEdit() const {return 0;}

	//Finds snap point by looking at sibling items or thr grid
	float GetSnapValue(float fCurrentValue,bool bAxis,float* pfScore = 0);
    		
	void CreateOverlay();
	void DeleteOverlay();
	void RecreateOverlay();

	//adds controls to the property overlay
	virtual void AddBasicOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);
	virtual void AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);
	virtual void AddExtraOverlayControls(Asura_Overlay* pxOverlay, Asura_Tabs_Dialog* pxTabs, Asura_Vector_2& xOffset) {} // CW - Empty for derived class use only


	// Adds all overlay dialogs for the Modifier tab to the overlay supplied - NDB
	void AddModifierOverlay( Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset );
	
	// Adds all overlay dialogs for the Animator tab to the overlay supplied - NDB
	void AddAnimatorOverlay( Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset );

	//updates any controls that have labels that change
	void UpdateOverlay();
	virtual void UpdateOverlayControls(Asura_Overlay* pxOverlay);

	//Updates the names of the tabs to add/remove * according to whether they contain anything of relevance
	void UpdateTabLabels();

	//updates controls when changing the selected modifier.
	void UpdateModifierOverlayControls();

	//updates controls when changing the selected animator.
	void UpdateAnimatorOverlayControls();

	// Add controls for modifier activation condition console vars
	void AddModifierActivationControls(Asura_GUIMenu_Modifier* pxModifier, Asura_Vector_2& xOffset);

	// Add controls for animator reset condition console vars
	void AddAnimatorResetControls(Asura_GUIMenu_Animator* pxAnimator, Asura_Vector_2& xOffset);

	//Add controls specific to the modifier type
	virtual void AddModifierOverlayControls(Asura_GUIMenu_Modifier* pxModifier, Asura_Vector_2& xOffset);

	//Used by AddModifierOverlayControls to add the dialogs to the overlay. Also adds them to a seperate list
	//for deletion later.
	void AddModifierDialog(Asura_Dialog* pxDialog);

	//Used by AddAnimatorOverlayControls to add the dialogs to the overlay. Also adds them to a seperate list
	//for deletion later.
	void AddAnimatorDialog(Asura_Dialog* pxDialog);

	//updates the asura menu item in response to overlay changes
	virtual void CopyToAsura();
	//copies asura data back to local copies in the tools data
	virtual void CopyFromAsura();

	//copies local data for current modifier to the asura data
	void CopyToAsuraModifier();

	//copy to/from asura for the current animator sequence
	void CopyToAsuraAnimator();
	void CopyFromAsuraAnimator();

	//recursively recalculates the IDs
	void RecalculateID();

	//updates referencse in response to an ID changing
	virtual void UpdateReferences(Asura_Hash_ID uOldID,Asura_Hash_ID uNewID,bool bPostCopy);

	//Gets the name of the flag that corresponds with this value , or 0 if it isn't a valid flag		
	virtual const char* GetLocalDataFlagName(u_int uFlagValue) const;

	// returns whether this object supports e.g. position, or UVs etc.
	virtual bool IsModifierTypeSupported( ASURA_GUIMENU_MODIFY_TYPE eType ) const;

	//Sets some default data in a newly created modifier.
	virtual void InitialiseModifier(Asura_GUIMenu_Modifier* pxModifier);

	//Called when switching between editing different modifiers , to allow any stuff for the previous modifier
	//to be cleared up.
	virtual void CleanupModifierEditing(){}

	void UpdateFromStyle(const Tools_GUIMenu_Root* pxStyle);

	//for preserving text IDs when applying styles to existing items
	virtual bool GetTextToPreserve(Asura_Hash_ID& uTextID, Asura_Hash_ID& uTextPage) const;
	virtual void ApplyPreservedText(Asura_Hash_ID uTextID, Asura_Hash_ID uTextPage);

	void AddToGroup(Tools_GUIMenu_Root* pxGroup);

	void AddToHashTree();
	void RemoveFromHashTree();


	//undo logging
	void LogUndo_Rectangle(const Asura_Bounding_Rectangle& xNewRect);


	
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_GUIMenu_Root*)iThis)->__FnName();}\
	virtual void __FnName();
	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_GUIMenu_Root*)iThis)->__FnName();}\
	virtual bool __FnName();


	OVERLAY_VOID_CALLBACK(Rename)

	OVERLAY_VOID_CALLBACK(CenterWithinParent)
	OVERLAY_VOID_CALLBACK(MoveToFront)
	OVERLAY_VOID_CALLBACK(MoveForwards)
	OVERLAY_BOOL_CALLBACK(CanMoveToFront)
	OVERLAY_VOID_CALLBACK(MoveToBack)
	OVERLAY_VOID_CALLBACK(MoveBackwards)
	OVERLAY_BOOL_CALLBACK(CanMoveToBack)

	OVERLAY_BOOL_CALLBACK(CanEditStyleFlags)
	
	OVERLAY_VOID_CALLBACK(SelectStyleMaster)
	OVERLAY_BOOL_CALLBACK(CanSelectStyleMaster)
	OVERLAY_VOID_CALLBACK(RemoveStyle)
	OVERLAY_BOOL_CALLBACK(CanRemoveStyle)
	OVERLAY_BOOL_CALLBACK(CanBeStyle)

	OVERLAY_BOOL_CALLBACK(CanEditColour)
	OVERLAY_BOOL_CALLBACK(CanEditTranslucency)
	OVERLAY_BOOL_CALLBACK(CanEditFlags)
	OVERLAY_BOOL_CALLBACK(CanEditAnchoring)
	OVERLAY_BOOL_CALLBACK(CanEditScaling)
	OVERLAY_BOOL_CALLBACK(CanEditPlatforms)

	OVERLAY_VOID_CALLBACK(AddModifier)
	OVERLAY_BOOL_CALLBACK(CanAddModifier)
	OVERLAY_VOID_CALLBACK(DeleteModifier)

	OVERLAY_BOOL_CALLBACK(IsModifierSelected)
	OVERLAY_BOOL_CALLBACK(IsEditableModifierSelected)

	OVERLAY_VOID_CALLBACK(AddAnimator)
	OVERLAY_VOID_CALLBACK(DeleteAnimator)

	OVERLAY_BOOL_CALLBACK(IsEditableAnimatorSelected)

	OVERLAY_VOID_CALLBACK(AddToGroup)
	OVERLAY_VOID_CALLBACK(RemoveFromGroup)
	OVERLAY_BOOL_CALLBACK(CanAddToGroup)
	OVERLAY_BOOL_CALLBACK(CanRemoveFromGroup)

	OVERLAY_VOID_CALLBACK(AddDisplayConditionVar)
	OVERLAY_VOID_CALLBACK(AddModifierActivationConditionVar)
	OVERLAY_VOID_CALLBACK(AddAnimatorResetConditionVar)

	OVERLAY_VOID_CALLBACK(EditComments)

	OVERLAY_VOID_CALLBACK(SetAllPlatforms)
	OVERLAY_VOID_CALLBACK(ClearAllPlatforms)

	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK

	static Tools_GUIMenu_Root* GetSelectedInstance() { return s_pxCurrentSelectedRoot; }

private:

	// If the state of the Hide tickbox menu item changes then recursively change the
	// state of all its children.
	static void HideTickboxStateChanged();

	u_int GetIndexInParent() const;
	void SetIndexInParent(u_int uIndex,bool bUpdateAsuraAndTree = true);
	void UpdateChildOrderInAsuraDataAndTree();

	int  FindTranslucencyDropListIndex( void);

	void UpdateModifierList();
	void DeleteModifierList();

	void UpdateAnimatorList();
	void DeleteAnimatorList();

protected :

	Asura_GUIMenu_Root* m_pxAsuraRoot;

	Tools_GUIMenu_Root* m_pxToolsParent;
	List<Tools_GUIMenu_Root*> m_xChildren;

	//rect before starting a drag operation
	Asura_Bounding_Rectangle m_xPreDragRect;


	//list of all instances of this class
	static List<Tools_GUIMenu_Root*> s_xList;

	//local copy of data for editing by overlay
	u_int m_uFlags;
	Asura_RGBA m_xColour;
	u_int m_uToolsFlags;

	bool m_bChanged;
	bool m_bPosOptionsChanged;
	bool m_bBeingDestroyed;

	bool m_bSelected;
	bool m_bHidden;

	//separate copy of the extents for editing on the overlay
	bool m_bMoved;
	bool m_bResized;
	float m_fX;
	float m_fY;
	float m_fW;
	float m_fH;

	//local copy of data for current modifier
	u_int m_uModifierStates;
	Asura_RGBA m_xModifierData_Colour;
	Asura_Vector_2 m_xModifierData_Vector2;
	Asura_String m_xModifierData_String;
	Asura_GUIMenu_UV_Set m_xModifierData_UV;
	int m_iModifierData;
	u_int m_uModifierData;
	float m_fModifierData;
	bool m_bModifierChanged;

	//local copy of data for current animator
	float m_fAnimatorSpeed;
	float m_fAnimatorDelay;
	bool m_bAnimatorChanged;
	bool m_bAnimatorShouldStartActive;
	bool m_bAnimatorInitialiseToEnd;

	Asura_String m_xName;

private :
	Asura_Vector_2 m_xTabsStart;
	Asura_Vector_2 m_xTabsEnd;


	Asura_Vector_2 m_xPixelOrigin;
	bool m_bPosOriginChanged;

	u_int m_uStyleChildEditableDataFlags;
	
	int	  m_iTranslucencyMode;
	bool  m_bTranslucencyChanged;

	bool m_bIsStyleChanged;
	bool m_bStyleContainerChanged;
	bool m_bDisplayConditionChanged;
	bool m_bScaleFlagChanged;

	bool m_bIsClampedChanged;

	int	m_iCompVal1;
	int m_iCompCond1;

	int	m_iCompVal2;
	int m_iCompCond2;

	bool m_bCompValChanged;

	bool m_bInHashTree;
	Asura_Hash_ID m_uID;//copy of the ID from the asura data mainly to make sure it isn't changing without the hash tree knowing about it)

	Asura_TreeDialog_Item* m_pxTreeItem;

	//data for modifier list
	int m_iModifierListSize;
	int m_iModifierIndex;
	bool m_bModifierIndexChanged;
	bool m_bModifierConditionChanged;
	const char** m_pszModifierStrings;
	static Asura_List_Dialog* s_pxModifierListDialog;

	//data for animator list
	int m_iAnimatorListSize;
	int m_iAnimatorIndex;
	bool m_bAnimatorIndexChanged;
	bool m_bAnimatorConditionChanged;
	char** m_pszAnimatorStrings;

	bool m_bOverlayCreated; // set true by a successful call to CreateOverlay

	static Asura_List_Dialog* s_pxAnimatorListDialog;

	// private static data
	static bool s_bIsRenaming;
	static Asura_Tabs_Dialog* s_pxTabsDialog;
	static Asura_Subwindow_Dialog* s_pxDispCondSubwindowDialog;
	static Asura_Vector_2 s_xModifierDialogOffset;//offset of controls specific to different modifier types
	static List<Asura_Dialog*> s_xModifierDialogList;//list of controls that will need deleting when a new modifier is selected.

	static Asura_Vector_2	   s_xAnimatorDialogOffset;//offset of controls specific to different animator types
	static List<Asura_Dialog*> s_xAnimatorDialogList;//list of controls that will need deleting when a new animator is selected.

	static int s_iStyleTab;
	static int s_iModifierTab;
	static int s_iAnimatorTab;
	static int s_iDisplayConditionTab;
	static int s_iPlatformTab;

	static Asura_TemplateHashTree<Tools_GUIMenu_Root*> s_xHashTree;

	static Tools_GUIMenu_Root* s_pxCurrentSelectedRoot;

public :
	static Asura_Vector_2 s_xOverlayPosition;
	static u_int s_uOverlayMinimised;
};


//Fake menu item for representing the file that contains the menus
class Tools_GUIMenu_File : public Tools_GUIMenu_Root
{
	typedef Tools_GUIMenu_Root PARENT;

public :
	Tools_GUIMenu_File(const char* szName,Tools_GUIMenu_Root* pxToolsParent);

	virtual bool IsValidChildType(int iItemType) const;
	virtual void SetModified();
	virtual const char* GetTypeName() const {return "File";}
	virtual bool IsFile() const {return true;}


	int GetSortKey() const {return m_iSortKey;}

	static void IgnoreSetModified(bool bIgnore) {s_bIgnoreSetModified = bIgnore;}


protected :
	virtual bool ConfirmDeletion();
	virtual void AddBasicOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);
	virtual void Update();


private :
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_GUIMenu_File*)iThis)->__FnName();}\
	void __FnName();
	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_GUIMenu_File*)iThis)->__FnName();}\
	bool __FnName();

	OVERLAY_VOID_CALLBACK(ExportFile);
	OVERLAY_BOOL_CALLBACK(CanExportFile);

	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK


	int m_iSortKey;
	bool m_bSortKeyChanged;

	static bool s_bIgnoreSetModified;
};


//overlay for displaying and changing comments
class Tool_Comments_Overlay : public Tools_Modal_Overlay
{
public :
	Tool_Comments_Overlay(Asura_String _Title, Asura_GUIMenu_Root* pxRootObject);
	void DisableCancel(){m_bDisableCancel = true;}

	bool DoModal();


protected:
	static Asura_GUIMenu_Root* s_pxRootObject;
	static Asura_List<Asura_GUIMenu_ToolComment>*	s_pxCommentList;
	static bool	s_bRefreshLayout;
private :
	virtual void InitialiseOverlay(class Asura_Overlay* Overlay);
	virtual void Update();

	bool m_bInputDebounced;
	bool m_bDisableCancel;

	//unfortunately this lot has to be static... but we should never end up
	//with more than 1 copy of the overlay in action at a time
	static void CloseSelected(){}

	static void AddComment( void);

	static void _EditComment( int iValue);
	static void _RemoveComment( int iValue);

	static bool ReturnValue;
	
};

#endif //ASURA_USE_GUI_MENU_SYSTEM
#endif //_TOOLS_GUIMENU_ROOT_H_

