//
// Tools_GUIMenu_Widget_Base.h
//
// Created by Richard Rice - 30/11/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//


#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif


#ifndef _TOOLS_GUIMENU_WIDGET_BASE_H_
#define _TOOLS_GUIMENU_WIDGET_BASE_H_
#ifdef ASURA_USE_GUI_MENU_SYSTEM


#include "GUIMenu/Tools_GUIMenu_Root.h"


class Asura_GUIMenu_Widget_Base;

class Tools_GUIMenu_Widget_Base : public Tools_GUIMenu_Root
{
    typedef Tools_GUIMenu_Root PARENT;
	friend class Tools_GUIMenu_PickItem_Link;

public :
	Tools_GUIMenu_Widget_Base(Asura_GUIMenu_Widget_Base* pxAsuraWidget,Tools_GUIMenu_Root* pxToolsParent);

	// virtual overrides
	virtual void InitialiseNewItem();

	virtual bool CanBeResized() const {return true;}
	virtual bool CanBeMoved() const;
	virtual bool IsWidget() const {return true;}

	virtual void RenderUI();

	//Adds controls for editing the condition settings for an element based on the widget type
	virtual void AddChildConditionOverlayControls(Tools_GUIMenu_Root* pxItem,Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset){};

	virtual bool IsValidChildType(int iItemType) const;

	virtual bool Verify();
	virtual bool NeedsPosOptions() const {return true;}

	//Gets confirmation as to whether a menu item should be deleted.
	virtual bool ConfirmDeletion();

protected :
	virtual void Update();

	// helper functions
	virtual void AddBasicOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);
	virtual void AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);
	virtual void AddExtraOverlayControls(Asura_Overlay* pxOverlay, Asura_Tabs_Dialog* pxTabs, Asura_Vector_2& xOffset);
	virtual void UpdateOverlayControls(Asura_Overlay* pxOverlay);
	virtual void UpdateReferences(Asura_Hash_ID uOldID,Asura_Hash_ID uNewID,bool bPostCopy);
	virtual const char* GetLocalDataFlagName(u_int uFlagValue) const;
	virtual bool IsNavExitSupported(int iLink) const {return true;} 	//Returns true if the nav exit is supported by the widget type

	void UpdateLinkControl(Asura_Overlay* pxOverlay,const char* szBaseText,Asura_Hash_ID uControlID,int iLink);
	bool CheckLinksCanBeEdited();
	void SetLink(int iLink,Tools_GUIMenu_Widget_Base* pxItem);
	void RenderLink(int iLink,const char* szLabel);

	// (sort of) static functions
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){return ((Tools_GUIMenu_Widget_Base*)iThis)->__FnName();}\
	virtual void __FnName();
	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_GUIMenu_Widget_Base*)iThis)->__FnName();}\
	virtual bool __FnName();
	#define OVERLAY_BOOLPARAM_CALLBACK(__FnName)\
	static void __FnName(bool bValue, void* pThis){return ((Tools_GUIMenu_Widget_Base*)pThis)->__FnName(bValue);}\
	virtual void __FnName(bool);
	#define OVERLAY_GETBOOL_CALLBACK(__FnName)\
	static bool __FnName(void* pThis){return ((Tools_GUIMenu_Widget_Base*)pThis)->__FnName();}\
	virtual bool __FnName();

	OVERLAY_VOID_CALLBACK(AddElement_Text)
	OVERLAY_VOID_CALLBACK(AddElement_Image)
	OVERLAY_VOID_CALLBACK(AddElement_Path)
	OVERLAY_VOID_CALLBACK(AddElement_Anim)
	OVERLAY_VOID_CALLBACK(AddElement_StatusText)
	OVERLAY_VOID_CALLBACK(AddElement_FMV)

	OVERLAY_VOID_CALLBACK(EditUpLink)
	OVERLAY_VOID_CALLBACK(EditDownLink)
	OVERLAY_VOID_CALLBACK(EditLeftLink)
	OVERLAY_VOID_CALLBACK(EditRightLink)
	OVERLAY_VOID_CALLBACK(ResetNavLinks)

	OVERLAY_BOOL_CALLBACK(CanEditNavLeft)
	OVERLAY_BOOL_CALLBACK(CanEditNavRight)
	OVERLAY_BOOL_CALLBACK(CanEditNavUp)
	OVERLAY_BOOL_CALLBACK(CanEditNavDown)
	OVERLAY_BOOL_CALLBACK(CanEditSelectable)
	OVERLAY_BOOL_CALLBACK(CanEditIsAlwaysSelectable)

	OVERLAY_BOOL_CALLBACK(CanEditOnRollInCommand)
	OVERLAY_BOOL_CALLBACK(CanEditOnRollOutCommand)

	OVERLAY_BOOLPARAM_CALLBACK(SetCanBeSelectable);
	OVERLAY_BOOLPARAM_CALLBACK(SetIsAlwaysSelectable);

public :
	OVERLAY_GETBOOL_CALLBACK(GetCanBeSelectable);
	OVERLAY_GETBOOL_CALLBACK(GetIsAlwaysSelectable);

protected :

	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK
	#undef OVERLAY_BOOLPARAM_CALLBACK
	#undef OVERLAY_GETBOOL_CALLBACK

	// data
	Asura_GUIMenu_Widget_Base* m_pxAsuraWidget;

	static bool s_bShowLinks;
	static bool s_bTwoWayLinks;
};




#endif //ASURA_USE_GUI_MENU_SYSTEM
#endif //_TOOLS_GUIMENU_WIDGET_BASE_H_
