//
// Tools_Mode_GUIMenu.cpp
//
// Created by Richard Rice - 29/11/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "GUIMenu/Tools_Mode_GUIMenu.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM

#include "ChunkSystem/Asura_Chunk_HashedLocalisedText.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_LocalisedText.h"
#include "Core/Asura_HashedLocalisedText.h"

#include "FMV/Asura_VideoToTexture_System.h"

#include "GUIMenu/Asura_GUIMenu_Element_FMV.h"
#include "GUIMenu/Asura_GUIMenu_Message.h"
#include "GUIMenu/Asura_GUIMenu_MessagePump.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Asura_GUIMenu_Widget_TextListBox.h"

#ifdef ASURA_USE_CONTROLMAP_MANAGER
#include "Input/Asura_ControlManager.h"
#endif

#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Tree_Dialog.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Direct3D.h"

#include "Tools_Overlays.h"
#include "Tools_System.h"
#include "GUIMenu/Tools_GUIMenu_AnimEdit.h"
#include "GUIMenu/Tools_GUIMenu_Audio_Overlay.h"
#include "GUIMenu/Tools_GUIMenu_System.h"
#include "GUIMenu/Tools_GUIMenu_Root.h"
#include "GUIMenu/Tools_GUIMenu_Menu_Page.h"
#include "GUIMenu/Tools_GUIMenu_Element_Animation.h"
#include "GUIMenu/Tools_GUIMenu_Element_FMV.h"
#include "GUIMenu/Tools_GUIMenu_Element_Image.h"
#include "GUIMenu/Tools_GUIMenu_Element_Path.h"
#include "GUIMenu/Tools_GUIMenu_Element_Text.h"
#include "GUIMenu/Tools_GUIMenu_Element_StatusText.h"
#include "GuiMenu/Tools_GUIMenu_ImportText.h"
#include "GUIMenu/Tools_GUIMenu_Widget_Base.h"
#include "GUIMenu/Tools_GUIMenu_Widget_Button.h"
#include "GUIMenu/Tools_GUIMenu_Widget_CheckBox.h"
#include "GUIMenu/Tools_GUIMenu_Widget_Slider.h"
#include "GUIMenu/Tools_GUIMenu_Widget_TextBox.h"
#include "GUIMenu/Tools_GUIMenu_Widget_ListBox.h"
#include "GUIMenu/Tools_GUIMenu_Widget_TextListBox.h"
#include "GUIMenu/Tools_GUIMenu_Widget_Group.h"
#include "GUIMenu/Tools_GUIMenu_Widget_AnimatingImage.h"
#include "GUIMenu/Tools_GUIMenu_Widget_ProgressBar.h"
#include "GUIMenu/Tools_GUIMenu_Widget_Numeric.h"
#include "GUIMenu/Tools_GUIMenu_PreviewOverlay.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Interface/Tools_ColourPicker.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Render_Options.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_MessageLog.h"
#include "Movement/Tools_Movement.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Texturing/Tools_PickTexture.h"

///////////////////////////////////////////////////////////////////////////////////////
/*--------------**
** Help Strings **
**--------------*/
static const char* g_aszGUIMenuModeHelpStrings [] =
{
	"LMB :             Select menu item under cursor , with preference being given to widgets",
	"+Ctrl :           Select topmost menu item under cursor",
	"Drag MMB :        Drags the menu display around , changing the menu offests",
	"Drag LMB :        Drag/resize selected menu item",
	"+Ctrl :           Snaps dragging to extents of other menu items",
	"+Shift :          Restricts dragging to a single axis",
	"Mousewheel :      Zoom in/out",
	"F11 :             Toggle preview menu",
	"",
	"Delete :          Delete selected menu item",
	"Ctrl+C :          Copy selected menu item",
	"Ctrl+V :          Pastes a new instance of the copied menu item",
	"Ctrl+ShifT+V :    Pastes over the selected item",
	"1 :               Select previous language",
	"2 :               Select next language",
	"3 :               Select previous screen settings preset",
	"4 :               Select next screen settings preset",
	"",
	"Shift+F1,...,Shift+F11  Save the current selection to the specified hotkey",
	"Ctrl+F1,...,Ctrl+F11    Select a previously hotkeyed item",
	"Ctrl+F12          Selects menu item which most recently used 'Goto Master Copy'.",
	"",
	"**Path Editing**",
	"LMB :             Select a point.",
	"LMB+Drag :        Drags the point under the cursor",
	"Shift+LMB :       Creates a new point at the cursor",
	"Delete :          Deletes the selected point",
	"",
	"H :               Hide the selected menu item and all its children",
	"Shift+H :         Unhide the selected menu item and all its children",
	"I :               Toggles 'Display Isolation' switch for the selected menu item and all its children.  This is a global setting for the menu system.",
	"",
	"Ctrl+Z :          Undo (Movement/Resize only)",
	"Ctrl+Y :          Redo (Movement/Resize only)",
	"< :               Undo (Movement/Resize only)",
	"> :               Redo (Movement/Resize only)",

	NULL
};


///////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------**
** Tools_MessageLogAction_GUIMenu **
**--------------------------------*/

//For associating menu items with message log entries
class Tools_MessageLogAction_GUIMenu : public Tools_MessageLogAction
{
public :
	Tools_MessageLogAction_GUIMenu(const Tools_GUIMenu_Root* pxItem) {m_uHashID = pxItem ? pxItem->GetID() : 0;}
	virtual void Execute()
	{
		//Select the item if it still exists (and we are in menu mode)
		if(!Tools_Mode_GUIMenu::GetInstance()) return;

		Tools_GUIMenu_Root* pxItem = Tools_Mode_GUIMenu::GetInstance()->FindItemByID(m_uHashID);
		if(pxItem)
		{
			Tools_Mode_GUIMenu::GetInstance()->SetSelection(pxItem);
		}
	}

	Asura_Hash_ID m_uHashID;
};

///////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------**
** Tools_GUIMenu_Undo_Rectangle **
**-----------------------------*/

class Tools_GUIMenu_Undo_Rectangle : public RootUndoData
{
public :
	Tools_GUIMenu_Undo_Rectangle(const Tools_GUIMenu_Root* pxItem, const Asura_Bounding_Rectangle& xOldRect)
	{
		m_uID = pxItem->GetID();
		m_xRect = xOldRect;
	}

	virtual RootUndoData* ProcessUndoData()
	{
		//Find the item being affected. Since most things aren't being logged for undo , it
		//is quite possible that it will no longer exist , in which case the undo can basically
		//be skipped
		Tools_GUIMenu_Root* pxItem = Tools_Mode_GUIMenu::GetInstance()->FindItemByID(m_uID);
		if(pxItem)
		{
			//swap the rectangle between the undo data and the menu item
			Asura_Bounding_Rectangle xTempRect = pxItem->GetRectangle();
			pxItem->SetRectangle(m_xRect);
			m_xRect = xTempRect;

			pxItem->SetModified();
		}
		return this;
	}

	Asura_Hash_ID m_uID;
	Asura_Bounding_Rectangle m_xRect;
};

///////////////////////////////////////////////////////////////////////////////////////
/*--------------------**
** Tools_Mode_GUIMenu **
**--------------------*/

Tools_GUIMenu_Root* Tools_Mode_GUIMenu::s_pxToolsMenuFile(NULL);
Tools_Mode_GUIMenu* Tools_Mode_GUIMenu::s_pxThis(NULL);

u_int Tools_Mode_GUIMenu::s_uBackgroundColour = 0xff000000;
u_int Tools_Mode_GUIMenu::s_uGridColour = 0x2f7f7f7f;
bool Tools_Mode_GUIMenu::s_bShowGrid = true;
int Tools_Mode_GUIMenu::s_iGridSize = 20;
float Tools_Mode_GUIMenu::s_fZoom = 1.0f;
int Tools_Mode_GUIMenu::s_iMenuOffsetX = 0;
int Tools_Mode_GUIMenu::s_iMenuOffsetY = 0;
int Tools_Mode_GUIMenu::s_iGridOriginX = 0;
int Tools_Mode_GUIMenu::s_iGridOriginY = 0;
bool Tools_Mode_GUIMenu::s_bSnapToGrid = false;
bool Tools_Mode_GUIMenu::s_bResizeChildrenWithParent = true;
bool Tools_Mode_GUIMenu::s_bRepositionChildrenWithinParent = false;
float Tools_Mode_GUIMenu::s_fRepositionChildren_CornerWeighting = 1.0f;
float Tools_Mode_GUIMenu::s_fRepositionChildren_CentreWeighting = 1.0f;
float Tools_Mode_GUIMenu::s_fRepositionChildren_StickTogetherWeighting = 1.0f;
bool Tools_Mode_GUIMenu::s_bHideSiblings = false;
bool Tools_Mode_GUIMenu::s_bRenderOutlineForHiddenItems = true;
bool Tools_Mode_GUIMenu::s_bAllowMouseSelectionOfHiddenItems = true;
bool Tools_Mode_GUIMenu::s_bDisableBilinearFiltering = false;

bool Tools_Mode_GUIMenu::s_bMenuPreview = false;
bool Tools_Mode_GUIMenu::s_bDoneOneOffStyleUpdate = false;
bool Tools_Mode_GUIMenu::s_bInitialUseCursor = false;


Asura_Vector_2 Tools_Mode_GUIMenu::s_xScreenSize(1024.0f,768.0f);
Asura_Vector_2 Tools_Mode_GUIMenu::s_xVirtualSize(0.0f,0.0f);
Asura_Bounding_Rectangle Tools_Mode_GUIMenu::s_xSafeArea(0.0f,0.0f,1024.0f,768.0f);
bool Tools_Mode_GUIMenu::s_bShowScreenBorder = false;
bool Tools_Mode_GUIMenu::s_bShowSafeArea = false;
u_int Tools_Mode_GUIMenu::s_uScreenBorderColour = 0x7fff0000;
u_int Tools_Mode_GUIMenu::s_uSafeAreaColour = 0x7f0000ff;
bool Tools_Mode_GUIMenu::s_bClipToScreen = false;
int Tools_Mode_GUIMenu::s_iScreenSettingPreset = -1;
int Tools_Mode_GUIMenu::s_iPreviewPlatform = TOOLS_PLATFORM_PC;
const char** Tools_Mode_GUIMenu::s_pszPlatformNames = 0;

bool Tools_Mode_GUIMenu::s_bCopyingMenu = false;

Asura_Function_Dialog* Tools_Mode_GUIMenu::s_pxListTextModifiersCurrentlySelectedMenu	= NULL;
Asura_Function_Dialog* Tools_Mode_GUIMenu::s_pxListTextModifiersCurrentlySelectedFile	= NULL;

static const char* g_apszScreenSettingNames[TOOLS_GUIMENU_NUM_SCREENSETTINGS] =
{
	"Current Resolution",	//TOOLS_GUIMENU_SCREENSETTINGS_CURRENTRES,
	"PSP",					//TOOLS_GUIMENU_SCREENSETTINGS_PSP,
	"PS2 PAL",				//TOOLS_GUIMENU_SCREENSETTINGS_PS2_PAL,
	"PS2 NTSC",				//TOOLS_GUIMENU_SCREENSETTINGS_PS2_NTSC,
	"PS2 NTSC Widescreen",	//TOOLS_GUIMENU_SCREENSETTINGS_PS2_NTSC_WIDE,
	"Wii PAL50 4:3",		//TOOLS_GUIMENU_SCREENSETTINGS_WII_PAL_4_3,
	"Wii PAL50 16:9",		//TOOLS_GUIMENU_SCREENSETTINGS_WII_PAL_16_9,
	"Wii NTSC/PAL60 4:3",	//TOOLS_GUIMENU_SCREENSETTINGS_WII_NTSC_4_3,
	"Wii NTSC/PAL60 16:9",	//TOOLS_GUIMENU_SCREENSETTINGS_WII_NTSC_16_9,

	"PS3 4:3",					//TOOLS_GUIMENU_SCREENSETTINGS_PS3_4_3,
	"PS3 16:9",					//TOOLS_GUIMENU_SCREENSETTINGS_PS3_16_9,

	"XBox 360 4:3 (Req)",	// TOOLS_GUIMENU_SCREENSETTINGS_XBOX360_4_3_REQUIRED,
	"XBox 360 4:3 (Rec)",//TOOLS_GUIMENU_SCREENSETTINGS_XBOX360_4_3_RECOMMENDED,
	"XBox 360 16:9 (Req)",	// TOOLS_GUIMENU_SCREENSETTINGS_XBOX360_16_9_REQUIRED,
	"XBox 360 16:9 (Rec)",//TOOLS_GUIMENU_SCREENSETTINGS_XBOX360_16_9_RECOMMENDED,


};


static int g_aiWidgetTypes[] =
{
	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_BUTTON,
	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_CHECKBOX,
	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_SLIDER,
	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTBOX,
	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX,
	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_GROUP,
//	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_ANIMATINGIMAGE,
	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_PROGRESSBAR,
	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_NUMERIC,
	Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX,
};
static int g_iNumWidgetTypes = sizeof(g_aiWidgetTypes) / sizeof(g_aiWidgetTypes[0]);
static Asura_Hash_ID g_uLastTextPageHash	= ASURA_HASH_ID_UNSET;
static Asura_Hash_ID g_uLastInitialTextID	= ASURA_HASH_ID_UNSET;		// remember the last text ID we chose, to default to next time
static ListSelection_Overlay g_xTextIDOverlay("Select Text");
static ListSelection_Overlay g_xTextPageOverlay("Select Text Page");

static const Asura_Hash_ID g_uMenuItemTreeID = Asura_GetHashID("MenuItemTree");

void Tools_Mode_GUIMenu::QuickSave(void)
{
	Tools_GUIMenu_System::SaveModified();
}

bool Tools_Mode_GUIMenu::IsModified()
{
	return Tools_GUIMenu_System::IsModified();
}


Tools_Mode_GUIMenu::Tools_Mode_GUIMenu()
:m_xOverlayPosition(0.0f,3.0f)
,m_uOverlayMinimised(0)
,m_xStyleOverlayPosition(20.0f,3.0f)
,m_uStyleOverlayMinimised(0)
,m_bStyleOverlayOpen(true)
,m_xUndo(10000)
,m_bDisableUndoLogging(false)
{
	s_pxThis = this;

	m_pxToolsMenu = 0;
	s_pxToolsMenuFile = 0;
	m_pxRoot = 0;

	m_pxSelectedItem = 0;
	m_pxClipboardItem = 0;

	m_bDrag = 0;
	m_uDragFlags = 0;

	m_bItemPlacement = false;
	m_iPlacementType = 0;
	m_pxPlacementParent = 0;
	m_pxPlacementSource = 0;

	m_pxPickItem = 0;

	m_pxTree = new Asura_TreeDialog_Tree;
//	m_pxTree->EnableMultipleSelection(true);
	m_bTreeSelectionChanged = false;

	m_iNumStyleTypes = 0;
	m_pszStyleTypeNames = 0;
	m_piStyleTypes = 0;
	m_iStyleTypeIndex = 0;
	m_bStyleTypeChanged = false;

	m_iNumStyles = 0;
	m_pszStyleNames = 0;
	m_ppxStyleItems = 0;
	m_iStyleIndex = 0;
	m_pxStyleListDialog = 0;

	m_iLanguageIndex = 0;
	m_iLanguageListSize = 0;
	m_pszLanguageNames = 0;
	m_piLanguageIDs = 0;

	m_iMenuHandle = ASURA_INVALID_HANDLE;

	m_bTemporaryModeInstance = false;

	for(int i=0;i<12;i++)
	{
		m_auHotkeyedSelection[i] = 0;
	}

	s_pszPlatformNames = new const char*[TOOLS_PLATFORM_NUM_PLATFORMS];
	for(int i=0; i<TOOLS_PLATFORM_NUM_PLATFORMS; i++)
	{
		s_pszPlatformNames[i] = Tools_Platform::GetPlatformName(i);
	}

	m_bIsolateDisplay = false;
}

Tools_Mode_GUIMenu::~Tools_Mode_GUIMenu()
{
 	delete m_pxTree;
	s_pxThis = 0;

	delete [] s_pszPlatformNames;
	s_pszPlatformNames = 0;

	s_pxListTextModifiersCurrentlySelectedMenu	= NULL;
	s_pxListTextModifiersCurrentlySelectedFile	= NULL;
}

void Tools_Mode_GUIMenu::InitialiseMode()
{
	TransferModeData(TOOLSMODEDATAFILE_LOAD);

	Asura_GUIMenu_FMV_System::Initialise();

	s_bMenuPreview = false;

	Tools_Render_Options::HideEnvironment = true;
	Tools_Movement::Enable(false);

	m_iPreviousScreenHeight = Asura_Viewport::GetHeight();
	if(!m_bTemporaryModeInstance)
	{
		CreateOverlay();
	}

	m_pxRoot = new Tools_GUIMenu_Root( NULL,NULL,"Root" );

	CreateToolsMenuItems();

	if(!m_bTemporaryModeInstance)
	{
		m_iMenuHandle = Asura_Menubar_System::CreateMenuSection();
		Asura_Menubar_Title* pxTitle;

		//add stuff to the tools menus
		pxTitle = Asura_Menubar_System::AddTitle(m_iMenuHandle,"Overlays");
		pxTitle->AddItem("Animators",Tools_GUIMenu_AnimEdit::ToggleOverlay,Tools_GUIMenu_AnimEdit::OverlayIsOn);
		pxTitle->AddItem("Style",ToggleStyleOverlay,StyleOverlayIsOn);

		if(m_bStyleOverlayOpen) CreateStyleOverlay();
		Tools_GUIMenu_PreviewOverlay::Initialise();
		Tools_GUIMenu_AnimEdit::Initialise();

		Tools_Help::AddHelpTopic("Menu Mode",g_aszGUIMenuModeHelpStrings);
	}

	//Update all style instances
	//Don't other with setting files as modified while doing this , since otherwise
	//they will all become modified every time the mode is used
	Tools_GUIMenu_File::IgnoreSetModified(true);
	m_pxRoot->UpdateStyleInstances();
	HandleStyleUpdates();//deal with updates caused by styles including other styles
	//update all menu items
	for(CLIF<Tools_GUIMenu_Root*> xLIF(Tools_GUIMenu_Root::GetList());!xLIF.done();xLIF.next())
	{
		xLIF()->Update();
	}

	Tools_GUIMenu_File::IgnoreSetModified(false);
	s_bDoneOneOffStyleUpdate = true;


	if(!m_bTemporaryModeInstance)
	{
		Verify();

		m_xUndo.Initialise();
		m_xUndo.SetVerbose(true);
	}
}

void Tools_Mode_GUIMenu::ShutdownMode()
{
	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	MenuPreview_End();
	m_xUndo.Shutdown();

	SetSelection(0);
	CancelPromptedSelection();
	
	m_pxToolsMenu = 0;
	s_pxToolsMenuFile = 0;
	m_pxSelectedItem = 0;
	m_pxClipboardItem = 0;

	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_GUIMENU);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_GUIMENU_STYLE);
	Tools_GUIMenu_PreviewOverlay::Shutdown();
	Tools_GUIMenu_AnimEdit::Shutdown();

	if(m_iMenuHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Menubar_System::DestroyMenuSection(m_iMenuHandle);
	}
	m_iMenuHandle = ASURA_INVALID_HANDLE;


	DeleteStyleTypeList();
	DeleteStyleNameList();

	Tools_Render_Options::HideEnvironment = false;
	Tools_Movement::Enable(true);

	//Make sure the current language is loaded
	if(!m_bTemporaryModeInstance)
	{
		ReloadText(Asura_LocalisedText::GetCurrentLanguage());
	}

	DeleteLanguageList();

	if(!m_bTemporaryModeInstance)
	{
		Tools_Help::RemoveHelpTopic("Menu Mode");
	}

	m_bIsolateDisplay = false;

	Asura_GUIMenu_FMV_System::Shutdown();

	delete m_pxRoot;
	m_pxRoot = 0;
}

void Tools_Mode_GUIMenu::TransferModeData(int iMode)
{
	const int iVersion = 1;
	//open the file
	Tools_ModeDataFile xFile("GUIMenu.dat",iMode, iVersion);

	//tranfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	xFile.Transfer(m_xOverlayPosition);
	xFile.Transfer(s_uGridColour);
	xFile.Transfer(Tools_GUIMenu_Root::s_xOverlayPosition);
	xFile.Transfer(m_xStyleOverlayPosition);
	xFile.Transfer(Tools_GUIMenu_PreviewOverlay::s_xOverlayPosition);
	xFile.Transfer(Tools_GUIMenu_AnimEdit::s_bOverlayOpen);
	xFile.Transfer(Tools_GUIMenu_AnimEdit::s_xOverlayPosition);
	xFile.Transfer(Tools_GUIMenu_AnimEdit::s_xEditSeqOverlayPosition);
	xFile.Transfer(m_bStyleOverlayOpen);
	xFile.Transfer(Tools_GUIMenu_PreviewOverlay::s_bUseCursor);

	xFile.Transfer(s_bShowScreenBorder);
	xFile.Transfer(s_bShowSafeArea);
	xFile.Transfer(s_uScreenBorderColour);
	xFile.Transfer(s_uSafeAreaColour);
	xFile.Transfer(s_bClipToScreen);
	xFile.Transfer(Tools_GUIMenu_PreviewOverlay::s_bCentreScreen);
	xFile.Transfer(Tools_GUIMenu_PreviewOverlay::s_bShowScreenBorder);
	xFile.Transfer(Tools_GUIMenu_PreviewOverlay::s_bShowSafeArea);
	xFile.Transfer(Tools_GUIMenu_PreviewOverlay::s_bClipToScreen);
	xFile.Transfer(s_bShowGrid);
	xFile.Transfer(s_iGridSize);

	if(xFile.GetVersion() < 1)
	{
		bool bJunk = false;
		xFile.Transfer(bJunk);
		xFile.Transfer(bJunk);
		xFile.Transfer(bJunk);
		xFile.Transfer(bJunk);
		xFile.Transfer(bJunk);
	}
	else
	{
		xFile.Transfer(m_uOverlayMinimised);
		xFile.Transfer(m_uStyleOverlayMinimised);
		xFile.Transfer(Tools_GUIMenu_Root::s_uOverlayMinimised);
		xFile.Transfer(Tools_GUIMenu_PreviewOverlay::s_uOverlayMinimised);
		xFile.Transfer(Tools_GUIMenu_AnimEdit::s_uOverlayMinimised);
	}

	for(int i=0;i<12;i++)
	{
		xFile.Transfer(m_auHotkeyedSelection[i]);
	}

	xFile.Transfer(s_uBackgroundColour);

	xFile.Transfer(s_bSnapToGrid);
	xFile.Transfer(s_bResizeChildrenWithParent);
	xFile.Transfer(s_bRepositionChildrenWithinParent);
	xFile.Transfer(s_fRepositionChildren_CornerWeighting);
	xFile.Transfer(s_fRepositionChildren_CentreWeighting);
	xFile.Transfer(s_fRepositionChildren_StickTogetherWeighting);
	xFile.Transfer(s_bRenderOutlineForHiddenItems);
	xFile.Transfer(s_bAllowMouseSelectionOfHiddenItems);


}

static const Tools_GUIMenu_Root* GetFile( const Tools_GUIMenu_Root* pxItem )
{
	while( pxItem != NULL )
	{
		if( pxItem->IsFile() )
		{
			break;
		}
		pxItem = pxItem->GetParent();
	}
	return( pxItem );
}
static const Tools_GUIMenu_Root* GetMenu( const Tools_GUIMenu_Root* pxItem )
{
	while( pxItem != NULL )
	{
		if( pxItem->IsMenu() )
		{
			break;
		}
		pxItem = pxItem->GetParent();
	}
	return( pxItem );
}

void Tools_Mode_GUIMenu::Update()
{

	m_xUndo.MarkUndoStartIfEndOfBatch();

	if(m_bStyleOverlayOpen && !Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_GUIMENU_STYLE))
	{
		//Style overlay has been closed using the 'X'
		//Toggle the overlay to clear other stuff up.
		ToggleStyleOverlay();
	}

	if(s_bMenuPreview)
	{
		MenuPreview_Update();
	}
	else
	{
#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
		Asura_VideoToTexture_System::Update();
#endif
		Asura_GUIMenu_FMV_System::Update();

		if(m_bTreeSelectionChanged)
		{
			m_bTreeSelectionChanged = false;
			SetSelectionFromTree();
			CancelPromptedSelection();
		}

		Tools_GUIMenu_AnimEdit::Update();

		//update all menu items on the current menu
		if(m_pxToolsMenu)
		{
			m_pxToolsMenu->UpdateRecursive();
		}

		//Check for any styles that have been modified
		HandleStyleUpdates();

		if(m_bStyleTypeChanged)
		{
 			
			UpdateStyleList();			
			m_bStyleTypeChanged = false;
		}

		HandleMouse();
		HandleKeyboard();

		const Tools_GUIMenu_Root* const pxSelected = GetSelectedItem();
		Asura_String xString;
		if( s_pxListTextModifiersCurrentlySelectedMenu != NULL )
		{
			xString = "Menu:";
			const Tools_GUIMenu_Root* const pxMenu = GetMenu( pxSelected );
			if( pxMenu != NULL )
			{
				xString += " ";
				xString += pxMenu->GetName();
			}
			s_pxListTextModifiersCurrentlySelectedMenu->UpdateLabel( xString.GetString() );
		}
		if( s_pxListTextModifiersCurrentlySelectedFile != NULL )
		{
			xString = "File:";
			const Tools_GUIMenu_Root* const pxFile = GetFile( pxSelected );
			if( pxFile != NULL )
			{
				xString += " ";
				xString += pxFile->GetName();
			}
			s_pxListTextModifiersCurrentlySelectedFile->UpdateLabel( xString.GetString() );
		}
	}

	//Check for the screen being resized , and resize the main overlay to match
	int iScreenHeight = Asura_Viewport::GetHeight();
	if(iScreenHeight != m_iPreviousScreenHeight)
	{
		m_iPreviousScreenHeight = iScreenHeight;
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_GUIMENU);
		CreateOverlay();
	}
	
}

void Tools_Mode_GUIMenu::Render2D()
{
	if(s_bMenuPreview)
	{
		MenuPreview_Render();
		return;
	}	
	
	m_pxRoot->ResizeScaledWorldBoundsRecursive();

	if (m_pxToolsMenu && m_pxToolsMenu->GetAsuraMenu())
	{
		m_pxToolsMenu->GetAsuraMenu()->Resize(m_pxToolsMenu->GetAsuraMenu()->GetUnscaledLocalBounds());
	}

	//draw a black background
	RenderBackdrop();

	DWORD uMagFilter;
	if(s_bDisableBilinearFiltering)
	{
		//disable filtering while rendering the menu items
		Asura_Direct3D::D3DDevice->GetSamplerState( 0, D3DSAMP_MAGFILTER, &uMagFilter );
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_NONE );
	}

	//cause the elements to be rendered
	if(s_bClipToScreen) EnableClipping();
	if(m_pxToolsMenu && !m_bIsolateDisplay) m_pxToolsMenu->RenderRecursive();
	Asura_Render::FlushDisplay();
	DisableClipping();

	if(s_bDisableBilinearFiltering)
	{
		//restore original filter setting
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, uMagFilter );
	}

	// Output FMV system debug info
	//Asura_GUIMenu_FMV_System::DebugOutput();

	//test
	//const float fFrameRate = Asura_Timers::GetFrameRate();
	//Asura_Text_Output::Print(3,1,0xff00ff00,0,"%4.1f", fFrameRate);
}

void Tools_Mode_GUIMenu::RenderUI()
{
	if(s_bMenuPreview)
	{
		return;
	}

	if(s_bShowGrid) RenderGrid();

	//render dim outlines of all menu bits
	if(m_pxToolsMenu && !m_bIsolateDisplay) m_pxToolsMenu->RenderOutlineRecursive(0xff3f3f3f);

	if(s_bShowSafeArea) RenderSafeArea();
	if(s_bShowScreenBorder) RenderScreenBorder();

	//outline for the selected item
	if(m_pxSelectedItem)
	{
		if( m_bIsolateDisplay)
		{
			if(s_bClipToScreen) EnableClipping();
			m_pxSelectedItem->RenderRecursive();
			Asura_Render::FlushDisplay();
			DisableClipping();
		}
		m_pxSelectedItem->RenderOutline(0xffffffff);
		m_pxSelectedItem->RenderEditHandles();
	}

	//pulsating outline for the item under the cursor
	Tools_GUIMenu_Root* pxCursorItem = GetItemUnderCursor();
	if(pxCursorItem)
	{
		float fAlpha = Asura_Maths::Sine( Asura_Timers::GetLocalUnscaledTime() * 2.0f );
		fAlpha = 255.0f * fAlpha * fAlpha;
		unsigned int uColour = 0x00ffffff + (Asura_Maths::FloatToInt( fAlpha )<<24);

		pxCursorItem->RenderOutline(uColour);
	}

	if(m_pxSelectedItem) m_pxSelectedItem->RenderUI();

	Tools_GUIMenu_AnimEdit::Render();

	Asura_Render::FlushDisplay();

	if(m_pxPickItem)
	{
		float fPrintX = Asura_Viewport::GetWidth()/2.0f;
		float fPrintY = Asura_Viewport::GetHeight()/32.0f;
		Asura_Text_Output::ConvertPixelsToPrintCoords(fPrintX,fPrintY);

		Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"%s",m_pxPickItem->GetPrompt());
		Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Press L to select from a list");
		Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Press escape to cancel");
	}
	else if(m_bItemPlacement)
	{
		float fPrintX = Asura_Viewport::GetWidth()/2.0f;
		float fPrintY = Asura_Viewport::GetHeight()/32.0f;
		Asura_Text_Output::ConvertPixelsToPrintCoords(fPrintX,fPrintY);

		if(m_iPlacementType == TOOLS_GUIMENU_PASTE)
		{
			Asura_Assert(m_pxPlacementSource,"");
			Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Click to paste copy of %s",m_pxPlacementSource->GetName());
		}
		else if(m_iPlacementType == TOOLS_GUIMENU_STYLE)
		{
			Asura_Assert(m_pxPlacementSource,"");
			Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Click to paste instance of %s",m_pxPlacementSource->GetName());
		}
		else if(m_iPlacementType == TOOLS_GUIMENU_IMPORTTEXT || m_iPlacementType == TOOLS_GUIMENU_IMPORTALLTEXT)
		{
			Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Click to choose the position of the imported text");
		}
		else
		{
			const char* szItemName = GetMenuItemTypeName(m_iPlacementType);
			Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Click to place %s",szItemName);
		}
		Asura_Text_Output::Print(fPrintX,fPrintY++,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Press escape to cancel");
		
	}

}

void Tools_Mode_GUIMenu::RenderBackdrop()
{
	//draw a black background
	float fWidth = (float)Asura_Viewport::GetWidth();
	float fHeight = (float)Asura_Viewport::GetHeight();
	//first draw a big black poly over everything that has taken place so far...
	Asura_Render::Overlay.SetRectXY(0,0,fWidth,fHeight);

	Asura_Render::Overlay.SetRectUV(0,0,1,1);
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Colour=s_uBackgroundColour|0xff000000;
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();
}

void Tools_Mode_GUIMenu::RenderGrid()
{
	int iWidth = Asura_Viewport::GetWidth();	
	int iHeight = Asura_Viewport::GetHeight();

	//get the top left of the screen in menu coords
	Asura_Vector_2 xTopLeft = ConvertToMenuCoords(Asura_Vector_2(0.0f, 0.0f));
	//and get the nearest grid point
	xTopLeft.x = SnapX(xTopLeft.x, true);
	xTopLeft.y = SnapY(xTopLeft.y, true);

	//draw vertical lines
	for(Asura_Vector_2 xPos = xTopLeft; ; xPos.x += (float)s_iGridSize)
	{
		Asura_Vector_2 xScreenPos = ConvertFromMenuCoords(xPos);
		int iX = Asura_Maths::FloatToInt_Near(xScreenPos.x);
		if(iX<0) continue;
		if(iX>=iWidth) break;

		Asura_Render::Render2DLine((float)iX, 0.0f, (float)iX, (float) iHeight, s_uGridColour);
	}

	//draw horizontal lines
	for(Asura_Vector_2 xPos = xTopLeft; ; xPos.y += (float)s_iGridSize)
	{
		Asura_Vector_2 xScreenPos = ConvertFromMenuCoords(xPos);
		int iY = Asura_Maths::FloatToInt_Near(xScreenPos.y);
		if(iY<0) continue;
		if(iY>=iHeight) break;

		Asura_Render::Render2DLine(0.0f, (float)iY, (float) iWidth, (float)iY, s_uGridColour);
	}

	Asura_Render::FlushDisplay();
}

Asura_Vector_2& Tools_Mode_GUIMenu::GetVirtualSize()
{
	if (s_xVirtualSize.x > 0.0f
		&& s_xVirtualSize.y > 0.0f)
	{
		return s_xVirtualSize;
	}
	else
	{
		return s_xScreenSize;
	}
}

void Tools_Mode_GUIMenu::RenderScreenBorder()
{
	Asura_Vector_2 xSize(s_xScreenSize);
	if (IsPreview() && Tools_GUIMenu_PreviewOverlay::s_bUseVirtualResolution)
	{
		xSize = GetVirtualSize();
	}
	Asura_Bounding_Rectangle xRect(0.0f,0.0f,xSize.x,xSize.y);
	if(IsPreview())
	{
		if(Tools_GUIMenu_PreviewOverlay::s_bCentreScreen)
		{
			xRect.Displace(Asura_Vector_2(Asura_Viewport::CentreX - (xSize.x * 0.5f), Asura_Viewport::CentreY - (xSize.y * 0.5f)));
		}
	}
	else
	{
		//convert the screen area from menu coordinates
		xRect.Displace(GetScreenBorderOffsetForCurrentMenu());
		xRect = ConvertFromMenuCoords(xRect);
	}

	//draw the rectangle
	Asura_Render::Render2DLine(xRect.MinX,xRect.MinY,xRect.MaxX,xRect.MinY,s_uScreenBorderColour);
	Asura_Render::Render2DLine(xRect.MaxX,xRect.MinY,xRect.MaxX,xRect.MaxY,s_uScreenBorderColour);
	Asura_Render::Render2DLine(xRect.MaxX,xRect.MaxY,xRect.MinX,xRect.MaxY,s_uScreenBorderColour);
	Asura_Render::Render2DLine(xRect.MinX,xRect.MaxY,xRect.MinX,xRect.MinY,s_uScreenBorderColour);

	Asura_Render::FlushDisplay();

}

void Tools_Mode_GUIMenu::RenderSafeArea()
{
	Asura_Bounding_Rectangle xRect = s_xSafeArea;

	Asura_Vector_2 xSize(s_xScreenSize);
	if (IsPreview() && Tools_GUIMenu_PreviewOverlay::s_bUseVirtualResolution)
	{
		xSize = GetVirtualSize();
		xRect.MinX *= GetVirtualSize().x / s_xScreenSize.x;
		xRect.MaxX *= GetVirtualSize().x / s_xScreenSize.x;
		xRect.MinY *= GetVirtualSize().y / s_xScreenSize.y;
		xRect.MaxY *= GetVirtualSize().y / s_xScreenSize.y;
	}

	if(IsPreview())
	{
		if(Tools_GUIMenu_PreviewOverlay::s_bCentreScreen)
		{
			xRect.Displace(Asura_Vector_2(Asura_Viewport::CentreX - (xSize.x * 0.5f), Asura_Viewport::CentreY - (xSize.y * 0.5f)));
		}
	}
	else
	{
		//convert the screen area from menu coordinates
		xRect.Displace(GetScreenBorderOffsetForCurrentMenu());
		xRect = ConvertFromMenuCoords(xRect);
	}


	//draw the rectangle
	Asura_Render::Render2DLine(xRect.MinX,xRect.MinY,xRect.MaxX,xRect.MinY,s_uSafeAreaColour);
	Asura_Render::Render2DLine(xRect.MaxX,xRect.MinY,xRect.MaxX,xRect.MaxY,s_uSafeAreaColour);
	Asura_Render::Render2DLine(xRect.MaxX,xRect.MaxY,xRect.MinX,xRect.MaxY,s_uSafeAreaColour);
	Asura_Render::Render2DLine(xRect.MinX,xRect.MaxY,xRect.MinX,xRect.MinY,s_uSafeAreaColour);

	Asura_Render::FlushDisplay();

}

void Tools_Mode_GUIMenu::CreateOverlay()
{
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Menu Mode",false,&m_xOverlayPosition,&m_uOverlayMinimised);
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	
	Asura_Dialog* pxDialog;

	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset, 0, 0);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxOverlay->SetCurrentParentDialog(pxTabsDialog);

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	//Add most of the controls to a main tab
	pxTabsDialog->AddTab("Main");

	pxDialog = new Asura_Function_Dialog("Create File",xOffset,CreateFile,(int)this);
	pxDialog->SetHelpText("Creates a new global file to contain menus.");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Function_Dialog("Create Menu",xOffset,CreateMenu,(int)this);
	pxDialog->SetHelpText("Creates a new menu in the most recently selected file.");
	pxDialog->SetIsEnabledFunction(CanCreateMenu,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Preview Menu (F11)",xOffset,MenuPreview,(int)this);
	pxDialog->SetHelpText("Test the current menu.");
	pxDialog->SetIsEnabledFunction(CanPreviewMenu,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Find By ID",xOffset,FindItemByIDSelectedByUser,(int)this);
	pxDialog->SetHelpText("Find a menu item by its hash ID");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Find By Texture",xOffset,FindByTexture,(int)this);
	pxDialog->SetHelpText("Find all menu items using a given texture.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Find Menu",xOffset,FindMenuByName,(int)this);
	pxDialog->SetHelpText("Find and select a menu by selecting from a list.");
	pxOverlay->AddDialog(pxDialog);


	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Set Font",xOffset, Tools_GUIMenu_Element_Text::SetFontOnSelection);
	pxDialog->SetHelpText("Applies a chosen font to the selected menu item and all of its children.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Verify",xOffset,ManualVerify,(int)this);
	pxDialog->SetHelpText("Verify the GUI Menu data and display a list of errors.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	//drop down for selecting the language
	BuildLanguageList();
	pxDialog = new Asura_DropDown_List_Dialog(xOffset,m_pszLanguageNames, &m_iLanguageIndex, 12, m_iLanguageListSize, ReloadText,	0, "Language"); 
	pxDialog->SetHelpText("Set the language to use for previewing the menu text");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog( "Reload text", xOffset, ReloadText );
	pxDialog->SetHelpText( "Reloads the menu text." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();


	const int iRadioButtonGroupID = 1;
	pxDialog = new Asura_Tickbox_Dialog("Resize Children",xOffset,&s_bResizeChildrenWithParent);
	pxDialog->SetRadioButtonGroup(iRadioButtonGroupID,false);
	pxDialog->SetHelpText("If ticked , then changing the width/height of a menu item will also cause any child menu items to be scaled.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Reposition Children",xOffset,&s_bRepositionChildrenWithinParent);
	pxDialog->SetRadioButtonGroup(iRadioButtonGroupID,false);
	pxDialog->SetHelpText("If ticked , then changing the width/height of a menu item will result in the child items being moved without being resized in an attempt to keep corner items in the corner etc.\n");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Isolate Selection",xOffset,&m_bIsolateDisplay);
	pxDialog->SetHelpText("If ticked, this will make sure that only the selected menu item and it's child objects will be displayed.\n");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	//determine the maximum height the treeview can be
	//The amount that can be used will need to be adjusted if the overlay gets stuff
	//added/removed from it.
	//RAR 05/09/2007
	float fMaxPrintY = Asura_Viewport::GetHeight() / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
	const float fTreeHeight = Asura_Maths::Max(10.0f,fMaxPrintY - 45.0f);


	pxDialog = new Asura_Tree_Dialog(xOffset,m_pxTree,30.0f,fTreeHeight,&m_bTreeSelectionChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;
	pxDialog->m_iControlID = g_uMenuItemTreeID;

	u_int uWidgetCount = 0;
	//Add buttons for creating all the widget types
	for(int i=0;i<g_iNumWidgetTypes;i++)
	{
		int iType = g_aiWidgetTypes[i];

		// Don't allow creation of textlistboxes as they are being phased out.
		// Don't add the textlistbox button on the overlay
		if ( iType != Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX)
		{
			if(uWidgetCount & 1)
			{
				//but button in second column
				xOffset.x = 11.0f;
			}
			else
			{
				//move onto next line and put button in first column
				xOffset.x = 1.0f;
				if( uWidgetCount > 0 )
				{
					xOffset.y+=Asura_Dialog::DefaultSpacing();
				}
			}

			pxDialog = new Asura_Function_Dialog(GetMenuItemTypeName(iType),xOffset,CreateWidget,iType);
			pxDialog->SetHelpText("Create a new widget");
			pxDialog->SetIsEnabledFunction(CanCreateWidget);
			pxOverlay->AddDialog(pxDialog);

			uWidgetCount++;
		}
	}

 	xOffset.x = 1.0f;
 	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Image",xOffset,CreateWidget,Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE);
	pxDialog->SetHelpText("Create a static image element");
	pxDialog->SetIsEnabledFunction(CanCreateWidget);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = 11.0f;
	pxDialog = new Asura_Function_Dialog("Static Text",xOffset,CreateWidget,Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT);
	pxDialog->SetHelpText("Create a static text element");
	pxDialog->SetIsEnabledFunction(CanCreateWidget);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;

	pxDialog = new Asura_Function_Dialog("Animation",xOffset,CreateWidget,Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_ANIMATION);
	pxDialog->SetHelpText("Create an animation element");
	pxDialog->SetIsEnabledFunction(CanCreateWidget);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = 11.0f;
	pxDialog = new Asura_Function_Dialog("Status Text",xOffset,CreateWidget,Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_STATUSTEXT);
	pxDialog->SetHelpText("Create a static text element");
	pxDialog->SetIsEnabledFunction(CanCreateWidget);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;

#ifdef ASURA_GUIMENU_SUPPORT_FMV_ELEMENTS 
	// If there is no FMV support then don't allow the create of FMV elements either
	pxDialog = new Asura_Function_Dialog("FMV",xOffset,CreateWidget,Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_FMV);
	pxDialog->SetHelpText("Create an FMV element");
	pxDialog->SetIsEnabledFunction(CanCreateWidget);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
#endif

	xOffset.y += Asura_Dialog::LargeSpacing() - Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Copy",xOffset,Copy,(int)this);
	pxDialog->SetHelpText("Designates the selected menu item (and its children) as the item to paste for subsequent paste operations.\nShortcut : Ctrl+C");
	pxDialog->SetIsEnabledFunction(CanCopy,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Paste",xOffset,Paste,(int)this);
	pxDialog->SetHelpText("Pastes the most recent menu item that has been chosen by 'Copy'\nShortcut : Ctrl+V.");
	pxDialog->SetIsEnabledFunction(CanPaste,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Paste Over",xOffset,PasteOver,(int)this);
	pxDialog->SetHelpText("Pastes the most recent menu item that has been chosen by 'Copy' over the selected item.\nThis will replace any existing child items in the selected item.\nShortcut : Ctrl+Shift+V.");
	pxDialog->SetIsEnabledFunction(CanPasteOver,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Audio",xOffset,Tools_GUIMenu_Audio_Overlay::EditAudio);
	pxDialog->SetHelpText("Edit GUIMenu Generic Audio");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Texture Flags",xOffset, EditTextureFlags, (int)this);
	pxDialog->SetHelpText("Edit the texture flags for textures used by the menus.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	//end of main tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//display tab
	pxTabsDialog->AddTab("Display");
	xOffset.y = fTabsStartY;

	pxDialog = new Asura_Dialog("Background",xOffset);
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Tools_ARGBColourPicker_Dialog("Background Colour",xOffset,Asura_Vector_2(1.5f,1.0f),&s_uBackgroundColour,NULL);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;


	pxDialog = new Asura_Numerical_Dialog("Grid Size",xOffset,&s_iGridSize,1,9999,1,"%d");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Zoom",xOffset,&s_fZoom,0.1f,99.0f,1.0f,"%.3f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Menu Offset X",xOffset,&s_iMenuOffsetX,-9999,9999,1,"%d");
	pxDialog->SetHelpText("Determines the screen position of the top left corner of the menu (for tools rendering)");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Menu Offset Y",xOffset,&s_iMenuOffsetY,-9999,9999,1,"%d");
	pxDialog->SetHelpText("Determines the screen position of the top left corner of the menu (for tools rendering)");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Grid Origin X",xOffset,&s_iGridOriginX,-9999,9999,1,"%d");
	pxDialog->SetHelpText("Determines one of the grid points in menu coordinates. All others will be a multiple of the grid size away from this.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Grid Origin Y",xOffset,&s_iGridOriginY,-9999,9999,1,"%d");
	pxDialog->SetHelpText("Determines one of the grid points in menu coordinates. All others will be a multiple of the grid size away from this.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Grid",xOffset,&s_bShowGrid);
	pxDialog->SetHelpText("Determines whether the grid should be shown");
	pxOverlay->AddDialog(pxDialog);
	

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Tools_ARGBColourPicker_Dialog("Grid Colour",xOffset,Asura_Vector_2(1.5f,1.0f),&s_uGridColour,NULL);
	((Tools_ARGBColourPicker_Dialog*) pxDialog)->EnableAlphaEditing(true);
	pxDialog->SetHelpText("The colour for the grid.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;

	pxDialog = new Asura_Tickbox_Dialog("Snap To Grid",xOffset,&s_bSnapToGrid);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Hide Siblings",xOffset,&s_bHideSiblings);
	pxDialog->SetHelpText("If ticked then the rendering of any sibling elements of the selection will be disabled.\nThis is to help with seeing the selected item when dealing with a stack of elements each of which is rendered in a different state");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Outline For Hidden Items",xOffset,&s_bRenderOutlineForHiddenItems);
	pxDialog->SetHelpText("If ticked then the outline of hidden menu items will still be shown.\nThe outline of the selected item is shown regardless");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Allow Selection Of Hidden Items",xOffset,&s_bAllowMouseSelectionOfHiddenItems);
	pxDialog->SetHelpText("Determines whether a hidden menu item can be selected by clicking on it.\nMenu items can always be selected from the tree view regardless of this setting");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();



	pxDialog = new Asura_Tickbox_Dialog("Disable Bilinear Filtering",xOffset,&s_bDisableBilinearFiltering);
	pxDialog->SetHelpText("Disables bilinear filtering while rendering the menu elements in tools.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if(s_iScreenSettingPreset < 0)
	{
		s_iScreenSettingPreset = TOOLS_GUIMENU_SCREENSETTINGS_CURRENTRES;
		UpdateScreenSettingsFromPreset();
	}

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_apszScreenSettingNames, &s_iScreenSettingPreset, 12, TOOLS_GUIMENU_NUM_SCREENSETTINGS, UpdateScreenSettingsFromPreset, 0, "Presets");
	pxDialog->SetHelpText("Used for selecting from some default values for the screen size and safe area.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	AddPlatformDropDown(pxOverlay, xOffset);

	pxDialog = new Asura_Tickbox_Dialog("Screen Border",xOffset, &s_bShowScreenBorder);
	pxDialog->SetHelpText("Enables rendering of a rectangle representing the extents of the screen.");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Tools_ARGBColourPicker_Dialog("Screen Border Colour",xOffset,Asura_Vector_2(1.5f,1.0f),&s_uScreenBorderColour,NULL);
	((Tools_ARGBColourPicker_Dialog*) pxDialog)->EnableAlphaEditing(true);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;
	
	pxDialog = new Asura_Tickbox_Dialog("Clip To Screen",xOffset, &s_bClipToScreen);
	pxDialog->SetHelpText("Causes anything outside of the specified screen area to be clipped out");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
//	pxDialog = new Asura_Tickbox_Dialog("Recentre For Preview",xOffset, &s_bCentreScreenForPreview);
//	pxDialog->SetHelpText("Forces the menu to be recentred when previewing , regardless of whether it has the flag");
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.y += Asura_Dialog::DefaultSpacing();

	
	pxDialog = new Asura_Numerical_Dialog("Screen Width",xOffset,&s_xScreenSize.x,0.0f,9999.0f,1.0f,"%.0f");
	pxDialog->SetHelpText("Extents of the screen to use for the border rectangle and for clipping");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Screen Height",xOffset,&s_xScreenSize.y,0.0f,9999.0f,1.0f,"%.0f");
	pxDialog->SetHelpText("Extents of the screen to use for the border rectangle and for clipping");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Numerical_Dialog("Virtual Screen Width",xOffset,&s_xVirtualSize.x,0.0f,9999.0f,1.0f,"%.0f");
	pxDialog->SetHelpText("Extents of the virtual screen to use for the border rectangle and for clipping. ignored if zero.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Virtual Screen Height",xOffset,&s_xVirtualSize.y,0.0f,9999.0f,1.0f,"%.0f");
	pxDialog->SetHelpText("Extents of the virtual screen to use for the border rectangle and for clipping. ignored if zero.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Safe Area",xOffset, &s_bShowSafeArea);
	pxDialog->SetHelpText("Enables rendering of a rectangle representing the safe area on the screen.");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Tools_ARGBColourPicker_Dialog("Safe Area Colour",xOffset,Asura_Vector_2(1.5f,1.0f),&s_uSafeAreaColour,NULL);
	((Tools_ARGBColourPicker_Dialog*) pxDialog)->EnableAlphaEditing(true);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;

	
	pxDialog = new Asura_Numerical_Dialog("Safe Area Min X",xOffset,&s_xSafeArea.MinX,0.0f,9999.0f,1.0f,"%.0f");
	pxDialog->SetHelpText("The area that should be considered to be the safe area");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Safe Area Max X",xOffset,&s_xSafeArea.MaxX,0.0f,9999.0f,1.0f,"%.0f");
	pxDialog->SetHelpText("The area that should be considered to be the safe area");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Safe Area Min Y",xOffset,&s_xSafeArea.MinY,0.0f,9999.0f,1.0f,"%.0f");
	pxDialog->SetHelpText("The area that should be considered to be the safe area");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Safe Area Max Y",xOffset,&s_xSafeArea.MaxY,0.0f,9999.0f,1.0f,"%.0f");
	pxDialog->SetHelpText("The area that should be considered to be the safe area");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//end of display tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//the tab with no good name
	pxTabsDialog->AddTab("Other");
	xOffset.y = fTabsStartY;

	pxDialog = new Asura_Dialog("Reposition Children Tweakage :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Corner Weight",xOffset,&s_fRepositionChildren_CornerWeighting,0.0f,999.0f,1.0f,"%.2f");
	pxDialog->SetHelpText("Determines how likely menu items are to stick to the corners and edges when using the reposition children option.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Centre Weight",xOffset,&s_fRepositionChildren_CentreWeighting,0.0f,999.0f,1.0f,"%.2f");
	pxDialog->SetHelpText("Determines how likely menu items are to stick to the centre when using the reposition children option.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Stick Together Weight",xOffset,&s_fRepositionChildren_StickTogetherWeighting,0.0f,999.0f,1.0f,"%.2f");
	pxDialog->SetHelpText("Determines how likely menu items are to stick to nearby elements when using the resposition children option.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Import Text",xOffset,ImportText,TOOLS_GUIMENU_IMPORTTEXT);
	pxDialog->SetHelpText("Create a group of text elements with a chosen stlye.\n The elements contain text picked from file that you browse for.\n You can choose what text to insert.");
	pxDialog->SetIsEnabledFunction(CanCreateWidget);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = 11.0f;
	pxDialog = new Asura_Function_Dialog("Import All Text",xOffset,ImportText,TOOLS_GUIMENU_IMPORTALLTEXT);
	pxDialog->SetHelpText("Create a group of text elements, each with a chosen style depending on thier text ID.\n This inserts all the text from the chosen file.\n It's main use is for credit screens");
	pxDialog->SetIsEnabledFunction(CanCreateWidget);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;

	const float fXOffset = 1.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("List Text Modifiers:",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	xOffset.x += fXOffset;
	{
		pxDialog = new Asura_Function_Dialog( "All", xOffset, ListTextModifiers, eLIST_ALL_TEXT_MODIFIERS );
		pxDialog->SetHelpText( "Lists all text items that have a text modifier." );
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog( "Menu: ", xOffset, ListTextModifiers, eLIST_TEXT_MODIFIERS_CURRENTLY_SELECTED_MENU );
		pxDialog->SetHelpText( "Lists all text items that have a text modifier, on the currently selected menu." );
		pxDialog->SetIsEnabledFunction( IsEnabledForCurrentlySelectedMenu );
		pxOverlay->AddDialog( pxDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();
		s_pxListTextModifiersCurrentlySelectedMenu = (Asura_Function_Dialog*)pxDialog;

		pxDialog = new Asura_Function_Dialog( "File: ", xOffset, ListTextModifiers, eLIST_TEXT_MODIFIERS_CURRENTLY_SELECTED_FILE );
		pxDialog->SetHelpText( "Lists all text items that have a text modifier, on the currently selected file." );
		pxOverlay->AddDialog( pxDialog );
		pxDialog->SetIsEnabledFunction( IsEnabledForCurrentlySelectedFile );
		xOffset.y += Asura_Dialog::DefaultSpacing();
		s_pxListTextModifiersCurrentlySelectedFile = (Asura_Function_Dialog*)pxDialog;
	}
	xOffset.x -= fXOffset;

	//end of  tab with no good name
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;



	//end of tabs
	xOffset.y = fTabsEndY + 1.0f;
	pxOverlay->SetCurrentParentDialog(0);

 	static u_int s_uSelectedTab = 0;
 	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);


	pxOverlay->m_uID = TOOLS_OVERLAY_GUIMENU;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
	
}

void Tools_Mode_GUIMenu::AddPlatformDropDown(Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog = new Asura_DropDown_List_Dialog(xOffset, s_pszPlatformNames, &s_iPreviewPlatform, TOOLS_PLATFORM_NUM_PLATFORMS, TOOLS_PLATFORM_NUM_PLATFORMS, NULL, 0, "Platform");
	pxDialog->SetHelpText("This is checked against the platform tickboxes on the menu items to determine which to show when previewing menus.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
}

Asura_Vector_2 Tools_Mode_GUIMenu::GetCursorPos(bool bInMenuCoords)
{
	if(bInMenuCoords) return GetCursorPosInMenuCoords();

	int iX,iY;
	Asura_Cursor::GetScreenSpaceCoords(iX,iY);
	return Asura_Vector_2((float)iX,(float)iY);
	
}

Asura_Vector_2 Tools_Mode_GUIMenu::GetCursorPosInMenuCoords()
{
	//get the cursor position
	int iX,iY;
	Asura_Cursor::GetScreenSpaceCoords(iX,iY);

	//convert to menu coordinates
	return ConvertToMenuCoords( Asura_Vector_2( static_cast<float>(iX),static_cast<float>(iY) ) );
}

Tools_GUIMenu_Root* Tools_Mode_GUIMenu::GetItemUnderCursor()
{
	if(Asura_Interface::UsingMenusOrOverlays()) return 0;

	if(!m_pxToolsMenu) return 0;

	Asura_Vector_2 xPos = GetCursorPosInMenuCoords();

	bool bPreferWidget = !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL);
	return m_pxToolsMenu->GetItemFromPosition(xPos,m_pxPickItem,bPreferWidget);
}

Asura_Vector_2 Tools_Mode_GUIMenu::ConvertToMenuCoords(const Asura_Vector_2& xPos)
{
	return Asura_Vector_2(xPos.x/s_fZoom - s_iMenuOffsetX,xPos.y/s_fZoom - s_iMenuOffsetY);
}

Asura_Vector_2 Tools_Mode_GUIMenu::ConvertFromMenuCoords(const Asura_Vector_2& xPos)
{
	return Asura_Vector_2((xPos.x + s_iMenuOffsetX)*s_fZoom, (xPos.y + s_iMenuOffsetY)*s_fZoom);
}

Asura_Bounding_Rectangle Tools_Mode_GUIMenu::ConvertFromMenuCoords(const Asura_Bounding_Rectangle& xRect)
{
	return Asura_Bounding_Rectangle(
		(xRect.MinX + s_iMenuOffsetX)*s_fZoom,
		(xRect.MinY + s_iMenuOffsetY)*s_fZoom,
		(xRect.MaxX + s_iMenuOffsetX)*s_fZoom,
		(xRect.MaxY + s_iMenuOffsetY)*s_fZoom);
}


void Tools_Mode_GUIMenu::HandleMouse()
{

	if(m_pxPickItem)
	{
		if(!Asura_Interface::UsingMenusOrOverlays())
		{
			if(DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
			{
				m_pxPickItem->OnFinish(GetItemUnderCursor());
				CancelPromptedSelection();
			}
		}
		if(DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
		{
			CancelPromptedSelection();
		}
	}
	else if(m_bItemPlacement)
	{
		if(!Asura_Interface::UsingMenusOrOverlays())
		{
			if(DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
			{
				CreateItemAtCursor(m_pxPlacementParent,m_iPlacementType);
				CancelMenuItemPlacement();
			}
		}
		if(DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
		{
			CancelMenuItemPlacement();
		}

	}
	else if(m_bDrag)
	{
		//keep all undoable actions in one batch while dragging
		m_xUndo.KeepUndoBatchOpenThisFrame();
		
		//update dragging
		bool bDragItem = !(m_uDragFlags & TOOLS_MENUS_DRAG_MENUOFFSET);
		bool bInMenuCoords = bDragItem;
		int iDragButton = bDragItem ? ASURA_KEYID_MOUSEBUTTON1 : ASURA_KEYID_MOUSEBUTTON3;
		
		if(Asura_UserInput::KeyPressed(iDragButton))
		{
			Asura_Vector_2 xCursorPos = GetCursorPos(bInMenuCoords);
			Asura_Vector_2 xOffset = xCursorPos - m_xDragStart;

			//If shift is held down then restrict to one axis
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
			{
				if(m_uDragFlags & TOOLS_MENUS_DRAG_RESTRICTX)
				{
					xOffset.y = 0.0f;
				}
				else if(m_uDragFlags & TOOLS_MENUS_DRAG_RESTRICTY)
				{
					xOffset.x = 0.0f;
				}
				else
				{
					//axis for axis restrict not locked in yet , so go for the one with largest movement
					float fScale = bInMenuCoords ? GetZoom() : 1.0f;

					if(Asura_Maths::Abs(xOffset.x) > Asura_Maths::Abs(xOffset.y))
					{
						xOffset.y = 0.0f;
						if(Asura_Maths::Abs(xOffset.x)*fScale > 8.0f )
						{
							m_uDragFlags |= TOOLS_MENUS_DRAG_RESTRICTX;
						}
					}
					else
					{
						xOffset.x = 0.0f;
						if(Asura_Maths::Abs(xOffset.y)*fScale > 8.0f )
						{
							m_uDragFlags |= TOOLS_MENUS_DRAG_RESTRICTY;
						}
					}
				}
			}
			else
			{
				m_uDragFlags &=~ (TOOLS_MENUS_DRAG_RESTRICTX|TOOLS_MENUS_DRAG_RESTRICTY);
			}

			if(bDragItem)
			{
				if(m_uDragFlags & TOOLS_MENUS_DRAG_PENDING)
				{
					//See if the mouse has moved enough to confirm that dragging is intended
					if((Asura_Maths::Abs(xOffset.x) > 5.0f) || (Asura_Maths::Abs(xOffset.y) > 5.0f)) 
					{
						m_uDragFlags &=~ TOOLS_MENUS_DRAG_PENDING;
						m_uDragFlags |= TOOLS_MENUS_DRAG_CONFIRMED;
					}
				}
				if((m_uDragFlags & TOOLS_MENUS_DRAG_CONFIRMED) && m_pxSelectedItem)
				{
					m_pxSelectedItem->UpdateDrag(xOffset,m_uDragFlags);
				}
			}
			else if(m_uDragFlags & TOOLS_MENUS_DRAG_MENUOFFSET)
			{
				if(m_uDragFlags & TOOLS_MENUS_DRAG_MENUOFFSET_PENDING)
				{
					//See if the mouse has moved enough to confirm that dragging is intended
					if((Asura_Maths::Abs(xOffset.x) > 5.0f) || (Asura_Maths::Abs(xOffset.y) > 5.0f)) 
					{
						m_uDragFlags &=~ TOOLS_MENUS_DRAG_MENUOFFSET_PENDING;
						m_uDragFlags |= TOOLS_MENUS_DRAG_MENUOFFSET_CONFIRMED;
					}
				}
				if(m_uDragFlags & TOOLS_MENUS_DRAG_MENUOFFSET_CONFIRMED)
				{
					s_iMenuOffsetX = m_iPreDragMenuOffsetX + Asura_Maths::FloatToInt_Near(xOffset.x/GetZoom());
					s_iMenuOffsetY = m_iPreDragMenuOffsetY + Asura_Maths::FloatToInt_Near(xOffset.y/GetZoom());
				}
			}

		}
		else
		{
			if(m_uDragFlags & TOOLS_MENUS_DRAG_PENDING)
			{
				//released button before it has been moved far enough to count as dragging.
				//so select the item under the cursor instead.
				//RAR 30/04/2007
				SetSelection(GetItemUnderCursor());
			}
			else
			{
				if(m_pxSelectedItem
					&& m_pxSelectedItem->GetParent())
				{
					m_pxSelectedItem->GetParent()->ResizePostChildChanges();
				}
			}
			m_bDrag = false;
			m_uDragFlags = 0;
		}
	}
	else if(!Asura_Interface::UsingMenusOrOverlays())
	{

		//left click for dragging and selection
		if(DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1))
		{
			m_bDrag = true;
			m_xDragStart = GetCursorPosInMenuCoords();
			m_uDragFlags = TOOLS_MENUS_DRAG_PENDING;

			if(m_pxSelectedItem)
			{
				//determine which extents should be dragged
				u_int uAdditionalFlags = m_pxSelectedItem->GetExtentsToDrag(m_xDragStart);

				{
					m_uDragFlags |= uAdditionalFlags;
					//get everything to remember their rectangle before dragging
					for(CLIF<Tools_GUIMenu_Root*> xLIF(Tools_GUIMenu_Root::GetList());!xLIF.done();xLIF.next())
					{
						xLIF()->StorePreDragState();
					}
				}
			}
		}
		//middle button to drag the view of the menu around
		else if(DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON3))
		{
			//also used for dragging the offset about
			m_bDrag = true;
			m_xDragStart = GetCursorPos(false);
			m_uDragFlags = TOOLS_MENUS_DRAG_MENUOFFSET_PENDING;
			m_iPreDragMenuOffsetX = s_iMenuOffsetX;
			m_iPreDragMenuOffsetY = s_iMenuOffsetY;
		}

		//Allow the mousewheel to be used for zooming in/out
		//The mousewheel always seems to be in use for a minimum of two frames at a time , so
		//limit use to every other frame.
		//RAR 19/02/2007
		static bool s_bUsedMouseWheelLastFrame = false;
		if (s_bUsedMouseWheelLastFrame)
		{
			s_bUsedMouseWheelLastFrame = false;
		}
		else if(!Asura_Overlay_System::CursorIsOverAnOverlay())
		{
			float fNewZoom = s_fZoom;
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEWHEELDOWN))
			{
				//zoom out
				if(s_fZoom > 1.0f)
				{
					fNewZoom = (float)Asura_Maths::FloatToInt_Chop(s_fZoom);
					if(s_fZoom == fNewZoom) fNewZoom -= 1.0f; 
				}
				else if(s_fZoom > 0.5f)
				{
					fNewZoom = 0.5f;
				}
				else if(s_fZoom > 0.25f)
				{
					fNewZoom = 0.125f;
				}

				s_bUsedMouseWheelLastFrame = true;
			}
			else if(Asura_UserInput::KeyPressed(ASURA_KEYID_MOUSEWHEELUP))
			{
				//zoom in.
				if(s_fZoom < 0.25f)
				{
					fNewZoom = 0.25f;
				}
				else if(s_fZoom < 0.5f)
				{
					fNewZoom = 0.5f;
				}
				else if(s_fZoom < 1.0f)
				{
					fNewZoom = 1.0f;
				}
				else
				{
					fNewZoom = 1.0f + (float)Asura_Maths::FloatToInt_Chop(s_fZoom);
				}
				s_bUsedMouseWheelLastFrame = true;
			}

			if(fNewZoom != s_fZoom)
			{
				//adjust offsets so that the cursor stays over the same spot
				Asura_Vector_2 xScreenCoords = GetCursorPos(false);
				Asura_Vector_2 xMenuCoords = GetCursorPos(true);
				s_fZoom = fNewZoom;
				if(s_fZoom <= 0.1f) s_fZoom = 0.1f;//in practice the above shouldn't allow it to go too low anyway

				s_iMenuOffsetX = Asura_Maths::FloatToInt_Near(xScreenCoords.x/s_fZoom - xMenuCoords.x);
				s_iMenuOffsetY = Asura_Maths::FloatToInt_Near(xScreenCoords.y/s_fZoom - xMenuCoords.y);
			}
		}
		
	}
}

void Tools_Mode_GUIMenu::HandleKeyboard()
{
	if(Asura_Overlay_System::OverlaysHaveKeyboardFocus()) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
 	{
		if(DebouncedKeyPressed('c'))
		{
			Copy();
		}
		if(DebouncedKeyPressed('v'))
		{
			//paste over selection if allowable , otherwise paste a new item
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
			{
				PasteOver();
			}
			else
			{
				PasteAtCursor();
			}
		}

		for(int i=0;i<12;i++)
		{
			if(DebouncedKeyPressed(ASURA_KEYID_F1 + i))
			{
				//restore previous selection
				if(m_auHotkeyedSelection[i])
				{
					SetSelectionByID(m_auHotkeyedSelection[i]);
				}
			}
		}

		if(DebouncedKeyPressed('z'))// 'ctrl+z' for  undo
		{
			Undo();			
		}
		if(DebouncedKeyPressed('y'))// 'ctrl+y' for  redo
		{
			Redo();			
		}
	
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
		if(m_pxSelectedItem && m_pxSelectedItem->GetID())
		{
			for(int i=0;i<11;i++)
			{
				if(DebouncedKeyPressed(ASURA_KEYID_F1 + i))
				{
					//save current selection
					m_auHotkeyedSelection[i] = m_pxSelectedItem->GetID();
				}
			}
		}


		if(DebouncedKeyPressed('h'))
		{
			if(m_pxSelectedItem) m_pxSelectedItem->SetHiddenRecursive(false);
		}
	}
	else
	{
		if(DebouncedKeyPressed(ASURA_KEYID_DEL))
		{
			if(m_pxSelectedItem && !m_pxSelectedItem->IsGeneratedByStyle() )
			{
				// Test if the animator tab is selected.
				// If so then potentially delete the currently selected animator.
				if ( Tools_GUIMenu_Root::IsAnimatorTabSelected() == true )
				{
					if ( m_pxSelectedItem->HasAnyAnimators() == true )
					{
						// Delete Animator
						if ( ShowYesNoOverlay("Delete Animator?\n\nWould you like to delete the selected animator?") )
						{
							m_pxSelectedItem->DeleteCurrentAnimator();
						}
					}
				}
				// Test if the modifier tab is selected.
				// If so then potentially delete the currently selected modifier.
				else if ( Tools_GUIMenu_Root::IsModifierTabSelected() == true )
				{
					if ( m_pxSelectedItem->HasAnyModifiers() == true )
					{
						// Delete selected modifier
						if ( ShowYesNoOverlay("Delete Modifier?\n\nWould you like to delete the selected modifier?") )
						{
							m_pxSelectedItem->DeleteCurrentModifier();
						}
					}
				}
				else
				{
					// If neither the Animator or Modifier tabs are selected then check for deletion of selected
					// menu item.
					if ( m_pxSelectedItem->ConfirmDeletion() )
					{
						DeleteMenuItem(m_pxSelectedItem);
					}
				}
			}
		}
		
		if(DebouncedKeyPressed('l') && m_pxPickItem)
		{
			PromptedSelection_SelectFromList();
		}

		if(DebouncedKeyPressed(ASURA_KEYID_F11))
		{
			MenuPreview();
		}

		if(DebouncedKeyPressed('1'))
		{
			PrevLanguage();
		}
		if(DebouncedKeyPressed('2'))
		{
			NextLanguage();
		}
		if(DebouncedKeyPressed('3'))
		{
			PrevScreenPreset();
		}
		if(DebouncedKeyPressed('4'))
		{
			NextScreenPreset();
		}

		if(DebouncedKeyPressed(','))// '<' for  undo
		{
			Undo();
			
		}
		if(DebouncedKeyPressed('.'))// '>' for  redo
		{
			Redo();
		}

		if(DebouncedKeyPressed('h'))
		{
			if(m_pxSelectedItem) m_pxSelectedItem->SetHiddenRecursive(true);
		}

		if(DebouncedKeyPressed('i'))
		{
			m_bIsolateDisplay = !m_bIsolateDisplay;
		}
	}
}

void Tools_Mode_GUIMenu::MenuPreview_HandleKeyboard()
{
	if(Asura_Overlay_System::OverlaysHaveKeyboardFocus()) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
 	{
	
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{

	}
	else
	{
		if(DebouncedKeyPressed('1'))
		{
			PrevLanguage();
		}
		if(DebouncedKeyPressed('2'))
		{
			NextLanguage();
		}
		if(DebouncedKeyPressed('3'))
		{
			PrevScreenPreset();
		}
		if(DebouncedKeyPressed('4'))
		{
			NextScreenPreset();
		}

		//End preview if escape is pressed
		//Or F11
		if(DebouncedKeyPressed(ASURA_KEYID_ESCAPE) || DebouncedKeyPressed(ASURA_KEYID_F11))
		{
			MenuPreview_End();
		}
	}
}


const float Tools_Mode_GUIMenu::SnapX( const float fX, bool bForceSnap )
{
	//probably want to snap to nearest pixel even if not snapping to the grid
	float fSnap = (s_bSnapToGrid || bForceSnap) ? (float)s_iGridSize : 1.0f;

	float fOffsetFromOrigin = fX - (float)s_iGridOriginX;	
	int iSnapMultiple = Asura_Maths::FloatToInt_Near(fOffsetFromOrigin/fSnap);

	return s_iGridOriginX + fSnap*iSnapMultiple;
}

const float Tools_Mode_GUIMenu::SnapY( const float fY, bool bForceSnap)
{
	//probably want to snap to nearest pixel even if not snapping to the grid
	float fSnap = (s_bSnapToGrid || bForceSnap)? (float)s_iGridSize : 1.0f;

	float fOffsetFromOrigin = fY - (float)s_iGridOriginY;	
	int iSnapMultiple = Asura_Maths::FloatToInt_Near(fOffsetFromOrigin/fSnap);

	return s_iGridOriginY + fSnap*iSnapMultiple;

}


void Tools_Mode_GUIMenu::AddToTree(Tools_GUIMenu_Root* pxItem)
{
	if(m_bTemporaryModeInstance) return;

	//don't add the root item to the tree
	if(pxItem->IsRoot()) return;

	Tools_GUIMenu_Root* pxParent = pxItem->GetParent();

	//find parent in tree
	Asura_TreeDialog_Item* pxParentTreeItem = 0;
	if(pxParent && !pxParent->IsRoot())
	{
		pxParentTreeItem = pxParent->GetTreeItem();

		Asura_Assert(pxParentTreeItem,"Failed to find parent in tree view");
	}

	if(!pxParentTreeItem) pxParentTreeItem = m_pxTree->GetRoot();

	char szTreeName[100];
	pxItem->GetNameForTree(szTreeName,sizeof(szTreeName));

	//add child to tree
	Asura_TreeDialog_Item* pxTreeItem = m_pxTree->AddItem(pxParentTreeItem,szTreeName,(int)pxItem);
	pxItem->SetTreeItem(pxTreeItem);
}

void Tools_Mode_GUIMenu::RemoveFromTree(Tools_GUIMenu_Root* pxItem)
{
	if(m_bTemporaryModeInstance) return;

	if(pxItem->IsRoot()) return;

	Asura_TreeDialog_Item* pxTreeItem = pxItem->GetTreeItem();
	Asura_Assert(pxTreeItem,"Failed to find item in tree view");

	if(pxTreeItem)
	{
		m_pxTree->DeleteItem(pxTreeItem);
		pxItem->SetTreeItem(0);
	}
}

void Tools_Mode_GUIMenu::UpdateParentInTree(Tools_GUIMenu_Root* pxItem)
{
	if(m_bTemporaryModeInstance) return;

	//find the existing tree item for this
	Asura_TreeDialog_Item* pxTreeItem = pxItem->GetTreeItem();
	Asura_Assert(pxTreeItem,"Failed to find item in tree view");

	//and the new parent
	Tools_GUIMenu_Root* pxParent = pxItem->GetParent();
	Asura_TreeDialog_Item* pxParentTreeItem = 0;
	if(pxParent && !pxParent->IsRoot())
	{
		pxParentTreeItem = pxParent->GetTreeItem();

		Asura_Assert(pxParentTreeItem,"Failed to find parent in tree view");
	}

	if(!pxParentTreeItem) pxParentTreeItem = m_pxTree->GetRoot();

	//move to the new parent in the tree
	m_pxTree->MoveItem(pxTreeItem,pxParentTreeItem);
	pxTreeItem->MakeVisible();

	
}

void Tools_Mode_GUIMenu::UpdateNameOnTree(Tools_GUIMenu_Root* pxItem)
{
	if(m_bTemporaryModeInstance) return;

	if(pxItem->IsRoot()) return;

	Asura_TreeDialog_Item* pxTreeItem = pxItem->GetTreeItem();
	Asura_Assert(pxTreeItem,"Failed to find item in tree view");

	if(pxTreeItem)
	{
		char szTreeName[100];
		pxItem->GetNameForTree(szTreeName,sizeof(szTreeName));

		pxTreeItem->SetName(szTreeName);
	}
}

void Tools_Mode_GUIMenu::UpdateChildOrderOnTree(Tools_GUIMenu_Root* pxToolsParent)
{
	if(m_bTemporaryModeInstance) return;
	
	Asura_TreeDialog_Item* pxParentTreeItem = 0;
	if(pxToolsParent == m_pxRoot)
	{
		pxParentTreeItem = m_pxTree->GetRoot();
	}
	else
	{
    	pxParentTreeItem = pxToolsParent->GetTreeItem();
		Asura_Assert(pxParentTreeItem,"Failed to find item in tree view");
	}

	if(pxParentTreeItem)
	{
		//move each item to the end of the list in turn , so that they end up in the correct order
		for(CLIF<Tools_GUIMenu_Root*> xChildLIF(pxToolsParent->GetChildList());!xChildLIF.done();xChildLIF.next())
		{
			Tools_GUIMenu_Root* pxItem = xChildLIF();

			Asura_TreeDialog_Item* pxTreeItem = pxItem->GetTreeItem();
			Asura_Assert(pxTreeItem,"Failed to find item in tree view");

			if(pxTreeItem)
			{
				m_pxTree->MoveItem(pxTreeItem,pxParentTreeItem);
			}
		}
	}
	
}

void Tools_Mode_GUIMenu::SetSelectionFromTree()
{
	Asura_TreeDialog_Item* pxTreeItem = m_pxTree->GetSelectedItem();

	if(pxTreeItem)
	{
		SetSelection((Tools_GUIMenu_Root*) pxTreeItem->GetUserData());
	}
	else
	{
		SetSelection(0);
	}
}

void Tools_Mode_GUIMenu::SetSelection(Tools_GUIMenu_Root* pxItem)
{
	if(m_pxSelectedItem) m_pxSelectedItem->OnDeselection();

	m_pxSelectedItem = pxItem;

	if(m_pxSelectedItem) m_pxSelectedItem->OnSelection();


	//change the tree view to match
	if(pxItem)
	{
		Asura_TreeDialog_Item* pxTreeItem = m_pxTree->FindByData((int)pxItem);
		Asura_Assert(pxTreeItem,"Failed to find item in tree view");
		m_pxTree->SetSelectedItem(pxTreeItem);

		//make the selection visible on the overlay
		Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_GUIMENU);
		if(pxOverlay)
		{
			Asura_Dialog* pxDialog = pxOverlay->GetDialog(g_uMenuItemTreeID);
			if(pxDialog)
			{
				((Asura_Tree_Dialog*)pxDialog)->MakeSelectionVisible();
			}
		}
	}
	else
	{
		m_pxTree->SetSelectedItem(0);
	}

	//determine the current menu
	if(pxItem)
	{
		if(pxItem->IsFakeMenuItem())
		{
			s_pxToolsMenuFile = pxItem;
		}
		else
		{
			m_pxToolsMenu = (Tools_GUIMenu_Menu_Page*) pxItem->GetMenuPage();
			s_pxToolsMenuFile = m_pxToolsMenu->GetParent();
		}
	}
}

void Tools_Mode_GUIMenu::SetSelectionByID(Asura_Hash_ID uItemHash)
{
	if(uItemHash)
	{
		Tools_GUIMenu_Root* pxItem = FindItemByID(uItemHash);
		if(pxItem) SetSelection(pxItem);
	}
	else
	{
		SetSelection(0);
	}
}

bool Tools_Mode_GUIMenu::CanCreateWidget()
{
	if(!s_pxThis->m_pxToolsMenu) return false;
	return !s_pxThis->m_pxToolsMenu->GetAsuraData()->IsContainerFull();
}

void Tools_Mode_GUIMenu::CreateWidget(int iType)
{
	s_pxThis->StartMenuItemPlacement(s_pxThis->m_pxToolsMenu,iType);
}

void Tools_Mode_GUIMenu::ImportText(int iType)
{
	s_pxThis->StartMenuItemPlacement(s_pxThis->m_pxToolsMenu,iType);
}

void Tools_Mode_GUIMenu::StartMenuItemPlacement(Tools_GUIMenu_Root* pxToolsParent,int iType,Tools_GUIMenu_Root* pxSourceItem /*=0*/)
{
	if(pxToolsParent->GetAsuraData()->IsContainerFull()) return;

	//SetSelection(0);
	m_bDrag = false;

	m_bItemPlacement = true;
	m_iPlacementType = iType;
	m_pxPlacementParent = pxToolsParent;
	m_pxPlacementSource = pxSourceItem;
}

void Tools_Mode_GUIMenu::CancelMenuItemPlacement()
{
	m_bItemPlacement = false;
	m_pxPlacementParent = 0;
	m_pxPlacementSource = 0;
}


void Tools_Mode_GUIMenu::CreateItemAtCursor(Tools_GUIMenu_Root* pxToolsParentItem,int iType)
{
	if(!pxToolsParentItem) return;

	if(iType == TOOLS_GUIMENU_PASTE)
	{
		Tools_GUIMenu_Root* pxCopy = CreateCopy(pxToolsParentItem,m_pxPlacementSource,TOOLS_MENUS_COPY_ATCURSOR);
		if(pxCopy)
		{
			SetSelection(pxCopy);
		}
		return;
	}
	else if(iType == TOOLS_GUIMENU_STYLE)
	{
		Tools_GUIMenu_Root* pxToolsItem = CreateCopy(pxToolsParentItem,m_pxPlacementSource,TOOLS_MENUS_COPY_ATCURSOR|TOOLS_MENUS_COPY_STYLE|TOOLS_MENUS_COPY_STYLEINSTANCE);
		if(pxToolsItem) SetSelection(pxToolsItem);
		return;
	}

	//Get position to create item at
	Asura_Vector_2 xPos = GetCursorPosInMenuCoords();
	xPos.x = SnapX(xPos.x);
	xPos.y = SnapY(xPos.y);

	if (iType == TOOLS_GUIMENU_IMPORTTEXT)
	{
		Tools_GUIMenu_ImportText::SelectFileToImportText(pxToolsParentItem,false, xPos);
		return;
	}

	else if (iType == TOOLS_GUIMENU_IMPORTALLTEXT)
	{
		Tools_GUIMenu_ImportText::SelectFileToImportText(pxToolsParentItem,true, xPos);
		return;
	}

	//create the new item
	Tools_GUIMenu_Root* pxToolsItem = CreateItem(pxToolsParentItem, iType, xPos);

	if(pxToolsItem)
	{
		SetSelection(pxToolsItem);
	}
	
}

Tools_GUIMenu_Root* Tools_Mode_GUIMenu::CreateItem(Tools_GUIMenu_Root* pxToolsParentItem,int iType, const Asura_Vector_2& xPos)
{
	if(!pxToolsParentItem) return 0;

	if(iType == TOOLS_GUIMENU_PASTE)
	{
		Asura_Assert(0,"Not exepecting 'paste' type to be used in this function");
		return 0;
	}

	//allocate the asura widget/element
	Asura_Bounding_Rectangle xRect(0.0f,0.0f,20.0f,20.0f);
	Asura_GUIMenu_Root* pxAsuraItem = Asura_GUIMenu_System::Create((Asura_GUIMenu_Root::Asura_GUIMenu_Object_Type)iType, xRect);
	if(!pxAsuraItem) return 0;
	
	//generate a default name
	const char* szBaseName = GetMenuItemTypeName(iType);


	Asura_GUIMenu_Root* pxAsuraParentItem = pxToolsParentItem->GetAsuraData();
	Asura_String xName = GenerateUniqueName(pxToolsParentItem,szBaseName);

	pxAsuraItem->SetName(xName);
	
	//attempt to add it to the menu
	if(!pxAsuraParentItem->AddToThisContainer(pxAsuraItem))
	{
		delete pxAsuraItem;
		ShowMessageOverlay("Error adding widget/element");
		return 0;
	}
	pxAsuraItem->GenerateHashID();


	//set the position
	pxAsuraItem->SetUnscaledLocalPos(xPos - pxToolsParentItem->GetAsuraData()->GetScaledWorldPos() - pxToolsParentItem->GetAsuraData()->GetChildPosOffset());


	//create the tools class
	Tools_GUIMenu_Root* pxToolsItem = AllocateToolsMenuItem(pxAsuraItem,pxToolsParentItem);
	pxToolsItem->InitialiseNewItem();
//	pxToolsParentItem->CalculateExtents();

	//flag as modified
	pxToolsItem->SetModified();
	return pxToolsItem;
}


/*
Asura_GUIMenu_Root* Tools_Mode_GUIMenu::AllocateAsuraMenuItem(int iType)
{
	switch(iType)
	{
		//menu
		case Asura_GUIMenu_Root::GUIMENU_TYPEID_MENUPAGE :
			Asura_Assert(0,"Not expecting to allocate menu via this function at the moment.");
			return 0;

		//widgets
		case Asura_GUIMenu_Root::GUIMENU_TYPEID_WIDGET_BUTTON :
			return new Asura_GUIMenu_Widget_Button;

		case Asura_GUIMenu_Root::GUIMENU_TYPEID_WIDGET_CHECKBOX :
			return new Asura_GUIMenu_Widget_CheckBox;

		case Asura_GUIMenu_Root::GUIMENU_TYPEID_WIDGET_SLIDER :
			return new Asura_GUIMenu_Widget_Slider;

		case Asura_GUIMenu_Root::GUIMENU_TYPEID_WIDGET_TEXTBOX :
			return new Asura_GUIMenu_Widget_TextBox;

		case Asura_GUIMenu_Root::GUIMENU_TYPEID_WIDGET_TEXTLISTBOX :
			return new Asura_GUIMenu_Widget_TextListBox;

		case Asura_GUIMenu_Root::GUIMENU_TYPEID_WIDGET_GROUP :
			return new Asura_GUIMenu_Widget_Group;

		case Asura_GUIMenu_Root::GUIMENU_TYPEID_WIDGET_ANIMATINGIMAGE :
			return new Asura_GUIMenu_Widget_AnimatingImage;

		//elements
		case Asura_GUIMenu_Root::GUIMENU_TYPEID_ELEMENT_IMAGE :
			return new Asura_GUIMenu_Element_Image;

		case Asura_GUIMenu_Root::GUIMENU_TYPEID_ELEMENT_TEXT :
			return new Asura_GUIMenu_Element_Text;

		case Asura_GUIMenu_Root::GUIMENU_TYPEID_ELEMENT_PATH :
			return new Asura_GUIMenu_Element_Path;

			
		default :
			Asura_Assert(0,"Unexpected menu item type");
			return 0;
	}
}
*/

const char* Tools_Mode_GUIMenu::GetMenuItemTypeName(int iType)
{
	switch(iType)
	{
		//menu
		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_MENUPAGE :
			return "Menu";

		//widgets
		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_BUTTON :
			return "Button";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_CHECKBOX :
			return "Checkbox";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_SLIDER :
			return "Slider";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTBOX :
			return "Textbox";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX :
			return "TextListbox";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_GROUP :
			return "Group";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_ANIMATINGIMAGE :
			return "Animated Image";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_PROGRESSBAR :
			return "ProgressBar";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_NUMERIC :
			return "Numeric";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX :
			return "Listbox";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOXENTRY :
			return "ListboxEntry";

		//elements
		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE :
			return "Image";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT :
			return "Text";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_PATH :
			return "Path";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_ANIMATION :
			return "Animation";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_STATUSTEXT :
			return "StatusText";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_FMV :
			return "FMV";

		default :
			Asura_Assert(0,"Unexpected widget type");
			return 0;
	}
}

const char* Tools_Mode_GUIMenu::GetShortMenuItemTypeName(int iType)
{
	switch(iType)
	{
		//menu
		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_MENUPAGE :
			return "Mn";

		//widgets
		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_BUTTON :
			return "Bn";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_CHECKBOX :
			return "CB";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_SLIDER :
			return "Sl";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTBOX :
			return "TB";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX :
			return "TLB";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_GROUP :
			return "Gp";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_ANIMATINGIMAGE :
			return "An";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_PROGRESSBAR :
			return "PB";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_NUMERIC :
			return "Nm";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX :
			return "LB";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOXENTRY :
			return "LBE";

		//elements
		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE :
			return "Im";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT :
			return "Tx";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_PATH :
			return "Pa";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_ANIMATION :
			return "AE";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_STATUSTEXT :
			return "STx";

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_FMV :
			return "FMV";

		default :
			Asura_Assert(0,"Unexpected widget type");
			return 0;
	}
}


Tools_GUIMenu_Root* Tools_Mode_GUIMenu::AllocateToolsMenuItem(Asura_GUIMenu_Root* pxAsuraItem,Tools_GUIMenu_Root* pxToolsParent)
{
	switch(pxAsuraItem->GetType())
	{
		//menu
		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_MENUPAGE :
			return new Tools_GUIMenu_Menu_Page((Asura_GUIMenu_Menu_Page*)pxAsuraItem,pxToolsParent);

		//widgets
		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_BUTTON :
			return new Tools_GUIMenu_Widget_Button((Asura_GUIMenu_Widget_Button*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_CHECKBOX :
			return new Tools_GUIMenu_Widget_CheckBox((Asura_GUIMenu_Widget_CheckBox*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_SLIDER :
			return new Tools_GUIMenu_Widget_Slider((Asura_GUIMenu_Widget_Slider*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTBOX :
			return new Tools_GUIMenu_Widget_TextBox((Asura_GUIMenu_Widget_TextBox*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX :
			return new Tools_GUIMenu_Widget_TextListBox((Asura_GUIMenu_Widget_TextListBox*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_GROUP :
			return new Tools_GUIMenu_Widget_Group((Asura_GUIMenu_Widget_Group*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_ANIMATINGIMAGE :
			return new Tools_GUIMenu_Widget_AnimatingImage((Asura_GUIMenu_Widget_AnimatingImage*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_PROGRESSBAR :
			return new Tools_GUIMenu_Widget_ProgressBar((Asura_GUIMenu_Widget_ProgressBar*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_NUMERIC :
			return new Tools_GUIMenu_Widget_Numeric((Asura_GUIMenu_Widget_Numeric*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX :
			return new Tools_GUIMenu_Widget_ListBox((Asura_GUIMenu_Widget_ListBox*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOXENTRY :
			return new Tools_GUIMenu_Widget_ListBoxEntry((Asura_GUIMenu_Widget_ListBoxEntry*)pxAsuraItem,pxToolsParent);

		//elements
		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE :
			return new Tools_GUIMenu_Element_Image((Asura_GUIMenu_Element_Image*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT :
			return new Tools_GUIMenu_Element_Text((Asura_GUIMenu_Element_Text*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_PATH :
			return new Tools_GUIMenu_Element_Path((Asura_GUIMenu_Element_Path*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_ANIMATION :
			return new Tools_GUIMenu_Element_Animation((Asura_GUIMenu_Element_Animation*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_STATUSTEXT :
			return new Tools_GUIMenu_Element_StatusText((Asura_GUIMenu_Element_StatusText*)pxAsuraItem,pxToolsParent);

		case Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_FMV :
			return new Tools_GUIMenu_Element_FMV((Asura_GUIMenu_Element_FMV*)pxAsuraItem,pxToolsParent);

		default :
			Asura_Assert(0,"Unexpected menu item type");
			return 0;

	}
}

void Tools_Mode_GUIMenu::DestroyMenuPage( Asura_GUIMenu_Menu_Page* pxGuiMenuPage )
{
	Tools_Mode_GUIMenu* pxInstance( Tools_Mode_GUIMenu::GetInstance() );
	Tools_GUIMenu_Root* pxToolsRoot(NULL);

	// if Tools mode exists try to find Tools_GuiMenu wrapper object
	if( pxInstance )
	{
		 pxToolsRoot = pxInstance->FindItemByID( pxGuiMenuPage->GetID() );
	}

	// if Tools_GuiMenu wrapper object exists make sure it is destroyed along with Asura object
	if( pxToolsRoot )
	{
		pxInstance->DeleteMenuItem( pxToolsRoot );
	}
	else
	{
		Asura_GUIMenu_System::DestroyMenuPage( pxGuiMenuPage,true );
	}

}

void Tools_Mode_GUIMenu::DeleteMenuItem(Tools_GUIMenu_Root* pxItem)
{
	CancelMenuItemPlacement();
	CancelPromptedSelection();
	//Is the selected item going to be deleted?
	if(m_pxSelectedItem)
	{
		if(m_pxSelectedItem->IsDescendentOf(pxItem)) SetSelection(0);
	}
	//Is the active menu being deleted?
	if(m_pxToolsMenu)
	{
		if(m_pxToolsMenu->IsDescendentOf(pxItem)) m_pxToolsMenu = 0;
	}

	if(pxItem == s_pxToolsMenuFile) 
	{
		s_pxToolsMenuFile = NULL;
	}

	pxItem->Destroy();

	//Force style updates to be done now , in case this was a member of a style
	HandleStyleUpdates();

}

void Tools_Mode_GUIMenu::StartPromptedSelection(Tools_GUIMenu_PickItem* pxPickItem)
{
	CancelPromptedSelection();
	m_pxPickItem = pxPickItem;
}

void Tools_Mode_GUIMenu::CancelPromptedSelection()
{
	delete m_pxPickItem;
	m_pxPickItem = 0;
}

void Tools_Mode_GUIMenu::PromptedSelection_SelectFromList()
{
	if(!m_pxPickItem) return;
	if(!m_pxToolsMenu) return;

	ListSelection_Overlay xOverlay(m_pxPickItem->GetPrompt());

	//get all eligible items from the current menu
	PromptedSelection_AddToListOverlay(m_pxToolsMenu,&xOverlay);

	if(!xOverlay.DoModal()) return;

	Tools_GUIMenu_Root* pxItem = (Tools_GUIMenu_Root*) xOverlay.GetSelectionData();
	
	m_pxPickItem->OnFinish(pxItem);
	CancelPromptedSelection();
	
}

void Tools_Mode_GUIMenu::PromptedSelection_AddToListOverlay(Tools_GUIMenu_Root* pxItem,ListSelection_Overlay* pxOverlay)
{
	if(m_pxPickItem->IsValidSelection(pxItem))
	{
		//get a list of the parents for building a path-type name
		Tools_GUIMenu_Root* apxItems[32];
		int iNumItems = 0;

		for(Tools_GUIMenu_Root* pxIt = pxItem; pxIt && !pxIt->IsFakeMenuItem(); pxIt = pxIt->GetParent())
		{
			if(iNumItems == 32)
			{
				Asura_Assert(0,"Menu item has unexpectedly large depth in hierarchy");
				break;
			}
			apxItems[iNumItems++] = pxIt;
		}

		if(iNumItems > 1)
		{
			//Since all the items are in the same menu ,there isn't much point of including the menu
			//name in the path.
			iNumItems--;
		}

		//stick the name together
		char szPath[128];
		int iPathSpace = 128;
		char* pcPos = szPath;

		for(int i=iNumItems-1;i>=0;i--)
		{
			if(i<(iNumItems-1) && iPathSpace >1)
			{
				*pcPos++ = '\\';
				iPathSpace--;
			}
			if(!iPathSpace) break;

			Asura_PrintfToBfr(pcPos,"%s",iPathSpace,true,apxItems[i]->GetName());
			int iLength = strlen(pcPos);
			pcPos += iLength;
			iPathSpace -= iLength;
		}

		pxOverlay->AddItem(szPath,(int)pxItem);
	}


	//check children as well
	for(CLIF<Tools_GUIMenu_Root*> xChildLIF(pxItem->GetChildList());!xChildLIF.done();xChildLIF.next())
	{
		PromptedSelection_AddToListOverlay(xChildLIF(),pxOverlay);
	}
}

bool Tools_Mode_GUIMenu::CanCreateMenu()
{
	if (Asura_GUIMenu_System::GetCurrentMenuListCount() >= ASURA_GUIMENU_MAX_MENU_PAGES)
	{
		return false;
	}
	return s_pxToolsMenuFile != 0;
}

void Tools_Mode_GUIMenu::CreateMenu()
{
	if(!CanCreateMenu()) return;

	Asura_String xName;
	if(!AskForMenuName(xName)) return;

	//create the asura and tools data for the menu

	// -- SK - 18.01.07 : Changed to use the AllocateToolsMenuItem function.  It makes it easier to add a created menu to a list i need.
	Asura_GUIMenu_Menu_Page* pxAsuraMenu = static_cast<Asura_GUIMenu_Menu_Page*>(Asura_GUIMenu_System::CreateMenuPage( xName, 0.0f, 0.0f, 600.0f, 600.0f));
	pxAsuraMenu->GenerateHashID();
	pxAsuraMenu->SetFilename(s_pxToolsMenuFile->GetName());
	m_pxToolsMenu = static_cast<Tools_GUIMenu_Menu_Page*>(AllocateToolsMenuItem( pxAsuraMenu, s_pxToolsMenuFile));
	m_pxToolsMenu->SetModified();

	//and select it
	SetSelection(m_pxToolsMenu);
}

const bool Tools_Mode_GUIMenu::AskForMenuName(Asura_String& xName)
{
	GetString_Overlay xOverlay("Enter name for menu","");
	const char* szNewName = 0;

	do
	{
		if(!xOverlay.DoModal()) return false;
		szNewName = xOverlay.GetString();

		//abort if no name was chosen
		if(!szNewName || !*szNewName) return false;

		//make sure the name isn't already in use 
		bool bOkay = true;
		for (u_int u = 0; u < Asura_GUIMenu_System::GetCurrentMenuListCount(); ++u)
		{
			if(!stricmp(Asura_GUIMenu_System::GetMenuPageByIndex(u)->GetNameAsChar(), szNewName))
			{
				bOkay = false;
				break;
			}
		}

		if(!bOkay)
		{
			ShowMessageOverlay("That name is already in use by another menu.");
			szNewName = 0;
		}
	}
	while(!szNewName);

	xName = szNewName;
	return true;
}

int Tools_Mode_GUIMenu::GetNumberOfWidgetTypes()
{
	return g_iNumWidgetTypes;
}

int Tools_Mode_GUIMenu::GetWidgetType(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<g_iNumWidgetTypes,"");
	return g_aiWidgetTypes[iIndex];
}

const char* Tools_Mode_GUIMenu::GetWidgetTypeName(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<g_iNumWidgetTypes,"");
	return GetMenuItemTypeName(g_aiWidgetTypes[iIndex]);
}


void Tools_Mode_GUIMenu::CreateToolsMenuItems()
{
	//If editing an environment , create the level specific file even if its not used by anything yet
	if(Tools_SaveAsura::GetFileType() == AsuraFile_Environment)
	{
		GetFileMenuItem(Tools_GUIMenu_System::GetLevelSpecificFile(),true);
	}

	for(u_int i=0;i<Asura_GUIMenu_System::GetCurrentMenuListCount();i++)
	{
		Asura_GUIMenu_Root* pxAsuraItem = Asura_GUIMenu_System::GetMenuPageByIndex(i);

		//Skipping any menus that haven't got a filename , in case there are any that have been created in code
		const char* szFilename = ((Asura_GUIMenu_Menu_Page*)pxAsuraItem)->GetFilename();
		if(!szFilename || !*szFilename) continue;


		Tools_GUIMenu_Root* pxFileItem = GetFileMenuItem(szFilename,true);

		CreateToolsMenuItems(pxAsuraItem,pxFileItem);
	}

	m_pxRoot->SortFiles();
	m_pxRoot->CheckRequiredReferencesRecursive();
}

Tools_GUIMenu_Root* Tools_Mode_GUIMenu::CreateToolsMenuItems(Asura_GUIMenu_Root* pxAsuraItem,Tools_GUIMenu_Root* pxToolsParent)
{
	//create the tools data for this item
	Tools_GUIMenu_Root* pxToolsItem = AllocateToolsMenuItem(pxAsuraItem,pxToolsParent);
	if(pxToolsItem)
	{
		//create children
		for(u_int i=0;i<pxAsuraItem->GetContainerCount();i++)
		{
			CreateToolsMenuItems(pxAsuraItem->GetContainerItem(i),pxToolsItem);
		}

		//update extents of this item (if required) , now that the children exist
		pxToolsItem->CalculateExtents();
	}

	return pxToolsItem;
}

bool Tools_Mode_GUIMenu::DebouncedKeyPressed(int iKey)
{
	if(m_pxSelectedItem && m_pxSelectedItem->IsUsingKey(iKey)) return false;
	if(Tools_GUIMenu_AnimEdit::IsUsingKey(iKey)) return false;
	return Asura_UserInput::DebouncedKeyPressed(iKey);
}

bool Tools_Mode_GUIMenu::KeyPressed(int iKey)
{
	if(m_pxSelectedItem && m_pxSelectedItem->IsUsingKey(iKey)) return false;
	if(Tools_GUIMenu_AnimEdit::IsUsingKey(iKey)) return false;
	return Asura_UserInput::KeyPressed(iKey);
}


void Tools_Mode_GUIMenu::HandleItemDeletion(Tools_GUIMenu_Root* pxItem)
{
	if(m_pxSelectedItem == pxItem) m_pxSelectedItem = 0;
	if(m_pxClipboardItem == pxItem) m_pxClipboardItem = 0;
	if(m_pxPlacementSource == pxItem) CancelMenuItemPlacement();

	//Check the array of items for the style overlay
	for(int i=0;i<m_iNumStyles;i++)
	{
		if(m_ppxStyleItems[i] == pxItem)
		{
			//Null the entry to make sure it doesn't get used
			m_ppxStyleItems[i] = 0;
			//and make sure the list gets recalculated in the next update
			m_bStyleTypeChanged = true;
			break;
		}
	}

	RemoveFromTree(pxItem);
}

bool Tools_Mode_GUIMenu::CanCopy()
{
	return m_pxSelectedItem != 0 && !m_pxSelectedItem->IsFakeMenuItem();
}

void Tools_Mode_GUIMenu::Copy()
{
	// Don't allow copying of textlistboxes as they are being phased out.
	if ( m_pxSelectedItem != NULL && 
		 m_pxSelectedItem->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX )
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Can't copy %s as it is a 'Text Listbox' which should not be used anymore.",m_pxSelectedItem->GetName());
		return;
	}

	m_pxClipboardItem = m_pxSelectedItem;

	if(m_pxClipboardItem)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"%s selected for copying",m_pxClipboardItem->GetName());
	}
}


Tools_GUIMenu_Root* Tools_Mode_GUIMenu::DetermineParentForPaste()
{
	if(!m_pxClipboardItem) return 0;;

	int iItemType = m_pxClipboardItem->GetType();

	if( iItemType == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_MENUPAGE)
	{
		//whole menus can always be pasted regardless of the selection
		return s_pxToolsMenuFile;
	}

	//check to see if the clipboard item can be pasted as a child of the selected item , or one of its parents
	for(Tools_GUIMenu_Root* pxSelected = m_pxSelectedItem;pxSelected;pxSelected = pxSelected->GetParent())
	{
		//don't allow an item to pasted within itself
		if(pxSelected->IsDescendentOf(m_pxClipboardItem)) continue;
		
		if(pxSelected->IsValidChildType(iItemType)) return pxSelected;
	}

	//Failing that , see if the current menu can take the item directly
	if(m_pxToolsMenu && m_pxToolsMenu->IsValidChildType(iItemType)) return m_pxToolsMenu;

	return 0;
}

bool Tools_Mode_GUIMenu::CanPaste()
{
	Tools_GUIMenu_Root* pxPasteParent = DetermineParentForPaste();
	if(!pxPasteParent) return false;

	//is there enough room to add more to the parent?
	if(!pxPasteParent->GetAsuraData()) return true;
	return !pxPasteParent->GetAsuraData()->IsContainerFull();
}

void Tools_Mode_GUIMenu::Paste()
{
	if(!CanPaste()) return;

	Tools_GUIMenu_Root* pxPasteParent = DetermineParentForPaste();
	if(!pxPasteParent) return;
	if(!m_pxClipboardItem) return;

	//ask for position to paste to ,unless doing an entire menu
	if(m_pxClipboardItem->IsMenu())
	{
		Tools_GUIMenu_Root* pxCopy = CreateCopy(pxPasteParent,m_pxClipboardItem,0);
		if(pxCopy)
		{
			SetSelection(pxCopy);
		}
	}
	else
	{
		StartMenuItemPlacement(pxPasteParent,TOOLS_GUIMENU_PASTE,m_pxClipboardItem);
	}

}

void Tools_Mode_GUIMenu::PasteAtCursor()
{
	if(!m_pxClipboardItem)
	{
		Tools_Timed_Message_System::AddMessage(0xffff00,"No item selected for copying");
		return;
	}

	Tools_GUIMenu_Root* pxPasteParent = DetermineParentForPaste();
	if(!pxPasteParent)
	{
		Tools_Timed_Message_System::AddMessage(0xffff00,"Failed to find suitable item to paste under.");
		return;
	}

	if(pxPasteParent->GetAsuraData() && pxPasteParent->GetAsuraData()->IsContainerFull())
	{
		Tools_Timed_Message_System::AddMessage(0xffff00,"Can't add any more items to %s.",pxPasteParent->GetName());
		return;
	}

	Tools_GUIMenu_Root* pxCopy = CreateCopy(pxPasteParent,m_pxClipboardItem,TOOLS_MENUS_COPY_ATCURSOR);

	if(pxCopy)
	{
		pxCopy->SetModified();
		SetSelection(pxCopy);

		Tools_Timed_Message_System::AddMessage(0xffffff,"Pasted %s",pxCopy->GetName());
	}
}


Tools_GUIMenu_Root* Tools_Mode_GUIMenu::CreateCopy(Tools_GUIMenu_Root* pxDestParent,const Tools_GUIMenu_Root* pxSource,u_int uCopyFlags,const char* const szDestName /* = NULL */ )
{
	if(!pxSource) return 0;

	if(pxSource->IsMenu())
	{
		s_bCopyingMenu = true;
	}

	//make the copy
	List<Tools_GUIMenu_HashPair> xMappingList;
	Tools_GUIMenu_Root* pxToolsItem = CreateCopy(pxDestParent,pxSource,uCopyFlags|TOOLS_MENUS_COPY_CLAMPTOPARENT,&xMappingList,szDestName );

	s_bCopyingMenu = false;

	//And fix up any internal references
	if(pxToolsItem)
	{
		pxToolsItem->AddCreationComment();
		
		UpdateAfterCopy(pxToolsItem,&xMappingList);

		if(pxToolsItem->IsStyleInstance())
		{
			pxToolsItem->UpdateStyleInstances();	
		}
	}

	return pxToolsItem;
}

Tools_GUIMenu_Root* Tools_Mode_GUIMenu::CreateCopy(Tools_GUIMenu_Root* pxDestParent,const Tools_GUIMenu_Root* pxSource,u_int uCopyFlags,List<Tools_GUIMenu_HashPair>* pxOutputHashMapping,const char* const szDestName /* = NULL */ )
{
	// pxDestParent->GetMenuPage()->
	if(!pxDestParent || !pxSource) return NULL;

	Asura_GUIMenu_Root* pxAsuraParentItem = pxDestParent->GetAsuraData();

	//If not doing a style application and the source was generated by a style , see if there is an equivalent
	//item in the destination parent already.
	//This is to cope with children that may already have been generated as a result of earlier style application
	Tools_GUIMenu_Root* pxToolsItem(NULL);
	Asura_GUIMenu_Root* pxAsuraItem(NULL);

	if( pxSource->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOXENTRY )
	{
		//Listbox entries are automatically generated by the listbox.
		//Don't want to create a new one , but instead try to find the existing
		//entry so that the rest of the copying can be done.
		for(CLIF<Tools_GUIMenu_Root*> xChildLIF(pxDestParent->GetChildList());!xChildLIF.done();xChildLIF.next())
		{
			if(xChildLIF()->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOXENTRY)
			{
				pxToolsItem = xChildLIF();
				pxAsuraItem = pxToolsItem->GetAsuraData();
				break;
			}
		}

		if(!pxToolsItem || !pxAsuraItem)
		{
			Asura_Assert(0,"Failed to find existing listbox entry to copy over");
			return NULL;
		}
	}

	//allocate the asura widget/element
	if(!pxAsuraItem)
	{
		Asura_String xName;

		if( szDestName )
		{
			xName = szDestName;
		}
		else if(pxSource->IsMenu())
		{
			//Ask for a name immediately when copying menus
			if(!AskForMenuName(xName)) return NULL;
		}
		else
		{
			//Give the copy a name based on the original , but make sure it is unique
			//amongst the sibling items.
			xName = pxSource->GetName();
			if(IsNameInUse(pxDestParent,xName.GetString()))
			{
				xName = GenerateUniqueName(pxDestParent,xName.GetString());
			}
		}

		Asura_Bounding_Rectangle xRect(0.0f,0.0f,0.0f,0.0f);
		pxAsuraItem = Asura_GUIMenu_System::Create((Asura_GUIMenu_Root::Asura_GUIMenu_Object_Type)pxSource->GetType(), xRect);
		
		if(!pxAsuraItem) return NULL;

		pxAsuraItem->SetName(xName);
	
		//attempt to add it to the menu
		if(pxAsuraParentItem && (!pxAsuraParentItem->AddToThisContainer(pxAsuraItem)) )
		{
			delete pxAsuraItem;
			ShowMessageOverlay("Error adding widget/element");
			return NULL;
		}

		if(pxAsuraItem->GetType() == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_MENUPAGE)
		{
			Asura_Assert(s_pxToolsMenuFile,"");
			
			//set the filename in the new menu
			Asura_GUIMenu_Menu_Page* pxMenuPage = (Asura_GUIMenu_Menu_Page*) pxAsuraItem;
			pxMenuPage->SetFilename(s_pxToolsMenuFile->GetName());

		}
		pxAsuraItem->GenerateHashID();
	}


	//store ID mapping from source to destination
	if(pxOutputHashMapping) 
	{
		Tools_GUIMenu_HashPair xMapping;
		xMapping.m_uOldID = pxSource->GetAsuraData()->GetID();
		xMapping.m_uNewID = pxAsuraItem->GetID();
		pxOutputHashMapping->add_entry(xMapping);
	}

	//create the tools class
	if(!pxToolsItem) pxToolsItem = AllocateToolsMenuItem(pxAsuraItem,pxDestParent);
	
	u_int uChildCopyFlags = uCopyFlags;
	//These flags don't get propagated down
	uChildCopyFlags &=~ (TOOLS_MENUS_COPY_ATCURSOR|TOOLS_MENUS_COPY_CLAMPTOPARENT|TOOLS_MENUS_COPY_STYLEINSTANCE);

	if(!(uCopyFlags & TOOLS_MENUS_COPY_STYLE) && pxSource->IsStyleInstance())
	{
		//Not copying a style , but the source is a style instance.
		//So copy the style hash when copying the children
		uChildCopyFlags |= TOOLS_MENUS_COPY_COPYSTYLEHASH;
	}

	//create copies of any children
	for(CLIF<Tools_GUIMenu_Root*> xChildLIF(pxSource->GetChildList());!xChildLIF.done();xChildLIF.next())
	{
		CreateCopy(pxToolsItem,xChildLIF(),uChildCopyFlags,pxOutputHashMapping);
	}

	// SK 28.02.07 - Moved copy to here after the children have been added.
	//copy over the new item
	pxToolsItem->Copy(pxSource,uCopyFlags);

	//set the position
	if(pxAsuraParentItem)
	{
		if(uCopyFlags & TOOLS_MENUS_COPY_ATCURSOR)
		{
			//place at the cursor
			Asura_Vector_2 xPos = GetCursorPosInMenuCoords();
			xPos.x = SnapX(xPos.x);
			xPos.y = SnapY(xPos.y);
			xPos -= pxDestParent->GetAsuraData()->GetScaledWorldPos() + pxDestParent->GetAsuraData()->GetChildPosOffset();

			pxAsuraItem->SetUnscaledLocalPos(xPos);
		}
		else
		{
			//any child items being copied will keep the same local position
			pxAsuraItem->SetUnscaledLocalPos(pxSource->GetAsuraData()->GetUnscaledLocalPos());

		}
	}

	if(uCopyFlags & TOOLS_MENUS_COPY_CLAMPTOPARENT)
	{
		//clamp the top item to the parent
		pxToolsItem->ClampToParentRect();
	}

	return pxToolsItem;
}

void Tools_Mode_GUIMenu::UpdateAfterCopy(Tools_GUIMenu_Root* pxToolsItem,List<Tools_GUIMenu_HashPair>* pxHashMapping)
{
	//Convert any internal references to the copied items
	if(pxHashMapping)
	{
		for(LIF<Tools_GUIMenu_HashPair> xMappingLIF(pxHashMapping);!xMappingLIF.done();xMappingLIF.next())
		{
			pxToolsItem->UpdateReferencesRecursive(xMappingLIF().m_uOldID,xMappingLIF().m_uNewID,true);
		}
	}

	//Make sure text boxes know about their elements etc
	pxToolsItem->CheckRequiredReferencesRecursive();
}

bool Tools_Mode_GUIMenu::CanPasteOver()
{
	//need a source and destination item

	if(!m_pxClipboardItem) return false;
	if(!m_pxSelectedItem) return false;

	//which must be different
	if(m_pxClipboardItem == m_pxSelectedItem) return false;

	//but of the same type
	if(m_pxClipboardItem->GetType() != m_pxSelectedItem->GetType()) return false;

	//Make sure neither is contained within the other (prevent things going pear-shaped with nested groups)
	if(m_pxClipboardItem->IsDescendentOf(m_pxSelectedItem) || m_pxSelectedItem->IsDescendentOf(m_pxClipboardItem)) return false;


	return true;
}

void Tools_Mode_GUIMenu::PasteOver()
{
	if(!CanPasteOver()) return;

	//delete any children from the selected item
	while(m_pxSelectedItem->GetChildList()->size())
	{
		DeleteMenuItem(m_pxSelectedItem->GetChildList()->first_entry());
	}


	List<Tools_GUIMenu_HashPair> xMappingList;

	u_int uCopyFlags = 0;
	if(m_pxClipboardItem->IsStyleInstance()) uCopyFlags |= TOOLS_MENUS_COPY_COPYSTYLEHASH;

	//create copies of any children
	for(CLIF<Tools_GUIMenu_Root*> xChildLIF(m_pxClipboardItem->GetChildList());!xChildLIF.done();xChildLIF.next())
	{
		CreateCopy(m_pxSelectedItem,xChildLIF(),uCopyFlags,&xMappingList);
	}

	//copy properties to the selected item
	m_pxSelectedItem->Copy(m_pxClipboardItem,uCopyFlags);

	//Update any internal references
	UpdateAfterCopy(m_pxSelectedItem,&xMappingList);


	//close/open overlay for the menu item
	m_pxSelectedItem->OnDeselection();
	m_pxSelectedItem->OnSelection();

	//clamp the selected item , in case it has become to large from the copying
	m_pxSelectedItem->ClampToParentRect();
	m_pxSelectedItem->SetModified();
}


const Asura_String Tools_Mode_GUIMenu::GenerateUniqueName( const Tools_GUIMenu_Root* pxParent,const char* const szBaseName )
{
	//remove any trailing spaces/numbers from the base name
	char szStrippedBaseName[100];
	Asura_PrintfToBfr(szStrippedBaseName,"%s",sizeof(szStrippedBaseName),true,szBaseName);

	int iLength = strlen(szStrippedBaseName);
	while(iLength > 0)
	{
		if((szStrippedBaseName[iLength-1] >= '0' && szStrippedBaseName[iLength-1] <= '9') ||
			szStrippedBaseName[iLength-1]== ' ' || szStrippedBaseName[iLength-1]== '_')
		{
			szStrippedBaseName[--iLength] = 0;
		}
		else
		{
			break;
		}
	}

	//find a unique name that starts with the base name
	char szNewName[120];
	for(int iIndex=1;;iIndex++)
	{
		Asura_PrintfToBfr(szNewName,"%s_%d",sizeof(szNewName),true,szStrippedBaseName,iIndex);

		if(!IsNameInUse(pxParent,szNewName))
		{
			break;
		}
	}

	return szNewName;
}

const bool Tools_Mode_GUIMenu::IsNameInUse( const Tools_GUIMenu_Root* pxParent,const char* const szNewName )
{
	for(CLIF<Tools_GUIMenu_Root*> xChildLIF(pxParent->GetChildList());!xChildLIF.done();xChildLIF.next())
	{
		const char* szName = xChildLIF()->GetName();
		if(szName && !stricmp(szName,szNewName))
		{
			return true;
		}
	}
	return false;
}

Tools_GUIMenu_Root* Tools_Mode_GUIMenu::GetFileMenuItem(const char* szFilename,bool bCreate)
{
	Asura_Assert(m_pxRoot,"");

	//search through the root item's children
	for(CLIF<Tools_GUIMenu_Root*> xChildLIF(m_pxRoot->GetChildList());!xChildLIF.done();xChildLIF.next())
	{
		const char* szName = xChildLIF()->GetName();
		if(szName && !stricmp(szName,szFilename))
		{
			return xChildLIF();
		}
	}

	if(!bCreate) return false;

	return new Tools_GUIMenu_File(szFilename,m_pxRoot);
}


void Tools_Mode_GUIMenu::CreateFile()
{
	GetString_Overlay xOverlay("Enter new filename (without path/extension):","");
	if(xOverlay.DoModal())
	{
		const char* szFilename = xOverlay.GetString();
		if(*szFilename)
		{
			if(!stricmp(szFilename,Tools_GUIMenu_System::GetLevelSpecificFile()))
			{
				//don't allow 'Level' to be picked.
				ShowMessageOverlay("%s is reserved for the level specific file",Tools_GUIMenu_System::GetLevelSpecificFile());
			}
			else
			{
				if(strlen(szFilename)>8 && Tools_Level_Export::IsPlatformSupported(Tools_Level_Export::Export_PS2))
				{
					ShowMessageOverlay("Warning.\nHaving menu filenames that are longer than 8 characters may cause problems for PS2 exports.");
				}
				
				//add an entry for this file to the tree
				Tools_GUIMenu_Root* pxFileItem = GetFileMenuItem(szFilename,true);
				m_pxRoot->SortFiles();


				//and select it
				SetSelection(pxFileItem);
			
			}
		}
	}
}

Tools_GUIMenu_Root* Tools_Mode_GUIMenu::FindItemByID(Asura_Hash_ID uID)
{
	if(!uID) return 0;
	return m_pxRoot->FindByID(uID);
}

bool Tools_Mode_GUIMenu::CanPreviewMenu()
{
	return m_pxToolsMenu!=0;	
}

void Tools_Mode_GUIMenu::MenuPreview()
{
	MenuPreview_Start();
}

void Tools_Mode_GUIMenu::MenuPreview_Start()
{
	//need to have a current menu ,and not already be previewing it
	if(!m_pxToolsMenu) return;	
	if(s_bMenuPreview) return;

	Asura_GUIMenu_System::SetSpeed(1.0f);

	//Get the initial state of Use Cursor since it will be changed by the preview update code
	s_bInitialUseCursor = Asura_GUIMenu_System::GetUseCursor();

	//make sure there is a starup widget etc.
	m_pxToolsMenu->CheckRequiredReferencesRecursive();

	Asura_GUIMenu_Menu_Page* pxMenu = m_pxToolsMenu->GetAsuraMenu();

	Asura_GUIMenu_MessagePump::PostMessage(pxMenu->GetID(),0,ASURA_GUIMENU_UI_MSG_ON_INIT,0);
	Asura_GUIMenu_System::SetFocusedMenuByID(pxMenu->GetID());


	//hide existing overlays
	Asura_Overlay_System::SetHideFlagOnCurrentOverlays(ASURA_OVERLAY_HIDEFLAG_MODE,true);
	//create preview mode overlay
	Tools_GUIMenu_PreviewOverlay::StartPreview(m_pxRoot);
	s_bMenuPreview = true;

}

void Tools_Mode_GUIMenu::MenuPreview_End()
{
	if(!s_bMenuPreview) return;
	
	Asura_GUIMenu_System::SetSpeed(1.0f);

	//deactivate all menus , since another menu may have been opened via a button press
	for(u_int i=0;i<Asura_GUIMenu_System::GetCurrentMenuListCount();i++)
	{
		Asura_GUIMenu_System::SetMenuPageActive(i,false);
	}
	// This removes any input focus on menus and pops the menu control off the control stack.
	Asura_GUIMenu_System::SetFocusedMenuByID(ASURA_HASH_ID_UNSET);

	//Reset stuff like path positions
	m_pxRoot->PostMenuPreviewCleanupRecursive();

	Tools_GUIMenu_PreviewOverlay::CloseOverlay();
	//unhide overlays
	Asura_Overlay_System::SetHideFlagOnCurrentOverlays(ASURA_OVERLAY_HIDEFLAG_MODE,false);

	Asura_GUIMenu_System::SetUseCursor(s_bInitialUseCursor);

	Asura_GUIMenu_FMV_System::StopAllForced();

	s_bMenuPreview = false;
}

void Tools_Mode_GUIMenu::MenuPreview_Update()
{
	Asura_GUIMenu_System::SetSpeed(Tools_GUIMenu_PreviewOverlay::GetPreviewSpeed());

#ifdef ASURA_USE_CONTROLMAP_MANAGER
	Asura_ControlManager::Update();
#endif

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	Asura_VideoToTexture_System::Update();
#endif

	Asura_GUIMenu_System::Update();
	Tools_GUIMenu_PreviewOverlay::Update();

	MenuPreview_HandleKeyboard();

}

void Tools_Mode_GUIMenu::MenuPreview_Render()
{
	for (u_int u = 0; u < Asura_GUIMenu_System::GetCurrentMenuListCount(); ++u)
	{
		if (Asura_GUIMenu_System::IsMenuPageActive(u))
		{
			// NY - 09 April 2009 - this code is the preview version of Asura_GUIMenu_Menu_Page::AutoResize
			Asura_GUIMenu_Menu_Page* pxMenu = Asura_GUIMenu_System::GetMenuPageByIndex(u);

			//get the menu's raw bounds in virtual space
			Asura_Bounding_Rectangle xMenuRect;
			pxMenu->GetUnscaledVirtualBounds(xMenuRect);


			//get preview screen bounds in virtual space
			Asura_Bounding_Rectangle xScreenRect(0.0f,0.0f,GetVirtualSize().x,GetVirtualSize().y);

			//centre the preview screen within the actual display
			if(Tools_GUIMenu_PreviewOverlay::s_bCentreScreen)
			{
				Asura_Vector_2 xDisplace;
				if (Tools_GUIMenu_PreviewOverlay::s_bUseVirtualResolution)
				{
					xDisplace.x = Asura_Viewport::CentreX - (GetVirtualSize().x * 0.5f);
					xDisplace.y = Asura_Viewport::CentreY - (GetVirtualSize().y * 0.5f);
				}
				else
				{
					xDisplace.x = Asura_Viewport::CentreX - (s_xScreenSize.x * 0.5f);
					xDisplace.y = Asura_Viewport::CentreY - (s_xScreenSize.y * 0.5f);
					xDisplace.x /= s_xScreenSize.x / GetVirtualSize().x;
					xDisplace.y /= s_xScreenSize.y / GetVirtualSize().y;
				}
				xScreenRect.Displace(xDisplace);
			}


			if (pxMenu->CanAutoResize())
			{
				//scale menu to preview bounds
				xMenuRect = xScreenRect;
			}
			else if(pxMenu->GetMenuPageFlags() & ASURA_GUIMENU_MENUPAGE_FLAG_CENTER_MENU_ON_SCREEN)
			{
				//centre menu within preview bounds
				xMenuRect.Displace(xScreenRect.GetDimension() * 0.5f - xMenuRect.GetDimension() * 0.5f - xMenuRect.GetTopLeft() + xScreenRect.GetTopLeft());
			}
			else
			{
				//offset into preview bounds
				xMenuRect.Displace(xScreenRect.GetTopLeft());
			}

			// new menu bounds in virtual space
			pxMenu->Resize( xMenuRect);
		}
	}
	RenderBackdrop();

	DWORD uMagFilter;
	if(s_bDisableBilinearFiltering)
	{
		//disable filtering while rendering the menu items
		Asura_Direct3D::D3DDevice->GetSamplerState( 0, D3DSAMP_MAGFILTER, &uMagFilter );
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_NONE );
	}

	if(Tools_GUIMenu_PreviewOverlay::s_bClipToScreen) EnableClipping();
	Asura_GUIMenu_System::Render();
	Asura_Render::FlushDisplay();
	DisableClipping();

	if(Tools_GUIMenu_PreviewOverlay::s_bShowSafeArea) RenderSafeArea();
	if(Tools_GUIMenu_PreviewOverlay::s_bShowScreenBorder) RenderScreenBorder();

	if(s_bDisableBilinearFiltering)
	{
		//restore original filter setting
		Asura_Direct3D::D3DDevice->SetSamplerState( 0, D3DSAMP_MAGFILTER, uMagFilter );
	}

}


void Tools_Mode_GUIMenu::MenuPreview_ProcessMessage(const Asura_Message_GUIMenu_Notify* pxData)
{
	if(!pxData) return;

	switch(ASURA_GUIMENU_GET_MAIN_MESSAGE(pxData->uActionType))
	{
		case ASURA_GUIMENU_UI_MSG_ON_DATA_REQUEST:
		{
			//If initialising a list box , then add some default entries
			if(pxData->WidgetType == Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX)
			{
				s_pxThis->MenuPreview_InitialiseListBox(pxData->uFromID);
			}
		}
		break;
	}
}

void Tools_Mode_GUIMenu::MenuPreview_InitialiseListBox(Asura_Hash_ID uID)
{
	//find the listbox being initialised
	const Tools_GUIMenu_Root* pxToolsItem = FindItemByID(uID);

	if(!uID)
	{
		Asura_Assert(0,"Failed to find tools item for listbox being initialised");
		return;
	}
	//make sure it is a listbox
	if(pxToolsItem->GetType() != Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX)
	{
		Asura_Assert(0,"Error initialising listbox. Item found was of the wrong type");
		return;
	}

	const Asura_GUIMenu_Widget_TextListBox* pxListBox = (const Asura_GUIMenu_Widget_TextListBox*) pxToolsItem->GetAsuraData();

	//give the listbox some default entries
	int iNumStrings = Asura_Maths::Min(pxListBox->GetMaxDisplayedEntries()+4,pxListBox->GetMaxAllowedEntries());


	//make sure we have enough dummy strings to use
	for(int i=m_xListBoxText.size();i<iNumStrings;i++)
	{
		char szBuffer[30];
		Asura_PrintfToBfr(szBuffer,"List Entry %d",sizeof(szBuffer),true,i+1);
		m_xListBoxText.add_entry(szBuffer);
	}

	//add the strings
	ASURA_GUIMENU_TEXTLISTBOX_CLEAR_ALL(uID);
	for(int i=0;i<iNumStrings;i++)
	{
		ASURA_GUIMENU_TEXTLISTBOX_SEND_TEXT(uID,m_xListBoxText[i].GetString());
	}
	
}

bool Tools_Mode_GUIMenu::ShouldUseOffsetAndZoom()
{
	//Use the zoom for rendering if in menu mode and not using the preview
	return Tools_System::IsActive() && (Tools_System::GetCurrentMode() == TOOLS_MODE_GUIMENU) && !IsPreview();
}

void Tools_Mode_GUIMenu::ToggleStyleOverlay()
{
	Asura_Assert(s_pxThis,"");
	if(s_pxThis->m_bStyleOverlayOpen)
	{
		s_pxThis->m_bStyleOverlayOpen = false;
		s_pxThis->CloseStyleOverlay();
	}
	else
	{
		s_pxThis->m_bStyleOverlayOpen = true;
		s_pxThis->CreateStyleOverlay();
	}
}

bool Tools_Mode_GUIMenu::StyleOverlayIsOn()
{
	Asura_Assert(s_pxThis,"");
	return s_pxThis->m_bStyleOverlayOpen;
}

void Tools_Mode_GUIMenu::CreateStyleOverlay()
{
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Style",false,&m_xStyleOverlayPosition,&m_uStyleOverlayMinimised);
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	Asura_Dialog* pxDialog;

	//setup drop down list for filtering by menu item type
	BuildStyleTypeList();

	m_iStyleTypeIndex = 0;
	m_bStyleTypeChanged = false;

	pxDialog = new Asura_DropDown_List_Dialog(xOffset,m_pszStyleTypeNames,&m_iStyleTypeIndex,10,m_iNumStyleTypes,0,&m_bStyleTypeChanged);
	((Asura_DropDown_List_Dialog*)pxDialog)->SetSortStrings(true);
	pxDialog->SetHelpText("Filters the list of styles by item type");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	//add list of styles
	UpdateStyleNameList();

	pxDialog = m_pxStyleListDialog = new Asura_List_Dialog(xOffset,m_pszStyleNames,&m_iStyleIndex,12,m_iNumStyles,0);
	m_pxStyleListDialog->SetDoubleClickFunction(CreateStyleInstance,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;

	pxDialog = new Asura_Function_Dialog("Goto Master Copy",xOffset,GotoStyle,(int)this);
	pxDialog->SetHelpText("Selects the root menu item in this style");
	pxDialog->SetIsEnabledFunction(CanGotoStyle,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Create",xOffset,CreateStyleInstance,(int)this);
	pxDialog->SetHelpText("Create an instance of the selected style");
	pxDialog->SetIsEnabledFunction(CanCreateStyleInstance,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Apply",xOffset,ApplyStyle,(int)this);
	pxDialog->SetHelpText("Applies the style to the selected item");
	pxDialog->SetIsEnabledFunction(CanApplyStyle,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	
	pxOverlay->m_uID = TOOLS_OVERLAY_GUIMENU_STYLE;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
}

void Tools_Mode_GUIMenu::CloseStyleOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_GUIMENU_STYLE);
	m_pxStyleListDialog = 0;
}


void Tools_Mode_GUIMenu::BuildStyleTypeList()
{
	DeleteStyleTypeList();

	int aiTypes[] =
	{
		0,//used for all types filter
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_MENUPAGE,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_BUTTON,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_CHECKBOX,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_SLIDER,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTBOX,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_TEXTLISTBOX,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_GROUP,
//		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_ANIMATINGIMAGE,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_PROGRESSBAR,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_IMAGE,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_TEXT,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_PATH,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_STATUSTEXT,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_ELEMENT_FMV,
		Asura_GUIMenu_Root::ASURA_GUIMENU_TYPEID_WIDGET_LISTBOX,
	};
	m_iNumStyleTypes = sizeof(aiTypes)/sizeof(int);

	m_pszStyleTypeNames = new const char*[m_iNumStyleTypes];
	m_piStyleTypes = new int[m_iNumStyleTypes];

	for(int i=0;i<m_iNumStyleTypes;i++)
	{
		m_pszStyleTypeNames[i] = aiTypes[i] ? GetMenuItemTypeName(aiTypes[i]) : "*All*";
		m_piStyleTypes[i] = aiTypes[i];
	}
}

void Tools_Mode_GUIMenu::DeleteStyleTypeList()
{
	m_iNumStyleTypes = 0;
	delete [] m_pszStyleTypeNames;
	m_pszStyleTypeNames = 0;
	delete [] m_piStyleTypes;
	m_piStyleTypes = 0;
}

void Tools_Mode_GUIMenu::UpdateStyleNameList()
{

	//get the initial selection (if any)
	Tools_GUIMenu_Root* pxSelectedItem = 0;
	if(m_iNumStyles && m_iStyleIndex < m_iNumStyles)
	{
		pxSelectedItem = m_ppxStyleItems[m_iStyleIndex];
	}

	DeleteStyleNameList();

	if(!m_piStyleTypes) return;

	//determine type to filter by
	int iType = m_piStyleTypes[m_iStyleTypeIndex];

	//Determine the number of matching styles
	m_iNumStyles = 0;
	for(CLIF<Tools_GUIMenu_Root*> xLIF(Tools_GUIMenu_Root::GetList());!xLIF.done();xLIF.next())
	{
		if(!xLIF()->IsStyle()) continue;
		if(iType && xLIF()->GetType() != iType) continue;

		m_iNumStyles++;
	}

	if(m_iNumStyles)
	{
		m_ppxStyleItems = new Tools_GUIMenu_Root*[m_iNumStyles];
		//find the items again to fill in the style list

		int iIndex = 0;
		for(CLIF<Tools_GUIMenu_Root*> xLIF(Tools_GUIMenu_Root::GetList());!xLIF.done();xLIF.next())
		{
			if(!xLIF()->IsStyle()) continue;
			if(iType && xLIF()->GetType() != iType) continue;

			m_ppxStyleItems[iIndex++] = xLIF();
		}
		Asura_Assert(iIndex == m_iNumStyles,"");

		//sort the items alphabetically
		qsort(m_ppxStyleItems,m_iNumStyles,sizeof(Tools_GUIMenu_Root*),SortStyleCallback);

		//and get a list of names for the overlay
		m_pszStyleNames = new char*[m_iNumStyles];
		for(int i=0;i<m_iNumStyles;i++)
		{
			const char* szName = m_ppxStyleItems[i]->GetName();
			m_pszStyleNames[i] = new char[strlen(szName)+1];
			strcpy(m_pszStyleNames[i],szName);
		}

		
	
	}
	else
	{
		//just make one entry for none
		m_iNumStyles = 1;
		m_ppxStyleItems = new Tools_GUIMenu_Root*[1];
		m_ppxStyleItems[0] = 0;

		m_pszStyleNames = new char*[1];
		m_pszStyleNames[0] = new char[7];
		strcpy(m_pszStyleNames[0],"*None*");
	}



	//update the selection index
	m_iStyleIndex = 0;
	if(pxSelectedItem)
	{
		for(int i=0;i<m_iNumStyles;i++)
		{
			if(pxSelectedItem == m_ppxStyleItems[i])
			{
				m_iStyleIndex = i;
				break;
			}
		}
	}

	//update the dialog if it already exists
	if(m_pxStyleListDialog)
	{
		m_pxStyleListDialog->ReplaceStrings(m_iNumStyles,m_pszStyleNames);
	}
}

void Tools_Mode_GUIMenu::DeleteStyleNameList()
{
	for(int i=0;i<m_iNumStyles;i++)
	{
		delete [] m_pszStyleNames[i];
	}
	delete [] m_pszStyleNames;
	m_pszStyleNames = 0;

	delete [] m_ppxStyleItems;
	m_ppxStyleItems = 0;


	m_iNumStyles = 0;
}

int Tools_Mode_GUIMenu::SortStyleCallback(const void* pItem1,const void* pItem2)
{
	const Tools_GUIMenu_Root* pxItem1 = *(const Tools_GUIMenu_Root**)pItem1;
	const Tools_GUIMenu_Root* pxItem2 = *(const Tools_GUIMenu_Root**)pItem2;

	return stricmp(pxItem1->GetName(),pxItem2->GetName());
}															  

bool Tools_Mode_GUIMenu::CanCreateStyleInstance()
{
	//need a currently selected style
	if(!m_iNumStyles) return false;
	Tools_GUIMenu_Root* pxStyle = m_ppxStyleItems[m_iStyleIndex];
	if(!pxStyle) return false;

	if(pxStyle->IsMenu())
	{
	    //a file needs to have been selected
		if(!s_pxToolsMenuFile) return false;
	}
	else
	{
		//need a current menu
		if(!m_pxToolsMenu) return false;
	}
	return true;    	
}

void Tools_Mode_GUIMenu::CreateStyleInstance()
{
	//Make sure there is a selected style
	if(!m_iNumStyles) return;

	Tools_GUIMenu_Root* pxStyle = m_ppxStyleItems[m_iStyleIndex];
	if(!pxStyle) return;
	if(!pxStyle->IsStyle()) return;

	//Determine where the style should be placed
	Tools_GUIMenu_Root* pxParent = GetParentForStyle(pxStyle);
	if(!pxParent) return;

	//make sure the parent can take the style item
	if(!pxParent->IsValidChildType(pxStyle->GetType())) return;

	
	if(pxStyle->IsMenu())
	{
		Tools_GUIMenu_Root* pxToolsItem = CreateCopy(pxParent,pxStyle,TOOLS_MENUS_COPY_STYLE|TOOLS_MENUS_COPY_STYLEINSTANCE);
		if(pxToolsItem) SetSelection(pxToolsItem);
	}
	else
	{
		//start placement of the style
		StartMenuItemPlacement(pxParent,TOOLS_GUIMENU_STYLE,pxStyle);
	}
}

bool Tools_Mode_GUIMenu::CanApplyStyle()
{
	//Need a selected item and style of the same type
	if(!m_pxSelectedItem) return false;
	
	if(!m_iNumStyles) return false;
	Tools_GUIMenu_Root* pxStyle = m_ppxStyleItems[m_iStyleIndex];
	if(!pxStyle) return false;

	return pxStyle->GetType() == m_pxSelectedItem->GetType();

}

void Tools_Mode_GUIMenu::ApplyStyle()
{
	if(!CanApplyStyle()) return;
	Tools_GUIMenu_Root* pxStyle = m_ppxStyleItems[m_iStyleIndex];
	if(!pxStyle->IsStyle()) return;

	m_pxSelectedItem->ApplyStyle(pxStyle->GetAsuraData()->GetID(),false);
}

bool Tools_Mode_GUIMenu::CanGotoStyle()
{
	if(!m_iNumStyles) return false;
	Tools_GUIMenu_Root* pxStyle = m_ppxStyleItems[m_iStyleIndex];
	return pxStyle != 0;

}

void Tools_Mode_GUIMenu::GotoStyle()
{
	if(!CanGotoStyle()) return;
	Tools_GUIMenu_Root* pxStyle = m_ppxStyleItems[m_iStyleIndex];

	SetSelection(pxStyle);
}


Tools_GUIMenu_Root* Tools_Mode_GUIMenu::GetParentForStyle(Tools_GUIMenu_Root* pxStyle)
{
	if(!pxStyle) return 0;

	//If the style is a menu , add to the selected menu file
	if(pxStyle->IsMenu()) return s_pxToolsMenuFile;

	//Otherwise place directly in the menu (for now)
	return m_pxToolsMenu;
}

void Tools_Mode_GUIMenu::FlagStyleForUpdate(Asura_Hash_ID uStyleID)
{
	Asura_Assert(s_pxThis,"");

	if(!s_pxThis->m_xStyleUpdateList.contains(uStyleID))
	{
		s_pxThis->m_xStyleUpdateList.add_entry(uStyleID);
	}
}

void Tools_Mode_GUIMenu::HandleStyleUpdates()
{
	
	for(LIF<Asura_Hash_ID> xUpdateLIF(&m_xStyleUpdateList);!xUpdateLIF.done();xUpdateLIF.next())
	{
		m_pxRoot->UpdateStyleInstances(xUpdateLIF());
	}

	//Clear the list after doing the updates to guard against recursive inclusion of styles
	while(m_xStyleUpdateList.size())
	{
		m_xStyleUpdateList.delete_first_entry();
	}
}


void Tools_Mode_GUIMenu::RemoveReferencesToStyle(Asura_Hash_ID uID)
{
	List<Tools_GUIMenu_Root*> xDeleteList;

	//Remove any references to this item as a style
	for(CLIF<Tools_GUIMenu_Root*> xLIF(Tools_GUIMenu_Root::GetList());!xLIF.done();xLIF.next())
	{
		if(xLIF()->GetStyleObjectHash() == uID)
		{
			if(xLIF()->IsStyleInstance())
			{
				//Unflag the members of this style instance
				xLIF()->DivorceFromStyle(true);
			}
			else if(xLIF()->IsGeneratedByStyle())
			{
				//This is an instance of a deleted style member , so add this to the list to delete
				xDeleteList.add_entry(xLIF());
			}
		}
	}
	while(xDeleteList.size())
	{
		xDeleteList.first_entry()->Destroy();
		xDeleteList.delete_first_entry();
	}
}

void Tools_Mode_GUIMenu::RemoveReferencesToMenuItem(Asura_Hash_ID uID)
{
	RemoveReferencesToStyle(uID);

	//Remove any other references to the item
	GetRootItem()->UpdateReferencesRecursive(uID, 0, false);
}

void Tools_Mode_GUIMenu::ReloadText()
{
	int iLanguageIndex = s_pxThis->m_iLanguageIndex;
	int iLanguage = s_pxThis->m_piLanguageIDs[iLanguageIndex];
	if (iLanguage<0) return;

	ReloadText(iLanguage);
}

void Tools_Mode_GUIMenu::ReloadText(int iLanguage)
{
	Asura_GUIMenu_System::ReloadAllMenuText(iLanguage);
}


void Tools_Mode_GUIMenu::BuildLanguageList()
{
	DeleteLanguageList();

	//just allocate enuogh space for all languages for simplicity
	m_iLanguageIndex = 0;
	m_iLanguageListSize = 0;
	m_pszLanguageNames = new const char*[ASURA_LT_L_NUM_LANGUAGES];
	m_piLanguageIDs = new int[ASURA_LT_L_NUM_LANGUAGES];

	//see which language files exist
	for(int i=0;i<ASURA_LT_L_NUM_LANGUAGES;i++)
	{
		// KOR - we are only checking the main menu text file,
		// not any of the extra project/platform specific ones.
		// This should be sufficient.
		char szFilename[50];
		Asura_PrintfToBfr(szFilename,"Misc\\Menu\\Menu_%s." ASURA_STANDARD_FILE_EXTENSION,sizeof(szFilename),true, Asura_LocalisedText::GetTwoCharCodeForLang((Asura_LocalisedText_Language)i));

		if(Tools_FileSystem::FileExists_CheckNetwork(szFilename))
		{
			m_pszLanguageNames[m_iLanguageListSize] =  Asura_LocalisedText::GetNameForLang((Asura_LocalisedText_Language)i);
			m_piLanguageIDs[m_iLanguageListSize] = i;
			if(i == Asura_LocalisedText::GetCurrentLanguage())
			{
				m_iLanguageIndex = m_iLanguageListSize;
			}		
			m_iLanguageListSize++;
		}
	}

	if(!m_iLanguageListSize)
	{
		m_pszLanguageNames[0] = "*None found*";
		m_piLanguageIDs[0] = -1;
		m_iLanguageListSize = 1;
	}
}

void Tools_Mode_GUIMenu::DeleteLanguageList()
{
	delete [] m_pszLanguageNames;
	m_pszLanguageNames = 0;
	delete [] m_piLanguageIDs;
	m_piLanguageIDs = 0;
	m_iLanguageListSize = 0;

}

void Tools_Mode_GUIMenu::ManualVerify()
{
	Verify();
}

bool Tools_Mode_GUIMenu::Verify()
{
	bool bRes = true;
	Tools_MessageLog::DeleteMessages(Tools_MessageLog::GUI_MENU,0);

	//Get current position to see if any errors were logged
	u_int uInitialPos = Tools_MessageLog::GetNextID();

	if(m_pxRoot)
	{
		bRes = m_pxRoot->VerifyRecursive();
	}

	if(Tools_MessageLog::AreThereAnyMessages(Tools_MessageLog::GUI_MENU, 0, uInitialPos))
	{
		//Found some errors ,so show message log
		Tools_MessageLog::ShowOverlay(Tools_MessageLog::GUI_MENU,0,uInitialPos);
	}

	return bRes;
}

void Tools_Mode_GUIMenu::AddVerifyMessage(const Tools_GUIMenu_Root* pxItem, const char* szFormat,...)
{
	char szBuffer[512];

	va_list xArgs;
	va_start(xArgs, szFormat);
	Asura_PrintfToBfr(szBuffer,szFormat,xArgs,sizeof(szBuffer),true);
	va_end(xArgs);

	Tools_MessageLog::AddMessage(Tools_MessageLog::GUI_MENU,"",new Tools_MessageLogAction_GUIMenu(pxItem),"%s",szBuffer);
}

void Tools_Mode_GUIMenu::HandleRenameAnimator(const char* szOldName, const char* szNewName)
{
	Asura_Assert(s_pxThis,"");
	if(s_pxThis->m_pxRoot) s_pxThis->m_pxRoot->HandleRenameAnimator(szOldName, szNewName);
}

void Tools_Mode_GUIMenu::HandleDeleteAnimator(const char* szName)
{
	Asura_Assert(s_pxThis,"");
	if(s_pxThis->m_pxRoot) s_pxThis->m_pxRoot->HandleDeleteAnimator(szName);
}


void Tools_Mode_GUIMenu::DoStyleUpdateIfRequired()
{
	//Enforced style update should only need to be done once in a given tools session.
	if(s_bDoneOneOffStyleUpdate) return;

	Asura_Assert(!s_pxThis,"This shouldn't be used while already in menu mode");
	if(s_pxThis) return;

	//create a temporary instance of the mode
	Tools_Mode_GUIMenu* pxMode = new Tools_Mode_GUIMenu;
	pxMode->m_bTemporaryModeInstance = true;
	pxMode->InitialiseMode();
	pxMode->ShutdownMode();
	delete pxMode;
	
}

bool Tools_Mode_GUIMenu::VerifyBeforeExport()
{
	bool bOkay;
	if(s_pxThis)
	{
		bOkay =  s_pxThis->Verify();
	}
	else
	{
		//Verify requires the mode to exist ,so make a temporary copy
		Tools_Mode_GUIMenu* pxMode = new Tools_Mode_GUIMenu;
		pxMode->m_bTemporaryModeInstance = true;
		pxMode->InitialiseMode();
		bOkay = pxMode->Verify();
		pxMode->ShutdownMode();
		delete pxMode;
	}
	return bOkay;
}


bool Tools_Mode_GUIMenu::ChooseText(const Asura_Hash_ID uTextPage, Asura_Hash_ID& uTextID)
{
	const Asura_HashedLocalisedText_Page* pxPage = Asura_HashedLocalisedText::GetPage(uTextPage);
	if(!pxPage || !pxPage->HasTextIDStrings())
	{
		ShowMessageOverlay("No menu strings available");
		return false;
	}

	Asura_Hash_ID uInitialTextID = ( uTextID == ASURA_HASH_ID_UNSET ) ? g_uLastInitialTextID : uTextID;

	if(g_uLastTextPageHash != uTextPage)
	{
		g_uLastInitialTextID	= ASURA_HASH_ID_UNSET;
		g_uLastTextPageHash		= uTextPage;
		g_xTextIDOverlay.Reset();
	}

	//If this is the first time ,then get the list of strings for the overlay
	if(!g_xTextIDOverlay.GetNumItems())
	{
		char szHelpTextBuffer[1024];

		//add option for no text
		g_xTextIDOverlay.AddItem("*None*",0);

		for(int i=0;i<pxPage->GetNumberOfStrings();i++)
		{
			const char* szID = pxPage->GetTextIDStringByIndex(i);
			Asura_Hash_ID uID = Asura_GetHashID(szID);

			const Asura_UnicodeChar* wszString = pxPage->GetString(uID);
			Asura_Assert(wszString,"");

			//convert the full string from unicode to use as help text
			Asura_PrintfToBfr(szHelpTextBuffer,"%S",sizeof(szHelpTextBuffer),true,wszString);

			g_xTextIDOverlay.AddItem(szID,uID,false,szHelpTextBuffer);
		}
	}

	g_xTextIDOverlay.SetInitialSelection(uInitialTextID);

	if(!g_xTextIDOverlay.DoModal()) return false;

	uTextID = g_xTextIDOverlay.GetSelectionData();
	g_uLastInitialTextID = uTextID;								// remember the last text ID we chose, to default to next time
	return true;
}

bool Tools_Mode_GUIMenu::ChooseTextPage(Asura_Hash_ID& uTextPage)
{
	Asura_Hash_ID uInitialTextPage = uTextPage;

	//If this is the first time ,then get the list of strings for the overlay
	if(!g_xTextPageOverlay.GetNumItems())
	{
		const Asura_TemplateHashTree<Asura_String>& xFilenameTree = Asura_GUIMenu_System::GetMenuTextFilenameTree();
		const u_int uNumberOfFiles = xFilenameTree.GetNumberOfNodes();
		for(u_int uFileIndex = 0; uFileIndex < uNumberOfFiles; ++uFileIndex)
		{
			const Asura_Hash_ID uTextPageHash = xFilenameTree.GetHashByIndex(uFileIndex);
			const Asura_String* const pxFilename = xFilenameTree.GetDataByHash(uTextPageHash);
			g_xTextPageOverlay.AddItem(pxFilename->GetString(),uTextPageHash);
		}

		g_xTextPageOverlay.AddItem("*Load New Text Page*", 0);
	}

	g_xTextPageOverlay.SetInitialSelection(uInitialTextPage);

	if(!g_xTextPageOverlay.DoModal()) return false;

	if(g_xTextPageOverlay.GetSelectionData())
	{
		uTextPage = g_xTextPageOverlay.GetSelectionData();
		return true;;
	}

	Asura_Hash_ID uReturnedHash = BrowseForTextPage();

	if (uReturnedHash)
	{
		uTextPage = uReturnedHash;
		TextUpdated();
		return true;
	}
	// Else return false
	return false;
}

Asura_Hash_ID Tools_Mode_GUIMenu::BrowseForTextPage()
{
	//get the user to select a new text file
	Tools_FileSystem_DialogInfo	xFileSelector;
	xFileSelector.DefaultExtension		= ".asr";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= "Misc";
	xFileSelector.FileFilter			= "Asura Files\0*.ASR;\0";
	xFileSelector.WindowTitle			= "Select the the text file to load.";
	xFileSelector.AllowMultipleSelect	= false;

	Asura_String xName = Tools_FileSystem::GetLoadFilename(xFileSelector);
	if(xName.IsBlank()) return ASURA_HASH_ID_UNSET;

	xName.TidyFilename();
	xName.MakeLowercase();

	//make sure the file is in the misc directory
	const char* szMiscDir = strstr(xName.GetString(), "misc\\");
	if(!szMiscDir)
	{
		ShowMessageOverlay("The file needs to be beneath the 'Misc' directory");
		return ASURA_HASH_ID_UNSET;
	}
	//get the name without 'Misc\' and without the language part
	char szStrippedName[MAX_PATH];
	Asura_PrintfToBfr(szStrippedName, "%s", sizeof(szStrippedName), true, szMiscDir+5);

	char* pcUnderscore = strrchr(szStrippedName, '_');
	if(!pcUnderscore)
	{
		ShowMessageOverlay("%s doesn't appear to be the a valid filename for a hashed text file.", xName.GetString());
		return ASURA_HASH_ID_UNSET;
	}
	*pcUnderscore = 0;

	//build the english filename
	char szEnglishFilename[MAX_PATH];
	Asura_PrintfToBfr(szEnglishFilename, "Misc\\%s_en.asr", sizeof(szEnglishFilename), true, szStrippedName);

	//try opening it , and make sure it contains text
	Tools_AsuraFileChunkLister xChunkLister;
	if(!xChunkLister.LoadAsuraFile(szEnglishFilename, true, true))
	{
		ShowMessageOverlay("Failed to load %s", szEnglishFilename);
		return ASURA_HASH_ID_UNSET;
	}

	const Asura_Chunk_HashedLocalisedText* pxChunk = (const Asura_Chunk_HashedLocalisedText*) xChunkLister.GetChunkByID(ASURA_CHUNK_HASHEDLOCALISEDTEXT);
	if(!pxChunk)
	{
		ShowMessageOverlay("Failed to find hashed text chunk within %s", szEnglishFilename);
		return ASURA_HASH_ID_UNSET;
	}

	//look in the chunk for the page ID
	const char* pcChunkBuffer = (const char*)(pxChunk+1);
	Asura_Chunk_ChunkBuffer_Stream xStream(pcChunkBuffer);

	Asura_Hash_ID uPageID = pxChunk->GetPageID(xStream);
	if(!uPageID)
	{
		ShowMessageOverlay("Failed to determine ID of text page within %s", szEnglishFilename);
		return ASURA_HASH_ID_UNSET;
	}

	//Looks valid , so load the page
	Asura_GUIMenu_System::LoadMenuText(szStrippedName, uPageID);

	return uPageID;
}

void Tools_Mode_GUIMenu::TextUpdated()
{
	g_xTextIDOverlay.Reset();
	g_xTextPageOverlay.Reset();
}

void Tools_Mode_GUIMenu::EnableClipping()
{
	Asura_Assert(s_pxThis,"");

	Asura_Vector_2 xSize(s_xScreenSize);
	if (IsPreview() && Tools_GUIMenu_PreviewOverlay::s_bUseVirtualResolution)
	{
		xSize = GetVirtualSize();
	}

	Asura_Bounding_Rectangle xRect(0.0f,0.0f,xSize.x,xSize.y);
	if(IsPreview())
	{
		if(Tools_GUIMenu_PreviewOverlay::s_bCentreScreen)
		{
			xRect.Displace(Asura_Vector_2(Asura_Viewport::CentreX - (xSize.x * 0.5f), Asura_Viewport::CentreY - (xSize.y * 0.5f)));
		}
	}
	else
	{
		//convert the screen area from menu coordinates
		xRect.Displace(s_pxThis->GetScreenBorderOffsetForCurrentMenu());
		xRect = ConvertFromMenuCoords(xRect);
	}


	float fClipMinX = xRect.MinX;
	float fClipMinY = xRect.MinY;
	float fClipWidth = xRect.MaxX - xRect.MinX;
	float fClipHeight = xRect.MaxY - xRect.MinY;

	if(fClipMinX < 0.0f)
	{
		fClipWidth += fClipMinX;
		fClipMinX = 0.0f;
	}
	if(fClipWidth < 0.0f)
	{
		fClipWidth = 0.0f;
	}
	if(fClipMinY < 0.0f)
	{
		fClipHeight += fClipMinY;
		fClipMinY = 0.0f;
	}
	if(fClipHeight < 0.0f)
	{
		fClipHeight = 0.0f;
	}
	
	Asura_Render::Tools_SetClippingRectangle(fClipMinX, fClipMinY, fClipWidth, fClipHeight);
}


void Tools_Mode_GUIMenu::DisableClipping()
{
	Asura_Render::FlushDisplay();
	Asura_Render::AllowFullScreenRendering();
}

void Tools_Mode_GUIMenu::UpdateScreenSettingsFromPreset()
{
	s_xVirtualSize.Zero();
	switch(s_iScreenSettingPreset)
	{
		case TOOLS_GUIMENU_SCREENSETTINGS_CURRENTRES :
		{
			s_xScreenSize.x = (float)Asura_Viewport::GetWidth();
			s_xScreenSize.y = (float)Asura_Viewport::GetHeight();
			s_xSafeArea.MinX = 0.0f;
			s_xSafeArea.MaxX = s_xScreenSize.x;
			s_xSafeArea.MinY = 0.0f;
			s_xSafeArea.MaxY = s_xScreenSize.y;
			s_iPreviewPlatform = TOOLS_PLATFORM_PC;
			break;
		}
		
		case TOOLS_GUIMENU_SCREENSETTINGS_PSP :
		{
			s_xScreenSize.x = 480.0f;
			s_xScreenSize.y = 272.0f;
			s_xSafeArea.MinX = 0.0f;
			s_xSafeArea.MaxX = 480.0f;
			s_xSafeArea.MinY = 0.0f;
			s_xSafeArea.MaxY = 272.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_PSP;
			break;
		}
		
		case TOOLS_GUIMENU_SCREENSETTINGS_PS2_PAL :
		{
			s_xScreenSize.x = 512.0f;
			s_xScreenSize.y = 512.0f;
			//no official area apparently, using 7.5% from edges (taken from rogue)
			s_xSafeArea.MinX = 38.0f;
			s_xSafeArea.MaxX = 474.0f;
			s_xSafeArea.MinY = 38.0f;
			s_xSafeArea.MaxY = 474.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_PS2;
			break;
		}

		case TOOLS_GUIMENU_SCREENSETTINGS_PS2_NTSC :
		{
			s_xScreenSize.x = 512.0f;
			s_xScreenSize.y = 448.0f;
			//no official area apparently, using 7.5% from edges (taken from rogue)
			s_xSafeArea.MinX = 38.0f;
			s_xSafeArea.MaxX = 474.0f;
			s_xSafeArea.MinY = 33.0f;
			s_xSafeArea.MaxY = 415.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_PS2;
			break;
		}

		case TOOLS_GUIMENU_SCREENSETTINGS_PS2_NTSC_WIDE :
		{
			s_xScreenSize.x = 640.0f;
			s_xScreenSize.y = 448.0f;
			//no official area apparently, using 7.5% from edges (taken from rogue)
			s_xSafeArea.MinX = 48.0f;
			s_xSafeArea.MaxX = 592.0f;
			s_xSafeArea.MinY = 33.0f;
			s_xSafeArea.MaxY = 415.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_PS2;
			break;
		}


		case TOOLS_GUIMENU_SCREENSETTINGS_WII_PAL_4_3 :
		{
			s_xScreenSize.x = 640.0f;
			s_xScreenSize.y = 528.0f;
			s_xSafeArea.MinX = 16.0f;
			s_xSafeArea.MaxX = 624.0f;
			s_xSafeArea.MinY = 32.0f;
			s_xSafeArea.MaxY = 496.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_WII;
			s_xVirtualSize.x = 640.0f;
			s_xVirtualSize.y = 480.0f;
			break;
		}
		
		case TOOLS_GUIMENU_SCREENSETTINGS_WII_PAL_16_9 :
		{
			s_xScreenSize.x = 640.0f;
			s_xScreenSize.y = 528.0f;
			s_xSafeArea.MinX = 7.0f;
			s_xSafeArea.MaxX = 633.0f;
			s_xSafeArea.MinY = 14.0f;
			s_xSafeArea.MaxY = 514.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_WII;
			s_xVirtualSize.x = 852.0f;
			s_xVirtualSize.y = 480.0f;
			break;
		}

		case TOOLS_GUIMENU_SCREENSETTINGS_WII_NTSC_4_3 :
		{
			s_xScreenSize.x = 640.0f;
			s_xScreenSize.y = 480.0f;
			s_xSafeArea.MinX = 16.0f;
			s_xSafeArea.MaxX = 624.0f;
			s_xSafeArea.MinY = 46.0f;
			s_xSafeArea.MaxY = 434.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_WII;
			break;
		}
		
		case TOOLS_GUIMENU_SCREENSETTINGS_WII_NTSC_16_9 :
		{
			s_xScreenSize.x = 640.0f;
			s_xScreenSize.y = 480.0f;
			s_xSafeArea.MinX = 7.0f;
			s_xSafeArea.MaxX = 633.0f;
			s_xSafeArea.MinY = 31.0f;
			s_xSafeArea.MaxY = 449.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_WII;
			s_xVirtualSize.x = 852.0f;
			s_xVirtualSize.y = 480.0f;
			break;
		}

		case TOOLS_GUIMENU_SCREENSETTINGS_PS3_4_3 :
		{
			//Middle 85% used
			s_xScreenSize.x = 960.0f;
			s_xScreenSize.y = 720.0f;
			s_xSafeArea.MinX = 72.0f;
			s_xSafeArea.MaxX = 888.0f;
			s_xSafeArea.MinY = 54.0f;
			s_xSafeArea.MaxY = 666.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_PS3;
			s_xVirtualSize.x = 1280.0f;
			s_xVirtualSize.y = 960.0f;
			break;
		}

		case TOOLS_GUIMENU_SCREENSETTINGS_PS3_16_9 :
		{
			//Middle 85% used
			s_xScreenSize.x = 1280.0f;
			s_xScreenSize.y = 720.0f;
			s_xSafeArea.MinX = 96.0f;
			s_xSafeArea.MaxX = 1184.0f;
			s_xSafeArea.MinY = 54.0f;
			s_xSafeArea.MaxY = 666.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_PS3;
			break;
		}


		case TOOLS_GUIMENU_SCREENSETTINGS_XBOX360_4_3_REQUIRED :
		{
			//Middle 90%
			s_xScreenSize.x = 960.0f;
			s_xScreenSize.y = 720.0f;
			s_xSafeArea.MinX = 48.0f;
			s_xSafeArea.MaxX = 912.0f;
			s_xSafeArea.MinY = 36.0f;
			s_xSafeArea.MaxY = 684.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_XBOX360;
			s_xVirtualSize.x = 1280.0f;
			s_xVirtualSize.y = 960.0f;
			break;
		}

		case TOOLS_GUIMENU_SCREENSETTINGS_XBOX360_4_3_RECOMMENDED :
		{
			//Middle 85%
			s_xScreenSize.x = 960.0f;
			s_xScreenSize.y = 720.0f;
			s_xSafeArea.MinX = 72.0f;
			s_xSafeArea.MaxX = 888.0f;
			s_xSafeArea.MinY = 54.0f;
			s_xSafeArea.MaxY = 666.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_XBOX360;
			s_xVirtualSize.x = 1280.0f;
			s_xVirtualSize.y = 960.0f;
			break;
		}

		case TOOLS_GUIMENU_SCREENSETTINGS_XBOX360_16_9_REQUIRED :
		{
			//Middle 90%
			s_xScreenSize.x = 1280.0f;
			s_xScreenSize.y = 720.0f;
			s_xSafeArea.MinX = 64.0f;
			s_xSafeArea.MaxX = 1216.0f;
			s_xSafeArea.MinY = 36.0f;
			s_xSafeArea.MaxY = 684.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_XBOX360;
			break;
		}

		case TOOLS_GUIMENU_SCREENSETTINGS_XBOX360_16_9_RECOMMENDED :
		{
			//Middle 85%
			s_xScreenSize.x = 1280.0f;
			s_xScreenSize.y = 720.0f;
			s_xSafeArea.MinX = 96.0f;
			s_xSafeArea.MaxX = 1184.0f;
			s_xSafeArea.MinY = 54.0f;
			s_xSafeArea.MaxY = 666.0f;
			s_iPreviewPlatform = TOOLS_PLATFORM_XBOX360;
			break;
		}

		default :
			Asura_Assert(0,"Unexpected screen setting preset");
			break;
	}
}


Asura_Vector_2 Tools_Mode_GUIMenu::GetScreenBorderOffsetForCurrentMenu()
{
	Asura_Assert(!s_bMenuPreview, "Not expecting this to be called during menu preview");

	if(m_pxToolsMenu)
	{
		Asura_GUIMenu_Menu_Page* pxMenu = m_pxToolsMenu->GetAsuraMenu();
		
		// NY - 03 October 2008 - centre borders on menu if it's centered and not resizing
		// arguably we want to do *something* when resizing...
		if((pxMenu->GetMenuPageFlags() & ASURA_GUIMENU_MENUPAGE_FLAG_CENTER_MENU_ON_SCREEN) != 0
			&& (pxMenu->GetMenuPageFlags() & ASURA_GUIMENU_MENUPAGE_FLAG_ALLOW_AUTO_RESIZE) == 0)
		{
			Asura_Vector_2 xMenuPos = pxMenu->GetUnscaledLocalPos();
			Asura_Vector_2 xDims = pxMenu->GetUnscaledLocalBounds().GetDimension();

			//get the menu position for centreing on the screen area defined by s_xScreenSize;
			Asura_Vector_2 xNewMenuPos;
			xNewMenuPos.x = (s_xScreenSize.x - xDims.x)*0.5f;
			xNewMenuPos.y = (s_xScreenSize.y - xDims.y)*0.5f;

			//get the reverse offset since we are moving the screen rather than moving the menu when outside of preview
			Asura_Vector_2 xOffset;
			xOffset.x = xMenuPos.x - xNewMenuPos.x;
			xOffset.y = xMenuPos.y - xNewMenuPos.y;

			return xOffset;
		}
	}
	return Asura_Vector_2(0.0f,0.0f);
}

void Tools_Mode_GUIMenu::PrevScreenPreset()
{
	s_iScreenSettingPreset--;
	if(s_iScreenSettingPreset < 0) s_iScreenSettingPreset = TOOLS_GUIMENU_NUM_SCREENSETTINGS - 1;

	UpdateScreenSettingsFromPreset();
	Tools_Timed_Message_System::AddMessage(0xffffff,"%s",g_apszScreenSettingNames[s_iScreenSettingPreset]);

}

void Tools_Mode_GUIMenu::NextScreenPreset()
{
	s_iScreenSettingPreset++;
	if(s_iScreenSettingPreset >= TOOLS_GUIMENU_NUM_SCREENSETTINGS) s_iScreenSettingPreset = 0;

	UpdateScreenSettingsFromPreset();
	Tools_Timed_Message_System::AddMessage(0xffffff,"%s",g_apszScreenSettingNames[s_iScreenSettingPreset]);
}																										

void Tools_Mode_GUIMenu::PrevLanguage()
{
	if(m_iLanguageListSize)
	{
		m_iLanguageIndex--;
		if(m_iLanguageIndex < 0) m_iLanguageIndex = m_iLanguageListSize-1;

		Tools_Timed_Message_System::AddMessage(0xffffff,"%s",m_pszLanguageNames[m_iLanguageIndex]);
		ReloadText();
	}
}

void Tools_Mode_GUIMenu::NextLanguage()
{
	if(m_iLanguageListSize)
	{
		m_iLanguageIndex++;
		if(m_iLanguageIndex >= m_iLanguageListSize) m_iLanguageIndex = 0;

		Tools_Timed_Message_System::AddMessage(0xffffff,"%s",m_pszLanguageNames[m_iLanguageIndex]);
		ReloadText();
	}
}

void Tools_Mode_GUIMenu::FindItemByIDSelectedByUser()
{
	GetString_Overlay xOverlay("Enter ID of item to find (in hex)","");	
	if(!xOverlay.DoModal()) return;

	Asura_Hash_ID uHash = 0;
	sscanf(xOverlay.GetString(),"%x",&uHash);

	Tools_GUIMenu_Root* pxItem = FindItemByID(uHash);
	if(!pxItem)
	{
		ShowMessageOverlay("Failed to find any menu item with that ID");
		return;
	}

	SetSelection(pxItem);
}

void Tools_Mode_GUIMenu::FindMenuByName()
{
	ListSelection_Overlay xOverlay("Select menu to find");

	//get a list of all menu names
	for(u_int i=0;i<Asura_GUIMenu_System::GetCurrentMenuListCount();i++)
	{
		Asura_GUIMenu_Root* pxAsuraItem = Asura_GUIMenu_System::GetMenuPageByIndex(i);

		//Skipping any menus that haven't got a filename , in case there are any that have been created in code
		const char* szFilename = ((Asura_GUIMenu_Menu_Page*)pxAsuraItem)->GetFilename();
		if(!szFilename || !*szFilename) continue;

		xOverlay.AddItem(pxAsuraItem->GetNameAsChar(), pxAsuraItem->GetID());
	}

	if(!xOverlay.GetNumItems())
	{
		ShowMessageOverlay("Failed to find any menus");
		return;
	}

	if(!xOverlay.DoModal()) return;	

	//select the selected menu
	SetSelectionByID(xOverlay.GetSelectionData());

}

void Tools_Mode_GUIMenu::FindByTexture()
{
	//Determine the texture to be found.
	int iTexture = Tools_GUIMenu_Element_Image::SelectTexture(ASURA_TEXTURE_NONE, "Find menu items using which texture?");	
	if(iTexture == ASURA_TEXTURE_NONE) return;

	u_int uInitialPos = Tools_MessageLog::GetNextID();
	
	const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTexture);
	Asura_Assert(pxTextureName, "");
	Tools_MessageLog::AddMessage(Tools_MessageLog::GUI_MENU,"Find", NULL,"Menu items using '%s'", pxTextureName->GetString());
	
	//Check alll the tools menu items for any using this texture
	bool bAnyFound = false;
	for(CLIF<Tools_GUIMenu_Root*> xItemLIF(Tools_GUIMenu_Root::GetList()); !xItemLIF.done(); xItemLIF.next())
	{
		const Tools_GUIMenu_Root* pxItem = xItemLIF();
		if(pxItem->UsesTexture(iTexture))
		{
			Tools_MessageLog::AddMessage(Tools_MessageLog::GUI_MENU,"Find",new Tools_MessageLogAction_GUIMenu(pxItem),"%s",pxItem->GetName());
			bAnyFound = true;
		}
	}

	if(!bAnyFound)
	{
		Tools_MessageLog::AddMessage(Tools_MessageLog::GUI_MENU,"Find", NULL,"None found.");
	}

	Tools_MessageLog::ShowOverlay(Tools_MessageLog::GUI_MENU,"Find",uInitialPos);
	
}

void Tools_Mode_GUIMenu::ListTextModifiers( int iValue )
{
	const Tools_GUIMenu_Root* pxSelected = NULL;
	switch( iValue )
	{
	case eLIST_ALL_TEXT_MODIFIERS:
		break;
	case eLIST_TEXT_MODIFIERS_CURRENTLY_SELECTED_MENU:
		pxSelected = GetMenu( GetSelectedItem() );
		break;
	case eLIST_TEXT_MODIFIERS_CURRENTLY_SELECTED_FILE:
		pxSelected = GetFile( GetSelectedItem() );
		break;
	}

	const u_int uInitialPos = Tools_MessageLog::GetNextID();
	int iFoundCount	= 0;
	for( CLIF<Tools_GUIMenu_Root*> xItemLIF( Tools_GUIMenu_Root::GetList() ); !xItemLIF.done(); xItemLIF.next() )
	{
		Tools_GUIMenu_Root* const pxToolsItem = xItemLIF();
		if( pxToolsItem == NULL )
		{
			continue;
		}
		const Asura_GUIMenu_Root* const pxAsuraRoot = pxToolsItem->GetAsuraData();
		if( pxAsuraRoot == NULL )
		{
			continue;
		}
		const int iModifierCount = pxAsuraRoot->GetModifierCount();
		if( iModifierCount == 0 )
		{
			continue;
		}

		bool bProcessed = false;
		Tools_GUIMenu_Root* pxStyleMasterCopy = NULL;
		for( int i = 0; i < iModifierCount; i++ )
		{
			char szModifierText[ 256 ];
			bool bTextModifier = false;
			const Asura_GUIMenu_Modifier* const pxModifier = pxAsuraRoot->GetModifier( i );
			const ASURA_GUIMENU_MODIFY_TYPE xType = pxModifier->GetModifyType(); 
			if( xType == Asura_GUIMenu_Modifier::DATA_TYPE_CHAR_STRING )
			{
				Asura_PrintfToBfr( szModifierText, "%s", sizeof( szModifierText ), true, pxModifier->GetCharStringData() );
				bTextModifier = true;
			}
			else if( xType == Asura_GUIMenu_Modifier::DATA_TYPE_UNICODE_STRING )
			{
				Asura_PrintfToBfr( szModifierText, "%S", sizeof( szModifierText ), true, pxModifier->GetUnicodeCharStringData() );
				bTextModifier = true;
			}
			if( bTextModifier )
			{
				if( !bProcessed )
				{
					const Tools_GUIMenu_Root* pxItem;
					// restricted to file or menu?
					if( pxSelected != NULL )
					{
						pxItem = pxToolsItem;
						while( ( pxItem != NULL ) && ( pxItem != pxSelected ) )
						{
							pxItem = pxItem->GetParent();
						}
						if( pxItem != pxSelected )
						{
							break;
						}
					}
					// is this a style instance
					pxItem = pxToolsItem;
					while( pxItem != NULL )
					{
						if( ( pxItem->GetAsuraData() != NULL ) && ( pxItem->IsStyleInstance() || pxItem->IsGeneratedByStyle() ) )
						{
							Asura_Hash_ID xStyleObjectHash = pxItem->GetAsuraData()->GetStyleObjectHash();
							pxStyleMasterCopy = Tools_Mode_GUIMenu::GetInstance()->FindItemByID( xStyleObjectHash );
							if( pxStyleMasterCopy != NULL )
							{
								break;
							}
						}													  
						pxItem = pxItem->GetParent();
					}
					bProcessed = true;
				}
				Tools_MessageLog::AddMessage( Tools_MessageLog::GUI_MENU, "Find", new Tools_MessageLogAction_GUIMenu( pxToolsItem ), "%s = %s", pxToolsItem->GetName(), szModifierText );
				if( pxStyleMasterCopy != NULL )
				{
					Tools_MessageLog::AddMessage( Tools_MessageLog::GUI_MENU, "Find", new Tools_MessageLogAction_GUIMenu( pxStyleMasterCopy ), "%s = %s -> Style Master Copy", pxToolsItem->GetName(), szModifierText );
				}
				iFoundCount++;
			}
		}
	}
	if( iFoundCount == 0 )
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::GUI_MENU, "Find", NULL, "None found." );
	}
	else
	{
		Tools_MessageLog::AddMessage( Tools_MessageLog::GUI_MENU, "Find", NULL, "%d found.", iFoundCount );
	}

	Tools_MessageLog::ShowOverlay( Tools_MessageLog:: GUI_MENU, "Find", uInitialPos );
}

bool Tools_Mode_GUIMenu::IsEnabledForCurrentlySelectedMenu( void )
{
	return( GetMenu( GetSelectedItem() ) != NULL );
}
bool Tools_Mode_GUIMenu::IsEnabledForCurrentlySelectedFile( void )
{
	return( GetFile( GetSelectedItem() ) != NULL );
}


void Tools_Mode_GUIMenu::Undo()
{
	if(m_xUndo.Undo())
	{
		
		//Force the rectangles of all the items to be set again , so that the local positions are recalculated
		m_pxRoot->SetRectangleRecursive();
		
		//The undo may have changed some styles , which will in turn cause other menu items to be updated.
		//Handle the update now , but with undo logging turned off.
		//Otherwise it would never be possible to undo more than one step
		//RAR 20/09/2007

		m_bDisableUndoLogging = true;
		HandleStyleUpdates();
		m_bDisableUndoLogging = false;
	}
}

void Tools_Mode_GUIMenu::Redo()
{
	if(m_xUndo.Redo())
	{
		//Force the rectangles of all the items to be set again , so that the local positions are recalculated
		m_pxRoot->SetRectangleRecursive();

		//The undo may have changed some styles , which will in turn cause other menu items to be updated.
		//Handle the update now , but with undo logging turned off.
		//Otherwise it would never be possible to undo more than one step
		//RAR 20/09/2007

		m_bDisableUndoLogging = true;
		HandleStyleUpdates();
		m_bDisableUndoLogging = false;
	}
}


void Tools_Mode_GUIMenu::LogUndo_Rectangle(const Tools_GUIMenu_Root* pxItem, const Asura_Bounding_Rectangle& xOldRect)
{
	if(!s_pxThis) return;
	if(s_pxThis->m_bDisableUndoLogging) return;

	Asura_Assert(pxItem,"");

	RootUndoData** ppxUndo = s_pxThis->m_xUndo.AllocateNextUndoData();
	if(ppxUndo) *ppxUndo = new Tools_GUIMenu_Undo_Rectangle(pxItem, xOldRect);
}


void Tools_Mode_GUIMenu::EditTextureFlags()
{
	List<int> xTextureList;
	Asura_GUIMenu_System::TraverseUsedTextures(Tools_GUIMenu_Element_Image::ListTexturesCallback,&xTextureList);

	bool bModified = false;

	Tools_PickTexture::SetTextureFilterCallback(Tools_GUIMenu_Element_Image::IsMenuTexture,&xTextureList);
	Tools_PickTexture::EditTextureFlags(PICKTEXTURE_USECALLBACKFILTER, &bModified);
	Tools_PickTexture::SetTextureFilterCallback(0);

	if(bModified)
	{
		Tools_GUIMenu_System::SetTextureFlagsModified();
	}

}


#endif //ASURA_USE_GUI_MENU_SYSTEM
