//
// Tools_HDR_Overlay.cpp
//
// Created by Andrew Southgate - 19 Feb 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//



#include "Core/Asura.h"
#ifdef ASURA_USE_HDR
#include "Core/Asura_Maths.h"

#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"

#include "Rendering/Asura_Direct3D_HDR.h"
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Viewport.h"

#include "Tools_Overlays.h"

#include "Asura_HDRHistogram_Dialog.h"

static Asura_Direct3D_Effect*	s_pxEffect;
static D3DXHANDLE				s_xDataTextureHandle;

static D3DXHANDLE				s_xShowRedHandle;
static D3DXHANDLE				s_xShowGreenHandle;
static D3DXHANDLE				s_xShowBlueHandle;
static D3DXHANDLE				s_xShowLumHandle;
static D3DXHANDLE				s_xXScaleHandle;


void Asura_HDRHistogram_Gadget::Initialise()
{
	s_pxEffect = Asura_Direct3D_Effects_System::GetEffect("OverlayQuad_HDRHistogram");
	Asura_Assert(s_pxEffect, "Unable to load effect OverlayQuad_HDRHistogram");

	if (s_pxEffect)
	{
		s_xDataTextureHandle = s_pxEffect->GetD3DEffect()->GetParameterByName(NULL, "g_xDataTexture");

		s_xShowRedHandle = s_pxEffect->GetD3DEffect()->GetParameterByName(NULL, "g_bShowRed");
		s_xShowGreenHandle = s_pxEffect->GetD3DEffect()->GetParameterByName(NULL, "g_bShowGreen");
		s_xShowBlueHandle = s_pxEffect->GetD3DEffect()->GetParameterByName(NULL, "g_bShowBlue");
		s_xShowLumHandle = s_pxEffect->GetD3DEffect()->GetParameterByName(NULL, "g_bShowLum");

		s_xXScaleHandle = s_pxEffect->GetD3DEffect()->GetParameterByName(NULL, "g_fXScale");

	}
}




Asura_HDRHistogram_Gadget::Asura_HDRHistogram_Gadget( const Asura_Vector_2& xOffset,
													 const float fWidth,
													 const float fHeight,
													 bool &bShowRedChannel,
													 bool &bShowGreenChannel,
													 bool &bShowBlueChannel,
													 bool &bShowLumChannel
													 )
: m_bShowRedChannel(bShowRedChannel), m_bShowGreenChannel(bShowGreenChannel), 
m_bShowBlueChannel(bShowBlueChannel), m_bShowLumChannel(bShowLumChannel)
{
	m_xOffset = xOffset;
	m_fWidth = fWidth / ASURA_TEXT_OUTPUT_FONT_WIDTH;
	m_fHeight = fHeight / ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	m_uCurrentTexture = 0;
	Asura_Direct3D::D3DDevice->CreateTexture(256, 1, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &m_pDataTexture[0], NULL);
	Asura_Direct3D::D3DDevice->CreateTexture(256, 1, 1, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &m_pDataTexture[1], NULL);
	D3DLOCKED_RECT xLockRect;
	m_pDataTexture[0]->LockRect(0, &xLockRect, NULL, 0);
	memset(xLockRect.pBits, 0xFF, 1024);
	m_pDataTexture[0]->UnlockRect(0);
	m_pDataTexture[1]->LockRect(0, &xLockRect, NULL, 0);
	memset(xLockRect.pBits, 0xFF, 1024);
	m_pDataTexture[1]->UnlockRect(0);

	m_fRange = 1.0f;
}
Asura_HDRHistogram_Gadget::~Asura_HDRHistogram_Gadget()
{
	SAFE_RELEASE(m_pDataTexture[0]);
	SAFE_RELEASE(m_pDataTexture[1]);
}


void Asura_HDRHistogram_Gadget::Update( const Asura_Vector_2& xOrigin )
{

}

void Asura_HDRHistogram_Gadget::UpdateHDR( const Asura_Direct3D_HDR::HDR_Histogram_Data &xData, float fRange )
{
	D3DLOCKED_RECT xLockRect;
	m_uCurrentTexture ^= 1;
	m_pDataTexture[m_uCurrentTexture]->LockRect(0, &xLockRect, NULL, 0);

	u_int *pData = (u_int *)xLockRect.pBits;
	for (int i = 0; i < 256; ++i, ++pData)
	{
		u_int uRed = xData.m_uRHeight ? xData.m_auR[i] * 255 / xData.m_uRHeight : 0;
		u_int uGreen = xData.m_uGHeight ? xData.m_auG[i] * 255 / xData.m_uGHeight : 0;
		u_int uBlue = xData.m_uBHeight ? xData.m_auB[i] * 255 / xData.m_uBHeight : 0;
		u_int uLum = xData.m_uLHeight ? xData.m_auL[i] * 255 / xData.m_uLHeight : 0;

		*pData = (uLum << 24) | (uRed << 16) | (uGreen << 8) | uBlue;
	}
	m_pDataTexture[m_uCurrentTexture]->UnlockRect(0);

	m_fRange = fRange;

}


////////////////////////////////////////////////////////////////////////////////
//
// Asura_HDRHistogram_Gadget::Render()
//
// Renders an HDRHistogram gadget
//
// Parameters:		xOrigin - Screen position of the parent overlay, measured in
//							  Asura_Text_Output units
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Asura_HDRHistogram_Gadget::Render( const Asura_Vector_2& xOrigin )
{
	Asura_Vector_2 xTopLeft = xOrigin;
	xTopLeft += m_xOffset;
	xTopLeft.x *= ASURA_TEXT_OUTPUT_FONT_WIDTH;
	xTopLeft.y *= ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	Asura_Vector_2 xDimensions( m_fWidth, m_fHeight );
	xDimensions.x *= ASURA_TEXT_OUTPUT_FONT_WIDTH;
	xDimensions.y *= ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	extern u_int Asura_OverlayBrightness;

	Asura_Render::Overlay.Colour = Asura_OverlayBrightness | GetColour( IsClickedThisFrame() );
	Asura_Render::Overlay.m_bGouraudShaded = false;
	Asura_Render::Overlay.m_bNonRotated = true;
	Asura_Render::Overlay.TextureIndex = -1;
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.m_eTextureOp = ASURA_TEXTUREOP_SINGLE_PASS;

	// Since override effects might use vertex shaders, we have to pass in
	// untransformed vertices. That means taking them from screen-space to
	// [-1,1] clip space
	{
		const float fViewportWidth = static_cast< float >( Asura_Viewport::GetWidth() );
		const float fViewportHeight = static_cast< float >( Asura_Viewport::GetHeight() );

		xTopLeft.x = (xTopLeft.x / fViewportWidth) * 2.0f - 1.0f;
		xTopLeft.y = 1.0f - (xTopLeft.y / fViewportHeight) * 2.0f;
		xDimensions.x = (xDimensions.x / fViewportWidth) * 2.0f;
		xDimensions.y = -(xDimensions.y / fViewportHeight) * 2.0f;
	}
	s_pxEffect->SetParameter(s_xShowRedHandle, m_bShowRedChannel);
	s_pxEffect->SetParameter(s_xShowGreenHandle, m_bShowGreenChannel);
	s_pxEffect->SetParameter(s_xShowBlueHandle, m_bShowBlueChannel);
	s_pxEffect->SetParameter(s_xShowLumHandle, m_bShowLumChannel);

	if (m_fRange <= .015625f) {
		s_pxEffect->SetParameter(s_xXScaleHandle, m_fRange * 256.0f);
	} else if (m_fRange <= .0625f) {
		s_pxEffect->SetParameter(s_xXScaleHandle, m_fRange * 64.0f);
	} else if (m_fRange <= .25f) {
		s_pxEffect->SetParameter(s_xXScaleHandle, m_fRange * 16.0f);
	} else if (m_fRange <= 1.0f) {
		s_pxEffect->SetParameter(s_xXScaleHandle, m_fRange * 4.0f);
	} else if (m_fRange <= 4.0f) {
		s_pxEffect->SetParameter(s_xXScaleHandle, m_fRange);
	} else if (m_fRange <= 16.0f) {
		s_pxEffect->SetParameter(s_xXScaleHandle, m_fRange * 1.0f / 4.0f);
	} else {
		s_pxEffect->SetParameter(s_xXScaleHandle, m_fRange * 1.0f / 16.0f);
	}


	s_pxEffect->SetParameter(s_xDataTextureHandle, m_pDataTexture[m_uCurrentTexture]);

	Asura_Render::Overlay.SetRectXY( xTopLeft.x, xTopLeft.y, xDimensions.x, xDimensions.y );
	Asura_Render::Overlay.SetRectUV( 0.0f, 0.0f, 1.0f, 1.0f );
	Asura_Direct3D::AddQuadToBatch( &Asura_Render::Overlay, s_pxEffect );
	// Unfortunately we need to force a flush as we set params on the effect directly
	// which would screw up if multiple quads were batched
	Asura_Direct3D::FlushQuadBatches();


}


Asura_HDRHistogram_Dialog::Asura_HDRHistogram_Dialog(const char* const szName, const Asura_Vector_2& xOffset, float fWidth, float fHeight, bool bAfterToneMap)
 : Asura_Dialog( szName, xOffset )
{
	Asura_Vector_2 xGadgetOffset = xOffset;
	xGadgetOffset.y += 1.5f;

	m_bShowRedChannel = true;
	m_bShowGreenChannel = true;
	m_bShowBlueChannel = true;
	m_bShowLumChannel = true;

	m_bAfterToneMap = bAfterToneMap;
	float fDelta;
	const char *pFmt;
	if (!m_bAfterToneMap)
	{
		m_fRange = 4.0f;
		m_fRangeMax = 32.0f;
		m_fRangeMin = 1.0f;
		fDelta = 0.1f;
		pFmt = "%.1f";
	} else {
		m_fRange = 1.0f;
		m_fRangeMax = 1.0f;
		m_fRangeMin = 0.01f;
		fDelta = 0.01f;
		pFmt = "%.2f";
	}


	AddGadget(new Asura_BoolTickbox_Gadget(xGadgetOffset, &m_bShowRedChannel, NULL));
	xGadgetOffset.x += Asura_BoolTickbox_Gadget::WIDTH;
	Asura_StringDisplay_Gadget *pStringGadget;
	AddGadget(pStringGadget = new Asura_StringDisplay_Gadget(xGadgetOffset, 0, NULL, NULL, 0, ASURA_TEXT_FORMAT_SMALLFONT));
	pStringGadget->SetLabel("R");
	xGadgetOffset.x += 1.5;
	AddGadget(new Asura_BoolTickbox_Gadget(xGadgetOffset, &m_bShowGreenChannel, NULL));
	xGadgetOffset.x += Asura_BoolTickbox_Gadget::WIDTH;
	AddGadget(pStringGadget = new Asura_StringDisplay_Gadget(xGadgetOffset, 0, NULL, NULL, 0, ASURA_TEXT_FORMAT_SMALLFONT));
	pStringGadget->SetLabel("G");
	xGadgetOffset.x += 1.5;
	AddGadget(new Asura_BoolTickbox_Gadget(xGadgetOffset, &m_bShowBlueChannel, NULL));
	xGadgetOffset.x += Asura_BoolTickbox_Gadget::WIDTH;
	AddGadget(pStringGadget = new Asura_StringDisplay_Gadget(xGadgetOffset, 0, NULL, NULL, 0, ASURA_TEXT_FORMAT_SMALLFONT));
	pStringGadget->SetLabel("B");
	xGadgetOffset.x += 1.5;
	AddGadget(new Asura_BoolTickbox_Gadget(xGadgetOffset, &m_bShowLumChannel, NULL));
	xGadgetOffset.x += Asura_BoolTickbox_Gadget::WIDTH;
	AddGadget(pStringGadget = new Asura_StringDisplay_Gadget(xGadgetOffset, 0, NULL, NULL, 0, ASURA_TEXT_FORMAT_SMALLFONT));
	pStringGadget->SetLabel("L");
	xGadgetOffset.x = xOffset.x;
	xGadgetOffset.y += Asura_Dialog::DefaultSpacing();

	// Texture picker gadget
	m_pxGadget = new Asura_HDRHistogram_Gadget( xGadgetOffset, fWidth, fHeight, m_bShowRedChannel, m_bShowGreenChannel, m_bShowBlueChannel, m_bShowLumChannel );
	xGadgetOffset.y += m_pxGadget->m_fHeight + .5f;
	AddGadget( m_pxGadget );

	AddGadget(new Asura_NumberSlider_Gadget(xGadgetOffset,&m_fRange, &m_fRangeMin, &m_fRangeMax, m_pxGadget->m_fWidth - 10.0f , "", NULL));

	xGadgetOffset.x += m_pxGadget->m_fWidth - 8.0f;
	Asura_Gadget *newGadget  = new Asura_IncrementorFloat_Gadget(xGadgetOffset,&m_fRange,m_fRangeMin,m_fRangeMax,-fDelta,NULL);
	AddGadget(newGadget);

	xGadgetOffset.x += newGadget->m_fWidth*2;

	newGadget  = new Asura_Float_Gadget(xGadgetOffset,&m_fRange,m_fRangeMin,m_fRangeMax,pFmt,NULL);
	AddGadget(newGadget);

	xGadgetOffset.x += newGadget->m_fWidth;

	newGadget  = new Asura_IncrementorFloat_Gadget(xGadgetOffset,&m_fRange,m_fRangeMin,m_fRangeMax,+fDelta,NULL);
	AddGadget(newGadget);

	xGadgetOffset.x = xOffset.x;
	xGadgetOffset.y += Asura_Dialog::DefaultSpacing();




	m_xMaxCoord.x = xGadgetOffset.x + m_pxGadget->m_fWidth;
	m_xMaxCoord.y = xGadgetOffset.y + 0.5f;

}

Asura_HDRHistogram_Dialog::~Asura_HDRHistogram_Dialog()
{
	if (m_bAfterToneMap)
	{
		Asura_Direct3D_HDR::SetPostToneMapHDRHistogramCallback(NULL, 0);
	} else {
		Asura_Direct3D_HDR::SetPreToneMapHDRHistogramCallback(NULL, 0);
	}
}

void Asura_HDRHistogram_Dialog::Update( const Asura_Vector_2& xOrigin )
{
	Asura_Dialog::Update(xOrigin);

}

void Asura_HDRHistogram_Dialog::Render( const Asura_Vector_2& xOrigin )
{
	Asura_Dialog::Render(xOrigin);
	if (m_bShowRedChannel || m_bShowGreenChannel || m_bShowBlueChannel || m_bShowLumChannel)
	{
		if (m_bAfterToneMap)
		{
			Asura_Direct3D_HDR::SetPostToneMapHDRHistogramCallback(HDR_CB, (u_int)this);
		} else {
			Asura_Direct3D_HDR::SetPreToneMapHDRHistogramCallback(HDR_CB, (u_int)this);
		}
	} else {
		if (m_bAfterToneMap)
		{
			Asura_Direct3D_HDR::SetPostToneMapHDRHistogramCallback(NULL, 0);
		} else {
			Asura_Direct3D_HDR::SetPreToneMapHDRHistogramCallback(NULL, 0);
		}
	}

}

void Asura_HDRHistogram_Dialog::HDR_CB(const Asura_Direct3D_HDR::HDR_Histogram_Data &xData)
{
	((Asura_HDRHistogram_Dialog*)xData.m_uAppDefined)->UpdateHDR(xData);
}

void Asura_HDRHistogram_Dialog::UpdateHDR(const Asura_Direct3D_HDR::HDR_Histogram_Data &xData)
{
	m_pxGadget->UpdateHDR(xData, m_fRange);
	if (m_bAfterToneMap)
	{
		Asura_Direct3D_HDR::SetPostToneMapHDRHistogramRange(m_fRange);
	} else {
		Asura_Direct3D_HDR::SetPreToneMapHDRHistogramRange(m_fRange);
	}
}


#endif // defined(ASURA_USE_HDR)
