//
// Tools_HDR_Overlay.cpp
//
// Created by Andrew Southgate - 19 Feb 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//



#include "Core/Asura.h"
#if defined ASURA_USE_HDR
#include "Core/Asura_Maths.h"

#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"

#include "Rendering/Asura_Direct3D_HDR.h"
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"

#include "Tools_Overlays.h"

#include "Tools_HDR_Overlay.h"
#include "Asura_HDRHistogram_Dialog.h"


bool	Tools_HDR_Overlay::s_bShowRedChannel;
bool	Tools_HDR_Overlay::s_bShowBlueChannel;
bool	Tools_HDR_Overlay::s_bShowGreenChannel;
bool	Tools_HDR_Overlay::s_bShowLumChannel;



Asura_Overlay* Tools_HDR_Overlay::s_pxOverlay = NULL;
Asura_Vector_2 Tools_HDR_Overlay::s_xOverlayPosition(1.0f, 3.0f);



void Tools_HDR_Overlay::Initialise( void )
{
	s_pxOverlay = 0;
	Asura_HDRHistogram_Gadget::Initialise();


}

void Tools_HDR_Overlay::Reset( void )
{
}

void Tools_HDR_Overlay::Shutdown( void )
{
	Reset();

	CloseOverlay();
}

void Tools_HDR_Overlay::ToggleOverlay( void )
{
	if ( IsOverlayOpen() )
	{
		CloseOverlay();
	}
	else
	{
		CreateOverlay();
	}
}

bool Tools_HDR_Overlay::IsOverlayOpen( void )
{
	if ( s_pxOverlay )
	{
		if ( Asura_Overlay_System::DoesOverlayExist( TOOLS_OVERLAY_HDR ) )
		{
			return true;
		}

		s_pxOverlay = 0;
	}

	return false;
}

void Tools_HDR_Overlay::CreateOverlay( void )
{
	if ( s_pxOverlay ) return;


	//	Asura_Dialog* pxNewDialog;
	//	Asura_Tickbox_Dialog* pxNewTickbox;

	// Store the overlay...
	s_pxOverlay		= new Asura_Overlay( "HDR", false, &s_xOverlayPosition );
	s_pxOverlay->m_uID	= TOOLS_OVERLAY_HDR;

	Asura_Vector_2	xOffset(1.0f, 2.5f);

	Asura_Dialog *pxDialog = new Asura_HDRHistogram_Dialog("Before Tonemap", xOffset, 256, 256);
	xOffset.x = ceilf(pxDialog->m_xMaxCoord.x + 1.5f);
	s_pxOverlay->AddDialog( pxDialog );

	pxDialog = new Asura_HDRHistogram_Dialog("After Tonemap", xOffset, 256, 256, true);
	xOffset.x = 1.0f;
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.5f;
	s_pxOverlay->AddDialog( pxDialog );

	// and then give it to the overlay system to look after 
	Asura_Overlay_System::SubmitNewOverlay( s_pxOverlay, true );

}

void Tools_HDR_Overlay::CloseOverlay( void )
{
	s_pxOverlay = 0;

	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_HDR );
}

#endif