//
// Tools_HelperObject_System.cpp
//
// Created by Richard Rice - 01/09/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "HelperObject/Tools_HelperObject_System.h"

//asura
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_NameTree.h"
#include "Core/Asura_String.h"
#include "FileAccess/Asura_FileAccess.h"

//tools
#include "HelperObject/Tools_HelperObject.h"
#include "Misc/list_tem.hpp"
#include "PC/FileSystem/Tools_FileSystem.h"

//includes not required by plugins
#ifdef ASURA_MAIN

//asura
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Tools_Chunk_HelperObject.h"

//tools
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_LoadedFileList.h"
#include "Misc/Tools_Timed_Messages.h"

#endif //ASURA_MAIN

List<Tools_HelperObject*> Tools_HelperObject_System::s_xList;
bool Tools_HelperObject_System::s_bLoaded = false;

#ifdef ASURA_MAIN
List<Asura_String> Tools_HelperObject_System::s_xModifiedFiles;
#endif

static const char* g_szExtension = ".HelperObject";


void Tools_HelperObject_System::Initialise()
{
}

void Tools_HelperObject_System::Shutdown()
{
	while(s_xList.size())
	{
		delete s_xList.first_entry();
		s_xList.delete_first_entry();
	}
	#ifdef ASURA_MAIN
	while(s_xModifiedFiles.size())
	{
		s_xModifiedFiles.delete_first_entry();
	}
	#endif

	s_bLoaded = false;
}

Tools_HelperObject* Tools_HelperObject_System::AllocateHelperObject()
{
	Tools_HelperObject* pxHelper = new Tools_HelperObject;
	s_xList.add_entry(pxHelper);
	return pxHelper;
}

void Tools_HelperObject_System::DeleteHelperObject(Tools_HelperObject* pxHelper)
{
	if(!pxHelper) return;
	Asura_Assert(s_xList.contains(pxHelper),"");

	s_xList.delete_entry(pxHelper);
	delete pxHelper;
}

const List<Tools_HelperObject*>* Tools_HelperObject_System::GetList()
{
	#ifdef ASURA_MAIN
	//Load the files if not done yet
	if(!s_bLoaded) Load();
	#endif
	return &s_xList;
}


void Tools_HelperObject_System::Load(const char* szAsuraFilename)
{
	s_bLoaded = true;

	//get the asura file's directory
	Asura_String xAsuraFilename = szAsuraFilename;
	Asura_String xDir;
	xAsuraFilename.SplitFilename(&xDir,0,0);

	List<Asura_String> xFileList;

	//Search for all helper object files in that dir
	//Find local files
	char szSearch[500];
	Asura_PrintfToBfr(szSearch,"%s\\*%s",sizeof(szSearch),true,xDir.GetString(),g_szExtension);

	Tools_FileSystem::FindFiles(szSearch, xFileList);

	#ifdef ASURA_PC_ALLOW_LOAD_FROM_NETWORK
	//Find network files
	Asura_FileAccess::PlatformGetNetworkDir(xDir.GetString(),szSearch,sizeof(szSearch)-25,false);
	strcat(szSearch,"\\*");
	strcat(szSearch,g_szExtension);

	Tools_FileSystem::FindFiles(szSearch, xFileList);
	#endif //ASURA_PC_ALLOW_LOAD_FROM_NETWORK

	Asura_NameTree xNameTree;

	//load the files in the list
	for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next())
	{
		//make sure this file hasn't already been dealt with
		const char* szName = Tools_FileSystem::StripPath(xFileLIF().GetString());

		int iJunkIndex;
		if(xNameTree.FindNodeIndexByName(szName,true,iJunkIndex)) continue;

		LoadAsuraFile(xFileLIF().GetString());

		#ifdef ASURA_MAIN
		//Add to files to be tracked for external changes
		Tools_LoadedFileList::TrackExternalModification(xFileLIF().GetString());
		#endif
	}
}


#ifdef ASURA_MAIN

void Tools_HelperObject_System::Load()
{
	Load(Tools_SaveAsura::GetMainFilename());
}

bool Tools_HelperObject_System::IsModified()
{
	return (s_xModifiedFiles.size()>0);
}

void Tools_HelperObject_System::SaveAll()
{
	//Not required by exports
	if(Tools_Level_Export::IsExport()) return;

	int iJunkIndex;
	Asura_NameTree xFileTree;

	//attempt to save all the modified files
	//(this ensures that any files that have lost all their objects will get saved)
	for(LIF<Asura_String> xModifiedLIF(&s_xModifiedFiles);!xModifiedLIF.done();)
	{
		//keep track of the files done
		xFileTree.FindNodeIndexByName(xModifiedLIF().GetString(),true,iJunkIndex);

		if(SaveFile(xModifiedLIF().GetString()))
		{
			xModifiedLIF.delete_current();
		}
		else
		{
			xModifiedLIF.next();
		}
	}

	//scan through all objects for other files to save
	for(LIF<Tools_HelperObject*> xHelperLIF(&s_xList);!xHelperLIF.done();xHelperLIF.next())
	{
		
		const char* szFilename = xHelperLIF()->m_xFile.GetString();

		if(!xFileTree.FindNodeIndexByName(szFilename,true,iJunkIndex))
		{
			SaveFile(szFilename);
		}
	}

}

void Tools_HelperObject_System::SaveModified()
{
	//Not required by exports
	if(Tools_Level_Export::IsExport()) return;

	for(LIF<Asura_String> xModifiedLIF(&s_xModifiedFiles);!xModifiedLIF.done();)
	{
		if(SaveFile(xModifiedLIF().GetString()))
		{
			xModifiedLIF.delete_current();
		}
		else
		{
			xModifiedLIF.next();
		}
	}
}

void Tools_HelperObject_System::SetModified(Tools_HelperObject* pxHelper)
{
	Asura_Assert(pxHelper,"");

	//get the filename
	const char* szFilename = pxHelper->m_xFile.GetString();

	//file already in list?
	for(LIF<Asura_String> xModifiedLIF(&s_xModifiedFiles);!xModifiedLIF.done();xModifiedLIF.next())
	{
		if(!xModifiedLIF().Compare(szFilename)) return;
	}

	//File wasn't already flagged as modified ,so check whether its going to be okay to write to it
	char szFullFilename[MAX_PATH];
	GetFullFilename(szFilename,szFullFilename,MAX_PATH);

	Tools_SaveAsura::WarnIfReadOnly(szFullFilename);	

	s_xModifiedFiles.add_entry(szFilename);
}

void Tools_HelperObject_System::GetFullFilename(const char* szFilename,char* szBuffer,int iBufferSize)
{
	Asura_String xAsuraFilename = Tools_SaveAsura::GetMainFilename();
	Asura_String xDir;
	xAsuraFilename.SplitFilename(&xDir,0,0);
	

	Asura_PrintfToBfr(szBuffer,"%s\\%s%s",iBufferSize,true,xDir.GetString(),szFilename,g_szExtension);
}

bool Tools_HelperObject_System::SaveFile(const char* szFilename)
{
	if(Tools_Level_Export::IsExport()) return false;

	char szFullFilename[MAX_PATH];
	GetFullFilename(szFilename,szFullFilename,MAX_PATH);
	if(!Tools_SaveAsura::CanSaveFile(szFullFilename,true))return false;


	Asura_SaveBlock xSaveBlock;

	//add chunks for all the objects in this file
	for(LIF<Tools_HelperObject*> xHelperLIF(&s_xList);!xHelperLIF.done();xHelperLIF.next())
	{
		if(!stricmp(xHelperLIF()->m_xFile.GetString(),szFilename))
		{
			Tools_Chunk_HelperObject::Create(xHelperLIF(),&xSaveBlock);
		}
	}

	//write out the save block

	xSaveBlock.AddAsuraTerminator();
	Tools_FileSystem::EnsureDirectoryExists(szFullFilename);
	bool bSuccess = xSaveBlock.SaveFile(szFullFilename,true);
    if(bSuccess)
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved %s.",szFullFilename);
		Tools_LoadedFileList::UpdateModifiedTime(szFullFilename);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save %s",szFullFilename);
	}

	return bSuccess;	
}

bool Tools_HelperObject_System::CanSaveToFile(const char* szName)
{
	char szFullFilename[MAX_PATH];
	GetFullFilename(szName,szFullFilename,MAX_PATH);

	return Tools_SaveAsura::CanSaveFile(szFullFilename,false);
}

#endif //ASURA_MAIN