
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Overlay_Hierarchy_ViewFacialAnimation.h
// Author : Andy Tate
// Date   : 20-May-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_OVERLAY_HIERARCHY_VIEWFACIALANIMATION_H__
#define __TOOLS_OVERLAY_HIERARCHY_VIEWFACIALANIMATION_H__

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Table_Dialog.h"

#include "FacialAnimation/Tools_FacialAnimation_AnimationHandler_HierarchyMode.h"


////////////////////////////////////////////////////////////////////////////////
// Forward class declarations...
////////////////////////////////////////////////////////////////////////////////

class Asura_FacialAnimation_AnimationHandler;

class Tools_FacialAnimation_Face;
class Tools_FacialAnimation_Face_MuscleGroup;


////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_Overlay_Hierarchy_ViewFacialAnimation
// Author       : Andy Tate
// Date         : 20-May-2008
//
// Description  : Class used in hierarchy-mode to view (and play with) facial-
//				  animation data.
////////////////////////////////////////////////////////////////////////////////

class Tools_Overlay_Hierarchy_ViewFacialAnimation : public Asura_Overlay
{
public:
	// Public functions...
	static void	CreateOverlay(void);
	static void	CloseOverlay(void);

	static bool	OverlayExists(void)		{ return(s_pxThis != NULL); }
	static void	ToggleOverlay(void)		{ if(OverlayExists()) CloseOverlay(); else CreateOverlay(); }

	// Overridden functions...
	virtual void	PerFrameUpdate(void);

	// Accessor functions...
	Asura_FacialAnimation_AnimationHandler*	GetAnimationHandler(void) const	{ return(m_pxAnimationHandler); }

	//Play/stop facial animations that are linked to body animations
	static void PlayLinkedFacialAnimation(const class Asura_Hierarchy_Anim* pxAnim);

protected:
	// Protected variables...

	// Protected functions...
	Tools_Overlay_Hierarchy_ViewFacialAnimation(void);
	virtual ~Tools_Overlay_Hierarchy_ViewFacialAnimation();

	void	AddOverlayControls(Asura_Vector_2& xOffset);

	// Face manipulation functions...
	void								SetCurrentFace(const Asura_Hash_ID uFaceHash, const bool bUpdateAnimationHandler);
	const Tools_FacialAnimation_Face*	GetCurrentFace(void) const;

	const Asura_Hash_ID					GetCurrentFaceHash(void) const	{return m_uCurrentFaceHash;}

	// Creation functions...
	static Asura_TableCell_Dialog* CreateTableCell(Asura_Table_Dialog& xTableDialog, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, void* const pxUserData);

	// Event functions...
	static void	OnSelectFaceSelected(int iUserData);
	static void OnReloadAnimationsSelected(int iUserData);

	static void OnPlayDialogueLineSelected(int iUserData);
	static void OnStopDialogueLineSelected(int iUserData);
	static void OnPlayFacialAnimationSelected(int iUserData);
	static void OnStopFacialAnimationSelected(int iUserData);

	static void	OnApplyToSkinSelected(int iUserData);
	static void	OnResetWeightsAndBalancesSelected(int iUserData);
	static void	OnLoadPoseSelected(int iUserData);

	static void	OnSaveAsPoseSelected(int iUserData);

	// Query functions...
	static bool	CanSelectFace(int iUserData);
	static bool	CanPlayDialogueLine(int iUserData);
	static bool	CanStopDialogueLine(int iUserData);
	static bool	CanBeAppliedToSkin(int iUserData);
	static bool CanPlayFacialAnimation(int iUserData);
	static bool CanStopFacialAnimation(int iUserData);
	static bool	IsFacialAnimationAppliedToSkin(int iUserData);


private:
	// Private enumerated types...
	enum TableColumn
	{
		TC_MUSCLE_GROUP,
		TC_CONTROLS,

		MAX_TABLE_COLUMNS
	};

	// Private typedefs...
	typedef Asura_Overlay PARENT;

	// Private variables...
	Asura_Hash_ID							m_uCurrentFaceHash;			// The hash-ID of the face we're currently using.
	bool									m_bApplyToSkin;				// Flag to allow this data to be applied to the current skin.
	u_int									m_uNumMuscleGroups;			// The number of muscle-groups for this face.
	const char**							m_pszMuscleGroupNames;		// A pointer to the array of names for this face's muscle-groups.
	u_int									m_uNumEyeValues;			// Should be either zero or ASURA_FACIALANIMATION_EYE_NUMPARAM depending on whether eyes are used by the face 

	Asura_Dialog*							m_pxSelectFaceDialogue;		// A pointer to the dialogue used when selecting a face.
	Asura_Tickbox_Dialog*					m_pxApplyToSkinDialogue;	// A pointer to the dialogue used to decide whether to apply the animation to the current skin.
	Asura_Table_Dialog*						m_pxTableDialogue;			// A pointer to the table dialogue to use.

	Tools_FacialAnimation_AnimationHandler_HierarchyMode*	m_pxAnimationHandler;		// A pointer to the animation-handler to use when updating.
	Asura_Handle							m_iDialogueLineHandle;		// The handle used when playing lines of dialogue.

	Asura_Hash_ID m_uFaceAnimHashId;
	Asura_String m_xAnimString;

	// Static variables...
	static const char*									s_aszTableColumnNames[MAX_TABLE_COLUMNS];	// Array of names for the table columns used.
	static Tools_Overlay_Hierarchy_ViewFacialAnimation*	s_pxThis;									// We only ever create a single instance of this overlay.

	// Private functions...
	Tools_Overlay_Hierarchy_ViewFacialAnimation(const Tools_Overlay_Hierarchy_ViewFacialAnimation& xOther);

	Tools_Overlay_Hierarchy_ViewFacialAnimation&	operator=(const Tools_Overlay_Hierarchy_ViewFacialAnimation& xOther);

};   // End of class 'Tools_Overlay_Hierarchy_ViewFacialAnimation'.



////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_TableCell_MuscleGroupApplyControlType_Dialog
// Author       : Andy Tate
// Date         : 20-May-2008
//
// Description  : Class used to handle creation of the dialogue to apply
//				  controls from a specific muscle-group to the current skin
//				  (in Tools hierarchy mode).
////////////////////////////////////////////////////////////////////////////////

class Tools_TableCell_MuscleGroupApplyControlType_Dialog : public Asura_TableCell_Dialog
{
public:
	// Public functions...
	Tools_TableCell_MuscleGroupApplyControlType_Dialog(const Tools_FacialAnimation_Face_MuscleGroup& xMuscleGroup, Asura_Table_Dialog& xTableDialogue, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, Tools_Overlay_Hierarchy_ViewFacialAnimation& xOverlay);
//	virtual ~Tools_TableCell_MuscleGroupApplyControlType_Dialog();

	// Overridden functions...
	virtual int		Compare(const Asura_TableCell_Dialog& xOther) const	{ return(0); }


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private typedefs...
	typedef Asura_TableCell_Dialog PARENT;

	// Private variables...
	Tools_Overlay_Hierarchy_ViewFacialAnimation&	m_xOverlay;			// A reference to the overlay we're a part of.
	const Tools_FacialAnimation_Face_MuscleGroup&		m_xMuscleGroup;		// A reference to the muscle-group we're using for this dialogue.

	// Static variables...
	static float	s_fUnusedWeight;	// Weight value used when we can't find the animation-handler.
	static float	s_fUnusedBalance;	// Balance value used when we can't find the animation-handler.

	// Private functions...
	Tools_TableCell_MuscleGroupApplyControlType_Dialog(void);
	Tools_TableCell_MuscleGroupApplyControlType_Dialog(const Tools_TableCell_MuscleGroupApplyControlType_Dialog& xOther);

	Tools_TableCell_MuscleGroupApplyControlType_Dialog&	operator=(const Tools_TableCell_MuscleGroupApplyControlType_Dialog& xOther);

};   // End of class 'Tools_TableCell_MuscleGroupApplyControlType_Dialog'.

////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_TableCell_EyeValueApplyControlType_Dialog
// Author       : Richard Rice
// Date         : 30-Oct-2008
//
// Description  : Class used to handle creation of the dialogue to apply
//				  controls from a specific eye value to the current skin
//				  (in Tools hierarchy mode).
////////////////////////////////////////////////////////////////////////////////

class Tools_TableCell_EyeValueApplyControlType_Dialog : public Asura_TableCell_Dialog
{
public:
	// Public functions...
	Tools_TableCell_EyeValueApplyControlType_Dialog(u_int uEyeValueIndex, Asura_Table_Dialog& xTableDialogue, const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn, Tools_Overlay_Hierarchy_ViewFacialAnimation& xOverlay);
//	virtual ~Tools_TableCell_EyeValueApplyControlType_Dialog();

	// Overridden functions...
	virtual int		Compare(const Asura_TableCell_Dialog& xOther) const	{ return(0); }


protected:
	// Protected variables...

	// Protected functions...


private:
	// Private typedefs...
	typedef Asura_TableCell_Dialog PARENT;

	// Private variables...
	Tools_Overlay_Hierarchy_ViewFacialAnimation&	m_xOverlay;			// A reference to the overlay we're a part of.
	u_int m_uEyeValueIndex;

	// Static variables...
	static float	s_fUnusedWeight;	// Weight value used when we can't find the animation-handler.

	// Private functions...
	Tools_TableCell_EyeValueApplyControlType_Dialog(void);
	Tools_TableCell_EyeValueApplyControlType_Dialog(const Tools_TableCell_EyeValueApplyControlType_Dialog& xOther);

	Tools_TableCell_EyeValueApplyControlType_Dialog&	operator=(const Tools_TableCell_EyeValueApplyControlType_Dialog& xOther);

};   // End of class 'Tools_TableCell_EyeValueApplyControlType_Dialog'.


#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

#endif   // ifndef __TOOLS_OVERLAY_HIERARCHY_VIEWFACIALANIMATION_H__


/// End of file. ///
