//
// Tools_MarkerPoint_Object.cpp
//
// Created by Richard Rice - 13/09/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Hierarchy/Tools_MarkerPoint_Object.h"

#include "Animation/Asura_AnimationAttachment.h"
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_UserInput.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/PFX/Asura_PFX_Enums.h"

#include "Tools_Overlays.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Hierarchy/Tools_Mode_Hierarchy.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_IniFile.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Movement/Tools_Movement.h"
#include "Objects/Tools_Object_System.h"
#include "Particles/Tools_PFX_Dialog.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Shapes/Tools_Shapes.h"



////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------**
** Tools_SaveMarkerConfig_Overlay **
**--------------------------------*/

class Tools_SaveMarkerConfig_Overlay : public Tools_Modal_Overlay
{
public :
	Tools_SaveMarkerConfig_Overlay(const List<Tools_MarkerPoint_Object*>& xObjectList)
	:Tools_Modal_Overlay("Save which attachments?")
	{
		m_pxObjectList = &xObjectList;
	}
	bool DoModal();

private :
	virtual void InitialiseOverlay(class Asura_Overlay* pxOverlay);

	void AddTickboxForMarker(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset,Tools_MarkerPoint_Object* pxObject);

	static void OkaySelected(int iThis)
	{((Tools_SaveMarkerConfig_Overlay*) iThis)->m_bOkay = true;}

	static void CancelSelected(int iThis)
	{((Tools_SaveMarkerConfig_Overlay*) iThis)->m_bOkay = false;}


	const List<Tools_MarkerPoint_Object*>* m_pxObjectList;
	bool m_bOkay;
};

bool Tools_SaveMarkerConfig_Overlay::DoModal()
{
	//and show the overlay
	ShowOverlay();

	return m_bOkay;
}

void Tools_SaveMarkerConfig_Overlay::InitialiseOverlay(Asura_Overlay* pxOverlay)
{
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	for(CLIF<Tools_MarkerPoint_Object*> xObjectLIF(m_pxObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		AddTickboxForMarker(pxOverlay,xOffset,xObjectLIF());
		
	}

	xOffset.y += 1.0f;

	pxDialog = new Asura_Function_Dialog("Okay",xOffset,OkaySelected,(int)this,&pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,CancelSelected,(int)this,&pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);

	
}

void Tools_SaveMarkerConfig_Overlay::AddTickboxForMarker(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset,Tools_MarkerPoint_Object* pxMarker)
{
	pxMarker->m_bSaveToConfig = pxMarker->GetShapeID() || pxMarker->GetSkinID();

	if(pxMarker->m_uPFXTemplate) pxMarker->m_bSaveToConfig = true;

	if(!pxMarker->IsHidden())
	{
		Asura_String xName;
		pxMarker->GetPathName(xName);

		Asura_Dialog* pxDialog = new Asura_Tickbox_Dialog(xName.GetString(),xOffset,&pxMarker->m_bSaveToConfig);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	//add for children
	for(LIF<Tools_MarkerPoint_Object*> xChildLIF(&pxMarker->m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		AddTickboxForMarker(pxOverlay,xOffset,xChildLIF());
	}
}



////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------**
** Tools_MarkerPoint_Object **
**--------------------------*/

Asura_Vector_2 Tools_MarkerPoint_Object::s_xOverlayPosition(10,10);
Tools_MarkerPoint_Object* Tools_MarkerPoint_Object::s_pxThis = 0; 

List<Tools_MarkerPoint_Config*> Tools_MarkerPoint_Object::s_xMarkerConfigList;

bool Tools_MarkerPoint_Object::s_b90DegreeRotation = false;

const char* g_pszMarkerConfigFile = "ToolsIniData/MarkerPoint.ini";

Tools_MarkerPoint_Object::Tools_MarkerPoint_Object(Asura_Animation* pxAnimation,const char* pszName,int iIndex,Tools_MarkerPoint_Object* pxParent)
:Tools_Object(TOT_MarkerPoint)
{
	m_xName = pszName;
	m_pxParentAnimation = pxAnimation;
	m_uMarkerHash = pxAnimation->GetSkin()->GetMarkerPointHashID(iIndex);
	m_pxAttachedAnimation = 0;
	m_bInUpdateMovement = false;
	m_uShapeID = ASURA_HASH_ID_UNSET;
	m_bShowChildMarkerPoints = false;
	m_pxParent = pxParent;

	m_uPFXTemplate = 0;
	m_bPFXChanged = false;
	m_pxAttachedPFX = 0;
	m_bMovePFXWithMarker = true;

	m_uAnimationFlags = AsuraAnimationFlag_Loop;
	m_bAnimFlagsChanged = false;
	m_ChangeData = false;

	Asura_String xMeshName = "ToolsShape_CutsceneTrackPoint";
	m_pxDefaultMesh = Mesh = Asura_Mesh_System::GetMesh(xMeshName);

	Asura_Hierarchy_Marker_Point* pxMarker = GetMarkerPoint();
	Asura_Assert(pxMarker, "");
	m_xInitialPosition = pxMarker->Position;
	m_xInitialOrientation = pxMarker->Matrix;

}

Tools_MarkerPoint_Object::~Tools_MarkerPoint_Object()
{
	RemoveAttachment();
}

void Tools_MarkerPoint_Object::Render()
{
	if(!m_pxAttachedAnimation)
	{
		Tools_Object::Render();
	}
	
	if(IsSelected() || IsGroupSelected())
	{
		//show some coloured lines to represent the axes
		Asura_Render::RenderAxes( Position, Orientation, 0.4f );
	}

}

void Tools_MarkerPoint_Object::UpdatePositionFromAsuraData()
{
	if(m_bInUpdateMovement) return;

	m_pxParentAnimation->GetMarkerLocationFromHash(m_uMarkerHash,&Position,&Orientation,true);

	//if(m_pxParent)
	//{
	//	Position = m_pxParent->Position + (Position*m_pxParent->Orientation);
	//	Orientation = m_pxParent->Orientation*Orientation;
	//}
	//else
	//{
	//	Position += Tools_Mode_Hierarchy::GetHierarchyPosition();
	//}

	for(LIF<Tools_MarkerPoint_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		xChildLIF()->UpdatePositionFromAsuraData();
	}

}

void Tools_MarkerPoint_Object::UpdateMovement(float* pMovementScale,float* pRotationScale)
{

	if(!s_b90DegreeRotation)
	{
		Asura_Hierarchy_Marker_Point* pxMarker = GetMarkerPoint();
		if(pxMarker)
		{
			//set position and orientaiton to be the marker points relative position/orientation
			Position = pxMarker->Position;
			Orientation = pxMarker->Matrix;
			m_bInUpdateMovement = true;
			Tools_Object::UpdateMovement(pMovementScale,pRotationScale);
			m_bInUpdateMovement = false;

			if(Position != pxMarker->Position || Orientation != pxMarker->Matrix)
			{
				pxMarker->Position = Position;
				pxMarker->Matrix = Orientation;
				SetParentSkinModified();

			}

			UpdatePositionFromAsuraData();
		}
	}
	else
	{
		Asura_Euler_3 xEuler(0,0,0);
		//rotate marker point in 90 degree increments
		if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus() && !Tools_Movement::DisableNumberPadControls)
		{
			if(!Asura_UserInput::KeyPressed(ASURA_KEYID_ALT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
			{
				if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_NUMPAD4)) //yaw left
				{
					xEuler.y -= Asura_Maths::Deg90;
				}
				if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_NUMPAD6)) //yaw right
				{
					xEuler.y += Asura_Maths::Deg90;
				}
				if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_NUMPAD2)) //pitch down
				{
					xEuler.x -= Asura_Maths::Deg90;
				}
				if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_NUMPAD8)) //pitch up
				{
					xEuler.x += Asura_Maths::Deg90;
				}
				if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_NUMPAD7)) //roll left
				{
					xEuler.z -= Asura_Maths::Deg90;
				}
				if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_NUMPAD9)) //roll right
				{
					xEuler.z += Asura_Maths::Deg90;
				}
			}
		}
		if(Tools_Movement::InvertYAxis)
		{
			//invert pitch
			xEuler.x = -xEuler.x;
		}

		Asura_Matrix_3x3 xMat;
		xEuler.ToMatrix(xMat);

		Asura_Hierarchy_Marker_Point* pxMarker = GetMarkerPoint();
		if(pxMarker)
		{
			pxMarker->Matrix = pxMarker->Matrix*xMat;
		}

		SetParentSkinModified();
	}
 
}

void Tools_MarkerPoint_Object::Update()
{
	//update visibility of child marker points
	for(LIF<Tools_MarkerPoint_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		xChildLIF()->SetHiddenFlag(ObjectHideFlag_Mode,!m_bShowChildMarkerPoints);
	}

	if(m_bPFXChanged)
	{
		AddPFXAttachment();
		m_bPFXChanged = false;
	}

	if(m_bAnimFlagsChanged)
	{
		UpdateAnimationFlags();
		m_bAnimFlagsChanged = false;
	}

	if ( m_ChangeData )
	{
		m_ChangeData = false;
		OnDeselection();
		OnSelection();
	}
}

void Tools_MarkerPoint_Object::OnSelection()
{
	s_pxThis = this;
	CreateOverlay();
}

void Tools_MarkerPoint_Object::OnDeselection()
{
	DeleteOverlay();
	s_pxThis = 0;
}

bool Tools_MarkerPoint_Object::IsHidden()
{
	//If a skin/shape has been added as an attachement here , don't hide the object
	if(Mesh!=m_pxDefaultMesh || m_pxAttachedAnimation) return false;

	return Tools_Object::IsHidden();
}

void Tools_MarkerPoint_Object::CreateOverlay()
{
	Asura_Overlay* pxOverlay;
	char szText[255];

	Asura_PrintfToBfr(szText, "%s - %s", 255, true, GetTypeDescription(), m_xName);
	pxOverlay = new Asura_Overlay(szText,false,&s_xOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	Asura_PrintfToBfr(szText, "Add Attachment : Shape -- %s", 255, true, Tools_ShapeTable::LookupObjectMeshName(m_uShapeID));
	
	pxDialog = new Asura_Function_Dialog(szText,xOffset,ChooseShape, &m_ChangeData );
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	Asura_PrintfToBfr(szText, "Add Attachment : Skin -- %s", 255, true, m_xSkinName);

	pxDialog = new Asura_Function_Dialog(szText,xOffset,ChooseSkin, &m_ChangeData);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	const char* szAnimName = "";
	if ( m_pxAttachedAnimation )
	{
		const Asura_Hash_ID uHash = m_pxAttachedAnimation->m_xPrimaryWrapper.GetPrimaryAnimHashID();

		const Asura_Hierarchy_Anim* const pxAnim = Asura_Animation_System::GetAnimByHash(uHash);
		if (pxAnim)
		{
			szAnimName = pxAnim->GetName();
		}
	}

	Asura_PrintfToBfr(szText, "Add Attachment : Anim -- %s", 255, true, szAnimName);
	pxDialog = new Asura_Function_Dialog(szText,xOffset,ChooseAnimation, &m_ChangeData);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove Attachment",xOffset,_RemoveAttachment);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Reverse",xOffset,&m_uAnimationFlags,AsuraAnimationFlag_Reversed,&m_bAnimFlagsChanged);
	pxDialog->SetHelpText("Reverses the animation.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Mirrored",xOffset,&m_uAnimationFlags,AsuraAnimationFlag_Mirror,&m_bAnimFlagsChanged);
	pxDialog->SetHelpText("Mirrors the animation.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Tools_PFXSelect_Dialog(Tools_PFXSelect_Dialog::MODE_EFFECTS, xOffset,&m_uPFXTemplate,&m_bPFXChanged);
	pxDialog->SetHelpText("Adds a PFX attachment to this marker point for testing purposes.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Move PFX with marker",xOffset,&m_bMovePFXWithMarker,&m_bPFXChanged);
	pxDialog->SetHelpText("Existing particles get moved as the marker point moves.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Rename",xOffset,Rename);
	pxDialog->SetHelpText("Renames the marker point.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Reset Position",xOffset,ResetPosition);
	pxDialog->SetHelpText("Resets the marker points position and orientation back to the original values.\n(A limited form of movement undo.)");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("90 Degree Rotation",xOffset,&s_b90DegreeRotation);
	pxDialog->SetHelpText("If ticked then each press of a rotation key will turn the marker point 90 degrees.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Child Markers",xOffset,&m_bShowChildMarkerPoints);
	pxDialog->SetHelpText("If ticked then marker point objects will be shown for any marker points belonging to the skin attached here.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	
	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;

 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
	
}

void Tools_MarkerPoint_Object::DeleteOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
}

void Tools_MarkerPoint_Object::ChooseSkin()
{
	Asura_Assert(s_pxThis,"");

	Asura_Hierarchy_Skin* pxSkin = 0;
	Asura_Hierarchy_Anim* pxAnim = 0;

	//Choose a skin and animation
	{
		Asura_String xSkinName;
		pxSkin = Tools_Animation_Files::ChooseSkin(0,&xSkinName);
		if(!pxSkin) return;

		pxAnim = Tools_Animation_Files::ChooseAnimation(pxSkin->Name.GetString(), NULL, NULL, NULL, NULL, true);

		s_pxThis->RemoveAttachment();
		s_pxThis->m_xSkinName = xSkinName;
		s_pxThis->m_pxAttachedAnimation = new Asura_Animation(pxSkin->Name.GetString());
	}
	Asura_Hash_ID uAnimHash = pxAnim ? pxAnim->GetHashID() : 0;
	s_pxThis->m_pxAttachedAnimation->m_xPrimaryWrapper.StartAnim2(uAnimHash,s_pxThis->m_uAnimationFlags,0.0f);
	s_pxThis->m_pxParentAnimation->LinkAnimation(s_pxThis->m_pxAttachedAnimation,s_pxThis->m_xName.GetString(),false);

	s_pxThis->CreateChildMarkerPoints();
}

void Tools_MarkerPoint_Object::ChooseAnimation()
{
	Asura_Assert(s_pxThis,"");

	if(!s_pxThis->m_pxAttachedAnimation)
	{
 		Tools_Timed_Message_System::AddMessage(0xffffff,"Please select a skin first");
		return;
	}

	Asura_Hierarchy_Skin* pxSkin = s_pxThis->m_pxAttachedAnimation->GetSkin();
	if(!pxSkin) return;

	Asura_Hash_ID uCurrentAnimHash = s_pxThis->GetAnimID();
	Asura_Hierarchy_Anim* pxCurrentAnim = uCurrentAnimHash ? Asura_Animation_System::GetAnimByHash(uCurrentAnimHash) : 0;

	Asura_Hierarchy_Anim* pxAnim = Tools_Animation_Files::ChooseAnimation(pxSkin->Name.GetString(), NULL, NULL, NULL, pxCurrentAnim, true);
	if(pxAnim == pxCurrentAnim) return;

	Asura_Hash_ID uAnimHash = pxAnim ? pxAnim->GetHashID() : 0;
	s_pxThis->m_pxAttachedAnimation->m_xPrimaryWrapper.StartAnim2(uAnimHash,s_pxThis->m_uAnimationFlags,0.0f);
}


void Tools_MarkerPoint_Object::ChooseShape()
{
	Asura_Assert(s_pxThis,"");
	
	Asura_Hash_ID uShapeID = Tools_ShapeTable::SelectShape(s_pxThis->m_uShapeID);
	if(uShapeID == ASURA_HASH_ID_UNSET) return;

	s_pxThis->SetShape(uShapeID);
}
	
 

void Tools_MarkerPoint_Object::_RemoveAttachment()
{
	Asura_Assert(s_pxThis,"");
	s_pxThis->RemoveAttachment();
}


void Tools_MarkerPoint_Object::RemoveAttachment()
{
	Mesh = m_pxDefaultMesh;
	m_uShapeID = ASURA_HASH_ID_UNSET;

	while(m_xChildList.size())
	{
		Tools_Object_System::DeleteObject(m_xChildList.first_entry());
		m_xChildList.delete_first_entry();
	}
	if(m_pxAttachedAnimation)
	{
		m_pxParentAnimation->UnlinkAnimation(m_pxAttachedAnimation);
		delete m_pxAttachedAnimation;
		m_pxAttachedAnimation = 0;
	}
	m_xSkinName.MakeBlank();

	RemovePFXAttachment();

}

void Tools_MarkerPoint_Object::ResetPosition()
{
	Asura_Assert(s_pxThis,"");
	
	Asura_Hierarchy_Marker_Point* pxMarker = s_pxThis->GetMarkerPoint();
	if(pxMarker)
	{
		pxMarker->Matrix = s_pxThis->m_xInitialOrientation;
		pxMarker->Position = s_pxThis->m_xInitialPosition;
	}

	s_pxThis->SetParentSkinModified();
}

void Tools_MarkerPoint_Object::SetShape(Asura_Hash_ID uShapeID)
{
	if(uShapeID == ASURA_HASH_ID_UNSET)
	{
		RemoveAttachment();
		return;
	}


	//shape already in shape table?
	const char* pszMeshName = Tools_ShapeTable::LookupObjectMeshName(uShapeID);
	if(!pszMeshName)
	{
		//get the filename for this shape and add it
		const char* pszFilename = Asura_HashedFilename_System::GetFilename(uShapeID,false);
		if(!pszFilename)
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to determine filename for shape");
			return;
		}
		Tools_ShapeTable::AddShapeByFilename(pszFilename);

		pszMeshName = Tools_ShapeTable::LookupObjectMeshName(uShapeID);
		if(!pszMeshName) return;
	}

	Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(pszMeshName);
	if(!pxMesh)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to get mesh for selected shape.");
		return;
	}

	//okay , we have the shape so use it
	RemoveAttachment();
	m_uShapeID = uShapeID;
	Mesh = pxMesh;
	
}

void Tools_MarkerPoint_Object::SetSkin(Asura_Hash_ID uSkinID,Asura_Hash_ID uAnimID)
{
	Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::GetSkinFromID(uSkinID);
	if(!pxSkin) return;

	if(uAnimID) Tools_Animation_Files::GetAnimFromID(uAnimID);
	
	RemoveAttachment();

	m_xSkinName = Asura_HashedFilename_System::GetFullFilename(uSkinID);

	m_pxAttachedAnimation = new Asura_Animation(pxSkin->Name.GetString());
    	
	m_pxAttachedAnimation->m_xPrimaryWrapper.StartAnim2(uAnimID,m_uAnimationFlags,0.0f);
	m_pxParentAnimation->LinkAnimation(m_pxAttachedAnimation,m_xName.GetString(),false);

	CreateChildMarkerPoints();

}

Asura_Hash_ID Tools_MarkerPoint_Object::GetShapeID()
{
	return m_uShapeID;
}

Asura_Hash_ID Tools_MarkerPoint_Object::GetSkinID()
{
	if(!m_pxAttachedAnimation) return ASURA_HASH_ID_UNSET;
	return Asura_GetHashID(m_pxAttachedAnimation->GetSkin()->Name.GetString());

}

Asura_Hash_ID Tools_MarkerPoint_Object::GetAnimID()
{
	if(!m_pxAttachedAnimation) return ASURA_HASH_ID_UNSET;
	return m_pxAttachedAnimation->m_xPrimaryWrapper.GetPrimaryAnimHashID();
}


void Tools_MarkerPoint_Object::LoadMarkerPointConfigFile()
{
	DeleteMarkerPointConfigs();

	Tools_IniFile xIni(g_pszMarkerConfigFile,false);
	if(!xIni.LoadIniFile()) return;

	//get a list of section names , which will correspond to the 'config' names
	List<char*> xSections = xIni.GetSectionNameList();

	for(LIF<char*> xSectLIF(&xSections);!xSectLIF.done();xSectLIF.next())
	{
		xIni.GetIniTextSection(xSectLIF());

		Tools_MarkerPoint_Config* pxConfig = new Tools_MarkerPoint_Config;
		s_xMarkerConfigList.add_entry(pxConfig);

		pxConfig->m_xName = xSectLIF();

		for(int iIndex = 0;;iIndex++)
		{
			char szKey[20];
			sprintf(szKey,"Name%d",iIndex);
			
			Asura_String xMarkerName;
			if(!xIni.GetStringInCurrentSection(szKey,xMarkerName)) break;

			Tools_MarkerPoint_Settings* pxMarker = new Tools_MarkerPoint_Settings;
			pxConfig->m_xMarkerList.add_entry(pxMarker);

			pxMarker->m_xMarkerName = xMarkerName;

			sprintf(szKey,"Shape%d",iIndex);
			xIni.GetIntInCurrentSection(szKey,*(int*)&pxMarker->m_uShapeID);
			sprintf(szKey,"Skin%d",iIndex);
			xIni.GetIntInCurrentSection(szKey,*(int*)&pxMarker->m_uSkinID);
			sprintf(szKey,"Anim%d",iIndex);
			xIni.GetIntInCurrentSection(szKey,*(int*)&pxMarker->m_uAnimID);

			sprintf(szKey,"PFX%d",iIndex);
			xIni.GetIntInCurrentSection(szKey,*(int*)&pxMarker->m_uPFXTemplate);
			sprintf(szKey,"MovePFX%d",iIndex);
			xIni.GetBoolInCurrentSection(szKey,pxMarker->m_bMovePFXWithMarker);
		}
	}	
}

void Tools_MarkerPoint_Object::SaveMarkerPointConfigFile()
{
	Tools_FileSystem::MakeWriteable(g_pszMarkerConfigFile);
	FILE* pFile = fopen(g_pszMarkerConfigFile,"wt");
	if(!pFile) return;

	for(LIF<Tools_MarkerPoint_Config*> xConfigLIF(&s_xMarkerConfigList);!xConfigLIF.done();xConfigLIF.next())
	{
		Tools_MarkerPoint_Config* pxConfig = xConfigLIF();

		fprintf(pFile,"[%s]\n",pxConfig->m_xName.GetString());

		int iIndex = 0;
		for(LIF<Tools_MarkerPoint_Settings*> xMarkerLIF(&pxConfig->m_xMarkerList);!xMarkerLIF.done();xMarkerLIF.next(),iIndex++)
		{
			Tools_MarkerPoint_Settings* pxMarker = xMarkerLIF();

			fprintf(pFile,"Name%d = %s\n",iIndex,pxMarker->m_xMarkerName.GetString());
			
			if(pxMarker->m_uShapeID != ASURA_HASH_ID_UNSET)
			{
				fprintf(pFile,"Shape%d = %d\n",iIndex,pxMarker->m_uShapeID);
			}
			if(pxMarker->m_uSkinID != ASURA_HASH_ID_UNSET)
			{
				fprintf(pFile,"Skin%d = %d\n",iIndex,pxMarker->m_uSkinID);
			}
			if(pxMarker->m_uAnimID != ASURA_HASH_ID_UNSET)
			{
				fprintf(pFile,"Anim%d = %d\n",iIndex,pxMarker->m_uAnimID);
			}
			if(pxMarker->m_uPFXTemplate)
			{
				fprintf(pFile,"PFX%d = %d\n",iIndex,pxMarker->m_uPFXTemplate);
				fprintf(pFile,"MovePFX%d = %s\n",iIndex,pxMarker->m_bMovePFXWithMarker ? "True" : "False");
				
			}
		}
		fprintf(pFile,"\n");
		
	}
	fclose(pFile);
}

void Tools_MarkerPoint_Object::DeleteMarkerPointConfigs()
{
	while(s_xMarkerConfigList.size())
	{
		delete s_xMarkerConfigList.first_entry();
		s_xMarkerConfigList.delete_first_entry();
	}
}

void Tools_MarkerPoint_Object::StoreMarkerPointSettings(const List<Tools_MarkerPoint_Object*>& xMarkerList)
{
	//ask which attachments to save
	Tools_SaveMarkerConfig_Overlay xOverlay(xMarkerList);
	if(!xOverlay.DoModal()) return;

	//ask for name of config
	
	GetString_Overlay xStringOverlay("Save under what name?","");
	if(!xStringOverlay.DoModal()) return;

	Asura_String xName = xStringOverlay.GetString();
	if(xName.IsBlank()) return;

	//name already used?
	for(LIF<Tools_MarkerPoint_Config*> xConfigLIF(&s_xMarkerConfigList);!xConfigLIF.done();xConfigLIF.next())
	{
		if(!stricmp(xConfigLIF()->m_xName.GetString(),xName.GetString()))
		{
			if(!ShowYesNoOverlay("That name has already been used.\nOverwrite existing attachment set?"))
			{
				return;
			}
			delete xConfigLIF();
			xConfigLIF.delete_current();
			break;
		}
	}

	//create and fill in the new set
	Tools_MarkerPoint_Config* pxConfig = new Tools_MarkerPoint_Config;
	s_xMarkerConfigList.add_entry(pxConfig);
	pxConfig->m_xName = xName;

	for(CLIF<Tools_MarkerPoint_Object*> xObjectLIF(&xMarkerList);!xObjectLIF.done();xObjectLIF.next())
	{
		xObjectLIF()->SaveToConfigIfFlagged(pxConfig);
	}

	SaveMarkerPointConfigFile();

}

void Tools_MarkerPoint_Object::RestoreMarkerPointSettings(const List<Tools_MarkerPoint_Object*>& xMarkerList)
{
	if(!s_xMarkerConfigList.size())
	{
		return;
	}
	
	//ask for set to use
	ListSelection_Overlay xOverlay("Use which attachment set?");

	for(LIF<Tools_MarkerPoint_Config*> xConfigLIF(&s_xMarkerConfigList);!xConfigLIF.done();xConfigLIF.next())
	{
		xOverlay.AddItem(xConfigLIF()->m_xName.GetString(),(int)xConfigLIF());
	}

	if(!xOverlay.DoModal()) return;

	Tools_MarkerPoint_Config* pxConfig = (Tools_MarkerPoint_Config*) xOverlay.GetSelectionData();
	
	for(LIF<Tools_MarkerPoint_Settings*> xMarkerLIF(&pxConfig->m_xMarkerList);!xMarkerLIF.done();xMarkerLIF.next())
	{
		Tools_MarkerPoint_Settings* pxSet = xMarkerLIF();

		//search for the marker point fro the name , parsing the 'path' as we go
		const char* pszPath = pxSet->m_xMarkerName.GetString();
		const List<Tools_MarkerPoint_Object*>* pxMarkerList = &xMarkerList;

		Tools_MarkerPoint_Object* pxObject = NULL;
		while(*pszPath)
		{
			bool bContinue = false;
			for(CLIF<Tools_MarkerPoint_Object*> xObjectLIF(pxMarkerList);!xObjectLIF.done();xObjectLIF.next())
			{
				int iNameLength = xObjectLIF()->m_xName.GetLength();
				if(!strnicmp(xObjectLIF()->m_xName.GetString(),pszPath,iNameLength))
				{
					if(pszPath[iNameLength] == '\\')
					{
						pszPath += iNameLength+1;
						pxMarkerList = &xObjectLIF()->m_xChildList;
						bContinue = true;
						break;
					}
					else if(pszPath[iNameLength] == 0)
					{
						pxObject = xObjectLIF();
						break;
					}
				}
			}
			if(!bContinue) break;
		}

		if(!pxObject) continue;

		if(pxSet->m_uSkinID)
		{
			pxObject->SetSkin(pxSet->m_uSkinID,pxSet->m_uAnimID);
		}
		else
		{
			pxObject->SetShape(pxSet->m_uShapeID);
		}

		pxObject->m_uPFXTemplate = pxSet->m_uPFXTemplate;
		pxObject->m_bMovePFXWithMarker = pxSet->m_bMovePFXWithMarker;
		pxObject->AddPFXAttachment();
	} 
}

void Tools_MarkerPoint_Object::SetTimer(float fTime)
{
	if(m_pxAttachedAnimation) m_pxAttachedAnimation->m_xPrimaryWrapper.SetTimer(fTime);

	for(LIF<Tools_MarkerPoint_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		xChildLIF()->SetTimer(fTime);
	}
}

void Tools_MarkerPoint_Object::CreateChildMarkerPoints()
{
	if(!m_pxAttachedAnimation) return;
	Asura_Hierarchy_Skin* pxSkin = m_pxAttachedAnimation->GetSkin();

	for(int i=0;i<pxSkin->NumberOfMarkerPoints;i++)
	{
		m_xChildList.add_entry(new Tools_MarkerPoint_Object(m_pxAttachedAnimation,pxSkin->MarkerPoint[i].GetName().GetString(),i,this));
	}
}

void Tools_MarkerPoint_Object::GetPathName(Asura_String& xOutput)
{
	//build list of all parents
	List<Tools_MarkerPoint_Object*> xList;

	Tools_MarkerPoint_Object* pxObject = this;
	while(pxObject)
	{
		xList.add_entry_start(pxObject);
		pxObject = pxObject->m_pxParent;
	}
	
	//concatenate all the names
	xOutput = "";

	while(xList.size())
	{
		xOutput += xList.first_entry()->m_xName.GetString();
		xList.delete_first_entry();
		if(xList.size()) xOutput += "\\";
	}
}

void Tools_MarkerPoint_Object::SaveToConfigIfFlagged(Tools_MarkerPoint_Config* pxConfig)
{
	if(m_bSaveToConfig)
	{
		Tools_MarkerPoint_Settings* pxSet = new Tools_MarkerPoint_Settings;
		pxConfig->m_xMarkerList.add_entry(pxSet);
	
		GetPathName(pxSet->m_xMarkerName);
		pxSet->m_uShapeID = GetShapeID();
		pxSet->m_uSkinID = GetSkinID();
		pxSet->m_uAnimID = GetAnimID();
		pxSet->m_uPFXTemplate = m_uPFXTemplate;
		pxSet->m_bMovePFXWithMarker = m_bMovePFXWithMarker;
	}
	//check children
	for(LIF<Tools_MarkerPoint_Object*> xChildLIF(&m_xChildList);!xChildLIF.done();xChildLIF.next())
	{
		xChildLIF()->SaveToConfigIfFlagged(pxConfig);
	}

}

void Tools_MarkerPoint_Object::SetParentSkinModified()
{
	if(m_pxParent)
	{
		//set skin used by parent marker point as modified
		if(m_pxParent->m_pxAttachedAnimation && !m_pxParent->m_xSkinName.IsBlank())
		{
			Tools_SaveAsura::SetSkinModified(m_pxParent->m_pxAttachedAnimation->GetSkin(),m_pxParent->m_xSkinName.GetString());
		}
	}	
	else
	{
		//otherwise parent skin is the one used by Tools_Mode_Hierarchy
		Tools_Mode_Hierarchy::SetSkinModified();
	}
}

void Tools_MarkerPoint_Object::AddPFXAttachment()
{
	RemovePFXAttachment();

	if(m_uPFXTemplate)
	{
		u_short usFlags = ASURA_PFX_EFFECT_IGNORE_MOD_LINKS;
		if(m_bMovePFXWithMarker) usFlags |= ASURA_PFX_EFFECT_MOVE_PARTICLES_WITH_EFFECT;

		m_pxAttachedPFX = new Asura_AnimationAttachment_PFX(m_uPFXTemplate,usFlags);
		m_pxAttachedPFX->SetDeleteWhenFinished(false);
		m_pxAttachedPFX->LinkAttachment(m_pxParentAnimation,m_xName.GetString(),false);
	}
}

void Tools_MarkerPoint_Object::RemovePFXAttachment()
{
	if(m_pxAttachedPFX)
	{
		m_pxAttachedPFX->UnlinkAttachment();
		delete m_pxAttachedPFX;
		m_pxAttachedPFX = 0;

	}
}

void Tools_MarkerPoint_Object::Rename()
{
	Asura_Assert(s_pxThis,"");
	Asura_Hierarchy_Skin* pxSkin = s_pxThis->m_pxParentAnimation->GetSkin();
	Asura_Hierarchy_Marker_Point* pxMarker = s_pxThis->GetMarkerPoint();
	if(!pxMarker) return;


	//ask for new name
	GetString_Overlay xOverlay("Rename marker point",pxMarker->GetName().GetString());
	
	while(true)
	{
		if(!xOverlay.DoModal()) return;

		const char* szName = xOverlay.GetString();
		if(!szName || !*szName) return;

		//make sure the name isn't already in use
		bool bOkay = true;
		for(int i=0;i<pxSkin->NumberOfMarkerPoints;i++)
		{
			if(&pxSkin->MarkerPoint[i] == pxMarker) continue;

			if(!stricmp(pxSkin->MarkerPoint[i].GetName().GetString(),szName))
			{
				ShowMessageOverlay("That name is already being used");
				bOkay = false;
				break;
			}

		}

		if(bOkay)
		{
			//Name is okay ,so apply it.
			pxMarker->SetName(szName);
			s_pxThis->m_xName = szName;
			s_pxThis->m_uMarkerHash = Asura_GetHashID(szName);
			s_pxThis->SetParentSkinModified();
			return;
		}
	}
}

void Tools_MarkerPoint_Object::UpdateAnimationFlags()
{
	if(!m_pxAttachedAnimation) return;

	float fTimer = m_pxAttachedAnimation->m_xPrimaryWrapper.GetTimer();
	Asura_Hash_ID uAnim = m_pxAttachedAnimation->m_xPrimaryWrapper.GetPrimaryAnimHashID();

	m_pxAttachedAnimation->m_xPrimaryWrapper.StartAnim2(uAnim,m_uAnimationFlags,0.0f);
	m_pxAttachedAnimation->m_xPrimaryWrapper.SetTimer(fTimer);
}

Asura_Hierarchy_Marker_Point* Tools_MarkerPoint_Object::GetMarkerPoint()
{
	int iIndex;
	if(m_pxParentAnimation->GetSkin()->GetMarkerPointIndex(m_uMarkerHash, iIndex))
	{
		return &m_pxParentAnimation->GetSkin()->MarkerPoint[iIndex];
	}
	else
	{
		//This may happen if a skin is attached to another instance of the same skin in hierarchy mode , and
		//the marker point on one instance gets renamed/deleted.
		//In practice anything calling this should cope with it being missing anyway , but it is likely
		//to leave a phantom marker point object hanging around until the tools objects are cleared up.
		//RAR 19/11/2007
		Asura_Assert(0,"Failed to find marker point for tools object");
		return 0;
	}
}

int Tools_MarkerPoint_Object::GetIndex() const
{
	int iIndex = -1;
	Asura_Hierarchy_Skin* pxSkin = m_pxParentAnimation->GetSkin();
	if(!pxSkin->GetMarkerPointIndex(m_uMarkerHash, iIndex))
	{
		Asura_Assert(0,"Failed to get index for marker point");
	}
	return iIndex;
}

void Tools_MarkerPoint_Object::Destroy()
{
	int iIndex;
	Asura_Hierarchy_Skin* pxSkin = m_pxParentAnimation->GetSkin();
	if(pxSkin->GetMarkerPointIndex(m_uMarkerHash, iIndex))
	{
		//make sure this gets removed from the list in hierarchy mode
		Tools_Mode_Hierarchy::HandleMarkerPointDeletion(this);

		//shuffle the marker points down to feel the free space.
		//keeping the same ordering , so that the marker point ordering continues to match
		//the list ordering in hierarchy mode.
		for(int i=iIndex; i<pxSkin->NumberOfMarkerPoints-1; i++)
		{
			pxSkin->MarkerPoint[i] = pxSkin->MarkerPoint[i+1];
		}
		pxSkin->NumberOfMarkerPoints--;
		SetParentSkinModified();

		Tools_Object::Destroy();

	}
	else
	{
		Asura_Assert(0, "Failed to get index of marker point being deleted");
	}
}

/////////////////////////////////////////////////////////////////////////////////////////////

Tools_MarkerPoint_Config::~Tools_MarkerPoint_Config()
{
	while(m_xMarkerList.size())
	{
		delete m_xMarkerList.first_entry();
		m_xMarkerList.delete_first_entry();
	}
}
