//
// Tools_MarkerPoint_Object.h
//
// Created by Richard Rice - 13/09/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_MARKERPOINT_OBJECT_H_
#define _TOOLS_MARKERPOINT_OBJECT_H_

#include "Core/Asura_String.h"
#include "Core/Asura_Hash.h"
#include "Objects/Tools_Object.h"

class Asura_Animation;
class Asura_Hierarchy_Marker_Point;

class Tools_MarkerPoint_Object : public Tools_Object
{
	public :
		Tools_MarkerPoint_Object(Asura_Animation* pxAnimation,const char* pszName,int iIndex,Tools_MarkerPoint_Object* pxParent);
		~Tools_MarkerPoint_Object();

		const char* GetName(){return m_xName.GetString();}
		const char* GetTypeDescription(){return "Marker Point";}

		void Render();
		void UpdatePositionFromAsuraData();
//		void UpdatePositionInAsuraData(bool bUpdateLinkedObjects);
		void UpdateMovement(float* pMovementScale,float* pRotationScale);
		void Update();
		void OnSelection();
		void OnDeselection();
		//Just allow the marker points for the main skin to be deleted. Allowing deleting of marker points
		//on attached skins is a little iffy if multiple selection is being used
		bool CanBeDestroyed(){return m_pxParent==0;}
		void Destroy();
//		bool CanBeMoved(){return false;}
		bool IsHidden();
		bool UseModuleLinking(){return false;}

		//sets time in attached anim , if there is one
		void SetTimer(float fTime);

		static void SaveMarkerPointConfigFile();
		static void LoadMarkerPointConfigFile();
		static void DeleteMarkerPointConfigs();

		static void StoreMarkerPointSettings(const List<Tools_MarkerPoint_Object*>& xMarkerList);
		static void RestoreMarkerPointSettings(const List<Tools_MarkerPoint_Object*>& xMarkerList);

		Asura_Animation* GetParentAnimation()const {return m_pxParentAnimation;}
		int GetIndex()const;
		void SetParentSkinModified();

		void SetSkin(Asura_Hash_ID uSkin,Asura_Hash_ID uAnim);
		void RemoveAttachment();

protected:
		void CreateOverlay();
		void DeleteOverlay();

		static void ChooseSkin();
		static void ChooseAnimation();
		static void ChooseShape();
		static void _RemoveAttachment();

		static void ResetPosition();
		static void Rename();

		Asura_Hierarchy_Marker_Point* GetMarkerPoint();
				

		void SetShape(Asura_Hash_ID uShape);
//		void SetSkin(Asura_Hash_ID uSkin,Asura_Hash_ID uAnim);

		Asura_Hash_ID GetShapeID();
		Asura_Hash_ID GetSkinID();
		Asura_Hash_ID GetAnimID();

		void CreateChildMarkerPoints();

		void GetPathName(Asura_String& xOutput);
		void SaveToConfigIfFlagged(struct Tools_MarkerPoint_Config*);

		void AddPFXAttachment();
		void RemovePFXAttachment();

		//Update animation with new animation flags.
		void UpdateAnimationFlags();


		//data
		Asura_String m_xName;
		Asura_Animation* m_pxParentAnimation;
		Asura_Hash_ID m_uMarkerHash;
		Asura_Mesh* m_pxDefaultMesh;

		bool m_bInUpdateMovement;

		Asura_Animation* m_pxAttachedAnimation;
		Asura_Hash_ID m_uShapeID;

		Asura_Vector_3 m_xInitialPosition;
		Asura_Matrix_3x3 m_xInitialOrientation;

		bool m_bSaveToConfig;//temp flag while saving attachment configs

		bool m_bShowChildMarkerPoints;
		List<Tools_MarkerPoint_Object*> m_xChildList;
		Tools_MarkerPoint_Object* m_pxParent;

		Asura_String m_xSkinName;

		Asura_Hash_ID m_uPFXTemplate;
		bool m_bMovePFXWithMarker;
		bool m_bPFXChanged;
		class Asura_AnimationAttachment_PFX* m_pxAttachedPFX;

		u_int m_uAnimationFlags;
		bool m_bAnimFlagsChanged;
		bool m_ChangeData;

		//static data for overlay
		static Asura_Vector_2 s_xOverlayPosition;
		static Tools_MarkerPoint_Object* s_pxThis;

		static bool s_b90DegreeRotation;


		static List<struct Tools_MarkerPoint_Config*> s_xMarkerConfigList;

		friend class Tools_SaveMarkerConfig_Overlay;

};

struct Tools_MarkerPoint_Config
{
	~Tools_MarkerPoint_Config();

	Asura_String m_xName;
	List<struct Tools_MarkerPoint_Settings*> m_xMarkerList;
};

struct Tools_MarkerPoint_Settings
{
	Tools_MarkerPoint_Settings()
	{
		m_uShapeID = m_uSkinID = m_uAnimID = ASURA_HASH_ID_UNSET;
		m_uPFXTemplate = 0;
		m_bMovePFXWithMarker = true;
	}

	Asura_String m_xMarkerName;
	Asura_Hash_ID m_uShapeID;
	Asura_Hash_ID m_uSkinID;
	Asura_Hash_ID m_uAnimID;

	Asura_Hash_ID m_uPFXTemplate;
	bool m_bMovePFXWithMarker;

};


#endif //_TOOLS_MARKERPOINT_OBJECT_H_
