//
// Tools_Mode_Hierarchy.h
//
// Created by Richard Rice - 02/04/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_MODE_HIERARCHY_H__
#define __TOOLS_MODE_HIERARCHY_H__

#include "Tools_Mode.h"

#include "Core/Asura_Handle.h"
#include "Hierarchy/Tools_MarkerPoint_Object.h"
#include "Hierarchy/Tools_SectionBoundingBox_Object.h"
#include "Misc/Tools_MessageLog.h"

class Tools_Joint_Object;
class Tools_SkinLOD;
class Asura_Hierarchy_Anim;
class Asura_Hierarchy_Skin;
class Asura_Hierarchy_Bone_Anim;
class Asura_Hierarchy_Sound;
class Asura_Photon;
class Asura_String;
class Asura_DropDown_List_Dialog;
class Asura_Tickbox_Dialog;
class Asura_AngleNumerical_Dialog;
class Asura_Function_Dialog;
class Asura_Numerical_Dialog;
class Asura_Dialog;

#define TOOLS_MAX_LOD_MODELS 3
#define TOOLS_MAX_HOTKEY_ANIM 10

///////////////////////////////////////////////////////////////////////////////////////
#define MAX_SECONDARY_ANIM 4

struct SecondaryAnimationData
{
	void Initialise(int index);
	void Reset();

	bool Used;
	bool Restart;
	bool SyncWithPrimary;
	Asura_Handle AnimHandle;
	Asura_Hierarchy_Anim* m_pxAnim;
	Asura_String m_xDirectory;
	float Weight;
	bool m_bEnabled;
	bool m_bMirrored;
	bool m_bReversed;

	float m_fTimer;
	bool m_bPaused;
	bool m_bTimerChanged;

	int OverlayID;
	void StartAnimation();
};

///////////////////////////////////////////////////////////////////////////////////////

class Tools_Photon_Object;

class Tools_Mode_Hierarchy : public Tools_Mode
{
public:

		Tools_Mode_Hierarchy();
		
		~Tools_Mode_Hierarchy()
		{
			s_pxThis = 0;
		}

		virtual void InitialiseMode(void);
		virtual void ShutdownMode(void);

		// JEB - 22 August 2006 15:23:18 - Made function virtual, allows derived classes to do work in the update
		virtual void Update(void);
		void RenderUI(void);
		void EarlyRender();
		

		virtual void QuickSave(void);
		virtual void PreSave();

 		bool IsModified();

        const Asura_Char* GetAnimationName( void ) const { return AnimName.GetString(); }
        const Asura_Char* GetSkinName( void ) const { return SkinName.GetString(); }
        const Asura_Char* GetSkinFilename( void ) const { return SkinFilename.GetString(); }

		Asura_Hierarchy_Anim* GetNextAnimation(Asura_Hierarchy_Anim* pxCurrentAnim,Asura_String* pxDirectory,Asura_String* pxFilename);
		Asura_Hierarchy_Anim* GetPreviousAnimation(Asura_Hierarchy_Anim* pxCurrentAnim,Asura_String* pxDirectory,Asura_String* pxFilename);

		void GetNextSkin(const char* szCurrentSkin,Asura_String& xOutputSkinName,Asura_String& xOutputSkinFile);
		void GetPreviousSkin(const char* szCurrentSkin,Asura_String& xOutputSkinName,Asura_String& xOutputSkinFile);

		static Asura_Animation*	GetCurrentHierarchy(void)	{ return((s_pxThis == NULL) ? NULL : s_pxThis->Hierarchy); }
		static Asura_Vector_3 GetHierarchyPosition();

		static Asura_Hierarchy_Anim* GetCurrentAnimation()	{ return((s_pxThis == NULL) ? NULL : s_pxThis->Animation); }
		static Asura_Hierarchy_Skin* GetCurrentSkin()		{ return((s_pxThis == NULL) ? NULL : s_pxThis->m_pxSkin); }

		static void SetSkinModified();
		static void SetAnimModified();

		static Asura_Vector_2* GetSkinSoundDefnOverlayPosition(){return &s_pxThis->m_xSkinSoundDefnOverlayPosition;}
		static Asura_Vector_2* GetAnimSoundDefnOverlayPosition(){return &s_pxThis->m_xAnimSoundDefnOverlayPosition;}

		static class Tools_Flag_List* GetAnimationSoundList() {return s_pxThis->m_pxAnimationSoundList;}

		static void PreReloadAnims();
		static void PostReloadAnims();

		static void PreReloadSkins();
		static void PostReloadSkins();

		//sets the default skin to use when entering the mode.
		static void SetDefaultSkin(const Asura_Hierarchy_Skin* pxSkin) {s_pxDefaultSkin = pxSkin;}

		virtual void RenderWorld_PostToneMap();

		//Removes the marker point object from the list stored for the mode
		static void HandleMarkerPointDeletion(Tools_MarkerPoint_Object* pxObject);

		static void StopLoopedSounds();

		static bool IsPrimaryAnimReversed();

		//Changes the skin/anim selection if currently in this mode
		static void SelectSkin(Asura_Hash_ID uSkinID);
		static void SelectAnim(Asura_Hash_ID uAnimID);

protected:

		virtual class Tools_Flag_List* GetAnimationFlagList();//gets the list of flags for the current animation

		virtual void AddAnimPropertiesOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
		virtual void AddSkinPropertiesOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset);
				
		//called when a new animation becomes selected , before the animation properties overlay is created
		virtual void OnSelectAnimation(Asura_Hierarchy_Anim* pxAnim){};
		virtual void OnDeselectAnimation(Asura_Hierarchy_Anim* pxAnim){};

		//and for skins
		virtual void OnSelectSkin(Asura_Hierarchy_Skin* pxSkin){};
		virtual void OnDeselectSkin(Asura_Hierarchy_Skin* pxSkin){};

		const Asura_Vector_3& GetHierarchyModeOrigin();

		virtual void AllocateHierarchy( const char* szName );

		class Asura_Animation* Hierarchy;

		// JEB - 22 August 2006 15:23:55 - Moved from private to protected, allows derived classes to add menu entries
		Asura_Handle MenuHandle;

		bool SetSkin(const char* pszSkinName,const char* pszSkinFilename);
		void SetShowAttachments(const bool bShow)	{ m_bShowMarkerPoints = bShow; }
		STATIC_VOID_FN(ChooseSkin)

		List<Tools_MarkerPoint_Object*> m_xMarkerList;

private:

		//loads/saves data for this mode
		void TransferModeData(int mode);

		void Tools_Mode_Hierarchy::HandleKeyboard(void);

		STATIC_VOID_FN(ToggleHierarchyOverlay)
		STATIC_BOOL_FN(HierarchyOverlayIsOn)
		void CreateHierarchyOverlay(void);

		STATIC_VOID_FN(ToggleAnimPropertiesOverlay)
		STATIC_BOOL_FN(AnimPropertiesOverlayIsOn)
		void CreateAnimPropertiesOverlay(void);

		STATIC_VOID_FN(ToggleSkinPropertiesOverlay)
		STATIC_BOOL_FN(SkinPropertiesOverlayIsOn)
		void CreateSkinPropertiesOverlay(void);

		STATIC_VOID_FN(ToggleAnimationControlOverlay)
		STATIC_BOOL_FN(AnimationControlOverlayIsOn)
		void CreateAnimationControlOverlay(void);
		void UpdateAnimationControlOverlay();

		STATIC_VOID_FN(ToggleHierarchyLODOverlay)
		STATIC_BOOL_FN(HierarchyLODOverlayIsOn)
		void CreateHierarchyLODOverlay(void);

		STATIC_VOID_FN(ToggleFrameReductionOverlay)
		STATIC_BOOL_FN(FrameReductionOverlayIsOn)
		void CreateFrameReductionOverlay(void);
		void UpdateFrameReductionOverlay();

		STATIC_VOID_FN(TogglePosableAnimOverlay)
		STATIC_BOOL_FN(PosableAnimOverlayIsOn)
		void CreatePosableAnimOverlay();

		void UpdateAnimationFlagOverlay();
		void UpdateAnimationFlagTimeDisplay();
		void UpdateAnimationSoundOverlay();
		void UpdateAnimationSoundTimeDisplay();

		STATIC_VOID_FN(ChooseAnim)
		void ChooseDirectory();
//		bool SetSkin(const char* pszSkinName,const char* pszSkinFilename);

		Asura_Hierarchy_Anim* GetNextAnimationMain(Asura_Hierarchy_Anim* pxCurrentAnim,Asura_String* pxDirectory,Asura_String* pxFilename,int iDirection);
		void GetNextSkinMain(const char* szCurrentSkin,Asura_String& xOutputSkinName,Asura_String& xOutputSkinFile,int iDirection);

		void NextAnimation();
		void PreviousAnimation();
		void NextSkin();
		void PreviousSkin();
		void SelectAnimation(Asura_Hierarchy_Anim* pxAnim,const char* pszFilename);
		void UpdateAnimNameList();
		void UpdatePrimaryAnimFlags();

		//Re-sets all the skins in the animation from the hash IDs.
		void SetSkinsInAsuraAnimation();


		class Asura_Hierarchy_Skin* LoadSkin(const Asura_String& xSkinName,const Asura_String& xSkinFileName);
		class Asura_Hierarchy_Anim* LoadAnim(const Asura_String& xAnimName,const Asura_String& xAnimFileName);

		void UpdateTreadmill();
		void RenderTreadmill();

		STATIC_VOID_FN(AddSecondaryAnim)

		void ChooseSecondaryAnim(SecondaryAnimationData* secondary);
		static void _ChooseSecondaryAnim(int iUserData){s_pxThis->ChooseSecondaryAnim((SecondaryAnimationData*)iUserData);}

		void RemoveSecondaryAnim(SecondaryAnimationData* secondary,bool reset);
		static void _RemoveSecondaryAnim(int iUserData){s_pxThis->RemoveSecondaryAnim((SecondaryAnimationData*)iUserData,true);}

		void RestartSecondaryAnim(SecondaryAnimationData* secondary);
		void NextSecondaryAnim(int iDirection);
		void UpdateSecondaryAnims();

		//STATIC_VOID_FN(AnimationControl_Rewind);
		STATIC_VOID_FN(AnimationControl_FrameBack)
		STATIC_VOID_FN(AnimationControl_Play)
		STATIC_VOID_FN(AnimationControl_Pause)
		STATIC_VOID_FN(AnimationControl_FrameForward)
		//STATIC_VOID_FN(AnimationControl_FastForward);
		STATIC_VOID_FN(AnimationControl_SetSliderMode)

		STATIC_VOID_FN(AddAnimationFlag)
		STATIC_VOID_FN(DeleteAnimationFlag)
		STATIC_VOID_FN(EditAnimationFlag)
		STATIC_VOID_FN(SetAnimationFlag)

		STATIC_VOID_FN(AddAnimationSound)
		STATIC_VOID_FN(DeleteAnimationSound)
		STATIC_VOID_FN(EditAnimationSound)

		STATIC_VOID_FN(EditSkinSoundDefns)
		STATIC_VOID_FN(EditAnimSoundDefns)

		STATIC_VOID_FN(GenerateLODModels)
		
		STATIC_VOID_FN(EditCollisionBB)
		STATIC_VOID_FN(ResetCollisionBB)
		STATIC_BOOL_FN(CanEditCollisionBB)
		STATIC_BOOL_FN(CanDrawCollisionMesh)

		STATIC_VOID_FN(RecalculateSectionCollisionMesh);
		STATIC_BOOL_FN(IsObjectHierarchy)

		STATIC_VOID_FN(CreateLight)

		STATIC_VOID_FN(SaveAttachments)
		STATIC_VOID_FN(RestoreAttachments)

		STATIC_BOOL_FN(CanEditLOD)


		void CreateMarkerPointObjects();
		void DeleteMarkerPointObjects();
		void UpdateMarkerPointObjects();
		STATIC_VOID_FN(CreateMarkerPoint);
		

		void CreateBoundingBoxObjects();
		void DeleteBoundingBoxObjects();

		//functions for saving/restoring animations to hotkeys
		void StoreAnimation(int iIndex);
		void RestoreAnimation(int iIndex);

		
		void BackupAnimationKeyFrames();
		void RestoreAnimationKeyFrames();
		void RemoveUnneededKeyFrames();

		void CalculateAnimationBB(bool bAlwaysGenerate);
		//checks to see if BB is needed for all anims in a selected dir.
		STATIC_VOID_FN(CheckAnimBoundingBoxes);


		STATIC_VOID_FN(StartLODBatch);
		void UpdateLODBatch();
		void CancelLODBatch();

		//functions for posable anim overlay
		STATIC_BOOL_FN(AnimIsPosable);
		STATIC_VOID_FN(ResetPosableAnim);

		//For assigning test sounds to keyframe flags
		STATIC_VOID_FN(EditTestSounds);
		STATIC_VOID_FN(AddTestSound);
		STATIC_VOID_FN(DeleteTestSound);
		STATIC_VOID_FN(LoadDefaultTestSounds);
		STATIC_BOOL_FN(IsEnabled_TestSoundEvent);
		void UpdateTestSounds();
		void BuildTestSoundNameArray();
		void DeleteTestSoundNameArray();
	

		void AnimationControl_UpdateSlideTimer();
		void AnimationControl_UpdateTimer();

		//get , or guess the original number of animation frames
		int GetNumAnimFrames(const Asura_Hierarchy_Anim* pxAnim) const;


		STATIC_VOID_FN(WriteAVI);

		//outputing hierarchy text file
		static void OutputSkinHierarchyTextFile() {s_pxThis->OutputSkinHierarchyTextFile(true);}
		static void OutputBasicSkinHierarchyTextFile() {s_pxThis->OutputSkinHierarchyTextFile(false);}
		void OutputSkinHierarchyTextFile(bool bFullFile);
		void CreateSkinHierarchyTextFile(Asura_Hierarchy_Skin* pxSkin,const char* pszFilename,bool bFullFile);

		STATIC_VOID_FN(OutputAnimHierarchyTextFile);
		void CreateAnimHierarchyTextFile(Asura_Hierarchy_Anim* pxAnim,const char* pszFilename);

		void OutputHierarchy(FILE* pFile,Asura_Hierarchy_Skin* pxSkin,Asura_Hierarchy_Anim* pxAnim,int iBoneIndex,int iIndent);

		//sets timer for all animations and attachment anims
		void SetHierarchyTimer(float fTime);

		// Helper function
		float ConvertParametricTimer( const float fTime );



		///////////////////////////
		//data
		Asura_Vector_2 HierarchyOverlayPosition;
		bool	HierarchyOverlayOpen;

		Asura_Vector_2 m_xAnimPropertiesOverlayPosition;
		bool	m_bAnimPropertiesOverlayOpen;

		Asura_Vector_2 m_xSkinPropertiesOverlayPosition;
		bool	m_bSkinPropertiesOverlayOpen;


		Asura_Vector_2 AnimationControlOverlayPosition;
		bool	AnimationControlOverlayOpen;

		Asura_Vector_2 HierarchyLODOverlayPosition;
		bool	HierarchyLODOverlayOpen;


		Asura_Vector_2 FrameReductionOverlayPosition;
		bool FrameReductionOverlayOpen;

		Asura_Vector_2 m_xPosableAnimOverlayPosition;
		bool m_bPosableAnimOverlayOpen;


		Asura_Vector_2 AnimationFlagOverlayPosition;
		Asura_Vector_2 AnimationSoundOverlayPosition;
		Asura_Vector_2 m_xSkinSoundDefnOverlayPosition;
		Asura_Vector_2 m_xAnimSoundDefnOverlayPosition;
		
		Asura_String SkinName;
		Asura_String AnimName;
		Asura_String DirectoryName;
        Asura_String SkinFilename;
		Asura_String m_xAnimFilename;

		class Asura_Hierarchy_Anim* Animation;
		class Asura_Hierarchy_Skin* m_pxSkin;

		Asura_Hash_ID* m_puSkinID;//IDs of all skins being shown for multi-part skins

		class Tools_Flag_List* AnimationFlagList;
		class Tools_Flag_List* m_pxAnimationSoundList;

		class Tools_Flag_IniFile* m_pxSoundIniFile;

		float AnimDuration;
		int m_iAnimFrames;
		float TreadmillSpeed;
		float TreadmillDirection;//in degrees
		class Asura_Mesh* TreadmillMesh;
		bool m_bAnimHasBB;
		float m_fLoopPoint;
		bool m_bWarnedAboutCompressedFrames;

		//flags for anim preview
		u_int m_uPrimaryAnimFlags;
		bool m_bAnimFlagsChanged;

		float m_fAnimationControl_Speed;
		float m_fAnimationControl_DeltaTime;
		float m_fAnimationControl_Timer;
		float m_fAnimationControl_SlideTimer;
		bool m_bAnimationControl_TimerChanged;
		bool m_bAnimationControl_SlideTimerChanged;

		float m_fTimeSliderMin;
		float m_fTimeSliderMax;

		enum AnimationTimerMode
		{
			TIMER_PROP = 0,
			TIMER_SECONDS = 1,
			TIMER_FRAMES = 2,
			TIMER_25FPS = 3,
			TIMER_30FPS = 4,
		};

		AnimationTimerMode m_eAnimationControl_TimerMode;

		float SelectedIntervalIndex;
		bool SelectedIntervalIndexChanged;
		bool AnimationFlagOverlayNeedsUpdate;
		bool FlagCoversInterval;
		bool FlagStartChanged;
		bool FlagEndChanged;

		int m_iSelectedSoundIndex;
		bool m_bSelectedSoundIndexChanged;
		bool m_bSoundStartChanged;
		bool m_bSoundEndChanged;
		bool m_bSoundLooped;
		bool m_bSoundLoopedChanged;

		//skin and anim sound definitions
		class ToolsAnimSoundDefnEdit* m_pxAnimSoundEdit;
		class ToolsSkinSoundDefnEdit* m_pxSkinSoundEdit;



		bool ModifiedFlag;
		bool LODModified;
		bool m_bSkinModified;
		bool m_bSkinSelectionChanged;

		SecondaryAnimationData SecondaryAnims[MAX_SECONDARY_ANIM];
		SecondaryAnimationData* m_pxMostRecentSecondary;

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
		Tools_SkinLOD* m_pxSkinLOD;
#endif
		float m_afLODDistances[TOOLS_MAX_LOD_MODELS];
		float m_afLODPercent[TOOLS_MAX_LOD_MODELS];

		//skin collision bounding box
		Asura_Bounding_Box m_xSkinCollisionBB;
		class Tools_VolumeEdit_Object* m_pxSkinCollisionVolume;

		bool m_bShowSkinAndAnimProperties;
		bool m_bDrawCollisionMesh;
		bool m_bWireframeMesh;
		bool m_bDrawMesh;
		bool m_bDrawSkeleton;

		bool m_bDrawMarkerPointAxes;
		bool m_bShadowed;
		bool m_bShowLights;
		bool m_bShowMarkerPoints;
        bool m_bShowSectionBoundingBoxes;
		bool m_bShowTreadmill;
		bool m_bUseMovementOffsets;
		int m_iMaxBonesPerVertex;//for testing rendering with fewer than 4 bones/vertex

		List<Tools_SectionBoundingBox_Object*> m_xBoundingBoxList;

		Tools_Object* m_pxCompassObject;

		Asura_Hash_ID m_auHotkeyedAnimations[TOOLS_MAX_HOTKEY_ANIM];

		//for storing a copy of the keyframes while testing removal
		//of unneeded frames
		int m_iBackupAnimationArraySize;
		Asura_Hierarchy_Bone_Anim* m_pxBackupAnimation;

		
		bool m_bDoFrameRemoval;
		bool m_bUpdateFrameRemoval;
		float m_fPositionTolerance;
		float m_fAngleTolerance;
		int m_iFramesBefore;
		int m_iFramesAfter;


		//for storing the position and orientation of the camera before entering this mode
		Asura_Vector_3 m_xPreModeCameraPosition;
		Asura_Matrix_3x3 m_xPreModeCameraOrientation;

		List<struct Tools_LODBatchEntry*> m_xLODBatchList;
		int m_iBatchStep;

		//posable anim settings
		Asura_Euler_3 m_xPosableHipOrient;
		Asura_Euler_3 m_xPosableHeadOrient;

		//temporary sounds for flags
		int m_iNumTestSounds;
		struct Tools_FlagSoundPair* m_pxTestSounds;

		int m_iTestSoundIndex;
		char** m_pszTestSoundNames;
		int m_iTestSoundNameArraySize;
		bool m_bTestSoundIndexChanged;
		Asura_Hash_ID m_uTestSoundEvent;
		bool m_bTestSoundEventChanged;
		Asura_Vector_2 m_xTestSoundOverlayPosition;

		//for recording one loop of an anim as an AVI
		bool m_bRecordingAVI;
		int m_iAVIFrame;

		//for use while reloading anims
		Asura_Hierarchy_Anim* m_pxCurrentAnimBackup;
		Asura_String m_xCurrentAnimFilenameBackup;

		//default skin to use when entering the mode
		static const Asura_Hierarchy_Skin* s_pxDefaultSkin;

		Asura_Vector_3 m_xPos;
		Asura_Vector_3 m_xOldPos;
		Asura_Matrix_3x3 m_xMat;



		List< Asura_Photon*> m_xPhotonsActiveAtStart;
 
		enum { NUM_TEMP_LIGHTS = 3 };
		int m_aiTempLightToolsIDs[ NUM_TEMP_LIGHTS ];
		
		enum DialogID
		{
			DialogID_Skin,
			DialogID_Anim,
			DialogID_PositionTolerance,
			DialogID_QuatTolerance,
			DialogID_SkinSoundProperties,
			DialogID_TimeSlider,
			DialogID_SliderMode,
			DialogID_SoundStartTime,
			DialogID_SoundEndTime,
			DialogID_FlagStartTime,
			DialogID_FlagEndTime,
			DialogID_ScaledTimer,
			DialogID_UnScaledTimer,
			DialogID_TestSoundList,
		};

		friend struct SecondaryAnimationData;


		static Tools_Mode_Hierarchy* s_pxThis;//pointer to the mode for use by the static members

};

struct Tools_LODBatchEntry
{
	float m_afDistance[TOOLS_MAX_LOD_MODELS];
	float m_afPercent[TOOLS_MAX_LOD_MODELS];
	Asura_String m_xFilename;
};

struct Tools_FlagSoundPair
{
	int m_iFlag;
	Asura_Hash_ID m_uSoundEvent;
};

class ToolsSoundDefnEdit
{
public :
	ToolsSoundDefnEdit(int iOverlayID);
	virtual ~ToolsSoundDefnEdit();

	void Update();
	void UpdateOverlay();

	static void ClearPasteBuffer();

protected :
	virtual const char* GetOverlayTitle() = 0;
	virtual Asura_Vector_2* GetOverlayPosition() = 0;
	virtual int& NumberOfSoundDefns() = 0;
	virtual Asura_Hierarchy_Sound*& SoundDefns() = 0;
	virtual void SetModified() = 0;

private :
	enum DialogID
	{
		DialogID_SoundProperties,
	};


	static void AddSound(int iThis){((ToolsSoundDefnEdit*)iThis)->AddSound();}
	static void DeleteSound(int iThis){((ToolsSoundDefnEdit*)iThis)->DeleteSound();}
	static void Copy(int iThis){((ToolsSoundDefnEdit*)iThis)->Copy();}
	static void Paste(int iThis){((ToolsSoundDefnEdit*)iThis)->Paste();}

	void AddSound();
	void DeleteSound();
	void Copy();
	void Paste();

	void BuildSoundNameArray();
	void DeleteSoundNameArray();
	void HandleSelectionChange();

	static bool NotUsingSoundEvent(int iThis)
	{return !((ToolsSoundDefnEdit*)iThis)->m_uSoundEvent;}

	static bool CanPaste() {return s_iPasteBufferSize != 0;}


	//////////////////////////////
	int m_iOverlayID;

	int m_iSoundSelection;
	int m_iSoundOffset;
	int m_iSoundIndex;


	Asura_Hash_ID m_uSoundEvent;
	bool m_bSoundEventChanged;

	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	Asura_Hash_ID m_uDialogueEvent;
	bool m_bDialogueEventChanged;
	#endif

	int m_iSoundNameArraySize;
	char** m_ppszSoundNames;
	
	static int s_iPasteBufferSize;
	static Asura_Hierarchy_Sound* s_pxPasteBuffer;

};

class ToolsAnimSoundDefnEdit : public ToolsSoundDefnEdit
{
public :
	ToolsAnimSoundDefnEdit(Asura_Hierarchy_Anim* pxAnim);

protected :
	virtual const char* GetOverlayTitle(){return "Anim Sound Marker Defns";}
	virtual Asura_Vector_2* GetOverlayPosition() {return Tools_Mode_Hierarchy::GetAnimSoundDefnOverlayPosition();}
	virtual int& NumberOfSoundDefns();
	virtual Asura_Hierarchy_Sound*& SoundDefns();
	virtual void SetModified() {Tools_Mode_Hierarchy::SetAnimModified();}

private :
	Asura_Hierarchy_Anim* m_pxAnim;

};

class ToolsSkinSoundDefnEdit : public ToolsSoundDefnEdit
{
public :
	ToolsSkinSoundDefnEdit(Asura_Hierarchy_Skin* pxSkin);

protected :
	virtual const char* GetOverlayTitle(){return "Skin Sound Marker Defns";}
	virtual Asura_Vector_2* GetOverlayPosition() {return Tools_Mode_Hierarchy::GetSkinSoundDefnOverlayPosition();}
	virtual int& NumberOfSoundDefns();
	virtual Asura_Hierarchy_Sound*& SoundDefns();
	virtual void SetModified() {Tools_Mode_Hierarchy::SetSkinModified();}

private :
	Asura_Hierarchy_Skin* m_pxSkin;
};


class Tools_MessageLogAction_SelectSkin : public Tools_MessageLogAction
{
public :
	Tools_MessageLogAction_SelectSkin(const Asura_Hierarchy_Skin* pxSkin);
	void Execute();
private :
	Asura_Hash_ID m_uHash;
};

class Tools_MessageLogAction_SelectAnim : public Tools_MessageLogAction
{
public :
	Tools_MessageLogAction_SelectAnim(const Asura_Hierarchy_Anim* pxAnim);
	void Execute();
private :
	Asura_Hash_ID m_uHash;
};



#endif//__TOOLS_MODE_HIERARCHY_H__
