//
// Tools_HistogramMacros.h
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif


#ifndef _included_histgram_h
#define _included_histgram_h

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct LS_Block
	{
		unsigned int begin_input, end_input, begin_output, end_output, grad; // these are all in 16.16 fixed
	}
	LS_Block;

#define MAX_LS_BLOCKS 4
typedef struct Lighting_Style
	{
		int darken_flag; // lighting applied to bitmap(s) ?
		unsigned int table_num_shades; // number of entries in TLT
		unsigned int palette_num_shades; // number of shades to make and add to histogram for white (less for darker colours)
		unsigned int num_blocks; // number of sections to the intensity-in -> intensity-out graph
		LS_Block blocks[ MAX_LS_BLOCKS ]; // max ??
	}
	Lighting_Style;

extern void AdjustLSBlocksForPalette (Lighting_Style * ls);

#ifdef _int64
#error "_int64 already defined"
#endif
#ifdef _uint64
#error "_uint64 already defined"
#endif
#if defined(__GNUC__)
	#define _int64 signed long long
	#define _uint64 unsigned long long
#elif defined(__WATCOMC__)
	#if __WATCOMC__ >= 1100
		#define _int64 signed __int64
		#define _uint64 unsigned __int64
	#endif
#elif defined(_MSC_VER)
	#if _INTEGRAL_MAX_BITS >= 64
		#define _int64 signed __int64
		#define _uint64 unsigned __int64
	#endif
#else
	#pragma message ("Unknown compiler: using 'double' as 64 bit data type")
	#define _int64 double
	#define _uint64 double
#endif

#ifdef _int64

#define SignedWideMulNarrowDiv(a,b,c) ((int)((_int64)(a)*(b)/(c)))
#define UnsignedWideMulNarrowDiv(a,b,c) ((unsigned int)((_uint64)(a)*(b)/(c)))
#define SignedMulFixed(a,b) ((int)(((_int64)(a)*(b))/65536))
#define UnsignedMulFixed(a,b) ((unsigned int)(((_uint64)(a)*(b))/65536))
#define SignedDivFixed(a,b) ((int)(((_int64)(a)*65536)/(b)))
#define UnsignedDivFixed(a,b) ((unsigned int)(((_uint64)(a)*65536)/(b)))

#elif defined __WATCOMC__

#pragma message("Watcom inline assembler for 64 bit math via #pragma")

int SignedWideMulNarrowDiv(int a, int b, int c);
# pragma aux SignedWideMulNarrowDiv = \
"imul   edx"\
"idiv   ebx" \
parm[eax] [edx] [ebx] \
value[eax];

unsigned int UnsignedWideMulNarrowDiv(unsigned int a, unsigned int b, unsigned int c);
# pragma aux UnsignedWideMulNarrowDiv = \
"mul    edx"\
"div    ebx" \
parm[eax] [edx] [ebx] \
value[eax];

int SignedMulFixed(int a, int b);
# pragma aux SignedMulFixed = \
"imul   edx" \
"shrd   eax,edx,16" \
parm[eax] [edx] \
value[eax] \
modify[edx];

unsigned int UnsignedMulFixed(unsigned int a, unsigned int b);
# pragma aux UnsignedMulFixed = \
"mul    edx" \
"shrd   eax,edx,16" \
parm[eax] [edx] \
value[eax] \
modify[edx];

int SignedDivFixed(int a, int b);
# pragma aux SignedDivFixed = \
"cdq" \
"shld   eax,edx,16" \
"idiv   ebx" \
parm[eax] [ebx] \
value[eax] \
modify[edx];

unsigned int UnsignedDivFixed(unsigned int a, unsigned int b);
# pragma aux UnsignedDivFixed = \
"xor    edx,edx" \
"shld   eax,edx,16" \
"div    ebx" \
parm[eax] [ebx] \
value[eax] \
modify[edx];

#elif defined _MSC_VER

#pragma message("MSC inline assembler for 64 bit math")

static int SignedWideMulNarrowDiv(int a, int const b, int const c)
{
	_asm
	{
		mov eax,a
		imul b
		idiv c
		mov a,eax
	}
	return a;
}

static unsigned int UnsignedWideMulNarrowDiv(unsigned int a, unsigned int const b, unsigned int const c)
{
	_asm
	{
		mov eax,a
		mul b
		div c
		mov a,eax
	}
	return a;
}

static int SignedMulFixed(int a, int const b)
{
	_asm
	{
		mov eax,a
		imul b
		shrd eax,edx,16
		mov a,eax
	}
	return a;
}

static unsigned int UnsignedMulFixed(unsigned int a, unsigned int const b)
{
	_asm
	{
		mov eax,a
		mul b
		shrd eax,edx,16
		mov a,eax
	}
	return a;
}

static int SignedDivFixed(int a, int const b)
{
	_asm
	{
		mov eax,a
		cdq
		shld eax,edx,16
		idiv b
		mov a,eax
	}
	return a;
}

static unsigned int UnsignedDivFixed(unsigned int a, unsigned int const b)
{
	_asm
	{
		mov eax,a
		xor edx,edx
		shld eax,edx,16
		div b
		mov a,eax
	}
	return a;
}

#else

#error "64 bit data type n/a"

#endif



#define RED_LUMINOSITY 77
#define GREEN_LUMINOSITY 150
#define BLUE_LUMINOSITY 29
#define MAX_PIXVAL 255

#define RGB_CLAMP_MACRO(r,g,b,maxval) do { _PCLAMP(r,g,b,maxval,RED_LUMINOSITY,GREEN_LUMINOSITY,BLUE_LUMINOSITY) _PCLAMP(g,r,b,maxval,GREEN_LUMINOSITY,RED_LUMINOSITY,BLUE_LUMINOSITY) _PCLAMP(b,r,g,maxval,BLUE_LUMINOSITY,RED_LUMINOSITY,GREEN_LUMINOSITY) } while (0)

#define _PCLAMP(p,i,j,maxval,Ep,Ei,Ej) \
	if ((p) > (maxval)) \
	{ \
		unsigned int iAdd = (maxval)-(unsigned int)(i); \
		unsigned int jAdd = (maxval)-(unsigned int)(j); \
		unsigned int energy = (Ep)*((p)-(maxval)); \
		unsigned int maxenergy = iAdd*(Ei)+jAdd*(Ej); \
		(p)=(maxval); \
		if (energy>maxenergy) \
		{ \
			(i) = (maxval); \
			(j) = (maxval); \
		} \
		else \
		{ \
			if (iAdd && energy) (i) = (i) + UnsignedWideMulNarrowDiv(iAdd , energy , maxenergy); \
			if (iAdd && energy) (j) = (j) + UnsignedWideMulNarrowDiv(jAdd , energy , maxenergy); \
		} \
	}

#define MAX_SHADES_PER_PIXEL(_lights) ((MAX_PIXVAL*(RED_LUMINOSITY+GREEN_LUMINOSITY+BLUE_LUMINOSITY)*(_lights)->palette_num_shades>>16)+1)

#define INIT_INSERT_SHADES_MACRO(_lights) \
	Lighting_Style __adjusted_lights = *_lights; \
	AdjustLSBlocksForPalette(&__adjusted_lights);

#define INSERT_SHADES_INTO_TLT_MACRO(lights,rval,gval,bval,count,insert) \
	_INSERT_SHADES_MACRO(lights,rval,gval,bval,count,insert,_TLT_GEN_STEPS)

#define INSERT_SHADES_INTO_PALETTE_MACRO(lights,rval,gval,bval,count,insert) \
	_INSERT_SHADES_MACRO(&__adjusted_lights,rval,gval,bval,count,insert,_PALETTE_GEN_STEPS)

#define _TLT_GEN_STEPS(_lights,_rval,_gval,_bval) \
	unsigned int num_steps_ = (_lights)->table_num_shades-1;

#define _PALETTE_GEN_STEPS(_lights,_rval,_gval,_bval) \
	unsigned int intensity_ = (unsigned int)(_rval)*RED_LUMINOSITY + (unsigned int)(_gval)*GREEN_LUMINOSITY + (unsigned int)(_bval)*BLUE_LUMINOSITY; /* in 16.16 fixed */ \
	unsigned int num_steps_ = (intensity_ * (_lights)->palette_num_shades >> 16) + 1;

#define _INSERT_SHADES_MACRO(_lights,_rval,_gval,_bval,_count,_insert,_initialize_num_steps) \
	{ \
		LS_Block const * blkP_ = (_lights)->blocks; \
		unsigned int n_blks_ = (_lights)->num_blocks-1; \
		 \
		_initialize_num_steps(_lights,_rval,_gval,_bval) \
		unsigned int step_size_ = (1<<16)/num_steps_; \
		 \
		unsigned int r_unit_ = (unsigned int)(_rval) * step_size_; \
		unsigned int g_unit_ = (unsigned int)(_gval) * step_size_; \
		unsigned int b_unit_ = (unsigned int)(_bval) * step_size_; \
		 \
		unsigned int r_add_ = UnsignedMulFixed(r_unit_, blkP_->grad); \
		unsigned int g_add_ = UnsignedMulFixed(g_unit_, blkP_->grad); \
		unsigned int b_add_ = UnsignedMulFixed(b_unit_, blkP_->grad); \
		 \
		unsigned int rf_=1<<15,gf_=1<<15,bf_=1<<15,xf_=step_size_; \
		 \
		unsigned int lc = num_steps_; \
		do \
		{ \
			if (xf_ > blkP_->end_input && n_blks_) \
			{ \
				unsigned int part_ = UnsignedMulFixed(blkP_->end_input - (xf_ - step_size_), blkP_->grad); \
				rf_ += (_rval) * part_; \
				gf_ += (_gval) * part_; \
				bf_ += (_bval) * part_; \
				 \
				++ blkP_; \
				Asura_Assert(xf_ > blkP_->begin_input,""); \
				-- n_blks_; \
				r_add_ = UnsignedMulFixed(r_unit_, blkP_->grad); \
				g_add_ = UnsignedMulFixed(g_unit_, blkP_->grad); \
				b_add_ = UnsignedMulFixed(b_unit_, blkP_->grad); \
				 \
				part_ = UnsignedMulFixed(xf_ - blkP_->begin_input, blkP_->grad); \
				rf_ += (_rval) * part_; \
				gf_ += (_gval) * part_; \
				bf_ += (_bval) * part_; \
			} \
			else \
			{ \
				rf_ += r_add_; \
				gf_ += g_add_; \
				bf_ += b_add_; \
			} \
			xf_ += step_size_; \
			 \
			{ \
				unsigned int rt_ = rf_, gt_ = gf_, bt_ = bf_; \
				RGB_CLAMP_MACRO (rt_,gt_,bt_,MAX_PIXVAL<<16); \
				_insert((unsigned char)(rt_>>16), (unsigned char)(gt_>>16), (unsigned char)(bt_>>16),(_count)); \
			} \
		} \
		while (--lc); \
	}

#ifdef __WATCOMC__
#pragma warning 555 5
#endif

#ifdef __cplusplus
};
#endif /* __cplusplus */

#endif /* !_included_histgram_h */
