//
// Tools_PSP_Texture_Writer.h
//
// Created by KJFL - 2004-02-11 15:08:10
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_PSP_TEXTURE_WRITER_H__
#define __TOOLS_PSP_TEXTURE_WRITER_H__

#include "../PSP/Rendering/Asura_PSP_TextureHeader.h"

// Needed for PALETTE_GENERATION_PTR
#include "ImageProcessing/PaletteGenerationDll.h"

// in middleware.
#include "NVidia/DDS_Utilities/dxtlib.h"

// storage for 1 mipmap level
class Tools_PSP_ImageData
{
	public:
		Tools_PSP_ImageData()
		{
			m_puPixelData=0;
			m_uWidth=0;
			m_uHeight=0;
			m_uImageSizeInBytes = 0;
		}

		~Tools_PSP_ImageData()
		{
			delete [] m_puPixelData;
		}

		void Reset(void)
		{
			delete [] m_puPixelData;

			m_puPixelData=0;
			m_uWidth=0;
			m_uHeight=0;
			m_uImageSizeInBytes = 0;
		}

		bool IsTooSmall(int iTextureFormat =  ASURA_PSP_TEXTUREFORMAT_8888) const
		{
			u_int uSizeInBytes = m_uWidth*m_uHeight*4;
			if(iTextureFormat == ASURA_PSP_TEXTUREFORMAT_IDX4)
			{
				uSizeInBytes = m_uWidth*m_uHeight/2;
			}
			else if(iTextureFormat == ASURA_PSP_TEXTUREFORMAT_IDX8)
			{
				uSizeInBytes = m_uWidth*m_uHeight;
			}
			else
			{
				Asura_Assert(iTextureFormat == ASURA_PSP_TEXTUREFORMAT_8888,"");
				uSizeInBytes = m_uWidth*m_uHeight*4;
			}
		
			return !m_uHeight || (uSizeInBytes/m_uHeight)<16;
		}
		
		u_int*	m_puPixelData;
		u_int	m_uWidth;
		u_int	m_uHeight;
		u_int	m_uImageSizeInBytes;
};

class Tools_PSP_Texture_Writer
{
	public:

		typedef void (PaletteAdjustmentHook)(u_int uNumEntries, u_int* puPalette);

		// 10 images allows for 1024x1024 to 2x2, which fits in with current texture assets
		enum { MAXIMUM_NUMBER_OF_IMAGES = 10 };

		Tools_PSP_Texture_Writer();
		~Tools_PSP_Texture_Writer();

		void Initialise(const u_int uMipLevel, const u_int uWidth, const u_int uHeight, const u_int uPitch, const void* pPixels);
		
		bool WriteOutFile(const char* szOutputFilename, const struct TextureConversionData& xConvertData, bool bSkybox = false);

		static void SetPaletteGenerationOverride(PALETTE_GENERATION_PTR const pfnPaletteGenerationOverride = NULL) { s_pfnPaletteGenerationOverride = pfnPaletteGenerationOverride; };
		static void SetPaletteAdjustmentHook(PaletteAdjustmentHook* const pfnPaletteAdjustment = NULL) { s_pfnPaletteAdjustment = pfnPaletteAdjustment; };

	private:
		// KOR - 07/12/2005 - helper functions that used to be just static in the cpp.
		PALETTE_GENERATION_PTR GetPaletteGenerationFunctionFromDllForPSP();

		void SwizzleTexture( unsigned char* const texels, const unsigned int stride, const unsigned int height );
		void SwizzleBitmap(u_char* pucDest, u_char* pucSrc, int iWidth, int iHeight, int iFormat );

		void GetImages();
		bool GeneratePalette( const enum ASURA_PSP_TEXTUREFORMAT eTextureFormat, const enum ASURA_PSP_CLUTFORMAT eCLUTFormat, const struct TextureConversionData& xConvertData );
		void ConvertPixelData( const enum ASURA_PSP_TEXTUREFORMAT eTextureFormat );
		static NV_ERROR_CODE PSPMipMapCallback(const void* pPixelData, size_t uCount, const MIPMapData* pxMipMapData, void* pUserData);
		NV_ERROR_CODE PSPMipMapCallback(const void* pPixelData, size_t uCount, const MIPMapData* pxMipMapData);

		static PALETTE_GENERATION_PTR s_pfnPaletteGenerationOverride;
		static PaletteAdjustmentHook* s_pfnPaletteAdjustment;

		Asura_PSP_TextureHeader	m_xHeader;		
		Tools_PSP_ImageData		m_axImage[MAXIMUM_NUMBER_OF_IMAGES];
		u_int					m_uNumberOfImages;
		u_int*					m_puPalette;
		u_int					m_uSizeOfPaletteInBytes;
};


#endif//__TOOLS_PSP_TEXTURE_WRITER_H__
