//
// Tools_ColourPicker_Internal.h
//
// Created by Richard Rice - 28/07/2024
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_COLOURPICKER_INTERNAL_H__
#define __TOOLS_COLOURPICKER_INTERNAL_H__

#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"

class Tools_ColourPicker;

//derived overlay class to allow the colour picker to have a per-frame update function
class Tools_ColourPicker_Overlay : public  Asura_Overlay
{
public :
	Tools_ColourPicker_Overlay(Tools_ColourPicker* pxPicker,const char* pszTitle,class Asura_Vector_2* pxPosition);

private :
	virtual void PerFrameUpdate(void);

	Tools_ColourPicker* m_pxPicker;
	
};



//Dialog for the Hue/Saturation rectangle
class Tools_ColourPicker_HueSat_Dialog : public Asura_Dialog
{
public :
	Tools_ColourPicker_HueSat_Dialog(Asura_Vector_2 xOffset,float* pfHue,float* pfSat,const float* pfLum,bool* pbChanged);
};

//Gadget for the Hue/Saturation rectangle
class Tools_ColourPicker_HueSat_Gadget : public Asura_Gadget
{
public :
	Tools_ColourPicker_HueSat_Gadget(Asura_Vector_2 xOffset,float* pfHue,float* pfSat,const float* pfLum,bool* pbChanged);

private :
	virtual void Render( const Asura_Vector_2& xOrigin );
	virtual void Update( const Asura_Vector_2& xOrigin );


	float* m_pfHue;
	float* m_pfSat;
	const float* m_pfLum;
	bool* m_pbChanged;
	
};

//Dialog for the Luminance strip
class Tools_ColourPicker_Lum_Dialog : public Asura_Dialog
{
public :
	Tools_ColourPicker_Lum_Dialog(Asura_Vector_2 xOffset,const float* pfHue,const float* pfSat,float* pfLum,bool* pbChanged);
};

//Gadget for the Luminance strip
class Tools_ColourPicker_Lum_Gadget : public Asura_Gadget
{
public :
	Tools_ColourPicker_Lum_Gadget(Asura_Vector_2 xOffset,const float* pfHue,const float* pfSat,float* pfLum,bool* pbChanged);

private :
	virtual void Render( const Asura_Vector_2& xOrigin );
	virtual void Update( const Asura_Vector_2& xOrigin );


	const float* m_pfHue;
	const float* m_pfSat;
	float* m_pfLum;
	bool* m_pbChanged;
	
};

//Dialog for the alpha strip
class Tools_ColourPicker_Alpha_Dialog : public Asura_Dialog
{
public :
	Tools_ColourPicker_Alpha_Dialog(Asura_Vector_2 xOffset,const float* pfRed,const float* pfGreen,const float* pfBlue,float* pfAlpha,bool* pbChanged);
};

//Gadget for the alpha strip
class Tools_ColourPicker_Alpha_Gadget : public Asura_Gadget
{
public :
	Tools_ColourPicker_Alpha_Gadget(Asura_Vector_2 xOffset,const float* pfRed,const float* pfGreen,const float* pfBlue,float* pfAlpha,bool* pbChanged);

private :
	virtual void Render( const Asura_Vector_2& xOrigin );
	virtual void Update( const Asura_Vector_2& xOrigin );


	const float* m_pfRed;
	const float* m_pfGreen;
	const float* m_pfBlue;
	float* m_pfAlpha;
	bool* m_pbChanged;
	
};


//Dialog for showing the current colour
class Tools_ColourPicker_Colour_Dialog : public Asura_Dialog
{
public :
	Tools_ColourPicker_Colour_Dialog(Asura_Vector_2 xOffset,Asura_Vector_2 xSize,const float* pfRed,const float* pfGreen,const float* pfBlue,float fMax);
	Tools_ColourPicker_Colour_Dialog(Asura_Vector_2 xOffset,Asura_Vector_2 xSize,const float* pfRed,const float* pfGreen,const float* pfBlue,const float* pfAlpha,float fMax);
};


//Gadget for drawing a coloured button
class Tools_ColourPicker_Colour_Gadget : public Asura_Gadget
{
public :
	Tools_ColourPicker_Colour_Gadget(Asura_Vector_2 xOffset,Asura_Vector_2 xSize,const float* pfRed,const float* pfGreen,const float* pfBlue,const float* pfAlpha,float fMax);
	Tools_ColourPicker_Colour_Gadget(Asura_Vector_2 xOffset,Asura_Vector_2 xSize,const Asura_Dialog_SharedValue<float>* pxSharedRed,const Asura_Dialog_SharedValue<float>* pxSharedGreen,const Asura_Dialog_SharedValue<float>* pxSharedBlue,const Asura_Dialog_SharedValue<float>* pxSharedAlpha,float fMax);

	void SetClickedFunction(void(*pfnClickedFunction)(int),int iUserData)
	{m_pfnIntFunction = pfnClickedFunction;m_iUserData = iUserData;}


private :
	virtual void Render( const Asura_Vector_2& xOrigin );
	virtual void Update( const Asura_Vector_2& xOrigin );


	union
	{
		struct
		{
			const float* m_pfRed;
			const float* m_pfGreen;
			const float* m_pfBlue;
			const float* m_pfAlpha;
		};

		struct
		{
			const Asura_Dialog_SharedValue<float>* m_pxSharedRed;
			const Asura_Dialog_SharedValue<float>* m_pxSharedGreen;
			const Asura_Dialog_SharedValue<float>* m_pxSharedBlue;
			const Asura_Dialog_SharedValue<float>* m_pxSharedAlpha;
		};
	};

	Asura_Vector_2 m_xSize;

	float m_fMax;//indicates maximum values for RGB
	bool m_bSharedValues;

	void(*m_pfnIntFunction)(int);
	int m_iUserData;


};


class Tools_ColourPicker_Sampler_Dialog : public Asura_Dialog
{
public :
	Tools_ColourPicker_Sampler_Dialog(Asura_Vector_2 xOffset,int iSampleSize,float* pfRed,float* pfGreen,float* pfBlue,bool* pbChanged);
	 
};

class Tools_ColourPicker_Sampler_Gadget : public Asura_Icon_Gadget
{
public :
	Tools_ColourPicker_Sampler_Gadget(Asura_Vector_2 xOffset,int iSampleSize,int iIconIndex,float* pfRed,float* pfGreen,float* pfBlue,bool* pbChanged);

	virtual void Update( const Asura_Vector_2& xOrigin );

	virtual void FocusLost(int iFocusType);


private :
	void StartPicking();
	void AcceptColour();
	void CancelPicking();

	float m_fRedCopy;
	float m_fGreenCopy;
	float m_fBlueCopy;
	int m_iSampleSize;

	float* m_pfRed;
	float* m_pfGreen;
	float* m_pfBlue;
	bool* m_pbChanged;

	bool m_bPicking;
	
};

//////////////////////////////////////////////////////////////////////////////

//Dialog for showing a preset colour that can be picked
class Tools_ColourPicker_PresetColour_Dialog : public Asura_Dialog
{
public :
	Tools_ColourPicker_PresetColour_Dialog(Asura_Vector_2 xOffset,Tools_ColourPicker* pxColourPicker,u_int* puColour,bool bCanBeChanged);
};


//Gadget for the preset colour dialog
class Tools_ColourPicker_PresetColour_Gadget : public Asura_Gadget
{
public :
	Tools_ColourPicker_PresetColour_Gadget(Asura_Vector_2 xOffset,Asura_Vector_2 xSize,Tools_ColourPicker* pxColourPicker,u_int* puColour,bool bCanBeChanged);



private :
	virtual void Render( const Asura_Vector_2& xOrigin );
	virtual void Update( const Asura_Vector_2& xOrigin );

	u_int* m_puColour;
	Tools_ColourPicker* m_pxColourPicker;
	bool m_bCanBeChanged;

};


#endif //__TOOLS_COLOURPICKER_INTERNAL_H__
