//
// Tools_Batch_GlobalIllumination.cpp
//
// Created by Andrew Weinkove - 16/09/03
//
// Code to enable batch processing of lighting calculations.
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Lighting/Tools_Batch_GlobalIllumination.h"

#include "Core/Asura_UserInput.h"
#include "Debugging/Asura_Assert.h"
#include "Light/Asura_GI_Solution.h"

#include "Lighting/Tools_Mode_PhotonMapping.h"
#include "Load&Save/Tools_BatchProcessing.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_IniFile.h"
#include "Tools_System.h"

void Tools_Batch_GlobalIllumination::BatchLighting_RelightLevel()
{
	Asura_Assert(Tools_BatchProcessing::IsGlobalIllumination(),"");
	
	Tools_IniFile* pxIniFile = Tools_BatchProcessing::GetIniFile();
	
	//read the settings in the default section, then in the section for this particular level.
	for(int iLoop = 0;iLoop<2;iLoop++)
	{
		if(iLoop == 0)
		{
			if(!pxIniFile->GetIniTextSection("Default"))
			{
				continue;
			}
		}
		else
		{
			if(!pxIniFile->GetIniTextSectionByIndex(Tools_BatchProcessing::GetSectionIndex()))
			{
				continue;
			}
		}

		// (AW - 18.08.2004) - There aren't actually any lighting options available in this
		// mode just yet, but if any need adding they should be added here.
		// See Tools_Level_Export::BatchExport_ExportLevel for an example of how to parse
		// the text file. New commands should also be added to s_apszIdentifiers below.

		pxIniFile->GetBoolInCurrentSection("Force_Recalculate_All",Tools_Mode_PhotonMapping::s_bRecalcAll_ForceRecalcAll);
		pxIniFile->GetBoolInCurrentSection("Quick_ELT",Tools_Mode_PhotonMapping::s_bRecalcAll_QuickELT);
		pxIniFile->GetUIntInCurrentSection("Min_Accuracy",Tools_Mode_PhotonMapping::s_uRecalcAll_MinAccuracy);
		pxIniFile->GetUIntInCurrentSection("Max_Accuracy",Tools_Mode_PhotonMapping::s_uRecalcAll_MaxAccuracy);
	}

	

	if(!Tools_BatchProcessing::IsAborted())
	{
		Tools_System::RequestPhotonMappingMode();
		
		//recalculate everything
		Tools_Mode_PhotonMapping::RecalculateAllMain();
		
		//If escape was held down , flag batch processing as aborted
		if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE))
		{
			Tools_BatchProcessing::Abort();
		}

		if (!Tools_BatchProcessing::IsAborted())
		{
			// (AW - 18.08.2004) - Save the relevant files here.
			// (RAR - 07.09.2004) - Use the mode quick save instead , since some non-lighting files
			//	may need saving as well (as a result of templates being updated);
			Tools_System::QuickSave();
		}
	}
}

void Tools_Batch_GlobalIllumination::BatchLighting_GetIdentifiers( List<const char*>* pxIdentifiers )
{
	static const char* const aszIdentifiers[] =
	{
		"File",
		"Accuracy",
		"Min_Accuracy",
		"Max_Accuracy",
		"Accuracy",
		"Force_Recalculate_All",
		"Quick_ELT",
	};

    int iNumIdentifiers = sizeof(aszIdentifiers) / sizeof(const char*);
    for( int i = 0; i < iNumIdentifiers; ++i )
    {
        pxIdentifiers->add_entry_end( aszIdentifiers[i] );
    }
}

