//
// Tools_Environment_LOS.cpp
//
// Created by KJFL - 2004-04-23 15:32:03
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Lighting/Tools_Environment_LOS.h"

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
#include "Core/Asura_TextureFlags.h"
#include "Materials/Tools_MaterialResponse_System.h"
#endif
			
bool					Tools_Environment_LOS::s_bIsInitialised = false;

u_int					Tools_Environment_LOS::s_uNumberOfZones;
Tools_EnvironmentLOSZone* Tools_Environment_LOS::s_pxZones;

u_int					Tools_Environment_LOS::s_uNumberOfModules;
Tools_Large_AABB_Tree*	Tools_Environment_LOS::s_pxModulesTree;
Tools_Large_AABB_Tree**	Tools_Environment_LOS::s_ppxPolyTrees;
Tools_Large_AABB_Tree**	Tools_Environment_LOS::s_ppxVertexTrees;
u_int*					Tools_Environment_LOS::s_puModuleToZoneIndex;


void 							(*Tools_Environment_LOS::s_pfAABBCallback)(u_int uIndex);
const Asura_Bounding_Box*		Tools_Environment_LOS::s_pxTestAABB;
u_int*							Tools_Environment_LOS::s_puModuleIndex;



bool Tools_Environment_LOS::PolygonTestCallback(u_int uIndex, float fBBDistance, Tools_Large_AABB_Tree_RayData* pxData )
{

	Tools_Environment_LOS_Data* pxInternalData = static_cast<Tools_Environment_LOS_Data*>(pxData);

	if ( fBBDistance > pxData->GetDistance() )
	{
		Asura_Assertf( false, ( "Code bug: BB dist (%f) > current dist (%f)", fBBDistance, pxData->GetDistance() ));
		return true;
	}


	Asura_Vector_3 xVertex[4];
	const Asura_Mesh* pxMesh = pxInternalData->GetCurrentZone()->m_pxModuleHandler->GetMeshForModule(pxInternalData->GetCurrentLocalModuleIndex());

	u_int uFlags = 0;

	if (pxMesh->PolygonFlagDataPtr && pxMesh->PolygonFlagDataValid)
	{
		uFlags = pxMesh->PolygonFlagDataPtr[uIndex].Flags;
	}


	if ( pxInternalData->GetPolyFlagsToIgnore() & uFlags)
	{
		/// you ain't seen me, right?
		return true;
	}

	//Check for required polygon flags
	const u_int uFlagsToRequire = pxInternalData->GetPolyFlagsToRequire();
	if(uFlagsToRequire)
	{
		if ((uFlagsToRequire & uFlags) != uFlagsToRequire)
		{
			return true;
		}
	}


	// KJFL - 3 January 2008 - ignoring tranparent polys is actually very costly, because on SM3 this
	// means querying the material response system and a hash look-up. So, best to do it last after the
	// other tests - possibly even after the ray-tri tests?
	if ( pxInternalData->GetIgnoreTransparent() )
	{
	
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		// KJFL - 15 February 2008 - need to use Tools_MaterialResponse_System as Asura_ side
		// won't be thread-safe for these calls when in Tools (it caches data in static members).

		int iMaterialIndex = 0;
		const Tools_MaterialResponse* pxMaterial = Tools_MaterialResponse_System::GetMaterialByHashID( pxMesh->TextureDataPtr[uIndex].m_auMaterialResponseHashID[0], &iMaterialIndex );

		if ( pxMaterial )
		{
			// if we have a material, is it slaved to a parent?
			int iParentIndex = 0;
			const Tools_MaterialResponse* pxParent = Tools_MaterialResponse_System::GetMaterialByHashID( pxMaterial->GetParentHash(), &iParentIndex );	
			
			if ( pxParent )
			{
				pxMaterial = pxParent;
			}

			// so, if it's not opaque, or has a texture in the transparency map slot, it's transparent and we'll ignore it and return out.
			if ( pxMaterial->GetBlendMode() != Asura_MaterialResponse::BLEND_MODE_OPAQUE || pxMaterial->GetTexture(Asura_MaterialResponse::TRANSPARENCY_MAP) != ASURA_TEXTURE_NONE )
			{
				return true;
			}
		}
#else
		if( Asura_Material_System::IsMaterialTransparent( pxMesh->TextureDataPtr[uIndex].m_iMaterialID ) )
		{
			return true;
		}
#endif

	}

	// KJFL - 2004-06-08 17:27:19 - want option to skip AsuraPolygonFlag_DontBlockLight polys if required, i.e. not all the time?
	// bit nasty - photon mapping needs to 'see' them to light them, but at the same time you don't want to block light with them.
	// So... 
	// 1) LT - just ignore them
	// 2) photon mapping - don't ignore them, but 'reflect' photons straight through with no energy loss?#
	// have FlagsToIgnore option in root los call

	const Asura_Vector_3& xPos = pxInternalData->GetCurrentZone()->m_pxModuleHandler->GetPositionOfModule( pxInternalData->GetCurrentLocalModuleIndex() );

	const Asura_Vector_3 xOriginInModuleSpace = pxData->GetRayOrigin() - xPos;

	const Asura_Mesh_Polygon& xPolygon =  pxMesh->PolygonDataPtr[uIndex];


	bool bFacingToward;
	float fDistance = Asura_Maths::RayIntersectsTri
	(
		xOriginInModuleSpace,
		pxData->GetRayDirection(),
		pxMesh->VertexDataPtr[xPolygon.VertexID[0]].Position,
		pxMesh->VertexDataPtr[xPolygon.VertexID[1]].Position,
		pxMesh->VertexDataPtr[xPolygon.VertexID[2]].Position,
		bFacingToward
	);

	if (fDistance >= 0.0f && fDistance < pxInternalData->GetDistance() )
	{
		pxInternalData->SetDistance( fDistance );

		pxInternalData->SetHitData( pxInternalData->GetCurrentZone()->m_uZoneHash, pxInternalData->GetCurrentLocalModuleIndex(), uIndex, bFacingToward );	

		return !pxInternalData->GetReturnOnFirstHit();
	}
	else if (xPolygon.IsQuad())
	{
		// A quad, check the other triangle.
		fDistance = Asura_Maths::RayIntersectsTri
		(
			xOriginInModuleSpace,
			pxData->GetRayDirection(),
			pxMesh->VertexDataPtr[xPolygon.VertexID[2]].Position,
			pxMesh->VertexDataPtr[xPolygon.VertexID[3]].Position,
			pxMesh->VertexDataPtr[xPolygon.VertexID[0]].Position,
			bFacingToward
		);
	
		if (fDistance >= 0.0f && fDistance < pxInternalData->GetDistance() )
		{
			pxInternalData->SetDistance( fDistance );
	
			pxInternalData->SetHitData( pxInternalData->GetCurrentZone()->m_uZoneHash, pxInternalData->GetCurrentLocalModuleIndex(), uIndex, bFacingToward );	

			return !pxInternalData->GetReturnOnFirstHit();

		}
	}

	return true;
}