//
// Tools_EnvironmentalLightTransport.h
//
// Created by KJFL - 2004-04-27 10:18:15
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_ENVIRONMENTALLIGHTTRANSPORT_H__
#define __TOOLS_ENVIRONMENTALLIGHTTRANSPORT_H__

#include "Core/Asura_Maths.h"
#include "Core/Asura_HashTree.h"

#ifndef ASURA_POLYFLAG_DARKEN_SCALE
#define ASURA_POLYFLAG_DARKEN_SCALE (0.5f)
#endif //ASURA_POLYFLAG_DARKEN_SCALE

class Tools_Environment_LOS_Data;

class Tools_EnvironmentalLightTransport_Handler
{
	// chunk loaders can access whatever they like
	friend class Asura_Chunk_EnvironmentalLightTransport;
	friend class Tools_EnvironmentalLightTransport;

	public:
		
		Tools_EnvironmentalLightTransport_Handler();
		~Tools_EnvironmentalLightTransport_Handler();
		
		void Initialise( void );
		void Reset( void );
		void Shutdown( void );

		bool DoesEnvironmentMatch( void );
		
		bool IsEnvironmentProcessed( void )
		{
			return m_uNumberOfModules!=0;
		}

		bool MakeBlankData( void );

		void GetDiffuseForVertex( Asura_SphericalHarmonics_Order2_RGB& xDiffuseSH, const u_int uVertex, const u_int uModule );

		u_int GetDiffuseForVertex( const Asura_Vector_3& xPosition, const Asura_Vector_3& xNormal, const u_int uPoly, const u_int uVertex, const u_int uModule, const u_int uPaintedColour = 0x808080 );

		// JAT 21/07/07 modify a vertex lighting SH to include the effects of vertex painting
		void UpdateLightingFromVertexPainting( Asura_SphericalHarmonics_Order2_RGB& xHarmonic, const class Asura_Mesh* const pxMesh, const int iPolygonIndex, const int iVertex );

		bool GetTransportHarmonic( Asura_SphericalHarmonics_Order3& xSH, const u_int uModule, const u_int uVertex );

		bool IsValid() {return m_uNumberOfModules && m_bValidData;}
		bool IsQuickData(){return m_bQuickData;}

		void Invalidate() {m_bValidData = false;}

		//Updates the ELT in response to a module being deleted
		void DeleteModule( u_int uModuleIndex);


		static u_int GetDirectLightingExtraSharpenFactor()
		{
			return s_uDirectLightingExtraSharpenFactor;
		}
		static float GetDirectLightingExtraIntensityFactor()
		{
			return s_fDirectLightingExtraIntensityFactor;
		}
		static float GetInterreflectedScaleFactor()
		{
			return s_fInterreflectedScaleFactor;
		}

		
		static void SetDirectLightingExtraSharpenFactor(const u_int uFactor)
		{
			s_uDirectLightingExtraSharpenFactor = uFactor;
		}
		static void SetDirectLightingExtraIntensityFactor(const float fFactor)
		{
			s_fDirectLightingExtraIntensityFactor = fFactor;
		}
		static void SetInterreflectedScaleFactor(const float fFactor)
		{
			s_fInterreflectedScaleFactor = fFactor;
		}
		static void SetSamplingHeight(const float fHeight)
		{
			s_fSamplingHeight = fHeight;
		}


	private:
		bool GetSmoothNormalForVertex( Asura_Vector_3& xNormal, u_int uModuleIndex, u_int uVertexIndex );
		static void GetInterpolatedSH( Asura_SphericalHarmonics_Order3& xReturnSH, const Asura_Vector_3& xPosition, const u_int uPolyIndex, const u_int uModuleIndex, const Asura_Hash_ID uZoneHash );

		bool ProcessEnvironment_FirstPass( const u_int uNumSamples );
		bool ProcessEnvironment_SecondPass( const u_int uNumSamples );
		void ProcessEnvironment_Finish( const u_int uNumSamples, bool bQuickApproximation);

		// multithreaded versions
		bool ProcessEnvironment_FirstPass_MT( const u_int uNumSamples );
		static void ProcessEnvironment_FirstPass_MT_Callback( const void* pxTaskDataPacket );
	
		bool ProcessEnvironment_SecondPass_MT( const u_int uNumSamples );
		static void ProcessEnvironment_SecondPass_MT_Callback( const void* pxTaskDataPacket );
		
		bool InitialiseModuleData( void );

		static bool ShouldTestModuleForELTLOS( Asura_Hash_ID uZone, u_int uModuleIndex, const void* pCurrentModuleIndex, const Tools_Environment_LOS_Data& xLOSData );

		class Tools_EnvironmentalLightTransport_Module*	m_pxModules;

		u_int								m_uNumberOfModules;

		bool									m_bValidData;
		bool									m_bQuickData;//Indicates whether ELT was calculated using the quicker/approximate version

		static u_int	s_uDirectLightingExtraSharpenFactor;
		static float	s_fDirectLightingExtraIntensityFactor;
		static float	s_fInterreflectedScaleFactor;
		static float	s_fSamplingHeight;
};

class Tools_EnvironmentalLightTransport
{
	// chunk loaders can access whatever they like
	friend class Asura_Chunk_EnvironmentalLightTransport;

	public:
		
		static void Initialise( void );
		static void Reset( void );
		static void Shutdown( void );

		static bool DoesEnvironmentMatch( void ) 
		{ return GetCurrentHandler().DoesEnvironmentMatch();}
		
		static bool IsEnvironmentProcessed( void ) 
		{return GetCurrentHandler().IsEnvironmentProcessed();}

		static bool MakeBlankData( void )
		{ return GetCurrentHandler().MakeBlankData(); }

		static void ProcessEnvironment( bool bQuickApproximation = false ) ;

		static void GetDiffuseForVertex( Asura_SphericalHarmonics_Order2_RGB& xDiffuseSH, const u_int uVertex, const u_int uModule )
		{ GetCurrentHandler().GetDiffuseForVertex(xDiffuseSH,uVertex,uModule); }

		static u_int GetDiffuseForVertex( const Asura_Vector_3& xPosition, const Asura_Vector_3& xNormal, const u_int uPoly, const u_int uVertex, const u_int uModule, const u_int uPaintedColour = 0x808080 )
		{ return GetCurrentHandler().GetDiffuseForVertex(xPosition,xNormal,uPoly,uVertex,uModule, uPaintedColour ); }

		// JAT 21/07/07 modify a vertex lighting SH to include the effects of vertex painting
		static void UpdateLightingFromVertexPainting( Asura_SphericalHarmonics_Order2_RGB& xHarmonic, const Asura_Mesh* const pxMesh, const int iPolygonIndex, const int iVertex )
		{
			return GetCurrentHandler().UpdateLightingFromVertexPainting( xHarmonic, pxMesh, iPolygonIndex, iVertex );
		}

		static bool GetTransportHarmonic( Asura_SphericalHarmonics_Order3& xSH, const u_int uModule, const u_int uVertex )
		{ return GetCurrentHandler().GetTransportHarmonic(xSH, uModule, uVertex); }

		static bool IsValid() {return GetCurrentHandler().IsValid();}
		static bool IsQuickData(){return GetCurrentHandler().IsQuickData();}

		static void Invalidate() {GetCurrentHandler().Invalidate();}

		//Updates the ELT in response to a module being deleted
		static void DeleteModule( u_int uModuleIndex)
		{ GetCurrentHandler().DeleteModule(uModuleIndex);}


	#if defined(ASURA_USE_STREAMING)
		// Handler functions...
		static Tools_EnvironmentalLightTransport_Handler* CreateHandler (const Asura_Hash_ID uHash);
		static bool DestroyHandler(const Asura_Hash_ID uHash);

		static void CreateAndSetHandler (const Asura_Hash_ID uHash) { s_pxCurrentHandler = CreateHandler(uHash); }
		static void SetCurrentHandler (Tools_EnvironmentalLightTransport_Handler* pxHandler);
		static void SetCurrentHandler (const Asura_Hash_ID uHash) { s_pxCurrentHandler = s_xHandlerTree.GetDataByHash(uHash); }
		static Tools_EnvironmentalLightTransport_Handler&	GetCurrentHandler(void) { return(*s_pxCurrentHandler); }
		static Tools_EnvironmentalLightTransport_Handler*	GetHandler(const Asura_Hash_ID uHash){ return(s_xHandlerTree.GetDataByHash(uHash)); }
	#else
		// Private functions...
		static Tools_EnvironmentalLightTransport_Handler& GetCurrentHandler (void){ return(s_xHandler); }
	#endif	// defined(ASURA_USE_STREAMING)

	private:
		static void ProcessEnvironmentCancelled();
		//static bool GetSmoothNormalForVertex( Asura_Vector_3& xNormal, u_int uModuleIndex, u_int uVertexIndex );
		//static void GetInterpolatedSH( Asura_SphericalHarmonics_Order3& xReturnSH, const Asura_Vector_3& xPosition, const u_int uPolyIndex, const u_int uModuleIndex );
		

		//static bool InitialiseModuleData( void );
	
	#if defined(ASURA_USE_STREAMING)
		// Private variables...
		static	Tools_EnvironmentalLightTransport_Handler* s_pxCurrentHandler;		// Currently active handler.
		static	Asura_TemplateHashTree<Tools_EnvironmentalLightTransport_Handler> s_xHandlerTree;			// Tree of all of the handlers available.
	#else
	// Private variables...
		static	Tools_EnvironmentalLightTransport_Handler s_xHandler;				// Single static-instance of a zone.
	#endif


};

class Tools_EnvironmentalLightTransport_Module
{
	public:

		Tools_EnvironmentalLightTransport_Module()
		{
			m_uNumberOfVertices = 0;
			m_pxLightTransportSHs = 0;
			m_pxInterreflectedSHs = 0;
		}

		~Tools_EnvironmentalLightTransport_Module()
		{
			delete [] m_pxLightTransportSHs;
			delete [] m_pxInterreflectedSHs;
		}

		void Shutdown()
		{
			m_uNumberOfVertices = 0;
			delete [] m_pxLightTransportSHs;
			m_pxLightTransportSHs = 0;
			delete [] m_pxInterreflectedSHs;
			m_pxInterreflectedSHs = 0;
		}

		u_int	m_uNumberOfVertices;
		Asura_SphericalHarmonics_Order3*		m_pxLightTransportSHs;
		Asura_SphericalHarmonics_Order3*		m_pxInterreflectedSHs;
};

#endif//__TOOLS_ENVIRONMENTALLIGHTTRANSPORT_H__
