//
// Tools_GlobalIllumination_LightProbe.cpp
//
// Created by KJFL - 2004-06-18 17:30:58
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Lighting/Tools_GlobalIllumination_LightProbe.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Viewport.h"
#include "Camera/Asura_Camera.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_PlatformRenderObject.h"


#include "Tools_Overlays.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_IniFile.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Shapes/Tools_Shapes.h"

Asura_Vector_2 Tools_GlobalIllumination_LightProbe::s_xOverlayPosition(10,10);

List<Tools_LightProbe_FavoriteShape*> Tools_GlobalIllumination_LightProbe::s_xFavList;
int Tools_GlobalIllumination_LightProbe::s_iFavNamesSize = 0;
const char** Tools_GlobalIllumination_LightProbe::s_ppszFavNames = 0;


Tools_GlobalIllumination_LightProbe::Tools_GlobalIllumination_LightProbe(const Asura_Vector_3* pxPosition,const Asura_Matrix_3x3* pxOrientation)
:Tools_Object(TOT_LightingTest)
{
	Position = *pxPosition;
	Orientation = *pxOrientation;
	//Mesh = Asura_Mesh_System::GetMesh("tools_lighting_ball");
	SetMesh("ToolsShape_GIProbe");


	SnapToLevel();

	m_uShapeID = ASURA_HASH_ID_UNSET;

	m_iFavSelection = 0;
	m_bFavSelectionChanged = false;
}

Tools_GlobalIllumination_LightProbe::~Tools_GlobalIllumination_LightProbe()
{
}




void Tools_GlobalIllumination_LightProbe::OnSelection()
{
	CreateOverlay();
}

void Tools_GlobalIllumination_LightProbe::OnDeselection()
{
	DeleteOverlay();
}

void Tools_GlobalIllumination_LightProbe::CreateOverlay()
{
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay(GetTypeDescription(),false,&s_xOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	DetermineCurrentFavoriteShapeIndex();

		
	int iListSize = Asura_Maths::Min(8,s_iFavNamesSize);
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, s_ppszFavNames, &m_iFavSelection, iListSize, s_iFavNamesSize, NULL, &m_bFavSelectionChanged);
	pxDialog->SetHelpText("Select shape/skin from a list of stored favorites");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Select Skin & Anim",xOffset,ChooseSkin,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Select Anim",xOffset,ChooseAnim,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Select Shape",xOffset,ChooseShape,(int)this);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Add To Favorites",xOffset,AddToFavorite,(int)this);
	pxDialog->SetHelpText("Adds the current shape/skin to the saved favorites list");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Delete From Favorites",xOffset,DeleteFromFavorite,(int)this);
	pxDialog->SetHelpText("Deletes the currently selected entry in the favorites list from the list.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	//set things up so that right clicking on the overlay will be equivalent to right clicking on the object
	//RAR 06/11/2007
	pxOverlay->SetContextMenuCallback(Tools_Object_System::AddContextMenuItemsFromOverlay, this);
	
	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;
 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
	
}

void Tools_GlobalIllumination_LightProbe::DeleteOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
}

void Tools_GlobalIllumination_LightProbe::UpdateWhileSelected()
{
	if(m_bFavSelectionChanged)
	{
		if(m_iFavSelection !=0)
		{
			SetShapeOrSkin(s_xFavList[m_iFavSelection-1]);
		}
		m_bFavSelectionChanged = false;
	}
}

void Tools_GlobalIllumination_LightProbe::ChooseSkin()
{
	Asura_Hierarchy_Skin* pxSkin = 0;
	Asura_Hierarchy_Anim* pxAnim = 0;

	//Choose a skin and animation
	if(!Tools_Animation_Files::ChooseSkin((const Asura_Hierarchy_Skin**)&pxSkin)) return;

	// (AAM - 07.02.2008) No need to require an anim any more, it'll just use the rest pose
	Tools_Animation_Files::ChooseAnimation(pxSkin,(const Asura_Hierarchy_Anim**)&pxAnim);

	ClearShapeAndAnim();
	SetSkinAndAnim(pxSkin,pxAnim);

}

void Tools_GlobalIllumination_LightProbe::ChooseAnim()
{
	if(!GetAnimation())
	{
		//have to choose a skin as well
		ChooseSkin();
		return;
	}

	Asura_Hierarchy_Anim* pxAnim = 0;

	if(!Tools_Animation_Files::ChooseAnimation(GetAnimation()->GetSkin(),(const Asura_Hierarchy_Anim**)&pxAnim)) return;

	SetAnim(pxAnim);

}

void Tools_GlobalIllumination_LightProbe::ChooseShape()
{
	Asura_Hash_ID uShapeID = Tools_ShapeTable::SelectShape(m_uShapeID);
	if(uShapeID == ASURA_HASH_ID_UNSET) return;

	//try to get the Asuar_Mesh for the shape selected
	const char* pszMeshName = Tools_ShapeTable::LookupObjectMeshName(uShapeID);
	if(!pszMeshName)
	{
		ShowMessageOverlay("Failed to get mesh name of selected shape.");
		return;
	}

	Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(pszMeshName);
	if(!pxMesh)
	{
		ShowMessageOverlay("failed to get mesh for selected shape.");
		return;
	}

	//okay , we have the shape so use it
	ClearShapeAndAnim();
	m_uShapeID = uShapeID;
	SetMesh(pxMesh);

}

void Tools_GlobalIllumination_LightProbe::ClearShapeAndAnim()
{
	SetMesh(Asura_Mesh_System::GetMesh("toolsshape_cutsceneactor"));
	m_uShapeID = ASURA_HASH_ID_UNSET;

}

void Tools_GlobalIllumination_LightProbe::LoadFavoriteShapeList()
{
	DeleteFavoriteShapeList();

	Tools_IniFile xIni("ToolsIniData\\LightProbe.ini",false);
	if(xIni.LoadIniFile())
	{
		//get a list of section names , which will correspond to the 'shape' names
		List<char*> xSections = xIni.GetSectionNameList();

		for(LIF<char*> xSectLIF(&xSections);!xSectLIF.done();xSectLIF.next())
		{
			xIni.GetIniTextSection(xSectLIF());
			
			Tools_LightProbe_FavoriteShape* pxFav = new Tools_LightProbe_FavoriteShape;
			pxFav->m_xName = xSectLIF();
			xIni.GetIntInCurrentSection("Shape",*(int*)&pxFav->m_uShapeID);
			xIni.GetIntInCurrentSection("Skin",*(int*)&pxFav->m_uSkinID);
			xIni.GetIntInCurrentSection("Anim",*(int*)&pxFav->m_uAnimID);

			s_xFavList.add_entry(pxFav);			
		}
	}

	BuildFavoriteNameArray();
}

void Tools_GlobalIllumination_LightProbe::BuildFavoriteNameArray()
{
	delete [] s_ppszFavNames;
	s_ppszFavNames = 0;
	s_iFavNamesSize = 0;

	s_iFavNamesSize = s_xFavList.size()+1;

	s_ppszFavNames = new const char*[s_iFavNamesSize];

	int iIndex = 0;
	s_ppszFavNames[iIndex++] = "*None*";

	for(LIF<Tools_LightProbe_FavoriteShape*> xFavLIF(&s_xFavList);!xFavLIF.done();xFavLIF.next())
	{
		s_ppszFavNames[iIndex++] = xFavLIF()->m_xName.GetString();
	}
}

void Tools_GlobalIllumination_LightProbe::SaveFavoriteShapeList()
{
	Tools_FileSystem::MakeWriteable("ToolsIniData\\LightProbe.ini");
	FILE* pFile = fopen("ToolsIniData\\LightProbe.ini","wt");
	if(!pFile) return;

	for(LIF<Tools_LightProbe_FavoriteShape*> xFavLIF(&s_xFavList);!xFavLIF.done();xFavLIF.next())
	{
		Tools_LightProbe_FavoriteShape* pxFav = xFavLIF();
		
		fprintf(pFile,"[%s]\n",xFavLIF()->m_xName.GetString());
		if(pxFav->m_uShapeID != ASURA_HASH_ID_UNSET)
		{
			fprintf(pFile,"Shape = %d\n",pxFav->m_uShapeID);
		}
		if(pxFav->m_uSkinID != ASURA_HASH_ID_UNSET)
		{
			fprintf(pFile,"Skin = %d\n",pxFav->m_uSkinID);
		}
		if(pxFav->m_uAnimID != ASURA_HASH_ID_UNSET)
		{
			fprintf(pFile,"Anim = %d\n",pxFav->m_uAnimID);
		}

		fprintf(pFile,"\n");
	}
	fclose(pFile);
}

void Tools_GlobalIllumination_LightProbe::DeleteFavoriteShapeList()
{
	while(s_xFavList.size())
	{
		delete s_xFavList.first_entry();
		s_xFavList.delete_first_entry();
	}
	delete [] s_ppszFavNames;
	s_ppszFavNames = 0;
	s_iFavNamesSize = 0;
}

void Tools_GlobalIllumination_LightProbe::DetermineCurrentFavoriteShapeIndex()
{
	m_iFavSelection = 0;
	m_bFavSelectionChanged = false;

	int iIndex = 1;

	Asura_Hash_ID uSkinID = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uAnimID = ASURA_HASH_ID_UNSET;
	if(GetAnimation())
	{
		uSkinID = GetAnimation()->GetSkin()->m_uNameHashID;
		uAnimID = GetAnimation()->m_xPrimaryWrapper.GetPrimaryAnimHashID();
	}

	for(LIF<Tools_LightProbe_FavoriteShape*> xFavLIF(&s_xFavList);!xFavLIF.done();xFavLIF.next(),iIndex++)
	{
		Tools_LightProbe_FavoriteShape* pxFav = xFavLIF();
		
		if((pxFav->m_uShapeID == m_uShapeID) && (pxFav->m_uSkinID == uSkinID) && (pxFav->m_uAnimID == uAnimID))
		{
			m_iFavSelection = iIndex;
			break;
		}	
		
	}
}

void Tools_GlobalIllumination_LightProbe::AddToFavorite()
{
	Asura_Hash_ID uSkinID = ASURA_HASH_ID_UNSET;
	Asura_Hash_ID uAnimID = ASURA_HASH_ID_UNSET;
	if(GetAnimation())
	{
		uSkinID = GetAnimation()->GetSkin()->m_uNameHashID;
		uAnimID = GetAnimation()->m_xPrimaryWrapper.GetPrimaryAnimHashID();
	}

	if(m_uShapeID == ASURA_HASH_ID_UNSET && uSkinID == ASURA_HASH_ID_UNSET && uAnimID == ASURA_HASH_ID_UNSET)
	{
		ShowMessageOverlay("A shape/skin must have been chosen first");
		return;
	}

	//check to make sure there isn't already an entry for this
	LIF<Tools_LightProbe_FavoriteShape*> xFavLIF(&s_xFavList);
	for(;!xFavLIF.done();xFavLIF.next())
	{
		Tools_LightProbe_FavoriteShape* pxFav = xFavLIF();
		
		if((pxFav->m_uShapeID == m_uShapeID) && (pxFav->m_uSkinID == uSkinID) && (pxFav->m_uAnimID == uAnimID))
		{
			ShowMessageOverlay("This shape/skin already exists in the favorites list.");
			return;
		}	
	}

	//ask for a name for the shape/skin;
	Asura_String xName;
	if(GetAnimation())
	{
		xName = GetAnimation()->GetSkin()->Name;
	}
	else
	{
		xName = Tools_ShapeTable::LookupObjectMeshName(m_uShapeID);
	}
	GetString_Overlay xOverlay("Save under what name?",xName.GetString());
	if(!xOverlay.DoModal()) return;

	xName = xOverlay.GetString();
	if(xName.IsBlank()) return;

	//create an entry for this shape/skin
	Tools_LightProbe_FavoriteShape* pxFav = new Tools_LightProbe_FavoriteShape;
	pxFav->m_xName = xName;
	pxFav->m_uShapeID = m_uShapeID;
	pxFav->m_uSkinID = uSkinID;
	pxFav->m_uAnimID = uAnimID;

	//add to the list , sroted alphabetically
	for(xFavLIF.restart();!xFavLIF.done();xFavLIF.next())
	{
		if(stricmp(xName.GetString(),xFavLIF()->m_xName.GetString()) < 0)
		{
			s_xFavList.add_entry_before(pxFav,xFavLIF());
			break;
		}
	}
	if(xFavLIF.done())
	{
		s_xFavList.add_entry(pxFav);
	}
		
	//save the changes
	SaveFavoriteShapeList();
	BuildFavoriteNameArray();

	//and update the overlay
	DeleteOverlay();
	CreateOverlay();

}

void Tools_GlobalIllumination_LightProbe::DeleteFromFavorite()
{
	if(m_iFavSelection == 0) return;
	

	int iIndex = 1;//start at 1 , to take into account that the zeroth entry is the special one
	for(LIF<Tools_LightProbe_FavoriteShape*> xFavLIF(&s_xFavList);!xFavLIF.done();xFavLIF.next(),iIndex++)
	{
		if(iIndex == m_iFavSelection)
		{
			if(!ShowYesNoOverlay("Delete %s from the favorites list?",xFavLIF()->m_xName.GetString()))
			{
				return;	
			}
			delete xFavLIF();
			xFavLIF.delete_current();
			break;
		}
	}

	//save the changes
	SaveFavoriteShapeList();
	BuildFavoriteNameArray();

	//and update the overlay
	DeleteOverlay();
	CreateOverlay();
	
}

void Tools_GlobalIllumination_LightProbe::SetShapeOrSkin(const Tools_LightProbe_FavoriteShape* pxFav)
{
	if(pxFav->m_uShapeID != ASURA_HASH_ID_UNSET)
	{
		//shape already in shape table?
		const char* pszMeshName = Tools_ShapeTable::LookupObjectMeshName(pxFav->m_uShapeID);
		if(!pszMeshName)
		{
			//get the filename for this shape and add it
			const char* pszFilename = Asura_HashedFilename_System::GetFilename(pxFav->m_uShapeID,false);
			if(!pszFilename)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to determine filename for shape");
				return;
			}
			Tools_ShapeTable::AddShapeByFilename(pszFilename);

			pszMeshName = Tools_ShapeTable::LookupObjectMeshName(pxFav->m_uShapeID);
			if(!pszMeshName) return;
		}

		Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(pszMeshName);
		if(!pxMesh)
		{
			ShowMessageOverlay("failed to get mesh for selected shape.");
			return;
		}

		//okay , we have the shape so use it
		ClearShapeAndAnim();
		m_uShapeID = pxFav->m_uShapeID;
		Mesh = pxMesh;
		
	}
	else
	{
		Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::GetSkinFromID(pxFav->m_uSkinID);
		if(!pxSkin) return;

		// (AAM - 07.02.2008) Don't return if we can't find the anim, use rest pose instead
		Asura_Hierarchy_Anim* pxAnim = Tools_Animation_Files::GetAnimFromID(pxFav->m_uAnimID);

		ClearShapeAndAnim();
		SetSkinAndAnim(pxSkin,pxAnim);
	}
}
