//
// Tools_Mode_PhotonMapping.h
//
// Created by KJL - 09/01/2004 10:02:31
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_MODE_PHOTONMAPPING_H__
#define __TOOLS_MODE_PHOTONMAPPING_H__

#include "Core/Asura_Maths.h"

#include "Tools_Mode.h"
#include "Misc/Tools_Colour.h"
#include "Misc/list_tem.hpp"
#include "Core/Asura_Handle.h"

class Tools_Mode_PhotonMapping : public Tools_Mode
{
	public:

		Tools_Mode_PhotonMapping()
		{
			Initialise();
		}
		~Tools_Mode_PhotonMapping()
		{
			Shutdown();
		}

		void Initialise(void);
		void Shutdown(void);
		
		void Update(void);
		void RenderWorld_PostToneMap(void);
		void RenderUI(void);

		void QuickSave(void);

		virtual bool IsModified();

		virtual void HandleMeshChange(int iModuleIndex);


		static void RecalcPhotonMapsForSelectedLights(void);
		static void DestroyCurrentLightSource(void);

		
		static void RecalcELT(void);
		static void QuickRecalcELT(void);

		static void RelightEverything(bool bIgnoreDisableRelight);
		static void RelightEverythingNow(bool bIgnoreDisableRelight);
		static void RelightEverything(){RelightEverything(true);}
		static void RecalcAllInvalidLights(void);
		static void QuickRecalcAllInvalidLights();

		static void RecalculateAll();
		static void RecalculateAllMain();

		static void RecalculateEntityGISolution();

		//checks the primary selection (light or polygon light) to see
		//if its using inverted colours for the overlay.
		static bool SelectedLightHasInvertedColour();

		static Asura_Vector_2 s_xModeOverlayPosition;
		static Asura_Vector_2 s_xLightPropertiesOverlayPosition;
		static Asura_Handle	s_iCurrentLightSourceHandle;
		static bool s_bHideStaticObjects;
		static bool s_bHideStaticObjectsChanged;
		static bool s_bHideContainerObjects;
		static bool s_bHideContainerObjectsChanged;
		static bool s_bHideIcons;
		static bool s_bShowLightRadius;

		static bool s_bInvertColourForShadowLights;


		//settings for recalculate all
		static u_int s_uRecalcAll_MinAccuracy;
		static u_int s_uRecalcAll_MaxAccuracy;
		static bool s_bRecalcAll_ForceRecalcAll;
		static bool s_bRecalcAll_QuickELT;

		static bool s_bPolygonSelection;//otherwise object selection is active

		static bool s_bShowGISampleFlag;

		//for light scale overlay
		static Asura_Vector_2 s_xLightScaleOverlayPosition;
		static float s_fLightScale_Power;
		static float s_fLightScale_Red;
		static float s_fLightScale_Green;
		static float s_fLightScale_Blue;
		static bool s_bLightScale_PolygonLights;
		static bool s_bLightScale_ParallelLights;
		static bool s_bLightScale_StandardLights;
		static bool s_bLightScale_SelectedOnly;

		
		static bool	s_bGlobalAmbienceChanged;
		static class Asura_RGBA s_xGlobalAmbience;

		static Asura_Handle s_iMenuHandle;
#ifdef ASURA_USE_TASK_SYSTEM
		// KJFL - 8 August 2008 - a test bool controlled by a GUI tickbox
		static bool s_bUseMultithreading;
#endif

	private:
		void HandleKeyboard();

		static void RecalcEverything(void);

		static void CreateLightSource(void);
		static void CreateParallelLightSource(void);
		static void CreatePolygonLightSource(void);

		
		static void CreateLightProbeObject(void);


		void CreateModeOverlay(void);

		static void CreateLightScaleOverlay();
		static void ApplyLightScale();
		static void AcceptLightScale();
		static void CancelLightScale();
		
		static void UpdateStaticObjectVisibility(void);
		static void UpdateContainerObjectVisibility();
		
		static void ImportLights();
		static void ImportCoronas();

		static void ApplyGlobalAmbienceToAllLightSets();


		//loads/saves data for this mode
		static void TransferModeData(int mode);

		static void DisableRelightClicked();

		static bool s_bDisableRelight;

		//ELT settings for overlay
		static bool s_bELTParamsChanged;
		static u_int s_uDirectLightingExtraSharpenFactor;
		static float s_fDirectLightingExtraIntensityFactor;
		static float s_fInterreflectedScaleFactor;

		// User tweakable rescale factors to improve spherical harmonic downsampling
		static bool s_bSHResampleParamsChanged;
		static float s_fSHZerothOrderResampleScale;
		static float s_fSHFirstOrderResampleScale;

		// Do we need to do a relight on next Update()?
		static bool	 s_bRelightRequired;
		static bool  s_bIgnoreDisableRelightOnNextRelight;

};


#endif//__TOOLS_MODE_PHOTONMAPPING_H__
