//
// Tools_PhotonMapping_DirectEstimation.cpp
//
// Created by KJFL - 2004-05-06 11:30:39
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Lighting/Tools_PhotonMapping_DirectEstimation.h"

#include "Lighting/Tools_Mode_PhotonMapping.h"
#include "Lighting/Tools_PhotonMapping_System.h"
#include "Lighting/Tools_PhotonMapping_VertexSampleStamp.h"


#include "Lighting/Tools_Environment_LOS.h"

#include "Rendering/Asura_Module.h"
#include "Mesh/Asura_Mesh.h"
#include "Misc/Tools_Adjacency.h"


#if 0
void Tools_PhotonMap_DirectEstimation::AddPhotonHit(const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth)
{
	const float fBandwidthSquared = fBandwidth * fBandwidth;

	Asura_Bounding_Box xBB;

	xBB.Reset();

	Asura_Vector_3 xV1 ( xPosition.x - fBandwidth, xPosition.y - fBandwidth, xPosition.z - fBandwidth );
	Asura_Vector_3 xV2 ( xPosition.x + fBandwidth, xPosition.y + fBandwidth, xPosition.z + fBandwidth );

	xBB.Set(xV1);
	xBB.Set(xV2);

	const u_int uNumberOfModules = Asura_Module_System::GetNumberOfModules();

	for (u_int uModule=0; uModule<uNumberOfModules; uModule++)
	{
		const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(uModule);

		Asura_Bounding_Box xModuleBB = Asura_Module_System::GetMeshForModule(uModule)->BoundingBox;
		xModuleBB.Displace(Asura_Module_System::GetPositionOfModule(uModule));

		if (!xModuleBB.Intersects(xBB)) continue;

		const u_int uNumberOfVertices = pxMesh->NumberOfVertices;

		m_pxModules[uModule].Reset(uNumberOfVertices);

		for (u_int uVertex=0; uVertex<uNumberOfVertices; uVertex++)
		{
			Asura_Vector_3 xDisplacement = pxMesh->VertexDataPtr[uVertex].Position + Asura_Module_System::GetPositionOfModule(uModule) - xPosition;

			const float fDistanceSquared = xDisplacement.MagnitudeSquared();

			if (fDistanceSquared < fBandwidthSquared)
			{
				float fContribution = (2.0f / (Asura_Maths::Pi * fBandwidth) ) * (1.0f - (fDistanceSquared / fBandwidthSquared));

				Asura_Vector_3 xDir = -xDirection;

				Asura_SphericalHarmonics_Order3& xIrradiance = m_pxModules[uModule].AddSample(uVertex, uNumberOfVertices ).m_xIrradiance;
				xIrradiance.AddSampleForDirection( fContribution * fEnergy, xDir);
			}

		}
	}
}
#else

static u_int g_uModuleIndex;
static float g_fBandwidth;
static float g_fBandwidthSquared;
static Tools_PhotonMap_DirectEstimation* g_pxMap;
static float g_fScaledEnergy;
static Asura_Vector_3 g_xPosition;
static Asura_Vector_3 g_xDirection;
static Asura_Vector_3 g_xPlaneNormal;

static void PhotonHitCallback( u_int uPolyIndex )
{
	const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(g_uModuleIndex);

	g_pxMap->m_pxModules[g_uModuleIndex].Reset(pxMesh->NumberOfVertices);

	u_int uNumberOfVertices = pxMesh->PolygonDataPtr[uPolyIndex].NumberOfVertices();

	for (u_int u=0; u<uNumberOfVertices; u++)
	{
		u_int uVertexIndex = pxMesh->PolygonDataPtr[uPolyIndex].VertexID[u];

		if (Tools_PhotonMapping_VertexSampleStamp::IsSampleValid( uVertexIndex, g_uModuleIndex ))
		{
			Asura_Vector_3 xDisplacement = pxMesh->VertexDataPtr[uVertexIndex].Position + Asura_Module_System::GetPositionOfModule(g_uModuleIndex) - g_xPosition;

			const float fDistanceSquared = xDisplacement.MagnitudeSquared();

			if (fDistanceSquared < g_fBandwidthSquared)
			{

				float fPlaneDistance = xDisplacement * g_xPlaneNormal;

				if (fPlaneDistance >= -0.001f)
				{

					float fContribution = g_fScaledEnergy * (1.0f - (fDistanceSquared / g_fBandwidthSquared));

					Asura_SphericalHarmonics_Order3& xIrradiance = g_pxMap->m_pxModules[g_uModuleIndex].AddSample( g_pxMap ,uVertexIndex, pxMesh->NumberOfVertices ).m_xIrradiance;
					xIrradiance.AddSampleForDirection( fContribution, g_xDirection);
					continue;
				}
			}
			//RAR 26/06/2006 Vertex was rejected ,so decrement the sample stamp again , in case another hit from the same
			//photon gets closer. (Should only be an issue if using polygons with DontBlockLight)
			Tools_PhotonMapping_VertexSampleStamp::ResetSampleStamp( uVertexIndex, g_uModuleIndex );
		}
	}
}

#if 1

class Asura_PhotonHitData : public Tools_Large_AABB_Tree_AABBData
{
	public:

		Asura_PhotonHitData(  const Asura_Bounding_Box& xTestAABB, Tools_Large_AABB_Tree_AABBData::TLAT_AABBTestCallback pfnCallback )
		: Tools_Large_AABB_Tree_AABBData( xTestAABB, pfnCallback )
		{
		}
	
		float m_fBandwidth;
		float m_fBandwidthSquared;
		
		Tools_PhotonMap_DirectEstimation* m_pxMap;
		float m_fScaledEnergy;
		
		Asura_Vector_3 m_xPosition;
		Asura_Vector_3 m_xDirection;
		Asura_Vector_3 m_xPlaneNormal;

		bool m_bUseStamp;
};
#include "Core/Asura_Multithread.h"
Asura_Mutex g_xMutexA;
Asura_Mutex g_xMutexB;
Asura_Mutex g_xMutex[256];

static void PhotonHitVertexCallback( u_int uVertexIndex, Tools_Large_AABB_Tree_AABBData* pxData )
{
	const Asura_PhotonHitData* const pxHitData = static_cast<Asura_PhotonHitData*>(pxData);

	const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(pxHitData->GetModuleIndex());

	u_int uNumberOfVerts = pxMesh->NumberOfVertices;

	if ( uNumberOfVerts != pxHitData->m_pxMap->m_pxModules[pxHitData->GetModuleIndex()].m_uNumberOfSamples )
	{
		g_xMutexA.Lock();
		// need to double check in case someone else gets there first!
		if ( uNumberOfVerts != pxHitData->m_pxMap->m_pxModules[pxHitData->GetModuleIndex()].m_uNumberOfSamples )
		{
			pxHitData->m_pxMap->m_pxModules[pxHitData->GetModuleIndex()].Reset(uNumberOfVerts);
		}
		g_xMutexA.Unlock();
	}

	// KJFL - 12 May 2008 - looks like I currently still need this stamp to avoid lighting DontBlockLight polys twice
	if ( pxHitData->m_bUseStamp )
	{
		if ( !Tools_PhotonMapping_VertexSampleStamp::IsSampleValid( uVertexIndex, pxHitData->GetModuleIndex() ) ) return;
	}
	
	{
		const Asura_Vector_3 xDisplacement = pxMesh->VertexDataPtr[uVertexIndex].Position + Asura_Module_System::GetPositionOfModule(pxHitData->GetModuleIndex()) - pxHitData->m_xPosition;

		const float fDistanceSquared = xDisplacement.MagnitudeSquared();

		if (fDistanceSquared < pxHitData->m_fBandwidthSquared)
		{

			float fPlaneDistance = xDisplacement * pxHitData->m_xPlaneNormal;

			if (fPlaneDistance >= -0.001f)
			{

				float fContribution = pxHitData->m_fScaledEnergy * (1.0f - (fDistanceSquared / pxHitData->m_fBandwidthSquared));

				u_int uHashLock = (pxHitData->GetModuleIndex() + uVertexIndex)&255;

				g_xMutex[uHashLock].Lock();
				Asura_SphericalHarmonics_Order3& xIrradiance = pxHitData->m_pxMap->m_pxModules[pxHitData->GetModuleIndex()].AddSample( pxHitData->m_pxMap ,uVertexIndex, pxMesh->NumberOfVertices ).m_xIrradiance;
				
				xIrradiance.AddSampleForDirection( fContribution, pxHitData->m_xDirection);
				g_xMutex[uHashLock].Unlock();
				return;
			}
		}
		
		//RAR 26/06/2006 Vertex was rejected ,so decrement the sample stamp again , in case another hit from the same
		//photon gets closer. (Should only be an issue if using polygons with DontBlockLight)
		if ( pxHitData->m_bUseStamp )
		{
			Tools_PhotonMapping_VertexSampleStamp::ResetSampleStamp( uVertexIndex, pxHitData->GetModuleIndex() );
		}
	}

//	g_xMutex.Unlock();
		

}
#endif

void Tools_PhotonMap_DirectEstimation::AddPhotonHit(const u_int uPolyIndex, const u_int uModuleIndex, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth )
{
	Asura_Bounding_Box xBB;

	xBB.Reset();

	Asura_Vector_3 xV1 ( xPosition.x - fBandwidth, xPosition.y - fBandwidth, xPosition.z - fBandwidth );
	Asura_Vector_3 xV2 ( xPosition.x + fBandwidth, xPosition.y + fBandwidth, xPosition.z + fBandwidth );

	xBB.Set(xV1);
	xBB.Set(xV2);

	g_fBandwidth = fBandwidth;
	g_fBandwidthSquared = fBandwidth * fBandwidth;
	g_pxMap = this;
	g_fScaledEnergy = (2.0f / (Asura_Maths::Pi * g_fBandwidth) ) * fEnergy;

	g_xDirection = -xDirection;
	g_xPosition = xPosition;

	{
		const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(uModuleIndex);
		g_xPlaneNormal = pxMesh->NormalsDataPtr[uPolyIndex].FaceNormal;	
	}


	Tools_Environment_LOS::TestWithAABB( xBB, &g_uModuleIndex, PhotonHitCallback );

}

void Tools_PhotonMap_DirectEstimation::AddPhotonHit(const u_int uPolyIndex, const u_int uModuleIndex, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth, bool bUseStamp )
{
	Asura_Bounding_Box xBB;

	xBB.Reset();

	Asura_Vector_3 xV1 ( xPosition.x - fBandwidth, xPosition.y - fBandwidth, xPosition.z - fBandwidth );
	Asura_Vector_3 xV2 ( xPosition.x + fBandwidth, xPosition.y + fBandwidth, xPosition.z + fBandwidth );

	xBB.Set(xV1);
	xBB.Set(xV2);

	Asura_PhotonHitData xData( xBB, PhotonHitVertexCallback );

	xData.m_fBandwidth = fBandwidth;
	xData.m_fBandwidthSquared = fBandwidth*fBandwidth;

	xData.m_pxMap = this;
	xData.m_fScaledEnergy = (2.0f / (Asura_Maths::Pi * fBandwidth) ) * fEnergy ;
	
	
	xData.m_xDirection = -xDirection;
	xData.m_xPosition = xPosition;

	const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(uModuleIndex);
	xData.m_xPlaneNormal = pxMesh->NormalsDataPtr[uPolyIndex].FaceNormal;		

	xData.m_bUseStamp = bUseStamp;

	Tools_Environment_LOS::FindVerticesInAABB( &xData );
}

#endif

#if 0
#if 0
void Tools_PhotonMap_DirectEstimation::AddPhotonHit(const u_int uPolyIndex, const u_int uModuleIndex, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth)
{
	const float fBandwidthSquared = fBandwidth * fBandwidth;

	const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(uModuleIndex);

	m_pxModules[uModuleIndex].Reset(pxMesh->NumberOfVertices);

	u_int uNumberOfVertices = pxMesh->PolygonDataPtr[uPolyIndex].NumberOfVertices();

	for (u_int u=0; u<uNumberOfVertices; u++)
	{
		u_int uVertexIndex = pxMesh->PolygonDataPtr[uPolyIndex].VertexID[u];

		Asura_Vector_3 xDisplacement = pxMesh->VertexDataPtr[uVertexIndex].Position + Asura_Module_System::GetPositionOfModule(uModuleIndex) - xPosition;

		const float fDistanceSquared = xDisplacement.MagnitudeSquared();

		if (fDistanceSquared < fBandwidthSquared)
		{
			float fContribution = (2.0f / (Asura_Maths::Pi * fBandwidth) ) * (1.0f - (fDistanceSquared / fBandwidthSquared));

			Asura_Vector_3 xDir = -xDirection;

			Asura_SphericalHarmonics_Order3& xIrradiance = m_pxModules[uModuleIndex].AddSample( uVertexIndex, pxMesh->NumberOfVertices ).m_xIrradiance;
			xIrradiance.AddSampleForDirection( fContribution * fEnergy, xDir);
		}
	}
}
#else
void Tools_PhotonMap_DirectEstimation::AddPhotonHit(const u_int uPolyIndex, const u_int uModuleIndex, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth)
{
	Asura_Vector_3 xDir = -xDirection;

	Tools_PhotonMapping_VertexSampleStamp::IncrementSampleNumber();

	AddPhotonHit_Recursive( uPolyIndex, uModuleIndex, xPosition, xDir, fEnergy, fBandwidth);
}
#endif

void Tools_PhotonMap_DirectEstimation::AddPhotonHit_Recursive(const u_int uPolyIndex, const u_int uModuleIndex, const Asura_Vector_3& xPosition, const Asura_Vector_3& xDirection, const float fEnergy, const float fBandwidth)
{
	const float fBandwidthSquared = fBandwidth * fBandwidth;

	const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(uModuleIndex);

	m_pxModules[uModuleIndex].Reset(pxMesh->NumberOfVertices);

	u_int uNumberOfVertices = pxMesh->PolygonDataPtr[uPolyIndex].NumberOfVertices();

	u_int uNumberOfSamplesAdded = 0;

	for (u_int u=0; u<uNumberOfVertices; u++)
	{
		u_int uVertexIndex = pxMesh->PolygonDataPtr[uPolyIndex].VertexID[u];

		if (Tools_PhotonMapping_VertexSampleStamp::IsSampleValid( uVertexIndex, uModuleIndex ))
		{
			Asura_Vector_3 xDisplacement = pxMesh->VertexDataPtr[uVertexIndex].Position + Asura_Module_System::GetPositionOfModule(uModuleIndex) - xPosition;

			const float fDistanceSquared = xDisplacement.MagnitudeSquared();

			if (fDistanceSquared < fBandwidthSquared)
			{
				float fContribution = (2.0f / (Asura_Maths::Pi * fBandwidth) ) * (1.0f - (fDistanceSquared / fBandwidthSquared));

				Asura_SphericalHarmonics_Order3& xIrradiance = m_pxModules[uModuleIndex].AddSample( uVertexIndex, pxMesh->NumberOfVertices ).m_xIrradiance;
				xIrradiance.AddSampleForDirection( fContribution * fEnergy, xDirection);

				++uNumberOfSamplesAdded;
			}
		}
	}

	if (uNumberOfSamplesAdded)
	{
		//let's try adjacent polys too then

		const Tools_Polygon_Adjacency* pxAdjacencyList = Tools_Adjacency::GetPolygonAdjacencies(uModuleIndex,uPolyIndex);

		for(const Tools_Adjacent_Polygon* pxAdj = pxAdjacencyList->GetFirstAdjacency(); pxAdj; pxAdj = pxAdj->GetNext())
		{

			AddPhotonHit_Recursive( pxAdj->GetPolygonIndex(), pxAdj->GetModuleIndex(), xPosition, xDirection, fEnergy, fBandwidth );
		}
	}
}
#endif


void Tools_PhotonMap_DirectEstimation::TransferSamplesFromModules()
{
	Asura_Assert(m_pxModules,"");
	Asura_Assert(!m_pxSamples,"Samples already allocated?");

	if(!m_uNumberOfSamples) return;

	//allocate storage
	m_pxSamples = new Tools_PhotonMap_DirectEstimation_Sample[m_uNumberOfSamples];

	//scan through all the modules copying the sample data
	u_int uSamplesCopied = 0;
	Tools_PhotonMap_DirectEstimation_Module* pxModule = m_pxModules;
	for(u_int u=0;u<m_uNumberOfModules;u++,pxModule++)
	{
		if(!pxModule->m_pxSamples || !pxModule->m_puSampleIndices) continue;

		for(u_int v=0;v<pxModule->m_uNumberOfSamples;v++)
		{
			u_int uIndex = pxModule->m_puSampleIndices[v];
			if(uIndex != 0xffffffff)
			{
				m_pxSamples[uIndex] = pxModule->m_pxSamples[v];
				uSamplesCopied++;
			}
		}
		//no longer need the sample data in the module
		delete [] pxModule->m_pxSamples;
		pxModule->m_pxSamples = 0;
	}

	Asura_Assert(uSamplesCopied == m_uNumberOfSamples,"Copied wrong number of samples?");
}


Tools_PhotonMap_DirectEstimation_Sample& Tools_PhotonMap_DirectEstimation_Module::AddSample( class Tools_PhotonMap_DirectEstimation* pxPhotonMap,const u_int uSample, const u_int uTotalNumberOfSamples )
{
	Asura_Assert( m_pxSamples, "Sample data not allocated?" );
	Asura_Assert( m_puSampleIndices, "Sample data not allocated?" );
	Asura_Assert( uSample<m_uNumberOfSamples, "invalid sample number" );

	//allocate index into final storage , if needed
	if(m_puSampleIndices[uSample] == 0xffffffff)
	{
		m_puSampleIndices[uSample] = pxPhotonMap->AllocateSampleIndex();
	}
	//and return position in the temporary storage						
	return m_pxSamples[uSample];
}
