//
// Tools_PhotonMapping_LightSource_Object.h
//
// Created by KJL - 15/01/2004 15:51:36
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_PHOTONMAPPING_LIGHTSOURCE_OBJECT_H__
#define __TOOLS_PHOTONMAPPING_LIGHTSOURCE_OBJECT_H__

#include "Core/Asura_String.h"
#include "Misc/Tools_Colour.h"
#include "Objects/Tools_Object.h"
#include "Core/Asura_Handle.h"

class Tools_PhotonMapping_LightSource_Object : public Tools_Object
{
	friend class Tools_Mode_PhotonMapping;	
	public :
		Tools_PhotonMapping_LightSource_Object(const Asura_Handle iLightSourceHandle);
		~Tools_PhotonMapping_LightSource_Object();

		void OnSelection();
		void OnDeselection();

		virtual void UpdatePositionInAsuraData(bool /*bUpdateLinkedObjects*/);
		virtual bool IsHidden();
	 	virtual void PrintInfo(float& x,float& y);

		static Tools_PhotonMapping_LightSource_Object* GetLightObjectFromID(u_int uID);

		//Indicates whether the tools objects for lights should currently exist..
		//This is needed so that updated template instances know whether to create objects
		//for their lights
		static bool HaveLightSourceObjectsBeenCreated(){return s_bObjectsCreated;}
		static void SetLightSourceObjectsCreated(bool bSet){s_bObjectsCreated = bSet;}

		//Creates a new light , together with its tools object
		static Tools_PhotonMapping_LightSource_Object* CreateLightSource(bool bParallel);

		//updates initial values for start of scale operation
		static void UpdateInitialLightValues();
		//applies the current light scale values from Tools_Mode_PhotonMapping to all lights
		static void ApplyLightScaleValues();

		virtual bool GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const;
		virtual bool SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam);

		void SetLightingModified();

		
	private:

		void CreateLightPropertiesOverlay(void);

		void RecalculatePhotons( void );

		Asura_Handle				m_iLightSourceHandle;

		// used to signal if the overlay has changed any values
		bool						m_bChanged;
		bool						m_bChanged_NoRelightNeeded;
		bool						m_bInnerRangeChanged;
		bool						m_bOuterRangeChanged;

		float						m_fPower;
		float						m_fAngle;
		mutable Tools_Colour_Pair	m_xColourPair;
		bool						m_bParallelSource;
		bool						m_bBleachedSource;
		bool						m_bShortRange;

		bool						m_bUseRange;
		float						m_fInnerRange;
		float						m_fOuterRange;

		float						m_fReflectionChance;
		u_int						m_uLightSets;
		
		int							m_iAccuracy;



		Asura_Guid					m_uTemplateGuid;
		u_int						m_uID;

		Asura_String				m_xName;

		//initial power/colour values for use with light scale overlay
		float						m_fInitialPower;
		mutable Asura_Vector_3		m_xInitialColour;

		mutable bool				m_bColoursInverted;



		void Render();		
		const char* GetName() {return m_xName.GetString();};
		virtual void UpdateWhileSelected();
		const char* GetTypeDescription(){return "Light";}
		void Destroy();
		bool IsSelectable();

				
		void UpdateInvertedColourStatus() const;
		void UpdateAsuraLight();
		void UpdateIfChanged();
		void UpdateName();

		// Copy/paste support
		virtual void Copy(const Tools_Object* pxObject);
		virtual bool CanBeCopied() const {return true;}
		virtual Tools_Object* CreateObjectOfThisType() const;

		//functions used as IsEnabled callbacks for some of the controls
		static bool IsEnabled_IsNotParallel();
		static bool IsEnabled_IsAngular();
		static bool IsEnabled_UsesRange();

		static void Rename();

		
		static Tools_PhotonMapping_LightSource_Object* s_pxThis;

		static List<Tools_PhotonMapping_LightSource_Object*> s_xObjectList;
		static bool s_bObjectsCreated;

		static bool	s_bShowCone;
		static bool	s_bWireframeCone;

	
};


#endif//__TOOLS_PHOTONMAPPING_LIGHTSOURCE_OBJECT_H__
