//
// Tools_PhotonMapping_System.cpp
//
// Created by KJL - 03/12/2003 16:33:15
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Lighting/Tools_PhotonMapping_System.h"
#include "Lighting/Tools_PhotonMapping_System_Internal.h"

#include "Light/Asura_GI_Solution.h"

#include "Lighting/Tools_Environment_LOS.h"
#include "Lighting/Tools_EnvironmentalLightTransport.h"
#include "Lighting/Tools_Mode_PhotonMapping.h"
#include "Lighting/Tools_PhotonMapping_VertexSampleStamp.h"
#include "Lighting/Tools_GlobalIllumination_EntitySamples.h"
#include "Lighting/Tools_LightSets.h"

// need a random bunch of things for the update/progress screen
#include "Debugging/Asura_Debug_Timers.h"
#include "Load&Save/Tools_BatchProcessing.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Adjacency.h"
#include "Core/Asura_Windows.h"
#include "Interface/Asura_Cursor.h"
#include "Rendering/Asura_Viewport.h"
#include "Core/Asura_Timers.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_Shaders.h"
#include "Rendering/Asura_Direct3D_BackBufferCapture.h"


// few random functions that need to be replaced with Asura core 
// ... if the random number stuff is good enough yet
static float randf(void)
{
	float f = ((float)rand())/((float)RAND_MAX);

	return f;
}

static void RandomDirection(Asura_Vector_3& v)
{
   float fLength2;

    //Choose random elements for x and y, until 
    //the squared length is within the unit sphere, but large
    //enough that the normalisation won't result in poor conditioning.
    do
    {
        v.x = 2.f*randf() - 1.f;
        v.y = 2.f*randf() - 1.f;
        v.z = 2.f*randf() - 1.f;

        fLength2 = v.x * v.x + v.y * v.y + v.z * v.z;
    }
    while(fLength2 > 1.f || fLength2 < 0.01f);

    fLength2 = 1.f / Asura_Maths::Sqrt(fLength2);

    v.x *= fLength2;
    v.y *= fLength2;
    v.z *= fLength2;
}


void Tools_PhotonMapping_BaseLightData::SetModified()
{
	Tools_SaveAsura::SetGlobalIlluminationModified();
}

void Tools_PhotonMapping_LightData::SetModified()
{
	if(m_xTemplateData.IsMissionLight())
		Tools_SaveAsura::SetMissionGlobalIlluminationModified();
	else
		Tools_SaveAsura::SetGlobalIlluminationModified();
}


void Tools_PhotonMapping_RegionSample::AddPhoton(Tools_PhotonMapping_PhotonHit* pxPhotonHit)
{
	//const Tools_PhotonMapping_PhotonHit& xPhotonHit = m_pxPhotonMap->GetPhotonHit(uIndex);

	Asura_Vector_3 xDisp = pxPhotonHit->GetPosition() - m_xCentre;
	float fDistanceSquared = xDisp.MagnitudeSquared();

	
	float fDelta = -pxPhotonHit->GetDirection() * m_xNormal;
	if (fDelta<=0.0f) return;

	//fDistanceSquared /= 100.0f;

	if (fDistanceSquared>m_fMaxDistanceSquared) return;

	if (m_uCurrentNumberOfPhotonsInSample<m_uMaxPhotonsInSample)
	{
		if (fDistanceSquared > m_fMaxDistanceSquared)
		{
			m_fMaxDistanceSquared = fDistanceSquared;
			m_uMostDistantPhoton = m_uCurrentNumberOfPhotonsInSample;
		}

		m_ppxPhotons[m_uCurrentNumberOfPhotonsInSample] = pxPhotonHit;
		m_pfPhotonsDistanceSquared[m_uCurrentNumberOfPhotonsInSample] = fDistanceSquared;

		m_uCurrentNumberOfPhotonsInSample++;
	}
	else if (fDistanceSquared < m_fMaxDistanceSquared)
	{
		m_ppxPhotons[m_uMostDistantPhoton] = pxPhotonHit;
		m_pfPhotonsDistanceSquared[m_uMostDistantPhoton] = fDistanceSquared;
		
		m_fMaxDistanceSquared = fDistanceSquared;

		
		for (u_int u=0; u<m_uMaxPhotonsInSample; u++)
		{
			float fDistanceSquared2 = m_pfPhotonsDistanceSquared[u];

			if (fDistanceSquared2 > m_fMaxDistanceSquared)
			{
				m_fMaxDistanceSquared = fDistanceSquared2;
				m_uMostDistantPhoton = u;
			}
		}

	}
	
	
		
}

void Tools_PhotonMapping_RegionSample::CalculateFlux(Asura_Vector_3& xFlux)
{
	xFlux.Zero();

	if (m_uCurrentNumberOfPhotonsInSample==0) return;

	float fSampleRadius = sqrtf(m_fMaxDistanceSquared);
	float fConeFilter = 1.0f / (1.1f * fSampleRadius);

	for (u_int u=0; u<m_uCurrentNumberOfPhotonsInSample; u++)
	{
		const Tools_PhotonMapping_PhotonHit& xPhotonHit = *m_ppxPhotons[u];
			

		float fDelta = -xPhotonHit.GetDirection() * m_xNormal;

		{
			float fDistance = sqrtf(m_pfPhotonsDistanceSquared[u]);

			float fWeight = 1.0f - (fDistance)*fConeFilter;
			
			xFlux += xPhotonHit.GetEnergy() * fDelta * fWeight;
		}
	}

	xFlux /= (Asura_Maths::Pi * (1.0f - 2.0f/(3.0f*1.1f))) * m_fMaxDistanceSquared;
}



bool								Tools_PhotonMapping_System::s_bBeenInitialised = false;
//u_int								Tools_PhotonMapping_System::s_uNumberOfLightingGroups;
//Tools_PhotonMapping_LightSource*	Tools_PhotonMapping_System::s_pxLightSources;


u_int								Tools_PhotonMapping_System::s_uNumberOfLightSources;
u_int								Tools_PhotonMapping_System::s_uNumberOfPolygonLightSources;

u_int								Tools_PhotonMapping_System::s_uMaximumNumberOfBounces = 2;
u_int								Tools_PhotonMapping_System::s_uNumberOfRayCastsPerLight = 50;
float								Tools_PhotonMapping_System::s_fSampleRadiusSquared = 16.0f;
float								Tools_PhotonMapping_System::s_fParallelBounceScale = 0.0f;

Tools_PhotonMapping_RegionSample*	Tools_PhotonMapping_System::s_pxSample;
bool								Tools_PhotonMapping_System::s_bPreviousSampleRadiusValid;

bool								Tools_PhotonMapping_System::s_bEnvironmentIrradianceValid = false;

Asura_SphericalHarmonics_Order3		Tools_PhotonMapping_System::s_xEnvironmentIrradianceRed;
Asura_SphericalHarmonics_Order3		Tools_PhotonMapping_System::s_xEnvironmentIrradianceGreen;
Asura_SphericalHarmonics_Order3		Tools_PhotonMapping_System::s_xEnvironmentIrradianceBlue;

Asura_TemplateHashTree<Asura_RGBA>	Tools_PhotonMapping_System::s_xGlobalAmbienceTree;

// User tweakable rescale factors to improve spherical harmonic downsampling
float								Tools_PhotonMapping_System::s_fSHZerothOrderResampleScale = 1.0f;
float								Tools_PhotonMapping_System::s_fSHFirstOrderResampleScale = 1.0f;

bool								Tools_PhotonMapping_System::s_bPreserveScreenDuringUpdateProgress = false;


Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>* Tools_PhotonMapping_System::s_pxLightDataList;
Asura_TrackingHandlePool<Tools_PhotonMapping_PolygonLightData>* Tools_PhotonMapping_System::s_pxPolygonLightDataList;

Asura_Hashed_List<Tools_PhotonMapping_BaseLightData>* Tools_PhotonMapping_System::s_pxCombinedLightList;

void Tools_PhotonMapping_System::Initialise(void)
{
	if (s_bBeenInitialised)
	{
		Asura_Assert( false, "Already been initialised" );
		return;
	}

	s_bBeenInitialised = true;

	s_pxLightDataList = new Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>(16);
	s_pxPolygonLightDataList = new Asura_TrackingHandlePool<Tools_PhotonMapping_PolygonLightData>(16);
	s_pxCombinedLightList = new Asura_Hashed_List<Tools_PhotonMapping_BaseLightData>(5);

	s_uNumberOfLightSources = 0;
	s_uNumberOfPolygonLightSources = 0;

	// Region samples no longer needed
	s_pxSample = 0;// new Tools_PhotonMapping_RegionSample(500);

	s_bPreviousSampleRadiusValid = false;

	s_xGlobalAmbienceTree.Reset();
	
	Tools_EnvironmentalLightTransport::Initialise();

	Tools_GlobalIllumination_EntitySamples::Initialise();
}

void Tools_PhotonMapping_System::Shutdown(void)
{
	if (!s_bBeenInitialised)
	{
		Asura_Assert( false, "Not yet been initialised" );
		return;
	}

	s_bBeenInitialised = false;

	delete s_pxLightDataList;
	delete s_pxPolygonLightDataList;

	//deleting the combined list won't remove all the entries , so need to delete all
	//by hand first
	Asura_Hashed_List_Deleter_It<Tools_PhotonMapping_BaseLightData> xIterator(s_pxCombinedLightList);
	for(xIterator.Restart();!xIterator.Done();)
	{
		xIterator.DeleteCurrent();
	}
	delete s_pxCombinedLightList;

	delete s_pxSample;

	Tools_EnvironmentalLightTransport::Shutdown();
	
	Tools_GlobalIllumination_EntitySamples::Shutdown();
	
}
	
void Tools_PhotonMapping_System::Reset( void )
{
	s_pxLightDataList->Reset();
	s_pxPolygonLightDataList->Reset();
	
	Asura_Hashed_List_Deleter_It<Tools_PhotonMapping_BaseLightData> xIterator(s_pxCombinedLightList);
	for(xIterator.Restart();!xIterator.Done();)
	{
		xIterator.DeleteCurrent();
	}
	
	Tools_EnvironmentalLightTransport::Reset();
	Tools_GlobalIllumination_EntitySamples::Reset();
}


void Tools_PhotonMapping_System::RecalculateEverything(u_int uMinAccuracy /*= 1*/,u_int uMaxAccuracy /*=TOOLS_PHOTONMAPPING_MAX_ACCURACY*/)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_PhotonMapping_System::RecalculateEverything");
	
	//s_pxLightDataList->Traverse(RecalculatePhotonMapForLightSource);

	if(uMaxAccuracy<1) uMaxAccuracy = 1;
	if(uMinAccuracy > uMaxAccuracy)
	{
		uMinAccuracy = uMaxAccuracy;
	}


	u_int uTotalNumberOfLights = 0;
	
	Asura_Hashed_List_It<Tools_PhotonMapping_BaseLightData> xIterator(s_pxCombinedLightList);
	for (xIterator.Restart(); !xIterator.Done(); xIterator.Next())
	{
    	Tools_PhotonMapping_BaseLightData* pxLightData = xIterator.GetCurrent();

		if ( !(pxLightData->GetLightSource()->IsParallel()) )
		{
			uTotalNumberOfLights++;
		}
	}

	u_int u=0;

	for (xIterator.Restart(); !xIterator.Done(); xIterator.Next())
	{
    	Tools_PhotonMapping_BaseLightData* pxLightData = xIterator.GetCurrent();

		if ( !(pxLightData->GetLightSource()->IsParallel()) )
		{
			u_int uRequiredAccuracy = pxLightData->GetLightSource()->GetAccuracy();
			uRequiredAccuracy = Asura_Maths::ClampToRange(uRequiredAccuracy,uMinAccuracy,uMaxAccuracy);

			
			RecalculatePhotonMapForLightSourceMain( u, uTotalNumberOfLights, pxLightData, uRequiredAccuracy);
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) break;
			u++;
		}
	}
}


bool Tools_PhotonMapping_System::RecalculateAllInvalidLights(u_int uMinAccuracy,u_int uMaxAccuracy)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_PhotonMapping_System::RecalculateAllInvalidLights");
	
	//s_pxLightDataList->Traverse(RecalculatePhotonMapForLightSource);
	if(uMaxAccuracy<1) uMaxAccuracy = 1;
	if(uMinAccuracy > uMaxAccuracy)
	{
		uMinAccuracy = uMaxAccuracy;
	}

	u_int uTotalNumberOfLights = 0;
	
	Asura_Hashed_List_It<Tools_PhotonMapping_BaseLightData> xIterator(s_pxCombinedLightList);
	for (xIterator.Restart(); !xIterator.Done(); xIterator.Next())
	{
    	Tools_PhotonMapping_BaseLightData* pxLightData = xIterator.GetCurrent();

		if ( !(pxLightData->GetLightSource()->IsParallel()))
		{
			u_int uValidity = pxLightData->GetLightSource()->GetPhotonMapValidity();
			u_int uRequiredAccuracy = pxLightData->GetLightSource()->GetAccuracy();
			uRequiredAccuracy = Asura_Maths::ClampToRange(uRequiredAccuracy,uMinAccuracy,uMaxAccuracy);
			if(uValidity < uRequiredAccuracy)
			{
				uTotalNumberOfLights++;
			}
		}
	}

	u_int u=1;

	for (xIterator.Restart(); !xIterator.Done(); xIterator.Next())
	{
    	Tools_PhotonMapping_BaseLightData* pxLightData = xIterator.GetCurrent();

		if ( !(pxLightData->GetLightSource()->IsParallel()))
		{
			u_int uValidity = pxLightData->GetLightSource()->GetPhotonMapValidity();
			u_int uRequiredAccuracy = pxLightData->GetLightSource()->GetAccuracy();
			uRequiredAccuracy = Asura_Maths::ClampToRange(uRequiredAccuracy,uMinAccuracy,uMaxAccuracy);
			if(uValidity < uRequiredAccuracy)
			{
				ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_PhotonMapping_System::RecalculateAllInvalidLights Calc");

				RecalculatePhotonMapForLightSourceMain( u, uTotalNumberOfLights, pxLightData, uRequiredAccuracy);
				u++;
			}
			if(Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE)) break;
		}
	}
	

	return uTotalNumberOfLights!=0;
}


void Tools_PhotonMapping_System::RecalculatePhotonMapForLightSource(const Asura_Handle iHandle)
{
	Tools_PhotonMapping_LightData* pxLightData = s_pxLightDataList->GetPointer(iHandle);

	if (pxLightData)
	{
		//For updating a single light , capture the backbuffer so the text can be rendered over it,
		//rather than over a black screen. This should make it easier to see the changes.
		s_bPreserveScreenDuringUpdateProgress = true;
		Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();

		RecalculatePhotonMapForLightSourceMain( iHandle, 1, pxLightData );

		s_bPreserveScreenDuringUpdateProgress = false;
	}
}

void Tools_PhotonMapping_System::RecalculatePhotonMapForPolygonLightSource(const Asura_Handle iHandle)
{
	Tools_PhotonMapping_PolygonLightData* pxLightData = s_pxPolygonLightDataList->GetPointer(iHandle);

	if (pxLightData)
	{
		//For updating a single light , capture the backbuffer so the text can be rendered over it,
		//rather than over a black screen. This should make it easier to see the changes.
		s_bPreserveScreenDuringUpdateProgress = true;
		Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();

		RecalculatePhotonMapForLightSourceMain( iHandle, 1, pxLightData );

		s_bPreserveScreenDuringUpdateProgress = false;

	}
}

void Tools_PhotonMapping_System::RecalculatePhotonMapForLights(const List<Asura_Handle>& xLightList,const List<Asura_Handle>& xPolygonLightList)
{
	int iCurrentLight = 1;
	int iTotalLights = xLightList.size() + xPolygonLightList.size();
	if(iTotalLights == 1)
	{
		//if only one light , the handle gets shown instead of a count
		if(xLightList.size())
			iCurrentLight = xLightList.first_entry();
		else
			iCurrentLight = xPolygonLightList.first_entry();
	}

	s_bPreserveScreenDuringUpdateProgress = true;
	Asura_Direct3D_BackBufferCapture::CaptureCurrentBackBuffer();

	//do the normal lights
	for(CLIF<Asura_Handle> xLightLIF(&xLightList);!xLightLIF.done();xLightLIF.next(),iCurrentLight++)
	{
		Tools_PhotonMapping_LightData* pxLightData = s_pxLightDataList->GetPointer(xLightLIF());
		if(pxLightData)
		{
			RecalculatePhotonMapForLightSourceMain(iCurrentLight , iTotalLights , pxLightData );
		}
	}

	//and the polygon lights
	for(CLIF<Asura_Handle> xLightLIF(&xPolygonLightList);!xLightLIF.done();xLightLIF.next(),iCurrentLight++)
	{
		Tools_PhotonMapping_PolygonLightData* pxLightData = s_pxPolygonLightDataList->GetPointer(xLightLIF());
		if(pxLightData)
		{
			RecalculatePhotonMapForLightSourceMain(iCurrentLight , iTotalLights , pxLightData );
		}
	}
	
	s_bPreserveScreenDuringUpdateProgress = false;
}


void Tools_PhotonMapping_System::RecalculatePhotonMapForLightSourceMain(u_int uLightNumber, u_int uTotalNumberOfLights, Tools_PhotonMapping_BaseLightData* pxLightData,u_int uAccuracyOveride /*= 0*/)
{
	#if defined (ASURA_USE_TASK_SYSTEM )
	if (Tools_Mode_PhotonMapping::s_bUseMultithreading)
	{
		RecalculatePhotonMapForLightSourceMain_MT( uLightNumber, uTotalNumberOfLights, pxLightData, uAccuracyOveride );
		return;
	}
	#endif

	// direct evaluation photon map

	Tools_PhotonMapping_LightSource* pxLightSource = pxLightData->GetLightSource();

	if(pxLightSource->ShouldIgnoreLight()) return;
	if (pxLightSource->IsParallel()) return;
	
//	float fTime = Asura_Timers::GetUncappedPhase();	

	//If a light's photon map is being recalculated ,then the GI samples will be out of date.
	//RAR 17/08/2005
	Asura_GI_Solution::InvalidateSamples();


	Tools_PhotonMap_DirectEstimation& xPhotonMap = pxLightData->m_xDirectEstimationPhotonMap;
	{
		u_int uNumberOfModules = Asura_Module_System::GetNumberOfModules();
		
		// clear the map ready for the new data
		xPhotonMap.Reset(uNumberOfModules);
		pxLightSource->SetHasPhotonMap(true);
	}
	Tools_PhotonMapping_VertexSampleStamp::Reset();

	const u_int uAccuracy = uAccuracyOveride ? uAccuracyOveride : pxLightSource->GetAccuracy();
	const u_int uNumSamples = GetNumberOfSamplesForAccuracyLevel( uAccuracy );

	pxLightSource->SetStratificationLevels( uNumSamples );

	float fBandwidth = GetSampleRadiusForAccuracyLevel( uAccuracy,pxLightSource->IsShortRange() );
	
	bool bShouldQuit = false;

	char szBuffer[255];

	if (uTotalNumberOfLights>1)
	{
		sprintf(szBuffer, "Photon-mapping for light %d of %d. (Accuracy %d)", uLightNumber, uTotalNumberOfLights, uAccuracy);
	}
	else
	{
		sprintf(szBuffer, "Photon-mapping for light %d. (Accuracy %d)", uLightNumber, uAccuracy);
	}

	//get dot product threshold for directional lights
	float fAngleDotThreshold = -1.0f;
	if(!pxLightSource->IsPolygonLight())
	{
		if(pxLightSource->GetAngle() != 360.0f)
		{
			fAngleDotThreshold = Asura_Maths::Cosine(Asura_Maths::Pi*pxLightSource->GetAngle()/360.0f);
		}
	}
	Asura_Vector_3 xLightDir;
	xLightDir.x = pxLightSource->GetOrientation().e13;
	xLightDir.y = pxLightSource->GetOrientation().e23;
	xLightDir.z = pxLightSource->GetOrientation().e33;

	bool bUseRange = pxLightSource->GetUseRange();
	float fInnerRange = pxLightSource->GetInnerRange();
	float fOuterRange = pxLightSource->GetOuterRange();
	float fReflectionChance = pxLightSource->GetReflectionChance();

	for (u_int u=0; u<uNumSamples; u++)
	{
		for (u_int v=0; v<uNumSamples; v++)
		{
	
			bShouldQuit = UpdateProgress(u*uNumSamples+v, uNumSamples*uNumSamples,  szBuffer );

			if (bShouldQuit) break;

			Asura_Vector_3 xDirection;
			Asura_Vector_3 xPosition;

			pxLightSource->GetStratifiedPhotonPositionAndDirection( xPosition, xDirection, u, v ); 

			if(fAngleDotThreshold != -1.0f)
			{
				if(xLightDir*xDirection < fAngleDotThreshold)
				{
					//ray falls outside of the desired cone
					continue;
				}
			}

			float fTotalDistance = 0;
			float fDistanceSinceBounce = 0;
			u_int uPrevPolyIndex = (u_int)-1;
			u_int uPrevModuleIndex = (u_int)-1;
			Asura_Hash_ID uPrevZoneHash = 0;

			//Moved incrementing of vertex sample stamp to here so a single vertex can't be hit multiple
			//times as a result of a photon passing through a don't block light polygon.
			//RAR 26/06/2006
			Tools_PhotonMapping_VertexSampleStamp::IncrementSampleNumber();

			for (u_int uNumberOfBounces=0; uNumberOfBounces<=s_uMaximumNumberOfBounces; uNumberOfBounces++)
			{
//				Asura_LOS_InputData xInputData( xPosition, xDirection, ASURA_GUID_UNREGISTERED );		
//				const Asura_LOS_OutputData* pxOutputData = Asura_LOS::TestAgainstEnvironment(&xInputData);

				
//				fDistance = pxOutputData->m_fDistanceToIntersection;
			//	if (!Tools_LOS::FindPolyInLOS(xPosition,xDirection,iPolyIndex,iModuleIndex,fDistance)) break;

				Tools_Environment_LOS_Data xLOSData(xPosition, xDirection);
		
				if (!Tools_Environment_LOS::TestWithRay(xLOSData)) break;

				const u_int uPolyIndex = xLOSData.GetPolyIndex();
				const u_int uModuleIndex = xLOSData.GetModuleIndex();
				const u_int uZoneHash = xLOSData.GetZoneHash();
				const float fDistance = xLOSData.GetDistance();

				
				//Make sure the photon doesn't hit the same polygon twice in a row.
				//This is mainly as a precaution in case rounding error causes a photon
				//to repeatedly hit the same don't block light polygon
				//RAR 03/12/2004
				if(uModuleIndex == uPrevModuleIndex && uPolyIndex == uPrevPolyIndex && uZoneHash == uPrevZoneHash)
				{
					break;
				}
				uPrevZoneHash = uZoneHash;
				uPrevModuleIndex = uModuleIndex;
				uPrevPolyIndex = uPolyIndex;

				#ifdef ASURA_USE_STREAMING
				const bool bHitCurrentZone = uZoneHash == Tools_Streaming_System::GetHashForThisLevel();
				Asura_Module_Handler* pxHandler = Asura_Module_System::GetHandler(uZoneHash);
				Asura_Assert(pxHandler,"Failed to find module handler");
				#else
				const bool bHitCurrentZone = true;
				Asura_Module_Handler* pxHandler = &Asura_Module_System::GetCurrentHandler();
				#endif

				Asura_Mesh* pxMesh = pxHandler->GetMeshForModule(uModuleIndex);

				
				int iPolyFlags = pxMesh->PolygonFlagDataPtr ? pxMesh->PolygonFlagDataPtr[uPolyIndex].Flags : 0;

				const Asura_Vector_3& xNormal = pxMesh->NormalsDataPtr[uPolyIndex].FaceNormal;
				//const Asura_Vector_3& xNormal = pxOutputData->m_xNormal;

				//float fCosineTerm = -xDirection * pxOutputData->m_xNormal;

				float fCosineTerm = -xDirection * xNormal;

				bool bHitWrongSide = false;			
				if(!(iPolyFlags & (AsuraPolygonFlag_DoubleSided|AsuraPolygonFlag_IgnoreNormalForLighting)))
				{
					// if the surface is facing away from photon, skip it
					if(fCosineTerm<=0.0f)
					{
						bHitWrongSide = true;
					}
				}

				float fSide = (fCosineTerm<=0.0f) ? -1.0f : 1.0f;
				
				// calculate position of impact
				//xPosition += xDirection * pxOutputData->m_fDistanceToIntersection;
				xPosition += xDirection * fDistance;

				fTotalDistance += fDistance;
				fDistanceSinceBounce += fDistance;

				if(bUseRange && fTotalDistance > fOuterRange)
				{
					//gone beyond light's maximum range
					break;
				}

					
	            const float fReflection = Asura_Random::GetFloat();
				if ( fReflection > fReflectionChance )
				{
					// register the hit
					//xPhotonMap.AddPhotonHit(xPosition,xDirection, pxLightSource->GetStratifiedPhotonEnergy(), fBandwidth);
					float fScaledBandwidth = fBandwidth;

					// Changed the bandwidth scaling to check for the distance since the last bounce ,so
					// that polygons flagged as don't block light don't have an effect on the outcome.
					// RAR 26/06/2006
					if (fDistanceSinceBounce> 40.0f)
					{
						fScaledBandwidth *= fTotalDistance/40.0f;
					}
					float fEnergy = pxLightSource->GetStratifiedPhotonEnergy();
					if(bUseRange && fTotalDistance > fInnerRange)
					{
						//linear falloff from inner range to outer range
						fEnergy *= (fOuterRange - fTotalDistance) /(fOuterRange - fInnerRange);
					}
					if(bHitCurrentZone && !bHitWrongSide)
					{
						xPhotonMap.AddPhotonHit(uPolyIndex, uModuleIndex, xPosition,xDirection, fEnergy, fScaledBandwidth);
					}

					//Photons will continue through polygons marked as don't block light
					if(!(iPolyFlags & AsuraPolygonFlag_DontBlockLight))
					{
						break;
					}
				}
				

				if((iPolyFlags & AsuraPolygonFlag_DontBlockLight))
				{
					//Don't block light flag....
					//For now let the the photon light the polygon hit 75% of the time as normal , 
					//but once that has happened it will always pass through the polygon and continue
					//on the other side , and will never bounce back.
					//RAR 03/12/2004
					xPosition -= xNormal * (0.001f * fSide);

					//Going to try not counting this towards the bounce limit for the light , in
					//case there are several grills in a row that need to be passed through
					uNumberOfBounces--;

				}
				else
				{
					if(bHitWrongSide) break;
					
					// bounce it
					xPosition += xNormal * (0.001f * fSide);
					do
					{
						RandomDirection(xDirection);						
					}
					while ((xDirection*xNormal)*fSide < 0.01f);

					fDistanceSinceBounce = 0.0f;
				}

				//fEnergy *= /*fCosineTerm **/ 0.25f;
			}
		}
		if (bShouldQuit) break;
		
	}

	if(bShouldQuit)
	{
		xPhotonMap.Clear();
	}
	else
	{
		xPhotonMap.TransferSamplesFromModules();
	}


	//Tools_Timed_Message_System::AddMessage(0xffffffff,"Time taken for raycasting: %8.1f secs",Asura_Timers::GetUncappedPhase()-fTime);
	
	pxLightSource->SetPhotonMapValidity(bShouldQuit ? 0 : uAccuracy);
}
#if defined (ASURA_USE_TASK_SYSTEM )
#include "Core/Threads/Asura_Task_Array.h"
static Tools_PhotonMapping_BaseLightData* g_pxLightData;
static Asura_Mutex g_xDontBlockLightMutex;
static u_int g_uRequiredAccuracy;

void Tools_PhotonMapping_System::PhotonMappingMT( const void* pData )
{
	Asura_Task_Array::TaskDataPacket* pxPacket = (Asura_Task_Array::TaskDataPacket*)( pData );

	u_int u = pxPacket->GetArrayStartIndex();
	pxPacket->SignalFinishedWithData();

	Tools_PhotonMapping_LightSource* pxLightSource = g_pxLightData->GetLightSource();
	Tools_PhotonMap_DirectEstimation& xPhotonMap = g_pxLightData->m_xDirectEstimationPhotonMap;

	//get dot product threshold for directional lights
	float fAngleDotThreshold = -1.0f;
	if(!pxLightSource->IsPolygonLight())
	{
		if(pxLightSource->GetAngle() != 360.0f)
		{
			fAngleDotThreshold = Asura_Maths::Cosine(Asura_Maths::Pi*pxLightSource->GetAngle()/360.0f);
		}
	}
	Asura_Vector_3 xLightDir;
	xLightDir.x = pxLightSource->GetOrientation().e13;
	xLightDir.y = pxLightSource->GetOrientation().e23;
	xLightDir.z = pxLightSource->GetOrientation().e33;

	bool bUseRange = pxLightSource->GetUseRange();
	float fInnerRange = pxLightSource->GetInnerRange();
	float fOuterRange = pxLightSource->GetOuterRange();
	float fReflectionChance = pxLightSource->GetReflectionChance();

	const u_int uAccuracy = g_uRequiredAccuracy;
	const u_int uNumSamples = GetNumberOfSamplesForAccuracyLevel( uAccuracy );

	float fBandwidth = GetSampleRadiusForAccuracyLevel( uAccuracy, pxLightSource->IsShortRange() );

	{
		for (u_int v=0; v<uNumSamples; v++)
		{
	
			Asura_Vector_3 xDirection;
			Asura_Vector_3 xPosition;

			pxLightSource->GetStratifiedPhotonPositionAndDirection( xPosition, xDirection, u, v ); 

			if(fAngleDotThreshold != -1.0f)
			{
				if(xLightDir*xDirection < fAngleDotThreshold)
				{
					//ray falls outside of the desired cone
					continue;
				}
			}

			float fTotalDistance = 0;
			float fDistanceSinceBounce = 0;
			u_int uPrevPolyIndex = (u_int)-1;
			u_int uPrevModuleIndex = (u_int)-1;
			Asura_Hash_ID uPrevZoneHash = 0;

			bool bPhotonHasPassedThroughADontBlockLightPoly = false;

			for (u_int uNumberOfBounces=0; uNumberOfBounces<=s_uMaximumNumberOfBounces; uNumberOfBounces++)
			{
//				Asura_LOS_InputData xInputData( xPosition, xDirection, ASURA_GUID_UNREGISTERED );		
//				const Asura_LOS_OutputData* pxOutputData = Asura_LOS::TestAgainstEnvironment(&xInputData);

				
//				fDistance = pxOutputData->m_fDistanceToIntersection;
			//	if (!Tools_LOS::FindPolyInLOS(xPosition,xDirection,iPolyIndex,iModuleIndex,fDistance)) break;

				Tools_Environment_LOS_Data xLOSData(xPosition, xDirection);
		
				if (!Tools_Environment_LOS::TestWithRay(xLOSData)) break;

				const u_int uPolyIndex = xLOSData.GetPolyIndex();
				const u_int uModuleIndex = xLOSData.GetModuleIndex();
				const u_int uZoneHash = xLOSData.GetZoneHash();
				const float fDistance = xLOSData.GetDistance();

				
				//Make sure the photon doesn't hit the same polygon twice in a row.
				//This is mainly as a precaution in case rounding error causes a photon
				//to repeatedly hit the same don't block light polygon
				//RAR 03/12/2004
				if(uModuleIndex == uPrevModuleIndex && uPolyIndex == uPrevPolyIndex && uZoneHash == uPrevZoneHash)
				{
					break;
				}
				uPrevZoneHash = uZoneHash;
				uPrevModuleIndex = uModuleIndex;
				uPrevPolyIndex = uPolyIndex;

				#ifdef ASURA_USE_STREAMING
				const bool bHitCurrentZone = uZoneHash == Tools_Streaming_System::GetHashForThisLevel();
				Asura_Module_Handler* pxHandler = Asura_Module_System::GetHandler(uZoneHash);
				Asura_Assert(pxHandler,"Failed to find module handler");
				#else
				const bool bHitCurrentZone = true;
				Asura_Module_Handler* pxHandler = &Asura_Module_System::GetCurrentHandler();
				#endif

				Asura_Mesh* pxMesh = pxHandler->GetMeshForModule(uModuleIndex);

				
				int iPolyFlags = pxMesh->PolygonFlagDataPtr ? pxMesh->PolygonFlagDataPtr[uPolyIndex].Flags : 0;

				const Asura_Vector_3& xNormal = pxMesh->NormalsDataPtr[uPolyIndex].FaceNormal;
				//const Asura_Vector_3& xNormal = pxOutputData->m_xNormal;

				//float fCosineTerm = -xDirection * pxOutputData->m_xNormal;

				float fCosineTerm = -xDirection * xNormal;

				bool bHitWrongSide = false;			
				if(!(iPolyFlags & (AsuraPolygonFlag_DoubleSided|AsuraPolygonFlag_IgnoreNormalForLighting)))
				{
					// if the surface is facing away from photon, skip it
					if(fCosineTerm<=0.0f)
					{
						bHitWrongSide = true;
					}
				}

				float fSide = (fCosineTerm<=0.0f) ? -1.0f : 1.0f;
				
				// calculate position of impact
				//xPosition += xDirection * pxOutputData->m_fDistanceToIntersection;
				xPosition += xDirection * fDistance;

				fTotalDistance += fDistance;
				fDistanceSinceBounce += fDistance;

				if(bUseRange && fTotalDistance > fOuterRange)
				{
					//gone beyond light's maximum range
					break;
				}

					
	            const float fReflection = Asura_Random::GetFloat();
				if ( fReflection > fReflectionChance )
				{
					// register the hit
			
					if ( (!bPhotonHasPassedThroughADontBlockLightPoly) && (iPolyFlags & AsuraPolygonFlag_DontBlockLight ))
					{
						bPhotonHasPassedThroughADontBlockLightPoly = true;
						g_xDontBlockLightMutex.Lock();
						//Moved incrementing of vertex sample stamp to here so a single vertex can't be hit multiple
						//times as a result of a photon passing through a don't block light polygon.
						Tools_PhotonMapping_VertexSampleStamp::IncrementSampleNumber();
					}

					//xPhotonMap.AddPhotonHit(xPosition,xDirection, pxLightSource->GetStratifiedPhotonEnergy(), fBandwidth);
					float fScaledBandwidth = fBandwidth;

					// Changed the bandwidth scaling to check for the distance since the last bounce ,so
					// that polygons flagged as don't block light don't have an effect on the outcome.
					// RAR 26/06/2006
					if (fDistanceSinceBounce> 40.0f)
					{
						fScaledBandwidth *= fTotalDistance/40.0f;
					}
					float fEnergy = pxLightSource->GetStratifiedPhotonEnergy();
					if(bUseRange && fTotalDistance > fInnerRange)
					{
						//linear falloff from inner range to outer range
						fEnergy *= (fOuterRange - fTotalDistance) /(fOuterRange - fInnerRange);
					}
					if(bHitCurrentZone && !bHitWrongSide)
					{
						xPhotonMap.AddPhotonHit(uPolyIndex, uModuleIndex, xPosition,xDirection, fEnergy, fScaledBandwidth, bPhotonHasPassedThroughADontBlockLightPoly);
					}

					//Photons will continue through polygons marked as don't block light
					if(!(iPolyFlags & AsuraPolygonFlag_DontBlockLight))
					{
						break;
					}
				}
				

				if((iPolyFlags & AsuraPolygonFlag_DontBlockLight))
				{
					//Don't block light flag....
					//For now let the the photon light the polygon hit 75% of the time as normal , 
					//but once that has happened it will always pass through the polygon and continue
					//on the other side , and will never bounce back.
					//RAR 03/12/2004
					xPosition -= xNormal * (0.001f * fSide);

					//Going to try not counting this towards the bounce limit for the light , in
					//case there are several grills in a row that need to be passed through
					uNumberOfBounces--;

				}
				else
				{
					if(bHitWrongSide) break;
					
					// bounce it
					xPosition += xNormal * (0.001f * fSide);
					do
					{
						RandomDirection(xDirection);						
					}
					while ((xDirection*xNormal)*fSide < 0.01f);

					fDistanceSinceBounce = 0.0f;
				}

				//fEnergy *= /*fCosineTerm **/ 0.25f;
			}

			if (bPhotonHasPassedThroughADontBlockLightPoly)
			{
				g_xDontBlockLightMutex.Unlock();
			}
		}
	}
}

void Tools_PhotonMapping_System::RecalculatePhotonMapForLightSourceMain_MT(u_int uLightNumber, u_int uTotalNumberOfLights, Tools_PhotonMapping_BaseLightData* pxLightData,u_int uAccuracyOveride /*= 0*/)
{
	// direct evaluation photon map

	Tools_PhotonMapping_LightSource* pxLightSource = pxLightData->GetLightSource();

	g_pxLightData = pxLightData;

	if(pxLightSource->ShouldIgnoreLight()) return;
	if (pxLightSource->IsParallel()) return;
	
	//If a light's photon map is being recalculated ,then the GI samples will be out of date.
	//RAR 17/08/2005
	Asura_GI_Solution::InvalidateSamples();

	Tools_Environment_LOS::EnsureDataIsValid();

	Tools_PhotonMap_DirectEstimation& xPhotonMap = pxLightData->m_xDirectEstimationPhotonMap;
	{
		u_int uNumberOfModules = Asura_Module_System::GetNumberOfModules();
		
		// clear the map ready for the new data
		xPhotonMap.Reset(uNumberOfModules);
		pxLightSource->SetHasPhotonMap(true);
	}
	Tools_PhotonMapping_VertexSampleStamp::Reset();

	const u_int uAccuracy = uAccuracyOveride ? uAccuracyOveride : pxLightSource->GetAccuracy();
	const u_int uNumSamples = GetNumberOfSamplesForAccuracyLevel( uAccuracy );
	g_uRequiredAccuracy = uAccuracy;

	pxLightSource->SetStratificationLevels( uNumSamples );

	bool bShouldQuit = false;

	char szBuffer[255];

	if (uTotalNumberOfLights>1)
	{
		sprintf(szBuffer, "Photon-mapping for light %d of %d. (Accuracy %d)", uLightNumber, uTotalNumberOfLights, uAccuracy);
	}
	else
	{
		sprintf(szBuffer, "Photon-mapping for light %d. (Accuracy %d)", uLightNumber, uAccuracy);
	}

	{
		Asura_Task_Array xTaskArray( uNumSamples, 1, PhotonMappingMT );
		
		do 
		{
			bShouldQuit = UpdateProgress( xTaskArray.GetCurrentIndex(), uNumSamples,  szBuffer );

			if (bShouldQuit) break;

			Sleep(100);

		} 
		while( !xTaskArray.HasFinished() );
	}

	if(bShouldQuit)
	{
		xPhotonMap.Clear();
	}
	else
	{
		xPhotonMap.TransferSamplesFromModules();
	}


	//Tools_Timed_Message_System::AddMessage(0xffffffff,"Time taken for raycasting: %8.1f secs",Asura_Timers::GetUncappedPhase()-fTime);
	
	pxLightSource->SetPhotonMapValidity(bShouldQuit ? 0 : uAccuracy);
}
#endif // defined (ASURA_USE_TASK_SYSTEM )

Asura_Handle Tools_PhotonMapping_System::CreateLightSource(void)
{
	// KJL - 16/01/2004 12:36:54 - actually we create everything connected to the light source (photon map etc) all in one go.
	Asura_Handle iHandle = s_pxLightDataList->Allocate();

	if ( iHandle != ASURA_INVALID_HANDLE )
	{
		++s_uNumberOfLightSources;
		
		Tools_PhotonMapping_LightData* pxLight = s_pxLightDataList->GetPointer(iHandle);
		pxLight->m_xLightSource.ResetToDefaults();
		pxLight->m_xTemplateData.m_uTemplateGuid = ASURA_GUID_UNREGISTERED;
		pxLight->m_xTemplateData.m_uID = 0;
		pxLight->m_xTemplateData.m_iTemplateVersion = 0;

		s_pxCombinedLightList->AddToList(pxLight);
	}
	else
	{
		Asura_Assert( iHandle != ASURA_INVALID_HANDLE, "Unable to allocate a new light source");
	}

	InvalidateEnvironmentIrradiance();

	return iHandle;
}

void Tools_PhotonMapping_System::DestroyLightSource(const Asura_Handle iHandle)
{
	Asura_Assert( iHandle != ASURA_INVALID_HANDLE, "Invalid handle");

	Tools_PhotonMapping_LightData* pxLight = s_pxLightDataList->GetPointer(iHandle);
	pxLight->m_xDirectEstimationPhotonMap.Clear();

	s_pxCombinedLightList->RemoveFromList(pxLight);
	s_pxLightDataList->Deallocate(iHandle);

	//deleting a light will invalidate the GI samples. RAR 17/08/2005
	Asura_GI_Solution::InvalidateSamples();
	InvalidateEnvironmentIrradiance();
	
	--s_uNumberOfLightSources;
}

Asura_Handle Tools_PhotonMapping_System::CreatePolygonLightSource(void)
{
	Asura_Handle iHandle = s_pxPolygonLightDataList->Allocate();

	if ( iHandle != ASURA_INVALID_HANDLE )
	{
		++s_uNumberOfPolygonLightSources;
		
		Tools_PhotonMapping_PolygonLightData* pxLight = s_pxPolygonLightDataList->GetPointer(iHandle);
		pxLight->m_xLightSource.ResetToDefaults();

		s_pxCombinedLightList->AddToList(pxLight);

	}
	else
	{
		Asura_Assert( iHandle != ASURA_INVALID_HANDLE, "Unable to allocate a new light source");
	}


	return iHandle;
}

void Tools_PhotonMapping_System::DestroyPolygonLightSource(const Asura_Handle iHandle)
{
	Asura_Assert( iHandle != ASURA_INVALID_HANDLE, "Invalid handle");

	Tools_PhotonMapping_PolygonLightData* pxLight = s_pxPolygonLightDataList->GetPointer(iHandle);
	pxLight->m_xLightSource.Shutdown();
	pxLight->m_xDirectEstimationPhotonMap.Clear();

	s_pxCombinedLightList->RemoveFromList(pxLight);
	s_pxPolygonLightDataList->Deallocate(iHandle);

	//deleting a light will invalidate the GI samples. RAR 17/08/2005
	Asura_GI_Solution::InvalidateSamples();
	
	--s_uNumberOfPolygonLightSources;
}


Tools_PhotonMapping_LightSource* Tools_PhotonMapping_System::GetLightSource(const Asura_Handle iHandle)
{
	Asura_Assert( iHandle != ASURA_INVALID_HANDLE, "Invalid handle");

	return &(s_pxLightDataList->GetPointer(iHandle)->m_xLightSource);
}

Tools_PhotonMapping_TemplateData* Tools_PhotonMapping_System::GetTemplateData(const Asura_Handle iHandle)
{
	Asura_Assert( iHandle != ASURA_INVALID_HANDLE, "Invalid handle");

	return &(s_pxLightDataList->GetPointer(iHandle)->m_xTemplateData);
}

Tools_PhotonMapping_PolygonLightSource* Tools_PhotonMapping_System::GetPolygonLightSource(const Asura_Handle iHandle)
{
	Asura_Assert( iHandle != ASURA_INVALID_HANDLE, "Invalid handle");

	return &(s_pxPolygonLightDataList->GetPointer(iHandle)->m_xLightSource);
}

Tools_PhotonMapping_PolygonLightSource* Tools_PhotonMapping_System::GetPolygonLightSourceFromID(int iModule,u_int uLightID)
{
	Asura_TrackingHandlePool<Tools_PhotonMapping_PolygonLightData>::Iterator xIterator = Tools_PhotonMapping_System::GetPolygonLightDataTrackingHandlePool()->GetIterator();

	for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
	{
		Tools_PhotonMapping_PolygonLightData* pxLightData = xIterator.GetCurrentPointer();

		if(pxLightData->m_xLightSource.GetModuleIndex()==iModule && pxLightData->m_xLightSource.GetLightID() == uLightID)
		{
			return &pxLightData->m_xLightSource;
		}
	}
	return 0;
}


Asura_Handle Tools_PhotonMapping_System::GetLightHandleFromID(u_int uID)
{
	Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>::Iterator xIterator = Tools_PhotonMapping_System::GetLightDataTrackingHandlePool()->GetIterator();

	for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
	{
		Tools_PhotonMapping_TemplateData* pxTemplate = GetTemplateData(xIterator.GetCurrentHandle());
		if(pxTemplate->m_uID == uID)
		{
			//find the light
			return xIterator.GetCurrentHandle();
		}
	}

	return ASURA_INVALID_HANDLE;

}


void Tools_PhotonMapping_System::SamplePhotonMap(const Asura_Vector_3& xPosition, const Asura_Vector_3& xNormal, Asura_RGBA& xColour)
{
	Asura_Vector_3 xFlux;
	xFlux.Zero();

	if (!s_bBeenInitialised)
	{
		xColour.Zero();
		return;
	}


////

	float fRadiusEstimate;

	//if(s_bPreviousSampleRadiusValid)
	{
		Asura_Vector_3 xDisplacement = xPosition - s_pxSample->m_xCentre;

		fRadiusEstimate = xDisplacement.Magnitude() + Asura_Maths::Sqrt(s_pxSample->m_fMaxDistanceSquared);
		fRadiusEstimate *= fRadiusEstimate;

		if (s_fSampleRadiusSquared<fRadiusEstimate) fRadiusEstimate = s_fSampleRadiusSquared;
	}

	s_pxSample->Reset(xPosition,xNormal);
	s_pxSample->m_fMaxDistanceSquared = fRadiusEstimate;


	Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>::Iterator xIterator = s_pxLightDataList->GetIterator();

	for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
	{
    	Tools_PhotonMapping_LightData* pxLightData = xIterator.GetCurrentPointer();

		if (!pxLightData->m_xLightSource.IsParallel()) 
		{
			Tools_PhotonMap& xPhotonMap = pxLightData->m_xPhotonMap;

			s_pxSample->SetCurrentPhotonMap(&xPhotonMap);
			xPhotonMap.AddPhotonsToSample(*s_pxSample);
		}
	}
	
	s_pxSample->CalculateFlux(xFlux);

	
////
	xFlux *= 10.0f;
	
	float fMax = 6.0f;

	if (xFlux.x>fMax) xFlux.x=fMax;
	if (xFlux.y>fMax) xFlux.y=fMax;
	if (xFlux.z>fMax) xFlux.z=fMax;

	xFlux.x = sqrtf(1.0f - expf(-xFlux.x));
	xFlux.y = sqrtf(1.0f - expf(-xFlux.y));
	xFlux.z = sqrtf(1.0f - expf(-xFlux.z));
		
	xColour.Set( xFlux.x, xFlux.y, xFlux.z, 0.0f);
}

bool Tools_PhotonMapping_System::GetDirectEstimatePhotonMapSample( const u_int uPoly, const u_int uVertex, const u_int uModule, const Asura_Vector_3& xNormal, Asura_RGBA& xColour )
{
	xColour.Zero();
	

	if (!s_bBeenInitialised)
	{
		return false;
	}
	#ifdef ASURA_USE_STREAMING
	//Module/vertex indeces from other zones will be meaningless , so don't attempt to light them. RAR 21/02/2006
	if(!Tools_Streaming_System::IsSetToMainZone()) return false;
	#endif

	Asura_Hashed_List_It<Tools_PhotonMapping_BaseLightData> xIterator(s_pxCombinedLightList);

	for (xIterator.Restart(); !xIterator.Done(); xIterator.Next())
	{
    	Tools_PhotonMapping_BaseLightData* pxLightData = xIterator.GetCurrent();

		const Tools_PhotonMapping_LightSource* pxLightSource = pxLightData->GetLightSource();
		if (!pxLightSource->IsParallel() && Tools_LightSets::IsMemberOfCurrentLightSets(pxLightSource->GetLightSets())) 
		{
				
			Tools_PhotonMap_DirectEstimation& xPhotonMap = pxLightData->m_xDirectEstimationPhotonMap;

//			if (!xPhotonMap.m_pxSamples) continue;

			u_int uVertexIndex = Asura_Module_System::GetMeshForModule(uModule)->PolygonDataPtr[uPoly].VertexID[uVertex];

			const Tools_PhotonMap_DirectEstimation_Sample* pxSample = xPhotonMap.GetSample( uVertexIndex, uModule );
			
			if (pxSample)
			{
				Asura_SphericalHarmonics_Order3 xIrradiance = pxSample->m_xIrradiance;
				xIrradiance.ScaleCoefficientsForIrradiance();

				float fPowerSign = Asura_Maths::Sign(pxLightSource->GetPower());
				
				float fLightR = xIrradiance.EvaluateInDirection( xNormal ) * Asura_Maths::Abs(pxLightSource->GetPower()) / 10.0f;
				float fLightG = fLightR;
				float fLightB = fLightR;


				if (pxLightSource->IsBleached())
				{
					fLightR *= pxLightSource->GetColour().x;
					fLightG *= pxLightSource->GetColour().y;
					fLightB *= pxLightSource->GetColour().z;
				}
				else
				{
					fLightR *= 255.0f;
					fLightG *= 255.0f;
					fLightB *= 255.0f;
				}

				const float fMax = 6.0f;

				if(fLightR<0) fLightR = 0.0f;
				if ( fLightR > fMax ) fLightR = fMax;
				fLightR = sqrtf( 1.0f - expf(-fLightR) );

				if(fLightG<0) fLightG = 0.0f;
				if ( fLightG > fMax ) fLightG = fMax;
				fLightG = sqrtf( 1.0f - expf(-fLightG) );

				if(fLightB<0) fLightB = 0.0f;
				if ( fLightB > fMax ) fLightB = fMax;
				fLightB = sqrtf( 1.0f - expf(-fLightB) );

			
				if (pxLightSource->IsBleached())
				{
				}
				else
				{
					fLightR *= pxLightSource->GetColour().x / 255.0f;
					fLightG *= pxLightSource->GetColour().y / 255.0f;
					fLightB *= pxLightSource->GetColour().z / 255.0f;
				}

				Asura_RGBA xAdditionColour
				(
					fLightR*fPowerSign,
					fLightG*fPowerSign,
					fLightB*fPowerSign,
					0.0f
				);

				xColour+=xAdditionColour;
			}
			else if(!xPhotonMap.IsVertexAndModuleIndexInRange(uVertex,uModule))
			{
				//dodgy ligting data for this vertex , so return false so a default light
				//value can be placed on it
				return false;
			}
		}
	}

	

	return true;
}



void Tools_PhotonMapping_System::CalculateEnvironmentIrradiance( void )
{
	s_xEnvironmentIrradianceRed.Zero();
	s_xEnvironmentIrradianceGreen.Zero();
	s_xEnvironmentIrradianceBlue.Zero();

	s_pxLightDataList->Traverse(CalculateSHForLightSource);

	s_xEnvironmentIrradianceRed.ScaleCoefficientsForIrradiance();
	s_xEnvironmentIrradianceGreen.ScaleCoefficientsForIrradiance();
	s_xEnvironmentIrradianceBlue.ScaleCoefficientsForIrradiance();

	s_bEnvironmentIrradianceValid = true;
}


void Tools_PhotonMapping_System::CalculateSHForLightSource(Tools_PhotonMapping_LightData* pxLightData)
{
	
	if (pxLightData)
	{
		const Tools_PhotonMapping_LightSource& xLightSource = pxLightData->m_xLightSource;

		if (xLightSource.IsParallel() && Tools_LightSets::IsMemberOfCurrentLightSets(xLightSource.GetLightSets()))
		{
			const Asura_Matrix_3x3& m_xOrientation = xLightSource.GetOrientation();
			const Asura_Vector_3 xDirection( -m_xOrientation.e13, -m_xOrientation.e23, -m_xOrientation.e33 );

			const Asura_Vector_3 xColour = xLightSource.GetColour() * xLightSource.GetPower() / 10.0f;

			s_xEnvironmentIrradianceRed.AddSampleForDirection( xColour.x, xDirection );
			s_xEnvironmentIrradianceGreen.AddSampleForDirection( xColour.y, xDirection );
			s_xEnvironmentIrradianceBlue.AddSampleForDirection( xColour.z, xDirection );
		}
	}
}

void Tools_PhotonMapping_System::RenderPhotonMapRepresentation(void)
{
//	if (!s_bBeenInitialised)
//	{
//		Asura_Assert( false, "Not yet been initialised" );
//		return;
//	}
//
//	if (!Tools_Render::HighlightSolid) return;
//
//	for (u_int u=0; u<s_uNumberOfPhotonHits && u<25000 ; u++)
//	{
//
//
//
//		Asura_Vector_3 xPosition = s_pxPhotonHits[u].GetPosition();
//		Asura_Vector_3 xPosition2 = xPosition;
//
//		xPosition2.y += 0.05f;
//		xPosition.y -= 0.05f;
//		
//
//		u_int uColour = 0xffff0000;
//		Asura_Render::RenderLine(xPosition,xPosition2,uColour);
//	}
}



u_int Tools_PhotonMapping_System::GetNumberOfSamplesForAccuracyLevel( u_int uAccuracy)
{
	switch (uAccuracy)
	{
		case 1:
		{
			return 50;
		}
		case 2:
		{
			return 250;
		}
		case 3:
		{
			return 500;
		}
		case 4:
		{
			return 1000;
		}
		case 5:
		{
			return 2000;
		}
		default:
		{
			return (uAccuracy-3)*1000;
		}
	}
}

float Tools_PhotonMapping_System::GetSampleRadiusForAccuracyLevel( u_int uAccuracy , bool bShortRange)
{
	// KJFL - 02 November 2004 - the relationship here is
	//
	// h2 = h1 * ((n1 / n2) ^ (1/5))
	//
	// where h are the filter sizes and n are the sample sizes (which are the square of numbers above)

	if(bShortRange)
	{
		// KJFL - 02 November 2004 - make it such that a 2000 (^2) sample photon light, which is currently our
		// accuracy 5 setting, means light will not leak more than 0.1m. All other accuracies will be scaled from
		// this.

		return 0.1f * powf( 2000.0f / GetNumberOfSamplesForAccuracyLevel(uAccuracy), 2.0f/5.0f);
	}
	else
	{
		switch (uAccuracy)
		{
			case 1:
			{
				return 4.0f;
			}
			case 2:
			{
				return 2.77f;
			}
			case 3:
			{
				return 2.1f;
			}
			case 4:
			{
				return 1.59f;
			}
			case 5:
			{
				return 1.2f;
			}
			default:
			{
				return 1.2f * powf( 2000.0f / GetNumberOfSamplesForAccuracyLevel(uAccuracy), 2.0f/5.0f);
			}
		}
	}
}

enum Tools_PhotonMap_Validity Tools_PhotonMapping_System::CheckPhotonMapsAreValid( void )
{
	enum Tools_PhotonMap_Validity eValid = TOOLS_PHOTONMAP_VALID;

	#ifdef ASURA_USE_STREAMING
	if(!Tools_Streaming_System::IsSetToMainZone())
	{
		//Don't bother testing validity of lighting if not set to the main zone. RAR 05/05/2006
		return TOOLS_PHOTONMAP_VALID;
	}
	#endif


	Asura_Hashed_List_It<Tools_PhotonMapping_BaseLightData> xIterator(s_pxCombinedLightList);

	for (xIterator.Restart(); !xIterator.Done(); xIterator.Next())
	{
    	Tools_PhotonMapping_BaseLightData* pxLightData = xIterator.GetCurrent();

		Tools_PhotonMapping_LightSource* pxLightSource = pxLightData->GetLightSource();

		//Checking all lights that have photon maps ,even if they don't actually use them , to
		//avoid any issues with having out of date photon maps with the wrong number of modules
		//RAR 06/09/2004
		if (pxLightSource->HasPhotonMap()) 
		{
			Tools_PhotonMap_DirectEstimation& xPhotonMap = pxLightData->m_xDirectEstimationPhotonMap;

			const u_int uNumberOfModules = Asura_Module_System::GetNumberOfModules();
			
			if ( xPhotonMap.m_uNumberOfModules != uNumberOfModules )
			{
				pxLightSource->SetPhotonMapValidity(0);
				pxLightSource->SetHasPhotonMap(false);
				xPhotonMap.Clear();
				pxLightData->SetModified();
				
			}
			
			else if(pxLightSource->IsPhotonMapPartiallyValid())
			{
				for ( u_int uModule=0; uModule<uNumberOfModules; uModule++ )
				{
					if (xPhotonMap.m_pxModules[uModule].m_uNumberOfSamples == 0) continue;

					const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(uModule);
	
					u_int uVertices = pxMesh->NumberOfVertices;
		
					if (xPhotonMap.m_pxModules[uModule].m_uNumberOfSamples != uVertices)
					{
						pxLightSource->SetPhotonMapValidity(0);
						//RAR Leave the photon map data for the moment , since while invalid , it may be better
						//than nothing. RAR 03/09/2004
						//xPhotonMap.Clear();
						pxLightData->SetModified();
						break;
					}
				}
			}
		}
		if(!pxLightSource->IsParallel()) 
		{
			if(!pxLightSource->IsPhotonMapPartiallyValid())
			{
				eValid = TOOLS_PHOTONMAP_INVALID;
			}
			else if(!pxLightSource->IsPhotonMapCompletelyValid())
			{
				eValid = (Tools_PhotonMap_Validity)Asura_Maths::Min(eValid,TOOLS_PHOTONMAP_REDUCEDACCURACY);
			}
		}

	}

	return eValid;
}


static Asura_Vector_3 g_xBBMin;
static Asura_Vector_3 g_xBBMax;

void Tools_PhotonMap::ConstructLeftBalancedTree(void)
{
	// need to do something here
	if (m_uNumberOfPhotonHits==0) return;

	Tools_PhotonMapping_PhotonHit* pxSortArray1 = new Tools_PhotonMapping_PhotonHit[m_uNumberOfPhotonHits+1];
	Tools_PhotonMapping_PhotonHit** ppxSortArray2 = new Tools_PhotonMapping_PhotonHit*[m_uNumberOfPhotonHits+1];

	Asura_Bounding_Box g_xBB;
	g_xBB.Reset();

	ppxSortArray2[0] = 0;
	for (u_int u=0; u<m_uNumberOfPhotonHits; u++)
	{
		ppxSortArray2[u+1] = m_pxDynamicPhotonHitArray->GetPointer(u);
		g_xBB.Set(ppxSortArray2[u+1]->GetPosition());
	}

	g_xBBMax = Asura_Vector_3(g_xBB.MaxX,g_xBB.MaxY, g_xBB.MaxZ);
	g_xBBMin = Asura_Vector_3(g_xBB.MinX,g_xBB.MinY, g_xBB.MinZ);

	BalanceSegment(pxSortArray1, ppxSortArray2, 1, 1, m_uNumberOfPhotonHits);

	delete [] ppxSortArray2;

//	delete [] m_pxPhotonHits;

	m_pxPhotonHits = pxSortArray1;
}



void Tools_PhotonMap::BalanceSegment(Tools_PhotonMapping_PhotonHit* pxBalancedArray, Tools_PhotonMapping_PhotonHit** ppxOriginalArray, const u_int uIndex, const u_int uStart, const u_int uEnd)
{
	u_int uMedian = 1;

	while ((4*uMedian)<=(uEnd-uStart+1))
	{
		uMedian += uMedian;
	}

	if ((3*uMedian)<=(uEnd-uStart+1))
	{
		uMedian += uMedian;
		uMedian += uStart-1;
	}
	else
	{
		uMedian = uEnd - uMedian + 1;
	}

	float fLengthAlongX = g_xBBMax.x-g_xBBMin.x;
	float fLengthAlongY = g_xBBMax.y-g_xBBMin.y;
	float fLengthAlongZ = g_xBBMax.z-g_xBBMin.z;


	u_int uAxis;

	if (fLengthAlongX > fLengthAlongY)
	{
		if (fLengthAlongX > fLengthAlongZ)
		{
			uAxis = 0;
		}
		else
		{
			uAxis = 2;
		}
	}
	else if (fLengthAlongY > fLengthAlongZ)
	{
		uAxis = 1;
	}
	else
	{
		uAxis = 2;
	}
	

	MedianSplit(ppxOriginalArray, uStart, uEnd, uMedian, uAxis);

	pxBalancedArray[uIndex] = *ppxOriginalArray[uMedian];
	pxBalancedArray[uIndex].SetSplittingPlane(uAxis);

	if (uMedian>uStart)
	{
		if (uStart<uMedian-1)
		{
			const float tmp = g_xBBMax[uAxis];

			g_xBBMax[uAxis] = pxBalancedArray[uIndex].GetPosition()[uAxis];
			BalanceSegment(pxBalancedArray,ppxOriginalArray,2*uIndex,uStart, uMedian-1);
			g_xBBMax[uAxis] = tmp;
		}
		else
		{
			pxBalancedArray[2*uIndex] = *ppxOriginalArray[uStart];
		}
	}

	if (uMedian<uEnd)
	{
		if (uMedian+1 < uEnd)
		{
			const float tmp = g_xBBMin[uAxis];

			g_xBBMin[uAxis] = pxBalancedArray[uIndex].GetPosition()[uAxis];
			BalanceSegment(pxBalancedArray,ppxOriginalArray,2*uIndex+1,uMedian+1, uEnd);
			g_xBBMin[uAxis] = tmp;
		}
		else
		{
			pxBalancedArray[2*uIndex+1] = *ppxOriginalArray[uEnd];
		}
	}
}

void Tools_PhotonMap::MedianSplit(Tools_PhotonMapping_PhotonHit** ppxArray, const u_int uStart, const u_int uEnd, const u_int uMedian, const u_int uAxis)
{
	u_int uLeft = uStart;
	u_int uRight = uEnd;

	while (uRight>uLeft)
	{
		const float f = ppxArray[uRight]->GetPosition()[uAxis];

		u_int i = uLeft-1;
		u_int j = uRight;

		for (;;)
		{
			while ( ppxArray[++i]->GetPosition()[uAxis] < f );
			while ( ppxArray[--j]->GetPosition()[uAxis] > f && j>uLeft);

			if (i>=j) break;

			Tools_PhotonMapping_PhotonHit* pxTemp = ppxArray[i];
			ppxArray[i] = ppxArray[j];
			ppxArray[j] = pxTemp;
		}

		Tools_PhotonMapping_PhotonHit* pxTemp = ppxArray[i];
		ppxArray[i] = ppxArray[uRight];
		ppxArray[uRight] = pxTemp;

		if (i>=uMedian)
		{
			uRight = i-1;
		}
		if (i<=uMedian)
		{
			uLeft =  i+1;
		}

	}
}
void Tools_PhotonMap::TestTree(const float fMaxDistance, const u_int uIndex, Tools_PhotonMapping_RegionSample& s_xSample)
{
	if (uIndex>m_uNumberOfPhotonHits) return;

	Tools_PhotonMapping_PhotonHit& xPhoton = m_pxPhotonHits[uIndex];


	u_int uAxis = xPhoton.GetPlaneAxis();
	
	float fPlaneDist = s_xSample.m_xCentre[uAxis] - xPhoton.GetPosition()[uAxis];

	if (fPlaneDist>0)
	{
		TestTree(fMaxDistance, uIndex*2+1, s_xSample);

		if (fPlaneDist*fPlaneDist<s_xSample.m_fMaxDistanceSquared)
		{
			TestTree(fMaxDistance, uIndex*2, s_xSample);
		}
	}
	else
	{
		TestTree(fMaxDistance, uIndex*2, s_xSample);

		if (fPlaneDist*fPlaneDist<s_xSample.m_fMaxDistanceSquared)
		{
			TestTree(fMaxDistance, uIndex*2+1, s_xSample);
		}
	}

	s_xSample.AddPhoton(&xPhoton);

	
}


void RecurseLightingGroup(const int iModuleIndex, const int iPolyIndex, const u_int uLightingGroup)
{
#if 0
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModuleIndex);

	// set the group
	pxMesh->m_puLightingGroup[iPolyIndex] = uLightingGroup;

	// then find all connected polygons that share a smoothing group, and put them in the same group
	const Tools_Polygon_Adjacency* pxAdjacencyList = Tools_Adjacency::GetPolygonAdjacencies(iModuleIndex,iPolyIndex);

	for(const Tools_Adjacent_Polygon* pxAdj = pxAdjacencyList->GetFirstAdjacency(); pxAdj; pxAdj = pxAdj->GetNext())
	{
		if (pxAdjPoly->GetNumSharedVertices()<2) continue;

		Asura_Mesh* pxMesh2 = Asura_Module_System::GetMeshForModule(pxAdj->GetModuleIndex());

		if (pxMesh2->m_puLightingGroup[pxAdj->GetPolygonIndex()]!=0xffffffff) continue;

		if (pxMesh2->GetSmoothingGroup(pxAdj->GetPolygonIndex())&pxMesh->GetSmoothingGroup(iPolyIndex)&0x7fffffff)
		{
			RecurseLightingGroup(pxAdj->GetModuleIndex(), pxAdj->GetPolygonIndex(), uLightingGroup);				
		}
	}

#endif
}

void Tools_PhotonMapping_System::CalculateLightingGroups(void)
{
#if 0
#if 0
	// KJL - 18/12/2003 16:25:13 - set all lighting groups to 0xffffffff, which indicates
	// they're unset.
	for(int i=0; i<Asura_Module_System::GetNumberOfModules(); i++)
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
		
		if (pxMesh->m_puLightingGroup) delete [] (pxMesh->m_puLightingGroup);

		pxMesh->m_puLightingGroup = new u_int [pxMesh->NumberOfPolygons];

		u_int uNumberOfPolys = pxMesh->NumberOfPolygons;
		for (u_int u=0; u<uNumberOfPolys; u++)
		{
			pxMesh->m_puLightingGroup[u] = 0xffffffff;
		}
	}

	s_uNumberOfLightingGroups = 0;

	for(int i=0; i<Asura_Module_System::GetNumberOfModules(); i++)
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
		
		// search for a polygon whose lighting group has not yet been set...
		u_int uNumberOfPolys = pxMesh->NumberOfPolygons;
		for (u_int u=0; u<uNumberOfPolys; u++)
		{
			if (pxMesh->m_puLightingGroup[u]!=0xffffffff) continue;
			

			RecurseLightingGroup(i,u,s_uNumberOfLightingGroups);

			// fire up a new group
			++s_uNumberOfLightingGroups;
		}
	}
#else
		// KJL - 18/12/2003 16:25:13 - set all lighting groups to 0xffffffff, which indicates
	// they're unset.
	for(int i=0; i<Asura_Module_System::GetNumberOfModules(); i++)
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
		
		if (pxMesh->m_puLightingGroup) delete [] (pxMesh->m_puLightingGroup);

		pxMesh->m_puLightingGroup = new u_int [pxMesh->NumberOfPolygons];

		u_int uNumberOfPolys = pxMesh->NumberOfPolygons;
		for (u_int u=0; u<uNumberOfPolys; u++)
		{
			pxMesh->m_puLightingGroup[u] = 0;
		}
	}

//	s_uNumberOfLightingGroups = 1;

#endif
#endif
}
/*

void Tools_ModuleSmoothing::CalculateNormals(int iModuleIndex)
{
	Asura_Assert(iModuleIndex>=0 && iModuleIndex<=Asura_Module_System::GetNumberOfModules(),"");
	Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(iModuleIndex);

	//make sure face normals are up to date
	if(!pxMesh->m_bFaceNormalsValid)
	{
		pxMesh->CreateFaceNormals();
	}


	for(int i=0;i<pxMesh->NumberOfPolygons;i++)
	{
		Asura_Mesh_Polygon* pxPoly = &pxMesh->PolygonDataPtr[i];
		int iNumVerts = pxPoly->NumberOfVertices();

		if( !pxMesh->NormalsDataPtr[i].VertexNormal )
		{
			pxMesh->NormalsDataPtr[i].VertexNormal = new Asura_Vector_3[4];
		}


		int iSmoothGroup = pxMesh->GetSmoothingGroup(i); 
		for(int j=0;j<iNumVerts;j++)
		{
			Asura_Vector_3 xNormal;
			xNormal.Zero();

			
			if(iSmoothGroup)
			{
				Asura_Vector_3 xNormalsFound[16];
				int iNumNormalsFound = 0;

				//get a list of polygons using this vertex
				int iVertexID = pxPoly->VertexID[j];
				const Tools_Polygon_Adjacency* pxAdj = Tools_Adjacency::GetVertexAdjacencies(iModuleIndex,iVertexID);

				for(int k=0;k<pxAdj->NumberOfAdjacentPolygons;k++)
				{
					int iModuleIndex2 = pxAdj->AdjacentPolygons[k].ModuleIndex;
					int iPolygonIndex2 = pxAdj->AdjacentPolygons[k].PolygonIndex;

					Asura_Mesh* pxMesh2 = Asura_Module_System::GetMeshForModule(iModuleIndex2);
					Asura_Assert(pxMesh2,"");
					if(!pxMesh2->m_bFaceNormalsValid) pxMesh2->CreateFaceNormals();

					int iSmoothGroup2 = pxMesh2->GetSmoothingGroup(iPolygonIndex2);
					
					//add on the face normal for this polygon , if any smoothing groups are shared
					if(iSmoothGroup & iSmoothGroup2)
					{
						//make sure this normal hasn't already been added
						bool bAlreadyFound = false;
						for(int l=0;l<iNumNormalsFound;l++)
						{
							if(xNormalsFound[l] == pxMesh2->NormalsDataPtr[iPolygonIndex2].FaceNormal)
							{
								bAlreadyFound = true;
								break;
							}

						}
						if(!bAlreadyFound)
						{
							xNormalsFound[iNumNormalsFound++] = pxMesh2->NormalsDataPtr[iPolygonIndex2].FaceNormal;
							xNormal += pxMesh2->NormalsDataPtr[iPolygonIndex2].FaceNormal;

							if(iNumNormalsFound>=16) iNumNormalsFound = 15;
						}
					}
			
				}
			}
			//normalise the vertex normal
			float fLength = xNormal.Magnitude();
			if(fLength)
			{
				xNormal /= fLength;
			}
			else
			{
				xNormal = pxMesh->NormalsDataPtr[i].FaceNormal;
			}

			pxMesh->NormalsDataPtr[i].VertexNormal[j] = xNormal;
		}
	}
	pxMesh->m_bVertexNormalsValid = true;
	pxMesh->m_bSmoothedAcrossModules = true;
}
*/

void Tools_PhotonMapping_System::CheckValidityOfGlobalIllumination(void)
{
	static u_int uFrameLastPrinted = 0xffffffff;

	if (Asura_Timers::GetFrameNumber() == uFrameLastPrinted)
	{
		return;
	}

	#ifdef ASURA_USE_STREAMING
	if(!Tools_Streaming_System::IsSetToMainZone())
	{
		//Don't bother testing the validity of lighting for anything other than the main zone , since
		//we only have lights for the main zone. RAR 05/05/2006
		return;
	}
	#endif

	//Flags to reduce the flood of warnings. Warn the first time something goes wrong , and only warn
	//again if it has been made right before going wrong again. RAR 03/09/2004
	static bool s_bWarnedAboutELT = false;
	static bool s_bWarnedAboutQuickELT = false;
	static bool s_bWarnedAboutPhotonMaps = false;
	static bool s_bWarnedAboutPhotonMapsReducedAcc = false;
	static bool s_bWarnedAboutGI = false;

	uFrameLastPrinted = Asura_Timers::GetFrameNumber();

	if(AreThereAnyParallelLights())
	{
		// If we haven't loaded an .elt file (or it was invalid etc.) then process
		// the environment from scratch.
		if ( !Tools_EnvironmentalLightTransport::IsEnvironmentProcessed() )
		{
			Tools_EnvironmentalLightTransport::MakeBlankData();
			if(!s_bWarnedAboutELT)
			{
				Tools_Timed_Message_System::AddMessage(0xffff4040,"WARNING: No ELT data found.");
				Tools_Timed_Message_System::AddMessage(0xffff4040,"You will need to recalculate ELT before infinite parallel lightsources will be visible.");
				s_bWarnedAboutELT = true;
			}
		}
		else if (! Tools_EnvironmentalLightTransport::DoesEnvironmentMatch() )
		{
			//Tools_EnvironmentalLightTransport::MakeBlankData();
			if(!s_bWarnedAboutELT)
			{
				Tools_Timed_Message_System::AddMessage(0xffff4040,"WARNING: Module data has changed since last ELT was calculated.");
				Tools_Timed_Message_System::AddMessage(0xffff4040,"You will need to recalculate ELT before infinite parallel lightsources will be visible.");
				s_bWarnedAboutELT = true;
			}
		}
		else
		{
			if(Tools_EnvironmentalLightTransport::IsQuickData())
			{
				//warn if lighting is correct , but using quick ELT
				if(!s_bWarnedAboutQuickELT)
				{
					Tools_Timed_Message_System::AddMessage(0xffffff00,"WARNING: Lighting calculated with Quick ELT.");
					s_bWarnedAboutQuickELT = true;
				}
			}
			else
			{
				s_bWarnedAboutQuickELT = false;
			}
			s_bWarnedAboutELT = false;
		}
	}

	Tools_PhotonMap_Validity ePhotonValid = Tools_PhotonMapping_System::CheckPhotonMapsAreValid();
	
	if (ePhotonValid == TOOLS_PHOTONMAP_INVALID  )
	{
		if(!s_bWarnedAboutPhotonMaps)
		{
			Tools_Timed_Message_System::AddMessage(0xffff4040,"WARNING: Photon maps are invalid (either not calculated or modules have changed).");
			Tools_Timed_Message_System::AddMessage(0xffff4040,"You will need to recalculate the photons maps.");
			s_bWarnedAboutPhotonMaps = true;
		}
	}
	else
	{
		if(ePhotonValid == TOOLS_PHOTONMAP_REDUCEDACCURACY)
		{
			if(!s_bWarnedAboutPhotonMapsReducedAcc)
			{
				Tools_Timed_Message_System::AddMessage(0xffffff00,"WARNING: Photon maps calculated at reduced accuracy.");
				s_bWarnedAboutPhotonMapsReducedAcc = true;
			}
		}
		else
		{
			s_bWarnedAboutPhotonMapsReducedAcc = false;
		}
		s_bWarnedAboutPhotonMaps = false;
	}

	if(!Asura_GI_Solution::GetCurrentHandler().AreSamplesValid())
	{
		if(!s_bWarnedAboutGI)
		{
			Tools_Timed_Message_System::AddMessage(0xffffff00,"WARNING: GI samples are invalid (or don't exist).");
			s_bWarnedAboutGI = true;		
		}
	}
	else
	{
		s_bWarnedAboutGI = false;
	}

}

bool Tools_PhotonMapping_System::UpdateProgress(int i, int j, const char* szHeading)
{
	static float s_fStartTime = 0;
	static float s_fLastTime = 0;

	Asura_Timers::SetMaxFrameRate(0.0f);
   	Asura_Timers::HandlePrimaryTimer();

	if (i == 0)
	{
		s_fLastTime = -1.0f;
		s_fStartTime = Asura_Timers::GetUncappedPhase();
	}

	// (MH - 16.02.2005) j can be 0 if there are no GI samples.
	//Asura_Assert(j!=0,"hey!");

    float fPercent = ( j ? ((float)i)/((float)j)*100.0f : 0.0f );
	float fTime = Asura_Timers::GetUncappedPhase();

	if (fTime-s_fLastTime<0.1f)
	{
		return false;
	}
	s_fLastTime = fTime;

	Asura_Windows::CheckForWindowsMessages();
	Asura_UserInput::Update();
	Asura_Cursor::OncePerFrameUpdate();

	//Avoid rendering anything with the reference rasterizer , since it will slow down the process massively
	//This can happen if tools is started up while batch processing while the computer is locked.
	//RAR 14/04/2008
	if(Asura_Windows::IsApplicationActive() && false == Tools_BatchProcessing::IsConsoleMode() && !Asura_Direct3D::IsReferenceRasterizer())
	{
		Asura_Render::BeginFrame();
		float width = (float)Asura_Viewport::GetWidth();
		float height = (float)Asura_Viewport::GetHeight();
		if(s_bPreserveScreenDuringUpdateProgress && !Asura_Direct3D_BackBufferCapture::IsCaptureDirty())
		{
			//render old back buffer as backdrop for the text
			D3DQuadVertex axVerts[4];
			short ausIndeces[6] = {0,1,2,2,3,0};
			for(int i=0;i<4;i++)
			{
				axVerts[i].X = (i==1 || i==2) ? width-0.5f : -0.5f;
				axVerts[i].Y = (i==2 || i==3) ? height-0.5f : -0.5f;
				axVerts[i].Z = 0;
				axVerts[i].RHW = 1000.0f;
				axVerts[i].Diffuse = 0xffffffff;
				axVerts[i].U = (i==1 || i==2) ? 1.0f : 0.0f;
				axVerts[i].V = (i==2 || i==3) ? 1.0f : 0.0f;
			}

		 	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_SINGLE_PASS);
		 	Asura_Render::CurrentStates.RequestTextureOpMode(ASURA_TEXTUREOP_CUSTOM);

			Asura_Direct3D_Shaders::RequestFixedFunctionVertexShader(D3DFVF_QUADVERTEX);
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(false);
			Asura_Direct3D::D3DDevice->SetTexture(0,Asura_Direct3D_BackBufferCapture::GetCaptureTexture());

			Asura_Direct3D::D3DDevice->DrawIndexedPrimitiveUP
				(
				D3DPT_TRIANGLELIST,
				0,
				4,
				2,
				(void*)ausIndeces,
				D3DFMT_INDEX16,
				(void*)axVerts,
				sizeof(D3DQuadVertex)
				);

			
			Asura_Render::CurrentStates.RequestZBufferWriteEnabled(true);
			Asura_Render::CurrentStates.RequestTextureIndex(0, ASURA_TEXTURE_INVALID);
		}
		else
		{
			//black out the screen
			Asura_Render::Overlay.SetRectXY( 0.0f, 0.0f, width, height );
			Asura_Render::Overlay.SetRectUV( 0.0f, 0.0f, 1.0f, 1.0f );
			Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
			Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
			Asura_Render::Overlay.Colour=0xff000000;
			Asura_Render::Overlay.Render();
			Asura_Render::FlushDisplay();
		}

		//show some text
		float print_x = width/2.0f;
		float print_y = height/2.0f;
		Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);

		print_y-=2.0f;

		if(szHeading)
		{
			Asura_Text_Output::Print(print_x,print_y,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"%s",szHeading);
		}
		
		print_y+=2.0f;
		Asura_Text_Output::Print(print_x,print_y,0xff00ff00,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"%3.0f%% Complete",fPercent);

		print_y+=4.0f;
		Asura_Text_Output::Print(print_x,print_y,0xffff0000,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Hold escape to abort");

		if (i!=0)
		{
			print_y+=2.0f;
			float fTimeTaken = (fTime-s_fStartTime);
			Asura_Text_Output::Print(print_x,print_y,0xff7f7f7f,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Time taken              : %.1fs",fTimeTaken);
			
			print_y+=2.0f;
			float fEstimatedTime = ((fTime-s_fStartTime)/(float)i)*(float)( j-i);
			Asura_Text_Output::Print(print_x,print_y,0xff7f7f7f,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Estimated time remaining: %.1fs",fEstimatedTime);

			print_y+=2.0f;
			float fEstimatedTotal = ((fTime-s_fStartTime)/(float)i)*(float)(j);
			Asura_Text_Output::Print(print_x,print_y,0xff7f7f7f,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Estimated total time    : %.1fs",fEstimatedTotal);
		}


		Tools_Timed_Message_System::Render();
		Asura_Cursor::Render();

		
		Asura_Render::EndFrame();
	}
	else
	{
		char acTitle[200];
		Asura_PrintfToBfr(acTitle,"%s - %s %3.0f%% Complete",sizeof(acTitle),true,Asura_Windows::GetDefaultWindowTitle(),szHeading ? szHeading : "",fPercent);
		Asura_Windows::SetWindowTitle(acTitle);
	}

	if (Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE))
	{
		if(Tools_BatchProcessing::IsBatch())
		{
			Tools_BatchProcessing::Abort();
		}
		return true;
	}
	else
	{
		return false;
	}

}



bool Tools_PhotonMapping_System::GetHighDynamicRangeSHsAtVertex(u_int uModule, u_int uVertex, Asura_SphericalHarmonics_Order3& xR, Asura_SphericalHarmonics_Order3& xG, Asura_SphericalHarmonics_Order3& xB )
{	

	Asura_SphericalHarmonics_Order3 xEnvR = Tools_PhotonMapping_System::GetEnvironmentIrradianceRed();
	Asura_SphericalHarmonics_Order3 xEnvG = Tools_PhotonMapping_System::GetEnvironmentIrradianceGreen();
	Asura_SphericalHarmonics_Order3 xEnvB = Tools_PhotonMapping_System::GetEnvironmentIrradianceBlue();

	xEnvR.Scale( 1.0f/255.0f );
	xEnvG.Scale( 1.0f/255.0f );
	xEnvB.Scale( 1.0f/255.0f );


	Asura_SphericalHarmonics_Order3 xTransportSH;
	if (Tools_EnvironmentalLightTransport::GetTransportHarmonic(  xTransportSH, uModule, uVertex ) )
	{

		xEnvR *= xTransportSH;
		xEnvG *= xTransportSH;
		xEnvB *= xTransportSH;
	}

	//Determine polygon flags used with vertex RAR 21/02/2006
	const Tools_Polygon_Adjacency* pxVertAdj = Tools_Adjacency::GetVertexAdjacencies(uModule,uVertex);
	u_int uPolyFlags = 0;
	const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(uModule);
	for(const Tools_Adjacent_Polygon* pxAdj = pxVertAdj->GetFirstAdjacency(); pxAdj; pxAdj = pxAdj->GetNext())
	{
		if(pxAdj->IsBetweenModules()) continue;
		const u_int uPolygonIndex = pxAdj->GetPolygonIndex();

		uPolyFlags |= pxMesh->GetPolygonFlags(uPolygonIndex);

	}
	if(!(uPolyFlags & AsuraPolygonFlag_IgnoreAmbientLight))
	{
		const Asura_RGBA& xGlobalAmbience = GetGlobalAmbience();

		// add in the overall ambience - this purposefully ignores the light transport harmonic
		xEnvR[0] += xGlobalAmbience.m_fR;
		xEnvG[0] += xGlobalAmbience.m_fG;
		xEnvB[0] += xGlobalAmbience.m_fB;
	}


	xR.Zero();
	xG.Zero();
	xB.Zero(); 

	xR += xEnvR;
	xG += xEnvG;
	xB += xEnvB;

	Asura_Hashed_List_It<Tools_PhotonMapping_BaseLightData> xIterator(s_pxCombinedLightList);

	for (xIterator.Restart(); !xIterator.Done(); xIterator.Next())
	{
    	Tools_PhotonMapping_BaseLightData* pxLightData = xIterator.GetCurrent();

		const Tools_PhotonMapping_LightSource* pxLightSource = pxLightData->GetLightSource();
		if (!pxLightSource->IsParallel()) 
		{
			Tools_PhotonMap_DirectEstimation& xPhotonMap = pxLightData->m_xDirectEstimationPhotonMap;

			const Tools_PhotonMap_DirectEstimation_Sample* pxSample = xPhotonMap.GetSample( uVertex, uModule );
			
			if (pxSample)
			{
				Asura_SphericalHarmonics_Order3 xIrradiance = pxSample->m_xIrradiance;
			//	xIrradiance.ScaleCoefficientsForIrradiance();

				xIrradiance.Scale( pxLightSource->GetPower() / 10.0f );

				xEnvR = xIrradiance;
				xEnvG = xIrradiance;
				xEnvB = xIrradiance;
			
				xEnvR.Scale( pxLightSource->GetColour().x );
				xEnvG.Scale( pxLightSource->GetColour().y );
				xEnvB.Scale( pxLightSource->GetColour().z );

				xR += xEnvR;
				xG += xEnvG;
				xB += xEnvB;

			}
		}
	}

	xR.ScaleCoefficientsForIrradiance();
	xG.ScaleCoefficientsForIrradiance();
	xB.ScaleCoefficientsForIrradiance();


	//Scale down if darken flag is set. RAR 07/03/2006
	if(uPolyFlags & AsuraPolygonFlag_Darken)
	{
		xR.Scale(ASURA_POLYFLAG_DARKEN_SCALE);
		xG.Scale(ASURA_POLYFLAG_DARKEN_SCALE);
		xB.Scale(ASURA_POLYFLAG_DARKEN_SCALE);
	}

	return true;
}

// rescale the SHs with our exposure function to take them from HDR
static void RescaleSH( Asura_SphericalHarmonics_Order3& xSH )
{
	float fIntensity =  xSH[0] + 0.0000001f + Asura_Maths::Sqrt( xSH[1] * xSH[1] + xSH[2] * xSH[2] + xSH[3] * xSH[3] ) ;

	float fScale = 2.0f * Asura_Maths::Sqrt( 1.0f - expf( -fIntensity) ) / fIntensity;

	xSH.Scale( fScale );
}

static void BlendNegativeAndPositiveSHs( float fAmbient, Asura_SphericalHarmonics_Order3& xPositive, Asura_SphericalHarmonics_Order3& xNegative )
{
	// scale factor for getting negative lights to counteract ambient light appropriately
	// (picked entirely through trial and error...)
	const float fNegScale = 2.50f;
	
	// scale up negative light for comparing against ambient
	xNegative.Scale( fNegScale );

	if( fAmbient >= xNegative[0] )
	{
		// if ambient is higher than negative light, just add the difference
		xPositive[0] += fAmbient - xNegative[0];
	}
	else
	{
		// more negative light than ambient , so the remainder will have to counteract the
		// directional light.
		float fNegAmbient = (fAmbient - xNegative[0]) / fNegScale;
		
		if( xPositive[0] > -fNegAmbient )
		{
			// apply the negative light evenly , by scaling down all the components
			xPositive.Scale( (xPositive[0] + fNegAmbient) / xPositive[0] );
		}
		else
		{
			// negative light is greater than the directional positive light , so just zero it
			xPositive.Zero();
		}
	}
}

			
bool Tools_PhotonMapping_System::GetSHsAtVertex(u_int uModule, u_int uVertex, Asura_SphericalHarmonics_Order2_RGB& xResult )
{
	Asura_SphericalHarmonics_Order3 xR;
	Asura_SphericalHarmonics_Order3 xG;
	Asura_SphericalHarmonics_Order3 xB; 
	
	// get the contribution from all the parallel lights
	Asura_SphericalHarmonics_Order3 xEnvR = Tools_PhotonMapping_System::GetEnvironmentIrradianceRed();
	Asura_SphericalHarmonics_Order3 xEnvG = Tools_PhotonMapping_System::GetEnvironmentIrradianceGreen();
	Asura_SphericalHarmonics_Order3 xEnvB = Tools_PhotonMapping_System::GetEnvironmentIrradianceBlue();

	// scale away the rgb 0->255 to 0->1
	xEnvR.Scale( 1.0f/255.0f );
	xEnvG.Scale( 1.0f/255.0f );
	xEnvB.Scale( 1.0f/255.0f );

	// we only want to be lit by parallel lights if the rays can reach us, so multiply by the transport harmonic for this vertex
	Asura_SphericalHarmonics_Order3 xTransportSH;
	if (Tools_EnvironmentalLightTransport::GetTransportHarmonic(  xTransportSH, uModule, uVertex ) )
	{
		xEnvR *= xTransportSH;
		xEnvG *= xTransportSH;
		xEnvB *= xTransportSH;
	}

	// Positive lighting values start with this parallel lights contribution

	// KJFL - 15 June 2006 - previously we just used the xEnvR G & B SHs as is, but there were a couple of bugs
	// that meant this had to change. 
	//
	// 1) Parallel lights were coming out too dark on entities (but global ambience and photon contributions were fine)
	//
	// 2) Coloured parallel lights in opposite directions would 'fight', e.g. a white light on one side and a red light 
	// on the other would result in the white side being tinted blue-green - effectively anti-red.
	//
	// The first bug seemed to be down to the parallel lighting contributions being scaled for irradiance (via the
	// ScaleCoefficientsForIrradiance() method) twice - once when they were generated and again after all the light
	// contributions had been summed. The second problem was simply due to the loss of accuracy when going down from
	// order 3 to order 2 harmonics - it also happens with photon lights, but to a much lesser degree.
	//
	// My solution was to resample the parallel lights into new harmonics, which reduces the fighting effects, 
	// and effectively passes a low frequency filter over the data again. I tried clamping the samples in each direction
	// to be positive, but this worked too well - the fighting went, but there was little contrast on objects lit with 
	// the lights.
	//
	// Removing the extra ScaleCoefficientsForIrradiance() fixed the intensity issues, but dramatically 
	// reduced the contrast on objects lit with infinite lights - so I've used a fiddle-factor of 4.0f which lifts
	// the whole brightness of the parallel lights back to match the effect on the environment. (This effectively means
	// the zeroth order term is scaled once, and the first order terms are scaled twice... hey, it works.)

	xR.Zero();
	xG.Zero();
	xB.Zero();

	// KJFL - 15 June 2006 - Set up a fake lightsource which we can use to generate random sample directions on a sphere. 
	// We can seem to get away with '16' which is 256 samples in total. (Too many and the algorithm is slow, too few
	// and it's not accurate enough.
	const u_int uNumSamples = 16;

	Tools_PhotonMapping_LightSource xLightSource;

	xLightSource.SetStratificationLevels( uNumSamples );
	for (u_int u=0; u<uNumSamples; u++)
	{
		for (u_int v=0; v<uNumSamples; v++)
		{
			Asura_Vector_3 xDirection;
			Asura_Vector_3 xPosition;

			xLightSource.GetStratifiedPhotonPositionAndDirection( xPosition, xDirection, u, v ); 

			// KJFL - 15 June 2006 - Get the sample for this direction, and project it to the new harmonic. 
			// I also tried clamping these samples to be positive, but that completely destroys the contrast
			// on infinite lights.

			float fLightR = xEnvR.EvaluateInDirection(xDirection);
			float fLightG = xEnvG.EvaluateInDirection(xDirection);
			float fLightB = xEnvB.EvaluateInDirection(xDirection);

			xR.AddSampleForDirection( fLightR, xDirection );
			xG.AddSampleForDirection( fLightG, xDirection );
			xB.AddSampleForDirection( fLightB, xDirection );
		}
	}

	// fiddle-factor of 4.0f to rescale the parallel lights to match the effect on the environment - see big comment above
	const float fScaleFactor = 4.0f / (uNumSamples * uNumSamples);
	xR.Scale( fScaleFactor );
	xG.Scale( fScaleFactor );
	xB.Scale( fScaleFactor );

	// We'll add up lighting components from negative lights separately, and munge them in at the end. They act more
	// as filters rather than true negative values.
	Asura_SphericalHarmonics_Order3 xNegR, xNegG, xNegB;
	xNegR.Zero();
	xNegG.Zero();
	xNegB.Zero();

	// scan through all the photon lights, and add their contributions
	Asura_Hashed_List_It<Tools_PhotonMapping_BaseLightData> xIterator(s_pxCombinedLightList);

	for (xIterator.Restart(); !xIterator.Done(); xIterator.Next())
	{
    	Tools_PhotonMapping_BaseLightData* pxLightData = xIterator.GetCurrent();

		const Tools_PhotonMapping_LightSource* pxLightSource = pxLightData->GetLightSource();

		// skip parallel lights, we've handle those already
		if (pxLightSource->IsParallel()) continue;
		
		// grab the SH sample for this vertex
		Tools_PhotonMap_DirectEstimation& xPhotonMap = pxLightData->m_xDirectEstimationPhotonMap;

		const Tools_PhotonMap_DirectEstimation_Sample* pxSample = xPhotonMap.GetSample( uVertex, uModule );
		
		if (pxSample)
		{
			Asura_SphericalHarmonics_Order3 xIrradiance = pxSample->m_xIrradiance;
		
			// check the sign of the light's power
			float fPowerSign = Asura_Maths::Sign(pxLightSource->GetPower());
					
			// KJFL - 21 October 2005 - 
			// Scale irradiance to the power level. The factor of 10.0f came in to renormalise lights
			// so that the value shown in the Tools was more artist friendly i.e. they could deal more
			// with units rather than fractional values.
			xIrradiance.Scale( Asura_Maths::Abs( pxLightSource->GetPower()) / 10.0f );

			// So, each R, G & B component shares the same irradiance intensity, but modulated
			// by the light's colour
			Asura_SphericalHarmonics_Order3 xLightR = xIrradiance;
			Asura_SphericalHarmonics_Order3 xLightG = xIrradiance;
			Asura_SphericalHarmonics_Order3 xLightB = xIrradiance;

			// if the light is "bleached", it saturates out to white - this means we factor in the
			// colour values early.
			if (pxLightSource->IsBleached())
			{
				xLightR.Scale( pxLightSource->GetColour().x );
				xLightG.Scale( pxLightSource->GetColour().y );
				xLightB.Scale( pxLightSource->GetColour().z );
			}
			else // otherwise treat the light as bright white for now
			{
				xLightR.Scale( 255.0f );
				xLightG.Scale( 255.0f );
				xLightB.Scale( 255.0f );
			}

			// rescale the SHs with our exposure function to take them from HDR
			RescaleSH( xLightR );
			RescaleSH( xLightG );
			RescaleSH( xLightB );

			// Now we've rescaled to display resolution, unbleached lights need to be scaled
			// by their colour components
			if (!pxLightSource->IsBleached())
			{
				xLightR.Scale( pxLightSource->GetColour().x  / 255.0f);
				xLightG.Scale( pxLightSource->GetColour().y  / 255.0f);
				xLightB.Scale( pxLightSource->GetColour().z  / 255.0f);
			}
			
			// depending on the power sign, add to the positive or negative SH totals
			if(fPowerSign<0)
			{
				xNegR += xLightR;
				xNegG += xLightG;
				xNegB += xLightB;
			}
			else
			{
				xR += xLightR;
				xG += xLightG;
				xB += xLightB;
			}
		}
		// else we didn't have a valid sample... if data is invalid
		// we'll just set to a white light and return
		else if( !xPhotonMap.IsVertexAndModuleIndexInRange( uVertex, uModule ) )
		{
			Asura_Assertf( false, ("Calculating the Photon Mapping has found a problem.\nModule %d Vertex %d does not seem to be valid.\nTry Recalculating all lighting.", uModule, uVertex));
			xR.Zero();
			xG.Zero();
			xB.Zero();
			
			xR[0] = 0.8f;
			xG[0] = 0.8f;
			xB[0] = 0.8f;
			
			// DLH - 31 August 2006 16:04:00 - I thought this might help stop random data being returned to the calling routine.
			xResult.Set( xR, xG, xB );
			
			return false;
		}
	
	}

	// scale both positive and negative total SHs ready for irradiance calcs
	xR.ScaleCoefficientsForIrradiance();
	xG.ScaleCoefficientsForIrradiance();
	xB.ScaleCoefficientsForIrradiance();

	xNegR.ScaleCoefficientsForIrradiance();
	xNegG.ScaleCoefficientsForIrradiance();
	xNegB.ScaleCoefficientsForIrradiance();

	//Determine polygon flags used with vertex RAR 21/02/2006
	const Tools_Polygon_Adjacency* pxVertAdj = Tools_Adjacency::GetVertexAdjacencies(uModule,uVertex);
	u_int uPolyFlags = 0;
	const Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(uModule);
	for(const Tools_Adjacent_Polygon* pxAdj = pxVertAdj->GetFirstAdjacency(); pxAdj; pxAdj = pxAdj->GetNext())
	{
		if(pxAdj->IsBetweenModules()) continue;
        	
		const u_int uPolygonIndex = pxAdj->GetPolygonIndex();
        uPolyFlags |= pxMesh->GetPolygonFlags(uPolygonIndex);

	}
	

	// Blend the positive SH, the negative SH and the generic ambient light.
	// The ambient light ignores all transport SHs. Use sparingly or things will look washed-out!
	if(uPolyFlags & AsuraPolygonFlag_IgnoreAmbientLight)
	{
		BlendNegativeAndPositiveSHs( 0.0f, xR, xNegR );
		BlendNegativeAndPositiveSHs( 0.0f, xG, xNegG );
		BlendNegativeAndPositiveSHs( 0.0f, xB, xNegB );
	}
	else
	{
		const Asura_RGBA& xGlobalAmbience = GetGlobalAmbience();

		BlendNegativeAndPositiveSHs( xGlobalAmbience.m_fR, xR, xNegR );
		BlendNegativeAndPositiveSHs( xGlobalAmbience.m_fG, xG, xNegG );
		BlendNegativeAndPositiveSHs( xGlobalAmbience.m_fB, xB, xNegB );
	}

	// JGP - Determine per vert per poly painted colour of vertex
	float fTotalRed = 0.0f;
	float fTotalGreen = 0.0f;
	float fTotalBlue = 0.0f;
	int iPaintedSamples = 0;

	for(const Tools_Adjacent_Polygon* pxAdj = pxVertAdj->GetFirstAdjacency(); pxAdj; pxAdj = pxAdj->GetNext())
	{
		if( pxAdj->IsBetweenModules( ) ) 
		{
			continue;
		}
        	
		const u_int uPolygonIndex = pxAdj->GetPolygonIndex();

		for ( int iFaceVert = 0; iFaceVert < pxMesh->PolygonDataPtr[ uPolygonIndex ].NumberOfVertices(); iFaceVert++ )
		{
			if ( pxMesh->PolygonDataPtr[ uPolygonIndex ].VertexID[ iFaceVert ] == uVertex )
			{
				const u_int uPaintedColour = pxMesh->m_pxPaintedColourDataPtr[ uPolygonIndex ].Colour[ iFaceVert ];
				const float fRed = static_cast<float>( ( uPaintedColour >> 16 ) & 0xff ) / 128.0f;
				const float fGreen = static_cast<float>( ( uPaintedColour >> 8 ) & 0xff ) / 128.0f;
				const float fBlue = static_cast<float>( ( uPaintedColour >> 0 ) & 0xff ) / 128.0f;
				fTotalRed += fRed;
				fTotalGreen += fGreen;
				fTotalBlue += fBlue;
				iPaintedSamples++;
			}
		}
	}

	if ( iPaintedSamples > 0 )
	{
		fTotalRed /= iPaintedSamples;
		fTotalGreen /= iPaintedSamples;
		fTotalBlue /= iPaintedSamples;
		xR.Scale( fTotalRed );
		xG.Scale( fTotalGreen );
		xB.Scale( fTotalBlue );
	}

	//Scale down if darken flag is set. RAR 07/03/2006
	if(uPolyFlags & AsuraPolygonFlag_Darken)
	{
		xR.Scale(ASURA_POLYFLAG_DARKEN_SCALE);
		xG.Scale(ASURA_POLYFLAG_DARKEN_SCALE);
		xB.Scale(ASURA_POLYFLAG_DARKEN_SCALE);
	}

	xResult.Set( xR, xG, xB );
	xResult.ScaleCoefficients( s_fSHZerothOrderResampleScale, s_fSHFirstOrderResampleScale );

	return true;
}


void Tools_PhotonMapping_System::SyncPolygonLightsWithMeshes()
{
	//first check each light to make sure they should still exist
	Asura_TrackingHandlePool<Tools_PhotonMapping_PolygonLightData>::Iterator xIterator = s_pxPolygonLightDataList->GetIterator();
	

	for (xIterator.Begin(); !xIterator.End(); )
	{
		Tools_PhotonMapping_PolygonLightData* pxLightData = xIterator.GetCurrentPointer();
		Asura_Handle iHandle = xIterator.GetCurrentHandle();
		xIterator.Next();//advance the iterator now ,to avoid problems if we delete the light

		if(!pxLightData->m_xLightSource.SyncWithMesh())
		{
			//this light needs deleting
			DestroyPolygonLightSource(iHandle);
		}
	}

	//now check through all the modules for any missing lights
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);

		Asura_Mesh_PolygonLight* pxMeshLightData = pxMesh->m_pxPolygonLights;
		for(int j=0;j<pxMesh->m_iNumberOfPolygonLights;j++,pxMeshLightData++)
		{
			//any light exist for this?
			bool bFound = false;
			for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
			{
				Tools_PhotonMapping_PolygonLightData* pxLightData = xIterator.GetCurrentPointer();
				if(pxLightData->m_xLightSource.GetModuleIndex() == i &&
					pxLightData->m_xLightSource.GetLightID() == pxMeshLightData->m_uLightID)
				{
					bFound = true;
					break;
				}
			}
			if(bFound) continue;

			//okay , we need to make a new light then
			Asura_Handle iHandle = Tools_PhotonMapping_System::CreatePolygonLightSource();
			if(iHandle == ASURA_INVALID_HANDLE) continue;

			Tools_PhotonMapping_PolygonLightSource* pxLightSource = Tools_PhotonMapping_System::GetPolygonLightSource(iHandle);
			pxLightSource->InitialiseFromMeshData(i,pxMeshLightData);
		}
	}
}

void Tools_PhotonMapping_System::DeleteModule(int iModuleIndex)
{
	//get index of module that will be moved down to replace this one (it will be the last one)
	int iMovedModuleIndex = Asura_Module_System::GetNumberOfModules()-1;

	Asura_TrackingHandlePool<Tools_PhotonMapping_PolygonLightData>::Iterator xIterator = s_pxPolygonLightDataList->GetIterator();
	
	for (xIterator.Begin(); !xIterator.End(); )
	{
		Tools_PhotonMapping_PolygonLightData* pxLightData = xIterator.GetCurrentPointer();
		Asura_Handle iHandle = xIterator.GetCurrentHandle();
		xIterator.Next();//advance the iterator now ,to avoid problems if we delete the light

		if(pxLightData->m_xLightSource.GetModuleIndex() == iModuleIndex)
		{
			//this light uses the module that has been deleted
			DestroyPolygonLightSource(iHandle);
		}
		else if(pxLightData->m_xLightSource.GetModuleIndex() == iMovedModuleIndex)
		{
			//this light uses the module that is having its index changed
			pxLightData->m_xLightSource.SetModuleIndex(iModuleIndex);
		}
	}
}


const Asura_RGBA& Tools_PhotonMapping_System::GetGlobalAmbience()
{
	return GetGlobalAmbience(Tools_LightSets::GetCurrentLightSets());
}

void Tools_PhotonMapping_System::SetGlobalAmbience(const Asura_RGBA& xColour)
{
	SetGlobalAmbience(Tools_LightSets::GetCurrentLightSets(),xColour);
}

const Asura_RGBA& Tools_PhotonMapping_System::GetGlobalAmbience(u_int uSets)
{
	Asura_RGBA* pxAmb = s_xGlobalAmbienceTree.GetDataByHash(uSets);
	if(!pxAmb)
	{
		//no entry for this value yet ,so add a default value and return that
		pxAmb = s_xGlobalAmbienceTree.AddNode(uSets, Asura_RGBA(0,0,0,0));
		Asura_Assert(pxAmb,"Failed to add node");
	}
	return *pxAmb;	
}

void Tools_PhotonMapping_System::SetGlobalAmbience(u_int uSets,const Asura_RGBA& xColour)
{
	Asura_RGBA* pxAmb = s_xGlobalAmbienceTree.GetDataByHash(uSets);
	if(pxAmb)
	{
		*pxAmb = xColour;
	}
	else
	{
		//no entry for this value yet ,so add as a new node
		pxAmb = s_xGlobalAmbienceTree.AddNode(uSets, xColour);
		Asura_Assert(pxAmb,"Failed to add node");
	}
}

bool Tools_PhotonMapping_System::HasGlobalAmbience(u_int uSets)
{
	Asura_RGBA* pxAmb = s_xGlobalAmbienceTree.GetDataByHash(uSets);
	return pxAmb!=0;
}


bool Tools_PhotonMapping_System::AreThereAnyParallelLights()
{
	Asura_Hashed_List_It<Tools_PhotonMapping_BaseLightData> xIterator(s_pxCombinedLightList);
	for (xIterator.Restart(); !xIterator.Done(); xIterator.Next())
	{
    	Tools_PhotonMapping_BaseLightData* pxLightData = xIterator.GetCurrent();

		if ( pxLightData->GetLightSource()->IsParallel() )
		{
			return true;
		}
	}

	return false;
}