//
// Tools_PhotonMapping_System.h
//
// Created by KJL - 03/12/2003 16:17:17
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_PHOTONMAPPING_SYSTEM_H__
#define __TOOLS_PHOTONMAPPING_SYSTEM_H__

#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_Mempool.h"
#include "Core/Asura_HashedList.h"

#include "Rendering/Asura_Colour.h"


class Tools_PhotonMap;
class Tools_PhotonMapping_LightSource;
class Tools_PhotonMapping_PolygonLightSource;
class Tools_PhotonMapping_RegionSample;
class Tools_PhotonMapping_BaseLightData;
class Tools_PhotonMapping_LightData;
class Tools_PhotonMapping_PolygonLightData;
class Tools_PhotonMapping_TemplateData;

//A default colour to fill in the prelighting data for those occasions where caculating ir properly
//isn't feasible. RAR 03/09/2004
#define TOOLS_DEFAULT_PRELIGHTINGVALUE		0x00808080

static const u_int TOOLS_PHOTONMAPPING_MAX_ACCURACY = 9;

//return valuesfor CheckPhotonMapsAreValud
enum Tools_PhotonMap_Validity
{
	TOOLS_PHOTONMAP_INVALID,//at least one light has an invalid photon map
	TOOLS_PHOTONMAP_REDUCEDACCURACY,//all photon maps valid , but at least one is done at lower accuracy than intended
	TOOLS_PHOTONMAP_VALID,//all photon maps valid , and calculated at the specified (or higher) accuracies
};

template<class T> class List;

class Tools_PhotonMapping_System
{
	friend class Tools_PhotonMapping_RegionSample;

	public:
		static void Initialise(void);
		static void Shutdown(void);
		static void Reset(void);

		static void SamplePhotonMap(const Asura_Vector_3& xPosition, const Asura_Vector_3& xNormal, Asura_RGBA& xColour);
		static bool GetDirectEstimatePhotonMapSample( const u_int uPoly, const u_int uVertex, const u_int uModule, const Asura_Vector_3& xNormal, Asura_RGBA& xColour );

		static void RenderPhotonMapRepresentation(void);

		static void RecalculateEverything(u_int uMinAccuracy = 1,u_int uMaxAccuracy = TOOLS_PHOTONMAPPING_MAX_ACCURACY);

		//! returns true if there were some invalid lights
		static bool RecalculateAllInvalidLights(u_int uMinAccuracy = 1,u_int uMaxAccuracy = TOOLS_PHOTONMAPPING_MAX_ACCURACY);

		static void RecalculatePhotonMapForLightSource(const Asura_Handle iHandle);
		static void RecalculatePhotonMapForPolygonLightSource(const Asura_Handle iHandle);

		static void RecalculatePhotonMapForLights(const List<Asura_Handle>& xLightList,const List<Asura_Handle>& xPolygonLightList);

		static void RecalculatePhotonMapForLightSourceMain(u_int uLightNumber, u_int uTotalNumberOfLights, Tools_PhotonMapping_BaseLightData* pxLightData,u_int uAccuracyOveride = 0);
	
		#if defined (ASURA_USE_TASK_SYSTEM )
		static void RecalculatePhotonMapForLightSourceMain_MT(u_int uLightNumber, u_int uTotalNumberOfLights, Tools_PhotonMapping_BaseLightData* pxLightData,u_int uAccuracyOveride = 0);
		static void PhotonMappingMT( const void* pData );
		#endif

																
		static Asura_Handle CreateLightSource(void);
		static void DestroyLightSource(const Asura_Handle iHandle);
		
		static Asura_Handle CreatePolygonLightSource(void);
		static void DestroyPolygonLightSource(const Asura_Handle iHandle);


		static Tools_PhotonMapping_LightSource* GetLightSource(const Asura_Handle iHandle);
		static Tools_PhotonMapping_TemplateData* GetTemplateData(const Asura_Handle iHandle);

		static Tools_PhotonMapping_PolygonLightSource* GetPolygonLightSource(const Asura_Handle iHandle);
		static Tools_PhotonMapping_PolygonLightSource* GetPolygonLightSourceFromID(int iModule,u_int uLightID);

		static Asura_Handle GetLightHandleFromID(u_int uID);

		

		static u_int GetNumberOfLights( void ) { return s_uNumberOfLightSources; };
		static u_int GetNumberOfPolygonLights( void ) { return s_uNumberOfPolygonLightSources; };

		static enum Tools_PhotonMap_Validity CheckPhotonMapsAreValid( void );

		//Creates/Deletes polygon lights so that they agree with the lighting data
		//in the module meshes
		static void SyncPolygonLightsWithMeshes(void);

// KJFL - 2004-06-09 14:49:20 - deprecated older interface functions
// don't want to delete in case they are needed again at some point
//
//		static void SetNumberOfBounces(u_int uBounces)
//		{
//			s_uMaximumNumberOfBounces = uBounces;
//		}
//
//		static u_int GetNumberOfBounces(void) 
//		{
//			return s_uMaximumNumberOfBounces;
//		}
//
//		static void SetParallelBounceScale(float fScale)
//		{
//			s_fParallelBounceScale = fScale;
//		}
//
//		static float GetParallelBounceScale(void)
//		{
//			return s_fParallelBounceScale;
//		}
//
//		static void SetNumberOfPhotonCast(u_int uNumberOfPhotons)
//		{
//			s_uNumberOfRayCastsPerLight = uNumberOfPhotons;
//		}
//
//		static u_int GetNumberOfPhotonCasts(void) 
//		{
//			return s_uNumberOfRayCastsPerLight;
//		}
//
//		static void SetSampleRadius(float fSampleRadius)
//		{
//			s_fSampleRadiusSquared = fSampleRadius*fSampleRadius;
//		}
//
//		static float GetSampleRadius(void) 
//		{
//			return Asura_Maths::Sqrt(s_fSampleRadiusSquared);
//		}
		
		static void CalculateEnvironmentIrradiance( void );

		static void InvalidateEnvironmentIrradiance(){s_bEnvironmentIrradianceValid = false;}

		static void CalculateSHForLightSource( Tools_PhotonMapping_LightData* pxLightData );

		static const Asura_SphericalHarmonics_Order3& GetEnvironmentIrradianceRed( void )
		{
			if(!s_bEnvironmentIrradianceValid) CalculateEnvironmentIrradiance();
			return s_xEnvironmentIrradianceRed;
		}
		static const Asura_SphericalHarmonics_Order3& GetEnvironmentIrradianceGreen( void )
		{
			if(!s_bEnvironmentIrradianceValid) CalculateEnvironmentIrradiance();
			return s_xEnvironmentIrradianceGreen;
		}
		static const Asura_SphericalHarmonics_Order3& GetEnvironmentIrradianceBlue( void )
		{
			if(!s_bEnvironmentIrradianceValid) CalculateEnvironmentIrradiance();
			return s_xEnvironmentIrradianceBlue;
		}
		static bool IsEnvironmentIrradianceZero()
		{
			if(!s_bEnvironmentIrradianceValid) CalculateEnvironmentIrradiance();
			return s_xEnvironmentIrradianceRed.IsZero() && s_xEnvironmentIrradianceGreen.IsZero() && s_xEnvironmentIrradianceBlue.IsZero();
			
		}
		

	
		static bool GetHighDynamicRangeSHsAtVertex(u_int uModule, u_int uVertex, Asura_SphericalHarmonics_Order3& xR, Asura_SphericalHarmonics_Order3& xG, Asura_SphericalHarmonics_Order3& xB );


		static bool GetSHsAtVertex(u_int uModule, u_int uVertex, Asura_SphericalHarmonics_Order2_RGB& xResult );


		// KJFL - 2004-06-09 15:18:38 - may be able to return a const pointer here... would probably
		// need a const iterator for the Asura_TrackingHandlePool though. Some callers need to be able
		// to change data in the pool etc.
		static Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>* GetLightDataTrackingHandlePool( void )
		{
			return s_pxLightDataList;
		}
		
		static Asura_TrackingHandlePool<Tools_PhotonMapping_PolygonLightData>* GetPolygonLightDataTrackingHandlePool( void )
		{
			return s_pxPolygonLightDataList;
		}

		static Asura_Hashed_List<Tools_PhotonMapping_BaseLightData>* GetCombinedLightList()
		{
			return s_pxCombinedLightList;
		}
		

		static void CheckValidityOfGlobalIllumination(void);

		static bool UpdateProgress(int i, int j, const char* szHeading);

		//Handles changes to polygon lights required as a result of a module being deleted
		static void DeleteModule(int iModuleIndex);

		//Global ambient values are in the range 0 to 0.5f
		//Global ambience for current light set
		static const Asura_RGBA& GetGlobalAmbience();
		static void SetGlobalAmbience(const Asura_RGBA& xColour);

		//Global ambience for specific light set
		static const Asura_RGBA& GetGlobalAmbience(u_int uSets);
		static void SetGlobalAmbience(u_int uSets,const Asura_RGBA& xColour);
		static bool HasGlobalAmbience(u_int uSets);

		static bool AreThereAnyParallelLights();

		// User tweakable rescale factors to improve spherical harmonic downsampling
		static void SetSHZerothOrderResampleScale( float fScale ) { s_fSHZerothOrderResampleScale = fScale; }
		static float GetSHZerothOrderResampleScale( void ) { return s_fSHZerothOrderResampleScale; }

		static void SetSHFirstOrderResampleScale( float fScale ) { s_fSHFirstOrderResampleScale = fScale; }
		static float GetSHFirstOrderResampleScale( void ) { return s_fSHFirstOrderResampleScale; }

	private:


		static u_int GetNumberOfSamplesForAccuracyLevel( u_int uAccuracy );
		static float GetSampleRadiusForAccuracyLevel( u_int uAccuracy , bool bShortRange );

		static void CalculateLightingGroups(void);



		static bool									s_bBeenInitialised;
		static u_int								s_uNumberOfLightSources;																				
		static u_int								s_uNumberOfPolygonLightSources;																				
													
		static u_int								s_uMaximumNumberOfBounces;
		static u_int								s_uNumberOfRayCastsPerLight;
		static float								s_fSampleRadiusSquared;
		static float								s_fParallelBounceScale;
																				
																				
		static Tools_PhotonMapping_RegionSample*	s_pxSample;
																			
		static bool									s_bPreviousSampleRadiusValid;

		static Asura_SphericalHarmonics_Order3		s_xEnvironmentIrradianceRed;
		static Asura_SphericalHarmonics_Order3		s_xEnvironmentIrradianceGreen;
		static Asura_SphericalHarmonics_Order3		s_xEnvironmentIrradianceBlue;
		static bool									s_bEnvironmentIrradianceValid;

		static Asura_TemplateHashTree<Asura_RGBA>	s_xGlobalAmbienceTree;

		// User tweakable rescale factors to improve spherical harmonic downsampling
		static float								s_fSHZerothOrderResampleScale;
		static float								s_fSHFirstOrderResampleScale;

		static Asura_TrackingHandlePool<Tools_PhotonMapping_LightData>* s_pxLightDataList;
		static Asura_TrackingHandlePool<Tools_PhotonMapping_PolygonLightData>* s_pxPolygonLightDataList;

		static Asura_Hashed_List<Tools_PhotonMapping_BaseLightData>* s_pxCombinedLightList;

		static bool s_bPreserveScreenDuringUpdateProgress;



};																		

#endif//__TOOLS_PHOTONMAPPING_SYSTEM_H__
