/**
*
* \file Tools_Level_Export.cpp
*
* Created by Kevin Lea - 29/11/2002 11:23:31
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/
#include "Core/Asura.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"

// Asura Platform-independent
#include "ChunkSystem/Asura_Huffman.h"
#include "ChunkSystem/Asura_Chunk_Blueprints.h"
#include "ChunkSystem/Asura_Chunk_Decals.h"
#include "ChunkSystem/Asura_Chunk_Material.h"
#include "ChunkSystem/Asura_Chunk_ResourceFileList.h"
#include "ChunkSystem/Asura_Chunk_Texture.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_LocalisedText.h"
#include "Core/Asura_SortAndSearch.h"
#include "Core/Asura_StringBuffer.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_Actor.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Light/Asura_Light.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_PatchSubdivision.h"
#include "Rendering/Asura_Viewport.h"
#include "Mesh/Asura_Env_System.h"
#include "SpecialFX/Asura_Fragment_System.h"

// Asura PC
#include "Rendering/Asura_Direct3D.h"
#include "Rendering/Asura_Direct3D_MRPShaderCompile.h"

// Tools
#include "Blueprint/Tools_Mode_Blueprint.h"
#include "Blueprint/Tools_Blueprint_Edit_Object.h"
#include "Entity/Tools_Entity_Object.h"
#include "Entity/Tools_Message_Editing.h"
#include "ImageProcessing/Tools_Tim2_Writer.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Load&Save/Tools_LoadedFileList.h"
#include "Load&Save/Tools_BatchProcessing.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_TextureConversionData.h"
#include "Load&Save/Tools_ResourceExportLog.h"
#include "Mesh/Tools_PlatformMesh_System.h"
#include "Misc/Tools_EnumerationReader.h"
#include "Misc/Tools_IniFile.h"
#include "MeshProcessing/Tools_SubdividedMesh.h"
#include "PC/OS/Tools_OS.h"
#include "PC/Tools_PC_Export.h"
#include "PSP/Tools_PSP_Export.h"
#include "PS2/PS2Strip/PS2Strip_ToolsInterface.h"
#include "PS3/Tools_PS3_Export.h"
#include "PS3/Tools_PS3_ShaderExport.h"
#include "Shapes/Tools_Shapes.h"
#include "Skybox/Tools_Mode_Skybox.h"
#include "Wii/Tools_Wii_Export.h"
#include "Wii/Tools_Wii_Textures.h"
#include "Xbox360/Tools_Xbox360_Export.h"

#ifdef ASURA_USE_WATER
#include "ChunkSystem/Asura_Chunk_Water.h"
#include "Rendering/Asura_Water.h"
#endif

#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
#include "ChunkSystem/Asura_Chunk_MaterialResponseOverrides.h"
#endif	// defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)

#include "ChunkSystem/Asura_Chunk_PFXOverrides.h"
#include "ChunkSystem/Asura_Chunk_FSFXOverrides.h"

#ifdef ASURA_USE_ANIMATION_IK_SOLVER
#include "Animation/IK/Tools_Animation_IK_Template_System.h"
#endif

#ifdef ASURA_WRITE_ZLIB
#include "Load&Save/Tools_Zlib.h"
#endif

#include "Particles/Tools_Mode_PFX.h"

#ifdef ASURA_USE_BOTANICALS
#include "Botanicals/Asura_Botanicals_InstanceManager.h"
#include "Botanicals/Asura_Botanicals_TemplateManager.h"
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"
#include "Botanicals/Tools_Mode_Botanicals.h"
#endif // #ifdef ASURA_USE_BOTANICALS

#ifdef ASURA_USE_FSFX
#include "FSFX/Tools_Mode_FSFX.h"
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"
#include "FSFX/Asura_FSFX_Component.h"
#endif //ASURA_USE_FSFX

#ifdef ASURA_USE_CARPET
#include "Carpet/Asura_Carpet_MeshManager.h"
#include "Carpet/Asura_Carpet_StyleManager.h"
#include "Carpet/Asura_Direct3D_Carpet_Game_Render.h"
#include "Carpet/Tools_Carpet_Mesh.h"
#endif //ASURA_USE_CARPET

#ifdef ASURA_USE_ATTRACTORS
#include "ChunkSystem/Asura_Chunk_Attractors.h"
#include "Attractors/Tools_Attractor_System.h"
#endif

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM 
#include "Core/Asura_MaterialResponse_System.h"
#include "ChunkSystem/Asura_Chunk_MaterialResponse.h"
#include "Materials/Tools_MaterialResponse_System.h"
#endif

#if ASURA_USE_GUI_MENU_SYSTEM
//#include "GUIMenu/Asura_GUIMenu_Message.h"
//#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Tools_GUIMenu_System.h"	
#endif

#ifdef ASURA_USE_DECAL_MANAGER
#include "SpecialFX/Asura_Decal_Manager.h"
#include "ChunkSystem/Asura_Chunk_Decals.h"
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
#include "Decals/Tools_StaticDecal_System.h"
#endif

#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "ChunkSystem/Asura_Chunk_Container.h"
#include "Entity/Container/Asura_Container_System.h"
#endif //ASURA_USE_CONTAINER_SYSTEM


//Temporary define to turn off the separate mission/environment export until the game code is ready for it
//RAR 18/09/2007
//#define TOOLS_DISABLE_SEPARATE_MISSION_EXPORT 1

// G L O B A L S ////////////////////////////////////////
//  AAB:  (14/11/2003 14:26:29):  Removed static, so these variables can be used in derived
// classes of Tools_Level_Export
u_int g_uAsuraFilesInExport = 0;
ResourceFileListEntry* g_axResourceFileListEntries = 0;
unsigned g_uBytesReservedForFileListChunk = 0;
unsigned g_uFileListChunkStartFilePtr = 0;
unsigned g_uBytesReservedForFileInfoChunk = 0;
unsigned g_uFileInfoChunkStartFilePtr = 0;
bool g_bPreparedHashMapFile = false;
u_int g_auMaxTIM2Dimensions[ASURA_MAX_NO_OF_TEXTURES];


static char g_szLevelExportTemp[50];
static HANDLE g_iLevelExportTempMutex = 0;
// For Yes/No/All overlay when invalid lightng is found during an export
bool g_bAskAboutInvalidLighting = true;

Asura_HashTree g_xMissingMaterialsEncountered;

Tools_Level_Export* Tools_Level_Export::This = 0;

char* Tools_Level_Export::s_szLogPath = 0;
char* Tools_Level_Export::s_szErrorLogPath = 0;
char* Tools_Level_Export::s_szSoundLogPath = 0;
char* Tools_Level_Export::s_szBlueprintLogPath = 0;
bool Tools_Level_Export::s_bAbort = false;
bool Tools_Level_Export::s_bUserAbort = false;
int Tools_Level_Export::s_iPercent = 0;
Asura_String Tools_Level_Export::s_xHeading;
const char* Tools_Level_Export::s_szMainHeading = 0;
int Tools_Level_Export::s_iNumErrors;
Tools_Level_Export::EXPORT_STAGE Tools_Level_Export::s_eThisStage = Tools_Level_Export::EXPORT_STAGE_UNDEFINED;
Tools_Level_Export::EXPORT_STAGE Tools_Level_Export::s_eCurrentStage = Tools_Level_Export::EXPORT_STAGE_UNDEFINED;
Tools_Level_Export::EXPORT_TYPE Tools_Level_Export::s_eExportType;
Asura_String Tools_Level_Export::s_xMessageDisplayHeading;
bool Tools_Level_Export::s_bSubdivideMeshes = false;
bool Tools_Level_Export::s_bRecalculatePrelighting = false;
bool Tools_Level_Export::s_bSwizzleTims = true;
float Tools_Level_Export::s_fFlatSubdivideSize = 0.5f;
float Tools_Level_Export::s_fCurveSubdivideSize = 0.5f;
bool Tools_Level_Export::s_bDontExportCutscenes = false;
bool Tools_Level_Export::s_bPS2MultiplayerExport = false;
bool Tools_Level_Export::s_bIncludeDualTexturingData = true;
bool Tools_Level_Export::s_bFullBrightPrelighting = false;
bool Tools_Level_Export::s_bOverrideFrameRemovalSettings = false;
float Tools_Level_Export::s_fAnimPositionTolerance = TOOLS_DEFAULT_ANIM_POSITION_TOLERANCE;
float Tools_Level_Export::s_fAnimAngleTolerance = TOOLS_DEFAULT_ANIM_ANGLE_TOLERANCE;
float Tools_Level_Export::s_fMinGCTexRes = 1024;
float Tools_Level_Export::s_fMaxMipDrops = 2;
u_int Tools_Level_Export::s_uPlatformsToExport = 0;
u_int Tools_Level_Export::s_uCurrentLightSetsToExport = 0;
u_int Tools_Level_Export::s_uLightSetsToExport = 1;
u_int Tools_Level_Export::s_uInitialLightSets = 0;
bool Tools_Level_Export::s_bExportAllCharacters = false;
bool Tools_Level_Export::s_bIsLevelExport = false;
bool Tools_Level_Export::s_bIsExport = false;
bool Tools_Level_Export::s_bIsSkyboxExport = false;
bool Tools_Level_Export::s_bIsCharacterExport = false;
bool Tools_Level_Export::s_bIsGlobalSoundEventExport = false;
bool Tools_Level_Export::s_bIsGUIMenuExport = false;
bool Tools_Level_Export::s_bIsGUIMenuAudioExport = false;
bool Tools_Level_Export::s_bIsCommonFileExport = false;
bool Tools_Level_Export::s_bUpdateExistingFile = false;
bool Tools_Level_Export::s_bDisplayingProgressMessage = false;
bool Tools_Level_Export::s_bDebugColourSubstrips = false;
bool Tools_Level_Export::s_bForceTextureGeneration = false;
bool Tools_Level_Export::s_bUseNewNVidiaExport = false;
bool Tools_Level_Export::s_bWiiStreamsUseBink = false;
bool Tools_Level_Export::s_bCompressionDisabled = false;

Asura_Vector_3 Tools_Level_Export::s_xExportCentre = Asura_Maths::ZeroVector3;

bool Tools_Level_Export::s_abExportLanguage[ASURA_LT_L_NUM_LANGUAGES];
bool Tools_Level_Export::s_bLanguageArrayInitialised = false;
bool Tools_Level_Export::s_bOnlyExportLocalisedGlobalSounds = false;

bool Tools_Level_Export::s_bFilterEntityExport = false;
List<int> Tools_Level_Export::s_xEntityClassesToIgnore;

int Tools_Level_Export::s_iForcedBotanicalTextureFlagsPS2 = 0;

u_int Tools_Level_Export::s_uMaximumTextureSize = 4096;
bool Tools_Level_Export::s_bUseMaximumTextureSize = false;

bool Tools_Level_Export::s_bMission_ShouldExportEnvironment = true;
bool Tools_Level_Export::s_bMission_ShouldExportMission = true;

bool Tools_Level_Export::s_bMission_IsExportingEnvironment = false;
bool Tools_Level_Export::s_bMission_IsExportingMission = false;

HANDLE Tools_Level_Export::s_pDDSHandle = NULL;

// please keep in sync with Tools_Level_Export::EXPORT_STAGE
Asura_Char* Tools_Level_Export::s_aszExportStageText[EXPORT_STAGE_TOTAL] =
{
	"Undefined",
	"In-between stages",
	"Initialising",
	"Beginning export",
	"Generating subset of waypoints to export",
	"Saving temporary files for export",
	"Compiling list of files to include",
	"Loading files",
	"Adjusting UVs",
	"Adding environment materials",
	"Finding environment textures",
	"Finding meshless entity textures",
	"Finding corona textures",
	"Finding billboard plant textures",
	"Finding carpet textures",
	"Adding environment textures",
	"Adding skin textures",
	"Adding specific skin",
	"Adding shape textures",
	"Adding botanical textures",
	"Stripifying environment",
	"Grouping environment vertices by position",
	"Water: Collecting meshes",
	"Water: Subdividing meshes",
	"Water: Splitting quads",
	"Water: Stripifying meshes",
	"PS2 Writing stripped env",
	"Compiling Environment shaders",
	"Sorting strips based on shader and material hash",
	"Exporting environment",
	"Botanicals: Finding used templates and shapes",
	"Botanicals: calculating shape export size",
	"Botanicals: Exporting shapes",
	"Botanicals: Exporting templates",
	"Adding shape",
	"Creating physics zones",
	"Saving environment to file",
	"Compiling material response pixel shaders",
	"Dialogue Event Sounds",
	"Asura Files",
	"Export Sounds From Ini File",
	"Streaming Textures",
	"Skybox Textures",
	"Adding Misc Textures",
	"Hierarchy Sounds",
	"Environment Sounds",
	"Entity Sounds",
	"Shapes",
	"Non-Environmental Sounds",
	"Loading Old Export File",
	"Localised Sounds (non-streamed)",
	"Localised Sounds (streamed)",
	"Sound Event Sounds",
	"Sound Events (ToolsGmSnd.asr)",
	"Sound Events (Individual)",
	"Streaming Sounds",
	"Compressing File",

	"Total"
};

// F U N C T I O N S /////////////////////////////////////////////////////////
extern bool FillOutResourceFileListChunk( HANDLE hFile );
extern bool LeaveSpaceForResourceFileListChunk( HANDLE hFile );

BOOL WriteFileString( HANDLE hFile, const Asura_String &xString, DWORD* puBytesWritten)
{
	if( NULL != puBytesWritten)
	{
		*puBytesWritten = 0;
	}
	// DLH - This code stops uninitialised values being written off of the end of the string.
	// DLH - This code stops the file write reading off of the end of the string into non-valid memory and crashing.	
	
	// DLH - Allocate as much memory 
	Asura_FixedArray< char, MAX_PATH> xBufferString;
	const u_int uStringIntSize = (xString.GetSizeInInts() << 2);
	
	Asura_Assertf( uStringIntSize <= xBufferString.GetSize(), ("String '%s' is greater in length than %d bytes", xString.GetString(), xBufferString.GetSize()));
	
	const char* pszStringSrc = xString.GetString();
	char* pszStringDst = &xBufferString[0];
	// DLH - Zero all memory that will be written out to the file
	memset( pszStringDst, 0, uStringIntSize);
	memcpy( pszStringDst, pszStringSrc, Asura_Maths::Min( uStringIntSize, strlen( pszStringSrc)));
	
	return( WriteFile( hFile, pszStringDst, uStringIntSize, puBytesWritten, NULL));
}

// C L A S S E S /////////////////////////////////////////////////////////////
Tools_Level_Export::Tools_Level_Export()
#if defined ASURA_USE_BOTANICALS
	: m_xBotanicalsMtlList( 128 )
#endif
{
	This = this;
	s_eExportType = Export_PC;
	TransferIniFileData(TOOLSMODEDATAFILE_LOAD);

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	m_pxMaterialsToExport = 0;
	m_pxSlaveMaterialsToExport = 0;
	m_pxMaterialsWithExportedTexturesTree = 0;
	m_pxMaterialsToIgnore = 0;

	m_pxMaterialsFromCommonExport = 0;

	m_uNumMaterialResponseOverridesForExport	= 0;
	m_pxMaterialResponseOverridesForExport		= NULL;
	#endif

	m_pxExportedTexturesList = 0;
	m_pxSkinTextureList = 0;
	m_pxSkinFileList = 0;
	m_pxAnimFileList = 0;
	m_pxAnimDirList = 0;
	m_pxAsuraFileStringTree = 0;
	m_pxSkinFileStringTree = 0;
	m_pxShapeFileStringTree = 0;

	m_pxSkinHashTree = 0;
	m_pxAnimHashTree = 0;
	m_pxAnimTableHashTree = 0;

	m_pxEnvExportedTexturesList = 0;
	m_bFakeLevelExport = false;
	m_pxPS2TextureSizeIni = 0;
	m_bIsCharacterExport = false;
	m_bIsToolsSoundExport = false;

	m_pxSoundEnum = 0;

	InitialiseTempDirectory();

#if defined ASURA_USE_BOTANICALS
	m_iNumBotanicalsMtls = 0;
#endif

	m_bCreateLanguageFiles = false;
	m_bExportingLanguageFile = false;
	m_bUsingSharedLanguageFile = false;
	
	m_bAddingStreamedSounds = false;
	
	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	m_bIncludeOldMaterialChunks = false;
	#endif

	if(!s_bLanguageArrayInitialised)
	{
		EnableAllLanguagesForExport();
		s_bLanguageArrayInitialised = true;
	}

	#ifdef ASURA_USE_ANIMATION_IK_SOLVER
	m_bIncludeIKTemplates = false;
	#endif
}

Tools_Level_Export::~Tools_Level_Export()
{
	CloseTempDirectory();
	
	delete m_pxPS2TextureSizeIni;
	delete m_pxSoundEnum;
	if(!Tools_BatchProcessing::IsBatch())
	{
		TransferIniFileData(TOOLSMODEDATAFILE_SAVE);
	}
	This = NULL;
}

void Tools_Level_Export::TransferIniFileData(int iMode)
{
	//open the file
	Tools_ModeDataFile xFile("LevelExport",iMode);

	//tranfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	if( false == xFile.GetError())
	{
		bool bJunk= false;
		float fJunk=0;
	
		xFile.Transfer(s_bSubdivideMeshes);
		xFile.Transfer(s_bSwizzleTims);
		xFile.Transfer(s_bRecalculatePrelighting);
		xFile.Transfer(s_fFlatSubdivideSize);
		xFile.Transfer(s_fCurveSubdivideSize);
    	xFile.Transfer( s_bDontExportCutscenes );
    	xFile.Transfer( s_bPS2MultiplayerExport );
    	xFile.Transfer( bJunk );//(used to be s_bCreateSeperateCutsceneFiles
		xFile.Transfer(bJunk);//(used to be anim frame removal settings)
		xFile.Transfer(fJunk);//(used to be anim frame removal settings)
		xFile.Transfer(fJunk);//(used to be anim frame removal settings)
    	xFile.Transfer(bJunk);//(used to be s_bIncludeDualTexturingData)
		xFile.Transfer(s_bUseMaximumTextureSize);
		xFile.Transfer(s_uMaximumTextureSize);
	}
}

void Tools_Level_Export::StartExport()
{
	s_bUserAbort = false;
	s_bAbort = false;
	s_iPercent = 0;
	s_xHeading = 0;
	s_iNumErrors = 0;
	Tools_Timed_Message_System::DisableTimer();
    g_bPreparedHashMapFile = false;
	s_bIsExport = true;

	m_iTotalAnimationFramesBefore = 0;
	m_iTotalAnimationFramesAfter = 0;
	m_iTotalAnimationSizeBefore = 0;
	m_iTotalAnimationSizeAfter = 0;

  // Reset PS2 texture dimension cache.
	if( Export_PS2 == s_eExportType )
	{
		for( u_int u = 0; u < ASURA_MAX_NO_OF_TEXTURES; ++u )
		{
			g_auMaxTIM2Dimensions[u] = 0xFFFFFFFFu;
		}
	}

	if(!IsFakeLevelExport())
	{
		// As we're now loading overrides on demand, we'll want to reload all to make sure we've got everything for export.
		Tools_Mode_PFX::Verify(false);
	}

	ResetExportSummaryStats();
}

void Tools_Level_Export::StartFileExport()
{
	m_iNumberOfUntexturedPolygons = 0;
	m_iNumberOfPolygonsWithMissingTexture = 0;
	m_iUntexturedTextureIndex = -1;
	m_iUntexturedMaterialIndex = -1;
	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	m_pxMaterialsToExport = new Asura_HashTree();
	m_pxSlaveMaterialsToExport = new Asura_HashTree();
	m_pxMaterialsWithExportedTexturesTree = new Asura_HashTree();
	m_pxMaterialsToIgnore = new Asura_HashTree();

	m_pxMaterialsFromCommonExport = new Asura_NameTree();

	#endif //ASURA_USE_MATERIAL_RESPONSE_SYSTEM
    m_pxExportedTexturesList = new Asura_NameTree;
	m_pxSkinTextureList = new Asura_NameTree;
	m_pxSkinFileList = new Asura_NameTree;
	m_pxAnimFileList = new Asura_NameTree;
	m_pxAnimDirList = new Asura_NameTree;
	m_pxAsuraFileStringTree = new Asura_StringTree;
	m_pxSkinFileStringTree = new Asura_StringTree;
	m_pxShapeFileStringTree = new Asura_StringTree;
	m_bIncludeEntityShapes = false;
	m_bIncludeCommonShapes = false;
	m_bIncludeSkinAndShapeSounds = false;

	m_pxSkinHashTree = new Asura_HashTree();
	m_pxAnimHashTree = new Asura_HashTree();
	m_pxAnimTableHashTree = new Asura_HashTree();

	m_pxSoundEventTree = new Asura_HashTree;
	m_pxSoundSampleTree = new Asura_HashTree;

	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	m_pxDialogueEventTree = new Asura_HashTree;
	m_pxDialogueLineTree = new Asura_HashTree;
	m_pxDialogueLineTemplateTree = new Asura_HashTree;
	#endif

	#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	m_pxFaceContainer					= new FaceContainer;
	m_pxFaceAnimationContainer			= new FaceAnimationContainer;
	m_pxLocalisedFaceAnimationContainer	= new FaceAnimationContainer;
	#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

	m_pxBlueprintTypeTree = new Asura_HashTree;
	
	#ifdef ASURA_USE_ANIMATION_IK_SOLVER
	m_bIncludeIKTemplates = false;
	#endif

	m_bCreateLanguageFiles = false;//will be marked as true during export if needed
	m_bExportingLanguageFile = false;
	m_bUsingSharedLanguageFile = false;
	m_xSharedLanguageFilename.MakeNull();

	Asura_Blueprint_System::StartExport();

#ifdef ASURA_USE_FSFX
	Tools_Mode_FSFX::StartExport();
#endif // def ASURA_USE_FSFX

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
	//Reset ignore lists in these systems
	// DF 19/04/07
	Tools_Mode_PFX::StartExport();
	#endif

	if( !Tools_Level_Export::IsFakeLevelExport() )		
	{
		//Make sure the correct platform's meshes are being used
		Tools_PlatformMesh_System::PushPlatform(s_eExportType,true);
		Tools_PlatformMesh_System::ReloadFilesIfNeeded();
	}
}

void Tools_Level_Export::EndFileExport()
{
	WriteExportSummary();
	ResetExportSummaryStats();

	if( !Tools_Level_Export::IsFakeLevelExport() )		
	{
		//restore previous platform meshes
		Tools_PlatformMesh_System::PopPlatform(true);
		Tools_PlatformMesh_System::ReloadFilesIfNeeded();
	}

	Asura_Blueprint_System::EndExport();

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	delete m_pxMaterialsToExport;
	m_pxMaterialsToExport = 0;

	delete m_pxSlaveMaterialsToExport;
	m_pxSlaveMaterialsToExport = 0;

	delete m_pxMaterialsWithExportedTexturesTree;
	m_pxMaterialsWithExportedTexturesTree = 0;

	delete m_pxMaterialsToIgnore;
	m_pxMaterialsToIgnore = 0;

	delete m_pxMaterialsFromCommonExport;
	m_pxMaterialsFromCommonExport = 0;
	#endif

	delete m_pxExportedTexturesList;
	m_pxExportedTexturesList = 0;

	delete m_pxSkinTextureList;
	m_pxSkinTextureList = 0;

	delete m_pxSkinFileList;
	m_pxSkinFileList = 0;

	delete m_pxAnimFileList;
	m_pxAnimFileList = 0;

	delete m_pxAnimDirList;
	m_pxAnimDirList = 0;


	while(m_xSoundList.size())
	{
		delete m_xSoundList.first_entry();
		m_xSoundList.delete_first_entry();
	}

	while(m_xExportedSoundList.size())
	{
		delete m_xExportedSoundList.first_entry();
		m_xExportedSoundList.delete_first_entry();
	}
	while(m_xLocalisedSoundList.size()) m_xLocalisedSoundList.delete_first_entry();
	while(m_xLocalisedStreamedSoundList.size()) m_xLocalisedStreamedSoundList.delete_first_entry();
	Asura_Assert(!m_xLocalisedStreamedSoundExportList.size(), "should have been deleted already");

	while(m_xSoundMappingList.size()) m_xSoundMappingList.delete_first_entry();
	while(m_xSoundPropertiesList.size()) m_xSoundPropertiesList.delete_first_entry();

	while(m_xSoundEventList.size()) m_xSoundEventList.delete_first_entry();
	while(m_xSoundSampleList.size()) m_xSoundSampleList.delete_first_entry();
	delete m_pxSoundEventTree;
	m_pxSoundEventTree = 0;
	delete m_pxSoundSampleTree;
	m_pxSoundSampleTree = 0;

	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	while(m_xDialogueEventList.size()) m_xDialogueEventList.delete_first_entry();
	while(m_xDialogueLineList.size()) m_xDialogueLineList.delete_first_entry();
	while(m_xDialogueLineTemplateList.size()) m_xDialogueLineTemplateList.delete_first_entry();
	delete m_pxDialogueEventTree;
	m_pxDialogueEventTree = 0;
	delete m_pxDialogueLineTree;
	m_pxDialogueLineTree = 0;
	delete m_pxDialogueLineTemplateTree;
	m_pxDialogueLineTemplateTree = 0;
	#endif

	#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	delete(m_pxFaceContainer);
	delete(m_pxFaceAnimationContainer);
	delete(m_pxLocalisedFaceAnimationContainer);
	m_pxFaceContainer					= NULL;
	m_pxFaceAnimationContainer			= NULL;
	m_pxLocalisedFaceAnimationContainer	= NULL;
	#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

	while(m_xBlueprintTypeList.size()) m_xBlueprintTypeList.delete_first_entry();
	delete m_pxBlueprintTypeTree;
	m_pxBlueprintTypeTree = 0;

	#ifdef ASURA_USE_GUI_MENU_SYSTEM
	while(m_xGUIMenuList.size()) m_xGUIMenuList.delete_first_entry();
	#endif



	delete m_pxSkinHashTree;
	m_pxSkinHashTree = 0;
	delete m_pxAnimHashTree;
	m_pxAnimHashTree = 0;
	delete m_pxAnimTableHashTree;
	m_pxAnimTableHashTree = 0;

	while(m_xAnimTableList.size()) m_xAnimTableList.delete_first_entry();

	m_xSeperatelyLoadedFiles.empty();
	m_xDirsToIgnore.empty();
	m_xFilesToIgnore.empty();
    m_xAsuraFileList.empty();

	delete m_pxAsuraFileStringTree;
	m_pxAsuraFileStringTree = 0;

	delete m_pxSkinFileStringTree;
	m_pxSkinFileStringTree = 0;

	delete m_pxShapeFileStringTree;
	m_pxShapeFileStringTree = 0;
		
#ifdef ASURA_USING_ALL_LEVEL_EXPORT
	Tools_Mode_PFX::EndFileExport();
#ifdef ASURA_USE_FSFX
	Tools_Mode_FSFX::EndFileExport();
#endif
#endif 

	while(m_xShapeFilenames.size())
	{
		m_xShapeFilenames.delete_first_entry();
	}

	while(m_xShapeMeshList.size())
	{
		m_xShapeMeshList.delete_first_entry();
	}

	while(m_xMiscTextureList.size())
	{
		m_xMiscTextureList.delete_first_entry();
	}

	while(m_xGUIMenuTextureList.size())
	{
		m_xGUIMenuTextureList.delete_first_entry();
	}
}

void Tools_Level_Export::EndExport(bool bContinuePrompt)
{
	//delete the temporary level export files
	DeleteTemporaryFiles();

	delete m_pxEnvExportedTexturesList;
	m_pxEnvExportedTexturesList = 0;

    m_xEnvAsuraFileList.empty();


    // Prompt the user to press a key before continuing, to ensure they see
    // any important error messages.
    if( ShouldPauseAtEndOfExport() && bContinuePrompt )
	{
		PauseAtEndOfExport();
    }

	Tools_Timed_Message_System::EnableTimer();
	Asura_Windows::RestoreDefaultWindowTitle();
	
	s_bIsExport = false;
}

void Tools_Level_Export::ResetExportSummaryStats()
{
	// clear the m_xExportStageStatsList
	for (int iStage = 0; iStage < EXPORT_STAGE_TOTAL; iStage++)
	{
		m_xExportStageStatsList[iStage].m_bUsed = false;
		m_xExportStageStatsList[iStage].m_pszText = s_aszExportStageText[iStage];
		m_xExportStageStatsList[iStage].m_xTimeStart = 0;
		m_xExportStageStatsList[iStage].m_xStageTime = 0;
	}

	s_eThisStage = EXPORT_STAGE_UNDEFINED;
	s_eCurrentStage = EXPORT_STAGE_UNDEFINED;
}

void Tools_Level_Export::WriteExportSummary()
{
	ExportStageStats xSortedStatsList[EXPORT_STAGE_TOTAL];
	ExportStageStats (&xListToOutput)[EXPORT_STAGE_TOTAL] = xSortedStatsList;

	// clock overall export time
	m_xExportStageStatsList[EXPORT_STAGE_OVERALL].m_xStageTime = Asura_Timers::GetCurTicks() - m_xExportStageStatsList[EXPORT_STAGE_OVERALL].m_xTimeStart;

	// clock last stage time
	m_xExportStageStatsList[s_eCurrentStage].m_xStageTime += Asura_Timers::GetCurTicks() - m_xExportStageStatsList[s_eCurrentStage].m_xTimeStart;

	Asura_Mem::Copy(&xSortedStatsList, &m_xExportStageStatsList, sizeof (m_xExportStageStatsList));

	// we don't want total time in the sorted list
	xSortedStatsList[EXPORT_STAGE_OVERALL].m_bUsed = false;

	Asura_SortAndSearch::Quicksort( xSortedStatsList, EXPORT_STAGE_TOTAL, CompareExportStageStatsElement);

	AddMessage(MT_LOGANDMESSAGE, "\n");
	AddMessage(MT_LOGANDMESSAGE, "Time Summary Report for \"%s\".....", s_szMainHeading);

	//  21% (1m25s) Adding Skins
	//  45% (2m10s) Optimising, subdiv....
	// ------------
	// 100% (4m30s) Total
	// ------------

	float fTotalExportTime = Asura_Timers::TicksToSeconds(m_xExportStageStatsList[EXPORT_STAGE_OVERALL].m_xStageTime);
	const u_int uTotalStagesToWrite = 3;	// only write the top 3 stages to message log
	u_int uWrittenStagesCount = 0;
	float fUnwrittenStagesTime = 0.0f;

	for (unsigned int uStage = 0; uStage < EXPORT_STAGE_TOTAL; uStage++)
	{
		if (xListToOutput[uStage].m_bUsed)
		{
			float fStageTime = Asura_Timers::TicksToSeconds(xListToOutput[uStage].m_xStageTime);

			if (uWrittenStagesCount < uTotalStagesToWrite)
			{
				AddMessage(MT_LOGANDMESSAGE, " %6.1f%% (%6.2fs) %s", (fStageTime / fTotalExportTime) * 100.0f, fStageTime, xListToOutput[uStage].m_pszText);
			}
			else
			{
				AddMessage(MT_LOGONLY, " %6.1f%% (%6.2fs) %s", (fStageTime / fTotalExportTime) * 100.0f, fStageTime, xListToOutput[uStage].m_pszText);
				fUnwrittenStagesTime += fStageTime;
			}

			uWrittenStagesCount++;
		}
	}

	if (fUnwrittenStagesTime > 0.0f)
	{
		// write "Others" stage
		AddMessage(MT_MESSAGELOGONLY, " %6.1f%% (%6.2fs) %s", (fUnwrittenStagesTime / fTotalExportTime) * 100.0f, fUnwrittenStagesTime, "Others");
	}

	AddMessage(MT_LOGANDMESSAGE, "------------------");
	AddMessage(MT_LOGANDMESSAGE, " %6.1f%% (%6.2fs) %s", 100.0f, fTotalExportTime, m_xExportStageStatsList[EXPORT_STAGE_OVERALL].m_pszText);
	AddMessage(MT_LOGANDMESSAGE, "------------------");
}

int Tools_Level_Export::CompareExportStageStatsElement( const void* pData1, const void* pData2 )
{
	const ExportStageStats& xData1 = *reinterpret_cast<const ExportStageStats*>(pData1);
	const ExportStageStats& xData2 = *reinterpret_cast<const ExportStageStats*>(pData2);
	if (xData1.m_xStageTime < xData2.m_xStageTime)
	{
		return 1;
	}
	if (xData1.m_xStageTime > xData2.m_xStageTime)
	{
		return -1;
	}
	return 0;
}

bool Tools_Level_Export::ShouldPauseAtEndOfExport( void ) const
{
	return !m_bFakeLevelExport && !Tools_BatchProcessing::IsBatch();
}

void Tools_Level_Export::PauseAtEndOfExport()
{
	//This may occur be called after EndExport() has been called , so
	//temporarily pretend we are in the export again , so that the messages get displayed.
	//RAR 14/12/2006
	bool bWasExport = s_bIsExport;
	if(!bWasExport)
	{
		s_bIsExport = true;
		Tools_Timed_Message_System::DisableTimer();
	}

	    AddMessage(MT_INFORMATION,"Press any key to continue.");
	    while( !Asura_UserInput::DebouncedHasAnyKeyBeenPressed() )
	    {
		    UpdateProgress();
	    }

	if(!bWasExport)
	{
		s_bIsExport = bWasExport;
	Tools_Timed_Message_System::EnableTimer();
	}
}

void Tools_Level_Export::OpenExportLog(const char* pszExportedFileName)
{
	Asura_Assert(!s_szLogPath,"Log path already set");
	Asura_Assert(!s_szErrorLogPath,"Error log path already set");

	//get the filename with path/extension
	Asura_String xFilename = pszExportedFileName;
	Asura_String xName;
	xFilename.SplitFilename(0,&xName,0);


	const char* pszPlatform = 0;
	switch(s_eExportType)
	{
		case Export_PC :
			pszPlatform = "PC";
			break;
		case Export_PS2 :
			pszPlatform = "PS2";
			break;
		case Export_PSP :
			pszPlatform = "PSP";
			break;
		case Export_PS3 :
			pszPlatform = "PS3";
			break;
		case Export_Xbox360 :
			pszPlatform = "Xbox360";
			break;
		case Export_Wii :
			pszPlatform = "Wii";
			break;

		default :
			Asura_Assert(0,"Unexpected platform type");
			break;
	}

	char szLogName[MAX_PATH];

	//generate the filenames for the export log , and error log
	Asura_PrintfToBfr(szLogName,"LogFiles\\Export\\%s_%s.txt",MAX_PATH,true,xName.GetString(),pszPlatform);
	s_szLogPath = new char[strlen(szLogName)+1];
	strcpy(s_szLogPath,szLogName);

	//make sure the directory exists 
	Tools_FileSystem::EnsureDirectoryExists(s_szLogPath);
	
	Asura_PrintfToBfr(szLogName,"LogFiles\\Export\\%s_%s_Errors.txt",MAX_PATH,true,xName.GetString(),pszPlatform);
	s_szErrorLogPath = new char[strlen(szLogName)+1];
	strcpy(s_szErrorLogPath,szLogName);

    // (NRM - 27.06.2003) generate the logfile for the sounds
    Asura_PrintfToBfr(szLogName,"LogFiles\\Export\\%s_%s_ExportedSounds.txt",MAX_PATH,true,xName.GetString(),pszPlatform);
	s_szSoundLogPath = new char[strlen(szLogName)+1];
	strcpy(s_szSoundLogPath,szLogName);

	// DLH - 18 April 2007 14:40:20 - export to XML Assert file to a different location
    Asura_PrintfToBfr(szLogName,"LogFiles\\Export\\%s_%s_Asserts.xml",MAX_PATH,true,xName.GetString(),pszPlatform);
    AsuraAssert_LogXMLFilename( szLogName);
    Asura_SimpleLogfileClearXML( szLogName);

	// RDM - 03/04/08 - Added new blueprint templated-value-type export
	Asura_PrintfToBfr(szLogName,"LogFiles\\Export\\%s_%s_BlueprintExports.txt",MAX_PATH,true,xName.GetString(),pszPlatform);
	s_szBlueprintLogPath = new char[strlen(szLogName)+1];
	strcpy(s_szBlueprintLogPath,szLogName);

	//Hierarchical logfile of resources exported.
	Asura_PrintfToBfr(szLogName,"LogFiles\\Export\\%s_%s_ResourceExportLog.txt",MAX_PATH,true,xName.GetString(),pszPlatform);
	Tools_ResourceExportLog::OpenLog(szLogName);
	

	//make sure the the files are cleared
	Asura_SimpleLogfileClear(s_szLogPath);
	Asura_SimpleLogfileClear(s_szErrorLogPath);
    Asura_SimpleLogfileClear(s_szSoundLogPath);
	Asura_SimpleLogfileClear(s_szBlueprintLogPath);
    
    // DLH - Add Assert to ID which Export this is
    if( Tools_BatchProcessing::IsBatch())
    {
		Asura_Assertf( false, ("BatchInfo: Export Name '%s' platform '%s'",xName.GetString(),pszPlatform));
}
}

void Tools_Level_Export::CloseExportLog()
{
	delete [] s_szLogPath;
	delete [] s_szErrorLogPath;
	delete [] s_szSoundLogPath;
	delete [] s_szBlueprintLogPath;
	s_szLogPath = 0;
	s_szErrorLogPath = 0;
	s_szSoundLogPath = 0;
	s_szBlueprintLogPath = 0;

	Tools_ResourceExportLog::CloseLog();
}

bool Tools_Level_Export::CheckPrelightingIsCorrect()
{
	bool bRet = false;
	// Make sure we're not pre-lighting the environment with entity lights.aw.
	if (Asura_Lighting_System::IsUsingEntityLights())
	{
		// Turn the option off.
		Asura_Lighting_System::ToggleUsingEntityLights();
		bRet = true;
	}
	// Re-calculate the lighting for any modules if necessary.
	bool bFirst = true;
	for (int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		Asura_Mesh *pxMesh = Asura_Module_System::GetMeshForModule(i);
		if (!pxMesh->PrelightingDataValid)
		{
			if (bFirst)
			{
				Tools_Timed_Message_System::AddMessage(0xff00ff00, "Updating pre-lighting data");
				bFirst = false;
			}
			Asura_Vector_3 xPosition;
			Asura_Module_System::GetPositionOfModule(i, xPosition);
			Asura_Lighting_System::CalculatePrelighting(*pxMesh, xPosition,i);
		}
	}
	return bRet;
}

void Tools_Level_Export::ExportSkybox_AllPlatforms()
{
	if(!This) return;
	
	//export skybox for each platform supported by project
	for(int iPlatform=0;iPlatform<ExportType_NumPlatforms;iPlatform++)
	{
		EXPORT_TYPE ePlatform = (EXPORT_TYPE) iPlatform;
		if(!This->IsPlatformAllowed(ePlatform)) continue;

		s_eExportType = ePlatform;
		This->ExportSkybox();
	}	

	s_eExportType = Export_PC;
}

void Tools_Level_Export::AddSkyboxFilesToExport()
{
	AddFileToExportList( Tools_SaveAsura::GetSkyboxFilename(), false );
    // Add PS2 skysphere files (if necessary)
    if (Export_PS2 == s_eExportType)
    {
       	if (Asura_Skybox::GetDrawClouds())
       	{
       	    const Asura_String xPS2SkysphereFileAndPath("..\\PS2_Release\\skyspher.asr");
       	    AddFileToExportList( xPS2SkysphereFileAndPath, false );
		}
    }

}

void Tools_Level_Export::ExportSkybox()
{
	//Early check to make sure destination files are writeable. RAR 17/01/2006
	if(!IsExportedSkyboxFileWriteable())
	{
		return;
	}

	s_szMainHeading = "Skybox Export";

	extern Asura_String EnvironmentFilename;

	Asura_String xLogName;
	EnvironmentFilename.SplitFilename(0,&xLogName,0);
	xLogName += "_Skybox";

	StartExport();
	OpenExportLog(xLogName.GetString());


	StartSkyboxExport();

	if(!IsAbortFlagSet())
	{
		StartFileExport();
	}

	
	if(!IsAbortFlagSet())
	{
		//make sure the individual asura files are up to date first.
		Tools_SaveAsura::SaveSkybox();	
	}

	if(!IsAbortFlagSet())
	{
		
		AddSkyboxFilesToExport();

		Asura_String xExportFilename;
		GetExportedSkyboxFilename( m_xLevelNameBackup, xExportFilename );
		ExportFilesInList( xExportFilename.GetString(),false);

	}
	
	EndFileExport();
	EndSkyboxExport();
	CloseExportLog();
	EndExport(false);
}

void Tools_Level_Export::GetExportedSkyboxFilename(const Asura_String& xEnvFilename,Asura_String& xOutputFilename)
{
	Asura_Assert(0,"Default implementation of GetExportedSkyboxFilename() not done");
	GetExportedLevelFilename("Skybox",xOutputFilename);
}

void Tools_Level_Export::StartSkyboxExport()
{
	
	EXPORT_TYPE eExportTypeBackup = s_eExportType;
	//temporary change to export type , since for this stage we don't want to
	//have GC endian swapping getting in the way
	s_eExportType = Export_PC;

	m_xLevelNameBackup = Tools_SaveAsura::GetMainFilename();
	if(Tools_SaveAsura::IsSkyboxModified())
	{
		//attempt to save files under their normal filenames first , since we
		//are going to lose the state of the modified flags.
		Tools_SaveAsura::SaveSkybox();
	}
	s_eExportType = eExportTypeBackup;


	//generate a temporary filename to use during level export , to avoid
	//problems from having read-only files
	Asura_String xName;
	m_xLevelNameBackup.SplitFilename(0,&xName,0);

	char szNewFilename[MAX_PATH];
	Asura_PrintfToBfr(szNewFilename,"%s\\%s.asr",MAX_PATH,true,GetTempDirectory(),xName.GetString());
	Tools_FileSystem::EnsureDirectoryExists(szNewFilename);

	Tools_SaveAsura::SetFilename(szNewFilename);
	Tools_SaveAsura::SetRealFilename( m_xLevelNameBackup );

    s_bIsSkyboxExport = true;
}

void Tools_Level_Export::EndSkyboxExport()
{
    s_bIsSkyboxExport = false;
	
	
	//restore the filename
	Tools_SaveAsura::SetFilename(m_xLevelNameBackup.GetString());
}

bool Tools_Level_Export::IsExportedSkyboxFileWriteable(bool bShowMessage)
{

	Asura_String xEnvName = Tools_SaveAsura::GetMainFilename();
	Asura_String xExportName;

	GetExportedSkyboxFilename(xEnvName,xExportName);

	return IsWriteableIncludingLanguageFiles(xExportName.GetString(),bShowMessage);

}

void Tools_Level_Export::ExportFilesInList(const char* pszFilename,bool bEnvironmentFile)
{
	//Check for any other resources that are going to need to be added as a result of using language files
	//shared across several exports.
	//RAR 10/07/2007
	if(GetSharedLanguageFilename(pszFilename,m_xSharedLanguageFilename))
	{
		Asura_Assert(!m_xSharedLanguageFilename.IsBlank(),"");
		GetResourcesFromExistingLanguageFile(pszFilename);
		m_bUsingSharedLanguageFile = true;
	}
			
			
	//allocate storage space for the resource file list
    g_uAsuraFilesInExport = m_xAsuraFileList.size();
	g_axResourceFileListEntries = new ResourceFileListEntry[g_uAsuraFilesInExport];

   	// Fill out list of names of Asura files we're going to output.
   	{
		Asura_String*	axFilenames		= new Asura_String[g_uAsuraFilesInExport];
   	    u_int			uNumFilenames	= 0;

		// Create a list of all of the filenames we've got...
		for(CLIF<Asura_String> xFileLIF(&m_xAsuraFileList);!xFileLIF.done();xFileLIF.next())
		{
            // Strip lengthy and unnecessary path prefixes.
			axFilenames[uNumFilenames++]	= Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( xFileLIF().GetString() );
		}

		switch(s_eExportType)
		{
			case Export_PS3:
			{
				u_int	uIndex	= 0;

				// Add environment chunks first...
				for(u_int u = 0; u < uNumFilenames; u++)
				{
					if(IsEnvironmentFile(axFilenames[u]))
					{
						g_axResourceFileListEntries[uIndex++].xFilename = axFilenames[u];
					}
				}

				// Add environment chunks first...
				for(u_int u = 0; u < uNumFilenames; u++)
				{
					if(!IsEnvironmentFile(axFilenames[u]))
					{
						g_axResourceFileListEntries[uIndex++].xFilename = axFilenames[u];
					}
				}
				break;
			}

			default:
			{
				u_int	uIndex	= 0;

				// No tests needed - add everything in one go...
				for(u_int u = 0; u < uNumFilenames; u++)
				{
					g_axResourceFileListEntries[uIndex++].xFilename = axFilenames[u];
				}
				break;
			}
		}		// End of switch.

		delete[](axFilenames);
   	}

   	// This macro assumes an error msg will be output elsewhere if the 
   	// specified expression is false.
   	#define CHECK( expr ) \
   	{ \
   	    /* Set abort flag if the expression is false. */ \
   	    if( !(expr) ) { SetAbortFlag(); } \
   	    \
   	    /* Check the abort flag independently because it may have been set \
   	       somewhere else, eg. by the user requesting an abort. */ \
   	    if( IsAbortFlagSet() ) { goto End; } \
   	}

   	HANDLE saveHandle = INVALID_HANDLE_VALUE;
   	DWORD bytesWritten;
   
	//open the file to write to
	Tools_FileSystem::EnsureDirectoryExists(pszFilename);
	saveHandle = CreateFile(pszFilename,GENERIC_WRITE,0,0,CREATE_ALWAYS,0,0);
	if(saveHandle == INVALID_HANDLE_VALUE)
	{
		AddMessage(MT_ERROR,"Failed to open %s for writing. Export aborted.",pszFilename);
		CHECK( false );
	}

	//write the header
	WriteFile(saveHandle,ASURA_UNCOMPRESSED_FILE_ID,ASURA_FILE_ID_NUM_CHARS,&bytesWritten,0);

   	// Leave space for the resource file list chunk at the beginning of the 
   	// file. We know the maximum size but we don't know how much space we'll
   	// actually require until we've tried adding each Asura file so just
   	// provide the maximum. (In most cases, we'll need the maximum anyway, or
   	// very close to it.)
   	// (MH - 08.01.2003) also leave space for the fileinfo chunk - this contains
   	// the filesize so we won't know it until the end.
   	CHECK( LeaveSpaceForFileInfoChunk(saveHandle) );
	CHECK( LeaveSpaceForResourceFileListChunk(saveHandle) );
	

	if(MissionExport_IsExportingMissionOnly())
	{
		//Add a filename resource to the mission to tell it the environment file that needs to be loaded
		//RAR 18/09/2008
		extern Asura_String EnvironmentFilename;
		Asura_String xExportedEnvironmentFilename;
		GetExportedLevelFilename(m_xLevelNameBackup,xExportedEnvironmentFilename);

		const char* szRelativeFilename = strstr(xExportedEnvironmentFilename.GetString(), "envs");
		Asura_Assert(szRelativeFilename, "Failed to get relative filename for environment file");
		if(szRelativeFilename)
		{
			AddFilenameResource(saveHandle, szRelativeFilename, false);
		}
	}
	
	if(!s_bIsGlobalSoundEventExport)
	{
		//Prevent global sound events from being exported. RAR 14/07/2006
		ExcludeGlobalSoundEvents();
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		//and global dialogue. RAR 08/11/2006
		ExcludeGlobalDialogueEvents();
		#endif
	}

	CHECK(AddBlueprints(saveHandle));

#ifdef ASURA_USE_CONTAINER_SYSTEM
	if (IsCommonFileExport())
	{
		CHECK(AddContainers(saveHandle));
	}
#endif //ASURA_USE_CONTAINER_SYSTEM

	#ifdef ASURA_USE_ANIMATION_IK_SOLVER
	if(m_bIncludeIKTemplates)
	{
		CHECK(Tools_Animation_IK_Template_System::AddToExport(saveHandle));
	}
	#endif

	if(s_eExportType == Export_Wii)
	{
		Tools_Wii_Textures::BeginExport(pszFilename);
	}

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		BeginMRPShaderExport();
	#endif

	// Add all the files in the list
	u_int u = 0;
	switch( s_eExportType )
	{
	case Export_PS3:
		for( CLIF<Asura_String> xFileLIF(&m_xAsuraFileList); !xFileLIF.done(); xFileLIF.next() )
		{
			if( !IsEnvironmentFile( xFileLIF() ) )
			{
				continue;
			}
			else
			{
				CHECK( AddAsuraFile(saveHandle, xFileLIF(), u++) );
			}
		}
		break;
	default:
		break;
	}

	if(MissionExport_IsExportingMissionOnly())
	{
		//If excluding environment tetxures , it needs to be done earlier , so that
		//it happens before the skin textures etc get added
		ExcludeEnvironmentTextures();
	}

	//Deal with skin textures first , to make sure they get their special flags
	AddSkinTextures(saveHandle);
	if(IsAbortFlagSet()) goto End;

#ifdef ASURA_EXPORT_MISC_TEXTURES_FIRST
	//Add other textures that have been queued up
	AddMiscTextures(saveHandle);
#endif

	if(ShouldIncludeSkyboxInLevelExport( bEnvironmentFile ) && !MissionExport_IsExportingMissionOnly())
	{
		AddSkyboxTextures(saveHandle);
	}

		//Deal with the environment textures
	if(bEnvironmentFile && !MissionExport_IsExportingMissionOnly())
	{
		AddEnvironmentTextures(saveHandle);
	}

		
	if(IsAbortFlagSet()) goto End;

	if(bEnvironmentFile || IsCommonFileExport())
	{

		//Determine permanently loaded sounds ,to help with working out which to export
		//(Not entirely sure this should be limited to env/common exports - RAR 12/06/2007
		Project_GetGameSoundList();

		//Add sound resources
		if(bEnvironmentFile)
		{
			if(!MissionExport_IsExportingEnvironmentOnly())
			{
				AddEnvironmentalSounds(saveHandle);
				if(IsAbortFlagSet()) goto End;
		
			#ifndef ASURA_USE_AUDIO_SYSTEM
				AddSounds(saveHandle,"Entity Sounds",ASURA_SID_ENTITY_MIN,ASURA_SID_ENTITY_MAX);
				if(IsAbortFlagSet()) goto End;
			#endif
			}
		}
	}
	else if(IsSkyboxExport())
	{
		AddSkyboxTextures(saveHandle);
	}
#if defined(ASURA_USE_GUI_MENU_SYSTEM)
	else if(IsGUIMenuAuidoExport())
	{
		AddGUIMenuAudioData(saveHandle);
	}
#endif	//defined(ASURA_USE_GUI_MENU_SYSTEM)

#ifndef ASURA_EXPORT_MISC_TEXTURES_FIRST
	//Add other textures that have been queued up
	AddMiscTextures(saveHandle);
#endif

	if(IsAbortFlagSet()) goto End;

	if (m_bIncludeSkinAndShapeSounds || bEnvironmentFile)
	{
		if (!SkipNonEnvironmentalSounds())
		{
			BuildNonEnvironmentalSoundList(m_xAsuraFileList);
			AddNonEnvironmentalSounds(saveHandle);
			if(IsAbortFlagSet()) goto End;
		}

		//Add sounds used by skins 
		AddSkinSounds(saveHandle);
		

		//write out the sound mappings
		AddSoundMappingResource(saveHandle);
		AddSoundPropertiesResource(saveHandle);
	}
	
	//This needs to be done after the other sounds , just in case any of the wav files being exported
	//have a non-dynamic sound ID given to them from elsewhere. RAR 13/07/2006
	CHECK(AddSoundEvents(saveHandle));
	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	CHECK(AddDialogueEvents(saveHandle));
	#endif

	#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	AddFaceAnimsUsedByStandardAnimsToExportList();
	CHECK(AddFacialAnimationData(saveHandle));
	#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

	// Add shapes
	if(m_bIncludeEntityShapes || m_bIncludeCommonShapes)
	{
		AddShapes( saveHandle );
	}

	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

		if( bEnvironmentFile )
		{
			// Always include the default material in a level export
			AddMaterialToExport( saveHandle, ASURA_DEFAULT_MATERIAL_RESPONSE );

//DF Don't need to do this is using new system, .ini files are redundant
#ifndef ASURA_TURN_OFF_ALWAYSEXPORTINIFILES_SUPPORT
			// Add materials in the "always export" text file
			const char* const szIniName = "ToolsIniFiles\\AlwaysExportMaterials.ini";
			Tools_IniFile xIni( szIniName );
			if( !xIni.LoadIniFile() )
			{
				AddMessage( MT_WARNING, "Failed to load %s", szIniName );
			}

			AddMessage( MT_LOGONLY, "Loaded %s", szIniName );

			while( xIni.GetNextIniTextLine() )
			{
				Asura_String xMaterialName;
				if( !xIni.GetString( xMaterialName ) ) continue;

				const Asura_Hash_ID uMaterialNameHash = xMaterialName.GetHash();
				if( Tools_MaterialResponse_System::GetMaterialByHashID( uMaterialNameHash ) )
				{
					AddMaterialToExport( saveHandle, uMaterialNameHash );
				}
				else
				{
					AddMessage( MT_ERROR, "Unable to find material '%s' listed in always-export file", xMaterialName.GetString() );
				}
			}
#else
			//Do this instead, read the materials from the list that was saved earlier and include them here
			// DF 19/04/07
			const Asura_String** pxMaterialNameList = new const Asura_String*[m_pxMaterialsFromCommonExport->GetNumberOfNodes()];
			//Asura_Hash_ID* pxMaterialHashList = new Asura_Hash_ID[m_pxMaterialsFromCommonExport->GetNumberOfNodes()];
			m_pxMaterialsFromCommonExport->MakeNameList(pxMaterialNameList);
			for(int i=0; i<m_pxMaterialsFromCommonExport->GetNumberOfNodes(); i++)
			{
				const Asura_String uMaterialName = *pxMaterialNameList[i];
				const Asura_Hash_ID uMaterialHash = uMaterialName.GetHash();
				if( Tools_MaterialResponse_System::GetMaterialByHashID( uMaterialHash ) )
				{
					AddMaterialToExport( saveHandle, uMaterialHash );
				}
				else
				{
					AddMessage( MT_ERROR, "Unable to find material '%s' listed in always-export file", uMaterialName );
				}
			}
			delete[] pxMaterialNameList;
			pxMaterialNameList = NULL;
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
			// [13/11/2008 AS] Include the materials from the decal system
			int iNumBuffers = Tools_StaticDecal_System::GetNumDecalBuffers();
			for(int i=0; i<iNumBuffers; i++)
			{	
				Tools_StaticDecal_Buffer* pxBuffer = Tools_StaticDecal_System::GetDecalBufferByIndex(i);		
				const Asura_DecalManager_Material *pxMgrMtl = Asura_Decal_Manager::GetMaterial(pxBuffer->m_uDecalMaterial);
				if (pxMgrMtl && pxMgrMtl->m_uMaterialResponseHash
#ifdef ASURA_DECAL_MANAGER_USE_COMMON_ASR
					&& pxMgrMtl->m_bExportAsStaticDecal
#endif
					)
				{
					AddMaterialToExport(saveHandle, pxMgrMtl->m_uMaterialResponseHash);
				}
			}
#endif
		}

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
		if ( s_bIsCommonFileExport )
		{
			// Always include the default material in a level export
			AddMaterialToExport( saveHandle, ASURA_DEFAULT_MATERIAL_RESPONSE );
        }
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
		if ( s_bIsCommonFileExport )
		{
            // [13/11/2008 AS] Include the materials from the decal system
			for(int i=0; i<Asura_Decal_Manager::GetNumDecalMaterials(); i++)
			{
				const Asura_DecalManager_Material *pxMgrMtl = Asura_Decal_Manager::GetDecalMaterialByIndex(i);

				if (!pxMgrMtl->m_bExportAsStaticDecal && pxMgrMtl->m_uMaterialResponseHash)
				{
					AddMaterialToExport(saveHandle, pxMgrMtl->m_uMaterialResponseHash);
				}
			}
		}
        else
        {
			// [13/11/2008 AS] Anything included as common shouldnt go into any other files
			for(int i=0; i<Asura_Decal_Manager::GetNumDecalMaterials(); i++)
			{
				const Asura_DecalManager_Material *pxMgrMtl = Asura_Decal_Manager::GetDecalMaterialByIndex(i);

				if (!pxMgrMtl->m_bExportAsStaticDecal && pxMgrMtl->m_uMaterialResponseHash)
				{
					AddMaterialToIgnore(pxMgrMtl->m_uMaterialResponseHash);
				}
			}
		}
#endif

		CHECK( AddMaterialResponses( saveHandle ) );

	#endif // defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	if(s_eExportType == Export_Wii)
	{
		// Write out alpha textures
		Tools_Wii_Textures::EndExport(saveHandle);
	}
	if(IsAbortFlagSet()) goto End;

	//Animation tables
	AddAnimationTables(saveHandle);

	AddFilenameResources(saveHandle);

	#ifdef ASURA_USE_ATTRACTORS
	//Attractors normally get exported after the environment file , but this can't be done for mission
	//exports. So deal with it here in that case		
	if ( !IsAbortFlagSet() && MissionExport_IsExportingMissionOnly())
	{
		CHECK(AddAttractorData( saveHandle ));
	}
	#endif

#if defined ASURA_USE_REVERBVOLUME_SYSTEM && ASURA_USE_AUDIO_SYSTEM
	if( !IsAbortFlagSet() && bEnvironmentFile && !MissionExport_IsExportingMissionOnly() )
	{
		CHECK( AddReverbs( saveHandle ) );
	}
#endif

    // Add all the files in the list
	switch( s_eExportType )
    {
	case Export_PS3:
		for( CLIF<Asura_String> xFileLIF(&m_xAsuraFileList); !xFileLIF.done(); xFileLIF.next() )
		{
			if( IsEnvironmentFile( xFileLIF() ) )
			{
				continue;
			}
			else
			{
				CHECK( AddAsuraFile(saveHandle, xFileLIF(), u++) );
			}
		}
		break;
	default:
	    for( CLIF<Asura_String> xFileLIF(&m_xAsuraFileList); !xFileLIF.done(); xFileLIF.next() )
	    {
			Asura_String dumbtemp = xFileLIF();
		    CHECK( AddAsuraFile(saveHandle, xFileLIF(), u++) );		
	    }
		break;
    }

	#ifdef ASURA_USE_GUI_MENU_SYSTEM
	//Add menus. Needs to happen after textures ,and probably after sounds.
	CHECK(AddGUIMenus(saveHandle));
	#endif

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		EndMRPShaderExport( saveHandle );
	#endif

	//Add the terminator
   	{
	    int Terminator;
	    Terminator = 0;
	    BOOL bOK = WriteFile(saveHandle,&Terminator,4,&bytesWritten,0);
   	    if( !bOK || 4 != bytesWritten )
   	    {
   	        AddMessage( MT_ERROR, "I/O error." );
			SetAbortFlag();
   	        goto End;
   	    }
   	}

   	// Fill out the file info and resource file list chunks now.
   	u_int uAsuraFileSize;
   	if( saveHandle != INVALID_HANDLE_VALUE )
   	{
		#if defined ASURA_ASSERT
   			DWORD uCurFilePtrPos =
		#endif
			SetFilePointer( saveHandle, 0, NULL, FILE_CURRENT );

   	    DWORD uBeginningFilePtrPos = SetFilePointer( saveHandle, 0, NULL, FILE_BEGIN );
   	    DWORD uEndFilePtrPos = SetFilePointer( saveHandle, 0, NULL, FILE_END );
   	    Asura_Assert(uCurFilePtrPos == uEndFilePtrPos, "");
   	    uAsuraFileSize = uEndFilePtrPos - uBeginningFilePtrPos;
   	}
   	CHECK( FillOutFileInfoChunk(saveHandle, uAsuraFileSize) );
	CHECK( FillOutResourceFileListChunk(saveHandle) );
	
  	LogAnimationExportStats();


End:

	//done!
    if( saveHandle != INVALID_HANDLE_VALUE )
    {
	    CloseHandle(saveHandle);
    }


   	// Clear the progress message.
	UpdateStatus(Tools_Level_Export::EXPORT_STAGE_END, NULL, 0);

	if(!IsAbortFlagSet())
	{
		if(!IsCompressionDisabled() && ShouldCompressExportedFiles())
		{
			AddMessage(MT_INFORMATION,"Compressing %s",pszFilename);
			CompressFile(pszFilename);
		}
		
		AddMessage(MT_INFORMATION,"Finished writing %s.",pszFilename);
	}
	else
	{
		DeleteFile(pszFilename);
		AddMessage( MT_INFORMATION, "Aborted." );
	}


	g_uAsuraFilesInExport = 0;
	delete [] g_axResourceFileListEntries;
	g_axResourceFileListEntries = 0;


	//Export language files if needed
	ExportLanguageFiles(pszFilename);

    #undef CHECK

}

void Tools_Level_Export::LoadFilesInList()
{
	//load any non-environmental sounds
	BuildNonEnvironmentalSoundList(m_xAsuraFileList);
	AddNonEnvironmentalSounds(INVALID_HANDLE_VALUE);
	if(IsAbortFlagSet())return;

	//force sound events sounds to be loaded RAR 7/12/2006
	LoadSoundEventWavs();
	
	UpdateStatus(EXPORT_STAGE_ASURA_FILES, "Asura Files", 0);

    // Load asura files in list
	int i=0;
	int iNumFiles = m_xSeperatelyLoadedFiles.size()+m_xAsuraFileList.size();
	for( CLIF<Asura_String> xFileLIF(&m_xAsuraFileList); !xFileLIF.done() && !IsAbortFlagSet(); xFileLIF.next(),i++ )
	{
		s_iPercent = i*100/iNumFiles;
		//ignore files that have already been loaded
  		if(Tools_LoadedFileList::IsFileLoaded(xFileLIF().GetString())) continue;

		UpdateProgress();

		//check for an existing anim of the same name , and skip if it exists
		//(this is just to get round standard_rest_anim that gets created automatically , but aslo has a file)
		Asura_String xName;
		xFileLIF().SplitFilename(0,&xName,0);
		if(Asura_Animation_System::GetAnim(xName.GetString())) continue;

		LoadAsuraFile(xFileLIF().GetString());
	}

	for( CLIF<Asura_String> xFileLIF(&m_xSeperatelyLoadedFiles); !xFileLIF.done() && !IsAbortFlagSet(); xFileLIF.next(),i++ )
	{
		s_iPercent = i*100/iNumFiles;
		//ignore files that have already been loaded
		if(Tools_LoadedFileList::IsFileLoaded(xFileLIF().GetString())) continue;

		UpdateProgress();

		//check for an existing anim of the same name , and skip if it exists
		//(this is just to get round standard_rest_anim that gets created automatically , but aslo has a file)
		Asura_String xName;
		xFileLIF().SplitFilename(0,&xName,0);
		if(Asura_Animation_System::GetAnim(xName.GetString())) continue;

		LoadAsuraFile(xFileLIF().GetString());
	}

	UpdateStatus(EXPORT_STAGE_END, 0, 0);
}

const char* Tools_Level_Export::GetPlatformExtension( void )
{
    return Tools_Platform::GetPlatformExtension(s_eExportType);
}

void Tools_Level_Export::GetExportedInGameFilesFilename( const Asura_String& xEnvFilename, Asura_String* pxOutputFilename )
{
    // By default, just create a file called "<level name>i.<platform ext>"
    // in the same directory as the environment exports. Projects can
    // override this method as required.

    Asura_String xEnvName;
    xEnvFilename.SplitFilename( pxOutputFilename, &xEnvName, NULL );

    *pxOutputFilename += "\\";
    *pxOutputFilename += xEnvName;
    *pxOutputFilename += "i";
    *pxOutputFilename += GetPlatformExtension();
}

void Tools_Level_Export::AddInGameFilesToList( List<Asura_String>* pxList, const Asura_String& xEnvFilename )
{
    // add skins and anims for PhysicalObject entities
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::AddInGameFilesToList Tools_Entity_Object");

    	
    	for( Asura_Hashed_List_It<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList()); !xIt.Done(); xIt.Next() )
		{
    		Tools_Entity_Object* pxEntityObject = xIt.GetCurrent();

			if(s_bFilterEntityExport && !m_bFakeLevelExport)
			{
				if(s_xEntityClassesToIgnore.contains(pxEntityObject->Entity->GetClassification()))
				{
					continue;
				}
			}
			//don't bother adding files for entities that aren't exported to this platform
			if(!pxEntityObject->ShouldAddFilesToExportList()) continue;
    
				TOOLS_RESOURCEEXPORTLOG("Entity",pxEntityObject->GetName(),false);
    		    
    	    pxEntityObject->AddFilesToExportList( *pxList );
			//add any textures used by the entity to the export as well
			pxEntityObject->Entity->TraverseUsedTextures(AddEntityTextureToExportList, NULL);
    	}
	}

	//Don't need to do this, including PFX from .ini file is redundant
#ifndef ASURA_TURN_OFF_ALWAYSEXPORTINIFILES_SUPPORT
	// (AW - 20.10.2004) - Add particle templates that are always required
	if( !ShouldIgnoreAlwaysExportPFXFlag() )
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::AddInGameFilesToList PFX");
		Tools_Mode_PFX::AddAlwaysExportParticleTemplates( pxList );
	}
#endif //ASURA_USING_ALL_LEVEL_EXPORT


#ifdef ASURA_USE_FSFX
	// JGP - Single call
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::AddInGameFilesToList FSFX");
	Tools_Mode_FSFX::ExportTemplates( pxList );
	}
#endif //ASURA_USE_FSFX

   // add any extra files e.g. anims which are referenced by static messages
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::AddInGameFilesToList Messages");
    Tools_Message_Editing::AddFilesToExportList( *pxList );
	}

	// (MH - 27.02.2006) every project was calling this function, so I've moved it 
	// into this common location. It shouldn't matter if it's called twice.
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::AddInGameFilesToList CutsceneFiles");
	AddMainFileCutsceneFilesToList( *pxList );
	}

#ifdef ASURA_USE_DECAL_MANAGER
	if (!IsDoingFakeLevelExport())
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::AddInGameFilesToList Decals");
		// Note: *All* decal textures are always exported. Could be improved, let the project-side
		// export code flag the decal templates for export/no-export, as we don't need to export decals
		// only used for weapons/surface types not in the level

		// Add the decal textures
		for(int i=0; i<Asura_Decal_Manager::GetNumDecalMaterials();i++)
		{
			Asura_DecalManager_Material *pxMtl = Asura_Decal_Manager::GetDecalMaterialByIndex(i);
			
#ifdef ASURA_DECAL_MANAGER_USE_COMMON_ASR
			bool bExportTextures = pxMtl->m_bExportAsStaticDecal;
#else
			bool bExportTextures = true;
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
			if (pxMtl->m_bExportAsStaticDecal)
			{
				int iNum = Tools_StaticDecal_System::GetNumDecalBuffers();
				bool bFound = false;

				for(int j=0;j<iNum;j++)
				{
					Tools_StaticDecal_Buffer *pxBuffer = Tools_StaticDecal_System::GetDecalBufferByIndex(j);
					
					if (pxBuffer->m_uDecalMaterial == pxMtl->m_uMtlNameHash)
					{
						bFound = true;
					}
				}

				// If the material isn't used in this level, don't export these textures
				if (!bFound)
				{
					bExportTextures = false;
				}
				
			}
#endif

			if (bExportTextures)
			{			
#if !defined(ASURA_TARGET_SM3)
				// Texture
				int iTex = Asura_Texture_Management::GetIndexFromHashID(Asura_GetHashID_IgnoreExtension(pxMtl->m_xTextureName.GetString()));
				if (iTex!=ASURA_TEXTURE_NONE)
				{	
					Tools_Level_Export::AddTextureToExportList(iTex);
				}
#endif

			}
		}

#ifndef ASURA_DECAL_MANAGER_USE_COMMON_ASR
		// And the decal definitions
		Tools_FileSystem::EnsureDirectoryExists(Tools_Level_Export::GetTempDirectory());
		Asura_String xExportFileName = Tools_Level_Export::GetTempDirectory();
		xExportFileName += "\\decals.asr";
		if (Tools_Level_Export::ShouldExportBigEndian())
		{		
			xExportFileName += ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
		}
		Asura_Chunk_Decals::Save(xExportFileName.GetString());
		AddFileToExportList(*pxList, xExportFileName, false);
#endif // ASURA_DECAL_MANAGER_USE_COMMON_ASR

#ifdef ASURA_USE_BOTANICALS
		// This exports the botanical collision meshes
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Mode_Botanicals::AddCollisionMeshesToExportList");
			Tools_Mode_Botanicals::AddCollisionMeshesToExportList( pxList );
		}
#endif //ASURA_USE_FSFX

	}
#endif

}


void Tools_Level_Export::GetExportedCutsceneFilename( const Asura_String& xEnvFilename, const Asura_String& xCutsceneName, Asura_String* pxOutputFilename )
{
    // By default, just create a file called "<cutscene name>.<platform ext>"
    // in the same directory as the environment exports. Projects can
    // override this method as required.

    xEnvFilename.SplitFilename( pxOutputFilename, NULL, NULL );
    *pxOutputFilename += "\\";
    *pxOutputFilename += xCutsceneName;
    *pxOutputFilename += GetPlatformExtension();
}

void Tools_Level_Export::AddMainFileCutsceneFilesToList(List<Asura_String>& xNameList)
{
	//add all the names of all the animation and skin files required by the cutscenes
	for(int i=0;i<Asura_Cutscene_System::GetNumberOfCutscenes();i++)
	{
		Asura_Cutscene* pxCutscene = Asura_Cutscene_System::GetCutsceneByIndex(i);
		if(s_bDontExportCutscenes)
		{
			if(!(pxCutscene->m_iCutsceneFlags & AsuraCutsceneFlag_AlwaysIncludeInExport))
			{
				//skip all cutscenes that aren't marked as always export
				continue;
			}
		}

		//check don't export to PS2 flag
		if(!ShouldCutsceneBeExportedToThisPlatform(pxCutscene))
		{
			continue;
		}

		if(!pxCutscene->ShouldAddResourcesToExport()) continue;


		AddCutsceneFilesToList(xNameList,pxCutscene);
	}	

	//add any extra textures used by the cutscenes
	Asura_NameTree xTextureTree;
	Asura_Cutscene_System::BuildUsedTextureTree(&xTextureTree);
	if(xTextureTree.GetNumberOfNodes())
	{
		const Asura_String** ppxNameList = new const Asura_String*[xTextureTree.GetNumberOfNodes()];
		xTextureTree.MakeNameList(ppxNameList);
		for(int i=0;i<xTextureTree.GetNumberOfNodes();i++)
		{
			if(ppxNameList[i]->IsBlank()) continue;
			int iIndex = Asura_Texture_Management::GetTextureIndex(ppxNameList[i]->GetString());
			if(iIndex != ASURA_TEXTURE_NONE)
			{
				AddTextureToExportList(iIndex);
			}
		}
		delete [] ppxNameList;
	}
}

void Tools_Level_Export::AddCutsceneFilesToList(List<Asura_String>& xNameList,Asura_Cutscene* pxCutscene)
{
	Asura_SimpleLogfileOutput( ( s_szLogPath, "Cutscene '%s'\n", pxCutscene->GetName().GetString() ) );
	TOOLS_RESOURCEEXPORTLOG("Cutscene",pxCutscene->GetName().GetString(),false);


	u_int uSharedCutsceneSize = 0;
	u_int uAddedCutsceneSize = 0;

	List<Asura_String> xCheckedNameList;

	Asura_Assert(pxCutscene,"");
	for(int j=0;j<2;j++)
	{
		Asura_NameList* pxNames = j ? &pxCutscene->SkinFileNames : &pxCutscene->AnimationFileNames;
		for(int k=0;k<pxNames->GetNumEntries();k++)
		{
			Asura_String& xName = pxNames->GetName(k);
			if(xName.IsBlank()) continue;

			// check we should add these anim/skin files
			if( (j == 0) && !pxCutscene->ShouldAddAnimToExport( xName, k ) ) continue;
			if( (j == 1) && !pxCutscene->ShouldAddSkinToExport( xName, k ) ) continue;

			const Asura_String* pxName = &xName;

			if(xCheckedNameList.contains(*pxName)) continue;

			#ifdef ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES

			//See if the skin/anim has platform specific versions
			Asura_String xStrippedName;
			Asura_String xPlatformFilename;
			xName.SplitFilename(0,&xStrippedName,0);


			Asura_Hash_ID uStrippedHash = Asura_GetHashID(xStrippedName.GetString());
			if(Asura_HashedFilename_System::DoesStrippedFilenameHavePlatformVersions(uStrippedHash))
			{
				//Get the platform version , so that can be added instead
				const char* szPlatformFilename = Asura_HashedFilename_System::GetFullFilename(uStrippedHash,false);
				if(szPlatformFilename)
				{
					xPlatformFilename = szPlatformFilename;
					pxName = &xPlatformFilename;
				}
			}
			#endif //ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES


			//Complain about any files that don't exist at the specified location
			if(!Tools_FileSystem::FileExists_CheckNetwork(pxName->GetString()))
			{
				AddMessage(MT_ERROR, "Cutscene %s uses %s %s that couldn't be found",pxCutscene->GetName().GetString(),j ? "skin" : "anim", pxName->GetString());
				continue;
			}

			if(!xNameList.contains(*pxName))
			{
				AddFileToList(xNameList, *pxName, (j == 1));

				u_int uFileSize = 0;
				if(Tools_FileSystem::GetSizeOfFile(pxName->GetString(), uFileSize))
				{
					uAddedCutsceneSize += uFileSize;
				}

				Asura_SimpleLogfileOutput( ( s_szLogPath, "\tAdding '%s', %u bytes\n", pxName->GetString(), uFileSize ) );
			}
			else
			{
				u_int uFileSize = 0;
				if(Tools_FileSystem::GetSizeOfFile(pxName->GetString(), uFileSize))
				{
					uSharedCutsceneSize += uFileSize;
				}

				Asura_SimpleLogfileOutput( ( s_szLogPath, "\tSharing '%s', %u bytes\n", pxName->GetString(), uFileSize ) );
			}

			xCheckedNameList.add_entry(*pxName);
		}
	}

	Asura_SimpleLogfileOutput( ( s_szLogPath, "%u bytes added, %u bytes shared, %u bytes total\n", uAddedCutsceneSize, uSharedCutsceneSize, uAddedCutsceneSize + uSharedCutsceneSize ) );

	pxCutscene->AddExtraFilesToList( xNameList );
}

bool Tools_Level_Export::ShouldCutsceneBeExportedToOwnFile(Asura_Cutscene* pxCutscene)
{
	Asura_Assert(pxCutscene,"");
	if(pxCutscene->m_iCutsceneFlags & AsuraCutsceneFlag_ForceIntoMainFile) return false;

	//check to see if cutscene takes control of the camera
	for(int i=0;i<pxCutscene->NumberOfActors;i++)
	{
		if(pxCutscene->Actors[i]->IsCamera()) return true;
	}

	return false;
}

bool Tools_Level_Export::ShouldCutsceneBeExportedToThisPlatform(Asura_Cutscene* pxCutscene)
{
	if((s_eExportType == Export_PS2) && (pxCutscene->m_iCutsceneFlags & AsuraCutsceneFlag_DontExportToPS2)) return false;
	return true;

}

void Tools_Level_Export::AddDirToExportList(List<Asura_String>& xNameList,const char* pszDirectory, Asura_String xFilter/*="*.asr"*/, bool bWarnOnEmpty /* = true */)
{
	This->AddDirToExportListMain(xNameList,pszDirectory,xFilter,bWarnOnEmpty);
}

void Tools_Level_Export::AddDirToExportListMain(List<Asura_String>& xNameList,const char* pszDirectory, Asura_String xFilter/*="*.asr"*/, bool bWarnOnEmpty /* = true */)
{
	List<Asura_String> xFileList;

	char szDirToUse[MAX_PATH + 1];
	Asura_FileAccess::PlatformGetNetworkDir( pszDirectory, szDirToUse, MAX_PATH + 1 );

	//read files from export list for directory if there is one
	bool bGotExportList = ProcessExportList( szDirToUse, &xFileList, &xFileList );

	if( !bGotExportList )
	{
		//otherwise just get all asura files in this directory and below
		char szSearch[MAX_PATH + 1];
		Asura_PrintfToBfr( szSearch, "%s\\%s", MAX_PATH + 1, true, szDirToUse, xFilter.GetString() );      
		Tools_FileSystem::FindFiles( szSearch, xFileList, TOOLS_FILEFIND_RECURSIVE|TOOLS_FILEFIND_IGNOREBACKUPDIR );
	}

	if(!xFileList.size() && bWarnOnEmpty)
	{
		AddMessage(MT_WARNING, "Tried to add dir \"%s\" to the export list, but it's empty.", pszDirectory);
	}

	#ifdef ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES
	//strip out files belonging to other platforms. RAR 25/04/2007
	RemoveFilesForOtherPlatforms(xFileList);
	#endif


	//add all these to the export list
	Tools_AsuraFileContents xFileContents;
	for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next())
	{

		bool bIsSkinFile = false;

		xFileContents.LoadAsuraFile(xFileLIF().GetString());	
		bIsSkinFile = xFileContents.GetSkinList()->size()!=0;

		const char* pszRelativeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( xFileLIF().GetString() );
		AddFileToList(xNameList,pszRelativeName,bIsSkinFile);
	}
}

void Tools_Level_Export::RemoveFilesForOtherPlatforms(List<Asura_String>& xFileList)
{
	Asura_StringTree xDePlatformedFileTree;
	
	//Go through and remove any files that aren't for the current platform, and find the platform specific files for this platform
	for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();)
	{
		char szBuffer[MAX_PATH];
		Asura_Assert(sizeof(szBuffer) > xFileLIF().GetLength(),"Buffer not large enough for filename");

		//Only want to check the relative part of the path for the platform names
		//RAR 18/09/2007
		const char* szRelativePath = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir(xFileLIF().GetString());
		int iRelativePathOffset = szRelativePath - xFileLIF().GetString();
		strcpy(szBuffer,xFileLIF().GetString());
		
		int iPlatform = Tools_Platform::RemovePlatformFromPath(szRelativePath,&szBuffer[iRelativePathOffset]);
		if(iPlatform != -1)
		{
			if(iPlatform != s_eExportType)
			{
				xFileLIF.delete_current();
				continue;
			}
			else
			{
				//Add the stripped platform filename to the name tree
				int iJunk;
				xDePlatformedFileTree.FindNodeIndexByName(szBuffer,true,iJunk);
			}
		}
		xFileLIF.next();
	}

	if(xDePlatformedFileTree.GetNumberOfNodes())
	{
		//Go through the file list again , and see if there are any non-platform specific entries that also had
		//platform entries for the current platform
		for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();)
		{
			int iJunk;
			if(xDePlatformedFileTree.FindNodeIndexByName(xFileLIF().GetString(),false,iJunk))
			{
				xFileLIF.delete_current();
			}
			else
			{
				xFileLIF.next();
			}
		}
	}
}

void Tools_Level_Export::AddFileToIgnore(const char* pszFilename)
{
	for(LIF<Asura_String> xIgnoreLIF(&m_xFilesToIgnore);!xIgnoreLIF.done();xIgnoreLIF.next())
	{
		if(!stricmp(xIgnoreLIF().GetString(),pszFilename))
		{
			//aready added
			return;
		}
	}

	m_xFilesToIgnore.add_entry(pszFilename);
	
	//remove this file if it has already been added
	for(LIF<Asura_String> xFileLIF(&m_xAsuraFileList);!xFileLIF.done();)
	{
		const char* pszRelativeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( xFileLIF().GetString());
		if(!stricmp(pszFilename,pszRelativeName))
		{
			//don't want this file... delete from skin tree  , and asura file list
			int iJunkIndex;
			m_pxSkinFileList->DeleteNodeByName(xFileLIF(),iJunkIndex);

			m_pxAnimFileList->DeleteNodeByName(xFileLIF(),iJunkIndex);
	
			//delete from file list
			xFileLIF.delete_current();
		}
		else
		{
			xFileLIF.next();
		}
	}
}


void Tools_Level_Export::AddSeperatelyLoadedFile(const char* pszFilename)
{
	//make sure the file hasn't already been added
	for(LIF<Asura_String> xNameLIF(&m_xSeperatelyLoadedFiles);!xNameLIF.done();xNameLIF.next())
	{
		if(!xNameLIF().Compare(pszFilename))
		{
			//already in list
			return;
		}
	}

	m_xSeperatelyLoadedFiles.add_entry(pszFilename);
}

//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Tools_Level_Export::AddFileToExportList().
// PARAMETERS   xNameList   - list to be added to
//              xName       - filename
//              bIsSkinFile - whether skin/hierarchy or other type of asura file
// RETURNS      None.
// DESCRIPTION  static function for entity system to add required files to 
//              levels when saving / exporting.
// AUTHOR       Mike Healey - 19.10.2002.
//
//////////////////////////////////////////////////////////////////////////////
bool Tools_Level_Export::AddFileToExportList(List<Asura_String>& xNameList, const Asura_String& xName, bool bIsSkinFile)
{
    return This->AddFileToList(xNameList, xName, bIsSkinFile);
}
// End function Tools_Level_Export::AddFileToExportList().

bool Tools_Level_Export::AddFileToExportList( const Asura_String& xName, bool bIsSkinFile)
{
    return This->AddFileToList(This->m_xAsuraFileList, xName, bIsSkinFile);
}


bool Tools_Level_Export::AddFileToList(List<Asura_String>& xNameList, const Asura_String& xInputName, bool bIsSkinFile)
{
	if(xInputName.IsBlank()) return false;

	bool bAdd =true;
	
	Asura_String xName = xInputName;
	// JAT 04/10/07 - test for a bad input filename (can come from elsewhere e.g. cutscene data)
	if ( ( xName.StartsWith( ".\\" ) == true ) || ( xName.StartsWith( "./" ) == true ) )
	{
		// JAT 04/10/07 - assert if a bad input filename has been given
		Asura_Assertf( false, ( "AddFileToList Bad filename '%s' shouldn't start with ./ or .\\ fixing it up automatically", xName.GetString() ) );
		// JAT 04/10/07 remove any leading ./ or .\ as well
		if ( xInputName.StartsWith( "./" ) )
		{
		 xName = xInputName.GetString() + 2;
		}
		if ( xInputName.StartsWith( ".\\" ) )
		{
		 xName = xInputName.GetString() + 2;
		}
	}

	//make sure the asura file isn't in one of the directories to ignore
	const char* pszRelativeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( xName.GetString() );
	for(LIF<Asura_String> xIgnoreLIF(&m_xDirsToIgnore);!xIgnoreLIF.done();xIgnoreLIF.next())
	{
		if(!strnicmp(xIgnoreLIF().GetString(),pszRelativeName,xIgnoreLIF().GetLength()))
		{
			//don't bother adding this file then
			return false;
		}		
	}
	//or files to ignore
	for(LIF<Asura_String> xIgnoreLIF(&m_xFilesToIgnore);!xIgnoreLIF.done();xIgnoreLIF.next())
	{
		if(!stricmp(xIgnoreLIF().GetString(),pszRelativeName))
		{
			//don't bother adding this file then
			return false;
		}		
	}

	//check for project file ignoreing
	if(Project_ShouldIgnoreFile(xName.GetString())) return false;


	//Make sure this filename isn't already contained in the list
	//Compare filenames without path , to avoid files that have been added from several places
    const char* szNewFile = Tools_FileSystem::StripPath( xName.GetString() );
	int iIndex;
	if(bIsSkinFile)
	{
		bAdd = !m_pxSkinFileStringTree->FindNodeIndexByName(szNewFile,true,iIndex);
	}
	else
	{
		bAdd = !m_pxAsuraFileStringTree->FindNodeIndexByName(szNewFile,true,iIndex);
	}


	if(bAdd)
	{
#ifdef ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES
		//See if the skin/anim has platform specific versions
		Asura_String xStrippedName;
		Asura_String xPlatformFilename;
		xName.SplitFilename(0,&xStrippedName,0);

		Asura_Hash_ID uStrippedHash = Asura_GetHashID(xStrippedName.GetString());
		if(Asura_HashedFilename_System::DoesStrippedFilenameHavePlatformVersions(uStrippedHash))
		{
			//Get the platform version , so that can be added instead
			const char* szPlatformFilename = Asura_HashedFilename_System::GetFullFilename(uStrippedHash,false);
			if(szPlatformFilename)
			{
				xName = szPlatformFilename;
			}
		}
#endif //ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES

		AsuraFileType eFileType = AsuraFile_Unknown;
		//Check the file type , since fragments need to be handled differently.
		//Fake level exports just load the file regardless of type ,so don't bother
		//with the check in that case
		//RAR 07/06/2007

		if(!m_bFakeLevelExport)
		{
			eFileType = Tools_AsuraFileContents::GetAsuraFileType(xName.GetString());
		}
		
		if(eFileType == AsuraFile_Fragment)
		{
			//non animating fragments are a pain , and currently get dealt with by the shape file list
			AddShapeFileToExportList(xName.GetString());
		}
		else
		{
		xNameList.add_entry(xName);
		}

		if (eFileType == AsuraFile_Skin || eFileType == AsuraFile_AnimFragment)
		{
			//also add to the list of skin files
			TOOLS_RESOURCEEXPORTLOG("Skin", xName.GetString(), true);

			AddFileToSkinList(xName);
		}
	
		if(!m_bFakeLevelExport)
		{
			if(eFileType == AsuraFile_Anim || eFileType == AsuraFile_AnimFragment)
		{
				TOOLS_RESOURCEEXPORTLOG("Anim", xName.GetString(), true);
			AddFileToAnimList(xName);
		}
		}
	}
	return bAdd;
}

bool Tools_Level_Export::ProcessExportList( const char* szDirectory, List<Asura_String>* pxSkinFileList, List<Asura_String>* pxAnimFileList )
{
	char szSearch[MAX_PATH];

    // Look for file "export_list.txt" in directory.
	Asura_PrintfToBfr( szSearch, "%s\\export_list.txt", MAX_PATH, true, szDirectory );
    HANDLE hListFile = CreateFile( szSearch, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL );
    if( INVALID_HANDLE_VALUE == hListFile )
    {
//        AddMessage( MT_WARNING, "WARNING: Export list file not found for directory \"%s\".", szDirectory );
        return false;
    }

    const DWORD uFileSize = GetFileSize( hListFile, NULL );
    if( INVALID_FILE_SIZE == uFileSize )
    {
        AddMessage( MT_ERROR, "Failed to get size of list file. Reason: %s", Tools_OS::GetLastSystemError() );
        SetAbortFlag();
		CloseHandle(hListFile);
        return false;
    }

    char* const acFileListBuffer = new char[uFileSize];
    if( !acFileListBuffer )
    {
        AddMessage( MT_ERROR, "Out of memory." );
        SetAbortFlag();
		CloseHandle(hListFile);
        return false;
    }

    DWORD uNumBytesRead;
    BOOL bReadOK = ReadFile( hListFile, acFileListBuffer, uFileSize, &uNumBytesRead, NULL );
    if( !bReadOK || uFileSize != uNumBytesRead )
    {
        AddMessage( MT_ERROR, "Failed to read list file. Reason: %s", Tools_OS::GetLastSystemError() );
        delete[] acFileListBuffer;
        SetAbortFlag();
		CloseHandle(hListFile);
        return false;
    }
	CloseHandle(hListFile);


    char szNextFile[MAX_PATH];
    const u_int uNumCharsInDir = Asura_PrintfToBfr( szNextFile, "%s\\", MAX_PATH, true, szDirectory );
    char* const szFileStart = &szNextFile[0] + uNumCharsInDir;

    bool bEncounteredSkinFile = false;
    bool bError               = false;

    const char* szIt = &acFileListBuffer[0];
    for( int iCharsLeft = uFileSize; !bError && iCharsLeft-- >= 0; )
    {
        // Skip leading whitespace.
        while( iCharsLeft >= 0 && isspace(*szIt) )
        {
            ++szIt;
            --iCharsLeft;
        }
        if( iCharsLeft < 0 ) { break; }

        const char* szLineStart = szIt;

        // Find newline.
        while( iCharsLeft >= 0 && '\n' != *szIt )
        {
            ++szIt;
            --iCharsLeft;
        }

        const char* szLineEnd = szIt++;

        // Skip trailing whitespace.
        while( szLineEnd > szLineStart && isspace(*szLineEnd) )
        {
            --szLineEnd;
        }

        bool bIsSkin = false;

        // Is there a '=' character in the line?
        for( const char* szEq = szLineStart; szEq <= szLineEnd; ++szEq )
        {
            if( '=' == *szEq )
            {
                bool bReportErrorAtThisLine = false;
                if( szEq > szLineStart && szEq < szLineEnd )
                {
                    const int iNumChars = szEq - szLineStart;
                    if( 4 == iNumChars && !strnicmp(szLineStart, "skin", 4) )
                    {
                        if( bEncounteredSkinFile )
                        {
                            AddMessage( MT_ERROR, "Error in export_list.txt: multiple skin entries." );
                            bError = true;
                        }
                        else
                        {
                            bIsSkin = true;
                            bEncounteredSkinFile = true;
                            szLineStart = szEq + 1;
                        }
                    }
                    else
                    {
                        bReportErrorAtThisLine = true;
                    }
                }
                else
                {
                    bReportErrorAtThisLine = true;
                }

                if( bReportErrorAtThisLine )
                {
                    AddMessage( MT_ERROR, "Error in export_list.txt. See log for details of location." );

                    int iNumCharsToCopy = (szLineEnd + 1) - szLineStart;
                    char* szCopy = new char[iNumCharsToCopy + 1];
                    if( szCopy )
                    {
                        strncpy( szCopy, szLineStart, iNumCharsToCopy );
                        szCopy[iNumCharsToCopy] = '\0';

		                Asura_SimpleLogfileOutput(( s_szLogPath,      "%s\n", szCopy ));
		                Asura_SimpleLogfileOutput(( s_szErrorLogPath, "%s\n", szCopy ));

                        delete[] szCopy;
                    }
                   
                    bError = true;
                }

                break;
            }
        }
        // End loop searching for '=' char in line.

        // Add to list.
        if( !bError && szLineEnd > szLineStart )
        {
            int iNumChars = Asura_Maths::Min( (szLineEnd + 1) - szLineStart, static_cast<int>(MAX_PATH - (uNumCharsInDir + 1)) );
            strncpy( szFileStart, szLineStart, iNumChars );
            *(szFileStart + iNumChars) = '\0';

            if( bIsSkin )
            {
                pxSkinFileList->add_entry_end( szNextFile );
            }
            else
            {
                pxAnimFileList->add_entry_end( szNextFile );
            }
        }
    }
    // End loop through file buffer.

    delete[] acFileListBuffer;

    if( !bError && bEncounteredSkinFile != !DirectoryIsAnimOnly(szDirectory) )
    {
        if( bEncounteredSkinFile )
        {
            AddMessage( MT_ERROR, "Error in export_list.txt: skin entry for anim only directory." );
        }
        else
        {
            AddMessage( MT_ERROR, "Error in export_list.txt: no skin entry." );
        }
        bError = true;
    }

    if( bError )
    {
        SetAbortFlag();
    }
    return !bError;
}

void Tools_Level_Export::CompressFile(const char* filename)
{
	const u_int uUncompressedFileSize = Asura_FileAccess::GetFileSize( filename );
	if(s_eExportType == Export_Xbox360)
	{
		//Xbox360 exports make use of the LZX compression instead
		//RAR 07/01/2008
		Tools_Xbox360_Export::CompressFile(filename);
	}
	else
	{
		const bool bBigEndian = ShouldExportBigEndian ();

#ifdef ASURA_WRITE_ZLIB
		if(!Tools_Zlib::CompressFile(filename, bBigEndian, true))
		{
			Asura_Assertf(IsAbortFlagSet(), ("Failed to compress %s",filename));
			
		}
#else	
		//open the file
		bool bSuccess = false;

		HANDLE handle = INVALID_HANDLE_VALUE;
		char* buffer = NULL;
		Asura_Huffman_CompressedFileHdr* pxFileHdr = NULL;

		// DLH - 18 April 2007 13:57:59 - Poor mans try{}catch{}
		do
		{
			handle = CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
			if(handle == INVALID_HANDLE_VALUE)
			{
				Asura_Assertf( false, ("File '%s' could not be openned for compression", filename));
				break;
			}

			//allocate a buffer for it , and read it in
			int fileSize = GetFileSize(handle,0);
			buffer = new char[fileSize];

			if( NULL == buffer)
			{
				Asura_Assertf( false, ("File '%s' could not allocate %d bytes for compression", filename, fileSize));
				break;
			}

			unsigned long bytesRead;
			if( 0 == ReadFile(handle,buffer,fileSize,&bytesRead,0))
			{
				Asura_Assertf( false, ("File '%s' failed to read for compression", filename));
				break;
			}

			CloseHandle(handle);
			handle = INVALID_HANDLE_VALUE;

			u_int uNewSize;
			bool	bCompressedOK	= Asura_Huffman_Encoder::AllocateAndCompressForASR( buffer, fileSize, &pxFileHdr, &uNewSize, bBigEndian );

			if( !bCompressedOK ) 
			{ 
				Asura_Assertf( false, ("File '%s' failed to compression", filename));
				break; 
			}

			//open the file and write out the compressed version
			handle = CreateFile(filename,GENERIC_WRITE,0,0,CREATE_ALWAYS,0,0);
			if(handle == INVALID_HANDLE_VALUE)
			{
				Asura_Assertf( false, ("File '%s' could not be openned for compression output", filename));
				break;
			}

			if( 0 == WriteFile( handle, pxFileHdr, uNewSize, &bytesRead, 0 ))
			{
				Asura_Assertf( false, ("File '%s' failed to write for compression", filename));
				break;
			}

			bSuccess = true;
		}
		while( false);

		// DLH - 18 April 2007 13:53:55 - Delete the resources
		if( INVALID_HANDLE_VALUE != handle)
		{
			CloseHandle(handle);
		}

		delete[] buffer;

		if( NULL != pxFileHdr)
		{
			Asura_Deallocate( pxFileHdr );
		}
#endif
	}

	Tools_Level_Export::UpdateStatus(Tools_Level_Export::EXPORT_STAGE_END, NULL, 0);


	const u_int uCompressedFileSize = Asura_FileAccess::GetFileSize( filename );
	CompressFileReport( filename, uUncompressedFileSize, uCompressedFileSize );
}

static Asura_String64 GetFormattedFileSize( const u_int uSize )
{
	static const u_int uKB = 1024;
	static const u_int uMB = 1024 * 1024;
	static const u_int uGB = 1024 * 1024 * 1024;

	u_int uWhole;
	u_int uRemainder;

	Asura_String64 xSize;
	if( uSize >= uGB )
	{
		uWhole = uSize / uGB;
		uRemainder = ( ( ( ( uSize % uGB ) / uMB ) * 100 ) / uKB );
		xSize.Print( "%u.%02uGB", uWhole, uRemainder );
	}
	else if ( uSize >= uMB )
	{
		uWhole = uSize / uMB;
		uRemainder = ( ( ( ( uSize % uMB ) / uKB ) * 100 ) / uKB );
		xSize.Print( "%u.%02uMB", uWhole, uRemainder );
	}
	else if ( uSize >= uKB )
	{
		uWhole = uSize / uKB;
		uRemainder = ( ( ( uSize % uKB ) * 100 ) / uKB );
		xSize.Print( "%u.%02uKB", uWhole, uRemainder );
	}
	else
	{
		xSize.Print( "%u", uSize );
	}
	return( xSize );
}

void Tools_Level_Export::CompressFileReport( const char* filename, const u_int uUncompressedFileSize, const u_int uCompressedFileSize )
{
	Asura_String64 xUncompressedFileSize = GetFormattedFileSize( uUncompressedFileSize );
	Asura_String64 xCompressedFileSize = GetFormattedFileSize( uCompressedFileSize );

	if( uCompressedFileSize > uUncompressedFileSize )
	{
		AddMessage( MT_INFORMATION, "Compressed file %s is bigger after compression! (%s->%s)", filename, xUncompressedFileSize.GetString(), xCompressedFileSize.GetString() );
		return;
	}

	// scale them down, so we can work out the percentage the file has been reduced by - otherwise, we could wrap with 32bit sizes
	const u_int uMaxForPercent				= 100000;
	u_int uUncompressedFileSizeForPercent	= uUncompressedFileSize;
	u_int uCompressedFileSizeForPercent		= uCompressedFileSize;
	while( ( uUncompressedFileSizeForPercent > uMaxForPercent ) || ( uCompressedFileSizeForPercent > uMaxForPercent ) )
	{
		uUncompressedFileSizeForPercent /= 2;
		uCompressedFileSizeForPercent	/= 2;
	}
	float fPercent = 99.0f;
	if( uUncompressedFileSizeForPercent != 0 )
	{
		fPercent = (float)( ( ( uUncompressedFileSizeForPercent - uCompressedFileSizeForPercent ) * 10000 ) / uUncompressedFileSizeForPercent );
		fPercent /= 100.0f;
	}

	AddMessage( MT_INFORMATION, "File '%s' reduced by %.2f%% (%s->%s)", filename, fPercent, xUncompressedFileSize.GetString(), xCompressedFileSize.GetString() );
}

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
void Tools_Level_Export::AddMaterialToExport(HANDLE xSaveHandle, const Asura_Hash_ID uMaterialID, Asura_HashTree& xMaterialExportTree, Asura_HashTree& xTextureExportTree, bool bIncludeTextures)
{
	const Tools_MaterialResponse* const pxMaterial = Tools_MaterialResponse_System::GetMaterialByHashID( uMaterialID );
			
	if(pxMaterial == NULL)
	{
		Asura_Assert(false, "Failed to find material to add to export.");
		int iJunk;
		if (uMaterialID && !g_xMissingMaterialsEncountered.FindNodeIndexByHash(uMaterialID, true, iJunk))
		{
			AddMessage( MT_ERROR, "Missing material: %s", Tools_MaterialResponse_System::GetMissingMaterialDescription( uMaterialID ) );
		}
	}
	else
	{
		int iJunk;
		if(bIncludeTextures)
		{
			//check against the tree of materials that have had their textures added
			const bool	bAlreadyInTextureTree = xTextureExportTree.FindNodeIndexByHash( uMaterialID, true, iJunk );

			if (!bAlreadyInTextureTree)
			{
				// add all loaded textures used in the material to the export 
				for ( u_int u=0; u<Asura_MaterialResponse::NUMBER_OF_TEXTURE_SLOTS && !IsAbortFlagSet(); ++u )
				{
					int iTexture = pxMaterial->GetTexture( u );

					if ( iTexture != ASURA_TEXTURE_NONE )
					{
						const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTexture);
						if(!pxTextureName)
						{
							// texture isn't loaded in
						}	
						else
						{
							TOOLS_RESOURCEEXPORTLOG("Texture Slot",Tools_MaterialResponse_System::GetTextureSlotDescription(u),false);
							if(!AddTexture( xSaveHandle, *pxTextureName, iTexture, u == Asura_MaterialResponse::ALBEDO_TEXTURE || u == Asura_MaterialResponse::ENVIRONMENT_MAP ))
							{
								AddMessage(MT_ERROR,"Failed to add texture %s for material %s",pxTextureName->GetString(), pxMaterial->GetName().GetString());
							}
						}
					}
				}

			}
		}

		const bool	bAlreadyInMaterialTree = xMaterialExportTree.FindNodeIndexByHash( uMaterialID, true, iJunk );

		if (!bAlreadyInMaterialTree)
		{
			//Material was added , so force the resource export log to be updated now
			//Needed to get the resource logfile working correctly in cases where a material is being exported
			//without textures. RAR 07/01/2009
			Tools_ResourceExportLog::ForceOutputToLog();
			
			// Make sure our parent is added too, we need their response curves
			if( pxMaterial->GetParentHash() != ASURA_HASH_ID_UNSET )
			{
				TOOLS_RESOURCEEXPORTLOG("Parent Material","",false);
				//Textures aren't needed for parent materials. RAR 07/01/2009
				AddMaterialToExport( xSaveHandle, pxMaterial->GetParentHash(), false );
			}
		}

	}
}

void Tools_Level_Export::AddMaterialToExport( HANDLE xSaveHandle, const Asura_Hash_ID uMaterialID, bool bIncludeTextures /*=true*/ )
{
	// are we ready for exporting?
	if (!m_pxMaterialsToExport || !m_pxSlaveMaterialsToExport || !m_pxMaterialsWithExportedTexturesTree || !m_pxMaterialsToIgnore) return;

	const Tools_MaterialResponse* const pxMaterial = Tools_MaterialResponse_System::GetMaterialByHashID( uMaterialID );

	int iJunk;
	if(m_pxMaterialsToIgnore->FindNodeIndexByHash(uMaterialID, false, iJunk))
	{
		return;
	}
			
	// is this a valid material?
	if (!pxMaterial) 
	{
		if (uMaterialID && !g_xMissingMaterialsEncountered.FindNodeIndexByHash(uMaterialID, true, iJunk))
		{
			AddMessage( MT_ERROR, "Missing material: %s", Tools_MaterialResponse_System::GetMissingMaterialDescription( uMaterialID ) );
		}
		return;
	}

	Asura_HashTree* const pxExportTree = (pxMaterial->GetParentHash() == ASURA_HASH_ID_UNSET ? m_pxMaterialsToExport : m_pxSlaveMaterialsToExport);

	// add the material to the tree if it's not already there
	TOOLS_RESOURCEEXPORTLOG("Material",pxMaterial->GetName().GetString(),false);
	AddMaterialToExport(xSaveHandle, uMaterialID, *pxExportTree, *m_pxMaterialsWithExportedTexturesTree, bIncludeTextures);

	for(u_int u = 0; u < m_uNumMaterialResponseOverridesForExport && !IsAbortFlagSet(); u++)
	{
		const Asura_Hash_ID							uOverrideMapID	= m_pxMaterialResponseOverridesForExport[u];
		const Asura_MaterialResponse_OverrideMap*	pxOverrideMap	= Asura_MaterialResponse_System::GetOverrideMap(uOverrideMapID);
		const Asura_MaterialResponse_Override*		pxOverride		= (pxOverrideMap == NULL) ? NULL : pxOverrideMap->GetOverride(uMaterialID);
		const Asura_Hash_ID							uOverrideID		= (pxOverride == NULL) ? ASURA_HASH_ID_UNSET : pxOverride->GetOverrideHashID();

		if(uOverrideID != ASURA_HASH_ID_UNSET)
		{
			TOOLS_RESOURCEEXPORTLOG("Material Override",pxOverrideMap->GetName(),false);
			{
				TOOLS_RESOURCEEXPORTLOG("Material",pxOverride->GetOverrideName().GetString(),false);
				AddMaterialToExport(xSaveHandle, uOverrideID, *pxExportTree, *m_pxMaterialsWithExportedTexturesTree, bIncludeTextures);
			}
		}
	}
}

void Tools_Level_Export::AddMaterialToIgnore(Asura_Hash_ID uMaterialID)
{
	if(!uMaterialID) return;

	//already in ignore list?
	int iJunkIndex;
	if(m_pxMaterialsToIgnore->FindNodeIndexByHash(uMaterialID, true, iJunkIndex))
	{
		return;
	}
	const Tools_MaterialResponse* const	pxMaterial	= Tools_MaterialResponse_System::GetMaterialByHashID( uMaterialID );

	//remove from materials to be exported
	m_pxMaterialsToExport->DeleteNodeByHash(uMaterialID, iJunkIndex);
	m_pxSlaveMaterialsToExport->DeleteNodeByHash(uMaterialID, iJunkIndex);

	if(pxMaterial)
	{
		//add textures used by this material to the ignore list
		for ( u_int u=0; u<Asura_MaterialResponse::NUMBER_OF_TEXTURE_SLOTS; ++u )
		{
			int iTexture = pxMaterial->GetTexture( u );

			if ( iTexture != ASURA_TEXTURE_NONE )
			{
				const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTexture);
				if(pxTextureName)
				{
					AddTextureToIgnore(pxTextureName->GetString());
				}
			}
		}
		//add the parent material to the ignore list
		if(pxMaterial->GetParentHash())
		{
			AddMaterialToIgnore(pxMaterial->GetParentHash());
	}

		//Ignore the override materials?
		//Not sure at the moment whether that would be the correct behavior
	}
}

void Tools_Level_Export::BeginMRPShaderExport()
{
	switch( s_eExportType )
	{
		case Export_PS3:
		{
			Tools_PS3_ShaderExport::BeginShaderExport();
			break;
		}

		case Export_Xbox360:
		{
			Tools_Xbox360_Export::BeginMRPShaderExport();
			break;
		}

		case Export_PC:
		{
			Tools_PC_Export::BeginMRPShaderExport();
			break;
		}

		default:
		{
			break;
		}
	}
}

void Tools_Level_Export::EndMRPShaderExport( const HANDLE xExportHandle )
{
	switch( s_eExportType )
	{
		case Export_PS3:
		{
			Tools_PS3_ShaderExport::EndShaderExport( xExportHandle );
			break;
		}

		case Export_Xbox360:
		{
			Tools_Xbox360_Export::EndMRPShaderExport( xExportHandle );
			break;
		}

		case Export_PC:
		{
			Tools_PC_Export::EndMRPShaderExport( xExportHandle );
			break;
		}

		default:
		{
			break;
		}
	}
}

void Tools_Level_Export::AddMRPShader_Internal( const Asura_Hash_ID uMaterial1,
	const Asura_Hash_ID uMaterial2, const bool bSkin )
{
	// Combine the materials to get a shader hash

	const Asura_MaterialResponse* const pxMaterial1 =
		Asura_MaterialResponse_System::GetMaterialByHashID( uMaterial1 );

	const Asura_MaterialResponse* const pxMaterial2 =
		Asura_MaterialResponse_System::GetMaterialByHashID( uMaterial2 );

	const Asura_Hash_ID uShaderHash =
		Asura_MaterialResponse_System::CombineShaderHashes( pxMaterial1, pxMaterial2, bSkin );

	// Call the platform-specific function

	switch( s_eExportType )
	{
		case Export_PC:
		{
			Asura_Direct3D_MRPShaderCompile::AddMRPShader( uShaderHash, uMaterial1, uMaterial2 );
			break;
		}

		case Export_PS3:
		{
			// TODO: PS3 currently wants different calls for smooth skin vs. object etc. Might need
			// more parameters passed here than just bSkin (or an enum)
			Asura_Assert( false, "Shouldn't yet call this for PS3." );
			break;
		}

		case Export_Xbox360:
		{
			Tools_Xbox360_Export::AddMRPShader( uShaderHash );
			break;
		}

		default:
		{
			break;
		}
	}
}

void Tools_Level_Export::AddMRPShaders( const Asura_Hash_ID uMaterial1,
	const Asura_Hash_ID uMaterial2, const bool bSkin )
{
	// Add the shader necessary for the base materials without any overrides
	AddMRPShader_Internal( uMaterial1, uMaterial2, bSkin );

	// Create a list of the project's override maps
	const u_int uNumOverrideMaps = Asura_MaterialResponse_System::GetNumOverrideMaps();
	Asura_Hash_ID* puOverrideHashes = Asura_MaterialResponse_System::CreateOverrideMapHashArray();

	// For each one, add the shader necessary for the materials
	for( u_int u = 0; u < uNumOverrideMaps; ++u )
	{
		const Asura_MaterialResponse_OverrideMap* const pxOverrideMap =
			Asura_MaterialResponse_System::GetOverrideMap( puOverrideHashes[u] );

		Asura_Assertf( pxOverrideMap, ("Failed to get material override map with hash 0x%X.",
			puOverrideHashes[u]) );

		if( !pxOverrideMap ) continue;

		// Get the override settings for the base materials

		const Asura_MaterialResponse_Override* const pxOverride1 =
			pxOverrideMap->GetOverride( uMaterial1 );

		const Asura_MaterialResponse_Override* const pxOverride2 =
			pxOverrideMap->GetOverride( uMaterial2 );

		// Get the replacement materials' hash IDs

		const Asura_Hash_ID uFinalMaterial1 =
			(pxOverride1 ? pxOverride1->GetOverrideHashID() : uMaterial1);

		const Asura_Hash_ID uFinalMaterial2 =
			(pxOverride2 ? pxOverride2->GetOverrideHashID() : uMaterial2);

		// Add shader necessary for the replacement materials
		AddMRPShader_Internal( uFinalMaterial1, uFinalMaterial2, bSkin );
	}

	// Clean up
	Asura_MaterialResponse_System::DestroyOverrideMapHashArray( puOverrideHashes );
}

#endif // def ASURA_USE_MATERIAL_RESPONSE_SYSTEM

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
void Tools_Level_Export::AddPFXToIgnore(const Asura_Hash_ID uPFXToIgnore)
{
	Tools_Mode_PFX::AddPFXToIgnoreList(uPFXToIgnore);
}
#endif

void Tools_Level_Export::UpdateProgress()
{
	if(!s_bIsExport) return;
	
	Asura_Windows::CheckForWindowsMessages();
	Asura_UserInput::Update();
	Asura_Cursor::OncePerFrameUpdate();

	bool bRender = true;
	//Avoid rendering anything with the reference rasterizer , since it will slow down the process massively
	//This can happen if tools is started up while batch processing while the computer is locked.
	//RAR 14/04/2008
	if(Tools_BatchProcessing::IsConsoleMode() || Asura_Direct3D::IsReferenceRasterizer())
	{
		bRender = false;
	}

	if(bRender)
	{
	//make sure that we aren't in wireframe mode
	Asura_Render::CurrentStates.RequestWireFrameMode(false);


	Asura_Render::BeginFrame();
	//black out the screen
	float width = (float)Asura_Viewport::GetWidth();
	float height = (float)Asura_Viewport::GetHeight();
	Asura_Render::Overlay.SetRectXY(0,0,width,height);
	Asura_Render::Overlay.SetRectUV(0,0,1,1);
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Colour=0xff000000;
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	//show some text
	float print_x = width/2;
	float print_y = height/2;
	print_y-=4;
	Asura_Text_Output::ConvertPixelsToPrintCoords(print_x,print_y);
	Asura_Text_Output::Print(print_x,print_y++,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"%s",s_szMainHeading);

	if(!IsFakeLevelExport())
	{
		Asura_Text_Output::Print(print_x,print_y++,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Hold escape to abort");
	}

    ++print_y;
	if(!s_xHeading.IsBlank())
	{
        if( 0 <= s_iPercent && s_iPercent <= 100 )
        {
		    Asura_Text_Output::Print(print_x,print_y,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"%s %d%%",s_xHeading.GetString(),s_iPercent);
        }
        else
        {
		    Asura_Text_Output::Print(print_x,print_y,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"%s",s_xHeading.GetString(),s_iPercent);
        }
	}
    ++print_y;

	if(s_iNumErrors)
	{
        ++print_y;
		Asura_Text_Output::Print(print_x,print_y++,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"Errors : %d",s_iNumErrors);
	}

	Tools_Timed_Message_System::Render();
	Asura_Cursor::Render();

	
	Asura_Render::EndFrame();
	}
	else if(!s_xHeading.IsBlank())
	{
		//Show progress in the title instead
		char acTitle[200];

		if( 0 <= s_iPercent && s_iPercent <= 100 )
		{
			Asura_PrintfToBfr(acTitle,"%s - %s %d%%",sizeof(acTitle),true,Asura_Windows::GetDefaultWindowTitle(),s_xHeading.GetString(),s_iPercent);
		}
		else
		{
			Asura_PrintfToBfr(acTitle,"%s - %s",sizeof(acTitle),true,Asura_Windows::GetDefaultWindowTitle(),s_xHeading.GetString());
		}
		Asura_Windows::SetWindowTitle(acTitle);
	}

	// stage change
	if (s_eThisStage != s_eCurrentStage)
	{
		Asura_Assert(s_eThisStage != EXPORT_STAGE_UNDEFINED, "Invalid stage");

		if (s_eCurrentStage == EXPORT_STAGE_UNDEFINED)
		{
			m_xExportStageStatsList[EXPORT_STAGE_OVERALL].m_bUsed = true;
			// start overall timer
			m_xExportStageStatsList[EXPORT_STAGE_OVERALL].m_xTimeStart = Asura_Timers::GetCurTicks();
		}
		else
		{
			// clock time for previous stage
			m_xExportStageStatsList[s_eCurrentStage].m_xStageTime += Asura_Timers::GetCurTicks() - m_xExportStageStatsList[s_eCurrentStage].m_xTimeStart;
		}

		// start new stage
		m_xExportStageStatsList[s_eThisStage].m_bUsed = true;
		m_xExportStageStatsList[s_eThisStage].m_xTimeStart = Asura_Timers::GetCurTicks();
		s_eCurrentStage = s_eThisStage;
	}

	Asura_Timers::SetMaxFrameRate(0.0f);
	Asura_Timers::HandlePrimaryTimer();

	//Don't allow fake level exports (quit to game) to be aborted. RAR 25/01/2006
	if(!s_bUserAbort && !IsFakeLevelExport() && Asura_UserInput::KeyPressed(ASURA_KEYID_ESCAPE))
	{
		s_bUserAbort = true;
		AddMessage(MT_INFORMATION,"Aborting...");
		if(Tools_BatchProcessing::IsBatch())
		{
			Tools_BatchProcessing::Abort();
		}
	}

}

void Tools_Level_Export::DisplayProgressMessage(const char* szMessage)
{
	if( true == Tools_BatchProcessing::IsConsoleMode())
	{
		return;
	}
	
	if(Asura_Direct3D::HasFrameBegun())
	{
		//Half-way through rendering a frame already , so don't bother trying to do progress messages
		//in this case.
		return;
	}

#if defined( ASURA_USE_PROFILING_SYSTEM )
	// pause profiling to avoid assert
	ASURA_CPUPROFILE_SCOPED_PAUSE
#endif


	Asura_Windows::CheckForWindowsMessages();
	Asura_UserInput::Update();
	Asura_Cursor::OncePerFrameUpdate();

	//make sure that we aren't in wireframe mode
	Asura_Render::CurrentStates.RequestWireFrameMode(false);


	Asura_Render::BeginFrame();
	//black out the screen
	float fWidth = (float)Asura_Viewport::GetWidth();
	float fHeight = (float)Asura_Viewport::GetHeight();
	Asura_Render::Overlay.SetRectXY(0,0,fWidth,fHeight);
	Asura_Render::Overlay.SetRectUV(0,0,1,1);
	Asura_Render::Overlay.TranslucencyMode = ASURA_TRANSLUCENCY_OFF;
	Asura_Render::Overlay.TextureIndex = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Colour=0xff000000;
	Asura_Render::Overlay.Render();
	Asura_Render::FlushDisplay();

	//show some text
	float fPrintX = fWidth/2;
	float fPrintY = fHeight/2;
	Asura_Text_Output::ConvertPixelsToPrintCoords(fPrintX,fPrintY);
	Asura_Text_Output::Print(fPrintX,fPrintY++,0xffffffff,ASURA_TEXT_FORMAT_CENTREJUSTIFIED,"%s",szMessage);
//	Asura_Cursor::Render();

	Asura_Render::EndFrame();

	s_bDisplayingProgressMessage = true;
}

void Tools_Level_Export::ClearProgressMessage()
{
	if(s_bDisplayingProgressMessage)
	{
		DisplayProgressMessage("");
		s_bDisplayingProgressMessage = false;
	}
}

void Tools_Level_Export::AddMessage( MessageType type, const char* szFormat, ... )
{
	char buffer[255];

	// format string and place in buffer
    {
	    va_list xArgs;
	    va_start( xArgs, szFormat );
        Asura_PrintfToBfr( buffer, szFormat, xArgs, 255, true );
	    va_end( xArgs );
    }

	//get the colour based on message type
	int colour;
	const char* szMessageType = 0;
	switch(type)
	{
		case MT_WARNING:
			colour = 0xffff00;//yellow
			szMessageType = "Warning";
			break;
		case MT_ERROR :
			colour = 0xff0000;//red
			szMessageType = "Error";
			break;

		default :
			colour = 0xffffff;//white
			szMessageType = "Info";
			break;
	}

	if ( This )
	{
		if(type == MT_HEADING)
		{
			//store this string in case any other message is displayed
			s_xMessageDisplayHeading = buffer;
		}

		//add to the message list
		if((type != MT_LOGONLY) && (type != MT_HEADING) && (type != MT_LOGANDMESSAGE) && (type != MT_MESSAGELOGONLY))
		{
			if(!s_xMessageDisplayHeading.IsBlank())
			{
				Tools_Timed_Message_System::AddMessage(0xffffff,"%s",s_xMessageDisplayHeading.GetString());
				s_xMessageDisplayHeading.MakeNull();
			}
			Tools_Timed_Message_System::AddUnloggedMessage(colour,"%s",buffer);
		}
	}

	if(s_szLogPath)
	{
		if(type == MT_ERROR)
		{
			Asura_SimpleLogfileOutput((s_szLogPath,"**Error** %s\n",buffer));
			Asura_SimpleLogfileOutput((s_szErrorLogPath,"%s\n",buffer));
		  	RecordError();
		}
		else if (type != MT_MESSAGELOGONLY)
		{
			Asura_SimpleLogfileOutput((s_szLogPath,"%s\n",buffer));
		}
	}

	if ( This )
	{
		This->UpdateProgress();
	}

	// add to message log system
	if ( type != MT_LOGONLY && type != MT_HEADING)
	{
		Tools_MessageLog::AddMessage("Export",szMessageType,"%s",buffer);
	}
}

bool Tools_Level_Export::ConversionDataFileMatches(const char* textureFilename,const TextureConversionData* pxConvertData)
{
	char filename[MAX_PATH];
	unsigned long uBytesTransferred;
	TextureConversionData xSavedConvertData;
	memset(&xSavedConvertData,0,sizeof(xSavedConvertData));
	bool bOkay = false;

	strcpy(filename,textureFilename);

	//replace the fileextension
	char* dotPos = strrchr(filename,'.');
	if(dotPos)*dotPos = 0;
	strcat(filename,".dat");

	//open the file
	HANDLE handle = CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	if(handle==INVALID_HANDLE_VALUE) return false;

	//read in the version number
	ReadFile(handle,&xSavedConvertData,sizeof(u_int),&uBytesTransferred,0);

	//ignore versions we don't know about
	//also ignore anything before version 16 ,since nvDXTlib was updated then
	if(xSavedConvertData.m_ucVersion <= TEXTURE_CONVERSION_DATA_VERSION && xSavedConvertData.m_ucVersion >= 16)
	{
		if(xSavedConvertData.m_usProjectVersion == pxConvertData->m_usProjectVersion)
		{
			unsigned uBytesToRead = sizeof(TextureConversionData)-sizeof(int);
			if(xSavedConvertData.m_ucVersion<=16) uBytesToRead = sizeof(TextureConversionData_V16)-sizeof(int);
			else if(xSavedConvertData.m_ucVersion<=17) uBytesToRead = sizeof(TextureConversionData_V17)-sizeof(int);
			else if(xSavedConvertData.m_ucVersion<=18) uBytesToRead = sizeof(TextureConversionData_V18)-sizeof(int);
			else if(xSavedConvertData.m_ucVersion<=19) uBytesToRead = sizeof(TextureConversionData_V19)-sizeof(int);

			
			//read in the rest
			ReadFile(handle,((char*)&xSavedConvertData)+sizeof(int),uBytesToRead,&uBytesTransferred,0);
			bOkay = true;

			//Zero any flags in the saved data that don't matter before doing the comparison
			//RAR 05/03/20008
			xSavedConvertData.ClearUnusedEntriesForPlatform(s_eExportType);			
    	                     
			//which flags don't match?
			int iDifferentTextureFlags = xSavedConvertData.m_iTextureFlags^pxConvertData->m_iTextureFlags;
			unsigned int uDifferentMaterialFlags = xSavedConvertData.m_uMaterialFlags^pxConvertData->m_uMaterialFlags;
			if(iDifferentTextureFlags)
			{
				bOkay = false;
			}
			if(uDifferentMaterialFlags)
			{
				bOkay = false;
			}


				if(xSavedConvertData.m_bSwizzled!=pxConvertData->m_bSwizzled)
			{
				bOkay = false;
			}


			//check palette/pixel format for PS2
			if(s_eExportType == Export_PS2)
			{
				if(xSavedConvertData.m_bUnpalettised)
				{
					//convert old format
					xSavedConvertData.m_iPixelFormat = TIM2_RGB32;
					xSavedConvertData.m_iPaletteFormat = 0;
				}

					if(xSavedConvertData.m_iPixelFormat != pxConvertData->m_iPixelFormat || xSavedConvertData.m_iPaletteFormat != pxConvertData->m_iPaletteFormat)
				{
					bOkay = false;
				}
			}
	
			if(s_eExportType == Export_PS2 && (xSavedConvertData.m_bUnpalettised != pxConvertData->m_bUnpalettised))
			{
				//unpalletised flag doesn't match
				bOkay = false;
			}


			// KOR - 08/12/2005 - added max mip drops in version 12.
			switch(s_eExportType)
			{
				// Mip drops only count for the following platforms; fall through.
				case Export_PSP:
//				case Export_PS3: //removed the check for PS3 max mip map
				case Export_Wii:
				{
					// Check whether the mip drops match
						if(pxConvertData->m_uMaxMipDrops != xSavedConvertData.m_uMaxMipDrops)
					{
						bOkay = false;
					}
				}
				break;
				default: break;
			}

			//Check new palette options. RAR 26/10/2006
			if(xSavedConvertData.m_bUseFixedPalette != pxConvertData->m_bUseFixedPalette) bOkay = false;
			if(xSavedConvertData.m_bWritePaletteAsRedScaleIndices != pxConvertData->m_bWritePaletteAsRedScaleIndices) bOkay = false;

			//Check dithering flag. RAR 10/11/2006
			if(xSavedConvertData.m_bDisableDithering != pxConvertData->m_bDisableDithering) bOkay = false;

#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
			//Check texture usage flags
			//Don't bother with doing this for older files to avoid having to do an immediate conversion of
			//most of the textures.
			//RAR 14/11/2007
			if(xSavedConvertData.m_ucVersion>=19)
			{

				bool bWasNormal = (xSavedConvertData.m_uTextureUsageFlags & Tools_MaterialResponse_System::TU_NORMAL_CHANNELS) || (xSavedConvertData.m_iTextureFlags & AsuraTextureFlag_NormalMap);
				bool bWasSingleChannel = (xSavedConvertData.m_uTextureUsageFlags & Tools_MaterialResponse_System::TU_SINGLE_CHANNEL) ? true : false;
				bool bWasColourChannel = (xSavedConvertData.m_uTextureUsageFlags & Tools_MaterialResponse_System::TU_COLOUR_CHANNELS) ? true : false;

				bool bIsNormal = (pxConvertData->m_uTextureUsageFlags & Tools_MaterialResponse_System::TU_NORMAL_CHANNELS) || (pxConvertData->m_iTextureFlags & AsuraTextureFlag_NormalMap);
				bool bIsSingleChannel = (pxConvertData->m_uTextureUsageFlags & Tools_MaterialResponse_System::TU_SINGLE_CHANNEL) ? true : false;
				bool bIsColourChannel = (pxConvertData->m_uTextureUsageFlags & Tools_MaterialResponse_System::TU_COLOUR_CHANNELS) ? true : false;

				if(bWasNormal != bIsNormal)	bOkay = false;

				//PC doesn't currently care about the single/colour channel flags
				if(s_eExportType != Export_PC)
				{
					if(bIsSingleChannel != bWasSingleChannel) bOkay = false;
					if(bIsColourChannel != bWasColourChannel) bOkay = false;
				}
			}
#endif //defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)

			if(xSavedConvertData.m_ucVersion < 22 && s_eExportType == Export_PS3)
			{
				//Changed behaviour of high quality compression flag
				if(xSavedConvertData.m_iTextureFlags & AsuraTextureFlag_HighQualityCompression)
				{
					bOkay = false;
				}
			}

			
			if(xSavedConvertData.m_uMaximumTextureSize != pxConvertData->m_uMaximumTextureSize) bOkay = false;


			if( xSavedConvertData.m_ucPlatformVersion != pxConvertData->m_ucPlatformVersion ) bOkay = false;
		}

	}

	CloseHandle(handle);

	return bOkay;
}

void Tools_Level_Export::CreateConversionDataFile(const char* textureFilename,const TextureConversionData* convertData)
{
	char filename[MAX_PATH];
	unsigned long bytes_transferred;

	strcpy(filename,textureFilename);

	//replace the fileextension
	char* dotPos = strrchr(filename,'.');
	if(dotPos)*dotPos = 0;
	strcat(filename,".dat");

	Asura_Assertf(!Tools_FileSystem::IsReadOnly(filename),("%s is readonly. It will be made writeable",filename));
	Tools_FileSystem::MakeWriteable(filename);



	//open the file
	HANDLE handle = CreateFile(filename,GENERIC_WRITE,0,0,CREATE_ALWAYS,0,0);
	if(handle==INVALID_HANDLE_VALUE) return;

	//and save
	WriteFile(handle,convertData,sizeof(*convertData),&bytes_transferred,0);

	CloseHandle(handle);
}

bool Tools_Level_Export::IsEnvironmentFile( const Asura_String& xFilename )
{
    if( m_xEnvironmentFileName.IsBlank() )
    {
        return false;
    }
    else
    {
        const char* szRelative1 = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( xFilename.GetString() );
        const char* szRelative2 = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( m_xEnvironmentFileName.GetString() );
        return( !stricmp(szRelative1, szRelative2) );
    }
}

void Tools_Level_Export::PadToQWordBoundary(HANDLE pSaveHandle)
{
	// (AW - 24.02.2005) - Because of the way that BHD loads its front end environment asynchronously
	// it has become necessary to make sure that the stripped environment starts on a QWord boundary.
    u_int uCurrentFilePtr = SetFilePointer( pSaveHandle, 0, NULL, FILE_CURRENT );
	int iNumBytesToPad = 16 - (uCurrentFilePtr & 15);
	if (iNumBytesToPad != 16)
	{
		Asura_Assert(iNumBytesToPad > 0 && iNumBytesToPad < 16, "Sanity check");
		// (AW - 24.02.2005) - Becaues we're using a chunk, we'll actually be adding (iNumBytesToPad + 16) extra bytes.
		Asura_SaveBlock xSaveBlock;
		Asura_Chunk_Padding::Create(&xSaveBlock, iNumBytesToPad);

		u_int uDataSize = xSaveBlock.GetSize() - ASURA_FILE_ID_NUM_CHARS;
		if(uDataSize)
		{
			DWORD uBytesWritten;
			WriteFile( pSaveHandle, xSaveBlock.GetStartPtr() + ASURA_FILE_ID_NUM_CHARS, uDataSize, &uBytesWritten, NULL );
		}
	}
#ifdef ASURA_ASSERT
	uCurrentFilePtr = SetFilePointer( pSaveHandle, 0, NULL, FILE_CURRENT );
	Asura_Assert(!(uCurrentFilePtr & 15), "Failed to pad to qword boundary.");
#endif //#ifdef ASURA_ASSERT
}

void Tools_Level_Export::AddStripifiedEnvironment(HANDLE pSaveHandle)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::AddStripifiedEnvironment");
	
	bool bExportingAnyModules = false;
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		if(ShouldIncludeModuleInExport(i))
		{
			bExportingAnyModules = true;
			break;
		}
	}
	if(!bExportingAnyModules)
	{
		//All flags have been flagged as not for export , so skip the environment export.
		//Such an export is unlikely to work especially well (or at all) , but it was
		//deemed to be more convenient to let the export go ahead and produce the file anyway.
		//RAR 07/01/2008
        Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "No modules have been included in this export." );
		return;
	}


	switch(s_eExportType)
	{
		case Export_PS2:
		{
			// If this is a PS2 export, save the PS2 stripped environment. Note that
			// this needs to appear after the texture section (currently inside the
			// environment Asura file.)
			PadToQWordBoundary( pSaveHandle );
		    PS2Strip::ExportEnvironment( pSaveHandle , s_bSubdivideMeshes );
			break;
		}
		case Export_Wii:
		{
			Tools_Wii_Export::ExportEnvironment(pSaveHandle, s_bSubdivideMeshes);
			break;
		}
		case Export_PC:
		{
#ifdef ASURA_TARGET_SM3
			Tools_PC_Export::ExportEnvironment_SM3( pSaveHandle );
#else
			Tools_PC_Export::ExportEnvironment( pSaveHandle );
#endif
			break;
		}
		case Export_PSP:
		{
			Tools_PSP_Export::ExportEnvironment( pSaveHandle );
			break;
		}
		case Export_PS3:
		{
			Tools_PS3_Export::ExportEnvironment( pSaveHandle );
			break;
		}
		case Export_Xbox360:
		{
			Tools_Xbox360_Export::ExportEnvironment( pSaveHandle );
			break;
		}
		default:
		{
			break;
		}
    }
}

void Tools_Level_Export::InitialiseSubdivision()
{
	// KJL - 12/06/2003 12:01:42 - the values here are a bit magic numbers atm
	// we'll either move to a default for each platform or (more likely) platform/project
	// specific settings

	switch(s_eExportType)
	{
		case Export_PC:
		{
			Asura_PatchSubdivision::SetFlatSubdivideSize(1000.0f);
			Asura_PatchSubdivision::SetCurveSubdivideSize(0.05f);
			Asura_PatchSubdivision::SetSubdivideFlat(false);

			break;
		}

		case Export_Wii:
		{
			Asura_PatchSubdivision::SetFlatSubdivideSize(1000.0f);
			Asura_PatchSubdivision::SetCurveSubdivideSize(1000.0f);
			Asura_PatchSubdivision::SetSubdivideFlat(false);

			break;
		}
		
		case Export_PSP:
		{
//			Asura_PatchSubdivision::SetFlatSubdivideSize(s_fFlatSubdivideSize);
			Asura_PatchSubdivision::SetFlatSubdivideSize(1000.f);
			Asura_PatchSubdivision::SetCurveSubdivideSize(1000.0f);
			Asura_PatchSubdivision::SetSubdivideFlat(false);
			break;
		}

		case Export_PS3:
		{
			Asura_PatchSubdivision::SetFlatSubdivideSize(1000.0f);
			Asura_PatchSubdivision::SetCurveSubdivideSize(0.05f);
			Asura_PatchSubdivision::SetSubdivideFlat(false);

			break;
		}

		case Export_Xbox360:
		{
			Asura_PatchSubdivision::SetFlatSubdivideSize(1000.0f);
			Asura_PatchSubdivision::SetCurveSubdivideSize(0.05f);
			Asura_PatchSubdivision::SetSubdivideFlat(false);

			break;
		}

		default:
		{
			Asura_PatchSubdivision::SetFlatSubdivideSize(s_fFlatSubdivideSize);
			Asura_PatchSubdivision::SetCurveSubdivideSize(s_fCurveSubdivideSize);
			Asura_PatchSubdivision::SetSubdivideFlat(true);

			break;
		}
    }


	Asura_PatchSubdivision::SubdivideNearWaypoints();

	Tools_SubdividedMesh::s_bRecalculatePrelighting = s_bRecalculatePrelighting;
}

void Tools_Level_Export::AddFragmentsToExport()
{
	//build a list of fragment types being used by the shapes
	List<Asura_Hash_ID> xFragmentHashList;

	for(int iCommon = 0;iCommon<2;iCommon++)
	{
		if((iCommon==0) && !m_bIncludeEntityShapes) continue;
		if((iCommon==1) && !m_bIncludeCommonShapes) continue;

		int iNumMeshes = iCommon ? Tools_ShapeTable::GetNumExportShapes() : Tools_ShapeTable::GetNumElements();

		for(int i=0;i<iNumMeshes;i++)
		{
			//ignore shapes that aren't going to be exported
			if(!Tools_ShapeTable::ShouldExportShapeByIndex(i,iCommon ? true : false)) continue;

			Asura_Mesh* pxMesh = Tools_ShapeTable::GetMeshByIndex(i,iCommon ? true : false);	

			if(!pxMesh ||
				pxMesh->m_uFragmentHash == ASURA_HASH_ID_UNSET) continue;
			if(!xFragmentHashList.contains(pxMesh->m_uFragmentHash))
			{
				xFragmentHashList.add_entry(pxMesh->m_uFragmentHash);
			}
		}
	}

    // See if any entities wish to contribute additional fragments.
    {
        // Put anything they add into a separate list so we can weed out 
        // duplicates below, without requiring them to do so.
        List<Asura_Hash_ID> xFragmentHashesFromEntities;
        for( Asura_Hashed_List_ConstIt<Tools_Entity_Object> xIt(Tools_Entity_Object::GetList()); !xIt.Done(); xIt.Next() )
	    {
    	    const Tools_Entity_Object* pxEntityObject = xIt.GetCurrent();
		    if( pxEntityObject->ShouldAddFilesToExportList() )
            {
                pxEntityObject->AddFragmentHashesToExportList( xFragmentHashesFromEntities );
            }
        }

        for( CLIF<Asura_Hash_ID> xHashIt(&xFragmentHashesFromEntities); !xHashIt.done(); xHashIt.next() )
        {
            const Asura_Hash_ID uNextHash = xHashIt();
            if( !xFragmentHashList.contains(uNextHash) )
            {
                xFragmentHashList.add_entry( uNextHash );

                // Ensure fragment data is loaded for any we're adding.
                Asura_Fragment_System::LoadSingleFragment( uNextHash );
            }
        }
    }

	for(LIF<Asura_Hash_ID> xFragmentHashLIF(&xFragmentHashList);!xFragmentHashLIF.done();xFragmentHashLIF.next())
	{
		Asura_FragmentType* pxFragmentType = Asura_Fragment_System::GetFragmentType(xFragmentHashLIF());
		if(!pxFragmentType) continue;

		//Get the filename for the fragments
		const char* pszFilename = Asura_HashedFilename_System::GetFullFilename(pxFragmentType->m_uFragmentHash);
		if(!pszFilename) continue;

		if(pxFragmentType->m_bAnimatedFragment)
		{
			if(!m_bFakeLevelExport)
			{
				//Animated fragment files go into the normal asura file list instead.
				//We don't bother adding the files to the list for the fake level export , since all we care about is whether they have been loaded ,
				//and this gets dealt with earlier. RAR 08/05/03
				AddFileToList( m_xAsuraFileList, pszFilename, true );
			}
		}
		else
		{
			AddShapeFileToExportList(pszFilename);
			}
	
	}
			
}

void Tools_Level_Export::AddFilenameResources(HANDLE hFile)
{
	for(LIF<Asura_String> xNameLIF(&m_xSeperatelyLoadedFiles);!xNameLIF.done();xNameLIF.next())
	{
		AddFilenameResource(hFile, xNameLIF().GetString(), false);
	}		
}

void Tools_Level_Export::AddFilenameResource(HANDLE hFile, const char* szFilename, bool bLocalisedFile)
{
	Asura_String xName = szFilename;

		//determine size of resource chunk;
		int iSize = sizeof(Asura_Chunk_ResourceFile);
	iSize += xName.GetSizeInInts()*4;

		//allocate buffer for chunk
		char* pcChunkBuffer = new char[iSize];
		//fill in resource chunk header
	u_int uFlags = bLocalisedFile ? Asura_Chunk_ResourceFile::FRF_LOCALISE_FILENAME : 0;
	((Asura_Chunk_ResourceFile*)pcChunkBuffer)->Create(iSize,uFlags,ASURA_RESOURCEFILE_TYPE_ASURAFILENAME,0,0);
		//and the filename
	strcpy(pcChunkBuffer+sizeof(Asura_Chunk_ResourceFile),xName.GetString());

		//add to the file
		unsigned long uBytesWritten = 0;
		WriteFile(hFile,pcChunkBuffer,iSize,&uBytesWritten,0);
		if(uBytesWritten != (unsigned long)iSize)
		{
            AddMessage( MT_ERROR, "ERROR: %s.", Tools_OS::GetLastSystemError() );
			SetAbortFlag();
		}
		delete [] pcChunkBuffer;
}		

void Tools_Level_Export::BatchExport_GetIdentifiers( List<const char*>* pxIdentifiers )
{
	static const char* const aszIdentifiers[] =
	{
		"Export_Level",
		"Export_Characters",
		"Export_Global_Sounds",
		"Export_Common",
		"Export_Menus",
		"ExportMission_Environment",
		"ExportMission_Mission",
		"File",
		"PC",
		"PS2",
		"Gamecube",
		"PSP",
		"PS3",
		"Xbox360",
		"Wii",
		"Subdivide",
		"Flat_Subdivide_Size",
		"Curved_Subdivide_Size",
		"Prelight",
		"Swizzle_Tims",
		"Dont_Include_Cutscene_Files",
		"Create_Seperate_Cutscene_Files",
		"Multiplayer",
		"Override_Animation_Frame_Removal",
		"Position_Tolerance",
		"Angle_Tolerance",
		"Gamecube_Min_TexRes",
		"Gamecube_Max_MipDrops",
		"Full_Bright",
		"Light_Sets",
		"Bundle_Textures",
		"Tools",
		"Maximum_Texture_Size",
		"Use_Maximum_Texture_Size",
		"Compression_Disabled",
	};

    int iNumIdentifiers = sizeof(aszIdentifiers) / sizeof(const char*);
    for( int i = 0; i < iNumIdentifiers; ++i )
    {
        pxIdentifiers->add_entry_end( aszIdentifiers[i] );
    }

	//add language names
	for(int i=0; i<ASURA_LT_L_NUM_LANGUAGES; i++)
	{
		pxIdentifiers->add_entry_end( Asura_LocalisedText::GetNameForLang((Asura_LocalisedText_Language) i));
	}
}

void Tools_Level_Export::UpdateFilesInList(const char* pszFilename)
{
	if(!m_xAsuraFileList.size()) return;
	
	//make sure the existing file is writeable
	if(Tools_FileSystem::IsReadOnly(pszFilename))
	{
		AddMessage( MT_ERROR, "%s is read-only",pszFilename);
		return;
	}
	
	UpdateStatus( EXPORT_STAGE_LOADING_OLD_EXPORT_FILE, "Loading Old Export File", -1 );

	//get the contents of the existing exported file
	Tools_AsuraFileChunkLister xChunkLister;
	if(!xChunkLister.LoadAsuraFile(pszFilename,false))
	{
        AddMessage( MT_ERROR, "Failed to open %s",pszFilename );
		return;
	}

	const bool bBigEndian = xChunkLister.IsBigEndian();
	if( bBigEndian != ShouldExportBigEndian())
	{
        AddMessage( MT_ERROR, "Endianess of old file doesn't match the expected endianess for this platform");
		return;
	}

	UpdateStatus( EXPORT_STAGE_END, NULL, -1 );
	
	const List<const Asura_Chunk_Header*>* pxChunkList = xChunkLister.GetChunkList();

	
	int iOffsetInOldFile = 8;
	int iByteOffsetAdjust = 0;
	bool bHasLanguageFile = false;

	LIF<Asura_String> xFileLIF(&m_xAsuraFileList);

	//search for the resource file list chunk
	CLIF<const Asura_Chunk_Header*> xChunkLIF(pxChunkList);
	for(;!xChunkLIF.done();xChunkLIF.next())
	{
		int iChunkID = xChunkLIF()->ID;
		if(bBigEndian) iChunkID = Asura_Endian::SwapBytes(iChunkID);

		if(iChunkID != ASURA_CHUNK_RESOURCEFILELIST)
		{
			//only chunk expected before the resource file list is the file info chunk
			if(iChunkID == ASURA_CHUNK_FILEINFO)
			{
				//check the flags in the file info chunk so it can be created again correctly at the end
				int iChunkFlags = xChunkLIF()->Flags;
				if(bBigEndian) iChunkFlags = Asura_Endian::SwapBytes(iChunkFlags);
				bHasLanguageFile = (iChunkFlags & Asura_Chunk_FileInfo::ChunkFlag_HasLocalisedFile) ? true : false;
			}
			else
			{
		        AddMessage( MT_ERROR, "Unexpected chunks found in existing level file");
				return;
			}
			
			int iChunkSize = xChunkLIF()->Size;
			if(bBigEndian) iChunkSize = Asura_Endian::SwapBytes(iChunkSize);
			
			iOffsetInOldFile += iChunkSize;
			continue;
		}

		//found chunk , read the entries
		const Asura_Chunk_ResourceFileList* pxListChunkHdr = reinterpret_cast<const Asura_Chunk_ResourceFileList*>( xChunkLIF() );
		
	    g_uAsuraFilesInExport = pxListChunkHdr->uNumEntries;
		if(bBigEndian) g_uAsuraFilesInExport = Asura_Endian::SwapBytes(g_uAsuraFilesInExport);
		g_axResourceFileListEntries = new ResourceFileListEntry[g_uAsuraFilesInExport];

		//Only the header of the resource file chunk ever gets endian swapped. The contents should always be little endian.
		const char* szNextFilename = reinterpret_cast<const char*>( pxListChunkHdr + 1 );
		for( unsigned u = 0; u < g_uAsuraFilesInExport; ++u )
		{
			g_axResourceFileListEntries[u].xFilename = szNextFilename;

			const int* piDataAfterName = reinterpret_cast<const int*>( szNextFilename );
			piDataAfterName += g_axResourceFileListEntries[u].xFilename.GetSizeInInts();

			g_axResourceFileListEntries[u].iByteOffset = *piDataAfterName++;
			g_axResourceFileListEntries[u].iNumBytes    = *piDataAfterName++;
			g_axResourceFileListEntries[u].bFileAddedOK = true;
			*piDataAfterName++;//bIsAsuraFile

			szNextFilename = reinterpret_cast<const char*>( piDataAfterName );
		}

		xChunkLIF.next();
		break;
	}


   	// This macro assumes an error msg will be output elsewhere if the 
   	// specified expression is false.
   	#define CHECK( expr ) \
   	{ \
   	    /* Set abort flag if the expression is false. */ \
   	    if( !(expr) ) { SetAbortFlag(); } \
   	    \
   	    /* Check the abort flag independently because it may have been set \
   	       somewhere else, eg. by the user requesting an abort. */ \
   	    if( IsAbortFlagSet() ) { goto End; } \
   	}
    #define OUTPUT( data_ptr, num_bytes ) \
    { \
        DWORD uNumBytesWritten; \
        BOOL bOK = WriteFile( saveHandle, (data_ptr), (num_bytes), &uNumBytesWritten, NULL ); \
        if( !bOK || (DWORD)(num_bytes) != uNumBytesWritten ) \
        { \
            if( ERROR_HANDLE_DISK_FULL == GetLastError() ) \
            { \
                Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Disk full. Export aborted." ); \
            } \
            else \
            { \
                Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." ); \
            } \
            return; \
        } \
    }

	char szTempExportName[MAX_PATH];
	Asura_PrintfToBfr(szTempExportName,"%s\\TempExport.Temp",sizeof(szTempExportName),true,GetTempDirectory());


   	HANDLE saveHandle = INVALID_HANDLE_VALUE;
   	DWORD bytesWritten;
   
	//open the file to write to
	Tools_FileSystem::EnsureDirectoryExists(szTempExportName);
	saveHandle = CreateFile(szTempExportName,GENERIC_WRITE,0,0,CREATE_ALWAYS,0,0);
	if(saveHandle == INVALID_HANDLE_VALUE)
	{
		AddMessage(MT_ERROR,"Failed to open %s for writing. Export aborted.",pszFilename);
		CHECK( false );
	}

	//write the header
	WriteFile(saveHandle,ASURA_UNCOMPRESSED_FILE_ID,ASURA_FILE_ID_NUM_CHARS,&bytesWritten,0);

   	// Leave space for the resource file list chunk at the beginning of the 
   	// file. We know the maximum size but we don't know how much space we'll
   	// actually require until we've tried adding each Asura file so just
   	// provide the maximum. (In most cases, we'll need the maximum anyway, or
   	// very close to it.)
   	// (MH - 08.01.2003) also leave space for the fileinfo chunk - this contains
   	// the filesize so we won't know it until the end.
   	CHECK( LeaveSpaceForFileInfoChunk(saveHandle) );
   	CHECK( LeaveSpaceForResourceFileListChunk(saveHandle) );


	
	//scan through the entries in the old file , looking for those we need to replace
	for(unsigned u=0;u<g_uAsuraFilesInExport;u++)
	{
		const char* szNameInFile = g_axResourceFileListEntries[u].xFilename.GetString();
		//Skip LevelExportTemp directory to help with comparison ,to avoid issues caused it by changing name.
		if(!strncmp(szNameInFile,"LevelExportTemp",15))
		{
			szNameInFile += 15;

			while(*szNameInFile && *szNameInFile != '\\' && *szNameInFile != '/') szNameInFile++;
			while(*szNameInFile && (*szNameInFile == '\\' || *szNameInFile == '/')) szNameInFile++;

		}

		
	
		//check for a matching filename
		for(xFileLIF.restart();!xFileLIF.done();xFileLIF.next())
		{
			const char* szReplacementName = xFileLIF().GetString();
			
			if(!strncmp(szReplacementName,"LevelExportTemp",15))
			{
				szReplacementName += 15;

				while(*szReplacementName && *szReplacementName != '\\' && *szReplacementName != '/') szReplacementName++;
				while(*szReplacementName && (*szReplacementName == '\\' || *szReplacementName == '/')) szReplacementName++;

			}

			if(!stricmp(szNameInFile,szReplacementName))
			{
				//we have a match
				break;
			}
		}
		if(xFileLIF.done())
		{
			//no match , adjust the entry for byte offset if any previous chunk sizes were changed
			g_axResourceFileListEntries[u].iByteOffset += iByteOffsetAdjust;
			continue;
		}

		//okay , need to add everything in the old file up to this chunk
		int iDesiredOffset =  g_axResourceFileListEntries[u].iByteOffset;
		for(;iOffsetInOldFile < iDesiredOffset && !xChunkLIF.done();xChunkLIF.next())
		{
			//copy this chunk
			int iChunkSize = xChunkLIF()->Size;
			if(bBigEndian) iChunkSize = Asura_Endian::SwapBytes(iChunkSize);

			OUTPUT(xChunkLIF(), iChunkSize);
			iOffsetInOldFile += iChunkSize;
		
		}

		if(iOffsetInOldFile!=iDesiredOffset)
		{
			AddMessage(MT_ERROR,"Error skipping chunks in existing file");
			CHECK(false);	
		}

		//Advance our position in the old file to skip the chunks for the asura file we are replacing
		iDesiredOffset += g_axResourceFileListEntries[u].iNumBytes;
		for(;iOffsetInOldFile < iDesiredOffset && !xChunkLIF.done();xChunkLIF.next())
		{
			int iChunkSize = xChunkLIF()->Size;
			if(bBigEndian) iChunkSize = Asura_Endian::SwapBytes(iChunkSize);

			iOffsetInOldFile += iChunkSize;
		}
		if(iOffsetInOldFile!=iDesiredOffset)
		{
			AddMessage(MT_ERROR,"Error skipping chunks in existing file");
			CHECK(false);	
		}

		//Insert the newer file in its place
		int iOldSize = g_axResourceFileListEntries[u].iNumBytes;
		CHECK(AddAsuraFile(saveHandle,xFileLIF(),u,true));

		//Check to see if the replacement was a different size. If it is then later
		//entries will need to be altered
		iByteOffsetAdjust += g_axResourceFileListEntries[u].iNumBytes - iOldSize;

		xFileLIF.delete_current();

	}

	//add anything thats left over from the old file
	for(;!xChunkLIF.done();xChunkLIF.next())
	{
		int iChunkSize = xChunkLIF()->Size;
		if(bBigEndian) iChunkSize = Asura_Endian::SwapBytes(iChunkSize);

		//copy this chunk
		OUTPUT(xChunkLIF(),iChunkSize);
	}

	//any files that didn't get replaced?
	if(m_xAsuraFileList.size())
	{
		AddMessage(MT_WARNING,"Failed to find %d file(s) to be updated in the existing file :",m_xAsuraFileList.size());
		for(LIF<Asura_String> xAsuraFileLIF(&m_xAsuraFileList); !xAsuraFileLIF.done(); xAsuraFileLIF.next())
		{
			AddMessage(MT_WARNING,"Failed to update %s.",xAsuraFileLIF().GetString());
		}
	}



	//Add the terminator
   	{
	    int Terminator;
	    Terminator = 0;
	    BOOL bOK = WriteFile(saveHandle,&Terminator,4,&bytesWritten,0);
   	    if( !bOK || 4 != bytesWritten )
   	    {
   	        AddMessage( MT_ERROR, "I/O error." );
   	        SetAbortFlag();
   	        goto End;
   	    }
   	}

   	// Fill out the file info and resource file list chunks now.
   	u_int uAsuraFileSize;
   	if( saveHandle != INVALID_HANDLE_VALUE )
   	{
		#if defined ASURA_ASSERT
   			DWORD uCurFilePtrPos =
		#endif
			SetFilePointer( saveHandle, 0, NULL, FILE_CURRENT );

   	    DWORD uBeginningFilePtrPos = SetFilePointer( saveHandle, 0, NULL, FILE_BEGIN );
   	    DWORD uEndFilePtrPos = SetFilePointer( saveHandle, 0, NULL, FILE_END );
   	    Asura_Assert(uCurFilePtrPos == uEndFilePtrPos, "");
   	    uAsuraFileSize = uEndFilePtrPos - uBeginningFilePtrPos;
   	}

	m_bCreateLanguageFiles = bHasLanguageFile;//won't cause the creation of the file when done here , but will affect the flags in the file info chunk
   	CHECK( FillOutFileInfoChunk(saveHandle, uAsuraFileSize) );
   	CHECK( FillOutResourceFileListChunk(saveHandle) );


End:

	//done!
    if( saveHandle != INVALID_HANDLE_VALUE )
    {
	    CloseHandle(saveHandle);
    }


   	// Clear the progress message.
	UpdateStatus(EXPORT_STAGE_END, NULL, -1);

	if(!IsAbortFlagSet())
	{
		if(!DeleteFile(pszFilename))
		{
			AddMessage( MT_ERROR, "Failed to delete old version of %s",pszFilename);
		}
		else if(!MoveFile(szTempExportName,pszFilename))
		{
			AddMessage( MT_ERROR, "Failed to move temporary file");
		}
		else
		{
		
			if(!IsCompressionDisabled() && ShouldCompressExportedFiles())
			{
				AddMessage(MT_INFORMATION,"Compressing %s",pszFilename);
				CompressFile(pszFilename);
			}
		
			AddMessage(MT_INFORMATION,"Finished writing %s.",pszFilename);
		}
		DeleteFile(szTempExportName);
	}
	else
	{
		DeleteFile(szTempExportName);
		AddMessage( MT_INFORMATION, "Aborted." );
	}


	delete [] g_axResourceFileListEntries;
	g_axResourceFileListEntries = 0;

    #undef CHECK
    #undef OUTPUT

}

u_int Tools_Level_Export::ScalePrelightValue(u_int uColour)
{
	if(!IsLevelExport()) return uColour;//only do the scaling while exporting levels
	Asura_Assert(s_eExportType<ASURA_NUM_LIGHT_SCALE_VALUES,"");

	if(s_bFullBrightPrelighting)
	{
		return (uColour&~0xffffff)|0x7f7f7f;
	}

	float fPrelightScale = Asura_Lighting_System::s_afPrelightScale[s_eExportType];

	if(fPrelightScale == 1.0f) return uColour;

	//split colour into its components
	int iRed = (uColour & 0xff0000)>>16;
	int iGreen = (uColour & 0x00ff00)>>8;
	int iBlue = (uColour & 0x0000ff);

	//scale them
	iRed = Asura_Maths::FloatToInt(iRed*fPrelightScale);
	if(iRed>255) iRed = 255;

	iGreen = Asura_Maths::FloatToInt(iGreen*fPrelightScale);
	if(iGreen>255) iGreen = 255;

	iBlue = Asura_Maths::FloatToInt(iBlue*fPrelightScale);
	if(iBlue>255) iBlue = 255;

	// (AAM - 16.09.2004) Don't just ignore alpha!
	const int iAlpha = (uColour & 0xFF000000) >> 24;

    //pack the colours back again
	return (iAlpha<<24)|(iRed<<16)|(iGreen<<8)|(iBlue);
}

float Tools_Level_Export::ScaleLightBrightness(float fBrightness)
{
	if(!IsLevelExport()) return fBrightness;//only do the scaling while exporting levels
	Asura_Assert(s_eExportType<ASURA_NUM_LIGHT_SCALE_VALUES,"");
	return fBrightness * Asura_Lighting_System::s_afRuntimeLightScale[s_eExportType];
}

#ifdef ASURA_USE_WATER

void Tools_Level_Export::AddWaterPackets(HANDLE hFile)
{

	//RAR 11/01/2005 - For the moment water is only available on PS2
	//RAR 02/03/2005 - Also maybe available on XBox for some projects.... using ShouldChunkBeExported to find out
	if(!ShouldChunkBeExported(ASURA_CHUNK_WATERPACKET)) return;

	Asura_SaveBlock xSaveBlock;

	Asura_Water_System::BuildWaterPackets(xSaveBlock, (s_eExportType == Export_PC));

	//add the contents of the save block to the file
    u_int uDataSize = xSaveBlock.GetSize() - ASURA_FILE_ID_NUM_CHARS;
	if(uDataSize)
	{
		DWORD uBytesWritten;
		WriteFile( hFile, xSaveBlock.GetStartPtr() + ASURA_FILE_ID_NUM_CHARS, uDataSize, &uBytesWritten, NULL );
	}

	AddMessage(MT_INFORMATION, "%i bytes written by water export (about %i vertices)", uDataSize, uDataSize / sizeof(Asura_Chunk_WaterPacket_Vertex));
}
#endif //ASURA_USE_WATER

const char* Tools_Level_Export::GetPlatformName(int iPlatform)
{
	return Tools_Platform::GetPlatformName(iPlatform);
}

const char* Tools_Level_Export::GetPlatformGraphicsDirectoryForFileExport( int iPlatform )
{
	static Asura_Char xBuffer[128];
	const Asura_Char* const szPrefix = "graphics";

	if( iPlatform != Export_PC )
	{
		Asura_PrintfToBfr( xBuffer,"%s_%s",128,true,szPrefix,GetPlatformName( iPlatform ) );
	}
	else
	{
		Asura_PrintfToBfr( xBuffer,"%s",128,true,szPrefix );
	}

	return xBuffer;
}

#if defined( ASURA_USE_CARPET )
bool Tools_Level_Export::AddCarpetMeshData( HANDLE pSaveHandle )
{
	switch(s_eExportType)
	{
		case Export_PC:
		{
			if ( true == Asura_Direct3D_Carpet_Game_Render::HasDataSet() )
			{
				Asura_Direct3D_Carpet_Game_Render::RelightEverything();
				return Asura_Direct3D_Carpet_Game_Render::ExportPC( pSaveHandle );
			}

			break;
		}
		case Export_Xbox360:
		{
			if ( true == Asura_Direct3D_Carpet_Game_Render::HasDataSet() )
			{
				Asura_Direct3D_Carpet_Game_Render::RelightEverything();
				return Asura_Direct3D_Carpet_Game_Render::ExportXBox360( pSaveHandle );
			}

			break;
		}
		case Export_PS3:
			{
				if ( true == Asura_Direct3D_Carpet_Game_Render::HasDataSet() )
				{
					Asura_Direct3D_Carpet_Game_Render::RelightEverything();
					return Asura_Direct3D_Carpet_Game_Render::ExportPS3( pSaveHandle );
				}

				break;
			}
		case Export_PS2:
		{
			PadToQWordBoundary( pSaveHandle );
			// Update the carpet lighting on PS2 export just to be sure
			Tools_Carpet_Mesh::CaptureLighting();
			return PS2Strip::ExportCarpetMeshChunk( pSaveHandle );
		}
		default:
		{
			AddMessage( MT_ERROR, "Carpet Mesh not supported for this platform yet!" );
			break;
		}
	}
	return true;
}
#endif // #if defined( ASURA_USE_CARPET )

#if defined( ASURA_USE_BOTANICALS )
bool Tools_Level_Export::AddBotanicalsData( HANDLE pSaveHandle )
{
	// (AAM - 25.09.2006) Ensure texture names and material chunks comes immediately before mesh data that references it
	{
		Asura_SaveBlock xSaveBlock;
		Asura_Chunk_TextureNames::Create( &m_xBotanicalsTextures, &xSaveBlock );
		Asura_Chunk_Material::Create( m_iNumBotanicalsMtls, &m_xBotanicalsMtlList[0], &xSaveBlock );

		DWORD uBytesWritten;
		WriteFile( pSaveHandle, xSaveBlock.GetStartPtr() + ASURA_FILE_ID_NUM_CHARS, xSaveBlock.GetSize() - ASURA_FILE_ID_NUM_CHARS, &uBytesWritten, NULL );
	}

	switch(s_eExportType)
	{
		case Export_PC:
		{
			if ( false == Asura_Direct3D_Botanicals_Game_Render::HasDataSet() )
			{
				const bool bSuccess = Asura_Direct3D_Botanicals_Game_Render::MakeDataSet( Asura_Botanicals_InstanceManager::GetInstanceList());
				if( false == bSuccess)
				{
					return( false);
				}
			}
			if ( true == Asura_Direct3D_Botanicals_Game_Render::HasDataSet() )
			{
				return Asura_Direct3D_Botanicals_Game_Render::Export( pSaveHandle );
			}

			break;
		}
		case Export_PS2:
		{
			PadToQWordBoundary( pSaveHandle );
			return PS2Strip::ExportBotanicalsChunk( pSaveHandle, m_aiBotanicalsMtlConvList );
		}
		case Export_Xbox360:
		{
			return Tools_Xbox360_Export::ExportBotanicals( pSaveHandle, m_aiBotanicalsMtlConvList );
		}
		case Export_PSP:
		{
			return Tools_PSP_Export::ExportBotanicals( pSaveHandle, m_aiBotanicalsMtlConvList ); 
		}
		case Export_PS3:
		{
			return Tools_PS3_Export::ExportBotanicals( pSaveHandle, m_aiBotanicalsMtlConvList );
		}
		default:
		{
			Asura_Assert( false, "Botanicals not supported for this platform yet!" );
			break;
		}
	}
	return true;
}

#endif // #if defined( ASURA_USE_BOTANICALS )


#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
bool Tools_Level_Export::AddStaticDecalsData(HANDLE pSaveHandle)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::AddStaticDecalsData");

	Tools_StaticDecal_System::UpdateBuffers(true,0,true);


	switch(s_eExportType)
	{
	case Export_PC:
		{
#ifdef ASURA_TARGET_SM3
			Tools_PC_Export::ExportStaticDecals( pSaveHandle );
#endif
			break;
		}
	case Export_PS3:
		{
			return Tools_PS3_Export::ExportStaticDecals( pSaveHandle );
			break;
		}
	case Export_Xbox360:
		{
			return Tools_Xbox360_Export::ExportStaticDecals( pSaveHandle );
			break;
		}
	}

	return true;
}
#endif // ASURA_USE_STATIC_DECAL_SYSTEM


// JGP - Attractor data

#ifdef ASURA_USE_ATTRACTORS

bool Tools_Level_Export::AddAttractorData( HANDLE xSaveHandle )
{

	Asura_SaveBlock xSaveBlock;
	Asura_Chunk_SaveBlock_Stream xStream( &xSaveBlock );

	Asura_Chunk_Attractors::Create( xStream );

	return ( xSaveBlock.WriteToFile( xSaveHandle ) != -1 );

}

#endif

void Tools_Level_Export::AddBlueprintTypeToExportList(Asura_Hash_ID uBlueprintType)
{
	Asura_Assert(This,"");
	if(uBlueprintType == ASURA_HASH_ID_UNSET) return;

	//blueprint already dealt with?
	int iJunkIndex;
	if(This->m_pxBlueprintTypeTree->FindNodeIndexByHash(uBlueprintType,true,iJunkIndex)) return;

	//add to the list for export
	This->m_xBlueprintTypeList.add_entry(uBlueprintType);
}

void Tools_Level_Export::AddBlueprintTypeToIgnore(Asura_Hash_ID uBlueprintType)
{
	if(uBlueprintType == ASURA_HASH_ID_UNSET) return;
	
	//remove from the list to export
	if(m_xBlueprintTypeList.contains(uBlueprintType)) m_xBlueprintTypeList.delete_entry(uBlueprintType);


	//and add to the list of blueprints that have been dealt with
	int iJunkIndex;
	m_pxBlueprintTypeTree->FindNodeIndexByHash(uBlueprintType,true,iJunkIndex);
}

bool Tools_Level_Export::AddBlueprints(HANDLE hFile)
{
	if(!m_xBlueprintTypeList.size()) return true;

	//create chunks for all the blueprints
	Asura_SaveBlock xSaveBlock;
	for(LIF<Asura_Hash_ID> xBlueLIF(&m_xBlueprintTypeList);!xBlueLIF.done();xBlueLIF.next())
	{
		Asura_Chunk_Blueprints::Create(xSaveBlock, xBlueLIF());
	}

	//write the chunks to the file
    u_int uDataSize = xSaveBlock.GetSize() - ASURA_FILE_ID_NUM_CHARS;

    DWORD uBytesWritten;
	WriteFile( hFile, xSaveBlock.GetStartPtr() + ASURA_FILE_ID_NUM_CHARS, uDataSize, &uBytesWritten, NULL );

	if(uBytesWritten != uDataSize)
	{
        AddMessage( MT_ERROR, "I/O error." );
		return false;
	}
	return true;

}

bool Tools_Level_Export::ShouldExportBlueprint_Static(const Asura_Blueprint* pxBlueprint, const bool bExportChildBlueprints)
{
	if(This == NULL)
	{
		Asura_Assert( false, "Tools_Level_Export not allocated." );
		return(true);
	}
	else
	{
		return(This->ShouldExportBlueprint(pxBlueprint, bExportChildBlueprints));
	}
}

bool Tools_Level_Export::ShouldExportBlueprint(const Asura_Blueprint* pxBlueprint, const bool bExportChildBlueprints)
{
	// If there's no export in progress, export anyway
	if(pxBlueprint->IsExported(bExportChildBlueprints)) return false;
	if( !Tools_Level_Export::IsExport() ) return true;

	// need to create a temporary blueprint edit object for templated type data
	Tools_Blueprint_Edit_Object* pxEditObject = Tools_Mode_Blueprint::CreateBlueprintEditObject_Static();

	int iPlatformBitfield = 0;
	Asura_TemplateHashTree<const Asura_Parameter*> xHandledParamTree;

	// does this blueprint (or its parent) contain a 'platform' parameter type?
	const Asura_Blueprint* pxCurrentBP = pxBlueprint;
	while( pxCurrentBP )
	{
		for( int iParam=0; iParam<pxCurrentBP->GetNumParameters(); ++iParam )
		{			
			const Asura_Parameter* pxParam = pxCurrentBP->GetParameterByIndex( iParam );
			
			// make sure we haven't already dealt with this parameter in a child blueprint
			if( !xHandledParamTree.GetDataByHash( pxParam->GetHashID() ) )
			{
				if( pxEditObject->GetTemplatedTypeFromTemplateID( pxParam->GetTemplate() ) == Tools_Blueprint_Edit_Object::TVT_EXPORT_PLATFORMS )
				{
					Asura_Value xValue;				
					if( pxCurrentBP->GetParameterValueByIndex( iParam, xValue ) )
					{
						iPlatformBitfield |= xValue.GetIntValue();
					}				

					// add it to the tree of already-handled parameters
					xHandledParamTree.AddNode( pxParam->GetHashID(), pxParam );
				}
			}
		}

		pxCurrentBP = pxCurrentBP->GetParent();
	}

	// clean up blueprint mode gubbins
	delete pxEditObject;

	// if we've found any platform export parameter, check whether we should export the blueprint for the current export type
	if( xHandledParamTree.GetNumberOfNodes() > 0 )
	{
		int iCurrentExportBitfield = (1 << Tools_Level_Export::s_eExportType);

		// if we're quitting to game, we also want to consider the platform set in the Tools_Platform_Mesh_System
		// if Tools_Platform_Mesh_System is set to 'default', consider that as /all/ platforms
		if( Tools_Level_Export::IsFakeLevelExport() )
		{			
			iCurrentExportBitfield |= (Tools_PlatformMesh_System::GetPlatform() == -1) ? 0xffffffff : (1 << Tools_PlatformMesh_System::GetPlatform());
		}

		return (iPlatformBitfield & iCurrentExportBitfield) != 0;
	}

	return true;
}

///////////////////
// AddContainers //
///////////////////
#ifdef ASURA_USE_CONTAINER_SYSTEM
bool Tools_Level_Export::AddContainers(HANDLE hFile)
{
	const Asura_TemplateHashTree<Asura_Container*>& xTree = Asura_Container_System::GetTree();
	if (xTree.GetNumberOfNodes() == 0)
	{
		// Nothing to save.
		return true;
	}
	
	Asura_SaveBlock xSaveBlock;
	Asura_Chunk_SaveBlock_Stream xStream( &xSaveBlock );

	Asura_TemplateHashTree_SaveIterator xIt(xTree.GetNumberOfNodes());
	for ( ; !xIt.Done() ; xIt.Next())
	{
		Asura_Container const * const * const ppxContainer = xTree.GetDataByIndex(xIt.GetCurrent());		
		if (!ppxContainer) continue;
		
		const Asura_Container* pxContainer = (*ppxContainer);
		if (pxContainer)
		{
			Asura_Chunk_Container::Create(pxContainer, xStream);
		}
	}

	return ( xSaveBlock.WriteToFile( hFile ) != -1 );
}
#endif //ASURA_USE_CONTAINER_SYSTEM


#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM 
bool Tools_Level_Export::AddMaterialResponses(HANDLE hFile)
{
	const int iNumNodes = m_pxMaterialsToExport->GetNumberOfNodes();
	const int iNumSlaves = m_pxSlaveMaterialsToExport->GetNumberOfNodes();

	// Don't do anything if no materials to export && no slave materials to export
	if ( ( iNumNodes == 0 ) && ( iNumSlaves == 0 ) )
	{
		return true;
	}

	//create chunk
	Asura_SaveBlock xSaveBlock;
	
	Asura_Chunk_MaterialResponse::Create( xSaveBlock, m_pxMaterialsToExport, m_pxSlaveMaterialsToExport );

	switch(s_eExportType)
	{
		case Export_PC:
		{
			Tools_PC_Export::ExportMaterialResponses( xSaveBlock, m_pxMaterialsToExport );
			break;
		}
		case Export_Xbox360:
		{
			Tools_Xbox360_Export::ExportMaterialResponses( xSaveBlock, m_pxMaterialsToExport );
			break;
		}
		case Export_PS3:
		{
			Tools_PS3_Export::ExportMaterialResponses( xSaveBlock, m_pxMaterialsToExport, m_pxSlaveMaterialsToExport );
		}
		default:
		{
			break;
		}
	}
	
	//write the chunks to the file
    u_int uDataSize = xSaveBlock.GetSize() - ASURA_FILE_ID_NUM_CHARS;

    DWORD uBytesWritten;
	WriteFile( hFile, xSaveBlock.GetStartPtr() + ASURA_FILE_ID_NUM_CHARS, uDataSize, &uBytesWritten, NULL );

	if(uBytesWritten != uDataSize)
	{
        AddMessage( MT_ERROR, "I/O error." );
		return false;
	}

	return true;
}

void Tools_Level_Export::AddMaterialResponseOverridesToExportList(void)
{
	Asura_String xFilename = GetTempDirectory();
	xFilename += "\\MaterialResponseOverrides.asr";

	// Complete the temporary filename generation...
	if(ShouldExportBigEndian())
	{
		xFilename	+= ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}

	// Save the file...
	if(Asura_Chunk_MaterialResponseOverrides::Save(xFilename.GetString()))
	{
		AddFileToExportList(xFilename, false);
	}
	else
	{
		AddMessage(MT_ERROR,"Failed to create \"%s\"", xFilename.GetString());
	}
}
#endif //ASURA_USE_MATERIAL_RESPONSE_SYSTEM
void Tools_Level_Export::AddPFXOverridesToExportList(void)
{
	Asura_String xFilename = GetTempDirectory();
	xFilename += "\\PFXOverrides.asr";

	// Complete the temporary filename generation...
	if(ShouldExportBigEndian())
	{
		xFilename	+= ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}

	// Save the file...
	if(Asura_Chunk_PFXOverrides::Save(xFilename.GetString()))
	{
		AddFileToExportList(xFilename, false);
	}
	else
	{
		AddMessage(MT_ERROR,"Failed to create \"%s\"", xFilename.GetString());
	}
}

void Tools_Level_Export::AddFSFXOverridesToExportList(void)
{
	Asura_String xFilename = GetTempDirectory();
	xFilename += "\\FSFXOverrides.asr";

	// Complete the temporary filename generation...
	if(ShouldExportBigEndian())
	{
		xFilename	+= ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}

	// Save the file...
	if(Asura_Chunk_FSFXOverrides::Save(xFilename.GetString()))
	{
		AddFileToExportList(xFilename, false);
	}
	else
	{
		AddMessage(MT_ERROR,"Failed to create \"%s\"", xFilename.GetString());
	}
}


const char* Tools_Level_Export::GetTempDirectory()
{
	if(g_szLevelExportTemp[0])
	{
		return g_szLevelExportTemp;
	}
	else
	{
		Asura_Assert(0,"LevelExportTemp directory not initialised yet");
		return "LevelExportTemp";
	}
}

void Tools_Level_Export::DeleteTemporaryFiles()
{
	char szSearch[MAX_PATH];
	Asura_PrintfToBfr(szSearch,"%s\\*.*",sizeof(szSearch),true,GetTempDirectory());


	//delete the temporary level export files
	List<Asura_String> xFileList;
	Tools_FileSystem::FindFiles(szSearch,xFileList,TOOLS_FILEFIND_RECURSIVE);
	for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next())
	{
		DeleteFile(xFileLIF().GetString());
	}

	//remove the (now) empty directories as well
	Asura_PrintfToBfr(szSearch,"%s\\*.*",sizeof(szSearch),true,GetTempDirectory());

	List<Asura_String> xDirList;
	Tools_FileSystem::FindDirectories(szSearch,xDirList,TOOLS_FILEFIND_RECURSIVE);
	for(LIB<Asura_String> xDirLIB(&xDirList);!xDirLIB.done();xDirLIB.next())
	{
		RemoveDirectory(xDirLIB().GetString());
	}

}

void Tools_Level_Export::InitialiseTempDirectory()
{
	Asura_Assert(!g_iLevelExportTempMutex,"Temp directory already initialised");

	//search for a free directory, create a mutex with the same name to make sure it isn't used by another instance
	for(int i=0;;i++)
	{
		Asura_Assert(i<99,"Something appears to be going wrong with finding a temp directory to use.");

		Asura_PrintfToBfr(g_szLevelExportTemp,"LevelExportTemp%d",sizeof(g_szLevelExportTemp),true,i);

		//attempt to create the mutex
		g_iLevelExportTempMutex = CreateMutex(0,false,g_szLevelExportTemp);
		if(GetLastError() == ERROR_ALREADY_EXISTS || !g_iLevelExportTempMutex)
		{
			if(g_iLevelExportTempMutex) CloseHandle(g_iLevelExportTempMutex);
			g_iLevelExportTempMutex = 0;
			continue;
		}

		//managed to create a new mutex , so this directory should be safe.
		//make sure it exists , and is empty
		Tools_FileSystem::EnsureDirectoryExists(g_szLevelExportTemp);
		DeleteTemporaryFiles();
		break;
	}
}

void Tools_Level_Export::CloseTempDirectory()
{
	Asura_Assert(g_iLevelExportTempMutex,"Temp directory not initialised");
	if(g_szLevelExportTemp[0])
	{
		//empty the directory and delete it
		DeleteTemporaryFiles();
		RemoveDirectory(g_szLevelExportTemp);
		g_szLevelExportTemp[0] = 0;
	}

	CloseHandle(g_iLevelExportTempMutex);
	g_iLevelExportTempMutex = 0;
}


bool Tools_Level_Export::IsLanguageAllowed(int iLanguage) const
{
	return (iLanguage == ASURA_LT_L_ENGLISH);
}

void Tools_Level_Export::ExportLanguageFiles(const char* szMainFilename)
{
	if(!m_bCreateLanguageFiles) return;

	for(int i=0;i<ASURA_LT_L_NUM_LANGUAGES && !IsAbortFlagSet();i++)
	{
		if(IsLanguageAllowed(i) && s_abExportLanguage[i])
		{
			ExportLanguageFile(szMainFilename,i);				
		}
	}
}

void Tools_Level_Export::ExportLanguageFile(const char* szMainFilename,int iLanguage)
{
	m_bExportingLanguageFile  = true;
	AddMessage(MT_INFORMATION,"Starting %s export.",Asura_LocalisedText::GetNameForLang((Asura_LocalisedText_Language)iLanguage));

	Asura_String xFilename;
	GetLanguageFilename(szMainFilename,iLanguage,xFilename);

   	// This macro assumes an error msg will be output elsewhere if the 
   	// specified expression is false.
   	#define CHECK( expr ) \
   	{ \
   	    /* Set abort flag if the expression is false. */ \
   	    if( !(expr) ) { SetAbortFlag(); } \
   	    \
   	    /* Check the abort flag independently because it may have been set \
   	       somewhere else, eg. by the user requesting an abort. */ \
   	    if( IsAbortFlagSet() ) { goto End; } \
   	}

   	HANDLE pSaveHandle = INVALID_HANDLE_VALUE;
   	DWORD uBytesWritten;
   
	//open the file to write to
	Tools_FileSystem::EnsureDirectoryExists(xFilename.GetString());
	pSaveHandle = CreateFile(xFilename.GetString(),GENERIC_WRITE,0,0,CREATE_ALWAYS,0,0);
	if(pSaveHandle == INVALID_HANDLE_VALUE)
	{
		AddMessage(MT_ERROR,"Failed to open %s for writing. Language export aborted.",xFilename.GetString());
		CHECK( false );
	}

	//write the header
	WriteFile(pSaveHandle,ASURA_UNCOMPRESSED_FILE_ID,ASURA_FILE_ID_NUM_CHARS,&uBytesWritten,0);
	
   	CHECK( LeaveSpaceForFileInfoChunk(pSaveHandle) );
	g_uBytesReservedForFileListChunk = 0;

	//add sounds listed for localisation
	AddLocalisedSounds(pSaveHandle,iLanguage);

#ifdef ASURA_USE_DIALOGUE_SYSTEM
	//Add localised parts of dialogue events
	CHECK( Tools_Level_Export::AddLocalisedDialogueEvents(pSaveHandle,iLanguage) );
#endif	// ASURA_USE_DIALOGUE_SYSTEM

#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	CHECK(AddLocalisedFacialAnimationData(pSaveHandle, iLanguage));
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)

	// add localised streamed sounds
	AddLocalisedStreamedSounds(pSaveHandle,iLanguage);

	//Add the terminator
   	{
	    int iTerminator;
	    iTerminator = 0;
	    BOOL bOK = WriteFile(pSaveHandle,&iTerminator,4,&uBytesWritten,0);
   	    if( !bOK || 4 != uBytesWritten )
   	    {
   	        AddMessage( MT_ERROR, "I/O error." );
			SetAbortFlag();
   	        goto End;
   	    }
   	}
 
   	// Fill out the file info chunk
   	u_int uAsuraFileSize;
   	if( pSaveHandle != INVALID_HANDLE_VALUE )
   	{
		#if defined ASURA_ASSERT
   			DWORD uCurFilePtrPos =
		#endif
			SetFilePointer( pSaveHandle, 0, NULL, FILE_CURRENT );

   	    DWORD uBeginningFilePtrPos = SetFilePointer( pSaveHandle, 0, NULL, FILE_BEGIN );
   	    DWORD uEndFilePtrPos = SetFilePointer( pSaveHandle, 0, NULL, FILE_END );
   	    Asura_Assert(uCurFilePtrPos == uEndFilePtrPos, "");
   	    uAsuraFileSize = uEndFilePtrPos - uBeginningFilePtrPos;
   	}
   	CHECK( FillOutFileInfoChunk(pSaveHandle, uAsuraFileSize) );

End:

	//done!
    if( pSaveHandle != INVALID_HANDLE_VALUE )
    {
	    CloseHandle(pSaveHandle);
    }


   	// Clear the progress message.
	UpdateStatus(EXPORT_STAGE_END, NULL, -1);

	if(!IsAbortFlagSet())
	{
		bool bCompress = !IsCompressionDisabled() && ShouldCompressExportedFiles();
		if(m_xLocalisedStreamedSoundList.size() && TargetPlatformSupportsStreamedSounds())
		{
			//Can't compress the file if it contains streamed sounds. RAR 05/09/2007
			bCompress = false;
		}
		
		if(bCompress)
		{
			AddMessage(MT_INFORMATION,"Compressing %s",xFilename.GetString());
			CompressFile(xFilename.GetString());
		}
		
		AddMessage(MT_INFORMATION,"Finished writing %s.",xFilename.GetString());
	}
	else
	{
		DeleteFile(xFilename.GetString());
		AddMessage( MT_INFORMATION, "Aborted." );
	}

	m_bExportingLanguageFile  = false;


}

bool Tools_Level_Export::IsWriteableIncludingLanguageFiles(const char* szFilename,bool bShowMessage, bool bOnlyCheckLanguageFiles /*= false*/)
{
	//check the main file
	if(!bOnlyCheckLanguageFiles)
	{
		if(Tools_FileSystem::IsReadOnly(szFilename))
		{
			if(bShowMessage)
			{
				AddMessage( MT_ERROR, "%s is read-only",szFilename);
			}
			return false;
		}
	}

	//check each supported language file
	for(int i=0;i<ASURA_LT_L_NUM_LANGUAGES;i++)
	{
		if(IsLanguageAllowed(i) && s_abExportLanguage[i])
		{
			Asura_String xLanguageFile;
			GetLanguageFilename(szFilename,i,xLanguageFile);


			if(Tools_FileSystem::IsReadOnly(xLanguageFile.GetString()))
			{
				if(bShowMessage)
				{
					AddMessage( MT_ERROR, "%s is read-only",xLanguageFile.GetString());
				}
				return false;
			}
		}
	}

	return true;
}

void Tools_Level_Export::GetLanguageFilename(const char* szFilename,int iLanguage,Asura_String& xOutputFilename)
{
	Asura_String xSharedLanguageFilename;
	if(GetSharedLanguageFilename(szFilename, xSharedLanguageFilename))
	{
		Asura_Assert(!xSharedLanguageFilename.IsBlank(),"");
		szFilename = xSharedLanguageFilename.GetString();
	}
	
	
	//create filename by replacing the extension by the code for the language
	char acFilename[MAX_PATH];
	strcpy(acFilename,szFilename);

	char* pcExtension = Tools_FileSystem::StripPath(acFilename);
	if(pcExtension) pcExtension = strrchr(pcExtension,'.');
	
	if(!pcExtension) pcExtension = &acFilename[strlen(acFilename)];
	sprintf(pcExtension,".%s%s",Asura_LocalisedText::GetTwoCharCodeForLang((Asura_LocalisedText_Language)iLanguage), Tools_Platform::IsBigEndian(s_eExportType) ? ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX : "");

	xOutputFilename = acFilename;
	
}

void Tools_Level_Export::EnableAllLanguagesForExport()
{
	for(int i=0;i<ASURA_LT_L_NUM_LANGUAGES; i++)
	{
		s_abExportLanguage[i] = true;
	}
}

void Tools_Level_Export::DisableAllLanguagesForExport()
{
	for(int i=0;i<ASURA_LT_L_NUM_LANGUAGES; i++)
	{
		s_abExportLanguage[i] = false;
	}
}

void Tools_Level_Export::GetResourcesFromExistingLanguageFile(const char* szMainExportFilename)
{
	//find a language that is being exported (should get english if it is being done)
	int iLanguage = -1;
	for(int i=0;i<ASURA_LT_L_NUM_LANGUAGES; i++)
	{
		if(s_abExportLanguage[i])
		{
			iLanguage = i;
			break;
		}
	}
	if(iLanguage == -1)
	{
		return;
	}

	Asura_String xFilename;
	GetLanguageFilename(szMainExportFilename, iLanguage, xFilename);

	//Get the contents of the old file.
	Tools_AsuraFileChunkLister xChunkLister;
	if(!xChunkLister.LoadAsuraFile(xFilename.GetString(),false))
	{
		//Failed to open file , but that isn't necessarily a problem.
		return;
	}

	//Add any resources that were used in the file to this export.
	//Currently we only care about dialogue lines (which will in turn result in the sounds being added).
	//RAR 10/07/2007
	bool bBigEndian = xChunkLister.IsBigEndian();
	for(CLIF<const Asura_Chunk_Header*> xChunkLIF(xChunkLister.GetChunkList()); !xChunkLIF.done(); xChunkLIF.next())
	{
		const Asura_Chunk_Header* pxChunk = xChunkLIF();

		int iChunkID = pxChunk->ID;
		if(bBigEndian) iChunkID = Asura_Endian::SwapBytes(iChunkID);

		switch(iChunkID)
		{
#ifdef ASURA_USE_DIALOGUE_SYSTEM
			case ASURA_CHUNK_DIALOGUE_LINE :
			{
				//Fortunately the line name is the first part of the chunk , so just grab that
				//and use it to find the hash ID.
				const char* szLineName = (const char*)(pxChunk+1);
				Asura_Hash_ID uLineHash = Asura_GetHashID(szLineName);
				AddDialogueLineToExportList(uLineHash);
				break;
			}
#endif
			default :
				break;
		}
	}
}

void Tools_Level_Export::SetCreateLanguageFiles(HANDLE pSaveHandle)
{
	if(m_bCreateLanguageFiles) return;
	m_bCreateLanguageFiles = true;

	//If using a language file that is shared across several levels , then we can't rely on the flag
	//in the file info chunk for loading the correct file. So add a resource chunk to deal with it instead.
	//RAR 10/07/2007
	//pSaveHandle may be invalid if we are only exporting localised files.
	//In that case there is nothing else to be done here.
	//RAR 09/03/2009
	if(m_bUsingSharedLanguageFile && !m_xSharedLanguageFilename.IsBlank() && pSaveHandle != INVALID_HANDLE_VALUE)
	{
		char acFilename[MAX_PATH];
		strcpy(acFilename,m_xSharedLanguageFilename.GetString());
		strlwr(acFilename);

		//search for 'release' in the path
		const char* pcPos = strstr(acFilename,"release");
		if(!pcPos)
		{		
			Asura_Assertf(0,("Failed to determine path relative to release directory for %s",acFilename));
			return;
		}
		
		//skip forward to '\' and '/'
		while(*pcPos && (*pcPos != '\\' && *pcPos != '/')) pcPos++;
		//and past those slashes
		while(*pcPos == '\\' || *pcPos == '/') pcPos++;

		//hopefully there is some name left
		if(!pcPos)
		{		
			Asura_Assertf(0,("Failed to determine path relative to release directory for %s",acFilename));
			return;
		}

		AddFilenameResource(pSaveHandle,pcPos,true);

	
	}
}

bool Tools_Level_Export::ShouldIncludeModuleInExport(int iModuleIndex)
{
	//Outside of exports , assume all modules are wanted
	if(!IsLevelExport() || IsDoingFakeLevelExport()) return true;

	//Otherwise check to see if the light sets match
	const Asura_Environment* pxEnv = Asura_Environment_System::GetEnvironment();
	Asura_Assert(pxEnv, "");

	//Test for 'using light sets' is probably redundant , but leaving it in to be safe.
	if(Tools_LightSets::IsUsingLightSets())
	{
		if(!Tools_LightSets::IsMemberOfCurrentLightSets(pxEnv->GetLightSets(iModuleIndex)))
		{
			return false;
		}
	}

	return true;
}

bool Tools_Level_Export::IsExportingAllModules()
{
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		if(!ShouldIncludeModuleInExport(i))
		{
			return false;
		}
	}
	return true;
}

void Tools_Level_Export::EnableAllPlatformsForExport()
{
	s_uPlatformsToExport = 0;

	for(int i=0;i<TOOLS_PLATFORM_NUM_PLATFORMS;i++)
	{
		if(This->IsPlatformAllowed((EXPORT_TYPE)i))
		{
			s_uPlatformsToExport |= (1<<i);
		}
	}
}

void Tools_Level_Export::SetCurrentLightSetsToExport(u_int uLightSets)
{
	Asura_Assert(!s_bIsLevelExport, "SetCurrentLightSetsToExport() shouldn't be being called once the export has already started");

	s_uCurrentLightSetsToExport = uLightSets;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Level_Export::AddSaveBlock
// Author       : Andy Tate
// Date         : 5-Sep-2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_Level_Export::AddSaveBlock(Asura_SaveBlock& xSaveBlock, HANDLE pSaveHandle)
{
	const char*	pcStartAddress	= xSaveBlock.GetStartPtr() + ASURA_FILE_ID_NUM_CHARS;
	const DWORD	dwDataSize		= xSaveBlock.GetSize() - ASURA_FILE_ID_NUM_CHARS;

	DWORD		dwBytesWritten;

	//write the chunks to the file
	WriteFile(pSaveHandle, pcStartAddress, dwDataSize, &dwBytesWritten, NULL);

	if(dwBytesWritten == dwDataSize)
	{
		return(true);
	}
	else
	{
		AddMessage(MT_ERROR, "I/O error.");
		return(false);
	}
}

/// End of function 'Tools_Level_Export::AddSaveBlock'. ///


/// End of file. ///
