/**
*
* \file Tools_Level_Export.cpp
*
* Created by Kevin Lea - 29/11/2002 11:23:31
* Splitted into separate file 12/11/2008
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"

// Asura
#include "Debugging/Asura_Debug_Timers.h"

// Tools
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Load&Save/Tools_LoadedFileList.h"
#include "Misc/Tools_IniFile.h"
#include "Skybox/Tools_Mode_Skybox.h"

#ifdef ASURA_USE_WATER
#include "ChunkSystem/Asura_Chunk_Water.h"
#include "Rendering/Asura_Water.h"
#endif

#ifdef ASURA_USE_BILLBOARD_PLANTS
#include "SpecialFX/Asura_BillboardPlant_System.h"
#include "SpecialFX/Asura_BillboardPlant_Batch.h"
#include "SpecialFX/Asura_BillboardPlant_TemplateStore.h"
#endif //ASURA_USE_BILLBOARD_PLANTS

#if defined ( ASURA_USE_TEXTURE_STREAMING )
#include "Core/Asura_Texture_Streaming.h"
#include "ChunkSystem/Asura_Chunk_Texture_Streaming.h"
#endif

#include "Particles/Tools_Mode_PFX.h"

#ifdef ASURA_USE_FSFX
#include "FSFX/Tools_Mode_FSFX.h"
#include "FSFX/Asura_FSFX_System.h"
#include "FSFX/Asura_FSFX_Template.h"
#include "FSFX/Asura_FSFX_Component.h"
#endif //ASURA_USE_FSFX

#if ASURA_USE_GUI_MENU_SYSTEM
//#include "GUIMenu/Asura_GUIMenu_Message.h"
//#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "GUIMenu/Tools_GUIMenu_System.h"	
#endif

#ifdef ASURA_USE_DECAL_MANAGER
#include "SpecialFX/Asura_Decal_Manager.h"
#include "ChunkSystem/Asura_Chunk_Decals.h"
#endif

// G L O B A L S ////////////////////////////////////////
extern ResourceFileListEntry* g_axResourceFileListEntries;
extern unsigned g_uBytesReservedForFileListChunk;

void Tools_Level_Export::ListAsuraFilesForExport(List<Asura_String>& nameList)
{
	m_xEnvironmentFileName = Tools_SaveAsura::GetMainFilename();

	#ifdef ASURA_USE_STREAMING
	//Probably want the contents of this file to happen even earlier than this ,but this will do for now.
	//RAR 06/03/2006
	AddFileToList(nameList, Tools_SaveAsura::GetStreamingConnectionsFilename(), false);
	#endif

	if(!MissionExport_IsExportingMissionOnly())
	{	
		AddFileToList( nameList, Tools_SaveAsura::GetMainFilename(), false );
	}
	
    if(!IsFakeLevelExport())
	{
		if(!MissionExport_IsExportingMissionOnly())
		{
		AddFileToList( nameList, Tools_SaveAsura::GetGlobalIlluminationFilename(), false );
		AddFileToList( nameList, Tools_SaveAsura::GetLightingFilename(), false );
		}
		if(!MissionExport_IsExportingEnvironmentOnly())
		{
		AddFileToList( nameList, Tools_SaveAsura::GetSoundsFilename(), false );
		AddFileToList( nameList, Tools_SaveAsura::GetNavigationFilename(), false );
		}
		if(!MissionExport_IsExportingMissionOnly())
		{
		AddFileToList( nameList, Tools_SaveAsura::GetModuleLinkingFilename(), false );
			if(ShouldIncludeSkyboxInLevelExport( true ))
		{
			AddSkyboxFilesToExport();
		}
		}
		if(!MissionExport_IsExportingEnvironmentOnly())
		{
		AddFileToList( nameList, Tools_SaveAsura::GetCutscenesFilename(), false );
	
	#ifdef ASURA_USE_BILLBOARD_PLANTS
		AddFileToList( nameList, Tools_SaveAsura::GetBillboardPlantsFilename(), false );
	#endif //ASURA_USE_BILLBOARD_PLANTS
	
		AddFileToList( nameList, Tools_SaveAsura::GetEntityFilename(), false );
	
	#if ASURA_USE_GUI_MENU_SYSTEM
			//add level specific menus
			AddGUIMenuFileToExportList(Tools_GUIMenu_System::GetLevelSpecificFile());
	#endif // ASURA_USE_GUI_MENU_SYSTEM
		}

		if(!MissionExport_IsExportingMissionOnly())
		{
			#ifdef ASURA_USE_REVERBVOLUME_SYSTEM
			AddFileToList( nameList, Tools_SaveAsura::GetReverbVolumesFilename(), false );
			#endif
	}
	}

	if(!MissionExport_IsExportingEnvironmentOnly())
	{
	//Work out which fragments are used
	AddFragmentsToExport();

	
	AddAsuraFilesFromTextFiles();
	}
}

void Tools_Level_Export::AddAsuraFilesFromTextFiles()
{
	//add files listed in common export text file
	AddAsuraFilesFromTextFile(Tools_SaveAsura::GetAllLevelsExportFilename(), true);
	//and those specific to this level
	AddAsuraFilesFromTextFile(Tools_SaveAsura::GetLevelSpecificExportFilename(), true);

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
	//If its a fake level export, make sure that all the Common_Export files are included in the level
	// DF 19/04/07
	if ( IsDoingFakeLevelExport() )
	{
		AddAsuraFilesFromTextFile(GetCommonExportFile(), false);
	}
#else
	AddAsuraFilesFromTextFile(GetCommonExportFile(), true);
#endif //ASURA_USING_ALL_LEVEL_EXPORT
}

void Tools_Level_Export::AddAsuraFilesFromTextFile(const char* pszFilename, const bool bAllowMaterials)
{
	Tools_IniFile xTextFile(pszFilename);
	if(!xTextFile.LoadIniFile())
	{
       	Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "Failed to open %s.",pszFilename);
		return;
	}
	else
	{
       	Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "Adding files listed in %s.",pszFilename);
	}

	//search for the asura file section
	if(!xTextFile.GetIniTextSection("Asura Files"))
	{
       	Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "No [Asura Files] section found.");
	}
	else
	{
		Tools_AsuraFileContents xFileContents;
		while(xTextFile.GetNextIniTextLine())
		{
			Asura_String xAsuraFilename;
			xTextFile.GetString(xAsuraFilename);

			//Determine whether this is a directory
			char szNetworkFilename[MAX_PATH];
			Asura_FileAccess::PlatformGetNetworkDir(xAsuraFilename.GetString(),szNetworkFilename,MAX_PATH);
			if(Tools_FileSystem::IsDirectory(szNetworkFilename))
			{
				//If its a directory name , then export everything within it
				AddAnimDirToExportList(m_xAsuraFileList,xAsuraFilename.GetString());			
			}
			else if(xFileContents.LoadAsuraFile(xAsuraFilename.GetString()))
			{	
				//find out if this is a skin file
				bool bIsSkinFile = xFileContents.GetSkinList()->size()!=0;

				//and add it
				AddFileToList(m_xAsuraFileList,xAsuraFilename,bIsSkinFile);
			}
			else
			{
	       		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Failed to find %s.",xAsuraFilename.GetString());
	       		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "(Referred to by %s).",pszFilename);
			}
		}
	}

	//search for the textures section
	if(!xTextFile.GetIniTextSection("Textures"))
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "No [Textures] section found.");
	}
	else
	{
		Tools_AsuraFileContents xFileContents;
		bool bIsFlagAssignmentLine = false;
		int iFlags = 0;
		while(xTextFile.GetNextIniTextLine())
		{
			Asura_String xAsuraFilename;
			xTextFile.GetString(xAsuraFilename);
			bIsFlagAssignmentLine = false;

			//are we setting texture flags?
			if( xAsuraFilename.StartsWith( "Flags" ) )
			{	
				//find the position of the '=' character if there is one
				int i = 0;
				while( (xAsuraFilename[i] != '=') && (i < xAsuraFilename.GetLength()) )
				{
					i++;
				}
				i++;
				if ( i <= xAsuraFilename.GetLength() )
				{
					//remove "Flags=" or "Flags =" or "Flags = " etc
					xAsuraFilename.Replace(0,i,"");
					xAsuraFilename.StripWhitespace( Asura_StringUtil::STRIP_BOTH_ENDS );
					//assign string value to flags variable for all following textures to be read

					iFlags = 0;
					bool bFirstTime = true;
					int iVal = 0;
					int iStartPos = 0;

					//if the string starts with "0x" skip processing it
					if( xAsuraFilename.StartsWith( "0x" ) )
					{
						iStartPos = 2;
					}
					for( i = iStartPos; i < xAsuraFilename.GetLength(); i++)
					{
						//if 0 to 9
						if(xAsuraFilename[i] >= '0' && xAsuraFilename[i] <= '9' ) 
						{
							//convert to int
							iVal = xAsuraFilename[i] & 0x0f;            
						}
						//if a to f
						else if((xAsuraFilename[i] >='A' && xAsuraFilename[i] <= 'F') || (xAsuraFilename[i] >='a' && xAsuraFilename[i] <= 'f')) 
						{
							//convert to int
							iVal = (xAsuraFilename[i] & 0x0f) + 9;
						}

						if( !bFirstTime ) iFlags <<= 4;
						iFlags |= iVal;
						bFirstTime = false;
					}

					//prevent this text file line from being added as texture
					bIsFlagAssignmentLine = true; 
				}
			}

			//add texture to export list
			if( !bIsFlagAssignmentLine )
			{
				//Determine whether this is a directory
				char szNetworkFilename[MAX_PATH];
				Asura_FileAccess::PlatformGetNetworkDir(xAsuraFilename.GetString(),szNetworkFilename,MAX_PATH);
				if(Tools_FileSystem::IsDirectory(szNetworkFilename))
				{
					//If its a directory name: todo add all textures with the directory							
				}
				else
				{
					bool bIsNew = false;

					const int iPrevTexResetGrp = Asura_Texture_Management::GetTextureResetGroup();
					Asura_Texture_Management::SetTextureResetGroup( ASURA_TEXTURE_LOADED_PERMANENT );

					int iTextureID = Asura_Texture_Management::AddTexture( xAsuraFilename, &bIsNew );

					//assign appropriate flags
					Asura_Texture_Management::SetTextureFlags(iTextureID,iFlags);
					Asura_Texture_Management::SetTextureResetGroup( iPrevTexResetGrp );

					//iTextureID = Asura_Texture_Management::GetLastAddedTexID();

					if ( ASURA_TEXTURE_NONE == iTextureID )
					{
						continue;
					}

					if ( ASURA_TEXTURE_INVALID == iTextureID )
					{
						continue;
					}

					if( true == bIsNew )
					{
						Asura_Texture_Management::LoadTextures();
					}

					if ( !Asura_Texture_Management::IsTextureIDValid( iTextureID ) )
					{
						continue;
					}

					if( !Asura_Texture_Management::IsTextureLoaded( iTextureID ) )
					{
						continue;
					}

					AddTextureToExportList(iTextureID);
				}
			}
		}
	}

// Add support for handling FSFX files to add from the export files
// Add the included FSFX to the export list
// DF 19/04/07
#ifdef ASURA_USE_FSFX
	if(!xTextFile.GetIniTextSection("FSFX"))
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "No [FSFX] section found.");
	}
	else
	{
		Tools_AsuraFileContents xFileContents;
		List<const Asura_FSFX_Template* const> xGlobalTemplateList;

		while(xTextFile.GetNextIniTextLine())
		{
			Asura_String xFSFXTemplateName;
			xTextFile.GetString(xFSFXTemplateName);

			const Asura_FSFX_Template* const pxTemplate = Asura_FSFX_System::FindTemplate( xFSFXTemplateName.GetString() );

			if ( ( pxTemplate ) && ( !xGlobalTemplateList.contains( pxTemplate ) ) )
			{
				xGlobalTemplateList.add_entry( pxTemplate );
			}
		}
		Tools_Mode_FSFX::ExportTemplateList(&m_xAsuraFileList, xGlobalTemplateList);
	}
#endif	//ASURA_USE_FSFX

	// Add support for handling PFX files to add from the export files
	// Add the included PFX to the export list
	// DF 19/04/07
	if(!xTextFile.GetIniTextSection("PFX"))
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "No [PFX] section found.");
}
	else
	{
		Tools_AsuraFileContents xFileContents;

		while(xTextFile.GetNextIniTextLine())
		{
			Asura_String xPFXFileName;
			xTextFile.GetString(xPFXFileName);

			//split filename
			Asura_String xTemplateName,xExtension;
			xPFXFileName.SplitFilename(0,&xTemplateName,&xExtension);

			Asura_Hash_ID uHash = Asura_GetHashID(xTemplateName.GetString());

			if(!xExtension.Compare("pfx"))
			{
				if(!Tools_Mode_PFX::AddTemplateToExportListByHash(&m_xAsuraFileList, uHash, ASURA_GUID_UNREGISTERED))
				{
					Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Failed to find template '%s'",xPFXFileName.GetString());
				}
			}
			else
			{
				Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Unexpected file type %s in %s",xPFXFileName.GetString(),pszFilename);
			}
		}
	}

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	if ( bAllowMaterials )
	{
		// Add support for handling materials files to add from the export files
		// Material file names are stored in a Asura_NameTree to be handled later.
		// DF 19/04/07
		if (!xTextFile.GetIniTextSection("Materials"))
		{
			Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "No [Materials] section found.");
		}
		else
		{
			while(xTextFile.GetNextIniTextLine())
			{
				Asura_String xMaterialName;
				if( !xTextFile.GetString( xMaterialName ) ) continue;

				int iJunk;
				m_pxMaterialsFromCommonExport->FindNodeIndexByName(xMaterialName, true, iJunk);
			}
		}
	}
#endif //ASURA_USE_MATERIAL_RESPONSE_SYSTEM
}

//calls AddFileToIgnore for each file in [asura files] section 
//and AddTextureToIgnore for each file in [Textures] section of text file.
void Tools_Level_Export::IgnoreAsuraFilesInTextFile(const char* pszFilename)
{
	Tools_IniFile xTextFile(pszFilename);
	if(!xTextFile.LoadIniFile())
	{
       	Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "Failed to open %s.",pszFilename);
		return;
	}
	else
	{
       	Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "Ignoring files listed in %s.",pszFilename);
	}

	//search for the asura file section
	if(!xTextFile.GetIniTextSection("Asura Files"))
	{
       	Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "No [Asura Files] section found.");
	}
	else
	{
		Tools_AsuraFileContents xFileContents;
		while(xTextFile.GetNextIniTextLine())
		{
			Asura_String xAsuraFilename;
			xTextFile.GetString(xAsuraFilename);

			//Determine whether this is a directory
			char szNetworkFilename[MAX_PATH];
			Asura_FileAccess::PlatformGetNetworkDir(xAsuraFilename.GetString(),szNetworkFilename,MAX_PATH);

			if(Tools_FileSystem::IsDirectory(szNetworkFilename))
			{
				//If its a directory , ignore everything within it
				AddAnimDirToIgnore(xAsuraFilename.GetString());
			}
			else
			{
				xAsuraFilename.MakeLowercase();
				AddFileToIgnore( xAsuraFilename.GetString() );
			}
		}
	}

	//search for the textures section
	if(!xTextFile.GetIniTextSection("Textures"))
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "No [Textures] section found.");
	}
	else
	{
		Tools_AsuraFileContents xFileContents;
		while(xTextFile.GetNextIniTextLine())
		{
			Asura_String xAsuraFilename;
			xTextFile.GetString(xAsuraFilename);

			//Determine whether this is a directory
			char szNetworkFilename[MAX_PATH];
			Asura_FileAccess::PlatformGetNetworkDir(xAsuraFilename.GetString(),szNetworkFilename,MAX_PATH);

			if(Tools_FileSystem::IsDirectory(szNetworkFilename))
			{
				//If its a directory: todo ignore the whole directory
			}
			else
			{
				xAsuraFilename.MakeLowercase();
				AddTextureToIgnore( xAsuraFilename.GetString() );
			}
		}
	}

#ifdef ASURA_USING_ALL_LEVEL_EXPORT

	// Add support for ignoring PFX files from the export files
	// Add the PFX to the ignore list to be handled later
	// DF 19/04/07
	if(!xTextFile.GetIniTextSection("PFX"))
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "No [PFX] section found.");
	}
	else
	{
		Tools_AsuraFileContents xFileContents;
		while(xTextFile.GetNextIniTextLine())
		{
			Asura_String xPFXFileName;
			xTextFile.GetString(xPFXFileName);

			//split filename
			Asura_String xTemplateName,xExtension;
			xPFXFileName.SplitFilename(0,&xTemplateName,&xExtension);

			Asura_Hash_ID uHash = Asura_GetHashID(xTemplateName.GetString());

			AddPFXToIgnore( uHash );
		}
	}
	#ifdef ASURA_USE_FSFX
	// Add support for ignoring FSFX files from the export files
	// Add the FSFX to the ignore list to be handled later
	// DF 19/04/07
	if (!xTextFile.GetIniTextSection("FSFX"))
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "No [FSFX] section found.");
	}
	else
	{
		Tools_AsuraFileContents xFileContents;
		while(xTextFile.GetNextIniTextLine())
		{
			Asura_String xFSFXName;
			xTextFile.GetString(xFSFXName);
			Asura_Hash_ID uHash = Asura_GetHashID(xFSFXName.GetString());

			Tools_Mode_FSFX::AddFSFXToIgnoreList(uHash);
		}
	}
#endif // ASURA_USE_FSFX
#endif //ASURA_USING_ALL_LEVEL_EXPORT
}

bool Tools_Level_Export::LoadAsuraFilesFromTextFile(const char* pszFilename)
{
	Tools_IniFile xTextFile(pszFilename);
	if(!xTextFile.LoadIniFile())
	{
		return false;
	}
	


	//search for the asura file section
	if(!xTextFile.GetIniTextSection("Asura Files"))
	{
		return false;
	}

	//build a list of all files to load
	List<Asura_String> xFileList;

	while(xTextFile.GetNextIniTextLine())
	{
		Asura_String xAsuraFilename;
		xTextFile.GetString(xAsuraFilename);

		//Determine whether this is a directory
		char szNetworkFilename[MAX_PATH];
		Asura_FileAccess::PlatformGetNetworkDir(xAsuraFilename.GetString(),szNetworkFilename,MAX_PATH);

		if(Tools_FileSystem::IsDirectory(szNetworkFilename))
		{
			//If its a directory , then get a list of files in a manner similar to AddAnimDirToExportList
			bool bGotExportList = ProcessExportList(szNetworkFilename,&xFileList,&xFileList);
			if(!bGotExportList)
			{
		        char szSearch[MAX_PATH + 1];
		        Asura_PrintfToBfr( szSearch, "%s\\*.asr", MAX_PATH + 1, true, szNetworkFilename );      
				Tools_FileSystem::FindFiles( szSearch, xFileList, TOOLS_FILEFIND_RECURSIVE|TOOLS_FILEFIND_IGNOREBACKUPDIR );
			}

		}
		else
		{
			xFileList.add_entry(xAsuraFilename);
		}
	}

	//Load all the files found
	for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next())
	{
		//ignore files that have already been loaded
		if(Tools_LoadedFileList::IsFileLoaded(xFileLIF().GetString())) continue;
		
		//check for an existing anim of the same name , and skip if it exists
		//(this is just to get round standard_rest_anim that gets created automatically , but also has a file)
		Asura_String xName;
		xFileLIF().SplitFilename(0,&xName,0);
		if(Asura_Animation_System::GetAnim(xName.GetString())) continue;

		LoadAsuraFile(xFileLIF().GetString());
	}


	//search for the textures section. This can do exactly the same as the add files.
	if(!xTextFile.GetIniTextSection("Textures"))
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY, "No [Textures] section found.");
	}
	else
	{
		Tools_AsuraFileContents xFileContents;
		bool bIsFlagAssignmentLine = false;
		int iFlags = 0;
		while(xTextFile.GetNextIniTextLine())
		{
			Asura_String xAsuraFilename;
			xTextFile.GetString(xAsuraFilename);
			bIsFlagAssignmentLine = false;

			//are we setting texture flags?
			if( xAsuraFilename.StartsWith( "Flags" ) )
			{	
				//find the position of the '=' character if there is one
				int i = 0;
				while( (xAsuraFilename[i] != '=') && (i < xAsuraFilename.GetLength()) )
				{
					i++;
				}
				i++;
				if ( i <= xAsuraFilename.GetLength() )
				{
					//remove "Flags=" or "Flags =" or "Flags = " etc
					xAsuraFilename.Replace(0,i,"");
					xAsuraFilename.StripWhitespace( Asura_StringUtil::STRIP_BOTH_ENDS );
					//assign string value to flags variable for all following textures to be read

					iFlags = 0;
					bool bFirstTime = true;
					int iVal = 0;
					int iStartPos = 0;

					//if the string starts with "0x" skip processing it
					if( xAsuraFilename.StartsWith( "0x" ) )
					{
						iStartPos = 2;
					}
					for( i = iStartPos; i < xAsuraFilename.GetLength(); i++)
					{
						//if 0 to 9
						if(xAsuraFilename[i] >= '0' && xAsuraFilename[i] <= '9' ) 
						{
							//convert to int
							iVal = xAsuraFilename[i] & 0x0f;            
						}
						//if a to f
						else if((xAsuraFilename[i] >='A' && xAsuraFilename[i] <= 'F') || (xAsuraFilename[i] >='a' && xAsuraFilename[i] <= 'f')) 
						{
							//convert to int
							iVal = (xAsuraFilename[i] & 0x0f) + 9;
						}

						if( !bFirstTime ) iFlags <<= 4;
						iFlags |= iVal;
						bFirstTime = false;
					}

					//prevent this text file line from being added as texture
					bIsFlagAssignmentLine = true; 
				}
			}

			//add texture to export list
			if( !bIsFlagAssignmentLine )
			{
				//Determine whether this is a directory
				char szNetworkFilename[MAX_PATH];
				Asura_FileAccess::PlatformGetNetworkDir(xAsuraFilename.GetString(),szNetworkFilename,MAX_PATH);
				if(Tools_FileSystem::IsDirectory(szNetworkFilename))
				{
					//If its a directory name: todo add all textures with the directory							
				}
				else
				{
					bool bIsNew = false;

					const int iPrevTexResetGrp = Asura_Texture_Management::GetTextureResetGroup();
					Asura_Texture_Management::SetTextureResetGroup( ASURA_TEXTURE_LOADED_PERMANENT );

					int iTextureID = Asura_Texture_Management::AddTexture( xAsuraFilename, &bIsNew );

					//assign appropriate flags
					Asura_Texture_Management::SetTextureFlags(iTextureID,iFlags);
					Asura_Texture_Management::SetTextureResetGroup( iPrevTexResetGrp );

					//iTextureID = Asura_Texture_Management::GetLastAddedTexID();

					if ( ASURA_TEXTURE_NONE == iTextureID )
					{
						continue;
					}

					if ( ASURA_TEXTURE_INVALID == iTextureID )
					{
						continue;
					}

					if( true == bIsNew )
					{
						Asura_Texture_Management::LoadTextures();
					}
					// No need to do anything further.
				}
			}
		}
	}

	return true;
}

void Tools_Level_Export::PreAddAsuraFile( HANDLE hFile, const Asura_String& xFilename)
{
    // Set some default progress text in some cases.
    {
        Asura_String xStatus;

        if( IsSkinFile(xFilename) )
        {
	        Asura_String xSkinName;
	        xFilename.SplitFilename( NULL, &xSkinName, NULL );
            
            xStatus = "Adding skin \"";
            xStatus += xSkinName;
            xStatus += "\"...";
        }

        if( !xStatus.IsBlank() )
        {
            UpdateStatus( EXPORT_STAGE_ADDING_SKIN, xStatus.GetString(), -1 );
        }
    }

	if(IsEnvironmentFile(xFilename))
	{
		//UV fiddling left to last moment now , to maximise chance of all texture flags etc being sorted properly.
		//(An issue with PS2 exports , since it needs to be able to get the texture sizes)
		//RAR 16/05/2006
		AdjustAllUVCoordinates();

	}
}

bool Tools_Level_Export::AddAsuraFile( HANDLE hFile, const Asura_String& filename, unsigned uIndexWithinFileList, bool bMissingFileIsError )
{
	#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
		#if defined(ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY)
			//Allow old material chunks while adding environment files
			if(IsEnvironmentFile(filename))
	{
				m_bIncludeOldMaterialChunks = true;
			}
		#else
			bool bOldIncludeOldMaterialChunks = m_bIncludeOldMaterialChunks;
	{
				// DLH - 01 May 2007 17:38:24 - Allow NON-ASR files to export the TEXTURENAME chunk
				Asura_String xFilename( filename);
				Asura_String xExtention;
				xFilename.SplitFilename( NULL, NULL, &xExtention);
				if( 0 != xExtention.Compare( "asr", true))
		{
					m_bIncludeOldMaterialChunks = true;
		}
	}
		#endif
	#endif

	bool bRetVal = AddAsuraFileMain(hFile, filename, uIndexWithinFileList, bMissingFileIsError);

	#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
		#if defined(ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY)
			//reset inclusion of material chunks
			m_bIncludeOldMaterialChunks = false;
		#else
			//reset inclusion of material chunks
			m_bIncludeOldMaterialChunks = bOldIncludeOldMaterialChunks;
		#endif
	#endif


	return bRetVal;
	}


bool Tools_Level_Export::AddAsuraFileMain( HANDLE hFile, const Asura_String& filename, unsigned uIndexWithinFileList, bool bMissingFileIsError )
{
	//ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::AddAsuraFileMain");

	PreAddAsuraFile(hFile,filename);

	char szFileToLoad[MAX_PATH + 1];

    // (MP - 13.03.2006) If there's a big endian version of the file
    // available then use that instead.
    bool bReplacedWithBE = false;
    if( ShouldExportBigEndian() && filename.EndsWith(ASURA_STANDARD_FILE_EXTENSION_LE) )
    {
        Asura_String xBEFilename = filename;
        xBEFilename += ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
        Asura_FileAccess::PlatformGetNetworkFilename( xBEFilename.GetString(), szFileToLoad, MAX_PATH + 1 );

        // See if that file exists, wherever we've been pointed to.
        HANDLE pvHandle = CreateFile( szFileToLoad, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
        if( INVALID_HANDLE_VALUE != pvHandle )
        {
            CloseHandle( pvHandle );
            bReplacedWithBE = true;
        }
    }

    if( !bReplacedWithBE )
    {
        Asura_FileAccess::PlatformGetNetworkFilename( filename.GetString(), szFileToLoad, MAX_PATH + 1 );
    }

    // Assume we'll fail unless we discover otherwise.
    g_axResourceFileListEntries[uIndexWithinFileList].bFileAddedOK = false;

	//read and decompress the file
	Tools_ReadAsuraFileOutput xReadFileOutput;
	char* buffer = Tools_FileSystem::ReadAsuraFile(szFileToLoad, &xReadFileOutput);


	if(!buffer)
	{
		switch(xReadFileOutput.m_uErrorCode)
	{
		case TOOLS_READASURAFILE_FAILED_TO_OPEN_FILE :
        if( bMissingFileIsError )
        {
            AddMessage( MT_ERROR, "ERROR: Failed to add Asura file \"%s\".", szFileToLoad );
            return false;
        }
        else
        {
		    AddMessage( MT_WARNING, "WARNING: Failed to open %s for reading.", szFileToLoad );
            return true;
        }

		case TOOLS_READASURAFILE_NOT_ASURA_FILE :
			Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "\"%s\" isn't an asura file.", szFileToLoad );
			return false;

		case TOOLS_READASURAFILE_ERROR_READING_FILE :
		    Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error while reading \"%s\".", szFileToLoad );
            return false;

		default :
			Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Error while reading \"%s\".", szFileToLoad );
		return false;
	}
	}


	Tools_Level_Export::AddMessage( Tools_Level_Export::MT_LOGONLY,
		"Adding %s%s",
		szFileToLoad,
		(bReplacedWithBE ? " (replaced LE file)" : "") );

	
	const char* chunkBuffer = buffer;
	
    // Get cur file ptr pos.
    const DWORD uFilePtrPosBeforeWrite = SetFilePointer( hFile, 0, NULL, FILE_CURRENT );

    unsigned uTotalNumBytesWritten = 0;
	const Asura_Chunk_Header* chunk = (Asura_Chunk_Header*) chunkBuffer;

	/// Kim Randell - 24/03/2003 15:07:46 - Easiest way of checking whether the file we are
	/// adding is big-endian or not. This has to be done because the Gamecube export must
	/// handle both the level specific ancillary files, such as entities and lighting, which
	/// have already been switched ath this point in the export, and the animation files,
	/// which have yet to be switched.
    bool bFileIsBigEndian = Asura_StringUtil::EndsWith( szFileToLoad, ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX, true );
	
	if( bFileIsBigEndian && !ShouldExportBigEndian() )
	{
		// We don't want to be reading this for little-endian architectures
		delete [] buffer;
		return true;
	}

	Asura_Chunk_Header* pxChunkCopy;
	if(bFileIsBigEndian)
	{
		// This will flip the header as necessary
		pxChunkCopy = new Asura_Chunk_Header(chunk->ID, chunk->Size, chunk->Version, chunk->Flags);
	}
	else
	{
		pxChunkCopy = (Asura_Chunk_Header*)chunk;
	}

	while(pxChunkCopy->Valid())
	{
		Asura_Assert(pxChunkCopy->Size,"");

		if( ShouldExportBigEndian() && !bFileIsBigEndian)
		{
			uTotalNumBytesWritten += ExportBigEndianAnimation(pxChunkCopy, hFile);
		}
		if(!ShouldExportBigEndian() || bFileIsBigEndian)
		if(ShouldChunkBeExported(pxChunkCopy->ID))
		{ 
			const Asura_Chunk_Header* pxChunkToWrite = chunk;
			char* pcModifedChunkBuffer = 0;
            
			// This could interact dangerously with my Gamecube exporter,
			// which parses big endian headers, flips them, and copies big endian bodies.
			// However, at present ModifyChunkBeforeOutput only affects PS2 exports
            DWORD uBytesToWrite = pxChunkCopy->Size;
			if(ModifyChunkBeforeOutput(filename,chunk,&pcModifedChunkBuffer))
			{
				pxChunkToWrite = (const Asura_Chunk_Header*) pcModifedChunkBuffer;
				Asura_Assert(pxChunkToWrite,"");
				uBytesToWrite = pxChunkToWrite->Size;
				ASURA_CHUNK_VERIFY(uBytesToWrite,u_int);
			}
			
			
			
			DWORD bytesWritten;
			WriteFile( hFile, pxChunkToWrite,uBytesToWrite, &bytesWritten, NULL );

			if( bytesWritten != uBytesToWrite )
            {
                Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error." );
                delete [] buffer;
				delete [] pcModifedChunkBuffer;
                return false;
            }
			
			uTotalNumBytesWritten += bytesWritten;
			delete [] pcModifedChunkBuffer;
	
		}

		//advance to the next chunk

		// KJFL - 27 June 2005 - handy when debugging endianess to know what the last chunk was...
		//Asura_Chunk_Header* pxPreviousChunk = (Asura_Chunk_Header*)chunkBuffer;

		chunkBuffer += pxChunkCopy->Size;
		chunk = (Asura_Chunk_Header*) chunkBuffer;
		if(!chunk->Valid())
		{
			// KOR - 16/12/2008 - This avoids attempting to read beyond the end of the buffer,
			// if the invalid header only has the first 4 bytes.
			break;
		}
		if(bFileIsBigEndian)
		{
			delete pxChunkCopy;
			pxChunkCopy = new Asura_Chunk_Header(chunk->ID, chunk->Size, chunk->Version, chunk->Flags);
		}
		else
		{
			pxChunkCopy = (Asura_Chunk_Header*)chunk;
		}
	}
	if(bFileIsBigEndian)
	{
		delete pxChunkCopy;
	}

    // Get new file ptr pos.
    const DWORD uFilePtrPosAfterWrite = SetFilePointer( hFile, 0, NULL, FILE_CURRENT );
    Asura_Assert( uFilePtrPosAfterWrite - uFilePtrPosBeforeWrite == uTotalNumBytesWritten, "" );

	delete [] buffer;

    // Fill out resource file list entry now we've succeeded in adding the
	// Asura file.
    g_axResourceFileListEntries[uIndexWithinFileList].bFileAddedOK = true;
    g_axResourceFileListEntries[uIndexWithinFileList].iByteOffset  = uFilePtrPosBeforeWrite - g_uBytesReservedForFileListChunk;
    g_axResourceFileListEntries[uIndexWithinFileList].iNumBytes    = uFilePtrPosAfterWrite - uFilePtrPosBeforeWrite;

    if( !PostAddAsuraFile(hFile,filename) ) { return false; }

    // Ok.
    return true;
}

bool Tools_Level_Export::PostAddAsuraFile(HANDLE hFile, const Asura_String& xFilename)
{
	//ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::PostAddAsuraFile");

	//for skins ,need to add a stripified skin resource
	if(IsSkinFile(xFilename))
	{
	    Asura_String xSkinName;
	    xFilename.SplitFilename( NULL, &xSkinName, NULL );

	    Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin( xSkinName.GetString() );

        if( pxSkin )
        {
			AdjustUVCoordinates(pxSkin);
			if( ShouldExportBigEndian() )
			{
				// This must come before the SkinSectionBBChunk
				if(!AddBigEndianSkinChunks(hFile, pxSkin)) { return false; }
			}

            if( !AddSkinSectionBBChunk(hFile, xSkinName, pxSkin) ) { return false; }

            if( !AddStripifiedSkin(hFile, pxSkin) ) { return false; }
        }
        else
        {
            Asura_Assertf( false, ("Skin \"%s\" doesn't exist.", xSkinName.GetString()) );
        }
	}

	if(IsEnvironmentFile(xFilename))
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::PostAddAsuraFile Environment");
		
		AddStripifiedEnvironment(hFile);
		#ifdef ASURA_USE_WATER
		if(!IsAbortFlagSet())
		{
			AddWaterPackets(hFile);
		}
		#endif
		#if defined( ASURA_USE_CARPET )
		if(!IsAbortFlagSet())
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::PostAddAsuraFile Carpet");
	
			if ( AddCarpetMeshData( hFile ) == false ) return false;
		}
		#endif // #if defined( ASURA_USE_CARPET )
		#if defined( ASURA_USE_BOTANICALS )
		if(!IsAbortFlagSet())
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::PostAddAsuraFile Botanical");

			if ( AddBotanicalsData( hFile ) == false ) return false;
		}
		#endif // #if defined( ASURA_USE_BOTANICALS )
		#if defined( ASURA_USE_STATIC_DECAL_SYSTEM )
		if(!IsAbortFlagSet())
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::PostAddAsuraFile StaticDecals");

			if ( AddStaticDecalsData( hFile ) == false ) return false;
		}
		#endif // #if defined( ASURA_USE_BOTANICALS )

#ifdef ASURA_USE_ATTRACTORS
		//Ignore attractors for mission exports. They will get included elsewhere.
		if ( !IsAbortFlagSet() && !MissionExport_IsExportingEnvironmentOnly())
		{
			if ( !AddAttractorData( hFile ) )
			{
				return false;
			}
		}
#endif
		#if defined ( ASURA_USE_TEXTURE_STREAMING )
		if( !IsAbortFlagSet() && ShouldExportStreamingTexturesFile() && s_bIsLevelExport )
		{
			// The general behaviour of GenerateTextureIfNecessary() when the streaming code is active
			// is to make small textures as part of a normal export. (Since the big ones are streamed.)
			// So currently we pretend we're not a level export to get the big textures.
			s_bIsLevelExport = false;
			ExportStreamingTexturesFile();
			s_bIsLevelExport = true;

			Asura_SaveBlock xSaveBlock;
	
			Asura_Chunk_SaveBlock_Stream xStream( &xSaveBlock );
			Asura_Chunk_Texture_Streaming::Create( xStream );

			xSaveBlock.WriteToFile( hFile );
	}
		#endif
	}

    // Final progress update, if necessary, and then clear the status heading.
    if( !s_xHeading.IsBlank() )
    {
        s_iPercent = 100;
        UpdateProgress();
		UpdateStatus(EXPORT_STAGE_END, NULL, -1);
    }

    return true;
}

/// End of file. ///