/**
*
* \file Tools_Level_Export.cpp
*
* Created by Kevin Lea - 29/11/2002 11:23:31
* Splitted into separate file 12/11/2008
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

// core
#include "Core/Asura.h"

// header
#include "Load&Save/LevelExport/Tools_Level_Export.h"

// Asura
#include "AI/Asura_AI_Navigation.h"
#include "Core/Asura_LocalisedText.h"
#include "Core/Asura_TextureManagement.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Light/Asura_GI_Solution.h"
#include "Physics/Asura_Physics_Environment.h"
#include "SpecialFX/Asura_Fragment_System.h"

// Tools
#include "Entity/Tools_EntityFilterOverlay.h"
#include "Entity/Tools_Entity_PhysicalObject.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Lighting/Tools_EnvironmentalLightTransport.h"
#include "Lighting/Tools_PhotonMapping_System.h"
#include "Lighting/Tools_LightSets.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_BatchProcessing.h"
#include "Mesh/Tools_PlatformMesh_System.h"
#include "Misc/Tools_IniFile.h"
#include "Misc/Tools_Render_Options.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Shapes/Tools_Shapes.h"
#include "Smoothing/Tools_ModuleSmoothing.h"
#include "Texturing/Tools_Texture_Management.h"
#include "Texturing/Tools_Mode_Texturing.h"
#include "Texturing/Tools_Texturing_Find.h"
#include "Tools_System.h"

#ifdef ASURA_USE_AXON
#include "ChunkSystem/Axon_Chunk_AnimTemplates.h"
#include "Entity/Locations/OrientedNode/CoverpointNode/Tools_EntityNode_CoverpointNode.h"
#endif

#ifdef ASURA_USE_BILLBOARD_PLANTS
#include "SpecialFX/Asura_BillboardPlant_System.h"
#include "SpecialFX/Asura_BillboardPlant_Batch.h"
#include "SpecialFX/Asura_BillboardPlant_TemplateStore.h"
#endif //ASURA_USE_BILLBOARD_PLANTS

#if defined ( ASURA_USE_TEXTURE_STREAMING )
#include "Core/Asura_Texture_Streaming.h"
#include "ChunkSystem/Asura_Chunk_Texture_Streaming.h"
#endif

#if defined(ASURA_TARGET_SM3)
#include "Rendering/Asura_Direct3D_EnvironmentRenderer.h"
#endif	//defined(ASURA_TARGET_SM3)

#ifdef ASURA_USE_CARPET
#include "Carpet/Asura_Carpet_MeshManager.h"
#include "Carpet/Asura_Carpet_StyleManager.h"
#include "Carpet/Asura_Direct3D_Carpet_Game_Render.h"
#include "Carpet/Tools_Carpet_Mesh.h"
#endif //ASURA_USE_CARPET

#ifdef ASURA_USE_ATTRACTORS
#include "ChunkSystem/Asura_Chunk_Attractors.h"
#include "Attractors/Tools_Attractor_System.h"
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
#include "Decals/Tools_StaticDecal_System.h"
#endif

// S T A T I C    I N I T /////////////////////////////////////////////////////////
static bool g_bAcceptInvalidLighting = false;

extern Asura_HashTree g_xMissingMaterialsEncountered;

void Tools_Level_Export::ExportLevelForEachPlatform()
{
	Asura_Assert(This,"");

	for(int i=0;i<TOOLS_PLATFORM_NUM_PLATFORMS;i++)
	{
		if(!IsPlatformSetToExport((EXPORT_TYPE)i)) continue;
		if(!This->IsPlatformAllowed((EXPORT_TYPE)i)) continue;

		ExportLevel(i);
		if(IsUserAbortFlagSet()) break;
	}

	if(!IsUserAbortFlagSet())
	{
	    // Prompt the user to press a key before continuing, to ensure they see
	    // any important error messages.
    	if( This->ShouldPauseAtEndOfExport())
		{
			This->PauseAtEndOfExport();
		}
	}
}

void Tools_Level_Export::ExportLevel(int iPlatform)
{
	s_eExportType = (EXPORT_TYPE)iPlatform;

	// (AAM - 23.01.2008) This needs to be static so it doesn't get destroyed at the end of the
	// function. The pause at the end of export is now handled outside this method
	static char szHeading[100];
	Asura_PrintfToBfr(szHeading,"%s Level Export",sizeof(szHeading),true,GetPlatformName(iPlatform));
	s_szMainHeading = szHeading;

	Asura_Assert(This,"");
	for(Tools_UsedLightSetIterator xIt(TOOLS_LIGHTSETITERATOR_EXPORT); !xIt.Done(); xIt.Next())
	{
		if((s_uLightSetsToExport & xIt.GetMaskedCurrent()) == xIt.GetMaskedCurrent())
		{
			s_uCurrentLightSetsToExport = xIt.GetCurrent();

			#ifndef TOOLS_DISABLE_SEPARATE_MISSION_EXPORT
			if(Tools_SaveAsura::IsEditingMission())
			{
				//export a file containing just the common environment data
				if(s_bMission_ShouldExportEnvironment)
				{
					s_bMission_IsExportingEnvironment = true;
					This->ExportLevel();
					s_bMission_IsExportingEnvironment = false;
				}
				//export a file containing the entities etc specific to the mission
				if(s_bMission_ShouldExportMission && !IsUserAbortFlagSet())
				{
					s_bMission_IsExportingMission = true;
					This->ExportLevel();
					s_bMission_IsExportingMission = false;
				}
			}
			else
			#endif
			{
	This->ExportLevel();
			}
		}
		if(IsUserAbortFlagSet()) break;
	}



	//Reset type to PC at the end to be on the safe side.
	s_eExportType = Export_PC;
}



void Tools_Level_Export::DoFakeLevelExport()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Level_Export::DoFakeLevelExport" );
	
	s_eExportType = Export_PC;
	s_szMainHeading = "Loading Files";
	Asura_Assert(This,"");

	//overide some of the options for a fake export
	bool bDontExportCutscenes = s_bDontExportCutscenes;
	s_bDontExportCutscenes = false;


	
	This->m_bFakeLevelExport = true;
	This->ExportLevel();
	This->m_bFakeLevelExport = false;

	//load any skin sounds
	Tools_Animation_Files::LoadSoundsForAllSkinsAndAnims();

	//reset the settings that were altered
	s_bDontExportCutscenes = bDontExportCutscenes;
}


void Tools_Level_Export::ExportLevel()
{
	ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::ExportLevel");
	
	//Early check to make sure destination files are writeable. RAR 17/01/2006
	if(!m_bFakeLevelExport && !IsExportedLevelFileWriteable())
	{
		return;
	}

    // (MH - 07.11.2002) switch to standby mode so we can later create all tools objects,
    // safe in the knowledge that they don't already exist.
    Tools_System::RequestStandbyMode(true);

	// KJFL - 2 April 2008 - if we're using streaming textures now's a good time to clean
	// the system and register all the textures we need to consider. Then they can be exported
	// with either small or large versions, as required.
	#if defined ( ASURA_USE_TEXTURE_STREAMING )
	if( ShouldExportStreamingTexturesFile() )
	{
		if(Tools_BatchProcessing::IsBatch())
		{
			//////////////////////////////////////////////////////////////////////////
			// NOTE: Texture-streaming is PROBABLY only appropriate to SM3 platforms,
			// but just to be on the safe side I've wrapped the following call, if
			// someone does decide we need texture-streaming on pre-SM3 platforms
			// then we'll need to implement an equivalent - ART 28/07/2008.
			//////////////////////////////////////////////////////////////////////////
#if defined(ASURA_TARGET_SM3)
			// Ensure that we have strips (normally done at render time, but when we batch-process there is no rendering)...
			Asura_Direct3D_EnvironmentRenderer::Tools_CreateData();
#endif	//defined(ASURA_TARGET_SM3)
		}
	
		Asura_Texture_Streaming_System::Reset();
		Asura_Texture_Streaming_System::Tools_FindStreamingTextures();
	}
	else
	{
		// Make sure we don't have any streaming textures set up if we're not using them.
		Asura_Texture_Streaming_System::Reset();
	}
	#endif
	
	if(s_bUpdateExistingFile && !m_bFakeLevelExport)
	{
		UpdateExportedLevel();
		return;	
	}
    
    
	
	extern Asura_String EnvironmentFilename;
	Asura_String xTmpFilename;

	Asura_String xBaseEnvName;
	if(m_bFakeLevelExport)
		xBaseEnvName = EnvironmentFilename;
	else if(Tools_SaveAsura::IsEditingMission() && !MissionExport_IsExportingEnvironmentOnly())
		xBaseEnvName = Tools_SaveAsura::GetMissionFilenameWithoutExtension();
	else
		xBaseEnvName = Tools_SaveAsura::GetMainFilename();

	StartExport();
	OpenExportLog(m_bFakeLevelExport ? "QuitToGame" : xBaseEnvName.GetString());
	
	StartLevelExport();

	UpdateStatus( EXPORT_STAGE_INITIALISING, "Initialising...", -1 );

	if(!m_bFakeLevelExport)
	{
		if(!PreLevelExportChecks())
		{
			SetAbortFlag();
			EndLevelExport();
			CloseExportLog();
			EndExport(false);
			return;
		}
	}

	bool bChangedPrelightingData = false;

	if(!m_bFakeLevelExport && !MissionExport_IsExportingMissionOnly())
	{
		// make sure normals are up to date, which may affect prelighting
		Tools_ModuleSmoothing::CheckModuleNormals();

		bChangedPrelightingData = CheckPrelightingIsCorrect();
		
		InitialiseSubdivision();
	}

	if(s_bIncludeDualTexturingData && s_eExportType == Export_PS2)
	{
		//check the secondary texture useage
		int iNumTextures;
		float fTextureUse;
		List<int> xTextureList;
		Tools_Mode_Texturing::GetPS2SecondaryTextureUsage(iNumTextures,fTextureUse,&xTextureList);
		if(iNumTextures > TOOLS_MAX_SECONDARY_TEXTURES)
		{
			AddMessage(MT_ERROR,"More than %d secondary textures used",TOOLS_MAX_SECONDARY_TEXTURES);			
			SetAbortFlag();
		}
		if(fTextureUse > 1.0f)
		{
			AddMessage(MT_ERROR,"Total size of secondary textures used is too high");			
			SetAbortFlag();
		}

		//make sure the textures don't have an alpha channel
		for(LIF<int> xTextureLIF(&xTextureList);!xTextureLIF.done();xTextureLIF.next())
		{
			const Asura_String* pxName = Asura_Texture_Management::GetTextureName(xTextureLIF());
			Asura_String xFilename;
			if(GetTextureFilenameForExport(pxName->GetString(),"tm2",xFilename,0))
			{
				if(DoesTextureHaveAlphaChannel(xFilename.GetString()))
				{
					AddMessage(MT_ERROR,"Secondary texture '%s' has an alpha channel",pxName->GetString());
					SetAbortFlag();
				}
			}
		}
	}

	//Ensure fragment shapes have been loaded
	if(!IsAbortFlagSet())
	{
		Asura_Fragment_System::LoadFragments();
	}

	if(!IsAbortFlagSet())
	{
		StartFileExport();
		//all shapes are wanted for environment exports
		if(!MissionExport_IsExportingEnvironmentOnly())
		{
		SetIncludeEntityShapes(true);
		SetIncludeCommonShapes(true);
	}

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
		if ( !m_bFakeLevelExport )
		{
			//If its not a fake export, ignore the files in the Common_Export file
			//Leave them in if its a fake export
			// DF 19/04/07
			IgnoreAsuraFilesInTextFile(GetCommonExportFile());
		}
#endif
	}

	if(!IsAbortFlagSet())
	{
		//make sure shapes are flagged as needed for export
		FlagShapeTableEntriesForExport();
	}
	
	
	if(!IsAbortFlagSet())
	{
		//Work out which shapes are in use by entities
		Tools_ShapeTable::RemoveZeroReferenceCountElements();
	}

	// JGP - Update compiled attractors
#ifdef ASURA_USE_ATTRACTORS
	if ( !IsAbortFlagSet() )
	{
		Tools_Attractor_System::GetInstance()->CompileAttractors();
	}
#endif

	if(!IsAbortFlagSet())
	{
		if( !m_bFakeLevelExport )
		{
            if(!IsExportingAllModules() && !MissionExport_IsExportingMissionOnly())
			{
				//Not exporting all modules, so we need to work out which waypoints to include
            	UpdateStatus( EXPORT_STAGE_GENERATING_WAYPOINTS_SUBSET, "Generating subset of waypoints to export...", -1 );
				Asura_AI_Navigation_System::GenerateSubsetForExport();
			}
		}
	}

	if(!IsAbortFlagSet())
	{
		if( !m_bFakeLevelExport )
		{
            UpdateStatus( EXPORT_STAGE_TEMPORARY_FILES_SAVE, "Saving temporary files for export...", -1 );

			#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM) && defined(ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY)
			//Allow old material chunks while saving the environment files
			m_bIncludeOldMaterialChunks = true;
			#endif

			//make sure the individual asura files are up to date first.
			Tools_SaveAsura::SaveAll();	

			#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM) && defined(ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY)
			//reset inclusion of material chunks
			m_bIncludeOldMaterialChunks = false;
			#endif

		}
	}

	if(!IsAbortFlagSet())
	{
		//create the environment file
		if( !MissionExport_IsExportingEnvironmentOnly())
		{
            UpdateStatus( EXPORT_STAGE_COMPILE_INCLUDE_FILE_LIST, "Compiling list of files to include...", -1 );

			ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::ExportLevel CompilingListOfFilesToInclude");

			AddInGameFilesToList( &m_xAsuraFileList, EnvironmentFilename );
		}


#ifdef ASURA_USING_ALL_LEVEL_EXPORT
		//Need to wait until here to do this so we can handle shapes getting added from AddInGamefilesToList (engine and game side)
		//DF 19/04/07 
		if ( !m_bFakeLevelExport )
		{
			//Remove Shapes flagged to be exported if they're in the common_export file
			Tools_ShapeTable::ExcludeCommonShapesFromLevelExport();
		}
#endif //ASURA_USING_ALL_LEVEL_EXPORT
		ListAsuraFilesForExport(m_xAsuraFileList);

		//Asura_String xBaseEnvName;
		//if(m_bFakeLevelExport)
			//xBaseEnvName = EnvironmentFilename;
		//else if(Tools_SaveAsura::IsEditingMission() && !MissionExport_IsExportingEnvironmentOnly())
			//xBaseEnvName = Tools_SaveAsura::GetMissionFilenameWithoutExtension();
		//else
			//xBaseEnvName = m_xLevelNameBackup;

		GetExportedLevelFilename( xBaseEnvName, xTmpFilename );

		if(m_bFakeLevelExport)
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::ExportLevel LoadingFilesInList");
            UpdateStatus( EXPORT_STAGE_LOADING_FILES, "Loading files...", -1 );
			LoadFilesInList();
		}
		else
		{
			ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::ExportLevel ExportingFilesInList");
            UpdateStatus( EXPORT_STAGE_START_EXPORT, "Beginning export...", -1 );
			ExportFilesInList( xTmpFilename.GetString(),true);
		}		

	// (MP - 02.04.2003) I don't think we want to do this, or at least not in
	// such an arbitrary manner. Hierarchy textures need to be available with the
	// hierarchy data since we throw out all skins, animation and associated
	// textures at runtime between cutscenes / in-game.
	//	//copy some of the lists into the environment lists (to avoid adding stuff again in cutscene files)
	//	m_pxEnvExportedTexturesList = m_pxExportedTexturesList;
	//	m_pxExportedTexturesList = 0;
	//	m_xEnvAsuraFileList = m_xAsuraFileList;

		//clear up the export lists
		EndFileExport();
	}

	if(m_bFakeLevelExport)
	{
		ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Level_Export::ExportLevel CreatingPhysicsZones");
		UpdateStatus( EXPORT_STAGE_CREATING_PHYSICS_ZONES, "Creating Physics Zones...", -1 );
		Asura_Physics_Environment::Tools_Create();
		UpdateStatus( EXPORT_STAGE_END, NULL, -1 );
	}


	if (bChangedPrelightingData)
	{
		// Invalidate the data again, so it'll get recalculated after export.
		for (int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
		{
			Asura_Mesh *pxMesh = Asura_Module_System::GetMeshForModule(i);
			pxMesh->PrelightingDataValid = false;
		}
	}
	
	if(!m_bFakeLevelExport)
	{

		//reset subdivision settings
		Tools_Render_Options::ApplySubdivisionMode();
	}

	EndLevelExport();
	CloseExportLog();
	EndExport(false);
}

void Tools_Level_Export::GetExportedLevelFilename(const Asura_String& xEnvironmentFilename,
												  Asura_String& xOutputFilename)
{
	//get the level name without path/extension
	Asura_String xName;
	xEnvironmentFilename.SplitFilename(0,&xName,0);

	const char* szPlatformName = GetPlatformName(s_eExportType);
	const char* szPlatformExt = GetPlatformExtension();
	if (szPlatformName && szPlatformExt)
	{
		char acFilename[MAX_PATH];

		Asura_PrintfToBfr(acFilename, "..\\%s_Release\\envs\\%s%s", MAX_PATH, true, szPlatformName, xName.GetString(), szPlatformExt);
		xOutputFilename = acFilename;
	}
	else
	{
		xOutputFilename = xName;
		xOutputFilename += ".unknown_platform";
	}
}

bool Tools_Level_Export::IsExportedLevelFileWriteable(bool bShowMessage)
{

	Asura_String xExportName;

	Asura_String xBaseEnvName;
	if(Tools_SaveAsura::IsEditingMission())
		xBaseEnvName = Tools_SaveAsura::GetMissionFilenameWithoutExtension();
	else
		xBaseEnvName = Tools_SaveAsura::GetMainFilename();

	GetExportedLevelFilename(xBaseEnvName,xExportName);
		 	
	return IsWriteableIncludingLanguageFiles(xExportName.GetString(),bShowMessage);
}

void Tools_Level_Export::UpdateExportedLevel()
{
	StartExport();
	StartLevelExport();
	StartFileExport();

	Tools_SaveAsura::SaveAll();	

	//list the files for updating
	AddFileToList( m_xAsuraFileList, Tools_SaveAsura::GetGlobalIlluminationFilename(), false );
    AddFileToList( m_xAsuraFileList, Tools_SaveAsura::GetLightingFilename(), false );
    AddFileToList( m_xAsuraFileList, Tools_SaveAsura::GetSoundsFilename(), false );
    AddFileToList( m_xAsuraFileList, Tools_SaveAsura::GetNavigationFilename(), false );
	AddFileToList( m_xAsuraFileList, Tools_SaveAsura::GetModuleLinkingFilename(), false );
    AddFileToList( m_xAsuraFileList, Tools_SaveAsura::GetSkyboxFilename(), false );
    AddFileToList( m_xAsuraFileList, Tools_SaveAsura::GetCutscenesFilename(), false );
#ifdef ASURA_USE_BILLBOARD_PLANTS
	AddFileToList( m_xAsuraFileList, Tools_SaveAsura::GetBillboardPlantsFilename(), false );
#endif //ASURA_USE_BILLBOARD_PLANTS
	AddFileToList( m_xAsuraFileList, Tools_SaveAsura::GetEntityFilename(), false );


	Asura_String xTmpFilename;
	Asura_String xBaseEnvName;
	if(Tools_SaveAsura::IsEditingMission() && !MissionExport_IsExportingEnvironmentOnly())
		xBaseEnvName = Tools_SaveAsura::GetMissionFilenameWithoutExtension();
	else
		xBaseEnvName = m_xLevelNameBackup;

	GetExportedLevelFilename( xBaseEnvName, xTmpFilename );

	UpdateFilesInList(xTmpFilename.GetString());

	EndFileExport();
	EndLevelExport();
	EndExport(false);

}

bool Tools_Level_Export::PreLevelExportChecks()
{
	if(Tools_System::WarnAboutModulesWithTooManyTriangles())
	{
		AddMessage(MT_ERROR,"Some modules have too many triangles for export to continue.");
		return false;
	}

	//Abort if there are any transparent environmental polygons that are flagged with 8 bit alpha
	int i8BitTrans = Tools_Texturing_Find::Find8BitTransparent(true);
	if(i8BitTrans)
	{
		#if 0
		if(Tools_BatchProcessing::IsBatch())
		{
			AddMessage(MT_ERROR,"Found %d environmental polygons with both 8 bit alpha and transparency",i8BitTrans);
		}
		else
		{
			ShowMessageOverlay("Error.\nFound %d environmental polygons with both 8 bit alpha and transparency flags.\nUse 'Find 8 Bit Transparent' to find them.",i8BitTrans);
		}
		return false;
		#else
		AddMessage(MT_ERROR,"Found %d environmental polygons with both 8 bit alpha and transparency",i8BitTrans);
		#endif
	}


	//warn if using quick/invalid ELT
	bool bELTValid = Tools_EnvironmentalLightTransport::DoesEnvironmentMatch();
	bool bQuickELT = Tools_EnvironmentalLightTransport::IsQuickData();
	if(!Tools_PhotonMapping_System::AreThereAnyParallelLights())
	{
		//The ELT doesn't matter if there are no parallel lights. 
		//So pretend it is all good.
		//RAR 22/05/2007
		bELTValid = true;
		bQuickELT = false;
	}
	bool bGIValid = Asura_GI_Solution::AreSamplesValid();
	Tools_PhotonMap_Validity ePhotonValid = Tools_PhotonMapping_System::CheckPhotonMapsAreValid();
	if(!bGIValid || !bELTValid || bQuickELT || ePhotonValid != TOOLS_PHOTONMAP_VALID)
	{
		char szMessage[1000] = "Warning.\nLighting isn't fully valid :\n";
		char* pszPos = szMessage;
		pszPos += strlen(pszPos);
		
		if(!bELTValid)
		{
			strcat(pszPos,"ELT is invalid\n");
			pszPos += strlen(pszPos);
			AddMessage(MT_WARNING,"Warning : ELT is invalid");
		}
		else if(bQuickELT)
		{
			strcat(pszPos,"Has quick ELT\n");
			pszPos += strlen(pszPos);
			AddMessage(MT_WARNING,"Warning : Has Quick ELT");
		}
		if(ePhotonValid == TOOLS_PHOTONMAP_INVALID)
		{
			strcat(pszPos,"Photon maps are invalid\n");
			pszPos += strlen(pszPos);
			AddMessage(MT_WARNING,"Warning : Photon maps are invalid");
		}
		else if(ePhotonValid == TOOLS_PHOTONMAP_REDUCEDACCURACY)
		{
			strcat(pszPos,"Photon maps are at reduced accuracy\n");
			pszPos += strlen(pszPos);
			AddMessage(MT_WARNING,"Warning : Photon maps are calculated at reduced accuracy");
		}
		if(!bGIValid)
		{
			strcat(pszPos,"GI samples are invalid\n");
			pszPos += strlen(pszPos);
			AddMessage(MT_WARNING,"Warning : GI samples are invalid");
		}

		strcat(pszPos,"Continue with level export?");

		#if 0
		if(!Tools_BatchProcessing::IsBatch())
		{
			if(g_bAskAboutInvalidLighting)
			{
				YesNoAll_Overlay xOverlay(szMessage);
				g_bAcceptInvalidLighting = xOverlay.DoModal();
				if(xOverlay.AllSelected())
				{
					g_bAskAboutInvalidLighting = false;
				}
			}
			
			if(!g_bAcceptInvalidLighting)
			{
				return false;
			}
		}
		#endif
	}
	return true;

}

void Tools_Level_Export::StartLevelExport( void )
{
    // Re-initialise entities before an export.
	Asura_Entity_System::InitialiseServerEntitiesFromBlueprints();

    //Need to create all the tools objects before saving , to ensure that any entities
	//flagged as merge with environment are up to date. RAR 06/10/2004
    Tools_Entity_Object::CreateAllObjects();

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
	// (MH - 21.01.2008) decals objects must exist to allow the code which ensures
	// they are up-to-date to be run.
	Tools_StaticDecal_System::CreateToolsObjects();
#endif //ASURA_USE_STATIC_DECAL_SYSTEM

#ifdef ASURA_USE_CARPET
	// (AAM - 22.01.2008) Ensure we have carpet data available to export. This is normally created
	// lazily in the render loop, but a batch export happens before that
	if( !Asura_Direct3D_Carpet_Game_Render::HasDataSet() )
	{
		const Asura_LinkList< Asura_Carpet_Triangle >& xCarpetTriangleList =
			Asura_Carpet_MeshManager::GetTriangleList();

		if( xCarpetTriangleList.Front() )
		{
			Asura_Direct3D_Carpet_Game_Render::MakeDataSet( xCarpetTriangleList );
		}
	}
#endif

#ifdef ASURA_USE_AXON
	Tools_EntityNode_CoverpointNode::CheckCoverpointsForValidity();
#endif

	if(!m_bFakeLevelExport)
	{
		if(s_bFilterEntityExport)
		{
			//ask for entities to include
			Tools_EntityFilterOverlay xOverlay("Export Which Entity Types?",&s_xEntityClassesToIgnore);
			xOverlay.ShowOverlay();
		}
		
		
		EXPORT_TYPE eExportTypeBackup = s_eExportType;
		//temporary change to export type , since for this stage we don't want to
		//have GC endian swapping getting in the way
		s_eExportType = Export_PC;

		m_xLevelNameBackup = Tools_SaveAsura::GetMainFilename();
		if(Tools_SaveAsura::This->IsModified())
		{
			//attempt to save files under thier normal filenames first , since we
			//are going to lose the state of the modified flags.
			//disable the export flag while doing this
			bool bWasExport = s_bIsExport;//(always true at the moment)
			s_bIsExport = false;
			Tools_SaveAsura::SaveModified();
			s_bIsExport = bWasExport;
		}
		s_eExportType = eExportTypeBackup;
        

		//generate a temporary filename to use during level export , to avoid
		//problems from having read-only files
		Asura_String xName;
		m_xLevelNameBackup.SplitFilename(0,&xName,0);

		char szNewFilename[MAX_PATH];
		Asura_PrintfToBfr(szNewFilename,"%s\\%s.asr",MAX_PATH,true,Tools_Level_Export::GetTempDirectory(),xName.GetString());
		Tools_FileSystem::EnsureDirectoryExists(szNewFilename);

		Tools_SaveAsura::SetFilename(szNewFilename);
		Tools_SaveAsura::SetRealFilename( m_xLevelNameBackup );

		//Change to the light set being exported
		s_uInitialLightSets = Tools_LightSets::GetCurrentLightSets();
		Tools_LightSets::SetCurrentLightSets(s_uCurrentLightSetsToExport);

		//Make sure the correct platform's meshes are being used
		Tools_PlatformMesh_System::PushPlatform(s_eExportType,true);
		Tools_PlatformMesh_System::ReloadFilesIfNeeded();
	}

	//This needs to be set sometime after the files are saved under the unmodified filenames , to
	//avoid having light scales applied to them. RAR 09/02/2004
    s_bIsLevelExport = true;
}

void Tools_Level_Export::EndLevelExport( void )
{
	bool bWasExportingAllModules = IsExportingAllModules();

    s_bIsLevelExport = false;
	
	if(!m_bFakeLevelExport)
	{
		if(!bWasExportingAllModules)
		{
			//If some modules weren't exported , then the physics environment will be incomplete , and should
			//be invalidated again.
			//RAR 12/12/2007
			Asura_Physics_Environment::Invalidate();
			Asura_AI_Navigation_System::DeleteSubsetForExport();
		}

		//restore the filename
		Tools_SaveAsura::SetFilename(m_xLevelNameBackup.GetString());

		//restore light set
		Tools_LightSets::SetCurrentLightSets(s_uInitialLightSets);

		//restore previous platform meshes
		Tools_PlatformMesh_System::PopPlatform(true);
		Tools_PlatformMesh_System::ReloadFilesIfNeeded();
	}

    Tools_Object_System::DeleteAllModeSpecific();

	// clear tree of missing materials
	g_xMissingMaterialsEncountered.Reset();
}

void Tools_Level_Export::BatchExport_ExportLevel()
{
	Asura_Assert(Tools_BatchProcessing::IsBatch(),"");
	Asura_Assert(This, "");

	Tools_IniFile* pxBatchIniFile = Tools_BatchProcessing::GetIniFile();

	//initialise array of platforms to export
	bool abPlatforms[ExportType_NumPlatforms];
	for(int i=0;i<ExportType_NumPlatforms;i++)
	{
		abPlatforms[i] = false;
	}

	//and languages
	DisableAllLanguagesForExport();
	bool bAnyLanguageSpecified = false;

	//export types to do
	bool bExportLevel = true;
	bool bExportCharacters = false;
	bool bExportCommon = false;
	bool bExportMenus = false;

	s_uLightSetsToExport = 0;

	// JAT 14/09/07 support for batch export of global sounds
	// Not putting this as a new platform because it is very specific to the export global sounds option
	bool bExportGlobalSounds = false;
	bool bExportGlobalSoundsForTools = false;

	//read the settings in the default section , then in the section for this export
	for(int iLoop = 0;iLoop<2;iLoop++)
	{
		if(iLoop == 0)
		{
			if(!pxBatchIniFile->GetIniTextSection("Default"))
			{
				continue;
			}
		}
		else
		{
			if(!pxBatchIniFile->GetIniTextSectionByIndex(Tools_BatchProcessing::GetSectionIndex()))
			{
				continue;
			}
		}
		This->BatchExport_ProcessIdentifiers(pxBatchIniFile);
		
		pxBatchIniFile->GetBoolInCurrentSection("Export_Level",bExportLevel);
		pxBatchIniFile->GetBoolInCurrentSection("Export_Characters",bExportCharacters);
		pxBatchIniFile->GetBoolInCurrentSection("Export_Common",bExportCommon);
		pxBatchIniFile->GetBoolInCurrentSection("Export_Menus",bExportMenus);
		pxBatchIniFile->GetBoolInCurrentSection("ExportMission_Environment", s_bMission_ShouldExportEnvironment);
		pxBatchIniFile->GetBoolInCurrentSection("ExportMission_Mission", s_bMission_ShouldExportMission);

		pxBatchIniFile->GetBoolInCurrentSection("PC",abPlatforms[Export_PC]);
		pxBatchIniFile->GetBoolInCurrentSection("PS2",abPlatforms[Export_PS2]);
		pxBatchIniFile->GetBoolInCurrentSection("PSP",abPlatforms[Export_PSP]);
		pxBatchIniFile->GetBoolInCurrentSection("PS3",abPlatforms[Export_PS3]);
		pxBatchIniFile->GetBoolInCurrentSection("Xbox360",abPlatforms[Export_Xbox360]);
		pxBatchIniFile->GetBoolInCurrentSection("Wii",abPlatforms[Export_Wii]);
		pxBatchIniFile->GetBoolInCurrentSection("Export_Global_Sounds",bExportGlobalSounds);
		pxBatchIniFile->GetBoolInCurrentSection("Tools", bExportGlobalSoundsForTools );

		pxBatchIniFile->GetBoolInCurrentSection("Subdivide",s_bSubdivideMeshes);
		pxBatchIniFile->GetFloatInCurrentSection("Flat_Subdivide_Size",s_fFlatSubdivideSize);
		pxBatchIniFile->GetFloatInCurrentSection("Curved_Subdivide_Size",s_fCurveSubdivideSize);
		pxBatchIniFile->GetBoolInCurrentSection("Prelight",s_bRecalculatePrelighting);
		pxBatchIniFile->GetBoolInCurrentSection("Swizzle_Tims",s_bSwizzleTims);
		pxBatchIniFile->GetBoolInCurrentSection("Dont_Include_Cutscene_Files",s_bDontExportCutscenes);
		pxBatchIniFile->GetBoolInCurrentSection("Multiplayer",s_bPS2MultiplayerExport);
        pxBatchIniFile->GetBoolInCurrentSection( "IncludeDualTextureData", s_bIncludeDualTexturingData );
		pxBatchIniFile->GetBoolInCurrentSection("Override_Animation_Frame_Removal",s_bOverrideFrameRemovalSettings);
		pxBatchIniFile->GetFloatInCurrentSection("Position_Tolerance",s_fAnimPositionTolerance);
		pxBatchIniFile->GetFloatInCurrentSection("Angle_Tolerance",s_fAnimAngleTolerance);
		pxBatchIniFile->GetFloatInCurrentSection("Gamecube_Min_TexRes",s_fMinGCTexRes);
		pxBatchIniFile->GetFloatInCurrentSection("Gamecube_Max_MipDrops",s_fMaxMipDrops);
		pxBatchIniFile->GetBoolInCurrentSection("Full_Bright",s_bFullBrightPrelighting);
		pxBatchIniFile->GetUIntInCurrentSection("Maximum_Texture_Size", s_uMaximumTextureSize);
		pxBatchIniFile->GetBoolInCurrentSection("Use_Maximum_Texture_Size", s_bUseMaximumTextureSize);
		pxBatchIniFile->GetBoolInCurrentSection("Compression_Disabled", s_bCompressionDisabled);

		Asura_String xLightSets;
		if(pxBatchIniFile->GetStringInCurrentSection("Light_Sets",xLightSets))
		{
			s_uLightSetsToExport = 0;
			//read in light sets to export from comma separated list
			const char* pcPos = xLightSets.GetString();
			while(pcPos)
			{
				u_int uSet;
				if(sscanf(pcPos,"%d",&uSet)==1)
				{
					s_uLightSetsToExport |= (1<<uSet);
					}
				//skip to next
				pcPos = strchr(pcPos,',');
				if(pcPos) pcPos++;
			}
		}
		
		for(int i=0; i<ASURA_LT_L_NUM_LANGUAGES; i++)
		{
			const char* szLangName = Asura_LocalisedText::GetNameForLang((Asura_LocalisedText_Language) i);
			bAnyLanguageSpecified |= pxBatchIniFile->GetBoolInCurrentSection(szLangName,s_abExportLanguage[i]);
		}
		
	}

	if (!s_bOverrideFrameRemovalSettings)
	{
		s_fAnimPositionTolerance = TOOLS_DEFAULT_ANIM_POSITION_TOLERANCE;
		s_fAnimAngleTolerance = TOOLS_DEFAULT_ANIM_ANGLE_TOLERANCE;
	}


	//If there were no language entries , enable all of them
	//RAR 31/05/2006
	if(!bAnyLanguageSpecified)
	{
		EnableAllLanguagesForExport();
	}
	//If no light sets were specified , then default to exporting just the first
	if(!s_uLightSetsToExport)
	{
		s_uLightSetsToExport = 1;
	}

	if (!This->BatchExport_ProjectExport())
	{
		//Common file export.
		//Putting this first , so that if a level export is done at the same time , it can refer to
		//an up-to-date common file. (For ExcludeCommonFiles in streaming builds).
		//RAR 12/12/2005
		if(bExportCommon)
		{
			for(int i=0;i<ExportType_NumPlatforms && !Tools_BatchProcessing::IsAborted();i++)
			{
				if(abPlatforms[i])
				{
					ExportCommonFile(i);
				}
			}
		}

		//Level exports
		if(bExportLevel)
		{
				for(int i=0;i<ExportType_NumPlatforms && !Tools_BatchProcessing::IsAborted();i++)
				{
					if(abPlatforms[i])
					{
						ExportLevel(i);
					}
				}
			}

		//Character exxports
		if(bExportCharacters)
		{
			s_bExportAllCharacters = true;//No option to export specific directories at the moment.
			for(int i=0;i<ExportType_NumPlatforms && !Tools_BatchProcessing::IsAborted();i++)
			{
				if(abPlatforms[i])
				{
					ExportCharacters(i);
				}
			}
		}

		//Sound event exports
		//Undecided if this should be in the 'not project export' part or not. We'll see who screams...
		//RAR 29/08/2006
		if(bExportGlobalSounds && !Tools_System::This->CanExportGlobalSoundsInCommon())
		{
			for(int i=0;i<ExportType_NumPlatforms && !Tools_BatchProcessing::IsAborted();i++)
			{
				if(abPlatforms[i])
				{
					ExportGlobalSoundEvents(i);
				}
			}
		}
		// JAT 14/09/07 support for batch export of global sounds
		if ( bExportGlobalSoundsForTools == true )
		{
			ExportGlobalSoundEventsForTools();
		}

		//GUIMenu export
		#ifdef ASURA_USE_GUI_MENU_SYSTEM
		if(bExportMenus)
		{
			for(int i=0;i<ExportType_NumPlatforms && !Tools_BatchProcessing::IsAborted();i++)
			{
				if(abPlatforms[i])
				{
					ExportGUIMenus(i,NULL);
				}
			}
		}
		#endif
	}
}

/// End of file. ///