/**
*
* \file Tools_Level_Export.cpp
*
* Created by Kevin Lea - 29/11/2002 11:23:31
* Splitted into separate file 12/11/2008
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"

// Asura
#include "Animation/Asura_AnimTable_System.h"
#include "Animation/Asura_Hierarchy.h"
#include "ChunkSystem/Asura_Chunk_AnimationTable.h"
#include "ChunkSystem/Asura_Chunk_Hierarchy.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Mesh/Asura_Mesh_System.h"

// Tools
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Load&Save/Tools_ResourceExportLog.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "PC/Tools_PC_Export.h"
#include "PSP/Tools_PSP_Export.h"
#include "PS2/PS2Strip/PS2Strip_Exporter.h"
#include "PS2/PS2Strip/PS2Strip_ToolsInterface.h"
#include "PS3/Tools_PS3_Export.h"
#include "Shapes/Tools_Shapes.h"
#include "Wii/Tools_Wii_Export.h"
#include "Xbox360/Tools_Xbox360_Export.h"

#ifdef ASURA_USE_AXON
#include "ChunkSystem/Axon_Chunk_AnimTemplates.h"
#include "Entity/Locations/OrientedNode/CoverpointNode/Tools_EntityNode_CoverpointNode.h"
#endif


void Tools_Level_Export::AddAnimDirToExportList(List<Asura_String>& xNameList,const char* pszDirectory, bool bWarnOnEmpty /* = true */, const bool bAllFilesAreAnims /*=false*/)
{
	This->AddAnimDirToExportListMain( xNameList, pszDirectory, bWarnOnEmpty, bAllFilesAreAnims );
}

void Tools_Level_Export::AddAnimDirToExportListMain(List<Asura_String>& xNameList,const char* pszDirectory, bool bWarnOnEmpty /* = true */, const bool bAllFilesAreAnims /*=false*/)
{
//	ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Level_Export::AddAnimDirToExportListMain");
	
	//make sure this directory hasn't already been done
	int iJunkIndex;
	if(m_pxAnimDirList->FindNodeIndexByName(pszDirectory,true,iJunkIndex))
	{
		//was already in list
		return;
	}

	List<Asura_String> xFileList;

    char szDirToUse[MAX_PATH + 1];
    Asura_FileAccess::PlatformGetNetworkDir( pszDirectory, szDirToUse, MAX_PATH + 1 );

	//read files from export list for directory if there is one
	bool bGotExportList = ProcessExportList( szDirToUse, &xFileList, &xFileList );

	if( !bGotExportList )
	{
	    //otherwise just get all asura files in this directory and below
        char szSearch[MAX_PATH + 1];
        Asura_PrintfToBfr( szSearch, "%s\\*.asr", MAX_PATH + 1, true, szDirToUse );      
		Tools_FileSystem::FindFilesLocalAndNetwork( szSearch, xFileList, TOOLS_FILEFIND_RECURSIVE|TOOLS_FILEFIND_IGNOREBACKUPDIR );
    }

	if(!xFileList.size() && bWarnOnEmpty)
	{
		AddMessage(MT_WARNING, "Tried to add anim dir \"%s\" to the export list, but it's empty.", pszDirectory);
	}
    
	#ifdef ASURA_SUPPORT_PLATFORM_HASHED_FILENAMES
	//strip out files belonging to other platforms. RAR 25/04/2007
	RemoveFilesForOtherPlatforms(xFileList);
	#endif

	//add all these to the export list
	Tools_AsuraFileContents xFileContents;
	for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next())
	{
//		ASURA_CPU_PROFILE_SCOPED_TIMER( "Tools_Level_Export::AddAnimDirToExportListMain File");
		bool bIsSkinFile = false;
		if ( !bAllFilesAreAnims )
		{
		xFileContents.LoadAsuraFile(xFileLIF().GetString());	
		bIsSkinFile = xFileContents.GetSkinList()->size()!=0;
		}

		// JAT 11/07/07 - assert & ignore trying to load an environment file from within an animation directory (stops it crashing later)
		// Yes it should never happen but we have to be robust and cope with bad data without crashing
		if ( xFileContents.IsEnvironment() == true )
		{
			Asura_Assertf( false, ( "Can't load an environment file '%s' from within an object/animation folder", xFileLIF().GetString() ) );
			continue;
		}

		const char* pszRelativeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( xFileLIF().GetString() );
		AddFileToList(xNameList,pszRelativeName,bIsSkinFile);
	}
}

void Tools_Level_Export::AddSkinAndAnimsToExportList(const char* szName)
{
	AddAnimTableToExportList(szName);
	AddSkinToExportList(szName);
	}

void Tools_Level_Export::AddSkinAndAnimsToExportList(Asura_Hash_ID uSkinAndTableHash)
{
	AddAnimTableToExportList(uSkinAndTableHash);
	AddSkinToExportList(uSkinAndTableHash);
}

void Tools_Level_Export::AddSkinToExportList(const char* szName)
{
	This->AddSkinToExportListMain( 0, szName);
}

void Tools_Level_Export::AddSkinToExportList(Asura_Hash_ID uSkinHash)
{
	This->AddSkinToExportListMain( uSkinHash , 0);
}

void Tools_Level_Export::AddSkinToExportListMain(Asura_Hash_ID uSkinHash,const char* szName)
{
	if(szName) uSkinHash = Asura_GetHashID(szName);
	if(!uSkinHash) return;

	//already seen this anim?
	int iJunkIndex;
	if(m_pxSkinHashTree->FindNodeIndexByHash(uSkinHash,true,iJunkIndex))
	{
		return;
	}

	//get the animation filename
	const char* szFilename = Asura_HashedFilename_System::GetFullFilename(uSkinHash);
	if(!szFilename)
	{
		if(szName)
			AddMessage(MT_ERROR,"Failed to get filename for skin '%s'",szName);
		else	
			AddMessage(MT_ERROR,"Failed to get filename for skin 0x%x",uSkinHash);

		return;
	}

    AddFileToList(m_xAsuraFileList, szFilename, true);
}

void Tools_Level_Export::AddAnimToExportList(const char* szName)
{
	This->AddAnimToExportListMain( 0, szName);
}

void Tools_Level_Export::AddAnimToExportList(Asura_Hash_ID uAnimHash)
{
	This->AddAnimToExportListMain( uAnimHash , 0);
}

void Tools_Level_Export::AddAnimToExportListMain(Asura_Hash_ID uAnimHash,const char* szName)
{
	if(szName) uAnimHash = Asura_GetHashID(szName);
	if(!uAnimHash) return;

	//already seen this anim?
	int iJunkIndex;
	if(m_pxAnimHashTree->FindNodeIndexByHash(uAnimHash,true,iJunkIndex))
	{
		return;
	}

	//get the animation filename
	const char* szFilename = Asura_HashedFilename_System::GetFullFilename(uAnimHash);
	if(!szFilename)
	{
		if(szName)
			AddMessage(MT_ERROR,"Failed to get filename for anim '%s'",szName);
		else	
			AddMessage(MT_ERROR,"Failed to get filename for anim 0x%x",uAnimHash);

		return;
	}

    AddFileToList(m_xAsuraFileList, szFilename, false);
}


void Tools_Level_Export::AddAnimTableToExportList(const char* szName)
{
	This->AddAnimTableToExportListMain( 0, szName);
}

void Tools_Level_Export::AddAnimTableToExportList(Asura_Hash_ID uTableHash)
{
	This->AddAnimTableToExportListMain( uTableHash , 0);
}

void Tools_Level_Export::AddAnimTableToIgnore( Asura_Hash_ID uTableHash )
{
	This->AddAnimTableToIgnoreMain( uTableHash );
}

void Tools_Level_Export::AddAnimTableToIgnoreMain( Asura_Hash_ID uTableHash )
{
#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
	// add to the list of 'already dealt with' anim tables so the exporter will just ignore this table
	int iJunkIndex = 0;
	m_pxAnimHashTree->FindNodeIndexByHash( uTableHash, true, iJunkIndex );

	// if already present, delete the table from the list of animtables to export
	Asura_AnimTable* pxAnimTable = Asura_AnimTable_System::GetAnimTable( uTableHash );
	if( pxAnimTable )
	{
		m_xAnimTableList.delete_entry( pxAnimTable );
	}
#endif
}

void Tools_Level_Export::AddAnimationTables(HANDLE pSaveHandle)
{
#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
	if(!m_xAnimTableList.size()) return;

	//build a saveblock containing all the chunks
	Asura_SaveBlock xSaveBlock;

	for(LIF<const Asura_AnimTable*> xAnimTableLIF(&m_xAnimTableList);!xAnimTableLIF.done();xAnimTableLIF.next())
	{
		Asura_Chunk_ExportedAnimationTable::Create(xAnimTableLIF(),&xSaveBlock);
	}

	//add the contents to the file
	unsigned long uBytesWritten;
	u_int uBytesToWrite = xSaveBlock.GetSize()-ASURA_FILE_ID_NUM_CHARS;
	WriteFile(pSaveHandle,xSaveBlock.GetStartPtr()+ASURA_FILE_ID_NUM_CHARS,uBytesToWrite,&uBytesWritten,0);

	if(uBytesToWrite != uBytesWritten)
	{
		AddMessage(MT_ERROR,"Error writing to file.");
		SetAbortFlag();
	}
#endif // ndef ASURA_DISABLE_ANIM_TABLE_CODE
}

void Tools_Level_Export::AddAnimTableToExportListMain(Asura_Hash_ID uTableHash,const char* szName)
{
#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
	if(szName) uTableHash = Asura_GetHashID(szName);
	if(!uTableHash) return;

	//already seen this table?
	int iJunkIndex;
	if(m_pxAnimTableHashTree->FindNodeIndexByHash(uTableHash,true,iJunkIndex))
	{
		return;
	}

	//Get hold of the table.
	const Asura_AnimTable* pxTable = Tools_Animation_Files::GetAnimTable(uTableHash,szName);
	if(!pxTable)
	{
		if(szName)
			AddMessage(MT_ERROR,"Failed to find animation table '%s'",szName);
		else	
			AddMessage(MT_ERROR,"Failed to find animation table 0x%x",uTableHash);

		return;
	}

	//add to list for adding anim table chunks later
	m_xAnimTableList.add_entry(pxTable);

	TOOLS_RESOURCEEXPORTLOG("Anim Table",pxTable->GetName(),true);


	//add all animations contained within
	u_int uNumWeapons = pxTable->GetNoOfWeaponStates();
	u_int uNumPrimary = pxTable->GetNoOfPrimaryStates();
	u_int uNumSecondary = pxTable->GetNoOfSecondaryStates();

	Asura_Hash_ID uNA = 0x00000DB3;//Asura_GetHashID("NA");
	Asura_Hash_ID uZero = 0x00000030;//Asura_GetHashID("0");

	for(u_int i=0;i<uNumWeapons;i++)
	{
		for(u_int j=0;j<uNumPrimary;j++)
		{
			for(u_int k=0;k<uNumSecondary;k++)
			{
				Asura_Hash_ID uAnimHash = pxTable->GetAnimID(i,j,k);
				if(uAnimHash == 0 || uAnimHash == uNA || uAnimHash == uZero) continue;

				const char* szFilename = Asura_HashedFilename_System::GetFullFilename(uAnimHash);
				Asura_Assertf( szFilename, ("Failed to find anim file for animtable %s '0x%x' (page %d, row(index) %d, row(XL) %d, column %d).", pxTable->GetName(), pxTable->GetNameHashID(), i+1, j+1, j+4, k+1 ));

				AddAnimToExportList(uAnimHash);
			}
		}
	}
#endif // #ifndef ASURA_DISABLE_ANIM_TABLE_CODE
}

void Tools_Level_Export::AddAnimDirToIgnore(const char* pszDirectory)
{
	if( (pszDirectory == NULL) || (pszDirectory[0] == '\0') )
	{
		Asura_Assert( false, "Invalid parameter passed to Tools_Level_Export::AddAnimDirToIgnore()." );
		return;
	}

	int iJunkIndex;
	if(!m_pxAnimDirList->FindNodeIndexByName(pszDirectory,true,iJunkIndex))
	{
		//make sure the directory has a final backslash on it , to help with doing the subdirectory tests
		char szDir[MAX_PATH];
		strcpy(szDir,pszDirectory);
		if(szDir[strlen(szDir)-1] != '\\')
		{
			strcat(szDir,"\\");
		}
		
		
		m_xDirsToIgnore.add_entry(szDir);

		//remove any asura files that have already been added from this directory
		for(LIF<Asura_String> xFileLIF(&m_xAsuraFileList);!xFileLIF.done();)
		{
			const char* pszRelativeName = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( xFileLIF().GetString());
			if(!strnicmp(szDir,pszRelativeName,strlen(szDir)))
			{
				//don't want this file... delete from skin tree  , and asura file list
				int iJunkIndex;
				m_pxSkinFileList->DeleteNodeByName(xFileLIF(),iJunkIndex);

				m_pxAnimFileList->DeleteNodeByName(xFileLIF(),iJunkIndex);
		
				//delete from file list
				xFileLIF.delete_current();
		 
			}
			else
			{
				xFileLIF.next();
			}
			
		}
	}
}

void Tools_Level_Export::AddFileToSkinList(const Asura_String &xName)
{
	int iJunkIndex;
	m_pxSkinFileList->FindNodeIndexByName(xName, true, iJunkIndex);
}

void Tools_Level_Export::AddFileToAnimList(const Asura_String &xName)
{
	int iJunkIndex;
	m_pxAnimFileList->FindNodeIndexByName(xName, true, iJunkIndex);
}

bool Tools_Level_Export::DirectoryIsAnimOnly(const char* directory)
{
	// see if this is in the Anims_Body_Types or Anims_Specialised directores
	char buffer[MAX_PATH];
	strcpy(buffer,directory);
	strlwr(buffer);

	if(strstr(buffer,"anims_body_types")) return true;
	if(strstr(buffer,"anims_specialised")) return true;

	return false;
}

bool Tools_Level_Export::IsSkinFile(const Asura_String& xFilename)
{
	int iJunkIndex;
	return m_pxSkinFileList->FindNodeIndexByName(xFilename,false,iJunkIndex);
}

bool Tools_Level_Export::AddStripifiedSkin( HANDLE pSaveHandle, Asura_Hierarchy_Skin* pxSkin )
{
    bool bOK = true;


	switch(s_eExportType)
	{
		case Export_PS2:
		{
			PS2Strip::ExportSkinLODThresholdChunk( pSaveHandle, pxSkin );
			
			float fUScale,fVScale;
			GetUVScaleForPS2Skin(pxSkin,&fUScale,&fVScale);
		
			// Allow per-project control over this weld threshold
			PS2Strip_Exporter::SetWeldTolerance(GetPS2WeldToleranceForSkins());

            if( !PS2Strip::ExportSkin(pSaveHandle, pxSkin, fUScale, fVScale) )
			{
				bOK = false;
				break;
			}

			// Restore this to the default
			PS2Strip_Exporter::SetWeldTolerance(ASURA_PS2_DEFAULT_WELD_THRESHOLD);			

			break;
		}

		case Export_PC:
		{	
			
			Asura_Hierarchy_Skin* pxLODSkin = pxSkin;
			while(pxLODSkin)
			{
				#if defined ASURA_USE_NEW_SKIN_RENDERERS

					if( !Tools_PC_Export::ExportSkin( pSaveHandle, *pxLODSkin ) )
					{
						bOK = false;
						break;
					}

				#else

					if(pxLODSkin->IsSkinnedModel())
					{
						#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
							if( !Tools_PC_Export::ExportSmoothSkin( pSaveHandle, pxLODSkin ) )
							{
								bOK = false;
								break;
							}
						#endif

						#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
							if( !Tools_PC_Export::ExportCharacter( pSaveHandle, pxLODSkin ) )
							{
								bOK = false;
								break;
							}
						#endif				
					}
					else
					{
						if ( !Tools_PC_Export::ExportObjectHierarchy( pSaveHandle, pxLODSkin ) )
						{
							bOK = false;
							break;
						}
					}

				#endif // !defined ASURA_USE_NEW_SKIN_RENDERERS

				pxLODSkin = pxLODSkin->m_pxNextLowerSkin;
			}

			
			break;
		}

		case Export_Wii:
		{
			Asura_Hierarchy_Skin* pxLODSkin = pxSkin;
			while(pxLODSkin)
			{
				if(pxLODSkin->IsSkinnedModel())
				{
					if( !Tools_Wii_Export::ExportSmoothSkin( pSaveHandle, pxLODSkin ) )
					{
						bOK = false;
						break;
					}
				}
				else
				{
					if( !Tools_Wii_Export::ExportHierarchy( pSaveHandle, pxLODSkin ) )
					{
						bOK = false;
						break;
					}
				}
				pxLODSkin = pxLODSkin->m_pxNextLowerSkin;
			}
			break;
		}

		case Export_PSP:
		{
			//CA - Fix for exporting LOD for PSP
			Asura_Hierarchy_Skin* pxLODSkin = pxSkin;
			while(pxLODSkin)
			{
				if(pxLODSkin->IsSkinnedModel())
			{
					if( !Tools_PSP_Export::ExportCharacter( pSaveHandle, pxLODSkin ) )
				{
					bOK = false;
					break;
				}
			}
			else
			{
					if ( !Tools_PSP_Export::ExportObjectHierarchy( pSaveHandle, pxLODSkin ) )
				{
					bOK = false;
					break;
				}
			}
				pxLODSkin = pxLODSkin->m_pxNextLowerSkin;
			}
			break;
		}

		case Export_PS3:
		{
			//CA - Fix for exporting LOD for PS3
			Asura_Hierarchy_Skin* pxLODSkin = pxSkin;
			while(pxLODSkin)
			{
				if(pxLODSkin->IsSkinnedModel())
			{
					if( !Tools_PS3_Export::ExportCharacter( pSaveHandle, pxLODSkin ) )
				{
					bOK = false;
					break;
				}
			}
			else
			{
					if ( !Tools_PS3_Export::ExportObjectHierarchy( pSaveHandle, pxLODSkin ) )
				{
					bOK = false;
					break;
				}
			}
				pxLODSkin = pxLODSkin->m_pxNextLowerSkin;
			}
			break;
		}

		case Export_Xbox360:
		{
 			Asura_Hierarchy_Skin* pxLODSkin = pxSkin;
			while( pxLODSkin )
			{

				#if defined ASURA_USE_NEW_SKIN_RENDERERS

					if( !Tools_Xbox360_Export::ExportSkin( pSaveHandle, *pxLODSkin ) )
					{
						bOK = false;
						break;
					}

				#else

					if( pxLODSkin->IsSkinnedModel() )
					{
						if( !Tools_Xbox360_Export::ExportCharacter( pSaveHandle, pxLODSkin ) )
						{
							bOK = false;
							break;
						}
					}
					else
					{
						if( !Tools_Xbox360_Export::ExportObjectHierarchy( pSaveHandle, pxLODSkin ) )
						{
							bOK = false;
							break;
						}
					}

				#endif // !defined ASURA_USE_NEW_SKIN_RENDERERS

				pxLODSkin = pxLODSkin->m_pxNextLowerSkin;
			}

			break;
		}


		default:
		{
            Asura_Assert( false, "Invalid export type." );
            bOK = false;
			break;
		}
	}

	return bOK;

}

float Tools_Level_Export::GetPS2WeldToleranceForSkins()
{
	return ASURA_PS2_DEFAULT_WELD_THRESHOLD;
}

/////////////////////////////////////////////////////////////////////////////////////////
// FUNCTION		Tools_Level_Export::AddShapes
// PARAMETERS	Handle of the file being exported
// RETURNS		None
// DESCRIPTION	Exports shapes for the relevant target platform
// AUTHOR		Richard M. Smith (13 August 2002)
/////////////////////////////////////////////////////////////////////////////////////////
void Tools_Level_Export::AddShapes( HANDLE pSaveHandle )
{
	AddShapeTextures(pSaveHandle);

	UpdateStatus(EXPORT_STAGE_SHAPES, "Shapes", -1);
	
// Loop over the shapes in the Tools_Shape_Table and export each one for the target platform
	List<Asura_Hash_ID> xExportedShapes;
	for(int iCommon = 0;iCommon<2;iCommon++)
	{
		if((iCommon==0) && !m_bIncludeEntityShapes) continue;
		if((iCommon==1) && !m_bIncludeCommonShapes) continue;

		int iNumMeshes = iCommon ? Tools_ShapeTable::GetNumExportShapes() : Tools_ShapeTable::GetNumElements();

		for(int i=0;i<iNumMeshes;i++)
		{
			//skip shapes that shouldn't be exported
			if(!Tools_ShapeTable::ShouldExportShapeByIndex(i,iCommon ? true : false)) continue;

			//or shapes that have already been done
			Asura_Hash_ID uShapeHash = Tools_ShapeTable::GetHashIDByIndex(i,iCommon ? true : false);
			if(xExportedShapes.contains(uShapeHash)) continue;
			xExportedShapes.add_entry(uShapeHash);

			//export this shape then
			const char*	szShapeFilename = "<ERROR>";

			bool bSuccess = Tools_ShapeTable::ExportShape( s_eExportType, i, pSaveHandle, szShapeFilename,iCommon ? true : false );

			if( !bSuccess )
			{
				AddMessage( Tools_Level_Export::MT_ERROR, "Failed to export shape %s.", szShapeFilename );
                SetAbortFlag();
                return;
			}
		}
	}


	//Deal with other shape files (fragments etc)
	u_int uLp = 0;
	for(LIF<Asura_String> xFilenameLIF(&m_xShapeFilenames);!xFilenameLIF.done();xFilenameLIF.next(),uLp++)
	{
		s_iPercent = (int)( ( uLp * 100 ) / m_xShapeFilenames.size() );

		const char* pszFilename = xFilenameLIF().GetString();
		bool bSuccess = Tools_ShapeTable::ExportShapeFile(s_eExportType,pszFilename,pSaveHandle);

		if( !bSuccess )
		{
			AddMessage( Tools_Level_Export::MT_ERROR, "Failed to export fragment %s.", pszFilename );
            SetAbortFlag();
            return;
		}

	}

	UpdateStatus(EXPORT_STAGE_END, NULL, -1);

} // End Function 'Tools_Level_Export::AddShapes'

void Tools_Level_Export::AddShapeFileToExportList(const char* pszFilename)
{
	//make sure this file hasn't already been added
	const char* pszName = Tools_FileSystem::StripPath(pszFilename);
	int iJunkIndex;
	if(m_pxShapeFileStringTree->FindNodeIndexByName(pszName,true,iJunkIndex)) return;

	//get a list of mesh names in the file
	Tools_AsuraFileContents xFileContents;
	if(!xFileContents.LoadAsuraFile(pszFilename))
	{
		return;
	}

	m_xShapeFilenames.add_entry(pszFilename);

	bool bLoaded = false;
	const List<Asura_String>* pxMeshList = xFileContents.GetMeshList();

	//add all the meshes to the shape mesh list
	for(CLIF<Asura_String> xMeshLIF(pxMeshList);!xMeshLIF.done();xMeshLIF.next())
	{
		Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(xMeshLIF().GetString());
		if(!pxMesh)
		{
			//try loading the file
			if(!bLoaded)
			{
				LoadAsuraFile(pszFilename);
				bLoaded = true;
			
				pxMesh = Asura_Mesh_System::GetMesh(xMeshLIF().GetString());
			}
		}
			
		if(pxMesh)
		{
			m_xShapeMeshList.add_entry(pxMesh);
		}
	}
}

void Tools_Level_Export::LogAnimationExportStats()
{
	if(IsAbortFlagSet()) return;
	if(!m_iTotalAnimationFramesBefore || !m_iTotalAnimationSizeBefore) return;

	float fFramePercent = (m_iTotalAnimationFramesAfter/(float)m_iTotalAnimationFramesBefore)*100.0f;
	float fSizePercent = (m_iTotalAnimationSizeAfter/(float)m_iTotalAnimationSizeBefore)*100.0f;

	AddMessage(MT_LOGONLY,"------------------------------");
	AddMessage(MT_LOGONLY,"|  Animation Export Stats    |");
	AddMessage(MT_LOGONLY,"|                            |");
	AddMessage(MT_LOGONLY,"|           Frames     Size  |");
	AddMessage(MT_LOGONLY,"|Before:   %7d   %6dK |",m_iTotalAnimationFramesBefore,m_iTotalAnimationSizeBefore/1024);
	AddMessage(MT_LOGONLY,"|After:    %7d   %6dK |",m_iTotalAnimationFramesAfter,m_iTotalAnimationSizeAfter/1024);
	AddMessage(MT_LOGONLY,"|Percent:  %6.2f%%   %6.2f%% |",fFramePercent,fSizePercent);
	AddMessage(MT_LOGONLY,"------------------------------");
}

unsigned int Tools_Level_Export::ExportBigEndianAnimation(Asura_Chunk_Header* pxChunk, HANDLE hFile)
{
	DWORD uTotalBytesWritten = 0;

	switch(pxChunk->ID)
	{
#ifndef ASURA_DISABLE_ANIM_TABLE_CODE
		case ASURA_CHUNK_ANIMATIONTABLE:
			{
            DWORD uBytesWritten;
			const Asura_Chunk_AnimationTable* pxAnimTableChunk = static_cast<const Asura_Chunk_AnimationTable*>( pxChunk );

            u_char* pucOutputBfr = new u_char[pxAnimTableChunk->Size];
            Asura_Chunk_Memory_Stream xStream( pxAnimTableChunk->Size, pucOutputBfr );

            // Output a converted header.
            xStream.WriteChunk_Start( ASURA_CHUNK_ANIMATIONTABLE, pxAnimTableChunk->Version, pxAnimTableChunk->Flags );

            // Counts.
			xStream << pxAnimTableChunk->NumberOfPrimaryStates;
			xStream << pxAnimTableChunk->NumberOfSecondaryStates;

            const u_int uNumEntries = pxAnimTableChunk->NumberOfPrimaryStates * pxAnimTableChunk->NumberOfSecondaryStates;

            const Asura_Hash_ID* puHashIDs = reinterpret_cast<const Asura_Hash_ID*>( pxAnimTableChunk + 1 );

            // Copy hash IDs.
            {
                // Table name.
                xStream << *puHashIDs++;

                // Master table info.
                const Asura_Hash_ID uMasterOrSlaveHash = *puHashIDs++;
                xStream << uMasterOrSlaveHash;
                if( uMasterOrSlaveHash == uASURA_ANIMTABLE_HASHID_SLAVE )
                {
                    // Write master table name too, since it turns out to
                    // have one.
                    xStream << *puHashIDs++;
                }

                // Anim state name.
                xStream << *puHashIDs++;

                // Main table hash IDs.
                xStream << Asura_WriteToStream_Array_Wrapper<Asura_Hash_ID>( uNumEntries, puHashIDs );
                puHashIDs += uNumEntries;
            }

            const Asura_AnimTable_AnimFlags* pxAnimFlags = reinterpret_cast<const Asura_AnimTable_AnimFlags*>( puHashIDs );

            // Maybe copy flag data too.
            if( pxAnimTableChunk->Version >= 5 && (pxAnimTableChunk->Flags & uASURA_CHUNK_ANIMTABLE_HAS_ENTRY_FLAGS) )
            {
                for( u_int u = 0; u < uNumEntries; ++u, ++pxAnimFlags )
                {
                    Asura_WriteToStream_Bitfield_Wrapper<u_char> xBitfieldWrapper( pxAnimFlags->m_ucAllFlags );
                    xBitfieldWrapper.Add( pxAnimFlags->m_bMirrored, 1 );
                    xBitfieldWrapper.Add( pxAnimFlags->m_bReversed, 1 );
                    xStream << xBitfieldWrapper;
                }
                xStream.Write4BytePadding();
            }

            // NB: Would need to force src ptr alignment to 4 bytes here for
            // any further data.

				xStream.WriteChunk_End	();

            // Check that matches the expected size.
            Asura_Assert( xStream.GetPosition() == static_cast<u_int>(pxAnimTableChunk->Size), "" );

            // Write out.
            WriteFile( hFile, pucOutputBfr, pxAnimTableChunk->Size, &uBytesWritten, NULL );
            uTotalBytesWritten += uBytesWritten;

            delete[] pucOutputBfr;

            break;
			}
#endif // #ifndef ASURA_DISABLE_ANIM_TABLE_CODE

		case ASURA_CHUNK_HIERARCHY_ANIM:
			{
				// Process, save back out swapped.
				Asura_Chunk_Hierarchy_Anim* pxHierAnimChunk = (Asura_Chunk_Hierarchy_Anim*)pxChunk;
				uTotalBytesWritten += AddAnimationToExport( hFile, pxHierAnimChunk);
			}
			break;

		case ASURA_CHUNK_HIERARCHY_SOUNDS :
		{
			//only interested in these chunks from animation files , since skins will be dealt with by AddBigEndianSkinChunks
			if(!(pxChunk->Flags & Asura_Chunk_Hierarchy_Sounds::ChunkFlag_AnimSounds)) break;
			
			const Asura_Chunk_Hierarchy_Sounds* pxSoundChunk = (const Asura_Chunk_Hierarchy_Sounds*) pxChunk;

			//extract the contents of the chunk
			int iNumberOfSoundDefns;
			Asura_Hierarchy_Sound* pxSoundDefns;
			Asura_String xName;

			if(!pxSoundChunk->Process(&iNumberOfSoundDefns, &pxSoundDefns, &xName)) break;

			//create the chunk again to endian swap it
			Asura_SaveBlock xSaveBlock;
			Asura_Chunk_Hierarchy_Sounds::Create(iNumberOfSoundDefns, pxSoundDefns, &xName, true, &xSaveBlock);
			uTotalBytesWritten = xSaveBlock.WriteToFile(hFile);

			delete [] pxSoundDefns;
			break;

		}

		default:
			break;
	}

	return uTotalBytesWritten;
}

u_int Tools_Level_Export::AddAnimationToExport(HANDLE pSaveHandle, const Asura_Hierarchy_Anim* pxAnim)
{
	Asura_Assert(pxAnim, "");
	if(!pxAnim) return 0;

	//Create a temporary chunk from the animation , as a means of copying the animation
	int iChunkSize = Asura_Chunk_Hierarchy_Anim::GetChunkBufferSize(pxAnim);
	char* pcChunkBuffer = new char[iChunkSize];
	Asura_Chunk_Memory_Stream xStream(iChunkSize,pcChunkBuffer);
	xStream.SetSwapEndian(false);

	Asura_Chunk_Hierarchy_Anim::Create(pxAnim, xStream);
	
	u_int uBytesWritten = AddAnimationToExport(pSaveHandle, (const Asura_Chunk_Hierarchy_Anim*) pcChunkBuffer);

	delete [] pcChunkBuffer;

	return uBytesWritten;
}

u_int Tools_Level_Export::AddAnimationToExport(HANDLE pSaveHandle, const Asura_Chunk_Hierarchy_Anim* pxHierAnimChunk)
{
	Asura_Assert(This,"");
	
	//Create a temporary copy of the animation from the chunk , so it can be safely modified before being written
	Asura_Hierarchy_Anim* pxHierAnim = pxHierAnimChunk->Process();

	// KOR - 28/04/2003 16:16:56 - Copied from ModifyChunkBeforeOutput,
	// as we save the chunk here instead of in the usual way.
	int iFramesBefore,iFramesAfter;
	if(This->ShouldExportAnimationWithoutRemovingKeyframes(*pxHierAnim))
	{
		iFramesBefore	= pxHierAnim->CalculateTotalKeyFrames();
		iFramesAfter	= iFramesBefore;
	}
	else
	{
		pxHierAnim->RemoveUnneededKeyFrames(s_fAnimPositionTolerance,s_fAnimAngleTolerance,&iFramesBefore,&iFramesAfter);
	}

	This->m_iTotalAnimationFramesBefore += iFramesBefore;
	This->m_iTotalAnimationFramesAfter += iFramesAfter;

	DWORD uBytesWritten = 0;
	if(This->ShouldExportAnimationUncompressed(*pxHierAnim))
	{
		AddMessage(MT_LOGONLY,"Exporting Uncompressed Anim - '%s'",pxHierAnim->Name.GetString());

		int iChunkSize = Asura_Chunk_Hierarchy_Anim::GetChunkBufferSize(pxHierAnim);
		char* pcNewChunkData = new char[iChunkSize];
		Asura_Chunk_Hierarchy_Anim::Create(pxHierAnim,pcNewChunkData,iChunkSize);

		WriteFile( pSaveHandle, pcNewChunkData, iChunkSize, &uBytesWritten, NULL );
		Asura_Assert(uBytesWritten == (unsigned)iChunkSize, "Error writing hierarchy animation chunk.");
		delete [] pcNewChunkData;
	}
	else
	{
		pxHierAnim->MakeCompressedKeyFrames();
		int iChunkSize = Asura_Chunk_Hierarchy_CompressedAnim::GetChunkBufferSize(pxHierAnim);
		char* pcNewChunkData = new char[iChunkSize];
		Asura_Chunk_Hierarchy_CompressedAnim::Create(pxHierAnim,pcNewChunkData);
		pxHierAnim->DeleteCompressedKeyFrames();

		WriteFile( pSaveHandle, pcNewChunkData, iChunkSize, &uBytesWritten, NULL );
		Asura_Assert(uBytesWritten == (unsigned)iChunkSize, "Error writing hierarchy animation chunk.");
		delete [] pcNewChunkData;
	}

	// Clean up
	delete pxHierAnim;

	return uBytesWritten;
}

#ifdef ASURA_USE_AXON
void Tools_Level_Export::AddAnimTemplatesToExportList()
{
	//save the animation templates to LevelExportTemp
	char szFilename[100];
	Asura_PrintfToBfr(szFilename,"%s\\AnimTemp.asr",sizeof(szFilename),true, GetTempDirectory());
	if(ShouldExportBigEndian())
	{
		strcat(szFilename,ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX);
	}

	if(Axon_Chunk_AnimTemplates::SaveAnimTemplateAndPackageData(szFilename))
	{
		AddFileToExportList(szFilename,false);
	}
	else
	{
		AddMessage(MT_ERROR,"Failed to create %s",szFilename);
	}
			
}
#endif //ASURA_USE_AXON

bool Tools_Level_Export::ShouldExportAnimationUncompressed(const Asura_Hierarchy_Anim& xAnimation) const
{
	// Standard practice - check what the animation thinks...
	return(xAnimation.m_bDontCompressKeyframes);
}

bool Tools_Level_Export::ShouldExportAnimationWithoutRemovingKeyframes(const Asura_Hierarchy_Anim& xAnimation) const
{
	// Standard practice - check what the animation thinks...
	return(xAnimation.m_bDontRemoveKeyframes);
}

/// End of file. ///