/**
*
* \file Tools_Level_Export.cpp
*
* Created by Kevin Lea - 29/11/2002 11:23:31
* Splitted into separate file 12/11/2008
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"

// Asura
#include "Animation/Asura_Animation.h"
#include "ChunkSystem/Asura_Chunk_MetaMusic.h"
#include "ChunkSystem/Asura_Chunk_ReverbPresets.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_LocalisedText.h"
#include "Debugging/Asura_Logfile.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_System.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Phonon.h"
#include "Sound/Asura_ReverbVolume.h"
#include "Sound/Asura_ReverbVolume_System.h"
#include "Sound/Asura_MetaMusic_System.h"

// Asura PC.
#include "ChunkSystem/Asura_Chunk_Stream_WindowsFile.h"

// Asura PS3.
#include "../PS3/ChunkSystem/Asura_Chunk_ResourceFile_PS3.h"
#include "../PS3/ChunkSystem/Asura_Chunk_ResourceFile_PS3_StreamedSounds.h"

// Asura PS2.
#include "../PS2/ChunkSystem/Asura_Chunk_ResourceFile_PS2.h"
#include "../PS2/ChunkSystem/Asura_Chunk_ResourceFile_PS2_PartiallyStreamedSounds.h"

// Asura PSP.
#include "../PSP/ChunkSystem/Asura_Chunk_ResourceFile_PSP.h"
#include "../PSP/ChunkSystem/Asura_Chunk_ResourceFile_PSP_StreamedSounds.h"

// Asura Wii.
#include "../Wii/ChunkSystem/Asura_Platform_Chunk_ResourceFile.h"
#include "../Wii/ChunkSystem/Asura_Chunk_ResourceFile_Wii_StreamedSounds.h"

// Asura Xbox360
#include "../Xbox360/ChunkSystem/Asura_Chunk_ResourceFile_Xbox360.h"
#include "../Xbox360/ChunkSystem/Asura_Chunk_ResourceFile_Xbox360_StreamedSounds.h"

// Tools
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_SoundResource.h"
#include "Load&Save/Tools_Sound_Export.h"
#include "Load&Save/Tools_ResourceExportLog.h"
#include "Misc/Tools_EnumerationReader.h"
#include "Misc/Tools_IniFile.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "PC/OS/Tools_OS.h"
#include "Shapes/Tools_Shapes.h"
#include "Sound/Tools_Mode_MetaMusic.h"
#include "Sound/Tools_PickSound.h"
#include "Sound/Tools_Sound_PS2Settings.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "Sound/Tools_SoundEvent_System.h"

#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Sound/Asura_Dialogue.h"
#endif

#ifdef ASURA_USE_DECAL_MANAGER
#include "SpecialFX/Asura_Decal_Manager.h"
#include "ChunkSystem/Asura_Chunk_Decals.h"
#endif

// F U N C T I O N S /////////////////////////////////////////////////////////////

// function based on PS2 newsgroups message by Nicolas Bcavin <nbecavin@kalisto.com>
// 02 March 2001 18:24 
// sce.dev.prog.spu 
// Re: Another loopy question
static bool PS2SoundIsLooping(char* pcVagData, int iDataSize)
{
    char* pcLoopByte = pcVagData + iDataSize - 15;

    if (*pcLoopByte == 0x03)
    {
        return true;
    }
    else
    {
        return false;
    }
}


// TOOLS_LEVEL_EXPORT SOUNDS RELATED FUNCTIONS /////////////

void Tools_Level_Export::AddSkinSounds(HANDLE pSaveHandle)
{
	
	UpdateStatus(EXPORT_STAGE_HIERARCHY_SOUNDS, "Hierarchy Sounds", -1);

	if(m_pxSkinFileList->GetNumberOfNodes())
	{
		//get a list of skin file names
		const Asura_String** ppxNameList = new const Asura_String*[m_pxSkinFileList->GetNumberOfNodes()];
		m_pxSkinFileList->MakeNameList(ppxNameList);

		for(int i=0;i<m_pxSkinFileList->GetNumberOfNodes() && !IsAbortFlagSet();i++)
		{
			AddSoundsForSkin(pSaveHandle,*ppxNameList[i]);
		}
		delete [] ppxNameList;
	}

	if(m_pxAnimFileList->GetNumberOfNodes())
	{
		//do the same for anims
		const Asura_String** ppxNameList = new const Asura_String*[m_pxAnimFileList->GetNumberOfNodes()];
		m_pxAnimFileList->MakeNameList(ppxNameList);

		for(int i=0;i<m_pxAnimFileList->GetNumberOfNodes() && !IsAbortFlagSet();i++)
		{
			AddSoundsForAnim(pSaveHandle,*ppxNameList[i]);
		}
		delete [] ppxNameList;
	}


	//Also need to add the sounds for any seperately loaded files that happen to be skins.
	//for now i'll do this by checking the filenames against the names of already loaded skins ,
	//exporting sounds for those that match. This may cause issues if the skins have yet to
	//be loaded in tools , but I don't think this should happen with current projects. RAR 11/12/2003

	for(LIF<Asura_String> xFileLIF(&m_xSeperatelyLoadedFiles);!xFileLIF.done();xFileLIF.next())
	{
		Asura_String xName;
		xFileLIF().SplitFilename(0,&xName,0);

		Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(xName.GetString());
		if(pxSkin)
		{
			AddSoundsForSkin(pSaveHandle,pxSkin);
		}
		
		Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnim(xName.GetString());
		if(pxAnim)
		{
			AddSoundsForAnim(pSaveHandle,pxAnim);
		}

	}

	UpdateStatus(EXPORT_STAGE_END, 0, -1);
}

void Tools_Level_Export::AddSoundsForSkin(HANDLE pSaveHandle,const Asura_String& xFilename)
{
	//determine skin name from filename
	Asura_String xSkinName;
	xFilename.SplitFilename(0,&xSkinName,0);
	//load the file if it isn't already loaded
	if(!Asura_Animation_System::GetSkin(xSkinName.GetString()))
	{
		if(!LoadAsuraFile(xFilename.GetString()))
		{
			AddMessage(MT_ERROR,"Failed to load %s.",xFilename.GetString());
			return;
		}
		if(!Asura_Animation_System::GetSkin(xSkinName.GetString()))
		{
			AddMessage(MT_ERROR,"AddSounds Failed to find skin %s.",xSkinName.GetString());
			return;
		}
	}
	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(xSkinName.GetString());
	if(pxSkin)
	{
		AddSoundsForSkin(pSaveHandle,pxSkin);
	}
}

void Tools_Level_Export::AddSoundsForSkin(HANDLE pSaveHandle,const Asura_Hierarchy_Skin* pxSkin)
{
	TOOLS_RESOURCEEXPORTLOG("Skin",pxSkin->Name.GetString(),false);

	//make sure sounds have been loaded (so IDs are assigned)
	Tools_Animation_Files::LoadSoundsForSkin(pxSkin);
	
	for(int i=0;i<pxSkin->m_iNumberOfSoundDefns;i++)
	{
		if(pxSkin->m_pxSoundDefns[i].m_uSoundEvent)
		{
			AddSoundEventToExportList(pxSkin->m_pxSoundDefns[i].m_uSoundEvent, pxSkin->Name.GetString());
		}
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		if(pxSkin->m_pxSoundDefns[i].m_uDialogueEvent)
		{
			AddDialogueEventToExportList(pxSkin->m_pxSoundDefns[i].m_uDialogueEvent);
		}
		#endif
		
		if(pxSkin->m_pxSoundDefns[i].m_xSoundName.IsBlank()) continue;

		#ifdef ASURA_USE_AUDIO_SYSTEM
			// Dummy sound ID
			int iSoundID = 0;
		#else
		int iSoundID = Tools_PickSound::GetSoundIDForFilename(pxSkin->m_pxSoundDefns[i].m_xSoundName.GetString(), GetMinHierarchySoundID(), GetMaxHierarchySoundID());
		if(iSoundID == ASURA_SID_NO_SOUND)
		{
			AddMessage(MT_ERROR,"No sound ID found for %s.",pxSkin->m_pxSoundDefns[i].m_xSoundName.GetString());
		}
		#endif

		int iFrequency = 0;
		if( Tools_SoundEvent_System::DoesPlatformUseCompressionQuality( s_eExportType ) )
		{
			iFrequency = TSS_QUALITY_MEDIUM;
		}
		AddSound(pSaveHandle,pxSkin->m_pxSoundDefns[i].m_xSoundName,iSoundID, iFrequency );
	}
}

void Tools_Level_Export::AddSoundsForAnim(HANDLE pSaveHandle,const Asura_String& xFilename)
{
	//determine skin name from filename
	Asura_String xAnimName;
	xFilename.SplitFilename(0,&xAnimName,0);
	//load the file if it isn't already loaded
	if(!Asura_Animation_System::GetAnim(xAnimName.GetString()))
	{
		if(!LoadAsuraFile(xFilename.GetString()))
		{
			AddMessage(MT_ERROR,"Failed to load %s.",xFilename.GetString());
			return;
		}
		if(!Asura_Animation_System::GetAnim(xAnimName.GetString()))
		{
			AddMessage(MT_ERROR,"AddSounds Failed to find anim %s.",xAnimName.GetString());
			return;
		}
	}
	Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnim(xAnimName.GetString());
	if(pxAnim)
	{
		AddSoundsForAnim(pSaveHandle,pxAnim);
	}
}

void Tools_Level_Export::AddSoundsForAnim(HANDLE pSaveHandle,const Asura_Hierarchy_Anim* pxAnim)
{
	TOOLS_RESOURCEEXPORTLOG("Anim",pxAnim->Name.GetString(),false);

	//make sure sounds have been loaded (so IDs are assigned)
	Tools_Animation_Files::LoadSoundsForAnim(pxAnim);
	
	for(int i=0;i<pxAnim->m_iNumberOfSoundDefns;i++)
	{
		if(pxAnim->m_pxSoundDefns[i].m_uSoundEvent)
		{
			AddSoundEventToExportList(pxAnim->m_pxSoundDefns[i].m_uSoundEvent, pxAnim->Name.GetString());
		}
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		if(pxAnim->m_pxSoundDefns[i].m_uDialogueEvent)
		{
			AddDialogueEventToExportList(pxAnim->m_pxSoundDefns[i].m_uDialogueEvent);
		}
		#endif
		
		
		if(pxAnim->m_pxSoundDefns[i].m_xSoundName.IsBlank()) continue;

		#ifdef ASURA_USE_AUDIO_SYSTEM
			// Dummy sound ID
			int iSoundID = 0;
		#else
		int iSoundID = Tools_PickSound::GetSoundIDForFilename(pxAnim->m_pxSoundDefns[i].m_xSoundName.GetString(),GetMinHierarchySoundID(), GetMaxHierarchySoundID());
		if(iSoundID == ASURA_SID_NO_SOUND)
		{
			AddMessage(MT_ERROR,"No sound ID found for %s.",pxAnim->m_pxSoundDefns[i].m_xSoundName.GetString());
		}
		#endif

		int iFrequency = 0;
		if( Tools_SoundEvent_System::DoesPlatformUseCompressionQuality( s_eExportType ) )
		{
			// Anim sounds also treaded differently, why are we not using the sound events/samples system
			iFrequency = TSS_QUALITY_MEDIUM;
		}
		AddSound(pSaveHandle,pxAnim->m_pxSoundDefns[i].m_xSoundName,iSoundID, iFrequency);
	}
}


void Tools_Level_Export::AddEnvironmentalSounds(HANDLE pSaveHandle)
{
	TOOLS_RESOURCEEXPORTLOG("Environment",0,false);
	
	//determine the sound events being used by phonons.
	//Not certain that this is the best place to handle it , but will do well enough.
	//RAR 14/07/2006
	for(int i = 0; i < Asura_Phonon_System::GetNumberOfPhonons(); i++)
	{
		const Asura_Phonon* pxPhonon = Asura_Phonon_System::GetPhononByIndex(i);
		if(!pxPhonon->IsMemberOfCurrentLightSet()) continue;
		if(!pxPhonon->IsEnabledForPlatform(s_eExportType)) continue;

		if(pxPhonon->GetEvent())
		{
			AddSoundEventToExportList(pxPhonon->GetEvent()->GetHashID(), pxPhonon->GetName()->GetString());
		}
	}
}

void Tools_Level_Export::AddSounds(HANDLE saveHandle,const char* pszHeading,int iMinID,int iMaxID)
{
	#ifndef ASURA_USE_AUDIO_SYSTEM	// GetNumberOfNamedSounds should return 0 when using the audio system

	int iNumSounds = Asura_Sound_System::GetNumberOfNamedSounds((Asura_Sound_ID)iMinID,(Asura_Sound_ID)iMaxID);

	if(!iNumSounds) return;

	//create an array to get the sounds names
	const Asura_String** nameList = new const Asura_String*[iNumSounds];
	//initialise array
	int i;
	for(i=0;i<iNumSounds;i++)
	{
		nameList[i] = NULL;	
	}

	//and get those names
	Asura_Sound_System::MakeNameList(nameList,(Asura_Sound_ID)iMinID,(Asura_Sound_ID)iMaxID);

	UpdateStatus(EXPORT_STAGE_ENTITY_SOUNDS, pszHeading, -1);
	
	//export sound resources for each of these
	for(i = 0;i<iNumSounds && !IsAbortFlagSet();i++)
	{
		s_iPercent = i*100/iNumSounds;
		if(nameList[i])
		{
			UpdateProgress();
			//add this sound
			Asura_Sound_ID eSoundID = Asura_Sound_System::GetSoundIDFromFileName(nameList[i]->GetString(),(Asura_Sound_ID)iMinID,(Asura_Sound_ID)iMaxID);

			AddSound(saveHandle,*nameList[i],eSoundID,0);

		}
	}
	UpdateStatus(EXPORT_STAGE_END, 0, -1);

	delete [] nameList;
	#endif // #ifndef ASURA_USE_AUDIO_SYSTEM
}

void Tools_Level_Export::AddSoundToExportList(int iSoundID,bool bOnlyUsedByHash /*= false*/)
{
	// TODO: TY 03.12.08 - Is this function actually called by anything?
	#ifndef ASURA_USE_AUDIO_SYSTEM	

	Asura_Assert(This,"");
	if(iSoundID==ASURA_SID_NO_SOUND) return;
	
	//make sure a filename is available
	const char* pszFilename = Asura_Sound_System::GetFileNameFromSoundID((Asura_Sound_ID)iSoundID);
	if(!pszFilename)
	{
		Tools_Level_Export::AddMessage(MT_ERROR,"Failed to get filename for sound %d",iSoundID);
		return;
	}

	//check for an existing entry
	for(LIF<SoundExportData*> xSoundLIF(&This->m_xSoundList);!xSoundLIF.done();xSoundLIF.next())
	{
		if(xSoundLIF()->m_iID == iSoundID)
		{
			if(!bOnlyUsedByHash) xSoundLIF()->m_bOnlyUsedByHash = false;		
			return;
		}
	}

	//add to the list
	SoundExportData* pxSound = new SoundExportData;
	pxSound->m_iID = iSoundID;
	pxSound->m_xFilename = pszFilename;
	pxSound->m_iFrequency = 0;
	pxSound->m_bOnlyUsedByHash = bOnlyUsedByHash;
	This->m_xSoundList.add_entry(pxSound);

	#endif // #ifndef ASURA_USE_AUDIO_SYSTEM
}

bool Tools_Level_Export::AddIniFileSoundsToExportList(const char* szIniFilename,bool (*pfnShouldExport)(int iSoundID) /*= 0*/)
{
	Asura_Assert(This,"");
	Tools_EnumerationReader* pxSoundEnum = GetSoundIDEnumeration();
	if(!pxSoundEnum) return false;

	return AddSoundsFromIniFile(szIniFilename,pxSoundEnum,This->m_xSoundList,pfnShouldExport);
}

void Tools_Level_Export::AddSoundToIgnore(int iSoundID,const char* szFilename)
{
	Asura_Assert(This,"");
	
	#ifndef ASURA_USE_AUDIO_SYSTEM
	//make sure there isn't already an entry for this sound ID
	if(iSoundID != ASURA_SID_DYNAMIC)
	{
		for(LIF<SoundExportData*> xExportedLIF(&This->m_xExportedSoundList);!xExportedLIF.done();xExportedLIF.next())
		{
			if(xExportedLIF()->m_iID == iSoundID)
			{
				//already exists...hopefully with the same filename
				Asura_Assert(xExportedLIF()->m_xFilename.EqualsIgnoringExtension(szFilename),"Sound ID already found in export list ,but with a different filename");
				return;
			}
		}
	}
	#else
	//Search for this filename in our list
	for(LIF<SoundExportData*> xExportedLIF(&This->m_xExportedSoundList);!xExportedLIF.done();xExportedLIF.next())
	{
		if(xExportedLIF()->m_xFilename.EqualsIgnoringExtension( szFilename ) )
		{
			return;
		}
	}
	#endif

	//add entry to already exported list
	SoundExportData* pxExportData = new SoundExportData;
	pxExportData->m_iID = iSoundID;
	pxExportData->m_xFilename = szFilename;
	This->m_xExportedSoundList.add_entry(pxExportData);
		
}

void Tools_Level_Export::AddSound( HANDLE saveHandle, const Asura_String& filename, int iSoundID , int iFrequency )
{
	SoundExportData xSound;
	xSound.m_xFilename = filename;
	xSound.m_iID = iSoundID;
	xSound.m_iFrequency = iFrequency;

	AddSound(saveHandle,&xSound);
}

void Tools_Level_Export::AddSound(HANDLE pSaveHandle,const SoundExportData* pxSound)
{
	bool bWavAlreadyExported = false;

	#ifdef ASURA_USE_AUDIO_SYSTEM

	int iSoundID = 0;

	// Check that this sound hasn't already been exported
	for( LIF<SoundExportData*> xExportedLIF( &m_xExportedSoundList ); !xExportedLIF.done(); xExportedLIF.next() )
	{
		if( pxSound->m_xFilename.EqualsIgnoringExtension( xExportedLIF()->m_xFilename ) )
		{
			bWavAlreadyExported = true;
			break;
		}
	}

	#else
	
	Asura_Sound_ID eSoundID;

	if(pxSound->m_iID==-1)
	{
		//get the sound ID for this sound
		eSoundID = Asura_Sound_System::GetSoundIDFromFileName(pxSound->m_xFilename.GetString());
		if(eSoundID == ASURA_SID_NO_SOUND)
		{
			//this shouldn't really be possible anyway
			AddMessage(MT_ERROR,"No sound ID found for %s.",pxSound->m_xFilename.GetString());
			return;
		}

		if(eSoundID <ASURA_SID_ENVIRONMENT_00 || eSoundID>ASURA_SID_ENVIRONMENT_MAX)
		{
			return;//only interested in environmental sounds
		}
	}
	else
	{
		eSoundID = (Asura_Sound_ID)pxSound->m_iID;
	}

	if(eSoundID == ASURA_SID_DYNAMIC)
	{
		Asura_Assert(pxSound->m_bOnlyUsedByHash,"Sound only used by hash flag should be set for sounds being exported with a dynamic sound ID");
	}

	int iSoundID = static_cast<int>( eSoundID );


	//see if this sound has already been exported
	for(LIF<SoundExportData*> xExportedLIF(&m_xExportedSoundList);!xExportedLIF.done();xExportedLIF.next())
	{
		if(pxSound->m_xFilename.EqualsIgnoringExtension(xExportedLIF()->m_xFilename))
		{
			//sound already done , so just add an entry to the sound mapping list
			//(don't bother adding a sound mapping if the sound is only used by hash ID , since 
			//it won't be needed)
			bWavAlreadyExported = true;

			if(eSoundID != xExportedLIF()->m_iID && !pxSound->m_bOnlyUsedByHash)
			{
				//If the existing sound ID is dynamic , then we will need to map using the hash ID instead
				Asura_Hash_ID uSoundHash = Asura_GetHashID(xExportedLIF()->m_xFilename.GetString());
				bool bMapByHash = (xExportedLIF()->m_iID == ASURA_SID_DYNAMIC);
				
				if(m_bFakeLevelExport)
				{
					if(!Asura_Sound_System::SoundIsLoaded(eSoundID))
					{
						Asura_Sound_System::LoadSoundIDMapping(eSoundID,(Asura_Sound_ID)xExportedLIF()->m_iID,bMapByHash ? uSoundHash : 0);
					}
				}
				else
				{
					Asura_SoundMapping xMapping;
					xMapping.m_iExistingSoundID = xExportedLIF()->m_iID;
					xMapping.m_iNewSoundID = eSoundID;
					xMapping.m_uExistingSoundHash = bMapByHash ? uSoundHash : 0;

					//Make sure this sound mapping doesn't already exist
					bool bFound = false;
					for(LIF<Asura_SoundMapping> xMapLIF(&m_xSoundMappingList);!xMapLIF.done();xMapLIF.next())
					{
						if(xMapLIF().m_iNewSoundID == xMapping.m_iNewSoundID)
						{
							//assert that the sound being mapped to is the same as well
							if(bMapByHash)
							{
								Asura_Assert(xMapLIF().m_uExistingSoundHash == xMapping.m_uExistingSoundHash,"Attempting to map one sound ID to different sound IDs?");
							}
							else
							{
								Asura_Assert(xMapLIF().m_iExistingSoundID == xMapping.m_iExistingSoundID,"Attempting to map one sound ID to different sound IDs?");
							}
							//already exists in list , so we can skip it
							bFound = true;
							break;
						}
					}
					if(!bFound)
					{
						m_xSoundMappingList.add_entry(xMapping);
					}
				}
			}
			if(bWavAlreadyExported) break;
		}
	}

	#endif	// #ifndef ASURA_USE_AUDIO_SYSTEM

	if(!bWavAlreadyExported)
	{
		TOOLS_RESOURCEEXPORTLOG("Wav",pxSound->m_xFilename.GetString(),true);
		
		
		//hasn't been exported yet , add to the list of sounds we've done
		SoundExportData* pxExportData = new SoundExportData;
		*pxExportData = *pxSound;
		pxExportData->m_iID = iSoundID;
		m_xExportedSoundList.add_entry(pxExportData);

		if(m_bFakeLevelExport)
		{
			#ifndef ASURA_USE_AUDIO_SYSTEM	// why do this...?
			//just load the sound if it doesn't exist
			if(!Asura_Sound_System::SoundIsLoaded(eSoundID))
			{
				Asura_Sound_System::LoadSound(pxSound->m_xFilename.GetString(),eSoundID);
			}
			#endif
		}
		else
		{
			if (pxExportData->m_bStreamed)
			{		
				Asura_Assert(pxExportData->m_bLocalise, "only supporting localised streamed sounds at the moment");
				m_xLocalisedStreamedSoundList.add_entry(pxExportData);
				SetCreateLanguageFiles(pSaveHandle);
			}
			else if(pxExportData->m_bLocalise)
			{
				//add to list for the localised export , instead of adding to the current file
				m_xLocalisedSoundList.add_entry(pxExportData);
				SetCreateLanguageFiles(pSaveHandle);
			}
			else
			{
				AddSound(pSaveHandle,pxExportData,-1);
			}
		}
	}

	#ifndef ASURA_USE_AUDIO_SYSTEM	// Sound properties are gone

	//Does the sound have some default sound properties to export?
	if(pxSound->HasSoundProperties())
	{
		//search for an existing entry
		bool bFound = false;
		for(LIF<Tools_SoundProperties> xPropLIF(&m_xSoundPropertiesList);!xPropLIF.done();xPropLIF.next())
		{
			if(xPropLIF().m_iSoundID == eSoundID)
			{
				//already exists in list , so we can skip it
				bFound = true;
				break;
			}
		}
		if(!bFound)
		{
			Tools_SoundProperties xProp;
			xProp.m_iSoundID = eSoundID;
			xProp.m_fPitchScale = pxSound->m_fPitchScale;
			xProp.m_fVolumeScale = pxSound->m_fVolumeScale;
			xProp.m_fDefaultInnerRadius = pxSound->m_fDefaultInnerRadius;
			xProp.m_fDefaultOuterRadius = pxSound->m_fDefaultOuterRadius;
			m_xSoundPropertiesList.add_entry(xProp);

			//If quitting to game , process the sound properties immediately
			if(m_bFakeLevelExport)
			{
				Asura_SoundProperties* pxProperties = Asura_Sound_System::GetSoundPropertiesByID((Asura_Sound_ID)eSoundID);
				if(pxProperties)
				{
					if(xProp.m_fPitchScale>=0)
					{
						pxProperties->m_fPitchScale = xProp.m_fPitchScale;
					}
					if(xProp.m_fVolumeScale>=0)
					{
						pxProperties->m_fVolumeScale = xProp.m_fVolumeScale;
					}
					if(xProp.m_fDefaultInnerRadius>=0)
					{
						pxProperties->m_fDefaultInnerRadius = xProp.m_fDefaultInnerRadius;
					}
					if(xProp.m_fDefaultOuterRadius>=0)
					{
						pxProperties->m_fDefaultOuterRadius = xProp.m_fDefaultOuterRadius;
					}
				}		
			}
		}
	}

	#endif	// #ifndef ASURA_USE_AUDIO_SYSTEM
}

static void LocaliseSoundFilename(char* const szOutput, const u_int uOutputSize,
	const char* const szEnglishFilename, const Asura_LocalisedText_Language eLanguage)
{
	Asura_PrintfToBfr(szOutput, "%s", uOutputSize, true, szEnglishFilename);

	//Search for a directory named 'en' in the filename
	char* pcPos = &szOutput[0];
	while(*pcPos)
	{
		if((pcPos[0] == '\\' || pcPos[0] == '/') &&
		   (pcPos[1] == 'e'  || pcPos[1] == 'E') &&
		   (pcPos[2] == 'n'  || pcPos[2] == 'N') &&
		   (pcPos[3] == '\\' || pcPos[3] == '/'))
		{
			pcPos++;//advance to the 'e'
			break;
		}
		pcPos++;
	}

	if(*pcPos)
	{
		//replace by the code for the current language
		const char* szLangCode = Asura_LocalisedText::GetTwoCharCodeForLang(eLanguage);
		pcPos[0] = szLangCode[0];
		pcPos[1] = szLangCode[1];

		//use this file if it exists (otherwise make use of the english file)
		if(!Tools_FileSystem::FileExists_CheckNetwork(szOutput))
		{
			Asura_PrintfToBfr(szOutput, "%s", uOutputSize, true, szEnglishFilename);
		}
	}
}

void Tools_Level_Export::AddSound(HANDLE pSaveHandle,const SoundExportData* pxSound,int iLanguage)
{
	if(pSaveHandle == INVALID_HANDLE_VALUE)
	{
		//May get here without a proper file handle if only set to export localised files.
		//In that cases , it is safe to skip the rest of this function.
		//RAR 09/03/2009
		return;
	}
	
	const char* szLocalisedFilename = pxSound->m_xFilename.GetString();
	char acLocalisedFilenameBuffer[MAX_PATH];

	if(iLanguage>= 0 && iLanguage != ASURA_LT_L_ENGLISH)
	{
		LocaliseSoundFilename(acLocalisedFilenameBuffer, MAX_PATH, szLocalisedFilename,
			static_cast<Asura_LocalisedText_Language>(iLanguage));
	}

	// if sound is streamed, and the platform supports streamed sounds, add as streamed, else fallthrough.
	if (pxSound->m_bStreamed)
	{
		Asura_Assert( m_bAddingStreamedSounds, "unexpected streamed sound");
		if (TargetPlatformSupportsStreamedSounds())
		{
			AddStreamedSoundToList(pxSound, szLocalisedFilename, pxSound->m_xFilename.GetHash() );
			return;
		}
	}

	HANDLE pFile;

	Asura_String xOutputFilename;
	bool bLoop = true;//appears loop flag should always be set. RAR 08/07/03
	if(ConvertSound(szLocalisedFilename,pxSound->m_iFrequency ,bLoop,&xOutputFilename, pxSound->m_bStreamed))
	{		
		//open the converted file
		pFile = CreateFile(xOutputFilename.GetString(),GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0)	;

		Asura_Assert(pFile!=INVALID_HANDLE_VALUE,"");


		//allocate a pcBuffer , and read in the file
		int iTrueFileSize = GetFileSize(pFile,0);
		int iFileSize = ((iTrueFileSize+3)/4)*4;
		char* pcBuffer = new char[iFileSize];
		unsigned long uBytesRead;
		ReadFile(pFile,pcBuffer,iTrueFileSize,&uBytesRead,0);
		CloseHandle(pFile);

  			if (s_eExportType == Export_PS2)
  			{
  			    bool bLooping = PS2SoundIsLooping(pcBuffer, iTrueFileSize);
  			    if (bLooping != bLoop)
  			    {
  			        AddMessage(MT_ERROR, "VAG loop flag (%d) is different from conversion data loop flag(%d)", bLooping, bLoop);
  			    }
  			    else
  			    {
  			        AddMessage(MT_LOGONLY, "VAG loop flag = %d.", bLooping);
  			    }
  			}

		Asura_String xFilenameForResource = pxSound->m_xFilename;//use un-localised filename in resource so that the same hash value is used for each language

		Asura_Assert(sizeof(Asura_Chunk_ResourceFile) == (7*sizeof(int)), "refactoring assumption");
		int iTotalSize = (sizeof(Asura_Chunk_ResourceFile)) + (xFilenameForResource.GetSizeInInts()*4) + (iFileSize);

		int iChunkFlags = 0;
		iChunkFlags |= (pxSound->m_bLooping ? Asura_Chunk_ResourceFile::SF_LOOPING : 0);

		//create the chunk header
		Asura_Chunk_ResourceFile xChunk;
		xChunk.Create(iTotalSize, iChunkFlags, ASURA_RESOURCEFILE_TYPE_SOUND, pxSound->m_iID, iFileSize);

		unsigned long uBytesWritten;
		WriteFile(pSaveHandle,&xChunk,sizeof(xChunk),&uBytesWritten,0);

		// add archivename (padded to int)
		char acNameBuffer[MAX_PATH] = {0};
		strcpy(acNameBuffer,xFilenameForResource.GetString());
		WriteFile(pSaveHandle,acNameBuffer,xFilenameForResource.GetSizeInInts()*4,&uBytesWritten,0);

		// add contents of file
		WriteFile(pSaveHandle,pcBuffer,iFileSize,&uBytesWritten,0);

		delete [] pcBuffer;

   		// (NRM - 27.06.2003) sound logging
		Asura_SimpleLogfileOutput((s_szSoundLogPath, "Adding sound ID : %6i \t size %7i \t file : %s \n", pxSound->m_iID, iFileSize, szLocalisedFilename));
	}
}


bool Tools_Level_Export::TargetPlatformSupportsStreamedSounds()
{
	return (	( s_eExportType == Export_PS2 )
			||	( s_eExportType == Export_PSP )
			||	( s_eExportType == Export_Wii )
			||	( s_eExportType == Export_Xbox360)
			||	( s_eExportType == Export_PS3 )
			#ifdef ASURA_USE_AUDIO_SYSTEM
			||	( s_eExportType == Export_PC )
			#endif
			);
}

void Tools_Level_Export::AddStreamedSoundToList(const SoundExportData* pxSound, const char* szLocalisedFilename, Asura_Hash_ID uFilenameHash)
{
	// PS2 streaming uses normal VAGs (not supporting partially streamed at the moment, as that would
	// require the VAGs to be split into 2 parts).
	Asura_String xOutputFilename;
	bool bLoop = true;//appears loop flag should always be set. RAR 08/07/03
	if(ConvertSound(szLocalisedFilename,pxSound->m_iFrequency ,bLoop,&xOutputFilename, pxSound->m_bStreamed))
	{
		SoundExportData* pxExportData = new SoundExportData;
		*pxExportData = *pxSound;
		pxExportData->m_xFilename = xOutputFilename;
		#ifndef ASURA_USE_AUDIO_SYSTEM
		Asura_Assert( pxExportData->m_iID == ASURA_SID_DYNAMIC, "using ID for filename hash");
		#endif
		pxExportData->m_iID = uFilenameHash;
		if( Tools_SoundEvent_System::DoesPlatformUseCompressionQuality( s_eExportType ) )
		{
			pxExportData->m_iFrequency = TSS_QUALITY_MEDIUM;
		}
		m_xLocalisedStreamedSoundExportList.add_entry(pxExportData);
	}
}

void Tools_Level_Export::AddStreamedSoundsFromList(HANDLE pSaveHandle)
{
	const int iNumSounds = m_xLocalisedStreamedSoundExportList.size();

	// get current position in export file - this is used to determine sound offsets in file.
	u_int uFilePosition = SetFilePointer( pSaveHandle, 0, NULL, FILE_CURRENT );
	if( INVALID_SET_FILE_POINTER == uFilePosition)
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "I/O error. Export aborted." );
		return;
	}

	// chunk header size
	const Asura_String xFilenameForResource = "streamed sounds";
	const u_int uHeaderSize = (sizeof(Asura_Chunk_ResourceFile)) + (xFilenameForResource.GetSizeInInts()*4);

	// header size (part of resource size)
	u_int uHeaderEntrySize = 0;

	//create the chunk header
	Asura_SaveBlock xSaveBlock;
	Asura_Chunk_SaveBlock_Stream xStream(&xSaveBlock);

	switch (s_eExportType)
	{
		case Export_PS2:
		{
			xStream.WriteResourceChunk_Start(
				ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
				ASURA_RESOURCEFILE_TYPE_PS2_PARTIALLYSTREAMEDSOUNDS,
				0,
				uASURA_PS2_PSS_FLAG_SOUNDIDS_ARE_DYNAMIC,
				xFilenameForResource.GetString());
			xStream << ASURA_PS2_PSS_CHUNK_VERSION;
			uHeaderEntrySize = sizeof(Asura_Chunk_ResourceFile_PS2_PartiallyStreamedSounds::HdrEntry);
			break;
		}
		case Export_PSP:
		{
			xStream.WriteResourceChunk_Start(
				ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
				ASURA_RESOURCEFILE_TYPE_PSP_STREAMINGSOUNDS,
				0,
				uASURA_PSP_STREAMEDSOUNDS_FLAG_SOUNDIDS_ARE_DYNAMIC,
				xFilenameForResource.GetString());
			xStream << ASURA_PSP_STREAMEDSOUNDS_CHUNK_VERSION;
			uHeaderEntrySize = sizeof(Asura_Chunk_ResourceFile_PSP_StreamedSounds::HdrEntry);
			break;
		}
		case Export_Wii:
		{
			xStream.WriteResourceChunk_Start(
				ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
				ASURA_RESOURCEFILE_TYPE_WII_STREAMINGSOUNDS,
				0,
				0,
				xFilenameForResource.GetString());
			xStream << ASURA_WII_STREAMEDSOUNDS_CHUNK_VERSION;
			uHeaderEntrySize = sizeof(Asura_Chunk_ResourceFile_Wii_StreamedSounds::HdrEntry);
			break;
		}
		default:
		{
			Asura_Assert( false, "Unhandled platform" );
			return;
		}
	}

	xStream << iNumSounds;

	// sound offsets start after headers which are after the chunk header
	uFilePosition += uHeaderSize + 2 * sizeof(int) + (iNumSounds * uHeaderEntrySize);

	u_int uPadding = 0;
	if(s_eExportType == Export_Wii)
	{
		// We need to pad to 32 byte aligned.
		if(uFilePosition % 32)
		{
			uPadding = (32 - uFilePosition) & 31;
			uFilePosition += uPadding;
			Asura_Assert(!(uFilePosition & 31), "Padding calculation failed");
		}
	}

	// write header for each file
	int iCount = 0;
	for(LIF<const SoundExportData*> xSoundLIF(&m_xLocalisedStreamedSoundExportList); !xSoundLIF.done() && !IsAbortFlagSet(); xSoundLIF.next(),iCount++)
	{
		// arbitrary 2% (out of 10%) of process allocated to this part of function
		s_iPercent = 90 + iCount*2/iNumSounds;
		UpdateProgress();

		HANDLE pFile;
		//open the converted file
		pFile = CreateFile(xSoundLIF()->m_xFilename.GetString(),GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0)	;
			
		Asura_Assertf(pFile!=INVALID_HANDLE_VALUE, ("failed to open file we just created (%s)", xSoundLIF()->m_xFilename.GetString()));

		const int iTrueFileSize = GetFileSize(pFile,0);
		const int uAlignment = (s_eExportType == Export_Wii) ? 32 : 4;
		const int iFileSize = (iTrueFileSize + uAlignment - 1) & ~(uAlignment - 1);

		CloseHandle(pFile);

		// write header entry for this sound
		Asura_Assert(iCount < iNumSounds, "incorrect streaming sound count");
		switch (s_eExportType)
		{
			case Export_PS2:
			{
				xStream << xSoundLIF()->m_iID;
				xStream << xSoundLIF()->m_iFrequency;
				xStream << 0u;
				xStream << 0u;
				xStream << uFilePosition;
				xStream << iFileSize;
				break;
			}
			case Export_PSP:
			{
				xStream << xSoundLIF()->m_iID;
				xStream << uFilePosition;
				xStream << iFileSize;
				break;
			}
			case Export_Wii:
			{
				xStream << xSoundLIF()->m_iID;
				xStream << uFilePosition;
				xStream << iFileSize;
				break;
			}
			default:
			{
				Asura_Assert( false, "Unhandled platform" );
				break;
			}
		}

		uFilePosition += iFileSize;
	}

	// Align files if necessary (padding defaults to zero).
	xStream.WritePadding(uPadding);

	// read in each file and add to export file
	iCount = 0;
	for(LIF<const SoundExportData*> xSoundLIF(&m_xLocalisedStreamedSoundExportList); !xSoundLIF.done() && !IsAbortFlagSet(); xSoundLIF.next(),iCount++)
	{
		// arbitrary 8% (out of 10%) of process allocated to this part of function
		s_iPercent = 90 + iCount*8/iNumSounds;
		UpdateProgress();

		HANDLE pFile;
		//open the converted file
		pFile = CreateFile(xSoundLIF()->m_xFilename.GetString(),GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0)	;
			
		Asura_Assertf(pFile!=INVALID_HANDLE_VALUE, ("failed to open file we just created (%s)", xSoundLIF()->m_xFilename.GetString()));

		//allocate a pcBuffer , and read in the file
		const int iTrueFileSize = GetFileSize(pFile,0);
		const int uAlignment = (s_eExportType == Export_Wii) ? 32 : 4;
		const int iFileSize = (iTrueFileSize + uAlignment - 1) & ~(uAlignment - 1);
		char* pcBuffer = new char[iFileSize];

		unsigned long uBytesRead;
		ReadFile(pFile,pcBuffer,iTrueFileSize,&uBytesRead,0);
		CloseHandle(pFile);

		if(s_eExportType == Export_PS2)
		{
			bool bLoop = true;//appears loop flag should always be set. RAR 08/07/03
			bool bLooping = PS2SoundIsLooping(pcBuffer, iTrueFileSize);
			if (bLooping != bLoop)
			{
				AddMessage(MT_ERROR, "VAG loop flag (%d) is different from conversion data loop flag(%d)", bLooping, bLoop);
			}
			else
			{
				AddMessage(MT_LOGONLY, "VAG loop flag = %d.", bLooping);
			}
		}

		// add the sound data
		xStream.Write(iFileSize, pcBuffer);

		delete [] pcBuffer;

		// sound logging
		Asura_SimpleLogfileOutput((s_szSoundLogPath, "Adding streamed sound ID : %6i \t size %7i \t file : %s \n", xSoundLIF()->m_iID, iFileSize, xSoundLIF()->m_xFilename.GetString()));
	}

	xStream.WriteResourceChunk_End();

	xSaveBlock.WriteToFile(pSaveHandle);
}

bool Tools_Level_Export::ConvertSound(const char* pszWavFilename, int iFrequency, bool bLooped, Asura_String* pxOutputName, bool bStreaming)
{
	HANDLE file;
	
	char szFileToLoad[MAX_PATH] = "";
	if(s_eExportType == Export_PS2)
	{
		//see if the .wav file exists in the ps2 directory
		Asura_String xPath,xName,xExtension;
		Asura_String xFilename = pszWavFilename;
		xFilename.SplitFilename(&xPath,&xName,&xExtension);

		char szTemp[MAX_PATH] = "";
		Asura_PrintfToBfr(szTemp,"%s\\ps2\\%s.%s",MAX_PATH,true,xPath.GetString(),xName.GetString(),xExtension.GetString());
		Asura_FileAccess::PlatformGetNetworkFilename(szTemp,szFileToLoad,MAX_PATH);

		if(!Tools_FileSystem::FileExists(szFileToLoad))
		{
			//file didn't exist , blank out the name to try in the normal directory
			szFileToLoad[0] = 0;
		}
	}

	if(!*szFileToLoad)
	{
		Asura_FileAccess::PlatformGetNetworkFilename(pszWavFilename,szFileToLoad,MAX_PATH);
	}

	//see if we can find the file
	file = CreateFile(szFileToLoad,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
	
	if(file == INVALID_HANDLE_VALUE)
	{
		AddMessage(MT_ERROR,"Failed to find %s.",szFileToLoad);
		return false;
	}
	CloseHandle(file);

	if( !Tools_SoundEvent_System::DoesPlatformUseCompressionQuality( s_eExportType ) )
	{
		if( !iFrequency ) iFrequency = Tools_Sound_PS2Settings::GetFrequency( pszWavFilename );
		if( !iFrequency ) iFrequency = Tools_SoundEvent_System::GetSampleRate(s_eExportType,pszWavFilename);
		if( !iFrequency ) iFrequency = GetDefaultMaxSoundFrequency( pszWavFilename );
	}
	else
	{
		// For these sounds the 'Max frequency' value is currently a look up value (TSS_QUALITY_*) for the compression quality
		// at this point (before the conversion propper) we replace it with the associated global quality value for this platform
		Asura_Assert( ( iFrequency >= TSS_QUALITY_USE_DEFAULT ) && ( iFrequency < TSS_QUALITY_NUM_ENTRIES ), "Invalid compression quality" );
		iFrequency = Tools_SoundEvent_System::GetPlatformCompressionQuality( s_eExportType, iFrequency );
	}
	
	ToolsSoundConversionData xConvertData;
	xConvertData.m_iMaxFrequency = iFrequency;
	xConvertData.m_bLoop = bLooped;
	

	//Allow project code to have a say in the conversion process
	Project_ModifySoundConversionSettings(&xConvertData, pszWavFilename, bStreaming);
	
	Asura_String xOutputName;

	bool bOkay = true;

	if ((s_eExportType == Export_PSP) && bStreaming && TargetPlatformSupportsStreamedSounds())
	{
		//create a temporary vag file
		Tools_FileSystem::GenerateFilenameForConvertedFile(szFileToLoad,"PSP_AT3","at3",xOutputName);
		Tools_FileSystem::EnsureDirectoryExists(xOutputName.GetString());

		//do the conversion
		if(!Tools_Sound_Export::ExportAT3File(szFileToLoad,xOutputName.GetString(),&xConvertData))
		{
			bOkay = false;
		}
	}
	else if((s_eExportType == Export_PS2) || (s_eExportType == Export_PSP))
	{
		//create a temporary vag file
		if(s_eExportType == Export_PSP)
			Tools_FileSystem::GenerateFilenameForConvertedFile(szFileToLoad,"PSP_Vags","vag",xOutputName);
		else
			Tools_FileSystem::GenerateFilenameForConvertedFile(szFileToLoad,"Vags","vag",xOutputName);
		Tools_FileSystem::EnsureDirectoryExists(xOutputName.GetString());

		//do the conversion
		if(!Tools_Sound_Export::ExportVagFile(szFileToLoad,xOutputName.GetString(),&xConvertData))
		{
			bOkay = false;
		}
	}
	else if(s_eExportType == Export_Wii)
	{
		if(bStreaming && s_bWiiStreamsUseBink)
		{
			// Create a bink.
			Tools_FileSystem::GenerateFilenameForConvertedFile(szFileToLoad,"BIKs","bik",xOutputName);
			Tools_FileSystem::EnsureDirectoryExists(xOutputName.GetString());

			if(!Tools_Sound_Export::ExportBinkFile(szFileToLoad,xOutputName.GetString(),&xConvertData))
			{
				bOkay = false;
			}
		}
		else
		{
			//create a temporary dsp file
			Tools_FileSystem::GenerateFilenameForConvertedFile(szFileToLoad,"DSPs","dsp",xOutputName);
			Tools_FileSystem::EnsureDirectoryExists(xOutputName.GetString());

			if(!Tools_Sound_Export::ExportDSPFile(szFileToLoad,xOutputName.GetString(),&xConvertData))
			{
				bOkay = false;
			}
		}
	}
	else if( s_eExportType == Export_Xbox360 )
	{
		// Create a temporary XMA or xWMA file
		#if defined ASURA_USE_AUDIO_SYSTEM
			const char* const szDir = /*bStreaming ? "xWMAs" :*/ "XMA2s";
			const char* const szExt = /*bStreaming ? "xwma" :*/ "xma";
		#else
			const char* const szDir = "XMAs";
			const char* const szExt = "xma";
		#endif

		Tools_FileSystem::GenerateFilenameForConvertedFile( szFileToLoad, szDir, szExt, xOutputName );
		Tools_FileSystem::EnsureDirectoryExists( xOutputName.GetString() );

		// (AAM - 13.10.2008) For now, we'll just stick to XMA2 format. With xWMA, we need to update
		// the DPDS data submitted with each packet when streaming, and I haven't figured out how to
		// do that cleanly yet

		//#if defined ASURA_USE_AUDIO_SYSTEM
		//if( bStreaming )
		//{
		//	if( !Tools_Sound_Export::ExportxWMAFile( szFileToLoad, xOutputName.GetString(), &xConvertData ) )
		//	{
		//		bOkay = false;
		//	}
		//}
		//else
		//#endif
		{
			if( !Tools_Sound_Export::ExportXMAFile( szFileToLoad, xOutputName.GetString(), &xConvertData ) )
			{
				bOkay = false;
			}
		}
	}
	else if( s_eExportType == Export_PS3 )
	{
		Tools_FileSystem::GenerateFilenameForConvertedFile( szFileToLoad, "MSFs", "msf", xOutputName );
		Tools_FileSystem::EnsureDirectoryExists( xOutputName.GetString() );

		// Create a temporary MSF file
		int iResult = Tools_Sound_Export::ExportMSFFile(szFileToLoad, xOutputName.GetString(), &xConvertData);
		if ( iResult != 0 )
		{
			Asura_OutputToDebugger(("[Tools_Level_Export::ConvertSound] - MSF export failed (result = %d)\n", iResult));
			bOkay = false;
		}
	}
	else
	{
		AddMessage(MT_LOGONLY,"Adding %s....",szFileToLoad);
		xOutputName = szFileToLoad;
	}
	ClearMessageDisplayHeading();
	
	if(bOkay)
	{
		if(pxOutputName) *pxOutputName = xOutputName;
	}
	return bOkay;	
}

Tools_EnumerationReader* Tools_Level_Export::LoadSoundIDEnumeration()
{
	if(!m_pxSoundEnum)
	{
		m_pxSoundEnum = new Tools_EnumerationReader;
		if(!m_pxSoundEnum->ReadEnumeration("ToolsIniFiles\\Asura_ProjectSoundIDs.h","enum Asura_Sound_ID"))
		{
			Asura_Assertf(0,("Error loading Asura_ProjectSoundIDs.h :\n%s",m_pxSoundEnum->GetError()));
			
			delete m_pxSoundEnum;
			m_pxSoundEnum = 0;

		}
	}
	return m_pxSoundEnum;
}

void Tools_Level_Export::BuildNonEnvironmentalSoundList(const List<Asura_String>& xNameList)
{
	//load the sound enumeration header file
	if(!LoadSoundIDEnumeration())
	{
		return;
	}

   	char szFilename[MAX_PATH];
	
	for(CLIF<Asura_String> xNameLIF(&xNameList);!xNameLIF.done();xNameLIF.next())
	{
		//build the name of the inifile to look at
		Asura_String xName;
		Asura_String xExtension;
		xNameLIF().SplitFilename(0,&xName,&xExtension);

		if(xExtension.Compare(ASURA_STANDARD_FILE_EXTENSION_LE) && xExtension.Compare(ASURA_STANDARD_FILE_EXTENSION_BE)) continue;

		sprintf(szFilename,"Sounds\\IniFiles\\%s.ini",xName.GetString());

		AddSoundsFromIniFile(szFilename,m_pxSoundEnum,m_xSoundList);
	}

	#if 1
	//add sounds for files in the seperately loaded file list
	if(ShouldAddSoundsForIgnoredAsuraFiles())
	{
		for(CLIF<Asura_String> xNameLIF(&m_xSeperatelyLoadedFiles);!xNameLIF.done();xNameLIF.next())
		{
			//build the name of the inifile to look at
			Asura_String xName;
			Asura_String xExtension;
			xNameLIF().SplitFilename(0,&xName,&xExtension);

			if(xExtension.Compare(ASURA_STANDARD_FILE_EXTENSION_LE) && xExtension.Compare(ASURA_STANDARD_FILE_EXTENSION_BE)) continue;
			
			sprintf(szFilename,"Sounds\\IniFiles\\%s.ini",xName.GetString());

			AddSoundsFromIniFile(szFilename,m_pxSoundEnum,m_xSoundList);
		}
		for(CLIF<Asura_String> xNameLIF(&m_xFilesToIgnore);!xNameLIF.done();xNameLIF.next())
		{
			//build the name of the inifile to look at
			Asura_String xName;
			Asura_String xExtension;
			xNameLIF().SplitFilename(0,&xName,&xExtension);

			if(xExtension.Compare(ASURA_STANDARD_FILE_EXTENSION_LE) && xExtension.Compare(ASURA_STANDARD_FILE_EXTENSION_BE)) continue;
			
			sprintf(szFilename,"Sounds\\IniFiles\\%s.ini",xName.GetString());

			AddSoundsFromIniFile(szFilename,m_pxSoundEnum,m_xSoundList);
		}
	}
	#endif

	//also check for ini files for any shapes
	List<Asura_Mesh*> xMeshList;
	for(int iCommon = 0;iCommon<2;iCommon++)
	{
		if((iCommon==0) && !m_bIncludeEntityShapes) continue;
		if((iCommon==1) && !m_bIncludeCommonShapes) continue;

		int iNumMeshes = iCommon ? Tools_ShapeTable::GetNumExportShapes() : Tools_ShapeTable::GetNumElements();

		for(int i=0;i<iNumMeshes;i++)
		{
			//ignore shapes that aren't going to be exported
			if(!Tools_ShapeTable::ShouldExportShapeByIndex(i,iCommon ? true : false)) continue;

			const char* pszName = Tools_ShapeTable::GetMeshNameByIndex(i,iCommon ? true : false);	
			if(!pszName) continue;
			sprintf(szFilename,"Sounds\\IniFiles\\%s.ini",pszName);

			AddSoundsFromIniFile(szFilename,m_pxSoundEnum,m_xSoundList);
		}
	}
}

void Tools_Level_Export::AddNonEnvironmentalSounds(HANDLE pSaveHandle)
{
	UpdateStatus( EXPORT_STAGE_NON_ENVIRONMENTAL_SOUNDS, "Non-Environmental Sounds", -1 );

	int i=0;
	for(LIF<SoundExportData*> xSoundLIF(&m_xSoundList);!xSoundLIF.done() && !IsAbortFlagSet();xSoundLIF.next(),i++)
	{
		s_iPercent = i*100/m_xSoundList.size();
		UpdateProgress();
		AddSound(pSaveHandle,xSoundLIF());
	}

	UpdateStatus( EXPORT_STAGE_END, 0, -1 );
}

bool Tools_Level_Export::AddSoundsFromIniFile(const char* pszFilename,Tools_EnumerationReader* pxSoundEnum,List<SoundExportData*>& xSoundList,bool (*pfnShouldExport)(int iSoundID) /*= 0*/)
{
	Tools_IniFile xSoundIni(pszFilename);
	if(!xSoundIni.LoadIniFile()) return false;
	AddMessage(MT_LOGONLY, "Loaded %s",pszFilename);

	List<char*> xSectionList = xSoundIni.GetSectionNameList();
	for(LIF<char*> xSectionLIF(&xSectionList);!xSectionLIF.done();xSectionLIF.next())
	{
		//Check for sound events section
		if(!stricmp(xSectionLIF(),"Sound Events"))
		{
			if(IsDoingFakeLevelExport()) continue;
			
			//Each line should correspond to a sound event to be added			
			xSoundIni.GetIniTextSection(xSectionLIF());
			while(xSoundIni.GetNextIniTextLine())
			{
				Asura_String xEventName;
				if(xSoundIni.GetString(xEventName))
				{
					//make sure the event exists
					Asura_Hash_ID uEventHash = Asura_GetHashID(xEventName.GetString());
					if(Asura_Sound_Event_Storage::FindEvent(uEventHash))
					{
						AddSoundEventToExportList(uEventHash, xEventName.GetString());
					}
					else
					{
						AddMessage(MT_ERROR, "Failed to find sound event '%s' (referred to by '%s')", xEventName.GetString(), pszFilename);
					}
				}
			}
			continue;
		}

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		//Check for dialogue events section
		if(!stricmp(xSectionLIF(),"Dialogue Events"))
		{
			if(IsDoingFakeLevelExport()) continue;

			//Each line should correspond to a dialogue event to be added			
			xSoundIni.GetIniTextSection(xSectionLIF());
			while(xSoundIni.GetNextIniTextLine())
			{
				Asura_String xEventName;
				if(xSoundIni.GetString(xEventName))
				{
					//make sure the event exists
					Asura_Hash_ID uEventHash = Asura_GetHashID(xEventName.GetString());
					if(Asura_Dialogue_Storage::FindEvent(uEventHash))
					{
						AddDialogueEventToExportList(uEventHash);
					}
					else
					{
						AddMessage(MT_ERROR, "Failed to find dialgue event '%s' (referred to by '%s')", xEventName.GetString(), pszFilename);
					}
				}
			}
			continue;
		}
		#endif //ASURA_USE_DIALOGUE_SYSTEM


		
		//translate the enumeration value
		int iValue;
		if(!pxSoundEnum->GetEnumerationValue(xSectionLIF(),iValue))
		{
			AddMessage(MT_ERROR, "Failed to find '%s' in the sound enumeration",xSectionLIF() );
			continue;
		}

		if(pfnShouldExport)
		{
			//check callback to see if this sound should be included. RAR 29/03/2006
			if(!pfnShouldExport(iValue)) continue;

		}

		//make sure we don't already have an entry for this ID
		bool bExists = false;
		for(LIF<SoundExportData*> xSoundLIF(&xSoundList);!xSoundLIF.done();xSoundLIF.next())
		{
			if(xSoundLIF()->m_iID == iValue)
			{
				bExists = true;
				break;
			}
		}
		if(bExists) continue;

		//get the filename
		xSoundIni.GetIniTextSection(xSectionLIF());
		Asura_String xFilename;
		if(!xSoundIni.GetStringInCurrentSection("data",xFilename))
		{
	        AddMessage(MT_ERROR, "Failed to find 'data' entry for %s in %s",xSectionLIF(),pszFilename );
			continue;
		}

		SoundExportData* pxSound = new SoundExportData;
		pxSound->m_iID = iValue;
		pxSound->m_iFrequency = 0;

		xSoundIni.GetIntInCurrentSection("ps2_frequency",pxSound->m_iFrequency);
		xSoundIni.GetFloatInCurrentSection("inner_radius",pxSound->m_fDefaultInnerRadius);
		xSoundIni.GetFloatInCurrentSection("outer_radius",pxSound->m_fDefaultOuterRadius);
		xSoundIni.GetFloatInCurrentSection("pitch",pxSound->m_fPitchScale);
		xSoundIni.GetFloatInCurrentSection("volume",pxSound->m_fVolumeScale);
	
		#if 1
		if(strnicmp(xFilename.GetString(),"sounds\\",7))
		{
			pxSound->m_xFilename = "sounds\\";
			pxSound->m_xFilename += xFilename;
		}
		else
		#endif
		{
			pxSound->m_xFilename = xFilename;
		}

		xSoundList.add_entry(pxSound);
	}
	return true;
}

void Tools_Level_Export::AddGameSoundsIniFile(const char* pszFilename)
{
	if(!LoadSoundIDEnumeration())
	{
		return;
	}

	if(!AddSoundsFromIniFile(pszFilename,m_pxSoundEnum,m_xExportedSoundList))
	{
		
        AddMessage(MT_ERROR, "Failed to load sound ini file '%s'",pszFilename );
		Asura_Assertf(0,("Failed to load sound ini file '%s'",pszFilename));
	}
} 


void Tools_Level_Export::LoadIniFileSounds()
{
	List<SoundExportData*> xSoundList;

	//load the sound enumeration header file
	Tools_EnumerationReader xSoundEnum;
	if(!xSoundEnum.ReadEnumeration("ToolsIniFiles\\Asura_ProjectSoundIDs.h","enum Asura_Sound_ID"))
	{
		return;
	}
	char szFilename[MAX_PATH];

	List<Asura_Hash_ID>	xDoneList;

	//add sounds for skins
	for(int i=0;i<Asura_Animation_System::GetNumberOfSkins();i++)
	{
		Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(i);
		sprintf(szFilename,"Sounds\\IniFiles\\%s.ini",pxSkin->Name.GetString());
		xDoneList.add_entry(Asura_GetHashID(pxSkin->Name.GetString()));

		AddSoundsFromIniFile(szFilename,&xSoundEnum,xSoundList);
	}
	
	//add sounds for skins that will be loaded by the cutscenes (these probably haven't been loaded yet)
	for(int i=0;i<Asura_Cutscene_System::GetNumberOfCutscenes();i++)
	{
		Asura_Cutscene* pxCutscene = Asura_Cutscene_System::GetCutsceneByIndex(i);
		Asura_NameList* pxNames =  &pxCutscene->SkinNames;
		for(int j=0;j<pxNames->GetNumEntries();j++)
		{
			Asura_String& xName = pxNames->GetName(j);
			if(xName.IsBlank()) continue;

			//already done this skin?
			Asura_Hash_ID uSkinHash = Asura_GetHashID(xName.GetString());
			if(xDoneList.contains(uSkinHash)) continue;
			xDoneList.add_entry(uSkinHash);

			sprintf(szFilename,"Sounds\\IniFiles\\%s.ini",xName.GetString());
			AddSoundsFromIniFile(szFilename,&xSoundEnum,xSoundList);
			
		}
	}


	
	//add sounds for shapes
	List<Asura_Mesh*> xMeshList;
	for(int iCommon = 0;iCommon<2;iCommon++)
	{
		if((iCommon==0) && !This->m_bIncludeEntityShapes) continue;
		if((iCommon==1) && !This->m_bIncludeCommonShapes) continue;
		
		int iNumMeshes = iCommon ? Tools_ShapeTable::GetNumExportShapes() : Tools_ShapeTable::GetNumElements();

		for(int i=0;i<iNumMeshes;i++)
		{
			//ignore shapes that aren't going to be exported
			if(!Tools_ShapeTable::ShouldExportShapeByIndex(i,iCommon ? true : false)) continue;

			const char* pszName = Tools_ShapeTable::GetMeshNameByIndex(i,iCommon ? true : false);	
			if(!pszName) continue;
			sprintf(szFilename,"Sounds\\IniFiles\\%s.ini",pszName);

			AddSoundsFromIniFile(szFilename,&xSoundEnum,xSoundList);

		}
	}
	

	//add sounds for this level
	{
		extern Asura_String EnvironmentFilename;
		Asura_String xName;
		EnvironmentFilename.SplitFilename(0,&xName,0);

		sprintf(szFilename,"Sounds\\IniFiles\\%s.ini",xName.GetString());
		AddSoundsFromIniFile(szFilename,&xSoundEnum,xSoundList);

	}

	#ifndef ASURA_USE_AUDIO_SYSTEM
	//now load all the listed sounds
	while(xSoundList.size())
	{
		SoundExportData* pxData = xSoundList.first_entry();
		xSoundList.delete_first_entry();

		Asura_Sound_System::LoadSound(pxData->m_xFilename.GetString(),(Asura_Sound_ID)pxData->m_iID);


		delete pxData;
	}
	#else
	Asura_Assert( xSoundList.size() == 0, "Expected sound list size to be zero here" );
	#endif

}

void Tools_Level_Export::AddSoundMappingResource(HANDLE hFile)
{
	if(!m_xSoundMappingList.size()) return;

	//create the resource chunk
	const char* pszResourceName = "Sound Mappings";
	int iSize = Tools_SoundResource::GetSoundMappingResourceSize(pszResourceName,m_xSoundMappingList);

	char* pcBuffer = new char[iSize];
	Tools_SoundResource::CreateSoundMappingResource(pszResourceName,m_xSoundMappingList,pcBuffer);

	//add to the file
	unsigned long uBytesWritten = 0;
	WriteFile(hFile,pcBuffer,iSize,&uBytesWritten,0);

	//clean up
	delete [] pcBuffer;
	while(m_xSoundMappingList.size()) m_xSoundMappingList.delete_first_entry();
}

void Tools_Level_Export::AddSoundPropertiesResource(HANDLE hFile)
{
	if(!m_xSoundPropertiesList.size()) return;

	//create the resource chunk
	const char* pszResourceName = "Sound Properties";
	int iSize = Tools_SoundResource::GetSoundPropertiesResourceSize(pszResourceName,m_xSoundPropertiesList);

	char* pcBuffer = new char[iSize];
	Tools_SoundResource::CreateSoundPropertiesResource(pszResourceName,m_xSoundPropertiesList,pcBuffer);

	//add to the file
	unsigned long uBytesWritten = 0;
	WriteFile(hFile,pcBuffer,iSize,&uBytesWritten,0);

	//clean up
	delete [] pcBuffer;
	while(m_xSoundPropertiesList.size()) m_xSoundPropertiesList.delete_first_entry();
}

int Tools_Level_Export::GetDefaultMaxSoundFrequency( const char* szWavFilename )
{
    if(s_eExportType == Export_PS2)
	    return TOOLS_DEFAULT_MAX_PS2_FREQUENCY;
	else
		return 22050;
}

#if defined ASURA_USE_REVERBVOLUME_SYSTEM && defined ASURA_USE_AUDIO_SYSTEM

bool Tools_Level_Export::AddReverbs( HANDLE pSaveHandle )
{
	Asura_HashTree xReverbsInLevel;

	// Gather a list of reverbs used by volumes in the level
	{
		const Asura_ReverbVolume_Controller* const pxController =
			Asura_ReverbVolume_System::GetReverbVolumeController();

		if( !pxController ) return false;

		const int iNumZones = pxController->GetNumberOfZones();

		for( int i = 0; i < iNumZones; ++i )
		{
			const Asura_ReverbVolume* const pxVolume =
				static_cast< const Asura_ReverbVolume* >( pxController->GetZoneByIndex(i) );

			if( !pxVolume ) return false;

			int iDummy;
			xReverbsInLevel.FindNodeIndexByHash( pxVolume->GetReverbType(), true, iDummy );
		}
	}

	// Write them out to chunk
	{
		Asura_SaveBlock xSaveBlock;
		Asura_Chunk_SaveBlock_Stream xStream( &xSaveBlock );

		Asura_Chunk_ReverbPresets::Create( xStream, xReverbsInLevel );

		return (xSaveBlock.WriteToFile( pSaveHandle ) != -1);
	}
}

#endif // defined ASURA_USE_REVERBVOLUME_SYSTEM && defined ASURA_USE_AUDIO_SYSTEM

#ifdef ASURA_USE_STREAMING
void Tools_Level_Export::AddGlobalMetaMusicToExportList()
{
	

	Asura_MetaMusic_Handler* pxHandler = Tools_Mode_MetaMusic::GetGlobalHandler();
	if(pxHandler->GetNumberOfTracks())
	{
		//save the global meta music tracks to a temporary file to do the endian swapping (if required)
		char szFilename[100];
		Asura_PrintfToBfr(szFilename,"%s\\GlobalMetaMusic.%s",sizeof(szFilename),true,GetTempDirectory(),ShouldExportBigEndian() ? ASURA_STANDARD_FILE_EXTENSION_BE : ASURA_STANDARD_FILE_EXTENSION_LE);

		Asura_SaveBlock xSaveBlock;
		Asura_Chunk_MetaMusic::Create(&xSaveBlock,pxHandler);
		xSaveBlock.AddAsuraTerminator();
		xSaveBlock.SaveFile(szFilename,false);

		//add the temp file to the export
		AddFileToExportList(szFilename,false);

	}
}

#endif //ASURA_USE_STREAMING

void Tools_Level_Export::AddLocalisedSounds(HANDLE pSaveHandle,int iLanguage)
{
	UpdateStatus(EXPORT_STAGE_LOCALISED_SOUNDS_NON_STREAMED, "Localised Sounds (non-streamed)", 0);

	int iCount = 0;
	for(LIF<const SoundExportData*> xSoundLIF(&m_xLocalisedSoundList); !xSoundLIF.done(); xSoundLIF.next(),iCount++)
	{
		s_iPercent = iCount*100/m_xLocalisedSoundList.size();
		UpdateProgress();

		if (xSoundLIF()->m_bStreamed)
		{
			m_xLocalisedStreamedSoundList.add_entry(xSoundLIF());
		}
		else
		{
			AddSound(pSaveHandle, xSoundLIF(), iLanguage);
		}
	}

	UpdateStatus(EXPORT_STAGE_END, 0, -1);
}

void Tools_Level_Export::AddLocalisedStreamedSounds(HANDLE pSaveHandle,int iLanguage)
{
	UpdateStatus(EXPORT_STAGE_LOCALISED_SOUNDS_STREAMED, "Localised Sounds (streamed)", 0);

	Asura_Assert(!m_bAddingStreamedSounds, "already adding streamed sounds");
	Asura_Assert(!m_xLocalisedStreamedSoundExportList.size(), "sound export list not empty");

	int iPercentageForAddingSounds = 100;
	if (s_eExportType == Export_PS2)
	{
		// first 90% is converting all the sounds, 10% for adding them to the file.
		iPercentageForAddingSounds = 90;
	}

	m_bAddingStreamedSounds = true;

	#ifdef ASURA_USE_AUDIO_SYSTEM

		// Convert into list of Tools_StreamingSoundExportInfo
		Asura_List<Tools_StreamingSoundExportInfo> xExportList;
		const Asura_LocalisedText_Language eLanguage = static_cast<Asura_LocalisedText_Language>(iLanguage);

		int iCount = 0;
		for(LIF<const SoundExportData*> xSoundLIF(&m_xLocalisedStreamedSoundList); !xSoundLIF.done() && !IsAbortFlagSet(); xSoundLIF.next(),iCount++)
		{
			Tools_StreamingSoundExportInfo* const pxInfo = new Tools_StreamingSoundExportInfo;
			Asura_Assert(pxInfo, "Failed to allocate streaming sound export info.");

			if(pxInfo)
			{
				const char* const szEnglishName = xSoundLIF()->m_xFilename.GetString();
				char szLocalisedName[MAX_PATH];

				LocaliseSoundFilename(szLocalisedName, MAX_PATH, szEnglishName, eLanguage);

				pxInfo->m_xExportName = szEnglishName;
				pxInfo->m_xFilename = szLocalisedName;
				pxInfo->m_bLooping = false;

				xExportList.AddToList(pxInfo);
			}
		}

		Asura_Chunk_Stream_WindowsFile xStream(pSaveHandle);
		CreateStreamingSoundsChunk(xStream, xExportList, true);

		// Asura_List doesn't own its contents, so we have to delete them manually
		for(Asura_List_It<Tools_StreamingSoundExportInfo> xIter(xExportList); !xIter.Done();)
		{
			// Delete the export info struct
			delete xIter.GetCurrent();

			// Delete the node itself and advance to next
			xIter.DeleteCurrent();
		}

	#else

		int iCount = 0;
		for(LIF<const SoundExportData*> xSoundLIF(&m_xLocalisedStreamedSoundList); !xSoundLIF.done() && !IsAbortFlagSet(); xSoundLIF.next(),iCount++)
		{
			s_iPercent = iCount*iPercentageForAddingSounds/m_xLocalisedStreamedSoundList.size();
			UpdateProgress();

			Asura_Assert(xSoundLIF()->m_bStreamed, "expected streamed sound");
			AddSound(pSaveHandle, xSoundLIF(), iLanguage);
		}

		// for streaming platforms, we've added all sounds to a list and converted them, now add to the file
		if (TargetPlatformSupportsStreamedSounds() && !IsAbortFlagSet())
		{
			AddStreamedSoundsFromList(pSaveHandle);
		}

	#endif

	UpdateStatus(EXPORT_STAGE_END, 0, -1);

	m_bAddingStreamedSounds = false;
	// clear out the streaming export list.
	while(m_xLocalisedStreamedSoundExportList.size()) 
	{
		delete m_xLocalisedStreamedSoundExportList.first_entry();
		m_xLocalisedStreamedSoundExportList.delete_first_entry();
	}
}


/// End of file. ///