//
// Tools_CommonFile_Export.cpp
//
// Created by Richard Rice - 12/12/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

//asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"

//tools
#include "Tools_System.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Core/Asura_TextureManagement.h"
#include "Physics/Asura_Physics_Ragdoll.h"
#include "ChunkSystem/Asura_Chunk_RagdollDescription.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
#include "Shapes/Tools_Shapes.h"
#endif //ASURA_USING_ALL_LEVEL_EXPORT

#ifdef ASURA_USE_DECAL_MANAGER
#include "SpecialFX/Asura_Decal_Manager.h"
#include "ChunkSystem/Asura_Chunk_Decals.h"
#endif


void Tools_Level_Export::ExportCommonFile()
{
	if(!This) return;

	//ask which platform to export for
	ListSelection_Overlay xOverlay("Export common file for :");

	xOverlay.AddItem("*All*",-1);

	if(!This->OnlyAllowCommonExportForAllPlatforms())
	{
		for(int iPlatform=0;iPlatform<ExportType_NumPlatforms;iPlatform++)
		{
			EXPORT_TYPE ePlatform = (EXPORT_TYPE) iPlatform;
			if(!This->IsPlatformAllowed(ePlatform)) continue;

			xOverlay.AddItem(GetPlatformName(iPlatform),iPlatform);
		}
	}

	if(!xOverlay.DoModal()) return;

	//export the selected platform(s)
	int iSelectedPlatform = xOverlay.GetSelectionData();

	//If doing all platforms , then check to make sure that they are all writeable before starting
	//RAR 12/06/2007
	if(iSelectedPlatform == -1)
	{
		bool bAbort = false;
		for(int iPlatform=0;iPlatform<ExportType_NumPlatforms;iPlatform++)
		{
			EXPORT_TYPE ePlatform = (EXPORT_TYPE) iPlatform;
			if(!This->IsPlatformAllowed(ePlatform)) continue;

			s_eExportType = ePlatform;
			Asura_String xExportName;
			This->GetCommonExportFilename(xExportName );

			if(!This->IsWriteableIncludingLanguageFiles(xExportName.GetString(),true))
			{
				bAbort = true;
			}

		}
		s_eExportType = Export_PC;


		if(bAbort)
		{
			AddMessage(MT_ERROR,"Aborting export.");
			return;
		}
	}


	for(int iPlatform=0;iPlatform<ExportType_NumPlatforms;iPlatform++)
	{
		if(iSelectedPlatform == iPlatform || iSelectedPlatform == -1)
		{
			EXPORT_TYPE ePlatform = (EXPORT_TYPE) iPlatform;
			if(!This->IsPlatformAllowed(ePlatform)) continue;

			s_eExportType = ePlatform;
			This->ExportCommonFileMain();
		}
	}
	s_eExportType = Export_PC;
}

void Tools_Level_Export::ExportCommonFile(int iPlatform)
{
	Asura_Assert(This,"");

	s_eExportType = (EXPORT_TYPE) iPlatform;
	This->ExportCommonFileMain();
	s_eExportType = Export_PC;
}

void Tools_Level_Export::StartCommonFileExport()
{
    s_bIsCommonFileExport = true;
	SetMainHeading("Common File Export");
}

void Tools_Level_Export::EndCommonFileExport()
{
	DeleteTemporaryFiles();
    s_bIsCommonFileExport = false;
}

void Tools_Level_Export::GetCommonExportFilename(Asura_String& xOutputFilename)
{
	char acFilename[MAX_PATH];

	const char* szName = "Common";

	const char* szPlatformName = GetPlatformName(s_eExportType);
	if (szPlatformName)
	{
		Asura_PrintfToBfr(acFilename, "..\\%s_Release\\misc\\%s.asr%s", MAX_PATH, true, 
			szPlatformName, szName, Tools_Platform::IsBigEndian(s_eExportType) ? ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX : "");
		xOutputFilename = acFilename;
	}
	else
	{
		xOutputFilename = szName;
		xOutputFilename += ".unknown_platform";
	}
}



void Tools_Level_Export::ExportCommonFileMain()
{
	const bool	bExportGlobalSounds	= Tools_System::This->CanExportGlobalSoundsInCommon();

	//make sure destination file is writeable
	Asura_String xExportName;
	GetCommonExportFilename(xExportName );

	if(!IsWriteableIncludingLanguageFiles(xExportName.GetString(),true))
	{
		return;
	}

	if(bExportGlobalSounds)
	{
		StartGlobalSoundEventExport();
	}
	StartExport();
	OpenExportLog("Common");
	StartCommonFileExport();
	StartFileExport();

	AddRagdollDescriptionsToExportList();

	//Default to not exporting shapes at the moment
	//RAR 12/12/2006
	SetIncludeEntityShapes(false);
#ifndef ASURA_USING_ALL_LEVEL_EXPORT
	SetIncludeCommonShapes(false);
#else
	// Allow shapes to be added to the common export, but only those specified in the common_export.txt
	// DF 19/04/07
	SetIncludeCommonShapes(true);
	Tools_ShapeTable::ExcludeNonCommonShapesFromCommonExport();
#endif //ASURA_USING_ALL_LEVEL_EXPORT

	if(!IsAbortFlagSet())
	{
		//make sure shapes are flagged as needed for export
		//RAR - CHECK - Update project code
		FlagShapeTableEntriesForExport();
	}

	if(!IsAbortFlagSet())
	{
		//Get files to include in export
		AddCommonExportFilesToList(m_xAsuraFileList);

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
		//Add files from the Common_Export.txt file to the common export list
		// DF - 19/04/07
		AddAsuraFilesFromTextFile(GetCommonExportFile(), false);
#endif
 		UpdateStatus( EXPORT_STAGE_START_EXPORT, "Beginning export...", -1 );
		ExportFilesInList( xExportName.GetString(),false);

	}

	//end everything
	EndFileExport();
	EndCommonFileExport();

	if(bExportGlobalSounds)
	{
		EndGlobalSoundEventExport();
	}

	CloseExportLog();
	EndExport(false);

}

void Tools_Level_Export::AddRagdollDescriptionsToExportList()
{
#ifndef ASURA_PHYSICS_DISABLE_RAGDOLL
	Asura_String xFilename = Tools_Level_Export::GetTempDirectory();
	xFilename += "\\RagdollDescriptions.asr";

	if (Tools_Level_Export::ShouldExportBigEndian())
	{
		xFilename += ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
	}

	Asura_SaveBlock xSaveBlock;
	Asura_Chunk_SaveBlock_Stream xStream(&xSaveBlock);

	const int iNumberOfRagdollDescriptions = Asura_Physics_Ragdoll_System::GetNumRagdollDescriptions();
	const char** pszRagdollDescriptionNames = new const char*[iNumberOfRagdollDescriptions];
	Asura_Physics_Ragdoll_System::GetRagdollDescriptionNames( pszRagdollDescriptionNames );
	for (int i = 0; i < iNumberOfRagdollDescriptions; i++)
	{
		const Asura_String xRagdollName( pszRagdollDescriptionNames[i] );
		Asura_Physics_Ragdoll_Description* pxRagdollDescription = Asura_Physics_Ragdoll_System::GetRagdollDescription( xRagdollName );
		Asura_Chunk_RagdollDescription::Create(xStream, *pxRagdollDescription);
	}
	delete[] pszRagdollDescriptionNames;

	xSaveBlock.AddAsuraTerminator();

	const bool bSaved = xSaveBlock.SaveFile(xFilename.GetString(), false);
	Asura_Assert(bSaved, "Couldn't save Ragdoll descriptions temporary file");
	if (bSaved)
	{
		Tools_Level_Export::AddFileToExportList(xFilename.GetString(), false);
	}
#endif // #ifndef ASURA_PHYSICS_DISABLE_RAGDOLL
}

void Tools_Level_Export::AddCommonExportFilesToList(List<Asura_String>& xNameList)
{
	//Add all blueprints
	for ( Asura_Blueprint_Type_ConstIterator xBPTypeIter; !xBPTypeIter.Done(); xBPTypeIter.Next() )
	{
		AddBlueprintTypeToExportList( xBPTypeIter.GetCurrentHash() );
	}

#ifdef ASURA_USE_AXON
	//Add all animation templates
	AddAnimTemplatesToExportList();
#endif

#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
	AddMaterialResponseOverridesToExportList();
#endif	// defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
	AddPFXOverridesToExportList();
	AddFSFXOverridesToExportList();


#ifdef ASURA_USE_ANIMATION_IK_SOLVER
	AddIKTemplatesToExport(true);
#endif

#ifdef ASURA_USE_DECAL_MANAGER
	if (!IsDoingFakeLevelExport())
	{
		// Note: *All* decal textures are always exported if we're exporting to the common .asr.
		//
		// If we export to each level instead, we have the option to remove the ones that aren't needed for that level
		// (although that isn't yet implemented)
		
#if !defined(ASURA_TARGET_SM3)
		// Add the decal textures
		for(int i=0; i<Asura_Decal_Manager::GetNumDecalMaterials();i++)
		{
			Asura_DecalManager_Material *pxMtl = Asura_Decal_Manager::GetDecalMaterialByIndex(i);

#ifdef ASURA_DECAL_MANAGER_USE_COMMON_ASR
			bool bExportTextures = !pxMtl->m_bExportAsStaticDecal;
#else	
			bool bExportTextures = false;
#endif			

			if (bExportTextures)
			{			
				// Texture
				int iTex = Asura_Texture_Management::GetIndexFromHashID(Asura_GetHashID_IgnoreExtension(pxMtl->m_xTextureName.GetString()));
				if (iTex!=ASURA_TEXTURE_NONE)
				{	
					Tools_Level_Export::AddTextureToExportList(iTex);
				}

			}
		}
#endif

#ifdef ASURA_DECAL_MANAGER_USE_COMMON_ASR
		// And the decal definitions
		Tools_FileSystem::EnsureDirectoryExists(Tools_Level_Export::GetTempDirectory());
		Asura_String xExportFileName = Tools_Level_Export::GetTempDirectory();
		xExportFileName += "\\decals.asr";
		if (Tools_Level_Export::ShouldExportBigEndian())
		{		
			xExportFileName += ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX;
		}
		Asura_Chunk_Decals::Save(xExportFileName.GetString());
		AddFileToExportList(xNameList, xExportFileName, false);
#endif
	}
#endif

}


/// End of file. ///
