//
// Tools_ModeDataFile.cpp
//
// Created by Kevin Lea - 05/10/2001 16:51:47
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Load&Save/Tools_BatchProcessing.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_OptionsFile.h" //for directory
#include "PC/FileSystem/Tools_FileSystem.h"


Tools_ModeDataFile::Tools_ModeDataFile( const char* szFilename, int iMode, int iCurrentVersion, const char* szPath ) 
:	m_iMode( iMode ),
	m_iCurrentVersion( iCurrentVersion ),
	m_xFileHandle( NULL ),
    m_uLoadFileSize( 0 ),
	m_bHadAnError( false ),
	m_pvBuffer( NULL ),
	m_uBufferSize( 0 )
{
	//get the full filename
	char aFull_filename[ MAX_PATH ];
	if ( szPath )
	{
		sprintf( aFull_filename, "%s\\%s", szPath, szFilename );
	}
	else
	{
		sprintf( aFull_filename, "%s\\%s", ToolsIniDataDirectory, szFilename);
	}
	
	if( m_iMode == TOOLSMODEDATAFILE_LOAD )
	{
		m_xFileHandle = CreateFile( aFull_filename, GENERIC_READ, 0, 0, OPEN_EXISTING, 0, 0 );
		//load version number , and compare with loader version number
		if( !Transfer( m_iCurrentVersion ) || iCurrentVersion < m_iCurrentVersion )
		{
			m_bHadAnError = true;
			return;
		}
        m_uLoadFileSize = GetFileSize( m_xFileHandle, NULL );
	}
	else
	{
		if( true == Tools_BatchProcessing::IsBatch())
		{
			m_bHadAnError = true;
			return;
		}
		
		//ensure directory exists
		if ( szPath )
		{
			CreateDirectory( szPath, 0 );
		}
		else
		{
			CreateDirectory( ToolsIniDataDirectory, 0 );
		}

		//force any existing file to be writeable to cope with evil artists that
		//add this directory to alienbrain
		Tools_FileSystem::MakeWriteable( aFull_filename ); 

		m_xFileHandle = CreateFile( aFull_filename, GENERIC_WRITE, 0, 0, CREATE_ALWAYS, 0, 0);
		Transfer( m_iCurrentVersion );
	}		
}

Tools_ModeDataFile::~Tools_ModeDataFile()
{
	//close the file
	if( m_xFileHandle != INVALID_HANDLE_VALUE )
	{
		CloseHandle( m_xFileHandle );
	}

	delete [] m_pvBuffer;
}

bool Tools_ModeDataFile::Transfer( void* vData, unsigned uData_size )
{
	unsigned long uBytes_transferred;
	if ( m_xFileHandle == INVALID_HANDLE_VALUE ) return false;
	if ( m_bHadAnError ) return false;
	
	if( m_iMode == TOOLSMODEDATAFILE_LOAD )
	{
		//read data into a temporary buffer to avoid overwriting the Buffer passed
		//on a failed read
		if( uData_size > m_uBufferSize )
		{
			m_uBufferSize = max( uData_size, 100 );
			delete [] m_pvBuffer;
			m_pvBuffer = new unsigned char[ m_uBufferSize ];
		}
		
		ReadFile( m_xFileHandle, m_pvBuffer, uData_size, &uBytes_transferred, 0 );
		if ( uBytes_transferred == uData_size )
		{
			memcpy( vData, m_pvBuffer, uData_size );
		}
	}
	else
	{
		Asura_Assert( false == Tools_BatchProcessing::IsBatch(), "This should not be called in batch mode.");
		WriteFile( m_xFileHandle, vData, uData_size, &uBytes_transferred, 0 );
	}

	if( uBytes_transferred != uData_size ) 
	{
		m_bHadAnError = true;
	}

	return ( !m_bHadAnError );
}
