//
// Tools_TextureConversionData.cpp
//
// Created by Richard Rice - 26/10/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "Load&Save/Tools_TextureConversionData.h"

#include "Core/Asura_TextureFlags.h"
#include "Core/Memory/Asura_Mem.h"
#include "Debugging/Asura_Assert.h"


static u_int g_uFixedPaletteSize = 0;
static u_int g_auFixedPalette[256];

void TextureConversionDataBase::SetFixedPalette(u_int uSize,const u_int* puPalette)
{
	if(uSize > 256)
	{
		Asura_Assert(0,"Fixed palette shouldn't contain more than 256 entires");
		uSize = 256;
	}

	g_uFixedPaletteSize = uSize;
	Asura_Mem::Copy(g_auFixedPalette,puPalette,sizeof(u_int)*uSize);

}

void TextureConversionDataBase::GetFixedPalette(u_int& uSize, const u_int*& puPalette)
{
	uSize = g_uFixedPaletteSize;
	puPalette = g_auFixedPalette;
}


void TextureConversionData::Initialise( Tools_Level_Export::EXPORT_TYPE eExportType )
{
	Asura_Mem::Set(this,0,sizeof(TextureConversionData));
	m_usProjectVersion = TOOLS_PROJECT_TEXTURE_CONVERSION_DATA_VERSION;
	m_ucVersion			= TEXTURE_CONVERSION_DATA_VERSION;
	switch( eExportType )
	{
	case Tools_Level_Export::Export_PC:
		m_ucPlatformVersion = TOOLS_PLATFORM_PC_TEXTURE_CONVERSION_DATA_VERSION;
		break;
	case Tools_Level_Export::Export_PS2:
		m_ucPlatformVersion = TOOLS_PLATFORM_PS2_TEXTURE_CONVERSION_DATA_VERSION;
		break;
	case Tools_Level_Export::Export_PSP:
		m_ucPlatformVersion = TOOLS_PLATFORM_PSP_TEXTURE_CONVERSION_DATA_VERSION;
		break;
	case Tools_Level_Export::Export_PS3:
		m_ucPlatformVersion = TOOLS_PLATFORM_PS3_TEXTURE_CONVERSION_DATA_VERSION;
		break;
	case Tools_Level_Export::Export_Xbox360:
		m_ucPlatformVersion = TOOLS_PLATFORM_Xbox360_TEXTURE_CONVERSION_DATA_VERSION;
		break;
	case Tools_Level_Export::Export_Wii:
		m_ucPlatformVersion = TOOLS_PLATFORM_Wii_TEXTURE_CONVERSION_DATA_VERSION;
		break;
	default:
		Asura_Assert( false, "Invalid export type. Now set default as TOOLS_PLATFORM_PC_TEXTURE_CONVERSION_DATA_VERSION" );
		m_ucPlatformVersion	= TOOLS_PLATFORM_PC_TEXTURE_CONVERSION_DATA_VERSION;
		break;
	}
}

void TextureConversionData::ClearUnusedEntriesForPlatform(Tools_Level_Export::EXPORT_TYPE eExportType)
{
	//determine which texture and material flags are important
    int iTextureFlagsToCheck = AsuraTextureFlag_8BitAlpha
  	                           | AsuraTextureFlag_PS2MoireFix
  	                           | AsuraTextureFlag_NoMipMaps
  	                           | AsuraTextureFlag_PS2DistantTexture
  	                           | AsuraTextureFlag_PS2Allow512
  	                           | AsuraTextureFlag_PS2AllowMipMapsOn512
  	                           | AsuraTextureFlag_PS2DontHalveSize
  	                           | AsuraTextureFlag_DontPremultiplyAlphaInPaletteGeneration
  	                           | AsuraTextureFlag_ReduceExtraForConsoles
							   | AsuraTextureFlag_PS2DontHalveExtraForMultiplayer
							   | AsuraTextureFlag_HighQualityCompression
							   | AsuraTextureFlag_SharpenMipMaps;



    u_int uMaterialFlagsToCheck = AsuraMaterialFlag_SelfIlluminate
                                | AsuraMaterialFlag_Transparent
                                | AsuraMaterialFlag_PS2AdditiveAlpha;

	if(eExportType == Tools_Level_Export::Export_PC || eExportType == Tools_Level_Export::Export_Xbox360)
	{
		//Most of the flags don't matter for PC export. RAR 03/12/2003
		iTextureFlagsToCheck = AsuraTextureFlag_NoMipMaps|AsuraTextureFlag_HighQualityCompression|AsuraTextureFlag_SharpenMipMaps;
		if( eExportType == Tools_Level_Export::Export_Xbox360 )
		{
			// Do check size reduction on consoles
			iTextureFlagsToCheck |= AsuraTextureFlag_ReduceExtraForConsoles;
		}

		//and similarly for the material flags.
		uMaterialFlagsToCheck = 0;
	                 
	}
	if(eExportType == Tools_Level_Export::Export_PS2)
	{
		//ignore alpha channel just used by Tim2 generation at the moment
	    iTextureFlagsToCheck |= AsuraTextureFlag_IgnoreAlphaChannel
	                         |  AsuraTextureFlag_Sharpen
	                         |  AsuraTextureFlag_SharpenMore;

		//add material flags that cause changes to DontPremultiplyAlpha
		uMaterialFlagsToCheck |= AsuraMaterialFlag_SelfIlluminate
	                           | AsuraMaterialFlag_Reflective
	                           | AsuraMaterialFlag_Specular_Soft
	                           | AsuraMaterialFlag_Specular_Sharp;
	}
	if(eExportType == Tools_Level_Export::Export_PSP)
	{
		iTextureFlagsToCheck |= AsuraTextureFlag_16ColourPalette
							  | AsuraTextureFlag_HalfForPSP;
		//add material flags that cause changes to DontPremultiplyAlpha
		uMaterialFlagsToCheck |= AsuraMaterialFlag_SelfIlluminate;
	}

	//reset the flags that won't get checeked
	m_iTextureFlags &= iTextureFlagsToCheck;
	m_uMaterialFlags &= uMaterialFlagsToCheck;




	//palette/pixel format is only for PS2
	if(eExportType != Tools_Level_Export::Export_PS2)
	{
		m_iPixelFormat = 0;
		m_iPaletteFormat = 0;
		m_bUnpalettised = false;
	}

	//Max mip map drops only needed by PSP and Wii
	if(eExportType != Tools_Level_Export::Export_PSP &&
		eExportType != Tools_Level_Export::Export_Wii)
	{
		m_uMaxMipDrops = 0;
	}
}

Asura_Hash_ID TextureConversionData::GetHash() const
{
	//Skip the version number when calculating the hash value
	return Asura_GetHashID_Binary(((char*)this)+4, sizeof(TextureConversionData)-4);
}
