//
// Tools_Overlay_ViewMaterialResponse.cpp
//
// Created by Richard Rice - 12/09/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Materials/Tools_Overlay_ViewMaterialResponse.h"

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM

//asura
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Text_Output.h"

//tools
#include "Tools_Overlays.h"
#include "Materials/Tools_MaterialResponse_System.h"
#include "Materials/Tools_Overlay_PickMaterialResponse.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Texturing/Tools_Texturing_MaterialResponse.h"
#include "Texturing/Tools_Texturing_Find.h"
#include "Texturing/Tools_Mode_Texturing.h"


enum 
{
	TOOLS_OVMR_SORT_MATERIALNAME,
	TOOLS_OVMR_SORT_TEXTURENAME,
	TOOLS_OVMR_SORT_POLYGONS,

	TOOLS_OVMR_NUM_SORT_MODES,
};

static const char* g_apszSortModeStrings[TOOLS_OVMR_NUM_SORT_MODES] =
{
	"Material",
	"Texture",
	"Polygons",
};





static float g_fLargeTextureWidth = 256.0f;
static float g_fLargeTextureHeight = 256.0f;
static float g_fThumbnailWidth = 64.0f;
static float g_fThumbnailHeight = 64.0f;
static float g_fThumbnailSpacing = 8.0f;

Tools_Overlay_ViewMaterialResponse* Tools_Overlay_ViewMaterialResponse::s_pxThis;
int Tools_Overlay_ViewMaterialResponse::s_iSortMode;
bool Tools_Overlay_ViewMaterialResponse::s_bShowPrimary = true;
bool Tools_Overlay_ViewMaterialResponse::s_bShowSecondary = true;

int Tools_Overlay_ViewMaterialResponse::s_iResult = false;
Asura_Hash_ID Tools_Overlay_ViewMaterialResponse::s_uSelectedMaterial = 0;

static Asura_Vector_2 g_xUserSetSubwindowSize(0.0f,0.0f);

Tools_Overlay_ViewMaterialResponse::Tools_Overlay_ViewMaterialResponse()
:Asura_Overlay(" ",false)
{
	Asura_Assert(!s_pxThis,"");
	s_pxThis = this;

	SetDefaultTitle();

	m_pxLargeImageDialog = 0;
	m_pxSubwindowDialog = 0;
	m_pxMaterialNameDialog = 0;
	m_pxMaterialPathDialog = 0;
	m_pxTextureNameDialog = 0;
	m_pxTexturePathDialog = 0;
	m_pxPolygonsDialog = 0;
	m_fSubwindowWidth = 0.0f;
	m_pxCurrentMaterial = 0;
	m_bRebuildThumbnails = false;
	s_uSelectedMaterial = 0;
}

Tools_Overlay_ViewMaterialResponse::~Tools_Overlay_ViewMaterialResponse()
{
	Asura_Assert(s_pxThis == this,"");
	s_pxThis = 0;
}

void Tools_Overlay_ViewMaterialResponse::InitialiseOverlay()
{
	int iViewportWidth = Asura_Viewport::GetWidth();
	int iViewportHeight = Asura_Viewport::GetHeight();

	float fViewportPrintWidth = (iViewportWidth / ASURA_TEXT_OUTPUT_FONT_WIDTH);
	float fViewportPrintHeight = (iViewportHeight / ASURA_TEXT_OUTPUT_FONT_HEIGHT);

	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	//main image dialog , intially with no texture
	m_pxLargeImageDialog = new Asura_Image_Dialog(xOffset,ASURA_TEXTURE_NONE,g_fLargeTextureWidth,g_fLargeTextureHeight);
	m_pxLargeImageDialog->SetTranslucent(false);
	AddDialog(m_pxLargeImageDialog);

	//add some blank text dialogs to the side
	xOffset.x = m_pxLargeImageDialog->m_xMaxCoord.x + 1.0f;
	xOffset.y = m_pxLargeImageDialog->m_xMaxCoord.y - Asura_Dialog::DefaultSpacing()*5;

	m_pxMaterialNameDialog = new Asura_Dialog("", xOffset);
	m_pxMaterialNameDialog->SetHelpText("The name of the current material");
	AddDialog(m_pxMaterialNameDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_pxMaterialPathDialog = new Asura_Dialog("", xOffset);
	m_pxMaterialPathDialog->SetHelpText("The name of the current material");
	AddDialog(m_pxMaterialPathDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_pxTextureNameDialog = new Asura_Dialog("", xOffset);
	m_pxTextureNameDialog->SetHelpText("The name of the albedo texture");
	AddDialog(m_pxTextureNameDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_pxTexturePathDialog = new Asura_Dialog("", xOffset);
	m_pxTexturePathDialog->SetHelpText("The name of the albedo texture");
	AddDialog(m_pxTexturePathDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	m_pxPolygonsDialog = new Asura_Dialog("", xOffset);
	m_pxPolygonsDialog->SetHelpText("The number of environmental polygons using this material");
	AddDialog(m_pxPolygonsDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//add sort and filter options below
	xOffset.x = 1.0f;
	xOffset.y += 1.0f;

	pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_apszSortModeStrings, &s_iSortMode, TOOLS_OVMR_NUM_SORT_MODES, TOOLS_OVMR_NUM_SORT_MODES, 0, &m_bRebuildThumbnails, "Sort");
	((Asura_DropDown_List_Dialog*)pxDialog)->SetOnlyUpdateSelectionIndexOnClick(true);
	pxDialog->SetHelpText("Determines how the material thumbnails are sorted");
	AddDialog(pxDialog);
	
	xOffset.x = pxDialog->m_xMaxCoord.x + 4.0f;
	pxDialog = new Asura_Tickbox_Dialog("Primary", xOffset, &s_bShowPrimary, &m_bRebuildThumbnails);
	pxDialog->SetHelpText("Determines whether materials used as a primary material on a polygon should be shown");
	AddDialog(pxDialog);
	
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Tickbox_Dialog("Secondary", xOffset, &s_bShowSecondary, &m_bRebuildThumbnails);
	pxDialog->SetHelpText("Determines whether materials used as a secondary material on a polygon should be shown");
	AddDialog(pxDialog);
		
	xOffset.x = 1.0f;	
	xOffset.y += Asura_Dialog::LargeSpacing();
	

	//create a large subwindow dialog to contain all the thumbails

	float fSubwindowHeight = fViewportPrintHeight - xOffset.y - 8.0f;
	m_fSubwindowWidth = g_xUserSetSubwindowSize.x ? g_xUserSetSubwindowSize.x : fViewportPrintWidth - 6.0f;
	m_pxSubwindowDialog = new Asura_Subwindow_Dialog(xOffset,m_fSubwindowWidth,fSubwindowHeight,false);
	AddDialog(m_pxSubwindowDialog);

 	xOffset.y = m_pxSubwindowDialog->m_xMaxCoord.y + 1.0f;

	//add control for resizing the subwindow dialog
	pxDialog = new Asura_Resize_Dialog(m_pxSubwindowDialog, &g_xUserSetSubwindowSize);
	AddDialog(pxDialog);


	pxDialog = new Asura_Function_Dialog("Okay",xOffset,OnOkay,(int)this);
	pxDialog->SetIsHiddenFunction(IsNotModal,(int)this);
	pxDialog->SetHelpText("Accept the current selection");
	AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,OnCancel,(int)this);
	pxDialog->SetIsHiddenFunction(IsNotModal,(int)this);
	pxDialog->SetHelpText("Cancel material selection");
	AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Select From Tree (L)",xOffset,OnSelectFromTree,(int)this);
	pxDialog->SetIsHiddenFunction(IsNotModal,(int)this);
	pxDialog->SetHelpText("Select material from tree of all materials instead");
	AddDialog(pxDialog);

	xOffset.x = 1.0f;
	pxDialog = new Asura_Function_Dialog("Close",xOffset,OnClose,(int)this);
	pxDialog->SetIsHiddenFunction(IsModal,(int)this);
	pxDialog->SetHelpText("Closes this overlay");
	AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	
	pxDialog = new Asura_Function_Dialog("Edit Material",xOffset,OnEditMaterial,(int)this);
	pxDialog->SetIsHiddenFunction(IsModal,(int)this);
	pxDialog->SetIsEnabledFunction(IsMaterialSelected,(int)this);
	pxDialog->SetHelpText("Shows the material properties overlay for the selected material");
	AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Find Polygons",xOffset,OnFindPolygons,(int)this);
	pxDialog->SetIsHiddenFunction(IsModal,(int)this);
	pxDialog->SetIsEnabledFunction(CanFindPolygons,(int)this);
	pxDialog->SetHelpText("Finds polygons using the selected material , adding them to the list managed by the 'Find' overlay");
	AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Replace",xOffset,OnReplace);
	pxDialog->SetIsHiddenFunction(IsModal,(int)this);
	pxDialog->SetIsEnabledFunction(IsMaterialSelected,(int)this);
	pxDialog->SetHelpText("Replaces instances of the selected material on the environment with a different material");
	AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	MakeDialogsRelativeToResizableDialog();


	UpdateThumbnails();
	
}


void Tools_Overlay_ViewMaterialResponse::CreateOverlay()
{
	//overlay already open?
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_VIEWMATERIALRESPONSE))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_VIEWMATERIALRESPONSE);
		return;
	}

	//create the overlay then
	Tools_Overlay_ViewMaterialResponse* pxOverlay = new Tools_Overlay_ViewMaterialResponse();
	
	pxOverlay->InitialiseOverlay();

	pxOverlay->m_uID = TOOLS_OVERLAY_VIEWMATERIALRESPONSE;
	Asura_Overlay_System::SubmitNewOverlay( pxOverlay);
}

void Tools_Overlay_ViewMaterialResponse::CloseOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_VIEWMATERIALRESPONSE);	
}

bool Tools_Overlay_ViewMaterialResponse::OverlayExists()
{
	return s_pxThis != 0;
}

Asura_Hash_ID Tools_Overlay_ViewMaterialResponse::PickMaterial(const char* szTitle,Asura_Hash_ID uInitialSelection /*= 0*/)
{
	//make sure the overlay is open
	bool bOverlayInitiallyOpen = OverlayExists(); 

	if(!bOverlayInitiallyOpen) CreateOverlay();
	
	Asura_Assert(s_pxThis,"Failed to get material overlay");
	if(!s_pxThis) return 0;

	//set the title
	if(szTitle) s_pxThis->SetTitle(szTitle);

	//update selection
	s_pxThis->SetSelectedMaterial(uInitialSelection);

	//run as modal overlay
	s_iResult = TOOLS_OVMR_RESULT_CANCEL;
	Tools_Modal_Overlay::DoModal(s_pxThis);

	CloseOverlay();

	if(s_iResult == TOOLS_OVMR_RESULT_SELECTFROMTREE)
	{
		//show the other material selection overlay instead
		Tools_Overlay_PickMaterialResponse xOverlay(szTitle);
		if(xOverlay.DoModal(uInitialSelection))
		{
			s_iResult = TOOLS_OVMR_RESULT_OKAY;
			return xOverlay.GetSelection();
		}
		else
		{
			s_iResult = TOOLS_OVMR_RESULT_CANCEL;
			return uInitialSelection;
		}
	}

	return (s_iResult == TOOLS_OVMR_RESULT_OKAY) ? s_uSelectedMaterial : uInitialSelection;
}


void Tools_Overlay_ViewMaterialResponse::UpdateThumbnails()
{
	//delete all the existing thumbnail dialogs 
	for(int i=0; i<m_xMaterialTree.GetNumberOfNodes(); i++)
	{
		Tools_OVMR_Material* pxMaterial = m_xMaterialTree.GetDataByIndex(i);
		if(pxMaterial->m_pxImageDialog)
		{
			DeleteSingleDialog(pxMaterial->m_pxImageDialog);
			pxMaterial->m_pxImageDialog = 0;
		}
	}


	m_xMaterialTree.Reset();
	m_pxCurrentMaterial = 0;


	//add all materials used by the environment
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
		if(!pxMesh->TextureDataPtr) continue;

		for(int j=0;j<pxMesh->NumberOfPolygons;j++)
		{
			Asura_Mesh_TextureInfo* pxTex = &pxMesh->TextureDataPtr[j];

			
			
			if(pxTex->m_auMaterialResponseHashID[0])
			{
				AddMaterial(pxTex->m_auMaterialResponseHashID[0],true,true);
			}
			if(pxTex->m_auMaterialResponseHashID[1])
			{
				bool bIncPolyCount = (pxTex->m_auMaterialResponseHashID[0] != pxTex->m_auMaterialResponseHashID[1]);
				AddMaterial(pxTex->m_auMaterialResponseHashID[1],bIncPolyCount,false);
			}
		}
	}

	int iNumMaterials = m_xMaterialTree.GetNumberOfNodes();
	if(!iNumMaterials) return;

	//copy the material pointers into an array for sorting
	Tools_OVMR_Material** ppxSortedMaterials = new Tools_OVMR_Material*[iNumMaterials];

	int iNumMaterialsToShow = 0;

	for(int i=0;i<iNumMaterials;i++)
	{
		Tools_OVMR_Material* pxMaterial = m_xMaterialTree.GetDataByIndex(i);
		if((s_bShowPrimary && pxMaterial->m_bUsedAsPrimary) ||
			(s_bShowSecondary && pxMaterial->m_bUsedAsSecondary))
		{
			ppxSortedMaterials[iNumMaterialsToShow++] = m_xMaterialTree.GetDataByIndex(i);
		}
	}

	//Sort the thumbnails
	qsort(ppxSortedMaterials, iNumMaterialsToShow, sizeof(Tools_OVMR_Material*), SortMaterialCallback);

	//Create image dialogs for each material
	Asura_Vector_2 xOffset;
	xOffset.x = g_fThumbnailSpacing / ASURA_TEXT_OUTPUT_FONT_WIDTH;
	xOffset.y = g_fThumbnailSpacing / ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	for(int i=0;i<iNumMaterialsToShow;i++)
	{
		
		Tools_OVMR_Material* pxMaterial = ppxSortedMaterials[i];

		//get the texture to show
		int iTexture = ASURA_TEXTURE_NONE;
		Tools_MaterialResponse* pxMaterialResponse = Tools_MaterialResponse_System::GetMaterialByHashID(pxMaterial->m_uMaterialID);

		if(pxMaterialResponse)
		{
			iTexture = pxMaterialResponse->GetTexture(Asura_MaterialResponse::ALBEDO_TEXTURE);
		}

		Asura_Image_Dialog* pxDialog = ppxSortedMaterials[i]->m_pxImageDialog = new Asura_Image_Dialog(xOffset, iTexture, g_fThumbnailWidth, g_fThumbnailHeight);
		pxDialog->SetTranslucent(false);
		if(pxMaterial->m_uMaterialID == s_uSelectedMaterial)
		{
			pxDialog->SetBackground(0xffff0000,1);
		}
		else
		{
			pxDialog->SetBackground(0xffffffff,1);
		}
		m_pxSubwindowDialog->AddDialog(pxDialog);

		//Find the position to use for the next image
		xOffset.x += (g_fThumbnailWidth+g_fThumbnailSpacing) / ASURA_TEXT_OUTPUT_FONT_WIDTH;
		
		//need to move onto next row?
		if((xOffset.x + ((g_fThumbnailWidth+g_fThumbnailSpacing) / ASURA_TEXT_OUTPUT_FONT_WIDTH)) > m_fSubwindowWidth)
		{
			xOffset.x = g_fThumbnailSpacing / ASURA_TEXT_OUTPUT_FONT_WIDTH;
			xOffset.y += (g_fThumbnailHeight+g_fThumbnailSpacing) / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
		}

	}

	delete [] ppxSortedMaterials;

}

void Tools_Overlay_ViewMaterialResponse::AddMaterial(Asura_Hash_ID uMaterialID, bool bIncreasePolygonCount,bool bPrimary)
{
	Tools_OVMR_Material* pxMaterial = m_xMaterialTree.GetDataByHash(uMaterialID);
	if(!pxMaterial)
	{
		pxMaterial = m_xMaterialTree.AddNode(uMaterialID);
		Asura_Assert(pxMaterial,"");

		pxMaterial->m_uMaterialID = uMaterialID;

		Tools_MaterialResponse* pxMaterialResponse = Tools_MaterialResponse_System::GetMaterialByHashID(pxMaterial->m_uMaterialID);

		if(pxMaterialResponse)
		{

			//split material into path and name components
			Asura_String xFullPath = pxMaterialResponse->GetName();

			xFullPath.SplitFilename(&pxMaterial->m_xMaterialPath,&pxMaterial->m_xMaterialName,0);
			if(pxMaterial->m_xMaterialName.IsNull()) pxMaterial->m_xMaterialName = "";
			if(pxMaterial->m_xMaterialPath.IsNull()) pxMaterial->m_xMaterialPath = "";
			
			
			//split filename into path and name components
			Asura_String xExtension;
			xFullPath = pxMaterialResponse->GetTextureFilename(Asura_MaterialResponse::ALBEDO_TEXTURE);

			xFullPath.SplitFilename(&pxMaterial->m_xTexturePath,&pxMaterial->m_xTextureName,&xExtension);
			if(pxMaterial->m_xTextureName.IsNull()) pxMaterial->m_xTextureName = "";
			if(pxMaterial->m_xTexturePath.IsNull()) pxMaterial->m_xTexturePath = "";

			if(!xExtension.IsBlank())
			{
				pxMaterial->m_xTextureName += ".";
				pxMaterial->m_xTextureName += xExtension;
			}
			
		}
		else
		{
			char szText[200];
			Asura_PrintfToBfr(szText,"*Not Found* : %s",sizeof(szText),true,Tools_MaterialResponse_System::GetMissingMaterialDescription(uMaterialID));
			
			pxMaterial->m_xMaterialName = szText;
			pxMaterial->m_xMaterialPath = "";
			pxMaterial->m_xTextureName = "";
			pxMaterial->m_xTexturePath = "";
		}
	}

	if(bPrimary)
	{
		pxMaterial->m_bUsedAsPrimary = true;
	}
	else
	{
		pxMaterial->m_bUsedAsSecondary = true;
	}

	if(bIncreasePolygonCount)
	{
		pxMaterial->m_uNumPolygons++;
	}
}

void Tools_Overlay_ViewMaterialResponse::PerFrameUpdate()
{
	Asura_Overlay::PerFrameUpdate();

	if(g_xUserSetSubwindowSize.x && g_xUserSetSubwindowSize.x != m_fSubwindowWidth)
	{
		m_fSubwindowWidth = g_xUserSetSubwindowSize.x;
		m_bRebuildThumbnails = true;
	}

	if(m_bRebuildThumbnails)
	{
		m_bRebuildThumbnails = false;
		UpdateThumbnails();
	}

	if(this == Asura_Overlay_System::GetCurrentOverlay())
	{
		//Determine which material is under the cursor
		float fMouseX,fMouseY;
		Asura_Cursor::GetScreenCoords(fMouseX,fMouseY);
		Asura_Vector_2 xMousePos(fMouseX,fMouseY);

		bool bMouseOverAThumbnail = false;

		Tools_OVMR_Material* pxMaterial = 0;
		for(int i=0; i<m_xMaterialTree.GetNumberOfNodes(); i++)
		{
			pxMaterial = m_xMaterialTree.GetDataByIndex(i);
			if(pxMaterial->m_pxImageDialog)
			{
				if(pxMaterial->m_pxImageDialog->IsPointInDialog(m_xOrigin, xMousePos))
				{
					SetCurrentMaterial(pxMaterial);
					bMouseOverAThumbnail = true;
					break;
				}
			}
		}

		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) || Asura_UserInput::KeyPressed(ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON1))
		{
			if(m_pxCurrentMaterial && bMouseOverAThumbnail)
			{
				SetSelectedMaterial(m_pxCurrentMaterial->m_uMaterialID);

				if(Asura_UserInput::KeyPressed(ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON1))
				{
					if(IsModal())
					{
						//double clicked a material , so end selection
						OnOkay();
					}
					else
					{
						OnEditMaterial();
					}
				}
			}
		}
	}

	if(IsModal())
	{
		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ENTER))
		{
			OnOkay();
		}
		else if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
		{
			OnCancel();
		}
		else if(Asura_UserInput::DebouncedKeyPressed('l'))
		{
			OnSelectFromTree();
		}
	}


}

void Tools_Overlay_ViewMaterialResponse::SetCurrentMaterial(Tools_OVMR_Material* pxMaterial)
{
	if(pxMaterial == m_pxCurrentMaterial) return;
	m_pxCurrentMaterial = pxMaterial;

	if(!pxMaterial) return;

	//update the enlarged image
	int iTexture = ASURA_TEXTURE_NONE;
	Tools_MaterialResponse* pxMaterialResponse = Tools_MaterialResponse_System::GetMaterialByHashID(pxMaterial->m_uMaterialID);

	if(pxMaterialResponse)
	{
		iTexture = pxMaterialResponse->GetTexture(Asura_MaterialResponse::ALBEDO_TEXTURE);
	}
	m_pxLargeImageDialog->SetTexture(iTexture);

	//update the various text dialogs
	char szText[200];

	Asura_PrintfToBfr(szText,"Material Name : %s",sizeof(szText),true,pxMaterial->m_xMaterialName.GetString());
	m_pxMaterialNameDialog->UpdateLabel(szText);

	Asura_PrintfToBfr(szText,"Material Path : %s",sizeof(szText),true,pxMaterial->m_xMaterialPath.GetString());
	m_pxMaterialPathDialog->UpdateLabel(szText);

	Asura_PrintfToBfr(szText,"Texture Name: %s",sizeof(szText),true,pxMaterial->m_xTextureName.GetString());
	m_pxTextureNameDialog->UpdateLabel(szText);

	Asura_PrintfToBfr(szText,"Texture Path: %s",sizeof(szText),true,pxMaterial->m_xTexturePath.GetString());
	m_pxTexturePathDialog->UpdateLabel(szText);


	Asura_PrintfToBfr(szText,"Polygons : %d",sizeof(szText),true,pxMaterial->m_uNumPolygons);
	m_pxPolygonsDialog->UpdateLabel(szText);
}

void Tools_Overlay_ViewMaterialResponse::SetSelectedMaterial(Asura_Hash_ID uMaterialID)
{
	s_uSelectedMaterial = uMaterialID;

	//about the border colours
	for(int i=0; i<m_xMaterialTree.GetNumberOfNodes(); i++)
	{
		Tools_OVMR_Material* pxMaterial = m_xMaterialTree.GetDataByIndex(i);
		if(pxMaterial->m_pxImageDialog)
		{
			Asura_Image_Dialog* pxDialog = pxMaterial->m_pxImageDialog;

			if(pxMaterial->m_uMaterialID == s_uSelectedMaterial)
			{
				pxDialog->SetBackground(0xffff0000,1);
			}
			else
			{
				pxDialog->SetBackground(0xffffffff,1);
			}
		}
	}
}

int Tools_Overlay_ViewMaterialResponse::SortMaterialCallback(const void* pElem1,const void* pElem2)
{
	const Tools_OVMR_Material* pxMat1 = *((const Tools_OVMR_Material**) pElem1);
	const Tools_OVMR_Material* pxMat2 = *((const Tools_OVMR_Material**) pElem2);

	switch(s_iSortMode)
	{
		case TOOLS_OVMR_SORT_MATERIALNAME :
			return stricmp(pxMat1->m_xMaterialName.GetString(), pxMat2->m_xMaterialName.GetString());

		case TOOLS_OVMR_SORT_TEXTURENAME :
			return stricmp(pxMat1->m_xTextureName.GetString(), pxMat2->m_xTextureName.GetString());

		case TOOLS_OVMR_SORT_POLYGONS :
			if(pxMat1->m_uNumPolygons < pxMat2->m_uNumPolygons) return -1;
			if(pxMat1->m_uNumPolygons > pxMat2->m_uNumPolygons) return 1;
			return 0;
	}

	Asura_Assert(0,"Unexpected sort mode");
	return 0;
}

bool Tools_Overlay_ViewMaterialResponse::WasCancelled()
{
	return s_iResult == TOOLS_OVMR_RESULT_CANCEL;
}

void Tools_Overlay_ViewMaterialResponse::OnOkay()
{
	s_iResult = TOOLS_OVMR_RESULT_OKAY;
	EndModal();	
}

void Tools_Overlay_ViewMaterialResponse::OnCancel()
{
	s_iResult = TOOLS_OVMR_RESULT_CANCEL;
	EndModal();	
}

void Tools_Overlay_ViewMaterialResponse::OnSelectFromTree()
{
	s_iResult = TOOLS_OVMR_RESULT_SELECTFROMTREE;
	EndModal();	
}

void Tools_Overlay_ViewMaterialResponse::OnClose()
{
	CloseOverlay();
}

void Tools_Overlay_ViewMaterialResponse::OnEditMaterial()
{
	if(s_uSelectedMaterial)
	{
		Tools_Texturing_MaterialResponse::EditMaterial(s_uSelectedMaterial);	
	}
}

bool Tools_Overlay_ViewMaterialResponse::IsNotModal()
{
	return !IsModal();
}

bool Tools_Overlay_ViewMaterialResponse::IsModal()
{
	return m_bAlwaysOnTop;
}

bool Tools_Overlay_ViewMaterialResponse::IsMaterialSelected()
{
	return s_uSelectedMaterial != 0;
}

bool Tools_Overlay_ViewMaterialResponse::CanFindPolygons()
{
	return s_uSelectedMaterial != 0 && Tools_Polygon_Selection::IsActive();
}


void Tools_Overlay_ViewMaterialResponse::OnFindPolygons()
{
	if(!s_uSelectedMaterial) return;

	Tools_Texturing_Find::FindMaterial(s_uSelectedMaterial);	
}

void Tools_Overlay_ViewMaterialResponse::OnReplace()
{
	if(!s_uSelectedMaterial) return;

	Asura_Hash_ID uOldMaterial = s_uSelectedMaterial;
	//ask for the replacement material
	Asura_Hash_ID uNewMaterial = PickMaterial("Replace with which material?", uOldMaterial);

	if(!uNewMaterial || uOldMaterial == uNewMaterial || WasCancelled())
	{
		//abort replacement , and open up the overlay again
		CreateOverlay();
		s_pxThis->SetSelectedMaterial(uOldMaterial);
		return;
	}

	//replace all instances of the old material with the new one
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		Asura_Mesh* const pxMesh = Asura_Module_System::GetMeshForModule(i);
		if( !pxMesh->TextureDataValid )
		{
			Asura_Assertf( false, ("Mesh '%s' has invalid texture data.", pxMesh->Name.GetString()) );
			continue;
		}

		bool bMeshModified = false;

		for(int j=0; j<pxMesh->NumberOfPolygons; j++)
		{
			if(pxMesh->TextureDataPtr[j].m_iMaterialID == ASURA_MATERIAL_NONE) continue;
			if(pxMesh->IsMergedObjectPolygon(j)) continue;

			bool bLoggedForUndo = false;
			
			for(int iMatSlot=0;iMatSlot<2;iMatSlot++)
			{
				if(pxMesh->TextureDataPtr[j].m_auMaterialResponseHashID[iMatSlot] == uOldMaterial)
				{
					//log for undo if not done so for this polygon yet
					if(!bLoggedForUndo)
					{
						Tools_Texturing_MaterialResponse::LogApplyMaterial(i, j);
						bLoggedForUndo = true;
					}

					pxMesh->TextureDataPtr[j].m_auMaterialResponseHashID[iMatSlot] = uNewMaterial;
					bMeshModified = true;
				}
			}
		}

		if(bMeshModified)
		{
			Tools_UpdateTexturingOnSurfaces( pxMesh );
			Tools_Texturing_MaterialResponse::SetMeshModified( pxMesh, true );
		}
	}

	//open the overlay again
	CreateOverlay();
	s_pxThis->SetSelectedMaterial(uNewMaterial);
}



#endif //ASURA_USE_MATERIAL_RESPONSE_SYSTEM
