//
// Tools_Overlay_ViewMaterialResponse.h
//
// Created by Richard Rice - 12/09/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_OVERLAY_VIEWMATERIALRESPONSE__
#define __TOOLS_OVERLAY_VIEWMATERIALRESPONSE__

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM

#include "Interface/Asura_Overlay.h"
#include "Core/Asura_HashTree.h"

class Asura_Image_Dialog;
class Asura_Subwindow_Dialog;


class Tools_OVMR_Material
{
public :
	Tools_OVMR_Material()
	{
		m_uMaterialID = 0;
		m_uNumPolygons = 0;
		m_pxImageDialog = 0;
		m_bUsedAsPrimary = false;
		m_bUsedAsSecondary = false;
	}

	Asura_Hash_ID m_uMaterialID;
	u_int m_uNumPolygons;
	Asura_Image_Dialog* m_pxImageDialog;

	Asura_String m_xMaterialName;
	Asura_String m_xMaterialPath;
	Asura_String m_xTextureName;
	Asura_String m_xTexturePath;
	bool m_bUsedAsPrimary;
	bool m_bUsedAsSecondary;
	
};

enum
{
	TOOLS_OVMR_RESULT_OKAY,
	TOOLS_OVMR_RESULT_CANCEL,
	TOOLS_OVMR_RESULT_SELECTFROMTREE,
};



class Tools_Overlay_ViewMaterialResponse : public Asura_Overlay
{
public :
	~Tools_Overlay_ViewMaterialResponse();

	static void CreateOverlay();
	static void CloseOverlay();

	static bool OverlayExists();

	//pick a material using a modal overlay
	static Asura_Hash_ID PickMaterial(const char* szTitle,Asura_Hash_ID uInitialSelection = 0);

	static bool WasCancelled();
	

private :
	Tools_Overlay_ViewMaterialResponse();

	virtual void PerFrameUpdate();

	void InitialiseOverlay();
	void UpdateThumbnails();

	void AddMaterial(Asura_Hash_ID uMaterialID, bool bIncreasePolygonCount,bool bPrimary);
	static int SortMaterialCallback(const void* pElem1,const void* pElem2);

	void SetCurrentMaterial(Tools_OVMR_Material* pxMaterial);
	void SetSelectedMaterial(Asura_Hash_ID uMaterialID);

	void SetDefaultTitle() {SetTitle("Materials");}

	void EndModal() {m_bAlwaysOnTop = false;}

	//overlay callbacks
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_Overlay_ViewMaterialResponse*)iThis)->__FnName();}\
	void __FnName();
	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_Overlay_ViewMaterialResponse*)iThis)->__FnName();}\
	bool __FnName();

	OVERLAY_VOID_CALLBACK(OnOkay)
	OVERLAY_VOID_CALLBACK(OnCancel)
	OVERLAY_VOID_CALLBACK(OnSelectFromTree)

	OVERLAY_VOID_CALLBACK(OnEditMaterial)
	OVERLAY_VOID_CALLBACK(OnClose)
	OVERLAY_VOID_CALLBACK(OnFindPolygons)



	OVERLAY_BOOL_CALLBACK(IsNotModal)
	OVERLAY_BOOL_CALLBACK(IsModal)
	OVERLAY_BOOL_CALLBACK(IsMaterialSelected)
	OVERLAY_BOOL_CALLBACK(CanFindPolygons)

	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK


	static void OnReplace();

	//////

	Asura_Image_Dialog* m_pxLargeImageDialog;
	Asura_Subwindow_Dialog* m_pxSubwindowDialog;
	Asura_Dialog* m_pxMaterialNameDialog;
	Asura_Dialog* m_pxTextureNameDialog;
	Asura_Dialog* m_pxMaterialPathDialog;
	Asura_Dialog* m_pxTexturePathDialog;
	Asura_Dialog* m_pxPolygonsDialog;

	bool m_bRebuildThumbnails;

	float m_fSubwindowWidth;

	static u_int s_uSelectedMaterial;

	Tools_OVMR_Material* m_pxCurrentMaterial;

	Asura_TemplateHashTree<Tools_OVMR_Material> m_xMaterialTree;

	static int s_iSortMode;
	static bool s_bShowPrimary;
	static bool s_bShowSecondary;

	static int s_iResult;

	static Tools_Overlay_ViewMaterialResponse* s_pxThis;
};



#endif //ASURA_USE_MATERIAL_RESPONSE_SYSTEM


#endif //__TOOLS_OVERLAY_VIEWMATERIALRESPONSE__
