//////////////////////////////////////////////////////////////////////////////
//
// Asura_TriStripper.h
//
// Copyright (c) 2006 Rebellion. All rights reserved.
//
// This file really needs renaming...
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___ASURA_TRISTRIPPER_H___
#define ___ASURA_TRISTRIPPER_H___ 1

// C L A S S E S /////////////////////////////////////////////////////////////

class Tools_TriStripper
{

public:

    enum OPTIMISE
    {
        OPTIMISE_CACHE,
        OPTIMISE_INDICES,
    };

    enum OUTPUT
    {
        OUTPUT_TRISTRIP,                    // Using degenerates.
        OUTPUT_TRILIST,
        OUTPUT_TRISTRIP_WITH_RESET_INDICES, // Will fallback to degenerates if platform doesn't support restart indices.
    };

    struct Stats
    {
        u_int uNumRealStrips;
        u_int uNumRealSingleTriStrips;
        float fAvgRealStripIndices;
        u_int uNumConcatdStrips;
        float fAvgConcatdStripIndices;
        float fIdxRedundancyProportion; // Proportion of indices in output strip(s) that are reset indices or end degenerate tris.
        float fTriRedundancyProportion; // Proportion of tris in output strip(s) that are degenerate due to concatenation. Will be 0 if using reset indices.
    };

    // Call either side of any other usage of this class.
    static void Initialise( void );
    static void Shutdown( void );

    // Optional modifications to the stripification process. These should be
    // set after Initialise() but before CreateStrips(). See Initialise()
    // implementation for defaults.
    static void SetOptimisationCriterion( OPTIMISE eOptimisationCriterion );
    static void SetOutputType( OUTPUT eOutputType );       
    static void SetVertexCacheSize( u_int uSize );
    static void SetShaderCycleCount( u_int uCount );
    static void SetShouldResetStatsBetweenStrips( bool bShouldReset );

    // Stripifies the input mesh, returning the number of indices in the
    // output strip. The input triangle array is expected to contain (3 *
    // uNumTris) entries. The caller is responsible for deallocating the
    // output strip index array via delete[].
    static u_int CreateStrips( u_int uNumTris, const u_short* pusTriIndices,
                               u_short** ppusStripIndices );

    // Rearranges the strip's vertices so that they occur in the order that
    // they are first used. The output permutation array will be a mapping
    // from new vertex index to old vertex index. The strip index array
    // provided is remapped to use the new vertex locations. The caller is
    // responsible for deallocating the output vertex permutation array via
    // delete[]. In the case where the indices are only using a subset of the
    // relevant vertex buffer, you can optionally determine which prefix of
    // that list is being used by checking *puNumUsedVertices.
    static void ComputeVertexPermutation( u_int uNumIndicesInStrip, u_short* pusStripIndices,
                                          u_int uNumVertices, u_short** ppusVertexPermutation,
                                          u_int* puNumUsedVertices = NULL );

    // Retrieves latest statistics from stripping operations. These are
    // either cumulative or only relating to the last strip, depending on
    // what you've set via SetShouldResetStatsBetweenStrips().
    static void GetStats( Stats* pxStatsOutput );

};

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___ASURA_TRISTRIPPER_H___
