//
// Tools_EnvironmentProcessor.h
//
// Created by KJL - 27/02/2003 11:48:58
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_ENVIRONMENTPROCESSOR_H__
#define __TOOLS_ENVIRONMENTPROCESSOR_H__

#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"

#include "Mesh/Asura_Mesh.h"

class Tools_SubdividedMesh;
class Tools_SubdividedMesh_Packet;

class Tools_EnvironmentProcessor_Vertex
{
	public:
		Asura_Vector_3						m_xPosition;
		Asura_Vector_3						m_xNormal;
		float								m_fU;
		float								m_fV;
		float								m_fU2;
		float								m_fV2;
		u_int								m_uDiffuseRGBA;

#if defined(ASURA_TARGET_SM3)
		Asura_Vector_3						m_xTangent;
		Asura_Vector_3						m_xBinormal;
		Asura_Vector_3						m_xTangent2;
		Asura_Vector_3						m_xBinormal2;

		Asura_SphericalHarmonics_Order3		m_xLightTransportHarmonic;
		Asura_SphericalHarmonics_Order2_RGB m_xDiffuseSH;

		Asura_Vector_3						m_xBrightestLightVector;	// VGP 13Feb2006

#endif	// defined(ASURA_TARGET_SM3)

};

class Tools_EnvironmentProcessor_Strip
{
	public:
		
		Tools_EnvironmentProcessor_Strip()
		{
			m_pxVertices = 0;
			m_pusIndices = 0;
			m_uPolygonFlags = 0;
		}
		
		~Tools_EnvironmentProcessor_Strip()
		{
			delete [] m_pxVertices;
			delete [] m_pusIndices;
		}

		static void Move(Tools_EnvironmentProcessor_Strip &dst, Tools_EnvironmentProcessor_Strip &src)
		{
			dst = src;
			src.m_pxVertices = NULL;
			src.m_pusIndices = NULL;
		}



		u_int								m_uNumberOfIndices;
		u_int								m_uNumberOfTriangles;
		u_int								m_uNumberOfVertices;
		u_int								m_uPolygonFlags;
											
		int									m_iMaterialID;

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_Hash_ID						m_auMaterialResponseHashID[TOOLS_NUM_MATERIALRESPONSE_PER_POLY];
#endif

		Tools_EnvironmentProcessor_Vertex*	m_pxVertices;
		u_short*							m_pusIndices;

		// module the strip belongs to
		u_int								m_uModule;

		// material IDs get converted for export, so keep a copy around for tests later
		int									m_iUnalteredMaterialID ;
};


class Tools_EnvironmentProcessor_Mesh
{
	public:
		
		Tools_EnvironmentProcessor_Mesh()
		{
			m_pxStrips = 0;
		}
		~Tools_EnvironmentProcessor_Mesh()
		{
			delete [] m_pxStrips;
		}


		u_int									m_uNumberOfStrips;
		Tools_EnvironmentProcessor_Strip*		m_pxStrips;

		u_int									m_uTotalNumberOfVertices;
		u_int									m_uTotalNumberOfIndices;

#if defined( ASURA_TOOLS)
		Asura_String	m_xMeshName;
#endif
};

class Tools_EnvironmentProcessor_PosMesh
{
	public:
		
		Tools_EnvironmentProcessor_PosMesh(Tools_EnvironmentProcessor_Mesh* pxMesh);

		~Tools_EnvironmentProcessor_PosMesh()
		{
			delete [] m_pxPositions;
			for(u_int u = 0; u < m_uNumberOfStrips; u++)
			{
				delete [] m_ppusIndices[u];
				delete [] m_ppusPositionMap[u];
			}
			delete [] m_ppusIndices;
			delete [] m_ppusPositionMap;
		}
		u_int									m_uNumberOfPositions;
		Asura_Vector_3*							m_pxPositions;

		unsigned short**						m_ppusPositionMap;

		u_int									m_uNumberOfStrips;
		unsigned short**						m_ppusIndices;

	private:
		// Adds any new positions to array, with corresponding indices.
		void AddStrip(Tools_EnvironmentProcessor_Strip* pxStrip, unsigned int uStripIndex);

		unsigned short AddVertex(Tools_EnvironmentProcessor_Vertex* pxVertex);

};

typedef void (*PROJECT_LIGHTING_FN) ();

class Tools_EnvironmentProcessor
{
	public:
		// (SB - 06/03/07) Colour adjustment callback
		typedef u_int (ColourAdjustmentCallback)(const unsigned int);//, const Asura_Mesh_Prelighting*);
		// (AS - 23/5/2007) Added callback to selectively split packet into multiple strips
		// return value is a project specified packet ID, same ID means same packet, different Id's mean different packets
		// value can be anything but 0 <= ID <= MaxPackets will be most optimal as that will just be an array lookup, anything else will be an non sorted array search
		typedef u_int (PacketSelectionCallback)(const Tools_SubdividedMesh& xMesh, const Tools_SubdividedMesh_Packet &xPacket, int iVertex);
		// (AS - 25/5/2007) Added callback to set strip flags
		// pVertIndexes are the indexes into the packet verticies of every vertex in this list (xStrip.m_uNumberOfVertices of them)
		// pusTriList are the pre-stripped triangles (indexed into pVertIndexes)
		// uNumTris is the number of triangles in pusTriList, eg len(pusTriList) == 3 * uNumTris
		typedef u_int (StripFlagCallback)(const Tools_SubdividedMesh& xMesh, const Tools_SubdividedMesh_Packet &xPacket, Tools_EnvironmentProcessor_Strip &xStrip, const u_int * pVertIndexes, const u_short* pusTriList, u_int uNumTris);


		static void Initialise( bool bShouldRemoveDuplicateVertices = false,  bool bShouldStripify = true, bool bShouldSubdivide = true, const u_int uPolygonFlagsToIgnor = AsuraPolygonFlag_Water|AsuraPolygonFlag_DontExport);
		static void Shutdown(void);

		// KOR - 22/05/2003 11:41:09 - for saving memory by calculating
		// which vertices share positions but not UVs, normals, etc.
		static void GroupByPosition(void);
		static const Tools_EnvironmentProcessor_PosMesh* GetPositionMesh(u_int uModuleIndex);

		static const Tools_EnvironmentProcessor_Mesh* GetProcessedMesh(u_int uModuleIndex);

		typedef int (*STRIP_CMP_FN)( const void*, const void* );
		static const Tools_EnvironmentProcessor_Strip* GetStripsSortedByMaterial( STRIP_CMP_FN pfnStripCmp = NULL );

		// KOR - 08/11/2006 - uses this to over-ride the Asura pre-lighting with a project-specific vertex colour
		// This is used by Simpsons to encode line tagging data into vertex colour.
		static void SetProjectLightingCallback(PROJECT_LIGHTING_FN pfnProjectLightingCallback) { s_pfnProjectLightingCallback = pfnProjectLightingCallback; }

		static u_int GetNumberOfProcessedMeshes(void)
		{
			return s_uNumberOfMeshes;
		}
	
		static u_int GetTotalNumberOfStrips(void)
		{
			return s_uTotalNumberOfStrips;
		}

		static u_int GetTotalNumberOfVertices(void)
		{
			return s_uTotalNumberOfVertices;
		}

		static u_int GetTotalNumberOfIndices(void)
		{
			return s_uTotalNumberOfIndices;
		}

		static u_int GetTotalNumberOfPositions(void)
		{
			return s_uTotalNumberOfPositions;
		}

		static void SetColourAdjustmentCallback(ColourAdjustmentCallback* const pfnColourAdustmentCallback = NULL) { s_pfnColourAdjustmentCallback = pfnColourAdustmentCallback; }
		static void SetPacketSelectionCallback(PacketSelectionCallback* const pfnPacketSelectionCallback = NULL) { s_pfnPacketSelectionCallback = pfnPacketSelectionCallback; }
		static void SetStripFlagCallback(StripFlagCallback* const pfnStripFlagCallback = NULL) { s_pfnStripFlagCallback = pfnStripFlagCallback; }

	private:
		
		static void StripifySubdividedMesh(const Tools_SubdividedMesh* pxMesh, Tools_EnvironmentProcessor_Mesh& xProcessedMesh);

		static void TriListRemoveSmallTriangles( Tools_EnvironmentProcessor_Mesh& xProcessedMesh, const float fMinArea );
		static void TriListSubdividedMesh(const Tools_SubdividedMesh* pxMesh, Tools_EnvironmentProcessor_Mesh& xProcessedMesh );

		static void ProcessVerticesAndEliminateDuplicates(Tools_SubdividedMesh& xMesh);

#if defined(ASURA_TARGET_SM3)
		static void CalculateTangentSpaceVectors(const Tools_EnvironmentProcessor_Vertex& rxVertex0, const Tools_EnvironmentProcessor_Vertex& rxVertex1, const Tools_EnvironmentProcessor_Vertex& rxVertex2, u_int uTextureIndex, Asura_Vector_3& xTangent, Asura_Vector_3& xBinormal);
		static void CalculateTriListTangentSpaceVectors(Tools_EnvironmentProcessor_Mesh& xProcessedMesh);
		static void CalculateTriStripTangentSpaceVectors(Tools_EnvironmentProcessor_Mesh& xProcessedMesh);
#endif	// defined(ASURA_TARGET_SM3)


		static Tools_EnvironmentProcessor_Mesh*		s_pxProcessedMesh;
		static u_int								s_uTotalNumberOfVertices;
		static u_int								s_uTotalNumberOfIndices;
		static u_int								s_uTotalNumberOfStrips;
		static u_int								s_uNumberOfMeshes;
		static bool									s_bBeenInitialised;

		static Tools_EnvironmentProcessor_Strip*	s_pxStripsSortedByMaterial;

		static Tools_EnvironmentProcessor_PosMesh**	s_ppxPosMesh;
		static bool									s_bBeenGrouped;
		static u_int								s_uTotalNumberOfPositions;

		static u_int								s_uTotalDuplicateSourceTris;
		static u_int								s_uTotalDegenerateSourceTris;
		static PROJECT_LIGHTING_FN					s_pfnProjectLightingCallback;

		static ColourAdjustmentCallback*			s_pfnColourAdjustmentCallback;
		static PacketSelectionCallback*             s_pfnPacketSelectionCallback;
		static StripFlagCallback*					s_pfnStripFlagCallback;
};


#endif//__TOOLS_ENVIRONMENTPROCESSOR_H__
