//
// Tools_HierarchyProcessor.cpp
//
// Created by BM - 15/04/2003 20:48:18
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "MeshProcessing/Tools_HierarchyProcessor.h"

#include "Mesh/Asura_Mesh.h"
#include "Animation/Asura_Hierarchy.h"
#include "MeshProcessing/Asura_TriStripper.h"
#include "Core/Asura_Material_System.h"


Tools_HierarchyProcessor_Strip*	Tools_HierarchyProcessor::s_pxProcessedStrip;
u_int							Tools_HierarchyProcessor::s_uTotalNumberOfVertices;
u_int							Tools_HierarchyProcessor::s_uTotalNumberOfIndices;
u_int							Tools_HierarchyProcessor::s_uTotalNumberOfTriangles;
u_int							Tools_HierarchyProcessor::s_uNumberOfStrips;
bool							Tools_HierarchyProcessor::s_bBeenInitialised = false;
bool							Tools_HierarchyProcessor::s_bRequiresVertexAlpha = false;
Tools_HierarchyProcessor_Strip*	Tools_HierarchyProcessor::s_pxCattedStrip = NULL;

Tools_HierarchyProcessor::ColourGenerationHook* Tools_HierarchyProcessor::s_pfnColourGenerationFunction = NULL;
Tools_HierarchyProcessor::VertexModificationHook* Tools_HierarchyProcessor::s_pfnVertexModificationFunction = NULL;

bool Tools_HierarchyProcessor::Initialise(const Asura_Hierarchy_Skin* pxSkin)
{
	if(!pxSkin)
	{
		Asura_Assert(pxSkin,"no skin to process");
		return false;
	}

	//Tools_Level_Export::UpdateStatus("Stripifying Hierarchy", 0);
	
	// reset bone positions etc. (hack because of const pointer from original export) 	
	Asura_Hierarchy_Skin* pxSkinCopy = (Asura_Hierarchy_Skin* ) pxSkin;
	pxSkinCopy->CalculateSkinNormalData();
	
	//reset totals
	s_uTotalNumberOfTriangles = 0;
	s_uTotalNumberOfVertices  = 0;
	s_uTotalNumberOfIndices   = 0;
	
	u_int uNumberOfStrips = pxSkin->NumberOfBones;
	s_uNumberOfStrips = uNumberOfStrips;

	s_bRequiresVertexAlpha = false;

	// create a number of strips to hold all the bone Hierarchys
	s_pxProcessedStrip = new Tools_HierarchyProcessor_Strip[uNumberOfStrips];
	
	for (u_int u=0; u<uNumberOfStrips; u++)
	{		
		Asura_Mesh* pxMesh = pxSkin->SectionData[u].Mesh;

		if(!pxMesh)
		{
			// KJL - 09/05/2003 12:15:02 - I originally asserted on this but actually it's quite common.
			//Asura_Assert(pxMesh,( "section with no mesh in hierarchy <%s>", pxSkin->Name.GetString() ));
		
			s_pxProcessedStrip[u].m_iMaterialID = ASURA_MATERIAL_NONE;
			s_pxProcessedStrip[u].m_pusIndices = 0;
			s_pxProcessedStrip[u].m_pxVertices = 0;
			s_pxProcessedStrip[u].m_uNumberOfIndices = 0;
			s_pxProcessedStrip[u].m_uNumberOfTriangles = 0;
			s_pxProcessedStrip[u].m_uNumberOfVertices = 0;
		}
		else 
		{
			MeshProcess(pxMesh,u);
		}		
	}

	s_bBeenInitialised = true;

	//Tools_Level_Export::UpdateStatus("Stripifying Hierarchy", 100);
	s_pxCattedStrip = NULL;
	return true;
}


bool Tools_HierarchyProcessor::MeshProcess(Asura_Mesh* pxMesh, u_int uStrip)
{
	Asura_Assert(pxMesh,"no mesh passed to MeshProcess");

	//Allow for enough space in case all triangles are double sided. RAR 30/01/2004
	u_int uTotalIndices = pxMesh->NumberOfTriangles*3*2;
	u_int uNumberOfUniqueVertices = 0;
	u_int uIndex = 0;
	u_int uNumberOfTriangles = 0;
	bool bHasMaterial = false;
	bool bMaterialIsTransparent = false;

	Tools_HierarchyProcessor_Vertex* pxWorkspaceVertices = new Tools_HierarchyProcessor_Vertex[uTotalIndices];
	unsigned short* usIndexData = new unsigned short[uTotalIndices];

	// fill up the workspace vertex buffer
	for (int iPoly=0; iPoly<pxMesh->NumberOfPolygons; iPoly++)
	{	
		int iIndexNumber[4];
		
		int iPolyFlags = 0;

		if (pxMesh->PolygonFlagDataPtr)
		{
			iPolyFlags = pxMesh->PolygonFlagDataPtr[iPoly].Flags;
		}

		if (iPolyFlags & AsuraPolygonFlag_DontRender) continue;

		bool bDoubleSided = (iPolyFlags & AsuraPolygonFlag_DoubleSided) ? true : false;

		if(pxMesh->TextureDataPtr[iPoly].m_iMaterialID != ASURA_MATERIAL_NONE)
		{
			bHasMaterial = true;
			if(Asura_Material_System::IsMaterialTransparent(pxMesh->TextureDataPtr[iPoly].m_iMaterialID))
			{
				bMaterialIsTransparent = true;
			}
		}
		

		//If double sided we will need to do 2 passes of adding the vertices , with
		//normals and vertex ordering being reversed for the second side of the polygons
		//RAR 30/01/2004
		for( int iMirror=0;iMirror < (bDoubleSided ? 2 : 1);iMirror++)
		{
			int v=0;
			do
			{
				if(!pxMesh->AreNormalsValid())
				{
					pxMesh->CreateNormals();
				}			
		
				int vID =  pxMesh->PolygonDataPtr[iPoly].VertexID[v];
				Asura_Vector_3& xPosition = pxMesh->VertexDataPtr[vID].Position;
				Asura_Vector_3& xNormal = pxMesh->NormalsDataPtr[iPoly].VertexNormal[v];

				Tools_HierarchyProcessor_Vertex xVertexData;

				xVertexData.m_fX = xPosition.x;
				xVertexData.m_fY = xPosition.y;
				xVertexData.m_fZ = xPosition.z;

				xVertexData.m_fNX = xNormal.x;
				xVertexData.m_fNY = xNormal.y;
				xVertexData.m_fNZ = xNormal.z;
				if(iMirror)
				{
					xVertexData.m_fNX  *= -1.0f; 
					xVertexData.m_fNY  *= -1.0f; 
					xVertexData.m_fNZ  *= -1.0f; 
				}

				
				const Asura_Mesh_TextureInfo* pxTexInfo = pxMesh->GetTextureInfoForExport(iPoly);
				xVertexData.m_fU = pxTexInfo->U[0][v];
				xVertexData.m_fV = pxTexInfo->V[0][v];
				/*
				xVertexData.m_fU = pxMesh->TextureDataPtr[iPoly].U[v];
				xVertexData.m_fV = pxMesh->TextureDataPtr[iPoly].V[v];
				*/
				//xVertexData.m_fAlpha = pxMesh->TextureDataPtr[iPoly].m_afAlpha[v];


#ifdef TOOLS_HIERARCHY_PROCESSOR_USE_VERTEX_FLAGS
				xVertexData.m_uFlags = pxMesh->GetVertexFlags(vID);
#endif

				// (SB - 21/03/07) Call colour generation function if set
				if (NULL != s_pfnColourGenerationFunction)
				{
					xVertexData.m_uRGBAColour = s_pfnColourGenerationFunction(*pxMesh, iPoly, v);
				}
				else
				{
					xVertexData.m_uRGBAColour = 0xff000000;
				}

				// (SB - 29/05/07) Call vertex modification function to override the vertex
				if (NULL != s_pfnVertexModificationFunction)
				{
					s_pfnVertexModificationFunction(xVertexData);
				}

				// work out which vertices are duplicates
				int i = uNumberOfUniqueVertices;
				bool bVertexNotUnique = false;
				while(i)
				{
					i--;

					if(pxWorkspaceVertices[i] == xVertexData)
					{
						bVertexNotUnique = true;
						break;
					}
				}
	
				if (bVertexNotUnique)
				{
					iIndexNumber[v] = i;
				}
				else
				{
					iIndexNumber[v] = uNumberOfUniqueVertices;

					pxWorkspaceVertices[uNumberOfUniqueVertices] = xVertexData;
					uNumberOfUniqueVertices++;
				}
				v++;
			}
		
			while(v<4 && pxMesh->PolygonDataPtr[iPoly].VertexID[v]!=0xffff);

			if(iMirror)
			{
				//reverse of double sided polygon , so reverse the vertex ordering.
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[1]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
	
				//take account of quads
				if (pxMesh->PolygonDataPtr[iPoly].IsQuad())
				{
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[3]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
				}
			}
			else
			{
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[1]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
	
				//take account of quads
				if (pxMesh->PolygonDataPtr[iPoly].IsQuad())
				{
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[3]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
				}
			}
			uNumberOfTriangles += pxMesh->PolygonDataPtr[iPoly].IsQuad() ? 2 : 1;
		}
	}

	if(!uNumberOfUniqueVertices)
	{
		//all polygons were marked don't render , or otherwise discarded. RAR 24/05/04
		s_pxProcessedStrip[uStrip].m_iMaterialID = ASURA_MATERIAL_NONE;
		s_pxProcessedStrip[uStrip].m_pusIndices = 0;
		s_pxProcessedStrip[uStrip].m_pxVertices = 0;
		s_pxProcessedStrip[uStrip].m_uNumberOfIndices = 0;
		s_pxProcessedStrip[uStrip].m_uNumberOfTriangles = 0;
		s_pxProcessedStrip[uStrip].m_uNumberOfVertices = 0;

		delete [] usIndexData;
		delete [] pxWorkspaceVertices;


		return true;
	}

	
	// test
	for(u_int u=0; u<uIndex; u++)
	{
		Asura_Assert(usIndexData[u]>=0,"");
		Asura_Assert(usIndexData[u]<uNumberOfUniqueVertices,"");
	}

	u_int uNumberOfVertices = uNumberOfUniqueVertices;

    Tools_TriStripper::Initialise();
    Tools_TriStripper::SetOutputType( Tools_TriStripper::OUTPUT_TRISTRIP_WITH_RESET_INDICES );

	u_short* pusStripIndices;
    u_int uNumberOfStrippedIndices = Tools_TriStripper::CreateStrips( uNumberOfTriangles, usIndexData, &pusStripIndices );

	u_short* pusPermutationTable;
    Tools_TriStripper::ComputeVertexPermutation( uNumberOfStrippedIndices, pusStripIndices, uNumberOfVertices, &pusPermutationTable );

    Tools_TriStripper::Shutdown();

	uNumberOfTriangles = uNumberOfStrippedIndices-2;

	// create new strip for putting data in
	s_pxProcessedStrip[uStrip].m_pxVertices = new Tools_HierarchyProcessor_Vertex[uNumberOfUniqueVertices];
	s_pxProcessedStrip[uStrip].m_pusIndices = pusStripIndices;

	// bounding box data
	//s_pxProcessedStrip[uStrip].m_xBBMin.x = pxMesh->BoundingBox.MinX;
	//s_pxProcessedStrip[uStrip].m_xBBMin.y = pxMesh->BoundingBox.MinY;
	//s_pxProcessedStrip[uStrip].m_xBBMin.z = pxMesh->BoundingBox.MinZ;

	//s_pxProcessedStrip[uStrip].m_xBBMax.x = pxMesh->BoundingBox.MaxX;
	//s_pxProcessedStrip[uStrip].m_xBBMax.y = pxMesh->BoundingBox.MaxY;
	//s_pxProcessedStrip[uStrip].m_xBBMax.z = pxMesh->BoundingBox.MaxZ;

	//s_pxProcessedStrip[uStrip].m_iMaterialID			= pxMesh->TextureDataPtr[0].m_iMaterialID;	
	s_pxProcessedStrip[uStrip].m_uNumberOfIndices	= uNumberOfStrippedIndices;
	s_pxProcessedStrip[uStrip].m_uNumberOfTriangles	= uNumberOfTriangles;
	s_pxProcessedStrip[uStrip].m_uNumberOfVertices	= uNumberOfVertices;
	
	// copy all the indexed vertex data
	for(u_int u=0; u<uNumberOfUniqueVertices; u++)
	{	
		u_short usIndex = pusPermutationTable[u];
		s_pxProcessedStrip[uStrip].m_pxVertices[u] = pxWorkspaceVertices[usIndex];

		//s_pxProcessedStrip[uStrip].m_pxVertices[u].m_fAlpha = pxWorkspaceVertices[usIndex].m_fAlpha;
		/*if(bMaterialIsTransparent && (s_pxProcessedStrip[uStrip].m_pxVertices[u].m_fAlpha != 1.f))
		{
			// the material's flagged as transparent and the vertex isn't opaque
			s_bRequiresVertexAlpha = true;
		}*/
	}	

	//s_bBeenInitialised = true;

	// BM - 02/04/2003 15:47:16 -  - We only have one texture per Hierarchy at 
	// the moment. If we ever have more than one this will need to be changed.
	// The conversion mechanism means that the exported material index will be 0
	s_pxProcessedStrip[uStrip].m_iMaterialID = bHasMaterial ? 0 : -1;
	
	// add up totals
	s_uTotalNumberOfTriangles += uNumberOfTriangles;
	s_uTotalNumberOfVertices  += uNumberOfVertices;
	s_uTotalNumberOfIndices   += uNumberOfStrippedIndices;

	// delete temp data
	delete [] usIndexData;
	delete [] pxWorkspaceVertices;
	delete [] pusPermutationTable;

	return true;
}	


void Tools_HierarchyProcessor::Shutdown(void)
{
	if (s_bBeenInitialised)
	{
		delete [] s_pxProcessedStrip;
	
		if(s_pxCattedStrip)
		{
			delete s_pxCattedStrip;
		}
		s_bBeenInitialised = false;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
	}	
}

const Tools_HierarchyProcessor_Strip* Tools_HierarchyProcessor::GetProcessedStrip(u_int u)
{
	if (s_bBeenInitialised && u < s_uNumberOfStrips)
	{
			return &s_pxProcessedStrip[u];
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_HierarchyProcessor::GetTotalNumberOfVertices()
{
	if (s_bBeenInitialised)
	{
			return s_uTotalNumberOfVertices;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_HierarchyProcessor::GetTotalNumberOfIndices()
{
	if (s_bBeenInitialised)
	{
			return s_uTotalNumberOfIndices;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_HierarchyProcessor::GetTotalNumberOfStrips()
{
	if (s_bBeenInitialised)
	{
			return s_uNumberOfStrips;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

bool Tools_HierarchyProcessor::ConcatenateStrips()
{
	if(s_bBeenInitialised)
	{
		s_pxCattedStrip = new Tools_HierarchyProcessor_Strip;
		s_pxCattedStrip->m_iMaterialID = s_pxProcessedStrip[0].m_iMaterialID;
		s_pxCattedStrip->m_uNumberOfIndices = s_uTotalNumberOfIndices;
		s_pxCattedStrip->m_uNumberOfTriangles = s_uTotalNumberOfTriangles;
		s_pxCattedStrip->m_uNumberOfVertices = s_uTotalNumberOfVertices;
		s_pxCattedStrip->m_pxVertices = new Tools_HierarchyProcessor_Vertex[s_uTotalNumberOfVertices];
		s_pxCattedStrip->m_pusIndices = new u_short[s_uTotalNumberOfIndices];
		unsigned int uBaseVertex = 0;
		unsigned int uBaseIndex = 0;
		for(unsigned int uStrip = 0; uStrip < s_uNumberOfStrips; uStrip++)
		{
			Tools_HierarchyProcessor_Strip* pxStrip = s_pxProcessedStrip + uStrip;
			for(unsigned int uVertex = 0; uVertex < pxStrip->m_uNumberOfVertices; uVertex++)
			{
				s_pxCattedStrip->m_pxVertices[uVertex + uBaseVertex] = pxStrip->m_pxVertices[uVertex];
			}
			for(unsigned int uIndex = 0; uIndex < pxStrip->m_uNumberOfIndices; uIndex++)
			{
				s_pxCattedStrip->m_pusIndices[uIndex + uBaseIndex] = unsigned short(pxStrip->m_pusIndices[uIndex] + uBaseVertex);
			}
			uBaseVertex += pxStrip->m_uNumberOfVertices;
			uBaseIndex += pxStrip->m_uNumberOfIndices;
		}
	}
	else
	{
		Asura_Assert(s_bBeenInitialised, "Hierarchy processor not yet initialised.");
		return false;
	}
	return true;
}

const Tools_HierarchyProcessor_Strip* Tools_HierarchyProcessor::GetCattedStrip()
{
	Asura_Assert(s_bBeenInitialised, "Hierarchy processor not yet initialised.");
	if(s_bBeenInitialised)
	{
		if(!s_pxCattedStrip)
		{
			// Try to cat if not done already
			if(!ConcatenateStrips())
			{
				return NULL;
			}
		}
		return s_pxCattedStrip;
	}
	return NULL;
}
