//
// Tools_HierarchyProcessor.h
//
// Created by BM - 15/04/2003 20:48:23
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_HIERARCHYPROCESSOR_H__
#define __TOOLS_HIERARCHYPROCESSOR_H__

#include "Core/Asura_Maths.h"
class Asura_Mesh;
class Asura_Hierarchy_Skin;

const float fTOOLS_HIERARCHYPROCESSOR_VERTEX_EPSILON = 0.005f;
const float fTOOLS_HIERARCHYPROCESSOR_VERTEX_EPSILON_SQD = 0.005f * 0.005f;

class Tools_HierarchyProcessor_Vertex
{
	public:
		float 			m_fX;
		float			m_fY;
		float			m_fZ;
		float			m_fNX;
		float			m_fNY;
		float			m_fNZ;
		float			m_fU;
		float			m_fV;
		u_int			m_uRGBAColour;
#ifdef TOOLS_HIERARCHY_PROCESSOR_USE_VERTEX_FLAGS
		u_int			m_uFlags;
#endif

		//float			m_fAlpha;

		inline bool operator == ( const Tools_HierarchyProcessor_Vertex& v) const
		{
			const Asura_Vector_3 xPos( m_fX, m_fY, m_fZ );
			const Asura_Vector_3 xOtherPos( v.m_fX, v.m_fY, v.m_fZ );

			const Asura_Vector_3 xNormal( m_fNX, m_fNY, m_fNZ );
			const Asura_Vector_3 xOtherNormal( v.m_fNX, v.m_fNY, v.m_fNZ );

			Asura_Vector_2 xUVDelta( m_fU, m_fV );
			xUVDelta.x -= v.m_fU;
			xUVDelta.y -= v.m_fV;

			return xPos.ApproximatelyEquals( xOtherPos )
				&& xNormal.ApproximatelyEquals( xOtherNormal )
				&& xUVDelta.MagnitudeSquared() < fTOOLS_HIERARCHYPROCESSOR_VERTEX_EPSILON_SQD
				&& m_uRGBAColour == v.m_uRGBAColour
#ifdef TOOLS_HIERARCHY_PROCESSOR_USE_VERTEX_FLAGS
				&& m_uFlags == v.m_uFlags
#endif
			 /* && Asura_Maths::Abs( m_fAlpha - v.m_fAlpha ) < fTOOLS_HIERARCHYPROCESSOR_VERTEX_EPSILON */;
		}
};

class Tools_HierarchyProcessor_Strip
{
	public:
		
		Tools_HierarchyProcessor_Strip()
		{
			m_pxVertices = 0;
			m_pusIndices = 0;
		}
		
		~Tools_HierarchyProcessor_Strip()
		{
			delete [] m_pxVertices;
			delete [] m_pusIndices;
		}

		u_int								m_uNumberOfIndices;
		u_int								m_uNumberOfTriangles;
		u_int								m_uNumberOfVertices;
		//Asura_Vector_3						m_xBBMin;
		//Asura_Vector_3						m_xBBMax;

		int									m_iMaterialID;

		Tools_HierarchyProcessor_Vertex*	m_pxVertices;
		u_short*							m_pusIndices;
};



class Tools_HierarchyProcessor
{
	public:
		typedef u_int (ColourGenerationHook)(const Asura_Mesh& xMesh, const int iPoly, const int iVertex);
		typedef void (VertexModificationHook)(Tools_HierarchyProcessor_Vertex& xVertex);

		static bool Initialise(const Asura_Hierarchy_Skin* pxSkin);
		static void Shutdown(void);

		static const Tools_HierarchyProcessor_Strip* GetProcessedStrip(u_int uIndex);
		static u_int GetTotalNumberOfVertices(void);
		static u_int GetTotalNumberOfIndices(void);
		static u_int GetTotalNumberOfStrips(void);
		static bool ConcatenateStrips(void);
		inline static bool DoesRequireVertexAlpha(void) { return false; } //s_bRequiresVertexAlpha; }
		static const Tools_HierarchyProcessor_Strip* GetCattedStrip();
	
		static void SetColourGenerationCallback(ColourGenerationHook* const pfnColourGenerationFunction = NULL) { s_pfnColourGenerationFunction = pfnColourGenerationFunction; }
		static void SetVertexModificationCallback(VertexModificationHook* const pfnVertexModificationFunction = NULL) { s_pfnVertexModificationFunction = pfnVertexModificationFunction; }

		static Tools_HierarchyProcessor_Strip*	s_pxProcessedStrip;
		static u_int							s_uTotalNumberOfVertices;
		static u_int							s_uTotalNumberOfIndices;
		static u_int							s_uTotalNumberOfTriangles;
		static u_int							s_uNumberOfStrips;
		static bool								s_bBeenInitialised;
		static bool								s_bRequiresVertexAlpha;

	private:

		static bool MeshProcess(Asura_Mesh* pxMesh, u_int uStrip);
		static Tools_HierarchyProcessor_Strip* s_pxCattedStrip;

		static ColourGenerationHook* s_pfnColourGenerationFunction;
		static VertexModificationHook* s_pfnVertexModificationFunction;
};

#endif//__TOOLS_HIERARCHYPROCESSOR_H__
