//
// Tools_HierarchyProcessor_MultiMaterial.cpp
//
// Created by BM - 15/04/2003 20:48:18
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "MeshProcessing/Tools_HierarchyProcessor_MultiMaterial.h"

#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"
#include "MeshEdit/Tools_Mode_MeshEdit.h"
#include "Animation/Asura_Hierarchy.h"
#include "MeshProcessing/Asura_TriStripper.h"
#include "Core/Asura_Material_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "MeshProcessing/Tools_MeshRestartIndex.h"

template<typename T>
void swap(T& x, T& y)
{
	T temp = x;
	x = y;
	y = temp;
}

List<Tools_HierarchyProcessor_MultiMaterial_Strip*> Tools_HierarchyProcessor_MultiMaterial::s_xProcessedStrips;
u_int							Tools_HierarchyProcessor_MultiMaterial::s_uTotalNumberOfVertices;
u_int							Tools_HierarchyProcessor_MultiMaterial::s_uTotalNumberOfIndices;
u_int							Tools_HierarchyProcessor_MultiMaterial::s_uTotalNumberOfTriangles;
u_int							Tools_HierarchyProcessor_MultiMaterial::s_uTotalNumberOfStrips;
bool							Tools_HierarchyProcessor_MultiMaterial::s_bBeenInitialised = false;
bool							Tools_HierarchyProcessor_MultiMaterial::s_bRequiresVertexAlpha = false;
int								Tools_HierarchyProcessor_MultiMaterial::s_iCurrentMaterial = ASURA_MATERIAL_NONE;
const Asura_Hierarchy_Skin*		Tools_HierarchyProcessor_MultiMaterial::s_pxCurrentSkin = NULL;
Tools_HierarchyProcessor_MultiMaterial_Strip* Tools_HierarchyProcessor_MultiMaterial::s_pxCattedStrip = NULL;

Tools_HierarchyProcessor_MultiMaterial::ColourGenerationHook* Tools_HierarchyProcessor_MultiMaterial::s_pfnColourGenerationFunction = NULL;
Tools_HierarchyProcessor_MultiMaterial::VertexModificationHook* Tools_HierarchyProcessor_MultiMaterial::s_pfnVertexModificationFunction = NULL;

bool Tools_HierarchyProcessor_MultiMaterial::MaterialSelectorFunction(Asura_Mesh* pxMesh, int iPolyIndex)
{
	if(pxMesh->TextureDataValid)
	{
		return s_iCurrentMaterial == pxMesh->TextureDataPtr[iPolyIndex].m_iMaterialID;
	}
	else
	{
		return s_iCurrentMaterial == ASURA_MATERIAL_NONE;
	}
}

bool Tools_HierarchyProcessor_MultiMaterial::Initialise(const Asura_Hierarchy_Skin* pxSkin, bool bStripify /*=true*/)
{
	if(!pxSkin)
	{
		Asura_Assert(pxSkin,"no skin to process");
		return false;
	}

	s_pxCurrentSkin = pxSkin;

	//Tools_Level_Export::UpdateStatus("Stripifying Hierarchy", 0);
	
	// reset bone positions etc. (hack because of const pointer from original export) 	
	Asura_Hierarchy_Skin* pxSkinCopy = (Asura_Hierarchy_Skin* ) pxSkin;
	pxSkinCopy->CalculateSkinNormalData();
	
	//reset totals
	s_uTotalNumberOfTriangles = 0;
	s_uTotalNumberOfVertices  = 0;
	s_uTotalNumberOfIndices   = 0;
	s_uTotalNumberOfStrips	  = 0;

	s_bRequiresVertexAlpha = false;
	
	// Enumerate materials

	List<Asura_Mesh*> xMeshList;
	List<int> xUsedMaterials;

	for (int u=0; u<pxSkin->NumberOfBones; u++)
	{		
		Asura_Mesh* pxMesh = pxSkin->SectionData[u].Mesh;

		if(pxMesh)
		{
			xMeshList.add_entry(pxMesh);

			if(pxMesh->TextureDataValid)
			{
				for(int iPoly = 0; iPoly < pxMesh->NumberOfPolygons; ++iPoly)
				{
					if(!xUsedMaterials.contains(pxMesh->TextureDataPtr[iPoly].m_iMaterialID))
					{
						xUsedMaterials.add_entry(pxMesh->TextureDataPtr[iPoly].m_iMaterialID);
					}
				}
			}
			else
			{
				// assume the mesh has ASURA_MATERIAL_NONE
				if(!xUsedMaterials.contains(ASURA_MATERIAL_NONE))
				{
					xUsedMaterials.add_entry(ASURA_MATERIAL_NONE);
				}
			}
		}		
	}

	// build a strip for each material

	Asura_Mesh* pxTempMesh = Asura_Mesh_System::GetMesh("tempMultiMaterialCollectorMesh", true);

	// We do two passes; first process opaque strips, then transparent ones.
	// This ensures they get rendered roughly correctly

	for( u_int uPass = 0; uPass < 2; ++uPass )
	{
		for(int iMaterial = 0; iMaterial < xUsedMaterials.size(); ++iMaterial)
		{
			// See if we should be handling this material in this pass

			const int iMatID = xUsedMaterials[ iMaterial ];
			const bool bTranslucent = Asura_Material_System::IsMaterialTransparent( iMatID )
								   || Asura_Material_System::IsMaterialPS2AdditiveAlpha( iMatID );

			if( ((uPass == 0) && bTranslucent) || ((uPass == 1) && !bTranslucent) ) continue;

			// Do the processing

			s_iCurrentMaterial = xUsedMaterials[iMaterial];

			for(unsigned short iBone = 0; iBone < pxSkin->NumberOfBones; ++iBone)
			{
				if(!pxSkin->SectionData[iBone].Mesh) continue;

				// extract polygons with the correct material from this bone's mesh into tempMesh
				Tools_Mode_MeshEdit::CopyMesh(pxSkin->SectionData[iBone].Mesh, pxTempMesh, MaterialSelectorFunction);

				if(!pxTempMesh->NumberOfPolygons) continue; // no polys with this material at this bone

				// Split everything down into triangles
				int* piPolyConv1, *piPolyConv2;
				Tools_Mode_MeshEdit::UnmergePolygons(pxTempMesh, (bool (*)(Asura_Mesh*, int))NULL, &piPolyConv1, &piPolyConv2);
				delete[] piPolyConv1;
				delete[] piPolyConv2;
				Asura_Assert(pxTempMesh->NumberOfPolygons == pxTempMesh->NumberOfTriangles, "Didn't unmerge everything");

				// create a strip from these polygons
				MeshProcess(pxTempMesh, iBone, bStripify);
			}
		}
	}

	Asura_Mesh_System::DeleteMesh(pxTempMesh);

	s_bBeenInitialised = true;

	//Tools_Level_Export::UpdateStatus("Stripifying Hierarchy", 100);
	return true;
}

void Tools_HierarchyProcessor_MultiMaterial::ConvertMaterialsForExport()
{
	List<Asura_Mesh*> xMeshList;

	for (int u=0; u<s_pxCurrentSkin->NumberOfBones; u++)
	{		
		Asura_Mesh* pxMesh = s_pxCurrentSkin->SectionData[u].Mesh;

		if(pxMesh)
		{
			xMeshList.add_entry(pxMesh);
		}
	}

	int aiMaterialMapping[ASURA_MAX_NO_OF_MATERIALS];
	Tools_SaveAsura::PrepareMeshMaterialConversionLists(xMeshList, aiMaterialMapping, 0);
	for(CLIF<Tools_HierarchyProcessor_MultiMaterial_Strip*> it(&s_xProcessedStrips); !it.done(); it.next())
	{
		Tools_HierarchyProcessor_MultiMaterial_Strip* pxStrip = it();
		const int iMaterialID = pxStrip->m_iMaterialID;

		int iMaterialArrayIdx = Asura_Material_System::TranslateMaterialIDToArrayIdx( iMaterialID );
        Asura_Assert( ASURA_MATERIAL_NONE == iMaterialArrayIdx || (0 <= iMaterialArrayIdx && iMaterialArrayIdx < ASURA_MAX_NO_OF_MATERIALS), "" );

		pxStrip->m_iMaterialID = ( iMaterialID != ASURA_MATERIAL_NONE ) ? aiMaterialMapping[iMaterialArrayIdx] : Tools_Level_Export::GetUntexturedMaterialIndex();
	}
}

bool Tools_HierarchyProcessor_MultiMaterial::MeshProcess(Asura_Mesh* pxMesh, unsigned short usBoneIndex, bool bStripify)
{
	Asura_Assert(pxMesh,"no mesh passed to MeshProcess");

	//Allow for enough space in case all triangles are double sided. RAR 30/01/2004
	u_int uTotalIndices = pxMesh->NumberOfTriangles*3*2;
	u_int uNumberOfUniqueVertices = 0;
	u_int uIndex = 0;
	u_int uNumberOfTriangles = 0;

	Tools_HierarchyProcessor_MultiMaterial_Vertex* pxWorkspaceVertices = new Tools_HierarchyProcessor_MultiMaterial_Vertex[uTotalIndices];
	unsigned short* usIndexData = new unsigned short[uTotalIndices];

	// fill up the workspace vertex buffer
	for (int iPoly=0; iPoly<pxMesh->NumberOfPolygons; iPoly++)
	{	
		int iIndexNumber[4];
		
		int iPolyFlags = 0;

		if (pxMesh->PolygonFlagDataPtr)
		{
			iPolyFlags = pxMesh->PolygonFlagDataPtr[iPoly].Flags;
		}

		if (iPolyFlags & AsuraPolygonFlag_DontRender) continue;

		bool bDoubleSided = (iPolyFlags & AsuraPolygonFlag_DoubleSided) ? true : false;

		//If double sided we will need to do 2 passes of adding the vertices , with
		//normals and vertex ordering being reversed for the second side of the polygons
		//RAR 30/01/2004
		for( int iMirror=0;iMirror < (bDoubleSided ? 2 : 1);iMirror++)
		{
			int v=0;
			do
			{
				if(!pxMesh->AreNormalsValid())
				{
					pxMesh->CreateNormals();
				}			
		
				int vID =  pxMesh->PolygonDataPtr[iPoly].VertexID[v];
				Asura_Vector_3& xPosition = pxMesh->VertexDataPtr[vID].Position;
				Asura_Vector_3& xNormal = pxMesh->NormalsDataPtr[iPoly].VertexNormal[v];

				Tools_HierarchyProcessor_MultiMaterial_Vertex xVertexData;

				xVertexData.m_xPosition = xPosition;

				xVertexData.m_xNormal = xNormal;
				if(iMirror)
				{
					xVertexData.m_xNormal  *= -1.0f; 
				}

				
				const Asura_Mesh_TextureInfo* pxTexInfo = pxMesh->GetTextureInfoForExport(iPoly);
				xVertexData.m_fU0 = pxTexInfo->U[0][v];
				xVertexData.m_fV0 = pxTexInfo->V[0][v];

				xVertexData.m_fAlpha = pxMesh->TextureDataPtr[iPoly].m_afAlpha[v];
				if( (xVertexData.m_fAlpha < 1.0f)
					&&
					(Asura_Material_System::IsMaterialTransparent(s_iCurrentMaterial) || Asura_Material_System::IsMaterialPS2AdditiveAlpha(s_iCurrentMaterial)) )
				{
					s_bRequiresVertexAlpha = true;
				}

				xVertexData.m_usBoneIndex = usBoneIndex;

				// (SB - 30/05/07) Call colour generation function if set - matches Tools_HierarchyProcessor
				if (NULL != s_pfnColourGenerationFunction)
				{
					xVertexData.m_uARGBColour = s_pfnColourGenerationFunction(*pxMesh, iPoly, v);
				}
				else
				{
					xVertexData.m_uARGBColour = 0xff000000;
				}

				// (SB - 30/05/07) Call vertex modification function to override the vertex - matches Tools_HierarchyProcessor
				if (NULL != s_pfnVertexModificationFunction)
				{
					s_pfnVertexModificationFunction(xVertexData);
				}

				/*
				xVertexData.m_fU = pxMesh->TextureDataPtr[iPoly].U[v];
				xVertexData.m_fV = pxMesh->TextureDataPtr[iPoly].V[v];
				*/
				//xVertexData.m_fAlpha = pxMesh->TextureDataPtr[iPoly].m_afAlpha[v];

				// work out which vertices are duplicates
				int i = uNumberOfUniqueVertices;
				bool bVertexNotUnique = false;
				while(i)
				{
					i--;

					if(pxWorkspaceVertices[i] == xVertexData)
					{
						bVertexNotUnique = true;
						break;
					}
				}
	
				if (bVertexNotUnique)
				{
					iIndexNumber[v] = i;
				}
				else
				{
					iIndexNumber[v] = uNumberOfUniqueVertices;

					pxWorkspaceVertices[uNumberOfUniqueVertices] = xVertexData;
					uNumberOfUniqueVertices++;
				}
				v++;
			}
		
			while(v<4 && pxMesh->PolygonDataPtr[iPoly].VertexID[v]!=0xffff);

			if(iMirror)
			{
				//reverse of double sided polygon , so reverse the vertex ordering.
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[1]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
	
				//take account of quads
				if (pxMesh->PolygonDataPtr[iPoly].IsQuad())
				{
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[3]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
				}
			}
			else
			{
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[1]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
	
				//take account of quads
				if (pxMesh->PolygonDataPtr[iPoly].IsQuad())
				{
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[3]);
					usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
				}
			}
			uNumberOfTriangles += pxMesh->PolygonDataPtr[iPoly].IsQuad() ? 2 : 1;
		}
	}

	if(!uNumberOfUniqueVertices)
	{
		//all polygons were marked don't render , or otherwise discarded. RAR 24/05/04
		delete [] usIndexData;
		delete [] pxWorkspaceVertices;

		return true;
	}

	
	// test
	for(u_int u=0; u<uIndex; u++)
	{
		Asura_Assert(usIndexData[u]>=0,"");
		Asura_Assert(usIndexData[u]<uNumberOfUniqueVertices,"");
	}

	u_int uNumberOfVertices = uNumberOfUniqueVertices;

	u_short* pusStripIndices = 0;
	u_int uNumberOfStrippedIndices = 0;
	u_short* pusPermutationTable = 0;

    if(bStripify)
	{
    	Tools_TriStripper::Initialise();
    	Tools_TriStripper::SetOutputType( Tools_TriStripper::OUTPUT_TRISTRIP_WITH_RESET_INDICES );

    	uNumberOfStrippedIndices = Tools_TriStripper::CreateStrips( uNumberOfTriangles, usIndexData, &pusStripIndices );

    	Tools_TriStripper::ComputeVertexPermutation( uNumberOfStrippedIndices, pusStripIndices, uNumberOfVertices, &pusPermutationTable );

    	Tools_TriStripper::Shutdown();

		u_short usRestartValue;
		const bool bUseRestartIndex = Tools_MeshRestartIndex::IsValidForCurrentPlatform( usRestartValue );
		if( !bUseRestartIndex )
		{
			uNumberOfTriangles = uNumberOfStrippedIndices - 2;
		}
	}
	else
	{
		//not stripifying , so just transfer the index list since it will be in the correct format. RAR 05/01/2007
		uNumberOfStrippedIndices = uNumberOfTriangles*3;
		pusStripIndices = usIndexData;
		usIndexData = 0;
	}

	// create new strip for putting data in
	Tools_HierarchyProcessor_MultiMaterial_Strip* pxNewStrip = new Tools_HierarchyProcessor_MultiMaterial_Strip();
	s_xProcessedStrips.add_entry(pxNewStrip);

	pxNewStrip->m_pxVertices = new Tools_HierarchyProcessor_MultiMaterial_Vertex[uNumberOfUniqueVertices];
	pxNewStrip->m_pusIndices = pusStripIndices;

	// bounding box data
	//s_pxProcessedStrip[uStrip].m_xBBMin.x = pxMesh->BoundingBox.MinX;
	//s_pxProcessedStrip[uStrip].m_xBBMin.y = pxMesh->BoundingBox.MinY;
	//s_pxProcessedStrip[uStrip].m_xBBMin.z = pxMesh->BoundingBox.MinZ;

	//s_pxProcessedStrip[uStrip].m_xBBMax.x = pxMesh->BoundingBox.MaxX;
	//s_pxProcessedStrip[uStrip].m_xBBMax.y = pxMesh->BoundingBox.MaxY;
	//s_pxProcessedStrip[uStrip].m_xBBMax.z = pxMesh->BoundingBox.MaxZ;

	// copy all the indexed vertex data
	for(u_int u=0; u<uNumberOfUniqueVertices; u++)
	{	
		u_short usIndex = pusPermutationTable ? pusPermutationTable[u] : (u_short)u;
		pxNewStrip->m_pxVertices[u] = pxWorkspaceVertices[usIndex];
	}	

	//s_pxProcessedStrip[uStrip].m_iMaterialID			= pxMesh->TextureDataPtr[0].m_iMaterialID;	
	pxNewStrip->m_uNumberOfIndices	= uNumberOfStrippedIndices;
	pxNewStrip->m_uNumberOfTriangles	= uNumberOfTriangles;
	pxNewStrip->m_uNumberOfVertices	= uNumberOfVertices;

	pxNewStrip->m_usBoneIndex = usBoneIndex;
	pxNewStrip->m_iMaterialID = s_iCurrentMaterial;

	//s_bBeenInitialised = true;

	// BM - 02/04/2003 15:47:16 -  - We only have one texture per Hierarchy at 
	// the moment. If we ever have more than one this will need to be changed.
	// The conversion mechanism means that the exported material index will be 0
	//s_pxProcessedStrip[uStrip].m_iMaterialID = bHasMaterial ? 0 : -1;
	
	// add up totals
	s_uTotalNumberOfTriangles += uNumberOfTriangles;
	s_uTotalNumberOfVertices  += uNumberOfVertices;
	s_uTotalNumberOfIndices   += uNumberOfStrippedIndices;
	s_uTotalNumberOfStrips	  += 1;

	// delete temp data
	delete [] usIndexData;
	delete [] pxWorkspaceVertices;
	delete [] pusPermutationTable;

	return true;
}	


void Tools_HierarchyProcessor_MultiMaterial::Shutdown(void)
{
	if (s_bBeenInitialised)
	{
		// delete all the processed strips
		while(s_xProcessedStrips.size())
		{
			delete s_xProcessedStrips.last_entry();
			s_xProcessedStrips.delete_last_entry();
		}
		delete s_pxCattedStrip;
		s_pxCattedStrip = NULL;
	
		s_bBeenInitialised = false;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
	}	
}

const Tools_HierarchyProcessor_MultiMaterial_Strip* Tools_HierarchyProcessor_MultiMaterial::GetProcessedStrip(u_int u)
{
	if (s_bBeenInitialised && u < s_uTotalNumberOfStrips)
	{
		return s_xProcessedStrips[u];
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_HierarchyProcessor_MultiMaterial::GetTotalNumberOfVertices()
{
	if (s_bBeenInitialised)
	{
		return s_uTotalNumberOfVertices;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_HierarchyProcessor_MultiMaterial::GetTotalNumberOfIndices()
{
	if (s_bBeenInitialised)
	{
		return s_uTotalNumberOfIndices;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_HierarchyProcessor_MultiMaterial::GetTotalNumberOfStrips()
{
	if (s_bBeenInitialised)
	{
		return s_uTotalNumberOfStrips;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

bool Tools_HierarchyProcessor_MultiMaterial::ConcatenateStrips()
{
	if(s_bBeenInitialised)
	{
		s_pxCattedStrip = new Tools_HierarchyProcessor_MultiMaterial_Strip;
		// Material is irrelevant for the catted strip - it's only used for the mesh data.
		s_pxCattedStrip->m_iMaterialID = ASURA_MATERIAL_NONE;
		s_pxCattedStrip->m_uNumberOfIndices = s_uTotalNumberOfIndices;
		s_pxCattedStrip->m_uNumberOfTriangles = s_uTotalNumberOfTriangles;
		s_pxCattedStrip->m_uNumberOfVertices = s_uTotalNumberOfVertices;
		s_pxCattedStrip->m_pxVertices = new Tools_HierarchyProcessor_MultiMaterial_Vertex[s_uTotalNumberOfVertices];
		s_pxCattedStrip->m_pusIndices = new u_short[s_uTotalNumberOfIndices];
		u_int uBaseVertex = 0;
		u_int uBaseIndex = 0;
		for(u_int uStrip = 0; uStrip < s_uTotalNumberOfStrips; ++uStrip)
		{
			const Tools_HierarchyProcessor_MultiMaterial_Strip* pxStrip = GetProcessedStrip(uStrip);
			for(u_int uVertex = 0; uVertex < pxStrip->m_uNumberOfVertices; ++uVertex)
			{
				s_pxCattedStrip->m_pxVertices[uVertex + uBaseVertex] = pxStrip->m_pxVertices[uVertex];
			}
			for(u_int uIndex = 0; uIndex < pxStrip->m_uNumberOfIndices; ++uIndex)
			{
				s_pxCattedStrip->m_pusIndices[uIndex + uBaseIndex] = static_cast<u_short>(pxStrip->m_pusIndices[uIndex] + uBaseVertex);
			}
			uBaseVertex += pxStrip->m_uNumberOfVertices;
			uBaseIndex += pxStrip->m_uNumberOfIndices;
		}
	}
	else
	{
		Asura_Assert(s_bBeenInitialised, "Hierarchy processor not yet initialised.");
		return false;
	}
	return true;
}

const Tools_HierarchyProcessor_MultiMaterial_Strip* Tools_HierarchyProcessor_MultiMaterial::GetCattedStrip()
{
	Asura_Assert(s_bBeenInitialised, "Hierarchy processor not yet initialised.");
	if(s_bBeenInitialised)
	{
		if(!s_pxCattedStrip)
		{
			// Try to cat if not done already
			if(!ConcatenateStrips())
			{
				return NULL;
			}
		}
		return s_pxCattedStrip;
	}
	return NULL;
}
