//
// Tools_HierarchyProcessor_MultiMaterial_NormalMapped.h
//
// Cloned by RJF from Tools_HierarchyProcessor.h 12-4-05
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_NORMALMAPPED_H__
#define __TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_NORMALMAPPED_H__

#include "Core/Asura_Maths.h"
#include "Misc/list_tem.hpp"
class Asura_Mesh;
class Asura_Hierarchy_Skin;

const float fTOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_NORMALMAPPED_VERTEX_EPSILON = 0.005f;
const float fTOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_NORMALMAPPED_VERTEX_EPSILON_SQD = 0.005f * 0.005f;

class Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Vertex
{
	public:
		Asura_Vector_3	m_xPosition;
		Asura_Vector_3	m_xNormal;
		float			m_fU0;
		float			m_fV0;
		float			m_fU1;
		float			m_fV1;

		float			m_fAlpha;
		unsigned short	m_usBoneIndex;
		Asura_Vector_3	m_xBinormal;
		Asura_Vector_3	m_xTangent;

		inline bool operator == ( const Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Vertex& v) const
		{
			Asura_Vector_2 xUVDelta0( m_fU0, m_fV0 );
			xUVDelta0.x -= v.m_fU0;
			xUVDelta0.y -= v.m_fV0;

			Asura_Vector_2 xUVDelta1( m_fU1, m_fV1 );
			xUVDelta1.x -= v.m_fU1;
			xUVDelta1.y -= v.m_fV1;


			// Normal, tangent and binormal should be created orthogonal, so only need to check if one is different

			return m_usBoneIndex == v.m_usBoneIndex
				&& Asura_Maths::Abs( m_fAlpha - v.m_fAlpha ) < fTOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_NORMALMAPPED_VERTEX_EPSILON
				&& xUVDelta0.MagnitudeSquared() < fTOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_NORMALMAPPED_VERTEX_EPSILON_SQD
				&& xUVDelta1.MagnitudeSquared() < fTOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_NORMALMAPPED_VERTEX_EPSILON_SQD
				&& m_xPosition.ApproximatelyEquals( v.m_xPosition )
				&& m_xNormal.ApproximatelyEquals( v.m_xNormal );
		}

		inline Asura_Hash_ID GetHashID( void ) const
		{
			// Hash into millimetre cubes.
			// This ignores a lot of the data, and will result in some different vertices having the same hash.
			// However, the point of the hash is simply to quickly file data - more in-depth checking is done later.
			const int iX = Asura_Maths::FloatToInt_Near(m_xPosition.x * 1000.f);
			const int iY = Asura_Maths::FloatToInt_Near(m_xPosition.y * 1000.f);
			const int iZ = Asura_Maths::FloatToInt_Near(m_xPosition.z * 1000.f);
			// Snaffled from Asura_Physics_SpatialHash
			const Asura_Hash_ID uHash = 7* iX + 503 * iY + 24847 * iZ;;
			return uHash;
		}

};

class Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Strip
{
	public:
		
		Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Strip()
		{
			m_pxVertices = 0;
			m_pusIndices = 0;
		}
		
		~Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Strip()
		{
			delete [] m_pxVertices;
			delete [] m_pusIndices;
		}

		u_int			m_uNumberOfIndices;
		u_int			m_uNumberOfTriangles;
		u_int			m_uNumberOfVertices;

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_Hash_ID	m_auMaterialResponseHashID[2];
#else
		int				m_iMaterialID;
#endif

		unsigned short	m_usBoneIndex;

		Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Vertex*	m_pxVertices;
		u_short*													m_pusIndices;
};

class Tools_HierarchyProcessor_MultiMaterial_NormalMapped
{
	public:
		static bool Initialise(const Asura_Hierarchy_Skin* pxSkin, bool bStripify = true);
		static void Shutdown(void);

		static const Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Strip* GetProcessedStrip(u_int uIndex);
		static u_int GetTotalNumberOfVertices(void);
		static u_int GetTotalNumberOfIndices(void);
		static u_int GetTotalNumberOfStrips(void);
		static const Asura_Hierarchy_Skin* GetCurrentSkin(void) { return s_pxCurrentSkin; }
		static void ConvertMaterialsForExport(void);
		inline static bool DoesRequireVertexAlpha(void) { return s_bRequiresVertexAlpha; }

		static bool HasHierarchyAnyNormalMappedMaterials(const Asura_Hierarchy_Skin* pxSkin);
		
		static u_int	s_uTotalNumberOfVertices;
		static u_int	s_uTotalNumberOfIndices;
		static u_int	s_uTotalNumberOfTriangles;
		static u_int	s_uTotalNumberOfStrips;
		static bool		s_bBeenInitialised;
		static bool		s_bRequiresVertexAlpha;
	
	private:

		static const Asura_Hierarchy_Skin* s_pxCurrentSkin;
		static bool MeshProcess(Asura_Mesh* pxMesh, unsigned short usBoneIndex, bool bStripify);
		static List<Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Strip*> s_xProcessedStrips;
		
#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		static Asura_Hash_ID s_auCurrentMaterialResponse[2];
#else
		static int s_iCurrentMaterial;
#endif

		static bool MaterialSelectorFunction(Asura_Mesh* pxMesh, int iPolyIndex);

		static void CalculateTangentSpaceVectors(const class Asura_Mesh* pxMesh, u_int uPoly, Asura_Vector_3& xTangent, Asura_Vector_3& xBinormal);
};




#endif//__TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_NORMALMAPPED_H__
