//
// Tools_ObjectProcessor.h
//
// Created by BM - 02/04/2003 15:19:34
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif

#ifndef __TOOLS_OBJECTPROCESSOR_H__
#define __TOOLS_OBJECTPROCESSOR_H__

#include "Core/Asura_Maths.h"
class Asura_Mesh;
class Asura_Hierarchy_Skin;

const float fTOOLS_OBJECTPROCESSOR_VERTEX_EPSILON_SQD = 0.005f * 0.005f;

class Tools_ObjectProcessor_Vertex
{
	public:
		float 			m_fX;
		float			m_fY;
		float			m_fZ;
		float			m_fNX;
		float			m_fNY;
		float			m_fNZ;
		float			m_fU;
		float			m_fV;

		inline bool operator == ( const Tools_ObjectProcessor_Vertex& v ) const
		{
			const Asura_Vector_3 xPos( m_fX, m_fY, m_fZ );
			const Asura_Vector_3 xOtherPos( v.m_fX, v.m_fY, v.m_fZ );

			const Asura_Vector_3 xNormal( m_fNX, m_fNY, m_fNZ );
			const Asura_Vector_3 xOtherNormal( v.m_fNX, v.m_fNY, v.m_fNZ );

			Asura_Vector_2 xUVDelta( m_fU, m_fV );
			xUVDelta.x -= v.m_fU;
			xUVDelta.y -= v.m_fV;

			return xPos.ApproximatelyEquals( xOtherPos )
				&& xNormal.ApproximatelyEquals( xOtherNormal )
				&& xUVDelta.MagnitudeSquared() < fTOOLS_OBJECTPROCESSOR_VERTEX_EPSILON_SQD;
		}
};

class Tools_ObjectProcessor_Strip
{
	public:
		
		Tools_ObjectProcessor_Strip()
		{
			m_pxVertices = 0;
			m_pusIndices = 0;
		}
		
		~Tools_ObjectProcessor_Strip()
		{
			delete [] m_pxVertices;
			delete [] m_pusIndices;
		}

		u_int								m_uNumberOfIndices;
		u_int								m_uNumberOfTriangles;
		u_int								m_uNumberOfVertices;
		Asura_Vector_3						m_xBBMin;
		Asura_Vector_3						m_xBBMax;

		int									m_iMaterialID;
		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		Asura_Hash_ID						m_auMaterialResponseHashID[2];
		#endif

		Tools_ObjectProcessor_Vertex*		m_pxVertices;
		u_short*							m_pusIndices;
};



class Tools_ObjectProcessor
{
	public:
		static bool Initialise(const Asura_Hierarchy_Skin* pxSkin);
		static bool Initialise(Asura_Mesh* pxMesh,Asura_Hash_ID uMatResponse1,Asura_Hash_ID uMatResponse2 ,bool bStripify = true);
		static bool Initialise(Asura_Mesh* pxMesh,int iMaterialIndex,bool bStripify = true);
		static void Shutdown(void);

		static const Tools_ObjectProcessor_Strip* GetProcessedStrip(void);
		static u_int GetTotalNumberOfVertices(void);
		static u_int GetTotalNumberOfIndices(void);
		static u_int GetTotalNumberOfStrips(void);
		
		static Tools_ObjectProcessor_Strip*		s_pxProcessedStrip;
		static u_int							s_uTotalNumberOfVertices;
		static u_int							s_uTotalNumberOfIndices;
		static u_int							s_uTotalNumberOfTriangles;
		static u_int							s_uNumberOfStrips;
		static bool								s_bBeenInitialised;
	
	private:
		static bool Initialise(Asura_Mesh* pxMesh,int iMaterialIndex, Asura_Hash_ID uMatResponse1,Asura_Hash_ID uMatResponse2 ,bool bStripify = true);
		

		static bool MeshProcess(Asura_Mesh* pxMesh, u_int uStrip);
};



#endif//__TOOLS_OBJECTPROCESSOR_H__
