//
// Tools_ObjectProcessor_NormalMapped.cpp
//
// Created by AJG - 22/06/2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "MeshProcessing/Tools_ObjectProcessor_NormalMapped.h"

#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_Material_System.h"
#include "Mesh/Asura_Mesh.h"

#include "MeshProcessing/Asura_TriStripper.h"
#include "MeshProcessing/Tools_MeshRestartIndex.h"
#include "MeshProcessing/Tools_TangentSpace.h"

Tools_ObjectProcessor_NormalMapped_Strip*	Tools_ObjectProcessor_NormalMapped::s_pxProcessedStrip;
u_int										Tools_ObjectProcessor_NormalMapped::s_uTotalNumberOfVertices;
u_int										Tools_ObjectProcessor_NormalMapped::s_uTotalNumberOfIndices;
u_int										Tools_ObjectProcessor_NormalMapped::s_uTotalNumberOfTriangles;
bool										Tools_ObjectProcessor_NormalMapped::s_bBeenInitialised = false;


bool Tools_ObjectProcessor_NormalMapped::Initialise(Asura_Mesh* pxMesh,
													int iMaterialIndex,
													bool bStripify /*=true*/,
													Tools_TriStripper::OUTPUT eOutputType /*= Tools_TriStripper::OUTPUT_TRISTRIP_WITH_RESET_INDICES*/,
													Asura_Hash_ID uMaterialResponseHashID1 /*= ASURA_HASH_ID_UNSET*/,
													Asura_Hash_ID uMaterialResponseHashID2 /*= ASURA_HASH_ID_UNSET*/)
													
{
	if(!pxMesh)
	{
		Asura_Assert(pxMesh,"no mesh to process");
		return false;
	}
	
	//reset totals
	s_uTotalNumberOfTriangles = 0;
	s_uTotalNumberOfVertices  = 0;
	s_uTotalNumberOfIndices   = 0;

	// KJL - 10/06/2003 10:49:20 - worst case scenario is that every poly is double sided (so tris * 3 verts * 2)
	u_int uTotalIndices = pxMesh->NumberOfTriangles*3*2;
	u_int uNumberOfUniqueVertices = 0;
	u_int uIndex = 0;

	s_pxProcessedStrip = new Tools_ObjectProcessor_NormalMapped_Strip;

	Tools_ObjectProcessor_NormalMapped_Vertex* pxWorkspaceVertices = new Tools_ObjectProcessor_NormalMapped_Vertex[uTotalIndices];
	unsigned short* usIndexData = new unsigned short[uTotalIndices];

	u_int uNumberOfTriangles = 0;
	
	if(!pxMesh->AreNormalsValid())
	{
		pxMesh->CreateNormals();
	}	
	
	// fill up the workspace vertex buffer
	for (int iPoly=0; iPoly<pxMesh->NumberOfPolygons; iPoly++)
	{	
		int v=0;
		int iIndexNumber[4];
		
		int iPolyFlags = 0;

		if (pxMesh->PolygonFlagDataPtr)
		{
			iPolyFlags = pxMesh->PolygonFlagDataPtr[iPoly].Flags;
		}

		if (iPolyFlags & AsuraPolygonFlag_DontRender) continue;

		// Calculate the tangent and binormal, per polygon (but store results per poly)
		Asura_Vector_3 xTangent;
		Asura_Vector_3 xBinormal;

		CalculateTangentSpaceVectors(pxMesh, iPoly, xTangent, xBinormal);

		do
		{
			
			
			int vID =  pxMesh->PolygonDataPtr[iPoly].VertexID[v];
			Asura_Vector_3& xPosition = pxMesh->VertexDataPtr[vID].Position;
			Asura_Vector_3& xNormal = pxMesh->NormalsDataPtr[iPoly].VertexNormal[v];

			Tools_ObjectProcessor_NormalMapped_Vertex xVertexData;

			xVertexData.m_xPosition	= xPosition;
			xVertexData.m_xNormal	= xNormal;
			xVertexData.m_xTangent	= Tools_TangentSpace::ClampPerpendicularToNormal (xNormal, xTangent, pxMesh->Name.GetString());
			xVertexData.m_xBinormal	= Tools_TangentSpace::ClampPerpendicularToNormal (xNormal, xBinormal, pxMesh->Name.GetString());

			const Asura_Mesh_TextureInfo* pxTexInfo = pxMesh->GetTextureInfoForExport(iPoly);
			xVertexData.m_fU = pxTexInfo->U[0][v];
			xVertexData.m_fV = pxTexInfo->V[0][v];
			xVertexData.m_fU2 = pxTexInfo->U[1][v];
			xVertexData.m_fV2 = pxTexInfo->V[1][v];
			xVertexData.m_fAlpha = pxMesh->TextureDataPtr[iPoly].m_afAlpha[v];

			// work out which vertices are duplicates
			int i = uNumberOfUniqueVertices;
			bool bVertexNotUnique = false;
			while(i)
			{
				i--;

				if(pxWorkspaceVertices[i] == xVertexData)
				{
					bVertexNotUnique = true;
					break;
				}
			}
	
			if (bVertexNotUnique)
			{
				iIndexNumber[v] = i;
			}
			else
			{
				iIndexNumber[v] = uNumberOfUniqueVertices;

				pxWorkspaceVertices[uNumberOfUniqueVertices] = xVertexData;
				uNumberOfUniqueVertices++;
			}
			v++;
		}
		
		while(v<4 && pxMesh->PolygonDataPtr[iPoly].VertexID[v]!=0xffff);

		usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
		usIndexData[uIndex++] = (unsigned short)(iIndexNumber[1]);
		usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
		uNumberOfTriangles++;

		//take account of quads
		if (pxMesh->PolygonDataPtr[iPoly].IsQuad())
		{
			usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
			usIndexData[uIndex++] = (unsigned short)(iIndexNumber[3]);
			usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
			uNumberOfTriangles++;
		}

		if(iPolyFlags & AsuraPolygonFlag_DoubleSided)
		{
			v=0;
			
			do
			{
				int vID =  pxMesh->PolygonDataPtr[iPoly].VertexID[v];
				Asura_Vector_3& xPosition = pxMesh->VertexDataPtr[vID].Position;
				Asura_Vector_3& xNormal = pxMesh->NormalsDataPtr[iPoly].VertexNormal[v];

				Tools_ObjectProcessor_NormalMapped_Vertex xVertexData;

				xVertexData.m_xPosition = xPosition;

				xVertexData.m_xNormal = -xNormal;

				float fDot;
				// make sure the tangent is perpendicular to the normal
				fDot = xNormal * xTangent;
				xVertexData.m_xTangent = (xTangent - xNormal * fDot);
				xVertexData.m_xTangent.Normalise();
		
				// make sure the binormal is perpendicular to the normal
				fDot = xNormal * xBinormal;
				xVertexData.m_xBinormal = (xBinormal - xNormal * fDot);
				xVertexData.m_xBinormal.Normalise();

				xVertexData.m_fU = pxMesh->TextureDataPtr[iPoly].U[0][v];
				xVertexData.m_fV = pxMesh->TextureDataPtr[iPoly].V[0][v];
				xVertexData.m_fU2 = pxMesh->TextureDataPtr[iPoly].U[1][v];
				xVertexData.m_fV2 = pxMesh->TextureDataPtr[iPoly].V[1][v];
				xVertexData.m_fAlpha = pxMesh->TextureDataPtr[iPoly].m_afAlpha[v];

				// work out which vertices are duplicates
				int i = uNumberOfUniqueVertices;
				bool bVertexNotUnique = false;
				while(i)
				{
					i--;

					if(pxWorkspaceVertices[i] == xVertexData)
					{
						bVertexNotUnique = true;
						break;
					}
				}
		
				if (bVertexNotUnique)
				{
					iIndexNumber[v] = i;
				}
				else
				{
					iIndexNumber[v] = uNumberOfUniqueVertices;

					pxWorkspaceVertices[uNumberOfUniqueVertices] = xVertexData;
					uNumberOfUniqueVertices++;
				}
				v++;
			}
			
			while(v<4 && pxMesh->PolygonDataPtr[iPoly].VertexID[v]!=0xffff);

			usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
			usIndexData[uIndex++] = (unsigned short)(iIndexNumber[1]);
			usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
			uNumberOfTriangles++;

			//take account of quads
			if (pxMesh->PolygonDataPtr[iPoly].IsQuad())
			{
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[0]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[3]);
				usIndexData[uIndex++] = (unsigned short)(iIndexNumber[2]);
				uNumberOfTriangles++;
			}
		}
	}
	
	// test
	for(u_int u=0; u<uIndex; u++)
	{
		Asura_Assert(usIndexData[u]>=0,"");
		Asura_Assert(usIndexData[u]<uNumberOfUniqueVertices,"");
	}

	u_int uNumberOfVertices = uNumberOfUniqueVertices;
    u_int uNumberOfStrippedIndices = 0;

	if(uNumberOfTriangles)
	{
		u_short* pusPermutationTable = 0;
		
		if(bStripify)
		{
        
        	Tools_TriStripper::Initialise();
		    Tools_TriStripper::SetOutputType( eOutputType );

		    u_short* pusStripIndices;
	        uNumberOfStrippedIndices = Tools_TriStripper::CreateStrips( uNumberOfTriangles, usIndexData, &pusStripIndices );

	        Tools_TriStripper::ComputeVertexPermutation( uNumberOfStrippedIndices, pusStripIndices, uNumberOfVertices, &pusPermutationTable );

        	Tools_TriStripper::Shutdown();

			u_short usRestartValue;
			const bool bUseRestartIndex = Tools_MeshRestartIndex::IsValidForCurrentPlatform( usRestartValue );
			if( !bUseRestartIndex )
			{
				uNumberOfTriangles = uNumberOfStrippedIndices - 2;
			}

			// create new strip for putting data in
			s_pxProcessedStrip->m_pusIndices = pusStripIndices;
			s_pxProcessedStrip->m_uNumberOfIndices	= uNumberOfStrippedIndices;



		}
		else
		{
			//not stripifying , so just copy across the triangle indices. RAR 14/08/2007
			s_pxProcessedStrip->m_uNumberOfIndices	= uNumberOfTriangles*3;
			s_pxProcessedStrip->m_pusIndices = new u_short[s_pxProcessedStrip->m_uNumberOfIndices];

			Asura_Mem::Copy(s_pxProcessedStrip->m_pusIndices, usIndexData,sizeof(unsigned short)*s_pxProcessedStrip->m_uNumberOfIndices);
		}

		s_pxProcessedStrip->m_pxVertices = new Tools_ObjectProcessor_NormalMapped_Vertex[uNumberOfUniqueVertices];

		// copy all the indexed vertex data
		for(u_int u=0; u<uNumberOfUniqueVertices; u++)
		{	
			u_short usIndex = pusPermutationTable ? pusPermutationTable[u] : (u_short) u;
			s_pxProcessedStrip->m_pxVertices[u].m_xPosition = pxWorkspaceVertices[usIndex].m_xPosition;
			
			s_pxProcessedStrip->m_pxVertices[u].m_xNormal = pxWorkspaceVertices[usIndex].m_xNormal;
			s_pxProcessedStrip->m_pxVertices[u].m_xTangent = pxWorkspaceVertices[usIndex].m_xTangent;
			s_pxProcessedStrip->m_pxVertices[u].m_xBinormal = pxWorkspaceVertices[usIndex].m_xBinormal;

			s_pxProcessedStrip->m_pxVertices[u].m_fU = pxWorkspaceVertices[usIndex].m_fU;
			s_pxProcessedStrip->m_pxVertices[u].m_fV = pxWorkspaceVertices[usIndex].m_fV;
			s_pxProcessedStrip->m_pxVertices[u].m_fU2 = pxWorkspaceVertices[usIndex].m_fU2;
			s_pxProcessedStrip->m_pxVertices[u].m_fV2 = pxWorkspaceVertices[usIndex].m_fV2;
			s_pxProcessedStrip->m_pxVertices[u].m_fAlpha = pxWorkspaceVertices[usIndex].m_fAlpha;
		}	

		//s_pxProcessedStrip[uStrip].m_iMaterialID			= pxMesh->TextureDataPtr[0].m_iMaterialID;	

		s_pxProcessedStrip->m_uNumberOfTriangles = uNumberOfTriangles;
		s_pxProcessedStrip->m_uNumberOfVertices	= uNumberOfVertices;
       	
       	delete[] pusPermutationTable;
	}
	else
	{
		s_pxProcessedStrip->m_pxVertices = 0;
		s_pxProcessedStrip->m_pusIndices = 0;

		s_pxProcessedStrip->m_uNumberOfIndices	= 0;
		s_pxProcessedStrip->m_uNumberOfTriangles = 0;
		s_pxProcessedStrip->m_uNumberOfVertices	= 0;
	}

	// bounding box data
	s_pxProcessedStrip->m_xBBMin.x = pxMesh->BoundingBox.MinX;
	s_pxProcessedStrip->m_xBBMin.y = pxMesh->BoundingBox.MinY;
	s_pxProcessedStrip->m_xBBMin.z = pxMesh->BoundingBox.MinZ;

	s_pxProcessedStrip->m_xBBMax.x = pxMesh->BoundingBox.MaxX;
	s_pxProcessedStrip->m_xBBMax.y = pxMesh->BoundingBox.MaxY;
	s_pxProcessedStrip->m_xBBMax.z = pxMesh->BoundingBox.MaxZ;

	//s_bBeenInitialised = true;

	// BM - 02/04/2003 15:47:16 -  - We only have one texture per object at 
	// the moment. If we ever have more than one this will need to be changed.
	// The conversion mechanism means that the exported material index will be 0

	// RAR - 07/04/2003 - Material index for the shape now passed to this function , to deal
	// with files with multiple shapes.

	s_pxProcessedStrip->m_auMaterialResponseHashID[0] = uMaterialResponseHashID1;
	s_pxProcessedStrip->m_auMaterialResponseHashID[1] = uMaterialResponseHashID2;
	s_pxProcessedStrip->m_iMaterialID = iMaterialIndex;
	
	// add up totals
	s_uTotalNumberOfTriangles += uNumberOfTriangles;
	s_uTotalNumberOfVertices  += uNumberOfVertices;
	s_uTotalNumberOfIndices   += uNumberOfStrippedIndices;

	// delete temp data
	delete [] usIndexData;
	delete [] pxWorkspaceVertices;

	s_bBeenInitialised = true;

	return true;
}	


void Tools_ObjectProcessor_NormalMapped::Shutdown(void)
{
	if (s_bBeenInitialised)
	{
		delete s_pxProcessedStrip;
	
		s_bBeenInitialised = false;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
	}	
}

const Tools_ObjectProcessor_NormalMapped_Strip* Tools_ObjectProcessor_NormalMapped::GetProcessedStrip()
{
	if (s_bBeenInitialised)
	{
		return s_pxProcessedStrip;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_ObjectProcessor_NormalMapped::GetTotalNumberOfVertices()
{
	if (s_bBeenInitialised)
	{
		return s_uTotalNumberOfVertices;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

u_int Tools_ObjectProcessor_NormalMapped::GetTotalNumberOfIndices()
{
	if (s_bBeenInitialised)
	{
		return s_uTotalNumberOfIndices;
	}
	else
	{
		Asura_Assert( s_bBeenInitialised, "Not yet Initialised!" );
		return 0;
	}
}

void Tools_ObjectProcessor_NormalMapped::CalculateTangentSpaceVectors(const class Asura_Mesh* pxMesh, u_int uPoly, Asura_Vector_3& xTangent, Asura_Vector_3& xBinormal)
{
	Asura_Vector_3	axPositions [3];
	Asura_Vector_2	axTextureCoords [3];

	// Initialise the vertex-data...

	for (u_int u = 0; u < 3; u++)
	{
		u_int	uVertexID	= pxMesh->PolygonDataPtr [uPoly].VertexID [u];


		// Fill in the data for this vertex...

		axPositions [u]			= pxMesh->VertexDataPtr [uVertexID].Position;
		axTextureCoords [u].x	= pxMesh->TextureDataPtr [uPoly].U [0][u];
		axTextureCoords [u].y	= 1.0f - pxMesh->TextureDataPtr [uPoly].V [0][u];
	}

	// Calculate the tangent AND binormal...

	Tools_TangentSpace::CalculateTangentAndBinormal	(axPositions, axTextureCoords, xTangent, xBinormal);
}
