
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_TangentSpaceVertex.h
// Author : Andy Tate
// Date   : 13-May-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#ifndef __TOOLS_TANGENTSPACEVERTEX_H__
#define __TOOLS_TANGENTSPACEVERTEX_H__

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class Asura_Mesh;
class Asura_Vector_2;
class Asura_Vector_3;
class Tools_MeshAdjacency;

////////////////////////////////////////////////////////////////////////////////
// Class declaration...
////////////////////////////////////////////////////////////////////////////////

class Tools_TangentSpace
{

public:

    static void CalculateTangentAndBinormalForPoly( const Asura_Mesh* pxMesh, int iPolyIdx, u_int uTextureIndex,
                                                    Asura_Vector_3& xTangent, Asura_Vector_3& xBinormal );
    static void CalculateTangentAndBinormal( const Asura_Vector_3* pxPositions, const Asura_Vector_2* pxTextureCoords,
                                             Asura_Vector_3& xTangent, Asura_Vector_3& xBinormal, const char* szMeshName = 0 );

	static	Asura_Vector_3	ClampPerpendicularToNormal	(const Asura_Vector_3& xNormal, const Asura_Vector_3& xInput, const char* pszMeshName = "<Unknown>", const int iPolygonNumber = -1, bool* pbErrorFlag = NULL);

private:

	Tools_TangentSpace  (void);
	Tools_TangentSpace  (Tools_TangentSpace const& rOther);
	~Tools_TangentSpace ();

	Tools_TangentSpace&	operator=	(Tools_TangentSpace const& rOther);

};   // End of class 'Tools_TangentSpace'.


//Deals with calculating smooted tangent/binormals for a mesh
//RAR 09/07/2007
class Tools_Mesh_TangentSpace
{
public :
	Tools_Mesh_TangentSpace();
	~Tools_Mesh_TangentSpace();

	//Set the module/mesh to calculate the tangents for
	void Initialise(const Asura_Mesh* pxMesh) {Initialise(pxMesh,-1);}
	void Initialise(int iModuleIndex){Initialise(0,iModuleIndex);}

	//Gets the smoothed tangent/binormal for the specified vertex in a polygon.
	//iVert is the vertex index into the mesh , not the into the polygon
	//If pbErrorFlag is used , then that flag will be set if there are any warnings in ClampPerpendicularToNormal()
	void GetTangentAndBinormal(int iPoly,int iVert, u_int uTextureIndex, const Asura_Vector_3& xVertexNormal, Asura_Vector_3& xOutputTangent, Asura_Vector_3& xOutputBinormal, bool* pbErrorFlag = NULL);

private :
	void Initialise(const Asura_Mesh* pxMesh, int iModuleIndex);

	int m_iModuleIndex;

	const Asura_Mesh* m_pxMesh;
	Tools_MeshAdjacency* m_pxMeshAdj;

	Asura_Vector_3 (*m_pxPolyTangents)[2];
	Asura_Vector_3 (*m_pxPolyBinormals)[2];

};


#endif   // ifndef __TOOLS_TANGENTSPACEVERTEX_H__


/// End of file. ///
