//
// Tools_VIPM.cpp
//
// Created by KOR - 2004-11-04
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER

// This file's header.
#include "MeshProcessing/Tools_VIPM.h"
#include "Rendering/Asura_PlatformRenderCharacter.h"

// KJL - 14/04/2003 14:46:04 - the view independent progressive mesh (VIPM) code I wrote for character LODs
Tools_VIPM::Tools_VIPM()
{
	VertexList = 0;
	IndexList = 0;
	ChangeList = 0;
	m_piVertMatch = 0;
}

Tools_VIPM::~Tools_VIPM()
{
	delete [] VertexList;
	delete [] IndexList;
	delete [] ChangeList;
	delete [] m_piVertMatch;
}

void Tools_VIPM::Create(Asura_Direct3D_CharacterVertex* vertices, unsigned short* indices, int numberOfVertices, int numberOfIndices, int* piOutputVertexOrdering /*= 0*/, int* piOutputPolygonOrdering /*= 0*/)
{
	VertexList = new Asura_Direct3D_CharacterVertex[numberOfVertices];
	for (int i=0; i<numberOfVertices; i++)
	{
		VertexList[i] = vertices[i];
	}
	TotalNumberOfVertices = numberOfVertices;

	if(piOutputVertexOrdering)
	{
		for (int i=0; i<numberOfVertices; i++)
		{
			piOutputVertexOrdering[i] = i;
		}
	}
	if(piOutputPolygonOrdering)
	{
		for (int i=0; i<numberOfIndices/3; i++)
		{
			piOutputPolygonOrdering[i] = i;
		}
	}

	IndexList = new unsigned short[numberOfIndices];
	for (int i=0; i<numberOfIndices; i++)
	{
		IndexList[i] = indices[i];
	}	
	TotalNumberOfTriangles = numberOfIndices/3;

	ConstructCollapseList(piOutputVertexOrdering,piOutputPolygonOrdering);	
#if 01

	for (int i=0; i<numberOfVertices; i++)
	{
		vertices[i] = VertexList[i];
	}
#endif
#if 01
	for (int i=0; i<numberOfIndices; i++)
	{
		indices[i] = IndexList[i];
	}	
#endif
}

void Tools_VIPM::ConstructCollapseList(int* piOutputVertexOrdering,int* piOutputPolygonOrdering)
{
	Tools_VIPM_Collapse collapse;

	NumberOfCollapses=0;
	CurrentNumberOfTriangles = TotalNumberOfTriangles;
	CurrentNumberOfVertices = TotalNumberOfVertices;
	CurrentNumberOfCollapses = 0;

	ChangeList = new Tools_VIPM_Change[TotalNumberOfVertices];

	//determine which vertices share the same location
	m_piVertMatch = new int[TotalNumberOfVertices];
	for(int i=0;i<TotalNumberOfVertices;i++)
	{
		m_piVertMatch[i] = i;
		for(int j=0;j<i;j++)
		{
			if(m_piVertMatch[j] != j) continue;
			if(	VertexList[i].m_xPosition == VertexList[j].m_xPosition)
			{
				m_piVertMatch[i] = j;
				break;
			}
		}
	}

	while(FindNextCollapse(collapse))
	{
		Tools_VIPM_Change* change = &ChangeList[NumberOfCollapses];
#if 01
		if (collapse.NumberOfTrisToRemove == 0)
		{
			Asura_Assert(NumberOfCollapses!=0,"stray vertex in model");

			change = &ChangeList[NumberOfCollapses-1];

			change->NumberOfVerticesRemoved++;
		}
		else
#endif
		{
			change->Replacement = collapse.CollapseToVertex;

			change->NumberOfChanges = collapse.NumberOfAffectedVertices;

			change->Index = new unsigned short[change->NumberOfChanges];
			change->Original = new int[change->NumberOfChanges];

			for (int i=0; i<change->NumberOfChanges; i++)
			{
				change->Original[i] = IndexList[collapse.AffectedVertices[i]];
				change->Index[i] = (unsigned short)collapse.AffectedVertices[i];
			}

			change->NumberOfTrianglesRemoved = collapse.NumberOfTrisToRemove;
			change->NumberOfVerticesRemoved = 1;

			NumberOfCollapses++;
		}

		CurrentNumberOfVertices--;
	
		/// Kevin Lea - 06/03/2002 11:38:43 - Step 1: swap the removal vertex to be the last in currently active selection
		Asura_Direct3D_CharacterVertex temp = VertexList[CurrentNumberOfVertices];
		VertexList[CurrentNumberOfVertices] = VertexList[collapse.VertexToRemove];
		VertexList[collapse.VertexToRemove] = temp;

		if(piOutputVertexOrdering)
		{
			//update the vertex ordering array
			int iTemp = piOutputVertexOrdering[CurrentNumberOfVertices];
			piOutputVertexOrdering[CurrentNumberOfVertices] = piOutputVertexOrdering[collapse.VertexToRemove];
			piOutputVertexOrdering[collapse.VertexToRemove] = iTemp;
		}

		/// Kevin Lea - 06/03/2002 11:58:16 - Step 2: relabel any occurrence of these vertices in the (entire) index list
		for (int i=0; i<TotalNumberOfTriangles*3; i++)
		{
			if (IndexList[i] == collapse.VertexToRemove)
			{
				IndexList[i] = (unsigned short)CurrentNumberOfVertices;
			}
			else if (IndexList[i] == CurrentNumberOfVertices)
			{
				IndexList[i] = (unsigned short)collapse.VertexToRemove;
			}
		}

		/// Kevin Lea - 06/03/2002 14:14:23 - Step 3: relabel any occurrence of these vertices in the change lists
		for (int i=0; i<NumberOfCollapses; i++)
		{
			Tools_VIPM_Change& changes = ChangeList[i];

			for(int j=0;j<changes.NumberOfChanges;j++)
			{
				if (changes.Original[j] == collapse.VertexToRemove)
				{
					changes.Original[j] = (unsigned short)CurrentNumberOfVertices;
				}
				else if (changes.Original[j] == CurrentNumberOfVertices)
				{
					changes.Original[j] = (unsigned short)collapse.VertexToRemove;
				}
			}

			if (changes.Replacement == collapse.VertexToRemove)
			{
				changes.Replacement = (unsigned short)CurrentNumberOfVertices;
			}
			else if (changes.Replacement == CurrentNumberOfVertices)
			{
				changes.Replacement = (unsigned short)collapse.VertexToRemove;
			}

		}

		//relabel indeces in vertex match array
		int iTemp = m_piVertMatch[collapse.VertexToRemove];
		m_piVertMatch[collapse.VertexToRemove] = m_piVertMatch[CurrentNumberOfVertices];
		m_piVertMatch[CurrentNumberOfVertices] = iTemp;

		for(int i=0;i<CurrentNumberOfVertices;i++)
		{
			if(m_piVertMatch[i] == collapse.VertexToRemove)
			{
				m_piVertMatch[i] = CurrentNumberOfVertices;
			}
			else if(m_piVertMatch[i] == CurrentNumberOfVertices)
			{
				m_piVertMatch[i] = collapse.VertexToRemove;
			}
		}


		/// Kevin Lea - 06/03/2002 15:20:10 - Step 4: rearrange index list so removal tris fall off list
		for (int t=0; t<collapse.NumberOfTrisToRemove; t++)
		{
			int index = collapse.RemoveTriangles[t];			
			int last = (--CurrentNumberOfTriangles)*3;
	
			for (int i=0; i<3; i++)
			{
				unsigned short temp = IndexList[index+i];
				IndexList[index+i] = IndexList[last+i];
				IndexList[last+i] = temp;

				for (int n=0; n<NumberOfCollapses; n++)
				{
					Tools_VIPM_Change& changes = ChangeList[n];

					for (int m=0; m<changes.NumberOfChanges; m++)
					{
						if (changes.Index[m] == index+i)
						{
							changes.Index[m] = (unsigned short)(last+i);
						}
						else if (changes.Index[m] == last+i)
						{
							changes.Index[m] = (unsigned short)(index+i);
						}
					}
				}
			}
			//Also need to correct triangle indeces of any entries in the collapse list that
			//haven't been processed yet. RAR.
			for(int t2=t+1;t2<collapse.NumberOfTrisToRemove; t2++)
			{
				if(collapse.RemoveTriangles[t2]==index)
				{
					Asura_Assert(0,"I don't think this case should be happening");
					collapse.RemoveTriangles[t2]=last;
				}
				else if(collapse.RemoveTriangles[t2]==last)
				{
					collapse.RemoveTriangles[t2]=index;
				}
			}

			if(piOutputPolygonOrdering)
			{
				int iTemp = piOutputPolygonOrdering[index/3];
				piOutputPolygonOrdering[index/3] = piOutputPolygonOrdering[last/3];
				piOutputPolygonOrdering[last/3] = iTemp;
			}

		}

		/// Kevin Lea - 06/03/2002 15:31:10 - Step 5: make changes to index list
		for (int i=0; i<change->NumberOfChanges; i++)
		{
			IndexList[change->Index[i]] = (unsigned short)change->Replacement;
		}
	}

	CurrentNumberOfCollapses = NumberOfCollapses;

	while(UpOneLevel());

	delete [] m_piVertMatch;
	m_piVertMatch = 0;

}

bool Tools_VIPM::FindNextCollapse(Tools_VIPM_Collapse& collapse)
{
	collapse.Cost = Asura_Maths::LargeFloat;

	for (int v=0; v<CurrentNumberOfVertices; v++)
	{
		/// Kevin Lea - 05/03/2002 12:34:04 - find all the tris containing v
		const int maxTris = 50;
		Tools_VIPM_Triangle triangles[maxTris];
		int numberOfTriangles = 0;

		for (int i=0; i<CurrentNumberOfTriangles; i++)
		{
			Tools_VIPM_Triangle& tri = triangles[numberOfTriangles];
			int foundTri = false;

			if (m_piVertMatch[IndexList[i*3]]==m_piVertMatch[v])
			{
				tri.TriangleIndex = i*3;
				tri.Vertex[0] = i*3+1;
				tri.Vertex[1] = i*3+2;
				foundTri = true;
			}
			else if (m_piVertMatch[IndexList[i*3+1]]==m_piVertMatch[v])
			{
				tri.TriangleIndex = i*3+1;
				tri.Vertex[0] = i*3;
				tri.Vertex[1] = i*3+2;
				foundTri = true;
			}
			else if (m_piVertMatch[IndexList[i*3+2]]==m_piVertMatch[v])
			{
				tri.TriangleIndex = i*3+2;
				tri.Vertex[0] = i*3;
				tri.Vertex[1] = i*3+1;
				foundTri = true;
			}

			if (foundTri)
			{
				Asura_Vector_3 a,b;

				int i0 = IndexList[i*3+0];
				int i1 = IndexList[i*3+1];
				int i2 = IndexList[i*3+2];

				tri.Normal = VertexList[i2].m_xNormal + VertexList[i1].m_xNormal + VertexList[i0].m_xNormal;
				tri.Normal.Normalise();

				numberOfTriangles++;
			}

			Asura_Assert(numberOfTriangles<maxTris,"not enough storage space");
		}
		if (!numberOfTriangles)
		{
			collapse.Cost = -1;

			collapse.VertexToRemove = v;
			collapse.CollapseToVertex = v;

			collapse.NumberOfTrisToRemove = 0;
			collapse.NumberOfAffectedVertices = 0;
			return true;
		}
		Asura_Assert(numberOfTriangles!=0,"paranoia check");

		//find vertices that share the same location as this one
		int iEquivalentVerts = 0;
		float fTotalUVDiff = 0.0f;
		for(int i=0;i<CurrentNumberOfVertices;i++)
		{
			if(i!=v && m_piVertMatch[i] == m_piVertMatch[v])
			{
				iEquivalentVerts++;
				fTotalUVDiff += Asura_Maths::Abs(VertexList[v].m_fU - VertexList[i].m_fU);
				fTotalUVDiff += Asura_Maths::Abs(VertexList[v].m_fV - VertexList[i].m_fV);
			}
		}


		/// Kevin Lea - 05/03/2002 14:34:20 - find all candidate vertices to collapse to
		int candidateVertices[maxTris*2];
		int numberOfCandidateVertices = 0;


		for (int t=0; t<numberOfTriangles; t++)
		{

			for (int i=0; i!=2; i++)
			{
				bool alreadyGotVertex = false;
				for (int u=0; u<numberOfCandidateVertices; u++)
				{
					if (candidateVertices[u] == m_piVertMatch[IndexList[triangles[t].Vertex[i]]])
					{
						alreadyGotVertex = true;
						break;
					}
				}
				if (!alreadyGotVertex)
				{
					candidateVertices[numberOfCandidateVertices++] = m_piVertMatch[IndexList[triangles[t].Vertex[i]]];
				}
			}
		}
		

			
		Asura_Assert(numberOfCandidateVertices!=0,"paranoia check");
		if (numberOfCandidateVertices==0) continue;

		/// Kevin Lea - 05/03/2002 14:20:18 - find best edge to collapse
		Asura_Vector_3 vertex(VertexList[v].m_xPosition);
		
		float leastCost = Asura_Maths::LargeFloat;
		int bestCollapseVertex = 0;
	
		bool onBorder = false;

		for (int u=0; u<numberOfCandidateVertices; u++)
		{
			int& c = candidateVertices[u];

			Asura_Vector_3 collapseTo(VertexList[c].m_xPosition);

			Asura_Vector_3 disp = vertex-collapseTo;
			
			float cost = disp.Magnitude();
			float curvature = 0.0f;
#if 01
			int numberOfAdjacentTris=0;
			int adjacentTris[maxTris];
			int numberOfOtherTris = 0;
			int otherTris[maxTris];
			for (int t=0; t<numberOfTriangles; t++)
			{
				bool adjacent = false;

				for (int i=0; i<2; i++)
				{
					if (m_piVertMatch[IndexList[triangles[t].Vertex[i]]] == c)
					{
						adjacent = true;
						break;
					}
				}
				if (adjacent)
				{
					//Asura_Assert(numberOfAdjacentTris!=2,"more than two tris at an edge");
					adjacentTris[numberOfAdjacentTris++]=t;
				}
				else
				{
					otherTris[numberOfOtherTris++]=t;
				}
			}

			for (int ot=0; ot<numberOfTriangles; ot++)
			{
				float mincurv = 3.00f;
				for (int at=0; at<numberOfAdjacentTris; at++)
				{
					float dot = triangles[adjacentTris[at]].Normal * triangles[ot].Normal;

					mincurv = Asura_Maths::Min(mincurv,(2.0f-dot));
				}

				curvature = Asura_Maths::Max(curvature,mincurv);
			}
			
			curvature = 1.0f;
			if (numberOfOtherTris==0)
			{
		//		curvature = 1000.0f;
			}
			if (numberOfAdjacentTris==1)
			{	
			//	curvature = 1000.0f;
				
				onBorder = true;
			}

			cost *= curvature;
			//cost = disp.Magnitude();

#endif
			if (cost<leastCost)
			{
				leastCost = cost;
				bestCollapseVertex = c;
			}
		}

		if(iEquivalentVerts)
		{
			leastCost += fTotalUVDiff*10.0f;
		}
		if (onBorder)
		{
			leastCost+=10000.0f;
		}

		if (leastCost<collapse.Cost)
		{
			collapse.Cost = leastCost;

			collapse.VertexToRemove = v;
			collapse.CollapseToVertex = bestCollapseVertex;

			collapse.NumberOfTrisToRemove = 0;
			collapse.NumberOfAffectedVertices = 0;

			for (int t=0; t<numberOfTriangles; t++)
			{
				bool triRemoved = false;
				for (int i=0; i<2; i++)
				{
					if (m_piVertMatch[IndexList[triangles[t].Vertex[i]]] == m_piVertMatch[bestCollapseVertex])
					{
						collapse.RemoveTriangles[collapse.NumberOfTrisToRemove++] = (triangles[t].TriangleIndex/3)*3;
						triRemoved = true;
						break;
					}
				}
				if (!triRemoved)
				{
					collapse.AffectedVertices[collapse.NumberOfAffectedVertices++] = triangles[t].TriangleIndex;
					
				}
			}
		}
	}

	
	if (collapse.Cost == Asura_Maths::LargeFloat)
	{
		return false;
	}
	else
	{
		return true;
	}
}

bool Tools_VIPM::UpOneLevel(void)
{
	if (CurrentNumberOfCollapses==0)
	{
		return false;
	}
	
	CurrentNumberOfCollapses--;

	Tools_VIPM_Change* change = &ChangeList[CurrentNumberOfCollapses];

	CurrentNumberOfTriangles += change->NumberOfTrianglesRemoved;
	CurrentNumberOfVertices += change->NumberOfVerticesRemoved;

	for (int i=0; i<change->NumberOfChanges; i++)
	{
		Asura_Assert(IndexList[change->Index[i]] == change->Replacement,"uhoh");	
		IndexList[change->Index[i]] = (unsigned short)change->Original[i];
	}
	return true;
}

bool Tools_VIPM::DownOneLevel(void)
{
	if (CurrentNumberOfCollapses==NumberOfCollapses)
	{
		return false;
	}
	
	Tools_VIPM_Change* change = &ChangeList[CurrentNumberOfCollapses];
	
	CurrentNumberOfTriangles -= change->NumberOfTrianglesRemoved;
	CurrentNumberOfVertices -= change->NumberOfVerticesRemoved;

	for (int i=0; i<change->NumberOfChanges; i++)
	{
		Asura_Assert(IndexList[change->Index[i]] == change->Original[i],"uhoh");	
		IndexList[change->Index[i]] = (unsigned short)change->Replacement;
	}
	


	CurrentNumberOfCollapses++;
	return true;
}

#endif //!defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
