/**
*
* \file Tools_Large_AABB_Tree.cpp
*
* Created by Kevin Lea - 21/06/2002 14:41:25
*
* Copyright (C) 2002 Rebellion, All Rights Reserved.
*
* \brief
*
**/

#include "Core/Asura.h"

#include "Misc/Tools_Large_AABB_Tree.h"
#include "Mesh/Asura_Mesh.h"
#include "Debugging/Asura_Assert.h"

Tools_Large_AABB_Tree::Tools_Large_AABB_Tree()
{
	Nodes = 0;	
}

Tools_Large_AABB_Tree::~Tools_Large_AABB_Tree()
{
	if (Nodes) delete [] Nodes;
}


void Tools_Large_AABB_Tree::TestTree(const Asura_Bounding_Box& testBB, void(*outputCallback)(unsigned int index))
{
	OutputCallback = outputCallback;
	TestBB = &testBB;

	if (!Nodes)
	{
		// KJL - 16/05/2002 17:33:22 -  if there is only one object, the node test will
		// always return an index of zero. Though I must say it's a complete waste of
		// time and space to use this data structure with such sparse information. I've
		// put this in to make sure all cases (e.g. only one module in the environment, a 
		// module with just one triangle) are catered for - even though one would hope 
		// such situations don't occur in a released game.
		if (TestBB->Intersects(RootBoundingBox))
		{
			OutputCallback(0);
		}
		return;
	}

	TestNode(0,this->RootBoundingBox);
}

static inline void ReconstructBBs( const Tools_Large_AABB_Tree_Node& xNode, const Asura_Bounding_Box& xParentBB, Asura_Bounding_Box& xLeftBB, Asura_Bounding_Box& xRightBB )
{
		/// Kevin Lea - 14/05/2002 10:57:23 - reconstruct left and right bounding boxes
	if (xNode.MinXBelongsToLeft)
	{
		xLeftBB.MinX  = xNode.ChildExtents.MinX;
		xRightBB.MinX = xParentBB.MinX;
	}
	else
	{
		xLeftBB.MinX  = xParentBB.MinX;
		xRightBB.MinX = xNode.ChildExtents.MinX;
	}
	if (xNode.MinYBelongsToLeft)
	{
		xLeftBB.MinY  = xNode.ChildExtents.MinY;
		xRightBB.MinY = xParentBB.MinY;
	}
	else
	{
		xLeftBB.MinY  = xParentBB.MinY;
		xRightBB.MinY = xNode.ChildExtents.MinY;
	}
	if (xNode.MinZBelongsToLeft)
	{
		xLeftBB.MinZ  = xNode.ChildExtents.MinZ;
		xRightBB.MinZ = xParentBB.MinZ;
	}
	else
	{
		xLeftBB.MinZ  = xParentBB.MinZ;
		xRightBB.MinZ = xNode.ChildExtents.MinZ;
	}
	if (xNode.MaxXBelongsToLeft)
	{
		xLeftBB.MaxX  = xNode.ChildExtents.MaxX;
		xRightBB.MaxX = xParentBB.MaxX;
	}
	else
	{
		xLeftBB.MaxX  = xParentBB.MaxX;
		xRightBB.MaxX = xNode.ChildExtents.MaxX;
	}
	if (xNode.MaxYBelongsToLeft)
	{
		xLeftBB.MaxY  = xNode.ChildExtents.MaxY;
		xRightBB.MaxY = xParentBB.MaxY;
	}
	else
	{
		xLeftBB.MaxY  = xParentBB.MaxY;
		xRightBB.MaxY = xNode.ChildExtents.MaxY;
	}
	if (xNode.MaxZBelongsToLeft)
	{
		xLeftBB.MaxZ  = xNode.ChildExtents.MaxZ;
		xRightBB.MaxZ = xParentBB.MaxZ;
	}
	else
	{
		xLeftBB.MaxZ  = xParentBB.MaxZ;
		xRightBB.MaxZ = xNode.ChildExtents.MaxZ;
	}

}


void Tools_Large_AABB_Tree::TestNode(int nodeIndex, Asura_Bounding_Box ParentBB) const
{
	Asura_Bounding_Box leftBB;
	Asura_Bounding_Box rightBB;

	Tools_Large_AABB_Tree_Node& node = this->Nodes[nodeIndex];

	// Kevin Lea - 14/05/2002 10:57:23 - reconstruct left and right bounding boxes
	ReconstructBBs( node, ParentBB, leftBB, rightBB );

	/// Kevin Lea - 14/05/2002 11:03:41 - if left is leaf, output index if overlap occurs
	if (node.LeftIsLeaf)
	{
		if (TestBB->Intersects(leftBB))
		{
			/// Kevin Lea - 14/05/2002 11:05:36 - output
			OutputCallback(node.LeftIndex);
		}
	}
	else if(TestBB->Intersects(leftBB))
	{
		/// Kevin Lea - 14/05/2002 10:58:52 - if test bounding box surrounds a node completely, 
		/// output all the contents of that node
		if(TestBB->Contains(leftBB))
		{	
			/// Kevin Lea - 14/05/2002 11:05:41 - output all
			OutputAll(node.LeftIndex);
		}
		else
		{
			/// Kevin Lea - 14/05/2002 11:06:38 - test further
			TestNode(node.LeftIndex,leftBB);
		}
	}

	/// Kevin Lea - 14/05/2002 11:03:41 - if Right is leaf, output index if overlap occurs
	if (node.RightIsLeaf)
	{
		if (TestBB->Intersects(rightBB))
		{
			/// Kevin Lea - 14/05/2002 11:05:36 - output
			OutputCallback(node.RightIndex);
		}
	}
	else if(TestBB->Intersects(rightBB))
	{
		/// Kevin Lea - 14/05/2002 10:58:52 - if test bounding box surrounds a node completely, 
		/// output all the contents of that node
		if(TestBB->Contains(rightBB))
		{	
			/// Kevin Lea - 14/05/2002 11:05:41 - output all
			OutputAll(node.RightIndex);
		}
		else
		{
			/// Kevin Lea - 14/05/2002 11:06:38 - test further
			TestNode(node.RightIndex,rightBB);
		}
	}
}
void Tools_Large_AABB_Tree::OutputAll(int nodeIndex) const
{
	Tools_Large_AABB_Tree_Node& node = this->Nodes[nodeIndex];

	// KJL - 16/05/2002 16:14:11 - merely recurse through and output every leaf
	if (node.LeftIsLeaf)
	{
		OutputCallback(node.LeftIndex);
	}
	else
	{
		OutputAll(node.LeftIndex);
	}

	if (node.RightIsLeaf)
	{
		OutputCallback(node.RightIndex);
	}
	else
	{
		OutputAll(node.RightIndex);
	}
}


void Tools_Large_AABB_Tree::TestTreeWithRay(const Asura_Vector_3& xRayOrigin, const Asura_Vector_3& xRayDirection, bool(*outputCallback)(unsigned int index, float fDistance), const float* pfMaxDistance)
{
	OutputCallback2 = outputCallback;
	m_pfMaxDistance = pfMaxDistance;

	if (!Nodes)
	{
		// KJL - 16/05/2002 17:33:22 -  if there is only one object, the node test will
		// always return an index of zero. Though I must say it's a complete waste of
		// time and space to use this data structure with such sparse information. I've
		// put this in to make sure all cases (e.g. only one module in the environment, a 
		// module with just one triangle) are catered for - even though one would hope 
		// such situations don't occur in a released game.
		if (Asura_Maths::RayIntersectsBoundingBox(xRayOrigin,xRayDirection,RootBoundingBox))
		{
			OutputCallback2(0,0.0f);
		}
		return;
	}

	TestNodeWithRay(0,this->RootBoundingBox,xRayOrigin,xRayDirection);

}

// Non-recursive, threadsafe version
void Tools_Large_AABB_Tree::TestTreeWithRay_ThreadSafe( Tools_Large_AABB_Tree_RayData* pxData, bool(*pfnCallback)(u_int uIndex, float fAABBDistance, Tools_Large_AABB_Tree_RayData* pxData) ) const
{
	if (!Nodes)
	{
		// KJL - 16/05/2002 17:33:22 -  if there is only one object, the node test will
		// always return an index of zero. Though I must say it's a complete waste of
		// time and space to use this data structure with such sparse information. I've
		// put this in to make sure all cases (e.g. only one module in the environment, a 
		// module with just one triangle) are catered for - even though one would hope 
		// such situations don't occur in a released game.
	
		float fAABBDistance = pxData->GetDistance();

		if ( pxData->RayIntersectsAABB( RootBoundingBox, fAABBDistance ) )
		{
			pfnCallback( 0, fAABBDistance, pxData );
		}

		return;
	}

	int aiIndexStack[32];
	Asura_Bounding_Box axBoundingBoxStack[32];

	aiIndexStack[0] = 0;
	axBoundingBoxStack[0] = this->RootBoundingBox;
	u_int uStack = 1;


	while ( uStack )
	{
		// get something off the stack
		Asura_Assert( uStack<32, "Code bug: stack size too small" );
		--uStack;
	
		Asura_Bounding_Box	axChildrenBB[2];
		float				afChildDistance[2];
		bool				abHit[2];
	
		const int iNodeIndex = aiIndexStack[ uStack ];
		const Tools_Large_AABB_Tree_Node& node = this->Nodes[iNodeIndex];

		// Kevin Lea - 14/05/2002 10:57:23 - reconstruct left and right bounding boxes
		ReconstructBBs( node, axBoundingBoxStack[ uStack ], axChildrenBB[0], axChildrenBB[1] );
	
		afChildDistance[0] = afChildDistance[1] = pxData->GetDistance();
		abHit[0] = pxData->RayIntersectsAABB( axChildrenBB[0], afChildDistance[0] );
		abHit[1] = pxData->RayIntersectsAABB( axChildrenBB[1], afChildDistance[1] );

		const bool	abIsLeaf[2] = { node.LeftIsLeaf, node.RightIsLeaf };
		const u_int	auNodeIndex[2] = { node.LeftIndex, node.RightIndex };
	
		u_int uFirst = 0;
		u_int uSecond = 1;
		
		if (abHit[0] && abHit[1])
		{
			if (afChildDistance[0]>afChildDistance[1])
			{
				uFirst = 1;
				uSecond = 0;
			}
		}

		if ( abIsLeaf[uFirst] && abHit[uFirst] )
		{
			if (!pfnCallback( auNodeIndex[uFirst], afChildDistance[uFirst], pxData ))
			{
				return;
			}
			// was this hit nearer than the second?
			abHit[uSecond] &= (pxData->GetDistance() > afChildDistance[uSecond] );
		}
		
		if ( abHit[uSecond] )
		{
			if ( abIsLeaf[uSecond] )
			{
				if (!pfnCallback( auNodeIndex[uSecond], afChildDistance[uSecond], pxData ))
				{
					return;
				}
			}
			else
			{
				aiIndexStack[ uStack ] = auNodeIndex[uSecond];
				axBoundingBoxStack[ uStack ] = axChildrenBB[uSecond];
				++uStack;
				Asura_Assert( uStack<32, "Code bug: stack size too small" );
			}
		}
			
		if ( !abIsLeaf[uFirst] && abHit[uFirst] )
		{
			aiIndexStack[ uStack ] = auNodeIndex[uFirst];
			axBoundingBoxStack[ uStack ] = axChildrenBB[uFirst];
			++uStack;
			Asura_Assert( uStack<32, "Code bug: stack size too small" );
		}

		
	}

}

// Non-recursive, threadsafe version
void Tools_Large_AABB_Tree::TestTreeWithAABB_ThreadSafe( Tools_Large_AABB_Tree_AABBData* pxData, void(*pfnCallback)(u_int uIndex, Tools_Large_AABB_Tree_AABBData* pxData)  ) const
{
	const Asura_Bounding_Box& xTestAABB = pxData->GetTestAABB();

	if (!Nodes)
	{
		if ( xTestAABB.Intersects( RootBoundingBox ) )
		{
			pfnCallback( 0, pxData );
		}

		return;
	}

	int aiIndexStack[32];
	Asura_Bounding_Box axBoundingBoxStack[32];
	bool abTestIntersection[32];

	aiIndexStack[0] = 0;
	axBoundingBoxStack[0] = this->RootBoundingBox;
	abTestIntersection[0] = true;
	u_int uStack = 1;


	while ( uStack )
	{
		// get something off the stack
		Asura_Assert( uStack<32, "Code bug: stack size too small" );
		--uStack;
	
		const int iNodeIndex = aiIndexStack[ uStack ];
		const Tools_Large_AABB_Tree_Node& xNode = this->Nodes[iNodeIndex];

		if ( abTestIntersection[ uStack ] )
		{
			Asura_Bounding_Box	xLeftAABB, xRightAABB;
		
			// Kevin Lea - 14/05/2002 10:57:23 - reconstruct left and right bounding boxes
			ReconstructBBs( xNode, axBoundingBoxStack[ uStack ], xLeftAABB, xRightAABB );
		
			if ( xLeftAABB.Intersects( xTestAABB ) )
			{
				if (xNode.LeftIsLeaf)
				{
					pfnCallback( xNode.LeftIndex, pxData );
				}
				else
				{
					aiIndexStack[ uStack ] = xNode.LeftIndex;
					axBoundingBoxStack[ uStack ] = xLeftAABB;
					abTestIntersection[ uStack ] = !(xTestAABB.Contains( xLeftAABB ));
					++uStack;
					Asura_Assert( uStack<32, "Code bug: stack size too small" );
				}
			}

			if ( xRightAABB.Intersects( xTestAABB ) )
			{
				if (xNode.RightIsLeaf)
				{
					pfnCallback( xNode.RightIndex, pxData );
				}
				else
				{
					aiIndexStack[ uStack ] = xNode.RightIndex;
					axBoundingBoxStack[ uStack ] = xRightAABB;
					abTestIntersection[ uStack ] = !(xTestAABB.Contains( xRightAABB ));;
					++uStack;
					Asura_Assert( uStack<32, "Code bug: stack size too small" );
				}
			}
		}
		else
		{
			if (xNode.LeftIsLeaf)
			{
				pfnCallback( xNode.LeftIndex, pxData );
			}
			else
			{
				aiIndexStack[ uStack ] = xNode.LeftIndex;
				abTestIntersection[ uStack ] = false;
				++uStack;
				Asura_Assert( uStack<32, "Code bug: stack size too small" );
			}

			if (xNode.RightIsLeaf)
			{
				pfnCallback( xNode.RightIndex, pxData );
			}
			else
			{
				aiIndexStack[ uStack ] = xNode.RightIndex;
				abTestIntersection[ uStack ] = false;
				++uStack;
				Asura_Assert( uStack<32, "Code bug: stack size too small" );
			}
		}
	}
}

// Returns true iff the search should continue.
bool Tools_Large_AABB_Tree::TestNodeWithRay(const int iNodeIndex, Asura_Bounding_Box ParentBB, const Asura_Vector_3& xRayOrigin, const Asura_Vector_3& xRayDirection)
{
	Asura_Bounding_Box leftBB;
	Asura_Bounding_Box rightBB;

	Tools_Large_AABB_Tree_Node& node = this->Nodes[iNodeIndex];

	// Kevin Lea - 14/05/2002 10:57:23 - reconstruct left and right bounding boxes
	ReconstructBBs( node, ParentBB, leftBB, rightBB );

	/// Kevin Lea - 14/05/2002 11:03:41 - if left is leaf, output index if overlap occurs
	float fLeftDistance;
	bool bLeftHit = Asura_Maths::RayIntersectsBoundingBox(xRayOrigin,xRayDirection, leftBB, &fLeftDistance);

	if (bLeftHit && (*m_pfMaxDistance < fLeftDistance)) bLeftHit = false;

	float fRightDistance;
	bool bRightHit = Asura_Maths::RayIntersectsBoundingBox(xRayOrigin,xRayDirection, rightBB, &fRightDistance);

	
	bool bLeftFirst = false;

	if (bLeftHit && bRightHit)
	{
		if (fLeftDistance<fRightDistance)
		{
			bLeftFirst = true;
		}
	}

	if (bLeftHit && bLeftFirst)
	{
		if (node.LeftIsLeaf)
		{
			if (!OutputCallback2(node.LeftIndex,fLeftDistance))
			{
				return false;
			}
		}
		else 
		{
			/// Kevin Lea - 14/05/2002 11:06:38 - test further
			if (!TestNodeWithRay(node.LeftIndex,leftBB,xRayOrigin,xRayDirection))
			{
				return false;
			}
		}
	}

	if (bRightHit && (*m_pfMaxDistance < fRightDistance)) bRightHit = false;

	if (bRightHit)
	{
		/// Kevin Lea - 14/05/2002 11:03:41 - if Right is leaf, output index if overlap occurs
		if (node.RightIsLeaf)
		{
			/// Kevin Lea - 14/05/2002 11:05:36 - output
			if (!OutputCallback2(node.RightIndex,fRightDistance))
			{
				return false;
			}
		}
		else
		{
			/// Kevin Lea - 14/05/2002 11:06:38 - test further
			if (!TestNodeWithRay(node.RightIndex,rightBB,xRayOrigin,xRayDirection))
			{
				return false;
			}
		}
	}

	
	if (bLeftHit && (*m_pfMaxDistance < fLeftDistance)) bLeftHit = false;

	if (bLeftHit && !bLeftFirst)
	{
		if (node.LeftIsLeaf)
		{
			if (!OutputCallback2(node.LeftIndex,fLeftDistance))
			{
				return false;
			}
		}
		else 
		{
			/// Kevin Lea - 14/05/2002 11:06:38 - test further
			if (!TestNodeWithRay(node.LeftIndex,leftBB,xRayOrigin,xRayDirection))
			{
				return false;
			}
		}
	}

	return true;
}


// KJL - 12/05/2002 16:03:43 - Tree creator - only really needs to be called in pre-processing step
unsigned int*				Tools_Large_AABB_Tree_Creator::OldObjectSet;
unsigned int*				Tools_Large_AABB_Tree_Creator::NewObjectSet;
const Asura_Vector_3*		Tools_Large_AABB_Tree_Creator::LeafCentre;
const Asura_Bounding_Box*	Tools_Large_AABB_Tree_Creator::LeafAABB;
Tools_Large_AABB_Tree*			Tools_Large_AABB_Tree_Creator::Tree;
unsigned int				Tools_Large_AABB_Tree_Creator::NodeIndex;

Tools_Large_AABB_Tree* Tools_Large_AABB_Tree_Creator::Create(const int numberOfObjects, const Asura_Bounding_Box* objectAABBs, const Asura_Vector_3* objectCentres)
{
	Asura_Assert(objectAABBs!=NULL,"no AABBs specified");

	Tree = new Tools_Large_AABB_Tree;

	Tree->m_uNumberOfObjects = numberOfObjects;

	// KJL - 16/05/2002 17:33:22 -  if there is only one object, the node test will
	// always return an index of zero. Though I must say it's a complete waste of
	// time and space to use this data structure with such sparse information. I've
	// put this in to make sure all cases (e.g. only one module in the environment, a 
	// module with just one triangle) are catered for - even though one would hope 
	// such situations don't occur in a released game.
	if (numberOfObjects==1)
	{
		Tree->RootBoundingBox = objectAABBs[0];
		// KJL - 16/05/2002 17:38:04 - Nodes is set to NULL by the constructor, we
		// can use that to test for this case later
		return Tree; 
	}

	Tree->Nodes = new Tools_Large_AABB_Tree_Node[numberOfObjects-1];
	Tree->RootBoundingBox.Reset();

	LeafAABB = objectAABBs;

	Asura_Vector_3* allocatedCentres = 0;

	if (objectCentres == NULL)
	{
		allocatedCentres = new Asura_Vector_3[numberOfObjects];

		for (int i=0; i<numberOfObjects; i++)
		{
			allocatedCentres[i] = LeafAABB[i].CalculateCentre();
		}

		LeafCentre = allocatedCentres;
	}
	else
	{
		LeafCentre = objectCentres;
	}

	for (int i=0; i<numberOfObjects; i++)
	{
		Tree->RootBoundingBox.Set(LeafAABB[i]);
	}

	OldObjectSet = new unsigned int [numberOfObjects];
	NewObjectSet = new unsigned int [numberOfObjects];
	int start = 0;
	int end = numberOfObjects;

	for(int i=start; i<end; i++)
	{
		OldObjectSet[i] = (unsigned int)i;
	}

	NodeIndex = 0;
	SortObjectsIntoNodes(start,end,Tree->RootBoundingBox);


	delete [] OldObjectSet;
	delete [] NewObjectSet;

	if (allocatedCentres)
	{
		delete [] allocatedCentres;
	}

	return Tree;
}


void Tools_Large_AABB_Tree_Creator::SortObjectsIntoNodes(int start, int end, Asura_Bounding_Box& parentBB)
{
	Tools_Large_AABB_Tree_Node& node = (Tree->Nodes[NodeIndex]);

	// KJL - 12/05/2002 17:05:44 - we want to split the objects into two nodes, which we'll call
	// 'left' and 'right'. Ideally we'd like a pretty even distribution between the two nodes,
	// and the bounding volumes of the 'left' and 'right' sets to overlap as little as possible.
	// It may be that a better splitting heuristic would give noticable speed gains in game-time
	// code.
	//
	// Okay, I'm going to go through each axis of the parentBB in turn, calculating a midpoint
	// value to sort the objects' centroids against. (Intuitively one would expect the longest axis
	// to give the best results, but there may be cases like long thin objects that counter this.)
	// I'll then pick an axis based on the most even split of objects (e.g. minimum sum of the
	// squares of the objects in each node), weighted by the sum of the nodes' volumes. Obviously
	// we could split the parent volume in any way, but this should be a fast way of getting
	// reasonable results.

	float score[3];
	int objectsToTheRight[3];
	int objectsToTheLeft[3];
	Asura_Bounding_Box leftBB[3],rightBB[3];
	
	for (int axis=0; axis<3; axis++)
	{

		float midpoint;
		switch(axis)
		{
			default:
			case 0:
			{
				midpoint = (parentBB.MaxX+parentBB.MinX)*0.5f;
				break;
			}
			case 1:
			{
				midpoint = (parentBB.MaxY+parentBB.MinY)*0.5f;
				break;
			}
			case 2:
			{
				midpoint = (parentBB.MaxZ+parentBB.MinZ)*0.5f;
				break;
			}
		}

		objectsToTheRight[axis] = 0;
		objectsToTheLeft[axis] = 0;

		leftBB[axis].Reset();
		rightBB[axis].Reset();

		for (int i=start; i<end; i++)
		{
			int objectIndex = OldObjectSet[i];
			if (LeafCentre[objectIndex].GetElement(axis) > midpoint)
			{
				// KJL - 12/05/2002 16:19:44 - in 'right' node
				++objectsToTheRight[axis];
				rightBB[axis].Set(LeafAABB[objectIndex]);
			}
			else
			{
				// KJL - 12/05/2002 16:19:57 - in 'left' node
				++objectsToTheLeft[axis];
				leftBB[axis].Set(LeafAABB[objectIndex]);
	
			}
		}

		// KJFL - 18 November 2005 - Improved heuristic: try to minimise surface areas of the BBs. 
		// Surface area is better than volume since it copes with flat planar cases, and is generally
		// better for ray-test cases etc. We'll use the sum of the squares of the boxes' surface areas
		// to balance the scoring.
		score[axis] = 0;
		if(objectsToTheLeft[axis]) score[axis] += leftBB[axis].GetSurfaceArea() * leftBB[axis].GetSurfaceArea();
		if(objectsToTheRight[axis]) score[axis] += rightBB[axis].GetSurfaceArea() * rightBB[axis].GetSurfaceArea();
	}

	// KJL - 12/05/2002 17:46:29 - pick the best axis based on the calculated scores
	int axis = 2;
	if (score[0]<score[1])
	{
		if (score[0]<score[2])
		{
			axis = 0;
		}
	}
	else
	{
		if (score[1]<score[2])
		{
			axis = 1;
		}
	}

	if (objectsToTheLeft[axis] == 0 || objectsToTheRight[axis] == 0)
	{
		// KJL - 12/05/2002 17:51:18 - after all that, we've ended up with an empty node...
		// let's take the first object and arbitrarily put that in the left node on its
		// own to make a leaf node, and put the rest in the right node. (In this situation
		// I expect that we're down to a small number of objects anyway.)
	
		for (int i=start; i<end; i++)
		{
			NewObjectSet[i] = OldObjectSet[i];
		}
		
		objectsToTheLeft[axis] = 1;
		leftBB[axis].Reset();
		leftBB[axis].Set(LeafAABB[NewObjectSet[start]]);
	
		objectsToTheRight[axis] = end-start-1;
		rightBB[axis].Reset();
		for (int i=start+1; i<end; i++)
		{
			unsigned int objectIndex = NewObjectSet[i];
			rightBB[axis].Set(LeafAABB[objectIndex]);
		}
	}
	else
	{
		// KJL - 12/05/2002 17:52:13 - okay, let's sort the objects into two sets
		float midpoint;
		switch(axis)
		{
			default:
			case 0:
			{
				midpoint = (parentBB.MaxX+parentBB.MinX)*0.5f;
				break;
			}
			case 1:
			{
				midpoint = (parentBB.MaxY+parentBB.MinY)*0.5f;
				break;
			}
			case 2:
			{
				midpoint = (parentBB.MaxZ+parentBB.MinZ)*0.5f;
				break;
			}
		}
		int right = end;
		int left = start;

		for (int i=start; i<end; i++)
		{
			unsigned int objectIndex = OldObjectSet[i];
			if (LeafCentre[objectIndex].GetElement(axis) > midpoint)
			{
				// KJL - 12/05/2002 16:19:44 - in 'right' node
				--right;
				NewObjectSet[right] = objectIndex;
			}
			else
			{
				// KJL - 12/05/2002 16:19:57 - in 'left' node
				NewObjectSet[left] = objectIndex;
				++left;
			}
		}
	}
	for (int i=start; i<end; i++)
	{
		OldObjectSet[i] = NewObjectSet[i];
	}
		
	if (leftBB[axis].MinX!=parentBB.MinX)
	{
		Asura_Assert(rightBB[axis].MinX==parentBB.MinX,"");

		node.MinXBelongsToLeft = true;
		node.ChildExtents.MinX = leftBB[axis].MinX;
	}
	else
	{
		node.MinXBelongsToLeft = false;
		node.ChildExtents.MinX = rightBB[axis].MinX;
	}
	if (leftBB[axis].MaxX!=parentBB.MaxX)
	{
		Asura_Assert(rightBB[axis].MaxX==parentBB.MaxX,"");

		node.MaxXBelongsToLeft = true;
		node.ChildExtents.MaxX = leftBB[axis].MaxX;
	}
	else
	{
		node.MaxXBelongsToLeft = false;
		node.ChildExtents.MaxX = rightBB[axis].MaxX;
	}
	
	if (leftBB[axis].MinY!=parentBB.MinY)
	{
		Asura_Assert(rightBB[axis].MinY==parentBB.MinY,"");

		node.MinYBelongsToLeft = true;
		node.ChildExtents.MinY = leftBB[axis].MinY;
	}
	else
	{
		node.MinYBelongsToLeft = false;
		node.ChildExtents.MinY = rightBB[axis].MinY;
	}
	if (leftBB[axis].MaxY!=parentBB.MaxY)
	{
		Asura_Assert(rightBB[axis].MaxY==parentBB.MaxY,"");

		node.MaxYBelongsToLeft = true;
		node.ChildExtents.MaxY = leftBB[axis].MaxY;
	}
	else
	{
		node.MaxYBelongsToLeft = false;
		node.ChildExtents.MaxY = rightBB[axis].MaxY;
	}

		
	if (leftBB[axis].MinZ!=parentBB.MinZ)
	{
		Asura_Assert(rightBB[axis].MinZ==parentBB.MinZ,"");

		node.MinZBelongsToLeft = true;
		node.ChildExtents.MinZ = leftBB[axis].MinZ;
	}
	else
	{
		node.MinZBelongsToLeft = false;
		node.ChildExtents.MinZ = rightBB[axis].MinZ;
	}
	if (leftBB[axis].MaxZ!=parentBB.MaxZ)
	{
		Asura_Assert(rightBB[axis].MaxZ==parentBB.MaxZ,"");

		node.MaxZBelongsToLeft = true;
		node.ChildExtents.MaxZ = leftBB[axis].MaxZ;
	}
	else
	{
		node.MaxZBelongsToLeft = false;
		node.ChildExtents.MaxZ = rightBB[axis].MaxZ;
	}

	if (objectsToTheLeft[axis]==1)
	{
		node.LeftIsLeaf = true;
		node.LeftIndex = NewObjectSet[start];
	}
	else
	{
		node.LeftIsLeaf = false;
		node.LeftIndex = ++NodeIndex;
		SortObjectsIntoNodes(start,start+objectsToTheLeft[axis],leftBB[axis]);
	}

	if (objectsToTheRight[axis]==1)
	{
		node.RightIsLeaf = true;
		node.RightIndex = NewObjectSet[end-1];
	}
	else
	{
		node.RightIsLeaf = false;
		node.RightIndex = ++NodeIndex;
		SortObjectsIntoNodes(start+objectsToTheLeft[axis],end,rightBB[axis]);
	}

}
	
