//
// Tools_Prompted_Selection.h
//
// Created by Richard Rice - 21/05/03
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_PROMPTED_SELECTION_H__
#define __TOOLS_PROMPTED_SELECTION_H__


/*
A series of functions for getting the user to pick a single module/polygon/vertex for whatever reason.
While such a selection is in progress it will take precedence over the selection process in Tools_Module_Selection
and Tools_Polygon_Selection. RAR 21/05/03
*/

class Tools_Object;
struct Tools_LOS_EdgeOutput; 

class Tools_Prompted_Selection
{
public :
	static void Initialise();
	static void Shutdown();
	static void Update();
	static void Render();

	static bool IsSelecting(){return s_bSelecting;}
	//Cancel any selection in progess
	static void EndSelection();

	//Prompts the user to choose a module (by clicking on it) , and calls the callback function when this is done
	static void ChooseModule(const char* pszPrompt,void (*pfnModuleCallback)(int iModuleIndex));
	//Similarly for polygons
	static void ChoosePolygon(const char* pszPrompt,void (*pfnPolygonCallback)(int iModuleIndex,int iPolygonIndex));
	//And for vertices
	static void ChooseVertex(const char* pszPrompt,void (*pfnVertexCallback)(int iModuleIndex,int iVertexIndex),bool (*pfnVertexFilter)(int iModuleIndex,int iVertexIndex)=0);
	//and objects
	static void ChooseObject(const char* pszPrompt,void (*pfnObjectCallback)(Tools_Object* pxObject));
	//and edges
    static void ChooseEdge(const char* pszPrompt,void (*pfnEdgeCallback)(Tools_LOS_EdgeOutput xEdge), bool bAllowDiagonalInQuad = false);

    // And finally, custom stuff. Callbacks should retrieve cursor details
    // from Tools_LOS directly. When not processing a click the pick callback
    // should return true if the cursor is over something; if processing a
    // click then it should just deal with the consequences (and the return
    // value is ignored). The render callback is provided with a suggested
    // colour value for the highlight of whatever's being picked.
    static void ChooseCustom( const char* szPrompt, bool (*pfnCustomPickCallback)(bool bClicked), void (*pfnCustomRenderCallback)(u_int uSuggestedARGB) );

private :
	static void InitialiseSelection(const char* pszPrompt);

	static void (*s_pfnModuleCallback)(int iModuleIndex);
	static void (*s_pfnPolygonCallback)(int iModuleIndex,int iPolygonIndex);
	static void (*s_pfnVertexCallback)(int iModuleIndex,int iVertexIndex);
	static void (*s_pfnObjectCallback)(Tools_Object* pxObject);
	static void (*s_pfnEdgeCallback)(Tools_LOS_EdgeOutput xEdge);

	static bool (*s_pfnVertexFilter)(int iModuleIndex,int iVertexIndex);

    static bool (*s_pfnCustomPickCallback)( bool bClicked );
    static void (*s_pfnCustomRenderCallback)( u_int uARGB );

	static bool s_bSelecting;
	static bool s_bButtonDebounced;
	static int s_iModuleIndex;
	static int s_iPolygonIndex;
	static int s_iVertexIndex;
	static Tools_Object* s_pxObject;
	static Tools_LOS_EdgeOutput s_xEdge;
	static bool s_bEdgeSelect_AllowDiagonalInQuad;
};

#endif //__TOOLS_PROMPTED_SELECTION_H__
