//
// Tools_XMLExporter.h
//
// Created by HE - 21 May 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_XMLEXPORTER_H__
#define __TOOLS_XMLEXPORTER_H__

// forward declarations
enum Asura_GUIMenu_Object_Type;
class Asura_GUIMenu_Menu_Page;

//////////////////////////////////////////////////////////////////////////////
//
// Tools_XMLExporter
//
//////////////////////////////////////////////////////////////////////////////

class Tools_XMLExporter
{

public:
	Tools_XMLExporter(const char* szPath = NULL, const char* szStyleSheet = NULL);
	~Tools_XMLExporter();

	//works out the filename and then writes the file using WriteToFile()
	bool WriteXMLToFile( const char* szFileName );

	//adds data to the buffer
	void AddToBuffer( const Asura_Char* szInString );
	void AddToBuffer( const int iData );
	void AddToBuffer( const float fData );
	void AddToBuffer( const u_int uData );
	void AddToBuffer( const bool bData );

	//adds to the buffer as a hex value, for hashes
	void AddToBufferHex( const Asura_Hash_ID uData );

	//escapes out any illegal chars that could break the XML before adding to the buffer
	//use this when exporting file/menu/image names etc as opposed to XML tags
	void AddToBufferEscaped( const Asura_Char* szInString );

	//increase / decrease number of tabs up to a predefined limit
	void IncTabs();
	void DecTabs();

	char* GetTabs() {return m_szTabs; }

private:
	//no copying or assignment, please
	Tools_XMLExporter(Tools_XMLExporter const&);
	Tools_XMLExporter& operator=(const Tools_XMLExporter& xExporter);

	//actually writes the file
	bool WriteToFile(char* szFileName);

	//resets the buffer between exports
	void Clear();

	int m_iNumTabs;	//tab counter
	char* m_szTabs;

	unsigned long m_uBufferStringLength;

	//buffer
	char* m_szBuffer;
	unsigned long int m_uMaxBufferSize;
	char* m_pcBufferPos;

	//stylesheet and output paths. forward rather than backslashes please!
	//stylesheet example: "<?xml-stylesheet type='text/xsl' href='../../misc/blueprint_styles.xsl'?>"
	char* m_szStyleSheet;
	char* m_szPath;

};

#endif//__TOOLS_XMLEXPORTER_H__