#ifdef ASURA_USE_AXON
//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_Axon_Interface.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_OBJECT_AXON_INTERFACE_H__
#define __TOOLS_OBJECT_AXON_INTERFACE_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

// core
#include "Core/Asura_LinkList.h"
#include "Network/Asura_Guid.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

class ListSelection_Overlay;
class Asura_Overlay;
class Asura_ServerEntity_Node;
class Asura_Vector_2;
class Tools_Entity_Node;
class Tools_EntityNode_GS2Attractor;
class Tools_EntityNode_GS2Node;
class Tools_EntityNode_GS2Spline;
class Tools_ModeDataFile;

//////////////////////////////////////////////////////////////////////////////
//
// Tools_Object_Axon_Interface
//
//////////////////////////////////////////////////////////////////////////////

typedef void ( TOGV_NodeOwnerGuidFn( Asura_ServerEntity_Node* pxStartNode, const Asura_Guid uOwnerGuid ) );

class Tools_Object_Axon_Interface
{
public:
	static void Initialise();
	static void Shutdown();
	static Tools_Object_Axon_Interface* GetInstance() { return s_pxThis; }

	virtual void InitialiseMode() {}
	virtual void TransferModeData( Tools_ModeDataFile& xFile );
	virtual void Update();
	virtual void Render() {}
	virtual void ShutdownMode();

	virtual void BuildGS2StructureList( ListSelection_Overlay& xList, const Tools_EntityNode_GS2Node* pxGS2Node = NULL );
	virtual void BuildGS2NodeBehaviourSetList( ListSelection_Overlay& xList, const Tools_EntityNode_GS2Node* pxGS2Node = NULL );
	virtual void BuildGS2SplineBehaviourSetList( ListSelection_Overlay& xList, const Tools_EntityNode_GS2Spline* pxGS2Spline = NULL );
	virtual void BuildGS2AttractorAttractTypesList( ListSelection_Overlay& xList, const Tools_EntityNode_GS2Attractor* pxGS2Attractor = NULL );
	virtual void BuildGS2AttractorEnRouteBehaviourSetList( ListSelection_Overlay& xList, const Tools_EntityNode_GS2Attractor* pxGS2Attractor = NULL );
	virtual bool GetAnimSetTotalMovementOffset( const u_int uAnimSet, const Asura_String xSkinName, Asura_Vector_3& xPositionOffset, Asura_Quat& xQOrientOffset );

	virtual void EstablishGS2ObjectOwnerData();
	virtual void ForEachGS2StartNode( TOGV_NodeOwnerGuidFn* pfnNodeOwnerGuidCallback ) = 0;
	virtual void UpdateBehaviourTrees();

	static bool CanCreateGS2LinkBetweenNodes();
	static bool CanCreateGS2LinkedNode();
	virtual Tools_EntityNode_GS2Node*   CreateGS2Node();
	virtual Tools_EntityNode_GS2Spline* CreateGS2Link();
	void CreateGS2LinkBetweenNodes();
	void CreateGS2LinkedNode();

	// Preview stuff.
	static void AddPreviewControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset );
	static void IncrementNumPreviewing() { ++s_iNumPreviewing; }
	static void DecrementNumPreviewing();
	static bool ShouldPropagatePreview() { return s_bPreviewPropagate; }
	static bool ShouldLoopPreview()		 { return s_bPreviewLoop; }
	static void StopPreview();
	virtual void* GetPreviewScoringData() { return NULL; }

	// Define this to return the skin name associated with this gamescene user guid.
	// This is used to create an animation for gamescene anim previews, so the skin must be/have already been loaded in.
	virtual Asura_String GetSkinNameFromGuid( const Asura_Guid uGuid ) const = 0;

protected:
	static Tools_Object_Axon_Interface* Project_CreateAxonInterface();

	virtual void AddPreviewSettingsControls( Asura_Overlay* pxNewOverlay, Asura_Vector_2& xOffset );
	static void RequestRefreshPreviewSettingsOverlay() { s_bRefreshPreviewSettingsOverlay = true; }

private:
	struct GS2OwnerData
	{
		Asura_Guid   m_uGuid;
		Asura_String m_xSkinName;
	};
	static void SetGS2StructureOwnerData( Asura_ServerEntity_Node* pxStartNode, const Asura_Guid uOwnerGuid );
	static void SetGS2NodeOwnerData( Asura_ServerEntity_Node* pxNode, void* pUserData );
	static void SetGS2NodeOwnerSkinName( Asura_ServerEntity_Node* pxNode, void* pUserData );

	static bool CanStartPreview();
	static bool CanStopPreview();
	static void StartPreview();
	static bool NoPreviewInProgress() { return !s_bPreviewInProgress; }
	static void PreviewFinished();
	static bool PreviewSettingsOverlayOpen();
	static bool PreviewSettingsOverlayNotOpen() { return !PreviewSettingsOverlayOpen(); }
	static u_int GetPreviewSettingsOverlayID() { return reinterpret_cast< u_int >( s_pxThis ) + 1; }
	static void OpenPreviewSettingsOverlay();
	static void ClosePreviewSettingsOverlay();

	static void TransferOldOverlayData( Tools_ModeDataFile& xFile );

private:
	static bool	s_bPreviewInProgress;
	static bool	s_bPreviewStopping;
	static int	s_iNumPreviewing;
	static bool s_bRefreshPreviewSettingsOverlay;
	static Asura_Vector_2 s_xPreviewSettingsOverlayPosition;
	static bool	s_bPreviewPropagate;
	static bool	s_bPreviewLoop;
	static Asura_LinkList< Asura_Guid > s_xPreviewInitialObjects;

	static Tools_Object_Axon_Interface* s_pxThis;

	enum { CURRENT_TRANSFER_VERSION = 5 };
};

//////////////////////////////////////////////////////////////////////////////
//
// 
//
//////////////////////////////////////////////////////////////////////////////

#endif //__TOOLS_OBJECT_AXON_INTERFACE_H__
#endif // ASURA_USE_AXON
