//////////////////////////////////////////////////////////////////////////////
//
// Tools_Mode_Object.h
//
// Copyright (C) 2008 Rebellion. All Rights Reserved.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_MODE_OBJECT_H__
#define __TOOLS_MODE_OBJECT_H__ 1

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura_Handle.h"
#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Debugging/Asura_Assert.h"
#include "Tools_Mode.h"

// F O R W A R D   D E C L A R A T I O N S ///////////////////////////////////

template< class T > class List;

//////////////////////////////////////////////////////////////////////////////

struct ToolsObjectCreationInfo
{
	Asura_String m_xCategory;
	Asura_String m_xName;
	Asura_String m_xHelpText;
	int m_iToolsObjectType;
	void* m_pData;

	void (*m_pfnObjectCreate)();
	void (*m_pfnObjectCreateWithData)(void* pSetupData);
};

class Tools_Mode_Object : public Tools_Mode
{
public:
	Tools_Mode_Object()
	{
        s_pxThis = this;
	}
	~Tools_Mode_Object()
	{
        s_pxThis = NULL;
	}

    virtual void InitialiseMode(void);
	virtual void ShutdownMode(void);
    virtual void PreShutdown(void);

	virtual void Update(void);
	virtual void RenderWorld_PostToneMap(void);
	virtual void RenderUI(void);

	virtual void PreSave();
	virtual void QuickSave(void);

	virtual bool IsModified();

	//loads/saves data for this mode
	static void TransferModeData(int mode);

	static void ToggleObjectModeOverlay(void);
	static bool ObjectModeOverlayIsOn(void);
    static void CreateObjectModeOverlay(void);

	static void ToggleReportOverlay(void);
	static bool ReportOverlayIsOn(void);
	static void CreateReportOverlay(void);
	static void CreateObjectReport(void);
	static void CreateGlassInitOverlay(void);
	static void CloseGlassInitOverlay(void);
	static void CreateDefaultGlass(void);
	static void CreateGlassFromVertices(void);
	static bool CanCreateGlassFromVertices(void);
	static void ClearVertexSelection();

	static void VerifyObjects( int iAutomatic );
	static void FixNamePrefixes();

    virtual void AddItemsToObjectModeOverlay(class Asura_Overlay* pxNewOverlay, class Asura_Vector_2& xOffset);
    virtual bool HandleKeyboard();
	static void HandleVertexSelection();

	// EntityReceivesActivateMessage() returns true if the guid refers to an entity which is ever
	// activated. The base class version deals with static ACTIVATE messages and ACTIVATE messages
	// sent to proxy triggers which target the entity.
	virtual bool EntityReceivesActivateMessage( Asura_Guid uGuid ) const;

	//build list of object types available for creation in this mode
	virtual void BuildObjectCreationList();

	//Add an object creation function to list. should only be used during BuildObjectCreationList
	//pszCategory is the name of the category that the object should appear under. It can be put in more than one if desired
	//pszName is the name of the object type that will be displayed in the list
	//pszHelpText is option help text to be shown when the mouse is over the list entry
	//pfnObjectCreate is the function to be called when the Create button is used
	//iToolsObjectType is the tools object type of the object created , for possible future use.
	void AddObjectCreationFunctionMain(	ToolsObjectCreationInfo* pxInfo, const char* pszName,const char* pszCategory,const char* pszHelpText, int iToolsObjectType);
	void AddObjectCreationFunction(const char* pszName,const char* pszCategory,const char* pszHelpText,void (*pfnObjectCreate)(),int iToolsObjectType);
	void AddObjectCreationFunctionWithData(const char* pszName,const char* pszCategory,const char* pszHelpText,void (*pfnObjectCreate)(void* pSetupData),int iToolsObjectType, void* pData);

	void RemoveObjectCreationFunction( int iToolsObjectType );

	static void CreateObject_SnapToVertex(){CreateObject(true);}
	static void CreateObject_AtCamera(){CreateObject(false);}

protected:
	virtual bool DoVerificationTests( const bool bAutomatic );

private:
	ToolsObjectCreationInfo* GetObjectCreationFunction( int iToolsObjectType );

	void BuildObjectCategoryStrings();
	void UpdateObjectTypeList();

	static bool VerifyToolsObjects( const bool bAutomatic );
	static bool VerifyEntityMessages( const bool bAutomatic );
	static bool VerifyProjectors();
	#ifdef ASURA_USE_AXON
	static bool VerifyAnimTemplates();
	#endif

	static void OutputWaypointBlockingStats();

	static void CreateObject_DoubleClick(int iJunk) {CreateObject(true);}
	static void CreateObject(bool bSnapToVertex);

public:
	///////////////////////////
	//data
	static Asura_Handle MenuHandle;

	static Asura_Vector_2 ObjectModeOverlayPosition;
	static bool	ObjectModeOverlayOpen;

	static Asura_Vector_2 ReportOverlayPosition;
	static bool ReportOverlayOpen;

	static Asura_Vector_2 GlassOverlayPosition;
	static bool s_bSelectingVertices;
    
	static int s_iNumSelectedVertices;
	static int s_aiSelectedModule[4];
	static int s_aiSelectedVertices[4];

protected:
    static Tools_Mode_Object* s_pxThis;

	static List<ToolsObjectCreationInfo*> s_xObjectCreationList;
	static int s_iNumObjectCategories;
	static const char** s_ppszObjectCategoryNames;

	static int s_iNumObjectTypes;
	static const char** s_ppszObjectTypeNames;
	static const char** s_ppszObjectHelpText;
	static struct ToolsObjectCreationInfo** s_ppxObjectCreationArray;

	static int s_iSelectedObjectCategory;
	static int s_iSelectedObjectType;
	static bool s_bSelectedCategoryChanged;
	static class Asura_List_Dialog* s_pxObjectListDialog;
};

#endif //__TOOLS_MODE_OBJECT_H__
