//
// Tools_Object_Gizmo.cpp
//
// Created by Richard Rice - 06/09/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "Objects/Tools_Object_Gizmo.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"

#include "Tools_Overlays.h"
#include "Interface/Tools_ContextMenu_System.h"
#include "Objects/Tools_Object_System.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------**
** Meshes **
**--------*/
static Asura_Mesh* CreateMoveMesh()
{
	Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh("ToolsShape_MoveGizmo");
	if(pxMesh) return pxMesh;
	pxMesh = Asura_Mesh_System::GetMesh("ToolsShape_MoveGizmo",true);

	const float fHeight = 0.25f;
	const float fRadius = fHeight/2.0f;
	const int iSides = 8;

	pxMesh->VertexDataValid = true;
	pxMesh->NumberOfVertices = iSides + 2;
	pxMesh->VertexDataPtr = new Asura_Mesh_Vertex[pxMesh->NumberOfVertices];

	for(int i=0;i<pxMesh->NumberOfVertices;i++)
	{
		Asura_Vector_3* pxVert = &pxMesh->VertexDataPtr[i].Position;

		if(i==0)
		{
			//vertex at middle of base
			pxVert->Zero();
		}
		else if(i==1)
		{
			//vertex at point of cone
			pxVert->Zero();
			pxVert->z = fHeight;
		}
		else
		{
			//vertices along the rim of the base
			float fAngle = Asura_Maths::Deg360 * ((i-2)/(float)iSides);
			pxVert->x = Asura_Maths::Cosine(fAngle)*fRadius;
			pxVert->y = Asura_Maths::Sine(fAngle)*fRadius;
			pxVert->z = 0.0f;
		}
	}

	pxMesh->PolygonDataValid = true;
	pxMesh->NumberOfPolygons = iSides*2;
	pxMesh->NumberOfTriangles = pxMesh->NumberOfPolygons;

	pxMesh->PolygonDataPtr = new Asura_Mesh_Polygon[pxMesh->NumberOfPolygons];

	//create the triangles along the length of the cone
	for(int i=0;i<iSides;i++)
	{
		Asura_Mesh_Polygon* pxPoly = &pxMesh->PolygonDataPtr[i];
		pxPoly->VertexID[0] = (u_short)(i+2);		
		pxPoly->VertexID[1] = (u_short)(((i+1)%iSides)+2);
		pxPoly->VertexID[2] = 1;
		pxPoly->VertexID[3] = 0xffff;
	}

	//Create the triangles at the base
	for(int i=0;i<iSides;i++)
	{
		Asura_Mesh_Polygon* pxPoly = &pxMesh->PolygonDataPtr[i+iSides];
		pxPoly->VertexID[0] = (u_short)(((i+1)%iSides)+2);
		pxPoly->VertexID[1] = (u_short)(i+2);		
		pxPoly->VertexID[2] = 0;
		pxPoly->VertexID[3] = 0xffff;
	}

	//initialise texturing
	pxMesh->TextureDataValid = true;
	pxMesh->TextureDataPtr = new Asura_Mesh_TextureInfo[pxMesh->NumberOfPolygons];

	//smoothing
	pxMesh->SmoothingGroupDataValid = true;
	pxMesh->SmoothingGroupDataPtr = new int[pxMesh->NumberOfPolygons];
	for(int i=0;i<pxMesh->NumberOfPolygons;i++)
	{
		pxMesh->SmoothingGroupDataPtr[i] = 0;
	}

	return pxMesh;
}

static Asura_Mesh* Create2AxisMoveMesh()
{
	Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh("ToolsShape_2AxisMoveGizmo");
	if(pxMesh) return pxMesh;
	pxMesh = Asura_Mesh_System::GetMesh("ToolsShape_2AxisMoveGizmo",true);

	Asura_Vector_3 xExtents(0.05f,0.05f,0.35f);

	pxMesh->VertexDataValid = true;
	pxMesh->NumberOfVertices = 8;
	pxMesh->VertexDataPtr = new Asura_Mesh_Vertex[pxMesh->NumberOfVertices];

	for(int i=0;i<pxMesh->NumberOfVertices;i++)
	{
		Asura_Vector_3* pxVert = &pxMesh->VertexDataPtr[i].Position;

		pxVert->x = (i & 1) ? -xExtents.x : xExtents.x;
		pxVert->y = (i & 2) ? -xExtents.y : xExtents.y;
		pxVert->z = (i & 4) ? xExtents.x : xExtents.z;
	}

	pxMesh->PolygonDataValid = true;
	pxMesh->NumberOfPolygons = 6;
	pxMesh->NumberOfTriangles = 12;

	pxMesh->PolygonDataPtr = new Asura_Mesh_Polygon[pxMesh->NumberOfPolygons];

	
	short asIndeces[24] =
	{
		0,1,3,2,
		1,5,7,3,
		5,4,6,7,
		4,0,2,6,
		0,4,5,1,
		2,3,7,6
	};
	for(int i=0;i<6;i++)
	{
		Asura_Mesh_Polygon* pxPoly = &pxMesh->PolygonDataPtr[i];
		pxPoly->VertexID[0] = asIndeces[i*4];		
		pxPoly->VertexID[1] = asIndeces[i*4+1];		
		pxPoly->VertexID[2] = asIndeces[i*4+2];		
		pxPoly->VertexID[3] = asIndeces[i*4+3];		
	}


	//initialise texturing
	pxMesh->TextureDataValid = true;
	pxMesh->TextureDataPtr = new Asura_Mesh_TextureInfo[pxMesh->NumberOfPolygons];

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	const Asura_Hash_ID uMaterial = Asura_GetHashID("Tools\\Colours");
	for(int i=0; i<pxMesh->NumberOfPolygons;i++)
	{
		Asura_Mesh_TextureInfo* pxTex = &pxMesh->TextureDataPtr[i];
		for(int j=0;j<4;j++)
		{
			pxTex->U[0][j] = 7.5f/8.0f;
			pxTex->V[0][j] = 0.5f/8.0f;
		}
		pxTex->m_auMaterialResponseHashID[0] = uMaterial;
	}
	#endif


	//smoothing
	pxMesh->SmoothingGroupDataValid = true;
	pxMesh->SmoothingGroupDataPtr = new int[pxMesh->NumberOfPolygons];
	for(int i=0;i<pxMesh->NumberOfPolygons;i++)
	{
		pxMesh->SmoothingGroupDataPtr[i] = 0;
	}

	return pxMesh;
}


static Asura_Mesh* CreateRotateMesh()
{
	Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh("ToolsShape_RotateGizmo");
	if(pxMesh) return pxMesh;
	pxMesh = Asura_Mesh_System::GetMesh("ToolsShape_RotateGizmo",true);

	Asura_Vector_3 xExtents(0.05f,0.2f,0.1f);

	pxMesh->VertexDataValid = true;
	pxMesh->NumberOfVertices = 8;
	pxMesh->VertexDataPtr = new Asura_Mesh_Vertex[pxMesh->NumberOfVertices];

	for(int i=0;i<pxMesh->NumberOfVertices;i++)
	{
		Asura_Vector_3* pxVert = &pxMesh->VertexDataPtr[i].Position;

		pxVert->x = (i & 1) ? -xExtents.x : xExtents.x;
		pxVert->y = (i & 2) ? -xExtents.y : xExtents.y;
		pxVert->z = (i & 4) ? -xExtents.z : xExtents.z;
	}

	pxMesh->PolygonDataValid = true;
	pxMesh->NumberOfPolygons = 6;
	pxMesh->NumberOfTriangles = 12;

	pxMesh->PolygonDataPtr = new Asura_Mesh_Polygon[pxMesh->NumberOfPolygons];

	
	short asIndeces[24] =
	{
		0,1,3,2,
		1,5,7,3,
		5,4,6,7,
		4,0,2,6,
		0,4,5,1,
		2,3,7,6
	};
	for(int i=0;i<6;i++)
	{
		Asura_Mesh_Polygon* pxPoly = &pxMesh->PolygonDataPtr[i];
		pxPoly->VertexID[0] = asIndeces[i*4];		
		pxPoly->VertexID[1] = asIndeces[i*4+1];		
		pxPoly->VertexID[2] = asIndeces[i*4+2];		
		pxPoly->VertexID[3] = asIndeces[i*4+3];		
	}


	//initialise texturing
	pxMesh->TextureDataValid = true;
	pxMesh->TextureDataPtr = new Asura_Mesh_TextureInfo[pxMesh->NumberOfPolygons];


	//smoothing
	pxMesh->SmoothingGroupDataValid = true;
	pxMesh->SmoothingGroupDataPtr = new int[pxMesh->NumberOfPolygons];
	for(int i=0;i<pxMesh->NumberOfPolygons;i++)
	{
		pxMesh->SmoothingGroupDataPtr[i] = 0;
	}

	return pxMesh;
}

static Asura_Mesh* CreateScaleMesh()
{
	Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh("ToolsShape_ScaleGizmo");
	if(pxMesh) return pxMesh;
	pxMesh = Asura_Mesh_System::GetMesh("ToolsShape_ScaleGizmo",true);

	Asura_Vector_3 xExtents(0.1f,0.1f,0.1f);

	pxMesh->VertexDataValid = true;
	pxMesh->NumberOfVertices = 8;
	pxMesh->VertexDataPtr = new Asura_Mesh_Vertex[pxMesh->NumberOfVertices];

	for(int i=0;i<pxMesh->NumberOfVertices;i++)
	{
		Asura_Vector_3* pxVert = &pxMesh->VertexDataPtr[i].Position;

		pxVert->x = (i & 1) ? -xExtents.x : xExtents.x;
		pxVert->y = (i & 2) ? -xExtents.y : xExtents.y;
		pxVert->z = (i & 4) ? -xExtents.z : xExtents.z;
	}

	pxMesh->PolygonDataValid = true;
	pxMesh->NumberOfPolygons = 6;
	pxMesh->NumberOfTriangles = 12;

	pxMesh->PolygonDataPtr = new Asura_Mesh_Polygon[pxMesh->NumberOfPolygons];

	
	short asIndeces[24] =
	{
		0,1,3,2,
		1,5,7,3,
		5,4,6,7,
		4,0,2,6,
		0,4,5,1,
		2,3,7,6
	};
	for(int i=0;i<6;i++)
	{
		Asura_Mesh_Polygon* pxPoly = &pxMesh->PolygonDataPtr[i];
		pxPoly->VertexID[0] = asIndeces[i*4];		
		pxPoly->VertexID[1] = asIndeces[i*4+1];		
		pxPoly->VertexID[2] = asIndeces[i*4+2];		
		pxPoly->VertexID[3] = asIndeces[i*4+3];		
	}


	//initialise texturing
	pxMesh->TextureDataValid = true;
	pxMesh->TextureDataPtr = new Asura_Mesh_TextureInfo[pxMesh->NumberOfPolygons];


	//smoothing
	pxMesh->SmoothingGroupDataValid = true;
	pxMesh->SmoothingGroupDataPtr = new int[pxMesh->NumberOfPolygons];
	for(int i=0;i<pxMesh->NumberOfPolygons;i++)
	{
		pxMesh->SmoothingGroupDataPtr[i] = 0;
	}

	return pxMesh;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------**
** Tools_Gizmo_Handler **
**---------------------*/

bool Tools_Gizmo_Handler::s_bWorldAxis = false;
bool Tools_Gizmo_Handler::s_bUniformScale = false;
float Tools_Gizmo_Handler::s_fSnapMoveGranularity = 0.5f;
Asura_Angle Tools_Gizmo_Handler::s_fSnapRotateGranularity = Asura_Maths::DegToRad(15.0f);
Asura_Vector_2 Tools_Gizmo_Handler::s_xOverlayPosition(10.0f,10.0f);
float Tools_Gizmo_Handler::s_fGizmoScale = 1.0f;

Tools_Object* Tools_Gizmo_Handler::s_pxCurrentObject;
List<Tools_Object_Gizmo*> Tools_Gizmo_Handler::s_xGizmoList;

void Tools_Gizmo_Handler::ToggleGizmos(Tools_Object* pxObject)
{
	//delete current gizmos regardless
	Tools_Object* pxPrevObject = s_pxCurrentObject;
	DeleteGizmos();

	//If they were for this object ,or no object is selected ,then we are done
	if(!pxObject || pxObject == pxPrevObject) return;
	if(!pxObject->CanBeMoved()) return;

	//Need to make sure the object being moved is the main selection , for the group movement
	//stuff to work.
	if(!pxObject->IsSelected())
	{
		if(pxObject->IsGroupSelected())
		{
			//reselect this as the main selection
			Tools_Object_System::SelectObject(pxObject,true);
		}
		else
		{
			//wasn't previously selected at all ,so replace the current selection
			Tools_Object_System::SelectObject(pxObject,false);

		}
	}


	s_pxCurrentObject = pxObject;
	//create movement gizmos
	for(int i=0;i<3;i++)
	{
		new Tools_Object_MoveGizmo(pxObject,i);
		new Tools_Object_RotateGizmo(pxObject,i);
		new Tools_Object_ScaleGizmo(pxObject,i);

		new Tools_Object_2AxisMoveGizmo(pxObject,i,(i+1)%3);
		new Tools_Object_2AxisMoveGizmo(pxObject,i,(i+2)%3);

	}

	//Make sure the relative positions are up to date , to allow for group movement via the gizmos
	Tools_Object_System::UpdateRelativePositions();
}

void Tools_Gizmo_Handler::DeleteGizmos()
{
	while(s_xGizmoList.size())
	{
		Tools_Object_System::DeleteObject(s_xGizmoList.first_entry());
	}

	s_pxCurrentObject = 0;
}

void Tools_Gizmo_Handler::ObjectDeleted(Tools_Object* pxObject)
{
	if(pxObject == s_pxCurrentObject)
	{
		DeleteGizmos();
	}
}

void Tools_Gizmo_Handler::ShowOverlay()
{
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_GIZMOPROPERTIES))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_GIZMOPROPERTIES);
	}
	else
	{
		Asura_Overlay* pxOverlay = new Asura_Overlay("Gizmo Properties",false,&s_xOverlayPosition);	

		Asura_Dialog* pxDialog;
		Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

		pxDialog = new Asura_Numerical_Dialog("Snapped Movement",xOffset,&s_fSnapMoveGranularity,0.0f,999.0f,0.1f,"%.3f");
		pxDialog->SetHelpText("The movement granularity to use while control is held down");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_AngleNumerical_Dialog("Snapped Rotation",xOffset,&s_fSnapRotateGranularity,0.0f,360.0f,0.1f,"%.3f");
		pxDialog->SetHelpText("The rotation granularity to use while control is held down");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Uniform Scale",xOffset,&s_bUniformScale);
		pxDialog->SetHelpText("Determines whether the scale gizmos should be applied to all axes uniformly.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Use World Axes",xOffset,&s_bWorldAxis);
		pxDialog->SetHelpText("Determines whether local or world axes are used.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Gizmo Scale",xOffset,&s_fGizmoScale, 0.01f, 999.0f,0.1f,"%.3f");
		pxDialog->SetHelpText("Scale factor for determining how large make the gizmos on screen.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();


		pxOverlay->m_uID = TOOLS_OVERLAY_GIZMOPROPERTIES;
		Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
	}
}

void Tools_Gizmo_Handler::CloseOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_GIZMOPROPERTIES);
}

void Tools_Gizmo_Handler::SelectionChanged(Tools_Object* pxNewSelection)
{
	if(s_xGizmoList.size() && pxNewSelection && s_pxCurrentObject)
	{
		//move the gizmos to the new selection
		s_pxCurrentObject = pxNewSelection;
		for(LIF<Tools_Object_Gizmo*> xGizmoLIF(&s_xGizmoList);!xGizmoLIF.done();xGizmoLIF.next())
		{
			xGizmoLIF()->SetControlledObject(pxNewSelection);
		}

		//Make sure the relative positions are up to date , to allow for group movement via the gizmos
		Tools_Object_System::UpdateRelativePositions();

	}
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------**
** Tools_Object_Gizmo **
**--------------------*/

static float g_fLastScale = 1.0f;
static u_int g_uDragFrameStamp = 0xffffffff;

Tools_Object_Gizmo::Tools_Object_Gizmo(Tools_Object* pxParent,int iAxis,float fSpokePos)
:m_pxControlledObject(pxParent)
,m_iAxis(iAxis)
,m_fSpokePosition(fSpokePos)
{
	Tools_Gizmo_Handler::s_xGizmoList.add_entry(this);

	Type = TOT_MovementGizmo;
	
	Asura_Assert(iAxis>=0 && iAxis<=2,"");
	UpdatePosition();
	//Virtual function , so orientation won't be set correctly on rotation gizmos until first update
	Tools_Object_Gizmo::UpdateOrientation();

	u_int uColour;
	if(m_iAxis == 0) uColour = 0xffff0000;
	else if(m_iAxis == 1) uColour = 0xff00ff00;
	else uColour = 0xff0000ff;
	SetColour(uColour);
}

Tools_Object_Gizmo::~Tools_Object_Gizmo()
{
	Tools_Gizmo_Handler::s_xGizmoList.delete_entry(this);
}


void Tools_Object_Gizmo::UpdatePosition()
{
	//Get the direction of the axis
	Asura_Vector_3 xAxisDir = GetAxisDir();
	
	#if 1
	float fScale;

	if(g_uDragFrameStamp == Asura_Timers::GetFrameNumber() || g_uDragFrameStamp == Asura_Timers::GetFrameNumber()-1)
	{
		//Keep the previous scale while dragging
		fScale = g_fLastScale;
	}
	else
	{
		//Get a scaling factor to use based on the distance from the camera
		float fDistance = (m_pxControlledObject->Position - *Asura_Camera::GetPosition()).Magnitude();
		fDistance *= Asura_Camera::GetLODMetric();

		fScale = fDistance/10.0f;
		g_fLastScale = fScale;
	}
	fScale *= Tools_Gizmo_Handler::GetGizmoScale();
	float fBaseDist = fScale;

	#else
	//Get a scaling factor to use based on the bounding box extents
	Asura_Bounding_Box xBox;
	m_pxControlledObject->GetBoundingBox(xBox);

	float fBaseDist = xBox.MaxX;
	if(-xBox.MinY > fBaseDist) fBaseDist = -xBox.MinY;
	if(xBox.MaxZ > fBaseDist) fBaseDist = xBox.MaxZ;
	if(fBaseDist < 0.5f) fBaseDist = 0.5f;
	fBaseDist += 0.5f;

	float fScale = fBaseDist / 2.0f;	
	if(fScale < 1.0f) fScale = 1.0f;//don't let them get too small
	#endif

	//Position relative to the parent object's position
	Position = m_pxControlledObject->Position + xAxisDir*(fBaseDist + m_fSpokePosition*fScale);

	//set size of gizmo
	SetScale(fScale);
}

void Tools_Object_Gizmo::UpdateOrientation()
{
	Asura_Vector_3 xAxisDir = GetAxisDir();
	Asura_Maths::OrientationFromDirection(Orientation,xAxisDir);
}

void Tools_Object_Gizmo::Render()
{
	//render line from centre of parent's bounding box to this
	u_int uColour;
	if(m_iAxis == 0) uColour = 0xffff0000;
	else if(m_iAxis == 1) uColour = 0xff00ff00;
	else uColour = 0xff0000ff;

	Asura_Render::RenderLine(Position,m_pxControlledObject->Position,uColour);

	Tools_Object::Render();
}

void Tools_Object_Gizmo::Update()
{
	UpdatePosition();
	UpdateOrientation();
}

Asura_Vector_3 Tools_Object_Gizmo::GetAxisDir(int iAxis) const
{
	Asura_Assert(iAxis>=0 && iAxis<=2,"");
	
	Asura_Vector_3 xAxisDir(0.0f,0.0f,0.0f);
	xAxisDir[iAxis] = (iAxis == 1) ? -1.0f : 1.0f;

	if(!Tools_Gizmo_Handler::IsWorldAxis())
	{
		xAxisDir = xAxisDir * m_pxControlledObject->Orientation;
	}

	return xAxisDir;
}

void Tools_Object_Gizmo::AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext)
{
	Tools_ContextMenu_Section* pxSection = xMenu.GetSection(TOOLS_CONTEXTMENU_SECTION_OBJECT);
	pxSection->AddSeparator();

	Tools_ContextMenu_Item* pxItem;
	
	pxItem = new Tools_ContextMenu_Item_Function("Show Gizmo Overlay",ContextMenu_ShowOverlay);
	pxSection->AddItem(pxItem);

	//Not calling Tools_Object::AddContextMenuItems() since none of the options are relevant
}

void Tools_Object_Gizmo::ContextMenu_ShowOverlay()
{
	Tools_Gizmo_Handler::ShowOverlay();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------**
** Tools_Object_MoveGizmo **
**------------------------*/


Tools_Object_MoveGizmo::Tools_Object_MoveGizmo(Tools_Object* pxParent,int iAxis)
:Tools_Object_Gizmo(pxParent,iAxis,0.1f)
,m_bDragOkay(false)
{
	//SetMesh(Asura_Mesh_System::GetMesh("ToolsShape_CutsceneTrackPoint"));
	SetMesh(CreateMoveMesh());
}

void Tools_Object_MoveGizmo::StartDrag(const Asura_Vector_2& xCursorPos)
{
	m_xInitialObjectPos = m_pxControlledObject->Position;
	m_bDragOkay = GetPointNearestCursor(xCursorPos,m_xInitialPoint);
}

void Tools_Object_MoveGizmo::UpdateDrag(const Asura_Vector_2& xCursorPos)
{
	if(!m_bDragOkay) return	;
	g_uDragFrameStamp = Asura_Timers::GetFrameNumber();

	//Get the nearest point to the cursor
	Asura_Vector_3 xPos2;
	if(!GetPointNearestCursor(xCursorPos,xPos2)) return;

	//set the object position based on the offset between the positions
	float fSnapGranularity = Tools_Gizmo_Handler::GetSnapMoveGranularity();
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && fSnapGranularity > 0.0f)
	{
		//snap movement to a multiple of the granularity
		Asura_Vector_3 xOffset = xPos2-m_xInitialPoint;
		float fDistance = xOffset.Magnitude();
		if(fDistance > 0.0f)
		{
			fDistance = Asura_Maths::FloatToInt_Near(fDistance/fSnapGranularity)*fSnapGranularity;
			xOffset.Normalise();
			m_pxControlledObject->Position = m_xInitialObjectPos + xOffset*fDistance;
		}
	}
	else
	{
		m_pxControlledObject->Position = m_xInitialObjectPos + xPos2-m_xInitialPoint;
	}

	m_pxControlledObject->UpdatePositionInAsuraData(true);
	Tools_Object_System::UpdateGroupMovement();
}

bool Tools_Object_MoveGizmo::GetPointNearestCursor( const Asura_Vector_2& xCursorPos,Asura_Vector_3& xOutput)
{
	//get line for cursor in world space
	Asura_Vector_3 xCursorLineStart = *Asura_Camera::GetPosition();
	Asura_Vector_3 xCursorLineDir;

	const Asura_ViewingArea* pxArea = Asura_Camera::GetViewingArea();
	float fViewX = xCursorPos.x/Asura_Viewport::GetWidth();
	float fViewY = xCursorPos.y/Asura_Viewport::GetHeight();
	float fRatio = pxArea->GetAspectRatio()*Asura_Camera::GetPhysicalScreenAspectRatio();

	xCursorLineDir.x = fViewX - 0.5f;
	xCursorLineDir.y = (fViewY - 0.5f)/fRatio;
	xCursorLineDir.z = 0.5f*Asura_Camera::GetOneOverTanHalfFovX();

	xCursorLineDir = xCursorLineDir * *Asura_Camera::GetOrientation();
	xCursorLineDir.Normalise();

	Asura_Vector_3 xAxisDir = GetAxisDir();

	//get the perpendicual to the axis and the cursor directions
	Asura_Vector_3 xNorm = xCursorLineDir^xAxisDir;
	float fNormMag = xNorm.Magnitude();
	if(fNormMag < 0.01f) return false;
	xNorm/=fNormMag;
	//get the perp to that normal and the cursor direction
	Asura_Vector_3 xNorm2 = xNorm^xCursorLineDir;
	xNorm2.Normalise();

	//get the distance between the two lines in the direction of xNorm2
	float fDistance = (xCursorLineStart - m_xInitialObjectPos)*xNorm2;
	//get the component of the axis direction in xNorm2
	float fDistance2 = xAxisDir*xNorm2;

	xOutput = m_xInitialObjectPos + xAxisDir*(fDistance/fDistance2);

	return true;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------**
** Tools_Object_2AxisMoveGizmo **
**-----------------------------*/


Tools_Object_2AxisMoveGizmo::Tools_Object_2AxisMoveGizmo(Tools_Object* pxParent,int iAxis, int iAxis2)
:Tools_Object_Gizmo(pxParent,iAxis,0.0f)
,m_bDragOkay(false)
,m_iSecondaryAxis(iAxis2)
{
	SetMesh(Create2AxisMoveMesh());

	//set the colour based on the 2 axes being used.
	//looks fairly nasty in practice ,but may make it more obvious which are equivalent
	u_int uColour = 0xff000000;
	if(iAxis==0 || iAxis2==0) uColour |= 0x800000;
	if(iAxis==1 || iAxis2==1) uColour |= 0x008000;
	if(iAxis==2 || iAxis2==2) uColour |= 0x000080;
	SetColour(uColour);

	//set the name based on the 2 axes
	char acName[100];
	int iMinAxis = Asura_Maths::Min(iAxis,iAxis2);
	int iMaxAxis = Asura_Maths::Max(iAxis,iAxis2);
	Asura_PrintfToBfr(acName,"Move Gizmo %c%c", sizeof(acName), true, 'X'+iMinAxis, 'X'+iMaxAxis);
	m_xName = acName;
	
}

void Tools_Object_2AxisMoveGizmo::StartDrag(const Asura_Vector_2& xCursorPos)
{
	m_xInitialObjectPos = m_pxControlledObject->Position;
	m_xInitialGizmoPos = Position;
	m_bDragOkay = GetPointNearestCursor(xCursorPos,m_xInitialPoint);
}

void Tools_Object_2AxisMoveGizmo::UpdateDrag(const Asura_Vector_2& xCursorPos)
{
	if(!m_bDragOkay) return	;
	g_uDragFrameStamp = Asura_Timers::GetFrameNumber();

	//Get the nearest point to the cursor
	Asura_Vector_3 xPos2;
	if(!GetPointNearestCursor(xCursorPos,xPos2)) return;

	//set the object position based on the offset between the positions
	float fSnapGranularity = Tools_Gizmo_Handler::GetSnapMoveGranularity();
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && fSnapGranularity > 0.0f)
	{
		//snap movement to a multiple of the granularity
		Asura_Vector_3 xOffset = xPos2-m_xInitialPoint;
		float fDistance = xOffset.Magnitude();
		if(fDistance > 0.0f)
		{
			fDistance = Asura_Maths::FloatToInt_Near(fDistance/fSnapGranularity)*fSnapGranularity;
			xOffset.Normalise();
			m_pxControlledObject->Position = m_xInitialObjectPos + xOffset*fDistance;
		}
	}
	else
	{
		m_pxControlledObject->Position = m_xInitialObjectPos + xPos2-m_xInitialPoint;
	}

	m_pxControlledObject->UpdatePositionInAsuraData(true);
	Tools_Object_System::UpdateGroupMovement();
}

bool Tools_Object_2AxisMoveGizmo::GetPointNearestCursor( const Asura_Vector_2& xCursorPos,Asura_Vector_3& xOutput)
{
	//get line for cursor in world space
	Asura_Vector_3 xCursorLineStart = *Asura_Camera::GetPosition();
	Asura_Vector_3 xCursorLineDir;

	const Asura_ViewingArea* pxArea = Asura_Camera::GetViewingArea();
	float fViewX = xCursorPos.x/Asura_Viewport::GetWidth();
	float fViewY = xCursorPos.y/Asura_Viewport::GetHeight();
	float fRatio = pxArea->GetAspectRatio()*Asura_Camera::GetPhysicalScreenAspectRatio();

	xCursorLineDir.x = fViewX - 0.5f;
	xCursorLineDir.y = (fViewY - 0.5f)/fRatio;
	xCursorLineDir.z = 0.5f*Asura_Camera::GetOneOverTanHalfFovX();

	xCursorLineDir = xCursorLineDir * *Asura_Camera::GetOrientation();
	xCursorLineDir.Normalise();

	Asura_Vector_3 xPlaneNormal = GetAxisDir() ^ GetAxisDir(m_iSecondaryAxis);

	//get the component of the line in the normal direction
	float fComponent = xPlaneNormal * xCursorLineDir;
	if(Asura_Maths::Abs(fComponent) < 0.01f) return false;

	//get the distance to travel in the normal direction
	float fDistance = (m_xInitialGizmoPos - xCursorLineStart)*xPlaneNormal;

	xOutput = xCursorLineStart + xCursorLineDir*(fDistance/fComponent);

	return true;
}

void Tools_Object_2AxisMoveGizmo::UpdateOrientation()
{
	Asura_Vector_3 xAxisDir = GetAxisDir(m_iSecondaryAxis);
	Asura_Maths::OrientationFromDirection(Orientation,xAxisDir);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------**
** Tools_Object_RotateGizmo **
**--------------------------*/


Tools_Object_RotateGizmo::Tools_Object_RotateGizmo(Tools_Object* pxParent,int iAxis)
:Tools_Object_Gizmo(pxParent,iAxis,0.8f)
,m_bDragOkay(false)
{
	//SetMesh(Asura_Mesh_System::GetMesh("ToolsShape_CutsceneTrackPoint"));
	SetMesh(CreateRotateMesh());
}

void Tools_Object_RotateGizmo::StartDrag(const Asura_Vector_2& xCursorPos)
{
	m_xInitialObjectOrient = m_pxControlledObject->Orientation;
	m_xRotateAxisDir = GetAxisDir((GetAxis()+2)%3);

	m_bDragOkay = GetPointOnPlane(xCursorPos,m_xInitialPoint);
}

void Tools_Object_RotateGizmo::UpdateDrag(const Asura_Vector_2& xCursorPos)
{
	if(!m_bDragOkay) return;

	//Get the current point on the rotation plane
	Asura_Vector_3 xInitialPoint,xCurrentPoint;
	if(!GetPointOnPlane(xCursorPos,xCurrentPoint)) return;

	//get unit vectors from the object to these points
	Asura_Vector_3 xDir1 = m_xInitialPoint - m_pxControlledObject->Position;
	float fMag1 = xDir1.Magnitude();
	if(fMag1 < 0.01f) return;
	xDir1 /= fMag1;

	Asura_Vector_3 xDir2 = xCurrentPoint - m_pxControlledObject->Position;
	float fMag2 = xDir2.Magnitude();
	if(fMag2 < 0.02f) return;
	xDir2 /= fMag2;

	//determine the angle between these
	Asura_Vector_3 xCross = xDir1^xDir2;
	float fAngle = Asura_Maths::ArcSine(xCross.Magnitude());

	if(xDir1*xDir2 < 0.0f)
	{
		fAngle = Asura_Maths::Deg180 - fAngle;
	}


	//Get the direction according to whether the cross product is in the same direction as the rotation axis
	if(xCross * m_xRotateAxisDir < 0.0f) fAngle = -fAngle;

	float fSnapGranularity = Tools_Gizmo_Handler::GetSnapRotateGranularity();
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && fSnapGranularity > 0.0f)
	{
		//snap to nearest multiple of the snap granularity
		fAngle = Asura_Maths::FloatToInt_Near(fAngle/fSnapGranularity)*fSnapGranularity;
	}


	Asura_Quat xQuat;
	xQuat.FromRotation(fAngle,m_xRotateAxisDir);

	Asura_Matrix_3x3 xRotate;
	xQuat.ToMatrix(xRotate);

	//apply the rotation to the object
	m_pxControlledObject->Orientation = xRotate*m_xInitialObjectOrient;
	m_pxControlledObject->UpdatePositionInAsuraData(true);
	Tools_Object_System::UpdateGroupMovement();

}

bool Tools_Object_RotateGizmo::GetPointOnPlane(const Asura_Vector_2& xCursorPos,Asura_Vector_3& xOutput)
{
	//This finds the point on the plane perpendicular to the angle of rotation that the cursor intersects.

	//Get the cursor direction in world space
	Asura_Vector_3 xCursorLineStart = *Asura_Camera::GetPosition();
	Asura_Vector_3 xCursorLineDir;

	const Asura_ViewingArea* pxArea = Asura_Camera::GetViewingArea();
	float fViewX = xCursorPos.x/Asura_Viewport::GetWidth();
	float fViewY = xCursorPos.y/Asura_Viewport::GetHeight();
	float fRatio = pxArea->GetAspectRatio()*Asura_Camera::GetPhysicalScreenAspectRatio();

	xCursorLineDir.x = fViewX - 0.5f;
	xCursorLineDir.y = (fViewY - 0.5f)/fRatio;
	xCursorLineDir.z = 0.5f*Asura_Camera::GetOneOverTanHalfFovX();

	xCursorLineDir = xCursorLineDir * *Asura_Camera::GetOrientation();
	xCursorLineDir.Normalise();

	//Get the distance to the rotation plane
	Asura_Vector_3 xOffset = m_pxControlledObject->Position-xCursorLineStart;
	float fPerpDist = xOffset * m_xRotateAxisDir;

	float fDirDot = xCursorLineDir * m_xRotateAxisDir;
	if(Asura_Maths::Abs(fDirDot) < 0.01f) return false;// abort if the direction is too close to being parallel

	xOutput =  xCursorLineStart + xCursorLineDir * (fPerpDist/fDirDot);

	return true;
}

void Tools_Object_RotateGizmo::UpdateOrientation()
{
	Asura_Vector_3 xZDir = GetAxisDir();
	Asura_Vector_3 xXDir = GetAxisDir((GetAxis()+2)%3);
	Asura_Vector_3 xYDir = xZDir ^ xXDir;

	Orientation.e11 = xXDir.x;
	Orientation.e21 = xXDir.y;
	Orientation.e31 = xXDir.z;
	Orientation.e12 = xYDir.x;
	Orientation.e22 = xYDir.y;
	Orientation.e32 = xYDir.z;
	Orientation.e13 = xZDir.x;
	Orientation.e23 = xZDir.y;
	Orientation.e33 = xZDir.z;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------**
** Tools_Object_ScaleGizmo **
**-------------------------*/


Tools_Object_ScaleGizmo::Tools_Object_ScaleGizmo(Tools_Object* pxParent,int iAxis)
:Tools_Object_Gizmo(pxParent,iAxis,1.3f)
,m_bDragOkay(false)
{
	SetMesh(CreateScaleMesh());
}

void Tools_Object_ScaleGizmo::StartDrag(const Asura_Vector_2& xCursorPos)
{
	m_xInitialScale = m_pxControlledObject->GetScale();
	m_xInitialObjectPos = m_pxControlledObject->Position;
	m_bDragOkay = GetCursorDistanceFromObject(xCursorPos,m_fInitialDistance);
}

void Tools_Object_ScaleGizmo::UpdateDrag(const Asura_Vector_2& xCursorPos)
{
	if(!m_bDragOkay) return;

	//get the current distance of the cursor from the object
	float fDist2;
	if(!GetCursorDistanceFromObject(xCursorPos,fDist2)) return;


	float fScaleMult = Asura_Maths::Abs(fDist2/m_fInitialDistance);
	Asura_Vector_3 xScale = m_xInitialScale;

	

	if(Tools_Gizmo_Handler::IsUniformScale())
	{
		//alter the scale components equally
		xScale *= fScaleMult;
	}
	else
	{
		//just affect the one scale axis
		xScale[GetAxis()] *= fScaleMult;
			
	}
	m_pxControlledObject->SetScale(xScale);
	m_pxControlledObject->UpdateScaleInAsuraData();

}

bool Tools_Object_ScaleGizmo::GetCursorDistanceFromObject( const Asura_Vector_2& xCursorPos,float& fOutput)
{
	//get line for cursor in world space
	Asura_Vector_3 xCursorLineStart = *Asura_Camera::GetPosition();
	Asura_Vector_3 xCursorLineDir;

	const Asura_ViewingArea* pxArea = Asura_Camera::GetViewingArea();
	float fViewX = xCursorPos.x/Asura_Viewport::GetWidth();
	float fViewY = xCursorPos.y/Asura_Viewport::GetHeight();
	float fRatio = pxArea->GetAspectRatio()*Asura_Camera::GetPhysicalScreenAspectRatio();

	xCursorLineDir.x = fViewX - 0.5f;
	xCursorLineDir.y = (fViewY - 0.5f)/fRatio;
	xCursorLineDir.z = 0.5f*Asura_Camera::GetOneOverTanHalfFovX();

	xCursorLineDir = xCursorLineDir * *Asura_Camera::GetOrientation();
	xCursorLineDir.Normalise();

	Asura_Vector_3 xAxisDir = GetAxisDir();
	//get the perpendicual to the axis and the cursor directions
	Asura_Vector_3 xNorm = xCursorLineDir^xAxisDir;
	float fNormMag = xNorm.Magnitude();
	if(fNormMag < 0.01f) return false;
	xNorm/=fNormMag;
	//get the perp to that normal and the cursor direction
	Asura_Vector_3 xNorm2 = xNorm^xCursorLineDir;
	xNorm2.Normalise();

	//get the distance between the two lines in the direction of xNorm2
	float fDistance = (xCursorLineStart - m_xInitialObjectPos)*xNorm2;
	//get the component of the axis direction in xNorm2
	float fDistance2 = xAxisDir*xNorm2;

	fOutput = (fDistance/fDistance2);

	return true;
}

bool Tools_Object_ScaleGizmo::IsHidden()
{
	if(!m_pxControlledObject->CanBeScaled()) return true;
	//The scaling only really makes sense if using local axes
	if(Tools_Gizmo_Handler::IsWorldAxis() && !Tools_Gizmo_Handler::IsUniformScale()) return true;
	return Tools_Object_Gizmo::IsHidden();
}