//
// Tools_PositionEdit_Object.h
//
// Created by Richard Rice - 29/01/2003
// 
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_POSITIONEDIT_OBJECT_H_
#define _TOOLS_POSITIONEDIT_OBJECT_H_

#include "Objects/Tools_Object.h"
#include "Core/Asura_String.h"

/*
Object for editing a position and possibly an orientation value for whatever purposes

Since it keeps hold of the position and orientation pointers it is obviously necessary to make
sure it gets deleted before they become invalid. This can be done by storing the objects ID value
and calling Delete(unsigned uID) when finished. (It is possible that the object will be deleted by the user before this,
so storing the pointer isn't an option);
*/

class Tools_PositionEdit_Object : public Tools_Object
{
	public :
		Tools_PositionEdit_Object(Asura_Vector_3* pxPosition,Asura_Matrix_3x3* pxOrientation = NULL,const char* pszName = NULL,const char* pszMessage = NULL,bool* pbModifiedFlag = 0);

		//set a direction vector to be updated by the object (and also for setting initial orientation)
		void SetDirectionToEdit(Asura_Vector_3* pxDirection);
		// Similarly, if the calling class wants to update a quaternion instead of a matrix.
		void SetQuatToEdit(Asura_Quat* pxOriQuat);

		//gets the ID of the object for the next few functions...
		unsigned GetID(){return m_uID;}

		//Find the position edit object with this ID , if it still exists
		static Tools_PositionEdit_Object* Find(unsigned uID);
		//Deletes the position edit object with this ID , if it still exists
		static void Delete(unsigned uID);


	protected :	

		virtual void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);
		virtual const char* GetName(){return m_xName.GetString();}
		virtual const char* GetTypeDescription(){return "Position Edit";}

		virtual void PrintInfo(float& x,float& y);
		virtual bool ConfirmDestruction(){return false;}
		virtual bool IsTemporaryEditObject() const {return true;}


		Asura_String m_xName;
		Asura_String m_xMessage;
		int m_uID;
		//pointers to position and orientation values that this object should update
		Asura_Vector_3* m_pxPosition;
		Asura_Matrix_3x3* m_pxOrientation;
		Asura_Vector_3* m_pxDirection;
		Asura_Quat* m_pxOriQuat;

		bool* m_pbModifiedFlag;


		static unsigned s_uNextID;

};

#endif //_TOOLS_POSITIONEDIT_OBJECT_H_
