//////////////////////////////////////////////////////////////////////
// Tools_VolumeBaseEdit_Object.h								//
// SJL 14/11/07														//
// Copyright (C) 2007 Rebellion, All Rights Reserved.			    //
//////////////////////////////////////////////////////////////////////


#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef _TOOLS_VOLUMEBASEEDIT_OBJECT_H_
#define _TOOLS_VOLUMEBASEEDIT_OBJECT_H_

#include "Core/Asura_String.h"
#include "Objects/Tools_Object.h"

class Asura_Volume;
class Tools_VolumeGroupEdit_Controller;

class Tools_VolumeBaseEdit_Object : public Tools_Object
{
private:
	typedef Tools_Object PARENT;
public :

	////////////////////////////
	// Constructor/Destructor //
	////////////////////////////

	Tools_VolumeBaseEdit_Object(const char* name, Asura_Volume* pxVolume, bool* pbModifiedFlag = 0);
	~Tools_VolumeBaseEdit_Object();

	///////////////
	// Accessors //
	///////////////

	//Set the object that owns this so it can be reselected when this objects editing is done.
	void SetOwner(Tools_Object* pxObject){m_pxParentObject = pxObject;}

	//Sets the orientation to edit ,and allows the volume to be orientable 
	void SetVolumeOrientation(Asura_Quat* pxOrientQuat);
	void SetVolumeOrientation(Asura_Matrix_3x3* pxOrientMat);

	//Sets/Gets controller for handling multiple volumes, so it can be informed when this object is selected/deselected.
	void SetController(Tools_VolumeGroupEdit_Controller* pxController){ m_pxController = pxController; }
	Tools_VolumeGroupEdit_Controller* GetController() const { return m_pxController; }

	//gets the ID of the object, each is assigned a unique ID when created.
	u_int GetID() const {return m_uID;}

	/////////////////////////////
	// Tools Virtual functions //
	/////////////////////////////

	// See Tools docs ;)
	virtual void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/) = 0;	// Update game entities position from tools objects position.
	virtual void OnSelection();
	virtual void OnDeselection();
	virtual void OnReselection();
	virtual void UpdateWhileSelected();
	virtual void Render() = 0;
	virtual void PrintInfo(float& x,float& y);
	virtual bool ConfirmDestruction(){return false;}
	virtual bool UsesLeftButton();
	virtual bool DisallowSelectionOfOtherTypes() const { return true; }
	virtual bool IsTemporaryEditObject() const {return true;}
	virtual const char* GetName(){return m_xName.GetString();}
	virtual const char* GetTypeDescription(){return "Volume";}
	virtual bool ObjectIsInLOS(const Asura_Vector_3& xStart, const Asura_Vector_3& xDir, float& fLowest) = 0;
	virtual void Destroy();

	virtual void SnapFaceToCursor(const bool bAlsoExpandOppositeFace) = 0;
	virtual void SelectFaceUnderCursor( const bool bGetFurthest ) = 0;

	/////////////////////
	// Other Functions //
	/////////////////////

	// Creates or destroys the tools overlay.
	void CreateOverlay();
	void DeleteOverlay();
	
	// Move object
	virtual void Displace(const Asura_Vector_3& xOffset) =0;
	virtual void RotateAbout(const Asura_Vector_3& xCentre, const Asura_Matrix_3x3& xOri) =0;

	//////////////////////
	// Static Functions //
	//////////////////////

	//Find the object with this ID in the tools_object list.
	static Tools_VolumeBaseEdit_Object* Find(u_int uID);

	//Deletes the volume edit object with this ID by calling Tools_Object_System DeleteObject on it
	static void Delete(u_int uID);

	// Get the currently selected volume.
	static Tools_VolumeBaseEdit_Object* GetSelectedVolume() {return s_pxSelectedVolume;}

protected :

	///////////////
	// Accessors //
	///////////////

	//Gets a position in the space of the unrotated volume , relative to the object's position
	Asura_Vector_3 GetRelativePosition(const Asura_Vector_3& xPosition) const;

	//returns true if a orientation pointer has been given so as to allow orientation editing
	bool OrientationEditingAllowed() const {return m_pxOrientQuat || m_pxOrientMat;}

	/////////////////////////////
	// Tools Virtual functions //
	/////////////////////////////

	// Overlay stuff
	virtual void AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);
	virtual Asura_Vector_2* GetOverlayPosition() {return &s_xOverlayPosition;}

	/////////////////////
	// Other Functions //
	/////////////////////

	// Called by static done function.
	void Done();

	// Add and remove help text.
	void AddVolumeEditHelp();
	void RemoveVolumeEditHelp();
	void ShowVolumeEditHelp();

	virtual void SetDefaultPositionFromVolume() =0;

	
	// May always be exited.
	virtual bool AllowEndVolumeEditing() const {return true;}

	
	
	//////////////////////
	// Static Functions //
	//////////////////////

	// Display the keys
	//	static void ShowVolumeEditHelp();

	// Called by dialogue close.
	static void Done(int iThis){((Tools_VolumeBaseEdit_Object*)iThis)->Done();}
	static void ShowVolumeEditHelp(int iThis){((Tools_VolumeBaseEdit_Object*)iThis)->ShowVolumeEditHelp();}

	// Sets the selected volume
	static void SetSelectedVolume(Tools_VolumeBaseEdit_Object* pxSelectedVolume) { s_pxSelectedVolume = pxSelectedVolume; }


	//////////
	// Data	//
	//////////

	// Name for display purposes
	Asura_String 				m_xName; 

	//optional orientation to edit
	Asura_Quat*					m_pxOrientQuat; 
	Asura_Matrix_3x3*			m_pxOrientMat; 

	 // Unique ID (Kind of like a GUID).
	u_int							m_uID;

	// Pointer to volume group edit object controlling this object
	Tools_VolumeGroupEdit_Controller* m_pxController; 

	// pointer to modified flag passed down through constructors.
	bool*						m_pbModifiedFlag;

	// various flags to mark if the object has changed
	bool						m_bChanged;
	bool						m_bUseOrientation;
	bool						m_bUseOrientationChanged;

	// Parent tools object
	Tools_Object*				m_pxParentObject;

	/////////////////
	// Static Data //
	/////////////////

	// Unique ID to assign to next created object.
	static u_int s_uNextID;   

	// Wireframe rendering on or off
	static bool	 s_bWireframe;

	// Backup of Tools_Object_System::DoingSnapToVertex,
	// so that we don't pay attention to the Esc key
	// when it is used to cancel snap to vertex
	static bool s_bIsDoingSnapToVertex;

	// Overlay position
	static Asura_Vector_2 s_xOverlayPosition; 	

	// static pointer to currently selected volume - set and unset in OnSelection/OnDeselection
	static Tools_VolumeBaseEdit_Object* s_pxSelectedVolume;


	const char*		    m_pszHelpTitle;
	const char**		m_apszHelpTexts;
};

#endif // _TOOLS_VOLUMEBASEEDIT_OBJECT_H_