//
// Tools_VolumeEdit_Object.cpp
//
// Created by Richard Rice - 10/05/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Objects/Tools_VolumeEdit_Object.h"


#include "Camera/Asura_Camera.h"
#include "Core/Asura_Los.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Cursor.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Render.h"

#include "Tools_Overlays.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_LOS.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Objects/Tools_Object_System.h"


Tools_VolumeEdit_Object* Tools_VolumeEdit_Object::s_pxThisVolume = 0;
Asura_Vector_2 Tools_VolumeEdit_Object::s_xOverlayPosition(10,10);
unsigned Tools_VolumeEdit_Object::s_uNextID = 1;
bool Tools_VolumeEdit_Object::s_bWireframe = false;
static const float s_fMaxExtents = 30000.f; // Fairly arbitary.

static const char* s_apszVolumeEditHelpStrings [] =
{
	"Esc :           End volume editing",
	"",
	"Z :             Set bounding box to match module under cursor",
	"X :             Expand bounding box to include module",
	"9 :             Move selected face in",
	"Shift+ 9 :      Move selected face in (faster)",
	"0 :             Move selected face out",
	"Shift + 0 :     Move selected face out (faster)",
	"B :             Snap selected face to point under cursor",
	"G :             As above ,but also moves the opposite face in the opposite direction",
	"",
	"LMB :           Select nearest face under cursor",
	"Shift+LMB :     Select furthest face under cursor",
	"1 :             Select Min X face",
	"2 :             Select Max X face",
	"3 :             Select Min Y face",
	"4 :             Select Max Y face",
	"5 :             Select Min Z face",
	"6 :             Select Max Z face",
	0
};



Tools_VolumeEdit_Object::Tools_VolumeEdit_Object(const char* name,Asura_Bounding_Box* volume,Tools_VolumeEdit_Object** referenceToThis,bool* pbModifiedFlag /*=0*/)
:Tools_Object(TOT_VolumeEdit)
{
	m_xName = name;
	m_pxVolume = volume;
	m_xWorldVolume = *m_pxVolume;
	m_bChanged = false;
	m_eSelectedFace = FACE_NONE;
//	m_pfSelectedFaceValue = 0;
	m_uID = s_uNextID++;
	m_pxParentObject = NULL;
	m_xOffset.Zero();
	m_pxOrientQuat = NULL;
	m_pxOrientMat = NULL;
	m_bUseOrientation = false;
	m_bUseOrientationChanged = false;
	m_bOverlayVolumeChanged = false;
	m_bMinOverlayVolumeChanged = false;
	m_bMaxOverlayVolumeChanged = false;
	m_bEditingRelativeVolume = false;
	m_pxController = 0;

	m_xMaxExtents = Asura_Vector_3( s_fMaxExtents ,s_fMaxExtents, s_fMaxExtents );
	m_xMinExtents = Asura_Vector_3( -s_fMaxExtents, -s_fMaxExtents, -s_fMaxExtents );

	Asura_String mesh_name = "ToolsShape_CutsceneTrackPoint";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);

	SetDefaultPositionFromVolume();

	m_pbModifiedFlag = pbModifiedFlag;
	m_ppxReferenceToThis = referenceToThis;
	if(m_ppxReferenceToThis) *m_ppxReferenceToThis = this;

}

Tools_VolumeEdit_Object::Tools_VolumeEdit_Object(const char* pszName)
{

	m_xName = pszName;
	m_pxVolume = 0;
	m_xRelativeVolume.Zero();
	m_xWorldVolume.Zero();
	m_xOverlayVolume.Zero();
	m_bChanged = false;
	m_eSelectedFace = FACE_NONE;
//	m_pfSelectedFaceValue = 0;
	m_uID = s_uNextID++;
	m_pxParentObject = NULL;
	m_xOffset.Zero();
	m_pxOrientQuat = NULL;
	m_pxOrientMat = NULL;
	m_bUseOrientation = false;
	m_bUseOrientationChanged = false;
	m_bOverlayVolumeChanged = false;
	m_bMinOverlayVolumeChanged = false;
	m_bMaxOverlayVolumeChanged = false;
	m_bEditingRelativeVolume = false;
	m_pxController = 0;

	m_xMaxExtents = Asura_Vector_3( s_fMaxExtents, s_fMaxExtents, s_fMaxExtents );
	m_xMinExtents = Asura_Vector_3( -s_fMaxExtents, -s_fMaxExtents, -s_fMaxExtents );

	Mesh = Asura_Mesh_System::GetMesh("ToolsShape_CutsceneTrackPoint");

//	UpdateWorldVolumeFromRelative();

	m_pbModifiedFlag = 0;
	m_ppxReferenceToThis = 0;
//	if(m_ppxReferenceToThis) *m_ppxReferenceToThis = this;
}


Tools_VolumeEdit_Object::~Tools_VolumeEdit_Object()
{
	if(m_ppxReferenceToThis)
	{
		Asura_Assert(*m_ppxReferenceToThis==this,"");
		//clear the pointer to this volume
		*m_ppxReferenceToThis = 0;
	}
}

void Tools_VolumeEdit_Object::SetVolumeOrientation(Asura_Quat* pxOrientQuat)
{
	m_pxOrientQuat = pxOrientQuat;
	m_pxOrientMat = NULL;

	m_bUseOrientation = m_pxOrientQuat && !(*m_pxOrientQuat == Asura_Maths::IdentityQuat);
	
	SetDefaultPositionFromVolume();

}

void Tools_VolumeEdit_Object::SetVolumeOrientation(Asura_Matrix_3x3* pxOrientMat)
{
	m_pxOrientQuat = NULL;
	m_pxOrientMat = pxOrientMat;

	m_bUseOrientation = m_pxOrientMat && *m_pxOrientMat != Asura_Maths::IdentityMatrix3x3;

	SetDefaultPositionFromVolume();

}






void Tools_VolumeEdit_Object::UpdatePositionInAsuraData(bool /*updateLinkedObjects*/)
{
	UpdateFromRelativeVolume();
}

void Tools_VolumeEdit_Object::UpdateFromRelativeVolume()
{
	UpdateWorldVolumeFromRelative();
	
	if(m_bEditingRelativeVolume)
	{
		*m_pxVolume = m_xRelativeVolume;
	}
	else
	{
		*m_pxVolume = m_xRelativeVolume;
		m_pxVolume->Displace(Position-m_xOffset);
	
		if(m_bUseOrientation)
		{
			if(m_pxOrientQuat)
			{
				m_pxOrientQuat->FromMatrix(Orientation);
			}
			else if(m_pxOrientMat)
			{
				*m_pxOrientMat = Orientation;
			}
			//If editing the orientation , then the bounding box will need to be offset to take into account
			//that it will be using a different pivot point(the centre of the volume) than the volume object
			Asura_Vector_3 xOffset = GetRelativePivot();
			m_pxVolume->Displace(-xOffset);
			xOffset = xOffset*Orientation;
			m_pxVolume->Displace(xOffset);

			

		}
		else
		{
			if(m_pxOrientQuat)
			{
				*m_pxOrientQuat = Asura_Maths::IdentityQuat;
			}
			else if(m_pxOrientMat)
			{
				*m_pxOrientMat = Asura_Maths::IdentityMatrix3x3;
			}
		}
	}
	
	if(m_pbModifiedFlag) *m_pbModifiedFlag = true;
}

void Tools_VolumeEdit_Object::SetDefaultPositionFromVolume()
{
	if(m_pxOrientMat)
	{
		Orientation = *m_pxOrientMat;
	}
	else if(m_pxOrientQuat)
	{
		m_pxOrientQuat->ToMatrix(Orientation);	
	}
	else
	{
		Orientation.Identity();
	}



	Position = m_xWorldVolume.CalculateCentre();
	if(PreferPositionAtBaseOfVolume())
	{
		Position.y = m_xWorldVolume.MaxY;
		if(Mesh && Mesh->BoundingBoxDataValid)
		{
			//offset position by MaxY of the mesh , so the bottom face lies on the
			//bottom face of the bounding box
			float fOffset = Mesh->BoundingBox.MaxY;
			Position.y -= fOffset;
			if(Position.y<m_xWorldVolume.MinY) Position.y = m_xWorldVolume.MinY;

		}
	}
	m_xRelativeVolume = m_xWorldVolume;
	m_xRelativeVolume.Displace(-Position);
	m_xOverlayVolume = m_xRelativeVolume;

	//negate the minimum valuse , to make them more convenient for the overlay controls
	m_xOverlayVolume.MinX = -m_xOverlayVolume.MinX;
	m_xOverlayVolume.MinY = -m_xOverlayVolume.MinY;
	m_xOverlayVolume.MinZ = -m_xOverlayVolume.MinZ;
	m_bOverlayVolumeChanged = false;
	m_bMinOverlayVolumeChanged = false;
	m_bMaxOverlayVolumeChanged = false;

	if(m_bUseOrientation)
	{
		Asura_Vector_3 xCentre = GetWorldPivot();
		Asura_Vector_3 xOffset = xCentre-Position;
		xOffset = xOffset*Orientation;

		Position = xCentre-xOffset;
	}

	Position +=m_xOffset;

}

void Tools_VolumeEdit_Object::UpdateWorldVolumeFromRelative()
{
	m_xWorldVolume = m_xRelativeVolume;
	m_xWorldVolume.Displace(Position-m_xOffset);

	if(m_bUseOrientation)
	{
		//If editing the orientation , then the bounding box will need to be offset to take into account
		//that it will be using a different pivot point(the centre of the volume) than the volume object
		Asura_Vector_3 xOffset = GetRelativePivot();
		m_xWorldVolume.Displace(-xOffset);
		xOffset = xOffset*Orientation;
		m_xWorldVolume.Displace(xOffset);
	}
}


void Tools_VolumeEdit_Object::OnSelection()
{
	if(m_pxController) m_pxController->VolumeSelected(this);

	s_pxThisVolume = this;
	CreateOverlay();

	AddVolumeEditHelp();	
}


void Tools_VolumeEdit_Object::OnDeselection()
{
	DeleteOverlay();
	s_pxThisVolume = 0;

	RemoveVolumeEditHelp();
}

void Tools_VolumeEdit_Object::AddVolumeEditHelp()
{
	Tools_Help::AddHelpTopic("Volume Editing",s_apszVolumeEditHelpStrings);
}

void Tools_VolumeEdit_Object::RemoveVolumeEditHelp()
{
	Tools_Help::RemoveHelpTopic("Volume Editing");
}


void Tools_VolumeEdit_Object::CreateOverlay()
{
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay(GetTypeDescription(),false,GetOverlayPosition());
	
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	AddOverlayControls(pxOverlay,xOffset);


	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;

	pxOverlay->m_xOrigin = *GetOverlayPosition();
 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_VolumeEdit_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	const char* pszExtentText = "Volume extents (metres) relative to the volume marker object.";

	pxDialog = new Asura_Numerical_Dialog("1: Min X",xOffset,&m_xOverlayVolume.MinX,m_xMinExtents.x,m_xMaxExtents.x,0.1f,"%.3f",&m_bMinOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("2: Max X",xOffset,&m_xOverlayVolume.MaxX,m_xMinExtents.x,m_xMaxExtents.x,0.1f,"%.3f",&m_bMaxOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("3: Min Y",xOffset,&m_xOverlayVolume.MinY,m_xMinExtents.y,m_xMaxExtents.y,0.1f,"%.3f",&m_bMinOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Numerical_Dialog("4: Max Y",xOffset,&m_xOverlayVolume.MaxY,m_xMinExtents.y,m_xMaxExtents.y,0.1f,"%.3f",&m_bMaxOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Numerical_Dialog("5: Min Z",xOffset,&m_xOverlayVolume.MinZ,m_xMinExtents.z,m_xMaxExtents.z,0.1f,"%.3f",&m_bMinOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Numerical_Dialog("6: Max Z",xOffset,&m_xOverlayVolume.MaxZ,m_xMinExtents.z,m_xMaxExtents.z,0.1f,"%.3f",&m_bMaxOverlayVolumeChanged);
	pxDialog->SetHelpText(pszExtentText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	if(OrientationEditingAllowed())
	{
		pxDialog = new Asura_Tickbox_Dialog("Use Orientation",xOffset,&m_bUseOrientation,&m_bUseOrientationChanged);
		pxDialog->SetHelpText("Volume will take its orientation from the volume marker object.\nBy default the volume is aligned to the world axes.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
	
	pxDialog = new Asura_Tickbox_Dialog("Wireframe",xOffset,&s_bWireframe);
	pxDialog->SetHelpText("Draw the volume as a wireframe model.\nThis is a global setting for all volume objects.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	if(AllowEndVolumeEditing())
	{
		pxDialog = new Asura_Function_Dialog("End volume editing",xOffset,Done,(int) this);
		pxDialog->SetHelpText("Deletes all the volume editing objects and reselects the owner of this volume.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	if(m_pxController)
	{
		m_pxController->AddOverlayControls(pxOverlay,xOffset);
	}

	//xOffset.y+=Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();
    //pxDialog = new Asura_Dialog( "Press Ctrl+H for Keys", xOffset );
    //pxOverlay->AddDialog(pxDialog);
	//xOffset.y+=Asura_Dialog::DefaultSpacing();

	xOffset.y+=Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();
    pxDialog = new Asura_Function_Dialog( "Show Keys", xOffset, ShowKeys );
	pxDialog->SetHelpText("Shows all the shortcut keys");
    pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//set things up so that right clicking on the overlay will be equivalent to right clicking on the object
	//RAR 06/11/2007
	pxOverlay->SetContextMenuCallback(Tools_Object_System::AddContextMenuItemsFromOverlay, this);


}


void Tools_VolumeEdit_Object::DeleteOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
}


void Tools_VolumeEdit_Object::Render()
{
	//render the flyable bit
	Tools_Object::Render();

    Asura_Render::CurrentStates.RequestWireFrameMode(false);

	u_int uColour;
	u_int uSelectedFaceColour;
	u_int uWireframeColour;	//colour to use as additional highlight when doing solid volumes
	if(IsSelected())
	{
		uColour = 0xafafff;
		uSelectedFaceColour = 0xff0000;
		uWireframeColour = 0x0000ff;
	}
	else if(m_pxController && (m_pxController->IsSelected() || m_pxController->IsHighlighted()))
	{
		uColour = 0xafffaf;
		uSelectedFaceColour = uColour;
		uWireframeColour = 0x00ff00;
	}
	else
	{
		uColour = 0xafafaf;
		uSelectedFaceColour = uColour;
		uWireframeColour = 0xffffff;
	}
	if(s_bWireframe)
	{
		uColour |= 0xff000000;
		uSelectedFaceColour |= 0xff000000;
	}
	else
	{
		uColour |= 0x70000000;
		uSelectedFaceColour |= 0x70000000;
	}
	
	Asura_Matrix_3x3 xOrient = m_bUseOrientation ? Orientation : Asura_Maths::IdentityMatrix3x3;
	Tools_Render::RenderCube(m_xRelativeVolume,Position,xOrient,uColour,s_bWireframe,m_eSelectedFace,uSelectedFaceColour);

	if(!s_bWireframe)
	{
		//do a wireframe outline as well
		Tools_Render::RenderCube(m_xRelativeVolume,Position,xOrient,uWireframeColour|0xff000000,true,m_eSelectedFace,uSelectedFaceColour|0xff000000);
	}
	

	Asura_Render::CurrentStates.RequestWireFrameMode(Asura_Render::GetWireframe());
}


void Tools_VolumeEdit_Object::PrintInfo(float& x,float& y)
{
	Tools_Object::PrintInfo(x,y);
	if(m_pxController)
	{
		m_pxController->PrintInfo(x,y);
	}
}


void Tools_VolumeEdit_Object::UpdateWhileSelected()
{
	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{


 		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_ALT) && !Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
		{
			if (!Asura_Interface::UsingMenusOrOverlays())
			{
            	if( Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) )
            	{
            	    SelectFaceUnderCursor( Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT ) );
            	}
			}

            if ( !Asura_UserInput::KeyPressed( ASURA_KEYID_SHIFT ) )
            {
			    if(Asura_UserInput::DebouncedKeyPressed('x'))
			    {
				    ExpandToIncludeModule();
			    }
			    if(Asura_UserInput::DebouncedKeyPressed('z'))
			    {
				    SetToModule();
			    }
                if(Asura_UserInput::KeyPressed('b'))
                {
                    SnapFaceToCursor(false);
                }
                if(Asura_UserInput::KeyPressed('g'))
                {
                    SnapFaceToCursor(true);
                }

			    if(Asura_UserInput::DebouncedKeyPressed('1'))
			    {
				    SetSelectedFace(FACE_MIN_X);
			    }
			    if(Asura_UserInput::DebouncedKeyPressed('2'))
			    {
				    SetSelectedFace(FACE_MAX_X);
			    }
			    if(Asura_UserInput::DebouncedKeyPressed('3'))
			    {
				    SetSelectedFace(FACE_MIN_Y);
			    }
			    if(Asura_UserInput::DebouncedKeyPressed('4'))
			    {
				    SetSelectedFace(FACE_MAX_Y);
			    }
			    if(Asura_UserInput::DebouncedKeyPressed('5'))
			    {
				    SetSelectedFace(FACE_MIN_Z);
			    }
			    if(Asura_UserInput::DebouncedKeyPressed('6'))
			    {
				    SetSelectedFace(FACE_MAX_Z);
			    }
                if( Asura_UserInput::DebouncedKeyPressed('7') ||
                    Asura_UserInput::DebouncedKeyPressed('8') )
                {
                    SetSelectedFace(FACE_NONE);
                }


            }
			//increase/decrease side.
			if(m_eSelectedFace != FACE_NONE)
			{
				float* pfSelectedFaceValue = 0;
				float* pfOppositeFaceValue = 0;

				switch(m_eSelectedFace)
				{
					case FACE_MIN_X :
						pfSelectedFaceValue = &m_xOverlayVolume.MinX;
						pfOppositeFaceValue = &m_xOverlayVolume.MaxX;
						break;
					case FACE_MIN_Y :
						pfSelectedFaceValue = &m_xOverlayVolume.MinY;
						pfOppositeFaceValue = &m_xOverlayVolume.MaxY;
			            break;
					case FACE_MIN_Z :
						pfSelectedFaceValue = &m_xOverlayVolume.MinZ;
						pfOppositeFaceValue = &m_xOverlayVolume.MaxZ;
			            break;
					case FACE_MAX_X :
						pfSelectedFaceValue = &m_xOverlayVolume.MaxX;
						pfOppositeFaceValue = &m_xOverlayVolume.MinX;
			            break;
					case FACE_MAX_Y :
						pfSelectedFaceValue = &m_xOverlayVolume.MaxY;
						pfOppositeFaceValue = &m_xOverlayVolume.MinY;
			            break;
			        case FACE_MAX_Z :
						pfSelectedFaceValue = &m_xOverlayVolume.MaxZ;
						pfOppositeFaceValue = &m_xOverlayVolume.MinZ;
			            break;
				}	
				


				float fAdjustment = 0;
				if(Asura_UserInput::KeyPressed('0'))
				{
					fAdjustment = Asura_Timers::GetFrameTime();
				}
				if(Asura_UserInput::KeyPressed('9'))
				{
					fAdjustment = -Asura_Timers::GetFrameTime();
				}
				if(fAdjustment)
				{
					if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
					{
						fAdjustment*=10.0f;
					}
					*pfSelectedFaceValue += fAdjustment;
					if(*pfSelectedFaceValue<-*pfOppositeFaceValue) *pfSelectedFaceValue = -*pfOppositeFaceValue;
					m_bOverlayVolumeChanged = true;
					m_bChanged = true;
				}
			}

            //Escape is equivalent to pressing the Done button.
            if ( Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_ESCAPE ) && AllowEndVolumeEditing() && !Tools_Help::IsDisplayingHelp())
			{
				Done();
				return;
			}

		}

	}

	if(m_bMinOverlayVolumeChanged)
	{
		if(m_xOverlayVolume.MinX<-m_xOverlayVolume.MaxX) m_xOverlayVolume.MinX=-m_xOverlayVolume.MaxX;
		if(m_xOverlayVolume.MinY<-m_xOverlayVolume.MaxY) m_xOverlayVolume.MinY=-m_xOverlayVolume.MaxY;
		if(m_xOverlayVolume.MinZ<-m_xOverlayVolume.MaxZ) m_xOverlayVolume.MinZ=-m_xOverlayVolume.MaxZ;
		m_bMinOverlayVolumeChanged = false;
		m_bOverlayVolumeChanged = true;
	}
	else if(m_bMaxOverlayVolumeChanged)
	{
		if(m_xOverlayVolume.MaxX<-m_xOverlayVolume.MinX) m_xOverlayVolume.MaxX=-m_xOverlayVolume.MinX;
		if(m_xOverlayVolume.MaxY<-m_xOverlayVolume.MinY) m_xOverlayVolume.MaxY=-m_xOverlayVolume.MinY;
		if(m_xOverlayVolume.MaxZ<-m_xOverlayVolume.MinZ) m_xOverlayVolume.MaxZ=-m_xOverlayVolume.MinZ;
		m_bMaxOverlayVolumeChanged = false;
		m_bOverlayVolumeChanged = true;
	}

	if(m_bOverlayVolumeChanged)
	{
		//update relative volume from overlay volume
		m_xRelativeVolume = m_xOverlayVolume;
		m_xRelativeVolume.MinX = - m_xRelativeVolume.MinX;
		m_xRelativeVolume.MinY = - m_xRelativeVolume.MinY;
		m_xRelativeVolume.MinZ = - m_xRelativeVolume.MinZ;
		m_bChanged = true;
		m_bOverlayVolumeChanged = false;
	}
	else
	{
		//update overlay volume from relative volume
		m_xOverlayVolume = m_xRelativeVolume;
		m_xOverlayVolume.MinX = - m_xOverlayVolume.MinX;
		m_xOverlayVolume.MinY = - m_xOverlayVolume.MinY;
		m_xOverlayVolume.MinZ = - m_xOverlayVolume.MinZ;
	}

	if(m_bUseOrientationChanged)
	{
		m_bUseOrientationChanged = false;
		UpdateFromRelativeVolume();
	}
	
	if(m_bChanged)
	{
		//update the asura data
		m_bChanged = false;
		UpdateFromRelativeVolume();
	}
		

	Tools_Object::UpdateWhileSelected();

	if(m_pxController)
	{
		m_pxController->Update();
	}
}


void Tools_VolumeEdit_Object::ExpandToIncludeModule()
{
	int moduleIndex;
	if(!Tools_LOS::FindModuleUnderCursor(moduleIndex)) return;

	//work out the world bounding box for this module
	Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(moduleIndex);

	Asura_Vector_3 offset;		
	Asura_Module_System::GetPositionOfModule(moduleIndex,offset);

	//get the bounding box
	if(!mesh->BoundingBoxDataValid) mesh->CalculateBoundingBox();
	Asura_Bounding_Box box = mesh->BoundingBox;

	//add on the modules location
	box.Displace(offset);


	for(int i=0;i<8;i++)
	{
		//get corner of mesh bounding box
		Asura_Vector_3 xCorner;
		xCorner.x = (i&1) ? box.MinX : box.MaxX;
		xCorner.y = (i&2) ? box.MinY : box.MaxY;
		xCorner.z = (i&4) ? box.MinZ : box.MaxZ;

		xCorner = GetRelativePosition(xCorner);
		
		
		//expand box to include this point
		m_xRelativeVolume.Set(xCorner);

	}
	
	UpdateFromRelativeVolume();

	if(m_pbModifiedFlag) *m_pbModifiedFlag = true;
}


void Tools_VolumeEdit_Object::SetToModule()
{
	int moduleIndex;
	if(!Tools_LOS::FindModuleUnderCursor(moduleIndex)) return;

	//work out the world bounding box for this module
	Asura_Mesh* mesh = Asura_Module_System::GetMeshForModule(moduleIndex);

	Asura_Vector_3 offset;		
	Asura_Module_System::GetPositionOfModule(moduleIndex,offset);

	//get the bounding box
	if(!mesh->BoundingBoxDataValid) mesh->CalculateBoundingBox();
	Asura_Bounding_Box box = mesh->BoundingBox;

	//add on the modules location
	box.Displace(offset);

	//set the volume equal to this box
	m_xWorldVolume = box;
	if(m_pxOrientQuat)
	{
		*m_pxOrientQuat = Asura_Maths::IdentityQuat;
	}
	else if(m_pxOrientMat)
	{
		*m_pxOrientMat = Asura_Maths::IdentityMatrix3x3;
	}

	SetDefaultPositionFromVolume();

	if(m_bEditingRelativeVolume)
	{
		*m_pxVolume = m_xRelativeVolume;
	}
	else
	{
		*m_pxVolume = m_xWorldVolume;
	}
	
	if(m_pbModifiedFlag) *m_pbModifiedFlag = true;

}

void Tools_VolumeEdit_Object::SnapFaceToCursor(const bool bAlsoExpandOppositeFace)
{
	if(m_eSelectedFace == FACE_NONE) return;

	int iPolyIndex;
	int iModuleIndex;
	Asura_Vector_3 xPosition;

	//find the location of the cursor
	if(!Tools_LOS::FindPolyAndPositionUnderCursor(iPolyIndex,iModuleIndex,xPosition)) return;

	xPosition = GetRelativePosition(xPosition);

	Asura_Bounding_Box xInitialRelativeVolume = m_xRelativeVolume;

	switch	(m_eSelectedFace)
	{
		case FACE_MIN_X :
			m_xRelativeVolume.MinX = xPosition.x;
			if(m_xRelativeVolume.MinX>m_xRelativeVolume.MaxX) m_xRelativeVolume.MinX=m_xRelativeVolume.MaxX;
			break;
		case FACE_MAX_X :
			m_xRelativeVolume.MaxX = xPosition.x;
			if(m_xRelativeVolume.MinX>m_xRelativeVolume.MaxX) m_xRelativeVolume.MaxX=m_xRelativeVolume.MinX;
			break;
		case FACE_MIN_Y :
			m_xRelativeVolume.MinY = xPosition.y;
			if(m_xRelativeVolume.MinY>m_xRelativeVolume.MaxY) m_xRelativeVolume.MinY=m_xRelativeVolume.MaxY;
			break;
		case FACE_MAX_Y :
			m_xRelativeVolume.MaxY = xPosition.y;
			if(m_xRelativeVolume.MinY>m_xRelativeVolume.MaxY) m_xRelativeVolume.MaxY=m_xRelativeVolume.MinY;
			break;
		case FACE_MIN_Z :
			m_xRelativeVolume.MinZ = xPosition.z;
			if(m_xRelativeVolume.MinZ>m_xRelativeVolume.MaxZ) m_xRelativeVolume.MinZ=m_xRelativeVolume.MaxZ;
			break;
		case FACE_MAX_Z :
			m_xRelativeVolume.MaxZ = xPosition.z;
			if(m_xRelativeVolume.MinZ>m_xRelativeVolume.MaxZ) m_xRelativeVolume.MaxZ=m_xRelativeVolume.MinZ;
			break;
		default :
			Asura_Assert(0,"Unexpected case");
			break;
			
	}

	if(bAlsoExpandOppositeFace)
	{
		switch	(m_eSelectedFace)
		{
			case FACE_MIN_X :
				m_xRelativeVolume.MaxX -= m_xRelativeVolume.MinX - xInitialRelativeVolume.MinX;
				if(m_xRelativeVolume.MinX>m_xRelativeVolume.MaxX) m_xRelativeVolume.MaxX=m_xRelativeVolume.MinX;
				break;
			case FACE_MAX_X :
				m_xRelativeVolume.MinX -= m_xRelativeVolume.MaxX - xInitialRelativeVolume.MaxX;
				if(m_xRelativeVolume.MinX>m_xRelativeVolume.MaxX) m_xRelativeVolume.MinX=m_xRelativeVolume.MaxX;
				break;
			case FACE_MIN_Y :
				m_xRelativeVolume.MaxY -= m_xRelativeVolume.MinY - xInitialRelativeVolume.MinY;
				if(m_xRelativeVolume.MinY>m_xRelativeVolume.MaxY) m_xRelativeVolume.MaxY=m_xRelativeVolume.MinY;
				break;
			case FACE_MAX_Y :
				m_xRelativeVolume.MinY -= m_xRelativeVolume.MaxY - xInitialRelativeVolume.MaxY;
				if(m_xRelativeVolume.MinY>m_xRelativeVolume.MaxY) m_xRelativeVolume.MinY=m_xRelativeVolume.MaxY;
				break;
			case FACE_MIN_Z :
				m_xRelativeVolume.MaxZ -= m_xRelativeVolume.MinZ - xInitialRelativeVolume.MinZ;
				if(m_xRelativeVolume.MinZ>m_xRelativeVolume.MaxZ) m_xRelativeVolume.MaxZ=m_xRelativeVolume.MinZ;
				break;
			case FACE_MAX_Z :
				m_xRelativeVolume.MinZ -= m_xRelativeVolume.MaxZ - xInitialRelativeVolume.MaxZ;
				if(m_xRelativeVolume.MinZ>m_xRelativeVolume.MaxZ) m_xRelativeVolume.MinZ=m_xRelativeVolume.MaxZ;
				break;
			default :
				Asura_Assert(0,"Unexpected case");
				break;

		}
	}



	UpdateFromRelativeVolume();
	
	if( m_xRelativeVolume.MinX>0 || m_xRelativeVolume.MinY>0 || m_xRelativeVolume.MinZ>0 ||
		m_xRelativeVolume.MaxX<0 || m_xRelativeVolume.MaxY<0 || m_xRelativeVolume.MaxZ<0)
	{
		SetDefaultPositionFromVolume();
	}
		
	if(m_pbModifiedFlag) *m_pbModifiedFlag = true;
}

void Tools_VolumeEdit_Object::SelectFaceUnderCursor( const bool bGetFurthest )
{
    //get the z direction for the mouse
    Asura_Vector_3 xZvec;
    Asura_Cursor::GetLOSDirection( xZvec );

    // Kevin Lea - 22/06/2001 12:49:20 - and transform this direction into worldspace
    const Asura_Vector_3*   pxCameraPosition    = Asura_Camera::GetPosition();
    const Asura_Matrix_3x3* pxCameraOrientation = Asura_Camera::GetOrientation();

    xZvec = xZvec * (*pxCameraOrientation);

    VolumeFace eNearestFace  = FACE_NONE;
    VolumeFace eFurthestFace = FACE_NONE;
    float      fNearest      = ASURA_LOS_MAX_DISTANCE;
    float      fFurthest     = 0.0f;

    Asura_Vector_3 axVertices[4];
    float          fDistance;
    bool           fFacingToward;
    for ( int i=(int)FACE_MIN_X; i<(int)FACE_END_ENUM; ++i )
    {
        GetFaceVertices( (VolumeFace)i, axVertices );

        fDistance = Asura_Maths::RayIntersectsTri( *pxCameraPosition,
            xZvec, axVertices[0], axVertices[1], axVertices[2], fFacingToward );
        if ( 0.0f <= fDistance && fDistance < fNearest )
        {
            fNearest     = fDistance;
            eNearestFace = (VolumeFace)i;
        }
        if ( fFurthest < fDistance && fDistance < ASURA_LOS_MAX_DISTANCE )
        {
            fFurthest     = fDistance;
            eFurthestFace = (VolumeFace)i;
        }

        fDistance = Asura_Maths::RayIntersectsTri( *pxCameraPosition,
            xZvec, axVertices[2], axVertices[3], axVertices[0], fFacingToward );
        if ( 0.0f <= fDistance && fDistance < fNearest )
        {
            fNearest     = fDistance;
            eNearestFace = (VolumeFace)i;
        }
        if ( fFurthest < fDistance && fDistance < ASURA_LOS_MAX_DISTANCE )
        {
            fFurthest     = fDistance;
            eFurthestFace = (VolumeFace)i;
        }
    }

    SetSelectedFace( bGetFurthest ? eFurthestFace : eNearestFace );
}

void Tools_VolumeEdit_Object::SetSelectedFace(enum VolumeFace eFace)
{
	m_eSelectedFace = eFace;

}

void Tools_VolumeEdit_Object::GetFaceVertices( enum VolumeFace eFace, Asura_Vector_3 axVertices[] )
{
    switch ( eFace )
    {
	    case FACE_MIN_X :
            for ( int i=0; i<4; ++i )
            {
                axVertices[i].x = m_xWorldVolume.MinX;
                axVertices[i].y = (i&2) ? m_xWorldVolume.MaxY : m_xWorldVolume.MinY;
                axVertices[i].z = ((i&1)^((i&2)>>1)) ? m_xWorldVolume.MaxZ : m_xWorldVolume.MinZ;
            }
		    break;

	    case FACE_MIN_Y :
            for ( int i=0; i<4; ++i )
            {
                axVertices[i].x = (i&2) ? m_xWorldVolume.MaxX : m_xWorldVolume.MinX;
                axVertices[i].y = m_xWorldVolume.MinY;
                axVertices[i].z = ((i&1)^((i&2)>>1)) ? m_xWorldVolume.MaxZ : m_xWorldVolume.MinZ;
            }
            break;

	    case FACE_MIN_Z :
            for ( int i=0; i<4; ++i )
            {
                axVertices[i].x = (i&2) ? m_xWorldVolume.MaxX : m_xWorldVolume.MinX;
                axVertices[i].y = ((i&1)^((i&2)>>1)) ? m_xWorldVolume.MaxY : m_xWorldVolume.MinY;
                axVertices[i].z = m_xWorldVolume.MinZ;
            }
            break;

	    case FACE_MAX_X :
            for ( int i=0; i<4; ++i )
            {
                axVertices[i].x = m_xWorldVolume.MaxX;
                axVertices[i].y = (i&2) ? m_xWorldVolume.MaxY : m_xWorldVolume.MinY;
                axVertices[i].z = ((i&1)^((i&2)>>1)) ? m_xWorldVolume.MaxZ : m_xWorldVolume.MinZ;
            }
            break;

	    case FACE_MAX_Y :
            for ( int i=0; i<4; ++i )
            {
                axVertices[i].x = (i&2) ? m_xWorldVolume.MaxX : m_xWorldVolume.MinX;
                axVertices[i].y = m_xWorldVolume.MaxY;
                axVertices[i].z = ((i&1)^((i&2)>>1)) ? m_xWorldVolume.MaxZ : m_xWorldVolume.MinZ;
            }
            break;

        case FACE_MAX_Z :
            for ( int i=0; i<4; ++i )
            {
                axVertices[i].x = (i&2) ? m_xWorldVolume.MaxX : m_xWorldVolume.MinX;
                axVertices[i].y = ((i&1)^((i&2)>>1)) ? m_xWorldVolume.MaxY : m_xWorldVolume.MinY;
                axVertices[i].z = m_xWorldVolume.MaxZ;
            }
            break;

        default:
            for ( int i=0; i<4; ++i )
            {
                axVertices[i].x =
                axVertices[i].y =
                axVertices[i].z = 0.0f;
            }
            break;
    }
	Asura_Vector_3 xCentre = GetWorldPivot();

	for ( int i=0; i<4; ++i )
    {
		if(m_bUseOrientation)
		{
			Asura_Vector_3 xOffset = xCentre-axVertices[i];
			xOffset = xOffset*Orientation;
			axVertices[i] = xCentre - xOffset;
		}
		
		axVertices[i] += m_xOffset;
	}
}

Tools_VolumeEdit_Object* Tools_VolumeEdit_Object::Find(unsigned uID)
{
	Asura_Assert(uID<s_uNextID,"Unexpected ID");

	if(!uID) return 0;

	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->Type != TOT_VolumeEdit) continue;

		Tools_VolumeEdit_Object* pxObject = (Tools_VolumeEdit_Object*) xObjectLIF();

		if(pxObject->GetID() == uID) return pxObject;
	}

	return 0;
}

void Tools_VolumeEdit_Object::Delete(unsigned uID)
{
	//find the object
	Tools_VolumeEdit_Object* pxObject = Find(uID);
	//and kill it
	if(pxObject)
	{
		Tools_Object_System::DeleteObject(pxObject);
	}
}

void Tools_VolumeEdit_Object::Done()
{
	if(m_pxController)
	{
		//if there is a controller , it will clear up everything
		m_pxController->EndVolumeEditing();
		return;
	}

	//reselect the owning object , if there was one
	if(m_pxParentObject)
	{
		//Make sure it still exists.
		//(In practice the volume edit object should already have been deleted before
		//the owner anyway).
		if(Tools_Object_System::ObjectList.contains(m_pxParentObject))
		{
			Tools_Object_System::SelectObject(m_pxParentObject);
		}
	}


	//and delete this object
	Tools_Object_System::DeleteObject(this);
}

void Tools_VolumeEdit_Object::SetOffset(const Asura_Vector_3& xOffset)
{
	Position += xOffset-m_xOffset;
	m_xOffset = xOffset;
}

Asura_Vector_3 Tools_VolumeEdit_Object::GetRelativePosition(const Asura_Vector_3& xPosition) const
{
	if(m_bUseOrientation)
	{
		//determine the centre of the volume in world space
		Asura_Vector_3 xCentreFromObject = GetRelativePivot();

		Asura_Vector_3 xCentre = (Position-m_xOffset) + (xCentreFromObject*Orientation);


		//get position relative to the centre of the volume
		Asura_Vector_3 xRel = xPosition - xCentre;

		//transform it into the space of the unrotated bounding box
		Asura_Matrix_3x3 xOrientInverse = Orientation;
		xOrientInverse.Transpose();
		xRel = xRel * xOrientInverse;

		//and make relative to the position of th object in the unrotated volume
		return xRel + xCentreFromObject;
			
	}
	else
	{
		return (xPosition - Position);
	}
}

void Tools_VolumeEdit_Object::ShowKeys()
{
	Tools_Help::ShowHelpTopic("Volume Editing");
}


bool Tools_VolumeEdit_Object::ObjectIsInLOS(const Asura_Vector_3& xStart, const Asura_Vector_3& xDir, float& fLowest)
{
	bool bInLOS = Tools_Object::ObjectIsInLOS(xStart,xDir,fLowest);

	//If selected , test against the volume as well.
	if(IsSelected())
	{
		//Do test against each face.
		//Not using RayIntersectsBoundingBox since I'm not sure I want to use its behaviour for tests starting from within.
	    bool fFacingToward;
		float fDistance;
		Asura_Vector_3 axVertices[4];
	    
	    for ( int i=(int)FACE_MIN_X; i<(int)FACE_END_ENUM; ++i )
	    {
	        GetFaceVertices( (VolumeFace)i, axVertices );

	        fDistance = Asura_Maths::RayIntersectsTri( xStart,
	            xDir, axVertices[0], axVertices[1], axVertices[2], fFacingToward );
	        if ( 0.0f <= fDistance && fDistance < fLowest )
	        {
	            fLowest = fDistance;
				bInLOS = true;
	        }

	        fDistance = Asura_Maths::RayIntersectsTri( xStart,
	            xDir, axVertices[2], axVertices[3], axVertices[0], fFacingToward );
	        if ( 0.0f <= fDistance && fDistance < fLowest )
	        {
	            fLowest = fDistance;
				bInLOS = true;
	        }
	    }
	}

	return bInLOS;
}

/////////////////////////////////////////////////////////////////////////////////////////////////

Tools_VolumeEdit_Controller::Tools_VolumeEdit_Controller(bool* pbModified)
{
	m_iNumberOfVolumes = 0;
	m_puVolumeIDs = 0;
	m_pbModified = pbModified;
	m_iSelectedIndex = 0;
	m_bSelectedIndexChanged = false;
}

Tools_VolumeEdit_Controller::~Tools_VolumeEdit_Controller()
{
	DeleteVolumeObjects();
}

void Tools_VolumeEdit_Controller::CreateVolumeObjects(bool bSelect)
{
	if(!m_iNumberOfVolumes)
	{
		m_iNumberOfVolumes = GetNumberOfVolumes();
		if(!m_iNumberOfVolumes) return;
	
		m_puVolumeIDs = new u_int[m_iNumberOfVolumes];
		for(int i=0;i<m_iNumberOfVolumes;i++)
		{
			m_puVolumeIDs[i] = 0;
		}
	}
	else
	{
		Asura_Assert(m_iNumberOfVolumes == GetNumberOfVolumes(),"Number of volumes out of sync");
	}


	//create a volume object for each volume
	for(int i=0;i<m_iNumberOfVolumes;i++)
	{
		CreateVolumeObject(i);
	}
 	if(bSelect) SelectVolume(m_iSelectedIndex);
}

void Tools_VolumeEdit_Controller::CreateVolumeObject(int iIndex)
{
	if(!m_iNumberOfVolumes) return;
	Asura_Assert(iIndex>=0 && iIndex<m_iNumberOfVolumes,"");

	Tools_VolumeEdit_Object* pxVolume = Tools_VolumeEdit_Object::Find(m_puVolumeIDs[iIndex]);
	if(!pxVolume)
	{
		Asura_Bounding_Box* pxBox = GetVolumeBox(iIndex);
		Asura_Quat* pxQuat = GetVolumeOrient(iIndex);

		Asura_Assert(pxBox,"Must have a bounding box");
		if(pxBox->MinX == pxBox->MaxX && pxBox->MinY == pxBox->MaxY && pxBox->MinZ == pxBox->MaxZ)
		{
			//set a default position for the volume
			Asura_Vector_3 xPosition = GetDefaultPosition();
			pxBox->MinX = pxBox->MaxX = xPosition.x;
			pxBox->MinY = pxBox->MaxY = xPosition.y;
			pxBox->MinZ = pxBox->MaxZ = xPosition.z;
		}

		char szVolumeName[200];
		sprintf(szVolumeName,"%s - Volume %d",GetName(),iIndex);

		pxVolume = new Tools_VolumeEdit_Object(szVolumeName,pxBox,0,m_pbModified);
		if(pxQuat)pxVolume->SetVolumeOrientation(pxQuat);
		pxVolume->SetController(this);

		//store the ID for the volume
		m_puVolumeIDs[iIndex] = pxVolume->GetID();	
	}

}

void Tools_VolumeEdit_Controller::DeleteVolumeObjects()
{
	for(int i=0;i<m_iNumberOfVolumes;i++)
	{
		Tools_VolumeEdit_Object::Delete(m_puVolumeIDs[i]);
	}
	delete [] m_puVolumeIDs;
	m_puVolumeIDs = 0;
	m_iNumberOfVolumes = 0;
}

void Tools_VolumeEdit_Controller::ShowVolumes(bool bShow)
{
	if(bShow)
	{
		CreateVolumeObjects(false);
	}
	else 
	{
		DeleteVolumeObjects();
	}
}

bool Tools_VolumeEdit_Controller::IsSelected() const
{
	Tools_VolumeEdit_Object* pxSelection = Tools_VolumeEdit_Object::GetSelectedVolume();
	if(!pxSelection) return false;

	for(int i=0;i<m_iNumberOfVolumes;i++)
	{
		if(m_puVolumeIDs[i] == pxSelection->GetID())
		{
			return true;
		}
	}
	return false;
}


void Tools_VolumeEdit_Controller::VolumeSelected(const Tools_VolumeEdit_Object* pxVolume)
{
	m_bSelectedIndexChanged = false;
	m_iSelectedIndex = -1;
	for(int i=0;i<m_iNumberOfVolumes;i++)
	{
		if(m_puVolumeIDs[i] == pxVolume->GetID())
		{
			m_iSelectedIndex = i;
			break;
		}
	}
	Asura_Assert(m_iSelectedIndex != -1,"");
}

void Tools_VolumeEdit_Controller::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	if(CanCreateVolume())
	{
		xOffset.y+=Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();
		pxDialog = new Asura_Function_Dialog("Create Volume",xOffset,CreateVolume,(int) this);
		pxDialog->SetHelpText("Add an extra volume to this object.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
	if(CanDeleteVolume())
	{
		pxDialog = new Asura_Function_Dialog("Delete Volume",xOffset,DeleteVolume,(int) this);
		pxDialog->SetHelpText("Permanently delete this volume.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
	if(m_iNumberOfVolumes>1)
	{
		xOffset.y+=Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();
		pxDialog = new Asura_Numerical_Dialog("Selected Volume",xOffset,&m_iSelectedIndex,0,m_iNumberOfVolumes-1,1,"%d",&m_bSelectedIndexChanged);
		pxDialog->SetHelpText("Change the currently selected volume in this set.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	Asura_Vector_2* pxPosition = GetOverlayPosition();
	if(pxPosition)
	{
		pxOverlay->m_xPosition = *pxPosition;
		pxOverlay->m_pxPositionBackup = pxPosition;
	}
	
}

void Tools_VolumeEdit_Controller::Update()
{
	//check for selection being changed using overlay controls
	if(m_bSelectedIndexChanged)
	{
		m_bSelectedIndexChanged = false;
		SelectVolume(m_iSelectedIndex);
	}
}

void Tools_VolumeEdit_Controller::PrintInfo(float& x,float& y)
{
	Asura_Text_Output::Print(x,y++,0xffffffff,0,"Volumes : %d",GetNumberOfVolumes());
}

void Tools_VolumeEdit_Controller::MoveVolumeObjects(const Asura_Vector_3& xOffset)
{
	for(int i=0;i<m_iNumberOfVolumes;i++)
	{
	 	Tools_VolumeEdit_Object* pxVolume = Tools_VolumeEdit_Object::Find(m_puVolumeIDs[i]);
		if(pxVolume)
		{
			pxVolume->Displace(xOffset);
		}
	}
}

void Tools_VolumeEdit_Controller::StartVolumeEditing()
{
	CreateVolumeObjects();
}

void Tools_VolumeEdit_Controller::EndVolumeEditing()
{
	DeleteVolumeObjects();
}

void Tools_VolumeEdit_Controller::CreateVolume()
{
	if(!CanCreateVolume()) return;

	//Need to delete all the volume objects first (since the pointers will most likely become invalid)
	DeleteVolumeObjects();

	//Add a new volume
	CreateVolumeMain();

	m_iSelectedIndex = GetNumberOfVolumes()-1;

	//And create the volume objects again
	CreateVolumeObjects();

	Tools_Timed_Message_System::AddMessage(0xffffff,"Volume created");
	if(m_pbModified) *m_pbModified = true;
	
}

void Tools_VolumeEdit_Controller::DeleteVolume()
{
	if(!CanDeleteVolume()) return;
	Asura_Assert(m_iNumberOfVolumes,"");
	if(m_iSelectedIndex< 0 || m_iSelectedIndex >= m_iNumberOfVolumes) return;
	
 	YesNo_Overlay xOverlay("Delete this volume?.");
	if(!xOverlay.DoModal()) return;

	//Delete the volume objects
	DeleteVolumeObjects();
	//delete the volume
	DeleteVolumeMain(m_iSelectedIndex);
	//and recreate the volume objects
	CreateVolumeObjects();

	if(m_pbModified) *m_pbModified = true;

}

void Tools_VolumeEdit_Controller::SelectVolume(int iIndex)
{
	if(!m_iNumberOfVolumes) return;
	m_iSelectedIndex = Asura_Maths::ClampToRange(iIndex,0,m_iNumberOfVolumes-1);

	//make the volume to be selected , if needed
	CreateVolumeObject(m_iSelectedIndex);

	//and select it
 	Tools_VolumeEdit_Object* pxVolume = Tools_VolumeEdit_Object::Find(m_puVolumeIDs[m_iSelectedIndex]);
	Tools_Object_System::SelectObject(pxVolume);
	
}

