//
// Tools_VolumeEdit_Object.h
//
// Created by Richard Rice - 10/05/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef _TOOLS_VOLUMEEDIT_OBJECT_H_
#define _TOOLS_VOLUMEEDIT_OBJECT_H_

#include "Core/Asura_String.h"


#include "Objects/Tools_Object.h"

class Tools_VolumeEdit_Controller;

//an object for , gasp , editing volumes!
class Tools_VolumeEdit_Object : public Tools_Object
{
	public :
		//constructor for editing a world BB
		Tools_VolumeEdit_Object(const char* name,Asura_Bounding_Box* volume,Tools_VolumeEdit_Object** referenceToThis,bool* pbModifiedFlag = 0);
		

		~Tools_VolumeEdit_Object();

		//gets the ID of the object for the next few functions...
		unsigned GetID() const {return m_uID;}

		//Find the volume edit object with this ID , if it still exists
		static Tools_VolumeEdit_Object* Find(unsigned uID);
		//Deletes the volume edit object with this ID , if it still exists
		static void Delete(unsigned uID);

		//Set the object that owns this bounding box.
		void SetOwner(Tools_Object* pxObject){m_pxParentObject = pxObject;}

		//Applies an offset to the edited bounding box for rendering purposes.
		void SetOffset(const Asura_Vector_3& xOffset);

		//Sets the orientation to edit ,and allows the volume to be orientatable
		void SetVolumeOrientation(Asura_Quat* pxOrientQuat);
		void SetVolumeOrientation(Asura_Matrix_3x3* pxOrientMat);

		//Sets controller for handling multiple volumes
		void SetController(Tools_VolumeEdit_Controller* pxController){m_pxController = pxController;}
		
		//Set the min/max extents vectors
		void SetMaxExtents( const Asura_Vector_3& xMax ) { m_xMaxExtents = xMax; }
		void SetMinExtents( const Asura_Vector_3& xMin ) { m_xMinExtents = xMin; }
		
		void UpdatePositionInAsuraData(bool /*updateLinkedObjects*/);
		const char* GetName(){return m_xName.GetString();}
		const char* GetTypeDescription(){return "Volume";}
		//void Destroy();
		void OnSelection();
		void OnDeselection();
		void UpdateWhileSelected();
		void Render();		
		void PrintInfo(float& x,float& y);
		bool ConfirmDestruction(){return false;}
		bool UsesLeftButton(){return true;}
		virtual bool IsTemporaryEditObject() const {return true;}
		virtual bool ObjectIsInLOS(const Asura_Vector_3& xStart, const Asura_Vector_3& xDir, float& fLowest);

		void CreateOverlay();
		void DeleteOverlay();

		void ExpandToIncludeModule();
		void SetToModule();
		void SnapFaceToCursor(const bool bAlsoExpandOppositeFace);
        void SelectFaceUnderCursor( const bool bGetFurthest );

		enum VolumeFace
		{
			FACE_NONE = -1,
			FACE_MIN_X,
			FACE_MAX_X,
			FACE_MIN_Y,
			FACE_MAX_Y,
			FACE_MIN_Z,
			FACE_MAX_Z,
            FACE_END_ENUM
		};

		void SetSelectedFace(enum VolumeFace eFace);
        void GetFaceVertices( enum VolumeFace eFace, Asura_Vector_3 axVertices[] );

		const Asura_Bounding_Box* GetVolume() const { return m_pxVolume; }

		static Tools_VolumeEdit_Object* GetSelectedVolume() {return s_pxThisVolume;}

		void Displace(const Asura_Vector_3& xOffset)
		{
			Position += xOffset;
			m_xWorldVolume.Displace(xOffset);
		}

	protected :

		//constructor for derived classes ,won't be valid without some further setting up
		Tools_VolumeEdit_Object(const char* pszName);


		void SetDefaultPositionFromVolume();
		void UpdateWorldVolumeFromRelative();

		void AddVolumeEditHelp();
		void RemoveVolumeEditHelp();

		virtual bool PreferPositionAtBaseOfVolume(){return true;}

		virtual Asura_Vector_3 GetRelativePivot() const {return m_xRelativeVolume.CalculateCentre();}
		virtual Asura_Vector_3 GetWorldPivot() const {return m_xWorldVolume.CalculateCentre();}

		//returns true if a orientation pointer has been given so as to allow orientation editing
		bool OrientationEditingAllowed() const {return m_pxOrientQuat || m_pxOrientMat;}

		//Gets a position in the space of the unrotated volume , relative to the object's position
		Asura_Vector_3 GetRelativePosition(const Asura_Vector_3& xPosition) const;

		//updates everything from the relative volume and position and orient
		void UpdateFromRelativeVolume();

		virtual bool AllowEndVolumeEditing() const {return true;}
		
		
		static void Done(int iThis){((Tools_VolumeEdit_Object*)iThis)->Done();}
		virtual void Done();

		static void ShowKeys();

		virtual void AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);
		virtual Asura_Vector_2* GetOverlayPosition() {return &s_xOverlayPosition;}
	   

		//data
		Asura_String 				m_xName;
		Asura_Bounding_Box*			m_pxVolume;
		Asura_Quat*					m_pxOrientQuat;//optional orientation to edit
		Asura_Matrix_3x3*			m_pxOrientMat;//optional orientation to edit
		Asura_Bounding_Box			m_xRelativeVolume;
		Asura_Bounding_Box			m_xWorldVolume;
		Asura_Bounding_Box			m_xOverlayVolume;//values shown on overlay
		Tools_VolumeEdit_Object**	m_ppxReferenceToThis;
		int							m_uID;
		Tools_VolumeEdit_Controller* m_pxController;

		Asura_Vector_3				m_xOffset;


		enum VolumeFace				m_eSelectedFace;
//		float*						m_pfSelectedFaceValue;
		bool*						m_pbModifiedFlag;

		bool						m_bChanged;
		bool						m_bUseOrientation;
		bool						m_bUseOrientationChanged;
		bool						m_bOverlayVolumeChanged;
		bool						m_bMinOverlayVolumeChanged;
		bool						m_bMaxOverlayVolumeChanged;
		bool						m_bEditingRelativeVolume;//m_pxVolume is pointer to a relative volume

		Asura_Vector_3				m_xMaxExtents;
		Asura_Vector_3				m_xMinExtents;

		Tools_Object*				m_pxParentObject;

		static unsigned				s_uNextID;
		static bool					s_bWireframe;


		//static data for overlay
		static Asura_Vector_2           s_xOverlayPosition;
		static Tools_VolumeEdit_Object* s_pxThisVolume;

};

//base class for handling editing of volumes for entities that may have multiple
//volumes attached to them
class Tools_VolumeEdit_Controller
{
	public :
		Tools_VolumeEdit_Controller(bool* pbModified);
		virtual ~Tools_VolumeEdit_Controller();

		virtual void StartVolumeEditing();
		virtual void EndVolumeEditing();

		//updates the currently selected index number		
		void VolumeSelected(const Tools_VolumeEdit_Object* pxVolume);
		//add overlay controls to volume's overlay
		void AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset);
		//update while one of the volumes is selected
		void Update();
		
		void PrintInfo(float& x,float& y);

		//shift the positions of all the volume objects
		void MoveVolumeObjects(const Asura_Vector_3& xOffset);

		//creates/deletes the volume edit objects without selecting them
		void ShowVolumes(bool bShow);

		//Returns true if one of the volumes is selected
		bool IsSelected() const;

		// Should highlight the volumes when rendering them?
		virtual bool IsHighlighted() const { return false; }

	protected :
		//Create and destroy the objects for editing the volumes
		void CreateVolumeObjects(bool bSelect = true);
		void DeleteVolumeObjects();

		virtual bool CanCreateVolume(){return false;}
		virtual bool CanDeleteVolume(){return false;}

		//Add/delete volumes if multiple volumes supported
		static void CreateVolume(int iThis){((Tools_VolumeEdit_Controller*)iThis)->CreateVolume();}
		void CreateVolume();
		
		static void DeleteVolume(int iThis){((Tools_VolumeEdit_Controller*)iThis)->DeleteVolume();}
		void DeleteVolume();

		//makes sure the volume object for this index exists
		void CreateVolumeObject(int iIndex);

		//Gets the current number of volumes
		virtual int GetNumberOfVolumes() = 0;
		//Gets a pointer to the bounding box for a given volume
		virtual Asura_Bounding_Box* GetVolumeBox(int iIndex) = 0;
		//Gets a pointer to the orientation for a given volume. Returns 0 if there is none
		virtual Asura_Quat* GetVolumeOrient(int iIndex) = 0;

		//Gets a default position for volume objects that have yet to gain a proper volume
		virtual Asura_Vector_3 GetDefaultPosition() = 0;

		//Gets the name of the owner , to use for part of the volume names
		virtual const char* GetName() = 0;

		//Creates the new volume in the asura data
		virtual void CreateVolumeMain(){};

		//Deletes the volume from the asura data
		virtual void DeleteVolumeMain(int iIndex){};

		void SelectVolume(int iIndex);

		//overide for overlay position
		virtual Asura_Vector_2* GetOverlayPosition(){return 0;}

	private :
		int m_iNumberOfVolumes;
		u_int* m_puVolumeIDs;
		
		int m_iSelectedIndex;	
		bool m_bSelectedIndexChanged;

		bool* m_pbModified;
};


#endif //_TOOLS_VOLUMEEDIT_OBJECT_H_

