//
// Tools_PC_Export.cpp
//
// Created by KJL - 12/04/2003 13:18:34
//
// Copyright (C) 2003 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "PC/Tools_PC_Export.h"

// Asura platform-independent.
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_FixedArray.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_HeapArray.h"
#include "Core/Asura_StringBuffer.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_Chunk_Stream_WindowsFile.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Debugging/Asura_LogFile.h"
#include "Mesh/Asura_Mesh.h"

// Asura PC.
#include "ChunkSystem/Asura_Chunk_ResourceFile_PC.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PC_Environment.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PC_MRPShaders.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PC_Object.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile_PC_ObjectHierarchy.h"
#include "Rendering/Asura_Direct3D_Effect.h"
#include "Rendering/Asura_Direct3D_Effects_System.h"
#include "Rendering/Asura_Direct3D_MRPShaderCompile.h"

// Tools.
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Materials/Tools_MaterialResponse_System.h"
#include "MeshProcessing/Tools_EnvironmentProcessor.h"
#include "MeshProcessing/Tools_HierarchyProcessor_MultiMaterial.h"
#include "MeshProcessing/Tools_HierarchyProcessor_MultiMaterial_NormalMapped.h"
#include "MeshProcessing/Tools_ObjectProcessor.h"
#include "MeshProcessing/Tools_ObjectProcessor_NormalMapped.h"
#include "MeshProcessing/Tools_SmoothSkin_Processor.h"
#include "MeshProcessing/Tools_SmoothSkin_Processor_PC.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "PC/FileSystem/Tools_File_Wrapper.h"

#if defined ASURA_TARGET_SM3
	#include "../PC/ChunkSystem/Asura_Chunk_ResourceFile_PC_Environment_SM3.h"
	#include "ChunkSystem/Asura_Chunk_ResourceFile_PC_D3DEffect.h"
	#include "Rendering/Asura_Direct3D_EnvironmentRenderer.h"
#endif

#if defined ASURA_USE_STATIC_DECAL_SYSTEM
	#include "SpecialFX/Asura_Decal_Template.h"
	#include "Decals/Tools_StaticDecal_System.h"
#endif

const char g_szEnvResourceFilename[] = "Env";

u_int Tools_PC_Export::CompressNormalToUByte4(Asura_Vector_3 xNormal)
{
	xNormal.Normalise();
	unsigned char ucX = (unsigned char)( (xNormal.x + 1.f) * (255.f / 2.f) );
	unsigned char ucY = (unsigned char)( (xNormal.y + 1.f) * (255.f / 2.f) );
	unsigned char ucZ = (unsigned char)( (xNormal.z + 1.f) * (255.f / 2.f) );
	return D3DCOLOR_XRGB(ucX, ucY, ucZ);
}

const float fMIN_COMPRESSED_TEXTURE_VALUE = -10.f;
const float fMAX_COMPRESSED_TEXTURE_VALUE = 10.f;

short Tools_PC_Export::CompressTexCoordToShort(float fInput, bool& bOutOfRange)
{
	bOutOfRange = false;

	// Clamp the value to the acceptable range, if necessary
	if(fInput < fMIN_COMPRESSED_TEXTURE_VALUE)
	{
		fInput = fMIN_COMPRESSED_TEXTURE_VALUE;
		bOutOfRange = true;
	}
	if(fInput > fMAX_COMPRESSED_TEXTURE_VALUE)
	{
		fInput = fMAX_COMPRESSED_TEXTURE_VALUE;
		bOutOfRange = true;
	}

	// fMinValue is going to map to -32768 and fMaxValue to 32767
	fInput *= 65535.f / (fMAX_COMPRESSED_TEXTURE_VALUE - fMIN_COMPRESSED_TEXTURE_VALUE);
	fInput += (-32768.f) - (65535.f * fMIN_COMPRESSED_TEXTURE_VALUE) / (fMAX_COMPRESSED_TEXTURE_VALUE - fMIN_COMPRESSED_TEXTURE_VALUE);
	return static_cast<short>(fInput);
}

void Tools_PC_Export::ExportEnvironment( HANDLE hOutput )
{
	const u_int uDIPIndexRangeMax = 0x10000;
	
	#define WRITEOUTDATA(a,b) WriteFile( hOutput, a, b, &bytesWritten, 0 );
	
	Tools_EnvironmentProcessor::Initialise();
	if(Tools_Level_Export::IsAbortFlagSet()) return;
	
	u_int uNumberOfMeshes = Tools_EnvironmentProcessor::GetNumberOfProcessedMeshes();
	u_int uNumberOfVertexBuffers = uNumberOfMeshes;
	u_int uNumberOfStrips = Tools_EnvironmentProcessor::GetTotalNumberOfStrips();

	u_int uFileSize = 0;
	
	// KJL - 04/03/2003 17:01:23 - okay, for the PC we'll be exporting
	//
	// Number Of Meshes (u_int)
	uFileSize += sizeof(u_int);

	// Total number of strips
	uFileSize += sizeof(u_int);

	// Number of Vertex/Index buffers
	uFileSize += sizeof(u_int);

	// Each mesh will contain:
	// Number of strips (u_int)
	// StartStripIndex (u_int)
	// VertexBuffer/IndexBuffer ID (u_int)
	uFileSize += (sizeof(u_int) * 3) * uNumberOfMeshes;

	// Each strip will contain:
	// NumberOfTris (u_int)
	// StartIndex (u_int)
	// MaterialID (int)
	// LowestVertex (u_int)
	// NumberOfVertices (u_int)
	uFileSize += (sizeof(u_int) * 5) * Tools_EnvironmentProcessor::GetTotalNumberOfStrips();

	// each Vertex/Index buffer description will be
	// NumberOfVertices (u_int)
	// NumberOfIndices (u_int)
	uFileSize += (sizeof(u_int) * 2) * uNumberOfVertexBuffers;

#ifdef ASURA_TARGET_SM3

	// The new environment renderer is still using an uncompressed vertex format on PC
	const u_int uVertexSize = sizeof(Asura_Vector_3)*2 //normal and position
							+ sizeof(u_int) //diffuse
							+ sizeof(float)*2 //U and V 1
							+ sizeof(float)*2; //U and V 2

#else

	const u_int uVertexSize = sizeof(Asura_Vector_3) // position
							+ sizeof(u_int)			 // packed normal
							+ sizeof(u_int)			 // diffuse colour
							+ sizeof(float)*2		 // UV1
							+ sizeof(float)*2		 // UV2
							;
#endif
	uFileSize += uVertexSize * Tools_EnvironmentProcessor::GetTotalNumberOfVertices();

	// And each index will be (u_short)
	uFileSize += sizeof(u_short) * Tools_EnvironmentProcessor::GetTotalNumberOfIndices();

	// KJL - 04/03/2003 17:22:10 - chunk size is all the data, 'filename' & header
	int uChunkSize = uFileSize + 4 + sizeof(Asura_Chunk_ResourceFile);

	Asura_Chunk_ResourceFile* pxResourceFile = new Asura_Chunk_ResourceFile;

	pxResourceFile->Create
	(
		uChunkSize,
		ASURA_CHUNK_PC_ENV_FLAG_DUALTEXTURE,
		ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
		ASURA_RESOURCEFILE_TYPE_PC_ENVIRONMENT,
		uFileSize,
		uASURA_CHUNK_PC_ENV_VERSION
	);
	

	unsigned long bytesWritten;

	WRITEOUTDATA( pxResourceFile, sizeof(Asura_Chunk_ResourceFile) );
	WRITEOUTDATA( g_szEnvResourceFilename, 4 );
	delete pxResourceFile;

	WRITEOUTDATA( &uNumberOfMeshes, sizeof(u_int) );
	WRITEOUTDATA( &uNumberOfStrips, sizeof(u_int) );
	WRITEOUTDATA( &uNumberOfVertexBuffers, sizeof(u_int) );

	// KJL - 05/03/2003 13:02:38 - all mesh descriptions in one go, so they can be loaded into one memory block
	u_int uStripStart = 0;

	for (u_int u=0; u<uNumberOfMeshes; u++)
	{
		const Tools_EnvironmentProcessor_Mesh* pxMesh = Tools_EnvironmentProcessor::GetProcessedMesh(u);

		WRITEOUTDATA( &(pxMesh->m_uNumberOfStrips), sizeof(u_int) );
		WRITEOUTDATA( &(uStripStart), sizeof(u_int) );
		WRITEOUTDATA( &(u), sizeof(u_int) ); // currently doing a VB per module

		uStripStart += pxMesh->m_uNumberOfStrips;
	}

	// KJL - 05/03/2003 13:03:00 - all strip descriptions in one go, so they can be loaded into one memory block
	for (u_int u=0; u<uNumberOfMeshes; u++)
	{
		const Tools_EnvironmentProcessor_Mesh* pxMesh = Tools_EnvironmentProcessor::GetProcessedMesh(u);

		u_int uStartIndex = 0;
		u_int uCurrentVertex = 0;
		for (u_int s=0; s<pxMesh->m_uNumberOfStrips; s++)
		{
			const Tools_EnvironmentProcessor_Strip& xStrip = pxMesh->m_pxStrips[s];	

			u_int uLowestVertexUsed = uCurrentVertex;
			u_int uNumberOfVertices = xStrip.m_uNumberOfVertices;
			
			WRITEOUTDATA( &(xStrip.m_uNumberOfTriangles), sizeof(u_int) );
			WRITEOUTDATA( &(uStartIndex), sizeof(u_int) );
			WRITEOUTDATA( &(xStrip.m_iMaterialID), sizeof(int) );
			WRITEOUTDATA( &(uLowestVertexUsed), sizeof(u_int) );
			WRITEOUTDATA( &(uNumberOfVertices), sizeof(u_int) );

			uStartIndex += xStrip.m_uNumberOfIndices;
			uCurrentVertex += xStrip.m_uNumberOfVertices;
		}
	}

	// KJL - 05/03/2003 13:30:04 - all the vertex/index buffer descriptions 
	// KJL - 05/03/2003 13:30:42 - currently doing one VB per module (can optimise later)
	for (u_int u=0; u<uNumberOfMeshes; u++)
	{
		const Tools_EnvironmentProcessor_Mesh* pxMesh = Tools_EnvironmentProcessor::GetProcessedMesh(u);

		WRITEOUTDATA( &(pxMesh->m_uTotalNumberOfVertices), sizeof(u_int) );
		WRITEOUTDATA( &(pxMesh->m_uTotalNumberOfIndices), sizeof(u_int) );

		// KJL - 05/03/2003 13:33:00 - write out vertex buffer data
		for (u_int s=0; s<pxMesh->m_uNumberOfStrips; s++)
		{
			Tools_EnvironmentProcessor_Strip& xStrip = pxMesh->m_pxStrips[s];

			for (u_int v=0; v<xStrip.m_uNumberOfVertices; v++)
			{
#ifdef ASURA_TARGET_SM3
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xPosition), sizeof(Asura_Vector_3) );
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xNormal), sizeof(Asura_Vector_3) );
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_uDiffuseRGBA), sizeof(u_int) );
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fU), sizeof(float) );
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fV), sizeof(float) );
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fU2), sizeof(float) );
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fV2), sizeof(float) );
#else
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xPosition), sizeof(Asura_Vector_3) );
				
				u_int uNormal = CompressNormalToUByte4(xStrip.m_pxVertices[v].m_xNormal);
				WRITEOUTDATA( &uNormal, sizeof(u_int) );

				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_uDiffuseRGBA), sizeof(u_int) );

				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fU), sizeof(float) );
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fV), sizeof(float) );
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fU2), sizeof(float) );
				WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fV2), sizeof(float) );

#endif	// ASURA_TARGET_SM3
			}
		}

		// KJL - 05/03/2003 13:33:08 - write out index buffer data
		u_int uCurrentVertex = 0;
		for (u_int s=0; s<pxMesh->m_uNumberOfStrips; s++)
		{
			Tools_EnvironmentProcessor_Strip& xStrip = pxMesh->m_pxStrips[s];

			u_int uCurrentStripVertex = uCurrentVertex;
			for (u_int v=0; v<xStrip.m_uNumberOfIndices; v++)
			{
				u_int uIndex = xStrip.m_pusIndices[v] + uCurrentStripVertex;
				Asura_Assertf( uIndex < uDIPIndexRangeMax, ("Vertex Index 0x%X is larger than allowed. Limit is 0x%X", uIndex, uDIPIndexRangeMax));
				u_short usIndex = static_cast< u_short>( uIndex);
				
				WRITEOUTDATA( &usIndex, sizeof(u_short) );
			}
			uCurrentVertex += xStrip.m_uNumberOfVertices;
		}
	}

	Tools_EnvironmentProcessor::Shutdown();



	#undef WRITEOUTDATA

}

class Tools_PC_Export_ExportEnvironment_SM3_BufferDesc
{
public:
	Tools_PC_Export_ExportEnvironment_SM3_BufferDesc()
	{
		this->m_uIBSize = 0;
		this->m_uVBSize = 0;
		this->m_uVBSizePre = 0;
	}

public:
	u_int m_uVBSizePre;
	u_int m_uVBSize;
	u_int m_uIBSize;
};

class Tools_PC_Export_ExportEnvironment_SM3_Strip
{
public:
	Tools_PC_Export_ExportEnvironment_SM3_Strip()
	{
		this->m_uBufferIndex = 0;
		this->m_uIBStart = 0;
		this->m_uVBStart = 0;
		this->m_uIBSize = 0;
		this->m_uIBSizeExtraStart = 0;
		this->m_uIBSizeExtraEnd = 0;
		this->m_uPrimitiveCount = 0;
	}

public:
	u_int m_uBufferIndex;
	u_int m_uVBStart;
	u_int m_uIBStart;
	
	u_int m_uIBSize;
	u_int m_uIBSizeExtraStart;
	u_int m_uIBSizeExtraEnd;
	u_int m_uPrimitiveCount;
};

#ifdef ASURA_TARGET_SM3

#if 0 // KJFL - work in progress new SM3 exporter
#include "MeshProcessing/Tools_EnvironmentProcessor_SM3.h"
void Tools_PC_Export::ExportEnvironment_SM3( HANDLE hOutput )
{
	#define WRITEOUTDATA(a,b) WriteFile( hOutput, a, b, &bytesWritten, 0 );

	Tools_EnvironmentProcessor_SM3 xEnv;

	const u_int uNumberOfPackets = xEnv.GetNumberOfPackets();
	const u_int uNumberOfVertexBuffers = xEnv.GetNumberOfVertexBuffers();
	const u_int uNumberOfIndexBuffers = xEnv.GetNumberOfIndexBuffers();
	
	u_int uFileSize = 0;

	// Total number of strips, VBs, IBs
	uFileSize += 3 * sizeof(u_int);
	
	// VertexBuffer Sizes
	uFileSize += sizeof( uint32) * uNumberOfVertexBuffers;
	
	// IndexBuffer Sizes
	uFileSize += sizeof( uint32) * uNumberOfIndexBuffers;

	// Strip data
	uFileSize += sizeof(Asura_Direct3D_EnvironmentRenderer_Strip) * (uNumberOfPackets);

	// Vertex data
	uFileSize += sizeof(Asura_Direct3D_EnvironmentRenderer_Vertex) * xEnv.GetTotalNumberOfVertices();

	// Index data
	uFileSize += sizeof(u_short) * xEnv.GetTotalNumberOfIndices();

	// And that's all!

	// KJL - 04/03/2003 17:22:10 - chunk size is all the data, 'filename' & header
	int uChunkSize = uFileSize + 4 + sizeof(Asura_Chunk_ResourceFile);

	u_int uChunkFlags = 0;
	
	uChunkFlags |= Asura_Chunk_ResourceFile_PC_Environment_SM3::ChunkFlag_MaterialResponse;
	uChunkFlags |= Asura_Chunk_ResourceFile_PC_Environment_SM3::ChunkFlag_UseTriLists;

	Asura_Chunk_ResourceFile* pxResourceFile = new Asura_Chunk_ResourceFile;

	pxResourceFile->Create
	(
		uChunkSize,
		uChunkFlags,
		ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
		ASURA_RESOURCEFILE_TYPE_PC_ENVIRONMENT_SM3,
		uFileSize,
		ASURA_CHUNK_RESOURCEFILE_PC_ENVIRONMENT_SM3_VERSION
	);

	unsigned long bytesWritten;

	WRITEOUTDATA( pxResourceFile, sizeof(Asura_Chunk_ResourceFile) );
	WRITEOUTDATA( g_szEnvResourceFilename, 4 );
	delete pxResourceFile;

	const Tools_EP_Packet* pxPackets = xEnv.GetPacketArray();

	// Vertex Buffer Info
	WRITEOUTDATA( &uNumberOfVertexBuffers, sizeof( uNumberOfVertexBuffers) );
	
	for( u_int u = 0; u < uNumberOfVertexBuffers; ++u )
	{
		u_int uVBSize = xEnv.GetSizeOfVertexBuffer(u);
		WRITEOUTDATA( &uVBSize, sizeof( uVBSize));
	}

	// Index Buffer Info
	WRITEOUTDATA( &uNumberOfIndexBuffers, sizeof( uNumberOfIndexBuffers) );
	
	for( u_int u = 0; u < uNumberOfIndexBuffers; ++u )
	{
		u_int uIBSize = xEnv.GetSizeOfIndexBuffer(u);
		WRITEOUTDATA( &uIBSize, sizeof( uIBSize));
	}

	// Strip info
	WRITEOUTDATA( &uNumberOfPackets, sizeof( uNumberOfPackets) );

	// Copying everything into a buffer and then writing it as one go is faster, but not hugely so for strip data I think
	// from the profiling I've done - may revert to a single strip at a time.
	Asura_Direct3D_EnvironmentRenderer_Strip* pxStrip = new Asura_Direct3D_EnvironmentRenderer_Strip[uNumberOfPackets];
	for (u_int u=0; u<uNumberOfPackets; ++u)
	{
		
		Asura_Direct3D_EnvironmentRenderer_Strip& xStrip = pxStrip[u];
		const Tools_EP_Packet& xPacket = pxPackets[u];

		xStrip.m_uNumberOfPrimitives = xPacket.m_uNumberOfIndices / 3;

		xStrip.m_uVertexBuffer = xPacket.m_uVertexBuffer;
		xStrip.m_uStartVertex = xPacket.m_uVertexBuffer_Offset;
		xStrip.m_uNumberOfVertices = xPacket.m_uNumberOfVertices;

		xStrip.m_uIndexBuffer = xPacket.m_uIndexBuffer;
		xStrip.m_uStartIndex = xPacket.m_uIndexBuffer_Offset;

		for (u_int v=0; v<TOOLS_NUM_MATERIALRESPONSE_PER_POLY; ++v)
		{
			xStrip.m_auMaterialResponseHashID[v] = xPacket.m_auMaterialResponseHashID[v];
		}

		xStrip.m_uPSHash = AddMRPShader( xPacket.m_auMaterialResponseHashID[0],	xPacket.m_auMaterialResponseHashID[1], false );
		
		xStrip.m_uModule = xPacket.m_uOriginalModule;

		// TODO - move to the processor
		xStrip.m_xBB.Zero();
		for( u_int v=0; v<xPacket.m_uNumberOfVertices; ++v )
		{
			xStrip.m_xBB.Set( xPacket.m_pxVertices[v].m_xPosition );
		}
	}
	WRITEOUTDATA( pxStrip, sizeof(Asura_Direct3D_EnvironmentRenderer_Strip)*uNumberOfPackets );
	delete [] pxStrip;

			


	// Vertex data
	for ( u_int u=0; u<uNumberOfPackets; ++u )
	{
		const Tools_EP_Packet& xPacket = pxPackets[u];

		// Copying everything into a buffer and then writing it as one go is *much* faster than a vertex at a time.
		Asura_Direct3D_EnvironmentRenderer_Vertex* pxRuntimeVertex = new Asura_Direct3D_EnvironmentRenderer_Vertex[xPacket.m_uNumberOfVertices];
		for ( u_int v=0; v<xPacket.m_uNumberOfVertices; ++v )
		{
			Asura_Direct3D_EnvironmentRenderer_Vertex& xRuntimeVertex = pxRuntimeVertex[v];
			const Tools_EP_Vertex& xToolsVertex = xPacket.m_pxVertices[v];

			xRuntimeVertex.SetPosition	(xToolsVertex.m_xPosition);
			xRuntimeVertex.SetDiffuseRGBA	(xToolsVertex.m_uDiffuseRGBA);
			xRuntimeVertex.SetTextureCoord	(0,	xToolsVertex.m_fU,	xToolsVertex.m_fV);
			xRuntimeVertex.SetTextureCoord	(1,	xToolsVertex.m_fU2,	xToolsVertex.m_fV2);
			xRuntimeVertex.SetNormal		(xToolsVertex.m_xNormal);
			xRuntimeVertex.SetTangentBinormal(xToolsVertex.m_xTangent,xToolsVertex.m_xBinormal);
			xRuntimeVertex.SetTangentBinormal2	(xToolsVertex.m_xTangent2,xToolsVertex.m_xBinormal2);
			xRuntimeVertex.SetDiffuseSH_Red		(xToolsVertex.m_xDiffuseSH.m_afRedCoefficients);
			xRuntimeVertex.SetDiffuseSH_Green	(xToolsVertex.m_xDiffuseSH.m_afGreenCoefficients);
			xRuntimeVertex.SetDiffuseSH_Blue		(xToolsVertex.m_xDiffuseSH.m_afBlueCoefficients);
		}
	
		// write out this packet's vertices all in one go
		WRITEOUTDATA( pxRuntimeVertex, sizeof(Asura_Direct3D_EnvironmentRenderer_Vertex)* xPacket.m_uNumberOfVertices );
		delete [] pxRuntimeVertex;
	}

	// Index data
	for ( u_int u=0; u<uNumberOfPackets; ++u )
	{
		// write out this packet's indices all in one go
		const Tools_EP_Packet& xPacket = pxPackets[u];
		WRITEOUTDATA( xPacket.m_pusIndices, sizeof( u_short ) * xPacket.m_uNumberOfIndices );
	}
	
#undef WRITEOUTDATA
	
}
#else
void Tools_PC_Export::ExportEnvironment_SM3( HANDLE hOutput )
{
#define WRITEOUTDATA(a,b) WriteFile( hOutput, a, b, &bytesWritten, 0 );
	static const u_int uBufferVertexCountMax = 0x10000;

	Tools_EnvironmentProcessor::Initialise( true);
	if(Tools_Level_Export::IsAbortFlagSet()) return;

	const Tools_EnvironmentProcessor_Strip* pxToolsStrips = Tools_EnvironmentProcessor::GetStripsSortedByMaterial();
	const u_int uNumberOfStrips = Tools_EnvironmentProcessor::GetTotalNumberOfStrips();
	
	Asura_HeapArray< Tools_PC_Export_ExportEnvironment_SM3_Strip> xArray_xStrips;
	Asura_HeapArray< Tools_PC_Export_ExportEnvironment_SM3_BufferDesc> xArray_xBufferDescs;
	
	xArray_xStrips.SetSize( uNumberOfStrips);
	xArray_xBufferDescs.SetSize( uNumberOfStrips);

	// DLH - Find out how many verts are needed per material
	Asura_TemplateHashTree< u_int> xHashTree_uMaterialVertexCount;

	for ( u_int u=0; u<uNumberOfStrips; ++u )
	{
		const Tools_EnvironmentProcessor_Strip& xToolsStrip = pxToolsStrips[u];
		if( 0 == xToolsStrip.m_uNumberOfVertices || 0 == xToolsStrip.m_uNumberOfIndices)
		{
			continue;
		}
		

		u_int uStripVertexCount = xToolsStrip.m_uNumberOfVertices;
		
		Asura_Hash_ID xMaterialHash0 = xToolsStrip.m_auMaterialResponseHashID[0];
		u_int* puMaterialVertexCount = xHashTree_uMaterialVertexCount.GetDataByHash( xMaterialHash0);
		if( NULL != puMaterialVertexCount)
		{
			*puMaterialVertexCount += uStripVertexCount;
		}
		else
		{
			xHashTree_uMaterialVertexCount.AddNode( xMaterialHash0, uStripVertexCount);
		}
	}
	
	// DLH - Place all a Material's verts in one Buffer.
	Asura_TemplateHashTree< u_int> xHashTree_uMaterialBufferStart;
	
	{
		u_int uNodeCount = xHashTree_uMaterialVertexCount.GetNumberOfNodes();
		u_int uBufferStart = 0;
		
		for( u_int uNodeIndex = 0; uNodeIndex < uNodeCount; uNodeIndex += 1)
		{
			const Asura_Hash_ID xMaterialHash0 = xHashTree_uMaterialVertexCount.GetHashByIndex( uNodeIndex);
			u_int* puMateriualVertexCount = xHashTree_uMaterialVertexCount.GetDataByIndex( uNodeIndex);
			Asura_Assert( NULL != puMateriualVertexCount, "This should not be NULL");
			
			if( NULL == puMateriualVertexCount)
			{
				continue;
			}
			
			u_int uMateriualVertexCount = *puMateriualVertexCount;
			u_int uBufferSize = xArray_xBufferDescs.Size();
			for( u_int uBufferIndex = uBufferStart; uBufferIndex < uBufferSize; uBufferIndex += 1)
			{
				Tools_PC_Export_ExportEnvironment_SM3_BufferDesc& xBuffer = xArray_xBufferDescs[ uBufferIndex];
				
				// DLH - The Material wants more than can fit in one buffer
				// DLH - Find first empty buffer and fill from there. Assumed first empty buffer has no filled buffers after it.
				if( uBufferVertexCountMax < uMateriualVertexCount)
				{
					if( 0 != xBuffer.m_uVBSizePre)
					{
						continue;
					}

					xHashTree_uMaterialBufferStart.AddNode( xMaterialHash0, uBufferIndex);
					
					for( u_int uBufferIndex2 = uBufferIndex; 0 < uMateriualVertexCount && uBufferIndex2 < uBufferSize; uBufferIndex2 += 1)
					{
						Tools_PC_Export_ExportEnvironment_SM3_BufferDesc& xBuffer2 = xArray_xBufferDescs[ uBufferIndex2];
						
						Asura_Assertf( 0 == xBuffer2.m_uVBSizePre, ("This buffer already has %d vertices in it.", xBuffer2.m_uVBSizePre));
						
						const u_int uMateriualVertexCountLimited = Asura_Maths::Min( uBufferVertexCountMax, uMateriualVertexCount);
						xBuffer2.m_uVBSizePre += uMateriualVertexCountLimited;
						uMateriualVertexCount -= uMateriualVertexCountLimited;
						
						uBufferStart = uBufferIndex2 + 1;
					}
					break;
				}
				
				const u_int uBufferVBSpace = uBufferVertexCountMax - xBuffer.m_uVBSizePre;
				
				if( uMateriualVertexCount <= uBufferVBSpace)
				{
					xHashTree_uMaterialBufferStart.AddNode( xMaterialHash0, uBufferIndex);
					xBuffer.m_uVBSizePre += uMateriualVertexCount;
					break;
				}
				
				uBufferStart = uBufferIndex + 1;
			}
			
		}
	}
	
	// Count how many invalid strips there are so we can omit them
	u_int uInvalidStrips = 0;
	u_int uBufferCount = 0;
	u_int uTotalIndices = 0;
	for ( u_int u=0; u<uNumberOfStrips; ++u )
	{
		const Tools_EnvironmentProcessor_Strip& xToolsStrip = pxToolsStrips[u];
		if(xToolsStrip.m_uNumberOfVertices && xToolsStrip.m_uNumberOfIndices)
		{
			Tools_PC_Export_ExportEnvironment_SM3_Strip& xBufferStrip = xArray_xStrips[ u];
			
			Asura_Assertf( xToolsStrip.m_uNumberOfIndices == (2 + xToolsStrip.m_uNumberOfTriangles), ("The Number of Indices %d and Triangles %d do not match", xToolsStrip.m_uNumberOfIndices, 2 + xToolsStrip.m_uNumberOfTriangles));
			
			xBufferStrip.m_uIBSize = xToolsStrip.m_uNumberOfIndices;
			// DLH - Add two extra degenerates indices to the start and end so the strip can be stiched to other strips
			// DLH - Add two and not one index to maintaine the front/back/wind order of the triangles
			// DLH - Add the extra an index the the end to maintaine the front/back/wind order of the triangles as the original is odd
			xBufferStrip.m_uIBSizeExtraStart = 2;
			xBufferStrip.m_uIBSizeExtraEnd = 2;
			if( 1 == (xBufferStrip.m_uIBSize & 1))
			{
				xBufferStrip.m_uIBSizeExtraEnd += 1;
			}
			xBufferStrip.m_uIBSize += xBufferStrip.m_uIBSizeExtraStart + xBufferStrip.m_uIBSizeExtraEnd;
			
			uTotalIndices += xBufferStrip.m_uIBSize;
			
			xBufferStrip.m_uPrimitiveCount = xBufferStrip.m_uIBSize - 2;
			
			// DLH - Find Buffer with enough space
			u_int uBufferSize = xArray_xBufferDescs.Size();
			u_int uBufferStart = 0;
			{
				Asura_Hash_ID xMaterialHash0 = xToolsStrip.m_auMaterialResponseHashID[0];
				u_int* puMaterialBufferStart = xHashTree_uMaterialBufferStart.GetDataByHash( xMaterialHash0);
				if( NULL != puMaterialBufferStart)
				{
					uBufferStart = *puMaterialBufferStart;
				}
			}
			for( u_int uBufferIndex = uBufferStart; uBufferIndex < uBufferSize; uBufferIndex += 1)
			{
				Tools_PC_Export_ExportEnvironment_SM3_BufferDesc& xBuffer = xArray_xBufferDescs[ uBufferIndex];
				
				u_int uBufferVBSizeNew = xBuffer.m_uVBSize + xToolsStrip.m_uNumberOfVertices;
				u_int uBufferIBSizeNew = xBuffer.m_uIBSize + xBufferStrip.m_uIBSize;
				
				{
					if( uBufferVertexCountMax < uBufferVBSizeNew && 0 != xBuffer.m_uVBSize)
					{
						continue;
					}
					Asura_Assertf( uBufferVBSizeNew <= uBufferVertexCountMax, ("Vertex Count %d too big for Vertex Buffer", uBufferVBSizeNew, uBufferVertexCountMax));
				}
#if 0
				{
					static const u_int uBufferIndexCountMax = 0x10000;
					if( uBufferIndexCountMax < uBufferIBSizeNew && 0 != xBuffer.m_uIBSize)
					{
						continue;
					}
					Asura_Assertf( uBufferIBSizeNew <= uBufferIndexCountMax, ("Index Count %d too big for Index Buffer", uBufferIBSizeNew, uBufferIndexCountMax));
				}
#endif
				
				// DLH - Update Buffer Strip
				xBufferStrip.m_uBufferIndex = uBufferIndex;
				xBufferStrip.m_uVBStart = xBuffer.m_uVBSize;
				xBufferStrip.m_uIBStart = xBuffer.m_uIBSize;
				
				// DLH - Update Buffer
				xBuffer.m_uIBSize = uBufferIBSizeNew;
				xBuffer.m_uVBSize = uBufferVBSizeNew;
				
				uBufferCount = Asura_Maths::Max( uBufferCount, uBufferIndex + 1);
				break;
			}
			continue;
		}
		else
		{
			++uInvalidStrips;
		}
	}

	if( uBufferCount < xArray_xBufferDescs.Size())
	{
		xArray_xBufferDescs.Shrink( uBufferCount);
	}

	u_int uStripCount = uNumberOfStrips - uInvalidStrips;
	u_int uVBCount = xArray_xBufferDescs.Size();
	u_int uIBCount = uVBCount;

	u_int uFileSize = 0;

	// Total number of strips, VBs, IBs
	uFileSize += 3 * sizeof(u_int);
	
	// VertexBuffer Sizes
	uFileSize += sizeof( uint32) * uVBCount;
	
	// IndexBuffer Sizes
	uFileSize += sizeof( uint32) * uIBCount;

	// Strip data
	uFileSize += sizeof(Asura_Direct3D_EnvironmentRenderer_Strip) * (uStripCount);

	// Vertex data
	uFileSize += sizeof(Asura_Direct3D_EnvironmentRenderer_Vertex) * Tools_EnvironmentProcessor::GetTotalNumberOfVertices();

	// Index data
	uFileSize += sizeof(u_short) * uTotalIndices;

	// And that's all!

	// KJL - 04/03/2003 17:22:10 - chunk size is all the data, 'filename' & header
	int uChunkSize = uFileSize + 4 + sizeof(Asura_Chunk_ResourceFile);

	u_int uChunkFlags = 0;
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM && !defined ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY
	uChunkFlags |= Asura_Chunk_ResourceFile_PC_Environment_SM3::ChunkFlag_MaterialResponse;
	#endif

	Asura_Chunk_ResourceFile* pxResourceFile = new Asura_Chunk_ResourceFile;

	pxResourceFile->Create
		(
		uChunkSize,
		uChunkFlags,
		ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
		ASURA_RESOURCEFILE_TYPE_PC_ENVIRONMENT_SM3,
		uFileSize,
		ASURA_CHUNK_RESOURCEFILE_PC_ENVIRONMENT_SM3_VERSION
		);

	unsigned long bytesWritten;

	WRITEOUTDATA( pxResourceFile, sizeof(Asura_Chunk_ResourceFile) );
	WRITEOUTDATA( g_szEnvResourceFilename, 4 );
	delete pxResourceFile;

	// Vertex Buffer Info
	WRITEOUTDATA( &uVBCount, sizeof( uVBCount) );
	
	for( u_int uVBIndex = 0; uVBIndex < uVBCount; uVBIndex += 1)
	{
		const Tools_PC_Export_ExportEnvironment_SM3_BufferDesc& xBuffer = xArray_xBufferDescs[ uVBIndex];
		
		uint32 uVBSize = xBuffer.m_uVBSize;
		WRITEOUTDATA( &uVBSize, sizeof( uVBSize));
		
		Asura_OutputToDebugger_Retail(( "VB %4d: %d\n", uVBIndex, uVBSize));
	}

	// Index Buffer Info
	WRITEOUTDATA( &uIBCount, sizeof( uIBCount) );
	
	for( u_int uIBIndex = 0; uIBIndex < uIBCount; uIBIndex += 1)
	{
		const Tools_PC_Export_ExportEnvironment_SM3_BufferDesc& xBuffer = xArray_xBufferDescs[ uIBIndex];

		uint32 uIBSize = xBuffer.m_uIBSize;
		WRITEOUTDATA( &uIBSize, sizeof( uIBSize));
		
		Asura_OutputToDebugger_Retail(( "IB %4d: %d\n", uIBIndex, uIBSize));
	}

	// Strip data
	WRITEOUTDATA( &uStripCount, sizeof( uStripCount) );
	
	for ( u_int u=0; u<uNumberOfStrips; ++u )
	{
		const Tools_EnvironmentProcessor_Strip& xToolsStrip = pxToolsStrips[u];
		if((xToolsStrip.m_uNumberOfVertices == 0) || (xToolsStrip.m_uNumberOfIndices == 0)) continue;

		const Tools_PC_Export_ExportEnvironment_SM3_Strip& xBufferStrip = xArray_xStrips[ u];
		const u_int uBufferIndex = xBufferStrip.m_uBufferIndex;
		//const Tools_PC_Export_ExportEnvironment_SM3_BufferDesc& xBuffer = xArray_xBufferDescs[ uBufferIndex];
		
		Asura_Direct3D_EnvironmentRenderer_Strip xRuntimeStrip;
		//xRuntimeStrip.m_uNumberOfPrimitives = xToolsStrip.m_uNumberOfTriangles;
		xRuntimeStrip.m_uNumberOfPrimitives = xBufferStrip.m_uPrimitiveCount;
		xRuntimeStrip.m_uVertexBuffer = uBufferIndex;
		Asura_Assertf( xRuntimeStrip.m_uVertexBuffer < uVBCount, ("Vertex Buffer %d if out of range. Vertex Buffer Count %d.", xRuntimeStrip.m_uVertexBuffer, uVBCount));
		xRuntimeStrip.m_uStartVertex = xBufferStrip.m_uVBStart;
		xRuntimeStrip.m_uNumberOfVertices = xToolsStrip.m_uNumberOfVertices;
		xRuntimeStrip.m_uIndexBuffer = uBufferIndex;
		Asura_Assertf( xRuntimeStrip.m_uIndexBuffer < uIBCount, ("Index Buffer %d if out of range. Index Buffer Count %d.", xRuntimeStrip.m_uIndexBuffer, uIBCount));
		xRuntimeStrip.m_uStartIndex = xBufferStrip.m_uIBStart;

		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM && !defined ASURA_MATERIAL_RESPONSE_CHARACTERS_ONLY
			xRuntimeStrip.m_auMaterialResponseHashID[0] = xToolsStrip.m_auMaterialResponseHashID[0];
			xRuntimeStrip.m_auMaterialResponseHashID[1] = xToolsStrip.m_auMaterialResponseHashID[1];

			Tools_Level_Export::AddMRPShaders( xToolsStrip.m_auMaterialResponseHashID[0],
				xToolsStrip.m_auMaterialResponseHashID[1], false );
		#else
			xRuntimeStrip.m_iMaterial = xToolsStrip.m_iMaterialID;
		#endif

		xRuntimeStrip.m_uModule = xToolsStrip.m_uModule;

		xRuntimeStrip.m_xBB.Reset();
		for( u_int v = 0; v < xToolsStrip.m_uNumberOfVertices; ++v )
		{
			xRuntimeStrip.m_xBB.Set( xToolsStrip.m_pxVertices[v].m_xPosition );
		}

		WRITEOUTDATA( &xRuntimeStrip, sizeof(Asura_Direct3D_EnvironmentRenderer_Strip) );
	}

	// Vertex data
	for ( u_int u=0; u<uNumberOfStrips; ++u )
	{
		const Tools_EnvironmentProcessor_Strip& xToolsStrip = pxToolsStrips[u];
		if((xToolsStrip.m_uNumberOfVertices == 0) || (xToolsStrip.m_uNumberOfIndices == 0)) continue;

		for (u_int v = 0; v < xToolsStrip.m_uNumberOfVertices; v++)
		{
			const Tools_EnvironmentProcessor_Vertex&	xToolsVertex	= xToolsStrip.m_pxVertices [v];
			Asura_Direct3D_EnvironmentRenderer_Vertex   xRuntimeVertex;

			xRuntimeVertex.SetPosition	(xToolsVertex.m_xPosition);
			xRuntimeVertex.SetDiffuseRGBA	(xToolsVertex.m_uDiffuseRGBA);
			xRuntimeVertex.SetTextureCoord	(0,	xToolsVertex.m_fU,	xToolsVertex.m_fV);
			xRuntimeVertex.SetTextureCoord	(1,	xToolsVertex.m_fU2,	xToolsVertex.m_fV2);
			xRuntimeVertex.SetNormal		(xToolsVertex.m_xNormal);
			xRuntimeVertex.SetTangentBinormal(xToolsVertex.m_xTangent,xToolsVertex.m_xBinormal);
			xRuntimeVertex.SetTangentBinormal2	(xToolsVertex.m_xTangent2,xToolsVertex.m_xBinormal2);
			xRuntimeVertex.SetDiffuseSH_Red		(xToolsVertex.m_xDiffuseSH.m_afRedCoefficients);
			xRuntimeVertex.SetDiffuseSH_Green	(xToolsVertex.m_xDiffuseSH.m_afGreenCoefficients);
			xRuntimeVertex.SetDiffuseSH_Blue		(xToolsVertex.m_xDiffuseSH.m_afBlueCoefficients);

			WRITEOUTDATA( &xRuntimeVertex, sizeof(Asura_Direct3D_EnvironmentRenderer_Vertex) );
		}
	}

	// Index data
#if defined( ASURA_ASSERT)
	u_int uAssertOutputIndices = 0;
#endif
	for ( u_int u=0; u<uNumberOfStrips; ++u )
	{
		const Tools_EnvironmentProcessor_Strip& xToolsStrip = pxToolsStrips[u];
		if((xToolsStrip.m_uNumberOfVertices == 0) || (xToolsStrip.m_uNumberOfIndices == 0)) continue;

		const Tools_PC_Export_ExportEnvironment_SM3_Strip& xBufferStrip = xArray_xStrips[ u];

#if defined( ASURA_ASSERT)
		u_int uAssertStripOutputIndices = 0;
#endif
		{		
			u_int uIndex = xToolsStrip.m_pusIndices[ 0];
			
			uIndex += xBufferStrip.m_uVBStart;
			uint16 usIndex = static_cast< uint16>( uIndex);
			
			const u_int uIndexCount = xBufferStrip.m_uIBSizeExtraStart;
			for( u_int uIndexIndex = 0; uIndexIndex < uIndexCount; uIndexIndex += 1)
			{
				WRITEOUTDATA( &usIndex, sizeof( usIndex));
#if defined( ASURA_ASSERT)
				uAssertStripOutputIndices += 1;
#endif
			}
		}

		{		
			const u_int uIndexCount = xToolsStrip.m_uNumberOfIndices;
			for( u_int uIndexIndex = 0; uIndexIndex < uIndexCount; uIndexIndex += 1)
			{
				u_int uIndex = xToolsStrip.m_pusIndices[ uIndexIndex];
				
				uIndex += xBufferStrip.m_uVBStart;
				uint16 usIndex = static_cast< uint16>( uIndex);
				
				WRITEOUTDATA( &usIndex, sizeof( usIndex));
#if defined( ASURA_ASSERT)
				uAssertStripOutputIndices += 1;
#endif
			}
		}
		
		{		
			u_int uIndex = xToolsStrip.m_pusIndices[ xToolsStrip.m_uNumberOfIndices - 1];
			
			uIndex += xBufferStrip.m_uVBStart;
			uint16 usIndex = static_cast< uint16>( uIndex);
			
			const u_int uIndexCount = xBufferStrip.m_uIBSizeExtraEnd;
			for( u_int uIndexIndex = 0; uIndexIndex < uIndexCount; uIndexIndex += 1)
			{
				WRITEOUTDATA( &usIndex, sizeof( usIndex));
#if defined( ASURA_ASSERT)
				uAssertStripOutputIndices += 1;
#endif
			}
		}
		
		Asura_Assertf( xBufferStrip.m_uIBSize == uAssertStripOutputIndices, ("Buffer is %d indices but %d have been output", xBufferStrip.m_uIBSize, uAssertStripOutputIndices));
#if defined( ASURA_ASSERT)
		uAssertOutputIndices += uAssertStripOutputIndices;
#endif

		//WRITEOUTDATA( xToolsStrip.m_pusIndices, sizeof(unsigned short) * xToolsStrip.m_uNumberOfIndices );
	}
	
	Asura_Assertf( uTotalIndices == uAssertOutputIndices, ("Should outputing %d indices but %d have been output", uTotalIndices, uAssertOutputIndices));

	Tools_EnvironmentProcessor::Shutdown();

#undef WRITEOUTDATA

}
#endif

#endif

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || (!ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER)

bool Tools_PC_Export::ExportCharacter( HANDLE hOutput, const Asura_Hierarchy_Skin* pxConstSkin)
{
	// Get a non-const copy of the skin to operate on
	// need to trace up the LOD root and back down again
	const Asura_Hierarchy_Skin* pxRootLODSkin = pxConstSkin;
	while(pxRootLODSkin->m_pxNextHigherSkin)
	{
		pxRootLODSkin = pxRootLODSkin->m_pxNextHigherSkin;
	}
	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkinByHash(pxRootLODSkin->m_uNameHashID);
	if(!pxSkin) return 0;
	while(pxSkin && (pxSkin->m_uNameHashID != pxConstSkin->m_uNameHashID))
	{
		pxSkin = pxSkin->m_pxNextLowerSkin;
	}
	if(!pxSkin) return 0;

	Tools_SmoothSkin_Processor xProcessor;

	xProcessor.SetIncludeNormalMapData( false );

	if( !xProcessor.Initialise( pxSkin ) ) return false;

	Tools_SmoothSkin_PC_Processor xPCProcessor(xProcessor);
	if( !xPCProcessor.Initialise() ) return false;

	Asura_Chunk_Stream_WindowsFile xFilestream( hOutput );
	bool bConvertMaterialIDs = true;
	if( !xPCProcessor.OutputToChunk(xFilestream, bConvertMaterialIDs) ) return false;

	return true;
}

#endif

#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER

bool Tools_PC_Export::ExportSmoothSkin( HANDLE hOutput, const Asura_Hierarchy_Skin* pxConstSkin)
{
	// Get a non-const copy of the skin to operate on
	// need to trace up the LOD root and back down again
	const Asura_Hierarchy_Skin* pxRootLODSkin = pxConstSkin;
	while(pxRootLODSkin->m_pxNextHigherSkin)
	{
		pxRootLODSkin = pxRootLODSkin->m_pxNextHigherSkin;
	}
	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkinByHash(pxRootLODSkin->m_uNameHashID);
	if(!pxSkin) return 0;
	while(pxSkin && (pxSkin->m_uNameHashID != pxConstSkin->m_uNameHashID))
	{
		pxSkin = pxSkin->m_pxNextLowerSkin;
	}
	if(!pxSkin) return 0;

	Tools_SmoothSkin_Processor xProcessor;

	xProcessor.SetIncludeNormalMapData( true );

	if( !xProcessor.Initialise( pxSkin ) ) return false;

	xProcessor.CreateBoneRemapTables(ASURA_MAX_NO_OF_BONES_PER_MATERIAL);

	Tools_SmoothSkin_PC_Processor xPCProcessor(xProcessor);
	if( !xPCProcessor.Initialise() ) return false;

	Asura_Chunk_Stream_WindowsFile xFilestream( hOutput );
	bool bConvertMaterialIDs = true;
	if( !xPCProcessor.OutputToChunk_SmoothSkinSM3(xFilestream, bConvertMaterialIDs) ) return false;

	return true;
}
#endif

////////////////////////////////////////////////////////////////////////////////
//
// Tools_PC_Export::ExportSkin()
//
// Exports a skinned mesh to PC Direct3D format
//
////////////////////////////////////////////////////////////////////////////////

#ifdef ASURA_TARGET_SM3

bool Tools_PC_Export::ExportSkin( const HANDLE hOutput, Asura_Hierarchy_Skin& pxConstSkin )
{
	//////////////////////////////////////////////////////////////////////////
	// Get a non-const copy of the skin to operate on
	// need to trace up the LOD root and back down again
	const Asura_Hierarchy_Skin* pxRootLODSkin = &pxConstSkin;
	while(pxRootLODSkin->m_pxNextHigherSkin)
	{
		pxRootLODSkin = pxRootLODSkin->m_pxNextHigherSkin;
	}
	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkinByHash(pxRootLODSkin->m_uNameHashID);
	if(!pxSkin) return 0;
	while(pxSkin && (pxSkin->m_uNameHashID != pxConstSkin.m_uNameHashID))
	{
		pxSkin = pxSkin->m_pxNextLowerSkin;
	}
	if(!pxSkin) return 0;
	//////////////////////////////////////////////////////////////////////////


	Tools_SmoothSkin_Processor xProcessor;
	xProcessor.SetIncludeNormalMapData( true );

	Tools_SmoothSkin_PC_Processor xPCProcessor(xProcessor);

	Asura_Chunk_Stream_WindowsFile xFilestream( hOutput );

	if( pxSkin->IsSkinnedModel() )
	{
		if( !xProcessor.Initialise( pxSkin ) ) return false;
		xProcessor.CreateBoneRemapTables(ASURA_MAX_NO_OF_BONES_PER_MATERIAL);
		if( !xPCProcessor.InitialiseV2() ) return false;
	}
	else
	{
		if( !Tools_HierarchyProcessor_MultiMaterial_NormalMapped::Initialise( pxSkin ) )	return false;
		if( !xPCProcessor.InitialiseV2FromHierarchyProcessor() )							return false;
	}

	xPCProcessor.OutputToChunkV2( xFilestream );

	if( !pxSkin->IsSkinnedModel() )
	{
		Tools_HierarchyProcessor_MultiMaterial_NormalMapped::Shutdown();
	}

	return true;
}

#endif

bool Tools_PC_Export::ExportObjectHierarchy( HANDLE hOutput, const Asura_Hierarchy_Skin* pxSkin)
{
	// initialise and stripify mesh
#if defined(ASURA_TARGET_SM3)
	#define TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE Tools_HierarchyProcessor_MultiMaterial_NormalMapped
	#define TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_STRIP_TOUSE Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Strip
#else
	#define TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE Tools_HierarchyProcessor_MultiMaterial
	#define TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_STRIP_TOUSE Tools_HierarchyProcessor_MultiMaterial_Strip
#endif
	
	if(!TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::Initialise(pxSkin))
	{
		return false;
	}

	TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::ConvertMaterialsForExport();

	#define WRITEOUTDATA(a,b) WriteFile( hOutput, a, b, &bytesWritten, 0 );

	Asura_String xName = pxSkin->Name;

	u_int uNumberOfStrips = TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::GetTotalNumberOfStrips();

	u_int uFileNameSize = (xName.GetSizeInInts()<<2);
	
	u_int uFileSize = 0;
	
	// BM - character will be one strip
	// BM - 29/03/2003 15:07:29 for the Xbox we'll be exporting:
	// String to tell us the filename of the Hierarchy (uFileNameSize)
	uFileSize += uFileNameSize;
	
	// number of strips + total number of vertices, + total number of indices
	uFileSize += (sizeof(u_int) * 3);

	// For each strip:
	// NumberOfTris (u_int)
	// uStartIndex(u_int)
	// MaterialID0 (int)
	// MaterialID1 (int)  (if material response system)
	// uLowestVertexUsed
	// uNumberOfVertices
	// boneindex (u_short)
	u_int uSizePerStrip = sizeof(u_int) * 5 + sizeof(u_short);
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	uSizePerStrip += sizeof(u_int); //for the extra material ID
	#endif
	uFileSize += uSizePerStrip * uNumberOfStrips;

	// And then each vertex will be
#if defined(ASURA_TARGET_SM3)
	const u_int uVertexSize = sizeof(float) * 17;
#else
	const u_int uVertexSize = sizeof(float) * 3 // position
							+ sizeof(u_int)		// normal
							+ sizeof(short) * 2 // texcoords
							+ sizeof(float)		// alpha
							;
#endif
	uFileSize += uVertexSize * TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::GetTotalNumberOfVertices();

	// And each index will be (u_short)
	uFileSize += (sizeof(u_short) * TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::GetTotalNumberOfIndices());

	// Chunk size is all the data, 'filename' & header
	int uChunkSize = uFileSize + uFileNameSize + sizeof(Asura_Chunk_ResourceFile);
	Asura_Chunk_ResourceFile* pxResourceFile = new Asura_Chunk_ResourceFile;

	u_int uChunkFlags = 0;
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	uChunkFlags |= Asura_Chunk_ResourceFile_PC_ObjectHierarchy::ChunkFlag_MaterialResponse;
	#endif

	pxResourceFile->Create
	(
		uChunkSize,
		uChunkFlags,
		ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
		ASURA_RESOURCEFILE_TYPE_PC_OBJECTHIERARCHY,
		uFileSize,
		ASURA_CHUNK_RESOURCEFILE_PC_OBJECTHIERARCHY_VERSION
	);
	
	unsigned long bytesWritten;

	u_int uTotalNumberOfVertices = TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::GetTotalNumberOfVertices();
	u_int uTotalNumberOfIndices = TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::GetTotalNumberOfIndices();

	WRITEOUTDATA( pxResourceFile, sizeof(Asura_Chunk_ResourceFile) );
	WRITEOUTDATA( xName.GetString(),uFileNameSize );
	WRITEOUTDATA( xName.GetString(),uFileNameSize );
	WRITEOUTDATA( &(uNumberOfStrips), sizeof(u_int) );
	WRITEOUTDATA( &(uTotalNumberOfVertices), sizeof(u_int));
	WRITEOUTDATA( &(uTotalNumberOfIndices), sizeof(u_int));

	delete pxResourceFile;


	u_int uStartIndex = 0;
	u_int uCurrentVertex = 0;

#if defined(ASURA_TARGET_SM3)
	for( u_int u= 0; u< uNumberOfStrips; u++)
	{
		const TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_STRIP_TOUSE& xStrip = *TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::GetProcessedStrip(u);		

		u_int uLowestVertexUsed = uCurrentVertex;
		u_int uNumberOfVertices = xStrip.m_uNumberOfVertices;

		WRITEOUTDATA( &(xStrip.m_uNumberOfTriangles), sizeof(u_int) );
		WRITEOUTDATA( &(uStartIndex), sizeof(u_int) );
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		WRITEOUTDATA( &(xStrip.m_auMaterialResponseHashID[0]), sizeof(int) );
		WRITEOUTDATA( &(xStrip.m_auMaterialResponseHashID[1]), sizeof(int) );
#else
		WRITEOUTDATA( &(xStrip.m_iMaterialID), sizeof(int) );
#endif
		WRITEOUTDATA( &(uLowestVertexUsed), sizeof(u_int) );
		WRITEOUTDATA( &(uNumberOfVertices), sizeof(u_int) );
		WRITEOUTDATA( &(xStrip.m_usBoneIndex), sizeof(u_short) );

		uStartIndex += xStrip.m_uNumberOfIndices;
		uCurrentVertex += xStrip.m_uNumberOfVertices;
	}

	for( u_int u= 0; u< uNumberOfStrips; u++)
	{
		// BM - 18/03/2003 17:50:00 - All the Vertex,Index buffer data
		const TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_STRIP_TOUSE& xStrip = *TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::GetProcessedStrip(u);		
	
		for (u_int v=0; v<xStrip.m_uNumberOfVertices; v++)
		{
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xPosition.x), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xPosition.y), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xPosition.z), sizeof(float) );

			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xNormal.x), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xNormal.y), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xNormal.z), sizeof(float) );

			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fU0), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fV0), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fU1), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fV1), sizeof(float) );

			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fAlpha), sizeof(float) );
			
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xTangent.x), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xTangent.y), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xTangent.z), sizeof(float) );

			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xBinormal.x), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xBinormal.y), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xBinormal.z), sizeof(float) );

		}
    }
#else // not defined(ASURA_TARGET_SM3)
	for( u_int u= 0; u< uNumberOfStrips; u++)
	{
		const TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_STRIP_TOUSE& xStrip = *TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::GetProcessedStrip(u);		

		u_int uLowestVertexUsed = uCurrentVertex;
		u_int uNumberOfVertices = xStrip.m_uNumberOfVertices;

		WRITEOUTDATA( &(xStrip.m_uNumberOfTriangles), sizeof(u_int) );
		WRITEOUTDATA( &(uStartIndex), sizeof(u_int) );
		WRITEOUTDATA( &(xStrip.m_iMaterialID), sizeof(int) );
		WRITEOUTDATA( &(uLowestVertexUsed), sizeof(u_int) );
		WRITEOUTDATA( &(uNumberOfVertices), sizeof(u_int) );
		WRITEOUTDATA( &(xStrip.m_usBoneIndex), sizeof(u_short) );

		uStartIndex += xStrip.m_uNumberOfIndices;
		uCurrentVertex += xStrip.m_uNumberOfVertices;
	}

	bool bOutOfRange, bAnyOutOfRange = false;

    for( u_int u= 0; u< uNumberOfStrips; u++)
	{
		// BM - 18/03/2003 17:50:00 - All the Vertex,Index buffer data
		const Tools_HierarchyProcessor_MultiMaterial_Strip& xStrip = *Tools_HierarchyProcessor_MultiMaterial::GetProcessedStrip(u);		
	
		for (u_int v=0; v<xStrip.m_uNumberOfVertices; v++)
		{
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xPosition.x), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xPosition.y), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_xPosition.z), sizeof(float) );

			u_int uNormal = CompressNormalToUByte4(xStrip.m_pxVertices[v].m_xNormal);
			WRITEOUTDATA( &uNormal, sizeof(u_int) );

			short sU = CompressTexCoordToShort(xStrip.m_pxVertices[v].m_fU0, bOutOfRange);
			bAnyOutOfRange |= bOutOfRange;
			short sV = CompressTexCoordToShort(xStrip.m_pxVertices[v].m_fV0, bOutOfRange);
			bAnyOutOfRange |= bOutOfRange;
			WRITEOUTDATA( &sU, sizeof(short) );
			WRITEOUTDATA( &sV, sizeof(short) );

			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fAlpha), sizeof(float) );
		}	
    }

	Asura_Assertf(!bAnyOutOfRange, ("Some texcoords on hierarchy %s were out of range and have been clamped.", pxSkin->Name));
#endif // else not defined(ASURA_TARGET_SM3)

	// KJL - 05/03/2003 13:33:08 - write out index buffer data

	uCurrentVertex = 0;
	for (u_int s=0; s< uNumberOfStrips; s++)
	{
		const TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_STRIP_TOUSE& xStrip = *TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::GetProcessedStrip(s);

		for (u_int v=0; v<xStrip.m_uNumberOfIndices; v++)
		{
			u_short usIndex = (u_short)(xStrip.m_pusIndices[v]+uCurrentVertex);
			WRITEOUTDATA( &usIndex, sizeof(u_short) );
		}
		uCurrentVertex += xStrip.m_uNumberOfVertices;
	}

	// wrap up and delete all the temp data
	TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE::Shutdown();

#undef WRITEOUTDATA
#undef TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_TOUSE
#undef TOOLS_HIERARCHYPROCESSOR_MULTIMATERIAL_STRIP_TOUSE

	return true;
}

#if defined(ASURA_TARGET_SM3) && defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
	#define TOOLS_OBJECTPROCESSOR_TOUSE Tools_ObjectProcessor_NormalMapped
	#define TOOLS_OBJECTPROCESSOR_STRIP_TOUSE Tools_ObjectProcessor_NormalMapped_Strip
#else
	#define TOOLS_OBJECTPROCESSOR_TOUSE Tools_ObjectProcessor
	#define TOOLS_OBJECTPROCESSOR_STRIP_TOUSE Tools_ObjectProcessor_Strip
#endif

bool Tools_PC_Export::ExportObject( HANDLE hOutput, Asura_Mesh* pxMesh, const Asura_Hash_ID uHashID,int iMaterialIndex, const Asura_Hash_ID uMaterialResponseHashID)
{
	// initialise and stripify mesh
#if defined(ASURA_TARGET_SM3) && defined( ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
	if(!TOOLS_OBJECTPROCESSOR_TOUSE::Initialise(pxMesh,iMaterialIndex, true,Tools_TriStripper::OUTPUT_TRISTRIP_WITH_RESET_INDICES, uMaterialResponseHashID))
#else
	if(!TOOLS_OBJECTPROCESSOR_TOUSE::Initialise(pxMesh,iMaterialIndex))
#endif
	{
		return false;
	}

	WriteObjectDataToChunk(hOutput,pxMesh,pxMesh->Name, uHashID);

	// wrap up and delete all the temp data
	TOOLS_OBJECTPROCESSOR_TOUSE::Shutdown();

	return true;
}


bool Tools_PC_Export::WriteObjectDataToChunk(HANDLE hOutput, const class Asura_Mesh* pxMesh,const Asura_String& xName, const Asura_Hash_ID uHashID)
{
	#define WRITEOUTDATA(a,b) WriteFile( hOutput, a, b, &bytesWritten, 0 );

	u_int uFileNameSize = (xName.GetSizeInInts()<<2);
	
	u_int uFileSize = 0;
	
	// Hash_ID
	uFileSize += sizeof(Asura_Hash_ID);
	
	// For each strip:
	// NumberOfTris (u_int)
	// NumberOfVertices (u_int)
	// NumberOfIndices (u_int)
#if defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)
	// MaterialD1 (int)
	// MaterialD2 (int)
	uFileSize += (sizeof(u_int) * 5) ;
#else
	// MaterialIndex (u_int)
	uFileSize += (sizeof(u_int) * 4) ;
#endif	// defined(ASURA_USE_MATERIAL_RESPONSE_SYSTEM)

	// min and max vector (3 floats) of BB - these will be needed for lighting
//	uFileSize += (sizeof(float) * 3 * 2);

	// And then each vertex will be
#if defined(ASURA_TARGET_SM3)
    const u_int uVertexSize = sizeof(float) * 3 // position
                            + sizeof(float) * 3 // normal
                            + sizeof(float) * 2 // texcoords
#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
							+ sizeof(float) * 2 // texcoords2
                            + sizeof(float) * 6 // tangenmt and binormal
							+ sizeof(float)     // alpha
#else
                            + sizeof(float)     // alpha
#endif
                            ;
#else
    const u_int uVertexSize = sizeof(float) * 3 // position
                            + sizeof(u_int)     // compressed normal
                            + sizeof(short) * 2 // texcoords
                            ;
#endif
	uFileSize += (uVertexSize * TOOLS_OBJECTPROCESSOR_TOUSE::GetTotalNumberOfVertices());

	// And each index will be (u_short)
	uFileSize += (sizeof(u_short) * TOOLS_OBJECTPROCESSOR_TOUSE::GetTotalNumberOfIndices());

	// Version 1 - LOD  min distance and hash ID added. RAR 09/05/03
	uFileSize += sizeof(float) + sizeof(Asura_Hash_ID);

	// Chunk size is all the data, 'filename' & header
	int uChunkSize = uFileSize + uFileNameSize + sizeof(Asura_Chunk_ResourceFile);
	Asura_Chunk_ResourceFile* pxResourceFile = new Asura_Chunk_ResourceFile;

	u_int uChunkFlags = 0;
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	uChunkFlags |= Asura_Chunk_ResourceFile_PC_Object::ChunkFlag_MaterialResponse;
	#endif

	pxResourceFile->Create
	(
		uChunkSize,
		uChunkFlags,
		ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
		ASURA_RESOURCEFILE_TYPE_PC_OBJECT,
		uFileSize
	);
	pxResourceFile->Version = ASURA_CHUNK_RESOURCEFILE_PC_OBJECT_VERSION;

	unsigned long bytesWritten;

	WRITEOUTDATA( pxResourceFile, sizeof(Asura_Chunk_ResourceFile) );
	WRITEOUTDATA( xName.GetString(),uFileNameSize );
	delete pxResourceFile;

	WRITEOUTDATA( &uHashID, sizeof(u_int) );

	{
		const TOOLS_OBJECTPROCESSOR_STRIP_TOUSE& xStrip = *TOOLS_OBJECTPROCESSOR_TOUSE::GetProcessedStrip();		

		WRITEOUTDATA( &(xStrip.m_uNumberOfTriangles), sizeof(u_int) );
		WRITEOUTDATA( &(xStrip.m_uNumberOfVertices), sizeof(u_int) );
		WRITEOUTDATA( &(xStrip.m_uNumberOfIndices), sizeof(u_int) );
#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		WRITEOUTDATA( &(xStrip.m_auMaterialResponseHashID[0]), sizeof(int) );
		WRITEOUTDATA( &(xStrip.m_auMaterialResponseHashID[1]), sizeof(int) );
#else
		WRITEOUTDATA( &(xStrip.m_iMaterialID), sizeof(int) );
#endif
//		WRITEOUTDATA( &(xStrip.m_xBBMin), sizeof(float) * 3);
//		WRITEOUTDATA( &(xStrip.m_xBBMax), sizeof(float) * 3);
	}
	
	{
		// BM - 18/03/2003 17:50:00 - All the Vertex,Index buffer data
		const TOOLS_OBJECTPROCESSOR_STRIP_TOUSE& xStrip = *TOOLS_OBJECTPROCESSOR_TOUSE::GetProcessedStrip();

#if defined(ASURA_TARGET_SM3)
		for (u_int v=0; v<xStrip.m_uNumberOfVertices; v++)
		{
#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
			Tools_ObjectProcessor_NormalMapped_Vertex& xVertex = xStrip.m_pxVertices[v];
			
			WRITEOUTDATA( &(xVertex.m_xPosition.x), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_xPosition.y), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_xPosition.z), sizeof(float) );

			WRITEOUTDATA( &(xVertex.m_xNormal.x), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_xNormal.y), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_xNormal.z), sizeof(float) );

			WRITEOUTDATA( &(xVertex.m_fU), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_fV), sizeof(float) );

			WRITEOUTDATA( &(xVertex.m_fU2), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_fV2), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_fAlpha), sizeof(float) );

			WRITEOUTDATA( &(xVertex.m_xTangent.x), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_xTangent.y), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_xTangent.z), sizeof(float) );

			WRITEOUTDATA( &(xVertex.m_xBinormal.x), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_xBinormal.y), sizeof(float) );
			WRITEOUTDATA( &(xVertex.m_xBinormal.z), sizeof(float) );

#else
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fX), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fY), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fZ), sizeof(float) );

			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fNX), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fNY), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fNZ), sizeof(float) );

			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fU), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fV), sizeof(float) );

	        static const float fOne = 1.0f;
            WRITEOUTDATA( &fOne, sizeof(float) );
#endif
		}
#else
		bool bOutOfRange, bAnyOutOfRange = false;
	
		for (u_int v=0; v<xStrip.m_uNumberOfVertices; v++)
		{
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fX), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fY), sizeof(float) );
			WRITEOUTDATA( &(xStrip.m_pxVertices[v].m_fZ), sizeof(float) );

			u_int uNormal = CompressNormalToUByte4(Asura_Vector_3(xStrip.m_pxVertices[v].m_fNX,
															      xStrip.m_pxVertices[v].m_fNY,
																  xStrip.m_pxVertices[v].m_fNZ));

			WRITEOUTDATA( &uNormal, sizeof(u_int) );

			short sU, sV;
			sU = CompressTexCoordToShort(xStrip.m_pxVertices[v].m_fU, bOutOfRange);
			bAnyOutOfRange |= bOutOfRange;
			sV = CompressTexCoordToShort(xStrip.m_pxVertices[v].m_fV, bOutOfRange);
			bAnyOutOfRange |= bOutOfRange;

			WRITEOUTDATA( &sU, sizeof(short) );
			WRITEOUTDATA( &sV, sizeof(short) );
		}	

		Asura_Assertf(!bAnyOutOfRange, ("Some texcoords on mesh %s were out of range and have been clamped.", pxMesh->Name));
#endif	// defined(ASURA_TARGET_SM3)
	}

//	for( u_int u= 0; u< uNumberOfStrips; u++)
	{	
		const TOOLS_OBJECTPROCESSOR_STRIP_TOUSE& xStrip = *TOOLS_OBJECTPROCESSOR_TOUSE::GetProcessedStrip();		
		
		for (u_int v=0; v<xStrip.m_uNumberOfIndices; v++)
		{
			u_short usIndex = (u_short)(xStrip.m_pusIndices[v]);
			WRITEOUTDATA( &usIndex, sizeof(u_short) );
		}
	}
	
	// Version 1 - LOD  min distance and hash ID added. RAR 09/05/03
	Asura_Hash_ID uLODHash = pxMesh->m_pxNextLowerMesh ? Asura_GetHashID(pxMesh->m_pxNextLowerMesh->Name.GetString()) : ASURA_HASH_ID_UNSET;

	WRITEOUTDATA(&pxMesh->m_fMinDistance,sizeof(float));
	WRITEOUTDATA(&uLODHash,sizeof(Asura_Hash_ID));

	return true;

#undef WRITEOUTDATA
}

#undef TOOLS_OBJECTPROCESSOR_TOUSE	

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

bool Tools_PC_Export::ExportMaterialResponses( Asura_SaveBlock& xSaveBlock, Asura_HashTree* const pxMaterialsToExport )
{
	Asura_Chunk_SaveBlock_Stream xStream( &xSaveBlock );
	
	const Asura_String xDUMMYNAME( "Material Response Texture" );

	// JAT 16/05/07 data format has changed so the version number ought to be increased
	const u_int uCURRENT_VERSION = Asura_MaterialResponse_System::EXPORTED_TEXTURE_VERSION;
	xStream.WriteResourceChunk_Start( ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC, ASURA_RESOURCEFILE_TYPE_PC_MATERIALRESPONSE, uCURRENT_VERSION, 0, xDUMMYNAME );

	int iNumberOfMaterialsToExport = pxMaterialsToExport->GetNumberOfNodes();

	Asura_Hash_ID* const puExportHashes = new Asura_Hash_ID[ iNumberOfMaterialsToExport ];
	Asura_Assert( puExportHashes, "Failed to allocate material response export array." );
	if( !puExportHashes ) return false;

	pxMaterialsToExport->MakeHashList( puExportHashes );
	
	xStream << iNumberOfMaterialsToExport;

	for( int i = 0; i < iNumberOfMaterialsToExport; ++i )
	{
		const Tools_MaterialResponse* const pxMaterial = Tools_MaterialResponse_System::GetMaterialByHashID( puExportHashes[i] );
		if( pxMaterial )
		{
			pxMaterial->WriteTextureLineToStream( xStream );
		}
		else
		{
			Asura_Assertf( false, ("Material to export (0x%x) couldn't be found.", puExportHashes[i]) );
		}
	}
	
	xStream.WriteResourceChunk_End();
	
	delete[] puExportHashes;

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_PC_Export::BeginMRPShaderExport()
//
// Begins the export process for material response pixel shaders
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_PC_Export::BeginMRPShaderExport()
{
	Asura_Direct3D_MRPShaderCompile::ResetLoadedShaderList();

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_PC_Export::EndMRPShaderExport()
//
// Finishes off the export process for material response pixel shaders. Stores
// the compiled shaders in a chunk
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_PC_Export::EndMRPShaderExport( const HANDLE xExportHandle )
{
	const int iNumShaders = Asura_Direct3D_MRPShaderCompile::GetNumShaders();
	if( iNumShaders == 0 ) return true;

	// Create the chunk

	Asura_Chunk_Stream_WindowsFile xStream( xExportHandle );

	xStream.WriteResourceChunk_Start( ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
		ASURA_RESOURCEFILE_TYPE_PC_MRPSHADERS,
		Asura_Chunk_ResourceFile_PC_MRPShaders::iCURRENT_VERSION, 0,
		"Material response pixel shaders" );

	xStream << iNumShaders;

	// Write out each shader

	Asura_TemplateHashTree_Iterator< ID3DXBuffer* > xIter( Asura_Direct3D_MRPShaderCompile::GetShaderTree() );
	while( !xIter.Done() )
	{
		const Asura_Hash_ID uNameHash = xIter.GetCurrentHash();
		ID3DXBuffer** const ppxShader = xIter.GetCurrent();

		xIter.Next();

		u_int uShaderSize = 0;
		const void* pShaderData = NULL;

		// If compilation failed, *ppxShader will be NULL, but we should still write an empty
		// shader to match the count in the chunk
		if( ppxShader && *ppxShader )
		{
			uShaderSize = (*ppxShader)->GetBufferSize();
			pShaderData = (*ppxShader)->GetBufferPointer();
		}

		xStream << uNameHash;
		xStream << uShaderSize;

		if( uShaderSize > 0 )
		{
			xStream.Write( uShaderSize, pShaderData );
		}

		SAFE_RELEASE( *ppxShader );
	}

	xStream.WriteResourceChunk_End();

	Asura_Direct3D_MRPShaderCompile::ResetLoadedShaderList();

	return true;
}

#endif // ASURA_USE_MATERIAL_RESPONSE_SYSTEM

////////////////////////////////////////////////////////////////////////////////
//
// Tools_PC_Export::ExportStaticDecals()
//
// Exports the static decal geometry. Decal materials/textures are handled
// separately (and platform-independent)
//
// Parameters:		hOutput - Handle to the output file
//
// Return value:	true iff successful
//
////////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_STATIC_DECAL_SYSTEM
bool Tools_PC_Export::ExportStaticDecals( const HANDLE hOutput )
{	
	Asura_Chunk_Memory_Stream xStream(16 * 1024 * 1024);	

	const u_int uCURRENT_VERSION = 1;
	const Asura_String xSTATICDECALS_RESOURCE_FILENAME( "StaticDecals" );
	xStream.WriteResourceChunk_Start( ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC, ASURA_RESOURCEFILE_TYPE_PC_STATICDECALS_SM3, uCURRENT_VERSION,0, xSTATICDECALS_RESOURCE_FILENAME );	

	// Kept the exported data very simple for now
	// Given more time, this could be improved - to enable better batching and/or less material/vb switching

	int iNumBuffers = Tools_StaticDecal_System::GetNumDecalBuffers();

	xStream << iNumBuffers;
	for(int i=0; i<iNumBuffers; i++)
	{	
		Tools_StaticDecal_System::OptimizeBuffer(i);
		Tools_StaticDecal_Buffer* pxBuffer = Tools_StaticDecal_System::GetDecalBufferByIndex(i);		

		xStream << pxBuffer->m_iModuleIndex;
		xStream << pxBuffer->m_uDecalMaterial;
		xStream << pxBuffer->m_iNumVerts;
		xStream << pxBuffer->m_iNumIndices;
		xStream << pxBuffer->m_xAABB;

		// Output the verts
		for(int j=0; j<pxBuffer->m_iNumVerts; j++)
		{
			Tools_StaticDecal_Vertex *pxVert = &pxBuffer->m_pxVerts[j];

			xStream << pxVert->xPos;
			xStream << pxVert->fU;
			xStream << pxVert->fV;
			xStream << pxVert->fCreationTime; // always 0.0f - 'creation time' value when used on dynamic decals
			xStream << pxVert->fAlpha;

			xStream << pxVert->xNormal;				
			xStream << pxVert->xTangent;				
			xStream << pxVert->xBinormal;

			xStream << pxVert->xSHRed.GetX();
			xStream << pxVert->xSHRed.GetY();
			xStream << pxVert->xSHRed.GetZ();
			xStream << pxVert->xSHRed.GetW();

			xStream << pxVert->xSHGreen.GetX();
			xStream << pxVert->xSHGreen.GetY();
			xStream << pxVert->xSHGreen.GetZ();
			xStream << pxVert->xSHGreen.GetW();

			xStream << pxVert->xSHBlue.GetX();
			xStream << pxVert->xSHBlue.GetY();
			xStream << pxVert->xSHBlue.GetZ();
			xStream << pxVert->xSHBlue.GetW();


		}

		// Output the indices
		for(int j=0; j<pxBuffer->m_iNumIndices; j++)
		{
			uint16 usTmp = (uint16)pxBuffer->m_piIndices[j];			
			xStream << usTmp;
		}
	}


	xStream.WriteResourceChunk_End();

	unsigned long uBytesWritten = 0;
	const void* pData = xStream.GetStartAddress();
	u_int uChunkSize = xStream.GetPosition();
	WriteFile(hOutput, pData, uChunkSize, &uBytesWritten, NULL);

	return true;
}
#endif //ASURA_USE_STATIC_DECAL_SYSTEM
