// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___DMADEBUG_H___
#define ___DMADEBUG_H___ 1

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "PS2/PS2Strip/PS2Strip_PS2Stuff.h"

// B E G I N   N A M E S P A C E /////////////////////////////////////////////

namespace DMADebug
{

// T Y P E S /////////////////////////////////////////////////////////////////

enum DBGMSG
{
    DBGMSG_ENTRY_TITLE,
    DBGMSG_ENTRY_FIELD,
    DBGMSG_ENTRY_FIELD_NL
};

enum DMA_ERR
{
	DMA_ERR_NO_ERROR   = 0,
	DMA_ERR_FATAL_CONT = 2, // A fatal error was found, but it's possible to keep going and look for more errors.
	DMA_ERR_FATAL_HALT = 4, // A fatal error was found, and it means it's not possible to look ahead for more errors.
	DMA_ERR_WARNING    = 8,
};

// G L O B A L S /////////////////////////////////////////////////////////////

extern char  gTempString[];
extern char* gOutputBuffer;

extern u_int *gDmaTransferQWord;
extern bool gDmaTransferQWordIsTag;		// Is this word a DMA tag?

extern u_int *gGifInput;

// P R O T O T Y P E S   A N D   I N L I N E   F U N C T I O N S /////////////

static inline bool IsFatal( int iErr )
{
    return( (iErr & (DMA_ERR_FATAL_HALT | DMA_ERR_FATAL_CONT)) != 0 );
}

static inline bool IsWarning( int iErr )
{
    return( (iErr & DMA_ERR_WARNING) != 0 );
}

bool DEBUG_parseConfig( const char* szInitString );
void DEBUG_reset( void );
int DEBUG_addError( int iErrNum, int iSeverity, const char* szErrString );
int DEBUG_addToOutputBuffer( const char* szToAdd );
int DEBUG_strlen( const char* szStr );
void DEBUG_addField( DBGMSG eType, char* final, const char* szFormat, ... );
void DEBUG_saveBuffers( const char* szFilename, bool bAppend );
void DEBUG_init( const char* cs );
void DEBUG_FreeAll( void );

int DMA_getCRC( u_int* puResult );
bool DMA_isValidAddress( void* pvAddr );
void *DMA_convertToMainMemAddress( void* pvAddr );
void DMA_parseConfig( const char* szInitString );
int DMA_emulateDMACycle( void );
int DMA_startDmaTransfer( int iChannelNum, u_int uCHCR );
int DMA_sceDmaSend( PS2STRIP_DMA_CHAN_ID eChanID, void* pvTag );
int DMA_sceDmaSendN( PS2STRIP_DMA_CHAN_ID eChanID, void* pvAddr, int iSize );

void GIF_parseConfig( const char* szInitString );
void GIF_saveGifRegisters( void );
bool GIF_isIdle( void );
int GIF_emulateGifCycle( void );
int GIF_disassemble( void );
int GIF_disassembleDmaList( PS2STRIP_DMA_CHAN_ID eChanID, void* pvTag );
int GIF_disassembleDmaListN( PS2STRIP_DMA_CHAN_ID eChanID, void* pvTag, int iNumQWs );
int GIF_disassembleXGKICK( void* pvAddr );
void GIF_reset( void );

void VIF_saveVifRegisters( void );
void VIF_parseConfig( const char* szInitString );
int VIF_disassemble( void );
int VIF_disassembleDmaList( PS2STRIP_DMA_CHAN_ID eChanID, void* pvTag );
int VIF_disassembleDmaListN( PS2STRIP_DMA_CHAN_ID eChanID, void* pvTag, int iNumQWs );

int DIS_dmaSend( PS2STRIP_DMA_CHAN_ID eChanID, u_int uCHCR, const char* szOptions );
int DIS_sceDmaSend( PS2STRIP_DMA_CHAN_ID eChanID, void* pvTag, const char* szOptions );
int DIS_sceDmaSendN( PS2STRIP_DMA_CHAN_ID eChanID, void* pvTag, int iNumQWs, const char* szOptions );

// E N D   N A M E S P A C E /////////////////////////////////////////////////

}

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___DMADEBUG_H___
