//////////////////////////////////////////////////////////////////////////////
//
// PS2Strip_PS2Stuff.h
//
// Copyright (c) 2002 Rebellion. All rights reserved.
//
// Description:
//
//     PS2 macros, types, etc., to save having to include PS2 headers.
//
//////////////////////////////////////////////////////////////////////////////

// B E G I N   F I L E   W R A P P E R ///////////////////////////////////////

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef ___PS2STRIP_PS2STUFF_H___
#define ___PS2STRIP_PS2STUFF_H___ 1

// M A C R O S ///////////////////////////////////////////////////////////////

// VIF1 register setting macros.
#define PS2STRIP_VIF1_SET_FLUSH(irq)					 (u_int)(((u_int)0x11 << 24) | ((u_int)(irq) << 31))
#define PS2STRIP_VIF1_SET_MSCNT(irq)					 (u_int)(((u_int)0x17 << 24) | ((u_int)(irq) << 31))
#define PS2STRIP_VIF1_SET_NOP(irq)						 ((u_int)(irq) << 31)
#define PS2STRIP_VIF1_SET_STCOL(irq)					 (u_int)(((u_int)0x31 << 24) | ((u_int)(irq) << 31))
#define PS2STRIP_VIF1_SET_STCYCL(wl, cl, irq)			 (u_int)((u_int)(cl) | ((u_int)(wl) << 8)  | ((u_int)0x01 << 24) | ((u_int)(irq) << 31))
#define PS2STRIP_VIF1_SET_STMASK(irq)					 (u_int)(((u_int)0x20 << 24) | ((u_int)(irq) << 31))
#define PS2STRIP_VIF1_SET_STMOD(stmod, irq)				 (u_int)((u_int)(stmod) | ((u_int)0x05 << 24) | ((u_int)(irq) << 31))
#define PS2STRIP_VIF1_SET_STROW(irq)					 (u_int)(((u_int)0x30 << 24) | ((u_int)(irq) << 31))
#define PS2STRIP_VIF1_SET_UNPACKR(vuaddr, num, cmd, irq) (u_int)((u_int)((vuaddr) | 0x8000) | ((u_int)(num) << 16) | ((u_int)(0x60 | (cmd)) << 24) | ((u_int)(irq) << 31))
#define PS2STRIP_VIF1_SET_UNPACK(vuaddr, num, cmd, irq)  (u_int)((u_int)(vuaddr) | ((u_int)(num) << 16) | ((u_int)(0x60 | (cmd)) << 24) | ((u_int)(irq) << 31))
#define PS2STRIP_VIF1_SET_ITOP(itop, irq)				 ((u_int)(itop) | ((u_int)0x04 << 24) | ((u_int)(irq) << 31))

// T Y P E S /////////////////////////////////////////////////////////////////

enum PS2STRIP_DMA_TAG_TYPE
{
    PS2STRIP_DMA_TAG_TYPE_REFE,
    PS2STRIP_DMA_TAG_TYPE_CNT,
    PS2STRIP_DMA_TAG_TYPE_NEXT,
    PS2STRIP_DMA_TAG_TYPE_REF,
    PS2STRIP_DMA_TAG_TYPE_REFS,
    PS2STRIP_DMA_TAG_TYPE_CALL,
    PS2STRIP_DMA_TAG_TYPE_RET,
    PS2STRIP_DMA_TAG_TYPE_END,
    PS2STRIP_DMA_TAG_TYPE_INVALID = 0xffffffff,
};

enum PS2STRIP_DMA_CHAN_ID
{
    PS2STRIP_DMA_CHAN_ID_VIF0,
    PS2STRIP_DMA_CHAN_ID_VIF1,
    PS2STRIP_DMA_CHAN_ID_GIF,
    PS2STRIP_DMA_CHAN_ID_FROMIPU,
    PS2STRIP_DMA_CHAN_ID_TOIPU,
    PS2STRIP_DMA_CHAN_ID_SIF0,
    PS2STRIP_DMA_CHAN_ID_SIF1,
    PS2STRIP_DMA_CHAN_ID_SIF2,
    PS2STRIP_DMA_CHAN_ID_FROMSPR,
    PS2STRIP_DMA_CHAN_ID_TOSPR,
};

union PS2Strip_DMA_Reg_CHCR
{
    struct
    {
        u_int DIR : 1,    // Direction
              p0  : 1,
              MOD : 2,    // Mode
              ASP : 2,    // Address stack pointer
              TTE : 1,    // Tag trasfer enable
              TIE : 1,    // Tag interrupt enable
              STR : 1,    // Start
              p1  : 7,
              TAG : 16;   // DMAtag
    };

    // For easy initialisation.
    u_int u32;
};

struct PS2Strip_DMA_Tag
{
    u_short           qwc;  // transfer count
    u_char            mark; // mark
    u_char            id;   // tag
    PS2Strip_DMA_Tag* next; // next tag
    u_int             p[2]; // padding
};

struct PS2Strip_VIF1_Mask
{
    u_int m0  : 2,
          m1  : 2,
          m2  : 2,
          m3  : 2,
          m4  : 2,
          m5  : 2,
          m6  : 2,
          m7  : 2,
          m8  : 2,
          m9  : 2,
          m10 : 2,
          m11 : 2,
          m12 : 2,
          m13 : 2,
          m14 : 2,
          m15 : 2;
};

struct PS2Strip_GIF_Reg_CNT
{
    u_int LOOPCNT : 15,	// Loop counter
          p0      :  1,
          REGCNT  :  4,	// Register number
          VUADDR  : 10,	// VU memory address
          p1      :  2;
};

struct PS2Strip_GIF_Reg_STAT
{
    u_int M3R   :  1, // Mask PATH3 by Register
          M3P   :  1, // Mask PATH3 by VIF
          IMT   :  1, // PATH3 IMAGE mode termination status
          PSE   :  1, // Pause
          p0    :  1,
          IP3   :  1, // Interruputed Path3
          P3Q   :  1, // Path3 in queue
          P2Q   :  1, // Path2 in queue
          P1Q   :  1, // Path1 in queue
          OPH   :  1, // Output Path
          APATH :  2, // Data Path
          DIR   :  1, // Direction
          p1    : 11,
          FQC   :  5, // GIF-FIFO Valid data counter
          p2    :  3;
};

struct PS2Strip_VIF1_V4_5_Data
{
    union
    {
        struct
        {
	        u_short usX : 5,
			        usY : 5,
			        usZ : 5,
			        usW : 1;
        };

        u_short usValue;
    };
};

// C O N S T A N T S /////////////////////////////////////////////////////////

// VIF1 unpack cmds.
static const u_int PS2STRIP_VIF1_CMD_S_32  =  0;
static const u_int PS2STRIP_VIF1_CMD_S_16  =  1;
static const u_int PS2STRIP_VIF1_CMD_S_8   =  2;
static const u_int PS2STRIP_VIF1_CMD_V2_32 =  4;
static const u_int PS2STRIP_VIF1_CMD_V2_16 =  5;
static const u_int PS2STRIP_VIF1_CMD_V2_8  =  6;
static const u_int PS2STRIP_VIF1_CMD_V3_32 =  8;
static const u_int PS2STRIP_VIF1_CMD_V3_16 =  9;
static const u_int PS2STRIP_VIF1_CMD_V3_8  = 10;
static const u_int PS2STRIP_VIF1_CMD_V4_32 = 12;
static const u_int PS2STRIP_VIF1_CMD_V4_16 = 13;
static const u_int PS2STRIP_VIF1_CMD_V4_8  = 14;
static const u_int PS2STRIP_VIF1_CMD_V4_5  = 15;

// Optional extras for VIF1 UNPACKs.
static const u_int PS2STRIP_VIF1_MASK_PROCESS_ENABLE = 0x10;       // OR onto one of the above cmds before passing to PS2STRIP_VIF1_SET_UNPACKR().
static const u_int PS2STRIP_VIF1_UNPACK_USN_FLAG     = 0x00004000; // OR onto result of calling PS2STRIP_VIF1_SET_UNPACKR().

// DMA tag constants.
static const u_int PS2STRIP_DMA_TAG_MAX_QWS  = (1 << 16) - 1;
static const u_int PS2STRIP_DMA_TAG_ID_SHIFT = 28;

// P R O T O T Y P E S   A N D   I N L I N E   F U N C T I O N S /////////////

static inline u_int Tools_PS2Strip_ConvertFloatForV45( float fIn )
{
    fIn *= 16.0f; // To fixed point with 4 fractional bits.
    fIn += 16.0f; // Bias so all values are positive.

    if( fIn <= 0.0f ) { return 0; }
    
    if( fIn >= 31.0f ) { return 31; }

    return static_cast<u_int>( fIn + 0.5f );
}

static inline short Tools_PS2Strip_ConvertFloatTo16BitFixed( u_int uNumFractionalBits, float fIn )
{
    fIn *= static_cast<float>( 1 << uNumFractionalBits );

    if( fIn >= 32767.0f ) { return 32767; }
    
    if( fIn <= -32768.0f ) { return -32768; }

    return static_cast<short>( fIn > 0.0f ? fIn + 0.5f : fIn - 0.5f );
}

// E N D   F I L E   W R A P P E R ///////////////////////////////////////////

#endif // #ifndef ___PS2STRIP_PS2STUFF_H___
