//////////////////////////////////////////////////////////////////////////////
//
// Tools_Modal_Overlay_SelectOverriddenItem.cpp
//
// Created 18 December 2008 by Andrew Southgate
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Tools_Modal_Overlay_SelectOverriddenItem.h"

#include "Tools_UI_Overrides_Filters.h"
#include "Tools_UI_Overrides.h"

////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectOverriddenItem::AcceptItem( int iUserData )
{
	Tools_Modal_Overlay_SelectOverriddenItem* pxThis = (Tools_Modal_Overlay_SelectOverriddenItem*)iUserData;
	pxThis->m_bAcceptItem = true;
}

void Tools_Modal_Overlay_SelectOverriddenItem::CancelItem( int iUserData )
{
	Tools_Modal_Overlay_SelectOverriddenItem* pxThis = (Tools_Modal_Overlay_SelectOverriddenItem*)iUserData;
	pxThis->m_bAcceptItem = false;
}

void Tools_Modal_Overlay_SelectOverriddenItem::DisableItem( int iUserData )
{
	Tools_Modal_Overlay_SelectOverriddenItem* pxThis = (Tools_Modal_Overlay_SelectOverriddenItem*)iUserData;
	pxThis->m_iItemIndex	= -1;
	pxThis->m_bAcceptItem	= true;
}


////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Public functions...
////////////////////////////////////////////////////////////////////////////////


Tools_Modal_Overlay_SelectOverriddenItem::Tools_Modal_Overlay_SelectOverriddenItem(
	Tools_UI_Overrides*	pxParent, 
	const char* const szTitle,
	const int iItemIndex,
	const int iOverrideItemIndex,
	const Asura_String *pszItemNames,
	const int iNumItems,
	const int iOverrideMapIndex,
	const Asura_String* pszOverrideMapNames,
	const int iNumOverrideMaps)
	: PARENT(szTitle)
	, m_pxParent(pxParent)
	, m_bAcceptItem(false)
	, m_pxOverridesFilters( NULL )
	, m_pxItemNamesDropDown( NULL )
	, m_pszFilteredItemNames( NULL )
	, m_iNumFilteredItems( 0 )
	, m_iFilteredItemIndex( 0 )
	, m_piFilteredItemIndicesLookup( NULL )
	, m_iItemIndex(iItemIndex)
	, m_pszItemNames(pszItemNames)
	, m_iNumItems(iNumItems)
	, m_xItemString("Item : ")
	, m_iOverrideIndex(iOverrideMapIndex)
	, m_pszOverrideNames(pszOverrideMapNames)
	, m_iNumOverrides(iNumOverrideMaps)
	, m_xOverrideNameString("Override Map: ")
	, m_xOverrideItemString("Override : ")
{
	m_xItemString				+= pszItemNames[ m_iItemIndex ];
	m_xOverrideNameString		+= pszOverrideMapNames[ m_iOverrideIndex ];
	m_xOverrideItemString		+= pszItemNames[ iOverrideItemIndex ];

	// initially unfiltered, so just copy the current Items
	m_pszFilteredItemNames = new const char*[ m_iNumItems ];
	Asura_Assert( ( m_pszFilteredItemNames != NULL ), "" );

	m_piFilteredItemIndicesLookup= new int[ m_iNumItems ];
	Asura_Assert( ( m_piFilteredItemIndicesLookup != NULL ), "" );

	for( int i = 0; i < m_iNumItems; i++ )
	{
		if( m_pszFilteredItemNames != NULL )
		{
			m_pszFilteredItemNames[ i ] = m_pszItemNames[ i ].GetString();
		}
		if( m_piFilteredItemIndicesLookup != NULL )
		{
			m_piFilteredItemIndicesLookup[ i ] = i;
		}
	}
	m_iFilteredItemIndex	= m_iItemIndex;
	m_iNumFilteredItems		= m_iNumItems;
}

/// End of function 'Tools_Modal_Overlay_SelectItem::Tools_Modal_Overlay_SelectItem'. ///

Tools_Modal_Overlay_SelectOverriddenItem::~Tools_Modal_Overlay_SelectOverriddenItem()
{
	delete m_pxOverridesFilters;
	m_pxOverridesFilters = NULL;

	delete [] m_pszFilteredItemNames;
	m_pszFilteredItemNames = NULL;

	delete [] m_piFilteredItemIndicesLookup;
	m_piFilteredItemIndicesLookup = NULL;
}



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Modal_Overlay_SelectOverriddenItem::InitialiseOverlay
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectOverriddenItem::InitialiseOverlay(Asura_Overlay* pxOverlay)
{
	Asura_Vector_2	xOffset(1.0f, 3.0f);
	Asura_Dialog*	pxNewDialog;

	// Initialise the dialogs...
	delete m_pxOverridesFilters;
	m_pxOverridesFilters = new Tools_UI_Overrides_FilterSet( m_pxParent, pxOverlay, NULL, xOffset, FiltersChanged, this );
	m_pxOverridesFilters->SetDefaults( m_pxParent->GetOverridesFilters() );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_StringDisplay_Dialog(m_xItemString.GetString(), xOffset);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_StringDisplay_Dialog(m_xOverrideNameString.GetString(), xOffset);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_StringDisplay_Dialog(m_xOverrideItemString.GetString(), xOffset);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::LargeSpacing();

	m_pxItemNamesDropDown = new Asura_DropDown_List_Dialog( xOffset, m_pszFilteredItemNames, &m_iFilteredItemIndex, 10, m_iNumFilteredItems, NULL, NULL, NULL, true );
	pxOverlay->AddDialog( m_pxItemNamesDropDown );
	xOffset.y 	+= Asura_Dialog::LargeSpacing();

	pxNewDialog	= new Asura_Function_Dialog("Accept", xOffset, AcceptItem, (int)this, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_Function_Dialog("Cancel", xOffset, CancelItem, (int)this, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	pxNewDialog	= new Asura_Function_Dialog("Disable", xOffset, DisableItem, (int)this, &pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y	+= Asura_Dialog::DefaultSpacing();

	ApplyFilters();
}

/// End of function 'Tools_Modal_Overlay_SelectOverriddenItem::InitialiseOverlay'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : FiltersChanged & ApplyFilters
// Author       : Mark Parry
// Date         : 6-July-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_Modal_Overlay_SelectOverriddenItem::FiltersChanged( void* const pxUserData )
{
	Tools_Modal_Overlay_SelectOverriddenItem* pxThis = (Tools_Modal_Overlay_SelectOverriddenItem*)pxUserData;
	pxThis->ApplyFilters();
}

void Tools_Modal_Overlay_SelectOverriddenItem::ApplyFilters( void )
{
	if( ( m_pxItemNamesDropDown != NULL ) && ( m_pxOverridesFilters != NULL ) && ( m_pszItemNames != NULL ) )
	{
		int iSelect = -1;
		m_iNumFilteredItems = 0;
		for( int i = 0; i < m_iNumItems; i++ )
		{
			const char* const szItemName = m_pszItemNames[ i ].GetString();
			if( m_pxOverridesFilters->MatchesFilters( szItemName ) )
			{
				if( i == m_iFilteredItemIndex )
				{
					iSelect = m_iNumFilteredItems;
				}
				if( ( iSelect == -1 ) && ( i == m_iItemIndex ) )
				{
					iSelect = m_iNumFilteredItems;
				}
				m_pszFilteredItemNames[ m_iNumFilteredItems ] = szItemName;
				m_piFilteredItemIndicesLookup[ m_iNumFilteredItems ] = i;
				m_iNumFilteredItems++;
			}
		}
		m_pxItemNamesDropDown->SelectEntry( 0 );
		m_pxItemNamesDropDown->ReplaceStrings( m_iNumFilteredItems, m_pszFilteredItemNames );
		m_pxItemNamesDropDown->SelectEntry( ( iSelect == -1 ) ? 0 : iSelect );
	}
}

int	Tools_Modal_Overlay_SelectOverriddenItem::GetSelectedIndex(void) const
{
	if( m_iItemIndex == -1 )
	{
		// override has been disabled
		return( m_iItemIndex );
	}
	return( m_piFilteredItemIndicesLookup[ m_iFilteredItemIndex ] );
}

bool Tools_Modal_Overlay_SelectOverriddenItem::DoModal(void)
{
	ShowOverlay();
	return( m_bAcceptItem);
}


/// End of file. ///

