//////////////////////////////////////////////////////////////////////////////
//
// Tools_UI_Overrides.cpp
//
// Created 10 December 2008 by Andrew Southgate
//
// Copyright (c) 2008 Rebellion. All rights reserved.
//
//////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// Asura independent header-files...
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Table_Dialog.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"

// Tools header-files...
#include "Tools_Overlays.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Materials/Tools_Overlay_MaterialResponseOverrides.h" // for Tools_Modal_Overlay_SelectCurrentOverrideMap

#include "Tools_Modal_Overlay_SelectOverriddenItem.h"
#include "Tools_UI_Overrides_Filters.h"
#include "Tools_UI_Overrides.h"


namespace
{

	class SelectCurrentOverrideDialog : public Asura_Function_Dialog
	{
	public:
		SelectCurrentOverrideDialog(Tools_UI_Overrides* const pxParent, const Asura_Vector_2& xOffset, void(*pfnCallback)(int), int iUserData, bool(*pfnIsEnabled)(int) )
			: m_pxParent( pxParent ), Asura_Function_Dialog( "", xOffset, pfnCallback, iUserData, NULL, NULL )
		{
			SetIsEnabledFunction(pfnIsEnabled, iUserData);
		}

		void Update( const Asura_Vector_2& xOrigin );

	private:
		typedef Asura_Function_Dialog PARENT;

		Tools_UI_Overrides* const m_pxParent;
	};

	void SelectCurrentOverrideDialog::Update( const Asura_Vector_2& xOrigin )
	{
		if( m_pxParent != NULL )
		{
			const Asura_Hash_ID	uCurrentHash		= m_pxParent->GetInterface().GetCurrentOverrideMap();
			const int			iCurrentIndex		= m_pxParent->GetIndexForOverride( uCurrentHash );
			const char*			szCurrentOverride	= ( iCurrentIndex >= 0 ) ? m_pxParent->GetOverrideNameArray()[ iCurrentIndex ].GetString() : "*** None ***";
			char				szText[256];
			Asura_PrintfToBfr( szText, "Select Current Override: %s", 256, true, szCurrentOverride );
			UpdateLabel( szText );
		}

		PARENT::Update( xOrigin );
	}

}




////////////////////////////////////////////////////////////////////////////////






////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::TableCell_OverrideHash::TableCell_OverrideHash
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////

 Tools_UI_Overrides::TableCell_OverrideHash::TableCell_OverrideHash( Asura_Table_Dialog& xTableDialogue,
																	 Tools_UI_Overrides* const pxParent,
																	 const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																	 const u_int uRow, const u_int uColumn )
																	 : PARENT( xTableDialogue, xOffset, xMinCellSize, uRow, uColumn )
																	 , m_pxParent( pxParent )
{
	Asura_Vector_2	xLocalOffset(0.0f, 0.0f);

	// Create the dialog...
	m_pxDialog = new TableCell_OverrideHash_EntryDialog( pxParent, uColumn, "", xLocalOffset, SelectNew, (int)(this) );
	AddDialogToPanel( m_pxDialog );
	xLocalOffset[0]	+= m_pxDialog->m_xMaxCoord[0];

	// Refresh the text...
	RefreshDialog();
}

/// End of function 'Tools_UI_Overrides::TableCell_OverrideHash::TableCell_OverrideHash'. ///

 ////////////////////////////////////////////////////////////////////////////////
 // Function     : Tools_UI_Overrides::TableCell_OverrideHash::SelectNew
 // Author       : Andrew Southgate
 // Date         : 11 December 2008
 ////////////////////////////////////////////////////////////////////////////////

 void Tools_UI_Overrides::TableCell_OverrideHash::SelectNew( void )
 {
	const Asura_String*		pszOverrideNameArray	= m_pxParent->GetOverrideNameArray();
	const Asura_Hash_ID*	puOverrideHashArray		= m_pxParent->GetOverrideHashArray();
	const int				iNumOverrides			= m_pxParent->GetOverrideNumEntries();

	const int				iOverrideIndex			= m_iColumn;
	const Asura_Hash_ID		uOverrideHash			= puOverrideHashArray[iOverrideIndex];

	const Asura_String*		pxNameArray				= m_pxParent->GetNameArray();
	const Asura_Hash_ID*	puHashArray				= m_pxParent->GetHashArray();
	const int				iNumEntries				= m_pxParent->GetNumEntries();

	const int				iIndex					= m_iRow;
	const Asura_Hash_ID		uHash					= puHashArray[iIndex];


	Asura_Hash_ID		uOverridenHash;
	if (!m_pxParent->GetInterface().GetOverrideMapEntry(uOverrideHash, uHash, uOverridenHash))
	{
		uOverridenHash = uHash;
	}
	int	iOverridenIndex	= m_pxParent->GetIndexForEntry(uOverridenHash, iIndex);

 	Tools_Modal_Overlay_SelectOverriddenItem xSelectionOverlay( m_pxParent, "Select override", iIndex, iOverridenIndex, pxNameArray, iNumEntries, iOverrideIndex, pszOverrideNameArray, iNumOverrides );
 
 	 if( xSelectionOverlay.DoModal() )
 	 {
		 int iSelIndex = xSelectionOverlay.GetSelectedIndex();
		 if (iSelIndex >= 0)
		 {
			 m_pxParent->GetInterface().SetOverrideMapEntry(uOverrideHash, pxNameArray[iIndex], pxNameArray[iSelIndex]);
		 }
		 else
		 {
			 m_pxParent->GetInterface().RemoveOverrideMapEntry(uOverrideHash, uHash);
		 }
 	 }
 }

 /// End of function 'Tools_UI_Overrides::TableCell_OverrideHash::SelectNew'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::TableCell_OverrideHash::Compare
// Author       : Andrew Southgate
// Date         : 10 December 2008
////////////////////////////////////////////////////////////////////////////////
int Tools_UI_Overrides::TableCell_OverrideHash::Compare(const Asura_TableCell_Dialog& xOther) const
{
	const TableCell_OverrideHash*	pxOtherOverrideDialog	= static_cast<const TableCell_OverrideHash*>(&xOther);
	const Asura_Dialog*				pxMySortDialog			= m_pxDialog;
	const Asura_Dialog*				pxOtherSortDialog		= pxOtherOverrideDialog->m_pxDialog;

	return(pxMySortDialog->m_xName.Compare(pxOtherSortDialog->m_xName));
}
/// End of function 'Tools_UI_Overrides::TableCell_OverrideHash::Compare'. ///



void Tools_UI_Overrides::TableCell_OverrideHash::RefreshDialog(void)
{
	Tools_UI_Overrides_Interface&	xInterface			= m_pxParent->GetInterface();
	const Asura_Hash_ID*			puOverrideHashArray	= m_pxParent->GetOverrideHashArray();
	const Asura_Hash_ID				uOverrideMapHash	= puOverrideHashArray[m_iColumn];
	const Asura_Hash_ID*			puItemHashArray		= m_pxParent->GetHashArray();
	const Asura_Hash_ID				uItemHash			= puItemHashArray[m_iRow];
	Asura_Hash_ID					uItemOverrideHash;


	if (xInterface.GetOverrideMapEntry(uOverrideMapHash, uItemHash, uItemOverrideHash) && uItemOverrideHash != uItemHash)
	{
		const Asura_String*	pxHeadings				= m_pxParent->GetHeadingArray();
		const int			iOverriddenItemIndex	= m_pxParent->GetIndexForEntry(uItemOverrideHash);
		const char*			szOverriddenItemName	= (iOverriddenItemIndex >= 0) ? pxHeadings[iOverriddenItemIndex].GetString() : "???";
		
		m_pxDialog->UpdateLabel(szOverriddenItemName);

	} else {
		m_pxDialog->UpdateLabel("*** none ***");
	}
	// Ensure our cell is big enough...
	UpdateSize();
}



static const Asura_Vector_2	g_xTestMinimumCellSize(6.0f, 2.0f);
static Asura_Vector_2 g_xTableSize(80.0f, 21.0f);

Tools_UI_Overrides::Tools_UI_Overrides(Tools_UI_Overrides_Interface &xInterface) 
	:	m_xInterface(xInterface),
		m_pxNameArray(NULL),
		m_pxHeadingArray(NULL),
		m_puHashArray(NULL),
		m_iNumEntries(0),
		m_pxOverrideNameArray(NULL),
		m_pxOverrideHeadingArray(NULL),
		m_puOverrideHashArray(NULL),
		m_pxOverridesFilters(NULL),
		m_pbEnableTableRows(NULL),
		m_iNumOverrides(0),
		m_bOnlyItemsWithAnOverride(false),
		m_bLastOnlyShowItemsWithAnOverride(false),
		m_pxTableDialog(NULL),
		m_pxNoOverridesDialog(NULL),
		m_pxSelectCurrentOverrideDialog(NULL)
{

}


Tools_UI_Overrides::~Tools_UI_Overrides() 
{
	delete m_pxOverridesFilters;
	m_pxOverridesFilters = NULL;
	DestroyTable();
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::AddDialogs
// Author       : Andrew Southgate
// Date         : 10 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides::AddDialogs( Asura_Overlay *pxOverlay, Asura_Vector_2& xOffset, const Asura_Vector_2& xTableSize )
{
	delete m_pxOverridesFilters;
	m_pxOverridesFilters = new Tools_UI_Overrides_FilterSet( this, pxOverlay, NULL, xOffset, FiltersChanged, this, AreAnyOverridesPresent);


	Asura_Tickbox_Dialog* pxNewTickbox;
	pxNewTickbox = new Asura_Tickbox_Dialog( "Only show templates with an override", xOffset, &m_bLastOnlyShowItemsWithAnOverride);
	pxNewTickbox->SetClickedFunction( OnlyShowItemsWithAnOverrideClicked, (int)this );
	pxNewTickbox->SetIsEnabledFunction( AreAnyOverridesPresent, (int)this );
	pxOverlay->AddDialog( pxNewTickbox );
	xOffset.y = pxNewTickbox->m_xMaxCoord.y + Asura_Tickbox_Dialog::DefaultSpacing();


	pxOverlay->AddDialog( m_pxNoOverridesDialog = new Asura_StringDisplay_Dialog( "There are currently no overrides.", xOffset ));

	// Table...
	m_pxTableDialog = new Asura_Table_Dialog( xOffset, xTableSize.x, xTableSize.y, false );
	m_pxTableDialog->SetIsRowEnabledCallBack( IsTableRowEnabled, this );
	m_pxTableDialog->SetGetColourCallBack( TableGetColourCallBack, this );
	pxOverlay->AddDialog( m_pxTableDialog);

	xOffset.y = m_pxTableDialog->m_xMaxCoord.y + Asura_Dialog::DefaultSpacing();

	pxOverlay->AddDialog( new Asura_Function_Dialog( "Create New Override", xOffset, CreateNewOverrideMap, (int)this ) );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxOverlay->AddDialog( m_pxSelectCurrentOverrideDialog = new SelectCurrentOverrideDialog( this, xOffset, SelectCurrentOverrideMap, (int)this, AreAnyOverridesPresent ) );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_Function_Dialog *pxFuncDialog = new Asura_Function_Dialog( "Delete Current Override", xOffset, DeleteCurrentOverrideMap, (int)this, NULL, NULL );
	pxFuncDialog->SetIsEnabledFunction(IsCurrentOverrideMapValid, (int)this);
	pxOverlay->AddDialog( pxFuncDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxFuncDialog = new Asura_Function_Dialog( "Copy Current Override", xOffset, CopyCurrentOverrideMap, (int)this, NULL, NULL );
	pxFuncDialog->SetIsEnabledFunction(IsCurrentOverrideMapValid, (int)this);
	pxOverlay->AddDialog( pxFuncDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxFuncDialog = new Asura_Function_Dialog( "Rename Current Override", xOffset, RenameCurrentOverrideMap, (int)this, NULL, NULL );
	pxFuncDialog->SetIsEnabledFunction(IsCurrentOverrideMapValid, (int)this);
	pxOverlay->AddDialog( pxFuncDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	CreateTable();

	m_pxOverridesFilters->PopulateFilters(true);
//	m_pxOverridesFilters->SetDefaults( m_pxParent->GetOverridesFilters() );

}
/// End of function 'Tools_UI_Overrides::AddDialogs'. ///


void Tools_UI_Overrides::FiltersChanged( void* const pxUserData )
{
	Tools_UI_Overrides* pxThis = (Tools_UI_Overrides*)pxUserData;
	pxThis->ApplyFilters();
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::ApplyFilters
// Author       : Andrew Southgate
// Date         : 10 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides::ApplyFilters( void )
{
	delete [] m_pbEnableTableRows;
	m_pbEnableTableRows = NULL;
	if ( m_pxTableDialog != NULL )
	{
		const u_int uRows			= m_pxTableDialog->GetNumRows();
		const u_int uNumEntries	= (u_int)m_iNumEntries;
		if( uRows == uNumEntries )
		{
			if( ( m_pxNameArray != NULL ) )
			{
				m_pbEnableTableRows = new bool[ uRows ];
				if( m_pbEnableTableRows != NULL )
				{
					for( u_int uRow = 0; uRow < uRows; uRow++ )
					{
						bool bEnableRow;

						bEnableRow = m_pxOverridesFilters ? m_pxOverridesFilters->MatchesFilters( m_pxNameArray[ uRow ].GetString() ) : true;
						bEnableRow &= m_bLastOnlyShowItemsWithAnOverride ? DoesItemHaveOverride( m_puHashArray[ uRow ] ) : true;

						m_pbEnableTableRows[ uRow ] = bEnableRow;
					}
				}
			}
		}
	}
	m_bOnlyItemsWithAnOverride = m_bLastOnlyShowItemsWithAnOverride;
}
/// End of function 'Tools_UI_Overrides::ApplyFilters'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::ChangeTableColumnColour
// Author       : Andrew Southgate
// Date         : 10 December 2008
////////////////////////////////////////////////////////////////////////////////
unsigned Tools_UI_Overrides::ChangeTableColumnColour( const unsigned uColour, const u_int uColumn )
{
	if( ( m_puOverrideHashArray != NULL ) &&
		( uColumn < (u_int)m_iNumOverrides ) &&
		( m_xInterface.IsOverrideMapActive(m_puOverrideHashArray[ uColumn ]) ) )
	{
		return( uColour & 0xffffff00 );		// make it yellow
	}
	return( uColour );
}

/// End of function 'Tools_UI_Overrides::ChangeTableColumnColour'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::CopyCurrentOverride
// Author       : Andrew Southgate
// Date         : 10 December 2008
////////////////////////////////////////////////////////////////////////////////

void Tools_UI_Overrides::CopyCurrentOverrideMap( void )
{
	const Asura_Hash_ID	uCurrentHash	= m_xInterface.GetCurrentOverrideMap();
	const int			iOverrideIndex	= GetIndexForOverride( uCurrentHash, -1 );
	if( iOverrideIndex < 0 )
	{
		Asura_Assert(false, "Cannot find index for specified override.");
		return;
	}
	bool bAwaitingValidOverride	= true;
	while( bAwaitingValidOverride )
	{
		Asura_String xTitle;
		xTitle += "Copy '";
		xTitle += m_pxOverrideNameArray[ iOverrideIndex ];
		xTitle += "' to...";
		Asura_String xOverrideName = GetOverrideName();

		GetString_Overlay xOverlay( xTitle.GetString(), xOverrideName.GetString() );
		if( !xOverlay.DoModal() || (!m_xInterface.DoesOverrideMapExist(Asura_GetHashID( xOverlay.GetString() )) && m_xInterface.CopyOverrideMap(uCurrentHash, xOverlay.GetString()) ) )
		{
			bAwaitingValidOverride	= false;
		}
	}

}

/// End of function 'Tools_UI_Overrides::CopyCurrentOverride'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::CreateCellDialog
// Author       : Andrew Southgate
// Date         : 10 December 2008
////////////////////////////////////////////////////////////////////////////////

Asura_TableCell_Dialog* Tools_UI_Overrides::CreateCellDialog(
	Asura_Table_Dialog& xTableDialog,
	const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize, const u_int uRow,
	const u_int uColumn, void* const pxUserData )
{
	Tools_UI_Overrides*	pxThis = (Tools_UI_Overrides*)pxUserData;
	return( new TableCell_OverrideHash( xTableDialog, pxThis, xOffset, xMinCellSize, uRow, uColumn ) );
}

/// End of function 'Tools_UI_Overrides::CreateCellDialog'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::CreateHeadingArray
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides::CreateHashArray(const Asura_String* pszNameArray, Asura_Hash_ID* &pxHashArray, int iNum)
{
	pxHashArray	= new Asura_Hash_ID[ iNum ];
	for( int i = 0; i < iNum; i++ )
	{
		pxHashArray[ i ] = pszNameArray[ i ].GetHash();
	}
}


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::CreateHeadingArray
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides::CreateHeadingArray(const Asura_String* pszNameArray, Asura_String* &pxHeadingArray, int iNum)
{
	pxHeadingArray = new Asura_String[iNum];

	for( int i = 0 ; i < iNum; i++ )
	{
		const Asura_String	xFullPath( pszNameArray[ i ] );

		Asura_String		xPath;
		Asura_String		xFilename;
		Asura_String		xExtension;

		xFullPath.SplitFilename(&xPath, &xFilename, &xExtension);

		pxHeadingArray[i] = "";
// 		pxHeadingArray[i] += "[";
// 		char buf[100];
// 		pxHeadingArray[i] += _itoa(i, buf, 10);
// 		pxHeadingArray[i] += "] ";

		if( xPath.IsBlank() )
		{
			pxHeadingArray[i]	+= xFilename;
		}
		else
		{
			pxHeadingArray[i]	+= "(";
			pxHeadingArray[i]	+= xPath;
			pxHeadingArray[i]	+= ")\n";
			pxHeadingArray[i]	+= xFilename;
		}

		if( !xExtension.IsBlank() )
		{
			pxHeadingArray[i]	+= ".";
			pxHeadingArray[i]	+= xExtension;
		}
	}

}
/// End of function 'Tools_UI_Overrides::CreateHeadingArray'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::CreateNewOverrideMap
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides::CreateNewOverrideMap()
{
	bool	bAwaitingValidOverride	= true;
	while( bAwaitingValidOverride )
	{
		Asura_String xOverrideName = GetOverrideName();
		GetString_Overlay xOverlay( "Enter override name", xOverrideName.GetString() );
		if( xOverlay.DoModal() )
		{
			const char*	  szOverrideName = xOverlay.GetString();

			if (m_xInterface.CreateNewOverrideMap(szOverrideName))
			{
				bAwaitingValidOverride	= false;

				m_xInterface.RefreshAll();
			}
		}
		else
		{
			bAwaitingValidOverride	= false;
		}
	}

}
/// End of function 'Tools_UI_Overrides::CreateNewOverrideMap'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::CreateTable
// Author       : Andrew Southgate
// Date         : 10 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides::CreateTable(void)
{
	// clear up everything
	delete [] m_pxNameArray; m_pxNameArray = NULL;
	delete [] m_puHashArray; m_puHashArray = NULL;
	delete [] m_pxHeadingArray; m_pxHeadingArray = NULL;

	delete [] m_pxOverrideNameArray; m_pxOverrideNameArray = NULL;
	delete [] m_puOverrideHashArray; m_puOverrideHashArray = NULL;
	delete [] m_pxOverrideHeadingArray; m_pxOverrideHeadingArray = NULL;

	// Get the entry and override names
	m_xInterface.GetEntryNames(m_pxNameArray, m_iNumEntries);
	m_xInterface.GetOverrideNames(m_pxOverrideNameArray, m_iNumOverrides);

	// Get the headings (first entry in each row/column)
	CreateHeadingArray(m_pxNameArray, m_pxHeadingArray, m_iNumEntries);

	m_pxOverrideHeadingArray = new Asura_String[m_iNumOverrides];
	for( int i = 0; i < m_iNumOverrides; i++ )
	{
		m_pxOverrideHeadingArray[i]	= m_pxOverrideNameArray[i];
	}

	// Create the hash arrays
	CreateHashArray(m_pxNameArray, m_puHashArray, m_iNumEntries);
	CreateHashArray(m_pxOverrideNameArray, m_puOverrideHashArray, m_iNumOverrides);


	// Create the table...
	if( ( m_iNumEntries > 0 ) && ( m_iNumOverrides > 0 ) && ( m_pxTableDialog != NULL ) )
	{
		const bool bUseMaterialHeadings = true;
		m_pxTableDialog->CreateTable( g_xTestMinimumCellSize, m_iNumEntries, m_iNumOverrides, CreateCellDialog, this, ( bUseMaterialHeadings ) ? m_pxHeadingArray : NULL, m_pxOverrideHeadingArray );
		if( bUseMaterialHeadings )
		{
			m_pxTableDialog->SortRowsUsingHeadings( false );
		}
		m_pxTableDialog->SortColumnsUsingHeadings( false );

		if( m_pxNoOverridesDialog != NULL )
		{
			m_pxNoOverridesDialog->SetHidden( true );
		}
	}
	else
	{
		if( m_pxNoOverridesDialog != NULL )
		{
			m_pxNoOverridesDialog->SetHidden( false );
		}
	}

	// and finally update the filters
	ApplyFilters();
}

/// End of function 'Tools_UI_Overrides::CreateTable'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::DeleteCurrentOverrideMap
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////

void Tools_UI_Overrides::DeleteCurrentOverrideMap( void )
{
	const Asura_Hash_ID	uCurrentHash	= m_xInterface.GetCurrentOverrideMap();
	const int			iOverrideIndex	= GetIndexForOverride( uCurrentHash, -1 );
	if( iOverrideIndex >= 0 )
	{
		const Asura_String&	xOverrideName	= m_pxOverrideNameArray[iOverrideIndex];
		Asura_String	xMessageString("Are you sure you want to delete the \"");

		xMessageString	+= xOverrideName;
		xMessageString	+= "\" override?";

		YesNo_Overlay	xSelectionOverlay(xMessageString);

		if(xSelectionOverlay.DoModal())
		{
			m_xInterface.DeleteOverrideMap(uCurrentHash);
		}

	}
}

/// End of function 'Tools_UI_Overrides::DeleteCurrentOverrideMap'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::DestroyTable
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides::DestroyTable(void)
{
	// Remove the current table...
	if( m_pxTableDialog != NULL )
	{
		m_pxTableDialog->DestroyTable();
	}

	// Tidy-up the data...
	delete[] m_pxNameArray;
	delete[] m_pxHeadingArray;
	delete[] m_puHashArray;

	delete[] m_pxOverrideNameArray;
	delete[] m_pxOverrideHeadingArray;
	delete[] m_puOverrideHashArray;

	// Reset the variables...
	m_pxNameArray		= NULL;
	m_pxHeadingArray	= NULL;
	m_puHashArray		= NULL;
	m_iNumEntries		= 0;

	m_pxOverrideNameArray		= NULL;
	m_pxOverrideHeadingArray	= NULL;
	m_puOverrideHashArray		= NULL;
	m_iNumOverrides				= 0;

	delete[]( m_pbEnableTableRows );
	m_pbEnableTableRows			= NULL;
}

/// End of function 'Tools_UI_Overrides::DestroyTable'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::DoesItemHaveOverride
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
bool Tools_UI_Overrides::DoesItemHaveOverride( const Asura_Hash_ID uHash )
{
	return m_xInterface.IsItemOverriden(uHash);
}
/// End of function 'Tools_UI_Overrides::DoesItemHaveOverride'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::GetIndexForOverride
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
int Tools_UI_Overrides::GetIndexForEntry( const Asura_Hash_ID uHash, const int iDefaultIndex) const
{
	for( int i = 0; i < m_iNumEntries; i++ )
	{
		if( uHash == m_puHashArray[ i ] )
		{
			return( i );
		}
	}
	return( iDefaultIndex );
}
/// End of function 'Tools_UI_Overrides::GetIndexForEntry'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::GetIndexForOverride
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
int Tools_UI_Overrides::GetIndexForOverride( const Asura_Hash_ID uHash, const int iDefaultIndex) const
{
	for( int i = 0; i < m_iNumOverrides; i++ )
	{
		if( uHash == m_puOverrideHashArray[ i ] )
		{
			return( i );
		}
	}
	return( iDefaultIndex );
}
/// End of function 'Tools_UI_Overrides::GetIndexForOverride'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::GetOverrideName
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
Asura_String Tools_UI_Overrides::GetOverrideName()
{
	Asura_Hash_ID						uOverrideHash;
	char szOverrideMapName[ 256 ];
	for( int i = 1; i < 1000; i++ )
	{
		Asura_PrintfToBfr( szOverrideMapName, "Override%d", sizeof( szOverrideMapName ), true, i );
		uOverrideHash	= Asura_GetHashID( szOverrideMapName );
		if (!m_xInterface.DoesOverrideMapExist(uOverrideHash))
		{
			return( Asura_String( szOverrideMapName ) );
		}
	}
	return( Asura_String( "Override" ) );

}
/// End of function 'Tools_UI_Overrides::GetOverrideName'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::IsCurrentOverrideMapValid 
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////

bool Tools_UI_Overrides::IsCurrentOverrideMapValid( int iUserData )
{
	return ((Tools_UI_Overrides*)(iUserData))->m_xInterface.GetCurrentOverrideMap() != ASURA_HASH_ID_UNSET;
}

/// End of function 'Tools_UI_Overrides::IsCurrentOverrideMapValid'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::IsTableRowEnabled
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
bool Tools_UI_Overrides::IsTableRowEnabled(const u_int uRow, void* const pxUserData)
{
	Tools_UI_Overrides* pxThis = (Tools_UI_Overrides*)pxUserData;
	if( ( pxThis->m_pbEnableTableRows != NULL ) && ( uRow < (u_int)pxThis->m_iNumEntries ) )
	{
		return( pxThis->m_pbEnableTableRows[ uRow ] );
	}
	return( true );

}
/// End of function 'Tools_UI_Overrides::IsTableRowEnabled'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::OnlyShowItemsWithAnOverrideClicked
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides::OnlyShowItemsWithAnOverrideClicked( void )
{
	if (m_bLastOnlyShowItemsWithAnOverride ^ m_bOnlyItemsWithAnOverride)
	{
		ApplyFilters();
	}
}
/// End of function 'Tools_UI_Overrides::OnlyShowItemsWithAnOverrideClicked'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::Refresh
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides::Refresh( void )
{
	DestroyTable();
	CreateTable();
}
/// End of function 'Tools_UI_Overrides::Refresh'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::RefreshCell
// Author       : Andrew Southgate
// Date         : 11 December 2008
////////////////////////////////////////////////////////////////////////////////
void Tools_UI_Overrides::RefreshCell( Asura_Hash_ID uOverrideMapHash, Asura_Hash_ID uEntryHash )
{
	const int iOverrideMapIndex = GetIndexForOverride(uOverrideMapHash);
	const int iEntryIndex = GetIndexForEntry(uEntryHash);

	if (iOverrideMapIndex >= 0 && iEntryIndex >= 0)
	{
		Asura_TableCell_Dialog* const pxCell = m_pxTableDialog->GetCell( iEntryIndex, iOverrideMapIndex );
		if( pxCell != NULL )
		{
			( (Tools_UI_Overrides::TableCell_OverrideHash*) pxCell )->RefreshDialog();
		}
	}
}
/// End of function 'Tools_UI_Overrides::RefreshCell'. ///


////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::RenameCurrentOverrideMap
// Author       : Andrew Southgate
// Date         : 10 December 2008
////////////////////////////////////////////////////////////////////////////////

void Tools_UI_Overrides::RenameCurrentOverrideMap( void )
{
	const Asura_Hash_ID	uCurrentHash	= m_xInterface.GetCurrentOverrideMap();
	const int			iOverrideIndex	= GetIndexForOverride( uCurrentHash, -1 );
	if( iOverrideIndex < 0 )
	{
		Asura_Assert(false, "Cannot find index for specified override.");
		return;
	}
	bool bAwaitingValidOverride	= true;
	while( bAwaitingValidOverride )
	{
		Asura_String xTitle;
		xTitle += "Rename '";
		xTitle += m_pxOverrideNameArray[ iOverrideIndex ];
		xTitle += "' to...";
		Asura_String xOverrideName = GetOverrideName();

		GetString_Overlay xOverlay( xTitle.GetString(), xOverrideName.GetString() );
		if( !xOverlay.DoModal() || (!m_xInterface.DoesOverrideMapExist(Asura_GetHashID( xOverlay.GetString() )) && m_xInterface.RenameOverrideMap(uCurrentHash, xOverlay.GetString()) ) )
		{
			bAwaitingValidOverride	= false;
		}
	}
}
/// End of function 'Tools_UI_Overrides::RenameCurrentOverrideMap'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_UI_Overrides::SelectCurrentOverride
// Author       : Andrew Southgate
// Date         : 10 December 2008
////////////////////////////////////////////////////////////////////////////////

void Tools_UI_Overrides::SelectCurrentOverrideMap(void)
{
 	const Asura_String*								pxOverrideNameArray	= GetOverrideNameArray();
	const Asura_Hash_ID*							puOverrideHashArray		= GetOverrideHashArray();
	const int										iNumOverrides			= m_iNumOverrides;

	const Asura_Hash_ID								uCurrentHash			= m_xInterface.GetCurrentOverrideMap();
	const int										iOverrideIndex			= GetIndexForOverride(uCurrentHash, 0);

	const char **									pszOverrideNameArray = new const char *[iNumOverrides];

	for (int i = 0; i < iNumOverrides; ++i)
	{
		pszOverrideNameArray[i] = pxOverrideNameArray[i].GetString();
	}

	Tools_Modal_Overlay_SelectCurrentOverrideMap	xSelectionOverlay("Select current override", iOverrideIndex, pszOverrideNameArray, iNumOverrides);

	if(xSelectionOverlay.DoModal())
	{
		const int	iNewOverrideIndex	= xSelectionOverlay.GetOverrideIndex();

		if(iNewOverrideIndex >= 0)
		{
			const Asura_Hash_ID	uNewOverrideHash	= puOverrideHashArray[iNewOverrideIndex];
			m_xInterface.ActivateOverrideMap(uNewOverrideHash);
		}
		else
		{
			m_xInterface.DeactivateOverrideMap(uCurrentHash);
		}
	}

	delete [] pszOverrideNameArray;
}

/// End of function 'Tools_UI_Overrides::SelectCurrentOverrideMap'. ///
