
///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_PFX_Overrides.cpp
// Author : Andy Southgate
// Date   : 02-Dec-2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

#include "Core/Asura.h"

// This file's header...
#include "Particles/Tools_PFX_Overrides.h"

// Asura independent header-files...
#include "Core/Asura_SortAndSearch.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Table_Dialog.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Tools_Chunk_PFXOverrides.h"
#include "SpecialFX/PFX/Asura_PFX_System.h"
#include "Materials/Tools_Overlay_MaterialResponseOverrides.h"

// Tools header-files...
#include "Tools_Overlays.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Particles/Tools_Mode_PFX.h"

#include "Overrides/Tools_UI_Overrides.h"


////////////////////////////////////////////////////////////////////////////////
// Constant declarations...
////////////////////////////////////////////////////////////////////////////////

static const char*	g_szPFXOverridesFilename	= "Misc\\PFXOverrides.asr";

static const Asura_Vector_2	g_xTestMinimumCellSize(6.0f, 2.0f);
static Asura_Vector_2 g_xTableSize(80.0f, 21.0f);

////////////////////////////////////////////////////////////////////////////////
// Static variable declarations...
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Private functions...
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// Protected functions...
////////////////////////////////////////////////////////////////////////////////

class CreateStringArrayFunctor
{
	Asura_String* m_pszNameArray;
	u_int m_i;

	static bool TraverseWithUserDataFunction(Asura_Hash_ID uHash, const Asura_String* pxData, void* pUserData)
	{
		CreateStringArrayFunctor *pxThis = (CreateStringArrayFunctor *)pUserData;

		pxThis->m_pszNameArray[pxThis->m_i++] = *pxData ;

		return true;
	}
	static int CompareStringsCaseInsensitive( const void* p1, const void* p2 )
	{
		const Asura_String*szOne = reinterpret_cast<const Asura_String*>(p1);
		const Asura_String*szTwo = reinterpret_cast<const Asura_String*>(p2);
		return stricmp(szOne->GetString(), szTwo->GetString());
	}

public:
	CreateStringArrayFunctor(const Asura_TemplateHashTree<Asura_String>& xNames, Asura_String* &pszNameArray, int &iNameSize)
	{
		pszNameArray = new Asura_String[iNameSize = xNames.GetNumberOfNodes()];
		m_pszNameArray = pszNameArray;
		m_i = 0;
		xNames.Traverse(TraverseWithUserDataFunction, this);

		Asura_SortAndSearch::Quicksort(pszNameArray, iNameSize, CompareStringsCaseInsensitive);
	}
};


bool Tools_UI_Overrides_PFXSourceInterface::ActivateOverrideMap(Asura_Hash_ID uOverrideMapHash )
{
	Asura_PFX_System::GetSourceTempResourceSet().EnableOverrideMap(uOverrideMapHash, true);
	return true;
}
bool Tools_UI_Overrides_PFXSourceInterface::CreateNewOverrideMap		(const Asura_String &xOverrideMapName)
{
	Asura_PFX_SourceTemplateOverrideMap *pxMap = Asura_PFX_System::GetSourceTempResourceSet().AllocateOverrideMap(xOverrideMapName.GetHash());
	if (pxMap)
	{
		pxMap->SetName(xOverrideMapName.GetString());
		RefreshAll();
	}
	return pxMap != NULL;
}
bool Tools_UI_Overrides_PFXSourceInterface::CopyOverrideMap			(Asura_Hash_ID uFrom, const Asura_String &xTo)
{
	Asura_PFX_SourceTemplateOverrideMap *pxNew  = Asura_PFX_System::GetSourceTempResourceSet().AllocateOverrideMap(xTo.GetHash());
	Asura_PFX_SourceTemplateOverrideMap *pxOld  = Asura_PFX_System::GetSourceTempResourceSet().GetOverrideMapByHash(uFrom);

	if (pxNew && pxOld)
	{
		*pxNew = *pxOld;
		pxNew->SetName(xTo.GetString());
		RefreshAll();
		return true;
	}
	return false;
}
bool Tools_UI_Overrides_PFXSourceInterface::DeactivateOverrideMap		(Asura_Hash_ID uOverrideMapHash )
{
	Asura_PFX_System::GetSourceTempResourceSet().DisableOverrideMap(uOverrideMapHash);
	return true;
}
bool Tools_UI_Overrides_PFXSourceInterface::DeleteOverrideMap			(Asura_Hash_ID uOverrideMapHash )
{
	Asura_PFX_System::GetSourceTempResourceSet().FreeOverrideMap(uOverrideMapHash);
	RefreshAll();
	return true;
}
bool Tools_UI_Overrides_PFXSourceInterface::RenameOverrideMap			(Asura_Hash_ID uFrom, const Asura_String &xTo )
{
	return CopyOverrideMap(uFrom, xTo) && DeleteOverrideMap(uFrom);
}

bool Tools_UI_Overrides_PFXSourceInterface::IsOverrideMapActive		(Asura_Hash_ID uOverrideMapHash)
{
	return Asura_PFX_System::GetSourceTempResourceSet().IsOverrideMapEnabled(uOverrideMapHash);
}
bool Tools_UI_Overrides_PFXSourceInterface::IsItemOverriden			(Asura_Hash_ID uItemHash)
{
	Asura_PFX_SourceTemplateResourceSet_ConstMapIterator xIter(Asura_PFX_System::GetSourceTempResourceSet());

	for (;!xIter.Done(); xIter.Next())
	{
		if (xIter.GetCurrent()->GetOverride(uItemHash) != uItemHash)
			return true;
	}
	return false;
}

bool Tools_UI_Overrides_PFXSourceInterface::DoesOverrideMapExist		(Asura_Hash_ID uOverrideMapHash)
{
	return Asura_PFX_System::GetSourceTempResourceSet().GetOverrideMapByHash(uOverrideMapHash) != NULL;
}

bool Tools_UI_Overrides_PFXSourceInterface::GetEntryNames(Asura_String* &pxNameArray, int &iNumEntries)
{
	CreateStringArrayFunctor(Asura_PFX_System::GetSourceTempNames(), pxNameArray, iNumEntries);
	return true;
}
bool Tools_UI_Overrides_PFXSourceInterface::GetOverrideNames(Asura_String* &pxOverrideNameArray, int &iNumOverrides)
{
	iNumOverrides = Asura_PFX_System::GetSourceTempResourceSet().GetNumOverrideMaps();
	pxOverrideNameArray = new Asura_String[iNumOverrides];
	int i = 0;
	for (Asura_PFX_SourceTemplateResourceSet_ConstMapIterator xIter(Asura_PFX_System::GetSourceTempResourceSet()); !xIter.Done(); ++i, xIter.Next())
	{
		pxOverrideNameArray[i] = xIter.GetCurrent()->GetName();
	}
	return true;
}

Asura_Hash_ID Tools_UI_Overrides_PFXSourceInterface::GetCurrentOverrideMap()
{
	Asura_PFX_SourceTemplateResourceSet_ConstMapIterator xIter(Asura_PFX_System::GetSourceTempResourceSet());

	for (; !xIter.Done(); xIter.Next())
	{
		if (Asura_PFX_System::GetSourceTempResourceSet().IsOverrideMapEnabled(xIter.GetCurrentHash()))
		{
			return xIter.GetCurrentHash();
		}
	}

	return  ASURA_HASH_ID_UNSET;
}

bool Tools_UI_Overrides_PFXSourceInterface::GetOverrideMapEntry		(Asura_Hash_ID uOverrideMapHash, Asura_Hash_ID uEntryHash, Asura_Hash_ID &uOverriddenEntryHash)
{
	Asura_PFX_SourceTemplateOverrideMap *pxMap = Asura_PFX_System::GetSourceTempResourceSet().GetOverrideMapByHash(uOverrideMapHash);

	if (pxMap)
	{
		uOverriddenEntryHash = pxMap->GetOverride(uEntryHash);
		return true;
	}
	uOverriddenEntryHash = uEntryHash;
	return false;
}
bool Tools_UI_Overrides_PFXSourceInterface::RemoveOverrideMapEntry		(Asura_Hash_ID uOverrideMapHash, Asura_Hash_ID uEntryHash)
{
	Asura_PFX_SourceTemplateOverrideMap *pxMap = Asura_PFX_System::GetSourceTempResourceSet().GetOverrideMapByHash(uOverrideMapHash);

	if (pxMap)
	{
		pxMap->RemoveOverride(uEntryHash);
		RefreshAll();
		return true;
	}
	return false;
}

bool Tools_UI_Overrides_PFXSourceInterface::SetOverrideMapEntry	(Asura_Hash_ID uOverrideMapHash, const Asura_String &xFrom, const Asura_String &xTo)
{
	Asura_PFX_SourceTemplateOverrideMap *pxMap = Asura_PFX_System::GetSourceTempResourceSet().GetOverrideMapByHash(uOverrideMapHash);

	if (pxMap)
	{
		Asura_Hash_ID uHashFrom = xFrom.GetHash();
		Asura_Hash_ID uHashTo = xTo.GetHash();
		Asura_PFX_SourceTemplateOverrideEntry *pxEntry = pxMap->GetOverrideEntry(uHashFrom);

		if (pxEntry)
		{
			if (uHashTo != uHashFrom)
			{
				pxEntry->SetNewName(xTo);
			}
			else
			{
				pxMap->RemoveOverride(uHashFrom);
			}
		} 
		else 
		{
			if (uHashTo != uHashFrom)
			{
				pxMap->CreateOverride(xFrom, xTo);

			}

		}

		Tools_PFX_Overrides::RefreshSourceCell(uOverrideMapHash, uHashFrom);

		return true;
	}
	return false;
}

void Tools_UI_Overrides_PFXSourceInterface::RefreshAll()
{
	Tools_PFX_Overrides::RefreshAllSources();
}




bool Tools_UI_Overrides_PFXParticleInterface::ActivateOverrideMap(Asura_Hash_ID uOverrideMapHash )
{
	Asura_PFX_System::GetParticleTempResourceSet().EnableOverrideMap(uOverrideMapHash, true);
	return true;
}
bool Tools_UI_Overrides_PFXParticleInterface::CreateNewOverrideMap		(const Asura_String &xOverrideMapName)
{
	Asura_PFX_ParticleTemplateOverrideMap *pxMap = Asura_PFX_System::GetParticleTempResourceSet().AllocateOverrideMap(xOverrideMapName.GetHash());
	if (pxMap)
	{
		pxMap->SetName(xOverrideMapName.GetString());
		RefreshAll();
	}
	return pxMap != NULL;
}
bool Tools_UI_Overrides_PFXParticleInterface::CopyOverrideMap			(Asura_Hash_ID uFrom, const Asura_String &xTo)
{
	Asura_PFX_ParticleTemplateOverrideMap *pxNew  = Asura_PFX_System::GetParticleTempResourceSet().AllocateOverrideMap(xTo.GetHash());
	Asura_PFX_ParticleTemplateOverrideMap *pxOld  = Asura_PFX_System::GetParticleTempResourceSet().GetOverrideMapByHash(uFrom);

	if (pxNew && pxOld)
	{
		*pxNew = *pxOld;
		pxNew->SetName(xTo.GetString());
		RefreshAll();
		return true;
	}
	return false;
}
bool Tools_UI_Overrides_PFXParticleInterface::DeactivateOverrideMap		(Asura_Hash_ID uOverrideMapHash )
{
	Asura_PFX_System::GetParticleTempResourceSet().DisableOverrideMap(uOverrideMapHash);
	return true;
}
bool Tools_UI_Overrides_PFXParticleInterface::DeleteOverrideMap			(Asura_Hash_ID uOverrideMapHash )
{
	Asura_PFX_System::GetParticleTempResourceSet().FreeOverrideMap(uOverrideMapHash);
	RefreshAll();
	return true;
}
bool Tools_UI_Overrides_PFXParticleInterface::RenameOverrideMap			(Asura_Hash_ID uFrom, const Asura_String &xTo )
{
	return CopyOverrideMap(uFrom, xTo) && DeleteOverrideMap(uFrom);
}

bool Tools_UI_Overrides_PFXParticleInterface::IsOverrideMapActive		(Asura_Hash_ID uOverrideMapHash)
{
	return Asura_PFX_System::GetParticleTempResourceSet().IsOverrideMapEnabled(uOverrideMapHash);
}
bool Tools_UI_Overrides_PFXParticleInterface::IsItemOverriden			(Asura_Hash_ID uItemHash)
{
	Asura_PFX_ParticleTemplateResourceSet_ConstMapIterator xIter(Asura_PFX_System::GetParticleTempResourceSet());

	for (;!xIter.Done(); xIter.Next())
	{
		if (xIter.GetCurrent()->GetOverride(uItemHash) != uItemHash)
			return true;
	}
	return false;
}

bool Tools_UI_Overrides_PFXParticleInterface::DoesOverrideMapExist		(Asura_Hash_ID uOverrideMapHash)
{
	return Asura_PFX_System::GetParticleTempResourceSet().GetOverrideMapByHash(uOverrideMapHash) != NULL;
}

bool Tools_UI_Overrides_PFXParticleInterface::GetEntryNames				(Asura_String* &pxNameArray, int &iNumEntries)
{
	CreateStringArrayFunctor(Asura_PFX_System::GetParticleTempNames(), pxNameArray, iNumEntries);
	return true;
}
bool Tools_UI_Overrides_PFXParticleInterface::GetOverrideNames			(Asura_String* &pxOverrideNameArray, int &iNumOverrides)
{
	iNumOverrides = Asura_PFX_System::GetParticleTempResourceSet().GetNumOverrideMaps();
	pxOverrideNameArray = new Asura_String[iNumOverrides];
	int i = 0;
	for (Asura_PFX_ParticleTemplateResourceSet_ConstMapIterator xIter(Asura_PFX_System::GetParticleTempResourceSet()); !xIter.Done(); ++i, xIter.Next())
	{
		pxOverrideNameArray[i] = xIter.GetCurrent()->GetName();
	}
	return true;
}

Asura_Hash_ID Tools_UI_Overrides_PFXParticleInterface::GetCurrentOverrideMap()
{
	Asura_PFX_ParticleTemplateResourceSet_ConstMapIterator xIter(Asura_PFX_System::GetParticleTempResourceSet());

	for (; !xIter.Done(); xIter.Next())
	{
		if (Asura_PFX_System::GetParticleTempResourceSet().IsOverrideMapEnabled(xIter.GetCurrentHash()))
		{
			return xIter.GetCurrentHash();
		}
	}

	return  ASURA_HASH_ID_UNSET;
}

bool Tools_UI_Overrides_PFXParticleInterface::GetOverrideMapEntry		(Asura_Hash_ID uOverrideMapHash, Asura_Hash_ID uEntryHash, Asura_Hash_ID &uOverriddenEntryHash)
{
	Asura_PFX_ParticleTemplateOverrideMap *pxMap = Asura_PFX_System::GetParticleTempResourceSet().GetOverrideMapByHash(uOverrideMapHash);

	if (pxMap)
	{
		uOverriddenEntryHash = pxMap->GetOverride(uEntryHash);
		return true;
	}
	uOverriddenEntryHash = uEntryHash;
	return false;
}
bool Tools_UI_Overrides_PFXParticleInterface::RemoveOverrideMapEntry		(Asura_Hash_ID uOverrideMapHash, Asura_Hash_ID uEntryHash)
{
	Asura_PFX_ParticleTemplateOverrideMap *pxMap = Asura_PFX_System::GetParticleTempResourceSet().GetOverrideMapByHash(uOverrideMapHash);

	if (pxMap)
	{
		pxMap->RemoveOverride(uEntryHash);
		RefreshAll();
		return true;
	}
	return false;
}

bool Tools_UI_Overrides_PFXParticleInterface::SetOverrideMapEntry(Asura_Hash_ID uOverrideMapHash, const Asura_String &xFrom, const Asura_String &xTo)
{
	Asura_PFX_ParticleTemplateOverrideMap *pxMap = Asura_PFX_System::GetParticleTempResourceSet().GetOverrideMapByHash(uOverrideMapHash);

	if (pxMap)
	{
		Asura_Hash_ID uHashFrom = xFrom.GetHash();
		Asura_Hash_ID uHashTo = xTo.GetHash();
		Asura_PFX_ParticleTemplateOverrideEntry *pxEntry = pxMap->GetOverrideEntry(uHashFrom);

		if (pxEntry)
		{
			if (uHashTo != uHashFrom)
			{
				pxEntry->SetNewName(xTo);
			}
			else
			{
				pxMap->RemoveOverride(uHashFrom);
			}
		} 
		else 
		{
			if (uHashTo != uHashFrom)
			{
				pxMap->CreateOverride(xFrom, xTo);
			}
		}

		Tools_PFX_Overrides::RefreshParticleCell(uOverrideMapHash, uHashFrom);

		return true;
	}
	return false;
}

void Tools_UI_Overrides_PFXParticleInterface::RefreshAll()
{
	Tools_PFX_Overrides::RefreshAllParticles();
}






////////////////////////////////////////////////////////////////////////////////
// Class        : Tools_PFX_Overrides_Overlay
// Author       : Richard Rice
// Date         : 15/11/2007
//
// Description  : Derived overlay class for detecting if it is closed by the X button
//				  
////////////////////////////////////////////////////////////////////////////////

class Tools_PFX_Overrides_Overlay : public Asura_Overlay
{
public :
	Tools_PFX_Overrides_Overlay(Tools_PFX_Overrides* pxOverrides, const char *szName, bool bAlwaysOnTop, Asura_Vector_2* pxPosition)
		:Asura_Overlay(szName, bAlwaysOnTop, pxPosition)
		, m_pxOverrides(pxOverrides)
	{}

	~Tools_PFX_Overrides_Overlay()
	{
		m_pxOverrides->OnOverlayDeletion(this);
	}

	Tools_PFX_Overrides* m_pxOverrides;
};


Tools_PFX_Overrides* Tools_PFX_Overrides::m_pxFirst = NULL;

Tools_PFX_Overrides::Tools_PFX_Overrides()
:	m_pxOverlay( NULL ), 
	m_pxTabsDialog( NULL ),
	m_xOverlayPosition( 6.0f, 6.0f ),
	m_xSourceOverridesUI(m_xSourceOverridesUIInterface),
	m_xParticleOverridesUI(m_xParticleOverridesUIInterface)
{
	m_pxNext = m_pxFirst;
	m_pxFirst = this;
}

Tools_PFX_Overrides::~Tools_PFX_Overrides()
{
	Tools_PFX_Overrides** ppx = &m_pxFirst;
	while( *ppx != NULL )
	{
		Tools_PFX_Overrides* px = *ppx;
		if( px == this )
		{
			//remove it from the chain
			*ppx = px->m_pxNext;
			break;
		}
		ppx = &px->m_pxNext;
	}

	// Close the dialog
	Close();
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_PFX_Overrides::CreateOverlay
// Author       : Andy Tate
// Date         : 26-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_PFX_Overrides::CreateOverlay(void)
{
	if( m_pxOverlay == NULL )
	{
		Tools_Mode_PFX::Verify(true);

		Asura_Assert( ( m_pxTabsDialog == NULL ), "" );
		Asura_Vector_2	xOffset(1.0f, 3.0f);
		Asura_Dialog* pxNewDialog;

		// Store the overlay...
		m_pxOverlay		= new Tools_PFX_Overrides_Overlay(this, "PFX Overrides", false, &m_xOverlayPosition);
		m_pxOverlay->m_uID	= TOOLS_OVERLAY_PFX_OVERRIDES;

		// Only show materials with an override?

		AddDialogs( xOffset, g_xTableSize );

		// Save
		pxNewDialog = new Asura_Function_Dialog( "Save", xOffset, Save, (int)this );
		m_pxOverlay->AddDialog( pxNewDialog );
		xOffset.y += Asura_Dialog::DefaultSpacing();

		// Save and close...
		pxNewDialog = new Asura_Function_Dialog("Save + Close", xOffset, SaveAndClose, (int)this );
		m_pxOverlay->AddDialog( pxNewDialog );
		xOffset.y		+= Asura_Dialog::DefaultSpacing();

//		pxNewDialog = new Asura_Resize_Dialog(m_pxSourceTableDialog, &g_xTableSize);
//		AddDialog(pxNewDialog);

		// Finish off...
//		m_pxOverlay->MakeDialogsRelativeToResizableDialog();
		Asura_Overlay_System::SubmitNewOverlay(m_pxOverlay);
	}
}


void Tools_PFX_Overrides::AddDialogs( Asura_Vector_2& xOffset, const Asura_Vector_2& xTableSize )
{

	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset, 0, 0);
	m_pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	m_pxOverlay->SetCurrentParentDialog(pxTabsDialog);

	pxTabsDialog->AddTab("Source Templates");
	pxTabsDialog->SelectTab(0);
	Asura_Vector_2 xTabOffset = xOffset;
	m_xSourceOverridesUI.AddDialogs(m_pxOverlay, xTabOffset, xTableSize);
	float fMaxY = Asura_Maths::Max(fMaxY, xTabOffset.y);
	pxTabsDialog->AddTab("Particle Templates");
	pxTabsDialog->SelectTab(1);

	xTabOffset = xOffset;
	m_xParticleOverridesUI.AddDialogs(m_pxOverlay, xTabOffset, xTableSize);
	fMaxY = Asura_Maths::Max(fMaxY, xTabOffset.y);
	xOffset.y = fMaxY;

	m_xSourceOverridesUI.Refresh();
	m_xParticleOverridesUI.Refresh();


	pxTabsDialog->SelectTab(0);


	m_pxOverlay->SetCurrentParentDialog(0);
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_PFX_Overrides::SaveAndClose
// Author       : Andy Tate
// Date         : 26-Apr-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_PFX_Overrides::SaveAndClose( void )
{
	// Save the file
	Save();
	// Close the dialog
	Close();
}

void Tools_PFX_Overrides::Save( void )
{
	// Save the file to disc...
	if( Tools_Chunk_PFXOverrides::Save( g_szPFXOverridesFilename ) )
	{
		Asura_String	xMessage("Saved \"");

		xMessage	+= g_szPFXOverridesFilename;
		xMessage	+= "\" successfully.";

		Tools_Timed_Message_System::AddMessage( 0x00FF00, xMessage.GetString() );
	}
	else
	{
		Asura_String	xMessage( "Failed to save \"");

		xMessage	+= g_szPFXOverridesFilename;
		xMessage	+= "\" - write-protected?";

		Tools_Timed_Message_System::AddMessage( 0xFF0000, xMessage.GetString() );
	}
}

void Tools_PFX_Overrides::Close( void )
{

	// Destroy our existing table...
	m_xSourceOverridesUI.DestroyTable();
	m_xParticleOverridesUI.DestroyTable();

	// Close the window...
	if( m_pxOverlay != NULL )
	{
		m_pxOverlay->m_bShouldClose = true;
	}

	// Flag as closed...
	m_pxOverlay							= NULL;
}

void Tools_PFX_Overrides::OnOverlayDeletion(Asura_Overlay* pxOverlay)
{
	//If the system still has a pointer to the overlay , then it hasn't cleared things up yet
	//and so needs to do SaveAndClose() to save and deallocate everything.
	//RAR 15/11/2007

	if(m_pxOverlay)
	{
		Asura_Assert(pxOverlay == m_pxOverlay, "Not deleting the expected overlay");
		SaveAndClose();
	}

	m_xSourceOverridesUI.RemoveDialogs();
	m_xParticleOverridesUI.RemoveDialogs();
}

/// End of function 'Tools_PFX_Overrides::SaveAndClose'. ///



////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_PFX_Overrides::RefreshAll
// Author       : Andy Tate
// Date         : 3-May-2007
////////////////////////////////////////////////////////////////////////////////

void Tools_PFX_Overrides::RefreshAllSources( void )
{
	Tools_PFX_Overrides* px = m_pxFirst;
	while( px != NULL )
	{
		px->m_xSourceOverridesUI.DestroyTable();
		px->m_xSourceOverridesUI.CreateTable();
		px = px->m_pxNext;
	}
}

void Tools_PFX_Overrides::RefreshAllParticles( void )
{
	Tools_PFX_Overrides* px = m_pxFirst;
	while( px != NULL )
	{
		px->m_xParticleOverridesUI.DestroyTable();
		px->m_xParticleOverridesUI.CreateTable();
		px = px->m_pxNext;
	}
}


void Tools_PFX_Overrides::RefreshSourceCell( Asura_Hash_ID uOverrideMapHash, Asura_Hash_ID uSourceHash  )
{
 	for( Tools_PFX_Overrides* px = m_pxFirst; px != NULL;  px = px->m_pxNext)
 	{
		px->m_xSourceOverridesUI.RefreshCell(uOverrideMapHash, uSourceHash);
 	}
}


void Tools_PFX_Overrides::RefreshParticleCell( Asura_Hash_ID uOverrideMapHash, Asura_Hash_ID uSourceHash  )
{
	for( Tools_PFX_Overrides* px = m_pxFirst; px != NULL;  px = px->m_pxNext)
	{
		px->m_xParticleOverridesUI.RefreshCell(uOverrideMapHash, uSourceHash);
	}
}


/// End of function 'Tools_PFX_Overrides::RefreshAll'. ///

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_PFX_Overrides::CreateTable
// Author       : Andy Tate
// Date         : 1-May-2007
////////////////////////////////////////////////////////////////////////////////



void Tools_PFX_Overrides::Initialise(void)
{
	LoadAsuraFile( g_szPFXOverridesFilename );

}

/// End of function 'Tools_PFX_Overrides::Initialise'. ///




void Tools_PFX_Overrides::Shutdown(void)
{
	if ( IsOpen() )
	{
		ToggleOverlay();
	}
}


void Tools_PFX_Overrides::ToggleOverlay(void)
{
	if(IsOpen())
	{
		SaveAndClose();
	}
	else
	{
		CreateOverlay();
	}
}

