//
// Tools_Mode_RagdollEditor.cpp
//
// Created by KJFL - 08 October 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
 
#include "Ragdoll/Tools_Mode_RagdollEditor.h"

// Asura 
#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "ChunkSystem/Asura_Chunk_RagdollDescription.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_UserInput.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Light/Asura_GI_Solution.h"
#include "Physics/Asura_Physics_Ragdoll.h"
#include "Physics/Asura_Physics_Ragdoll_Description.h"
#include "Rendering/Asura_Render.h"

// Tools
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save//Tools_AsuraFileContents.h"
#include "Misc/Tools_Render_Options.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Movement/Tools_Movement.h"
#include "Objects/Tools_Object_System.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Lighting/Tools_Photon_Object.h"
#include "Ragdoll/Tools_HierarchyNode_Object.h"
#include "Ragdoll/Tools_RagdollCollision_Object.h"
#include "Ragdoll/Tools_RagdollJoint_Object.h"


// Asura PC specific 
// These PC includes are required to force regeneration of skin strips when entering this mode.
#include "Rendering/Asura_Platform_RenderObjectHierarchy.h"
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
#include "Rendering/Asura_PlatformRenderCharacter.h"
#endif
#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
#include "Rendering/Asura_Direct3D_SmoothSkinRenderer.h"
#endif

Tools_Mode_RagdollEditor*	Tools_Mode_RagdollEditor::s_pxThis = NULL;
Asura_Hierarchy_Skin*		Tools_Mode_RagdollEditor::s_pxLastUsedSkin = NULL;
Asura_Physics_Ragdoll_Description* Tools_Mode_RagdollEditor::s_pxLastUsedRagdollDesc = NULL;

// TODO - haven't quite decided if these are global, static or what
static Asura_Vector_3 axBonePosition[ASURA_MAX_NO_OF_BONES_PER_MODEL];
static Asura_Matrix_3x3 axBoneOrientation[ASURA_MAX_NO_OF_BONES_PER_MODEL];

static const char* g_szNoneString = "*None*";

void Tools_Mode_RagdollEditor::InitialiseMode(void)
{
	m_xModeOverlayPosition = Asura_Vector_2(3.0f,8.0f);

	// We need to hide the environment, and draw the ragdoll
	// whatever the visibility system say.
    Asura_Hierarchy_System::SetNoCullingFlag( true );
	Tools_Render_Options::HideEnvironment = true;

	
	// Store current camera position and orientation for when we leave this mode
	m_xPreModeCameraPosition = Tools_Movement::GetCameraObject()->Position;
	m_xPreModeCameraOrientation = Tools_Movement::GetCameraObject()->Orientation;
	
 	// Teleport to near the origin
	{	
		// Step back a little so a human-sized character fits nicely
		// TODO - may want to rearrange camera automatically based on size of skin used?
 		Asura_Vector_3 xPosition(0.0f,-1.0f,1.5f);
	
		// Face along -z axis - usually this means characters rendered with identity
		// orientation will be facing us.
		Asura_Matrix_3x3 xOrientation;
 		xOrientation.Identity();
		xOrientation.Element[2][2] = -1.0f;
		xOrientation.Element[0][0] = -1.0f;

		Tools_Movement::TeleportCameraToPosition(&xPosition,&xOrientation);
	}

	// Enable object system controls (selecting nodes etc)
	Tools_Object_System::EnableAllControls(true);

	m_pxSkin = 0;
	m_pxAnim = 0;

	m_ppxHierarchyNodes = 0;
	m_uNumberOfNodes = 0;

	m_ppxRagdollCollisionObjects = 0;
	m_uNumberOfRagdollCollisionObjects = 0;

	m_pxRagdollDesc = 0;

	m_bHideSkin = true;
	m_bHideSkeleton = false;
	m_bHideNodes = false;
	m_bHideCollisionObjects = false;
	m_bHideJoints = false;
	m_bHideLights = false;

//	new Tools_Photon_Object( &Asura_Maths::ZeroVector3, &Asura_Maths::IdentityMatrix3x3 );




#if 0
	// Create some default lights
	
	Asura_Vector_3 axLightPos[ NUM_TEMP_LIGHTS ] =
	{
		Asura_Vector_3(0.0f,-1.0f,-3.0f),
		Asura_Vector_3(2.6f,-1.0f,1.5f),
		Asura_Vector_3(-2.6f,-1.0f,1.5f)
	};

	for( u_int u = 0; u < NUM_TEMP_LIGHTS; ++u )
	{
		Tools_Object* const pxNewLight = new Tools_Photon_Object( &axLightPos[u], &Asura_Maths::IdentityMatrix3x3 );
		if( pxNewLight )
		{
			m_aiTempLightToolsIDs[u] = pxNewLight->GetToolsID();
		}
	}

	{
		// DLH - 15 March 2007 12:52:30 - Find all the Active Photon, turen them off and note them so they can be turned back on later.
		Asura_TrackingHandlePool<Asura_Photon>* pxPhotonList = Asura_Photon_System::GetDynamicPhotonList();
		Asura_TrackingHandlePool<Asura_Photon>::Iterator xIterator = pxPhotonList->GetIterator();
		
		this->m_xPhotonsActiveAtStart.empty();

		for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
		{
			Asura_Photon* pxPhoton = xIterator.GetCurrentPointer();
			
			if( false == pxPhoton->IsActive())
			{
				continue;
			}
			
			pxPhoton->SetAsActive( false);
			this->m_xPhotonsActiveAtStart.add_entry( pxPhoton);
		}
	}

#endif

	// Prevent the environment lighting from affecting th models in this mode
	Asura_GI_Solution::DisableStaticLighting(true);

	// If the Asura file loaded is a skin/hierarchy, the force regeneration of skin/hierarchy
	// strips when entering this mode, if we might have modified the mesh/texturing/materials
	// in a different mode (i.e. if we're editing a skin)
	if( Tools_SaveAsura::GetFileType() == AsuraFile_Skin )
	{
		#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
		Asura_Direct3D_CharacterRenderer::GetCurrentHandler().Reset();
		#endif
		#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
		Asura_Direct3D_SmoothSkinRenderer::Reset();
		#endif
		Asura_Direct3D_ObjectHierarchyRenderer::GetCurrentHandler().Reset();
	}

	// We need to create the mode overlay before setting up the skin from any defaults,
	// as the SetSkin method will try to change the appropriate overlay dialog.
	CreateModeOverlay();
	
	// If we have a skin available from last time, use that.
	if( s_pxLastUsedSkin && s_pxLastUsedRagdollDesc )
	{
		const char* szFilename = Asura_HashedFilename_System::GetFullFilename(Asura_GetHashID(s_pxLastUsedSkin->Name.GetString()));
		if(szFilename && Tools_AsuraFileContents::GetAsuraFileType(szFilename) == AsuraFile_Skin)
		{
			SetSkinAndRagdoll(s_pxLastUsedSkin->Name.GetString(),szFilename, s_pxLastUsedRagdollDesc );
		}
	}

	// Otherwise if the Asura file loaded is a skin/hierarchy, 
	// select that file as default to streamline editing.
	else if ( Tools_SaveAsura::GetFileType() == AsuraFile_Skin  )
	{
		Asura_String xFileName = Tools_SaveAsura::GetMainFilename();
		Asura_String xSkinName;
		xFileName.SplitFilename(0,&xSkinName,0);

		SetSkinAndRagdoll( xSkinName.GetString(),xFileName.GetString() );
	}

}
void Tools_Mode_RagdollEditor::ShutdownMode(void)
{
	// Revert our rendering options back to normal.
    Asura_Hierarchy_System::SetNoCullingFlag( false );
	Tools_Render_Options::HideEnvironment = false;
	
	// Restore the camera position
	Tools_Movement::TeleportCameraToPosition(&m_xPreModeCameraPosition,&m_xPreModeCameraOrientation);

	// Re-enable lighting
	Asura_GI_Solution::DisableStaticLighting(false);

	// Close overlays
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODE);

	DeleteObjects();
}

void Tools_Mode_RagdollEditor::CreateModeOverlay( void )
{
	Asura_Overlay* pxOverlay = new Asura_Overlay("Ragdoll Editor",false,&m_xModeOverlayPosition);
	pxOverlay->m_uID = TOOLS_OVERLAY_MODE;

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset(0.5f,1.5f);


	// Add the main subwindow control for containing all the other controls
	float fSubwindowWidth = 50.0f;
	float fSubwindowHeight = 30.0f;
	Asura_Subwindow_Dialog* pxMainDialog = new Asura_Subwindow_Dialog(xOffset,fSubwindowWidth,fSubwindowHeight,true);
	pxOverlay->AddDialog(pxMainDialog);
	pxMainDialog->SetOutlineHidden(true);

	// Add control for resizing the main dialog
	pxDialog = new Asura_Resize_Dialog(pxMainDialog, 0);
	pxOverlay->AddDialog(pxDialog);
	
	// Add skin/hierarchy selection buttons
	{
		Asura_Subwindow_Dialog* pxSubwindowDialog = new Asura_Subwindow_Dialog(xOffset,fSubwindowWidth,fSubwindowHeight,true);
		pxMainDialog->AddDialog(pxSubwindowDialog);
		pxSubwindowDialog->UpdateLabel("Skin & Ragdoll");

		Asura_Vector_2 xLocalOffset(1.0f,1.0f);
	
		pxDialog = new Asura_Function_Dialog( "Select Skin & Ragdoll", xLocalOffset, ChooseSkin );
		pxDialog->SetHelpText("Select a skin/hierarchy, and a ragdoll which has a matching number of bones. The tooltips for the names below are the full filenames.");
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_StringDisplay_Dialog( "Skin: *None*", xLocalOffset);
		pxDialog->SetControlID( DialogID_Skin );
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_StringDisplay_Dialog( "Ragdoll: *None*", xLocalOffset);
		pxDialog->SetControlID( DialogID_Ragdoll );
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing();

		xOffset.y+=xLocalOffset.y+Asura_Dialog::LargeSpacing()*1;
	}
		
	// Add rendering options buttons
	{
		Asura_Subwindow_Dialog* pxSubwindowDialog = new Asura_Subwindow_Dialog(xOffset,fSubwindowWidth,fSubwindowHeight,true);
		pxMainDialog->AddDialog(pxSubwindowDialog);
		pxSubwindowDialog->UpdateLabel("Render Options");
	
		Asura_Vector_2 xLocalOffset(1.0f,1.0f);
#if 0
		// little experiment with some buttons
		Asura_Vector_2 xButtonSize(32.0f,32.0f);

		pxDialog = new Asura_Button_Dialog( xLocalOffset, -1, 256, Asura_Maths::ZeroVector2, xButtonSize, ToggleRenderSkin );
		pxSubwindowDialog->AddDialog(pxDialog);

		xLocalOffset.x+=4;

		pxDialog = new Asura_Button_Dialog( xLocalOffset, -1, 256, Asura_Maths::ZeroVector2, xButtonSize, ToggleRenderSkin );
		pxSubwindowDialog->AddDialog(pxDialog);

		xLocalOffset.x+=4;

		pxDialog = new Asura_Button_Dialog( xLocalOffset, -1, 256, Asura_Maths::ZeroVector2, xButtonSize, ToggleRenderSkin );
		pxSubwindowDialog->AddDialog(pxDialog);

		xLocalOffset.x+=4;

		pxDialog = new Asura_Button_Dialog( xLocalOffset, -1, 256, Asura_Maths::ZeroVector2, xButtonSize, ToggleRenderSkin );
		pxSubwindowDialog->AddDialog(pxDialog);
#endif

		pxDialog = new Asura_Tickbox_Dialog("Hide Skin Mesh (m)",xLocalOffset,&m_bHideSkin );
		pxDialog->SetHelpText("Untick to draw the character mesh.");
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Hide Hierarchy Skeleton",xLocalOffset,&m_bHideSkeleton);
		pxDialog->SetHelpText("Untick to draw the skeleton as lines between nodes.");
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Hide Hierarchy Nodes", xLocalOffset,&m_bHideNodes);
		pxDialog->SetHelpText("If unticked spheres are drawn around the nodes in the hierarchy.");
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Hide CollisionObjects",xLocalOffset,&m_bHideCollisionObjects);
		pxDialog->SetHelpText("Hide the ragdoll Collision Objects.");
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Hide Joints",xLocalOffset,&m_bHideJoints);
		pxDialog->SetHelpText("Hide the ragdoll Joints.");
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Hide Tools Lights",xLocalOffset,&m_bHideLights);
		pxDialog->SetHelpText("Hide the temporary light objects.");
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing();
/*
		pxDialog = new Asura_Tickbox_Dialog("Show Section Bounding Boxes (b)",xLocalOffset,&m_bHideSkin);
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing();
*/
		xOffset.y+=xLocalOffset.y+Asura_Dialog::LargeSpacing()*1;
	}

	// Add collision shape creation buttons
	{
		Asura_Subwindow_Dialog* pxSubwindowDialog = new Asura_Subwindow_Dialog(xOffset,fSubwindowWidth,fSubwindowHeight,true);
		pxMainDialog->AddDialog(pxSubwindowDialog);
		pxSubwindowDialog->UpdateLabel("Collisions & Joints");

		Asura_Vector_2 xLocalOffset(1.0f,1.0f);
	
		pxDialog = new Asura_Function_Dialog( "Create Sphere", xLocalOffset, CreateSphere );
		pxDialog->SetHelpText("Create a sphere at currently selected node.");
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing() * 2;

		pxDialog = new Asura_Function_Dialog( "Create Capsule (c)", xLocalOffset, CreateCapsule );
		pxDialog->SetHelpText("Create a capsule between currently selected nodes.");
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing() * 2;

		pxDialog = new Asura_Function_Dialog( "Create Joint (j)", xLocalOffset, CreateJoint );
		pxDialog->SetHelpText("Create a joint at currently selected node.");
		pxSubwindowDialog->AddDialog(pxDialog);
		xLocalOffset.y+=Asura_Dialog::DefaultSpacing() * 2;

		xOffset.y+=xLocalOffset.y+Asura_Dialog::LargeSpacing()*1;
	}

	// Finish off...
	//pxOverlay->MakeDialogsRelativeToResizableDialog();
	//pxOverlay->SetResizeUsingDialogBorder(true);

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_Mode_RagdollEditor::Update(void)
{
	if ( Asura_UserInput::DebouncedKeyPressed( 'm' ) ) 
	{
		m_bHideSkin = !m_bHideSkin;
	}

	if ( Asura_UserInput::DebouncedKeyPressed( 'c' ) ) 
	{
		CreateCapsule();
	}
	
	if ( Asura_UserInput::DebouncedKeyPressed( 'j' ) ) 
	{
		CreateJoint();
	}

	Tools_Object_System::SetTypeHidden( TOT_ToolsPhoton, ObjectHideFlag_Mode, m_bHideLights );
	Tools_Object_System::SetTypeHidden( TOT_HierarchyNode, ObjectHideFlag_Mode, m_bHideNodes );
	Tools_Object_System::SetTypeHidden( TOT_RagdollCollision, ObjectHideFlag_Mode, m_bHideCollisionObjects );
	Tools_Object_System::SetTypeHidden( TOT_RagdollJoint, ObjectHideFlag_Mode, m_bHideJoints );
}

void Tools_Mode_RagdollEditor::RenderWorld_PostToneMap(void)
{
	if (!m_pxSkin) return;

	if (!m_pxAnim) return;
	
	// TODO - this will most likely need changing as ragdoll moves?
	Asura_Vector_3 xPivotPosition; xPivotPosition.Zero();
	Asura_Matrix_3x3 xPivotOrientation; xPivotOrientation.Identity();

	if (m_pxAnim )
	{
		// we always want to update the bone positions, even if we're not going to draw the mesh
	//	m_pxAnim->Update( xPivotPosition, xPivotOrientation, 0.0f, ASURA_HIERARCHY_RENDER_DONTRENDER | ASURA_HIERARCHY_RENDER_UPDATEBONESALWAYS );
	
		// draw the mesh if required
		if (!m_bHideSkin)
		{
			m_pxAnim->Render( xPivotPosition, xPivotOrientation, 0 );
		}
		
		// extract the bone positions and orientations


		// Get the tween storage, so we can get the currrent location for the bones. If the character is not
		// on screen this will probably be set to the previous pose when the character was rendered.
		Asura_Hierarchy_Tween* pxTweenStorage = m_pxAnim->GetTweenStore();
		Asura_Assert( pxTweenStorage, "Need tween storage to get marker point positions" );
		if ( pxTweenStorage->IsPositionDataValid() )  
		{
		#if 0
			for( int i = 0; i < m_pxSkin->NumberOfBones; ++i )
			{
				Asura_Matrix_3x3	xBoneOri;
				Asura_Vector_3		xBonePos;

				pxTweenStorage->GetBoneData(i).GetOrientation(xBoneOri);
				pxTweenStorage->GetBoneData(i).GetPosition(xBonePos);
			
				// If ragdoll is active then all bone positions & orientations are world relative
				if ( pxTweenStorage->IsPositionDataInWorldspace() )
				{
					axBonePosition[i] = xBonePos;
					axBoneOrientation[i] = xBoneOri;
				}
				else
				{
					// Compute the worldspace positions by hand, cascading the transformations bone to bone
					if(i)
					{
						int iParentIndex = m_pxSkin->ParentBone[i];
						Asura_Assert( iParentIndex>=0 && iParentIndex<i, "Error in hierarchy - bone has parent out of expected index range" );

						axBonePosition[i] = xBonePos * axBoneOrientation[iParentIndex]+axBonePosition[iParentIndex];
						axBoneOrientation[i] =  axBoneOrientation[iParentIndex] * xBoneOri;
					}
					else
					{
						axBonePosition[0] = xBonePos * xPivotOrientation + xPivotPosition;
						axBoneOrientation[0] = xPivotOrientation * xBoneOri;
					}
				}

			}
		
			// update node object positions
			for( int i = 0; i < m_pxSkin->NumberOfBones; i++ )
			{
				m_ppxHierarchyNodes[i]->UpdatePosition( axBonePosition[i], axBoneOrientation[i] );
			}

			// update collision object positions
			{
				Asura_List_It<Tools_RagdollCollision_Object> xIt( Tools_RagdollCollision_Object::GetList() );

				for ( ; !xIt.Done() ; xIt.Next() )
				{
					Tools_RagdollCollision_Object* pxToolsObject = xIt.GetCurrent();

					Asura_Physics_Ragdoll_Desc_CollisionObject* pxCollisionObject = pxToolsObject->GetCollisionObject();

					u_int uNode = pxCollisionObject->GetHierarchyNode();

					pxToolsObject->UpdatePosition( axBonePosition[uNode], axBoneOrientation[uNode] );
				}
			}

			// update joint positions
			{
				Asura_List_It<Tools_RagdollJoint_Object> xIt( Tools_RagdollJoint_Object::GetList() );

				for ( ; !xIt.Done() ; xIt.Next() )
				{
					Tools_RagdollJoint_Object* pxToolsObject = xIt.GetCurrent();

					Asura_Physics_Ragdoll_Desc_Joint* pxJoint = pxToolsObject->GetJoint();

					u_int uNode = pxJoint->GetHierarchyNode();

					pxToolsObject->UpdatePosition( axBonePosition[uNode], axBoneOrientation[uNode] );
				}
			}
#endif
			UpdateNodeLinks();

			// render the skeleton if required
			if ( !m_bHideSkeleton )
			{
				// Purple for skeleton
				const u_int uSkeletonColour = 0xFF9030FF;
				
				for (int iBoneID = 1; iBoneID < m_pxSkin->NumberOfBones; iBoneID++)
				{
					const int iParentIndex = m_pxSkin->ParentBone[iBoneID];
					
					// Draw a line from parent to this bone
					Asura_Render::RenderLine( axBonePosition[iParentIndex], axBonePosition[iBoneID], uSkeletonColour );
				}				
			}
		}
	}
	

}

void Tools_Mode_RagdollEditor::RenderUI(void)
{

}

void Tools_Mode_RagdollEditor::UpdateNodeLinks( void )
{
	// TODO - preallocate once per model?
	u_int* puNodes = new u_int[m_pxSkin->NumberOfBones];

	// Start with all nodes unassociated
	for( int i = 0; i < m_pxSkin->NumberOfBones; i++ )
	{
		puNodes[i] = Asura_Physics_Ragdoll_Description::uINVALID_INDEX;

	}

	// Find the nodes which are definitely associated with collision objects
	{
		Asura_List_It<Tools_RagdollCollision_Object> xIt( Tools_RagdollCollision_Object::GetList() );

		for ( ; !xIt.Done() ; xIt.Next() )
		{
			Tools_RagdollCollision_Object* pxToolsObject = xIt.GetCurrent();

			Asura_Physics_Ragdoll_Desc_CollisionObject* pxCollisionObject = pxToolsObject->GetCollisionObject();

			u_int uNode = pxCollisionObject->GetHierarchyNode();
			
			puNodes[uNode] = pxToolsObject->GetCollisionObjectIndex();
		}
	}

	// Now scan down the hierarchy and set any unassociated nodes to match their parent
	for( int i = 0; i < m_pxSkin->NumberOfBones; i++ )
	{
		if ( puNodes[i] == Asura_Physics_Ragdoll_Description::uINVALID_INDEX )
		{
			int iParent = m_ppxHierarchyNodes[i]->GetParentBoneIndex();

			if ( iParent>=0 )
			{
				puNodes[i] = puNodes[iParent];
			}
		}
	}

	// Set the node data - overrides are handled internally
	for( int i = 0; i < m_pxSkin->NumberOfBones; i++ )
	{
		u_int uCollisionObject = puNodes[i];

		m_ppxHierarchyNodes[i]->SetCollisionObject( uCollisionObject );
	}

	delete [] puNodes;
}

// called when the Tools object is being deleted (e.g. user selects object and presses delete)
void Tools_Mode_RagdollEditor::DeleteCollisionObjectCallback( Tools_RagdollCollision_Object* pxObject )
{
	if (!s_pxThis->m_pxRagdollDesc) return;

	// remove the object from the ragdoll desc, and fix up any pointers as a result
	s_pxThis->m_pxRagdollDesc->Tools_RemoveCollisionObject( pxObject->GetCollisionObject() );
}
void Tools_Mode_RagdollEditor::DeleteJointObjectCallback( Tools_RagdollJoint_Object* pxObject )
{
	if (!s_pxThis->m_pxRagdollDesc) return;

	// remove the object from the ragdoll desc, and fix up any pointers as a result
	s_pxThis->m_pxRagdollDesc->Tools_RemoveJoint( pxObject->GetJoint() );
}

const char* Tools_Mode_RagdollEditor::GetNameOfCollisionObject( u_int uIndex )
{
	// KJFL - 20 November 2008 - there's a laughable number of hoops to jump
	// through to get the name of a collision object from it's index, simply
	// because they're not actually stored in the in-game data, but are 
	// auto-generated when the associated Tools objects are created.
	// We may want to just store a list of name pointers which we update as
	// collision objects get made, but we're only likely to be calling this
	// irregularly (e.g. when clicking to add collision objects to joints),
	// so perhaps it's not worth it.
	if ( uIndex != Asura_Physics_Ragdoll_Description::uINVALID_INDEX
		&& s_pxThis && s_pxThis->m_pxRagdollDesc )
	{
		const Asura_Physics_Ragdoll_Desc_CollisionObject* pxColObj = s_pxThis->m_pxRagdollDesc->Tools_GetCollisionObject( uIndex );

		if (pxColObj)
		{
			Asura_List_It<Tools_RagdollCollision_Object> xIt( Tools_RagdollCollision_Object::GetList() );

			for ( ; !xIt.Done() ; xIt.Next() )
			{
				Tools_RagdollCollision_Object* pxToolsObject = xIt.GetCurrent();

				if ( pxToolsObject->GetCollisionObject() == pxColObj )
				{
					return pxToolsObject->GetName();
				}
			}
		}
	}

	return g_szNoneString;
}


const Tools_RagdollCollision_Object* Tools_Mode_RagdollEditor::GetToolsCollisionObject( u_int uIndex )
{
	if ( uIndex != Asura_Physics_Ragdoll_Description::uINVALID_INDEX && s_pxThis && s_pxThis->m_pxRagdollDesc )
	{
		const Asura_Physics_Ragdoll_Desc_CollisionObject* pxColObj = s_pxThis->m_pxRagdollDesc->Tools_GetCollisionObject( uIndex );

		if (pxColObj)
		{
			Asura_List_It<Tools_RagdollCollision_Object> xIt( Tools_RagdollCollision_Object::GetList() );

			for ( ; !xIt.Done() ; xIt.Next() )
			{
				Tools_RagdollCollision_Object* pxToolsObject = xIt.GetCurrent();

				if ( pxToolsObject->GetCollisionObject() == pxColObj )
				{
					return pxToolsObject;
				}
			}
		}
	}

	return 0;
}

void Tools_Mode_RagdollEditor::ChooseSkin( void )
{
	Asura_String xSkinName,xSkinFilename;
	/*
	if (m_pxRagdollDescription)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Choosing a skin now will deselect the Ragdoll description being edited; remember to save it if necessary!!");
	}
*/
	Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::ChooseSkin( &xSkinName, &xSkinFilename );
	if( !pxSkin ) return;

	s_pxThis->SetSkinAndRagdoll( xSkinName.GetString(), xSkinFilename.GetString() );
}

bool Tools_Mode_RagdollEditor::SelectRagdoll( void )
{
	ListSelection_Overlay xOverlay( "Select Ragdoll Description" );
	const char* szNewRagdollString = "[Create New Description]";
	xOverlay.AddItem(szNewRagdollString);

	bool bSelectNewName = false;
	const u_int uNumDescriptions = Asura_Physics_Ragdoll_System::GetNumRagdollDescriptions( m_pxSkin->NumberOfBones );
	if ( uNumDescriptions == 0 )
	{
		bSelectNewName = true;
	}

	const char** pszRagdollDescriptionNames = new const char*[uNumDescriptions];
	Asura_Physics_Ragdoll_System::GetRagdollDescriptionNames( pszRagdollDescriptionNames, m_pxSkin->NumberOfBones );
	for ( u_int u = 0; u < uNumDescriptions; u++ )
	{
		const char* szRagdollDescriptionName = pszRagdollDescriptionNames[u];
		xOverlay.AddItem( szRagdollDescriptionName, Asura_GetHashID( szRagdollDescriptionName ) );
	}			

	// quit if user pressed escape
	if ( !xOverlay.DoModal() ) 
	{ 
		delete[] pszRagdollDescriptionNames;
		return false;
	}

	if (bSelectNewName || strcmp(xOverlay.GetSelectionString(),szNewRagdollString)==0)
	{
		const char* szSuggestedDefault = m_pxSkin->GetName();

		GetString_Overlay xOverlay( "New Ragdoll Description - please enter a unique name", szSuggestedDefault );

		while (true)
		{
			// quit if user pressed escape
			if(!xOverlay.DoModal()) 
			{
				delete[] pszRagdollDescriptionNames;
				return false;
			}

			Asura_String xName = xOverlay.GetString();

			Asura_Physics_Ragdoll_Description* pxRagdollDescription = Asura_Physics_Ragdoll_System::CreateNewRagdollDescription( xName );
			if (pxRagdollDescription)
			{
				Tools_Timed_Message_System::AddMessage(0x00ff00, "New Ragdoll created.");

				pxRagdollDescription->Tools_Initialise( m_pxSkin->NumberOfBones );
				m_pxRagdollDesc = pxRagdollDescription;
				break;
			}
			else
			{
				Tools_Timed_Message_System::AddMessage(0xff0000, "Not created - the name '%s' is already in use.", xName);
			}
		}
	}
	else
	{	
		m_pxRagdollDesc = Asura_Physics_Ragdoll_System::GetRagdollDescription( (xOverlay.GetSelectionString()) );
	}
	
	delete[] pszRagdollDescriptionNames;

	return true;
}


void Tools_Mode_RagdollEditor::SaveRagdoll( void )
{
	// Check for modified status? Current we just save...
	if (!m_pxRagdollDesc) return;

	Tools_FileSystem::EnsureDirectoryExists( "Ragdolls" );

	Asura_String xFilename = "Ragdolls/";
	xFilename += m_pxRagdollDesc->GetName();
	xFilename += ".ragdoll";

	Tools_Timed_Message_System::AddMessage(0xffff00, "Saving Ragdoll to %s", xFilename );

	Asura_SaveBlock xSaveBlock;
	Asura_Chunk_SaveBlock_Stream xStream(&xSaveBlock);
	Asura_Chunk_RagdollDescription::Create(xStream, *m_pxRagdollDesc);
	xSaveBlock.AddAsuraTerminator();

	if (xSaveBlock.SaveFile(xFilename.GetString(), false))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00, "Saved %s successfully.", xFilename );
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000, "Unable to save Ragdoll (Write-protected file?)");
	}
}


bool Tools_Mode_RagdollEditor::SetSkinAndRagdoll( const char* szSkinName, const char* szSkinFilename, Asura_Physics_Ragdoll_Description* pxDesc )
{
	//see if skin is already loaded
	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(szSkinName);
	if(!pxSkin)
	{
		//try loading it then
		LoadAsuraFile(szSkinFilename);
		pxSkin = Asura_Animation_System::GetSkin(szSkinName);
	}
	if(!pxSkin) return false;

	//SkinFilename = szSkinFilename;

	//make sure we have the Asura_Meshes for the selected skin
	Tools_Animation_Files::LoadMeshForSkin(pxSkin);

	m_pxSkin = pxSkin;
	s_pxLastUsedSkin = pxSkin;

	DeleteObjects();

	// no we have the skin setup, we must select the ragdoll to use
	bool bSuccess;
	
	if ( pxDesc )
	{
		m_pxRagdollDesc = pxDesc;
		bSuccess = true;
	}
	else
	{
		bSuccess = SelectRagdoll();
	}
	

	if ( bSuccess)
	{
		s_pxLastUsedRagdollDesc = m_pxRagdollDesc;

		// create all the tools objects for this skin / ragdoll
		CreateObjects();
	}
	else
	{
		// user tried pressing escape at some point
		m_pxSkin = 0;
		s_pxLastUsedSkin = 0;
		s_pxLastUsedRagdollDesc = 0;
	}

	// update the overlay with our new names
	Asura_Overlay* pxModeOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_MODE);
	if(pxModeOverlay)
	{
		// Find the control to be changed
		Asura_Dialog* pxDialog = pxModeOverlay->GetDialog(DialogID_Skin);

		if (pxDialog)
		{
			Asura_String xNewLabel("Skin: ");
			xNewLabel += bSuccess ? szSkinName : g_szNoneString;  

			pxDialog->UpdateLabel( xNewLabel.GetString() );
			pxDialog->SetHelpText( szSkinFilename );
		}

		pxDialog = pxModeOverlay->GetDialog(DialogID_Ragdoll);

		if (pxDialog)
		{
			Asura_String xNewLabel("Ragdoll: ");
			xNewLabel += bSuccess ? m_pxRagdollDesc->GetName() : g_szNoneString; 

			pxDialog->UpdateLabel( xNewLabel.GetString() );
			pxDialog->SetHelpText( "File is in Ragdolls/ folder" );
		}
	}

	//set the default directory name
//	DirectoryName = Tools_Animation_Files::GetDefaultAnimDirForSkin(SkinName.GetString());

	return true;
}

void Tools_Mode_RagdollEditor::CreateObjects()
{
	if( !m_pxSkin ) return;
	
	m_uNumberOfNodes = m_pxSkin->NumberOfBones;

	m_ppxHierarchyNodes = new Tools_HierarchyNode_Object*[m_uNumberOfNodes];

	// set up the animation, and update it so we can extract the bone positions and orientations
	Asura_Vector_3 xPivotPosition; xPivotPosition.Zero();
	Asura_Matrix_3x3 xPivotOrientation; xPivotOrientation.Identity();

	m_pxAnim = new Asura_Animation( m_pxSkin->GetName() );
	m_pxAnim->Update( xPivotPosition, xPivotOrientation, 0.0f, ASURA_HIERARCHY_RENDER_DONTRENDER | ASURA_HIERARCHY_RENDER_UPDATEBONESALWAYS );
	
	// extract the bone positions and orientations
	Asura_Hierarchy_Tween* pxTweenStorage = m_pxAnim->GetTweenStore();
	Asura_Assert( pxTweenStorage, "Need tween storage to get marker point positions" );
	
	if ( pxTweenStorage->IsPositionDataValid() )  
	{
		for( int i = 0; i < m_pxSkin->NumberOfBones; ++i )
		{
			Asura_Matrix_3x3	xBoneOri;
			Asura_Vector_3		xBonePos;

			pxTweenStorage->GetBoneData(i).GetOrientation(xBoneOri);
			pxTweenStorage->GetBoneData(i).GetPosition(xBonePos);
		
			// If ragdoll is active then all bone positions & orientations are world relative
			if ( pxTweenStorage->IsPositionDataInWorldspace() )
			{
				axBonePosition[i] = xBonePos;
				axBoneOrientation[i] = xBoneOri;
			}
			else
			{
				// Compute the worldspace positions by hand, cascading the transformations bone to bone
				if(i)
				{
					int iParentIndex = m_pxSkin->ParentBone[i];
					Asura_Assert( iParentIndex>=0 && iParentIndex<i, "Error in hierarchy - bone has parent out of expected index range" );

					axBonePosition[i] = xBonePos * axBoneOrientation[iParentIndex]+axBonePosition[iParentIndex];
					axBoneOrientation[i] =  axBoneOrientation[iParentIndex] * xBoneOri;
				}
				else
				{
					axBonePosition[0] = xBonePos * xPivotOrientation + xPivotPosition;
					axBoneOrientation[0] = xPivotOrientation * xBoneOri;
				}
			}

		}
	}
	
	// create the hierarchy nodes, and set their positions
	for( u_int u = 0; u < m_uNumberOfNodes; ++u )
	{
		m_ppxHierarchyNodes[u] = new Tools_HierarchyNode_Object( m_pxSkin, u, (m_pxRagdollDesc->Tools_GetHierarchyNode(u)) );
		m_ppxHierarchyNodes[u]->UpdatePosition( axBonePosition[u], axBoneOrientation[u] );
	}

	if ( m_pxRagdollDesc )
	{
		// create collision objects, and update positions based on the nodes
		u_int uNumberOfCollisionObjects = m_pxRagdollDesc->GetNumberOfCollisionObjects();
		for ( u_int u=0; u<uNumberOfCollisionObjects; ++u )
		{
			Asura_Physics_Ragdoll_Desc_CollisionObject* pxColObj = (m_pxRagdollDesc->Tools_GetCollisionObject(u));
			Tools_RagdollCollision_Object* pxToolsObject = new Tools_RagdollCollision_Object( pxColObj, u, s_pxThis->m_ppxHierarchyNodes[pxColObj->GetHierarchyNode()]->GetName() );
		
			u_int uNode = pxColObj->GetHierarchyNode();
			pxToolsObject->UpdatePosition( axBonePosition[uNode], axBoneOrientation[uNode] );
		}

		// create the joints, and update positions based on nodes also
		u_int uNumberOfJoints = m_pxRagdollDesc->GetNumberOfJoints();
		for ( u_int u=0; u<uNumberOfJoints; ++u )
		{
			Asura_Physics_Ragdoll_Desc_Joint* pxJoint = (m_pxRagdollDesc->Tools_GetJoint(u));
			Tools_RagdollJoint_Object* pxToolsObject = new Tools_RagdollJoint_Object( pxJoint, s_pxThis->m_ppxHierarchyNodes[pxJoint->GetHierarchyNode()]->GetName() );
			
			Asura_Matrix_3x3 xOri;
			pxJoint->GetJointOrientation().ToMat3( xOri );

			pxToolsObject->UpdatePosition( axBonePosition[pxJoint->GetHierarchyNode()], xOri );
		}
	}
}

void Tools_Mode_RagdollEditor::DeleteObjects()
{
	if ( !m_ppxHierarchyNodes ) return;

	// We only need to zero the pointer, as the description memory is held by the ragdoll code.
	// We need to zero it before deleting the other things, so the deletion callback won't
	// try to change the actual description stored in the ragdoll system.
	m_pxRagdollDesc = 0;
		
	for( u_int u = 0; u < m_uNumberOfNodes; ++u )
	{
		Tools_Object_System::DeleteObject( m_ppxHierarchyNodes[u] );
	}
	
	delete [] m_ppxHierarchyNodes;

	m_ppxHierarchyNodes = 0;
	m_uNumberOfNodes = 0;

	{
		Tools_RagdollCollision_Object* pxObject;
		while( (pxObject = Tools_RagdollCollision_Object::GetList().GetHeadObject()) != NULL )
		{
			Tools_Object_System::DeleteObject( pxObject );
		}
	}
	{
		Tools_RagdollJoint_Object* pxObject;
		while( (pxObject = Tools_RagdollJoint_Object::GetList().GetHeadObject()) != NULL )
		{
			Tools_Object_System::DeleteObject( pxObject );
		}
	}

	delete m_pxAnim;
	m_pxAnim = 0;
}



void Tools_Mode_RagdollEditor::CreateSphere( void )
{
	const u_int uNotANode = 0xffffffff;
	u_int uSelectedNode = uNotANode;

	if (Tools_Object_System::SelectedObjectList.size() > 0) 
	{
		for( u_int u = 0; u < s_pxThis->m_uNumberOfNodes; ++u )
		{
			if ( s_pxThis->m_ppxHierarchyNodes[u] == Tools_Object_System::SelectedObjectList[0] )
			{
				uSelectedNode = u;
				break;
			}
		}
	}

	if (uSelectedNode == uNotANode)
	{
		// TODO - display a message explaining you need something selected
		return;
	}

	Asura_Physics_Ragdoll_Desc_CollisionObject* pxCollisionObject = s_pxThis->m_pxRagdollDesc->Tools_AddCollisionObject();
	u_int uCollisionObjectIndex = s_pxThis->m_pxRagdollDesc->GetNumberOfCollisionObjects()-1;

	if (!pxCollisionObject)
	{
		// TODO - display a message explaining about limits
		return;
	}

	Asura_AVector3 xEntents(0.10f,0.10f,0.10f);
	*pxCollisionObject = Asura_Physics_Ragdoll_Desc_CollisionObject( Asura_Maths::IdentityAMatrix3x3, Asura_Maths::ZeroAVector3, xEntents, uSelectedNode, Asura_Physics_Ragdoll_Desc_CollisionObject::COLLISION_SPHERE );

	Tools_RagdollCollision_Object* pxToolsObject = new Tools_RagdollCollision_Object( pxCollisionObject, uCollisionObjectIndex, s_pxThis->m_ppxHierarchyNodes[uSelectedNode]->GetName() );
	
	u_int uNode = pxCollisionObject->GetHierarchyNode();
	pxToolsObject->UpdatePosition( axBonePosition[uNode], axBoneOrientation[uNode] );
	
}

void Tools_Mode_RagdollEditor::CreateCapsule( void )
{
	const u_int uNotANode = 0xffffffff;
	u_int uSelectedNode1 = uNotANode;
	u_int uSelectedNode2 = uNotANode;
	const List<Tools_Object*>* pxList = &(Tools_Object_System::SelectedObjectList);

	if (!pxList) return;

	if ( pxList->size() !=2 )
	{
		// TODO - warning message about needing 2 collision objects
		return;
	}

	{
		for( u_int u = 0; u < s_pxThis->m_uNumberOfNodes; ++u )
		{
			if ( s_pxThis->m_ppxHierarchyNodes[u] == Tools_Object_System::SelectedObjectList[0] )
			{
				uSelectedNode1 = u;
			}

			if ( s_pxThis->m_ppxHierarchyNodes[u] == Tools_Object_System::SelectedObjectList[1] )
			{
				uSelectedNode2 = u;
			}
		}
	}

	if (uSelectedNode1 == uNotANode || uSelectedNode2 == uNotANode ) 
	{
		// TODO - display a message explaining you need something selected
		return;
	}

	// we want uSelectedNode1 to be the first in the hierarchy, i.e. the lowest index
	if ( uSelectedNode2 < uSelectedNode1 )
	{
		u_int uSwap = uSelectedNode1;
		uSelectedNode1 = uSelectedNode2;
		uSelectedNode2 = uSwap;
	}

	Asura_Physics_Ragdoll_Desc_CollisionObject* pxCollisionObject = s_pxThis->m_pxRagdollDesc->Tools_AddCollisionObject();
	u_int uCollisionObjectIndex = s_pxThis->m_pxRagdollDesc->GetNumberOfCollisionObjects()-1;

	if (!pxCollisionObject)
	{
		// TODO - display a message explaining about limits
		return;
	}

	// The orientation needs to be such that the z-axis points along the two nodes
	// this means that are other two axis are abitrary currently.
	Asura_Vector_3 xAlong = axBonePosition[uSelectedNode2]-axBonePosition[uSelectedNode1];

	Asura_Matrix_3x3 xOrient;
	Asura_Maths::OrientationFromDirection( xOrient, xAlong );

	// What we actually want to store is the orientation relative to our base node
	Asura_Matrix_3x3 xTranspose = axBoneOrientation[uSelectedNode1];
	xTranspose.Transpose();
	xOrient = xTranspose*xOrient;

	// default X & Y extents are capsule radius, Z extent is cylinder half-length
	Asura_AVector3 xExtents( 0.10f,0.10f, xAlong.Magnitude()/2.0f );
	
	Asura_AVector3 xPosition( 0.0f, 0.0f, xExtents.GetZ() );

	*pxCollisionObject = Asura_Physics_Ragdoll_Desc_CollisionObject( Asura_AMatrix3x3(xOrient), xPosition, xExtents, uSelectedNode1, Asura_Physics_Ragdoll_Desc_CollisionObject::COLLISION_CAPSULE );

	Tools_RagdollCollision_Object* pxToolsObject = new Tools_RagdollCollision_Object( pxCollisionObject, uCollisionObjectIndex, s_pxThis->m_ppxHierarchyNodes[uSelectedNode1]->GetName() );
	
	u_int uNode = pxCollisionObject->GetHierarchyNode();
	pxToolsObject->UpdatePosition( axBonePosition[uNode], axBoneOrientation[uNode] );
	
}

void Tools_Mode_RagdollEditor::CreateJoint( void )
{
	const u_int uNotANode = 0xffffffff;
	u_int uSelectedNode = uNotANode;

	if (Tools_Object_System::SelectedObjectList.size() > 0) 
	{
		for( u_int u = 0; u < s_pxThis->m_uNumberOfNodes; ++u )
		{
			if ( s_pxThis->m_ppxHierarchyNodes[u] == Tools_Object_System::SelectedObjectList[0] )
			{
				uSelectedNode = u;
				break;
			}
		}
	}

	if (uSelectedNode == uNotANode)
	{
		// TODO - display a message explaining you need something selected
		return;
	}

	Asura_Physics_Ragdoll_Desc_Joint* pxJoint = s_pxThis->m_pxRagdollDesc->Tools_AddJoint();
		
	if ( !pxJoint ) 
	{
		// TODO - display a message about limits for number of joints
		return;
	}

	Asura_AVector3 xPosA, xPosB;
	xPosA.Zero();
	xPosB.Zero();
	u_int uObjectA = Asura_Physics_Ragdoll_Description::uINVALID_INDEX;
	u_int uObjectB = Asura_Physics_Ragdoll_Description::uINVALID_INDEX;

	// TODO - auto find adjacent collision objects to hopefully save time (or something)? 

	*pxJoint = Asura_Physics_Ragdoll_Desc_Joint( xPosA, xPosB, uObjectA, uObjectB, uSelectedNode );

	Tools_RagdollJoint_Object* pxToolsObject = new Tools_RagdollJoint_Object( pxJoint, s_pxThis->m_ppxHierarchyNodes[uSelectedNode]->GetName() );

	// First start with the hierarchy node as the basis for the joint orientation
	Asura_AMatrix3x3 xOrientation( axBoneOrientation[uSelectedNode] );
		
	int iChildNode = -1;

	for ( int i=0; i<(int)s_pxThis->m_uNumberOfNodes; ++i )
	{
		int iParent = s_pxThis->m_pxSkin->ParentBone[i];

		if ( iParent == (int)uSelectedNode )
		{
			iChildNode = i;
			break;
		}
	}

	if ( iChildNode != -1 )
	{
		// Get the vector for this bone's child in the rest pose
		Asura_AVector3 xPosition( s_pxThis->m_pxSkin->BoneRestPose[iChildNode].Position );

		// If this vector isn't tiny, we'll use this as a guide for the joint orientation
		if ( xPosition.NormaliseReturningMagnitude() > 0.00001f )
		{
			// find the rotation needed to make this vector the z-axis
			Asura_AMatrix3x3 xRotation;

			Asura_Quat xQuat;
			Asura_AVector3 xZAxis( 0.0f, 0.0f, 1.0f );

			// KJFL - 20 January 2009 - TODO
			// I was expecting this to be xPosition and not -xPosition, but I ended up with
			// flipped axes on every test case. Am I missing something, and will this 
			// eventually break with a certain hierarchy setup?
			xQuat.RotationFromAToB( -xPosition, xZAxis );
			xQuat.ToMatrix( xRotation );

			// update the joint orientation, so now z-axis is the "rest" direction
			xOrientation = xRotation * xOrientation;
		}
	}

	pxJoint->SetOriginalJointOrientation( xOrientation );

	Asura_Matrix_3x3 xOri;
	xOrientation.ToMat3( xOri );
	pxToolsObject->UpdatePosition( axBonePosition[uSelectedNode], xOri );

	Tools_Object_System::SelectObject( pxToolsObject );
}

bool Tools_Mode_RagdollEditor::IsModified()
{
	return true;
}
void Tools_Mode_RagdollEditor::PreSave()
{
	SaveRagdoll();
}

// End of file. //
