//
// Tools_Mode_RagdollEditor.h
//
// Created by KJFL - 08 October 2008
//
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_MODE_RAGDOLLEDITOR_H__
#define __TOOLS_MODE_RAGDOLLEDITOR_H__

#include "Core/Asura_Maths.h"
#include "Core/Asura_String.h"
#include "Tools_Mode.h"

class Asura_Animation;
class Asura_Hierarchy_Skin;
class Tools_HierarchyNode_Object;
class Tools_RagdollCollision_Object;
class Tools_RagdollJoint_Object;
class Asura_Physics_Ragdoll_Description;

class Tools_Mode_RagdollEditor : public Tools_Mode
{
	public:

		Tools_Mode_RagdollEditor()
		{
			s_pxThis = this;
		}
		~Tools_Mode_RagdollEditor()
		{
			s_pxThis = 0;
		}

		void InitialiseMode(void);
		void ShutdownMode(void);
	
		void Update(void);
		void RenderWorld_PostToneMap(void);
		void RenderUI(void);
		
		virtual bool IsModified();
		virtual void PreSave();

		// called when the Tools object is being deleted (e.g. user selects object and presses delete)
		static void DeleteCollisionObjectCallback( Tools_RagdollCollision_Object* pxObject );
		static void DeleteJointObjectCallback( Tools_RagdollJoint_Object* pxObject );

		static const char* GetNameOfCollisionObject( u_int uIndex );
		static const Tools_RagdollCollision_Object* GetToolsCollisionObject( u_int uIndex );

	protected:

		void CreateModeOverlay( void );

		static void ChooseSkin( void );
		
		static void CreateSphere( void );
		static void CreateCapsule( void );
		static void CreateJoint( void );

		bool SetSkinAndRagdoll( const char* szSkinName, const char* szSkinFilename, Asura_Physics_Ragdoll_Description* pxDesc = NULL );
		bool SelectRagdoll( void );
		void SaveRagdoll( void );

		void CreateObjects();
		void DeleteObjects();

		void UpdateNodeLinks( void );

	private:

		enum DialogID
		{
			DialogID_Skin,
			DialogID_Ragdoll,
		};

		static Tools_Mode_RagdollEditor*	s_pxThis;
		static Asura_Hierarchy_Skin*		s_pxLastUsedSkin;
		static Asura_Physics_Ragdoll_Description* s_pxLastUsedRagdollDesc;

		Asura_Matrix_3x3					m_xPreModeCameraOrientation;
		Asura_Vector_3						m_xPreModeCameraPosition;

		Asura_Vector_2						m_xModeOverlayPosition;
		Asura_Hierarchy_Skin*				m_pxSkin;
		Asura_Animation*					m_pxAnim;

		Tools_HierarchyNode_Object**		m_ppxHierarchyNodes;
		u_int								m_uNumberOfNodes;

		Tools_RagdollCollision_Object**		m_ppxRagdollCollisionObjects;
		u_int								m_uNumberOfRagdollCollisionObjects;

		Asura_Physics_Ragdoll_Description*	m_pxRagdollDesc;

		bool								m_bHideSkin;
		bool								m_bHideSkeleton;
		bool								m_bHideNodes;
		bool								m_bHideCollisionObjects;
		bool								m_bHideJoints;
		bool								m_bHideLights;



};

#endif//__TOOLS_MODE_RAGDOLLEDITOR_H__

