//
// Tools_Mode_Skybox.cpp
//
// Created by Kevin Lea - 03/10/2001 12:16:08
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Skybox/Tools_Mode_Skybox.h"

#include "Core/Asura_TextureManagement.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Rendering/Asura_Render.h"
#include "SpecialFX/Asura_Skybox.h"

#include "Tools_Overlays.h"
#include "Tools_System.h"
#include "Interface/Tools_ColourPicker.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Texturing/Tools_PickTexture.h"


#if defined( ASURA_TARGET_SM3)
float Tools_SkyboxOverlay::s_afSkyboxColour[ 3 ];
float Tools_SkyboxOverlay::s_afHazeColour1[ 3 ];
float Tools_SkyboxOverlay::s_fHazeStrength1;
float Tools_SkyboxOverlay::s_afHazeColour2[ 3 ];
float Tools_SkyboxOverlay::s_fHazeStrength2;
float Tools_SkyboxOverlay::s_afSunColour1[ 3 ];
float Tools_SkyboxOverlay::s_fSunStrength1;
float Tools_SkyboxOverlay::s_afSunColour2[ 3 ];
float Tools_SkyboxOverlay::s_afSunDir1[ 2 ];
float Tools_SkyboxOverlay::s_afSunDir2[ 2 ];
float Tools_SkyboxOverlay::s_fSunStrength2;
bool  Tools_SkyboxOverlay::s_bDisableHaze;
bool  Tools_SkyboxOverlay::s_bDisable2Haze;
bool  Tools_SkyboxOverlay::s_bDisableSun;
bool  Tools_SkyboxOverlay::s_bDisable2Sun;
bool  Tools_SkyboxOverlay::s_b1MirrorBelowHorizon;
bool  Tools_SkyboxOverlay::s_b2MirrorBelowHorizon;
bool Tools_SkyboxOverlay::s_bUseNewSkyboxShader = false;
bool Tools_SkyboxOverlay::s_bNewSkyboxParamaterModified = false;
bool Tools_SkyboxOverlay::s_bNewSkyboxParamaterModified2 = false;
#endif

float Tools_SkyboxOverlay::s_fSkyRed;
float Tools_SkyboxOverlay::s_fSkyGreen;
float Tools_SkyboxOverlay::s_fSkyBlue;
float Tools_SkyboxOverlay::s_fSkyObjectRed;
float Tools_SkyboxOverlay::s_fSkyObjectGreen;
float Tools_SkyboxOverlay::s_fSkyObjectBlue;
Asura_Angle Tools_SkyboxOverlay::s_fSkyOrientation;
bool Tools_SkyboxOverlay::s_bSkyboxParamaterModified = false;
bool Tools_SkyboxOverlay::s_bDrawClouds = true;
bool Tools_SkyboxOverlay::s_bViewSecondCloudTexture = false;
bool Tools_SkyboxOverlay::s_bBackTextureIsFrontUpsideDown = false;
bool Tools_SkyboxOverlay::s_bRightTextureIsLeftUpsideDown = false;
bool Tools_SkyboxOverlay::s_bMirrorBelowHorizon = false;
Asura_Vector_2 Tools_SkyboxOverlay::s_xSkyboxPropertiesOverlayPosition(0,0);
char Tools_SkyboxOverlay::s_aszTextureString[Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES][MAX_PATH+7] = 
{
	"DowN:  ",
	"FRont: ",
	"BacK:  ",
	"LefT:  ",
	"RighT: ",
	"UP:    ",
	"Sky:    ",
	"Sky2:  "
};

bool Tools_SkyboxOverlay::s_bOverlayShown = false;

void Tools_SkyboxOverlay::ToggleOverlay()
{
	if(IsOverlayOn())
	{
		CloseOverlay();		
	}
	else
	{
		CreateSkyboxPropertiesOverlay();
	}
}

void Tools_SkyboxOverlay::CloseOverlay(bool bAskAboutSave /*= true*/)
{
	if(IsOverlayOn())
	{
		//close the overlay
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKYBOXPROPERTIES);

		PostOverlayClosing(bAskAboutSave);

	}
}

void Tools_SkyboxOverlay::PostOverlayClosing(bool bAskAboutSave)
{
	//save overlay position to dat file
	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	// Go back to the first cloud texture, so artists see how the level will look normally when in other modes
	s_bViewSecondCloudTexture = false;
	Asura_Skybox::SetUseSecondCloudTexture(false);

	if(bAskAboutSave && Tools_SaveAsura::IsSkyboxModified())
	{
		//ask whether changes should be saved
		if(ShowYesNoOverlay("Save changes to skybox?"))
		{
			Tools_SaveAsura::SaveSkybox();
		}
	}

	s_bOverlayShown = false;

}

bool Tools_SkyboxOverlay::IsOverlayOn()
{
	return Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_SKYBOXPROPERTIES);
}


void Tools_SkyboxOverlay::Update(void)
{
	if(s_bSkyboxParamaterModified)
	{
		//update the skybox
#if defined( ASURA_TARGET_SM3)
		Asura_Skybox::SetSkyboxColour( s_afSkyboxColour );
		Asura_Skybox::SetHaze( s_afHazeColour1, s_afHazeColour2, s_bDisableHaze, s_bDisable2Haze, s_b1MirrorBelowHorizon, s_b2MirrorBelowHorizon );
		Asura_Skybox::SetSun( s_afSunColour1, s_afSunColour2, s_bDisableSun, s_bDisable2Sun );
		Asura_Skybox::SetHazeStrength( s_fHazeStrength1, s_fHazeStrength2 );
		Asura_Skybox::SetSunSize( s_fSunStrength1, s_fSunStrength2 );
		Asura_Skybox::SetSunDir( s_afSunDir1, s_afSunDir2 );
#endif

		//update the skybox
		Asura_Skybox::SetColours(s_fSkyRed, s_fSkyGreen, s_fSkyBlue);
		Asura_Skybox::SetObjectColours(s_fSkyObjectRed, s_fSkyObjectGreen, s_fSkyObjectBlue);
		Asura_Skybox::SetOrientationAroundYAxis(Asura_Maths::DegToRad(s_fSkyOrientation));
		Asura_Skybox::SetDrawClouds(s_bDrawClouds);
		Asura_Skybox::SetUseSecondCloudTexture(s_bViewSecondCloudTexture);
		Asura_Skybox::SetBackTextureIsFrontUpsideDown(s_bBackTextureIsFrontUpsideDown);
		Asura_Skybox::SetRightTextureIsLeftUpsideDown(s_bRightTextureIsLeftUpsideDown);
		Asura_Skybox::SetMirrorBelowHorizon(s_bMirrorBelowHorizon);
		//reset the flag
		s_bSkyboxParamaterModified = false;
		//and set the skybox file modified flag
		Tools_SaveAsura::SetSkyboxModified();
	}

#if defined( ASURA_TARGET_SM3)
	if( s_bNewSkyboxParamaterModified )
	{
		Asura_Skybox::SetUseNewSkyboxShader( s_bUseNewSkyboxShader );
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKYBOXPROPERTIES);
		CreateSkyboxPropertiesOverlay();
		//and set the skybox file modified flag
		Tools_SaveAsura::SetSkyboxModified();
		s_bNewSkyboxParamaterModified = false;
	}
#endif

#if defined( ASURA_TARGET_SM3)
	if( s_bNewSkyboxParamaterModified2 )
	{
		Asura_Skybox::SetHaze( s_afHazeColour1, s_afHazeColour2, s_bDisableHaze, s_bDisable2Haze, s_b1MirrorBelowHorizon, s_b2MirrorBelowHorizon );
		Asura_Skybox::SetSun( s_afSunColour1, s_afSunColour2, s_bDisableSun, s_bDisable2Sun );
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKYBOXPROPERTIES);
		CreateSkyboxPropertiesOverlay();
		//and set the skybox file modified flag
		Tools_SaveAsura::SetSkyboxModified();
		s_bNewSkyboxParamaterModified2 = false;
	}
#endif

	if(s_bOverlayShown && !IsOverlayOn())
	{
		//overlay was being shown , but is no longer
		PostOverlayClosing(true);
	}
}

void Tools_SkyboxOverlay::TransferModeData(int iMode)
{

	//open the file
	Tools_ModeDataFile xFile("SkyboxMode.dat",iMode);

	//tranfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	xFile.Transfer(s_xSkyboxPropertiesOverlayPosition);
}

#if defined( ASURA_TARGET_SM3)
void Tools_SkyboxOverlay::AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, int iTab )
{
	Asura_Dialog*				pxDialog;
	Tools_ColourPicker_Dialog*	pxColourPickerDialog;
//	float						fTempPosition1;

	switch( iTab )
	{
	case 0:	// Skybox
		{
			xOffset.y += 1.0f;
			pxDialog = new Asura_StringDisplay_Dialog( "Colour", xOffset );
			pxOverlay->AddDialog( pxDialog );
			xOffset.x += 5.0f;

			pxColourPickerDialog = new Tools_ColourPicker_Dialog( "Skybox Colour", xOffset, Asura_Vector_2( 1.5f, 1.0f ), &s_afSkyboxColour[ 0 ], &s_afSkyboxColour[ 1 ], &s_afSkyboxColour[ 2 ], 255.0f, &s_bSkyboxParamaterModified );
			pxColourPickerDialog->EnableHDR(true);
			pxOverlay->AddDialog(pxColourPickerDialog);

			xOffset.x -= 5.0f;
		}
		break;
	case 1: //Haze
		{
			if( s_bDisableHaze )
			{
				pxDialog = new Asura_StringDisplay_Dialog( "Haze", xOffset );
				pxOverlay->AddDialog( pxDialog );
				xOffset.x += 9.0f;
				pxDialog = new Asura_Tickbox_Dialog("Disable",xOffset,&s_bDisableHaze,&s_bNewSkyboxParamaterModified2);
				pxOverlay->AddDialog(pxDialog);
				xOffset.x -= 9.0f;
			}
			else
			{
				pxDialog = new Asura_StringDisplay_Dialog( "First Haze", xOffset );
				pxOverlay->AddDialog( pxDialog );

				xOffset.x += 9.0f;
				pxDialog = new Asura_Tickbox_Dialog("Disable",xOffset,&s_bDisableHaze,&s_bNewSkyboxParamaterModified2);
				pxOverlay->AddDialog(pxDialog);
				xOffset.x -= 9.0f;

				//Second line
				xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;
				pxDialog = new Asura_StringDisplay_Dialog( "Colour", xOffset );
				pxOverlay->AddDialog( pxDialog );

				xOffset.x += 9.0f;
				pxColourPickerDialog = new Tools_ColourPicker_Dialog( "First Haze Colour", xOffset, Asura_Vector_2( 1.5f, 1.0f ), &s_afHazeColour1[ 0 ], &s_afHazeColour1[ 1 ], &s_afHazeColour1[ 2 ], 255.0f, &s_bSkyboxParamaterModified );
				pxColourPickerDialog->EnableHDR(true);
				pxOverlay->AddDialog(pxColourPickerDialog);
				xOffset.x -= 9.0f;

				xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

				pxDialog = new Asura_StringDisplay_Dialog( "Strength", xOffset );
				pxOverlay->AddDialog( pxDialog );
				xOffset.x += 9.0f;
				pxDialog = new Asura_Scrollbar_Dialog( xOffset, &(s_fHazeStrength1), 0.0f, 10.0f, true, 25.0f, "%0.3f", &s_bSkyboxParamaterModified );
				pxOverlay->AddDialog(pxDialog);
				xOffset.x -= 9.0f;

				xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

				pxDialog = new Asura_Tickbox_Dialog("Mirror Below Horizon",xOffset,&s_b1MirrorBelowHorizon,&s_bNewSkyboxParamaterModified2);
				pxOverlay->AddDialog(pxDialog);

				xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;
				pxDialog = new Asura_Divider_Dialog( xOffset, 0xFFFFFFFF );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y+=Asura_Dialog::SmallSpacing();

				if( s_bDisable2Haze )
				{
					pxDialog = new Asura_StringDisplay_Dialog( "Second Haze", xOffset );
					pxOverlay->AddDialog( pxDialog );
					xOffset.x += 9.0f;
					pxDialog = new Asura_Tickbox_Dialog("Disabled",xOffset,&s_bDisable2Haze,&s_bNewSkyboxParamaterModified2);
					pxOverlay->AddDialog(pxDialog);
					xOffset.x -= 9.0f;
				}
				else
				{
					pxDialog = new Asura_StringDisplay_Dialog( "Second Haze", xOffset );
					pxOverlay->AddDialog( pxDialog );

					xOffset.x += 9.0f;
					pxDialog = new Asura_Tickbox_Dialog("Disable",xOffset,&s_bDisable2Haze,&s_bNewSkyboxParamaterModified2);
					pxOverlay->AddDialog(pxDialog);
					xOffset.x -= 9.0f;

					//Second line
					xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;
					pxDialog = new Asura_StringDisplay_Dialog( "Colour", xOffset );
					pxOverlay->AddDialog( pxDialog );

					xOffset.x += 9.0f;
					pxColourPickerDialog = new Tools_ColourPicker_Dialog( "First Haze Colour", xOffset, Asura_Vector_2( 1.5f, 1.0f ), &s_afHazeColour2[ 0 ], &s_afHazeColour2[ 1 ], &s_afHazeColour2[ 2 ], 255.0f, &s_bSkyboxParamaterModified );
					pxColourPickerDialog->EnableHDR(true);
					pxOverlay->AddDialog(pxColourPickerDialog);
					xOffset.x -= 9.0f;

					xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

					pxDialog = new Asura_StringDisplay_Dialog( "Strength", xOffset );
					pxOverlay->AddDialog( pxDialog );
					xOffset.x += 9.0f;
					pxDialog = new Asura_Scrollbar_Dialog( xOffset, &(s_fHazeStrength2), 0.0f, 10.0f, true, 25.0f, "%0.3f", &s_bSkyboxParamaterModified );
					pxOverlay->AddDialog(pxDialog);
					xOffset.x -= 9.0f;

					xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

					pxDialog = new Asura_Tickbox_Dialog("Mirror Below Horizon",xOffset,&s_b2MirrorBelowHorizon,&s_bNewSkyboxParamaterModified2);
					pxOverlay->AddDialog(pxDialog);

					xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

					pxDialog = new Asura_Function_Dialog( "Swap First Haze with Second Haze", xOffset, SwapHaze );
					pxOverlay->AddDialog(pxDialog);
					xOffset.y += Asura_Dialog::DefaultSpacing();
				}
			}
		}
		break;
	case 2: //Sun
		{
			if( s_bDisableSun )
			{
				pxDialog = new Asura_StringDisplay_Dialog( "Corona", xOffset );
				pxOverlay->AddDialog( pxDialog );

				xOffset.x += 9.0f;
				pxDialog = new Asura_Tickbox_Dialog("Disable",xOffset,&s_bDisableSun,&s_bNewSkyboxParamaterModified2);
				pxOverlay->AddDialog(pxDialog);
				xOffset.x -= 9.0f;
			}
			else
			{
				pxDialog = new Asura_StringDisplay_Dialog( "First Corona", xOffset );
				pxOverlay->AddDialog( pxDialog );

				xOffset.x += 9.0f;
				pxDialog = new Asura_Tickbox_Dialog("Disable",xOffset,&s_bDisableSun,&s_bNewSkyboxParamaterModified2);
				pxOverlay->AddDialog(pxDialog);
				xOffset.x -= 9.0f;

				//Second line
				xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

				pxDialog = new Asura_StringDisplay_Dialog( "Colour", xOffset );
				pxOverlay->AddDialog( pxDialog );

				xOffset.x += 9.0f;
				pxColourPickerDialog = new Tools_ColourPicker_Dialog( "First Corona Colour", xOffset, Asura_Vector_2( 1.5f, 1.0f ), &s_afSunColour1[ 0 ], &s_afSunColour1[ 1 ], &s_afSunColour1[ 2 ], 255.0f, &s_bSkyboxParamaterModified );
				pxColourPickerDialog->EnableHDR(true);
				pxOverlay->AddDialog(pxColourPickerDialog);
				xOffset.x -= 9.0f;

				xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

				pxDialog = new Asura_StringDisplay_Dialog( "Size", xOffset );
				pxOverlay->AddDialog( pxDialog );

				xOffset.x += 9.0f;
				pxDialog = new Asura_Scrollbar_Dialog( xOffset, &(s_fSunStrength1), 0.0f, 7.0f, true, 25.0f, "%0.3f", &s_bSkyboxParamaterModified );
				pxOverlay->AddDialog(pxDialog);
				xOffset.y+=Asura_Dialog::DefaultSpacing();
				xOffset.x -= 9.0f;

				xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

				//Position
				pxDialog = new Asura_StringDisplay_Dialog( "Pitch", xOffset );
				pxOverlay->AddDialog( pxDialog );
				xOffset.x += 9.0f;
				pxDialog = new Asura_Scrollbar_Dialog( xOffset, &(s_afSunDir1[ 0 ] ), -90.0f, 90.0f, true, 25.0f, "%0.2f", &s_bSkyboxParamaterModified );
				pxOverlay->AddDialog(pxDialog);
				xOffset.x -= 9.0f;

				xOffset.y+=Asura_Dialog::SmallSpacing();
				pxDialog = new Asura_StringDisplay_Dialog( "Yaw", xOffset );
				pxOverlay->AddDialog( pxDialog );

				xOffset.x += 9.0f;
				pxDialog = new Asura_Scrollbar_Dialog( xOffset, &(s_afSunDir1[ 1 ] ), 0.0f, 360.0f, true, 25.0f, "%0.2f", &s_bSkyboxParamaterModified );
				pxOverlay->AddDialog(pxDialog);
				xOffset.x -= 9.0f;

				xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;
				pxDialog = new Asura_Divider_Dialog( xOffset, 0xFFFFFFFF );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y+=Asura_Dialog::SmallSpacing();

				if( s_bDisable2Sun )
				{
					pxDialog = new Asura_StringDisplay_Dialog( "Second Corona", xOffset );
					pxOverlay->AddDialog( pxDialog );

					xOffset.x += 9.0f;
					pxDialog = new Asura_Tickbox_Dialog("Disable",xOffset,&s_bDisable2Sun,&s_bNewSkyboxParamaterModified2);
					pxOverlay->AddDialog(pxDialog);
					xOffset.x -= 9.0f;
				}
				else
				{
					pxDialog = new Asura_StringDisplay_Dialog( "Second Corona", xOffset );
					pxOverlay->AddDialog( pxDialog );

					xOffset.x += 9.0f;
					pxDialog = new Asura_Tickbox_Dialog("Disable",xOffset,&s_bDisable2Sun,&s_bNewSkyboxParamaterModified2);
					pxOverlay->AddDialog(pxDialog);
					xOffset.x -= 9.0f;

					//Second line
					xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

					pxDialog = new Asura_StringDisplay_Dialog( "Colour", xOffset );
					pxOverlay->AddDialog( pxDialog );

					xOffset.x += 9.0f;
					pxColourPickerDialog = new Tools_ColourPicker_Dialog( "Second Corona Colour", xOffset, Asura_Vector_2( 1.5f, 1.0f ), &s_afSunColour2[ 0 ], &s_afSunColour2[ 1 ], &s_afSunColour2[ 2 ], 255.0f, &s_bSkyboxParamaterModified );
					pxColourPickerDialog->EnableHDR(true);
					pxOverlay->AddDialog(pxColourPickerDialog);
					xOffset.x -= 9.0f;

					xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

					pxDialog = new Asura_StringDisplay_Dialog( "Size", xOffset );
					pxOverlay->AddDialog( pxDialog );

					xOffset.x += 9.0f;
					pxDialog = new Asura_Scrollbar_Dialog( xOffset, &(s_fSunStrength2), 0.0f, 7.0f, true, 25.0f, "%0.3f", &s_bSkyboxParamaterModified );
					pxOverlay->AddDialog(pxDialog);
					xOffset.y+=Asura_Dialog::DefaultSpacing();
					xOffset.x -= 9.0f;

					xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;

					//Position
					pxDialog = new Asura_StringDisplay_Dialog( "Pitch", xOffset );
					pxOverlay->AddDialog( pxDialog );
					xOffset.x += 9.0f;
					pxDialog = new Asura_Scrollbar_Dialog( xOffset, &(s_afSunDir2[ 0 ] ), -90.0f, 90.0f, true, 25.0f, "%0.2f", &s_bSkyboxParamaterModified );
					pxOverlay->AddDialog(pxDialog);
					xOffset.x -= 9.0f;

					xOffset.y+=Asura_Dialog::SmallSpacing();
					pxDialog = new Asura_StringDisplay_Dialog( "Yaw", xOffset );
					pxOverlay->AddDialog( pxDialog );

					xOffset.x += 9.0f;
					pxDialog = new Asura_Scrollbar_Dialog( xOffset, &(s_afSunDir2[ 1 ] ), 0.0f, 360.0f, true, 25.0f, "%0.2f", &s_bSkyboxParamaterModified );
					pxOverlay->AddDialog(pxDialog);
					xOffset.x -= 9.0f;

					xOffset.y += Asura_Dialog::SmallSpacing() * 2.0f;
					pxDialog = new Asura_Function_Dialog( "Swap First Corona with Second Corona", xOffset, SwapSun );
					pxOverlay->AddDialog(pxDialog);
					xOffset.y += Asura_Dialog::DefaultSpacing();
				}
			}
		}
		break;
	}
}

void Tools_SkyboxOverlay::SwapHaze( void )
{
	float afTemp[ 3 ];
	bool bTemp;

	for( int i = 0; i < 3; ++i )
	{
		afTemp[ i ]				= s_afHazeColour1[ i ];
		s_afHazeColour1[ i ]	= s_afHazeColour2[ i ];
		s_afHazeColour2[ i ]	= afTemp[ i ];
	}
	bTemp						= s_b1MirrorBelowHorizon;
	s_b1MirrorBelowHorizon		= s_b2MirrorBelowHorizon;
	s_b2MirrorBelowHorizon		= bTemp;

	afTemp[ 0 ]					= s_fHazeStrength1;
	s_fHazeStrength1			= s_fHazeStrength2;
	s_fHazeStrength2			= afTemp[ 0 ];

	Asura_Skybox::SetHaze( s_afHazeColour1, s_afHazeColour2, s_bDisableHaze, s_bDisable2Haze, s_b1MirrorBelowHorizon, s_b2MirrorBelowHorizon );
	Asura_Skybox::SetHazeStrength( s_fHazeStrength1, s_fHazeStrength2 );
	Tools_SaveAsura::SetSkyboxModified();

	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_SKYBOXPROPERTIES );
	CreateSkyboxPropertiesOverlay();
}

void Tools_SkyboxOverlay::SwapSun( void )
{
	float afTemp[ 3 ];
	bool bTemp;
	int i;

	for( i = 0; i < 3; ++i )
	{
		afTemp[ i ]			= s_afSunColour1[ i ];
		s_afSunColour1[ i ]	= s_afSunColour2[ i ];
		s_afSunColour2[ i ]	= afTemp[ i ];
	}
	for( i = 0; i < 2; ++i )
	{
		afTemp[ i ]			= s_afSunDir1[ i ];
		s_afSunDir1[ i ]	= s_afSunDir2[ i ];
		s_afSunDir2[ i ]	= afTemp[ i ];
	}
	bTemp						= s_b1MirrorBelowHorizon;
	s_b1MirrorBelowHorizon		= s_b2MirrorBelowHorizon;
	s_b2MirrorBelowHorizon		= bTemp;

	afTemp[ 0 ]					= s_fSunStrength1;
	s_fSunStrength1			= s_fSunStrength2;
	s_fSunStrength2			= afTemp[ 0 ];

	Asura_Skybox::SetSun( s_afSunColour1, s_afSunColour2, s_bDisableSun, s_bDisable2Sun );
	Asura_Skybox::SetSunSize( s_fSunStrength1, s_fSunStrength2 );
	Asura_Skybox::SetSunDir( s_afSunDir1, s_afSunDir2 );
	Tools_SaveAsura::SetSkyboxModified();

	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_SKYBOXPROPERTIES );
	CreateSkyboxPropertiesOverlay();
}

void Tools_SkyboxOverlay::CreateNewSkyboxPropertiesOverlay( void )
{
	//load values from .dat file
	TransferModeData(TOOLSMODEDATAFILE_LOAD);

	//initialise all the static variables	
#if defined( ASURA_TARGET_SM3)
	Asura_Skybox::GetSkyboxColour( s_afSkyboxColour );
	Asura_Skybox::GetHaze( s_afHazeColour1, s_afHazeColour2, s_bDisableHaze, s_bDisable2Haze, s_b1MirrorBelowHorizon, s_b2MirrorBelowHorizon );
	Asura_Skybox::GetSun( s_afSunColour1, s_afSunColour2, s_bDisableSun, s_bDisable2Sun );
	Asura_Skybox::GetHazeStrength( &s_fHazeStrength1, &s_fHazeStrength2 );
	Asura_Skybox::GetSunSize( &s_fSunStrength1, &s_fSunStrength2 );
	Asura_Skybox::GetSunDir( s_afSunDir1, s_afSunDir2 );
	s_bUseNewSkyboxShader = Asura_Skybox::GetUseNewSkyboxShader();
#endif

	//create the overlay
	Asura_Overlay*				pxOverlay;
	Asura_Dialog*				pxDialog;

	pxOverlay				= new Asura_Overlay( "Skybox Properties", false, &s_xSkyboxPropertiesOverlayPosition );
	Asura_Vector_2 xOffset	= Asura_Vector_2( 1.0f, 3.0f );

	static const char* s_aszTabs[ 3 ] =
	{
		"Skybox",
		"Haze",
		"Corona",
	};

	//start of tabs
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog( xOffset, s_aszTabs, 3, 20 );
	pxOverlay->AddDialog( pxTabsDialog );
	pxOverlay->PushCurrentParentDialog( pxTabsDialog );
	xOffset = pxTabsDialog->GetTopDialogPosition();

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;
	for( int iTab=0; iTab < 3; ++iTab )
	{
		pxTabsDialog->SelectTab(iTab);
		xOffset.y = fTabsStartY;

		AddOverlayControls( pxOverlay, xOffset, iTab );

		if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;
	}

	//End of tabs
	xOffset.y = fTabsEndY + 1.0f;
	xOffset.x = 1.0f;

	pxOverlay->PopCurrentParentDialog();
	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer( &s_uSelectedTab );

	xOffset.y+=Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use new skybox shader",xOffset,&s_bUseNewSkyboxShader,&s_bNewSkyboxParamaterModified);
	pxOverlay->AddDialog(pxDialog);

	pxOverlay->m_uID = TOOLS_OVERLAY_SKYBOXPROPERTIES;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);

	s_bOverlayShown = true;
}
#endif

void Tools_SkyboxOverlay::CreateSkyboxPropertiesOverlay(void)
{
	//load values from .dat file
	TransferModeData(TOOLSMODEDATAFILE_LOAD);
#if defined( ASURA_TARGET_SM3)
	if( Asura_Skybox::GetUseNewSkyboxShader() )
	{
		CreateNewSkyboxPropertiesOverlay();
	}
	else
#endif
	{
		//initialise all the static variables	
		Asura_Skybox::GetColours(s_fSkyRed,s_fSkyGreen,s_fSkyBlue);
		Asura_Skybox::GetObjectColours(s_fSkyObjectRed,s_fSkyObjectGreen,s_fSkyObjectBlue);
		Asura_Skybox::GetOrientationAroundYAxis(s_fSkyOrientation);
		s_fSkyOrientation = Asura_Maths::RadToDeg(s_fSkyOrientation);
		s_bDrawClouds = Asura_Skybox::GetDrawClouds();
		s_bBackTextureIsFrontUpsideDown = Asura_Skybox::GetBackTextureIsFrontUpsideDown();
		s_bRightTextureIsLeftUpsideDown = Asura_Skybox::GetRightTextureIsLeftUpsideDown();
		s_bMirrorBelowHorizon = Asura_Skybox::GetMirrorBelowHorizon();
#if defined( ASURA_TARGET_SM3)
		s_bUseNewSkyboxShader = Asura_Skybox::GetUseNewSkyboxShader();
#endif

		for(int i=0; i<Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES; i++)
		{
			const Asura_String& xName = Asura_Skybox::GetTexture(i);

			if (!xName.IsNull())
			{
				strcpy(&s_aszTextureString[i][7],xName.GetString());
			}
		}

		//create the overlay
		Asura_Overlay* pxOverlay;
		pxOverlay = new Asura_Overlay("Skybox Properties",false,&s_xSkyboxPropertiesOverlayPosition);
		Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
		
		Asura_Dialog* pxDialog;
		Tools_ColourPicker_Dialog* pxColourPickerDialog;
	#if defined (ASURA_USE_STREAMING) || !defined(ASURA_TARGET_SM3)
		char szText[200];
	#endif //#if defined (ASURA_USE_STREAMING) || !defined(ASURA_TARGET_SM3)
	    	
		
		//RAR 10/03/2006 Skybox name only needs to be edited when using streaming at the moment
		#ifdef ASURA_USE_STREAMING
		const char* szSkyboxName = Asura_Skybox::GetSkyboxName();

		Asura_PrintfToBfr(szText,"Name : %s",200,true,szSkyboxName ? szSkyboxName : "*None*");

		pxDialog = new Asura_Function_Dialog(szText,xOffset,SetSkyboxName);
		pxDialog->SetHelpText("The skybox name determines the filename used when doing a skybox export.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::LargeSpacing();
		#endif //ASURA_USE_STREAMING

		pxDialog = new Asura_StringDisplay_Dialog("Skybox Colour",xOffset);
		pxOverlay->AddDialog(pxDialog);

		xOffset.x += 15.0f;
		pxDialog = new Asura_StringDisplay_Dialog("Skybox Object Colour",xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.x -= 15.0f;

		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxColourPickerDialog = new Tools_ColourPicker_Dialog("Skybox Colour",xOffset,Asura_Vector_2(4.0f,3.0f),&s_fSkyRed,&s_fSkyGreen,&s_fSkyBlue,255.0f,&s_bSkyboxParamaterModified);
		pxColourPickerDialog->EnableHDR(true);
		pxOverlay->AddDialog(pxColourPickerDialog);

		xOffset.x += 15.0f;
		pxColourPickerDialog = new Tools_ColourPicker_Dialog("Skybox Object Colour",xOffset,Asura_Vector_2(4.0f,3.0f),&s_fSkyObjectRed,&s_fSkyObjectGreen,&s_fSkyObjectBlue,255.0f,&s_bSkyboxParamaterModified);
		// (AAM - 13.02.2008) Don't enable HDR on the object colour picker - it gets stored in an int
		// and I don't have time to sort it out at the moment
		pxOverlay->AddDialog(pxColourPickerDialog);
		xOffset.x -= 15.0f;

		xOffset.y+=4.0f;

		pxDialog = new Asura_Numerical_Dialog("Orientation",xOffset,&(s_fSkyOrientation), 0,360.0f,1.0f,"%0.0f",&s_bSkyboxParamaterModified);
		pxDialog->SetHelpText("Orientation of skybox in degrees.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::LargeSpacing();


		pxDialog = new Asura_Function_Dialog("Reload textures", xOffset, ReloadTextures);
		pxDialog->SetHelpText("Force all textures used by skybox to be reloaded.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog("Clear all textures", xOffset, ClearAllTextures);
		pxDialog->SetHelpText("Resets all sides of the skybox to having no texture.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog(s_aszTextureString[Asura_Skybox::ASURA_SKYBOX_FR],xOffset,SelectTextureString_FR);
		pxDialog->SetHelpText("Select texture for front of skybox.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog(s_aszTextureString[Asura_Skybox::ASURA_SKYBOX_BK],xOffset,SelectTextureString_BK, NULL, AllowSelectTexture_Back);
		pxDialog->SetHelpText("Select texture for back of skybox.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog(s_aszTextureString[Asura_Skybox::ASURA_SKYBOX_LF],xOffset,SelectTextureString_LF);
		pxDialog->SetHelpText("Select texture for left of skybox.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog(s_aszTextureString[Asura_Skybox::ASURA_SKYBOX_RT],xOffset,SelectTextureString_RT, NULL, AllowSelectTexture_Right);
		pxDialog->SetHelpText("Select texture for right of skybox.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog(s_aszTextureString[Asura_Skybox::ASURA_SKYBOX_UP],xOffset,SelectTextureString_UP);
		pxDialog->SetHelpText("Select texture for top of skybox.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog(s_aszTextureString[Asura_Skybox::ASURA_SKYBOX_DN],xOffset,SelectTextureString_DN, NULL, AllowSelectTexture_Down);
		pxDialog->SetHelpText("Select texture for bottom of skybox.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog(s_aszTextureString[Asura_Skybox::ASURA_SKYBOX_FRACTALSKY],xOffset,SelectTextureString_FractalSky);
		pxDialog->SetHelpText("Select texture for the clouds.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Function_Dialog(s_aszTextureString[Asura_Skybox::ASURA_SKYBOX_FRACTALSKY_2],xOffset,SelectTextureString_FractalSky2);
		pxDialog->SetHelpText("Select second texture for the clouds.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Use left texture upside-down as right texture", xOffset, &s_bRightTextureIsLeftUpsideDown, &s_bSkyboxParamaterModified);
		pxDialog->SetHelpText("Use the often wasted bottom half of the left texture upside-down as the right texture (saves up to 256kb on PS2) - Not yet implemented for PSP or Gamecube");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Use front texture upside-down as back texture", xOffset, &s_bBackTextureIsFrontUpsideDown, &s_bSkyboxParamaterModified);
		pxDialog->SetHelpText("Use the often wasted bottom half of the front texture upside-down as the back texture (saves up to 256kb on PS2) - Not yet implemented for PSP or Gamecube");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Mirror below horizon", xOffset, &s_bMirrorBelowHorizon, &s_bSkyboxParamaterModified);
		pxDialog->SetHelpText("Simulate water reflections by mirroring the skybox below the horizon");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("Draw Clouds",xOffset,&s_bDrawClouds,&s_bSkyboxParamaterModified);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Tickbox_Dialog("View Second Clouds",xOffset,&s_bViewSecondCloudTexture,&s_bSkyboxParamaterModified);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::LargeSpacing();

	#ifndef ASURA_TARGET_SM3
		Asura_PrintfToBfr(szText,"Reflectance Map : %s",200,true,Asura_Render::GetReflectanceMapName());
		pxDialog = new Asura_Function_Dialog(szText,xOffset,SelectReflectanceMap);
		pxDialog->SetHelpText("Select the reflectance texture map for the level.\nThis texture will be used for materials that have the reflective flag set.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::LargeSpacing();
	#endif //#ifndef ASURA_TARGET_SM3

		pxDialog = new Asura_Function_Dialog("Edit Flags",xOffset,EditTextureFlags);
		pxDialog->SetHelpText("Edit texture flags for textures used by the skybox");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		
#ifdef ASURA_TARGET_SM3
		xOffset.y+=Asura_Dialog::LargeSpacing();
		pxDialog = new Asura_Tickbox_Dialog("Use new skybox shader",xOffset,&s_bUseNewSkyboxShader,&s_bNewSkyboxParamaterModified);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::LargeSpacing();
#endif

		pxOverlay->m_uID = TOOLS_OVERLAY_SKYBOXPROPERTIES;

		Asura_Overlay_System::SubmitNewOverlay(pxOverlay);

		s_bOverlayShown = true;
	}
}

void Tools_SkyboxOverlay::SelectTexture(int iFace)
{
	static Asura_String s_xDefaultPath = "Graphics";

	Tools_FileSystem_DialogInfo xInfo;

	xInfo.DefaultExtension = "";
	xInfo.DefaultFilename = "";

	xInfo.DefaultPath			= s_xDefaultPath.GetString();
	xInfo.FileFilter = "Texture Files\0*.bmp;*.jpg;*.dds;*.tga;*.png\0";
	xInfo.WindowTitle = "Load Texture...";

	Asura_String xName = Tools_FileSystem::GetLoadFilename(xInfo);
	Asura_String xRelativeName;

	if (xName.IsNull()) return;

	//store the directory the file was selected from , for next time a selection is made
	xName.SplitFilename(&s_xDefaultPath,0,0);

	{
		_strlwr(xName.GetString());
		char *pcStart = strstr(xName.GetString(),"graphics");

		if (!pcStart)
		{
			Asura_Assert(0,"File not in 'graphics' folder");
			return;
		}
		else
		{
			pcStart+=strlen("graphics");
			xRelativeName = pcStart;
		}
	}

	if(Tools_SaveAsura::GetFileType() == AsuraFile_Environment)
	{
		if(!Tools_SaveAsura::TextureAllowedForThisFile(xRelativeName))
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"The texture isn't in the 'environments' directory");
			return;
		}
	}

	Asura_Skybox::SetTexture(xRelativeName.GetString(),iFace);
	strcpy(&s_aszTextureString[iFace][7],xRelativeName.GetString());

	// Kevin Lea - 03/10/2001 16:47:22 - the two lines below are a hacky way of resizing the overlay to cope
	// with the changing text. Some sort of CheckForResize method is probably needed.
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKYBOXPROPERTIES);
	CreateSkyboxPropertiesOverlay();

	Tools_SaveAsura::SetSkyboxModified();
}

void Tools_SkyboxOverlay::ClearAllTextures()
{
	// Check they meant to do this.
	YesNo_Overlay xOverlay(" Warning.\n This will clear all skybox and cloud textures! Continue?");
	if (xOverlay.DoModal())
	{
		// Blank the textures.
		for (int i=0;i<Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES;i++)
		{
			Asura_Skybox::SetTexture(NULL,i);
			s_aszTextureString[i][7] = 0;
		}
		// Redraw the overlay.
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKYBOXPROPERTIES);
		CreateSkyboxPropertiesOverlay();
		// Set the file as changed.
		Tools_SaveAsura::SetSkyboxModified();
	}
}

void Tools_SkyboxOverlay::SelectTextureString_FR(void)
{
	SelectTexture(Asura_Skybox::ASURA_SKYBOX_FR);
}

void Tools_SkyboxOverlay::SelectTextureString_BK(void)
{
	SelectTexture(Asura_Skybox::ASURA_SKYBOX_BK);
}

void Tools_SkyboxOverlay::SelectTextureString_LF(void)
{
	SelectTexture(Asura_Skybox::ASURA_SKYBOX_LF);
}

void Tools_SkyboxOverlay::SelectTextureString_RT(void)
{
	SelectTexture(Asura_Skybox::ASURA_SKYBOX_RT);
}

void Tools_SkyboxOverlay::SelectTextureString_UP(void)
{
	SelectTexture(Asura_Skybox::ASURA_SKYBOX_UP);
}

void Tools_SkyboxOverlay::SelectTextureString_DN(void)
{
	SelectTexture(Asura_Skybox::ASURA_SKYBOX_DN);
}

void Tools_SkyboxOverlay::SelectTextureString_FractalSky(void)
{
	SelectTexture(Asura_Skybox::ASURA_SKYBOX_FRACTALSKY);
}

void Tools_SkyboxOverlay::SelectTextureString_FractalSky2(void)
{
	SelectTexture(Asura_Skybox::ASURA_SKYBOX_FRACTALSKY_2);
}

void Tools_SkyboxOverlay::ReloadTextures()
{
	for(int i=0; i<Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES; i++)
	{
		const Asura_String& xName = Asura_Skybox::GetTexture(i);
		if(!xName.IsBlank())
		{
			int iIndex = Asura_Texture_Management::GetTextureIndex(xName.GetString());
			if(iIndex != ASURA_TEXTURE_NONE)
			{
				Asura_Texture_Management::ReloadTexture(iIndex);
			}
		}
	}
}

#ifndef ASURA_TARGET_SM3
void Tools_SkyboxOverlay::SelectReflectanceMap()
{
	//ask for the new texture
	int iCurrentTexture = Asura_Render::GetReflectanceMap();
	int iNewTexture = Tools_PickTexture::PickTexture("Pick Reflectance Map",iCurrentTexture,PICKTEXTURE_ALLTEXTURES);

	if(iNewTexture != ASURA_TEXTURE_NONE && iNewTexture != iCurrentTexture)
	{
		//get the xName of the texture , and update in Asura_Render
		const Asura_String* pxName = Asura_Texture_Management::GetTextureName(iNewTexture);
		Asura_Assert(pxName,"");

		Asura_Render::SetReflectanceMapName(pxName->GetString());

		if(Tools_SaveAsura::GetFileType() == AsuraFile_Environment)
		{
			//set the skybox file as modified (since it also stores this)
			Tools_SaveAsura::SetSkyboxModified();
		}

		if(IsOverlayOn())
		{
			//delete/create the overlay to update the text
			Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKYBOXPROPERTIES);
			CreateSkyboxPropertiesOverlay();
		}

	}
}
#endif //#ifndef ASURA_TARGET_SM3

bool Tools_SkyboxOverlay::AllowSelectTexture_Back(void)
{
	return !Asura_Skybox::GetBackTextureIsFrontUpsideDown();
}

bool Tools_SkyboxOverlay::AllowSelectTexture_Right(void)
{
	return !Asura_Skybox::GetRightTextureIsLeftUpsideDown();
}

bool Tools_SkyboxOverlay::AllowSelectTexture_Down()
{
	return !Asura_Skybox::GetMirrorBelowHorizon();
}

bool Tools_SkyboxOverlay::IsSkyboxTexture(int iTextureID)
{
	if(iTextureID == ASURA_TEXTURE_NONE) return false;

	for(int i=0;i<Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES;i++)
	{
		Asura_String xTextureName = Asura_Skybox::GetTexture(i);
		if(xTextureName.IsBlank()) continue;

		int iSkyTextureID = Asura_Texture_Management::GetTextureIndex(xTextureName);
		if(iTextureID == iSkyTextureID) return true;
	}
	return false;
}

bool Tools_SkyboxOverlay::IsReflectanceTexture(int iTextureID)
{
	if(iTextureID == ASURA_TEXTURE_NONE) return false;

	Asura_String xTextureName = Asura_Render::GetReflectanceMapName();
	if(xTextureName.IsBlank()) return false;

	int iReflectTextureID = Asura_Texture_Management::GetTextureIndex(xTextureName);
	return (iTextureID == iReflectTextureID);
}


void Tools_SkyboxOverlay::EditTextureFlags()
{
	Tools_PickTexture::EditTextureFlags(PICKTEXTURE_SKYBOX);
}


void Tools_SkyboxOverlay::SetSkyboxName()
{
	GetString_Overlay xOverlay("Set Skybox Name",Asura_Skybox::GetSkyboxName());
	if(xOverlay.DoModal())
	{
		const char* szName = xOverlay.GetString();
		if(szName && !*szName) szName = 0;
		
		Asura_Skybox::SetSkyboxName(szName);
		Tools_SaveAsura::SetSkyboxModified();

		if(IsOverlayOn())
		{
			//delete/create the overlay to update the text
			Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKYBOXPROPERTIES);
			CreateSkyboxPropertiesOverlay();
		}
	}
}