//
// Tools_Mode_Skybox.h 
//
// Created by Kevin Lea - 03/10/2001 12:16:20
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_MODE_SKYBOX_H__
#define __TOOLS_MODE_SKYBOX_H__

#include "Core/Asura_Maths.h"
#include "SpecialFX/Asura_Skybox.h"

#include "PC/FileSystem/Tools_FileSystem.h"
#include "Tools_Mode.h"

class Asura_Overlay;

class Tools_SkyboxOverlay
{
	public:


		static void ToggleOverlay();
		static bool IsOverlayOn();
		static void CloseOverlay(bool bAskAboutSave = true);

		static void Update(void);

#ifndef ASURA_TARGET_SM3
		static void SelectReflectanceMap();
#endif //#ifndef ASURA_TARGET_SM3

		//returns true if index of a skybox texture
		static bool IsSkyboxTexture(int iTextureID);
		static bool IsReflectanceTexture(int iTextureID);

	private:
		//loads/saves data for this mode
		static void TransferModeData(int iMode);

		static void PostOverlayClosing(bool bAskAboutSave);

		static void CreateSkyboxPropertiesOverlay(void);
#if defined( ASURA_TARGET_SM3)
		static void CreateNewSkyboxPropertiesOverlay( void );
		static void AddOverlayControls( Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset, int iTab );
		static void SwapHaze( void );
		static void SwapSun( void );
#endif
		static void SelectTexture(int iFace);

		static void ClearAllTextures();
		static void SelectTextureString_FR(void);
		static void SelectTextureString_BK(void);
		static void SelectTextureString_LF(void);
		static void SelectTextureString_RT(void);
		static void SelectTextureString_UP(void);
		static void SelectTextureString_DN(void);
		static void SelectTextureString_FractalSky(void);
		static void SelectTextureString_FractalSky2(void);
		static void ReloadTextures();

		static void SetSkyboxName();

		static void EditTextureFlags();

		static bool AllowSelectTexture_Back(void);
		static bool AllowSelectTexture_Right(void);
		static bool AllowSelectTexture_Down();

#if defined( ASURA_TARGET_SM3)
		static float s_afSkyboxColour[ 3 ];
		static float s_afHazeColour1[ 3 ];
		static float s_fHazeStrength1;
		static float s_afHazeColour2[ 3 ];
		static float s_fHazeStrength2;
		static float s_afSunColour1[ 3 ];
		static float s_fSunStrength1;
		static float s_afSunColour2[ 3 ];
		static float s_fSunStrength2;
		static float s_afSunDir1[ 2 ];
		static float s_afSunDir2[ 2 ];
#endif

		static float s_fSkyRed;
		static float s_fSkyGreen;
		static float s_fSkyBlue;
		static float s_fSkyObjectRed;
		static float s_fSkyObjectGreen;
		static float s_fSkyObjectBlue;
		static Asura_Angle s_fSkyOrientation;

		static bool s_bSkyboxParamaterModified;
#if defined( ASURA_TARGET_SM3)
		static bool s_bNewSkyboxParamaterModified;
		static bool s_bNewSkyboxParamaterModified2;
#endif
		static bool s_bDrawClouds;
		static bool s_bViewSecondCloudTexture;
		static bool s_bBackTextureIsFrontUpsideDown;
		static bool s_bRightTextureIsLeftUpsideDown;
		static bool s_bMirrorBelowHorizon;
		static bool s_bOverlayShown;
#if defined( ASURA_TARGET_SM3)
		static bool s_bUseNewSkyboxShader;
		static bool s_b1MirrorBelowHorizon;
		static bool s_b2MirrorBelowHorizon;
		static bool s_bDisableHaze;
		static bool s_bDisable2Haze;
		static bool s_bDisableSun;
		static bool s_bDisable2Sun;
#endif

		static Asura_Vector_2 s_xSkyboxPropertiesOverlayPosition;
		static char s_aszTextureString[Asura_Skybox::ASURA_SKYBOX_NUM_TEXTURES][MAX_PATH+7];

};

#endif//__TOOLS_MODE_SKYBOX_H__
