//
// Tools_Bink_Conversion.cpp
//
// Created by KOR - 06 July 2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

// This file's header.
#include "Sound/Conversion/Tools_Bink_Conversion.h"

// Asura
#include "Debugging/Asura_Assert.h"

// Tools
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Load&Save/Tools_Sound_Export.h"
#include "PC/FileSystem/Tools_FileSystem.h"

// Misc
#include <windows.h>

/******************************************************************************/
/* Functions                                                                  */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// Tools_ConvertBinkFile()
//
// Converts a wave file into an Bink. Does nothing if the file already exists
// and doesn't need reconverting
//
// Parameters:		szInputFileName - Path of input file
//					szOutputFileName - Path of output file
//					pxConvertData - Pointer to conversion options structure
//					puFailureCode - Optional pointer to output failure code
//
// Return value:	true for success, false for failure
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_ConvertBinkFile( const char* const szInputFileName,
						  const char* const szOutputFileName,
						  const ToolsSoundConversionData* const pxConvertData,
						  u_int* const puFailureCode /*= 0*/ )
{
#define SET_FAILURE_CODE(__code) \
	if( puFailureCode ) \
	{ \
		*puFailureCode = __code; \
	}

	// Parameter check
	if( !szInputFileName || !szOutputFileName )
	{
		Asura_Assert( false, "NULL parameter passed to Tools_ConvertBinkFile()." );
		
		SET_FAILURE_CODE(Tools_ConvertBinkFile_FailedToOpenWav);

		return false;
	}

	// Make sure we have a target directory
	Tools_FileSystem::EnsureDirectoryExists(szOutputFileName);
	// Execute the Bink encoder tool
	// Command line can get rather long.
	const u_int uCommandLineLength = 1024;
	char szCommandLine[uCommandLineLength];
	const char szExe[] = "bink.exe";
	// Switches:
	// /R - frequency. Note: turned this off since it seemed to be causing glitches.
	// /C - number of channels
	// /B - bitrate
	// /L - encoding quality - 0 = lossless, 99 = very lossy
	// /O - overwrite
	// /# - suppress window
	// KOR - 06/07/2007 - let's set this low quality and see if the audio guys complain.
	Asura_PrintfToBfr(szCommandLine, "%s \"%s\" \"%s\" /C1 /B16 /L4 /O /#", uCommandLineLength, true, szExe, szInputFileName, szOutputFileName);

	PROCESS_INFORMATION xProcessInfo;
	STARTUPINFO xStartupInfo = {0};
	xStartupInfo.cb = sizeof( STARTUPINFO );
	// Suppress the window, otherwise it pops up and annoyingly steals focus.
	xStartupInfo.dwFlags = STARTF_USESHOWWINDOW;
	xStartupInfo.wShowWindow = SW_SHOWMINNOACTIVE;

	if( !CreateProcess( NULL, szCommandLine, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &xStartupInfo, &xProcessInfo ) )
	{
#if defined ASURA_MAIN
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Failed to encode sound. Check that %s is present.", szExe );
#else
		Asura_Assertf( false, ("Failed to encode sound. Check that %s is present.", szExe ));
#endif

		SET_FAILURE_CODE(Tools_ConvertBinkFile_FailedToLoadExe);

		return false;
	}

	WaitForSingleObject( xProcessInfo.hProcess, INFINITE );

	// TODO: Check the exit code for failure

	CloseHandle( xProcessInfo.hThread );
	CloseHandle( xProcessInfo.hProcess );

	SET_FAILURE_CODE(Tools_ConvertBinkFile_Success);

#undef SET_FAILURE_CODE

	return true;
}