//
// Tools_DialogueEventTab_Event.h
//
// Created by Richard Rice - 30/10/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_DIALOGUEEVENTTAB_EVENT__
#define __TOOLS_DIALOGUEEVENTTAB_EVENT__

#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Sound/Asura_Dialogue.h"
#include "Sound/Tools_DialogueEvent_Overlay.h"



//Stores a copy of most things from Asura_Dialogue_Event for editing by the overlay
class Tools_DialogueEvent
{
public :
	Tools_DialogueEvent();
	~Tools_DialogueEvent();

	void CopyFromAsuraEvent(const class Asura_Dialogue_Event* pxEvent);
	void CopyToAsuraEvent(class Asura_Dialogue_Event* pxEvent) const;
	void CopyMessagesToAsuraEvent(class Asura_Dialogue_Event* pxEvent) const;


	Tools_DialogueEvent& operator = (const Tools_DialogueEvent& xEvent);
	void Copy(const class Asura_Dialogue_Event* pxThisEvent,const Tools_DialogueEvent& xEvent,const bool* pbProperties);

	//bool CanBeEventMapping()  const;


	int				m_iPlaybackType;
	float			m_fReplayDelayTime; // minimum amount of time that must have passed before the dialogue event will accept any more play requests. 
	int				m_iPlayProbability;	// the probability that this event will play at all when triggered. (0-100, defaults to 100).
	int				m_iPriority; // 0 = min priority, 200 = max priority, default 100
	bool			m_bGlobalEvent;
	Asura_Hash_ID	m_uInitialMessage;
	Asura_Hash_ID	m_uFinalMessage;

	u_int			m_uNumPlayable;
	Asura_Dialogue_PlayableIDListItem* m_pxPlayables;

	u_int 			m_uNumMessages;
	Asura_Dialogue_Message** m_ppxMessages;

	Asura_String	m_xFilename;

};


//for handling a single event editing tab
class Tools_DialogueEventTab_Event : public Tools_DialogueEventTab
{
public :
	Tools_DialogueEventTab_Event(class Asura_Tabs_Dialog* pxTabsDialog,Asura_Hash_ID uEventHash);
	~Tools_DialogueEventTab_Event();

	virtual void PerFrameUpdate();
	virtual Asura_Hash_ID GetHashID() {return m_uEventHash;}
	virtual bool IsEvent(){return true;}
	virtual const char* GetName() const;
	virtual const char* GetTabName() const {return m_xTabName.GetString();}
	virtual void UpdateFromAsuraData();
	virtual bool Delete();
	virtual void Copy();
	virtual void Paste();
	virtual void PasteMultiple();
	virtual void Duplicate();
	virtual void Rename();
	virtual bool CanBeClosed() const {return !m_bBeingEdited && !m_bMessageEditing;}

	static bool IsOkayToAddEvent(Asura_Hash_ID uThisEvent,Asura_Hash_ID uEventToAdd);
private :
	void SetModified();

	void AddOverlayControls(class Asura_Tabs_Dialog* pxTabsDialog);
	
	void BuildPlayableList();
	void DeletePlayableList();

	void BuildMessageList();
	void DeleteMessageList();


	void UpdateTabName();

	void UpdatePlayableDataFromToolsEvent();
	void UpdatePlayableDataInToolsEvent();

	Asura_Hash_ID GetSelectedPlayableTemplate() const;
	const Asura_Char* GetSelectedPlayableTemplateName() const;
	
	//overlay callback functions
	//defines for routeing static callback functions to non-static versions
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_DialogueEventTab_Event*)iThis)->__FnName();}\
	void __FnName();

	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_DialogueEventTab_Event*)iThis)->__FnName();}\
	bool __FnName();


	OVERLAY_VOID_CALLBACK(AddEvent)
	OVERLAY_VOID_CALLBACK(AddLine)
	OVERLAY_VOID_CALLBACK(ReplaceLine)
	OVERLAY_VOID_CALLBACK(RemovePlayable)
	OVERLAY_VOID_CALLBACK(MoveUp)
	OVERLAY_VOID_CALLBACK(MoveDown)
	OVERLAY_VOID_CALLBACK(Sort)
	OVERLAY_VOID_CALLBACK(EditPlayable)
	OVERLAY_VOID_CALLBACK(ViewLines)
	OVERLAY_VOID_CALLBACK(PlaySound)
	OVERLAY_VOID_CALLBACK(StopSound)
	OVERLAY_VOID_CALLBACK(AddMessage)
	OVERLAY_VOID_CALLBACK(DeleteMessage)
	OVERLAY_VOID_CALLBACK(EditMessage)
	OVERLAY_VOID_CALLBACK(SetInitialMessage)
	OVERLAY_VOID_CALLBACK(SetFinalMessage)
	OVERLAY_VOID_CALLBACK(SetPlayableStartMessage)
	OVERLAY_VOID_CALLBACK(SetPlayableEndMessage)
	OVERLAY_VOID_CALLBACK(CreateTrigger)
	OVERLAY_VOID_CALLBACK(CreateController)
	OVERLAY_VOID_CALLBACK(EditSelectedPlayable)

	OVERLAY_BOOL_CALLBACK(CanEditGlobal)
	OVERLAY_BOOL_CALLBACK(CanAddEvent)
	OVERLAY_BOOL_CALLBACK(CanAddLine)
	OVERLAY_BOOL_CALLBACK(CanReplaceLine)
	OVERLAY_BOOL_CALLBACK(CanRemovePlayable)
	OVERLAY_BOOL_CALLBACK(CanMoveUp)
	OVERLAY_BOOL_CALLBACK(CanMoveDown)
	OVERLAY_BOOL_CALLBACK(CanSort)
	OVERLAY_BOOL_CALLBACK(CanEditPlayable)
	OVERLAY_BOOL_CALLBACK(CanEditPlayableWeight)
	OVERLAY_BOOL_CALLBACK(CanEditPlayableDelay)
	OVERLAY_BOOL_CALLBACK(CanEditPlayableMessage)
	OVERLAY_BOOL_CALLBACK(CanDeleteMessage)
	OVERLAY_BOOL_CALLBACK(CanEditMessage)


	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK

	void AbortMessageEditing();

	void UpdateInitialMessageDialog();
	void UpdateFinalMessageDialog();
	void UpdatePlayableStartMessageDialog();
	void UpdatePlayableEndMessageDialog();
	void UpdateMessageDialog(Asura_Dialog* pxDialog,const char* szPrefix,Asura_Hash_ID uMessageID);
	Asura_Hash_ID SelectMessageBlock(const char* szTitle,Asura_Hash_ID uInitialSelection);

	/////////////

	Asura_Hash_ID m_uEventHash;
	class Asura_Dialogue_Event* m_pxEvent;

	Tools_DialogueEvent m_xEventData;

	int m_iSelectedPlayable;
	u_int m_uPlayableListSize;
	char** m_pszPlayableList;
	char** m_pszPlayableHelp;
	class Asura_List_Dialog* m_pxPlayableListDialog;

	int m_iSelectedMessage;
	u_int m_uMessageListSize;
	char** m_pszMessageList;
	class Asura_List_Dialog* m_pxMessageListDialog;

	float m_fPlayableDelay;
	u_int m_uPlayableWeight;

	class Asura_Dialog* m_pxInitialMessageDialog;
	class Asura_Dialog* m_pxFinalMessageDialog;
	class Asura_Dialog* m_pxPlayableStartMessageDialog;
	class Asura_Dialog* m_pxPlayableEndMessageDialog;
	class Asura_Function_Dialog* m_pxEditTemplateDialog;

//	bool m_bMinChanged;
//	bool m_bMaxChanged;
	bool m_bFileChanged;
//	bool m_bCategoryChanged;
	bool m_bSelectedPlayableChanged;
	bool m_bChanged;

	bool m_bBeingEdited;
	bool m_bMessageEditing;

	Asura_String m_xTabName;

	static bool s_bPasteBufferValid;
	static Tools_DialogueEvent s_xPasteBuffer;
};

#endif //ASURA_USE_DIALOGUE_SYSTEM
#endif //__TOOLS_DIALOGUEEVENTTAB_EVENT__