//
// Tools_DialogueEventTab_LineTemplate.cpp
//
// Created by Richard Rice - 03/11/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Sound/Tools_DialogueEventTab_LineTemplate.h"

#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Sound/Asura_Dialogue.h"

#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_MessageLog.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Sound/Tools_Dialogue_System.h"
#include "Sound/Tools_SoundMixer_Overlay.h"
#include "Sound/Tools_PickDialogueLineTemplate_Overlay.h"
#include "Sound/Tools_Audio_DSP.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_DialogueLineTemplateProperty_Overlay **
**----------------------------------*/

//Stuff for property copying overlay
enum Tools_DialogueLineTemplate_Properties
{

	Tools_DialogueLineTemplateProp_Mixer,
	Tools_DialogueLineTemplateProp_Gain,
	Tools_DialogueLineTemplateProp_MinGain,
	Tools_DialogueLineTemplateProp_InnerRange,
	Tools_DialogueLineTemplateProp_OuterRange,
	Tools_DialogueLineTemplateProp_Voiceover,
	Tools_DialogueLineTemplateProp_NoPSPWav,
	Tools_DialogueLineTemplateProp_NoSubtitle,
	Tools_DialogueLineTemplateProp_MixerPreset,
	Tools_DialogueLineTemplateProp_Streamed,
	Tools_DialogueLineTemplateProp_SampleRate_PS2,
	Tools_DialogueLineTemplateProp_SampleRate_PSP,

	Tools_DialogueLineTemplateProp_NumProp,
};

static const char* g_aszPropertyNames[Tools_DialogueLineTemplateProp_NumProp] =
{
	
	"Mixer",
	"Gain",
	"Minimum Gain Clamp",
	"Inner Range",
	"Outer Range",
	"Voiceover",
	"No PSP Wav",
	"No Subtitle",
	"Mixer Preset",
	"Streamed",
	"Sample Rate - PS2",
	"Sample Rate - PSP",
};

static const char* g_aszCompressionQualityStrings[] =	// This should match the TOOLS_SOUND_SAMPLE_QUALITY enum
{
	"Perfect",
	"High",
	"Medium",
	"Low",
	"Very Low"
};


//Overlay for selecting properties to paste

class Tools_DialogueLineTemplateProperty_Overlay : public ListSelection_Overlay
{
public :
	Tools_DialogueLineTemplateProperty_Overlay()
	:ListSelection_Overlay("Select properties to paste")
	{
		//initialise the list
		SetSortList(false);

		for(int i=0;i<Tools_DialogueLineTemplateProp_NumProp;i++)
		{
			AddItem(g_aszPropertyNames[i],i);
			SetItemTicked(i,s_bSelected[i]);
		}
	}
	
	virtual bool DoModal()
	{
		if(!ListSelection_Overlay::DoModal()) return false;

		//copy settings to static array
		for(int i=0;i<Tools_DialogueLineTemplateProp_NumProp;i++)
		{
			s_bSelected[i] = GetItemTicked(i);
		}
		return true;
	}

	const bool* GetSelectedArray() const {return s_bSelected;}

protected :
	virtual bool IsTickboxList() const {return true;}
	virtual const char* GetListHeadingText() const {return "Properties :";}
	virtual const char* GetTickAllButtonText() const {return "Select All";}
	virtual const char* GetTickNoneButtonText() const {return "Deselect All";}


	static bool s_bSelected[Tools_DialogueLineTemplateProp_NumProp];
};

bool Tools_DialogueLineTemplateProperty_Overlay::s_bSelected[Tools_DialogueLineTemplateProp_NumProp];

//////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Tools_DialogueLineTemplate **
**---------------------------*/

Tools_DialogueLineTemplate::Tools_DialogueLineTemplate()
{
}

Tools_DialogueLineTemplate::~Tools_DialogueLineTemplate()
{
}

void Tools_DialogueLineTemplate::CopyFromAsuraTemplate(const class Asura_Dialogue_Line_Template* pxTemplate)
{
	m_fGain = pxTemplate->m_fGain;
	m_fInnerRange = pxTemplate->m_fInnerRange;
	m_fOuterRange = pxTemplate->m_fOuterRange;
	m_fMinGain = pxTemplate->m_fMinGain;
	m_bVoiceover = pxTemplate->m_bVoiceover;
	m_bNoWavForPSP = pxTemplate->m_bNoWavForPSP;
	m_bNoSubtitle = pxTemplate->m_bNoSubtitle;
	m_bStreamed = pxTemplate->m_bStreamed;
	m_uMixerPreset = pxTemplate->m_uMixerPreset;

	m_uSampleRateForPS2 = pxTemplate->m_usSampleRateForPS2;
	m_bUseSampleRatePS2 = m_uSampleRateForPS2 != 0;
	if(!m_uSampleRateForPS2) m_uSampleRateForPS2 = Tools_Dialogue_System::GetDefaultSampleRatePS2();

	m_uSampleRateForPSP = pxTemplate->m_usSampleRateForPSP;
	m_bUseSampleRatePSP = m_uSampleRateForPSP != 0;
	if(!m_uSampleRateForPSP) m_uSampleRateForPSP = Tools_Dialogue_System::GetDefaultSampleRatePSP();

	m_uCompressionQuality360 = pxTemplate->m_usCompressionQuality360;
	m_uCompressionQualityPS3 = pxTemplate->m_usCompressionQualityPS3;

	m_xMixingChannelName = pxTemplate->m_xMixingChannelName;

	m_uDSPChain = pxTemplate->m_uDSPChain;
}

void Tools_DialogueLineTemplate::CopyToAsuraTemplate(class Asura_Dialogue_Line_Template* pxTemplate) const
{
	pxTemplate->m_fGain = m_fGain;
	pxTemplate->m_fInnerRange = m_fInnerRange;
	pxTemplate->m_fOuterRange = m_fOuterRange;
	pxTemplate->m_fMinGain = m_fMinGain;
	pxTemplate->m_bVoiceover = m_bVoiceover;
	pxTemplate->m_bNoWavForPSP = m_bNoWavForPSP;
	pxTemplate->m_bNoSubtitle = m_bNoSubtitle;
	pxTemplate->m_bStreamed = m_bStreamed;
	pxTemplate->m_uMixerPreset = m_uMixerPreset;

	pxTemplate->m_usSampleRateForPS2 = m_bUseSampleRatePS2 ? (u_short)m_uSampleRateForPS2 : 0;
	pxTemplate->m_usSampleRateForPSP = m_bUseSampleRatePSP ? (u_short)m_uSampleRateForPSP : 0;

	pxTemplate->m_usCompressionQualityPS3 = static_cast<u_short>( m_uCompressionQualityPS3 );
	pxTemplate->m_usCompressionQuality360 = static_cast<u_short>( m_uCompressionQuality360 );

	pxTemplate->m_xMixingChannelName = m_xMixingChannelName;

	pxTemplate->m_uDSPChain = m_uDSPChain;
}

Tools_DialogueLineTemplate& Tools_DialogueLineTemplate::operator = (const Tools_DialogueLineTemplate& xTemplate)
{
	if(this != &xTemplate) 
	{
		m_fGain = xTemplate.m_fGain;
		m_fInnerRange = xTemplate.m_fInnerRange;
		m_fOuterRange = xTemplate.m_fOuterRange;
		m_fMinGain = xTemplate.m_fMinGain;
		m_bVoiceover = xTemplate.m_bVoiceover;
		m_bNoWavForPSP = xTemplate.m_bNoWavForPSP;
		m_bNoSubtitle = xTemplate.m_bNoSubtitle;
		m_uMixerPreset = xTemplate.m_uMixerPreset;
		m_bStreamed = xTemplate.m_bStreamed;
		m_uSampleRateForPS2 = xTemplate.m_uSampleRateForPS2;
		m_uSampleRateForPSP = xTemplate.m_uSampleRateForPSP;
		m_bUseSampleRatePS2 = xTemplate.m_bUseSampleRatePS2;
		m_bUseSampleRatePSP = xTemplate.m_bUseSampleRatePSP;
		m_xMixingChannelName = xTemplate.m_xMixingChannelName;
		m_uCompressionQualityPS3 = xTemplate.m_uCompressionQualityPS3;
		m_uCompressionQuality360 = xTemplate.m_uCompressionQuality360;
		m_uDSPChain = xTemplate.m_uDSPChain;
	}
	return *this;
}

void Tools_DialogueLineTemplate::Copy(const Tools_DialogueLineTemplate& xTemplateToCopy,const bool* pbProperties)
{
	if(this == &xTemplateToCopy) return;


	if(pbProperties[Tools_DialogueLineTemplateProp_Mixer])
	{
		m_xMixingChannelName = xTemplateToCopy.m_xMixingChannelName;
	}
	if(pbProperties[Tools_DialogueLineTemplateProp_Gain])
	{
		m_fGain = xTemplateToCopy.m_fGain;
	}
	if(pbProperties[Tools_DialogueLineTemplateProp_MinGain])
	{
		m_fMinGain = xTemplateToCopy.m_fMinGain;
	}
	if(pbProperties[Tools_DialogueLineTemplateProp_InnerRange])
	{
		m_fInnerRange = xTemplateToCopy.m_fInnerRange;
	}
	if(pbProperties[Tools_DialogueLineTemplateProp_OuterRange])
	{
		m_fOuterRange = xTemplateToCopy.m_fOuterRange;
	}
	if(pbProperties[Tools_DialogueLineTemplateProp_Voiceover])
	{
		m_bVoiceover = xTemplateToCopy.m_bVoiceover;
	}
	if(pbProperties[Tools_DialogueLineTemplateProp_NoPSPWav])
	{
		m_bNoWavForPSP = xTemplateToCopy.m_bNoWavForPSP;
	}
	if(pbProperties[Tools_DialogueLineTemplateProp_NoSubtitle])
	{
		m_bNoSubtitle = xTemplateToCopy.m_bNoSubtitle;
	}
	if (pbProperties[Tools_DialogueLineTemplateProp_MixerPreset])
	{
		m_uMixerPreset = xTemplateToCopy.m_uMixerPreset;
	}
	if(pbProperties[Tools_DialogueLineTemplateProp_Streamed])
	{
		m_bStreamed = xTemplateToCopy.m_bStreamed;
	}
	if(pbProperties[Tools_DialogueLineTemplateProp_SampleRate_PS2])
	{
		m_bUseSampleRatePS2 = xTemplateToCopy.m_bUseSampleRatePS2;
		m_uSampleRateForPS2 = xTemplateToCopy.m_uSampleRateForPS2;
	}
	if(pbProperties[Tools_DialogueLineTemplateProp_SampleRate_PSP])
	{
		m_bUseSampleRatePSP = xTemplateToCopy.m_bUseSampleRatePSP;
		m_uSampleRateForPSP = xTemplateToCopy.m_uSampleRateForPSP;
	}
	 
}




//////////////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------------**
** Tools_DialogueEventTab_LineTemplate **
**-------------------------------------*/

bool Tools_DialogueEventTab_LineTemplate::s_bPasteBufferValid = false;
Tools_DialogueLineTemplate Tools_DialogueEventTab_LineTemplate::s_xPasteBuffer;

Tools_DialogueEventTab_LineTemplate::Tools_DialogueEventTab_LineTemplate(Asura_Tabs_Dialog* pxTabsDialog,Asura_Hash_ID uTemplateHash)
{
	m_uTemplateHash = uTemplateHash;
	m_pxTemplate = Asura_Dialogue_Storage::GetLineTemplateByHash(uTemplateHash);
	Asura_Assert(m_pxTemplate,"");

	m_xTemplateData.CopyFromAsuraTemplate(m_pxTemplate);
	m_bChanged = false;
	m_bMinChanged = false;
	m_bMaxChanged = false;

	m_bBeingEdited = false;

	AddOverlayControls(pxTabsDialog);

	UpdatePresetName();
}

void Tools_DialogueEventTab_LineTemplate::PerFrameUpdate()
{
	if(m_bMinChanged)
	{
		m_xTemplateData.m_fGain = Asura_Maths::Max(m_xTemplateData.m_fGain,m_xTemplateData.m_fMinGain);
		m_xTemplateData.m_fOuterRange = Asura_Maths::Max(m_xTemplateData.m_fOuterRange,m_xTemplateData.m_fInnerRange);

		m_bMinChanged = false;
		m_bChanged = true;
	}
	if(m_bMaxChanged)
	{
		m_xTemplateData.m_fMinGain = Asura_Maths::Min(m_xTemplateData.m_fGain,m_xTemplateData.m_fMinGain);
		m_xTemplateData.m_fInnerRange = Asura_Maths::Min(m_xTemplateData.m_fOuterRange,m_xTemplateData.m_fInnerRange);

		m_bMaxChanged = false;
		m_bChanged = true;
	}

	if(m_bChanged)
	{
		//update the asura data
		m_xTemplateData.CopyToAsuraTemplate(m_pxTemplate);
		SetModified();

		m_bChanged = false;
	}
}

Tools_DialogueEventTab_LineTemplate::~Tools_DialogueEventTab_LineTemplate()
{
}



void Tools_DialogueEventTab_LineTemplate::AddOverlayControls(Asura_Tabs_Dialog* pxTabsDialog)
{
	//create new tab on the tab dialog
	UpdateTabName();
	pxTabsDialog->AddTab(m_xTabName.GetString());

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = pxTabsDialog->GetTopDialogPosition();


	pxDialog = new Tools_SoundMixer_Dialog(xOffset,&m_xTemplateData.m_xMixingChannelName,&m_bChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Gain",xOffset,&m_xTemplateData.m_fGain,0.01f,1.5f,0.01f,"%0.2f",&m_bMaxChanged);
	pxDialog->SetHelpText("The gain (volume) of the dialogue lines.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Minimum Gain Clamp",xOffset,&m_xTemplateData.m_fMinGain,0.01f,1.5f,0.01f,"%0.2f",&m_bMinChanged);
	pxDialog->SetHelpText("The gain below which the gain of the audio for the line must never go. This means that the audio can be heard anywhere.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Numerical_Dialog("Inner Range",xOffset,&m_xTemplateData.m_fInnerRange, 0.5f,9999.0f,0.1f,"%0.1f",&m_bMinChanged);
	pxDialog->SetHelpText("The inner range of the sound.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Outer Range",xOffset,&m_xTemplateData.m_fOuterRange, 0.5f,9999.0f,0.1f,"%0.1f",&m_bMaxChanged);
	pxDialog->SetHelpText("The outer range of the sound.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Voiceover",xOffset,&m_xTemplateData.m_bVoiceover,&m_bChanged);
	pxDialog->SetHelpText("If this is ticked then audio will be delivered direct to speakers as voiceover.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Tickbox_Dialog("No PSP Wav",xOffset,&m_xTemplateData.m_bNoWavForPSP,&m_bChanged);
	pxDialog->SetHelpText("If this is ticked then the audio for the line is not exported to the PSP and the line is only played as a subtitle.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("No Subtitle",xOffset,&m_xTemplateData.m_bNoSubtitle,&m_bChanged);
	pxDialog->SetHelpText("If this is ticked then the subtitles won't be displayed.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Streamed",xOffset,&m_xTemplateData.m_bStreamed,&m_bChanged);
	pxDialog->SetHelpText("If this is ticked then the dialogue will be streamed (if the target platform supports it).");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	
	//tickbox for using sample rate
	pxDialog = new Asura_Tickbox_Dialog("",xOffset,&m_xTemplateData.m_bUseSampleRatePS2,&m_bChanged);
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);

	//sample rate value
	xOffset.x += 3.0f;
	pxDialog = new Asura_Numerical_Dialog("PS2 Sample Rate",xOffset,&m_xTemplateData.m_uSampleRateForPS2, 1,99999,1,"%d",&m_bChanged);
	pxDialog->SetIsEnabledFunction(IsEnabled_SampleRatePS2,(int)this);
	pxDialog->SetHelpText("Maximum sample rate to use when exporting this sound.");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x -= 3.0f;

	//tickbox for using sample rate
	pxDialog = new Asura_Tickbox_Dialog("",xOffset,&m_xTemplateData.m_bUseSampleRatePSP,&m_bChanged);
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);

	//sample rate value
	xOffset.x += 3.0f;
	pxDialog = new Asura_Numerical_Dialog("PSP Sample Rate",xOffset,&m_xTemplateData.m_uSampleRateForPSP, 1,99999,1,"%d",&m_bChanged);
	pxDialog->SetIsEnabledFunction(IsEnabled_SampleRatePSP,(int)this);
	pxDialog->SetHelpText("Maximum sample rate to use when exporting this sound.");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x -= 3.0f;

	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Compression quality
	pxDialog = new Asura_Dialog("Compression quality :",xOffset);
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.x += 3.0f;
	pxDialog = new Asura_DropDown_List_Dialog	( xOffset
												, g_aszCompressionQualityStrings
												, &m_xTemplateData.m_uCompressionQualityPS3
												, TSS_QUALITY_NUM_ENTRIES
												, TSS_QUALITY_NUM_ENTRIES
												, 0
												, &m_bChanged
												, "PS3" );

	pxDialog->SetHelpText("Quality setting when compressing this sound during export");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_DropDown_List_Dialog	( xOffset
												, g_aszCompressionQualityStrings
												, &m_xTemplateData.m_uCompressionQuality360
												, TSS_QUALITY_NUM_ENTRIES
												, TSS_QUALITY_NUM_ENTRIES
												, 0
												, &m_bChanged
												, "Xbox 360" );

	pxDialog->SetHelpText("Quality setting when compressing this sound during export");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x -= 3.0f;
	
	//mixer preset
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Mixer Preset", xOffset, SelectMixerPreset, (int)this, NULL, NULL, false );
	pxDialog->SetHelpText("Assigns a mixer preset to be queued when a line from the template is active");
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Dynamic_Dialog( &m_xPresetName, xOffset );
	pxTabsDialog->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxTabsDialog, Asura_Tabs_Dialog::s_uTopLeftPositionID);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	#ifdef ASURA_USE_AUDIO_SYSTEM
	// DSP chain selection
	pxDialog = new Tools_DSPChain_DropDownList_Dialog( xOffset, 10, &m_xTemplateData.m_uDSPChain, &m_bChanged, "DSP Chain" );
	pxDialog->SetHelpText( "Set the DSP chain to apply to dialogue played through this template");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	#endif
	
}

const char* Tools_DialogueEventTab_LineTemplate::GetName() const
{
	return m_pxTemplate->m_xName.GetString();

}

void Tools_DialogueEventTab_LineTemplate::UpdateFromAsuraData()
{
	m_xTemplateData.CopyFromAsuraTemplate(m_pxTemplate);
}

bool Tools_DialogueEventTab_LineTemplate::Delete()
{
	return Tools_Dialogue_System::DeleteLineTemplate(m_uTemplateHash,true);
}

void Tools_DialogueEventTab_LineTemplate::Copy()
{
	s_xPasteBuffer = m_xTemplateData;
	s_bPasteBufferValid = true;
	Tools_Timed_Message_System::AddMessage(0xffffff,"%s copied.",m_pxTemplate->m_xName.GetString());
}

void Tools_DialogueEventTab_LineTemplate::Paste()
{
	if(s_bPasteBufferValid)
	{
	
		m_xTemplateData = s_xPasteBuffer;
		m_bChanged = true;
		Tools_Timed_Message_System::AddMessage(0xffffff,"Line template pasted.");
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Line template paste buffer is empty.");
	}
}


void Tools_DialogueEventTab_LineTemplate::PasteMultiple()
{

	#if 1
	//Get a list of templates to paste over
	m_bBeingEdited = true;
	Tools_PickDialogueLineTemplate_Overlay::PickLineTemplate("Paste over which templates?",0,true);
	m_bBeingEdited = false;


	u_int uNumSelected = Tools_PickDialogueLineTemplate_Overlay::GetNumSelected();
	if(!uNumSelected) return;

	//Determine which properties should be pasted
	Tools_DialogueLineTemplateProperty_Overlay xPropertyOverlay;
	if(!xPropertyOverlay.DoModal()) return;

	//Do the copying
	for(u_int i=0;i<uNumSelected;i++)
	{
		Asura_Hash_ID uTemplateHash = Tools_PickDialogueLineTemplate_Overlay::GetSelection(i);
		Asura_Dialogue_Line_Template* pxTemplate = Asura_Dialogue_Storage::GetLineTemplateByHash(uTemplateHash);
		if(!pxTemplate) continue;

		//get a tools copy of the template properties
		Tools_DialogueLineTemplate xToolsTemplate;
		xToolsTemplate.CopyFromAsuraTemplate(pxTemplate);

		//Do the copy
		xToolsTemplate.Copy(m_xTemplateData,xPropertyOverlay.GetSelectedArray());

		//Update the asura template and flag as modified
		xToolsTemplate.CopyToAsuraTemplate(pxTemplate);
		Tools_Dialogue_System::SetAudioTemplatesModified();

		//Make sure any tab with this template is updated
		Tools_DialogueEvent_Overlay::UpdateLineTemplateFromAsuraData(uTemplateHash);

	}

	#endif

}


void Tools_DialogueEventTab_LineTemplate::Duplicate()
{
	//get a default name for the template
	//get current template name without any numbers at the end
	char szBaseName[100];
	Asura_PrintfToBfr(szBaseName,"%s",sizeof(szBaseName),true,m_pxTemplate->m_xName.GetString());
	int iLength = strlen(szBaseName);
	while(iLength>0 && szBaseName[iLength-1]>='0' && szBaseName[iLength-1]<='9') iLength--;
	szBaseName[iLength] = 0;
	

	char szTemplateName[200];
	for(int iIndex=0;;iIndex++)
	{
		Asura_PrintfToBfr(szTemplateName,"%s%d",200,false,szBaseName,iIndex);

		if(!Asura_Dialogue_Storage::GetLineTemplateByHash(Asura_GetHashID(szTemplateName))) break;
	}

	
	
	//ask for a name  , offering the default name
	GetString_Overlay xStringOverlay("Name new template",szTemplateName);
	do
	{
		if(!xStringOverlay.DoModal()) break;
		
		const char* szTemplateName = xStringOverlay.GetString();
		if(!szTemplateName || !*szTemplateName) break;

		//make sure this template doesn't already exist
		Asura_Hash_ID uTemplateHash = Asura_GetHashID(szTemplateName);
		if(Asura_Dialogue_Storage::GetLineTemplateByHash(uTemplateHash))
		{
			ShowMessageOverlay("A template already exists using the same hash ID.");
			continue;
		}

		bool bAlreadyExists;
		Asura_Dialogue_Line_Template* pxTemplate = Asura_Dialogue_Storage::CreateLineTemplate(szTemplateName,bAlreadyExists);
		Asura_Assert(pxTemplate && !bAlreadyExists, "Error creating line template");

		//copy this template's properties to the new one
		m_xTemplateData.CopyToAsuraTemplate(pxTemplate);
		Tools_Dialogue_System::SetAudioTemplatesModified();
		//and edit it
		Tools_DialogueEvent_Overlay::EditLineTemplate(uTemplateHash);
		break;
		
	}while(true);
}

void Tools_DialogueEventTab_LineTemplate::Rename()
{
	const char* szNewName = 0;
	Asura_Hash_ID uOldHash = m_uTemplateHash;
	Asura_Hash_ID uNewHash = 0;
	bool bOkay = false;
	
	//ask for new name
	GetString_Overlay xStringOverlay("Rename Template",m_pxTemplate->m_xName.GetString());
	while(xStringOverlay.DoModal())
	{
		szNewName = xStringOverlay.GetString();
		if(!szNewName || !*szNewName) return;

		if(!strcmp(szNewName,m_pxTemplate->m_xName.GetString()))
		{
			//name not changed
			return;
		}

		//make sure no existing templates exists with this hash
		uNewHash = Asura_GetHashID(szNewName);

		if(uOldHash != uNewHash && Asura_Dialogue_Storage::GetLineTemplateByHash(uNewHash))
		{
			ShowMessageOverlay("A template already exists using the same hash ID.");
			continue;
		}

		bOkay = true;
		break;
	}
	if(!bOkay) return;

	//Update the template and its position in the storage system
	if(!Asura_Dialogue_Storage::RenameLineTemplate(m_pxTemplate->m_xName.GetString(),szNewName))
	{
		Asura_Assert(0,"Error renaming template");
		return;
	}
	SetModified();

	//update this tab
	m_uTemplateHash = uNewHash;
	UpdateTabName();

	//Update dialogue lines
	Tools_Dialogue_System::UpdateReferencesToTemplates();
}

void Tools_DialogueEventTab_LineTemplate::UpdateTabName()
{
	char szTabName[MAX_TAB_NAME_LENGTH+2];
	Asura_PrintfToBfr(szTabName,"T: %s",sizeof(szTabName),true,m_pxTemplate->m_xName.GetString());
	if(strlen(szTabName) > MAX_TAB_NAME_LENGTH)
	{
		//name is too long for tab , so truncate it
		strcpy(&szTabName[MAX_TAB_NAME_LENGTH-3],"...");
	}

	m_xTabName = szTabName;
}

void Tools_DialogueEventTab_LineTemplate::SetModified()
{
	Tools_Dialogue_System::SetAudioTemplatesModified();
}

bool Tools_DialogueEventTab_LineTemplate::IsEnabled_SampleRatePS2()
{
	return m_xTemplateData.m_bUseSampleRatePS2;
}

bool Tools_DialogueEventTab_LineTemplate::IsEnabled_SampleRatePSP()
{
	return m_xTemplateData.m_bUseSampleRatePSP;
}

void Tools_DialogueEventTab_LineTemplate::SelectMixerPreset()
{
	// get the name of the preset to change
	ListSelection_Overlay xOverlay("Select a Preset");

	xOverlay.SetSortList(false);

	xOverlay.AddItem("None", ASURA_HASH_ID_UNSET);

	for(u_int i=0;i<Asura_Sound_Mixer_System::GetNumberOfPresets();i++)
	{
		const Asura_Sound_Mixer_Preset* pxPreset = Asura_Sound_Mixer_System::GetPresetByIndex(i);
		xOverlay.AddItem(pxPreset->GetName(),pxPreset->GetHashID());
	}

	xOverlay.SetInitialSelection( m_xTemplateData.m_uMixerPreset );

	if(!xOverlay.DoModal()) return;

	m_xTemplateData.m_uMixerPreset = xOverlay.GetSelectionData();
	
	UpdatePresetName();

	m_bChanged = true;
}

void Tools_DialogueEventTab_LineTemplate::UpdatePresetName()
{
	m_xPresetName = "Selected Preset: ";

	if (m_xTemplateData.m_uMixerPreset != ASURA_HASH_ID_UNSET)
	{
		const Asura_Sound_Mixer_Preset* pxPreset = Asura_Sound_Mixer_System::FindPreset( m_xTemplateData.m_uMixerPreset );
		if (pxPreset)
		{
			m_xPresetName += pxPreset->GetName();
		}
		else
		{
			m_xPresetName += "Missing";
		}
	}
	else
	{
		m_xPresetName += "None";
	}
}

#endif //ASURA_USE_DIALOGUE_SYSTEM
