//
// Tools_DialogueEvent_Dialogs.h
//
// Created by Richard Rice - 01/11/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Sound/Tools_DialogueEvent_Dialogs.h"

#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Sound/Asura_Dialogue.h"
#include "Sound/Asura_Dialogue_System.h"

#include "Sound/Tools_Dialogue_System.h"
#include "Sound/Tools_PickDialogueEvent_Overlay.h"
#include "Sound/Tools_DialogueEvent_Overlay.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------**
** Tools_DialogueCharacter_Dialog **
**--------------------------------*/

u_int Tools_DialogueCharacter_Dialog::GetNumberOfHashValues() const
{
	return Tools_Dialogue_System::GetNumberOfCharacters();
}

const char* Tools_DialogueCharacter_Dialog::GetNameByIndex(int iIndex) const
{
	return Tools_Dialogue_System::GetCharacterNameByIndex(iIndex);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------**
** Tools_PickDialogueEvent_Dialog **
**--------------------------------*/

Tools_PickDialogueEvent_Dialog::Tools_PickDialogueEvent_Dialog(const char* szPrefix,const Asura_Vector_2& xOffset,Asura_Hash_ID* puEventHash,bool* pbModified /*= 0*/)
:Asura_Function_Dialog(szPrefix,xOffset,OnClicked,0)
{
	m_xPrefix = szPrefix;
	m_puEventHash = puEventHash;
	m_uEventBackup = *puEventHash;
	m_pbModified = pbModified;

	SetCallback(OnClicked,(int)this);

	UpdateEventLabel();
}

void Tools_PickDialogueEvent_Dialog::PerFrameUpdate()
{
	Asura_Function_Dialog::PerFrameUpdate();

	//update the label if the event hash has changed externally
	if(m_uEventBackup != *m_puEventHash)
	{
		m_uEventBackup = *m_puEventHash;
		UpdateEventLabel();
	}
}

void Tools_PickDialogueEvent_Dialog::UpdateEventLabel()
{
	char szText[100];
	if(*m_puEventHash == 0)
	{
		Asura_PrintfToBfr(szText,"%s*None*",sizeof(szText),true,m_xPrefix.GetString());
	}
	else
	{
		const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_System::FindEvent(*m_puEventHash);
		if(pxEvent)
		{
			Asura_PrintfToBfr(szText,"%s%s",sizeof(szText),true,m_xPrefix.GetString(),pxEvent->GetName());
		}
		else
		{
			Asura_PrintfToBfr(szText,"%s*Unknown* (0x%x)",sizeof(szText),true,m_xPrefix.GetString(),*m_puEventHash);
		}
	}
	if(m_pxOverlay)
	{
		m_pxOverlay->UpdateDialogLabel(this,szText);
	}
	else
	{
		UpdateLabel(szText);
	}
}




void Tools_PickDialogueEvent_Dialog::OnClicked()
{
	//show the event picking overlay
	Asura_Hash_ID uHash =  Tools_PickDialogueEvent_Overlay::PickEvent("Select dialogue event", *m_puEventHash);

	if(uHash != *m_puEventHash)
	{
		*m_puEventHash = uHash;
		m_uEventBackup = uHash;
		if(m_pbModified) *m_pbModified = true;
		UpdateEventLabel();
	}

}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------**
** Tools_EditDialogueEvent_Dialog **
**--------------------------------*/
	 
Tools_EditDialogueEvent_Dialog::Tools_EditDialogueEvent_Dialog(const char* szName,const Asura_Vector_2& xOffset,Asura_Hash_ID* puEventHash)
:Asura_Function_Dialog(szName,xOffset,OnClicked,0)
{
	m_puEventHash = puEventHash;

	//The callback user data is set here , rather than in the Asura_Function_Dialog constructor ,to avoid a warning
	SetCallback(OnClicked,(int)this);

	//set some default help text
	SetHelpText("Edit the selected dialogue event.");
}

void Tools_EditDialogueEvent_Dialog::PerFrameUpdate()
{
	Asura_Function_Dialog::PerFrameUpdate();

	//disable control if no valid event has been selected
	SetEnabled(Asura_Dialogue_Storage::FindEvent(*m_puEventHash)!=0);
}



void Tools_EditDialogueEvent_Dialog::OnClicked()
{
	Tools_DialogueEvent_Overlay::EditEvent(*m_puEventHash);
}


#endif //ASURA_USE_DIALOGUE_SYSTEM
