//
// Tools_DialogueEvent_Overlay.h
//
// Created by Richard Rice - 30/10/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif					 

#ifndef __TOOLS_DIALOGUEEVENT_OVERLAY__
#define __TOOLS_DIALOGUEEVENT_OVERLAY__

#ifdef ASURA_USE_DIALOGUE_SYSTEM

#include "Core/Asura_Hash.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Sound/Asura_Sound_Event.h"

#include "Misc/list_tem.hpp"

class Tools_DialogueEventTab;

class Tools_DialogueEvent_Overlay : public Asura_Overlay
{
friend class Tools_Dialogue_System;

private :
	Tools_DialogueEvent_Overlay(int iIndex);
	~Tools_DialogueEvent_Overlay();

public :
	//Opens tab for editing an event
	static bool EditEvent(Asura_Hash_ID uEventHash);
	static bool EditLineTemplate(Asura_Hash_ID uLineTemplateHash);

	//Close tab for event. Returns false if not allowed to close it
	static bool CloseEventTab(Asura_Hash_ID uEventHash);
	static bool CloseLineTemplateTab(Asura_Hash_ID uTemplateHash);

	//Closes overlay if open
	static void CloseOverlay();

	//Functions used by the tabs
	static void PlayDialogue(Asura_Hash_ID uEventHash);
	static void StopSound();
	static void UpdateEventFromAsuraData(Asura_Hash_ID uEventHash);
	static void UpdateLineTemplateFromAsuraData(Asura_Hash_ID uTemplateHash);



private :
	void PerFrameUpdate(void);

	void ShowOverlay();
	void EditEventInternal(Asura_Hash_ID uEventHash);
	void EditLineTemplateInternal(Asura_Hash_ID uTemplateHash);

	void CloseTabsIfTooLarge(u_int uTabToPreserve1,u_int uTabToPreserve2);

	static Tools_DialogueEvent_Overlay* GetEventOverlay(bool bCreate);
	static Tools_DialogueEvent_Overlay* GetLineTemplateOverlay(bool bCreate);

	//loads/saves data for this mode
	static void TransferModeData(int iMode);

	static void UpdateSound();

	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_DialogueEvent_Overlay*)iThis)->__FnName();}\
	void __FnName();

	OVERLAY_VOID_CALLBACK(CloseTab);
	OVERLAY_VOID_CALLBACK(Delete);
	OVERLAY_VOID_CALLBACK(Copy);
	OVERLAY_VOID_CALLBACK(Paste);
	OVERLAY_VOID_CALLBACK(PasteMultiple);
	OVERLAY_VOID_CALLBACK(Duplicate);
	OVERLAY_VOID_CALLBACK(Rename);
	OVERLAY_VOID_CALLBACK(CreateEvent);
	OVERLAY_VOID_CALLBACK(CreateCodeEvent);

	#undef OVERLAY_VOID_CALLBACK

	void CloseTabMain(u_int uTab);

	static Tools_DialogueEvent_Overlay* s_apxThis[2];
	static int s_aiOverlayID[2];

	class Asura_Tabs_Dialog* m_pxTabsDialog;
	int m_iOverlayIndex;

	static Asura_Handle s_iDialogueHandle;


	//list of tabs on the overlay
	List<Tools_DialogueEventTab*> m_xTabList;

public :
	static Asura_Vector_2 s_axOverlayPosition[2];
	static bool s_bCombineOverlays;

};

////////////////////////////////////////////////////////////////////////////////////////////

//base class for the sample and event editing tabs
class Tools_DialogueEventTab
{
public :
	virtual ~Tools_DialogueEventTab(){};
	virtual void PerFrameUpdate(){};

	virtual Asura_Hash_ID GetHashID() = 0;
	virtual bool IsEvent(){return false;}
	virtual bool IsLineTemplate(){return false;}

	virtual const char* GetName() const = 0;
	virtual const char* GetTabName() const = 0;

	//A request to delete the asura data associated with the tab (but not the tab itself)
	virtual bool Delete() {return false;}

	//This gets called to test which tabs can be safely closed when there are too many open
	virtual bool CanBeClosed() const {return true;}

	//updates data in tab from the asura data
	virtual void UpdateFromAsuraData() {};

	virtual void Copy(){};
	virtual void Paste(){};
	virtual void PasteMultiple(){};
	virtual void Duplicate(){};
	virtual void Rename(){};

protected :
	enum
	{
		MAX_TAB_NAME_LENGTH = 25
	};
};

////////////////////////////////////////////////////////////////////////////////////////////

//drop down list for selecting event/sample file
class Tools_DialogueFilename_Dialog : public Asura_DropDown_List_Dialog
{
public :
	Tools_DialogueFilename_Dialog(const Asura_Vector_2& xOffset,Asura_String* pxFilename,bool* pbModified);
	~Tools_DialogueFilename_Dialog();


private:
	void PerFrameUpdate();

	static void OnSelectionChanged(int iThis){((Tools_DialogueFilename_Dialog*)iThis)->OnSelectionChanged();}
	void OnSelectionChanged();

	void UpdateFilenameList();
	void DeleteFilenameList();

	Asura_String* m_pxFilename;
	bool* m_pbModified;

	Asura_String m_xFilenameBackup;
	bool m_bSelectionChanged;

	int m_iSelectedIndex;
	bool m_bInitialised;

	u_int m_uFilenameListSize;
	const char** m_pszFilenameList;
	class Asura_NameTree* m_pxFilenameTree;

	//for determining validity of current list
	int m_iListValidID;
	static int s_iListValidID;

};




#endif //ASURA_USE_DIALOGUE_SYSTEM

#endif //__TOOLS_DIALOGUEEVENT_OVERLAY__
