// Tools_Overlay_DynamicMusic.cpp
// Thomas Young - 13.08.08
// Copyright (C) 2008 Rebellion, All Rights Reserved.

#ifdef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

// I N C L U D E S ///////////////////////////////////////////////////////////

#include "Core/Asura.h"											//Main Asura header
#include "Tools_Overlay_DynamicMusic.h"							//My header

#include "Core/Asura_Hash.h"									//Other Asura headers
#include "Core/Asura_NameTree.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_WAVAlign_Dialog.h"
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_Dynamic_Music_System.h"

#include "Entity/Tools_Entity_DynamicMusicTrigger_Object.h"		//Tools Headers
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Entity_Object.h"					
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Entity/Triggers/Asura_ServerEntity_DynamicMusicTrigger.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Movement/Tools_Movement.h"
#include "Objects/Tools_Object.h"
#include "Objects/Tools_Object_System.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Sound/Tools_Sound_System.h"
#include "Tools_Overlays.h"

// C O N S T A N T S /////////////////////////////////////////////////////////

static const u_int uTOOLS_TRIGGER_ENTITY_LIGHT_GREEN = 0x00D6FDB3;

// G L O B A L   V A R I A B L E S ///////////////////////////////////////////

static const int	iMAX_TRIGGER_SUFFIX_LENGTH	=	16;
static const char*	g_aszTriggerSuffix[]		=	{	" (Transition)"		,
														" (Play)"			,
														" (Stop)"			,
														" (Join/Release)"	};

static const char* const g_aszTriggerTypes[] =	//This should have DYNAMIC_MUSIC_NUM_TRIGGER_TYPES entries
{
	"Transition",
	"Play Track",
	"Stop Playback",
	"Join/Release"
};

static const char* const g_aszJOINREL_ActionTypes[] =
{
	"No change",
	"Join",
	"Release"
};

static u_int s_uTriggerListSelectedTab = 0;

// S T A T I C   M E M B E R   V A R I A B L E S /////////////////////////////

bool						Tools_Overlay_DynamicMusic::s_bMainOverlayOpen				= false;
Asura_Vector_2				Tools_Overlay_DynamicMusic::s_axOverlayPositions			[DM_NUM_OVERLAYS];
bool						Tools_Overlay_DynamicMusic::s_bIgnore						= false;
bool						Tools_Overlay_DynamicMusic::s_bOverlayShouldBeOpen			= false;
Asura_Dialog*				Tools_Overlay_DynamicMusic::s_apxLayerNameButtons			[ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS];

int							Tools_Overlay_DynamicMusic::s_iTrackListArraySize			= 0;
char**						Tools_Overlay_DynamicMusic::s_ppszTrackListNames			= 0;

int							Tools_Overlay_DynamicMusic::s_iTrackListSelectionIndex		= 0;
bool						Tools_Overlay_DynamicMusic::s_bTrackListSelectionChanged	= 0;

bool						Tools_Overlay_DynamicMusic::s_bTrackModified				= false;
bool						Tools_Overlay_DynamicMusic::s_bForceTrackOverlayOpen		= false;
int							Tools_Overlay_DynamicMusic::s_iIntroEnd_Numeric				= 0;
int							Tools_Overlay_DynamicMusic::s_iOutroStart_Numeric			= 0;
bool						Tools_Overlay_DynamicMusic::s_bChangedIntroOutro_Numeric	= false;
Asura_Dialog*				Tools_Overlay_DynamicMusic::s_apxLayerFilesDialogs			[ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS];
Asura_Dialog*				Tools_Overlay_DynamicMusic::s_apxLayerClearDialogs			[ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS];
Asura_Numerical_Dialog*		Tools_Overlay_DynamicMusic::s_pxIntroOutroDialogs			[2];
Asura_WAVAlign_Dialog*		Tools_Overlay_DynamicMusic::s_pxWAVAlignDialog				= 0;

char**							Tools_Overlay_DynamicMusic::s_ppszEntityTriggerListNames		= 0;
int								Tools_Overlay_DynamicMusic::s_iEntityTriggerListSelectionIndex	= 0;
Asura_HeapArray<Asura_Hash_ID>	Tools_Overlay_DynamicMusic::s_xEntityTriggerHashList;

char**							Tools_Overlay_DynamicMusic::s_ppszCodeTriggerListNames			= 0;
int								Tools_Overlay_DynamicMusic::s_iCodeTriggerListSelectionIndex	= 0;
Asura_HeapArray<Asura_Hash_ID>	Tools_Overlay_DynamicMusic::s_xCodeTriggerHashList;

Asura_Hash_ID					Tools_Overlay_DynamicMusic::s_uSelectedTriggerHash = ASURA_HASH_ID_UNSET;

Asura_Dialog*					Tools_Overlay_DynamicMusic::s_pxTRANS_TrackNameDialog = NULL;
Asura_Dialog*					Tools_Overlay_DynamicMusic::s_pxTRANS_UseIntroTick = NULL;
Asura_Channel_Graph_Dialog*		Tools_Overlay_DynamicMusic::s_pxTRANS_ChannelDialog = NULL;
bool							Tools_Overlay_DynamicMusic::s_bTRANS_HasTrackToTransition = false;
bool							Tools_Overlay_DynamicMusic::s_bTRANS_UseIntroTick = false;
Asura_Channel_Graph_Channel*	Tools_Overlay_DynamicMusic::s_apxTRANS_Channels	[NUM_CHANNELS];
Asura_Channel_Graph_Block*		Tools_Overlay_DynamicMusic::s_apxTRANS_ChannelBlocks [NUM_CHANNEL_BLOCKS];
bool							Tools_Overlay_DynamicMusic::s_bTRANS_ChannelDialogModified = false;

Asura_Channel_Graph_Dialog*		Tools_Overlay_DynamicMusic::s_pxSTOP_ChannelDialog = NULL;
Asura_Channel_Graph_Block*		Tools_Overlay_DynamicMusic::s_pxSTOP_ChannelDialogOutroBlock = NULL;
bool							Tools_Overlay_DynamicMusic::s_bSTOP_ChannelDialogModified = false;
bool							Tools_Overlay_DynamicMusic::s_bSTOP_JustOutroTick = false;

Asura_Channel_Graph_Dialog*		Tools_Overlay_DynamicMusic::s_apxJOINREL_ChannelDialog [ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS];
bool							Tools_Overlay_DynamicMusic::s_abJOINREL_ChannelDialogModified[ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS];

int								Tools_Overlay_DynamicMusic::s_aiJOINREL_ActionTypeIndex	[ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS];
bool							Tools_Overlay_DynamicMusic::s_abJOINREL_ActionTypeChanged [ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS];

bool							Tools_Overlay_DynamicMusic::s_bTriggerTypeChanged = false;
int								Tools_Overlay_DynamicMusic::s_iTriggerTypeSelectionIndex = 0;
bool							Tools_Overlay_DynamicMusic::s_bGenericChangeRequiringSave = false;

static bool RoughlyEqual ( const float fVar1, const float fVar2 )
{
	return ( Asura_Maths::Abs( fVar1 - fVar2 ) < 0.05f );
}

// F U N C T I O N   D E F I N I T I O N S ///////////////////////////////////

void Tools_Overlay_DynamicMusic::ToggleOverlay ()
{
	if( s_bMainOverlayOpen )
	{
		s_bOverlayShouldBeOpen = false;
		CloseOverlay();
	}
	else
	{
		s_bOverlayShouldBeOpen = true;
		OpenOverlay();
	}
}

void Tools_Overlay_DynamicMusic::OpenOverlay ()
{
	if(s_bMainOverlayOpen)
	{
		return;
	}
	s_bMainOverlayOpen = true;

	//Open the overlay

	for ( int iOverlay = 0; iOverlay < DM_NUM_OVERLAYS; ++iOverlay )
	{
		s_axOverlayPositions[ iOverlay ] = Asura_Vector_2( 0, 0 );
	}

	TransferModeData(TOOLSMODEDATAFILE_LOAD);

	CreateOverlay_Main ();

	//warn if sound is muted
	if(Tools_Sound_System::IsMusicMixerMuted())
	{
		ShowMessageOverlay("Warning.\nThe music mixer is muted.\n");		
	}

	// init track list stuff

	s_iTrackListArraySize		= 0;
	s_ppszTrackListNames		= 0;
	s_iTrackListSelectionIndex	= -1;
	s_bTrackListSelectionChanged = false;

	//init track stuff
	s_pxWAVAlignDialog = 0;
	for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++i )
	{
		s_apxLayerFilesDialogs[i] = 0;
		s_apxLayerClearDialogs[i] = 0;
	}
	s_bTrackModified = false;
	s_bForceTrackOverlayOpen = false;
}

void Tools_Overlay_DynamicMusic::CloseOverlay	()
{
	if(!s_bMainOverlayOpen)
	{
		return;
	}

	s_bMainOverlayOpen = false;

	//Close down the overlay

	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	//Free memory
	DeleteTrackListArray	();
	DeleteTriggerListArray	();

	//close overlays
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DMUSIC_MAIN);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DMUSIC_TRACKLIST);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DMUSIC_TRACK);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DMUSIC_TRIGLIST);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DMUSIC_TRIGGER_EDIT);

	s_pxWAVAlignDialog = 0;

	Asura_Dynamic_Music_System::Stop ();
}

bool Tools_Overlay_DynamicMusic::IsOverlayOpen ()
{
	return s_bMainOverlayOpen;
}

void Tools_Overlay_DynamicMusic::Initialise (void)
{
	
}


void Tools_Overlay_DynamicMusic::Shutdown (void)
{
	CloseOverlay();
}


void Tools_Overlay_DynamicMusic::Update (void)
{
	if( !s_bMainOverlayOpen )
	{
		return;
	}

	if( IsModified() )
	{
		Tools_SaveAsura::SetDynamicMusicModified();
	}

	if( ( s_bTrackListSelectionChanged && Asura_Overlay_System::DoesOverlayExist( TOOLS_OVERLAY_DMUSIC_TRACK ) ) )
	{
		s_bForceTrackOverlayOpen = false;
		s_bTrackListSelectionChanged = false;
		if( s_iTrackListSelectionIndex != -1 )
		{
			CreateOverlay_Track();
		}
		else
		{
			Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DMUSIC_TRACK);
			s_pxWAVAlignDialog = 0;
		}
	}
	else if ( s_bForceTrackOverlayOpen )
	{
		s_bForceTrackOverlayOpen = false;
		s_bTrackListSelectionChanged = false;
		if( s_iTrackListSelectionIndex != -1)
		{
			CreateOverlay_Track();
		}
	}

	if( Asura_Overlay_System::DoesOverlayExist( TOOLS_OVERLAY_DMUSIC_TRACK ) )
	{
		if( s_pxWAVAlignDialog )
		{
			if( s_bChangedIntroOutro_Numeric )
			{
				s_bChangedIntroOutro_Numeric = false;

				const int iMaxBars = static_cast<int>( s_pxWAVAlignDialog->GetTrackDuration() / s_pxWAVAlignDialog->GetBarDuration() );

				if( s_iIntroEnd_Numeric > iMaxBars )
				{
					s_iIntroEnd_Numeric = iMaxBars;
				}

				if( s_iOutroStart_Numeric > iMaxBars )
				{
					s_iOutroStart_Numeric = iMaxBars;
				}

				// The intro/outro time was entered numerically by the user, set it on the wavalign dialog
				s_pxWAVAlignDialog->SetIntroMarkerTime( static_cast<float>( s_iIntroEnd_Numeric   ) * s_pxWAVAlignDialog->GetBarDuration() );
				s_pxWAVAlignDialog->SetOutroMarkerTime( static_cast<float>( s_iOutroStart_Numeric ) * s_pxWAVAlignDialog->GetBarDuration() );

				// set the track modified so we update our underlying data in the following code
				s_bTrackModified = true;
			}
			
			Asura_Hash_ID uTrackHash = Asura_GetHashID( s_ppszTrackListNames[ s_iTrackListSelectionIndex ] );
			if( s_bTrackModified )
			{
				s_bTrackModified = false;

				Asura_Dynamic_Music_TrackData* pxTrackData = Asura_Dynamic_Music_System::GetTrackByHash( uTrackHash );

				if(pxTrackData)
				{
					const float fBarDuration = (60.0f / static_cast<float>( pxTrackData->m_iBPM ) ) * static_cast<float>( pxTrackData->m_iBeatsPerBar );
					if(fBarDuration > 0.0f)
					{
						s_pxWAVAlignDialog->SetBarDuration( fBarDuration );
					}
					pxTrackData->m_fBodyOffset	= s_pxWAVAlignDialog->GetIntroMarkerTime();
					pxTrackData->m_fOutroOffset = s_pxWAVAlignDialog->GetOutroMarkerTime();
				}

				// Update the intro/outro numeric dialogs
				UpdateIntroOutroDialogs();

				Tools_SaveAsura::SetDynamicMusicModified();
				Asura_Dynamic_Music_System::Stop ();
			}
			else
			{
				if( Asura_Dynamic_Music_System::TrackIsPlaying( uTrackHash ) )
				{
					s_pxWAVAlignDialog->SetPlaybackMarkerTime( Asura_Dynamic_Music_System::GetTrackPlayTime( uTrackHash ) );
				}
				else
				{
					s_pxWAVAlignDialog->ClearPlaybackMarker();
				}
			}
		}
	}

	// This overlay is only active for CODE TRIGGERS. ENTITY TRIGGERS use the Tools_Entity_Object overlay.
	if( Asura_Overlay_System::DoesOverlayExist( TOOLS_OVERLAY_DMUSIC_TRIGGER_EDIT ) )
	{
		UpdateTriggerEditOverlay();
	}
}


bool Tools_Overlay_DynamicMusic::IsModified ()
{
	return s_bTrackModified;
}


void Tools_Overlay_DynamicMusic::TransferModeData(const int iMode)
{
	Tools_ModeDataFile xFile( "DynamicMusicOverlay.dat", iMode );

	//The position of the overlays
	for ( int iOverlay = 0; iOverlay < DM_NUM_OVERLAYS; ++iOverlay )
	{
		xFile.Transfer( s_axOverlayPositions[ iOverlay ] );
	}
}


void  Tools_Overlay_DynamicMusic::Button_Main_ShowTrackListOverlay ()
{
	CreateOverlay_TrackList();
}

void Tools_Overlay_DynamicMusic::Button_Main_ShowTriggerList ()
{
	CreateOverlay_TriggerList();
}

void  Tools_Overlay_DynamicMusic::Button_Main_CreateDynamicMusicTrigger ()
{
	Tools_Entity_DynamicMusicTrigger_Object* pxObject = Tools_Entity_DynamicMusicTrigger_Object::Create(false);
	Tools_Object_System::SelectObject(pxObject);

	//Assign the trigger to the correct entity file.
	if(pxObject)
	{
		Tools_SecondaryEntityFile_System::AddToCurrentFile(pxObject);
	}

	Tools_SaveAsura::SetDynamicMusicModified();
}

void Tools_Overlay_DynamicMusic::Button_Main_SaveClose ()
{
	Tools_SaveAsura::SaveSounds();
	ToggleOverlay();
}

void Tools_Overlay_DynamicMusic::Button_Main_SetLayerName ( int iLayer )
{
	GetString_Overlay xOverlay("Enter name for the new track","");

	Asura_String xName;
	
	if(!xOverlay.DoModal()) 
	{
		return;
	}
	xName = xOverlay.GetString();

	//see if the name is okay
	if(xName.IsBlank()) 
	{
		return;
	}
	
	Asura_Dynamic_Music_System::SetLayerName( iLayer, xName );

	if( s_apxLayerNameButtons[iLayer] )
	{
		s_apxLayerNameButtons[iLayer]->UpdateLabel( xName.GetString() );
	}
}

void Tools_Overlay_DynamicMusic::CreateOverlay_Main ()
{
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Dynamic Music", false, &s_axOverlayPositions[ DM_OVERLAY_MAIN ] );

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxDialog = new Asura_Function_Dialog("Show tracks overlay", xOffset, Button_Main_ShowTrackListOverlay);
	pxDialog->SetHelpText("Show the 'Tracks' overlay, allowing you to view and modify the list of music tracks");
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.y += Asura_Dialog::LargeSpacing();
	
	pxDialog = new Asura_Function_Dialog("Create a dynamic music trigger", xOffset, Button_Main_CreateDynamicMusicTrigger );
	pxDialog->SetHelpText("Create a new dynamic music trigger for affecting the dynamic music system");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Show trigger list", xOffset, Button_Main_ShowTriggerList );
	pxDialog->SetHelpText("Show the dynamic music trigger list overlay");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Divider_Dialog(xOffset);
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Dialog("Layer names", xOffset);
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	char szText[128];
	for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++i )
	{
		if(i)
		{
			xOffset.y += Asura_Dialog::DefaultSpacing ();
		}

		Asura_PrintfToBfr(szText,"Layer %d: %s", 128, true, i + 1, Asura_Dynamic_Music_System::GetLayerName( i ) );
		s_apxLayerNameButtons[i] = new Asura_Function_Dialog( szText, xOffset, Button_Main_SetLayerName, i );
		pxOverlay->AddDialog( s_apxLayerNameButtons[i] );
	}

	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Divider_Dialog(xOffset);
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Function_Dialog("Save & Close overlay", xOffset, Button_Main_SaveClose );
	pxDialog->SetHelpText("Save changes to the dynamic music and close the overlay");
	pxOverlay->AddDialog(pxDialog);
	
	pxOverlay->m_uID = TOOLS_OVERLAY_DMUSIC_MAIN;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);

//	ValidateTriggers ();
}


void Tools_Overlay_DynamicMusic::CreateOverlay_TrackList ()
{
	//delete old overlay if it exists
	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_DMUSIC_TRACKLIST );

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Dynamic Music Track List", false, &s_axOverlayPositions[ DM_OVERLAY_TRACK_LIST ] );

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2( 1.0f, 3.0f );

	pxDialog = new Asura_Dialog("Tracks", xOffset );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	CreateTrackListArray ();

	pxDialog = new Asura_List_Dialog( xOffset, s_ppszTrackListNames, &s_iTrackListSelectionIndex, 12, s_iTrackListArraySize, NULL, &s_bTrackListSelectionChanged );
	pxDialog->SetHelpText("These are all of the tracks that can be played");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;

	xOffset.y += Asura_Dialog::LargeSpacing();
	
	pxDialog = new Asura_Function_Dialog("New track", xOffset, Button_TrackList_NewTrack );
	pxDialog->SetHelpText("Create a new track");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::SmallSpacing();
	
	pxDialog = new Asura_Function_Dialog("Edit selected track", xOffset, Button_TrackList_EditTrack );
	pxDialog->SetHelpText("Edit the selected track");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::SmallSpacing();
	
	pxDialog = new Asura_Function_Dialog("Delete selected track", xOffset, Button_TrackList_DeleteTrack );
	pxDialog->SetHelpText("Delete the selected track");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();
	
	pxDialog = new Asura_Function_Dialog("Close overlay", xOffset, Button_TrackList_CloseOverlay );
	pxDialog->SetHelpText("Close the track list overlay");
	pxOverlay->AddDialog(pxDialog);

	pxOverlay->m_uID = TOOLS_OVERLAY_DMUSIC_TRACKLIST;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}


void Tools_Overlay_DynamicMusic::CreateTrackListArray ()
{
	DeleteTrackListArray();

	s_iTrackListArraySize = Asura_Dynamic_Music_System::GetNumberOfTracks();
	if( s_iTrackListArraySize > 0 )
	{	
		s_ppszTrackListNames = new char*[s_iTrackListArraySize];
		for( int i = 0; i < s_iTrackListArraySize; i++ )
		{
			Asura_Dynamic_Music_TrackData* pxTrackData = Asura_Dynamic_Music_System::GetTrackByIndex( i );
			
			int iChars = pxTrackData->m_xName.GetLength();
			s_ppszTrackListNames[i] = new char [ iChars + 1 ];
			Asura_Mem::Copy( s_ppszTrackListNames[ i ], pxTrackData->m_xName.GetString(), iChars );
			s_ppszTrackListNames[ i ][ iChars ] = 0;
		}
	}
}


void Tools_Overlay_DynamicMusic::DeleteTrackListArray ()
{
	for(int i = 0; i < s_iTrackListArraySize; ++i)
	{
		delete []s_ppszTrackListNames[i];
	}
	delete [] s_ppszTrackListNames;
	s_ppszTrackListNames = 0;
	
	s_iTrackListArraySize = 0;
}


void  Tools_Overlay_DynamicMusic::Button_TrackList_NewTrack ()
{
	Asura_Dynamic_Music_System::Stop();

	//get a name for the new piece
	GetString_Overlay xOverlay("Enter name for the new track","");

	Asura_String xName;
	bool bOkay = false;
	while(!bOkay)
	{
		if(!xOverlay.DoModal()) 
		{
			return;
		}
		xName = xOverlay.GetString();

		//see if the name is okay
		if(xName.IsBlank()) 
		{
			return;
		}

		Asura_Hash_ID uNameHash = Asura_GetHashID(xName.GetString());

		bOkay = !Asura_Dynamic_Music_System::TrackExists( uNameHash );
		
		if( !bOkay )
		{
			ShowMessageOverlay("That name already exists for a different track, please enter a new name\n");
		}
	}

	Asura_Dynamic_Music_TrackData* pxTrackData = Asura_Dynamic_Music_System::CreateNewTrack();
	Asura_Assert( pxTrackData, "Failed to create new track" );
	if(pxTrackData)
	{
		pxTrackData->m_xName = xName;

		//recreate the track list overlay to include this new track
		CreateOverlay_TrackList ();

		//Find the new track in the list and select it
		s_iTrackListSelectionIndex = -1;

		//search for the name in the sorted list
		for(int i = 0; i < s_iTrackListArraySize; i++)
		{
			if( !strcmp( xName.GetString(), s_ppszTrackListNames[ i ] ) )
			{
				s_iTrackListSelectionIndex = i;
				break;
			}
		}

		//Make sure that the track overlay gets shown during 'update'
		s_bForceTrackOverlayOpen = true;
	}
}


void Tools_Overlay_DynamicMusic::Button_TrackList_EditTrack ()
{
	if( s_iTrackListSelectionIndex == -1 )
	{
		return;
	}
	if(!s_ppszTrackListNames)
	{
		return;
	}
	CreateOverlay_Track ();
}


void Tools_Overlay_DynamicMusic::Button_TrackList_DeleteTrack ()
{
	Asura_Dynamic_Music_System::Stop();

	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DMUSIC_TRACK);

	if( s_iTrackListSelectionIndex == -1 )
	{
		return;
	}
	if(!s_ppszTrackListNames)
	{
		return;
	}
	
	Asura_Dynamic_Music_System::DeleteTrack( Asura_GetHashID( s_ppszTrackListNames[ s_iTrackListSelectionIndex ] ) );
	s_iTrackListSelectionIndex = -1;
	CreateOverlay_TrackList ();
}


void Tools_Overlay_DynamicMusic::Button_TrackList_CloseOverlay ()
{
	DeleteTrackListArray();

	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_DMUSIC_TRACKLIST );
	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_DMUSIC_TRACK );
	s_pxWAVAlignDialog = 0;
}


void Tools_Overlay_DynamicMusic::CreateOverlay_Track ()
{
	s_pxWAVAlignDialog = 0;
	for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++i)
	{
		s_apxLayerFilesDialogs[i] = 0;
		s_apxLayerClearDialogs[i] = 0;
	}
	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_DMUSIC_TRACK );
	
	//early outs
	if( s_iTrackListSelectionIndex == -1 )
	{
		return;
	}
	if(!s_ppszTrackListNames)
	{
		return;
	}

	//Get the appropriate data for the selected track
	Asura_Dynamic_Music_TrackData* pxTrackData = Asura_Dynamic_Music_System::GetTrackByHash( Asura_GetHashID( s_ppszTrackListNames[ s_iTrackListSelectionIndex ] ) );

	if(!pxTrackData)
	{
		return;
	}

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Dynamic Music Track Editor", false, &s_axOverlayPositions[ DM_OVERLAY_TRACK ] );

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2( 1.0f, 3.0f );

	char szText[128];

	Asura_PrintfToBfr( szText, "Track name : %s", 128, true, pxTrackData->m_xName.GetString() );
	pxDialog = new Asura_Function_Dialog(szText, xOffset, Button_Track_Rename);
	pxDialog->SetHelpText("Rename the track");
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.y += Asura_Dialog::LargeSpacing();

	const int iIconTexture = Tools_Dialog::GetIconTexture();

	for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++i)
	{
		if( pxTrackData->m_axLayerFiles[ i ].GetLength() > 0 )
		{
			const Asura_String xLayerName = pxTrackData->m_axLayerFiles[ i ].GetString();
			Asura_String xDisplayName;
			xLayerName.SplitFilename( 0, &xDisplayName, 0 );
			Asura_PrintfToBfr( szText, "%s : %s", 128, true, Asura_Dynamic_Music_System::GetLayerName( i ), xDisplayName.GetString() );
		}
		else
		{
			Asura_PrintfToBfr( szText, "%s : [ No sample ]", 128, true, Asura_Dynamic_Music_System::GetLayerName( i ) );
		}
		pxDialog = new Asura_Function_Dialog( szText, xOffset, Button_Track_SelectSample, i );
		pxDialog->SetHelpText("Select a sample for this layer of the track");
		s_apxLayerFilesDialogs[i] = pxDialog;
		pxOverlay->AddDialog(pxDialog);	

		xOffset.x += 34.0f;

		Asura_Icon_Dialog* pxIconDlg = new Asura_Icon_Dialog( xOffset, iIconTexture, TOOLS_ICON_DELETE );
		pxIconDlg->SetClickedFunction( Button_Track_ClearSample, i );
		pxIconDlg->SetHelpText( "Clear the sample for this layer of the track" );
		
		if( pxTrackData->m_axLayerFiles[ i ].GetLength() == 0 )
		{
			pxIconDlg->SetEnabled( false );
		}

		pxOverlay->AddDialog(pxIconDlg);	

		s_apxLayerClearDialogs[i] = pxIconDlg;

		xOffset.x -= 34.0f;

		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Volume", xOffset, &pxTrackData->m_fVolume, 0, 1.5f, 0.01f, "%.3f", &s_bTrackModified);
	pxDialog->SetHelpText("Volume setting for this track");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Numerical_Dialog("BPM", xOffset, &pxTrackData->m_iBPM, 0, 320, 1, "%d", &s_bTrackModified);
	pxDialog->SetHelpText("Beats per minute of the music tracks in this track");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Numerical_Dialog("Beats per bar", xOffset, &pxTrackData->m_iBeatsPerBar, 0, 60, 1, "%d", &s_bTrackModified);
	pxDialog->SetHelpText("Beats per bar of the music tracks in this track, this is used to sync musical elements and triggers");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();

	s_pxWAVAlignDialog = new Asura_WAVAlign_Dialog( xOffset, 360, 100, &s_bTrackModified );
	if( s_pxWAVAlignDialog )
	{
		for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++i )
		{
			s_pxWAVAlignDialog->AddWAVEntry( i );
			if(pxTrackData->m_axLayerFiles[i].GetLength() > 0)
			{
				Asura_String xWAV = pxTrackData->m_axLayerFiles[i];
				if( xWAV.GetLength() > 0 )
				{
					s_pxWAVAlignDialog->LoadWAV( xWAV.GetString(), i );
				}
			}
		}
		const float fBarDuration = (60.0f / static_cast<float>( pxTrackData->m_iBPM ) ) * static_cast<float>( pxTrackData->m_iBeatsPerBar );
		if(fBarDuration > 0.0f)
		{
			s_pxWAVAlignDialog->SetBarDuration( fBarDuration );
		}
		if( !( ( pxTrackData->m_fBodyOffset == 0.0f ) && ( pxTrackData->m_fOutroOffset == 0.0f ) ) )
		{
			s_pxWAVAlignDialog->SetIntroMarkerTime( pxTrackData->m_fBodyOffset  );
			s_pxWAVAlignDialog->SetOutroMarkerTime( pxTrackData->m_fOutroOffset );
		}
		
		pxOverlay->AddDialog( s_pxWAVAlignDialog );
	}

	Asura_Vector_2 xOldOffset = xOffset;

	xOffset.x += 17.0f;
	xOffset.y -= Asura_Dialog::SmallSpacing() * 3.0f;

	s_pxIntroOutroDialogs[0] = new Asura_Numerical_Dialog("Intro End (bars)", xOffset, &s_iIntroEnd_Numeric, 0, 4096, 1, "%d", &s_bChangedIntroOutro_Numeric);
	s_pxIntroOutroDialogs[0]->SetHelpText("Position, in bars, of the end of the track intro period");
	s_pxIntroOutroDialogs[0]->SetEnabled( s_pxWAVAlignDialog ? s_pxWAVAlignDialog->AnyWAVsLoaded() : false );
	pxOverlay->AddDialog(s_pxIntroOutroDialogs[0] );

	xOffset.y += Asura_Dialog::DefaultSpacing();

	s_pxIntroOutroDialogs[1] = new Asura_Numerical_Dialog("Outro Start (bars)", xOffset, &s_iOutroStart_Numeric, 0, 4096, 1, "%d", &s_bChangedIntroOutro_Numeric);
	s_pxIntroOutroDialogs[1]->SetHelpText("Position, in bars, of the start of the track outro period");
	s_pxIntroOutroDialogs[1]->SetEnabled( s_pxWAVAlignDialog ? s_pxWAVAlignDialog->AnyWAVsLoaded() : false );
	pxOverlay->AddDialog( s_pxIntroOutroDialogs[1] );

	UpdateIntroOutroDialogs();

	xOffset.y = s_pxWAVAlignDialog->m_xMaxCoord.y;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x -= 17.0f;

	pxDialog = new Asura_Function_Dialog("Play track from intro", xOffset, Button_Track_Play, Asura_Dynamic_Music_System::TRACK_PART_INTRO, &s_bIgnore, Enabled_Track_Play);
	pxDialog->SetHelpText("Start the track playing from the intro portion");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Play track from body", xOffset, Button_Track_Play, Asura_Dynamic_Music_System::TRACK_PART_BODY, &s_bIgnore, Enabled_Track_Play );
	pxDialog->SetHelpText("Start the track playing, skip the intro");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Play track from outro", xOffset, Button_Track_Play, Asura_Dynamic_Music_System::TRACK_PART_OUTRO, &s_bIgnore, Enabled_Track_Play );
	pxDialog->SetHelpText("Play the outro of this track");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Halt dynamic music playback", xOffset, Button_Track_Stop, &s_bIgnore, Enabled_Track_Stop);
	pxDialog->SetHelpText("Stop the dynamic music system and clear any active triggers");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Close overlay", xOffset, Button_Track_CloseOverlay );
	pxDialog->SetHelpText("Close the track editor overlay");
	pxOverlay->AddDialog(pxDialog);

	pxOverlay->m_uID = TOOLS_OVERLAY_DMUSIC_TRACK;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

bool Tools_Overlay_DynamicMusic::Enabled_Track_Play ()
{
	return !Asura_Dynamic_Music_System::IsPlaying();
}

bool Tools_Overlay_DynamicMusic::Enabled_Track_Stop ()
{
	return Asura_Dynamic_Music_System::IsPlaying();
}

void Tools_Overlay_DynamicMusic::UpdateIntroOutroDialogs ()
{
	if( !s_pxIntroOutroDialogs[0] ) return;
	if( !s_pxIntroOutroDialogs[1] ) return;
	
	bool bEnabled = ( s_pxWAVAlignDialog && s_pxWAVAlignDialog->AnyWAVsLoaded() );
	
	s_pxIntroOutroDialogs[0]->SetEnabled( bEnabled );
	s_pxIntroOutroDialogs[1]->SetEnabled( bEnabled );

	if( bEnabled )
	{	
		if( s_pxWAVAlignDialog && ( s_pxWAVAlignDialog->GetBarDuration() > 0.0f ) )
		{
			// Get the intro and outro time from the wav align dialog and update the numerical dialogs
			s_iIntroEnd_Numeric		= static_cast<int>( s_pxWAVAlignDialog->GetIntroMarkerTime() / s_pxWAVAlignDialog->GetBarDuration() );
			s_iOutroStart_Numeric	= static_cast<int>( s_pxWAVAlignDialog->GetOutroMarkerTime() / s_pxWAVAlignDialog->GetBarDuration() );
		}
	}

	s_bChangedIntroOutro_Numeric = false;	// dont flag as modified
}

void Tools_Overlay_DynamicMusic::Button_Track_Rename ()
{
	const Asura_Hash_ID uOldHash = Asura_GetHashID( s_ppszTrackListNames[ s_iTrackListSelectionIndex ] );

	//Rename the track
	Asura_Dynamic_Music_TrackData* pxTrackData = Asura_Dynamic_Music_System::GetTrackByHash( uOldHash );

	if( !pxTrackData )
	{
		return;
	}

	GetString_Overlay xOverlay("Enter name for the new track","");

	if(!xOverlay.DoModal()) 
	{
		return;
	}

	pxTrackData->m_xName = xOverlay.GetString();

	// recreate the track list
	CreateTrackListArray();
	CreateOverlay_TrackList();

	//Find the new track in the list and select it
	s_iTrackListSelectionIndex = -1;

	//search for the name in the sorted list
	for(int i = 0; i < s_iTrackListArraySize; i++)
	{
		if( !strcmp( pxTrackData->m_xName.GetString(), s_ppszTrackListNames[ i ] ) )
		{
			s_iTrackListSelectionIndex = i;
			break;
		}
	}

	CreateOverlay_Track();
	
	const Asura_Hash_ID uNewHash = pxTrackData->m_xName.GetHash();
	// find all references to the old name in all triggers and modify them
	for( int iTrigger = 0; iTrigger < Asura_Dynamic_Music_System::GetNumberOfTriggerData(); ++iTrigger )
	{
		Asura_Dynamic_Music_Trigger_Data* pxTrigger = Asura_Dynamic_Music_System::GetTriggerDatabyIndex( iTrigger );

		if( pxTrigger )
		{
			pxTrigger->ReplaceTrackReferences( uOldHash, uNewHash );
		}
	}

	s_bTrackModified = true;

	// TODO: Should probably deselect any triggers here since they will have the wrong text if the reference this track
}


void Tools_Overlay_DynamicMusic::Button_Track_SelectSample	( int iLayer )
{
	//Get the current track and prompt for the user to set the music track
	Asura_Dynamic_Music_TrackData* pxTrackData = Asura_Dynamic_Music_System::GetTrackByHash( Asura_GetHashID( s_ppszTrackListNames[ s_iTrackListSelectionIndex ] ) );
	
	if( pxTrackData )
	{
		if( PromptAndSetMusicTrack ( pxTrackData, iLayer ) )
		{
			//Update the dialog
			if( s_apxLayerFilesDialogs[ iLayer ] )
			{
				char szText[128];
				
				const Asura_String xLayerName = pxTrackData->m_axLayerFiles[ iLayer ].GetString();
				Asura_String xDisplayName;
				xLayerName.SplitFilename( 0, &xDisplayName, 0 );
				Asura_PrintfToBfr( szText, "%s : %s", 128, true, Asura_Dynamic_Music_System::GetLayerName( iLayer ), xDisplayName.GetString() );

				s_apxLayerFilesDialogs[ iLayer ]->UpdateLabel( szText );
				s_apxLayerClearDialogs[ iLayer ]->SetEnabled( true );
			}
			s_bTrackModified = true;
		}
	}
}


void Tools_Overlay_DynamicMusic::Button_Track_ClearSample ( int iLayer )
{
	Asura_Dynamic_Music_TrackData* pxTrackData = Asura_Dynamic_Music_System::GetTrackByHash( Asura_GetHashID( s_ppszTrackListNames[ s_iTrackListSelectionIndex ] ) );
	if( pxTrackData )
	{
		pxTrackData->m_axLayerFiles[ iLayer ] = "";
		s_pxWAVAlignDialog->ClearWAV( iLayer );
		
		//Update the dialog
		if( s_apxLayerFilesDialogs[ iLayer ] )
		{
			char szText[128];
			Asura_PrintfToBfr( szText, "%s : [ No sample ]", 128, true, Asura_Dynamic_Music_System::GetLayerName( iLayer ) );
			s_apxLayerFilesDialogs[ iLayer ]->UpdateLabel( szText );
			s_apxLayerClearDialogs[ iLayer ]->SetEnabled( false );
		}
		s_bTrackModified = true;
	}
}


void  Tools_Overlay_DynamicMusic::Button_Track_Play ( int iPos )
{
	Asura_Dynamic_Music_System::PlayTrack( Asura_GetHashID( s_ppszTrackListNames[ s_iTrackListSelectionIndex ] ), iPos );
}

void  Tools_Overlay_DynamicMusic::Button_Track_Stop ()
{
	if( s_pxWAVAlignDialog )
	{
		s_pxWAVAlignDialog->ClearPlaybackMarker();
	}
	Asura_Dynamic_Music_System::Stop ();
}

bool Tools_Overlay_DynamicMusic::PromptAndSetMusicTrack ( Asura_Dynamic_Music_TrackData* pxTrack, int iLayer )
{
	//Prompt for a file
	const Asura_Hash_ID uSelectedStream = Tools_Audio_StreamingOverlay::SelectStream();
	
	if( uSelectedStream != ASURA_HASH_ID_UNSET )
	{	
		Asura_Audio_StreamInfo* pxStreamInfo = Asura_Audio_System::GetStreamInfo( uSelectedStream );

		if( pxStreamInfo )
		{
			pxTrack->m_axLayerFiles[ iLayer ] = pxStreamInfo->m_szFilename;
			if( s_pxWAVAlignDialog->LoadWAV( pxStreamInfo->m_szFilename, iLayer ) )
			{
				return true;
			}
			else
			{
				pxTrack->m_axLayerFiles[ iLayer ] = "";
			}
		}
	}

	return false;
}


void Tools_Overlay_DynamicMusic::Button_Track_CloseOverlay	()
{
	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_DMUSIC_TRACK );
}


void Tools_Overlay_DynamicMusic::CreateOverlay_TriggerList ()
{
	//delete old overlay if it exists
	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_DMUSIC_TRIGLIST );

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Dynamic Music Trigger List", false, &s_axOverlayPositions[ DM_OVERLAY_TRIGGER_LIST ] );

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2( 1.0f, 3.0f );

	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset, 0, 0);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	const float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	//
	// Entity triggers list
	//
	pxTabsDialog->AddTab("Entity Triggers");
	pxTabsDialog->SelectTab(0);

	CreateTriggerListArray();

	// one hash ID per string in this list box
	pxDialog = new Asura_List_Dialog( xOffset, s_ppszEntityTriggerListNames, &s_iEntityTriggerListSelectionIndex, 12, s_xEntityTriggerHashList.Size(), NULL );
	pxDialog->SetHelpText("These are all of the Entity Triggers which exist");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Create Entity Trigger", xOffset, Button_Main_CreateDynamicMusicTrigger );
	pxDialog->SetHelpText("Create a new Entity dynamic music trigger for affecting the dynamic music system");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Jump camera to trigger", xOffset, Button_TrigList_JumpToTrig );
	pxDialog->SetHelpText("Move the camera to this trigger's position in the world");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	fTabsEndY = Asura_Maths::Max(xOffset.y, fTabsEndY);
	xOffset.y = fTabsStartY;

	//
	// Code triggers list
	//
	pxTabsDialog->AddTab("Code Triggers");
	pxTabsDialog->SelectTab(1);

	pxDialog = new Asura_List_Dialog( xOffset, s_ppszCodeTriggerListNames, &s_iCodeTriggerListSelectionIndex, 12, s_xCodeTriggerHashList.Size(), NULL );
	pxDialog->SetHelpText("These are all of the Code Triggers which exist");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Create Code Trigger", xOffset, Button_TrigList_CreateCodeTrigger );
	pxDialog->SetHelpText("Create a new Code dynamic music trigger for affecting the dynamic music system");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	fTabsEndY = Asura_Maths::Max(xOffset.y, fTabsEndY);
	xOffset.y = fTabsEndY + 1.0f;

	pxDialog = new Asura_Function_Dialog("Test trigger", xOffset, Button_TrigList_TestTrigger );
	pxDialog->SetHelpText("Test the selected trigger, as if it was triggered in game");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Show trigger details", xOffset, Button_TrigList_ShowTrigProps );
	pxDialog->SetHelpText("Select the trigger object and show its details");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Rename selected trigger", xOffset, Button_TrigList_RenameTrigger );
	pxDialog->SetHelpText("Renames the selected trigger");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Delete selected trigger", xOffset, Button_TrigList_DeleteTrigger );
	pxDialog->SetHelpText("Deletes the selected trigger");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Close Overlay", xOffset, Button_TrigList_CloseOverlay );
	pxDialog->SetHelpText("Close this overlay");
	pxOverlay->AddDialog(pxDialog);

	pxTabsDialog->SelectTab(0);
	pxTabsDialog->SetSelectedTabPointer(&s_uTriggerListSelectedTab);

	pxOverlay->m_uID = TOOLS_OVERLAY_DMUSIC_TRIGLIST;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}


void Tools_Overlay_DynamicMusic::CreateTriggerListArray ()
{
	DeleteTriggerListArray();

	u_int uTriggerDataTotal = Asura_Dynamic_Music_System::GetNumberOfTriggerData();
	if( uTriggerDataTotal > 0 )
	{
		u_int uEntityTriggerCount = 0;
		u_int uCodeTriggerCount = 0;

		//
		// Count number of entity/code triggers we have
		//
		for( u_int iTrig = 0; iTrig < uTriggerDataTotal; iTrig++ )
		{
			Asura_Dynamic_Music_Trigger_Data* pxTriggerData = Asura_Dynamic_Music_System::GetTriggerDatabyIndex( iTrig );
			if (pxTriggerData)
			{
				if (pxTriggerData->m_bCodeTrigger)
				{
					uCodeTriggerCount++;
				}
				else
				{
					uEntityTriggerCount++;
				}
			}
		}

		if (uEntityTriggerCount > 0)
		{
			s_ppszEntityTriggerListNames = new char*[uEntityTriggerCount];
			s_xEntityTriggerHashList.SetSize(uEntityTriggerCount);
		}

		if (uCodeTriggerCount > 0)
		{
			s_ppszCodeTriggerListNames = new char*[uCodeTriggerCount];
			s_xCodeTriggerHashList.SetSize(uCodeTriggerCount);
		}

		// reuse counts
		uEntityTriggerCount = 0;
		uCodeTriggerCount = 0;

		for( u_int i = 0; i < uTriggerDataTotal; i++ )
		{
			Asura_Dynamic_Music_Trigger_Data* pxTriggerData = Asura_Dynamic_Music_System::GetTriggerDatabyIndex( i );
						
			if( pxTriggerData )
			{
				if (pxTriggerData->m_bCodeTrigger)
				{
					int iChars = pxTriggerData->m_xCodeTriggerName.GetLength();
					s_ppszCodeTriggerListNames[uCodeTriggerCount] = new char [ iChars + iMAX_TRIGGER_SUFFIX_LENGTH + 1 ];
					Asura_Mem::Copy( s_ppszCodeTriggerListNames[uCodeTriggerCount], pxTriggerData->m_xCodeTriggerName.GetString(), iChars );
					s_ppszCodeTriggerListNames[uCodeTriggerCount][iChars] = 0;

					// Concatenate the trigger name with its type, seems like it might be useful
					strcat( s_ppszCodeTriggerListNames[uCodeTriggerCount], g_aszTriggerSuffix[ pxTriggerData->m_eType ] );

					s_xCodeTriggerHashList[uCodeTriggerCount] = pxTriggerData->m_uHash;
					uCodeTriggerCount++;
				}
				else
				{
					Asura_ServerEntity_DynamicMusicTrigger* pxEntity = Asura_ServerEntity_DynamicMusicTrigger::FindEntityByHash(pxTriggerData->m_uHash);

					s_ppszEntityTriggerListNames[uEntityTriggerCount] = 0;

					Asura_String xTriggerName("<Unmatched Entity Trigger>");
					if( pxEntity )
					{
						xTriggerName = pxEntity->ToolsData->Name;
						int iChars = strlen( xTriggerName.GetString() );
						s_ppszEntityTriggerListNames[uEntityTriggerCount] = new char [ iChars + iMAX_TRIGGER_SUFFIX_LENGTH + 1 ];
						Asura_Mem::Copy( s_ppszEntityTriggerListNames[uEntityTriggerCount], xTriggerName.GetString(), iChars );
						s_ppszEntityTriggerListNames[uEntityTriggerCount][iChars] = 0;

						// Concatenate the trigger name with its type, seems like it might be useful
						strcat( s_ppszEntityTriggerListNames[uEntityTriggerCount], g_aszTriggerSuffix[ pxTriggerData->m_eType ] );

						s_xEntityTriggerHashList[uEntityTriggerCount] = pxTriggerData->m_uHash;
						uEntityTriggerCount++;
					}
					else
					{
						Asura_Assertf(false, ("Found Music Trigger data with no matching entity [hash %d]", pxTriggerData->m_uHash));
					}

				}
			}
		}

		// resize to number we actually added
		if ((s_xEntityTriggerHashList.Size() > 0) && (uEntityTriggerCount < s_xEntityTriggerHashList.Size()))
		{
			Asura_Assertf(false, ("Only matched Music Trigger data to entities: %d/%d.", uEntityTriggerCount, s_xEntityTriggerHashList.Size()));

			s_xEntityTriggerHashList.Shrink(uEntityTriggerCount);
		}
	}
}


void Tools_Overlay_DynamicMusic::DeleteTriggerListArray ()
{
	for(u_int i = 0; i < s_xEntityTriggerHashList.Size(); ++i)
	{
		if (s_ppszEntityTriggerListNames[i])
		{
			delete []s_ppszEntityTriggerListNames[i];
		}
	}
	if (s_ppszEntityTriggerListNames)
	{
		delete [] s_ppszEntityTriggerListNames;
		s_ppszEntityTriggerListNames = NULL;
	}

	for(u_int i = 0; i < s_xCodeTriggerHashList.Size(); ++i)
	{
		if (s_ppszCodeTriggerListNames[i])
		{
			delete []s_ppszCodeTriggerListNames[i];
		}
	}

	if (s_ppszCodeTriggerListNames)
	{
		delete [] s_ppszCodeTriggerListNames;
		s_ppszCodeTriggerListNames = NULL;
	}

	s_xEntityTriggerHashList.Clear();
	s_xCodeTriggerHashList.Clear();
}


Asura_Hash_ID Tools_Overlay_DynamicMusic::GetSelectedTriggerHash ( )
{
	// Get currently selected trigger hash

	Asura_Hash_ID uTriggerHash = ASURA_HASH_ID_UNSET;

	if (s_uTriggerListSelectedTab == 0)
	{
		if( s_iEntityTriggerListSelectionIndex >= 0)
		{
			if (s_xEntityTriggerHashList.Size() > 0)
			{
				uTriggerHash = s_xEntityTriggerHashList[s_iEntityTriggerListSelectionIndex];
			}
		}
	}
	else if (s_uTriggerListSelectedTab == 1)
	{
		if( s_iCodeTriggerListSelectionIndex >= 0)
		{
			if (s_xCodeTriggerHashList.Size() > 0)
			{
				uTriggerHash = s_xCodeTriggerHashList[s_iCodeTriggerListSelectionIndex];
			}
		}
	}
	else
	{
		Asura_Assert (false, "undefined tab index");
	}

	return uTriggerHash;
}

void Tools_Overlay_DynamicMusic::CloseTriggerEditOverlay ()
{
	//delete old overlay if it exists
	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_DMUSIC_TRIGGER_EDIT );
}

void Tools_Overlay_DynamicMusic::CreateOverlay_SingleTrigger ()
{
	CloseTriggerEditOverlay();

	Asura_Hash_ID uTriggerHash = GetSelectedTriggerHash();

	Asura_Overlay* pxOverlay;
	Asura_String xOverlayName("Dynamic Music Code Trigger");
	Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::GetTriggerDataByHash( uTriggerHash );
	if (pxData && pxData->m_bCodeTrigger)
	{
		xOverlayName += " - " + pxData->m_xCodeTriggerName;
	}

	pxOverlay = new Asura_Overlay(xOverlayName.GetString(), false, &s_axOverlayPositions[ DM_OVERLAY_TRIGGER_EDIT ] );

	Asura_Vector_2 xOffset = Asura_Vector_2( 1.0f, 3.0f );

	AddTriggerOverlayControls(pxOverlay, xOffset, uTriggerHash);

	pxOverlay->m_uID = TOOLS_OVERLAY_DMUSIC_TRIGGER_EDIT;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
}

void Tools_Overlay_DynamicMusic::AddTriggerOverlayControls(class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset, Asura_Hash_ID uTriggerHash)
{
	Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::GetTriggerDataByHash( uTriggerHash );
	Asura_Assert( pxData, "Failed to find trigger data in dynamic music system" );

	ClearOverlayControlPointers();

	if(!pxData)
	{
		s_uSelectedTriggerHash = 0;
		return;
	}

	Asura_Dialog* pxDialog = NULL;

	s_uSelectedTriggerHash = uTriggerHash;

	s_iTriggerTypeSelectionIndex = static_cast<int>( pxData->m_eType );

	s_bTriggerTypeChanged = false;
	s_bGenericChangeRequiringSave = false;

	pxDialog = new Asura_DropDown_List_Dialog( xOffset, g_aszTriggerTypes, &s_iTriggerTypeSelectionIndex, DYNAMIC_MUSIC_NUM_TRIGGER_TYPES, DYNAMIC_MUSIC_NUM_TRIGGER_TYPES, 0, &s_bTriggerTypeChanged, "Trigger Type" );
	pxDialog->SetHelpText("Select the type of this trigger, only one type can apply to a trigger");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();

	switch(pxData->m_eType)
	{
	case DYNAMIC_MUSIC_TRIGGER_TRANSITION	: AddTriggerOverlayControls_Trans		( pxOverlay, xOffset, pxData );	break;
	case DYNAMIC_MUSIC_TRIGGER_PLAY			: AddTriggerOverlayControls_Play		( pxOverlay, xOffset, pxData );	break;
	case DYNAMIC_MUSIC_TRIGGER_STOP			: AddTriggerOverlayControls_Stop		( pxOverlay, xOffset, pxData );	break;
	case DYNAMIC_MUSIC_TRIGGER_JOINRELEASE	: AddTriggerOverlayControls_JoinRel		( pxOverlay, xOffset, pxData );	break;

	default:
		{
			Asura_Assert( false, "Unrecognised trigger type in dynamic music trigger" );
		}
	}

	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Divider_Dialog( xOffset );
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Test trigger now", xOffset, Button_TrigEdit_TestTrigger, &s_bIgnore, Enabled_TestTrigger );
	pxDialog->SetHelpText("Activate this trigger in the dynamic music system");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Halt dynamic music playback", xOffset, Button_TrigEdit_HaltPlayback, &s_bIgnore, Enabled_HaltPlayback);
	pxDialog->SetHelpText("Stop the dynamic music system and clear any active triggers");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Overlay_DynamicMusic::AddTriggerOverlayControls_Trans (Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset, Asura_Dynamic_Music_Trigger_Data* pxData)
{
	//Add the overlay controls for the transition trigger
	if(!pxData)
	{
		return;
	}

	Asura_Dialog*	pxDialog;
	char			szText[128];

	Asura_Dynamic_Music_TrackData* pxTrack = Asura_Dynamic_Music_System::GetTrackByHash( pxData->TriggerData.m_pxLayerTransition->m_uTrack );

	if(pxTrack)
	{
		Asura_PrintfToBfr(szText,"Next Track: %s", 128, true, pxTrack->m_xName.GetString() );
		s_bTRANS_HasTrackToTransition = true;
	}
	else
	{
		Asura_PrintfToBfr(szText,"Next Track: [None]", 128, true );
		s_bTRANS_HasTrackToTransition = false;
	}

	s_bTRANS_UseIntroTick = false;

	s_pxTRANS_TrackNameDialog = new Asura_Function_Dialog(szText, xOffset, Button_TRANS_SelectTrack);
	s_pxTRANS_TrackNameDialog->SetHelpText("Select the track into which to transition");
	pxOverlay->AddDialog(s_pxTRANS_TrackNameDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Clear next track", xOffset, Button_TRANS_ClearTrack);
	pxDialog->SetHelpText("Set there to be no track to transition into");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();

	s_bTRANS_ChannelDialogModified = false;
	s_pxTRANS_ChannelDialog = new Asura_Channel_Graph_Dialog( xOffset, 35, &s_bTRANS_ChannelDialogModified );
	s_pxTRANS_ChannelDialog->SetGraphDomain(0.0f, pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[2].m_fTime);

	//Add a channel for the previous track
	s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ] = s_pxTRANS_ChannelDialog->AddChannel();
	if( s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ] )
	{
		s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ]->SetDetachableFlag		( false );
		s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ]->SetRatioResizingFlag		( false );
		s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ]->SetAnchorStartBlockFlag	( true );

		//Add the body and outro of the previous track
		s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_PREV_BODY ]	= s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ]->AddBlock( "Body" , 0.0f , pxData->TriggerData.m_pxLayerTransition->m_fPrevOutroStartOffset, true, false );
		s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_PREV_OUTRO]	= s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ]->AddBlock( "Outro", pxData->TriggerData.m_pxLayerTransition->m_fPrevOutroStartOffset, pxData->TriggerData.m_pxLayerTransition->m_fPrevOutroEndOffset - pxData->TriggerData.m_pxLayerTransition->m_fPrevOutroStartOffset, true, false  );

		//Add volume attenuation curve points
		s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ]->SetControlPoint (0, pxData->TriggerData.m_pxLayerTransition->m_xPrevTrackAmplitude[0].m_fTime, pxData->TriggerData.m_pxLayerTransition->m_xPrevTrackAmplitude[0].m_fAmplitude, true, true);
		s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ]->SetControlPoint (1, pxData->TriggerData.m_pxLayerTransition->m_xPrevTrackAmplitude[1].m_fTime, pxData->TriggerData.m_pxLayerTransition->m_xPrevTrackAmplitude[1].m_fAmplitude, false, false);
		s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ]->SetControlPoint (2, pxData->TriggerData.m_pxLayerTransition->m_xPrevTrackAmplitude[2].m_fTime, pxData->TriggerData.m_pxLayerTransition->m_xPrevTrackAmplitude[2].m_fAmplitude, false, true);
	}

	//If we have a channel to transition to, add a channel for that as well
	if( s_bTRANS_HasTrackToTransition )
	{
		s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]  = s_pxTRANS_ChannelDialog->AddChannel();
		if( s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ] )
		{
			s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]->SetDetachableFlag		( false );
			s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]->SetRatioResizingFlag		( false );
			s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]->SetAnchorEndBlockFlag	( true );

			if( pxData->TriggerData.m_pxLayerTransition->m_bUseIntro )
			{
				s_apxTRANS_ChannelBlocks[CHANNEL_BLOCK_NEXT_INTRO]	= s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]->AddBlock( "Intro", pxData->TriggerData.m_pxLayerTransition->m_fNextIntroStartOffset, pxData->TriggerData.m_pxLayerTransition->m_fNextIntroEndOffset - pxData->TriggerData.m_pxLayerTransition->m_fNextIntroStartOffset, true, false  );
				if( s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_INTRO ] )
				{
					//Make the block a nice colour if it is representing the correct duration
					const float fDuration = pxData->TriggerData.m_pxLayerTransition->m_fNextIntroEndOffset - pxData->TriggerData.m_pxLayerTransition->m_fNextIntroStartOffset;
					const Asura_Dynamic_Music_TrackData* pxTrackData = Asura_Dynamic_Music_System::GetTrackByHash( pxData->TriggerData.m_pxLayerTransition->m_uTrack );
					if( pxTrackData && RoughlyEqual( fDuration, pxTrackData->m_fBodyOffset ) )
					{
						s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_INTRO ]->SetColour( uTOOLS_TRIGGER_ENTITY_LIGHT_GREEN );
					}
					else
					{
						s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_INTRO ]->SetColour( 0x00FFFFFF );
					}
				}
			}
			s_apxTRANS_ChannelBlocks[CHANNEL_BLOCK_NEXT_BODY] = s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]->AddBlock( "Body" , pxData->TriggerData.m_pxLayerTransition->m_fNextIntroEndOffset, 60.0f - pxData->TriggerData.m_pxLayerTransition->m_fNextIntroEndOffset, true, false );

			//Add volume attenuation curve points
			s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]->SetControlPoint (0, pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[0].m_fTime, pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[0].m_fAmplitude, false, true);
			s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]->SetControlPoint (1, pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[1].m_fTime, pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[1].m_fAmplitude, false, false);
			s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]->SetControlPoint (2, pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[2].m_fTime, pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[2].m_fAmplitude, true, true);

			if(pxTrack)
			{
				s_pxTRANS_ChannelDialog->SetSnapInterval( pxTrack->GetBarDuration() );
			}
		}
	}
	else
	{
		s_pxTRANS_ChannelDialog->SetSnapInterval( -1.0f );
	}

	pxOverlay->AddDialog( s_pxTRANS_ChannelDialog );

	xOffset.y += 10.0f;

	pxDialog = new Asura_Dialog( "Next track settings", xOffset );
	pxOverlay->AddDialog( pxDialog );

	xOffset.y += Asura_Dialog::LargeSpacing();

	s_pxTRANS_UseIntroTick = new Asura_Tickbox_Dialog("Use next track's intro", xOffset, &s_bTRANS_UseIntroTick);
	s_pxTRANS_UseIntroTick->SetHelpText("Select whether the intro of the next track should be skipped");
	pxOverlay->AddDialog(s_pxTRANS_UseIntroTick);
	s_bTRANS_UseIntroTick = pxData->TriggerData.m_pxLayerTransition->m_bUseIntro;
	s_pxTRANS_UseIntroTick->SetEnabled( s_bTRANS_HasTrackToTransition );

	xOffset.y += Asura_Dialog::LargeSpacing();

	for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++i )
	{
		Asura_PrintfToBfr(szText,"Start with %s muted", 128, true, Asura_Dynamic_Music_System::GetLayerName( i ) );
		pxDialog = new Asura_Tickbox_Dialog( szText, xOffset, &pxData->TriggerData.m_pxLayerTransition->m_bMuteLayers[i], &s_bGenericChangeRequiringSave );
		xOffset.y += Asura_Dialog::DefaultSpacing ();

		pxOverlay->AddDialog(pxDialog);
	}

	//Lets see if we can do without this...

	/*pxDialog = new Asura_Divider_Dialog( xOffset );
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Expand trigger duration", xOffset, _TRANS_Button_Increase_Time );
	pxDialog->SetHelpText("Expand the time over which the trigger runs");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Reduce trigger duration", xOffset, _TRANS_Button_Reduce_Time );
	pxDialog->SetHelpText("Reduce the time over which the trigger runs");
	pxOverlay->AddDialog(pxDialog);*/
}


void Tools_Overlay_DynamicMusic::Button_TRANS_SelectTrack ()
{
	Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::GetTriggerDataByHash( s_uSelectedTriggerHash );

	if( pxData )
	{
		Asura_Hash_ID xNewTrack = PromptForDynamicMusicTrack();
		if( xNewTrack == ASURA_HASH_ID_UNSET )
		{
			return;
		}
		pxData->TriggerData.m_pxLayerTransition->m_uTrack = xNewTrack;

		Asura_Dynamic_Music_TrackData* pxTrack = Asura_Dynamic_Music_System::GetTrackByHash( pxData->TriggerData.m_pxLayerTransition->m_uTrack );

		const bool bTrackHasIntro = ( pxTrack->m_fBodyOffset > 0.0f );

		if( bTrackHasIntro )
		{
			//This tickbox is a pain in the arse to deal with so I'm copping out and just always setting it to on
			s_bTRANS_UseIntroTick = true;
			pxData->TriggerData.m_pxLayerTransition->m_bUseIntro = true;
		}
		else
		{
			for( int iPoint = 0; iPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION; ++iPoint )
			{
				if( pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[iPoint].m_fTime < pxData->TriggerData.m_pxLayerTransition->m_fNextIntroStartOffset )
				{
					pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[iPoint].m_fTime = pxData->TriggerData.m_pxLayerTransition->m_fNextIntroStartOffset;
				}
			}

		}
		Tools_SaveAsura::SetDynamicMusicModified();
		RecreateOverlay ();
		Tools_Entity_DynamicMusicTrigger_Object::RecreateCurrentOverlay();
	}
}

void Tools_Overlay_DynamicMusic::Button_TRANS_ClearTrack ()
{
	Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::GetTriggerDataByHash( s_uSelectedTriggerHash );
	if( pxData )
	{
		pxData->TriggerData.m_pxLayerTransition->m_uTrack = ASURA_HASH_ID_UNSET;

		if( s_pxTRANS_TrackNameDialog )
		{
			s_pxTRANS_TrackNameDialog->UpdateLabel("Next Track: [None]");
		}
		s_bTRANS_HasTrackToTransition = false;

		delete s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ];
		s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ] = 0;

		//these will have been deleted
		s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_INTRO ] = 0;
		s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_BODY  ] = 0;

		s_pxTRANS_UseIntroTick->SetEnabled( false );

		s_pxTRANS_ChannelDialog->SetSnapInterval( -1.0f );

		Tools_SaveAsura::SetDynamicMusicModified();
	}
}

void Tools_Overlay_DynamicMusic::Button_TRANS_ReduceTime ()
{
	Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::GetTriggerDataByHash( s_uSelectedTriggerHash );
	if( pxData )
	{
		pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[2].m_fTime = Asura_Maths::Max( pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[2].m_fTime - 1.0f, pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[1].m_fTime );
		if( s_pxTRANS_ChannelDialog )
		{
			s_pxTRANS_ChannelDialog->SetGraphDomain(0.0f, pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[2].m_fTime);
		}
	}
}


void Tools_Overlay_DynamicMusic::Button_TRANS_IncreaseTime ()
{
	Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::GetTriggerDataByHash( s_uSelectedTriggerHash );
	if( pxData )
	{
		pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[2].m_fTime = Asura_Maths::Min( pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[2].m_fTime + 1.0f, 120.0f );
		if( s_pxTRANS_ChannelDialog )
		{
			s_pxTRANS_ChannelDialog->SetGraphDomain(0.0f, pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[2].m_fTime);
		}
	}
}

Asura_Hash_ID Tools_Overlay_DynamicMusic::PromptForDynamicMusicTrack ()
{
	ListSelection_Overlay xOverlay("Select Dynamic Music Track");

	for( int i = 0; i < Asura_Dynamic_Music_System::GetNumberOfTracks(); i++ )
	{
		Asura_Dynamic_Music_TrackData* pxTrack = Asura_Dynamic_Music_System::GetTrackByIndex( i );
		xOverlay.AddItem( pxTrack->m_xName.GetString(), static_cast<int>( Asura_GetHashID( pxTrack->m_xName.GetString() ) ) );
	}

	//show selection overlay
	if(!xOverlay.DoModal()) 
	{
		return ASURA_HASH_ID_UNSET;
	}
	return static_cast<Asura_Hash_ID>( xOverlay.GetSelectionData() );
}

void Tools_Overlay_DynamicMusic::ClearOverlayControlPointers ()
{
	//Clear down all the pointers to overlay controls

	//Transition trigger
	s_pxTRANS_TrackNameDialog		= 0;
	s_pxTRANS_ChannelDialog			= 0;

	for(int i = 0; i < NUM_CHANNELS; ++i)
	{
		s_apxTRANS_Channels[i] = 0;
	}
	for(int i = 0; i < NUM_CHANNEL_BLOCKS; ++i)
	{
		s_apxTRANS_ChannelBlocks[i] = 0;
	}

	//Join release
	for( int iLayer = 0; iLayer < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++iLayer )
	{
		s_apxJOINREL_ChannelDialog		[iLayer] = 0;
		s_abJOINREL_ChannelDialogModified[iLayer] = false;

		s_aiJOINREL_ActionTypeIndex		[iLayer] = 0;
		s_abJOINREL_ActionTypeChanged	[iLayer] = false;
	}

	//Stop
	s_pxSTOP_ChannelDialog				= 0;
	s_pxSTOP_ChannelDialogOutroBlock	= 0;
}

void Tools_Overlay_DynamicMusic::RecreateOverlay ()
{
	//Make sure nothing is dangling
	ClearOverlayControlPointers ();

	// This will only exist for Code Triggers
	if( Asura_Overlay_System::DoesOverlayExist( TOOLS_OVERLAY_DMUSIC_TRIGGER_EDIT ) )
	{
		CloseTriggerEditOverlay();
		CreateOverlay_SingleTrigger();
	}
}

void Tools_Overlay_DynamicMusic::AddTriggerOverlayControls_Play (Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset, Asura_Dynamic_Music_Trigger_Data* pxData)
{
	Asura_Dialog*	pxDialog;

	Asura_Dynamic_Music_TrackData* pxTrack = Asura_Dynamic_Music_System::GetTrackByHash( pxData->TriggerData.m_pxPlay->m_uTrack );

	char szText[128];
	if(pxTrack)
	{
		Asura_PrintfToBfr(szText,"Track: %s", 128, true, pxTrack->m_xName.GetString() );
	}
	else
	{
		Asura_PrintfToBfr(szText,"Track: [None]", 128, true );
	}

	pxDialog = new Asura_Function_Dialog(szText, xOffset, Button_PLAY_SelectTrack);
	pxDialog->SetHelpText("Select the track into which to transition");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();

	for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++i )
	{
		Asura_PrintfToBfr(szText,"Start with %s muted", 128, true, Asura_Dynamic_Music_System::GetLayerName( i ) );
		pxDialog = new Asura_Tickbox_Dialog( szText, xOffset, &pxData->TriggerData.m_pxPlay->m_bMuteLayers[i], &s_bGenericChangeRequiringSave );
		xOffset.y += Asura_Dialog::DefaultSpacing ();

		pxOverlay->AddDialog(pxDialog);
	}
}

void Tools_Overlay_DynamicMusic::Button_PLAY_SelectTrack ()
{
	Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::GetTriggerDataByHash( s_uSelectedTriggerHash );

	if( pxData )
	{
		Asura_Hash_ID xNewTrack = PromptForDynamicMusicTrack();
		if( xNewTrack == ASURA_HASH_ID_UNSET )
		{
			return;
		}

		pxData->TriggerData.m_pxPlay->m_uTrack = xNewTrack;

		Tools_SaveAsura::SetDynamicMusicModified();

		RecreateOverlay ();
		Tools_Entity_DynamicMusicTrigger_Object::RecreateCurrentOverlay();
	}
}

void Tools_Overlay_DynamicMusic::AddTriggerOverlayControls_Stop (Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset, Asura_Dynamic_Music_Trigger_Data* pxData)
{
	if( !pxData )
	{
		return;
	}

	Asura_Dynamic_Music_Stop_Trigger_Data* pxTriggerData = pxData->TriggerData.m_pxStop;

	if(!pxTriggerData)
	{
		return;
	}

	Asura_Dialog*	pxDialog;

	pxDialog = new Asura_Tickbox_Dialog( "Unqueue all triggers", xOffset, &pxTriggerData->m_bUnqueueAllTriggers, &s_bGenericChangeRequiringSave );
	pxDialog->SetHelpText( "Tick this to clear the triggers queue and start this trigger immediately on activation" );
	pxOverlay->AddDialog( pxDialog );

	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog( "Activate on bar boundry", xOffset, &pxTriggerData->m_bActivateOnBarBoundry, &s_bGenericChangeRequiringSave );
	if( pxTriggerData->m_bJustOutro )
	{
		pxDialog->SetEnabled( false );
	}
	pxDialog->SetHelpText( "Tick this to wait for a bar boundry before activating the trigger" );
	pxOverlay->AddDialog( pxDialog );

	xOffset.y += Asura_Dialog::DefaultSpacing();

	s_bSTOP_JustOutroTick = pxTriggerData->m_bJustOutro;

	pxDialog = new Asura_Tickbox_Dialog( "Just play outro", xOffset, &s_bSTOP_JustOutroTick );
	pxDialog->SetHelpText( "Tick this to bypass the body of the track and immediately play the outro on activation" );
	pxOverlay->AddDialog( pxDialog );

	xOffset.y += Asura_Dialog::LargeSpacing ();

	s_bSTOP_ChannelDialogModified = false;
	s_pxSTOP_ChannelDialog = new Asura_Channel_Graph_Dialog( xOffset, 35, &s_bSTOP_ChannelDialogModified );
	s_pxSTOP_ChannelDialog->SetGraphDomain(0.0f, pxTriggerData->m_xTrackAmplitude[ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION - 1].m_fTime);
	//Add a channel
	Asura_Channel_Graph_Channel* pxChannel = s_pxSTOP_ChannelDialog->AddChannel();

	if( pxChannel )
	{
		pxChannel->SetDetachableFlag		( false );
		pxChannel->SetRatioResizingFlag		( false );
		pxChannel->SetAnchorStartBlockFlag	( true );

		//Add a block representing the body
		if( pxTriggerData->m_bJustOutro )
		{
			s_pxSTOP_ChannelDialogOutroBlock = pxChannel->AddBlock( "Outro", 0.0f, pxTriggerData->m_xTrackAmplitude[ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION - 1].m_fTime, false, false );
		}
		else
		{
			pxChannel->AddBlock( "Body", 0.0f, pxTriggerData->m_fBodyEndTime, true, false );
			s_pxSTOP_ChannelDialogOutroBlock = pxChannel->AddBlock( "Outro", pxTriggerData->m_fBodyEndTime, pxTriggerData->m_xTrackAmplitude[ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION - 1].m_fTime, true, false );
		}

		//Add amplitude curve points
		for( int iPoint = 0; iPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION; ++iPoint )
		{
			bool bAnchor = bool( ( iPoint == 0 ) || ( iPoint == ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION - 1 ) );
			pxChannel->SetControlPoint (iPoint, 
				pxTriggerData->m_xTrackAmplitude[iPoint].m_fTime, 
				pxTriggerData->m_xTrackAmplitude[iPoint].m_fAmplitude,
				bAnchor, 
				bAnchor);
		}
	}

	pxOverlay->AddDialog( s_pxSTOP_ChannelDialog );

	xOffset.y += 5.0f;
}


void Tools_Overlay_DynamicMusic::AddTriggerOverlayControls_JoinRel (Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset, Asura_Dynamic_Music_Trigger_Data* pxData)
{
	if( !pxData )
	{
		return;
	}

	Asura_Dynamic_Music_JoinRelease_Trigger_Data* pxTriggerData = pxData->TriggerData.m_pxJoinRelease;

	if(!pxTriggerData)
	{
		return;
	}

	Asura_Dialog*	pxDialog;

	pxDialog = new Asura_Tickbox_Dialog( "Activate on bar boundry", xOffset, &pxTriggerData->m_bActivateOnBarBoundry, &s_bGenericChangeRequiringSave );
	pxDialog->SetHelpText( "Tick this to wait for a bar boundry before activating the trigger" );
	pxOverlay->AddDialog( pxDialog );

	xOffset.y += Asura_Dialog::LargeSpacing ();

	char szText[128];

	for( int iLayer = 0; iLayer < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++iLayer )
	{
		Asura_PrintfToBfr(szText,"Changes to %s", 128, true, Asura_Dynamic_Music_System::GetLayerName( iLayer ) );

		pxDialog = new Asura_Dialog(szText, xOffset);
		pxOverlay->AddDialog( pxDialog );

		const float fOffset = pxDialog->GetWidth() + 2.0f;
		xOffset.x += fOffset;

		s_aiJOINREL_ActionTypeIndex[iLayer] = static_cast<int>(pxTriggerData->m_xLayerChanges[iLayer].m_eAction);

		pxDialog = new Asura_DropDown_List_Dialog( xOffset, g_aszJOINREL_ActionTypes, &s_aiJOINREL_ActionTypeIndex[iLayer], 3, 3, 0, &s_abJOINREL_ActionTypeChanged[iLayer], "" );
		pxDialog->SetHelpText("Select the action to perform on this layer");
		pxOverlay->AddDialog(pxDialog);

		xOffset.x -= fOffset;

		switch( s_aiJOINREL_ActionTypeIndex[iLayer] )
		{
		case Asura_Dynamic_Music_JoinRelease_Trigger_Data::ACTION_NONE:
			{
				//Nothing to show if there is no action
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}
			break;

		case Asura_Dynamic_Music_JoinRelease_Trigger_Data::ACTION_ENABLE:
		case Asura_Dynamic_Music_JoinRelease_Trigger_Data::ACTION_DISABLE:
			{
				xOffset.y += Asura_Dialog::DefaultSpacing();

				s_apxJOINREL_ChannelDialog[iLayer] = new Asura_Channel_Graph_Dialog( xOffset, 35, &s_abJOINREL_ChannelDialogModified[iLayer] );
				s_apxJOINREL_ChannelDialog[iLayer]->SetGraphDomain(0.0f, pxTriggerData->m_xLayerChanges[iLayer].m_axAmplitude[ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION - 1].m_fTime);
				//Add a channel
				Asura_Channel_Graph_Channel* pxChannel = s_apxJOINREL_ChannelDialog[iLayer]->AddChannel();

				if( pxChannel )
				{
					pxChannel->SetDetachableFlag		( false );
					pxChannel->SetRatioResizingFlag		( false );
					pxChannel->SetAnchorStartBlockFlag	( true );

					//Add a block representing the body
					pxChannel->AddBlock( "Body" , 0.0f , pxTriggerData->m_xLayerChanges[iLayer].m_axAmplitude[ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION - 1].m_fTime, false, false );

					//Add amplitude curve points
					for( int iPoint = 0; iPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION; ++iPoint )
					{
						bool bAnchorX = bool( ( iPoint == 0 ) || ( iPoint == ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION - 1 ) );
						pxChannel->SetControlPoint (iPoint, 
							pxTriggerData->m_xLayerChanges[iLayer].m_axAmplitude[iPoint].m_fTime, 
							pxTriggerData->m_xLayerChanges[iLayer].m_axAmplitude[iPoint].m_fAmplitude,
							bAnchorX, 
							true);
					}
				}

				pxOverlay->AddDialog( s_apxJOINREL_ChannelDialog[iLayer] );

				xOffset.y += 5.0f;
			}
			break;

		default:
			{
				Asura_Assert(false, "Unknown join/release action");
			}
			break;
		}

		if( iLayer != ( ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS - 1 ) )
		{
			xOffset.y += Asura_Dialog::DefaultSpacing();
		}
	}
}

void Tools_Overlay_DynamicMusic::Button_TrigEdit_TestTrigger ()
{
	Asura_Assert( s_uSelectedTriggerHash != ASURA_HASH_ID_UNSET, "Invalid Trigger hash");

	if( s_uSelectedTriggerHash != ASURA_HASH_ID_UNSET )
	{
		Asura_Dynamic_Music_System::ActivateTrigger( s_uSelectedTriggerHash );
	}
}

void Tools_Overlay_DynamicMusic::Button_TrigEdit_HaltPlayback ()
{
	Asura_Dynamic_Music_System::Stop();
}

bool Tools_Overlay_DynamicMusic::Enabled_TestTrigger ()
{
	Asura_Assert( s_uSelectedTriggerHash != ASURA_HASH_ID_UNSET, "Invalid selected trigger");

	if( s_uSelectedTriggerHash != ASURA_HASH_ID_UNSET )
	{
		return !Asura_Dynamic_Music_System::IsTriggerActive ( s_uSelectedTriggerHash );
	}
	return false;
}

bool Tools_Overlay_DynamicMusic::Enabled_HaltPlayback ()
{
	return Asura_Dynamic_Music_System::IsPlaying ();
}

void Tools_Overlay_DynamicMusic::Button_TrigList_CreateCodeTrigger()
{
	Asura_String xName;
	if (!Tools_Object_System::AskForName(xName, "Dynamic Music Trigger", 0, false, Tools_Entity_DynamicMusicTrigger_Object::GetNamePrefix()))
	{
		return;
	}

	//Create a new trigger in the dynamic music system
	Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::CreateNewTriggerData(true);
	if( pxData )
	{
		pxData->m_uHash = xName.GetHash();
		pxData->SetTriggerType( DYNAMIC_MUSIC_TRIGGER_PLAY );
		pxData->m_xCodeTriggerName = xName;

		Tools_Overlay_DynamicMusic::TriggersHaveChanged();
		Tools_SaveAsura::SetDynamicMusicModified();
	}
	else
	{
		Asura_Assert(false, "Unable to create Dynamic music Trigger data");
	}
}

void Tools_Overlay_DynamicMusic::Button_TrigList_TestTrigger		()
{
	// Get the selected trigger and activate it
	Asura_Hash_ID uTriggerHash = GetSelectedTriggerHash();

	if( uTriggerHash != ASURA_HASH_ID_UNSET)
	{
		Asura_Dynamic_Music_Trigger_Data* pxTriggerData = Asura_Dynamic_Music_System::GetTriggerDataByHash( uTriggerHash );
		if( pxTriggerData )
		{
			Asura_Dynamic_Music_System::ActivateTrigger( pxTriggerData->m_uHash);
		}
	}
}

void Tools_Overlay_DynamicMusic::Button_TrigList_JumpToTrig		()
{
	if( s_iEntityTriggerListSelectionIndex >= 0)
	{
		Asura_Hash_ID uTriggerHash = s_xEntityTriggerHashList[s_iEntityTriggerListSelectionIndex];
		Asura_Dynamic_Music_Trigger_Data* pxTriggerData = Asura_Dynamic_Music_System::GetTriggerDataByHash( uTriggerHash );
		if( pxTriggerData )
		{
			Asura_Assert(!pxTriggerData->m_bCodeTrigger, "Trying to jump to Code Trigger!");

			Asura_ServerEntity_DynamicMusicTrigger* pxEntity = Asura_ServerEntity_DynamicMusicTrigger::FindEntityByHash(pxTriggerData->m_uHash);

			if( pxEntity )
			{
				Asura_Vector_3 xPosition = pxEntity->ToolsData->Position;
				Asura_Vector_3 xFwdVec;
				Asura_Camera::GetForwardVector(xFwdVec);
				xPosition -= xFwdVec * 2.0f;
				Asura_Matrix_3x3 xMat = *Asura_Camera::GetOrientation();
				Tools_Movement::TeleportCameraToPosition(&xPosition, &xMat);
			}
		}
	}
}

void Tools_Overlay_DynamicMusic::Button_TrigList_ShowTrigProps	()
{
	// Get the selected trigger hash
	Asura_Hash_ID uTriggerHash = GetSelectedTriggerHash();

	if( uTriggerHash != ASURA_HASH_ID_UNSET)
	{
		Asura_Dynamic_Music_Trigger_Data* pxOldTriggerData = NULL;
		if (s_uSelectedTriggerHash != ASURA_HASH_ID_UNSET)
		{
			pxOldTriggerData = Asura_Dynamic_Music_System::GetTriggerDataByHash( s_uSelectedTriggerHash );
		}

		Asura_Dynamic_Music_Trigger_Data* pxNewTriggerData = Asura_Dynamic_Music_System::GetTriggerDataByHash( uTriggerHash );

		if( pxNewTriggerData )
		{
			// Find its server entity
			Asura_ServerEntity_DynamicMusicTrigger* pxOldEntity = Asura_ServerEntity_DynamicMusicTrigger::FindEntityByHash(s_uSelectedTriggerHash);
			Asura_ServerEntity_DynamicMusicTrigger* pxNewEntity = Asura_ServerEntity_DynamicMusicTrigger::FindEntityByHash(uTriggerHash);

			//
			// Deselect old
			//
			if (pxOldEntity)
			{
				Tools_Object_System::DeselectObject( pxOldEntity->ToolsData->ToolsObject );
			}

			if (pxOldTriggerData && pxOldTriggerData->m_bCodeTrigger)
			{
				CloseTriggerEditOverlay();
			}

			//
			// Select new
			//
			if (pxNewTriggerData->m_bCodeTrigger)
			{
				CreateOverlay_SingleTrigger();
			}
			else
			{
				if( pxNewEntity )
				{
					Tools_Object_System::SelectObject( pxNewEntity->ToolsData->ToolsObject );
				}
			}
		}
	}
}


void Tools_Overlay_DynamicMusic::Button_TrigList_RenameTrigger ()
{
	Asura_Hash_ID uTriggerHash = GetSelectedTriggerHash();

	if( uTriggerHash != ASURA_HASH_ID_UNSET )
	{
		Asura_Dynamic_Music_Trigger_Data* pxTriggerData = Asura_Dynamic_Music_System::GetTriggerDataByHash( uTriggerHash );
		if (pxTriggerData && pxTriggerData->m_bCodeTrigger)
		{
			Asura_String xNewName(pxTriggerData->m_xCodeTriggerName);
			if(Tools_Object_System::AskForName(xNewName, "Rename Trigger :" ))
			{
				pxTriggerData->m_xCodeTriggerName = xNewName;

				// recalc new name hash
				pxTriggerData->m_uHash = xNewName.GetHash();
				Tools_SaveAsura::SetEntityModified();

				Tools_Overlay_DynamicMusic::TriggersHaveChanged();
				Tools_SaveAsura::SetDynamicMusicModified();
			}
		}
	}
}

void Tools_Overlay_DynamicMusic::Button_TrigList_DeleteTrigger ()
{
	// "Are you sure you want to delete" box?

	Asura_Hash_ID uTriggerHash = GetSelectedTriggerHash();

	if( uTriggerHash != ASURA_HASH_ID_UNSET )
	{
		Asura_Dynamic_Music_System::Stop();

		Asura_Dynamic_Music_Trigger_Data* pxTriggerData = Asura_Dynamic_Music_System::GetTriggerDataByHash( uTriggerHash );
		if (pxTriggerData && pxTriggerData->m_bCodeTrigger)
		{
			//delete entity?
	/*		Asura_ServerEntity_DynamicMusicTrigger* pxEntity = Asura_ServerEntity_DynamicMusicTrigger::FindEntityByHash(uTriggerHash);
			if (pxEntity)
			{
				Tools_Object_System::DeleteObject( pxEntity->ToolsData->ToolsObject );
			}
	*/
			Asura_Dynamic_Music_System::DeleteTriggerData( uTriggerHash );

			Tools_SaveAsura::SetEntityModified();

			Tools_Overlay_DynamicMusic::TriggersHaveChanged();
			Tools_SaveAsura::SetDynamicMusicModified();

			// if this is the one in the trigger edit window, then close it
			if (uTriggerHash == s_uSelectedTriggerHash)
			{
				Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_DMUSIC_TRIGGER_EDIT);
			}
		}
	}
}

void Tools_Overlay_DynamicMusic::Button_TrigList_CloseOverlay	()
{
	DeleteTriggerListArray();

	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_DMUSIC_TRIGLIST );
}


void Tools_Overlay_DynamicMusic::TriggersHaveChanged ()
{
	// If the triggers have changed and our list is open, refresh it
	if( Asura_Overlay_System::DoesOverlayExist( TOOLS_OVERLAY_DMUSIC_TRIGLIST ) )
	{
		CreateOverlay_TriggerList ();
	}
}


void Tools_Overlay_DynamicMusic::ValidateTriggers( )
{
	// This function checks for invalid (garbage) triggers in the DMS and removes them
	// it should be removed at some point since it just fixes problems from a bug
	int iRemoved = 0;

	int iNumTriggers = Asura_Dynamic_Music_System::GetNumberOfTriggerData();
	if( iNumTriggers > 0 )
	{	
		for( int i = iNumTriggers - 1; i >= 0 ; i-- )
		{
			Asura_Dynamic_Music_Trigger_Data* pxTriggerData = Asura_Dynamic_Music_System::GetTriggerDatabyIndex( i );

			if( pxTriggerData )
			{
				if (!pxTriggerData->m_bCodeTrigger)
				{
					Asura_ServerEntity_DynamicMusicTrigger* pxServerEntity = Asura_ServerEntity_DynamicMusicTrigger::FindEntityByHash(pxTriggerData->m_uHash);
					if (pxServerEntity)
					{
						Tools_Entity_Object* pxEntity = pxServerEntity->ToolsData->ToolsObject;

						if( !pxEntity || ( pxEntity->Type != TOT_Entity_DynamicMusicTrigger ) )
						{
							Asura_Dynamic_Music_System::DeleteTriggerData( pxTriggerData->m_uHash );
							++iRemoved;
						}
					}
					// we've got no entity that matches this trigger data
					else
					{
						Asura_Dynamic_Music_System::DeleteTriggerData( pxTriggerData->m_uHash );
						++iRemoved;
					}
				}
			}
		}
	}
	Asura_Assertf( iRemoved == 0, ( "Removed %d dynamic music triggers which were corrupt or invalid", iRemoved ) );
}

// per frame update when code trigger overlay is active
bool Tools_Overlay_DynamicMusic::UpdateTriggerEditOverlay ()
{
	bool bRecreateMain = false;
	bool bRecreateSub = false;

	if (s_uSelectedTriggerHash == ASURA_HASH_ID_UNSET)
	{
		return false;
	}

	Asura_Dynamic_Music_Trigger_Data* pxData = Asura_Dynamic_Music_System::GetTriggerDataByHash( s_uSelectedTriggerHash );
	Asura_Assert( pxData, "Failed to find trigger data in dynamic music system" );

	if( !pxData )
	{
		// things have gone horribly wrong
		return false;
	}

	if( s_bTriggerTypeChanged )
	{
		s_bTriggerTypeChanged = false;

		pxData->SetTriggerType( static_cast<Asura_Dynamic_Music_TriggerTypes>( s_iTriggerTypeSelectionIndex ) );
		Tools_SaveAsura::SetDynamicMusicModified();

		RecreateOverlay ();
		bRecreateMain = true;

		// recreate the triggers list, if we have one
		Tools_Overlay_DynamicMusic::TriggersHaveChanged();
	}
	else
	{
		//Update using the appropriate trigger update function
		switch(pxData->m_eType)
		{
			case DYNAMIC_MUSIC_TRIGGER_TRANSITION	: bRecreateSub = UpdateTriggerEditOverlay_Trans	( pxData );	break;
			case DYNAMIC_MUSIC_TRIGGER_PLAY			: bRecreateSub = UpdateTriggerEditOverlay_Play		( pxData );	break;
			case DYNAMIC_MUSIC_TRIGGER_STOP			: bRecreateSub = UpdateTriggerEditOverlay_Stop		( pxData );	break;
			case DYNAMIC_MUSIC_TRIGGER_JOINRELEASE	: bRecreateSub = UpdateTriggerEditOverlay_JoinRel	( pxData );	break;

			default:
				{
					Asura_Assert( false, "Unrecognised trigger type in dynamic music trigger" );
				}
		}

		if( s_bGenericChangeRequiringSave )
		{
			s_bGenericChangeRequiringSave = false;
			Tools_SaveAsura::SetDynamicMusicModified();
		}
	}
	return (bRecreateMain || bRecreateSub);
}

bool Tools_Overlay_DynamicMusic::UpdateTriggerEditOverlay_Trans ( Asura_Dynamic_Music_Trigger_Data* pxData )
{
	bool bRecreate = false;

	if( !pxData )
	{
		return false;
	}
	if(!s_pxTRANS_ChannelDialog) return false;

	//Check the 'use intro' checkbox
	if( s_bTRANS_HasTrackToTransition && ( s_bTRANS_UseIntroTick != pxData->TriggerData.m_pxLayerTransition->m_bUseIntro ) )
	{
		pxData->TriggerData.m_pxLayerTransition->m_bUseIntro = s_bTRANS_UseIntroTick;
		if( !s_bTRANS_UseIntroTick )
		{
			for( int iPoint = 0; iPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION; ++iPoint )
			{
				if( pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[iPoint].m_fTime < pxData->TriggerData.m_pxLayerTransition->m_fNextIntroEndOffset )
				{
					pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[iPoint].m_fTime = pxData->TriggerData.m_pxLayerTransition->m_fNextIntroEndOffset;
				}
			}
		}

		bRecreate = true;
		RecreateOverlay ();
	}

	if( s_bTRANS_ChannelDialogModified )
	{
		s_bTRANS_ChannelDialogModified = false;
		//Save the changes that were made to the channel dialog

		//Previous track data
		if( s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_PREV_OUTRO ] )
		{
			pxData->TriggerData.m_pxLayerTransition->m_fPrevOutroStartOffset	= s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_PREV_OUTRO ]->GetStartPosition();
			pxData->TriggerData.m_pxLayerTransition->m_fPrevOutroEndOffset		= s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_PREV_OUTRO ]->GetEndPosition	();
		}
		if( s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ] )
		{
			ASURA_STATIC_ASSERT ( ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION == 3 );
			for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION; ++i )
			{
				pxData->TriggerData.m_pxLayerTransition->m_xPrevTrackAmplitude[i].m_fTime		= s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ]->GetControlPointX( i );
				pxData->TriggerData.m_pxLayerTransition->m_xPrevTrackAmplitude[i].m_fAmplitude	= s_apxTRANS_Channels[ CHANNEL_PREVIOUS_TRACK ]->GetControlPointY( i );
			}
		}

		//Next track data
		if( s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_INTRO ] )
		{
			pxData->TriggerData.m_pxLayerTransition->m_fNextIntroStartOffset	= s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_INTRO ]->GetStartPosition();
			pxData->TriggerData.m_pxLayerTransition->m_fNextIntroEndOffset		= s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_INTRO ]->GetEndPosition	();

			//Make the block a nice colour if it is representing the correct duration
			const float fDuration = pxData->TriggerData.m_pxLayerTransition->m_fNextIntroEndOffset - pxData->TriggerData.m_pxLayerTransition->m_fNextIntroStartOffset;
			Asura_Dynamic_Music_TrackData* pxTrackData = Asura_Dynamic_Music_System::GetTrackByHash( pxData->TriggerData.m_pxLayerTransition->m_uTrack );
			if( pxTrackData && RoughlyEqual( fDuration, pxTrackData->m_fBodyOffset ) )
			{
				s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_INTRO ]->SetColour( uTOOLS_TRIGGER_ENTITY_LIGHT_GREEN );
			}
			else
			{
				s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_INTRO ]->SetColour( 0x00FFFFFF );
			}
		}
		if( s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ] )
		{
			ASURA_STATIC_ASSERT ( ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION == 3 );
			for( int i = 0; i < ASURA_DYNAMIC_MUSIC_SYSTEM_FADE_RESOLUTION; ++i )
			{
				pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[i].m_fTime		= s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]->GetControlPointX( i );
				pxData->TriggerData.m_pxLayerTransition->m_xNewTrackAmplitude[i].m_fAmplitude	= s_apxTRANS_Channels[ CHANNEL_NEXT_TRACK ]->GetControlPointY( i );
			}
			pxData->TriggerData.m_pxLayerTransition->m_fNextIntroEndOffset = s_apxTRANS_ChannelBlocks[ CHANNEL_BLOCK_NEXT_BODY ]->GetStartPosition();
		}
		Tools_SaveAsura::SetDynamicMusicModified();
	}

	//Update the playback time
	s_pxTRANS_ChannelDialog->HidePlaybackMarker();
	if( s_pxTRANS_ChannelDialog && Asura_Dynamic_Music_System::IsTriggerActive( pxData->m_uHash ) )
	{
		const float fPlayTime = Asura_Dynamic_Music_System::GetTriggerPlayTime( pxData->m_uHash );
		s_pxTRANS_ChannelDialog->SetPlaybackMarker( fPlayTime );
		s_pxTRANS_ChannelDialog->ShowPlaybackMarker();
	}

	return (bRecreate);
}


bool Tools_Overlay_DynamicMusic::UpdateTriggerEditOverlay_Play ( Asura_Dynamic_Music_Trigger_Data* pxData )
{
	//Nothing to do
	return false;
}


bool Tools_Overlay_DynamicMusic::UpdateTriggerEditOverlay_Stop ( Asura_Dynamic_Music_Trigger_Data* pxData )
{
	bool bRecreate = false;
	Asura_Dynamic_Music_Stop_Trigger_Data* pxTriggerData = pxData->TriggerData.m_pxStop;

	if(!pxTriggerData)
	{
		return false;
	}
	if(!s_pxSTOP_ChannelDialog) return false;

	if( s_bSTOP_ChannelDialogModified)
	{
		s_bSTOP_ChannelDialogModified = false;
		//Copy the values out of the channel graph dialog
		Asura_Channel_Graph_Channel* pxChannel = s_pxSTOP_ChannelDialog->GetChannelByIndex(0);

		if( pxChannel )
		{
			for( int iControlPoint = 0; iControlPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_STOP_RESOLUTION; ++iControlPoint )
			{
				pxTriggerData->m_xTrackAmplitude[iControlPoint].m_fTime			= pxChannel->GetControlPointX( iControlPoint );
				pxTriggerData->m_xTrackAmplitude[iControlPoint].m_fAmplitude	= pxChannel->GetControlPointY( iControlPoint );
			}
			if( !pxTriggerData->m_bJustOutro && s_pxSTOP_ChannelDialogOutroBlock )
			{
				pxTriggerData->m_fBodyEndTime = s_pxSTOP_ChannelDialogOutroBlock->GetStartPosition();
			}
		}
		Tools_SaveAsura::SetDynamicMusicModified();
	}

	//Update the play time marker
	float fPlayTime = 0.0f;
	if( Asura_Dynamic_Music_System::IsTriggerActive( pxData->m_uHash ) )
	{
		fPlayTime = Asura_Dynamic_Music_System::GetTriggerPlayTime( pxData->m_uHash );
	}

	if (s_pxSTOP_ChannelDialog)
	{
		if( fPlayTime > 0.0f )
		{
			s_pxSTOP_ChannelDialog->ShowPlaybackMarker	();
			s_pxSTOP_ChannelDialog->SetPlaybackMarker	(fPlayTime);
		}
		else
		{
			s_pxSTOP_ChannelDialog->HidePlaybackMarker();
		}
	}

	//Check the 'just use outro' tick
	if( s_bSTOP_JustOutroTick != pxTriggerData->m_bJustOutro )
	{
		pxTriggerData->m_bJustOutro = s_bSTOP_JustOutroTick;
		if( pxTriggerData->m_bJustOutro )
		{
			pxTriggerData->m_bActivateOnBarBoundry = true;
		}

		Tools_SaveAsura::SetDynamicMusicModified();

		bRecreate = true;
		RecreateOverlay ();
	}
	return bRecreate;
}


bool Tools_Overlay_DynamicMusic::UpdateTriggerEditOverlay_JoinRel ( Asura_Dynamic_Music_Trigger_Data* pxData )
{
	bool bRecreate = false;

	Asura_Dynamic_Music_JoinRelease_Trigger_Data* pxTriggerData = pxData->TriggerData.m_pxJoinRelease;

	if(!pxTriggerData)
	{
		return false;
	}

	//Update the playback time
	float fPlayTime = 0.0f;
	if( Asura_Dynamic_Music_System::IsTriggerActive( pxData->m_uHash ) )
	{
		fPlayTime = Asura_Dynamic_Music_System::GetTriggerPlayTime( pxData->m_uHash );
	}

	for( int iLayer = 0; iLayer < ASURA_DYNAMIC_MUSIC_SYSTEM_NUM_LAYERS; ++iLayer )
	{
		if( s_abJOINREL_ActionTypeChanged[iLayer] )
		{
			s_abJOINREL_ActionTypeChanged[iLayer] = false;

			pxTriggerData->ChangeLayerAction( iLayer, static_cast<Asura_Dynamic_Music_JoinRelease_Trigger_Data::Actions>( s_aiJOINREL_ActionTypeIndex[iLayer] ) );

			bRecreate = true;
			RecreateOverlay ();

			Tools_SaveAsura::SetDynamicMusicModified();
		}
		else
		{
			if( s_abJOINREL_ChannelDialogModified[iLayer] )
			{
				s_abJOINREL_ChannelDialogModified[iLayer] = false;
				Asura_Assert( s_apxJOINREL_ChannelDialog[iLayer], "You forgot to reset the modified bool somewhere" );
				if( s_apxJOINREL_ChannelDialog[iLayer] )
				{
					//Update our trigger with the modified information
					Asura_Channel_Graph_Channel* pxChannel = s_apxJOINREL_ChannelDialog[iLayer]->GetChannelByIndex(0);

					for( int iPoint = 0; iPoint < ASURA_DYNAMIC_MUSIC_SYSTEM_JOIN_REL_RESOLUTION; ++iPoint )
					{
						pxTriggerData->m_xLayerChanges[ iLayer ].m_axAmplitude[iPoint].m_fAmplitude = pxChannel->GetControlPointY( iPoint );
						pxTriggerData->m_xLayerChanges[ iLayer ].m_axAmplitude[iPoint].m_fTime		= pxChannel->GetControlPointX( iPoint );
					}
					Tools_SaveAsura::SetDynamicMusicModified();
				}
			}

			if( s_apxJOINREL_ChannelDialog[iLayer] )
			{
				//Update the play time marker
				if( fPlayTime > 0.0f )
				{
					s_apxJOINREL_ChannelDialog[iLayer]->ShowPlaybackMarker	();
					s_apxJOINREL_ChannelDialog[iLayer]->SetPlaybackMarker	(fPlayTime);
				}
				else
				{
					s_apxJOINREL_ChannelDialog[iLayer]->HidePlaybackMarker();
				}
			}
		}
	}

	return bRecreate;
}

#endif //ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM

//eof
