//
// Tools_Phonon_Object.cpp
//
// Created by Richard Rice - 05/06/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"

#include "Sound/Tools_Phonon_Object.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Triggers/Asura_ServerEntity_SoundController.h"
#include "Entity/Triggers/Asura_ClientEntity_SoundController.h"
#include "Entity/Asura_Entity_Classification.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Mesh/Asura_Mesh.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Text_Output.h"
#include "Sound/Asura_Phonon.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Sound_Event_System.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "Volume/Asura_VolumeGroup.h"

#include "Tools_System.h"
#include "Tools_Overlays.h"
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Entity_Object.h"
#include "Lighting/Tools_LightSets.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_Sound_Export.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_VolumeEdit_Object.h"
#include "Objects/Tools_VolumeGroupEdit_Controller.h"
#include "Objects/Tools_VolumeBaseEdit_Object.h"

#include "Objects/Tools_ObjectSystem_Param.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Sound/Tools_Mode_Sound.h"
#include "Sound/Tools_Sound_PS2Settings.h"
#include "Sound/Tools_PickSound.h"
#include "Sound/Tools_PickSoundEvent_Dialog.h"
#include "Sound/Tools_PickSoundEvent_Overlay.h"


static const char* NoSoundSelectedText = "* No Sound Selected";
static const char* NoFilenameAvailableText = "* Filename not available";



Asura_Phonon Tools_Phonon_Object::s_xTestPhonon;
Asura_Handle Tools_Phonon_Object::s_iTestPhononHandle = ASURA_INVALID_HANDLE;

Tools_Phonon_Object* Tools_Phonon_Object::s_pxSelectedObject = 0;

//declarations for params that get shared between all selected objects
TOOLS_DECLARE_SHARED_PARAM(bool,	0xBC765FBB,	Phonon_Spatial)
TOOLS_DECLARE_SHARED_PARAM(bool,	0x3DA24803,	Phonon_Cuboid)
TOOLS_DECLARE_SHARED_PARAM(bool,	0xA41CDD83,	Phonon_UseOrient)
//TOOLS_DECLARE_SHARED_PARAM(bool,	0x56266F2C,	Phonon_Random)
//TOOLS_DECLARE_SHARED_PARAM(bool,	0x3943096F,	Phonon_Active)
TOOLS_DECLARE_SHARED_PARAM(float,	0x14ED76A5,	Phonon_CuboidInnerX)
TOOLS_DECLARE_SHARED_PARAM(float,	0x14ED76A6,	Phonon_CuboidInnerY)
TOOLS_DECLARE_SHARED_PARAM(float,	0x14ED76A7,	Phonon_CuboidInnerZ)
TOOLS_DECLARE_SHARED_PARAM(float,	0x1F8FE920,	Phonon_CuboidOuterX)
TOOLS_DECLARE_SHARED_PARAM(float,	0x1F8FE921,	Phonon_CuboidOuterY)
TOOLS_DECLARE_SHARED_PARAM(float,	0x1F8FE922,	Phonon_CuboidOuterZ)
TOOLS_DECLARE_SHARED_PARAM(float,	0x1D209E90,	Phonon_InnerRange)
TOOLS_DECLARE_SHARED_PARAM(float,	0x53FF5B8B,	Phonon_OuterRange)
//TOOLS_DECLARE_SHARED_PARAM(float,	0xDB0EF8E3,	Phonon_MinVolume)
//TOOLS_DECLARE_SHARED_PARAM(float,	0xAD03E275,	Phonon_MaxVolume)
//TOOLS_DECLARE_SHARED_PARAM(float,	0x06B9D457,	Phonon_MinPitch)
//TOOLS_DECLARE_SHARED_PARAM(float,	0x70987085,	Phonon_MaxPitch)
TOOLS_DECLARE_SHARED_PARAM(bool,	0x49396175,	Phonon_Looping)
//TOOLS_DECLARE_SHARED_PARAM(float,	0xA3F4B05C,	Phonon_MinTriggerTime)
//TOOLS_DECLARE_SHARED_PARAM(float,	0x7CCCF20A,	Phonon_MaxTriggerTime)
TOOLS_DECLARE_SHARED_PARAM(bool,	0x9C114729,	Phonon_UseBoundingBox)
TOOLS_DECLARE_SHARED_PARAM(bool,	0x1af71076,	Phonon_Retriggerable)
TOOLS_DECLARE_SHARED_PARAM(bool,	0xD4AAAE08,	Phonon_InvertBoundingBox)

TOOLS_DECLARE_SHARED_PARAM(bool,	0x86584579,	Phonon_UseRetriggerRadius)
TOOLS_DECLARE_SHARED_PARAM(float,	0x566590AE,	Phonon_RetriggerRadius)
TOOLS_DECLARE_SHARED_PARAM(bool,	0x9D138520,	Phonon_UseRetriggerBoundingBox)

//Now uses the same ID for Active as Tools_Entity_Object
Tools_ObjectSystem_SharedValue< bool > g_xParam_Phonon_Active( g_uHash_Entity_StartsActive);


static const Asura_Hash_ID g_uHash_LightSource_LightSet0 = 0xBD2D1016; 
static const Asura_Hash_ID g_uHash_LightSource_LightSetLast = g_uHash_LightSource_LightSet0+TOOLS_MAX_LIGHT_SETS-1; 
static Tools_ObjectSystem_SharedValue<bool> g_axParam_LightSource_LightSet[TOOLS_MAX_LIGHT_SETS];

static const Asura_Hash_ID g_uHash_Phonon_Platform0 = 0x1BE79714; 
static const Asura_Hash_ID g_uHash_Phonon_PlatformLast = g_uHash_Phonon_Platform0+TOOLS_PLATFORM_NUM_PLATFORMS-1; 
static Tools_ObjectSystem_SharedValue<bool> g_axParam_Phonon_Platform[TOOLS_PLATFORM_NUM_PLATFORMS];



Tools_Phonon_Object::Tools_Phonon_Object(const Asura_Phonon* pxPhonon)
:Tools_Object(TOT_Phonon)
{

	Tools_Mode_Sound::PhononList.add_entry(this);


	m_uSoundGuid = pxPhonon->GetGuid();
	m_xUserPhonon.Initialise();
	m_xUserPhonon = *pxPhonon;

	Asura_String mesh_name = "ToolsShape_Sound";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);

	float fQuatMag = m_xUserPhonon.m_xOrient.MagnitudeSquared();
	if(fQuatMag<0.9f || fQuatMag > 1.1f)
	{
		//reset dodgy quats
		m_xUserPhonon.m_xOrient	= Asura_Maths::IdentityQuat;
	}
	
	Position = m_xUserPhonon.m_xInitialPosition;
	m_xUserPhonon.m_xOrient.ToMatrix(Orientation);

	m_bChanged = false;
	m_bMinimumModified = false;
	m_bMaximumModified = false;
	m_bOverlayNeedsRecreating = false;
    m_bStartsActiveChanged = false;

	m_pxClipVolumeController = 0;


	m_fPS2Frequency = 0;
	m_bPS2FrequencyModified = false;


	m_uVolumeEditID = 0;
	m_uRetriggerVolumeEditID = 0;

	m_bDontDisplaceBoundingBox = false;

	m_bDontDisplaceRetriggerBoundingBox = false;

	UpdateInitialValues();

	UpdateUserPhononName();
	UpdateName();

	//make sure the light sets in the entity match those in the sound. RAR 14/06/2006
	//And the platforms. RAR 15/04/2008
	Asura_ServerEntity_SoundController* pxEntity = Asura_ServerEntity_SoundController::GetControllerFromPhonon(GetPhonon()->GetGuid());
	if(pxEntity && pxEntity->ToolsData)
	{
		pxEntity->ToolsData->m_uLightSets = m_xUserPhonon.m_uLightSets;
		pxEntity->ToolsData->m_uPlatforms = m_xUserPhonon.m_uPlatforms;
	}

	SetDistantIconTexture("\\tools\\Sound.dds");

}

Tools_Phonon_Object::~Tools_Phonon_Object()
{
	if(m_uVolumeEditID) Tools_VolumeEdit_Object::Delete(m_uVolumeEditID);
	if(m_uRetriggerVolumeEditID) Tools_VolumeEdit_Object::Delete(m_uRetriggerVolumeEditID);

	if(Tools_Mode_Sound::PhononList.contains(this))
	{
		Tools_Mode_Sound::PhononList.delete_entry(this);
	}
	delete m_pxClipVolumeController;
}


void Tools_Phonon_Object::UpdatePositionInAsuraData(bool /*updateLinkedObjects*/)
{
	//Displace the bounding boxes as well if the position has changed
	if(Position != m_xUserPhonon.m_xInitialPosition)
	{
		MoveBoundingBoxes(Position-m_xUserPhonon.m_xInitialPosition);
	}

	m_xUserPhonon.m_xPosition = Position;
	m_xUserPhonon.m_xInitialPosition = Position;
	m_xUserPhonon.m_xOrient.FromMatrix(Orientation);
	m_bChanged = true;
}

void Tools_Phonon_Object::UpdateInitialValues()
{
}



void Tools_Phonon_Object::Render()
{
	
	if (Tools_Mode_Sound::ShowOuterRadius)
	{
		if (IsSelected() || (Tools_Object_System::ObjectUnderCursorList.contains(this) && !Tools_Object_System::SelectedObject))
		{
			// draw inner/outer spheres or cuboids depending on the sound
			if(m_xUserPhonon.m_bUseCuboidFalloff)
			{
				Asura_Bounding_Box inner;
				inner.Reset();
				inner.Set(m_xUserPhonon.m_xInnerCuboidRadius);	
				inner.Set(-m_xUserPhonon.m_xInnerCuboidRadius);	

				Asura_Bounding_Box outer;
				outer.Reset();
				outer.Set(m_xUserPhonon.m_xOuterCuboidRadius);	
				outer.Set(-m_xUserPhonon.m_xOuterCuboidRadius);
				
				Asura_Matrix_3x3 xOrient;
				xOrient.Identity();
				if(m_xUserPhonon.m_bUseOrient) xOrient = Orientation;	

				
				Tools_Render::RenderCube(inner,Position,xOrient,0x7f0000ff,false);
				Tools_Render::RenderCube(outer,Position,xOrient,0x7fff0000,false);

			}
			else
			{
				extern void Render_Sphere( const Asura_Vector_3& xCentre, float fRadius, int iColour );
				Render_Sphere(m_xUserPhonon.m_xInitialPosition,m_xUserPhonon.m_fInnerRadius,0x7f0000ff);
				Render_Sphere(m_xUserPhonon.m_xInitialPosition,m_xUserPhonon.m_fOuterRadius,0x7fff0000);
			}

			if(m_xUserPhonon.m_bRetriggerable)
			{
				if(m_xUserPhonon.m_bUseRetriggerRadius)
				{
					extern void Render_Sphere( const Asura_Vector_3& xCentre, float fRadius, int iColour );
					Render_Sphere(m_xUserPhonon.m_xInitialPosition,m_xUserPhonon.m_fRetriggerRadius,0x7f00ff00);
				}
				else if(m_xUserPhonon.m_bUseRetriggerBoundingBox && !Tools_VolumeEdit_Object::Find(m_uRetriggerVolumeEditID))
				{
					Tools_Render::RenderCube(m_xUserPhonon.m_xRetriggerBoundingBox,0x7f00ff00,false);
					Tools_Render::RenderCube(m_xUserPhonon.m_xRetriggerBoundingBox,0xff00ff00,true);
				}
			}
		}
	}
	
	Tools_Object::Render();
}


const char* Tools_Phonon_Object::GetName()
{
	return m_xName.GetString();
}

void Tools_Phonon_Object::UpdateName()
{
	if(m_xUserPhonon.m_xName.IsBlank())
	{
		Asura_Phonon* pxPhonon = GetPhonon();
	    Asura_Guid uPhononGuid = pxPhonon->GetGuid();
	    Asura_Assert(uPhononGuid != ASURA_GUID_UNREGISTERED, "phonon has no guid");

		//generate a default name  from the wav name
		char acName[100];

		const char* pszWavName = 0;
		if (m_xUserPhonon.m_pxEvent)
		{
			pszWavName = m_xUserPhonon.m_pxEvent->GetName();
		}

		if(!pszWavName) pszWavName = "Sound";

		int iID = uPhononGuid-ASURA_GUID_TOOLSCREATEDENTITY_START;
		Asura_PrintfToBfr(acName,"%s (%d)",100,true,pszWavName,iID);
		
		m_xName = acName;
	}
	else
	{
		m_xName = m_xUserPhonon.m_xName.GetString();
	}
}

void Tools_Phonon_Object::Destroy()
{
	SetModified();


    //If the phonon has a sound controller , destroy that as well
	Asura_Phonon* pxPhonon = GetPhonon();
    Asura_Guid uPhononGuid = pxPhonon->GetGuid();
    Asura_Assert(uPhononGuid != ASURA_GUID_UNREGISTERED, "phonon has no guid");
    if (uPhononGuid != ASURA_GUID_UNREGISTERED)
    {
        Asura_ServerEntity_SoundController* pxSoundControllerEntity = 
            Asura_ServerEntity_SoundController::GetControllerFromPhonon(uPhononGuid);
        if (pxSoundControllerEntity)
        {
			pxSoundControllerEntity->SetAsDestroyed();
			Tools_SaveAsura::SetEntityModified();

			//also delete the client entity
			Asura_Guid uGuid = pxSoundControllerEntity->GetGuid();
			Asura_Entity_Client* pxClientEntity = Asura_Entity_Client::FindEntity(uGuid);
			if (pxClientEntity)
			{
				pxClientEntity->SetAsDestroyed();
			}

        }
    }


	Asura_Phonon_System::DeallocatePhonon(pxPhonon);

	Tools_Object::Destroy();		


	//update the total sound memory used
	if(Tools_System::IsInSoundMode())
	{
		Tools_Mode_Sound::InvalidateSoundSize();
	}
}

void Tools_Phonon_Object::OnSelection()
{
	Tools_Object::OnSelection();

	if (Tools_Mode_Sound::IsInitialised())
    {
        s_pxSelectedObject = this;
	    
	    CreatePhononPropertiesOverlay();
    }
}


void Tools_Phonon_Object::OnDeselection()
{
    s_pxSelectedObject = 0;
	if (Tools_Mode_Sound::IsInitialised())
    {
	    Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_PHONONPROPERTIES);
	    Tools_Object::OnDeselection();

		StopTestSound();
    }
}

void Tools_Phonon_Object::OnReselection()
{
	if (Tools_Mode_Sound::IsInitialised())
    {
	    Asura_Assert(s_pxSelectedObject==this,"");
	    CreatePhononPropertiesOverlay();
    }
}

void Tools_Phonon_Object::CreatePhononPropertiesOverlay(void)
{
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_PHONONPROPERTIES)) return;
	Asura_Assert(s_pxSelectedObject,"");
	
	UpdateUserPhononName();
	
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Sound Properties",false,&Tools_Mode_Sound::PhononPropertiesOverlayPosition,&Tools_Mode_Sound::s_uPhononPropertiesOverlayMinimised);
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	
	Asura_Dialog* pxDialog;

	Asura_Dialog* pxSoundEventDialog = pxDialog = new Tools_PickSoundEvent_Dialog("Event : ",xOffset,&m_xUserPhonon.m_uEventHashID,&m_bChanged);
	pxDialog->SetHelpText("Select the sound event to use.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//add control for editing the selected sound event
	pxDialog = new Tools_EditSoundEvent_Dialog("Edit",xOffset,&m_xUserPhonon.m_uEventHashID);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxSoundEventDialog,Asura_Vector_2(3.0f,0.0f),false,true);
	

	pxDialog = new Asura_Function_Dialog("Rename",xOffset,_Rename);
	pxDialog->SetHelpText("Rename this phonon.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Play Sound",xOffset,_PlaySound);
	pxDialog->SetHelpText("Play a test sound using the current properties.");
	pxOverlay->AddDialog(pxDialog);


	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Function_Dialog("Stop",xOffset,StopTestSound);
	pxDialog->SetHelpText("Stops the test sound.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y+=Asura_Dialog::LargeSpacing();

	//Tab control
	static char* s_aszTabs[3] = {"Main","Light Sets","Platform"};
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset,s_aszTabs,3);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	pxOverlay->SetCurrentParentDialog(pxTabsDialog);	


	//Main Tab
	pxTabsDialog->SelectTab(0);
	xOffset.y = fTabsStartY;


	pxDialog = new Asura_SharedTickbox_Dialog("Cuboid",xOffset,&g_xParam_Phonon_Cuboid);
	pxDialog->SetHelpText("Sound uses cuboid extents (instead of spherical).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=12.0f;

	pxDialog = new Asura_SharedTickbox_Dialog("Use Orient",xOffset,&g_xParam_Phonon_UseOrient);
	pxDialog->SetHelpText("Allows the orientation of a cuboid sound to be set.\nOtherwise the cube will be aligned to the world axes.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x=1.0f;
	xOffset.y+=Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_SharedTickbox_Dialog("Starts active",xOffset,&g_xParam_Phonon_Active);
	pxDialog->SetHelpText("Sound is playing when the level starts.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x=1.0f;
	xOffset.y+=Asura_Dialog::LargeSpacing();


	const char* pszInnerText = "The inner range defines the point at which the sound will start to attenuate from full volume.";
	const char* pszOuterText = "The outer range defines the point at which the sound's volume reaches 0.";
	
	//Add controls for inner outer cube editing , or inner outer radius
	if(m_xUserPhonon.m_bUseCuboidFalloff)
	{
		float fCuboidY = xOffset.y;
		//heading for inner cuboid
		pxDialog = new Asura_Dialog("Inner Range",xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		
		//controls for inner cuboid
		pxDialog = new Asura_Numerical_Dialog("X",xOffset,&g_xParam_Phonon_CuboidInnerX, 0.5f,9999.0f,0.1f,"%0.1f");
		pxDialog->SetHelpText(pszInnerText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Y",xOffset,&g_xParam_Phonon_CuboidInnerY, 0.5f,9999.0f,0.1f,"%0.1f");
		pxDialog->SetHelpText(pszInnerText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Z",xOffset,&g_xParam_Phonon_CuboidInnerZ, 0.5f,9999.0f,0.1f,"%0.1f");
		pxDialog->SetHelpText(pszInnerText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::LargeSpacing();

		xOffset.y=fCuboidY;
		xOffset.x+=15.0f;
		//heading for outer cuboid
		pxDialog = new Asura_Dialog("Outer Range",xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		
		//controls for outer cuboid
		pxDialog = new Asura_Numerical_Dialog("X",xOffset,&g_xParam_Phonon_CuboidOuterX, 0.5f,9999.0f,0.1f,"%0.1f");
		pxDialog->SetHelpText(pszOuterText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Y",xOffset,&g_xParam_Phonon_CuboidOuterY, 0.5f,9999.0f,0.1f,"%0.1f");
		pxDialog->SetHelpText(pszOuterText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Z",xOffset,&g_xParam_Phonon_CuboidOuterZ, 0.5f,9999.0f,0.1f,"%0.1f");
		pxDialog->SetHelpText(pszOuterText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::LargeSpacing();

		xOffset.x=1.0f;

	}
	else
	{
		pxDialog = new Asura_Numerical_Dialog("Inner Range",xOffset,&g_xParam_Phonon_InnerRange, 0.5f,9999.0f,0.1f,"%0.1f");
		pxDialog->SetHelpText(pszInnerText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_Numerical_Dialog("Outer Range",xOffset,&g_xParam_Phonon_OuterRange, 0.5f,9999.0f,0.1f,"%0.1f");
		pxDialog->SetHelpText(pszOuterText);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::LargeSpacing();
	}


	pxDialog = new Asura_SharedTickbox_Dialog("Retriggerable",xOffset,&g_xParam_Phonon_Retriggerable);
	pxDialog->SetHelpText("Sound retriggers on re-entering phonon range (ignored by looping phonons, which always retrigger).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_SharedTickbox_Dialog("Use Retrigger Radius",xOffset,&g_xParam_Phonon_UseRetriggerRadius);
	pxDialog->SetHelpText("Causes the retrigger radius to be used instead of the outer radius for determining when to retrigger.");
	pxDialog->SetIsEnabledFunction(IsEnabled_UseRetriggerRadius);
	pxOverlay->AddDialog( pxDialog );
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Retrigger Radius",xOffset,&g_xParam_Phonon_RetriggerRadius, 0.0f,9999.0f,0.1f,"%0.1f");
	pxDialog->SetIsEnabledFunction(IsEnabled_RetriggerRadius);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Use Retrigger Box",xOffset,&g_xParam_Phonon_UseRetriggerBoundingBox);
	pxDialog->SetHelpText("Causes the retrigger box to be used instead of the outer radius for determining when to retrigger.");
	pxDialog->SetIsEnabledFunction(IsEnabled_UseRetriggerBoundingBox);
	pxOverlay->AddDialog( pxDialog );
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Retrigger Box",xOffset,_EditRetriggerBoundingBox);
	pxDialog->SetHelpText("If used then the sound will be triggered again by entering the box.");
	pxDialog->SetIsEnabledFunction(IsEnabled_RetriggerBoundingBox);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_SharedTickbox_Dialog("Use Clipping Volumes",xOffset,&g_xParam_Phonon_UseBoundingBox);
	pxDialog->SetHelpText("Sound should only be audible in the clipping volumes defined for it.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Invert Clipping Volume",xOffset,&g_xParam_Phonon_InvertBoundingBox);
	pxDialog->SetHelpText("If inverted  then the sound is only played when outside of the clipping volumes.\nOnly does anything if Use Clipping Volume is on.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Clipping Volume",xOffset,_EditBoundingBox);
	pxDialog->SetHelpText("Edit the optional clipping volumes for the sound.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("PS2 Sample Rate",xOffset,&m_fPS2Frequency, 0,44100.0f,1.0f,"%.0f",&m_bPS2FrequencyModified);
	pxDialog->SetHelpText("Sets the sample rate to use when creating the .VAG files.\nChanges are saved to Sounds/PS2SoundSettings.txt.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	//End of main tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//Light set tab
	pxTabsDialog->SelectTab(1);
	xOffset.y = fTabsStartY;

	//add a checkbox for each light set
	for(u_int i=0;i<TOOLS_MAX_LIGHT_SETS;i++)
	{
		const char* szSetName = Tools_LightSets::GetLightSetNameByIndex(i);
		if(!szSetName) continue;

		g_axParam_LightSource_LightSet[i].SetParamID(g_uHash_LightSource_LightSet0+i);

		pxDialog = new Asura_SharedTickbox_Dialog(szSetName,xOffset,&g_axParam_LightSource_LightSet[i]);
		pxDialog->SetHelpText("Causes sound to be exported along with this light set during level exports.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	//End of light set tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//Platform tab
	pxTabsDialog->SelectTab(2);
	xOffset.y = fTabsStartY;

	//add a checkbox for each platform
	for(u_int i=0;i<TOOLS_PLATFORM_NUM_PLATFORMS;i++)
	{
		if(!Tools_Level_Export::IsPlatformSupported((Tools_Level_Export::EXPORT_TYPE)i)) continue;
		
		const char* szPlatformName = Tools_Platform::GetPlatformName(i);

		g_axParam_Phonon_Platform[i].SetParamID(g_uHash_Phonon_Platform0+i);

		pxDialog = new Asura_SharedTickbox_Dialog(szPlatformName,xOffset,&g_axParam_Phonon_Platform[i]);
		pxDialog->SetHelpText("Flags the sound for export for this platform.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	//End of platform tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;


	//End of tabs
	pxOverlay->SetCurrentParentDialog(0);
	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);
	xOffset.y = fTabsEndY + 1.0f;


	pxOverlay->m_uID = TOOLS_OVERLAY_PHONONPROPERTIES;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
	pxOverlay->m_xOrigin = pxOverlay->m_xPosition;//so the overlay doesn't swoop down into position everytime it gets remade

	//set things up so that right clicking on the overlay will be equivalent to right clicking on the object
	//RAR 06/11/2007
	pxOverlay->SetContextMenuCallback(Tools_Object_System::AddContextMenuItemsFromOverlay, this);

	m_bMinimumModified = false;
	m_bMaximumModified = false;
}

void Tools_Phonon_Object::UpdateUserPhononName()
{
	m_fPS2Frequency = 0;
	m_bPS2FrequencyModified = false;
	
}

void Tools_Phonon_Object::UpdateWhileSelected()
{
	if (Tools_Mode_Sound::IsInitialised())
    {
	    if(m_bOverlayNeedsRecreating)
	    {
		    m_bOverlayNeedsRecreating = false;
		    //close and reopen the overlay
		    Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_PHONONPROPERTIES);
		    CreatePhononPropertiesOverlay();
	    }	
        
    }
}

void Tools_Phonon_Object::Update()
{
	Tools_Object::Update();
	UpdateIfChanged();
}


Asura_Guid Tools_Phonon_Object::GetGuid() const
{
    // see if there is a sound entity controlling our sound
	Asura_Phonon* pxPhonon = GetPhonon();

    Asura_Guid uPhononGuid = pxPhonon->GetGuid();
    Asura_Assert(uPhononGuid != ASURA_GUID_UNREGISTERED, "phonon has no guid");
    if (uPhononGuid != ASURA_GUID_UNREGISTERED)
    {
        const Asura_ServerEntity_SoundController* pxSoundControllerEntity = 
            Asura_ServerEntity_SoundController::GetControllerFromPhonon(uPhononGuid);
        if (pxSoundControllerEntity)
        {
            return pxSoundControllerEntity->GetGuid();
        }
    }

    return ASURA_GUID_UNREGISTERED;
}

bool Tools_Phonon_Object::AcquireGuid()
{
	Asura_Phonon* pxPhonon = GetPhonon();

	//Only assign a guid if the sound file is writeable 
	if(!CanSaveFile() && !Tools_SaveAsura::IsTemplateEditing())
	{
		ShowMessageOverlay("Can't assign guid to sound object because the sound file isn't checked out.");
		return false;
	}

    Asura_Assert(GetGuid() == ASURA_GUID_UNREGISTERED, "guid leak ahoy");
    Asura_Assert(pxPhonon->GetGuid() != ASURA_GUID_UNREGISTERED, "phonon has no guid");
	
	Tools_GuidRange eRange;
	if(Tools_SaveAsura::IsTemplateEditing())
	{
		//Using the main entity range for template files , to simplify the saving code.
		//Everything is in the same file in that case anyway , so the different ranges aren't needed
		eRange = TOOLS_GUIDRANGE_ENTITY;
	}
	else if(BelongsToMissionFile())
	{
		eRange = TOOLS_GUIDRANGE_MISSIONSOUND;
	}
	else
	{
		eRange = TOOLS_GUIDRANGE_SOUND;
	}
	Asura_Guid uEntityGuid = Asura_Entity_System::GetCreatedByToolsGuid(eRange);

    // create a sound controller entity.
    Asura_ServerEntity_SoundController* pxServerController = new Asura_ServerEntity_SoundController(uEntityGuid, pxPhonon->GetGuid(), pxPhonon->m_bStartsActive);
	new Asura_ClientEntity_SoundController(pxServerController->GetGuid(),pxPhonon->GetGuid(), pxPhonon->m_bStartsActive);
	Tools_SaveAsura::SetEntityModified();

	pxServerController->ToolsData = new Tools_Entity_Data;
	pxServerController->ToolsData->Name = "Phonon";

	SetModified();

	return true;
}


// (MH - 19.11.2002) the phonon handles these messages via Asura_ServerEntity_SoundController
bool Tools_Phonon_Object::HandlesThisMessage(int messageID)
{
	if(messageID == ASURA_MSG_ACTIVATE ||
	   messageID == ASURA_MSG_DEACTIVATE ||
	   messageID == ASURA_MSG_TRIGGER)
	{
		return true;
	}
	return false;
}

void Tools_Phonon_Object::RemoveDudSoundControllers()
{
	//scan for any sound controller entities that no longer have a phonon
	for(Asura_Classified_List_It<Asura_Entity_Server> it(Asura_Entity_Server::GetList());!it.Done();it.Next())
	{
		Asura_Entity_Server* pxEntity = it.GetCurrent();
		//ignore entities marked for destruction
		if(pxEntity->IsDestroyed()) continue;
		//and those with non-tools guids
		if(pxEntity->GetGuid()<ASURA_GUID_TOOLSCREATEDENTITY_START || pxEntity->GetGuid()>ASURA_GUID_TOOLSCREATEDENTITY_END) continue;
		//just interested in sound controllers
		if(pxEntity->GetClassification() != AsuraEntityClass_SoundController) continue;

		//check the guid for this controller to see if the phonon exists
		Asura_ServerEntity_SoundController* pxSoundController = (Asura_ServerEntity_SoundController*) pxEntity;
		Asura_Guid	uPhononGuid = pxSoundController->GetPhononGuid();
		Asura_Phonon *pxPhonon = Asura_Phonon_System::GetPhonon(uPhononGuid);
		if(pxPhonon)
		{
			// (AW - 29.03.2005) - Not a dud. While we're here though, we are going to check
			// that the active status is in sync.
			if (pxSoundController->IsActive() != pxPhonon->m_bStartsActive)
			{
				pxSoundController->SetStartsActive(pxPhonon->m_bStartsActive);
				Tools_SaveAsura::SetEntityModified();
			}
			continue;	
		}

		//need to delete this one then
		pxSoundController->SetAsDestroyed();

		//also delete the client entity
		Asura_Guid uGuid = pxSoundController->GetGuid();
		Asura_Entity_Client* pxClientEntity = Asura_Entity_Client::FindEntity(uGuid);
		if (pxClientEntity)
		{
			pxClientEntity->SetAsDestroyed();
		}
		Tools_SaveAsura::SetEntityModified();
	}
}

Asura_Phonon* Tools_Phonon_Object::GetPhonon() const
{
	return(Asura_Phonon_System::GetPhonon(m_uSoundGuid));
}



void Tools_Phonon_Object::_PlaySound()
{
	if(s_pxSelectedObject)s_pxSelectedObject->PlaySound();
}

void Tools_Phonon_Object::PlaySound()
{
	StopTestSound();
	if(!s_xTestPhonon.GetClipVolume()) s_xTestPhonon.Initialise();
	s_xTestPhonon = m_xUserPhonon;
	s_iTestPhononHandle = Asura_Sound_Event_System::CreatePhononInstance(&s_xTestPhonon);
	Asura_Sound_Event_System::TriggerPhononInstanceEventPlay(s_iTestPhononHandle,&s_xTestPhonon);
}


void Tools_Phonon_Object::StopTestSound()
{
	if(s_iTestPhononHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Sound_Event_System::Stop(s_iTestPhononHandle);
	}
}


void Tools_Phonon_Object::SelectSound()
{
	Asura_Assert(s_pxSelectedObject,"");
	s_pxSelectedObject->PickSound();
}

bool Tools_Phonon_Object::PickSound()
{
	//For sound events this gets used when creating the phonon
	Tools_PickSoundEvent_Overlay xOverlay("Pick event for new phonon");
	m_xUserPhonon.m_uEventHashID = xOverlay.DoModal(0);
	m_bChanged = true;

	return m_xUserPhonon.m_uEventHashID!=0;
}


void Tools_Phonon_Object::UpdateIfChanged()
{
	
	if(m_bMinimumModified)
	{
		//One of the mins was changed. Make sure all the maxs are higher
		if(m_xUserPhonon.m_fOuterRadius<m_xUserPhonon.m_fInnerRadius)m_xUserPhonon.m_fOuterRadius=m_xUserPhonon.m_fInnerRadius;

		if(m_xUserPhonon.m_xOuterCuboidRadius.x<m_xUserPhonon.m_xInnerCuboidRadius.x)m_xUserPhonon.m_xOuterCuboidRadius.x=m_xUserPhonon.m_xInnerCuboidRadius.x;
		if(m_xUserPhonon.m_xOuterCuboidRadius.y<m_xUserPhonon.m_xInnerCuboidRadius.y)m_xUserPhonon.m_xOuterCuboidRadius.y=m_xUserPhonon.m_xInnerCuboidRadius.y;
		if(m_xUserPhonon.m_xOuterCuboidRadius.z<m_xUserPhonon.m_xInnerCuboidRadius.z)m_xUserPhonon.m_xOuterCuboidRadius.z=m_xUserPhonon.m_xInnerCuboidRadius.z;

		m_bMinimumModified = false;
		m_bChanged = true;
	}
	if(m_bMaximumModified)
	{
		//One of the maxs was changed. Make sure all the mins are lower
		if(m_xUserPhonon.m_fOuterRadius<m_xUserPhonon.m_fInnerRadius)m_xUserPhonon.m_fInnerRadius=m_xUserPhonon.m_fOuterRadius;

		if(m_xUserPhonon.m_xOuterCuboidRadius.x<m_xUserPhonon.m_xInnerCuboidRadius.x)m_xUserPhonon.m_xInnerCuboidRadius.x=m_xUserPhonon.m_xOuterCuboidRadius.x;
		if(m_xUserPhonon.m_xOuterCuboidRadius.y<m_xUserPhonon.m_xInnerCuboidRadius.y)m_xUserPhonon.m_xInnerCuboidRadius.y=m_xUserPhonon.m_xOuterCuboidRadius.y;
		if(m_xUserPhonon.m_xOuterCuboidRadius.z<m_xUserPhonon.m_xInnerCuboidRadius.z)m_xUserPhonon.m_xInnerCuboidRadius.z=m_xUserPhonon.m_xOuterCuboidRadius.z;

		m_bMaximumModified = false;
		m_bChanged = true;
	}

	
	
	if (!(m_xUserPhonon==*GetPhonon()))
	{
		m_bChanged = true;
	}
		
	

	if(m_bChanged)
	{
		//Force update of position/orientation in user phonon from the tools object , just in case these
		//are getting out of sync. RAR 19/02/2007
		m_xUserPhonon.m_xPosition = Position;
		m_xUserPhonon.m_xInitialPosition = Position;
		m_xUserPhonon.m_xOrient.FromMatrix(Orientation);
		
		
		const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent(m_xUserPhonon.m_uEventHashID);
		if(pxEvent != m_xUserPhonon.m_pxEvent)
		{
			m_xUserPhonon.m_pxEvent = pxEvent;
			Tools_Mode_Sound::InvalidateSoundSize();			
		}

		GetPhonon()->Set(m_xUserPhonon);
		Asura_Assert(m_xUserPhonon==*GetPhonon(), "Set() failed to make phonons equal");
		SetModified();

		m_bChanged = false;
	}	
	
	//make sure the light sets in the entity match those in the sound. RAR 14/06/2006
	Asura_Phonon* pxPhonon = GetPhonon();
	Asura_ServerEntity_SoundController* pxEntity = Asura_ServerEntity_SoundController::GetControllerFromPhonon(pxPhonon->GetGuid());
	if(pxEntity && pxEntity->ToolsData)
	{
		pxEntity->ToolsData->m_uLightSets = m_xUserPhonon.m_uLightSets;
		pxEntity->ToolsData->m_uPlatforms = m_xUserPhonon.m_uPlatforms;
	}


	if (m_bStartsActiveChanged)
    {
		// see if there is an entity which needs updating.
        if (pxEntity)
        {
			pxEntity->SetStartsActive(pxPhonon->m_bStartsActive);
        }
        m_bStartsActiveChanged = false;

		if(pxPhonon->m_bStartsActive)
		{
			pxPhonon->Activate();
		}
		else
		{
			pxPhonon->Deactivate();
		}
	}

}


void Tools_Phonon_Object::_EditBoundingBox()
{
	if(s_pxSelectedObject)
	{
		s_pxSelectedObject->EditBoundingBox();
	}
}

void Tools_Phonon_Object::EditBoundingBox()
{
	if(!m_pxClipVolumeController)
	{
		m_pxClipVolumeController = new Tools_Object_VolumeGroupEdit_Controller(this, m_xUserPhonon.m_pxClipVolume, &m_bChanged);

		if(!m_xUserPhonon.m_bUseClipVolume)
		{
			m_xUserPhonon.m_bUseClipVolume = true;
			m_bChanged = true;
		}
	}
	m_pxClipVolumeController->StartVolumeEditing();
}

void Tools_Phonon_Object::_EditRetriggerBoundingBox()
{
	if(s_pxSelectedObject)
	{
		s_pxSelectedObject->EditRetriggerBoundingBox();
	}
}

void Tools_Phonon_Object::EditRetriggerBoundingBox()
{
		
	//see if the volume edit object already exists
	Tools_VolumeEdit_Object* pxVolumeObject = Tools_VolumeEdit_Object::Find(m_uRetriggerVolumeEditID);

	if(!pxVolumeObject)
	{
		Asura_Bounding_Box* pxBB = &m_xUserPhonon.m_xRetriggerBoundingBox;

		//set a default value for the bounding box if needed
		if(pxBB->GetVolume()==0)
		{
			pxBB->Reset();
			pxBB->Set(Position);
			pxBB->Grow(0.5f);
		}

		char acName[200];
		Asura_PrintfToBfr(acName,"%s Retrigger Box",200,true,GetName());

		pxVolumeObject = new Tools_VolumeEdit_Object(acName,pxBB,0,&m_bChanged);
		pxVolumeObject->SetOwner(this);
		m_uRetriggerVolumeEditID = pxVolumeObject->GetID();

		if(!m_xUserPhonon.m_bUseRetriggerBoundingBox)
		{
			m_xUserPhonon.m_bUseRetriggerBoundingBox = true;
			m_xUserPhonon.m_bUseRetriggerRadius = true;
			m_bChanged = true;
		}
	}

	Tools_Object_System::SelectObject(pxVolumeObject);
}

void Tools_Phonon_Object::MoveBoundingBoxes(const Asura_Vector_3& xOffset)
{

	if(!m_bDontDisplaceBoundingBox)
	{
		m_xUserPhonon.m_pxClipVolume->DisplaceVolumes(xOffset);

		if(m_pxClipVolumeController)
		{
			m_pxClipVolumeController->MoveVolumeObjects(xOffset);			
		}
	}

	if(!m_bDontDisplaceRetriggerBoundingBox)
	{
		m_xUserPhonon.m_xRetriggerBoundingBox.Displace(xOffset);

		Tools_VolumeEdit_Object* pxVolumeObject = Tools_VolumeEdit_Object::Find(m_uRetriggerVolumeEditID);
		if(pxVolumeObject) pxVolumeObject->Position += xOffset;
	}

	m_bChanged = true;
}

bool Tools_Phonon_Object::IsSelectable()
{
	return GetPhonon()->IsMemberOfTemplateInstance() ? false : Tools_Object::IsSelectable();
}

bool Tools_Phonon_Object::IsHidden()
{
	//Sound may be hidden if not a member of any visible light set
	if(Tools_Object_System::IsHideByLightSetEnabled())
	{
		if(!(m_xUserPhonon.m_uLightSets & Tools_Object_System::GetVisibleLightSets())) return true;
	}

	if(Tools_LightSets::ShouldHideObjectsInOtherSets())
	{
		if(!Tools_LightSets::IsMemberOfCurrentLightSets(m_xUserPhonon.m_uLightSets)) return true;
	}


	return Tools_Object::IsHidden();
}


Tools_Phonon_Object* Tools_Phonon_Object::GetPhononObjectFromSoundGuid(Asura_Guid uGuid)
{
	if(uGuid == ASURA_GUID_UNREGISTERED) return 0;

	for(LIF<Tools_Phonon_Object*> xPhononLIF(&Tools_Mode_Sound::PhononList);!xPhononLIF.done();xPhononLIF.next())
	{
		if(xPhononLIF()->GetPhonon()->GetGuid() == uGuid) return xPhononLIF();
	}
	return 0;
}


void Tools_Phonon_Object::_Rename()
{
	if(s_pxSelectedObject)
	{
		s_pxSelectedObject->Rename();
	}
}

void Tools_Phonon_Object::Rename()
{
	Asura_String xName = GetName();

	if(Tools_Object_System::AskForName(xName,"Rename phonon",this,true))
	{
		if(strcmp(xName.GetString(),GetName()))
		{
			m_xUserPhonon.m_xName = xName;

			UpdateName();
			m_bChanged = true;
		}
	}
}

bool Tools_Phonon_Object::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{
	if(uParamID>=g_uHash_LightSource_LightSet0 && uParamID<=g_uHash_LightSource_LightSetLast)
	{
		u_int uBit = 1 << (uParamID - g_uHash_LightSource_LightSet0);
		pxParam->SetBool((m_xUserPhonon.m_uLightSets & uBit) ? true : false);
	}
	else if(uParamID>=g_uHash_Phonon_Platform0 && uParamID<=g_uHash_Phonon_PlatformLast)
	{
		u_int uBit = 1 << (uParamID - g_uHash_Phonon_Platform0);
		pxParam->SetBool((m_xUserPhonon.m_uPlatforms & uBit) ? true : false);
	}
	else
	{
		switch(uParamID)
		{
			case g_uHash_Phonon_Cuboid :
				pxParam->SetBool(m_xUserPhonon.m_bUseCuboidFalloff);
				break;

			case g_uHash_Phonon_UseOrient :
				pxParam->SetBool(m_xUserPhonon.m_bUseOrient);
				break;


			case g_uHash_Entity_StartsActive :
				pxParam->SetBool(m_xUserPhonon.m_bStartsActive);
				break;

			case g_uHash_Phonon_CuboidInnerX :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				pxParam->SetFloat(m_xUserPhonon.m_xInnerCuboidRadius.x);
				break;

			case g_uHash_Phonon_CuboidInnerY :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				pxParam->SetFloat(m_xUserPhonon.m_xInnerCuboidRadius.y);
				break;

			case g_uHash_Phonon_CuboidInnerZ :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				pxParam->SetFloat(m_xUserPhonon.m_xInnerCuboidRadius.z);
				break;

			case g_uHash_Phonon_CuboidOuterX :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				pxParam->SetFloat(m_xUserPhonon.m_xOuterCuboidRadius.x);
				break;

			case g_uHash_Phonon_CuboidOuterY :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				pxParam->SetFloat(m_xUserPhonon.m_xOuterCuboidRadius.y);
				break;

			case g_uHash_Phonon_CuboidOuterZ :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				pxParam->SetFloat(m_xUserPhonon.m_xOuterCuboidRadius.z);
				break;

			case g_uHash_Phonon_InnerRange :
				if(m_xUserPhonon.m_bUseCuboidFalloff) return false;
				pxParam->SetFloat(m_xUserPhonon.m_fInnerRadius);
				break;

			case g_uHash_Phonon_OuterRange :
				if(m_xUserPhonon.m_bUseCuboidFalloff) return false;
				pxParam->SetFloat(m_xUserPhonon.m_fOuterRadius);
				break;


			case g_uHash_Phonon_UseBoundingBox :
				pxParam->SetBool(m_xUserPhonon.m_bUseClipVolume);
				break;

			case g_uHash_Phonon_InvertBoundingBox :
				pxParam->SetBool(m_xUserPhonon.m_bInvertClipVolume);
				break;

			case g_uHash_Phonon_Retriggerable :
				pxParam->SetBool(m_xUserPhonon.m_bRetriggerable);
				break;

			case g_uHash_Phonon_UseRetriggerRadius :
				pxParam->SetBool(m_xUserPhonon.m_bUseRetriggerRadius);
				break;

			case g_uHash_Phonon_RetriggerRadius :
				pxParam->SetFloat(m_xUserPhonon.m_fRetriggerRadius);
				break;

			case g_uHash_Phonon_UseRetriggerBoundingBox :
				pxParam->SetBool(m_xUserPhonon.m_bUseRetriggerBoundingBox);
				break;

			default :
				return false;
		}
	}
	return true;
}

bool Tools_Phonon_Object::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	if(uParamID>=g_uHash_LightSource_LightSet0 && uParamID<=g_uHash_LightSource_LightSetLast)
	{
		u_int uBit = 1 << (uParamID - g_uHash_LightSource_LightSet0);
		bool bSet = pxParam->GetBool();
		if(bSet)
			m_xUserPhonon.m_uLightSets |= uBit;
		else
			m_xUserPhonon.m_uLightSets &=~ uBit;

	}
	else if(uParamID>=g_uHash_Phonon_Platform0 && uParamID<=g_uHash_Phonon_PlatformLast)
	{
		u_int uBit = 1 << (uParamID - g_uHash_Phonon_Platform0);
		bool bSet = pxParam->GetBool();
		if(bSet)
			m_xUserPhonon.m_uPlatforms |= uBit;
		else
			m_xUserPhonon.m_uPlatforms &=~ uBit;

	}
	else
	{
		switch(uParamID)
		{
			case g_uHash_Phonon_Cuboid :
				m_xUserPhonon.m_bUseCuboidFalloff = pxParam->GetBool();
				m_bOverlayNeedsRecreating = true;
				break;

			case g_uHash_Phonon_UseOrient :
				m_xUserPhonon.m_bUseOrient = pxParam->GetBool();
				break;


			case g_uHash_Entity_StartsActive :
				m_xUserPhonon.m_bStartsActive = pxParam->GetBool();
				m_bStartsActiveChanged = true;
				break;

			case g_uHash_Phonon_CuboidInnerX :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				m_xUserPhonon.m_xInnerCuboidRadius.x = pxParam->GetFloat();
				m_bMinimumModified = true;
				break;

			case g_uHash_Phonon_CuboidInnerY :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				m_xUserPhonon.m_xInnerCuboidRadius.y = pxParam->GetFloat();
				m_bMinimumModified = true;
				break;

			case g_uHash_Phonon_CuboidInnerZ :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				m_xUserPhonon.m_xInnerCuboidRadius.z = pxParam->GetFloat();
				m_bMinimumModified = true;
				break;

			case g_uHash_Phonon_CuboidOuterX :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				m_xUserPhonon.m_xOuterCuboidRadius.x = pxParam->GetFloat();
				m_bMaximumModified = true;
				break;

			case g_uHash_Phonon_CuboidOuterY :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				m_xUserPhonon.m_xOuterCuboidRadius.y = pxParam->GetFloat();
				m_bMaximumModified = true;
				break;

			case g_uHash_Phonon_CuboidOuterZ :
				if(!m_xUserPhonon.m_bUseCuboidFalloff) return false;
				m_xUserPhonon.m_xOuterCuboidRadius.z = pxParam->GetFloat();
				m_bMaximumModified = true;
				break;

			case g_uHash_Phonon_InnerRange :
				if(m_xUserPhonon.m_bUseCuboidFalloff) return false;
				m_xUserPhonon.m_fInnerRadius = pxParam->GetFloat();
				m_bMinimumModified = true;
				break;

			case g_uHash_Phonon_OuterRange :
				if(m_xUserPhonon.m_bUseCuboidFalloff) return false;
				m_xUserPhonon.m_fOuterRadius = pxParam->GetFloat();
				m_bMaximumModified = true;
				break;


			case g_uHash_Phonon_UseBoundingBox :
				m_xUserPhonon.m_bUseClipVolume = pxParam->GetBool();
				break;

			case g_uHash_Phonon_InvertBoundingBox :
				m_xUserPhonon.m_bInvertClipVolume = pxParam->GetBool();
				break;


			case g_uHash_Phonon_Retriggerable :
				m_xUserPhonon.m_bRetriggerable = pxParam->GetBool();
				break;
				
			case g_uHash_Phonon_UseRetriggerRadius :
				m_xUserPhonon.m_bUseRetriggerRadius = pxParam->GetBool();
				if(m_xUserPhonon.m_bUseRetriggerRadius)
				{
					m_xUserPhonon.m_bUseRetriggerBoundingBox = false;
				}
				break;

			case g_uHash_Phonon_RetriggerRadius :
				m_xUserPhonon.m_fRetriggerRadius = pxParam->GetFloat();
				break;

			case g_uHash_Phonon_UseRetriggerBoundingBox :
				m_xUserPhonon.m_bUseRetriggerBoundingBox = pxParam->GetBool();
				if(m_xUserPhonon.m_bUseRetriggerBoundingBox)
				{
					m_xUserPhonon.m_bUseRetriggerRadius = false;
				}

				break;

			default :
				return false;
		}
	}
	m_bChanged = true;
	UpdateIfChanged();
	return true;
}


void Tools_Phonon_Object::SoundEventDeleted(const Asura_Sound_Event* pxEvent)
{
	//search for any tools objects using the event being deleted
	for(LIF<Tools_Phonon_Object*> xObjectLIF(&Tools_Mode_Sound::PhononList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->m_xUserPhonon.m_pxEvent == pxEvent)
		{
			xObjectLIF()->m_xUserPhonon.m_pxEvent = 0;
			xObjectLIF()->m_xUserPhonon.m_uEventHashID = 0;
		
			Tools_Mode_Sound::InvalidateSoundSize();			
			xObjectLIF()->SetModified();
		}
	}


	StopTestSound();
}

void Tools_Phonon_Object::SoundEventRenamed(Asura_Hash_ID uOldHash,Asura_Hash_ID uNewHash)
{
	//search for any tools objects using the event
	for(LIF<Tools_Phonon_Object*> xObjectLIF(&Tools_Mode_Sound::PhononList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->m_xUserPhonon.m_uEventHashID == uOldHash)
		{
			xObjectLIF()->m_xUserPhonon.m_uEventHashID = uNewHash;
			xObjectLIF()->UpdateName();
		
			xObjectLIF()->SetModified();
		}
	}

}

void Tools_Phonon_Object::HandleSoundEventReload()
{
	//get each phonon to find its sound event again
	for(LIF<Tools_Phonon_Object*> xObjectLIF(&Tools_Mode_Sound::PhononList);!xObjectLIF.done();xObjectLIF.next())
	{
		xObjectLIF()->m_xUserPhonon.m_pxEvent = Asura_Sound_Event_Storage::GetEventByHash(xObjectLIF()->m_xUserPhonon.m_uEventHashID);
		Tools_Mode_Sound::InvalidateSoundSize();			
	}

	//and the same for the asura phonons
	for(int i=0;i<Asura_Phonon_System::GetNumberOfPhonons();i++)
	{
		Asura_Phonon* pxPhonon = Asura_Phonon_System::GetPhononByIndex(i);
		pxPhonon->m_pxEvent = Asura_Sound_Event_Storage::GetEventByHash(pxPhonon->m_uEventHashID);
	}

}


bool Tools_Phonon_Object::IsEnabled_UseRetriggerRadius()
{
	if(!s_pxSelectedObject) return false;
	return s_pxSelectedObject->m_xUserPhonon.m_bRetriggerable;
}
bool Tools_Phonon_Object::IsEnabled_RetriggerRadius()
{
	if(!s_pxSelectedObject) return false;
	return s_pxSelectedObject->m_xUserPhonon.m_bRetriggerable && s_pxSelectedObject->m_xUserPhonon.m_bUseRetriggerRadius;
}
bool Tools_Phonon_Object::IsEnabled_UseRetriggerBoundingBox()
{
	if(!s_pxSelectedObject) return false;
	return s_pxSelectedObject->m_xUserPhonon.m_bRetriggerable;
}
bool Tools_Phonon_Object::IsEnabled_RetriggerBoundingBox()
{
	if(!s_pxSelectedObject) return false;
	return s_pxSelectedObject->m_xUserPhonon.m_bRetriggerable && s_pxSelectedObject->m_xUserPhonon.m_bUseRetriggerBoundingBox;
}




Tools_Object* Tools_Phonon_Object::CreateObjectOfThisType() const
{
	Asura_Phonon* pxPhonon = Tools_Mode_Sound::CreateDefaultPhonon();
	return pxPhonon ? new Tools_Phonon_Object(pxPhonon) : 0;

}

void Tools_Phonon_Object::Copy(const Tools_Object* pxObject)
{
	Asura_Assert(pxObject->Type == TOT_Phonon,"");

	const Tools_Phonon_Object* pxFromPhonon = (const Tools_Phonon_Object*) pxObject;

	delete m_pxClipVolumeController;
	m_pxClipVolumeController = 0;

	// store user phonon position , and name
    Asura_Vector_3 position = m_xUserPhonon.m_xInitialPosition;
	Asura_String xName = m_xUserPhonon.m_xName;
	m_xUserPhonon.m_iSoundEventHandle = ASURA_INVALID_HANDLE;
	m_xUserPhonon.Set(pxFromPhonon->m_xUserPhonon);

	// overwrite pasted postion and name with stored position
    m_xUserPhonon.m_xPosition = position;
    m_xUserPhonon.m_xInitialPosition = position;
	m_xUserPhonon.m_xName = xName;


	//Shift the bounding boxes to be take into account the different position from the object being copied
	MoveBoundingBoxes(Position-pxObject->Position);

	m_bChanged = true;

	m_bStartsActiveChanged = true;
			
	//Tools_SaveAsura::SetSoundsModified();

	UpdateUserPhononName();

	//If this was a newly created object  ,then attempt to generate a name from the object being copied. RAR 16/06/2006
	if(Tools_Object_System::GetCopyType() == TOOLS_COPYOBJECT_PASTENEW || Tools_Object_System::GetCopyType() == TOOLS_COPYOBJECT_DUPLICATE)
	{
		GenerateCopiedName(pxFromPhonon);
	}

	UpdateName();

}


void Tools_Phonon_Object::GenerateCopiedName(const Tools_Phonon_Object* pxFromObject)
{
	const Asura_String& xFromName = pxFromObject->m_xUserPhonon.m_xName;
	if(xFromName.IsBlank()) return;


	//If there is a numerical postfix on the name , cut it off , so it can be incremented when making the new name
	char szPrefix[200];
	Asura_PrintfToBfr(szPrefix,"%s",sizeof(szPrefix),true,xFromName.GetString());

	char* pcEnd = &szPrefix[strlen(szPrefix)-1];

	while(pcEnd>&szPrefix[0])
	{
		if(*pcEnd == ' ' || (*pcEnd >= '0' && *pcEnd <= '9'))
		{
			pcEnd--;
		}
		else
		{
			break;
		}
	}
	pcEnd++;

	//Get the number (if any) from the end
	int iPostfixNumber = 0;

	sscanf(pcEnd,"%d",&iPostfixNumber);
	*pcEnd = 0;

	char szNewName[300];
	do
	{
		Asura_PrintfToBfr(szNewName,"%s %d",sizeof(szNewName),true,szPrefix,++iPostfixNumber);
	}
	while(Tools_Object_System::IsNameInUse(szNewName));

	m_xUserPhonon.m_xName = szNewName;

}

bool Tools_Phonon_Object::BelongsToMissionFile() const
{
	return m_uSoundGuid >= ASURA_PHONONGUID_MISSIONFILE_START;
}

bool Tools_Phonon_Object::CanSaveFile() const
{
	if(BelongsToMissionFile())
	{
		return Tools_SaveAsura::CanSaveFile(Tools_SaveAsura::GetMissionSoundsFilename(), false);
	}
	else
	{
		return Tools_SaveAsura::CanSaveFile(Tools_SaveAsura::GetSoundsFilename(), false);
	}
}


void Tools_Phonon_Object::SetModified()
{
	if(BelongsToMissionFile())
		Tools_SaveAsura::SetMissionSoundsModified();
	else
		Tools_SaveAsura::SetSoundsModified();
}

void Tools_Phonon_Object::PrintInfo(float& x, float& y)
{
	Tools_Object::PrintInfo(x,y);

	if(Tools_SaveAsura::IsEditingMission())
	{
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Mission : %s",(m_uSoundGuid >= ASURA_PHONONGUID_MISSIONFILE_START) ? "Yes" : "No");
	}

}

//////////////////////////////////////////////////////////////////////////////////////

Tools_MessageLogAction_Phonon::Tools_MessageLogAction_Phonon(const Asura_Phonon* pxPhonon)
{
	m_uGuid = pxPhonon ? pxPhonon->GetGuid() : ASURA_GUID_UNREGISTERED;
}

void Tools_MessageLogAction_Phonon::Execute()
{
	Tools_Phonon_Object* pxPhononObject = Tools_Phonon_Object::GetPhononObjectFromSoundGuid(m_uGuid);
	if(pxPhononObject)
	{
		Tools_Object_System::GotoAndSelect(pxPhononObject);
	}	
	else
	{
		Tools_Timed_Message_System::AddMessage(0xffff00,"Failed to find the tools object for that phonon.");
	}
}