//
// Tools_PickSound.cpp
//
// Created by Richard Rice - 27/02/2021
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Sound/Tools_PickSound.h"

#include "Core/Asura_UserInput.h"
#include "Core/Asura_Windows.h"
#include "Debugging/Asura_Assert.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Interface.h"
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Phonon.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Text_Output.h"

#include "Misc/Tools_Timed_Messages.h"
#include "PC/FileSystem/Tools_FileSystem.h"


static Asura_String LastFileLoaded;

#ifdef ASURA_USE_AUDIO_SYSTEM
	Asura_Hash_ID Tools_PickSound::PickSoundFromDisk(Asura_Hash_ID uCurrentID, List<Asura_Hash_ID> *pxOutputSelectionList /*= 0*/, const char *szMessage /*= 0*/)
#else
	Asura_Sound_ID Tools_PickSound::PickSoundFromDisk(Asura_Sound_ID eCurrentID,Asura_Sound_ID eMinID,Asura_Sound_ID eMaxID,List<Asura_Sound_ID>* pxOutputSelectionList /*= 0*/, const char* szMessage /*=0*/)
#endif

{

#ifdef ASURA_USE_AUDIO_SYSTEM
	Asura_Hash_ID eReturnID = uCurrentID;
#else
	Asura_Sound_ID eReturnID = eCurrentID;
#endif

	Tools_FileSystem_DialogInfo info;

	info.DefaultExtension = "";
	info.DefaultFilename = LastFileLoaded.GetString();
	if (!info.DefaultFilename) info.DefaultFilename = "";

	Asura_String path = "sounds";
	if(LastFileLoaded.GetLength())
	{
		LastFileLoaded.SplitFilename(&path,0,0);
	}

	

	info.DefaultPath = path.GetString();//"Sounds\\";
	info.FileFilter = "Sound Files\0*.wav\0";
	info.WindowTitle = szMessage ? szMessage : "Load Sound...";
	info.AllowMultipleSelect = true;


	Asura_String name = Tools_FileSystem::GetLoadFilename(info);
	if(name.IsNull()) return eReturnID;

	// JGP - Need to set this...
	LastFileLoaded = name;

	while(!name.IsNull())
	{
		char* relative_name;
		
		_strlwr(name.GetString());
		char *start = strstr(name.GetString(),"sounds");

		if (!start)
		{
			Asura_Assert(0,"File not in 'sounds' folder");
			relative_name = name.GetString();
		}
		else
		{
			relative_name = start;
		}

#ifdef ASURA_USE_AUDIO_SYSTEM
		Asura_Audio_System::LoadWave(relative_name,false);
		eReturnID = Asura_GetHashID(relative_name);
		if(pxOutputSelectionList) pxOutputSelectionList->add_entry(eReturnID);
#else
		//assign SID for this sound
		Asura_Sound_ID eSoundID;
		if(eMinID == ASURA_SID_FIRST_DYNAMIC && eMaxID == ASURA_SID_LAST_DYNAMIC)
		{
			eSoundID = GetSoundIDForSoundEventFilename(relative_name);
		}
		else
		{
			eSoundID = GetSoundIDForFilename(relative_name,eMinID,eMaxID);
		}

		if(eSoundID == ASURA_SID_NO_SOUND)
		{
			Asura_Assert(0, "failed to load picked sound");
		}
		else
		{
			eReturnID = eSoundID;
			if(pxOutputSelectionList) pxOutputSelectionList->add_entry(eSoundID);
		}
#endif

		//get the next selected file
		name = Tools_FileSystem::GetNextLoadFilename();

	}
	return eReturnID;
}

#ifdef ASURA_USE_AUDIO_SYSTEM

Asura_Hash_ID Tools_PickSound::PickSoundForSoundEventSample(Asura_Hash_ID uCurrentID,List<Asura_Hash_ID>* pxOutputSelectionList /*= 0*/, const char* szMessage /*= 0*/)
{
	return PickSoundFromDisk(uCurrentID,pxOutputSelectionList, szMessage);
}

#else

Asura_Sound_ID Tools_PickSound::GetSoundIDForFilename(const char* pszRelativeFilename,Asura_Sound_ID eMinID,Asura_Sound_ID eMaxID)
{
	//Is sound already loaded in that range?
	Asura_Sound_ID eSoundID = Asura_Sound_System::GetSoundIDFromFileName(pszRelativeFilename,eMinID,eMaxID);
	if(eSoundID == ASURA_SID_NO_SOUND)
	{
	    bool ok = false;
		for (int eSoundID = eMinID; eSoundID <= eMaxID; eSoundID++)
		{
		    if (!Asura_Sound_System::SoundIsLoaded(static_cast<Asura_Sound_ID>(eSoundID)))
		    {
		        ok = Asura_Sound_System::LoadSound(pszRelativeFilename, static_cast<Asura_Sound_ID>(eSoundID));
				if(ok)
				{
					return (Asura_Sound_ID)eSoundID;
				}
				break;
		    }
		}
		return ASURA_SID_NO_SOUND;

	}

	return eSoundID;
}


Asura_Sound_ID Tools_PickSound::GetSoundIDForSoundEventFilename(const char* pszRelativeFilename)
{
	//See if sound is already loaded
	Asura_Hash_ID uWavHash = Asura_GetHashID(pszRelativeFilename);
	Asura_Sound_ID eSoundID = Asura_Sound_System::GetSoundIDFromHash(uWavHash);

	if(eSoundID == ASURA_SID_NO_SOUND)
	{
		//attempt to load as a dynamic sid
		if(Asura_Sound_System::LoadSound(pszRelativeFilename,ASURA_SID_DYNAMIC))
		{
			eSoundID = Asura_Sound_System::GetSoundIDFromHash(uWavHash);
		}
	}

	if(eSoundID == ASURA_SID_NO_SOUND)
	{
		Asura_Assertf(0,("Failed to load %s",pszRelativeFilename));
	}
	else
	{
		//make sure we got back the file we first thought of
		const char* szFilename = Asura_Sound_System::GetFileNameFromSoundID(eSoundID);
		Asura_Assert(szFilename,"");

		Asura_Assertf(!stricmp(szFilename,pszRelativeFilename),("Found wrong wav file.\nRequested %s\nFound %s",pszRelativeFilename,szFilename));
	}
	return eSoundID;
}

Asura_Sound_ID Tools_PickSound::PickSoundForSoundEventSample(Asura_Sound_ID eCurrentID,List<Asura_Sound_ID>* pxOutputSelectionList /*= 0*/, const char* szMessage /*=0*/)
{
	//return PickSound("Pick Wav for for sample",eCurrentID,ASURA_SID_FIRST_DYNAMIC,ASURA_SID_LAST_DYNAMIC);
	return PickSoundFromDisk(eCurrentID,ASURA_SID_FIRST_DYNAMIC,ASURA_SID_LAST_DYNAMIC,pxOutputSelectionList, szMessage);
}

#endif //ASURA_USE_AUDIO_SYSTEM
