//
// Tools_PickSoundSample_Overlay.h
//
// Created by Richard Rice - 04/07/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_PICKSOUNDSAMPLE_OVERLAY__
#define __TOOLS_PICKSOUNDSAMPLE_OVERLAY__

#include "Core/Asura_Hash.h"
#include "Core/Asura_Handle.h"
#include "Misc/Tools_Modal_Overlay.h"

class Tools_SoundSamplePreview;

class Tools_PickSoundSample_Overlay : public Tools_Modal_Overlay
{
public :
	Tools_PickSoundSample_Overlay(const char* szTitle,bool bMultipleSelect = false);
	~Tools_PickSoundSample_Overlay();

	//Sets default filters to use when opening overlay
	//(If one are set then the most recently used will be kept)
	static void SetFileFilter(const char* szFileFilter){s_xFileFilter = szFileFilter;}
	static void SetCategoryFilter(const char* szCategoryFilter){s_xCategoryFilter = szCategoryFilter;};
	static void SetSubCategoryFilter(const char* szSubCategoryFilter){s_xSubCategoryFilter = szSubCategoryFilter;};

	//The initial selection is mostly ignored if using multiple selection
	Asura_Hash_ID DoModal(Asura_Hash_ID uInitialSelection);

	//Gets selection if multiple selection is used
	int GetNumSelected() const;
	Asura_Hash_ID GetSelection(int iIndex) const;

private :
	virtual void InitialiseOverlay(class Asura_Overlay* pxOverlay);
	virtual void Update();

	void BuildFilterLists();
	void DeleteFilterLists();
	void BuildSubCategoryList();
	void DeleteSubCategoryList();

	float CalculateDropDownListWidth();


	void BuildSampleList();
	void DeleteSampleList();
	Asura_Hash_ID GetSampleHash(int iIndex) const;

	void UpdateSampleInfoText();
	void UpdateCompressionFlags(int iQualityIdx);

	static void OnOkay();
	static void OnCancel();
	static void OnNewSample();
	static void OnEditSample();

	static void OnDoubleClickList(int iJunk){OnOkay();}

	static void TestSound();
	static void StopSound();

	static bool IsFilterEnabled(int iFilterIdx);
	static void SetFilterPreset(int iPresetValue);
	static int GetCompressionFromFilter(int iFilter);

	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_PickSoundSample_Overlay*)iThis)->__FnName();}\
	void __FnName();
	
	OVERLAY_VOID_CALLBACK(DeleteSamples)

	#undef OVERLAY_VOID_CALLBACK

	enum
	{
		OVERLAY_CANCEL,
		OVERLAY_OKAY,
		OVERLAY_NEWSAMPLE,
		OVERLAY_EDITSAMPLE,
	} m_eResult;



	Asura_Hash_ID m_uInitialSelection;

	Asura_Hash_ID m_uMostRecentSelectedSample;//hash of most recent event selected by user
	Asura_Hash_ID m_uSelectedSample;//hash of currently selected event
	int m_iSampleIndex;
	bool m_bSampleChanged;

	bool m_bMultipleSelection;
	List<Asura_Hash_ID> m_xSelectionList;

	int m_iSelectedFileFilter;
	int m_iSelectedCategoryFilter;
	int m_iSelectedSubCategoryFilter;
	bool m_bFilterChanged;
	bool m_bCategoryChanged;

	//list for file filter
	u_int m_uFileListSize;
	const char** m_apszFileList;
	class Asura_StringTree* m_pxFileStringTree;

	//list for category filter
	u_int m_uCategoryListSize;
	const char** m_apszCategoryList;
	class Asura_StringTree* m_pxCategoryStringTree;

	//list for subcategory filter
	u_int m_uSubCategoryListSize;
	const char** m_apszSubCategoryList;
	class Asura_StringTree* m_pxSubCategoryStringTree;


	//list of sample names
	u_int m_uSampleListSize;
	const char** m_apszSampleList;
	class Asura_StringTree* m_pxSampleStringTree;

	class Asura_Dialog* m_pxInputDialog;
	float m_fInputTimer;
	char m_szInput[20];

	//sample filter & flag
	int m_iSampleRateFilter;
	int m_iCompressionQualityFilter; // Flag
	bool m_bSampleFilterEnabled;
	bool m_bCompressionFilterEnabled;

	enum
	{
		uNUM_SAMPLERATE_PLATFORMS = 3,	// PS2, PSP, Wii - these platforms use the sample rate
		uNUM_COMPRESSION_PLATFORMS = 2	// PS3, XBOX360 - they use compression quality
	};

	int m_aiSamplePlatforms[uNUM_SAMPLERATE_PLATFORMS];
	int m_aiCompressionPlatforms[uNUM_COMPRESSION_PLATFORMS];
	const char* m_aszSamplePlatformNames[uNUM_SAMPLERATE_PLATFORMS];
	const char* m_aszCompressionPlatformNames[uNUM_COMPRESSION_PLATFORMS];
	int m_iSamplePlatformIndex;
	int m_iCompressionPlatformIndex;

	Asura_String m_xSampleInfoText;

	//text search filter
	int m_iSearchOption;
	Asura_String m_xSearchText;


	class Asura_List_Dialog* m_pxSampleListDialog;
	class Asura_DropDown_List_Dialog* m_pxSubCategoryListDialog;

	Tools_SoundSamplePreview* m_pxSoundSamplePreview;
	static bool s_bAutoPreview;

	//most recently used filters
	static Asura_String s_xFileFilter;
	static Asura_String s_xCategoryFilter;
	static Asura_String s_xSubCategoryFilter;

	static Tools_PickSoundSample_Overlay* s_pxThis;
};

#endif //__TOOLS_PICKSOUNDSAMPLE_OVERLAY__
