//
// Tools_SoundConversionPreview.h
//
// Created by Richard Rice - 11/03/2009
// 
// Copyright (C) 2009 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_SOUNDCONVERSIONPREVIEW__
#define __TOOLS_SOUNDCONVERSIONPREVIEW__

//size of history of converted files
#define TOOLS_SOUNDCONVERSIONPREVIEW_NUMFILES 10

class Tools_SoundConversionPreview_ConvertedFile;

class Tools_SoundConversionPreview
{
public :
	static void Initialise();
	static void Shutdown();

	static void Update();

	//Starts conversion of a sound for a given platform/quality
	//Sound will be started during the next Update() in which it is ready
	static void PreviewSound(const char* szFilename, int iPlatform, int iQuality);

	//Plays the original sound without any platform conversion stuff.
	static void PreviewSound(const char* szFilename);

	//stops any sound conversion and playback
	static void StopAll();

	//stop any sound being played by the system
	static void StopSound();

	//Invalidate the history of converted files. To cope with the definitions of the quality levels changing
	static void InvalidateFiles();

private :
	static bool ConvertSound(int iIndex, const char* szFilename, int iPlatform, int iQuality);

	//stop any sound conversion in progress
	static void StopConversion();

	static void PlaySound(Tools_SoundConversionPreview_ConvertedFile* pxFile);


	//flag sound to start playing when ready
	static void SetSoundToPlay(int iIndex);

	static void ConvertSound_PS3(const void* pData);
	static void ConvertSound_PS3_Main(Tools_SoundConversionPreview_ConvertedFile* pxFile);

	static void ConvertSound_XBox360(const void* pData);
	static void ConvertSound_XBox360_Main(Tools_SoundConversionPreview_ConvertedFile* pxFile);

	///////////////
	static Tools_SoundConversionPreview_ConvertedFile* s_pxConvertedFiles;

	static int s_iSoundToPlay;

	static bool s_bAbortThread;
	static class Asura_SyncEvent* s_pxThreadFinishedSignal;
};

//for storting details of recently converted files
class Tools_SoundConversionPreview_ConvertedFile
{
public :
	Tools_SoundConversionPreview_ConvertedFile();

	void SetLastUsedStamp();

	bool Matches(const char* szFilename, int iPlatform, int iQuality) const;

	bool IsInvalid(){return m_eState == FILESTATE_INVALID;}
	bool IsReady() {return m_eState == FILESTATE_READY;}

	void Invalidate(){m_eState = FILESTATE_INVALID;}

	Asura_String m_xRelativeFilename;//original wav file
	Asura_String m_xFullFilename;//local or network file to convert from
	Asura_String m_xConvertedFilename;//name of the converted file
	Asura_String m_xError;//filled in if converison fails
	int m_iPlatform;//platform this was converted for
	int m_iQuality;//quality settng used for the conversion
	u_int m_uFileSize;//size of intermediate file
	u_int m_uLastUsedStamp;//used to determine how recently this entry was used

	enum
	{
		FILESTATE_INVALID,
		FILESTATE_BEING_CONVERTED,
		FILESTATE_READY
	} m_eState;

};

#endif //__TOOLS_SOUNDCONVERSIONPREVIEW__