//
// Tools_SoundEventTab_Event.cpp
//
// Created by Richard Rice - 07/07/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//


#include "Core/Asura.h"
#include "Sound/Tools_SoundEventTab_Event.h"

#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Dialog.h"
#include "Sound/Asura_Phonon.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_System.h"

#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_MessageLog.h"
#include "Sound/Tools_Audio_DSP.h"
#include "Sound/Tools_PickSoundEvent_Overlay.h"
#include "Sound/Tools_SoundMixer_Overlay.h"
#include "Sound/Tools_Phonon_Object.h"
#include "Sound/Tools_PickSoundSample_Overlay.h"
#include "Sound/Tools_SoundEvent_System.h"
#include "Sound/Tools_SurroundPan_Dialog.h"
#include "Sound/Tools_Mode_Sound.h"

#include "Tools_System.h"


static char* g_aszPlaybackTypes[] =
{
	"Simultaneous",
	"All Sequential",
	"One By One",
	"Random",
	"CD Player Random",
	"Variable",
};

static char* g_aszPlaybackTypeHelp[] =
{
	"Plays all events/samples at once",
	"One at a time , triggering next one when previous finishes,",
	"Plays next in list each time the event is played.",
	"Pick at random.",
	"Each time it is triggered, the sample chosen is picked at random from those that haven't been played yet. When all samples have been played the process is restarted.",
	"The volume and pitch of the samples change randomly during playback as defined by the variable playback settings.\nThis is otherwise treated the same as the simultaneous playback type.",
};

static char* g_asz3dTypes[] =
{
	"Point",
	"Solid",
	"No Pan",
	"Direct",
	"Random Pos",
};


static char* g_asz3dTypeHelp[] =
{
	"Pan relative to point position.",
	"No pan while inside the inner radius.",
	"No pan.",
	"Position doesn't modify pan and volume.",
	"Position doesn't modify pan and volume, and pan is random around the player.",
};

//////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_MessageLogAction_EditEvent **
**----------------------------------*/
class Tools_MessageLogAction_EditEvent : public Tools_MessageLogAction
{
public :
	Tools_MessageLogAction_EditEvent(const Asura_Sound_Event* pxEvent) {m_uID = pxEvent->GetHashID();}
	void Execute() {Tools_SoundEvent_Overlay::EditEvent(m_uID);}

private:
	Asura_Hash_ID m_uID;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------**
** Tools_SoundEventProperty_Overlay **
**----------------------------------*/

//Stuff for property copying overlay
enum Tools_SoundEvent_Properties
{
	Tools_SoundEventProp_Filename,
	Tools_SoundEventProp_Category,
	Tools_SoundEventProp_SubCategory,
	Tools_SoundEventProp_Mixer,

	Tools_SoundEventProp_MinVol,
	Tools_SoundEventProp_MaxVol,
	Tools_SoundEventProp_MinPitch,
	Tools_SoundEventProp_MaxPitch,
	Tools_SoundEventProp_MinDelay,
	Tools_SoundEventProp_MaxDelay,
	Tools_SoundEventProp_MinDuration,
	Tools_SoundEventProp_MaxDuration,

	Tools_SoundEventProp_MinInterval,
	Tools_SoundEventProp_MaxInterval,
	Tools_SoundEventProp_MinVolumeChangeTime,
	Tools_SoundEventProp_MaxVolumeChangeTime,
	Tools_SoundEventProp_MinPitchChangeTime,
	Tools_SoundEventProp_MaxPitchChangeTime,

	Tools_SoundEventProp_InnerRadius,
	Tools_SoundEventProp_OuterRadius,
	Tools_SoundEventProp_PlayProbability,
	Tools_SoundEventProp_Priority,

	Tools_SoundEventProp_PlaybackType,
	Tools_SoundEventProp_3dSoundType,

	Tools_SoundEventProp_PanOverride,
	Tools_SoundEventProp_SurroundPan,

	Tools_SoundEventProp_Looping,
	Tools_SoundEventProp_Occlusion,
	Tools_SoundEventProp_DistanceFilter,
	Tools_SoundEventProp_Doppler,
	Tools_SoundEventProp_EnvironmentalReverb,
	Tools_SoundEventProp_MasterDSP,
	Tools_SoundEventProp_GlobalSound,

	Tools_SoundEventProp_PlayableList,

	Tools_SoundEventProp_EventMapping,

	Tools_SoundEventProp_MaxInstances,
	Tools_SoundEventProp_MinFreeVoices,
	Tools_SoundEventProp_InstanceGroup,

	Tools_SoundEventProp_SoundType,

	Tools_SoundEventProp_DSPChain,

	Tools_SoundEventProp_NumProp,
};

static const char* g_aszPropertyNames[Tools_SoundEventProp_NumProp] =
{
	"Filename",
	"Category",
	"Subcategory",
	"Mixer",

	"Min Volume",
	"Max Volume",
	"Min Pitch",
	"Max Pitch",
	"Min Delay",
	"Max Delay",
	"Min Duration",
	"Max Duration",

	"Min Interval",
	"Max Interval",
	"Min Volume Change Time",
	"Max Volume Change Time",
	"Min Pitch Change Time",
	"Max Pitch Change Time",

	"Inner Range",
	"Outer Range",
	"Play Probability",
	"Priority",

	"Playback Type",
	"3D Sound Type",

	"Pan Override",
	"Surround Pan",

	"Loop Event",
	"Occlusion",
	"Distance Filter",
	"Doppler",
	"Environmental Reverb",
	"Master DSP",
	"Global Sound",

	"Sound/Event List",

	"Event Mapping",

	"Max Instances",
	"Min Free Voices",
	"Instance Group",

	"Sound Type",

	"DSP Chain",

};


//Overlay for selecting properties to paste

class Tools_SoundEventProperty_Overlay : public ListSelection_Overlay
{
public :
	Tools_SoundEventProperty_Overlay()
	:ListSelection_Overlay("Select properties to paste")
	{
		//initialise the list
		SetSortList(false);

		for(int i=0;i<Tools_SoundEventProp_NumProp;i++)
		{
			AddItem(g_aszPropertyNames[i],i);
			SetItemTicked(i,s_bSelected[i]);
		}
	}
	
	virtual bool DoModal()
	{
		if(!ListSelection_Overlay::DoModal()) return false;

		//copy settings to static array
		for(int i=0;i<Tools_SoundEventProp_NumProp;i++)
		{
			s_bSelected[i] = GetItemTicked(i);
		}
		return true;
	}

	const bool* GetSelectedArray() const {return s_bSelected;}

protected :
	virtual bool IsTickboxList() const {return true;}
	virtual const char* GetListHeadingText() const {return "Properties :";}
	virtual const char* GetTickAllButtonText() const {return "Select All";}
	virtual const char* GetTickNoneButtonText() const {return "Deselect All";}


	static bool s_bSelected[Tools_SoundEventProp_NumProp];
};

bool Tools_SoundEventProperty_Overlay::s_bSelected[Tools_SoundEventProp_NumProp];




//////////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------**
** Tools_SoundEvent **
**------------------*/


Tools_SoundEvent::Tools_SoundEvent()
{
	m_pxPlayableID = 0;
}

Tools_SoundEvent::~Tools_SoundEvent()
{
	delete [] m_pxPlayableID;
}

void Tools_SoundEvent::CopyFromAsuraEvent(const Asura_Sound_Event* pxEvent)
{
	m_fMinVol = pxEvent->GetMinVolume();
	m_fMaxVol = pxEvent->GetMaxVolume();
	m_fMinPitch = pxEvent->GetMinPitch();
	m_fMaxPitch = pxEvent->GetMaxPitch();
	m_fMinDelay = pxEvent->GetMinDelay();
	m_fMaxDelay = pxEvent->GetMaxDelay();
	m_fMinDuration = pxEvent->GetMinDuration();
	m_fMaxDuration = pxEvent->GetMaxDuration();

	m_xSurroundPan = pxEvent->GetSurroundPan();
	m_iPlayProbability = pxEvent->GetPlayProbability();
	m_fPriority = pxEvent->GetPriority();
	m_iPlaybackType = pxEvent->GetPlaybackType();

	m_bLooping = pxEvent->IsLooping();
	m_bUseOcclusion = pxEvent->UsesOcclusion();
	m_bUseDistanceFilter = pxEvent->UsesDistanceFilter();
	m_bUseDoppler = pxEvent->UsesDoppler();
	m_bUseEnvironmentalReverb = pxEvent->UsesEnvironmentalReverb();
	m_bUseMasterDSP = pxEvent->UsesMasterDSP();
	m_bPanOverride = pxEvent->IsPanOverride();
	m_bGlobalSound = pxEvent->IsGlobalSound();
	m_bEventMapping = pxEvent->IsEventMapping();
	
	m_i3dSoundType = pxEvent->Get3dSoundType();
	m_fInnerRadius = pxEvent->GetInnerRadius();
	m_fOuterRadius = pxEvent->GetOuterRadius();
	m_uMaxInstances = pxEvent->GetMaxInstances();
	m_uMinFreeVoices = pxEvent->GetMinFreeVoices();

	if(pxEvent->GetPlaybackType() == ASURA_PT_VARIABLE)
	{
		m_xVariableParams = *pxEvent->GetVariableParams();
	}
	else
	{
		m_xVariableParams.Initialise();
	}


	m_uNumPlayable = pxEvent->GetNumPlayableID();

	delete [] m_pxPlayableID;
	m_pxPlayableID = m_uNumPlayable ? new Asura_Sound_PlayableID[m_uNumPlayable] : 0;
	for(u_int i=0;i<m_uNumPlayable;i++)
	{
		m_pxPlayableID[i] = pxEvent->GetPlayableID(i);
	}

	m_xCategory = pxEvent->GetEventType();
	m_xSubCategory = pxEvent->GetEventSubType();
	m_xFilename = pxEvent->GetFilename();
	m_xMixer = pxEvent->GetMixerChannelName();
	m_xInstanceGroupName = pxEvent->GetInstanceGroupName();

	m_uSoundType = (int)pxEvent->GetSoundType();

	m_uDSPChainHash = pxEvent->GetDSPChainHash();
}

void Tools_SoundEvent::CopyToAsuraEvent(Asura_Sound_Event* pxEvent) const
{
	pxEvent->SetVolume(m_fMinVol,m_fMaxVol);
	pxEvent->SetPitch(m_fMinPitch,m_fMaxPitch);
	pxEvent->SetDelay(m_fMinDelay,m_fMaxDelay);
	pxEvent->SetDuration(m_fMinDuration, m_fMaxDuration);

	pxEvent->SetSurroundPan(m_xSurroundPan);
	pxEvent->SetPlayProbability((u_char)m_iPlayProbability);
	pxEvent->SetPriority(m_fPriority);
	pxEvent->SetPlaybackType((ASURA_SOUND_PLAYBACK_TYPE) m_iPlaybackType);

	pxEvent->SetLooping(m_bLooping);
	pxEvent->SetOcclusion(m_bUseOcclusion);
	pxEvent->SetDistanceFilter(m_bUseDistanceFilter);
	pxEvent->SetDoppler(m_bUseDoppler);
	pxEvent->SetEnvironmentalReverb(m_bUseEnvironmentalReverb);
	pxEvent->SetMasterDSP(m_bUseMasterDSP);
	pxEvent->SetPanOverride(m_bPanOverride);
	pxEvent->SetGlobalSound(m_bGlobalSound);
	pxEvent->SetEventMapping(m_bEventMapping);
	
	pxEvent->Set3dSoundType((ASURA_3DSOUND_TYPE)m_i3dSoundType);
	pxEvent->SetRadii(m_fInnerRadius, m_fOuterRadius);
	pxEvent->SetMaxInstances(m_uMaxInstances);
	pxEvent->SetMinFreeVoices(m_uMinFreeVoices);

	if(pxEvent->GetPlaybackType() == ASURA_PT_VARIABLE)
	{
		pxEvent->SetVariableParams(m_xVariableParams);
	}

	if(pxEvent->SetPlayableIDList(m_pxPlayableID,m_uNumPlayable))
	{
		Tools_Mode_Sound::InvalidateSoundSize();
	}

	pxEvent->SetEventType(m_xCategory.GetString());
	pxEvent->SetEventSubType(m_xSubCategory.GetString());
	pxEvent->SetFilename(m_xFilename.GetString());
	pxEvent->SetMixerChannelName(m_xMixer.GetString());
	pxEvent->SetInstanceGroupName(m_xInstanceGroupName.GetString());

	pxEvent->SetSoundType( m_uSoundType );

	pxEvent->SetDSPChainHash( m_uDSPChainHash );
}

Tools_SoundEvent& Tools_SoundEvent::operator = (const Tools_SoundEvent& xEvent)
{
	if(this != &xEvent) 
	{
		m_fMinVol = xEvent.m_fMinVol;
		m_fMaxVol = xEvent.m_fMaxVol;
		m_fMinPitch = xEvent.m_fMinPitch;
		m_fMaxPitch = xEvent.m_fMaxPitch;
		m_fMinDelay = xEvent.m_fMinDelay;
		m_fMaxDelay = xEvent.m_fMaxDelay;
		m_fMinDuration = xEvent.m_fMinDuration;
		m_fMaxDuration = xEvent.m_fMaxDuration;

		m_xSurroundPan = xEvent.m_xSurroundPan;
		m_iPlayProbability = xEvent.m_iPlayProbability;
		m_fPriority = xEvent.m_fPriority;
		m_iPlaybackType = xEvent.m_iPlaybackType;
		m_bLooping = xEvent.m_bLooping;
		m_bUseOcclusion = xEvent.m_bUseOcclusion;
		m_bUseDistanceFilter = xEvent.m_bUseDistanceFilter;
		m_bUseDoppler = xEvent.m_bUseDoppler;
		m_bUseEnvironmentalReverb = xEvent.m_bUseEnvironmentalReverb;
		m_bUseMasterDSP = xEvent.m_bUseMasterDSP;
		m_bPanOverride = xEvent.m_bPanOverride;
		m_bGlobalSound = xEvent.m_bGlobalSound;
		m_bEventMapping = xEvent.m_bEventMapping;

		m_i3dSoundType = xEvent.m_i3dSoundType;
		m_fInnerRadius = xEvent.m_fInnerRadius;
		m_fOuterRadius = xEvent.m_fOuterRadius;
		m_uMaxInstances = xEvent.m_uMaxInstances;
		m_uMinFreeVoices = xEvent.m_uMinFreeVoices;

		m_xVariableParams = xEvent.m_xVariableParams;

		m_xCategory = xEvent.m_xCategory;
		m_xSubCategory = xEvent.m_xSubCategory;
		m_xFilename = xEvent.m_xFilename;
		m_xMixer = xEvent.m_xMixer;
		m_xInstanceGroupName = xEvent.m_xInstanceGroupName;

		m_uSoundType = xEvent.m_uSoundType;

		m_uDSPChainHash = xEvent.m_uDSPChainHash;

		delete [] m_pxPlayableID;
		m_uNumPlayable = xEvent.m_uNumPlayable;
		m_pxPlayableID = m_uNumPlayable ? new Asura_Sound_PlayableID[m_uNumPlayable] : 0;
		for(u_int i=0;i<m_uNumPlayable;i++)
		{
			m_pxPlayableID[i] = xEvent.m_pxPlayableID[i];
		}
		
	}
	return *this;
}

void Tools_SoundEvent::Copy(const Asura_Sound_Event* pxThisEvent,const Tools_SoundEvent& xEventToCopy,const bool* pbProperties)
{
	if(this == &xEventToCopy) return;

	if(pbProperties[Tools_SoundEventProp_MinVol])
	{
		m_fMinVol = xEventToCopy.m_fMinVol;
		m_fMaxVol = Asura_Maths::Max(m_fMinVol, m_fMaxVol);
	}
	
	if(pbProperties[Tools_SoundEventProp_MaxVol])
	{
		m_fMaxVol = xEventToCopy.m_fMaxVol;
		m_fMinVol = Asura_Maths::Min(m_fMinVol, m_fMaxVol);
	}
	
	if(pbProperties[Tools_SoundEventProp_MinPitch])
	{
		m_fMinPitch = xEventToCopy.m_fMinPitch;
		m_fMaxPitch = Asura_Maths::Max(m_fMinPitch, m_fMaxPitch);
	}
	
	if(pbProperties[Tools_SoundEventProp_MaxPitch])
	{
		m_fMaxPitch = xEventToCopy.m_fMaxPitch;
		m_fMinPitch = Asura_Maths::Min(m_fMinPitch, m_fMaxPitch);
	}
	
	if(pbProperties[Tools_SoundEventProp_MinDelay])
	{
		m_fMinDelay = xEventToCopy.m_fMinDelay;
		m_fMaxDelay = Asura_Maths::Max(m_fMinDelay, m_fMaxDelay);
	}
	
	if(pbProperties[Tools_SoundEventProp_MaxDelay])
	{
		m_fMaxDelay = xEventToCopy.m_fMaxDelay;
		m_fMinDelay = Asura_Maths::Min(m_fMinDelay, m_fMaxDelay);
	}

	if(pbProperties[Tools_SoundEventProp_MinDuration])
	{
		m_fMinDuration = xEventToCopy.m_fMinDuration;
		m_fMaxDuration = Asura_Maths::Max(m_fMinDuration, m_fMaxDuration);
	}

	if(pbProperties[Tools_SoundEventProp_MaxDuration])
	{
		m_fMaxDuration = xEventToCopy.m_fMaxDuration;
		m_fMinDuration = Asura_Maths::Min(m_fMinDuration, m_fMaxDuration);
	}
	
	if(pbProperties[Tools_SoundEventProp_SurroundPan])
	{
		m_xSurroundPan = xEventToCopy.m_xSurroundPan;
	}
	
	if(pbProperties[Tools_SoundEventProp_PlayProbability])
	{
		m_iPlayProbability = xEventToCopy.m_iPlayProbability;
	}

	if(pbProperties[Tools_SoundEventProp_Priority])
	{
		m_fPriority = xEventToCopy.m_fPriority;
	}
	
	if(pbProperties[Tools_SoundEventProp_PlaybackType])
	{
		m_iPlaybackType = xEventToCopy.m_iPlaybackType;
	}
	
	if(pbProperties[Tools_SoundEventProp_Looping])
	{
		m_bLooping = xEventToCopy.m_bLooping;
	}
	
	if(pbProperties[Tools_SoundEventProp_Occlusion])
	{
		m_bUseOcclusion = xEventToCopy.m_bUseOcclusion;
	}

	if(pbProperties[Tools_SoundEventProp_DistanceFilter])
	{
		m_bUseDistanceFilter = xEventToCopy.m_bUseDistanceFilter;
	}

	if(pbProperties[Tools_SoundEventProp_Doppler])
	{
		m_bUseDoppler = xEventToCopy.m_bUseDoppler;
	}

	if(pbProperties[Tools_SoundEventProp_EnvironmentalReverb])
	{
		m_bUseEnvironmentalReverb = xEventToCopy.m_bUseEnvironmentalReverb;
	}

	if(pbProperties[Tools_SoundEventProp_MasterDSP])
	{
		m_bUseMasterDSP = xEventToCopy.m_bUseMasterDSP;
	}

	if(pbProperties[Tools_SoundEventProp_PanOverride])
	{
		m_bPanOverride = xEventToCopy.m_bPanOverride;
	}
	
	if(pbProperties[Tools_SoundEventProp_GlobalSound])
	{
		m_bGlobalSound = xEventToCopy.m_bGlobalSound;
	}
	
	if(pbProperties[Tools_SoundEventProp_3dSoundType])
	{
   		m_i3dSoundType = xEventToCopy.m_i3dSoundType;
	}
	
	if(pbProperties[Tools_SoundEventProp_InnerRadius])
	{
		m_fInnerRadius = xEventToCopy.m_fInnerRadius;
	}
	
	if(pbProperties[Tools_SoundEventProp_OuterRadius])
	{
		m_fOuterRadius = xEventToCopy.m_fOuterRadius;
	}

	if(pbProperties[Tools_SoundEventProp_MaxInstances])
	{
		m_uMaxInstances = xEventToCopy.m_uMaxInstances;
	}

	if(pbProperties[Tools_SoundEventProp_MinFreeVoices])
	{
		m_uMinFreeVoices = xEventToCopy.m_uMinFreeVoices;
	}

	if(pbProperties[Tools_SoundEventProp_MinInterval])
	{
		m_xVariableParams.m_fMinInterval = xEventToCopy.m_xVariableParams.m_fMinInterval;
	}

	if(pbProperties[Tools_SoundEventProp_MaxInterval])
	{
		m_xVariableParams.m_fMaxInterval = xEventToCopy.m_xVariableParams.m_fMaxInterval;
	}

	if(pbProperties[Tools_SoundEventProp_MinVolumeChangeTime])
	{
		m_xVariableParams.m_fMinVolumeChangeTime = xEventToCopy.m_xVariableParams.m_fMinVolumeChangeTime;
	}

	if(pbProperties[Tools_SoundEventProp_MaxVolumeChangeTime])
	{
		m_xVariableParams.m_fMaxVolumeChangeTime = xEventToCopy.m_xVariableParams.m_fMaxVolumeChangeTime;
	}

	if(pbProperties[Tools_SoundEventProp_MinPitchChangeTime])
	{
		m_xVariableParams.m_fMinPitchChangeTime = xEventToCopy.m_xVariableParams.m_fMinPitchChangeTime;
	}

	if(pbProperties[Tools_SoundEventProp_MaxPitchChangeTime])
	{
		m_xVariableParams.m_fMaxPitchChangeTime = xEventToCopy.m_xVariableParams.m_fMaxPitchChangeTime;
	}


	
	if(pbProperties[Tools_SoundEventProp_Category])
	{
		m_xCategory = xEventToCopy.m_xCategory;
	}

	if(pbProperties[Tools_SoundEventProp_SubCategory])
	{
		m_xSubCategory = xEventToCopy.m_xSubCategory;
	}
	
	if(pbProperties[Tools_SoundEventProp_Filename])
	{
		//Make sure it is okay to change the file
		if(Tools_SoundEvent_System::CanTransferEvent(pxThisEvent,xEventToCopy.m_xFilename.GetString()))
		{
			m_xFilename = xEventToCopy.m_xFilename;
		}
		else
		{
			//report an error
			Tools_MessageLog::AddMessage(Tools_MessageLog::SOUND_EVENT,"Paste Error",new Tools_MessageLogAction_EditEvent(pxThisEvent),"Failed to transfer '%s' since source and destination aren't both writeable",pxThisEvent->GetName());
		}
	}
	
	if(pbProperties[Tools_SoundEventProp_Mixer])
	{
		m_xMixer = xEventToCopy.m_xMixer;
	}
	
	if(pbProperties[Tools_SoundEventProp_InstanceGroup])
	{
		m_xInstanceGroupName = xEventToCopy.m_xInstanceGroupName;
	}

	
	if(pbProperties[Tools_SoundEventProp_PlayableList])
	{
		//make sure that pasting over this event won't cause it to include itself
		bool bOkay = true;
		for(u_int i=0;i<xEventToCopy.m_uNumPlayable;i++)
		{
			if(xEventToCopy.m_pxPlayableID[i].m_eType == ASURA_SOUND_PLAYABLE_EVENT)
			{
				if(!Tools_SoundEventTab_Event::IsOkayToAddEvent(pxThisEvent->GetHashID(),xEventToCopy.m_pxPlayableID[i].m_uHashID))
				{
					bOkay = false;
				}
			}
		}

		if(bOkay)
		{
			delete [] m_pxPlayableID;
			m_uNumPlayable = xEventToCopy.m_uNumPlayable;
			m_pxPlayableID = m_uNumPlayable ? new Asura_Sound_PlayableID[m_uNumPlayable] : 0;
			for(u_int i=0;i<m_uNumPlayable;i++)
			{
				m_pxPlayableID[i] = xEventToCopy.m_pxPlayableID[i];
			}
		}
		else
		{
			//report an error
			Tools_MessageLog::AddMessage(Tools_MessageLog::SOUND_EVENT,"Paste Error",new Tools_MessageLogAction_EditEvent(pxThisEvent),"Couldn't paste event list over '%s' because it would result in self-inclusion",pxThisEvent->GetName());
		}
	}
	
	if(pbProperties[Tools_SoundEventProp_EventMapping])
	{
		m_bEventMapping = xEventToCopy.m_bEventMapping;
	}

	if (pbProperties[Tools_SoundEventProp_SoundType])
	{
		m_uSoundType = xEventToCopy.m_uSoundType;
	}

	if(pbProperties[Tools_SoundEventProp_DSPChain])
	{
		m_uDSPChainHash = xEventToCopy.m_uDSPChainHash;
	}


	//If only some settings have been copied ,the event mapping flag may be illegally set
	if(!CanBeEventMapping())
	{
		m_bEventMapping = false;
	}
}

bool Tools_SoundEvent::CanBeEventMapping() const
{
	//Allowable if the playable list contains exactly one event
	if(m_uNumPlayable != 1) return false;
	return m_pxPlayableID[0].m_eType == ASURA_SOUND_PLAYABLE_EVENT;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_SoundEventTab_Event **
**---------------------------*/

bool Tools_SoundEventTab_Event::s_bPasteBufferValid = false;
Tools_SoundEvent Tools_SoundEventTab_Event::s_xPasteBuffer;

Tools_SoundEventTab_Event::Tools_SoundEventTab_Event(Asura_Tabs_Dialog* pxTabsDialog,Asura_Hash_ID uEventHash)
{
	m_uEventHash = uEventHash;
	m_pxEvent = Asura_Sound_Event_Storage::GetEventByHash(uEventHash);
	Asura_Assert(m_pxEvent,"");

	m_xEventData.CopyFromAsuraEvent(m_pxEvent);
	m_bMinChanged = false;
	m_bMaxChanged = false;
	m_bFileChanged = false;
	m_bCategoryChanged = false;
	m_bChanged = false;
	m_bSoundTypeChanged = false;

	m_bBeingEdited = false;

	m_uPlayableListSize = 0;
	m_pszPlayableList = 0;
	m_iSelectedPlayable = 0;
	m_pxPlayableListDialog = 0;
	m_iSelectedSoundType = m_xEventData.m_uSoundType;

	AddOverlayControls(pxTabsDialog);
}

void Tools_SoundEventTab_Event::AddOverlayControls(Asura_Tabs_Dialog* pxTabsDialog)
{
	//create new tab on the tab dialog
	UpdateTabName();
	pxTabsDialog->AddTab(m_xTabName.GetString());

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = pxTabsDialog->GetTopDialogPosition();

	//filename
	pxDialog = new Tools_SoundFilename_Dialog(xOffset,&m_xEventData.m_xFilename,&m_bFileChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//category
	pxDialog = new Tools_SoundCategory_Dialog(xOffset,&m_xEventData.m_xCategory,&m_bCategoryChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//sub category
	pxDialog = new Tools_SoundSubCategory_Dialog(xOffset,&m_xEventData.m_xCategory,&m_xEventData.m_xSubCategory,&m_bChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//mixer
	pxDialog = new Tools_SoundMixer_Dialog(xOffset,&m_xEventData.m_xMixer,&m_bChanged);
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// 	// Only add this to the Sound Event overlay if the Sound_Event_system function pointers have been defined.
	if ( Asura_Sound_Event_System::GetSoundRegistryFunctionDefined() )
	{
		// Call the project specific add data function here
		if ( Tools_System::GetCurrentMode() == TOOLS_MODE_SOUND )
		{
			Asura_StringValuePair_DropDownList_Dialog* pxStringPairDialog = new Asura_StringValuePair_DropDownList_Dialog(xOffset, "Sound Type", &m_iSelectedSoundType, &m_bSoundTypeChanged);
			
			Tools_Mode_Sound* pxToolsModeSound = ( Tools_Mode_Sound* )Tools_System::GetCurrentModeClass();

			if ( pxToolsModeSound )
			{
				pxToolsModeSound->BuildSoundTypeList( pxStringPairDialog );
			}
		

			pxStringPairDialog->InitialiseData();
			pxStringPairDialog->CreateGadgets(xOffset);
			pxStringPairDialog->SetRelativePosition(pxTabsDialog, Asura_Tabs_Dialog::s_uTopLeftPositionID);
			pxTabsDialog->AddDialog(pxStringPairDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
		}
	}

	//add controls for editing the event

	//table of min/maxes
	{
		const float fCol0 = xOffset.x;
		const float fCol1 = fCol0+10.0f;
		const float fCol2 = fCol1+10.0f;

		xOffset.x = fCol1+4.0f;
		pxDialog = new Asura_Dialog("Min",xOffset);
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol2+4.0f;
		pxDialog = new Asura_Dialog("Max",xOffset);
		pxTabsDialog->AddDialog(pxDialog);
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	

		//volume
		xOffset.x = fCol0;
		pxDialog = new Asura_Dialog("Volume",xOffset);
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol1;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_fMinVol, 0.01f,1.33f,0.01f,"%0.2f ",&m_bMinChanged);
		pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
		pxDialog->SetHelpText("Minimum volume multiplier.\nNote that 1.33 is effectively maximum , since all volumes are scaled by 0.75 and capped at 1.0 at the end.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol2;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_fMaxVol, 0.01f,1.33f,0.01f,"%0.2f ",&m_bMaxChanged);
		pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
		pxDialog->SetHelpText("Maximum volume multiplier.\nNote that 1.33 is effectively maximum , since all volumes are scaled by 0.75 and capped at 1.0 at the end.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		//pitch
		xOffset.x = fCol0;
		pxDialog = new Asura_Dialog("Pitch",xOffset);
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol1;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_fMinPitch, 0.01f,2.0f,0.01f,"%0.2f ",&m_bMinChanged);
		pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
		pxDialog->SetHelpText("Minimum pitch multiplier.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol2;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_fMaxPitch, 0.01f,2.0f,0.01f,"%0.2f ",&m_bMaxChanged);
		pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
		pxDialog->SetHelpText("Maximum pitch multiplier.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		//delay
		xOffset.x = fCol0;
		pxDialog = new Asura_Dialog("Delay",xOffset);
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol1;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_fMinDelay, 0,999.0f,0.1f,"%0.1f",&m_bMinChanged);
		pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
		pxDialog->SetHelpText("Minimum start delay.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol2;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_fMaxDelay, 0,999.0f,0.1f,"%0.1f",&m_bMaxChanged);
		pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
		pxDialog->SetHelpText("Maximum start delay.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		//duration
		xOffset.x = fCol0;
		pxDialog = new Asura_Dialog("Duration",xOffset);
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol1;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_fMinDuration, 0,999.0f,0.1f,"%0.1f",&m_bMinChanged);
		pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
		pxDialog->SetHelpText("If the maximum duration is non-zero then these two values are used to generate a random (maximum) duration for the event.\nIf non-looping the event may still end before the duration is reached.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol2;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_fMaxDuration, 0,999.0f,0.1f,"%0.1f",&m_bMaxChanged);
		pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
		pxDialog->SetHelpText("If the maximum duration is non-zero then these two values are used to generate a random (maximum) duration for the event.\nIf non-looping the event may still end before the duration is reached.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();


		//variable playback heading
		xOffset.x = fCol0;
		pxDialog = new Asura_Dialog("Variable playback settings :",xOffset);
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();


		//Interval
		xOffset.x = fCol0;
		pxDialog = new Asura_Dialog("Interval",xOffset);
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol1;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_xVariableParams.m_fMinInterval, 0.1f,999.0f,0.1f,"%0.1f",&m_bMinChanged);
		pxDialog->SetIsEnabledFunction(IsEnabled_VariableParam,(int)this);
		pxDialog->SetHelpText("Minimum time until the next random pitch/volume is chosen.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol2;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_xVariableParams.m_fMaxInterval, 0.1f,999.0f,0.1f,"%0.1f",&m_bMaxChanged);
		pxDialog->SetIsEnabledFunction(IsEnabled_VariableParam,(int)this);
		pxDialog->SetHelpText("Maximum time until the next random pitch/volume is chosen.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		//volume change time
		xOffset.x = fCol0;
		pxDialog = new Asura_Dialog("Vol Change Time",xOffset);
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol1;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_xVariableParams.m_fMinVolumeChangeTime, 0.0f,1.0f,0.01f,"%0.2f ",&m_bMinChanged);
		pxDialog->SetIsEnabledFunction(IsEnabled_VariableParam,(int)this);
		pxDialog->SetHelpText("Minimum time over which the volume changes to its new value.\nThis is done as a proportion of the time until the next value is chosen.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol2;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_xVariableParams.m_fMaxVolumeChangeTime, 0.0f,1.0f,0.01f,"%0.2f ",&m_bMaxChanged);
		pxDialog->SetIsEnabledFunction(IsEnabled_VariableParam,(int)this);
		pxDialog->SetHelpText("Maximum time over which the volume changes to its new value.\nThis is done as a proportion of the time until the next value is chosen.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();

		//pitch	change time
		xOffset.x = fCol0;
		pxDialog = new Asura_Dialog("Pitch Change Time",xOffset);
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol1;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_xVariableParams.m_fMinPitchChangeTime, 0.0f,1.0f,0.01f,"%0.2f ",&m_bMinChanged);
		pxDialog->SetIsEnabledFunction(IsEnabled_VariableParam,(int)this);
		pxDialog->SetHelpText("Minimum time over which the pitch changes to its new value.\nThis is done as a proportion of the time until the next value is chosen.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);

		xOffset.x = fCol2;
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_xEventData.m_xVariableParams.m_fMaxPitchChangeTime, 0.0f,1.0f,0.01f,"%0.2f ",&m_bMaxChanged);
		pxDialog->SetIsEnabledFunction(IsEnabled_VariableParam,(int)this);
		pxDialog->SetHelpText("Maximum time over which the pitch changes to its new value.\nThis is done as a proportion of the time until the next value is chosen.");
		pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
		pxTabsDialog->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::LargeSpacing();

	
		xOffset.x = fCol0;
	}

	//inner and outer range
	pxDialog = new Asura_Numerical_Dialog("Inner Range",xOffset,&m_xEventData.m_fInnerRadius, 0.5f,9999.0f,0.1f,"%0.1f",&m_bMinChanged);
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Outer Range",xOffset,&m_xEventData.m_fOuterRadius, 0.5f,9999.0f,0.1f,"%0.1f",&m_bMaxChanged);
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	//play probability
	pxDialog = new Asura_Numerical_Dialog("Play Probability",xOffset,&m_xEventData.m_iPlayProbability,0,100,1,"%d",&m_bChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxDialog->SetHelpText("The probability that this sound event will play at all when triggered.");
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//priority
	pxDialog = new Asura_Numerical_Dialog("Priority",xOffset,&m_xEventData.m_fPriority,0.0f,2.0f,0.01f,"%.2f",&m_bChanged);
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//playback type
	Asura_DropDown_List_Dialog* pxListDialog = new Asura_DropDown_List_Dialog(xOffset,g_aszPlaybackTypes,&m_xEventData.m_iPlaybackType,6,6,NULL,&m_bChanged,"Playback type");
	pxListDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxListDialog->SetListHelpStrings(g_aszPlaybackTypeHelp);
	pxListDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxListDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//3d type
	pxListDialog = new Asura_DropDown_List_Dialog(xOffset,g_asz3dTypes,&m_xEventData.m_i3dSoundType,5,5,NULL,&m_bChanged,"3d type");
	pxListDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxListDialog->SetListHelpStrings(g_asz3dTypeHelp);
	pxListDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxListDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	const float fTickboxStartY = xOffset.y;

	//looping
	pxDialog = new Asura_Tickbox_Dialog("Loop Event",xOffset,&m_xEventData.m_bLooping,&m_bChanged);
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//occlusion
	pxDialog = new Asura_Tickbox_Dialog("Occlusion",xOffset,&m_xEventData.m_bUseOcclusion,&m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEdit3DEffects,(int)this);
	pxDialog->SetHelpText("Event will use occlusion (LOS to determine whether to reduce loudness).");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//distance filter
	pxDialog = new Asura_Tickbox_Dialog("Distance Filter",xOffset,&m_xEventData.m_bUseDistanceFilter,&m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEdit3DEffects,(int)this);
	pxDialog->SetHelpText("Event will use distance filter (low pass filter for distant sounds).");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//doppler
	pxDialog = new Asura_Tickbox_Dialog("Doppler",xOffset,&m_xEventData.m_bUseDoppler,&m_bChanged);
	pxDialog->SetIsEnabledFunction(CanEdit3DEffects,(int)this);
	pxDialog->SetHelpText("Event will use doppler (pitch change depending on velocity relative to listener).");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//environmental reverb 
	pxDialog = new Asura_Tickbox_Dialog("Env Reverb",xOffset,&m_xEventData.m_bUseEnvironmentalReverb,&m_bChanged);
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetHelpText("Event will use environmental reverb (reverb setting controlled by reverb volumes).");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//master DSP
	pxDialog = new Asura_Tickbox_Dialog("Master DSP",xOffset,&m_xEventData.m_bUseMasterDSP,&m_bChanged);
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetHelpText("Event will be affected by master DSP effects.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//pan override
	pxDialog = new Asura_Tickbox_Dialog("Pan Override",xOffset,&m_xEventData.m_bPanOverride,&m_bChanged);
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();




	pxDialog = new Asura_Function_Dialog("Play Event",xOffset,PlaySound,(int)this);
	pxDialog->SetHelpText("Plays this sound event.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Stop Event",xOffset,StopSound,(int)this);
	pxDialog->SetHelpText("Stops playback of test sound.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Reload Wavs",xOffset,ReloadWavs,(int)this);
	pxDialog->SetHelpText("Reload wav files associated with this event.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);


	//put the pan dialog to the side of the tickboxes
	xOffset.x += 15.0f;
	xOffset.y = fTickboxStartY;
	pxDialog = new Tools_SurroundPan_Dialog(xOffset,&m_xEventData.m_xSurroundPan,&m_bChanged);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.x -= 15.0f;
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;


	//global sound tickbox and event mapping in bottom right
	xOffset.x += 32.0f;


	//event mapping controls
	xOffset.y += 1.0f;
	pxDialog = new Asura_Tickbox_Dialog("Event Mapping",xOffset,&m_xEventData.m_bEventMapping,&m_bChanged);
	pxDialog->SetHelpText("Enabling the event mapping option will cause this event to map to a single target event.\nWhenever this event is played it will ignore the settings and go for the target instead.");
	pxDialog->SetIsEnabledFunction(IsEnabled_EventMapping,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);

	xOffset.x += 13.0f;
	pxDialog = new Asura_Function_Dialog("Set",xOffset,CreateMapping,(int)this);
	pxDialog->SetHelpText("Selects a single event for this event to be mapped to.\nThis will replace the current playable list with the new event.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.x -= 13.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Global Sound",xOffset,&m_xEventData.m_bGlobalSound,&m_bChanged);
	pxDialog->SetHelpText("Marks this event (and any events/samples it uses) for inclusion in the global game sounds file.");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);




	//Start a new column to the right
	xOffset = pxTabsDialog->GetTopDialogPosition();
	xOffset.x += 32.0f;
	
	//list of events/samples
	BuildPlayableList();
	
	pxDialog = new Asura_Dialog("Events/Samples :",xOffset);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = m_pxPlayableListDialog = new Asura_List_Dialog(xOffset,m_pszPlayableList,&m_iSelectedPlayable,8,m_uPlayableListSize,NULL);
	pxDialog->SetHelpText("List of events/samples played by this event");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + 1.0f;

	//function dialogs for manipulating event list
	pxDialog = new Asura_Function_Dialog("Add Sample",xOffset,AddSample,(int)this);
	pxDialog->SetHelpText("Add a sample to the list");
	pxDialog->SetIsEnabledFunction(CanAddSample,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Add Event",xOffset,AddEvent,(int)this);
	pxDialog->SetHelpText("Add an event to the list");
	pxDialog->SetIsEnabledFunction(CanAddEvent,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Remove",xOffset,RemovePlayable,(int)this);
	pxDialog->SetHelpText("Remove the selected event/sample from the list.");
	pxDialog->SetIsEnabledFunction(CanRemovePlayable,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//Put the next 4 over to the right
	xOffset.y -= Asura_Dialog::DefaultSpacing()*3;
	xOffset.x += 12.0f;
	pxDialog = new Asura_Function_Dialog("Edit",xOffset,EditPlayable,(int)this);
	pxDialog->SetHelpText("Edits the selected event/sample.");
	pxDialog->SetIsEnabledFunction(CanEditPlayable,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Move Up",xOffset,MoveUp,(int)this);
	pxDialog->SetHelpText("Moves the selected event/sample earlier in the list");
	pxDialog->SetIsEnabledFunction(CanMoveUp,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Move Down",xOffset,MoveDown,(int)this);
	pxDialog->SetHelpText("Moves the selected event/sample later in the list");
	pxDialog->SetIsEnabledFunction(CanMoveDown,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Sort",xOffset,Sort,(int)this);
	pxDialog->SetHelpText("Sorts the event/sample list alphabetically");
	pxDialog->SetIsEnabledFunction(CanSort,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.x -= 12.0f;

	//a little gap , and then the max instance controls
	xOffset.y += 1.0f;
 
	//max instances
	pxDialog = new Asura_Numerical_Dialog("Max Instances", xOffset, &m_xEventData.m_uMaxInstances, 0, 999, 1, "%d", &m_bChanged);
	pxDialog->SetHelpText("Attempts to play this event will be ignored if there are already this many instances of it playing.\n"
						  "This doesn't apply to any sub events triggered by an event.\n"
						  "If set to zero , then the global default will be used instead.");
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//min free voices
	pxDialog = new Asura_Numerical_Dialog("Min Free Voices", xOffset, &m_xEventData.m_uMinFreeVoices, 0, 999, 1, "%d", &m_bChanged);
	pxDialog->SetHelpText("Attempts to play this event will be ignored if there aren't this many voices free in the sound system.\n"
						  "This doesn't apply to any sub events triggered by an event.");
		
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//instance group
	pxDialog = new Tools_SoundInstanceGroup_Dialog(xOffset, &m_xEventData.m_xInstanceGroupName, &m_bChanged);
	pxDialog->SetHelpText("If this is assigned a group then all instances of events with the same group name will be counted when determining how many instances of this event are being played.");
	pxDialog->SetIsEnabledFunction(IsNotMapping,(int)this);
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	#ifdef ASURA_USE_AUDIO_SYSTEM
	// DSP chain selection
	pxDialog = new Tools_DSPChain_DropDownList_Dialog( xOffset, 10, &m_xEventData.m_uDSPChainHash, &m_bChanged, "DSP Chain" );
	pxDialog->SetHelpText( "Set the DSP chain to apply to this event when played");
	pxDialog->SetRelativePosition(pxTabsDialog,Asura_Tabs_Dialog::s_uTopLeftPositionID);
	pxTabsDialog->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	#endif
}

Tools_SoundEventTab_Event::~Tools_SoundEventTab_Event()
{
	DeletePlayableList();
}

void Tools_SoundEventTab_Event::PerFrameUpdate()
{
 	//make sure the mins stay less than the maxes
	if(m_bMinChanged)
	{
		m_xEventData.m_fMaxVol = Asura_Maths::Max(m_xEventData.m_fMinVol,m_xEventData.m_fMaxVol);
		m_xEventData.m_fMaxPitch = Asura_Maths::Max(m_xEventData.m_fMinPitch,m_xEventData.m_fMaxPitch);
		m_xEventData.m_fMaxDelay = Asura_Maths::Max(m_xEventData.m_fMinDelay,m_xEventData.m_fMaxDelay);
		m_xEventData.m_fMaxDuration = Asura_Maths::Max(m_xEventData.m_fMinDuration,m_xEventData.m_fMaxDuration);
		m_xEventData.m_fOuterRadius = Asura_Maths::Max(m_xEventData.m_fInnerRadius,m_xEventData.m_fOuterRadius);
		m_xEventData.m_xVariableParams.m_fMaxInterval = Asura_Maths::Max(m_xEventData.m_xVariableParams.m_fMinInterval,m_xEventData.m_xVariableParams.m_fMaxInterval);
		m_xEventData.m_xVariableParams.m_fMaxVolumeChangeTime = Asura_Maths::Max(m_xEventData.m_xVariableParams.m_fMinVolumeChangeTime,m_xEventData.m_xVariableParams.m_fMaxVolumeChangeTime);
		m_xEventData.m_xVariableParams.m_fMaxPitchChangeTime = Asura_Maths::Max(m_xEventData.m_xVariableParams.m_fMinPitchChangeTime,m_xEventData.m_xVariableParams.m_fMaxPitchChangeTime);

		m_bMinChanged = false;
		m_bChanged = true;
	}
	
	if(m_bMaxChanged)
	{
		m_xEventData.m_fMinVol = Asura_Maths::Min(m_xEventData.m_fMinVol,m_xEventData.m_fMaxVol);
		m_xEventData.m_fMinPitch = Asura_Maths::Min(m_xEventData.m_fMinPitch,m_xEventData.m_fMaxPitch);
		m_xEventData.m_fMinDelay = Asura_Maths::Min(m_xEventData.m_fMinDelay,m_xEventData.m_fMaxDelay);
		m_xEventData.m_fMinDuration = Asura_Maths::Min(m_xEventData.m_fMinDuration,m_xEventData.m_fMaxDuration);
		m_xEventData.m_fInnerRadius = Asura_Maths::Min(m_xEventData.m_fInnerRadius,m_xEventData.m_fOuterRadius);
		m_xEventData.m_xVariableParams.m_fMinInterval = Asura_Maths::Min(m_xEventData.m_xVariableParams.m_fMinInterval,m_xEventData.m_xVariableParams.m_fMaxInterval);
		m_xEventData.m_xVariableParams.m_fMinVolumeChangeTime = Asura_Maths::Min(m_xEventData.m_xVariableParams.m_fMinVolumeChangeTime,m_xEventData.m_xVariableParams.m_fMaxVolumeChangeTime);
		m_xEventData.m_xVariableParams.m_fMinPitchChangeTime = Asura_Maths::Min(m_xEventData.m_xVariableParams.m_fMinPitchChangeTime,m_xEventData.m_xVariableParams.m_fMaxPitchChangeTime);

		m_bMaxChanged = false;
		m_bChanged = true;
	}

	if(m_bFileChanged)
	{
		//Make sure it is okay to transfer the event
		if(!Tools_SoundEvent_System::CanTransferEvent(m_pxEvent,m_xEventData.m_xFilename.GetString()))
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Can't transfer event since source and destination aren't both writeable");
		
			m_xEventData.m_xFilename = m_pxEvent->GetFilename();
			m_bFileChanged = false;
		}
		else
		{
			//Set modified before updating the asura data so that both the source and destination files are flagged
			SetModified();
			m_bChanged = true;
		}
		m_bFileChanged = false;
	}

	if(m_bCategoryChanged)
	{
		//If the category has changed from default , then change the mixer channel as well if there is a suitable one that matches
		if(!stricmp(m_pxEvent->GetEventType(),"default") && stricmp(m_xEventData.m_xCategory.GetString(),"default"))
		{
			Asura_Hash_ID uMixerHash = Asura_GetHashID(m_xEventData.m_xCategory.GetString());
			if(Asura_Sound_Mixer_System::FindChannel(uMixerHash))
			{
				m_xEventData.m_xMixer = m_xEventData.m_xCategory;
			}
		}
		m_bCategoryChanged = false;
		m_bChanged = true;
	}

	if ( m_bSoundTypeChanged )
	{
		m_xEventData.m_uSoundType = m_iSelectedSoundType;
		m_xEventData.CopyToAsuraEvent(m_pxEvent);
		m_bSoundTypeChanged = false;
		m_bChanged = true;
	}

	if(m_bChanged)
	{
		int iMaxSamplesPlayed = Tools_SoundEvent_System::GetMaxSamplesPlayed(m_pxEvent);
		
		//update the asura data
		m_xEventData.CopyToAsuraEvent(m_pxEvent);
		SetModified();

		//Warn if the change has resulted in the max samples played going over the limit
		int iNewMaxSamplesPlayed = Tools_SoundEvent_System::GetMaxSamplesPlayed(m_pxEvent);
		if(iNewMaxSamplesPlayed > iMaxSamplesPlayed && iNewMaxSamplesPlayed > uASURA_SOUND_EVENT_MAX_FOR_SIMULTANEOUS_PLAY)
		{
			ShowMessageOverlay("Warning.\nSound event '%s' may play up to %d simultaneous samples.\nThe max allowed is %d",GetName(),iNewMaxSamplesPlayed,uASURA_SOUND_EVENT_MAX_FOR_SIMULTANEOUS_PLAY);
		}

		m_bChanged = false;
	}
}

void Tools_SoundEventTab_Event::BuildPlayableList()
{
	DeletePlayableList();

	if(!m_xEventData.m_uNumPlayable)
	{
		//no events/samples added yet , so just put in a dummy entry for an empty list
		m_uPlayableListSize = 1;
		m_pszPlayableList = new char*[1];
		m_pszPlayableList[0] = new char[20];
		strcpy(m_pszPlayableList[0],"*Empty List*");
	}
	else
	{
		//get a list of all the event/sample names
		m_uPlayableListSize = m_xEventData.m_uNumPlayable;
		m_pszPlayableList = new char*[m_uPlayableListSize];

		for(u_int i=0;i<m_uPlayableListSize;i++)
		{
			const char* szPlayableName = 0;
			const char* szPrefix = 0;
			if(m_xEventData.m_pxPlayableID[i].m_eType == ASURA_SOUND_PLAYABLE_EVENT)
			{
				const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByHash(m_xEventData.m_pxPlayableID[i].m_uHashID);
				if(pxEvent) szPlayableName = pxEvent->GetName();
				szPrefix = "E: ";
			}
			else
			{
				Asura_Assert(m_xEventData.m_pxPlayableID[i].m_eType == ASURA_SOUND_PLAYABLE_SAMPLE,"");
				const Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::GetSampleByHash(m_xEventData.m_pxPlayableID[i].m_uHashID);
				if(pxSample) szPlayableName = pxSample->GetName();
				szPrefix = "S: ";
			}

			if(szPlayableName)
			{
				m_pszPlayableList[i] = new char[strlen(szPlayableName)+1+strlen(szPrefix)];
				sprintf(m_pszPlayableList[i],"%s%s",szPrefix,szPlayableName);
			}
			else
			{
				m_pszPlayableList[i] = new char[20];
				Asura_PrintfToBfr(m_pszPlayableList[i],"%s 0x%x",20,true,(m_xEventData.m_pxPlayableID[i].m_eType == ASURA_SOUND_PLAYABLE_EVENT) ? "Event" : "Sample",m_xEventData.m_pxPlayableID[i].m_uHashID);
			}
		}
	}

	m_iSelectedPlayable = Asura_Maths::ClampToRange(m_iSelectedPlayable,0,(int)m_uPlayableListSize);

	//update the dialog if it already exists
	if(m_pxPlayableListDialog)
	{
		m_pxPlayableListDialog->ReplaceStrings(m_uPlayableListSize,m_pszPlayableList);
	}
}

void Tools_SoundEventTab_Event::DeletePlayableList()
{
	for(u_int i=0;i<m_uPlayableListSize;i++)
	{
		delete [] m_pszPlayableList[i];
	}
	delete [] m_pszPlayableList;
	m_pszPlayableList = 0;
	m_uPlayableListSize = 0;
}

bool Tools_SoundEventTab_Event::CanAddEvent()
{
	return !m_xEventData.m_bEventMapping && (m_xEventData.m_iPlaybackType != ASURA_PT_VARIABLE);
}

void Tools_SoundEventTab_Event::AddEvent()
{
	if(!CanAddEvent()) return;

	//use the event picking overlay to get a new event

	Tools_PickSoundEvent_Overlay xOverlay("Pick events to add",true);
	xOverlay.SetFileFilter(m_pxEvent->GetFilename());
	if(!stricmp(m_pxEvent->GetEventType(),"Default"))
	{
		xOverlay.SetCategoryFilter("");
		xOverlay.SetSubCategoryFilter("");
	}
	else
	{
		xOverlay.SetCategoryFilter(m_pxEvent->GetEventType());
		xOverlay.SetSubCategoryFilter("");
	}

	//Setting m_bBeingEdited makes sure that this can't be deleted as a result of too many tabs being opened
	m_bBeingEdited = true;
	xOverlay.DoModal(0);
	m_bBeingEdited = false;

	u_int uNumSelected = xOverlay.GetNumSelected();
	if(!uNumSelected) return;

	//get the events to be added
	List<Asura_Hash_ID> xEventList;
	u_int uNumSkipped = 0;
	for(u_int i=0;i<uNumSelected;i++)
	{
		Asura_Hash_ID uEventHash = xOverlay.GetSelection(i);
		//make sure that adding these event won't result in the event including itself
		if(!IsOkayToAddEvent(m_uEventHash,uEventHash))
		{
			uNumSkipped++;
			continue;
		}

		xEventList.add_entry(uEventHash);
	}

	if(uNumSkipped)
	{
  		ShowMessageOverlay("Unable to add %d of the selected events , because it would result in an event including itself.",uNumSkipped);
	}

	if(!xEventList.size()) return;

	



	//add to the list of playables
	Asura_Sound_PlayableID* pxNewPlayableID = new Asura_Sound_PlayableID[m_xEventData.m_uNumPlayable+xEventList.size()];
	for(u_int i=0;i<m_xEventData.m_uNumPlayable;i++)
	{
		pxNewPlayableID[i] = m_xEventData.m_pxPlayableID[i];
	}
	for(int i=0;i<xEventList.size();i++)
	{
		pxNewPlayableID[m_xEventData.m_uNumPlayable+i].m_uHashID = xEventList[i];
		pxNewPlayableID[m_xEventData.m_uNumPlayable+i].m_eType = ASURA_SOUND_PLAYABLE_EVENT;
	}

	delete [] m_xEventData.m_pxPlayableID;
	m_xEventData.m_pxPlayableID = pxNewPlayableID;
	m_xEventData.m_uNumPlayable += (u_int)xEventList.size();

	//rebuild the overlay list , and select the new event
	BuildPlayableList();
	m_iSelectedPlayable = m_xEventData.m_uNumPlayable-1;

	//Force the asura event to be updated
	m_bChanged = true;
}

bool Tools_SoundEventTab_Event::IsOkayToAddEvent(Asura_Hash_ID uThisEvent,Asura_Hash_ID uEventToAdd)
{
	//make sure that adding this event won't result in the event including itself
	List<Asura_Hash_ID> xEventQueue;
	xEventQueue.add_entry(uEventToAdd);

	for(LIF<Asura_Hash_ID> xEventLIF(&xEventQueue);!xEventLIF.done();xEventLIF.next())
	{
		if(xEventLIF() == uThisEvent)
		{
			return false;
		}

		Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByHash(xEventLIF());
		if(!pxEvent) continue;

		for(u_int i=0;i<pxEvent->GetNumPlayableID();i++)
		{
			const Asura_Sound_PlayableID& xID = pxEvent->GetPlayableID(i);
			if(xID.m_eType != ASURA_SOUND_PLAYABLE_EVENT) continue;

			if(!xEventQueue.contains(xID.m_uHashID))
			{
				xEventQueue.add_entry(xID.m_uHashID);
			}
		}
	}
	return true;
}

bool Tools_SoundEventTab_Event::CanAddSample()
{
	return !m_xEventData.m_bEventMapping;
}

void Tools_SoundEventTab_Event::AddSample()
{
	if(!CanAddSample()) return;

	//use the sample picking overlay to get a new sample

	Tools_PickSoundSample_Overlay xOverlay("Pick samples to add",true);
	xOverlay.SetFileFilter(m_pxEvent->GetFilename());
	if(!stricmp(m_pxEvent->GetEventType(),"Default"))
	{
		xOverlay.SetCategoryFilter("");
		xOverlay.SetSubCategoryFilter("");
	}
	else
	{
		xOverlay.SetCategoryFilter(m_pxEvent->GetEventType());
		xOverlay.SetSubCategoryFilter("");
	}
	//Setting m_bBeingEdited makes sure that this can't be deleted as a result of too many tabs being opened
	m_bBeingEdited = true;
	xOverlay.DoModal(0);
	m_bBeingEdited = false;

	u_int uNumSelected = xOverlay.GetNumSelected();
	if(!uNumSelected) return;


	//add to the list of playable hashes
	Asura_Sound_PlayableID* pxNewPlayableID = new Asura_Sound_PlayableID[m_xEventData.m_uNumPlayable+uNumSelected];
	for(u_int i=0;i<m_xEventData.m_uNumPlayable;i++)
	{
		pxNewPlayableID[i] = m_xEventData.m_pxPlayableID[i];
	}
	
	for(u_int i=0;i<uNumSelected;i++)
	{
		pxNewPlayableID[m_xEventData.m_uNumPlayable+i].m_uHashID = xOverlay.GetSelection(i);
		pxNewPlayableID[m_xEventData.m_uNumPlayable+i].m_eType = ASURA_SOUND_PLAYABLE_SAMPLE;
	}

	delete [] m_xEventData.m_pxPlayableID;
	m_xEventData.m_pxPlayableID = pxNewPlayableID;
	m_xEventData.m_uNumPlayable += uNumSelected;

	//rebuild the overlay list , and select the new sample
	BuildPlayableList();
	m_iSelectedPlayable = m_xEventData.m_uNumPlayable-1;

	//Force the asura event to be updated
	m_bChanged = true;
}

bool Tools_SoundEventTab_Event::CanRemovePlayable()
{
	return m_xEventData.m_uNumPlayable!=0 && !m_xEventData.m_bEventMapping;
}

void Tools_SoundEventTab_Event::RemovePlayable()
{
	if(!CanRemovePlayable()) return;

	m_xEventData.m_uNumPlayable--;
	for(u_int i=m_iSelectedPlayable;i<m_xEventData.m_uNumPlayable;i++)
	{
		m_xEventData.m_pxPlayableID[i] = m_xEventData.m_pxPlayableID[i+1];
	}

	//rebuild the overlay list 
	BuildPlayableList();

	//Force the asura event to be updated
	m_bChanged = true;
}

bool Tools_SoundEventTab_Event::CanMoveUp()
{
	return (m_iSelectedPlayable > 0);
}

void Tools_SoundEventTab_Event::MoveUp()
{
	if(!CanMoveUp()) return;

	//swap entries in the hash list
	Asura_Sound_PlayableID xTemp = m_xEventData.m_pxPlayableID[m_iSelectedPlayable-1];
	m_xEventData.m_pxPlayableID[m_iSelectedPlayable-1] = m_xEventData.m_pxPlayableID[m_iSelectedPlayable];
	m_xEventData.m_pxPlayableID[m_iSelectedPlayable] = xTemp;

	m_iSelectedPlayable--;

	//rebuild the overlay list 
	BuildPlayableList();

	//Force the asura event to be updated
	m_bChanged = true;
}

bool Tools_SoundEventTab_Event::CanMoveDown()
{
	return m_iSelectedPlayable+1 < (int)m_xEventData.m_uNumPlayable;
}

void Tools_SoundEventTab_Event::MoveDown()
{
	if(!CanMoveDown()) return;

	//swap entries in the hash list
	Asura_Sound_PlayableID xTemp = m_xEventData.m_pxPlayableID[m_iSelectedPlayable+1];
	m_xEventData.m_pxPlayableID[m_iSelectedPlayable+1] = m_xEventData.m_pxPlayableID[m_iSelectedPlayable];
	m_xEventData.m_pxPlayableID[m_iSelectedPlayable] = xTemp;

	m_iSelectedPlayable++;

	//rebuild the overlay list 
	BuildPlayableList();

	//Force the asura event to be updated
	m_bChanged = true;
}

bool Tools_SoundEventTab_Event::CanSort()
{
	return m_xEventData.m_uNumPlayable >= 2;
}

static int SortPlayableCallback(const void* p1,const void* p2)
{
	Asura_Sound_PlayableID* pxPlayable1 = (Asura_Sound_PlayableID*)p1;
	Asura_Sound_PlayableID* pxPlayable2 = (Asura_Sound_PlayableID*)p2;

	const char* szName1 = 0;
	if(pxPlayable1->m_eType == ASURA_SOUND_PLAYABLE_EVENT)
	{
		const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByHash(pxPlayable1->m_uHashID);
		if(pxEvent) szName1 = pxEvent->GetName();
	}
	else
	{
		const Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::GetSampleByHash(pxPlayable1->m_uHashID);
		if(pxSample) szName1 = pxSample->GetName();
	}

	const char* szName2 = 0;
	if(pxPlayable2->m_eType == ASURA_SOUND_PLAYABLE_EVENT)
	{
		const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByHash(pxPlayable2->m_uHashID);
		if(pxEvent) szName2 = pxEvent->GetName();
	}
	else
	{
		const Asura_Sound_Sample* pxSample = Asura_Sound_Event_Storage::GetSampleByHash(pxPlayable2->m_uHashID);
		if(pxSample) szName2 = pxSample->GetName();
	}

	if(szName1 && szName2)
	{
		return stricmp(szName1,szName2);
	}
	else if(szName1)
	{
		return -1;
	}
	else if(szName2)
	{
		return 1;
	}
	else
	{
		return 0;
	}

}

void Tools_SoundEventTab_Event::Sort()
{
	qsort(m_xEventData.m_pxPlayableID,m_xEventData.m_uNumPlayable,sizeof(Asura_Sound_PlayableID),SortPlayableCallback);

	//rebuild the overlay list 
	BuildPlayableList();

	//Force the asura event to be updated
	m_bChanged = true;
}


bool Tools_SoundEventTab_Event::CanEditPlayable()
{
	if(!m_xEventData.m_uNumPlayable) return false;

	if(m_xEventData.m_pxPlayableID[m_iSelectedPlayable].m_eType == ASURA_SOUND_PLAYABLE_EVENT)
	{
		return Asura_Sound_Event_Storage::GetEventByHash(m_xEventData.m_pxPlayableID[m_iSelectedPlayable].m_uHashID)!=0;
	}
	else
	{
		return Asura_Sound_Event_Storage::GetSampleByHash(m_xEventData.m_pxPlayableID[m_iSelectedPlayable].m_uHashID)!=0;
	}
}

void Tools_SoundEventTab_Event::EditPlayable()
{
	if(!CanEditPlayable()) return;

	if(m_xEventData.m_pxPlayableID[m_iSelectedPlayable].m_eType == ASURA_SOUND_PLAYABLE_EVENT)
	{
		Tools_SoundEvent_Overlay::EditEvent(m_xEventData.m_pxPlayableID[m_iSelectedPlayable].m_uHashID);
	}
	else
	{
		Tools_SoundEvent_Overlay::EditSample(m_xEventData.m_pxPlayableID[m_iSelectedPlayable].m_uHashID);
	}
}

void Tools_SoundEventTab_Event::SetModified()
{
	Tools_SoundEvent_System::SetModified(m_pxEvent);
}

void Tools_SoundEventTab_Event::PlaySound()
{
	Tools_SoundEvent_Overlay::PlaySoundEvent(m_uEventHash);
}

void Tools_SoundEventTab_Event::StopSound()
{
	Tools_SoundEvent_Overlay::StopSound();
}

const char* Tools_SoundEventTab_Event::GetName() const
{
	return m_pxEvent->GetName();
}

void Tools_SoundEventTab_Event::UpdateFromAsuraData()
{
	m_xEventData.CopyFromAsuraEvent(m_pxEvent);
	BuildPlayableList();
}

bool Tools_SoundEventTab_Event::HandleReload()
{
	m_pxEvent = Asura_Sound_Event_Storage::GetEventByHash(m_uEventHash);
	if(m_pxEvent)
	{
		UpdateFromAsuraData();
		return true;
	}
	return false;
}

bool Tools_SoundEventTab_Event::Delete()
{
	return Tools_SoundEvent_System::DeleteEvent(m_uEventHash,true);
}

void Tools_SoundEventTab_Event::Copy()
{
	s_xPasteBuffer = m_xEventData;
	s_bPasteBufferValid = true;
	Tools_Timed_Message_System::AddMessage(0xffffff,"%s copied.",m_pxEvent->GetName());
}

void Tools_SoundEventTab_Event::Paste()
{
	if(s_bPasteBufferValid)
	{
		//make sure that pasting over this event won't cause it to include itself
		for(u_int i=0;i<s_xPasteBuffer.m_uNumPlayable;i++)
		{
			if(s_xPasteBuffer.m_pxPlayableID[i].m_eType == ASURA_SOUND_PLAYABLE_EVENT)
			{
				if(!IsOkayToAddEvent(m_uEventHash,s_xPasteBuffer.m_pxPlayableID[i].m_uHashID))
				{
					ShowMessageOverlay("Unable to paste over this event , because it would result in an event including itself.");
					return;
				}
			}
		}
		

		
		m_xEventData = s_xPasteBuffer;
		BuildPlayableList();
		m_bFileChanged = true;
		m_bChanged = true;
		Tools_Timed_Message_System::AddMessage(0xffffff,"Event pasted.");
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Event paste buffer is empty.");
	}
}


void Tools_SoundEventTab_Event::PasteMultiple()
{

	//Get a list of events to paste over
	Tools_PickSoundEvent_Overlay xOverlay("Paste over which events?",true);
	xOverlay.DoModal(0);

	u_int uNumSelected = xOverlay.GetNumSelected();
	if(!uNumSelected) return;

	//Determine which properties should be pasted
	Tools_SoundEventProperty_Overlay xPropertyOverlay;
	if(!xPropertyOverlay.DoModal()) return;

	u_int uMessageLogPos = Tools_MessageLog::GetNextID();
	
	//Do the copying
	for(u_int i=0;i<uNumSelected;i++)
	{
		Asura_Hash_ID uEventHash = xOverlay.GetSelection(i);
		Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByHash(uEventHash);
		if(!pxEvent) continue;

		//get a tools copy of the event properties
		Tools_SoundEvent xToolsSoundEvent;
		xToolsSoundEvent.CopyFromAsuraEvent(pxEvent);

		//set modified before the copy in case the file is being changed
		Tools_SoundEvent_System::SetModified(pxEvent);

		//Do the copy
		xToolsSoundEvent.Copy(pxEvent,m_xEventData,xPropertyOverlay.GetSelectedArray());

		//Update the asura event and flag as modified
		xToolsSoundEvent.CopyToAsuraEvent(pxEvent);
		Tools_SoundEvent_System::SetModified(pxEvent);

		//Make sure any tab with this event is updated
		Tools_SoundEvent_Overlay::UpdateEventFromAsuraData(uEventHash);

	}

	//If there were any errors , open the message log
	if(Tools_MessageLog::AreThereAnyMessages(Tools_MessageLog::SOUND_EVENT, 0, uMessageLogPos))
	{
		Tools_MessageLog::ShowOverlay(Tools_MessageLog::SOUND_EVENT,"Paste Error",uMessageLogPos);
	}

}


void Tools_SoundEventTab_Event::Duplicate()
{
	//get a default name for the event
	//get current sample name without any numbers at the end
	char szBaseName[100];
	Asura_PrintfToBfr(szBaseName,"%s",sizeof(szBaseName),true,m_pxEvent->GetName());
	int iLength = strlen(szBaseName);
	while(iLength>0 && szBaseName[iLength-1]>='0' && szBaseName[iLength-1]<='9') iLength--;
	szBaseName[iLength] = 0;
	

	char szEventName[200];
	for(int iIndex=0;;iIndex++)
	{
		Asura_PrintfToBfr(szEventName,"%s%d",200,false,szBaseName,iIndex);

		if(!Asura_Sound_Event_Storage::GetEventByHash(Asura_GetHashID(szEventName))) break;
	}

	
	
	//ask for a name  , offering the default name
	GetString_Overlay xStringOverlay("Name new event",szEventName);
	do
	{
		if(!xStringOverlay.DoModal()) break;
		
		const char* szEventName = xStringOverlay.GetString();
		if(!szEventName || !*szEventName) break;

		//make sure this sample doesn't already exist
		Asura_Hash_ID uEventHash = Asura_GetHashID(szEventName);
		if(Asura_Sound_Event_Storage::GetEventByHash(uEventHash))
		{
			ShowMessageOverlay("An event already exists using the same hash ID.");
			continue;
		}

		bool bAlreadyExists;
		Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::CreateEvent(szEventName, bAlreadyExists);
		Asura_Assert(pxEvent && !bAlreadyExists, "Error creating sound event");

		//copy this sample's properties to the new one
		m_xEventData.CopyToAsuraEvent(pxEvent);
		Tools_SoundEvent_System::SetModified(pxEvent);
		Tools_SoundEvent_System::RegisterNewEvent(pxEvent);
		//and edit it
		Tools_SoundEvent_Overlay::EditEvent(uEventHash);
		break;
		
	}while(true);
}

void Tools_SoundEventTab_Event::Rename()
{
	const char* szNewName = 0;
	Asura_Hash_ID uOldHash = m_uEventHash;
	Asura_Hash_ID uNewHash = 0;
	bool bOkay = false;
	
	//ask for new name
	GetString_Overlay xStringOverlay("Rename Event",m_pxEvent->GetName());
	while(xStringOverlay.DoModal())
	{
		szNewName = xStringOverlay.GetString();
		if(!szNewName || !*szNewName) return;

		if(!strcmp(szNewName,m_pxEvent->GetName()))
		{
			//name not changed
			return;
		}

		//make sure no existing sample exists with this hash
		uNewHash = Asura_GetHashID(szNewName);

		if(uOldHash != uNewHash && Asura_Sound_Event_Storage::GetEventByHash(uNewHash))
		{
			ShowMessageOverlay("An event already exists using the same hash ID.");
			continue;
		}

		bOkay = true;
		break;
	}
	if(!bOkay) return;

	//Update the sample and its position in the storage system
	if(!Asura_Sound_Event_Storage::RenameEvent(m_pxEvent->GetName(),szNewName))
	{
		Asura_Assert(0,"Error renaming event");
		return;
	}
	SetModified();

	//update this tab
	m_uEventHash = uNewHash;
	UpdateTabName();

	//Update any events using this event
	Asura_Sound_PlayableID xOldID,xNewID;
	xOldID.m_uHashID = uOldHash;
	xOldID.m_eType = ASURA_SOUND_PLAYABLE_EVENT;
	xNewID.m_uHashID = uNewHash;
	xNewID.m_eType = ASURA_SOUND_PLAYABLE_EVENT;
	for(u_int i=0;i<Asura_Sound_Event_Storage::GetNumberOfEvents();i++)
	{
		Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::GetEventByIndex(i);
		if(pxEvent->ReplacePlayableID(xOldID,xNewID))
		{
			Tools_SoundEvent_System::SetModified(pxEvent);

			//Update the tab for this event if it was being edited
			Tools_SoundEvent_Overlay::UpdateEventFromAsuraData(pxEvent->GetHashID());
		}
	}

	//Update phonons
	Tools_Phonon_Object::SoundEventRenamed(uOldHash,uNewHash);
	Asura_Phonon_System::SoundEventRenamed(uOldHash,uNewHash);

}

void Tools_SoundEventTab_Event::UpdateTabName()
{
	char szTabName[MAX_TAB_NAME_LENGTH+2];
	Asura_PrintfToBfr(szTabName,"E: %s",sizeof(szTabName),true,m_pxEvent->GetName());
	if(strlen(szTabName) > MAX_TAB_NAME_LENGTH)
	{
		//name is too long for tab , so truncate it
		strcpy(&szTabName[MAX_TAB_NAME_LENGTH-3],"...");
	}

	m_xTabName = szTabName;
}


void Tools_SoundEventTab_Event::CreateMapping()
{
	//Get the default event for the mapping
	Asura_Hash_ID uEventHash = 0;
	if(m_xEventData.m_uNumPlayable)
	{
		if(m_xEventData.m_pxPlayableID[0].m_eType == ASURA_SOUND_PLAYABLE_EVENT)
		{
			uEventHash = m_xEventData.m_pxPlayableID[0].m_uHashID;
		} 
	}

	//Ask for the event to use
	Tools_PickSoundEvent_Overlay xOverlay("Map to which event?");
	uEventHash = xOverlay.DoModal(uEventHash);
	if(xOverlay.WasCancelled()) return;
	if(!uEventHash) return;

	//Make sure its safe to add the event
	if(uEventHash)
	{
		if(!IsOkayToAddEvent(m_uEventHash,uEventHash))
		{
	  		ShowMessageOverlay("Unable to add event , because it would result in an event including itself.");
			return;
		}
	}

	//Clear out the playable list , and then replace with the single event
	delete [] m_xEventData.m_pxPlayableID;
	
	m_xEventData.m_uNumPlayable = 1;
	m_xEventData.m_pxPlayableID = new Asura_Sound_PlayableID[1];
	m_xEventData.m_pxPlayableID[0].m_eType = ASURA_SOUND_PLAYABLE_EVENT;
	m_xEventData.m_pxPlayableID[0].m_uHashID = uEventHash;
	
	m_xEventData.m_bEventMapping = true;

	//rebuild the overlay list
	BuildPlayableList();
	m_iSelectedPlayable = 0;

	//Force the asura event to be updated
	m_bChanged = true;
		
}

bool Tools_SoundEventTab_Event::CanEdit3DEffects()
{
	// Spatialised effects like occlusion, distance filter etc. don't make sense for direct sounds

	return m_xEventData.m_i3dSoundType != ASURA_SOUND_DIRECT
		&& m_xEventData.m_i3dSoundType != ASURA_SOUND_RANDOMPOS
		&& IsNotMapping();
}

bool Tools_SoundEventTab_Event::IsNotMapping()
{
	return !m_xEventData.m_bEventMapping;
}

bool Tools_SoundEventTab_Event::IsEnabled_EventMapping()
{
	//Eventmapping tickbox can be changed if its currently on , or if its off and the event is a legal mapping
	if(m_xEventData.m_bEventMapping) return true;
	return m_xEventData.CanBeEventMapping();
}

bool Tools_SoundEventTab_Event::IsEnabled_VariableParam()
{
	//must be a variable playback event , and not an event mapping
	return !m_xEventData.m_bEventMapping && (m_xEventData.m_iPlaybackType == ASURA_PT_VARIABLE);
}

void Tools_SoundEventTab_Event::ReloadWavs()
{
	Tools_SoundEventTab_Event::StopSound();
	Tools_SoundEvent_System::ReloadWavsForEvent(m_uEventHash, true);
}

//eof
