//
// Tools_SoundEventTab_Event.h
//
// Created by Richard Rice - 07/07/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_SOUNDEVENTTAB_EVENT__
#define __TOOLS_SOUNDEVENTTAB_EVENT__

#include "Sound/Tools_SoundEvent_Overlay.h"

#include "Sound/Asura_Sound_Event_System.h"

//Stores a copy of most things from Asura_Sound_Event for editing by the overlay
class Tools_SoundEvent
{
public :
	Tools_SoundEvent();
	~Tools_SoundEvent();

	void CopyFromAsuraEvent(const class Asura_Sound_Event* pxEvent);
	void CopyToAsuraEvent(class Asura_Sound_Event* pxEvent) const;


	Tools_SoundEvent& operator = (const Tools_SoundEvent& xEvent);
	void Copy(const class Asura_Sound_Event* pxThisEvent,const Tools_SoundEvent& xEvent,const bool* pbProperties);

	bool CanBeEventMapping()  const;


	float 								m_fMinVol;		// pick random vol in range [min,max]
	float 								m_fMaxVol;
	float 								m_fMinPitch;	// pick random pitch in range [min,max]
	float 								m_fMaxPitch;
	float 								m_fMinDelay;	// pick random start delay in range [min,max]
	float 								m_fMaxDelay;
	float 								m_fMinDuration;
	float 								m_fMaxDuration;
	Asura_SurroundPan					m_xSurroundPan;
	int									m_iPlayProbability;	// the probability that this event will play at all when triggered. (0-100, defaults to 100).
	float								m_fPriority;	// 0.0f to 2.0f, 1.0f is default
	int									m_iPlaybackType;
	bool								m_bLooping;
	bool 								m_bUseOcclusion;
	bool 								m_bUseDistanceFilter;
	bool 								m_bUseDoppler;
	bool 								m_bUseEnvironmentalReverb;
	bool								m_bUseMasterDSP;
	bool								m_bPanOverride;
	bool								m_bGlobalSound;
	bool								m_bEventMapping;
	
	// 3d data
	int									m_i3dSoundType;
	float 								m_fInnerRadius;
	float 								m_fOuterRadius;

	u_int								m_uMaxInstances;
	u_int								m_uMinFreeVoices;

	//variable playback settings
	Asura_Sound_VariableParams			m_xVariableParams;

	//playable list
	u_int								m_uNumPlayable;
	Asura_Sound_PlayableID*				m_pxPlayableID;

	Asura_String						m_xCategory;
	Asura_String						m_xSubCategory;
	Asura_String						m_xFilename;
	Asura_String						m_xMixer;
	Asura_String						m_xInstanceGroupName;

	Asura_Hash_ID						m_uSoundType;

	Asura_Hash_ID						m_uDSPChainHash;

};

////////////////////////////////////////////////////////////////////////////////////////////

//for handling a single event editing tab
class Tools_SoundEventTab_Event : public Tools_SoundEventTab
{
public :
	Tools_SoundEventTab_Event(class Asura_Tabs_Dialog* pxTabsDialog,Asura_Hash_ID uEventHash);
	~Tools_SoundEventTab_Event();

	virtual void PerFrameUpdate();
	virtual Asura_Hash_ID GetHashID() {return m_uEventHash;}
	virtual bool IsEvent(){return true;}
	virtual const char* GetName() const;
	virtual const char* GetTabName() const {return m_xTabName.GetString();}
	virtual void UpdateFromAsuraData();
	virtual bool HandleReload();
	virtual bool Delete();
	virtual void Copy();
	virtual void Paste();
	virtual void PasteMultiple();
	virtual void Duplicate();
	virtual void Rename();
	virtual bool CanBeClosed() const {return !m_bBeingEdited;}

	static bool IsOkayToAddEvent(Asura_Hash_ID uThisEvent,Asura_Hash_ID uEventToAdd);
private :
	void SetModified();

	void AddOverlayControls(class Asura_Tabs_Dialog* pxTabsDialog);

	void BuildPlayableList();
	void DeletePlayableList();

	
	void UpdateTabName();

	
	//overlay callback functions
	//defines for routeing static callback functions to non-static versions
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_SoundEventTab_Event*)iThis)->__FnName();}\
	void __FnName();

	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_SoundEventTab_Event*)iThis)->__FnName();}\
	bool __FnName();

	OVERLAY_VOID_CALLBACK(AddEvent)
	OVERLAY_VOID_CALLBACK(AddSample)
	OVERLAY_VOID_CALLBACK(RemovePlayable)
	OVERLAY_VOID_CALLBACK(MoveUp)
	OVERLAY_VOID_CALLBACK(MoveDown)
	OVERLAY_VOID_CALLBACK(Sort)
	OVERLAY_VOID_CALLBACK(EditPlayable)
	OVERLAY_VOID_CALLBACK(PlaySound)
	OVERLAY_VOID_CALLBACK(StopSound)
	OVERLAY_VOID_CALLBACK(ReloadWavs)
	OVERLAY_VOID_CALLBACK(CreateMapping)


	OVERLAY_BOOL_CALLBACK(CanAddEvent)
	OVERLAY_BOOL_CALLBACK(CanAddSample)
	OVERLAY_BOOL_CALLBACK(CanRemovePlayable)
	OVERLAY_BOOL_CALLBACK(CanMoveUp)
	OVERLAY_BOOL_CALLBACK(CanMoveDown)
	OVERLAY_BOOL_CALLBACK(CanSort)
	OVERLAY_BOOL_CALLBACK(CanEditPlayable)

	OVERLAY_BOOL_CALLBACK(CanEdit3DEffects);

	OVERLAY_BOOL_CALLBACK(IsNotMapping)
	OVERLAY_BOOL_CALLBACK(IsEnabled_EventMapping)

	OVERLAY_BOOL_CALLBACK(IsEnabled_VariableParam)

	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK

	/////////////

	Asura_Hash_ID m_uEventHash;
	class Asura_Sound_Event* m_pxEvent;

	Tools_SoundEvent m_xEventData;

	int m_iSelectedPlayable;
	u_int m_uPlayableListSize;
	char** m_pszPlayableList;
	class Asura_List_Dialog* m_pxPlayableListDialog;

	bool m_bMinChanged;
	bool m_bMaxChanged;
	bool m_bFileChanged;
	bool m_bCategoryChanged;
	bool m_bChanged;
	bool m_bSoundTypeChanged;

	int m_iSelectedSoundType;

	bool m_bBeingEdited;

	Asura_String m_xTabName;

	static bool s_bPasteBufferValid;
	static Tools_SoundEvent s_xPasteBuffer;
};

#endif //__TOOLS_SOUNDEVENTTAB_EVENT__
