//
// Tools_SoundEventTab_Sample.h
//
// Created by Richard Rice - 07/07/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_SOUNDEVENTTAB_SAMPLE__
#define __TOOLS_SOUNDEVENTTAB_SAMPLE__

#include "Misc/Tools_Platform.h"
#include "Sound/Tools_SoundEvent_Overlay.h"

//Stores a copy of most things from Asura_Sound_Sample for editing by the overlay
class Tools_SoundSample
{
public :
	void CopyFromAsuraSample(const class Asura_Sound_Sample* pxSample);
	void CopyToAsuraSample(class Asura_Sound_Sample* pxSample) const;

	Tools_SoundSample& operator = (const Tools_SoundSample& xSample);
	void Copy(const class Asura_Sound_Sample* pxThisSample,const Tools_SoundSample& xSample,const bool* pbProperties);


#ifdef ASURA_USE_AUDIO_SYSTEM
	Asura_Hash_ID m_uWave;
#else
	Asura_Sound_ID m_eSoundID;
#endif

	float m_fVolume;
	float m_fPitch;
	float m_fDuration;
	bool m_bLooping;
	
	bool m_abUseSampleRate[TOOLS_PLATFORM_NUM_PLATFORMS];
	int m_aiSampleRate[TOOLS_PLATFORM_NUM_PLATFORMS];

	bool m_abUseCompressionQuality[TOOLS_PLATFORM_NUM_PLATFORMS];
	int m_aiCompressionQuality[TOOLS_PLATFORM_NUM_PLATFORMS];

	Asura_String						m_xCategory;
	Asura_String						m_xSubCategory;
	Asura_String						m_xFilename;

};


////////////////////////////////////////////////////////////////////////////////////////////

//for handling a single sample editing tab
class Tools_SoundEventTab_Sample : public Tools_SoundEventTab
{
public :
	Tools_SoundEventTab_Sample(class Asura_Tabs_Dialog* pxTabsDialog,Asura_Hash_ID uSampleHash);
	//~Tools_SoundEventTab_Sample();

	virtual void PerFrameUpdate();
	virtual Asura_Hash_ID GetHashID() {return m_uSampleHash;}
	virtual bool IsSample(){return true;}
	virtual const char* GetName() const;
	virtual const char* GetTabName() const {return m_xTabName.GetString();}
	virtual void UpdateFromAsuraData();
	virtual bool HandleReload();
	virtual bool Delete();
	virtual void Copy();
	virtual void Paste();
	virtual void PasteMultiple();
	virtual void Duplicate();
	virtual void Rename();

private :
	void SetModified();

	void AddOverlayControls(class Asura_Tabs_Dialog* pxTabsDialog);
	void UpdateWavDialog();
	void UpdateWavDataStringDialog();

	void UpdateTabName();


	//overlay callback functions
	//defines for routeing static callback functions to non-static versions
	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_SoundEventTab_Sample*)iThis)->__FnName();}\
	void __FnName();

	#define OVERLAY_BOOL_CALLBACK(__FnName)\
	static bool __FnName(int iThis){return ((Tools_SoundEventTab_Sample*)iThis)->__FnName();}\
	bool __FnName();

	OVERLAY_VOID_CALLBACK(SelectWav)
	OVERLAY_VOID_CALLBACK(PlaySound)
	OVERLAY_VOID_CALLBACK(StopSound)
	OVERLAY_VOID_CALLBACK(ReloadWav)
	OVERLAY_VOID_CALLBACK(CreateEventFromSample)
	OVERLAY_VOID_CALLBACK(FindReferencesToWav)

	OVERLAY_VOID_CALLBACK(PreviewOriginal)
	OVERLAY_VOID_CALLBACK(PreviewPS3)
	OVERLAY_VOID_CALLBACK(PreviewXBox360)


	#undef OVERLAY_VOID_CALLBACK
	#undef OVERLAY_BOOL_CALLBACK

	void PreviewCompression_Platform(int iPlatform);
	void PreviewCompression_PlatformAndQuality(int iPlatformAndQuality);
	void PreviewCompression(int iPlatform, int iQuality);

	static bool IsEnabled_SampleRate(int iData){return *(bool*)iData;}
	static bool IsEnabled_CompressionQuality(int iData){return *(bool*)iData;}

	/////////////

	Asura_Hash_ID m_uSampleHash;
	class Asura_Sound_Sample* m_pxSample;

	Tools_SoundSample m_xSampleData;

	class Asura_Function_Dialog* m_pxWavDialog;	
	class Asura_StringDisplay_Dialog* m_pxWavDataStringDialog;

	bool m_bChanged;
	bool m_bFileChanged;

	Asura_String m_xTabName;		

	static bool s_bPasteBufferValid;
	static Tools_SoundSample s_xPasteBuffer;
};

#endif //__TOOLS_SOUNDEVENTTAB_SAMPLE__
