//
// Tools_SoundEvent_Overlay.h
//
// Created by Richard Rice - 05/07/2006
// 
// Copyright (C) 2006 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_SOUNDEVENT_OVERLAY__
#define __TOOLS_SOUNDEVENT_OVERLAY__

#include "Core/Asura_Hash.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Sound/Asura_Sound_Event.h"

#include "Misc/list_tem.hpp"

class Tools_SoundEventTab;
class Tools_SoundSamplePreview;

class Tools_SoundEvent_Overlay : public Asura_Overlay
{
friend class Tools_SoundEvent_System;

private :
	Tools_SoundEvent_Overlay(int iIndex);
	~Tools_SoundEvent_Overlay();

public :
	//Opens tab for editing event/sample
	static bool EditEvent(Asura_Hash_ID uEventHash);
	static bool EditSample(Asura_Hash_ID uSampleHash);

	//Close tab for event/sample. Returns false if not allowed to close it
	static bool CloseEventTab(Asura_Hash_ID uEventHash);
	static bool CloseSampleTab(Asura_Hash_ID uSampleHash);

	//Closes overlay if open
	static void CloseOverlay();

	//Functions used by the tabs
	static void PlaySoundEvent(Asura_Hash_ID uEventHash);
	static void PlaySoundSample(Asura_Hash_ID uSampleHash);
	static void StopSound();
	static void UpdateEventFromAsuraData(Asura_Hash_ID uEventHash);
	static void UpdateSampleFromAsuraData(Asura_Hash_ID uSampleHash);

	//Called after reloading sound events/samples
	//Causes each tab to find its event/sample again
	static void HandleReload();


private :
	void PerFrameUpdate(void);

	void ShowOverlay();
	void EditEventInternal(Asura_Hash_ID uEventHash);
	void EditSampleInternal(Asura_Hash_ID uEventHash);

	void CloseTabsIfTooLarge(u_int uTabToPreserve1,u_int uTabToPreserve2);

	static Tools_SoundEvent_Overlay* GetEventOverlay(bool bCreate);
	static Tools_SoundEvent_Overlay* GetSampleOverlay(bool bCreate);

	//loads/saves data for this mode
	static void TransferModeData(int iMode);

	static void UpdateSound();

	#define OVERLAY_VOID_CALLBACK(__FnName)\
	static void __FnName(int iThis){((Tools_SoundEvent_Overlay*)iThis)->__FnName();}\
	void __FnName();


	OVERLAY_VOID_CALLBACK(CloseTab);
	OVERLAY_VOID_CALLBACK(Delete);
	OVERLAY_VOID_CALLBACK(Copy);
	OVERLAY_VOID_CALLBACK(Paste);
	OVERLAY_VOID_CALLBACK(PasteMultiple);
	OVERLAY_VOID_CALLBACK(Duplicate);
	OVERLAY_VOID_CALLBACK(Rename);
	OVERLAY_VOID_CALLBACK(CreateEvent);
	OVERLAY_VOID_CALLBACK(CreateCodeEvent);
	OVERLAY_VOID_CALLBACK(CreateSample);

	#undef OVERLAY_VOID_CALLBACK

	bool IsSoundEventTab();

	void CloseTabMain(u_int uTab);

	static Asura_Vector_2 s_axOverlayPosition[2];
	static Tools_SoundEvent_Overlay* s_apxThis[2];
	static bool s_bCombineEventAndSampleOverlay;

	class Asura_Tabs_Dialog* m_pxTabsDialog;
	int m_iOverlayIndex;

	static Asura_Handle s_iEventSoundHandle;
	static Tools_SoundSamplePreview s_xSoundSamplePreview;

	//list of tabs on the overlay
	List<Tools_SoundEventTab*> m_xTabList;
};

////////////////////////////////////////////////////////////////////////////////////////////

//base class for the sample and event editing tabs
class Tools_SoundEventTab
{
public :
	virtual ~Tools_SoundEventTab(){};
	virtual void PerFrameUpdate(){};

	virtual Asura_Hash_ID GetHashID() = 0;
	virtual bool IsEvent(){return false;}
	virtual bool IsSample(){return false;}

	virtual const char* GetName() const = 0;
	virtual const char* GetTabName() const = 0;

	//A request to delete the asura data associated with the tab (but not the tab itself)
	virtual bool Delete() {return false;}

	//This gets called to test which tabs can be safely closed when there are too many open
	virtual bool CanBeClosed() const {return true;}

	//updates data in tab from the asura data
	virtual void UpdateFromAsuraData() {};

	//Gets the tab to find its sound eent/sample again
	//Returns false if it fails
	virtual bool HandleReload(){return false;}

	virtual void Copy(){};
	virtual void Paste(){};
	virtual void PasteMultiple(){};
	virtual void Duplicate(){};
	virtual void Rename(){};

protected :
	enum
	{
		MAX_TAB_NAME_LENGTH = 25
	};
};


////////////////////////////////////////////////////////////////////////////////////////////

//drop down list for selecting event/sample category
class Tools_SoundCategory_Dialog : public Asura_DropDown_List_Dialog
{
public :
	Tools_SoundCategory_Dialog(const Asura_Vector_2& xOffset,Asura_String* pxCategory,bool* pbModified);
	~Tools_SoundCategory_Dialog();


private:
	void PerFrameUpdate();

	static void OnSelectionChanged(int iThis){((Tools_SoundCategory_Dialog*)iThis)->OnSelectionChanged();}
	void OnSelectionChanged();

	void UpdateCategoryList();
	void DeleteCategoryList();

	Asura_String* m_pxCategory;
	bool* m_pbModified;

	Asura_String m_xCategoryBackup;
	bool m_bSelectionChanged;


	int m_iSelectedIndex;
	bool m_bInitialised;

	u_int m_uCategoryListSize;
	const char** m_pszCategoryList;
	class Asura_NameTree* m_pxCategoryNameTree;

	//for determining validity of current list
	int m_iListValidID;
	static int s_iListValidID;

};

////////////////////////////////////////////////////////////////////////////////////////////
//drop down list for selecting event/sample sub category
class Tools_SoundSubCategory_Dialog : public Asura_DropDown_List_Dialog
{
public :
	Tools_SoundSubCategory_Dialog(const Asura_Vector_2& xOffset,const Asura_String* pxCategory,Asura_String* pxSubCategory,bool* pbModified);
	~Tools_SoundSubCategory_Dialog();


private:
	void PerFrameUpdate();

	static void OnSelectionChanged(int iThis){((Tools_SoundSubCategory_Dialog*)iThis)->OnSelectionChanged();}
	void OnSelectionChanged();

	void UpdateSubCategoryList();
	void DeleteSubCategoryList();

	const Asura_String* m_pxCategory;
	Asura_String* m_pxSubCategory;
	bool* m_pbModified;

	Asura_String m_xCategoryBackup;
	Asura_String m_xSubCategoryBackup;
	bool m_bSelectionChanged;


	int m_iSelectedIndex;
	bool m_bInitialised;

	u_int m_uSubCategoryListSize;
	const char** m_pszSubCategoryList;
	class Asura_NameTree* m_pxSubCategoryNameTree;

	//for determining validity of current list
	int m_iListValidID;
	static int s_iListValidID;

};


////////////////////////////////////////////////////////////////////////////////////////////

//drop down list for selecting event/sample file
class Tools_SoundFilename_Dialog : public Asura_DropDown_List_Dialog
{
public :
	Tools_SoundFilename_Dialog(const Asura_Vector_2& xOffset,Asura_String* pxFilename,bool* pbModified);
	~Tools_SoundFilename_Dialog();


private:
	void PerFrameUpdate();

	static void OnSelectionChanged(int iThis){((Tools_SoundFilename_Dialog*)iThis)->OnSelectionChanged();}
	void OnSelectionChanged();

	void UpdateFilenameList();
	void DeleteFilenameList();

	Asura_String* m_pxFilename;
	bool* m_pbModified;

	Asura_String m_xFilenameBackup;
	bool m_bSelectionChanged;

	int m_iSelectedIndex;
	bool m_bInitialised;

	u_int m_uFilenameListSize;
	const char** m_pszFilenameList;
	class Asura_NameTree* m_pxFilenameTree;

	//for determining validity of current list
	int m_iListValidID;
	static int s_iListValidID;

};

////////////////////////////////////////////////////////////////////////////////////////////
//drop down list for selecting event instance group
class Tools_SoundInstanceGroup_Dialog : public Asura_DropDown_List_Dialog
{
public :
	Tools_SoundInstanceGroup_Dialog(const Asura_Vector_2& xOffset,Asura_String* pxInstanceGroup,bool* pbModified);
	~Tools_SoundInstanceGroup_Dialog();


private:
	void PerFrameUpdate();

	static void OnSelectionChanged(int iThis){((Tools_SoundInstanceGroup_Dialog*)iThis)->OnSelectionChanged();}
	void OnSelectionChanged();

	void UpdateInstanceGroupList();
	void DeleteInstanceGroupList();

	Asura_String* m_pxInstanceGroup;
	bool* m_pbModified;

	Asura_String m_xInstanceGroupBackup;
	bool m_bSelectionChanged;


	int m_iSelectedIndex;
	bool m_bInitialised;

	u_int m_uInstanceGroupListSize;
	const char** m_pszInstanceGroupList;
	class Asura_NameTree* m_pxInstanceGroupNameTree;

	//for determining validity of current list
	int m_iListValidID;
	static int s_iListValidID;

};

#endif //__TOOLS_SOUNDEVENT_OVERLAY__
