//
// Tools_StreamingBackgroundSound_Object.cpp
//
// Created by Richard Rice - 09/01/04
// 
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Sound/Tools_StreamingBackgroundSound_Object.h"

#include "Camera/Asura_Camera.h"
#include "Core/Asura_UserInput.h"
#include "Entity/Sound/Asura_ServerEntity_StreamingBackgroundSoundController.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Network/Asura_Message.h"
#include "Rendering/Asura_Render.h"
#include "Sound/Asura_StreamingBackgroundSound_System.h"
#include "Volume/Asura_VolumeGroup.h"

#include "Tools_Overlays.h"
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Entity_Object.h"
#include "Lighting/Tools_LightSets.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_ObjectSystem_Param.h"
#include "Objects/Tools_VolumeEdit_Object.h"
#include "Objects/Tools_VolumeGroupEdit_Controller.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Sound/Tools_Mode_Sound.h"
#include "Sound/Tools_SoundMixer_Overlay.h"


//Defines for making the callback functions for Tools_StreamingBackgroundSound_Object
#define CALLBACK_VOID(__Function) Asura_Callback_MemberFunction<void,Tools_StreamingBackgroundSound_Object>(this, &Tools_StreamingBackgroundSound_Object::__Function) 
#define CALLBACK_BOOL(__Function) Asura_Callback_MemberFunction<bool,Tools_StreamingBackgroundSound_Object>(this, &Tools_StreamingBackgroundSound_Object::__Function) 
#define CALLBACK_VOID_BOOL(__Function) Asura_Callback_MemberFunction_1Arg<void,bool,Tools_StreamingBackgroundSound_Object>(this, &Tools_StreamingBackgroundSound_Object::__Function) 

static const u_int g_uInnerColour = 0x0000ff;
static const u_int g_uOuterColour = 0x7ff00ef;
static const u_int g_uOverlapColour = 0xff0000;


Asura_Vector_2 Tools_StreamingBackgroundSound_Object::s_xOverlayPosition(10,10);

bool Tools_StreamingBackgroundSound_Object::s_bShowInnerBox = true;
bool Tools_StreamingBackgroundSound_Object::s_bShowOuterBox = true;
bool Tools_StreamingBackgroundSound_Object::s_bShowSelectedOnly = true;
bool Tools_StreamingBackgroundSound_Object::s_bWireframe = false;
bool Tools_StreamingBackgroundSound_Object::s_bShowOverlap = true;

List<Tools_StreamingBackgroundSound_Object*> Tools_StreamingBackgroundSound_Object::s_xObjectList;


static const Asura_Hash_ID g_uHash_LightSource_LightSet0 = 0xBD2D1016; 
static const Asura_Hash_ID g_uHash_LightSource_LightSetLast = g_uHash_LightSource_LightSet0+TOOLS_MAX_LIGHT_SETS-1; 
static Tools_ObjectSystem_SharedValue<bool> g_axParam_LightSource_LightSet[TOOLS_MAX_LIGHT_SETS];

//TOOLS_DECLARE_SHARED_PARAM(bool,	0x7B78C998,	StreamingSound_Active)
TOOLS_DECLARE_SHARED_PARAM(int,		0x8D15564C,	StreamingSound_Type)
TOOLS_DECLARE_SHARED_PARAM(bool,	0x9121C0BA,	StreamingSound_UseOrient)
TOOLS_DECLARE_SHARED_PARAM(float,	0x4647A316,	StreamingSound_InnerRadius)
TOOLS_DECLARE_SHARED_PARAM(float,	0xEB40857B,	StreamingSound_OuterRadius)
TOOLS_DECLARE_SHARED_PARAM(float,	0x3571C7AB,	StreamingSound_InnerHeight)
TOOLS_DECLARE_SHARED_PARAM(float,	0xDA6AAA10,	StreamingSound_OuterHeight)
TOOLS_DECLARE_SHARED_PARAM(int,		0x9DA967B6,	StreamingSound_Priority)
TOOLS_DECLARE_SHARED_PARAM(bool,	0x1AEAA45F,	StreamingSound_UseClipVolume)
TOOLS_DECLARE_SHARED_PARAM(bool,	0x9FED56D2, StreamingSound_InvertClipVolume)

//Now uses the same ID for Active as Tools_Entity_Object
Tools_ObjectSystem_SharedValue< bool > g_xParam_StreamingSound_Active( g_uHash_Entity_StartsActive);


static const char* g_aszTypeNames[] =
{
	"Cuboid",
	"Sphere",
	"Cylinder",
};


Tools_StreamingBackgroundSound_Object::Tools_StreamingBackgroundSound_Object(int iIndex)
:Tools_Object(TOT_StreamingBackgroundSound)
{
	s_xObjectList.add_entry(this);
	Asura_Assert(iIndex>=0 && iIndex < Asura_StreamingBackgroundSound_System::s_iNumberOfSounds,"");

	m_iIndex = iIndex;
	Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);
	m_iType = pxSound->m_iType;
	m_fVolume = pxSound->m_fVolume;
	m_uMixer = pxSound->m_uMixer;
	m_xInnerBox = pxSound->m_xInnerBox;
	m_xOuterBox = pxSound->m_xOuterBox;
	m_xOrient = pxSound->m_xOrient;
	m_bUseOrient = (pxSound->m_xOrient.w != 1.0f);
	m_bInnerBoxChanged = false;
	m_bOuterBoxChanged = false;
	m_iInnerVolumeID = 0;
	m_iOuterVolumeID = 0;
	m_pxClipVolumeController = 0;
	m_bChanged = false;
	m_uLightSets = pxSound->m_uLightSets;
	m_uSoundFlags = pxSound->m_uFlags;
	m_fInnerRadius = pxSound->m_fInnerRadius;
	m_fOuterRadius = pxSound->m_fOuterRadius;
	m_fInnerHeight = pxSound->m_fInnerHeight;
	m_fOuterHeight = pxSound->m_fOuterHeight;
	m_iPriority = pxSound->m_iPriority;
	m_bOverlapListValid = false;
	m_uOverlapListLightSets = 0;

	Asura_PrintfToBfr(m_szName,"Streaming Sound %d",24,true,m_iIndex);

	
	SetMesh("ToolsShape_StreamingSound");

	Position = pxSound->m_xPosition;
	m_xPreviousPosition = Position;
	if(m_bUseOrient)
	{
		m_xOrient.ToMatrix(Orientation);
	}
	else
	{
		Orientation.Identity();
	}

	SetDistantIconTexture("\\tools\\Sound.dds");

}

Tools_StreamingBackgroundSound_Object::~Tools_StreamingBackgroundSound_Object()
{
	Tools_VolumeEdit_Object::Delete(m_iInnerVolumeID);
	Tools_VolumeEdit_Object::Delete(m_iOuterVolumeID);
	delete m_pxClipVolumeController;
	s_xObjectList.delete_entry(this);
}

void Tools_StreamingBackgroundSound_Object::UpdatePositionInAsuraData(bool /*bUpdateLinkedObjects*/)
{
	bool bChanged = false;

	if(Position != m_xPreviousPosition)
	{
		Asura_Vector_3 xOffset = Position-m_xPreviousPosition;
		m_xPreviousPosition = Position;	

		//move the bounding boxes as well
		Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);

		m_xInnerBox.Displace(xOffset);
		m_xOuterBox.Displace(xOffset);

		if(pxSound->m_pxClipVolume)
		{
			pxSound->m_pxClipVolume->DisplaceVolumes(xOffset);
		}

		if(m_pxClipVolumeController)
		{
			m_pxClipVolumeController->MoveVolumeObjects(xOffset);			
		}
		
		pxSound->m_xInnerBox = m_xInnerBox;
		pxSound->m_xOuterBox = m_xOuterBox;
		pxSound->m_xPosition = Position;

		bChanged = true;

	}
	
	if(m_bUseOrient)
	{
		//If orientations are being used , then update the box orient to match
		m_xOrient.FromMatrix(Orientation);

		Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);
		pxSound->m_xOrient = m_xOrient;

		bChanged = true;
	}

	if(bChanged)
	{
		m_bOverlapListValid = false;
		Asura_StreamingBackgroundSound_System::InvalidateVolumeTree();
		Tools_SaveAsura::SetSoundsModified();
	}
}

void Tools_StreamingBackgroundSound_Object::Render()
{
	//render the object itself
	Tools_Object::Render();
	
	//and draw the volumes if needed
	if(!s_bShowSelectedOnly || IsSelected() || IsGroupSelected())
	{
	    Asura_Render::CurrentStates.RequestWireFrameMode(false);
		
		//oriented volumes are done relative to the centre of the outer volume
		Asura_Vector_3 xCentre = m_xOuterBox.CalculateCentre();
		Asura_Matrix_3x3 xMat;
		m_xOrient.ToMatrix(xMat);
		
		
		extern void Render_Sphere( const Asura_Vector_3& xCentre, float fRadius, int iColour );
		
		if(s_bShowInnerBox)
		{
			if(IsSphere())
			{
				Render_Sphere(Position, m_fInnerRadius, g_uInnerColour|0x3f000000); 
			}
			else if(IsCylinder())
			{
				Asura_Vector_3 xAxis(0.0f, m_fInnerHeight/2.0f, 0.0f);
				if(m_bUseOrient)
				{
					xAxis = xAxis * Orientation;
				}
				Tools_Render::RenderCylinder(Position-xAxis, Position+xAxis, m_fInnerRadius, g_uInnerColour|0xff000000, true);
				if(!s_bWireframe) Tools_Render::RenderCylinder(Position-xAxis, Position+xAxis, m_fInnerRadius, g_uInnerColour|0x3f000000, false);
			}
			else
			{
				if(m_iInnerVolumeID)
				{
					if(!Tools_VolumeEdit_Object::Find(m_iInnerVolumeID))
					{
						m_iInnerVolumeID = 0;
					}
				}
				//only want to draw volume if there isn't a seperate volume edit object
				if(!m_iInnerVolumeID)
				{
					Asura_Bounding_Box xRelativeBox = m_xInnerBox;
					xRelativeBox.Displace(-xCentre);
					
					u_int uColour = g_uInnerColour;
					Tools_Render::RenderCube(xRelativeBox,xCentre,xMat,uColour|0xff000000,true);
					if(!s_bWireframe) Tools_Render::RenderCube(xRelativeBox,xCentre,xMat,uColour|0x3f000000,false);
				}
			}
		}
		if(s_bShowOuterBox)
		{
			if(IsSphere())
			{
				Render_Sphere(Position, m_fOuterRadius, g_uOuterColour|0x3f000000); 
			}
			else if(IsCylinder())
			{
				Asura_Vector_3 xAxis(0.0f, m_fOuterHeight/2.0f, 0.0f);
				if(m_bUseOrient)
				{
					xAxis = xAxis * Orientation;
				}
				Tools_Render::RenderCylinder(Position-xAxis, Position+xAxis, m_fOuterRadius, g_uOuterColour|0xff000000, true);
				if(!s_bWireframe) Tools_Render::RenderCylinder(Position-xAxis, Position+xAxis, m_fOuterRadius, g_uOuterColour|0x3f000000, false);
			}
			else
			{
				if(m_iOuterVolumeID)
				{
					if(!Tools_VolumeEdit_Object::Find(m_iOuterVolumeID))
					{
						m_iOuterVolumeID = 0;
					}
				}
				//only want to draw volume if there isn't a seperate volume edit object
				if(!m_iOuterVolumeID)
				{
					Asura_Bounding_Box xRelativeBox = m_xOuterBox;
					xRelativeBox.Displace(-xCentre);

					u_int uColour = g_uOuterColour;
					Tools_Render::RenderCube(xRelativeBox,xCentre,xMat,uColour|0xff000000,true);
					if(!s_bWireframe) Tools_Render::RenderCube(xRelativeBox,xCentre,xMat,uColour|0x3f000000,false);
				}
			}

		}

		if(s_bShowOverlap && m_bOverlapListValid)
		{
			//draw cubes for the overlaps
			for(LIF<Asura_Bounding_Box> xBoxLIF(&m_xOverlapList);!xBoxLIF.done();xBoxLIF.next())
			{
				u_int uColour = g_uOverlapColour;
				Tools_Render::RenderCube(xBoxLIF(),uColour|0xff000000,true);
				if(!s_bWireframe) Tools_Render::RenderCube(xBoxLIF(),uColour|0x5f000000,false);
				
			}
		}
	
		Asura_Render::CurrentStates.RequestWireFrameMode(Asura_Render::GetWireframe());
 	}

}

void Tools_StreamingBackgroundSound_Object::Update()
{
	if(m_bChanged)
	{
		Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);
		
		pxSound->m_iType = m_iType;
		pxSound->m_fVolume = m_fVolume;
		pxSound->m_uMixer = m_uMixer;
		pxSound->m_uLightSets = m_uLightSets;
		pxSound->m_uFlags = m_uSoundFlags;
		pxSound->m_fInnerRadius = m_fInnerRadius;
		pxSound->m_fOuterRadius = m_fOuterRadius;
		pxSound->m_fInnerHeight = m_fInnerHeight;
		pxSound->m_fOuterHeight = m_fOuterHeight;
		pxSound->m_iPriority = m_iPriority;

		if(m_bUseOrient)
		{
			//If orientations are being used , then update the box orient to match
			m_xOrient.FromMatrix(Orientation);
			pxSound->m_xOrient = m_xOrient;
		}
		else
		{
			m_xOrient = Asura_Maths::IdentityQuat;
			pxSound->m_xOrient = m_xOrient;
		}


		Asura_StreamingBackgroundSound_System::InvalidateVolumeTree();

		m_bChanged = false;
		m_bOverlapListValid = false;

		Tools_SaveAsura::SetSoundsModified();
	}

	if(!IsCuboid())
	{
		Tools_VolumeEdit_Object::Delete(m_iInnerVolumeID);
		Tools_VolumeEdit_Object::Delete(m_iOuterVolumeID);
	}
}

void Tools_StreamingBackgroundSound_Object::UpdateWhileSelected()
{
	//Make sure the inner box stays inside the outer box.
	//Only do this while the object is selected , to prevent overshooting while editing the boxes
	//from immediately taking effect.

	if(m_bInnerBoxChanged)
	{
		//make sure the outer box is outside the inner box
		if(m_xOuterBox.MinX > m_xInnerBox.MinX) m_xOuterBox.MinX = m_xInnerBox.MinX;
		if(m_xOuterBox.MinY > m_xInnerBox.MinY) m_xOuterBox.MinY = m_xInnerBox.MinY;
		if(m_xOuterBox.MinZ > m_xInnerBox.MinZ) m_xOuterBox.MinZ = m_xInnerBox.MinZ;
		if(m_xOuterBox.MaxX < m_xInnerBox.MaxX) m_xOuterBox.MaxX = m_xInnerBox.MaxX;
		if(m_xOuterBox.MaxY < m_xInnerBox.MaxY) m_xOuterBox.MaxY = m_xInnerBox.MaxY;
		if(m_xOuterBox.MaxZ < m_xInnerBox.MaxZ) m_xOuterBox.MaxZ = m_xInnerBox.MaxZ;
	}
	else if(m_bOuterBoxChanged)
	{
		//make sure the outer box is outside the inner box
		if(m_xInnerBox.MinX < m_xOuterBox.MinX) m_xInnerBox.MinX = m_xOuterBox.MinX;
		if(m_xInnerBox.MinY < m_xOuterBox.MinY) m_xInnerBox.MinY = m_xOuterBox.MinY;
		if(m_xInnerBox.MinZ < m_xOuterBox.MinZ) m_xInnerBox.MinZ = m_xOuterBox.MinZ;
		if(m_xInnerBox.MaxX > m_xOuterBox.MaxX) m_xInnerBox.MaxX = m_xOuterBox.MaxX;
		if(m_xInnerBox.MaxY > m_xOuterBox.MaxY) m_xInnerBox.MaxY = m_xOuterBox.MaxY;
		if(m_xInnerBox.MaxZ > m_xOuterBox.MaxZ) m_xInnerBox.MaxZ = m_xOuterBox.MaxZ;
	}

	if(m_bInnerBoxChanged || m_bOuterBoxChanged)
	{
		m_bInnerBoxChanged = false;
		m_bOuterBoxChanged = false;
		m_bOverlapListValid = false;

		//update position to be at the centre of the box
		Position = m_xInnerBox.CalculateCentre();
		m_xPreviousPosition = Position;

		Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);
		
		pxSound->m_xInnerBox = m_xInnerBox;
		pxSound->m_xOuterBox = m_xOuterBox;
		pxSound->m_xOrient = m_xOrient;

		Asura_StreamingBackgroundSound_System::InvalidateVolumeTree();

		Tools_SaveAsura::SetSoundsModified();
	}
	//check for the current light set changing
	if(m_uOverlapListLightSets != Tools_LightSets::GetCurrentLightSets())
	{
		m_bOverlapListValid = false;
	}

	if(s_bShowOverlap && !m_bOverlapListValid)
	{
		UpdateOverlapList();
	}
}

void Tools_StreamingBackgroundSound_Object::Destroy()
{
	//If there is a controller entity ,delete that as well
	Asura_Guid uGuid = GetGuid();
	if(uGuid != ASURA_GUID_UNREGISTERED)
	{
		Asura_Entity_Server* pxEntity = Asura_Entity_Server::FindEntity(uGuid);
		if(pxEntity)
		{
			pxEntity->SetAsDestroyed();
		}
	}

	//delete the asura sound
	Asura_StreamingBackgroundSound_System::DeleteSound(m_iIndex);

	
	//If this wasn't the last sound in the array , then we need to find the tools object
	//for the last sound , and alter it to take into account that it will have had its
	//index changed
	int iLastIndex = Asura_StreamingBackgroundSound_System::s_iNumberOfSounds;
	if(iLastIndex != m_iIndex)
	{
		for(LIF<Tools_StreamingBackgroundSound_Object*> xObjectLIF(&s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
		{
			if(xObjectLIF()->m_iIndex == iLastIndex)
			{
				//Update this object
				xObjectLIF()->m_iIndex = m_iIndex;
				strcpy(xObjectLIF()->m_szName,m_szName);
			}
		}
	}

	
	Tools_Object::Destroy();		

	Tools_SaveAsura::SetSoundsModified();
}

void Tools_StreamingBackgroundSound_Object::OnSelection()
{
	CreateOverlay();

	//Determine whether the orientation is being used , and update the object's orientation to match.
	//(Done here , since this will allow things to be updated after finishing box editing)
	m_bUseOrient = (m_xOrient.w != 1.0f);
	if(m_bUseOrient)
	{
		m_xOrient.ToMatrix(Orientation);
	}
}

void Tools_StreamingBackgroundSound_Object::OnDeselection()
{
	DeleteOverlay();
	m_bOverlapListValid = false;
}

void Tools_StreamingBackgroundSound_Object::CreateOverlay()
{
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Streaming Sound",false,&s_xOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

 
	//start of tabs
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset,0,0);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxOverlay->SetCurrentParentDialog(pxTabsDialog);

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;

	//Main tab
	pxTabsDialog->AddTab("Main");
	xOffset.y = fTabsStartY;

	
	
	pxDialog = new Asura_Function_Dialog("",xOffset,_SelectFile,(int)this);
	pxDialog->m_iControlID = DialogID_Filename;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Clear File",xOffset,_ClearFile,(int)this);
	pxDialog->SetHelpText("Clears the filename associated with this volume , turning it into an anti-sound volume.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Priority", xOffset, &g_xParam_StreamingSound_Priority, 0, 999, 1,"%d");
	pxDialog->SetHelpText("The priority is used for determining which sounds to play when in multiple volumes.\n"
							"If in the space covered by two sounds with differing priorities, then the lower one will be faded out based on the distance into the higher one.\n"
							"The default sound is considered to have a priority of -1 ,so as to be lower than everything else.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();


	xOffset.x += 1.0f;
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_aszTypeNames, &g_xParam_StreamingSound_Type, 3, 3, NULL);
	pxDialog->SetHelpText("The type of volume to use");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	xOffset.x -= 1.0f;

	pxDialog = new Asura_SharedTickbox_Dialog("Use Orient", xOffset, &g_xParam_StreamingSound_UseOrient);
	pxDialog->SetHelpText("Determines whether the orientation of a cylinder or cuboid volume should be used");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Numerical_Dialog("Inner Range", xOffset, &g_xParam_StreamingSound_InnerRadius, 0.5f,9999.0f,0.1f,"%0.1f");
	pxDialog->SetHelpText("The inner range defines the point at which the sound will start to attenuate from full volume.");
	pxDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_Range));
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Outer Range", xOffset, &g_xParam_StreamingSound_OuterRadius, 0.5f,9999.0f,0.1f,"%0.1f");
	pxDialog->SetHelpText("The outer range defines the point at which the sound's volume reaches 0.");
	pxDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_Range));
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Inner Height", xOffset, &g_xParam_StreamingSound_InnerHeight, 0.5f,9999.0f,0.1f,"%0.1f");
	pxDialog->SetHelpText("The inner height defines the point at which the sound will start to attenuate from full volume.");
	pxDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_Height));
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Outer Height", xOffset, &g_xParam_StreamingSound_OuterHeight, 0.5f,9999.0f,0.1f,"%0.1f");
	pxDialog->SetHelpText("The outer height defines the point at which the sound's volume reaches 0.");
	pxDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_Height));
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Edit Inner Box",xOffset,_EditInnerVolume,(int)this);
	pxDialog->SetHelpText("Edit the inner volume for this sound");
	pxDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_Box));
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Outer Box",xOffset,_EditOuterVolume,(int)this);
	pxDialog->SetHelpText("Edit the outer volume for this sound");
	pxDialog->SetIsEnabledFunction(CALLBACK_BOOL(IsEnabled_Box));
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Numerical_Dialog("Volume",xOffset,&m_fVolume,0.01f,1.5f,0.01f,"%.02f",&m_bChanged);
	pxDialog->SetHelpText("Multiplier to use for volume.\nNote that 1.33 is effectively maximum , since all volumes are scaled by 0.75 and capped at 1.0 at the end.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Active",xOffset,&g_xParam_StreamingSound_Active);
	pxDialog->SetHelpText("This is active/playing at the start of the level.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Use Clipping Volume",xOffset,&g_xParam_StreamingSound_UseClipVolume);
	pxDialog->SetHelpText("Sound should only be audible in the clipping volumes defined for it.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_SharedTickbox_Dialog("Invert Clipping Volume",xOffset,&g_xParam_StreamingSound_InvertClipVolume);
	pxDialog->SetHelpText("If inverted  then the sound is only played when outside of the clipping volumes.\nOnly does anything if Use Clipping Volume is on.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Clipping Volume",xOffset,_EditClipVolume, (int)this);
	pxDialog->SetHelpText("Edit the optional clipping volumes for the sound.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_SoundMixerHash_Dialog(xOffset,8,&m_uMixer,&m_bChanged);
	pxDialog->SetHelpText("The mixer to use for this sound.\nIf set to *Default* then the 'Streaming Sounds' mixer will be used");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Inner",xOffset,&s_bShowInnerBox);
	pxDialog->SetHelpText("Show the inner volumes of the sound objects");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Outer",xOffset,&s_bShowOuterBox);
	pxDialog->SetHelpText("Show the outer volumes of the sound objects");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Selected Only",xOffset,&s_bShowSelectedOnly);
	pxDialog->SetHelpText("Only show the volumes of the selected objects");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Overlaps",xOffset,&s_bShowOverlap);
	pxDialog->SetHelpText("Show red cubes for any undesirable overlaps involving the selected object :\nAny overlap of inner volumes.\nAny overlap of outer volumes if both are stereo.\nAny cases of 3 volumes overlapping.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Wireframe",xOffset,&s_bWireframe);
	pxDialog->SetHelpText("Use wireframe cubes for showing the inner/outer volume.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//End of main tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//Light set tab
	pxTabsDialog->AddTab("Light Sets");
	xOffset.y = fTabsStartY;

	//add a checkbox for each light set
	for(u_int i=0;i<TOOLS_MAX_LIGHT_SETS;i++)
	{
		const char* szSetName = Tools_LightSets::GetLightSetNameByIndex(i);
		if(!szSetName) continue;

		g_axParam_LightSource_LightSet[i].SetParamID(g_uHash_LightSource_LightSet0+i);

		pxDialog = new Asura_SharedTickbox_Dialog(szSetName,xOffset,&g_axParam_LightSource_LightSet[i]);
		pxDialog->SetHelpText("Causes sound to be exported along with this light set during level exports.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}


	//End of light set tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//End of tabs
	xOffset.y = fTabsEndY + 1.0f;
	pxOverlay->SetCurrentParentDialog(0);
	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);



	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);

	UpdateOverlay();
}


void Tools_StreamingBackgroundSound_Object::DeleteOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
}

void Tools_StreamingBackgroundSound_Object::UpdateOverlay()
{
	if(!IsSelected()) return;
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_SELECTEDOBJECT);
	if(!pxOverlay) return;

	Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);
	
	
	char szText[200];
	//update filename
	if(pxSound->m_xFilename.IsBlank())
	{
		strcpy(szText,"File : <Anti-Sound Volume>");
	}
	else
	{
		Asura_String xName;
		pxSound->m_xFilename.SplitFilename(0,&xName,0);
		Asura_PrintfToBfr(szText,"File : %s",200,true,xName.GetString());
	}
	pxOverlay->UpdateDialogLabel(DialogID_Filename,szText);
	
}


void Tools_StreamingBackgroundSound_Object::Create()
{
	//create a new asura sound
	int iIndex = Asura_StreamingBackgroundSound_System::CreateSound(*Asura_Camera::GetPosition());
	Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(iIndex);
	pxSound->m_uLightSets = Tools_LightSets::GetLightSetsForNewObject();
	//create the tools object for it
	Tools_Object* pxObject = new Tools_StreamingBackgroundSound_Object(iIndex);
	pxObject->Orientation = *Asura_Camera::GetOrientation();
	//and select it
	Tools_Object_System::SelectObject(pxObject);
}

Tools_Object* Tools_StreamingBackgroundSound_Object::CreateObjectOfThisType() const
{
	//create a new asura sound
	int iIndex = Asura_StreamingBackgroundSound_System::CreateSound(*Asura_Camera::GetPosition());
	Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(iIndex);
	pxSound->m_uLightSets = Tools_LightSets::GetLightSetsForNewObject();
	//create the tools object for it
	Tools_Object* pxObject = new Tools_StreamingBackgroundSound_Object(iIndex);
	pxObject->Orientation = *Asura_Camera::GetOrientation();

	return pxObject;
}


void Tools_StreamingBackgroundSound_Object::CreateAllObjects()
{
	SyncWithControllerEntities();

	for(int i=0;i<Asura_StreamingBackgroundSound_System::s_iNumberOfSounds;i++)
	{
		new Tools_StreamingBackgroundSound_Object(i);
	}
}

void Tools_StreamingBackgroundSound_Object::DeleteAllObjects()
{
	while(s_xObjectList.size())
	{
		Tools_Object_System::DeleteObject(s_xObjectList.first_entry());
	}
}

void Tools_StreamingBackgroundSound_Object::EditInnerVolume()
{
	if(!IsCuboid()) return;

	//does the volume already exist?
	Tools_VolumeEdit_Object* pxVolume = Tools_VolumeEdit_Object::Find(m_iInnerVolumeID);
	if(!pxVolume)
	{
		//need to create it then.
		//generate a name
		char szName[200];
		sprintf(szName,"%s - Inner",GetName());

		//and create the volume
		pxVolume = new Tools_StreamingVolumeEdit_Object(szName,&m_xInnerBox,&m_bInnerBoxChanged,false);
		pxVolume->SetOwner(this);
		pxVolume->SetVolumeOrientation(&m_xOrient);
		m_iInnerVolumeID = pxVolume->GetID();
	}

	//select the volume
	Tools_Object_System::SelectObject(pxVolume);
}

void Tools_StreamingBackgroundSound_Object::EditOuterVolume()
{
	if(!IsCuboid()) return;

	//does the volume already exist?
	Tools_VolumeEdit_Object* pxVolume = Tools_VolumeEdit_Object::Find(m_iOuterVolumeID);
	if(!pxVolume)
	{
		//need to create it then.
		//generate a name
		char szName[200];
		sprintf(szName,"%s - Outer",GetName());

		//and create the volume
		pxVolume = new Tools_StreamingVolumeEdit_Object(szName,&m_xOuterBox,&m_bOuterBoxChanged,true);
		pxVolume->SetOwner(this);
		pxVolume->SetVolumeOrientation(&m_xOrient);
		m_iOuterVolumeID = pxVolume->GetID();
	}

	//select the volume
	Tools_Object_System::SelectObject(pxVolume);
}

void Tools_StreamingBackgroundSound_Object::SelectFile()
{
	Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);

	if(Tools_Mode_Sound::SelectStreamingSound(pxSound->m_xFilename))
	{
		pxSound->m_uFilenameHash = Asura_GetHashID(pxSound->m_xFilename.GetString());
		m_bOverlapListValid = false;
		UpdateOverlay();
		
		Tools_SaveAsura::SetSoundsModified();

		Asura_StreamingBackgroundSound_System::StopAll();

	}
}

void Tools_StreamingBackgroundSound_Object::ClearFile()
{
	Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);
	pxSound->m_xFilename.MakeBlank();
	pxSound->m_uFilenameHash = ASURA_HASH_ID_UNSET;

	UpdateOverlay();
	Tools_SaveAsura::SetSoundsModified();

	Asura_StreamingBackgroundSound_System::StopAll();
}

void Tools_StreamingBackgroundSound_Object::UpdateOverlapList()
{
	while(m_xOverlapList.size()) m_xOverlapList.delete_first_entry();

	//give up if oriented for now
	if(m_xOrient.w != 1.0f) return;

	//only update overlaps for current light set
	if(!Tools_LightSets::IsMemberOfCurrentLightSets(m_uLightSets)) return;
	
	Asura_Bounding_Box xThisOuterBox = GetOuterExtents();
	Asura_Bounding_Box xThisInnerBox = GetInnerExtents();
	
	//first get a list of all the objects that overlap with this one at all
	List<Tools_StreamingBackgroundSound_Object*> xNearList;
	for(LIF<Tools_StreamingBackgroundSound_Object*> xObjectLIF(&s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()==this) continue;
		if(!Tools_LightSets::IsMemberOfCurrentLightSets(xObjectLIF()->m_uLightSets)) return;
		if(xObjectLIF()->m_iPriority != m_iPriority) continue;

		if(xObjectLIF()->m_xOrient.w != 1.0f) continue;

		Asura_Bounding_Box xThatOuterBox = xObjectLIF()->GetOuterExtents();

		if(xThatOuterBox.Intersects(xThisOuterBox))
		{
			xNearList.add_entry(xObjectLIF());
		}
	}

	const float fMinExtent = 0.1f;//Minimum amount of overlap to worry about

	for(int i=0;i<xNearList.size();i++)
	{
		//get the overlap between the 2 outer bounding boxes
		Asura_Bounding_Box xOverlap;
		Asura_Bounding_Box xThatInnerBox = xNearList[i]->GetInnerExtents();
		Asura_Bounding_Box xThatOuterBox = xNearList[i]->GetOuterExtents();
												   
		xOverlap.MinX = Asura_Maths::Max(xThisOuterBox.MinX,xThatOuterBox.MinX);
		xOverlap.MinY = Asura_Maths::Max(xThisOuterBox.MinY,xThatOuterBox.MinY);
		xOverlap.MinZ = Asura_Maths::Max(xThisOuterBox.MinZ,xThatOuterBox.MinZ);
		xOverlap.MaxX = Asura_Maths::Min(xThisOuterBox.MaxX,xThatOuterBox.MaxX);
		xOverlap.MaxY = Asura_Maths::Min(xThisOuterBox.MaxY,xThatOuterBox.MaxY);
		xOverlap.MaxZ = Asura_Maths::Min(xThisOuterBox.MaxZ,xThatOuterBox.MaxZ);
		

		//check for overlaps in inner volumes

		if(xThisInnerBox.Intersects(xThatInnerBox))
		{
			Asura_Bounding_Box xBox;
			xBox.MinX = Asura_Maths::Max(xThisInnerBox.MinX,xThatInnerBox.MinX);
			xBox.MinY = Asura_Maths::Max(xThisInnerBox.MinY,xThatInnerBox.MinY);
			xBox.MinZ = Asura_Maths::Max(xThisInnerBox.MinZ,xThatInnerBox.MinZ);
			xBox.MaxX = Asura_Maths::Min(xThisInnerBox.MaxX,xThatInnerBox.MaxX);
			xBox.MaxY = Asura_Maths::Min(xThisInnerBox.MaxY,xThatInnerBox.MaxY);
			xBox.MaxZ = Asura_Maths::Min(xThisInnerBox.MaxZ,xThatInnerBox.MaxZ);

			Asura_Vector_3 xExtent = xBox.CalculateExtents();
			if(xExtent.x > fMinExtent && xExtent.y > fMinExtent && xExtent.z > fMinExtent)
			{
				m_xOverlapList.add_entry(xBox);
			}
		}

		//check for 3-way intersection of outer boxes
		for(int j=i+1;j<xNearList.size();j++)
		{
			
			Asura_Bounding_Box xOuterBox3 = xNearList[j]->GetOuterExtents();
	
			if(xOverlap.Intersects(xOuterBox3))
			{
				Asura_Bounding_Box xBox;
				xBox.MinX = Asura_Maths::Max(xOverlap.MinX,xOuterBox3.MinX);
				xBox.MinY = Asura_Maths::Max(xOverlap.MinY,xOuterBox3.MinY);
				xBox.MinZ = Asura_Maths::Max(xOverlap.MinZ,xOuterBox3.MinZ);
				xBox.MaxX = Asura_Maths::Min(xOverlap.MaxX,xOuterBox3.MaxX);
				xBox.MaxY = Asura_Maths::Min(xOverlap.MaxY,xOuterBox3.MaxY);
				xBox.MaxZ = Asura_Maths::Min(xOverlap.MaxZ,xOuterBox3.MaxZ);

				Asura_Vector_3 xExtent = xBox.CalculateExtents();
				if(xExtent.x > fMinExtent && xExtent.y > fMinExtent && xExtent.z > fMinExtent)
				{
					m_xOverlapList.add_entry(xBox);
				}
					
			}
		}
	}
	m_bOverlapListValid = true;
	m_uOverlapListLightSets = Tools_LightSets::GetCurrentLightSets();
}


void Tools_StreamingBackgroundSound_Object::Copy(const Tools_Object* pxObject)
{
	Asura_Assert(pxObject->Type == TOT_StreamingBackgroundSound,"");

	const Tools_StreamingBackgroundSound_Object* pxFromObject = (const Tools_StreamingBackgroundSound_Object*) pxObject;

	delete m_pxClipVolumeController;
	m_pxClipVolumeController = 0;

	//Copy data from tools object
	m_iType = pxFromObject->m_iType;
	m_fVolume = pxFromObject->m_fVolume;
	m_uMixer = pxFromObject->m_uMixer;
	m_xInnerBox = pxFromObject->m_xInnerBox;
	m_xOuterBox = pxFromObject->m_xOuterBox;
	m_uLightSets = pxFromObject->m_uLightSets;
	m_uSoundFlags = pxFromObject->m_uSoundFlags;
	m_fInnerRadius = pxFromObject->m_fInnerRadius;
	m_fOuterRadius = pxFromObject->m_fOuterRadius;
	m_fInnerHeight = pxFromObject->m_fInnerHeight;
	m_fOuterHeight = pxFromObject->m_fOuterHeight;
	m_iPriority = pxFromObject->m_iPriority;
	m_bOverlapListValid = false;

	//Don't copy the orientation , but instead determine whether the orientation is being used
	m_bUseOrient = pxFromObject->m_xOrient.w != 1.0f;
	if(m_bUseOrient)
	{
		m_xOrient.FromMatrix(Orientation);	
	}
	else
	{
		m_xOrient = Asura_Maths::IdentityQuat;
	}

	//Shift the bounding boxes based on the difference in position
	m_xInnerBox.Displace(Position-pxObject->Position);
	m_xOuterBox.Displace(Position-pxObject->Position);


	//Update the asura sounds
	Asura_StreamingBackgroundSound* pxToSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);
	const Asura_StreamingBackgroundSound* pxFromSound = Asura_StreamingBackgroundSound_System::GetSound(pxFromObject->m_iIndex);

	Asura_Assert(pxToSound && pxFromSound,"");

	pxToSound->m_xFilename = pxFromSound->m_xFilename;
	pxToSound->m_uFilenameHash = pxFromSound->m_uFilenameHash;
	
	pxToSound->m_iType = pxFromSound->m_iType;
	pxToSound->m_fVolume = pxFromSound->m_fVolume;
	pxToSound->m_uMixer = pxFromSound->m_uMixer;
	pxToSound->m_xInnerBox = pxFromSound->m_xInnerBox;
	pxToSound->m_xOuterBox = pxFromSound->m_xOuterBox;
	pxToSound->m_xOrient = m_xOrient;
	pxToSound->m_uLightSets = pxFromSound->m_uLightSets;
	pxToSound->m_uFlags = pxFromSound->m_uFlags;
	pxToSound->m_fInnerRadius = pxFromSound->m_fInnerRadius;
	pxToSound->m_fOuterRadius = pxFromSound->m_fOuterRadius;
	pxToSound->m_fInnerHeight = pxFromSound->m_fInnerHeight;
	pxToSound->m_fOuterHeight = pxFromSound->m_fOuterHeight;
	pxToSound->m_iPriority = pxFromSound->m_iPriority;
	if(pxFromSound->m_pxClipVolume)
	{
		if(!pxToSound->m_pxClipVolume) pxToSound->m_pxClipVolume = new Asura_VolumeGroup;
		pxToSound->m_pxClipVolume->Copy(pxFromSound->m_pxClipVolume);

		pxToSound->m_pxClipVolume->DisplaceVolumes(Position-pxObject->Position);
	}
	else
	{
		delete pxToSound->m_pxClipVolume;
		pxToSound->m_pxClipVolume = 0;
	}

	Asura_StreamingBackgroundSound_System::InvalidateVolumeTree();

	Tools_SaveAsura::SetSoundsModified();
}



bool Tools_StreamingBackgroundSound_Object::GetParameter(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam) const
{
	if(uParamID>=g_uHash_LightSource_LightSet0 && uParamID<=g_uHash_LightSource_LightSetLast)
	{
		u_int uBit = 1 << (uParamID - g_uHash_LightSource_LightSet0);
		pxParam->SetBool((m_uLightSets & uBit) ? true : false);

		return true;
	}


	switch(uParamID)
	{
		case g_uHash_Entity_StartsActive :
			pxParam->SetBool((m_uSoundFlags & ASURA_STREAMINGBACKGROUNDSOUNDFLAG_ACTIVE) ? true : false);
			return true;

		case g_uHash_StreamingSound_Type :
			pxParam->SetInt(m_iType);
			return true;

		case g_uHash_StreamingSound_UseClipVolume :
			pxParam->SetBool((m_uSoundFlags & ASURA_STREAMINGBACKGROUNDSOUNDFLAG_USECLIPVOLUME) ? true : false);
			return true;

		case g_uHash_StreamingSound_InvertClipVolume :
			pxParam->SetBool((m_uSoundFlags & ASURA_STREAMINGBACKGROUNDSOUNDFLAG_INVERTCLIPVOLUME) ? true : false);
			return true;


		case g_uHash_StreamingSound_UseOrient :
		{
			pxParam->SetBool(m_bUseOrient);
			return true;
		}

		case g_uHash_StreamingSound_InnerRadius :
			pxParam->SetFloat(m_fInnerRadius);
			return true;

		case g_uHash_StreamingSound_OuterRadius :
			pxParam->SetFloat(m_fOuterRadius);
			return true;

		case g_uHash_StreamingSound_InnerHeight :
			pxParam->SetFloat(m_fInnerHeight);
			return true;

		case g_uHash_StreamingSound_OuterHeight :
			pxParam->SetFloat(m_fOuterHeight);
			return true;

		case g_uHash_StreamingSound_Priority :
			pxParam->SetInt(m_iPriority);
			return true;



	}

	return false;
}

bool Tools_StreamingBackgroundSound_Object::SetParameter(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	if(uParamID>=g_uHash_LightSource_LightSet0 && uParamID<=g_uHash_LightSource_LightSetLast)
	{
		u_int uBit = 1 << (uParamID - g_uHash_LightSource_LightSet0);
		bool bSet = pxParam->GetBool();
		if(bSet)
			m_uLightSets |= uBit;
		else
			m_uLightSets &=~ uBit;

		m_bChanged = true;
		return true;
	}

	switch(uParamID)
	{
		case g_uHash_Entity_StartsActive :
		{
			if(pxParam->GetBool())
			{
				m_uSoundFlags |= ASURA_STREAMINGBACKGROUNDSOUNDFLAG_ACTIVE;
			}
			else
			{
				m_uSoundFlags &=~ ASURA_STREAMINGBACKGROUNDSOUNDFLAG_ACTIVE;
			}
			m_bChanged = true;
			
			return true;
		}

		case g_uHash_StreamingSound_Type :
		{
			m_iType = pxParam->GetInt();
			m_bChanged = true;
			
			return true;
		}

		case g_uHash_StreamingSound_UseClipVolume :
		{
			if(pxParam->GetBool())
			{
				m_uSoundFlags |= ASURA_STREAMINGBACKGROUNDSOUNDFLAG_USECLIPVOLUME;
			}
			else
			{
				m_uSoundFlags &=~ ASURA_STREAMINGBACKGROUNDSOUNDFLAG_USECLIPVOLUME;
			}
			m_bChanged = true;
			
			return true;
		}

		case g_uHash_StreamingSound_InvertClipVolume :
		{
			if(pxParam->GetBool())
			{
				m_uSoundFlags |= ASURA_STREAMINGBACKGROUNDSOUNDFLAG_INVERTCLIPVOLUME;
			}
			else
			{
				m_uSoundFlags &=~ ASURA_STREAMINGBACKGROUNDSOUNDFLAG_INVERTCLIPVOLUME;
			}
			m_bChanged = true;
			
			return true;
		}

		case g_uHash_StreamingSound_UseOrient :
		{
			m_bUseOrient = pxParam->GetBool();
			m_bChanged = true;

			return true;
		}

		case g_uHash_StreamingSound_InnerRadius :
		{
			m_fInnerRadius = pxParam->GetFloat();
			m_fOuterRadius = Asura_Maths::Max(m_fInnerRadius,m_fOuterRadius);
			m_bChanged = true;
			
			return true;
		}

		case g_uHash_StreamingSound_OuterRadius :
		{
			m_fOuterRadius = pxParam->GetFloat();
			m_fInnerRadius = Asura_Maths::Min(m_fInnerRadius,m_fOuterRadius);
			m_bChanged = true;
			
			return true;
		}

		case g_uHash_StreamingSound_InnerHeight :
		{
			m_fInnerHeight = pxParam->GetFloat();
			m_fOuterHeight = Asura_Maths::Max(m_fInnerHeight,m_fOuterHeight);
			m_bChanged = true;
			
			return true;
		}

		case g_uHash_StreamingSound_OuterHeight :
		{
			m_fOuterHeight = pxParam->GetFloat();
			m_fInnerHeight = Asura_Maths::Min(m_fInnerHeight,m_fOuterHeight);
			m_bChanged = true;
			
			return true;
		}


		case g_uHash_StreamingSound_Priority :
		{
			m_iPriority = pxParam->GetInt();
			m_bChanged = true;
			
			return true;
		}

	}
	
	return false;
}


bool Tools_StreamingBackgroundSound_Object::IsSelectable()
{
	return GetSound()->IsMemberOfTemplateInstance() ? false : Tools_Object::IsSelectable();
}

bool Tools_StreamingBackgroundSound_Object::IsHidden()
{
	//Sound may be hidden if not a member of any visible light set
	if(Tools_Object_System::IsHideByLightSetEnabled())
	{
		if(!(m_uLightSets & Tools_Object_System::GetVisibleLightSets())) return true;
	}

	if(Tools_LightSets::ShouldHideObjectsInOtherSets())
	{
		if(!Tools_LightSets::IsMemberOfCurrentLightSets(m_uLightSets)) return true;
	}


	return Tools_Object::IsHidden();
}

Asura_Guid Tools_StreamingBackgroundSound_Object::GetGuid() const
{
	Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);
	return pxSound->m_uGuid;
}

bool Tools_StreamingBackgroundSound_Object::AcquireGuid()
{
	Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);
	if(pxSound->m_uGuid == ASURA_GUID_UNREGISTERED)
	{
		//Don't allow a guid to be acquired if the main entity file is readonly.
		//This is needed in case other files are wanting to store this guid
		//which won't remain fixed unless the entity file is saved
		
		//Only assign a guid if the sound file is writeable 
		if(!CanSaveFile() && !Tools_SaveAsura::IsTemplateEditing())
		{
			ShowMessageOverlay("Can't assign guid to sound object because the sound file isn't checked out.");
			return false;
		}

		Tools_GuidRange eRange;
		if(Tools_SaveAsura::IsTemplateEditing())
		{
			//Using the main entity range for template files , to simplify the saving code.
			//Everything is in the same file in that case anyway , so the different ranges aren't needed
			eRange = TOOLS_GUIDRANGE_ENTITY;
		}
//		else if(BelongsToMissionFile())
//		{
//			eRange = TOOLS_GUIDRANGE_MISSIONSOUND;
//		}
		else
		{
			eRange = TOOLS_GUIDRANGE_SOUND;
		}
		Asura_Guid uGuid = Asura_Entity_System::GetCreatedByToolsGuid(eRange);

		
		
		//create a controller entity ,and associate it with this sound

		Asura_ServerEntity_StreamingBackgroundSoundController* pxEntity = new Asura_ServerEntity_StreamingBackgroundSoundController(uGuid);
		pxEntity->SetSoundID(pxSound->m_uID);

		pxEntity->ToolsData = new Tools_Entity_Data;
		pxEntity->ToolsData->Name = "Streaming Sound";

		pxSound->m_uGuid = uGuid;

		Tools_SaveAsura::SetSoundsModified();
	}
	else
	{
		Asura_Assert(0,"Shouldn't be using AcquireGuid() on an object that already has one.");
	}
	return true;
}

bool Tools_StreamingBackgroundSound_Object::HandlesThisMessage(int iMessageID)
{
	return (iMessageID == ASURA_MSG_ACTIVATE) || (iMessageID == ASURA_MSG_DEACTIVATE);
}

void Tools_StreamingBackgroundSound_Object::SyncWithControllerEntities()
{

	//search for any controller entities that correspond to sounds that no longer exist
	for(Asura_Hashed_List_It<Asura_ServerEntity_StreamingBackgroundSoundController> xIt(Asura_ServerEntity_StreamingBackgroundSoundController::GetList()); !xIt.Done(); xIt.Next())
	{
		Asura_ServerEntity_StreamingBackgroundSoundController* pxEntity = xIt.GetCurrent();
	
		//any matching attractor?
		u_int uID = pxEntity->GetSoundID();
		Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSoundByID(uID);

		if(pxSound)
		{
			if(pxSound->m_uGuid != pxEntity->GetGuid())
			{
				//Sound exists ,but isn't assigned to the same controller.
				//Reassign if the sound isn't supposed to be connected to a different entity
				if(pxSound->m_uGuid == ASURA_GUID_UNREGISTERED)
				{
					pxSound->m_uGuid = pxEntity->GetGuid();
					Tools_SaveAsura::SetSoundsModified();
				}
				else
				{
					pxEntity->SetAsDestroyed();
					Tools_SaveAsura::SetEntityModified();
				}
			}
		}
		else
		{
			pxEntity->SetAsDestroyed();
			Tools_SaveAsura::SetEntityModified();
		}
	}

	//search the sounds for any that refer to controller entities
	for(int iIndex=0 ; iIndex<Asura_StreamingBackgroundSound_System::s_iNumberOfSounds; iIndex++)
	{
		Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(iIndex);
		Asura_Guid uGuid = pxSound->m_uGuid;
		if(uGuid != ASURA_GUID_UNREGISTERED)
		{
			Asura_ServerEntity_StreamingBackgroundSoundController* pxEntity = Asura_ServerEntity_StreamingBackgroundSoundController::FindEntity(uGuid);

			if(pxEntity)
			{
				//make sure the entity agrees about the ID of the sound that it is attached to
				pxEntity->SetSoundID(pxSound->m_uID);
			}
			else
			{
				//entity nolonger exists ,so remove the guid
				pxSound->m_uGuid = ASURA_GUID_UNREGISTERED;
				Tools_SaveAsura::SetSoundsModified();
			}
		}
	}
}



bool Tools_StreamingBackgroundSound_Object::IsSphere() const
{
	return (m_iType == ASURA_STREAMINGBACKGROUNDSOUNDTYPE_SPHERE);
}

bool Tools_StreamingBackgroundSound_Object::IsCylinder() const
{
	return (m_iType == ASURA_STREAMINGBACKGROUNDSOUNDTYPE_CYLINDER);
}

bool Tools_StreamingBackgroundSound_Object::IsCuboid() const
{
	return (m_iType == ASURA_STREAMINGBACKGROUNDSOUNDTYPE_CUBOID);
}


bool Tools_StreamingBackgroundSound_Object::IsEnabled_Range()
{
	return m_iType == ASURA_STREAMINGBACKGROUNDSOUNDTYPE_SPHERE || m_iType == ASURA_STREAMINGBACKGROUNDSOUNDTYPE_CYLINDER;
}

bool Tools_StreamingBackgroundSound_Object::IsEnabled_Height()
{
	return m_iType == ASURA_STREAMINGBACKGROUNDSOUNDTYPE_CYLINDER;
}

bool Tools_StreamingBackgroundSound_Object::IsEnabled_Box()
{
	return m_iType == ASURA_STREAMINGBACKGROUNDSOUNDTYPE_CUBOID;
}

Asura_Bounding_Box Tools_StreamingBackgroundSound_Object::GetInnerExtents() const
{
	if(IsSphere())
	{
		Asura_Bounding_Box xBox;
		xBox.SetToPoint(Position);
		xBox.Grow(m_fInnerRadius);

		return xBox;
	}
	else if(IsCylinder())
	{
		// Calculate a bounding box for a cylinder oriented up the y-axis.
		Asura_Bounding_Box xBB;
		xBB.MinX = -m_fInnerRadius;
		xBB.MaxX = m_fInnerRadius;
		xBB.MinY = -m_fInnerHeight/2.0f;
		xBB.MaxY = m_fInnerHeight/2.0f;
		xBB.MinZ = -m_fInnerRadius;
		xBB.MaxZ = m_fInnerRadius;
		// Rotate it.
		if(m_bUseOrient)
		{
			xBB.FromOBB(xBB, Orientation);
		}
		// Displace to cylinder position.
		xBB.Displace(Position);
		return xBB;
	}
	else
	{
		return m_xInnerBox;
	}
}

Asura_Bounding_Box Tools_StreamingBackgroundSound_Object::GetOuterExtents() const
{
	if(IsSphere())
	{
		Asura_Bounding_Box xBox;
		xBox.SetToPoint(Position);
		xBox.Grow(m_fOuterRadius);

		return xBox;
	}
	else if(IsCylinder())
	{
		// Calculate a bounding box for a cylinder oriented up the y-axis.
		Asura_Bounding_Box xBB;
		xBB.MinX = -m_fOuterRadius;
		xBB.MaxX = m_fOuterRadius;
		xBB.MinY = -m_fOuterHeight/2.0f;
		xBB.MaxY = m_fOuterHeight/2.0f;
		xBB.MinZ = -m_fOuterRadius;
		xBB.MaxZ = m_fOuterRadius;
		// Rotate it.
		if(m_bUseOrient)
		{
			xBB.FromOBB(xBB, Orientation);
		}
		// Displace to cylinder position.
		xBB.Displace(Position);
		return xBB;
	}
	else
	{
		return m_xOuterBox;
	}
}


void Tools_StreamingBackgroundSound_Object::EditClipVolume()
{
	Asura_StreamingBackgroundSound* pxSound = Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);

	if(!pxSound->m_pxClipVolume) pxSound->m_pxClipVolume = new Asura_VolumeGroup;
	

	if(!m_pxClipVolumeController)
	{
		m_pxClipVolumeController = new Tools_Object_VolumeGroupEdit_Controller(this, pxSound->m_pxClipVolume, &m_bChanged);

		if(!(m_uSoundFlags & ASURA_STREAMINGBACKGROUNDSOUNDFLAG_USECLIPVOLUME))
		{
			m_uSoundFlags |= ASURA_STREAMINGBACKGROUNDSOUNDFLAG_USECLIPVOLUME;
			m_bChanged = true;
		}
	}
	m_pxClipVolumeController->StartVolumeEditing();
}

bool Tools_StreamingBackgroundSound_Object::CanSaveFile() const
{
	return Tools_SaveAsura::CanSaveFile(Tools_SaveAsura::GetSoundsFilename(), false);
}


/////////////////////////////////////////////////////////////////////////////////////////////////

Tools_StreamingVolumeEdit_Object::Tools_StreamingVolumeEdit_Object(const char* pszName,Asura_Bounding_Box* pxBox,bool* pbModifiedFlag,bool bOuter)
:Tools_VolumeEdit_Object(pszName,pxBox,0,pbModifiedFlag)
{
	m_bOuter = bOuter;
}


void Tools_StreamingVolumeEdit_Object::UpdateWhileSelected()
{
	Tools_VolumeEdit_Object::UpdateWhileSelected();

	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{


 		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && !Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
		{

            if ( Asura_UserInput::KeyPressed( ASURA_KEYID_ALT ) )
            {
			    if(Asura_UserInput::DebouncedKeyPressed('9'))
			    {
					SnapToVolume(false);
			    }
			    if(Asura_UserInput::DebouncedKeyPressed('0'))
			    {
				    SnapToVolume(true);
			    }
            }
		}

	}
}

void Tools_StreamingVolumeEdit_Object::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Function_Dialog("Snap In",xOffset,SnapIn,(int) this);
	pxDialog->SetHelpText("Snaps the selected face in to line up with the next sound volume in that direction.\nShortcut : Alt + 9.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Snap Out",xOffset,SnapOut,(int) this);
	pxDialog->SetHelpText("Snaps the selected face out to line up with the next sound volume in that direction.\nShortcut : Alt + 0.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	if(m_bOuter)
	{
		pxDialog = new Asura_Function_Dialog("Set To Inner",xOffset,SetToOtherVolume,(int) this);
		pxDialog->SetHelpText("Sets this volume to be the same as the inner volume.");
	}
	else
	{
		pxDialog = new Asura_Function_Dialog("Set To Outer",xOffset,SetToOtherVolume,(int) this);
		pxDialog->SetHelpText("Sets this volume to be the same as the outer volume.");
	}
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();



	Tools_VolumeEdit_Object::AddOverlayControls(pxOverlay,xOffset);
}

void Tools_StreamingVolumeEdit_Object::SnapToVolume(bool bOut)
{
	if(m_eSelectedFace == FACE_NONE)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No face has been selected.");
		return;
	}

	//if(m_bUseOrientation) return;

	//min and max member pointers along the direction being moved
	float Asura_Bounding_Box::*apfSide[2];

	//min and max member pointers for the perpendicular directions
	float Asura_Bounding_Box::*apfMin[2];
	float Asura_Bounding_Box::*apfMax[2];

	//set up the member pointers according to which side is being moved
	switch(m_eSelectedFace)
	{
		case FACE_MIN_X :
		case FACE_MAX_X :
			apfSide[0] = &Asura_Bounding_Box::MinX;
			apfSide[1] = &Asura_Bounding_Box::MaxX;
			apfMin[0] = &Asura_Bounding_Box::MinY;
			apfMax[0] = &Asura_Bounding_Box::MaxY;
			apfMin[1] = &Asura_Bounding_Box::MinZ;
			apfMax[1] = &Asura_Bounding_Box::MaxZ;
			break;
		case FACE_MIN_Y :
		case FACE_MAX_Y :
			apfSide[0] = &Asura_Bounding_Box::MinY;
			apfSide[1] = &Asura_Bounding_Box::MaxY;
			apfMin[0] = &Asura_Bounding_Box::MinX;
			apfMax[0] = &Asura_Bounding_Box::MaxX;
			apfMin[1] = &Asura_Bounding_Box::MinZ;
			apfMax[1] = &Asura_Bounding_Box::MaxZ;
			break;
		case FACE_MIN_Z :
		case FACE_MAX_Z :
			apfSide[0] = &Asura_Bounding_Box::MinZ;
			apfSide[1] = &Asura_Bounding_Box::MaxZ;
			apfMin[0] = &Asura_Bounding_Box::MinY;
			apfMax[0] = &Asura_Bounding_Box::MaxY;
			apfMin[1] = &Asura_Bounding_Box::MinX;
			apfMax[1] = &Asura_Bounding_Box::MaxX;
			break;
	}

	float *pfCurrentValue;
	float fBest;
	bool bGreater;
	bool bMax;

	//determine if we are increasing or decreasing the selected value
	switch(m_eSelectedFace)
	{
		case FACE_MIN_X :
		case FACE_MIN_Y :
		case FACE_MIN_Z :
			pfCurrentValue = &(m_pxVolume->*apfSide[0]);
			bGreater = !bOut;
			bMax = false;
			break;

		case FACE_MAX_X :
		case FACE_MAX_Y :
		case FACE_MAX_Z :
			pfCurrentValue = &(m_pxVolume->*apfSide[1]);
			bGreater = bOut;
			bMax = true;
			break;

	}
	fBest = bGreater ? Asura_Maths::LargeFloat : -Asura_Maths::LargeFloat;
	bool bBestValid = false;

	Tools_StreamingBackgroundSound_Object* pxObject = (Tools_StreamingBackgroundSound_Object*) m_pxParentObject;

	for(LIF<Tools_StreamingBackgroundSound_Object*> xObjectLIF(&Tools_StreamingBackgroundSound_Object::s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
//		if(xObjectLIF() == m_pxParentObject) continue;

		//only snap against volumes with the same orient
		if(!(xObjectLIF()->m_xOrient == pxObject->m_xOrient)) continue;

		//check against the inner and outer volumes
		for(int iInner=0;iInner<2;iInner++)
		{
			const Asura_Bounding_Box* pxBB = iInner ? &xObjectLIF()->m_xInnerBox : &xObjectLIF()->m_xOuterBox;

			//check the overlap in the 2 axes perpendicular to the direction being moved
			bool bOverlap = true;
			
			for(int i=0;i<2;i++)
			{
				float fOverlap = Asura_Maths::Min(pxBB->*apfMax[i],m_pxVolume->*apfMax[i]) -
								 Asura_Maths::Max(pxBB->*apfMin[i],m_pxVolume->*apfMin[i]);

				float fSideLength = m_pxVolume->*apfMax[i] - m_pxVolume->*apfMin[i];

				//consider it to be an overlap along this axis if the overlap is greater than 10 cm , or
				//is at least half the side length.
				if(!(fOverlap >= 0.1f || fOverlap >= fSideLength * .5f))
				{
					bOverlap = false;
				}
					
			}

			if(bOverlap)
			{
				//check the min and max along the axis being moved to see if they are a better snap point
				for(int i=0;i<2;i++)
				{
					if(bGreater)
					{
						if(pxBB->*apfSide[i] > *pfCurrentValue && fBest > pxBB->*apfSide[i])
						{
							fBest = pxBB->*apfSide[i];
							bBestValid = true;
						}
					}
					else
					{
						if(pxBB->*apfSide[i] < *pfCurrentValue && fBest < pxBB->*apfSide[i])
						{
							fBest = pxBB->*apfSide[i];
							bBestValid = true;
						}
					}
				}	
			}
		}
	}

	if(bBestValid)
	{
		if(bMax && fBest <= m_pxVolume->*apfSide[0])
		{
			//Max would be altered to be less than the min
			bBestValid = false;
		}
		if(!bMax && fBest >= m_pxVolume->*apfSide[1])
		{
			//Min would be altered to be greater than the max
			bBestValid = false;
		}
	}	

	if(bBestValid)
	{
		*pfCurrentValue = fBest;
		m_xWorldVolume = *m_pxVolume;
		SetDefaultPositionFromVolume();
		if(m_pbModifiedFlag) *m_pbModifiedFlag = true;

	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"There is nothing in that direction that the selected face could be snapped to.");
	}
}

void Tools_StreamingVolumeEdit_Object::SetToOtherVolume()
{
	if(!m_pxParentObject) return;

	Tools_StreamingBackgroundSound_Object* pxObject = (Tools_StreamingBackgroundSound_Object*) m_pxParentObject;

	//set the volume to equal the other volume (inner/outer) that belongs to the sound object
	if(m_bOuter)
	{
		pxObject->m_xOuterBox = pxObject->m_xInnerBox;
	}
	else
	{
		pxObject->m_xInnerBox = pxObject->m_xOuterBox;
	}

	m_xWorldVolume = *m_pxVolume;
	SetDefaultPositionFromVolume();
	if(m_pbModifiedFlag) *m_pbModifiedFlag = true;

}

Asura_Vector_3 Tools_StreamingVolumeEdit_Object::GetRelativePivot() const
{
	if(!m_bOuter && m_pxParentObject)
	{
		//using the centre of the outer bounding box.
		Tools_StreamingBackgroundSound_Object* pxObject = (Tools_StreamingBackgroundSound_Object*) m_pxParentObject;
		Asura_Vector_3 xOffset = pxObject->m_xOuterBox.CalculateCentre()-Position;
		//xOffset = xOffset*Orientation;
		xOffset = Orientation.ApplyTranspose(xOffset);
		return xOffset;
	}
	else
	{
		return Tools_VolumeEdit_Object::GetRelativePivot();
	}
}
 
Asura_Vector_3 Tools_StreamingVolumeEdit_Object::GetWorldPivot() const
{
	if(!m_bOuter && m_pxParentObject)
	{
		//use the centre of the outer bounding box
		Tools_StreamingBackgroundSound_Object* pxObject = (Tools_StreamingBackgroundSound_Object*) m_pxParentObject;
		return pxObject->m_xOuterBox.CalculateCentre();
	}
	else
	{
		return Tools_VolumeEdit_Object::GetWorldPivot();
	}
}

void Tools_StreamingVolumeEdit_Object::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	if(m_bUseOrientation && !m_bOuter)
	{
		//update the relative volume from the world volume
		m_xRelativeVolume = m_xWorldVolume;
		m_xRelativeVolume.Displace(-Position);

		if(m_bUseOrientation)
		{
			Asura_Vector_3 xOffset = GetRelativePivot();
			m_xRelativeVolume.Displace(xOffset);
			xOffset = xOffset*Orientation;
			m_xRelativeVolume.Displace(-xOffset);
		}
		

	}
	Tools_VolumeEdit_Object::UpdatePositionInAsuraData(bUpdateLinkedObjects);
}


Asura_StreamingBackgroundSound* Tools_StreamingBackgroundSound_Object::GetSound()
{
	return Asura_StreamingBackgroundSound_System::GetSound(m_iIndex);
}

Tools_StreamingBackgroundSound_Object* Tools_StreamingBackgroundSound_Object::GetSoundObjectFromID(u_int uID)
{
	for(LIF<Tools_StreamingBackgroundSound_Object*> xObjectLIF(&s_xObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		if(xObjectLIF()->GetSound()->m_uID == uID)
		{
			return xObjectLIF();
		}
	}
	return  0;
}