
#include "Core/Asura.h"

#include "TextureTiler/Tools_TextureTiler_Render.h"

#include "Tools_Overlays.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Rendering/Asura_Render.h"

// <-- class Tools_TT_StringContainer -->	
Tools_TT_StringContainer::Tools_TT_StringContainer( const u_int uStringSize /* = 128 */ ):
	m_uStringSize( uStringSize ),
	m_uStringCollectionSize( 0 ),
	m_uStringCollectionCapacity( 0 ),
	m_ppszStringCollection(NULL)
{}

Tools_TT_StringContainer::~Tools_TT_StringContainer()
{
	if( m_ppszStringCollection )
	{
		for( u_int u(0);u<GetCapacity();++u )
		{
			delete [] m_ppszStringCollection[u];
		}

		delete [] m_ppszStringCollection;
	}
}


void Tools_TT_StringContainer::SetCapacity( const u_int uCapacity )
{
	if( GetCapacity() < uCapacity  )
	{
		ReplaceContainer( uCapacity );
	}

	m_uStringCollectionSize = 0;
}

const bool Tools_TT_StringContainer::Append( const char* szString )
{
	bool bRtn(false);

	if( Size() < GetCapacity() )
	{
		strncpy( m_ppszStringCollection[Size()],szString,m_uStringSize );
		++m_uStringCollectionSize;

		bRtn = true;
	}

	return bRtn;
}

void Tools_TT_StringContainer::Clear()
{
	m_uStringCollectionSize = 0;
}

void Tools_TT_StringContainer::ReplaceContainer( const u_int uSize )
{
	if( m_ppszStringCollection )
	{
		// delete old strings 
		for( u_int u(0);u<GetCapacity();++u )
		{
			delete [] m_ppszStringCollection[u];
		}

		// delete old string container 
		delete [] m_ppszStringCollection;
	}

	// create new string container
	m_uStringCollectionCapacity = uSize;

	m_ppszStringCollection = new char*[GetCapacity()];

	for( u_int u(0);u<GetCapacity();++u )
	{
		m_ppszStringCollection[u] = new char[m_uStringSize];
	}

	m_uStringCollectionSize = 0;
}

// <-- class numerical dialog -->
Tools_TT_OverlayNumericalDialog::Tools_TT_OverlayNumericalDialog( const int iValue /* = 0 */):
	m_iValue(iValue),
	m_bChanged(true)
{}

Asura_Numerical_Dialog* Tools_TT_OverlayNumericalDialog::CreateAsuraDialog( const char* pcName, const Asura_Vector_2& xOffset, int iMin, int iMax,int iDelta, const char* pcPrintFormat )
{
	return new Asura_Numerical_Dialog( pcName,xOffset,&m_iValue,iMin,iMax,iDelta,pcPrintFormat,&m_bChanged );
}

// <-- class confirmation overlay -->
void(*Tools_TT_ConfirmationOverlay::s_pxAction) (void);
void(*Tools_TT_ConfirmationOverlay::s_pxOnExit) (void);
// float Tools_TT_ConfirmationOverlay::s_fOverlayWidth(30.0f);
bool Tools_TT_ConfirmationOverlay::s_bIsActive(false);

void Tools_TT_ConfirmationOverlay::Create( const Asura_String& xText,void(*pxAction) (void),void(*pxOnExit) (void) /* = NULL */ )
{
	if( s_bIsActive )
	{
		return;
	}

	s_pxAction = pxAction;
	s_pxOnExit = pxOnExit;

	Asura_Overlay* pxOverlay = new Asura_Overlay( "Are You Sure?",true );
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);
	
	pxDialog = new Asura_Dialog( xText.GetString(),xOffset );
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Yes",xOffset,&Yes );
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Function_Dialog("No",xOffset,&No );
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//pxDialog = new Asura_Tickbox_Dialog("Don't show this message again",xOffset,&s_abOptionFlag[TOOLS_TT_DONTSHOW_REMOVE_CONFIRMATION] );
	//pxDialog->SetHelpText("Check the box to prevent this confirmation message from being shown again");
	//pxOverlay->AddDialog(pxDialog);
	//xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxOverlay->m_uID = TOOLS_OVERLAY_TT_CONFIRMATION;

	Asura_Overlay_System::SubmitNewOverlay( pxOverlay,false );
		
}

void Tools_TT_ConfirmationOverlay::Yes()
{
	(*(s_pxAction))();
	Exit();
}

void Tools_TT_ConfirmationOverlay::Exit()
{
	s_bIsActive = false;	
	Asura_Overlay_System::CloseOverlays( TOOLS_OVERLAY_TT_CONFIRMATION );

	if( s_pxOnExit )
	{
		(*(s_pxOnExit))();
	}
}

void Tools_TT_ConfirmationOverlay::No()
{
	Exit();
}

// <-- class overlay status -->
Tools_TT_OverlayStatus::Tools_TT_OverlayStatus():
	m_xPosition(),
	m_bRequested(false), 			
	m_bCreated(false),			
	m_bMinimised(false)
{}

// <-- Render methods --> 
void Tools_TT_RenderTexture( const Asura_Vector_2& xTopLeft,const Asura_Vector_2& xBottomRight,const int iTextureID,const int iBackgroundColour /* = Tools_TT_Colour::Null */,const ASURA_TRANSLUCENCY_MODE eTranslucency /* = ASURA_TRANSLUCENCY_OFF */ )
{
	const float fDeltaX( xBottomRight.x - xTopLeft.x );
	const float fDeltaY( xBottomRight.y - xTopLeft.y );
	
	//draw background
	Asura_Render::Overlay.SetRectXY( xTopLeft.x,xTopLeft.y,fDeltaX,fDeltaY );
	Asura_Render::Overlay.SetRectUV( 0,0,1,1 );
	Asura_Render::Overlay.TranslucencyMode = eTranslucency;
	Asura_Render::Overlay.TextureIndex     = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Colour           = iBackgroundColour;
	Asura_Render::Overlay.Render();
	
	Asura_Render::FlushDisplay();

	//draw the texture.
	Asura_Render::Overlay.TextureIndex = iTextureID;
	Asura_Render::Overlay.Colour       = 0xffffffff;
	Asura_Render::Overlay.SetRectXY( xTopLeft.x,xTopLeft.y,fDeltaX,fDeltaY );
	Asura_Render::Overlay.SetRectUV(0,0,1,1);
   	Asura_Render::Overlay.Render();
	
   	Asura_Render::FlushDisplay();
}

void Tools_TT_RenderTexture( const Asura_Bounding_Rectangle& xRectangle,const Asura_Bounding_Rectangle& xUV,const int iTextureID,const int iBackgroundColour /* = Tools_TT_Colour::Null */,const ASURA_TRANSLUCENCY_MODE eTranslucency /* = ASURA_TRANSLUCENCY_OFF */ )
{
	const float fDeltaX( xRectangle.GetWidth() );
	const float fDeltaY( xRectangle.GetHeight() );
	
	const float fDeltaU( xUV.GetWidth() );
	const float fDeltaV( xUV.GetHeight() );

	//draw background
	Asura_Render::Overlay.SetRectXY( xRectangle.MinX,xRectangle.MinY,fDeltaX,fDeltaY );
	Asura_Render::Overlay.SetRectUV( 0,0,1,1 );
	Asura_Render::Overlay.TranslucencyMode = eTranslucency;
	Asura_Render::Overlay.TextureIndex     = ASURA_TEXTURE_NONE;
	Asura_Render::Overlay.Colour           = iBackgroundColour;
	Asura_Render::Overlay.Render();
	
	Asura_Render::FlushDisplay();

	//draw the texture.
	Asura_Render::Overlay.TextureIndex = iTextureID;
	Asura_Render::Overlay.Colour       = 0xffffffff;
	Asura_Render::Overlay.SetRectXY( xRectangle.MinX,xRectangle.MinY,fDeltaX,fDeltaY );
	Asura_Render::Overlay.SetRectUV( xUV.MinX,xUV.MinY,fDeltaU,fDeltaV );
   	Asura_Render::Overlay.Render();
	
   	Asura_Render::FlushDisplay();
}


void Tools_TT_RenderTexture( Asura_Bounding_Rectangle xRectangle,const Tools_TT_Tile& xTexture,const bool bFixAspectRatio /* = false */,const int iTextureID,const int iBackgroundColour /* = Tools_TT_Colour::Null */,const ASURA_TRANSLUCENCY_MODE eTranslucency /* = ASURA_TRANSLUCENCY_OFF */ )
{
	if(bFixAspectRatio)
	{
		Tools_TT_ScaleRectangleToFixAspectRatio(xRectangle,xTexture);
	}

	Tools_TT_RenderTexture( xRectangle.GetTopLeft(),xRectangle.GetBottomRight(),xTexture.GetTextureID(),iBackgroundColour,eTranslucency );
}

void Tools_TT_RenderUIBorder( const Tools_TT_UIComponent& xUIComponent )
{
	Asura_Bounding_Rectangle xBoarder( xUIComponent.GetInnerRectangle() );

	xBoarder.MinX--;
	xBoarder.MinY--;
	xBoarder.MaxX++;
	xBoarder.MaxY++;

	Tools_TT_RenderBorder( xBoarder,xUIComponent.GetBorderColour() );
}

void Tools_TT_RenderBorder( const Asura_Vector_2& xTopLeft,const Asura_Vector_2& xBottomRight, const u_int uColour )
{
	// top left to bottom left
	Asura_Render::Render2DLine( xTopLeft.x,xTopLeft.y,xTopLeft.x,xBottomRight.y,uColour );
	// bottom left to bottom right
	Asura_Render::Render2DLine( xTopLeft.x,xBottomRight.y,xBottomRight.x,xBottomRight.y,uColour );
	// bottom right to top right
	Asura_Render::Render2DLine( xBottomRight.x,xBottomRight.y,xBottomRight.x,xTopLeft.y,uColour );
	// top right to top left
	Asura_Render::Render2DLine( xBottomRight.x,xTopLeft.y,xTopLeft.x,xTopLeft.y,uColour );
}

void Tools_TT_RenderBorder( const Asura_Bounding_Rectangle& xRectangle, const u_int uColour )
{
	// top left to bottom left
	Asura_Render::Render2DLine( xRectangle.MinX,xRectangle.MinY,xRectangle.MinX,xRectangle.MaxY,uColour );
	// bottom left to bottom right
	Asura_Render::Render2DLine( xRectangle.MinX,xRectangle.MaxY,xRectangle.MaxX,xRectangle.MaxY,uColour );
	// bottom right to top right
	Asura_Render::Render2DLine( xRectangle.MaxX,xRectangle.MaxY,xRectangle.MaxX,xRectangle.MinY,uColour );
	// top right to top left
	Asura_Render::Render2DLine( xRectangle.MaxX,xRectangle.MinY,xRectangle.MinX,xRectangle.MinY,uColour );
}

void Tools_TT_RenderTTL( const Asura_Bounding_Rectangle& xTTLScreenRectangle,const Tools_TT_TiledTextureList& xTTL,const int iBackgroundColour /* = Tools_TT_Colour::Null */,const ASURA_TRANSLUCENCY_MODE eTranslucency /* = ASURA_TRANSLUCENCY_OFF */ )
{	
	for( u_int u(0); u < xTTL.Size(); ++u )
	{
		xTTL[u].Render( xTTLScreenRectangle,iBackgroundColour,eTranslucency );
	}
}

void Tools_TT_RenderTTLSubTextureHighlight( const Asura_Bounding_Rectangle& xTTLScreenRectangle, const Tools_TT_Tile& xSubtexture,const int iColour )
{		
	// convert UV rectangle to screen coordinates
	Asura_Bounding_Rectangle xRectangle( xSubtexture.GetUVRectangle() );

	xRectangle.MinX *= xTTLScreenRectangle.GetWidth();
	xRectangle.MinX += xTTLScreenRectangle.MinX;
	
	xRectangle.MinY *= xTTLScreenRectangle.GetHeight(); 
	xRectangle.MinY += xTTLScreenRectangle.MinY;

	xRectangle.MaxX *= xTTLScreenRectangle.GetWidth(); 
	xRectangle.MaxX += xTTLScreenRectangle.MinX;

	xRectangle.MaxY *= xTTLScreenRectangle.GetHeight(); 
	xRectangle.MaxY += xTTLScreenRectangle.MinY;
	
	Tools_TT_RenderBorder( xRectangle,iColour );
}

