//
// Tools_Overlay_ViewTextures.cpp
//
// Created by Richard Rice - 12/09/2007
// 
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"
#include "Texturing/Tools_Overlay_ViewTextures.h"


//asura
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_UserInput.h"
#include "Core/Asura_Timers.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Mesh/Asura_Env_System.h"
#include "Mesh/Asura_Env.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Module.h"
#include "Rendering/Asura_Viewport.h"
#include "Rendering/Asura_Text_Output.h"

//tools
#include "Tools_Overlays.h"
#include "GUIMenu/Tools_GUIMenu_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Materials/Tools_MaterialResponse_System.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Skybox/Tools_Mode_Skybox.h"
#include "Texturing/Tools_PickTexture.h"
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
#include "Texturing/Tools_Texturing_MaterialResponse.h"
#endif
#include "Texturing/Tools_Texturing_Find.h"
#include "Texturing/Tools_Mode_Texturing.h"
#include "Texturing/Tools_Texture_Management.h"


struct Tools_MaterialPolygonCount
{
	int m_iPolygonCount;
	bool m_bCurrentLightSet;
};

enum 
{
	TOOLS_OVT_SORT_TEXTURENAME,
	TOOLS_OVT_SORT_POLYGONS,
	TOOLS_OVT_SORT_SIZE,

	TOOLS_OVT_NUM_SORT_MODES,
};

static const char* g_aszSortModeStrings[TOOLS_OVT_NUM_SORT_MODES] =
{
	"Name",
	"Polygons",
	"Size",
};

enum 
{
	TOOLS_OVT_FLAGMODE_SHOW,
	TOOLS_OVT_FLAGMODE_HIGHLIGHT,

	TOOLS_OVT_NUM_FLAGMODE,
};

static const char* g_aszFlagModeStrings[TOOLS_OVT_NUM_FLAGMODE] =
{
	"Show",
	"Highlight",
};


static const char* g_aszTextureUsageStrings[TOOLS_OVT_NUMUSAGE] = 
{
	"*Any*",
	"Environment",
	"Light Set",
	"Cubemap",
	"GUI Menu",
	"Skybox",

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	"Albedo",
	"Normal Map",
	"Specular Map",
	"Transparency Map",
	"Self Illumination",
	"Environment Map",
	"Reflectivity",
	"Parallax Map",
	"Detail Map",
	"Detail Normal Map",
	#endif
};

static const char* g_aszTextureUsageHelpStrings[TOOLS_OVT_NUMUSAGE] = 
{
	"Show all loaded textures",
	"Show textures used on environment polygons",
	"Show textures used on environment polygons in the current light set",
	"Show cubemap textures",
	"Show textures used by the GUIMenu system",
	"Show textures used on the skybox",

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	"Show textures used in this manner by material response",
	"Show textures used in this manner by material response",
	"Show textures used in this manner by material response",
	"Show textures used in this manner by material response",
	"Show textures used in this manner by material response",
	"Show textures used in this manner by material response",
	"Show textures used in this manner by material response",
	"Show textures used in this manner by material response",
	"Show textures used in this manner by material response",
	"Show textures used in this manner by material response",
	#endif
};


static const int g_iNumTextureFlagStrings = 23;
static const char* g_aszTextureFlagStrings[g_iNumTextureFlagStrings] = 
{
	"*Any*",
	"8 Bit Alpha",
	"16 Colour Palette",
	"PS2 Moire Fix",
	"No Mip Maps",
	"Wii Only No Mip Maps",
	"PS2 Distant Texture",
	"Full Motion Video",
	"PS2 Allow 512",
	"PS2 Allow Mipmaps on 512",
	"PS2 Don't Halve Size",
	"PS2 Don't Halve Extra For Multiplayer",
	"Reduce extra level for consoles",
	"PS2 Sharpen",
	"PS2 Sharpen More",
	"XBox Half Size",
	"PSP Half Size",
	"Accept Carpet",
	"AD Receiver",
	"High-Quality Compression",
	"Sharpen Mip Maps",
	"Clamp-U",
	"Clamp-V"
};

static int g_aiTextureFlagValues[g_iNumTextureFlagStrings] =
{
	0,
	AsuraTextureFlag_8BitAlpha,
	AsuraTextureFlag_16ColourPalette,
	AsuraTextureFlag_PS2MoireFix,
	AsuraTextureFlag_NoMipMaps,
	AsuraTextureFlag_WIIOnly_NoMipMaps,
	AsuraTextureFlag_PS2DistantTexture,
	AsuraTextureFlag_FMV,
	AsuraTextureFlag_PS2Allow512,
	AsuraTextureFlag_PS2AllowMipMapsOn512,
	AsuraTextureFlag_PS2DontHalveSize,
	AsuraTextureFlag_PS2DontHalveExtraForMultiplayer,
	AsuraTextureFlag_ReduceExtraForConsoles,
	AsuraTextureFlag_Sharpen,
	AsuraTextureFlag_SharpenMore,
	AsuraTextureFlag_HalfForXBox,
	AsuraTextureFlag_HalfForPSP,
	AsuraTextureFlag_AcceptCarpet,
	AsuraTextureFlag_AdReceiver,
	AsuraTextureFlag_HighQualityCompression,
	AsuraTextureFlag_SharpenMipMaps,
	AsuraTextureFlag_ClampU,
	AsuraTextureFlag_ClampV
};

static const char* g_aszTextureFlagHelpStrings[g_iNumTextureFlagStrings] = 
{
	0,
	//AsuraTextureFlag_8BitAlpha
	"For textures that want to use their alpha channel for more than on/off transparency.\nMay not actualy end up as 8 bits on all platforms",
	//AsuraTextureFlag_16ColourPalette,
	"PSP and PS2\nExport as a 4 bit palettised texture.\n",
	//AsuraTextureFlag_PS2MoireFix,
	"PS2 only\nCauses the first mip map to be a scaled up version of the second mip map.",
	//AsuraTextureFlag_NoMipMaps,
	"Prevent mip map generation",
	//AsuraTextureFlag_WIIOnly_NoMipMaps,
	"Wii only\nPrevent mip map generation.",
	//AsuraTextureFlag_PS2DistantTexture,
	"PS2 : Prevent mip map generation.\nPSP : Causes special handling in game for large objects in the distance",
	//AsuraTextureFlag_FMV,
	"Marks textures that are placeholders for FMV",
	//AsuraTextureFlag_PS2Allow512,
	"PSP and PS2\nOtherwise textures will be dropped down to 256x256",
	//AsuraTextureFlag_PS2AllowMipMapsOn512,
	"PS2 only\nOtherwise environment 512 textures won't generate mip maps",
	//AsuraTextureFlag_PS2DontHalveSize,
	"PS2 only\nPrevents the automatic halving of size upon export.",
	//AsuraTextureFlag_PS2DontHalveExtraForMultiplayer,
	"PS2 only\nPrevents texture being halved in size again in multiplayer exports.",
	//AsuraTextureFlag_ReduceExtraForConsoles,
	"All but PC\nHalf the texture size on export.",
	//AsuraTextureFlag_Sharpen,
	"PS2 only\nApplies sharpen filter on exported texture",
	//AsuraTextureFlag_SharpenMore,
	"PS2 only\nApplies sharpen filter on exported texture",
	//AsuraTextureFlag_HalfForXBox,
	"Appears to be used as a hacky alternative to high quality compression for XBox360?",
	//AsuraTextureFlag_HalfForPSP,
	"PSP only\nHalg the texture size on export",
	//AsuraTextureFlag_AcceptCarpet,
	"Used by carpet mode to determine which polygons carpet can be sprayed onto.",
	//AsuraTextureFlag_AdReceiver,
	"At a guess this is used for flagging textures that are placeholders for adverts",
	//AsuraTextureFlag_HighQualityCompression,
	"Texture may be exported as uncompressed or with less compression.\nExact effect depends on platform and type of texture",
	//AsuraTextureFlag_SharpenMipMaps,
	"PS3,Xbox360, PC\nSharpen mip map generation",
	//AsuraTextureFlag_ClampU,
	"PSP only\nPrevents UV coordinates from wrapping round horizontally",
	//AsuraTextureFlag_ClampV
	"PSP only\nPrevents UV coordinates from wrapping round vertically",
};



enum Tools_OVT_DisplayMode
{
	TOOLS_OVT_DISPLAYMODE_OPAQUE,
	TOOLS_OVT_DISPLAYMODE_ALPHAONLY,
	TOOLS_OVT_DISPLAYMODE_TRANSPARENT_BLACK,
	TOOLS_OVT_DISPLAYMODE_TRANSPARENT_MAGENTA,

	TOOLS_OVT_DISPLAYMODE_NUMMODES,
};

static const char* g_aszDisplayModeStrings[TOOLS_OVT_DISPLAYMODE_NUMMODES] =
{
	"Opaque",
	"Alpha Only",
	"Black Background",
	"Magenta Background",
};

static const char* g_aszDisplayModeHelpStrings[TOOLS_OVT_DISPLAYMODE_NUMMODES] =
{
	"Textures are rendered without the alpha channel",
	"Textures are rendered using just the alpha channel as greyscale",
	"Textures are rendered translucent using a black background",
	"Textures are rendered translucent using a magenta background",
};

static const char* g_aszViewTexturesHelpStrings [] =
{
	"LMB    : Select a texture",
	"+Shift : Select textures in between this and the previous one",
	"+Ctrl  : Enables multiple selection",
	0
};




#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
//Static assert checks that the enumation of texture usage is updated to include all the slots from Asura_MaterialResponse
ASURA_STATIC_ASSERT(TOOLS_OVT_USAGE_LASTMATERIALRESPONSE-TOOLS_OVT_USAGE_FIRSTMATERIALRESPONSE == Asura_MaterialResponse::NUMBER_OF_TEXTURE_SLOTS-1);
#endif



static float g_fLargeTextureWidth = 256.0f;
static float g_fLargeTextureHeight = 256.0f;
static float g_fThumbnailWidth = 64.0f;
static float g_fThumbnailHeight = 64.0f;
static float g_fThumbnailSpacing = 8.0f;
static int g_iThumbnailBorder = 2;

Tools_Overlay_ViewTextures* Tools_Overlay_ViewTextures::s_pxThis;
int Tools_Overlay_ViewTextures::s_iSortMode;
int Tools_Overlay_ViewTextures::s_iUsageFilter;
int Tools_Overlay_ViewTextures::s_iLastUsageFilter;
int Tools_Overlay_ViewTextures::s_iTextureFlagFilter;
int Tools_Overlay_ViewTextures::s_iTextureFlagMode;
int Tools_Overlay_ViewTextures::s_iDisplayMode;

int Tools_Overlay_ViewTextures::s_iResult = false;
int Tools_Overlay_ViewTextures::s_iSelectedTexture = ASURA_TEXTURE_NONE;

static Asura_Vector_2 s_xOverlayPosition(2.0f,3.0f);
static Asura_Vector_2 g_xUserSetSubwindowSize(0.0f,0.0f);

Tools_Overlay_ViewTextures::Tools_Overlay_ViewTextures()
:Asura_Overlay(" ",false, &s_xOverlayPosition)
{
	Asura_Assert(!s_pxThis,"");
	s_pxThis = this;

	SetDefaultTitle();

	m_pxLargeImageDialog = 0;
	m_pxSubwindowDialog = 0;
	m_pxTextureNameDialog = 0;
	m_pxTexturePathDialog = 0;
	m_pxTextureDimensionDialog = 0;
	m_pxPolygonsDialog = 0;
	m_fSubwindowWidth = 0.0f;
	m_pxCurrentTexture = 0;
	m_pxLastClickedTexture = 0;
	m_bUpdateThumbnails = false;
	for(int i=0; i<TOOLS_OVT_NUMFLAGTEXTDIALOG; i++)
	{
		m_apxFlagTextDialog[i] = 0;
	}
	m_iNumVisibleTextures = 0;
	m_ppxVisibleTextures = 0;

	m_pxFlagSharedValues = new Tools_OVT_SharedValue<bool>[g_iNumTextureFlagStrings];
	for(int i=0; i<g_iNumTextureFlagStrings; i++)
	{
		m_pxFlagSharedValues[i].SetParamID(i);
	}

	m_bUpdateThumbnails = false;
	m_bUpdateDisplayMode = false;
	m_bUsageFilterChanged = false;

	Tools_Help::AddHelpTopic("View Textures Overlay",g_aszViewTexturesHelpStrings);

}

Tools_Overlay_ViewTextures::~Tools_Overlay_ViewTextures()
{
	Tools_Help::RemoveHelpTopic("View Textures Overlay");

	delete [] m_ppxVisibleTextures;
	delete [] m_pxFlagSharedValues;

	Asura_Assert(s_pxThis == this,"");
	s_pxThis = 0;
}

void Tools_Overlay_ViewTextures::InitialiseOverlay()
{
	int iViewportWidth = Asura_Viewport::GetWidth();
	int iViewportHeight = Asura_Viewport::GetHeight();

	float fViewportPrintWidth = (iViewportWidth / ASURA_TEXT_OUTPUT_FONT_WIDTH);
	float fViewportPrintHeight = (iViewportHeight / ASURA_TEXT_OUTPUT_FONT_HEIGHT);

	Asura_Vector_2 xOffset(1.0f,3.0f);
	Asura_Dialog* pxDialog;

	//main image dialog , intially with no texture
	m_pxLargeImageDialog = new Asura_Image_Dialog(xOffset,ASURA_TEXTURE_NONE,g_fLargeTextureWidth,g_fLargeTextureHeight);
	m_pxLargeImageDialog->SetTranslucent(false);
	AddDialog(m_pxLargeImageDialog);

	//add some blank text dialogs to the side
	xOffset.x = m_pxLargeImageDialog->m_xMaxCoord.x + 1.0f;
	
	for(int i=0; i<TOOLS_OVT_NUMFLAGTEXTDIALOG; i++)
	{
		m_apxFlagTextDialog[i] = new Asura_Dialog("", xOffset);
		AddDialog(m_apxFlagTextDialog[i]);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}

	
	xOffset.y = m_pxLargeImageDialog->m_xMaxCoord.y - Asura_Dialog::DefaultSpacing()*4;

	m_pxPolygonsDialog = new Asura_Dialog("", xOffset);
	m_pxPolygonsDialog->SetHelpText("The number of environmental polygons using this texture");
	AddDialog(m_pxPolygonsDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_pxTextureDimensionDialog = new Asura_Dialog("", xOffset);
	AddDialog(m_pxTextureDimensionDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_pxTexturePathDialog = new Asura_Dialog("", xOffset);
	AddDialog(m_pxTexturePathDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	m_pxTextureNameDialog = new Asura_Dialog("", xOffset);
	AddDialog(m_pxTextureNameDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	//add a subwindow containing flag tickboxes to the right
	xOffset.x = m_pxLargeImageDialog->m_xMaxCoord.x + 20.0f;
	xOffset.y = m_pxLargeImageDialog->m_xOffset.y;

	pxDialog = new Asura_Dialog("Edit Texture Flags :", xOffset);
	AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_Subwindow_Dialog* pxSubwindowDialog = new Asura_Subwindow_Dialog(xOffset, 35.0f, 14.0f, false);
	AddDialog(pxSubwindowDialog);
	Asura_Vector_2 xFlagOffset(1.0f,1.0f);

	for(int i=1; i<g_iNumTextureFlagStrings; i++)
	{
		pxDialog = new Asura_SharedTickbox_Dialog( g_aszTextureFlagStrings[i], xFlagOffset, &m_pxFlagSharedValues[i]);
		pxDialog->SetHelpText(g_aszTextureFlagHelpStrings[i]);
		pxSubwindowDialog->AddDialog(pxDialog); 
		xFlagOffset.y += Asura_Dialog::DefaultSpacing();
	}




	//add sort and filter options below
	xOffset.x = 1.0f;
	xOffset.y = m_pxLargeImageDialog->m_xMaxCoord.y + 1.0f;


	pxDialog = new Asura_Dialog("Sort By :", xOffset);
	AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_aszSortModeStrings, &s_iSortMode, TOOLS_OVT_NUM_SORT_MODES, TOOLS_OVT_NUM_SORT_MODES, 0, &m_bUpdateThumbnails);
	((Asura_DropDown_List_Dialog*)pxDialog)->SetOnlyUpdateSelectionIndexOnClick(true);
	pxDialog->SetHelpText("Determines how the material thumbnails are sorted");
	AddDialog(pxDialog);
	

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Dialog("Usage :", xOffset);
	AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_aszTextureUsageStrings, &s_iUsageFilter, TOOLS_OVT_NUMUSAGE, TOOLS_OVT_NUMUSAGE, 0, &m_bUsageFilterChanged);
	((Asura_DropDown_List_Dialog*)pxDialog)->SetListHelpStrings(g_aszTextureUsageHelpStrings);
	((Asura_DropDown_List_Dialog*)pxDialog)->SetOnlyUpdateSelectionIndexOnClick(true);
	((Asura_DropDown_List_Dialog*)pxDialog)->SetSortStrings(true, 1);
	pxDialog->SetHelpText("Filter textures based on how they are used");
	AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Dialog("Flags :", xOffset);
	AddDialog(pxDialog);
	

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_aszFlagModeStrings, &s_iTextureFlagMode, TOOLS_OVT_NUM_FLAGMODE, TOOLS_OVT_NUM_FLAGMODE, 0, &m_bUpdateThumbnails);
	((Asura_DropDown_List_Dialog*)pxDialog)->SetOnlyUpdateSelectionIndexOnClick(true);
	pxDialog->SetHelpText("Determines whether the texture filter hides textures without the given flag , or highlights texture that have it");
	AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x;
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_aszTextureFlagStrings, &s_iTextureFlagFilter, g_iNumTextureFlagStrings, g_iNumTextureFlagStrings, 0, &m_bUpdateThumbnails);
	((Asura_DropDown_List_Dialog*)pxDialog)->SetOnlyUpdateSelectionIndexOnClick(true);
	((Asura_DropDown_List_Dialog*)pxDialog)->SetSortStrings(true, 1);
	pxDialog->SetHelpText("Filter textures based on texture flag");
	AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Dialog("Alpha :", xOffset);
	AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, g_aszDisplayModeStrings, &s_iDisplayMode, TOOLS_OVT_DISPLAYMODE_NUMMODES, TOOLS_OVT_DISPLAYMODE_NUMMODES, 0, &m_bUpdateDisplayMode );
	((Asura_DropDown_List_Dialog*)pxDialog)->SetListHelpStrings(g_aszDisplayModeHelpStrings);
	((Asura_DropDown_List_Dialog*)pxDialog)->SetOnlyUpdateSelectionIndexOnClick(true);
	pxDialog->SetHelpText("Determines how the textures get rendered to show the alpha channel");
	AddDialog(pxDialog);


		
	xOffset.x = 1.0f;	
	xOffset.y += Asura_Dialog::LargeSpacing();
	

	//create a large subwindow dialog to contain all the thumbails

	float fSubwindowHeight = fViewportPrintHeight - xOffset.y - 8.0f;
	m_fSubwindowWidth = g_xUserSetSubwindowSize.x ? g_xUserSetSubwindowSize.x : fViewportPrintWidth - 6.0f;
	m_pxSubwindowDialog = new Asura_Subwindow_Dialog(xOffset,m_fSubwindowWidth,fSubwindowHeight,false);
	AddDialog(m_pxSubwindowDialog);

 	xOffset.y = m_pxSubwindowDialog->m_xMaxCoord.y + 1.0f;

	//add control for resizing the subwindow dialog
	pxDialog = new Asura_Resize_Dialog(m_pxSubwindowDialog, &g_xUserSetSubwindowSize);
	AddDialog(pxDialog);


	pxDialog = new Asura_Function_Dialog("Okay",xOffset,OnOkay,(int)this);
	pxDialog->SetIsHiddenFunction(IsNotModal,(int)this);
	pxDialog->SetHelpText("Accept the current selection");
	AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,OnCancel,(int)this);
	pxDialog->SetIsHiddenFunction(IsNotModal,(int)this);
	pxDialog->SetHelpText("Cancel texture selection");
	AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;


    pxDialog = new Asura_Function_Dialog( "Help", xOffset, ShowKeys );
    AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Reload",xOffset,ReloadTexture,(int)this);
	pxDialog->SetHelpText("Force the selected textures to be reloaded");
	AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;

	pxDialog = new Asura_Function_Dialog("Load Texture (L)",xOffset,LoadTexture,(int)this);
	pxDialog->SetIsHiddenFunction(IsNotModal,(int)this);
	pxDialog->SetHelpText("Select and load a new texture , browsing for files on the hard disk");
	AddDialog(pxDialog);

	xOffset.x = 1.0f;
	pxDialog = new Asura_Function_Dialog("Close",xOffset,OnClose,(int)this);
	pxDialog->SetIsHiddenFunction(IsModal,(int)this);
	pxDialog->SetHelpText("Closes this overlay");
	AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	

	MakeDialogsRelativeToResizableDialog();


	UpdateTextureInfo();
	
}


void Tools_Overlay_ViewTextures::CreateOverlay()
{
	//overlay already open?
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_VIEWTEXTURES))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_VIEWTEXTURES);
		return;
	}

	//create the overlay then
	Tools_Overlay_ViewTextures* pxOverlay = new Tools_Overlay_ViewTextures();
	
	pxOverlay->InitialiseOverlay();

	pxOverlay->m_uID = TOOLS_OVERLAY_VIEWTEXTURES;
	Asura_Overlay_System::SubmitNewOverlay( pxOverlay);
}

void Tools_Overlay_ViewTextures::CloseOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_VIEWTEXTURES);	
}

bool Tools_Overlay_ViewTextures::OverlayExists()
{
	return s_pxThis != 0;
}

int Tools_Overlay_ViewTextures::PickTexture(const char* szTitle,int iInitialSelection /*=ASURA_TEXTURE_NONE*/, int iInitialTextureFlags /*= 0*/, Asura_Hash_ID* puResult /*= 0*/)
{
	//make sure the overlay is open
	bool bOverlayInitiallyOpen = OverlayExists(); 

	if(!bOverlayInitiallyOpen) CreateOverlay();
	
	Asura_Assert(s_pxThis,"Failed to get texture overlay");
	if(!s_pxThis) return 0;

	//set the title
	if(szTitle) s_pxThis->SetTitle(szTitle);

	//update selection
	if(iInitialSelection != ASURA_TEXTURE_NONE)
	{
		s_pxThis->SetSelectedTexture(iInitialSelection);
	}

	//run as modal overlay
	s_iResult = TOOLS_OVT_RESULT_CANCEL;
	s_iSelectedTexture = ASURA_TEXTURE_NONE;
	Tools_Modal_Overlay::DoModal(s_pxThis);

	CloseOverlay();

	s_iUsageFilter = s_iLastUsageFilter;//revert the filter if it was set before using PickTexture()

	if(s_iResult == TOOLS_OVT_RESULT_LOADTEXTURE)
	{
		//show the other material selection overlay instead
		s_iSelectedTexture = Tools_PickTexture::PickTextureFromDisk(iInitialTextureFlags, puResult);
		s_iResult = (s_iSelectedTexture != ASURA_TEXTURE_NONE) ? TOOLS_OVT_RESULT_OKAY : TOOLS_OVT_RESULT_CANCEL;
	}

	if(s_iResult == TOOLS_OVT_RESULT_OKAY)
	{
		if(puResult) *puResult = Asura_Texture_Management::GetHashIDFromIndex(s_iSelectedTexture);
		return s_iSelectedTexture;
	}
	else
	{
		return iInitialSelection;
	}
}

void Tools_Overlay_ViewTextures::SetTextureUsageFilter(Tools_OVT_TextureUsage eUsageFilter)
{
	s_iUsageFilter = eUsageFilter;
	if(s_pxThis) s_pxThis->m_bUpdateThumbnails = true;
}


void Tools_Overlay_ViewTextures::UpdateTextureInfo()
{
	//delete all the existing thumbnail dialogs 
	for(int i=0; i<m_xTextureTree.GetNumberOfNodes(); i++)
	{
		Tools_OVT_Texture* pxTexture = m_xTextureTree.GetDataByIndex(i);
		if(pxTexture->m_pxImageDialog)
		{
			DeleteSingleDialog(pxTexture->m_pxImageDialog);
			pxTexture->m_pxImageDialog = 0;
		}
	}


	m_xTextureTree.Reset();
	ClearSelection();
	m_pxCurrentTexture = 0;
	m_pxLastClickedTexture = 0;

	delete [] m_ppxVisibleTextures;
	m_ppxVisibleTextures = 0;
	m_iNumVisibleTextures = 0;

	//add all loaded textures
	const Asura_String** ppxNameList = new const Asura_String*[ASURA_MAX_NO_OF_TEXTURES];
	Asura_Mem::Set(ppxNameList, 0, sizeof(Asura_String*)*ASURA_MAX_NO_OF_TEXTURES);
	Asura_Texture_Management::MakeNameList(ppxNameList);

	for(int i=0; i<ASURA_MAX_NO_OF_TEXTURES; i++)
	{
		if(ppxNameList[i])
		{
			AddTexture(i, 0, 0, 0);
		}
	}
	delete [] ppxNameList;


	//Determine how much the textures are used on the environment
	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	//First find out which materials are being used , and how often
	Asura_TemplateHashTree<Tools_MaterialPolygonCount> xPolyCountTree;
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
		if(!pxMesh->TextureDataPtr) continue;

		Asura_Environment* pxEnv = Asura_Environment_System::GetEnvironment();
		bool bUsedByCurrentLightSet = Tools_LightSets::IsMemberOfCurrentLightSets(pxEnv->GetLightSets(i));

		for(int j=0;j<pxMesh->NumberOfPolygons;j++)
		{
			Asura_Mesh_TextureInfo* pxTex = &pxMesh->TextureDataPtr[j];

			if(pxTex->m_auMaterialResponseHashID[0])
			{
				bool bAlreadyExists = false;
				Tools_MaterialPolygonCount* pxCount = xPolyCountTree.AddOrFindNode(pxTex->m_auMaterialResponseHashID[0], &bAlreadyExists);
				if(bAlreadyExists)
				{
					pxCount->m_iPolygonCount++;
					pxCount->m_bCurrentLightSet |= bUsedByCurrentLightSet;
				}
				else
				{
					pxCount->m_iPolygonCount = 1;
					pxCount->m_bCurrentLightSet = bUsedByCurrentLightSet;
				}
			}
			if(pxTex->m_auMaterialResponseHashID[1] && pxTex->m_auMaterialResponseHashID[1] != pxTex->m_auMaterialResponseHashID[0])
			{
				bool bAlreadyExists = false;
				Tools_MaterialPolygonCount* pxCount = xPolyCountTree.AddOrFindNode(pxTex->m_auMaterialResponseHashID[1], &bAlreadyExists);
				if(bAlreadyExists)
				{
					pxCount->m_iPolygonCount++;
					pxCount->m_bCurrentLightSet |= bUsedByCurrentLightSet;
				}
				else
				{
					pxCount->m_iPolygonCount = 1;
					pxCount->m_bCurrentLightSet = bUsedByCurrentLightSet;
				}
			}
		}
	}
	//Iterate over those materials and add the textures that they use
	for(int i=0; i<xPolyCountTree.GetNumberOfNodes(); i++)
	{
		Tools_MaterialPolygonCount* pxCount = xPolyCountTree.GetDataByIndex(i);
		Asura_Hash_ID uMaterial = xPolyCountTree.GetHashByIndex(i);

		Tools_MaterialResponse* pxMaterial = Tools_MaterialResponse_System::GetMaterialByHashID(uMaterial);
		if(!pxMaterial) continue;

		u_int uUsageFlags =	1<<TOOLS_OVT_USAGE_ENVIRONMENT;
		if(pxCount->m_bCurrentLightSet)
		{
			uUsageFlags |= (1<<TOOLS_OVT_USAGE_LIGHTSET);
		}

		for(int iSlot=0; iSlot<Asura_MaterialResponse::NUMBER_OF_TEXTURE_SLOTS; iSlot++)
		{
			Asura_Hash_ID uTextureHash = pxMaterial->GetTextureHash(iSlot);
			if(!uTextureHash) continue;
			int iTextureID = pxMaterial->GetTexture(iSlot);
			if(iTextureID == ASURA_TEXTURE_NONE) return;

			AddTexture(iTextureID, uUsageFlags, uMaterial, pxCount->m_iPolygonCount);
		}				
	
	}
	#else //#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	for(int i=0;i<Asura_Module_System::GetNumberOfModules();i++)
	{
		Asura_Mesh* pxMesh = Asura_Module_System::GetMeshForModule(i);
		if(!pxMesh->TextureDataPtr) continue;

		Asura_Environment* pxEnv = Asura_Environment_System::GetEnvironment();
		bool bUsedByCurrentLightSet = Tools_LightSets::IsMemberOfCurrentLightSets(pxEnv->GetLightSets(i));

		u_int uUsageFlags =	1<<TOOLS_OVT_USAGE_ENVIRONMENT;
		if(bUsedByCurrentLightSet)
		{
			uUsageFlags |= (1<<TOOLS_OVT_USAGE_LIGHTSET);
		}


		for(int j=0;j<pxMesh->NumberOfPolygons;j++)
		{
			Asura_Mesh_TextureInfo* pxTex = &pxMesh->TextureDataPtr[j];

			if(pxTex->m_iMaterialID != ASURA_MATERIAL_NONE)
			{
				Asura_Material* pxMaterial = Asura_Material_System::GetMaterial(pxTex->m_iMaterialID);
				if(!pxMaterial) continue;

				u_int uPolygonID = ((i+1)<<16)|j;
				
				int aiTextureID[6] = {pxMaterial->m_iTextureID[0],pxMaterial->m_iTextureID[1],
										pxMaterial->m_iNormalMapID[0],pxMaterial->m_iNormalMapID[1],
										pxMaterial->m_iSpecularMapID[0],pxMaterial->m_iSpecularMapID[1]};
				for(int k=0; k<6; k++)
				{
					if(aiTextureID[k] == ASURA_TEXTURE_NONE) continue;

					//For non material response use a polygon ID instead of the material response ID to prevent
					//the same texture being counted on a polygon multiple times
					AddTexture(aiTextureID[k], uUsageFlags, uPolygonID, 1);

				}
				
			}
		}
	}
	#endif //#else #ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	//check for usage by (loaded) materials
	u_int uNumMaterials = Tools_MaterialResponse_System::GetNumMaterials();
	for(u_int i=0; i<uNumMaterials; i++)
	{
		Tools_MaterialResponse* pxMaterial = Tools_MaterialResponse_System::GetMaterialByIndex_WithoutLoading(i);
		if(!pxMaterial || !pxMaterial->IsLoaded()) continue;

		for(int iSlot=0; iSlot<Asura_MaterialResponse::NUMBER_OF_TEXTURE_SLOTS; iSlot++)
		{
			Asura_Hash_ID uTextureHash = pxMaterial->GetTextureHash(iSlot);
			if(!uTextureHash) continue;
			int iTextureID = Asura_Texture_Management::GetIndexFromHashID(uTextureHash);
			if(iTextureID == ASURA_TEXTURE_NONE) continue;//don't bother showing material textures that haven't been loaded yet
			
			AddTexture(iTextureID, 1<<(iSlot+TOOLS_OVT_USAGE_FIRSTMATERIALRESPONSE), 0, 0);
			
		}
	}
	#endif

	//check for usage by GUImenu
	List<int> xGUITextureList;
	Asura_GUIMenu_System::TraverseUsedTextures(ListGUITexturesCallback,&xGUITextureList);
	for(LIF<int> xTextureLIF(&xGUITextureList); !xTextureLIF.done(); xTextureLIF.next())
	{
		AddTexture(xTextureLIF(), 1<<TOOLS_OVT_USAGE_GUIMENU, 0, 0);
	}
	


	UpdateThumbnails();
}

void Tools_Overlay_ViewTextures::ListGUITexturesCallback(int iTextureIndex,void* pList)
{
	List<int>* pxList = (List<int>*) pList;

	if(iTextureIndex != ASURA_TEXTURE_NONE)
	{
		if(!pxList->contains(iTextureIndex))
		{
			pxList->add_entry(iTextureIndex);
		}
	}
}


void Tools_Overlay_ViewTextures::UpdateThumbnails()
{
	//delete all the existing thumbnail dialogs 
	for(int i=0; i<m_xTextureTree.GetNumberOfNodes(); i++)
	{
		Tools_OVT_Texture* pxTexture = m_xTextureTree.GetDataByIndex(i);
		if(pxTexture->m_pxImageDialog)
		{
			DeleteSingleDialog(pxTexture->m_pxImageDialog);
			pxTexture->m_pxImageDialog = 0;
		}
		pxTexture->m_bHighlight = false;
		pxTexture->m_bVisible = false;
		pxTexture->m_iVisibleIndex = -1;
	}

	delete [] m_ppxVisibleTextures;
	m_ppxVisibleTextures = 0;
	m_iNumVisibleTextures = 0;


	int iNumTextures = m_xTextureTree.GetNumberOfNodes();
	if(!iNumTextures) return;

	//copy the texture pointers into an array for sorting
	m_ppxVisibleTextures = new Tools_OVT_Texture*[iNumTextures];
	m_iNumVisibleTextures = 0;


	u_int uTextureFlag = g_aiTextureFlagValues[s_iTextureFlagFilter];
	for(int i=0;i<iNumTextures;i++)
	{
		Tools_OVT_Texture* pxTexture = m_xTextureTree.GetDataByIndex(i);

		if(s_iUsageFilter)
		{
			if(!(pxTexture->m_uUsageFlags & (1<<s_iUsageFilter))) continue;
		}
		if(uTextureFlag)
		{
			bool bHasFlag = (Asura_Texture_Management::GetTextureFlags(pxTexture->m_iTextureID) & uTextureFlag) != 0;
			if(s_iTextureFlagMode == TOOLS_OVT_FLAGMODE_SHOW)
			{
				if(!bHasFlag) continue;
			}
			else
			{
				if(bHasFlag) pxTexture->m_bHighlight = true;
			}
		}


		m_ppxVisibleTextures[m_iNumVisibleTextures++] = pxTexture;
		pxTexture->m_bVisible = true;
	}

	//deselect hidden entries
	for(int i=0;i<iNumTextures;i++)
	{
		Tools_OVT_Texture* pxTexture = m_xTextureTree.GetDataByIndex(i);
		if(pxTexture->m_bSelected && !pxTexture->m_bVisible)
		{
			SetSelected(pxTexture, false);
		}
	}


	//Sort the thumbnails
	qsort(m_ppxVisibleTextures, m_iNumVisibleTextures, sizeof(Tools_OVT_Texture*), SortTextureCallback);

	//Create image dialogs for each texture
	Asura_Vector_2 xOffset;
	xOffset.x = g_fThumbnailSpacing / ASURA_TEXT_OUTPUT_FONT_WIDTH;
	xOffset.y = g_fThumbnailSpacing / ASURA_TEXT_OUTPUT_FONT_HEIGHT;

	for(int i=0;i<m_iNumVisibleTextures;i++)
	{
		
		Tools_OVT_Texture* pxTexture = m_ppxVisibleTextures[i];
		pxTexture->m_iVisibleIndex = i;

		//get the texture to show
		int iTexture = pxTexture->m_iTextureID;

		Asura_Image_Dialog* pxDialog = pxTexture->m_pxImageDialog = new Asura_Image_Dialog(xOffset, iTexture, g_fThumbnailWidth, g_fThumbnailHeight);
		pxDialog->SetTranslucent(false);
		if(pxTexture->m_bSelected)
		{
			pxDialog->SetBorder(0xffff0000,g_iThumbnailBorder);
		}
		else if(pxTexture->m_bHighlight)
		{
			pxDialog->SetBorder(0xff00ff00,g_iThumbnailBorder);
		}
		else
		{
			pxDialog->SetBorder(0xffffffff, 1);
		}
		m_pxSubwindowDialog->AddDialog(pxDialog);

		//Find the position to use for the next image
		xOffset.x += (g_fThumbnailWidth+g_fThumbnailSpacing) / ASURA_TEXT_OUTPUT_FONT_WIDTH;
		
		//need to move onto next row?
		if((xOffset.x + ((g_fThumbnailWidth+g_fThumbnailSpacing) / ASURA_TEXT_OUTPUT_FONT_WIDTH)) > m_fSubwindowWidth)
		{
			xOffset.x = g_fThumbnailSpacing / ASURA_TEXT_OUTPUT_FONT_WIDTH;
			xOffset.y += (g_fThumbnailHeight+g_fThumbnailSpacing) / ASURA_TEXT_OUTPUT_FONT_HEIGHT;
		}
	}
	if(m_pxLastClickedTexture && !m_pxLastClickedTexture->m_bVisible)
	{
		m_pxLastClickedTexture = 0;
	}

	FlagForResize();
	UpdateDisplayMode();
}

void Tools_Overlay_ViewTextures::UpdateDisplayMode()
{
	UpdateDisplayMode(m_pxLargeImageDialog);
	for(int i=0; i<m_iNumVisibleTextures; i++)
	{
		UpdateDisplayMode(m_ppxVisibleTextures[i]->m_pxImageDialog);
	}
	m_bUpdateDisplayMode = false;
}

void Tools_Overlay_ViewTextures::UpdateDisplayMode(Asura_Image_Dialog* pxDialog)
{
	if(!pxDialog) return;
	switch(s_iDisplayMode)
	{
		case TOOLS_OVT_DISPLAYMODE_OPAQUE :
			pxDialog->SetBackground(0xff000000);
			pxDialog->SetTranslucent(false);
			pxDialog->SetTextureOp(ASURA_TEXTUREOP_INVALID);
			break;

		case TOOLS_OVT_DISPLAYMODE_ALPHAONLY :
			pxDialog->SetBackground(0xff000000);
			pxDialog->SetTranslucent(false);
			pxDialog->SetTextureOp(ASURA_TEXTUREOP_VIEWTEXTUREALPHA);
			break;

		case TOOLS_OVT_DISPLAYMODE_TRANSPARENT_BLACK :
			pxDialog->SetBackground(0xff000000);
			pxDialog->SetTranslucent(true);
			pxDialog->SetTextureOp(ASURA_TEXTUREOP_INVALID);
			break;

		case TOOLS_OVT_DISPLAYMODE_TRANSPARENT_MAGENTA :
			pxDialog->SetBackground(0xffff00ff);
			pxDialog->SetTranslucent(true);
			pxDialog->SetTextureOp(ASURA_TEXTUREOP_INVALID);
			break;
		default :
			Asura_Assert(0,"Unexpected display mode");
			break;
	}
}


void Tools_Overlay_ViewTextures::AddTexture(int iTextureID, u_int uUsageFlags, Asura_Hash_ID uMaterial, int iPolyCount)
{
	if(iTextureID == ASURA_TEXTURE_NONE) return;

	Asura_Hash_ID uTextureHash = Asura_Texture_Management::GetHashIDFromIndex(iTextureID);
	Asura_Assert(uTextureHash, "");

	Tools_OVT_Texture* pxTexture = m_xTextureTree.GetDataByHash(uTextureHash);
	if(!pxTexture)
	{
		pxTexture = m_xTextureTree.AddNode(uTextureHash);
		Asura_Assert(pxTexture,"");

		pxTexture->m_iTextureID = iTextureID;

		const Asura_String* pxTextureName = Asura_Texture_Management::GetTextureName(iTextureID);

		if(pxTextureName && !pxTextureName->IsBlank())
		{

			//split texture filename into path and name components
			Asura_String xExtension;
			
			pxTextureName->SplitFilename(&pxTexture->m_xTexturePath,&pxTexture->m_xTextureName,&xExtension);
			if(pxTexture->m_xTextureName.IsNull()) pxTexture->m_xTextureName = "";
			if(pxTexture->m_xTexturePath.IsNull()) pxTexture->m_xTexturePath = "";

			if(!xExtension.IsBlank())
			{
				pxTexture->m_xTextureName += ".";
				pxTexture->m_xTextureName += xExtension;
			}
			
		}
		else
		{
			char szText[200];
			Asura_PrintfToBfr(szText,"*Not Found* : %s",sizeof(szText),true,uTextureHash);
			
			pxTexture->m_xTextureName = szText;
			pxTexture->m_xTexturePath = "";
		}

		if(Asura_Texture_Management::IsTextureCubemap(iTextureID))
		{
			pxTexture->m_uUsageFlags |= TOOLS_OVT_USAGE_CUBEMAP;
		}
		if(Tools_SkyboxOverlay::IsSkyboxTexture(iTextureID) || Tools_SkyboxOverlay::IsReflectanceTexture(iTextureID))
		{
			pxTexture->m_uUsageFlags |= TOOLS_OVT_USAGE_SKYBOX;
		}

		Tools_Texture_Management::GetTextureSize(pxTexture->m_iTextureID, pxTexture->m_iWidth, pxTexture->m_iHeight);
		pxTexture->m_iSize = pxTexture->m_iWidth*pxTexture->m_iHeight;
	}

	pxTexture->m_uUsageFlags |= uUsageFlags;


	if(iPolyCount)
	{
		//Only increate the count if this material is different from the previous one
		if(pxTexture->m_uMostRecentMaterial != uMaterial)
		{
			pxTexture->m_uMostRecentMaterial = uMaterial;
			pxTexture->m_uNumPolygons += iPolyCount;
		}
	}
}

void Tools_Overlay_ViewTextures::PerFrameUpdate()
{
	Asura_Overlay::PerFrameUpdate();

	bool bUsedDropdown = false;
	if(g_xUserSetSubwindowSize.x && g_xUserSetSubwindowSize.x != m_fSubwindowWidth)
	{
		m_fSubwindowWidth = g_xUserSetSubwindowSize.x;
		m_bUpdateThumbnails = true;
	}

	if(m_bUsageFilterChanged)
	{
		s_iLastUsageFilter = s_iUsageFilter;
		m_bUsageFilterChanged = false;
		m_bUpdateThumbnails = true;
	}
	
	if(m_bUpdateThumbnails)
	{
		m_bUpdateThumbnails = false;
		bUsedDropdown = true;
		UpdateThumbnails();
	}
	if(m_bUpdateDisplayMode)
	{
		UpdateDisplayMode();
		bUsedDropdown = true;
	}

	if(this == Asura_Overlay_System::GetCurrentOverlay())
	{
		//Determine which texture is under the cursor
		float fMouseX,fMouseY;
		Asura_Cursor::GetScreenCoords(fMouseX,fMouseY);
		Asura_Vector_2 xMousePos(fMouseX,fMouseY);

		bool bMouseOverAThumbnail = false;

		Tools_OVT_Texture* pxTexture = 0;
		Tools_OVT_Texture* pxMouseOverTexture = 0;
		for(int i=0; i<m_iNumVisibleTextures; i++)
		{
			pxTexture = m_ppxVisibleTextures[i];
			if(pxTexture->m_pxImageDialog)
			{
				if(pxTexture->m_pxImageDialog->IsPointInDialog(m_xOrigin, xMousePos))
				{
					SetCurrentTexture(pxTexture);
					pxMouseOverTexture = pxTexture;
					bMouseOverAThumbnail = true;
					break;
				}
			}
		}

		if(!bUsedDropdown && !Asura_Overlay_System::GetFocusGadget() && !Tools_Help::IsDisplayingHelp())
		{
			if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1) || Asura_UserInput::KeyPressed(ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON1))
			{
				if(pxMouseOverTexture)
				{

					if(Asura_UserInput::KeyPressed(ASURA_KEYID_DOUBLECLICK_MOUSEBUTTON1))
					{
						ClearSelection();
						SetSelected(m_pxCurrentTexture, true);
						if(IsModal())
						{
							//double clicked a texture , so end selection
							OnOkay();
						}
						else
						{
							//OnEditMaterial();
						}
					}
					else
					{
						if(m_pxLastClickedTexture && m_pxLastClickedTexture->m_bVisible && Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
						{
							//Add everything between this and the most recent selection to the selected list
							SelectMultiple(m_pxLastClickedTexture, pxMouseOverTexture, true);
						}
						else if(m_pxLastClickedTexture && m_pxLastClickedTexture->m_bVisible && Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
						{
							//Replace selection with everything between this and the most recent selection
							SelectMultiple(m_pxLastClickedTexture, pxMouseOverTexture, false);
						}
						else if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
						{
							//add/remove from the current selection
							SetSelected(pxMouseOverTexture, !pxMouseOverTexture->m_bSelected);
						}
						else
						{
							//replace the current selection
							ClearSelection();
							SetSelected(pxMouseOverTexture, true);
						}
						m_pxLastClickedTexture = pxMouseOverTexture;
					}
				}
			}

			if(IsModal())
			{
				if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ENTER))
				{
					OnOkay();
				}
				else if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
				{
					OnCancel();
				}
				else if(Asura_UserInput::DebouncedKeyPressed('l'))
				{
					LoadTexture();
				}
			}
			else
			{
				if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ENTER))
				{
					OnClose();
					Asura_Overlay_System::s_bUsedKeypressThisFrame = true;
				}
				else if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
				{
					OnClose();
					Asura_Overlay_System::s_bUsedKeypressThisFrame = true;

				}
			}

		}
	}
}

void Tools_Overlay_ViewTextures::SetCurrentTexture(Tools_OVT_Texture* pxTexture)
{
	if(pxTexture == m_pxCurrentTexture) return;
	m_pxCurrentTexture = pxTexture;

	if(!pxTexture) return;

	//update the enlarged image
	int iTexture = pxTexture->m_iTextureID;
	m_pxLargeImageDialog->SetTexture(iTexture);

	//update the various text dialogs
	char szText[200];

	Asura_PrintfToBfr(szText,"Texture Name: %s",sizeof(szText),true,pxTexture->m_xTextureName.GetString());
	m_pxTextureNameDialog->UpdateLabel(szText);

	Asura_PrintfToBfr(szText,"Texture Path: %s",sizeof(szText),true,pxTexture->m_xTexturePath.GetString());
	m_pxTexturePathDialog->UpdateLabel(szText);

	Asura_PrintfToBfr(szText,"Polygons : %d",sizeof(szText),true,pxTexture->m_uNumPolygons);
	m_pxPolygonsDialog->UpdateLabel(szText);

	if(pxTexture->m_iWidth)
		Asura_PrintfToBfr(szText,"Size : %d x %d",sizeof(szText),true,pxTexture->m_iWidth, pxTexture->m_iHeight);
	else
		Asura_PrintfToBfr(szText,"Size : Unknown",sizeof(szText),true);
	m_pxTextureDimensionDialog->UpdateLabel(szText);


	//add the names of any texture flags used
	int iFlags = Asura_Texture_Management::GetTextureFlags(pxTexture->m_iTextureID);
	int iNumFlags = 0;
	for(int i=0; i<g_iNumTextureFlagStrings && iNumFlags<TOOLS_OVT_NUMFLAGTEXTDIALOG; i++)
	{
		if(iFlags & g_aiTextureFlagValues[i])
		{
			m_apxFlagTextDialog[iNumFlags++]->UpdateLabel(g_aszTextureFlagStrings[i]);
		}
	}
	//reset the unused entries
	for(int i=iNumFlags; i<TOOLS_OVT_NUMFLAGTEXTDIALOG; i++)
	{
		m_apxFlagTextDialog[iNumFlags++]->UpdateLabel("");
	}
}


void Tools_Overlay_ViewTextures::SetSelected(Tools_OVT_Texture* pxTexture, bool bSelect)
{
	if(bSelect)
	{
		if(pxTexture->m_bSelected)
		{
			//remove from the list so it can be added back on the end
			m_xSelection.delete_entry(pxTexture);
		}
		pxTexture->m_bSelected = true;
		m_xSelection.add_entry(pxTexture);
	}
	else if(pxTexture->m_bSelected)
	{
		pxTexture->m_bSelected = false;
		m_xSelection.delete_entry(pxTexture);
	}

	if(pxTexture->m_pxImageDialog)
	{
		Asura_Image_Dialog* pxDialog = pxTexture->m_pxImageDialog;

		if(pxTexture->m_bSelected)
		{
			pxDialog->SetBorder(0xffff0000,g_iThumbnailBorder);
		}
		else if(pxTexture->m_bHighlight)
		{
			pxDialog->SetBorder(0xff00ff00,g_iThumbnailBorder);
		}
		else
		{
			pxDialog->SetBorder(0xffffffff,1);
		}
	}
}

void Tools_Overlay_ViewTextures::SelectMultiple(Tools_OVT_Texture* pxStart, Tools_OVT_Texture* pxEnd, bool bAddToSelection)
{
	if(!bAddToSelection) ClearSelection();

	Asura_Assert(pxStart && pxStart->m_iVisibleIndex>=0, "");
	Asura_Assert(pxEnd && pxEnd->m_iVisibleIndex>=0, "");

	if(pxEnd->m_iVisibleIndex > pxStart->m_iVisibleIndex)
	{
		for(int i=pxStart->m_iVisibleIndex; i<=pxEnd->m_iVisibleIndex; i++)
		{
			SetSelected(m_ppxVisibleTextures[i], true);
		}
	}
	else
	{
		for(int i=pxStart->m_iVisibleIndex; i>=pxEnd->m_iVisibleIndex; i--)
		{
			SetSelected(m_ppxVisibleTextures[i], true);
		}
	}
}

void Tools_Overlay_ViewTextures::ClearSelection()
{
	while(m_xSelection.size())
	{
		SetSelected(m_xSelection.first_entry(), false);
	}
}

void Tools_Overlay_ViewTextures::SetSelectedTexture(int iTextureID)
{
	ClearSelection();

	if(iTextureID == ASURA_TEXTURE_NONE) return;

	//search for the entry for this texture
	Asura_Hash_ID uTextureHash = Asura_Texture_Management::GetHashIDFromIndex(iTextureID);
	Asura_Assert(uTextureHash, "");
	if(!uTextureHash) return;

	Tools_OVT_Texture* pxTexture = m_xTextureTree.GetDataByHash(uTextureHash);
	if(!pxTexture) return;

	//adjust the filters to make the texture visible
	if(!pxTexture->m_bVisible)
	{
		//reset usage filter if needed
		if(s_iUsageFilter)
		{
			if(!(pxTexture->m_uUsageFlags &(1<<s_iUsageFilter)))
			{
				s_iUsageFilter = 0;
			}
		}
		//reset flag filter if needed
		if(s_iTextureFlagFilter && s_iTextureFlagMode == TOOLS_OVT_FLAGMODE_SHOW)
		{
			u_int uTextureFlag = g_aiTextureFlagValues[s_iTextureFlagFilter];
			bool bHasFlag = (Asura_Texture_Management::GetTextureFlags(pxTexture->m_iTextureID) & uTextureFlag) != 0;
			if(!bHasFlag)
			{
				s_iTextureFlagFilter = 0;
			}
		}
		UpdateThumbnails();
	}

	Asura_Assert(pxTexture->m_bVisible, "Failed to unhide texture being selected");
	if(pxTexture->m_bVisible)
	{
		SetSelected(pxTexture, true);
	}

}

int Tools_Overlay_ViewTextures::SortTextureCallback(const void* pElem1,const void* pElem2)
{
	const Tools_OVT_Texture* pxTex1 = *((const Tools_OVT_Texture**) pElem1);
	const Tools_OVT_Texture* pxTex2 = *((const Tools_OVT_Texture**) pElem2);

	switch(s_iSortMode)
	{
		case TOOLS_OVT_SORT_TEXTURENAME :
			return stricmp(pxTex1->m_xTextureName.GetString(), pxTex2->m_xTextureName.GetString());

		case TOOLS_OVT_SORT_POLYGONS :
			if(pxTex1->m_uNumPolygons < pxTex2->m_uNumPolygons) return -1;
			if(pxTex1->m_uNumPolygons > pxTex2->m_uNumPolygons) return 1;
			return 0;

		case TOOLS_OVT_SORT_SIZE :
			if(pxTex1->m_iSize < pxTex2->m_iSize) return -1;
			if(pxTex1->m_iSize > pxTex2->m_iSize) return 1;
			return 0;

	}

	Asura_Assert(0,"Unexpected sort mode");
	return 0;
}

bool Tools_Overlay_ViewTextures::WasCancelled()
{
	return s_iResult == TOOLS_OVT_RESULT_CANCEL;
}

void Tools_Overlay_ViewTextures::OnOkay()
{
	if(m_xSelection.size())
	{
		s_iSelectedTexture = m_xSelection.last_entry()->m_iTextureID;
	}
	s_iResult = TOOLS_OVT_RESULT_OKAY;
	EndModal();	
}

void Tools_Overlay_ViewTextures::OnCancel()
{
	s_iResult = TOOLS_OVT_RESULT_CANCEL;
	EndModal();	
}

void Tools_Overlay_ViewTextures::LoadTexture()
{
	s_iResult = TOOLS_OVT_RESULT_LOADTEXTURE;
	EndModal();	
}

void Tools_Overlay_ViewTextures::OnClose()
{
	CloseOverlay();
}

//void Tools_Overlay_ViewTextures::OnEditMaterial()
//{
//	if(s_uSelectedMaterial)
//	{
//		Tools_Texturing_MaterialResponse::EditMaterial(s_uSelectedMaterial);	
//	}
//}

void Tools_Overlay_ViewTextures::ReloadTexture()
{
	for(LIF<Tools_OVT_Texture*> xSelectionLIF(&m_xSelection); !xSelectionLIF.done(); xSelectionLIF.next())
	{
		Asura_Texture_Management::ReloadTexture(xSelectionLIF()->m_iTextureID);
	}
}


bool Tools_Overlay_ViewTextures::IsNotModal()
{
	return !IsModal();
}

bool Tools_Overlay_ViewTextures::IsModal()
{
	return m_bAlwaysOnTop;
}

bool Tools_Overlay_ViewTextures::IsTextureSelected()
{
	return m_xSelection.size()>0;
}





bool Tools_Overlay_ViewTextures::GetParameterFromSelection(Asura_Hash_ID uParamID,class Tools_Parameter* pxParam,bool* pbMixed)
{
	if(!s_pxThis) return false;

	bool bAnyGet = false;
	pxParam->m_bChanged = false;
	int iChangedCount = 0;

	for(LIF<Tools_OVT_Texture*>  xSelectionLIF(&s_pxThis->m_xSelection);!xSelectionLIF.done();xSelectionLIF.next())
	{
		if(s_pxThis->GetParameterFromTexture(xSelectionLIF(), uParamID, pxParam))
		{
			//Keep track of how many times the value changes.
			//If this is the first time the value was got , then count as changed , even
			//if it wasn't changed from the default value
			if(pxParam->m_bChanged || !bAnyGet)
			{
				iChangedCount++;
				pxParam->m_bChanged = false;
			}	
			bAnyGet = true;
		}
	}
	if(pbMixed) *pbMixed = (iChangedCount>1);

	return bAnyGet;

}

bool Tools_Overlay_ViewTextures::SetParameterForSelection(Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	bool bAnySet = false;
	for(LIF<Tools_OVT_Texture*>  xSelectionLIF(&s_pxThis->m_xSelection);!xSelectionLIF.done();xSelectionLIF.next())
	{
		if(s_pxThis->SetParameterForTexture(xSelectionLIF(), uParamID, pxParam))
		{
			bAnySet = true;
		}
	}
	return bAnySet;
}

bool Tools_Overlay_ViewTextures::GetParameterFromTexture(Tools_OVT_Texture* pxTexture,Asura_Hash_ID uParamID,class Tools_Parameter* pxParam)
{
	//At the moment the only acceptable values for uParamID are index values into the flag list
	Asura_Assert(uParamID < (u_int)g_iNumTextureFlagStrings, "");

	u_int uFlagBit = g_aiTextureFlagValues[uParamID];
	bool bSet = (Asura_Texture_Management::GetTextureFlags(pxTexture->m_iTextureID) & uFlagBit) != 0;
	pxParam->SetBool(bSet);
	return true;
}

bool Tools_Overlay_ViewTextures::SetParameterForTexture(Tools_OVT_Texture* pxTexture,Asura_Hash_ID uParamID,const class Tools_Parameter* pxParam)
{
	//At the moment the only acceptable values for uParamID are index values into the flag list
	Asura_Assert(uParamID < (u_int)g_iNumTextureFlagStrings, "");

	u_int uFlag = g_aiTextureFlagValues[uParamID];
	bool bAddFlag = pxParam->GetBool();

	u_int uOldFlags = Asura_Texture_Management::GetTextureFlags(pxTexture->m_iTextureID);
	u_int uNewFlags = uOldFlags;

	if(bAddFlag)
	{
		uNewFlags |= uFlag;
	}
	else
	{
		uNewFlags &=~ uFlag;
	}
	
	//deal with sily flag combinations
	if( bAddFlag && uFlag == AsuraTextureFlag_PS2DontHalveSize)
	{
		uNewFlags &=~ AsuraTextureFlag_ReduceExtraForConsoles;
	}

	if( bAddFlag && uFlag == AsuraTextureFlag_ReduceExtraForConsoles)
	{
		uNewFlags &=~ AsuraTextureFlag_PS2DontHalveSize;
	}
	
	Asura_Texture_Management::SetTextureFlags(pxTexture->m_iTextureID, uNewFlags);


	//force texture to be reloaded if the compression settings change
	const u_int uChangedFlags = uOldFlags ^ uNewFlags;
	const u_int uFlagsToCheck = AsuraTextureFlag_HighQualityCompression | AsuraTextureFlag_NoMipMaps | AsuraTextureFlag_SharpenMipMaps;
	if( uChangedFlags & uFlagsToCheck )
	{
		Asura_Texture_Management::ReloadTexture(pxTexture->m_iTextureID);
	}

	if(uChangedFlags)
	{
		//set the appropriate file as modified
		if(pxTexture->m_uUsageFlags & (1<<TOOLS_OVT_USAGE_GUIMENU))
		{
			Tools_GUIMenu_System::SetTextureFlagsModified();
		}
		else if(pxTexture->m_uUsageFlags & (1<<TOOLS_OVT_USAGE_SKYBOX))
		{
			Tools_SaveAsura::SetSkyboxModified();
		}
		else
		{
			Tools_SaveAsura::SetMainFileModified();
		}

	}

	m_bUpdateThumbnails = true;

	return true;
}

void Tools_Overlay_ViewTextures::ShowKeys()
{
	Tools_Help::ShowHelpTopic("View Textures Overlay");
}





