#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __Tools_Texturing_Texture_h
#define __Tools_Texturing_Texture_h

#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"
#include "Misc/list_tem.hpp"
#include "Tools_Undo.h"

///////////////////////////////////////////////////////////////////////////////////////
class Tools_UV_Editor;
struct PolygonID;
///////////////////////////////////////////////////////////////////////////////////////
struct Float_Rectangle
{
	float top;
	float left;
	float bottom;
	float right;
};


class NodeEditUndo : public VariableSizeUndo
{
public :
	NodeEditUndo()
	:VariableSizeUndo(1000){}

	void Log_NodeMove(int index,const Asura_Vector_2& position);
	void Log_UsedAreaChange(const Float_Rectangle& rect);
	void Log_TextureChange(int index);

	void SetNumNodes(int nodes);
	int NumNodes;//The number of nodes that the current undo data expects
	Tools_UV_Editor* TheTexture;
	
};

//undo classes for each type of undo
class NodeEditUndoData_NodeMoved : public RootUndoData
{
public :
	NodeEditUndoData_NodeMoved(Tools_UV_Editor* pxTextureEdit,int _index,const Asura_Vector_2& _position);
	virtual RootUndoData* ProcessUndoData();

	Tools_UV_Editor* m_pxTextureEdit;
	int index;
	Asura_Vector_2 position;
};

class NodeEditUndoData_UsedAreaChange : public RootUndoData
{
public :
	NodeEditUndoData_UsedAreaChange(Tools_UV_Editor* pxTextureEdit,const Float_Rectangle& _rect);
	virtual RootUndoData* ProcessUndoData();

	Tools_UV_Editor* m_pxTextureEdit;
	Float_Rectangle rect;
};

class NodeEditUndoData_TextureChange : public RootUndoData
{
public :

	NodeEditUndoData_TextureChange(Tools_UV_Editor* pxTextureEdit,int _index);

	virtual RootUndoData* ProcessUndoData();

	Tools_UV_Editor* m_pxTextureEdit;

	int index;
};

///////////////////////////////////////////////////////////////////////////////////////
enum NodeEditLeftButtonMode
{
	NodeLBM_SelectAndMove,
	NodeLBM_Selection,
	NodeLBM_Move,
	NodeLBM_Scale,
	NodeLBM_Rotate,
	NodeLBM_Zoom,
	NodeLBM_EditShownArea,
};

enum NodeEditAxisRestrict
{
	NodeEdit_NoRestriction,	
	NodeEdit_XOnly,	
	NodeEdit_YOnly,	
};

//flags for determing what the mouse is doing when being dragged in node edit mode
#define TexturingDragMode_Nodes				0x00000001 //dragging 1 or more nodes
#define TexturingDragMode_Horizontal		0x00000002 //dragging left of texture
#define TexturingDragMode_Vertical			0x00000004 //dragging top of texture
#define TexturingDragMode_ScrollTexture		0x00000008 //scrolling shown area
#define TexturingDragMode_RectSelection 	0x00000010 
#define TexturingDragMode_ScaleSelected 	0x00000020 
#define TexturingDragMode_RotateSelected	0x00000040 
#define TexturingDragMode_Zoom				0x00000080 
#define TexturingDragMode_RotateResize		0x00010000
#define TexturingDragMode_LeftClickModes 	0x0000ffff
#define TexturingDragMode_RightClickModes 	0xffff0000

/*
For storing details of the current texture being worked on while texture mapping.
*/

class Tools_UV_Editor
{
public :
	Tools_UV_Editor();
	~Tools_UV_Editor();


	void CreateMouseModeOverlay();
	void CloseMouseModeOverlay();
	/////////////////////////////////////////////////////////////////////////
	void Render();//draw the texture and nodes and joining lines
	//convert uv coordinates to a location on screen
	Asura_Vector_2 GetScreenPosition(const Asura_Vector_2& uv);
	Float_Rectangle GetScreenPosition(const Float_Rectangle& rect);
	//conversion the other mway round
	Asura_Vector_2 GetUVFromScreenPosition(const Asura_Vector_2& screen);
	/////////////////////////////////////////////////////////////////////////

	//The various stages for rendering the texture and nodes in the corner of the screen.
	//These can be called separately if other modes need to do extra stuff
	void RenderTexture();
	void RenderLines();
	void RenderNodes();
	void RenderSelectionBox();
	void RenderLine(const Asura_Vector_2& xUVStart,const Asura_Vector_2& xUVEnd,u_int uColour);


	void NodesToCorners();
	
	void CopyFromPolygon(PolygonID* polygon,int iChannel);//copy the texturing from a polygon into the node info
	
	void ClearMaterialAndTexture();

	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	void SetMaterialResponse(int iChannel, Asura_Hash_ID uMaterialResponseHashID);
	#endif
	void SetMaterialID(int iMaterialID);
	void SetTextureID(int iTextureID);

	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Asura_Hash_ID GetMaterialResponse(int iChannel) const {return m_auMaterialResponseHashID[iChannel];}
	#endif
	int GetMaterialID() const {return m_iMaterialID;}
	int GetTextureID() const {return m_iTextureID;}

	void SetTextureChannel(int iChannel){m_iTextureChannel = iChannel;}

	//Gets the texture displayed based on the selected material/texture channel or selected texture
	int GetTextureToDisplay();

	void HandleMouse();

	void LeftButtonPressed(Asura_Vector_2& MousePos);
	void LeftButtonHeld(Asura_Vector_2& MousePos);
	void LeftButtonReleased(Asura_Vector_2& MousePos);

	void RightButtonPressed(Asura_Vector_2& MousePos);
	void RightButtonHeld(Asura_Vector_2& MousePos);
	void RightButtonReleased(Asura_Vector_2& MousePos);

	void InitaliseNodesForTextureWrap();//makes sure that the nodes start off rectangular
	void EnsureNodesAreStillOkayForTextureWrap();//makes sure the nodes stay that way

	void PrepareRotationOrScale(bool selected_only);
	void RotateNodes(const Asura_Vector_2& centre,Asura_Angle angle,bool selected_only);
	void ScaleNodes(const Asura_Vector_2& centre,float scale,bool selected_only,int iAxisRestrict);

	void ShiftAndScaleNodes(float fUOldOrigin,float fVOldOrigin,float fUNewOrigin,float fVNewOrigin,float fUScale,float fVScale);

	void ClampNodesToUsedArea(bool log_undo = true);
	void ExpandUsedAreaToContainNodes();
	void FitUsedAreaToNodes();
	void DetermineSuitableShownArea(Asura_Vector_2* DesiredCentre_UV = 0);
	void UpdateUsedTextureArea(const Float_Rectangle& rect);//Changes used texture area. Doesn't adjust nodes,or log for undo 

	void PrepareNodeUndo();

	void SetNumNodes(int num_nodes);

	void SelectAllNodesForDragIfNeeded();
	
	void SnapCursorToNearestNode();

	void EvenlyDistributeUVs();

	float GetWidthScale();
	float GetHeightScale();

	void SnapNodesToNearestTexel();
	void SnapNodeToNearestTexel(int iIndex);

	void SetTranslucencyMode(enum ASURA_TRANSLUCENCY_MODE eMode) { m_eTranslucency = eMode; }

	/////////////////////////////////////////////////////////////////////////
	Float_Rectangle UsedArea_UV; //area of texture being used as uv coords
	Float_Rectangle ShownArea_UV; //area of texture currently drawn
	
	int NumNodes;
	Asura_Vector_2* Nodes;
	Asura_Vector_2* m_pxInitialNodes;//for storing the unclamped positions of nodes during rotation/resizing
	Asura_Vector_2* BackupNodes;//used to pointer to the normal 4 nodes during multiple node mode
	List<int> SelectedNodeList;//indeces of nodes currently being moved
	int CurrentNodeIndex;//node that will get moved by keyboard commands

	//how the mouse will act
	static int LeftButtonMode;//takes values from enum NodeEditLeftButtonMode
	static int AxisRestrict;//takes values from enum NodeEditAxisRestrict

	

	int ZoomScale;
	float m_fScale; //width in pixels to draw one copy of the texture

	//m_uMaterialResponseHashID, m_iMaterialID , m_iTextureID are alternative ways in which the texture to display
	//can be determined. Only one of those should be set at a time
	//m_iTextureChannel is only relevant when a material is set

	int m_iTextureChannel;//index of channel in material to use

	bool NodesMoved;//flag to indicate whether nodes have been moved this frame

	//for handling left click and drag in node edit mode
	int DragMode;
	Asura_Vector_2 DragInitial;//mouse position at start
	Asura_Vector_2 DragPrevious;//mouse position last frame
	Asura_Vector_2 DragCentre;//centre of nodes at start of drag.. for rotation
	Asura_Vector_2 DragInitial_UV;//uv coordinate mouse was over at start
	bool m_bClearSelectionAfterDrag;

	int m_iToolsTextureIndex;
	
	NodeEditUndo NodeUndo;

	static Asura_Vector_2 MouseModeOverlayPosition;
	static u_int s_uMouseModeOverlayMinimised;

	ASURA_TRANSLUCENCY_MODE m_eTranslucency;

private :

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	// For now, we need to keep the other material ID around as well. Carpet, PFX etc. use the UV editor too
	Asura_Hash_ID m_auMaterialResponseHashID[2];
#endif
	int m_iMaterialID; //index of currently selected material
	int m_iTextureID; 


};




#endif
