//
// Tools_UVEdit.h
//
// Created by Richard Rice - 24/04/2008
// 
// Copyright (C) 2008 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_UVEDIT_H__
#define __TOOLS_UVEDIT_H__



class Tools_UVEdit_Node;
class Tools_UVEdit_NodeManager;
template<class T> class List;

enum Tools_UVEditSelectMode
{
	TOOLS_UVEDITSELECTMODE_VERTEX,
	TOOLS_UVEDITSELECTMODE_EDGE,
	TOOLS_UVEDITSELECTMODE_POLYGON,
};

class Tools_UVEdit
{
	friend class Tools_UVEdit_Overlay;
	friend class Tools_UVEdit_Dialog;
	friend class Tools_UVEdit_Gadget;
	friend class Tools_UVEdit_NodeIterator;

public :
	Tools_UVEdit();
	~Tools_UVEdit();

	void ShowOverlay();
	void CloseOverlay();
	void ToggleOverlay();
	bool IsOverlayOpen() const;

	void ClearMaterialAndTexture();
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	void SetMaterialResponse(int iChannel, Asura_Hash_ID uMaterialResponseHashID);
	#endif
	void SetMaterialID(int iMaterialID);
	void SetTextureID(int iTextureID);

	//update UVs of nodes from the source mesh
	void UpdateNodesFromSource();

	//Update the material/texture being shown based on the polygons selected
	void UpdateMaterial();

	//For displaying the vertex selection in the world
	void RenderWorld();



public :
	int GetEditedChannels() const {return m_iEditedChannels;}
	bool GetShowAllChannels() const {return m_bShowAllChannels;}

	const List<Tools_UVEdit_Node*>* GetNodeList();
	const List<Tools_UVEdit_Node*>* GetSelectedNodeList();

	bool IsAnySelection() const;

	void UpdateEquivalentToSelectionFlags() const;

private :
	void Update();

	//updates labels/scales etc on various dialogs
	void UpdateOverlayControls();

	//Gets the texture displayed based on the selected material/texture channel or selected texture
	int GetTextureToDisplay() const;

	//Converts a UV coordinate to a pixel position within the UV editor
	Asura_Vector_2 ConvertUVToPosition(const Asura_Vector_2& xUV, bool bSnapToPixel) const;
	Asura_Vector_2 ConvertPositionToUV(const Asura_Vector_2& xPos) const;

	Asura_Vector_2 ConvertUVDeltaToPositionDelta(const Asura_Vector_2& xUV) const;


	u_int GetSelectedNodeColour() const {return BlendOverlayAlpha(m_uSelectedNodeColour);}
	u_int GetUnselectedNodeColour() const {return BlendOverlayAlpha(m_uUnselectedNodeColour);}
	u_int GetEquivalentNodeColour() const {return BlendOverlayAlpha(m_uEquivalentNodeColour);}
	u_int GetPrimaryEdgeColour() const {return BlendOverlayAlpha(m_uPrimaryEdgeColour);}
	u_int GetSecondaryEdgeColour() const {return BlendOverlayAlpha(m_uSecondaryEdgeColour);}
	u_int GetSelectionRectangleColour() const {return BlendOverlayAlpha(m_uSelectionRectangleColour);}
	u_int GetGridColour() const {return BlendOverlayAlpha(m_uGridColour);}
	u_int GetGridAxisColour() const {return BlendOverlayAlpha(m_uGridAxisColour);}

	//reset to using default colours
	void SetDefaultColours();
	void SetDefaultColours_Confirm();
	//blends in the alpha from the overlay brightness to the colour
	u_int BlendOverlayAlpha(u_int uColour) const;

	//Zoom in/out by a multiple of 2. 
	//Zoomfocus is the point within the view that should stay fixed
	void ZoomIn(const Asura_Vector_2& xZoomFocus);
	void ZoomOut(const Asura_Vector_2& xZoomFocus);

	void UpdateTextureSize();

	//recalculate bounding rectangle of selection
	void UpdateSelectionRectangle(bool bUpdateCentre);

	//Gets the selection rectangle in pixel coordinates , possibly padded out if too small
	Asura_Bounding_Rectangle GetSelectionRectangleToRender() const;

	//Returns true if suitable nodes are selected for the selection centre to be relavent
	bool ShouldShowSelectionCentre() const;
	bool ShouldShowSelectionRectangle() const;

	//Gets the centre of the selection in pixel coordinates
	Asura_Vector_2 GetSelectionCentre() const;

	//gets the drag rectangle in relative pixel coordinates, clamped to the edit area
	Asura_Bounding_Rectangle GetClampedDragRectangle(const Asura_Vector_2& xRelCursorPos) const;

	//per frame handling of mouse dragging
	void HandleMouse(const Asura_Vector_2& xRelCursorPos, int iButton);
	bool IsDragSelection() const {return (m_eDragMode == TOOLS_UVEDITDRAGMODE_SELECTION || m_eDragMode == TOOLS_UVEDITDRAGMODE_DESELECTION)&& m_bIsMouseDrag;}

	void HandleLeftMouseClick_Freeform(const Asura_Vector_2& xRelCursorPos, bool bClicked);
	void HandleLeftMouseClick_NonFreeform(const Asura_Vector_2& xRelCursorPos, bool bClicked);

	void UpdateAxisRestrict(const Asura_Vector_2& xRelCursorPos);
	
	void StorePreDragState(const Asura_Vector_2& xRelCursorPos);
	void UpdateDrag_Selection(const Asura_Vector_2& xRelCursorPos);
	void UpdateDrag_Rotate(const Asura_Vector_2& xRelCursorPos);
	void UpdateDrag_FreeformRotate(const Asura_Vector_2& xRelCursorPos);
	void UpdateDrag_Move(const Asura_Vector_2& xRelCursorPos);
	void UpdateDrag_MoveCentre(const Asura_Vector_2& xRelCursorPos);
	void UpdateDrag_Scale(const Asura_Vector_2& xRelCursorPos);
	void UpdateDrag_Stretch(const Asura_Vector_2& xRelCursorPos);
	void UpdateDrag_Pan(const Asura_Vector_2& xRelCursorPos);
	void UpdateDrag_ResizeEditArea(u_int uResizeFlags, const Asura_Vector_2& xOffset);

	//Snaps UV coordinate to nearest texel
	Asura_Vector_2 SnapToNearestTexel(const Asura_Vector_2& xUV) const;

	//Initialises settings for the UV editing control
	void UpdateEnabledStateForUVControl();
	void UpdateUVControl();

	//Gets the nearest selectable node to the cursor position
	Tools_UVEdit_Node* Tools_UVEdit::GetNearestNode(const Asura_Vector_2& xPos,float& fDist);
	//Gets the first node in the nearest selectable edge
	Tools_UVEdit_Node* Tools_UVEdit::GetNearestEdge(const Asura_Vector_2& xPos,float& fDist);
	//Gets a node in the polygon containing the position
	Tools_UVEdit_Node* Tools_UVEdit::GetPolygon(const Asura_Vector_2& xPos);

	//for helping with rectangle selection of edges
	bool RectangleIntersectsEdge(const Asura_Bounding_Rectangle& xUVRect, const Tools_UVEdit_Node* pxFirstNode) const;
	bool RectangleIntersectsPolygon(const Asura_Bounding_Rectangle& xUVRect, const Tools_UVEdit_Node* pxFirstNode) const;
	bool PolygonContainsPoint(const Asura_Vector_2& xUVPos, const Tools_UVEdit_Node* pxFirstNode) const;

	//gets uv values for the grid
	float GetGridSizeU() const {return m_fGridSize / m_xTextureSize.x;}
	float GetGridSizeV() const {return m_fGridSize / m_xTextureSize.y;}

	float GetScaleX() const {return m_fScale;}
	float GetScaleY() const {return m_fScale*m_xTextureSize.y/m_xTextureSize.x;}


	//Overlay callbacks
	bool IsChecked_MouseMode_Move()					{return m_eMouseMode == TOOLS_UVEDITMOUSEMODE_MOVE;}
	void SetChecked_MouseMode_Move(bool bSet)		{if(bSet) m_eMouseMode = TOOLS_UVEDITMOUSEMODE_MOVE;}
	bool IsChecked_MouseMode_Rotate()				{return m_eMouseMode == TOOLS_UVEDITMOUSEMODE_ROTATE;}
	void SetChecked_MouseMode_Rotate(bool bSet)		{if(bSet) m_eMouseMode = TOOLS_UVEDITMOUSEMODE_ROTATE;}
	bool IsChecked_MouseMode_Scale()				{return m_eMouseMode == TOOLS_UVEDITMOUSEMODE_SCALE;}
	void SetChecked_MouseMode_Scale(bool bSet)		{if(bSet) m_eMouseMode = TOOLS_UVEDITMOUSEMODE_SCALE;}
	bool IsChecked_MouseMode_Freeform()				{return m_eMouseMode == TOOLS_UVEDITMOUSEMODE_FREEFORM;}
	void SetChecked_MouseMode_Freeform(bool bSet)	{if(bSet) m_eMouseMode = TOOLS_UVEDITMOUSEMODE_FREEFORM;}

	bool IsChecked_SnapMode_Grid()					{return m_eSnapMode == TOOLS_UVEDITSNAP_GRID;}
	void SetChecked_SnapMode_Grid(bool bSet)		{m_eSnapMode = bSet ? TOOLS_UVEDITSNAP_GRID : TOOLS_UVEDITSNAP_NONE;}
	bool IsChecked_SnapMode_Pixel()					{return m_eSnapMode == TOOLS_UVEDITSNAP_PIXEL;}
	void SetChecked_SnapMode_Pixel(bool bSet)		{m_eSnapMode = bSet ? TOOLS_UVEDITSNAP_PIXEL : TOOLS_UVEDITSNAP_NONE;}
	bool IsChecked_SnapMode_Vertex()				{return m_eSnapMode == TOOLS_UVEDITSNAP_VERTEX;}
	void SetChecked_SnapMode_Vertex(bool bSet)		{m_eSnapMode = bSet ? TOOLS_UVEDITSNAP_VERTEX : TOOLS_UVEDITSNAP_NONE;}

	bool IsChecked_SelectMode_Vertex()				{return m_eSelectMode == TOOLS_UVEDITSELECTMODE_VERTEX;}
	void SetChecked_SelectMode_Vertex(bool bSet)  	{if(bSet) m_eSelectMode = TOOLS_UVEDITSELECTMODE_VERTEX;}
	bool IsChecked_SelectMode_Edge()				{return m_eSelectMode == TOOLS_UVEDITSELECTMODE_EDGE;}
	void SetChecked_SelectMode_Edge(bool bSet)  	{if(bSet) m_eSelectMode = TOOLS_UVEDITSELECTMODE_EDGE;}
	bool IsChecked_SelectMode_Polygon()				{return m_eSelectMode == TOOLS_UVEDITSELECTMODE_POLYGON;}
	void SetChecked_SelectMode_Polygon(bool bSet)  	{if(bSet) m_eSelectMode = TOOLS_UVEDITSELECTMODE_POLYGON;}


	bool IsEnabled_TextureSize()					{return m_bOverrideTextureSize;}


	void MirrorHorizontal();
	void MirrorVertical();
	void ZoomExtents();

	static void ShowKeys();

	bool IsEnabled_UVControl();

	//loads/saves settings for the editor
	void TransferModeData(int iMode);

	void RelaxNodes();
	void WeldNodes();

	void OpenScaleOverlay();
	void CloseScaleOverlay();
	void UpdateScaleOverlay();

	void ScaleOverlay_Okay();
	void ScaleOverlay_Cancel();
	void ScaleOverlay_ApplyScale(float fUScale, float fVScale);

	void OpenSettingsOverlay();

	//Allows the user to force a specific texture to be used within the UV editor window
	void SetOverrideTexture();

	//selection undo
	void UndoSelection();
	void RedoSelection();

private :
	enum Tools_UVEditMouseMode
	{
		TOOLS_UVEDITMOUSEMODE_MOVE = 0,
		TOOLS_UVEDITMOUSEMODE_ROTATE = 1,
		TOOLS_UVEDITMOUSEMODE_SCALE = 2,
		TOOLS_UVEDITMOUSEMODE_FREEFORM = 3,
	};

	enum Tools_UVEditDragMode
	{
		TOOLS_UVEDITDRAGMODE_NONE,
		TOOLS_UVEDITDRAGMODE_SELECTION,
		TOOLS_UVEDITDRAGMODE_DESELECTION,
		TOOLS_UVEDITDRAGMODE_FREEFORMROTATE,
		TOOLS_UVEDITDRAGMODE_ROTATE,
		TOOLS_UVEDITDRAGMODE_MOVE,
		TOOLS_UVEDITDRAGMODE_MOVEIFDRAG,
		TOOLS_UVEDITDRAGMODE_MOVECENTRE,
		TOOLS_UVEDITDRAGMODE_SCALE,
		TOOLS_UVEDITDRAGMODE_STRETCH_TL,
		TOOLS_UVEDITDRAGMODE_STRETCH_TR,
		TOOLS_UVEDITDRAGMODE_STRETCH_BL,
		TOOLS_UVEDITDRAGMODE_STRETCH_BR,
		TOOLS_UVEDITDRAGMODE_PAN,
	};

		
	enum Tools_UVEditSnap
	{
		TOOLS_UVEDITSNAP_NONE = 0,
		TOOLS_UVEDITSNAP_GRID = 1,
		TOOLS_UVEDITSNAP_PIXEL = 2,
		TOOLS_UVEDITSNAP_VERTEX = 3,
	};

	

	int m_iOverlayID;

	//pixel size of main UV editing area
	//(actual dialog may be a tad bit larger for a border)
	Asura_Vector_2 m_xEditAreaSize;

	//Number of pixels to represent one tile of the texture (horizontally)
	float m_fScale;

	//Texture size to use for snapping UVs to nearest texel
	bool m_bOverrideTextureSize;
	Asura_Vector_2 m_xTextureSize;
	float m_fTextureBrightness;

	//UV coordinates of top-left of view
	Asura_Vector_2 m_xTopLeftUV;

	//Grid (measured in pixels)
	float m_fGridSize;
	bool m_bShowGrid;

	//restrict UV editing to X/Y axis
	bool m_bButtonXRestrict;
	bool m_bButtonYRestrict;

	bool m_bLockSelection;

	//For editing/showing UVs of a single selected node
	bool m_bUVControlEnabled;
	bool m_bUVControlChanged_U;
	bool m_bUVControlChanged_V;
	bool m_bShowUVAsPixels;
	Asura_Vector_2 m_xSelectedUV;

	int m_iEditedChannels;//bitfield of UV coordinate channels being edited
	bool m_bShowAllChannels;//show now-edited UVs as well , but 'greyed' out
	bool m_bNeedToRecreateNodes;


	//Various ways of defining the texture/material to show
	#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Asura_Hash_ID m_auMaterialResponseHashID[2];
	#endif
	int m_iMaterialID;
	int m_iTextureID;

	bool m_bUseOverrideTexture;
	mutable int m_iOverrideTextureID;
	Asura_String m_xOverrideTextureName;


	Tools_UVEditMouseMode m_eMouseMode;

	Tools_UVEdit_NodeManager* m_pxNodeManager;
	//frame from which the polygon selection was taken
	u_int m_uPolygonSelectionFrameStamp;

	//bounding rectangle of selected nodes (in UV coordinates).
	Asura_Bounding_Rectangle m_xSelectionRectangleUV;

	//Centre of selection to use for rotation
	Asura_Vector_2 m_xSelectionCentreUV;


	//snap to grid/pixel
	Tools_UVEditSnap m_eSnapMode;

	//vertex/edge/polygon selection
	Tools_UVEditSelectMode m_eSelectMode;

	//mouse drag stuff
	Tools_UVEditDragMode m_eDragMode;
	Asura_Vector_2 m_xMouseDragStartRelative;
	Asura_Vector_2 m_xMouseDragStartScreen;
	bool m_bIsMouseDrag;//indicates mouse has moved enough to be considered a drag selection
	Asura_Bounding_Rectangle m_xPreDragSelectionRectangleUV;
	Asura_Vector_2 m_xPreDragSelectionCentreUV;
	Asura_Vector_2 m_xPreDragTopLeftUV;
	Asura_Vector_2 m_xPreDragEditAreaSize;
	Asura_Vector_2 m_xPreDragOverlayPosition;
	Asura_Vector_2 m_xPreDragNodeUV;//Initial UV of node clicked on for drag
	//deals with current state of axis restrict while dragging
	bool m_bDragXRestrict;
	bool m_bDragYRestrict;
	bool m_bShiftXRestrict;
	bool m_bShiftYRestrict;

	bool m_bUpdateOverlayControls;

	int m_iRelaxNode_NumIteration;
	float m_fRelaxNode_Amount;
	bool m_bRelaxNode_LockBoundary;

	Asura_Angle m_fRotationSnap;//snap interval for rotation

	bool m_bScaleOverlay_Active;
	float m_fScaleOverlay_U;
	float m_fScaleOverlay_V;
	bool m_bScaleOverlay_Uniform;
	bool m_bScaleOverlay_UChanged;
	bool m_bScaleOverlay_VChanged;
	Asura_Vector_2 m_xPreScaleSelectionCentreUV;

	//For rendering verts in 3D view
	bool m_bRenderSelectedVerts;
	bool m_bRenderCursorVert;
	const Tools_UVEdit_Node* m_pxNodeUnderCursor;


	//colours
	u_int m_uSelectedNodeColour;
	u_int m_uUnselectedNodeColour;
	u_int m_uEquivalentNodeColour;
	u_int m_uPrimaryEdgeColour;
	u_int m_uSecondaryEdgeColour;
	u_int m_uSelectionRectangleColour;
	u_int m_uGridColour;
	u_int m_uGridAxisColour;
	u_int m_uSelectedVertexColour;
	u_int m_uCursorVertexColour;

	
	static Asura_Vector_2 s_xPosition;
	static u_int s_uMinimised;
	static Asura_Vector_2 s_xScaleOverlayPosition;
	static Asura_Vector_2 s_xSettingsOverlayPosition;

};

#endif //__TOOLS_UVEDIT_H__
