//
// Tools_Mode.h
//
// Created by Kevin Lea - 20/06/2001 14:47:19
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_MODE_H__
#define __TOOLS_MODE_H__

class Tools_Mode
{
public:
	Tools_Mode()
	{
	}
	virtual ~Tools_Mode()
	{
	}

	virtual void Update(void){}

	//Render scene elements before tone mapping (things that should look like part of the world)
	virtual void RenderWorld_PreToneMap(void){}

	//Render scene elements after tone mapping (3D boxes and spheres, flashing polygons etc.)
	virtual void RenderWorld_PostToneMap(void){}

	//EarlyRender is called early enough so that anything drawn will appear in screenshots.
	//Basically a semi-hack to allow cutscenes to appear in tools , without the other tools specific junk
	//RAR 13/08/02
	virtual void EarlyRender(void){}

	//Render 2D elements that aren't part of the tools UI.
	//Only really applies to menu editing and the like
	virtual void Render2D(){}

	//Render 2D elements that are part of the tools UI
	virtual void RenderUI(void){}


	//Returns true if the RenderWorld()/RenderUI functions should be called while doing a modal overlay
	virtual bool ShouldRenderDuringModalOverlay() {return false;}

	//New Initialise/shutdown fucntions called by Tools_System after creation/before destruction
	//(New versions created to get around the problem of not being able to call them as virtual functions
	//with the constructor/destructor).
	virtual void InitialiseMode(){}
	virtual void ShutdownMode(){}

	//QuickSave overridden for modes that need to save things other than the .asura file	
	virtual void QuickSave(void){};


	virtual bool IsModified(){return false;}

	//called before changing modes (before the save prompt)
	virtual void PreShutdown(){};

	//so modes can hide/unhide objects as needed for editing message blocks
	virtual void PreMessageBlockEditing(){};
	virtual void PostMessageBlockEditing(){};

	//bool IsActive(void){return};

	// Prepare mode for save.
	virtual void PreSave() {}
	virtual void PostSave() {}

	//Called when a module's mesh gets changed behind the mode's back.
	//Any cached data based on the mesh will need to be fixed up as best it can.
	virtual void HandleMeshChange(int iModuleIndex) {};

	//Override to add mode-specific stuff to the right-click menu
	virtual void AddContextMenuItems(class Tools_ContextMenu& xMenu, const class Tools_ContextMenu_Context& xContext){}

};

//defines for static overlay functions that will use ThisMode to call the non static versions
#define STATIC_VOID_FN(functionName)\
	static void _ ## functionName (){s_pxThis->functionName();}\
	void functionName();

#define STATIC_BOOL_FN(functionName)\
	static bool _ ## functionName (){return s_pxThis->functionName();}\
	bool functionName();

#endif//__TOOLS_MODE_H__
