//
// Tools_System.h
//
// Created by Kevin Lea - 25/06/2001 14:53:13
// 
// Copyright (C) 2001 Rebellion, All Rights Reserved.
//
 
#ifdef ASURA_USE_PRAGMA_ONCE
    #pragma once
#endif

#ifndef __TOOLS_SYSTEM_H__								 
#define __TOOLS_SYSTEM_H__

#include "Core/Asura_Handle.h"
#include "Misc/Tools_Modal_Overlay.h"

#if defined( ASURA_USE_PROFILING_SYSTEM)
	#include "Debugging/Asura_CPUProfile_Renderer.h"
#endif

#if defined(ASURA_TARGET_SM3)
	#include "Rendering/Asura_DeferredShading.h"
#endif

#include "ChunkSystem/Asura_Chunk_FileInfo.h"

enum TOOLS_MODE
{
	TOOLS_MODE_NULL,//not a mode as such , but used as a possibility for the next mode
	TOOLS_MODE_QUITTOGAME,//likewise
	TOOLS_MODE_QUITTODESKTOP,//ditto
	TOOLS_MODE_STANDBY,
	TOOLS_MODE_SOUND,
	TOOLS_MODE_TEXTURING,
	TOOLS_MODE_TEXTURETILER,
	TOOLS_MODE_SMOOTHING,
	TOOLS_MODE_FLAGS,
	TOOLS_MODE_MODULELINKING,
	TOOLS_MODE_WAYPOINTING,
	TOOLS_MODE_MESHEDIT,
	TOOLS_MODE_CUTSCENE,
	TOOLS_MODE_OBJECT,
	TOOLS_MODE_ASSETMANAGER,
	TOOLS_MODE_HIERARCHY,
	TOOLS_MODE_PARTICLES,
	TOOLS_MODE_LOD,
	TOOLS_MODE_REGIONS,
	TOOLS_MODE_BILLBOARDPLANTS,
	TOOLS_MODE_WEATHER,
	TOOLS_MODE_MODULE,
	TOOLS_MODE_EXTENDEDPARTICLES,
	TOOLS_MODE_SHAPEDATA,
	TOOLS_MODE_EXTENDEDWEATHER,
	TOOLS_MODE_METAMUSIC,
	TOOLS_MODE_PHOTONMAPPING,
	TOOLS_MODE_PFX,
	TOOLS_MODE_BOTANICALS,
	TOOLS_MODE_BOTANICALCREATION,
	TOOLS_MODE_CARPET,
	TOOLS_MODE_FSFX,
	TOOLS_MODE_STREAMING,
	TOOLS_MODE_ATTRACTORS,
	TOOLS_MODE_VERTEXPAINTING,
	TOOLS_MODE_ANIMATION,
	TOOLS_MODE_GUIMENU,
	TOOLS_MODE_DECALS,
	TOOLS_MODE_CONTAINER,
	TOOLS_MODE_RAGDOLLEDITOR,

	TOOLS_MODE_FIRSTPROJECTSPECIFIC,//starting point for project specific modes
};

class Asura_Menubar_Title;

class Tools_System
{
	public:
		Tools_System();
		virtual ~Tools_System(){This = NULL;}

		static bool IsActive( void )
		{
			return ( s_bToolsActive );
		}
		//returns true during Tools_System::Shutdown()
		static bool IsShuttingDown() {return s_bShuttingDown;}

		//allocate/deallocate the tools system class.
		static void Allocate();
		static void Deallocate();
		
		static void Initialise(bool loadEnvironment = false);
		static bool MainLoop(void);
		static void Shutdown(void);

        // (MH - 22.07.2002) added network init, etc. to cope with server-side cutscenes
        static void NetworkMessageHandler(class Asura_Message *message);

		//extra initialising/shutdown for project specific stuff. (adding extra menu items and the like)
		static void Project_Initialise();
		static void Project_Shutdown();

		//allocate the single Tools_System class instance (or derived class)
		static void Project_Allocate_Tools_System();

		//get the directory containing inifiles for flag names etc for this project.
		static const char* Project_GetIniFileDirectory();

		static void Render();

		//project specific per-frame update for tools
		virtual void Project_Render(){}
		//project specific per-frame Render for tools
		virtual void Project_LateRender(){}

		//project specific per-frame update for tools
		virtual void Project_Update(){}

		// Project-specific function to create save/exit confirmation overlays...
		virtual Tools_Modal_Overlay* CreateExitConfirmationOverlay(const TOOLS_MODE eNextMode) const;
		virtual Tools_Modal_Overlay* CreateSaveConfirmationOverlay(const TOOLS_MODE eNextMode) const;

		//calls constructors for each of the tools classes that require it
		void AllocateAllToolsClasses();
		void DeallocateAllToolsClasses();

		//Allocates tools system class , and other project derivable classes
		//Used by batch processing to allow access to some virtual functions before
		//tools is ready to be initialised properly. RAR 29/08/2006
		static void AllocateToolsSystemAndToolsClasses();
		static void DeallocateToolsSystemAndToolsClasses();

		//Sets a default mode to enter based on the extension.
		static void SetDefaultMode(const char* szExtension);

		// Sets a default mode to enter based on the file type 
		static void SetDefaultMode( const AsuraFileType eFileType );

		
		//virtual functions for allocating the various tools systems that can be derived from
		//for project specific stuff
		virtual void Allocate_Tools_SaveAsura();
		virtual void Allocate_Tools_Level_Export();
		virtual void Allocate_Tools_Animation_Files();
		virtual void Allocate_Tools_Message_Editing();
#if defined(ASURA_SUPPORT_BODYPART_SYSTEM)
		virtual void Allocate_Tools_Hierarchy_BodyPart_System();
#endif	// defined(ASURA_SUPPORT_BODYPART_SYSTEM)
		virtual void Allocate_Tools_ShapeTable();
		virtual void Allocate_Tools_PickTexture();
		virtual void Allocate_Tools_PickTiledTexture();
		virtual void Allocate_Tools_OptionsFile();
		virtual void Allocate_Tools_BatchWaypointing();
		virtual void Allocate_Tools_EntityTemplate_System();
		virtual void Allocate_Tools_Movement();
#ifdef ASURA_USE_DIALOGUE_SYSTEM
		virtual void Allocate_Tools_Dialogue_System();
#endif
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
		virtual void Allocate_Tools_Texturing_MaterialResponse();
#endif
		virtual void Allocate_Tools_LightSets();


		// Adds all the export options to the File title bar
		virtual void AddExportOptions( Asura_Menubar_Title* pxTitle );

		//Gets the default height for the camera when using dynamics
		virtual float GetDefaultPlayerHeight(){return 1.8f;}
		virtual float GetDefaultCrouchHeight(){return 1.0f;}
		virtual float GetDefaultProneHeight(){return 0.3f;}

        virtual u_int GetPS2RenderedTriLimit( void ) const { return 12500; }
        virtual u_int GetPS2TextureUploadLimit( void ) const { return 4 * 1024 * 1024; }
        virtual u_int GetPS2CollisionPolyLimit( void ) const { return 25; }


		//Functions for defining project specific polygon flags.
		//(put here instead of Tools_Mode_Flags so they can be accessed outside of flags mode).
		virtual int GetNumberOfProjectPolygonFlags() const {return 0;}
		//Get the flag value for the iIndex'th polygon flag
		virtual int GetProjectPolygonFlagValue(int iIndex) const {return 0;}
		//Get the flag name for the iIndex'th polygon flag
		virtual const char* GetProjectPolygonFlagName(int iIndex) const {return 0;}
		//Get the (optional) help text for the iIndex'th polygon flag
		virtual const char* GetProjectPolygonFlagHelpText(int iIndex) const {return 0;}

		//Functions for defining project specific vertex flags
		//(put here instead of Tools_Mode_Flags so they can be accessed outside of flags mode).
		virtual int GetNumberOfProjectVertexFlags() const {return 0;}
		//Get the flag value for the iIndex'th polygon flag
		virtual u_int GetProjectVertexFlagValue(int iIndex) const {return 0;}
		//Get the flag name for the iIndex'th polygon flag
		virtual const char* GetProjectVertexFlagName(int iIndex) const {return 0;}
		//Get the (optional) help text for the iIndex'th polygon flag
		virtual const char* GetProjectVertexFlagHelpText(int iIndex) const {return 0;}


		//Determine whether the timer should be updated in MainLoop().
		//Default is/was to deal with it there , but RogueTrooper now handles
		//the timer elsewhere.
		virtual bool HandleTimerInMainLoop() const{ return true;}
		
		// Allow Tools to export global-sound data in "Common.asr" instead of "GmSnd.asr"...
		virtual bool CanExportGlobalSoundsInCommon(void) const	{ return(false); }

		virtual void ConvertEnvStartPositionsToEntities();

		//Displays warning message if any non-scenery modules have too many triangles (dependent
		//on the project settings). Returns true if the level exports should be aborted
		static bool WarnAboutModulesWithTooManyTriangles();

		//Override and return something other than zero to set the warning level
		//for modules with too many triangles
		virtual int GetModuleTriangleWarningLevel() const { return 0;}

		//Override and return something other than zero to set the error level
		//for modules with too many triangles
		virtual int GetModuleTriangleErrorLevel() const { return 0;}


		static void RequestAutoQuitToGame(void)
		{
			s_bAutoQuitToGame = true;
		}

		static void RequestQuitToGame(void)
		{
			RequestNextMode(TOOLS_MODE_QUITTOGAME);
		}
		static void RequestQuitToDesktop(void)
		{
			RequestNextMode(TOOLS_MODE_QUITTODESKTOP);
		}

		static void OpenIniFiles();
		

		static int GetCurrentMode(void)
		{
			return CurrentMode;
		}
		static class Tools_Mode* GetCurrentModeClass() {return CurrentModeClass;}

		static void HandleOverlays(void);

		static void RequestStandbyMode();
		static void RequestStandbyMode(bool bSuppressSavePrompt);
		static bool IsInStandbyMode(void);

		static void RequestTexturingMode();
		static bool IsInTexturingMode();

		static void RequestTextureTilerMode();
		static bool IsInTextureTilerMode();

		static void RequestSoundMode(void);
		static bool IsInSoundMode(void);

		static void RequestSmoothingMode(void);
		static bool IsInSmoothingMode(void);

		static void RequestFlagsMode(void);
		static bool IsInFlagsMode(void);

		static void RequestModuleLinkingMode(void);
		static bool IsInModuleLinkingMode(void);

		static void RequestWaypointingMode(void);
		static void RequestWaypointingModeSuppressSavePrompt(void);
		static bool IsInWaypointingMode(void);
			
		static void RequestMeshEditMode(void);
		static bool IsInMeshEditMode(void);

		static void RequestParticlesMode(void);
		static bool IsInParticlesMode(void);

		static void RequestCutsceneMode(void);
		static bool IsInCutsceneMode(void);

		static void RequestObjectMode(void);
		static bool IsInObjectMode(void);

		static void RequestHierarchyMode(void);
		static bool IsInHierarchyMode(void);

		static void RequestRagdollEditorMode(void);
		static bool IsInRagdollEditorMode(void);

		static void RequestLODMode(void);
		static bool IsInLODMode(void);

		static void RequestRegionsMode(void);
		static bool IsInRegionsMode(void);

		static void RequestBillboardPlantsMode();
		static bool IsInBillboardPlantsMode();

		static void RequestWeatherMode();
		static bool IsInWeatherMode();

		static void RequestModuleMode();
		static bool IsInModuleMode();

#ifndef ASURA_SUPPORT_DYNAMIC_MUSIC_SYSTEM
		static void RequestMetaMusicMode();
		static bool IsInMetaMusicMode();
#endif

		static void RequestShapeDataMode();
		static bool IsInShapeDataMode();

		static void RequestPhotonMappingMode();
		static bool IsInPhotonMappingMode();

		static void RequestPFXMode();
		static bool IsInPFXMode();

#ifdef ASURA_USE_BOTANICALS
		static void RequestBotanicalsMode();
		static bool IsInBotanicalsMode();

		static void RequestBotanicalCreationMode();
		static bool IsInBotanicalCreationMode();
#endif // #ifdef ASURA_USE_BOTANICALS

#ifdef ASURA_USE_ASSET_MANAGER
		static void RequestAssetManagerMode();
		static bool IsInAssetManagerMode();
#endif

#ifdef ASURA_USE_CARPET
		static void RequestCarpetMode();
		static bool IsInCarpetMode();
#endif // #ifdef ASURA_USE_CARPET

#ifdef ASURA_USE_FSFX
		static void RequestFSFXMode();
		static bool IsInFSFXMode();
#endif

#ifdef ASURA_USE_ATTRACTORS
		static void RequestAttractorsMode();
		static bool IsInAttractorsMode();
#endif

#ifdef ASURA_USE_STREAMING
		static void RequestStreamingMode();
		static bool IsInStreamingMode();
#endif

		static void RequestVertexPaintingMode();
		static bool IsInVertexPaintingMode();

#ifdef ASURA_USE_GUI_MENU_SYSTEM
		static void RequestGUIMenuMode();
		static bool IsInGUIMenuMode();
#endif

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
		static void RequestDecalsMode();
		static bool IsInDecalsMode();
#endif

#ifdef ASURA_USE_CONTAINER_SYSTEM
		static void RequestContainerMode();
		static bool IsInContainerMode();
#endif //ASURA_USE_CONTAINER_SYSTEM

		//can be called after exiting tools to determine whether the quit to desktop option was chosen
		static bool ShouldQuitToDesktop(){return QuitToDesktop;}


		static void RequestNextMode(int mode,bool bSuppressSavePrompt = false);
		static void SaveAllAndChangeToNextMode();
		static void SaveAndChangeToNextMode();
		static void SaveModifiedAndChangeToNextMode();
		static void ChangeToNextMode();
		static void CancelModeChange();
		
		static void Project_ChangeToNextMode();

		static void QuickSave();

		static void ToggleShowAllModuleOutlines(){ShowAllModuleOutlines=!ShowAllModuleOutlines;};
		static bool IsShowingAllModuleOutlines(){return ShowAllModuleOutlines;}

		static void ToggleShowCursorModuleOutline(){ShowCursorModuleOutline=!ShowCursorModuleOutline;};
		static bool IsShowingCursorModuleOutline(){return ShowCursorModuleOutline;}


		static bool IsSwappingMouseButtons(){return s_bSwapMouseButtons;}
		static void SetSwapMouseButtons(bool bSet);

		static bool IsRestrictMouseToWindowInSubwindowTools(){return s_bRestrictMouseToWindowInSubwindowTools;}
		static void SetRestrictMouseToWindowInSubwindowTools(bool bSet);
		static bool IsRestrictMouseToWindowInSubwindowGame(){return s_bRestrictMouseToWindowInSubwindowGame;}
		static void SetRestrictMouseToWindowInSubwindowGame(bool bSet);
		static bool IsRestrictMouseToWindowInFullScreenTools(){return s_bRestrictMouseToWindowInFullScreenTools;}
		static void SetRestrictMouseToWindowInFullScreenTools(bool bSet);
		static bool IsRestrictMouseToWindowInFullScreenGame(){return s_bRestrictMouseToWindowInFullScreenGame;}
		static void SetRestrictMouseToWindowInFullScreenGame(bool bSet);
		static void UpdateRestrictMouseToWindow();
		static void SetUseWindowsCursor(bool bSet);
		static bool GetUseWindowsCursor(){return s_bUseWindowsCursor;}

		static void ViewTextures();

		static void (*UpdateCurrentMode)(void);

		static int MakeListOfFiles(const char* load_name, char**& filenames, char**& shortnames);

		static void RenderAllModuleOutlines();
		static void RenderCursorModuleOutline();

		static void ScreenShot(bool bSuppressText = false); // Text suppression only turns off the success message, as we probably still want to know when it's failed
		static void StartAVI();
		static void EndAVI();

		static void RenderCursorInformation();
		static void ToggleShowCursorInformation(){ShowCursorInformation=!ShowCursorInformation;}
		static bool IsShowingCursorInformation(){return ShowCursorInformation;}

		// Allows mode to hide tools status text (FPS, 
		// orient and pos text displayed in top left 
		// corner of screen) - NDB.
		static void SetHideToolsStatusText( const bool bHide ){ s_bHideToolsStatusText = bHide; }
		static bool IsHidingToolsStatusText(){ return s_bHideToolsStatusText; }

#if defined( ASURA_USE_PROFILING_SYSTEM)
		static void ToggleShowProfileBars(){ Asura_CPUProfile_Renderer::EnableBars( false == Asura_CPUProfile_Renderer::IsBarsEnabled());}
		static bool IsShowingProfileBars(){ return( true == Asura_CPUProfile_Renderer::IsBarsEnabled());}
		static void ToggleShowProfileText(){ Asura_CPUProfile_Renderer::EnableText( false == Asura_CPUProfile_Renderer::IsTextEnabled());}
		static bool IsShowingProfileText(){ return( true == Asura_CPUProfile_Renderer::IsTextEnabled());}
#endif

#if defined( ASURA_TARGET_SM3)		
		static void ToggleShowDeferredBuffers(){ Asura_DeferredShading::SetShowDeferredBuffers( false == Asura_DeferredShading::IsShowingDeferredBuffers());}
#endif

        static void RenderPS2PerfMetrics( void );
        static void ToggleShowPS2PerfMetrics( void ) { s_bShowPS2PerfMetrics = !s_bShowPS2PerfMetrics; }
        static bool IsShowingPS2PerfMetrics( void ) { return s_bShowPS2PerfMetrics; }

		static bool ShouldNotCompressTexturesOnLoad(void)
		{
			return s_bDontCompressTexturesOnLoad;
		}

		
		//asks for a colour sample to be calculated next frame
		static void RequestColourSampleUnderCursor(int iSampleSize);
		//gets the most recently calculated sample
		static void GetColourSampleUnderCursor(float* pfRed,float* pfGreen,float* pfBlue);

		//displays a screen listing all polygon counts
		static void ShowPolygonCounts();

		static void ApplicationReactivated(){s_bHasBeenMinimised = true;s_uReactivationCount++;}
		//GetReactivationCount() is increased whenever tools regains focus ,and can be used for determining
		//whether any alt-tabbing has being going on.
		static u_int GetReactivationCount() {return s_uReactivationCount;}

		static void SetRecordScreenshots(bool bRecord)
		{
			s_bRecordScreenshots = bRecord;
		}
		static bool IsRecordingScreenshots()
		{
			return s_bRecordScreenshots;
		}

		//adds controls to general options overlay
		static void AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset, int iTab);


#ifdef ASURA_USE_BOTANICALS
		static void ToggleHideBotanicals();
		static bool IsHideBotanicalsChecked() {return s_bHideBotanicals;};
		static void ToggleBotanicalsStatistics()		  {s_bBotanicalsStatistics = !s_bBotanicalsStatistics;};
		static bool IsBotanicalsStatisticsChecked()	  {return s_bBotanicalsStatistics;};
#endif // #ifdef ASURA_USE_BOTANICALS

#ifdef ASURA_USE_CARPET
		static void ToggleHideCarpet()		  {s_bHideCarpet=!s_bHideCarpet;};
		static bool IsHideCarpetChecked()	  {return s_bHideCarpet;};
		static void ToggleCarpetStatistics()		  {s_bCarpetStatistics = !s_bCarpetStatistics;};
		static bool IsCarpetStatisticsChecked()	  {return s_bCarpetStatistics;};
#endif // #ifdef ASURA_USE_CARPET

		//accessors for saving/loading options... doesn't update Asura_Windows immediately
		static bool GetIdleWhileMinimised(){return s_bIdlePriorityWhileMinimised;}
		static void SetIdleWhileMinimised(bool bSet) {s_bIdlePriorityWhileMinimised = bSet;}

		// Sound menu enabled menu item functions
		static bool SoundMenuEnabled_SwitchToSoundMode	();
		static bool SoundMenuEnabled_SoundOptions		();
		static bool SoundMenuEnabled_DialogueSystem		();
		static bool SoundMenuEnabled_DynamicMusicSystem	();
		static bool SoundMenuEnabled_DSPEffects			();
		static bool SoundMenuEnabled_ReverbDefintions	();
		static bool SoundMenuEnabled_StreamingSoundList	();
		static bool SoundMenuEnabled_Mixers				();
		static bool SoundMenuEnabled_AudioMonitor		();

		static bool SoundMenuTicked_SwitchToSoundMode	();
		static bool SoundMenuTicked_SoundOptions		();
		static bool SoundMenuTicked_DialogueSystem		();
		static bool SoundMenuTicked_DynamicMusicSystem	();
		static bool SoundMenuTicked_DSPEffects			();
		static bool SoundMenuTicked_ReverbDefintions	();
		static bool SoundMenuTicked_StreamingSoundList	();
		static bool SoundMenuTicked_Mixers				();
		static bool SoundMenuTicked_AudioMonitor		();

		static void SoundMenuAction_SwitchToSoundMode	();
		static void SoundMenuAction_SoundOptions		();
		static void SoundMenuAction_DialogueSystem		();
		static void SoundMenuAction_DynamicMusicSystem	();
		static void SoundMenuAction_DSPEffects			();
		static void SoundMenuAction_ReverbDefintions	();
		static void SoundMenuAction_StreamingSoundList	();
		static void SoundMenuAction_Mixers				();
		static void SoundMenuAction_AudioMonitor		();
		
		static bool s_bHideBotanicals;
		static bool s_bHideCarpet;
		static bool s_bCarpetStatistics;
		static bool s_bBotanicalsStatistics;


		static bool ShowAllModuleOutlines;
		static bool ShowCursorModuleOutline;
		static bool ShowCursorInformation;
        static bool s_bShowPS2PerfMetrics;

		static bool ReadyToQuitOut;

		static int CurrentMode;
		static int NextMode;

		static class Tools_Mode* CurrentModeClass;

		static Asura_Handle BaseMenuHandle;

		static bool RequestHardwareMouse;

		static float UserSetGameTimeScale;//last value set by the user , as opposed to by cutscenes or whatever


		static bool s_bDontCompressTexturesOnLoad;
        static bool s_bIsCurrentDirectorySet;

		//contains flag section names that are common to different projects 
		//(though the actual names of flags may differ)
		static class Tools_Flag_IniFile* CommonFlagIni;

		//'this' substitute for all the static functions
		static Tools_System* This;

	private :
		void CalculateColourSample();

		static void SwapMouseButtonsClicked();
		static void UseWindowsCursorClicked();
		
		static void IdleWhileMinimisedClicked();
		
		static void DoFirstFrameChecksIfRequired();

		static void CopyPositionToClipboard();

		//Resets various Special FX etc when returning from game
		static void ResetSystemsAfterReturningFromGame();

		static bool QuitToDesktop;
		static bool s_bAutoQuitToGame;

		int InitialPatchesSubdivLevel;//store state of Asura_Render::PatchesSubdivLevel for restoration upon exit
		bool m_bInitialPauseWhileMinimised;

		int m_iColourSample_RequestedSize;
		float m_fColourSample_Red;
		float m_fColourSample_Green;
		float m_fColourSample_Blue;

		static bool s_bHasBeenMinimised;
		static u_int s_uReactivationCount;

		static bool s_bSwapMouseButtons;//indicates left/right button swapped
		static bool s_bInitialSwapMouseButtons;//state of button swapping when entering tools

		static bool s_bRestrictMouseToWindowInSubwindowTools;
		static bool s_bRestrictMouseToWindowInSubwindowGame;
		static bool s_bRestrictMouseToWindowInFullScreenTools;
		static bool s_bRestrictMouseToWindowInFullScreenGame;
		static bool s_bUseWindowsCursor;

		static bool s_bIdlePriorityWhileMinimised;

		static bool s_bRecordScreenshots;

		static bool s_bToolsActive;
		static bool s_bShuttingDown;

		static bool s_bHideToolsStatusText;  // if true FPS, orient and pos text will not be shown

		//default mode to enter when entering tools.
		static int s_iDefaultMode;
};


class Tools_ExitConfirmationOverlay : public Tools_Modal_Overlay
{
public :
	Tools_ExitConfirmationOverlay(TOOLS_MODE eNextMode);

protected :
	virtual void InitialiseOverlay(class Asura_Overlay* pxOverlay);
	virtual void Update();
};

class Tools_SaveConfirmationOverlay : public Tools_Modal_Overlay
{
public :
	Tools_SaveConfirmationOverlay();

protected :
	virtual void InitialiseOverlay(class Asura_Overlay* pxOverlay);
	virtual void Update();
};

#endif//__TOOLS_SYSTEM_H__

