//
// Tools_Mode_VertexPainting.h
//
// Created by JGP 12/06/06
//
// Copyright (C) 2006 Rebellion (Derby), All Rights Reserved.
//

#ifdef ASURA_USE_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TOOLS_MODE_VERTEXPAINTING_H__
#define __TOOLS_MODE_VERTEXPAINTING_H__


// base class
#include "Tools_Mode.h"

// required
#include "Core/Asura_Maths.h"
#include "Core/Asura_HeapArray.h"
#include "Misc/Tools_DynamicArray.h"

class Asura_Mesh_Vertex;
class Asura_Vertex_3;

/*
	Tools_Mode_VertexPainting
*/
class Tools_Mode_VertexPainting : public Tools_Mode
{

public:

	Tools_Mode_VertexPainting();
	virtual ~Tools_Mode_VertexPainting();

	// Tools_Mode functions

	virtual void InitialiseMode();
	virtual void ShutdownMode();

	// set modified

	virtual void QuickSave();
	virtual bool IsModified();
	virtual void Update();
	virtual void RenderWorld_PostToneMap();

	virtual void HandleMeshChange(int iModuleIndex);

private:

	enum VertexOperation
	{
		OP_REPLACE,
		OP_ADD,
		OP_SUBTRACT,
		OP_MODULATE,
		NUM_OPS
	};
	
	class ModuleInformation
	{
	public:
		ModuleInformation();
		
	public:
		bool m_bVisible;
	};

	// update

	void HandleKeyboard();

	// Painting

	void UpdatePainting();
	void RenderPainting();
	void RenderPainting_Start();
	void RenderPainting_Cube( const Asura_Vector_3& xMeshVertex, const uint32 uColour);
	void RenderPainting_Stop();

	// Initialise / Close Overlay

	void InitialiseOverlay();
	void CloseOverlay();

	// callbacks

	STATIC_VOID_FN( OnApply );
	STATIC_BOOL_FN( IsApplyEnabled );

	STATIC_VOID_FN( ResetAll );

	// functions

	void OnApplyVertex();
	void OnApplyVertexPerPoly();
	void ApplyTo( unsigned int& uColour, unsigned int uModulateColour, uint32 uMaskColour);
	const unsigned int MakeColour( const int iR, const int iG, const int iB );
	void InvalidateAllLighting();
	
	uint32 GetRGBMask();

	STATIC_VOID_FN( Undo );
	STATIC_VOID_FN( Redo );
	
	// module control
	STATIC_VOID_FN( HideSelectedModules );
	STATIC_VOID_FN( UnHideAllModules );

	// static this pointer

	static Tools_Mode_VertexPainting* s_pxThis;

	// static data

	static Asura_Vector_2 s_xOverlayPosition;

	// data

	bool m_bVertexPerPolySelection;
	bool m_bPaintingActive;
	bool m_bPaintingOnFloor;
	bool m_bPaintingOn;
	Asura_Vector_3 m_xPaintingPosition;
	int m_iPaintingiModule, m_iPaintingPoly;
	float m_fPaintingRadius;
	Tools_DynamicArray<struct ModulePolyVertTime> m_xPaintedVerts;

	float m_fColourRed, m_fColourGreen, m_fColourBlue;
	VertexOperation m_eOp;
	class VertexPaintingUndo* m_pxUndoBuffer;
	
	Asura_HeapArray< ModuleInformation> m_axModuleInformation;
	
	bool m_bPaintMaskR;
	bool m_bPaintMaskG;
	bool m_bPaintMaskB;
};


#endif // __TOOLS_MODE_VERTEXPAINTING_H__
