/**
*
* \file Tools_Wii_DisplayList.cpp
*
* Created by Andrew Southgate - 30/08/2007
*
* Copyright (C) 2007 Rebellion, All Rights Reserved.
*
* \brief
*
**/


#include "Core/Asura.h"

// This file's header
#include "Tools_Wii_DisplayList.h"

//#define CHECK_WII_OUTPUT

struct Wii_TriangleStrip
{
	u_int m_uNumIndicies;
	Wii_TriangleStrip *m_pNext;
	Wii_TriangleStrip *m_pPrev;

	u_short* TriStrip()
	{
		return (u_short*)(((char*)this) + sizeof(*this));
	}
	u_short GetIndex(u_int uIndex)
	{
		return TriStrip()[uIndex];

	}
	u_int NumIndicies()
	{
		return m_uNumIndicies;
	}
	void Release()
	{
		delete [] (char*)this;
	}
	void LinkAfter(Wii_TriangleStrip *pPrev)
	{
		m_pPrev = pPrev;
		if (pPrev)
		{
			m_pNext = pPrev->m_pNext;
			if (m_pNext)
				m_pNext->m_pPrev = this;
			pPrev->m_pNext = this;
		} else {
			m_pNext = NULL;
		}
	}

	static Wii_TriangleStrip *Allocate(u_int uNumIndicies, const u_short *pusIndicies, bool bReverseWinding)
	{
		u_int uPad = bReverseWinding && uNumIndicies > 4 ? 1 : 0;

		Wii_TriangleStrip* pxRet = (Wii_TriangleStrip*)new char[sizeof(Wii_TriangleStrip) + (uNumIndicies + uPad) * sizeof(short)];

		if (pxRet) {
			pxRet->m_uNumIndicies = uNumIndicies + uPad;
			pxRet->m_pNext = NULL;
			pxRet->m_pPrev = NULL;
			if (uPad)
				*pxRet->TriStrip() = *pusIndicies;
			memcpy(pxRet->TriStrip() + uPad, pusIndicies, uNumIndicies * sizeof(short));
			if (bReverseWinding && !uPad)
			{
				u_short usTmp = pxRet->TriStrip()[1];
				pxRet->TriStrip()[1] = pxRet->TriStrip()[2];
				pxRet->TriStrip()[2] = usTmp;
			}
		}
		return pxRet;
	}
};

class Tools_Wii_StripSet
{
public:
	Wii_TriangleStrip *m_pHead;
	Wii_TriangleStrip *m_pTail;
	u_int m_uNumStrips;
	u_int m_uNumIndicies;
	u_int m_uNumTris;

	Tools_Wii_StripSet()
	{
		m_pHead = NULL;
		m_pTail = NULL;
		m_uNumStrips = 0;
		m_uNumIndicies = 0;
		m_uNumTris = 0;
	}
	~Tools_Wii_StripSet()
	{
		while (m_pHead)
		{
			Wii_TriangleStrip *pStrip = m_pHead;
			m_pHead = m_pHead->m_pNext;
			pStrip->Release();
		}
	}

	void ParseStrips(const unsigned short* pusIndicies, u_int uNumIndicies);
};

#ifdef CHECK_WII_OUTPUT
static void CheckForDegenerates(const unsigned short* pusIndicies, u_int uNumIndicies)
{
	unsigned int uLastSeen[2] = {0xFFFFFFFF, 0xFFFFFFFF};

	for(u_int uIndex = 0; uIndex < uNumIndicies; uLastSeen[(uIndex++) & 1] = pusIndicies[uIndex])
	{
		Asura_Assert(uLastSeen[0] != pusIndicies[uIndex] && uLastSeen[1] != pusIndicies[uIndex], "");
	}

}

class Triangle
{
public:
	Triangle *pNext;
	Triangle *pPrev;
	u_int uVerts[3];
	bool bMarker;

	Triangle *Next()
	{
		return pNext;
	}
	void Link(Triangle *pxPrev, Triangle *pxNext)
	{
		pPrev = pxPrev;
		pNext = pxNext;
		if (pxPrev)
			pxPrev->pNext = this;
		if (pxNext)
			pxNext->pPrev = this;
	}
	Triangle(u_int u0, u_int u1, u_int u2)
	{
		Asura_Assert(u0 != u1 && u1 != u2 && u0 != u2, "");
		uVerts[0] = u0;
		uVerts[1] = u1;
		uVerts[2] = u2;
		pNext = NULL;
		pPrev = NULL;
	}

	static bool Compare(Triangle *pOne, Triangle *pTwo)
	{
		if (pOne->uVerts[0] == pTwo->uVerts[0] && pOne->uVerts[1] == pTwo->uVerts[1] && pOne->uVerts[2] == pTwo->uVerts[2])
			return true;
		if (pOne->uVerts[0] == pTwo->uVerts[1] && pOne->uVerts[1] == pTwo->uVerts[2] && pOne->uVerts[2] == pTwo->uVerts[0])
			return true;
		if (pOne->uVerts[0] == pTwo->uVerts[2] && pOne->uVerts[1] == pTwo->uVerts[0] && pOne->uVerts[2] == pTwo->uVerts[1])
			return true;

		return false;
	}

};
#endif

void Tools_Wii_StripSet::ParseStrips(const unsigned short* pusIndicies, u_int uNumIndicies)
{
	if (uNumIndicies == 0)
		return;

	unsigned int uLastSeen[2] = {pusIndicies[0], pusIndicies[0]};
	u_int uVertIndex, uIndex, uStripStart;
#ifdef CHECK_WII_OUTPUT
	struct TriList {
		Triangle * pHead;
		Triangle * pTail;
		u_int uNumTris;

		void Add(Triangle *pTri)
		{
			pTri->Link(pTail, NULL);
			pTail = pTri;
			if (!pHead)
				pHead = pTri;
			uNumTris++;
		}

		TriList()
		{
			pHead = pTail = NULL;
			uNumTris = 0;
		}
		~TriList()
		{
			while (pHead)
			{
				Triangle *pTri = pHead->Next();
				delete pHead;
				pHead = pTri;
			}
		}
		static bool Compare(TriList &One, TriList &Two)
		{
			Triangle * pRunner[2] = {One.pHead, Two.pHead};

			while (pRunner[0] != NULL && pRunner[1] != NULL)
			{
				pRunner[0]->bMarker = true;
				pRunner[1]->bMarker = true;
				pRunner[0] = pRunner[0]->Next();
				pRunner[1] = pRunner[1]->Next();
			}

			pRunner[0] = One.pHead;
			pRunner[1] = Two.pHead;

			while (pRunner[0] != NULL && pRunner[1] != NULL)
			{
				if (pRunner[1]->bMarker && Triangle::Compare(pRunner[0], pRunner[1]))
				{
					pRunner[1]->bMarker = false;
				} 
				else if (pRunner[1]->pNext && pRunner[1]->pNext->bMarker && Triangle::Compare(pRunner[0], pRunner[1]->pNext))
				{
					pRunner[1]->pNext->bMarker = false;
				}
				else if (pRunner[1]->pPrev && pRunner[1]->pPrev->bMarker && Triangle::Compare(pRunner[0], pRunner[1]->pPrev))
				{
					pRunner[1]->pPrev->bMarker = false;
				} else {
					Triangle * pTemp = Two.pHead;
					for (;pTemp; pTemp = pTemp->Next())
					{
						if (pTemp->bMarker && Triangle::Compare(pRunner[0], pTemp))
						{
							pTemp->bMarker = false;
							break;
						}
					}
					if (!pTemp)
						return false;
				}
				pRunner[0] = pRunner[0]->Next();
				pRunner[1] = pRunner[1]->Next();
			}
			return pRunner[0] == pRunner[1];
		}
	} xList1, xList2;

	for(uIndex = 0; uIndex < uNumIndicies; uLastSeen[(uIndex++) & 1] = uVertIndex)
	{
		uVertIndex = (u_int)pusIndicies[uIndex];
		if (uVertIndex != uLastSeen[0] && uVertIndex != uLastSeen[1] && uLastSeen[0] != uLastSeen[1])
		{
			u_int uSelector = (uIndex + 1) & 1;
			xList1.Add(new Triangle(uLastSeen[(uIndex+uSelector+1) & 1], uLastSeen[(uIndex+uSelector) & 1], uVertIndex));
		}
	}
	uLastSeen[0] = pusIndicies[0];
	uLastSeen[1] = pusIndicies[0];
#endif

	for(uIndex = 0, uStripStart = 0; uIndex < uNumIndicies; ++uIndex)
	{
		uVertIndex = (u_int)pusIndicies[uIndex];

		if ( uVertIndex == uLastSeen[0] || uVertIndex == uLastSeen[1] || uLastSeen[0] == uLastSeen[1])
		{
			u_int uNumVerts = uIndex - uStripStart;
			if (uNumVerts > 2)
			{
#ifdef CHECK_WII_OUTPUT
				CheckForDegenerates(pusIndicies + uStripStart, uNumVerts);
#endif
				Wii_TriangleStrip *pStrip = Wii_TriangleStrip::Allocate(uNumVerts, pusIndicies + uStripStart, (uStripStart & 1) == 1);
				pStrip->LinkAfter(m_pTail);
				if (!m_pHead)
					m_pHead = pStrip;
				m_pTail = pStrip;

				m_uNumStrips++;
				m_uNumIndicies += pStrip->m_uNumIndicies;
				m_uNumTris += pStrip->m_uNumIndicies - 2;
			} 

			if (uLastSeen[0] == uVertIndex)
			{
				uStripStart = uIndex; 
			} else if (uLastSeen[1] == uVertIndex) {
				uStripStart = uIndex - 1;
			} else if ( uStripStart > 0) {
				uStripStart = uIndex - 1;
			} else {
				uStripStart = 0;
			}
		}
		uLastSeen[1] = uLastSeen[0];
		uLastSeen[0] = uVertIndex;
	}

	u_int uNumVerts = uIndex - uStripStart;
	if (uNumVerts > 2)
	{
#ifdef CHECK_WII_OUTPUT
		CheckForDegenerates(pusIndicies + uStripStart, uNumVerts);
#endif
		Wii_TriangleStrip *pStrip = Wii_TriangleStrip::Allocate(uNumVerts, pusIndicies + uStripStart, (uStripStart & 1) == 1);
		pStrip->LinkAfter(m_pTail);
		if (!m_pHead)
			m_pHead = pStrip;
		m_pTail = pStrip;

		m_uNumStrips++;
		m_uNumIndicies += pStrip->m_uNumIndicies;
		m_uNumTris += pStrip->m_uNumIndicies - 2;
	} 
#ifdef CHECK_WII_OUTPUT
	for(Wii_TriangleStrip *pxStrip = m_pHead; pxStrip; pxStrip = pxStrip->m_pNext)
	{
		u_short *pusIndicies = pxStrip->TriStrip();
		uLastSeen[0] = pusIndicies[0];
		uLastSeen[1] = pusIndicies[0];

		for(uIndex = 0; uIndex < pxStrip->m_uNumIndicies; uLastSeen[(uIndex++) & 1] = uVertIndex)
		{
			uVertIndex = (u_int)pusIndicies[uIndex];
			if (uVertIndex != uLastSeen[0] && uVertIndex != uLastSeen[1] && uLastSeen[0] != uLastSeen[1])
			{
				u_int uSelector = (uIndex + 1) & 1;
				xList2.Add(new Triangle(uLastSeen[(uIndex+uSelector+1) & 1], uLastSeen[(uIndex+uSelector) & 1], uVertIndex));
			}
		}
	}
	if (!TriList::Compare(xList1, xList2))
	{
		static char Logfile[] = "logfiles\\Wii_Fuckup.txt";

		Asura_SimpleLogfileClear(Logfile);

		Asura_SimpleLogfileOutput((Logfile, "--------------------------------------------------------------------------------\n\n" ));
		Asura_SimpleLogfileOutput((Logfile, "Index Dump\n" ));
		for (u_int i = 0; i < uNumIndicies; ++i)
			Asura_SimpleLogfileOutput((Logfile, "%d ", pusIndicies[i]));
		Asura_SimpleLogfileOutput((Logfile, "--------------------------------------------------------------------------------\n\n" ));
		Asura_SimpleLogfileOutput((Logfile, "Tri Dump \n"));
		Asura_SimpleLogfileOutput((Logfile, "%d <> %d\n", xList1.uNumTris, xList2.uNumTris));

		Triangle * pRunner[2] = {xList1.pHead, xList2.pHead};
		while (pRunner[0] != NULL && pRunner[1] != NULL)
		{
			Asura_SimpleLogfileOutput((Logfile, "(%d, %d, %d) %s (%d, %d, %d)\n", pRunner[0]->uVerts[0], pRunner[0]->uVerts[1], pRunner[0]->uVerts[2], Triangle::Compare(pRunner[0], pRunner[1]) ? "==" : "!=", pRunner[1]->uVerts[0], pRunner[1]->uVerts[1], pRunner[1]->uVerts[2] ));

			pRunner[0] = pRunner[0]->Next();
			pRunner[1] = pRunner[1]->Next();
		}
		Asura_SimpleLogfileOutput((Logfile, "--------------------------------------------------------------------------------\n\n" ));
		Asura_SimpleLogfileOutput((Logfile, "Optimised Strips \n"));
		u_int uStrip = 0;
		for(Wii_TriangleStrip *pxStrip = m_pHead; pxStrip; pxStrip = pxStrip->m_pNext)
		{
			u_short *pusIndicies = pxStrip->TriStrip();
			Asura_SimpleLogfileOutput((Logfile, "Strip[%d]\n", uStrip++));
			for (u_int uIndex = 0; uIndex < pxStrip->m_uNumIndicies;++uIndex)
			{
				Asura_SimpleLogfileOutput((Logfile, "%d ", pusIndicies[uIndex]));
			}
			Asura_SimpleLogfileOutput((Logfile, "\n", uStrip++));
		}
		Asura_Assert(0, "");
	}
#endif

}

void Tools_Wii_DisplayListFactory::BeginList()
{
	Asura_Assert(m_pxStripSet == NULL, "");
	m_pxStripSet = new Tools_Wii_StripSet();
}

void Tools_Wii_DisplayListFactory::AddStrip(u_short *pusIndicies, u_int uNumIndicies)
{
	Asura_Assert(m_pxStripSet != NULL, "");
	m_pxStripSet->ParseStrips(pusIndicies, uNumIndicies);
}

u_int Tools_Wii_DisplayListFactory::GenerateList(void *&pxDisplayList, u_int &uDisplayListSize)
{
	Asura_Assert(m_pxStripSet != NULL, "");
	// Per Strip
	u_int uSize = 3  * m_pxStripSet->m_uNumStrips + SizePerVert() * m_pxStripSet->m_uNumIndicies;
	// Align to 32 bytes
	// this is a copy of what the wii SDK does 
	// if it is already a multiple of 32 bytes then this will add an extra 32 bytes padding
	uSize = uSize + 32 - (uSize & 31);
	uDisplayListSize = uSize;

	void *pDisplayListMem = pxDisplayList = new char[uDisplayListSize];

	for(Wii_TriangleStrip *pxStrip = m_pxStripSet->m_pHead; pxStrip; pxStrip = pxStrip->m_pNext)
	{
		Store<u_char>(pDisplayListMem, 0x98 | StripType());
		Store<u_short>(pDisplayListMem, static_cast<u_short>(pxStrip->m_uNumIndicies));
		for (u_int uIndex = 0; uIndex < pxStrip->m_uNumIndicies;++uIndex)
		{
			StoreVert(pDisplayListMem, pxStrip->GetIndex(uIndex));
		}
	}
	if ((reinterpret_cast<u_int>(pDisplayListMem) - reinterpret_cast<u_int>(pxDisplayList)) & 31)
	{
		while ((reinterpret_cast<u_int>(pDisplayListMem) - reinterpret_cast<u_int>(pxDisplayList)) & 31)
			Store<u_char>(pDisplayListMem, 0); //GX_NOP;
	} else {
		for (int i = 0; i < 32; ++i)
			Store<u_char>(pDisplayListMem, 0); //GX_NOP;
	}

	Asura_Assert((reinterpret_cast<u_int>(pDisplayListMem) - reinterpret_cast<u_int>(pxDisplayList)) == uDisplayListSize, "Error calculating display list size");

	return m_pxStripSet->m_uNumTris;

}

u_int Tools_Wii_DisplayListFactory::EndList(void *&pxDisplayList, u_int &uDisplayListSize)
{
	Asura_Assert(m_pxStripSet != NULL, "");
	u_int uNumTris = GenerateList(pxDisplayList, uDisplayListSize);
	delete m_pxStripSet;
	m_pxStripSet = NULL;
	return uNumTris;
}


u_int Tools_Wii_DisplayListFactory::CreateDisplayList(void *&pxDisplayList, u_int &uDisplayListSize, u_short *pusIndicies, u_int uNumIndicies)
{
	Asura_Assert(m_pxStripSet == NULL, "");
	Tools_Wii_StripSet xStripSet;
	m_pxStripSet = &xStripSet;

	AddStrip(pusIndicies, uNumIndicies);

	u_int uNumTris = GenerateList(pxDisplayList, uDisplayListSize);

	m_pxStripSet = NULL;

	return uNumTris;
}

