///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Xbox360_Export.cpp
// Author : Kevin Floyer-Lea
// Date   : 26-Sep-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

#include "Core/Asura.h"

#include "Xbox360/Tools_Xbox360_Export.h"

#define ASURA_EXPORT_X360_EDGEAADATA 1

// Asura Platform-independent
#include "Animation/Asura_Hierarchy.h"
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render.h"
#include "Botanical/Asura_Direct3D_Botanicals_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanical_Game_Render_Manager.h"
#include "Botanicals/Asura_Botanical_Game_Render_Type.h"
#include "Botanicals/Asura_Botanical_Game_Render_Type_BasicInteractive.h"
#include "Botanicals/Asura_Botanicals_InstanceManager.h"
#include "Botanicals/Asura_Botanicals_TemplateManager.h"
#include "ChunkSystem/Asura_Chunk_ResourceFile.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Memory/Asura_Mem.h"
#include "Core/Asura_FixedArray.h"
#include "Core/Asura_HalfFloat.h"
#include "Core/Asura_HashTree.h"
#include "Core/Asura_LinkList.h"
#include "Core/Asura_StringBuffer.h"
#include "Core/Asura_TextureManagement.h"
#include "FileAccess/Asura_FileAccess.h"
#include "Mesh/Asura_Mesh.h"
#include "Rendering/Asura_Colour.h"
#include "Rendering/Asura_Module.h"

// Asura PC
#include "ChunkSystem/Asura_Chunk_Stream_WindowsFile.h"
#include "Rendering/Asura_Direct3D.h"

// Asura Xbox 360
#include "Xbox360/ChunkSystem/Asura_Chunk_ResourceFile_Xbox360.h"
#include "Xbox360/ChunkSystem/Asura_Chunk_ResourceFile_Xbox360_Botanicals.h"
#include "Xbox360/ChunkSystem/Asura_Chunk_ResourceFile_Xbox360_Environment.h"

// Tools
#include "BotanicalCreation/Tools_BotanicalCreation_BotanicalShape.h"
#include "BotanicalCreation/Tools_BotanicalCreation_Rendering.h"
#include "ImageProcessing/Tools_Xbox360_Texture_Writer.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Materials/Tools_MaterialResponse_System.h"
#include "MeshProcessing/Tools_EnvironmentProcessor.h"
#include "MeshProcessing/Tools_HierarchyProcessor_MultiMaterial_NormalMapped.h"
#include "MeshProcessing/Tools_MeshRestartIndex.h"
#include "MeshProcessing/Tools_ObjectProcessor_NormalMapped.h"
#include "MeshProcessing/Tools_SmoothSkin_Processor.h"
#include "MeshProcessing/Tools_SmoothSkin_Processor_Xbox360.h"
#include "MeshProcessing/Tools_Tangentspace.h"
#ifdef ASURA_EXPORT_X360_EDGEAADATA
#include "MeshProcessing/Tools_AAEdges.h"
#endif
#include "PC/FileSystem/Tools_FileSystem.h"
#include "PC/FileSystem/Tools_File_Wrapper.h"
#include "PC/OS/Tools_OS.h"

#ifdef ASURA_USE_STATIC_DECAL_SYSTEM
#include "SpecialFX/Asura_Decal_Template.h"
#include "Decals/Tools_StaticDecal_System.h"
#endif

/******************************************************************************/
/* Macros                                                                     */
/******************************************************************************/

/******************************************************************************/
/* Classes		                                                              */
/******************************************************************************/

#if defined( ASURA_USE_BOTANICALS)
class Tools_Xbox360_Export_Botanical_Template
{
public:
	const Asura_Botanicals_Template* m_pxTemplate;
	
	int m_iModuleIndex;
	int m_iShapeIndex;
	
	Asura_LinkList< const Asura_Botanicals_Instance*> m_xListInstances;
};
#endif

/******************************************************************************/
/* Constants                                                                  */
/******************************************************************************/

// Should use Asura_String when writing chunks streams, because the stream
// takes care of the 4-byte padding for us

const Asura_String xENV_RESOURCE_FILENAME( "Env" );
const Asura_String xBOTANICALS_RESOURCE_FILENAME( "Botanicals" );
const Asura_String xSTATICDECALS_RESOURCE_FILENAME( "StaticDecals" );

/******************************************************************************/
/* Tools_Xbox360_Export Static Class Data                                     */
/******************************************************************************/

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	Asura_HashTree Tools_Xbox360_Export::s_xShaderTree;
	Tools_Xbox360_RDF Tools_Xbox360_Export::s_xShaderRDF;
#endif

/******************************************************************************/
/* Tools_Xbox360_Export Static Class Functions                                */
/******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Xbox360_Export::EnvironmentVertex::WriteToChunkStream
// Author       : Kevin Floyer-Lea
// Date         : 22-Sep-2005
//
// Description  : Writes this vertex to the specified chunk-stream.
//
// Parameters   : A reference to the chunk-stream to write the data to.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::EnvironmentVertex::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << m_xPosition;

	xStream << m_uDiffuse;
	
	xStream << m_xNormal;
	xStream << m_xTangent;
	xStream << m_xBinormal;
	xStream << m_xTangent2;
	xStream << m_xBinormal2;

	xStream << m_xU;
	xStream << m_xV;
	xStream << m_xU2;
	xStream << m_xV2;

	xStream << Asura_WriteToStream_Array_Wrapper< Asura_HalfFloat >( 4, m_axSHRedCoefficients );
	xStream << Asura_WriteToStream_Array_Wrapper< Asura_HalfFloat >( 4, m_axSHGreenCoefficients );
	xStream << Asura_WriteToStream_Array_Wrapper< Asura_HalfFloat >( 4, m_axSHBlueCoefficients );
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Xbox360_Export::ObjectVertex::WriteToChunkStream
// Author       : Kevin Floyer-Lea
// Date         : 22-Sep-2005
//
// Description  : Writes this vertex to the specified chunk-stream.
//
// Parameters   : A reference to the chunk-stream to write the data to.
//
// Return value : None.
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::ObjectVertex::WriteToChunkStream( Asura_Chunk_Stream& xStream ) const
{
	xStream << m_xPosition;

	xStream << m_xU;
	xStream << m_xV;
	xStream << m_xU2;
	xStream << m_xV2;
	xStream << m_fAlpha;

	xStream << m_xNormal;
	xStream << m_xTangent;
	xStream << m_xBinormal;
}

/******************************************************************************/
/* Tools_Xbox360_Export Static Class Functions                                */
/******************************************************************************/

// (AAM - 12.06.2006) Uncomment this to merge strips with the same material across modules,
// leaving just one strip for each material in the environment. Eventually this might move
// up into independent code if it's a win on other platforms. Downside is dynamic lights
// have an even bigger hit since each strip's BB becomes potentially much larger.

//#define MERGE_STRIPS

#if defined MERGE_STRIPS

static bool CopyStrip( Tools_EnvironmentProcessor_Strip& xDest, const Tools_EnvironmentProcessor_Strip& xSource )
{
	delete[] xDest.m_pusIndices;
	delete[] xDest.m_pxVertices;

	xDest = xSource;

	// Copy indices

	xDest.m_pusIndices = new u_short[ xSource.m_uNumberOfIndices ];
	if( !xDest.m_pusIndices ) return false;

	memcpy( xDest.m_pusIndices, xSource.m_pusIndices, sizeof( u_short ) * xSource.m_uNumberOfIndices );

	// Copy vertices

	xDest.m_pxVertices = new Tools_EnvironmentProcessor_Vertex[ xSource.m_uNumberOfVertices ];
	if( !xDest.m_pxVertices ) return false;

	memcpy( xDest.m_pxVertices, xSource.m_pxVertices, sizeof( Tools_EnvironmentProcessor_Vertex ) * xSource.m_uNumberOfVertices );

	return true;
}

static bool AppendStrip( Tools_EnvironmentProcessor_Strip& xDest, const Tools_EnvironmentProcessor_Strip& xSource )
{
	// Allocate memory for the merged data

	const u_int uTotalNumIndices = xDest.m_uNumberOfIndices + xSource.m_uNumberOfIndices + 1;

	u_short* pusNewIndices = new u_short[ uTotalNumIndices ];
	if( !pusNewIndices ) return false;

	Tools_EnvironmentProcessor_Vertex* pxNewVertices = new Tools_EnvironmentProcessor_Vertex[ xDest.m_uNumberOfVertices + xSource.m_uNumberOfVertices ];
	if( !pxNewVertices )
	{
		delete[] pusNewIndices;
		return false;
	}

	u_short usRestartValue;
	Tools_MeshRestartIndex::IsValidForCurrentPlatform( usRestartValue );

	// Append the new set of indices

	memcpy( pusNewIndices, xDest.m_pusIndices, sizeof( u_short ) * xDest.m_uNumberOfIndices );
	pusNewIndices[ xDest.m_uNumberOfIndices ] = usRestartValue;
	memcpy( pusNewIndices + xDest.m_uNumberOfIndices + 1, xSource.m_pusIndices, sizeof( u_short ) * xSource.m_uNumberOfIndices );

	// Patch up the index values in the second set

	for( u_int u = xDest.m_uNumberOfIndices + 1; u < uTotalNumIndices; ++u )
	{
		if( pusNewIndices[u] != usRestartValue )
		{
			pusNewIndices[u] = static_cast< u_short >( pusNewIndices[u] + xDest.m_uNumberOfVertices );
		}
	}

	delete[] xDest.m_pusIndices;
	xDest.m_pusIndices = pusNewIndices;

	// Append the new set of vertices

	memcpy( pxNewVertices, xDest.m_pxVertices, sizeof( Tools_EnvironmentProcessor_Vertex ) * xDest.m_uNumberOfVertices );
	memcpy( pxNewVertices + xDest.m_uNumberOfVertices, xSource.m_pxVertices, sizeof( Tools_EnvironmentProcessor_Vertex ) * xSource.m_uNumberOfVertices );

	delete[] xDest.m_pxVertices;
	xDest.m_pxVertices = pxNewVertices;

	// Increment counts

	xDest.m_uNumberOfIndices += xSource.m_uNumberOfIndices + 1;
	xDest.m_uNumberOfTriangles += xSource.m_uNumberOfTriangles;
	xDest.m_uNumberOfVertices += xSource.m_uNumberOfVertices;

	return true;
}

static Tools_EnvironmentProcessor_Strip* MergeStrips( const Tools_EnvironmentProcessor_Strip* const pxOriginalStrips, const u_int uNumOriginalStrips, u_int& uNumNewStrips )
{
	// Merge all strips with the same material

	Tools_EnvironmentProcessor_Strip* pxNewStrips = new Tools_EnvironmentProcessor_Strip[ uNumOriginalStrips ];
	if( !pxNewStrips ) return NULL;

	if( !CopyStrip( pxNewStrips[0], pxOriginalStrips[0] ) ) return NULL;

	u_int uStripIndex = 0;
	u_int uOriginalStripIndex = 1;

	while( uOriginalStripIndex < uNumOriginalStrips )
	{
		const Tools_EnvironmentProcessor_Strip& xPreviousOriginalStrip = pxOriginalStrips[ uOriginalStripIndex - 1 ];
		const Tools_EnvironmentProcessor_Strip& xThisOriginalStrip = pxOriginalStrips[ uOriginalStripIndex ];

		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
			const bool bSameMaterial = (xPreviousOriginalStrip.m_uMaterialResponseHashID == xThisOriginalStrip.m_uMaterialResponseHashID);
		#else
			const int iPreviousMaterialID = xPreviousOriginalStrip.m_iUnalteredMaterialID;
			const int iThisMaterialID = xThisOriginalStrip.m_iUnalteredMaterialID;
			const bool bSameMaterial = (Asura_Material_System::CompareMaterials( iPreviousMaterialID, iThisMaterialID ) == 0);
		#endif

		if( bSameMaterial
		 && pxNewStrips[ uStripIndex ].m_uNumberOfVertices + xThisOriginalStrip.m_uNumberOfVertices < 65536 )
		{
			// Append this to the previous strip

			if( !AppendStrip( pxNewStrips[ uStripIndex ], xThisOriginalStrip ) ) return NULL;
		}
		else
		{
			// Start off a new entry

			++uStripIndex;

			if( !CopyStrip( pxNewStrips[ uStripIndex ], xThisOriginalStrip ) ) return NULL;
		}

		++uOriginalStripIndex;
	}

	uNumNewStrips = uStripIndex + 1;

	return pxNewStrips;
}

#endif // defined MERGE_STRIPS

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Xbox360_Export::ExportEnvironment
// Author       : Kevin Floyer-Lea
// Date         : 26-Sep-2005
//
// Description  : 
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::ExportEnvironment( const HANDLE hOutput )
{
	// go grab the environment
	Tools_EnvironmentProcessor::Initialise( true );

	// but quit if the user pressed abort during the environment stripify etc.
	if (!Tools_Level_Export::IsAbortFlagSet())
	{
		// get the environment in strips, sorted by material / shader etc

		const Tools_EnvironmentProcessor_Strip* const pxOriginalStrips = Tools_EnvironmentProcessor::GetStripsSortedByMaterial();
		const u_int uNumOriginalStrips = Tools_EnvironmentProcessor::GetTotalNumberOfStrips();
		u_int uNumberOfStrips;

#if defined MERGE_STRIPS

		const Tools_EnvironmentProcessor_Strip* const pxStrips = MergeStrips( pxOriginalStrips, uNumOriginalStrips, uNumberOfStrips );

		if( !pxStrips )
		{
			Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Unable to merge environment strips." );
			Tools_EnvironmentProcessor::Shutdown();
			Tools_Level_Export::SetAbortFlag();

			return;
		}

#else
		const Tools_EnvironmentProcessor_Strip* const pxStrips = pxOriginalStrips;
		uNumberOfStrips = uNumOriginalStrips;
#endif

#ifdef ASURA_EXPORT_X360_EDGEAADATA
		//////////////////////////////////////////////////////////////////////////
		// Find and map edges used for anti-aliasing
		Tools_AAEdges xEdges;
		//
		// Find AABB for all strips
		Asura_Bounding_Box xBB;
		xBB.Reset();
		// scan through all strips vertices and find bounding box
		for ( u_int s=0; s<uNumberOfStrips; ++s )
		{
			const Tools_EnvironmentProcessor_Strip& xStrip = pxStrips[s];
			for ( u_int v = 0; v < xStrip.m_uNumberOfVertices; ++v )
			{
				xBB.Set( xStrip.m_pxVertices[v].m_xPosition );
			}
		}
		//
		// Find edges
		xEdges.Create( xBB, uNumberOfStrips );
		//
		// associate AA edges with specific strips
		xEdges.BeginStripsMapping();
		for ( int s=uNumberOfStrips-1; s>=0; --s )
		{
			Tools_Level_Export::UpdateStatus( Tools_Level_Export::EXPORT_STAGE_COMPILING_ENVIRONMENT_SHADERS, "Mapping Anti-Alias edges...", 100 - ((s * 100) / uNumberOfStrips) );
			xEdges.MapStrip( pxStrips[s], s );
		}
		xEdges.EndStripsMapping();
		//////////////////////////////////////////////////////////////////////////
#endif

		Asura_Chunk_Stream_WindowsFile xStream(hOutput);

		xStream.WriteResourceChunk_Start( ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
			ASURA_RESOURCEFILE_TYPE_XBOX360_ENVIRONMENT, iASURA_CHUNK_XBOX360_ENVIRONMENT_VERSION,
			0, xENV_RESOURCE_FILENAME );

		// Write the data...
		xStream << uNumberOfStrips;

		u_int uBaseIndex = 0;
		u_int uIBOffset = 0;

#ifdef ASURA_EXPORT_X360_EDGEAADATA
		u_int uAAEdgeOffset = 0;
#endif
		// KJFL - 2004-02-05 17:52:40 - all strip descriptions in one go
		// Order and size perfectly matches "Asura_Xbox360_EnvironmentRenderer_Strip",
		// so all strips can be loaded in one go. 
		for ( u_int s=0; s<uNumberOfStrips; ++s )
		{
			const Tools_EnvironmentProcessor_Strip& xStrip = pxStrips[s];

			// scan through strip vertices and find bounding box
			Asura_Bounding_Box xStripBB;

			xStripBB.Reset();
			for ( u_int v = 0; v < xStrip.m_uNumberOfVertices; ++v )
			{
				xStripBB.Set( xStrip.m_pxVertices[v].m_xPosition );
			}

			xStream << xStrip.m_uNumberOfVertices;
			xStream << xStrip.m_uNumberOfIndices;

			// We just use one VB/IB pair, changing parameters to DrawIndexedVertices()
			xStream << uBaseIndex;
			xStream << uIBOffset;

			uBaseIndex += xStrip.m_uNumberOfVertices;
			uIBOffset += xStrip.m_uNumberOfIndices;

			#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
				const Asura_Hash_ID uMaterial1 = xStrip.m_auMaterialResponseHashID[0];
				const Asura_Hash_ID uMaterial2 = xStrip.m_auMaterialResponseHashID[1];

				xStream << uMaterial1;
				xStream << uMaterial2;
			#else
				xStream << xStrip.m_iMaterialID;
			#endif

			xStream << xStrip.m_uModule;
			xStream << xStrip.m_uNumberOfTriangles;

#ifdef ASURA_EXPORT_X360_EDGEAADATA
			// Export AA edges info
			const u_int	uNumStripEdges = xEdges.GetNumberOfEdgesForStrip( s );
			xStream << uAAEdgeOffset*2;
			xStream << uNumStripEdges*2;
			uAAEdgeOffset += uNumStripEdges;
#else
			xStream << (u_int)0;
			xStream << (u_int)0;
#endif

			xStream << xStripBB;

			#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
				// Ensure we export the pixel shaders required for this strip
			Tools_Level_Export::AddMRPShaders( uMaterial1, uMaterial2, false );
			#endif
		}

#ifdef ASURA_EXPORT_X360_EDGEAADATA
		Asura_Assertf( (int)uAAEdgeOffset == xEdges.GetTotalNumberOfEdgeIndices()/2, ( "Edge count mismatched, this is impossible & might crash the game! count %d, num %d", uAAEdgeOffset, xEdges.GetTotalNumberOfEdgeIndices()/2 ) );
#endif

		const u_int uNumberOfVertices = Tools_EnvironmentProcessor::GetTotalNumberOfVertices();
		xStream << uNumberOfVertices;

		// Xbox360 has MaxVertexIndex of 0xFFFFFF
		Asura_Assertf( uNumberOfVertices-1 <= 0xFFFFFF, ("Too many vertices for 360 (%u).",
			uNumberOfVertices) );

		// KJFL - 2004-02-05 17:52:08 -  all the vertex data in one go
		for ( u_int uStrip=0; uStrip<uNumberOfStrips; ++uStrip )
		{
			const Tools_EnvironmentProcessor_Strip& xStrip = pxStrips[uStrip];
			
#if defined( ASURA_ASSERT)
			const u_int uModuleIndex = xStrip.m_uModule;
			const u_int uModuleCount = Asura_Module_System::GetNumberOfModules();
			const Asura_Mesh* pxModuleMesh = uModuleIndex < uModuleCount ? (Asura_Module_System::GetModuleData( uModuleIndex)->Mesh) : NULL;
			const char* pxModuleName = (NULL != pxModuleMesh) ? (pxModuleMesh->Name.GetString()) : "Unknown";
#endif

			for (u_int v = 0; v < xStrip.m_uNumberOfVertices; ++v )
			{
				const Tools_EnvironmentProcessor_Vertex&	xVertex	= xStrip.m_pxVertices [v];
				EnvironmentVertex							xOutputVertex;

				// position
				xOutputVertex.m_xPosition = xVertex.m_xPosition;

				// diffuse
				xOutputVertex.m_uDiffuse = xVertex.m_uDiffuseRGBA;

				// uvs
				xOutputVertex.m_xU	= xVertex.m_fU;
				xOutputVertex.m_xV	= xVertex.m_fV;

				xOutputVertex.m_xU2	= xVertex.m_fU2;
				xOutputVertex.m_xV2	= xVertex.m_fV2;

				// normal/tangent/binormal
				Asura_Assertf( 1e-3f < xVertex.m_xNormal.MagnitudeSquared(), ("Normal Length too small on mesh '%s'", pxModuleName));
				xOutputVertex.m_xNormal = xVertex.m_xNormal;

#if defined ASURA_TARGET_SM3

				Asura_Assertf( 1e-3f < xVertex.m_xTangent.MagnitudeSquared(), ("Tangent Length too small on mesh '%s'", pxModuleName));
				Asura_Assertf( 1e-3f < xVertex.m_xBinormal.MagnitudeSquared(), ("Binormal Length too small on mesh '%s'", pxModuleName));
				Asura_Assertf( 1e-3f < xVertex.m_xTangent2.MagnitudeSquared(), ("Tangent Length too small on mesh '%s'", pxModuleName));
				Asura_Assertf( 1e-3f < xVertex.m_xBinormal2.MagnitudeSquared(), ("Binormal Length too small on mesh '%s'", pxModuleName));

				xOutputVertex.m_xTangent = xVertex.m_xTangent;
				xOutputVertex.m_xBinormal = xVertex.m_xBinormal;
				xOutputVertex.m_xTangent2 = xVertex.m_xTangent2;
				xOutputVertex.m_xBinormal2 = xVertex.m_xBinormal2;

				// Need to swizzle the spherical harmonics

				// SH Red
				xOutputVertex.m_axSHRedCoefficients[0] = xVertex.m_xDiffuseSH.m_afRedCoefficients[1];
				xOutputVertex.m_axSHRedCoefficients[1] = xVertex.m_xDiffuseSH.m_afRedCoefficients[2];
				xOutputVertex.m_axSHRedCoefficients[2] = xVertex.m_xDiffuseSH.m_afRedCoefficients[3];
				xOutputVertex.m_axSHRedCoefficients[3] = xVertex.m_xDiffuseSH.m_afRedCoefficients[0];
			
				// SH Green
				xOutputVertex.m_axSHGreenCoefficients[0] = xVertex.m_xDiffuseSH.m_afGreenCoefficients[1];
				xOutputVertex.m_axSHGreenCoefficients[1] = xVertex.m_xDiffuseSH.m_afGreenCoefficients[2];
				xOutputVertex.m_axSHGreenCoefficients[2] = xVertex.m_xDiffuseSH.m_afGreenCoefficients[3];
				xOutputVertex.m_axSHGreenCoefficients[3] = xVertex.m_xDiffuseSH.m_afGreenCoefficients[0];
			
				// SH Blue
				xOutputVertex.m_axSHBlueCoefficients[0] = xVertex.m_xDiffuseSH.m_afBlueCoefficients[1];
				xOutputVertex.m_axSHBlueCoefficients[1] = xVertex.m_xDiffuseSH.m_afBlueCoefficients[2];
				xOutputVertex.m_axSHBlueCoefficients[2] = xVertex.m_xDiffuseSH.m_afBlueCoefficients[3];
				xOutputVertex.m_axSHBlueCoefficients[3] = xVertex.m_xDiffuseSH.m_afBlueCoefficients[0];

#endif	// defined ASURA_TARGET_SM3

				// Write this vertex...
				xOutputVertex.WriteToChunkStream( xStream );
			}
		}

		const u_int uNumberOfIndices = Tools_EnvironmentProcessor::GetTotalNumberOfIndices();
		xStream << uNumberOfIndices;

		// KJFL - 2004-02-05 17:52:08 -  all the index data in one go
		for ( u_int uStrip=0; uStrip<uNumberOfStrips; ++uStrip )
		{
			const Tools_EnvironmentProcessor_Strip& xStrip = pxStrips[uStrip];

			for (u_int uIndex = 0; uIndex < xStrip.m_uNumberOfIndices; ++uIndex)
			{
				xStream << xStrip.m_pusIndices [uIndex];
			}
		}

#ifdef ASURA_EXPORT_X360_EDGEAADATA
		// Output AA edge indices
		{
			xStream << xEdges.GetTotalNumberOfEdgeIndices();

			int iEdgeCount = 0;

			Tools_LinkedAntiAliasEdge* pxEdge = xEdges.GetSortedEdges();
			while ( pxEdge )
			{
				xStream << pxEdge->m_auIdx[0];
				xStream << pxEdge->m_auIdx[1];

				pxEdge = pxEdge->m_pxNext;
				iEdgeCount++;
			}
			
			Asura_Assertf( iEdgeCount == xEdges.GetTotalNumberOfEdgeIndices()/2, ( "Sanity check failed, environment anti-alias data is corrupted. (wrote %d edges, expected %d)", iEdgeCount, xEdges.GetTotalNumberOfEdgeIndices()/2 ) );
		}
#else
		xStream << (int)0;
#endif

#if defined MERGE_STRIPS
		delete[] pxStrips;
#endif
		xStream.WriteResourceChunk_End();

		Tools_EnvironmentProcessor::Shutdown();
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportTexture()
//
// Exports a texture to Xbox 360 format
//
// Parameters:		szInputName - Filename of texture to convert
//					szOutputName - Output filename
//					iTextureIndex - Asura texture id for this texture
//					xConvertData - Conversion data
//
// Return value:	true iff successful
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_Xbox360_Export::ExportTexture( const char* szInputName, const char* szOutputName, const int iTextureIndex, const TextureConversionData& xConvertData )
{
	// Eventually, we might want to push everything through the bundler. For now though,
	// check the usage and only do this for normal or alpha-only maps. That's because
	// the .xpr files come out slightly larger than the equivalent output from DDS utils

	if( Tools_Xbox360_Texture_Writer::ShouldExportAsXPR( iTextureIndex ) )
	{
		return ExportTextureAsXPR( szInputName, szOutputName, iTextureIndex, xConvertData );
	}
	else
	{
		return Tools_Level_Export::CreateDDSFile( szInputName, szOutputName, iTextureIndex, xConvertData);
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportTextureAsXPR()
//
// Exports a single texture into a .xpr bundle
//
// Parameters:		szInputName - Input file name of texture
//					szOutputName - Path to output file
//					iTextureIndex - Texture ID for input texture
//					xConvertData - Texture conversion info
//
// Return value:	true iff successful
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_Xbox360_Export::ExportTextureAsXPR( const char* szInputName, const char* szOutputName, const int iTextureIndex, const TextureConversionData& xConvertData )
{
	const char* szInternalTextureName =
		Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( szInputName );

	// Strip off the folder name, leaving the slash in place

	if( Asura_StringUtil::StartsWith( szInternalTextureName, "Graphics\\", true ) )
	{
		szInternalTextureName += strlen( "Graphics" );
	}
	else if( Asura_StringUtil::StartsWith( szInternalTextureName, "Graphics_Xbox360\\", true ) )
	{
		szInternalTextureName += strlen( "Graphics_Xbox360" );
	}
	else
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR,
			"Texture '%s' isn't in the Graphics or Graphics_Xbox360 folders.",
			szInternalTextureName );

		return false;
	}

	Tools_Xbox360_RDF xRDF;

	if( !Tools_Xbox360_Texture_Writer::BeginTextureExport( xRDF, szOutputName ) )
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR,
			"Unable to begin exporting texture '%s'.", szInternalTextureName );

		return false;
	}

	const bool bAdded = Tools_Xbox360_Texture_Writer::AddTextureToRDF( xRDF, szInternalTextureName,
		szInputName, iTextureIndex, xConvertData );

	Tools_Xbox360_Texture_Writer::EndTextureExport( xRDF );

	if( !bAdded )
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR,
			"Unable to add texture '%s' to .rdf.", szInternalTextureName );

		return false;
	}

	if( !Tools_Xbox360_Texture_Writer::RunBundler( xRDF ) )
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Unable to run Bundler." );
		return false;
	}

	Tools_Xbox360_Texture_Writer::DeleteRDF( xRDF );

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportCharacter()
//
// Exports a character smooth skin to Xbox 360 format
//
// Parameters:		hOutput - Handle to output file
//					pxSkin - Pointer to skin to export
//
// Return value:	true iff successful
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_Xbox360_Export::ExportCharacter( const HANDLE hOutput, Asura_Hierarchy_Skin* pxSkin )
{
	Tools_SmoothSkin_Processor xProcessor;
	xProcessor.SetIncludeNormalMapData( true );
	if( !xProcessor.Initialise( pxSkin ) ) return false;

	xProcessor.CreateBoneRemapTables(ASURA_MAX_NO_OF_BONES_PER_MATERIAL);

	Tools_SmoothSkin_Processor_Xbox360 xXbox360Processor;
	if( !xXbox360Processor.Initialise( xProcessor ) ) return false;

	if( !xXbox360Processor.OutputToChunk( hOutput ) ) return false;

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportSkin()
//
// Exports a skinned mesh to Xbox 360 format
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_Xbox360_Export::ExportSkin( const HANDLE hOutput, Asura_Hierarchy_Skin& xSkin )
{
	Tools_SmoothSkin_Processor_Xbox360 xXbox360Processor;

	if( xSkin.IsSkinnedModel() )
	{
		Tools_SmoothSkin_Processor xProcessor;
		xProcessor.SetIncludeNormalMapData( true );

		if( !xProcessor.Initialise( &xSkin ) ) return false;
		if( !xXbox360Processor.Initialise( xProcessor ) ) return false;
	}
	else
	{
		if( !Tools_HierarchyProcessor_MultiMaterial_NormalMapped::Initialise( &xSkin ) )
		{
			return false;
		}

		if( !xXbox360Processor.InitialiseFromHierarchyProcessor() ) return false;

		Tools_HierarchyProcessor_MultiMaterial_NormalMapped::Shutdown();
	}

	Asura_Chunk_Stream_WindowsFile xStream( hOutput );
	xXbox360Processor.OutputToChunk( xStream );

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Xbox360_Export::ExportObject
// Author       : Kevin Floyer-Lea
// Date         : 26-Sep-2005
//
// Description  : 
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////

bool Tools_Xbox360_Export::ExportObject (const HANDLE hOutput,
										 Asura_Mesh* pxMesh,
										 const Asura_Hash_ID uHashID,
										 const int iMaterialIndex,
										 const Asura_Hash_ID uMaterialResponseHashID /*= ASURA_HASH_ID_UNSET*/
										 )
{
	// initialise and stripify mesh
	if (Tools_ObjectProcessor_NormalMapped::Initialise(pxMesh, iMaterialIndex, true, Tools_TriStripper::OUTPUT_TRISTRIP_WITH_RESET_INDICES, uMaterialResponseHashID))
	{
		const Asura_String&	xName				= pxMesh->Name;
		const u_int			uFileNameSize		= (xName.GetSizeInInts() << 2);
		const u_int			uNumberOfVertices	= Tools_ObjectProcessor_NormalMapped::GetTotalNumberOfVertices();
		const u_int			uNumberOfIndices	= Tools_ObjectProcessor_NormalMapped::GetTotalNumberOfIndices();
		
		u_int				uFileSize			= 0;
		
		// Hashed copy of the filename
		uFileSize += sizeof( Asura_Hash_ID );
		
		// Number of strips (u_int)
		uFileSize += sizeof( u_int );

		// For each object strip:
		// MaterialID (int)
		// StartIndex (u_int)
		// NumberOfIndices (u_int)
		// NumberOfTriangles (u_int)
		uFileSize += sizeof( u_int ) * 4 * 1 /* Don't have multiple strips at the moment */;

		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
			// Secondary material hash ID
			uFileSize += sizeof( u_int );
		#endif

		// Number of vertices (u_int)
		// And then each vertex will be sizeof( ObjectVertex )
		uFileSize += sizeof( u_int ) + sizeof( ObjectVertex ) * uNumberOfVertices;

		// Number of indices (u_int)
		// And each index will be (u_short)
		uFileSize += sizeof( u_int ) + sizeof( u_short ) * uNumberOfIndices;

		// Chunk size is all the data, 'filename' & header
		int							uChunkSize	= uFileSize + uFileNameSize + sizeof(Asura_Chunk_ResourceFile);
		u_char*						pucData		= new u_char [uChunkSize];

		Asura_Chunk_Memory_Stream	xStream (uChunkSize, pucData);

		Asura_Chunk_ResourceFile	xResourceFile;

		xResourceFile.Create
		(
			uChunkSize,
			0,
			ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
			ASURA_RESOURCEFILE_TYPE_XBOX360_OBJECT,
			uFileSize
		);

		unsigned long uBytesWritten = 0;
		
		xResourceFile.Version = 0;


		const Tools_ObjectProcessor_NormalMapped_Strip&	xStrip	= *Tools_ObjectProcessor_NormalMapped::GetProcessedStrip ();
        

		// Write the data...

		xStream.SetSwapEndian	(Tools_Level_Export::ShouldExportBigEndian());
		xStream.Write			(sizeof(xResourceFile), &xResourceFile);

		xStream << xName;
		xStream << uHashID;

		// Just one strip for now
		xStream << static_cast< u_int >(1);

		#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
			xStream << xStrip.m_auMaterialResponseHashID[0];
			xStream << xStrip.m_auMaterialResponseHashID[1];
		#else
			xStream << xStrip.m_iMaterialID;
		#endif

		xStream << static_cast< u_int >(0);		// Always start at 0, since only one strip at the moment
		xStream << xStrip.m_uNumberOfIndices;
		xStream << xStrip.m_uNumberOfTriangles;

		xStream << xStrip.m_uNumberOfVertices;

		// Write-out the vertices for this strip...

		for (u_int v = 0; v < xStrip.m_uNumberOfVertices; v++)
		{
			const Tools_ObjectProcessor_NormalMapped_Vertex&	xVertex	= xStrip.m_pxVertices [v];

			ObjectVertex xOutputVertex;

			// position
			xOutputVertex.m_xPosition = xVertex.m_xPosition;

			// uvs
			xOutputVertex.m_xU	= xVertex.m_fU;
			xOutputVertex.m_xV	= xVertex.m_fV;
			xOutputVertex.m_xU2	= xVertex.m_fU2;
			xOutputVertex.m_xV2	= xVertex.m_fV2;
			xOutputVertex.m_fAlpha = xVertex.m_fAlpha;

			// normal/tangent/binormal
			xOutputVertex.m_xNormal = xVertex.m_xNormal;
			xOutputVertex.m_xTangent = xVertex.m_xTangent;
			xOutputVertex.m_xBinormal = xVertex.m_xBinormal;

			// Write this vertex...
			xOutputVertex.WriteToChunkStream( xStream );
		}

		xStream << xStrip.m_uNumberOfIndices;

		// Write the indices in one go...
		{
			Asura_WriteToStream_Array_Wrapper<u_short>	xIndexArray (xStrip.m_uNumberOfIndices, xStrip.m_pusIndices);

			// Write the index data...
			xStream << xIndexArray;
		}

		// Wrap up and delete all the temp data...
		WriteFile										(hOutput, pucData, uChunkSize, &uBytesWritten, NULL);
		delete []										(pucData);
		Tools_ObjectProcessor_NormalMapped::Shutdown	();
		return											(true);
	}

	else
	{
		return	(false);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Function     : Tools_Xbox360_Export::ExportObjectHierarchy
// Author       : Kevin Floyer-Lea
// Date         : 26-Sep-2005
//
// Description  : 
//
// Parameters   : 
//
// Return value : 
////////////////////////////////////////////////////////////////////////////////

bool Tools_Xbox360_Export::ExportObjectHierarchy (const HANDLE hOutput,
												  const Asura_Hierarchy_Skin* pxSkin)
{
	// initialise and stripify mesh
	if (Tools_HierarchyProcessor_MultiMaterial_NormalMapped::Initialise(pxSkin))
	{
		u_short				usRestartIndex;

		const bool			bUseRestartIndex	= Tools_MeshRestartIndex::IsValidForCurrentPlatform (usRestartIndex);
		const Asura_String	xName				= pxSkin->Name;
	
		const u_int			uNumberOfStrips		= Tools_HierarchyProcessor_MultiMaterial_NormalMapped::GetTotalNumberOfStrips();
		const u_int			uNumberOfVertices	= Tools_HierarchyProcessor_MultiMaterial_NormalMapped::GetTotalNumberOfVertices();
		const u_int			uNumberOfIndices	= Tools_HierarchyProcessor_MultiMaterial_NormalMapped::GetTotalNumberOfIndices();
		
		u_int				uStartIndex			= 0;
		u_int				uStartVertex		= 0;
		

		Asura_Chunk_Memory_Stream xStream;

		xStream.WriteResourceChunk_Start( ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC, ASURA_RESOURCEFILE_TYPE_XBOX360_OBJECTHIERARCHY, 0, 0, xName );
		
		xStream << pxSkin->m_uNameHashID;
		xStream << uNumberOfStrips;
		xStream << uNumberOfVertices;
		xStream << uNumberOfIndices;

		Tools_HierarchyProcessor_MultiMaterial_NormalMapped::ConvertMaterialsForExport();

		// Write each strip in-turn...
		for( u_int uStrip = 0; uStrip < uNumberOfStrips; ++uStrip )
		{
			const Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Strip* pxStrip = Tools_HierarchyProcessor_MultiMaterial_NormalMapped::GetProcessedStrip( uStrip );

			#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
				xStream << pxStrip->m_auMaterialResponseHashID[0];
				xStream << pxStrip->m_auMaterialResponseHashID[1];
			#else
				xStream << pxStrip->m_iMaterialID;
			#endif

			xStream << static_cast<u_int>(pxStrip->m_usBoneIndex);

			xStream << uStartIndex;
			xStream << pxStrip->m_uNumberOfIndices;

			uStartIndex += pxStrip->m_uNumberOfIndices;

			xStream << pxStrip->m_uNumberOfTriangles;
		}

		Asura_Assert	((uNumberOfIndices == uStartIndex), "Mismatch in the number of indices for the current object-hierarchy.");

		// Write out all of the vertices in one go...
		for (u_int s = 0; s < uNumberOfStrips; s++)
		{
			const Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Strip&	xStrip = *Tools_HierarchyProcessor_MultiMaterial_NormalMapped::GetProcessedStrip (s);

			// Write-out the vertices for this strip...
			for (u_int v = 0; v < xStrip.m_uNumberOfVertices; v++)
			{
				const Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Vertex& xVertex = xStrip.m_pxVertices[v];

				ObjectVertex xOutputVertex;

				// position
				xOutputVertex.m_xPosition = xVertex.m_xPosition;

				// uvs
				xOutputVertex.m_xU = xVertex.m_fU0;
				xOutputVertex.m_xV = xVertex.m_fV0;

				xOutputVertex.m_xU2 = xVertex.m_fU1;
				xOutputVertex.m_xV2 = xVertex.m_fV1;
				xOutputVertex.m_fAlpha = xVertex.m_fAlpha;


				// normal/tangent/binormal
				xOutputVertex.m_xNormal = xVertex.m_xNormal;
				xOutputVertex.m_xTangent = xVertex.m_xTangent;
				xOutputVertex.m_xBinormal = xVertex.m_xBinormal;

				// Write this vertex...
				xOutputVertex.WriteToChunkStream( xStream );
			}
		}

		
		// Write out all of the indices in one go...
		for (u_int s = 0; s < uNumberOfStrips; s++)
		{
			const Tools_HierarchyProcessor_MultiMaterial_NormalMapped_Strip&	xStrip = *Tools_HierarchyProcessor_MultiMaterial_NormalMapped::GetProcessedStrip (s);

			// Need to add the offset to each index - write them individually...
			for (u_int i = 0; i < xStrip.m_uNumberOfIndices; i++)
			{
				u_short	usIndex	= xStrip.m_pusIndices [i];
				
				// Is this a restart-index?
				if (!bUseRestartIndex || (usIndex != usRestartIndex))
				{
					// No - add the start index and validate...
					usIndex	= usIndex + static_cast<u_short>(uStartVertex);

					Asura_Assert	((xStrip.m_uNumberOfVertices > xStrip.m_pusIndices [i]), "Object-hierarchy index out of range.");
				}

				// Write out this index...
				xStream << usIndex;
			}

			// Step onto the next index...
			uStartVertex  += xStrip.m_uNumberOfVertices;
		}

		Asura_Assert	((uNumberOfVertices == uStartVertex), "Mismatch in the number of vertices for the current object-hierarchy.");

		// Wrap up and delete all the temp data...

		xStream.WriteResourceChunk_End();

		const void* pData = xStream.GetStartAddress();
		u_int uChunkSize = xStream.GetPosition();
		unsigned long uBytesWritten = 0;

		WriteFile														(hOutput, pData, uChunkSize, &uBytesWritten, NULL);
	
		Tools_HierarchyProcessor_MultiMaterial_NormalMapped::Shutdown	();
		return															(true);
	}

	else
	{
		return	(false);
	}
}

#if defined ASURA_USE_BOTANICALS

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::FindUsedBotanicalsTemplatesAndShapes()
//
// Produces lists of botanicals templates actually used in the current level
// and the botanicals shapes they use. Also computes file size required for this
// export data
//
// Parameters:		uNumUsedTemplates - Reference to output number of templates
//										that are in the list
//					xUsedTemplateList - Reference to output list of templates
//										used in the level
//					uNumUsedShapes - Reference to output number of shapes that
//									 are in the list
//					xUsedShapeList - Reference to output list of shapes used by
//									 templates in the level
//					uFileSize - Reference to file size that will be incremented
//								as appropriate for this export
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::FindUsedBotanicalsTemplatesAndShapes( u_int& uNumUsedTemplates, Asura_LinkList< Tools_Xbox360_Export_Botanical_Template >& xBlockTemplateList,
																 u_int& uNumUsedShapes, Asura_LinkList< const Tools_BotanicalShape* >& xUsedShapeList)
{
	const int iNumInstances = Asura_Botanicals_InstanceManager::GetNumInstances();

	// Look through all instances and gather the used templates and shapes

	int iCurrInstanceTimes100 = 0;

	const Asura_LinkList< Asura_Botanicals_Instance* >& xInstanceList = Asura_Botanicals_InstanceManager::GetInstanceList();

	for( Asura_ListElt< Asura_Botanicals_Instance* >* pxCurrInstanceListEntry = xInstanceList.Front();
		 pxCurrInstanceListEntry;
		 pxCurrInstanceListEntry = pxCurrInstanceListEntry->Next() )
	{
		Tools_Level_Export::UpdateStatus( Tools_Level_Export::EXPORT_STAGE_BOTANICALS_FIND_USED_TEMPLATES_AND_SHAPES, "Botanicals: Finding used templates and shapes...", iCurrInstanceTimes100 / iNumInstances );
		iCurrInstanceTimes100 += 100;

		Asura_Botanicals_Instance* const pxCurrInstance = pxCurrInstanceListEntry->Data();
		const Asura_Botanicals_Template* const pxCurrTemplate = Asura_Botanicals_TemplateManager::GetTemplate( pxCurrInstance->iTemplateIdx );

		if( pxCurrTemplate->eType != ASURA_BOTANICALS_TEMPLATE_BOTANICALSHAPE // No point supporting shape-based botanicals on 360
		 || pxCurrTemplate->bMergeRenderMeshWithEnv
		 || !pxCurrTemplate->pxBotanicalShape )
		{
			continue;
		}

		int iModuleIdx = pxCurrInstance->iModuleIdx;
		if ( ( iModuleIdx < 0 ) || ( iModuleIdx >= Asura_Module_System::GetNumberOfModules() ) )
		{
			Asura_Direct3D_BGR_Manager& xBGRManager = Asura_Direct3D_Botanicals_Game_Render::GetDirect3DBGRManager();
			Asura_Assertf( false, ( "Invalid module index %d exporting template[%d] '%s' Instance %d\n", 
						   iModuleIdx, pxCurrInstance->iTemplateIdx, pxCurrTemplate->xTemplateName.GetString(), ( iCurrInstanceTimes100/100 ) ) );
			iModuleIdx = xBGRManager.GetModuleIndexFromPosition( pxCurrInstance->xPosition, pxCurrInstance->xOrientation );
			pxCurrInstance->iModuleIdx = iModuleIdx;
		}
		
		Tools_Xbox360_Export_Botanical_Template* pxBlockTemplate = NULL;
		
		{
			for( Asura_ListElt< Tools_Xbox360_Export_Botanical_Template >* pxCurrBlockTemplate = xBlockTemplateList.Front(); NULL != pxCurrBlockTemplate; pxCurrBlockTemplate = pxCurrBlockTemplate->Next())
			{
				Tools_Xbox360_Export_Botanical_Template& xBlockTemplate = pxCurrBlockTemplate->Data();
				
				if( iModuleIdx != xBlockTemplate.m_iModuleIndex)
				{
					continue;
				}
				
				if( pxCurrTemplate != xBlockTemplate.m_pxTemplate)
				{
					continue;
				}
				
				pxBlockTemplate = &xBlockTemplate;
				break;
			}
		}
		
		if( NULL != pxBlockTemplate)
		{
			Asura_Assert( iModuleIdx == pxBlockTemplate->m_iModuleIndex, "Export has a bug");
			Asura_Assert( pxCurrTemplate == pxBlockTemplate->m_pxTemplate, "Export has a bug");
			pxBlockTemplate->m_xListInstances.Push_Front( pxCurrInstance);
			continue;
		}

		const Tools_BotanicalShape* const pxToolsShape = pxCurrTemplate->pxBotanicalShape->GetToolsShape();
		int iShapeIndex = 0;
		
		for( Asura_ListElt< const Tools_BotanicalShape*>* pxItCurrShape = xUsedShapeList.Front(); NULL != pxItCurrShape; pxItCurrShape = pxItCurrShape->Next())
		{
			const Tools_BotanicalShape* pxCurrShape = pxItCurrShape->Data();
			
			if( pxToolsShape == pxCurrShape)
			{
				break;
			}
			
			iShapeIndex += 1;
		}
		
		if( uNumUsedShapes <= static_cast< u_int>( iShapeIndex))
		{
			xUsedShapeList.Push_Back( pxToolsShape );
			++uNumUsedShapes;
		}

		{
#if defined( ASURA_ASSERT)
			int iShapeIndexLast = 0;
#endif
			bool bInserted = false;
			for( Asura_ListElt< Tools_Xbox360_Export_Botanical_Template >* pxCurrBlockTemplate = xBlockTemplateList.Front(); NULL != pxCurrBlockTemplate; pxCurrBlockTemplate = pxCurrBlockTemplate->Next())
			{
				Tools_Xbox360_Export_Botanical_Template& xBlockTemplate = pxCurrBlockTemplate->Data();
				
#if defined( ASURA_ASSERT)
				Asura_Assert( iShapeIndexLast <= xBlockTemplate.m_iShapeIndex, "BlockTemplates are not in Shape order");
				iShapeIndexLast = xBlockTemplate.m_iShapeIndex;
#endif
				if( xBlockTemplate.m_iShapeIndex < iShapeIndex)
				{
					continue;
				}
				
				xBlockTemplateList.Insert_Before( pxCurrBlockTemplate, Tools_Xbox360_Export_Botanical_Template());
				pxBlockTemplate = &(pxCurrBlockTemplate->Prev()->Data());
				bInserted = true;
				break;
			}

			if( false == bInserted)		
			{
				xBlockTemplateList.Push_Back( Tools_Xbox360_Export_Botanical_Template());
				pxBlockTemplate = &(xBlockTemplateList.Back()->Data());
			}
		}
			
		pxBlockTemplate->m_iModuleIndex = iModuleIdx;
		pxBlockTemplate->m_iShapeIndex = iShapeIndex;
		pxBlockTemplate->m_pxTemplate = pxCurrTemplate;
		pxBlockTemplate->m_xListInstances.Push_Front( pxCurrInstance);
		
		++uNumUsedTemplates;
	}
	
	u_int uCurrShapeTimes100 = 0;

	// Now that we know which shapes are in the level, calculate their export file size

	for( Asura_ListElt< const Tools_BotanicalShape* >* pxCurrShapeListEntry = xUsedShapeList.Front();
		 pxCurrShapeListEntry;
		 pxCurrShapeListEntry = pxCurrShapeListEntry->Next() )
	{
		Tools_Level_Export::UpdateStatus( Tools_Level_Export::EXPORT_STAGE_BOTANICALS_CALC_SHAPE_EXPORT_SIZE, "Botanicals: calculating shape export size...", uCurrShapeTimes100 / uNumUsedShapes );
		uCurrShapeTimes100 += 100;

		const Tools_BotanicalShape* const pxCurrShape = pxCurrShapeListEntry->Data();

		const int iNumElements = pxCurrShape->GetNumElements();

		// Loop over each element

		for( int i = 0; i < iNumElements; ++i )
		{
			const Tools_BotanicalElement* const pxCurrElement = pxCurrShape->GetElementByIndex(i);
			if( !pxCurrElement )
			{
				Asura_Assert( false, "Botanicals element is invalid." );
				continue;
			}

			const int iNumLODs = pxCurrElement->GetNumLODs();

			// Loop over each LOD level

			for( int j = 0; j < iNumLODs; ++j )
			{
				const Tools_BotanicalElementLOD* const pxCurrLOD = pxCurrElement->GetLODByIndex(j);
				if( !pxCurrLOD )
				{
					Asura_Assert( false, "Botanicals LOD is invalid." );
					continue;
				}

				switch( pxCurrElement->m_eType )
				{
					case TOOLS_BOTANICALS_ELEMENTTYPE_1SIDED_MESH:
					case TOOLS_BOTANICALS_ELEMENTTYPE_2SIDED_MESH:
					{
						IncrementExportSize_Geometry( *pxCurrLOD );
						break;
					}

					case TOOLS_BOTANICALS_ELEMENTTYPE_BILLBOARDS:
					{
						IncrementExportSize_Billboards( *pxCurrLOD );
						break;
					}

					case TOOLS_BOTANICALS_ELEMENTTYPE_IMPOSTER:
					{
						IncrementExportSize_Imposter( *pxCurrLOD );
						break;
					}

					default:
					{
						Asura_Assert( false, "Unknown botanical element type." );
						break;
					}
				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::IncrementExportSize_Geometry()
//
// Calculates the size required to export a given geometric botanicals LOD and
// increments the passed count by that much
//
// Parameters:		uFileSize - File size. Will be incremented as appropriate
//					xElementLOD - Geometry LOD to process
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::IncrementExportSize_Geometry( const Tools_BotanicalElementLOD& xElementLOD )
{
	if( !xElementLOD.m_pxMesh || !xElementLOD.m_pxMesh->TextureDataValid )
	{
		Asura_Assert( false, "Invalid parameter passed to Tools_Xbox360_Export::IncrementExportSize_Geometry()." );
		return;
	}

	if( !Tools_ObjectProcessor_NormalMapped::Initialise( xElementLOD.m_pxMesh, 0, true, Tools_TriStripper::OUTPUT_TRILIST ) ) return;

	Tools_ObjectProcessor_NormalMapped::Shutdown();
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::IncrementExportSize_Billboards()
//
// Calculates the size required to export a given billboards botanicals LOD and
// increments the passed count by that much
//
// Parameters:		uFileSize - File size. Will be incremented as appropriate
//					xElementLOD - Billboards LOD to process
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::IncrementExportSize_Billboards( const Tools_BotanicalElementLOD& xElementLOD )
{
	if( !xElementLOD.m_pxMesh || !xElementLOD.m_pxMesh->TextureDataValid )
	{
		Asura_Assert( false, "Invalid parameter passed to Tools_Xbox360_Export::IncrementExportSize_Geometry()." );
		return;
	}

}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::IncrementExportSize_Imposter()
//
// Calculates the size required to export a given imposter botanicals LOD and
// increments the passed count by that much
//
// Parameters:		uFileSize - File size. Will be incremented as appropriate
//					xElementLOD - Imposter LOD to process
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::IncrementExportSize_Imposter( const Tools_BotanicalElementLOD& xElementLOD )
{
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportBotanicalsElementLOD_Geometry()
//
// Exports the given geometric botanicals LOD to a chunk stream
//
// Parameters:		xStream - Output chunk stream
//					xElementLOD - Geometry LOD to process
//					piMaterialConversionList - Conversion array mapping from
//											   full material IDs to indices into
//											   the botanicals material chunk.
//											   Must be ASURA_MAX_NO_OF_MATERIALS
//											   in size
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::ExportBotanicalsElementLOD_Geometry( Asura_Chunk_Stream& xStream, const Tools_BotanicalElementLOD& xElementLOD, const int* const piMaterialConversionList, const Tools_BGR_Type_BoneWeightCalc* pxBoneWeightData)
{
	if( !xElementLOD.m_pxMesh || !xElementLOD.m_pxMesh->TextureDataValid )
	{
		Asura_Assert( false, "Invalid parameter passed to Tools_Xbox360_Export::ExportBotanicalsElementLOD_Geometry()." );
		return;
	}

	// We'll assume only one material per LOD mesh so we can use the object processor
	const int iMaterialID = xElementLOD.m_pxMesh->TextureDataPtr[0].m_iMaterialID;

	// Important - can't use tristrips because there's no way to insert correct degenerates between instances


	// Strip out the evil 'double sided' flag - as the botanicals have proper double-sided lighting!
	for(int i=0;i<xElementLOD.m_pxMesh->NumberOfPolygons;i++)
	{		
		if(xElementLOD.m_pxMesh->PolygonFlagDataValid)
		{
			if (xElementLOD.m_pxMesh->PolygonFlagDataPtr[i].Flags & AsuraPolygonFlag_DoubleSided)
			{
				xElementLOD.m_pxMesh->PolygonFlagDataPtr[i].Flags-=AsuraPolygonFlag_DoubleSided;
			}
		}				
	}

	if( !Tools_ObjectProcessor_NormalMapped::Initialise( xElementLOD.m_pxMesh, iMaterialID, true, Tools_TriStripper::OUTPUT_TRILIST ) ) return;
	const Tools_ObjectProcessor_NormalMapped_Strip& xStrip = *Tools_ObjectProcessor_NormalMapped::GetProcessedStrip();

	Asura_Assertf( xStrip.m_uNumberOfIndices > 0 && xStrip.m_uNumberOfVertices > 0,
		("Botanicals mesh '%s' has no visible polygons.", xElementLOD.m_pxMesh->Name.GetString()) );
	   
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	xStream << xElementLOD.m_pxMesh->TextureDataPtr[0].m_auMaterialResponseHashID[0];
#else
	xStream << Asura_Material_System::ConvertIDToIndex( xStrip.m_iMaterialID, piMaterialConversionList );
#endif
	xStream << xStrip.m_uNumberOfIndices;
	xStream << xStrip.m_uNumberOfTriangles;
	xStream << xStrip.m_uNumberOfVertices;

	for( u_int u = 0; u < xStrip.m_uNumberOfVertices; ++u )
	{
		const Tools_ObjectProcessor_NormalMapped_Vertex& xVertex = xStrip.m_pxVertices[u];

		xStream << xVertex.m_xPosition;
		xStream << static_cast< Asura_HalfFloat >( xVertex.m_fU );
		xStream << static_cast< Asura_HalfFloat >( xVertex.m_fV );

		xStream << Asura_CompressedVector3_101010N(xVertex.m_xNormal);
		xStream << Asura_CompressedVector3_101010N(xVertex.m_xTangent);
		xStream << Asura_CompressedVector3_101010N(xVertex.m_xBinormal);
		
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
		const float fBoneWeight = pxBoneWeightData->CalculateBoneWeight( Asura_AVector3( xVertex.m_xPosition));
#else
		const float fBoneWeight = 1.0f;
#endif
		xStream << static_cast< float>( fBoneWeight);
	}

	// Have to write out the indices as 32-bit for now, since there's no
	// single 16-bit component vertex declaration type

	for( u_int u = 0; u < xStrip.m_uNumberOfIndices; ++u )
	{
		xStream << static_cast< u_int >( xStrip.m_pusIndices[u] );
	}

	Tools_ObjectProcessor_NormalMapped::Shutdown();
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportBotanicalsElementLOD_Billboards()
//
// Exports the given billboards botanicals LOD to a chunk stream
//
// Parameters:		xStream - Output chunk stream
//					xElementLOD - Billboards LOD to process
//					piMaterialConversionList - Conversion array mapping from
//											   full material IDs to indices into
//											   the botanicals material chunk.
//											   Must be ASURA_MAX_NO_OF_MATERIALS
//											   in size
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::ExportBotanicalsElementLOD_Billboards( Asura_Chunk_Stream& xStream, const Tools_BotanicalElementLOD& xElementLOD, const int* const piMaterialConversionList, const Tools_BGR_Type_BoneWeightCalc* pxBoneWeightData)
{
	if( !xElementLOD.m_pxMesh || !xElementLOD.m_pxMesh->TextureDataValid )
	{
		Asura_Assert( false, "Invalid parameter passed to Tools_Xbox360_Export::ExportBotanicalsElementLOD_Billboards()." );
		return;
	}


	const Tools_BotanicalRenderable_Billboards xRenderable( &xElementLOD );
	const int iNumBillboards = xRenderable.GetNumBillboards();
	const int iNumVertices = iNumBillboards * 4;

	// (AAM - 22.09.2006) I initially used 3 verts per billboard with the D3DPT_RECTLIST primitive
	// type, but those don't get clipped to the frustum


	// We'll assume only one material per LOD mesh
#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	const uint32 uMaterialID = xElementLOD.m_pxMesh->TextureDataPtr[0].m_auMaterialResponseHashID[0];
	xStream << uMaterialID;	
#else	
	const int iMaterialID = xElementLOD.m_pxMesh->TextureDataPtr[0].m_iMaterialID;
	xStream << Asura_Material_System::ConvertIDToIndex( iMaterialID, piMaterialConversionList );
#endif

	xStream << iNumVertices;

	// Per-vertex data

	for( int i = 0; i < iNumBillboards; ++i )
	{
		const Tools_Botanicals_Billboard& xBillboard = xRenderable.GetBillboard(i);

		const Asura_HalfFloat xHalfWidth = xBillboard.fWidth * 0.5f;
		const Asura_HalfFloat xHalfHeight = xBillboard.fHeight * 0.5f;
		const Asura_HalfFloat xU[2] = { xBillboard.fU[0], xBillboard.fU[1] };
		const Asura_HalfFloat xV[2] = { xBillboard.fV[0], xBillboard.fV[1] };

		

		// Upper left corner
		xStream << -xHalfWidth << -xHalfHeight;
		xStream << xU[0] << xV[0];		

		// Upper right corner
		xStream << xHalfWidth << -xHalfHeight;
		xStream << xU[1] << xV[0];


		// Lower right corner
		xStream << xHalfWidth << xHalfHeight;
		xStream << xU[1] << xV[1];

		// Lower left corner
		xStream << -xHalfWidth << xHalfHeight;
		xStream << xU[0] << xV[1];
	}

	// Per-billboard data

	for( int i = 0; i < iNumBillboards; ++i )
	{
		const Tools_Botanicals_Billboard& xBillboard = xRenderable.GetBillboard(i);

		// Position relative to instance location
		const Asura_HalfFloat axPos[4] = { xBillboard.xPos.x, xBillboard.xPos.y, xBillboard.xPos.z, 1.0f };
		xStream << Asura_WriteToStream_Array_Wrapper< Asura_HalfFloat >( 4, axPos );

		// Normal (faces away from the cluster centre)
		xStream << Asura_CompressedVector3_101010N(xBillboard.xNormal);

		// Tangent Frame - Same at each vertex. But got to handle
		// possible U/V flips, so it's not the same for all billboards?	

		const Asura_CompressedVector3_101010N	xTangent( ((xBillboard.fU[0] < xBillboard.fU[1]) ? +1.0f : -1.0f) , 0.0f, 0.0f);
		const Asura_CompressedVector3_101010N	xBinormal( 0.0f, ((xBillboard.fV[0] < xBillboard.fV[1]) ? +1.0f : -1.0f) , 0.0f);

		xStream << xTangent;	
		xStream << xBinormal;	
		
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
		const float fBoneWeight = pxBoneWeightData->CalculateBoneWeight( Asura_AVector3( xBillboard.xPos));
#else
		const float fBoneWeight = 1.0f;
#endif
		xStream << static_cast< float>( fBoneWeight);
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportBotanicalsElementLOD_Imposter()
//
// Exports the given imposter botanicals LOD to a chunk stream
//
// Parameters:		xStream - Output chunk stream
//					xElementLOD - Imposter LOD to process
//					piMaterialConversionList - Conversion array mapping from
//											   full material IDs to indices into
//											   the botanicals material chunk.
//											   Must be ASURA_MAX_NO_OF_MATERIALS
//											   in size
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::ExportBotanicalsElementLOD_Imposter( Asura_Chunk_Stream& xStream, const Tools_BotanicalElementLOD& xElementLOD, const int* const piMaterialConversionList, const Tools_BGR_Type_BoneWeightCalc* pxBoneWeightData)
{
	// Number of different orientations defined for this imposter
	int iNumImposterAngles = xElementLOD.GetNumImposters();

	xStream << iNumImposterAngles;																 // 1					

	const Tools_Botanicals_Imposter *pxImp = xElementLOD.GetImposterByIndexConst(0);	

#ifdef ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	uint32 uMaterialID = 0;
	if (pxImp)
	{
		if (pxImp->m_pxMesh)
		{		
			if (pxImp->m_pxMesh->TextureDataValid)
			{
				uMaterialID = pxImp->m_pxMesh->TextureDataPtr[0].m_auMaterialResponseHashID[0];
			}
			else
			{
				uMaterialID = ASURA_HASH_ID_UNSET;
			}
		}
	}

	xStream << uMaterialID;	
#else
	int iMaterialID = 0;
	if (pxImp)
	{
		if (pxImp->m_pxMesh)
		{		
			if (pxImp->m_pxMesh->TextureDataValid)
			{
				iMaterialID = pxImp->m_pxMesh->TextureDataPtr[0].m_iMaterialID;
			}
			else
			{
				iMaterialID = ASURA_MATERIAL_NONE;
			}
		}
	}

	iMaterialID = Asura_Material_System::ConvertIDToIndex( iMaterialID, piMaterialConversionList ); // 2 x4bytes
	xStream << iMaterialID;
#endif		

	for(int i=0;i<iNumImposterAngles;i++)
	{
		const Tools_Botanicals_Imposter *pxImp = xElementLOD.GetImposterByIndexConst(i);
		Asura_Assert(pxImp->m_bValid, "Attempting to export invalid/uninitialised imposter\n");
			
		xStream << pxImp->m_xBillboard.fWidth;						
		xStream << pxImp->m_xBillboard.fHeight;						
		xStream << pxImp->m_xBillboard.fU[0];						
		xStream << pxImp->m_xBillboard.fV[0];						
		xStream << pxImp->m_xBillboard.fU[1];						
		xStream << pxImp->m_xBillboard.fV[1];						
		xStream << Asura_CompressedVector3_101010N(pxImp->m_xBillboard.xNormal);		

		const Asura_CompressedVector3_101010N	xTangent( ((pxImp->m_xBillboard.fU[0] < pxImp->m_xBillboard.fU[1]) ? +1.0f : -1.0f) , 0.0f, 0.0f);
		const Asura_CompressedVector3_101010N	xBinormal( 0.0f, ((pxImp->m_xBillboard.fV[0] < pxImp->m_xBillboard.fV[1]) ? +1.0f : -1.0f) , 0.0f);

		xStream << xTangent;	
		xStream << xBinormal;	


		xStream << pxImp->m_xBillboard.xPos;						
		xStream << pxImp->m_fAngleRange;							
		xStream << pxImp->m_fFadeRange;								
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportBotanicalsShapes()
//
// Exports a list of botanicals shapes to a chunk stream
//
// Parameters:		xStream - Reference to output chunk stream
//					xShapeList - Reference to list of botanicals shapes
//					piMaterialConversionList - Array for converting materal IDs
//											   to export indices. Must be
//											   ASURA_MAX_NO_OF_MATERIALS in size
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::ExportBotanicalsShapes( Asura_Chunk_Stream& xStream, const Asura_LinkList< const Tools_BotanicalShape* >& xShapeList, const int* const piMaterialConversionList )
{
	const u_int uNumShapes = xShapeList.Size();
	u_int uCurrShapeTimes100 = 0;

	for( Asura_ListElt< const Tools_BotanicalShape* >* pxCurrShapeListEntry = xShapeList.Front();
		 pxCurrShapeListEntry;
		 pxCurrShapeListEntry = pxCurrShapeListEntry->Next() )
	{
		Tools_Level_Export::UpdateStatus( Tools_Level_Export::EXPORT_STAGE_BOTANICALS_EXPORTING_SHAPES, "Botanicals: Exporting shapes...", uCurrShapeTimes100 / uNumShapes );
		uCurrShapeTimes100 += 100;

		float fShapeBoundingRadius = 0;
		Asura_Bounding_Box xShapeBoundingBox;
		xShapeBoundingBox.Reset();

		const Tools_BotanicalShape* const pxCurrShape = pxCurrShapeListEntry->Data();
		Tools_BGR_Type xBGRType( *pxCurrShape);
		
		xStream << Asura_GetHashID( pxCurrShape->GetName() );
		
		const int iNumElements = pxCurrShape->GetNumElements();
		xStream << iNumElements;

		// Loop over elements
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
		Tools_BGR_Type_BoneWeightCalc xBoneWeightData;
		const Asura_BGR_Type_BasicInteractive* const pxTypeBasicInteractive = xBGRType.GetBasicInteractive();
		if( NULL != pxTypeBasicInteractive)
		{
			xBoneWeightData.SetupBoneWeight( *pxTypeBasicInteractive);
		}
#endif


		for( int i = 0; i < iNumElements; ++i )
		{
			const Tools_BotanicalElement* const pxCurrElement = pxCurrShape->GetElementByIndex(i);
			if( !pxCurrElement )
			{
				Asura_Assert( false, "Botanicals element is invalid." );
				continue;
			}

			xStream << static_cast< u_int >( pxCurrElement->m_eType ); // Ensure 32-bit

			const int iNumLODs = pxCurrElement->GetNumLODs();
			xStream << iNumLODs;

#if defined( ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS) && 1 == ASURA_BOTANICAL_SUPPORT_SECTION_VERSIONS
			xStream << xBGRType.GetSectionID( (pxCurrShape->GetSectionByIndex( i))->m_xName.GetHash());
			xStream << xBGRType.GetVersionID( (pxCurrShape->GetVersionByIndex( i))->m_xName.GetHash());
#endif
			Asura_Bounding_Box xOverallBB;
			xOverallBB.Reset();

			// Loop over LODs

			for( int j = 0; j < iNumLODs; ++j )
			{
				const Tools_BotanicalElementLOD* const pxCurrLOD = pxCurrElement->GetLODByIndex(j);
				if( !pxCurrLOD )
				{
					Asura_Assert( false, "Botanicals LOD is invalid." );
					continue;
				}

				switch( pxCurrElement->m_eType )
				{
					case TOOLS_BOTANICALS_ELEMENTTYPE_1SIDED_MESH:
					case TOOLS_BOTANICALS_ELEMENTTYPE_2SIDED_MESH:
					{
						ExportBotanicalsElementLOD_Geometry( xStream, *pxCurrLOD, piMaterialConversionList
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
							, &xBoneWeightData
#else
							, NULL
#endif
						);
						break;
					}

					case TOOLS_BOTANICALS_ELEMENTTYPE_BILLBOARDS:
					{
						ExportBotanicalsElementLOD_Billboards( xStream, *pxCurrLOD, piMaterialConversionList
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
							, &xBoneWeightData
#else
							, NULL
#endif
						 );
						break;
					}

					case TOOLS_BOTANICALS_ELEMENTTYPE_IMPOSTER:
					{
						ExportBotanicalsElementLOD_Imposter( xStream, *pxCurrLOD, piMaterialConversionList
#if defined( ASURA_BOTANICAL_SUPPORT_INTERACTIVE)
							, &xBoneWeightData
#else
							, NULL
#endif
						 );
						break;
					}

					default:
					{
						Asura_Assert( false, "Unknown botanicals element type." );
						break;
					}
				}

				if( pxCurrLOD->m_pxMesh )
				{
					Asura_Assert( pxCurrLOD->m_pxMesh->VertexDataValid, "Botanicals mesh vertex data isn't valid." );

					for( int k = 0; k < pxCurrLOD->m_pxMesh->NumberOfVertices; ++k )
					{
						const Asura_Vector_3& xPosition = pxCurrLOD->m_pxMesh->VertexDataPtr[k].Position;
						xOverallBB.Set( xPosition);
						
						const float fOriginRadius = xPosition.Magnitude();
						fShapeBoundingRadius = Asura_Maths::Max( fOriginRadius, fShapeBoundingRadius);
					}
				}
			}

			const Asura_Vector_3 xBSCentre = xOverallBB.CalculateCentre();
			const float fBSRadius = xOverallBB.CalculateRadius();
			
			xShapeBoundingBox.Set( xOverallBB);

			// Assume elements' vertices are roughly centred horizontally
			xStream << xBSCentre.y;
			xStream << fBSRadius;

			// Append fade data

			for( int j = 0; j < iNumLODs; ++j )
			{
				const Tools_BotanicalElementLOD* const pxCurrLOD = pxCurrElement->GetLODByIndex(j);
				if( !pxCurrLOD )
				{
					Asura_Assert( false, "Botanicals LOD is invalid." );
					continue;
				}

				// In Tools the values are higher for nearer distances

				const float fMinDistance = (1.0f - pxCurrLOD->m_fLod1) / pxCurrShape->m_fInvMaxDrawDistance;
				const float fMinFullAlphaDistance = (1.0f - pxCurrLOD->m_fLod1 + pxCurrLOD->m_fFadeLength1) / pxCurrShape->m_fInvMaxDrawDistance;
				const float fMaxDistance = (1.0f - pxCurrLOD->m_fLod0) / pxCurrShape->m_fInvMaxDrawDistance;
				const float fMaxFullAlphaDistance = (1.0f - pxCurrLOD->m_fLod0 - pxCurrLOD->m_fFadeLength0) / pxCurrShape->m_fInvMaxDrawDistance;
				const u_int uNumVisibleInstances = 0;

				xStream << fMinDistance;
				xStream << fMinFullAlphaDistance;
				xStream << fMaxFullAlphaDistance;
				xStream << fMaxDistance;
				xStream << uNumVisibleInstances;
			}
		}
		
		xBGRType.SetBoundingBox( xShapeBoundingBox);
		xBGRType.SetBoundingRadius( fShapeBoundingRadius);
		
#if defined( ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA) && 1 == ASURA_BOTANICAL_SUPPORT_DYNAMIC_DATA
		{
			Asura_BGR_DynamicMemoryLayout& xDynamicDataLayout = xBGRType.GetDynamicDataLayout();
			
			xDynamicDataLayout.SetupDynamicData( xBGRType);
		}
#endif
		
		xBGRType.Export( xStream);
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportBotanicalsTemplates()
//
// Exports a list of botanicals templates to a chunk stream
//
// Parameters:		xStream - Reference to output chunk stream
//					xTemplateList - Reference to list of botanicals templates
//
// Return value:	None
//
////////////////////////////////////////////////////////////////////////////////

void Tools_Xbox360_Export::ExportBotanicalsTemplates( Asura_Chunk_Stream& xStream, const Asura_LinkList< Tools_Xbox360_Export_Botanical_Template >& xTemplateList,
													  const Asura_LinkList< const Tools_BotanicalShape* >& xShapeList )
{
	const u_int uNumTemplates = xTemplateList.Size();
	u_int uCurrTemplateTimes100 = 0;

//	const Asura_LinkList< Asura_Botanicals_Instance* >& xInstanceList = Asura_Botanicals_InstanceManager::GetInstanceList();

	for( Asura_ListElt< Tools_Xbox360_Export_Botanical_Template >* pxCurrTemplateElement = xTemplateList.Front();
		 pxCurrTemplateElement;
		 pxCurrTemplateElement = pxCurrTemplateElement->Next() )
	{
		Tools_Level_Export::UpdateStatus( Tools_Level_Export::EXPORT_STAGE_BOTANICALS_EXPORTING_TEMPLATES, "Botanicals: Exporting templates...", uCurrTemplateTimes100 / uNumTemplates );
		uCurrTemplateTimes100 += 100;

		const Tools_Xbox360_Export_Botanical_Template& xCurrBlockTemplate = pxCurrTemplateElement->Data();
		const Asura_Botanicals_Template* const pxCurrTemplate = xCurrBlockTemplate.m_pxTemplate;
		
		Asura_Bounding_Box xBoundingBoxTemplate;
		
		xBoundingBoxTemplate.Reset();

		// DLH - 15 December 2006 15:06:59 - Create a list of just the Instances required
		const Asura_LinkList< const Asura_Botanicals_Instance*>& xInstanceListSave = xCurrBlockTemplate.m_xListInstances;

#if 0
		for( Asura_ListElt< Asura_Botanicals_Instance* >* pxCurrInstanceElement = xInstanceList.Front(); pxCurrInstanceElement; pxCurrInstanceElement = pxCurrInstanceElement->Next() )
		{
			const Asura_Botanicals_Instance* const pxCurrInstance = pxCurrInstanceElement->Data();
			if( pxCurrInstance->iTemplateUID != pxCurrTemplate->iUID )
			{
				continue;
			}
			
			xInstanceListSave.Push_Back( pxCurrInstance);
			
			xBoundingBoxTemplate.Set( pxCurrInstance->xPosition);
		}
#endif

#if 1
		for( const Asura_ListElt< const Asura_Botanicals_Instance* >* pxCurrInstanceElement = xInstanceListSave.Front(); pxCurrInstanceElement; pxCurrInstanceElement = pxCurrInstanceElement->Next() )
		{
			const Asura_Botanicals_Instance* const pxCurrInstance = pxCurrInstanceElement->Data();
			
			xBoundingBoxTemplate.Set( pxCurrInstance->xPosition);
		}
#endif

		u_int uShapeIndex = 0;
		for( Asura_ListElt< const Tools_BotanicalShape* >* pxCurrShapeElement = xShapeList.Front();
			pxCurrShapeElement;
			pxCurrShapeElement = pxCurrShapeElement->Next() )
		{
			const Tools_BotanicalShape* pxShapeElement = pxCurrShapeElement->Data();
			if( pxShapeElement == pxCurrTemplate->pxBotanicalShape->GetToolsShape() ) 
			{
				break;
			}

			++uShapeIndex;
		}

		xStream << pxCurrTemplate->xTemplateName;
		xStream << uShapeIndex;
		xStream << xBoundingBoxTemplate;
		xStream << static_cast< uint32>(xInstanceListSave.Size());

		// Loop over instances that use this template
		// TODO: create a list of instances per template in the finding step
		for( Asura_ListElt< const Asura_Botanicals_Instance* >* pxCurrInstanceElement = xInstanceListSave.Front();
			 pxCurrInstanceElement;
			 pxCurrInstanceElement = pxCurrInstanceElement->Next() )
		{
			const Asura_Botanicals_Instance* const pxCurrInstance = pxCurrInstanceElement->Data();
			if( pxCurrInstance->iTemplateUID != pxCurrTemplate->iUID ) 
			{
				continue;
			}

			//////////////////////////////////////////////////////////////////////////
			// This data is used by the vertex-shader, the subsequent data is extra 
			// data needed per-vertex by the Xbox360 when it reads in. This doesn't
			// sound like a good thing for runtime, but I guess we'll have to live
			// with it for now (ART 09/07/08).
			//////////////////////////////////////////////////////////////////////////
			{
				// Construct a world matrix for each instance

				D3DXMATRIX xInstanceMatrix, xTemp;

				// Rotation
				Asura_Mem::Set( &xInstanceMatrix, 0, sizeof( xInstanceMatrix ) );

				for( u_int u = 0; u < 3; ++u )
				{
					for( u_int v = 0; v < 3; ++v )
					{
						xInstanceMatrix.m[u][v] = pxCurrInstance->xOrientation.Element[v][u];
					}
				}

				xInstanceMatrix._44 = 1.0f;

				// Translation

				D3DXMatrixTranslation( &xTemp, pxCurrInstance->xPosition.x, pxCurrInstance->xPosition.y, pxCurrInstance->xPosition.z );
				D3DXMatrixMultiplyTranspose( &xInstanceMatrix, &xInstanceMatrix, &xTemp );

				// Output as 3 rows of normalised vectors stored in a u_int
				for( u_int u = 0; u < 3; ++u )
				{
					xStream << Asura_CompressedVector3_101010N(xInstanceMatrix.m[u][0], xInstanceMatrix.m[u][1], xInstanceMatrix.m[u][2]);
				}

				// Output position...
				for( u_int u = 0; u < 3; ++u )
				{
					xStream << Asura_HalfFloat(xInstanceMatrix.m[u][3]);
				}

				// Alpha - this will be fixed up in the instance VB when LODs are calculated
				xStream << Asura_HalfFloat(1.0f);

				// Static lighting
				Asura_SphericalHarmonics_Order2_RGB xSH;
				Asura_GI_Solution::GetStaticIrradiance( xSH, pxCurrInstance->xPosition );

				xStream << Asura_HalfFloat(xSH.m_afRedCoefficients[1] * pxCurrInstance->fBrightness);
				xStream << Asura_HalfFloat(xSH.m_afRedCoefficients[2] * pxCurrInstance->fBrightness);
				xStream << Asura_HalfFloat(xSH.m_afRedCoefficients[3] * pxCurrInstance->fBrightness);
				xStream << Asura_HalfFloat(xSH.m_afRedCoefficients[0] * pxCurrInstance->fBrightness);

				xStream << Asura_HalfFloat(xSH.m_afGreenCoefficients[1] * pxCurrInstance->fBrightness);
				xStream << Asura_HalfFloat(xSH.m_afGreenCoefficients[2] * pxCurrInstance->fBrightness);
				xStream << Asura_HalfFloat(xSH.m_afGreenCoefficients[3] * pxCurrInstance->fBrightness);
				xStream << Asura_HalfFloat(xSH.m_afGreenCoefficients[0] * pxCurrInstance->fBrightness);

				xStream << Asura_HalfFloat(xSH.m_afBlueCoefficients[1] * pxCurrInstance->fBrightness);
				xStream << Asura_HalfFloat(xSH.m_afBlueCoefficients[2] * pxCurrInstance->fBrightness);
				xStream << Asura_HalfFloat(xSH.m_afBlueCoefficients[3] * pxCurrInstance->fBrightness);
				xStream << Asura_HalfFloat(xSH.m_afBlueCoefficients[0] * pxCurrInstance->fBrightness);

				// Bone offset
				xStream << Asura_HalfFloat(0.0f);
				xStream << Asura_HalfFloat(0.0f);
				xStream << Asura_HalfFloat(0.0f);

				// Scale factor, needed for billboard clusters
				xStream << Asura_HalfFloat(pxCurrInstance->fScale);
			}

			// Output position as floats too, to avoid conversions when doing instance frustum cull
			xStream << pxCurrInstance->xPosition;

			// Visible flag
			xStream << static_cast< bool >( false );

			// padding
			xStream << static_cast< bool >( false );

			// Module index
			xStream << static_cast< short >( pxCurrInstance->iModuleIdx );
			Asura_Assertf( ( pxCurrInstance->iModuleIdx >= 0 && pxCurrInstance->iModuleIdx < Asura_Module_System::GetNumberOfModules() ), ( "Invalid module index %d exporting template[%d] '%s'\n", 
						   pxCurrInstance->iModuleIdx, pxCurrInstance->iTemplateIdx, pxCurrTemplate->xTemplateName.GetString() ) );
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportBotanicals()
//
// Exports botanicals data to an Xbox 360 level file
//
// Parameters:		hOutput - Handle to the output file
//					piMaterialConversionList - Material ID conversion list. Must
//											   be ASURA_MAX_NO_OF_MATERIALS in size
//
// Return value:	true iff successful
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_Xbox360_Export::ExportBotanicals( const HANDLE hOutput, const int* const piMaterialConversionList )
{
	Asura_LinkList< Tools_Xbox360_Export_Botanical_Template > xTemplateExportList;
	Asura_LinkList< const Tools_BotanicalShape* > xShapeExportList;

	u_int uNumShapesToExport = 0;
	u_int uNumTemplatesToExport = 0;
	
	FindUsedBotanicalsTemplatesAndShapes( uNumTemplatesToExport, xTemplateExportList, uNumShapesToExport, xShapeExportList);

	// Create the buffer and streams

	Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
	
	{
		// DLH - 18 December 2006 16:22:48 - Create the Chunk Data in a SaveBlock
		// DLH - 18 December 2006 16:23:06 - This saves having to know what the export file size is before saving
		Asura_Chunk_SaveBlock_Stream xStream( pxSaveBlock);
		
		xStream.SetSwapEndian( Tools_Level_Export::ShouldExportBigEndian() );

		// Write the chunk header

		xStream.WriteChunk_Start( ASURA_CHUNK_RESOURCEFILE, 0, 0 );

		// Version number

		xStream << ((uint32)(Asura_Chunk_ResourceFile_Xbox360_Botanicals::CURRENT_VERSION));
		xStream << static_cast< uint32>( Asura_BGR_Manager::s_uChunkVersion);

		// Export shapes

		xStream << uNumShapesToExport;
		ExportBotanicalsShapes( xStream, xShapeExportList, piMaterialConversionList );

		// Export templates

		xStream << uNumTemplatesToExport;
		ExportBotanicalsTemplates( xStream, xTemplateExportList, xShapeExportList );

		xStream.WriteChunk_End();
	}

	// Write the in-memory chunk data to the export file in one call

#define WRITEOUTDATA(a,b) WriteFile( hOutput, a, b, &bytesWritten, 0 );

	{
		// DLH - 18 December 2006 16:23:26 - Extra the chunk data and save to the file
		const uint32 uBytesHeaderSize = sizeof( Asura_Chunk_Header) + ASURA_FILE_ID_NUM_CHARS;
		uint32 uBytesToWrite = pxSaveBlock->GetSize() - uBytesHeaderSize;
		const void* pDataStart = pxSaveBlock->GetStartPtr() + uBytesHeaderSize;
		const u_int uBytesResourceFilename = 4 * xBOTANICALS_RESOURCE_FILENAME.GetSizeInInts();
		uint32 uChunkSize = uBytesToWrite + uBytesResourceFilename + sizeof( Asura_Chunk_ResourceFile);

		Asura_Chunk_ResourceFile* pxResourceFile = new Asura_Chunk_ResourceFile;

		pxResourceFile->Create
			(
			uChunkSize,
			0,
			ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC,
			ASURA_RESOURCEFILE_TYPE_XBOX360_BOTANICALS,
			uBytesToWrite
			);

		unsigned long bytesWritten;

		WRITEOUTDATA( pxResourceFile, sizeof(Asura_Chunk_ResourceFile) );
		const char* szStringResourceFilename = xBOTANICALS_RESOURCE_FILENAME.GetString();
		WRITEOUTDATA( szStringResourceFilename,  uBytesResourceFilename);

		WRITEOUTDATA( pDataStart, uBytesToWrite);

		delete pxResourceFile;
	}

#undef WRITEOUTDATA

	delete pxSaveBlock;

	return true;
}

#endif // defined ASURA_USE_BOTANICALS

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

bool Tools_Xbox360_Export::ExportMaterialResponses( Asura_SaveBlock& xSaveBlock, Asura_HashTree* const pxMaterialsToExport )
{
	Asura_Chunk_SaveBlock_Stream xStream( &xSaveBlock );
	
	const Asura_String xDUMMYNAME( "Material Response Texture" );

	// JAT 16/05/07 data format has changed so the version number ought to be increased
	const u_int uCURRENT_VERSION = Asura_MaterialResponse_System::EXPORTED_TEXTURE_VERSION;
	xStream.WriteResourceChunk_Start( ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC, ASURA_RESOURCEFILE_TYPE_XBOX360_MATERIALRESPONSE, uCURRENT_VERSION, 0, xDUMMYNAME );

	int iNumberOfMaterialsToExport = pxMaterialsToExport->GetNumberOfNodes();

	Asura_Hash_ID* const puExportHashes = new Asura_Hash_ID[ iNumberOfMaterialsToExport ];
	Asura_Assert( puExportHashes, "Failed to allocate material response export array." );
	if( !puExportHashes ) return false;

	pxMaterialsToExport->MakeHashList( puExportHashes );
	
	xStream << iNumberOfMaterialsToExport;

	for( int i = 0; i < iNumberOfMaterialsToExport; ++i )
	{
		const Tools_MaterialResponse* const pxMaterial = Tools_MaterialResponse_System::GetMaterialByHashID( puExportHashes[i] );
		if( pxMaterial )
		{
			// On 360, scale the graph values into [0,1]
			pxMaterial->WriteTextureLineToStream( xStream, true );
		}
		else
		{
			Asura_Assertf( false, ("Material to export (0x%x) couldn't be found.", puExportHashes[i]) );
		}
	}
	
	xStream.WriteResourceChunk_End();

	delete [] puExportHashes;

	return true;
}

template< u_int u >
static void AddPercentageDefine( const float fValue, const char* const szDefine0,
	const char* const szDefine1, Asura_Hash_ID& uHash, u_int& uBit,
	Asura_FixedArray< const char*, u >& aszUsedDefines, u_int& uNumUsedDefines )
{
	// Tools materials store values as percentages. Use two bits in the hash to represent which
	// define we used

	if( fValue <= 1.0f )
	{
		aszUsedDefines[ uNumUsedDefines++ ] = szDefine0;
		// Don't set the two bits
		uBit <<= 2;
	}
	else if( fValue >= 99.0f )
	{
		aszUsedDefines[ uNumUsedDefines++ ] = szDefine1;
		// Set the bits to 01
		uHash |= uBit;
		uBit <<= 2;
	}
	else
	{
		// Set the bits to 11
		uHash |= uBit;
		uBit <<= 1;
		uHash |= uBit;
		uBit <<= 1;
	}
}

void Tools_Xbox360_Export::AddMRPShader( const Asura_Hash_ID uShaderHash )
{
	int iIndex = 0;
	if( s_xShaderTree.FindNodeIndexByHash( uShaderHash, true, iIndex ) ) return;

	Asura_LinkList< Asura_String > xMacroList;
	u_int uBitMask = 1;

	// First check the texture slots

	static const Asura_String axTEXTURE_MACROS[] =
	{
		"USE_ALBEDO_1=true",
		"USE_ALBEDO_2=true",
		"USE_NORMAL_1=true",
		"USE_NORMAL_2=true",
		"USE_SPECULAR_1=true",
		"USE_SPECULAR_2=true",
		"USE_TRANSPARENCY_1=true",
		"USE_TRANSPARENCY_2=true",
		"USE_SELFILLUMINATION_1=true",
		"USE_SELFILLUMINATION_2=true",
		"USE_ENVIRONMENT_1=true",
		"USE_ENVIRONMENT_2=true",
		"USE_REFLECTIVITY_1=true",
		"USE_REFLECTIVITY_2=true",
		"USE_PARALLAX_1=true",
		"USE_PARALLAX_2=true",
		"USE_ALBEDO_DETAIL_MAP_1=true",
		"USE_ALBEDO_DETAIL_MAP_2=true",
		"USE_NORMAL_DETAIL_MAP_1=true",
		"USE_NORMAL_DETAIL_MAP_2=true",
	};

	const u_int uNUM_TEXTURE_MACROS = sizeof( axTEXTURE_MACROS ) / sizeof( axTEXTURE_MACROS[0] );
	const u_int uNUM_TEXTURE_SLOTS = Asura_MaterialResponse::NUMBER_OF_TEXTURE_SLOTS * 2;
	ASURA_STATIC_ASSERT( uNUM_TEXTURE_MACROS == uNUM_TEXTURE_SLOTS );

	for( u_int u = 0; u < uNUM_TEXTURE_MACROS; ++u )
	{
		if( uShaderHash & uBitMask )
		{
			xMacroList.PushBack( axTEXTURE_MACROS[u] );
		}

		uBitMask <<= 1;
	}

	// Then the colourise options

	static const Asura_String axCOLOURISE_MACROS[] =
	{
		// Values added depending on bits
		"COLOURISE_DIFFUSE=",
		"COLOURISE_SPECULAR=",
		"METALICITY=",
	};

	const u_int uNUM_COLOURISE_MACROS =
		sizeof( axCOLOURISE_MACROS ) / sizeof( axCOLOURISE_MACROS[0] );

	for( u_int u = 0; u < uNUM_COLOURISE_MACROS; ++u )
	{
		const u_int uValueMask = uBitMask | (uBitMask << 1);
		const u_int uValueShift = uNUM_TEXTURE_SLOTS + u*2;

		const u_int uValue = (uShaderHash & uValueMask) >> uValueShift;

		Asura_StringBuffer<2> xValue;
		xValue.Print( "%u", uValue );

		Asura_String xCompleteMacro = axCOLOURISE_MACROS[u];
		xCompleteMacro += xValue.GetString();

		xMacroList.PushBack( xCompleteMacro );

		uBitMask <<= 2;
	}

	// Check individual bits
	{
		if( uShaderHash & uASURA_MRPS_HASH_BITMASK_BLEND_INTO_ALBEDO )
		{
			xMacroList.PushBack( "BLEND_INTO_ALBEDO=true" );
		}

		if( uShaderHash & uASURA_MRPS_HASH_BITMASK_IGNORE_SPECULAR )
		{
			xMacroList.PushBack( "IGNORE_SPECULAR=true" );
		}

		xMacroList.PushBack( "NOT_EFFECT" );
	}

	const u_int uNumMacros = xMacroList.Size();
	const char** pszMacroArray = new const char*[ uNumMacros ];

	Asura_Assert( pszMacroArray, "Failed to allocate array for pixel shader macros." );
	if( !pszMacroArray ) return;

	// Flatten the linked list into an array
	{
		u_int uIndex = 0;

		for( Asura_LinkList_ConstIt< Asura_String > xIter( xMacroList );
			!xIter.Done(); xIter.Next() )
		{
			const Asura_String* const pxCurrMacro = xIter.GetCurrent();
			pszMacroArray[ uIndex++ ] = pxCurrMacro->GetString();
		}
	}

	// Add shader description to .rdf file
	{
		Asura_StringBuffer<11> xShaderName;
		xShaderName.Print( "0x%.08X", uShaderHash );

		const bool bSkin = ((uShaderHash & uASURA_MRPS_HASH_BITMASK_SKIN) != 0);
		const bool bTransparent = ((uShaderHash & uASURA_MRPS_HASH_BITMASK_TRANSPARENT) != 0);

		const char* const szFilename = (bSkin ? "Skin_PS.ps" : "Env_MaterialResponse.fx");
		const char* const szFunction = (bTransparent ? "PShader_Forward" : "PShader_Deferred");

		Tools_Xbox360_Texture_Writer::AddPixelShaderToRDF( s_xShaderRDF, xShaderName.GetString(),
			szFilename, szFunction, uNumMacros, pszMacroArray );
	}

	delete[] pszMacroArray;
}

////////////////////////////////////////////////////////////////////////////////
//
// ExtractShaderResource()
//
// Callback function to extract an enumerated shader source file from resources
// and save it off into the export temp folder
//
////////////////////////////////////////////////////////////////////////////////

static BOOL CALLBACK ExtractShaderResource( HMODULE xModule,
	LPCTSTR szType, LPTSTR szName, LONG_PTR uParam )
{
	Asura_Assert( !IS_INTRESOURCE( szName ), "Expected a string resource name." );

	const HRSRC xResInfoHandle = FindResource( xModule, szName, szType );
	Asura_Assertf( xResInfoHandle, ("Failed to find resource '%s' of type '%s'.", szName, szType) );
	if( !xResInfoHandle ) return FALSE;

	const HGLOBAL xResHandle = LoadResource( xModule, xResInfoHandle );
	Asura_Assertf( xResHandle, ("Failed to load resource '%s' of type '%s'.", szName, szType) );
	if( !xResHandle ) return FALSE;

	const LPVOID pResData = LockResource( xResHandle );
	Asura_Assertf( pResData, ("Failed to lock resource '%s' of type '%s'.", szName, szType) );
	if( !pResData ) return FALSE;

	const DWORD uResSize = SizeofResource( xModule, xResInfoHandle );

	Asura_String xOutputFileName = Tools_Level_Export::GetTempDirectory();
	Tools_FileSystem::EnsureDirectoryExists( xOutputFileName.GetString() );
	xOutputFileName += "\\";
	xOutputFileName += szName;

	Tools_File_Wrapper xOutputFile;
	const bool bSuccess = xOutputFile.Create( xOutputFileName.GetString(), true );

	Asura_Assertf( bSuccess, ("Failed to open file '%s' for writing.", szName) );
	if( !bSuccess ) return FALSE;

	const u_int uNumBytesWritten = xOutputFile.Write( pResData, uResSize );
	Asura_Assertf( uNumBytesWritten == uResSize, ("Failed to write to '%s'.", szName) );

	xOutputFile.Close();
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::BeginMRPShaderExport()
//
// Begins the export process for material response pixel shaders. Extracts the
// source files into the temp folder and sets up the RDF file
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_Xbox360_Export::BeginMRPShaderExport()
{
	s_xShaderTree.Reset();

	// Extract the shader source files from resources and dump them in the temp folder
	EnumResourceNames( NULL, "360SHADER", ExtractShaderResource, 0 );

	// Set things up for Bundler
	{
		Asura_String xTempRDF = Tools_Level_Export::GetTempDirectory();
		Tools_FileSystem::EnsureDirectoryExists( xTempRDF.GetString() );
		xTempRDF += "\\MRPShaders.rdf";

		Tools_Xbox360_Texture_Writer::BeginTextureExport( s_xShaderRDF, xTempRDF.GetString() );
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::EndMRPShaderExport()
//
// Finishes off the export process for material response pixel shaders.
// Finalises the RDF file, runs Bundler, and stores the output in a chunk
//
////////////////////////////////////////////////////////////////////////////////

bool Tools_Xbox360_Export::EndMRPShaderExport( const HANDLE xExportHandle )
{
	Tools_Xbox360_Texture_Writer::EndTextureExport( s_xShaderRDF );

	if( s_xShaderTree.GetNumberOfNodes() > 0 )
	{
		Tools_Level_Export::UpdateStatus( Tools_Level_Export::EXPORT_STAGE_COMPILING_MATERIAL_RESPONSE_PIXEL_SHADERS, "Compiling material response pixel shaders...", -1 );

		Tools_Xbox360_Texture_Writer::RunBundler( s_xShaderRDF );
		Tools_Xbox360_Texture_Writer::CreateBundleChunk(  s_xShaderRDF, xExportHandle );

		// No need to delete the RDF file, the entire temp folder is destroyed by Tools anyway

		s_xShaderTree.Reset();
	}

	return true;
}

#endif // ASURA_USE_MATERIAL_RESPONSE_SYSTEM





////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export::ExportStaticDecals()
//
// Exports the static decal geometry. Decal materials/textures are handled
// separately (and platform-independent)
//
// Parameters:		hOutput - Handle to the output file
//
// Return value:	true iff successful
//
////////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_STATIC_DECAL_SYSTEM
bool Tools_Xbox360_Export::ExportStaticDecals( const HANDLE hOutput )
{

	// It's the familiar 'completely arbitrary fixed-size buffer' design pattern again...
	Asura_Chunk_Memory_Stream xStream(16 * 1024 * 1024);	

	const u_int uCURRENT_VERSION = 1;
	xStream.WriteResourceChunk_Start( ASURA_RESOURCEFILE_TYPE_PLATFORMSPECIFIC, ASURA_RESOURCEFILE_TYPE_XBOX360_STATICDECALS, uCURRENT_VERSION,0, xSTATICDECALS_RESOURCE_FILENAME );	

	// Kept the exported data very simple for now
	// Given more time, this could be improved - to enable better batching and/or less material/vb switching	
	int iNumBuffers = Tools_StaticDecal_System::GetNumDecalBuffers();
	
	xStream << iNumBuffers;
	for(int i=0; i<iNumBuffers; i++)
	{	
		Tools_StaticDecal_System::OptimizeBuffer(i);
		Tools_StaticDecal_Buffer* pxBuffer = Tools_StaticDecal_System::GetDecalBufferByIndex(i);		
		
		
		xStream << pxBuffer->m_iModuleIndex;
		xStream << pxBuffer->m_uDecalMaterial;
		xStream << pxBuffer->m_iNumVerts;
		xStream << pxBuffer->m_iNumIndices;
		xStream << pxBuffer->m_xAABB;

		// Output the verts
		for(int j=0; j<pxBuffer->m_iNumVerts; j++)
		{
			Tools_StaticDecal_Vertex *pxVert = &pxBuffer->m_pxVerts[j];
			
			xStream << pxVert->xPos;
			xStream << pxVert->fU;
			xStream << pxVert->fV;
			xStream << pxVert->fCreationTime; // always 0.0f - 'creation time' value when used on dynamic decals
			xStream << pxVert->fAlpha;

			xStream << Asura_CompressedVector3_101010N(pxVert->xNormal);
			xStream << Asura_CompressedVector3_101010N(pxVert->xTangent);
			xStream << Asura_CompressedVector3_101010N(pxVert->xBinormal);

			xStream << (Asura_HalfFloat)pxVert->xSHRed.GetX();
			xStream << (Asura_HalfFloat)pxVert->xSHRed.GetY();
			xStream << (Asura_HalfFloat)pxVert->xSHRed.GetZ();
			xStream << (Asura_HalfFloat)pxVert->xSHRed.GetW();

			xStream << (Asura_HalfFloat)pxVert->xSHGreen.GetX();
			xStream << (Asura_HalfFloat)pxVert->xSHGreen.GetY();
			xStream << (Asura_HalfFloat)pxVert->xSHGreen.GetZ();
			xStream << (Asura_HalfFloat)pxVert->xSHGreen.GetW();

			xStream << (Asura_HalfFloat)pxVert->xSHBlue.GetX();
			xStream << (Asura_HalfFloat)pxVert->xSHBlue.GetY();
			xStream << (Asura_HalfFloat)pxVert->xSHBlue.GetZ();
			xStream << (Asura_HalfFloat)pxVert->xSHBlue.GetW();
		}

		// Output the indices
		for(int j=0; j<pxBuffer->m_iNumIndices; j++)
		{
			uint16 usTmp = (uint16)pxBuffer->m_piIndices[j];			
			xStream << usTmp;
		}
	}


	xStream.WriteResourceChunk_End();
	
	unsigned long uBytesWritten = 0;
	const void* pData = xStream.GetStartAddress();
	u_int uChunkSize = xStream.GetPosition();
	WriteFile(hOutput, pData, uChunkSize, &uBytesWritten, NULL);

	return true;
}
#endif //ASURA_USE_STATIC_DECAL_SYSTEM

bool Tools_Xbox360_Export::CompressFile(const char* szFilename)
{
	if(Tools_FileSystem::IsReadOnly(szFilename))
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Failed to compress %s - Not writeable",szFilename);
		return false;
	}

	//get a temporary name for the compressed file
	Asura_String xFullFilename = szFilename;
	Asura_String xPath,xName;

	xFullFilename.SplitFilename(&xPath, &xName, 0);
	if(xPath.IsBlank()) xPath = ".";

	char acTempName[MAX_PATH];
	if(!GetTempFileName(xPath.GetString(),xName.GetString(),0,acTempName))
	{
		Asura_Assert(0,"Error getting temporary filename");
		return false;
	}


	//prepare the command line arguments for the compressor
	char acCommand[1024];
	Asura_PrintfToBfr(acCommand,"xbcompress.exe /q /y \"%s\" \"%s\"",sizeof(acCommand),true,szFilename,acTempName);

	//And execute the compression tool
	PROCESS_INFORMATION xProcessInfo;
	STARTUPINFO xStartupInfo = {0};
	xStartupInfo.cb = sizeof( STARTUPINFO );

	if( !CreateProcess( NULL, acCommand, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &xStartupInfo, &xProcessInfo ) )
	{
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "Failed to compress file. Check that xbcompress.exe is present." );
		return false;
	}
	else
	{
		//Wait for the process to end , checking periodically for escape being pressed
		while(true)
		{
			DWORD uResult = WaitForSingleObject( xProcessInfo.hProcess, 100 );
			if(uResult == WAIT_OBJECT_0)
			{
				//process has finished
				break;
			}

			Tools_Level_Export::UpdateStatus(Tools_Level_Export::EXPORT_STAGE_COMPRESSING_FILE, "Compressing File", -1);
			if(Tools_Level_Export::IsAbortFlagSet())
			{
				break;
			}
			
		}

   		if(Tools_Level_Export::IsAbortFlagSet())
		{
			//stop the compression
			BOOL bTerminated = TerminateProcess(xProcessInfo.hProcess, 0);
			Asura_Assertf(bTerminated, ("Failed to stop xbcompress.\n Error : %s",Tools_OS::GetLastSystemError()));
			WaitForSingleObject( xProcessInfo.hProcess, INFINITE );
			CloseHandle( xProcessInfo.hThread );
			CloseHandle( xProcessInfo.hProcess );

			//delete the partially written file
			DeleteFile(acTempName);

			return false;
		}
		else
		{
   			DWORD uProcessExitCode = static_cast< u_int>( ~0);
   			GetExitCodeProcess( xProcessInfo.hProcess, &uProcessExitCode);

			CloseHandle( xProcessInfo.hThread );
			CloseHandle( xProcessInfo.hProcess );

			if(uProcessExitCode != 0)
			{
				Tools_Level_Export::AddMessage( Tools_Level_Export::MT_ERROR, "xbcompress.exe returned an error while compressing file." );
				return false;
			}

			//delete the original and rename the temporary file
			if(DeleteFile(szFilename))
			{
				if(!MoveFile(acTempName,szFilename))
				{
					Asura_Assert(0,"Error renaming compressed file back to original name.");
					return false;
				}
			}
			else
			{
				Asura_Assert(0,"Failed to delete old copy of file being compressed");
				DeleteFile(acTempName);
				return false;
			}

		}
	}

	return true;
}
