///////////////////////////////////////////////////////////////////////////////
//
// File   : Tools_Xbox360_Export.h
// Author : Andy Tate
// Date   : 26-Sep-2005
//
// Copyright (C) 2005 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

#if defined ASURA_USE_PRAGMA_ONCE
	#pragma once
#endif   // ifdef ASURA_USE_PRAGMA_ONCE

#if !defined __TOOLS_XBOX360_EXPORT_H__
#define __TOOLS_XBOX360_EXPORT_H__

/******************************************************************************/
/* Included Files                                                             */
/******************************************************************************/

// Asura Platform-independent
#include "Core/Asura_HalfFloat.h"
#include "Core/Asura_Hash.h"
#include "Core/Asura_Maths.h"
#include "Core/Maths/Asura_CompressedVector3_101010N.h"

// Asura PC
#include "Core/Asura_PlatformBasics.h"

/******************************************************************************/
/* Classes                                                                    */
/******************************************************************************/

struct Asura_Botanicals_Template;
class Asura_Chunk_Stream;
class Asura_HashTree;
class Asura_Hierarchy_Skin;
template< class T > class Asura_LinkList;
class Asura_Mesh;
class Asura_SaveBlock;
struct TextureConversionData;
class Tools_BGR_Type_BoneWeightCalc;
class Tools_BotanicalElementLOD;
class Tools_BotanicalShape;
class Tools_Xbox360_Export_Botanical_Template;
struct Tools_Xbox360_RDF;

////////////////////////////////////////////////////////////////////////////////
//
// Tools_Xbox360_Export
//
// Handles exporting of data to Xox 360 format
//
////////////////////////////////////////////////////////////////////////////////

class Tools_Xbox360_Export
{
public:

	// Texture exporting function...
	static bool ExportTexture( const char* szInputName, const char* szOutputName, const int iTextureIndex, const TextureConversionData& xConvertData );
	static bool ExportTextureAsXPR( const char* szInputName, const char* szOutputName, const int iTextureIndex, const TextureConversionData& xConvertData );

	// Mesh exporting functions...
	static void ExportEnvironment( const HANDLE hOutput );	
	
	static bool ExportCharacter( const HANDLE hOutput, Asura_Hierarchy_Skin* pxSkin );
	static bool ExportObjectHierarchy( const HANDLE hOutput, const Asura_Hierarchy_Skin* pxSkin );

	static bool ExportObject( const HANDLE hOutput,
							  Asura_Mesh* pxMesh,
							  const Asura_Hash_ID uHashID,
							  const int iMaterialIndex,
							  const Asura_Hash_ID uMaterialResponseHashID = ASURA_HASH_ID_UNSET );

	static bool ExportSkin( const HANDLE hOutput, Asura_Hierarchy_Skin& xSkin );

#if defined ASURA_USE_BOTANICALS
	static bool ExportBotanicals( const HANDLE hOutput, const int* const piMaterialConversionList );
#endif

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
	static bool ExportMaterialResponses( Asura_SaveBlock& xSaveBlock, Asura_HashTree* const pxMaterialsToExport );

	static bool BeginMRPShaderExport();
	static bool EndMRPShaderExport( const HANDLE xExportHandle );

	static void AddMRPShader( const Asura_Hash_ID uShaderHash );
#endif

#if defined ASURA_USE_STATIC_DECAL_SYSTEM
	static bool ExportStaticDecals( const HANDLE hOutput );
#endif

	//Compresses a file using the LZX compression
	static bool CompressFile(const char* szFilename);

protected:

	class EnvironmentVertex
	{
	public:

		Asura_Vector_3 m_xPosition;

		u_int m_uDiffuse;

		Asura_HalfFloat m_xU;
		Asura_HalfFloat m_xV;
		Asura_HalfFloat m_xU2;
		Asura_HalfFloat m_xV2;

		Asura_CompressedVector3_101010N m_xNormal;
		Asura_CompressedVector3_101010N m_xTangent;
		Asura_CompressedVector3_101010N m_xBinormal;
		Asura_CompressedVector3_101010N m_xTangent2;
		Asura_CompressedVector3_101010N m_xBinormal2;

		Asura_HalfFloat m_axSHRedCoefficients[4];
		Asura_HalfFloat m_axSHGreenCoefficients[4];
		Asura_HalfFloat m_axSHBlueCoefficients[4];

		void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	};

	class ObjectVertex
	{
	public:

		Asura_Vector_3 m_xPosition;

		Asura_HalfFloat m_xU;
		Asura_HalfFloat m_xV;
		Asura_HalfFloat m_xU2;
		Asura_HalfFloat m_xV2;

		float m_fAlpha;

		Asura_CompressedVector3_101010N m_xNormal;
		Asura_CompressedVector3_101010N m_xTangent;
		Asura_CompressedVector3_101010N m_xBinormal;

		void WriteToChunkStream( Asura_Chunk_Stream& xStream ) const;
	};

#if defined ASURA_USE_BOTANICALS

	// Botanicals export helper functions

	static void FindUsedBotanicalsTemplatesAndShapes( u_int& uNumUsedTemplates, Asura_LinkList< Tools_Xbox360_Export_Botanical_Template >& xUsedTemplateList,
													  u_int& uNumUsedShapes, Asura_LinkList< const Tools_BotanicalShape* >& xUsedShapeList);

	static void ExportBotanicalsShapes( Asura_Chunk_Stream& xStream, const Asura_LinkList< const Tools_BotanicalShape* >& xShapeList, const int* const piMaterialConversionList );
	static void ExportBotanicalsTemplates( Asura_Chunk_Stream& xStream, const Asura_LinkList< Tools_Xbox360_Export_Botanical_Template >& xTemplateList,
										   const Asura_LinkList< const Tools_BotanicalShape* >& xShapeList ); // Temp shape list to find index

	static void IncrementExportSize_Geometry( const Tools_BotanicalElementLOD& xElementLOD );
	static void IncrementExportSize_Billboards( const Tools_BotanicalElementLOD& xElementLOD );
	static void IncrementExportSize_Imposter( const Tools_BotanicalElementLOD& xElementLOD );

	static void ExportBotanicalsElementLOD_Geometry( Asura_Chunk_Stream& xStream, const Tools_BotanicalElementLOD& xElementLOD, const int* const piMaterialConversionList, const Tools_BGR_Type_BoneWeightCalc* pxBoneWeightData);
	static void ExportBotanicalsElementLOD_Billboards( Asura_Chunk_Stream& xStream, const Tools_BotanicalElementLOD& xElementLOD, const int* const piMaterialConversionList, const Tools_BGR_Type_BoneWeightCalc* pxBoneWeightData);
	static void ExportBotanicalsElementLOD_Imposter( Asura_Chunk_Stream& xStream, const Tools_BotanicalElementLOD& xElementLOD, const int* const piMaterialConversionList, const Tools_BGR_Type_BoneWeightCalc* pxBoneWeightData);

#endif // defined ASURA_USE_BOTANICALS

#if defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM

	static Asura_HashTree s_xShaderTree;

	static Tools_Xbox360_RDF s_xShaderRDF;

#endif // defined ASURA_USE_MATERIAL_RESPONSE_SYSTEM
};

#endif // !defined __TOOLS_XBOX360_EXPORT_H__
